/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*******************************************************************************
*
* This app needs to be run as the primary DPDK application. The parameters
* include the various devices to be configured in the system and the CPU
* core masks.
* NOTE: user is not expected to pass vdev (EventDev/EthDev) arguments 
* to this process from command line. 
* Example Usage:
* 
* ./adk_dp_app.linux-x86_64_exe  -c 0x00F  --proc-type auto  -- -w 0x2 -r 0x4 -t 0x4 -C 0x4
*
*******************************************************************************/


#include <getopt.h>
#include <stdint.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <fcntl.h>

#include "adk_dp_app_config.h"
#include "adk_sys.h"
#include "adk_ph_dev.h"
#include "adk_pal.h"
#include "adk_modules_enabled.h"
#include "adk_ethernet.h"
#include "adk_ip.h"

static void
print_usage (
    void)
{
    const char *usage_str =
        "  Usage: --vdev options not not supported\n"
        "  Following are the supported options\n"
        "  Options:\n"
        "  --proc-type auto\n"
        "  -w, --worker-mask=core_mask  Run work on CPUs in core mask\n"
        "  -C, --cppi-mask=core_mask    Run CPPI bridge driver on CPUs in core mask\n"
        "  -r, --rx-mask=core_mask      Run NIC rx on CPUs in core mask\n"
        "  -t, --tx-mask=core_mask      Run NIC tx on CPUs in core mask\n"
        "  -s,  --sched-mask=core_mask   Run QM scheduler on CPUs in core mask\n"
        "  -v, --ip-version=version     Run with only IPv4 (4) or IPv6 (6)\n"
        "  -i, --ipsec           Enable IPsec\n"
        "  -c  --crypto-mask=core_mask   Run CRYPTO BD on CPUs in core mask\n"
        "  -o, --ordered         Use ordered scheduling\n"
        "  -p, --ph              Enable PH\n"
        "  -q, --quiet           Minimize printed output\n"
        "\n";

    fprintf(stderr, "%s", usage_str);
    exit(1);
}

int main (
    int argc,
    char **argv)
{
    uint8_t         opt = ' ';
    adk_sys_handle_t sys_handle;
    int arg_c = argc;

    /* Parse cli options to check for any unsupported args*/
    while(arg_c != 0)
    {
            arg_c--;
            if (strstr(argv[arg_c], "vdev") != NULL)
            {
                    print_usage();
                    return 1;
            }
    }


    if (adk_dp_setup(argc, argv) != 0)
    {
        printf("Failed in ADK DP inittialization \n");
        return 1;
    }

    sys_handle = init_adk_modules();
    if(!sys_handle){
	    printf("Failed in System init \n");
	    return 1;
    }

    if (adk_dp_init(sys_handle)){
	    printf("Failed to init DP \n");
	    return 1;
    }

    printf("Successfully completed DP initializations!\n");

    do
    {
        adk_dump_dev_stats(opt);
        printf("Press Enter to display stats or q to quit\n");
        opt = getchar();
    } while (opt != 'q');

    adk_dp_shutdown();
    return 0;
}

