/**************************************************************************
 ** INTEL CONFIDENTIAL
 ** Copyright 2011 - 2014 Intel Corporation All Rights Reserved.
 **
 ** The source code contained or described herein and all documents related to 
 ** the source code ("Material") are owned by Intel Corporation or its suppliers or 
 ** licensors. Title to the Material remains with Intel Corporation or its suppliers 
 ** and licensors. The Material contains trade secrets and proprietary and confidential
 ** information of Intel or its suppliers and licensors. The Material is protected by 
 ** worldwide copyright and trade secret laws and treaty provisions. No part of the 
 ** Material may be used, copied, reproduced, modified, published, uploaded, posted, 
 ** transmitted, distributed, or disclosed in any way without Intel's prior express written
 ** permission.
 **
 ** No license under any patent, copyright, trade secret or other intellectual property 
 ** right is granted to or conferred upon you by disclosure or delivery of the Materials, 
 ** either expressly, by implication, inducement, estoppel or otherwise. Any license under 
 ** such intellectual property rights must be express and approved by Intel in writing.
 *************************************************************************/

/**
 * @file
 *
 * This sample demonstrates basic EM <--> ADK EM communication.
 * This creates 2 queues (static and dynamic) attached to EO.
 * ADK application sends events to these and on receiving this 
 * application sends event to ADK.
 * EM application creates queues for EM->ADK in the EO upon received the 
 * sync event sent by ADK application after ADK is initialized and em_attached is called.
 *
 * Command line arguments: ./eventMachine -c 0xff -d librte_bus_vmbus.so  -- -t -a test_appl_adk_em
 *                            -f ../programs/common/config/SNF_EM_Config.xml
 *                            -f $ADK_SRC/ncp_adk/test/adk_dp_app/SNF_EM_ADK_Config.xml
 */
#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"

#include <libgen.h>

#include "adk_dp_app_config.h"
#include "adk_sys.h"
#include "adk_ph_dev.h"
#include "adk_pal.h"
#include "adk_modules_enabled.h"
#include "adk_ethernet.h"
#include "adk_ip.h"
#include "adk_ipsec.h"
/*
 * Test configuration
 */
#define IP_HDR_LEN 20
#define IPV6_HDR_LEN 40
#define UDP_HDR_LEN 8
#define DATA_LEN_IPV4 18
#define DATA_LEN_IPV6 48

#define VIRTUAL_CORE_ID_FOR_EM_START 25
#define NUM_QIDS_PER_WORKER 4
/* ADK Entry Point
 * 0 - Unused  TODO: Need to check
 * 1 - IPv4
 * 2 - IPv6
 */
#define NUM_ADK_ENTRY_POINT 3
#define EM_RECV_QUEUE_ID_IP   3276801 //0x320001
#define EM_RECV_QUEUE_ID_IPV6 3277057 //0x320101

static void printPacket(unsigned char *pkt, int pktLen);

typedef struct adk_app_ipv4_msg_t
{
    unsigned char ipHdr[IP_HDR_LEN];
    unsigned char udpHdr[UDP_HDR_LEN];
    unsigned char data[DATA_LEN_IPV4];
}__attribute__((packed))adk_app_ipv4_msg_t;


typedef struct adk_app_ipv6_msg_t
{
	unsigned char ipv6Hdr[IPV6_HDR_LEN];
	unsigned char udpHdr[UDP_HDR_LEN];
	unsigned char data[DATA_LEN_IPV6];
}__attribute__((packed))adk_app_ipv6_msg_t;

/* Handcrafted UDP/IP packet. Size 60 bytes */
static adk_app_ipv4_msg_t ipPacket = {
	.ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
			  0x66, 0xBD, 0x64, 0x64, 0x64, 0x01, 0xc8, 0xc8, 0xc8, 0x01},
	.udpHdr = {0x00, 0x3F, 0x00, 0x3F, 0x00, 0x1A, 0xA2, 0xE8},
	.data = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
			 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11}};
static adk_app_ipv6_msg_t ipv6Packet = {
	/*Change the 4th and 5th octet to modify payload length to test frag from EO path*/
	.ipv6Hdr = {0x60 , 0x00 , 0x00 , 0x00 , 0x00 , 0x38 , 0x11 , 0x01 , 
			 0x20 , 0x03 , 0x00 , 0x00 , 0xab , 0xcd , 0xef , 0x98 , 
			 0xaa , 0xbb , 0xcc , 0x11 , 0x00 , 0x00 , 0x00 , 0x01 , 
 			 0x20 , 0x03 , 0x00 , 0x00 , 0xab , 0xcd , 0xef , 0x98 , 
			 0xaa , 0xbb , 0xcc , 0x11 , 0x00 , 0x00 , 0x00 , 0x02}, 

	.udpHdr = {0x00, 0x3F, 0x00, 0x3F, 0x00, 0x38, 0x00, 0x26},

	.data = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
			 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11}};

#define CORE_USED  	1

/** Spin value before restarting the test */
#define DELAY_SPIN_COUNT	  	500000

#define TIME_OUT 		100000
#define NONEM_QUEUE_IP  (long unsigned int)0x400
#define NONEM_QUEUE_IPV6  (long unsigned int)0x800
#define NUM_EM_ENTRY_QUEUES 3
#define NUM_NONEM_ENTRY_QUEUES 3


typedef struct {
	em_queue_t em_queue[NUM_EM_ENTRY_QUEUES];
	em_queue_t nonem_queue[NUM_NONEM_ENTRY_QUEUES];
	int rcv_cnt;
} em_adk_common_t;

extern void *shm_app_addr;
extern em_conf_t em_conf;
static shared_app_common_t *s;
static em_adk_common_t *ns;
extern uint8_t is_unified_em_adk_set;
extern uint8_t number_em_cores;
extern uint8_t em_start_port;

uint8_t *nem_qcfg;
adk_xml_cfg_t adkCfg;
/*
 * Local function prototypes
 */
void test_appl_adk_em_init_local(void);
void test_appl_adk_em_start(void);
void test_appl_adk_em_stop(void);
void test_appl_adk_em_pre_init(void * , void *);
/*
 * 
 * EO start function.
 *
 * Creates the queue group used in this test case.
 *
 */
static em_status_t
non_em_start(eo_context_common_t * eo_ctx, em_eo_t eo)
{
	(void)eo_ctx;
	(void)eo;

	usleep(100);

	printf("\nStart non_em Application\n");

	return EM_OK;
}

/*
 *
 * EO stop function.
 *
 */
static em_status_t
non_em_stop(eo_context_common_t * eo_ctx, __attribute__ ((unused)) em_eo_t eo)
{
	em_status_t ret;
	ret = em_eo_delete(eo_ctx->this_eo_id);
	CHECK_RET(ret != EM_OK, "em_eo_delete ret=(%u). EO: %" PRI_EO "\n", ret,
			eo_ctx->this_eo_id);

	return EM_OK;
}

static void
create_nonem_queues(void)
{
	em_core_mask_t mask;
	em_status_t ret;
	em_queue_group_t nonem_group_ip;
	em_queue_group_t nonem_group_ipv6;

	/* Create queues for EM tonon EM */

	ns->nonem_queue[0] = (my_device_id << 16) | NONEM_QUEUE_IP;  // non-em queue ID for IPv4
	ns->nonem_queue[1] = (my_device_id << 16) | NONEM_QUEUE_IPV6;// non-em queue ID for IPv6

	em_core_mask_zero(&mask);
	em_core_mask_set(VIRTUAL_CORE_ID_FOR_EM_START, &mask);
	nonem_group_ip = em_queue_group_create("_qgr_ip", &mask, 0, NULL);

	ret = em_queue_create_static("static_nonem_q_ip", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_LOW,
					nonem_group_ip, (my_device_id << 16) | NONEM_QUEUE_IP);

	CHECK_RET(ret != EM_OK, "Static queue creation failed: %" PRI_QUEUE "\n", NONEM_QUEUE_IP);
	printf("Static queue from non EM IPv4 created, queue: %" PRI_QUEUE "\n", NONEM_QUEUE_IP);

	em_core_mask_zero(&mask);
	em_core_mask_set(VIRTUAL_CORE_ID_FOR_EM_START+1, &mask);
	nonem_group_ipv6 = em_queue_group_create("_qgr_ipv6", &mask, 0, NULL);


	ret = em_queue_create_static("static_nonem_q_ipv6", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_LOW,
					nonem_group_ipv6, (my_device_id << 16) | NONEM_QUEUE_IPV6);

	CHECK_RET(ret != EM_OK, "Static queue creation failed: %" PRI_QUEUE "\n", NONEM_QUEUE_IPV6);
	printf("Static queue from non EM IPv6 created, queue: %" PRI_QUEUE "\n", NONEM_QUEUE_IPV6);
	*nem_qcfg = 1;
}

/*
 *
 * EO receive function.
 *
 */
static void
non_em_receive(eo_context_common_t * eo_ctx, em_event_t event,
		   __attribute__ ((unused)) em_event_type_t type, em_queue_t queue,
		   queue_context_common_t * queue_ctx)
{
	(void)queue_ctx;
	nonem_event_t *my_event;
	em_status_t ret;
	em_queue_t dest_queue = 0;
	uint32_t *tx_core = em_event_to_nonem_param(event);
	uint8_t *msg = NULL;
	int pktLen = 0;

	my_event = em_event_pointer(event);
	/* update the count of data events received on this core */
	s->core_stats[CORE_USED].stat.rcv_ev_cnt += 1;
	printf("Event received on EO: %s. I'm on core_id %02i from nonem core %d. Event seq %x. Queue ID:  0x%lx\n",
			eo_ctx->name, em_core_id(), *tx_core, my_event->seq, queue);
	struct adk_sys_event_metadata *ev = (struct adk_sys_event_metadata *)em_event_to_nonem_param (event);
	printf("#### IP Offset at EM: %u \n", ev->ip_offset);
	pktLen = em_ext_event_size(event);

	printf("Received packet length = %d\n",pktLen);
	printf("Received packet with %d number of segments\n",em_ext_segment_count(event));

	for (int k = 0; k < em_ext_segment_count(event); ++k)
	{
		printf("\n\nseg %d %p size=%d\n", k, em_ext_segment_pointer(event, k), em_ext_segment_size(event, k)); 
		pktLen = em_ext_segment_size(event, k);
		msg = (uint8_t *)em_ext_segment_pointer(event, k);
		printPacket((unsigned char *)msg, pktLen);
	}

	em_free(event);
	s->core_stats[CORE_USED].stat.free++;
	printf("Core %d stats %" PRIu64 " free %" PRIu64 " alloc %" PRIu64 "\n", em_core_id(),
		   s->core_stats[CORE_USED].stat.rcv_ev_cnt, s->core_stats[CORE_USED].stat.free,
		   s->core_stats[CORE_USED].stat.alloc);

	switch(queue) {
	    case  EM_RECV_QUEUE_ID_IP:
            	/*Allocate IPv4 packet*/
            	event = em_alloc(sizeof(adk_app_ipv4_msg_t), EM_EVENT_TYPE_SW, EM_POOL_DEFAULT);
                ev = (struct adk_sys_event_metadata *)em_event_to_nonem_param (event);
                ev->udp_cksum_gen = 1;
            	/* Send event with IPv4 packet copied inside */
            	msg = em_event_pointer(event);
            	dest_queue = ns->nonem_queue[0];
            	memcpy((void *)msg, (void *)&ipPacket, sizeof(ipPacket));
            	printPacket((unsigned char *)msg, sizeof(ipPacket));
            	break;
	    case  EM_RECV_QUEUE_ID_IPV6:
            	/*Allocate IPv6 packet*/
            	event = em_alloc(sizeof(adk_app_ipv6_msg_t), EM_EVENT_TYPE_SW, EM_POOL_DEFAULT);
                ev = (struct adk_sys_event_metadata *)em_event_to_nonem_param (event);
                ev->udp_cksum_gen = 1;
            	/* Send event with IPv6 packet copied inside */
            	msg = em_event_pointer(event);
	    	dest_queue = ns->nonem_queue[1];
	    	memcpy((void *)msg, (void *)&ipv6Packet, sizeof(ipv6Packet));
	    	printPacket((unsigned char *)msg, sizeof(ipv6Packet));
	    	break;
	    default:
	        printf("Error: Packet recevied from unknown queue \n");
		break;
	}

	s->core_stats[CORE_USED].stat.alloc++;
	printf("Sending event %p to queue %lx\n", event, dest_queue);
	ret = em_send(event, dest_queue);
	CHECK_RET(ret != EM_OK, "Event send ret= (%u)! queue: %" PRI_QUEUE " \n", ret, dest_queue);
}

static void
printPacket(unsigned char *pkt, int pktLen)
{
	int i;

	printf("\n================= Len = %d =====================\n", pktLen);
	printf("Packet : ");
	for (i = 0; i < pktLen; i++)
	{
		printf("%02x ", pkt[i]);
		if ( i%16 == 0 )
			printf("\n");
	}
	printf("\n==============================================\n");
}

/*
 * Global init and startup of the non EM test application.
 *
 */
void
test_appl_adk_em_start(void)
{
	em_eo_t eo;
	em_status_t ret;
	em_queue_group_t group;

	em_core_mask_t mask;
	static char q_grp_name[] = "qgr";
	queue_context_common_t *q_ctx;
	int i = 0;

	printf("####### Running %s#########\n", __func__);

	/*
	 * Create the EO and 2 parallel queues, add them to the EO
	 */
	memset(&s->eo_context[0], 0, sizeof(eo_context_common_t));
	memset(&s->app_done, 0, sizeof(s->app_done));

	eo = em_app_eo_create("cre del appl", (em_start_func_t) non_em_start, NULL,
				  (em_stop_func_t) non_em_stop, NULL,
				  (em_receive_func_t) non_em_receive, &s->eo_context[0]);

	/* Init EO context */
	s->eo_context[0].this_eo_id = eo;
	s->eo_context[0].qgrp_num = 0;

	em_core_mask_zero(&mask);
	em_core_mask_set_count(em_core_count(), &mask);

	em_core_mask_zero(&mask);
	em_core_mask_set(CORE_USED, &mask);

	group = em_app_queue_group_create(q_grp_name, &mask, 0, NULL);
	q_ctx = &s->queue_context[s->eo_context[0].qgrp_num];

#ifdef EM_OEM
	q_ctx->queue_id = ((my_device_id << 16) | EM_QUEUE_STATIC_MIN);
#else
	q_ctx->queue_id = ((my_device_id << 16) | (EM_QUEUE_STATIC_MIN << 8)) + 1;
#endif
	ret = em_queue_create_static("staticQ", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_LOW, group,
					 q_ctx->queue_id);
	CHECK_RET(ret != -EM_OK, "Static queue creation EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n",
		  eo, q_ctx->queue_id);
	printf("Static queue created for IPv4 EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n", eo,
		   q_ctx->queue_id);
	ns->em_queue[i++] = q_ctx->queue_id;
	q_ctx++;  // Pointing to next element in array

	/*Dynamic Queue Creation*/
	q_ctx->queue_id =
		em_queue_create("dynamicQ", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_NORMAL, group);
	CHECK_RET(q_ctx->queue_id == EM_QUEUE_UNDEF,
		  "Dynamic queue creation EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n", eo,
		  q_ctx->queue_id);
	printf("Dynamic queue created for Dynamic EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n", eo,
		   q_ctx->queue_id);
	ns->em_queue[i++] = q_ctx->queue_id;
        q_ctx++;  // Pointing to next element in array

	/* EM queue create for IPv6 */
	q_ctx->queue_id = ((my_device_id << 16) | ((EM_QUEUE_STATIC_MIN+1) << 8)) + 1;
	ret = em_queue_create_static("staticQ", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_LOW, group,
					 q_ctx->queue_id);
	CHECK_RET(ret != -EM_OK, "Static queue creation EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n",
		  eo, q_ctx->queue_id);
	printf("Static queue created for IPv6 EO: %" PRI_EO ", queue: %" PRI_QUEUE "\n", eo,
		   q_ctx->queue_id);
	ns->em_queue[i++] = q_ctx->queue_id;
	q_ctx++; // Pointing to next element in array

	q_ctx = &s->queue_context[s->eo_context[0].qgrp_num];
	for (i = 0; i <= 2; i++) {
		q_ctx->queue_group = group;
		ret = em_queue_set_context(q_ctx->queue_id, q_ctx);
		CHECK_RET(ret != EM_OK, "em_queue_set_context ret=(%u):queue: %" PRI_QUEUE " \n",
			  ret, q_ctx->queue_id);

		ret = em_eo_add_queue(eo, q_ctx->queue_id);
		CHECK_RET(ret != EM_OK,
			  "add queue ret=(%u). EO: %" PRI_EO " queue: %" PRI_QUEUE "\n", ret, eo,
			  q_ctx->queue_id);

		q_ctx++;
	}
	create_nonem_queues();
}

void test_appl_adk_em_pre_init(void *em_config , void *adk_config_xml)
{
	em_conf_t *emcfg = (em_conf_t *)em_config;

	/* Parse common params for all test cases */
	if (adk_non_em_common_pre_init(em_config, adk_config_xml, &adkCfg) < 0)
	{
		ADK_APPL_EXIT_FAILURE("Failed to parse and configure XML!");
	}

	adk_sys_handle_t sys_handle;
	int argc = 1; 
	char *argv [20];
	
	/* Setting the Unified DP setup mode*
	 * EAL and HLP initialization wont be happening in DP setup under this mode*
	 */
	is_unified_em_adk_set = 1;
	/* Number of EM cores used, will be passed to ADK *
	 * And ADK will create eventDev port for EM cores *
	 */
	number_em_cores = emcfg->core_count;

	char str[512];
	strncpy(str, adkCfg.cmd_line_args, 512);

	char *p2 = strtok(str, " ");
	while (p2)
	{
		argv[argc++] = p2;
		p2 = strtok(0, " ");
	}
	argv[argc] = 0;
        argv[0] = NULL;

	int arg_c = argc;

	printf("Following are the arguments passed for ADK-DP setup :\n");
	while(arg_c ){
		printf("argv[%d] %s ", argc-arg_c, argv[argc-arg_c]);
		arg_c--;
	}

	if (adk_dp_setup(argc, argv) != 0) {
		printf("adk_dp_setup() failed");
		return ;
	}

	sys_handle = init_adk_modules();
	if(!sys_handle){
		printf("Failed in System init \n");
		return ;
	}

	if (adk_dp_init(sys_handle)){
		printf("Failed to init DP \n");
		return ;
	}

	emcfg->nonem_cfg[INDEX].start_port_id = em_start_port ; // Number of evdev Ports used by ADK

	printf("Successfully completed DP initializations!\n");
}

/*
 * Verify the stats of  test application.
 *
 */
void
test_appl_adk_em_stop(void)
{
	uint32_t waitCount = 0;
	char thisFile[] = __FILE__;

	/* Point to shared data */
	s = (shared_app_common_t *) shm_app_addr;
	if (!s) {
		printf("ERROR: s==NULL, line: %d func:%s#########\n", __LINE__, __FUNCTION__);
		exit(0);
	}

	while (!s->app_done && !s->app_failed) {
		usleep(10);
		waitCount++;
		if (waitCount > TIME_OUT) {
			printResult(TIMEOUT, curr_app, basename(thisFile));
			return;
		}
	}

	if (s->app_failed) {
		printResult(API_FAILED, curr_app, basename(thisFile));
		return;
	}
	printResult(PASS, curr_app, basename(thisFile));

	printf("Core %d received stats %" PRIu64 "  free %" PRIu64 " alloc %" PRIu64 "\n",
		   CORE_USED, s->core_stats[CORE_USED].stat.rcv_ev_cnt,
		   s->core_stats[CORE_USED].stat.free, s->core_stats[CORE_USED].stat.alloc);

	printf("\n Please press q to quit the appilcation \n");
}

void
test_appl_adk_em_init_local(void)
{
	s = (shared_app_common_t *) (shm_app_addr);
	nem_qcfg = (uint8_t *) shm_app_addr + sizeof(shared_app_common_t);
	ns = (em_adk_common_t *) (((uint8_t *) shm_app_addr) + EM_APP_DATA_SIZE -
					sizeof(em_adk_common_t));
}
