/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*
 * ============================================================================
 *
 *       Filename:  adk_dp_app_config.h
 *
 * ============================================================================
 */

#ifndef __ADK_DP_APP_CONFIG_H__
#define __ADK_DP_APP_CONFIG_H__

#include "adk_sys.h"
#ifdef EM_SNF_ADK
#include "em_common.h"
#include "em_config.h"
#endif
#define   SELECT_INTERFACE_ENABLED
#undef   ENABLE_TX

/*
 * Note: RTE_MAX_LCORE is 128, but only 64 cores are supported
 * in the core mask parsing logic
 */
#define DP_APP_MAX_LCORES 64

/* Maximum number of DPDK ports to handle */
#define DPDK_MAX_PORTS      64

/* DPDK port numbering starts from 0 */
#define DPDK_BASE_PORT_ID   0

#define BATCH_SIZE           32

#define STAGE1_QUEUE_INDEX  0
#define OAM_QUEUE_INDEX     1

#define NUM_NIC_PORTS   2

/* Mask for running samples with either IPv4 or IPv6 or both */
#define SAMPLE_USE_IPV4   0b01
#define SAMPLE_USE_IPV6   0b10

/* Number of FPS sockets */
#define NUM_FPS_SOCKS   2
#define FPS_LOCAL_IP_ADDR_BASE  0x64646401
#define FPS_REMOTE_IP_ADDR_BASE 0xc8c8c801
#define FPS_LOCAL_PORT  2152
#define FPS_REMOTE_PORT 2152

/* eMBMS multicast IP address */
#define FPS_EMBMS_LOCAL_IP_ADDR 0xE0000101

#define NUM_BB_CONFIG_FPS_SOCKS   1
#define FPS_LOCAL_BB_CONFIG_ADDR  0xA

/* Multicast PCEP ID per multicast group */
#define FPS_MULTICAST_BB_CONFIG_ADDR  0x2A
#define FPS_MULTICAST_BB_CONFIG_SESSION  0x11

#define FPS_BB_CONFIG_SESSION_ID  0x22

#define NUM_ADK_ULI_QIDS_FROM_EM 1
#define NUM_WORKERS          2
#define NUM_QIDS_PER_WORKER  4
#define NUM_PH_THREADS       4

/* bridge driver informaiton */
/* CPPI configs */
#define MAX_CPPI_DEVICES        1
#define MAX_CPPI_WORKERS        16

#define CPPI_INST_ID_BASE       0

#ifdef SELECT_INTERFACE_ENABLED
#define CP_READ_FD  0
#define DP_WRITE_FD 1
#define ADK_POLL_NO_TIMEOUT -1

#define MESSAGE_PIPE "/tmp/udprelay_eventdev_ph_named_pipe"
#define FIFO_MODE (S_IRWXU | S_IRWXG | S_IRWXO)
#endif

#define DP_APP_EVENTDEV_ENQUEUE_DEPTH 64
#define DP_APP_EVENTDEV_DEQUEUE_DEPTH 32
#define DP_APP_EVENTDEV_WORKER_DEQUEUE_DEPTH 8

#ifdef EM_SNF_ADK

/* Number of non-em devices in configuration */
#define INDEX 0

#define VIRT_CORE_0  25

#define NUM_EM_QUEUES 4
#define NUM_NONEM_QUEUES 10

typedef struct {
   em_queue_t em_queue[NUM_EM_QUEUES];
   em_queue_t nonem_queue[NUM_NONEM_QUEUES];
   int rcv_cnt;
} em_nonem_common_t;

typedef struct {
        uint8_t event_type;
        /* Sequence number */
        uint32_t seq;
        uint8_t data[];
} nonem_event_t;

#define EM_APP_DATA_SIZE (8*1024*1024)

/** Get rid of path in filename - only for unix-type paths using '/' */
#define NO_PATH(file_name) (strrchr((file_name), '/') ? \
             strrchr((file_name), '/') + 1 : (file_name))

#define ADK_APPL_EXIT_FAILURE(...)  do {         \
   fprintf(stderr,               \
         "Appl Error: %s:%i, %s() - ",    \
         NO_PATH(__FILE__), __LINE__, __func__); \
   fprintf(stderr, __VA_ARGS__);       \
   fprintf(stderr, "\n\n");         \
   fflush(stderr);               \
   exit(1);          \
} while (0)

#endif

#define ADK_ETH_DEV_ID           3

extern uint8_t ip_ver_in_use;

#define ADK_SAMPLE_USES_V4  (ip_ver_in_use & SAMPLE_USE_IPV4)
#define ADK_SAMPLE_USES_V6  (ip_ver_in_use & SAMPLE_USE_IPV6)

int adk_dp_setup(int argc, char **argv);
int adk_dp_init(adk_sys_handle_t sysHandle);
adk_sys_handle_t init_adk_modules(void);

void adk_ipt_flows_setup(uint16_t vsi);
void ipt_flows_setup(void);

int adk_dp_shutdown(void);

void adk_dump_dev_stats(uint8_t opt);

void adk_dump_port_stats(uint16_t port);

void adk_dump_ies_table(void);

#ifdef ADK_HOST_PATH_USE_NETD

uint16_t 
init_adk_slowpath_dev(adk_sys_handle_t sys_handle);

int shutdown_adk_netd(void);

#endif

void dp_app_egress_qos_create(adk_sys_handle_t sys_handle, uint16_t port_id);
void dp_app_egress_qos_delete(adk_sys_handle_t sys_handle, uint16_t port_id);
adk_handle_t dp_app_egress_qos_handle_get(uint16_t port_id, uint8_t cpk_port, uint8_t id);
int dp_app_qos_queue_stats_print(adk_sys_handle_t sys_handle,  adk_handle_t qh); 

uint32_t dp_app_cpk_get_l1_node(uint16_t dpdk_eth_dev_id,
                                 uint32_t hlp_port_num);
uint32_t dp_app_cpk_get_vport_id(uint16_t dpdk_eth_dev_id,
                                  uint32_t hlp_port_num);
int dp_app_cpk_BB_CONFIG_qos_hierarchy_create(uint16_t vport_dev_id,
                                          uint32_t parent_node_id,
                                          uint32_t *child_node_id);





void dp_app_sp_qos_create(adk_sys_handle_t sys_handle, uint16_t dev_id);
void dp_app_sp_qos_delete(adk_sys_handle_t sys_handle, uint16_t dev_id);
adk_handle_t dp_app_sp_qos_handle_get(uint16_t dev_id, uint16_t id);
#ifdef EM_SNF_ADK
int adk_non_em_common_pre_init(void *em_config , void *adk_config_xml, adk_xml_cfg_t *adkCfg_p);
#endif
#endif /* ifndef __ADK_DP_APP_CONFIG_H__ */

