/****************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  test_ph.h
 *
 *      $HeadURL:
 *      $Revision:
 *      $Date:
 * @addtogroup _adkSamples ADK Sample Applications
 * @{
 *
 * @brief Internal function used by the ADK sample applications.
 *
 * This file contains the functions used by the ADK samples for sending
 * packets to PH, due to exceptions or for packet capture.
 *
 ***********************************************************************/

#ifndef __ADK_SAMPLE_COMMON_H__
#define __ADK_SAMPLE_COMMON_H__

#include "test_shm.h"
#include "adk_test_shm.h"
#include "adk_ipsec.h"
#include "adk_ph_dev.h"
#include "adk_ip.h"
#include "adk_ipv6.h"
#include "adk_ip_utility.h"
#include <rte_pmd_ice_dsi_tm.h>

#define CPK_HLP_PORT            20
#define CPM_ENCRYPT_PORT        21
#define CPM_DECRYPT_PORT        22

#ifdef IES_DEBUG_TRACE
#define ies_trace(fmt, args...) printf("\n[IES_TRACE] " fmt, ##args)
#else
#define ies_trace(fmt, args...)
#endif

#define MAX_PAYLOAD 2048 /* maximum payload size*/

#define NUM_PORTS       20
#define ADK_HLP_PORT_NUM_START 0
#define BG_ID           20   /* Set to non-zero to create tagged ports */

#define APP_L2_DOMAIN_MASK  0x1ff
#define APP_MAC_ADDR_MASK       0xFFFFFFFFFFFF
#define APP_VSI_QUEUE_TBL_MIN_RULES 8
#define APP_VSI_QUEUE_TBL_MAX_RULES 8
#define APP_L2_DOMAIN_WIDTH 9
#define APP_MAC_ADDR_WIDTH  48
#define APP_VSI_QUEUE_TBL_IDX 0xfff

/*QoS definitions*/
#define QOS_VPORT_NUM_LEAF_NODES  8
#define QOS_SHAPING_RATE_10G        (10000000000)
#define QOS_SHAPING_RATE_1G         (1000000000)
#define QOS_DEFAULT_PRIORITY 0
#define QOS_DEFAULT_WEIGHT   1
#define QOS_PORT_ROOT_NODE_ID 0x2000
#define QOS_SL_NODE_ID 0x4000
#define QOS_FL_NODE_ID 0x6000
#define QOS_SL_SHAPER_PROFILE_ID 1
#define QOS_FL_SHAPER_PROFILE_ID 2
#define QOS_SL_NUM_QUEUES 2
#define QOS_FL_NUM_QUEUES 8
#define QOS_FL_SP_NUM_QUEUES 3
#define QOS_FL_WRED_NUM_QUEUES 4
#define QOS_FL_DEFAULT_QUEUE_ID 7
#define QOS_WFQ_NUM_QUEUES 5
#define MAX_L3_INTF 64
#define MAX_L2_INTF 64 
#define MAX_SHAPER_PROFILES 64 
#define QOS_FL_MAX_NODES 128

#define NON_ZERO_STATS(x,fmt,args...) \
do {\
    if (x) {\
        printf(fmt,##args);\
    }\
}while(0);

typedef struct shaper_profile_s{
    uint32_t profile_id;
    bool configured;
    uint8_t ref_cnt;
}shaper_profile_t;

typedef struct qos_non_leaf_node_counts_s{
    uint64_t txPkts;
    uint64_t txBytes;
}qos_non_leaf_node_stats_t;

typedef struct qos_fl_sched_s{
	uint32_t node_id;
    adk_interface_handle_t enet_intf_handle;
    adk_interface_handle_t l3_intf_handle;
    shaper_profile_t shaper_profile;
    uint16_t tx_queues[QOS_FL_NUM_QUEUES];
}qos_fl_sched_t;

typedef struct qos_sl_sched_s{
	uint32_t node_id;
    adk_interface_handle_t l2_intf_handle;
    shaper_profile_t shaper_profile;
    uint16_t tx_queues[QOS_SL_NUM_QUEUES];
    adk_interface_handle_t ip_if_handle[QOS_FL_MAX_NODES];
}qos_sl_sched_t;

/*QoS definitions ends */

/* Default MAC AGing timer value in minutes */
#define ADK_ETH_DEFAULT_MAC_AGING_TIMER     5

typedef struct domain_mapper_tbl_rule_attr_s
{
    uint32_t port;
    uint32_t vsi_id;
    uint16_t vlan_id;   /* Set to ADK_HOS_ANY_VLAN to wildcard this field */
    uint16_t eth_sw_domain;
    uint8_t vrd_id;
    bool enable_mac_learning;
    bool is_ivl;
} domain_mapper_tbl_rule_attr_t;

adk_status_t
domain_mapper_tbl_rule_add(
    domain_mapper_tbl_rule_attr_t *rule_attr,
    int *domain_id);

adk_status_t
domain_mapper_tbl_rule_delete(int domain_id);

adk_status_t
config_mac_aging (
    uint32_t port,
    uint32_t age_out_time); /* In millisecs */

adk_status_t
ies_vlan_create_helper (
    uint16_t vlan_id);

adk_status_t
ies_vlan_delete_helper (
    uint16_t vlan_id);

adk_status_t
ies_vlan_port_add_helper (
    uint16_t vlan_id,
    uint32_t port);

adk_status_t
ies_vlan_port_rem_helper (
    uint16_t vlan_id,
    uint32_t port);

typedef void (*ies_event_handler_callback_fn)(const struct ies_event *ies_event);

int setup_hlp(ies_event_handler_callback_fn ies_func_adk);

#ifdef ADK_NETD_NETLINK
adk_status_t netd_phy_interface_up(void);
#endif

adk_status_t direct_traffic_phy(adk_hos_rule_id_t* default_vsi_q_rule_id);
adk_status_t direct_traffic_remove(adk_hos_rule_id_t* default_vsi_q_rule_id);

void print_ip_addr_stats(adk_ip_address_stats_t *st);

void print_ip_global_stats(adk_ip_global_stats_t *ipStats);
void print_ipsec_global_stats(adk_module_handle_t ipsec_handle, bool ipv6_mode);

void print_ip_l4_stats(adk_ip_l4_stats_t *ipL4Stats);

void print_ip_interface_stats(adk_ip_interface_stats_t *ipIfaceStats);

void print_ip_utility_stats(adk_module_handle_t ipHandle);

uint32_t get_ip_if_handle_index_in_SL(qos_sl_sched_t *sl_node_info_p, adk_interface_handle_t ip_if_handle);
qos_sl_sched_t *get_sl_node_info_from_enet_if(adk_interface_handle_t enet_if_handle);
qos_fl_sched_t *get_fl_node_info_from_ip_if(adk_interface_handle_t ip_if_handle);
bool is_shaper_profile_configured(uint32_t shaper_profile_id, uint32_t *index);
int cpk_qos_fl_sched_create(adk_sys_handle_t sysHandle,
                            uint16_t vport_id,
							uint16_t vport_dev_id,
							uint16_t parent_node_id,
 							uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle,
 							adk_interface_handle_t ip_if_handle);

int cpk_qos_fl_sched_delete(adk_sys_handle_t sysHandle,
                            uint16_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t ip_if_handle);

int cpk_qos_sl_sched_create(adk_sys_handle_t sysHandle,
                            uint8_t vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle);
int cpk_qos_sl_sched_delete(adk_sys_handle_t sysHandle,
                            uint8_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle);

int cpk_qos_fl_stats_read(uint16_t vport_dev_id,
                          adk_interface_handle_t ip_if_handle);

int adk_qos_phb_stats_read(adk_sys_handle_t sysHandle,
                           adk_interface_handle_t ip_if_handle);

int adk_qos_sl_queue_stats_read(adk_sys_handle_t sysHandle,
                          adk_interface_handle_t enet_if_handle);

int cpk_qos_sl_stats_read(uint16_t vport_dev_id,
                          adk_interface_handle_t enet_if_handle);

int configure_ip_qos_classifier(adk_module_handle_t modHandle, adk_interface_handle_t ip_if_handle, bool isIPv6); 
int configure_ip_ingress_qos_classifier(adk_module_handle_t modHandle, adk_interface_handle_t ip_if_handle, bool isIPv6); 

void print_policing_stats(adk_sys_handle_t sysHandle, adk_qos_policer_handle_t policing_handle);

int cpk_qos_get_sl_sched_stats(uint16_t vport_dev_id,
                               adk_interface_handle_t enet_if_handle,
                               qos_non_leaf_node_stats_t *node_stats);

int cpk_qos_get_fl_sched_stats(uint16_t vport_dev_id,
                              adk_interface_handle_t ip_if_handle,
                              qos_non_leaf_node_stats_t *node_stats);

void print_ip_routes(adk_module_handle_t ip_handle, uint8_t ipRouteTblId);

adk_status_t ip_print_qos_policies_all(adk_module_handle_t ipHandle, adk_interface_handle_t ipIfHandle);

adk_status_t ip_print_ingress_classifier_all(adk_module_handle_t ipHandle);

adk_status_t ip_print_pbr_policies_all(adk_module_handle_t ipHandle); 

#endif /* ifndef __ADK_SAMPLE_COMMON_H__ */

/*! end addtogroup _adkSamples
 * @}
 */
