/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_sample_os.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _adkSamples ADK Sample Applications
 * @{
 *
 * @brief OS abstraction layer APIs. 
 *
 * This file contains all OS specific APIs that are not abstracted by
 * RTE. The APIs in this file should only be used in the sample
 * application, these must not be used in the ADK core code.
 *
 ***********************************************************************/

#ifndef __ADK_SAMPLE_OS_H__
#define __ADK_SAMPLE_OS_H__

#include "adk_ph_dev.h"
#include <semaphore.h>
#include <pthread.h>

/* Sempahore related porting */
typedef sem_t adk_sample_sem_t;
typedef pthread_t adk_sample_thread_id_t;

/* Semaphore init */
#define ADK_SAMPLE_SEM_INIT(__semaphore, __shared, __initValue) \
    { \
        sem_init (&(__semaphore), (__shared), (__initValue));\
    }

#define ADK_SAMPLE_SEM_WAIT(__semaphore) \
    { \
        sem_wait (&(__semaphore)); \
    }

#define ADK_SAMPLE_SEM_POST(__semaphore) \
    { \
        sem_post (&(__semaphore)); \
    }

#define ADK_SAMPLE_SEM_DESTROY(__semaphore, __shared, __initValue) \
    { \
        sem_destroy (&(__semaphore)); \
    }

/* Macro to create the thread, it needs the PH process config
 * data.
 */
#define ADK_SAMPLE_THREAD_CREATE(__pduProcessConfigData, __threadId) \
    { \
        { \
            if (pthread_create (&(__threadId), \
                                NULL, \
                                adk_ph_process_pdu_fn, \
                                &(__pduProcessConfigData)) != 0) \
            { \
                printf ("Failed to create the thread, FAILED\n"); \
            } \
        } \
    }

#define ADK_SAMPLE_THREAD_CREATE_V2(__pduProcessConfigData, __threadId) \
    { \
        { \
            if (pthread_create (&(__threadId), \
                                NULL, \
                                adk_ph_process_pdu_fn_v2, \
                                &(__pduProcessConfigData)) != 0) \
            { \
                printf ("Failed to create the thread, FAILED\n"); \
            } \
        } \
    }

/* Macro to join the thread. */
#define ADK_SAMPLE_THREAD_JOIN(__threadId) \
    { \
        if (pthread_join ((__threadId), \
                          NULL) != 0) \
        { \
            printf ("Failed to wait for the thread, FAILED\n"); \
        } \
    }

/* Macro to exit the thread. */
#define ADK_SAMPLE_THREAD_EXIT() \
    { \
        pthread_exit (NULL); \
    }

#endif /* __ADK_SAMPLE_OS_H__ */
 
/*! end addtogroup _adkSamples
 * @}
 */
