/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  test_ph.h
 *
 *      $HeadURL:
 *      $Revision:
 *      $Date:
 * @addtogroup _adkSamples ADK Sample Applications
 * @{
 *
 * @brief Internal function used by the ADK sample applications.
 *
 * This file contains the functions used by the ADK samples for sending
 * packets to PH, due to exceptions or for packet capture.
 *
 ***********************************************************************/

#ifndef __ADK_SAMPLE_PH_H__
#define __ADK_SAMPLE_PH_H__

#include "test_shm.h"
#include "adk_test_shm.h"

#include "adk_ph.h"
#include "adk_ph_dev.h"

#define IPV6_CBFNID         1
#define IPV6_PAKCAP_CBFNID  3
#define IPV4_CBFNID         4
#define IPV4_PAKCAP_CBFNID  6

#define PH_NONE 0
#define PH_DEV  3

#define IPv6_DEVICE PH_THREAD_1
#define IPv4_DEVICE PH_THREAD_1
#define IPv6_THREAD 0
#define IPv4_THREAD 0

extern adk_module_handle_t ip_handle, ipv6_handle, enet_handle;

extern test_config_t *testConfShm_p;

typedef struct test_config_index_s {
    int             phIndex;
    test_config_t  *testConfShm_p;
} test_config_index_t;

extern test_config_index_t exceptionRock[NUM_PH_THREADS];

int init_ph_dev_thread(uint8_t phIndex, adk_ph_dev_device_type_t type);
adk_status_t ph_dev_cleanup(uint8_t devId);

void cppi_stats_print(void);

adk_status_t
cp_init_ph_threads(
    int thread_1,
    int thread_2,
    int thread_3,
    int thread_4);

#endif /* ifndef __ADK_SAMPLE_PH_H__ */

/*! end addtogroup _adkSamples
 * @}
 */
