/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*
 * ============================================================================
 *
 *       Filename:  adk_test_shm.h
 *
 *    Description:  Header file for testing structures.
 *
 *        Version:  1.0
 *        Created:  24 Apr, 2017
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Scott Titus (sdt), scott.titus@lsi.com
 *        Company:  LSI
 *
 * ============================================================================
 */

#ifndef __ADK_TEST_SHM_H__
#define __ADK_TEST_SHM_H__

#include "rte_pci.h"
#include "test_shm.h"

#include "adk_dp_app_config.h"

#include "adk_pal.h"
#include "adk_ph_dev.h"
#include "adk_dp_app_config.h"
#include "adk_ip.h"

#define NUM_NIC_PORTS   2

#define MAX_NAC_PORTS   20

#define DP_APP_VSI_MAX  4
#define DP_APP_HLP_PORTS 21
#define DP_APP_QUEUES_PER_VPORT 1

#define NUM_WORKERS          2
#define NUM_QIDS_PER_WORKER  4
#define NUM_PH_THREADS       4

#define TEST_CONFIG_MEMORY_POOL "adk_test_config_pool"

#ifdef ADK_HOST_PATH_USE_NETD
/* CPPI/Netdriver configs */
#define MAX_CPPI_DEVICES        1
#define MAX_CPPI_WORKERS        16

#define CPPI_INST_ID_BASE       0

#define MAX_CPPI_APP_INSTANCES  1
#define NETD_MAX_PORTS          64
#define CPPI_BURST_SIZE         16
#define CPPI_QUEUE_CACHE_SZ     64

#define CPPI_DEV_ID_BASE        0
#define CPPI_FQ_ID_BASE         0
#define CPPI_AQ_ID_BASE         100
#define CPPI_RXQ_ID_BASE        200
#define CPPI_TXQ_ID_BASE        300
#endif

#ifdef SELECT_INTERFACE_ENABLED
#define CP_READ_FD  0
#define DP_WRITE_FD 1
#define ADK_POLL_NO_TIMEOUT -1

#define MESSAGE_PIPE "/tmp/udprelay_eventdev_ph_named_pipe"
#define FIFO_MODE (S_IRWXU | S_IRWXG | S_IRWXO)
#endif

#define TEST_MAX_PCI_STR_LEN    100


/*!
 * =============================================================
 * @internal
 * @struct dpdk_eth_dev_pci_t  ""
 *
 * @brief PCI description of the DPDK EthDev ports
 *
 * =============================================================
 */
typedef struct dpdk_eth_dev_pci_t_s {
    uint8_t ethDevId;
    uint8_t ifIndex;
    struct rte_pci_addr addr;
    struct rte_pci_id   id;
    char    deviceName[TEST_MAX_PCI_STR_LEN];
    char    driverName[TEST_MAX_PCI_STR_LEN];
} dpdk_eth_dev_pci_t;

/*!
 * =============================================================
 * @internal
 * @struct test_dpdk_eth_dev_ports_t  ""
 *
 * @brief Information describing the EthDev ports DPDK owns
 *
 * =============================================================
 */
typedef struct test_dpdk_eth_dev_ports_s {
    uint8_t                 numTnPorts;
    uint8_t                 numHostPorts;
    uint8_t                 numEthDevs;
    uint16_t                vport_id;
    uint16_t                vdev_port_id;
    uint8_t                 tnPorts_p[DPDK_MAX_PORTS];
    uint8_t                 hostPorts_p[DPDK_MAX_PORTS];
    dpdk_eth_dev_pci_t      ethDevInfo_p[DPDK_MAX_PORTS];
    uint8_t                 bridge_eth_instance_id; 
} test_dpdk_eth_dev_ports_t;

/*!
 * =============================================================
 * @internal
 * @struct test_conf_ph_dev_t udprelay_ph.h "udprelay_ph.h"
 *
 * @brief Items for configuring PH dev
 *
 * Detailed description starts here
 *
 * =============================================================
 */
typedef struct test_conf_ph_dev_s {
    uint8_t                     devId;
    uint8_t                     portId;
    uint16_t                    numRxQueues;
    uint16_t                    numTxQueues;
    uint16_t                    numCallbackFns;
    bool                        interruptEnabled;
    int                         dpToCpPipe_fds[2];
    uint16_t                    rxQueueId[ADK_PH_DEV_MAX_NUM_QUEUES];
    struct rte_ring            *rxRing[ADK_PH_DEV_MAX_NUM_QUEUES];
    uint16_t                    txQueueId[ADK_PH_DEV_MAX_NUM_QUEUES];
    struct rte_ring            *txRing[ADK_PH_DEV_MAX_NUM_QUEUES];
    int                         ipv4PktCapReenable[NUM_NIC_PORTS];
    adk_flow_handle_t           ipv6PktCapHandle[NUM_NIC_PORTS];
    int                         ipv6PktCapReenable[NUM_NIC_PORTS];
    int                         phPoll;
    int                         phPollFail;
    int                         ipv4UnknownCbFnId;
    int                         ipv6UnknownCbFnId;
    int                         phIntr;
    int                         phIntrFail;
} test_conf_ph_dev_t;   /* ------  end of struct test_conf_ph_dev_t  ------ */


/*!
 * =============================================================
 * @internal
 * @struct test_config_t  udp6relay_ph.h "udp6relay_ph.h"
 *
 * @brief Structure for variables shared across processes
 *
 *
 * =============================================================
 */
typedef struct test_config_s
{
    test_dpdk_eth_dev_ports_t       ethDev;
    adk_ph_handle_t                 phHandle[NUM_PH_THREADS];
    adk_ph_pipeline_bind_handle_t   phPipelineBindHandle[NUM_PH_THREADS];
    adk_interface_handle_t          ipv6If[NUM_NIC_PORTS];
    adk_interface_handle_t          ipv4If[NUM_NIC_PORTS];
    adk_interface_handle_t          enetIf_IPv4[NUM_NIC_PORTS];
    adk_interface_handle_t          enetIf_IPv6[NUM_NIC_PORTS];

    /* VSI-ETH-DEVS */
    uint16_t            tn_vsi_eth_dev;
    uint16_t            tn_pcap_vsi_eth_dev;
    uint16_t            BB_CONFIG_vsi_eth_dev;

    /* per worker qm related information to be passed to ADK-QAL-Init */
    adk_pal_worker_info_t  evdev_port_list[DP_APP_MAX_LCORES];

    /* queue-id to be bind with worker ports */
    int qid[NUM_QIDS_PER_WORKER*NUM_WORKERS];

    uint8_t             evdev_id;
    uint8_t             eth_tx_qid;
    uint32_t            dpLcoreList[DP_APP_MAX_LCORES];
    uint32_t            numDpLcores;
    struct rte_mempool *packet_pool;

    uint16_t            numDevices;
    uint16_t            phDevToPhIndex[ADK_PH_DEV_MAX_NUM_DEVICES];
    test_conf_ph_dev_t  phDev[ADK_PH_DEV_MAX_NUM_DEVICES];

    int     retValue;
} test_config_t;   /* ------  end of struct test_config_t  ------ */

extern test_config_t   *testConfShm_p;

#endif /* ifndef __ADK_TEST_SHM_H__ */

