/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef __IPT_NAC_SETUP_H__
#define __IPT_NAC_SETUP_H__

#include <getopt.h>
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>

#include <rte_pmd_ice_dsi.h>

#include <ies_sdk.h>
#include <ies_event_types.h>
#include <ies_api_port.h>

/*
#define IES_DEBUG_TRACE
*/

#ifdef IES_DEBUG_TRACE
#define ies_trace(fmt, args...) printf("\n[IES_TRACE] " fmt, ##args)
#else
#define ies_trace(fmt, args...)
#endif

#define E_IPT_SWITCH_ID     (0x00)
#define IES_SHM_KEY         (1234)

#define E_IPT_IES_DOMAIN_MAPPER_PREC_1  (4000)
#define E_IPT_IES_DOMAIN_MAPPER_PREC_2  (4001)
#define E_IPT_IES_DOMAIN_MAPPER_PREC_3  (4002)

#define E_IPT_IES_PRI_PROFILE (0x01)

#define E_IPT_CPK_PORT          (20)
#define E_IPT_CPM_EGRESS_PORT   (22)

#define E_IPT_L2_DOMAIN         (0x0)
#define E_IPT_L2D_SCENARIO_ID   (0xF)

#define E_IPT_TRIG_GROUP_ID  (10000 - 1)

#define E_IPT_SCENARIO  (IES_MAP_SCEN_IDX_INGRS_NON_IP + 1)

#define E_IPT_FPP_BB_CONFIG_HASH_TBL_IDX (20)
#define E_IPT_BB_CONFIG_FPP_TBL_LOC  IES_ACL_FFU_GROUP_2
#define E_IPT_BB_CONFIG_HASH_PROFILE_ID     (48)
/*
 * The bit mask for 0th position starts after the ether type field.
 * To Do: Clarification about bit mask is pending in JIRA CSSY-1240.
 */
#define E_IPT_BB_CONFIG_PCEP_HLP_DI_BYTE_BIT_MASK     (0x3000)

extern char ipt_ies_error_name[IES_ERROR_MAX_NAME_SIZE];


#define debug printf
#define IPT_IES_ERROR_NAME(ies_ret) \
    ((ies_error_get_name(ies_ret, ipt_ies_error_name, \
                         IES_ERROR_MAX_NAME_SIZE) == IES_OK) ? \
     ipt_ies_error_name : "ERROR_UNKNOWN")

/**********************
 * HLP Configurations *
 *********************/

#include <ies_sdk.h>
#include <ies_event_types.h>
#include <ies_api_port.h>

void ies_event_handler_func_adk(const struct ies_event *ies_event);
void print_switch_state(enum ies_switch_state ies_sw_state);
int setup_hlp(void);
int e_ipt_hlp_port_config(int port_id);
int e_ipt_lan_tx_trig_config(void);
int e_ipt_pri_map(void);
int e_ipt_domain_add(int hlp_port, int vlan, int vsi);
int e_ipt_scenario_add(int hlp_port, int vlan);
int e_ipt_l2fdb_add(uint32_t l2d, uint32_t vlan, uint64_t dmac, int32_t  lport);
int e_ipt_BB_CONFIG_terminate_dmac(uint64_t dmac, int vlan, int vsi_id, int vsi_q);

int e_ipt_vlan_create ( uint16_t vlan_id);
int e_ipt_vlan_port_add ( uint16_t vlan_id, uint32_t port);

int terminate_hlp(void);


int e_ipt_setup_hlp(void);



#endif /* ifndef __IPT_NAC_SETUP_H__ */

