/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"

#include <adk_dp_app_config.h>
#include <getopt.h>
#include <stdint.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <fcntl.h>

#ifdef SELECT_INTERFACE_ENABLED
#include <sys/stat.h>
#include <signal.h>
#include <poll.h>
#endif

#include <rte_config.h>
#include <rte_eal.h>
#include <rte_version.h>
#include <rte_mempool.h>
#include <rte_mbuf.h>
#include <rte_launch.h>
#include <rte_malloc.h>
#include <rte_random.h>
#include <rte_cycles.h>
#include <rte_ethdev.h>
#include <rte_eventdev.h>
#include <rte_event_framework.h>
#include <rte_bridge_eth.h>
#include <rte_bridge_timer.h>
#include <rte_bridge_cppi.h>
#include <rte_bridge_crypto.h>
#ifdef ADK_DEVICE_SNR
#include <rte_pmd_ice_dsi.h>
#include <ipt_nac_setup.h>
#endif


#define ADK_USE_COMPLETION_QUEUE

#if ADK_DPDK_VERSION >= 18050016
#include <rte_bus_vdev.h>
#endif

/* ADK header files */
#include <adk_ethernet.h>
#include <adk_ip.h>
#include <adk_ipv6.h>
#include <adk_qos_include.h>
#include "adk_modules_enabled.h"
#include <adk_ipsec.h>

#include <adk_sys.h>
#include <adk_pal_dp_export.h>
#include <adk_ph_dev.h>
#include <adk_dev_net.h>

#include "adk_test_shm.h"
#include "test_shm.h"

#include "adk_sample_dpdk_ports.h"
#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <rte_log.h>
#include "adk_nd_common.h"

/* Added as pert of ADK-EM integration */
#include "pdk_include.h"

/* Added as part of the service core migration */
#include <rte_service.h>
#include <rte_service_component.h>
#include <rte_bridge_eth_priv.h>
#include <rte_bridge_timer_priv.h>
#include <rte_bridge_crypto_priv.h>
#include <rte_bridge_cppi_priv.h>
#include <rte_event_queue_dispatcher.h>
/* #define RDK_DEBUG_TRACE */

#ifdef RDK_DEBUG_TRACE
#define rdk_trace(fmt, args...) printf("\n[RDK_TRACE] " fmt, ##args)
#else
#define rdk_trace(fmt, args...)
#endif

#ifdef ADK_DEVICE_SNR
#define DP_APP_SNR_EGRESS_QOS
#include <libae_client.h>
struct vsi_info_t{
    uint16_t    vports;
    uint16_t    rx_queues;
    uint16_t    tx_queues;
};
uint16_t tx_qs, rx_qs;
#ifdef ADK_USE_COMPLETION_QUEUE
uint16_t cp_qs;
#endif
#define SHAPING_RATE_1G         (1000000000)
#define SHAPING_RATE_100M        (100000000)
#define SHAPING_RATE_10M          (10000000)
#define SHAPING_RATE_1M            (1000000)
#define DP_APP_VSI_MAX  4
#define DP_APP_HLP_PORTS 21
#define DP_APP_QUEUES_PER_VPORT 1
#define MAX_NUM_EVENTS 8192
uint16_t  vport_dev_list[DP_APP_VSI_MAX][DP_APP_HLP_PORTS];
uint32_t  vport_l1_node_list[DP_APP_VSI_MAX][DP_APP_HLP_PORTS];

#define     DP_APP_DEV_NET_RL_QUEUES 8
adk_handle_t    queue_list[DP_APP_VSI_MAX][DP_APP_HLP_PORTS][DP_APP_QUEUES_PER_VPORT];
adk_handle_t    devNetRlQHandle[2*RTE_MAX_ETHPORTS][DP_APP_DEV_NET_RL_QUEUES] = {0};

#endif

/*
 *0 - Unused (TODO: Need to relook into this)
 *1 - IPv4
 *2 - IPv6
 */
#define NUM_ADK_ENTRY_POINT 3
test_config_t *testConfShm_p = NULL;
bool is_primary = false;

#ifndef debug_printf
#  ifdef ADK_DEBUG
#define debug_printf(fmt, args...) printf(fmt, ##args)
#  else
#define debug_printf(fmt, args...)
#  endif
#endif

/******* Module initialization variables and CBs *********/
#define MAX_NO_OF_MODULES 4
#define MODULE_NAME_LEN_MAX 50
#define NUM_PORTS 2

uint8_t is_unified_em_adk_set = 0;
uint8_t number_em_cores = 0;
uint8_t em_start_port = 10;

char module_name[MAX_NO_OF_MODULES][MODULE_NAME_LEN_MAX] = {
        ADK_IP_MODULE_NAME,
        ADK_IPV6_MODULE_NAME,
        ADK_ETHERNET_MODULE_NAME,
	ADK_IPSEC_MODULE_NAME
};


adk_module_init_func_t func[MAX_NO_OF_MODULES] = {
        adk_ip_init,
        adk_ipv6_init,
        adk_ethernet_init,
	adk_ipsec_init
};

/* ****** ****** ****** ****** ****** ****** ****** ******/


/*
 *******************************************************
 *              QM related variables
 *******************************************************
 */
/* NIC-Rx */
struct bd_data
{
    uint8_t event_dev_id;
    uint8_t event_port_id;
    uint8_t qid;
};


bool is_adk_master;
bool cppi_netd_port_in_use = true;
adk_dev_id_t dev_net_id;
adk_sys_handle_t    sysHandle1;

bool enablePh = false;
bool enableIPsec = false;
bool crypto_bd_in_use = false;
uint8_t ip_ver_in_use = (SAMPLE_USE_IPV4 | SAMPLE_USE_IPV6);

static uint64_t cppi_lcore_mask = 0;
static uint64_t rx_lcore_mask;
static uint64_t tx_lcore_mask;
static uint64_t worker_lcore_mask;
static uint64_t crypto_lcore_mask = 0;

/* Service IDs used by the DPDK service setups */
/* DP_APP_USE_DPDK_SERVICE_CORES related info */
static unsigned int queue_dispatcher_service_id;
static unsigned int rx_service_id;
static unsigned int tx_service_id;
static unsigned int crypto_service_id;
static unsigned int cppi_service_id;
static unsigned int timer_service_id;
bool timer_bd_in_use = true;

/* Pointers used by the DPDK service setups
 * Allocation: adk_dp_setup()
 * De-Allocation: adk_dp_shutdown() */
static struct rte_bridge_crypto_service_fn_args *bridge_crypto_args;
static struct rte_bridge_cppi_service_fn_args *bridge_cppi_args;
static struct rte_bridge_timer_service_fn_args *bridge_timer_args;
static struct rte_bridge_eth_service_fn_args *bridge_eth_rx_args;
static struct rte_bridge_eth_service_fn_args *bridge_eth_tx_args;

#if 0
uint64_t cpk_tx_success = 0;
uint64_t cpk_tx_drop = 0;

uint64_t cpk_enqueue_success = 0;
uint64_t cpk_enqueue_drop = 0;

void eth_bridge_tx_completion_callback_fn(struct rte_mbuf *m, void *data);
void eth_bridge_tx_enqueue_callback_fn(struct rte_mbuf *m, void *data);
#endif

#define FUNC_CPPI   0x01
#define FUNC_RX     0x02
#define FUNC_TX     0x04
#define FUNC_SCHED  0x08
#define FUNC_WORKER 0x10
#define FUNC_CRYPTO 0x20

typedef struct core_functions_s
{
    unsigned    coreId;
    unsigned    funcsOnCore;
} core_functions_t;

static unsigned cppiCorePacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static unsigned rxCorePacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static unsigned txCorePacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static unsigned workerCoreIdPacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static unsigned workerCoreIndexPacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static unsigned cryptoCorePacked[DP_APP_MAX_LCORES] =
                                    {[0 ... (DP_APP_MAX_LCORES - 1)] = 0xff};
static core_functions_t coreInUsePacked[DP_APP_MAX_LCORES] =
                                {[0 ... (DP_APP_MAX_LCORES - 1)] = {0xff,0}};
static unsigned cores_needed=0;

/***************** ADK-EM Integration Related info Begin ***********/
/* num_worker_queues includes DL specific queues as well as UL queues for EM traffic */
#define NUM_EM_CORES 4
/* Qids to receive from EM - currently only one for IP ULI */
#define NUM_QIDS_FROM_EM 3
/* Assuming 24 are real cores */
#define VIRTUAL_CORE_ID_FOR_EM_START 25
#define EVENT_TYPE_SYNC   0
#define EM_SYNC_RECV_QUEUE_ID 0x320001
/* Sync message structure */

/* event dev port ids for em to send to ADK-EvDev */
int  *em_portIds;
/***************** ADK - EM Related info End ***********/

static uint8_t num_worker_queues = 0;
static uint8_t num_worker_cores = 0;
static uint8_t num_service_cores = 0;
static uint8_t num_cppi_cores = 0;
static uint8_t num_crypto_cores = 0;
static uint8_t num_rx_cores = 0;
static uint8_t num_tx_cores = 0;
static uint8_t dir_evq_id_start = 0;
static uint8_t dir_evp_id_start = 0;
static uint8_t next_dir_evq_id = 0;
static uint8_t next_dir_evp_id = 0;
static uint8_t num_dir_ports = 0;

static int sched_type = RTE_SCHED_TYPE_ATOMIC;
static struct rte_event_queue_dispatcher_args worker_data[DP_APP_MAX_LCORES];
#define DP_APP_MAX_N_BDS    10

#define CRYPTO_DEV_ID 0
#define CRYPTO_QP_ID 0
#define NUM_CRYPTO_QUEUE_PAIRS  1
#define MAX_NUM_IPSEC_SA  2000

enum {
    DP_APP_ETH_RX_BD=0,
    DP_APP_ETH_TX_BD,
    DP_APP_TIMER_BD,
    DP_APP_CRYPTO_BD,
    DP_APP_CPPI_BD,
};

static int8_t eventdev_id;
struct bd_data   bd_info[DP_APP_MAX_N_BDS] = {{0}};


/* queue-id to be bind with worker ports */
int  *worker_qid;

static volatile int done = 0;
static int quiet = 0;

struct rte_mempool *packet_pool;
static uint8_t evdev_id, timer_inst_id = 0;
#if 0
char * tm_error_name(enum rte_tm_error_type type)
{
    switch(type) {
    case RTE_TM_ERROR_TYPE_NONE:
        return ("RTE_TM_ERROR_TYPE_NONE");
	RTE_TM_ERROR_TYPE_NONE, /**< No error. */
	RTE_TM_ERROR_TYPE_UNSPECIFIED, /**< Cause unspecified. */
	RTE_TM_ERROR_TYPE_CAPABILITIES,
	RTE_TM_ERROR_TYPE_LEVEL_ID,
	RTE_TM_ERROR_TYPE_WRED_PROFILE,
	RTE_TM_ERROR_TYPE_WRED_PROFILE_GREEN,
	RTE_TM_ERROR_TYPE_WRED_PROFILE_YELLOW,
	RTE_TM_ERROR_TYPE_WRED_PROFILE_RED,
	RTE_TM_ERROR_TYPE_WRED_PROFILE_ID,
	RTE_TM_ERROR_TYPE_SHARED_WRED_CONTEXT_ID,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_COMMITTED_RATE,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_COMMITTED_SIZE,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_PEAK_RATE,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_PEAK_SIZE,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_PKT_ADJUST_LEN,
	RTE_TM_ERROR_TYPE_SHAPER_PROFILE_ID,
	RTE_TM_ERROR_TYPE_SHARED_SHAPER_ID,
	RTE_TM_ERROR_TYPE_NODE_PARENT_NODE_ID,
	RTE_TM_ERROR_TYPE_NODE_PRIORITY,
	RTE_TM_ERROR_TYPE_NODE_WEIGHT,
	RTE_TM_ERROR_TYPE_NODE_PARAMS,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_SHAPER_PROFILE_ID,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_SHARED_SHAPER_ID,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_N_SHARED_SHAPERS,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_WFQ_WEIGHT_MODE,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_N_SP_PRIORITIES,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_CMAN,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_WRED_PROFILE_ID,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_SHARED_WRED_CONTEXT_ID,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_N_SHARED_WRED_CONTEXTS,
	RTE_TM_ERROR_TYPE_NODE_PARAMS_STATS,
	RTE_TM_ERROR_TYPE_NODE_ID,
}
#endif

#define DP_APP_VPORTS_PER_VSI    1
#define DP_APP_VPORT_NUM_QUEUES  8
#define DP_APP_SHAPING_RATE_10G        (10000000000)
#define DP_APP_SHAPING_RATE_1G         (1000000000)
#define RTE_TM_DEFAULT_PRIORITY 0
#define RTE_TM_DEFAULT_WEIGHT   1
#define RTE_TM_BASE_ROOT_NODE_ID 0x4000
#define DP_APP_VPORT_MAX_DEVS   64

static int vdev_list[DP_APP_VPORT_MAX_DEVS];
static int vport_node_id_list[DP_APP_VPORT_MAX_DEVS];

uint16_t bh_eth_dev_id;
struct rte_eth_dev_info   bh_eth_dev_info;
#ifdef EM_SNF_ADK
extern int parse_config_file(char *xml_file, cfg_arr_t *cfg_arr);
#endif
#ifdef ADK_USE_COMPLETION_QUEUE
int dp_app_vsi_init(uint8_t id, uint16_t rx_queues, uint16_t tx_queues, uint16_t __attribute((unused)) cp_queues, uint16_t *vsi_dev_port_id);
int dp_app_vsi_init(uint8_t id, uint16_t rx_queues, uint16_t tx_queues, uint16_t __attribute((unused)) cp_queues, uint16_t *vsi_dev_port_id)
#else
int dp_app_vsi_init(uint8_t id, uint16_t rx_queues, uint16_t tx_queues, uint16_t *vsi_dev_port_id);
int dp_app_vsi_init(uint8_t id, uint16_t rx_queues, uint16_t tx_queues, uint16_t *vsi_dev_port_id)
#endif
{

    /* Create a eth-dev for BH port */
    char vsi_dev_name[16];
    char vsi_dev_args[128];
    int             ret;
    struct rte_pmd_ice_dsi_dev_info ice_dsi_info;
    uint16_t     vid;

    snprintf(vsi_dev_name, 16, "net_ice_dsi%d", id);
    if (enableIPsec)
    {
#ifdef ADK_USE_COMPLETION_QUEUE
	    snprintf(vsi_dev_args, 128, "pci-bdf=b4:00.0,rxq=%d,txq=%d,ipsec_enable=1,cmpltnq=%d",
			    rx_queues, tx_queues, cp_queues);
#else
	    snprintf(vsi_dev_args, 128, "pci-bdf=b4:00.0,rxq=%d,txq=%d,ipsec_enable=1",
			    rx_queues, tx_queues);
#endif
    }
    else
    {
#ifdef ADK_USE_COMPLETION_QUEUE
        snprintf(vsi_dev_args, 128, "pci-bdf=b4:00.0,rxq=%d,txq=%d,cmpltnq=%d",
			    rx_queues, tx_queues, cp_queues);
#else
        snprintf(vsi_dev_args, 128, "pci-bdf=b4:00.0,rxq=%d,txq=%d",
			    rx_queues, tx_queues);
#endif
    }

            /* Create device only if primary */
            /* Create a vport for the CPK port */
            rdk_trace("rte_vdev_init()\n");
            ret = rte_vdev_init(vsi_dev_name, vsi_dev_args);
            if (ret) {
                printf("vdev %s init with args %s failed\n",
                        vsi_dev_name, vsi_dev_args);
                return -1;
            } else {
                printf("Creating vdev %s (%s), ",
                        vsi_dev_name, vsi_dev_args);
        }
        rdk_trace("rte_eth_dev_get_port_by_name()\n");
        ret = rte_eth_dev_get_port_by_name(vsi_dev_name, &vid);
        *vsi_dev_port_id = vid;
        if (ret) {
            printf("rte_eth_dev_get_port_by_name() failed. errno = %d\n",
                    ret);
            return -1;
        } else {
            printf("port ID = %d, Success!\n", *vsi_dev_port_id);
        }

        rdk_trace("rte_eth_dev_info_get()\n");
        rte_eth_dev_info_get(*vsi_dev_port_id, &bh_eth_dev_info);
        rte_pmd_ice_dsi_get_dev_info(*vsi_dev_port_id, &ice_dsi_info);
        printf("port ID = %d, VSI = %d, #rx-queues = %d, #tx-queues = %d, #cp-queues = %d, Success!\n",
                *vsi_dev_port_id,
                ice_dsi_info.vsi_number,
                bh_eth_dev_info.nb_rx_queues,
                bh_eth_dev_info.nb_tx_queues,
                bh_eth_dev_info.nb_cp_queues);
    return 0;
} /* dp_app_vsi_init */


/*
 * Direct Ports: 1 (rx) + 1 (tx) + PH (4) + CPPI (1) + Crypto (1)
 * Worker Ports: 1 per core and type is as per argv
 * 
 * Total Event Ports: < N worker cores>: < direct event port start > < EM Ports >
 * Event Ports: < 0 (1) worker cores>: < 1 ( 1-->) direct event port start > < EM Ports >
 * Event Qeues: < Qids Per worker + Em recv queues > < direct event queue id >
 * Event Qeues: < 0-3 (4) Qids Per worker + 4 (1) EM Recv Queue > < 5--> direct event queue id>
 */

int dp_app_qm_init(void);
int dp_app_qm_init(void)
{

    char qm_dev_name[16];
    char qm_dev_args[128] = "";

    int ret=0;
    uint8_t dir_port_start = 0;
    uint8_t dir_queue_start = 0;
    char qm_dev_dq_args[128];
    char qm_dev_dp_args[128];
    char qm_dev_dq_args_t[128];
    char qm_dev_dp_args_t[128];
    int i;
    /* struct rte_event_dev_info  dev_info; */

       next_dir_evq_id = dir_evq_id_start = dir_queue_start = num_worker_queues = NUM_QIDS_PER_WORKER;

    next_dir_evq_id += NUM_ADK_ENTRY_POINT;
    next_dir_evp_id = dir_evp_id_start = dir_port_start = num_worker_cores;

    num_dir_ports = 2 + NUM_PH_THREADS;
    num_dir_ports += (cppi_netd_port_in_use) ? 1 : 0;
    num_dir_ports += (timer_bd_in_use) ? 1 : 0;
    num_dir_ports += (crypto_bd_in_use) ? 1 : 0;

	/* Em queue starts after worker core queues - 
	 * Qids Per worker or for DL
	 * Qids from EM are for UL
	 * For e.g. if there are 4 NUM_QIDS_PER_WORKER and 3 NUM_QIDS_FROM_EM then there will be 7 queues
	 * All 7 will be linking to all worker cores.
    * Qid 0-3 will be for DL 
    * Qid 4-6 will be for UL (from EM)
	 */
    /* Allocate space for storing worker queue info */
    worker_qid = malloc(sizeof(int)*num_worker_queues);

    printf("%s() Required: Worker Ports: %d, Worker Queues: %d, Direct Ports & Queues = %d\n",
            __func__,num_worker_cores,num_worker_queues,num_dir_ports);

       /* Create HQM device - only for primary */
       snprintf(qm_dev_dq_args, 128, "dir_queue_ids=");
       snprintf(qm_dev_dp_args, 128, "dir_port_ids=");

       for(i=0;i< num_dir_ports; i++)
       {

          if(i!=(num_dir_ports-1)) {
             snprintf(qm_dev_dp_args_t, 128, "%d:",dir_port_start++);
             snprintf(qm_dev_dq_args_t, 128, "%d:",dir_queue_start++);
          } else {
             snprintf(qm_dev_dp_args_t, 128, "%d",dir_port_start++);
             snprintf(qm_dev_dq_args_t, 128, "%d",dir_queue_start++);
          }

          strcat(qm_dev_dp_args, qm_dev_dp_args_t);
          strcat(qm_dev_dq_args, qm_dev_dq_args_t);

       }

    snprintf(qm_dev_name, 16, "event_ihqm1");

       /* Create a QM vdev */
       rdk_trace("rte_vdev_init()\n");
       ret = rte_vdev_init(qm_dev_name, qm_dev_args);
       if (ret) {
          printf("vdev %s init with args %s failed\n", qm_dev_name, qm_dev_args);
          return -1;
       } else {
          printf("Created vdev %s (%s), ", qm_dev_name, qm_dev_args);
       }
    if(is_unified_em_adk_set)
    {
	    em_portIds = malloc(sizeof(int)*number_em_cores);
	    for ( i =0; i < number_em_cores; i ++)
          {
            em_portIds[i] = num_worker_cores + num_dir_ports + i;
          }
    }
    return ret;
} /* dp_app_qm_init */

static void dp_app_next_dir_ev_p_q_set_get(uint8_t *evp, uint8_t *evq)
{
    if(((dir_evp_id_start + num_dir_ports)) <= next_dir_evp_id)
    {
        rte_panic("%s() all the created directed event-ports are used up. \
                  create additional directed evp&evq\n", __func__);
    }

    *evp = next_dir_evp_id++;
    *evq = next_dir_evq_id++;

    return;

}


#ifdef DP_APP_SNR_EGRESS_QOS
void dp_app_tm_error_print(struct rte_tm_error *error);
void dp_app_tm_error_print(struct rte_tm_error *error)
{
    printf("\n TM Error is %s \n", error->message);
}


int dp_app_vport_create(uint8_t parent_eth_dev,
                        uint8_t vport_id, uint32_t    vport_parent_node_id,
                        uint16_t *vport_dev_id);

/* DMAC level L2 node for EMCA Shaping */
#define DP_APP_CPK_BB_CONFIG_L2_NODE_BASE (RTE_TM_BASE_ROOT_NODE_ID + (DP_APP_VPORT_MAX_DEVS * 2))

/* Get the L1 node based on DPDK Ethernet device ID and HLP Port number */
uint32_t dp_app_cpk_get_l1_node(uint16_t dpdk_eth_dev_id,
                                 uint32_t hlp_port_num)
{
    return vport_l1_node_list[dpdk_eth_dev_id][hlp_port_num];
}

/* Get the vport id */
uint32_t dp_app_cpk_get_vport_id(uint16_t dpdk_eth_dev_id,
                                  uint32_t hlp_port_num)
{
    return vport_dev_list[dpdk_eth_dev_id][hlp_port_num];
}

int dp_app_cpk_BB_CONFIG_qos_hierarchy_create(uint16_t vport_dev_id,
                                          uint32_t parent_node_id,
                                          uint32_t *child_node_id)
{
    struct rte_tm_node_params node_params = {0};
    int wfq_weights[8] = {1,1,1,1,1,1,1,1};
    int ret;
    struct rte_tm_error tm_error;
    static int BB_CONFIG_l2_node_count = 1;

    node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
    node_params.shared_shaper_id  = NULL;
    node_params.n_shared_shapers  = 0;
    node_params.nonleaf.wfq_weight_mode= wfq_weights;
    node_params.nonleaf.n_sp_priorities = 8;
    node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
                    RTE_TM_STATS_N_BYTES |
                    RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
                    RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
                    RTE_TM_STATS_N_PKTS_RED_DROPPED |
                    RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
                    RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
                    RTE_TM_STATS_N_BYTES_RED_DROPPED |
                    RTE_TM_STATS_N_PKTS_QUEUED |
                    RTE_TM_STATS_N_BYTES_QUEUED);

    *child_node_id = DP_APP_CPK_BB_CONFIG_L2_NODE_BASE + BB_CONFIG_l2_node_count;
    BB_CONFIG_l2_node_count++;

    /* Add a Level 2 node */
    rdk_trace("rte_tm_node_add()\n");
    ret = rte_tm_node_add(vport_dev_id,
                *child_node_id,
                parent_node_id,
                RTE_TM_DEFAULT_PRIORITY,
                RTE_TM_DEFAULT_WEIGHT,
                RTE_TM_NODE_LEVEL_ID_ANY,
                &node_params,
                &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add L2 node .errno = %d\n", ret);
        dp_app_tm_error_print(&tm_error);
        return 1;
    }

    return 0;
}

int dp_app_cpk_qos_hierarchy_create(uint8_t vport_id, uint16_t vport_dev_id, uint32_t *l1_node);

int dp_app_cpk_qos_hierarchy_create(uint8_t vport_id, uint16_t vport_dev_id, uint32_t *l1_node)
{
    struct rte_tm_node_params node_params = {0};
    int wfq_weights[8] = {1,1,1,1,1,1,1,1};
    uint32_t     root_node_id = 0;
    uint32_t     l1_node_id = 0;
    int ret;
#ifndef ADK_QOS_GEN3_INTERFACE
    uint32_t     tx_queue_id, queue_index;
#endif
    struct rte_tm_error tm_error;

    node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
    node_params.shared_shaper_id  = NULL;
    node_params.n_shared_shapers  = 0;
    node_params.nonleaf.wfq_weight_mode= wfq_weights;
    node_params.nonleaf.n_sp_priorities = 8;
    node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
                    RTE_TM_STATS_N_BYTES |
                    RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
                    RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
                    RTE_TM_STATS_N_PKTS_RED_DROPPED |
                    RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
                    RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
                    RTE_TM_STATS_N_BYTES_RED_DROPPED |
                    RTE_TM_STATS_N_PKTS_QUEUED |
                    RTE_TM_STATS_N_BYTES_QUEUED);

    root_node_id = RTE_TM_BASE_ROOT_NODE_ID +( vport_id * 2);
    /* add a root node for a vport  */
    rdk_trace("rte_tm_node_add()\n");
    ret = rte_tm_node_add(vport_dev_id,
                root_node_id,
                RTE_TM_NODE_ID_NULL,
                RTE_TM_DEFAULT_PRIORITY,
                RTE_TM_DEFAULT_WEIGHT,
                RTE_TM_NODE_LEVEL_ID_ANY,
                &node_params,
                &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add vport root.errno = %d\n", ret);
        dp_app_tm_error_print(&tm_error);
        return 1;
    }

    l1_node_id = root_node_id+ 1;

    /* add a Level 1 node for a vport  */
    rdk_trace("rte_tm_node_add()\n");
    ret = rte_tm_node_add(vport_dev_id,
                l1_node_id,
                root_node_id,
                RTE_TM_DEFAULT_PRIORITY,
                RTE_TM_DEFAULT_WEIGHT,
                RTE_TM_NODE_LEVEL_ID_ANY,
                &node_params,
                &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add vport L1 node .errno = %d\n", ret);
        dp_app_tm_error_print(&tm_error);
        return 1;
    }
    *l1_node = l1_node_id;

#ifndef ADK_QOS_GEN3_INTERFACE

    printf("Added vport-dev-id %d, root=%d L1-node=%d, leaf-nodes=",
            vport_dev_id, root_node_id, l1_node_id);
    tx_queue_id = 0;

    for (queue_index=0; queue_index< tx_qs; queue_index++, tx_queue_id++)
    {
        /* Create leaf node for each tx queue  */
        node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
        node_params.shared_shaper_id  = NULL;
        node_params.n_shared_shapers  = 0;
        node_params.leaf.cman = RTE_TM_CMAN_TAIL_DROP;
        node_params.leaf.wred.wred_profile_id = RTE_TM_WRED_PROFILE_ID_NONE;
        node_params.leaf.wred.shared_wred_context_id = NULL;
        node_params.leaf.wred.n_shared_wred_contexts= 0;
        node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
                    RTE_TM_STATS_N_BYTES |
                    RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
                    RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
                    RTE_TM_STATS_N_PKTS_RED_DROPPED |
                    RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
                    RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
                    RTE_TM_STATS_N_BYTES_RED_DROPPED |
                    RTE_TM_STATS_N_PKTS_QUEUED |
                    RTE_TM_STATS_N_BYTES_QUEUED);

        /* configure leaf-level node */
        rdk_trace("rte_tm_node_add()\n");
        ret = rte_tm_node_add(vport_dev_id,
                    tx_queue_id,
                    l1_node_id,
                    RTE_TM_DEFAULT_PRIORITY,
                    RTE_TM_DEFAULT_WEIGHT,
                    RTE_TM_NODE_LEVEL_ID_ANY,
                    &node_params,
                    &tm_error);
        if (ret) {
            printf("rte_tm_node_add() failed to add queue %d leaf node."
                " errno = %d\n", tx_queue_id, ret);
            dp_app_tm_error_print(&tm_error);
            return 1;
        } else {
            printf("%d\t", tx_queue_id);
        }
    }
    printf("\n");
#endif
    return 0;
}

#ifdef  ADK_QOS_GEN3_INTERFACE
void dp_app_sp_qos_create(adk_sys_handle_t sys_handle, uint16_t dev_id)
{
    adk_qos_queue_params_t queue;
    adk_status_t ret_val;
    int i;
    /* Create slowpath[netdriver] QoS Queues 
     *
     */
    if (dev_id != USHRT_MAX) {
        for (i=0; i <DP_APP_DEV_NET_RL_QUEUES; i++) {
            memset(&queue,0x0, sizeof(queue));
            queue.deviceId = dev_id; 
            queue.queueId = ADK_QOS_GET_QUEUE_ID;
            queue.parentNodeHandle = 0; /* Dev net Tx FIFO/Queue ID */
            queue.shapingMode = ADK_QOS_PPS;
            queue.qosRates.peak.rate = 16;
            queue.qosRates.peak.bucketSize = 2;

            ret_val = adk_qos_queue_create(sys_handle, &queue, 0, &devNetRlQHandle[dev_id][i]);
            if (ret_val != ADK_SUCCESS) {
                printf("Failed to create queue with id %d\n", i);
            } else {
                printf("Created dev[%d] RL queue - Handle:0x%p\n",dev_id, devNetRlQHandle[dev_id][i]);
            }

        }
    }
    return;
}

void dp_app_egress_qos_create(adk_sys_handle_t sys_handle, uint16_t port_id)
{
    adk_status_t ret_val;
    uint8_t start, end;
    adk_qos_queue_params_t queue;
    int i,j;
    adk_qos_bm_policy_t bmPolicy;

    if(port_id == bh_eth_dev_id){
        start = 0;
        end = DP_APP_HLP_PORTS -2;
    } else if(port_id == BB_CONFIG_eth_dev) {
        start = DP_APP_HLP_PORTS -1 ;
        end = DP_APP_HLP_PORTS -1 ;
    } else {
        return;
    }
    /* Create QoS Egress Queues 
     *
     */
    for(j=start;j<=end;j++) {
        for (i=0; i <DP_APP_QUEUES_PER_VPORT; i++) {
            memset(&queue,0x0, sizeof(queue));
            queue.deviceId = vport_dev_list[port_id][j];
            queue.queueId = ADK_QOS_GET_QUEUE_ID;
            queue.parentNodeHandle = (adk_handle_t) (uint64_t) vport_l1_node_list[port_id][j];
            queue.schedulingMode = ADK_QOS_SP;
            queue.serviceQuantum = i+1;
            queue.queuePriority  = i;
            queue.bmPolicy = ADK_BM_RED;
            queue.shapingMode = ADK_QOS_PR;
            queue.qosRates.peak.rate = SHAPING_RATE_1M;
            queue.qosRates.peak.bucketSize = 2*1024;
#if 0
            queue.qosRates.sr_mr_u.sustain.rate = SHAPING_RATE_1M;
            queue.qosRates.sr_mr_u.sustain.bucketSize = 2*1024;
#endif
            ret_val = adk_qos_queue_create(sys_handle, &queue, 0, &queue_list[port_id][j][i]);
            if (ret_val != ADK_SUCCESS)
            {
                printf("Failed to create queue with id %d\n", i);
                return ;
            }
            else
            {
                printf("Created queue on vport %d - Handle:0x%p\n", queue.deviceId, queue_list[port_id][j][i]);
            }

            memset(&bmPolicy, 0x0, sizeof(bmPolicy));
            bmPolicy.dropPolicy = ADK_BM_RED;
            bmPolicy.params_u.redConfigNewParams.minThreshold = 1;
            bmPolicy.params_u.redConfigNewParams.maxThreshold = 100;
            bmPolicy.params_u.redConfigNewParams.max_queue_length = 1024*1024*2;
            bmPolicy.params_u.redConfigNewParams.queueSizeAvgFactor = 1; 
            ret_val = adk_qos_queue_bm_policy_set(sys_handle, queue_list[port_id][j][i], &bmPolicy);
            if (ret_val != ADK_SUCCESS)
            {
                printf("Failed to create queue with id %d\n", i);
                return ;
            }
            else
            {
                printf("Created queue with id %d\n", i);
            }
        }
    }
    return;
}

adk_handle_t 
dp_app_sp_qos_handle_get(uint16_t dev_id, uint16_t id)
{
    return (devNetRlQHandle[dev_id][id]);
}

adk_handle_t 
dp_app_egress_qos_handle_get(uint16_t port_id, uint8_t cpk_port, uint8_t id)
{
    return (queue_list[port_id][cpk_port][id]);
}

int dp_app_qos_queue_stats_print(adk_sys_handle_t sys_handle, adk_handle_t qh)
{

    adk_qos_queue_stats_counters_t   queueStats;

    if (adk_qos_queue_stats_counters_get(sys_handle, 
                qh, &queueStats) !=ADK_SUCCESS) {
        printf(" Failed to get queue stats get\n");
        return 1;
    } else {
        printf("Queue Stats %"ADKPRIh" :: ", qh);
        printf("\t pkt xmt=%d", queueStats.q_tx_pdu_count);
        printf("\t pkt drop=%d", queueStats.q_dropCount);
        printf("\t bytes xmt=%ld", queueStats.q_tx_octets_count);
        printf("\t bytes drop=%ld\n", queueStats.q_dropOctetCount);
    }
    return 0;

}

void dp_app_sp_qos_delete(adk_sys_handle_t sys_handle, uint16_t dev_id)
{
    adk_status_t ret_val;
    int i;

    /* Delete slowpath[netdriver] QoS Queues 
     *
     */
    if (dev_id != USHRT_MAX) {
        for (i=0; i <DP_APP_DEV_NET_RL_QUEUES; i++) {
            ret_val = adk_qos_queue_delete(sys_handle, devNetRlQHandle[dev_id][i], 0, 0);
            if (ret_val != ADK_SUCCESS) {
                printf("Failed to delete queue with id %d handle 0x%p\n", i, devNetRlQHandle[dev_id][i]);
                return;
            } else {
                printf("Deleted dev[%d] RL queue - Handle:0x%p\n", dev_id, devNetRlQHandle[dev_id][i]);
            }
        }
    }
    return;
}

void dp_app_egress_qos_delete(adk_sys_handle_t sys_handle, uint16_t port_id)
{
    adk_status_t ret_val;
    uint8_t start, end;
    int i,j;

    if(port_id == bh_eth_dev_id){
        start = 0;
        end = DP_APP_HLP_PORTS -2;
    } else if(port_id == BB_CONFIG_eth_dev) {
        start = DP_APP_HLP_PORTS -1;
        end = DP_APP_HLP_PORTS -1;
    } else {
        return;
    }
    /* Create QoS Egress Queues 
     *
     */
    for(j=start;j<=end;j++) {
        for (i=0; i <DP_APP_QUEUES_PER_VPORT; i++) {
            ret_val = adk_qos_queue_delete(sys_handle, queue_list[port_id][j][i], 0, 0);
            if (ret_val != ADK_SUCCESS) {
                printf("Failed to delete queue with id %d\n", i);
            } else {
                printf("Deleted queue - Handle:0x%p\n", queue_list[port_id][j][i]);
                queue_list[port_id][j][i] = 0x00;
            }
        }
    }
    return;
}



#else

void dp_app_sp_qos_create(adk_sys_handle_t sys_handle, uint16_t dev_id)
{
    RTE_SET_USED(sys_handle);
    RTE_SET_USED(dev_id);
    return;
}
void dp_app_egress_qos_create(adk_sys_handle_t sys_handle, uint16_t port_id)
{
    RTE_SET_USED(sys_handle);
    RTE_SET_USED(port_id);
    return;
}

void dp_app_sp_qos_delete(adk_sys_handle_t sys_handle, uint16_t dev_id)
{
    RTE_SET_USED(sys_handle);
    RTE_SET_USED(dev_id);
    return;
}

void dp_app_egress_qos_delete(adk_sys_handle_t sys_handle, uint16_t port_id)
{
    RTE_SET_USED(sys_handle);
    RTE_SET_USED(port_id);
    return;
}

adk_handle_t 
dp_app_sp_qos_handle_get(uint16_t dev_id, uint16_t id)
{
    RTE_SET_USED(dev_id);
    RTE_SET_USED(id);
    return 0;
}

adk_handle_t 
dp_app_egress_qos_handle_get(uint16_t port_id, uint8_t cpk_port, uint8_t id)
{
    RTE_SET_USED(port_id);
    RTE_SET_USED(cpk_port);
    RTE_SET_USED(id);
    return 0;
}

int dp_app_qos_queue_stats_print(adk_sys_handle_t sys_handle, adk_handle_t qh)
{
    RTE_SET_USED(sys_handle);
    RTE_SET_USED(qh);
    return 0;
}
#endif

int dp_app_vport_create(uint8_t parent_eth_dev,
                        uint8_t vport_id, uint32_t    vport_parent_node_id,
                        uint16_t *vport_dev_id)
{
    char vport_dev_name[16];
    char vport_dev_args[128];
    int ret;

    /*Create a VPORT */
    /* Create a node from shared hierarchy
    * Create vaport with that node as root
    * */

    /* Shared node id range
        * 0-20 : L1 (NIS) port nodes
        * 0xFFFF0000 - 0xFFFFFFFF : preassigned node id range allocated by ae-driver
        * 21- 0xFFFEFFFF  : Application managed node-ids */
    snprintf(vport_dev_name, 16, "net_ice_tm%d", vport_id);
        
	/*  Now create a vport-eth-dev based on vport-node-id*/
	snprintf(vport_dev_args, 128, "parent-port-id=%d,cpk-port=%d,root-shared-node=%d",
					parent_eth_dev, vport_parent_node_id, vport_parent_node_id);

	/* Create a vport for the CPK port */
	rdk_trace("rte_vdev_init()\n");
	ret = rte_vdev_init(vport_dev_name, vport_dev_args);
	if (ret) {
			printf("vdev %s init with args %s failed\n",
							vport_dev_name, vport_dev_args);
			return 1;
	} else {
			printf("Creating vdev %s (%s), ",
							vport_dev_name, vport_dev_args);
	}

    rdk_trace("rte_eth_dev_get_port_by_name()\n");
    ret = rte_eth_dev_get_port_by_name(vport_dev_name, vport_dev_id);
    if (ret) {
        printf("rte_eth_dev_get_port_by_name() failed. errno = %d\n",
            ret);
        return 1;
    } else {
        printf("port ID = %d, Success!\n", *vport_dev_id);
    }

    /* Store the details to clean at the exit time */
    vport_node_id_list[*vport_dev_id]     = vport_parent_node_id;
    vdev_list[*vport_dev_id] = *vport_dev_id;
    vport_dev_list[parent_eth_dev][vport_parent_node_id] = *vport_dev_id;
    testConfShm_p->ethDev.vport_id = vport_id;
    testConfShm_p->ethDev.vdev_port_id = *vport_dev_id;
    return 0;
}
#if 0
int dp_app_vport_cleanup(void);
int dp_app_vport_cleanup(void)
{
    int i=0;
    int ret;
    ae_handle cpk_ae_hdl;
    uint8_t bus  = 0, dev = 0, function = 0;

    rdk_trace("ae_client_open()\n");
    ret = ae_client_open(bus, dev, function, &cpk_ae_hdl);
    if (ret) {
       printf("\n ae_client_open is failed for BUS:%d, DEV:%d, FUNC:%d\n",
                   bus,dev,function);
       return -1;
    }
    for(i=0;i<DP_APP_VPORT_MAX_DEVS; i++){
        if(vdev_list[i] != 0 ){
            char vport_dev_name[16];
            rdk_trace("rte_eth_dev_stop()\n");
            rte_eth_dev_stop(vdev_list[i]);
            rdk_trace("rte_eth_dev_get_name_by_port()\n");
            rte_eth_dev_get_name_by_port(i, vport_dev_name);
            rdk_trace("rte_vdev_uninit()\n");
            rte_vdev_uninit(vport_dev_name);
            rdk_trace("ae_client_shared_hier_node_delete()\n");
            ae_client_shared_hier_node_delete(cpk_ae_hdl, vport_node_id_list[i]);

        }
    }
    return 0;
}
#endif
#endif

static uint64_t
parse_cmask(const char *cmask)
{
    long value;
    char *end_ptr = NULL;

    if (cmask == NULL)
        return -1;
    errno = 0;

    value = strtoll(cmask, &end_ptr, 16);

    if(end_ptr == cmask )
        return -1;

    if ((errno == ERANGE && (value == LONG_MAX || value == LONG_MIN))
        || (errno != 0 && value == 0))
    {
        return -1;
    }

    return value;
}

static struct option long_options[] = {
    {"worker-mask", required_argument, 0, 'w'},
    {"cppi-mask", required_argument, 0, 'C'},
    {"rx-mask", required_argument, 0, 'r'},
    {"tx-mask", required_argument, 0, 't'},
    {"sched-mask", required_argument, 0, 's'},
    {"ip-version", required_argument, 0, 'v'},
    {"ph", no_argument, 0, 'p'},
    {"ordered", no_argument, 0, 'o'},
    {"quiet", no_argument, 0, 'q'},
    {0, 0, 0, 0}
};

int adk_resource_init(unsigned num_ports);

static void
usage (
    void)
{
    const char *usage_str =
        "  Usage: eventdev_pipeline [options]\n"
        "  Options:\n"
        "  -w, --worker-mask=core_mask  Run work on CPUs in core mask\n"
        "  -C, --cppi-mask=core_mask    Run CPPI bridge driver on CPUs in core mask\n"
        "  -r, --rx-mask=core_mask      Run NIC rx on CPUs in core mask\n"
        "  -t, --tx-mask=core_mask      Run NIC tx on CPUs in core mask\n"
        "  -s,  --sched-mask=core_mask   Run QM scheduler on CPUs in core mask\n"
        "  -v, --ip-version=version     Run with only IPv4 (4) or IPv6 (6)\n"
        "  -i, --ipsec           Enable IPsec\n"
        "  -c  --crypto-mask=core_mask   Run CRYPTO BD on CPUs in core mask\n"
        "  -o, --ordered         Use ordered scheduling\n"
        "  -p, --ph              Enable PH\n"
        "  -q, --quiet           Minimize printed output\n"
        "\n";

    fprintf(stderr, "%s", usage_str);
    exit(1);
}

static void
parse_app_args (
    int argc,
    char** argv)
{
    /* Parse cli options*/
    int option_index;
    int c;
    opterr = 0;
    int i;
    unsigned funcsOnCore;
    uint64_t    mask;
    int dpdkCoreIndex = 0;
	opterr = 0;
	optind= 0;

    for ( ; ; )
    {
        c = getopt_long(argc, argv, "w:C:r:t:s:c:v:oqpi", long_options,
                        &option_index);
        if (c == -1)
            break;

        switch (c)
        {
        case 'w':
            worker_lcore_mask = parse_cmask(optarg);
            break;

        case 'C':
            cppi_lcore_mask = parse_cmask(optarg);
            break;

        case 'r':
            rx_lcore_mask = parse_cmask(optarg);
            break;

        case 't':
            tx_lcore_mask = parse_cmask(optarg);
            break;

        case 'p':
            enablePh = true;
            break;

        case 'i':
            enableIPsec = true;
            crypto_bd_in_use = true;
            break;

        case 'c':
            crypto_lcore_mask = parse_cmask(optarg);
            break;
  
        case 'o':
            sched_type = RTE_SCHED_TYPE_ORDERED;
            break;

        case 'v':
            if (*optarg == '4')
            {
                ip_ver_in_use = SAMPLE_USE_IPV4;
            }
            else if (*optarg == '6')
            {
                ip_ver_in_use = SAMPLE_USE_IPV6;
            }
            else
            {
                usage();
            }
            break;

        case 'q':
            quiet = 0;
            break;

        default:
            usage();
        }
    }

    /* FIXME: Verify that coremask core is valid */

    if (worker_lcore_mask == 0 || rx_lcore_mask == 0 || tx_lcore_mask == 0)
        usage();

    for (i = 0; i < DP_APP_MAX_LCORES; i++)
    {
        /* Skip cores DPDK doesn't know about */
        if (-1 == rte_lcore_index(i))
        {
            continue;
        }

        mask = (1UL << i);

        funcsOnCore = 0;
        if (0 != cppi_lcore_mask)
        {
            if (mask == (cppi_lcore_mask & mask))
            {
                funcsOnCore |= FUNC_CPPI;
                cppiCorePacked[num_cppi_cores++] = i;
                printf("[%s()] Core %d does cppi!\n", __func__, i);
            }
        }
        if (mask == (rx_lcore_mask & mask))
        {
            funcsOnCore |= FUNC_RX;
            rxCorePacked[num_rx_cores++] = i;
            printf("[%s()] Core %d does rx!\n", __func__, i);
        }
        if (mask == (tx_lcore_mask & mask))
        {
            funcsOnCore |= FUNC_TX;
            txCorePacked[num_tx_cores++] = i;
            printf("[%s()] Core %d does tx!\n", __func__, i);
        }
        if (0 != crypto_lcore_mask)
        {
            if (mask == (crypto_lcore_mask & mask))
            {
                funcsOnCore |= FUNC_CRYPTO;
                cryptoCorePacked[num_crypto_cores++] = i;
                printf("[%s()] Core %d does crypto!\n", __func__, i);
            }
        }

        if (mask == (worker_lcore_mask & mask))
        {
            funcsOnCore |= FUNC_WORKER;
            workerCoreIdPacked[num_worker_cores] = i;
            workerCoreIndexPacked[num_worker_cores] =
                                                    rte_lcore_index(i);
            num_worker_cores++;
            printf("[%s()] Core %d does work!\n", __func__, i);
        }

        if (0 != funcsOnCore)
        {
            cores_needed++;
            coreInUsePacked[dpdkCoreIndex].funcsOnCore = funcsOnCore;
            coreInUsePacked[dpdkCoreIndex].coreId = i;
            debug_printf("    core %d in use with funcs 0x%02x\n",
                         i, funcsOnCore);
        }
        dpdkCoreIndex++;
    }

#if 0
    if (DP_APP_MAX_LCORES <= dpdkCoreIndex)
    {
        rte_panic("[%s] Too many cores in use: %d, max of %d\n", __func__,
                  dpdkCoreIndex, DP_APP_MAX_LCORES);
    }
#endif

}

/* packet memry pool creation*/
static struct rte_mempool *
create_pkt_pool (
    void)
{

    rdk_trace("rte_pktmbuf_pool_create()\n");
    return rte_pktmbuf_pool_create("packet_pool",
                                   /* mbufs */ 65536,
                                   /* cache_size */ 256,
                                   /* priv_size */ 512,
                                   /* data_room_size */ RTE_MBUF_DEFAULT_BUF_SIZE,
                                   rte_socket_id());
}

static int
initialize_cryptodevs(unsigned *num_cdevs)
{
    unsigned cdev_id, cdev_count;
    int retValue;

    rdk_trace("rte_cryptodev_count()\n");
    cdev_count = rte_cryptodev_count();
    if (cdev_count == 0)
    {
            printf ("No crypto devices available\n");
            return -1;
    }

    for (cdev_id = 0; cdev_id < cdev_count; cdev_id++) {
            struct rte_cryptodev_qp_conf qp_conf;

            struct rte_cryptodev_config conf = {
                    .nb_queue_pairs = 1,
                    .socket_id = SOCKET_ID_ANY,
		};

                rdk_trace("rte_cryptodev_configure()\n");
		retValue = rte_cryptodev_configure(cdev_id, &conf);
		if (retValue < 0)
		{
			printf ("Failed to configure cryptodev %u\n", cdev_id);
			return -1;
		}

		qp_conf.nb_descriptors = 2048;

                rdk_trace("rte_cryptodev_queue_pair_setup()\n");
		retValue = rte_cryptodev_queue_pair_setup(cdev_id, 0, &qp_conf,
						SOCKET_ID_ANY, NULL);
		if (retValue < 0) {
			printf("Failed to setup queue pair\n");
			return -1;
		}

                rdk_trace("rte_cryptodev_start()\n");
		retValue = rte_cryptodev_start(cdev_id);
		if (retValue < 0) {
			printf("Failed to start device %u\n", cdev_id);
			return -1;
		}
	}
    *num_cdevs = cdev_id;
    return retValue;
}

#if 0
void eth_bridge_tx_completion_callback_fn(struct rte_mbuf *m, void __attribute((unused)) *data)
{
    static int i = 0;
#if 0
    rte_pktmbuf_dump(stdout,m,m->pkt_len);
    printf("\nCallback function %s\n",__func__);
#endif
    if (RTE_MBUF_TX_CMPLTN_SUCCESS == rte_pktmbuf_tx_cmpltn_status(m)) {
        cpk_tx_success++; 
    }else {
        cpk_tx_drop++;
    }
    i++;
#if 0 
    if (!(i % 10000)) {
        printf("\nCPK TX success : %lu\n", cpk_tx_success);
        printf("\nCPK TX drop: %lu\n", cpk_tx_drop);
    }
    rte_pktmbuf_free(m);
#endif
}

void eth_bridge_tx_enqueue_callback_fn(struct rte_mbuf *m, void __attribute((unused)) *data)
{
    static int i = 0;
#if 1
    rte_pktmbuf_dump(stdout,m,m->pkt_len);
    printf("\nCallback function %s\n",__func__);
#endif
    if (RTE_MBUF_TX_ENQUEUE_SUCCESS == rte_pktmbuf_tx_enqueue_status(m)) {
        cpk_enqueue_success++; 
    }else {
        cpk_enqueue_drop++;
    }
    i++;

    if (!(i % 1000)) {
        printf("\nCPK ENQUEUE success : %lu\n", cpk_tx_success);
        printf("\nCPK ENQUEUE drop: %lu\n", cpk_tx_drop);
    }
}
#endif

static void
init_bridge_crypto (
	uint8_t event_dev_id,
	uint8_t crypto_event_port_id)
{
    struct rte_bridge_crypto_instance_config inst_config;
    struct rte_bridge_crypto_qp_config config;
    uint8_t instance_id = 0;
    uint8_t crypto_qp_id = 0;
    uint8_t crypto_dev_id;
    int ret;

    crypto_dev_id = 0;

    /* Creating crypto bridge driver instance */
    memset (&inst_config, 0x00, sizeof(struct rte_bridge_crypto_instance_config));
    memset (&config, 0x00, sizeof(struct rte_bridge_crypto_qp_config));
    config.crypto_dev_id = crypto_dev_id;
    inst_config.event_dev_id = event_dev_id;
    inst_config.event_port_id = crypto_event_port_id;

    rdk_trace("rte_bridge_crypto_instance_create()\n");
    ret = rte_bridge_crypto_instance_create(instance_id, &inst_config);
    if (ret)
        rte_panic("[%s()] Failed to create bridge crypto instance\n", __func__);

    rdk_trace("rte_bridge_crypto_qp_add()\n");
    ret = rte_bridge_crypto_qp_add(
             instance_id,
             crypto_qp_id,
             &config);

    if (ret)
        rte_panic("[%s()] Failed to add crypto queue pair \n", __func__);
} /* init_bridge_crypto */
static rte_eth_dev_cb_fn port_cb(uint16_t port_id, enum rte_eth_event_type event, void *cb_arg, void *ret_param)
{
        (void)event;
        (void)ret_param;
        uint16_t q = *(uint16_t*)cb_arg;
        printf("Restarting Port: %u RxQ: %u\n", port_id, q);

        rte_eth_dev_rx_queue_stop(port_id, q);
        rte_eth_dev_rx_queue_start(port_id, q);

        return 0;
} /* rte_eth_dev_cb_fn */
/*
 * initilize all ethernet ports, create rx&tx qeueues.
 * create a bridge ethernet instance and add all the ethernet ports to it.
 */
static int
init_bridge_eth (
    __attribute__((unused))unsigned num_ports,
    uint8_t event_dev_id,
    uint8_t rx_event_port_id,
    uint8_t tx_event_port_id,
    int32_t event_queue_index,
    struct rte_mempool *mp)
{
    uint8_t port_id;
    int ret;
    /* int32_t evqid = qid[event_queue_index]; */
    int32_t evqid = worker_qid[event_queue_index];

    const struct rte_bridge_eth_instance_config config = {
        .event_dev_id = event_dev_id,
        .rx_event_port_id = rx_event_port_id,
        .tx_event_port_id = tx_event_port_id,
    };

    uint8_t bridge_eth_instance_id = 0;

    rdk_trace("rte_bridge_eth_instance_create()\n");
    ret = rte_bridge_eth_instance_create(bridge_eth_instance_id, &config);
    if (ret)
        rte_panic("[%s()] Failed to create the bridge eth instance.\n",
                __func__);

    if (!mp)
    {
        printf("[%s()] Invalid mbuf pool\n", __func__);
        return -1;
    }

    testConfShm_p->ethDev.bridge_eth_instance_id = bridge_eth_instance_id;

    port_id = bh_eth_dev_id;
    {
        static const struct rte_eth_conf port_conf_default = {
            .rxmode = {
                .max_rx_pkt_len = 9192,
                .split_hdr_size = 0,
                .offloads = DEV_RX_OFFLOAD_JUMBO_FRAME
            },
            .rx_adv_conf = {
                .rss_conf = {
                    .rss_hf = ETH_RSS_IP | ETH_RSS_UDP | ETH_RSS_TCP,
                },
            },
            /* enable rx queue interrupt */
            .intr_conf = {.rxq = 0},
            .txmode = {
                .mq_mode = ETH_MQ_TX_NONE,
                .offloads = DEV_TX_OFFLOAD_MULTI_SEGS,
#ifdef ADK_USE_COMPLETION_QUEUE
                .nb_cp_q = DP_APP_VPORT_NUM_QUEUES, 
#endif
#if 0 /* Is replaced with MACRO in 18.11 */
                .security = 1
#endif
            },
            .link_speeds = 0
        };
        /* rx_qs, tx_qs set by primary max upto 8 or less if device limit is less */
        uint16_t rx_q_size = 1024;
        uint16_t tx_q_size = 2048;

        struct ice_dsi_txq_conf ice_dsi_txconf = {0};
        struct rte_pmd_ice_dsi_dev_info ice_dsi_info;

        struct rte_eth_conf port_conf = port_conf_default;
        uint16_t q_id;
        struct rte_eth_dev_info dev_info;
        struct rte_eth_txconf *txconf = NULL;

        /* Offloading crypto encryption and decrytpion when ipsec is enabled */
        if(enableIPsec)
        {
            port_conf.rxmode.offloads |= DEV_RX_OFFLOAD_SECURITY;
            port_conf.txmode.offloads |= DEV_TX_OFFLOAD_SECURITY;
        }

        rdk_trace("rte_eth_dev_info_get()\n");
        rte_eth_dev_info_get(port_id, &dev_info);
        txconf = &dev_info.default_txconf;

        /* enable multi segment handling on the txmit direction */
#ifdef ADK_QOS_GEN3_INTERFACE
        txconf->tx_deferred_start = 1;
#endif

        rte_pmd_ice_dsi_get_dev_info(port_id, &ice_dsi_info);
        printf("[%s()] Detected driver %s for device %d, VSI ID %d\n",
                __func__, dev_info.driver_name, port_id, ice_dsi_info.vsi_number);

    	/* 8 RX queues for 8 priority levels */
    	rx_qs = (dev_info.max_rx_queues > DP_APP_VPORT_NUM_QUEUES)? DP_APP_VPORT_NUM_QUEUES : dev_info.max_rx_queues;
    	/* TX queues */
    	tx_qs = (dev_info.max_tx_queues > ADK_DP_VPORT_NUM_TX_QUEUES )? ADK_DP_VPORT_NUM_TX_QUEUES : dev_info.max_tx_queues;
#ifdef ADK_USE_COMPLETION_QUEUE
        cp_qs =  DP_APP_VPORT_NUM_QUEUES ;
#endif

        /* Configure the Ethernet device. */
        printf("Configuring Eth-Device %d with %d Rx Queues and %d Tx Queues\n",
                port_id, rx_qs, tx_qs);
        rdk_trace("rte_eth_dev_configure()\n");
        ret = rte_eth_dev_configure(port_id, rx_qs, tx_qs, &port_conf);
        if (ret)
        {
            printf("[%s()] Failed to configure the eth dev\n",
                    __func__);
            return -1;
        }

        rdk_trace("rte_bridge_eth_dev_add()\n");
        ret = rte_bridge_eth_dev_add(bridge_eth_instance_id, port_id);
        if (ret) {
            printf("[%s()] Failed to add eth dev to bridge driver\n",
                    __func__);
            return -1;
        } else {
            printf("Sucessfully added eth dev [%d] to bridge driver instance[%d]\n",
                    port_id, bridge_eth_instance_id);
        }
#ifdef ADK_SYS_OVERLOAD_PROTECTION
        /*Register overload protection application function with bridge-driver */
        rdk_trace("rte_bridge_eth_rx_cb_register()\n");
        ret = rte_bridge_eth_rx_cb_register(bridge_eth_instance_id,
                port_id,
                adk_sys_dp_overlod_protection,
                NULL);
        if (ret) {
            printf("[%s()] Failed to register overload cb for eth dev to bridge driver\n",
                    __func__);
            return -1;
        } else {
            printf("Sucessfully registred overload cb for eth dev [%d] to bridge driver instance[%d]\n",
                    port_id, bridge_eth_instance_id);
        }
#endif

        {
            ae_handle cpk_ae_hdl;
            /* BDF of a CPK device  */
            uint8_t bus  = 0, dev = 0, function = 0;
            uint16_t  vport_id ;

            rdk_trace("ae_client_open()\n");
            ret = ae_client_open(bus, dev, function, &cpk_ae_hdl);
            if (ret) {
                printf("\n ae_client_open is failed for BUS:%d, DEV:%d, FUNC:%d\n",
                        bus,dev,function);
                return -1;
            }

            {
                uint16_t vport_dev_id;
                uint16_t tx_queue_id;

                /* Create fixed vport of name 10 */
                /*Create a VPORT
                 * port_id => VSI-ETH-DEV-ID*/
                vport_id = 20;
                ret = dp_app_vport_create(port_id, vport_id, vport_id,  &vport_dev_id);
                if(ret) {
                    printf("\n  dp_app_vport_create failed \n");
                    ae_client_close(cpk_ae_hdl);
                    return -1;
                }

                rdk_trace("rte_eth_dev_info_get()\n");
                rte_eth_dev_info_get(port_id, &dev_info);
                txconf = &dev_info.default_txconf;

#ifdef ADK_USE_COMPLETION_QUEUE
                txconf->cp_mode = RTE_ETH_TX_CP_QUEUE; 
                /* Allocate and set up completion queues */
                for (q_id = 0; q_id < cp_qs; q_id++)
                {
                    struct rte_eth_cpconf cp_conf;
                    rdk_trace("rte_eth_cp_queue_setup()\n");
                    ret = rte_eth_cp_queue_setup(port_id,
                            q_id,
                            tx_q_size,
                            rte_eth_dev_socket_id(port_id),
                            &cp_conf);
                    if (ret)
                    {
                        printf("[%s()] Failed to setup completion queue %d on "
                                "eth dev %d\n", __func__,
                                q_id, port_id);
                        return -1;
                    }
                    else {
                        printf("[%s()] Configured completion queue %d on "
                                "eth dev %d\n", __func__, q_id, port_id);
                    }
                }

#if 0 
                rte_bridge_eth_tx_cb_register(port_id, eth_bridge_tx_completion_callback_fn, NULL); 
#endif

#endif

                /* Add all the 8-queues to Ethernet-Bridge Driver */
                for (tx_queue_id = 0; tx_queue_id < tx_qs; tx_queue_id++)
                {
                    /* Update port_id in txconf struct for each queue */
                    txconf->extended_conf = &ice_dsi_txconf;
                    txconf->tx_rs_thresh = 0;
                    txconf->tx_free_thresh = 0;

                    /*Queue setups are done but queues wont be started untill application
                      explicit starts the tx queues*/
                    txconf->tx_deferred_start = 1;
                    txconf->max_queue_len  = 0xFFFF;
#ifdef ADK_USE_COMPLETION_QUEUE
                    txconf->cpq_id = tx_queue_id % cp_qs;
#endif

                    /* ret = rte_eth_dev_tx_queue_stop(port_id, tx_queue_id);  */

                    rdk_trace("rte_eth_tx_queue_setup()\n");
                    ret = rte_eth_tx_queue_setup(port_id, tx_queue_id, tx_q_size,
                            rte_eth_dev_socket_id(port_id),
                            txconf);
                    if (ret) {
                        rte_panic("[%s()] Failed to setup tx queue %d on eth dev %d\n",
                                __func__, tx_queue_id, port_id);
                        return -1;
                    } else {
#ifdef ADK_DEBUG
                        printf("[%s()] Configured tx queue %d on eth dev %d\n",
                                __func__, tx_queue_id, port_id);
#endif
                    }
                    rdk_trace("rte_bridge_eth_tx_queue_add()\n");
                    ret = rte_bridge_eth_tx_queue_add(bridge_eth_instance_id,
                            port_id, tx_queue_id, NULL);
                    if (ret) {
                        rte_panic("[%s()] Failed to add tx queue %d on port %d to bridge eth driver\n",
                                __func__, tx_queue_id, port_id);
                    } else {
                        printf("[%s()] Added tx queue %d on device %d to eth bridge %d\n",
                                __func__, tx_queue_id, port_id, bridge_eth_instance_id);
                    }
                }
#if 0
                /* Create Egress QoS hierarchy for VPORT */
                /* 1 queues => L1 scheduler => Root Scheduler (VPORT) */
                ret = dp_app_cpk_qos_hierarchy_create(vport_id, vport_dev_id, &vport_l1_node_list[port_id][vport_id]);
                if(ret) {
                    rte_panic("\n  dp_app_cpk_qos_hierarchy_create failed \n");
                    return -1;
                }
#endif

            }
        }

        for (q_id = 0; q_id < tx_qs; q_id++)
        {
#ifndef DP_APP_SNR_EGRESS_QOS
            txconf->tx_rs_thresh = 0;
            txconf->tx_free_thresh = 0;

            rdk_trace("rte_eth_tx_queue_setup()\n");
            ret = rte_eth_tx_queue_setup(port_id,
                    q_id,
                    tx_q_size,
                    rte_eth_dev_socket_id(port_id),
                    txconf);
            if (ret)
            {
                printf("[%s()] Failed to setup tx queue %d on "
                        "eth dev %d\n", __func__,
                        q_id, port_id);
                return -1;
            } else {
                printf("[%s()] Configured tx queue %d on "
                        "eth dev %d\n", __func__, q_id, port_id);
            }
            rdk_trace("rte_bridge_eth_tx_queue_add()\n");
            ret = rte_bridge_eth_tx_queue_add(bridge_eth_instance_id, port_id, q_id, NULL);
            if (ret)
            {
                printf("[%s()] Failed to add tx queue %d on port %d to bridge eth driver\n",
                        __func__, q_id, port_id);
            } else {
                printf("[%s()] Added tx queue %d on device %d to eth bridge %d\n",
                        __func__, q_id, port_id, bridge_eth_instance_id);
            }

#endif
        }

        /* Allocate and set up 1 RX queue per Ethernet port. */
        for (q_id = 0; q_id < rx_qs; q_id++)
        {
            struct rte_event ev;
            rdk_trace("rte_eth_rx_queue_setup()\n");
            ret = rte_eth_rx_queue_setup(port_id,
                    q_id,
                    rx_q_size,
                    rte_eth_dev_socket_id(port_id),
                    NULL, mp);
            if (ret)
            {
                printf("[%s()] Failed to setup rx queue %d on "
                        "eth dev %d\n", __func__,
                        q_id, port_id);
                return -1;
            }
#ifdef ADK_DEBUG
            else {
                printf("[%s()] Configured rx queue %d on "
                        "eth dev %d\n", __func__, q_id, port_id);
            }
#endif
            rte_eth_dev_callback_register(port_id, RTE_ETH_EVENT_QUEUE_STATE, (void*)port_cb, (void*)&q_id);

            /* USed first atomic queue to receive traffic from NIC
             * into worker*/

            ev.queue_id = evqid;
            ev.sched_type = sched_type;
            ev.priority = 0;

            const struct rte_bridge_eth_rx_queue_config rx_q_config = {
                .rx_queue_flags = 0,
                .servicing_weight = 1,
            };
            rdk_trace("rte_bridge_eth_rx_queue_add()\n");
            ret = rte_bridge_eth_rx_queue_add(bridge_eth_instance_id,
                    &ev,
                    port_id,
                    q_id,
                    &rx_q_config);
            if (ret < 0)
            {
                printf("Add NIC Queue=%u to bridge instance %u failed!\n", q_id,
                        bridge_eth_instance_id);
                return ret;
            }
#ifdef ADK_DEBUG
            else {
                printf("[%s()] Added Rx Queue %u on device %d to bridge %u!\n",
                        __func__, q_id, port_id, bridge_eth_instance_id);
            }
#endif
        }

        /* Start the Ethernet port. */
        rdk_trace("rte_eth_dev_start()\n");
        ret = rte_eth_dev_start(port_id);
        if (ret) {
            printf("\n  rte_eth_dev_start() failed for eth-dev %d \n", port_id);
            return ret;
        }
        /* Display the port MAC address. */
        struct ether_addr addr;
        rdk_trace("rte_eth_macaddr_get()\n");
        rte_eth_macaddr_get(port_id, &addr);
        printf("Port %u MAC: %02" PRIx8 " %02" PRIx8 " %02" PRIx8
                " %02" PRIx8 " %02" PRIx8 " %02" PRIx8 "\n",
                (unsigned)port_id,
                addr.addr_bytes[0], addr.addr_bytes[1],
                addr.addr_bytes[2], addr.addr_bytes[3],
                addr.addr_bytes[4], addr.addr_bytes[5]);

        /* Enable RX in promiscuous mode for the Ethernet device. */
    }

    return 0;
} /* init_bridge_eth */

static int
init_bridge_timer (
      uint8_t event_dev_id,
      uint8_t port_id,
      int lcore_id)
{
   int ret;

   /* Create an instance of the timer bridge driver, and tell it which
    * event port to use for injecting timer events into the eventdev.
    */
   rdk_trace("rte_bridge_timer_instance_create()\n");
   ret = rte_bridge_timer_instance_create(timer_inst_id, event_dev_id, port_id);
   if (ret)
      rte_panic("[%s()] Failed to create the bridge timer instance.\n",
            __func__);

   /* Configure the instance to run on the first lcore in the system. */
   rdk_trace("rte_bridge_timer_conf()\n");
   ret = rte_bridge_timer_conf(timer_inst_id, lcore_id);
   if (ret)
      rte_panic("[%s()] Failed to configure the bridge timer instance.\n",
            __func__);

   return 0;
}


/* DP_APP_USE_DPDK_SERVICE_CORES related info */
static uint8_t setup_event_dev(void)
{
	struct rte_event_dev_config config = {0};
	struct rte_event_queue_conf queue_config = {0};
	struct rte_event_port_conf port_config = {0};
	uint8_t queue_id;
	uint8_t priority;
	int prod_port;
	int cons_port;
	int timer_port = 0;

	uint8_t ph_port[NUM_PH_THREADS];
	int crypto_port = 0;
	int crypto_qid = 0;
	int ev_qid;
	int ev_port_id;
	int eth_tx_qid;
	int eth_rx_qid;
	int timer_qid = 0;
	int cppi_bd_evqid = 0;
	int cppi_bd_evport = 0;
	uint8_t ph_qid[NUM_PH_THREADS];
	uint8_t i, j;
	int ret;
	struct rte_event_dev_info dev_info;
	timer_bd_in_use = true;
	/* Get the device ID for event_ihqm0 if primary and ihqm1 is secondary . */
	rdk_trace("rte_event_dev_get_dev_id()\n");
	eventdev_id = rte_event_dev_get_dev_id("event_ihqm1");

	if (eventdev_id < 0)
		rte_panic("Failed to get event_dev id for ADK HQM\n");

	rdk_trace("rte_event_dev_info_get()\n");
	rte_event_dev_info_get(eventdev_id, &dev_info);

	if(dev_info.event_dev_cap & RTE_EVENT_DEV_CAP_QUEUE_ALL_TYPES) {
		printf("Event-dev supports QUEUE_ALL_TYPES\n");
	} else {
		printf("Event-dev doesn't supports QUEUE_ALL_TYPES\n");
	}

	/* Configure only if primary */
	/* Configure the event device. */
	config.dequeue_timeout_ns = 0;
	/*TODO: Setting this limit, to fix RDK1912 issue
	 *Additional check on number of events in rte_ring_create 
	 *original number of events 12288*/

	config.nb_events_limit = MAX_NUM_EVENTS;
	/* One queue per worker, plus one directed queue per bridge driver */
	config.nb_event_queues = num_worker_queues + num_dir_ports + NUM_ADK_ENTRY_POINT;
	config.nb_event_queue_flows = dev_info.max_event_queue_flows;
	config.nb_event_port_dequeue_depth = dev_info.max_event_port_dequeue_depth;
	config.nb_event_port_enqueue_depth = dev_info.max_event_port_enqueue_depth;

	/* One port per worker, plus one directed port per bridge driver, plus EM cores */
	config.nb_event_ports = num_worker_cores + num_dir_ports + number_em_cores;
	config.event_dev_cfg = 0;
	config.nb_single_link_event_ports = num_dir_ports;

	rdk_trace("rte_event_dev_configure()\n");
	ret = rte_event_dev_configure(eventdev_id, &config);
	if (ret)
		rte_panic("Failed to configure the event dev\n");

	em_start_port = num_worker_cores + num_dir_ports;

	/*Assign directed ev-p&q for different BDs */

	dp_app_next_dir_ev_p_q_set_get(&bd_info[DP_APP_ETH_RX_BD].event_port_id,
			&bd_info[DP_APP_ETH_RX_BD].qid);

	dp_app_next_dir_ev_p_q_set_get(&bd_info[DP_APP_ETH_TX_BD].event_port_id,
			&bd_info[DP_APP_ETH_TX_BD].qid);

	bd_info[DP_APP_ETH_RX_BD].event_dev_id = eventdev_id;
	bd_info[DP_APP_ETH_TX_BD].event_dev_id = eventdev_id;

	if(timer_bd_in_use)
	{
		dp_app_next_dir_ev_p_q_set_get(&bd_info[DP_APP_TIMER_BD].event_port_id,
				&bd_info[DP_APP_TIMER_BD].qid);
		bd_info[DP_APP_TIMER_BD].event_dev_id = eventdev_id;
	}


	if (cppi_netd_port_in_use) {
		dp_app_next_dir_ev_p_q_set_get(&bd_info[DP_APP_CPPI_BD].event_port_id,
				&bd_info[DP_APP_CPPI_BD].qid);
		bd_info[DP_APP_CPPI_BD].event_dev_id = eventdev_id;
	}

	if (enableIPsec)
	{
		dp_app_next_dir_ev_p_q_set_get(&bd_info[DP_APP_CRYPTO_BD].event_port_id,
				&bd_info[DP_APP_CRYPTO_BD].qid);
		bd_info[DP_APP_CRYPTO_BD].event_dev_id = eventdev_id;
	}

	/* Create load-balanced event queues for workers*/
	queue_config.nb_atomic_flows = dev_info.max_event_queue_flows;
	queue_config.nb_atomic_order_sequences = dev_info.max_event_queue_flows;
#if 1 /* Either configure max sequence number - when schdule type is ALL (including ordered) Or use only ATM */
	/* Total is 4096 for all queues and max a queue can have is 1024 */
	queue_config.nb_atomic_order_sequences = 256 /* Max hqm supports */;
	queue_config.event_queue_cfg = RTE_EVENT_QUEUE_CFG_ALL_TYPES;
#else
	/* event_queue_cfg to 0 indicates ATM+PARALLEL (not ordered) */
	queue_config.event_queue_cfg = 0;
	queue_config.schedule_type = RTE_SCHED_TYPE_ATOMIC;
	queue_config.nb_atomic_order_sequences = 0;
#endif
	for (i=0; i < NUM_ADK_ENTRY_POINT; i++)
	{
		queue_config.priority = 0;
		ret = rte_event_queue_setup(eventdev_id, i, &queue_config);
		if (ret < 0)
			rte_panic("Failed to create the scheduled QID[%d]\n", i);
		printf("Created ADK entry point : queue %d, nb_atomic_flows %d, nb_atomic_order_sequences %d\n",
				i, queue_config.nb_atomic_flows, queue_config.nb_atomic_order_sequences);
	}
	for (i = 0, ev_qid = NUM_ADK_ENTRY_POINT; i < num_worker_queues; i++, ev_qid++)
	{
		queue_config.priority = 0;
		rdk_trace("rte_event_queue_setup()\n");
		ret = rte_event_queue_setup(eventdev_id, ev_qid, &queue_config);
		if (ret < 0)
			rte_panic("Failed to create the scheduled QID[%d]\n", i);
		worker_qid[i] = ev_qid;
		testConfShm_p->qid[i] = ev_qid;
		printf("Created LB queue %d, nb_atomic_flows %d, nb_atomic_order_sequences %d\n",
				worker_qid[i], queue_config.nb_atomic_flows, queue_config.nb_atomic_order_sequences);
	}

	/* Create directed event queues. Although the NIC Rx and timer bridge
	 * driver ports don't receive packets, we create and link one queue so
	 * the eventdev PMD recognizes them as directed ports.
	 */

	/* The NIC Tx queue is configured with the highest priority */
	queue_config.priority = 0;
	/*
	 * Directed (SINGLE_LINK) queue, so no atomic flows needed, but
	 * non-zero to workaround an eventdev validation check.
	 */

	queue_config.nb_atomic_flows = 32;
	queue_config.nb_atomic_order_sequences = 32;
	queue_config.event_queue_cfg = RTE_EVENT_QUEUE_CFG_SINGLE_LINK;

	eth_tx_qid = bd_info[DP_APP_ETH_TX_BD].qid;
	rdk_trace("rte_event_queue_setup()\n");
	ret = rte_event_queue_setup(eventdev_id, eth_tx_qid, &queue_config);
	if (ret)
		rte_panic("Failed to setup the eth-tx-directed QID error %d\n", ret);
	printf("Created Tx BD queue %d\n", eth_tx_qid);

	eth_rx_qid = bd_info[DP_APP_ETH_RX_BD].qid;
	rdk_trace("rte_event_queue_setup()\n");
	ret = rte_event_queue_setup(eventdev_id, eth_rx_qid, &queue_config);
	if (ret < 0)
		rte_panic("Failed to create the eth-rx-directed QID\n");
	printf("Created Rx BD queue %d\n", eth_rx_qid);

	if (enableIPsec)
	{
		crypto_qid = bd_info[DP_APP_CRYPTO_BD].qid;
		rdk_trace("rte_event_queue_setup()\n");
		ret = rte_event_queue_setup(eventdev_id, crypto_qid, &queue_config);
		if (ret)
			rte_panic("Failed to setup the bridge crypto directed QID\n");
		printf("Created Crypto BD queue %d\n", crypto_qid);
	}

	if(timer_bd_in_use)
	{
		timer_qid = bd_info[DP_APP_TIMER_BD].qid;
		rdk_trace("rte_event_queue_setup()\n");
		ret = rte_event_queue_setup(eventdev_id, timer_qid, &queue_config);
		if (ret)
			rte_panic("Failed to setup the bridge timer directed QID\n");
	}


	if (cppi_netd_port_in_use) {
		cppi_bd_evqid = bd_info[DP_APP_CPPI_BD].qid;
		rdk_trace("rte_event_queue_setup()\n");
		ret = rte_event_queue_setup(eventdev_id,
				cppi_bd_evqid,
				&queue_config);
		if (ret)
			rte_panic("Failed to setup the cppi directed QID\n");
		else
			printf("cppi_bd_evqid %u configured\n", cppi_bd_evqid);
	}

	queue_config.event_queue_cfg = RTE_EVENT_QUEUE_CFG_SINGLE_LINK;
	for (i = 0; i < NUM_PH_THREADS; i++)
	{
		dp_app_next_dir_ev_p_q_set_get( &ph_port[i], &ph_qid[i]);
		rdk_trace("rte_event_queue_setup()\n");
		ret = rte_event_queue_setup(eventdev_id, ph_qid[i], &queue_config);
		if (ret < 0)
			rte_panic(" {%s} Failed to create the ph directed QID %d\n",
					__func__, i);
		printf("Created PH queue %d\n", ph_qid[i]);
	}

	printf("\n RDK-VERSION is 0x%X \n", RDK_VERSION);
	/* Create worker ports and bridge driver ports. */
	port_config.dequeue_depth = DP_APP_EVENTDEV_WORKER_DEQUEUE_DEPTH;
	port_config.enqueue_depth = DP_APP_EVENTDEV_ENQUEUE_DEPTH;
	/* port_config.new_event_threshold = dev_info.max_num_events; */
	/* FIXME: Configure correct value */
	port_config.new_event_threshold = MAX_NUM_EVENTS;
	port_config.event_port_cfg = 0;
	ev_port_id = 0;
	for (i = 0; i < DP_APP_MAX_LCORES; i++)
	{
		/* if we don't have a worker core, then skip */
		if (FUNC_WORKER != (coreInUsePacked[i].funcsOnCore & FUNC_WORKER))
		{
			continue;
		}

		if ((FUNC_RX == (coreInUsePacked[i].funcsOnCore & FUNC_RX)) ||
				(FUNC_TX == (coreInUsePacked[i].funcsOnCore & FUNC_TX)))
		{
			port_config.dequeue_depth = DP_APP_EVENTDEV_DEQUEUE_DEPTH;
		}

		worker_data[coreInUsePacked[i].coreId].event_port_id = ev_port_id;
		worker_data[coreInUsePacked[i].coreId].event_dev_id = eventdev_id;
		worker_data[coreInUsePacked[i].coreId].multi_events = true;

		rdk_trace("rte_event_port_setup()\n");
		ret = rte_event_port_setup(eventdev_id, worker_data[coreInUsePacked[i].coreId].event_port_id,
				&port_config);
		if (ret < 0)
			rte_panic("Failed to create worker port #%d for core %d, i: %d\n",
					ev_port_id, coreInUsePacked[i].coreId, i);

		ev_port_id++;

		printf("Created Worker i: %d, Port %d coreId: %d, funcsOnCore: 0x%02x, deQ_depth: %d\n",
				i,
				worker_data[coreInUsePacked[i].coreId].event_port_id,
				coreInUsePacked[i].coreId,
				coreInUsePacked[i].funcsOnCore,
				port_config.dequeue_depth);
	}

	port_config.new_event_threshold = MAX_NUM_EVENTS/2;
	port_config.dequeue_depth = DP_APP_EVENTDEV_DEQUEUE_DEPTH;
	port_config.event_port_cfg = 0;
	port_config.event_port_cfg |= RTE_EVENT_PORT_CFG_SINGLE_LINK;

	prod_port = bd_info[DP_APP_ETH_RX_BD].event_port_id;
	rdk_trace("rte_event_port_setup()\n");
	ret = rte_event_port_setup(eventdev_id, prod_port, &port_config);
	if (ret < 0)
		rte_panic("Failed to create the producer port\n");
	printf("Created Rx BD (Producer only) Port %d\n", prod_port);

	port_config.new_event_threshold = 256;
	cons_port = bd_info[DP_APP_ETH_TX_BD].event_port_id;
	rdk_trace("rte_event_port_setup()\n");
	ret = rte_event_port_setup(eventdev_id, cons_port, &port_config);
	if (ret < 0)
		rte_panic("Failed to create the consumer port\n");
	printf("Created Tx BD (Consumer only) Port %d\n", cons_port);

	if (enableIPsec)
	{
		port_config.new_event_threshold = 1024;
		crypto_port = bd_info[DP_APP_CRYPTO_BD].event_port_id;
		rdk_trace("rte_event_port_setup()\n");
		ret = rte_event_port_setup(eventdev_id, crypto_port, &port_config);
		if (ret)
			rte_panic("Failed to setup the crypto port\n");
		printf("\n Set UP crypto port = %d \n", crypto_port);
	}

	if(timer_bd_in_use)
	{
		port_config.new_event_threshold = 1024;// DP_APP_TIMER_BD_NEW_EVNT_THRESHOLD;
		port_config.dequeue_depth       = DP_APP_EVENTDEV_DEQUEUE_DEPTH;//Put in MACRO: DP_APP_TIMER_BD_EVNT_PORT_DEQ_DEPTH;
		port_config.enqueue_depth       = DP_APP_EVENTDEV_ENQUEUE_DEPTH;//DP_APP_TIMER_BD_EVNT_PORT_ENQ_DEPTH;
		timer_port = bd_info[DP_APP_TIMER_BD].event_port_id;
		rdk_trace("rte_event_port_setup()\n");
		ret = rte_event_port_setup(eventdev_id, timer_port, &port_config);
		if (ret)
			rte_panic("Failed to setup the timer port\n");
		printf("Created Timer Port %d\n", timer_port);
	}


	if (cppi_netd_port_in_use) {
		cppi_bd_evport = bd_info[DP_APP_CPPI_BD].event_port_id;
		port_config.new_event_threshold = 1024;
		printf("try to configure cppi_bd_evport %u cppi_bd_evqid %u\n",
				cppi_bd_evport, cppi_bd_evqid);
		rdk_trace("rte_event_port_setup()\n");
		ret = rte_event_port_setup(eventdev_id, cppi_bd_evport, &port_config);
		if (ret)
			rte_panic("Failed to setup the cppi port\n");
		else
			printf("Created CPPI BD Port %d\n", cppi_bd_evport);
		/* printf("cppi_bd_evport %u cppi_bd_evqid %u configured\n", cppi_bd_evport, cppi_bd_evqid); */
	}

	for (i = 0; i < NUM_PH_THREADS; i++)
	{
		rdk_trace("rte_event_port_setup()\n");
		ret = rte_event_port_setup(eventdev_id, ph_port[i], &port_config);
		if (ret < 0)
			rte_panic("Failed to create ph 0 port\n");
		printf("Created PH-Port %d\n", ph_port[i]);
	}

	if(is_unified_em_adk_set)
	{	 /* Create worker ports for EM */
		port_config.dequeue_depth = DP_APP_EVENTDEV_DEQUEUE_DEPTH;
		port_config.enqueue_depth = DP_APP_EVENTDEV_ENQUEUE_DEPTH;
		/* port_config.new_event_threshold = dev_info.max_num_events; */
		/* FIXME: Configure correct value */
		port_config.new_event_threshold = MAX_NUM_EVENTS/2;
		port_config.event_port_cfg = 0;
		for (i = 0; i < number_em_cores; i++)
		{
			rdk_trace("rte_event_port_setup()\n");
			ret = rte_event_port_setup(eventdev_id, em_portIds[i], &port_config);
			if (ret < 0)
				rte_panic("Failed to create em_ports\n");
			printf("Created EM-Port %d\n", em_portIds[i]);
		}
	}

	/*
	 * Map worker-ports to event queues. If the eventdev PMD supports it, the
	 * priority field defines the priority of the queue with respect to the port.
	 */
	for (i = 0 ; i < DP_APP_MAX_LCORES; i++)
	{
		/* Only connect queues to worker ports */
		if (0xff == workerCoreIdPacked[i])
			continue;
		for (j = 0; j < NUM_ADK_ENTRY_POINT; j++)
		{
			queue_id = j;
			priority = 0;
			ret = rte_event_port_link(eventdev_id,
					worker_data[workerCoreIdPacked[i]].event_port_id,
					&queue_id, &priority, 1);
			if (ret != 1)
				rte_panic("Failed map worker%d port %d to qid %d\n",
						workerCoreIdPacked[i],
						worker_data[workerCoreIdPacked[i]].event_port_id, j);

			printf("Linked ADK Entry Point:  Queue %d to Worker Port %d\n",
					queue_id, worker_data[workerCoreIdPacked[i]].event_port_id);
		}

		for (j = 0; j < num_worker_queues; j++)
		{
			queue_id = worker_qid[j];
			priority = 0;
			rdk_trace("rte_event_port_link()\n");
			ret = rte_event_port_link(eventdev_id,
					worker_data[workerCoreIdPacked[i]].event_port_id,
					&queue_id, &priority, 1);
			if (ret != 1)
				rte_panic("Failed map worker%d port %d to qid %d\n",
						workerCoreIdPacked[i],
						worker_data[workerCoreIdPacked[i]].event_port_id, j);

			printf("Linked LB i: %d, Queue %d to Worker Port %d\n", i,
					queue_id, worker_data[workerCoreIdPacked[i]].event_port_id);
		}

	}

	/* Map the NIC Tx bridge driver queue to its port */
	queue_id = eth_tx_qid;
	priority = 0;

	printf("Linking Tx BD queue %d to Tx BD (Consumer Port) %d\n", queue_id, cons_port);
	rdk_trace("rte_event_port_link()\n");
	ret = rte_event_port_link(eventdev_id, cons_port, &queue_id, &priority, 1);
	if (ret != 1)
		rte_panic("Failed to map NIC Tx port to its queue\n");

	/* Link producer port to its QID */
	queue_id = eth_rx_qid;
	priority = 0;

	printf("Linking Rx BD queue %d to Rx BD (Producer Port) %d\n", queue_id, prod_port);
	rdk_trace("rte_event_port_link()\n");
	ret = rte_event_port_link(eventdev_id, prod_port, &queue_id, &priority, 1);
	if (ret != 1)
		rte_panic("Failed to map NIC Rx port to its queue\n");

	if (enableIPsec)
	{
		queue_id = crypto_qid;
		priority = 0;
		printf("Linking crypto queue %d to crypto port %d \n", queue_id, crypto_port);
		rdk_trace("rte_event_port_link()\n");
		ret = rte_event_port_link(eventdev_id, crypto_port, &queue_id,
				&priority, 1);
		if (ret != 1)
			rte_panic("Failed to map crypto port to its queue\n");
	}

	if(timer_bd_in_use)
	{
		queue_id = timer_qid;
		priority = 0;
		printf("Linking Timer queue %d to Timer Port %d\n",
				queue_id, timer_port);
		rdk_trace("rte_event_port_link()\n");
		ret = rte_event_port_link(eventdev_id, timer_port, &queue_id,
				&priority, 1);
		if (ret != 1)
			rte_panic("Failed to map timer port to its queue\n");
	}


	if (cppi_netd_port_in_use) {
		/* Map the CPPI bridge driver queue to its netdriver port */
		queue_id = cppi_bd_evqid;
		priority = 0;
		printf("Linking CPPI BD queue %d to Producer Port %d\n", queue_id, cppi_bd_evport);

		rdk_trace("rte_event_port_link()\n");
		ret = rte_event_port_link(eventdev_id, cppi_bd_evport, &queue_id, &priority, 1);
		if (ret != 1)
			rte_panic("Failed to map CPPI Tx port to its queue\n");
		/* printf("linked cppi_bd_evport %u and cppi_bd_evqid %u\n", cppi_bd_evport, cppi_bd_evqid); */
	}

	/* Link PH port to its QID */
	for (i = 0; i < NUM_PH_THREADS; i++)
	{
		queue_id = ph_qid[i];
		priority = 0;
		printf("Linking PH queue %d to PH Port %d\n", queue_id, ph_port[i]);
		rdk_trace("rte_event_port_link()\n");
		ret = rte_event_port_link(eventdev_id, ph_port[i], &queue_id, &priority, 1);
		if (ret != 1)
			rte_panic("Failed to map PH port to prod_qid\n");
	}

	testConfShm_p->eth_tx_qid = eth_tx_qid;
	for (i = 0; i < NUM_PH_THREADS; i++)
	{
		testConfShm_p->phDev[i].portId = ph_port[i];
		testConfShm_p->phDev[i].rxQueueId[0] = ph_qid[i];
		testConfShm_p->phPipelineBindHandle[i] = 0;
	}

	return (uint8_t) eventdev_id;
}

static void
cppi_init(int cppi_inst_id)
{
    int     err;
    struct rte_bridge_cppi_instance_config inst_config = {0};

#ifdef ADK_DEBUG
    printf("  {%s} ins: %d\n", __func__, cppi_inst_id);
#endif

    /* Create a CPPI instance */
    rdk_trace("rte_bridge_cppi_instance_create()\n");
    err = rte_bridge_cppi_instance_create(cppi_inst_id, &inst_config);
    if(err)
        rte_panic("could not create a cppi instance %d\n", err);
}

#ifdef ADK_DEVICE_SNR
int setup_hlp_with_em(bool is_adk_master,uint8_t numPorts,uint8_t *portId);
int terminate_hlp(void);
#endif

#ifdef ADK_HOST_PATH_USE_NETD

uint16_t 
init_adk_slowpath_dev(adk_sys_handle_t sys_handle)
{
#define DP_APP_DEV_NET_N_QUEUES   8

    adk_dev_net_info_t            dev_info;
    adk_dev_net_config_params_t   config_params;
    int i;
    uint16_t    q_size = 1024;
    adk_status_t    ret;
 

    if (0 == cppi_lcore_mask) {
        printf(" CPPI-BD is required to setup adk_dev_net[NETD] \n");
        return USHRT_MAX;
    }

    sysHandle1 = sys_handle;
    /*The type should be ADK_DEV_NET_TYPE_NETD */
    dev_info.dev_type = ADK_DEV_NET_TYPE_NETD;
    dev_info.dm_type = ADK_DEV_NET_DM_TYPE_SOFT;

    /*use the smae packet memory pool that is registred with
     * ADK(adk-sys_resource_register)) to receive on
     * this device also */
    dev_info.pkt_mem_pool = NULL;

    /*Applicaiton is required to a create CPPI-BD with event-port and
     * event-queue. 
     * refer @init_bridge_cppi() */
    dev_info.bd_info.bd_instance_id = CPPI_INST_ID_BASE;

    /* Event-queue that is connected to CPPI-BD's event-port */
    dev_info.bd_info.bd_event_queue_id = bd_info[DP_APP_CPPI_BD].qid;

    /*This Device is full managed by ADK. 
     * Application must set the mgmt_type to ADK_DEV_NET_MGMT_TYPE_ADK_FULL */
    dev_info.mgmt_type = ADK_DEV_NET_MGMT_TYPE_ADK_FULL;

    dev_info.n_member_netdevs = 0;
    dev_info.dev_id = 0;

    /*Create a device net. This API returns the created device ID. 
     * Application has to use this device id to refer the current device in the
     * subsequent adk_dev_net*() API*/
    if(adk_dev_net_create(sys_handle , &dev_info, &dev_net_id) !=ADK_SUCCESS)
    {
        rte_panic(" adk_dev_net_create failed \n");
    }

    config_params.num_rx_queues = DP_APP_DEV_NET_N_QUEUES;
    config_params.num_tx_queues = DP_APP_DEV_NET_N_QUEUES;
    config_params.rx_sched_mode = ADK_DEV_NET_SCHED_MODE_RR;
    config_params.tx_sched_mode = ADK_DEV_NET_SCHED_MODE_RR;
    if(adk_dev_net_configure(sys_handle, dev_net_id, &config_params )!=ADK_SUCCESS)
    {
        rte_panic(" adk_dev_net_configure failed for %d linux dev \n",
                dev_net_id);
    }
    for(i=0;i<DP_APP_DEV_NET_N_QUEUES; i++) {
        if(adk_dev_net_rx_queue_setup(sys_handle, dev_net_id, i, 1,q_size) != ADK_SUCCESS) {
            rte_panic("adk_dev_net_rx_queue_setup failed for dev_net %d,"
                    "rx_queue %d \n", dev_net_id, i);
        }
        if(adk_dev_net_tx_queue_setup(sys_handle, dev_net_id, i, 1, q_size)!=ADK_SUCCESS) {
            rte_panic(" adk_dev_net_tx_queue_setup failed for dev_net %d, tx_queue %d \n", dev_net_id, i);
        }
    }
    /* This block of code to start the device and also do the synchronization
     * with ADK NetDriver. The ADK NetDriver must be loaded before this API
     * call. Other wise this API returns an error  ADK_E_NETD_FD_OPEN_FAILED*/
    do {
        uint8_t opt;
        ret = adk_dev_net_start(sys_handle, dev_net_id);
        if(ret==ADK_E_NETD_FD_OPEN_FAILED) {
            printf(" adk_dev_net_start failed NETD is not ready, make sure ADK NetDriver is loaded\n");
            printf(" want to proceed with out ADK netdriver? [y/n]\n");
            opt = getchar();
            if(opt=='y') {
                break;
            }
        } 
        /*sleep to aovid continuous trying */
        sleep(1);
    }while(ret==ADK_E_NETD_FD_OPEN_FAILED);
    printf(" adk_dev_net_start success \n");

    if(ret!=ADK_SUCCESS) {
        dev_net_id = USHRT_MAX;
        printf(" adk_dev_net_start failed error ret=%d \n",ret);
    } else {
        ret = adk_dev_net_stop(sys_handle, dev_net_id);
        if(ret!=ADK_SUCCESS) {
            printf(" adk_dev_net_stop failed error ret=%d \n",ret);
        }
        printf(" adk_dev_net_stop success \n");
        ret = adk_dev_net_start(sys_handle, dev_net_id);
        if(ret!=ADK_SUCCESS) {
            printf(" adk_dev_net_start failed error ret=%d \n",ret);
        }
        printf(" adk_dev_net_start success \n");
    }

    printf("\n %s() completed \n", __func__);
    return dev_net_id;
}

static void
init_bridge_cppi(uint8_t event_dev_id,
        uint8_t event_port_id)
{
    int err;
    uint8_t cppi_inst_id = CPPI_INST_ID_BASE;
    const struct rte_bridge_cppi_instance_config inst_config = {
        event_dev_id, event_port_id, 0, 0, 0};
    static uint8_t num_inst = 0;

    if (num_inst++ < MAX_CPPI_APP_INSTANCES) {
        /* Create a cppi instance */
        rdk_trace("rte_bridge_cppi_instance_create()\n");
        err = rte_bridge_cppi_instance_create(cppi_inst_id,
                &inst_config);
        if(err)
            rte_panic("could not create a cppi instance %d\n", err);
    }

}

/*
 * to stop and deleate previously created dev-net
 * */
int shutdown_adk_netd(void)
{
    if((0!=cppi_lcore_mask) && (dev_net_id!=USHRT_MAX)) {
        int ret;
        ret = adk_dev_net_stop(sysHandle1, dev_net_id);
        if(ret!=ADK_SUCCESS) {
            printf(" adk_dev_net_stop failed NETD is not ready\n");
        } 
        ret = adk_dev_net_delete(sysHandle1, dev_net_id);
        if(ret!=ADK_SUCCESS) {
            rte_panic(" adk_dev_net_delete failed error ret=%d \n",ret);
        } else {
            printf("adk_dev_net_delete success devid=%d \n",dev_net_id);
        }
    }
    printf("%s() completed \n", __func__);

    return 0;
}

static void
adk_dp_app_sig_handler(int sig_num)
{
    printf("perform cleanups on receiving signal %d\n", sig_num);
    adk_dp_shutdown();
    shutdown_adk_netd();
    signal(sig_num, SIG_DFL);
    raise(sig_num);
    return;
}

static void
setup_signal_handlers(void)
{
    struct sigaction sig_action;
    memset(&sig_action, 0 , sizeof(sig_action));
    sig_action.sa_handler = &adk_dp_app_sig_handler;

    if (sigaction(SIGINT, &sig_action, NULL) == -1 ||
        sigaction(SIGSEGV, &sig_action, NULL) == -1 ||
        sigaction(SIGILL, &sig_action, NULL) == -1 ||
        sigaction(SIGHUP, &sig_action, NULL) == -1)
        rte_panic("failed to setup signal handlers\n");

    printf("signal handlers setup successful\n");
    return;
}
#endif /* ADK_HOST_PATH_USE_NETD */

static int
service_register(const char *name, 
                 bool mt_safe, 
                 void *service_arg, 
                 rte_service_func fn, 
                 unsigned int *service_id)
{
    struct rte_service_spec service = {0};
    strncpy(service.name, name, RTE_SERVICE_NAME_MAX - 1);
    service.callback = fn;
    service.callback_userdata = service_arg;
    if (mt_safe)
        service.capabilities |= RTE_SERVICE_CAP_MT_SAFE;
    rdk_trace("rte_service_component_register()\n");
    return rte_service_component_register(&service, service_id);
}

static int
setup_qd_service(void)
{
    int i;
    if (service_register("qd_svc",
                         true,
                         worker_data,
                         rte_event_queue_dispatcher_service_fn,
                         &queue_dispatcher_service_id))
        rte_panic("Failed to register dispatcher as a service\n");
    printf("Service ID: %d, Name: Queue Dispatcher Service, Cores: ",
                 queue_dispatcher_service_id);
    for (i = 0; i < num_worker_cores; i++) 
    {
        if (rte_service_map_lcore_set(queue_dispatcher_service_id,
                                      workerCoreIdPacked[i],
                                      true))
            rte_panic("Failed to set the dispatcher's service map\n");
        printf("%d, ", workerCoreIdPacked[i]);
    }
    printf("\n");
    rte_service_component_runstate_set(queue_dispatcher_service_id, 1);
    return 0;
}

static int
setup_bridge_timer_service(void)
{
   int i;
   if (service_register("rdk_timer_bd",
            true,
            bridge_timer_args,
            rte_bridge_timer_service_fn,
            &timer_service_id))
      rte_panic("Failed to register bridge_timer as a service\n");
   printf("Service ID: %d, Name: Timer Service, Cores: ",
         timer_service_id);
   if (rte_bridge_timer_start(bridge_timer_args->instance_id,
            NULL))
      rte_panic("Failed to start bridge_timer\n");
   for (i = 0; i < num_worker_cores; i++)
   {
      printf("%d, ", workerCoreIdPacked[i]);
      if (rte_service_map_lcore_set(timer_service_id,
               workerCoreIdPacked[i],
               true))
         rte_panic("Failed to set the timer BD's service map\n");
   }
   printf("\n");
   rte_service_component_runstate_set(timer_service_id, 1);
   return 0;
}



static int
setup_bridge_eth_rx_service(struct rte_bridge_eth_rx_start_args *bridge_eth_rx_start_args)
{
    int i;

    if (service_register("rdk_eth_rx_bd",
                         true,
                         bridge_eth_rx_args, 
                         rte_bridge_eth_rx_service_fn,
                         &rx_service_id))
        rte_panic("Failed to register Rx bridge_eth as a service\n");
    printf("Service ID: %d, Name: Eth Rx Service, Cores: ",
                 rx_service_id);

    if (rte_bridge_eth_rx_start(bridge_eth_rx_args->instance_id,
            bridge_eth_rx_start_args))
        rte_panic("Failed to start Rx bridge_eth\n");

    for (i = 0; i < num_rx_cores; i++) 
    {
        if (rte_service_map_lcore_set(rx_service_id,
            rxCorePacked[i], true))
            rte_panic("Failed to set the Rx BD's service map\n");
         printf("%d, ", rxCorePacked[i]);
    }
    printf("\n");
    rte_service_component_runstate_set(rx_service_id, 1);
    return 0;
}

static int 
setup_bridge_eth_tx_service(void)
{
    int i;

    if (service_register("rdk_eth_tx_bd",
                         true,
                         bridge_eth_tx_args, rte_bridge_eth_tx_service_fn,
                         &tx_service_id))
        rte_panic("Failed to register Tx bridge_eth as a service\n");
    printf("Service ID: %d, Name: Eth Tx Service, Cores: ",
                 tx_service_id);

    if (rte_bridge_eth_tx_start(bridge_eth_tx_args->instance_id,
                                NULL))
        rte_panic("Failed to start Tx bridge_eth\n");

    for (i = 0; i < num_tx_cores; i++) 
    {
        if (rte_service_map_lcore_set(tx_service_id, txCorePacked[i], true))
            rte_panic("Failed to set the Tx BD's service map\n");
         printf("%d, ", txCorePacked[i]);
    }
    printf("\n");
    rte_service_component_runstate_set(tx_service_id, 1);
    return 0;
}

static int
setup_bridge_crypto_service(void)
{
    int i;

    if (service_register("rdk_crypto_bd",
        true,
        bridge_crypto_args,
        rte_bridge_crypto_service_fn,
        &crypto_service_id))
        rte_panic("Failed to register bridge_crypto as a service\n");
    printf("Service ID: %d, Name: Crypto Service, Cores: ",
                 crypto_service_id);

    if (rte_bridge_crypto_start(bridge_crypto_args->instance_id,
                                NULL))
        rte_panic("Failed to start bridge_crypto\n");
    printf("Instance ID: %d", bridge_crypto_args->instance_id);

    for (i = 0; i < num_crypto_cores; i++) 
    {
        if (rte_service_map_lcore_set(crypto_service_id,
                                      cryptoCorePacked[i],
                                      true))
            rte_panic("Failed to set the crypto BD's service map\n");
         printf("%d, ", cryptoCorePacked[i]);
    }
    printf("\n");
    rte_service_component_runstate_set(crypto_service_id, 1);
    return 0;
}

static int
setup_bridge_cppi_service(void)
{
    int i;

    if (service_register("rdk_cppi_bd",
        true,
        bridge_cppi_args,
        rte_bridge_cppi_service_fn,
        &cppi_service_id))
        rte_panic("Failed to register bridge_cppi as a service\n");
    printf("Service ID: %d, Name: CPPI Service, Cores: ",
                 cppi_service_id);

    if (rte_bridge_cppi_start(bridge_cppi_args->instance_id, NULL))
        rte_panic("Failed to start bridge_cppi\n");
    printf("Instance ID: %d", bridge_cppi_args->instance_id);

    for (i = 0; i < num_cppi_cores; i++) 
    {
        if (rte_service_map_lcore_set(cppi_service_id,
                                      cppiCorePacked[i],
                                      true))
            rte_panic("Failed to set the cppi BD's service map\n");
         printf("%d, ", cppiCorePacked[i]);
    }
    printf("\n");
    rte_service_component_runstate_set(cppi_service_id, 1);
    return 0;
}

static int
bridge_drivers_init(void)
{
    if ((rx_lcore_mask || tx_lcore_mask) && rte_bridge_eth_init())
    {
        printf("Failed to rte_bridge_eth_init()\n");
        return -1;
    }

    if (enableIPsec && rte_bridge_crypto_init())
    {
        printf("Failed to rte_bridge_crypto_init()\n");
        return -1;
    }

    if (rte_bridge_timer_init())
    {
        printf("Failed to rte_bridge_timer_init()\n");
        return -1;
    }

    if ((0 != cppi_lcore_mask) && rte_bridge_cppi_init())
    {
        printf("Failed to rte_bridge_cppi_init()\n");
        return -1;
    }

    return 0;
}

int adk_dp_setup(int argc, char **argv)
{
    adk_status_t    retValue = ADK_SUCCESS;
    unsigned        num_eth_devs;
    unsigned int    lcoreIndex, i, j;
    uint32_t        coreId;
    int             err;
    uint8_t         portId[20];
    uint8_t         numPorts;
#ifdef SELECT_INTERFACE_ENABLED
    int             ret;
    int             pipe;
    char            pipeName[sizeof(MESSAGE_PIPE)+10];
#endif
    adk_sys_version_info_t  adkVerInfo;
    unsigned        num_cdevs = 0;
    if(!is_unified_em_adk_set)
    {
   /* FIXME: ADK should initialize hlp only for the number of ports
    * it controls. Currently initializing for all 20 ports
    */
    for (numPorts=0;numPorts < 20;numPorts++)
    {
        portId[numPorts] = numPorts;
    }
	    is_adk_master = 1;
    printf("\nINFO: Initializing HLP is_adk_master = %d : %s\n",is_adk_master, (is_adk_master?"MASTER":"SLAVE"));
    if( setup_hlp_with_em(is_adk_master,numPorts,portId))
    {
        rte_panic("HLP Initialization failed!!\n");
    }
    else
    {
        printf("Completed HLP Setup\n");
    }

    rdk_trace("rte_eal_init()\n");
    err = rte_eal_init(argc, argv);
    if (err < 0)
        rte_panic("%s, rte_eal_init failed\n",__func__);

    argc -= err;
    argv += err;
    }

    if (rte_eal_process_type() == RTE_PROC_PRIMARY)
    {
       is_primary = true;
       printf("\nINFO: ADK is PRIMARY PROCESS\n");
    }
    else
    {
       printf("\nINFO: ADK is SECONDARY PROCESS. Returning..\n");
       return 1;
    }
    /* Parse cli options*/
    parse_app_args(argc, argv);
    if(false == enableIPsec || false == crypto_bd_in_use || 0 == crypto_lcore_mask)
    {
        rte_panic("[%s()] ERR: Please provide -i and -c options.. can't proceed\n", __func__);
    }

   timer_bd_in_use = true;

    /*create QM HQM */
    err = dp_app_qm_init();
    if (err)
        rte_panic("[%s()] Failed to initialize the QM VDEV.\n", __func__);

    /* Get Number of NIC ports bind to DPDK */

    rdk_trace("rte_eth_dev_count_avail()\n");
    num_eth_devs = rte_eth_dev_count_avail();

    printf("\n %d ethernet devices found\n", num_eth_devs);

    /* Create in built VSI for TN use cases */
    /* For ADK - we need only 1 vport with 8 queues to represent bridge interface */
    /* Assume net_ice_dsi1 as ADK port
     * FIXME: Take this number as argv
     */
#ifdef ADK_USE_COMPLETION_QUEUE
    err = dp_app_vsi_init(ADK_ETH_DEV_ID, DP_APP_VPORT_NUM_QUEUES,ADK_DP_VPORT_NUM_TX_QUEUES,DP_APP_VPORT_NUM_QUEUES, &bh_eth_dev_id);
#else
    err = dp_app_vsi_init(ADK_ETH_DEV_ID, DP_APP_VPORT_NUM_QUEUES,ADK_DP_VPORT_NUM_TX_QUEUES, &bh_eth_dev_id);
#endif
    if (err)
        rte_panic("[%s()] Failed to initialize the VSI-DEV.\n", __func__);

    /* Get Number of NIC ports bind to DPDK */
    num_eth_devs = rte_eth_dev_count_avail();
    if (num_eth_devs == 0)
        rte_panic("No ethernet devices found\n");
    printf("\n %d ethernet devices found after VSI INIT\n", num_eth_devs);

    if (!quiet)
    {
        printf("\nRTE Version:                %s\n", rte_version());
        adk_sys_adk_version_info_get(0, &adkVerInfo);
        printf("ADK Version:                %s\n", adkVerInfo.adk_version);
        printf("ADK Build Date:             %s\n", adkVerInfo.adk_build_date);
        printf("ADK Release Date:           %s\n", adkVerInfo.adk_release_date);
        printf("ADK SCM Tag:                %s\n", adkVerInfo.adk_scm_tag);
        printf("ADK Built against DPDK:     %s\n", adkVerInfo.adk_rte_version);
        printf("ADK/NetD FP Intf Version:   %s\n", adkVerInfo.adk_netd_fp_intf);
        printf("\n");


        printf("  Config:\n");
        printf("\teth devices: %u\n", num_eth_devs);
        printf("\tworker mask: 0x%.16lx\n", worker_lcore_mask);
        printf("\tcppi mask:   0x%.16lx\n", cppi_lcore_mask);
        printf("\trx mask:     0x%.16lx\n", rx_lcore_mask);
        printf("\ttx mask:     0x%.16lx\n", tx_lcore_mask);
        printf("\tph enabled   %s\n", ((false == enablePh) ? "false" : "true"));
        if (sched_type == RTE_SCHED_TYPE_ORDERED)
            printf("\tqid0 type: ordered\n");
        if (sched_type == RTE_SCHED_TYPE_ATOMIC)
            printf("\tqid0 type: atomic\n");
        printf("\n");
    }

    if (!quiet)
    {
        printf("Number of cores available: %u\n", rte_lcore_count());
        printf("Number of cores to be used: %u\n", cores_needed);
    }

    if (rte_lcore_count() < cores_needed)
        rte_panic("Too few cores\n");

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    bridge_eth_rx_args = rte_malloc(NULL,
                                    sizeof(*bridge_eth_rx_args),
                                    RTE_CACHE_LINE_SIZE);
    bridge_eth_tx_args = rte_malloc(NULL,
                                    sizeof(*bridge_eth_tx_args),
                                    RTE_CACHE_LINE_SIZE);
    bridge_timer_args = rte_malloc(NULL,
                                    sizeof(*bridge_timer_args),
                                    RTE_CACHE_LINE_SIZE);
    bridge_crypto_args = rte_malloc(NULL,
                                    sizeof(*bridge_crypto_args),
                                    RTE_CACHE_LINE_SIZE);
    bridge_cppi_args = rte_malloc(NULL,
                                    sizeof(*bridge_cppi_args),
                                    RTE_CACHE_LINE_SIZE);
    if(!bridge_eth_rx_args || !bridge_eth_tx_args || !bridge_crypto_args 
       || !bridge_timer_args || !bridge_cppi_args)
        rte_panic("Could not allocate BD args memory\n");
    if (err)
        rte_panic("Failed to set the worker lcore mask\n");

    rdk_trace("rte_event_dev_count()\n");
    const uint8_t ndevs = rte_event_dev_count();
        if (ndevs == 0)
            rte_panic("****Number of event devs must be atleast 1. But found %d..can't proceed******\n",ndevs);
    /* Allocate a memory pool for test shared data */
    retValue = test_shm_create(TEST_CONFIG_MEMORY_POOL, sizeof(test_config_t),
                               TEST_SHM_LOOKUP_LCORE,
                               (test_shm_handle_t **)&testConfShm_p);
    if (retValue != ADK_SUCCESS)
    {
        rte_panic("Couldn't create TEST_CONFIG_MEMORY_POOL\n");
    }
    testConfShm_p->tn_vsi_eth_dev = bh_eth_dev_id;
    for (i = 0; i < ADK_PH_DEV_MAX_NUM_DEVICES; i++)
    {
        testConfShm_p->phDev[i].devId = 0xff;
        for (j = 0; j < ADK_PH_DEV_MAX_NUM_QUEUES; j += 1)
        {
            testConfShm_p->phDev[i].rxQueueId[j] = 0xffff;
            testConfShm_p->phDev[i].txQueueId[j] = 0xffff;
        }
    }

    /*
     * Get the DPDK port info
     */
    dpdk_to_adk_port_number_get();

    /* Setup Event Dev */
    evdev_id = setup_event_dev();
    printf("evdev_id: %d, eventdev_id: %d\n", evdev_id, eventdev_id);


    /*Create packet memory pool */
    packet_pool = create_pkt_pool();
    if (NULL == packet_pool)
    {
        rte_panic("Couldn't create packet_pool\n");
    }

    testConfShm_p->evdev_id = evdev_id;
    testConfShm_p->packet_pool = packet_pool;
    /* Initialize the bridge drivers */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    err = bridge_drivers_init();
    if (err)
        rte_panic("Failed to initialize the bridge drivers.\n");

    /* Setup Bridge Driver*/
    err = init_bridge_eth(num_eth_devs,
                          evdev_id,
                          bd_info[DP_APP_ETH_RX_BD].event_port_id,
                          bd_info[DP_APP_ETH_TX_BD].event_port_id,
                          STAGE1_QUEUE_INDEX,
                          packet_pool);

    if (err)
        rte_panic("Failed to setup the bridge eth\n");

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    /* Initialize the queue dispatcher */
    if (rte_event_queue_dispatcher_init())
        rte_panic("Failed to initialize the queue dispatcher.\n");

    if (enableIPsec)
    {
        err = initialize_cryptodevs(&num_cdevs);
        init_bridge_crypto(evdev_id,
		       bd_info[DP_APP_CRYPTO_BD].event_port_id);
    }

/* Configure the timer bridge driver */
    if (timer_bd_in_use)
    {
       init_bridge_timer(evdev_id,
             bd_info[DP_APP_TIMER_BD].event_port_id,
             workerCoreIdPacked[0]);
    }

    if (0 != cppi_lcore_mask)
    {
        if (cppi_netd_port_in_use) {
            setup_signal_handlers();
            /* Configure the CPPI bridge driver */
            init_bridge_cppi(evdev_id,
                    bd_info[DP_APP_CPPI_BD].event_port_id);
        }
        else
            cppi_init(CPPI_INST_ID_BASE);
    }

    /* Note: service cores added *after* init_bridge_timer(), because
     * rte_timer_subsystem_init() does not initialize the timer structures
     * for service cores.
     * Currently as SQM is not used timer_bd not required
     */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    for (i = 0; i < DP_APP_MAX_LCORES; i++)
    {
        if (coreInUsePacked[i].funcsOnCore) 
        {
            ret = rte_service_lcore_add(coreInUsePacked[i].coreId);
            RTE_VERIFY(ret == 0 || ret == -EALREADY);
            num_service_cores++;
        }
    }
    /* Register the timer bridge driver to execute on all worker cores
     * (although only the first lcore sends timer events).
     */
if (timer_bd_in_use)
{
   bridge_timer_args->instance_id = 0;
   bridge_timer_args->work = 1;
   err = setup_bridge_timer_service();
   if (err)
      rte_panic("Failed to register the timer bridge driver with "
            "the integrated framework\n");
}

    /* Register the NIC Rx bridge driver. */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    struct rte_bridge_eth_rx_start_args bridge_eth_rx_start_args;
    bridge_eth_rx_start_args.lcore_ids = rxCorePacked;
    bridge_eth_rx_start_args.lcore_array_len = __builtin_popcountll(rx_lcore_mask);
    bridge_eth_rx_args->instance_id = 0;
    bridge_eth_rx_args->work = 128;
    err = setup_bridge_eth_rx_service(&bridge_eth_rx_start_args);
    if (err)
        rte_panic("Failed to register the NIC Rx bridge driver with "
                  "the integrated framework\n");

    /* Register the NIC Tx bridge driver. */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    bridge_eth_tx_args->instance_id = 0;
    bridge_eth_tx_args->work = 128;
    err = setup_bridge_eth_tx_service();
    if (err)
        rte_panic("Failed to register the NIC Tx bridge driver with "
                  "the integrated framework\n");

    if (enableIPsec)
    {
       /* DP_APP_USE_DPDK_SERVICE_CORES related info */
        bridge_crypto_args->instance_id = 0;
        bridge_crypto_args->work = 128;
        err = setup_bridge_crypto_service();
        if (err)
            rte_panic("Failed to register the crypto bridge driver with "
                    "the integrated framework\n");
    }

    if (0 != cppi_lcore_mask)
    {
       /* DP_APP_USE_DPDK_SERVICE_CORES related info */
        bridge_cppi_args->instance_id = 0;
        bridge_cppi_args->work = MAX_CPPI_WORKERS;
        err = setup_bridge_cppi_service();
        if (err)
        {
            rte_panic("Failed to register the CPPI bridge driver with the "
                      "integrated framework\n");
        }
    }

#ifdef SELECT_INTERFACE_ENABLED
    for (i = 0; i < ADK_PH_DEV_MAX_NUM_QUEUES; i++)
    {
        /*
         * Create and open the named pipe/fifo for message passing between
         * processes. This is the DP write side of the pipe.
         */
        memset(&pipeName, 0x00, sizeof(pipeName));
        snprintf(pipeName, sizeof(MESSAGE_PIPE)+8, MESSAGE_PIPE"_%u", i);
        pipeName[sizeof(MESSAGE_PIPE)+9] = '\000';
        ret = mkfifo(pipeName, FIFO_MODE);
        if ((-1 == ret) && (EEXIST != errno))
        {
            rte_panic("  {%s} Unable to create FIFO[%d] '%s', '%s' (%d)\n",
                      __func__, i, pipeName, strerror(errno), errno);
        }

        pipe = open(pipeName, (O_RDWR | O_NONBLOCK));
        if (-1 == pipe)
        {
            rte_panic("  {%s} Unable to open DP Pipe[%d] '%s', '%s' (%d)\n",
                      __func__, i, pipeName, strerror(errno), errno);
        }

        testConfShm_p->phDev[i].dpToCpPipe_fds[DP_WRITE_FD] = pipe;
    }
#endif


    /*
     * Queue dispatcher equivalent
     * Register the worker callback functions per event-queue
     *
     * */
    for (i = 0; i < NUM_ADK_ENTRY_POINT ; i++)
    {
	    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
	    rdk_trace("rte_event_queue_dispatcher_register_queue_function()\n");
	    err = rte_event_queue_dispatcher_register_queue_function(evdev_id,
			    i,
			    adk_pal_event_dispatcher,
			    NULL);
	    if (err)
		    rte_panic("Failed to register queue %d's callback\n",i);

    }
    for (i = 0; i < num_worker_queues; i++)
    {
       /* DP_APP_USE_DPDK_SERVICE_CORES related info */
        rdk_trace("rte_event_queue_dispatcher_register_queue_function()\n");
        err = rte_event_queue_dispatcher_register_queue_function(evdev_id,
                                                    worker_qid[i],
                                                    adk_pal_event_dispatcher,
                                                    NULL);
        if (err)
            rte_panic("Failed to register queue %d's callback\n",
                      worker_qid[i]);
    }

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    /* Register the queue dispatcher service */
    err = setup_qd_service();
    if (err)
        rte_panic("Failed to register the queue dispatcher\n");

    /* Launch the worker cores, skipping the main thread's lcore */
    coreId = rte_lcore_id();
    /* Checking if the main thread was assigned as service core by mistake */
    for (lcoreIndex = 0; lcoreIndex < DP_APP_MAX_LCORES; lcoreIndex++)
    {
        if (coreInUsePacked[lcoreIndex].coreId == coreId)
            rte_panic("Error: DPDK doesn't allow master lcore to be run as a service core!!\n");
    }
    for (lcoreIndex = 0; lcoreIndex < DP_APP_MAX_LCORES; lcoreIndex++)
    {
        if (0xff == coreInUsePacked[lcoreIndex].coreId)
            continue;

        printf("[%s()] Launching core %d (index: %d)\n", __func__,
               coreInUsePacked[lcoreIndex].coreId, lcoreIndex);
        /* DP_APP_USE_DPDK_SERVICE_CORES related info */
         /* wait for lcores before start() */
        rte_eal_wait_lcore(coreInUsePacked[lcoreIndex].coreId);
        rdk_trace("rte_service_lcore_start()");
        err = rte_service_lcore_start(coreInUsePacked[lcoreIndex].coreId);
        RTE_VERIFY(err == 0 || err == -EALREADY);
        if (err)
            rte_panic("Failed to launch scheduler\n");
    }
    /* Initialize ADK data plane modules */
    if (adk_resource_init(num_eth_devs))
        rte_panic("Failed to initialize ADK data plane!!\n");

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    /* Start the services */
        rte_service_runstate_set(queue_dispatcher_service_id, 1);
        rte_service_runstate_set(rx_service_id, 1);
        rte_service_runstate_set(tx_service_id, 1);
        if (enableIPsec)
        {
            rte_service_runstate_set(crypto_service_id, 1);
        }
        rte_service_runstate_set(timer_service_id, 1);
        if (0 != cppi_lcore_mask)
        {
            rte_service_runstate_set(cppi_service_id, 1);
        }

    /* Kick off the event dev */
    rdk_trace("rte_event_dev_start()\n");
    ret = rte_event_dev_start(eventdev_id);
    if (ret)
        rte_panic("[%s()] rte_event_dev_start failed with %d\n",
                  __func__, ret);

    printf("[%s()] Core %d is the main thread\n", __func__, coreId);
    return 0;
} /* adk_dp_setup */

void adk_dump_dev_stats(uint8_t opt)
{
    do
    {
        struct rte_bridge_eth_stats eth_stats;

        rdk_trace("rte_bridge_eth_stats_get()\n");
        rte_bridge_eth_stats_get(0, &eth_stats);

        printf("\n");
        printf(" rx_packets   ::%" PRId64 "\n", eth_stats.rx_packets);
        printf(" rx_enq_fail  ::%" PRId64 "\n", eth_stats.rx_enq_fail);
        printf(" rx_enq_retry ::%" PRId64 "\n", eth_stats.rx_enq_retry);
        printf(" tx_packets   ::%" PRId64 "\n", eth_stats.tx_packets);
        printf(" tx_fail      ::%" PRId64 "\n", eth_stats.tx_fail);
        printf(" tx_retry     ::%" PRId64 "\n", eth_stats.tx_retry);
        if (opt == 'e') {
            printf("Event Device Stats\n");
            rdk_trace("rte_event_dev_dump()\n");
            rte_event_dev_dump(eventdev_id, stdout);
        }

        if (opt == 'n') {
            printf("ADK Device Net Stats\n");
            adk_dev_net_stats_get(sysHandle1, dev_net_id);
        }
    } while (0);

    return;
}

void adk_dump_port_stats(uint16_t port)
{
    struct rte_eth_xstat_name xstats_names[100];
    int xtats_array_sz;

    rdk_trace("rte_eth_xstats_get_names()\n");
    xtats_array_sz = rte_eth_xstats_get_names(port, xstats_names, 100);
    if (xtats_array_sz <= 100)
    {
        int k;
        struct rte_eth_xstat xstats[100];

        printf("Port-%d's, %d extended stats are:\n", port, xtats_array_sz);
        for (k = 0; k < xtats_array_sz; k++)
        {
            xstats[k].id = k;
            xstats[k].value = 0;
        }

        rdk_trace("rte_eth_xstats_get()\n");
        xtats_array_sz = rte_eth_xstats_get(port, xstats, 100);
        if ((xtats_array_sz < 0) || (xtats_array_sz > 100))
        {
            printf("Port %d need larger xstats array size = %d\n",
                   port, xtats_array_sz);
            return;
        }
        for (k = 0; k < xtats_array_sz; k++)
        {
            if ((xstats[k].value) ||
                (!strcmp(xstats_names[k].name, "rx_good_packets")) ||
                (!strcmp(xstats_names[k].name, "tx_good_packets")) ||
                (!strcmp(xstats_names[k].name, "rx_good_bytes")) ||
                (!strcmp(xstats_names[k].name, "tx_good_bytes")) ||
                (!strcmp(xstats_names[k].name, "rx_unicast_packets")) ||
                (!strcmp(xstats_names[k].name, "tx_unicast_packets")) ||
                (!strcmp(xstats_names[k].name, "rx_multicast_packets")) ||
                (!strcmp(xstats_names[k].name, "tx_multicast_packets")) ||
                (!strcmp(xstats_names[k].name, "rx_broadcast_packets")) ||
                (!strcmp(xstats_names[k].name, "tx_broadcast_packets")) ||
                (!strcmp(xstats_names[k].name, "rx_good_packets")))
            printf("\t%s: %lu\n", xstats_names[k].name, xstats[k].value);
        }
    } else {
        printf("Port %d need larger xstats name array size = %d\n",
                port, xtats_array_sz);
    }

    return;
}

void adk_dump_ies_table(void)
{
    uint8_t opt;
    uint32_t ies_ret;

    do {
        printf("Enter:\n"
               "\tm for mapper table\n"
               "\tf for MAC FDB table\n"
               "\td for Domain mapper FDB table\n"
               "\tt for Triggers\n"
               );
        opt = getchar();
	if (opt != 27)
	{
	   continue;
	}
#if defined(ADK_DEVICE_SNR) && (RDK_REL_MAJOR < 14)
        if(opt == 'm') {
            ies_ret= ies_mapper_dump(E_IPT_SWITCH_ID);
            if (ies_ret != IES_OK) {
                printf("IES Error %s, Failed to dump scenario table \n",
                        IPT_IES_ERROR_NAME(ies_ret));
                return;
            }
        }
        if(opt == 'f') {
            ies_ret = ies_addr_dbg_dump_table(E_IPT_SWITCH_ID);
            if (ies_ret != IES_OK) {
                printf("IES Error %s, Failed to dump l2fdb\n",
                        IPT_IES_ERROR_NAME(ies_ret));
                return;
            }
        }
        if(opt == 'd') {
            ies_ret = ies_domain_dump(E_IPT_SWITCH_ID);
            if (ies_ret != IES_OK) {
                printf("IES Error %s, Failed to dump domain mapper table \n",
                        IPT_IES_ERROR_NAME(ies_ret));
                return;
            }
        }
        if(opt == 't') {
            ies_ret = ies_trigger_dump(E_IPT_SWITCH_ID);
            if (ies_ret != IES_OK) {
                printf("IES Error %s, Failed to dump triggers \n",
                        IPT_IES_ERROR_NAME(ies_ret));
                return;
            }
        }
#else
        RTE_SET_USED(ies_ret);
#endif
    } while (0);

    return;
}

int adk_dp_shutdown(void)
{
    unsigned int lcoreIndex, i;
    uint32_t coreId;
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    int err;
    coreId = rte_lcore_id();
    is_unified_em_adk_set = 0;
    number_em_cores = 0;
    em_start_port = 0;
#if 0
    dp_app_vport_cleanup();
#endif
#ifdef SELECT_INTERFACE_ENABLED
    for (i = 0; i < ADK_PH_DEV_MAX_NUM_QUEUES; i++)
    {
        if (0 != testConfShm_p->phDev[i].dpToCpPipe_fds[DP_WRITE_FD])
        {
            close(testConfShm_p->phDev[i].dpToCpPipe_fds[DP_WRITE_FD]);
        }
    }
#endif
    shutdown_adk_netd();

    rte_eth_dev_stop(bh_eth_dev_id);
    terminate_hlp();

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    /* Stop the services *before* stopping the service cores */
    rte_service_component_runstate_set(queue_dispatcher_service_id, 0);
    rte_service_component_runstate_set(rx_service_id, 0);
    rte_service_component_runstate_set(tx_service_id, 0);
    rte_service_component_runstate_set(timer_service_id, 0);
    if (enableIPsec)
    {
        rte_service_component_runstate_set(crypto_service_id, 0);
    }
    if (0 != cppi_lcore_mask)
    {
        rte_service_component_runstate_set(cppi_service_id, 0);
    }

    for (lcoreIndex = 0; lcoreIndex < DP_APP_MAX_LCORES; lcoreIndex++)
    {
        if ((0xff == coreInUsePacked[lcoreIndex].coreId) ||
            (coreInUsePacked[lcoreIndex].coreId == coreId))
            continue;

        printf("[%s()] Stopping core %d (index: %d)\n", __func__,
               coreInUsePacked[lcoreIndex].coreId, lcoreIndex);
        /* DP_APP_USE_DPDK_SERVICE_CORES related info */
        rdk_trace("rte_service_lcore_stop()\n");
        rte_service_lcore_stop(coreInUsePacked[lcoreIndex].coreId);
    }

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    rte_free(bridge_eth_rx_args);
    rte_free(bridge_eth_tx_args);
    rte_free(bridge_timer_args);
    rte_free(bridge_crypto_args);
    rte_free(bridge_cppi_args);
    rdk_trace("rte_eal_mp_wait_lcore()\n");
    rte_eal_mp_wait_lcore();

    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    err = rte_eal_cleanup();
    if (err) 
    {
        printf("Error from rte_eal_cleanup(), %d\n", err);
        return err;
    }
    /* Cleanup done automatically by kernel on app exit */
    return 0;
}

#include "adk_pal.h"

int adk_resource_init(unsigned num_ports)
{
    uint32_t port_id, i;
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    uint32_t evpcoreIndex, wccoreIndex;
    adk_sys_config_t adk_cfg_parms =
    {
        .adk_mode = ADK_SYS_CONFIG_MODE_SHARED_DATAPLANE,
        .pktio_mempool = NULL,
        .num_worker_lcores = 0,
        .worker_lcores = NULL
    };
    adk_config_handle_t sysConfigHandle;


    uint8_t eth_ports_list[DPDK_MAX_PORTS];

    adk_sys_resource_t              res[255];

    adk_sys_rsrc_evdev_info_t       evdev_info;
    adk_sys_rsrc_evp_info_t         evp_info[32];
    adk_sys_rsrc_evq_info_t         evq_info[8];

    adk_sys_rsrc_timer_dev_info_t   timer_dev_info;
    adk_sys_rsrc_wcore_info_t       wc_info;
    uint8_t                         wc_list[DP_APP_MAX_LCORES];
    adk_sys_rsrc_eth_dev_info_t     eth_dev_info;
    uint32_t                        evq_cfg_flags;

#ifdef ADK_DEVICE_SNR
    adk_sys_rsrc_crypto_dev_info_t	 inline_crypto_dev_info;
#endif
    adk_sys_rsrc_crypto_dev_info_t	 crypto_dev_info;
    adk_sys_rsrc_crypto_dev_qpair_info_t crypto_qp_info;

    memset(&evdev_info, 0x00, sizeof(evdev_info));
    memset(&evp_info, 0x00, sizeof(evp_info));
    memset(&evq_info, 0x00, sizeof(evq_info));
    memset(&timer_dev_info, 0x00, sizeof(timer_dev_info));
    memset(&wc_info, 0x00, sizeof(wc_info));
    memset(&crypto_dev_info, 0x00, sizeof(crypto_dev_info));
    memset(&crypto_qp_info, 0x00, sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));

    if (adk_sys_config(&adk_cfg_parms, &sysConfigHandle))
    {
        printf("Failed to config adk system\n");
        return 1;
    }
   
    num_ports=1;
    for (port_id = DPDK_BASE_PORT_ID; port_id < num_ports; port_id++)
    {
        eth_ports_list[port_id] = bh_eth_dev_id;
    }
    /* register event-dev resource with ADK */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    evpcoreIndex = 0;
    wccoreIndex = 0;
    for(i=0;i< DP_APP_MAX_LCORES; i++)
    {
        if (FUNC_WORKER == (coreInUsePacked[i].funcsOnCore & FUNC_WORKER))
        {
            evp_info[evpcoreIndex].ev_port_id  = worker_data[coreInUsePacked[i].coreId].event_port_id;
            evp_info[evpcoreIndex].lcore_id    = coreInUsePacked[i].coreId;
            evpcoreIndex++;
        }

        if (coreInUsePacked[i].funcsOnCore)
        {
            wc_list[wccoreIndex]              = coreInUsePacked[i].coreId;
            wccoreIndex++;
        }
    }

    evq_cfg_flags              = ADK_SYS_RSRC_EVQ_CFG_ALL_TYPES;

    for (i=0;i < num_worker_queues;i++)
    {
       evq_info[i].event_queue_id =  worker_qid[i];
       evq_info[i].mode           =  ADK_SYS_RSRC_MODE_DEDICATED;
       evq_info[i].evq_cfg_flags  =  evq_cfg_flags;
       evq_info[i].num_priorities =  8;
    }

    evdev_info.event_dev_id    =  eventdev_id;
    evdev_info.event_port_list_p = evp_info;
    evdev_info.num_ev_ports    = num_worker_cores;
    evdev_info.event_q_list_p  = evq_info;
    evdev_info.num_ev_queues   = num_worker_queues;

    /* Fill in the worker-core information */
    /* DP_APP_USE_DPDK_SERVICE_CORES related info */
    wc_info.num_worker_lcores   = num_service_cores;
    wc_info.worker_lcores       = wc_list;

    if (enableIPsec == TRUE)
    {
#ifdef ADK_DEVICE_SNR
        /* fill in the inline device details first
        */
        memset(&inline_crypto_dev_info, 0, sizeof(inline_crypto_dev_info));
        inline_crypto_dev_info.dev_type     = ADK_SYS_RSRC_CRYPTO_DEV_TYPE_INLINE;
        inline_crypto_dev_info.dev_id 	 = bh_eth_dev_id;
#endif

        /* Fill in the lookaside crypto-dev information */
        crypto_dev_info.dev_type     = ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA;
        crypto_dev_info.dev_id 	 = CRYPTO_DEV_ID;
        crypto_dev_info.num_q_pairs  = NUM_CRYPTO_QUEUE_PAIRS;
        crypto_dev_info.num_sas	 = MAX_NUM_IPSEC_SA;

        crypto_qp_info.queue_pair_id = CRYPTO_QP_ID;
        crypto_qp_info.mode		 = ADK_SYS_RSRC_MODE_DEDICATED;
        crypto_qp_info.usage		 = ADK_SYS_RSRC_CRYPTO_DEV_QP_USE_IPSEC_BOTH;
        crypto_qp_info.bd_ev_queue_id = bd_info[DP_APP_CRYPTO_BD].qid;

        crypto_dev_info.qp_info_p	 = &crypto_qp_info;
    }

    /* Fill in the timer-dev information */
    timer_dev_info.type         = ADK_SYS_RSRC_TIMER_DEV_TYPE_EVNT_TIMER_BD;
    timer_dev_info.timer_bd_instance_id = timer_inst_id;

    /* Fill in eth-dev information*/
    eth_dev_info.num_eth_devs   = num_ports;
    for(i=0;i<num_ports;i++) {
        eth_dev_info.eth_dev_list[i] = eth_ports_list[i];
    }
    eth_dev_info.event_queue_id = bd_info[DP_APP_ETH_TX_BD].qid;

    res[0].type = ADK_SYS_RSRC_TYPE_EVNT_DEV;
    res[0].res_info_p.evdev_info_p = &evdev_info;

    res[1].type = ADK_SYS_RSRC_TYPE_WCORE;
    res[1].res_info_p.wcore_list_info_p = &wc_info;

    res[2].type = ADK_SYS_RSRC_TYPE_TIMER_DEV;
    res[2].res_info_p.timer_dev_info_p = &timer_dev_info;

    res[3].type = ADK_SYS_RSRC_TYPE_PKT_MEMORY;
    res[3].res_info_p.pktio_mempool =  (void *)packet_pool;

    res[4].type = ADK_SYS_RSRC_TYPE_ETH_DEV;
    res[4].res_info_p.eth_dev_info_p =  &eth_dev_info;

    if (enableIPsec == TRUE)
    {
#ifdef ADK_DEVICE_SNR
        res[5].type = ADK_SYS_RSRC_TYPE_CRYPTO_DEV;
        res[5].res_info_p.crypto_dev_info_p = &inline_crypto_dev_info;
        res[6].type = ADK_SYS_RSRC_TYPE_CRYPTO_DEV;
        res[6].res_info_p.crypto_dev_info_p = &crypto_dev_info;
#else
        res[5].type = ADK_SYS_RSRC_TYPE_CRYPTO_DEV;
        res[5].res_info_p.crypto_dev_info_p = &crypto_dev_info;
#endif

#ifdef ADK_DEVICE_SNR
        if (adk_sys_resource_register(res,7) != ADK_SUCCESS)
        {
            printf("Failed to register resources with ADK\n");
            return 1;
        }
#else
        if (adk_sys_resource_register(res,6) != ADK_SUCCESS)
        {
            printf("Failed to register resources with ADK\n");
            return 1;
        }
#endif
    }
    else
    {
        if (adk_sys_resource_register(res,5) != ADK_SUCCESS)
        {
            printf("Failed to register resources with ADK\n");
            return 1;
        }
    }

#if 0
    /* Register devices before Sys DP init */
    timer_dev_info.type = ADK_SYS_TIMER_DEV_TYPE_EVNT_TIMER_BD;
    timer_dev_info.timer_bd_instance_id = timer_inst_id;
    if (adk_sys_timer_dev_register(&timer_dev_info))
    {
        printf("Failed to register timer device instance!\n");
        return 1;
    }
#endif
    printf("\n\nData plane infra setup complete!!\n");

    return 0;
}


adk_sys_handle_t init_adk_modules(void)
{
	adk_status_t ret_val = ADK_SUCCESS;
	adk_sys_handle_t sys_handle;
	adk_module_handle_t moduleHandle;
	adk_sys_conf_t sys_conf;
	adk_ph_dev_instance_config_t conf;
	uint8_t i = 0;

	memset(&sys_conf, 0x00, sizeof(sys_conf));
	sys_conf.chipInstance = 1;
	sys_conf.logLevel = ADK_SEV_LEVEL_INFO;
	sys_conf.multiThread = true;
	ret_val = adk_sys_init(&sys_conf, &sys_handle);
	if (ret_val != ADK_SUCCESS) {
		printf("Failed to init adk system\n");
		return 0;
	}
	printf("ADK initialization Done , sysHandle: %"PRIx64" \n", (long unsigned int)sys_handle);

	conf.cppiInstanceId = CPPI_INST_ID_BASE;
	if(adk_ph_dev_instance_create(sys_handle, &conf) != ADK_SUCCESS)
	{
		printf("ADK PH Dev create Failed\n");
		return 0;
	}
	for( i = 0; i < MAX_NO_OF_MODULES; i++ ) {
		if(ADK_SUCCESS == adk_module_register(sys_handle, (uint8_t *) module_name[i],
					(adk_module_init_func_t)func[i], &moduleHandle))
		{
			adk_module_attr_info_t g_log_attr = { ADK_MODULE_LOG_LEVEL, {ADK_SEV_LEVEL_INFO} };

			if (ADK_SUCCESS == adk_module_start(sys_handle, moduleHandle)) {
				adk_module_attr_set(sys_handle, moduleHandle, &g_log_attr);
			} else {
				printf("Failed to start module %s\n", module_name[i]);
				return 0;
			}
		}
		else
		{
			printf("%s module registration failed!\n",module_name[i]);
			return 0;
		}
	}

	return sys_handle;
}


int  adk_dp_init(adk_sys_handle_t sys_handle)
{

	if (adk_sys_dp_init())
	{
		printf("Failed to initialize ADK DP System!\n");
		return 1;
	}

	init_adk_slowpath_dev(sys_handle);

	if (adk_ethernet_dp_init())
	{
		printf("Failed to initialize Ethernet data plane component\n");
		return 1;
	}

	if (adk_ip_dp_init())
	{
		printf("Failed to initialize IPv4 data plane component\n");
		return 1;
	}

	if (adk_ipv6_dp_init())
	{
		printf("Failed to initialize IPv4 data plane component\n");
		return 1;
	}

	
	if (adk_ipsec_dp_init())
	{
		printf("Failed to initialize IPSec data plane component\n");
		return 1;
	}

	if (adk_ph_dev_dp_instance_init())
	{
		printf("Failed to initialize ADK DP PH Dev System!\n");
		return 0;
	}


	return 0;
}
#ifdef EM_SNF_ADK
int adk_non_em_common_pre_init(void *em_config , void *adk_config_xml, adk_xml_cfg_t *adkCfg_p)
{
	em_conf_t *emcfg = (em_conf_t *)em_config;
	cfg_arr_t ncfg = {adkCfg_p, ADKCFG_INDEX};
#if 0
	struct rte_event_dev_config eventdevConfig = { 0 };
	struct rte_event_dev_info eventdevInfo = { 0 };
	int eventdevId = -1;
	int retValRte;
#endif
	/**************/

	if (adk_config_xml) {
		if(0 != parse_config_file(adk_config_xml, &ncfg))
		{
			ADK_APPL_EXIT_FAILURE("NonEM Configuration file parsing failed\n");
		}
	}
	else
	{
		ADK_APPL_EXIT_FAILURE("ADK (nonEM) Configuration XML file not provided\n");
	}
	/************/
#define ADK_CURR_XML_VERSION 1
	if (adkCfg_p->xmlVersion < ADK_CURR_XML_VERSION)
	{
		ADK_APPL_EXIT_FAILURE("XML Parser is recent, XML passed is old. Please pass correct XML\n");
	}
#if 0
	/* Get event dev id from name and configure event dev */
	eventdevId = rte_event_dev_get_dev_id(adkCfg_p->eventdev.name);
	if (eventdevId < 0) {
		/* initialize event dev */
		if (rte_vdev_init(adkCfg_p->eventdev.name, NULL) < 0) {
			ADK_APPL_EXIT_FAILURE("Error creating eventdev\n");
		}
	}
	else {
		ADK_APPL_EXIT_FAILURE("Eventdev already created (may be already used by other entity..). Failurev\n");
	}
	eventdevId = rte_event_dev_get_dev_id(adkCfg_p->eventdev.name);
	if (eventdevId < 0) {
		ADK_APPL_EXIT_FAILURE("Failed to get eventdev id\n");
	}
	/* Get eventDev info */
	retValRte = rte_event_dev_info_get(eventdevId, &eventdevInfo);
	if (retValRte != 0) {
		ADK_APPL_EXIT_FAILURE("rte_event_dev_info_get() fails!");
	}

	/* zero flows means set to max */
	if (0 == adkCfg_p->eventdev.num_event_queue_flows)
		adkCfg_p->eventdev.num_event_queue_flows = eventdevInfo.max_event_queue_flows;
	if (0 == adkCfg_p->eventdev.event_port_dequeue_depth)
		adkCfg_p->eventdev.event_port_dequeue_depth = eventdevInfo.max_event_port_dequeue_depth;
	if (0 == adkCfg_p->eventdev.event_port_enqueue_depth)
		adkCfg_p->eventdev.event_port_enqueue_depth = eventdevInfo.max_event_port_enqueue_depth;

	/* Sanity of event dev configuration */
	if (adkCfg_p->eventdev.num_event_queue_flows > eventdevInfo.max_event_queue_flows ||
			adkCfg_p->eventdev.num_event_queues > eventdevInfo.max_event_queues ||
			adkCfg_p->eventdev.num_event_ports > eventdevInfo.max_event_ports ||
			adkCfg_p->eventdev.num_events_limit > (uint32_t) eventdevInfo.max_num_events ||
			adkCfg_p->eventdev.event_port_dequeue_depth > eventdevInfo.max_event_port_dequeue_depth ||
			adkCfg_p->eventdev.event_port_enqueue_depth > eventdevInfo.max_event_port_enqueue_depth) {
		ADK_APPL_EXIT_FAILURE("Maximum eventDev configuration exceeded!");
	}
	/* Set eventDev parameters */
	eventdevConfig.nb_event_queues = adkCfg_p->eventdev.num_event_queues;
	eventdevConfig.nb_event_ports = adkCfg_p->eventdev.num_event_ports;
	eventdevConfig.nb_events_limit = adkCfg_p->eventdev.num_events_limit;
	eventdevConfig.nb_event_queue_flows = adkCfg_p->eventdev.num_event_queue_flows;
	eventdevConfig.nb_event_port_dequeue_depth = adkCfg_p->eventdev.event_port_dequeue_depth;
	eventdevConfig.nb_event_port_enqueue_depth = adkCfg_p->eventdev.event_port_enqueue_depth;
	eventdevConfig.nb_single_link_event_ports = adkCfg_p->eventdev.single_link_event_port;
	eventdevConfig.dequeue_timeout_ns = 0;

	/* Configure eventDev */
	retValRte = rte_event_dev_configure(eventdevId, &eventdevConfig);

	if (retValRte) {
		ADK_APPL_EXIT_FAILURE("Failed to configure the event dev!");
	} else {
		printf("eventdev %d configured!\n", eventdevId);
	}
#endif
	/**********Set em config with non em attr to be used in em_init when pre_init returns ****/
	int i =0;

	emcfg->num_nonem = 1;
	/*ADK EvDev ID from XML*/
	emcfg->nonem_cfg[INDEX].nonem_eventdev_id = 1; /*eventdevId; */
	/*currently supporting 2 ADK entry point*/
	emcfg->nonem_cfg[INDEX].num_dest = NUM_QIDS_FROM_EM;

	for (i = 0; i < NUM_ADK_ENTRY_POINT; i++)
	{
		/* Em queue starts after worker core queues -
		 * * Qids Per worker or for DL
		 * * Qids from EM are for UL
		 * * For e.g. if there are 4 NUM_QIDS_PER_WORKER and 3 NUM_QIDS_FROM_EM then there will be 7 queues
		 * * All 7 will be linking to all worker cores.
		 * * Qid 0-3 will be for DL
		 * * Qid 4-6 will be for UL (from EM)
		 * */
		emcfg->nonem_cfg[INDEX].core_queue_map[i].virt_core_id = VIRTUAL_CORE_ID_FOR_EM_START + i; //virtual code start
		emcfg->nonem_cfg[INDEX].core_queue_map[i].eventdev_qid = i + 1;//queue_id for ADK entry point
	}
	emcfg->nonem_cfg[INDEX].num_port = emcfg->core_count;
	emcfg->nonem_cfg[INDEX].nonem_coremask._u64[0] = adkCfg_p->core_mask;
	printf("\nNonEM (ADK) Core mask 0x%lx\n", emcfg->nonem_cfg[INDEX].nonem_coremask._u64[0]);

	/************/
	return 0;
} /* common_pre_init */
#endif
