/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2017 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"

#include "ipt_nac_setup.h"


uint8_t ies_switch_state = 0;
char ipt_ies_error_name[IES_ERROR_MAX_NAME_SIZE];

void ies_event_handler_func_adk(const struct ies_event *ies_event)
{
    int i;
    char ies_event_name[IES_EVENT_MAX_NAME_SIZE];

    if (ies_event != NULL)
    {
        ies_trace("ies_event_get_name()\n");
        if (ies_event_get_name(ies_event->type, ies_event_name,
                               IES_EVENT_MAX_NAME_SIZE) == IES_OK)
            printf("IES_EVENT : %s\n", ies_event_name);
        else
            printf("IES_EVENT : Unknown\n");
        /**
        * Now handle the event itself...
        * Place holder for now with just print statements
        */
        switch (ies_event->type)
        {
        case IES_EVENT_ADDR_UPDATE:
        case IES_EVENT_ADDR_FLUSHED:
            for (i = 0; i < ies_event->info.addr_update.num_entries; i++)
            {
                printf("%s MAC Address 0x%06lx seen from port %d, with"
                       " VLAN %d in L2 domain %d\n",
                       ((ies_event->info.addr_update.entries[i].type ==
                         IES_EVENT_ADDR_LEARNED) &&
                        (ies_event->type == IES_EVENT_ADDR_UPDATE))?
                       "Learnt" : "Aged out",
                       ies_event->info.addr_update.entries[i].addr,
                       ies_event->info.addr_update.entries[i].port,
                       ies_event->info.addr_update.entries[i].vlan,
                       ies_event->info.addr_update.entries[i].l2_domain);
            }
            break;
        case IES_EVENT_PORT_STATE:
            printf("Port %d is %s\n", ies_event->info.port_state.port,
                   (ies_event->info.port_state.status ==
                    IES_PORT_STATUS_LINK_UP)? "UP" : "DOWN");
            break;
        case IES_EVENT_SWITCH_STATE:
            printf("Switch State is %s\n",
                   (ies_event->info.switch_state.status ==
                    IES_SWITCH_STATE_UP)? "UP" : "DOWN or HOLD");
            ies_switch_state = 1;
            break;
        default:
            break;
        }
        fflush(stdout);
    }
}


void print_switch_state(enum ies_switch_state ies_sw_state)
{
    switch (ies_sw_state)
    {
    case IES_SWITCH_STATE_DOWN:
        printf("Switch state is DOWN\n");
        break;
    case IES_SWITCH_STATE_HOLD:
        printf("Switch state is HOLD\n");
        break;
    case IES_SWITCH_STATE_UP:
        printf("Switch state is UP\n");
        break;
    default:
        printf("Invalid Switch state\n");
        break;
    }
}


/* This function is called by the master/slave process */
/* whether is it master/slave, number of ports and list of ports in array is
 * passed */
int setup_hlp_with_em(bool is_master,uint8_t numPorts,uint8_t *portId);
int setup_hlp_with_em(bool is_master,uint8_t numPorts,uint8_t *portId)
{
    int sw = E_IPT_SWITCH_ID;
    int ret;
    enum ies_switch_state ies_sw_state;
    bool use_shm = true;
    int shm_key = IES_SHM_KEY;
    uint32_t port, event_mask;
    uint64_t attr_val = 0;
    enum ies_port_attr get_attr = IES_PA_PHY_TYPE_ABILITY;
    enum ies_port_mode port_mode;
    enum ies_port_phy_type phy_type;

    /* Setup for mult-process application with shared memmory and make this
     * process as the master processes */
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHARED_MEMORY, &use_shm);
    if (ret != IES_OK)
    {
       printf("IES Failed to enable shared memory...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHM_KEY, &shm_key);
    if (ret != IES_OK)
    {
       printf("IES Failed to configure SHM Key...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_IS_MASTER, &is_master);
    if (ret != IES_OK)
    {
       printf("IES Failed to setup in master mode...errno = %d\n", ret);
       return 1;
    }

    /* Initialize IES */
    event_mask = IES_EVENT_SWITCH_STATE |
                 IES_EVENT_PORT_STATE |
                 IES_EVENT_ADDR_UPDATE |
                 IES_EVENT_ADDR_FLUSHED |
                 IES_EVENT_ERROR |
                 IES_EVENT_TIMEOUT;
    ies_trace("ies_initialize()\n");
    ret = ies_initialize(ies_event_handler_func_adk, event_mask);
    if (ret != IES_OK)
    {
        printf("%s:IES initialization failed: %d\n",__func__, ret);
        return 1;
    }

    do {
        usleep(100);
        /* Get switch state */
        ies_trace("ies_switch_get_state()\n");
        if (ies_switch_get_state(sw, &ies_sw_state) != IES_OK)
        {
            printf("Error in ies_switch_get_state\n");
            return 1;
        }
    } while (ies_sw_state != IES_SWITCH_STATE_UP);

    print_switch_state(ies_sw_state);
    printf("IES initialization complete\n");
    fflush(stdout);

    /* Now turn up all the ports */
    for (port = 0; port < numPorts; port++)
    {
        ies_trace("ies_port_get_mode()\n");
        if (ies_port_get_mode(sw, portId[port], &port_mode) != IES_OK)
        {
            printf("Error in ies_port_get_mode for port %d\n", portId[port]);
            return 1;
        }

        /* If port is not UP. Then set it to UP */
/* TO DO: Not available in latest RDK        if (port_mode != IES_PORT_MODE_UP) */
        if (port_mode != IES_PORT_MODE_ADMIN_UP)
        {
            /* Before that, set the phy attribute as needed */

            do {
                ies_trace("ies_port_get_attr()\n");
                ret = ies_port_get_attr(sw, portId[port], get_attr, &attr_val);
                if (ret != IES_OK) {
                    printf("Failed to get phy type ability for port = %d. Retrying ...\n",
                           portId[port]);
                } else {
                    printf("Phy type ability = %lx for port = %d\n",
                           attr_val, portId[port]);
                }
            } while (ret != IES_OK);

            if (((portId[port] >= 4) && (portId[port] < 8)) || ((portId[port] >= 12) && (portId[port] < 20)))
                phy_type = IES_PORT_PHY_TYPE_25GBASE_CR;
            else
                phy_type = IES_PORT_PHY_TYPE_10G_SFI;

            do {
                ies_trace("ies_port_set_attr()\n");
                ret = ies_port_set_attr(sw, portId[port], IES_PA_PHY_TYPE, &phy_type);
                if (ret != IES_OK) {
                    printf("Failed to set Phy type for port = %d. Retrying ...\n", portId[port]);
                } else {
                    printf("Set Phy type for port = %d\n", portId[port]);
                }
            } while (ret != IES_OK);

            port_mode = IES_PORT_MODE_ADMIN_UP;
            do {
                ies_trace("ies_port_set_mode()\n");
                ret = ies_port_set_mode(sw, portId[port], port_mode);
                if (ret != IES_OK) {
                    printf("Failed to set port mode to UP for port = %d\n", portId[port]);
                } else {
                    printf("Set port mode to UP for port = %d\n", portId[port]);
                }
            } while (ret != IES_OK);
        } else {
            printf("port_mode already UP for port %d\n", portId[port]);
        }
    }

    printf("\nAll HLP ports are now UP!\n");

    return 0;
}


int e_ipt_lan_tx_trig_config(void)
{
    int ies_ret;
    int port_set;
    int group = E_IPT_TRIG_GROUP_ID;
    int rule  = 0;
    struct ies_trigger_cond cond;
    struct ies_trigger_action action;

    memset(&cond, 0, sizeof(cond));
    memset(&action, 0, sizeof(action));

   /* create port set */
    ies_trace("ies_portset_create()\n");
    ies_ret = ies_portset_create(E_IPT_SWITCH_ID, &port_set);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to create port \n", 
               IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
   /* Add cpk port to the port set */
    ies_trace("ies_portset_add_port()\n");
    ies_ret = ies_portset_add_port(E_IPT_SWITCH_ID, port_set, E_IPT_CPK_PORT);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add cpk port to portset \n", 
               IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }

   /* create a trigger
    * If (src_port == CPK-port && MD.Type == LAN_TX && MD.ESN[31] == 1)
    * */
    cond.mask = IES_TRIGGER_COND_MATCH_METADATA1;
    cond.cfg.rx_portset  = port_set;
    cond.cfg.match_metadata0   = IES_TRIGGER_MATCHCASE_MATCHIFEQUAL;
    cond.cfg.match_metadata1   = IES_TRIGGER_MATCHCASE_MATCHIFEQUAL;
    cond.param.metadata[0].source   = IES_TRIGGER_METADATA_SOURCE_INPUT;
    cond.param.metadata[0].mask     = 0xFF;
    cond.param.metadata[0].value    = 0x1; /* LAN_TX */
    cond.param.metadata[0].select   = 0; /* 0th byte : ONPI-TYPE*/

    cond.param.metadata[1].source   = IES_TRIGGER_METADATA_SOURCE_INPUT;
    cond.param.metadata[1].mask     = 0x80; /* MSb of the ESN word */
    cond.param.metadata[1].value    = 0x80; 
    cond.param.metadata[1].select   = 31;

    /* Trigger metadat update action to set the ONPI-TYPE to DSI-TX */
    action.mask = IES_TRIGGER_ACTION_METADATA | IES_TRIGGER_ACTION_FORWARDING;
    action.param.forwarding_action.action= IES_TRIGGER_FORWARD_ACTION_REDIRECT_PORT;
    action.param.forwarding_action.arg.red_port.new_dest_logical_port= E_IPT_CPM_EGRESS_PORT;

    action.param.metadata.action    = IES_TRIGGER_MASKED_16b_SET;
    action.param.metadata.value     = 0x16; /* DSI-TX */
    action.param.metadata.slot      = 3;

    ies_trace("ies_trigger_add()\n");
    ies_ret = ies_trigger_add(E_IPT_SWITCH_ID, group, rule, 
                              &cond, &action);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add trigger rule %d \n", 
               IPT_IES_ERROR_NAME(ies_ret), rule);
        return -1;
    }
    
    /* Trigger metadat update action to copy TC from LAN-TX format to DSI-TX format */
    action.param.metadata.action    = IES_TRIGGER_MASKED_16b_COPY;
    action.param.metadata.mask      = 0x03FF;
    action.param.metadata.source    = 28;
    action.param.metadata.offset    = 4;
    action.param.metadata.slot      = 2;
    rule = rule +1;
    ies_trace("ies_trigger_add()\n");
    ies_ret = ies_trigger_add(E_IPT_SWITCH_ID, group, rule, 
                              &cond, &action);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add trigger rule %d \n", 
               IPT_IES_ERROR_NAME(ies_ret), rule);
        return -1;
    }

    /* Trigger metadat update action to copy L2-DOMAIN from LAN-TX format to DSI-TX format */
    action.param.metadata.action    = IES_TRIGGER_MASKED_16b_COPY;
    action.param.metadata.mask      = 0x1FFF;
    action.param.metadata.source    = 30;
    action.param.metadata.offset    = 20;
    action.param.metadata.slot      = 1;

    rule = rule +1;
    ies_trace("ies_trigger_add()\n");
    ies_ret = ies_trigger_add(E_IPT_SWITCH_ID, group, rule, 
                              &cond, &action);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add trigger rule %d \n", 
               IPT_IES_ERROR_NAME(ies_ret), rule);
        return -1;
    }
    return 0;
}

int e_ipt_pri_map(void)
{
    int ies_ret;
    union ies_qos_pri_map_value value;
    int i;

    memset(&value, 0, sizeof(value));

   /* Set pri-map table as vlan-pri*/
    value.rx_vpri.profile = E_IPT_IES_PRI_PROFILE;
    for(i=0;i<IES_QOS_NUM_TC; i++)
    {
        value.rx_vpri.tc[2*i]           = 7 - i;
        value.rx_vpri.int_vpri[2*i]     = i*2;
        value.rx_vpri.tc[2*i + 1]       = 7 - i;
        value.rx_vpri.int_vpri[2*i + 1] = i*2 + 1;

    }
    ies_trace("ies_qos_set_pri_map()\n");
    ies_ret = ies_qos_set_pri_map(E_IPT_SWITCH_ID, IES_QOS_PRI_MAP_RX_VPRI,
                                   &value);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set priority map" ,
               IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
    return 0;

}

int e_ipt_hlp_port_config(int port_id)
{
    int ies_ret;
    bool replace_vlan= false;
    bool di_enable= true;
    struct ies_port_hash_profile port_hash_profile;

   /* Reset all port stats */
    ies_trace("ies_stats_reset()\n");
    ies_ret = ies_stats_reset(E_IPT_SWITCH_ID, (int *) &port_id, 1);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to reset port stats for HLP"
                      " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret), port_id);
        return -1;
    }

    ies_trace("ies_port_set_attr()\n");
    ies_ret = ies_port_set_attr(E_IPT_SWITCH_ID, port_id, IES_PA_REPLACE_VLAN_FIELDS, &replace_vlan );
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set the port VLAN replace flag"
                      " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret), port_id);
        return -1;
    }

    ies_trace("ies_port_set_attr()\n");
    ies_ret = ies_port_set_attr(E_IPT_SWITCH_ID, port_id, IES_PA_PARSER_DI_ENABLE, &di_enable);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set the port VLAN replace flag"
                      " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret), port_id);
        return -1;
    }

    port_hash_profile.profile = E_IPT_BB_CONFIG_HASH_PROFILE_ID;

    port_hash_profile.metadata = 1;
    ies_trace("ies_port_set_attr()\n");
    ies_ret = ies_port_set_attr(E_IPT_SWITCH_ID, port_id, IES_PA_DEF_HASH_PROFILE, &port_hash_profile);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set the port VLAN replace flag"
                      " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret), port_id);
        return -1;
    }

    /*CPK port */
    port_id = E_IPT_CPK_PORT;

    ies_trace("ies_port_set_attr()\n");
    ies_ret = ies_port_set_attr(E_IPT_SWITCH_ID, port_id, IES_PA_REPLACE_VLAN_FIELDS, &replace_vlan );
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set the port VLAN replace flag"
                      " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret), port_id);
        return -1;
    }


    return 0;
}



int e_ipt_domain_add(int hlp_port, int vlan, int vsi)
{

    uint32_t ies_ret;
    struct ies_domain_cond domain_cond;
    struct ies_domain_param domain_param;
    int32_t dglort;
    int precedence = E_IPT_IES_DOMAIN_MAPPER_PREC_1;
    int domain_id;
    RTE_SET_USED(vlan);

    ies_trace("ies_vsi_map_vsi_qid_to_port()\n");
    ies_ret = ies_vsi_map_vsi_qid_to_port(E_IPT_SWITCH_ID, vsi,
                                          hlp_port, &dglort);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to get VSI Queue mapping for HLP"
               " port-%d!\n", IPT_IES_ERROR_NAME(ies_ret),
                hlp_port);
        return -1;
    }

    memset(&domain_cond, 0, sizeof(domain_cond));
    memset(&domain_param, 0, sizeof(domain_param));

    domain_cond.port_value = hlp_port;
    domain_cond.port_mask = 0x1F;
    domain_cond.tag_type = IES_DOMAIN_VLAN_TAG_TYPE_VLAN1;
    domain_cond.vlan1_value = vlan;
    domain_cond.vlan1_mask = 0xFFF;

    domain_param.enable = true;
    domain_param.learning = false; 
    domain_param.independent_learning = false;
    domain_param.update_domain = true;
    domain_param.l2d = E_IPT_L2_DOMAIN;
    domain_param.scen_id = E_IPT_L2D_SCENARIO_ID;
    domain_param.tc_sel[0] = IES_DOMAIN_TC_SRC_VPRI;
    domain_param.priority_profile = E_IPT_IES_PRI_PROFILE;

    ies_trace("ies_domain_add_entry()\n");
    ies_ret = ies_domain_add_entry(E_IPT_SWITCH_ID, &domain_cond, 
                                   &domain_param, precedence, &domain_id);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add domain entry\n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }

    return 0;
}

int e_ipt_scenario_add(int hlp_port, int vlan)
{

    uint32_t ies_ret;
    int32_t dglort;
#if 0
    union ies_mapper_value value;
    int entry_id;
    uint64_t   scenario_id;
    struct ies_acl_remap_action remap_cfg;
    RTE_SET_USED(remap_cfg);
#endif
    struct ies_acl_rule_info acl_rule;
    struct ies_hash_profile profile;

    RTE_SET_USED(vlan);
    RTE_SET_USED(acl_rule);
    RTE_SET_USED(dglort);
    RTE_SET_USED(hlp_port);

#if 0
    ies_ret = ies_acl_create_scenario_id(E_IPT_SWITCH_ID, &scenario_id);

    printf("created scenario id 0x%"PRIx64"\n",scenario_id);

    memset(&value, 0, sizeof(value));

    value.scenario.cond_mask        = IES_MAP_SCEN_COND_L2D;
    value.scenario.cond.l2d         = E_IPT_L2_DOMAIN;
    value.scenario.cond.l2d_mask    = 0X1FF;

    value.scenario.act.acl_scen_en = true;
    value.scenario.act.acl_scen    = scenario_id;
    value.scenario.act.trig_id_en  = false;
    value.scenario.act.trig_parser_err = true;

    value.scenario.index            = E_IPT_SCENARIO;
    value.scenario.src_scen =  IES_MAP_SCEN_IDX_INGRS_NON_IP ;

    ies_trace("ies_mapper_add_entry()\n");
    ies_ret = ies_mapper_add_entry (E_IPT_SWITCH_ID, IES_MAPPER_SCENARIO, &value, &entry_id);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add scenario entry\n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
    printf("scenario entry id 0x%"PRIx32"\n",entry_id);

    value.scenario.index            = IES_MAP_SCEN_IDX_EGRS_DSI + 1;
    value.scenario.src_scen =  IES_MAP_SCEN_IDX_EGRS_DSI ;

    ies_trace("ies_mapper_add_entry()\n");
    ies_ret = ies_mapper_add_entry (E_IPT_SWITCH_ID, IES_MAPPER_SCENARIO, &value, &entry_id);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add scenario entry\n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
    printf("scenario entry id 0x%"PRIx32"\n",entry_id);
#endif

    memset(&profile, 0, sizeof(profile));

    profile.mpls.di.deep_inspection  =  true;
    profile.mpls.di.type             = IES_HASH_DI_TYPE_L2;

    /*each bit controls corresponding bytes 0:63  */
    profile.mpls.di.di_vector_lo     = E_IPT_BB_CONFIG_PCEP_HLP_DI_BYTE_BIT_MASK;

    /*each bit controls corresponding bytes 64:127  */
    profile.mpls.di.di_vector_hi     = 0x00;
    profile.mpls.mdata_mask          = 0x00;
    
    ies_trace("ies_hash_set_profile()\n");
    ies_ret = ies_hash_set_profile(E_IPT_SWITCH_ID,
                    E_IPT_BB_CONFIG_HASH_PROFILE_ID,
                    &profile,
                    IES_HASH_PROF_MPLS_HASH_KEYS);	
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to set HASH profile for BB_CONFIG \n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
    return 0;
}



int e_ipt_BB_CONFIG_terminate_dmac(uint64_t dmac, int vlan, int vsi_id, int vsi_q)
{
    int ies_ret;
    int lport;
    RTE_SET_USED(vlan);

    ies_trace("ies_vsi_map_vsi_qid_to_port()\n");
    ies_ret = ies_vsi_map_vsi_qid_to_port(E_IPT_SWITCH_ID,
                                          vsi_id,
                                          vsi_q, &lport);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to get lport for a VSI & rx-queue \n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }

    
    printf("\n DGLORT / LPORT id for VSI[%d]/RxQ[%d] is %d\n", vsi_id, vsi_q, lport);

    e_ipt_l2fdb_add(E_IPT_L2_DOMAIN, 0, dmac, lport);

    return 0;

}

int e_ipt_l2fdb_add(uint32_t l2d, 
                    uint32_t vlan, 
                    uint64_t dmac,
                    int32_t  lport)
{

    uint32_t ies_ret;

    struct ies_addr_req_entry entry;

    RTE_SET_USED(vlan);
    memset(&entry, 0x0, sizeof(entry));
    entry.mac_addr_entry.mac_addr = dmac ;
    entry.mac_addr_entry.vlan = vlan;
    entry.mac_addr_entry.l2_domain = l2d;
    entry.mac_addr_entry.type = IES_ADDR_STATIC;
    entry.mac_addr_entry.port = lport;

    ies_trace("ies_addr_add()\n");
    ies_ret = ies_addr_add(E_IPT_SWITCH_ID, 1, &entry, 0);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add DMAC to L2FDB \n",
                      IPT_IES_ERROR_NAME(ies_ret));
        return -1;
    }
    return 0;
}

int e_ipt_vlan_create (
    uint16_t vlan_id)
{

    uint32_t ies_ret;
    int hlp_port  = 0;

    ies_trace("ies_vlan_create()\n");
    ies_ret = ies_vlan_create(E_IPT_SWITCH_ID, vlan_id);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to create VLAN ID %d\n",
                      IPT_IES_ERROR_NAME(ies_ret), vlan_id);
        return -1;
    }

    for(hlp_port = 0 ; hlp_port<=20;hlp_port++) {
    ies_trace("ies_vlan_add_port()\n");
    ies_ret = ies_vlan_add_port(E_IPT_SWITCH_ID, vlan_id, hlp_port,
                                false);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add CPK HLP port[%d] as member"
                      " to VLAN %d\n", IPT_IES_ERROR_NAME(ies_ret), hlp_port, vlan_id);
        return -1;
    }
    }

    return 0;
}


int e_ipt_vlan_port_add (
    uint16_t vlan_id,
    uint32_t port)
{
    uint32_t ies_ret;

    ies_trace("ies_vlan_add_port()\n");
    ies_ret = ies_vlan_add_port(E_IPT_SWITCH_ID, vlan_id, port, false);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add HLP port-%d as member to"
                      " VLAN %d\n", IPT_IES_ERROR_NAME(ies_ret),
                      port, vlan_id);
        return -1;
    }

    return 0;
}


int terminate_hlp(void)
{
    ies_trace("ies_terminate()\n");
    if (ies_terminate(1) != IES_OK)
    {
        printf("Error in forced ies_terminate!\n");
        return 1;
    }

    return 0;
}

/* This function is called only by the secondary IPT process */
int e_ipt_setup_hlp(void)
{
    int ret;

    bool use_shm = true, is_master = false;
    int shm_key = IES_SHM_KEY;  /* This must be same as used by ADK library */
    uint32_t event_mask;

    /* Setup for mult-process application with shared memory and make this
     * process as the secondary process */
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHARED_MEMORY, &use_shm);
    if (ret != IES_OK)
    {
       printf("IES Failed to enable shared memory...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHM_KEY, &shm_key);
    if (ret != IES_OK)
    {
       printf("IES Failed to configure SHM Key...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_IS_MASTER, &is_master);
    if (ret != IES_OK)
    {
       printf("IES Failed to setup in master mode...errno = %d\n", ret);
       return 1;
    }

    /* Initialize IES */
    event_mask = IES_EVENT_ADDR_UPDATE |
                 IES_EVENT_ADDR_FLUSHED ;
    ies_trace("ies_initialize()\n");
    ret = ies_initialize(ies_event_handler_func_adk, event_mask);
    if (ret != IES_OK)
    {
        printf("IES initialization failed: %d\n", ret);
        return 1;
    }

    return 0;
}
