#if 0
###/* **************************************************************************
###**                                                                          *
###** INTEL CONFIDENTIAL                                                       *
###** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
###**                                                                          *
###** The source code contained or described herein and all documents related  *
###** to the source code ("Material") are owned by Intel Corporation or its    *
###** suppliers or licensors. Title to the Material remains with Intel         *
###** Corporation or its suppliers and licensors. The Material contains trade  *
###** secrets and proprietary and confidential information of Intel or its     *
###** suppliers and licensors. The Material is protected by worldwide          *
###** copyright and trade secret laws and treaty provisions. No part of the    *
###** Material may be used, copied, reproduced, modified, published, uploaded, *
###** posted, transmitted, distributed, or disclosed in any way without        *
###** Intel's prior express written permission.                                *
###**                                                                          *
###** No license under any patent, copyright, trade secret or other            *
###** intellectual property right is granted to or conferred upon you by       *
###** disclosure or delivery of the Materials, either expressly, by            *
###** implication, inducement, estoppel or otherwise. Any license under such   *
###** intellectual property rights must be express and approved by Intel       *
###** in writing.                                                              *
###**                                                                          *
###** **************************************************************************
###**/
###/*!
### * @file  adk_sample_alarm.c
### *
### *      $HeadURL$
### *      $Revision$
### *      $Date$
### * @addtogroup _adkSamples ADK Sample Applications
### * @{
### *
### * @brief Supporting alarms for ADK sample tests.
### *
### * This file contains all alarm thread entry function. It makes use of private
### * ncp register read and write APIs in the file to check the MPIC interrupt
### * status, to report current raised alarms and to clear existing alarms.
### *
### ***********************************************************************/
####include <stdio.h>
####include <stdlib.h>
####include <string.h>
####include <unistd.h>
####include <adk_sys.h>
####include <getopt.h>
####include <string.h>
####include <malloc.h>
####include "adk_sample_alarm.h"
###
###ncp_hdl_t __alarmConfig;
###
###static ncp_st_t adk_ncp_reg_write(ncp_dev_hdl_t devHdl,
###                                  ncp_region_id_t regionId,
###                                  uint64_t offset,
###                                  uint32_t *val);
###
###static ncp_st_t adk_ncp_reg_read(ncp_dev_hdl_t devHdl,
###                                 ncp_region_id_t regionId,
###                                 uint64_t offset,
###                                 uint32_t *val);
###
###static ncp_st_t adk_ncp_reg_mpic_reenable(ncp_dev_hdl_t devHdl);
###
###static ncp_st_t adk_ncp_reg_write(ncp_dev_hdl_t devHdl,
###                                  ncp_region_id_t regionId,
###                                  uint64_t offset,
###                                  uint32_t *val)
###{
###    ncp_st_t ncpStatus = NCP_ST_SUCCESS;
###    ncpStatus = ncp_dev_block_write32(
###        devHdl,
###        regionId,
###        offset,
###        val,
###        1, /* xferCount */
###        0 /* ioflags */);
###    return ncpStatus;
###}
###
###static ncp_st_t adk_ncp_reg_read(ncp_dev_hdl_t devHdl,
###                                 ncp_region_id_t regionId,
###                                 uint64_t offset,
###                                 uint32_t *val)
###{
###    ncp_st_t ncpStatus = NCP_ST_SUCCESS;
###    ncpStatus = ncp_dev_block_read32(
###        devHdl,
###        regionId,
###        offset,
###        val,
###        1 /* count */,
###        0 /* flags */);
###    return ncpStatus;
###}
###
###static ncp_st_t adk_ncp_reg_mpic_reenable(ncp_dev_hdl_t devHdl)
###{
###    uint16_t         node;
###    uint16_t         target;
###    ncp_region_id_t      regionId;
###    uint64_t         offset;
###    ncp_st_t             ncpStatus = NCP_ST_SUCCESS;
###    uint32_t         mpic_val;
###
###    /* this is the co-ordinates of the MPIC vector priority register */
###    node = 0x193;
###    target = 0x0;
###    regionId = NCP_REGION_ID(node, target);
###    offset = 0x41A00;
###
###    /* read from MPIC vector/priority register */
###    ncpStatus = adk_ncp_reg_read(devHdl, regionId, offset, &mpic_val);
###    if(ncpStatus != NCP_ST_SUCCESS){
###        fprintf(stdout, "Err: read from MPIC register failed...\n");
###        return ncpStatus;
###    }
###
###    /* check the mask bit, if it is set we need to clear it, otherwise no worries */
###    if(mpic_val & ADK_ALARM_NCP_REG_MPIC_MSK_BIT){
###        /* make sure the activity bit is 0 */
###        while (mpic_val & ADK_ALARM_NCP_REG_MPIC_ACT_BIT){
###        }
###        /* activity bit is 0 so we can clear the msk bit */
###        mpic_val &= ~ADK_ALARM_NCP_REG_MPIC_MSK_BIT;
###
###        /* write back to mpic register */
###        ncpStatus = adk_ncp_reg_write(devHdl, regionId, offset, &mpic_val);
###        if(ncpStatus != NCP_ST_SUCCESS){
###            fprintf(stdout, "Err: write to MPIC register failed...\n");
###        return ncpStatus;
###        }
###    }
###    return ncpStatus;
###}
###
###/*!
### * =============================================================
### * @internal
### * adk_alarm_event_report
### *
### * @param[in]   ncp_hdl_t           accesses the global variable
### * @param[out]  none                none
### *
### * @brief  Alarm event reporting function.
### *
### * This API is used by the alarm thread entry function. It runs
### * in blocked mode and reports alarm event at the terminal only
### * when there is an alarm event that occurs in the system.
### * =============================================================
### */
###void adk_alarm_event_report(void)
###{
###    ncp_st_t ncpStatus = NCP_ST_SUCCESS;
###    ncp_alarm_source_t  acp_alarm;
###    bool  blocking;
###    ncp_hdl_t ncpHdl;
###    ncp_dev_hdl_t devHdl;
###
###    acp_alarm.irq = 0;
###    acp_alarm.regionId = 0;
###    acp_alarm.offset = 0;
###    acp_alarm.value = 0;
###
###    /* we want to wait to see an alarm */
###    blocking = TRUE;
###    ncpHdl = __alarmConfig;
###
###    /* get the device handle */
###    ncpStatus = ncp_config_dev_hdl_get(ncpHdl, &devHdl);
###    if(ncpStatus != NCP_ST_SUCCESS){
###        fprintf(stdout, "Err: ACP alarm clear status register, devHdl failed.\n");
###    }
###
###    /* make sure that MPIC is ready and interrupts are enabled */
###    adk_ncp_reg_mpic_reenable(devHdl);
###
###    /* get and report alarm, block if there is none */
###    ncpStatus = ncp_alarm_get(ncpHdl, blocking, &acp_alarm);
###    if (ncpStatus == NCP_ST_SUCCESS)
###    {
###        printf("Current ACP device alarms \n");
###        printf("   IRQ |  source register       | value\n");
###        printf(" ------+------------------------+-----------\n");
###    }
###    else
###    {
###        ncp_status_print("ncp_alarm_get", ncpStatus);
###    }
###
###    while (ncpStatus == NCP_ST_SUCCESS)
###    {
###        printf("%6d | ", acp_alarm.irq);
###
###        if (acp_alarm.regionId == 0xffffffff)
###        {
###            printf("   (unknown)           | \n");
###        }
###        else
###        {
###            printf(" 0x%03x.0x%02x.0x%08x | 0x%08x\n",
###                NCP_NODE_ID(acp_alarm.regionId),
###                NCP_TARGET_ID(acp_alarm.regionId),
###                acp_alarm.offset,
###                acp_alarm.value);
###        }
###
###        /* clear the corresponding interrupt status register */
###        ncpStatus = adk_ncp_reg_write(devHdl, acp_alarm.regionId, acp_alarm.offset, &acp_alarm.value);
###        if(ncpStatus != NCP_ST_SUCCESS){
###            fprintf(stdout, "Err: write to interrupt status register failed...\n");
###        }
###
###        /* reenable mpic */
###        ncpStatus = adk_ncp_reg_mpic_reenable(devHdl);
###        if(ncpStatus != NCP_ST_SUCCESS){
###            fprintf(stdout, "Err: reenabling MPIC register failed...\n");
###        }
###
###        /* block to get yet another alarm and continue */
###        ncpStatus = ncp_alarm_get(ncpHdl, blocking, &acp_alarm);
###    }
###    fprintf(stdout, "Exiting alarm thread...\n");
###}
###
###void *adk_alarm_linux_thread_entry(void __attribute__((unused))*data)
###{
###    adk_alarm_event_report();
###    ADK_SAMPLE_ALARM_THREAD_EXIT();
###}
###
###/*! end addtogroup _adkSamples
### * @}
### */
#endif
