/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include <stdint.h>
#ifdef ADK_NETD_NETLINK
#include <linux/if.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <linux/if_ether.h>
#include "linux/adk_netd_gen_comm.h"
#endif
#include <rte_config.h>
#include <rte_eal.h>
#include <rte_version.h>
#include <rte_mempool.h>
#include <rte_mbuf.h>
#include <rte_launch.h>
#include <rte_malloc.h>
#include <rte_random.h>
#include <rte_cycles.h>
#include <rte_ethdev.h>
#include <rte_eventdev.h>
#include <rte_event_framework.h>
#include <rte_bridge_eth.h>
#include <rte_bridge_timer.h>
#include <rte_bridge_cppi.h>
#include <rte_bridge_crypto.h>

#if ADK_DPDK_VERSION >= 18050016
#include <rte_bus_vdev.h>
#endif

#include <ies_sdk.h>
#include <ies_event_types.h>
#include <ies_api_port.h>

#include <adk_sys_types.h>
#include <adk_error.h>
#include "adk_hos.h"
#include "adk_hos_constants.h"

#include "adk_test_shm.h"
#include "test_shm.h"

#include "adk_sample_common.h"

#ifndef debug_printf
#  ifdef ADK_DEBUG
#define debug_printf(fmt, args...) printf(fmt, ##args)
#  else
#define debug_printf(fmt, args...)
#  endif
#endif

#define IES_ERROR_NAME(ies_ret) \
    ((ies_error_get_name(ies_ret, ies_error_name, \
                         IES_ERROR_MAX_NAME_SIZE) == IES_OK) ? \
     ies_error_name : "HOS_UNKNOWN")

#define PORT_WIDTH      5
#define PORT_MASK       0x1f
#define VLAN_ID_WIDTH   12
#define VLAN_ID_MASK    0xfff

/*QoS Definitions*/

uint16_t g_num_fl_nodes = 0;
uint16_t g_num_sl_nodes = 0;

uint32_t g_vsi_num_tx_queues = 0;
uint32_t g_vsi_num_max_tx_queues = 0;
uint32_t g_shaper_profile_count = 0;

qos_fl_sched_t g_fl_map[MAX_L3_INTF];
qos_sl_sched_t g_sl_map[MAX_L2_INTF];
shaper_profile_t g_shaper_profiles_tbl[MAX_SHAPER_PROFILES];


adk_status_t
domain_mapper_tbl_rule_add(
    domain_mapper_tbl_rule_attr_t *rule_attr,
    int *domain_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_domain_cond domain_cond;
    struct ies_domain_param domain_param;
    int prec = PORT_WIDTH;

    memset(&domain_cond, 0, sizeof(domain_cond));
    memset(&domain_param, 0, sizeof(domain_param));
    domain_cond.port_value = rule_attr->port;
    domain_cond.port_mask = PORT_MASK;
    if (rule_attr->vlan_id)
    {
        domain_cond.tag_type = IES_DOMAIN_VLAN_TAG_TYPE_ONLY_VLAN1;
        if (rule_attr->vlan_id != ADK_HOS_ANY_VLAN)
        {
            domain_cond.vlan1_value = rule_attr->vlan_id;
            domain_cond.vlan1_mask = VLAN_ID_MASK;
            prec += VLAN_ID_WIDTH;
        }
    }
    else
    {
        domain_cond.tag_type = IES_DOMAIN_VLAN_TAG_TYPE_NONE;
    }
    domain_param.enable = true;
    domain_param.learning = rule_attr->enable_mac_learning;
    domain_param.independent_learning = rule_attr->is_ivl;
    domain_param.update_domain = true;
    /* TODO: This needs to be taken care off */
    domain_param.priority_profile = ADK_HOS_DOM_PRI_MAP_PROFILE_ID;
    domain_param.tc_sel[0] = IES_DOMAIN_TC_SRC_VPRI;
    domain_param.l2d = rule_attr->eth_sw_domain;
    domain_param.l3d = rule_attr->vrd_id;
    domain_param.scen_id = 0;

    printf("ies_domain_add_entry port = %d, vlan = %d,"
                        " l2-domain = %d, vrf = %d \n",
                        rule_attr->port, rule_attr->vlan_id,
                        rule_attr->eth_sw_domain, rule_attr->vrd_id);
    ies_ret = ies_domain_add_entry(ADK_HLP_SWITCH_ID, &domain_cond, 
                                   &domain_param, prec, (int *) domain_id);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add domain entry\n",
                      IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto domain_mapper_tbl_rule_add_exit;
    }

domain_mapper_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
domain_mapper_tbl_rule_delete(int domain_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    printf("ies_domain_delete_entry rule ID = %d\n",
                        domain_id);

    ies_ret = ies_domain_delete_entry(ADK_HLP_SWITCH_ID, domain_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        printf("IES Error %s, Failed to delete domain entry ID-%d\n",
                      IES_ERROR_NAME(ies_ret),
                      (int) domain_id);
    }

    return retValue;
}

adk_status_t
config_mac_aging (
    uint32_t port,
    uint32_t age_out_time)  /* In millisecs */
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    enum ies_port_attr attr;

    attr = IES_PA_ADDRESS_AGING_TIME;
    printf("ies_port_set_attr is called with MAC Aging time %d millisecs"
                        " for HLP port %d\n", age_out_time, port);
    ies_ret = ies_port_set_attr(ADK_HLP_SWITCH_ID, port, attr, &age_out_time);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to configure MAC age out time for"
                      " port-%d\n", IES_ERROR_NAME(ies_ret), port);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto config_mac_aging_exit;
    }

config_mac_aging_exit:

    return retValue;
}

adk_status_t
ies_vlan_create_helper (
    uint16_t vlan_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    printf("ies_vlan_create VLAN ID %d\n", vlan_id);
    ies_ret = ies_vlan_create(ADK_HLP_SWITCH_ID, vlan_id);
    if (ies_ret == IES_ERR_EXISTS)
    {
        /* TODO: This would happen when same VLAN is used to create as routed
         * VLAN on a different sub-interface. This needs to be analyzed */
        printf("Warning: VLAN ID %d already exists.\n", vlan_id);
    }
    else if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to create VLAN ID %d\n",
                      IES_ERROR_NAME(ies_ret), vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_create_helper_exit;
    }

    printf("ies_vlan_add_port VLAN ID %d, port %d\n",
                        vlan_id, CPK_HLP_PORT);
    ies_ret = ies_vlan_add_port(ADK_HLP_SWITCH_ID, vlan_id, CPK_HLP_PORT,
                                false);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add CPK HLP port as member"
                      " to VLAN %d\n", IES_ERROR_NAME(ies_ret), vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_create_helper_exit;
    }

    printf("Created VLAN ID %d in HLP\n", vlan_id);

_ies_vlan_create_helper_exit:

    return retValue;
}

adk_status_t
ies_vlan_delete_helper (
    uint16_t vlan_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    printf("ies_vlan_delete_port VLAN ID %d, port %d\n",
                        vlan_id, CPK_HLP_PORT);
    ies_ret = ies_vlan_delete_port(ADK_HLP_SWITCH_ID, vlan_id,
                                   CPK_HLP_PORT);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to remove CPK HLP port as member of"
                      " VLAN %d\n", IES_ERROR_NAME(ies_ret), vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_delete_helper_exit;
    }

    printf("ies_vlan_destroy VLAN ID %d\n", vlan_id);
    ies_ret = ies_vlan_destroy(ADK_HLP_SWITCH_ID, vlan_id, false);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to destroy VLAN ID %d\n",
                      IES_ERROR_NAME(ies_ret), vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_delete_helper_exit;
    }

    printf("Deleted VLAN ID %d from HLP\n", vlan_id);

_ies_vlan_delete_helper_exit:

    return retValue;
}

adk_status_t
ies_vlan_port_add_helper (
    uint16_t vlan_id,
    uint32_t port)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    printf("ies_vlan_add_port VLAN ID %d, port %d\n",
                        vlan_id, port);
    ies_ret = ies_vlan_add_port(ADK_HLP_SWITCH_ID, vlan_id, port, false);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to add HLP port-%d as member to"
                      " VLAN %d\n", IES_ERROR_NAME(ies_ret),
                      port, vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_port_add_helper_exit;
    }

_ies_vlan_port_add_helper_exit:

    return retValue;
}

adk_status_t
ies_vlan_port_rem_helper (
    uint16_t vlan_id,
    uint32_t port)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    printf("ies_vlan_delete_port VLAN ID %d, port %d\n",
                        vlan_id, port);
    ies_ret = ies_vlan_delete_port(ADK_HLP_SWITCH_ID, vlan_id, port);
    if (ies_ret != IES_OK)
    {
        printf("IES Error %s, Failed to remove HLP port-%d as member of"
                      " VLAN %d\n", IES_ERROR_NAME(ies_ret),
                      port, vlan_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _ies_vlan_port_rem_helper_exit;
    }

    printf("Removed port %d from VLAN ID in HLP %d\n", port, vlan_id);

_ies_vlan_port_rem_helper_exit:

    return retValue;
}

static int enable_l4_checksum_validation(uint32_t port)
{
    int ret;
    enum ies_port_parser_validation l4CksumConfig = IES_PORT_PARSER_VALIDATION_STORED_IN_METADATA;
    bool partialCksum = true;

    ret = ies_port_set_attr(ADK_HLP_SWITCH_ID, port, IES_PA_L4_CHECKSUM_VALIDATION, (void *)&l4CksumConfig);
    if (IES_OK != ret)
    {
        printf("%s: failed to enable l4 cksum validation for port %d. ret=%d\n", __func__, port, ret);
        return 1;
    }

    ret = ies_port_set_attr(ADK_HLP_SWITCH_ID, port, IES_PA_L4_PARTIAL_CHECKSUM, (void *)&partialCksum);
    if (IES_OK != ret)
    {
        printf("%s: failed to enable PARTIAL l4 cksum validation for port %d. ret=%d\n", __func__, port, ret);
        return 1;
    }

    return 0;
}

int setup_hlp(ies_event_handler_callback_fn ies_func_adk)
{
    int ret;
    bool use_shm = true;
    int shm_key = ADK_HOS_IES_SHM_KEY;
    bool is_master = false;
    uint32_t event_mask;
    uint32_t port;

    /* Setup for mult-process application with shared memmory and make this
     * process as the master processes */
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHARED_MEMORY, &use_shm);
    if (ret != IES_OK)
    {
       printf("IES Failed to enable shared memory...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_SHM_KEY, &shm_key);
    if (ret != IES_OK)
    {
       printf("IES Failed to configure SHM Key...errno = %d\n", ret);
       return 1;
    }
    ies_trace("ies_init_set_attr()\n");
    ret = ies_init_set_attr(IES_INIT_ATTR_IS_MASTER, &is_master);
    if (ret != IES_OK)
    {
       printf("IES Failed to setup in master mode...errno = %d\n", ret);
       return 1;
    }

    /* Initialize IES */
    event_mask = IES_EVENT_SWITCH_STATE |
                 IES_EVENT_PORT_STATE |
                 IES_EVENT_ERROR |
                 IES_EVENT_TIMEOUT;
    ies_trace("ies_initialize()\n");
    ret = ies_initialize(ies_func_adk, event_mask);
    if (ret != IES_OK)
    {
        printf("%s:IES initialization failed: %d\n",__func__, ret);
        return 1;
    }

    for (port = 0; port < 20; port++)
    {
        if(0 != (ret = enable_l4_checksum_validation(port)))
            return ret;
    }

    return enable_l4_checksum_validation(ADK_HOS_CPM_DECRYPT_PORT);
}

#ifdef ADK_NETD_NETLINK
adk_status_t netd_phy_interface_up(void)
{
    adk_status_t ret = ADK_SUCCESS;
    struct sockaddr_nl src_addr, dest_addr;
    struct nlmsghdr *nlh = NULL;
    int sock_fd;
    struct iovec iov;
    struct msghdr msg;
    struct adk_netd_gen_nl_msg_t *msgPtr;
    int ifId;
    int linkStatus;
    int link_duplex;

    int seq=0;

    ifId = ADK_HLP_PORT_NUM_START;
    linkStatus = 1;
    link_duplex = 1;

    sock_fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_GEN_COMM_NETDEV );

    if (sock_fd == -1)
    {
        ret = -1;
        printf("Socket open failed\n");
        return ret;
    }
    printf("opened socket sock_fd=%d\n", sock_fd);

    memset(&src_addr, 0, sizeof(src_addr));
    src_addr.nl_family = AF_NETLINK;
    src_addr.nl_pid = getpid();  /* self pid */
    src_addr.nl_groups = 0;  /* not in mcast groups */
    if (bind(sock_fd, (struct sockaddr*)&src_addr, sizeof(src_addr)) < 0)
    {
        ret = -1;
        close(sock_fd);
        printf("Socket bind failed\n");
        return ret;
    }
    printf("Socket bind doned\n");

    memset(&dest_addr, 0, sizeof(dest_addr));
    dest_addr.nl_family = AF_NETLINK;
    dest_addr.nl_pid = 0;   /* For Linux Kernel */
    dest_addr.nl_groups = 0; /* unicast */

    nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
    memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
    nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);

    nlh->nlmsg_pid = getpid();  /* self pid */
    nlh->nlmsg_flags =  NLM_F_ACK|NLM_F_REQUEST;
    nlh->nlmsg_seq =  seq++;

    msgPtr = (struct adk_netd_gen_nl_msg_t *)(NLMSG_DATA(nlh));
    msgPtr->mesgCode = ADK_NETD_GEN_LNK_STATUS_CHANGE;

    msgPtr->nlGenMsg.linkStsMsg.linkStatus =  linkStatus;
    msgPtr->nlGenMsg.linkStsMsg.ifId =  ifId;
    /* check speeds and set default speed */
    msgPtr->nlGenMsg.linkStsMsg.linkSpeed =  ADK_NETD_MAC_PHY_SPEED1000;
    msgPtr->nlGenMsg.linkStsMsg.linkDuplex =  link_duplex;

    memset(&msg, 0, sizeof(msg));

    iov.iov_base = (void *)nlh;
    iov.iov_len = nlh->nlmsg_len;
    msg.msg_name = (void *)&dest_addr;
    msg.msg_namelen = sizeof(dest_addr);
    msg.msg_iov = &iov;
    msg.msg_iovlen = 1;

    if (sendmsg(sock_fd, &msg, 0) < 0)
    {
        ret = -1;
        printf("Socket sendmsg failed(errno=%d)\n", errno);
        close(sock_fd);
        free(nlh);
        return ret;
    }

    printf("Link status msg sent for nac_eth%d\n",ADK_HLP_PORT_NUM_START);

#ifdef ADK_DEBUG
    /*netlink_chk_recv_ack(sock_fd);*/
#endif

    close(sock_fd);
    free(nlh);
    return ADK_SUCCESS;
}
#endif

adk_status_t direct_traffic_phy(adk_hos_rule_id_t* default_vsi_q_rule_id)
{
    adk_status_t ret = ADK_SUCCESS;
    uint16_t eth_dev = testConfShm_p->ethDev.tnPorts_p[0];
    uint32_t vsi_id = 0,vsi_q = 0;
    uint32_t iPort = ADK_HLP_PORT_NUM_START;
    uint8_t  i=0;
    bool replace_vlan;
    uint32_t ies_ret, dglort;
    struct ies_acl_table_info acl_tbl;
    struct ies_acl_rule_info acl_rule;
    uint8_t mac_nac_eth12[6] = {0x00,0xa0,0xc9,0x00,0x01,0x0c};

    /*NOTE:VSI QUEUE Table is getting removed from ADK HOS 
    hence create it in the application itself to add the rule*/
    /* Create VSI Queue table */
    memset(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = IES_ACL_FFU_GROUP_2;
    acl_tbl.acl_instance = IES_ACL_NO_INSTANCE ;
    acl_tbl.scenario = (IES_ACL_SCENARIO_INGRS_NONE_IP | \
            IES_ACL_SCENARIO_PARSER_ERR | \
            IES_ACL_SCENARIO_INGRS_IPV4 | \
            IES_ACL_SCENARIO_INGRS_IPV6 | \
            IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
            IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
            IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
            IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6);
    acl_tbl.condition = (IES_ACL_MATCH_METADATA | \
                         IES_ACL_MATCH_DMAC);
    acl_tbl.mask.meta.sel = IES_ACL_MATCH_L2_DOMAIN;

    acl_tbl.mask.meta.l2_domain = APP_L2_DOMAIN_MASK;
    acl_tbl.mask.dmac = APP_MAC_ADDR_MASK;
    acl_tbl.min_entries = APP_VSI_QUEUE_TBL_MIN_RULES;
    acl_tbl.max_entries = APP_VSI_QUEUE_TBL_MAX_RULES;

    printf("ies_acl_create_table:VSI Queue table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
            APP_VSI_QUEUE_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        printf("Failed to  create the VSI Queue Table\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }

    ret = adk_sys_eth_dev_vsi_get((uint16_t) eth_dev,
            &vsi_id);
    if (ret != ADK_SUCCESS)
    {
        printf("adk_sys_eth_dev_vsi_get failed\n");
        return ret;
    }

    ies_ret =  ies_vsi_map_vsi_qid_to_port(ADK_HLP_SWITCH_ID, vsi_id,
            vsi_q, (int *) &dglort);
    if (ies_ret != IES_OK)
    {
        printf("ies_vsi_map_vsi_qid_to_port failed\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }

    memset(&acl_rule, 0, sizeof(acl_rule));

    acl_rule.precedence += APP_L2_DOMAIN_WIDTH;
    acl_rule.condition |= IES_ACL_MATCH_METADATA;
    acl_rule.cond.mask.meta.sel |= IES_ACL_MATCH_L2_DOMAIN;
    acl_rule.cond.mask.meta.l2_domain = APP_L2_DOMAIN_MASK;
    acl_rule.cond.val.meta.l2_domain = BG_ID;


    uint64_t dmac_mask = APP_MAC_ADDR_MASK;
    uint64_t dmac =  mac_nac_eth12[0];
    dmac = (dmac << 8) | mac_nac_eth12[1];
    dmac = (dmac << 8) | mac_nac_eth12[2];
    dmac = (dmac << 8) | mac_nac_eth12[3];
    dmac = (dmac << 8) | mac_nac_eth12[4];
    dmac = (dmac << 8) | mac_nac_eth12[5];


    acl_rule.condition |= IES_ACL_MATCH_DMAC;
    acl_rule.precedence += APP_MAC_ADDR_WIDTH;
    acl_rule.cond.val.dmac = (dmac & dmac_mask);
    acl_rule.cond.mask.dmac = dmac_mask;


    acl_rule.action = IES_ACL_ACTION_REDIRECT;
    acl_rule.act_val.logical_port = dglort;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, APP_VSI_QUEUE_TBL_IDX,
                               &acl_rule, (int *) default_vsi_q_rule_id);
    if (ies_ret != IES_OK)
    {
        printf("failed add entry in VSI queue table\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }
    for(i=0; i < NUM_PORTS; ++i)
    {
        uint32_t aging_timer = 1000 * 60 * ADK_ETH_DEFAULT_MAC_AGING_TIMER;
        ret = config_mac_aging((iPort+i), aging_timer);
        if (ret != ADK_SUCCESS)
        {
            printf("Failed to set MAC aging timer for port\n");
            return ret;
        }

        /* Use packet VLAN */
        replace_vlan = false;
        printf("ies_port_set_attr Disable VLAN replacement for HLP"
                " port %d\n", (iPort+i));
        ret = ies_port_set_attr(ADK_HLP_SWITCH_ID, (iPort+i),
                IES_PA_REPLACE_VLAN_FIELDS,
                (void *) &replace_vlan);
        if (ret != IES_OK)
        {
            printf("IES Error: Failed to disable VLAN overide behavior"
                    " for HLP port%d\n", (iPort+i));
            return ADK_E_HW_PROGRAMMING_ERROR;
        }
    }
    /*Creating Flooding domain for broadcast pkts*/
    const int portlist[2] = {12,13};
    ret = ies_domain_add_ports_to_floodset(ADK_HLP_SWITCH_ID, BG_ID,
            IES_DOMAIN_FLOODSET_UNICAST,2, portlist);
    if (ret != IES_OK)
    {
        printf("Adding port 12 & 13 to UNICAST floodset failed\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }

    ret = ies_domain_add_ports_to_floodset(0, 20,
            IES_DOMAIN_FLOODSET_MULTICAST,2, portlist);
    if (ret != IES_OK)
    {
        printf("Adding port 12 & 13 to MULTICAST floodset failed\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }
    /*Creating Flooding domain for broadcast pkts*/
    ret = ies_domain_add_ports_to_floodset(0, 20,
            IES_DOMAIN_FLOODSET_BROADCAST,2, portlist);
    if (ret != IES_OK)
    {
        printf("Adding port 12 & 13 to BROADCAST floodset failed\n");
        return ADK_E_HW_PROGRAMMING_ERROR;
    }
    return ret;
} /* direct traffic */

adk_status_t direct_traffic_remove(adk_hos_rule_id_t *default_vsi_q_rule_id)
{
    adk_status_t ret_val = ADK_SUCCESS;
    uint32_t ies_ret;

    ies_ret = ies_acl_delete_rule(ADK_HLP_SWITCH_ID, APP_VSI_QUEUE_TBL_IDX, *default_vsi_q_rule_id);
    if (ies_ret != IES_OK)
    {
        ret_val = ADK_E_HW_PROGRAMMING_ERROR;
        printf("Failed to delete entry from HOS VSI queue table\n");
    }
    printf("ies_acl_destroy_table: APP VSI Queue table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    APP_VSI_QUEUE_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        ret_val = ADK_E_HW_PROGRAMMING_ERROR;
        printf("Failed to destroy ies_acl_destroy_table: APP VSI Queue table\n");
    }

    return ret_val;
}

#define PRINT_NON_ZERO(fmt, cntr) \
    if (cntr != 0) { \
        printf(fmt, cntr); \
    }
void print_ipsec_global_stats(adk_module_handle_t ipsec_handle, bool ipv6_mode)
{
    adk_ipsec_stat_t ipsecStats;
    adk_status_t ret_val;

    memset (&ipsecStats, 0x00, sizeof(adk_ipsec_stat_t));
    ret_val = adk_ipsec_get_stat (
            ipsec_handle,
            (ipv6_mode?ADK_IPSEC_IPV6:ADK_IPSEC_IPV4),
            &ipsecStats);

    if (ret_val != ADK_SUCCESS)
    {
        printf ("\nUnable to get IPsec statistics!\n");
        return;
    }

    printf("\nIPSEC Global Statistics"" (non zero counts only):\n\n");
    PRINT_NON_ZERO ("Total plain text packets bypassed in IPsec at ingress stage: %lu\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_in_pol_bypass_count);
    PRINT_NON_ZERO ("Total plain text packets dropped in IPsec at ingress stage: %lu\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_in_pol_discard_drop);
    PRINT_NON_ZERO ("Total SPD IN default drops in IPsec at ingress stage: %lu\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_in_default_discard_drop);
    PRINT_NON_ZERO ("Total keep alive packet drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_keepalive_pdu_drop);
    PRINT_NON_ZERO ("Total unknown NATT packet drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_unknown_natt_pdu_drop);
    PRINT_NON_ZERO ("Total ip fragmentation failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_ip_fragment_drop);
    PRINT_NON_ZERO ("Total policy action DROP drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_pol_drop);
    PRINT_NON_ZERO ("Total SA not present drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_sa_not_present_drop);
    PRINT_NON_ZERO ("Total authentication failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_auth_fail_drop);
    PRINT_NON_ZERO ("Total pad verification drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_pad_verification_fail_drop);
    PRINT_NON_ZERO ("Total sequence number verification failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_seqno_check_fail_drop);
    PRINT_NON_ZERO ("Total hard life time expiry drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_sa_hard_life_expire_drop);
    PRINT_NON_ZERO ("Total SA validation failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_sa_validation_fail_drop);
    PRINT_NON_ZERO ("Total SPD OUT default discard drops in IPsec at ingress stage: %lu\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_out_default_discard_drop);
    PRINT_NON_ZERO ("Total packets dropped in inbound processing in IPsec at ingress stage: %lu\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_out_not_ipsec_drop);
    PRINT_NON_ZERO ("Total dummy packet drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_dummy_pkt_drop);
    PRINT_NON_ZERO ("Total TFC packet drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_tfc_pad_drop);
    PRINT_NON_ZERO ("Total exception not registered drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_exception_not_registered);
    PRINT_NON_ZERO ("Total inline decryptrion failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_inline_decryption_failed);
    PRINT_NON_ZERO ("Total lookaside crypto send failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_la_send_to_crypto_failed);
    PRINT_NON_ZERO ("Total pad removal failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_pad_removal_fail_drop);
    PRINT_NON_ZERO ("Total ARW update failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_arw_update_fail_drop);
    PRINT_NON_ZERO ("Total packets dropped due to next stage send failure in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_send_to_next_stage_failed);
    PRINT_NON_ZERO ("Total packet processing failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_pkt_process_fail_drop);
    PRINT_NON_ZERO ("Total packet dropped while processing buffered fragments in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_buffered_single_sa_pkt_process_fail_drop);
    PRINT_NON_ZERO ("Total mbuf clone creation failures in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_mbuf_clone_failed);
    PRINT_NON_ZERO ("Total too small packet received drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_too_small_pkt_drop);
    PRINT_NON_ZERO ("Total packet drops due to buffer allocation failure in lookaside in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_la_no_buffer_drop);
    PRINT_NON_ZERO ("Total pad allocation failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_pad_alloc_fail_drop);
    PRINT_NON_ZERO ("Total reassembly failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_reassembly_fail_drop);
    PRINT_NON_ZERO ("Total packet drops due to invalid destination function in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop);
    PRINT_NON_ZERO ("Total bypass policy lookup failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_spd_in_lookup_fail_drop);
    PRINT_NON_ZERO ("Total unsupported L4 protocol drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_unsupported_l4_proto_drop);
    PRINT_NON_ZERO ("Total exception send failure drops in IPsec at ingress stage: %u\n",
            ipsecStats.ingressStats.ipsec_ing_global_exception_send_failure_drop);


    PRINT_NON_ZERO ("Total packets bypassed in IPsec at egress stage: %lu\n",
            ipsecStats.egressStats.ipsec_eg_global_spd_pol_bypassCnt);
    PRINT_NON_ZERO ("Total packets dropped policy action DROP in IPsec at egress stage: %lu\n",
            ipsecStats.egressStats.ipsec_eg_global_spd_pol_discard_drop);
    PRINT_NON_ZERO ("Total packets dropped to policy action is not a protect IPsec at egress stage: %lu\n",
            ipsecStats.egressStats.ipsec_eg_global_spd_out_not_ipsec_drop);
    PRINT_NON_ZERO ("Total exception not registered drops in IPsec at egress stage: %u\n",
            ipsecStats.egressStats.ipsec_eg_global_exception_not_registered);
    PRINT_NON_ZERO ("Total default drops in IPsec at egress stage: %lu\n",
            ipsecStats.egressStats.ipsec_eg_global_spd_pol_default_discard_drop);
    PRINT_NON_ZERO ("Total sequence number overflow drops in IPsec at egress stage: %u\n",
            ipsecStats.egressStats.ipsec_eg_global_seqno_overflow_drop);
    PRINT_NON_ZERO ("Total SA hard life expire drop in egress : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_sa_hardlife_expire_drop);
    PRINT_NON_ZERO ("Total number of IPsec SA not found drop in egress : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_sa_not_found_drop);
    PRINT_NON_ZERO ("Total packets dropped due to DF bit set in IPsec in egress stage: %u\n",
            ipsecStats.egressStats.ipsec_eg_global_prefrag_reqd_df_bit_set_drop_count);
    PRINT_NON_ZERO ("Total pre-frag error in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_pre_frag_error_drop);
    PRINT_NON_ZERO ("Total pad alloc failure in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_pad_alloc_fail_drop);
    PRINT_NON_ZERO ("Total packet dropped due to unavailabilty of the security session in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_no_security_session_drop);
    PRINT_NON_ZERO ("Total packet drops due to crypto send failure in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_la_send_to_crypto_failed);
    PRINT_NON_ZERO ("Total packet due to buffur allocation failure in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_la_no_buffer_drop);
    PRINT_NON_ZERO ("Total too small packet received drops in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_too_small_pkt_drop);
    PRINT_NON_ZERO ("Total lookaside operation failure drop in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_la_crypto_op_failed);
    PRINT_NON_ZERO ("Total mbuf clone failures in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_mbuf_clone_failed);
    PRINT_NON_ZERO ("Total outer header buffer allocation failure drops in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_outer_hdr_buf_alloc_fail_drop);
    PRINT_NON_ZERO ("Total exception send failure drops in IPsec in egress stage : %u\n",
            ipsecStats.egressStats.ipsec_eg_global_exception_send_failure_drop);

} /* print_ipsec_global_stats */

void print_ip_global_stats(adk_ip_global_stats_t *ipStats)
{
    printf("IP Global Statistics::\n");
    PRINT_NON_ZERO("ipOutTooBigDrops                 : %u\n", ipStats->ipOutTooBigDrops);
    PRINT_NON_ZERO("ipInReasmToutFragDrops           : %u\n", ipStats->ipInReasmToutFragDrops);
    PRINT_NON_ZERO("ipInReasmNoResourceDrops         : %u\n", ipStats->ipInReasmNoResourceDrops);
    PRINT_NON_ZERO("ipInReasmFragBufferFullDrops     : %u\n", ipStats->ipInReasmFragBufferFullDrops);
    PRINT_NON_ZERO("ipInReasmHolesExceededDrops      : %u\n", ipStats->ipInReasmHolesExceededDrops);
    PRINT_NON_ZERO("ipInClassifierDefaultDrops       : %u\n", ipStats->ipInClassifierDefaultDrops);
    PRINT_NON_ZERO("ipNoExceptionRegisteredDrops     : %u\n", ipStats->ipNoExceptionRegisteredDrops);
    PRINT_NON_ZERO("ipExceptionSendFailureDrops      : %u\n", ipStats->ipExceptionSendFailureDrops);
    PRINT_NON_ZERO("ipInPolicerDrops                 : %u\n", ipStats->ipInPolicerDrops);
    PRINT_NON_ZERO("ipInPolicerDrops                 : %u\n", ipStats->ipInClassifierDrops);
    PRINT_NON_ZERO("ipInSendToEoFailDrops            : %u\n", ipStats->ipInSendToEoFailDrops);
    PRINT_NON_ZERO("ipInSendToHostFailDrops          : %u\n",ipStats->ipInSendToHostFailDrops);
    PRINT_NON_ZERO("ipInClassifierUnknownActionDrops : %u\n",ipStats->ipInClassifierUnknownActionDrops);
    PRINT_NON_ZERO("ipInReasmDuplicateDrops          : %u\n", ipStats->ipInReasmDuplicateDrops);
    PRINT_NON_ZERO("ipOutInvalidIpHdrLenDrops        : %u\n", ipStats->ipOutInvalidIpHdrLenDrops);
    PRINT_NON_ZERO("ipOutNonLocalSipFromEoDrops      : %u\n", ipStats->ipOutNonLocalSipFromEoDrops);
    PRINT_NON_ZERO("ipInReasmToutMbufAllocFails      : %u\n", ipStats->ipInReasmToutMbufAllocFails);
    PRINT_NON_ZERO("ipOutFragMbufAllocFailDrops      : %u\n", ipStats->ipOutFragMbufAllocFailDrops);
    PRINT_NON_ZERO("ipOutTruncatedPktDrops           : %u\n",ipStats->ipOutTruncatedPktDrops);
    PRINT_NON_ZERO("ipInInvalidIfDrops               : %u\n",ipStats->ipInInvalidIfDrops);
    PRINT_NON_ZERO("ipOutInvalidIfDrops              : %u\n",ipStats->ipOutInvalidIfDrops);
    PRINT_NON_ZERO("ipOutNoRouteFoundDrops           : %u\n",ipStats->ipOutNoRouteFoundDrops);
    PRINT_NON_ZERO("ipOutFragDFSetDrops              : %u\n",ipStats->ipOutFragDFSetDrops);
    PRINT_NON_ZERO("ipInReasmTouts                   : %u\n",ipStats->ipInReasmTouts);

    printf("-- --\n");
}

void print_ip_l4_stats(adk_ip_l4_stats_t *ipL4Stats)
{
    printf("\nIP L4 Statistics"
            " (non-zero stats only):\n\n");

    PRINT_NON_ZERO("ipTcpInPackets      : %lu\n", ipL4Stats->ipTcpInPackets);
    PRINT_NON_ZERO("TCP in octets       : %lu\n", ipL4Stats->ipTcpInOctets);
    PRINT_NON_ZERO("TCP out packets     : %lu\n", ipL4Stats->ipTcpOutPackets);
    PRINT_NON_ZERO("TCP out octets      : %lu\n", ipL4Stats->ipTcpOutOctets);
    PRINT_NON_ZERO("SCTP in packets     : %lu\n", ipL4Stats->ipSctpInPackets);
    PRINT_NON_ZERO("SCTP in octets      : %lu\n", ipL4Stats->ipSctpInOctets);
    PRINT_NON_ZERO("SCTP out packets    : %lu\n", ipL4Stats->ipSctpOutPackets);
    PRINT_NON_ZERO("SCTP out octets     : %lu\n", ipL4Stats->ipSctpOutOctets);

    printf("-- -- \n");
}

void print_ip_addr_stats(adk_ip_address_stats_t *st)
{
    PRINT_NON_ZERO("ipInOctets      : %lu\n", st->ipInOctets);
    PRINT_NON_ZERO("ipInPkts        : %lu\n", st->ipInPkts);
    PRINT_NON_ZERO("ipOutOctets     : %lu\n", st->ipOutOctets);
    PRINT_NON_ZERO("ipOutPkts       : %lu\n", st->ipOutPkts);
    PRINT_NON_ZERO("ipInReasmPkts   : %lu\n", st->ipInReasmPkts);
    PRINT_NON_ZERO("ipInErrors      : %u\n", st->ipInErrors);

    printf("-- --\n");
}

void print_ip_interface_stats(adk_ip_interface_stats_t *ipIfStats)
{

    PRINT_NON_ZERO("ifInOctets: %"PRIu64"\n", ipIfStats->ifInOctets);
    PRINT_NON_ZERO("ifInPackets: %lu\n", ipIfStats->ifInPkts);
    PRINT_NON_ZERO("ifInMcastOctets: %lu\n", ipIfStats->ifInMcastOctets);
    PRINT_NON_ZERO("ifInMcastPkts: %lu\n", ipIfStats->ifInMcastPkts);
    PRINT_NON_ZERO("ifOutMcastOctets: %lu\n", ipIfStats->ifOutMcastOctets);
    PRINT_NON_ZERO("ifOutMcastPkts: %lu\n", ipIfStats->ifOutMcastPkts);
    PRINT_NON_ZERO("ifOutOctets: %"PRIu64"\n", ipIfStats->ifOutOctets);
    PRINT_NON_ZERO("ifOutPackets: %lu\n", ipIfStats->ifOutPkts);

    PRINT_NON_ZERO("ifInIpCksumErrDrops: %u\n", ipIfStats->ifInIpCksumErrDrops);
    PRINT_NON_ZERO("ifInMBcastDMacUcastDipErrDrops: %u\n", ipIfStats->ifInMBcastDMacUcastDipErrDrops);
    PRINT_NON_ZERO("ifInBadIpLengthErrDrops: %u\n", ipIfStats->ifInBadIpLengthErrDrops);
    PRINT_NON_ZERO("ifOutTtlExceededErrDrops: %u\n", ipIfStats->ifOutTtlExceededErrDrops);
    PRINT_NON_ZERO("ifInVersionMismatchErrDrops: %u\n", ipIfStats->ifInVersionMismatchErrDrops);
    PRINT_NON_ZERO("ifInInvalidVersionErrDrops: %u\n", ipIfStats->ifInInvalidVersionErrDrops);
    PRINT_NON_ZERO("ifInIpOptionsPresentDrops: %u\n", ipIfStats->ifInIpOptionsPresentDrops);
    PRINT_NON_ZERO("ifInUnknownProtoErrDrops: %u\n", ipIfStats->ifInUnknownProtoErrDrops);
    PRINT_NON_ZERO("ifInUDPCksumErrDrops: %u\n", ipIfStats->ifInUDPCksumErrDrops);
    PRINT_NON_ZERO("ifInICMPCksumErrDrops: %u\n", ipIfStats->ifInICMPCksumErrDrops);
    PRINT_NON_ZERO("ifInZeroDipDrops: %u\n", ipIfStats->ifInZeroDipDrops);

    PRINT_NON_ZERO("ifOutFragPkts : %lu\n", ipIfStats->ifOutFragPkts);
    PRINT_NON_ZERO("ifOutFragments : %lu\n", ipIfStats->ifOutFragments);
    PRINT_NON_ZERO("ifOutDroppedOctets: %u\n", ipIfStats->ifOutDroppedOctets);
    PRINT_NON_ZERO("ifOutDroppedPackets: %u\n", ipIfStats->ifOutDroppedPkts);
    PRINT_NON_ZERO("ifRoutingDisabledDrops: %u\n", ipIfStats->ifRoutingDisabledDrops);
    PRINT_NON_ZERO("ifOutQoSLookupFailDrops : %u\n",ipIfStats->ifOutQoSLookupFailDrops);

    printf("-- --\n");
}

void print_ip_utility_stats(adk_module_handle_t ipHandle)
{
    adk_ip_utility_stats_t ipUtilStats;

    printf(" IP utility stats::\n");
    if(ADK_SUCCESS != adk_ip_utility_stats_get(ipHandle, &ipUtilStats))
        goto _print_ip_utility_stats_end;

    PRINT_NON_ZERO("ipInSentToEoPkts            :%lu\n", ipUtilStats.ipInSentToEoPkts);
    PRINT_NON_ZERO("ipInSentToHostPkts          :%lu\n",ipUtilStats.ipInSentToHostPkts);
    PRINT_NON_ZERO("ipInSentToIPSecPkts         :%lu\n",ipUtilStats.ipInSentToIPSecPkts);
    PRINT_NON_ZERO("ipOutEOOrigPkts             :%lu\n",ipUtilStats.ipOutEOOrigPkts);
    PRINT_NON_ZERO("ipOutNetDOrigPkts           :%lu\n",ipUtilStats.ipOutNetDOrigPkts);
    PRINT_NON_ZERO("ipOutNonLocalSipFromHostPkts:%lu\n",ipUtilStats.ipOutNonLocalSipFromHostPkts);
    PRINT_NON_ZERO("ipOutSentToEnetPkts         :%lu\n",ipUtilStats.ipOutSentToEnetPkts);
    PRINT_NON_ZERO("ipOutSentToIPSecPkts        :%lu\n",ipUtilStats.ipOutSentToIPSecPkts);
    PRINT_NON_ZERO("ipOutRcvdFromIPSecPkts      :%lu\n",ipUtilStats.ipOutRcvdFromIPSecPkts);
    PRINT_NON_ZERO("ipReasmEnqueuedPkts         :%lu\n",ipUtilStats.ipReasmEnqueuedPkts);
    PRINT_NON_ZERO("ipReasmInvalidReasmAction   :%u\n",ipUtilStats.ipReasmInvalidReasmAction);
    PRINT_NON_ZERO("ipReasmReqPkts              :%lu\n",ipUtilStats.ipReasmReqPkts);
    PRINT_NON_ZERO("ipReasmTimerCbAfterCleanup  :%u\n",ipUtilStats.ipReasmTimerCbAfterCleanup);
    PRINT_NON_ZERO("ipInUDPOctets               :%lu\n",ipUtilStats.ipInUDPOctets);
    PRINT_NON_ZERO("ipInUDPPkts                 :%lu\n",ipUtilStats.ipInUDPPkts);
    PRINT_NON_ZERO("ipOutUDPOctets              :%lu\n",ipUtilStats.ipOutUDPOctets);
    PRINT_NON_ZERO("ipOutUDPPkts                :%lu\n",ipUtilStats.ipOutUDPPkts);

    PRINT_NON_ZERO("ipIpsecFragsCreated              :%lu\n",ipUtilStats.ipIpsecFragsCreated);
    PRINT_NON_ZERO("ipIpsecFragsRequested       :%lu\n",ipUtilStats.ipIpsecFragsRequested);
    _print_ip_utility_stats_end:
    printf("-- --\n");
}

uint32_t get_ip_if_handle_index_in_SL(qos_sl_sched_t *sl_node_info_p, adk_interface_handle_t ip_if_handle)
{
    uint16_t i = 0;
    for (i = 0; i < QOS_FL_MAX_NODES; i++) {
        if (ip_if_handle == sl_node_info_p->ip_if_handle[i]) {
            return i;
        }
    }
    return QOS_FL_MAX_NODES;
}

qos_sl_sched_t *get_sl_node_info_from_enet_if(adk_interface_handle_t enet_if_handle)
{
    uint16_t i = 0;
    for (i = 0; i < MAX_L2_INTF ; i++) {
        if (enet_if_handle == g_sl_map[i].l2_intf_handle) {
            return &g_sl_map[i];
        }
    }
    return NULL;
}

qos_fl_sched_t *get_fl_node_info_from_ip_if(adk_interface_handle_t ip_if_handle)
{
    uint16_t i = 0;
    for (i = 0; i < MAX_L3_INTF ; i++) {
        if (ip_if_handle == g_fl_map[i].l3_intf_handle) {
           return &g_fl_map[i];
        }
    }
    return NULL;
}

bool is_shaper_profile_configured(uint32_t shaper_profile_id, uint32_t *index)
{
    uint16_t i = 0;
    for (i = 0; i < g_shaper_profile_count; i++) {
        if ((shaper_profile_id == g_shaper_profiles_tbl[i].profile_id) && g_shaper_profiles_tbl[i].configured) {
            *index = i;
            return true;
        }
    }
    *index = i;
    return false;
}

int cpk_qos_sl_sched_create(adk_sys_handle_t sysHandle,
                            uint8_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle)
{
    struct rte_tm_node_params node_params = {0};
    struct rte_tm_shaper_params shaper_profile;
    struct rte_tm_capabilities tm_capabilities;
    uint32_t shaper_profile_id;
    struct rte_tm_error tm_error;
    int *wfq_weights = NULL;
    uint32_t     sl_node_id = QOS_SL_NODE_ID + g_num_sl_nodes;
    uint32_t     loop_idx = 0;
    uint32_t     tx_queue_id = 0;
    uint32_t     shaper_profile_id_index = 0;
	struct rte_eth_dev_info ethdev_info;
    int ret;

    /*Get the TM capabilities*/

    /*Get information about number of TX queues configured on the VSI device*/
    rte_eth_dev_info_get(testConfShm_p->ethDev.tnPorts_p[0],
						 &ethdev_info);

    g_vsi_num_max_tx_queues = ethdev_info.nb_tx_queues;

    ret = rte_tm_capabilities_get(vport_dev_id, &tm_capabilities, &tm_error);
    if(ret) {
        printf("\n  rte_tm_capabilities_get failed \n");
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }


    if(tm_capabilities.sched_sp_n_priorities_max) {
        wfq_weights = malloc(sizeof(int) * tm_capabilities.sched_sp_n_priorities_max);
        for ( loop_idx = 0; loop_idx < tm_capabilities.sched_sp_n_priorities_max; loop_idx++) {
            memset(&wfq_weights[loop_idx], 1, sizeof(int));
        }
    }
    /*Create a Second level scheduler */
    /*CIR shapers are not supported on L1(root nodes)*/
    node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
    node_params.shared_shaper_id  = NULL;
    node_params.n_shared_shapers  = 0;
    node_params.nonleaf.wfq_weight_mode = wfq_weights;
    /*TODO: Get the max number of strict priorities supported by the PSM nodes and use it below*/
    node_params.nonleaf.n_sp_priorities = tm_capabilities.sched_sp_n_priorities_max;
    node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
            RTE_TM_STATS_N_BYTES |
            RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
            RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
            RTE_TM_STATS_N_PKTS_RED_DROPPED |
            RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
            RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
            RTE_TM_STATS_N_BYTES_RED_DROPPED |
            RTE_TM_STATS_N_PKTS_QUEUED |
            RTE_TM_STATS_N_BYTES_QUEUED);

    /* add a root node for a vport  */
    ret = rte_tm_node_add(vport_dev_id,
            QOS_PORT_ROOT_NODE_ID,
            RTE_TM_NODE_ID_NULL,
            QOS_DEFAULT_PRIORITY,
            QOS_DEFAULT_WEIGHT,
            RTE_TM_NODE_LEVEL_ID_ANY,
            &node_params,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add vport root.errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }

    shaper_profile_id = QOS_SL_SHAPER_PROFILE_ID;

    if (!is_shaper_profile_configured(shaper_profile_id, &shaper_profile_id_index)) {
        /* Create a shaper profile with 10 Gbps*/
        shaper_profile.mode = RTE_TM_SHAPER_MODE_BYTES;
        shaper_profile.peak.rate         = QOS_SHAPING_RATE_10G + QOS_SHAPING_RATE_1G;
        shaper_profile.peak.size         = 4*4*1024;
        shaper_profile.committed.rate    = QOS_SHAPING_RATE_10G;
        shaper_profile.committed.size    = shaper_profile.peak.size;
        shaper_profile.pkt_length_adjust = RTE_PMD_ICE_DSI_TM_PKT_LEN_ADJ_FROM_L2_HDR; /*Ether header + fcs*/

        ret = rte_tm_shaper_profile_add(vport_dev_id, shaper_profile_id,
                &shaper_profile, &tm_error);

        if(ret) {
            printf("\n  rte_tm_shaper_profile_add failed \n");
            printf("\nTM error : %s\n",tm_error.message);
            return 1;
        }
        printf("\nAdded shaper profile for SL , id %d \n", shaper_profile_id);
        g_shaper_profiles_tbl[g_shaper_profile_count].profile_id = shaper_profile_id;
        g_shaper_profiles_tbl[g_shaper_profile_count].configured = true;
        g_shaper_profiles_tbl[g_shaper_profile_count].ref_cnt = 1;
        g_shaper_profile_count++;
    } else {
        g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt++;
    }

    /*Create a Second level scheduler */
    node_params.shaper_profile_id = shaper_profile_id;
    node_params.shared_shaper_id  = NULL;
    node_params.n_shared_shapers  = 0;
    node_params.nonleaf.wfq_weight_mode = wfq_weights;
    /*TODO: Get the max number of strict priorities supported by the PSM nodes and use it below*/
    node_params.nonleaf.n_sp_priorities = tm_capabilities.sched_sp_n_priorities_max;
    node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
            RTE_TM_STATS_N_BYTES |
            RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
            RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
            RTE_TM_STATS_N_PKTS_RED_DROPPED |
            RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
            RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
            RTE_TM_STATS_N_BYTES_RED_DROPPED |
            RTE_TM_STATS_N_PKTS_QUEUED |
            RTE_TM_STATS_N_BYTES_QUEUED);

    /* add a root node for a vport  */
    ret = rte_tm_node_add(vport_dev_id,
            sl_node_id,
            QOS_PORT_ROOT_NODE_ID,
            QOS_DEFAULT_PRIORITY,
            QOS_DEFAULT_WEIGHT,
            RTE_TM_NODE_LEVEL_ID_ANY,
            &node_params,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add vport root.errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }

    printf("\nCreated SL node : 0x%x\n", sl_node_id);

    g_sl_map[g_num_sl_nodes].node_id = sl_node_id;
    g_sl_map[g_num_sl_nodes].l2_intf_handle = enet_if_handle;
    g_sl_map[g_num_sl_nodes].shaper_profile.profile_id = shaper_profile_id;
    g_sl_map[g_num_sl_nodes].shaper_profile.configured = true;


    /*Create SP queues for non-IP traffic here
      SP1 : prio = 0, SP2 : prio = 1
     */
    for (loop_idx = 0; loop_idx < QOS_SL_NUM_QUEUES; loop_idx++) {
        /* Create leaf nodes  */
        node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
        node_params.shared_shaper_id  = NULL;
        node_params.n_shared_shapers  = 0;
        node_params.leaf.cman = RTE_TM_CMAN_TAIL_DROP;
        node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
                RTE_TM_STATS_N_BYTES |
                RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
                RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
                RTE_TM_STATS_N_PKTS_RED_DROPPED |
                RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
                RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
                RTE_TM_STATS_N_BYTES_RED_DROPPED |
                RTE_TM_STATS_N_PKTS_QUEUED |
                RTE_TM_STATS_N_BYTES_QUEUED);

        tx_queue_id = g_vsi_num_tx_queues++;

        /* configure leaf-level node */
        ret = rte_tm_node_add(vport_dev_id,
                tx_queue_id,
                sl_node_id,
                loop_idx,
                loop_idx + 1,
                RTE_TM_NODE_LEVEL_ID_ANY,
                &node_params,
                &tm_error);
        if (ret) {
            printf("rte_tm_node_add() failed to add queue %d leaf node."
                    " errno = %d\n", tx_queue_id, ret);
           printf("\nTM error : %s\n",tm_error.message);
            return 1;
        } else {
           printf("Create Leaf queue 0x%x\n", tx_queue_id);
            g_sl_map[g_num_sl_nodes].tx_queues[loop_idx] = tx_queue_id;
        }
        /*Register this SL queue into QoS module*/
        adk_qos_sl_queue_register(sysHandle, tx_queue_id);
        ret = rte_eth_dev_tx_queue_start(ethdev_id, tx_queue_id);
        if (ret) {
            printf("rte_eth_dev_tx_queue_start() failed to start queue %u "
                    " errno = %d\n", tx_queue_id, ret);
            return 1;
        } else {
            printf("Started tx_queue 0x%x\t", tx_queue_id);
        }
    }

    /*Configure the systemwide default tranmit queue - queueId 1 */
    if (adk_qos_default_queue_id_set(sysHandle, g_sl_map[g_num_sl_nodes].tx_queues[1]) != ADK_SUCCESS) {
        printf("\nDefault QoS queue configuration failed \n");
    }else{
        printf("\nConfigured default QoS transmit queue %u \n", g_sl_map[g_num_sl_nodes].tx_queues[1]);
    }

    /*Configure the systemwide PTP tranmit queue - queueId 0 */
    if (adk_qos_ptp_queue_id_set(sysHandle, g_sl_map[g_num_sl_nodes].tx_queues[0]) != ADK_SUCCESS) {
        printf("\nPTP queue configuration failed \n");
    }else{
        printf("\nConfigured PTP transmit queue %u \n", g_sl_map[g_num_sl_nodes].tx_queues[0]);
    }

    g_num_sl_nodes++;

    return 0;
}

int cpk_qos_sl_sched_delete(adk_sys_handle_t sysHandle,
                            uint8_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle)
{
    struct rte_tm_error tm_error;
    qos_sl_sched_t *sl_info_p = get_sl_node_info_from_enet_if(enet_if_handle);
    uint32_t     sl_node_id = sl_info_p->node_id;
    uint32_t     shaper_profile_id_index = 0;
    uint32_t     loop_idx = 0;
    int ret;

    for (loop_idx = 0; loop_idx < QOS_SL_NUM_QUEUES; loop_idx++) {

        ret = rte_eth_dev_tx_queue_stop(ethdev_id, sl_info_p->tx_queues[loop_idx]);
        if (ret) {
            printf("rte_eth_dev_tx_queue_stop() failed to stop queue %u "
                    " errno = %d\n", sl_info_p->tx_queues[loop_idx], ret);
            return 1;
        } else {
            printf("Stopped tx_queue 0x%x\t", sl_info_p->tx_queues[loop_idx]);
        }
        /* Delete leaf nodes  */
        ret = rte_tm_node_delete(vport_dev_id,
                sl_info_p->tx_queues[loop_idx],
                &tm_error);
        if (ret) {
            printf("rte_tm_node_delete() failed to delete queue %d leaf node."
                    " errno = %d\n", sl_info_p->tx_queues[loop_idx], ret);
           printf("\nTM error : %s\n",tm_error.message);
            return 1;
        } else {
           printf("Deleted Leaf queue 0x%x\n", sl_info_p->tx_queues[loop_idx]);
            sl_info_p->tx_queues[loop_idx] = 0;
        }
    }

    /*Delete Second level scheduler */
    ret = rte_tm_node_delete(vport_dev_id,
            sl_node_id,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_delete() failed to add vport root.errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }
    printf("\nDelete SL node : 0x%x\n", sl_node_id);

    if (is_shaper_profile_configured(QOS_SL_SHAPER_PROFILE_ID, &shaper_profile_id_index)) {

        g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt--;
        if (!g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt) {
            ret = rte_tm_shaper_profile_delete(vport_dev_id, QOS_SL_SHAPER_PROFILE_ID, &tm_error);

            if(ret) {
                printf("\n  rte_tm_shaper_profile_delete failed \n");
                printf("\nTM error : %s\n",tm_error.message);
                return 1;
            }
            printf("\nDeleted shaper profile for SL , id %d \n", QOS_SL_SHAPER_PROFILE_ID);
            g_shaper_profiles_tbl[shaper_profile_id_index].profile_id = 0;
            g_shaper_profiles_tbl[shaper_profile_id_index].configured = false;
            g_shaper_profile_count--;
        }
    }
    /* delete root node for a vport  */
    ret = rte_tm_node_delete(vport_dev_id,
            QOS_PORT_ROOT_NODE_ID,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_delete() failed to delete vport root.errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }


    sl_info_p->node_id = 0;
    sl_info_p->l2_intf_handle = 0;
    sl_info_p->shaper_profile.profile_id = 0;
    sl_info_p->shaper_profile.configured = false;

    /*Reset the systemwide default tranmit queue */
    if (adk_qos_default_queue_id_reset(sysHandle) != ADK_SUCCESS) {
        printf("\nDefault QoS queue Reset configuration failed \n");
    }else{
        printf("\nReset default QoS \n");
    }

    g_num_sl_nodes--;

    return 0;
}

int cpk_qos_fl_sched_create(adk_sys_handle_t sysHandle,
                            uint16_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t parent_node_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t enet_if_handle,
                            adk_interface_handle_t ip_if_handle)
{

#define QOS_PRIO_WFQ_QUEUES 7

    struct rte_tm_node_params node_params = {0};
    struct rte_tm_shaper_params shaper_profile;
    struct rte_tm_capabilities tm_capabilities;
    qos_sl_sched_t *sl_node_info_p;
    uint32_t shaper_profile_id;
    uint32_t shaper_profile_id_index;
    struct rte_tm_error tm_error;
    int *wfq_weights = NULL;
    uint32_t     tx_queue_id, queue_index;
    uint32_t     fl_node_id = QOS_FL_NODE_ID + g_num_fl_nodes;
    uint32_t     loop_idx = 0;
    uint32_t     priority;
    int ret;

    uint32_t wred_profile_id = vport_dev_id;
    struct rte_tm_wred_params profile = {
        .red_params = {
            /* Colors Green / Yellow / Red */
            [RTE_TM_GREEN] = {.min_th = 1024*1024, .max_th = 2*1024*1024, .maxp_inv = 10, .wq_log2 = 9},
            [RTE_TM_YELLOW] = {.min_th = 1024*1024, .max_th = 2*1024*1024, .maxp_inv =10, .wq_log2 = 9},
            [RTE_TM_RED] = {.min_th = 1024*1024, .max_th = 2*1024*1024, .maxp_inv = 10, .wq_log2 = 9},
        },
        .packet_mode = 0};

    /*Get the TM capabilities*/

    ret = rte_tm_capabilities_get(vport_dev_id, &tm_capabilities, &tm_error);
    if(ret) {
        printf("\n  rte_tm_capabilities_get failed \n");
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }


    if(tm_capabilities.sched_sp_n_priorities_max) {
        wfq_weights = malloc(sizeof(int) * tm_capabilities.sched_sp_n_priorities_max);
        for ( loop_idx = 0; loop_idx < tm_capabilities.sched_sp_n_priorities_max; loop_idx++) {
            memset(&wfq_weights[loop_idx], 1, sizeof(int));
        }
    }
 
    sl_node_info_p = get_sl_node_info_from_enet_if(enet_if_handle);
    if (NULL == sl_node_info_p) {
        printf("\n Error in fetching SL node details\n");
        return 1;
    }
    
    shaper_profile_id = QOS_FL_SHAPER_PROFILE_ID;

    if (!is_shaper_profile_configured(shaper_profile_id, &shaper_profile_id_index)) {

        /* Create a shaper profile with 1Gbps*/
        shaper_profile.mode = RTE_TM_SHAPER_MODE_BYTES;
        shaper_profile.committed.rate    = QOS_SHAPING_RATE_10G;
        shaper_profile.committed.size    = 4*1024;
        shaper_profile.peak.rate         = QOS_SHAPING_RATE_10G + QOS_SHAPING_RATE_1G; 
        shaper_profile.peak.size         = 4*1024;
        shaper_profile.pkt_length_adjust = RTE_PMD_ICE_DSI_TM_PKT_LEN_ADJ_FROM_L2_HDR;

        ret = rte_tm_shaper_profile_add(vport_dev_id, shaper_profile_id,
                &shaper_profile, &tm_error);

        if(ret) {
            printf("\n  rte_tm_shaper_profile_add failed \n");
            printf("\nTM error : %s\n",tm_error.message);
            return 1;
        }
        printf("\nAdded shaper profile for FL , id %d \n", shaper_profile_id);
        g_shaper_profiles_tbl[g_shaper_profile_count].profile_id = shaper_profile_id;
        g_shaper_profiles_tbl[g_shaper_profile_count].configured = true;
        g_shaper_profiles_tbl[g_shaper_profile_count].ref_cnt = 1;
        g_shaper_profile_count++;
    } else {
        g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt++;
    }

    wred_profile_id = fl_node_id;

    /* Create a WRED profile to be added to all queues */
    ret = rte_tm_wred_profile_add(vport_dev_id, wred_profile_id, &profile, &tm_error);
    if (ret) {
        printf("\nTM error : %s\n",tm_error.message);
        rte_panic("[%s()] rte_tm_wred_profile_add() failed"
                " for port %u, wred profile id %u with error %d\n",
                __func__, vport_dev_id, wred_profile_id, ret);
    } else {
        printf("WRED Profile created for port id %u\n", vport_dev_id);
    }

    /*Create a Second level scheduler */
    node_params.shaper_profile_id = shaper_profile_id;
    node_params.shared_shaper_id  = NULL;
    node_params.n_shared_shapers  = 0;
    node_params.nonleaf.wfq_weight_mode = wfq_weights;
    /*TODO: Get the max number of strict priorities supported by the PSM nodes and use it below*/
    node_params.nonleaf.n_sp_priorities = tm_capabilities.sched_sp_n_priorities_max;
    node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
            RTE_TM_STATS_N_BYTES |
            RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
            RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
            RTE_TM_STATS_N_PKTS_RED_DROPPED |
            RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
            RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
            RTE_TM_STATS_N_BYTES_RED_DROPPED |
            RTE_TM_STATS_N_PKTS_QUEUED |
            RTE_TM_STATS_N_BYTES_QUEUED);

    /* add a FL node for a SL scheduler*/
    ret = rte_tm_node_add(vport_dev_id,
            fl_node_id,
            parent_node_id,
            QOS_DEFAULT_PRIORITY,
            QOS_DEFAULT_WEIGHT,
            RTE_TM_NODE_LEVEL_ID_ANY,
            &node_params,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_add() failed to add vport L1 node .errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }

    printf("\nCreated FL node : 0x%x\n", fl_node_id);

    g_fl_map[g_num_fl_nodes].node_id = fl_node_id;
    g_fl_map[g_num_fl_nodes].l3_intf_handle = ip_if_handle;
    g_fl_map[g_num_fl_nodes].enet_intf_handle = enet_if_handle;
    g_fl_map[g_num_fl_nodes].shaper_profile.profile_id = shaper_profile_id;
    g_fl_map[g_num_fl_nodes].shaper_profile.configured = true;


    for (queue_index=0; queue_index< QOS_FL_NUM_QUEUES ; queue_index++) {

        /* Create 8 leaf nodes  */
        node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;
        node_params.shared_shaper_id  = NULL;
        node_params.n_shared_shapers  = 0;

        /*Queues - SP1, SP2, SP3, AF1, AF2, AF3, AF4, BE*/
        if ((queue_index >= QOS_FL_SP_NUM_QUEUES) && (queue_index < QOS_FL_DEFAULT_QUEUE_ID)) {
            node_params.leaf.cman = RTE_TM_CMAN_WRED;
            node_params.leaf.wred.wred_profile_id = wred_profile_id;
            node_params.leaf.wred.shared_wred_context_id = NULL;
            node_params.leaf.wred.n_shared_wred_contexts= 0;
        } else {
            node_params.leaf.cman = RTE_TM_CMAN_TAIL_DROP;
        }
        node_params.stats_mask = ( RTE_TM_STATS_N_PKTS |
                RTE_TM_STATS_N_BYTES |
                RTE_TM_STATS_N_PKTS_GREEN_DROPPED |
                RTE_TM_STATS_N_PKTS_YELLOW_DROPPED |
                RTE_TM_STATS_N_PKTS_RED_DROPPED |
                RTE_TM_STATS_N_BYTES_GREEN_DROPPED |
                RTE_TM_STATS_N_BYTES_YELLOW_DROPPED |
                RTE_TM_STATS_N_BYTES_RED_DROPPED |
                RTE_TM_STATS_N_PKTS_QUEUED |
                RTE_TM_STATS_N_BYTES_QUEUED);

        tx_queue_id = g_vsi_num_tx_queues++;

        /*Last 5 queues of the FL topology needs to scheduled in WFQ manner hence gives same priority*/
        priority = queue_index >= QOS_FL_SP_NUM_QUEUES ? QOS_PRIO_WFQ_QUEUES : queue_index;

        /* configure leaf-level node */
        ret = rte_tm_node_add(vport_dev_id,
                tx_queue_id,
                fl_node_id,
                priority,
                priority+1,
                RTE_TM_NODE_LEVEL_ID_ANY,
                &node_params,
                &tm_error);
        if (ret) {
            printf("rte_tm_node_add() failed to add queue %d leaf node."
                    " errno = %d\n", tx_queue_id, ret);
            printf("\nTM error : %s\n",tm_error.message);
            return 1;
        } else {
            printf("Created leaf queue 0x%x\n", tx_queue_id);
            g_fl_map[g_num_fl_nodes].tx_queues[queue_index] = tx_queue_id;
        }

        /*Register this PHB queue into QoS module*/
        adk_qos_phb_queue_register(sysHandle, tx_queue_id);

        ret = rte_eth_dev_tx_queue_start(ethdev_id, tx_queue_id);
        if (ret) {
            printf("rte_eth_dev_tx_queue_start() failed to start queue %u "
                    " errno = %d\n", tx_queue_id, ret);
            return 1;
        } else {
            printf("Started tx_queue 0x%x\n", tx_queue_id);
        }
    }

    sl_node_info_p->ip_if_handle[g_num_fl_nodes] = ip_if_handle;

    g_num_fl_nodes++;

    printf("\n");
    return 0;
}

int cpk_qos_fl_sched_delete(adk_sys_handle_t sysHandle,
                            uint16_t __attribute((unused)) vport_id,
                            uint16_t vport_dev_id,
                            uint16_t ethdev_id,
                            adk_interface_handle_t ip_if_handle)
{

#define QOS_PRIO_WFQ_QUEUES 7

    struct rte_tm_error tm_error;
    uint32_t     queue_index;
    uint32_t     fl_node_id = QOS_FL_NODE_ID + g_num_fl_nodes;
    uint32_t     shaper_profile_id_index = 0;
    qos_fl_sched_t *fl_node_info_p = 0;
    qos_sl_sched_t *sl_node_info_p;
    uint32_t index = 0;
    int ret;

    uint32_t wred_profile_id = vport_dev_id;
    fl_node_info_p = get_fl_node_info_from_ip_if(ip_if_handle);
    fl_node_id = fl_node_info_p->node_id;
    wred_profile_id = fl_node_id;

    for (queue_index=0; queue_index< QOS_FL_NUM_QUEUES ; queue_index++) {

        ret = rte_eth_dev_tx_queue_stop(ethdev_id, fl_node_info_p->tx_queues[queue_index]);
        if (ret) {
            printf("rte_eth_dev_tx_queue_stop() failed to stop queue %u "
                    " errno = %d\n", fl_node_info_p->tx_queues[queue_index], ret);
            return 1;
        } else {
            printf("Stopped tx_queue 0x%x\n", fl_node_info_p->tx_queues[queue_index]);
        }
        /*Register this PHB queue into QoS module*/
        adk_qos_phb_queue_unregister(sysHandle, fl_node_info_p->tx_queues[queue_index]);

        /* Delete leaf-level node */
        ret = rte_tm_node_delete(vport_dev_id,
                fl_node_info_p->tx_queues[queue_index],
                &tm_error);
        if (ret) {
            printf("rte_tm_node_delete() failed to delete queue %d leaf node."
                    " errno = %d\n", fl_node_info_p->tx_queues[queue_index], ret);
            printf("\nTM error : %s\n",tm_error.message);
            return 1;
        } else {
            printf("Delete leaf queue 0x%x\n", fl_node_info_p->tx_queues[queue_index]);
        }
    }

    /* Create a WRED profile to be added to all queues */
    ret = rte_tm_wred_profile_delete(vport_dev_id, wred_profile_id, &tm_error);
    if (ret) {
        printf("\nTM error : %s\n",tm_error.message);
        rte_panic("[%s()] rte_tm_wred_profile_delete() failed"
                " for port %u, wred profile id %u with error %d\n",
                __func__, vport_dev_id, wred_profile_id, ret);
    } else {
        printf("WRED Profile created for port id %u\n", vport_dev_id);
    }

    /* Delete a FL node for a SL scheduler*/
    ret = rte_tm_node_delete(vport_dev_id,
            fl_node_id,
            &tm_error);
    if (ret) {
        printf("rte_tm_node_delete() failed to delete vport L1 node .errno = %d\n", ret);
        printf("\nTM error : %s\n",tm_error.message);
        return 1;
    }

    printf("\nDeleted FL node : 0x%x\n", fl_node_id);

    if (is_shaper_profile_configured(QOS_FL_SHAPER_PROFILE_ID, &shaper_profile_id_index)) {

        g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt--;
        if (!g_shaper_profiles_tbl[shaper_profile_id_index].ref_cnt) {
        ret = rte_tm_shaper_profile_delete(vport_dev_id, QOS_FL_SHAPER_PROFILE_ID, &tm_error);

        if(ret) {
            printf("\n  rte_tm_shaper_profile_delete failed \n");
            printf("\nTM error : %s\n",tm_error.message);
            return 1;
        }
        printf("\nDeleted shaper profile for FL , id %d \n", QOS_FL_SHAPER_PROFILE_ID);
        g_shaper_profiles_tbl[shaper_profile_id_index].profile_id = 0;
        g_shaper_profiles_tbl[shaper_profile_id_index].configured = false;
        g_shaper_profile_count--;
        }
    }

    sl_node_info_p = get_sl_node_info_from_enet_if(fl_node_info_p->enet_intf_handle);
    if (NULL == sl_node_info_p) {
        printf("\n Error in fetching SL node details\n");
        return 1;
    }

    index = get_ip_if_handle_index_in_SL(sl_node_info_p, ip_if_handle);
    if (index != QOS_FL_MAX_NODES) {
        sl_node_info_p->ip_if_handle[index] = 0;
    }

    fl_node_info_p->node_id = 0;
    fl_node_info_p->l3_intf_handle = 0; 
    fl_node_info_p->shaper_profile.profile_id = 0;
    fl_node_info_p->shaper_profile.configured = false;

    g_num_fl_nodes--;

    printf("\n");
    return 0;
}

int cpk_qos_get_sl_sched_stats(uint16_t vport_dev_id,
                               adk_interface_handle_t enet_if_handle,
                               qos_non_leaf_node_stats_t *node_stats)
{
    struct rte_tm_error tm_error;
    struct rte_tm_node_stats stats;
    uint64_t stats_mask = 0;
    qos_sl_sched_t *sl_node_info_p;
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    qos_non_leaf_node_stats_t fl_node_stats;
    int ret;

    if (node_stats != NULL) {

        memset(node_stats, 0, sizeof(qos_non_leaf_node_stats_t));
        /*Read Second level node stats */
        sl_node_info_p = get_sl_node_info_from_enet_if(enet_if_handle);
        if ((NULL != sl_node_info_p) && (sl_node_info_p->node_id)) {
            for (loop_idx = 0; loop_idx < QOS_SL_NUM_QUEUES; loop_idx++) {
                /* Create leaf nodes  */
                tx_queue_id = sl_node_info_p->tx_queues[loop_idx];

                /* Get tm node stats */
                ret = rte_tm_node_stats_read(vport_dev_id,
                        tx_queue_id,
                        &stats,
                        &stats_mask,
                        0,
                        &tm_error);
                if (ret) {
                    printf("rte_tm_node_stats_read() failed errno = %d\n", ret);
                    printf("\nTM error : %s\n",tm_error.message);
                    return 1;
                }
                if (stats_mask & RTE_TM_STATS_N_PKTS) {
                    node_stats->txPkts += stats.n_pkts;
                }
                if (stats_mask & RTE_TM_STATS_N_BYTES) {
                    node_stats->txBytes += stats.n_bytes;
                }
            }
            /*Read all the FL scheduler stats part of this SL hierarchy*/
            for (loop_idx = 0; loop_idx < QOS_FL_MAX_NODES; loop_idx++) {
                if (sl_node_info_p->ip_if_handle[loop_idx]) {
                    if (!cpk_qos_get_fl_sched_stats(vport_dev_id, sl_node_info_p->ip_if_handle[loop_idx], &fl_node_stats)) {
                        node_stats->txPkts  += fl_node_stats.txPkts;
                        node_stats->txBytes += fl_node_stats.txBytes;
                    }
                }
            }
        }
        return 0;
    }
    return 1;
}

int cpk_qos_sl_stats_read(uint16_t vport_dev_id,
                          adk_interface_handle_t enet_if_handle)
{
    struct rte_tm_error tm_error;
    struct rte_tm_node_stats stats;
    uint64_t stats_mask = 0;
    qos_sl_sched_t *sl_node_info_p;
    qos_non_leaf_node_stats_t node_stats = {0};
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    int ret;

    /*Read Second level node stats */
    sl_node_info_p = get_sl_node_info_from_enet_if(enet_if_handle);
    if ((NULL != sl_node_info_p) && (sl_node_info_p->node_id)) {
        for (loop_idx = 0; loop_idx < QOS_SL_NUM_QUEUES; loop_idx++) {
            /* Create leaf nodes  */
            tx_queue_id = sl_node_info_p->tx_queues[loop_idx];

            /* Get tm node stats */
            ret = rte_tm_node_stats_read(vport_dev_id,
                    tx_queue_id,
                    &stats,
                    &stats_mask,
                    0,
                    &tm_error);
            if (ret) {
                printf("rte_tm_node_stats_read() failed errno = %d\n", ret);
                printf("\nTM error : %s\n",tm_error.message);
                return 1;
            }
            printf("\n-------------------------------------------------------- \n");
            printf("\nSecond level scheduler leaf queue %u stats : \n", tx_queue_id);
            if (stats_mask & RTE_TM_STATS_N_PKTS) {
                NON_ZERO_STATS(stats.n_pkts,"\nNumber of pkts:%lu \n",stats.n_pkts);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES) {
                NON_ZERO_STATS(stats.n_bytes,"\nNumber of bytes:%lu \n",stats.n_bytes);
            }
            if (stats_mask & RTE_TM_STATS_N_PKTS_QUEUED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_queued,"\nNumber of pkts queued :%lu \n",stats.leaf.n_pkts_queued);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_QUEUED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_queued,"\nNumber of bytes queued:%lu \n",stats.leaf.n_bytes_queued);
            }

            if (stats_mask & RTE_TM_STATS_N_PKTS_GREEN_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_GREEN],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_GREEN, stats.leaf.n_pkts_dropped[RTE_TM_GREEN]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_GREEN_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_GREEN],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_GREEN, stats.leaf.n_bytes_dropped[RTE_TM_GREEN]);
            }
            if (stats_mask & RTE_TM_STATS_N_PKTS_YELLOW_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_YELLOW],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_YELLOW, stats.leaf.n_pkts_dropped[RTE_TM_YELLOW]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_YELLOW_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_YELLOW],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_YELLOW, stats.leaf.n_bytes_dropped[RTE_TM_YELLOW]);
            }
            if (stats_mask & RTE_TM_STATS_N_PKTS_RED_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_RED],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_RED, stats.leaf.n_pkts_dropped[RTE_TM_RED]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_RED_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_RED],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_RED, stats.leaf.n_bytes_dropped[RTE_TM_RED]);
            }
            printf("\n-------------------------------------------------------- \n");

        }
        printf("\nSecond level scheduler node stats : \n");
        if (!cpk_qos_get_sl_sched_stats(vport_dev_id, enet_if_handle, &node_stats)) {
            NON_ZERO_STATS(node_stats.txPkts, "\nNumber of pkts:%lu \n", node_stats.txPkts);
            NON_ZERO_STATS(node_stats.txBytes,"\nNumber of bytes:%lu \n", node_stats.txBytes);
            printf("\n-------------------------------------------------------- \n");
        }else {
            printf("\nFailed to fetch Second level scheduler node stats\n");
        }
    }

    return 0;
}

int cpk_qos_get_fl_sched_stats(uint16_t vport_dev_id,
                              adk_interface_handle_t ip_if_handle,
                              qos_non_leaf_node_stats_t *node_stats)
{
    struct rte_tm_error tm_error;
    struct rte_tm_node_stats stats;
    struct rte_tm_node_stats non_leaf_node_stats = {0};
    uint64_t stats_mask = 0;
    qos_fl_sched_t *fl_node_info_p = 0;
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    int ret;


    if (node_stats != NULL) {
        memset(node_stats, 0 , sizeof(qos_non_leaf_node_stats_t));
        /*Read First level node stats */
        fl_node_info_p = get_fl_node_info_from_ip_if(ip_if_handle);
        if ((NULL != fl_node_info_p) && (fl_node_info_p->node_id)) {
            for (loop_idx = 0; loop_idx < QOS_FL_NUM_QUEUES; loop_idx++) {
                /* leaf nodes  */
                tx_queue_id = fl_node_info_p->tx_queues[loop_idx];

                /* Get tm node stats */
                ret = rte_tm_node_stats_read(vport_dev_id,
                        tx_queue_id,
                        &stats,
                        &stats_mask,
                        0,
                        &tm_error);
                if (ret) {
                    printf("rte_tm_node_stats_read() failed errno = %d\n", ret);
                    printf("\nTM error : %s\n",tm_error.message);
                    return 1;
                }
                if (stats_mask & RTE_TM_STATS_N_PKTS) {
                    non_leaf_node_stats.n_pkts += stats.n_pkts;
                    node_stats->txPkts += stats.n_pkts;
                }
                if (stats_mask & RTE_TM_STATS_N_BYTES) {
                    node_stats->txBytes += stats.n_bytes;
                }

            }
        }
        return 0;
    }
    return 1;
}

int cpk_qos_fl_stats_read(uint16_t vport_dev_id,
                          adk_interface_handle_t ip_if_handle)
{
    struct rte_tm_error tm_error;
    struct rte_tm_node_stats stats;
    struct rte_tm_node_stats non_leaf_node_stats = {0};
    uint64_t stats_mask = 0;
    qos_fl_sched_t *fl_node_info_p = 0;
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    int ret;

    /*Read First level node stats */

    fl_node_info_p = get_fl_node_info_from_ip_if(ip_if_handle);
    if ((NULL != fl_node_info_p) && (fl_node_info_p->node_id)) {
        printf("\nFL scheduler node 0x%x\n",fl_node_info_p->node_id);
        for (loop_idx = 0; loop_idx < QOS_FL_NUM_QUEUES; loop_idx++) {
            /* leaf nodes  */
            tx_queue_id = fl_node_info_p->tx_queues[loop_idx];

            /* Get tm node stats */
            ret = rte_tm_node_stats_read(vport_dev_id,
                    tx_queue_id,
                    &stats,
                    &stats_mask,
                    0,
                    &tm_error);
            if (ret) {
                printf("rte_tm_node_stats_read() failed errno = %d\n", ret);
                printf("\nTM error : %s\n",tm_error.message);
                return 1;
            }
           printf("\n-------------------------------------------------------- \n");
            printf("\nFirst level scheduler leaf queue %u stats : \n", tx_queue_id);
            if (stats_mask & RTE_TM_STATS_N_PKTS) {
               NON_ZERO_STATS(stats.n_pkts,"\nNumber of pkts:%lu \n",stats.n_pkts);
                non_leaf_node_stats.n_pkts += stats.n_pkts;
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES) {
                NON_ZERO_STATS(stats.n_bytes,"\nNumber of bytes:%lu \n",stats.n_bytes);
                non_leaf_node_stats.n_bytes+= stats.n_bytes;
            }
           if (stats_mask & RTE_TM_STATS_N_PKTS_QUEUED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_queued,"\nNumber of pkts queued :%lu \n",stats.leaf.n_pkts_queued);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_QUEUED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_queued,"\nNumber of bytes queued:%lu \n",stats.leaf.n_bytes_queued);
            }

            if (stats_mask & RTE_TM_STATS_N_PKTS_GREEN_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_GREEN],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_GREEN, stats.leaf.n_pkts_dropped[RTE_TM_GREEN]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_GREEN_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_GREEN],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_GREEN, stats.leaf.n_bytes_dropped[RTE_TM_GREEN]);
            }
            if (stats_mask & RTE_TM_STATS_N_PKTS_YELLOW_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_YELLOW],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_YELLOW, stats.leaf.n_pkts_dropped[RTE_TM_YELLOW]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_YELLOW_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_YELLOW],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_YELLOW, stats.leaf.n_bytes_dropped[RTE_TM_YELLOW]);
            }
            if (stats_mask & RTE_TM_STATS_N_PKTS_RED_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_pkts_dropped[RTE_TM_RED],"\nNumber of pkts dropped (color:%u):%lu \n", RTE_TM_RED, stats.leaf.n_pkts_dropped[RTE_TM_RED]);
            }
            if (stats_mask & RTE_TM_STATS_N_BYTES_RED_DROPPED) {
                NON_ZERO_STATS(stats.leaf.n_bytes_dropped[RTE_TM_RED],"\nNumber of bytes dropped (color:%u):%lu \n", RTE_TM_RED, stats.leaf.n_bytes_dropped[RTE_TM_RED]);
            }
            printf("\n-------------------------------------------------------- \n");

        }
        printf("\nFirst level scheduler node stats : \n");
        NON_ZERO_STATS(non_leaf_node_stats.n_pkts, "\nNumber of pkts:%lu \n",non_leaf_node_stats.n_pkts);
        NON_ZERO_STATS(non_leaf_node_stats.n_bytes,"\nNumber of bytes:%lu \n",non_leaf_node_stats.n_bytes);
        printf("\n-------------------------------------------------------- \n");

    }

    return 0;
}

int adk_qos_phb_stats_read(adk_sys_handle_t sysHandle, adk_interface_handle_t ip_if_handle)
{
    qos_fl_sched_t *fl_node_info_p = 0;
    adk_qos_phb_queue_statistics_t queueStats = {0};
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    int ret;

    fl_node_info_p = get_fl_node_info_from_ip_if(ip_if_handle);
    if ((NULL != fl_node_info_p) && (fl_node_info_p->node_id)) {
        for (loop_idx = 0; loop_idx < QOS_FL_NUM_QUEUES; loop_idx++) {
            /* PHB  queues */
            tx_queue_id = fl_node_info_p->tx_queues[loop_idx];

            memset(&queueStats, 0 , sizeof(adk_qos_phb_queue_statistics_t));

            /* Get ADK_PHB stats */
            ret = adk_qos_get_phb_queue_statistics(sysHandle,
                    tx_queue_id,
                    &queueStats);
            if (ret) {
                printf("\nFailed to fetch PHB queue stats\n");
                return -1;
            }
            printf("\n-------------------------------------------------------- \n");
            printf("\nPHB queue stats for queue %u stats : \n", tx_queue_id);
            PRINT_NON_ZERO("ipv4TxPkts %lu\n", queueStats.ipv4TxPkts);
            PRINT_NON_ZERO("ipv4TxBytes %lu\n", queueStats.ipv4TxBytes);
            PRINT_NON_ZERO("ipv4TxPktsDropped %lu\n", queueStats.ipv4TxPktsDropped);
            PRINT_NON_ZERO("ipv4TxBytesDropped %lu\n", queueStats.ipv4TxBytesDropped);
            PRINT_NON_ZERO("ipv6TxPkts %lu\n", queueStats.ipv6TxPkts);
            PRINT_NON_ZERO("ipv6TxBytes %lu\n", queueStats.ipv6TxBytes);
            PRINT_NON_ZERO("ipv6TxPktsDropped %lu\n", queueStats.ipv6TxPktsDropped);
            PRINT_NON_ZERO("ipv6TxBytesDropped %lu\n", queueStats.ipv6TxBytesDropped);
            printf("\n-------------------------------------------------------- \n");
        }
    }

    return 0;
}
int adk_qos_sl_queue_stats_read(adk_sys_handle_t sysHandle, adk_interface_handle_t enet_if_handle)
{
    qos_sl_sched_t *sl_node_info_p = 0;
    adk_qos_sl_queue_statistics_t queueStats = {0};
    uint32_t     tx_queue_id = 0;
    uint32_t     loop_idx = 0;
    int ret;

    sl_node_info_p = get_sl_node_info_from_enet_if(enet_if_handle);
    if ((NULL != sl_node_info_p) && (sl_node_info_p->node_id)) {
        for (loop_idx = 0; loop_idx < QOS_SL_NUM_QUEUES; loop_idx++) {
            /* PHB  queues */
            tx_queue_id = sl_node_info_p->tx_queues[loop_idx];

            memset(&queueStats, 0 , sizeof(adk_qos_sl_queue_statistics_t));

            /* Get ADK SL queue stats */
            ret = adk_qos_get_sl_queue_statistics(sysHandle,
                    tx_queue_id,
                    &queueStats);
            if (ret) {
                printf("\nFailed to fetch SL queue stats\n");
                return -1;
            }
            printf("\n-------------------------------------------------------- \n");
            printf("\nSL queue stats for queue %u stats : \n", tx_queue_id);
            PRINT_NON_ZERO("nonIPTxPkts %lu\n", queueStats.nonIPTxPkts);
            PRINT_NON_ZERO("nonIPTxBytes %lu\n", queueStats.nonIPTxBytes);
            PRINT_NON_ZERO("nonIPTxPktsDropped %lu\n", queueStats.nonIPTxPktsDropped);
            PRINT_NON_ZERO("nonIPTxBytesDropped %lu\n", queueStats.nonIPTxBytesDropped);
            PRINT_NON_ZERO("ipv4TxPkts %lu\n", queueStats.ipv4TxPkts);
            PRINT_NON_ZERO("ipv4TxBytes %lu\n", queueStats.ipv4TxBytes);
            PRINT_NON_ZERO("ipv4TxPktsDropped %lu\n", queueStats.ipv4TxPktsDropped);
            PRINT_NON_ZERO("ipv4TxBytesDropped %lu\n", queueStats.ipv4TxBytesDropped);
            PRINT_NON_ZERO("ipv6TxPkts %lu\n", queueStats.ipv6TxPkts);
            PRINT_NON_ZERO("ipv6TxBytes %lu\n", queueStats.ipv6TxBytes);
            PRINT_NON_ZERO("ipv6TxPktsDropped %lu\n", queueStats.ipv6TxPktsDropped);
            PRINT_NON_ZERO("ipv6TxBytesDropped %lu\n", queueStats.ipv6TxBytesDropped);
            printf("\n-------------------------------------------------------- \n");
        }
    }

    return 0;
}
int configure_ip_qos_classifier(adk_module_handle_t modHandle, adk_interface_handle_t ip_if_handle, bool isIPv6) {

    adk_ip_qos_info_t qosInfoIpv4 = {0};
    adk_ipv6_qos_info_t qosInfoIpv6 = {0};
    qos_fl_sched_t *fl_sched_info = NULL;
    adk_status_t retVal = 0;

    fl_sched_info = get_fl_node_info_from_ip_if(ip_if_handle);
    if (NULL == fl_sched_info) {
        return -1;
    }
   
    if (!isIPv6) {
        qosInfoIpv4.qosRule.ruleMask = ADK_QOS_CLASSIFIER_PROTOCOL_TYPE_MASK; 
        qosInfoIpv4.qosRule.protocol = 0x11;
        qosInfoIpv4.qosAction.queueId = fl_sched_info->tx_queues[4];
        qosInfoIpv4.qosAction.pktColor =  RTE_TM_YELLOW;

        retVal = adk_ip_qos_policy_add(modHandle, ip_if_handle, 0 , &qosInfoIpv4);
        if (retVal) {
            printf("\nFailed to add QoS classifier rule\n");
            return -1;
        }
        retVal = adk_ip_qos_policy_build(modHandle, ip_if_handle);
        if (retVal) {
            printf("\nFailed to build QoS classifier rule\n");
            return -1;
        }
    }else {
        qosInfoIpv6.qosRule.ruleMask = ADK_QOS_CLASSIFIER_PROTOCOL_TYPE_MASK; 
        qosInfoIpv6.qosRule.protocol = 0x11;
        qosInfoIpv6.qosAction.queueId = fl_sched_info->tx_queues[0];
        qosInfoIpv6.qosAction.pktColor =  RTE_TM_GREEN;
        retVal = adk_ipv6_qos_policy_add(modHandle, ip_if_handle, 0 , &qosInfoIpv6);
        if (retVal) {
            printf("\nFailed to add QoS classifier rule\n");
            return -1;
        }
        retVal = adk_ipv6_qos_policy_build(modHandle, ip_if_handle);
        if (retVal) {
            printf("\nFailed to build QoS classifier rule\n");
            return -1;
        }
    }
    return 0;
}

int configure_ip_ingress_qos_classifier(adk_module_handle_t modHandle, adk_interface_handle_t ip_if_handle, bool isIPv6) {

    adk_ip_ingress_qos_info_t qosInfoIpv4 = {0};
    adk_ipv6_ingress_qos_info_t qosInfoIpv6 = {0};
    qos_fl_sched_t *fl_sched_info = NULL;
    adk_status_t retVal = 0;

    fl_sched_info = get_fl_node_info_from_ip_if(ip_if_handle);
    if (NULL == fl_sched_info) {
        return -1;
    }
   
    if (!isIPv6) {
        qosInfoIpv4.dscp = 32; 
        qosInfoIpv4.phbClass = ADK_QOS_DS_PHB_CLASS_EF_2; 
        retVal = adk_ip_ingress_qos_policy_add(modHandle, ip_if_handle, &qosInfoIpv4);
        if (retVal) {
            printf("\nFailed to add QoS classifier rule\n");
            return -1;
        }
    }else {
        qosInfoIpv6.dscp = 32;
        qosInfoIpv6.phbClass = ADK_QOS_DS_PHB_CLASS_EF_2;
        retVal = adk_ipv6_ingress_qos_policy_add(modHandle, ip_if_handle, &qosInfoIpv6);
        if (retVal) {
            printf("\nFailed to add QoS classifier rule\n");
            return -1;
        }
    }
    return 0;
}
void print_policing_stats(adk_sys_handle_t sysHandle, adk_qos_policer_handle_t policing_handle)
{
    adk_qos_policing_flow_stats_t policerStats = {0};
    if (ADK_SUCCESS != adk_qos_policing_flow_stats_get(sysHandle, policing_handle, &policerStats)) {
        printf("\nFailed to fetch policer stats for policing handle %p\n", policing_handle);
    }
    
    PRINT_NON_ZERO("Green pkts  %lu\n", policerStats.green_pkts);
    PRINT_NON_ZERO("Yellow pkts %lu\n", policerStats.yellow_pkts);
    PRINT_NON_ZERO("Red pkts    %lu\n", policerStats.red_pkts);

}

void print_ip_routes(adk_module_handle_t ip_handle, uint8_t ipRouteTblId)
{
    uint32_t numRoutes=0;
    adk_ip_route_info_t *routeInfo = NULL;

    adk_ip_get_routes_all(ip_handle, ipRouteTblId, &numRoutes, &routeInfo);

    printf("---\nIPv4: Route-table %d :: found %d routes::\n", ipRouteTblId, numRoutes);
    for(uint32_t i=0;i<numRoutes;i++)
    {
        printf("dip=0x%x/%d :: is-connected=%d, l2-if-hd=%p, NH=0x%x, route-mtu=%d\n",
                routeInfo[i].destAddr.addr, routeInfo[i].destAddr.prefixWidth, routeInfo[i].connectedRoute,
                routeInfo[i].l2IfHd, routeInfo[i].nextHop, routeInfo[i].routeMTU);
    }
    printf("---\n");

    if(numRoutes)
        free(routeInfo);
}
/*This function will print info configured by adk_ip_pbr_add_policy API*/
adk_status_t ip_print_pbr_policies_all(adk_module_handle_t ipHandle)
{
    uint32_t *numPbrPolicies=(uint32_t *)calloc(1,sizeof(uint32_t));
    adk_ip_pbr_info_t *ipPbrPol_p;
    uint32_t cntr;
    adk_status_t retVal=ADK_SUCCESS;

    if(numPbrPolicies == NULL)
    {
	printf("Failed to allocate memory for number of policies");
	return ADK_E_INTERNAL_ERROR;
    }

    retVal =  adk_ip_get_pbr_policies_all(ipHandle,numPbrPolicies,&ipPbrPol_p);
    if(retVal != ADK_SUCCESS)
    {
	printf("ERROR!! adk_ip_get_pbr_policies_all failed");
	return retVal;
    }
    else
    {
	if(*numPbrPolicies == 0)
	    printf("\nNo PBR policies configured");
	else
	{
	    printf("\n==============Printing IPv4 PBR Policy/ies info============\n");

	    printf("\nFetched %u pbr policies",*numPbrPolicies);

	    for(cntr = 0 ; cntr < (*numPbrPolicies) ; cntr++)
	    {
		printf("\nPolicy no.-->%u: Rule mask:%d Addr:0x%x/%u Action:%hhu\n"
			,cntr
			,(ipPbrPol_p+cntr)->pbrRule.pbrRuleMask
			,(ipPbrPol_p+cntr)->pbrRule.srcAddrInfo.addr
			,(ipPbrPol_p+cntr)->pbrRule.srcAddrInfo.prefixWidth
			,(ipPbrPol_p+cntr)->pbrAction.routeTableId);
	    }
	}
    }
    free(ipPbrPol_p);
    return retVal;
}

/* This function will print the info configured by adk_ip_ingress_classifier_rule_add API*/
adk_status_t ip_print_ingress_classifier_all(adk_module_handle_t ipHandle)
{
    uint32_t *numClassifiers = (uint32_t *)calloc(1,sizeof(uint32_t));
    adk_ip_ingress_classifier_info_t *ingressClassifier_p;
    uint32_t cntr;
    adk_status_t retVal=ADK_SUCCESS;

    if (numClassifiers == NULL)
    {
	printf("\nFailed to allocate memory for numClassifiers\n");
	return ADK_E_INTERNAL_ERROR;
    }

    retVal = adk_ip_get_ingress_classifiers_all(ipHandle,numClassifiers,&ingressClassifier_p);
    if (retVal != ADK_SUCCESS)
    {
	printf("\nERROR!! adk_ip_get_ingress_classifiers_all failed\n");
	return retVal;
    }
    else
    {
	if (*numClassifiers == 0)
	    printf("\nNo ingress classifier rule configured\n");
	else
	{
	    printf("\n==============Printing IPv4 ingress classifier info============\n");
	    printf("\nFetched %u classifiers' info", *numClassifiers);
	    for (cntr = 0; cntr<(*numClassifiers); cntr++)
	    {
		printf("\nRule-->%u\n Rule::RuleMask:%d Protocol:%hhu  DIP:0x%x/%u SIP:0x%x/%u Interface Handle:%"ADKPRIh"\nIcmpTypeBegin:%u  IcmpTypeEnd:%u IcmpCodeBegin:%u IcmpCodeEnd:%u\ntcpSrcPortBegin:%u tcpSrcPortEnd:%u tcpDestportBegin:%u tcpDestPortEnd:%u \nudpSrcPortBegin:%u udpSrcPortEnd:%u udpDestportBegin:%u udpDestPortEnd:%u \nsctpSrcPortBegin:%u sctpSrcPortEnd:%u sctpDestportBegin:%u sctpDestPortEnd:%u \nAction::Action:%d, enablePolicing:%hhu , policingHandle:%"ADKPRIh",eventMachineQueueId:%u\n",
			cntr + 1,
			(ingressClassifier_p + cntr)->ClassifierRule.ruleMask,
			(ingressClassifier_p + cntr)->ClassifierRule.protocol,
			(ingressClassifier_p + cntr)->ClassifierRule.destAddr.addr,
			(ingressClassifier_p + cntr)->ClassifierRule.destAddr.prefixWidth,
			(ingressClassifier_p + cntr)->ClassifierRule.srcAddr.addr,
			(ingressClassifier_p + cntr)->ClassifierRule.srcAddr.prefixWidth,
			(ingressClassifier_p + cntr)->ClassifierRule.ipIntfHandle,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.icmp.type.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.icmp.type.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.icmp.code.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.icmp.code.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.tcp.srcPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.tcp.srcPort.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.tcp.destPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.tcp.destPort.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.udp.srcPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.udp.srcPort.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.udp.destPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.udp.destPort.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.sctp.srcPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.sctp.srcPort.end,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.sctp.destPort.begin,
			(ingressClassifier_p + cntr)->ClassifierRule.protoInfo_u.sctp.destPort.end,
			(ingressClassifier_p + cntr)->ClassifierAction.action,
			(ingressClassifier_p + cntr)->ClassifierAction.enablePolicing,
			(ingressClassifier_p + cntr)->ClassifierAction.policingHandle,
			(ingressClassifier_p + cntr)->ClassifierAction.actionAttr_u.eventMachineQueueId);
	    }
	}
    }
    free(ingressClassifier_p);
    return retVal;
}

/* This function will print the Egress QoS classifier info configured by adk_ip_qos_policy_add*/
adk_status_t ip_print_qos_policies_all(adk_module_handle_t ipHandle,adk_interface_handle_t ipIfHandle)
{
    adk_status_t retVal = ADK_SUCCESS;
    uint32_t cntr, *numQosPolicies = (uint32_t *)calloc(1,sizeof(uint32_t)) ;
    adk_ip_qos_info_t  *ipQosPol_p;

    if (numQosPolicies == NULL)
    {
	printf("\nFailed to allocate memory for numQosPolicies\n");
	return ADK_E_INTERNAL_ERROR;
    }

    retVal = adk_ip_get_qos_policies_all(ipHandle, ipIfHandle, numQosPolicies, &ipQosPol_p);
    if (retVal != ADK_SUCCESS)
    {
	printf("\nERROR!! adk_ip_get_qos_policies_all failed\n");
	return retVal;
    }
    else
    {
	if (*numQosPolicies == 0)
	    printf("\nNo QoS policy configured configured\n");
	else
	{
	    printf("\n==============Printing IPv4 egress QoS Policies============\n");
	    printf("\nFetched %u Egress QoS Policy/ies for interface %"ADKPRIh"\n", *numQosPolicies,ipIfHandle);

	    for (cntr = 0; cntr< *numQosPolicies; cntr++)
	    {
		printf("\nPolicy: %u\nRule:: RuleMask:%u, SIP type:%d, v4 SIP:0x%x/%u v6 SIP:0x%x%x%x%x/%u \nDIP type:%d v4 DIP:0x%x/%u v6 DIP:0x%x%x%x%x/%u protocol:%hhu dscp:%hhu\nAction::queueId:%hu color:%d",
			cntr + 1,
			(ipQosPol_p + cntr)->qosRule.ruleMask,
			(ipQosPol_p + cntr)->qosRule.srcAddr.type,
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v4Addr.addr,
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v4Addr.prefixWidth,
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v6Addr.addr[0],
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v6Addr.addr[1],
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v6Addr.addr[2],
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v6Addr.addr[3],
			(ipQosPol_p + cntr)->qosRule.srcAddr.addr_u.v6Addr.prefixWidth,
			(ipQosPol_p + cntr)->qosRule.destAddr.type,
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v4Addr.addr,
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v4Addr.prefixWidth,
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v6Addr.addr[0],
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v6Addr.addr[1],
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v6Addr.addr[2],
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v6Addr.addr[3],
			(ipQosPol_p + cntr)->qosRule.destAddr.addr_u.v6Addr.prefixWidth,
			(ipQosPol_p + cntr)->qosRule.protocol,
			(ipQosPol_p + cntr)->qosRule.dscp,
			(ipQosPol_p + cntr)->qosAction.queueId,
			(ipQosPol_p + cntr)->qosAction.pktColor
			    );
	    }
	}
    }

    free(ipQosPol_p);
    return retVal;
}
