/* **************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2017 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/

#include "adk_dp_app_config.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifdef SELECT_INTERFACE_ENABLED
/* Include files for poll signalling interface */
#include <signal.h>
#include <poll.h>
#endif

#include <rte_eventdev.h>
#include <rte_event_framework.h>
#include <rte_bridge_cppi.h>

#include "adk_ip.h"
#include "adk_ipv6.h"
#include "adk_ethernet.h"
#include "adk_sample_os.h"
#include "adk_sys.h"
#include "adk_fps.h"

#include "adk_ph.h"
#include "adk_ph_dev.h"
#include "adk_pal.h"

#include "adk_sample_ph.h"
#include "test_shm.h"

#define TRANSPORT_RX_RING_NAME "ADK_PH_DEV_DEVICE_RX_"
#define TRANSPORT_TX_RING_NAME "ADK_PH_DEV_DEVICE_TX_"

int ipv4NumExceptionsPhDev[ADK_PH_DEV_MAX_NUM_DEVICES];
int ipv6NumExceptionsPhDev[ADK_PH_DEV_MAX_NUM_DEVICES];

uint16_t                shutdownPhIndex_g = 0xffff;

test_config_index_t     exceptionRock[NUM_PH_THREADS];

uint32_t                simpleLock = 0;

void
cppi_stats_print(void)
{
    struct rte_cppi_queue_stats qStats;
    int                         devId;
    int                         j;

    printf("\nCPPI Stats:\n");

    for (j = 0; j < ADK_PH_DEV_MAX_NUM_DEVICES; j++)
    {
        if (0xff == testConfShm_p->phDev[j].devId)
        {
            continue;
        }

        devId = testConfShm_p->phDev[j].devId;

        if (0 < testConfShm_p->phDev[j].numRxQueues)
        {
#if RDK_REL_MAJOR >= 13
            rte_cppi_queue_stats_get(devId, 0, CPPI_RING_FQ, &qStats);
#else
            rte_bridge_cppi_get_stats(0, devId, 0, &qStats);
#endif
            printf("    FreeQ[%d:%d:%d]: enq: %4zd, proc: %4zd\n",
                   0, devId, 0, qStats.nb_requested, qStats.nb_processed);
        }

        if (0 < testConfShm_p->phDev[j].numTxQueues)
        {
#if RDK_REL_MAJOR >= 13
            rte_cppi_queue_stats_get(devId, 0, CPPI_RING_AQ, &qStats);
#else
            rte_bridge_cppi_get_stats(0, devId, 1, &qStats);
#endif
            printf("    FreeQ[%d:%d:%d]: enq: %4zd, proc: %4zd\n",
                   0, devId, 1, qStats.nb_requested, qStats.nb_processed);
        }
    }

    printf("\n");
}

adk_status_t
cp_init_ph_threads(
    int thread_1,
    int thread_2,
    int thread_3,
    int thread_4)
{
    adk_status_t retValue = ADK_SUCCESS;


    if (PH_DEV == thread_1)
    {
        retValue = init_ph_dev_thread(0, ADK_PH_DEV_TYPE_RING);
        if (retValue != ADK_SUCCESS)
        {
            printf("ADK PH init 0 Failed\n");
            return 1;
        }
        printf("ADK PH init 0 success\n");
    }

    if (PH_DEV == thread_2)
    {
        retValue = init_ph_dev_thread(1, ADK_PH_DEV_TYPE_RING);
        if (retValue != ADK_SUCCESS)
        {
            printf("ADK PH init 1 Failed\n");
            return 1;
        }
        printf("ADK PH init 1 success\n");
    }

    if (PH_DEV == thread_3)
    {
        retValue = init_ph_dev_thread(2, ADK_PH_DEV_TYPE_RING);
        if (retValue != ADK_SUCCESS)
        {
            printf("ADK PH Packet capture init 2 Failed\n");
            return 1;
        }
        printf("ADK PH init 2 success\n");
    }

    if (PH_DEV == thread_4)
    {
        retValue = init_ph_dev_thread(3, ADK_PH_DEV_TYPE_RING);
        if (retValue != ADK_SUCCESS)
        {
            printf("ADK PH Packet capture init 3 Failed\n");
            return 1;
        }
        printf("ADK PH init 3 success\n");
    }

    return retValue;
}

/* PH thread initialization. Application may prefer to keep
 * separate thread for IP packet processing.
 */

static uint32_t
call_back_fn_ipv4_ph_dev(
    uint16_t            devId,
    uint16_t            queueId,
    void               *pduMetaData_p,
    void               *pdu_p,
    void               *data)
{
    adk_ip_ph_dev_rx_param_t ipv4_cpu_params;
    adk_status_t                retValue = ADK_SUCCESS;
    uint32_t                    i;
    adk_ph_pdu_buf_rx_t        *task;

    RTE_SET_USED(queueId);
    RTE_SET_USED(pdu_p);
    RTE_SET_USED(data);


    printf("  {%s} Begin: Received IPv4 exception packet. CBFNID: %d\n",
           __func__, adk_ph_dev_cb_fn_id_get(pduMetaData_p));
    printf("  {%s} IPv4 exception packet pointer %p\n", __func__,
                pduMetaData_p);

    if ((IPV4_CBFNID != adk_ph_dev_cb_fn_id_get(pduMetaData_p)) &&
        (IPV4_PAKCAP_CBFNID != adk_ph_dev_cb_fn_id_get(pduMetaData_p)))
    {
        testConfShm_p->phDev[0].ipv4UnknownCbFnId++;
        printf("  {%s} Not IPv4 PDU based on Callback function ID\n", __func__);
        return 0;
    }

    retValue = test_shm_lookup(TEST_CONFIG_MEMORY_POOL,
                               (test_shm_handle_t **)&testConfShm_p);
    if (retValue != ADK_SUCCESS)
    {
        printf("  {%s} Shared memory for TEST_CONFIG_MEMORY_POOL failed\n",
               __func__);
        return 1;
    }

    task = (adk_ph_pdu_buf_rx_t *)pduMetaData_p;
    retValue = adk_ip_ph_dev_rx_param_decode(ip_handle, pduMetaData_p,
                                             &ipv4_cpu_params);
    if(retValue != ADK_SUCCESS)
    {
        printf("  {%s} failed to decode the exception \n", __func__);
    }

    printf("  {%s} PDU Params:", __func__);
    for(i=0; i<8; i++)
    {
        if ((i % 4) == 0)
        {
            printf("\n\t");
        }
        printf("0x%08"PRIxPTR"  ", task->pduParams[i]);
    }
    printf("\n");

    printf("___________IPv4 PH decoded info_________________\n");
    printf(" count              %d \n",   ++ipv4NumExceptionsPhDev[devId] );
    printf(" port               %d \n",              ipv4_cpu_params.port );
    printf(" exception_type     %d \n",    ipv4_cpu_params.exception_type );
    printf(" interface_id       %d \n",      ipv4_cpu_params.interface_id );
    printf(" timestamp_msb      %d \n",     ipv4_cpu_params.timestamp_msb );
    printf(" timestamp_lsb      %d \n",     ipv4_cpu_params.timestamp_lsb );
    printf(" ipv4_header_offset %d \n",  ipv4_cpu_params.ip_header_offset );
    printf(" vrd_id             %d \n",            ipv4_cpu_params.vrd_id );
    printf(" linux_intf_id      %d \n",     ipv4_cpu_params.linux_intf_id );
    printf(" context            %d (0x%04x)\n",   ipv4_cpu_params.context,
                                                  ipv4_cpu_params.context );
    printf("________________________________________________\n");


#ifdef ADK_DEBUG
    printf("\n");
    printf("  {%s} pduMetaData_p: %p, pdu_p: %p\n", __func__, pduMetaData_p,
           pdu_p);
    for (i = 0; i < ((struct rte_mbuf *)pduMetaData_p)->pkt_len; i++)
    {
        if (0 == (i % 16))
        {
            printf("\n%08X:", i);
        }

        printf(" %02X", ((uint8_t *)pdu_p)[i]);
    }
    printf("\n");
#endif
    printf("  {%s} End\n", __func__);

    return 0;
}

static uint32_t
call_back_fn_ipv6_ph_dev(
    uint16_t            devId,
    uint16_t            queueId,
    void               *pduMetaData_p,
    void               *pdu_p,
    void               *data)
{
    adk_ipv6_ph_dev_rx_param_t ipv6_params;
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t i;
    adk_ph_pdu_buf_rx_t        *task;
    uint16_t    cbFnId;

    RTE_SET_USED(queueId);
    RTE_SET_USED(pdu_p);
    RTE_SET_USED(data);


    cbFnId = adk_ph_dev_cb_fn_id_get(pduMetaData_p);
/*    printf("  {%s} Begin: Received IPv6 exception packet. CBFNID: %d\n",
           __func__, cbFnId);
    printf("  {%s} IPv6 exception packet pointer %p\n", __func__,
                pduMetaData_p); */

    if ((IPV6_CBFNID != cbFnId) &&
        (IPV6_PAKCAP_CBFNID != cbFnId))
    {
        testConfShm_p->phDev[0].ipv6UnknownCbFnId++;
/*         printf("  {%s} Not IPv6 PDU based on Callback function ID: %d\n",
               __func__, cbFnId); */
        return 0;
    }

    retValue = test_shm_lookup(TEST_CONFIG_MEMORY_POOL,
                               (test_shm_handle_t **)&testConfShm_p);
    if (retValue != ADK_SUCCESS)
    {
        printf("  {%s} Shared memory for TEST_CONFIG_MEMORY_POOL failed\n",
               __func__);
        return 1;
    }


    task = (adk_ph_pdu_buf_rx_t *)pduMetaData_p;
    retValue = adk_ipv6_ph_dev_rx_param_decode(ipv6_handle, pduMetaData_p,
                                            &ipv6_params);
    if(retValue != ADK_SUCCESS)
    {
        printf("  {%s} failed to decode the exception \n", __func__);
    }

    printf("  {%s} PDU Params:", __func__);
    for(i=0; i<8; i++)
    {
        if ((i % 4) == 0)
        {
            printf("\n\t");
        }
        printf("0x%08"PRIxPTR"  ", task->pduParams[i]);
    }
    printf("\n");

    printf("___________IPv6 PH decoded info_________________\n");
    printf(" count              %d \n", ++ipv6NumExceptionsPhDev[devId] );
    printf(" port               %d \n",              ipv6_params.port );
    printf(" exception_type     %d \n",    ipv6_params.exception_type );
    printf(" interface_id       %d \n",      ipv6_params.interface_id );
    printf(" timestamp_msb      %d \n",     ipv6_params.timestamp_msb );
    printf(" timestamp_lsb      %d \n",     ipv6_params.timestamp_lsb );
    printf(" ipv6_header_offset %d \n",ipv6_params.ipv6_header_offset );
    printf(" vrd_id             %d \n",            ipv6_params.vrd_id );
    printf(" linux_intf_id      %d \n",     ipv6_params.linux_intf_id );
    printf(" capture context    %d (0x%04x)\n",   ipv6_params.context,
                                                  ipv6_params.context );
    printf("________________________________________________\n");


#ifdef ADK_DEBUG
    printf("  {%s} pduMetaData_p: %p, pdu_p: %p\n", __func__, pduMetaData_p,
           pdu_p);
    printf("\n");
    for (i = 0; i < ((struct rte_mbuf *)pduMetaData_p)->pkt_len; i++)
    {
        if (0 == (i % 16))
        {
            printf("\n%08X:", i);
        }

        printf(" %02X", ((uint8_t *)pdu_p)[i]);
    }
    printf("\n");
#endif
    printf("  {%s} End\n", __func__);

    return 0;
}

static void *
adk_ph_dev_exception(void *data)
{
#define MAX_RX_PACKETS  16
    adk_status_t        retValue = ADK_SUCCESS;
    int                 phIndex;
    int                 queueId;
    int                 i;
    uint16_t            numPktsRecv = 0;
    test_conf_ph_dev_t *phDev_p;
#ifdef ENABLE_TX
    int16_t             numPktsXmit = 0;
#endif
    struct rte_mbuf    *recvBuffer_p[MAX_RX_PACKETS];
    test_config_index_t    *configData_p;
    adk_ip_pktcap_status_t  ipv4PktCapStatus;
    adk_ipv6_pktcap_status_t    ipv6PktCapStatus;
#ifdef SELECT_INTERFACE_ENABLED
    int             ret;
    struct pollfd   waitFds[3];
    uint8_t         dpMsg[1024*ADK_EVENT_SIZE];
    int             numFds;
#endif

    if (0 != adk_ph_dev_dp_instance_init())
    {
        printf("failed to init PH DP\n");
        return(NULL);
    }

    configData_p = (test_config_index_t *)data;
    phIndex = configData_p->phIndex;
    printf("  {%s} Begin for PH Dev thread %d\n", __func__, phIndex);


    queueId = -1;

    phDev_p = &(testConfShm_p->phDev[phIndex]);

#ifdef SELECT_INTERFACE_ENABLED
    waitFds[0].fd = phDev_p->dpToCpPipe_fds[CP_READ_FD];
    waitFds[0].events = POLLIN | POLLERR | POLLHUP | POLLNVAL;
    waitFds[0].revents = 0;
#endif

    simpleLock = 1;

    while(TRUE)
    {
        if (queueId >= (phDev_p->numRxQueues - 1))
        {
            queueId = 0;
        } else
        {
            queueId = 0; /*queueId + 1;*/
        }


        if (shutdownPhIndex_g == phIndex)
        {
            shutdownPhIndex_g = 0xffff;
            printf("  {%s} Exiting\n", __func__);
            break;
        }
#ifdef SELECT_INTERFACE_ENABLED
        if (phDev_p->interruptEnabled == true)
        {
            ret = poll(waitFds, (nfds_t) 1, ADK_POLL_NO_TIMEOUT);
            if (ret < 0)
            {
                phDev_p->phPollFail++;
                printf("  {%s} poll failed %d/%d '%s'\n",
                       __func__, ret, errno, strerror(errno));
/*                 sleep(5); */
/*                 sleep(0); */
                continue;
            } else if (ret == 0)
            {
                printf("  {%s} poll returned %d -- timeout\n", __func__, ret);
                sleep(5);
                continue;
            }
/*             printf("  {%s} poll:  numFd: %d '%s' %d\n",
                   __func__, ret, strerror(errno), errno); */
            phDev_p->phPoll++;
            numFds = ret;

            for (i = 0; i < numFds; i++)
            {
                if (POLLIN == (waitFds[i].revents & POLLIN))
                {
                    /*while (0 < read(waitFds[i].fd, dpMsg, 1))*/
                    ret =  read(waitFds[i].fd, dpMsg, 1);
                    if (-1 == ret)
                    {
                        phDev_p->phIntrFail++;

                        printf("  {%s} poll returned %d/%d '%s'\n",
                               __func__, ret, errno, strerror(errno));
                        continue;
                    }
                    phDev_p->phIntr += ret;
                    printf("  {%s} Pipe[%d] recieved: num: %d, msg: 0x%02x\n",
                           __func__, i, ret, dpMsg[0]);

                    if (dpMsg[0] == 2)
                    {
                        shutdownPhIndex_g = 0xffff;
                        printf("Got thread shutdown message\n");
                        goto _return;
                    }
                }
            }
        }
#endif
        if (phDev_p->devId == 0xff)
        {
            continue;
        }
        numPktsRecv = adk_ph_dev_rx_burst(phDev_p->devId, queueId,
                                          &(recvBuffer_p[0]), 1);
        if ((int16_t)numPktsRecv > MAX_RX_PACKETS)
        {
            rte_panic("  phIndex[%d]: {%s} recived too many (%d) packets",
                      phIndex, __func__, numPktsRecv);
        } else if ((int16_t)numPktsRecv < 0)
        {
            printf("  {%s} rx burst failed: %d\n", __func__, numPktsRecv);
            continue;
        }

/*         printf("  phIndex[%d]:  {%s} received %d out of %d PDU with FNCBID %d\n",
               phIndex, __func__,   numPktsRecv, MAX_RX_PACKETS,
               adk_ph_dev_cb_fn_id_get(recvBuffer_p[0])); */

        if (numPktsRecv == 0)
        {
            //sleep(0);
            continue;
        } else
        {
            for (i = 0; i < NUM_NIC_PORTS; i += 1)
            {
                if (NULL != phDev_p->ipv4PktCapHandle[i])
                {
                    adk_ip_packet_capture_status_get(ip_handle,
                                                phDev_p->ipv4PktCapHandle[i],
                                                &ipv4PktCapStatus);
                    if ( ! ipv4PktCapStatus.is_enabled)
                    {
                        phDev_p->ipv4PktCapReenable[i]++;

                        /* The capturing has been stopped autonomously by ADK
                         * as the PH Dev Queue was full.
                         * Enable the packet capture if needed */
                        printf("IPv4 Packet capture %d is disabled and "
                               "enabling it  phIndex[%d]\n", i, phIndex);
                        retValue = adk_ip_packet_capture_enable(ip_handle,
                                                phDev_p->ipv4PktCapHandle[i]);
                        if (ADK_SUCCESS != retValue)
                        {
                            printf("  {%s} IPv4 packet capture enable failed\n",
                                   __func__);
                        }
                    }
                }
                if (NULL != phDev_p->ipv6PktCapHandle[i])
                {
                    adk_ipv6_packet_capture_status(ipv6_handle,
                                                   phDev_p->ipv6PktCapHandle[i],
                                                   &ipv6PktCapStatus);
                    if ( ! ipv6PktCapStatus.is_enabled)
                    {
                        phDev_p->ipv6PktCapReenable[i]++;

                        /* The capturing has been stopped autonomously by ADK
                         * as the PH Dev Queue was full.
                         * Enable the packet capture if needed */
                        printf("IPv6 Packet capture %d is disabled and "
                               "enabling it  phIndex[%d]\n", i, phIndex);
                        retValue = adk_ipv6_packet_capture_enable(ipv6_handle,
                                                phDev_p->ipv6PktCapHandle[i]);
                        if (ADK_SUCCESS != retValue)
                        {
                            printf("  {%s} IPv6 packet capture enable failed\n",
                                   __func__);
                        }
                    }
                }
            }

            /* sleep(5); */
        }

#ifdef ENABLE_TX
        if (adk_ph_dev_cb_fn_id_get(recvBuffer_p[0]) == IPV4_CBFNID)
        {

            numPktsXmit = adk_ph_dev_tx_burst(phDev_p->devId,
                                              queueId, &recvBuffer_p[0],
                                              numPktsRecv);
            if ((numPktsXmit < 0) || (numPktsXmit > 1))
            {
                printf(" {%s} Transmit failed, (%d)\n", __func__, -numPktsXmit);
            } else
            {
                printf(" {%s} Sent 1 PDU \n", __func__);
            }
        }
#endif
        for (i = 0; i < numPktsRecv; i++)
        {
#ifdef ADK_DEBUG
            printf("\n  phIndex[%d]: __________________RXED PKT______________\n",
                   phIndex);
            rte_pktmbuf_dump(stdout, recvBuffer_p[i], recvBuffer_p[i]->pkt_len);
            printf("\n  phIndex[%d]: ________________________________________\n",
                   phIndex);
#endif
        }

        for (i = 0; i < numPktsRecv; i++)
        {
/*             printf("  {%s} freeing pkt[%d]: %p\n", __func__, i,
                        recvBuffer_p[i]); */
            retValue = adk_ph_dev_buffer_free(phDev_p->devId,
                                              recvBuffer_p[i]);
            if (ADK_SUCCESS != retValue)
            {
                printf("  phIndex[%d]:  {%s} mbuf free failed (%x)\n",
                    phIndex, __func__, retValue);
            }
        }
    }

#ifdef SELECT_INTERFACE_ENABLED
_return:
#endif
    printf("  phIndex[%d]:  {%s} End\n", phIndex, __func__);

    testConfShm_p->retValue = retValue;

    return(data);
}

static void
ph_dev_thread_shutdown(uint8_t phIndex)
{
    int         loopCount = 0;
#ifdef SELECT_INTERFACE_ENABLED
    uint8_t     fdMesg[ADK_EVENT_SIZE];
    int         controlFd;
    int         ret;
#endif

    while ((shutdownPhIndex_g != 0xffff) &&  (loopCount++ < 10))
    {
        sleep(1);
    }

    if (loopCount == 10)
    {
        printf("  {%s} Thread shutdown failed to aquire semaphore\n", __func__);
    }

    shutdownPhIndex_g = phIndex;

#ifdef SELECT_INTERFACE_ENABLED
    /* Send a message to wake up thread */
    controlFd = testConfShm_p->phDev[phIndex].dpToCpPipe_fds[DP_WRITE_FD];
    fdMesg[0] = 2;
    ret = write(controlFd, fdMesg, ADK_EVENT_SIZE);
    if (ret < 0)
    {
        printf("  {%s} poll failed %d/%d '%s'\n",
               __func__, ret, errno, strerror(errno));
    } else if (ret == 0)
    {
        printf("  {%s} poll returned %d -- timeout\n", __func__, ret);
        sleep(5);
    }
#endif


    loopCount = 0;
    while ((shutdownPhIndex_g != 0xffff) &&  (loopCount++ < 10))
    {
        sleep(1);
    }

    if (loopCount == 10)
    {
        printf("  {%s} Thread shutdown failed to reset semaphore\n", __func__);
    }

    shutdownPhIndex_g = 0xffff;

    printf("  {%s} phIndex (%d) thread stopped\n", __func__, phIndex);
}

adk_status_t
ph_dev_cleanup(uint8_t phIndex)
{
    adk_status_t    retValue = ADK_SUCCESS;
    int             i, j;
    uint16_t        devId;
    test_conf_ph_dev_t *phDev_p;

    phDev_p = &(testConfShm_p->phDev[phIndex]);
    devId = phDev_p->devId;

    printf("  {%s} Cleaning up PH Dev index: %d, devId: %d\n", __func__,
           phIndex, devId);

    if (0xff == phDev_p->devId)
    {
        printf("  {%s} DevId %d is not PHv3\n", __func__, devId);
        goto _return;
    }

    retValue = adk_ph_dev_device_stop(devId);
    if (retValue != ADK_SUCCESS)
    {
        printf("  {%s} Failed to stop PH Dev %d\n", __func__, devId);
        goto _return;
    }

    for (i = 0; i < ADK_PH_DEV_MAX_NUM_QUEUES; i++)
    {
        if (0 != phDev_p->txCbFnHandle[i])
        {
            for (j = 0; j < ADK_PH_DEV_MAX_NUM_QUEUES; j += 1)
            {
                adk_ph_dev_tx_callback_remove(devId, i,
                                              phDev_p->txCbFnHandle[i]);
                if (retValue != ADK_SUCCESS)
                {
                    printf("  {%s} Failed to remove Tx Cb Fn %d\n", __func__, i);
                    goto _return;
                }
                phDev_p->txCbFnHandle[i] = 0;
            }
        }
    }

    for (i = 0; i < phDev_p->numTxQueues; i++)
    {
        retValue = adk_ph_dev_tx_queue_remove(devId, i);
        if (retValue != ADK_SUCCESS)
        {
            printf("  {%s} Failed to remove Tx Q %d\n", __func__, i);
            goto _return;
        }
        phDev_p->txQueueId[i] = 0xffff;

        if (NULL != phDev_p->txRing[i])
        {
            rte_ring_free(phDev_p->txRing[i]);
            phDev_p->txRing[i] = NULL;
        }
    }

    for (i = 0; i < ADK_PH_DEV_MAX_NUM_QUEUES; i++)
    {
        if (0 != phDev_p->rxCbFnHandle[i])
        {
            for (j = 0; j < ADK_PH_DEV_MAX_NUM_QUEUES; j += 1)
            {
                adk_ph_dev_rx_callback_remove(devId, i,
                                              phDev_p->rxCbFnHandle[j]);
                if (retValue != ADK_SUCCESS)
                {
                    printf("  {%s} Failed to remove Rx Cb Fn %d\n", __func__, i);
                    goto _return;
                }
                phDev_p->rxCbFnHandle[i] = 0;
            }
        }
    }

    for (i = 0; i < phDev_p->numRxQueues; i++)
    {
        printf ("  {%s} Rx queue remove %d\n", __func__, i);
        retValue = adk_ph_dev_rx_queue_remove(devId, i);
        if (retValue != ADK_SUCCESS)
        {
            printf("  {%s} Failed to remove Rx Q %d\n", __func__, i);
            goto _return;
        }
        phDev_p->rxQueueId[i] = 0xffff;

        if (NULL != phDev_p->rxRing[i])
        {
            printf ("  {%s} freeing ring %d %p\n", __func__, i,
                    phDev_p->rxRing[i]);
            rte_ring_free(phDev_p->rxRing[i]);
            phDev_p->rxRing[i] = NULL;
        }
    }

    retValue = adk_ph_dev_device_close(devId);
    if (retValue != ADK_SUCCESS)
    {
        printf("  {%s} Failed to close PH Dev %d\n", __func__, devId);
        goto _return;
    }

    ph_dev_thread_shutdown(phDev_p->devId);

    phDev_p->devId = 0xff;

_return:
    return retValue;
}


int
init_ph_dev_thread(uint8_t phIndex, adk_ph_dev_device_type_t type)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    int                             ret;
    adk_sample_thread_id_t          threadId ;
    char                            threadName[26];
    int                             numCbFns = 0;
    adk_ph_dev_device_config_t      devConf;
    adk_ph_dev_rx_queue_config_t    rxQueueConf;
#ifdef ENABLE_TX
    adk_ph_dev_tx_queue_config_t    txQueueConf;
#endif
    uint16_t                        devId;
    uint8_t                         queueId;
    uint8_t                         cbFnIndex;
    struct rte_ring                *ring_p;
    char                            ring_name[RTE_RING_NAMESIZE];
    test_conf_ph_dev_t              phDev;
    int                             i;
#ifdef SELECT_INTERFACE_ENABLED
    int                             pipe;
    char                            pipeName[sizeof(MESSAGE_PIPE)+10];
#endif

    simpleLock = 0;

    retValue = test_shm_lookup(TEST_CONFIG_MEMORY_POOL,
                               (test_shm_handle_t **)&testConfShm_p);
    if (retValue != ADK_SUCCESS)
    {
        printf("Shared memory for TEST_CONFIG_MEMORY_POOL failed\n");
        goto init_ph_thread_return;
    }

    testConfShm_p->numDevices++;

    printf("testConfShm_p 4: %p, phIndex: %d\n", testConfShm_p, phIndex);

#ifdef SELECT_INTERFACE_ENABLED
    /*
     * Open the named pipe/fifo for message passing between
     * processes. This is the CP read side of the pipe.
     */
    memset(&pipeName, 0x00, sizeof(pipeName));
    snprintf(pipeName, sizeof(MESSAGE_PIPE)+8, MESSAGE_PIPE"_%u", phIndex);
    pipeName[sizeof(MESSAGE_PIPE)+9] = '\000';
    pipe = open(pipeName, (O_RDONLY | O_NONBLOCK));
    if (-1 == pipe)
    {
        rte_panic("  {%s} Unable to open CP Pipe[%d] '%s', '%s' (%d)\n",
                  __func__, phIndex, pipeName, strerror(errno), errno);
    }
    printf("  {%s} Opened pipe: %s\n", __func__, pipeName);
    testConfShm_p->phDev[phIndex].dpToCpPipe_fds[CP_READ_FD] = pipe;
#endif

    /* Fill the pipeline related data */
    memset(&phDev, 0, sizeof(test_conf_ph_dev_t));
    phDev.rxCbFn[numCbFns] = (void *)call_back_fn_ipv6_ph_dev;
    phDev.txCbFn[numCbFns] = NULL;
    numCbFns++;

    phDev.rxCbFn[numCbFns] = (void *)call_back_fn_ipv4_ph_dev;
    phDev.txCbFn[numCbFns] = NULL;
    numCbFns++;

    phDev.rxCbFn[numCbFns] = (void *)call_back_fn_ipv4_ph_dev;
    phDev.txCbFn[numCbFns] = NULL;
    numCbFns++;

    phDev.numRxQueues++;
#ifdef ENABLE_TX
    phDev.numTxQueues = 1;
#endif

    memset(&devConf, 0, sizeof(adk_ph_dev_device_config_t));

    devConf.numRxQueues = phDev.numRxQueues;
    devConf.numTxQueues = phDev.numTxQueues;
    devConf.memPool_p = testConfShm_p->packet_pool;
#ifdef SELECT_INTERFACE_ENABLED
    testConfShm_p->phDev[phIndex].interruptEnabled = true;
    devConf.interruptEnabled = testConfShm_p->phDev[phIndex].interruptEnabled;
    devConf.controlFd = testConfShm_p->phDev[phIndex].dpToCpPipe_fds[DP_WRITE_FD];
#endif
    devConf.type = type;
    if (ADK_PH_DEV_TYPE_EVENTDEV == type)
    {
        devConf.u.evd.portId = testConfShm_p->phDev[phIndex].portId;
    } else if (ADK_PH_DEV_TYPE_RING == type)
    {
        devConf.u.rng.portId = testConfShm_p->phDev[phIndex].portId;
    } else
    {
        rte_panic("  {%s} Invalid PH dev type %d\n", __func__, type);
    }

    if (phIndex == 3)
    {
        devConf.coalesceEnabled = false;
    }

    retValue = adk_ph_dev_device_configure(&devConf, &devId);
    if (ADK_SUCCESS != retValue)
    {
        rte_panic("  {%s} Unable to configure PH Dev device\n", __func__);
    }

    for (i = 0; i < numCbFns; i++)
    {
        testConfShm_p->phDev[phIndex].rxCbFn[i] = phDev.rxCbFn[i];
        testConfShm_p->phDev[phIndex].txCbFn[i] = phDev.txCbFn[i];
    }

    testConfShm_p->phDev[phIndex].devId = devId;
    testConfShm_p->phDev[phIndex].numRxQueues = phDev.numRxQueues;
    testConfShm_p->phDev[phIndex].numTxQueues = phDev.numTxQueues;
    testConfShm_p->phDev[phIndex].numCallbackFns = numCbFns;

    printf("  {%s}: portId: %d, rxQueueId[0]: %d, txQueueId[0]: %d\n",
           __func__,
           testConfShm_p->phDev[phIndex].devId,
           testConfShm_p->phDev[phIndex].rxQueueId[0],
           testConfShm_p->phDev[phIndex].txQueueId[0]);

    for (queueId = 0; queueId < devConf.numRxQueues; queueId++)
    {
        if (ADK_PH_DEV_TYPE_EVENTDEV == type)
        {
            rxQueueConf.u.evd.queueId = testConfShm_p->phDev[phIndex].rxQueueId[queueId];
        } else if (ADK_PH_DEV_TYPE_RING == type)
        {
            memset(&rxQueueConf, 0, sizeof(adk_ph_dev_rx_queue_config_t));

            memset(ring_name, 0, sizeof(char) * RTE_RING_NAMESIZE);
            snprintf(ring_name, RTE_RING_NAMESIZE,
                     TRANSPORT_RX_RING_NAME"%u_%u_%u",
                     phIndex, devId, queueId);

            ring_p = rte_ring_create(ring_name, 256, rte_socket_id(), 0);
            if (ring_p == NULL)
            {

                rte_panic("  {%s} Failed to create ring FAILED %s \n",
                          __func__, rte_strerror(rte_errno));
            }

            testConfShm_p->phDev[phIndex].rxRing[queueId] = ring_p;

            strncpy(rxQueueConf.u.rng.name, ring_name, RTE_RING_NAMESIZE);

            rxQueueConf.u.rng.name[RTE_RING_NAMESIZE - 1] = '0';
        }

        retValue = adk_ph_dev_rx_queue_setup(devId, queueId, &rxQueueConf);
        if (retValue != ADK_SUCCESS)
        {
            printf("  {%s} Rx queue setup failed: (%d)", __func__, retValue);
        }

        for (cbFnIndex = 0;
             cbFnIndex < numCbFns;
             cbFnIndex++)
        {

            retValue = adk_ph_dev_rx_callback_add(devId, queueId,
                            testConfShm_p->phDev[phIndex].rxCbFn[cbFnIndex],
                            NULL,
                            &(testConfShm_p->phDev[phIndex].rxCbFnHandle[cbFnIndex]));
            if ((retValue != ADK_SUCCESS) &&
                ((ADK_STATUS_ERROR_CODE_GET(retValue)) != ADK_W_CBFN_EXISTS))
            {
                printf ("  {%s} Failed to register callback fn %d. 0x%x\n",
                        __func__, cbFnIndex, retValue);
                return 1;
            } else if ((ADK_STATUS_ERROR_CODE_GET(retValue)) != ADK_W_CBFN_EXISTS)
            {
                testConfShm_p->phDev[phIndex].rxCbFnHandle[cbFnIndex] = 0;
            }
            printf("  {%s} Successfully registered callback fn %d\n\n",
                   __func__, cbFnIndex);
        }
    }

#ifdef ENABLE_TX
    for (queueId = 0; queueId < devConf.numTxQueues; queueId++)
    {
        if (ADK_PH_DEV_TYPE_EVENTDEV == type)
        {
            txQueueConf.u.evd.queueId = testConfShm_p->phDev[phIndex].txQueueId[queueId];

            retValue = adk_ph_dev_tx_queue_setup(devId, queueId, &txQueueConf);
            if (retValue != ADK_SUCCESS)
            {
                rte_panic("  {%s} Tx queue setup failed: (%d)", __func__,
                          retValue);
            }
        } else if (ADK_PH_DEV_TYPE_RING == type)
        {
#if 1
            printf("[%s] Tx over RINGs isn't supported\n", __func__);
#else
            memset(&txQueueConf, 0, sizeof(adk_ph_dev_tx_queue_config_t));

            memset(ring_name, 0, sizeof(char) * RTE_RING_NAMESIZE);
            snprintf(ring_name, RTE_RING_NAMESIZE,
                     TRANSPORT_TX_RING_NAME"%u_%u",
                     phIndex, queueId);

            ring_p = rte_ring_create(ring_name, 256, rte_socket_id(), 0);
            if (ring_p == NULL)
            {
                rte_panic("  {%s} Failed to create ring FAILED\n", __func__);
            }

            testConfShm_p->phDev[phIndex].txRing[queueId] = ring_p;

            strncpy(txQueueConf.u.rng.name, ring_name, RTE_RING_NAMESIZE);

            txQueueConf.u.rng.name[RTE_RING_NAMESIZE - 1] = '0';

            retValue = adk_ph_dev_tx_queue_setup(devId, queueId, &txQueueConf);
            if (retValue != ADK_SUCCESS)
            {
                rte_panic("  {%s} tx queue setup failed: (%d)", __func__,
                    retValue);
            }
#endif
        }
    }
#endif

    adk_ph_dev_device_start(devId);


    exceptionRock[phIndex].phIndex = phIndex;
    exceptionRock[phIndex].testConfShm_p = testConfShm_p;
    /* Create the thread to process the PDUs */
    if (pthread_create (&(threadId), NULL,
                        adk_ph_dev_exception,
                        &(exceptionRock[phIndex])) != 0)
    {
        rte_panic("Failed to create the thread, FAILED\n");
    }

    while (simpleLock == 0)
    {
        sleep(1);
    }

    printf("PH thread ID: %p\n", (void *)threadId);
    retValue = testConfShm_p->retValue;
    if (retValue != ADK_SUCCESS)
    {
        printf ("  {%s} failed\n", __func__);
        goto init_ph_thread_return;
    }

    snprintf(threadName, 15, "ph-v3-%d", phIndex);
    threadName[15] = 0;
    ret = pthread_setname_np(threadId, threadName);
    if (0 < ret)
    {
        retValue = -1;
        printf("ph thread name set failed: %d, '%s', %"PRIx64" '%s', (%d)\n",
               phIndex, threadName, (uint64_t)threadId, strerror(ret), ret);
        goto init_ph_thread_return;
    }

init_ph_thread_return:
    return retValue;
}

