/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"

#include <string.h>

#include "rte_errno.h"
#include "rte_memzone.h"
#include "rte_lcore.h"

#include "adk_error.h"

#include "test_shm.h"

adk_status_t
test_shm_create(
        const char *name,
        const size_t mem_size,
        uint32_t lookupMethod,
        test_shm_handle_t **shm_hdl)
{
    adk_status_t retValue = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;
    uint32_t socketId = 0;

    /* Validation */
    if (!name)
    {
        printf("Invalid pointer\n");
        retValue = ADK_E_INVALID_PARAMETER;
        goto _return;
    }

    mz = rte_memzone_lookup(name);
    if (mz)
    {
        /* Update user pointer */
        *shm_hdl = mz->addr;
        printf("Shared memory is already created\n");
        goto _return;
    }

    /* Get socket ID */
    if (TEST_SHM_LOOKUP_LCORE == lookupMethod)
    {
        socketId = rte_socket_id();
    } else
    {
        retValue = ADK_E_INVALID_PARAMETER;
        printf("Only TEST_SHM_LOOKUP_LCORE supported\n");
        goto _return;
    }

    /* Allocate a memory zone */
    mz = rte_memzone_reserve(name, mem_size, socketId, 0);
    if (mz == NULL)
    {
        printf("Unable create shared memory - %s\n", name);
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _return;
    }

    /* Initialize the reverved shared memory */
    memset(mz->addr, 0, mem_size);

    /* Update user pointer */
    *shm_hdl = mz->addr;

_return:
    return retValue;
}

adk_status_t
test_shm_lookup(
        const char *name,
        test_shm_handle_t **shm_hdl)
{
    adk_status_t retValue = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;

    /* Validation */
    if (!name)
    {
        printf("Invalid pointer\n");
        retValue = ADK_E_INVALID_PARAMETER;
        goto _return;
    }

    /* Get memzone handle */
    mz = rte_memzone_lookup(name);
    if (mz == NULL)
    {
        printf("Unable attach to shared memory - %s\n", name);
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _return;
    }

    /* Update user pointer */
    *shm_hdl = mz->addr;

_return:
    return retValue;
}

adk_status_t
test_shm_free(
        const char *name)
{
    adk_status_t retValue = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;
    int             rteErr = 0;

    /* Validation */
    if (!name)
    {
        printf("Invalid pointer\n");
        retValue = ADK_E_INVALID_PARAMETER;
        goto _return;
    }

    /* Get memzone handle */
    mz = rte_memzone_lookup(name);
    if (mz == NULL)
    {
        printf("Unable find shared memory - %s\n", name);
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _return;
    }

    rteErr = rte_memzone_free(mz);
    if (0 != rteErr)
    {
        printf("Unable remove shared memory - %s, "
                           "error code: %s (%d)\n", name,
                           rte_strerror(-rteErr), rteErr);
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _return;
    }

_return:
    return retValue;
}
