#
# Makefile for Unix targets
#
# $Revision: e5b83dc51c5933c459722bb3a64d7373645a6e6d $
#

include $(ADK_CODE)/config/makefile.modules
include $(RTE_SDK)/mk/rte.vars.mk
include $(RTE_SDK)/mk/rte.app.mk

LIB_DIRS= -L$(RDK_INSTALL)/lib -L$(RDK_INSTALL)/rdk_user/lib -L$(RDK_INSTALL)/rdk/install/lib \
          -lae_client \
          -lies_sdk

INCLUDE+= -I$(RDK_INSTALL)/include -I$(RDK_INSTALL)/rdk_user/include \
          -I$(RDK_INSTALL)/rdk/install/include -I. \
          -I../include $(ADK_SAMPLES_INCDIR) -I../common/include \
		  -I../../adk_dp_app

LIB_DIRS += -L$(ADK_LIBDIR) $(ADK_LIBDIRS)

ifeq ($(INTEL_ITT),true)
LIB_DIRS+= -L$(INTEL_ITT_INSTALL_LIB)
endif

CFLAGS_COMMON += -pthread
LIBS = -Wl,--start-group

# The order of the libs is important to ensure that all symbols are found

ifeq ($(ADK_MODULE_FPS), ENABLED)
LIBS += -ladk-fps
endif



ifeq ($(ADK_MODULE_GTPU), ENABLED)
LIBS += -ladk-gtpu
endif

ifeq ($(ADK_MODULE_IP), ENABLED)
LIBS += -ladk-ip
endif

ifeq ($(ADK_MODULE_IPV6), ENABLED)
LIBS += -ladk-ipv6
endif

ifeq ($(ADK_MODULE_ETH_OAM), ENABLED)
LIBS += -ladk-ethoam
endif

ifeq ($(ADK_MODULE_ETHERNET), ENABLED)
LIBS += -ladk-ethernet
endif

ifeq ($(ADK_MODULE_BFD), ENABLED)
LIBS += -ladk-bfd
endif

ifeq ($(ADK_MODULE_TWAMP), ENABLED)
LIBS += -ladk-twamp
endif

LIBS += -ladk-ph \
		-ladk-sif

ifeq ($(ADK_MODULE_IPSEC), ENABLED)
LIBS += -ladk-ipsec
endif

LIBS += -ladk-utils \
		-ladk-Gen \
		-ladk-pal

LIBS += $(call linkerprefix, $(LDLIBS))
LIBS += -lrte_pmd_ihqm_eventdev \
        -lrte_pmd_ice_dsi \
        -lrte_pmd_ice_dsi_tm \
        -lrte_pmd_qat \
        -lem


LIBS += -Wl,--end-group

ifeq ($(INTEL_ITT),true)
LIBS+= -littnotify -Wl,--no-as-needed
endif

OBJS = $(SRCS:.c=.$(AXXIA_RTE_TARGET).o)
OBJS_COMMON += $(COMMON_SRCS:.c=.$(AXXIA_RTE_TARGET).o)
TEST_DEPS = $(SRCS:.c=.$(AXXIA_RTE_TARGET).d)
TEST_DEPS += $(COMMON_SRCS:.c=.$(AXXIA_RTE_TARGET).d)
LINT_DEPS = $(SRCS:.c=.lint)
LINT_DEPS += $(COMMON_SRCS:.c=.lint)
TEST_EXE=$(SRCS:.c=.$(AXXIA_RTE_TARGET)_exe)

XMLS_UPDATE = $(XMLS:.xml=.xmlup)
DATS = $(XMLS:.xml=.dat)

ifeq "$(MAKECMDGOALS)" "debug"
FLAGS=$(DBGFLAGS)
else
FLAGS=$(RELFLAGS)
endif
export FLAGS

all: debug traffic

debug release code: $(SUBDIRS) $(TEST_EXE)

clobber: $(SUBDIRS)
	-$(ADK_Q)$(RM) -rf *_exe *.o *.d $(LINT_DEPS) $(DATS) *_db dbs

clean: $(SUBDIRS)
	-$(ADK_Q)$(RM) -rf $(TEST_EXE) $(OBJS) $(OBJS_COMMON) $(TEST_DEPS) $(LINT_DEPS) $(DATS) *_db

.PHONY: $(SUBDIRS)

$(SUBDIRS):
	$(ADK_Q)$(MAKE) -C $@ $(MAKECMDGOALS)

install install_hdrs: $(SUBDIRS)

lint: $(LINT_OBJS)

%.$(AXXIA_RTE_TARGET).d %.$(AXXIA_RTE_TARGET).o: %.c $(COMMON_SRCS)
	-$(ADK_Q)$(RM) -f $@
ifeq ($(LIB_EXT),.so)
	$(ADK_Q)$(CC) $(ADK_LDFLAGS) $(CFLAGS_COMMON) $(FLAGS) $(CFLAGS) $(CFLAGS_COMMON_SHARED) $(INCLUDE) -c $< -o $*.$(AXXIA_RTE_TARGET).o
else
	$(ADK_Q)$(CC) $(ADK_LDFLAGS) $(CFLAGS_COMMON) $(FLAGS) $(CFLAGS) $(CFLAGS_COMMON_STATIC) $(INCLUDE) -c $< -o $*.$(AXXIA_RTE_TARGET).o
endif

%_exe: %.o $(OBJS_COMMON)
	-$(ADK_Q)$(RM) -f $@
ifeq ($(LIB_EXT),.so)
	$(ADK_Q)$(CC) $(ADK_LDFLAGS) $(CFLAGS_COMMON) $(FLAGS) $(@:_exe=.o) $(LIB_DIRS) $(ADK_THIRDPARTY_LIBDIR) $(LIBS) $(ADK_LIBS) $(ADK_THIRDPARTY_LIBS) -ldl -lm -o $@ $(OBJS_COMMON)
else
	$(ADK_Q)$(CC) $(ADK_LDFLAGS) $(CFLAGS_COMMON) $(FLAGS) $(@:_exe=.o) $(LIB_DIRS) $(ADK_THIRDPARTY_LIBDIR) $(LIBS) $(ADK_LIBS) $(ADK_THIRDPARTY_LIBS) -ldl -lm -o $@ $(OBJS_COMMON)
endif

traffic: $(DATS) $(SUBDIRS)

%.dat: %.xml
	$(ACPTRAFGEN) $<
	touch $@

xml_update: $(XMLS_UPDATE) topology.xmlup topology_ipsec.xmlup $(SUBDIRS)

%.xmlup: %.xml
	$(CONFIG) -U $^

%.lint: %.c
	$(LINT) $(ADK_STRICT_CFLAGS) $(OS_FLAGS) $(CFLAGS) $(INCLUDE) $<


depend: $(TEST_DEPS)

ifeq "$(MAKECMDGOALS)" "debug"
-include $(TEST_DEPS)
endif
ifeq "$(MAKECMDGOALS)" "release"
-include $(TEST_DEPS)
endif
ifeq "$(MAKECMDGOALS)" "code"
-include $(TEST_DEPS)
endif
ifeq "$(MAKECMDGOALS)" "lint"
-include $(TEST_DEPS)
endif
