/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernet  Ethernet Module
 * @{
 *
 * @brief   This file has all the data structures and function
 *          prototypes defined by the ADK Ethernet module. This needs to
 *          be included by the application to be able to use Ethernet
 *          ADK module.
 *
 *  **********************************************************************/

#ifndef _ADK_ETHERNET_H_
#define _ADK_ETHERNET_H_

#include <adk_sys.h>
#include <adk_ethernet_conf.h>
#include <adk_ph_dev.h>
#include <adk_qos_include.h>
#include <adk_qos_dscp_pcp.h>
#include <adk_qos_policing_include.h>
#include "adk_domain.h"

/* *************************************************
***************** CONSTANT STRINGS ****************
***************************************************/
#define ADK_ETHERNET_ULI_DIRECT \
    "adk_ethernet.Ethernet_Encapsulation_direct"
#define ADK_ETHERNET_ULI_INDIRECT \
    "adk_ethernet.Ethernet_Encapsulation_indirect"
#define ADK_ETHERNET_INGRESS_ROOT \
    "adk_ethernet.Ethernet_Processing"

#define ADK_ETHERNET_IPV4_IMC_DIRECT \
    "adk_ethernet.Ethernet_IP_Fwd_IMC_direct"
#define ADK_ETHERNET_IPV6_IMC_DIRECT \
    "adk_ethernet.Ethernet_IPv6_Fwd_IMC"
#define ADK_ETHERNET_PTP_IMC_DIRECT \
    "adk_ethernet.Ethernet_PTP_IMC"
#define ADK_ETHERNET_LCP_IMC_DIRECT \
    "adk_ethernet.Ethernet_LCP_direct"
#define ADK_ETHERNET_OAM_IMC_INGRESS \
    "adk_ethernet.Ethernet_OAM_ingress_direct"
#define ADK_ETHERNET_OAM_IMC_EGRESS \
    "adk_ethernet.Ethernet_OAM_egress_direct"

/* *************************************************
***************** DATE TYPES **********************
***************************************************/

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_status_t
 *
 * @brief   The Ethernet module specific error code.
 *
 * This data type is used to return Ethernet module specific
 * error codes.
 *
 * =============================================================
 */
typedef enum adk_ethernet_status_s
{
    ADK_ETHERNET_E_MIN = (ADK_ETHERNET_MODULE_ID << 24),
    ADK_ETHERNET_E_INVALID_IF_HANDLE,
    ADK_ETHERNET_E_INVALID_FLOW_HANDLE,
    ADK_ETHERNET_E_INVALID_IF_TYPE,
    ADK_ETHERNET_E_INVALID_PORT_NUMBER,
    ADK_ETHERNET_E_INVALID_PROTO_TYPE,
    ADK_ETHERNET_E_INVALID_LLC_TYPE,
    ADK_ETHERNET_E_INVALID_VLAN_ID,
    ADK_ETHERNET_E_INVALID_BG_ID,
    ADK_ETHERNET_E_INVALID_PORT_TAG_TYPE,
    ADK_ETHERNET_E_INVALID_MCAST_GROUP_HANDLE,
    ADK_ETHERNET_E_INVALID_MAC_ADDR,
    ADK_ETHERNET_E_INVALID_IPV4_ADDR,
    ADK_ETHERNET_E_INVALID_MAC_SUBNET,
    ADK_ETHERNET_E_INVALID_PRIORITY,
    ADK_ETHERNET_E_INVALID_DSCP,
    ADK_ETHERNET_E_INVALID_POLICING_LEVEL,
    ADK_ETHERNET_E_INVALID_OUTPUT_RATE,
    ADK_ETHERNET_E_INVALID_VLAN_ASSIGN_POLICY_TYPE,
    ADK_ETHERNET_E_INVALID_FLOW_ACTION_TYPE,
    ADK_ETHERNET_E_INVALID_DUPLEX_MODE,
    ADK_ETHERNET_E_INVALID_ENCAP,
    ADK_ETHERNET_E_INVALID_FWD_MODE,
    ADK_ETHERNET_E_INVALID_STATS_TYPE,
    ADK_ETHERNET_E_INVALID_ASSOCIATION_MASK,
    ADK_ETHERNET_E_INVALID_MTU,
    ADK_ETHERNET_E_INCOMPAT_PORT_TYPE,
    ADK_ETHERNET_E_INCOMPAT_IF_TYPE,
    ADK_ETHERNET_E_INCOMPAT_STATS_TYPE,
    ADK_ETHERNET_E_INCOMPAT_FLOW_TYPE,
    ADK_ETHERNET_E_INCOMPAT_IF_STATE,
    ADK_ETHERNET_E_INCOMPAT_FWD_MODE,
    ADK_ETHERNET_E_INCOMPAT_VLAN_ASSIGN_POLICY_TYPE,
    ADK_ETHERNET_E_INCOMPAT_EIOA_VLAN_ETH_TYPE,
    ADK_ETHERNET_E_INCONSIST_IF_STATE,
    ADK_ETHERNET_E_UNSUPPORT_STATS_TYPE,
    ADK_ETHERNET_E_INTF_ALREADY_EXISTS,
    ADK_ETHERNET_E_NON_EXISTENT_INTF,
    ADK_ETHERNET_E_INTF_IS_ALREADY_MEMBER_OF_MCAST_GROUP,
    ADK_ETHERNET_E_INTF_IS_NOT_MEMBER_OF_MCAST_GROUP,
    ADK_ETHERNET_E_EIOA_VLAN_ETH_TYPE_FULL,
    ADK_ETHERNET_E_EIOA_VLAN_ETH_TYPE_ABSENT,
    ADK_ETHERNET_E_EXCEEDED_ETHTYPE_SUPPORTED,
    ADK_ETHERNET_E_EXCEEDED_DMACS_SUPPORTED,
    ADK_ETHERNET_E_INVALID_EXCEPTION_TYPE,
    ADK_ETHERNET_E_MAX
} adk_ethernet_status_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_proto_t
 *
 * @brief   The 16-bit protocol field in Ethernet header
 *
 *  This data type is used for 16-bit ethType protocol field in the Ethernet
 *  header.
 *
 * =============================================================
 */
typedef uint16_t adk_ethernet_proto_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_sub_type_t
 *
 * @brief   The 8-bit ether sub-type field in Ethernet header
 *
 *  This data type is used for 8-bit ether sub type field in the Ethernet
 *  header for layer 2 control packets.
 *
 * =============================================================
 */
typedef uint8_t adk_ethernet_sub_type_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_vlan_id_t
 *
 * @brief   The 12-bit VLAN ID in VLAN header
 *
 *  This data type is used for 12-bit VLAN ID field within the 4-byte VLAN
 *  header.
 *
 * =============================================================
 */
typedef uint16_t adk_ethernet_vlan_id_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_bg_id_t
 *
 * @brief   The 12-bit Bridge Group ID
 *
 * =============================================================
 */
typedef uint16_t adk_ethernet_bg_id_t;

/*!
 * adk_ethernet_multicast_group_handle_t identifies a multicast group within a
 * module.
 */
typedef adk_handle_t adk_ethernet_multicast_group_handle_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_mac_addr_t
 *
 * @brief   The 48-bit MAC address
 *
 *  This data type is used for 48-bit MAC address fields, in the Ethernet
 *  header. It is represented using an array of 6 unsigned chars.
 *
 * =============================================================
 */
typedef uint8_t adk_ethernet_mac_addr_t[6];

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_priority_t
 *
 * @brief   The 3-bit priority or PCP in VLAN tag
 *
 *  This data type is used for 3-bit priority field, or p-bits in the VLAN
 *  header.
 *
 * =============================================================
 */
typedef uint8_t adk_ethernet_priority_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_ipv4_addr_t
 *
 * @brief   The 4 byte IPv4 address
 *
 *  This data type is used for mapping next hop IP address to the next hop MAC
 *  address (for ARP)
 *
 * =============================================================
 */
typedef uint32_t adk_ethernet_ipv4_addr_t;

#define ADK_ETHERNET_IPV6_ADDR_SIZE         4
/*!
 * =============================================================
 *
 * @typedef adk_ethernet_ipv6_addr_t
 *
 * @brief   The 16 byte IPv6 address
 *
 *  This data type is used for mapping next hop IPv6 address to the next hop MAC
 *  address (for ND)
 *
 * =============================================================
 */

typedef struct adk_ethernet_ipv6_addr_s
{
    /*! The array of 32 bit words containing the IPv6 address.
     *  The higher bytes of the IPv6 address are stored starting
     *  in ipv6Addr[0].
     */
    uint32_t addr[ADK_ETHERNET_IPV6_ADDR_SIZE];
} adk_ethernet_ipv6_addr_t;

/*!
 * =============================================================
 *
 * @typedef adk_ethernet_log_level_t
 *
 * @brief   The 32-bit Ethernet module's log-level
 *
 *  This data type is used for setting the log-level of Ethernet module. It is
 *  represented using unsigned long integer and is a logical OR or following
 *  log-types defined by Ethernet module:
 *
 *  ADK_ETHERNET_LOG_FATAL      For logging any unrecoverable fatal error
 *
 *  ADK_ETHERNET_LOG_ERROR      For logging all other errors
 *
 *  ADK_ETHERNET_LOG_WARNING    For logging any unexpected input, output, state
 *                              or event
 *
 *  ADK_ETHERNET_LOG_INFO       For logging all resource allocations, etc
 *
 *  ADK_ETHERNET_LOG_FUNCS      For logging entry to all internal/external
 *                              functions
 *
 *  ADK_ETHERNET_LOG_HW         For logging all access to EIOA hardware
 *
 *  ADK_ETHERNET_LOG_PTN        For logging all learning/unlearning of Tree
 *                              entries
 *
 *  ADK_ETHERNET_LOG_SE_MEM     For logging all access to SE memory
 *
 *  ADK_ETHERNET_LOG_ALL        For logging all above
 *
 *  The following are some of the pre-defined commonly used log-levels.
 *
 *  ADK_ETHERNET_LOG_LEVEL_ERROR    enable ADK_ETHERNET_LOG_ERROR &
 *                                  ADK_ETHERNET_LOG_FATAL only
 *
 *  ADK_ETHERNET_LOG_LEVEL_WARNING  enable ADK_ETHERNET_LOG_WARNING,
 *                                  ADK_ETHERNET_LOG_ERROR &
 *                                  ADK_ETHERNET_LOG_FATAL only
 *
 *  ADK_ETHERNET_LOG_LEVEL_INFO     enable ADK_ETHERNET_LOG_INFO,
 *                                  ADK_ETHERNET_LOG_WARNING,
 *                                  ADK_ETHERNET_LOG_ERROR &
 *                                  ADK_ETHERNET_LOG_FATAL only
 *
 *  ADK_ETHERNET_LOG_LEVEL_VERBOSE  enable all types of logs. Same as
 *                                  ADK_ETHERNET_LOG_ALL
 *
 *  ADK_ETHERNET_LOG_LEVEL_DEFAULT  Default setting same as
 *                                  ADK_ETHERNET_LOG_LEVEL_INFO
 *
 * =============================================================
 */
typedef uint32_t adk_ethernet_log_level_t;
#define ADK_ETHERNET_LOG_DISABLE      (0 << 0)
#define ADK_ETHERNET_LOG_FATAL        (1 << 0)
#define ADK_ETHERNET_LOG_ERROR        (1 << 1)
#define ADK_ETHERNET_LOG_WARNING      (1 << 2)
#define ADK_ETHERNET_LOG_INFO         (1 << 3)
#define ADK_ETHERNET_LOG_FUNCS        (1 << 4)
#define ADK_ETHERNET_LOG_HW           (1 << 5)
#define ADK_ETHERNET_LOG_PTN          (1 << 6)
#define ADK_ETHERNET_LOG_SE_MEM       (1 << 7)
#define ADK_ETHERNET_LOG_SUBNET       (1 << 8)

#define ADK_ETHERNET_LOG_ALL          0xFFFFFFFF

#define ADK_ETHERNET_LOG_LEVEL_ERROR         (ADK_ETHERNET_LOG_ERROR | \
                                              ADK_ETHERNET_LOG_FATAL)
#define ADK_ETHERNET_LOG_LEVEL_WARNING       (ADK_ETHERNET_LOG_LEVEL_ERROR | \
                                              ADK_ETHERNET_LOG_WARNING)
#define ADK_ETHERNET_LOG_LEVEL_INFO          (ADK_ETHERNET_LOG_LEVEL_WARNING | \
                                              ADK_ETHERNET_LOG_INFO)
#define ADK_ETHERNET_LOG_LEVEL_VERBOSE       ADK_ETHERNET_LOG_ALL

#define ADK_ETHERNET_LOG_LEVEL_DEFAULT       ADK_ETHERNET_LOG_LEVEL_ERROR


/*!
 * =============================================================
 *
 * @enum adk_ethernet_ingress_flow_action_type_t
 *
 * @brief   The different actions for the ingress flow
 *
 *  This data type is used to configure the forwarding behavior in the
 *  Ethernet ingress flow classification filer. The actions can be:
 *
 *  ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_HOST  Send the packet to host via net driver
 *
 *  ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV4  Send the packet to IPv4 Module
 *
 *  ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV6  Send the packet to IPv6 Module
 *
 *  ADK_ETHERNET_ASSOCIATION_RULE_TYPE_DROP          Drop the packet
 *
 *  Note that, the default action to handover packet to higher layer module.
 *
 * =============================================================
 */
typedef enum adk_ethernet_ingress_flow_action_type_s
{
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_INVALID = 0,
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_HOST,
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV4,
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV6,
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_DROP,
    ADK_ETHERNET_ASSOCIATION_RULE_TYPE_MAX
} adk_ethernet_ingress_flow_action_type_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_cpu_data_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief 
 *
 *  This data structure is used to specify the data to be used in case the
 *  packet needs to be sent to host CPU via netdriver. This consists of:
 *
 *  priority            The priority to be given to the packet.
 *
 *  cppi_queue_id       Transmit cppi queue index
 *
 *  cppi_dev_id         CPPI Device identifier
 *
 *  rl_queue_handle     Rate limiting Queue handle
 *
 *  Note: Refer to the PH module documentation for details on PH Bind Handle and
 *  PH callback ID.
 *
 *
 * =============================================================
 */
typedef struct adk_ethernet_cpu_data_s
{
    adk_ethernet_priority_t         priority;
    uint16_t                        cppi_queue_id;
    uint8_t                         cppi_dev_id;
    adk_handle_t                    mtm_queue_handle;
} adk_ethernet_cpu_data_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_switch_virtual_if_data_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   The attributes of the switch virtual interface
 *
 *  This data type is used to specify the attributes of switch virtual
 *  interface. Using these attributes, a switch virtual interface is created.
 *
 *  bridge_if_handle    The Bridge Handle returned by adk_ethernet_bridge_interface_create
 *
 *  vlan_id             The VLAN tag
 *
 *  For ingress, a packet is received on a switch virtual interface when it 
 *  comes with destination MAC address same as MAC address that of bridge 
 *  interface and with VLAN id specified in this structure.
 *
 *  At egress, packet to be sent out of switch virtual interface are either 
 *  sent out with tag matching the vlan_id. 
 *
 * =============================================================
 */
typedef struct adk_ethernet_switch_virtual_if_data_s
{
    adk_interface_handle_t  bridge_if_handle;
    adk_ethernet_vlan_id_t  vlan_id;
} adk_ethernet_switch_virtual_if_data_t;


/*!
 * =============================================================
 *
 * @struct adk_ethernet_bridge_intf_attr_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   Base attributes for bridge Ethernet interface
 *
 * This structure is used to specify the attributes of bridge Ethernet
 * interface at the time of bridge interface creation. Like the base
 * attributes of a physical Ethernet interface, these attributes cannot be
 * modified at runtime. To modify these attribute, the interface needs to be
 * deleted and re-created with modified bridge attributes.
 *
 * For each L2 domain, there would be one such interface. SNR is programmed
 * with bridge source mac as L2 terminating. When packet is egressing via this 
 * interface, only Ethernet II (DIX) format encapsulation is supported.
 * Packet is transmitted either priority tagged or untagged as per
 * the configuration of adk_ethernet_priority_tagged_encap_config()
 *
 * This structure consists of following bridge attributes:
 *
 * eth_dev_id DPDK device ID for the DSI VSI
 *
 * src_mac Source MAC address to be used in Ethernet header for encapsulation
 *          of packets sent by higher layer module to Ethernet module.
 *
 * l2_domain_id L2 domain for which want to create the bridge interface. 
 *           Data plane processing gets L2 domain from DSI VSI meta data
 *
 * =============================================================
 */
typedef struct adk_ethernet_bridge_intf_attr_s
{
    uint16_t                        eth_dev_id;
    adk_ethernet_mac_addr_t         src_mac;
    int                             l2_domain_id;
} adk_ethernet_bridge_intf_attr_t;

#define ADK_ETHERNET_SWITCH_VIRTUAL_IF_BGID_MASK         (0x1 << 0)
#define ADK_ETHERNET_SWITCH_VIRTUAL_IF_CTAG_MASK         (0x1 << 1)

/*!
 * =============================================================
 *
 * @struct adk_ethernet_mac_subnet_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   Ethernet subnet MAC address
 *
 * This structure is used to specify one or a range of MAC addresses. However,
 * not every arbitrary range of MAC address can be specified but only "subnet
 * type" of ranges similar to IPv4 address range within a IPv4 subnet. To
 * specify only one MAC address, the num_valid_msb_bits should be set to 48. A
 * value less than 48 implies range of MAC addresses. For example,
 *
 *      mac = 0x102030405060 & num_valid_msb_bits = 46, specifies
 *
 *  10:20:30:40:50:60, 10:20:30:40:50:61, 10:20:30:40:50:62, 10:20:30:40:50:63
 *
 * By setting num_valid_msb_bits = 0, the entire MAC address can be wild-carded
 *
 * =============================================================
 */
typedef struct adk_ethernet_mac_subnet_s
{
    adk_ethernet_mac_addr_t mac;
    uint8_t num_valid_msb_bits;
} adk_ethernet_mac_subnet_t;

#define ADK_ETHERNET_FLOW_MASK_ETYPE  (0x1 << 0)
#define ADK_ETHERNET_FLOW_MASK_DMAC   (0x1 << 1)

/*!
* =============================================================
*
* @enum adk_ethernet_interface_type_t
*
* @brief   Ethernet interface type
*
*  This data type is used to configure  Ethernet ingress  flow rule for
*  bridge or physical interface to terminate the traffic accordingly.
*  This can be:
*
*  ADK_ETHERNET_INTERFACE_TYPE_BRIDGE bridge interface
*
*  ADK_ETHERNET_INTERFACE_TYPE_PHY physical interface
*
* =============================================================
*/

typedef enum adk_ethernet_interface_type_s
{
    ADK_ETHERNET_INTERFACE_TYPE_INVALID = 0,
    ADK_ETHERNET_INTERFACE_TYPE_BRIDGE,
    ADK_ETHERNET_INTERFACE_TYPE_PHY,
    ADK_ETHERNET_INTERFACE_TYPE_MAX
} adk_ethernet_interface_type_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_ingress_flow_data_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   Ethernet Ingress Flow Data
 *
 * This structure is used to specify the input fields for ingress flow creation.
 * This consists of
 *
 * mask    Mask to specify the input fields in the structure which are valid. If
 *         mask bit is not set then the corresponding field is a wild-card field
 *         and thus would match any value. The bit fields within this mask are:
 *
 *          ADK_ETHERNET_FLOW_MASK_ETYPE      for protocol type (proto)
 *
 *          ADK_ETHERNET_FLOW_MASK_DMAC       for destination MAC address
 *
 *
 * proto            Ethernet protocol type
 *
 * dest_mac         Destination MAC address
 *
 * intfType          bridge or physical of type adk_ethernet_interface_type_t
 *
 * Out of these, the destination MAC address, being subnet style, can be
 * partially (for multicast) wild-carded.
 *
 * Note that if the proto is wild-carded then destination MAC
 * address must be wildcarded.
 *
 * =============================================================
 */
typedef struct adk_ethernet_ingress_flow_data_s
{
    uint32_t mask;
    adk_ethernet_proto_t proto;
    adk_ethernet_mac_subnet_t dest_mac;
    adk_ethernet_interface_type_t intfType;
} adk_ethernet_ingress_flow_data_t;


/*!
 * =============================================================
 *
 * @struct adk_ethernet_ingress_flow_action_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   Ethernet Ingress Flow Action
 *
 *  This data structure is used to specify the output action and associated data
 *  for packets undergoing Ethernet ingress flow classification filter.
 *  Every flow classification rule in the filter has this data type specified
 *  and is applied for packets matching the rule. The default action is to handover
 *  the packet to higher layer module. The data structure consists of:
 *
 *  action_type         Output action type which can be to
 *
 *                          1. Send the packet to host CPU via net driver 
 *
 *                          2. Drop the packet
 *
 * enablePolicing       1 indicates policing enabled and 0 indicates policing disabled
 *
 * policingHandle       This field determines the policing profile
 *                      handle. Relevant only if enablePolicing flag is set to 1.
 *                      Application must create this policing flow handle using
 *                      adk_qos_policing_flow_create API
 *
 *
 *
 * Note that, For dropping, no associated data is required. Application is
 * not supposed to create a flow entry for handing over to higher layer module.
 *
 * =============================================================
 */
typedef struct adk_ethernet_ingress_flow_action_s
{
    adk_ethernet_ingress_flow_action_type_t action_type;
    bool enablePolicing;
    adk_qos_policer_handle_t policingHandle;
} adk_ethernet_ingress_flow_action_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_global_statistics_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   The Ethernet global statistics
 *
 *  This data type consists of the following global statistics maintained by Ethernet
 *  module:
 *
 *  intf_not_defined        :   Number of packets on Unknown Interface.
 *  port_not_defined        :   Number of packets on Unknown Port.
 *  pkts_rcvd_from_nca      :   Number of packets received from NCA.
 *  no_link_in_aggregate    :   No link found in aggregate.
 *  time_index_alloc_error  :   Egress time index allocation failure.
 *  pkt_clone_failure           :   Packet clone failures
 *  ingressflow_lookup_failure : Default L3 module for ethertype in packet not defined
 *
 * =============================================================
 */
typedef struct adk_ethernet_global_statistics_s
{
    uint32_t intf_not_defined;
    uint32_t port_not_defined;
    uint32_t time_index_alloc_error;
    uint32_t ts_tx_complete_notification_failed;
    uint32_t pkt_clone_failure;
    uint32_t ingressflow_lookup_failure;
    uint32_t ingressflowDrops;
    uint32_t ethPacketsRateLimitingDrops;
    uint32_t arp_buffer_full_pkt_rollover_discard;
    uint32_t arp_buffer_fragment_pkt_rollover_discard;
    uint32_t nd_buffer_full_pkt_rollover_discard;
    uint32_t nd_buffer_fragment_pkt_rollover_discard;
    uint32_t exceptionDrops;
    uint32_t arp_buffer_arp_entry_rem_drop;
    uint32_t nd_buffer_neigh_entry_rem_drop;
    uint32_t neigh_state_unreachable_drop;
    uint32_t non_l2term_frames_discard;
    uint32_t netd_origin_non_bridge_frames_discard;
    uint32_t ethtype_queue_map_not_found_drop;
    uint32_t unknown_output_interface_drop;
    uint32_t buffering_memory_not_available_drop;
    uint32_t nic_send_failure_drop;
    uint32_t nexthop_buffer_stage_not_found_drop;
    uint32_t nexthop_buffer_proc_drop;
    uint32_t nexthop_buffer_timeout_error;
    uint32_t failed_to_sent_next_stage_drop;
    uint32_t vlan_header_stripping_failure_drop;
    uint32_t egress_encap_pkt_failure_drop;
    uint32_t unknown_multicast_addr_drop;
} adk_ethernet_global_statistics_t;

/*!
* =============================================================
*
* @struct adk_ethernet_bridge_interface_statistics_t adk_ethernet.h "adk_ethernet.h"
*
* @brief   The Ethernet Bridge interface statistics
*
*  This data type consists of following Bridge interface statistics maintained
*  by Ethernet module. These are transmit, receive and error counters.
*
*  inUnknownVlan            :   Number of frames received on unknown SVIF interface.
*  inEthTypeMismatch        :   Number of packets received with invalid EtherType.
*  inFrames                 :   Number of Unicast/Mutlicast/Broadcast packtes received.
*  inOctets                 :   Number of Unicast/Mutlicast/Broadcast Octets received.
*  OutFrames                :   Number of Unicast/Mutlicast/Broadcast packtes transmitted.
*  OutOctets                :   Number of Unicast/Mutlicast/Broadcast Octets transmitted.
*  inPrioTaggedFrames       :   Number of priority tagged packets received
*  inMcastFrames            :   Number of Multicast/Broadcast packets received.
*  OutMcastFrames           :   Number of Multicast/Broadcast packets transmitted.
*
* =============================================================
*/
typedef struct adk_ethernet_bridge_interface_statistics_s
{
    uint32_t inUnknownVlan;
    uint32_t inEthTypeMismatch;
    uint64_t inFrames;  /* Unicast/Mutlicast/Broadcast*/
    uint64_t inOctets;  /* Unicast/Mutlicast/Broadcast*/
    uint64_t OutFrames; /* Unicast/Mutlicast/Broadcast*/
    uint64_t OutOctets; /* Unicast/Mutlicast/Broadcast*/
    uint32_t inPrioTaggedFrames;
    uint64_t inMcastFrames; /*Mutlicast/Broadcast*/
    uint64_t OutMcastFrames; /*Mutlicast/Broadcast*/
} adk_ethernet_bridge_interface_statistics_t __rte_cache_aligned;

/*!
* =============================================================
*
* @struct adk_ethernet_vlan_interface_statistics_t adk_ethernet.h "adk_ethernet.h"
*
* @brief   The Ethernet Vlan interface statistics
*
*  This data type consists of following Switch virtual interface statistics maintained
*  by Ethernet module. These are transmit, receive and error counters.
*
*  inEthTypeMismatch     :   Number of packets received with invalid EtherType.
*  inFrames              :   Number of Unicast/Mutlicast/Broadcast packtes received.
*  inOctets              :   Number of Unicast/Mutlicast/Broadcast Octets received.
*  OutFrames             :   Number of Unicast/Mutlicast/Broadcast packtes transmitted.
*  OutOctets             :   Number of Unicast/Mutlicast/Broadcast Octets transmitted.
*  inMcastFrames         :   Number of Multicast/Broadcast packets received.
*  OutMcastFrames        :   Number of Multicast/Broadcast packets transmitted.
*
* =============================================================
*/
typedef struct adk_ethernet_vlan_interface_statistics_s
{
    uint32_t inEthTypeMismatch;
    uint64_t inFrames;      /* Unicast/Mutlicast/Broadcast*/
    uint64_t inOctets;      /* Unicast/Mutlicast/Broadcast*/
    uint64_t OutFrames;     /* Unicast/Mutlicast/Broadcast*/
    uint64_t OutOctets;     /* Unicast/Mutlicast/Broadcast*/
    uint64_t inMcastFrames; /* Mutlicast/Broadcast*/
    uint64_t OutMcastFrames; /* Mutlicast/Broadcast*/
} adk_ethernet_vlan_interface_statistics_t __rte_cache_aligned;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_encap_flow_stats_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   The Ethernet ingress flow statistics.
 *
 *  This structure has the statistics of Ethernet encap flow.
 *  rx_pkts             Number of packets received by ingress flow.
 *  pkts_sent_to_host   Number of packets sent to host via netdriver
 *  pkts_sent_to_ipv4   Number of packets sent to ADK IPv4 Module
 *  pkts_sent_to_ipv6   Number of packets sent to ADK IPv6 Module
 *  pkts_drop           Number of packets drop as part of ingress flow action
 * =============================================================
 */
/* Ingress flow level data plane stats */
typedef struct adk_ethernet_ingress_flow_stats_s
{
    uint64_t rx_pkts;
    uint64_t pkts_sent_to_host;
    uint64_t pkts_sent_to_ipv4; 
    uint64_t pkts_sent_to_ipv6;
    uint64_t pkts_drop;
} adk_ethernet_ingress_flow_stats_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_port_statistics_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   The physical Ethernet interface statistics
 *
 *  This data type consists of the statistics for a physical Ethernet interface.
 *  These statistics directly correspond to the EIOA port the Ethernet interface
 *  represents.
 *
 * =============================================================
 */
typedef struct adk_ethernet_port_statistics_s
{
    uint64_t tx_octets;
    uint64_t tx_bad_octets;
    uint64_t tx_frm;
    uint64_t tx_bad_frm;
    uint64_t tx_mcast;
    uint64_t tx_bcast;
    uint64_t tx_pause;
    uint64_t tx_fcs;
    uint64_t tx_defer;
    uint64_t tx_xsdfr;
    uint64_t tx_1col;
    uint64_t tx_mcol;
    uint64_t tx_lcol;
    uint64_t tx_xscol;
    uint64_t tx_frag;
    uint64_t tx_usize;
    uint64_t tx_64;
    uint64_t tx_65;
    uint64_t tx_128;
    uint64_t tx_256;
    uint64_t tx_512;
    uint64_t tx_1024;
    uint64_t tx_1519;
    uint64_t tx_osize;
    uint64_t tx_jab;
    uint64_t tx_urun;
    uint64_t tx_stat_ovfl;

    uint64_t rx_octets;
    uint64_t rx_bad_octets;
    uint64_t rx_frm;
    uint64_t rx_bad_frm;
    uint64_t rx_mcast;
    uint64_t rx_bcast;
    uint64_t rx_ctl;
    uint64_t rx_pause;
    uint64_t rx_unk_op;
    uint64_t rx_fcs;
    uint64_t rx_align;
    uint64_t rx_length;
    uint64_t rx_code;
    uint64_t rx_frag;
    uint64_t rx_usize;
    uint64_t rx_64;
    uint64_t rx_65;
    uint64_t rx_128;
    uint64_t rx_256;
    uint64_t rx_512;
    uint64_t rx_1024;
    uint64_t rx_1519;
    uint64_t rx_osize;
    uint64_t rx_jab;
    uint64_t rx_drop;
    uint64_t rx_carrier;
    uint64_t rx_urun;
    uint64_t rx_stat_ovfl;

} adk_ethernet_port_statistics_t;

/*!
 * =============================================================
 * @enum adk_ethernet_table_group_t
 *
 * @brief
 *
 * This enumeration defines different group of tables that can be
 * dumped in user readable csv format.
 * =============================================================
 */
typedef enum adk_ethernet_table_group_s
{
    /*! Non inclusive lower bound on table group type */
    ADK_ETHERNET_TABLES_NONE = 0,
    /*! Interface related tables */
    ADK_ETHERNET_TABLES_INTF_ATTR = (1 << 0),
    /*! Encpasulation related tables */
    ADK_ETHERNET_TABLES_ENCAPSULATION = (1 << 1),
    /*! Switching related tables */
    ADK_ETHERNET_TABLES_SWITCHING = (1 << 2),
    /*! QoS/Priority Marking specific tables */
    ADK_ETHERNET_TABLES_QOS_AND_PRIORITY_MARKING = (1 << 3),
    /*! BVI specific tables */
    ADK_ETHERNET_TABLES_VIRTUAL_INTERFACE = (1 << 4),
    /* Inclusive upper bound on table group type for all tables */
    ADK_ETHERNET_TABLES_ALL = ((1 << 5) - 1)
} adk_ethernet_table_group_t;

/*!
 * =============================================================
 * @enum adk_ethernet_arp_status_t
 *
 * @brief
 *
 * This enumeration defines arp entry status
 * =============================================================
 */
typedef enum adk_ethernet_arp_status_s
{
    ADK_ETHERNET_NEIGHBOR_INVALID = 0,
    /*! Indicates neighbor is reachable */
    ADK_ETHERNET_NEIGHBOR_REACHABLE,
    /*! Indicates neighbor might not be reachable*/
    ADK_ETHERNET_NEIGHBOR_STALE,
    /*! Indicates neighbor is not reachable and
        no exception needs to be sent to CPU */
    ADK_ETHERNET_NEIGHBOR_BUFFER_WITHOUT_NOTIFYING,
    ADK_ETHERNET_NEIGHBOR_MAX
} adk_ethernet_arp_status_t;

typedef adk_ethernet_arp_status_t adk_ethernet_neigh_status_t;

/*!
 * =============================================================
 * @enum adk_ethernet_arp_entry_t
 *
 * @brief
 *
 * This structure defines arp entry
 * =============================================================
 */

typedef struct adk_ethernet_arp_entry_s {
    adk_ethernet_ipv4_addr_t       ip_addr;
    adk_ethernet_arp_status_t      arp_status;
    adk_ethernet_mac_addr_t        dest_mac;
}adk_ethernet_arp_entry_t;

/*!
 * =============================================================
 * @enum adk_ethernet_neighbor_entry_t
 *
 * @brief
 *
 * This structure defines neighbor entry
 * =============================================================
 */

typedef struct adk_ethernet_neighbor_entry_s {
    adk_ethernet_ipv6_addr_t       ipv6_addr;
    adk_ethernet_neigh_status_t    neigh_status;
    adk_ethernet_mac_addr_t        dest_mac;
}adk_ethernet_neighbor_entry_t;

/*!
 * =============================================================
 * @enum adk_ethernet_exception_type_t
 *
 * @brief
 *
 * Defines the supported ETH exception types.
 *
 * =============================================================
 */

typedef enum adk_ethernet_exception_type_s
{
    ADK_ETHERNET_EXCEPTION_TYPE_INVALID = 0,
    ADK_ETHERNET_EXCEPTION_TYPE_UNRESOLVED_ARP,
    ADK_ETHERNET_EXCEPTION_TYPE_UNRESOLVED_IPV6_NEIGHBOR,
    ADK_ETHERNET_EXCEPTION_TYPE_NEIGHBOR_STALE_ENTRY_USED,
    ADK_ETHERNET_EXCEPTION_TYPE_MAX
} adk_ethernet_exception_type_t;
/*!
 * =============================================================
 *
 * @struct adk_ethernet_exception_handler_t
 *
 * @brief   This structure defines the parameters used by exception handler
 * function.
 *
 * type       Type field decides the type of exception
 *
 * devId      PH Device ID returned by adk_ph_dev_device_configure
 *
 * queueId    PH QueueID used in adk_ph_dev_rx_queue_setup
 *
 * cbFnHandle PH Callback Fn handle return by adk_ph_dev_callback_fn_add()
 *
 * policer_handle Handle for host policer
 *
 * =============================================================
 */
typedef struct adk_ethernet_exception_handler_s {
    /*!
     * type field decides the type of exception.
     */
    adk_ethernet_exception_type_t type;
    uint8_t devId;
    uint16_t queueId;
    uint8_t priority;
    adk_ph_dev_cbfn_handle_t cbFnHandle;
    adk_handle_t policer_handle;
} adk_ethernet_exception_handler_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_arp_exception_data_t
 *
 * @brief   The Ethernet module's parameters received in the
 *          callback function data along with the packet due to ARP lookup failure.
 *
 * This data type consists of all the exception parameters that Ethernet module
 * can send along with the packet in the task parameters. Below is the list:
 *
 * ifHandle          interface handle corresponding to bridge or svif
 *
 * vlanId            vlanId of svif on which exception is raised  
 *
 * next_hop_address   Next hop IP address
 *
 * =============================================================
 */
typedef struct adk_ethernet_arp_exception_data_s
{
    adk_interface_handle_t ifHandle;
    adk_ethernet_vlan_id_t  vlanId;
    adk_ethernet_ipv4_addr_t next_hop_address;
} adk_ethernet_arp_exception_data_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_ipv6_neighbor_exception_data_t
 *
 * @brief   The Ethernet module's parameters received in the
 *          callback function data along with the packet due to ipv6 neighbor lookup failure.
 *
 * This data type consists of all the exception parameters that Ethernet module
 * can send along with the packet in the task parameters. Below is the list:
 *
 * ifHandle          interface handle corresponding to bridge or svif
 * vlanId            vlanId of svif on which exception is raised  
 * v6_next_hop_address   Next hop IPv6  address
 *
 * =============================================================
 */
typedef struct adk_ethernet_ipv6_neighbor_exception_data_s
{
    adk_interface_handle_t ifHandle;
    adk_ethernet_vlan_id_t  vlanId;
    adk_ethernet_ipv6_addr_t v6_next_hop_address;
} adk_ethernet_ipv6_neighbor_exception_data_t;

/*!
* =============================================================
*
* @struct adk_ethernet_neighbor_stale_entry_exception_data_t adk_ethernet.h "adk_ethernet.h"
*
* @brief   The Ethernet module's exception data parameters received in the
*          callback function data from ADK DP to application
*
* This data type consists of the exception data parameters that Ethernet
* module can send in the task parameters. Below is the list:
*
*   ifHandle          interface handle corresponding to bridge or svif
*
*   vlanId            vlanId of svif on which exception is raised  
*
*   isIPv6            IPv4 or IPv6 neighbor
*
*   nextHop           IPv4 or IPv6 next hop address.
*
* =============================================================
*/
typedef struct adk_ethernet_neighbor_stale_entry_exception_data_s
{
    adk_interface_handle_t   ifHandle;
    adk_ethernet_vlan_id_t   vlanId;
    union {
        adk_ethernet_ipv4_addr_t v4Address;
        adk_ethernet_ipv6_addr_t v6Address;
    } nextHop;
    bool isIPv6;
} adk_ethernet_neighbor_stale_entry_exception_data_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_recv_exception_param_t
 *
 * @brief   The Ethernet module's exception data parameters received in the
 *          callback function data from ADK DP to application
 *
 * This data type consists of all the exception data parameters that Ethernet
 * module can send in the task parameters. Below is the list:
 *
 * exception_type           Indicates the type of exception whose data is sent from
 *                          ADK DP to application.
 *
 * arp_exception_data       This consists of unresolved ARP exception parameters that
 *                          Ethernet module can send in the task parameters.
 *
 * v6_neighbor_exception_data      This consists of unresolved IPv6 Neighbor exception parameters that
 *                                   Ethernet module can send in the task parameters.
 *
 * stale_entry_exception_data      This consists of stale entry neighbor exception parameters that
 *                                 Ethernet module can send in the task parameters .
 * =============================================================
 */
typedef struct adk_ethernet_recv_exception_param_s
{
    adk_ethernet_exception_type_t   exception_type;
    union {
        adk_ethernet_arp_exception_data_t arp_exception_data;
        adk_ethernet_ipv6_neighbor_exception_data_t v6_neighbor_exception_data;
        adk_ethernet_neighbor_stale_entry_exception_data_t stale_entry_exception_data;
    }exception_data;
} adk_ethernet_recv_exception_param_t;

#define ADK_ETH_MAC_TABLE_INTF_LEVEL         1
#define ADK_ETH_MAC_TABLE_INTF_BGID_LEVEL    2
/*!
 * =============================================================
 *
 * @struct adk_eth_mac_table_options_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   This structure is used to specify the options for Ethernet
 *          MAC Table dump and flush.
 *
 * mask          Specifies interface level or interface+bg_id level MAC Table
 *               dump required. ADK_ETH_MAC_TABLE_INTF_LEVEL or
 *               ADK_ETH_MAC_TABLE_INTF_VLAN_LEVEL
 *
 * if_handle     Interface ID for interface level dump or flush
 *
 * bg_id         Bridge Group Instance Id for interface+bgId level dump or flush
 *
 * flsuh_static_entries  Flag to enable flushing of static MAC entries
 *                       Currently this flag used only in LLS flows
 *
 * =============================================================
 */
typedef struct adk_eth_mac_table_options_s
{
    uint8_t mask;
    adk_interface_handle_t if_handle;
    adk_ethernet_vlan_id_t bg_id;
    bool flush_static_entries;
} adk_eth_mac_table_options_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_mac_table_entry_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   Ethernet MAC Table Entry Structure
 *
 * This structure is used to return MAC Table entries dumped
 *
 * bg_id            Ethernet protocol type
 *
 * mac              MAC address
 *
 * if_id            MAC learnt interface
 *
 * time_remaining   Remaining time for the MAC entry to expire in seconds
 *                  This field is valid only for AXM55xx
 *
 * =============================================================
 */
typedef struct adk_ethernet_mac_table_entry_s
{
    adk_ethernet_mac_addr_t mac;
    adk_ethernet_vlan_id_t bg_id;
    adk_port_number_t if_id;
    uint32_t time_remaining;
} adk_ethernet_mac_table_entry_t;

/*!
 * =============================================================
 *
 * @struct adk_ethernet_port_tag_type_t adk_ethernet.h "adk_ethernet.h"
 *
 * @brief  Type of vlan tagging for outgoing packet
 *
 *  This data structure is used to specify the type of vlan configurations
 *  for the egress side of a port
 *
 *  ADK_ETHERNET_PORT_TAG_TYPE_UNTAGGED         Untagged
 *
 *  ADK_ETHERNET_PORT_TAG_TYPE_SINGLE_TAGGED    Singly tagged
 *
 * =============================================================
 */
typedef enum adk_ethernet_port_tag_type_s
{
    ADK_ETHERNET_PORT_TAG_TYPE_INVALID = 0,
    ADK_ETHERNET_PORT_TAG_TYPE_UNTAGGED,
    ADK_ETHERNET_PORT_TAG_TYPE_SINGLE_TAGGED,
    ADK_ETHERNET_PORT_TAG_TYPE_MAX
} adk_ethernet_port_tag_type_t;

/*!
 * =============================================================
 *
 * @enum adk_ethernet_encap_t
 *
 * @brief   The encapsulation format for output Ethernet frames
 *
 *  This data type is used to configure the encpsulation format of packets being
 *  sent out of physical or aggregate Ethernet interface. Note this, this
 *  attribute applies only for packets which are sent to Ethernet module by the
 *  higher layer module like IPv4 and not for packets undergoing Ethernet
 *  switching. The format can be either Ethernet II (DIX) or IEEE 802.3
 *  (LLC-SNAP).
 *
 * =============================================================
 */
typedef enum adk_ethernet_encap_s
{
    ADK_ETHERNET_ENCAP_INVALID = 0,
    ADK_ETHERNET_ENCAP_DIX,
    ADK_ETHERNET_ENCAP_LLC_SNAP,
    ADK_ETHERNET_ENCAP_MAX
} adk_ethernet_encap_t;


/*!
 * =============================================================
 *
 * @struct adk_ethernet_llc_hdr_t   adk_ethernet.h "adk_ethernet.h"
 *
 * @brief   The 24-bit LLC header (IEEE 802.2) This is used in case 16-bit
 * protocol type field in not available in the Ethernet header.
 *
 *  This data type is used for specifying entire 24-bit LLC header by providing
 *  all the 3 components in it.
 *
 * =============================================================
 */
typedef struct adk_ethernet_llc_hdr_s
{
    uint8_t dsap;
    uint8_t ssap;
    uint8_t control;
} adk_ethernet_llc_hdr_t;

#define ADK_ETH_BRIDGE_DEFAULT_TIMESTAMP_QUAD_GROUP            3
#define ADK_ETH_BRIDGE_DEFAULT_TIMESTAMP_QUAD_PORT            12
#define ADK_ETH_MAX_NUM_PORT_PER_QUAD    4
#define ADK_ETH_FIRST_QUAD_PORT_INDEX    0

/*!
 * =============================================================
 * @enum adk_ethernet_quad_group_t
 *
 * @brief
 *
 * Defines the QUAD group of HLP Ports
 *
 * =============================================================
 */

typedef enum adk_ethernet_quad_group_t
{
    ADK_ETHERNET_QUAD_GROUP_0 = 0,
    ADK_ETHERNET_QUAD_GROUP_1,
    ADK_ETHERNET_QUAD_GROUP_2,
    ADK_ETHERNET_QUAD_GROUP_3,
    ADK_ETHERNET_QUAD_GROUP_4,
    ADK_ETHERNET_QUAD_GROUP_MAX
} adk_ethernet_quad_group_t;

/*!
 * @addtogroup _ethernetFunctions  Ethernet Functions
 * @{
 */
/* *************************************************
***************** FUNCTION PROTOTYPES *************
***************************************************/

adk_status_t adk_ethernet_init(adk_module_handle_t);

adk_status_t
adk_ethernet_service_function_update (
    adk_module_handle_t eth_mod_handle);

adk_status_t
adk_ethernet_log_level_set (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_log_level_t log_level);

adk_status_t
adk_ethernet_log_enable (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_log_level_t log_level);

adk_status_t
adk_ethernet_log_disable (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_log_level_t log_level);

adk_status_t adk_ethernet_bridge_interface_create(adk_module_handle_t eth_mod_handle,
        const adk_ethernet_bridge_intf_attr_t *bridge_intf_attr_p,
        adk_interface_handle_t *bridge_if_handle_p);

adk_status_t adk_ethernet_bridge_interface_delete(adk_module_handle_t eth_mod_handle,
        adk_interface_handle_t bridge_if_handle_p);

adk_status_t
adk_ethernet_switch_virtual_if_create (
    adk_module_handle_t eth_mod_handle,
    const adk_ethernet_switch_virtual_if_data_t *svif_data,
    adk_interface_handle_t *v_if_handle_p);

adk_status_t
adk_ethernet_switch_virtual_if_delete (
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t v_if_handle);

adk_status_t
adk_ethernet_ingress_flow_create (
    adk_module_handle_t eth_mod_handle,
    const adk_ethernet_ingress_flow_data_t *in_flow_data,
    const adk_ethernet_ingress_flow_action_t *flow_action,
    adk_flow_handle_t *flow_handle_p);

adk_status_t
adk_ethernet_ingress_flow_delete (
    adk_module_handle_t eth_mod_handle,
    adk_flow_handle_t flow_handle);

adk_status_t
adk_ethernet_ingress_flow_statistics_get (
    adk_module_handle_t eth_mod_handle,
    adk_flow_handle_t flow_handle,
    adk_ethernet_ingress_flow_stats_t* flow_stats);
#ifdef ADK_DEBUG
adk_status_t
adk_ethernet_ingress_flow_statistics_print (
    adk_module_handle_t eth_mod_handle,
    adk_flow_handle_t flow_handle);
#endif

adk_status_t
adk_ethernet_bridge_interface_statistics_get (
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    adk_ethernet_bridge_interface_statistics_t *stats);

adk_status_t
adk_ethernet_vlan_interface_statistics_get (
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    adk_ethernet_vlan_interface_statistics_t *stats);
adk_status_t adk_ethernet_global_statistics_get(
    adk_module_handle_t,
    adk_ethernet_global_statistics_t *);
#ifdef ADK_DEBUG
adk_status_t adk_ethernet_bridge_interface_statistics_print(
    adk_module_handle_t,
    adk_interface_handle_t);
adk_status_t adk_ethernet_vlan_interface_statistics_print(
    adk_module_handle_t,
    adk_interface_handle_t);
#endif
adk_status_t adk_ethernet_port_statistics_get(
    adk_module_handle_t,
    uint32_t port,
    adk_ethernet_port_statistics_t *,
    bool clear_on_read);
#ifdef ADK_DEBUG
adk_status_t
adk_ethernet_port_stats_print (
    adk_module_handle_t eth_mod_handle,
    uint32_t port);
#endif
#ifdef ADK_DEBUG
adk_status_t
adk_ethernet_global_stats_print (
    adk_module_handle_t eth_mod_handle);
#endif
adk_status_t
adk_ethernet_tables_dump (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_table_group_t table_group);

adk_status_t adk_ethernet_arp_entry_add(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv4_addr_t ip_addr,
                    adk_ethernet_mac_addr_t dest_mac,
                    adk_ethernet_arp_status_t arpStatus);

adk_status_t adk_ethernet_arp_entry_rem(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv4_addr_t ip_addr);

adk_status_t adk_ethernet_arp_entry_get(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv4_addr_t ip_addr,
                    adk_ethernet_mac_addr_t dest_mac,
                    adk_ethernet_arp_status_t* arpStatus);

adk_status_t adk_ethernet_get_arp_entries_all(
                    adk_module_handle_t eth_mod_handle,
                    uint32_t *numArpEntries,
                    adk_ethernet_arp_entry_t **arp_entries_p);

adk_status_t  adk_ethernet_priority_tagged_encap_config(
        adk_module_handle_t eth_mod_handle,
        adk_interface_handle_t if_handle,
        bool priority_tagging_flag);

adk_dp_status_t
adk_ethernet_dp_init (
    void);


adk_status_t
adk_ethernet_register_exception(  adk_module_handle_t eth_mod_handle,
      const adk_ethernet_exception_handler_t  *exceptionHandler_p);

adk_status_t
adk_ethernet_unregister_exception(  adk_module_handle_t eth_mod_handle,
        const adk_ethernet_exception_type_t exceptionType);

adk_status_t
adk_ethernet_neighbor_entry_add(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv6_addr_t ipv6_addr,
                    adk_ethernet_mac_addr_t dest_mac,
                    adk_ethernet_neigh_status_t neigh_status);
adk_status_t
adk_ethernet_neighbor_entry_rem(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv6_addr_t ipv6_addr);

adk_status_t
adk_ethernet_neighbor_entry_get(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv6_addr_t ipv6_addr,
                    adk_ethernet_mac_addr_t dest_mac,
                    adk_ethernet_neigh_status_t *neigh_status);


adk_status_t adk_ethernet_get_neighbor_entries_all(
                    adk_module_handle_t eth_mod_handle,
                    uint32_t *numNeighborEntries,
                    adk_ethernet_neighbor_entry_t **neighbor_entries_p);

#ifdef ADK_DEBUG
adk_status_t adk_ethernet_arp_buffer_nexthop_debug(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_ipv4_addr_t ip_addr,
                    uint32_t* num_inflight_pkts,
                    uint32_t* num_full_buffered_pkts,
                    uint32_t* num_frag_buffered_pkts);
#endif



adk_status_t adk_ethernet_autoconnection_enable (
                    adk_module_handle_t eth_mod_handle);

adk_status_t adk_ethernet_autoconnection_disable (
                    adk_module_handle_t eth_mod_handle);

adk_status_t adk_ethernet_autoconnection_status_get(
                    adk_module_handle_t eth_mod_handle, bool *status);


adk_status_t adk_ethernet_bridge_interface_mac_addr_modify(adk_module_handle_t eth_mod_handle,
                    adk_ethernet_mac_addr_t   src_mac,
                    adk_interface_handle_t bridge_if_handle);

adk_status_t adk_ethernet_ethertype_p_bits_map_del(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_proto_t eth_type);

adk_status_t adk_ethernet_ethertype_p_bits_map_add(
                    adk_module_handle_t eth_mod_handle,
                    adk_ethernet_proto_t eth_type,
                    adk_ethernet_priority_t pcp);

typedef void (*adk_eth_mac_dump_cb) (
    void *,
    adk_ethernet_mac_table_entry_t *);

adk_status_t
adk_ethernet_mac_table_dump (
    adk_module_handle_t eth_mod_handle,
    adk_eth_mac_table_options_t mac_table_dump,
    adk_eth_mac_dump_cb mac_dump_cb,
    void* handle);

adk_status_t
adk_ethernet_dscp_p_bits_map_add (
    adk_module_handle_t  eth_mod_handle,
    uint8_t dscp,
    adk_ethernet_priority_t  pcp);

adk_status_t
adk_ethernet_dscp_p_bits_map_del (
    adk_module_handle_t  eth_mod_handle,
    uint8_t dscp);

adk_status_t
adk_ethernet_vlan_eth_type_add (
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    adk_ethernet_proto_t eth_type);

adk_status_t adk_ethernet_ethertype_based_qos_policy_set(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t ifHandle,
    adk_ethernet_proto_t ethType,
    adk_qos_queue_id_t  queueId);

adk_status_t adk_ethernet_ethertype_based_qos_policy_rem(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t ifHandle,
    adk_ethernet_proto_t ethType);

adk_status_t adk_ethernet_non_bridge_origin_queue_id_set(
    adk_module_handle_t eth_mod_handle,
    adk_qos_queue_id_t  queueId);

adk_status_t adk_ethernet_non_bridge_origin_queue_id_reset(
    adk_module_handle_t eth_mod_handle);

adk_status_t adk_ethernet_bridge_timestamp_quad_group_set (
        adk_module_handle_t eth_mod_handle,
        adk_ethernet_quad_group_t quadGroup);
#endif /* #ifndef _ADK_ETHERNET_H_ */

/*! end addtogroup _ethernetFunctions
 * @}
 *
 * end addtogroup _ethernet
 * @}
 */
