/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet_conf.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernet  Ethernet Module
 * @{
 *
 * @brief   This file has all the compile time configurable constants used by
 *          Ethernet module.
 *
 *  **********************************************************************/

#ifndef _ADK_ETH_CONF_H_
#define _ADK_ETH_CONF_H_

/*! The default VLAN tag used by the Ethernet module to assign to the packets
 * received on Access ports and to assign untagged packets received on Trunk and
 * Tunnel ports. */
#define ADK_ETH_DEFAULT_VLAN                0
#define ADK_ETH_DEFAULT_PCP                 0

#define ADK_ETH_DEVICE_MAX                  8
#define ADK_ETH_PORTS_MAX                   64
#define ADK_ETH_PORT_MASK                   0x3F
#define ADK_ETH_ENCAP_FLOWS_MAX             2048
#define ADK_ETH_INGRESS_FLOWS_MAX           1024
#define ADK_ETH_NETD_TIMESTAMP_FLOWS_ID     1025
#define ADK_ETH_EGRESS_FLOWS_MAX           256
#define ADK_ETH_IF_SUB_IF_MAX               256
#define ADK_ETH_IF_SVIF_IF_MAX              256

/*! Maximum number of LAG interfaces that are supported */
#define ADK_ETH_LAG_INTF_MAX                3

/* Maximum number of physical member port or links within a LAG interface */
#define ADK_ETH_MAX_INTF_IN_AGGR            4

/*! The default task priority to be used in the MPP while sending task to other
 * engines */
#define ETH_TASK_PRIORITY                   4

/*! The default task priority to be used in the MPP while sending task to
 * Expander engine for multicast */
#define ETH_MCAST_TASK_PRIORITY             4

/*! The default task priority to be used in the MPP while sending time stamp
 * allocation responses to NCA */
#define TIME_INDEX_TASK_PRIORITY            0

/*! The default age-out time duration for the time slot index hash table
 */
#define TIME_INDEX_TABLE_TIMER_VALUE       0x80000001 /* 1 seconds */

/*! The default age-out time duration for the learned MAC addresses. This value
 * needs to be in Neuvo format */
#ifdef ADK_HW_PLATFORM
#define DEFAULT_MAC_TABLE_TIMER_VALUE   0x8000012C  /* 5 min */
#else /* Some small values for simulation platform */
#define DEFAULT_MAC_TABLE_TIMER_VALUE   0x00001000
#endif

#define PCX_DEFAULT_MAC_TABLE_TIMER             5 /* 5 Minutes */

/*! The default age-out time duration for persistent flow->link assignment
 * within the aggregate interface */
#ifdef ADK_HW_PLATFORM
#define AGGREGATION_TABLE_TIMER_VALUE   0x80000001  /* 1 sec */
#else /* Some small values for simulation platform */
#define AGGREGATION_TABLE_TIMER_VALUE   0x00001000
#endif

/*! Use of multicast MAC/IPv4 address is allowed in ARP table by default.
 * Undefine below line (comment out) to disallow multicast IP/MAC addresses in
 * ARP table */
#define ETH_ALLOW_MCAST_MAC_IP_ARP

/* #ifdef ADK_XEON2_PROFILE_1 */
/*! Maximum number of MAC addresses that will be learnt by the system. Note
 * that, this is not a absolute maximum but is a rough estimate */
#define ADK_ETH_NUM_MACS_MAX        12000

/*! The maximum number of multicast group IDs supported by Ethernet module */
#define ADK_ETH_NUM_MCAST_GROUPS    1024

/*! The maximum number of flows supported by Ethernet module (excluding
 * encapsulation flows). */
#define ADK_ETH_NUM_ETH_FLOWS       60000

/*! The maximum number of switch virtual interfaces supported by Ethernet module
 * */
#define ADK_ETH_NUM_SVIF            64
/*! The maximum number of bridge domains support by Ethernet module
 * */
#define ADK_ETH_NUM_BRIDGE          64
/* #endif */

#define ETH_MAC_TABLE_FLUSH_MSG_SIZE (ETH_MAC_ADDR_SIZE + ETH_VLAN_ID_SIZE)
#define ETH_MAC_TABLE_FLUSH_MSG_MAX_NUM  1

#define ETH_MAC_FDB_CPU_BASED       1

#define ADK_ETH_NUM_ETHERTYPES      16

#endif /* #ifndef _ADK_ETH_CONF_H_ */

/*!
 * end addtogroup _ethernet
 * @}
 */
