/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet_utility_counters.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernetUtilityCounters  IPsec Utility Counters
 * @{
 *
 * @brief
 *
 * This file contains ADK APIs to enable/disable utility counters and get API to fetch the counter values. 
 *
 ***********************************************************************/

#ifndef _ADK_ETH_UTILITY_COUNTER_H_
#define _ADK_ETH_UTILITY_COUNTER_H_

/*!
 * =============================================================
 * @internal
 * @struct eth_ing_utility_counters_t "adk_ethernet_utility_counters.h"
 *
 * @brief
 *
 * Data structure for ingress utility counters.
 * =============================================================
 */

typedef struct eth_ing_utility_counters_s
{

    /*! Number of packets entered on eth lli */
    uint64_t eth_ing_pkt_proc_lli_entry_count;
    /*! Number of packets dropped on eth lli */
    uint64_t eth_ing_pkt_proc_lli_drop;
    /*! Number of non-L2 terminating packets received*/
    uint64_t eth_ing_non_l2term_frames_received;
    /*! Number of phy terminating packets sent to netd */
    uint64_t eth_ing_phy_term_pkt_sent_to_netd;
    /*! Number of packets sent to netd */
    uint64_t eth_ing_pkt_sent_to_netd;
    /*! Number of packets sent to L3 module */
    uint64_t eth_ing_pkts_sent_to_l3;
    /*! Number of packets sent to IPv6 module */
    uint64_t eth_ing_pkts_sent_to_ipv6;
    /*! Number of packets sent to IPv4 module */
    uint64_t eth_ing_pkts_sent_to_ipv4;
    /*! number of packets sent to ethernet policer stage */
    uint64_t eth_ing_pkts_sent_to_policer_stage;
    /*! number of packets received in ethernet policer stage */
    uint64_t eth_ing_pkts_recv_in_policer_stage;
    /*! number of packets vlan_header is stripped */
    uint64_t eth_ing_pkts_vlan_tag_stripped;
    /*! Number of broadcast/multicast received */
    uint64_t eth_ing_multicast_packets_received;
    /*! Number of packets going for SW ingress flow lookup*/
    uint64_t eth_ing_pkt_sent_to_sw_ingress_flow_lookup;
} eth_ing_utility_counters_t;

/*!
 * =============================================================
 * @internal
 * @struct eth_eg_utility_counters_t "adk_ethernet_utility_counters.h"
 *
 * @brief
 *
 * Data structure for egress utility counters.
 * =============================================================
 */

typedef struct eth_eg_utility_counters_s
{
    /*! Number of packets entered on eth uli */
    uint64_t eth_eg_pkt_proc_uli_entry_count;
    /*! Number of packets received from IPv4 module */
    uint64_t eth_eg_ipv4_pkts_received;
    /*! Number of packets received from IPv6 module */
    uint64_t eth_eg_ipv6_pkts_received;
    /*! Number of Multicast packets received from IPv6 module */
    uint64_t eth_eg_ipv6_multicast_pkts_received;
    /*! Number of egress packets received from netd*/
    uint64_t eth_eg_pkts_received_from_netd;
    /*! Number of egress non bridge netd origin packets received */
    uint64_t eth_eg_non_bridge_netd_origin_pkts_received;
    /*! Number of egress non bridge netd origin packets transmitted */
    uint64_t eth_eg_non_bridge_netd_origin_pkts_transmitted;
    /*! Number of packets sent to nic*/
    uint64_t eth_eg_pkts_sent_to_nic;
    /*! Number of packets sent to ph*/
    uint64_t eth_eg_pkts_sent_to_ph;
    /*! Number of packets dropped on eth uli */
    uint64_t eth_eg_pkt_proc_uli_drop;
    /*! Number of packets for which arp exception sent */
    uint64_t eth_eg_pkt_arp_exception_sent;
    /*! Number of packets for which nd exception sent */
    uint64_t eth_eg_pkt_nd_exception_sent;
    /*! Number of packets for which stale entry used exception sent*/
    uint64_t eth_eg_pkt_stale_entry_used_exception_sent;
    /*! Number of packets sent to buffering stage */
    uint64_t eth_eg_pkt_sent_to_buffering_stage;
    /*! Number of packets received in buffering stage */
    uint64_t eth_eg_pkt_recv_in_buffering_stage;
    /*! Number of packets dropped in buffering stage */
    uint64_t eth_eg_pkt_dropped_in_buffering_stage;
    /*! nexthop buffer info free error */
    uint64_t eth_eg_nexthop_buffer_info_free_error;
    /*! nexthop buffer Lock Allocation error */
    uint64_t eth_eg_nexthop_buffer_lock_alloc_error;
    /*! Number of IPv4 nexthop buffer overflow error*/
    uint64_t eth_eg_ipv4_nexthop_buffer_overflow_error;
    /*! Number of IPv4 nexthop buffer timer create failure */
    uint64_t eth_eg_ipv4_nexthop_buffer_timer_create_failure;
    /*! Number of IPv4 nexthop buffer timeout */
    uint64_t eth_eg_ipv4_nexthop_buffer_timeout_count;
    /*! Number of IPv4 nexthop buffer timer restart error */
    uint64_t eth_eg_ipv4_nexthop_buffer_timer_restart_error;
    /*! Number of IPv6 nexthop buffer overflow error*/
    uint64_t eth_eg_ipv6_nexthop_buffer_overflow_error;
    /*! Number of IPv6 nexthop buffer timer create failure */
    uint64_t eth_eg_ipv6_nexthop_buffer_timer_create_failure;
    /*! Number of IPv6 nexthop buffer timeout */ 
    uint64_t eth_eg_ipv6_nexthop_buffer_timeout_count;
    /*! Number of IPv6 nexthop buffer timer restart error */
    uint64_t eth_eg_ipv6_nexthop_buffer_timer_restart_error;
    /*! Number of packets timestamp alloc request*/
    uint64_t eth_eg_pkts_tstamp_alloc_requests;
    /*! Number of packets timestamp free request*/
    uint64_t eth_eg_pkts_tstamp_free_requests;
    /*! Number of packets timestamp alloc success*/
    uint64_t eth_eg_pkts_tstamp_alloc_success;
} eth_eg_utility_counters_t;

/*!
 * ================================================================
 * @struct adk_eth_utility_counters_t adk_ethernet_utility_counters.h "adk_ethernet_utility_counters.h"
 *
 * @brief
 *
 * The data structure is used to define utility counters maintained by
 * the Ethernet ADK
 *
 * ================================================================
 */
typedef struct adk_eth_utility_counters_s {

    /*! ingress side utility counters */
    eth_ing_utility_counters_t    ingressUtilityCounters;
    /*! egress side utility counters */
    eth_eg_utility_counters_t     egressUtilityCounters;

} adk_eth_utility_counters_t; /* end of struct adk_eth_utility_counters_t */

/*!
 * =============================================================
 *
 * adk_eth_set_utility_counters
 *
 * @param[in]  ethHandle  - Ethernet module handle. 
 * @param[in]   utility_cnt_state - Boolean variable to enable/disable
                                    utility counter.
 *
 * @brief   Function to configure utility counters.
 *
 * This function configures utility counters.On disabling the utility
 * counters, all the values will get reset to zero.
 *
 * =============================================================
 */
adk_status_t
adk_eth_set_utility_counters(adk_module_handle_t ethHandle, bool utility_cnt_state);

/*!
 * =============================================================
 * @fn adk_eth_get_utility_counters
 *
 * @param[in]    ethHandle - Ethernet module handle.
 *
 * @param[out]   ethUtilityCounter_p - Ethernet related utility counter data structure pointer
 *               gets set by the Ethernet ADK
 *
 * @retval ADK_SUCCESS - Successfully printed utility counters
 * @retval ADK_E_INVALID_HANDLE - eth module handle is invalid.
 * @retval  ADK_E_INVALID_PARAMETER - Invalid parameter
 *
 * @brief
 *
 * This API allows the application to retreive all Ethernet related utility counters
 * maintained by the ETH ADK module. It is expected that the application
 * would have allocated a chunk of memory equivalent to hold data structure of
 * size adk_eth_utility_counters_t.
 *
 * =============================================================
 */
adk_status_t
adk_eth_get_utility_counters(adk_module_handle_t           ethHandle,
                               adk_eth_utility_counters_t *ethUtilityCnt_p);

/*!
 * =============================================================
 * @fn adk_eth_print_utility_counters
 *
 * @param[in]    ethHandle - Ethernet module handle.
 *
 * @retval ADK_SUCCESS - Successfully printed utility counters
 * @retval ADK_E_INVALID_HANDLE - eth module handle is invalid.
 * @retval  ADK_E_INTERNAL_ERROR -  Internal error 
 *
 * @brief
 *
 * This API allows the application to print all ethernet related utility counters
 * maintained by the Ethernet ADK module.
 *
 * =============================================================
 */

adk_status_t
adk_eth_print_utility_counters(adk_module_handle_t     ethHandle);

#endif
