/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_eth_netlink.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernet  Ethernet Module
 * @{
 *      @addtogroup _ethernetFunctions  Ethernet Functions
 *      @{
 *
 * @brief   This files has the source code for ethernet and netdriver
 *          netlink communication
 *
 *  **********************************************************************/

#if !defined(ETH_ALEN)
 #define ETH_ALEN 6
#endif

#ifdef ADK_NETD_NETLINK

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netpacket/packet.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <linux/sockios.h>
#include <sys/ioctl.h>
#include <linux/net_tstamp.h>
#include <linux/if.h>
#include <string.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <linux/netlink.h>
#include "linux/adk_netd_vei_if.h"
#include "linux/adk_netd_gen_comm.h"
#include "adk_eth.h"

#define MAX_PAYLOAD 2048 /* maximum payload size*/

int netlink_chk_recv_ack(int sock_fd);

int netlink_chk_recv_ack(int sock_fd)
{
    unsigned int len;
    char buf[4096];
    struct iovec iov1 = { buf, sizeof(buf) };
    struct sockaddr_nl sa;
    struct nlmsghdr *nh1;
    struct adk_netd_vei_ret_t *msg2;
    struct msghdr msg1=
        { (void *)&sa, sizeof(sa), &iov1, 1, NULL, 0, 0 };

    len = recvmsg(sock_fd, &msg1, 0);

    for (nh1 = (struct nlmsghdr *) ((void *)buf);
    NLMSG_OK (nh1, len) ;
    nh1 = (struct nlmsghdr *)((void *)((NLMSG_NEXT (nh1, len))))
    ) 
    {
        /* The end of multipart message. */
        if (nh1->nlmsg_type == NLMSG_DONE)
        {
            msg2 = (struct adk_netd_vei_ret_t *) (NLMSG_DATA(nh1));
            printf("msg send done.\n");
            printf("msg2->origMsg.mesgCode = %d\n", msg2->origMsg.mesgCode);
            if (msg2->returnCode == 0)
                printf("SUCCESS!!!\n");
            else
                printf("FAILED: msg2->returnCode = %d\n", msg2->returnCode);


            switch(msg2->origMsg.mesgCode)
            {
                case ADK_NETD_VEI_CREATE_VIRT_ETH_IF:

                if (msg2->returnCode == 0)
                {
                    printf("ifID successfully created\n");
                }

                printf("Orig msg contents..\n");
                printf("VEI name=%s\n", msg2->origMsg.nlMsg.createIfMsg.name);
                printf("ADK Id =%d\n", msg2->origMsg.nlMsg.createIfMsg.adkId);
                break;

                case ADK_NETD_VEI_DELETE_VIRT_ETH_IF:

                printf("Orig msg contents..\n");
                printf("Interface Id =%d\n", msg2->origMsg.nlMsg.deleteIfMsg.adkId);
                break;


                case ADK_NETD_VEI_GET_STATS:
                printf("STATS..\n");
                if (msg2->returnCode == 0)
                {
                    printf("total pkts Tx from stack=%lu\n", msg2->stats.total_net_tx_packets);
                    printf("total pkts sent to EIOA =%lu\n", msg2->stats.total_tx_packets);

                    printf("total bytes Tx from stack=%lu\n", msg2->stats.total_net_tx_bytes);
                    printf("total bytes sent to EIOA=%lu\n", msg2->stats.total_tx_bytes);

                    printf("total pkts dropped =%lu\n", msg2->stats.total_tx_drop_packets);

                    printf("total pkts recvd from EIOA =%lu\n", msg2->stats.total_rx_packets);
                    printf("total pkts sent to stack=%lu\n", msg2->stats.total_net_rx_packets);

                    printf("total bytes recvd from  EIOA=%lu\n", msg2->stats.total_rx_bytes);
                    printf("total bytes sent to stack =%lu\n", msg2->stats.total_net_rx_bytes);

                    printf("total Rx pkts dropped =%lu\n", msg2->stats.total_rx_drop_packets);
                }
                break;


            default:
                break;
            }

        }
     else
        if (nh1->nlmsg_type == NLMSG_ERROR)
        {
            /* Do some error handling. */
            printf("nlmsg error.\n");
        }

    /* Continue with parsing payload. */

    }
   return 0;
}

adk_status_t netd_bridge_interface_up(adk_module_handle_t eth_mod_handle)
{
    adk_status_t ret = ADK_SUCCESS;
    struct sockaddr_nl src_addr, dest_addr;
    struct nlmsghdr *nlh = NULL;
    int sock_fd;
    struct iovec iov;
	struct msghdr msg;
	struct adk_netd_vei_msg_t *msgPtr;
	int ifId;
	int linkStatus;
    adk_sys_handle_t sys_handle;

	int seq=0;

    ifId = 0;
    linkStatus = 1;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);
	sock_fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_VEI_NETDEV );

	if (sock_fd == -1)
	{
        ret = ADK_E_NETD_SOCK_OPEN_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket open failed\n");
		return ret;
	}

    ADK_ETH_LOG_INFO(eth_mod_handle, "opened socket sock_fd=%d\n", sock_fd);

	memset(&src_addr, 0, sizeof(src_addr));
	src_addr.nl_family = AF_NETLINK;
	src_addr.nl_pid = getpid();  /* self pid */
	src_addr.nl_groups = 0;  /* not in mcast groups */
	if (bind(sock_fd, (struct sockaddr*)&src_addr, sizeof(src_addr)) < 0)
	{
        ret = ADK_E_NETD_SOCK_BIND_FAILED;
        close(sock_fd);
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket bind failed\n");
		return ret;
	}

    ADK_ETH_LOG_INFO(eth_mod_handle, "Socket bind doned\n");

	memset(&dest_addr, 0, sizeof(dest_addr));
	dest_addr.nl_family = AF_NETLINK;
	dest_addr.nl_pid = 0;   /* For Linux Kernel */
	dest_addr.nl_groups = 0; /* unicast */


    nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
    if(NULL == nlh)
	    return ADK_E_MEM_UNAVAILABLE;

    memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
    nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);

	nlh->nlmsg_pid = getpid();  /* self pid */
	nlh->nlmsg_flags =  NLM_F_ACK|NLM_F_REQUEST;
	nlh->nlmsg_seq =  seq++;

	msgPtr = (struct adk_netd_vei_msg_t *)(NLMSG_DATA(nlh));
	msgPtr->mesgCode = ADK_NETD_VEI_LNK_STATUS_CHANGE;

	msgPtr->nlMsg.linkStsMsg.type = ADK_NETD_VEI_TYPE_1;
	msgPtr->nlMsg.linkStsMsg.adkId = ifId;
	msgPtr->nlMsg.linkStsMsg.linkStatus =  linkStatus;

	memset(&msg, 0, sizeof(msg));

	iov.iov_base = (void *)nlh;
	iov.iov_len = nlh->nlmsg_len;
	msg.msg_name = (void *)&dest_addr;
	msg.msg_namelen = sizeof(dest_addr);
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;

	if (sendmsg(sock_fd, &msg, 0) < 0)
	{
        ret = ADK_E_NETD_SOCK_MSG_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket sendmsg failed(errno=%d)\n", errno);
        close(sock_fd);
        free(nlh);
		return ret;
	}

    ADK_ETH_LOG_INFO(eth_mod_handle, "Link status msg sent\n");


#ifdef ADK_DEBUG
	/*netlink_chk_recv_ack(sock_fd);*/
#endif

	close(sock_fd);
    free(nlh);
    return ADK_SUCCESS;
}


adk_status_t netd_bridge_interface_create
(
adk_module_handle_t eth_mod_handle,
const uint8_t* bridge_mac
)
{
    adk_status_t ret = ADK_SUCCESS;
    struct msghdr msg;
    struct adk_netd_vei_msg_t *msgPtr; 
    int seq=0;
    int adkId=0;
    int sock_fd;
    struct sockaddr_nl src_addr, dest_addr;
    struct nlmsghdr *nlh = NULL;
    struct iovec iov;
    adk_sys_handle_t sys_handle;
    /* IP Address of the interface. */
    
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);
    sock_fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_VEI_NETDEV);

    if (sock_fd == -1)
    {
        ret = ADK_E_NETD_SOCK_OPEN_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket open failed\n");
		return ret;
    }

    memset(&src_addr, 0, sizeof(src_addr));
    src_addr.nl_family = AF_NETLINK;
    src_addr.nl_pid = getpid();  /* self pid */
    src_addr.nl_groups = 0;  /* not in mcast groups */
    if (bind(sock_fd, (struct sockaddr*)&src_addr, sizeof(src_addr)) < 0)
    {
        ret = ADK_E_NETD_SOCK_BIND_FAILED;
        close(sock_fd);
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket bind failed\n");
		return ret;
    }


    /* Send a Create VEI1 msg */
    memset(&dest_addr, 0, sizeof(dest_addr));
    dest_addr.nl_family = AF_NETLINK;
    dest_addr.nl_pid = 0;   /* For Linux Kernel */
    dest_addr.nl_groups = 0; /* unicast */

    /* create VEI1 */


    nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
    if(NULL == nlh)
	    return ADK_E_MEM_UNAVAILABLE;

    memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
    nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
    nlh->nlmsg_pid = getpid();  /* self pid */
    nlh->nlmsg_flags =  NLM_F_ACK|NLM_F_REQUEST;
    nlh->nlmsg_seq =  seq++;

    msgPtr = (struct adk_netd_vei_msg_t *)(NLMSG_DATA(nlh));
    msgPtr->mesgCode = ADK_NETD_VEI_CREATE_VIRT_ETH_IF ;
    strcpy( msgPtr->nlMsg.createIfMsg.name, "br0"); 
    msgPtr->nlMsg.createIfMsg.type =  ADK_NETD_VEI_TYPE_1;
    msgPtr->nlMsg.createIfMsg.adkId =  adkId;
    msgPtr->nlMsg.createIfMsg.macAddr[0]=bridge_mac[0];
    msgPtr->nlMsg.createIfMsg.macAddr[1]=bridge_mac[1];
    msgPtr->nlMsg.createIfMsg.macAddr[2]=bridge_mac[2];
    msgPtr->nlMsg.createIfMsg.macAddr[3]=bridge_mac[3];
    msgPtr->nlMsg.createIfMsg.macAddr[4]=bridge_mac[4];
    msgPtr->nlMsg.createIfMsg.macAddr[5]=bridge_mac[5];


    memset(&msg, 0, sizeof(msg));

    iov.iov_base = (void *)nlh;
    iov.iov_len = nlh->nlmsg_len;
    msg.msg_name = (void *)&dest_addr;
    msg.msg_namelen = sizeof(dest_addr);
    msg.msg_iov = &iov;
    msg.msg_iovlen = 1;

    if (sendmsg(sock_fd, &msg, 0) < 0)
    {
        ret = ADK_E_NETD_SOCK_MSG_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket sendmsg failed(errno=%d)\n", errno);
        close(sock_fd);
        free(nlh);
		return ret;
    }

#ifdef ADK_DEBUG
    /*netlink_chk_recv_ack(sock_fd);*/
#endif

    close(sock_fd);
    free(nlh);
    return ret;
}

adk_status_t netd_bridge_interface_delete(adk_module_handle_t eth_mod_handle)
{
    adk_status_t ret = ADK_SUCCESS;
    struct msghdr msg;
    struct sockaddr_nl src_addr, dest_addr;
    int sock_fd;
    struct nlmsghdr *nlh = NULL;
    adk_sys_handle_t sys_handle;
    int adkId=0;
    struct adk_netd_vei_msg_t *msgPtr; 
    struct iovec iov;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);
    sock_fd = socket(PF_NETLINK, SOCK_RAW, NETLINK_VEI_NETDEV);

    if (sock_fd == -1)
    {
        ret = ADK_E_NETD_SOCK_OPEN_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket open failed\n");
		return ret;
    }

    memset(&src_addr, 0, sizeof(src_addr));
    src_addr.nl_family = AF_NETLINK;
    src_addr.nl_pid = getpid();  /* self pid */
    src_addr.nl_groups = 0;  /* not in mcast groups */
    if (bind(sock_fd, (struct sockaddr*)&src_addr, sizeof(src_addr)) < 0)
    {
        ret = ADK_E_NETD_SOCK_BIND_FAILED;
        close(sock_fd);
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket bind failed\n");
		return ret;
    }

	memset(&dest_addr, 0, sizeof(dest_addr));
	dest_addr.nl_family = AF_NETLINK;
	dest_addr.nl_pid = 0;   /* For Linux Kernel */
	dest_addr.nl_groups = 0; /* unicast */

	/* Fill the netlink message header */
    nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
    if(NULL == nlh)
	    return ADK_E_MEM_UNAVAILABLE;

    memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
    nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
	nlh->nlmsg_pid = getpid();  /* self pid */
	nlh->nlmsg_flags =  NLM_F_ACK|NLM_F_REQUEST;
	nlh->nlmsg_seq =  0;

	msgPtr = (struct adk_netd_vei_msg_t *)(NLMSG_DATA(nlh));
	msgPtr->mesgCode = ADK_NETD_VEI_DELETE_VIRT_ETH_IF;
	msgPtr->nlMsg.deleteIfMsg.type = ADK_NETD_VEI_TYPE_1;
	msgPtr->nlMsg.deleteIfMsg.adkId = adkId;
#if 0
	strcpy( msgPtr->nlMsg.statsReqMsg.name, interface);
#endif

	memset(&msg, 0, sizeof(msg));
	iov.iov_base = (void *)nlh;
	iov.iov_len = nlh->nlmsg_len;
	msg.msg_name = (void *)&dest_addr;
	msg.msg_namelen = sizeof(dest_addr);
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;

	if (sendmsg(sock_fd, &msg, 0) < 0)
	{
        ret = ADK_E_NETD_SOCK_MSG_FAILED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Socket sendmsg failed(errno=%d)\n", errno);
        close(sock_fd);
        free(nlh);
		return ret;
	}

    ADK_ETH_LOG_INFO(eth_mod_handle, "Delete VEI msg sent.\n");

#ifdef ADK_DEBUG
	/* netlink_chk_recv_ack(); */
#endif

	close(sock_fd);
    free(nlh);
    return ret;
}

#endif /*ADK_NETD_NETLINK*/

/*!     end addtogroup _ethernetFunction
 *      @}
 *  end addtogroup _ethernet
 * @}
 */
