/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth_tables.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernetFunctions  Ethernet Module
 *
 * @{
 *
 * @brief   This file has all the low level functions which operate upon
 *          data plane tables.
 *
 *  **********************************************************************/
#include <adk_eth.h>
#include "adk_eth_dp_pvt.h"

adk_dp_status_t
adk_eth_dp_table_init (
    adk_module_handle_t eth_mod_handle,
    adk_eth_dp_pvt_data_t *eth_dp_pvt_p)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    adk_dp_eth_svif_if_hash_t svif_if_hash_table_entry;
    adk_dp_eth_ethertype_queue_map_hash_t  ethtype_queue_map_hash_table_entry;
    adk_sys_handle_t sys_handle;
    uint32_t i;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    /* initialize data path related tables and structures */
    ADK_MEMSET(eth_dp_pvt_p, 0, sizeof(adk_eth_dp_pvt_data_t));

    /* Initialize Ethernet global stats structures */
    eth_dp_pvt_p->gl_stats_p = (adk_dp_eth_gl_stats_t *)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
			 ADK_ETH_STATS_OBJ,
			 ADK_MAX_NUM_CORES, sizeof(adk_dp_eth_gl_stats_t),
                          ADK_ETH_DP_OBJECT_ALIGN);
    if (eth_dp_pvt_p->gl_stats_p == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet global stats structure!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->eth_fc_ver++;

    /* Initialize svif-interface table */
    eth_dp_pvt_p->eth_svif_if = (adk_dp_eth_svif_intf_t **)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
			 ADK_ETH_SVIF_INTF_OBJ,
			 ADK_ETH_IF_SVIF_IF_MAX,
                          sizeof(adk_dp_eth_svif_intf_t *),
                          ADK_ETH_DP_PTR_ALIGN);
    if (eth_dp_pvt_p->eth_svif_if == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet svif-interface table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize svif-interface hash table */
    eth_dp_pvt_p->eth_svif_if_hash_table = (adk_dp_eth_svif_if_hash_t *)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
			 ADK_ETH_SVIF_HASH_INTF_OBJ,
			 ADK_ETH_SVIF_INTF_MAX,
                          sizeof(adk_dp_eth_svif_if_hash_t),
                          ADK_ETH_DP_PTR_ALIGN);
    if (eth_dp_pvt_p->eth_svif_if_hash_table == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate svif-interface hash table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    svif_if_hash_table_entry.l2_domain_id = 0;
    svif_if_hash_table_entry.vlan = 0;
    svif_if_hash_table_entry.stats_id = ETH_INVALID_NS_ID;
    for (i = 0; i < ADK_ETH_SVIF_INTF_MAX; i++)
    {
        eth_dp_pvt_p->eth_svif_if_hash_table[i].hash_entry =
            svif_if_hash_table_entry.hash_entry;
    }

    /* Initialize Ethernet arp Lookup table */
    eth_dp_pvt_p->eth_arp_hash_table.hash_tbl = (adk_dp_eth_neighbor_hash_entry_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                ADK_ETH_ARP_OBJ,
                (ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY*ADK_ETH_NO_OF_NEIGH_ENTRY),
                sizeof(adk_dp_eth_neighbor_hash_entry_t),
                ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_arp_hash_table.hash_tbl == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate arp hash table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->eth_arp_hash_table.tbl = (adk_dp_eth_arp_entry_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                ADK_ETH_ARP_OBJ,
                ADK_ETH_NO_OF_NEIGH_ENTRY,
                sizeof(adk_dp_eth_arp_entry_t),
                ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_arp_hash_table.tbl == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate arp hash index table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    ADK_ETH_DP_NEIGH_TABLE_IDX_INIT(eth_dp_pvt_p->eth_arp_hash_table,
            ADK_ETH_NO_OF_NEIGH_ENTRY);

    eth_dp_pvt_p->eth_arp_hash_table.overflow_list.head = NULL;
    eth_dp_pvt_p->eth_arp_hash_table.overflow_list.count = 0;
    eth_dp_pvt_p->eth_arp_hash_table.overflow_pending_list.head = NULL;
    eth_dp_pvt_p->eth_arp_hash_table.overflow_pending_list.count = 0;

    /* Initialize arp buffer table */
    eth_dp_pvt_p->eth_arp_buffer_tbl = (adk_eth_dp_arp_buffer_tbl_t *)
            ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
            ADK_ETH_ARP_BUFFER_OBJ,
            1, ADK_ETH_ARP_BUFFER_TBL_HASH_MAX *
            ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION * sizeof(adk_eth_dp_arp_buffer_tbl_t),
            ADK_ETH_DP_PTR_ALIGN);
    if(!eth_dp_pvt_p->eth_arp_buffer_tbl)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet arp buffer"
                          " table!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize arp buffer lock */
    eth_dp_pvt_p->arp_buffer_lock = (void *) ADK_ALLOC_LOCK();
    if(NULL == eth_dp_pvt_p->arp_buffer_lock)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet arp buffer"
                          " lock!\n");
        goto _adk_eth_dp_table_init_exit;
    }
    ADK_INIT_LOCK(eth_dp_pvt_p->arp_buffer_lock,
            ADK_LOCK_SHARED,
            ADK_LOCK_VALUE); 

    /* Initialize Ethernet neighbor Lookup table */
    eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl = (adk_dp_eth_neighbor_hash_entry_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                ADK_ETH_NEIGH_OBJ,
                (ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY*ADK_ETH_NO_OF_NEIGH_ENTRY),
                sizeof(adk_dp_eth_neighbor_hash_entry_t),
                ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate neighbor hash table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->eth_neighbor_hash_table.tbl = (adk_dp_eth_neighbor_entry_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                ADK_ETH_NEIGH_OBJ,
                ADK_ETH_NO_OF_NEIGH_ENTRY,
                sizeof(adk_dp_eth_neighbor_entry_t),
                ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_neighbor_hash_table.tbl == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate neighbor hash index table\n");
        goto _adk_eth_dp_table_init_exit;
    }


    ADK_ETH_DP_NEIGH_TABLE_IDX_INIT(eth_dp_pvt_p->eth_neighbor_hash_table,
            ADK_ETH_NO_OF_NEIGH_ENTRY);

    eth_dp_pvt_p->eth_neighbor_hash_table.overflow_list.head = NULL;
    eth_dp_pvt_p->eth_neighbor_hash_table.overflow_list.count = 0;
    eth_dp_pvt_p->eth_neighbor_hash_table.overflow_pending_list.head = NULL;
    eth_dp_pvt_p->eth_neighbor_hash_table.overflow_pending_list.count = 0;


    /* Initialize neighbor buffer table */
    eth_dp_pvt_p->eth_neighbor_buffer_tbl = (adk_eth_dp_neighbor_buffer_tbl_t *)
            ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
            ADK_ETH_NEIGH_BUFFER_OBJ,
            1, ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MAX *
            ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION * sizeof(adk_eth_dp_neighbor_buffer_tbl_t),
            ADK_ETH_DP_PTR_ALIGN);
    if(!eth_dp_pvt_p->eth_neighbor_buffer_tbl)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet neighbor buffer"
                          " table!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize neighbor buffer lock */
    eth_dp_pvt_p->neighbor_buffer_lock = (void *) ADK_ALLOC_LOCK();
    if(NULL == eth_dp_pvt_p->neighbor_buffer_lock)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet neighbor buffer"
                          " lock!\n");
        goto _adk_eth_dp_table_init_exit;
    }
    ADK_INIT_LOCK(eth_dp_pvt_p->neighbor_buffer_lock,
            ADK_LOCK_SHARED,
            ADK_LOCK_VALUE);

    /* Initialize ipv4 neighbor buffer timer handle list */
    eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl = (adk_eth_dp_nexthop_v4_timer_index_tbl_t*)
            ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
            ADK_ETH_NEXTHOP_V4_TIMER_INDEX_OBJ,
            1, (ADK_ETH_ARP_BUFFER_TIMER_INDEX_MAX*sizeof(adk_eth_dp_nexthop_v4_timer_index_tbl_t)), 
            ADK_ETH_DP_PTR_ALIGN);

    if(!eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate Ethernet IPv4 neighbor buffer"
                " timer table!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize ipv6 neighbor buffer timer handle list */
    eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl = (adk_eth_dp_nexthop_v6_timer_index_tbl_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                ADK_ETH_NEXTHOP_V6_TIMER_INDEX_OBJ,
                1, (ADK_ETH_NEIGHBOR_BUFFER_TIMER_INDEX_MAX * sizeof(adk_eth_dp_nexthop_v6_timer_index_tbl_t)),
                ADK_ETH_DP_PTR_ALIGN);

    if(!eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate Ethernet IPv6 neighbor buffer"
                " timer table!\n");
        goto _adk_eth_dp_table_init_exit;
    }


    /*create mempool for eth buffer info*/
    eth_dp_pvt_p->eth_buffer_info_pool = rte_mempool_create(ADK_ETHERNET_DP_BUFFER_INFO_POOL_NAME,
                                                            ADK_ETHERNET_MAX_NO_OF_BUFFER_INFO,
                                                            sizeof(adk_dp_eth_buffer_info_t),
                                                            ADK_ETHERNET_DP_BUFFER_INFO_POOL_CACHE_SIZE,
                                                            0, NULL, NULL, NULL, NULL, rte_socket_id(), 0);

    if(eth_dp_pvt_p->eth_buffer_info_pool == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to create ethernet buffer info mempool\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /*create mempool for eth nexthop buffer*/
    eth_dp_pvt_p->eth_nexthop_buffer_pool = rte_mempool_create(ADK_ETHERNET_DP_NEXTHOP_BUFFER_POOL_NAME,
                                                            ADK_ETHERNET_MAX_NO_OF_NEXTHOP_BUFFER,
                                                            sizeof(adk_dp_eth_next_hop_buffer_t),
                                                            ADK_ETHERNET_DP_NEXTHOP_BUFFER_POOL_CACHE_SIZE,
                                                            0, NULL, NULL, NULL, NULL, rte_socket_id(), 0); 
    if(eth_dp_pvt_p->eth_nexthop_buffer_pool == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to create ethernet nexthop buffer mempool\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize Ethernet Ingress Flow Lookup table */
    eth_dp_pvt_p->ingressFlowTbl = adk_lpm_create(
					ADK_MODULE_MODULEID_GET(eth_mod_handle),
                                        ADK_ETH_INGRESS_FLOW_OBJ, ADK_ETH_INGRESS_FLOW_LOOKUP_TABLE_WIDTH,
                                        ADK_LPM_DEFAULT_PENDING_SIZE);
    if (eth_dp_pvt_p->ingressFlowTbl == (adk_lpm_handle_t) NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to build Ethernet Ingress flow lookup"
                          " table-0!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize Ethernet ingress flow table of pointers */
    eth_dp_pvt_p->eth_ingress_flow_list = (adk_dp_eth_ingress_flow_t **)
                               ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_INGRESS_FLOW_LIST_OBJ,
						 ADK_ETH_INGRESS_FLOWS_MAX,
                                                 sizeof(adk_dp_eth_ingress_flow_t *),
                                                 ADK_ETH_DP_TABLE_ELEM_ALIGN);
    if (eth_dp_pvt_p->eth_ingress_flow_list == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet ingress flow list table!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->bridge_intf = (adk_dp_eth_bridge_intf_t **)
                       ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_BRIDGE_INTF_OBJ,
					 ADK_ETH_NUM_BRIDGE,
                                            sizeof(adk_dp_eth_bridge_intf_t *),
                                            ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->bridge_intf == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate Ethernet bridge_info table!\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->default_l2_domain = (uint32_t *)
                        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_DEFAULT_BRIDGE,
					    1,
                                            sizeof(uint32_t),
                                            ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->default_l2_domain == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate memory for default bridge!\n");
        goto _adk_eth_dp_table_init_exit;
    }


    eth_dp_pvt_p->autoconn_status = (uint8_t *)
                        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_AUTO_CONN_OBJ,
                        1, sizeof(uint8_t), ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->autoconn_status == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate memory for autoconnection status\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize Ethernet Type to PCP Map table */
    eth_dp_pvt_p->ethTypePcpMapTbl = adk_lpm_create(
            ADK_MODULE_MODULEID_GET(eth_mod_handle),
            ADK_ETH_TYPE_PCP_MAP_OBJ, ADK_ETH_TYPE_PCP_MAP_LOOKUP_WIDTH,
            ADK_LPM_DEFAULT_PENDING_SIZE);
    if (eth_dp_pvt_p->ethTypePcpMapTbl == (adk_lpm_handle_t) NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to build Ethernet EthType to PCP Map table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /* Initialize etherType to Queue Mapping hash table */
    eth_dp_pvt_p->eth_ethertype_queue_map_hash_table = (adk_dp_eth_ethertype_queue_map_hash_t *)
                                ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                                ADK_ETH_TYPE_QUEUE_MAP_OBJ,
                                ADK_ETH_ETHTYPE_QUEUE_MAP_MAX,
                                sizeof(adk_dp_eth_ethertype_queue_map_hash_t),
                                ADK_ETH_DP_PTR_ALIGN);
    if (eth_dp_pvt_p->eth_ethertype_queue_map_hash_table == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate EtherType to Queue Map hash table\n");
        goto _adk_eth_dp_table_init_exit;
    }

    ethtype_queue_map_hash_table_entry.ethType = 0;
    ethtype_queue_map_hash_table_entry.queueId = ADK_QOS_INVALID_QUEUE_ID;
    for (i = 0; i < ADK_ETH_ETHTYPE_QUEUE_MAP_MAX; i++)
    {
        eth_dp_pvt_p->eth_ethertype_queue_map_hash_table[i].hash_entry =
            ethtype_queue_map_hash_table_entry.hash_entry;
    }

    /* Initize Queue ID for non bridge originated netd traffic*/
    eth_dp_pvt_p->eth_non_bridge_origin_queue_id = (uint16_t *)
                        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_NON_BRIDGE_ORIGIN_QUEUE_OBJ,
                        1, sizeof(uint16_t), ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_non_bridge_origin_queue_id == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate memory for non bridge origin queue\n");
        goto _adk_eth_dp_table_init_exit;
    }

    *eth_dp_pvt_p->eth_non_bridge_origin_queue_id = ADK_QOS_INVALID_QUEUE_ID;


    eth_dp_pvt_p->eth_bridge_timestamp_quad_group = (adk_ethernet_quad_group_t *)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_BRIDGE_TIMESTAMP_QUAD_GROUP_OBJ,
                1, sizeof(adk_ethernet_quad_group_t), ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate memory for ethernet bridge timestamp quad group\n");
        goto _adk_eth_dp_table_init_exit;
    }

    eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port = (uint8_t *)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_BRIDGE_TIMESTAMP_QUAD_GROUP_PORT_OBJ,
                1, sizeof(uint8_t), ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate memory for ethernet bridge timestamp quad group port\n");
        goto _adk_eth_dp_table_init_exit;
    }

    *eth_dp_pvt_p->eth_bridge_timestamp_quad_group = ADK_ETH_BRIDGE_DEFAULT_TIMESTAMP_QUAD_GROUP;
    *eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port = ADK_ETH_BRIDGE_DEFAULT_TIMESTAMP_QUAD_PORT;

    eth_dp_pvt_p->eth_dscp_to_pcp_map = (uint8_t *)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_DSCP_TO_PCP_MAP_OBJ,
                1, ((sizeof(uint8_t)) * (ADK_ETH_MAX_DSCP + 1)) , ADK_ETH_DP_PTR_ALIGN);

    if (eth_dp_pvt_p->eth_dscp_to_pcp_map == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                "Failed to allocate memory for ethernet dscp to pcp map\n");
        goto _adk_eth_dp_table_init_exit;
    }

    /*Enable Ethernet utility counters by default*/
    eth_dp_pvt_p->isUtilityCntEnabled = true;

    /* Set the cookie to indicate data plane tables to be ready. Use memory
     * barrier to enforce this to be the last instruction generated by the
     * compiler. */
    barrier();
    eth_dp_pvt_p->cookie = ADK_ETHERNET_MODULE_COOKIE;

_adk_eth_dp_table_init_exit:

    if (ret != ADK_SUCCESS)
    {
        if (eth_dp_pvt_p->gl_stats_p)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->gl_stats_p);
        if (eth_dp_pvt_p->eth_svif_if)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_svif_if);
        if (eth_dp_pvt_p->eth_svif_if_hash_table)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_svif_if_hash_table);
        if (eth_dp_pvt_p->eth_arp_hash_table.hash_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_hash_table.hash_tbl);
        if (eth_dp_pvt_p->eth_arp_hash_table.tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_hash_table.tbl);

        if (eth_dp_pvt_p->eth_arp_buffer_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_buffer_tbl);
        if(eth_dp_pvt_p->arp_buffer_lock)
        {
            ADK_DESTROY_LOCK(eth_dp_pvt_p->arp_buffer_lock);
            ADK_FREE_LOCK(eth_dp_pvt_p->arp_buffer_lock);
        }
        if (eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl);
        if (eth_dp_pvt_p->eth_neighbor_hash_table.tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_hash_table.tbl);
        if (eth_dp_pvt_p->eth_neighbor_buffer_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_buffer_tbl);
        if(eth_dp_pvt_p->neighbor_buffer_lock)
        {
            ADK_DESTROY_LOCK(eth_dp_pvt_p->neighbor_buffer_lock);
            ADK_FREE_LOCK(eth_dp_pvt_p->neighbor_buffer_lock);
        }
        if(eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl);
        if(eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl);
        /*TODO will be called from stop function of EM DP APP*/
        /*if(eth_dp_pvt_p->eth_buffer_info_pool)
            rte_mempool_free((struct rte_mempool *)eth_dp_pvt_p->eth_buffer_info_pool);
        if(eth_dp_pvt_p->eth_nexthop_buffer_pool)
            rte_mempool_free((struct rte_mempool *)eth_dp_pvt_p->eth_nexthop_buffer_pool);*/
        if (eth_dp_pvt_p->eth_ingress_flow_list)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_ingress_flow_list);
        if (eth_dp_pvt_p->ingressFlowTbl)
            adk_lpm_free(eth_dp_pvt_p->ingressFlowTbl);
        if (eth_dp_pvt_p->bridge_intf)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->bridge_intf);
        if (eth_dp_pvt_p->default_l2_domain)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->default_l2_domain);
        if (eth_dp_pvt_p->autoconn_status)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->autoconn_status);
        if(eth_dp_pvt_p->ethTypePcpMapTbl)
            adk_lpm_free(eth_dp_pvt_p->ethTypePcpMapTbl);
        if (eth_dp_pvt_p->eth_ethertype_queue_map_hash_table)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_ethertype_queue_map_hash_table);
        if (eth_dp_pvt_p->eth_non_bridge_origin_queue_id)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_non_bridge_origin_queue_id);
        if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_bridge_timestamp_quad_group);
        if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port);
        if (eth_dp_pvt_p->eth_dscp_to_pcp_map)
            ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_dscp_to_pcp_map);
     }

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}

adk_dp_status_t
adk_eth_dp_table_shutdown (
    adk_module_handle_t eth_mod_handle,
    adk_eth_dp_pvt_data_t *eth_dp_pvt_p)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_neighbor_table_t *eth_neighbor_hash_table;
    adk_dp_eth_arp_table_t *eth_arp_hash_table;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    if (eth_dp_pvt_p->gl_stats_p)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->gl_stats_p);
    if (eth_dp_pvt_p->eth_svif_if)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_svif_if);
    if (eth_dp_pvt_p->eth_svif_if_hash_table)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_svif_if_hash_table);
    eth_arp_hash_table = &eth_dp_pvt_p->eth_arp_hash_table;
    ADK_ETH_DP_CLEANUP_OVERFLOW_TBL(eth_arp_hash_table);
    if (eth_dp_pvt_p->eth_arp_hash_table.hash_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_hash_table.hash_tbl);
    if (eth_dp_pvt_p->eth_arp_hash_table.tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_hash_table.tbl);

    if (eth_dp_pvt_p->eth_arp_buffer_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_arp_buffer_tbl);
    if(eth_dp_pvt_p->arp_buffer_lock)
    {
        ADK_DESTROY_LOCK(eth_dp_pvt_p->arp_buffer_lock);
        ADK_FREE_LOCK(eth_dp_pvt_p->arp_buffer_lock);
    }
    if (eth_dp_pvt_p->ingressFlowTbl)
        adk_lpm_free(eth_dp_pvt_p->ingressFlowTbl);
    if (eth_dp_pvt_p->eth_ingress_flow_list)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_ingress_flow_list);
  
    eth_neighbor_hash_table = &eth_dp_pvt_p->eth_neighbor_hash_table;
    ADK_ETH_DP_CLEANUP_OVERFLOW_TBL(eth_neighbor_hash_table);
    if (eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_hash_table.hash_tbl);
    if (eth_dp_pvt_p->eth_neighbor_hash_table.tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_hash_table.tbl);
    if (eth_dp_pvt_p->eth_neighbor_buffer_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_neighbor_buffer_tbl);
    if(eth_dp_pvt_p->neighbor_buffer_lock)
    {
        ADK_DESTROY_LOCK(eth_dp_pvt_p->neighbor_buffer_lock);
        ADK_FREE_LOCK(eth_dp_pvt_p->neighbor_buffer_lock);
    }
    if(eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_nexthop_v4_timer_index_tbl);
    if(eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_nexthop_v6_timer_index_tbl);
    /*TODO will be called from stop function of EM DP APP*/
    /*if(eth_dp_pvt_p->eth_buffer_info_pool)
         rte_mempool_free((struct rte_mempool *)eth_dp_pvt_p->eth_buffer_info_pool);
    if(eth_dp_pvt_p->eth_nexthop_buffer_pool)
         rte_mempool_free((struct rte_mempool *)eth_dp_pvt_p->eth_nexthop_buffer_pool);*/
    if (eth_dp_pvt_p->bridge_intf)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->bridge_intf);
    if (eth_dp_pvt_p->default_l2_domain)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->default_l2_domain);
    if (eth_dp_pvt_p->autoconn_status)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->autoconn_status);
    if(eth_dp_pvt_p->ethTypePcpMapTbl)
        adk_lpm_free(eth_dp_pvt_p->ethTypePcpMapTbl);
    if (eth_dp_pvt_p->eth_ethertype_queue_map_hash_table)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_ethertype_queue_map_hash_table);
    if (eth_dp_pvt_p->eth_non_bridge_origin_queue_id)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_non_bridge_origin_queue_id);
    if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_bridge_timestamp_quad_group);
    if (eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_bridge_timestamp_quad_group_port);
    if (eth_dp_pvt_p->eth_dscp_to_pcp_map)
        ADK_DP_SHM_FREE(eth_dp_pvt_p->eth_dscp_to_pcp_map);

    ADK_MEMSET(eth_dp_pvt_p, 0, sizeof(adk_eth_dp_pvt_data_t));

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}

adk_dp_status_t
adk_eth_dp_svif_add (
    adk_module_handle_t eth_mod_handle,
    adk_dp_eth_svif_if_hash_t *svif_if_hash_table_p,
    adk_dp_eth_svif_intf_t *dp_svif_intf_p)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    uint32_t i, hash_val, hash_idx;
    adk_dp_eth_svif_if_hash_t entry;
    adk_sys_handle_t sys_handle;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Inserting bridge 0x%x and vlan 0x%x in svif table\n",
                     dp_svif_intf_p->bgid, dp_svif_intf_p->vlan);

    hash_val = ADK_ETHERNET_HASH(dp_svif_intf_p->bgid, dp_svif_intf_p->vlan);
    hash_idx = hash_val & ADK_ETH_SVIF_INTF_MASK;

    for (i = 0; i < 8; i++)
    {
        entry.hash_entry = svif_if_hash_table_p[hash_idx].hash_entry;
        if (entry.stats_id == ETH_INVALID_NS_ID)
            goto _adk_eth_dp_svif_add_do;

        if ((hash_idx & 0x7) == 0x7)
            hash_idx = hash_idx & ~0x7;
        else
            hash_idx++;
    }

    hash_idx = (hash_idx & ~0x7) + 0x8;

    for (i = 0; i < ADK_ETH_SVIF_INTF_MAX - 8; i++, hash_idx++)
    {
        if (hash_idx == ADK_ETH_SUB_INTF_MAX)
            hash_idx = 0;

        entry.hash_entry = svif_if_hash_table_p[hash_idx].hash_entry;
        if (entry.stats_id == ETH_INVALID_NS_ID)
        {
            goto _adk_eth_dp_svif_add_do;
        }
    }

    ret = ADK_DP_E_ERROR;
    ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                      "Entry not inserted in dp table\n");
    goto _adk_eth_dp_svif_add_exit;

_adk_eth_dp_svif_add_do:

    entry.l2_domain_id = dp_svif_intf_p->bgid;
    entry.vlan = dp_svif_intf_p->vlan;
    entry.stats_id = dp_svif_intf_p->statsId;
    svif_if_hash_table_p[hash_idx].hash_entry = entry.hash_entry;

_adk_eth_dp_svif_add_exit:

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}

adk_dp_status_t
adk_eth_dp_svif_rem (
    adk_module_handle_t eth_mod_handle,
    adk_dp_eth_svif_if_hash_t *svif_if_hash_table_p,
    adk_dp_eth_svif_intf_t *dp_svif_intf_p)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    uint32_t i, hash_val, hash_idx;
    adk_dp_eth_svif_if_hash_t entry;
    adk_sys_handle_t sys_handle;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Deleting port 0x%x and vlan 0x%x ...\n",
                     dp_svif_intf_p->bgid, dp_svif_intf_p->vlan);

    hash_val = ADK_ETHERNET_HASH(dp_svif_intf_p->bgid, dp_svif_intf_p->vlan);
    hash_idx = hash_val & ADK_ETH_SVIF_INTF_MASK;

    for (i = 0; i < 8; i++)
    {
        entry.hash_entry = svif_if_hash_table_p[hash_idx].hash_entry;
        if ((entry.l2_domain_id == dp_svif_intf_p->bgid) &&
            (entry.vlan == dp_svif_intf_p->vlan) &&
            (entry.stats_id == dp_svif_intf_p->statsId))
        {
            goto _adk_eth_dp_svif_rem_do;
        }
        
        if ((hash_idx & 0x7) == 0x7)
            hash_idx = hash_idx & ~0x7;
        else
            hash_idx++;
    }

    hash_idx = (hash_idx & ~0x7) + 0x8;

    for (i = 0; i < ADK_ETH_SVIF_INTF_MAX - 8; i++, hash_idx++)
    {
        if (hash_idx == ADK_ETH_SVIF_INTF_MAX)
            hash_idx = 0;

        entry.hash_entry = svif_if_hash_table_p[hash_idx].hash_entry;
        if ((entry.l2_domain_id == dp_svif_intf_p->bgid) &&
            (entry.vlan == dp_svif_intf_p->vlan) &&
            (entry.stats_id == dp_svif_intf_p->statsId))
        {
            goto _adk_eth_dp_svif_rem_do;
        }
    }

    ret = ADK_DP_E_ERROR;
    ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                      "Entry not removed from Bridge table\n");
    goto _adk_eth_dp_svif_rem_exit;

_adk_eth_dp_svif_rem_do:

    entry.l2_domain_id = 0;
    entry.vlan = 0;
    entry.stats_id = ETH_INVALID_NS_ID;
    svif_if_hash_table_p[hash_idx].hash_entry = entry.hash_entry;

_adk_eth_dp_svif_rem_exit:

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}

adk_status_t
adk_eth_dp_ingress_flow (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_eth_flow_internal_data_t *eth_flow_data_p)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_sys_handle_t sys_handle;
    adk_handle_t value;
    adk_ethernet_ingress_flow_data_t *flow_data_p;
    adk_dp_eth_ingress_flow_key_t ingFlowKey;
    uint16_t size = 0;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);

    flow_data_p = &(eth_flow_data_p->u.ingress_flow);

    /* Build the input key */

    ADK_MEMSET(&ingFlowKey, 0x0, sizeof(adk_dp_eth_ingress_flow_key_t));

    if(flow_data_p->mask & ADK_ETHERNET_FLOW_MASK_ETYPE)
    {
        ingFlowKey.ether_type = flow_data_p->proto;
        size += ETH_TYPE_WIDTH;
    }

    if (flow_data_p->mask & ADK_ETHERNET_FLOW_MASK_DMAC)
    {
        ADK_MEMCPY(ingFlowKey.dmac, flow_data_p->dest_mac.mac, ADK_ETH_MAC_ADDR_SIZE);
        size += flow_data_p->dest_mac.num_valid_msb_bits;
    }

    if (oper == ADK_ETH_TREE_OPER_ADD)
    {
        value = ((adk_handle_t)(uintptr_t) eth_flow_data_p->dp_u.dp_ing_flow_p);
        ret = adk_lpm_add(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_INGRESS_FLOW_OBJ, eth_internal_data_p->dp_pvt_p->ingressFlowTbl,
                          (unsigned char *) &ingFlowKey,
                          size, value);
        if (ret != ADK_SUCCESS)
        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to add entry in Ingress Flow Table\n");
            goto _adk_eth_dp_ingress_flow_exit;
        }
    }

    if (oper == ADK_ETH_TREE_OPER_DEL)
    {
        if (adk_lpm_is_rule_present(eth_internal_data_p->
                                    dp_pvt_p->ingressFlowTbl,
                                    (unsigned char *) &ingFlowKey, size,
                                    &value))
        {
            ret = adk_lpm_delete(eth_internal_data_p->dp_pvt_p->ingressFlowTbl,
                                 (unsigned char *)
                                 &ingFlowKey, size);
            if (ret != ADK_SUCCESS)
            {
                ret = ADK_E_INTERNAL_ERROR;
                ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                  "Failed to delete entry from Ingress Flow"
                                  " Table\n");
                goto _adk_eth_dp_ingress_flow_exit;
            }
        }
        else
        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to find entry to remove from Ingress Flow"
                              " Table\n");
            goto _adk_eth_dp_ingress_flow_exit;
        }
    }

_adk_eth_dp_ingress_flow_exit:

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}

adk_status_t
adk_eth_cpu_flow (
    adk_module_handle_t __attribute__ ((unused)) eth_mod_handle,
    adk_eth_tree_oper_t __attribute__ ((unused)) oper,
    adk_eth_flow_internal_data_t __attribute__ ((unused)) *eth_flow_data_p)
{
    return ADK_SUCCESS;
}

adk_status_t
adk_eth_dp_arp_entry_add(
        adk_dp_eth_arp_table_t *eth_arp_table,
        uint32_t ip_addr,
        uint16_t index)
{
    adk_dp_eth_neighbor_hash_entry_t  *arp_hash_entry = NULL;
    uint32_t   key = 0;
    uint16_t hash_key, i;

    key = ADK_ETHERNET_HASH(key, ip_addr);

    hash_key = key & ADK_ETH_NEIGHBOR_HASH_MASK;

    /* Lookup the Hash entry*/
    hash_key = hash_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    arp_hash_entry = &eth_arp_table->hash_tbl[hash_key];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, arp_hash_entry++)
    {   
        /* search for free entry */
        if(!arp_hash_entry->is_enabled)
        {
            arp_hash_entry->index       = index;
            arp_hash_entry->hash_value  = hash_key;
            arp_hash_entry->is_enabled  = TRUE;

            return(ADK_SUCCESS);
        }
    }

    ADK_ETH_DP_ADD_OVERFLOW_ENTRY(eth_arp_table->overflow_list,
            eth_arp_table->free_list, eth_arp_table->tbl, index);

    return ADK_SUCCESS;
}

adk_status_t
adk_eth_dp_arp_entry_del( 
        adk_dp_eth_arp_table_t *eth_arp_table,
        uint32_t ip_addr)
{
    adk_dp_eth_neighbor_hash_entry_t  *arp_hash_entry = NULL;
    adk_dp_eth_arp_entry_t *arp_entry = NULL;
    uint32_t   key = 0;
    uint16_t hash_key, i;
    uint16_t index;

    key = ADK_ETHERNET_HASH(key, ip_addr);

    hash_key = key & ADK_ETH_NEIGHBOR_HASH_MASK;

    /* Lookup the Hash entry*/
    hash_key = hash_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    arp_hash_entry = &eth_arp_table->hash_tbl[hash_key];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, arp_hash_entry++)
    {
        if(arp_hash_entry->hash_value == hash_key)
        { 
            arp_entry = &eth_arp_table->tbl[arp_hash_entry->index];

            if(ip_addr == arp_entry->ipv4_addr)
            {
                index =  arp_hash_entry->index;
                arp_entry->is_enabled = FALSE;
                /* reset the hash entry */
                memset(arp_hash_entry,0,sizeof(adk_dp_eth_neighbor_hash_entry_t));
                /* move the neighbor table entry to pending list */
                ADK_ETH_DP_RELEASE_DISTRIBUTED_IDX(eth_arp_table->pending_list,
                        eth_arp_table->free_list,
                        eth_arp_table->tbl,
                        index);

                return ADK_SUCCESS;
            }
        }
    }
     /* check in the overflow list */
    adk_dp_eth_overflow_node_t *overflow_node;
    overflow_node = eth_arp_table->overflow_list.head;
    while(overflow_node)
    {   
        /*compare destination address */
        arp_entry = &eth_arp_table->tbl[overflow_node->idx];
        
        if(ip_addr == arp_entry->ipv4_addr)
        {      
            arp_entry->is_enabled = FALSE;
            /* check in overflow list */
            ADK_ETH_DP_RELEASE_OVERFLOW_ENTRY(eth_arp_table->overflow_list,
                    eth_arp_table->overflow_pending_list, eth_arp_table->pending_list,
                    eth_arp_table->free_list, eth_arp_table->tbl, overflow_node->idx);

            return(ADK_SUCCESS);
        }
        overflow_node = overflow_node->next;
    }

    return ADK_E_INTERNAL_ERROR;
}

adk_status_t
adk_eth_dp_arp (
        adk_module_handle_t eth_mod_handle,
        adk_eth_tree_oper_t oper,
        adk_ethernet_ipv4_addr_t ip_addr,
        adk_ethernet_mac_addr_t dest_mac,
        adk_ethernet_arp_status_t arp_status)
{

    adk_status_t ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_arp_entry_t *arp_entry = NULL;
    adk_dp_eth_arp_table_t *eth_arp_table;
    uint16_t index;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);

    eth_arp_table = &eth_internal_data_p->dp_pvt_p->eth_arp_hash_table;

    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK(eth_arp_table);
    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(eth_arp_table);


    if (oper == ADK_ETH_TREE_OPER_ADD)
    {
        if((ADK_ETHERNET_NEIGHBOR_REACHABLE ==arp_status ||
                    ADK_ETHERNET_NEIGHBOR_STALE == arp_status) &&
                (NULL == dest_mac))
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ADK_E_INVALID_PARAMETER,
                    "Failed to add/update arp entry since MAC address is NULL\n");
            return ADK_E_INVALID_PARAMETER;
        }

        /*Check if neighbor entry already exist else add it new*/
        if(ADK_SUCCESS !=  adk_eth_dp_arp_entry_lookup(eth_arp_table,ip_addr,&arp_entry))
        {
            /* reserve neighbor index */
            ADK_ETH_DP_RSRV_IDX(eth_arp_table->free_list, eth_arp_table->tbl, index);

            eth_arp_table->tbl[index].ipv4_addr = ip_addr;

            if(ADK_ETHERNET_NEIGHBOR_REACHABLE == arp_status ||
                    ADK_ETHERNET_NEIGHBOR_STALE == arp_status)
            {
                memcpy(eth_arp_table->tbl[index].dest_mac, dest_mac, sizeof(adk_ethernet_mac_addr_t));
            }
            eth_arp_table->tbl[index].arp_status = arp_status;
            eth_arp_table->tbl[index].is_enabled = TRUE;

            eth_arp_table->tbl[index].have_inflight = true;
            eth_arp_table->tbl[index].stale_exception_raised = EXCEPTION_NOT_RAISED;

            if(ADK_SUCCESS != adk_eth_dp_arp_entry_add(eth_arp_table,ip_addr,index))
            {         
                return ADK_E_INTERNAL_ERROR;  
            }
        }
        else
        {
            /*If entry already exists then overwrite*/
            if(ADK_ETHERNET_NEIGHBOR_REACHABLE == arp_status ||
                    ADK_ETHERNET_NEIGHBOR_STALE == arp_status)
            {
                memcpy(arp_entry->dest_mac, dest_mac, sizeof(adk_ethernet_mac_addr_t));
            }
            arp_entry->arp_status = arp_status;
            arp_entry->stale_exception_raised = EXCEPTION_NOT_RAISED;
        }

    }
    if (oper == ADK_ETH_TREE_OPER_DEL)
    {
        if(ADK_SUCCESS != adk_eth_dp_arp_entry_del(eth_arp_table,ip_addr))
        {
            return ADK_E_INTERNAL_ERROR;
        }
    }


    eth_internal_data_p->dp_pvt_p->eth_fc_ver++;

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}


adk_status_t
adk_eth_dp_neighbor_key_generate(uint32_t *ipv6_addr,
        uint32_t *key,
        uint16_t *key1,
        uint16_t *key2)
{
    uint32_t    init_value = 0;
    init_value = _mm_crc32_u32(init_value, ipv6_addr[0]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[1]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[2]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[3]);
    *key = init_value;
    *key1 = init_value & ADK_ETH_NEIGHBOR_HASH_MASK;
    *key2 = (init_value>>16) & ADK_ETH_NEIGHBOR_HASH_MASK;

    return(ADK_SUCCESS);
}

adk_status_t
adk_eth_dp_neighbor_hash_entry_add(adk_dp_eth_neighbor_table_t *eth_neighbor_table, uint8_t *neighIpv6Addr,
                                        uint16_t index)
{
    uint32_t    key;
    uint16_t first_key, second_key, i;
    adk_dp_eth_neighbor_hash_entry_t  *neigh_hash_entry = NULL;

    /* Generate key for neighbor table lookup */
    adk_eth_dp_neighbor_key_generate((uint32_t*)neighIpv6Addr,
            &key,
            &first_key,
            &second_key);

    /* update the Hash entry, OR handle collision entry*/
    first_key = first_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    neigh_hash_entry = &eth_neighbor_table->hash_tbl[(first_key)];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, neigh_hash_entry++)
    {
        /* search for free entry */
        if(!neigh_hash_entry->is_enabled)
        {
            neigh_hash_entry->index       = index;
            neigh_hash_entry->hash_value  = key;
            neigh_hash_entry->is_enabled  = TRUE;

            return(ADK_SUCCESS);
        }
    }

    /* update the Hash entry, OR handle collision entry*/
    second_key = second_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    neigh_hash_entry = &eth_neighbor_table->hash_tbl[(second_key)];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, neigh_hash_entry++)
    {
        /* search for free entry */
        if(!neigh_hash_entry->is_enabled)
        {
            neigh_hash_entry->index       = index;
            neigh_hash_entry->hash_value  = key;
            neigh_hash_entry->is_enabled  = TRUE;

            return(ADK_SUCCESS);
        }
    }

    return ADK_E_INTERNAL_ERROR;
}

adk_status_t
adk_eth_dp_neighbor_hash_entry_lookup(adk_dp_eth_neighbor_table_t *eth_neighbor_table, uint8_t *neighIpv6Addr,
        adk_dp_eth_neighbor_hash_entry_t **neigh_looked_up_entry)
{

    uint32_t    key;
    uint16_t first_key, second_key, i;
    adk_dp_eth_neighbor_hash_entry_t  *neigh_hash_entry = NULL;
    adk_dp_eth_neighbor_entry_t *neigh_entry;

    /* Generate key for neighbor table lookup */
    adk_eth_dp_neighbor_key_generate((uint32_t *)neighIpv6Addr,
            &key,
            &first_key,
            &second_key);

    /* Get the Hash entry*/
    first_key = first_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    neigh_hash_entry = &eth_neighbor_table->hash_tbl[(first_key)];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, neigh_hash_entry++)
    {
        if((neigh_hash_entry->hash_value == key))
        {
            neigh_entry = &eth_neighbor_table->tbl[neigh_hash_entry->index];

            if((*((uint64_t *)&neighIpv6Addr[0]) ==
                        *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[0]))) &&
                    (*((uint64_t *)&neighIpv6Addr[8]) ==
                     *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[2]))))
            {
                *neigh_looked_up_entry = neigh_hash_entry;
                return(ADK_SUCCESS);
            }
        }
    }


    second_key = second_key*ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY;
    neigh_hash_entry = &eth_neighbor_table->hash_tbl[(second_key)];

    for(i=0; i<ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY; i++, neigh_hash_entry++)
    {
        if((neigh_hash_entry->hash_value == key))
        {
            neigh_entry = &eth_neighbor_table->tbl[neigh_hash_entry->index];

            if((*((uint64_t *)&neighIpv6Addr[0]) ==
                        *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[0]))) &&
                    (*((uint64_t *)&neighIpv6Addr[8]) ==
                     *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[2]))))
            {
                *neigh_looked_up_entry = neigh_hash_entry;
                return(ADK_SUCCESS);
            }
        }
    }

    return ADK_E_INTERNAL_ERROR;
}

adk_status_t
adk_eth_dp_neighbor_hash_overflow_lookup(adk_dp_eth_neighbor_table_t *eth_neighbor_table, uint8_t *neighIpv6Addr,
                            adk_dp_eth_overflow_node_t **overflow_looked_up_node)
{
    /* check in the overflow list */
    adk_dp_eth_overflow_node_t *node;
    adk_dp_eth_neighbor_entry_t *neigh_entry;
    node=eth_neighbor_table->overflow_list.head;

    while(node)
    {
        /*compare destination IPv6 address */
        neigh_entry = &eth_neighbor_table->tbl[node->idx];

        if((*((uint64_t *)&neighIpv6Addr[0]) ==
                    *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[0]))) &&
                (*((uint64_t *)&neighIpv6Addr[8]) ==
                 *((uint64_t *)&((neigh_entry)->ipv6_addr.addr[2]))))
        {
            *overflow_looked_up_node = node;
            return(ADK_SUCCESS);
        }
        node = node->next;
    }

    return ADK_E_INTERNAL_ERROR;
}

adk_status_t
adk_eth_dp_neigh (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_ipv6_addr_t *ipv6_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_neigh_status_t neigh_status)
{
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_neighbor_hash_entry_t *neigh_hash_entry = NULL;
    adk_dp_eth_neighbor_entry_t *neigh_entry = NULL;
    adk_dp_eth_neighbor_table_t *eth_neighbor_table;
    adk_dp_eth_overflow_node_t *overflow_node;
    uint16_t index;
    
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);

    eth_neighbor_table = &eth_internal_data_p->dp_pvt_p->eth_neighbor_hash_table;

    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK(eth_neighbor_table);
    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(eth_neighbor_table);

    ipv6_addr->addr[0] = adk_htonl(ipv6_addr->addr[0]);
    ipv6_addr->addr[1] = adk_htonl(ipv6_addr->addr[1]);
    ipv6_addr->addr[2] = adk_htonl(ipv6_addr->addr[2]);
    ipv6_addr->addr[3] = adk_htonl(ipv6_addr->addr[3]);

    if (oper == ADK_ETH_TREE_OPER_ADD)
    {
        if((ADK_ETHERNET_NEIGHBOR_REACHABLE == neigh_status || 
            ADK_ETHERNET_NEIGHBOR_STALE == neigh_status) && 
            (NULL == dest_mac))
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ADK_E_INVALID_PARAMETER,
                    "Failed to add/update neighbor entry since MAC address is NULL\n");
            return ADK_E_INVALID_PARAMETER;
        }

        /*Check if neighbor entry already exist else add it new*/
        if(ADK_SUCCESS !=  adk_eth_dp_neighbor_entry_lookup(eth_neighbor_table,(uint8_t*)&ipv6_addr->addr[0],&neigh_entry))
        {
            /* reserve neighbor index */
            ADK_ETH_DP_RSRV_IDX(eth_neighbor_table->free_list, eth_neighbor_table->tbl, index);

            eth_neighbor_table->tbl[index].ipv6_addr.addr[0] = ipv6_addr->addr[0];
            eth_neighbor_table->tbl[index].ipv6_addr.addr[1] = ipv6_addr->addr[1];
            eth_neighbor_table->tbl[index].ipv6_addr.addr[2] = ipv6_addr->addr[2];
            eth_neighbor_table->tbl[index].ipv6_addr.addr[3] = ipv6_addr->addr[3];

            if(ADK_ETHERNET_NEIGHBOR_REACHABLE == neigh_status ||
               ADK_ETHERNET_NEIGHBOR_STALE == neigh_status)
            {
                memcpy(eth_neighbor_table->tbl[index].dest_mac, dest_mac, sizeof(adk_ethernet_mac_addr_t));
            }
            eth_neighbor_table->tbl[index].neigh_status = neigh_status;
            eth_neighbor_table->tbl[index].is_enabled = TRUE;

            eth_neighbor_table->tbl[index].have_inflight = true;
            eth_neighbor_table->tbl[index].stale_exception_raised = EXCEPTION_NOT_RAISED;

            if(ADK_SUCCESS != adk_eth_dp_neighbor_hash_entry_add(eth_neighbor_table,
                        (uint8_t*)&eth_neighbor_table->tbl[index].ipv6_addr.addr[0],index))
            {
                ADK_ETH_DP_ADD_OVERFLOW_ENTRY(eth_neighbor_table->overflow_list,
                        eth_neighbor_table->free_list, eth_neighbor_table->tbl, index);
            }
        }
        else
        {
            /*If entry already exists then overwrite*/
            if(ADK_ETHERNET_NEIGHBOR_REACHABLE == neigh_status ||
                    ADK_ETHERNET_NEIGHBOR_STALE == neigh_status)
            {
                memcpy(neigh_entry->dest_mac, dest_mac, sizeof(adk_ethernet_mac_addr_t));
            }
            neigh_entry->neigh_status = neigh_status;
            neigh_entry->stale_exception_raised = EXCEPTION_NOT_RAISED;
        }
    }
    if (oper == ADK_ETH_TREE_OPER_DEL)
    {
        if(ADK_SUCCESS == adk_eth_dp_neighbor_hash_entry_lookup(eth_neighbor_table,(uint8_t*)&ipv6_addr->addr[0],&neigh_hash_entry))
        {
            index =  neigh_hash_entry->index;

            eth_neighbor_table->tbl[index].is_enabled = FALSE;
            /* reset the hash entry */
            memset(neigh_hash_entry,0,sizeof(adk_dp_eth_neighbor_hash_entry_t));
            /* move the neighbor table entry to pending list */
            ADK_ETH_DP_RELEASE_DISTRIBUTED_IDX(eth_neighbor_table->pending_list,
                    eth_neighbor_table->free_list,
                    eth_neighbor_table->tbl,
                    index);
        }
        else 
        {
            if(ADK_SUCCESS == adk_eth_dp_neighbor_hash_overflow_lookup(eth_neighbor_table, (uint8_t*)&ipv6_addr->addr[0],&overflow_node))
            {
                index =  overflow_node->idx;

                eth_neighbor_table->tbl[index].is_enabled = FALSE;
                /* check in overflow list */
                ADK_ETH_DP_RELEASE_OVERFLOW_ENTRY(eth_neighbor_table->overflow_list,
                        eth_neighbor_table->overflow_pending_list, eth_neighbor_table->pending_list,
                        eth_neighbor_table->free_list, eth_neighbor_table->tbl, overflow_node->idx);
            }
            else
            {
                ADK_ETH_LOG_ERROR(eth_mod_handle, ADK_E_INVALID_PARAMETER,
                        "Failed to find the neighbor entry \n");
                return ADK_E_INVALID_PARAMETER;
            }
        }
    }

    eth_internal_data_p->dp_pvt_p->eth_fc_ver++;
    return(ADK_SUCCESS);
}

adk_status_t
adk_eth_dp_neigh_get (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_ipv6_addr_t *ipv6_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_neigh_status_t *neigh_status)
{
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_neighbor_entry_t *neigh_entry = NULL;
    adk_dp_eth_neighbor_table_t *eth_neighbor_table;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);

    eth_neighbor_table = &eth_internal_data_p->dp_pvt_p->eth_neighbor_hash_table;

    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK(eth_neighbor_table);
    ADK_ETH_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(eth_neighbor_table);

    ipv6_addr->addr[0] = adk_htonl(ipv6_addr->addr[0]);
    ipv6_addr->addr[1] = adk_htonl(ipv6_addr->addr[1]);
    ipv6_addr->addr[2] = adk_htonl(ipv6_addr->addr[2]);
    ipv6_addr->addr[3] = adk_htonl(ipv6_addr->addr[3]);

    if(ADK_SUCCESS !=  adk_eth_dp_neighbor_entry_lookup(eth_neighbor_table,(uint8_t*)&ipv6_addr->addr[0],&neigh_entry))
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ADK_E_INVALID_PARAMETER,
                "Failed to find the neighbor entry \n");

        return ADK_E_INVALID_PARAMETER;
    }
    else 
    {
        memcpy(dest_mac, neigh_entry->dest_mac, sizeof(adk_ethernet_mac_addr_t));
        *neigh_status = neigh_entry->neigh_status;
    }
    return(ADK_SUCCESS);
}

adk_status_t adk_eth_map_proto_to_pcp(
        adk_module_handle_t eth_mod_handle,
        adk_eth_tree_oper_t oper,
        adk_ethernet_proto_t eth_type,
        adk_ethernet_priority_t priority)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_sys_handle_t sys_handle;
    uint32_t size = ADK_ETH_TYPE_PCP_MAP_LOOKUP_WIDTH;
    adk_ethernet_priority_t *pcp_val;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);

    if (oper == ADK_ETH_TREE_OPER_ADD)
    {
        if (adk_lpm_is_rule_present(eth_internal_data_p->dp_pvt_p->ethTypePcpMapTbl,
                    (unsigned char *) &eth_type, size,
                    (adk_handle_t*)&pcp_val))
        {
            *pcp_val = priority;
            goto _adk_eth_map_proto_to_pcp_exit;
        }
        /* Allocate a DP table element */
        pcp_val = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle),
                     ADK_ETH_TYPE_PCP_MAP_OBJ,
                     1, sizeof(adk_ethernet_priority_t),
                        ADK_ETH_DP_OBJECT_ALIGN);
        if (pcp_val == NULL)
        {
            ret = ADK_E_MEM_UNAVAILABLE;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to allocate DP memory for PCP value %d\n",
                              priority);
            goto _adk_eth_map_proto_to_pcp_exit;
        }

        *pcp_val = priority;
        ret = adk_lpm_add(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_TYPE_PCP_MAP_OBJ,
                eth_internal_data_p->dp_pvt_p->ethTypePcpMapTbl, (unsigned char *) &eth_type,
                size, ((adk_handle_t)(uintptr_t)pcp_val));
        if (ret != ADK_SUCCESS)
        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to add entry in EthType to PCP Map Table\n");
            goto _adk_eth_map_proto_to_pcp_exit;
        }
    }
    else if (oper == ADK_ETH_TREE_OPER_DEL)
    {  
        if (adk_lpm_is_rule_present(eth_internal_data_p->dp_pvt_p->ethTypePcpMapTbl,
                    (unsigned char *) &eth_type, size,
                    (adk_handle_t*)&pcp_val))
        {   
            ret = adk_lpm_delete(eth_internal_data_p->dp_pvt_p->ethTypePcpMapTbl,
                                 (unsigned char *)&eth_type, size);
                       if (ret != ADK_SUCCESS)
            {
                ret = ADK_E_INTERNAL_ERROR;
                ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                  "Failed to delete entry from ethType PCP Map table\n");
                goto _adk_eth_map_proto_to_pcp_exit;
            }
            adk_eth_list_add(eth_internal_data_p, ADK_ETH_TYPE_PCP_MAP_LIST_ID,
                                (uintptr_t) pcp_val);
            goto _adk_eth_map_proto_to_pcp_exit;
        }
        else
        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete entry from ethType PCP Map table\n");
        }
    }
    else
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to delete entry from ethType PCP Map table\n");
        ret = ADK_E_INTERNAL_ERROR;
    }
_adk_eth_map_proto_to_pcp_exit:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}

static bool eth_ethertype_to_queue_hash_entry_add(
        adk_dp_eth_ethertype_queue_map_hash_t  *ethertype_queue_map_hash_table,
        uint32_t hash_idx,
        adk_dp_eth_ethertype_queue_map_hash_t  *hash_entry)

{
    adk_dp_eth_ethertype_queue_map_hash_t  entry;
    uint32_t i;
    for (i = 0; i < 8; i++)
    {
        entry.hash_entry = ethertype_queue_map_hash_table[hash_idx].hash_entry;
        if (entry.ethType == hash_entry->ethType)
        {
            ethertype_queue_map_hash_table[hash_idx].queueId = hash_entry->queueId;
            return true;
        }
        if (entry.queueId == ADK_QOS_INVALID_QUEUE_ID)
        {
            memcpy(&ethertype_queue_map_hash_table[hash_idx],
                    hash_entry,sizeof(adk_dp_eth_ethertype_queue_map_hash_t));
            return true;
        }
        if ((hash_idx & 0x7) == 0x7)
            hash_idx = hash_idx & ~0x7;
        else
            hash_idx++;
    }

    hash_idx = (hash_idx & ~0x7) + 0x8;

    for (i = 0; i < ADK_ETH_ETHTYPE_QUEUE_MAP_MAX - 8; i++, hash_idx++)
    {
        if (hash_idx == ADK_ETH_ETHTYPE_QUEUE_MAP_MAX)
            hash_idx = 0;

        entry.hash_entry = ethertype_queue_map_hash_table[hash_idx].hash_entry;
        if (entry.ethType == hash_entry->ethType)
        {
            ethertype_queue_map_hash_table[hash_idx].queueId = hash_entry->queueId;
            return true;
        }

        if (entry.queueId == ADK_QOS_INVALID_QUEUE_ID)
        {
            memcpy(&ethertype_queue_map_hash_table[hash_idx],
                    hash_entry,sizeof(adk_dp_eth_ethertype_queue_map_hash_t));
            return true;
        }
    }
    return false;
}

static bool eth_ethertype_to_queue_hash_entry_rem(
        adk_dp_eth_ethertype_queue_map_hash_t  *ethertype_queue_map_hash_table,
        uint32_t hash_idx,
        adk_dp_eth_ethertype_queue_map_hash_t  *hash_entry)

{
    adk_dp_eth_ethertype_queue_map_hash_t  *entry;
    uint32_t i;
    for (i = 0; i < 8; i++)
    {
        entry = &ethertype_queue_map_hash_table[hash_idx];
        if (entry->ethType == hash_entry->ethType) 
        {
            entry->ethType = 0;
            entry->queueId = ADK_QOS_INVALID_QUEUE_ID;
            return true;
        }
        if ((hash_idx & 0x7) == 0x7)
            hash_idx = hash_idx & ~0x7;
        else
            hash_idx++;
    }

    hash_idx = (hash_idx & ~0x7) + 0x8;

    for (i = 0; i < ADK_ETH_ETHTYPE_QUEUE_MAP_MAX - 8; i++, hash_idx++)
    {
        if (hash_idx == ADK_ETH_ETHTYPE_QUEUE_MAP_MAX)
            hash_idx = 0;

        entry = &ethertype_queue_map_hash_table[hash_idx];
        if (entry->ethType == hash_entry->ethType)
        {
            entry->ethType = 0;
            entry->queueId = ADK_QOS_INVALID_QUEUE_ID;
            return true;
        }
    }

    return false;
}

adk_status_t adk_eth_map_ethertype_to_queue(
        adk_module_handle_t eth_mod_handle,
        adk_interface_handle_t ifHandle,
        adk_eth_tree_oper_t oper,
        adk_ethernet_proto_t ethType,
        adk_qos_queue_id_t  queueId)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t *eth_internal_data_p;
    adk_eth_intf_internal_data_t    *if_data_p = NULL;
    adk_dp_eth_ethertype_queue_map_hash_t hash_entry; 
    uint32_t hash_val, hash_idx;
    adk_sys_handle_t sys_handle;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(eth_mod_handle);
    if_data_p = (adk_eth_intf_internal_data_t *)
    ADK_SYS_INTERFACE_ATTR_PTR_GET(ifHandle);

    hash_val = ADK_ETHERNET_HASH(if_data_p->if_id, ethType);
    hash_idx = hash_val & ADK_ETH_ETHTYPE_QUEUE_MAP_MASK;

    hash_entry.ethType = ethType;
    hash_entry.queueId = queueId;

    if (oper == ADK_ETH_TREE_OPER_ADD)
    {
        if(!eth_ethertype_to_queue_hash_entry_add(eth_internal_data_p->dp_pvt_p->eth_ethertype_queue_map_hash_table,
                    hash_idx, &hash_entry))

        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Entry not inserted in dp table\n");
            goto _adk_eth_map_ethertype_to_queue_exit;
        }
    }
    else if(oper == ADK_ETH_TREE_OPER_DEL)
    {
        if(!eth_ethertype_to_queue_hash_entry_rem(eth_internal_data_p->dp_pvt_p->eth_ethertype_queue_map_hash_table,
                                              hash_idx, &hash_entry))
        {
            ret = ADK_E_INTERNAL_ERROR;
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Entry not remove in dp table\n");
            goto _adk_eth_map_ethertype_to_queue_exit;

        }
    }
    else
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Invalid operation type\n");
        ret = ADK_E_INTERNAL_ERROR;
    }
_adk_eth_map_ethertype_to_queue_exit:

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}
    /*! end addtogroup _ethernetFunctsons  Ethernet Module
 * @}
 */
