/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet_intf.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernet  Ethernet Module
 * @{
 *      @addtogroup _ethernetFunctions  Ethernet Functions
 *      @{
 *
 * @brief   This files has the source code for the top level APIs that manage
 *          Ethernet tagged and untagged interfaces. This essentially includes
 *          physical and VLAN sub-interfaces.
 *
 *  **********************************************************************/

#include <adk_eth.h>
#include <adk_ethernet_export.h>

#ifdef ADK_HOS_OLD_IMPL
adk_status_t
adk_eth_intf_acl_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    bool is_ipv6,
    bool disable_ip,
    bool enable_acl,
    uint32_t rule_set_id)
{
    adk_status_t ret = ADK_SUCCESS;

#ifdef ADK_DEVICE_SNR
    adk_sys_handle_t sys_handle;
    adk_eth_intf_internal_data_t *sub_if_data_p = NULL;
    adk_eth_intf_internal_data_t *if_data_p = NULL;
    adk_eth_intf_internal_data_t *if_bridge_data_p = NULL;
    uint32_t stats_id, if_num;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to configure ACL ID on Ethernet intf\n");
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    if (!adk_eth_valid_if_handle(if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle %"ADKPRIh "\n",
                          if_handle);
        goto _adk_eth_intf_acl_config_exit;
    }

    sub_if_data_p = (adk_eth_intf_internal_data_t *)
        ADK_SYS_INTERFACE_ATTR_PTR_GET(if_handle);
    
    if(sub_if_data_p->type == INTERFACE_TYPE_VLAN)
    {
        if_data_p = (adk_eth_intf_internal_data_t *)
            ADK_SYS_INTERFACE_ATTR_PTR_GET(sub_if_data_p->u.vlan_if.if_handle);

        if_num = ADK_ETH_INTERFACE_ID_GET(sub_if_data_p->u.vlan_if.if_handle);

        stats_id = ADK_SYS_INTERFACE_ID_GET(if_handle);

        if (if_data_p->u.phy_if.is_nac)
        {
            uint32_t tbl_idx;
            adk_hos_rule_id_t rule_id;
            adk_hos_outer_ip_tbl_rule_attr_t hos_outer_ip_rule;

            /* Fill in the new data */
            ADK_MEMSET(&hos_outer_ip_rule, 0x0, sizeof(hos_outer_ip_rule));
            hos_outer_ip_rule.outer_ip_id = stats_id;
            hos_outer_ip_rule.is_ipv6 = is_ipv6;
            hos_outer_ip_rule.ip_traffic_disabled = disable_ip;
            hos_outer_ip_rule.acl_enabled = enable_acl;
            hos_outer_ip_rule.ip_acl_id = rule_set_id;

            if (!is_ipv6)
            {
                tbl_idx = ADK_HOS_OUTER_IP4_TBL;
                rule_id = sub_if_data_p->ip4_outer_id_rule_id[0];
            }
            else
            {
                tbl_idx = ADK_HOS_OUTER_IP6_TBL;
                rule_id = sub_if_data_p->ip6_outer_id_rule_id[0];
            }
            ret = adk_hos_rule_update(tbl_idx, rule_id,
                                      (adk_hos_rule_attr_t *) &hos_outer_ip_rule);
            if (ret != ADK_SUCCESS)
            {
                ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                  "Failed to add outer IP HOS rule\n");
                goto _adk_eth_intf_acl_config_exit;
            }
        }
    }
    else if(sub_if_data_p->type == INTERFACE_TYPE_SWITCH_VIRTUAL)
    {
        adk_ethernet_internal_data_t *eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);
        if_bridge_data_p = (adk_eth_intf_internal_data_t *)
                    ADK_SYS_INTERFACE_ATTR_PTR_GET(sub_if_data_p->u.switch_virtual_if.svif_data.bridge_if_handle);

        uint16_t bg_id = if_bridge_data_p->u.bridge_if.l2_domain_id;
        adk_eth_vlan_info_t *vlan_info = eth_internal_data_p->vlan_intf_list[bg_id];
        stats_id = ADK_SYS_INTERFACE_ID_GET(if_handle);

        while(vlan_info) {
            if_data_p = (adk_eth_intf_internal_data_t *)
                ADK_SYS_INTERFACE_ATTR_PTR_GET(vlan_info->if_handle);

            /* TODO: To be extended for LAG */
            if (if_data_p->u.phy_if.is_nac)
            {
                uint32_t tbl_idx;
                adk_hos_rule_id_t rule_id;
                adk_hos_outer_ip_tbl_rule_attr_t hos_outer_ip_rule;

                adk_eth_intf_internal_data_t *vlan_sub_if_data_p = (adk_eth_intf_internal_data_t *)
                    ADK_SYS_INTERFACE_ATTR_PTR_GET(vlan_info->vlan_if_handle);

                /* Fill in the new data */
                ADK_MEMSET(&hos_outer_ip_rule, 0x0, sizeof(hos_outer_ip_rule));
                hos_outer_ip_rule.outer_ip_id = stats_id;
                hos_outer_ip_rule.is_ipv6 = is_ipv6;
                hos_outer_ip_rule.ip_traffic_disabled = disable_ip;
                hos_outer_ip_rule.acl_enabled = enable_acl;
                hos_outer_ip_rule.ip_acl_id = rule_set_id;

                if (!is_ipv6)
                {
                    tbl_idx = ADK_HOS_OUTER_IP4_TBL;
                    rule_id = vlan_sub_if_data_p->ip4_outer_id_rule_id[0];
                }
                else
                {
                    tbl_idx = ADK_HOS_OUTER_IP6_TBL;
                    rule_id = vlan_sub_if_data_p->ip6_outer_id_rule_id[0];
                }
                ret = adk_hos_rule_update(tbl_idx, rule_id,
                                          (adk_hos_rule_attr_t *) &hos_outer_ip_rule);
                if (ret != ADK_SUCCESS)
                {
                    ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                      "Failed to add outer IP HOS rule\n");
                    goto _adk_eth_intf_acl_config_exit;
                }
            }
            vlan_info = vlan_info->next;
        }
    }
    else
    {
        ret = ADK_ETHERNET_E_INVALID_IF_TYPE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                      "Unexpected interface type %d\n",
                      sub_if_data_p->type);
    }

_adk_eth_intf_acl_config_exit:

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

#else
    RTE_SET_USED(eth_mod_handle);
    RTE_SET_USED(if_handle);
    RTE_SET_USED(is_ipv6);
    RTE_SET_USED(disable_ip);
    RTE_SET_USED(enable_acl);
    RTE_SET_USED(rule_set_id);
#endif

    return (ret);
}

#endif

#ifdef ADK_HOS_OLD_IMPL

adk_status_t
adk_eth_intf_vrd_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    uint32_t vrd_id)
{
    adk_status_t ret = ADK_SUCCESS;

#ifdef ADK_DEVICE_SNR
    adk_sys_handle_t sys_handle;
    adk_eth_intf_internal_data_t *sub_if_data_p = NULL;
    adk_eth_intf_internal_data_t *if_data_p = NULL;
    adk_ethernet_internal_data_t    *eth_internal_data_p = NULL;
    adk_eth_intf_internal_data_t    *if_bridge_data_p = NULL;
    uint32_t if_num;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to configure ACL ID on Ethernet intf\n");
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);
    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (!adk_eth_valid_if_handle(if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle %"ADKPRIh "\n",
                          if_handle);
        goto _adk_eth_intf_vrd_config_exit;
    }

    sub_if_data_p = (adk_eth_intf_internal_data_t *)
        ADK_SYS_INTERFACE_ATTR_PTR_GET(if_handle);
    
    if(sub_if_data_p->type == INTERFACE_TYPE_VLAN)
    {
        if_data_p = (adk_eth_intf_internal_data_t *)
            ADK_SYS_INTERFACE_ATTR_PTR_GET(sub_if_data_p->u.vlan_if.if_handle);

        if_num = ADK_ETH_INTERFACE_ID_GET(sub_if_data_p->u.vlan_if.if_handle);

        if (if_data_p->u.phy_if.is_nac)
        {
            adk_hos_domain_mapper_tbl_rule_attr_t hos_domain_mapper_rule;

            /* Fill in the new data */
            ADK_MEMSET(&hos_domain_mapper_rule, 0x0,
                       sizeof(hos_domain_mapper_rule));
            hos_domain_mapper_rule.vrd_id = vrd_id;

            ret = adk_hos_rule_update(
                ADK_HOS_DOMAIN_MAPPER_TBL, sub_if_data_p->bvi_vrd_rule_id[0],
                (adk_hos_rule_attr_t *) &hos_domain_mapper_rule);
            if (ret != ADK_SUCCESS)
            {
                ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                  "Failed to update domain mapper HOS rule\n");
                goto _adk_eth_intf_vrd_config_exit;
            }
        }
    } else if(sub_if_data_p->type == INTERFACE_TYPE_SWITCH_VIRTUAL)
    {
        if_bridge_data_p = (adk_eth_intf_internal_data_t *)
                    ADK_SYS_INTERFACE_ATTR_PTR_GET(sub_if_data_p->u.switch_virtual_if.svif_data.bridge_if_handle);

        uint16_t bg_id = if_bridge_data_p->u.bridge_if.l2_domain_id;
        adk_eth_vlan_info_t *vlan_info = eth_internal_data_p->vlan_intf_list[bg_id];
        while(vlan_info) {
            if_data_p = (adk_eth_intf_internal_data_t *)
                ADK_SYS_INTERFACE_ATTR_PTR_GET(vlan_info->if_handle);

            if (if_data_p->u.phy_if.is_nac)
            {
                adk_hos_domain_mapper_tbl_rule_attr_t hos_domain_mapper_rule;

                /* Fill in the new data */
                ADK_MEMSET(&hos_domain_mapper_rule, 0x0,
                           sizeof(hos_domain_mapper_rule));
                hos_domain_mapper_rule.vrd_id = vrd_id;

                sub_if_data_p = (adk_eth_intf_internal_data_t *)
                    ADK_SYS_INTERFACE_ATTR_PTR_GET(vlan_info->vlan_if_handle);

                ret = adk_hos_rule_update(
                    ADK_HOS_DOMAIN_MAPPER_TBL, sub_if_data_p->bvi_vrd_rule_id[0],
                    (adk_hos_rule_attr_t *) &hos_domain_mapper_rule);
                if (ret != ADK_SUCCESS)
                {
                    ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                                      "Failed to update domain mapper HOS rule\n");
                    goto _adk_eth_intf_vrd_config_exit;
                }
            }
            /* Store vrd_id to use in adk_ethernet_vlan_id_add() if later any more
             * port/valn added to this bridge
             * */
            vlan_info->vrd_id = vrd_id;
            vlan_info = vlan_info->next;
        }
    }
    else
    {
        ret = ADK_ETHERNET_E_INVALID_IF_TYPE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                      "Unexpected interface type %d\n",
                      sub_if_data_p->type);
    }
_adk_eth_intf_vrd_config_exit:

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

#else
    RTE_SET_USED(eth_mod_handle);
    RTE_SET_USED(if_handle);
    RTE_SET_USED(vrd_id);
#endif

    return (ret);
}

#endif

#ifdef ADK_HOS_OLD_IMPL

adk_status_t
adk_eth_intf_pcap_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    bool is_ipv6,
    bool at_ingress,
    uint32_t pcap_id,
    bool enable_pcap)
{
    adk_status_t ret = ADK_SUCCESS;

#ifdef ADK_DEVICE_SNR
    adk_sys_handle_t sys_handle;
    adk_eth_intf_internal_data_t *sub_if_data_p = NULL;
    adk_eth_intf_internal_data_t *if_data_p = NULL;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to configure IP PCAP on Ethernet intf\n");
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    if (!adk_eth_valid_if_handle(if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle %"ADKPRIh "\n",
                          if_handle);
        goto _adk_eth_intf_pcap_config_exit;
    }

    sub_if_data_p = (adk_eth_intf_internal_data_t *)
        ADK_SYS_INTERFACE_ATTR_PTR_GET(if_handle);

    if (sub_if_data_p->type != INTERFACE_TYPE_VLAN)
    {
        ret = ADK_E_UNSUPPORTED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Unsupported Interface type %"ADKPRIh "\n",
                          if_handle);
        goto _adk_eth_intf_pcap_config_exit;
    }

    if_data_p = (adk_eth_intf_internal_data_t *)
        ADK_SYS_INTERFACE_ATTR_PTR_GET(sub_if_data_p->u.vlan_if.if_handle);

    if (!if_data_p->u.phy_if.is_nac)
    {
        ret = ADK_E_UNSUPPORTED;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Unsupported phy Intf type %"ADKPRIh "\n",
                          if_handle);
        goto _adk_eth_intf_pcap_config_exit;
    }

    if (at_ingress)
    {
        uint32_t tbl_idx;
        adk_hos_rule_id_t rule_id;

        if (!is_ipv6)
        {
            tbl_idx = ADK_HOS_OUTER_IP4_TBL;
            rule_id = sub_if_data_p->ip4_outer_id_rule_id[0];
        }
        else
        {
            tbl_idx = ADK_HOS_OUTER_IP6_TBL;
            rule_id = sub_if_data_p->ip6_outer_id_rule_id[0];
        }
        ret = adk_hos_rule_pcap(tbl_idx, rule_id, pcap_id, enable_pcap);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to configure IP PCAP on eth-intf\n");
            goto _adk_eth_intf_pcap_config_exit;
        }
    }
    else if (enable_pcap)
    {
        adk_hos_e_outer_ip_tbl_rule_attr_t hos_e_outer_ip_rule;
        adk_hos_rule_id_t *rule_id;

        hos_e_outer_ip_rule.port = if_data_p->if_id;
        hos_e_outer_ip_rule.vlan_id = sub_if_data_p->u.vlan_if.vlan_id;
        if (!is_ipv6)
        {
            hos_e_outer_ip_rule.ether_type = ETHER_TYPE_IPV4;
            rule_id = &sub_if_data_p->ip4_e_outer_rule_id;
        }
        else
        {
            hos_e_outer_ip_rule.ether_type = ETHER_TYPE_IPV6;
            rule_id = &sub_if_data_p->ip6_e_outer_rule_id;
        }
        hos_e_outer_ip_rule.pcap_id = pcap_id;
        hos_e_outer_ip_rule.outer_ip_id = ADK_SYS_INTERFACE_ID_GET(if_handle);
        ret = adk_hos_rule_add(ADK_HOS_E_OUTER_IP_TBL,
                               (adk_hos_rule_attr_t *) &hos_e_outer_ip_rule,
                               rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to add egress IP PCAP on eth-intf\n");
            goto _adk_eth_intf_pcap_config_exit;
        }
    }
    else    /* disable egress pcap */
    {
        adk_hos_rule_id_t rule_id;

        if (!is_ipv6)
        {
            rule_id = sub_if_data_p->ip4_e_outer_rule_id;
        }
        else
        {
            rule_id = sub_if_data_p->ip6_e_outer_rule_id;
        }

        ret = adk_hos_rule_delete(ADK_HOS_E_OUTER_IP_TBL, rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to removed egress IP PCAP on eth-intf\n");
            goto _adk_eth_intf_pcap_config_exit;
        }
    }

_adk_eth_intf_pcap_config_exit:

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

#else
    RTE_SET_USED(eth_mod_handle);
    RTE_SET_USED(if_handle);
    RTE_SET_USED(is_ipv6);
    RTE_SET_USED(at_ingress);
    RTE_SET_USED(pcap_id);
    RTE_SET_USED(enable_pcap);
#endif

    return (ret);
}

#endif

/*!
 * =============================================================
 *
 * adk_ethernet_bridge_interface_create
 *
 * @param[in] eth_mod_handle      Handle to the Ethernet module
 * @param[in] bridge_intf_attr_p  Structure containing attributes for the 
 *                                bridge interface to be created
 * @param[out] bridge_if_handle_p Pointer to the Interface Handle created
 *
 * @retval  ADK_SUCCESS             Successful completion
 * @retval  ADK_E_INVALID_HANDLE    Invalid Ethernet module handle specified
 * @retval  ADK_E_INVALID_PARAMETER Pointer passed as NULL
 * @retval  ADK_E_MEM_UNAVAILABLE   Operation failed due to insufficient memory
 *                                  specified
 * @retval  ADK_E_HW_PROGRAMMING_ERROR HLP table programming error
 *
 * @brief   API to create a bridge interface
 *
 * This API creates a bridge interface and sets all its initial
 * attributes as specified in the passed attributes data structure. 
 * On bridge port packet with ethertype 0x8100 is treated as tagged packet
 * For each L2 domain single bridge interface can be created
 *
 * =============================================================
 */
adk_status_t adk_ethernet_bridge_interface_create(adk_module_handle_t eth_mod_handle,
        const adk_ethernet_bridge_intf_attr_t *bridge_intf_attr_p,
        adk_interface_handle_t *bridge_if_handle_p)
{
    adk_status_t                    ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t    *eth_internal_data_p = NULL;
    adk_eth_intf_internal_data_t    *b_if_data_p = NULL;
    int                             bg_id;
    uint32_t                        b_if_num, stats_id;
    adk_sys_handle_t                sys_handle;
    adk_dp_eth_bridge_intf_t        *dp_bridge_intf_p = NULL;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to create switch virtual interface\n");
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)
    {
        adk_eth_lists_clear(eth_internal_data_p);
        ADK_ETH_LOG_INFO(eth_mod_handle, "All ethernet lists cleared\n");
    }

    if (bridge_if_handle_p == NULL) {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "NULL Interface Handle Pointer\n");
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }

    if (bridge_intf_attr_p == NULL) {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "NULL Bridge Interface Data Pointer\n");
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }

    bg_id = bridge_intf_attr_p->l2_domain_id;

    /* Although IES APIs don't enforce this condition, let's ADK enforce this to begin with */
    if (ADK_ETH_IS_BG_ID_INVALID(bg_id))
    {
        ret = ADK_ETHERNET_E_INVALID_BG_ID;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret, "Invalid Bridge Group ID 0x%04x\n",
                          bg_id);
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }

    /* Validate whether bridge interface already created for the L2 Domain*/
    if(eth_internal_data_p->dp_pvt_p->bridge_map_id[bg_id] != ADK_ETH_MAX_BG_ID) 
    {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Bridge Interface already created for L2 Domain %d\n",bg_id);
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }
    b_if_num = adk_eth_create_bridge_if_id(bg_id);

    ADK_ETH_LOG_INFO(eth_mod_handle, "Created bridge-interface ID 0x%04x\n",
                     b_if_num);
    
    /* Allocate a NameSpace entry for statistics */
    stats_id = adk_rm_resource_reserve(eth_internal_data_p->bridgeIf_id_ns_hdl);
    if (stats_id == ADK_RM_RSRC_INVALID)
    {
        ret = ADK_E_RESOURCE_UNAVAILABLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to allocate Stats NS ID for b_if_num %d\n",
                          b_if_num);
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }

    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Allocated Stats NS ID 0x%08x for b_if_num 0x%04x\n",
                     stats_id, b_if_num);
    
    ADK_SYS_INTERFACE_HANDLE_ALLOC(eth_mod_handle, ADK_ETH_INTF_OBJ,
                                   sizeof(adk_eth_intf_internal_data_t),
                                   bridge_if_handle_p);
    if (*bridge_if_handle_p == 0) {
        ADK_ETH_LOG_FATAL(eth_mod_handle, ADK_E_MEM_UNAVAILABLE,
                          "Memory allocation failed\n");
        ret = ADK_E_MEM_UNAVAILABLE;
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }

    b_if_data_p = (adk_eth_intf_internal_data_t *) 
        ADK_SYS_INTERFACE_ATTR_PTR_GET(*bridge_if_handle_p);
    
    ADK_SYS_INTERFACE_ID_SET(*bridge_if_handle_p, stats_id);

    b_if_data_p->cookie = ADK_ETHERNET_INTF_COOKIE;
    b_if_data_p->type = INTERFACE_TYPE_BRIDGE;
    b_if_data_p->if_id = b_if_num;
    ADK_MEMCPY(&b_if_data_p->u.bridge_if.src_mac, bridge_intf_attr_p->src_mac,
                   sizeof(adk_ethernet_mac_addr_t));
    b_if_data_p->u.bridge_if.l2_domain_id = bridge_intf_attr_p->l2_domain_id;
    b_if_data_p->u.bridge_if.priority_tag = false;

    /* Init the DP table element for the sub-interface */
    dp_bridge_intf_p = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_BRIDGE_INTF_OBJ,
					 1, sizeof(adk_dp_eth_bridge_intf_t),
                                      ADK_ETH_DP_OBJECT_ALIGN);
    if (dp_bridge_intf_p == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate DP memory for BVI entry"
                          " (bgid - %d)\n", bg_id);
        goto _adk_ethernet_bridge_interface_create_exit_1;
    }
    
    dp_bridge_intf_p->eth_dev_id = bridge_intf_attr_p->eth_dev_id;
    dp_bridge_intf_p->l2_domain_id = bg_id;
    dp_bridge_intf_p->statsId = stats_id;
    dp_bridge_intf_p->bridgeIntfId = b_if_num;
    dp_bridge_intf_p->priority_tag = false;
    dp_bridge_intf_p->bridgeIntfHandle = *bridge_if_handle_p;
    eth_internal_data_p->dp_pvt_p->bridge_intf[stats_id] = dp_bridge_intf_p;
    eth_internal_data_p->dp_pvt_p->bridge_map_id[bg_id] = stats_id;
    /* This is only for L3 CALL when there is only one L2 domain */
    *(eth_internal_data_p->dp_pvt_p->default_l2_domain) = bridge_intf_attr_p->l2_domain_id;

    ADK_MEMCPY(&dp_bridge_intf_p->src_mac, bridge_intf_attr_p->src_mac,
                   sizeof(adk_ethernet_mac_addr_t));

    b_if_data_p->hos_l2_term_br_intf_rule_id = 0;
    b_if_data_p->hos_mac_mapper_rule_id = ADK_HOS_MAC_MAPPER_ID_INVALID;
    b_if_data_p->hos_multicast_mac_mapper_rule_id = ADK_HOS_MAC_MAPPER_ID_INVALID;
    b_if_data_p->hos_multicast_v6_mac_mapper_rule_id = ADK_HOS_MAC_MAPPER_ID_INVALID;
    b_if_data_p->hos_broadcast_mac_mapper_rule_id = ADK_HOS_MAC_MAPPER_ID_INVALID;
        
        
        

#ifdef ADK_NETD_NETLINK
    /* Create NetDriver Bridge interface */
    if(ADK_SUCCESS != netd_bridge_interface_create(eth_mod_handle, bridge_intf_attr_p->src_mac))
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to create netd Bridge interface\n");
        goto _adk_ethernet_bridge_interface_create_exit;
    }         
    
    /* Create NetDriver Bridge interface */
    if(ADK_SUCCESS != netd_bridge_interface_up(eth_mod_handle))
    { 
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to send link status change message for netd Bridge interface\n");
        goto _adk_ethernet_bridge_interface_create_exit;
    }         
#endif

    ret = adk_sys_flow_cache_update_version(sys_handle);
    if(ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to update global flow cache version");
        goto _adk_ethernet_bridge_interface_create_exit;
    }
#ifdef ADK_NETD_NETLINK
_adk_ethernet_bridge_interface_create_exit:

    if (ret == ADK_SUCCESS) {
        ADK_ETH_LOG_INFO(eth_mod_handle, 
                         "Created bridge interface 0x%"ADKPRIh "\n",
                         *bridge_if_handle_p);
    } else {
        if (bridge_if_handle_p && *bridge_if_handle_p) {
            ADK_SYS_INTERFACE_HANDLE_FREE(*bridge_if_handle_p);
            *bridge_if_handle_p = 0;
        }

        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to create bridge interface\n");
    }
#endif

_adk_ethernet_bridge_interface_create_exit_1:
    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return(ret);
}

/*!
 * =============================================================
 *
 * adk_ethernet_bridge_interface_delete
 *
 * @param[in] eth_mod_handle        Handle to the Ethernet module
 * @param[in] bridge_if_handle_p    Handle of the virtual interface to be deleted
 *
 * @retval  ADK_SUCCESS             Successful completion
 * @retval  ADK_E_INVALID_HANDLE    Invalid Ethernet module handle specified
 * @retval  ADK_ETHERNET_E_INVALID_IF_HANDLE Invalid interface handle specified
 * @retval  ADK_ETHERNET_E_INCOMPAT_IF_TYPE  Interface specified is not a switch
 *                                      virtual interface
 * @retval  ADK_E_HW_PROGRAMMING_ERROR HLP table programming error
 *
 * @brief  Function to delete a bridge interface
 *
 * This function deletes a BG_ID based bridge interface, deallocates the
 * resources reserved for it and frees up the internal data structure associated
 * with it. The HLP entries, such as L2 term entry, is also removed from HLP.
 *
 * =============================================================
 */
adk_status_t
adk_ethernet_bridge_interface_delete (
    adk_module_handle_t  eth_mod_handle,
    adk_interface_handle_t  bridge_if_handle)
{
    adk_status_t                    ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t    *eth_internal_data_p = NULL;
    adk_eth_intf_internal_data_t    *b_if_data_p = NULL;
    uint32_t                        stats_id;
    adk_sys_handle_t                sys_handle;
    adk_dp_eth_bridge_intf_t        *dp_bridge_intf_p = NULL;
    uint32_t                        bg_id;
    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to delete bridge interface\n");
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)
    {
        adk_eth_lists_clear(eth_internal_data_p);
        ADK_ETH_LOG_INFO(eth_mod_handle, "All ethernet lists cleared\n");
    }

    if (bridge_if_handle == NULL) {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "NULL Interface Handle Pointer\n");
        goto _adk_ethernet_bridge_if_delete_exit_1;
    }

    if (!adk_eth_valid_if_handle(bridge_if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle 0x%"ADKPRIh "\n", bridge_if_handle);
        goto _adk_ethernet_bridge_if_delete_exit;
    }

    /* Get the data for pattern creation */
    b_if_data_p = (adk_eth_intf_internal_data_t *)
        ADK_SYS_INTERFACE_ATTR_PTR_GET(bridge_if_handle);

    if (b_if_data_p->type != INTERFACE_TYPE_BRIDGE)
    {
        ret = ADK_ETHERNET_E_INCOMPAT_IF_TYPE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Incompatible Interface type %s\n",
                          adk_eth_intf_type_name(b_if_data_p->type));
        goto _adk_ethernet_bridge_if_delete_exit;
    }

    stats_id = ADK_SYS_INTERFACE_ID_GET(bridge_if_handle);
    if(b_if_data_p->hos_l2_term_br_intf_rule_id)
    {
        ret = adk_hos_rule_delete(ADK_HOS_L2_TERM_INTF_TBL,
                b_if_data_p->hos_l2_term_br_intf_rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete Bridge MAC-Term HOS rule\n");
            goto _adk_ethernet_bridge_if_delete_exit;
        }
    }
    if(ADK_HOS_MAC_MAPPER_ID_INVALID != b_if_data_p->hos_mac_mapper_rule_id)
    {
        ret = adk_hos_mac_mapper_delete(b_if_data_p->hos_mac_mapper_rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete Bridge MAC mapper HOS rule\n");
            goto _adk_ethernet_bridge_if_delete_exit;
        }
    }
    if(ADK_HOS_MAC_MAPPER_ID_INVALID != b_if_data_p->hos_multicast_mac_mapper_rule_id)
    {
        ret = adk_hos_mac_mapper_delete(b_if_data_p->hos_multicast_mac_mapper_rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete mutlicast MAC mapper HOS rule\n");
            goto _adk_ethernet_bridge_if_delete_exit;
        }
    }
    if(ADK_HOS_MAC_MAPPER_ID_INVALID != b_if_data_p->hos_multicast_v6_mac_mapper_rule_id)
    {
        ret = adk_hos_mac_mapper_delete(b_if_data_p->hos_multicast_v6_mac_mapper_rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete mutlicast_v6 MAC mapper HOS rule\n");
            goto _adk_ethernet_bridge_if_delete_exit;
        }
    }
    if(ADK_HOS_MAC_MAPPER_ID_INVALID != b_if_data_p->hos_broadcast_mac_mapper_rule_id)
    {
        ret = adk_hos_mac_mapper_delete(b_if_data_p->hos_broadcast_mac_mapper_rule_id);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                    "Failed to delete broadcast MAC mapper HOS rule\n");
            goto _adk_ethernet_bridge_if_delete_exit;
        }
    }
    /* Init the DP table element for the sub-interface */
    dp_bridge_intf_p = eth_internal_data_p->dp_pvt_p->bridge_intf[stats_id];
    eth_internal_data_p->dp_pvt_p->bridge_intf[stats_id] = NULL;
    bg_id = b_if_data_p->u.bridge_if.l2_domain_id;
    eth_internal_data_p->dp_pvt_p->bridge_map_id[bg_id] = ADK_ETH_MAX_BG_ID;

    adk_rcu_distributed_ver_update();

    adk_eth_list_add(eth_internal_data_p, ADK_ETH_BRIDGE_ID_LIST_ID,
                     (uintptr_t) stats_id);

    adk_eth_list_add(eth_internal_data_p, ADK_ETH_BRIDGE_LIST_ID,
                     (uintptr_t) dp_bridge_intf_p);

    /* Free the NameSpace entry for statistics */
    if (adk_rm_resource_free(eth_internal_data_p->bridgeIf_id_ns_hdl, stats_id) == FALSE)
    {
        ret = ADK_E_INTERNAL_ERROR;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to free Stats NS ID 0x%08x!\n", stats_id);
        goto _adk_ethernet_bridge_if_delete_exit;
    }

#ifdef ADK_NETD_NETLINK
    /* Create NetDriver Bridge interface */
    if(ADK_SUCCESS != netd_bridge_interface_delete(eth_mod_handle))
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to delete netd Bridge interface\n");
        goto _adk_ethernet_bridge_if_delete_exit;
    }         
#endif
 
    ADK_SYS_INTERFACE_HANDLE_FREE(bridge_if_handle);

    ret = adk_sys_flow_cache_update_version(sys_handle);
    if(ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to update global flow cache version");
        goto _adk_ethernet_bridge_if_delete_exit;
    }

_adk_ethernet_bridge_if_delete_exit:

    if (ret == ADK_SUCCESS) {
        ADK_ETH_LOG_INFO(eth_mod_handle, 
                         "Deleted bridge interface 0x%"ADKPRIh "\n",
                         bridge_if_handle);
    } else {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to delete bridge interface\n");
    }

_adk_ethernet_bridge_if_delete_exit_1:
    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return(ret);
}

/*!
 * =============================================================
 *
 * adk_ethernet_switch_virtual_if_create
 *
 * @param[in] eth_mod_handle    Handle to the Ethernet module
 * @param[in] svif_data         The attributes of the switch virtual interface.
 * @param[out] v_if_handle_p    Pointer to the switch virtual interface
 *                              handle created
 *
 * @retval  ADK_SUCCESS             Successful completion
 * @retval  ADK_E_INVALID_HANDLE    Invalid Ethernet module handle specified
 * @retval  ADK_ETHERNET_E_INVALID_IF_HANDLE Invalid interface handle specified
 * @retval  ADK_ETHERNET_E_INCOMPAT_IF_TYPE Operation not permitted on passed
 * interface type
 * @retval  ADK_ETHERNET_E_INVALID_VLAN_ID   Invalid VLAN tag specified
 * @retval  ADK_ETHERNET_E_INVALID_BG_ID     Invalid BG_ID specified
 * @retval  ADK_E_HW_PROGRAMMING_ERROR       HLP table programming error
 *
 * @brief   Function to create a switch virtual interface
 *
 * This function creates a switch virtual interface on top of a bridge.
 * This interface is used to re-direct packets received on switch interfaces
 * to higher layer or CPU if the destination MAC address
 * is a bridge MAC address and matching VLAN. In the transmit direction,
 * the source MAC address is given in bridge interface configuration.
 *
 * =============================================================
 */
adk_status_t
adk_ethernet_switch_virtual_if_create (
    adk_module_handle_t eth_mod_handle,
    const adk_ethernet_switch_virtual_if_data_t *svif_data,
    adk_interface_handle_t *v_if_handle_p)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_dp_status_t dp_ret = ADK_SUCCESS;
    adk_eth_intf_internal_data_t *if_bridge_intf_p = NULL;
    adk_if_bridge_data_t* if_bridge_data_p = NULL;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_svif_intf_t *dp_svif_intf_p = NULL;
    adk_eth_intf_internal_data_t    *svif_if_data_p = NULL;
    uint32_t stats_id = ADK_RM_RSRC_INVALID;
    adk_ethernet_internal_data_t    *eth_internal_data_p = NULL;
    adk_eth_vlan_info_t *vlan_info=NULL, **vlan_info_list;
    uint8_t vrd_id = ADK_ETH_UNKNOWN_VRD;
    uint32_t v_if_num;
    adk_ethernet_vlan_id_t vlan_id = 1;
    adk_ethernet_bg_id_t bg_id = ADK_ETHERNET_INVALID_BRIDGE_ID;
    adk_dp_eth_bridge_intf_t        *dp_bridge_intf_p = NULL;
    uint32_t                        vsi_id;
    uint32_t                        if_num;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to add VLAN ID 0x%04x for bridge interface %"ADKPRIh "\n",
                          svif_data->vlan_id, svif_data->bridge_if_handle);
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)
    {
        adk_eth_lists_clear(eth_internal_data_p);
        ADK_ETH_LOG_INFO(eth_mod_handle, "All ethernet lists cleared\n");
    }

    if (v_if_handle_p == NULL) {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "NULL Interface Handle Pointer\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    if (svif_data == NULL) {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "NULL Switch Virtual Interface Data Pointer\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    vlan_id = svif_data->vlan_id;
    if (ADK_ETH_IS_VLAN_INVALID(svif_data->vlan_id))
    {
        ret = ADK_ETHERNET_E_INVALID_VLAN_ID;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid VLAN ID 0x%04x\n", vlan_id);
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    if (!adk_eth_valid_if_handle(svif_data->bridge_if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle %"ADKPRIh "\n", svif_data->bridge_if_handle);
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    if_bridge_intf_p = (adk_eth_intf_internal_data_t *)
                ADK_SYS_INTERFACE_ATTR_PTR_GET(svif_data->bridge_if_handle);

    if (if_bridge_intf_p->type != INTERFACE_TYPE_BRIDGE)
    {
        ret = ADK_ETHERNET_E_INCOMPAT_IF_TYPE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Incompatible Interface type %s\n",
                          adk_eth_intf_type_name(if_bridge_intf_p->type));
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    bg_id = if_bridge_intf_p->u.bridge_if.l2_domain_id;
   
    v_if_num = adk_eth_create_virtual_if_id(bg_id);

    ADK_ETH_LOG_INFO(eth_mod_handle, "Created virtual-interface ID 0x%04x\n",
                     v_if_num);

    /* Allocate a NameSpace entry for statistics */
    stats_id = adk_rm_resource_reserve(eth_internal_data_p->svif_id_ns_hdl);

    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Allocated Stats NS ID 0x%08x for v_if_num 0x%04x\n",
                     stats_id, v_if_num);
    
    ADK_SYS_INTERFACE_HANDLE_ALLOC(eth_mod_handle, ADK_ETH_INTF_OBJ,
                                   sizeof(adk_eth_intf_internal_data_t),
                                   v_if_handle_p);
    if (*v_if_handle_p == 0)
    {
        ADK_ETH_LOG_FATAL(eth_mod_handle, ADK_E_MEM_UNAVAILABLE,
                          "Memory allocation failed\n");
        ret = ADK_E_MEM_UNAVAILABLE;
        goto  _adk_ethernet_switch_virtual_if_create_exit;
    }

    /* First 64 IDs are reserved for Bridge interfaces */
    ADK_SYS_INTERFACE_ID_SET(*v_if_handle_p, stats_id + ADK_ETH_NUM_BRIDGE);

    svif_if_data_p = (adk_eth_intf_internal_data_t *)
                    ADK_SYS_INTERFACE_ATTR_PTR_GET(*v_if_handle_p);

    svif_if_data_p->cookie = ADK_ETHERNET_INTF_COOKIE;
    svif_if_data_p->type = INTERFACE_TYPE_SWITCH_VIRTUAL;
    svif_if_data_p->u.switch_virtual_if.svif_data.vlan_id = vlan_id;
    svif_if_data_p->u.switch_virtual_if.svif_data.bridge_if_handle = svif_data->bridge_if_handle;

    /* Init the DP table element for the sub-interface */
    dp_svif_intf_p = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_SUB_INTF_OBJ,
					 1, sizeof(adk_dp_eth_svif_intf_t),
                                      ADK_ETH_DP_OBJECT_ALIGN);
    if (dp_svif_intf_p == NULL)
    {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate DP memory for svif-intf"
                          " (bridge-%d, vlan-%d)\n", bg_id, vlan_id);
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    eth_internal_data_p->dp_pvt_p->eth_svif_if[stats_id] = dp_svif_intf_p;
    dp_svif_intf_p->statsId = stats_id;
    dp_svif_intf_p->bgid = bg_id;
    dp_svif_intf_p->vlan = vlan_id;
    dp_svif_intf_p->svifIntfHandle = *v_if_handle_p;

    /* Hook up the DP table element with CP Sub-Interface struct */
    svif_if_data_p->dp_if_ptr = dp_svif_intf_p;

    /* Add the DP table element to the DP table */
    dp_ret = adk_eth_dp_svif_add(eth_mod_handle,
                               eth_internal_data_p->dp_pvt_p->eth_svif_if_hash_table,
                               dp_svif_intf_p);
    if (dp_ret != ADK_SUCCESS)
    {
        ret = ADK_E_INTERNAL_ERROR;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to add sub interface to dataplane\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    vlan_info_list = &eth_internal_data_p->vlan_intf_list[bg_id];
    if(*vlan_info_list) {
        /* Use vrd_id configured for this bridge in ip interface create,
         * if already exists. adk_eth_intf_vrd_config() will set vrd_id as part of
         * IP interface creation
         * */
        vrd_id = (*vlan_info_list)->vrd_id;
    }

    vlan_info = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(eth_mod_handle), ADK_ETH_VLAN_INFO_OBJ, 1, sizeof(adk_eth_vlan_info_t));
    if(!vlan_info) {
        ret = ADK_E_MEM_UNAVAILABLE;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to allocate memory for vlan info\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

/*    vlan_info->if_handle = if_handle; */ 
/* Don't have the notion of interface on external physical port */
    vlan_info->vlan_id = vlan_id;
    vlan_info->vlan_if_handle = *v_if_handle_p;
    vlan_info->stats_id = stats_id;
    vlan_info->next = *vlan_info_list;
    *vlan_info_list = vlan_info;

    if (vlan_id != 0)
    {
        if(eth_internal_data_p->vlan_ref_cnt[vlan_id] == 0) {
            debug_print("New vlan added %d\n", ret);
        }
        eth_internal_data_p->vlan_ref_cnt[vlan_id]++;
    }

    if_bridge_data_p = &(if_bridge_intf_p->u.bridge_if);
    uint64_t bridge_mac =  (if_bridge_data_p->src_mac)[0];
    bridge_mac = (bridge_mac << 8) | (if_bridge_data_p->src_mac)[1];
    bridge_mac = (bridge_mac << 8) | (if_bridge_data_p->src_mac)[2];
    bridge_mac = (bridge_mac << 8) | (if_bridge_data_p->src_mac)[3];
    bridge_mac = (bridge_mac << 8) | (if_bridge_data_p->src_mac)[4];
    bridge_mac = (bridge_mac << 8) | (if_bridge_data_p->src_mac)[5];

    if_num = ADK_SYS_INTERFACE_ID_GET(svif_data->bridge_if_handle);
    dp_bridge_intf_p = eth_internal_data_p->dp_pvt_p->bridge_intf[if_num];

    ret = adk_sys_eth_dev_vsi_get(dp_bridge_intf_p->eth_dev_id,
            &vsi_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to get the vsi number from the eth device ID\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    adk_hos_l2_term_intf_tbl_rule_attr_t hos_l2_term_intf_rule = 
    {
        .l2_domain = if_bridge_data_p->l2_domain_id,
        .dmac = bridge_mac,
        .vlan_id = vlan_id,
        .l2_interface = stats_id,
        .vsi_id = vsi_id,
        .vsi_q = 0,
        .is_l2_term = true,
        .is_l2_m_b = false
    };

    ret = adk_hos_rule_add(ADK_HOS_L2_TERM_INTF_TBL,
                               (adk_hos_rule_attr_t *) &hos_l2_term_intf_rule,
                               &svif_if_data_p->hos_l2_term_svif_intf_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to add SVIF Term-Interface ID HOS rule for unicast traffic\n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }
    /* Add rule for multicast/broadcast traffic. These 
       are treated same as L2 terminating traffic */

    hos_l2_term_intf_rule.dmac = MULTICAST_MAC_ADDR;
    hos_l2_term_intf_rule.is_l2_m_b = true;
    ret = adk_hos_rule_add(ADK_HOS_L2_TERM_INTF_TBL,
                               (adk_hos_rule_attr_t *) &hos_l2_term_intf_rule,
                               &svif_if_data_p->hos_l2_term_svif_intf_m_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to add SVIF Term-Interface ID HOS rule for multicast \n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }


    hos_l2_term_intf_rule.dmac = MULTICAST_MAC_ADDR_V6;
    hos_l2_term_intf_rule.is_l2_m_b = true;
    ret = adk_hos_rule_add(ADK_HOS_L2_TERM_INTF_TBL,
            (adk_hos_rule_attr_t *) &hos_l2_term_intf_rule,
            &svif_if_data_p->hos_l2_term_svif_intf_m_v6_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to add SVIF Term-Interface ID HOS rule for multicast_v6 \n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    hos_l2_term_intf_rule.dmac = BROADCAST_MAC_ADDR;
    hos_l2_term_intf_rule.is_l2_m_b = true;
    ret = adk_hos_rule_add(ADK_HOS_L2_TERM_INTF_TBL,
            (adk_hos_rule_attr_t *) &hos_l2_term_intf_rule,
            &svif_if_data_p->hos_l2_term_svif_intf_b_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to add SVIF Term-Interface ID HOS rule for broadcast \n");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }

    ret = adk_sys_flow_cache_update_version(sys_handle);
    if(ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to update global flow cache version");
        goto _adk_ethernet_switch_virtual_if_create_exit;
    }
_adk_ethernet_switch_virtual_if_create_exit:

    if (ret == ADK_SUCCESS)
    {
        ADK_ETH_LOG_INFO(eth_mod_handle,
                         "Added BG_ID 0x%04x for VLAN 0x%04x\n",
                         bg_id,  vlan_id);
    }
    else
    {
        if (vlan_info)
        {
            ADK_NVM_FREE(vlan_info);
        }
        if (dp_svif_intf_p)
        {
            ADK_DP_SHM_FREE(dp_svif_intf_p);
        }
        if (v_if_handle_p && *v_if_handle_p)
        {
            ADK_SYS_INTERFACE_HANDLE_FREE(*v_if_handle_p);
        }
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to add BG_ID 0x%04x for VLAN 0x%04x\n",
                          bg_id, vlan_id);
    }

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}

/*!
 * =============================================================
 *
 * adk_ethernet_switch_virtual_if_delete
 *
 * @param[in] eth_mod_handle    Handle to the Ethernet module
 * @param[in] v_if_handle       Handle of the virtual interface to be deleted
 *
 * @retval  ADK_SUCCESS             Successful completion
 * @retval  ADK_E_INVALID_HANDLE    Invalid Ethernet module handle specified
 * @retval  ADK_ETHERNET_E_INVALID_IF_HANDLE Invalid Interface Handle specified
 * @retval  ADK_ETHERNET_E_INCOMPAT_IF_TYPE  Interface specified is not a switch
 *                                      virtual interface
 * @retval  ADK_E_HW_PROGRAMMING_ERROR HLP table programming error
 *
 * @brief  API to delete a switch virtual interface
 *
 * This API deletes a VLAN based switch virtual interface, deallocates the 
 * resources reserved for it and frees up the internal data structure associated
 * with it.
 *
 * =============================================================
 */
adk_status_t adk_ethernet_switch_virtual_if_delete(
        adk_module_handle_t eth_mod_handle,
        adk_interface_handle_t v_if_handle)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_dp_status_t dp_ret = ADK_SUCCESS;
    adk_eth_intf_internal_data_t *svif_if_data_p = NULL;
    adk_eth_intf_internal_data_t *if_bridge_data_p = NULL;
    adk_sys_handle_t sys_handle;
    adk_dp_eth_svif_intf_t       *dp_svif_intf_p = NULL;
    uint32_t stats_id = ADK_RM_RSRC_INVALID;
    adk_ethernet_internal_data_t    *eth_internal_data_p = NULL;
    adk_eth_vlan_info_t *vlan_info=NULL, *prev_vlan_info=NULL, **vlan_info_list;
    adk_ethernet_vlan_id_t vlan_id = 1;
    adk_ethernet_bg_id_t bg_id = 0;
    uint32_t found_vlan = 0;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to delete switch virtual Interface %"ADKPRIh "\n",
                          v_if_handle);
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)
    {
        adk_eth_lists_clear(eth_internal_data_p);
        ADK_ETH_LOG_INFO(eth_mod_handle, "All ethernet lists cleared\n");
    }

    if (!adk_eth_valid_if_handle(v_if_handle))
    {
        ret = ADK_ETHERNET_E_INVALID_IF_HANDLE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface handle %"ADKPRIh "\n", v_if_handle);
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    svif_if_data_p = (adk_eth_intf_internal_data_t *)
                ADK_SYS_INTERFACE_ATTR_PTR_GET(v_if_handle);

    if (svif_if_data_p->type != INTERFACE_TYPE_SWITCH_VIRTUAL)
    {
        ret = ADK_ETHERNET_E_INCOMPAT_IF_TYPE;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Incompatible Interface type %s\n",
                          adk_eth_intf_type_name(svif_if_data_p->type));
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    vlan_id = svif_if_data_p->u.switch_virtual_if.svif_data.vlan_id;

    if_bridge_data_p = (adk_eth_intf_internal_data_t *)
                ADK_SYS_INTERFACE_ATTR_PTR_GET(svif_if_data_p->u.switch_virtual_if.svif_data.bridge_if_handle);

    bg_id = if_bridge_data_p->u.bridge_if.l2_domain_id;


    /* TODO: ALgorithm to be rechecked/optimized */
    vlan_info_list = &eth_internal_data_p->vlan_intf_list[bg_id];

    vlan_info = *vlan_info_list;
    while(vlan_info) {
        if(vlan_info->vlan_if_handle == v_if_handle)
        {
            found_vlan = 1;
            stats_id = vlan_info->stats_id;
            
            if(prev_vlan_info) {
                prev_vlan_info->next = vlan_info->next;
            }
            else {
                *vlan_info_list = vlan_info->next;
            }
            break;
        } else {
            prev_vlan_info = vlan_info;
            vlan_info = vlan_info->next;
        }
    }

    if(!found_vlan) {
        ret = ADK_E_INTERNAL_ERROR;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "VLAN info is not found. Unexpected error... \n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    if (vlan_id != 0)
    {
        eth_internal_data_p->vlan_ref_cnt[vlan_id] --;
        if(eth_internal_data_p->vlan_ref_cnt[vlan_id] == 0)
	{
            debug_print("vlan %d deleted\n", vlan_id);
	}
    }
    ret = adk_hos_rule_delete(ADK_HOS_L2_TERM_INTF_TBL,
                               svif_if_data_p->hos_l2_term_svif_intf_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to delete SVIF intf HOS rule for unicast traffic\n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Freed Stats NS ID 0x%08x\n",
                     stats_id);
    ret = adk_hos_rule_delete(ADK_HOS_L2_TERM_INTF_TBL,
                               svif_if_data_p->hos_l2_term_svif_intf_m_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to delete SVIF intf HOS rule for multicast traffic\n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    ret = adk_hos_rule_delete(ADK_HOS_L2_TERM_INTF_TBL,
            svif_if_data_p->hos_l2_term_svif_intf_m_v6_rule_id);
    if (ret != ADK_SUCCESS)
    {    
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to delete SVIF intf HOS rule for multicast_v6 traffic\n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    } 

    ret = adk_hos_rule_delete(ADK_HOS_L2_TERM_INTF_TBL,
            svif_if_data_p->hos_l2_term_svif_intf_b_rule_id);
    if (ret != ADK_SUCCESS)
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to delete SVIF intf HOS rule for broadcast traffic\n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }


    /* Init the DP table element for the sub-interface */
    dp_svif_intf_p = eth_internal_data_p->dp_pvt_p->eth_svif_if[stats_id];

    /* Add the DP table element to the DP table */
    dp_ret = adk_eth_dp_svif_rem(eth_mod_handle,
                               eth_internal_data_p->dp_pvt_p->eth_svif_if_hash_table,
                               dp_svif_intf_p);
    if (dp_ret != ADK_SUCCESS)
    {
        ret = ADK_E_INTERNAL_ERROR;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to remove svif interface from dataplane\n");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

    eth_internal_data_p->dp_pvt_p->eth_svif_if[stats_id] = NULL;

    adk_rcu_distributed_ver_update();

    adk_eth_list_add(eth_internal_data_p, ADK_ETH_SVIF_ID_LIST_ID,
                     (uintptr_t) stats_id);

    adk_eth_list_add(eth_internal_data_p, ADK_ETH_SVIF_INTF_LIST_ID,
                     (uintptr_t) dp_svif_intf_p);

    ADK_NVM_FREE(vlan_info);

    /* Free the NameSpace entry for statistics */
    if (adk_rm_resource_free(eth_internal_data_p->svif_id_ns_hdl, stats_id) == FALSE)
    {
        ret = ADK_E_INTERNAL_ERROR;
        ADK_ETH_LOG_FATAL(eth_mod_handle, ret,
                          "Failed to free Stats NS ID 0x%08x!\n", stats_id);
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }
    ADK_ETH_LOG_INFO(eth_mod_handle,
                     "Freed Stats NS ID 0x%08x\n",
                     stats_id);


    ret = adk_sys_flow_cache_update_version(sys_handle);
    if(ret != ADK_SUCCESS)
    {   
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                "Failed to update global flow cache version");
        goto _adk_ethernet_switch_virtual_if_delete_exit;
    }

_adk_ethernet_switch_virtual_if_delete_exit:

    if (ret == ADK_SUCCESS)
    {
        svif_if_data_p->cookie = 0;
        ADK_MEMSET(svif_if_data_p, 0, sizeof(adk_eth_intf_internal_data_t));
        ADK_SYS_INTERFACE_HANDLE_FREE(v_if_handle);
        ADK_ETH_LOG_INFO(eth_mod_handle,
                         "Deleted BG_ID 0x%04x for Interface %"ADKPRIh " and VLAN 0x%04x\n",
                         bg_id, v_if_handle, vlan_id);
    }
    else
    {
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Failed to delete BG_ID 0x%04x for Interface %"ADKPRIh " and VLAN 0x%04x\n",
                          bg_id, v_if_handle, vlan_id);
    }

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}

/*!
 * =============================================================
 *
 * adk_eth_intf_get_vlan 
 *
 * @param[in] eth_mod_handle    Handle to the Ethernet module
 * @param[in] interface_id      Ethernet interface ID
 * @param[out] vlan_id          Extracted Vlan
 *
 * @retval  ADK_SUCCESS             Successful completion
 * @retval  ADK_E_INVALID_HANDLE    Invalid Ethernet module handle specified
 * @retval  ADK_E_INVALID_PARAMETER Invalid Interface ID specified
 *
 * @brief  API to extract VLAN from ethernet interface ID
 *
 * This API extracts VLAN from switch virtual interface. This is a helper
 * API needed for correct HLP programming of IP classifier rule
 *
 * =============================================================
 */
adk_status_t
adk_eth_intf_get_vlan(adk_module_handle_t eth_mod_handle,
    const uint32_t interface_id,
    adk_ethernet_vlan_id_t*  vlan_id)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    adk_ethernet_internal_data_t *eth_internal_data_p = NULL;
    adk_dp_eth_svif_intf_t* eth_svif_if_s = NULL;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        ADK_ETH_LOG_FORCE("Failed to get vlan from Ethernet Interface ID %d\n",
                          interface_id);
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)
    {
        adk_eth_lists_clear(eth_internal_data_p);
        ADK_ETH_LOG_INFO(eth_mod_handle, "All ethernet lists cleared\n");
    }

    if (interface_id >= (ADK_ETH_IF_SVIF_IF_MAX + ADK_ETH_NUM_BRIDGE))
    {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface ID %d\n", interface_id);
        goto _adk_eth_intf_get_vlan_exit;
    }

    if(interface_id < ADK_ETH_NUM_BRIDGE)
    {
        *vlan_id = ADK_HOS_ANY_VLAN;
        goto _adk_eth_intf_get_vlan_exit;
    }

    eth_svif_if_s = eth_internal_data_p->dp_pvt_p->eth_svif_if[interface_id - ADK_ETH_NUM_BRIDGE];
    if (NULL == eth_svif_if_s)
    {
        ret = ADK_E_INVALID_PARAMETER;
        ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                          "Invalid Interface ID %d\n", interface_id);
        goto _adk_eth_intf_get_vlan_exit;
    }

    *vlan_id = eth_svif_if_s->vlan;

_adk_eth_intf_get_vlan_exit:
    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return (ret);
}

/*!     end addtogroup _ethernetFunction
 *      @}
 *  end addtogroup _ethernet
 * @}
 */
