/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet_misc.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernet  Ethernet Module
 * @{
 *      @addtogroup _ethernetFunctions  Ethernet Functions
 *      @{
 *
 * @brief   This files has all the miscellaneous APIs.
 *
 *  **********************************************************************/

#include <adk_eth.h>

adk_status_t
adk_ethernet_tables_dump (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_table_group_t table_group)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_ethernet_internal_data_t *eth_internal_data_p = NULL;
    adk_sys_handle_t sys_handle;

    /* Validate the passed handles */
    if (!adk_eth_valid_handle(eth_mod_handle))
    {
        ADK_ETH_LOG_FORCE("Invalid Ethernet module handle %"ADKPRIh "\n",
                          eth_mod_handle);
        return (ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(eth_mod_handle);

    ADK_ETH_LOG_FUNC_ENTRY(eth_mod_handle);

    ADK_ETH_ACQUIRE_MODULE_LOCK(eth_mod_handle, ret);

    eth_internal_data_p = ADK_MODULE_INFO_PTR_GET(eth_mod_handle);

    if ((table_group & ADK_ETHERNET_TABLES_ALL) != ADK_ETHERNET_TABLES_ALL)
    {
        goto _adk_ethernet_tables_dump_exit;
    }

    for (uint32_t i = 0; i < ADK_ETH_MAX_LIST_ID; i++)
    {
        ret = adk_eth_list_dump(&eth_internal_data_p->eth_list[i]);
        if (ret != ADK_SUCCESS)
        {
            ADK_ETH_LOG_ERROR(eth_mod_handle, ret,
                              "Failed to dump Ethernet list-%d!\n", i);
            goto _adk_ethernet_tables_dump_exit;
        }
    }

_adk_ethernet_tables_dump_exit:

    ADK_ETH_RELEASE_MODULE_LOCK(eth_mod_handle, ret);
_lock_release_failed:
_lock_acquire_failed:

    ADK_ETH_LOG_FUNC_EXIT(eth_mod_handle);

    return ret;
}

/*!     end addtogroup _ethernetFunction
 *      @}
 *  end addtogroup _ethernet
 * @}
 */
