/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth_dp.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernetFunctions  Ethernet Module
 *
 * @{
 *
 * @brief   This file contains the Ethernet data plane software. This include
 *          the entry functions into the Ethernet data plane module.
 *
 *  **********************************************************************/

#include <adk_eth_dp_pvt.h>
#include "adk_ph_dev_dp.h"
/* Module info pointer stored for DP process to access tables */
adk_eth_dp_pvt_data_t *adk_eth_dp_pvt_g_p = NULL;

/* Ethernet global stats pointers */
adk_dp_eth_gl_stats_t *adk_eth_gl_stats_g_p = NULL;

/* List of all Ethernet svif-interfaces */
adk_dp_eth_svif_intf_t **adk_eth_svif_if_g = NULL;

/* Pointer to the Ethernet sv-interface hash table */
adk_dp_eth_svif_if_hash_t *adk_eth_svif_if_hash_table_g_p = NULL;

/* The LPM tree for arp table */
adk_dp_eth_arp_table_t *adk_eth_arp_hash_tbl_g = NULL;

/* The arp buffer info table*/
adk_eth_dp_arp_buffer_tbl_t *adk_eth_arp_buffer_tbl_g = NULL;

/* The Neighbor Hash Table*/
adk_dp_eth_neighbor_table_t *adk_eth_neighbor_hash_table_g_p = NULL;

/* The neighbor buffer info table*/
adk_eth_dp_neighbor_buffer_tbl_t *adk_eth_neighbor_buffer_tbl_g = NULL;

/* The ipv4 nexthop timer index table*/
adk_eth_dp_nexthop_v4_timer_index_tbl_t *adk_eth_nexthop_v4_timer_index_tbl_g = NULL;

/* The ipv6 nexthop timer index table*/
adk_eth_dp_nexthop_v6_timer_index_tbl_t *adk_eth_nexthop_v6_timer_index_tbl_g = NULL;

/*The eth buffer info mempool*/
struct rte_mempool *adk_eth_buffer_info_pool_g = NULL;

 /*The eth nexthop buffer mempool*/
struct rte_mempool *adk_eth_nexthop_buffer_pool_g = NULL;

/* The LPM tree which classifies the ingress flows */
adk_lpm_handle_t adk_eth_ingress_flow_tbl_g = 0;

/* Pointer to the list of ingress flows */
adk_dp_eth_ingress_flow_t **adk_eth_ingress_flow_list_g = 0;

/* List of all Ethernet bridge interfaces */
adk_dp_eth_bridge_intf_t **adk_dp_eth_bridge_intf_t_g = {NULL};

/* Mapping of bridge L2 domain to stats ID */
uint32_t* adk_dp_eth_bridge_map_id_g = NULL;

/* Supported EtherType hos ruld IDs*/
adk_hos_rule_id_t* adk_eth_ether_type_g = NULL;

/* default l2 domain */
uint32_t* adk_dp_eth_default_l2_domain_g = NULL;

/*Auto Connection status*/
uint8_t* adk_dp_eth_auto_conn_status_g = NULL;

/* Macro definition for module level cycle count */
uint64_t adk_eth_uli_rx_ts_g, adk_eth_uli_tx_ts_g;
uint64_t adk_eth_lli_rx_ts_g, adk_eth_lli_tx_ts_g;

struct rte_mempool *eth_mempool_p;

/*The eth type to pcp map table*/
adk_lpm_handle_t adk_eth_ether_type_pcp_map_tbl_g = NULL;

/*The etherType to Queue Map hash table*/
adk_dp_eth_ethertype_queue_map_hash_t *adk_eth_ether_type_queue_map_tbl_g = NULL;

/*The ethernet non bridge origin queue ID*/
uint16_t *adk_eth_non_bridge_origin_queue_id_g = NULL;

/*The ethernet bridge timestamp quad group*/
adk_ethernet_quad_group_t *adk_eth_bridge_timestamp_quad_group_g = NULL;

/*The ethernet bridge timestamp quad group port*/
uint8_t *adk_eth_bridge_timestamp_quad_group_port_g = NULL;

/*The ethernet DSCP to PCP Map*/
uint8_t *adk_eth_dscp_to_pcp_map_g = NULL;

static adk_status_t adk_eth_add_ethertype_hos_rule
(
uint32_t ether_type,
adk_hos_rule_id_t *hos_rule_id
) 
{
    adk_status_t ret = ADK_SUCCESS;
    adk_hos_ethertype_tbl_rule_attr_t hos_ethertype_rule;
    hos_ethertype_rule.ether_type = ether_type; 

    ret = adk_hos_rule_add(ADK_HOS_SUPPORTED_ETHERTYPE_TBL,
                                (adk_hos_rule_attr_t *) &hos_ethertype_rule,
                                hos_rule_id); 
    if (ret != ADK_SUCCESS)
    {
        ADK_DP_LOG_ERROR(eth_mod_handle, 
                          "Failed to add supported ethertype HOS rule\n");
        return ret;
    }

    return ADK_SUCCESS;
}

/* Function to initialize the Ethernet data plane. This function assumes only
 * one caller and hence, does not take any locks to prevent protection from
 * parallel invocation. This function must be called before spawning the worker
 * threads for packet processing. */
adk_dp_status_t
adk_ethernet_dp_init (
    void)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;
    adk_eth_dp_pvt_data_t * dp_pvt_p;
    uint32_t iBridge;

    /* Lookup the shared memory that gets created by control plane Ethernet
     * module initialization */
    mz = rte_memzone_lookup(ADK_ETHERNET_SHM_MEMZONE_NAME);
    if (mz == NULL)
    {
        /* This indicates incorrect initialization sequence error where DP
         * is being intialized before CP initialization */
        ADK_DP_LOG_ERROR_1S(ADK_ETHERNET_MODULE_ID, "Error: Failed to lookup"
                            " Ethernet shared memory %s",
                            ADK_ETHERNET_SHM_MEMZONE_NAME);
        ret = ADK_E_MEM_UNAVAILABLE;
        goto _adk_ethernet_dp_init_exit;
    }

    /* Check if the Ethernet module has been completely initialized */
    dp_pvt_p = (adk_eth_dp_pvt_data_t *) mz->addr;
    if (dp_pvt_p->cookie != ADK_ETHERNET_MODULE_COOKIE)
    {
        /* This indicates race condition between the CP and DP app. If this
         * error does not go away soon, then this indicates some initialization
         * sequence error */
        ADK_DP_LOG_INFO(ADK_ETHERNET_MODULE_ID, "Ethernet Module Initialization"
                        " from control plane not yet complete. Please wait and"
                        " try again later.");
        ret = ADK_E_MEM_UNAVAILABLE;
        goto _adk_ethernet_dp_init_exit;
    }

    /* Attach to the Ethernet shared memory */
    adk_eth_dp_pvt_g_p = dp_pvt_p;

    /* Initialize the LLI/ULI function pointers used in the data plane
     * processing during bind */
    adkDestEntryFunction_g[ADK_ETH_LLI_FN_ID] = adk_eth_dp_process_pkt;
    adkDestEntryFunction_g[ADK_ETH_LLI_RING_FN_ID] = adk_eth_dp_process_pkt_ring;
    adkDestEntryFunction_g[ADK_ETH_ULI_FN_ID] = adk_eth_dp_pkt_proc_uli;
    adkDestEntryFunction_g[ADK_ETH_ULI_RING_FN_ID] = adk_eth_dp_pkt_proc_ring_uli;
    adkDestEntryFunction_g[ADK_ETH_ULI_NETD_FN_ID] = adk_eth_dp_pkt_proc_netd_v2;
#ifdef ADK_DEVICE_SNR
    adkDestEntryFunction_g[ADK_ETH_ULI_NETD_BRIDGE_FN_ID] = adk_eth_dp_pkt_proc_netd_bridge;
    adkDestEntryFunction_g[ADK_ETH_TS_PKT_TX_FN_ID] = adk_eth_dp_ts_pkt_transmitted;
#endif
    adkDestEntryFunction_g[ADK_ETH_ULI_PKT_BUFFER_FN_ID] = adk_eth_dp_buffer_packet;
    adkDestEntryFunction_g[ADK_ETH_POLICING_FN_ID] = adk_eth_dp_pkt_proc_police;

    /* Initialize all ethernet global variables used in the data plane packet
     * processing */
    adk_eth_arp_hash_tbl_g = &adk_eth_dp_pvt_g_p->eth_arp_hash_table;
    adk_eth_arp_buffer_tbl_g = adk_eth_dp_pvt_g_p->eth_arp_buffer_tbl;
    adk_eth_neighbor_hash_table_g_p = &adk_eth_dp_pvt_g_p->eth_neighbor_hash_table;
    adk_eth_neighbor_buffer_tbl_g = adk_eth_dp_pvt_g_p->eth_neighbor_buffer_tbl;
    adk_eth_nexthop_v4_timer_index_tbl_g = adk_eth_dp_pvt_g_p->eth_nexthop_v4_timer_index_tbl;
    adk_eth_nexthop_v6_timer_index_tbl_g = adk_eth_dp_pvt_g_p->eth_nexthop_v6_timer_index_tbl;
    adk_eth_buffer_info_pool_g = adk_eth_dp_pvt_g_p->eth_buffer_info_pool;
    adk_eth_nexthop_buffer_pool_g = adk_eth_dp_pvt_g_p->eth_nexthop_buffer_pool;
    adk_eth_ingress_flow_tbl_g = adk_eth_dp_pvt_g_p->ingressFlowTbl;
    adk_eth_ingress_flow_list_g = adk_eth_dp_pvt_g_p->eth_ingress_flow_list;
    adk_dp_eth_bridge_intf_t_g = adk_eth_dp_pvt_g_p->bridge_intf;
    adk_eth_svif_if_g = adk_eth_dp_pvt_g_p->eth_svif_if;
    adk_eth_svif_if_hash_table_g_p = adk_eth_dp_pvt_g_p->eth_svif_if_hash_table;
    adk_eth_gl_stats_g_p = adk_eth_dp_pvt_g_p->gl_stats_p;
    adk_dp_eth_bridge_map_id_g = adk_eth_dp_pvt_g_p->bridge_map_id;
    adk_dp_eth_default_l2_domain_g = adk_eth_dp_pvt_g_p->default_l2_domain;
    adk_dp_eth_auto_conn_status_g = adk_eth_dp_pvt_g_p->autoconn_status;
    adk_eth_ether_type_g = adk_eth_dp_pvt_g_p->adk_eth_ether_type;
    adk_eth_ether_type_pcp_map_tbl_g = adk_eth_dp_pvt_g_p->ethTypePcpMapTbl;
    adk_eth_ether_type_queue_map_tbl_g = adk_eth_dp_pvt_g_p->eth_ethertype_queue_map_hash_table;
    adk_eth_non_bridge_origin_queue_id_g = adk_eth_dp_pvt_g_p->eth_non_bridge_origin_queue_id;
    adk_eth_bridge_timestamp_quad_group_g = adk_eth_dp_pvt_g_p->eth_bridge_timestamp_quad_group;
    adk_eth_bridge_timestamp_quad_group_port_g = adk_eth_dp_pvt_g_p->eth_bridge_timestamp_quad_group_port;
    adk_eth_dscp_to_pcp_map_g = adk_eth_dp_pvt_g_p->eth_dscp_to_pcp_map;
    /* Initialize all bridge ID to stats ID */
    for(iBridge = 0; iBridge < ADK_ETH_MAX_BG_ID; iBridge++)
        adk_eth_dp_pvt_g_p->bridge_map_id[iBridge] = ADK_ETH_MAX_BG_ID;


    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_IPV4,
                                    &adk_eth_ether_type_g[0]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_ARP,
                                    &adk_eth_ether_type_g[1]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_VLAN,
                                    &adk_eth_ether_type_g[2]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_IPv6,
                                    &adk_eth_ether_type_g[3]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_LACP,
                                    &adk_eth_ether_type_g[4]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_CFM,
                                    &adk_eth_ether_type_g[5]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_PTP,
                                    &adk_eth_ether_type_g[6]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_NSN_TEST_FRAME,
                                    &adk_eth_ether_type_g[7]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_EAPOL,
                                    &adk_eth_ether_type_g[8]);

    adk_eth_add_ethertype_hos_rule(ADK_ETHERNET_ETHERTYPE_LLDP,
                                    &adk_eth_ether_type_g[9]);

    adk_sys_config_pktio_mempool_get((void **) &eth_mempool_p);

    /* Initialize time stamp index manager */
#ifdef ADK_DEVICE_SNR
    adk_netd_fp_ts_init(-1);
#endif

_adk_ethernet_dp_init_exit:

    return ret;
}

adk_status_t
adk_eth_dp_handle_exception(
        adk_pktio_task_t *ph_task,
        uint8_t type,
        void *exception_data)
{
    adk_ph_dev_dp_metadata_t *phMeta_p;

    adk_ethernet_dp_exception_handler_t * exceptionHdl
        = adk_eth_dp_pvt_g_p->exceptionHandlerTble[type - 1];

    if(!exceptionHdl)
    {
        ADK_DP_LOG_ERROR_1D(ADK_ETHERNET_MODULE_ID,"No ETH Exception handler"
                "configured for Exception %d ",type);
        return ADK_E_DROP_TASK;
    }

    ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Eth Exception handler action STH via PH!!\n");

    /* Update PH metadata */
    phMeta_p = (adk_ph_dev_dp_metadata_t *)
        &(ADK_METADATA_PARAM(ph_task,imc_param_start));
    phMeta_p->policerFlowId = exceptionHdl->policerId;

    ADK_MD_SET_ADK_DEV_ID(ph_task, exceptionHdl->devId);
    ADK_MD_SET_DEV_TX_QUEUE_ID(ph_task, exceptionHdl->queueId);
    phMeta_p->priority = exceptionHdl->priority;
    phMeta_p->cbFnId = exceptionHdl->cbFnId;
    phMeta_p->moduleId = ADK_ETHERNET_MODULE_ID;

    eth_ph_metadata_t *ph_metadata_p;
    ph_metadata_p = (eth_ph_metadata_t *)&phMeta_p->imc_param_start;
    ADK_METADATA_PARAM(ph_task, pdu_flow_id) = 0;

    switch (type)
    {
        case ADK_ETHERNET_EXCEPTION_TYPE_UNRESOLVED_ARP:
            ph_metadata_p->exception_type = type;

            memcpy(&ph_metadata_p->exception_dp_data.arp_exception_data,
                    (adk_ethernet_arp_dp_exception_data_t*)exception_data,
                    sizeof(adk_ethernet_arp_dp_exception_data_t));

            debug_print("DP exception parameters:  ifHandle-%p,next_hop_address-%x\n",
                    ph_metadata_p->exception_dp_data.arp_exception_data.ifHandle,
                    ph_metadata_p->exception_dp_data.arp_exception_data.next_hop_address);

            break;
        case ADK_ETHERNET_EXCEPTION_TYPE_UNRESOLVED_IPV6_NEIGHBOR:
            ph_metadata_p->exception_type = type;

            memcpy(&ph_metadata_p->exception_dp_data.neigh_exception_data,
                    (adk_ethernet_neigh_dp_exception_data_t*)exception_data,
                    sizeof(adk_ethernet_neigh_dp_exception_data_t));

            debug_print("DP exception parameters:  ifHandle-%p,next_hop_address-%x:%x:%x:%x\n",
                    ph_metadata_p->exception_dp_data.neigh_exception_data.ifHandle,
                    ph_metadata_p->exception_dp_data.neigh_exception_data.v6_neigh_address.addr[0],
                    ph_metadata_p->exception_dp_data.neigh_exception_data.v6_neigh_address.addr[1],
                    ph_metadata_p->exception_dp_data.neigh_exception_data.v6_neigh_address.addr[2],
                    ph_metadata_p->exception_dp_data.neigh_exception_data.v6_neigh_address.addr[3]);
            break;
        case ADK_ETHERNET_EXCEPTION_TYPE_NEIGHBOR_STALE_ENTRY_USED:
            ph_metadata_p->exception_type = type;

            memcpy(&ph_metadata_p->exception_dp_data.neigh_stale_exception_data,
                    (adk_ethernet_neigh_dp_stale_entry_exception_data_t*)exception_data,
                    sizeof(adk_ethernet_neigh_dp_stale_entry_exception_data_t));

            debug_print("DP exception parameters:  ifHandle-%p\n",
                    ph_metadata_p->exception_dp_data.neigh_stale_exception_data.ifHandle);

            debug_print("DP exception parameters:  nexthop address ");
            if(!ph_metadata_p->exception_dp_data.neigh_stale_exception_data.isIPv6)
            {
                debug_print("%x\n",ph_metadata_p->exception_dp_data.neigh_stale_exception_data.next_hop.v4_address);
            }
            else
            {
                debug_print("%x:%x:%x:%x\n",
                    ph_metadata_p->exception_dp_data.neigh_stale_exception_data.next_hop.v6_address.addr[0],
                    ph_metadata_p->exception_dp_data.neigh_stale_exception_data.next_hop.v6_address.addr[1],
                    ph_metadata_p->exception_dp_data.neigh_stale_exception_data.next_hop.v6_address.addr[2],
                    ph_metadata_p->exception_dp_data.neigh_stale_exception_data.next_hop.v6_address.addr[3]);
            }
            break;
        default:
            return ADK_E_DROP_TASK;

    }
    if(ADK_SUCCESS != adk_ph_dev_dp_to_cp_send((void *)ph_task, ADK_PAL_DP_BATCH_CB_FN_ETH, exceptionHdl))
    {
        return ADK_E_DROP_TASK;
    }
    return ADK_E_TASK_CONSUMED;
}

adk_dp_status_t
adk_eth_dp_arp_buffer_add
(
adk_eth_dp_arp_buffer_tbl_t* arp_buf_tbl,
uint32_t nh_ipAddr,
adk_dp_eth_buffer_info_t* buffer_info
)
{
    uint32_t hash_val, hash_index, i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Arp Buffer Tbl: Inserting entry with"
            " next hop 0x%x and buffer info %p...\n",
            nh_ipAddr, buffer_info);

    hash_val = ADK_ETHERNET_HASH(0, nh_ipAddr);

    hash_index = hash_val & ADK_ETH_ARP_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION;

    /*Add with same input should overwrite. Hence first find if entry is present*/
    for(i=0; i< ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION; i++)
    {
        /* nh_ipAddr is overloaded to denote present flag
         * (nh_ipAddr == 0) => not present*/
        if(arp_buf_tbl[hash_index + i].nh_ipAddr)
        {
            if(arp_buf_tbl[hash_index + i].nh_ipAddr == nh_ipAddr)
            {
                /*Exact match found. Atomically overwrite buffer info*/
                arp_buf_tbl[hash_index + i].buffer_info = buffer_info;
                buffer_info->hash_index = hash_index + i;
                goto _adk_eth_dp_arp_buffer_add_done;
            }
        }
    }

    /*If here means entry not present & add to first free position*/
    for(i=0; i< ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION; i++)
    {
        if(!(arp_buf_tbl[hash_index + i].nh_ipAddr))
        {
            /*This entry is free, update the data*/
            arp_buf_tbl[hash_index + i].buffer_info = buffer_info;
            arp_buf_tbl[hash_index + i].nh_ipAddr = nh_ipAddr;
            buffer_info->hash_index = hash_index + i;
            goto _adk_eth_dp_arp_buffer_add_done;
        }
    }

    /*If here means no free slot in collision list.
    There will not be any overflow list. Like Gen 2,
    There would be a limit on the number of next hops
    for which there would be parallel arp buffer.
    This will also protect the system from overflowing the memory */
      
    return ADK_DP_E_ERROR;

_adk_eth_dp_arp_buffer_add_done:
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_eth_dp_arp_buffer_rem
(
adk_eth_dp_arp_buffer_tbl_t* arp_buf_tbl,
uint32_t nh_ipAddr
)
{
    uint32_t hash_val,hash_index,i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Arp Buffer Tbl: Removing next hop 0x%x...\n",
           nh_ipAddr);

    hash_val = ADK_ETHERNET_HASH(0, nh_ipAddr);

    hash_index = hash_val & ADK_ETH_ARP_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION;

    /*Find if entry is present*/
    for(i=0; i< ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION; i++)
    {
        /* nh_ipAddr is overloaded to denote present flag
         * (nh_ipAddr == 0) => not present*/
        if(arp_buf_tbl[hash_index + i].nh_ipAddr)
        {
            if(arp_buf_tbl[hash_index + i].nh_ipAddr == nh_ipAddr)
            {
                /*Exact match found. Reset DIP to denote entry is free*/
                arp_buf_tbl[hash_index + i].nh_ipAddr = 0;
                return ADK_SUCCESS;
            }
        }
    }

    /*If here means entry not present, hence returen error*/
    return ADK_DP_E_ERROR;
}

adk_dp_status_t
adk_eth_dp_arp_buffer_get(
adk_eth_dp_arp_buffer_tbl_t* arp_buf_tbl,
uint32_t nh_ipAddr,
adk_dp_eth_buffer_info_t** arp_buffer_p)
{
    uint32_t hash_val,hash_index,i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Arp buffer tbl: Searching for nexthop 0x%x ...\n",
        nh_ipAddr);

    hash_val = ADK_ETHERNET_HASH(0, nh_ipAddr);

    hash_index = hash_val & ADK_ETH_ARP_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION;

    for(i=0; i< ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION; i++)
    {
        /* nh_ipAddr is overloaded to denote present flag
         * (nh_ipAddr == 0) => not present*/
        if(arp_buf_tbl[hash_index + i].nh_ipAddr)
        {
            if(arp_buf_tbl[hash_index + i].nh_ipAddr == nh_ipAddr)
            {
                /*Exact match found. return arp buffer pointer*/
                *arp_buffer_p = arp_buf_tbl[hash_index + i].buffer_info;
                return ADK_SUCCESS;
            }
        }
    }

    /*If here means entry not present*/
    return ADK_DP_E_ERROR;

}

static inline uint32_t eth_dp_ipv6_hash
(
uint8_t* nh_ipv6Addr
)
{
    uint32_t hash_val;

    hash_val = ADK_ETHERNET_HASH(0, ((uint32_t*)nh_ipv6Addr)[0]);
    hash_val = ADK_ETHERNET_HASH(hash_val, ((uint32_t*)nh_ipv6Addr)[1]);
    hash_val = ADK_ETHERNET_HASH(hash_val, ((uint32_t*)nh_ipv6Addr)[2]);
    hash_val = ADK_ETHERNET_HASH(hash_val, ((uint32_t*)nh_ipv6Addr)[3]);
    return hash_val;
}

static inline bool eth_dp_neigh_match
(
adk_eth_dp_neighbor_buffer_tbl_t* neigh_buf_tbl,
uint64_t* nh_ipv6Addr,
uint32_t hash_val,
uint32_t index
)
{
    if(!(neigh_buf_tbl[index].enable))
        return false;

    if(neigh_buf_tbl[index].hash_value != hash_val)
        return false;

    if((*nh_ipv6Addr !=
            *((uint64_t *)&(neigh_buf_tbl[index].nh_ipv6Addr.addr[0]))))
        return false;

    if(nh_ipv6Addr[1] !=
            *((uint64_t *)&(neigh_buf_tbl[index].nh_ipv6Addr.addr[2])))
        return false;

    return true;
}

adk_dp_status_t
adk_eth_dp_neighbor_buffer_add
(
adk_eth_dp_neighbor_buffer_tbl_t* neigh_buf_tbl,
uint8_t* nh_ipv6Addr,
adk_dp_eth_buffer_info_t* buffer_info
)
{
    uint32_t hash_val, hash_index, i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Neighbor Buffer Tbl: Inserting entry with"
            " next hop 0x%x 0x%x 0x%x 0x%x and buffer info %p...\n",
            ((uint32_t*)nh_ipv6Addr)[0], ((uint32_t*)nh_ipv6Addr)[1],
            ((uint32_t*)nh_ipv6Addr)[2], ((uint32_t*)nh_ipv6Addr)[3], buffer_info);

    hash_val = eth_dp_ipv6_hash(nh_ipv6Addr);

    hash_index = hash_val & ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION;

    /*Add with same input should overwrite. Hence first find if entry is present*/
    for(i=0; i< ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION; i++)
    {
        if(!(neigh_buf_tbl[hash_index + i].enable))
            continue;

        if(!eth_dp_neigh_match(neigh_buf_tbl, (uint64_t*)&nh_ipv6Addr[0],
            hash_val, hash_index + i))
            continue;
            
        neigh_buf_tbl[hash_index + i].buffer_info = buffer_info;
        buffer_info->hash_index = hash_index + i;
        goto _adk_eth_dp_neigh_buffer_add_done;
    }

    /*If here means entry not present & add to first free position*/
    for(i=0; i< ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION; i++)
    {
        if(neigh_buf_tbl[hash_index + i].enable)
            continue;

        /*This entry is free, update the data*/
        memcpy((uint8_t*)(&neigh_buf_tbl[hash_index + i].nh_ipv6Addr), nh_ipv6Addr,
            sizeof(adk_ethernet_ipv6_addr_t));
        neigh_buf_tbl[hash_index + i].buffer_info = buffer_info;
        neigh_buf_tbl[hash_index + i].hash_value = hash_val;
        neigh_buf_tbl[hash_index + i].enable = true;
        buffer_info->hash_index = hash_index + i;
        goto _adk_eth_dp_neigh_buffer_add_done;
    }

    /*If here means no free slot in collision list.
    There will not be any overflow list. Like Gen 2,
    There would be a limit on the number of next hops
    for which there would be parallel arp buffer.
    This will also protect the system from overflowing the memory */
      
    return ADK_DP_E_ERROR;

_adk_eth_dp_neigh_buffer_add_done:
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_eth_dp_neighbor_buffer_rem
(
adk_eth_dp_neighbor_buffer_tbl_t* neigh_buf_tbl,
uint8_t* nh_ipv6Addr
)
{
    uint32_t hash_val,hash_index,i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Neighbor Buffer Tbl: Removing entry with"
            " next hop 0x%x 0x%x 0x%x 0x%x...\n",
            ((uint32_t*)nh_ipv6Addr)[0], ((uint32_t*)nh_ipv6Addr)[1],
            ((uint32_t*)nh_ipv6Addr)[2], ((uint32_t*)nh_ipv6Addr)[3]);

    hash_val = eth_dp_ipv6_hash(nh_ipv6Addr);

    hash_index = hash_val & ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION;

    /*Find if entry is present*/
    for(i=0; i< ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION; i++)
    {
        if(!(neigh_buf_tbl[hash_index + i].enable))
            continue;

        if(!eth_dp_neigh_match(neigh_buf_tbl, (uint64_t*)&nh_ipv6Addr[0],
            hash_val, hash_index + i))
            continue;

        /*Exact match found. Reset DIP to denote entry is free*/
        neigh_buf_tbl[hash_index + i].enable = false;
        return ADK_SUCCESS;
    }

    /*If here means entry not present, hence returen error*/
    return ADK_DP_E_ERROR;
}

adk_dp_status_t
adk_eth_dp_neighbor_buffer_get(
adk_eth_dp_neighbor_buffer_tbl_t* neigh_buf_tbl,
uint8_t* nh_ipv6Addr,
adk_dp_eth_buffer_info_t** buffer_info_p)
{
    uint32_t hash_val,hash_index,i;

    ADK_ETH_DP_LOG_INFO_DEBUG("Neighbor Buffer Tbl: Searching entry with"
            " next hop 0x%x 0x%x 0x%x 0x%x...\n",
            ((uint32_t*)nh_ipv6Addr)[0], ((uint32_t*)nh_ipv6Addr)[1],
            ((uint32_t*)nh_ipv6Addr)[2], ((uint32_t*)nh_ipv6Addr)[3]);

    hash_val = eth_dp_ipv6_hash(nh_ipv6Addr);

    hash_index = hash_val & ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MASK;
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION;

    for(i=0; i< ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION; i++)
    {
        if(!(neigh_buf_tbl[hash_index + i].enable))
            continue;

        if(!eth_dp_neigh_match(neigh_buf_tbl, (uint64_t*)&nh_ipv6Addr[0],
            hash_val, hash_index + i))
            continue;

        /*Exact match found. return neigh buffer pointer*/
        *buffer_info_p = neigh_buf_tbl[hash_index + i].buffer_info;
        return ADK_SUCCESS;
    }

    /*If here means entry not present*/
    return ADK_DP_E_ERROR;

}

void
adk_eth_dp_egress_stats(
struct rte_mbuf *task,
void *data)
{
    adk_metadata_t * adk_md_p = ADK_METADATA_PTR(task);
    adk_sys_common_egress_stats_mdata_t *egress_stats = &adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_ETHERNET_INTERFACE_STATS];
    adk_dp_eth_svif_intf_t *dp_svif_intf_p = NULL;
    adk_dp_eth_bridge_intf_t  *dp_bridge_intf_p = NULL;
    adk_dp_eth_stats_t *dp_svif_intf_stats_p = NULL;
    adk_dp_eth_stats_t *dp_bridge_intf_stats_p = NULL;
    int coreIndex = ADK_LCORE_INDEX;
    adk_egress_user_param_t *params = NULL;

    params = (adk_egress_user_param_t *)data;

    if ((TRUE != params->xmit_cb) &&
        (params->stats_mask == ADK_STATS_ETHERNET_INTERFACE_MASK))
    {
        if(rte_pktmbuf_tx_enqueue_status(task) == RTE_MBUF_TX_ENQUEUE_SUCCESS)
        {
            if(egress_stats->stats_id < ADK_ETH_NUM_BRIDGE)
            {
                dp_bridge_intf_p = adk_dp_eth_bridge_intf_t_g[egress_stats->stats_id];
                dp_bridge_intf_stats_p = &(dp_bridge_intf_p->intf_stats[coreIndex]);
                adk_eth_dp_update_tx_stats(dp_bridge_intf_stats_p, task, adk_md_p);
            }
            else
            {
                dp_svif_intf_p = adk_eth_svif_if_g[egress_stats->stats_id - ADK_ETH_NUM_BRIDGE];
                dp_svif_intf_stats_p = &(dp_svif_intf_p->intf_stats[coreIndex]);
                adk_eth_dp_update_tx_stats(dp_svif_intf_stats_p, task, adk_md_p);
            }
        }
    }
    return;
}

void adk_eth_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess)
{
    /*Current worker core id */
    uint32_t coreIndex = ADK_LCORE_INDEX;
    debug_print("adk_eth_dp_pal_batch_cb_fn %d\n",cbEventType);
    switch(cbEventType)
    {
        case ADK_PAL_DP_BATCH_CB_EVENT_NIC:
            if(sendSuccess)
            {
                debug_print("ADK_PAL_DP_BATCH_CB_EVENT_NIC in adk_eth_dp_pal_batch_cb_fn\n");
                return;
            }
            else
            {
                adk_eth_gl_stats_g_p[coreIndex].nic_send_failure_drop++;
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
            }    
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_PH:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_PH_POLICER:
            if(cbParam != NULL)
            {
                debug_print("ADK_PAL_DP_BATCH_CB_EVENT_PH POLICER in adk_eth_dp_pal_batch_cb_fn\n");
                adk_ethernet_dp_exception_handler_t * exceptionHdl = cbParam;
                adk_ph_dev_pal_batch_handle_return(exceptionHdl->devId, exceptionHdl->queueId,
                        sendSuccess, cbEventType);
            }
            if(!sendSuccess)
            {
                adk_eth_gl_stats_g_p[coreIndex].exceptionDrops++;
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_NETD:
            if(sendSuccess)
            {
                debug_print("ADK_PAL_DP_BATCH_CB_EVENT_NETD in adk_eth_dp_pal_batch_cb_fn\n");
                return;
            }
            else
            {
                if(cbParam != NULL)
                {
                    adk_dp_eth_ingress_flow_stats_t* dp_ingress_flow_stats_p = cbParam;
                    dp_ingress_flow_stats_p->pkts_drop++;
                }
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER:
            if(sendSuccess)
            {
                debug_print("ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER in adk_eth_dp_pal_batch_cb_fn\n");
                return;
            }
            else
            {
                uint8_t pkt_drops = adk_eth_task_chained_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                adk_eth_gl_stats_g_p[coreIndex].failed_to_sent_next_stage_drop+=pkt_drops;
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_TIMER:
        case ADK_PAL_DP_BATCH_CB_EVENT_EM:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER:
        case ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO:
            return;
            break;
        default:
            break;
    }
    return;
}
 /*!  end addtogroup _ethernetFunctions  Ethernet Module
 * @}
 */
