/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth_dp_lli.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ethernetFunctions  Ethernet Module
 *
 * @{
 *
 * @brief   This file contains the Ethernet data plane software to handle
 *          packets received from external ports.
 *
 *  **********************************************************************/

#include <adk_eth_dp_pvt.h>
#include "adk_qos_include_internal.h"

/* Module info pointer stored for DP process to access tables */
extern adk_eth_dp_pvt_data_t *adk_eth_dp_pvt_g_p;

/* Ethernet global stats pointers */
extern adk_dp_eth_gl_stats_t *adk_eth_gl_stats_g_p;

/* List of all Ethernet svif-interfaces */
extern adk_dp_eth_svif_intf_t **adk_eth_svif_if_g;

/* Pointer to the list of Ethernet sv-interface table */
extern adk_dp_eth_svif_if_hash_t *adk_eth_svif_if_hash_table_g_p;

/* The LPM tree which classifies the ingress flows */
extern adk_lpm_handle_t adk_eth_ingress_flow_tbl_g;

/* Pointer to the list of ingress flows */
extern adk_dp_eth_ingress_flow_t **adk_eth_ingress_flow_list_g;

/* List of all Ethernet Egress VLAN Info */
extern adk_dp_eth_bridge_intf_t **adk_dp_eth_bridge_intf_t_g;

/* Map of bridge l2 domain to interface ID */
extern uint32_t* adk_dp_eth_bridge_map_id_g;

/* Default L2 domain for L2 call */
extern uint32_t* adk_dp_eth_default_l2_domain_g;

/*Auto connection status*/
extern uint8_t* adk_dp_eth_auto_conn_status_g;

/* Macro definition for module level cycle count */
extern uint64_t adk_eth_uli_rx_ts_g, adk_eth_uli_tx_ts_g;
extern uint64_t adk_eth_lli_rx_ts_g, adk_eth_lli_tx_ts_g;

extern struct rte_mempool *eth_mempool_p;

#ifdef ADK_DP_NO_BATCH_PROC

/*!
 * =============================================================
 * adk_eth_dp_process_pkt
 *
 * Ethernet entry function for packets received on external
 * network interface.
 *
 * =============================================================
 */

adk_status_t
adk_eth_dp_process_pkt (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    __attribute__ ((unused)) void *ring,
    __attribute__ ((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    uint16_t  fn_id = 0;

    ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_RX_TS();

    INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                            ingressUtilityCounters.eth_ing_pkt_proc_lli_entry_count);

    {
        ret = adk_eth_dp_process_pkt_single(task_p[0], coreIndex, &fn_id);
        if (unlikely(ADK_E_INTERNAL_ERROR == ret))
        {
            INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                                ingressUtilityCounters.eth_ing_pkt_proc_lli_drop);
            /* Free the task */
            adk_pal_dp_release_send((task_p[0]), ADK_PAL_DP_BATCH_CB_FN_ETH, NULL);
            adk_pktio_task_buffer_free((adk_pktio_task_t *)(task_p[0]));
            return ret;
        }
        else if (ADK_E_TASK_CONSUMED == ret)
        {
            return ret;
        }
    }

    ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_TX_TS();
    if (adkDestEntryFunction_g[fn_id])
    {
#if defined (ADK_PAL_DYNAMIC_PPL)
        ret = adk_pal_dp_stage_send((void **)task_p, task_count, task_mask, fn_id, 0x00,
                0x00, ADK_PAL_DP_BATCH_CB_FN_ETH, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
        if(ret == ADK_E_INTERNAL_ERROR)
        {
            adk_eth_gl_stats_g_p[coreIndex].failed_to_sent_next_stage_drop++;
            adk_pal_dp_release_send((task_p[0]), ADK_PAL_DP_BATCH_CB_FN_ETH, NULL);
            adk_pktio_task_buffer_free((adk_pktio_task_t *)(task_p[0]));
            ret = ADK_DP_E_ERROR;
            return ret;
        }
#else
        (adkDestEntryFunction_g[destFunc])(task_p, task_count, task_mask, ring_hdl,
                                           next_stage_entry_hdl, coreIndex);
#endif
    }

    INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                                ingressUtilityCounters.eth_ing_pkts_sent_to_l3);

    return ret;
}
#endif

adk_status_t
adk_eth_dp_process_pkt_single (
    void *task_p,
    int coreIndex,
    uint16_t *fn_id)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t *task = (adk_pktio_task_t *)task_p;
    adk_dp_eth_svif_intf_t *dp_svif_if_p = NULL;
    adk_dp_eth_bridge_intf_t *dp_bridge_if_p = NULL;
    uint32_t bridge_stats_id = 0;
    adk_dp_eth_stats_t *dp_eth_intf_stats_p = NULL;
    uint8_t *pduData_p;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    adk_ip_dp_lli_imc_data_t *ip_imc_p;
    adk_dp_eth_ingress_flow_t *dp_flow_p = NULL;
    adk_dsi_rx_metadata_t *dsi_md_p = NULL;
    adk_dp_eth_ingress_flow_stats_t* dp_ingress_flow_stats_p = NULL;
    bool non_l2_terminating = false;
    bool strip_vlan = false;

    if (unlikely((adk_eth_dp_pvt_g_p == NULL) ||
                 (adk_eth_dp_pvt_g_p->cookie != ADK_ETHERNET_MODULE_COOKIE)))
    {
        /* This error is expected when packets are received before Ethernet
         * module has been initialized. */
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Error (LLI): Ethernet module"
                        " uninitialized! Dropping pkt!");
        return ADK_E_INTERNAL_ERROR;
    }

    adk_eth_gl_stats_g_p[coreIndex].proc_cntr++;

    ADK_PDU_PTR_GET(task, pduData_p, uint8_t *);


    ADK_DP_LOG_INFO_1D_DEBUG(ADK_ETHERNET_MODULE_ID, "Received packet on port %d\n", adk_md_p->input_intf_id);

#ifdef ADK_DEBUG
    adk_dp_eth_hdr_t eth_hdr;
    /* parse ethernet header and extract fields */
    adk_eth_dp_parse_hdr(pduData_p, &eth_hdr);
    debug_print("Received packet with following ethernet header:\n");
    adk_eth_dp_print_hdr(&eth_hdr);
#endif

    if (unlikely(adk_md_p->input_intf_id >= ADK_ETH_PORTS_MAX))
    {
        ADK_DP_LOG_ERROR_1D(ADK_ETHERNET_MODULE_ID, "Error (LLI): Received pkt"
                            " on invalid port %d! Dropping pkt!", adk_md_p->input_intf_id);
        adk_eth_gl_stats_g_p[coreIndex].port_not_defined++;
        return ADK_E_INTERNAL_ERROR;
    }

    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task_p);

#ifdef ADK_DEBUG
    debug_print("RSS Hash: %d\n", task->hash.rss);
    debug_print("Port: %d\n", ADK_PACKET_INPUT_PORT(task));
    debug_print("Device: %d\n", ADK_MBUF_METADATA_PARAM(task, port));
    {
        debug_print("Pkt-Type: %d\n", dsi_md_p->packet_type);
        debug_print("\t%s\n", adk_hos_is_l2_term(dsi_md_p) ?
                    "L2-Terminating" : "Switching-Pkt");
        if (adk_hos_is_l3_term(dsi_md_p))
        {
            debug_print("\tL3-Terminating\n");
        }
        else
        {
            debug_print("\tRouting-Pkt\n");
        }
        if(adk_hos_is_ip_mcast(dsi_md_p))
            debug_print("\tIP Multicast-pkt\n");
        if(adk_hos_is_ip_bcast(dsi_md_p))
            debug_print("\tIP Broadcast-pkt\n");
        if(adk_hos_is_ip_stats_invalid(dsi_md_p))
        {
            debug_print("\tIP stats ID invalid\n");
        }
        else
        {
            debug_print("\tIP stats ID valid\n");
            debug_print("\tIP stats ID %d\n", adk_hos_get_ip_stats_id(dsi_md_p));
        }
        
        debug_print("L2-Domain: %d\n", dsi_md_p->l2_domain);
        debug_print("L3-Domain: %d\n", dsi_md_p->l3_domain);
        debug_print("Outer ID: %d\n", dsi_md_p->outer_id);
        debug_print("Inner ID: %d\n", dsi_md_p->inner_id);
        debug_print("Flags: 0x%03x\n", dsi_md_p->flags);
        debug_print("Policer RED? : %s\n", adk_hos_is_policer_red(dsi_md_p) ?
                    "Yes" : "No");
        debug_print("L4 Checksum: 0x%08x\n", dsi_md_p->checksum);
        debug_print("ACL ID: %d\n", dsi_md_p->acl_id);
        debug_print(" ++++ IPSEC related data ++++ \n");
        debug_print("SPD ID: %x\n",  dsi_md_p->spd_id);
        debug_print("ipsec_offloaded : %d\n", dsi_md_p->ipsec_offloaded);
        debug_print("ipsec_status : %d\n", dsi_md_p->ipsec_status);
        debug_print("sa_idx : %d\n", dsi_md_p->sa_idx);
        debug_print("adk_hos_is_ethertype_invalid %d\n", adk_hos_is_ethertype_invalid(dsi_md_p));
        debug_print("adk_hos_is_ipproto_invalid %d\n", adk_hos_is_ipproto_invalid(dsi_md_p));
        debug_print("adk_hos_is_l4_chksum_err %d\n", adk_hos_is_l4_chksum_err(dsi_md_p));
    }
#endif

    /*
     * Lookup sub interface table to get sub interface from incoming port &
     * vlan id field.
     */

    if (likely(adk_hos_is_l2_interface_valid(dsi_md_p)))
    {
        /* L2 Interface HLP table is small and guaranteed to be in HLP entirely*/
        dp_svif_if_p = adk_eth_svif_if_g[adk_hos_get_l2_interface(dsi_md_p)];
        if(unlikely(NULL == dp_svif_if_p)) {
            adk_eth_gl_stats_g_p[coreIndex].intf_not_defined++;
            ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "SVIF interface is not defined");
            return ADK_E_INTERNAL_ERROR;
        }

        ADK_DP_LOG_INFO_1P_DEBUG(ADK_ETHERNET_MODULE_ID, "Retrieved svif - %p\n", dp_svif_if_p);
        ADK_DP_LOG_INFO_1P_DEBUG(ADK_ETHERNET_MODULE_ID, "Packet received on svif - %p\n", dp_svif_if_p);
        dp_eth_intf_stats_p = &(dp_svif_if_p->intf_stats[coreIndex]);

        /* Fill in the ADK common metadata */
        adk_md_p->input_intf_id =  dp_svif_if_p->statsId + ADK_ETH_NUM_BRIDGE;
        task->l2_len = ETH_VLAN_HEADER_LEN;
    }
    else 
    {
        if(likely(adk_hos_is_l2_term(dsi_md_p)) || (pduData_p[0] & 0x01))
        {
            if (likely(!(pduData_p[0] & 0x01)))
            {
                bridge_stats_id = adk_dp_eth_bridge_map_id_g[dsi_md_p->l2_domain];
            }
            else
            {
                bridge_stats_id = adk_dp_eth_bridge_map_id_g[*adk_dp_eth_default_l2_domain_g];
                INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                        ingressUtilityCounters.eth_ing_multicast_packets_received);
                /* Multicast or broadcast packet will be treated same as L2 terminating packet */
                ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Muticast or broadcast packet received");
            }
            if(unlikely(bridge_stats_id >= ADK_ETH_MAX_BG_ID))
            {
                adk_eth_gl_stats_g_p[coreIndex].intf_not_defined++;
                ADK_DP_LOG_ERROR_1D(ADK_ETHERNET_MODULE_ID, "Bridge interface is"
                        " not defined for L2 domain %d", *adk_dp_eth_default_l2_domain_g);
                return ADK_E_INTERNAL_ERROR;
            }

            dp_bridge_if_p = adk_dp_eth_bridge_intf_t_g[bridge_stats_id];
            if(unlikely(NULL == dp_bridge_if_p)) {
                adk_eth_gl_stats_g_p[coreIndex].intf_not_defined++;
                ADK_DP_LOG_ERROR_1D(ADK_ETHERNET_MODULE_ID, "Bridge interface is"
                        " not defined for L2 domain %d", dsi_md_p->l2_domain);
                return ADK_E_INTERNAL_ERROR;
            }

            ADK_DP_LOG_INFO_1D_DEBUG(ADK_ETHERNET_MODULE_ID, "Retrieved bridge interface for L2 domain %d", dsi_md_p->l2_domain);
            dp_eth_intf_stats_p = &(dp_bridge_if_p->intf_stats[coreIndex]);
            ADK_DP_LOG_INFO_1P_DEBUG(ADK_ETHERNET_MODULE_ID, "Retrieved bridge If stats structure - %p\n",
                    dp_eth_intf_stats_p);

            /* Fill in the ADK common metadata */
            adk_md_p->input_intf_id = dp_bridge_if_p->statsId;

            task->l2_len = ETH_NO_VLAN_HEADER_LEN;
            if ((*(uint16_t*)&pduData_p[ETH_HDR_VLAN_HDR_OFFSET]) == adk_htons(ETHER_TYPE_VLAN))
            {
                task->l2_len = ETH_VLAN_HEADER_LEN;
                ret = adk_eth_vlan_header_proc(task, &pduData_p, dp_bridge_if_p, coreIndex, &strip_vlan);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    return ADK_E_INTERNAL_ERROR;
                }
            }
            ADK_DP_LOG_INFO_1P_DEBUG(ADK_ETHERNET_MODULE_ID, "Packet received on bridge interface %p \n", dp_bridge_if_p);
        }
        else
        {
            non_l2_terminating = true;
            INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                    ingressUtilityCounters.eth_ing_non_l2term_frames_received);
            ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Non-muticast or non-broadcast packet received");
            if(unlikely(adk_hos_is_ingress_flow_invalid(dsi_md_p)))
            {
                adk_eth_gl_stats_g_p[coreIndex].non_l2term_frames_discard++;

                ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "The frame is neither Unicast L2 terminating nor multi/broadcast."
                        " Packet does not match ingress flow. Hence packet drop!");
                return ADK_E_INTERNAL_ERROR;
            }
        }
    }
    if(unlikely(adk_hos_is_ethertype_invalid(dsi_md_p)))
    {
        dp_eth_intf_stats_p->inEthTypeMismatch++;
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Packet does not match valid ethertypes, Hence packet drop!");
        return ADK_E_INTERNAL_ERROR;
    }

    /* First see if any HLP ingress rule matches */
    if(!adk_hos_is_ingress_flow_invalid(dsi_md_p))
    {
        dp_flow_p = adk_eth_ingress_flow_list_g[adk_hos_get_ingress_flow(dsi_md_p)];
        if(unlikely(NULL == dp_flow_p))
        {
            adk_eth_gl_stats_g_p[coreIndex].ingressflow_lookup_failure++;
            ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Error in handover packet to higher layer module.");
            return ADK_E_INTERNAL_ERROR;
        }
    }
    else
    {
        if(ADK_DP_ETH_AUTO_CONN_DISABLE == *adk_dp_eth_auto_conn_status_g && (!(pduData_p[0] & 0x01)))
        {
            adk_eth_dp_update_rx_stats(dp_eth_intf_stats_p, task, pduData_p);
            ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Default Ingress flow."
                    " Handover packet to higher layer module!");
            /* Send default 0 as bg_id */
            ADK_MD_SET_LINUX_INTF_ID(task, 0 + ADK_ETH_BG_ID_OFFSET);

            if ((*(uint16_t*)&pduData_p[(task->l2_len - ETHER_TYPE_SIZE)]) == adk_htons(ETHER_TYPE_IPV4))
            {
                /* Fill in the ADK common metadata */
                ip_imc_p = (adk_ip_dp_lli_imc_data_t *)
                    &(ADK_METADATA_PARAM(task, imc_param_start));
                ip_imc_p->ipsec_data = IPLLI_IMC_ETH_OUTER;
                *fn_id = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
                INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                        ingressUtilityCounters.eth_ing_pkts_sent_to_ipv4);
            }
            else if ((*(uint16_t*)&pduData_p[(task->l2_len - ETHER_TYPE_SIZE)]) == adk_htons(ETHER_TYPE_IPV6))
            {
                *fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
                INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                        ingressUtilityCounters.eth_ing_pkts_sent_to_ipv6);
            }
            else
            {
                adk_eth_gl_stats_g_p[coreIndex].ingressflow_lookup_failure++;
                ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Error in handover packet to higher layer module."
                        " Only IPv4 and IPv6 packet is allowed to follow default ingress flow!");
                return ADK_E_INTERNAL_ERROR;
            }

            /* Fill in the ADK common metadata */
            adk_md_p->ip_offset = task->l2_len;
            return ADK_SUCCESS;
        }
        else
        {
            adk_eth_gl_stats_g_p[coreIndex].unknown_multicast_addr_drop++;
            ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Unknow mulitcast address traffic received");
            return ADK_E_INTERNAL_ERROR;
        }
    }
    ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Ethernet ingress flow lookup successful\n");
    adk_md_p->pdu_flow_id = dp_flow_p->flowId;
    dp_ingress_flow_stats_p = &(dp_flow_p->flow_stats[coreIndex]);
    dp_ingress_flow_stats_p->numPkts++;

    /* check flow action is send to CPU or NCA */
    switch(dp_flow_p->actionType)
    {
    case ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_HOST:

        /*Setting the linux interface id */
        if(dp_flow_p->intfType == ADK_ETHERNET_INTERFACE_TYPE_BRIDGE) 
        {
            /* Send default 0 as bg_id */
            ADK_MD_SET_LINUX_INTF_ID(task, 0 + ADK_ETH_BG_ID_OFFSET);

            if(strip_vlan)
            {
                adk_eth_vlan_header_strip(task, &pduData_p, coreIndex);
            }
            adk_eth_dp_update_rx_stats(dp_eth_intf_stats_p, task, pduData_p);
        }
        else
        {
            /*Else case is not introduced only for incrementing utility counters ,
              yet more changes to come here*/
            INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                    ingressUtilityCounters.eth_ing_phy_term_pkt_sent_to_netd);
        }
        adk_slowpath_dev_dp_metadata_t *spDevMd_p = NULL;
        spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,
                    imc_param_start));
        spDevMd_p->policerFlowId = dp_flow_p->policerId;
        spDevMd_p->flowId = dp_flow_p->flowId;
        spDevMd_p->moduleId = ADK_ETHERNET_MODULE_ID;
        /*
         * send the packet to CPU via netdriver
         */
        if (adk_pal_dp_netd_send((void *)task, ADK_PAL_DP_BATCH_CB_FN_ETH, NULL) != ADK_E_TASK_CONSUMED)
        {
            dp_ingress_flow_stats_p->pkts_drop++;
            /* The actual error logging must be available from SIF */
            ADK_DP_LOG_INFO(ADK_ETHERNET_MODULE_ID, "Error (LLI): Failed to"
                    " send packet to netdriver. Dropping packet!");
            return ADK_E_INTERNAL_ERROR;
        }
        INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                ingressUtilityCounters.eth_ing_pkts_sent_to_policer_stage);

        ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Sent packet to netdriver successfully!");
        return ADK_E_TASK_CONSUMED;

    case ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV4:
        /* Fill in the ADK common metadata */
        ip_imc_p = (adk_ip_dp_lli_imc_data_t *)
            &(ADK_METADATA_PARAM(task, imc_param_start));
        ip_imc_p->ipsec_data = IPLLI_IMC_ETH_OUTER;
        *fn_id = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
        /* Fill in the ADK common metadata */
        if(strip_vlan)
        {
            adk_eth_vlan_header_strip(task, &pduData_p, coreIndex);
        }
         /* Send default 0 as bg_id */
        ADK_MD_SET_LINUX_INTF_ID(task, 0 + ADK_ETH_BG_ID_OFFSET);
        adk_md_p->ip_offset = task->l2_len;
        adk_eth_dp_update_rx_stats(dp_eth_intf_stats_p, task, pduData_p);

        dp_ingress_flow_stats_p->pkts_sent_to_ipv4++;

        INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                ingressUtilityCounters.eth_ing_pkts_sent_to_ipv4);
        return ADK_SUCCESS;
        break;
    case ADK_ETHERNET_ASSOCIATION_RULE_TYPE_SEND_TO_IPV6:
        *fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
        /* Fill in the ADK common metadata */
        if(strip_vlan)
        {
            adk_eth_vlan_header_strip(task, &pduData_p, coreIndex);
        }
         /* Send default 0 as bg_id */
        ADK_MD_SET_LINUX_INTF_ID(task, 0 + ADK_ETH_BG_ID_OFFSET);
        adk_md_p->ip_offset = task->l2_len;
        adk_eth_dp_update_rx_stats(dp_eth_intf_stats_p, task, pduData_p);

        dp_ingress_flow_stats_p->pkts_sent_to_ipv6++;

        INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                ingressUtilityCounters.eth_ing_pkts_sent_to_ipv6);

        break;
    case ADK_ETHERNET_ASSOCIATION_RULE_TYPE_DROP:
        /* Since the flow indication is to drop, just return internal error. The packet 
           will be freed */
        if(strip_vlan)
        {
            task->l2_len -= ETH_VLAN_HDR_LEN;
        }
        adk_eth_dp_update_rx_stats(dp_eth_intf_stats_p, task, pduData_p);

        dp_ingress_flow_stats_p->pkts_drop++;
        ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "Ingress flow action is drop!");
        /* Returning error indicates parent function to drop the packet and free memory */
        return ADK_E_INTERNAL_ERROR;

    default:
        /* This should not happen ordinarily and indicates error in flow
         * action configuration */
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Error (LLI): Unsupported"
                         " ingress flow action. Dropping packet!");
        return ADK_E_INTERNAL_ERROR;
    }

    adk_eth_gl_stats_g_p[coreIndex].proc_cntr++;

    return (ret);
}

void adk_eth_dp_slowpath_policer_stats(void *user_params)
{

    adk_dp_eth_ingress_flow_t *dp_flow_p = NULL;
    uint32_t eth_classifier_flow_id;
    int coreIndex = ADK_LCORE_INDEX;

    if(!user_params)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "\nInvalid slow path parameters\n");
        return;
    }

    eth_classifier_flow_id = ((adk_slowpath_user_param_t *)user_params)->flow_id;

    if(eth_classifier_flow_id == ADK_ETH_NETD_TIMESTAMP_FLOWS_ID)
    {
        INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                ingressUtilityCounters.eth_ing_pkt_sent_to_netd);
        return;
    }
    if(eth_classifier_flow_id >= ADK_ETH_INGRESS_FLOWS_MAX)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "\nInvalid ethernet classifier flow id\n");
        return;
    }

    dp_flow_p = adk_eth_ingress_flow_list_g[eth_classifier_flow_id]; 
    if(NULL == dp_flow_p)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "\nInvalid ethernet classifier flow id\n");
        return;
    }

    if(((adk_slowpath_user_param_t *)user_params)->policeFlowId != UINT_MAX) /* Policing is enabled */
    {
        ADK_DP_LOG_INFO_1D_DEBUG(ADK_ETHERNET_MODULE_ID, "\nEthernet Classifier flow id ==>%u\n", eth_classifier_flow_id);

        if(((adk_slowpath_user_param_t *)user_params)->pkt_color == ADK_QOS_DP_GREEN)
        {
            ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "\nPacket marked green by policer\n");
            dp_flow_p->flow_stats[coreIndex].pkts_sent_to_host++;
            INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                        ingressUtilityCounters.eth_ing_pkt_sent_to_netd);
        }
        else
        {
            ADK_DP_LOG_INFO_DEBUG(ADK_ETHERNET_MODULE_ID, "\nPacket marked red/yellow by policer\n");
            dp_flow_p->flow_stats[coreIndex].pkts_drop++;
            adk_eth_gl_stats_g_p[coreIndex].ethPacketsRateLimitingDrops++;
        }
    }
    else /*Policing is disabled */
    {
            dp_flow_p->flow_stats[coreIndex].pkts_sent_to_host++;
            INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                        ingressUtilityCounters.eth_ing_pkt_sent_to_netd);
    }
    return;
}

adk_status_t adk_eth_dp_pkt_proc_police (void **task_p,
          __attribute__((unused)) uint8_t task_count,
          __attribute__((unused)) uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex)
{
    uint8_t policeColor = ADK_QOS_DP_GREEN;
    adk_pktio_task_t *task = (adk_pktio_task_t *)(*task_p);
    adk_slowpath_dev_dp_metadata_t *spDevMd_p;
    adk_eth_dp_policer_meta_data_t *eth_policer_md_p = NULL;
    adk_dp_eth_ingress_flow_stats_t* dp_ingress_flow_stats_p = NULL;

    eth_policer_md_p = (adk_eth_dp_policer_meta_data_t *)&(ADK_METADATA_PARAM(task, imc_param_start));

    dp_ingress_flow_stats_p = &adk_eth_ingress_flow_list_g[eth_policer_md_p->flowId]->flow_stats[coreIndex];
 
    INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                            ingressUtilityCounters.eth_ing_pkts_recv_in_policer_stage);
    if(unlikely( ADK_SUCCESS != adk_qos_dp_policing_color_blind(
                                    eth_policer_md_p->policerFlowId,
                                    ADK_TASK_PDU_SIZE(task), &policeColor)))
    {
        /*Free the task*/
        adk_eth_gl_stats_g_p[coreIndex].ingressflowDrops++;
        adk_pktio_task_buffer_free(task);
        return ADK_SUCCESS;
    }
    /* check policing color */
    if(unlikely(ADK_QOS_DP_RED == policeColor))
    {
        /* Dropping the packet */
        adk_eth_gl_stats_g_p[coreIndex].ethPacketsRateLimitingDrops++;
        adk_pktio_task_buffer_free(task);
        return ADK_SUCCESS;
    }

    spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,
                imc_param_start));
    spDevMd_p->policerFlowId = ADK_QOS_INVALID_POLICER_FLOW_ID;

    /*
     * send the packet to CPU via netdriver
     */
    if (adk_pal_dp_netd_send(task, ADK_PAL_DP_BATCH_CB_FN_ETH, NULL) != ADK_SUCCESS)
    {
        dp_ingress_flow_stats_p->pkts_drop++;
        /* The actual error logging must be available from SIF */
        ADK_DP_LOG_INFO(ADK_ETHERNET_MODULE_ID, "Error (LLI): Failed to"
                " send packet to netdriver. Dropping packet!");
        adk_pktio_task_buffer_free(task);
        return ADK_E_INTERNAL_ERROR;
    }
    dp_ingress_flow_stats_p->pkts_sent_to_host++;
    INCREMENT_ETH_UTILITY_COUNTER(adk_eth_dp_pvt_g_p->ethUtilityCnt[coreIndex].
                                        ingressUtilityCounters.eth_ing_pkt_sent_to_netd);

    return ADK_SUCCESS;
}
#ifdef ADK_DEVICE_SNR
#if 0
static inline adk_status_t
adk_eth_packet_clone(struct rte_mbuf *in_task, struct rte_mbuf **pkt)
{
    struct rte_mbuf *cloned_pkt = NULL;
    struct rte_mbuf *pkt_hdr = NULL;

    /* Create new direct mbufs for the header. */
    pkt_hdr = rte_pktmbuf_alloc(eth_mempool_p);

    if (unlikely(pkt_hdr == NULL)) {
        debug_print("\nError: Memory unavailable to allocate mbuf \n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    if (unlikely ((cloned_pkt =
                   rte_pktmbuf_clone(in_task, eth_mempool_p)) == NULL)) {
        /*
         * Free the direct mbufs.
         */
        adk_pktio_task_buffer_free(pkt_hdr);
        debug_print("\nError: DPDK cloning failed \n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    /* Init ADK metadata and set the oper type as NEW */
    adk_pkt_metadata_init((adk_pktio_task_t *) pkt_hdr,
                          ADK_FP_ORIGIN_DOMAIN, 0, ADK_TASK_TYPE_NEW);

    /* Copy TN timestamp */
    pkt_hdr->timestamp = in_task->timestamp;

    /* Chaining direct mbuf with cloned bufs. */
    pkt_hdr->next = cloned_pkt;

    /* update packet length and number of segments.*/
    pkt_hdr->pkt_len = (uint16_t)(pkt_hdr->data_len + in_task->pkt_len);
    pkt_hdr->nb_segs = (uint8_t)(in_task->nb_segs + 1);

    /* Copy DSI meta data */
    adk_dsi_rx_metadata_t *in_dsi_rx_md_p = ADK_DSI_RX_METADATA_PTR(in_task);
    adk_dsi_rx_metadata_t *hdr_dsi_rx_md_p = ADK_DSI_RX_METADATA_PTR(pkt_hdr);
    ADK_MEMCPY(hdr_dsi_rx_md_p, in_dsi_rx_md_p, sizeof(adk_dsi_rx_metadata_t));

    /* Copy ADK meta data */
    adk_metadata_t *in_adk_md_p = ADK_METADATA_PTR(in_task);
    adk_metadata_t *hdr_adk_md_p = ADK_METADATA_PTR(pkt_hdr);
    ADK_MEMCPY(hdr_adk_md_p, in_adk_md_p, sizeof(adk_metadata_t));

    *pkt = pkt_hdr;

    return ADK_SUCCESS;
}
adk_status_t
adk_eth_packet_clone(struct rte_mbuf *inputPkt, struct rte_mbuf **new_pkt)
{
    struct rte_mbuf *pkt_clone, *pkt_segment, *pkt, *prev;
    uint8_t nb_segs;

    pkt = inputPkt;
    nb_segs = pkt->nb_segs;
    pkt_clone = rte_pktmbuf_alloc(eth_mempool_p);
    if(pkt_clone == NULL)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Mbuf allocation failed for Pkt capture.");
        return ADK_E_MEM_UNAVAILABLE;
    }
 
    pkt_clone->nb_segs = pkt->nb_segs;
    pkt_clone->pkt_len = pkt->pkt_len;
    pkt_clone->port = pkt->port;

    pkt_segment = pkt_clone;
    prev = NULL;

    /*copy ADK metadata for the first segment*/
    adk_metadata_t *adk_md_cap = ADK_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)adk_md_cap,(void *) ADK_METADATA_PTR(pkt),
                offsetof(adk_metadata_t,imc_param_start));
    adk_md_cap->task_type = (uint8_t)ADK_TASK_TYPE_NEW;

    adk_dsi_tx_metadata_t  *dsi_tx_md_p = ADK_DSI_TX_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)dsi_tx_md_p,(void *) ADK_DSI_TX_METADATA_PTR(pkt),
                sizeof(adk_dsi_tx_metadata_t));
    /*copy all segments payload*/
    while(nb_segs)
    {
        rte_memcpy(rte_pktmbuf_mtod(pkt_segment, void *),
                rte_pktmbuf_mtod(pkt, void *), rte_pktmbuf_data_len(pkt));
        pkt_segment->data_len = rte_pktmbuf_data_len(pkt);
        nb_segs--; 
        if(nb_segs)
        {
            prev = pkt_segment;
            pkt_segment->next = NULL;
            pkt_segment = rte_pktmbuf_alloc(eth_mempool_p);
            if(pkt_segment == NULL)
            {
                ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,
                    "Mbuf allocation failed for Pkt capture. Dropping packet!");
                break;
            }
            prev->next = pkt_segment;
            pkt= pkt->next;
        }
    }

    if(unlikely(pkt_segment == NULL))
    {
        pkt = pkt_clone;
        while(pkt != NULL)
        {
            prev= pkt->next;
            rte_pktmbuf_free(pkt); /* free all allocated segments */
            pkt = prev;
        }
        return ADK_E_MEM_UNAVAILABLE;
    }
    __rte_mbuf_sanity_check(pkt_clone, 1);
    *new_pkt = pkt_clone;
    return ADK_SUCCESS;
}
#else
adk_status_t
adk_eth_packet_clone(
    struct rte_mbuf *inputPkt,
    struct rte_mbuf **new_pkt,
    uint16_t mbuf_clone_size,
    uint8_t no_clone_segments)
{
    struct rte_mbuf *pkt_clone, *pkt;
    uint8_t pkt_clone_size = 0;

    if(mbuf_clone_size > ADK_ETH_PACKET_CLONE_MAX_SIZE)
    {
        return ADK_E_INTERNAL_ERROR;
    }
    /*Currently supports cloning only one segments since we dont have
    use case to copy multiple segments in ethernet exception data*/
    if((no_clone_segments == 0) &&
        no_clone_segments > ADK_ETH_PACKET_CLONE_SEGMENTS)
    {
        return ADK_E_INTERNAL_ERROR;
    }
    
    pkt = inputPkt;

    pkt_clone = rte_pktmbuf_alloc(eth_mempool_p);
    if(pkt_clone == NULL)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID, "Mbuf allocation failed for Pkt capture.");
        return ADK_E_MEM_UNAVAILABLE;
    }

    if(pkt_clone->pkt_len < mbuf_clone_size)
        pkt_clone_size = pkt_clone->pkt_len;
    else
        pkt_clone_size = mbuf_clone_size;

    pkt_clone->nb_segs = no_clone_segments;
    pkt_clone->pkt_len = pkt_clone_size;
    pkt_clone->port = pkt->port;

    /*copy ADK metadata for the first segment*/
    adk_metadata_t *adk_md_cap = ADK_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)adk_md_cap,(void *) ADK_METADATA_PTR(pkt),
            offsetof(adk_metadata_t,imc_param_start));
    adk_md_cap->task_type = (uint8_t)ADK_TASK_TYPE_NEW;
    ADK_METADATA_PARAM(pkt_clone, next_frag) = NULL;

    adk_dsi_tx_metadata_t  *dsi_tx_md_p = ADK_DSI_TX_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)dsi_tx_md_p,(void *) ADK_DSI_TX_METADATA_PTR(pkt),
            sizeof(adk_dsi_tx_metadata_t));

    rte_memcpy(rte_pktmbuf_mtod(pkt_clone, void *),
            rte_pktmbuf_mtod(pkt, void *), pkt_clone_size);

    pkt_clone->data_len = pkt_clone_size;
    pkt_clone->next = NULL;

    __rte_mbuf_sanity_check(pkt_clone, 1);
    *new_pkt = pkt_clone;
    return ADK_SUCCESS;
}
#endif

#endif

 /*!  end addtogroup _ethernetFunctions  Ethernet Module
 * @}
 */
