/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernet  Ethernet Module
 *
 * @{
 *
 * @brief   This file has all the data types and prototypes for low
 *          layer internal functions and other helper functions which are
 *          required by Ethernet APIs to carry out the tasks. The definitions
 *          in this file are not used by data plane software.
 *
 *  **********************************************************************/

#ifndef _ADK_ETH_H_
#define _ADK_ETH_H_

#include <adk_include.h>
#include <adk_qos_include_internal.h>
#include <adk_ethernet.h>
#include <adk_eth_constants.h>
#include <adk_ethernet_export.h>
#include <adk_eth_dp.h>
#include <adk_rm.h>
#include <adk_lpm.h>
#include <adk_rcu.h>
#include <adk_hos.h>
#include <adk_qos_include_internal.h>

/**************************************************************
* Interface Number & Attributes in TAG
* bits:4[0]  = Port Number
* bits:3[4]  = Forwarding Mode
* bits:1[7]  = Promiscuous Mode Flag (Routed Ports)
* bits:3[7]  = VLAN Assignment Policy (Switch Ports)
* bits:1[10] = Policing/PCP-Mapping Enable Flag (Switch ports)
* bits:1[11] = Priority Mapping Flag (Switch ports)
* bits:4[12] = Original Port Number (Aggregate Interface)
**************************************************************/

/* *********************************
************ MACROS ***************
***********************************/

#define ADK_SYS_HANDLE_GET(mod) \
    ((adk_module_info_t *) mod)->sysHandle

/* *********************************
********** LOG MACROS *************
***********************************/

#define ADK_ETH_LOG_FATAL(mod_handle, err, fmt, args ...) \
    do { \
        ADK_LOG_FATAL_ERROR(sys_handle, mod_handle, fmt, ## args); \
        ADK_LOG_FATAL_ERROR(sys_handle, mod_handle, "%s\n", \
                            adk_sys_status_get(sys_handle, err)); \
    } while (0)

#define ADK_ETH_LOG_ERROR(mod_handle, err, fmt, args ...) \
    do { \
        ADK_LOG_ERROR(sys_handle, mod_handle, fmt, ## args); \
        ADK_LOG_ERROR(sys_handle, mod_handle, "%s\n", \
                      adk_sys_status_get(sys_handle, err)); \
    } while (0)

#define ADK_ETH_LOG_WARNING(mod_handle, fmt, args ...) \
    ADK_LOG_WARNING(sys_handle, mod_handle, fmt, ## args)

#define ADK_ETH_LOG_INFO(mod_handle, fmt, args ...) \
    ADK_LOG_INFO(sys_handle, mod_handle, fmt, ## args)

#define ADK_ETH_LOG_FUNC_ENTRY(mod_handle) \
    ADK_LOG_FUNC_ENTRY(sys_handle, mod_handle)

#define ADK_ETH_LOG_FUNC_EXIT(mod_handle) \
    ADK_LOG_FUNC_EXIT(sys_handle, mod_handle)

#define ADK_ETH_LOG_HW(mod_handle, fmt, args ...) \
    ADK_LOG_HW(sys_handle, mod_handle, fmt, ## args)

#define ADK_ETH_LOG_PTN_LEARN(mod_handle, tree_name, ptn, action) \
    ADK_LOG_PTN_LEARN(sys_handle, mod_handle, tree_name, ptn, action)

#define ADK_ETH_LOG_PTN_UNLEARN(mod_handle, tree_name, ptn, xsize, action) \
    ADK_LOG_PTN_UNLEARN(sys_handle, mod_handle, tree_name, ptn, xsize, action)

#define ADK_ETH_LOG_SUBNET_ADD(mod_handle, tree_name, ptn, action) \
    ADK_LOG_SUBNET_ADD(sys_handle, mod_handle, tree_name, ptn, action)

#define ADK_ETH_LOG_SUBNET_DEL(mod_handle, tree_name, ptn) \
    ADK_LOG_SUBNET_DEL(sys_handle, mod_handle, tree_name, ptn)

#define ADK_ETH_LOG_SUBNET_QUERY(mod_handle, tree_name, ptn, xsize, action) \
    ADK_LOG_SUBNET_QUERY(sys_handle, mod_handle, tree_name, ptn, xsize, action)

#define ADK_ETH_LOG_VPTN_LEARN(mod_handle, tree_name, vruleidx, no_vptns, \
                               vptn, action) \
    ADK_LOG_VPTN_LEARN(sys_handle, mod_handle, tree_name, vruleidx, no_vptns, \
                       vptn, action)

#define ADK_ETH_LOG_VPTN_UNLEARN(mod_handle, tree_name, vruleidx, vptn) \
    ADK_LOG_VPTN_UNLEARN(sys_handle, mod_handle, tree_name, vruleidx, vptn)

#define ADK_ETH_LOG_VPTN_CLEAR(mod_handle, tree_name, vptn) \
    ADK_LOG_VPTN_CLEAR(sys_handle, mod_handle, tree_name, vptn)

#define ADK_ETH_LOG_VPTN_BUILD(mod_handle, tree_name, vptn) \
    ADK_LOG_VPTN_BUILD(sys_handle, mod_handle, tree_name, vptn)

#define ADK_ETH_LOG_SE_WRITE(mod_handle, se_addr, idx, char_string) \
    ADK_LOG_SE_WRITE(sys_handle, mod_handle, se_addr, idx, char_string)

#define ADK_ETH_LOG_SE_READ(mod_handle, se_addr, idx, char_string) \
    ADK_LOG_SE_READ(sys_handle, mod_handle, se_addr, idx, char_string)

#define ADK_ETH_LOG_MTM_READ(mod_handle, mtm_addr, idx, char_string) \
    ADK_LOG_MTM_READ(sys_handle, mod_handle, mtm_addr, idx, char_string)

#define ADK_ETH_LOG_FORCE(fmt, args ...) \
    ADK_FORCE_LOG_ERROR(fmt, ## args);

#define ADK_ETH_NON_ZERO_CNTR_WRITE(fmt, cntr) \
    if (cntr != 0) { \
        ADK_INFO_WRITE(fmt, cntr) \
    }

/* *********************************
******* VALIDATION MACROS *********
***********************************/

#define ADK_ETH_IS_PORT_NUMBER_INVALID(port) \
    ( \
        ((port >> 4) > 7) || /* Only eight EIOAs - EIOA0 to EIOA7 */ \
        (((port >> 4) == 0) && ((port & 0x7) > 4)) || /* For EIOA0 and EIOA1 in legacy mode */ \
        (((port >> 4) == 1) && ((port & 0xF) > 4)) || /* For EIOA1 in non-legacy mode */ \
        (((port >> 4) > 1) && ((port & 0xF) > 1))   /* For EIOA2 to EIOA7 */ \
    )

#define ADK_ETH_IS_FLOW_CONTROL_TYPE_INVALID(type) \
    ((type <= ADK_ETHERNET_FLOW_CONTROL_INVALID) || (type >= ADK_ETHERNET_FLOW_CONTROL_MAX))

#define ADK_ETH_IS_STP_STATE_INVALID(type) \
    ((type <= ADK_ETHERNET_STP_STATE_INVALID) || (type >= ADK_ETHERNET_STP_STATE_MAX))

#define ADK_ETH_IS_MAC_BROADCAST(mac_addr) \
    ((mac_addr[0] == 0xFF) && (mac_addr[1] == 0xFF) && \
     (mac_addr[2] == 0xFF) && (mac_addr[3] == 0xFF) && \
     (mac_addr[4] == 0xFF) && (mac_addr[5] == 0xFF))

#define ADK_ETH_IS_MAC_ZERO(mac_addr) \
    ((mac_addr[0] == 0x00) && (mac_addr[1] == 0x00) && \
     (mac_addr[2] == 0x00) && (mac_addr[3] == 0x00) && \
     (mac_addr[4] == 0x00) && (mac_addr[5] == 0x00))

#define ADK_ETH_IS_MAC_MULTICAST(mac_addr) \
    (mac_addr[0] & ADK_ETH_MAC_ADDR_MULTICAST_MASK)

#define ADK_ETH_IS_MAC_SUBNET_BROADCAST(mac_subnet) \
    ((mac_subnet.num_valid_msb_bits == 48) && \
     (ADK_ETH_IS_MAC_BROADCAST(mac_subnet.mac)))

#define ADK_ETH_IS_MAC_SUBNET_MULTICAST(mac_subnet) \
    ((mac_subnet.num_valid_msb_bits >= 8) && \
     (mac_subnet.mac[0] & ADK_ETH_MAC_ADDR_MULTICAST_MASK))

#define ADK_ETH_IS_MAC_SUBNET_LEN_INVALID(mac_subnet) \
    (mac_subnet->num_valid_msb_bits > 48)

#define ADK_ETH_IS_MAC_SUBNET_LEN_ZERO(mac_subnet) \
    (mac_subnet.num_valid_msb_bits == 0)

#define ADK_ETH_IS_ENCAP_TYPE_INVALID(encap) \
    ((encap <= ADK_ETHERNET_ENCAP_INVALID) || (encap >= ADK_ETHERNET_ENCAP_MAX))

#define ADK_ETH_IS_ADK_ETHERNET_FWD_MODE_INVALID(fwd_mode) \
    ((fwd_mode <= ADK_ETHERNET_FWD_MODE_INVALID) || (fwd_mode >= ADK_ETHERNET_FWD_MODE_MAX))

#define ADK_ETH_IS_HASH_OFFSETS_INVALID(start_offset, stop_offset) \
    ((stop_offset <= start_offset) || (stop_offset >= 60))

#define ADK_ETH_IS_STATS_TYPE_INVALID(stats_type) \
    ((stats_type <= ADK_ETHERNET_STATS_TYPE_INVALID) || \
     (stats_type >= ADK_ETHERNET_STATS_TYPE_MAX))

#define ADK_ETH_IS_PROTO_INVALID(eth_type) \
    (eth_type < ADK_ETH_MIN_PROTO_TYPE)

#define ADK_ETH_IS_LLC_INVALID(llc_header) \
    (((llc_header.dsap == 0) && (llc_header.ssap == 0) && \
      (llc_header.control == 0)) || \
     ((llc_header.dsap == 0xAA) && (llc_header.ssap == 0xAA) && \
      (llc_header.control == 0x03)))

#define ADK_ETH_IS_LLC_ZERO(llc_header) \
    ((llc_header.dsap == 0) && (llc_header.ssap == 0) && \
     (llc_header.control == 0))

#define ADK_ETH_IS_VLAN_INVALID(vlan_id) \
    (vlan_id > ADK_ETH_MAX_VLAN_ID)

#define ADK_ETH_IS_PRIORITY_INVALID(priority) \
    (priority > ADK_ETH_MAX_PRIORITY)

#define ADK_ETH_IS_DSCP_INVALID(dscp) \
    (dscp > ADK_ETH_MAX_DSCP)

#define ADK_ETH_IS_ADK_ETHERNET_POLICING_LEVEL_INVALID(police_level) \
    ((police_level <= ADK_ETHERNET_POLICING_LEVEL_INVALID) || \
     (police_level >= ADK_ETHERNET_POLICING_LEVEL_MAX))

#define ADK_ETH_IS_MTU_INVALID(mtu) \
    ((mtu < ADK_ETH_MTU_MIN) || (mtu > ADK_ETH_MTU_MAX))

#define ADK_ETH_IS_SPEED_INVALID(speed) \
    ((speed <= ADK_ETHERNET_SPEED_INVALID) || (speed >= ADK_ETHERNET_SPEED_MAX))

#define ADK_ETH_IS_INGRESS_FLOW_ACTION_TYPE_INVALID(action_type) \
    ((action_type <= ADK_ETHERNET_ASSOCIATION_RULE_TYPE_INVALID) || \
     (action_type >= ADK_ETHERNET_ASSOCIATION_RULE_TYPE_MAX))

#define ADK_ETH_IS_EGRESS_FLOW_ACTION_TYPE_INVALID(action_type) \
    ((action_type <= ADK_ETHERNET_EGRESS_FLOW_RULE_TYPE_INVALID) || \
     (action_type >= ADK_ETHERNET_EGRESS_FLOW_RULE_TYPE_MAX))

#define ADK_ETH_IS_HASH_FLOW_ACTION_TYPE_INVALID(action_type) \
    ((action_type <= ADK_ETHERNET_HASH_RULE_TYPE_INVALID) || \
     (action_type >= ADK_ETHERNET_HASH_RULE_TYPE_MAX))

#define ADK_ETH_IS_VLAN_ASSIGN_POLICY_TYPE_INVALID(policy_type) \
    (policy_type >= ADK_ETHERNET_VLAN_ASSIGN_MAX)

#define ADK_ETH_IS_EIOA_TS_INDEX_INVALID(index) \
    (index >= ADK_ETH_EIOA_MAX_TX_INDEX)

#define ADK_ETH_IS_IPV4_ADDR_INVALID(ip_addr) \
    (((ip_addr >> 28) == 0xE) || (ip_addr == 0xFFFFFFFF))

/* *************************************
******* MACROS FOR LOCK/UNLOCK *********
****************************************/
#if 1

#define ADK_ETH_ACQUIRE_MODULE_LOCK(mod_handle, _ret) \
    ADK_ACQUIRE_MODULE_LOCK((mod_handle), (_ret))

#define ADK_ETH_RELEASE_MODULE_LOCK(mod_handle, _ret) \
    ADK_RELEASE_MODULE_LOCK((mod_handle), (_ret))

#else

#define ADK_ETH_ACQUIRE_MODULE_LOCK(mod_handle) \
    do { \
        adk_status_t _adk_st; \
        _adk_st = ADK_ACQUIRE_MODULE_LOCK(mod_handle); \
        if (_adk_st) \
        { \
            ADK_ETH_LOG_ERROR(mod_handle, _adk_st, \
                              "Failed to acquire Ethernet module lock\n"); \
            _adk_st = ADK_ERROR_SEV_LEVEL_SET(_adk_st); \
            ADK_ETH_LOG_FUNC_EXIT(mod_handle); \
            return (_adk_st); \
        } \
    } while (0)


#define ADK_ETH_RELEASE_MODULE_LOCK(mod_handle) \
    do { \
        adk_status_t _adk_st; \
        _adk_st = ADK_RELEASE_MODULE_LOCK(mod_handle); \
        if (_adk_st) \
        { \
            ADK_ETH_LOG_FATAL(mod_handle, _adk_st, \
                              "Failed to release Ethernet module lock\n"); \
            _adk_st = ADK_FATAL_SEV_LEVEL_SET(_adk_st); \
            ADK_ETH_LOG_FUNC_EXIT(mod_handle); \
            return (_adk_st); \
        } \
    } while (0)

#endif

/* *********************************
 ******* MACROS FOR PRINT *********
 ***********************************/
#define ADK_ETH_MAC_ADDR_BYTES(mac_addr) \
    mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]

/* *************************************
******* MACROS FOR OPERATIONS *********
***************************************/
#define ADK_ETH_LLC_INT(llc_header) \
    ((((llc_header).dsap) << 16) + \
     (((llc_header).ssap) << 8) + \
     (((llc_header).control)))

#define ADK_ETH_INTERFACE_ID_GET(if_handle) \
    (((adk_eth_intf_internal_data_t *) \
      ADK_SYS_INTERFACE_ATTR_PTR_GET(if_handle))->if_id)

#define SET_BIT(data, pos) data = ((data) | (1 << (pos)))

#define CLEAR_BIT(data, pos) data = ((data) & (~(1 << (pos))))

#define CHECK_BIT(data, pos) ((data) & (1 << (pos)))

#define ADK_ETH_GET_EIOA0_SLOT1_BIT_MAP(allocBitMap) \
    (((allocBitMap) >> 1) & 0xF)

#define ADK_ETH_GET_EIOA1_SLOT1_BIT_MAP(allocBitMap) \
    (((allocBitMap) >> 6) & 0xF)

#define ONLY_SINGLE_BIT_IS_ON(bitMap) \
    (((bitMap) > 0) && (!((bitMap) & (bitMap - 1))))

/* ****************************************************
******* MACROS FOR CONVERTING HANDLE TO BGID *********
******************************************************/
#define ADK_CONVERT_HDL_TO_BGID(__HANDLE) ((uintptr_t)__HANDLE)
#define ADK_CONVERT_HDL_TO_BGID_PTR(__HANDLE) ((uintptr_t *)__HANDLE)

#define ADK_ETH_IS_BG_ID_INVALID(bg_id) \
    (bg_id >= ADK_ETH_MAX_BG_ID)

/* *********************************
 ******* MACROS FOR PRINT *********
 ***********************************/
#define ADK_ETHERNET_IPV4_ADDR_BYTES(ip_addr) \
    ((ip_addr >> 24) & 0xFF), ((ip_addr >> 16) & 0xFF), \
    ((ip_addr >> 8) & 0xFF), (ip_addr & 0xFF)

#define ADK_ETHERNET_MAC_ADDR_BYTES(mac_addr) \
    mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]

/* *********************************
********* DATA STRUCTURES *********
***********************************/

/*!
 * =============================================================
 * @internal
 * @enum adk_ethernet_intf_type_t adk_eth.h "adk_eth.h"
 *
 * @brief Interface types
 *
 * The interface types provided by Ethernet module which can be physical
 * Ethernet interface (represents an EIOA port), aggregate interface or a VLAN
 * based logical interface.
 *
 * =============================================================
 */
typedef enum adk_ethernet_intf_type_s
{
    INTERFACE_TYPE_INVALID = 0,
    INTERFACE_TYPE_PHY,
    INTERFACE_TYPE_BRIDGE,
    INTERFACE_TYPE_VLAN,
    INTERFACE_TYPE_SWITCH_VIRTUAL,
    INTERFACE_TYPE_MAX
} adk_ethernet_intf_type_t;

/*!
 * =============================================================
 * @internal
 * @enum adk_eth_custom_bind_type_t adk_eth.h "adk_eth.h"
 *
 * @brief Ethernet Custom Bind Types
 *
 * The ways of custom binding supported by Ethernet. This also includes type for
 * unbinding operation.
 *
 * ADK_ETH_CUSTOM_BIND_NO_PARAM This is the most efficient type of bind where
 *                              the processing control jumps to destination
 *                              module's entry function right from the flow
 *                              classfication filter itself.
 *
 * ADK_ETH_CUSTOM_BIND_FLOWID_PARAM
 *
 *                              This is the next preferred way of binding when
 *                              Ethernet flow ID itself is required as paramter
 *                              by the destination module and hence same flow
 *                              classification filter cannot be used as above.
 *
 * ADK_ETH_CUSTOM_BIND_REMOVE   This type is used to remove existing custom
 *                              bind
 *
 * =============================================================
 */
typedef enum adk_eth_custom_bind_type_s
{
    ADK_ETH_CUSTOM_BIND_NO_PARAM = 0,
    ADK_ETH_CUSTOM_BIND_FLOWID_PARAM,
    ADK_ETH_CUSTOM_BIND_REMOVE
} adk_eth_custom_bind_type_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_if_phy_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Interface attributes
 *
 * This structure has the physical interface attributes which is specified at
 * the time of phy interface creation. It has both the base as well as extended
 * attributes stored in it.
 *
 * =============================================================
 */
typedef struct adk_if_phy_data_s
{
    bool is_nac;
    bool promiscuous;
    bool ingress_flow_filter;
    bool egress_flow_filter;
    bool stp;
    bool stp_state;
    adk_ethernet_proto_t vlan_eth_type[2];
    adk_interface_handle_t aggr_if;
    adk_ethernet_mac_addr_t allowed_mac;
    uint32_t hash_start_offset;
    uint32_t hash_stop_offset;
    uint32_t vsi_id;
} adk_if_phy_data_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_if_vlan_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Logical interface data
 *
 * This structure has the information about the logical interface which is
 * specified at the time of logical interface creation. It has the physical
 * interface handle on which it was created along with the VLAN ID which defines
 * this interface. It also has the name space resource ID for maintaining
 * statistics.
 *
 * =============================================================
 */
typedef struct adk_if_vlan_data_s
{
    adk_ethernet_vlan_id_t vlan_id;
    adk_interface_handle_t if_handle;
} adk_if_vlan_data_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_if_bridge_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Bridge interface data
 *
 * This structure has the information about the bridge interface. This
 * has the switch MAC address and the L2 domain on which it's definition is based.
 * It also has the name space resource ID for maintaining statistics.
 *
 * =============================================================
 */
typedef struct adk_if_bridge_data_s
{
    adk_ethernet_mac_addr_t             src_mac;
    int                                 l2_domain_id; 
    bool                                priority_tag; 
} adk_if_bridge_data_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_if_virtual_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Switch Virtual interface data
 *
 * This structure has the information about the switch virtual interface. This
 * has the switch MAC address and the VLAN ID on which it's definition is based.
 * It also has the name space resource ID for maintaining statistics.
 *
 * =============================================================
 */
typedef struct adk_if_virtual_data_s
{
    adk_ethernet_switch_virtual_if_data_t svif_data;
} adk_if_virtual_data_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_eth_intf_internal_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Generic interface data
 *
 * This structure has the information about the type and associated data of
 * any type of interface.
 *
 * =============================================================
 */
typedef struct adk_eth_intf_internal_data_s
{
    uint32_t cookie;                /* Ethernet module specific cookie */
    adk_ethernet_intf_type_t type;
    uint32_t if_id;
    uint32_t inif_flags;
    uint32_t outif_flags;
    union
    {
        adk_if_vlan_data_t vlan_if;
        adk_if_bridge_data_t bridge_if;
        adk_if_virtual_data_t switch_virtual_if;
    } u;
    adk_dp_eth_svif_intf_t *dp_if_ptr;
    /* HOS specific rule IDs */
    adk_hos_rule_id_t hos_l2_term_br_intf_rule_id;
    int hos_mac_mapper_rule_id;
    int hos_multicast_mac_mapper_rule_id;
    int hos_multicast_v6_mac_mapper_rule_id;
    int hos_broadcast_mac_mapper_rule_id;
    adk_hos_rule_id_t hos_l2_term_svif_intf_rule_id;
    adk_hos_rule_id_t hos_l2_term_svif_intf_m_rule_id;
    adk_hos_rule_id_t hos_l2_term_svif_intf_m_v6_rule_id;
    adk_hos_rule_id_t hos_l2_term_svif_intf_b_rule_id;
#ifdef ADK_HOS_OLD_IMPL
    adk_hos_rule_id_t bvi_vrd_rule_id[ADK_HOS_CPK_PORT];
    adk_hos_rule_id_t mac_term_rule_id;
    adk_hos_rule_id_t ip4_outer_id_rule_id[ADK_HOS_CPK_PORT];
    adk_hos_rule_id_t ip6_outer_id_rule_id[ADK_HOS_CPK_PORT];
    adk_hos_rule_id_t ip4_e_outer_rule_id;
    adk_hos_rule_id_t ip6_e_outer_rule_id;
#endif
} adk_eth_intf_internal_data_t;

/*!
 * =============================================================
 * @internal
 * @enum adk_ethernet_flow_type_t adk_eth.h "adk_eth.h"
 *
 * @brief Flow types
 *
 * The various types of flows defined by Ethernet module. These
 * can be simple ingress flow, egress encapsulation flow or extended
 * ingress association flow.
 *
 * =============================================================
 */
typedef enum adk_ethernet_flow_type_s
{
    ETH_FLOW_TYPE_INVALID = 0,
    ETH_FLOW_TYPE_SIMPLE_INGRESS,
    ETH_FLOW_TYPE_EXT_INGRESS_ASSOC,
    ETH_FLOW_TYPE_POLICING,
    ETH_FLOW_TYPE_SIMPLE_EGRESS,
    ETH_FLOW_TYPE_EGRESS_ENCAP,
    ETH_FLOW_TYPE_IPV4_UDP_ENCAP,
    ETH_FLOW_TYPE_MPLS_ATM_ENCAP,
    ETH_FLOW_TYPE_HASH,
    ETH_FLOW_TYPE_CPU_ORIGIN,
    ETH_FLOW_TYPE_MAX
} adk_ethernet_flow_type_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_eth_flow_internal_data_t adk_eth.h "adk_eth.h"
 *
 * @brief Generic flow data
 *
 * This structure has the information about the type and associated
 * data for various type of flows defined by ethernet module.
 *
 * =============================================================
 */
typedef struct adk_eth_flow_internal_data_s
{
    uint32_t cookie;                /* Ethernet module specific cookie */
    adk_ethernet_flow_type_t type;
    union
    {
        adk_ethernet_ingress_flow_data_t ingress_flow;
        uint32_t unused;
    } u;
    adk_ethernet_ingress_flow_action_t flow_action;
    adk_flow_handle_t destFlow;
    adk_hos_rule_id_t hos_l2_ingress_rule_id; 
    uint32_t pvt_data;
    union
    {
        adk_dp_eth_ingress_flow_t       *dp_ing_flow_p;
        adk_dp_eth_cpu_flow_result_t    *dp_cpu_flow_p;
    } dp_u;
} __attribute__ ((aligned(64))) adk_eth_flow_internal_data_t;

/*!
 * =============================================================
 * @internal
 * @enum adk_eth_pkt_oper_t
 *
 * @brief Packet forwarding type
 *
 * The data structure specifies the type of forwarding. It can
 * be either ethernet switching or ethernet encapsulation of
 * packets sent by higher layer module.
 *
 * =============================================================
 */
typedef enum adk_eth_pkt_oper_s
{
    ETH_PKT_OPER_INVALID = 0,
    ETH_PKT_OPER_SWITCH,
    ETH_PKT_OPER_ENCAP,
    ETH_PKT_OPER_MAX
} adk_eth_pkt_oper_t;

/*!
 * =============================================================
 * @internal
 * @enum adk_eth_tree_oper_t
 *
 * @brief tree operation type
 *
 * The Ethernet module for its API operations needs to add or delete entries
 * from its trees. This data type specified if an entry needs to be removed
 * or added.
 *
 * =============================================================
 */
typedef enum adk_eth_tree_oper_s
{
    ADK_ETH_TREE_OPER_INVALID = 0,
    ADK_ETH_TREE_OPER_ADD,
    ADK_ETH_TREE_OPER_DEL,
    ADK_ETH_TREE_OPER_MAX
} adk_eth_tree_oper_t;

/*!
 * =============================================================
 * @internal
 * @enum adk_eth_tree_data_t
 *
 * @brief tree data
 *
 * The Ethernet module stores the information about the trees that it may need
 * to manipulate at runtime. This struture has the information about any
 * Tree belonging to Ethernet module. It consists of the ID of the tree, input
 * pattern width, the default action and a flag to indicate if subnet data base
 * is enabled for the tree.
 *
 * =============================================================
 */

/*!
 * =============================================================
 * @internal
 * @enum adk_eth_time_slot_data
 *
 * @brief Data for time slot index management
 *
 * The Ethernet module stores the data for time slot index management. It is
 * required since time slot indices are shared for multiple ports in couple of
 * scenarios.
 *
 * =============================================================
 */
typedef struct adk_eth_time_slot_data_s
{
    uint32_t eioa0_slot1_namespace_id;
    uint32_t eioa1_slot1_namespace_id;
    uint32_t allocation_bit_map;
} adk_eth_time_slot_data_t;


/*!
 * =============================================================
 * @internal
 * @struct adk_ethernet_mac_fdb_dump_cb_t
 *
 * @brief  This structure is used to passed to the call back
 *         function invoked by Glib
 *         API for each valid MAC Table entry. This call back function
 *         filter the table according to the option and call the
 *         applications call back function.
 *
 * =============================================================
 */
typedef struct adk_ethernet_mac_fdb_dump_cb_s{
    adk_eth_mac_table_options_t  dump_options;
    adk_eth_mac_dump_cb          dumpCb;
    void                         *rock;
} adk_ethernet_mac_fdb_dump_cb_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_eth_vlan_info_t
 *
 * @brief  This structure is used to store details of interface
 *         created on a VLAN
 *
 * =============================================================
 */
typedef struct adk_eth_vlan_info_s {
    adk_interface_handle_t  if_handle;
    adk_interface_handle_t  vlan_if_handle;
    uint16_t                vlan_id;
    uint8_t                 vrd_id;
    uint32_t                stats_id;
    struct adk_eth_vlan_info_s *next;
}adk_eth_vlan_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_eth_bridge_src_mac_t
 *
 * @brief  This structure is used to store details of bridge
 *         source MAC for filtering
 *
 * =============================================================
 */
typedef struct
{
    uint64_t smac;
    adk_hos_rule_id_t filter_rule_id;
    uint8_t  ref_cnt;
} adk_eth_bridge_src_mac_t;


/*!
 * =============================================================
 * @internal
 * @enum adk_ethernet_internal_data_t
 *
 * @brief Ethernet module data
 *
 * This structure is the way to access any of the data maintained by Ethernet
 * module internally. It consists of current log-level, array of function
 * IDs, array of active aggregated interfaces, name space resources for
 * statistics and egress flows along with their handles. Using the Ethernet
 * module handle, the Ethernet module can access this internal data structure
 * and thus access any of the internal information needed to carry out API
 * operations.
 *
 * =============================================================
 */
/* Structure to hold all the Ethernet module specific internal data */
typedef struct adk_ethernet_internal_data_s
{
    uint32_t cookie;                    /* Ethernet module specific cookie */
    adk_eth_time_slot_data_t adk_eth_time_slot_data;
    adk_rm_hdl_t eth_flows_ns_hdl;
    adk_rm_hdl_t svif_id_ns_hdl;
    adk_rm_hdl_t bridgeIf_id_ns_hdl;
#ifdef ADK_DEVICE_SNR
    adk_hos_rule_id_t default_vsi_q_rule_id;
#endif
    void *macfdb_lock;
    /* Flag to indicate pending RCU sync for the cores */ 
    bool sync_pending[ADK_MAX_NUM_CORES];
    /* Last counter value for the cores at which the RCU sync was pending */
    uint32_t sync_cntr[ADK_MAX_NUM_CORES];
    adk_eth_list_t eth_list[ADK_ETH_MAX_LIST_ID];
    adk_eth_vlan_info_t  *vlan_intf_list[ADK_ETH_MAX_BG_ID];
    uint16_t                vlan_ref_cnt[ADK_ETH_MAX_VLAN_ID];
    adk_eth_bridge_src_mac_t  bridge_mac_list[ADK_ETH_MAX_BG_ID];
    adk_eth_dp_pvt_data_t *dp_pvt_p;
} adk_ethernet_internal_data_t;

typedef struct
{
    uint32_t bits;
    uint32_t single_bit_mask;
} adk_eth_range_ptn_info;


/* *********************************
******* FUNCTION PROTOTYPES *******
***********************************/

/* Top level private APIs to be called by other modules */
adk_status_t
adk_ethernet_sys_log (
    adk_module_handle_t eth_mod_handle,
    uint32_t log_level);

adk_status_t
adk_ethernet_get_imc (
    adk_module_handle_t eth_mod_handle,
    void *bindInfo_p,
    uint32_t *imcFnId);

adk_status_t
adk_ethernet_get_entry_func (
    adk_module_handle_t eth_mod_handle,
    void *bindInfo_p,
    uint32_t *entry_fn_id);

adk_status_t
adk_ethernet_custom_bind (
    adk_sys_handle_t sysHandle,
    adk_bind_info_t *bindInfo_p);

adk_status_t
adk_ethernet_custom_unbind (
    adk_sys_handle_t sysHandle,
    adk_bind_info_t *bindInfo_p);

/* Internal low level APIs to be called by Ethernet top level APIs */

bool adk_eth_check_rcu_sync(adk_module_handle_t eth_mod_handle);

bool adk_eth_check_rcu_status(adk_module_handle_t eth_mod_handle);

void adk_eth_wait_rcu_sync(adk_module_handle_t eth_mod_handle);

#define ADK_ETH_RCU_SYNCHRONIZE() \
    do { \
        if (adk_eth_check_rcu_sync(eth_mod_handle)) \
        { \
            adk_eth_wait_rcu_sync(eth_mod_handle); \
        } \
    } while (0)

/* Internal low level functions called for managing pending list of
 * objects to be freed */

adk_status_t
adk_eth_list_create(
    uint32_t max_elem,
    adk_eth_list_t *list_p);

adk_status_t
adk_eth_list_destroy(
    adk_eth_list_t *list_p);

void
adk_eth_list_add(
    adk_ethernet_internal_data_t *eth_internal_data_p,
    adk_eth_list_id_t list_id,
    uintptr_t data);

void
adk_eth_lists_clear(
    adk_ethernet_internal_data_t *eth_internal_data_p);

adk_status_t
adk_eth_list_dump(
    adk_eth_list_t *list_p);

adk_status_t adk_eth_log_level_set(adk_module_handle_t,
                                   adk_ethernet_log_level_t);

adk_status_t
adk_eth_if_owner (
    adk_module_handle_t eth_mod_handle,
    uint32_t if_num,
    char *vp_name,
    char *vlan_vp_name);

adk_status_t
adk_eth_inif_flags_stats (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t if_num,
    uint32_t stats_id,
    uint32_t flags);

adk_status_t
adk_eth_outif_flags_stats (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t if_num,
    uint32_t stats_id,
    uint32_t flags);

adk_status_t
adk_eth_link_tag_msb (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    uint32_t if_num,
    uint32_t *hash_start_offset,
    uint32_t *hash_stop_offset);

adk_status_t adk_eth_set_eioa_port_attr(adk_module_handle_t,
                                        uint32_t port,
                                        char *vp_name,
                                        char *vlan_vp_name);

adk_status_t
adk_eth_intf_flow_data_set (
    adk_module_handle_t eth_mod_handle,
    uint32_t if_id,
    const adk_ethernet_mac_addr_t src_mac,
    adk_ethernet_encap_t encap);

adk_status_t
adk_eth_intf_flow_data_clear (
    adk_module_handle_t eth_mod_handle,
    uint32_t if_id);

adk_status_t adk_eth_if_info_se_set(adk_module_handle_t,
                                    uint32_t ns_intf_flow_id,
                                    const adk_ethernet_mac_addr_t src_mac,
                                    adk_ethernet_encap_t);

adk_status_t adk_eth_if_info_se_reset(adk_module_handle_t,
                                      uint32_t ns_intf_flow_id);

adk_status_t
adk_eth_setup_static_port (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t in_port,
    uint32_t out_port);

adk_status_t
adk_eth_default_vlan (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_vlan_id_t vlan_id);

adk_status_t
adk_eth_cpu_param (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t input_key,
    adk_ph_pipeline_bind_handle_t ph_bind_handle);

adk_status_t
adk_eth_cpu_mtm_queue (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t input_key,
    uint32_t mtm_queue_id);

adk_status_t
adk_eth_arp_table (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_ipv4_addr_t ip_addr,
    const adk_ethernet_mac_addr_t dest_mac);

adk_status_t adk_eth_encap_flow_create(adk_module_handle_t,
                                       uint32_t flow_id,
                                       uint32_t if_num,
                                       const adk_ethernet_mac_addr_t dmac,
                                       adk_ethernet_mac_addr_t smac,
                                       adk_ethernet_proto_t proto,
                                       adk_ethernet_encap_t encap,
                                       bool dont_reset_stats);
adk_status_t
adk_eth_encap_flow_delete (
    adk_module_handle_t eth_mod_handle,
    uint32_t flow_id);

adk_status_t
adk_eth_encap_flow_data_set (
    adk_module_handle_t eth_mod_handle,
    uint32_t flow_id,
    const adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_mac_addr_t src_mac,
    adk_ethernet_proto_t proto,
    adk_ethernet_encap_t encap);

adk_status_t
adk_eth_encap_flow_data_clear (
    adk_module_handle_t eth_mod_handle,
    uint32_t flow_id);

adk_status_t
adk_eth_encap_flow_attr (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t flow_id,
    uint32_t if_num);

adk_status_t
adk_eth_encap_flow_dmac (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t flow_id,
    const adk_ethernet_mac_addr_t dest_mac);

adk_status_t
adk_eth_stag_eth_type (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t port,
    adk_ethernet_proto_t eth_type);

adk_status_t
adk_eth_ctag_stag_map (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t port,
    adk_ethernet_vlan_id_t c_tag,
    adk_ethernet_mac_subnet_t mac,
    adk_ethernet_proto_t eth_type,
    adk_ethernet_vlan_id_t s_tag);

adk_status_t adk_eth_vlan_id(adk_module_handle_t, adk_eth_tree_oper_t,
                             uint32_t port, adk_ethernet_vlan_id_t,
                             uint32_t sub_if_num, uint32_t stats_id);

adk_status_t adk_eth_out_vlan_id(adk_module_handle_t, adk_eth_tree_oper_t,
                                 uint32_t port, adk_ethernet_bg_id_t,
                                 adk_ethernet_vlan_id_t vlan_id);

adk_status_t adk_eth_map_subif_ns_id(adk_module_handle_t, adk_eth_tree_oper_t,
                                     uint32_t sub_if_num,
                                     uint32_t stats_id);

adk_status_t
adk_eth_map_ns_intf_id (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t stats_id,
    uint32_t intf_id);

adk_status_t
adk_eth_switch_virtual_intf (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_vlan_id_t stag,
    adk_ethernet_vlan_id_t ctag,
    uint32_t v_if_num,
    uint32_t stats_id);

adk_status_t
adk_eth_switch_virtual_intf_tags (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t v_if_num,
    adk_ethernet_vlan_id_t stag,
    adk_ethernet_vlan_id_t ctag);

adk_status_t adk_eth_recv_mac(adk_module_handle_t, adk_eth_tree_oper_t,
                              uint32_t port, adk_ethernet_mac_subnet_t,
                              uint32_t ret_flags);

adk_status_t
adk_eth_mac_addr (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_mac_addr_t mac,
    uint32_t port);

adk_status_t adk_eth_promisc_mode(adk_module_handle_t, uint32_t port,
                                  bool enable);

adk_status_t
adk_eth_ingress_flow (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t if_num,
    uint32_t mask,
    adk_ethernet_proto_t proto,
    adk_ethernet_llc_hdr_t llc_header,
    adk_ethernet_mac_subnet_t dmac,
    const adk_ethernet_ingress_flow_action_t *,
    bool apply_extended_filter,
    uint32_t flow_id);

adk_status_t
adk_eth_cpu_flow (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_eth_flow_internal_data_t *flow_data_p);

adk_status_t
adk_eth_default_association (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t if_num);


adk_status_t adk_eth_link_aggregation(adk_module_handle_t,
                                      adk_eth_tree_oper_t,
                                      adk_eth_pkt_oper_t,
                                      uint32_t port);

adk_status_t adk_eth_aggr_link(adk_module_handle_t,
                               adk_eth_tree_oper_t,
                               uint32_t aggr_port,
                               uint32_t member_link);

adk_status_t adk_eth_constituent_link(adk_module_handle_t,
                                      adk_eth_tree_oper_t,
                                      uint32_t aggr_port,
                                      uint32_t key,
                                      uint32_t selected_port);

adk_status_t
adk_eth_mtu (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    uint16_t mtu);

adk_status_t
adk_eth_max_pkt_size (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    uint16_t mtu);

adk_status_t adk_eth_if_tx_enable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_tx_disable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_rx_enable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_rx_disable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_allow_mac(adk_module_handle_t, uint32_t port,
                               bool promiscuous_mode,
                               const adk_ethernet_mac_addr_t src_mac,
                               const adk_ethernet_mac_addr_t aggr_mac,
                               const adk_ethernet_mac_addr_t allow_mac);

adk_status_t adk_eth_port_stats_get(adk_module_handle_t, uint32_t port,
                                    adk_ethernet_port_statistics_t *);

adk_status_t adk_eth_port_stats_reset(uint32_t port);

adk_status_t adk_eth_clear_se_state(adk_module_handle_t, uint32_t if_ns_id,
                                    uint32_t size);

adk_status_t adk_eth_global_stats_get(adk_module_handle_t, uint32_t ns_id,
                                      adk_ethernet_global_statistics_t *);

adk_status_t adk_eth_if_full_duplex_set(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_full_duplex_reset(adk_module_handle_t,
                                          uint32_t port);

adk_status_t adk_eth_if_autoneg_enable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_autoneg_disable(adk_module_handle_t, uint32_t port);

adk_status_t adk_eth_if_autoneg_force(adk_module_handle_t, uint32_t port);

adk_status_t
adk_eth_eioa_vlan_eth_type (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    adk_ethernet_proto_t *eth_type);

adk_status_t
adk_eth_if_time_stamp_get (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    uint32_t ts_index,
    uint32_t *time_stamp);

adk_status_t
adk_eth_if_flow_control_set (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    bool enable,
    uint32_t pause_type,
    uint16_t time);

adk_status_t
adk_eth_if_loopback (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    bool enable);

adk_status_t
adk_eth_if_src_mac_set (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    const adk_ethernet_mac_addr_t mac_addr);

bool
adk_eth_valid_handle (
    adk_module_handle_t eth_mod_handle);

bool
adk_eth_valid_if_handle (
    adk_interface_handle_t if_handle);

bool
adk_eth_valid_flow_handle (
    adk_flow_handle_t flow_handle);

adk_status_t
adk_eth_map_sys_log (
    adk_module_handle_t eth_mod_handle,
    adk_log_level_t sys_log_level,
    adk_ethernet_log_level_t *eth_log_level);

adk_status_t
adk_eth_map_eth_log (
    adk_module_handle_t eth_mod_handle);

adk_status_t
adk_eth_is_routed_intf (
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle);

uint32_t
adk_eth_create_bridge_if_id(uint32_t ns_id);

uint32_t
adk_eth_get_bridge_if_id(uint32_t if_id);

uint32_t
adk_eth_create_subif_id (
    uint32_t port,
    adk_ethernet_vlan_id_t vlan_id);

uint32_t
adk_eth_create_virtual_if_id (
    uint32_t ns_id);

uint32_t
adk_eth_get_virtual_if_ns_id (
    uint32_t if_id);

adk_status_t
adk_eth_valid_ingress_flow_data (
    adk_module_handle_t eth_mod_handle,
    const adk_ethernet_ingress_flow_data_t *in_flow_data);

adk_status_t
adk_eth_valid_ingress_flow_action (
    adk_module_handle_t eth_mod_handle,
    const adk_ethernet_ingress_flow_data_t *in_flow_data,
    const adk_ethernet_ingress_flow_action_t *in_flow_action);

adk_status_t
adk_eth_valid_association_flow_action (
    adk_module_handle_t eth_mod_handle,
    bool apply_extended_filter,
    const adk_ethernet_ingress_flow_action_t *in_flow_action);

adk_status_t
adk_eth_fpp_range (
    uint32_t data,
    uint32_t data_width,
    uint32_t *ptn_data,
    uint32_t *no_bits,
    bool *last_ptn,
    void **rock);

const char *
adk_eth_intf_type_name (
    adk_ethernet_intf_type_t type);

const char *
adk_ethernet_encap_type_name (
    adk_ethernet_encap_t encap_type);

const char *
adk_eth_flow_type_name (
    adk_ethernet_flow_type_t flow_type);

const char *
adk_eth_flow_action_name (
    adk_ethernet_ingress_flow_action_type_t flow_action);

const char *
adk_eth_error_name (
    adk_status_t err);

int8_t
adk_eth_map_port_to_index (
    uint8_t port);

adk_status_t
adk_eth_custom_bind_ingress_flow (
    adk_module_handle_t eth_mod_handle,
    uint32_t if_num,
    adk_ethernet_proto_t proto,
    adk_ethernet_mac_subnet_t dmac,
    adk_eth_custom_bind_type_t bind_type,
    uint32_t flow_id,
    uint32_t bind_data);

adk_status_t
adk_eth_custom_bind_hash_flow (
    adk_module_handle_t eth_mod_handle,
    uint32_t if_num,
    uint16_t eth_type,
    bool proto_only,
    uint32_t hash_value,
    adk_eth_custom_bind_type_t bind_type,
    uint32_t flow_id,
    uint32_t bind_data);

adk_status_t adk_eth_custom_bind_interface(adk_module_handle_t eth_mod_handle,
                                           adk_eth_tree_oper_t,
                                           uint32_t flow_id,
                                           uint32_t f_pl_func_id);

/* ****************************************************
******* FUNCTIONS SPECIFIC TO DATAPLANE TABLES *******
******************************************************/

adk_dp_status_t
adk_eth_dp_init (
    adk_sys_handle_t sysHandle,
    adk_module_handle_t eth_mod_handle);

adk_dp_status_t
adk_eth_dp_table_init (
    adk_module_handle_t eth_mod_handle,
    adk_eth_dp_pvt_data_t *eth_dp_pvt_p);

adk_dp_status_t
adk_eth_dp_table_shutdown (
    adk_module_handle_t eth_mod_handle,
    adk_eth_dp_pvt_data_t *eth_dp_pvt_p);

adk_dp_status_t
adk_eth_dp_svif_add (
    adk_module_handle_t eth_mod_handle,
    adk_dp_eth_svif_if_hash_t *svif_if_hash_table_p,
    adk_dp_eth_svif_intf_t *dp_svif_intf_p);

adk_dp_status_t
adk_eth_dp_svif_rem (
    adk_module_handle_t eth_mod_handle,
    adk_dp_eth_svif_if_hash_t *svif_if_hash_table_p,
    adk_dp_eth_svif_intf_t *dp_svif_intf_p);

adk_status_t
adk_eth_dp_ingress_flow (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_eth_flow_internal_data_t *flow_data_p);

adk_status_t
adk_eth_dp_arp (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_ipv4_addr_t ip_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_arp_status_t arp_status);

adk_status_t
adk_eth_dp_arp_get (
    adk_dp_eth_arp_table_t *eth_arp_table,
    adk_ethernet_ipv4_addr_t ip_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_arp_status_t* arp_status);

adk_status_t
adk_eth_dp_arp_entries_get (
        adk_dp_eth_arp_table_t *eth_arp_table,
        uint32_t *numArpEntries, 
        adk_ethernet_arp_entry_t **arp_entries_p);

adk_status_t
adk_eth_dp_interface_stats_get (
    adk_module_handle_t enetHandle,
    uint32_t interfaceID,
    adk_eth_intf_internal_data_t *if_data_p,
    void *stats);

adk_status_t
adk_eth_dp_global_stats_get (
    adk_module_handle_t enetHandle,
    adk_ethernet_global_statistics_t *stats);

void
adk_eth_dp_timer_notify (
    void *rock);

adk_status_t
adk_eth_dp_neigh (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_ipv6_addr_t *ipv6_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_neigh_status_t neigh_status);

adk_status_t
adk_eth_dp_neigh_get (
    adk_module_handle_t eth_mod_handle,
    adk_ethernet_ipv6_addr_t *ipv6_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_neigh_status_t *neigh_status);

adk_status_t
adk_eth_dp_neighbor_entries_get (
        adk_dp_eth_neighbor_table_t *eth_neigh_table,
        uint32_t *numNeighEntries,
        adk_ethernet_neighbor_entry_t **neigh_entries_p);

adk_status_t 
adk_eth_map_proto_to_pcp(
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    adk_ethernet_proto_t eth_type,
    adk_ethernet_priority_t priority);

adk_status_t 
adk_eth_map_ethertype_to_queue(
        adk_module_handle_t eth_mod_handle,
        adk_interface_handle_t ifHandle,
        adk_eth_tree_oper_t oper,
        adk_ethernet_proto_t ethType,
        adk_qos_queue_id_t  queueId);
/* *******************************************
******* HW HELPER FUNCTION PROTOTYPES *******
*********************************************/

adk_status_t
adk_eth_pcx_port_learning_options_set (
    adk_module_handle_t eth_mod_handle,
    uint8_t port,
    bool learning_enabled);

adk_status_t
adk_eth_pcx_port_ingress_vlan (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint8_t port,
    uint16_t cvlanId,
    uint16_t svlanId,
    bool isOne2OneSvlan);

adk_status_t
adk_eth_pcx_port_flooding_option_set (
    adk_module_handle_t eth_mod_handle,
    uint8_t port,
    bool flooding_enabled);

adk_status_t
adk_eth_set_eioa_port_pcx_mode (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    bool pcx_mode);

adk_status_t
adk_eth_set_eioa_port_pcx_attr (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    bool enable_pcx_switching,
    bool use_bridging_vps);

adk_status_t
adk_eth_set_eioa_pcx_bridging_vps (
    adk_module_handle_t eth_mod_handle);

adk_status_t
adk_eth_set_eioa_pcx_bridging_dest_vps (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t dest_idx);

adk_status_t
adk_eth_set_eioa_port_vp (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    char *vp_name,
    char *vlan_vp_name);

adk_status_t
adk_eth_pcx_bridge_table_entry (
    adk_module_handle_t eth_mod_handle,
    adk_eth_tree_oper_t oper,
    uint32_t if_num,
    adk_ethernet_mac_addr_t mac,
    uint32_t vlan_id,
    bool isOne2OneSvlan);

adk_status_t
adk_eth_set_eioa_port_default_vlan (
    adk_module_handle_t eth_mod_handle,
    uint32_t port,
    uint32_t vlan_id);

adk_status_t
adk_ethernet_shutdown (
    adk_module_handle_t eth_mod_handle);

adk_status_t
adk_ethernet_ph_rx_parse (
    void *task,
    void **dp_message,
    adk_module_handle_t eth_mod_handle);

#ifdef ADK_NETD_NETLINK
adk_status_t netd_bridge_interface_up(adk_module_handle_t eth_mod_handle);

adk_status_t netd_bridge_interface_create
(
adk_module_handle_t eth_mod_handle,
const uint8_t* bridge_mac
);

adk_status_t netd_bridge_interface_delete(adk_module_handle_t eth_mod_handle);

int netlink_chk_recv_ack(int sock_fd);

#endif

#endif /* #ifndef _ADK_ETH_H_ */

/*! end addtogroup _ethernet
 * @}
 */
