/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth_constants.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernet  Ethernet Module
 *
 * @{
 *
 * @brief   This file has the constant definitions which are used in both
 *          control plane software and data plane software.
 *
 *  **********************************************************************/

#include <adk_ethernet_conf.h>

#ifndef _ADK_ETH_CONSTANTS_H_
#define _ADK_ETH_CONSTANTS_H_

/* Constants */

#define ADK_ETHERNET_MODULE_COOKIE          0x721AD430
#define ADK_ETHERNET_INTF_COOKIE            0x721AD41F
#define ADK_ETHERNET_FLOW_COOKIE            0x721AD4f0
#define ADK_MCAST_GROUP_COOKIE              0xABCD7856
#define ADK_ETH_STATS_DONOT_CLEAR_ON_READ   0
#define ADK_ETH_DO_LOOP_UNTIL_BREAK     1
#define ADK_ETH_EIOA_HASH_LEN_WIDTH         4
#define ADK_ETH_EIOA_HASH_FLAG_WIDTH        4
#define ADK_ETH_EIOA_HASH_WIDTH             16
#define EXCEPTION_FALSE                     0xFF
#define ETH_INVALID_QOS_QID                 0xFFFF
#define ADK_ETH_INVALID_PORT_INDEX          -1
#define ADK_ETH_UNKNOWN_SVIF                255
#define ADK_ETH_UNKNOWN_VRD                 63

#define ADK_ETH_DP_OBJECT_ALIGN             64
#define ADK_ETH_DP_TABLE_ALIGN              64
#define ADK_ETH_DP_TABLE_ELEM_ALIGN         16
#define ADK_ETH_DP_PTR_ALIGN                8
#define ADK_ETH_DP_NO_ALIGN                 0

/* Compressed bridge MAC addr */
#define ADK_ETH_BRIDGE_MAC_MAPPER_START           1
#define ADK_ETH_BRIDGE_MAC_UNICAST_CMAC_START     ADK_ETH_BRIDGE_MAC_MAPPER_START
#define ADK_ETH_BRIDGE_MAC_UNICAST_CMAC_1         ADK_ETH_BRIDGE_MAC_UNICAST_CMAC_START
#define ADK_ETH_BRIDGE_MAC_UNICAST_CMAC_END       20
#define ADK_ETH_BRIDGE_MAC_M_B_CMAC_START         (ADK_ETH_BRIDGE_MAC_UNICAST_CMAC_END + 1)
#define ADK_ETH_BRIDGE_MAC_M_B_CMAC_1             ADK_ETH_BRIDGE_MAC_M_B_CMAC_START
#define ADK_ETH_BRIDGE_MAC_M_B_CMAC_2             (ADK_ETH_BRIDGE_MAC_M_B_CMAC_1 + 1)
#define ADK_ETH_BRIDGE_MAC_M_B_CMAC_3             (ADK_ETH_BRIDGE_MAC_M_B_CMAC_2 + 1)
#define ADK_ETH_BRIDGE_MAC_MAPPER_END             96

#define ADK_ETH_LIST_MAX_SIZE               100

#define ADK_ETH_RCU_POLL_PERIOD             1   /* in micro-secs */

/* default for non timestamp applications */
#define ADK_ETH_TS_NOTIMESTAMP               0

/* Allocate a TS index */
#define ADK_ETH_TS_MAP_REQ                   1

/* Allocation successfull */
#define ADK_ETH_TS_MAP_SUCCESS               2

/* Allocation failed */
#define ADK_ETH_TS_MAP_FAILURE               3

/* Free the TS map */
#define ADK_ETH_TS_MAP_FREE              4

/* For future */
#define ADK_ETH_TS_MAP_ABORT                 5

/* No free index */
#define ADK_ETH_NO_TS_FREE_INDEX            64

/* Max indices per slot */
#define ADK_ETH_MAX_PORT_TS_INDICES              64

/* Max stats types for a physical port */
#define ADK_ETH_MAX_STATS               200
#define ADK_ETH_STATS_NAME_SIZE         10

/* All below widths are in bits */

#define ETH_PHY_PORT_WIDTH              8
#define ETH_PHY_PCX_PORT_WIDTH          6
#define ADK_ETH_AGGR_INTF_MAX           4
/* Size of the array storing LAG member links. This is LCM (Least Common
 * Multiple) of 1, 2, .., ADK_ETH_MAX_INTF_IN_AGGR */
#define ETH_LAG_LINK_IDX_MAX            12
#define ETH_INVALID_PHY_PORT            0xFF

#define ADK_ETH_MIN_PKT_SIZE        64
#define ADK_ETH_MAC_ADDR_SIZE       6
#define ADK_ETH_CRC_SIZE            4

#define VLAN_ID_WIDTH               12
#define INPUT_INTF_WIDTH            (VLAN_ID_WIDTH + ETH_PHY_PORT_WIDTH)

#define VLAN_SUB_IF_WIDTH           12
#define ADK_ETH_SUB_INTF_MAX        (1 << VLAN_SUB_IF_WIDTH)
#define ADK_ETH_SUB_INTF_MASK       (ADK_ETH_SUB_INTF_MAX - 1)

#define ADK_ETH_SVIF_INTF_MAX       (1 << VLAN_SUB_IF_WIDTH)
#define ADK_ETH_SVIF_INTF_MASK      (ADK_ETH_SVIF_INTF_MAX - 1)  /* For now, let's asssume these many */
                                                              /* switch virtual interfaces are supported */  
#define ADK_ETH_NO_OF_NEIGH_ENTRY     1024
#define ADK_ETH_NO_OF_NEIGH_COLLISON_ENTRY  8
#define ADK_ETH_NEIGHBOR_HASH_MASK (ADK_ETH_NO_OF_NEIGH_ENTRY-1)

#define SUB_INTF_FLOW_WIDTH         10
#define ADK_ETH_SUB_INTF_FLOW_MAX   (1 << SUB_INTF_FLOW_WIDTH)  /* Number of flows per VLAN sub intf */
#define ADK_ETH_SUB_INTF_FLOW_MASK  (ADK_ETH_SUB_INTF_FLOW_MAX - 1)

#define ETHTYPE_QUEUE_WIDTH          8
#define ADK_ETH_ETHTYPE_QUEUE_MAP_MAX       (1 << ETHTYPE_QUEUE_WIDTH)
#define ADK_ETH_ETHTYPE_QUEUE_MAP_MASK      (ADK_ETH_ETHTYPE_QUEUE_MAP_MAX - 1)

/* Tree width definitions for CPU Flow lookup table */
#define LLC_HDR_WIDTH               24
#define LINUX_INTF_WIDTH            16
#define VLAN_ID_WIDTH               12

#define MAX_DSCP                    (ADK_ETH_MAX_DSCP + 1)
#define ADK_ETH_MAX_PCP             (ADK_ETH_MAX_PRIORITY + 1)
#define MAX_L2CP_TYPES              10

#define FLAG_WIDTH                  1
#define EXCEPTION_TYPE_WIDTH        8
#define ETH_INTF_NS_ID_WITDH        24
#define CPU_PATH_KEY_WIDTH          24
#define FWD_MODE_WIDTH              3
#define VLAN_ASSIGN_POLICY_WIDTH    3
#define ETH_TYPE_WIDTH              16
#define LLC_HDR_WIDTH               24
#define OUI_HDR_WIDTH               24
#define ETH_FLOW_ID_WIDTH           24
#define MAC_ADDR_WIDTH              48
#define TAG_POSITION_WIDTH          2
#define PRIORITY_WIDTH              3
#define DSCP_WIDTH                  8
#define PKT_LENGTH_WIDTH            16
#define VLAN_TAG_LEVEL_WIDTH        2
#define IPV4_ADDRESS_WIDTH          32
#define EGRESS_PATH_TYPE_WIDTH      4
#define LINK_AGGREGATION_KEY_WIDTH  4
#define TIME_SLOT_BIT_MAP_WIDTH     16
#define TIME_SLOT_INDEX_WIDTH       8

/* Upper and Lower Limits */
#define ADK_ETH_INTF_MAX                    1024
#define ADK_ETH_PHY_INTF_MAX                (1 << ETH_PHY_PORT_WIDTH)
#define ADK_ETH_NUM_FLOWS_MAX               2048
#define ADK_ETH_ENCAP_VP_FLOW_ID_MAX        4096
#define ADK_ETH_MIN_PROTO_TYPE              0x600
#define ADK_ETH_MAX_VLAN_ID                 0xFFF
#define ADK_ETH_MAX_BG_ID                   64
#define ADK_ETH_BG_ID_OFFSET                256
#define ADK_ETH_MIN_ETH_TYPE                0x600
#define ADK_ETH_MAX_PRIORITY                0x7
#define ADK_ETH_MAX_DSCP                    0x3f
#define ADK_ETH_MTU_MIN                     64
#define ADK_ETH_MTU_MAX                     10000
#define ADK_ETH_EIOA_MAX_TX_INDEX           64
#define ADK_ETH_MAX_HASH_LENGTH             60
#define ADK_ETH_EIOA0_SLOT1_MIN_PORT_INDEX   1
#define ADK_ETH_EIOA0_SLOT1_MAX_PORT_INDEX   4
#define ADK_ETH_EIOA1_SLOT1_MIN_PORT_INDEX   6
#define ADK_ETH_EIOA1_SLOT1_MAX_PORT_INDEX   9
#define ADK_ETH_MAX_TIME_SLOT_INDICES       2048

/* Masks */
#define ETH_BYTE_MASK       0xFF
#define ETH_2BYTE_MASK      0xFFFF
#define ETH_3BYTE_MASK      0xFFFFFF
#define ETH_PHY_PORT_MASK   ((1 << ETH_PHY_PORT_WIDTH) - 1)
#define VLAN_ID_MASK        ((1 << VLAN_ID_WIDTH) - 1)
#define ADK_ETH_MAC_ADDR_MULTICAST_MASK     0x1
#define MAC_ADDR_MASK       0xFFFFFFFFFFFF
#define MULTICAST_MAC_ADDR_MASK 0x010000000000
#define MULTICAST_MAC_ADDR 0x010000000000
#define MULTICAST_MAC_ADDR_MASK_BITS            8
#define MULTICAST_MAC_ADDR_V6                   0x333300000000
#define MULTICAST_MAC_ADDR_V6_MASK_BITS         16
#define BROADCAST_MAC_ADDR                      0xFFFFFFFFFFFF
#define BROADCAST_MAC_ADDR_MASK_BITS            48
#define UNICAST_MAC_ADDR_MASK_BITS              48
#define ETH_PRI_MASK                ((1 << PRIORITY_WIDTH) - 1)
#define ETH_PRI_VALID_FLAG          (1 << PRIORITY_WIDTH)
#define ETH_PRI_MARK_FLAG           (1 << (PRIORITY_WIDTH + 1))

/* PCP based Rx Counters */
#define VLAN_BYTES_RCVD            0x00
#define VLAN_PKTS_RCVD             0x08
#define PCP_RX_COUNTER_SIZE        0x10

/* VLAN based Rx Counters */
#define BYTES_RCVD                 0x00
#define UNICAST_PKTS_RCVD          0x08
#define MULTICAST_PKTS_RCVD        0x0C
#define BROADCAST_PKTS_RCVD        0x10

/* VLAN based Tx Counters */
#define UNICAST_PKTS_TX            0x14
#define BYTES_TX                   0x18

/* Rx Counters */
#define PKTS_RCVD                   0x00
#define PKTS_BOUNDING               0x04
#define IPV4_PKTS_DIRECT_BIND       0x08
#define IPV6_PKTS_DIRECT_BIND       0x0C
#define IPV4_PKTS_INDIRECT_BIND     0x10
#define PTP_PKTS_DIRECT_BIND        0x14
#define LCP_PKTS_DIRECT_BIND        0x18
#define STATIC_FWD_NOT_SETUP        0x1C
/* Tx Counters */
#define ENCAP_PKTS                  0x20
#define PKTS_SENT                   0x24
/* Rx Drop Counters */
#define NO_STAG_MAPPING             0x2C
#define UNKNOWN_PORT                0x30
#define NO_MULTICAST_GROUP          0x34
#define INVALID_VLAN_TAG            0x38
#define UNSUPPORTED_IEEE_802_3_PKT  0x3C
#define UNKNOWN_ASSOCIATION         0x40
#define ASSOCIATION_RULE_DROP       0x44
#define UNKNOWN_DEST_MAC            0x48
#define ETH_RUNT_PKT                0x4C
/* Tx Drop Counters */
#define ETH_MTU_FAILED              0x50
#define UNRESOLVED_ARP              0x54
#define NO_OUTPUT_QOS_POLICY        0x58
#define NO_CUSTOM_BIND_FN           0x5C
/* Rx-Tx Drop Counters */
#define BINDING_ERROR               0x60
#define INVALID_NUM_VLAN_TAGS       0x64
#define NO_FLOW_ID_DEFINED          0x68
#define NO_ENCAP_FLOW_FOUND         0x6C
#define PDUS_DISCARDED              0x70
#define OAM_PKTS_DIRECT_BIND        0x74
#define RED_COLORED_PKT             0x78
/* Misc Counters */
#define SRC_MAC_EXPIRED             0x7C
#define AGGR_LINK_REMOVED           0x80
#define STP_DISABLED                0x84
#define STP_FILTERED                0x88
#define SMAC_GROUP_ADDR             0x8C
#define UNKNOWN_VIRTUAL_INTF        0x90

#define TOTAL_COUNTER_SPACE         0x100

/* Global Counters */
#define NO_SUB_INTF_NS_ID_MAPPING   0x00
#define INVALID_EGRESS_ENCAP_TYPE   0x04
#define STATIC_MAC_ENTRY_NONEXIST   0x08
#define NO_CPU_PATH_PARAM           0x0C
#define HASH_OPERATION_FAILED       0x10
#define HASH_STOPPING_EXP_TIMER     0x14
#define HASH_NEW_ENTRY_ADDED        0x18
#define PKTS_SENT_TO_PH             0x1C
#define PKTS_SENT_TO_NCA            0x20
#define NO_PORT_NS_ID_MAPPING       0x24
#define PKTS_RCVD_FROM_NCA          0x28
#define NO_ENCAP_FLOW_DMAC          0x2C
#define NO_LINK_IN_AGGREGATE        0x30
#define UNKNOWN_NS_ID               0x34
#define CPU_PKTS_IPV4_DIRECT_BIND   0x38
#define TIME_INDEX_RELATED_FAILURE  0x3C

/* All sizes below are in bytes */
#define ETH_MAC_ADDR_SIZE           6
#define ETH_VLAN_ID_SIZE            2
#define ETH_VLAN_TPID_SIZE          2
#define ETH_VLAN_HDR_SIZE           4
#define ETH_TYPE_SIZE               2

/* Offsets in the SE memory for egress flow */
#define SRC_MAC_OFFSET              0
#define ETH_ENCAP_OFFSET            7
#define INTF_INFO_SE_MEM_SIZE       8

#define DEST_MAC_OFFSET             8
#define ETH_PROTO_OFFSET            14
#define INTF_NUM_OFFSET             16
#define FLOW_ID_OFFSET              18
#define FLOW_RX_COUNTER_OFFSET      20
#define FLOW_STATS_SIZE             4
#define FLOW_SE_MEM_SIZE            24


/* Various flags */

#define TOP_VLAN_TAG            0x0
#define SECOND_VLAN_TAG         0x1

#define EGRESS_TYPE_ETH_SWITCH  0x0
#define EGRESS_TYPE_IPV4_ENCAP  0x1

#define ACTUAL_VLAN_TAG         0x0
#define ASSIGNED_VLAN_TAG       0x1

/* Flag to enable checks for hairpin path while switching */
#define ETH_IGNORE_LOOP         0x0
#define ETH_CHECK_LOOP          0x1

#define ENCAP_NO_L3_PIC         0x0
#define ENCAP_VIA_L3_PIC        0x1

/* VLAN TAG Present/Absent definition */
#define PRESENT                         0
#define ABSENT                          1

#define DIX_ENCAP                       0
#define SNAP_ENCAP                      1

#define ETH_ASSOCIATION_RULE_CONTINUE       0x0
#define ETH_ASSOCIATION_RULE_SEND_TO_PORT   0x1
#define ETH_ASSOCIATION_RULE_BIND           0x2
#define ETH_ASSOCIATION_RULE_SEND_TO_PH     0x3
#define ETH_ASSOCIATION_RULE_SEND_TO_NCA    0x4
#define ETH_ASSOCIATION_RULE_DROP           0x5
/* Actions not exposed to application */
#define ETH_ASSOCIATION_RULE_CUSTOM_BIND    0xF

#define ETH_EXCEPTION_ACTION_SEND_TO_PORT   0x1
#define ETH_EXCEPTION_ACTION_SEND_TO_CPU    0x2
#define ETH_EXCEPTION_ACTION_SEND_TO_NCA    0x3

#define HASH_OPER_DELETE                0
#define HASH_OPER_ALLOCATE              1
#define HASH_OPER_INSERT_UPDATE         2
#define HASH_OPER_STOP_TIMER            3

#define PER_PORT_FLAG                   0x0
#define SYSTEM_WIDE_FLAG                0x1
#define ACL_OFF_FLAG                    0x0
#define ACL_ON_FLAG                     0x1
#define OVERRIDE_OFF_FLAG               0x0
#define OVERRIDE_ON_FLAG                0x1
#define INGRESS_FLAG                    0x0
#define EGRESS_FLAG                     0x1
#define INGRESS_STP_FLAG                0x0
#define EGRESS_STP_FLAG                 0x1
#define INGRESS_POLICE_FLAG             0x0
#define EGRESS_POLICE_FLAG              0x1
#define NON_RED_COLOR_FLAG              0x0
#define RED_COLOR_FLAG                  0x1
#define INGRESS_PRI_MAP_FLAG            0x0
#define EGRESS_PRI_MAP_FLAG             0x1

#define ETH_PORT_BASED_VLAN             0x0
#define ETH_PORT_PROTO_BASED_VLAN       0x1
#define ETH_PORT_LLC_BASED_VLAN         0x2
#define ETH_PORT_SMAC_BASED_VLAN        0x3
#define ETH_PORT_PRI_SMAC_BASED_VLAN    0x4

#define ETH_PORT_BASED_VLAN_MAX         ETH_PORT_PRI_SMAC_BASED_VLAN

#define ETH_PROTO_BASED_VLAN            0x5
#define ETH_LLC_BASED_VLAN              0x6
#define ETH_SMAC_BASED_VLAN             0x7

#define ETH_POLICING_DISABLED           0x0
#define ETH_POLICING_TRTCM_COLOR_BLIND  0x1
#define ETH_POLICING_TRTCM_COLOR_AWARE  0x2
#define ETH_POLICING_SRTCM_COLOR_BLIND  0x3
#define ETH_POLICING_SRTCM_COLOR_AWARE  0x4

#define ADK_ETHERNET_INVALID_BRIDGE_ID  0xFF

/* Bit positions of information in value returned by destination MAC lookup
 * tree */
#define SOURCE_ADDR_BIT                 2
#define BROADCAST_MAC_BIT               1
#define MULTICAST_MAC_BIT               0

/* Bit positions of information in iParam0 global register */
#define MSB_TAG_INTF_NUM_BIT            0
#define MSB_TAG_PHY_PORT_BIT            8

/* Bit positions of information in %INPUT_INTERFACE */
#define ETH_PHY_PORT_BIT                0
#define FWD_MODE_BIT                    (ETH_PHY_PORT_BIT + ETH_PHY_PORT_WIDTH)     /* 4, 8 */
#define PCP_BASED_INTF_STATS_BIT        (FWD_MODE_BIT + FWD_MODE_WIDTH)             /* 7, 11 */
#define VLAN_BASED_INTF_STATS_BIT       (PCP_BASED_INTF_STATS_BIT + FLAG_WIDTH)     /* 8, 12 */
#define INGRESS_POLICING_BIT            (VLAN_BASED_INTF_STATS_BIT + FLAG_WIDTH)    /* 9, 13 */
#define EGRESS_POLICING_BIT             (VLAN_BASED_INTF_STATS_BIT + FLAG_WIDTH)    /* 9, 13 */
#define INGRESS_POLICING_TYPE           (INGRESS_POLICING_BIT + FLAG_WIDTH)         /* 10, 14 */
#define EGRESS_POLICING_TYPE            (EGRESS_POLICING_BIT + FLAG_WIDTH)          /* 10, 14 */
#define INGRESS_PRI_MAPPING_BIT         (INGRESS_POLICING_TYPE + FLAG_WIDTH)        /* 11, 15 */
#define EGRESS_PRI_MAPPING_BIT          (EGRESS_POLICING_TYPE + FLAG_WIDTH)         /* 11, 15 */
#define ENCAP_TYPE_BIT                  (EGRESS_PRI_MAPPING_BIT + FLAG_WIDTH)       /* 12, 16 */
#define INGRESS_FLOW_FILTER_BIT         (ENCAP_TYPE_BIT + FLAG_WIDTH)               /* 13, 17 */
#define EGRESS_FLOW_FILTER_BIT          (ENCAP_TYPE_BIT + FLAG_WIDTH)               /* 13, 17 */
#define STP_CNTRL_BIT                   (EGRESS_FLOW_FILTER_BIT + FLAG_WIDTH)       /* 14, 18 */
#define VLAN_ASSIGN_POLICY_BIT          (STP_CNTRL_BIT + FLAG_WIDTH)                /* 15, 19 */
#define INGRESS_DSCP_BASE_Q_SEL         (VLAN_ASSIGN_POLICY_BIT + FLAG_WIDTH)
/* 16, 20 */
#define EGRESS_DSCP_BASE_Q_SEL         (VLAN_ASSIGN_POLICY_BIT + FLAG_WIDTH)
/* 16, 20 */

/* Ethernet Interface FWD modes */
#define FWD_MODE_INVALID                0
#define FWD_MODE_ETH_SWITCH_ACCESS      1
#define FWD_MODE_ETH_SWITCH_TRUNK       2
#define FWD_MODE_ETH_SWITCH_TUNNEL      3
#define FWD_MODE_HIGHER_LAYER           4
#define FWD_MODE_STATIC                 5
#define FWD_MODE_HL_HASH_DIX            6

/* Pipeline flows for NCA path */
#define ETH_NCA_PATH_NO_MTM_FLOW_ID     0
#define ETH_NCA_PATH_VIA_MTM_FLOW_ID    1

#define ETH_VLAN_HDR_LEN           4
#define ETH_NO_VLAN_HEADER_LEN    14
#define ETH_VLAN_HEADER_LEN       18
#define ETH_HDR_VLAN_HDR_OFFSET   12
#define ETH_HDR_VLAN_ID_OFFSET    14
#define ETHER_TYPE_VLAN       0x8100
#define ETHER_TYPE_IPV4       0x0800
#define ETHER_TYPE_IPV6       0x86DD
#define ETHER_TYPE_SIZE            2
#define ETH_IP_PROTO_TCP           6
#define ETH_IP_PROTO_UDP          17
#define ETH_IP_PROTO_SCTP        132

#define ETH_IPV4_HDR_PROTO_OFFSET   9
#define ETH_IPV4_HDR_ADDR_OFFSET    12
#define ETH_IPV6_HDR_PROTO_OFFSET   6
#define ETH_IPV6_HDR_SADDR_OFFSET   8
#define ETH_IPV6_HDR_DADDR_OFFSET   24
#define ETH_IPV4_ADDR_SIZE          4
#define ETH_L4_PORT_SIZE            2

/* Different flags for Ethernet egress encapsulation processing */
#define ETH_SEND_UNMODIFIED             1
#define ETH_ADD_HEADER                  0
#define ETH_USE_SED                     1
#define ETH_CAN_USE_PQM                 0
#define ETH_VIA_MTM                     1
#define ETH_BYPASS_MTM                  0
#define ETH_ARP_BASED_ENCAP             1
#define ETH_GENERIC_ENCAP               0
#define ETH_VIA_CUSTOM_SED              1
#define ETH_NO_CUSTOM_SED               0
#define ETH_VIA_CUSTOM_PIPELINE         1
#define ETH_NO_CUSTOM_PIPELINE          0

#define ENCAP_NO_CTAG                   1
#define ENCAP_ADD_CTAG                  0
#define ENCAP_NO_STAG                   0
#define ENCAP_NO_ATM_HDR                1
#define ENCAP_ADD_ATM_HDR               0

/* Combining some of the above flags for virtual interface processing */
#define UNMODIFIED                      0
#define GENERIC_PQM                     1
#define GENERIC_SED                     2
#define IPV4_PQM                        3
#define IPV4_SED                        4

/* Arp/ND buffer related constants */
#define NEXTHOP_BUFFER_MAX_FLOW             3
#define NEXTHOP_BUFFER_MAX_PKTS_PER_FLOW    128
#define ADK_ETH_ARP_BUFFER_TBL_HASH_WIDTH 10
#define ADK_ETH_ARP_BUFFER_TBL_HASH_MAX (1<<ADK_ETH_ARP_BUFFER_TBL_HASH_WIDTH)
#define ADK_ETH_ARP_BUFFER_TBL_HASH_MASK (ADK_ETH_ARP_BUFFER_TBL_HASH_MAX-1)
#define ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION 8
#define ADK_ETH_ARP_BUFFER_TIMER_INDEX_MAX (ADK_ETH_ARP_BUFFER_TBL_HASH_MAX * ADK_ETH_ARP_BUFFER_TBL_MAX_COLLISION)
/* TODO: It's not necessary for both buffers to have the same dimension */
#define ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_WIDTH 10
#define ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MAX (1<<ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_WIDTH)
#define ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MASK (ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MAX-1)
#define ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION 8
#define ADK_ETH_NEIGHBOR_BUFFER_TIMER_INDEX_MAX (ADK_ETH_NEIGHBOR_BUFFER_TBL_HASH_MAX * ADK_ETH_NEIGHBOR_BUFFER_TBL_MAX_COLLISION)

#define ADK_ETHERNET_DP_BUFFER_INFO_POOL_NAME  "adk_eth_buffer_info_pool"
#define ADK_ETHERNET_DP_BUFFER_INFO_POOL_CACHE_SIZE 16
#define ADK_ETHERNET_MAX_NO_OF_BUFFER_INFO 511  // TODO +(ADK_MAX_NUM_CORES * ADK_ETHERNET_DP_BUFFER_INFO_POOL_CACHE_SIZE))

#define ADK_ETHERNET_DP_NEXTHOP_BUFFER_POOL_NAME "adk_eth_buffer_pool"
#define ADK_ETHERNET_DP_NEXTHOP_BUFFER_POOL_CACHE_SIZE 16 
#define ADK_ETHERNET_MAX_NO_OF_NEXTHOP_BUFFER 511 //TODO +(ADK_MAX_NUM_CORES * ADK_ETHERNET_DP_NEXTHOP_BUFFER_POOL_CACHE_SIZE)

/*************************************
******* MACROS ETH EXCEPTION **********
*************************************/
#define ETH_EXC_CODE_MAX_NUM 3

#define ETH_EXC_DOMAIN_TN_PORT               0
#define ETH_EXC_DOMAIN_FP_ORIGIN             1
#define ETH_EXC_DOMAIN_OS_ORIGIN             2
#define ETH_EXC_DOMAIN_MAX_NUM               3

/*********************************************************
******** Ethernet module's Tree widths **************
*********************************************************/

#define ADK_ETH_EXCEPTION_HANDLER_LOOKUP_TREE_WIDTH \
    EXCEPTION_TYPE_WIDTH

#define ADK_ETH_CPU_PATH_PARAM_TREE_WIDTH \
    CPU_PATH_KEY_WIDTH

#define ADK_ETH_CPU_PATH_MTM_QUEUE_TREE_WIDTH \
    CPU_PATH_KEY_WIDTH

#define ADK_ETH_STATIC_PORT_MAP_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH

#define ADK_ETH_VLAN_ASSIGNMENT_TREE_WIDTH \
    VLAN_ASSIGN_POLICY_WIDTH + ETH_PHY_PORT_WIDTH

#define ADK_ETH_VLAN_ASSIGNMENT_EXT_TREE_WIDTH \
    FLAG_WIDTH + ETH_PHY_PORT_WIDTH + PRIORITY_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_CHECK_STP_STATE_TREE_WIDTH \
    FLAG_WIDTH + ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_VLAN_TREE_WIDTH     \
    ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_OUT_VLAN_TREE_WIDTH     \
    ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_RECV_MAC_TREE_WIDTH     \
    ETH_PHY_PORT_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_DEST_MAC_LOOKUP_TREE_WIDTH \
    MAC_ADDR_WIDTH

#define ADK_ETH_MULTICAST_LOOKUP_TREE_WIDTH \
    VLAN_ID_WIDTH + PRIORITY_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_ENCAP_FLOW_DMAC_TREE_WIDTH \
    ETH_FLOW_ID_WIDTH

#define ADK_ETH_ENCAP_FLOW_ATTR_TREE_WIDTH \
    ETH_FLOW_ID_WIDTH

#define ADK_ETH_STAG_ETH_TYPE_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + ETH_TYPE_WIDTH

#define ADK_ETH_CTAG_STAG_MAP_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_FLOW_ID_LOOKUP_TREE_WIDTH \
    INPUT_INTF_WIDTH + LLC_HDR_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_ARP_LOOKUP_TABLE_WIDTH \
    IPV4_ADDRESS_WIDTH

#define ADK_ETH_INGRESS_FLOW_LOOKUP_TABLE_WIDTH \
    ETH_TYPE_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_EGRESS_FLOW_LOOKUP_TABLE_WIDTH \
    LINUX_INTF_WIDTH + ETH_TYPE_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_CPU_FLOW_LOOKUP_TABLE_WIDTH \
    LLC_HDR_WIDTH + ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_ASSOCIATION_TREE_WIDTH  \
    ETH_PHY_PORT_WIDTH + LLC_HDR_WIDTH + MAC_ADDR_WIDTH * 2 + \
    VLAN_ID_WIDTH + PRIORITY_WIDTH

#define ADK_ETH_TYPE_PCP_MAP_LOOKUP_WIDTH \
    ETH_TYPE_WIDTH

#define ADK_ETH_ASSOCIATION_TREE_NUM_FIELDS 6

#define LOOKUP_POLICING_POLICY_TREE_WIDTH \
    FLAG_WIDTH + INPUT_INTF_WIDTH + VLAN_ID_WIDTH + PRIORITY_WIDTH + \
    MAC_ADDR_WIDTH

#define LOOKUP_INTF_PCP_POLICING_POLICY_TREE_WIDTH \
    FLAG_WIDTH + INPUT_INTF_WIDTH + PRIORITY_WIDTH

#define LOOKUP_INTF_PCP_STATS_TREE_WIDTH \
    INPUT_INTF_WIDTH + PRIORITY_WIDTH

#define LOOKUP_INTF_VLAN_STATS_TREE_WIDTH \
    INPUT_INTF_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_EGRESS_FLOW_LOOKUP_TREE_WIDTH \
    INPUT_INTF_WIDTH + LLC_HDR_WIDTH + MAC_ADDR_WIDTH

#define ADK_ETH_HASH_FLOW_LOOKUP_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + ETH_TYPE_WIDTH + ADK_ETH_EIOA_HASH_WIDTH

#define ADK_ETH_MTU_TREE_WIDTH  \
    INPUT_INTF_WIDTH + PKT_LENGTH_WIDTH

#define ADK_ETH_ARP_TREE_WIDTH  \
    IPV4_ADDRESS_WIDTH

#define ADK_ETH_MAP_SUB_INTF_NS_ID_TREE_WIDTH \
    INPUT_INTF_WIDTH

#define ADK_ETH_MAP_INPUT_PORT_NS_ID_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH

#define ADK_ETH_MAP_OUTPUT_PORT_NS_ID_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH

#define ADK_ETH_MAP_NS_INTF_ID_TREE_WIDTH \
    ETH_INTF_NS_ID_WITDH

#define ADK_ETH_MAP_INPUT_PORT_NS_ID_FOR_TIME_SLOT_ID_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH

#define ADK_ETH_LOOKUP_VIRTUAL_INTERFACE_TREE_WIDTH \
    VLAN_ID_WIDTH + VLAN_ID_WIDTH

#define ADK_ETH_GET_VIRTUAL_INTF_TAGS_WIDTH \
    VLAN_ID_WIDTH

#define ADK_ETH_PRIORITY_MAP_LOOKUP_TREE_WIDTH \
    FLAG_WIDTH + ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH + DSCP_WIDTH + \
    FLAG_WIDTH

#define DSCP_P_BITS_MAP_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + DSCP_WIDTH

#define ADK_ETH_QOS_POLICY_LOOKUP_TREE_WIDTH    \
    ETH_PHY_PORT_WIDTH + VLAN_ID_WIDTH + PRIORITY_WIDTH

#define ADK_ETH_L3_QOS_POLICY_LOOKUP_TREE_WIDTH \
    INPUT_INTF_WIDTH + PRIORITY_WIDTH

#define ADK_ETH_LINK_AGGREGATION_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + EGRESS_PATH_TYPE_WIDTH

#define ADK_ETH_CHECK_AGGR_LINK_VALIDITY_WIDTH \
    ETH_PHY_PORT_WIDTH + ETH_PHY_PORT_WIDTH

#define ADK_ETH_GET_CONSTITUENT_LINK_TREE_WIDTH \
    ETH_PHY_PORT_WIDTH + LINK_AGGREGATION_KEY_WIDTH

#define CUSTOM_BIND_INTERFACE_TREE_WIDTH    19

/* Default MAC AGing timer value in minutes */
#define ADK_ETH_DEFAULT_MAC_AGING_TIMER     5

#define ADK_ETH_MAX_INTF              (ADK_ETH_PORTS_MAX + ADK_ETH_AGGR_INTF_MAX)
#define ADK_ETH_VLAN_MAX_STATS_ID     (ADK_ETH_MAX_INTF * ADK_ETH_MAX_VLAN_ID)

#define ADK_ETH_DP_LOG_INFO_DEBUG debug_print

#endif /* #ifndef _ADK_ETH_CONSTANTS_H_ */

/*! end addtogroup _ethernet
 * @}
 */
