/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_eth_dp_pvt.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernet  Ethernet Module
 *
 * @{
 *
 * @brief   This file has the data types and functions which are used only by
 *          data plane software.
 *
 *  **********************************************************************/
#ifndef __ADK_ETH_DP_PVT_H__
#define __ADK_ETH_DP_PVT_H__

#include <adk_include.h>
#include <adk_sif.h>
#include <adk_dpdk.h>
#include <adk_pmu.h>
#include <adk_rcu.h>
#include <adk_hos_dp.h>
#ifdef ADK_TX_CKSUM_OL
#include <adk_cksum.h>
#endif
#include <adk_eth_dp.h>
#include <adk_ethernet_export.h>
#include <adk_qos_dp_export.h>
#include "adk_sif_dp_log_lib.h"
#include <rte_hash_crc.h>

#ifdef ADK_MODULE_ETH_OAM_ENABLED
#include <adk_eth_oam_dp_export.h>
#endif

#include <adk_netd_fp_ts_index.h>

#define ADK_ETH_INVALID_PORT 0xFFFFFFFF
#define ADK_ETH_BGID_MASK (ADK_ETH_NUM_BRIDGE - 1)

#define ADK_ETH_UNRESOLVED_ARP_EXCEPTION_PKT_COUNT 100

void
adk_eth_dp_print_mac (
    uint8_t *macAddr);

void
adk_eth_dp_print_hdr (
    adk_dp_eth_hdr_t *eth_hdr);

void
adk_eth_dp_print_pdu_hdr (
    uint8_t *eth_hdr);

adk_dp_status_t
adk_eth_dp_parse_hdr (
    void *pduData,
    adk_dp_eth_hdr_t *eth_hdr);

adk_dp_status_t
adk_eth_vlan_header_remove(
    adk_pktio_task_t *task,
    adk_dp_eth_hdr_t *eth_hdr);

adk_dp_status_t
adk_eth_vlan_header_strip(
    adk_pktio_task_t *task,
    uint8_t **pduData,
    int coreIndex);

adk_dp_status_t
adk_eth_vlan_header_proc(
    adk_pktio_task_t *task,
    uint8_t **pduData,
    adk_dp_eth_bridge_intf_t *dp_bridge_if_p,
    int coreIndex,
    bool *strip_vlan);
uint8_t adk_eth_task_chained_buffer_free(adk_pktio_task_t *task_p);

extern uint64_t adk_eth_uli_rx_ts_g, adk_eth_uli_tx_ts_g;
extern uint64_t adk_eth_lli_rx_ts_g, adk_eth_lli_tx_ts_g;

extern adk_dp_eth_svif_intf_t **adk_eth_svif_if_g;

#ifdef BLOCK_COUNT_ENABLED
#define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_RX_TS() \
    adk_dp_measure_block_cycle_rx_ts(&adk_eth_lli_rx_ts_g)
#define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_TX_TS() \
    adk_dp_measure_block_cycle_tx_ts(&adk_eth_lli_rx_ts_g, \
            &adk_eth_lli_tx_ts_g, \
            "ETHERNET LLI MODULE", \
            PMU_ETHERNET_LLI_MODULE)
#define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_ULI_RX_TS() \
    adk_dp_measure_block_cycle_rx_ts(&adk_eth_uli_rx_ts_g)
#define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_ULI_TX_TS() \
    adk_dp_measure_block_cycle_tx_ts(&adk_eth_uli_rx_ts_g, \
            &adk_eth_uli_tx_ts_g, \
            "ETHERNET ULI MODULE", \
            PMU_ETHERNET_ULI_MODULE)
#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_ULI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ETHERNET_ULI_TX_TS()
#endif
/*Supported EtherTypes*/
#define ADK_ETHERNET_ETHERTYPE_IPV4            0x0800
#define ADK_ETHERNET_ETHERTYPE_ARP             0x0806
#define ADK_ETHERNET_ETHERTYPE_VLAN            0x8100
#define ADK_ETHERNET_ETHERTYPE_IPv6            0x86DD
#define ADK_ETHERNET_ETHERTYPE_LACP            0x8809
#define ADK_ETHERNET_ETHERTYPE_CFM             0x8902
#define ADK_ETHERNET_ETHERTYPE_PTP             0x88F7
#define ADK_ETHERNET_ETHERTYPE_NSN_TEST_FRAME  0xA5A5
#define ADK_ETHERNET_ETHERTYPE_EAPOL           0x888E
#define ADK_ETHERNET_ETHERTYPE_LLDP            0x88CC

/*Packet clone size for raising exceptions*/
#define ADK_ETH_PACKET_CLONE_MAX_SIZE 200
#define ADK_ETH_PACKET_CLONE_SEGMENTS 1

adk_dp_status_t
adk_eth_dp_svif_get (
    adk_dp_eth_svif_if_hash_t *svif_if_hash_table_p,
    uint16_t l2_domain_id,
    uint16_t vlan,
    adk_dp_eth_svif_intf_t **dp_svif_intf_p);

adk_status_t
adk_eth_dp_process_pkt_single (
    void *task_p,
    int coreIndex,
    uint16_t *fn_id);

adk_status_t
adk_eth_dp_pkt_proc_uli (
    void **task,
    uint8_t task_count,
    uint32_t task_mask,
    void *ring,
    adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t adk_eth_dp_pkt_proc_ring_uli(void **task_p,
                                          __attribute__ ((unused)) uint8_t task_count,
                                          uint32_t task_mask,
                                          void *ring,
                                          adk_sif_cfg_pipeline_info_t * next_stage_entry,
                                          __attribute__ ((unused)) int coreIndex);

adk_status_t
adk_eth_dp_pkt_proc_uli_single (
    void *task,
    int coreIndex,
    uint16_t *destFunc);

#ifdef ADK_DEVICE_SNR
adk_status_t
adk_eth_dp_bridging_uli (
    void *task_p,
    int coreIndex,
    uint16_t __attribute__ ((unused)) *destFunc,
    uint32_t __attribute__ ((unused)) *dst_mod_id,
    adk_pal_dp_stage_info_t **stageInfo,
    uint32_t                          egress_port,
    uint16_t                           bg_id,
    adk_dp_eth_hdr_t *eth_hdr,
    bool oam_flow,
    uint8_t dscp,
    uint8_t ing_pcp);

adk_status_t
adk_eth_dp_bridge_oam(
    void *task,
    int coreIndex);

void
adk_eth_get_dscp(adk_pktio_task_t *task_p,
                adk_dp_eth_hdr_t *eth_hdr,
                uint8_t *dscp,
                bool *is_ip);

adk_status_t
adk_eth_dp_ts_pkt_transmitted(
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    __attribute__ ((unused)) void *ring,
    __attribute__ ((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t
adk_eth_packet_clone(
    struct rte_mbuf *inputPkt,
    struct rte_mbuf **new_pkt,
    uint16_t mbuf_clone_size,
    uint8_t no_clone_segments);

adk_status_t
adk_eth_dp_arp_get (
    adk_dp_eth_arp_table_t *eth_arp_table,
    adk_ethernet_ipv4_addr_t ip_addr,
    adk_ethernet_mac_addr_t dest_mac,
    adk_ethernet_arp_status_t* arp_status);

adk_status_t
adk_eth_dp_arp_entries_get (
        adk_dp_eth_arp_table_t *eth_arp_table,
        uint32_t *numArpEntries,
        adk_ethernet_arp_entry_t **arp_entries_p);

adk_status_t
adk_eth_dp_buffer_packet(
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    __attribute__ ((unused)) void *ring,
    __attribute__ ((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t
adk_eth_dp_buffer_packet_single(
    void *task,
    uint32_t task_mask,
    int coreIndex);

adk_dp_status_t
adk_eth_dp_encap_and_send_pkt(
void* task,
uint8_t *dest_mac,
int coreIndex,
bool flush);

adk_status_t adk_eth_dp_decode_interface
(
void* task,
adk_dp_eth_svif_intf_t **dp_svif_intf_pp,
adk_dp_eth_bridge_intf_t  **dp_bridge_intf_pp,
int coreIndex
);

adk_status_t
adk_eth_dp_buffer_packet(
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    __attribute__ ((unused)) void *ring,
    __attribute__ ((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t adk_eth_dp_flush_pkt
(
void* task,
uint32_t task_mask,
adk_ethernet_mac_addr_t dest_mac,
int coreIndex,
bool flush
);

adk_status_t adk_eth_dp_task_ring_free_or_flush
(
adk_dp_eth_buffer_info_t* nexthop_buffer_info,
uint32_t nexthop_entry_index,
bool flush,
uint32_t task_mask,
adk_ethernet_mac_addr_t dest_mac,
int coreIndex
);

adk_status_t adk_eth_dp_next_hop_buffer_store
(
adk_dp_eth_buffer_info_t* nexthop_buffer_info,
void* task,
uint32_t task_mask,
int coreIndex
);

adk_status_t adk_eth_dp_flush_pkts
(
adk_dp_eth_buffer_info_t* nexthop_buffer_info,
uint32_t task_mask,
adk_ethernet_mac_addr_t dest_mac,
int coreIndex
);

adk_status_t adk_eth_dp_next_hop_buffer
(
void* task,
uint32_t task_mask,
int coreIndex
);

adk_status_t
adk_eth_dp_buffer_packet_single
(
void *task,
uint32_t task_mask,
int coreIndex
);

adk_dp_status_t
adk_eth_dp_neighbor_entry_lookup
(
adk_dp_eth_neighbor_table_t *eth_neigh_table,
uint8_t *neighIpv6Addr,
adk_dp_eth_neighbor_entry_t **neigh_entry
);

adk_status_t
adk_eth_dp_neighbor_entries_get (
        adk_dp_eth_neighbor_table_t *eth_neigh_table,
        uint32_t *numNeighEntries,
        adk_ethernet_neighbor_entry_t **neigh_entries_p);

#endif /*ADK_DEVICE_SNR*/

#endif /* #ifndef __ADK_ETH_DP_PVT_H__ */
/*!  end addtogroup _ethernet  Ethernet Module
 * @}
 */
