/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ethernet_export.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ethernet  Ethernet Module
 *
 * @{
 *
 * @brief
 *
 * This file contains all data plane software specific data structures which
 * are shared internally with other ADK modules' data plane components. This
 * mostly includes the IMC data structure for binding Ethernet with other
 * modules.
 *
 * TODO: Rename this file as adk_eth_dp_imc.h
 *
 ***********************************************************************/

#ifndef _ADK_ETHERNET_EXPORT_H_
#define _ADK_ETHERNET_EXPORT_H_

#include <adk_sys.h>
#include <adk_ethernet.h>

/* The flow ID to be used by the upper layer module, when they need packet to be
 * sent unmodified on the specified interface */

#define ADK_ETH_INVALID_ENCAP_FLOW_ID  0xFFFFFFFF

typedef struct adk_next_hop_s
{
    bool is_ipv6;
    union
    {
        adk_ethernet_ipv4_addr_t ipv4_addr;
        adk_ethernet_ipv6_addr_t ipv6_addr;
    } u;
} adk_next_hop_t;

/*!
 * =============================================================
 * @struct adk_ethernet_dp_uli_imc_param_t adk_ethernet_export.h "adk_ethernet_export.h"
 *
 * @brief
 *
 * The data structure defines the Ethernet ULI IMC parameters that needs to be
 * passed along with the packet by higher layer protocol modules for Ethernet
 * encapsulation and transmission.
 *
 * =============================================================
 */
typedef struct adk_ethernet_dp_uli_imc_param_s
{
    /* The DSCP value to be used as input for DSCP->PCP mapping */
    uint8_t dscp;
    /* The default PCP value to be used for marking and QoS selection */
    uint8_t pcp;
    /* The output interface to be used for sending packet out when no
     * encap flow ID is available (i.e. when F2F bind is NOT used)
     * Use ADK_ETH_IF_SVIF_IF_MAX to indicate packet is sent to 
     * bridge instead of SVIF  */
    uint32_t output_intf_id;
    /* Flag to indicate if packet is unfragmented or first fragment.
     * Needed for LAG hashing */
    bool is_unfrag_first_frag;
    /* Number of fragments*/
    uint8_t frag_count;
    /* L4 protocol above IP layer. Needed for LAG hashing */
    uint8_t l4_proto;
    /* Flag to identify oam switching flow */
    bool is_oam_switching_flow;
    /* TODO: OAM related params can be moved to a structure? */
    /* oam flow type (wire:0 relay:1) */
    bool oam_flow_type;
    /* Bridge ID */
    uint16_t bg_id;
    /* Next hop address for arp/neighbor resolution */
    adk_next_hop_t next_hop;
} __attribute__ ((aligned(ADK_METADATA_ALIGN))) adk_ethernet_dp_uli_imc_param_t;

#ifdef ADK_HOS_OLD_IMPL
/* Helper function for IP modules to configure IP ACL on the underlying
 * Ethernet L2 interface */
adk_status_t
adk_eth_intf_acl_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    bool is_ipv6,
    bool disable_ip,
    bool enable_acl,
    uint32_t rule_set_id);

/* Helper function for IP modules to configure IP VRD on the underlying
 * Ethernet L2 interface */
adk_status_t
adk_eth_intf_vrd_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    uint32_t vrd_id);

/* Helper function for IP modules to configure IP packet capture on the
 * underlying Ethernet L2 interface */
adk_status_t
adk_eth_intf_pcap_config(
    adk_module_handle_t eth_mod_handle,
    adk_interface_handle_t if_handle,
    bool is_ipv6,
    bool at_ingress,
    uint32_t pcap_id,
    bool enable_pcap);
#endif

/* TODO: Remove all below entry function prototype as they are all programmed
 * dynamically at initialization time */

adk_status_t
adk_eth_dp_process_pkt (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    void *ring,
    adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t
adk_eth_dp_process_pkt_ring (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    void *ring,
    adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t
adk_eth_dp_pkt_proc_netd_v2 (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    void *ring,
    adk_sif_cfg_pipeline_info_t *ptr2,
    int coreIndex);

adk_status_t
adk_eth_dp_pkt_proc_netd (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    int coreIndex);

adk_status_t 
adk_ethernet_rsrc_notify_fn_cbfn(
    adk_module_handle_t mod_handle, 
    adk_sys_rsrc_oper_t oper,
    adk_sys_resource_type_t type,
    adk_sys_resource_info_t rsrc);

adk_status_t
adk_eth_dp_pkt_proc_netd_bridge (
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    void *ptr1,
    adk_sif_cfg_pipeline_info_t *ptr2,
    int coreIndex);

adk_status_t
adk_eth_dp_interface_id_get(void *task_p,
                            uint32_t *interface_id);

adk_status_t
adk_eth_intf_get_vlan(adk_module_handle_t eth_mod_handle,
    const uint32_t interface_id,
    adk_ethernet_vlan_id_t*  vlan_id);

#endif /* _ADK_ETHERNET_EXPORT_H_ */
/*!  end addtogroup _ethernet  Ethernet Module
 * @}
 */
