/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ip_config.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ip  IPv4 Module
 * @{
 *
 * @brief
 *
 * This file contains default parameters that customer can tune
 * to expand the capability. Some of the parameters may be restricted
 * by the resource availability in the fast path.
 *
 ***********************************************************************/
#ifndef _ADK_IP_CONFIG_H_
#define _ADK_IP_CONFIG_H_

#include "adk_config_overrides.h"

/* --------------------------------------------------------------------
 *                      CUSTOMER PROVIDED DIMENSIONS
 * -------------------------------------------------------------------- */

/*! The maximum number of IPv4 interfaces */
#define ADK_IP_MAX_INTERFACES                                           (201)

/*! Maximum number of IP addresses than can be set
 * for an IP interface */
#define ADK_IP_MAX_ADDR_PER_INTERFACE                                   (201)

/*! Maximum number of virtual ip addresses allowed
 * in the system */
#define ADK_IP_MAX_VIRTUAL_IP_ADDRESSES                                 (200)

/*! Maximum number of IPv4 addresses in the system
 * (this includes local as well as virtual ip addresses)
 * */
#define ADK_IP_MAX_ADDRESSES                                            (201)

/*! MAX number of rules for IP Ingress Association Ordered */
#define ADK_IP_MAX_INGRESS_CLASSIFIER_RULES                             (1024)

/*! Max IPV4 Routing tables supported by ADK is 201 */
#define ADK_IP_MAX_ROUTING_TABLES                                       (201)

/*! MAX IPV4 routes allowed in system */
#define ADK_IP_MAX_ROUTES                                               (1200)

/*! MAX IPV4 routing policy based rules */
#define ADK_IP_MAX_PBR_RULES                                            (201)

/*! MAX number of rules for IP Egress QOS classifier */
#define ADK_IP_MAX_EGRESS_QOS_CLASSIFIER_RULES        (128)


/* --------------------------------------------------------------------
 *
 * -------------------------------------------------------------------- */


/*! Default MTU for IP interface when user
 * specified value during creation is 0 */
#define ADK_IP_INTERFACE_DEFAULT_MTU                                    (1500)

/*! Maximum number of simultaneous reassembly sessions supported
 * Since mempool is efficient with 2^n-1 entries, 2047 is used.*/
#define ADK_IP_MAX_REASSEMBLY_SESSIONS                                  (2047)

/*! The re-assembly timeout value used by IPv4 module while
 * re-assembling fragmented packets */
#ifndef ADK_IP_REASSEMBLY_TIMEOUT
#define ADK_IP_REASSEMBLY_TIMEOUT                                       (100000)  /* 100 milli sec */

#define ADK_REASSEMBLY_TIMEOUT_MILLISECONDS                             (ADK_IP_REASSEMBLY_TIMEOUT/1000)
#endif

/*! Maximum number of fragments that can be created
 * while fragmenting a packet */
#define ADK_IP_MAX_FRAG_SUPPORTED                                       (128)


#endif  /* #ifndef _ADK_IP_CONFIG_H_ */


/*! end addtogroup _ip
 * @}
 */
