/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_ip_common.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief
 *
 * This file contains internal common functions used by the IPv4 APIs. This
 * includes functions for common settings across, common parameter validations,
 * etc.
 *
 ***********************************************************************/

#include "adk_ip_common.h"

uint32_t ip_bit_mask[33] = /* Mask with top <n> bits set */
{
    0x00000000, 0x80000000, 0xC0000000, 0xE0000000,     /* 0-3 */
    0xF0000000, 0xF8000000, 0xFC000000, 0xFE000000,     /* 4-7 */
    0xFF000000, 0xFF800000, 0xFFC00000, 0xFFE00000,     /* 8-11 */
    0xFFF00000, 0xFFF80000, 0xFFFC0000, 0xFFFE0000,     /* 12-15 */
    0xFFFF0000, 0xFFFF8000, 0xFFFFC000, 0xFFFFE000,     /* 16-19 */
    0xFFFFF000, 0xFFFFF800, 0xFFFFFC00, 0xFFFFFE00,     /* 20-23 */
    0xFFFFFF00, 0xFFFFFF80, 0xFFFFFFC0, 0xFFFFFFE0,     /* 24-27 */
    0xFFFFFFF0, 0xFFFFFFF8, 0xFFFFFFFC, 0xFFFFFFFE,     /* 28-31 */
    0xFFFFFFFF                  /* 32 */
};

adk_status_t adk_ip_cpu_flow_policing_config (
    adk_module_handle_t  __attribute__((unused))  ipHandle, 
    uint32_t  __attribute__((unused))  cpu_flow_id,
    uint32_t  __attribute__((unused))  policingHandle,
    bool  __attribute__((unused))  addEntry)
{
    return ADK_SUCCESS;
}


adk_status_t adk_ip_rsrc_policing_config(
    adk_module_handle_t __attribute__((unused))  ipHandle,
    uint32_t __attribute__((unused))  filter_flow_id,
    uint32_t __attribute__((unused))  policing_flow_id,
    bool __attribute__((unused))  addEntry)
{
    return ADK_SUCCESS;
}

bool adk_ipv4_valid_handle(adk_module_handle_t ipHandle)
{
    ip_module_info_t *ipModuleInfo_p = NULL;

    if (!ADK_MODULE_HANDLE_VALIDATE(ipHandle))
        return FALSE;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipHandle);

    if (ipModuleInfo_p == NULL)
        return FALSE;

    if (ipModuleInfo_p->cookie != ADK_IPV4_MODULE_COOKIE)
        return FALSE;

    return TRUE;
}

bool adk_ipv4_interface_valid_handle(adk_interface_handle_t ipIfHandle)
{
    ip_interface_info_t *if_pvt_info;

    if (!ADK_SYS_VALID_INTERFACE_HANDLE(ipIfHandle))
        return FALSE;

    if_pvt_info = (ip_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(ipIfHandle);

    if (if_pvt_info == NULL)
        return FALSE;

    if (if_pvt_info->cookie != ADK_IPV4_INTF_COOKIE)
        return FALSE;

    return TRUE;
}

bool adk_ipv4_vif_valid_handle(adk_interface_handle_t ipVIfHandle)
{
    ip_virtual_interface_info_t *vif_pvt_info;

    if (!ADK_SYS_VALID_INTERFACE_HANDLE(ipVIfHandle))
        return FALSE;

    vif_pvt_info = (ip_virtual_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(ipVIfHandle);

    if (vif_pvt_info == NULL)
        return FALSE;

    if (vif_pvt_info->cookie != ADK_IPV4_V_INTF_COOKIE)
        return FALSE;

    return TRUE;
}

/*!
 * =============================================================
 * @internal
 * TO DO: Remove this API
 * adk_ipv4_vrd_valid_handle
 *
 * @param[in] ipVrdHandle   IP VRD instance handle
 *
 * @param[out] vrdId        Pointer to the ID for the VRD handle
 *
 * @retval TRUE         The IP VRF handle is valid
 *
 * @retval FALSE        The IP VRF handle is invalid
 *
 * @brief
 *
 * API to validate an IP VRF handle and return ID associated with it, if valid.
 * =============================================================
 */
bool adk_ipv4_vrd_valid_handle(
__attribute__((unused)) adk_ip_vrd_handle_t ipVrdHandle,
uint32_t        *vrdId
)
{
#if ENABLE_VRD
    ip_vrd_info_t       *vrd_info_p;

    vrd_info_p = (ip_vrd_info_t *) (ipVrdHandle);

    if (vrd_info_p == NULL)
    {
        *vrdId = ADK_IP_DEFAULT_VRD_ID;
        return TRUE;
    }

    if (vrd_info_p->cookie != ADK_IPV4_VRD_COOKIE)
        return FALSE;

    *vrdId = vrd_info_p->vrd_id;
#else
    *vrdId = ADK_IP_DEFAULT_VRD_ID;
#endif /* ENABLE_VRD */

    return TRUE;
}

bool adk_ipv4_flow_valid_handle(adk_flow_handle_t ipFlowHandle)
{
    ip_flow_info_t *ip_flow_info_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(ipFlowHandle))
        return FALSE;

    ip_flow_info_p = (ip_flow_info_t *)
                     ADK_SYS_FLOW_ATTR_PTR_GET(ipFlowHandle);

    if (ip_flow_info_p == NULL)
        return FALSE;

    if (ip_flow_info_p->cookie != ADK_IPV4_FLOW_COOKIE)
        return FALSE;

    return TRUE;
}

/*!
 * =============================================================
 *
 * adk_ipv4_sys_log
 *
 * @param[in]  ipHandle   IPv4 Module Handle
 * @param[in]  log_level        SIF defined log level
 *
 * @retval ADK_SUCCESS          Successful completion
 * @retval ADK_E_INVALID_HANDLE Invalid IPv4 module handle parameter
 *
 * @brief  Function to set the log-level for IPv4 module through service
 *         funtion registered with SIF.
 *
 * This function is used to set the log-level for IPv4 module via SIF using
 * adk_module_attr_set() API. This function must not be called directly.
 *
 * =============================================================
 */
adk_status_t adk_ipv4_sys_log(adk_module_handle_t ipHandle,
                              adk_log_level_t sys_log_level)
{
    adk_status_t                ret = ADK_SUCCESS;
    adk_log_level_t             ipv4_log_level;
    adk_sys_handle_t sys_handle;

    if (!adk_ipv4_valid_handle(ipHandle))
    {
        ADK_IPV4_LOG_FORCE("Invalid IPv4 module handle %"ADKPRIh"\n",
                           ipHandle);
        ADK_IPV4_LOG_FORCE("Failed to set log-level 0x%x for IPv4\n",
                           sys_log_level);
        return(ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);

    ADK_IPV4_LOG_FUNC_ENTRY(ipHandle);

    ADK_ACQUIRE_MODULE_LOCK(ipHandle, ret);

    ret = adk_ipv4_map_sys_log(ipHandle, sys_log_level, &ipv4_log_level);
    if (ret != ADK_SUCCESS)
        goto _adk_ipv4_sys_log_exit;

    ADK_MODULE_LOG_LEVEL_SET(ipHandle, ipv4_log_level);

_adk_ipv4_sys_log_exit:

    if (ret == ADK_SUCCESS) {
        ADK_IPV4_LOG_INFO(ipHandle,
                          "IPv4 module %"ADKPRIh" log-level set to 0x%x\n",
                          ipHandle, ipv4_log_level);
    } else {
        ADK_IPV4_LOG_ERROR(ipHandle, ret,
                           "Failed to set log-level %d\n", sys_log_level);
    }

    ADK_RELEASE_MODULE_LOCK(ipHandle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV4_LOG_FUNC_EXIT(ipHandle);

    return(ret);
}

/*!
 * =============================================================
 *
 * adk_ip_log_level_set
 *
 * @param[in]  ipHandle   IPv4 Module Handle
 * @param[in]  log_level        Logical OR of different log types
 *
 * @retval ADK_SUCCESS          Successful completion
 * @retval ADK_E_INVALID_HANDLE Invalid IPv4 module handle parameter
 *
 * @brief  Function to set the log-level for IPv4 module
 *
 * This function is used to set the log-level for IPv4 module. This
 * completely overwrites the existing log-level setting.
 *
 * =============================================================
 */
adk_status_t adk_ip_log_level_set(adk_module_handle_t ipHandle,
                                  adk_log_level_t log_level)
{
    adk_status_t                    ret = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;

    if (!adk_ipv4_valid_handle(ipHandle))
    {
        ADK_IPV4_LOG_FORCE("Invalid IPv4 module handle %"ADKPRIh"\n",
                           ipHandle);
        ADK_IPV4_LOG_FORCE("Failed to set IPv4 log-level 0x%x\n",
                           log_level);
        return(ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);

    ADK_IPV4_LOG_FUNC_ENTRY(ipHandle);

    ADK_ACQUIRE_MODULE_LOCK(ipHandle, ret);

    ADK_MODULE_LOG_LEVEL_SET(ipHandle, log_level);

    ADK_IPV4_LOG_INFO(ipHandle,
                      "IPv4 module %"ADKPRIh" log-level set to 0x%x\n",
                      ipHandle, log_level);

    ADK_RELEASE_MODULE_LOCK(ipHandle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV4_LOG_FUNC_EXIT(ipHandle);

    return(ret);
}

/*!
 * =============================================================
 *
 * adk_ip_log_enable
 *
 * @param[in]  ipHandle   IPv4 Module Handle
 * @param[in]  log_level        Logical OR of different log types to be enabled
 *
 * @retval ADK_SUCCESS          Successful completion
 * @retval ADK_E_INVALID_HANDLE Invalid IPv4 module handle parameter
 *
 * @brief  Function to enable one or more log types
 *
 * This function is used to enable one or more log type without affecting the
 * other log-types which are already enabled or disabled.
 *
 * =============================================================
 */
adk_status_t adk_ip_log_enable(adk_module_handle_t ipHandle,
                               adk_log_level_t log_level)
{
    adk_log_level_t curr_log_level;
    adk_sys_handle_t sys_handle;

    if (!adk_ipv4_valid_handle(ipHandle))
    {
        ADK_IPV4_LOG_FORCE("Invalid IPv4 module handle %"ADKPRIh"\n",
                           ipHandle);
        ADK_IPV4_LOG_FORCE("Failed to enable log 0x%x\n", log_level);
        return(ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);

    curr_log_level = ADK_MODULE_LOG_LEVEL_GET(ipHandle);
    curr_log_level |= log_level;

    ADK_MODULE_LOG_LEVEL_SET(ipHandle, curr_log_level);

    ADK_IPV4_LOG_INFO(ipHandle,
                      "Enabled log-level 0x%x for IPv4 module %"ADKPRIh"\n",
                      log_level, ipHandle);

    return(ADK_SUCCESS);
}

/*!
 * =============================================================
 *
 * adk_ip_log_disable
 *
 * @param[in]  ipHandle   IPv4 Module Handle
 * @param[in]  log_level        Logical OR of different log types to be disabled
 *
 * @retval ADK_SUCCESS          Successful completion
 * @retval ADK_E_INVALID_HANDLE Invalid IPv4 module handle parameter
 *
 * @brief  Function to disable one or more log types
 *
 * This function is used to disable one or more log type without affecting the
 * other log-types which are already enabled or disabled.
 *
 * =============================================================
 */
adk_status_t adk_ip_log_disable(adk_module_handle_t ipHandle,
                                adk_log_level_t log_level)
{
    adk_log_level_t curr_log_level;
    adk_sys_handle_t sys_handle;

    if (!adk_ipv4_valid_handle(ipHandle))
    {
        ADK_IPV4_LOG_FORCE("Invalid IPv4 module handle %"ADKPRIh"\n",
                           ipHandle);
        ADK_IPV4_LOG_FORCE("Failed to disable log 0x%x\n", log_level);
        return(ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);

    curr_log_level = ADK_MODULE_LOG_LEVEL_GET(ipHandle);
    curr_log_level &= ~log_level;

    ADK_MODULE_LOG_LEVEL_SET(ipHandle, curr_log_level);

    ADK_IPV4_LOG_INFO(ipHandle,
                      "Disabled log-level 0x%x for IPv4 module %"ADKPRIh"\n",
                      log_level, ipHandle);

    return(ADK_SUCCESS);
}
