/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 *
 * @file  adk_ip_dump_internal.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief
 *
 * This file contains low level functions to dump debug information e.g.
 * functions to dump tree entries.
 *
 ***********************************************************************/

#include "adk_ip_common.h"
#include <adk_lpm.h>

/*!
 * =============================================================
 * @internal
 *
 * adk_ipv4_dump_tables
 *
 * @param[in] ipHandle      IPv4 module Handle
 *
 * @param[in] table_group   Group of tables to dump
 *
 * @retval ADK_SUCCESS                  Successful completion
 *
 * @brief
 *
 * This function dump all the tree entries belonging to the specified
 * group of tables in csv format.
 * =============================================================
 */
adk_status_t adk_ipv4_dump_tables(adk_module_handle_t ipHandle,
                                  adk_ip_table_group_t table_group)
{
    adk_status_t retValue = ADK_SUCCESS;
    ip_module_info_t *ipModuleInfo_p;
    adk_sys_handle_t sys_handle;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);

    ADK_IPV4_LOG_FUNC_ENTRY(ipHandle);

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipHandle);


    if(table_group & ADK_IP_TABLES_INTF_ATTR) {
        retValue = adk_ip_intf_attr_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                               "Failed to dump IP interface attribute tree\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

    if(table_group & ADK_IP_TABLES_ROUTING) {
        ADK_TABLE_ENTRY_WRITE("DIP Table Dump\n");
        ADK_TABLE_ENTRY_WRITE("VRDID, DIP, MASK, INVALID_DIP, INGRESS_ASSOC, DIRECTROUTE, REASSEMBLY, INGRESS_ASSOC_MASK ");
        ADK_TABLE_ENTRY_WRITE("NH, IPIFID ");
        ADK_TABLE_ENTRY_WRITE("\n============================================================================================\n");
        adk_lpm_dump(ipModuleInfo_p->ip_dp_tbl_info_p->dipTbl, adk_ip_dip_dump_cb_fn, NULL);

        ADK_TABLE_ENTRY_WRITE("===============================================================================================\n\n");

        ADK_TABLE_ENTRY_WRITE("SIP Table Dump\n");
        ADK_TABLE_ENTRY_WRITE("VRDID, SIP, MASK, INVALID_SIP ");
        ADK_TABLE_ENTRY_WRITE("\n============================================================================================\n");
        adk_lpm_dump(ipModuleInfo_p->ip_dp_tbl_info_p->sipTbl, adk_ip_sip_dump_cb_fn, NULL);

        ADK_TABLE_ENTRY_WRITE("===============================================================================================\n\n");

        retValue = adk_ip_dp_pbr_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                    "Failed to dump IP PBR Tables\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

#if 0
    if(table_group & ADK_IP_TABLES_VRD) {
        retValue = adk_ip_dest_mtu_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                               "Failed to dump IP Dest MTU Table\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }
#endif
    if(table_group & ADK_IP_TABLES_FLOWCACHE) {
        retValue = adk_ip_dp_fc_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                                "Failed to dump IP DP Flow cache Table\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

    if(table_group & ADK_IP_TABLES_REASM) {
        retValue = adk_ip_dp_reasm_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                                "Failed to dump IP Reassembly Tables\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

    if(table_group & ADK_IP_TABLES_EXCEPTIONS) {
        retValue = adk_ip_dp_exception_handler_tbl_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                    "Failed to dump IP Exception Handler Tables\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

    if(table_group & ADK_IP_TABLES_VRD) {
        retValue = adk_ip_dp_vrdHostmode_dump(ipHandle);
        if(retValue != ADK_SUCCESS) {
            retValue = ADK_E_PTN_LIST_FAILED;
            ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                    "Failed to dump IP VRD Hostmode Tables\n");
            goto _adk_ipv4_dump_tables_exit;
        }
    }

    return(retValue);

_adk_ipv4_dump_tables_exit:

    ADK_IPV4_LOG_ERROR(ipHandle, retValue,
                       "Failed to dump IPv4 module's trees\n");

    return(retValue);
}

adk_status_t
adk_ip_intf_attr_tbl_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    uint32_t    i;
    adk_ip_dp_interface_entry_t *dp_entry;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);

    ADK_TABLE_ENTRY_WRITE("IP Interface Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("ID, Type, Status, VrdId, StatsId, ");
    ADK_TABLE_ENTRY_WRITE("InAcl, InRulesetId, ");
    ADK_TABLE_ENTRY_WRITE("OutAcl, OutRulesetId, ");
    ADK_TABLE_ENTRY_WRITE("MTU, InQoSProfileId, EgressQoS, ");
    ADK_TABLE_ENTRY_WRITE("InPolicing");
    ADK_TABLE_ENTRY_WRITE("\n=======================================================================================\n");
    for(i=0;i<ADK_DP_MAX_IP_INTERFACE;i++)
    {
        dp_entry = &(ipModuleInfo_p->ip_dp_tbl_info_p->ipInterfaceTble[i]);
        if(dp_entry->ifType != ADK_IP_DP_INTF_TYPE_INVALID)
        {
            ADK_TABLE_ENTRY_WRITE("%d, ", i);
            if(dp_entry->ifType == ADK_IP_DP_INTF_TYPE_ETH)
            {
                ADK_TABLE_ENTRY_WRITE("ETH, ")
            }
            else
            {
                ADK_TABLE_ENTRY_WRITE("INVALID, ")
            }
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dp_entry->vrd_id);
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dp_entry->if_stats_id);
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dp_entry->ingress_ruleset_id);
            ADK_TABLE_ENTRY_WRITE("%s, ", (dp_entry->enableOutAcl)?"ENABLED":"DISABLED");
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dp_entry->mtu);
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dp_entry->ingress_qos_prof_id);
            ADK_TABLE_ENTRY_WRITE("%s, ", (dp_entry->enableQoS)?"ENABLED":"DISABLED");
            ADK_TABLE_ENTRY_WRITE("%s, ", (dp_entry->enablePolicing)?"ENABLED":"DISABLED");
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }
    ADK_TABLE_ENTRY_WRITE("=======================================================================================\n\n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ip_dest_mtu_tbl_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    uint32_t    i;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);

    ADK_TABLE_ENTRY_WRITE("IP Destination MTU Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("VRD Id, Dest Address, MTU");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(i=0;i<ADK_IPV4_DST_MTU_TBL_HASH_MAX*ADK_IPV4_DST_MTU_TBL_MAX_COLLISION;i++) {
        if(ipModuleInfo_p->ip_dp_tbl_info_p->ipDestMtuTbl[i].dipAddr_nw){
            ADK_TABLE_ENTRY_WRITE("%d, ", ipModuleInfo_p->ip_dp_tbl_info_p->ipDestMtuTbl[i].vrdid);
            ADK_TABLE_ENTRY_WRITE("0x%x, ", adk_ntohl(ipModuleInfo_p->ip_dp_tbl_info_p->ipDestMtuTbl[i].dipAddr_nw));
            ADK_TABLE_ENTRY_WRITE("%d", ipModuleInfo_p->ip_dp_tbl_info_p->ipDestMtuTbl[i].mtu);
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ip_dp_exception_handler_tbl_dump(adk_module_handle_t ip_handle)
{
    uint32_t j;
    ip_module_info_t *ipModuleInfo_p;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);
    ADK_TABLE_ENTRY_WRITE("IP Exception Handler table Dump\n");
    ADK_TABLE_ENTRY_WRITE("Exception, Path");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");
    for(j=ADK_IP_OPTIONS_EXCEPTION; j< ADK_IP_EXCEPTION_MAX; j++)
    {
        if(ipModuleInfo_p->ip_dp_tbl_info_p->exceptionHandlerTble[j])
        {
            ADK_TABLE_ENTRY_WRITE("%d",j);
                ADK_TABLE_ENTRY_WRITE("\n");
        }
    }
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n\n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ip_dp_vrdHostmode_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    bool *vrdHostModeTble;
    int vrd_id;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);
    vrdHostModeTble = ipModuleInfo_p->ip_dp_tbl_info_p->vrdHostModeTble;

    ADK_TABLE_ENTRY_WRITE("IP VRD Hostmode Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("VRD_ID, Host_Mode");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(vrd_id = 0; vrd_id <= ADK_IP_DEFAULT_VRD_ID; vrd_id++)
    {
        ADK_TABLE_ENTRY_WRITE("%d, ",vrd_id);
        ADK_TABLE_ENTRY_WRITE("%d, ",vrdHostModeTble[vrd_id]);
        ADK_TABLE_ENTRY_WRITE("\n");
    }
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n\n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ip_dp_pbr_tbl_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    uint32_t pbrStateId;
    adk_ip_pbr_ctx_t *pbrCtx_p;
    ip_flow_info_t *flow_info_p;
    adk_ip_pbr_rule_t *pbrRules_p;
    adk_ip_pbr_action_t *pbrAction_p;
    uint16_t rule_index;
    adk_acl_rule_t cp_rule;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);


    ADK_TABLE_ENTRY_WRITE(" \t IPv4 PBR Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|Index");
    ADK_TABLE_ENTRY_WRITE("|ruleMask");
    ADK_TABLE_ENTRY_WRITE("|SIP");
    ADK_TABLE_ENTRY_WRITE("|Action");

    ADK_TABLE_ENTRY_WRITE("\n====================================================== \n");

    pbrCtx_p = (adk_ip_pbr_ctx_t *)
                    ipModuleInfo_p->ip_dp_tbl_info_p->ipPbrInfo_p;

    /*Get the rules from CP list*/
    adk_acl_get_reset(pbrCtx_p->cp_list);
    rule_index = 0;
    while(ADK_SUCCESS == adk_acl_next_rule_get(
            pbrCtx_p->cp_list, &cp_rule))
    {
        flow_info_p = (ip_flow_info_t *)
                                    ADK_SYS_FLOW_ATTR_PTR_GET(cp_rule.user_data);
        pbrRules_p = &(flow_info_p->u.pbr_flow.rule);
        pbrAction_p = &(flow_info_p->u.pbr_flow.action);
        pbrStateId = ADK_SYS_FLOWID_GET(cp_rule.user_data);
        ADK_TABLE_ENTRY_WRITE("|%05d",rule_index);
        ADK_TABLE_ENTRY_WRITE("|%04x",pbrRules_p->pbrRuleMask);

        if(pbrRules_p->pbrRuleMask & ADK_IP_PBR_SRC_ADDR_MASK)
        {
            ADK_TABLE_ENTRY_WRITE("|<0x%x/%d>",pbrRules_p->srcAddrInfo.addr,
                    pbrRules_p->srcAddrInfo.prefixWidth);
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }
        ADK_TABLE_ENTRY_WRITE("|<%d>",pbrAction_p->routeTableId);
         rule_index++;
    }
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n\n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ip_dp_reasm_tbl_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    adk_ip_dp_reassembly_tbl_t *ipReassemblyTbl;
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p;
    struct rte_mempool *ipReassemblySessionTbl;
    adk_sys_handle_t sys_handle;
    uint32_t row_index, col_index, active_session_count= 0;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ip_handle);

    ipReassemblyTbl = ipModuleInfo_p->ip_dp_tbl_info_p->ipReassemblyTbl;
    ipReassemblySessionTbl = ipModuleInfo_p->ip_dp_tbl_info_p->ipReassemblySessionTbl;
    
    ADK_TABLE_ENTRY_WRITE("IP Reassembly Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("RowIndex, count_hashid\n");
    ADK_TABLE_ENTRY_WRITE("    Col_index, hash_id, sess_entry_p, sip, dip, ip_id, protocol, vrd_id,");
    ADK_TABLE_ENTRY_WRITE(" timer handle, num_holes, numEnqueues, version, defragLength, assembledBytes\n");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(row_index =0; row_index < IP_REASSEMBLY_MAX; row_index++)
    {
        if(ipReassemblyTbl[row_index].count_hashid != 0)
        {
            ADK_TABLE_ENTRY_WRITE("0x%x, %d\n ",row_index,ipReassemblyTbl[row_index].count_hashid);
            active_session_count += ipReassemblyTbl[row_index].count_hashid;
            for(col_index =0; col_index < ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION; col_index++)
            {
                session_entry_p = ipReassemblyTbl[row_index].buckets[col_index].sess_entry_p;
                if(session_entry_p != NULL)
                {
                    ADK_TABLE_ENTRY_WRITE("    %d, 0x%x, 0x%"PRIx64 ", ",col_index, ipReassemblyTbl[row_index].
                            buckets[col_index].hash_id, (unsigned long)session_entry_p);
                    ADK_TABLE_ENTRY_WRITE("0x%x, 0x%x, %d, %d, %d, %d, %d, %d, %d, %d, %d \n",
                    ntohl(session_entry_p->sip), ntohl(session_entry_p->dip), session_entry_p->ip_id, session_entry_p->protocol,
                    session_entry_p->vrd_id, session_entry_p->timer, session_entry_p->num_holes, session_entry_p->numEnqueues,
                    session_entry_p->version, session_entry_p->defragLength,session_entry_p->assembledBytes);
                }
            }
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_IPV4_LOG_EXT_API(ip_handle, "rte_mempool_in_use_count", "getting number of entries allocated from reassembly session table memory pool\n");

    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");
    ADK_TABLE_ENTRY_WRITE("Active Reassembly session count : %d\n",
                active_session_count);
    ADK_TABLE_ENTRY_WRITE("Active Reassembly session count from mempool: %d\n",
                rte_mempool_in_use_count(ipReassemblySessionTbl));
    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ip_dp_fc_tbl_dump(adk_module_handle_t ip_handle)
{
    ip_module_info_t *ipModuleInfo_p;
    uint32_t i, core_id, per_core_count;
    uint8_t *fc_aging_tbl_p;

    ipModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ip_handle);

    ADK_TABLE_ENTRY_WRITE("IP LLI 5 tuple Flow Cache Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("Core Id, Version Id, Hit Count, VRD Id, DIP, SIP, Proto, Dport, Sport, Action");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(core_id =0; core_id<ADK_MAX_NUM_CORES; core_id++)
    {
        per_core_count = ipModuleInfo_p->ip_dp_tbl_info_p->
                fc_global_maintenance_p[core_id].lli5_fc_usage_count;
        if(per_core_count != 0)
        {
            ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
            ADK_TABLE_ENTRY_WRITE("usage_count:%d\n ", per_core_count);
            ADK_TABLE_ENTRY_WRITE("\n------------------------------------\n");
        }
        fc_aging_tbl_p = ipModuleInfo_p->ip_dp_tbl_info_p->
            lliNodeTbl_5_count_p[core_id].fc_maintenance;
        for(i=0; i<IP_MAX_FLOWS_PER_CPU; i++)
        {
            /*Check for used entry and print only used one*/
            if(fc_aging_tbl_p[i/4] & (1 <<((i%4)*2)))
            {
                adk_ip_dp_lli_hash5_tbl_entry_t *node_entry =
                    &(ipModuleInfo_p->ip_dp_tbl_info_p->lliNodeTbl_5_p[
                            core_id*IP_MAX_FLOWS_PER_CPU + i]);
                ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->version_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->hit);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->vrd_id);
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->dest_addr));
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->src_addr));
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->protocol);
                ADK_TABLE_ENTRY_WRITE("0x%x, ", adk_ntohs(node_entry->dest_port));
                ADK_TABLE_ENTRY_WRITE("0x%x, ", adk_ntohs(node_entry->src_port));
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->action);
                ADK_TABLE_ENTRY_WRITE("\n");
            }
        }
    }

    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");

    ADK_TABLE_ENTRY_WRITE("IP LLI 3 tuple Flow Cache Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("Core Id, Version Id, Hit Count, VRD Id, DIP, SIP, Proto, Action");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(core_id =0; core_id<ADK_MAX_NUM_CORES; core_id++)
    {
        per_core_count = ipModuleInfo_p->ip_dp_tbl_info_p->
                    fc_global_maintenance_p[core_id].lli3_fc_usage_count;
        if(per_core_count != 0)
        {
            ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
            ADK_TABLE_ENTRY_WRITE("usage_count:%d\n ", per_core_count);
            ADK_TABLE_ENTRY_WRITE("\n------------------------------------\n");
        }
        fc_aging_tbl_p = ipModuleInfo_p->ip_dp_tbl_info_p->
            lliNodeTbl_3_count_p[core_id].fc_maintenance;
        for(i=0; i<per_core_count; i++)
        {
            /*Check for used entry and print only used one*/
            if(fc_aging_tbl_p[i/4] & (1 <<((i%4)*2)))
            {
                adk_ip_dp_lli_hash3_tbl_entry_t *node_entry =
                    &(ipModuleInfo_p->ip_dp_tbl_info_p->lliNodeTbl_3_p[
                            core_id*IP_MAX_FLOWS_PER_CPU + i]);
                ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->version_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->hit);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->vrd_id);
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->dest_addr));
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->src_addr));
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->protocol);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->action);
                ADK_TABLE_ENTRY_WRITE("\n");
            }
        }
    }

    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");

    ADK_TABLE_ENTRY_WRITE("IP ULI 5 tuple Flow Cache Table Dump\n");
    ADK_TABLE_ENTRY_WRITE("Core Id, Version Id, Hit Count, VRD Id, DIP, SIP, Proto, Dport, Sport, Action");
    ADK_TABLE_ENTRY_WRITE("\n=================================================\n");

    for(core_id =0; core_id<ADK_MAX_NUM_CORES; core_id++)
    {
        per_core_count = ipModuleInfo_p->ip_dp_tbl_info_p->
                fc_global_maintenance_p[core_id].uli5_fc_usage_count;
        if(per_core_count != 0)
        {
            ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
            ADK_TABLE_ENTRY_WRITE("usage_count:%d\n ", per_core_count);
            ADK_TABLE_ENTRY_WRITE("\n------------------------------------\n");
        }
        fc_aging_tbl_p = ipModuleInfo_p->ip_dp_tbl_info_p->
            uliNodeTbl_5_count_p[core_id].fc_maintenance;
        for(i=0; i<per_core_count; i++)
        {
            /*Check for used entry and print only used one*/
            if(fc_aging_tbl_p[i/4] & (1 <<((i%4)*2)))
            {
                adk_ip_dp_uli_hash5_tbl_entry_t *node_entry =
                    &(ipModuleInfo_p->ip_dp_tbl_info_p->uliNodeTbl_5_p[
                            core_id*IP_MAX_FLOWS_PER_CPU + i]);
                ADK_TABLE_ENTRY_WRITE("%d, ", core_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->version_id);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->hit);
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->vrd_id);
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->dest_addr));
                ADK_TABLE_ENTRY_WRITE("0x%08x, ", adk_ntohl(node_entry->src_addr));
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->protocol);
                ADK_TABLE_ENTRY_WRITE("0x%x, ", adk_ntohs(node_entry->dest_port));
                ADK_TABLE_ENTRY_WRITE("0x%x, ", adk_ntohs(node_entry->src_port));
                ADK_TABLE_ENTRY_WRITE("%d, ", node_entry->action);
                ADK_TABLE_ENTRY_WRITE("\n");
            }
        }
    }

    ADK_TABLE_ENTRY_WRITE("=================================================\n\n");

    return ADK_SUCCESS;
}

void adk_ip_dip_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm,
                          __attribute__((unused)) void *rock,
                           uint8_t *key,
                           uint16_t depth,
                           adk_handle_t value)
{
    int i;

    adk_ip_dp_route_info_t *dipResult = (adk_ip_dp_route_info_t *) value;

    ADK_TABLE_ENTRY_WRITE("0x%02x, ", key[0]); /*vrdid*/
    ADK_TABLE_ENTRY_WRITE("0x");
    for(i=1;i<=4;i++)
        ADK_TABLE_ENTRY_WRITE("%02x", key[i]);

    ADK_TABLE_ENTRY_WRITE(", %d, ", (depth-8)); /*mask*/

    if(dipResult)
    {
        ADK_TABLE_ENTRY_WRITE("0, ");
        ADK_TABLE_ENTRY_WRITE("%d, ", dipResult->isTerminating);
        if(dipResult->isTerminating)
        {
            ADK_TABLE_ENTRY_WRITE("0x0, ");
            ADK_TABLE_ENTRY_WRITE("0, ");
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("0, ");
            ADK_TABLE_ENTRY_WRITE("0x0, ");
            ADK_TABLE_ENTRY_WRITE("0x%x, ", dipResult->u.nh_if.nextHop);
            ADK_TABLE_ENTRY_WRITE("%d, ", dipResult->u.nh_if.interfaceId);
        }
    }
    else
    {
        ADK_TABLE_ENTRY_WRITE("1, ");
        ADK_TABLE_ENTRY_WRITE("0, ");
        ADK_TABLE_ENTRY_WRITE("0, ");
        ADK_TABLE_ENTRY_WRITE("0, ");
        ADK_TABLE_ENTRY_WRITE("0x0, ");
        ADK_TABLE_ENTRY_WRITE("0x0, ");
        ADK_TABLE_ENTRY_WRITE("0, ");
    }

    ADK_TABLE_ENTRY_WRITE("\n");

    return;
}

void adk_ip_sip_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm,
                          __attribute__((unused)) void *rock,
                           uint8_t *key,
                           uint16_t depth,
                           __attribute__((unused)) adk_handle_t value)
{
    int i;

    ADK_TABLE_ENTRY_WRITE("0x%02x, ", key[0]); /*vrdid*/
    ADK_TABLE_ENTRY_WRITE("0x");
    for(i=1;i<=4;i++)
        ADK_TABLE_ENTRY_WRITE("%02x", key[i]);

    ADK_TABLE_ENTRY_WRITE(", %d, ", (depth-8)); /*mask*/

    ADK_TABLE_ENTRY_WRITE("\n");

    return;
}
