/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include <adk_ip_pbr_acl.h>
#include <adk_ip_common.h>

#include "adk_dpdk.h"

/*
 * Rule and trace formats definitions.
 */

enum {
        FIRST_BYTE_FIELD_IPV4_PBR,
        SRC_FIELD_IPV4_PBR,
        NUM_FIELDS_IPV4_PBR
};

enum {
        FIRST_BYTE_INPUT_IP_PBR,
        SRC_INPUT_IPV4_PBR,
};

static struct rte_acl_field_def pbr_defs[NUM_FIELDS_IPV4_PBR] = {
        /* first input field - always one byte long. */
        {
                .type = RTE_ACL_FIELD_TYPE_BITMASK,
                .size = sizeof (uint8_t),
                .field_index = FIRST_BYTE_FIELD_IPV4_PBR,
                .input_index = FIRST_BYTE_INPUT_IP_PBR,
                .offset = offsetof (adk_ip_pbr_ptn_t, first_byte),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = SRC_FIELD_IPV4_PBR,
                .input_index = SRC_INPUT_IPV4_PBR,
                .offset = offsetof(adk_ip_pbr_ptn_t, sip),
        },
};

RTE_ACL_RULE_DEF(pbr_rule, NUM_FIELDS_IPV4_PBR);

adk_ip_pbr_tree_t *adk_ip_pbr_acl_create(adk_module_handle_t ipHandle, uint8_t index)
{
    char name[20];
    struct rte_acl_ctx * ctx=NULL;
    struct rte_acl_param prm;
    
    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);
    snprintf(name, 20, "ADK_IP_PBR_%1d", index);
    prm.name = name;
    prm.socket_id = SOCKET_ID_ANY;
    prm.rule_size = RTE_ACL_RULE_SZ(NUM_FIELDS_IPV4_PBR);
    prm.max_rule_num = ADK_IP_MAX_PBR_RULES + 1;

    ADK_IPV4_LOG_EXT_API(ipHandle, "rte_acl_create", "creating new ACL context: %s\n", prm.name);
    ctx = rte_acl_create(&prm);

    return((adk_ip_pbr_tree_t *)ctx);
}

int adk_ip_pbr_ruleset_free(adk_module_handle_t ipHandle, adk_ip_pbr_tree_t *pbr_tree)
{
    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);
    ADK_IPV4_LOG_EXT_API(ipHandle, "rte_acl_free", "de-allocating all memory used by ACL context\n");
    rte_acl_free((struct rte_acl_ctx *)pbr_tree);
    return 1;
}

int adk_ip_pbr_rule_add(adk_module_handle_t ipHandle,
		 	adk_ip_pbr_tree_t *pbr_tree,
		 	adk_ip_pbr_acl_rule_t *pbr_rule,
		 	adk_ip_pbr_val_t  pbr_val,
		 	uint32_t precedence)
{
    int rc;
    struct pbr_rule v;

    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);
    ADK_MEMSET(&v, 0, sizeof(v));

    v.field[FIRST_BYTE_FIELD_IPV4_PBR].value.u8 = 0;
    v.field[FIRST_BYTE_FIELD_IPV4_PBR].mask_range.u8  = 0xFF;

    v.field[SRC_FIELD_IPV4_PBR].value.u32 = pbr_rule->sip_addr;
    v.field[SRC_FIELD_IPV4_PBR].mask_range.u32 = pbr_rule->sip_mask;
    if(0xFF == pbr_rule->sip_mask)
        v.field[SRC_FIELD_IPV4_PBR].mask_range.u32 = 0xFFFFFFFF;

    v.data.category_mask = 1 /*LEN2MASK(RTE_ACL_MAX_CATEGORIES)*/;
    v.data.priority = precedence;
    v.data.userdata = (uint32_t) (size_t)pbr_val;

    ADK_IPV4_LOG_EXT_API(ipHandle, "rte_acl_add_rules", "adding acl rules to ACL context\n");
    rc = rte_acl_add_rules((struct rte_acl_ctx *)pbr_tree, (struct rte_acl_rule *)&v, 1);
    if (rc != 0) {
        return 0;
    }

    return 1;
}

int adk_ip_pbr_ruleset_commit(adk_module_handle_t ipHandle, adk_ip_pbr_tree_t *pbr_tree)
{
    struct rte_acl_config cfg;
    int ret;

    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);
    ADK_MEMSET(&cfg, 0, sizeof(cfg));
    cfg.num_fields = NUM_FIELDS_IPV4_PBR;
    ADK_MEMCPY(&cfg.defs, pbr_defs, sizeof(pbr_defs));
    cfg.num_categories = 1;
    cfg.max_size = 0;

    ADK_IPV4_LOG_EXT_API(ipHandle, "rte_acl_build", "building ACL context\n");
    ret = rte_acl_build((struct rte_acl_ctx *)pbr_tree, &cfg);
    if (ret != 0)
    {
        return 0;
    }
    return 1;

}

void adk_ip_pbr_ruleset_clear(adk_module_handle_t ipHandle, adk_ip_pbr_tree_t *pbr_tree)
{
    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipHandle);
    /* Remove all the rules. Does not clear the internal run time memory */
    ADK_IPV4_LOG_EXT_API(ipHandle, "rte_acl_reset_rules", "deleting all rules from ACL context\n");
    rte_acl_reset_rules((struct rte_acl_ctx *)pbr_tree);
}
