/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include <math.h>
#include "adk_sys_lcore.h"
#include <adk_ip_common.h>
#ifdef IP_FLOW_HASH_CACHE
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#endif
#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#include <adk_hos_dp.h>
#include <adk_qos_dp_export.h>
#include <adk_cksum.h>
#include <adk_timer.h>
#include <adk_pmu.h>
#include <adk_ip_dp_internal.h>
#include <adk_ip_dp_internal_fc.h>

#include "adk_ph_dev_dp.h"

#ifdef BLOCK_COUNT_ENABLED
uint64_t ip_uli_rx_em_init, ip_uli_tx_em_init;
uint64_t ip_uli_rx_ts_1, ip_uli_tx_ts_1;
uint64_t ip_uli_rx_ts_2, ip_uli_tx_ts_2;
uint64_t ip_lli_rx_ts_1, ip_lli_tx_ts_1;
uint64_t ip_lli_rx_ts_2, ip_lli_tx_ts_2;
#endif

/*Module info pointer stored for DP process to access tables*/
adk_ip_dp_tbl_info_t *ip_dp_tbl_info_p;

/*Reassembly table accessed only by dp process*/
adk_ip_dp_reassembly_tbl_t *ipReassemblyTbl;
struct rte_mempool *ipReassemblySessionTbl;


/*PBR per rule counter*/
uint32_t *pbrRuleStats;

/*PBR action table*/
adk_ip_dp_pbr_action_t *pbrActionTable_p;

/*Per VRD, CP version id*/
uint16_t *ip_cp_version_id_p;

ipIdSeqDB_t ipIdSeqDB[ADK_MAX_NUM_CORES];
ipIdSeqRange_t ipIdSeqRange[ADK_MAX_NUM_CORES];

/* flow-cache tables are thread-local copies so as to scale for
 * multi-core environment.
 */
#ifdef IP_FLOW_HASH_CACHE
adk_ip_dp_fc_count_t *fc_global_maintenance_p;

/* 3-tuple (SIP, DIP, Protocol) based flow-cache table */
adk_ip_dp_flow_cache_entry_t *lliCacheTbl_3_p;
adk_ip_dp_lli_hash3_tbl_entry_t *lliNodeTbl_3_p;
adk_ip_dp_fc_maintenance_t *lliNodeTbl_3_count_p;

/* 5-tuple (SIP, DIP, Protocol, SPort, DPort) based flow-cache table */
adk_ip_dp_flow_cache_entry_t *lliCacheTbl_5_p;
adk_ip_dp_lli_hash5_tbl_entry_t *lliNodeTbl_5_p;
adk_ip_dp_fc_maintenance_t *lliNodeTbl_5_count_p;

/* 5-tuple based flow-cache table */
adk_ip_dp_flow_cache_entry_t *uliCacheTbl_5_p;
adk_ip_dp_uli_hash5_tbl_entry_t *uliNodeTbl_5_p;
adk_ip_dp_fc_maintenance_t *uliNodeTbl_5_count_p;
#endif

uint8_t ip_dscp_map[MAX_DSCP_VALUES] =
      {0, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID,
       1, INVALID_DSCP_ID, 2, INVALID_DSCP_ID, 3, INVALID_DSCP_ID, 4, INVALID_DSCP_ID,
       5, INVALID_DSCP_ID, 6, INVALID_DSCP_ID, 7, INVALID_DSCP_ID, 8, INVALID_DSCP_ID,
       9, INVALID_DSCP_ID, 10, INVALID_DSCP_ID, 11, INVALID_DSCP_ID, 12, INVALID_DSCP_ID,
       13, INVALID_DSCP_ID, 14, INVALID_DSCP_ID, 15, INVALID_DSCP_ID, 16, INVALID_DSCP_ID,
       17, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, 18, INVALID_DSCP_ID,
       19, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID,
       20, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID, INVALID_DSCP_ID
      };

/*!
 * =============================================================
 * adk_ip_dp_init()
 * 
 * IP DP Init function
 * =============================================================
 */
adk_status_t
adk_ip_dp_init(void)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t  i;    
    const struct rte_memzone *mz = NULL;
    uint32_t *worker_lcores_list=NULL, num_worker_lcores=0;

    /* Lookup the shared memory that gets created by control plane IP
     * module initialization */
    mz = rte_memzone_lookup(ADK_IP_SHM_MEMZONE_NAME);
    if (mz == NULL)
    {
        ADK_IP_DP_LOG_FATAL_ERROR("Failed to lookup IP shared memory");
        return ADK_DP_E_ERROR;
    }

    /* Check if the IP module has been completely initialized */
    ip_dp_tbl_info_p = (adk_ip_dp_tbl_info_t *) mz->addr;
    if (ip_dp_tbl_info_p->cookie != ADK_IPV4_MODULE_COOKIE)
    {
        ADK_IP_DP_LOG_FATAL_ERROR("IP Module Initialization from control plane "
                    "not yet complete. Please wait and try again later.");
        return ADK_DP_E_ERROR;
    }

    /* Initialize the other global variable used in DP*/
    ipReassemblyTbl = ip_dp_tbl_info_p->ipReassemblyTbl;
    ipReassemblySessionTbl = ip_dp_tbl_info_p->ipReassemblySessionTbl;

    pbrRuleStats = ip_dp_tbl_info_p->pbrRuleStats ;
    pbrActionTable_p = ip_dp_tbl_info_p->pbrActionTable_p;
    ip_cp_version_id_p = ip_dp_tbl_info_p->ip_cp_version_id ;

#ifdef IP_FLOW_HASH_CACHE

    fc_global_maintenance_p = ip_dp_tbl_info_p->fc_global_maintenance_p; 
    lliCacheTbl_5_p = ip_dp_tbl_info_p->lliCacheTbl_5_p;
    lliNodeTbl_5_p = ip_dp_tbl_info_p->lliNodeTbl_5_p;
    lliNodeTbl_5_count_p = ip_dp_tbl_info_p->lliNodeTbl_5_count_p;

    lliCacheTbl_3_p = ip_dp_tbl_info_p->lliCacheTbl_3_p;
    lliNodeTbl_3_p = ip_dp_tbl_info_p->lliNodeTbl_3_p;
    lliNodeTbl_3_count_p = ip_dp_tbl_info_p->lliNodeTbl_3_count_p;

    uliCacheTbl_5_p = ip_dp_tbl_info_p->uliCacheTbl_5_p;
    uliNodeTbl_5_p = ip_dp_tbl_info_p->uliNodeTbl_5_p;
    uliNodeTbl_5_count_p = ip_dp_tbl_info_p->uliNodeTbl_5_count_p;

#endif

    /* Initialize IP Sequence Number Range per DP core */
    worker_lcores_list = ip_dp_tbl_info_p->worker_lcores_list;
    num_worker_lcores = ip_dp_tbl_info_p->num_worker_lcores;
    i = 0;
    ipIdSeqRange[worker_lcores_list[i]].min = 0;
    ipIdSeqRange[worker_lcores_list[i++]].max =
                         (pow(2,16) / num_worker_lcores)-1;
    for (; i < num_worker_lcores; i++)
    {
        ipIdSeqRange[worker_lcores_list[i]].min =
            ipIdSeqRange[worker_lcores_list[i - 1]].max + 1;
        ipIdSeqRange[worker_lcores_list[i]].max =
            ipIdSeqRange[worker_lcores_list[i - 1]].max +
            (pow(2, 16)/ num_worker_lcores);
    }

    for (i = 0; i < num_worker_lcores; i++)
    {
        ipIdSeqDB[worker_lcores_list[i]].ipIdSeq =
            ipIdSeqRange[worker_lcores_list[i]].min;
        ADK_IP_DP_LOG_INFO_DEBUG("CORE %d: MIN %d MAX %d \n", worker_lcores_list[i],
                ipIdSeqRange[worker_lcores_list[i]].min, 
                ipIdSeqRange[worker_lcores_list[i]].max);
    }

    /* Function ID registration */
    ret = adk_ip_dp_fn_id_init();

    return ret;

}

/* DP Fn init */
adk_status_t adk_ip_dp_fn_id_init(void)
{
    adkDestEntryFunction_g[ADK_IPV4_LLI_FN_ID]= adk_ip_dp_pkt_proc_lli;
    adkDestEntryFunction_g[ADK_IPV4_LLI_RING_FN_ID]= adk_ip_dp_pkt_proc_ring_lli;
    adkDestEntryFunction_g[ADK_IPV4_ULI_FN_ID]= adk_ip_dp_pkt_proc_uli;
    adkDestEntryFunction_g[ADK_IPV4_ULI_RING_FN_ID]= adk_ip_dp_pkt_proc_ring_uli;
    adkDestEntryFunction_g[ADK_IPV4_REASSEMBLE_FN_ID]= adk_ip_dp_pkt_proc_reassemble;
    adkDestEntryFunction_g[ADK_IPV4_REASM_TOUT_FN_ID]= adk_ip_dp_pkt_proc_reasm_tout;
    adkDestEntryFunction_g[ADK_IPV4_NETD_FN_ID]= adk_ip_dp_pkt_proc_netd;
    adkDestEntryFunction_g[ADK_IPV4_PCAP_FN_ID]= adk_ip_dp_pkt_proc_pcap;
    adkDestEntryFunction_g[ADK_IPV4_POLICING_FN_ID]= adk_ip_dp_pkt_proc_police;
#ifdef UDP_CHECKSUM_STAGE_AFTER_BB_IC_MODULE
    adkDestEntryFunction_g[ADK_UDP_ULI_FN_ID]= adk_udp_dp_pkt_proc_uli;
    adkDestEntryFunction_g[ADK_UDP_ULI_RING_FN_ID]= adk_udp_dp_pkt_proc_ring_uli;
#endif
#ifdef UDP_CHECKSUM_STAGE_AFTER_IP
    adkDestEntryFunction_g[ADK_UDP_LLI_FN_ID]= adk_udp_dp_pkt_proc_lli;
    adkDestEntryFunction_g[ADK_UDP_LLI_RING_FN_ID]= adk_udp_dp_pkt_proc_ring_lli;
#endif

    return ADK_SUCCESS;
}

/* Clone packet */
struct rte_mbuf* adk_pktmbuf_clone(struct rte_mbuf * inputPkt,
                            struct rte_mempool * pool)
{
    struct rte_mbuf *pkt_clone, *pkt_segment, *pkt, *prev;
    uint8_t nb_segs;

    pkt = inputPkt;
    nb_segs = pkt->nb_segs;
    pkt_clone = rte_pktmbuf_alloc(pool);
    if(pkt_clone == NULL)
    {
        ADK_IP_DP_LOG_ERROR("Mbuf allocation failed in adk_pktmbuf_clone");
        return NULL;
    }
 
    pkt_clone->nb_segs = pkt->nb_segs;
    pkt_clone->pkt_len = pkt->pkt_len;
    pkt_clone->port = pkt->port;

    pkt_segment = pkt_clone;
    prev = NULL;

    /*copy ADK metadata for the first segment*/
    adk_metadata_t *adk_md_cap = ADK_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)adk_md_cap,(void *) ADK_METADATA_PTR(pkt),
                offsetof(adk_metadata_t,imc_param_start));
    adk_md_cap->task_type = (uint8_t)ADK_TASK_TYPE_NEW;

    /*copy all segments payload*/
    while(nb_segs)
    {
        rte_memcpy(rte_pktmbuf_mtod(pkt_segment, void *),
                rte_pktmbuf_mtod(pkt, void *), rte_pktmbuf_data_len(pkt));
        pkt_segment->data_len = rte_pktmbuf_data_len(pkt);
        nb_segs--; 
        if(nb_segs)
        {
            prev = pkt_segment;
            pkt_segment->next = NULL;
            pkt_segment = rte_pktmbuf_alloc(pool);
            if(pkt_segment == NULL)
            {
                ADK_IP_DP_LOG_ERROR(
                    "Mbuf allocation failed in adk_pktmbuf_clone. Dropping packet!");
                break;
            }
            prev->next = pkt_segment;
            pkt= pkt->next;
        }
    }

    if(unlikely(pkt_segment == NULL))
    {
        pkt = pkt_clone;
        while(pkt != NULL)
        {
            prev= pkt->next;
            rte_pktmbuf_free(pkt); /* free all allocated segments */
            pkt = prev;
        }
        return NULL;
    }
    __rte_mbuf_sanity_check(pkt_clone, 1);
    return pkt_clone;
}

#ifdef IP_FLOW_HASH_CACHE
void
adk_ip_dp_flow_cache_monitor(void)
{
    uint8_t coreIndex = ADK_LCORE_INDEX;
    bool tbl_updated_flag=FALSE;

    /*Check if IP DP tables were initialized*/
    if(ip_dp_tbl_info_p == NULL)
    {
        return;
    }

    /*Update timer counter per core*/
    fc_global_maintenance_p[coreIndex].timer_msec++;
    
    if((fc_global_maintenance_p[coreIndex].lli5_fc_usage_count >=
                             IP_MAX_FLOWS_PER_CPU) &&
            !(fc_global_maintenance_p[coreIndex].timer_msec & 0xFF))
    {
        adk_ip_dp_lli5_fc_aging_maintenance(coreIndex,
                lliNodeTbl_5_count_p[coreIndex].fc_maintenance);
        tbl_updated_flag =TRUE;
    }

    if((fc_global_maintenance_p[coreIndex].lli3_fc_usage_count >=
                     IP_MAX_FLOWS_PER_CPU) &&
            !(fc_global_maintenance_p[coreIndex].timer_msec & 0xFF))
    {
        adk_ip_dp_lli3_fc_aging_maintenance(coreIndex,
                lliNodeTbl_3_count_p[coreIndex].fc_maintenance);
        tbl_updated_flag =TRUE;
    }

    if((fc_global_maintenance_p[coreIndex].uli5_fc_usage_count >=
                         IP_MAX_FLOWS_PER_CPU) &&
            !(fc_global_maintenance_p[coreIndex].timer_msec & 0xFF))
    {
        adk_ip_dp_uli5_fc_aging_maintenance(coreIndex,
                uliNodeTbl_5_count_p[coreIndex].fc_maintenance);
        tbl_updated_flag =TRUE;
    }

    if(tbl_updated_flag == TRUE)
        return;
    
    if(fc_global_maintenance_p[coreIndex].timer_msec >= IPV4_FC_TIMER_EPOCH)
    {
        fc_global_maintenance_p[coreIndex].timer_msec =0;
        switch(fc_global_maintenance_p[coreIndex].fc_table)
        {
            case ADK_IP_DP_FC_LLI5: 
                /*LLI 5 tuple Flow cache maintenance*/
                if(fc_global_maintenance_p[coreIndex].lli5_fc_usage_count >=
                                     IPV4_FC_THRESHOLD)
                
                {
                    adk_ip_dp_lli5_fc_aging_maintenance(coreIndex,
                            lliNodeTbl_5_count_p[coreIndex].fc_maintenance);
                }
                fc_global_maintenance_p[coreIndex].fc_table = ADK_IP_DP_FC_LLI3;
                break;

            case ADK_IP_DP_FC_LLI3:
                /*LLI 3 tuple Flow cache maintenance*/

                if(fc_global_maintenance_p[coreIndex].lli3_fc_usage_count >=
                             IPV4_FC_THRESHOLD)
                {
                    adk_ip_dp_lli3_fc_aging_maintenance(coreIndex,
                            lliNodeTbl_3_count_p[coreIndex].fc_maintenance);
                }
                fc_global_maintenance_p[coreIndex].fc_table = ADK_IP_DP_FC_ULI5;
                break;

            case ADK_IP_DP_FC_ULI5:
                /*ULI 5 tuple Flow cache maintenance*/
                if(fc_global_maintenance_p[coreIndex].uli5_fc_usage_count >=
                                         IPV4_FC_THRESHOLD)
                {
                    adk_ip_dp_uli5_fc_aging_maintenance(coreIndex,
                            uliNodeTbl_5_count_p[coreIndex].fc_maintenance);
                }
                fc_global_maintenance_p[coreIndex].fc_table = ADK_IP_DP_FC_LLI5;
                break;

            default:
                fc_global_maintenance_p[coreIndex].fc_table = ADK_IP_DP_FC_LLI5;
                break;
        }
    }
}
#endif

/*!
 * =============================================================
 * adk_ip_dp_pkt_proc_pacp()
 * =============================================================
 */
adk_status_t adk_ip_dp_pkt_proc_pcap (void **task_p,
          __attribute__((unused)) uint8_t task_count,
          __attribute__((unused)) uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          __attribute__((unused)) int coreIndex)
{
    adk_status_t st;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(task_p[0]);
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(ph_task);
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(ph_task);
    uint32_t intfId = dsi_md_p->outer_id & ADK_HOS_OUTER_IP_ID_MASK;
    adk_ip_dp_interface_entry_t *intfEntry;
    ip_ph_metadata_t *ph_metadata_p;
    adk_ph_dev_dp_metadata_t *phMeta_p;
    uint8_t *pdu;
    adk_ip_direction_t dir;

    ADK_PDU_OFFSET_PTR_GET(ph_task, pdu, uint8_t *, 0);

    ADK_IP_DP_LOG_INFO_DEBUG("IP DP PCAP function for packet\n");

    if(intfId >= ADK_DP_MAX_IP_INTERFACE)
    {
        /*TODO global stats*/
        ADK_IP_DP_LOG_INFO("Wrong intf id, dropping IP pcap packet!");
        adk_pktio_task_buffer_free(ph_task);
        return ADK_SUCCESS;
    }

#if ADK_HOS_OLD_IMPL
    if(adk_hos_is_pcap_egress(dsi_md_p))
    {
        dir = ADK_IP_DIR_EGRESS;
        /* Change exception domain to non-TN port so that the different
         * intf_id's are not decoded by CP decode function*/
        adk_md_p->exception_domain = ADK_FP_ORIGIN_DOMAIN;
    }
#endif
    else
    {
        dir = ADK_IP_DIR_INGRESS;
    }

    intfEntry = &(ip_dp_tbl_info_p->ipInterfaceTble[intfId]);
    if((intfEntry->ifType != ADK_IP_DP_INTF_TYPE_ETH)
       || (intfEntry->pkt_cap_info[dir].enableCapture == false))
    {
        //ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipHwPcapDrops++;
        ADK_IP_DP_LOG_INFO("Mis config, dropping IP pcap packet!");
        adk_pktio_task_buffer_free(ph_task);
        return ADK_SUCCESS;
    }
   
    /*Set ADK MD*/
    adk_md_p->input_intf_id = intfId;
    /* Ether type, VLAN if present */
    if (adk_ntohs(*((uint16_t *)&pdu[12])) == 0x8100)
    {
        ADK_MD_GET_IP_OFF(ph_task) = 18;
    }
    else
    {
        ADK_MD_GET_IP_OFF(ph_task) = 14;
    }

    /* Update PH metadata */
    phMeta_p = (adk_ph_dev_dp_metadata_t *)
            &(ADK_MD_GET_IMC_PARAM_START(ph_task));

    ADK_MD_SET_ADK_DEV_ID(ph_task, intfEntry->
                                    pkt_cap_info[dir].devId);
    ADK_MD_SET_DEV_TX_QUEUE_ID(ph_task, intfEntry->
                                    pkt_cap_info[dir].queueId);
    phMeta_p->priority = 0;
    phMeta_p->cbFnId = intfEntry->pkt_cap_info[dir].cbFnId;
    phMeta_p->policerFlowId = UINT_MAX;

    ph_metadata_p = (ip_ph_metadata_t *)&phMeta_p->imc_param_start;
    ADK_METADATA_PARAM(ph_task, pdu_flow_id) =
        intfEntry->pkt_cap_info[dir].context;
    ph_metadata_p->exception_type = ADK_IP_NO_EXCEPTION;

    st = adk_ph_dev_dp_to_cp_send((void *)ph_task, ADK_PAL_DP_BATCH_CB_FN_IP, NULL);
    if(st != ADK_SUCCESS)
    {
        //ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipPktCaptureFailed++;
        adk_pktio_task_buffer_free(ph_task);
        if(ADK_ERROR_SEV_LEVEL_SET(ADK_E_RX_BUFFER_FULL) == st)
        {
            ADK_IP_DP_LOG_INFO_DEBUG(
            "Capture pkt enqueue failed. Stopping Capture!\n");
            intfEntry->pkt_cap_info[dir].enableCapture =
                                        false;
        }
    }

    return ADK_SUCCESS;
}

adk_status_t adk_ip_dp_send_exception(adk_pktio_task_t *ph_task,
                                        uint8_t type, int coreIndex,
                                        uint8_t routeTableId, adk_ip_direction_t dir)
{
    adk_ph_dev_dp_metadata_t *phMeta_p;
#ifdef ADK_QOS_GEN3_INTERFACE
    adk_slowpath_dev_dp_metadata_t *spDevMd_p;
#endif

    adk_ip_dp_exception_handler_t * exceptionHdl
             = ip_dp_tbl_info_p->exceptionHandlerTble[type];
    adk_pktio_task_t *next_task;
    
    if(!exceptionHdl)
    {
        ADK_IP_DP_LOG_INFO_DEBUG("No IP Exception handler configured for Exception");
        ADK_IP_DP_LOG_INFO_DEBUG(" Type:%d. Dropping the packet!!\n",
                           type);
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipNoExceptionRegisteredDrops++;
        return ADK_E_DROP_TASK;
    }

    while(ph_task)
    {
        next_task = (adk_pktio_task_t *)ADK_MD_GET_NEXT_FRAG(ph_task);
        ADK_IP_DP_LOG_INFO_DEBUG("IP Exception handler action STH via PH!!\n");
        /*Set PH meta data*/
#if 0
        //TODO enable host policing for PH path.
#ifdef ADK_QOS_GEN3_INTERFACE
        spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task));
        phMeta_p = (adk_ph_dev_dp_metadata_t *)&(spDevMd_p->imc_param_start);
        spDevMd_p->policerFlowId = exceptionHdl->policerId;
        ADK_METADATA_PARAM(ph_task, egress_stats)[ADK_SYS_EGRESS_STATS_MDATA_QUEUE_LOC].stats_info = exceptionHdl->statsId;
#else
#endif
#endif
        /* Update PH metadata */
        phMeta_p = (adk_ph_dev_dp_metadata_t *)
                        &(ADK_MD_GET_IMC_PARAM_START(ph_task));
        phMeta_p->policerFlowId = exceptionHdl->policerId;

        ADK_MD_SET_ADK_DEV_ID(ph_task, exceptionHdl->devId);
        ADK_MD_SET_DEV_TX_QUEUE_ID(ph_task, exceptionHdl->queueId);
        phMeta_p->priority = exceptionHdl->priority;
        phMeta_p->cbFnId = exceptionHdl->cbFnId;
        phMeta_p->moduleId = ADK_IP_MODULE_ID;

        ip_ph_metadata_t *ph_metadata_p;
        ph_metadata_p = (ip_ph_metadata_t *)&phMeta_p->imc_param_start;
        ADK_METADATA_PARAM(ph_task, pdu_flow_id) = 0;

        /* Exception Data */
        ph_metadata_p->exception_type = type;
        ph_metadata_p->dir = dir;
        ph_metadata_p->interface_id = ADK_METADATA_PARAM(ph_task, input_intf_id);
        ph_metadata_p->routeTableId = routeTableId;

        if(!next_task) // for last frag of chain mbuf next_task_p will be NULL
        {
            ADK_METADATA_PARAM(ph_task, task_type) = ADK_TASK_TYPE_FWD;
        }
        else
        {
            ADK_METADATA_PARAM(ph_task, task_type) = ADK_TASK_TYPE_NEW;
        }

        if(ADK_SUCCESS != adk_ph_dev_dp_to_cp_send((void *)ph_task, ADK_PAL_DP_BATCH_CB_FN_IP, exceptionHdl))
        {
            adk_pal_dp_release_send(ph_task, ADK_PAL_DP_BATCH_CB_FN_IP, NULL);
            adk_pktio_task_buffer_free(ph_task);
        }
        ph_task = next_task;
    }
    return ADK_E_TASK_CONSUMED;

}


/*!
 * =============================================================
 * adk_ip_dp_pbr()
 * 
 * Performs the pbr lookup & stores the action table entry 
 * index in the passed pointer.
 * The possible return values
 * TRUE: If PBR lookup successfull
 * FALSE: If PBR lookup fails and fall-back to DIP based lookup
 * =============================================================
 */
bool adk_ip_dp_pbr( uint32_t src_addr_nw, adk_ip_pbr_val_t *pbr_val)
{
    adk_status_t ret;
    adk_ip_pbr_ptn_t pbr_ptn;
    adk_ip_pbr_ctx_t *pbrCtx_p = (adk_ip_pbr_ctx_t *)
                    (ip_dp_tbl_info_p->ipPbrInfo_p);

    pbr_ptn.sip = src_addr_nw;
    pbr_ptn.first_byte = 0;

    /* PBR enabled, but mapped ruleset to interface is invalid */
    if(!pbrCtx_p)
    {
        ADK_IP_DP_LOG_INFO("PBR uninitialized!\n");
        return FALSE;
    }

    ADK_IP_DP_LOG_INFO_DEBUG("Doing PBR Lookup on 0x%x\n", adk_ntohl(src_addr_nw));
    /* lookup PBR ACL */
    ret = adk_ip_dp_pbr_lookup(pbrCtx_p->pbr_tree[pbrCtx_p->pbr_tree_index],
                            &pbr_ptn, pbr_val);
    if(unlikely(!ret || !(*pbr_val)))
    {
        ADK_IP_DP_LOG_INFO("Entry not found in PBR ACL tree");
        return FALSE;
    }

    return TRUE;
}

adk_dp_status_t
adk_ip_dp_qos_policy_get(uint32_t ifres_index, uint8_t dscp,
                         uint32_t *queueId)
{
    uint32_t hash_val,hash_index,i;
    adk_ip_qostbl_ovflow_t *temp;

    if(ifres_index >= ADK_DP_MAX_IP_INTERFACE)
    {
        ADK_IP_DP_LOG_ERROR("IP interface id larger than the max supported");
        return ADK_DP_E_ERROR;
    }

    hash_val = SISD_CRC_HASH(0,ifres_index);
    hash_val = SISD_CRC_HASH(hash_val, dscp);

    hash_index = hash_val & ADK_IPV4_QOS_TBL_HASH_MASK;
    hash_index *= ADK_IPV4_QOS_TBL_MAX_COLLISION;
    
    for(i =0; i< ADK_IPV4_QOS_TBL_MAX_COLLISION ;i++)
    {
        if((ip_dp_tbl_info_p->ipQosTbl[hash_index + i].qos_entry_p != NULL)&&
            (ip_dp_tbl_info_p->ipQosTbl[hash_index + i].hash_val == hash_val))
        {
            if((ip_dp_tbl_info_p->ipQosTbl[hash_index + i].qos_entry_p->ifid == ifres_index)&&
                (ip_dp_tbl_info_p->ipQosTbl[hash_index + i].qos_entry_p->dscp == dscp))
            {
                *queueId = ip_dp_tbl_info_p->ipQosTbl[hash_index + i].qos_entry_p->queueid;
                goto done;
            }
        }
    } 
    if(i == ADK_IPV4_QOS_TBL_MAX_COLLISION)
    {
        temp = ip_dp_tbl_info_p->ipQos_ovflow_list_head;
        while(temp != NULL)
        {
            if((temp->ifid == ifres_index)&&(temp->dscp == dscp))
            {
                *queueId = temp->queueid;
                return ADK_SUCCESS;
            }
            temp = temp->next;
        }
     	return ADK_DP_E_ERROR;
    }
done:
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ip_dp_dip_get(uint8_t vrd_id,
                  uint32_t dip,
                  adk_handle_t *dipResult)
{
    unsigned char key[(IP_DIP_TBL_WIDTH/8)+1] = {};
    int size = 0;
    adk_handle_t lpmVal;

    if(unlikely(ADK_DP_E_ERROR ==
                adk_ip_dp_diptbl_key_create(vrd_id, dip, 32, key, &size)))
    {
        return ADK_DP_E_ERROR;
    }

    //ADK_IP_DP_LOG_INFO_DEBUG("Searching entry for Dest IP %d.%d.%d.%d in DIP table ...\n",
	  // dip&0x000000ff, (dip>>8)&0x000000ff, (dip>>16)&0x000000ff, (dip>>24)&0x000000ff);
    
    lpmVal = adk_lpm_lookup(ip_dp_tbl_info_p->dipTbl, key);
    if(unlikely(ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID) == lpmVal))
    {
	return ADK_DP_E_ERROR;
    }
    *dipResult = lpmVal;
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ip_dp_sip_get(uint8_t vrd_id,
                  uint32_t sip,
                  adk_handle_t *sipResult)
{
    unsigned char key[(IP_SIP_TBL_WIDTH/8)+1] = {};
    int size = 0;
    adk_handle_t lpmVal;

    if(unlikely(ADK_DP_E_ERROR ==
                adk_ip_dp_siptbl_key_create(vrd_id, sip, 32, key, &size)))
    {
        return ADK_DP_E_ERROR;
    }

    //ADK_IP_DP_LOG_INFO_DEBUG("Searching entry for Vrd Id:0x%x Src IP:%d.%d.%d.%d in SIP table ...\n",
	  // vrd_id, sip&0x000000ff, (sip>>8)&0x000000ff, (sip>>16)&0x000000ff, (sip>>24)&0x000000ff);
    
    lpmVal = adk_lpm_lookup(ip_dp_tbl_info_p->sipTbl, key);
    if(unlikely(ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID) == lpmVal))
    {
	return ADK_DP_E_ERROR;
    }
    *sipResult = lpmVal;
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ip_dp_dest_mtu_get(uint16_t ipVrdId,
                       uint32_t dipAddr_nw,
                       uint16_t *mtu_p)
{
    uint32_t hash_val,hash_index,i;

    ADK_IP_DP_LOG_INFO_DEBUG("Dest MTU Tbl: Searching with VrdId 0x%x, DIP 0x%x ...\n",
	   ipVrdId, dipAddr_nw);
    
    hash_val = SISD_CRC_HASH(0,ipVrdId);
    hash_val = SISD_CRC_HASH(hash_val,dipAddr_nw);

    hash_index = hash_val & ADK_IPV4_DST_MTU_TBL_HASH_MASK; 
    /*hash table is allocated as 1D array. Hence multiply with collision width*/
    hash_index *= ADK_IPV4_DST_MTU_TBL_MAX_COLLISION;
    
    for(i=0; i< ADK_IPV4_DST_MTU_TBL_MAX_COLLISION; i++)
    {
        /* DIP is overloaded to denote present flag
         * (DIP == 0) => not present*/
        if(ip_dp_tbl_info_p->ipDestMtuTbl[hash_index + i].dipAddr_nw)
        {
            if((ip_dp_tbl_info_p->ipDestMtuTbl[hash_index + i].dipAddr_nw == dipAddr_nw)
               &&(ip_dp_tbl_info_p->ipDestMtuTbl[hash_index + i].vrdid == ipVrdId))
            {
                /*Exact match found. return MTU*/
                *mtu_p = ip_dp_tbl_info_p->ipDestMtuTbl[hash_index + i].mtu;
                return ADK_SUCCESS;
            }
        }
    }

    /*If here means entry not present*/
    return ADK_DP_E_ERROR;
}

adk_dp_status_t adk_ip_dp_diptbl_key_create(uint8_t vrd_id,
                                            uint32_t dip,
                                            uint8_t mask,
                                            unsigned char *key,
					    int *size)
{
    uint32_t *tmp_32;
    key[0] = vrd_id;
    tmp_32 = (uint32_t *) (void *)(&key[1]);
    *tmp_32 = dip;
    *size = 8 + mask;
    return ADK_SUCCESS;
}

adk_dp_status_t adk_ip_dp_siptbl_key_create(uint8_t vrd_id,
                                            uint32_t sip,
                                            uint8_t mask,
                                            unsigned char *key,
					    int *size)
{
    uint32_t *tmp_32;
    key[0] = vrd_id;
    tmp_32 = (uint32_t *) (void *)(&key[1]);
    *tmp_32 = sip;
    *size = 8 + mask;
    return ADK_SUCCESS;
}



void adk_ip_dp_egressflowtbl_key_create(uint32_t l2If,
			                uint32_t nh,
			                uint8_t *key)
{
#ifdef ADK_DEBUG
    int i;
#endif
    ADK_MEMCPY(key,(uint8_t *)&l2If, 3 * sizeof(uint8_t));
    ADK_MEMCPY(key+3, (uint8_t *)&nh, sizeof(uint32_t));
#ifdef ADK_DEBUG
    ADK_IP_DP_LOG_INFO_DEBUG("IP Egress Flow Table Key is - ");
    for(i=0;i<7;i++) ADK_IP_DP_LOG_INFO_DEBUG("%x ",key[i]);
    ADK_IP_DP_LOG_INFO_DEBUG("\n");
#endif
}
            
int adk_ip_dp_acl_lookup(adk_ip_acl_tree_t *acl_tree,
		      adk_ip_acl_ptn_t  *acl_ptn,
		      adk_ip_acl_val_t  *acl_val)
{
    int ret;
    const uint8_t * data[] = {(uint8_t *)acl_ptn};

    ret = rte_acl_classify_alg((struct rte_acl_ctx *)acl_tree, data,
                               (uint32_t *) acl_val, 1, 1,
                               RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_IP_DP_LOG_INFO_1D("Error in IP ACL classify: %d",
                               ret);
        return 0;
    }
    
    return 1;
}

adk_dp_status_t adk_ip_dp_ing_classifier_lookup(adk_ip_ing_classifier_ctx_t *ipIngClassifierCtx_p,
		      adk_ip_ing_classifier_ptn_t  *ingClassifierPtn,
		      uint32_t  *result_p)
{
    int ret;
    const uint8_t * data[] = {(uint8_t *)ingClassifierPtn};

    ret = rte_acl_classify_alg((struct rte_acl_ctx *)ipIngClassifierCtx_p, data,
                               result_p, 1, 1,
                               RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_IP_DP_LOG_INFO_1D("Error in rte_acl_classify_alg for IP Ingress Classifier: %d",
                               ret);
        return ADK_E_INTERNAL_ERROR;
    }
    
    return ADK_SUCCESS;
}/* adk_ip_dp_ing_classifier_lookup */
int adk_ip_dp_pbr_lookup(adk_ip_pbr_tree_t *pbr_tree,
		      adk_ip_pbr_ptn_t  *pbr_ptn,
		      adk_ip_pbr_val_t  *pbr_val)
{
    int ret;
    const uint8_t * data[] = {(uint8_t *)pbr_ptn};

    ret = rte_acl_classify_alg((struct rte_acl_ctx *)pbr_tree, data,
                               (uint32_t *) pbr_val, 1, 1,
                               RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_IP_DP_LOG_INFO_1D("Error in IP PBR ACL classify: %d",
                               ret);
        return 0;
    }
    
    return 1;
}

adk_dp_status_t adk_ip_dp_egress_qos_classifier_lookup(
        adk_ip_egress_qos_classifier_ctx_t *ipEgressQoSClassifierCtx_p,
        adk_ip_egress_qos_classifier_ptn_t  *egressQoSClassifierPtn,
        uint32_t  *result_p)
{
    int ret;
    const uint8_t * data[] = {(uint8_t *)egressQoSClassifierPtn};

    ret = rte_acl_classify_alg((struct rte_acl_ctx *)ipEgressQoSClassifierCtx_p, data,
                               result_p, 1, 1,
                               RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_IP_DP_LOG_INFO_1D("Error in rte_acl_classify_alg for IP QoS Classifier: %d",
                               ret);
        return ADK_E_INTERNAL_ERROR;
    }
    
    return ADK_SUCCESS;
}/* adk_ip_dp_egress_qos_classifier_lookup*/

void adk_ip_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess)
{
    RTE_SET_USED(dpdkMbuf);
    /*Current worker core id */
    uint32_t coreIndex = ADK_LCORE_INDEX;

    switch(cbEventType)
    {
        case ADK_PAL_DP_BATCH_CB_EVENT_EM:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER:
            {
                /* Note: EM_ORDER callback will be called when packet is sent to em stage function but not after `actual` em_send. This
                    can cause incorrect flow stats increment in case em_send fails. This is known issue and is TBD */
                uint32_t flow_id = (uintptr_t)cbParam;
                debug_print("adk_ip_dp_pal_batch_cb_fn called for packet sent to EM\n");
                if(sendSuccess){
                    ip_dp_tbl_info_p->ipIngFlowStats[coreIndex * ADK_DP_MAX_IP_ING_ASSOC_FLOWS + flow_id].txPkts++;
                    INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInSentToEoPkts, 1);
                }
                else
                {
                    ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInSendToEoFailDrops++;
                    adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                }
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_NETD:
                if(!sendSuccess)
                {
                    ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInSendToHostFailDrops++;
                    adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                }
            debug_print("adk_ip_dp_pal_batch_cb_fn called for packet sent to NETD\n");
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_PH:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_PH_POLICER:
                debug_print("adk_ip_dp_pal_batch_cb_fn called for exception packet\n");
                if(cbParam != NULL)
                {
                    adk_ip_dp_exception_handler_t * exceptionHdl = cbParam;
                    adk_ph_dev_pal_batch_handle_return(exceptionHdl->devId, exceptionHdl->queueId,
                            sendSuccess, cbEventType);
                }
                if(!sendSuccess)
                {
                    ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipExceptionSendFailureDrops++;
                    adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_RELEASE:
                debug_print("adk_ip_dp_pal_batch_cb_fn called for packet released (dropped, parked for reassembly)\n");
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER:
        case ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO:
        case ADK_PAL_DP_BATCH_CB_EVENT_TIMER:
        case ADK_PAL_DP_BATCH_CB_EVENT_NIC:
            break;
        default:
            break;
    }
} /* adk_ip_dp_pal_batch_cb_fn */


#if 0
adk_dp_status_t
adk_ip_dp_police_action( uint8_t policeColor,
                         adk_ip_qos_policing_params_t *qos_police_params_p)
{
    adk_ip_qos_police_action_t action;
    uint32_t __attribute__((unused)) value;
    switch (policeColor)
    {
    case ADK_QOS_DP_RED:
        action = qos_police_params_p->violateAction;
        value = qos_police_params_p->violateValue;
        break;

    case ADK_QOS_DP_YELLOW:
        action = qos_police_params_p->exceedAction;
        value = qos_police_params_p->exceedValue;
        break;

    case ADK_QOS_DP_GREEN:
        action = qos_police_params_p->conformAction;
        value = qos_police_params_p->conformValue;
        break;

    default:
        ADK_IP_DP_LOG_INFO("Invalid Police Color: Dropping packet");
        return ADK_DP_E_ERROR;
    }

    switch(action)
    {
    case ADK_IP_POLICE_ACTION_SEND:
        return(ADK_SUCCESS);

    case ADK_IP_POLICE_ACTION_DROP:
        /*TODO: Count*/
        /*Return Error to Drop.. but for now returning success to avoid drop*/
        return(ADK_SUCCESS);

    case ADK_IP_POLICE_ACTION_SET_PREC:
        /*TODO use action value*/
        return(ADK_SUCCESS);

    case ADK_IP_POLICE_ACTION_SET_QOS:
        /*TODO use action value*/
        return(ADK_SUCCESS);

    default:
        ADK_IP_DP_LOG_INFO("Invalid Police Action: Dropping packet");
        return(ADK_DP_E_ERROR);
    }
    ADK_IP_DP_LOG_INFO_DEBUG("policing action value - %d\n", value);
}

adk_dp_status_t
adk_ip_dp_qos_trtcm_params_gen(
    uint32_t        cir, /* bits/sec */
    uint32_t        pir, /* bits/sec */
    uint32_t        cbs, /* bytes = 2 * max. PDU size (recommended) */
    uint32_t        pbs, /* bytes */
    uint32_t        clockSpeed,
    adk_ip_dp_qos_policing_param_t *policingParams_p) 
{
    uint32_t    cirTicks = 0, pirTicks = 0, cbsTicks = 0, pbsTicks = 0;

    /* Validate pointer */
    if (!policingParams_p)
    {
        ADK_IP_DP_LOG_ERROR("Invalid pointer: policingParams_p.");
        return ADK_DP_E_ERROR;
    }

    /* Initialize the params to zero*/
    ADK_MEMSET(policingParams_p, 0, sizeof(adk_ip_dp_qos_policing_param_t));

    /* Populate the parameters */
    cirTicks = ((cir == 0) ? 0 : (((clockSpeed * 1.0)/(cir/8)) + 0.9999));
    pirTicks = ((pir == 0) ? 0 : (((clockSpeed * 1.0)/(pir/8)) + 0.9999));

    cbsTicks = cbs * cirTicks;
    pbsTicks = pbs * pirTicks;

    policingParams_p->pbs_ticks = pbsTicks;
    policingParams_p->cbs_ticks = cbsTicks;
    policingParams_p->pir_ticks = pirTicks;
    policingParams_p->cir_ticks = cirTicks;
    /* Initial Excess token buckets to PBS */
    policingParams_p->pir_bucket_token = pbsTicks;
    /* Initial Committed token buckets to CBS */
    policingParams_p->cir_bucket_token = cbsTicks;

    return ADK_SUCCESS;
} 

adk_dp_status_t adk_ip_dp_qos_trtcm_color_blind(
                adk_ip_dp_qos_policing_param_t *dpPoliceParams_p,
                uint32_t packetSize, uint8_t * policeColor)
{
    uint32_t deltaTime, currentTime;
    /* The value of the incoming packet converted to the # of ticks for cir. */
    uint32_t cirBucketpacketSizeInTick;
    /* The value of the incoming packet converted to the # of ticks for pir. */
    uint32_t pirBucketpacketSizeInTick;

    if(!dpPoliceParams_p)
    {
        ADK_IP_DP_LOG_ERROR("Invalid Policing Param pointer. Dropping packet!");
	return ADK_DP_E_ERROR;
    }
    
    /*Get current time*/
    currentTime = adk_dp_get_cycle_count(FALSE);

    /* Store the time elapse / delta time from the previous incoming
     * packet to the current packet */
    if (currentTime > dpPoliceParams_p->prev_time)
    {
        deltaTime = currentTime - dpPoliceParams_p->prev_time;
    }
    else
    {
        deltaTime = 0x3B9ACA00 - dpPoliceParams_p->prev_time + currentTime;
    }
    deltaTime = deltaTime * 1;

    dpPoliceParams_p->prev_time = currentTime;

    /*
     * Convert the incoming packet size in unit of tick.
     * Multiply packet size with pir and cir.
     */
    cirBucketpacketSizeInTick   = packetSize * dpPoliceParams_p->cir_ticks;
    pirBucketpacketSizeInTick   = packetSize * dpPoliceParams_p->pir_ticks;

    /*
     * Calculate the number of the token inside cir bucket
     * The value ranges from 0 to "committedBurstSize"
     * When the value is greater than committedBurstSize than set the value
     * to committedBurstSize
     */
    dpPoliceParams_p->cir_bucket_token += deltaTime;
    if (dpPoliceParams_p->cir_bucket_token > dpPoliceParams_p->cbs_ticks)
    {
        dpPoliceParams_p->cir_bucket_token = dpPoliceParams_p->cbs_ticks;
    }

    /*
     * Calculate the number of the token inside pir bucket
     * The value ranges from 0 to "peakBurstSize"
     * When the value is greater than peakBurstSize than set the value
     * to peakBurstSize
     */
    dpPoliceParams_p->pir_bucket_token += deltaTime;
    if (dpPoliceParams_p->pir_bucket_token > dpPoliceParams_p->pbs_ticks)
    {
        dpPoliceParams_p->pir_bucket_token = dpPoliceParams_p->pbs_ticks;
    }

    /*
     * Compare the incoming packet size with the number of pir bucket tokens
     * Which is, test if there is enough token for the incoming packet
     */

    /*
     * Then compare the incoming packet size with the number of cir bucket
     * tokens which is, test if there is enough token for the incoming
     * packet
     * Note: If the packet fails pir bucket it will be fail cir bucket
     * as well.
     */
    if (dpPoliceParams_p->pir_bucket_token < pirBucketpacketSizeInTick)
    {
        *policeColor =  ADK_QOS_DP_RED;
    }
    else if ( dpPoliceParams_p->cir_bucket_token <  cirBucketpacketSizeInTick)
    {
        *policeColor = ADK_QOS_DP_YELLOW;
        dpPoliceParams_p->pir_bucket_token -= pirBucketpacketSizeInTick;
    }
    else
    {
        *policeColor = ADK_QOS_DP_GREEN;
        dpPoliceParams_p->cir_bucket_token -= cirBucketpacketSizeInTick;
        dpPoliceParams_p->pir_bucket_token -= pirBucketpacketSizeInTick;
    }

#ifdef ADK_DEBUG
    if (*policeColor == ADK_QOS_DP_RED)
    {
        /* Count RED packets. */
        dpPoliceParams_p->red_pkts++;
    } else if (*policeColor == ADK_QOS_DP_YELLOW)
    {
        /* Count Yellow packets. */
        dpPoliceParams_p->yellow_pkts++;
    } else
    {
        /* Count Green packets. */
        dpPoliceParams_p->green_pkts++;
    }
#endif
    
    return ADK_SUCCESS;
}
#endif

