/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include <math.h>
#include "adk_sys_lcore.h"
#include <adk_ip_common.h>
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif

#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#include <adk_hos_dp.h>
#include <adk_qos_dp_export.h>
#include <adk_cksum.h>
#include <adk_timer.h>
#include <adk_pmu.h>
#include <adk_ip_dp_internal.h>
#include <adk_ip_dp_internal_fc.h>
#include <adk_em.h>
#include "adk_ph_dev_dp.h"
#include "adk_sif_dp.h"

#define INGRESS_IF_ID(ph_task) ADK_METADATA_PTR(ph_task)->input_intf_id
#define IP_PKT_LEN(ipHdr)  rte_be_to_cpu_16(ipHdr->total_length)
#define ETHER_TYPE_IPV4 0x0800
#define ETHER_TYPE_OFFSET(mbuf) (mbuf->l2_len - 2)



static inline void ip_inc_l4_in_stats(int coreIndex, uint8_t proto, uint16_t len)
{
    if(ADK_IP_PROTOCOL_TCP == proto)
    {
        ++ip_dp_tbl_info_p->ipL4Stats[coreIndex].ipTcpInPackets;
        ip_dp_tbl_info_p->ipL4Stats[coreIndex].ipTcpInOctets += len;
    }
    else if(ADK_IP_PROTOCOL_SCTP == proto)
    {
        ++ip_dp_tbl_info_p->ipL4Stats[coreIndex].ipSctpInPackets;
        ip_dp_tbl_info_p->ipL4Stats[coreIndex].ipSctpInOctets += len;
    }
    else if(ADK_IP_PROTOCOL_UDP == proto)
    {
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInUDPPkts, 1);
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInUDPOctets, len);
    }
}

adk_status_t classify_and_terminate(
    adk_pktio_task_t *ph_task,
    int coreIndex
);

static adk_status_t lli_pass1 (void **task_p,
                                 int coreIndex,
                    adk_pal_dp_stage_info_t **stage_info_p);

static adk_status_t lli_pass2(void **task_p,
                   int coreIndex,
                                       adk_pal_dp_stage_info_t **stage_info_p);

static adk_status_t lli_single_pass(void **task_p,
                   int coreIndex,
                    adk_pal_dp_stage_info_t **stage_info_p);

adk_status_t adk_ip_dp_pkt_proc_lli_single (void **task_p,
                                 int coreIndex,
                                 adk_pal_dp_stage_info_t **stage_info_p);

static inline uint8_t getIpIfId(
    adk_pktio_task_t *ph_task
)
{

    if(likely(INGRESS_IF_ID(ph_task) >= ADK_ETH_NUM_BRIDGE))
    { return INGRESS_IF_ID(ph_task) - ADK_ETH_NUM_BRIDGE + ADK_ETH_BR_MAX;}
    else
        {return INGRESS_IF_ID(ph_task);}
}

static inline adk_status_t set_params_for_iplli_to_ipuli(
    adk_pktio_task_t *ph_task,
    adk_pal_dp_stage_info_t **stage_info_p
    )
{
    adk_ip_dp_uli_imc_data_t *ipuliImc = (adk_ip_dp_uli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task));      /* Set IP ULI params and send to IP ULI */
    ADK_METADATA_PTR(ph_task)->innerIPVersion = 0;
    ADK_METADATA_PTR(ph_task)->outerIPVersion = 0;
    memset(ipuliImc, 0, sizeof(adk_ip_dp_uli_imc_data_t));
    ipuliImc->sender = IP_ULI_SENDER_IPLLI;
    ipuliImc->pktOrigin = IP_ULI_SENDER_IPLLI;
    ph_task->l3_len = IP_HDR_LEN;
    ADK_MD_GET_NEXT_FRAG(ph_task) = NULL;
    ADK_IP_DP_LOG_INFO_DEBUG("imcForIpULI: sender=0x%x, pktOrigin=0x%x\n", ipuliImc->sender, ipuliImc->pktOrigin);
    return adk_pal_dp_stage_info_get(adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT], stage_info_p);
}


static inline adk_status_t set_params_for_iplli_to_ipseclli(
        adk_pktio_task_t *ph_task,
        int coreIndex,
        adk_pal_dp_stage_info_t **stage_info_p,
        bool isLocal
)
{
    adk_ipsec_dp_lli_imc_param_t *ipsecLliImc = (adk_ipsec_dp_lli_imc_param_t *)(&(ADK_MD_GET_IMC_PARAM_START(ph_task)));
    memset(ipsecLliImc, 0, sizeof(adk_ipsec_dp_lli_imc_param_t));
    ipsecLliImc->isLocal = isLocal;
    ipsecLliImc->pktOrigin = IP_ULI_SENDER_IPLLI;
    ADK_MD_GET_NEXT_FRAG(ph_task) = NULL;
    INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInSentToIPSecPkts, 1);
    ADK_IP_DP_LOG_INFO_DEBUG("imcForIpsecLLI: pktOrigin=%d, isLocal=%d\n", ipsecLliImc->pktOrigin, ipsecLliImc->isLocal);
    return adk_pal_dp_stage_info_get(adkModuleMgrCpuServiceFn_g[ADK_IPSEC_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT], stage_info_p);
}

/* Reduce number of arguments */
static inline void get_l4_params(
    struct ipv4_hdr *ipHdr,
    uint16_t *src_port_nw,
    uint16_t *dest_port_nw,
    uint16_t *l4_len
)
{
    *l4_len = rte_be_to_cpu_16(ipHdr->total_length) - IP_HDR_LEN;

    struct udp_hdr * udphdr = (struct udp_hdr *) ((uint8_t *)ipHdr + IP_HDR_LEN);
    struct icmp_hdr * icmphdr = NULL;

    if((ipHdr->next_proto_id == IP_FILTER_PROTOCOL_UDP) ||
            (ipHdr->next_proto_id == IP_FILTER_PROTOCOL_UDP_LITE))
    {
        *src_port_nw = udphdr->src_port;
        *dest_port_nw = udphdr->dst_port;
        *l4_len = rte_be_to_cpu_16(udphdr->dgram_len);
    }
    else if((ipHdr->next_proto_id == IP_FILTER_PROTOCOL_TCP) ||
            (ipHdr->next_proto_id == IP_FILTER_PROTOCOL_SCTP))
    {
        *src_port_nw = udphdr->src_port;
        *dest_port_nw = udphdr->dst_port;
    }
    else if(ipHdr->next_proto_id == IP_FILTER_PROTOCOL_ICMP)
    {
        icmphdr = (struct icmp_hdr *) ((uint8_t *)ipHdr + IP_HDR_LEN);
        *src_port_nw = rte_cpu_to_be_16(icmphdr->icmp_type); /*type*/
        *dest_port_nw = rte_cpu_to_be_16(icmphdr->icmp_code); /*code*/
    }
    else if(ipHdr->next_proto_id == IP_FILTER_PROTOCOL_IGMP)
    {
        icmphdr = (struct icmp_hdr *) ((uint8_t *)ipHdr + IP_HDR_LEN);
        *src_port_nw = rte_cpu_to_be_16(icmphdr->icmp_type); /*type*/
        *dest_port_nw = 0;
    }
    else
    {
        *src_port_nw = 0;
        *dest_port_nw = 0;
    }
}
 

/*!
 * =============================================================
 * adk_ip_dp_pkt_proc_lli()
 *
 * Brief:
 * This function is an interface to ethernet module and performs
 * following:
 *
 * a. IP header checksum validation.
 * b. Route Lookup.
 * c. ACL lookup.
 * d. Interface-level policing.
 * e. Ingress classifier/Advance Lookup.
 * f. Reassembly of fragmented packets.
 *
 * =============================================================
 */
adk_status_t adk_ip_dp_pkt_proc_lli (void **task_p,
          uint8_t __attribute__((unused)) task_count,
          uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_pal_dp_stage_info_t *stage_info_p = NULL;


    adk_pktio_task_t *main_task, *main_next, *main_prev = NULL,
                     *head_task = NULL;
    main_task = (adk_pktio_task_t *)(task_p[0]);

    while(main_task)
    {
        main_next = (adk_pktio_task_t *) ADK_MD_GET_NEXT_FRAG(main_task);

        ret = adk_ip_dp_pkt_proc_lli_single((void **)(&main_task), coreIndex, &stage_info_p);

        if(ADK_E_TASK_CONSUMED == ret)
        {
            /* This return code is returned when all the chained tasks are consumed.
             * Hence no more work to be done
             */
            break;
        }
        if(ADK_E_TASK_PARKED == ret)
        {
            //ADK_METADATA_PARAM(main_task, task_type) = ADK_TASK_TYPE_NEW;
            ret = adk_pal_dp_release_send(main_task, ADK_PAL_DP_BATCH_CB_FN_IP, NULL);
        }
        else if(unlikely((ADK_E_INTERNAL_ERROR == ret)
                ||(ADK_E_DROP_TASK == ret)))
        {
            /* processed pkt to be dropped or consumed. Remove from the main
             * chain */
            if(main_prev)
            {
                ADK_MD_GET_NEXT_FRAG(main_prev) = main_next;
            }

            ret = adk_pal_dp_release_send(main_task, ADK_PAL_DP_BATCH_CB_FN_IP, NULL);
            /*Free the task*/
            adk_pktio_task_t *cur_task, *tmp_task;
            cur_task = (adk_pktio_task_t *)(main_task);
            while(cur_task != main_next)
            {
                tmp_task = (adk_pktio_task_t *)ADK_MD_GET_NEXT_FRAG(cur_task);
                /*Free the task*/
                adk_pktio_task_buffer_free(cur_task);
                cur_task = tmp_task;
            }
        }
        else
        {
            if(!head_task)
                head_task = main_task;

            main_prev = main_task;
        }
        main_task = main_next;
    }

    if(head_task)
    {
        task_p = (void **)(&head_task);
    if (stage_info_p && adkDestEntryFunction_g[stage_info_p->fn_id])
        {
            ret = adk_pal_dp_stage_send(task_p, task_count, task_mask, stage_info_p->fn_id, 0x00, 0x00, ADK_PAL_DP_BATCH_CB_FN_IP, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
            if(unlikely(ADK_E_INTERNAL_ERROR == ret))
            {
                adk_pal_dp_release_send(task_p[0], ADK_PAL_DP_BATCH_CB_FN_IP, NULL);
                /* Free the task */
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(task_p[0]));
                ADK_IP_DP_LOG_INFO("Failed to enQ to next stage in IP LLI");
                ret = ADK_E_INTERNAL_ERROR;
            }
        }
    }

    return ADK_SUCCESS;
} /* adk_ip_dp_pkt_proc_lli */

/*
 * 1. non-unicast dmac - unicast ip
 * 2. ip options
 * 3. blocked sip
 */
static inline adk_status_t check_for_blocked(
    adk_pktio_task_t *ph_task,
    int coreIndex,
    struct ipv4_hdr * ipHdr,
    adk_ip_dp_interface_entry_t *ingIfEntry,
    adk_ip_dp_intf_stats_t *if_stats
)
{
    adk_status_t st = ADK_SUCCESS;
    uint8_t      *ethHdr = NULL;
    adk_handle_t lkup_result={0};
    ADK_PDU_PTR_GET(ph_task, ethHdr, uint8_t *);
    /*M/BCast counters
     * TO DO: Move out of header_validation function
     * */

    if(likely(!(IS_MBCAST(ipHdr))))
    {
        /*Unicast DIP*/
        if(ethHdr && unlikely((ethHdr[0] & 0x1) && ingIfEntry &&
                (ingIfEntry->ifType == ADK_IP_DP_INTF_TYPE_ETH)))
        {
            /* M/Bcast DMAC & Ucast DIP. Invalid combination.
             * However, if the packet is a subnet directed broadcast,
             * then this check is not needed as subnet broadcasts can be
             * received as L2 Unicast as well as L2 broadcast */
            if(!((ADK_SUCCESS == adk_ip_dp_dip_get(ADK_IP_DEFAULT_VRD_ID, ipHdr->dst_addr, &lkup_result) )
                    && lkup_result
                    && ((((adk_ip_dp_route_info_t *)lkup_result)->isTerminating))
            ))
            {
                INC_NON_NULL(if_stats, ifInMBcastDMacUcastDipErrDrops, 1);
                st = ADK_E_DROP_TASK;
            }
        }
    }

    if(unlikely(IP_ADDR_ZERO == ipHdr->dst_addr))
    {
        INC_NON_NULL(if_stats, ifInZeroDipDrops, 1);
        st = ADK_E_DROP_TASK;
        goto check_for_blocked_exit;
    }

    if(unlikely((ipHdr->version_ihl & 0x0f) > 5 )) /* (20 bytes = 5 words) */
    {
        INC_NON_NULL(if_stats, ifInIpOptionsPresentDrops, 1);
        st = adk_ip_dp_send_exception(ph_task, ADK_IP_OPTIONS_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
        goto check_for_blocked_exit;
    }


check_for_blocked_exit:
    return st;
}

static inline void strip_eth_padding(
        adk_pktio_task_t *ph_task,
        uint16_t    ip_len
)
{
    if((ADK_MD_GET_IP_OFF(ph_task) + ip_len) < ADK_TASK_PDU_SIZE(ph_task))
    {
        uint16_t     eth_pad_bytes;
        eth_pad_bytes = ADK_TASK_PDU_SIZE(ph_task) -
                (ADK_MD_GET_IP_OFF(ph_task) + ip_len);
        ADK_PDU_TAIL_PULL(ph_task, eth_pad_bytes);
    }
}

/*!
 * =============================================================
 * adk_ip_dp_extract_ip_l4_hdr()
 *
 * IP DP header extraction function
 * =============================================================
 */
adk_ip_dp_pkt_error_t
adk_ip_dp_extract_ip_l4_hdr(struct ipv4_hdr *ipHdr,
        uint16_t *src_port_nw, uint16_t *dest_port_nw,
        uint16_t pkt_size, uint16_t *ip_length
        ,uint16_t *udp_length
)
{
    uint16_t tot_len = IP_PKT_LEN(ipHdr);

    *ip_length = tot_len;

    if(unlikely(tot_len < sizeof(struct ipv4_hdr)))
    {
        *ip_length = pkt_size;
        ADK_IP_DP_LOG_INFO("Total Length less than IHL");
        return ADK_IP_DP_PKT_BAD_LENGTH;
    }

    if(unlikely(tot_len > pkt_size))
    {
        *ip_length = pkt_size;
        ADK_IP_DP_LOG_INFO_DEBUG("Total Length(%d) more than pkt size(%d)", tot_len, pkt_size);
        return ADK_IP_DP_PKT_TRUNCATED;
    }

    get_l4_params(ipHdr, src_port_nw, dest_port_nw, udp_length);

#if 0
    ADK_IP_DP_LOG_INFO_DEBUG("dest_addr=%d.%d.%d.%d, src_addr=%d.%d.%d.%d, ",
            ipHdr->dst_addr&0x000000ff, (ipHdr->dst_addr>>8)&0x000000ff,
            (ipHdr->dst_addr>>16)&0x000000ff, (ipHdr->dst_addr>>24)&0x000000ff,
            ipHdr->src_addr&0x000000ff, (ipHdr->src_addr>>8)&0x000000ff,
            (ipHdr->src_addr>>16)&0x000000ff, (ipHdr->src_addr>>24)&0x000000ff);
    ADK_IP_DP_LOG_INFO_DEBUG("ttl=%d, prot=%d, dest_port_nw=0x%04x, \
            src_port_nw=0x%04x\n", ipHdr->time_to_live,
            ipHdr->next_proto_id, *dest_port_nw, *src_port_nw);
#endif

    return ADK_IP_DP_PKT_OK;
}


static inline adk_status_t l3_header_validation(
        adk_pktio_task_t *ph_task,
        int coreIndex,
        struct ipv4_hdr * ipHdr,
        uint16_t    * ip_len,
        adk_ip_dp_interface_entry_t *ingIfEntry,
        adk_ip_dp_intf_stats_t *if_stats
)
{
    adk_status_t st = ADK_SUCCESS;
    adk_ip_dp_pkt_error_t ip_parse_result;
    uint16_t src_port_nw=0, dest_port_nw=0, udp_length=0;

    /*
     * Verify IPv4 header checksum
     */
#ifdef ADK_RX_CKSUM_OL
    switch(ph_task->ol_flags & PKT_RX_IP_CKSUM_MASK)
    {
    case PKT_RX_IP_CKSUM_GOOD:
        break;

    case PKT_RX_IP_CKSUM_UNKNOWN:
        /*OL data not available, validate cksum*/
        if(unlikely(ip_hdr_csum((void *)ipHdr, (ipHdr->version_ihl & 0x0f))))
        {
            INC_NON_NULL(if_stats, ifInIpCksumErrDrops, 1);
            ADK_IP_DP_LOG_INFO("Incorrect IP header cksum. Dropping packet!");
            st = ADK_E_DROP_TASK;
            goto header_val_exit;
        }
        break;

    default: /* PKT_RX_IP_CKSUM_BAD , PKT_RX_IP_CKSUM_NONE*/
        /* IP cksum is not correct */
        INC_NON_NULL(if_stats, ifInIpCksumErrDrops, 1);
        ADK_IP_DP_LOG_INFO("Incorrect IP header checksum. Dropping packet!");
        st = ADK_E_DROP_TASK;
        goto header_val_exit;
    }
#else
    if(unlikely(ip_hdr_csum((void *)ipHdr, (ipHdr->version_ihl & 0x0f))))
    {
        INC_NON_NULL(if_stats, ifInIpCksumErrDrops, 1);
        ADK_IP_DP_LOG_INFO("Incorrect IP header checksum. Dropping packet!");
        st = ADK_E_DROP_TASK;
        goto header_val_exit;
    }
#endif

    /* parse ipv4 & L4 header, check for errors and extract required fields */
    ip_parse_result = adk_ip_dp_extract_ip_l4_hdr(
            ipHdr, &src_port_nw, &dest_port_nw,
            (ADK_TASK_PDU_SIZE(ph_task) - ADK_MD_GET_IP_OFF(ph_task)), ip_len, &udp_length);

    if(unlikely(ip_parse_result == ADK_IP_DP_PKT_BAD_LENGTH))
    {
        INC_NON_NULL(if_stats, ifInBadIpLengthErrDrops, 1);
        st = ADK_E_DROP_TASK;
        goto header_val_exit;
    }
    if(unlikely(ip_parse_result == ADK_IP_DP_PKT_TRUNCATED))
    {
        INC_NON_NULL(if_stats, ifInBadIpLengthErrDrops, 1);
        st = ADK_E_DROP_TASK;
        goto header_val_exit;
    }

    st = check_for_blocked(ph_task, coreIndex, ipHdr, ingIfEntry, if_stats);
    if(ADK_SUCCESS != st) /* just in case we add more code after this */
        goto header_val_exit;

    header_val_exit:
    return st;
}


static inline adk_status_t ttl_check_and_decrement(
        adk_pktio_task_t *ph_task,
        int coreIndex,
        struct ipv4_hdr *ipHdr,
        adk_ip_dp_intf_stats_t *if_stats
)
{
    uint32_t    tmp_32;
    /* Check for TTL */
    if((0 == ipHdr->time_to_live) || (1 == ipHdr->time_to_live))
    {
        ADK_IP_DP_LOG_INFO("IP TTL check failed!");
        INC_NON_NULL(if_stats, ifOutTtlExceededErrDrops, 1);
        return adk_ip_dp_send_exception(ph_task, ADK_IP_TTL_EXPIRED_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
    }

    /* TTL Decrement */
    ipHdr->time_to_live--;
    tmp_32 = ~(~(ipHdr->hdr_checksum) - 0x1);
    ipHdr->hdr_checksum = (tmp_32 & 0xFFFF) + (tmp_32 >> 16);
    return ADK_SUCCESS;
}

static inline adk_status_t route_lookup(
    struct ipv4_hdr *ipHdr,
    adk_ip_dp_route_info_t **ipRouteInfo_p
)
{
    adk_handle_t lkup_result;
    adk_dp_status_t dp_st;

    /* TO DO: we need lip table here. We should store ip stats id in that table */
    dp_st = adk_ip_dp_dip_get(ADK_IP_DEFAULT_VRD_ID, ipHdr->dst_addr, &lkup_result);
    if(likely(ADK_SUCCESS == dp_st))
    {
        *ipRouteInfo_p = (adk_ip_dp_route_info_t *)lkup_result;
        ADK_IP_DP_LOG_INFO_DEBUG("IP LLI: dip lookup success. dip=0x%x, is-terminating=%d\n",
                adk_ntohl(ipHdr->dst_addr), (*ipRouteInfo_p)->isTerminating);
        return ADK_SUCCESS;
    }
    else if ((ADK_SUCCESS != dp_st))
    {
        *ipRouteInfo_p = NULL;
        ADK_IP_DP_LOG_INFO("Route lookup failed");
        return ADK_E_CFG_ENTRY_NOT_FOUND;
    }
    else
    {
        *ipRouteInfo_p = NULL;
        /* Drop the packet since DIP address validation failed*/
        ADK_IP_DP_LOG_INFO("DIP lookup returned NULL");
        return ADK_E_CFG_ENTRY_NOT_FOUND;
    }
} /* route_lookup */

/* return value:
 0 : yes,
 1 : yes, sse-address
 2 : no
 */
static inline uint8_t is_terminating(
    adk_pktio_task_t *ph_task
)
{
    if (likely(adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(ph_task))))
    {
        if(likely(ADK_HOS_DEFAULT_IP_STATS_ID != adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(ph_task))))
            return 0;
        else
            return 1;
    }
    else
        return 2;
}


adk_status_t classify_and_terminate(
    adk_pktio_task_t *ph_task,
    int coreIndex
)
{
    uint32_t flow_id = 0;
    adk_status_t ret = ADK_SUCCESS;
    bool run_sw_classif = false;
    uint8_t ipsec_data = ((adk_ip_dp_lli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task)))->ipsec_data;

    if (likely(IPLLI_IMC_IPSEC_INNER_LOOKASIDE != ipsec_data))
    { /* non decrypted or inline decrypted */
        if (likely(!adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(ph_task))))
            flow_id = adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(ph_task));
        else
            run_sw_classif = true;
    }
    else
    { /* packet decrypted by adk-ipsec using look-aside method */
        run_sw_classif = true;
    }

    if (unlikely(true == run_sw_classif))
    {
        uint16_t sp_nw, dp_nw, l4_len;
        adk_dp_status_t dp_st;
        struct ipv4_hdr *ipHdr;
        adk_ip_ing_classifier_ptn_t classifierRules;
        ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
        get_l4_params(ipHdr, &sp_nw, &dp_nw, &l4_len);

        /* run classifier in sw */
        uint32_t ipIfId = getIpIfId(ph_task);
        classifierRules.ipIntfHandle = adk_htonl(ipIfId);
        classifierRules.proto = ipHdr->next_proto_id;
        classifierRules.dip = ipHdr->dst_addr;
        classifierRules.dPort = dp_nw;
        classifierRules.sip = ipHdr->src_addr;
        classifierRules.sPort = sp_nw;
        dp_st = adk_ip_dp_ing_classifier_lookup(
                ip_dp_tbl_info_p->ingClassifierInfo_p->ipIngClassifierCtx[ip_dp_tbl_info_p->ingClassifierInfo_p->aclTreeIndex],
                &classifierRules, &flow_id);
        if (unlikely((dp_st != ADK_SUCCESS) || !flow_id))
        {
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInClassifierDefaultDrops++;
            return ADK_E_INTERNAL_ERROR;
        }
    }

    /* <<    83 cycles (minus 48 cycle count overhead=35 cycles)   >> */
    adk_ip_dp_ing_classifier_action_info_t *ingClassifierAction_p =
            &(ip_dp_tbl_info_p->ingClassifierActionDp_p[ip_dp_tbl_info_p->ingClassifierInfo_p->aclTreeIndex][flow_id]);
    /* <<    83 cycles (minus 48 cycle count overhead=35 cycles)   >> */

    ip_dp_tbl_info_p->ipIngFlowStats[coreIndex * ADK_DP_MAX_IP_ING_ASSOC_FLOWS + flow_id].rxPkts ++;

    /* TO DO: use ip lli specific scratchpad to avoid this */
    struct ipv4_hdr *ipHdr;
    ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
    uint16_t l4_len = rte_be_to_cpu_16(ipHdr->total_length) - IP_HDR_LEN;

    if (likely(ADK_IP_ING_CLASSIFIER_ACTION_SEND_TO_EO == ingClassifierAction_p->action))
    {
        uintptr_t cbParam = flow_id;

        ADK_IP_DP_LOG_INFO_DEBUG("Incrementing Rx L4 stats for EO path, proto=%d, len=%d\n", ipHdr->next_proto_id, l4_len);
        ip_inc_l4_in_stats(coreIndex, ipHdr->next_proto_id, l4_len);

        /*This is needed only for ORDERED SHED*/
        if(ADK_METADATA_PARAM(ph_task, event_sched_type) != RTE_SCHED_TYPE_ATOMIC)
        {
            adk_sif_em_dp_lli_meta_data_t *em_lli_md_p = (adk_sif_em_dp_lli_meta_data_t *)&(ADK_METADATA_PARAM(ph_task,
                    imc_param_start));
            em_lli_md_p->eventMachineQueueId = ingClassifierAction_p->eventMachineQueueId;

            ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_2();
            ret = adk_pal_dp_stage_send((void **)&ph_task, 1, 0x1, ADK_SIF_EM_LLI_FN_ID, em_lli_md_p->eventMachineQueueId,
                    0x00, ADK_PAL_DP_BATCH_CB_FN_IP, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER, (void *)cbParam);
        }
        else
        {
#ifdef ADK_HOS_DP_TEST
    /* Copy DSI VSI meta data from original location to post EM header */
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(ph_task);
    uint8_t* post_em_header_ptr = (uint8_t*)((uintptr_t) ph_task + sizeof(struct rte_mbuf) + EM_HDR_SIZE);
    memcpy(post_em_header_ptr, dsi_md_p, sizeof(adk_dsi_rx_metadata_t));
#endif

            ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_2();
            ret = adk_pal_dp_em_send(ph_task, ingClassifierAction_p->eventMachineQueueId,
                    ADK_PAL_DP_BATCH_CB_FN_IP, (void *)cbParam);
        }
        if(ADK_SUCCESS != ret && ADK_E_TASK_CONSUMED != ret)
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInSendToEoFailDrops++;

        return ret;
    }
    else if (likely(ADK_IP_ING_CLASSIFIER_ACTION_SEND_TO_HOST == ingClassifierAction_p->action))
    {
        adk_slowpath_dev_dp_metadata_t *spDevMd_p;
        spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(ph_task, imc_param_start));
        spDevMd_p->policerFlowId = ingClassifierAction_p->policerFlowId;
        spDevMd_p->flowId=flow_id;
        spDevMd_p->moduleId=ADK_IP_MODULE_ID;
        spDevMd_p->l4_protocol = ipHdr->next_proto_id;
        spDevMd_p->l4_len = l4_len;

        if ((IPLLI_IMC_IPSEC_INNER_LOOKASIDE == ipsec_data) || (IPLLI_IMC_IPSEC_INNER_INLINE == ipsec_data))
        { /* Copy ethernet header in case of decryption */
            uint8_t *ethHdr;
            ADK_PDU_PTR_GET(ph_task, ethHdr, uint8_t *);
            *((uint16_t *)(ethHdr + ETHER_TYPE_OFFSET(ph_task))) = htons(ETHER_TYPE_IPV4);
            rte_memcpy(((uint8_t *)ipHdr) - ph_task->l2_len, ethHdr, ph_task->l2_len);
            rte_pktmbuf_adj(ph_task, ADK_MD_GET_IP_OFF(ph_task) - ph_task->l2_len);
        }

        ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_2();
        if (likely(adk_pal_dp_netd_send((void *)ph_task,ADK_PAL_DP_BATCH_CB_FN_IP, NULL) == ADK_E_TASK_CONSUMED))
        {
            return ADK_E_TASK_CONSUMED;
        }
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInSendToHostFailDrops++;

        return ADK_E_INTERNAL_ERROR;
    }
    else if (ADK_IP_ING_CLASSIFIER_ACTION_DROP == ingClassifierAction_p->action)
    {
        if(1 == flow_id)
        { /* We have hit the default rule. So, packet didn't match any user configured rule */
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInClassifierDefaultDrops++;
        }
        else
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInClassifierDrops++;

        return ADK_E_DROP_TASK;
    }

    ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInClassifierUnknownActionDrops++;

    return ADK_E_DROP_TASK;
}

    /*
static inline get_ip_addr_stats_id(
    adk_pktio_task_t *ph_task
)
{
    if outer 
        if decrypted
            if outer == inner
                return addr_stats[stats_id from hos]
            else 
                return addr_stats[route_lookup result]
        else
            return addr_stats[stats_id from hos]
    else if inner
        if outer == inner
            return NULL
        else
            return addr_stats[stats_id from hos]
    else // look-aside
           return addr_stats[route_lookup result]
}
*/

static inline void qos_stats(
    adk_ip_dp_interface_entry_t *ingIfEntry,
    adk_ip_dp_intf_stats_t *if_stats,
    struct ipv4_hdr *ipHdr
)
{
    if(likely(ingIfEntry && ingIfEntry->enableQoS && if_stats))
    {
        uint8_t in_dscp;
        adk_qos_ds_phb_class_t phb_class;
        in_dscp = (ipHdr->type_of_service >> 2) & 0x3F;
        phb_class = ingIfEntry->dscpToPhbClass[in_dscp];
        if (ADK_QOS_DS_PHB_CLASS_NONE != phb_class)
        {
            /*Increment the RX QoS Statistics after validation of IP packet*/
            ++(if_stats->ifRxQoSstats[phb_class].rxPkts);
            if_stats->ifRxQoSstats[phb_class].rxBytes += IP_PKT_LEN(ipHdr);
        }
    }
}

static inline void lli_single_entry_logs(
        adk_pktio_task_t *ph_task,
        adk_ip_dp_lli_imc_data_t *iplliImc

);
static inline void lli_single_exit_logs(
        adk_pktio_task_t *ph_task,
        adk_ip_dp_lli_imc_data_t *iplliImc
);

static inline void inc_entry_if_stats(
        adk_pktio_task_t *ph_task,
        adk_ip_dp_intf_stats_t *if_stats,
        struct ipv4_hdr *ipHdr
)
{
    ++if_stats->ifInPkts;
    if_stats->ifInOctets += (ADK_TASK_PDU_SIZE(ph_task) - ADK_MD_GET_IP_OFF(ph_task));
    if(unlikely(IS_MCAST(ipHdr)))
    {
        ++if_stats->ifInMcastPkts;
        if_stats->ifInMcastOctets += (ADK_TASK_PDU_SIZE(ph_task) - ADK_MD_GET_IP_OFF(ph_task));
    }

    if(unlikely(IS_MCAST(ipHdr)))
    {
        ++if_stats->UifInBcastPkts;
        if_stats->UifInBcastOctets += (ADK_TASK_PDU_SIZE(ph_task) - ADK_MD_GET_IP_OFF(ph_task));
    }
} /* inc_entry_if_stats */


static inline adk_status_t proc_non_terminating(
        adk_pktio_task_t *ph_task,
        int coreIndex,
        adk_ip_dp_intf_stats_t *if_stats,
        struct ipv4_hdr *ipHdr,
        adk_pal_dp_stage_info_t **stage_info_p,
        bool toIpsec
)
{
    adk_status_t st;
    if (ADK_METADATA_PTR(ph_task)->in_routing_disabled)
    { /*Drop the packet*/
        ++if_stats->ifRoutingDisabledDrops;

        return adk_ip_dp_send_exception(ph_task, ADK_IP_FORWARDING_DISABLED_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
    }

    st = ttl_check_and_decrement(ph_task, coreIndex, ipHdr, if_stats);
    if (likely(ADK_SUCCESS == st))
    {
        if(toIpsec)
            return set_params_for_iplli_to_ipseclli(ph_task, coreIndex, stage_info_p, FALSE);
        else
            return set_params_for_iplli_to_ipuli(ph_task, stage_info_p);
    }
    else
        return st;

} /* proc_non_terminating */

static adk_status_t lli_pass1 (void **task_p,
                                 int coreIndex,
                    adk_pal_dp_stage_info_t **stage_info_p)
{
    struct ipv4_hdr *ipHdr;
    adk_ip_dp_interface_entry_t *ingIfEntry;
    adk_ip_dp_intf_stats_t *if_stats;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    adk_status_t st;

    ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(ph_task) + IP_HDR_LEN);
    ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
 
    /* Setting outerIpStatsId to invalid when packet first comes from ethernet.
     * For terminating returning packets, we will check if the stats_id derived for inner
     * packet is same as this and only increment address stats if it isn't.
     */
    ADK_METADATA_PARAM(ph_task, outerIpStatsId) = ADK_SYS_INVALID_MDATA_ID;
    ADK_METADATA_PARAM(ph_task, outerIPVersion) = ADK_IPV4_VER;
    uint8_t ipIfId = getIpIfId(ph_task);
    if(ADK_DP_MAX_IP_INTERFACE == ipIfId)
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInInvalidIfDrops++;
        return ADK_E_DROP_TASK;
    }

    /* Get ip if handle from ipIfTble & retrieve the ip attributes */
    ingIfEntry = &(ip_dp_tbl_info_p->ipInterfaceTble[ipIfId]);
    if (unlikely( (ingIfEntry->ifType != ADK_IP_DP_INTF_TYPE_ETH) ))
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInInvalidIfDrops++;
        return ADK_E_DROP_TASK;
    }

    ADK_METADATA_PTR(ph_task)->in_routing_disabled = ingIfEntry->routingDisabled;
    if_stats = &(ip_dp_tbl_info_p->ipInterfaceStatsTble[coreIndex][ingIfEntry->if_stats_id]);
    inc_entry_if_stats(ph_task, if_stats, ipHdr);

    /* Verify if this is an IPV4 packet */
    if(likely(4 == (ipHdr->version_ihl) >> 4)) /* (iphdr_version_ihl & 0xf0) >> 4 */
    {
        if (likely(adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(ph_task))))
        {
            /* Since the packet was decrypted, we assume: 1. good ip hdr; 2. outer terminating; 3. non-fragmented; 4. not mbcast */
            strip_eth_padding(*task_p, IP_PKT_LEN(ipHdr));

            /* TO DO: 
            if outer == inner
                return addr_stats[stats_id from hos]
            else 
                return addr_stats[route_lookup result]
            */
            adk_ip_dp_route_info_t *ipRouteInfo_p;
            adk_ip_dp_ip_addr_stats_t *ip_addr_stats;
            st = route_lookup(ipHdr, &ipRouteInfo_p);
            ip_addr_stats = &(ip_dp_tbl_info_p->ipAddrStatsTble[coreIndex][ipRouteInfo_p->u.termination.ip_addr_stats_id]);
            ++ip_addr_stats->ipInPkts;
            ip_addr_stats->ipInOctets += rte_be_to_cpu_16(ipHdr->total_length);
            ADK_METADATA_PARAM(ph_task, outerIpStatsId) = ipRouteInfo_p->u.termination.ip_addr_stats_id;
            qos_stats(ingIfEntry, if_stats, ipHdr);


            return set_params_for_iplli_to_ipseclli(*task_p, coreIndex, stage_info_p, TRUE);
        } /* decrypted */
        else
        { /* Process plaintext */
            if (likely(!adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(ph_task))))
            {
                if (likely(ADK_SUCCESS == check_for_blocked(ph_task, coreIndex, ipHdr, ingIfEntry, if_stats)))
                {
                    /* TO DO: ipsec doesn't work without this.. why? */
                    strip_eth_padding(*task_p, IP_PKT_LEN(ipHdr));
                    qos_stats(ingIfEntry, if_stats, ipHdr);
                    uint8_t term;
                    if (likely(2 != (term = is_terminating(*task_p))))
                    {
                        adk_ip_dp_ip_addr_stats_t *ip_addr_stats;
                        if (likely(0 == term))
                            ip_addr_stats = &(ip_dp_tbl_info_p->ipAddrStatsTble[coreIndex][adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(ph_task))]);
                        else
                            ip_addr_stats = NULL;

                        INC_NON_NULL(ip_addr_stats, ipInPkts, 1);
                        INC_NON_NULL(ip_addr_stats, ipInOctets, rte_be_to_cpu_16(ipHdr->total_length));

                        /* Not an SSE address & UDP & not-fragmented & udp-cksum-val flag true & HW cksum ok */
                        if (ip_addr_stats &&
                            ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id &&
                            !(ipHdr->fragment_offset & 0xFF3F)             &&
                            ADK_SYS_UDP_VALIDATE_FLAG_GET               &&
                            unlikely(adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(ph_task)))
                        )
                        {
                            ++if_stats->ifInUDPCksumErrDrops;
                            return ADK_E_DROP_TASK;
                        }    /*bad udp l4 len or cksum */
                        else /* checksum validation is disabled or l4 cksum is valid */
                        {
                            bool isReassembled = false;

                            /* not sse and fragmented */
                            if (ip_addr_stats && (ipHdr->fragment_offset & 0xFF3F))
                            { /* reassemble */
                                uint32_t hash_val;
                                INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmReqPkts, 1);
                                /* Fragmented pkt. Do reassembly */
                                adk_ip_dp_get_reassembly_hash((adk_pktio_task_t **)task_p, ADK_MD_GET_IP_OFF(ph_task), &hash_val);
                                st = adk_ip_dp_reassemble(coreIndex, &ph_task, ADK_IP_DEFAULT_VRD_ID, 0, hash_val);
                                if (ADK_SUCCESS != st)
                                    return st;

                                ++ip_addr_stats->ipInReasmPkts;
                                *task_p = (void *)ph_task;
                                ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
                                ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(ph_task) + IP_HDR_LEN);
                                isReassembled = true;
                            } /* end of Reassembly block*/

                            /* Check if its a supported l4 protocol. UDP is considered always allowed */
                            if (ADK_IP_PROTOCOL_UDP != ipHdr->next_proto_id && !ADK_IP_ALLOWED_PROTO(ipHdr->next_proto_id))
                            {
                                INC_NON_NULL(if_stats, ifInUnknownProtoErrDrops, 1);
                                INC_NON_NULL(ip_addr_stats, ipInErrors, 1);
                                return adk_ip_dp_send_exception(ph_task, ADK_IP_UNSUPPORTED_PROTO_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
                            }

                            if (isReassembled && ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id)
                            {
                                uint16_t src_port_nw = 0, dest_port_nw = 0, udp_length = 0;
                                get_l4_params(ipHdr, &src_port_nw, &dest_port_nw, &udp_length);
                                if (likely(ADK_TASK_PDU_SIZE(ph_task) >= (ADK_MD_GET_L4_OFF(ph_task) + udp_length)) &&
                                    (!ADK_SYS_UDP_VALIDATE_FLAG_GET || (ADK_SYS_UDP_VALIDATE_FLAG_GET &&
                                                                        likely(!ipv4_udp_checksum(ph_task, udp_length, ipHdr->src_addr, ipHdr->dst_addr)))))
                                {
                                    return set_params_for_iplli_to_ipseclli(*task_p, coreIndex, stage_info_p, TRUE);
                                } /* l4 len, cksum is good */
                                else
                                {
                                    ++if_stats->ifInUDPCksumErrDrops;
                                    return ADK_E_DROP_TASK;
                                } /* l4 len or cksum is bad */
                            }     /* is UDP */
                            else
                                return set_params_for_iplli_to_ipseclli(*task_p, coreIndex, stage_info_p, TRUE);

                        } /* good l4 len and cksum */
                    }     /* terminating */
                    else
                    {
                        return proc_non_terminating(ph_task, coreIndex, if_stats, ipHdr,
                                stage_info_p, true /* send to ipseclli */);
                    } /* non terminating */
                }     /* non blocked packets */
                else
                {
                    return ADK_E_DROP_TASK;
                } /* blocked packets */
            }     /* ip header good */
            else
            {
                /* Bad length and truncated packet errors are expected to be checked by HLP and adk_hos_is_ip_hdr_bad() would return TRUE in that case
               We know that header is corrupted. We want to increment specific error counters. Hence we are calling this api */
                uint16_t ip_len;
                return l3_header_validation(ph_task, coreIndex, ipHdr, &ip_len, ingIfEntry, if_stats);
            } /* ip header bad */
        }     /* process plaintext end */
    }         /* v4 packet */
    else
    {
        if(6 == (ipHdr->version_ihl) >> 4)
            {++if_stats->ifInVersionMismatchErrDrops;}
        else
            {++if_stats->ifInInvalidVersionErrDrops;}

        return adk_ip_dp_send_exception(ph_task, ADK_IP_INVALID_VERSION_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
    } /* non v4 packet */
   
}

/* Only inline decrypted packets will land here */
static adk_status_t lli_pass2(void **task_p,
                   int coreIndex,
                                       adk_pal_dp_stage_info_t **stage_info_p)
{
    struct ipv4_hdr *ipHdr;
    adk_status_t st;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    ADK_PDU_OFFSET_PTR_GET(*task_p, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(*task_p));

    if (likely(!adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(ph_task))))
    {
        if (likely(ADK_SUCCESS == check_for_blocked(ph_task, coreIndex, ipHdr, NULL, NULL)))
        {
            uint8_t term = 0;
            if (likely(2 != (term = is_terminating(*task_p))))
            {
                adk_ip_dp_ip_addr_stats_t *ip_addr_stats;
                if (0 == term)
                { /* unicast terminating ip. increment IN stats only if inner IP is different than outer */
                    ip_addr_stats = &(ip_dp_tbl_info_p->ipAddrStatsTble[coreIndex][adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(ph_task))]);
                    if ((ADK_IPV4_VER != ADK_METADATA_PARAM(ph_task, outerIPVersion)) ||
                            ADK_METADATA_PARAM(ph_task, outerIpStatsId) != adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(ph_task)))
                    {
                        ++ip_addr_stats->ipInPkts;
                        ip_addr_stats->ipInOctets += rte_be_to_cpu_16(ipHdr->total_length);
                    } /* outer and inner ip are different */
                }
                else
                    ip_addr_stats = NULL; /* mbcast packets */

                /* not sse & not udp & not fragmented & l4-cksum-validate flag set & cksum bad */
                if (unlikely(ip_addr_stats                                  &&
                            ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id     &&
                            !(ipHdr->fragment_offset & 0xFF3F)              &&
                            ADK_SYS_UDP_VALIDATE_FLAG_GET                   &&
                            adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(ph_task)))
                )
                {
                    /* TO DO: Increment interface error counter here */
                    return ADK_E_DROP_TASK;
                }    /* l4 cksum bad */
                else /* checksum validation is disabled or l4 cksum is valid */
                {
                    bool isReassembled = false;

                    /* not sse & fragmented */
                    if (ip_addr_stats && (ipHdr->fragment_offset & 0xFF3F))
                    { /* reassemble */
                        uint32_t hash_val;
                        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmReqPkts, 1);
                        /* Fragmented pkt. Do reassembly */
                        adk_ip_dp_get_reassembly_hash((adk_pktio_task_t **)task_p, ADK_MD_GET_IP_OFF(ph_task), &hash_val);
                        st = adk_ip_dp_reassemble(coreIndex, &ph_task, ADK_IP_DEFAULT_VRD_ID, ADK_DP_MAX_IP_STATS, hash_val);
                        if (ADK_SUCCESS != st)
                            return st;

                        ++ip_addr_stats->ipInReasmPkts;
                        *task_p = (void *)ph_task;
                        ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
                        ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(ph_task) + IP_HDR_LEN);
                        isReassembled = true;
                    } /* end of Reassembly block*/

                    /* Check if its a supported l4 protocol. UDP is considered always allowed */
                    if (ADK_IP_PROTOCOL_UDP != ipHdr->next_proto_id && !ADK_IP_ALLOWED_PROTO(ipHdr->next_proto_id))
                    {
                        INC_NON_NULL(ip_addr_stats, ipInErrors, 1);
                        return adk_ip_dp_send_exception(ph_task, ADK_IP_UNSUPPORTED_PROTO_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
                    }

                    if (isReassembled && ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id)
                    {
                        uint16_t src_port_nw = 0, dest_port_nw = 0, udp_length = 0;
                        get_l4_params(ipHdr, &src_port_nw, &dest_port_nw, &udp_length);

                        if (ADK_TASK_PDU_SIZE(ph_task) < (ADK_MD_GET_L4_OFF(ph_task) + udp_length) ||
                                (ADK_SYS_UDP_VALIDATE_FLAG_GET &&
                                        unlikely(ipv4_udp_checksum(ph_task, udp_length, ipHdr->src_addr, ipHdr->dst_addr))))
                        {
                            /* TO DO: Increment interface error here */
                            return ADK_E_DROP_TASK;
                        }
                    } /* post reassembly l4 len,cksum */

                    ph_task = (adk_pktio_task_t *)(*task_p);
                    ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(*task_p) + IP_HDR_LEN);
                    return classify_and_terminate(ph_task, coreIndex);
                } /* l4 cksum good */
            }     /* terminating */
            else
            {
                return proc_non_terminating(ph_task, coreIndex, NULL, ipHdr,
                        stage_info_p, false /* send to ipuli */);
            } /* non terminating */

        } /* non blocked packets */
        else
        {
            return ADK_E_DROP_TASK;
        } /* blocked packets */
    }     /* ip header good */
    else
    {
        /* Bad length and truncated packet errors are expected to be checked by HLP and adk_hos_is_ip_hdr_bad() would return TRUE in that case
               We know that header is corrupted. We want to increment specific error counters. Hence we are calling this api */
        uint16_t ip_len;
        return l3_header_validation(ph_task, coreIndex, ipHdr, &ip_len, NULL, NULL);
    } /* ip header bad */

    return ADK_E_DROP_TASK;
}

static adk_status_t lli_single_pass(void **task_p,
                   int coreIndex,
                    adk_pal_dp_stage_info_t **stage_info_p)
{
    adk_ip_dp_interface_entry_t *ingIfEntry = NULL;
    adk_ip_dp_intf_stats_t *if_stats = NULL;
    struct ipv4_hdr *ipHdr;
    adk_status_t st;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    ADK_PDU_OFFSET_PTR_GET(*task_p, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(*task_p));

    uint8_t ipIfId = getIpIfId(ph_task);
    if(ADK_DP_MAX_IP_INTERFACE == ipIfId)
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInInvalidIfDrops++;
        return ADK_E_DROP_TASK;
    }
    /* Get ip if handle from ipIfTble & retrieve the ip attributes */
    ingIfEntry = &(ip_dp_tbl_info_p->ipInterfaceTble[ipIfId]);
    if (unlikely((ingIfEntry->ifType != ADK_IP_DP_INTF_TYPE_ETH) ))
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInInvalidIfDrops++;
        return ADK_E_DROP_TASK;
    }

    if_stats = &(ip_dp_tbl_info_p->ipInterfaceStatsTble[coreIndex][ingIfEntry->if_stats_id]);
    inc_entry_if_stats(ph_task, if_stats, ipHdr);

    ADK_METADATA_PTR(ph_task)->in_routing_disabled = ip_dp_tbl_info_p->ipInterfaceTble[INGRESS_IF_ID(ph_task)].routingDisabled;

    if (likely(!adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(ph_task))))
    {
        /* Verify if this is an IPV4 packet */
        if (likely(4 == (ipHdr->version_ihl) >> 4))
        {
            if (likely(!adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(ph_task))))
            {
                if (likely(ADK_SUCCESS == check_for_blocked(ph_task, coreIndex, ipHdr, NULL, if_stats)))
                {
                    strip_eth_padding(ph_task, IP_PKT_LEN(ipHdr));
                    qos_stats(ingIfEntry, if_stats, ipHdr);

                    uint8_t term=0;
                    if (likely(2 != (term = is_terminating(*task_p))))
                    {
                        adk_ip_dp_ip_addr_stats_t *ip_addr_stats;
                        if(likely(0 == term))
                            ip_addr_stats = &(ip_dp_tbl_info_p->ipAddrStatsTble[coreIndex][adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(ph_task))]);
                        else
                            ip_addr_stats = NULL;

                        /* not sse & not udp & not fragmented & cksum-validate-flag set & cksum bad */
                        if (ip_addr_stats                                   &&
                            ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id     &&
                            !(ipHdr->fragment_offset & 0xFF3F)              &&
                            ADK_SYS_UDP_VALIDATE_FLAG_GET                   &&
                            unlikely(adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(ph_task))))
                        {
                            ++if_stats->ifInUDPCksumErrDrops;
                            return ADK_E_DROP_TASK;
                        }    /* l4 cksum bad */
                        else /* checksum validation is disabled or l4 cksum is valid */
                        {
                            bool isReassembled = false;
                            INC_NON_NULL(ip_addr_stats, ipInPkts, 1);
                            INC_NON_NULL(ip_addr_stats, ipInOctets, rte_be_to_cpu_16(ipHdr->total_length));

                            /* not-sse and fragmented */
                            if (ip_addr_stats && (ipHdr->fragment_offset & 0xFF3F))
                            { /* reassemble */
                                uint32_t hash_val;
                                INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmReqPkts, 1);

                                /* Fragmented pkt. Do reassembly */
                                adk_ip_dp_get_reassembly_hash((adk_pktio_task_t **)task_p, ADK_MD_GET_IP_OFF(ph_task), &hash_val);
                                st = adk_ip_dp_reassemble(coreIndex, &ph_task, ADK_IP_DEFAULT_VRD_ID, ADK_DP_MAX_IP_STATS, hash_val);
                                if (ADK_SUCCESS != st)
                                    return st;

                                ++ip_addr_stats->ipInReasmPkts;
                                *task_p = (void *)ph_task;
                                ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
                                ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(ph_task) + IP_HDR_LEN);
                                isReassembled = true;
                            } /* end of Reassembly block*/

                            /* Check if its a supported l4 protocol. UDP is considered always allowed */
                            if (ADK_IP_PROTOCOL_UDP != ipHdr->next_proto_id && !ADK_IP_ALLOWED_PROTO(ipHdr->next_proto_id))
                            {
                                INC_NON_NULL(if_stats, ifInUnknownProtoErrDrops, 1);
                                INC_NON_NULL(ip_addr_stats, ipInErrors, 1);
                                return adk_ip_dp_send_exception(ph_task, ADK_IP_UNSUPPORTED_PROTO_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
                            }

                            if (isReassembled && ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id)
                            {
                                uint16_t src_port_nw = 0, dest_port_nw = 0, udp_length = 0;
                                get_l4_params(ipHdr, &src_port_nw, &dest_port_nw, &udp_length);

                                if (ADK_TASK_PDU_SIZE(ph_task) < (ADK_MD_GET_L4_OFF(ph_task) + udp_length) ||
                                        (ADK_SYS_UDP_VALIDATE_FLAG_GET &&
                                                unlikely(ipv4_udp_checksum(ph_task, udp_length, ipHdr->src_addr, ipHdr->dst_addr))))
                                {
                                    ++if_stats->ifInUDPCksumErrDrops;
                                    return ADK_E_DROP_TASK;
                                }
                            }

                            ph_task = (adk_pktio_task_t *)(*task_p);
                            ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(*task_p) + IP_HDR_LEN);
                            return classify_and_terminate(ph_task, coreIndex);
                        } /* l4 cksum good */
                    }     /* terminating */
                    else
                    {
                        return proc_non_terminating(ph_task, coreIndex, if_stats, ipHdr,
                                stage_info_p, false /* send to ipuli */);
                    } /* non terminating */
                }     /* non blocked packets */
                else
                {
                    return ADK_E_DROP_TASK;
                } /* blocked packets */
            }     /* ip header good */
            else
            {
                /* Bad length and truncated packet errors are expected to be checked by HLP and adk_hos_is_ip_hdr_bad() would return TRUE in that case
               We know that header is corrupted. We want to increment specific error counters. Hence we are calling this api */
                uint16_t ip_len;
                return l3_header_validation(ph_task, coreIndex, ipHdr, &ip_len, NULL, if_stats);
            } /* ip header bad */
        }     /* v4 packet */
        else
        {
            if(6 == (ipHdr->version_ihl) >> 4)
            {   INC_NON_NULL(if_stats, ifInVersionMismatchErrDrops, 1);}
            else
            {INC_NON_NULL(if_stats, ifInInvalidVersionErrDrops, 1);}

            return adk_ip_dp_send_exception(ph_task, ADK_IP_INVALID_VERSION_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
        } /* non v4 packet */
    }     /* Packet not decrypted by HLP */
    else
    { /* TO DO: counter.. Packet decrypted and ipsec bypass is enabled. This shouldn't happen */
        return ADK_E_DROP_TASK;
    } /* Packet decrypted when ipsec module bypass is enabled */

} /* lli_single_pass */

static adk_status_t lli_pass2_lookaside(void **task_p,
                   int coreIndex,
                    adk_pal_dp_stage_info_t **stage_info_p)
{
    struct ipv4_hdr *ipHdr;
    adk_status_t st;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    ADK_PDU_OFFSET_PTR_GET(*task_p, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(*task_p));
    adk_ip_dp_intf_stats_t *if_stats=NULL;
    adk_ip_dp_interface_entry_t *ingIfEntry=NULL;
    ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(*task_p) + IP_HDR_LEN);

    if (likely(4 == (ipHdr->version_ihl) >> 4))
    {
        uint16_t ip_len;
        if(likely(ADK_SUCCESS == l3_header_validation(ph_task, coreIndex, ipHdr, &ip_len,
                ingIfEntry, if_stats)))
        {
            strip_eth_padding(ph_task, IP_PKT_LEN(ipHdr));
            qos_stats(ingIfEntry, if_stats, ipHdr);

            adk_ip_dp_route_info_t *ipRouteInfo_p;
            adk_ip_dp_ip_addr_stats_t *ip_addr_stats = NULL;
            bool terminate;
            st = route_lookup(ipHdr, &ipRouteInfo_p);
            if(likely(ADK_SUCCESS == st))
            {
                if(likely( (1 == ipRouteInfo_p->isTerminating)))
                {
                    terminate = true;
                    if(likely((ADK_HOS_DEFAULT_IP_STATS_ID != ipRouteInfo_p->u.termination.ip_addr_stats_id)))
                        ip_addr_stats = &(ip_dp_tbl_info_p->ipAddrStatsTble[coreIndex][ipRouteInfo_p->u.termination.ip_addr_stats_id]);
                }
                else
                    terminate = false;
            }
            else
                terminate = false;


            if(likely(terminate))
            {
                uint16_t src_port_nw = 0, dest_port_nw = 0, udp_length = 0;
                get_l4_params(ipHdr, &src_port_nw, &dest_port_nw, &udp_length);

                /* not sse & udp & not fragmented & validate-cksum-flag true & cksum bad */
                if (ip_addr_stats                                   &&
                    ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id     &&
                    !(ipHdr->fragment_offset & 0xFF3F)              &&
                    ADK_SYS_UDP_VALIDATE_FLAG_GET                   &&
                    unlikely(ipv4_udp_checksum_no_ol(ph_task, udp_length, ipHdr->src_addr, ipHdr->dst_addr))
                )
                {
                    /* TO DO: Increment interface error here */
                    return ADK_E_DROP_TASK;
                }    /* l4 cksum bad */
                else /* checksum validation is disabled or l4 cksum is valid */
                {
                    bool isReassembled = false;
                    INC_NON_NULL(ip_addr_stats, ipInPkts, 1);
                    INC_NON_NULL(ip_addr_stats, ipInOctets, rte_be_to_cpu_16(ipHdr->total_length));

                    if (ip_addr_stats && (ipHdr->fragment_offset & 0xFF3F))
                    { /* reassemble */
                        uint32_t hash_val;
                        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmReqPkts, 1);

                        /* Fragmented pkt. Do reassembly */
                        adk_ip_dp_get_reassembly_hash((adk_pktio_task_t **)task_p, ADK_MD_GET_IP_OFF(ph_task), &hash_val);
                        st = adk_ip_dp_reassemble(coreIndex, &ph_task, ADK_IP_DEFAULT_VRD_ID, ADK_DP_MAX_IP_STATS, hash_val);
                        if (ADK_SUCCESS != st)
                            return st;

                        INC_NON_NULL(ip_addr_stats, ipInReasmPkts, 1);
                        *task_p = (void *)ph_task;
                        ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
                        ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(ph_task) + IP_HDR_LEN);
                        isReassembled = true;
                    } /* end of Reassembly block*/

                    /* Check if its a supported l4 protocol. UDP is considered always allowed */
                    if (ADK_IP_PROTOCOL_UDP != ipHdr->next_proto_id &&
                            !ADK_IP_ALLOWED_PROTO(ipHdr->next_proto_id))
                    {
                        INC_NON_NULL(ip_addr_stats, ipInErrors, 1);
                        return adk_ip_dp_send_exception(ph_task, ADK_IP_UNSUPPORTED_PROTO_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
                    }

                    if (isReassembled && ADK_IP_PROTOCOL_UDP == ipHdr->next_proto_id)
                    {

                        if (ADK_TASK_PDU_SIZE(ph_task) < (ADK_MD_GET_L4_OFF(ph_task) + udp_length) ||
                                (ADK_SYS_UDP_VALIDATE_FLAG_GET &&
                                        unlikely(ipv4_udp_checksum_no_ol(ph_task, udp_length, ipHdr->src_addr, ipHdr->dst_addr))))
                        {
                            /* TO DO: Increment interface error here */
                            return ADK_E_DROP_TASK;
                        }
                    }

                    ph_task = (adk_pktio_task_t *)(*task_p);
                    ADK_MD_SET_L4_OFF(ph_task, ADK_MD_GET_IP_OFF(*task_p) + IP_HDR_LEN);
                    return classify_and_terminate(ph_task, coreIndex);
                } /* l4 cksum good */

            } /* terminating */
            else
            {
                return proc_non_terminating(ph_task, coreIndex, if_stats, ipHdr,
                        stage_info_p, false /* send to ipuli */);
            } /* non terminating */
        }
        else
            return ADK_E_DROP_TASK;
    }
    else
    {
        if(6 == (ipHdr->version_ihl) >> 4)
        {   INC_NON_NULL(if_stats, ifInVersionMismatchErrDrops, 1);}
        else
            {INC_NON_NULL(if_stats, ifInInvalidVersionErrDrops, 1);}

        return adk_ip_dp_send_exception(ph_task, ADK_IP_INVALID_VERSION_EXCEPTION, coreIndex, 0, ADK_IP_DIR_INGRESS);
    }
} /* lli_pass2_lookaside */


adk_status_t adk_ip_dp_pkt_proc_lli_single (void **task_p,
    int coreIndex,
    adk_pal_dp_stage_info_t **stage_info_p)
{
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    adk_ip_dp_lli_imc_data_t *iplliImc = (adk_ip_dp_lli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task));
    adk_status_t st = ADK_E_DROP_TASK;
    lli_single_entry_logs(*task_p, (adk_ip_dp_lli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(*task_p)));
    if(IPLLI_IMC_ETH_OUTER == iplliImc->ipsec_data)
    {
        if (likely(!(ADK_SYS_IPSEC_BYPASS_GET)) )
        {
            ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_1();
            st = lli_pass1(task_p, coreIndex, stage_info_p);
            ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_1();
        } /* ipsec not bypassed */
        else
        {
            ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2();
            st = lli_single_pass(task_p, coreIndex, stage_info_p);
        } /* ipsec bypassed */
    }
    /* If it is a plaintext packet coming back from ipsec. We can directly send it to the classifier */
    else if (IPLLI_IMC_IPSEC_OUTER == iplliImc->ipsec_data)
    {
        ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2();
        st = classify_and_terminate(ph_task, coreIndex);
    }
    else if(IPLLI_IMC_IPSEC_INNER_INLINE == iplliImc->ipsec_data)
    {
        ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2();
        st = lli_pass2(task_p, coreIndex, stage_info_p);
    }
    else if(IPLLI_IMC_IPSEC_INNER_LOOKASIDE == iplliImc->ipsec_data)
    {
        ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2();
        st = lli_pass2_lookaside(task_p, coreIndex, stage_info_p);
    }
    else
    {
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInUnknownSender, 1);
        st = ADK_E_DROP_TASK;
    }

    lli_single_exit_logs(ph_task, (adk_ip_dp_lli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task)));
    return st;
}


/* lli_single_entry_logs: Separates out entry logs for code readability. No error checking done */
static inline void lli_single_entry_logs(
        adk_pktio_task_t *ph_task,
        adk_ip_dp_lli_imc_data_t *iplliImc

)
{
#if defined(ADK_DEBUG)
    struct ipv4_hdr *ipHdr;
    ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
    ADK_IP_DP_LOG_INFO_DEBUG("IP LLI_single entry::\n");

    ADK_IP_DP_LOG_IP_HDR(ipHdr);


    ADK_IP_DP_LOG_INFO_DEBUG("ipIfId=0x%x, ip-offset=%d, l4-offset=%d, iphdrPtr: %p,PH task: %p\n",
            getIpIfId(ph_task), ADK_MD_GET_IP_OFF(ph_task), ADK_MD_GET_L4_OFF(ph_task), ipHdr, ph_task);

    ADK_IP_DP_LOG_INFO_DEBUG("ip-lli-imc::ipsec-data=0x%02x, pcp=0x%02x, profile-id=0x%02x\n",
                                iplliImc->ipsec_data, iplliImc->pcp, iplliImc->profile_id);

    ADK_IP_DP_LOG_INFO_DEBUG("\n \tis-inline-decrypted=%d,\n \tis-L3-terminating=%d,\n \tis-classifier-invalid=%d,\n \tclassifier-rule-id=%d,\n \tis-ip-hdr-bad=%d,\n "
            "\tis-l4-cksum-err=%d\n",  adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(ph_task)), adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(ph_task))
    );

    if(!adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(ph_task)))
    {
        uint32_t flow_id = adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(ph_task));
        ADK_IP_DP_LOG_INFO_DEBUG("\tclassifier-action=%d\n",
                ip_dp_tbl_info_p->ingClassifierActionDp_p[ip_dp_tbl_info_p->ingClassifierInfo_p->aclTreeIndex][flow_id].action);
    }
#else
    RTE_SET_USED(ph_task);
    RTE_SET_USED(iplliImc);
#endif
} /* lli_single_entry_logs */

/* lli_single_eit_logs: Separates out entry logs for code readability. No error checking done */
static inline void lli_single_exit_logs(
        adk_pktio_task_t *ph_task,
        adk_ip_dp_lli_imc_data_t *iplliImc
)
{
#if defined(ADK_DEBUG)
    struct ipv4_hdr *ipHdr;
    ADK_PDU_OFFSET_PTR_GET(ph_task, ipHdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(ph_task));
    ADK_IP_DP_LOG_INFO_DEBUG("IP LLI_single exit::\n");

    ADK_IP_DP_LOG_IP_HDR(ipHdr);

    ADK_IP_DP_LOG_INFO_DEBUG("ipIfId=0x%x, ip-offset=%d, l4-offset=%d, iphdrPtr: %p,PH task: %p\n",
                                INGRESS_IF_ID(ph_task), ADK_MD_GET_IP_OFF(ph_task), ADK_MD_GET_L4_OFF(ph_task), ipHdr, ph_task);

    ADK_IP_DP_LOG_INFO_DEBUG("ip-lli-imc::ipsec-data=0x%02x, pcp=0x%02x, profile-id=0x%02x\n",
                                iplliImc->ipsec_data, iplliImc->pcp, iplliImc->profile_id);

    ADK_IP_DP_LOG_INFO_DEBUG("\n \tis-inline-decrypted=%d,\n \tis-L3-terminating=%d,\n \tis-classifier-invalid=%d,\n \tclassifier-rule-id=%d,\n \tis-ip-hdr-bad=%d,\n "
            "\tis-l4-cksum-err=%d\n",  adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(ph_task)), adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(ph_task)),
            adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(ph_task))
    );

    if(!adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(ph_task)))
    {
        uint32_t flow_id = adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(ph_task));
        ADK_IP_DP_LOG_INFO_DEBUG("\tclassifier-action=%d\n",
                ip_dp_tbl_info_p->ingClassifierActionDp_p[ip_dp_tbl_info_p->ingClassifierInfo_p->aclTreeIndex][flow_id].action);
    }
#else
    RTE_SET_USED(ph_task);
    RTE_SET_USED(iplliImc);
#endif
} /* lli_single_exit_logs */

adk_status_t adk_ip_dp_pkt_proc_police (void **task_p,
          __attribute__((unused)) uint8_t task_count,
          __attribute__((unused)) uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex)
{
    uint8_t policeColor = ADK_QOS_DP_GREEN;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(ph_task);
    struct ipv4_hdr *iphdr;
    adk_ip_dp_policer_meta_data_t *ip_policer_md_p = NULL;
    uint8_t flow_id;
    adk_pal_dp_stage_info_t *em_lli_stage_info_p = NULL;
    uint8_t tasks_sent;
    adk_dp_status_t dp_st;
    uint8_t fn_id;

    adk_ip_dp_interface_entry_t *ingIfEntry;
    uint32_t ingressIfId;

    ip_policer_md_p = (adk_ip_dp_policer_meta_data_t *)&(ADK_METADATA_PARAM(ph_task, imc_param_start));

    /* Get the ingress if id from l2 if handle */
    ingressIfId = adk_md_p->input_intf_id;

    /* Get ip if handle from ipIfTble */
    ingIfEntry = &(ip_dp_tbl_info_p->ipInterfaceTble[ingressIfId]);

    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
                           adk_md_p->ip_offset);

    if(unlikely( ADK_SUCCESS !=
                adk_qos_dp_policing_color_blind(
                    ip_policer_md_p->ing_action.policerFlowId,
                    rte_be_to_cpu_16(iphdr->total_length), &policeColor) ) )
    {
        /*Free the task*/
        adk_pktio_task_buffer_free(ph_task);
        return ADK_SUCCESS;
    }

    /* check policing color */
    if(unlikely(ADK_QOS_DP_RED == policeColor))
    {
        /* Dropping the packet */
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInPolicerDrops++;
        adk_pktio_task_buffer_free(ph_task);
        return ADK_SUCCESS;
    }
    
    flow_id = ip_policer_md_p->ingress_classifier_flow_id;

    if(ADK_METADATA_PARAM(ph_task, event_sched_type) == RTE_SCHED_TYPE_ORDERED)
    {
        fn_id = ADK_SIF_EM_LLI_FN_ID;
    }
    else
    {
        fn_id = ADK_SIF_EM_LLI_DIRECT_FN_ID;
    }

    adk_sif_em_dp_lli_meta_data_t *em_lli_md_p = (adk_sif_em_dp_lli_meta_data_t *)&(ADK_METADATA_PARAM(ph_task,
                imc_param_start));
    em_lli_md_p->eventMachineQueueId = ip_policer_md_p->ing_action.eventMachineQueueId;
    /*Get the stage info from PAL*/
    dp_st = adk_pal_dp_stage_info_get(fn_id, &em_lli_stage_info_p);
    if(dp_st != ADK_SUCCESS)
    {
        /* Note: this API would fail only if you have passed NULL pointer.
         * Hence, don't think we need a counter here
         */
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Failed to get EM LLI stage from PAL");
        adk_pktio_task_buffer_free(ph_task);
        return ADK_E_INTERNAL_ERROR;
    }

    tasks_sent = adk_pal_dp_next_stage_event_handle_with_stage_info(
            (void **)&ph_task, 1, 0x1,
            em_lli_stage_info_p, em_lli_md_p->eventMachineQueueId,
            0x00, ADK_METADATA_PARAM(ph_task, task_type));
    if(tasks_sent == 0)
    {
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Failed to enqueue to EM LLI stage");
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInSendToEoFailDrops++;
        adk_pktio_task_buffer_free(ph_task);
        return ADK_E_INTERNAL_ERROR;
    }
    INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipInSentToEoPkts, 1);
    ip_dp_tbl_info_p->ipIngFlowStats[coreIndex * ADK_DP_MAX_IP_ING_ASSOC_FLOWS + flow_id].txPkts ++;
    ADK_IP_DP_LOG_INFO_DEBUG("\nadk_em_send_to_eo() success!\n");

    return ADK_SUCCESS;
} /*end of adk_ip_dp_pkt_proc_police */

void adk_ip_dp_slowpath_policer_stats(void *user_params)
{
    uint32_t classifier_flow_id;

    if(!user_params)
    {
        ADK_IP_DP_LOG_ERROR("\nInvalid slow path parameters\n");
        return;
    }

    classifier_flow_id = ((adk_slowpath_user_param_t *)user_params)->flow_id;

    if(((adk_slowpath_user_param_t *)user_params)->policeFlowId != UINT_MAX) /* Policing is enabled */
    {
        ADK_IP_DP_LOG_INFO_DEBUG("\nClassifier flow id ==>%u\n", classifier_flow_id);

        if(((adk_slowpath_user_param_t *)user_params)->pkt_color == ADK_QOS_DP_GREEN)
        {
            ADK_IP_DP_LOG_INFO_DEBUG("\nPacket marked green by policer\n");
            ip_dp_tbl_info_p->ipCppiStats[classifier_flow_id].cppiTxPkts++;
            INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].UipInSentToHostPkts, 1);
            ADK_IP_DP_LOG_INFO_DEBUG("Incrementing Rx L4 stats for NETD path, policer gave GREEN. proto=%d, len=%d\n", ((adk_slowpath_user_param_t *)user_params)->l4_protocol, ((adk_slowpath_user_param_t *)user_params)->l4_len);
            ip_inc_l4_in_stats(ADK_LCORE_INDEX, ((adk_slowpath_user_param_t *)user_params)->l4_protocol, ((adk_slowpath_user_param_t *)user_params)->l4_len);
        }
        else
        {
            ADK_IP_DP_LOG_INFO_DEBUG("\nPacket marked red/yellow by policer\n");
            ip_dp_tbl_info_p->ipCppiStats[classifier_flow_id].cppiPolicerDropPkts++;
            ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].ipInSendToHostFailDrops++;
        }
    }
    else /*Policing is disabled */
    {
        ip_dp_tbl_info_p->ipCppiStats[classifier_flow_id].cppiTxPkts++;
        ADK_IP_DP_LOG_INFO_DEBUG("Incrementing Rx L4 stats for NETD path, policer disabled. proto=%d, len=%d\n", ((adk_slowpath_user_param_t *)user_params)->l4_protocol, ((adk_slowpath_user_param_t *)user_params)->l4_len);
        ip_inc_l4_in_stats(ADK_LCORE_INDEX, ((adk_slowpath_user_param_t *)user_params)->l4_protocol, ((adk_slowpath_user_param_t *)user_params)->l4_len);
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].UipInSentToHostPkts, 1);
    }

    return;
}

