/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include <math.h>
#include "adk_sys_lcore.h"
#include <adk_ip_common.h>
#ifdef IP_FLOW_HASH_CACHE
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#endif
#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#include <adk_hos_dp.h>
#include <adk_qos_dp_export.h>
#include <adk_cksum.h>
#include <adk_timer.h>
#include <adk_pmu.h>
#include <adk_ip_dp_internal.h>
#include <adk_ip_dp_internal_fc.h>

#include "adk_ph_dev_dp.h"

static uint32_t activeReasmContexts;

static void reassembly_session_entry_cleanup(uint16_t res_index,
            uint8_t collision_index, int32_t coreIndex,
            adk_ip_dp_reassembly_session_tbl_t *session_entry_p)
{
    uint32_t i;

    /* TO DO: send mbuf here instead of NULL */
    adk_ip_dp_acquire_reassembly_lock_non_atomic(NULL, res_index);

    ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmTouts++;

    /* single-sa case: non-zero frag_offset means packets have landed back in IP and are currently parked in ip reassembly context */
    for(i =0; i < session_entry_p->numEnqueues; ++i)
    {
        adk_pktio_task_buffer_free((adk_pktio_task_t *)
                session_entry_p->enqd_tasks[i].frag_task);
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmToutFragDrops++;
    }

    session_entry_p->sip = 0;
    session_entry_p->dip = 0;
    session_entry_p->ip_id = 0;
    session_entry_p->protocol = 0;
    /*empty list counter */
    session_entry_p->numEnqueues = 0;
    session_entry_p->version++;
    /*make current bucket entry as invalid */
    ipReassemblyTbl[res_index].buckets[collision_index].sess_entry_p =
                            NULL;
    ipReassemblyTbl[res_index].buckets[collision_index].hash_id=0;
    ipReassemblyTbl[res_index].count_hashid -= 1;

    __atomic_fetch_sub(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);

    rte_mempool_put(ipReassemblySessionTbl, (void *)session_entry_p);
    adk_ip_dp_release_reassembly_lock_non_atomic(NULL, res_index);
}

#ifdef ADK_TIMER
void reassembly_timer_cb_fn(void *reasm_col_row_index,
                     adk_timer_event_type_t event_type)
{
    int32_t coreIndex = ADK_LCORE_INDEX;
    if(event_type == ADK_TIMER_EVENT_TYPE_DELETE)
    {
        return;
    }

    /* ADK_TIMER_EVENT_TYPE_TIMEOUT */
    uint32_t flow_id, queue_id, ip_term_stats_id;
    /*Obtain Session table indexes*/
    uint16_t res_index = ((uint64_t)reasm_col_row_index) & 0xFFFF;
    uint8_t collision_index = (((uint64_t)reasm_col_row_index) & 0x00FF0000) >> 16;
    uint8_t version_timer = (((uint64_t)reasm_col_row_index) & 0x00FF000000) >> 24;

    adk_ip_dp_acquire_reassembly_lock_non_atomic(NULL, res_index);
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p;
    session_entry_p = ipReassemblyTbl[res_index].
                buckets[collision_index].sess_entry_p;
    if(!session_entry_p || session_entry_p->version != version_timer)
    {
        /* Race condition, timeout event for an already cleaned up reassembly
         * session. Ignore the timeout event*/
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmTimerCbAfterCleanup, 1);
        adk_ip_dp_release_reassembly_lock_non_atomic(NULL, res_index);
        return;
    }
    flow_id = session_entry_p->flow_id;
    queue_id = session_entry_p->queue_id;
    ip_term_stats_id = session_entry_p->ip_term_stats_id;

    ADK_IP_DP_LOG_INFO_DEBUG("Reassembly timeout for packet: sip=0x%x, dip=0x%x, pkt-id=%d, proto=%d\n", adk_ntohl(session_entry_p->sip), adk_ntohl(session_entry_p->dip),
            session_entry_p->ip_id, session_entry_p->protocol);

    if(ipReassemblyTbl[res_index].buckets[collision_index].ipsecCb)
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmToutFragDrops += ipReassemblyTbl[res_index].buckets[collision_index].ipsecCb(res_index, collision_index);
    }

    adk_ip_dp_release_reassembly_lock_non_atomic(NULL, res_index);
    /*TODO check for lock optimization possibilitiy*/
    uint32_t ret,retry_count=0;
    struct rte_mbuf *meta_mbuf = NULL;
    struct rte_mempool *mempool= NULL;
    adk_status_t retVal;

    adk_sys_config_pktio_mempool_get((void **) &mempool);
    /*check for mempool validity */
    if(NULL == mempool)
    {
        ADK_IP_DP_LOG_INFO_DEBUG("Failed to get mempool info from sys_config api\n");
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmToutMbufAllocFails++;
        /* clear reassembly session entries*/
        reassembly_session_entry_cleanup(res_index, collision_index, coreIndex,
                session_entry_p);
        return;
    }

    meta_mbuf = rte_pktmbuf_alloc(mempool);
    if(NULL == meta_mbuf)
    {
        ADK_IP_DP_LOG_INFO_DEBUG("mbuf allocation failed for reasm metadata\n");
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmToutMbufAllocFails++;
        /* clear reassembly session entries*/
        reassembly_session_entry_cleanup(res_index, collision_index, coreIndex,
                session_entry_p);
        return ;
    }
    adk_ip_dp_reass_timer_mdata_t *reass_timer_md_p =
        (adk_ip_dp_reass_timer_mdata_t *)
        &(ADK_MD_GET_IMC_PARAM_START(meta_mbuf));
    reass_timer_md_p->collisionIndex = collision_index;
    reass_timer_md_p->res_index = res_index;
    reass_timer_md_p->flow_id = flow_id;
    reass_timer_md_p->queue_id = queue_id;
    reass_timer_md_p->ip_term_stats_id = ip_term_stats_id;
    reass_timer_md_p->version = version_timer;

    /*TODO: look for optimization here*/
    adk_pal_dp_stage_info_t *reasm_stage_info_p = NULL;
    /*Get the stage info from PAL*/
    retVal = adk_pal_dp_stage_info_get(ADK_IPV4_REASM_TOUT_FN_ID,
            &reasm_stage_info_p);
    if(retVal != ADK_SUCCESS)
    {
        ADK_IP_DP_LOG_INFO("Failed to get next stage info from PAL");
        /* clear reassembly session entries*/
        reassembly_session_entry_cleanup(res_index, collision_index, coreIndex,
                session_entry_p);
        /* Free the temporary task to avoid memory leak */
        adk_pktio_task_buffer_free(meta_mbuf);
        return;
    }
    reasm_stage_info_p->event_q_id = queue_id;
    /* Enqueue new event to PAL */
    do{
        ret = adk_pal_dp_next_stage_event_handle_with_stage_info(
                (void **)&meta_mbuf, 1, 0x1,
                reasm_stage_info_p, flow_id,
                0x00, ADK_TASK_TYPE_NEW);
        retry_count++;
    }while(ret == 0 && retry_count <= REASM_ENQ_RETRY_COUNT);

    if(ret == 0)
    {
        /* clear reassembly session entries*/
        reassembly_session_entry_cleanup(res_index, collision_index, coreIndex,
                session_entry_p);
        /* Free the temporary task to avoid memory leak */
        adk_pktio_task_buffer_free(meta_mbuf);
    }
    return;
}
#endif

adk_status_t adk_ip_dp_pkt_proc_reasm_tout(void **task_p,
                        __attribute__((unused)) uint8_t task_count,
                        __attribute__((unused)) uint32_t task_mask,
                        __attribute__((unused)) void *ring,
                        __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
                        int coreIndex)
{
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p;

    adk_ip_dp_reass_timer_mdata_t *reass_md_p =
         (adk_ip_dp_reass_timer_mdata_t *)&(ADK_MD_GET_IMC_PARAM_START(ph_task));

    session_entry_p = ipReassemblyTbl[reass_md_p->res_index].
                buckets[reass_md_p->collisionIndex].sess_entry_p;
    if(session_entry_p && session_entry_p->version == reass_md_p->version)
    {
        reassembly_session_entry_cleanup(reass_md_p->res_index,reass_md_p->collisionIndex, coreIndex,
            session_entry_p);
    }
    else
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmTimerCbAfterCleanup, 1);

    /* Free the temporary task to avoid memory leak */
    adk_pktio_task_buffer_free(ph_task);
    return ADK_SUCCESS;

}

adk_status_t adk_ip_dp_pkt_proc_reassemble (void **task_p,
          __attribute__((unused)) uint8_t task_count,
          __attribute__((unused)) uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex)
{
    adk_status_t st;
    adk_pktio_task_t *ph_task = (adk_pktio_task_t *)(*task_p);
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(ph_task);
    adk_ip_dp_reass_meta_data_t *reass_md_p =
            (adk_ip_dp_reass_meta_data_t *)
            &(ADK_MD_GET_IMC_PARAM_START(ph_task));
    struct ipv4_hdr *iphdr;
    struct udp_hdr *udphdr;
    struct icmp_hdr *icmphdr;
    adk_ip_dp_intf_stats_t *ip_stats = NULL;
    uint16_t src_port_nw, dest_port_nw, udp_len=0, tmp_16;

    uint16_t if_stats_id;
    adk_ip_dp_interface_entry_t *ingIfEntry;
    uint32_t ingressIfId;

    adk_pal_dp_stage_info_t stage_info = {0};
#ifdef IP_FLOW_HASH_CACHE
    adk_ip_dp_lli_hash3_tbl_entry_t *hash3Entry=NULL;
    adk_ip_dp_lli_hash5_tbl_entry_t *hash5Entry=NULL;
#endif

    /* Get the ingress if id from l2 if handle */
    ingressIfId = adk_md_p->input_intf_id;

    /* Get ip if handle from ipIfTble */
    ingIfEntry = &(ip_dp_tbl_info_p->ipInterfaceTble[ingressIfId]);

    if_stats_id = ingIfEntry->if_stats_id; 

    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
            ADK_MD_GET_IP_OFF(ph_task));
    st = adk_ip_dp_reassemble(coreIndex, (adk_pktio_task_t **)(&ph_task),
            reass_md_p->vrdid,
            reass_md_p->ip_term_stats_id,
            reass_md_p->hash_val);

    if(ADK_E_INTERNAL_ERROR == st)
    {
        adk_pktio_task_buffer_free(ph_task);
        return ADK_SUCCESS;
    }
    else if(ADK_E_TASK_CONSUMED == st)
    {
        /*Fragment parked for reassembly*/
        return ADK_SUCCESS;
    }

    /*Reassembly completed*/

    /*Get the interface stats id*/
    ip_stats = &(ip_dp_tbl_info_p->
            ipInterfaceStatsTble[coreIndex][reass_md_p->ip_term_stats_id]);

    /* After reassembly task pointer might be different. Hence the map
     * the new pointer for the Bulk processing IP LLI loop*/
    adk_md_p = ADK_METADATA_PTR(ph_task);
    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
            ADK_MD_GET_IP_OFF(ph_task));
    *task_p = (void *)ph_task;
    /* Get the L4 fields after reassembly */
    if((iphdr->next_proto_id == IP_FILTER_PROTOCOL_UDP) ||
            (iphdr->next_proto_id == IP_FILTER_PROTOCOL_UDP_LITE) ||
            (iphdr->next_proto_id == IP_FILTER_PROTOCOL_TCP) ||
            (iphdr->next_proto_id == IP_FILTER_PROTOCOL_SCTP))
    {
        ADK_PDU_OFFSET_PTR_GET(ph_task, udphdr, struct udp_hdr *,
                ADK_MD_GET_L4_OFF(ph_task));
        src_port_nw = udphdr->src_port;
        dest_port_nw = udphdr->dst_port;
        udp_len = rte_be_to_cpu_16(udphdr->dgram_len);
    }
    else if(iphdr->next_proto_id == IP_FILTER_PROTOCOL_ICMP)
    {
        ADK_PDU_OFFSET_PTR_GET(ph_task, icmphdr, struct icmp_hdr *,
                ADK_MD_GET_L4_OFF(ph_task));
        tmp_16 = icmphdr->icmp_type; /*type*/
        src_port_nw = rte_cpu_to_be_16(tmp_16);
        tmp_16 = icmphdr->icmp_code; /*code*/
        dest_port_nw = rte_cpu_to_be_16(tmp_16);
    }
    else if(iphdr->next_proto_id == IP_FILTER_PROTOCOL_IGMP)
    {
        ADK_PDU_OFFSET_PTR_GET(ph_task, icmphdr, struct icmp_hdr *,
                ADK_MD_GET_L4_OFF(ph_task));
        tmp_16 = icmphdr->icmp_type; /*type*/
        src_port_nw = rte_cpu_to_be_16(tmp_16);
        dest_port_nw = 0;
    }
    else
    {
        src_port_nw = 0;
        dest_port_nw = 0;
    }

    if(((ADK_E_INTERNAL_ERROR == st)
            ||(ADK_E_DROP_TASK == st)))
    {
        /*Free the task*/
        adk_pktio_task_buffer_free(ph_task);
    }

    if (adkDestEntryFunction_g[stage_info.fn_id])
        (adkDestEntryFunction_g[stage_info.fn_id])(task_p, 1, 1, NULL,
                NULL, coreIndex);

    return ADK_SUCCESS;
}


static inline void adk_ip_dp_get_staggered_timeout_value(adk_timer_info_t *timeval)
{
    if(activeReasmContexts < IP_REASSEMBLY_MAX/3)
        timeval->interval_milli_sec = ip_dp_tbl_info_p->reasmStage1;
    else if(activeReasmContexts < IP_REASSEMBLY_MAX/2)
        timeval->interval_milli_sec = ip_dp_tbl_info_p->reasmStage2;
    else if(activeReasmContexts < (IP_REASSEMBLY_MAX*2)/3)
        timeval->interval_milli_sec = ip_dp_tbl_info_p->reasmStage3;
    else
        timeval->interval_milli_sec = ip_dp_tbl_info_p->reasmStage4;
}

/* res_index and bucket_index are used to allocate entry at right index and right
 * collision bucket for that index. These 2 are also 'combined' and passed to timer
 * callback function for cleanup.
 */
static inline adk_status_t setup_context_and_start_timer
(
        uint16_t coreIndex,
        adk_pktio_task_t **ph_task_p,
        uint32_t hash_val,
        uint16_t res_index,
        uint8_t bucket_index,
        uint16_t ip_offset,
        clear_ipsec_ctx_for_hash ipsecCb,
        uint32_t toutVal
)
{
    int ret;
    void *reasm_p;
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p = NULL;
    /* TO DO: Get this from reassembly timout config */
    adk_timer_info_t timer_info = {ADK_REASSEMBLY_TIMEOUT_MILLISECONDS};
    uint64_t reasm_col_row_index = 0;
    struct ipv4_hdr *iphdr;

    ADK_PDU_OFFSET_PTR_GET(*ph_task_p, iphdr, struct ipv4_hdr *, ip_offset);
    if(rte_mempool_get(ipReassemblySessionTbl, &reasm_p) < 0)
    {
        ADK_IP_DP_LOG_ERROR("Failed to alloc reassembly session entry");
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmSessionAllocFailDrops++;
        return ADK_E_INTERNAL_ERROR;
    }
    session_entry_p = (adk_ip_dp_reassembly_session_tbl_t *)reasm_p;
    session_entry_p->vrd_id = ADK_IP_DEFAULT_VRD_ID;

#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
    /* timer create */
    session_entry_p->flow_id = ADK_METADATA_PARAM(*ph_task_p, lock_id);
    session_entry_p->queue_id = ADK_METADATA_PARAM(*ph_task_p, event_queue_id);
    /* We don't have ip_term_stats_id when called from ipsec. Hence setting to an invalid value */
    session_entry_p->ip_term_stats_id = ADK_DP_MAX_IP_STATS;

    /* Add 5 tuple to session entry (this will be used in start_reassembly function)
     * Later on, when actual fragment is sent back by ipsec, ip-lli will call
     * adk_ip_dp_reassemble_internal and exact bucket (allocated in this function) will be
     * found and 'actual' reassembly logic will be executed (adk_ip_dp_reasm_sessionTbl_update) */
    session_entry_p->sip = iphdr->src_addr;
    session_entry_p->dip = iphdr->dst_addr;
    session_entry_p->ip_id = iphdr->packet_id;
    session_entry_p->protocol = iphdr->next_proto_id;
    session_entry_p->vrd_id = ADK_IP_DEFAULT_VRD_ID;

    reasm_col_row_index = (uint64_t)((uint64_t)res_index |
                                    (((uint64_t)bucket_index) << 16) |
                                    (((uint64_t)(session_entry_p->version)) << 24));

    /* Decide on timer value based on forwarded or terminating */
    if(0 == toutVal)
        adk_ip_dp_get_staggered_timeout_value(&timer_info);
    else
        timer_info.interval_milli_sec = toutVal;

    ADK_IP_DP_LOG_INFO_DEBUG("Creating reassembly timer for row_index: %d, col_index:%d, timeout (ms)=%d\n",
            res_index, bucket_index, timer_info.interval_milli_sec);
    ret = adk_timer_create(coreIndex, timer_info, reassembly_timer_cb_fn, (void *)reasm_col_row_index,
            &(session_entry_p->timer));
    if(unlikely(0 == ret))
    {
        ADK_IP_DP_LOG_ERROR("Failed to create timer for IP reassembly");
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmTimerCreateFailDrops++;
        return ADK_E_INTERNAL_ERROR;
    }
#endif
#endif
    ipReassemblyTbl[res_index].buckets[bucket_index].hash_id = hash_val;
    ipReassemblyTbl[res_index].buckets[bucket_index].ipsecCb = ipsecCb;
    ipReassemblyTbl[res_index].buckets[bucket_index].sess_entry_p = session_entry_p;
    ipReassemblyTbl[res_index].count_hashid += 1;
    __atomic_fetch_add(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);

    return ADK_SUCCESS;
}

inline void adk_ip_dp_acquire_reassembly_lock_non_atomic(
        __attribute__((unused)) adk_pktio_task_t *ph_task,
        uint16_t res_index
)
{
#if 0 /* TO DO: Enable after mbuf is available in timer cb */
    if(ADK_METADATA_PARAM(ph_task, event_sched_type) == RTE_SCHED_TYPE_ATOMIC)
    {
        /* no lock needed */
    }
    else
    {
        rte_spinlock_lock(&(ipReassemblyTbl[res_index].lock));
    }
#else
    rte_spinlock_lock(&(ipReassemblyTbl[res_index].lock));
#endif

}

inline void adk_ip_dp_release_reassembly_lock_non_atomic(
        __attribute__((unused)) adk_pktio_task_t *ph_task,
        uint16_t res_index
)
{
#if 0 /* TO DO: Enable after mbuf is available in timer cb */
    if(ADK_METADATA_PARAM(ph_task, event_sched_type) == RTE_SCHED_TYPE_ATOMIC)
    {
        /* no lock would have been taken. so no need to unlock */
    }
    else
    {
        rte_spinlock_unlock(&(ipReassemblyTbl[res_index].lock));
    }
#else
    rte_spinlock_unlock(&(ipReassemblyTbl[res_index].lock));
#endif

}

inline adk_status_t adk_ip_dp_get_reassembly_hash(adk_pktio_task_t **ph_task_p,
        uint16_t ip_offset, uint32_t *hash_val)
{
    adk_pktio_task_t *ph_task = *ph_task_p;
    struct ipv4_hdr *iphdr;

    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
            ip_offset);
    *hash_val = SISD_CRC_HASH(0, (iphdr->packet_id << 16) +
            (iphdr->next_proto_id << 8) + ADK_IP_DEFAULT_VRD_ID); /* TBD: this will not work if we introduce vrd-id again */
    *hash_val = SISD_CRC_HASH(*hash_val, iphdr->src_addr);
    *hash_val = SISD_CRC_HASH(*hash_val, iphdr->dst_addr);

    return ADK_SUCCESS;
}

inline adk_status_t adk_ip_dp_get_reassembly_hash_row(adk_pktio_task_t **ph_task_p,
        uint16_t ip_offset, uint32_t *hash_val, uint16_t *res_index)
{
    adk_pktio_task_t *ph_task = *ph_task_p;
    struct ipv4_hdr *iphdr;

    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
            ip_offset);
    *hash_val = SISD_CRC_HASH(0, (iphdr->packet_id << 16) +
            (iphdr->next_proto_id << 8) + ADK_IP_DEFAULT_VRD_ID); /* TBD: this will not work if we introduce vrd-id again */
    *hash_val = SISD_CRC_HASH(*hash_val, iphdr->src_addr);
    *hash_val = SISD_CRC_HASH(*hash_val, iphdr->dst_addr);

    *res_index = *hash_val & IP_DP_REASS_ID_MASK;

    return ADK_SUCCESS;
}


/* adk_ip_dp_ipsec_lock_and_start_reassembly : This API is specifically exposed to be used by IPSec
 * to handle Single-SA case. IPSec needs to buffer the encrypted fragments until initial fragment is
 * received in order to perform policy lookup. These packets can't be buffered indefinitely. Hence
 * IP reassembly timer is applied for them. In order to achieve this, IP exposes this API.
 * IPSec will call this when it receives first encrypted fragment (initial or non-initial).
 *
 * Note: Lock Handling: In current design, IP and IPSec modules access their own tables during single-sa
 * reassembly start and clear-off (e.g. at the time of timer expiry). A lock is needed in both the modules.
 * In order to avoid deadlock scenarios by using 2 locks, it is decided that single lock will be used.
 * This function will acquire and release the lock for its processing.
 * IPSec should use acquire and release APIs exposed by IP for any processing based on this hash-id.
 *
 * Prerequisite: Reassembly lock should be taken through exposed API before calling this function.
 *
 * This API does the following:
 * 1. Acquire lock using res_index provided.
 * 2. Check if we can find a free bucket for reassembly context
 * 3. If no, release the lock and return error
 * 4. If yes, populate and start the timer
 * 5. store callback function, release the lock and return bucket id.
 *
 * Note: We don't have ip_term_stats_id here when calling from ipsec.
 *
 */
inline adk_status_t adk_ip_dp_ipsec_start_reassembly(uint16_t coreIndex,
        adk_pktio_task_t **ph_task_p,
        uint32_t hash_val,
        uint16_t res_index,
        uint16_t ip_offset,
        clear_ipsec_ctx_for_hash ipsecCb,
        uint32_t preFragBufferTimeOutVal,
        uint8_t *bucket_index
    )
{

    adk_status_t ret = ADK_SUCCESS;
    uint16_t i=0;
    struct ipv4_hdr *iphdr;
    bool slot_found=false;
    uint32_t bkt_visit_cnt;

    ADK_PDU_OFFSET_PTR_GET(*ph_task_p, iphdr, struct ipv4_hdr *, ip_offset);

    if( adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(*ph_task_p)) )
    {
            preFragBufferTimeOutVal = 0;
    }

    *bucket_index= 0xff;

    if(ipReassemblyTbl[res_index].count_hashid == 0)
    {
        /* No reassembly context at res_index in ipReassemblyTbl. Allocate one at 0th index */
        slot_found = true;
        *bucket_index = 0;
    }
    else
    {
        /* We already have active context/s at index 'res_index' in ipReassemblyTbl.
         * Now, find if we have an empty slot OR if we already have a reassembly context
         * for this packet.
         *
         * If i reaches ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION, we have as many active contexts different
         * that current packet. So, we need to drop the packet.
         *
         * If bkt_visit_cnt becomes count_hashid, we have visited all the active contexts, we can break out of the
         * loop.
         *
         * This loop first checks if slot is empty. If yes, it records the slot index and sets slot_found to true.
         * Loop is still continued to check if there already is a matching context.
         *
         * If slot is not empty, we check if hash-id matches.
         * If yes,
         *      We further check if there is exact 5-tuple match. If yes, we return that res_index and bucket_id without starting the timer
         * If no,
         *      We just increment i and bckt_visit_cnt and continue
         */
        i = 0;
        bkt_visit_cnt = i;
        while((i < ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION) &&
                (bkt_visit_cnt != ipReassemblyTbl[res_index].count_hashid) )
            //in ovflow case, count_hashid may be more than collision len
        {
            /* storing first free bucket entry index */
            if(ipReassemblyTbl[res_index].buckets[i].sess_entry_p == NULL && false == slot_found)
            {
                *bucket_index = i;
                slot_found = true;
            }
            else
            {
                if(hash_val == ipReassemblyTbl[res_index].buckets[i].hash_id)
                {
                    adk_ip_dp_reassembly_session_tbl_t *session_entry_p=NULL;
                    /* Make sure that the re-assembly is for the correct packet */
                    session_entry_p = ipReassemblyTbl[res_index].buckets[i].sess_entry_p;

                    if((session_entry_p->sip != iphdr->src_addr)||
                            (session_entry_p->dip != iphdr->dst_addr)||
                            (session_entry_p->ip_id !=  iphdr->packet_id)||
                            (session_entry_p->protocol != iphdr->next_proto_id)||
                            (session_entry_p->vrd_id != ADK_IP_DEFAULT_VRD_ID))
                    {
                        bkt_visit_cnt++;
                        i++;
                        continue;
                    }
                    /* We have reached here means the flow is already being reassembled.
                     * This should never happen.
                     */
                    ADK_IP_DP_LOG_INFO_DEBUG("flow with hashid:0x%x already being reassembled.\n", hash_val);
                    *bucket_index = i;
                    ret = ADK_SUCCESS;
                    return ret;
                }
                bkt_visit_cnt++;
            }
            i++;
        }
    }

    if(false==slot_found && i == ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION)
    {
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmHashCollisionDrops++;
        return  ADK_E_INTERNAL_ERROR;
    }

    /* if bucket_index is 0xff and we reached here means either there was no reassembly going on for this hash-id or there were some contexts at
     * res_index (which didn't match current packet flow) and above loop broke of when bkt_visit_cnt reached cnt_hashid
     */
    if(0xff == *bucket_index)
    {
        *bucket_index = i;
    }

    ret = setup_context_and_start_timer(coreIndex, ph_task_p, hash_val, res_index, *bucket_index, ip_offset, ipsecCb, preFragBufferTimeOutVal);

    return ret;

}

/*!
 * =============================================================
 * adk_ip_dp_reassemble_internal()
 *
 * IP DP function to perform reassembly
 * Return values:
 *   ADK_SUCCESS          : Reassembly succesful
 *   ADK_E_TASK_PARKED  : Task is parked for reasembly
 *   ADK_E_INTERNAL_ERROR : Internal error due to reassembly lookup failure
 *
 * Note: ip_term_stats_id should be validated before using. It will be set to invalid value (ADK_DP_MAX_IP_STATS) in
 * certain cases.
 * =============================================================
 */
adk_status_t adk_ip_dp_reassemble_internal(uint16_t coreIndex,
                        adk_pktio_task_t **ph_task_p,
                        uint8_t vrd_id, uint16_t ip_term_stats_id,
                        uint32_t hash_val)
{
    struct ipv4_hdr *iphdr;
    adk_pktio_task_t *ph_task = *ph_task_p;
    uint16_t res_index, i=0, frag_offset, frag_info;
    uint32_t bkt_visit_cnt;
    uint8_t collisionIndex=0, free_bkt_index = 0;
    uint16_t total_len =0;
    bool  updated_free_bkt_index = false;
    bool frag_handled = false;
    struct rte_mbuf *last_task, *prev_task;
    adk_ip_dp_reasm_error_t stbl_result;
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p=NULL;
    void *reasm_p;
    uint64_t reasm_col_row_index = 0x0;
#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
    adk_timer_info_t timer_info = {ADK_REASSEMBLY_TIMEOUT_MILLISECONDS};
    int ret;
#endif
#endif

    ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
            ADK_MD_GET_IP_OFF(ph_task));
    frag_info = rte_be_to_cpu_16(iphdr->fragment_offset);

    ADK_IP_DP_LOG_INFO_DEBUG("Looking up from IP Reassembly table:%p with IP Id:0x%x \n",
           ipReassemblyTbl, iphdr->packet_id);
    ADK_IP_DP_LOG_INFO_DEBUG("Sip:%x Dip:%x Proto:%x\n",iphdr->src_addr,
            iphdr->dst_addr,iphdr->next_proto_id);

    res_index = hash_val & IP_DP_REASS_ID_MASK;

    adk_ip_dp_acquire_reassembly_lock_non_atomic(ph_task, res_index);
    if(ipReassemblyTbl[res_index].count_hashid == 0)
    {
        collisionIndex = 0x0;
        if(rte_mempool_get(ipReassemblySessionTbl, &reasm_p) < 0)
        {
            ADK_IP_DP_LOG_ERROR("Failed to alloc reassembly session entry");
            adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmSessionAllocFailDrops++;
            return ADK_E_INTERNAL_ERROR;
        }

        session_entry_p = (adk_ip_dp_reassembly_session_tbl_t *)reasm_p;
        session_entry_p->vrd_id = vrd_id;

#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
        /* timer create */
        adk_ip_dp_get_staggered_timeout_value(&timer_info);
        ADK_IP_DP_LOG_INFO_DEBUG("Creating reassembly timer for row_index: %d, col_index:%d, timeout (ms)=%d\n",
                                res_index, collisionIndex, timer_info.interval_milli_sec);
        session_entry_p->flow_id = ADK_METADATA_PARAM(ph_task, lock_id);
        session_entry_p->queue_id = ADK_METADATA_PARAM(ph_task, event_queue_id);
        session_entry_p->ip_term_stats_id = ip_term_stats_id;

        reasm_col_row_index = (uint64_t)((uint64_t) res_index |
                (((uint64_t)collisionIndex) << 16) |
                (((uint64_t)(session_entry_p->version)) << 24));

        ret = adk_timer_create(coreIndex, timer_info, reassembly_timer_cb_fn,
                  (void *)reasm_col_row_index,
                            &(session_entry_p->timer));
        if(unlikely(0 == ret))
        {
            ADK_IP_DP_LOG_ERROR("Failed to create timer for IP reassembly");
            adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmTimerCreateFailDrops++;
            return ADK_E_INTERNAL_ERROR;
        }
#endif
#endif
        ipReassemblyTbl[res_index].buckets[0].hash_id = hash_val;
        ipReassemblyTbl[res_index].buckets[0].ipsecCb = NULL;
        ipReassemblyTbl[res_index].buckets[0].sess_entry_p = session_entry_p;

        ipReassemblyTbl[res_index].count_hashid += 1;
        /*Initialize sessionTbl & perform the hole algo*/
        stbl_result = adk_ip_dp_reasm_sessionTbl_update(
                                session_entry_p, *ph_task_p);
        frag_handled = true;
        __atomic_fetch_add(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);

    }
    else
    {
        i = 0;
        bkt_visit_cnt = i;
        while((i < ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION) &&
              (bkt_visit_cnt != ipReassemblyTbl[res_index].count_hashid) )
             //in ovflow case, count_hashid may be more than collision len
        {
            /* storing first free bucket entry index */
            if(ipReassemblyTbl[res_index].buckets[i].sess_entry_p ==
                    NULL)
            {
                if(updated_free_bkt_index == false)
                {
                    free_bkt_index = i;
                    updated_free_bkt_index = true;
                }
            }
            else
            {
                if(hash_val == ipReassemblyTbl[res_index].buckets[i].hash_id)
                {
                    /* Make sure that the re-assembly is for the correct packet */
                    session_entry_p =
                        ipReassemblyTbl[res_index].buckets[i].sess_entry_p;

                    if((session_entry_p->sip != iphdr->src_addr)||
                       (session_entry_p->dip != iphdr->dst_addr)||
                       (session_entry_p->ip_id !=  iphdr->packet_id)||
                       (session_entry_p->protocol != iphdr->next_proto_id)||
                       (session_entry_p->vrd_id != vrd_id))
                    {
                        bkt_visit_cnt++;
                        i++;
                        continue;
                    }

                    collisionIndex = i;
                    stbl_result = adk_ip_dp_reasm_sessionTbl_update(
                            session_entry_p, *ph_task_p);
                        frag_handled = true;
                        break;
                }
                bkt_visit_cnt++;
            }
            i++;
        }
    }
    /* current flow not found in collision list, add new bucket entry */
    if(frag_handled == false)
    {
        if(updated_free_bkt_index == false)
        {
            if(i == ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION)
            {
                ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmHashCollisionDrops++;
                ADK_IP_DP_LOG_INFO_DEBUG("Reached Max of collision list.. Dropping task\n");
                adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
                return ADK_E_INTERNAL_ERROR;
            }
            free_bkt_index = i;
        }

        if(rte_mempool_get(ipReassemblySessionTbl, &reasm_p) < 0)
        {
            ADK_IP_DP_LOG_ERROR("Failed to alloc reassembly session entry");
            adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmSessionAllocFailDrops++;
            return ADK_E_INTERNAL_ERROR;
        }
        session_entry_p = (adk_ip_dp_reassembly_session_tbl_t *)reasm_p;
        session_entry_p->vrd_id = vrd_id;

#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
        /* timer create */
        adk_ip_dp_get_staggered_timeout_value(&timer_info);
        ADK_IP_DP_LOG_INFO_DEBUG("Creating reassembly timer for row_index: %d, col_index:%d, timeout (ms)=%d\n",
                               res_index, free_bkt_index, timer_info.interval_milli_sec);
        session_entry_p->flow_id = ADK_METADATA_PARAM(ph_task, lock_id);
        session_entry_p->queue_id = ADK_METADATA_PARAM(ph_task, event_queue_id);
        session_entry_p->ip_term_stats_id = ip_term_stats_id;

        reasm_col_row_index = (uint64_t)((uint64_t)res_index |
                (((uint64_t)free_bkt_index) << 16) |
                (((uint64_t)(session_entry_p->version)) << 24));

        ret = adk_timer_create(coreIndex, timer_info, reassembly_timer_cb_fn,
                  (void *)reasm_col_row_index,
                            &(session_entry_p->timer));
        if(unlikely(0 == ret))
        {
            ADK_IP_DP_LOG_ERROR("Failed to create timer for IP reassembly");
            adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
            ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmTimerCreateFailDrops++;
            return ADK_E_INTERNAL_ERROR;
        }
#endif
#endif
        ipReassemblyTbl[res_index].buckets[free_bkt_index].hash_id = hash_val;
        ipReassemblyTbl[res_index].buckets[free_bkt_index].ipsecCb = NULL;
        ipReassemblyTbl[res_index].buckets[free_bkt_index].sess_entry_p = session_entry_p;

        ipReassemblyTbl[res_index].count_hashid += 1;
        stbl_result = adk_ip_dp_reasm_sessionTbl_update(session_entry_p,
                                                        *ph_task_p);

        __atomic_fetch_add(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);
    }

    switch(stbl_result)
    {
#if 0
    case ADK_IP_DP_REASM_PKT_MISMATCH:
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].defragIdMismatchErrors++;
            adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
        return ADK_E_INTERNAL_ERROR;
#endif

    case ADK_IP_DP_REASM_HOLES_EXCEEDED:
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmHolesExceededDrops++;
        adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
        return ADK_E_INTERNAL_ERROR;

    case ADK_IP_DP_REASM_DUP_BYTES:
        ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmDuplicateDrops++;
        adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
        return ADK_E_INTERNAL_ERROR;

    case ADK_IP_DP_REASM_ENQ_DONE:
    case ADK_IP_DP_REASM_ENQ_COPY:
        session_entry_p->numEnqueues++;

        frag_offset = (frag_info & 0x1FFF) *8;
        if(session_entry_p->numEnqueues == 1)
        {
            session_entry_p->enqd_tasks[0].frag_offset = frag_offset;
            session_entry_p->enqd_tasks[0].frag_task = (adk_pktio_task_t *)*ph_task_p;
            ADK_IP_DP_LOG_INFO_DEBUG(" Packet enqueued in reassembly park list at position 0\n");
            ADK_IP_DP_LOG_INFO_DEBUG(" Total number of parked tasks:%d\n",
                    session_entry_p->numEnqueues);
        }
        else
        {
            if(session_entry_p->numEnqueues > ADK_IP_DP_REASS_MAX_FRAGS)
            {
                ADK_IP_DP_LOG_INFO_DEBUG(" reached Max frag handling limit for Reassembly\n");
                ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].ipInReasmFragBufferFullDrops++;
                /* current frag task will be freed by calling function ,
                 * so reduce enqueue count*/
                session_entry_p->numEnqueues--;
                adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
                return ADK_E_INTERNAL_ERROR;
            }

            i = session_entry_p->numEnqueues-1;
            while(i > 0 && frag_offset < session_entry_p->enqd_tasks[i-1].frag_offset)
            {
                session_entry_p->enqd_tasks[i].frag_offset =
                    session_entry_p->enqd_tasks[i-1].frag_offset;
                session_entry_p->enqd_tasks[i].frag_task =
                    session_entry_p->enqd_tasks[i-1].frag_task;
                i = i-1;
            }
            session_entry_p->enqd_tasks[i].frag_offset = frag_offset;
            session_entry_p->enqd_tasks[i].frag_task = (adk_pktio_task_t *)*ph_task_p;
            ADK_IP_DP_LOG_INFO_DEBUG(" Packet enqueued in reassembly park list at position %d\n", i);
            ADK_IP_DP_LOG_INFO_DEBUG(" Total number of parked tasks:%d\n",
                    session_entry_p->numEnqueues);
        }
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmEnqueuedPkts, 1);
        break;

    default:
        ADK_IP_DP_LOG_INFO_DEBUG(" Invalid IP reassembly action, Internal Error\n");
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[coreIndex].UipReasmInvalidReasmAction, 1);
        adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
        return ADK_E_INTERNAL_ERROR;
    }

    if(stbl_result == ADK_IP_DP_REASM_ENQ_DONE)
    {
        /* task is enqueued to parked list*/
        adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
        ADK_MEASURE_CYCLE_DIFF(&(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE]), 1);
        ADK_MEASURE_CYCLE_CHECK_EXIT(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE], coreIndex, ADK_TERMINATION_STAGE);
        return ADK_E_TASK_PARKED;
    }
    else
    {
        /* (ADK_IP_DP_REASM_ENQ_COPY == stbl_result)
         * reassembly is done ,chain all parked tasks
         * pick up the first task from parked list*/
        ph_task = (adk_pktio_task_t *)session_entry_p->enqd_tasks[0].frag_task;
        total_len = session_entry_p->defragLength + IP_HDR_LEN;

        /* Chain all parked tasks*/
        i = session_entry_p->numEnqueues - 1;
        while(i > 0)
        {
            last_task = (struct rte_mbuf *)(session_entry_p->enqd_tasks[i].frag_task);
            prev_task = (struct rte_mbuf *)(session_entry_p->enqd_tasks[i-1].frag_task);
            rte_pktmbuf_adj(last_task, (ADK_MD_GET_IP_OFF(last_task)+IP_HDR_LEN));
            rte_pktmbuf_chain(prev_task, last_task);
            session_entry_p->enqd_tasks[i].frag_task = NULL;
            i--;
        }
        session_entry_p->enqd_tasks[i].frag_task = NULL;

        /* Clean the session entry*/
        session_entry_p->sip = 0;
        session_entry_p->dip = 0;
        session_entry_p->ip_id = 0;
        session_entry_p->protocol = 0;
        session_entry_p->numEnqueues = 0;
        session_entry_p->version++;
        ipReassemblyTbl[res_index].buckets[collisionIndex].sess_entry_p =
                   NULL;
        ipReassemblyTbl[res_index].buckets[collisionIndex].hash_id=0;
        ipReassemblyTbl[res_index].count_hashid -= 1;

        if(ipReassemblyTbl[res_index].buckets[collisionIndex].ipsecCb)
        {
            ipReassemblyTbl[res_index].buckets[collisionIndex].ipsecCb(res_index, collisionIndex);
            ipReassemblyTbl[res_index].buckets[collisionIndex].ipsecCb = NULL;
        }

        __atomic_fetch_sub(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);
        rte_mempool_put(ipReassemblySessionTbl, (void *)session_entry_p);
        adk_ip_dp_release_reassembly_lock_non_atomic(ph_task, res_index);
#ifdef ADK_DP_MAINTENANCE
#ifdef ADK_TIMER
        adk_timer_delete(coreIndex, session_entry_p->timer);
#endif
#else

#endif

        /*Update the ip header*/
        ADK_PDU_OFFSET_PTR_GET(ph_task, iphdr, struct ipv4_hdr *,
                ADK_MD_GET_IP_OFF(ph_task));
        iphdr->total_length = rte_cpu_to_be_16(total_len);
        iphdr->fragment_offset = 0x0;
        /* update IP Hdr checksum*/
        iphdr->hdr_checksum = 0;
        iphdr->hdr_checksum = ip_hdr_csum((void *)iphdr, 5);
        ((adk_ip_dp_lli_internal_mdata_t *)
         (&(ADK_MD_GET_IMC_PARAM_START(ph_task))))->is_reassembled = 1;

        *ph_task_p = (void *)ph_task;
        ADK_IP_DP_LOG_INFO_DEBUG("Reassemly completed\n");
#ifdef ADK_DEBUG
        /*ADK_IP_DP_LOG_INFO_DEBUG("Reassembled packet::");
        rte_pktmbuf_dump(stdout, (struct rte_mbuf *)ph_task, ADK_TASK_PDU_SIZE(ph_task));
        ADK_IP_DP_LOG_INFO_DEBUG("------\n");*/
#endif
        return ADK_SUCCESS;
    }
}

adk_ip_dp_reasm_error_t adk_ip_dp_reasm_sessionTbl_update(
    adk_ip_dp_reassembly_session_tbl_t *session_entry_p,
                                    adk_pktio_task_t *frag_task)
{
    int16_t i;
    uint16_t inFragLen, inFragEndOffset, inFragOffset, fragLen, eth_pad_bytes,
             packet_len, strip_bytes, holeStartOffset, holeEndOffset,
              frag_info;
    uint8_t *pdu;
    holeInfo_t *hole_p, *last_hole_p;
    adk_ip_dp_reasm_error_t error_code;
    struct ipv4_hdr *iphdr;

    ADK_PDU_OFFSET_PTR_GET(frag_task, iphdr, struct ipv4_hdr *, ADK_MD_GET_IP_OFF(frag_task));

    if(session_entry_p->numEnqueues == 0)
    {
        session_entry_p->sip = iphdr->src_addr;
        session_entry_p->dip = iphdr->dst_addr;
        session_entry_p->ip_id = iphdr->packet_id;
        session_entry_p->protocol = iphdr->next_proto_id;

        session_entry_p->num_holes = 1;
        session_entry_p->numEnqueues = 0;
        session_entry_p->assembledBytes = 0;
        session_entry_p->defragLength = 0;
        session_entry_p->holes[0].holeStartOffset = 0;
        session_entry_p->holes[0].holeEndOffset = 0xFFFF;
        hole_p = (holeInfo_t *)session_entry_p->holes;
    }
    else
    {
        hole_p = (holeInfo_t *)session_entry_p->holes;
    }

    fragLen = rte_be_to_cpu_16(iphdr->total_length);
    frag_info = rte_be_to_cpu_16(iphdr->fragment_offset);
    inFragOffset = (frag_info & 0x1FFF) * 8;

    inFragLen = fragLen - IP_HDR_LEN;
    inFragEndOffset = inFragOffset + inFragLen - 1;

    packet_len = inFragLen;
    strip_bytes = 0;
    error_code = ADK_IP_DP_REASM_ENQ_DONE;

    if(((frag_info & IPV4_HDR_MF_MASK) == 0) && (inFragOffset != 0))
    {
        /* Here is the last fragment of the packet. This will give us the
         * total length of the IP payload */
        session_entry_p->defragLength = inFragEndOffset + 1;
        ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler:Last fragment received \n");

        last_hole_p = (holeInfo_t *)(session_entry_p->holes +
                            (session_entry_p->num_holes-1));

        /* Now we know the true length of last hole. Update it */
        if(0xFFFF == last_hole_p->holeEndOffset)
            last_hole_p->holeEndOffset = inFragEndOffset;

        /*Strip the eth padding bytes if present*/
        eth_pad_bytes = ADK_TASK_PDU_SIZE(frag_task) - (ADK_MD_GET_IP_OFF(frag_task) + fragLen);
        if( eth_pad_bytes )
            ADK_PDU_TAIL_PULL(frag_task, eth_pad_bytes);
    }

   /* DEBUGGING */
   ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: received frag - start=%d, end=%d\n", inFragOffset, inFragEndOffset);
   ADK_IP_DP_LOG_INFO_DEBUG("- - - - - - - - -\n");
   for(i=0;i<session_entry_p->num_holes;i++)
   {
      ADK_IP_DP_LOG_INFO_DEBUG("hole%d: start:%d, end=%d\n", i,
               session_entry_p->holes[i].holeStartOffset, session_entry_p->holes[i].holeEndOffset);
   }
   ADK_IP_DP_LOG_INFO_DEBUG("- - - - - - - - -\n");
   /* */

    /* Backward loop to be able to modify numHoles from within the loop */
    for(i = session_entry_p->num_holes -1; i >=0; i--)
    {
        holeStartOffset = hole_p->holeStartOffset;
        holeEndOffset = hole_p->holeEndOffset;

        /* First check the most common case where fragment shrinks the hole
         * from beginning */
        if ((inFragOffset == holeStartOffset) &&
            (inFragEndOffset < holeEndOffset))
        {
            hole_p->holeStartOffset = inFragEndOffset + 1;
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment shrings the hole%d from beginning\n", i);
            break;
        }

        /* Next most common case is where the fragment splits the existing
         * hole into two */
        if ((inFragOffset > holeStartOffset) &&
            (inFragEndOffset < holeEndOffset))
        {
            /* It is not often that we exceed max threshold supported */
            if (session_entry_p->num_holes <
                    ADK_IP_MAX_HOLES_SUPPORTED)
            {
                /* Create new hole and store it as the last hole in the list */
                last_hole_p = (holeInfo_t *)(session_entry_p->holes +
                    session_entry_p->num_holes);
                last_hole_p->holeStartOffset = inFragEndOffset + 1;
                last_hole_p->holeEndOffset = hole_p->holeEndOffset;
                session_entry_p->num_holes++;
                hole_p->holeEndOffset = inFragOffset - 1;
               ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment splits the hole%d in 2\n", i);
                break;
            }else {
                /* Creation of new hole will result in exceeding the max
                 * number of holes supported. So, ignore this frame by not
                 * enqueing it at all */
                error_code = ADK_IP_DP_REASM_HOLES_EXCEEDED;
                ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: num holes exceeds max (%d)\n", ADK_IP_MAX_HOLES_SUPPORTED);
                break;
            }
        }

        /* Next most common case is where the fragment perfectly fills up
         * the hole */
        if ((inFragOffset == holeStartOffset) &&
            (inFragEndOffset == holeEndOffset))
        {
            /* Remove the hole by moving the last hole in current
             * position. If this itself is the last hole then it may
             * overwrite itself. */
            session_entry_p->num_holes--;
            last_hole_p = (holeInfo_t *)(session_entry_p->holes +
                session_entry_p->num_holes);
            hole_p->holeStartOffset = last_hole_p->holeStartOffset;
            hole_p->holeEndOffset = last_hole_p->holeEndOffset;
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment perfectly fills the hole%d\n", i);
            break;
        }

        /* Next most common case is where the fragment shrinks the hole from
         * the end */
        if ((inFragOffset > holeStartOffset) &&
            (inFragEndOffset == holeEndOffset))
        {
            hole_p->holeEndOffset = inFragOffset - 1;
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment shrinks the hole%d from end\n", i);
            break;
        }

        /* Finally, we consider all the overlapping cases where not entire
         * fragment gets enqueued. Instead, some header and/or trailer part
         * is cut off */
        if ((inFragOffset < holeStartOffset) &&
            (inFragEndOffset >= holeStartOffset) &&
            (inFragEndOffset < holeEndOffset))
        {
            strip_bytes += (holeStartOffset - inFragOffset);
            packet_len -= (holeStartOffset - inFragOffset);
            hole_p->holeStartOffset = inFragEndOffset + 1;
            ADK_PDU_HEAD_PULL(frag_task, pdu, (uint8_t *), strip_bytes);
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment partially fills the hole%d from beginning. fragment stripped from start\n", i);
            break;
        }

        /* Check for only trailer overlap */
        if ((inFragEndOffset > holeEndOffset) &&
            (inFragOffset <= holeEndOffset) &&
            (inFragOffset > holeStartOffset))
        {
            packet_len -= (inFragEndOffset - holeEndOffset);
            ADK_PDU_TAIL_PULL(frag_task, (inFragEndOffset - holeEndOffset));
            hole_p->holeEndOffset = inFragOffset - 1;
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment partially fills the hole%d from end. fragment stripped from end\n", i);
            break;
        }

        /* If both ends are overlapping then remove the hole */
        if ((inFragOffset <= holeStartOffset) &&
            (inFragEndOffset >= holeEndOffset))
        {
            strip_bytes += (holeStartOffset - inFragOffset);
            packet_len = (holeEndOffset - holeStartOffset + 1);

            ADK_PDU_HEAD_PULL(frag_task, pdu, (uint8_t *), strip_bytes);
            ADK_PDU_TAIL_PULL(frag_task, (inFragEndOffset - holeEndOffset));

            session_entry_p->num_holes--;
            last_hole_p = (holeInfo_t *)(session_entry_p->holes +
                session_entry_p->num_holes);

            hole_p->holeStartOffset = last_hole_p->holeStartOffset;
            hole_p->holeEndOffset = last_hole_p->holeEndOffset;
            ADK_IP_DP_LOG_INFO_DEBUG("IP: Reasembler: fragment overlaps the hole%d. hole is removed and fragment stripped from start & end\n", i);
            break;
        }

        /* Packet does not lie in the current hole. Nor does it partially
         * overlap the current hole. Continue with next hole */
        hole_p++;
    }

    /* Check if any error occured in above loop */
    if (error_code != ADK_IP_DP_REASM_ENQ_DONE)
    {
        return error_code;
    }

    /* If packet is not found in any hole, then it must be completely
     * overlapping packet. Do not enqueue this packet */
    if (i == -1)
    {
        return ADK_IP_DP_REASM_DUP_BYTES;
    }

    /* If here, then enque the fragment. Also, accumulate the number of bytes
     * assembled so far and check if re-assembly is complete */
    session_entry_p->assembledBytes += packet_len;

    if ((session_entry_p->defragLength != 0) &&
        (session_entry_p->assembledBytes >=
         session_entry_p->defragLength))
    {
        /* If we have got the last fragment and all the bytes needed for
         * complete re-assembly, get the re-assembled packet out */
        /* With the re-assembly complete, reset the state for re-use */
        return ADK_IP_DP_REASM_ENQ_COPY;
    }
    else
    {
        return ADK_IP_DP_REASM_ENQ_DONE;
    }
}

void adk_ip_dp_reassemblytbl_key_create(uint16_t ip_id,
                                        uint8_t protocol,
                                        uint32_t src_addr,
                                        uint32_t dest_addr,
                                        uint8_t *key)
{
    ADK_MEMCPY(key,(uint8_t *)&ip_id, sizeof(uint16_t));
    ADK_MEMCPY(key+2, (uint8_t *)&protocol, sizeof(uint8_t));
    ADK_MEMCPY(key+3, (uint8_t *)&src_addr, sizeof(uint32_t));
    ADK_MEMCPY(key+7, (uint8_t *)&dest_addr, sizeof(uint32_t));
}

/*!
 * =============================================================
 * adk_ip_dp_fragment()
 *
 * IP DP function to process fragmentation
 * =============================================================
 */
adk_ip_dp_frag_error_t adk_ip_dp_fragment(adk_pktio_task_t **task_out_p,
                      uint32_t mtu, uint8_t *num_frags, uint16_t *total_payload,
                      uint8_t genL4PartialCksum)
{
#ifdef DPDK_IP_FRAG
    struct rte_mbuf *task_frags[ADK_IP_MAX_FRAG_SUPPORTED];
    int num_frag,i;
    uint16_t port;
    struct ipv4_hdr *frag_hdr;
    uint8_t *pdu;
#ifdef ADK_TX_CKSUM_OL
    uint32_t udp_cksum = 0;
    struct udp_hdr *udpHdr=NULL;
#endif

    adk_pktio_task_t *parent_chained_task = (adk_pktio_task_t *)ADK_MD_GET_NEXT_FRAG((*task_out_p));

    /* ethernet header strip if present*/
    ADK_PDU_HEAD_PULL((*task_out_p),pdu, (uint8_t *),ADK_METADATA_PARAM((*task_out_p), ip_offset));

    *total_payload  = ADK_TASK_PDU_SIZE(*task_out_p) - ADK_METADATA_PARAM((*task_out_p), ip_offset)
                            - IP_HDR_LEN;

    /* make sure frag length is multiple of 8 */
    mtu = ((mtu- IP_HDR_LEN) & 0xFFF8) + IP_HDR_LEN;

    num_frag = rte_ipv4_fragment_packet(
            (struct rte_mbuf *)(*task_out_p),task_frags,
                (uint16_t)ADK_IP_MAX_FRAG_SUPPORTED,
                (uint16_t)mtu, (*task_out_p)->pool,
                ip_dp_tbl_info_p->packet_indirect_pool);

    if (unlikely (num_frag < 0))
    {
        switch(-num_frag)
        {
            case ENOMEM:
                return ADK_IP_DP_FRAG_MEM_ALLOC_FAILED;
                break;

            case ENOTSUP:
                ADK_IP_DP_LOG_INFO(
                "Fragmentation can't be done: DF bit set");
                return ADK_IP_DP_FRAG_DFSET;
                break;

            case EINVAL:
                ADK_IP_DP_LOG_INFO(
                "Fragmentation can't be done: Frags_needed > MAX supported");
                return ADK_IP_DP_FRAG_LIMIT_EXCEED;
                break;
        }
    }

    port = (*task_out_p)->port;

    for(i=0;i < num_frag; i++)
    {
        task_frags[i]->ol_flags = 0;
        task_frags[i]->port = port;
        if(i == (num_frag -1))
        {
            /*Retain the task type of the original pkt for the last fragment*/
            adk_pkt_metadata_init(task_frags[i],
                              ADK_MD_GET_EXCP_DOMAIN(*task_out_p),
                              ADK_METADATA_PARAM(*task_out_p,func_idx),
                              ADK_TASK_TYPE_FWD);
            ADK_MD_GET_NEXT_FRAG(task_frags[i]) = (struct rte_mbuf *)parent_chained_task;
        }
        else
        {
            adk_pkt_metadata_init(task_frags[i],
                              ADK_MD_GET_EXCP_DOMAIN(*task_out_p),
                              ADK_METADATA_PARAM(*task_out_p,func_idx),
                              ADK_TASK_TYPE_NEW);
            ADK_MD_GET_NEXT_FRAG(task_frags[i]) = task_frags[i+1];
        }
        ADK_METADATA_PARAM(task_frags[i], proc_stage_idx) =
            ADK_METADATA_PARAM(*task_out_p, proc_stage_idx);
        ADK_METADATA_PARAM(task_frags[i], input_intf_id) =
            ADK_METADATA_PARAM(*task_out_p, input_intf_id);
        frag_hdr = rte_pktmbuf_mtod(task_frags[i], struct ipv4_hdr *);
#ifdef ADK_TX_CKSUM_OL
        task_frags[i]->ol_flags |= (PKT_TX_IPV4 | PKT_TX_IP_CKSUM);
        task_frags[i]->l3_len = IP_HDR_LEN;
        task_frags[i]->l2_len = 0; /*will be updated by Eth/IPsec*/
        /*calc UDP checksum*/
        if(i != 0)
        {
            struct rte_mbuf *tmp = task_frags[i]->next; /*get first indirect buffer pointer*/
            while(tmp)
            {
               uint8_t *data = rte_pktmbuf_mtod(tmp, uint8_t *);
               udp_cksum += adk_raw_cksum((void *)(data), rte_pktmbuf_data_len(tmp));
               tmp = tmp->next;
            }
        }
        else
        {
            /* TODO fix the L4 OL, do this based on original pkt OL flag
             * Set l4_len*/
            /*UDP cksum offload for first fragment*/
            task_frags[i]->ol_flags |= PKT_TX_UDP_CKSUM;
            uint16_t tmp_16;
            udpHdr = rte_pktmbuf_mtod(task_frags[i]->next,  struct udp_hdr *);
            tmp_16 = adk_psd_hdr_cksum((struct ipv4_hdr *)frag_hdr,
                          udpHdr->dgram_len);
            udp_cksum += tmp_16;
        }
#else
        frag_hdr->hdr_checksum = ip_hdr_csum((void *)frag_hdr, ((frag_hdr->version_ihl) & 0x0f));
#endif
    }
#ifdef ADK_TX_CKSUM_OL
    udpHdr->dgram_cksum = adk_raw_cksum_reduce(udp_cksum);
#endif
    rte_pktmbuf_free((struct rte_mbuf *)(*task_out_p));
    *task_out_p = task_frags[0];
    *num_frags = num_frag;

#else
    INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].UipIpsecFragsRequested, 1);
    uint16_t num_frag = 1, *tmp_16_p, tmp_16, copied, i, offset, length;
    uint16_t pdu_length, frag_length, to_copy, frags_needed, src_offset;
    uint16_t remaining, tail_room, new_frag_length, req_room, push_length;
    uint16_t in_frag_info, copy_length;
    const uint8_t *temp;
    uint8_t *pdu, *pdu_frag;
    uint32_t udp_cksum = 0;
    bool last_fragment = FALSE;
    adk_pktio_task_t *current, *prev = NULL, *new_seg;
    adk_pktio_task_t *free_mbuf_p,*free_next_p;
    adk_ip_dp_frag_error_t ret = ADK_IP_DP_FRAG_OK;
    adk_pktio_task_t *parent_chained_task = (adk_pktio_task_t *)ADK_MD_GET_NEXT_FRAG((*task_out_p));
    adk_metadata_t  *adk_md_initial_p= ADK_METADATA_PTR(*task_out_p);

    /* make sure frag length is multiple of 8 */
    mtu = ((mtu- IP_HDR_LEN) & 0xFFF8) + IP_HDR_LEN;
    pdu_length = ADK_TASK_PDU_SIZE(*task_out_p) - ADK_MD_GET_IP_OFF((*task_out_p)) ;
    remaining = pdu_length - mtu;
    frag_length = mtu - IP_HDR_LEN;
    to_copy = frag_length;
    copied = frag_length;
    prev = *task_out_p;
    new_frag_length = mtu;

    frags_needed = (pdu_length - IP_HDR_LEN)/frag_length;
    if((pdu_length - IP_HDR_LEN)%frag_length)
    {
        frags_needed++;
    }

    ADK_PDU_OFFSET_PTR_GET((*task_out_p), pdu, uint8_t *,
            ADK_MD_GET_IP_OFF((*task_out_p)));
    src_offset = ADK_MD_GET_IP_OFF((*task_out_p)) + mtu;

    /* If Don't Fragment flag is set */
    tmp_16_p = (uint16_t *)(void *)(pdu + 6);
    in_frag_info = rte_be_to_cpu_16(*tmp_16_p);
    if( unlikely(in_frag_info & (IPV4_HDR_DF_MASK)))
    {
        ADK_IP_DP_LOG_INFO("Fragmentation can't be done: DF bit set");
        return ADK_IP_DP_FRAG_DFSET;
    }

    if(unlikely(frags_needed > ADK_IP_MAX_FRAG_SUPPORTED))
    {
        ADK_IP_DP_LOG_INFO(
        "Fragmentation can't be done: Frags_needed > MAX supported");
        return ADK_IP_DP_FRAG_LIMIT_EXCEED;
    }

    if(remaining <= frag_length)
    {
        to_copy = remaining;
        last_fragment = TRUE;
        new_frag_length = to_copy + IP_HDR_LEN;
    }


    while(num_frag <= frags_needed)
    {
        current = (adk_pktio_task_t *)rte_pktmbuf_alloc(prev->pool);
        if(current == NULL)
        {
            ret = ADK_IP_DP_FRAG_MEM_ALLOC_FAILED;
            goto _adk_ip_dp_fragment_exit;
        }

        ADK_TASK_PDU_SIZE(current) = 0;
        adk_pkt_metadata_init(current,
                              ADK_MD_GET_EXCP_DOMAIN(prev),
                              ADK_METADATA_PARAM(prev,func_idx),
                              ADK_TASK_TYPE_NEW);
        ADK_METADATA_PARAM(current, proc_stage_idx) =
                              ADK_METADATA_PARAM(prev, proc_stage_idx);
        current->port = prev->port;
        ADK_METADATA_PARAM(current, input_intf_id) =
                              ADK_METADATA_PARAM(prev, input_intf_id);
        ADK_METADATA_PARAM(current, dev_tx_queue_id) = 
            ADK_METADATA_PARAM(prev, dev_tx_queue_id);
        ADK_METADATA_PARAM(current, pktColor) = ADK_METADATA_PARAM(prev, pktColor);
        ADK_PDU_PTR_GET(current, pdu_frag, uint8_t *);
        ADK_IP_DP_LOG_INFO_DEBUG("\nTask buffer allocated, task:%p, pdu:%px, pduSize:%d\n",
           current,
           pdu_frag,
           ADK_TASK_PDU_SIZE(current));

        tail_room = rte_pktmbuf_tailroom(current);

        if(new_frag_length <= tail_room)
        {
            ADK_PDU_TAIL_PUSH((current), (new_frag_length));
        }
        else
        {
            ADK_PDU_TAIL_PUSH((current),(tail_room));
            req_room = new_frag_length - tail_room;
            while(req_room)
            {
                new_seg= (adk_pktio_task_t *)rte_pktmbuf_alloc(prev->pool);
                if(new_seg == NULL)
                {
                    ret = ADK_IP_DP_FRAG_MEM_ALLOC_FAILED;
                    goto _adk_ip_dp_fragment_exit;
                }
                push_length = (req_room >= tail_room)?tail_room:req_room;
                ADK_PDU_TAIL_PUSH((new_seg),(push_length));
                rte_pktmbuf_chain(current, new_seg);
                req_room -= push_length;
            }
        }

        /*Copy ip header*/
        ADK_MEMCPY(pdu_frag, pdu , IP_HDR_LEN);
        tmp_16_p = (uint16_t *)(void *)(pdu_frag + 2); /*total len*/
        *tmp_16_p = rte_cpu_to_be_16(ADK_TASK_PDU_SIZE((current)));
        tmp_16_p = (uint16_t *)(void *)(pdu_frag + 6); /*frag info*/
        tmp_16 = in_frag_info + (copied >> IPV4_HDR_FO_SHIFT);
        if(last_fragment == FALSE)
        {
            tmp_16 |= (IPV4_HDR_MF_MASK);
        }
        *tmp_16_p = rte_cpu_to_be_16(tmp_16);
        tmp_16_p = (uint16_t *)(void *)(pdu_frag + 10);
        *tmp_16_p = 0;
        current->ol_flags |= (PKT_TX_IPV4 | PKT_TX_IP_CKSUM);
        current->l3_len = IP_HDR_LEN;
        current->l2_len = 0; /*will be updated by Eth/IPsec*/

        copy_length = (to_copy >= (tail_room - IP_HDR_LEN))?(tail_room - IP_HDR_LEN):to_copy;
        temp = rte_pktmbuf_read((*task_out_p), src_offset, copy_length,
                (void *)(pdu_frag+IP_HDR_LEN));
        if(temp != (pdu_frag+IP_HDR_LEN))
        {
            /*Copy ip payload*/
            ADK_MEMCPY((pdu_frag + IP_HDR_LEN),
                    (temp), copy_length);
        }

        if(genL4PartialCksum)
            udp_cksum += adk_raw_cksum((void *)(pdu_frag + IP_HDR_LEN), copy_length);

        /*copy remaining payload to next segments*/
        offset = src_offset + copy_length;
        length = to_copy - copy_length;
        new_seg = current->next;

        while(length)
        {
            ADK_PDU_PTR_GET(new_seg, pdu_frag, uint8_t *);
            copy_length = (length >= tail_room)?tail_room:length;
            temp = rte_pktmbuf_read((*task_out_p), offset, copy_length,
                    (void *)(pdu_frag));
            if(temp != (pdu_frag))
            {
                /*Copy ip payload*/
                ADK_MEMCPY((pdu_frag ),
                        (temp), (copy_length));
            }

            if(genL4PartialCksum)
                udp_cksum += adk_raw_cksum((void *)pdu_frag, copy_length);

            length -= copy_length;
            offset += copy_length;
            new_seg = new_seg->next;
        }

        adk_metadata_t * adk_md_frag_p= ADK_METADATA_PTR(current);
        rte_memcpy(adk_md_frag_p, adk_md_initial_p,
                        (sizeof(adk_metadata_t)));
        ADK_MD_SET_IP_OFF(current, 0);
        ADK_METADATA_PARAM(current, outerIpPktLen) = ADK_TASK_PDU_SIZE(current);

        ADK_MD_GET_NEXT_FRAG(prev) = (struct rte_mbuf *)current;
        if(last_fragment == TRUE)
        {
            /* If there is already fragmentation done in ipsec, this will create problem */
            ADK_MD_GET_NEXT_FRAG(current) = (struct rte_mbuf *)parent_chained_task;
            num_frag++;
            *num_frags = num_frag;
            copied += to_copy;
            *total_payload = copied;
            break;
        }
        prev = current;
        src_offset += to_copy;
        copied += to_copy;
        remaining = remaining - to_copy;
        num_frag++;
        if(remaining <= frag_length)
        {
            to_copy = remaining;
            last_fragment = TRUE;
            new_frag_length = to_copy + IP_HDR_LEN;
        }
    }

    /*
     * Configure the 1st fragment task
     */
    if(rte_pktmbuf_is_contiguous(*task_out_p))
    {
        ADK_PDU_TAIL_PULL((*task_out_p), (pdu_length - mtu));
    }
    else
    {
        struct rte_mbuf *m = (*task_out_p);
        uint16_t len = m->data_len, nb_segs=1;
        uint16_t pkt_len= ADK_MD_GET_IP_OFF((*task_out_p)) + mtu;

        m->pkt_len = pkt_len;

        while(m->next && (len < pkt_len))
        {
            m = m->next;
            len += m->data_len;
            nb_segs++;
        }
        if(m->next)
        {
            /*Free segments*/
            struct rte_mbuf *next = m->next;
            struct rte_mbuf *m_next;
            while (next != NULL) {
                m_next = next->next;
                rte_pktmbuf_free_seg(next);
                next = m_next;
            }
            m->next = NULL;
            /*Update Head Fragment segment count*/
            (*task_out_p)->nb_segs = nb_segs;
        }
        m->data_len -=(len-pkt_len);
    }
    tmp_16_p = (uint16_t *)(void *)(pdu + 2); /*total len*/
    *tmp_16_p = rte_cpu_to_be_16(mtu);
    ADK_METADATA_PARAM(*task_out_p, outerIpPktLen) = mtu;
    tmp_16_p = (uint16_t *)(void *)(pdu + 6); /*frag info*/
    tmp_16 = in_frag_info | IPV4_HDR_MF_MASK;
    *tmp_16_p = rte_cpu_to_be_16(tmp_16);
    tmp_16_p = (uint16_t *)(void *)(pdu + 10);
    *tmp_16_p = 0;

    (*task_out_p)->ol_flags |= (PKT_TX_IPV4 | PKT_TX_IP_CKSUM);
    (*task_out_p)->l3_len = IP_HDR_LEN;
    (*task_out_p)->l2_len = 0; /*will be updated by Eth/IPsec*/

    /*UDP cksum offload for first fragment*/
    if(genL4PartialCksum)
    {
        struct udp_hdr *udpHdr=NULL;
        udpHdr = (struct udp_hdr *)(void *)(pdu + IP_HDR_LEN);
        tmp_16 = adk_psd_hdr_cksum((struct ipv4_hdr *)pdu,
                udpHdr->dgram_len);
        udp_cksum += tmp_16;
        udpHdr->dgram_cksum = adk_raw_cksum_reduce(udp_cksum);
    }

_adk_ip_dp_fragment_exit:
    if(ADK_IP_DP_FRAG_MEM_ALLOC_FAILED == ret)
    {
        ADK_IP_DP_LOG_ERROR("Fragmentation mbuf alloc failed");
        ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].ipOutFragMbufAllocFailDrops++;
        /* Free already allocated mbufs to avoid memory leak*/
        free_mbuf_p = ADK_MD_GET_NEXT_FRAG((*task_out_p));
        for(i = 1; i < num_frag; i++)
        {
            free_next_p = ADK_MD_GET_NEXT_FRAG(free_mbuf_p);
            rte_pktmbuf_free(free_mbuf_p);
            free_mbuf_p = free_next_p;
        }
    }
    else
        INC_IP_UTIL_CNTR(ip_dp_tbl_info_p->ipGlobalStatsTable[ADK_LCORE_INDEX].UipIpsecFragsCreated, num_frag);

    return ret;
#endif
    return ADK_IP_DP_FRAG_OK;
}
