/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IP_ACL_H__
#define __ADK_IP_ACL_H__

#include <adk_hash.h>

#define IP_ACL_MAX_RULES_PER_SET    64

#define IP_ACL_MAX_NUM_RULES 256

typedef uint32_t adk_ip_acl_val_t;

typedef struct _adk_ip_acl_val_pvt_info_s
{
    uint8_t action;
    uint8_t enable_counter;
    uint16_t stats_id;
} adk_ip_acl_val_pvt_info_t;

#define ACL_LKUP_MAX_NUM_PKT 8

typedef void adk_ip_acl_tree_t;

typedef struct _adk_ip_acl_rule_s
{
    uint32_t sip_addr;
    uint8_t sip_mask;
    uint32_t dip_addr;
    uint8_t dip_mask;
    uint16_t sport_begin;
    uint16_t sport_end;
    uint16_t dport_begin;
    uint16_t dport_end;
    uint8_t proto_begin;
    uint8_t proto_end;
    uint16_t dscp_begin;
    uint16_t dscp_end;
    uint16_t frag_type_begin;
    uint16_t frag_type_end;
} adk_ip_acl_rule_t;

typedef struct __attribute__((__packed__))  _adk_ip_acl_ptn_s
{
    uint8_t proto;
    uint32_t sip_addr;
    uint32_t dip_addr;
    uint16_t srcPort;
    uint16_t destPort;
    uint16_t dscp;
    uint16_t frag_type;
}adk_ip_acl_ptn_t;

adk_ip_acl_tree_t *adk_ip_acl_create(adk_module_handle_t ipHandle, uint16_t ruleSetId, uint8_t index);

int adk_ip_acl_ruleset_free(adk_module_handle_t ipHandle, adk_ip_acl_tree_t *acl_tree);

int adk_ip_acl_rule_add(adk_module_handle_t ipHandle,
		 	adk_ip_acl_tree_t *acl_tree,
		 	adk_ip_acl_rule_t *acl_rule,
		 	adk_ip_acl_val_t  acl_val,
		 	uint32_t precedence);

int adk_ip_dp_acl_lookup(adk_ip_acl_tree_t *acl_tree,
		      adk_ip_acl_ptn_t  *acl_ptn,
		      adk_ip_acl_val_t  *acl_val);

int adk_ip_acl_ruleset_commit(adk_module_handle_t ipHandle, adk_ip_acl_tree_t *acl_tree);

void adk_ip_acl_ruleset_clear(adk_module_handle_t ipHandle, adk_ip_acl_tree_t *acl_tree);

#endif
