/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file adk_ip_common.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ip  IPv4 Module
 *
 * @{
 *
 * @brief
 *
 * This file contains top level data structures and definitions for
 * the IP module.
 *
 ***********************************************************************/
#ifndef _ADK_IP_COMMON_H_
#define _ADK_IP_COMMON_H_

#include <adk_include.h>
#include <adk_ip_defs.h>
#include <adk_ip.h>
#include <adk_ip_unsupported.h>
#include <adk_rm.h>
#include <adk_lpm.h>
#include <adk_rcu.h>
#include <adk_qos_include_internal.h>
#include <adk_hos.h>
#include <adk_hos_conf.h>
#include <adk_cp_acl_list.h>
#include "adk_ip_cp_export.h"
#include "adk_ip_dp_export.h"
#include "adk_ip_acl.h"
#include "adk_ip_pbr_acl.h"
#include "adk_ip_ing_classifier.h"
#include "adk_ip_qos_classifier.h"
#include "adk_ip_dp.h"
#include <adk_ip_utility.h>


#define ADK_ETH_BR_MAX 1
#define ADK_IPV4_MODULE_COOKIE  0xAD41930D
#define ADK_IPV4_INTF_COOKIE    0xAD419142
#define ADK_IPV4_V_INTF_COOKIE  0xAD419143
#define ADK_IPV4_TUN_INTF_COOKIE 0xAD419144
#define ADK_IPV4_FLOW_COOKIE    0xAD419F70
#define ADK_IPV4_POLICY_COOKIE  0xAD419B50
#define ADK_IPV4_PBR_RULE_COOKIE 0xAD419F10
#define ADK_IPV4_FILTER_COOKIE  0xAD419F17
#define ADK_IPV4_VRD_COOKIE         0xAD419516
#define ADK_IPV4_DSCP_GROUP_COOKIE  0xAD419D5C
#define ADK_IPV4_PKT_CAP_COOKIE     0xAD419942

#define ADK_IP_INGRESS_ASSOC_COMMON_MASK (ADK_IP_INGRESS_ASSOC_DEST_ADDR_MASK |\
                                          ADK_IP_INGRESS_ASSOC_SRC_ADDR_MASK |\
                                          ADK_IP_INGRESS_ASSOC_PROTOCOL_MASK |\
                                          ADK_IP_INGRESS_ASSOC_DEST_PORT_MASK |\
                                          ADK_IP_INGRESS_ASSOC_SRC_PORT_MASK |\
                                          ADK_IP_INGRESS_ASSOC_VRD_ID_MASK)

#define ADK_IPV4_TNL_TBL_HASH_WIDTH 10
#define ADK_IPV4_TNL_TBL_HASH_MAX (1<<ADK_IPV4_TNL_TBL_HASH_WIDTH)
#define ADK_IPV4_TNL_TBL_HASH_MASK (ADK_IPV4_TNL_TBL_HASH_MAX - 1)
#define ADK_IPV4_TNL_TBL_MAX_COLLISION 8

#define ADK_IPV4_EGRESS_TBL_HASH_WIDTH 10
#define ADK_IPV4_EGRESS_TBL_HASH_MAX (1<<ADK_IPV4_EGRESS_TBL_HASH_WIDTH)
#define ADK_IPV4_EGRESS_TBL_HASH_MASK (ADK_IPV4_EGRESS_TBL_HASH_MAX-1)
#define ADK_IPV4_EGRESS_TBL_MAX_COLLISION 8

#define ADK_IPV4_DST_MTU_TBL_HASH_WIDTH 10
#define ADK_IPV4_DST_MTU_TBL_HASH_MAX (1<<ADK_IPV4_EGRESS_TBL_HASH_WIDTH)
#define ADK_IPV4_DST_MTU_TBL_HASH_MASK (ADK_IPV4_EGRESS_TBL_HASH_MAX-1)
#define ADK_IPV4_DST_MTU_TBL_MAX_COLLISION 8

#define ADK_IPV4_QOS_TBL_HASH_WIDTH 8
#define ADK_IPV4_QOS_TBL_HASH_MAX (1<<ADK_IPV4_QOS_TBL_HASH_WIDTH)
#define ADK_IPV4_QOS_TBL_HASH_MASK (ADK_IPV4_QOS_TBL_HASH_MAX-1)
#define ADK_IPV4_QOS_TBL_MAX_COLLISION 8

#define ADK_IP_VRD_ID_INVALID 999

/* Pending List size */
#define ADK_IP_DIP_TBL_PENDING_SIZE ADK_LPM_DEFAULT_PENDING_SIZE
#define ADK_IP_SIP_TBL_PENDING_SIZE ADK_LPM_DEFAULT_PENDING_SIZE

#define ADK_IPV4_LOG_FATAL(mod_handle, err, fmt, args...) \
    do { \
        ADK_LOG_FATAL_ERROR(sys_handle, mod_handle, fmt, ##args); \
        ADK_LOG_FATAL_ERROR(sys_handle, mod_handle, "%s\n", \
                      adk_sys_status_get(sys_handle, err)); \
    } while (0)

#define ADK_IPV4_LOG_ERROR(mod_handle, err, fmt, args...) \
    do { \
        ADK_LOG_ERROR(sys_handle, mod_handle, fmt, ##args); \
        ADK_LOG_ERROR(sys_handle, mod_handle, "%s\n", \
                      adk_sys_status_get(sys_handle, err)); \
    } while (0)

#define ADK_IPV4_LOG_WARNING(mod_handle, fmt, args...) \
    ADK_LOG_WARNING(sys_handle, mod_handle, fmt, ##args)

#define ADK_IPV4_LOG_INFO(mod_handle, fmt, args...) \
    ADK_LOG_INFO(sys_handle, mod_handle, fmt, ##args)

#define ADK_IPV4_LOG_FUNC_ENTRY(mod_handle) \
    ADK_LOG_FUNC_ENTRY(sys_handle, mod_handle)

#define ADK_IPV4_LOG_FUNC_EXIT(mod_handle) \
    ADK_LOG_FUNC_EXIT(sys_handle, mod_handle)

#define ADK_IPV4_LOG_HW(mod_handle, fmt, args...) \
    ADK_LOG_HW(sys_handle, mod_handle, fmt, ##args)

#define ADK_IPV4_LOG_PTN_LEARN(mod_handle, tree_name, ptn, action) \
    ADK_LOG_PTN_LEARN(sys_handle, mod_handle, tree_name, ptn, action)

#define ADK_IPV4_LOG_PTN_UNLEARN(mod_handle, tree_name, ptn, xsize, action) \
    ADK_LOG_PTN_UNLEARN(sys_handle, mod_handle, tree_name, ptn, xsize, action)

#define ADK_IPV4_LOG_SUBNET_ADD(mod_handle, tree_name, ptn, action) \
    ADK_LOG_SUBNET_ADD(sys_handle, mod_handle, tree_name, ptn, action)

#define ADK_IPV4_LOG_SUBNET_DEL(mod_handle, tree_name, ptn) \
    ADK_LOG_SUBNET_DEL(sys_handle, mod_handle, tree_name, ptn)

#define ADK_IPV4_LOG_SUBNET_QUERY(mod_handle, tree_name, ptn, xsize, action) \
    ADK_LOG_SUBNET_QUERY(sys_handle, mod_handle, tree_name, ptn, xsize, action)

#define ADK_IPV4_LOG_VPTN_LEARN(mod_handle, tree_name, vruleidx, no_vptns, \
                                vptn, action) \
    ADK_LOG_VPTN_LEARN(sys_handle, mod_handle, tree_name, vruleidx, no_vptns, \
                       vptn, action)

#define ADK_IPV4_LOG_VPTN_UNLEARN(mod_handle, tree_name, vruleidx, vptn) \
    ADK_LOG_VPTN_UNLEARN(sys_handle, mod_handle, tree_name, vruleidx, vptn)

#define ADK_IPV4_LOG_VPTN_CLEAR(mod_handle, tree_name, vptn) \
    ADK_LOG_VPTN_CLEAR(sys_handle, mod_handle, tree_name, vptn)

#define ADK_IPV4_LOG_VPTN_BUILD(mod_handle, tree_name, vptn) \
    ADK_LOG_VPTN_BUILD(sys_handle, mod_handle, tree_name, vptn)

#define ADK_IPV4_LOG_RPTN_LEARN(mod_handle, tree_name, ruleidx, ptn, no_rptns, \
                                rptn, action) \
    ADK_LOG_RPTN_LEARN(sys_handle, mod_handle, tree_name, ruleidx, ptn, \
                       no_rptns, rptn, action)

#define ADK_IPV4_LOG_RPTN_UNLEARN(mod_handle, tree_name, ruleidx, ptn) \
    ADK_LOG_RPTN_UNLEARN(sys_handle, mod_handle, tree_name, ruleidx, ptn)

#define ADK_IPV4_LOG_RPTN_BUILD(mod_handle, tree_name, ptn) \
    ADK_LOG_RPTN_BUILD(sys_handle, mod_handle, tree_name, ptn)

#define ADK_IPV4_LOG_SE_WRITE(mod_handle, se_addr, idx, char_string) \
    ADK_LOG_SE_WRITE(sys_handle, mod_handle, se_addr, idx, char_string)

#define ADK_IPV4_LOG_SE_READ(mod_handle, se_addr, idx, char_string) \
    ADK_LOG_SE_READ(sys_handle, mod_handle, se_addr, idx, char_string)

#define ADK_IPV4_LOG_FORCE(fmt, args...) \
    ADK_FORCE_LOG_ERROR(fmt, ##args);

#define ADK_IPV4_LOG_EXT_API(mod_handle, api, fmt, args...) \
    ADK_LOG_EXT_API(sys_handle, mod_handle, api, fmt, ##args)

#define ADK_IP_NON_ZERO_CNTR_WRITE(fmt, cntr) \
    if (cntr != 0) { \
        ADK_INFO_WRITE(fmt, cntr) \
    }

/* Validation macros */
#define ADK_IPV4_IS_MC_IP_ADDR(addr) \
    ((addr & IP_MULTICAST_CLASS_MASK) == IP_MULTICAST_CLASS)

#define ADK_IPV4_IS_LB_IP_ADDR(addr) \
    ((addr & IP_LOOPBACK_CLASS_MASK) == IP_LOOPBACK_CLASS)

#define ADK_IPV4_IS_BC_IP_ADDR(addr) \
    (addr == IP_BROADCAST_ADDRESS)

#define ADK_IPV4_IS_ZERO_IP_ADDR(addr) \
    (addr == 0)

#define ADK_IPV4_IS_IP_PREFIX_LEN_INVALID(prefixLen) \
    ((prefixLen == 0) || (prefixLen >= 32))

#define ADK_IPV4_IS_IP_INTF_ADDR_INVALID(if_addr) \
    (((if_addr.prefixWidth >= 4) && ADK_IPV4_IS_MC_IP_ADDR(if_addr.addr)) || \
     ((if_addr.prefixWidth >= 8) && ADK_IPV4_IS_LB_IP_ADDR(if_addr.addr)) || \
     (ADK_IPV4_IS_ZERO_IP_ADDR(if_addr.addr)) || \
     ADK_IPV4_IS_IP_PREFIX_LEN_INVALID(if_addr.prefixWidth))

#define ADK_IPV4_IS_DIRECTION_INVALID(dir) \
    ((dir != ADK_IP_FILTER_IN) && (dir != ADK_IP_FILTER_OUT))

#define ADK_IPV4_IS_IP_IF_ADDR_INVALID(if_addr) \
    (ADK_IPV4_IS_ZERO_IP_ADDR(if_addr) || \
     ADK_IPV4_IS_MC_IP_ADDR(if_addr) || \
     ADK_IPV4_IS_BC_IP_ADDR(if_addr) || \
     ADK_IPV4_IS_LB_IP_ADDR(if_addr))

#define ADK_IPV4_IS_DIP_ADDR_INVALID(dip_addr) \
    (((dip_addr.prefixWidth >= 4) && \
      ADK_IPV4_IS_MC_IP_ADDR(dip_addr.addr)) || \
     ((dip_addr.prefixWidth >= 8) && \
      ADK_IPV4_IS_LB_IP_ADDR(dip_addr.addr)) || \
     ((dip_addr.prefixWidth == 32) && \
      ADK_IPV4_IS_BC_IP_ADDR(dip_addr.addr)) || \
     (dip_addr.prefixWidth > 32))

#define ADK_IP_ALLOWED_PROTO(protocol) (ip_dp_tbl_info_p->allowedL4protos[protocol/(sizeof(ip_dp_tbl_info_p->allowedL4protos[0])*8)] & (1<< (protocol%(sizeof(ip_dp_tbl_info_p->allowedL4protos[0])*8) )))

/* Global constant mask bits defined in adk_ip_common.c file */
extern uint32_t ip_bit_mask[];

typedef enum adk_ipv4_oper_s {
    ADK_IPV4_OPER_ADD = 0,
    ADK_IPV4_OPER_REMOVE,
    ADK_IPV4_OPER_MODIFY
} adk_ipv4_oper_t;

typedef enum adk_ipv4_obj_s {
    ADK_IP_MOD_INFO_OBJ,
    ADK_IP_FILT_RULE_OBJ,
    ADK_IP_ACL_RULE_OBJ,
    ADK_IP_PBR_POLICY_OBJ,
    ADK_IP_PBR_RULE_OBJ,
    ADK_IP_VRD_OBJ,
    ADK_IP_MAX_OBJ,
    ADK_IP_ADDR_OBJ,
    ADK_IP_INGRESS_ASSOC_OBJ,
    ADK_IP_DP_MAX_IP_EG_OBJ,
    ADK_IP_TUN_INTF_OBJ,
    ADK_IP_EXCEP_OBJ,
    ADK_IP_HOS_RULE_OBJ,
    ADK_IP_INTF_OBJ,
    ADK_IP_DIP_OBJ,
    ADK_IP_PKT_CAP_OBJ,
    ADK_IP_ROUTE_ADD_OBJ,
    ADK_IP_VIRT_INTF_OBJ,
    ADK_IP_DIP_TBL_OBJ,
    ADK_IP_SIP_TBL_OBJ,
    ADK_IP_IN_ASSOC_TBL_OBJ,
    ADK_IP_VRD_HMODE_TBL_OBJ,
    ADK_IP_REASS_TBL_OBJ,
    ADK_IP_TUN_TBL_OBJ,
    ADK_IP_EGRESS_TBL_OBJ,
    ADK_IP_QOS_TBL,
    ADK_IP_STATS_OBJ,
    ADK_IP_ADDR_STATS_OBJ,
    ADK_IP_ACL_ACT_OBJ,
    ADK_IP_CP_VERID_OBJ,
    ADK_IP_DEST_MTU_OBJ,
    ADK_IP_IN_FLOW_TBL_OBJ,
    ADK_IP_FLOW_CACHE_TBL_OBJ,
    ADK_IP_EGRESS_ASSOC_TBL_OBJ,
    ADK_IP_PBR_RULE_STATS_OBJ,
    ADK_IP_PBR_ACT_OBJ,
    ADK_IP_VRD_PRB_RULE_OBJ,
    ADK_IP_QOS_POL_ADD_OBJ,
    ADK_IP_DIP_ADD_OBJ,
    ADK_IP_SIP_ADD_OBJ,
    ADK_IP_TUN_ADD_OBJ,
    ADK_IP_ING_CLASSIFIER_INFO_OBJ,
    ADK_IP_ING_CLASSIFIER_RULE_INFO_OBJ,
    ADK_IP_ING_CLASSIFIER_RULE_ACT_OBJ,
    ADK_IP_IFACE_HANDLE_OBJ,
    ADK_IP_EGRESS_QOS_CLASSIFIER_INFO_OBJ,
    ADK_IP_CPPI_STATS,
    ADK_IP_L4_STATS_OBJ
} adk_ipv4_obj_t;

/*!
 * =============================================================
 * @enum adk_ip_sip_filter_action_t
 * TO DO: remove 'filter' from this name
 * @brief
 *
 * Defines the type of action that can be taken for packet with
 * different SIP address.
 *
 * =============================================================
 */
typedef enum adk_ip_sip_filter_action_e {
    /*! Allow packet to go through for forwarding and local termination */
    ADK_IP_SIP_ADDR_ACTION_ALLOW = 0,
    /*! Block packet from forwarding and from terminating locally */
    ADK_IP_SIP_ADDR_ACTION_BLOCK_ALL
} adk_ip_sip_filter_action_t;


/*!
 * =============================================================
 * @internal
 * @struct adk_ip_global_stats_bkp_t
 *
 * @brief IP global stats backup stats
 *
 * Deleted IP interface rxPkts & txPkts are backed up here
 * for IP global stats derivation.
 *
 * =============================================================
 */
typedef struct adk_ip_global_stats_bkp_s {
    /*! Total number of received packets */
    uint64_t ipInPkts;

    /*! Total number of transmitted packets */
    uint64_t ipOutPkts;

    /*! Total number of packets dropped due to policing. */
    uint32_t ipPolicerDrop;

}adk_ip_global_stats_bkp_t;

typedef struct inner_ip_entry_s {
    bool rule_added;
#ifdef ADK_HOS_OLD_IMPL
    adk_hos_rule_id_t rule_id;
    adk_hos_inner_ip_tbl_rule_attr_t rule_attr;
#endif
} inner_ip_entry_t;


/*! Width of the routing domain id in number of bits. This must be set a value
 * within the range of [1  6] bits. This must not be set to any other value
 * other than the given range as that would require fast path code changes */
#define ADK_IP_VRD_ID_WIDTH                     6

/*! Maximum number of virtual routing instances. This is derived from the
 * ADK_IP_VRD_WITH definition. This must be in sync with ADK_IP_VRD_WITH
 * definition. The range of VRD-IDs would be [0  ADK_IP_VRD_ID_MAX-1]*/
#define ADK_IP_VRD_ID_MAX                      1 /* (1 << ADK_IP_VRD_ID_WIDTH ) */

/*! Default virtual routing domain ID. This is used for packets which are not
 * assigned to any VR domain. This must be set to a value with in the range
 * of [0   ADK_IP_VRD_ID_MAX-1] only.*/
/* Update: We are re-purposing vrd-id field for routingTableId. This needs to change if vrd has to be re-introduced */
#define ADK_IP_DEFAULT_VRD_ID                  (ADK_IP_VRD_ID_MAX - 1)



typedef struct ip_sse_addr_rule_ids_s
{
    adk_ip_addr_prefix_t ip;
    adk_hos_rule_id_t ruleId;
    bool inUse;
}ip_sse_addr_rule_ids_t;

/*!
 * ============================================================
 * @typedef adk_ip_vrd_handle_t
 *
 * @brief
 *
 * IP Virtual Routing Instance handle
 *
 * =============================================================
 */
typedef adk_handle_t adk_ip_vrd_handle_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_module_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * global state information for the IP handle. state information
 * for IP module's objects (if, flow) etc are stored in respective
 * handles.
 * =============================================================
 */
typedef struct ip_module_info_s {
    uint32_t cookie; /* IPv4 module specific cookie */
    uint8_t ipModuId;
    uint32_t moduleFlowId;
    /* User Defined Namespace resources */
    adk_rm_hdl_t ipIngClassifierRuleIDHandle;  /* For IP ingress assoc otree rule IDs */
    adk_rm_hdl_t ipPbrPolicyRuleIdHandle; /* For PBR rule IDs */
    adk_rm_hdl_t ipVrdIDHandle; /* For VRD IDs */
    adk_rm_hdl_t ipStatsHandle; /* For IP Stats IDs */
    adk_rm_hdl_t ipAddrStatsHandle; /* For IP Address Stats IDs */
    adk_rm_hdl_t ipIngFlowStatsHandle; /* For IP Ing Flow Stats IDs */
    adk_rm_hdl_t ipRmEgFlowHandle; /* For IP Egress Flow IDs */
    /* For IP egress qos classifier rule IDs per interface */
    adk_rm_hdl_t ipEgressQoSClassifierRuleIDHandle[ADK_DP_MAX_IP_INTERFACE];  
    /*Entry function pointers in DP process*/
    uint16_t       cpuEntryFn_LLI[_DEST_FUNC_MAX];
    uint16_t       cpuEntryFn_ULI[_DEST_FUNC_MAX];
    /* For VRD handles */
    adk_ip_vrd_handle_t vrdHandles[/*ADK_IP_DEFAULT_VRD_ID*/ 1];
    adk_ip_global_stats_bkp_t ipGlobalStatsTable_bkp;
    uint32_t sif_pcap_id;
    uint32_t hos_pcap_id;
    bool hw_pcap_enabled;
    /*DP tbl data*/
    adk_ip_dp_tbl_info_t *ip_dp_tbl_info_p;
    ip_sse_addr_rule_ids_t sseAddrRuleIds[64]; /* TO DO: 64 is randomly chosen right now. Replace it after getting some upper cap */
    uint16_t numRoutesProgrammed;
    adk_acl_handle_t    routeInfo;
    uint8_t  ipv4IfaceCount;
    uint16_t aggrIpv4AddrCount;
    uint16_t aggrIpv4VirtAddrCount;
    uint8_t numPbrPolicies;
} ip_module_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_interface_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Interface information for an IP interface handle.
 * =============================================================
 */
typedef struct ip_interface_info_s {
    uint32_t cookie;
    adk_ip_interface_t if_info;
    adk_ip_interface_ingress_policing_attr_t if_ingress_policing_attr;
    uint32_t if_ingress_flags;  /* Contains the rule-set ID as well */
    uint32_t if_egress_flags;
    uint8_t ingress_ruleset_id;
    uint8_t numSecAddr;
    adk_ip_addr_prefix_t ip_prefix[IP_SEC_ADDR_MAX];
    uint16_t ip_prefix_stats_id[IP_SEC_ADDR_MAX];
    uint8_t vrd_id;
    uint16_t   ip_stats_id;
    uint16_t   ip_addr_stats_id; /* for primary ip address */
    adk_hos_rule_id_t prim_addr_term_rule_id[2]; /*Inner & Outer Table. 1 interface address and 2 subnet bcast addresses */
    adk_hos_rule_id_t sec_addr_term_rule_id[2][IP_SEC_ADDR_MAX];
    uint32_t sys_egress_stats_id;
} ip_interface_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_virtual_interface_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Interface information for a virtual IP interface handle.
 * =============================================================
 */
typedef struct ip_virtual_interface_info_s {
    uint32_t cookie;
    uint32_t stats_id;
    uint32_t ip_addr_stats_id;
    /*adk_ip_dp_intf_stats_t   ip_stats[ADK_MAX_NUM_CORES][ADK_IP_DSCP_GROUPS_MAX+1];*/
    adk_ip_virtual_interface_t vif_info;
    uint8_t vrd_id;
    adk_hos_rule_id_t ip_term_rule_id[1]; /*Inner & Outer tables*/
} ip_virtual_interface_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_vrd_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * virtual routing domain information for a VRD handle.
 * =============================================================
 */
typedef struct ip_vrd_info_s {
    uint32_t cookie;
    uint8_t vrd_id;
} ip_vrd_info_t;


/*!
 * =============================================================
 * @internal
 * @enum ip_flow_type_t
 *
 * @brief
 *
 * IP flow types
 *
 * =============================================================
 */
typedef enum ip_flow_type_t {
    /*!< IP egress flows */
    IP_FLOW_TYPE_EGRESS_FLOW = 0,
    /*!< Flows representing packets matching a PBR Rule */
    IP_FLOW_TYPE_PBR_RULE,
    /*!< Flows for traffic rate policing */
    IP_FLOW_TYPE_POLICE_FLOW,
    /*!< Ingress classifier ordered flows */
    IP_FLOW_TYPE_INGRESS_CLASSIFIER_FLOW,
    /*!< Egress QoS classifier ordered flows */
    IP_FLOW_TYPE_EGRESS_QOS_CLASSIFIER_FLOW,
    /*!< Flow type upper limit */
    IP_FLOW_TYPE_INVALID
} ip_flow_type_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_ing_classifier_flow_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Flow information for an IP ingress classifier ordered flow.
 * =============================================================
 */
typedef struct ip_ing_classifier_flow_info_s {
    uint16_t precedence;
    adk_ip_ingress_classifier_rule_t rule;
    adk_ip_ingress_classifier_action_t action;
    adk_acl_rule_t cp_rule;
} ip_ing_classifier_flow_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_pbr_flow_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Flow information for an PBR flow.
 * =============================================================
 */
typedef struct ip_pbr_flow_info_s {
    uint16_t precedence;
    uint16_t rule_index;
    adk_ip_pbr_rule_t rule;
    adk_ip_pbr_action_t action;
    adk_acl_rule_t cp_rule;
} ip_pbr_flow_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_qos_classifier_flow_info_t
 *
 * @brief
 *
 * Flow information for an IP egress QOS classifier flow.
 * =============================================================
 */
typedef struct ip_qos_classifier_flow_info_s {
    uint16_t precedence;
    adk_ip_qos_info_t flow_info;
    adk_acl_rule_t cp_rule;
} ip_qos_classifier_flow_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_flow_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Flow information for an IP flow.
 * =============================================================
 */
typedef struct ip_flow_info_s {
    uint32_t cookie;
    uint32_t flow_stats_id;
    ip_flow_type_t type;
    union {
        ip_pbr_flow_info_t pbr_flow;
        ip_ing_classifier_flow_info_t  ingClassifierFlow;
        ip_qos_classifier_flow_info_t  egress_qos_flow; 
    } u;
} ip_flow_info_t;

/*!
 * =============================================================
 * @typedef adk_ip_pktcap_handle_t
 *
 * @brief
 *
 * IP Capture Point handle
 *
 * =============================================================
 */
typedef adk_handle_t adk_ip_pktcap_handle_t;

/*!
 * =============================================================
 * @struct adk_ip_pktcap_status_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * Status of a packet capture point. This tells if the packet capture
 * has been autonomously stopped by ADK or not.
 *
 * =============================================================
 */
typedef struct adk_ip_pktcap_status_s
{
    /*!
     * Boolean flag indicating if the current packet capturing status.
     * If set to TRUE, the capturing is ON. If FALSE, then capturing
     * has been stopped autonomously by ADK due to packet drop by MTM
     * buffer manager while rate limiting. Application must call
     * adk_ip_packet_capture_enable() API to re-start pkt capturing.
     */
    bool is_enabled;
} adk_ip_pktcap_status_t;

/*!
 * =============================================================
 * @struct adk_ip_pktcap_rule_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * Packet capture point definition. This provides the rule for
 * packet capture point identification. This consists of IP interface
 * handle and the ingress or egress direction.
 *
 * =============================================================
 */
typedef struct adk_ip_pktcap_rule_s
{
    /*!
     * IPv4 interface handle on which capturing needs to be enabled.
     */
    adk_interface_handle_t ipIfHandle;
    /*!
     * Direction in which capture has to be enabled i.e. Ingress or Egress.
     */
    adk_ip_direction_t direction;
} adk_ip_pktcap_rule_t;

/*!
 * =============================================================
 * @struct adk_ip_pktcap_attr_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * Packet capture point attributes. This provides the parameters
 * associated with packet capturing towards CPU. It consists of
 * MTM queue handle (for rate limiting), PH parameters and user
 * specified context. The context can be used by application to
 * differentiate between captured traffic flows belonging to
 * different capture points.
 *
 * =============================================================
 */
typedef struct adk_ip_pktcap_attr_s
{
    /*!
     * MTM Queue to be used for rate limiting captured traffic.
     * For autonomous stopping upon loss of packets, the MTM
     * queue must be configured with supporting MTM script
     * (e.g. ADK_QOS_BM_ETHERNET_SSM_TAIL_DROP_PKT_CAPTURE).
     * This member is Deprecated.
     */
#ifndef ADK_QOS_GEN3_INTERFACE
    adk_handle_t mtmQueueHandle;
#else
    adk_handle_t rl_queue_handle;
#endif

    /*!
     * The 24-bit context field is optional and the value in this field
     * will be delivered to the receiving function at CPU to identify
     * capture point if needed.
     */
    uint32_t context;
} adk_ip_pktcap_attr_t;

/*!
 * =============================================================
 * @struct adk_ip_pktcap_info_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * Packet capture point having the definition and the attributes.
 *
 * =============================================================
 */
typedef struct adk_ip_pktcap_info_s
{
    /*!
     * Packet capture point definition.
     */
    adk_ip_pktcap_rule_t pktCapRule;
    /*!
     * Packet capture point attributes.
     */
    adk_ip_pktcap_attr_t pktCapAttr;
} adk_ip_pktcap_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_pktcap_info_t
 *
 * @brief
 *
 * Information for a packet capture point.
 * =============================================================
 */
typedef struct ip_pktcap_info_s
{
    uint32_t cookie;
    adk_ip_pktcap_info_t pktcap_info;
} ip_pktcap_info_t;

/* For ADK_IP_NO_ROUTE_EXCEPTION*/

typedef struct ip_ph_metadata_s
{
    adk_ip_exception_type_t exception_type;
    uint8_t routeTableId;
    uint32_t interface_id;
    adk_ip_direction_t dir;
} __attribute__((aligned(ADK_METADATA_ALIGN))) ip_ph_metadata_t;




bool adk_ipv4_valid_handle(adk_module_handle_t ipv4_mod_handle);

bool adk_ipv4_interface_valid_handle(adk_interface_handle_t ipIfHandle);

bool adk_ipv4_vrd_valid_handle(adk_ip_vrd_handle_t ipVrdHandle,
                                     uint32_t        *vrdId);

bool adk_ipv4_vif_valid_handle(adk_interface_handle_t ipIfHandle);

bool adk_ipv4_flow_valid_handle(adk_flow_handle_t ipFlowHandle);

const char *adk_ipv4_error_name(adk_status_t err);

adk_status_t adk_ipv4_map_sys_log(adk_module_handle_t ipv4_mod_handle,
                                  adk_log_level_t sys_log_level,
                                  adk_log_level_t *ipv4_log_level);

adk_status_t adk_ipv4_sys_log(adk_module_handle_t ipv4_mod_handle,
                              adk_log_level_t sys_log_level);

adk_status_t adk_ip_cpu_flow_policing_config (
    adk_module_handle_t ipHandle, 
    uint32_t cpu_flow_id,
    uint32_t policingHandle,
    bool addEntry);

adk_status_t adk_ip_rsrc_policing_config(
    adk_module_handle_t ipHandle,
    uint32_t filter_flow_id,
    uint32_t policing_flow_id,
    bool addEntry);

adk_status_t adk_ip_shutdown(adk_module_handle_t);

adk_status_t adk_ip_get_imc(adk_module_handle_t ipHandle,
                            void *bindInfo_p,
                            uint32_t *imcFnId);

adk_status_t adk_ipv4_dump_tables(adk_module_handle_t ipHandle,
                                  adk_ip_table_group_t table_group);

adk_status_t
adk_ip_intf_attr_tbl_dump(adk_module_handle_t ip_handle) ;

adk_status_t
adk_ip_dest_mtu_tbl_dump(adk_module_handle_t ip_handle);

adk_status_t
adk_ip_dp_fc_tbl_dump(adk_module_handle_t ip_handle);

adk_status_t
adk_ip_dp_reasm_tbl_dump(adk_module_handle_t ip_handle);

adk_status_t
adk_ip_dp_exception_handler_tbl_dump(adk_module_handle_t ip_handle);

adk_status_t
adk_ip_dp_vrdHostmode_dump(adk_module_handle_t ip_handle);

adk_status_t
adk_ip_dp_pbr_tbl_dump(adk_module_handle_t ip_handle);

void adk_ip_dip_dump_cb_fn(adk_lpm_handle_t lpm, 
                          void *rock,
                          uint8_t *key, 
                          uint16_t depth, 
                          adk_handle_t value);

void adk_ip_sip_dump_cb_fn(adk_lpm_handle_t lpm,
                          void *rock,
                          uint8_t *key,
                          uint16_t depth,
                          adk_handle_t value);

adk_status_t adk_ip_ingress_classifier_stats_print (
    adk_module_handle_t ipHandle,
    adk_flow_handle_t ipFlowHandle);


adk_status_t adk_ip_packet_capture_start(
    adk_module_handle_t ipHandle,
    const adk_ip_pktcap_info_t *pktCapInfo_p,
    adk_ip_pktcap_handle_t *pktCapHandle);

adk_status_t adk_ip_packet_capture_stop(
    adk_module_handle_t ipHandle,
    adk_ip_pktcap_handle_t pktCapHandle);

adk_status_t adk_ip_packet_capture_status_get(
    adk_module_handle_t ipHandle,
    adk_ip_pktcap_handle_t pktCapHandle,
    adk_ip_pktcap_status_t *pktCapStatus_p);

adk_status_t adk_ip_packet_capture_enable(
    adk_module_handle_t ipHandle,
    adk_ip_pktcap_handle_t pktCapHandle);

adk_status_t adk_ip_pbr_create(
        adk_module_handle_t  ipHandle,
        adk_ip_pbr_ctx_t  **ipPbrInfo_pp);

adk_status_t adk_ip_pbr_delete(
        adk_module_handle_t  ipHandle,
        adk_ip_pbr_ctx_t  **ipPbrInfo_pp);

#endif


/*! end addtogroup _ip
 * @}
 */
