/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_ip_defs.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ip  IPv4 Module
 *
 * @{
 *
 * @brief
 *
 * IP module's specific definitions shared by C code, code and C-NP scripts.
 *
 ***********************************************************************/
#ifndef _ADK_IP_DEFS_H_
#define _ADK_IP_DEFS_H_

#include <adk_ip_config.h>

#define IP_HDR_PRESENT_FLAG_BIT         0
#define IP_FLOW_ID_PRESENT_FLAG_BIT     1

#define IP_HDR_TTL_BYPASS_FLAG_BIT      0
#define IP_META_DATA_IN_PKT_FLAG_BIT    1
#define IP_HDR_ENCAP_DF_FLAG_BIT        2
#define IP_L4_CSUM_UPDATE_FLAG_BIT      4
#define IP_ADD_UDP_HEADER_FLAG_BIT      5
#define IP_USE_FLOW_SIP_FLAG_BIT        6

#define IP_HDR_TTL_BYPASS_FLAG      (1 << IP_HDR_TTL_BYPASS_FLAG_BIT)
#define IP_META_DATA_IN_PKT_FLAG    (1 << IP_META_DATA_IN_PKT_FLAG_BIT)
#define IP_HDR_ENCAP_DF_BIT_FLAG    (1 << IP_HDR_ENCAP_DF_FLAG_BIT)
#define IP_L4_CSUM_UPDATE_FLAG      (1 << IP_L4_CSUM_UPDATE_FLAG_BIT)
#define IP_ADD_UDP_HEADER           (1 << IP_ADD_UDP_HEADER_FLAG_BIT)
#define IP_USE_FLOW_SIP             (1 << IP_USE_FLOW_SIP_FLAG_BIT)

#define IP_REASSEMBLY_HASH_FAILURE 0x01

#define IP_REASSEMBLY_NO_ERROR          0x0
#define IP_REASSEMBLY_HOLES_EXCEEDED    0x1
#define IP_REASSEMBLY_DUP_BYTES         0x2
#define IP_REASSEMBLY_EXPIRED           0x3
#define IP_REASSEMBLY_PKT_MISMATCH      0x4

#define IPV4_BASE_HDR_LEN 20
#define POLICE_PARAM_SIZE 32

#define IP_BROADCAST_ADDRESS 0xffffffff
#define IP_MULTICAST_CLASS   0xe0000000
#define IP_MULTICAST_CLASS_MASK 0xf0000000
#define IP_LOOPBACK_CLASS   0x7f000000
#define IP_LOOPBACK_CLASS_MASK 0xff000000

#define IP_SEC_ADDR_MAX (ADK_IP_MAX_ADDR_PER_INTERFACE - 1)

#define IP_IF_MTU_MIN 64
#define IP_IF_MTU_MAX 9706

#define IP_OUTER_PMTU_PTB_MASK     0x8000
#define IP_OUTER_PMTU_MASK         0x7FFF

/* A point to point Ethernet link is identified by its IP address having
 * prefix width of 31. For such links, the network address and subnet
 * broadcast addresses are not applicable. The two available address in
 * 255.255.255.254 subnet mask are used as host addresses at the two ends */
#define IP_P2P_IF_ADDR_WIDTH 31

#define IP_UDP_HEADER_LENGTH    8

#define IP_INTERFACE_WIDTH 24
#define IPV4_ADDR_WIDTH 32
#define IPV4_SUBNET_WIDTH 32
#define IPV4_L4PORT_WIDTH 16
#define IPV4_PROTO_WIDTH 8
#define IP_ASSOCIATION_ID_WIDTH 24
#define IP_FLOW_WIDTH IP_ASSOCIATION_ID_WIDTH

#define IP_ACL_PERMIT 1
#define IP_ACL_DENY 2
#define IP_ACL_SEND_TO_HOST 3

#define IP_NCA_PATH_NO_MTM_FLOW_ID      0
#define IP_NCA_PATH_VIA_MTM_FLOW_ID     1

#define IP_ACL_PERMIT_DEFAULT 0b010000000000000000000000

#define IP_EXC_DOMAIN_TN_PORT               0
#define IP_EXC_DOMAIN_FP_ORIGIN             1
#define IP_EXC_DOMAIN_OS_ORIGIN             2
#define IP_EXC_DOMAIN_MAX_NUM               3

#if 0
#define IP_EXCEPTION_DOMAIN_WIDTH       2
#define IP_EXCEPTION_DOMAIN_MASK        ((1 << IP_EXCEPTION_DOMAIN_WIDTH) - 1)
#define IP_EXCEPTION_TYPE_WIDTH         4
#define IP_EXCEPTION_TYPE_MASK          ((1 << IP_EXCEPTION_TYPE_WIDTH) - 1)
#define IP_EXCEPTION_LOOKUP_WIDTH       (IP_EXCEPTION_DOMAIN_WIDTH + \
                                         IP_EXCEPTION_TYPE_WIDTH)
#define IP_EXCEPTION_CPU_STATE_OFFSET   0
#define IP_EXCEPTION_NONPH_STATE_OFFSET 1
#define IP_EXCEPTION_NCAQID_OFFSET      2
#define IP_EXCEPTION_PH_HANDLE_OFFSET   2
#define IP_EXCEPTION_PH_CBID_OFFSET    10
#endif


#define IP_DEST_ADDR_TREE_NUM_FIELDS   1

#define IP_DEST_ADDR_TREE_WIDTH        IPV4_ADDR_WIDTH

#define IP_DEST_ADV_LKUP_BIT           0
#define IP_DEST_DEFRAG_BIT             1
#define IP_LOCALLY_CONNECTED_NW_BIT    2
#define IP_DEST_IF_ADDR_BIT            7
#define IP_DEST_L2_LKUP_BIT            8
#define IP_DEST_ADDR_LKUP_BIT          9
#define IP_DEST_SRC_ADDR_LKUP_BIT      10
#define IP_DEST_PROTO_LKUP_BIT         11
#define IP_DEST_DPORT_LKUP_BIT         12
#define IP_DEST_SPORT_LKUP_BIT         13

#define IP_DEST_DROP                   1
#define IP_DEST_CPU_BIT                2
#define IP_DEST_FAST_FWD_BIT           3
#define IP_DEST_ASSOC_FILTER_BIT       6
#define IP_DEST_POLICE_BIT             8

#define IPV4_ADV_PROTO_WIDTH    IPV4_PROTO_WIDTH
#define IPV4_ADV_ASSOC_WIDTH    IP_ASSOCIATION_ID_WIDTH
#define IPV4_ADV_ADDR_WIDTH     IPV4_ADDR_WIDTH
#define IPV4_ADV_DEST_NET_WIDTH 24
#define IPV4_ADV_DEF_PROTO      0x00
#define IPV4_ADV_DEF_ADDR       0x00000000
#define IPV4_ADV_DEF_ASSOC      0xffffff
#define IPV4_ADV_LKUP_WIDTH     110

#define IP_NAPT_TREE_WIDTH  48
#define IP_NAPT_TREE_NUM_FIELDS 2
#define IP_NAPT_TREE_IPADDR_WIDTH IPV4_ADDR_WIDTH
#define IP_NAPT_TREE_L4PORT_WIDTH 16

#define IP_EGRESS_IF_LOOKUP_NUM_FIELDS 1
#define IP_EGRESS_IF_ID_WIDTH    IP_INTERFACE_WIDTH
#define IP_EGRESS_IF_MTU_WIDTH    16
#define IP_DF_LOGIC_WIDTH         1
#define IP_EGRESS_IF_LOOKUP_WIDTH IP_INTERFACE_WIDTH
#define IP_EGRESS_IF_DEFAULT_RETURN 0x00ffff

#define IP_FLOW_ENCAP_ONLY_BIT      8

#define IP_EGRESS_FLOW_LOOKUP_WIDTH (IP_INTERFACE_WIDTH + IPV4_ADDR_WIDTH)

/* Must be contiguous with upper Nat-Outside flag offset */
#define IP_NAT_INSIDE_FLAG_OFFSET       0
/* Must be contiguous with upper PBR flag offset */
#define IP_NAT_OUTSIDE_FLAG_OFFSET      1
#define IP_IF_PBR_FLAG_OFFSET           2   /* Ingress only */
/* Must be contiguous with upper extended filter flag, policing flag
 * and DSCP assign flag */
#define IP_BASIC_FILTER_FLAG_OFFSET     3
#define IP_EXT_FILTER_FLAG_OFFSET       4
#define IP_IF_POLICE_FLAG_OFFSET        5   /* Ingress only */
#define IP_IF_DSCP_ASSIGN_FLAG_OFFSET   6   /* Ingress only */
#define IP_IF_QOS_FLAG_OFFSET           5   /* Egress only */
#define IP_IF_DSCP_PCP_MAP_FLAG_OFFSET  6   /* Egress only */
#define IP_IF_FLAG_UNUSED               7
#define IP_IF_FILTER_RULE_SET_OFFSET    8

#define IP_IF_FILTER_RULE_SET_WIDTH     4
#define IP_IF_FILTER_RULE_SET_MASK      ((1 << IP_IF_FILTER_RULE_SET_WIDTH) - 1)

#define IP_IF_FILTER_INVALID_RULE_SET   0

/* The VP Flow IDs - also used for demux */
#define IP_DIP_REASSEMBLE               0
#define IP_NAT_REASSEMBLE               1
#define IP_DIP_NAT_TIMEOUT              2
#define IP_ACL_REASSEMBLE               3
#define IP_ACL_TIMEOUT                  4

/* Move to adk_ip_config.h file when used */
#define IP_NUM_IPV4_FLOWS       1000

/* Number of fields in the PBR lookup */
#define IP_NUM_PBR_INPUT_FIELDS    7

#define IP_PH_TREE_WIDTH      IP_ASSOCIATION_ID_WIDTH
#define IP_FLOW_NCAQ_INFO_OFFSET      16

#define IPV4_IF_ADDR_LOOKUP_WIDTH IPV4_ADDR_WIDTH

#define IP_ENCAP_ONLY_FLOW_LOOKUP_WIDTH  8

#define IPV4_ASSOC_FILTER_NUM_FIELDS    5

/*
 * Traffic management tree related definitions.
 */
#define IPV4_DSCP_WIDTH          8
#define IPV4_DSCP_MAP_TREE_WIDTH (IP_ASSOCIATION_ID_WIDTH + IPV4_DSCP_WIDTH)
#define IPV4_QOS_TREE_WIDTH      (IP_INTERFACE_WIDTH + IPV4_DSCP_WIDTH)
#define IPV4_QOS_MTM_QUEUE_FLAG_BIT     0
#define IPV4_QOS_PCP_VALUE_BIT          1
#define IPV4_QOS_PCP_FLAG_BIT           4

/*
 * IP filter specific definitions
 * */
#define IP_FILTER_ELEMENT_L2IF        1
#define IP_FILTER_ELEMENT_TOS         2
#define IP_FILTER_ELEMENT_FRAG        3
#define IP_FILTER_ELEMENT_PROTO       4
#define IP_FILTER_ELEMENT_SADDR       5
#define IP_FILTER_ELEMENT_DADDR       6
#define IP_FILTER_ELEMENT_PROTO_INFO1 7
#define IP_FILTER_ELEMENT_PROTO_INFO2 8
#define IP_FILTER_ELEMENT_PROTO_INFO3 9
#define IP_FILTER_NUM_ELEMENTS        9

#define IP_BASIC_FILTER_NUM_ELEMENTS  4

/* Bit positions of output fields or flags from ACL rule */
#define IP_FILTER_ACTION_BIT                0
#define IP_FILTER_CNTR_ENABLE_BIT           2
#define IP_FILTER_POLICE_ENABLE_BIT         3
#define IP_FILTER_CPU_NOPH_BIT              4
#define IP_FILTER_STATEFUL_ENABLE_BIT       5
#define IP_FILTER_FRAG_POLICE_ENABLE_BIT    6
#define IP_FILTER_NEW_DSCP_OFFSET           10
#define IP_FILTER_DSCP_REMAP_BIT            (IP_FILTER_NEW_DSCP_OFFSET + 6)

/* Bit positions in IPV4_COLOR_TOS global register */
#define IP_TOS_BIT                  0
#define IP_DSCP_BIT                 2
#define IP_DSCP_REMAP_BIT           8
#define IP_POLICE_COLOR_BIT         9

/* Input field widths in ACL rule */
#define IP_FILTER_FRAG_WIDTH          2
#define IP_FILTER_TOS_WIDTH           8
#define IP_FILTER_INFO1_WIDTH         16
#define IP_FILTER_INFO2_WIDTH         16
#define IP_FILTER_INFO3_WIDTH         8
#define IP_FILTER_ICMP_FIELD_WIDTH    8
#define IP_FILTER_IGMP_FIELD_WIDTH    8
#define IP_FILTER_TCP_FLAGS_WIDTH     8

/* Offset to various actions for fragments in SE mem */
#define IP_FRAG_FILTER_ACTION_OFFSET    32
#define IP_DEFRAG_INIT_TS_OFFSET        48
#define IP_DEFRAG_IN_INTF_PORT_OFFSET   56

/* Counter offset for packets matching ACL filter rule in SE mem */
#define IP_FILTER_COUNTER_OFFSET        0

#define IP_FILTER_ACCEPT_PDU          1
#define IP_FILTER_DENY_PDU            2
#define IP_FILTER_FORWARD_PDU_TO_CPU  3

#define IP_FILTER_PROTOCOL_ICMP         1
#define IP_FILTER_PROTOCOL_IGMP         2
#define IP_FILTER_PROTOCOL_TCP          6
#define IP_FILTER_PROTOCOL_UDP         17
#define IP_FILTER_PROTOCOL_UDP_LITE   136
#define IP_FILTER_PROTOCOL_SCTP       132

/*Filter rule field validation*/
#define IP_FILTER_MAX_PROTO         0xFF
#define IP_FILTER_MAX_DSCP          0x3F
#define IP_FILTER_MAX_ICMP_TYPE     0xFF
#define IP_FILTER_MAX_ICMP_CODE     0xFF
#define IP_FILTER_MAX_IGMP_TYPE     0xFF
#define IP_FILTER_MAX_SRC_PORT      0xFFFF
#define IP_FILTER_MAX_DST_PORT      0xFFFF

#define IP_POLICE_PARAM_SIZE 64

/*
 * interface default forwarding action definitions
 *
 */
#define ACTION_TYPE_WIDTH 4
#define IP_IF_DROP 1
#define IP_IF_FWD_EGRESS_IF 2

#define IP_GLOBAL_STATS  NCP_NS_BASEID_ip_if_stats

#define IP_TOTAL_RECV_PKTS                 0x00
#define IP_TOTAL_TX_PKTS                   0x04
#define IP_INVALID_VERSION                 0x08
#define IP_BAD_CSUM                        0x0c
#define IP_DEFRAG_PKT_CNT                  0x10
#define IP_DEFRAG_HASH_ALLOC_FAILURE_CNT   0x14
#define IP_NO_NEXTHOP_CNT                  0x18
#define IP_INGRESS_ASSOCIATION_FAILURE_CNT 0x1c
#define IP_FILTER_PDU_DENY_CNT             0x20
#define IP_FRAGS_POST_TIMEOUT_CNT          0x24
#define IP_EXCEPTION_DROPS                 0x28
#define IP_TTL_EXPIRED                     0x2c
#define IP_EGRESS_RECV_CNT                 0x30
#define IP_DEFRAG_HASH_DELETE_FAILURE_CNT  0x34
#define IP_FILTER_PDU_ACCEPT_CNT           0x38
#define IP_FILTER_PDU_CPU_SEND_CNT         0x3c
#define IP_FRAGS_DROPPED_CNT               0x40
#define IP_DBG_ENCAP_ONLY_RQST             0x44
#define IP_INVALID_INPUT_IF_CNT            0x48
#define IP_POLICE_MARK_CNT                 0x4c
#define IP_POLICE_DROP_CNT                 0x50
#define IP_DEFRAG_TIMEOUT_CNT              0x54
#define IP_TOTAL_DECAPSULATED_PKTS         0x58
#define IP_UNKNOWN_ASSOCIATION_MASK_CNT    0x5c
#define IP_NO_ENCAP_FLOW_ID                0x60
#define IP_NO_IF_DEFAULT_ACTION            0x64
#define IP_ASSOCIATION_DROP_CNT            0x68
#define IP_FLOW_ACL_FILTER_DROP            0x6c
#define L4_INVALID_CHECKSUM_CNT            0x70
#define IP_INGRESS_ASSOC_CPU_SEND_CNT      0x74
#define IP_FRAG_REASSEMBLE_PDU_CNT         0x78
#define IP_FRAGS_IGNORED_CNT               0x7C
#define IP_DBG_EGRESS_PKTS_NO_FLOWID_WH    0x80
#define IP_DBG_EGRESS_PKTS_NO_FLOWID_NH    0x84
#define IP_INVALID_HDR_LENGTH              0x88
#define IP_FILTER_ILLEGAL_FRAG_CNT         0x8C
#define IP_FRAG_ID_MISMATCH_CNT            0x90
#define IP_ACL_UNKNOWN_RULE_SET            0x94
#define IP_FILTER_FRAG_RSRC_POLICE_DROP         0x98
#define IP_FILTER_HASH_INSERT_W_TIMER_DUP_CNT   0x9C
#define IP_FILTER_HASH_INSERT_W_TIMER_FAIL_CNT  0xA0
#define IP_INVALID_SIP_ADDR                0xA4
/* Not a counter but a place holder to generate IP Identification field. Always
 * keep this at the end */
#define IP_PACKET_ID                       0xA8
#define IP_PACKET_ID_2                     0xAC

/*
 * Ingress association state.
 */
#define IP_INGRESS_FLOW_RECV_PKTS           0x00
#define IP_INGRESS_FLOW_TX_PKTS             0x04
#define IP_PH_CORRELATOR                    0x08
#define IP_TRANSLATE_COMPUTE_UDP_CHKSUM     0x0c
#define IP_TRANSLATE_SINGLE_HOP_PDU_DROP    0x10

/*
 * Egress association state.
 */
#define IP_EGRESS_FLOW_TX_PKTS              0x00
#define IP_DEBUG_EGRESS_PKTS_WITH_FLOWID_NH 0x04
#define IP_DEBUG_EGRESS_PKTS_WITH_FLOWID_WH 0x08

#define IP_INGRESS_PROCESSING       0x01
#define IP_EGRESS_PROCESSING        0x02
#define IP_IPSEC_EGRESS_PROCESSING  0x03

#define IPV4_PCP_WIDTH 3
#define IP_NUM_PCP_VALUES       (1 << IPV4_PCP_WIDTH)
#define IP_NUM_DSCP_VALUES                  64
#define IP_DEFAULT_PCP  0

#endif

/*! end addtogroup _ip
 * @}
 */
