/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef __ADK_IP_DP_H__
#define __ADK_IP_DP_H__

#include "adk_rcu_distributed_util.h"
#include <adk_ip_utility.h>

#define ADK_IP_SHM_MEMZONE_NAME   "adk_ip_shm_zone"

#define IP_TUNNEL_IF_ID_MIN 512
/* We can offload max 1024 rules in HOS. With 24 ip addresses per interfaces, max allowed interfaces
 * works out to be 42
 */
#define ADK_DP_MAX_IP_INTERFACE (ADK_IP_MAX_INTERFACES)

#define ADK_DP_MAX_IP_STATS ADK_DP_MAX_IP_INTERFACE
#define ADK_DP_MAX_IP_ING_ASSOC_FLOWS 1024
#define ADK_DP_MAX_IP_EG_ASSOC_FLOWS 1024
#define ADK_DP_MAX_IP_EG_FLOWS (1024)

#define ADK_IP_TUNNEL_ID_APP_TO_ADK(app_id) \
    (app_id + ADK_IP_MAX_INTERFACES)
#define ADK_IP_TUNNEL_ID_ADK_TO_APP(adk_id) \
    (adk_id - ADK_IP_MAX_INTERFACES)
#define ADK_IP_TUNNEL_ID_NETD_TO_ADK(netd_id) \
    (netd_id - IP_TUNNEL_IF_ID_MIN + ADK_IP_MAX_INTERFACES)
#define ADK_IP_TUNNEL_ID_ADK_TO_NETD(adk_id) \
    (adk_id - ADK_IP_MAX_INTERFACES + IP_TUNNEL_IF_ID_MIN)

#define IP_FLOW_HASH_WIDTH 11

#define IP_FLOW_HASH_MAX   (1<<IP_FLOW_HASH_WIDTH)
#define IP_FLOW_HASH_MASK  (IP_FLOW_HASH_MAX - 1)
#define IP_MAX_FLOWS_PER_CPU 1024  /*TODO look for dependency on ageing data structure*/

#define IP_FLOW_HASH_COLLISION_SIZE 4

/*DP Table defines*/
#define IP_ADV_LKUP_MASK_WIDTH 8
#define IP_VRDID_WIDTH         8
#define IP_DIP_TBL_WIDTH  (IP_VRDID_WIDTH+IPV4_ADDR_WIDTH)
#define IP_SIP_TBL_WIDTH  (IP_VRDID_WIDTH+IPV4_ADDR_WIDTH)
#define IP_ING_ASSOC_TBL_WIDTH (IP_ADV_LKUP_MASK_WIDTH + IP_VRDID_WIDTH +\
                 IPV4_PROTO_WIDTH + IPV4_ADDR_WIDTH + IPV4_L4PORT_WIDTH +\
                 IPV4_ADDR_WIDTH + IPV4_L4PORT_WIDTH)


#define MAX_DSCP_IDS       22

#define ADK_IP_DP_LOG_ERROR(__msg) ADK_DP_LOG_ERROR(ADK_IP_MODULE_ID, __msg)

#define ADK_CONVERT_TO_LPM_DATA(__DATA) ((adk_handle_t)(uintptr_t)__DATA)

#ifdef ADK_UTILITY_COUNTER_ENABLE
#define INC_IP_UTIL_CNTR(cntr, cnt) \
do{\
if(ip_dp_tbl_info_p->utilCntrEnabled)\
    cntr += cnt;\
}while(0);

#else /* ADK_UTILITY_COUNTER_ENABLE */
#define INC_IP_UTIL_CNTR(cntr, cnt)
#endif /* ADK_UTILITY_COUNTER_ENABLE */

typedef struct adk_ip_dp_intf_stats_s
{
    /*! Release counters */

    /* verified */
    /* Incremented before any validation */
    uint64_t ifInOctets;
    uint64_t ifInPkts;
    uint64_t ifInMcastPkts;
    uint64_t ifInMcastOctets;
    /* */

    /* for ifOutOctets, we have 3 counters in DP */
    uint64_t ifOutUcastPkts;
    uint64_t ifOutMcastPkts;
    uint64_t ifOutBcastPkts;
    /* */

    /* for ifOutPkts, we have 3 counters in DP */
    uint64_t ifOutUcastOctets;
    uint64_t ifOutMcastOctets;
    uint64_t ifOutBcastOctets;
    /* */

    uint32_t ifInIpCksumErrDrops;
    uint32_t ifInMBcastDMacUcastDipErrDrops;
    uint32_t ifInBadIpLengthErrDrops; /* Total length < 20 OR hdr.total_length > number-of-bytes-received */
    uint32_t ifOutTtlExceededErrDrops;
    uint32_t ifInVersionMismatchErrDrops;
    uint32_t ifInInvalidVersionErrDrops;
    uint32_t ifInIpOptionsPresentDrops;
    uint32_t ifInUnknownProtoErrDrops; /* this can't be increased for inner packet. is that ok? */
    uint32_t ifInUDPCksumErrDrops;
    uint32_t ifInZeroDipDrops;

    uint64_t ifOutFragPkts;
    uint64_t ifOutFragments;
    uint32_t ifOutDroppedPkts;
    uint32_t ifOutDroppedOctets;
    /* end verified */

    /*Add these */
    uint32_t ifInIcmpCksumErrors;
    uint64_t ifInDiscardedMcastPkts;
    uint64_t ifInDiscardedMcastOctets;
    /* */
    /*! End of Release counters */

    /* Other error counters */
    uint32_t ifRoutingDisabledDrops;
    uint32_t ifOutQoSLookupFailDrops;
    /* end of other error counters */

    adk_ip_ingress_qos_stats_t ifRxQoSstats[ADK_QOS_MAX_PHB_CLASS_IDX];

    /* utility counters */
    uint64_t UifInBcastPkts;
    uint64_t UifInBcastOctets;
    /* utility counters end */

}__attribute__((aligned(64))) adk_ip_dp_intf_stats_t;

typedef struct adk_ip_dp_ip_addr_stats_s
{
    uint64_t ipInOctets;
    uint64_t ipInPkts;
    uint64_t ipOutOctets;
    uint64_t ipOutPkts;
    uint64_t ipInReasmPkts;
    uint32_t ipInErrors;
}__attribute__((aligned(64))) adk_ip_dp_ip_addr_stats_t;


typedef struct adk_ip_dp_global_stats_s {

    /* release counters */
	uint32_t ipOutTooBigDrops;
	uint32_t ipInReasmToutFragDrops;

	/* IP reassembly frag drops due to lack of resources */
	uint32_t ipInReasmHashCollisionDrops;
	uint32_t ipInReasmTimerCreateFailDrops;
	uint32_t ipInReasmSessionAllocFailDrops;
	/* */

    /* end of release counters */

	/* Non release drop counters */
	uint32_t ipInReasmFragBufferFullDrops;
	uint32_t ipInReasmHolesExceededDrops;
	uint32_t ipInClassifierDefaultDrops;
	uint32_t ipNoExceptionRegisteredDrops;
	uint32_t ipExceptionSendFailureDrops;
	uint32_t ipInPolicerDrops;
	uint32_t ipInClassifierDrops;
	uint32_t ipInSendToEoFailDrops;
	uint32_t ipInSendToHostFailDrops;
    uint32_t ipInClassifierUnknownActionDrops;
	uint32_t ipInReasmDuplicateDrops;
	uint32_t ipOutInvalidIpHdrLenDrops;
	uint32_t ipOutNonLocalSipFromEoDrops;
	uint32_t ipInReasmToutMbufAllocFails;
	uint32_t ipOutFragMbufAllocFailDrops;
	uint32_t ipOutTruncatedPktDrops;
	uint32_t ipInInvalidIfDrops;
    uint32_t ipOutInvalidIfDrops;
    uint32_t ipOutNoRouteFoundDrops;
    uint32_t ipOutFragDFSetDrops;
	uint32_t ipInReasmTouts;
	/* End of Non release drop counters */

    /* Utility counters */
    uint64_t UipReasmReqPkts;
    uint64_t UipReasmEnqueuedPkts;
    uint32_t UipReasmTimerCbAfterCleanup;
    uint32_t UipReasmInvalidReasmAction; /* This would indicate memory corruption */
    uint64_t UipInSentToHostPkts;
    uint64_t UipInSentToEoPkts;
    uint64_t UipInSentToIPSecPkts;
    uint64_t UipInUDPPkts;
    uint64_t UipInUDPOctets;
    uint64_t UipInUnknownSender;

    uint64_t UipOutNetDOrigPkts;
    uint64_t UipOutEOOrigPkts;
    uint64_t UipOutSentToIPSecPkts;
    uint64_t UipOutRcvdFromIPSecPkts;
    uint64_t UipOutSentToEnetPkts;
    uint64_t UipOutNonLocalSipFromHostPkts;
    uint64_t UipOutUDPPkts;
    uint64_t UipOutUDPOctets;
    uint64_t UipIpsecFragsCreated;
    uint64_t UipIpsecFragsRequested;
    uint64_t UipOutUnknownSender;
    /* End of utility counters */

} __attribute__((aligned(64))) adk_ip_dp_global_stats_t;

typedef struct adk_ip_dp_ing_flow_stats_s
{
    uint32_t rxPkts;
    uint32_t txPkts;
}__attribute__((aligned(64))) adk_ip_dp_ing_flow_stats_t;

/* Following is defined with the assumption that CPPI runs on a single core */
typedef struct adk_ip_dp_cppi_stats_s
{
  uint32_t cppiTxPkts;
  uint32_t cppiPolicerDropPkts;
}__attribute__((aligned(64))) adk_ip_dp_cppi_stats_t;


typedef struct adk_ip_dp_l4_stats_s
{
    uint64_t ipTcpInPackets;
    uint64_t ipTcpInOctets;
    uint64_t ipTcpOutPackets;
    uint64_t ipTcpOutOctets;
    uint64_t ipSctpInPackets;
    uint64_t ipSctpInOctets;
    uint64_t ipSctpOutPackets;
    uint64_t ipSctpOutOctets;
}adk_ip_dp_l4_stats_t;

typedef enum adk_ip_dp_intf_type_e {
    ADK_IP_DP_INTF_TYPE_INVALID=0,
    ADK_IP_DP_INTF_TYPE_ETH
} __attribute__((__packed__)) adk_ip_dp_intf_type_t;

typedef struct adk_ip_dp_pkt_capture_s
{
    uint32_t context;
    uint16_t queueId;
    uint8_t devId;
    uint8_t cbFnId;
    bool enableCapture;
    bool enableHwCapture;
} __attribute__((__packed__)) adk_ip_dp_pkt_capture_t;

typedef struct adk_ip_dp_interface_entry_s
{
    uint32_t          policerFlowId;
    uint32_t          if_addr;

    uint32_t          sys_egress_stats_id;
    /* each core shall have its own stats.
     * CP core will accumulate stats from all cores and return
     * to application.
     */
    uint16_t          if_stats_id;
    uint16_t          mtu;
    uint16_t          outerPathMtu;
    bool              la_flag;
    uint8_t           la_flag_version;
    adk_ip_dp_intf_type_t    ifType;
    uint8_t           ingress_ruleset_id;
    uint8_t           vrd_id;
    uint8_t           outer_vrd_id;
    uint8_t           ingress_qos_prof_id;
    bool              enableOutAcl;
    bool              enableQoS;
    bool              enablePolicing;
    bool              routingDisabled;
    adk_ip_dp_pkt_capture_t pkt_cap_info[2];
    uint8_t           dscp_lkup[64];
    adk_qos_ds_phb_class_t dscpToPhbClass[IP_NUM_DSCP_VALUES];
}__attribute__((aligned(64))) adk_ip_dp_interface_entry_t;

/*!
 * =============================================================
 * @struct adk_ip_dp_udp_encap_info_t adk_ip_dp_export.h "adk_ip_dp_export.h"
 *
 * @brief
 *
 * Definition of adk_ip_dp_udp_encap_info_t
 *
 * =============================================================
 */
typedef struct adk_ip_dp_udp_encap_info_s {
    /*!
     * ipVrdId provides the Routing Domain ID.
     */
    uint32_t ipVrdId;
    /*!
     * destIPAddr (in nw order) to be used for IP headers encapsulation.
     */
    uint32_t destIPAddr;
    /*!
     * srcIPAddr (in nw order) to be used for IP headers encapsulation.
     */
    uint32_t srcIPAddr;
    /*!
     * dontFragment is a boolean flag that will specified in IP header.
     */
    bool dontFragment;
    /*!
     * TTL to be used for IP header encapsulation.
     */
    uint8_t ttl;
    /*!
     * Protocol value to be used for IP header encapsulation.
     */
    uint8_t proto;
    /*!
     * Flag to add UDP header.
     */
    bool addUdpHdr;
    /*!
     * destPort defines the L4 (UDP) destination port that should be
     * used for adding UDP header. This must be in nw order.
     */
    uint16_t destPort;
    /*!
     * srcPort defines the L4 (UDP) source port that should be used for
     * adding UDP header. This must be in nw order.
     */
    uint16_t srcPort;
    /*!
     * For internal use.
     */
    uint32_t hash_val;
    /*!
      * mtu defines the path mtu set by upper layer protocol
      */
    uint16_t mtu;
} adk_ip_dp_udp_encap_info_t;


#if 0
typedef struct adk_ip_dp_qos_policing_param_s
{
    uint32_t pbs_ticks;
    uint32_t cbs_ticks;
    uint16_t pir_ticks;
    uint16_t cir_ticks;
    uint32_t pir_bucket_token;
    uint32_t cir_bucket_token;
    uint32_t prev_time;
#ifdef ADK_DEBUG
    uint32_t green_pkts;
    uint32_t yellow_pkts;
    uint32_t red_pkts;
#endif
} adk_ip_dp_qos_policing_param_t;
#endif

#define ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION 7
#define ADK_IPV4_REASSEMBLY_SESSION_TBL_MAX 16000
#define ADK_IP_DP_REASS_MAX_FRAGS 64
#define ADK_IP_DP_REASM_SESSION_CACHE_SIZE 16
#define ADK_IP_DP_REASM_SESSION_TBL_NAME "adk_ip_reasm_session_tbl"

#define IP_DP_REASS_ID_WIDTH 14
#define IP_REASSEMBLY_MAX  (1<<IP_DP_REASS_ID_WIDTH)
#define IP_DP_REASS_ID_MASK ((1<<IP_DP_REASS_ID_WIDTH)-1)

typedef struct hole_info {
    uint16_t  holeStartOffset;
    uint16_t  holeEndOffset;
}holeInfo_t;

/* Maximum number of holes - this includes the one permanent hole */
#define ADK_IP_MAX_HOLES_SUPPORTED          5

typedef struct adk_ip_dp_enqd_task_list_s
{
    uint16_t frag_offset;
    adk_pktio_task_t *frag_task;
}adk_ip_dp_enqd_task_list_t;

typedef struct adk_ip_dp_reassembly_session_tbl_s
{
    uint32_t timer;
    uint32_t sip;
    uint32_t dip;
    uint16_t ip_id;
    uint8_t protocol;
    uint8_t vrd_id;
    uint8_t num_holes;
    uint8_t numEnqueues;
    uint16_t defragLength;
    uint16_t assembledBytes;
    uint8_t version;
    uint8_t queue_id;
    uint32_t flow_id;
    uint16_t ip_term_stats_id;
    holeInfo_t holes[ADK_IP_MAX_HOLES_SUPPORTED];
    adk_ip_dp_enqd_task_list_t enqd_tasks[ADK_IP_DP_REASS_MAX_FRAGS];
}adk_ip_dp_reassembly_session_tbl_t; /*TODO: look for size optimization*/

typedef struct adk_ip_dp_bucket_entry_s
{
    uint32_t hash_id;
    clear_ipsec_ctx_for_hash ipsecCb;
    adk_ip_dp_reassembly_session_tbl_t *sess_entry_p;
}adk_ip_dp_bucket_entry_t;

typedef struct adk_ip_dp_reassembly_tbl_s
{
    uint16_t count_hashid;
    rte_spinlock_t lock;
    adk_ip_dp_bucket_entry_t buckets[ADK_IPV4_REASSEMBLY_TBL_MAX_COLLISION];
}__attribute__((aligned(64))) adk_ip_dp_reassembly_tbl_t;

/*TODO overflow list*/

typedef enum adk_ip_dp_fc_table_e {
    /*! IP LLI5 Flow cache*/
    ADK_IP_DP_FC_LLI5 = 0,
    /*! IP LLI3 Flow cache */
    ADK_IP_DP_FC_LLI3,
    /*! IP ULI5 Flow cache */
    ADK_IP_DP_FC_ULI5,
    /*! IP ULI1 Flow cache */
    ADK_IP_DP_FC_ULI1,
} adk_ip_dp_fc_table_t;

typedef struct adk_ip_dp_flow_cache_entry_s
{
    void *node_ptr;
    uint32_t hash_val;
}adk_ip_dp_flow_cache_entry_t;

typedef struct adk_ip_dp_fc_count_s
{
    uint32_t timer_msec;      /*timer count in milli sec*/
    uint16_t lli5_fc_usage_count;
    uint16_t lli3_fc_usage_count;
    uint16_t uli5_fc_usage_count;
    uint16_t lli5_fc_global_free_index;
    uint16_t lli3_fc_global_free_index;
    uint16_t uli5_fc_global_free_index;
    adk_ip_dp_fc_table_t fc_table;
}__attribute__((aligned(64))) adk_ip_dp_fc_count_t;

typedef struct adk_ip_dp_fc_maintenance_s
{
    uint8_t fc_maintenance[(IP_MAX_FLOWS_PER_CPU*2)/8];
}__attribute__((aligned(64))) adk_ip_dp_fc_maintenance_t;

typedef struct adk_ip_dp_lli_hash3_tbl_entry_s
{
    uint32_t dest_addr;
    uint32_t src_addr;
    uint8_t protocol;
    uint8_t vrd_id;
    uint8_t action;
    uint32_t hit;
    uint16_t version_id;
    uint16_t cacheEntryId;
    adk_ip_dp_intf_stats_t *ip_term_stats;
    adk_flow_handle_t ipsec_data;
    void *user_data; /*TODO: use union for mem efficiency*/
    adk_pal_dp_stage_info_t stage_info;
    uint32_t destFlowId;
    uint32_t mtu;
    uint16_t egressIfId;
    uint16_t ing_flow_stats_id;
    uint32_t destModId;
    uint16_t next_free_index;
    uint16_t own_index;
    uint8_t fc_type;
    bool enableQoS;
    uint32_t interface_id;
    uint16_t dscp_lkup[MAX_DSCP_IDS];
}__attribute__((aligned(64))) adk_ip_dp_lli_hash3_tbl_entry_t;

typedef struct adk_ip_dp_lli_hash5_tbl_entry_s
{
    uint32_t dest_addr;
    uint32_t src_addr;
    uint16_t dest_port;
    uint16_t src_port;
    uint8_t protocol;
    uint8_t vrd_id;
    uint8_t action;
    uint32_t hit;
    uint16_t version_id;
    uint16_t cacheEntryId;
    adk_ip_dp_intf_stats_t *ip_term_stats;
    adk_pal_dp_stage_info_t stage_info;
    void *user_data; /*TODO: use union for mem efficiency*/
    adk_flow_handle_t ipsec_data;
    uint32_t destFlowId;
    uint32_t mtu;
    uint32_t destModId;
    uint16_t egressIfId;
    uint16_t ing_flow_stats_id;
    uint16_t next_free_index;
    uint16_t own_index;
    uint8_t fc_type;
    bool enableQoS;
    uint32_t interface_id;
    uint16_t dscp_lkup[MAX_DSCP_IDS];
}__attribute__((aligned(64))) adk_ip_dp_lli_hash5_tbl_entry_t;

typedef struct adk_ip_dp_uli_hash5_tbl_entry_s
{
    uint32_t dest_addr;
    uint32_t src_addr;
    uint16_t dest_port;
    uint16_t src_port;
    adk_pal_dp_stage_info_t stage_info;
    uint32_t destFlowId;
    uint32_t destModId;
    uint32_t hit;
    uint16_t cacheEntryId;
    uint16_t version_id;
    uint16_t egressIfId;
    uint16_t mtu;
    uint16_t next_free_index;
    uint16_t own_index;
    adk_flow_handle_t ipsec_data;
    uint8_t vrd_id;
    uint8_t protocol;
    uint8_t action;
    bool enableQoS;
    bool update_src_addr;
    uint16_t dscp_lkup[MAX_DSCP_IDS];
}__attribute__((aligned(64))) adk_ip_dp_uli_hash5_tbl_entry_t;

/*!
 * =============================================================
 * @internal
 * @struct ip_bind_data_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Bind destination information.
 * =============================================================
 */
typedef struct ip_bind_dest_info_s {
    adk_pal_dp_stage_info_t stage_info;
    uint32_t destFlowId;
    uint16_t destModId;
} ip_bind_dest_info_t;

typedef struct adk_ip_dp_nh_if_info_s {
    adk_ip_addr_t nextHop;
    uint16_t  interfaceId;
} __attribute__((__packed__))adk_ip_dp_nh_if_info_t;

typedef struct adk_ip_dp_termination_flow_info_s {
    uint32_t ip_addr_stats_id;
}__attribute__((__packed__))adk_ip_dp_termination_flow_info_t;

/*
 * This structure is used by PBR as well, positioning of union and flag
 * isTerminating should not be changed, as it is required to
 * atomically modify a PBR rule action
*/
typedef struct  adk_ip_dp_route_info_s {
    union info{
        adk_ip_dp_nh_if_info_t   nh_if;
        adk_ip_dp_termination_flow_info_t termination;
    }u;
    bool isTerminating;
    bool virtualIf;
    uint16_t routeMTU;
    uint32_t virtual_if_stats_id;
    size_t ip_term_hos_rule_id[2];
    /* We will detect implicit route modification by user and disallow it. prefixWidth is used for this */
    uint8_t prefixWidth;
}__attribute__((__packed__))adk_ip_dp_route_info_t;

typedef struct adk_ip_dp_exception_handler_s {
    /*!
     * type field decides the type of exception.
     */
    adk_ip_exception_type_t type;
    uint8_t devId;
    uint16_t queueId;
    uint8_t  priority;
    uint8_t  cbFnId;
    uint32_t policerId;
    uint32_t statsId;
} adk_ip_dp_exception_handler_t;


typedef struct adk_ip_dp_ing_flow_info_s {
    uint32_t flow_id;

#ifdef ADK_QOS_GEN3_INTERFACE
    uint32_t policerId;
    uint32_t statsId;
#endif
}adk_ip_dp_ing_flow_info_t;

typedef struct adk_ip_pbr_val_pvt_info_s
{
    uint16_t stats_id;
    adk_ip_dp_route_info_t dp_route_info;
} adk_ip_pbr_val_pvt_info_t;


typedef adk_ip_pbr_action_t adk_ip_dp_pbr_action_t;

#if 0
typedef struct adk_ip_egress_flowtbl_s
{
    uint32_t nh;
    uint32_t ifid;
    adk_flow_handle_t result;
}adk_ip_egress_flowtbl_entry_t;
#endif

typedef struct adk_ip_hash_tbl_s
{
    uint32_t hash_val;
    uint32_t flow_id;
}adk_ip_hash_tbl_t;

typedef struct adk_ip_hash_ovflow_s
{
    uint32_t flow_id;
    struct adk_ip_hash_ovflow_s *next;
}adk_ip_hash_ovflow_t;

typedef struct adk_ip_dp_eg_flow_info_s {
    uint32_t nh;
    uint32_t ifid;
}adk_ip_dp_eg_flow_info_t;

typedef struct adk_ip_hash_pending_list_s
{
    uint16_t *resource;
    adk_ip_hash_ovflow_t *ovflow_pending_p;
    uint16_t count;
}adk_ip_hash_pending_list_t;

typedef struct adk_ip_lpm_pending_list_s
{
    union {
        adk_rcu_pending_list_t *pending_list;
        uint16_t *resource;
    }u;
    uint32_t list_count;
}adk_ip_lpm_pending_list_t;

typedef struct adk_ip_dest_mtu_tbl_hdr_s
{
    uint16_t vrdid;
    uint16_t mtu;
    uint32_t dipAddr_nw;
}adk_ip_dest_mtu_tbl_t;

typedef struct adk_ip_qos_tbl_s
{
    uint32_t ifid;
    uint8_t dscp;
    uint32_t queueid;
}adk_ip_qos_tbl_t;

typedef struct adk_ip_qos_hdr_s
{
    uint32_t hash_val;
    adk_ip_qos_tbl_t *qos_entry_p;
}adk_ip_qos_tbl_hdr_t;

typedef struct adk_ip_qostbl_ovflow_s
{
    uint32_t ifid;
    uint8_t dscp;
    uint32_t queueid;
    struct adk_ip_qostbl_ovflow_s *next;
}adk_ip_qostbl_ovflow_t;

typedef struct adk_ip_dp_vrd_pbr_rulesetid_s {
       uint8_t pbr_policy_set_id;
       bool pbr_enabled;
}adk_ip_dp_vrd_pbr_rulesetid_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ip_pbr_ctx_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * PBR Policy Set information.
 * =============================================================
 */
typedef struct ip_pbr_ctx_s {
    uint32_t cookie;
    uint32_t num_pending_rule_ids;
    uint32_t pendingRuleIds[ADK_IP_MAX_PBR_RULES];
    /*Two copies: Active & Shadow*/
    adk_ip_pbr_tree_t *pbr_tree[2];
    uint8_t pbr_tree_index;
    adk_acl_handle_t cp_list;
} adk_ip_pbr_ctx_t;




/*!
 * =============================================================
 * global IP DP state information.
 * =============================================================
 */
typedef struct adk_ip_dp_tbl_info_s {
    adk_lpm_handle_t dipTbl;
    adk_ip_lpm_pending_list_t dipTbl_pending;
    adk_lpm_handle_t sipTbl;

    /*Egress Flow structures*/
    adk_ip_hash_tbl_t *ip_egress_hashtbl;
    adk_ip_hash_ovflow_t *ip_egress_ovflow_list_head;
    adk_ip_dp_vrd_pbr_rulesetid_t *ip_vrd_pbr_rule;
    adk_ip_dp_eg_flow_info_t *ip_egflow_tbl;
    adk_ip_hash_pending_list_t ip_egress_flow_pending;
    adk_ip_dest_mtu_tbl_t *ipDestMtuTbl;
    adk_ip_qos_tbl_hdr_t *ipQosTbl;
    /*Classifier tables are maintained for each IP interface separately*/
    adk_ip_egress_qos_classifer_info_t *egressQoSClassifierInfo_p[ADK_DP_MAX_IP_INTERFACE];
    adk_ip_dp_egress_qos_classifier_action_info_t *egressQoSClassifierActionDp_p[ADK_DP_MAX_IP_INTERFACE][2];
    adk_ip_qostbl_ovflow_t *ipQos_ovflow_list_head;
    adk_ip_dp_interface_entry_t *ipInterfaceTble;
    adk_ip_dp_intf_stats_t      **ipInterfaceStatsTble;
    adk_ip_dp_ip_addr_stats_t   **ipAddrStatsTble;
    adk_ip_dp_reassembly_tbl_t *ipReassemblyTbl;
    struct rte_mempool *ipReassemblySessionTbl;
    bool *vrdHostModeTble;
    adk_ip_dp_udp_encap_info_t *ipEgressAssocTbl;
    uint32_t *aclRuleStats;
    uint32_t *pbrRuleStats;
    adk_ip_pbr_ctx_t *ipPbrInfo_p;
    adk_ip_dp_ing_flow_info_t *ipIngFlowTbl;
    adk_ip_dp_ing_flow_stats_t *ipIngFlowStats;
    adk_ip_dp_cppi_stats_t *ipCppiStats;
    adk_ip_dp_pbr_action_t *pbrActionTable_p;
    adk_ip_dp_exception_handler_t *exceptionHandlerTble[ADK_IP_EXCEPTION_MAX];
    /* FIXME: to hold IP ingress classification otree information */
    adk_ip_ing_classifer_info_t *ingClassifierInfo_p;
    adk_ip_dp_ing_classifier_action_info_t *ingClassifierActionDp_p[2];
    /* CP version Id for invalidating the flow cache in data path */
    uint16_t   *ip_cp_version_id;
    adk_ip_dp_fc_count_t *fc_global_maintenance_p;
    adk_ip_dp_flow_cache_entry_t *lliCacheTbl_5_p;
    adk_ip_dp_lli_hash5_tbl_entry_t *lliNodeTbl_5_p;
    adk_ip_dp_fc_maintenance_t *lliNodeTbl_5_count_p;
    adk_ip_dp_flow_cache_entry_t *lliCacheTbl_3_p;
    adk_ip_dp_lli_hash3_tbl_entry_t *lliNodeTbl_3_p;
    adk_ip_dp_fc_maintenance_t *lliNodeTbl_3_count_p;
    adk_ip_dp_flow_cache_entry_t *uliCacheTbl_5_p;
    adk_ip_dp_uli_hash5_tbl_entry_t *uliNodeTbl_5_p;
    adk_ip_dp_fc_maintenance_t *uliNodeTbl_5_count_p;
    adk_ip_dp_global_stats_t ipGlobalStatsTable[ADK_MAX_NUM_CORES];
    /* Cookie for validation */
    uint32_t cookie;
    /*Worker List*/
    uint32_t worker_lcores_list[ADK_MAX_NUM_CORES];
    uint32_t num_worker_lcores;
    struct rte_mempool *packet_indirect_pool;
    uint8_t ipsec_bypass;
    uint32_t reasmTout;
    uint8_t reasmPrcntToutChange;
    uint32_t reasmMinTout;
    uint32_t reasmStage1;
    uint32_t reasmStage2;
    uint32_t reasmStage3;
    uint32_t reasmStage4;
    uint64_t  allowedL4protos[4];
    /* This array will be indexed by l2InterfaceId similar to how we index interface table in DP */
    adk_interface_handle_t *ifaceHandles;
    uint8_t  defRouteTblId;
    adk_ip_dp_l4_stats_t ipL4Stats[ADK_MAX_NUM_CORES];
    bool utilCntrEnabled;
} adk_ip_dp_tbl_info_t;


adk_dp_status_t adk_ip_dp_table_init(adk_module_handle_t ipHandle);

adk_dp_status_t adk_ip_dp_table_shutdown(adk_module_handle_t ipHandle);

adk_dp_status_t adk_ip_dp_interface_add(adk_module_handle_t ipHandle,
                                        adk_interface_handle_t ipIfHdl,
                                        uint16_t stats_id,
                                        uint32_t sys_egress_stats_id);

adk_dp_status_t adk_ip_dp_interface_remove(adk_module_handle_t ipHandle,
                                           adk_interface_handle_t ipIfHdl);

adk_dp_status_t adk_ip_dp_interface_stats_get(adk_module_handle_t ipHandle,
                                              adk_interface_handle_t ipIfHdl,
                                              adk_ip_interface_stats_t *stats_p);

adk_status_t adk_ip_dp_interface_stats_clear(adk_module_handle_t ipHandle);

adk_dp_status_t
adk_ip_dp_virtual_if_stats_get(adk_module_handle_t ipHandle,
                               uint16_t ip_stats_id,
                              adk_ip_interface_stats_t *stats_p);

adk_dp_status_t adk_ip_dp_qos_policy_add(adk_module_handle_t ipHandle,
                                         uint32_t ipIfId, uint8_t dscp,
                                         uint32_t queueId);

adk_dp_status_t adk_ip_dp_qos_policy_delete(adk_module_handle_t ipHandle,
                                         uint32_t ipIfId, uint8_t dscp);

adk_dp_status_t adk_ip_dp_dip_add(adk_module_handle_t ipHandle,
                                  uint8_t vrd_id,
                                  uint32_t dip, uint8_t mask,
                                  adk_handle_t dipResult);

adk_dp_status_t adk_ip_dp_dip_hw_rule_remove(adk_module_handle_t ipHandle,
                                     uint8_t vrd_id,
                                     uint32_t dip,
                                     uint8_t mask);

adk_dp_status_t adk_ip_dp_dip_remove(adk_module_handle_t ipHandle,
                                     uint8_t vrd_id,
                                     uint32_t dip,
                                     uint8_t mask);

adk_dp_status_t adk_ip_dp_diptbl_key_create(uint8_t vrd_id,
                                            uint32_t dip,
                                            uint8_t mask,
                                            unsigned char *key,
                                            int *size);

adk_dp_status_t adk_ip_dp_sip_add(adk_module_handle_t ipHandle,
                                  uint8_t vrd_id,
                                  uint32_t sip, uint8_t mask,
                                  adk_handle_t sipResult);

adk_dp_status_t adk_ip_dp_sip_remove(adk_module_handle_t ipHandle,
                                     uint8_t vrd_id,
                                     uint32_t sip,
                                     uint8_t mask);

adk_dp_status_t adk_ip_dp_siptbl_key_create(uint8_t vrd_id,
                                            uint32_t sip,
                                            uint8_t mask,
                                            unsigned char *key,
                                            int *size);

adk_dp_status_t adk_ip_dp_egress_flow_add(adk_module_handle_t ipHandle,
                                          uint32_t l2If,
                                          uint32_t nh,
                                          uint32_t flow_id);

adk_dp_status_t adk_ip_dp_egress_flow_remove(adk_module_handle_t ipHandle,
                                             uint32_t ipIfId,
                                             uint32_t nh,
                                             uint32_t flow_id);

static inline void
adk_ip_lpm_clear_pending_list(adk_rm_hdl_t ipRmHandle,
				adk_ip_lpm_pending_list_t *ipDpPendingList_p)
{
    uint32_t i;
    for(i = 0; i < ipDpPendingList_p->list_count; i++)
        adk_rm_resource_free(ipRmHandle, ipDpPendingList_p->u.resource[i]);

    ipDpPendingList_p->list_count = 0;
}

static inline void
adk_ip_clear_pending_list(adk_rm_hdl_t ipRmHandle,
				adk_ip_hash_pending_list_t *ipDpPendingList_p)
{
    uint32_t i;
    adk_ip_hash_ovflow_t *current, *temp;
    for(i = 0; i <ipDpPendingList_p->count; i++)
    {
        adk_rm_resource_free(ipRmHandle, ipDpPendingList_p->resource[i]);
    }
    /*Handle overflow pending list*/
    if(ipDpPendingList_p->ovflow_pending_p)
    {
        current = ipDpPendingList_p->ovflow_pending_p;
        temp = NULL;
        while(current)
        {
            temp = current->next;
            ADK_DP_SHM_FREE(current);
            current = temp;
        }
    }
    ipDpPendingList_p->ovflow_pending_p = NULL;
    ipDpPendingList_p->count = 0;
}

static inline void
adk_ip_pushto_pending_list(adk_ip_hash_ovflow_t *current,
        adk_ip_hash_pending_list_t *pending_list)
{
    /*Move deleted entry to pending list*/
    pending_list->resource[pending_list->count] = current->flow_id;
    pending_list->count++;

    /*Update Overflow Pending list*/
    if(NULL == pending_list->ovflow_pending_p)
    {
        pending_list->ovflow_pending_p = current;
        current->next = NULL;
    }
    else
    {
        /*Insert current node at head of the list*/
        current->next = pending_list->ovflow_pending_p;
        pending_list->ovflow_pending_p = current;
    }
    /*Update RCU version*/
    adk_rcu_distributed_ver_update();
}

void
adk_ip_lpm_rcu_distributed_check(adk_rm_hdl_t ipRmHandle,
   adk_ip_lpm_pending_list_t *ipDpPendingList_p, uint32_t flow_id);

void
adk_ip_rcu_distributed_check(adk_rm_hdl_t ipRmHandle,
   adk_ip_hash_pending_list_t *ipDpPendingList_p, uint32_t flow_id);

void adk_ip_dp_egressflowtbl_key_create(uint32_t l2If,
			                uint32_t nh,
			                uint8_t *key);

adk_dp_status_t
adk_ip_dp_dest_mtu_add(adk_module_handle_t ipHandle,
            uint16_t ipVrdId, uint32_t dipAddr,
            uint16_t mtu);

adk_dp_status_t
adk_ip_dp_dest_mtu_rem(adk_module_handle_t ipHandle,
                          uint16_t ipVrdId,
                          uint32_t dipAddr);

adk_dp_status_t adk_ip_dp_vrd_host_mode(adk_module_handle_t ipHandle,
                                        uint8_t vrd_id,
                                        bool host_mode);

adk_dp_status_t
adk_ip_dp_ip_addr_stats_get(adk_module_handle_t ipHandle,
                              uint16_t ip_addr_stats_id,
                              adk_ip_address_stats_t *stats_p);

adk_dp_status_t
adk_ip_dp_ip_addr_stats_clear(adk_module_handle_t ipHandle);

adk_dp_status_t
adk_ip_dp_global_stats_get(adk_module_handle_t ipHandle,
                              adk_ip_global_stats_t *stats_p);

adk_dp_status_t
adk_ip_dp_global_stats_clear(adk_module_handle_t ipHandle);

adk_dp_status_t
adk_ip_dp_l4_stats_get(adk_module_handle_t ipHandle,
                              adk_ip_l4_stats_t *stats_p);

adk_dp_status_t
adk_ip_dp_l4_stats_clear(adk_module_handle_t ipHandle);

adk_dp_status_t
adk_ip_dp_utility_stats_get(adk_module_handle_t ipHandle,
                              adk_ip_utility_stats_t *stats_p);

adk_dp_status_t
adk_ip_dp_utility_stats_clear(adk_module_handle_t ipHandle);


adk_status_t
adk_ip_dp_ingress_classifier_stats_get(adk_module_handle_t ipHandle,
    uint32_t classFlowId,
        adk_ip_ingress_classifier_stats_t *stats_p);

adk_status_t
adk_ip_dp_ingress_classifier_stats_clear(adk_module_handle_t ipHandle);

adk_status_t
adk_ip_dp_get_reassembly_hash(adk_pktio_task_t **ph_task_p,
        uint16_t ip_offset, uint32_t *hash_val);

void
adk_ip_dp_egress_stats(struct rte_mbuf *mbuf, void *data);

void adk_ip_dp_slowpath_policer_stats(void *data);
#endif
