/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IP_DP_EXPORT_H__
#define __ADK_IP_DP_EXPORT_H__

/*!
 * =============================================================
 * @struct adk_ip_dp_lli_imc_data_t adk_ip_dp_export.h "adk_ip_dp_export.h"
 *
 * @brief 
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPv4 module LLI for the purpose of packet processing.
 *
 * =============================================================
 */
typedef struct adk_ip_dp_lli_imc_data_s
{
     /* DSCP remapping profile ID received from Ethernet */
    uint8_t profile_id;
     /* PCP value received from Ethernet */
    uint8_t pcp;

    /* This flag conveys 3 pieces of information to IP LLI:
     *  - where the packet is coming from (Ethernet/IPSec)
     *  - Is it the outer or inner packet
     *  - When was the decryption done (inline or look-aside)
     *  - Possible values IPLLI_IMC_ETH_OUTER, IPSEC_DONE_PLAINTEXT_PACKET, IPSEC_DONE_PACKET_DECRYPTED_INLINE, IPSEC_DONE_PACKET_DECRYPTED_LOOKASIDE
     */
    uint8_t ipsec_data; /* 1B */

}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_lli_imc_data_t;


#define IP_ULI_SENDER_EO    0x01
#define IP_ULI_SENDER_NETD  0x02
#define IP_ULI_SENDER_IPSEC 0x03
#define IP_ULI_SENDER_IPLLI 0x04

/*!
 * =============================================================
 * @struct adk_ip_dp_uli_imc_data_t adk_ip_dp_export.h "adk_ip_dp_export.h"
 *
 * @brief 
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPv4 module ULI for the purpose of packet processing.
 *
 * =============================================================
 */
typedef struct adk_ip_dp_uli_imc_data_s
{
    /* Value can be : IP_ULI_SENDER_EO, IP_ULI_SENDER_NETD, IP_ULI_SENDER_IPSEC,
     * IP_ULI_SENDER_IPLLI */
    uint8_t sender;

    /* IP needs original sender info to decide what action to take regarding udp
     * checksum generation.
     * IP will send this info to ipsec and ipsec should set it back.
     * Valid values: IP_ULI_SENDER_EO, IP_ULI_SENDER_NETD, IP_ULI_SENDER_IPLLI
     */
    uint8_t pktOrigin;

    /* In outbound direction, if the IPsec SA mode configuration is inline, then 
     * IPsec will be setting do_not_post_frag flag in IP IMC because inline mode and
     * postFrag won’t go together. IP can do fragmentation on the packet in ULI only
     * when this flag is not set. If do_not_post_frag flag is set and fragmentation has 
     * to be done on packet in IP, then packet has to be dropped.
     */
    bool do_not_post_frag;

    /* L4 protocol of inner IP header.
     * This data would be filled up by IPsec ULI.
     * IP module should not use inner_l4_protocol if innerIPVersion in adk_metadata_t
     * is 0.
     */
    uint8_t inner_l4_protocol;

    /* Ethernet expects number of fragments created in its IMC.
     * In pre-frag case, this is generated by IPSec.
     * IPSec should set it in IP IMC and IP will send it to ethernet
     */
    uint8_t frag_count;

}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_uli_imc_data_t;

/*!
 * =============================================================
 * @enum adk_ip_dp_frag_error_e
 *
 * @brief
 *
 * This enum defines ipv4 fragmentation error types.
 *
 * =============================================================
 */
typedef enum adk_ip_dp_frag_error_e
{
    ADK_IP_DP_FRAG_OK = 0,
    ADK_IP_DP_FRAG_DFSET,
    ADK_IP_DP_FRAG_LIMIT_EXCEED,
    ADK_IP_DP_FRAG_MEM_ALLOC_FAILED
} adk_ip_dp_frag_error_t;

adk_status_t adk_ip_dp_pkt_proc_lli (void **task_p,
                      uint8_t task_count, uint32_t task_mask, void *ring, 
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

adk_status_t adk_ip_dp_pkt_proc_uli (void **task_p,
                      uint8_t task_count, uint32_t task_mask, void *ring,
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

adk_status_t adk_ip_dp_pkt_proc_netd (void **task_p,
          uint8_t task_count, uint32_t task_mask, void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

adk_status_t adk_ip_dp_pkt_proc_em_uli(void ** arg);

adk_status_t adk_ip_dp_pkt_proc_pcap (void **task_p,
          uint8_t task_count, uint32_t task_mask, void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

adk_ip_dp_frag_error_t
adk_ip_dp_fragment(adk_pktio_task_t **task_out_p,
        uint32_t mtu, uint8_t *num_frags, uint16_t *total_payload,
        uint8_t genL4PartialCksum);

inline uint16_t ipv4_udp_checksum_gen(adk_pktio_task_t *task,
                           uint16_t udp_len, uint32_t src_addr_nw, uint32_t dest_addr_nw);

#ifdef IP_FLOW_HASH_CACHE
void
adk_ip_dp_flow_cache_monitor(void);
#endif

#ifdef UDP_CHECKSUM_STAGE_AFTER_BB_IC_MODULE
adk_status_t adk_udp_dp_pkt_proc_uli (void **task_p,
                        uint8_t task_count,
                        uint32_t task_mask,
                        __attribute__((unused)) void *ring,
                        __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        int coreIndex);

adk_status_t adk_udp_dp_pkt_proc_ring_uli (void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask,
                        void *ring,
                        adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);
#endif

/* This callback will be called whenever timer started from IPSec is deleted.
 * res_index and bucket_index should be used to identify exact reassembly context in IPSec.
 * IP will acquire lock before calling this callback.
 */
typedef uint16_t (*clear_ipsec_ctx_for_hash)(uint16_t res_index, uint8_t bucket_index);

/* adk_ip_dp_ipsec_start_reassembly : This API will find the exact bucket for a given res_index and start the timer.
 * Prerequisite: Reassembly lock should be taken through exposed API adk_ip_dp_acquire_reassembly_lock_non_atomic before calling this function.
 *
 * On success, IP will return the bucket index that should be used by ipsec for flow related metadata and parking the tasks.
 * If return value is not ADK_SUCCESS, bucket_index will be invalid and shouldn't be used for any
 * further processing.
 *
 *  [IN] hash_val   : hash value. caller should get the value by calling adk_ip_dp_get_reassembly_hash_row() before calling this API.
 *  [IN] res_index  : Reassembly index. User should call adk_ip_dp_get_reassembly_hash() to get the value and pass it here.
 *  [IN] ip_offset  : Will be used internally to access ip header.
 *  [IN] ipsecCb    : This callback will be called on timeout or when reassembly is complete.
 *  [IN] preFragBufferTimeOutVal : If packet is non-terminating, this value will be used for timer (value in milliseconds).
 *  [OUT] bucket_index : This API will populate this field with bucket_index used by IP.
 */
adk_status_t adk_ip_dp_ipsec_start_reassembly(uint16_t coreIndex,
        adk_pktio_task_t **ph_task_p,
        uint32_t hash_val,
        uint16_t res_index,
        uint16_t ip_offset,
        clear_ipsec_ctx_for_hash ipsecCb,
        uint32_t preFragBufferTimeOutVal,
        uint8_t *bucket_index
    );


inline void adk_ip_dp_acquire_reassembly_lock_non_atomic(
        adk_pktio_task_t *ph_task,
        uint16_t res_index
);

/* adk_ip_dp_release_reassembly_lock_non_atomic : This API should be called to release the lock that was
 * taken using adk_ip_dp_acquire_reassembly_lock_non_atomic()
 */
inline void adk_ip_dp_release_reassembly_lock_non_atomic(
        adk_pktio_task_t *ph_task,
        uint16_t res_index
);

/* adk_ip_dp_get_reassembly_hash_row: Get hash_value and reassembly index for the flow
 * [IN] ip_offset : Ip header offset
 * [OUT] hash_value : Hash value for the flow
 * [OUT] res_index  : Reassembly table index for this flow
 */
adk_status_t adk_ip_dp_get_reassembly_hash_row(adk_pktio_task_t **ph_task_p,
        uint16_t ip_offset, uint32_t *hash_val, uint16_t *res_index);

void adk_ip_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);
#endif /* __ADK_IP_DP_EXPORT_H__ */
