/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef __ADK_IP_DP_INTERNAL_H__
#define __ADK_IP_DP_INTERNAL_H__

#define IS_MCAST(ipHdr) ((ipHdr->dst_addr&0x000000f0) == 0xE0)
#define IS_BCAST(ipHdr) (ipHdr->dst_addr == 0xFFFFFFFF)
#define IS_MBCAST(ipHdr) ( IS_MCAST(ipHdr) ||  IS_BCAST(ipHdr) )
#define IP_ADDR_ZERO 0

/*Module info pointer stored for DP process to access tables*/
extern adk_ip_dp_tbl_info_t *ip_dp_tbl_info_p;

/*Reassembly table accessed only by dp process*/
extern adk_ip_dp_reassembly_tbl_t *ipReassemblyTbl;
extern struct rte_mempool *ipReassemblySessionTbl;

/*ACL per rule counter*/
extern uint32_t *pbrRuleStats;

/*PBR action table*/
extern adk_ip_dp_pbr_action_t *pbrActionTable_p;

/*Per VRD, CP version id*/
extern uint16_t *ip_cp_version_id_p;

/* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
extern uint64_t ip_uli_rx_em_init, ip_uli_tx_em_init;
extern uint64_t ip_uli_rx_ts_1, ip_uli_tx_ts_1;
extern uint64_t ip_uli_rx_ts_2, ip_uli_tx_ts_2;
extern uint64_t ip_lli_rx_ts_1, ip_lli_tx_ts_1;
extern uint64_t ip_lli_rx_ts_2, ip_lli_tx_ts_2;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_1() adk_dp_measure_block_cycle_rx_ts(&ip_lli_rx_ts_1)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_1() adk_dp_measure_block_cycle_tx_ts(&ip_lli_rx_ts_1, &ip_lli_tx_ts_1, "IP LLI MODULE PASS 1", PMU_IP_LLI_MODULE_PASS_1)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2() adk_dp_measure_block_cycle_rx_ts(&ip_lli_rx_ts_2)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_2() adk_dp_measure_block_cycle_tx_ts(&ip_lli_rx_ts_2, &ip_lli_tx_ts_2, "IP LLI MODULE PASS 2", PMU_IP_LLI_MODULE_PASS_2)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_TS_1() adk_dp_measure_block_cycle_rx_ts(&ip_uli_rx_ts_1)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_TS_1() adk_dp_measure_block_cycle_tx_ts(&ip_uli_rx_ts_1, &ip_uli_tx_ts_1, "IP ULI MODULE PASS 1", PMU_IP_ULI_MODULE_PASS_1)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_TS_2() adk_dp_measure_block_cycle_rx_ts(&ip_uli_rx_ts_2)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_TS_2() adk_dp_measure_block_cycle_tx_ts(&ip_uli_rx_ts_2, &ip_uli_tx_ts_2, "IP ULI MODULE PASS 2", PMU_IP_ULI_MODULE_PASS_2)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_EM_INIT_TS() adk_dp_measure_block_cycle_rx_ts(&ip_uli_rx_em_init)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_EM_INIT_TS() adk_dp_measure_block_cycle_tx_ts(&ip_uli_rx_em_init, &ip_uli_tx_em_init, "IP ULI MODULE EM PATH INIT", PMU_IP_ULI_MODULE_EM_PATH_INIT)

#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_1()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_1()

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_RX_TS_2()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_LLI_TX_TS_2()

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_TS_1()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_TS_1()

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_TS_2()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_TS_2()

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_RX_EM_INIT_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IP_ULI_TX_EM_INIT_TS()

#endif /* BLOCK_COUNT_ENABLED */

#define MAX_DSCP_VALUES    64
#define INVALID_DSCP_ID    21 /*0 - 20 valid ids*/

#define ADK_IP_DP_LOG_FATAL_ERROR(__msg) \
         ADK_DP_LOG_FATAL_ERROR(ADK_IP_MODULE_ID, __msg)

#define ADK_IP_DP_LOG_INFO(__msg) ADK_DP_LOG_INFO(ADK_IP_MODULE_ID, __msg)

#define ADK_IP_DP_LOG_INFO_1D(__msg, __data) \
         ADK_DP_LOG_INFO_1D(ADK_IP_MODULE_ID, __msg, __data)

#define ADK_IP_DP_LOG_INFO_DEBUG debug_print

/* Fragment Offset */
#define IPV4_HDR_DF_SHIFT           14
#define IPV4_HDR_MF_SHIFT           13
#define IPV4_HDR_FO_SHIFT           3

#define IPV4_HDR_DF_MASK            (1 << IPV4_HDR_DF_SHIFT)
#define IPV4_HDR_MF_MASK            (1 << IPV4_HDR_MF_SHIFT)
#define IPV4_HDR_FRAG_MASK          0x3FFF

#define IP_PROTO_UDP   17
#define IP_PROTO_ESP   50
#define IP_PROTO_AH    51
#define IP_HDR_LEN 20
#define IP_UDP_HDR_LEN 28
#define UDP_HDR_LEN 8

#define IP_DP_LLI_ACTION_DONE 0 
#define IP_DP_LLI_ACTION_ING_ASSOC 1 
#define IP_DP_LLI_ACTION_SLOW_PATH 2

#define DSCP_ENTRY_PRESENT_MASK 0x8000
#define ACL_STATS_ID_WIDTH 10
#define ACL_STATS_ID_MASK ((1 << ACL_STATS_ID_WIDTH)-1)
#define REMAPPED_PCP_MASK 0x7000 

#define REASM_ENQ_RETRY_COUNT 0

#define ADK_IP_DP_HASH_REASSEMBLE 1
#define ADK_IP_DP_HASH_INGRESS_ASSOC_BIND 2
#define ADK_IP_DP_HASH_FWD_BIND 4
#define ADK_IP_DP_HASH_INGRESS_ASSOC_INVOKE_CB 8

#if defined(ADK_DEBUG)

#define ADK_IP_DP_LOG_IP_HDR(ipHdr) do{\
    int i=0;\
    unsigned char *hdr = (unsigned char *)ipHdr;\
    ADK_IP_DP_LOG_INFO_DEBUG("IP HDR: ");\
    for(i=0;i<IP_HDR_LEN;i++)\
    {\
        ADK_IP_DP_LOG_INFO_DEBUG("%02x ", hdr[i]);\
    }\
    ADK_IP_DP_LOG_INFO_DEBUG("\n");\
}while(0);
#else
#define ADK_IP_DP_LOG_IP_HDR(ipHdr)
#endif

typedef struct _ipIdSeqRange_t {
	uint16_t min;
	uint16_t max;
} ipIdSeqRange_t;

typedef struct _ipIdSeqDB_t {
	uint16_t ipIdSeq;
}__attribute__((aligned(64))) ipIdSeqDB_t;

extern ipIdSeqDB_t ipIdSeqDB[ADK_MAX_NUM_CORES];
extern ipIdSeqRange_t ipIdSeqRange[ADK_MAX_NUM_CORES];

extern uint8_t ip_dscp_map[MAX_DSCP_VALUES];

typedef struct adk_ip_dp_lli_internal_mdata_s
{
    bool is_reassembled;
} __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_lli_internal_mdata_t;

typedef struct adk_ip_dp_reass_timer_mdata_s
{
    uint16_t ip_term_stats_id;
    uint16_t collisionIndex;
    uint16_t res_index;
    uint16_t queue_id;
    uint32_t flow_id;
    uint8_t version;
} __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_reass_timer_mdata_t;

typedef struct adk_ip_dp_reass_meta_data_s
{
    uint16_t ip_term_stats_id;
    uint8_t vrdid;
    uint8_t ruleset_id;
    uint32_t hash_val;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_reass_meta_data_t;

typedef struct adk_ip_dp_policer_meta_data_s
{
    adk_ip_dp_ing_classifier_action_info_t ing_action;
    uint32_t ingress_classifier_flow_id;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ip_dp_policer_meta_data_t;

/*!
 * =============================================================
 * @enum adk_ip_dp_pkt_fc_type
 *
 * @brief
 *
 * This enum defines ipv4 packet fragmentation type
 *
 * =============================================================
 */

typedef enum adk_ip_dp_pkt_fc_type
{
    ADK_IP_DP_PKT_UNFRAGMENTED = 0,
    ADK_IP_DP_PKT_FIRST_FRAGMENT,
    ADK_IP_DP_PKT_NON_FIRST_FRAGMENT
} adk_ip_dp_pkt_fc_type_t;

/*!
 * =============================================================
 * @enum adk_ip_dp_pkt_error_e
 *
 * @brief
 *
 * This enum defines ipv4 packet error types.
 *
 * =============================================================
 */

typedef enum adk_ip_dp_pkt_error_e
{
    ADK_IP_DP_PKT_OK = 0,
    ADK_IP_DP_PKT_BAD_VERSION,
    ADK_IP_DP_PKT_BAD_LENGTH,
    ADK_IP_DP_PKT_TRUNCATED,
} adk_ip_dp_pkt_error_t;


/*!
 * =============================================================
 * @enum adk_ip_dp_reassembly_error_e
 *
 * @brief
 *
 * This enum defines ipv4 reassembly error types.
 *
 * =============================================================
 */
typedef enum adk_ip_dp_reassembly_error_e
{
    /*ADK_IP_DP_REASM_PKT_MISMATCH=1,*/
    ADK_IP_DP_REASM_HOLES_EXCEEDED = 0,
    ADK_IP_DP_REASM_DUP_BYTES,
    ADK_IP_DP_REASM_ENQ_DONE,
    ADK_IP_DP_REASM_ENQ_COPY
}adk_ip_dp_reasm_error_t;

adk_ip_dp_pkt_error_t
adk_ip_dp_extract_ip_l4_hdr(struct ipv4_hdr *iphdr,
                            uint16_t *src_port_nw, uint16_t *dest_port_nw,
                            uint16_t pkt_size, uint16_t *ip_length
#ifdef UDP_CHECKSUM_ENABLED
                            ,uint16_t *udp_length
#endif
                           );

adk_status_t adk_ip_dp_termination_classifer_path(adk_pktio_task_t *ph_task,
        int coreIndex
);

adk_ip_dp_reasm_error_t
adk_ip_dp_reasm_sessionTbl_update(
            adk_ip_dp_reassembly_session_tbl_t *session_entry_p,
            adk_pktio_task_t *frag_task);

adk_status_t
adk_ip_dp_pkt_proc_ring_lli (void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask, 
                        void *ring, adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);

adk_status_t adk_ip_dp_pkt_proc_lli_single (void **task_p,
                                 int coreIndex,
                                 adk_pal_dp_stage_info_t **stage_info_p);

adk_status_t
adk_ip_dp_pkt_proc_ring_uli (void **task_p,
            uint8_t __attribute__((unused)) task_count,
            uint32_t task_mask,
            void *ring, adk_sif_cfg_pipeline_info_t *next_stage_entry,
            __attribute__((unused)) int coreIndex);


#ifdef UDP_CHECKSUM_STAGE_AFTER_IP
adk_status_t
adk_udp_dp_pkt_proc_lli (void **task_p,
            uint8_t task_count, uint32_t task_mask,
            __attribute__((unused)) void *ring,
            __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
            int coreIndex);

adk_status_t
adk_udp_dp_pkt_proc_ring_lli (void **task_p,
			uint8_t __attribute__((unused)) task_count,
			uint32_t task_mask,
			__attribute__((unused)) int coreIndex);
#endif

adk_status_t
adk_ip_dp_pkt_proc_reassemble (void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

adk_status_t
adk_ip_dp_pkt_proc_reasm_tout(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

adk_status_t
adk_ip_dp_reassemble_internal(uint16_t coreIndex, adk_pktio_task_t **ph_task_p,
                uint8_t vrd_id, uint16_t term_stats_id, uint32_t hash);


adk_status_t
adk_ip_dp_pkt_proc_police (void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          __attribute__((unused)) void *ring,
          __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

adk_status_t adk_ip_dp_send_exception(adk_pktio_task_t *ph_task,
                                        uint8_t type, int coreIndex,
                                        uint8_t routeTableId, adk_ip_direction_t dir);

bool adk_ip_dp_pbr( uint32_t src_addr_nw, adk_ip_pbr_val_t *pbr_val);

adk_dp_status_t
adk_ip_dp_qos_policy_get(uint32_t ipIfId, uint8_t dscp,
                                         uint32_t *queueId);

adk_dp_status_t
adk_ip_dp_dip_get(uint8_t vrd_id,
                                  uint32_t dip,
                                  adk_handle_t *dipResult);

adk_dp_status_t
adk_ip_dp_sip_get(uint8_t vrd_id,
                                  uint32_t sip,
                                  adk_handle_t *sipResult);
adk_dp_status_t
adk_ip_dp_egress_flow_get(uint32_t l2If,
			uint32_t nh, adk_ip_dp_eg_flow_info_t **result);

adk_dp_status_t
adk_ip_dp_dest_mtu_get(uint16_t ipVrdId,
                       uint32_t dipAddr,
                       uint16_t *mtu_p);

void
adk_ip_dp_reassemblytbl_key_create(uint16_t ip_id,
			uint8_t protocol, uint32_t src_addr,
			uint32_t dest_addr, uint8_t *key);


adk_status_t
adk_ip_dp_fn_id_init(void);


void
reassembly_timer_cb_fn(void *reasm_col_row_index,
                   adk_timer_event_type_t event_type);

struct
rte_mbuf* adk_pktmbuf_clone(struct rte_mbuf * inputPkt,
                            struct rte_mempool * pool);

#if 0
adk_dp_status_t
adk_ip_dp_police_action( uint8_t policeColor,
                         adk_ip_qos_policing_params_t *qos_police_params_p);

adk_dp_status_t adk_ip_dp_qos_trtcm_params_gen(
                              uint32_t cir,
                              uint32_t pir,
                              uint32_t cbs,
                              uint32_t pbs,
                              uint32_t clockSpeed,
                              adk_ip_dp_qos_policing_param_t *policingParams_p);

adk_dp_status_t adk_ip_dp_qos_trtcm_color_blind(
                             adk_ip_dp_qos_policing_param_t *dpPoliceParams_p,
                             uint32_t pduSize, uint8_t * policeColor);
#endif

/*UDP cksum verification with H/W no offload*/
static inline
uint16_t ipv4_udp_checksum_no_ol(adk_pktio_task_t *task,
                           uint16_t udp_len, 
                           uint32_t src_addr_nw, 
                           uint32_t dest_addr_nw)
{
    uint32_t cksum=0;
    struct ipv4_psd_header {
            uint32_t src_addr; /* IP address of source host. */
            uint32_t dst_addr; /* IP address of destination host. */
            uint8_t  zero;     /* zero. */
            uint8_t  proto;    /* L4 protocol type. */
            uint16_t len;      /* L4 length. */
    } psd_hdr;
    uint8_t *buff;

    uint16_t total_len=0 ,seg_len = (task->data_len - ADK_MD_GET_L4_OFF(task));
    ADK_PDU_OFFSET_PTR_GET(task, buff, uint8_t *, ADK_MD_GET_L4_OFF(task));
    /*Cksum 0 is valid*/
    if(0 == *((uint16_t *)&(buff[6])))
        return 0;
    if(likely(udp_len <= seg_len))
    {
        cksum += adk_raw_cksum((void *)buff, udp_len);
    }
    else
    {
        bool odd_len = false;
        /*Multi segmented mbuf*/
        while((total_len+seg_len) < udp_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)buff, seg_len));
                odd_len = ((seg_len-1) % 2);
            }
            else
            {
                cksum += adk_raw_cksum((void *)buff, seg_len);
                odd_len = (seg_len % 2);
            }
            total_len+=seg_len;
            /*Access next segment*/
            task = (adk_pktio_task_t *)(task->next);
            buff = rte_pktmbuf_mtod(task, uint8_t *);
            seg_len = task->data_len;
        }
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)buff,
                                               (udp_len - total_len)));
        }
        else
        {
            cksum += adk_raw_cksum((void *)buff, (udp_len - total_len));
        }
    }
    buff = (uint8_t *)&psd_hdr;
    psd_hdr.src_addr = src_addr_nw;
    psd_hdr.dst_addr = dest_addr_nw;
    psd_hdr.zero = 0;
    psd_hdr.proto = 17;
    psd_hdr.len = rte_cpu_to_be_16(udp_len);
    cksum += adk_raw_cksum((void *)buff, sizeof(psd_hdr));
    cksum = adk_raw_cksum_reduce(cksum);
    cksum = (~cksum) & 0xffff;
    return cksum;
}


static inline
uint16_t ipv4_udp_checksum_ol(
        __attribute__((unused)) adk_pktio_task_t *task,
        __attribute__((unused)) uint16_t udp_len,
        __attribute__((unused)) uint32_t src_addr_nw,
        __attribute__((unused)) uint32_t dest_addr_nw)
{
#if 0
    /* TO DO: We may not need is_reassembled flag. Review and remove. refer no_ol api for better handling */
    if( ((adk_ip_dp_lli_internal_mdata_t *)
         (&(ADK_MD_GET_IMC_PARAM_START(task))))->is_reassembled )
    {
        ADK_PDU_OFFSET_PTR_GET(task, buff, uint8_t *, ADK_MD_GET_L4_OFF(task));

        /*Cksum 0 is valid*/
        pkt_cksum = *((uint16_t *)&(buff[6]));
        if(0 == pkt_cksum)
        {
            return 0;
        }
        
        /*Partial cksum available for the head fragment*/
        ADK_IP_DP_LOG_INFO_DEBUG("dsi_md_p->checksum=%d\n", dsi_md_p->checksum);
        cksum = rte_cpu_to_be_16(dsi_md_p->checksum) + pkt_cksum;
        seg_len = (task->data_len - ADK_MD_GET_L4_OFF(task));
        total_len = seg_len;
        /*Access next segment*/
        task = (adk_pktio_task_t *)(task->next);
        buff = rte_pktmbuf_mtod(task, uint8_t *);
        seg_len = task->data_len;

        bool odd_len = false;
        /*Multi segmented mbuf*/
        while((total_len+seg_len) < udp_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)buff, seg_len));
                odd_len = ((seg_len-1) % 2);
            }
            else
            {
                cksum += adk_raw_cksum((void *)buff, seg_len);
                odd_len = (seg_len % 2);
            }
            total_len+=seg_len;
            /*Access next segment*/
            task = (adk_pktio_task_t *)(task->next);
            buff = rte_pktmbuf_mtod(task, uint8_t *);
            seg_len = task->data_len;
        }
        
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)buff,
                                               (udp_len - total_len)));
        }
        else
        {
            cksum += adk_raw_cksum((void *)buff, (udp_len - total_len));
        }

        buff = (uint8_t *)&psd_hdr;
        psd_hdr.src_addr = src_addr_nw;
        psd_hdr.dst_addr = dest_addr_nw;
        psd_hdr.zero = 0;
        psd_hdr.proto = ADK_IP_PROTOCOL_UDP;
        psd_hdr.len = rte_cpu_to_be_16(udp_len);
        cksum += adk_raw_cksum((void *)buff, sizeof(psd_hdr));
        cksum = adk_raw_cksum_reduce(cksum);
        ADK_IP_DP_LOG_INFO_DEBUG("calculated cksum=%d\n", cksum);
        cksum = (~cksum) & 0xffff;

        return cksum;
    }
    else
    {
        /*Cksum result available from h/w for non-fragmented pkts*/
        if(adk_hos_is_l4_chksum_err(dsi_md_p))
            return 1;
        else
            return 0;
    }

#else
    return 0;
#endif
}


/* Verify Rx UDP checksum
 * Return value:
 * 0: Good cksum
 * non-0: Bad cksum
 */
static inline
uint16_t ipv4_udp_checksum(adk_pktio_task_t *task,
                           uint16_t udp_len,
			   uint32_t src_addr_nw,
			   uint32_t dest_addr_nw)
{
    /* TO DO: We have observed that the partial checksum by HW wrong
     * for fragment length > 2000. Hence falling back to SW for such reassembled
     * packets.
     */
    if(task->data_len <= 2000)
    {
        uint32_t cksum = 0;
        struct ipv4_psd_header
        {
            uint32_t src_addr; /* IP address of source host. */
            uint32_t dst_addr; /* IP address of destination host. */
            uint8_t zero;      /* zero. */
            uint8_t proto;     /* L4 protocol type. */
            uint16_t len;      /* L4 length. */
        } psd_hdr;
        uint8_t *buff;
        uint16_t total_len = 0, seg_len, pkt_cksum;
        adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);

        ADK_PDU_OFFSET_PTR_GET(task, buff, uint8_t *, ADK_MD_GET_L4_OFF(task));

        /*Cksum 0 is valid*/
        pkt_cksum = *((uint16_t *)&(buff[6]));
        if (0 == pkt_cksum)
        {
            return 0;
        }

        /*Partial cksum available for the head fragment*/
        ADK_IP_DP_LOG_INFO_DEBUG("dsi_md_p->checksum=%d\n", dsi_md_p->checksum);
        cksum = rte_cpu_to_be_16(dsi_md_p->checksum) + pkt_cksum;
        seg_len = (task->data_len - ADK_MD_GET_L4_OFF(task));
        total_len = seg_len;
        /*Access next segment*/
        task = (adk_pktio_task_t *)(task->next);
        buff = rte_pktmbuf_mtod(task, uint8_t *);
        seg_len = task->data_len;

        bool odd_len = false;
        /*Multi segmented mbuf*/
        while ((total_len + seg_len) < udp_len)
        {
            if (odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)buff, seg_len));
                odd_len = ((seg_len - 1) % 2);
            }
            else
            {
                cksum += adk_raw_cksum((void *)buff, seg_len);
                odd_len = (seg_len % 2);
            }
            total_len += seg_len;
            /*Access next segment*/
            task = (adk_pktio_task_t *)(task->next);
            buff = rte_pktmbuf_mtod(task, uint8_t *);
            seg_len = task->data_len;
        }

        if (odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)buff,
                    (udp_len - total_len)));
        }
        else
        {
            cksum += adk_raw_cksum((void *)buff, (udp_len - total_len));
        }

        buff = (uint8_t *)&psd_hdr;
        psd_hdr.src_addr = src_addr_nw;
        psd_hdr.dst_addr = dest_addr_nw;
        psd_hdr.zero = 0;
        psd_hdr.proto = ADK_IP_PROTOCOL_UDP;
        psd_hdr.len = rte_cpu_to_be_16(udp_len);
        cksum += adk_raw_cksum((void *)buff, sizeof(psd_hdr));
        cksum = adk_raw_cksum_reduce(cksum);
        ADK_IP_DP_LOG_INFO_DEBUG("calculated cksum=%d\n", cksum);
        cksum = (~cksum) & 0xffff;

        return cksum;
    }
    else
        return (ipv4_udp_checksum_no_ol(task, udp_len, src_addr_nw, dest_addr_nw));
}

static inline
uint16_t ip_hdr_csum(const void *buff, size_t len)
{
    const uint16_t *buf=buff;
    size_t byte_len=4*len;
    uint32_t sum = 0;

    /* Calculate the sum */
    sum = 0;
    while (byte_len > 1)
    {
        sum += *buf++;
        if (sum & 0x80000000)
            sum = (sum & 0xFFFF) + (sum >> 16);
        byte_len -= 2;
    }
 
    /* Add the carries */
    while (sum >> 16)
        sum = (sum & 0xFFFF) + (sum >> 16);
 
    /* Return the one's complement of sum */
    return ((uint16_t)(~sum));
}

static inline uint16_t
adk_psd_hdr_cksum(struct ipv4_hdr *ip_hdr, uint16_t udp_len)
{
    struct ipv4_psd_header {
        uint32_t src_addr; /* IP address of source host. */
        uint32_t dst_addr; /* IP address of destination host. */
        uint8_t  zero;     /* zero. */
        uint8_t  proto;    /* L4 protocol type. */
        uint16_t len;      /* L4 length. */
    } psd_hdr;

    psd_hdr.src_addr = ip_hdr->src_addr;
    psd_hdr.dst_addr = ip_hdr->dst_addr;
    psd_hdr.zero = 0;
    psd_hdr.proto = ip_hdr->next_proto_id;
    psd_hdr.len = udp_len;

    return adk_raw_cksum((void *)(&psd_hdr), sizeof(psd_hdr));
}
static inline
adk_status_t adk_ip_dp_reassemble(uint16_t coreIndex, adk_pktio_task_t **ph_task_p,
        uint8_t vrd_id, uint16_t ip_term_stats_id, uint32_t hash)
{
    adk_status_t st;
    st = adk_ip_dp_reassemble_internal(coreIndex, ph_task_p,
            vrd_id, ip_term_stats_id, hash);
    return st;
}

#endif
