/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef __ADK_IP_DP_INTERNAL_FC_H__
#define __ADK_IP_DP_INTERNAL_FC_H__

#ifdef IP_FLOW_HASH_CACHE

/*Flow cache DP maintenance*/
#define IPV4_FC_TIMER_EPOCH 30000
#define IPV4_FC_THRESHOLD  ((IP_MAX_FLOWS_PER_CPU * 75)/100)  /* 75% of IP_MAX_FLOWS_PER_CPU*/

/* flow-cache tables are thread-local copies so as to scale for
** multi-core environment.
**/
extern adk_ip_dp_fc_count_t *fc_global_maintenance_p;

/* 3-tuple (SIP, DIP, Protocol) based flow-cache table */
extern adk_ip_dp_flow_cache_entry_t *lliCacheTbl_3_p;
extern adk_ip_dp_lli_hash3_tbl_entry_t *lliNodeTbl_3_p;
extern adk_ip_dp_fc_maintenance_t *lliNodeTbl_3_count_p;

/* 5-tuple (SIP, DIP, Protocol, SPort, DPort) based flow-cache table */
extern adk_ip_dp_flow_cache_entry_t *lliCacheTbl_5_p;
extern adk_ip_dp_lli_hash5_tbl_entry_t *lliNodeTbl_5_p;
extern adk_ip_dp_fc_maintenance_t *lliNodeTbl_5_count_p;

/* DIP based flow-cache table */
extern adk_ip_dp_flow_cache_entry_t *uliCacheTbl_1_p;
extern adk_ip_dp_fc_maintenance_t *uliNodeTbl_1_count_p;

/* 5-tuple based flow-cache table */
extern adk_ip_dp_flow_cache_entry_t *uliCacheTbl_5_p;
extern adk_ip_dp_uli_hash5_tbl_entry_t *uliNodeTbl_5_p;
extern adk_ip_dp_fc_maintenance_t *uliNodeTbl_5_count_p;

adk_status_t adk_ip_dp_lli_5tuple_flow_cache_path(
      adk_pktio_task_t **ph_task, int coreIndex,
      adk_pal_dp_stage_info_t *stage_info_p,
      uint32_t *destModId,
      uint16_t src_port, uint16_t dest_port, uint16_t ip_len,
      uint8_t in_dscp, uint8_t out_dscp,
#ifdef UDP_CHECKSUM_ENABLED
      uint16_t udp_length,
#endif
      uint32_t ingressIfId, adk_ip_dp_intf_stats_t *ip_stats,
      adk_ip_dp_lli_hash5_tbl_entry_t **hash5Entry_p,
      uint16_t *ingressAssocRulesMask_p,
      adk_ip_dp_intf_stats_t **ip_term_stats_p,
      uint8_t *action, uint8_t fc_type);

adk_status_t
adk_ip_dp_lli_3tuple_flow_cache_path(
      adk_pktio_task_t **ph_task_p, int coreIndex,
      adk_pal_dp_stage_info_t *stage_info_p,
      uint32_t *destModId,
      uint16_t ip_len, uint8_t in_dscp, uint8_t out_dscp,
      uint32_t ingressIfId, adk_ip_dp_intf_stats_t *ip_stats,
      adk_ip_dp_lli_hash3_tbl_entry_t **hash3Entry_p,
      adk_ip_dp_lli_hash5_tbl_entry_t **hash5Entry_p,
      uint16_t *src_port_p, uint16_t *dest_port_p,
      uint16_t *ingressAssocRulesMask_p, uint8_t *action
#ifdef UDP_CHECKSUM_ENABLED
      ,uint16_t *udp_length_p
#endif
      ,adk_ip_dp_intf_stats_t **ip_term_stats
      ,uint8_t fc_type
      );

adk_ip_dp_lli_hash3_tbl_entry_t *
adk_ip_dp_lli_3tuple_flow_cache_lookup(int coreIndex,
                                       struct ipv4_hdr *iphdr,
                                       uint8_t vrd_id,
                                       uint8_t filter_ruleset_id,
                                       uint8_t fc_type);

adk_ip_dp_lli_hash5_tbl_entry_t *
adk_ip_dp_lli_5tuple_flow_cache_lookup(int coreIndex,
                                       struct ipv4_hdr *iphdr,
                                       uint8_t vrd_id,
                                       uint8_t filter_ruleset_id,
                                       uint16_t dest_port,
                                       uint16_t src_port,
                                       uint8_t fc_type);

adk_ip_dp_uli_hash5_tbl_entry_t *
adk_ip_dp_uli_5tuple_flow_cache_lookup(int coreIndex,
                                       uint32_t hash_val,
                                       uint8_t vrd_id,
                                       uint32_t dest_addr_nw,
                                       uint32_t src_addr_nw,
                                       uint8_t protocol,
                                       uint16_t dest_port,
                                       uint16_t src_port);

static inline
void update_lli_hash3_hit_maintenance(int coreIndex,
		adk_ip_dp_lli_hash3_tbl_entry_t *hash3Entry)
{
	hash3Entry->hit++;
	/*set hit flag in the DP maintenance data*/
	lliNodeTbl_3_count_p[coreIndex].fc_maintenance[(hash3Entry->own_index & 
			(IP_MAX_FLOWS_PER_CPU-1))/4] |= (2 << (hash3Entry->own_index%4)*2);
}

static inline
void update_lli_hash5_hit_maintenance(int coreIndex,
        adk_ip_dp_lli_hash5_tbl_entry_t *hash5Entry)
{
    hash5Entry->hit++;
    /*set hit flag in the DP maintenance data*/
    lliNodeTbl_5_count_p[coreIndex].fc_maintenance[(hash5Entry->own_index &
            (IP_MAX_FLOWS_PER_CPU-1))/4] |= (2 << (hash5Entry->own_index%4)*2);
}

static inline
void adk_ip_dp_lli_5tuple_flow_cache_add(int coreIndex,
                                   adk_ip_dp_lli_hash5_tbl_entry_t *hash5Entry)
{
    uint16_t hash5_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
     &(lliCacheTbl_5_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash5Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].lli5_fc_usage_count++;
    cacheEntry->node_ptr = hash5Entry;
    hash5_node_index = hash5Entry->own_index;
    fc_global_maintenance_p[coreIndex].lli5_fc_global_free_index =
                                             hash5Entry->next_free_index;
    /*Set used flag and hit flag in DP maintenance data*/
    lliNodeTbl_5_count_p[coreIndex].fc_maintenance[hash5_node_index/4] |= 
                    (3 << (((hash5_node_index)%4)*2));
    hash5Entry->hit++;
}

static inline
void adk_ip_dp_lli_5tuple_flow_cache_release(int coreIndex,
                                   adk_ip_dp_lli_hash5_tbl_entry_t *hash5Entry)
{
    uint16_t hash5_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
      &(lliCacheTbl_5_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash5Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].lli5_fc_usage_count--;
    hash5_node_index = hash5Entry->own_index;
    cacheEntry->node_ptr = NULL;
    /*Push the current index as global free index*/
    hash5Entry->next_free_index =
             fc_global_maintenance_p[coreIndex].lli5_fc_global_free_index;
    fc_global_maintenance_p[coreIndex].lli5_fc_global_free_index =
                                         hash5_node_index;
    /*reset used flag and hit flag in DP maintenance data*/
    lliNodeTbl_5_count_p[coreIndex].fc_maintenance[hash5_node_index/4] &= 
                    ~(3 << (((hash5_node_index)%4)*2));
}

static inline
void adk_ip_dp_lli_3tuple_flow_cache_add(int coreIndex,
                                   adk_ip_dp_lli_hash3_tbl_entry_t *hash3Entry)
{
    uint16_t hash3_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
      &(lliCacheTbl_3_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash3Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].lli3_fc_usage_count++;
    cacheEntry->node_ptr = hash3Entry;
    hash3_node_index = hash3Entry->own_index;
    /*Update global free node index*/
    fc_global_maintenance_p[coreIndex].lli3_fc_global_free_index =
                                         hash3Entry->next_free_index;
    /*Set used flag and hit flag in DP maintenance data*/
    lliNodeTbl_3_count_p[coreIndex].fc_maintenance[hash3_node_index/4] |=
                     (3 << (((hash3_node_index)%4)*2));
    hash3Entry->hit++;
}

static inline
void adk_ip_dp_lli_3tuple_flow_cache_release(int coreIndex,
                                   adk_ip_dp_lli_hash3_tbl_entry_t *hash3Entry)
{
    uint16_t hash3_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
      &(lliCacheTbl_3_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash3Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].lli3_fc_usage_count--;
    cacheEntry->node_ptr = NULL;
    hash3_node_index = hash3Entry->own_index;
    /*Push the current index as global free index*/
    hash3Entry->next_free_index =
             fc_global_maintenance_p[coreIndex].lli3_fc_global_free_index;
    fc_global_maintenance_p[coreIndex].lli3_fc_global_free_index =
                 hash3_node_index;
    /*reset used flag and hit flag in DP maintenance data*/
    lliNodeTbl_3_count_p[coreIndex].fc_maintenance[hash3_node_index/4] &= 
                    ~(3 << (((hash3_node_index)%4)*2));
}

static inline
void adk_ip_dp_uli_5tuple_flow_cache_add(int coreIndex,
                                   adk_ip_dp_uli_hash5_tbl_entry_t *hash5Entry)
{
    uint16_t hash5_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
     &(uliCacheTbl_5_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash5Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].uli5_fc_usage_count++;
    cacheEntry->node_ptr = hash5Entry;

    hash5_node_index = hash5Entry->own_index;
    /*Update global free node index*/
    fc_global_maintenance_p[coreIndex].uli5_fc_global_free_index =
                         hash5Entry->next_free_index;
    /*Set used flag and hit flag in DP maintenance data*/
    uliNodeTbl_5_count_p[coreIndex].fc_maintenance[hash5_node_index/4] |=
                     (3 << (((hash5_node_index)%4)*2));
    hash5Entry->hit++;
}

static inline
void adk_ip_dp_uli_5tuple_flow_cache_release(int coreIndex,
                                   adk_ip_dp_uli_hash5_tbl_entry_t *hash5Entry)
{
    uint16_t hash5_node_index;
    adk_ip_dp_flow_cache_entry_t * cacheEntry=
     &(uliCacheTbl_5_p[coreIndex*(IP_FLOW_HASH_MAX*IP_FLOW_HASH_COLLISION_SIZE)
                                     + hash5Entry->cacheEntryId]);
    fc_global_maintenance_p[coreIndex].uli5_fc_usage_count--;
    cacheEntry->node_ptr = NULL;
    hash5_node_index = hash5Entry->own_index;
    /*Push the current index as global free index*/
    hash5Entry->next_free_index =
         fc_global_maintenance_p[coreIndex].uli5_fc_global_free_index;
    fc_global_maintenance_p[coreIndex].uli5_fc_global_free_index =
                                 hash5_node_index;
    /*reset used flag and hit flag in DP maintenance data*/
    uliNodeTbl_5_count_p[coreIndex].fc_maintenance[hash5_node_index/4] &= 
                    ~(3 << (((hash5_node_index)%4)*2));
}

static inline
void update_uli_hash5_hit_maintenance(int coreIndex,
                    adk_ip_dp_uli_hash5_tbl_entry_t *hash5Entry)
{
    hash5Entry->hit++;
    /*set hit flag in the DP maintenance data*/
    uliNodeTbl_5_count_p[coreIndex].fc_maintenance[(hash5Entry->own_index &
        (IP_MAX_FLOWS_PER_CPU-1))/4] |= (2 << (hash5Entry->own_index%4)*2);
}

static inline
void adk_ip_dp_lli5_fc_aging_maintenance(int coreIndex, 
        uint8_t *lli_hash5_maintenance_local)
{
    uint32_t i;
    for(i = 0; i< IP_MAX_FLOWS_PER_CPU; i++)
    {
        /*Check for each entry usage*/
        if(lli_hash5_maintenance_local[i/4] & (1 <<((i%4)*2)))
        {
            /*check hit flag status for this entry and reset if it is set*/
            if(lli_hash5_maintenance_local[i/4] & (2 <<((i%4)*2)))
            {
                lli_hash5_maintenance_local[i/4] &= ~(2 << ((i%4)*2));
            }
            else
            {
                /*Not hit in the current epoch ,hence release entry*/
                adk_ip_dp_lli_5tuple_flow_cache_release(coreIndex,
                        &(lliNodeTbl_5_p[coreIndex*IP_MAX_FLOWS_PER_CPU +i]));
            }
        }
    }
}

static inline
void adk_ip_dp_lli3_fc_aging_maintenance(int coreIndex, 
        uint8_t *lli_hash3_maintenance_local)
{
    uint32_t i;
    for(i = 0; i< IP_MAX_FLOWS_PER_CPU; i++)
    {
        /*Check for each entry usage*/
        if(lli_hash3_maintenance_local[i/4] & (1 <<((i%4)*2)))
        {
            /*check hit flag status for this entry and reset if it is set*/
            if(lli_hash3_maintenance_local[i/4] & (2 <<((i%4)*2)))
            {
                lli_hash3_maintenance_local[i/4] &= ~(2 << ((i%4)*2));
            }
            else
            {
                /*Not hit in the current epoch ,hence release entry*/
                adk_ip_dp_lli_3tuple_flow_cache_release(coreIndex,
                        &(lliNodeTbl_3_p[coreIndex*IP_MAX_FLOWS_PER_CPU +i]));
            }
        }
    }
}

static inline
void adk_ip_dp_uli5_fc_aging_maintenance(int coreIndex, 
        uint8_t *uli_hash5_maintenance_local)
{
    uint32_t i;
    for(i = 0; i< IP_MAX_FLOWS_PER_CPU; i++)
    {
        /*Check for each entry usage*/
        if(uli_hash5_maintenance_local[i/4] & (1 <<((i%4)*2)))
        {
            /*check hit flag status for this entry and reset if it is set*/
            if(uli_hash5_maintenance_local[i/4] & (2 <<((i%4)*2)))
            {
                uli_hash5_maintenance_local[i/4] &= ~(2 << ((i%4)*2));
            }
            else
            {
                /*Not hit in the current epoch ,hence release entry*/
                adk_ip_dp_uli_5tuple_flow_cache_release(coreIndex,
                        &(uliNodeTbl_5_p[coreIndex*IP_MAX_FLOWS_PER_CPU +i]));
            }
        }
    }
}

#endif
#endif
