/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IP_ING_CLASSIFIER_H__
#define __ADK_IP_ING_CLASSIFIER_H__

#define ADK_IP_CLASSIFIER_RULE_MAX_PRIORITY 0xFFFF
/*!
 * =============================================================
 * @internal
 * @struct adk_ip_ing_classifier_info_t adk_ip_common.h "adk_ip_common.h"
 *
 * @brief
 *
 * Ingress Classifier Ordered Tree information.
 * =============================================================
 */
typedef void adk_ip_ing_classifier_ctx_t;

typedef struct {
    uint32_t cookie;
    /*Two copies: Active & Shadow*/
    adk_ip_ing_classifier_ctx_t *ipIngClassifierCtx[2];
    uint8_t aclTreeIndex;
    adk_acl_handle_t cpList;
} adk_ip_ing_classifer_info_t;

/* Ingress Classifier Related Structures */

typedef struct {
    uint8_t  proto;
    uint32_t sip;
    uint32_t dip;
    uint16_t sPort;
    uint16_t dPort;
    uint32_t ipIntfHandle;
} __attribute__((__packed__)) adk_ip_ing_classifier_ptn_t;

/* Currently DP uses same structures exposed to the application */
typedef struct adk_ip_dp_ing_classifier_action_info_s{ 
    adk_ip_ingress_classifier_action_type_e action;
    bool policingEnabled;
    uint32_t policerFlowId;
    uint32_t eventMachineQueueId;
}adk_ip_dp_ing_classifier_action_info_t;

/* Ingress Classifier Ordered tree related declaration */
adk_status_t adk_ip_ingress_classifier_create(
        adk_module_handle_t ipHandle,
        adk_ip_ing_classifer_info_t  **ingClassifierInfo_pp);

void adk_ip_ingress_classifier_delete(
        adk_module_handle_t ipHandle,
        adk_ip_ing_classifer_info_t  **ingClassifierInfo_pp);

adk_status_t adk_ip_ing_classifier_delete(
        adk_module_handle_t ipHandle,
        adk_ip_ing_classifier_ctx_t *ctx);

adk_status_t adk_ip_classifer_add_validate_arg(
        adk_module_handle_t ipHandle,
        const adk_ip_ingress_classifier_rule_t *ipIngClassifierRule_p,
        const adk_ip_ingress_classifier_action_t *ipIngClassifierAction_p);

adk_status_t classifier_build_ordered_tree(adk_module_handle_t ipHandle,
        adk_ip_ing_classifier_ctx_t *ctx);

adk_status_t classifier_add_rule_to_hw (
    adk_module_handle_t ipHandle,
    uint32_t precedence,
    uint32_t ingClassifierFlowId,
    const adk_ip_ingress_classifier_rule_t *ipIngClassifierRule_p,
    const adk_ip_ingress_classifier_action_t *ipIngClassifierAction_p,
    uint8_t treeIndex);

adk_dp_status_t adk_ip_dp_ing_classifier_lookup(
        adk_ip_ing_classifier_ctx_t *ipIngClassifierCtx_p,
        adk_ip_ing_classifier_ptn_t  *ingClassifierPtn,
        uint32_t  *result_p);
#endif
