/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file adk_ip_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ip  IPv4 Module
 *
 * @{
 *
 * @brief
 *    This file contains VRD specific APIs 
 *
 ***********************************************************************/

#ifndef _ADK_IP_INTERNAL_H_
#define _ADK_IP_INTERNAL_H_

#ifdef ENABLE_VRD

/*!
 * =============================================================
 * @typedef adk_ip_vrd_handle_t
 *
 * @brief
 *
 * IP Virtual Routing Instance handle
 *
 * =============================================================
 */
typedef adk_handle_t adk_ip_vrd_handle_t;

/*!
 * =============================================================
 * @struct adk_ip_vrd_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create IP virtual routing domain.
 *
 * =============================================================
 */
typedef struct adk_ip_vrd_s {
   /*! Virtual routing domain id as assigned by the application.
    * It is application's responsibility to assign a unique ID
    * in the range 0 to (ADK_IP_VRD_ID_MAX - 1) except the ID
    * reserved for the default VRD i.e. ADK_IP_DEFAULT_VRD_ID.
    */
   uint32_t vrd_id;

} adk_ip_vrd_t;

adk_status_t adk_ip_vrd_create(
      adk_module_handle_t ipHandle,
      const adk_ip_vrd_t *vrdAttr_p,
      adk_ip_vrd_handle_t *ipVrdHandle);

adk_status_t adk_ip_vrd_delete(
      adk_module_handle_t ipHandle,
      adk_ip_vrd_handle_t ipVrdHandle);

adk_status_t adk_ip_vrd_mode(
      adk_module_handle_t ipHandle,
      adk_ip_vrd_handle_t ipVrdHandle,
      bool host_mode);

adk_status_t adk_ip_vrd_handle_get(
      adk_module_handle_t ipHandle,
      uint32_t vrd_id,
      adk_ip_vrd_handle_t *ipVrdHandle_p);


#endif /* ENABLE_VRD */

#endif
