/*
 * adk_ip_unsupported.c
 *
 *  Created on: Jun 13, 2019
 *      Author: tvmande
 */

#ifndef _ADK_IP_UNSUPPORTED_H_
#define _ADK_IP_UNSUPPORTED_H_

#include "adk_ip_config.h"
#include "adk_ph_dev.h"
#include "adk_domain.h"
#include "adk_ip.h"



/*
 * IP Network Address (Port) Translation specific data structures
 * and definitions.
 */


/*!
 * =============================================================
 * @enum adk_ip_napt_rules_mask_t
 *
 * @brief
 *
 * Defines the mask for NAPT rules data structure (adk_ip_napt_rules_t).
 *
 * =============================================================
 */
typedef enum adk_ip_napt_rules_mask_e {
    /*! incoming Source IP address mask */
    ADK_IP_NAPT_IN_SRC_ADDR_MASK = 0x01,
    /*! incoming Source L4 port mask */
    ADK_IP_NAPT_IN_SRC_L4PORT_MASK = 0x02,
    /*! outgoing Source IP address mask */
    ADK_IP_NAPT_OUT_SRC_ADDR_MASK = 0x04,
    /*! outgoing Source L4 port mask */
    ADK_IP_NAPT_OUT_SRC_L4PORT_MASK = 0x08
} adk_ip_napt_rules_mask_t;

/*!
 * =============================================================
 * @struct adk_ip_napt_rules_t adk_ip.h "adk_ip.h"
 *
 * @brief
 *
 * This structure defines the parameters that are required to
 * configure NAPT rules.
 *
 * =============================================================
 */
typedef struct adk_ip_napt_rules_s {
    /*!
     * NAPT rule fileds selection mask.
     */
    uint8_t rulesMask;
    /*!
     * inSrcAddr defines the Source IP address of the packet that
     * needs to be translated.
     */
    adk_ip_addr_t inSrcAddr;
    /*!
     * inSrcL4Port defines the source L4 Port of the packet that
     * needs to be translated.
     */
    uint16_t inSrcL4Port;
    /*!
     * outSrcAddr defines the IP address the packet after performing
     * address translation.
     */
    adk_ip_addr_t outSrcAddr;
    /*!
     * outSrcL4Port defines the L4 Port of the packet after performing
     * port translation.
     */
    uint16_t outSrcL4Port;
} adk_ip_napt_rules_t;

adk_status_t adk_ip_destination_mtu_add(
    adk_module_handle_t ipHandle,
    adk_ip_addr_t dipAddr,
    uint32_t mtu);

adk_status_t adk_ip_destination_mtu_rem(
    adk_module_handle_t ipHandle,
    adk_ip_addr_t dipAddr);

adk_status_t adk_ip_napt_rule_add (
    adk_module_handle_t ipHandle,
    const adk_ip_napt_rules_t *naptRules_p);

adk_status_t adk_ip_napt_rule_delete (
    adk_module_handle_t ipHandle,
    const adk_ip_napt_rules_t *naptRules_p);

#endif /* _ADK_IP_UNSUPPORTED_H_ */
