/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipsec.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ipsec  IPsec Module
 * @{
 *
 * @brief
 *
 * This file contains all ADK API function definitions
 *
 ***********************************************************************/
#ifndef _ADK_IPSEC_H_
#define _ADK_IPSEC_H_

#include "adk_ipsec_struct.h"

/*!
 * @addtogroup _ipsecFunctions  IPsec Functions
 * @{
 */
/* API Function Definitions */
adk_status_t
adk_ipsec_init (
               adk_module_handle_t ipsecHandle);

adk_status_t  adk_ipsec_service_function_update (adk_module_handle_t ipsecHandle);

adk_status_t
adk_ipsec_add_sp(
                adk_module_handle_t        ipsecHandle,
                adk_ipsec_sec_pol_t        *secPol_p,
                uint16_t                   polIndex,
                bool                       polByIndex,
                bool                       polCommit,
                adk_handle_t               eSANsHandle,
                adk_handle_t               iSANsHandle);

adk_status_t
adk_ipsec_add_sp_in(
                   adk_module_handle_t        ipsecHandle,
                   adk_ipsec_sec_pol_in_t     *secPol_p,
                   uint16_t                   polIndex,
                   bool                       polByIndex,
                   bool                       polCommit);

adk_status_t
adk_ipsec_del_sp (
        adk_module_handle_t ipsecHandle,
        uint16_t            polIndex,
        bool                polCommit);

adk_status_t
adk_ipsec_del_sp_in (
        adk_module_handle_t ipsecHandle,
        uint16_t            polIndex,
        bool                polCommit);

adk_status_t
adk_ipsec_update_sp (
        adk_module_handle_t	     ipsecHandle,
        uint16_t                 polIndex,
        adk_ipsec_sec_pol_attr_t *newSecPolAttr_p);

adk_status_t
adk_ipsec_update_sp_in (
         adk_module_handle_t    ipsecHandle,
         uint16_t               polIndex,
         adk_ipsec_pol_action_t newAction);

adk_status_t
adk_ipsec_table_build(
         adk_module_handle_t        ipsecHandle,
         adk_ipsec_table_t          ipsecTblId);

adk_status_t
adk_ipsec_add_sa (adk_module_handle_t ipsecHandle,
                  adk_ipsec_sa_t      *secAsc_p,
                  uint8_t             dscpVal,
                  uint8_t             numPol,
                  uint16_t            startPolIndex,
                  adk_handle_t        *saNsHandle);

adk_status_t
adk_ipsec_del_sa(adk_module_handle_t      ipsecHandle,
                 adk_ipsec_sa_direction_t saDir,
                 adk_handle_t             saNsHandle,
                 uint8_t                  numPol,
                 uint16_t                 startPolIndex);

adk_status_t
adk_ipsec_update_sa (adk_module_handle_t ipsecHandle,
                     adk_ipsec_upd_sa_t  *newSecAscUpdate_p,
                     uint8_t             dscpVal,
                     uint8_t             numPol,
                     uint16_t            startPolIndex,
                     adk_handle_t        oldSaNsHandle,
                     adk_handle_t        *newSaNsHandle);

adk_status_t
adk_ipsec_get_stat (
        adk_module_handle_t  ipsecHandle,
        adk_ipsec_ip_version_t  ipVersion,
        adk_ipsec_stat_t     *ipsecStat_p);

adk_status_t
adk_ipsec_get_sa_stat(adk_module_handle_t         ipsecHandle,
                      adk_handle_t                saNsHandle,
                      adk_ipsec_sa_direction_t    saDirection,
                      adk_ipsec_sa_stat_t         *saStat_p);

adk_status_t
adk_ipsec_get_num_sp (
                     adk_module_handle_t ipsecHandle,
                     uint32_t        *numSP);

adk_status_t
adk_ipsec_get_num_sp_in (adk_module_handle_t ipsecHandle,
                         uint32_t            *numSP_p);

adk_status_t
adk_ipsec_get_sp (
                adk_module_handle_t           ipsecHandle,
                adk_ipsec_sec_pol_sel_lkup_t  *secPolSelLkup_p,
                adk_ipsec_sec_pol_t           *secPol_p,
                uint16_t                      polIndex,
                bool                          byIndex);

adk_status_t
adk_ipsec_get_sp_in (
                    adk_module_handle_t             ipsecHandle,
                    adk_ipsec_sec_pol_sel_lkup_t    *secPolSelLkup_p,
                    adk_ipsec_sec_pol_in_t          *secPol_p,
                    uint16_t                        polIndex,
                    bool                            byIndex);

adk_status_t
adk_ipsec_get_sp_all(
                     adk_module_handle_t	ipsecHandle,
                     uint32_t               numSp,
                     adk_ipsec_sec_pol_t    *secPol_p);

adk_status_t
adk_ipsec_get_sp_in_all(adk_module_handle_t      ipsecHandle,
                        uint32_t                 numSp,
                        adk_ipsec_sec_pol_in_t   *secPolList_p);

adk_status_t adk_ipsec_dp_init (void);

adk_status_t
adk_ipsec_get_sa (adk_module_handle_t       ipsecHandle,
                 adk_handle_t               saNsHandle,
                 adk_ipsec_sa_direction_t   saDirection,
                 adk_ipsec_sa_t             *secAsc_p);

adk_status_t
adk_ipsec_register_exception (
        adk_module_handle_t ipsecHandle,
        const adk_ipsec_exception_handler_t *exceptionHandler_p);

adk_status_t
adk_ipsec_unregister_exception (
    adk_module_handle_t ipsecHandle,
    const adk_ipsec_exception_type_t exceptionType);

adk_status_t 
adk_ipsec_prefrag_buffering_timeout_config (adk_module_handle_t ipsecHandle,
                                            uint32_t timeOutVal);
adk_status_t
adk_ipsec_get_sa_all (adk_module_handle_t        ipsecHandle,
                  adk_ipsec_sa_direction_t       saDirection,
                  uint32_t                       *numSa_p,
                  adk_ipsec_sa_t                 **secAscList_p);
 
adk_status_t
adk_ipsec_dp_init(void);

#ifdef ADK_DEBUG
adk_status_t adk_ipsec_dump_sa(adk_module_handle_t ipsecHandle,
                               adk_ipsec_sa_direction_t direction);

adk_status_t adk_ipsec_dump_policies(adk_module_handle_t ipsecHandle);
#endif

adk_status_t
adk_ipsec_shutdown(adk_module_handle_t ipsecHandle);

adk_status_t adk_ipsec_log_level_set(adk_module_handle_t ipsecHandle,
                                     adk_log_level_t log_level);

uint16_t
adk_ipsec_dp_egress_tx_cb(
        uint16_t port_id,
        uint16_t queue,
        struct rte_mbuf *pkts[],
        uint16_t nb_pkts,
        void *user_param);

#endif

/*! end addtogroup _ipsecFunctions
 * @}
 *
 * end addtogroup _ipsec
 * @}
 */
