/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include <rte_bridge_crypto.h>
#include <rte_eventdev.h>
#include <adk_cksum.h>
#include <ice_dsi_ipsec.h>
#include <adk_sys.h>
#include <adk_pmu.h>
#include <adk_metadata.h>
#include <adk_ipsec_struct.h>
#include "adk_ipsec_common.h"
#include "adk_ipsec_dp.h"
#include "adk_ipsec_export.h"
#include <adk_ip_dp_export.h>
#include <adk_timer.h>
#include <adk_hos_dp.h>
#include <adk_ipv6_dp_export.h>
#include <adk_ipv6_dp_frag.h>
#include <adk_sys_dp_flow_cache_uli_func.h>


extern adk_ipsec_dp_table_info_t *adk_ipsec_dp_tbl_gbl_ptr;
extern ipsecIdSeqDB_t adk_ipsec_IdSeqDB[ADK_MAX_NUM_CORES];
extern ipsecIdSeqRange_t adk_ipsec_IdSeqRange[ADK_MAX_NUM_CORES];
__thread adk_sif_dp_scratchpad_space_t *ipsec_scratch_pad = NULL;
/* By default, IPSEC_ULI_SUB_BLOCKS would be commented.
 * If needed to take cycle count for sub blocks, please
 * enable this macro.
 */
//#define IPSEC_ULI_SUB_BLOCKS
/* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
    uint64_t ipsec_uli_rx_ts, ipsec_uli_tx_ts, ipsec_post_crypto_rx_ts, ipsec_post_crypto_tx_ts;
    uint64_t ipsec_uli_parse_pkt_rx_ts, ipsec_uli_parse_pkt_tx_ts;
    uint64_t ipsec_uli_spd_lkup_rx_ts, ipsec_uli_spd_lkup_tx_ts;
    uint64_t ipsec_uli_prepare_proto_rx_ts, ipsec_uli_prepare_proto_tx_ts;
    uint64_t ipsec_uli_prepare_crypto_rx_ts, ipsec_uli_prepare_crypto_tx_ts;
    uint64_t ipsec_uli_after_crypto_rx_ts, ipsec_uli_after_crypto_tx_ts;
    uint64_t ipsec_uli_variables_init_rx_ts, ipsec_uli_variables_init_tx_ts;
    uint64_t ipsec_uli_action_ipsec_rx_ts, ipsec_uli_action_ipsec_tx_ts;
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_rx_ts, &ipsec_uli_tx_ts, "IPSEC ULI MODULE", PMU_IPSEC_ULI_MODULE)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_post_crypto_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_post_crypto_rx_ts, &ipsec_post_crypto_tx_ts, "IPSEC ULI POST CRYPTO MODULE", PMU_IPSEC_ULI_POST_CRYPTO_MODULE)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_parse_pkt_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_parse_pkt_rx_ts, &ipsec_uli_parse_pkt_tx_ts, "IPSEC ULI MODULE PARSE PKT", PMU_IPSEC_ULI_MODULE_PARSE_PKT)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_spd_lkup_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_spd_lkup_rx_ts, &ipsec_uli_spd_lkup_tx_ts, "IPSEC ULI MODULE SPD LKUP", PMU_IPSEC_ULI_MODULE_SPD_LKUP)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_prepare_proto_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_prepare_proto_rx_ts, &ipsec_uli_prepare_proto_tx_ts, "IPSEC ULI MODULE PREPARE PROTO", PMU_IPSEC_ULI_MODULE_PREPARE_PROTO)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_prepare_crypto_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_prepare_crypto_rx_ts, &ipsec_uli_prepare_crypto_tx_ts, "IPSEC ULI MODULE PREPARE CRYPTO", PMU_IPSEC_ULI_MODULE_PREPARE_CRYPTO)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_after_crypto_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_after_crypto_rx_ts, &ipsec_uli_after_crypto_tx_ts, "IPSEC ULI MODULE AFTER CRYPTO", PMU_IPSEC_ULI_MODULE_AFTER_CRYPTO)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_variables_init_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_variables_init_rx_ts, &ipsec_uli_variables_init_tx_ts, "IPSEC ULI MODULE VARIABLES INIT", PMU_IPSEC_ULI_MODULE_VARIABLES_INIT)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_uli_action_ipsec_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_uli_action_ipsec_rx_ts, &ipsec_uli_action_ipsec_tx_ts, "IPSEC ULI MODULE ACTION IPSEC", PMU_IPSEC_ULI_MODULE_ACTION_IPSEC)

#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_TX_TS()
#endif

static inline adk_status_t
adk_ipsec_dp_pkt_proc_uli_single (adk_ipsec_dp_uli_common_params_t *info);

static inline adk_status_t
adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single(adk_ipsec_dp_uli_common_params_t *info);
/*!
 * =============================================================
 * adk_ipsec_dp_pkt_proc_uli()
 *
 * Structure of task params :
 *
 * pduParams[7] = pointer to pdu at the begining of inner IP header.
 *
 * Brief:
 * This function is the entry point for encrypting plain text IP
 * packets received from DSP or WIFI port. The function parses the
 * inner header and performs Security
 * Policy Database lookup to determine SA to be applied. Tunnel header and ESP
 * headers are attached to the packet.
 *
 * TBD: Packet needs to be handed off to SPP for actual encryption.
 *
 * Once processing is complete the excrypted packet is handed off to IP ULI -
 * adk_ip_dp_pkt_proc_uli for tunnel header processing.
 *
 * =============================================================
 */
adk_status_t
adk_ipsec_dp_pkt_proc_uli (void **taskList, uint8_t task_count,
                           uint32_t task_mask,
                           __attribute__((unused)) void *ring,
                           __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
                           int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    uint8_t i, drop_count=0;
    uint32_t cur_bit, drop_mask=0;
    adk_pktio_task_t *currentTask, *nextTask;
    adk_ipsec_dp_uli_common_params_t info = {0};
    info.taskInfo = (adk_pktio_task_t *)taskList[0];
    info.coreIndex = coreIndex;
    for(i=0, cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(!(task_mask & cur_bit))
            continue;

        rte_prefetch0(adk_ipsec_dp_tbl_gbl_ptr);
        currentTask = (adk_pktio_task_t *)(taskList[i]);
        nextTask =
            ADK_METADATA_PARAM((adk_pktio_task_t *)(currentTask), next_frag);
        if (ADK_E_FLOW_CACHE_PKT_FAST_PATH == (ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreIndex))->pktPath) {
            ret = adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single(&info);
        }else {
            ret = adk_ipsec_dp_pkt_proc_uli_single(&info);
        }
        if(unlikely((ADK_E_INTERNAL_ERROR == ret) ||
                    (ADK_E_DROP_TASK == ret) ||
                    (ADK_E_INVALID_HANDLE == ret)))
        {
            while (currentTask)
            {
                /*Free the task*/
                nextTask =
                    ADK_METADATA_PARAM((adk_pktio_task_t *)(currentTask), next_frag);
                adk_pal_dp_release_send(currentTask, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                ADK_METADATA_PARAM((adk_pktio_task_t *)(currentTask), next_frag) = NULL;
                adk_pktio_task_buffer_free (currentTask);
                currentTask = nextTask;
            }
            drop_mask |= cur_bit;
            drop_count ++;
        }
        else if (unlikely(ADK_E_TASK_CONSUMED == ret))
        {
            drop_mask |= cur_bit;
            drop_count ++;
        }
        else
        {
            if (likely(info.operType == ADK_IPSEC_INLINE))
            {
                if (likely(adkDestEntryFunction_g[info.destFunc]))
                {
                    ret = adk_pal_dp_stage_send((void **)taskList, 1, 1, info.destFunc, 0x00,
                            0x00, ADK_PAL_DP_BATCH_CB_FN_IPSEC, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
                    if(unlikely(ret == ADK_E_INTERNAL_ERROR))
                    {
                        adk_pal_dp_release_send(taskList[0], ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                        adk_pktio_task_buffer_free (taskList[0]);
                        return ADK_E_DROP_TASK;
                    }
                }
                else
                {
                    adk_pal_dp_release_send(taskList[0], ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                    adk_pktio_task_buffer_free (taskList[0]);
                    return ADK_E_DROP_TASK;
                }
            }
        }
    }
    task_count -= drop_count;

    return ret;
}


/*!
 * =============================================================
 * adk_ipsec_dp_pkt_proc_uli_from_crypto ()
 *
 * Structure of task params :
 *
 * pduParams[7]
 *
 * Brief:
 * This function is called only when the IPSec ULI functionality is pipelined.
 * This stage will send the packet to crypto accelator.
 *
 * Currently, the crypto accelator is not integrated, so it sends
 * the packet to IPv4 module.
 *
 * =============================================================
 */
adk_status_t
adk_ipsec_dp_pkt_proc_uli_from_crypto (void **taskList, __attribute__((unused)) uint8_t task_count,
                                       __attribute__((unused)) uint32_t task_mask,
                                       __attribute__((unused)) void *ring,
                                       __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
                                       int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t *taskInfo = NULL;
    struct rte_mbuf *pktMbuf = NULL;
    ipsec_dp_outbound_meta_data_t *ipsecMetaData = NULL;
    struct rte_crypto_op* op_data = NULL;
    adk_metadata_t *adkMetaData = NULL;  
    adk_ipv6_dp_uli_imc_data_t *ipv6_imc = NULL;
    adk_ip_dp_uli_imc_data_t *ipUliImc = NULL;
    uint16_t destFunc;
    uint8_t cryptoStatus;
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_RX_TS();
    op_data = (struct rte_crypto_op *)taskList[0];
    pktMbuf = op_data->sym->m_src;
    taskInfo = (adk_pktio_task_t *)pktMbuf;
    cryptoStatus = op_data->status;
    ipsecMetaData = (ipsec_dp_outbound_meta_data_t *)&(ADK_METADATA_PARAM(taskInfo, imc_param_start));
    adkMetaData = ADK_METADATA_PTR(taskInfo);
    rte_crypto_op_free(op_data);

    if (unlikely(RTE_CRYPTO_OP_STATUS_SUCCESS != cryptoStatus))
    {
        if (ADK_IPSEC_IPV4 == ipsecMetaData->dscpVal_version)
        {
            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].egressStats.ipsec_eg_global_la_crypto_op_failed;
        }
        else
        {
            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].egressStats.ipsec_eg_global_la_crypto_op_failed;
        }
        adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (taskInfo);
        return ADK_E_DROP_TASK;
    }
    
    /* Any packet received from QAT after encryption is considered to be a fast
     * path originated packet. Hence, updating the exception domain in the
     * metadata here.
     */
    adkMetaData->exception_domain = ADK_FP_ORIGIN_DOMAIN;

    adk_ipsec_dp_tbl_gbl_ptr->eSATbl[ipsecMetaData->saId].inFlightCnt--;
    adk_ipsec_dp_tbl_gbl_ptr->SAUtilityCnt[coreIndex][ipsecMetaData->saId].egSaUtilityCounters.ipsec_eg_sa_inflight_pkt--;
    ++adk_ipsec_dp_tbl_gbl_ptr->egSAStats[coreIndex][ipsecMetaData->saId].ipsec_eg_sa_tot_out_pkts;
    adkMetaData->saID = ADK_IPSEC_INVALID_SA_ID;

    if (ADK_IPSEC_IPV4 == ipsecMetaData->dscpVal_version)
    {
        ipUliImc = (adk_ip_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
        destFunc = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].egressUtilityCounters.ipsec_eg_la_num_pdus_rcvd_after_outbound_processing);
        ipUliImc->frag_count = 1;
        ipUliImc->sender = IP_ULI_SENDER_IPSEC;
        ipUliImc->pktOrigin = ipsecMetaData->path;
        ipUliImc->do_not_post_frag = FALSE;
        ipUliImc->inner_l4_protocol = 0xFF;
    }
    else
    {
        ipv6_imc = (adk_ipv6_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
        destFunc = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].egressUtilityCounters.ipsec_eg_la_num_pdus_rcvd_after_outbound_processing);
        ipv6_imc->frag_count = 1;
        ipv6_imc->sender = IP6_ULI_SENDER_IPSEC;
        ipv6_imc->pktOrigin = ipsecMetaData->path;
        ipv6_imc->do_not_post_frag = FALSE;
        ipv6_imc->inner_l4_protocol = 0xFF;
        ipv6_imc->frag_count = INVALID_FRAG_COUNT;
    }
    taskList[0] = taskInfo;
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_TX_TS();
    if (likely(adkDestEntryFunction_g[destFunc]))
    {
        ret = adk_pal_dp_stage_send((void **)taskList, 1, 1, destFunc, 0x00,
                0x00, ADK_PAL_DP_BATCH_CB_FN_IPSEC, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
        if(unlikely(ret == ADK_E_INTERNAL_ERROR))
        {
            adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
            adk_pktio_task_buffer_free (taskInfo);
            return ADK_E_DROP_TASK;
        }
    }
    else
    {
        adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (taskInfo);
        return ADK_E_DROP_TASK;
    }

    return ret;
}

static inline adk_status_t
adk_ipsec_pre_frag_helper_function(adk_ipsec_dp_uli_common_params_t *info,
                                   adk_ipsec_stat_t *globalStats)
{
#define egSaInfo adk_ipsec_dp_tbl_gbl_ptr->eSATbl[saId]    
    adk_status_t ret = ADK_SUCCESS;
    struct rte_mbuf   *pktMbuf = (struct rte_mbuf *)info->taskInfo;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    adk_pktio_task_t *currentTask;
    uint16_t finalPktLen = 0;
    uint8_t *pdu, i;
    /* IMP: num_frags is set to 1 to be sent to IP incase of no pre-frag */
    uint8_t num_frags = 1;
    info->operType = ADK_IPSEC_INLINE;
    uint32_t egressInterfaceStatsId = adkMetaData->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id;

    /* Fragmenting the inner packet based on frag_mtu */
    if (ADK_IPSEC_IPV4 == info->version)
    {
        adk_ip_dp_frag_error_t ipv4_frag_result;
        uint16_t total_payload=0;

        ipv4_frag_result = adk_ip_dp_fragment(&info->taskInfo, info->frag_mtu, &num_frags, &total_payload, 0/* TO DO: should be based on config */);
        if (unlikely(ADK_IP_DP_FRAG_OK != ipv4_frag_result))
        {
            ++globalStats->egressStats.ipsec_eg_global_pre_frag_error_drop;
            return ADK_E_DROP_TASK;
        }
    }
    else
    {
        adk_ipv6_dp_frag_error_t ipv6_frag_result;
        uint32_t fragOctets = 0;
        ipv6_frag_result = adk_ipv6_dp_fragment(&info->taskInfo, info->coreIndex, info->frag_mtu, &num_frags, &fragOctets, IPV6_INTERFACE_TYPE_TUNNEL);
        if (unlikely(ADK_IPV6_DP_FRAG_OK != ipv6_frag_result))
        {
            ++globalStats->egressStats.ipsec_eg_global_pre_frag_error_drop;
            return ADK_E_DROP_TASK;
        }
        /* Adding extra 8 bytes for the fragmentation header */
        pktMbuf->l3_len += 8;
    }
    currentTask = info->taskInfo;
    /* Adding the tunnel header and trailer for all the frags. */
    for (i = 0; (i < num_frags) && (currentTask); i++)
    {
         pktMbuf = (struct rte_mbuf *)currentTask;
         adkMetaData = ADK_METADATA_PTR(currentTask); 
         ADK_PDU_PTR_GET(currentTask, pdu, uint8_t *);
         if (ADK_IPSEC_IPV4 == info->version)
         {
             struct ipv4_hdr *ipv4Hdr = NULL;
             ipv4Hdr = (struct ipv4_hdr *)pdu;
             info->innerPktLen = ntohs(ipv4Hdr->total_length);
         }
         else
         {
             struct ipv6_hdr *ipv6Hdr = NULL;
             ipv6Hdr = (struct ipv6_hdr *)pdu;
             info->innerPktLen = ntohs(ipv6Hdr->payload_len) + sizeof(struct ipv6_hdr);
         }
         uint16_t polIndex, saId;
         if ((i == 0) || (info->singleSa))
         {
             polIndex = info->polIndex;
             saId = info->saId;
         }
         else
         {
             polIndex = info->nonInitialPolIndex;
             saId = info->nonInitialSaId;
         }
         rte_prefetch0(&adk_ipsec_dp_tbl_gbl_ptr->eSATbl[saId]);
         ret = adk_ipsec_prepare_proto_op(info->coreIndex, currentTask, adkMetaData, &egSaInfo,
                                          info->version, info->innerPktLen,
                                          saId,
                                          info->configDF,
                                          globalStats, (uint16_t *)&info->innerIPHdrOffset,
                                          (i != (num_frags - 1))?true:false);
         if (likely(ret == ADK_SUCCESS))
         {
             if (!egSaInfo.ipVersion)
             {
                 if (unlikely(egSaInfo.isNATTEnable))
                     info->ipHdrLen = IPV4_HDR_LEN + UDPV4_HDR_LEN;
                 else
                     info->ipHdrLen = IPV4_HDR_LEN;
             }
             else
             {
                 info->ipHdrLen = sizeof(struct ipv6_hdr);
             }

             finalPktLen = adk_ipsec_inline_prepare_crypto_op(currentTask,
                                                &egSaInfo,
                                                info->version,
                                                info->innerPktLen);
             if (unlikely(0 == finalPktLen))
             {
                 ++globalStats->egressStats.ipsec_eg_global_pad_alloc_fail_drop;
                 return ADK_E_DROP_TASK;
             }

             egSaInfo.bytesUsed += finalPktLen;
             /* modify outer IP header before sending packet for encryption */
             /* now fix the total length field of outer ip */
             if (!egSaInfo.ipVersion)
             {
                 struct ipv4_hdr *outerIpv4 = NULL;
                 uint16_t ipId;
                 outerIpv4 = rte_pktmbuf_mtod(pktMbuf, struct ipv4_hdr *);
                 /*Set the IP Id*/
                 ipId = ( (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq >= adk_ipsec_IdSeqRange[info->coreIndex].max) ?
                          (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq = adk_ipsec_IdSeqRange[info->coreIndex].min) :
                          (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq++) );
                 outerIpv4->packet_id = rte_cpu_to_be_16(ipId);
                 outerIpv4->total_length = adk_htons(finalPktLen+info->ipHdrLen+
                                                     egSaInfo.digestLen);
                 if (unlikely(egSaInfo.isNATTEnable))
                 {
                     struct udp_hdr *outerUdp = NULL;
                     outerUdp = (struct udp_hdr*)(outerIpv4 + 1);
                     outerUdp->dgram_len = adk_htons(finalPktLen + UDPV4_HDR_LEN
                                                     + egSaInfo.digestLen);
                     /* as per RFC 3948, section 2.1
                      * the IPv4 UDP Checksum SHOULD be transmitted as a zero value
                      */
                     outerUdp->dgram_cksum = 0;
                     pktMbuf->l2_len = UDPV4_HDR_LEN;
                 }
                 pktMbuf->outer_l3_len = sizeof (struct ipv4_hdr);
                 pktMbuf->ol_flags |= (PKT_TX_OUTER_IP_CKSUM | PKT_TX_OUTER_IPV4 | PKT_TX_TUNNEL_IPIP);
             }
             else 
             {
                 struct ipv6_hdr *outerIpv6;
                 pktMbuf->outer_l3_len = sizeof (struct ipv6_hdr);
                 pktMbuf->l4_len = UDPV4_HDR_LEN;
                 outerIpv6 = rte_pktmbuf_mtod(pktMbuf, struct ipv6_hdr *);
                 outerIpv6->payload_len = adk_htons(finalPktLen +
                                                    egSaInfo.digestLen);
                 if(pktMbuf->ol_flags & PKT_TX_IPV4)
                 {
                     pktMbuf->ol_flags |= (PKT_TX_OUTER_IPV6 | PKT_TX_TUNNEL_IPIP);
                 }
                 else
                 {
                     pktMbuf->ol_flags |= (PKT_TX_OUTER_IPV6 | PKT_TX_TUNNEL_IPIP | PKT_TX_IPV6);
                 }
             }
             
             /*  Increment stats and counters as appropriate. Note that
              *  this is common for both inline and lookaside
              */
             INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, info->version, ipsec_eg_num_pdus_sent_for_outbound_processing);
             INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, info->version, ipsec_eg_tx_pkts);
             if (likely(egSaInfo.secSession))
             {
#if RDK_REL_MAJOR > 14
                 struct ice_dsi_ipsec_metadata iceDsiMdata = { 0 };
//               iceDsiMdata.esn_hi = adk_ipsec_dp_tbl_gbl_ptr->eSATbl[saId].seqNumber >> 32;
                 iceDsiMdata.l4_paylen = info->innerPktLen - pktMbuf->l3_len - pktMbuf->l4_len;
                 /* This field is not used by CPM in egress path,
                  * so, we won't set anything
                  * iceDsiMdata.trailer_len = NOTHING
                  */
                 rte_security_set_pkt_metadata(adk_ipsec_dp_tbl_gbl_ptr->secCtxDP,
                                  egSaInfo.secSession,
                                  pktMbuf, (void *)&iceDsiMdata);
#else
                 rte_security_set_pkt_metadata(adk_ipsec_dp_tbl_gbl_ptr->secCtxDP,
                                  egSaInfo.secSession,
                                  pktMbuf, NULL);
#endif
             }
             else
             {
                 ++globalStats->egressStats.ipsec_eg_global_no_security_session_drop;
                 return ADK_E_DROP_TASK;
             }

             adkMetaData->exception_domain = ADK_FP_ORIGIN_DOMAIN;

             adkMetaData->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS]
                 .stats_id = egressInterfaceStatsId;
             
             adkMetaData->innerIPVersion = info->version;
             adkMetaData->innerIPHdrOffset = info->innerIPHdrOffset;

             if (!egSaInfo.ipVersion)
             {
                 adkMetaData->l4_offset = sizeof (struct ipv4_hdr);
                 adk_ip_dp_uli_imc_data_t *ipUliImc = NULL;
                 ipUliImc = (adk_ip_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
                 ipUliImc->sender = IP_ULI_SENDER_IPSEC;
                 ipUliImc->pktOrigin = info->pktOrigin;
                 ipUliImc->do_not_post_frag = TRUE;
                 ipUliImc->inner_l4_protocol = info->nextHdr;
                 ipUliImc->frag_count = num_frags;
                 info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
             }
             else
             {
                 adkMetaData->l4_offset = sizeof (struct ipv6_hdr);
                 adk_ipv6_dp_uli_imc_data_t *ipv6_imc = NULL;
                 ipv6_imc = (adk_ipv6_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
                 ipv6_imc->sender = IP6_ULI_SENDER_IPSEC;
                 ipv6_imc->pktOrigin = info->pktOrigin;
                 ipv6_imc->do_not_post_frag = TRUE;
                 ipv6_imc->inner_l4_protocol = info->nextHdr;
                 ipv6_imc->frag_count = num_frags;
                 info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
             }
             ++adk_ipsec_dp_tbl_gbl_ptr->egSAStats[info->coreIndex][saId].ipsec_eg_sa_tot_out_pkts;
         }
         else
         {
             return ret;
         }
         currentTask = ADK_METADATA_PARAM((adk_pktio_task_t *)(currentTask), next_frag);
    }
    return ADK_SUCCESS;
}

/* Function to perform pre-fragmentation based on Single SA or Two SAs if
 * preFrag is enabled, pathMTU is not equal to 0 and final output packet length
 * is greater than pathMTU.
 * If SingleSa is not enabled, then second policy lookup is done on the non-initial 
 * fragments. If the lookup is successful, secondPolIndex and secondSAID are retieved.
 * Once the fragmentation is done, the fragments are encapsulated with tunnel header and
 * crypto trailer.
 * The packet is sent to IP module after updating the required metadata flags.
 */
 
static adk_status_t
adk_ipsec_proc_pre_frag(adk_ipsec_dp_uli_common_params_t *info)
{
    adk_status_t      ret = ADK_SUCCESS;
    struct rte_mbuf   *pktMbuf = (struct rte_mbuf *)info->taskInfo;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    uint8_t *pdu;
    adk_ipsec_stat_t      *globalStats;
    info->singleSa = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].useSingleSAInPrefrag; 
    
    ADK_PDU_PTR_GET(info->taskInfo, pdu, uint8_t *);

    if (adkMetaData->ip_offset != 0)
    {
        ADK_PDU_HEAD_PULL(info->taskInfo, pdu, (uint8_t *), adkMetaData->ip_offset);
        if(NULL == pdu)
        {
            ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid PDU pointer. IP offset is greater than PDU size\n");
            return ADK_E_DROP_TASK;
        }
        adkMetaData->l4_offset -= adkMetaData->ip_offset;
        adkMetaData->ip_offset = 0;
    }

    if (ADK_IPSEC_IPV4 == info->version)
    {
        adkMetaData->l4_offset = IPV4_HDR_LEN;
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
        struct ipv4_hdr *ipv4Hdr = NULL;
        ipv4Hdr = (struct ipv4_hdr *)pdu;
        if((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP_ULI_SENDER_IPLLI != info->pktOrigin))
        {
            struct udp_hdr  *udpHdr = NULL;
            udpHdr = (struct udp_hdr*)(ipv4Hdr + 1);
            udpHdr->dgram_cksum = 0;
            if(pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
            {
                udpHdr->dgram_cksum = ipv4_udp_checksum_gen(pktMbuf,
                                    rte_be_to_cpu_16(udpHdr->dgram_len),
                                    ipv4Hdr->src_addr, ipv4Hdr->dst_addr);
            }
        }
        pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        if(IP_ULI_SENDER_IPLLI == info->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }
    }
    else
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
        struct ipv6_hdr *ipv6Hdr = NULL;
        ipv6Hdr = (struct ipv6_hdr *)pdu;
        if((IPPROTO_TCP == info->nextHdr) ||
           (IPPROTO_UDP == info->nextHdr) ||
           (IPPROTO_SCTP == info->nextHdr))
        {
            struct udp_hdr  *udpHdr = NULL; 
            udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
            if((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP6_ULI_SENDER_IP6LLI != info->pktOrigin))
            {
                udpHdr->dgram_cksum = 0;
                if(pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    udpHdr->dgram_cksum = adk_ipv6_udp_checksum_generation(pktMbuf, adkMetaData->l4_offset, (adk_ipv6_hdr_t *)ipv6Hdr);
                }
            }
        }
        pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        pktMbuf->ol_flags &= ~PKT_TX_IPV6;
        if(IP6_ULI_SENDER_IP6LLI == info->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }

    }
     /* If singleSA is not set, perform second policy lookup for non-initial
     * fragments.
     */
    if (!info->singleSa)
    {   
        info->spdPtn->sport = 0;
        info->spdPtn->dport = 0;
        /* Second Policy lookup */
        ret = adk_ipsec_spd_lkup(info->spdPtn, &info->nonInitialPolIndex);    
        if (likely(ipsec_scratch_pad->flow_cache_entry))
        {
            ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->nonIntialPolIndex, info->nonInitialPolIndex);
            ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->nonInitialpolAction, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].action);
            ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->nonInitialESAstate, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].eSAstate);
            ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->nonInitialSAId, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].eSAIdx);
        }
        if (unlikely(ret != ADK_SUCCESS))
        {
            ++globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop;
            return ADK_E_DROP_TASK;
        }
        if (unlikely(info->nonInitialPolIndex == 0))
        {
            adk_ipsec_audit_event_t auditEventData = {0};
            ++globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop;
            ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_NO_POLICY_2ND_FRAGMENT_2SA_PREFRAG,
                                                &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
            }
            return ret;
        }
        else
        {
            if (unlikely(ADK_IPSEC_SP_ACTION_IPSEC != adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].action))
            {
                ++globalStats->egressStats.ipsec_eg_global_spd_out_not_ipsec_drop;
                return ADK_E_DROP_TASK;
            }
            info->nonInitialSaId = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].eSAIdx;
            if (unlikely(adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].eSAstate ==
                INVALID_SA))
            {
                INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, info->version, ipsec_eg_exception_sent);
                adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].saNotFoundErrDrop++;
                return ADK_E_DROP_TASK;
            }
        } 
    } /* !info->singleSa */
    
    ret = adk_ipsec_pre_frag_helper_function(info, globalStats);
    if(likely(ipsec_scratch_pad->flow_cache_entry))
       ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->ol_flags, pktMbuf->ol_flags);
    return ret;
}

/* Function to perform pre-fragmentation based on Single SA or Two SAs if
 * preFrag is enabled, pathMTU is not equal to 0 and final output packet length
 * is greater than pathMTU.
 * If SingleSa is not enabled, then second policy lookup is done on the non-initial 
 * fragments. If the lookup is successful, secondPolIndex and secondSAID are retieved.
 * Once the fragmentation is done, the fragments are encapsulated with tunnel header and
 * crypto trailer.
 * The packet is sent to IP module after updating the required metadata flags.
 */
 
static adk_status_t
adk_ipsec_proc_pre_frag_uli_flow_cache_pkt_path(adk_ipsec_dp_uli_common_params_t *info, 
                                                adk_sif_dp_scratchpad_space_t *scratch_pad_p)
{
    adk_status_t      ret = ADK_SUCCESS;
    struct rte_mbuf   *pktMbuf = (struct rte_mbuf *)info->taskInfo;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    uint8_t *pdu;
    uint8_t version = scratch_pad_p->ip_version;
    adk_ipsec_stat_t      *globalStats;
    uint8_t     nextHdr = 0;
    info->singleSa = scratch_pad_p->flow_cache_entry->useSingleSAInPrefrag; 
    scratch_pad_p->pktOrigin = info->pktOrigin;
    ADK_PDU_PTR_GET(info->taskInfo, pdu, uint8_t *);
    
    if (scratch_pad_p->ip_offset != 0)
    {
        ADK_PDU_HEAD_PULL(info->taskInfo, pdu, (uint8_t *), scratch_pad_p->ip_offset);
        if(NULL == pdu)
        {
            ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid PDU pointer. IP offset is greater than PDU size\n");
            return ADK_E_DROP_TASK;
        }
        scratch_pad_p->l4_offset -= scratch_pad_p->ip_offset;
        scratch_pad_p->ip_offset = 0;
    }

    if (ADK_IPSEC_IPV4 == version)
    {
        struct ipv4_hdr *ipv4Hdr = NULL;
        scratch_pad_p->l4_offset = IPV4_HDR_LEN;
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
        ipv4Hdr = (struct ipv4_hdr *)pdu;
        nextHdr = scratch_pad_p->flow_cache_entry->nextHdr;
        if((ADK_IP_PROTOCOL_UDP == nextHdr) && (IP_ULI_SENDER_IPLLI != scratch_pad_p->pktOrigin))
        {
            struct udp_hdr  *udpHdr = NULL;
            udpHdr = (struct udp_hdr*)(ipv4Hdr + 1);
            udpHdr->dgram_cksum = 0;
            if(pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
            {
                udpHdr->dgram_cksum = ipv4_udp_checksum_gen(pktMbuf,
                                    rte_be_to_cpu_16(udpHdr->dgram_len),
                                    ipv4Hdr->src_addr, ipv4Hdr->dst_addr);
            }
        }
        pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        if(IP_ULI_SENDER_IPLLI == scratch_pad_p->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }
    }
    else
    {
        struct ipv6_hdr *ipv6Hdr = NULL;
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
        ipv6Hdr = (struct ipv6_hdr *)pdu;
        nextHdr = scratch_pad_p->flow_cache_entry->nextHdr;
        if((IPPROTO_TCP == nextHdr) ||
           (IPPROTO_UDP == nextHdr) ||
           (IPPROTO_SCTP == nextHdr))
        {
            struct udp_hdr  *udpHdr = NULL;
            udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
            if((ADK_IP_PROTOCOL_UDP == nextHdr) && (IP6_ULI_SENDER_IP6LLI != scratch_pad_p->pktOrigin))
            {
                udpHdr->dgram_cksum = 0;
                if(pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    udpHdr->dgram_cksum = adk_ipv6_udp_checksum_generation(pktMbuf, adkMetaData->l4_offset, (adk_ipv6_hdr_t *)ipv6Hdr);
                }
            }
        }
        pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        pktMbuf->ol_flags &= ~PKT_TX_IPV6;
        if(IP6_ULI_SENDER_IP6LLI == scratch_pad_p->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }

    }
    /* If singleSA is not set, perform second policy lookup for non-initial
     * fragments.
     */
    if (!info->singleSa)
    {   
        info->nonInitialPolIndex = scratch_pad_p->flow_cache_entry->nonIntialPolIndex;
        if (unlikely(info->nonInitialPolIndex == 0))
        {
            adk_ipsec_audit_event_t auditEventData = {0};
            globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop++;
            ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_NO_POLICY_2ND_FRAGMENT_2SA_PREFRAG,
                                                &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
            }
            return ret;
        }
        else
        {
            if (unlikely(ADK_IPSEC_SP_ACTION_IPSEC != scratch_pad_p->flow_cache_entry->nonInitialpolAction))
            {
                ++globalStats->egressStats.ipsec_eg_global_spd_out_not_ipsec_drop;
                return ADK_E_DROP_TASK;
            }
            info->nonInitialSaId = scratch_pad_p->flow_cache_entry->nonInitialSAId;
            if (unlikely(scratch_pad_p->flow_cache_entry->nonInitialESAstate ==
                INVALID_SA))
            {
                INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, version, ipsec_eg_exception_sent);
                ++adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->nonInitialPolIndex].saNotFoundErrDrop;
                return ADK_E_DROP_TASK;
            }
        } 
    } /* !info->singleSa */
    
    return adk_ipsec_pre_frag_helper_function(info, globalStats);
}

/* This function would be invoked in adk_ipsec_dp_pkt_proc_uli_single
 * and adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single when
 * policy action is IPsec.
 */
static adk_status_t
adk_ipsec_dp_pkt_proc_action_ipsec (adk_ipsec_dp_uli_common_params_t *info)
{
#define saInfo adk_ipsec_dp_tbl_gbl_ptr->eSATbl[info->saId]
    adk_status_t ret = ADK_SUCCESS;
    struct rte_mbuf *pktMbuf = (struct rte_mbuf *)info->taskInfo;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    struct udp_hdr  *udpHdr = NULL;
    info->innerPktLen = 0;
    adk_ipsec_stat_t *globalStats = NULL;
    bool eSAstate, preFragEnabled;
    uint16_t pathMTU, ip_offset;
    uint8_t *pdu;
    ADK_PDU_PTR_GET(info->taskInfo, pdu, uint8_t *);
    int coreIndex = info->coreIndex;
    adk_sif_dp_scratchpad_space_t *scratch_pad;
    scratch_pad = (ADK_SIF_DP_GET_SCRATCHPAD_PTR(info->coreIndex));
    bool isFastPath = (ADK_E_FLOW_CACHE_PKT_FAST_PATH == (ADK_SIF_DP_GET_SCRATCHPAD_PTR(info->coreIndex))->pktPath);
    if (likely(isFastPath)) 
    {
        eSAstate = scratch_pad->flow_cache_entry->eSAstate;
        info->saId = scratch_pad->flow_cache_entry->SAId;
        preFragEnabled = scratch_pad->flow_cache_entry->preFragEnabled;
        pathMTU = scratch_pad->flow_cache_entry->pathMtu;
        ip_offset = scratch_pad->ip_offset;
        info->configDF = scratch_pad->flow_cache_entry->configDF;
    } 
    else 
    {
        eSAstate = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].eSAstate;
        info->saId = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].eSAIdx;
        preFragEnabled = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].preFragEnable;
        pathMTU = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].pathMTU;
        ip_offset = adkMetaData->ip_offset;
        info->configDF = adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].configDF;
    }
    if (ADK_IPSEC_IPV4 == info->version) 
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
        ipv4Hdr = (struct ipv4_hdr *)(pdu + ip_offset);
        info->innerPktLen = ntohs(ipv4Hdr->total_length);
        if ((ADK_IP_PROTOCOL_UDP != info->nextHdr) || (IP_ULI_SENDER_IPLLI == info->pktOrigin))
        {
            /* If nextHdr is not UDP or if it is forwarded packet */
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
    }
    else if (ADK_IPSEC_IPV6 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
        ipv6Hdr = (struct ipv6_hdr *)(pdu + ip_offset);
        info->innerPktLen = ntohs(ipv6Hdr->payload_len) + sizeof(struct ipv6_hdr);
        if ((ADK_IP_PROTOCOL_UDP != info->nextHdr) || (IP6_ULI_SENDER_IP6LLI == info->pktOrigin))
        {
            /* If nextHdr is not UDP or if it is forwarded packet */
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
    }

    rte_prefetch0(&adk_ipsec_dp_tbl_gbl_ptr->eSATbl[info->saId]);
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_RX_TS();
#endif
    if (unlikely(INVALID_SA == eSAstate))
    {
        adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].saNotFoundErrDrop++;
        ++globalStats->egressStats.ipsec_eg_global_sa_not_found_drop;
        adk_ipsec_sa_aquire_excep_data_t sa_aquire_excep_data = {0};
        sa_aquire_excep_data.polIndex = info->polIndex - 1;
        ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_SA_ACQUIRE,
                                            &sa_aquire_excep_data, info->coreIndex,
                                            info->version, ADK_IPSEC_OUTBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
        }
        return ret;
    }
    uint32_t finalPktLen = 0;
    uint16_t ipId;
    /* If pathMTU is 0, Pre-fragmentation will not be done even if preFragEnable
     * is set. 
     */
    if ((preFragEnabled) &&
        (pathMTU != 0))
    {
        uint16_t dataLen;
        if (!saInfo.ipVersion)
        {
            if (unlikely(saInfo.isNATTEnable))
                info->ipHdrLen = IPV4_HDR_LEN + UDPV4_HDR_LEN;
            else
                info->ipHdrLen = IPV4_HDR_LEN;
        }
        else
        {
            info->ipHdrLen = adkMetaData->l4_offset - adkMetaData->ip_offset;
        }
        dataLen = RTE_ALIGN_CEIL((info->innerPktLen)  + 2, saInfo.blockSize);
        finalPktLen = /* Outer IP header length */
            info->ipHdrLen +
            /* ESP header length */
            ESPV4_HDR_LEN + 
            /* Encryption algo IV length */
            saInfo.ivLen +
            /* Inner IP packet lenth + Necessary Padding length according to crypto block size */
            dataLen +
            /* ICV length */
            saInfo.digestLen;    
        if (finalPktLen > pathMTU)
        {
            if (unlikely((ADK_IPSEC_IPV4 == info->version) && (ipv4Hdr->fragment_offset & rte_cpu_to_be_16(IPV4_HDR_DF_FLAG))))
            {
                adk_ipsec_audit_event_t auditEventData = {0};
                ++globalStats->egressStats.ipsec_eg_global_prefrag_reqd_df_bit_set_drop_count;
                ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_EG_PREFRAG_REQD_BUT_DF_BIT_SET,
                                                    &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
                if(unlikely(ADK_E_INVALID_HANDLE == ret))
                {
                    ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
                }
                return ret;
            }

            /* Fragment inner packet based on frag_mtu */
            info->frag_mtu = pathMTU -
                info->ipHdrLen -
                ESPV4_HDR_LEN -
                saInfo.ivLen -
                (saInfo.blockSize - 1) -
                saInfo.digestLen;
            if (likely(isFastPath))
            {
            scratch_pad->mtu = info->frag_mtu;
            ret = adk_ipsec_proc_pre_frag_uli_flow_cache_pkt_path(info, 
                                                                  scratch_pad);
            }
            else
            {
                ret = adk_ipsec_proc_pre_frag(info);
            }
            return ret;
        }
    } /* preFragEnable && pathMTU !=0 */
    if (unlikely(ADK_IPSEC_SA_LOOKASIDE == saInfo.saMode))
    {
        info->operType = ADK_IPSEC_LOOKASIDE;
        if (ADK_IPSEC_IPV4 == info->version)
        {
            if ((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP_ULI_SENDER_IPLLI != info->pktOrigin))
            {
                udpHdr = (struct udp_hdr *)(ipv4Hdr + 1);
                udpHdr->dgram_cksum = 0;
                if (pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    udpHdr->dgram_cksum = ipv4_udp_checksum_gen(pktMbuf,
                                          rte_be_to_cpu_16(udpHdr->dgram_len),
                                          ipv4Hdr->src_addr, ipv4Hdr->dst_addr);
                }
            }
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
            ipv4Hdr->hdr_checksum = rte_ipv4_cksum(ipv4Hdr);
        }
        else
        {
            if ((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP6_ULI_SENDER_IP6LLI != info->pktOrigin))
            {
                udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
                udpHdr->dgram_cksum = 0;
                if (pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    udpHdr->dgram_cksum = adk_ipv6_udp_checksum_generation(pktMbuf, adkMetaData->l4_offset, (adk_ipv6_hdr_t *)ipv6Hdr);
                }
            }
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
    } /* ADK_IPSEC_SA_LOOKASIDE */
    else
    {
        if (ADK_IPSEC_IPV4 == info->version)
        {
            if ((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP_ULI_SENDER_IPLLI != info->pktOrigin))
            {
                udpHdr = (struct udp_hdr *)(ipv4Hdr + 1);
                udpHdr->dgram_cksum = 0;
                if (pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    /* Offload the checksum calculation to hardware */
                    udpHdr->dgram_cksum = rte_ipv4_phdr_cksum((struct ipv4_hdr *)ipv4Hdr,
                            pktMbuf->ol_flags);
                }
            }
        }
        else
        {
            if ((ADK_IP_PROTOCOL_UDP == info->nextHdr) && (IP6_ULI_SENDER_IP6LLI != info->pktOrigin))
            {
                udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
                udpHdr->dgram_cksum = 0;
                if (pktMbuf->ol_flags & PKT_TX_UDP_CKSUM)
                {
                    /* Offload the checksum calculation to hardware */
                    uint32_t cksum = 0;
                    cksum = adk_ipv6_phdr_cksum((void *)ipv6Hdr, adk_ntohs(udpHdr->dgram_len));
                    udpHdr->dgram_cksum  = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);
                }
            }
        }
    }
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_ACTION_IPSEC_TX_TS();
#endif
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_RX_TS();
#endif
    ret = adk_ipsec_prepare_proto_op(info->coreIndex, info->taskInfo, adkMetaData, &saInfo,
            info->version, info->innerPktLen,
            info->saId,
            info->configDF,
            globalStats, (uint16_t *)&info->innerIPHdrOffset, false);
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_PROTO_TX_TS();
#endif
    if (likely(ret == ADK_SUCCESS))
    {
        if (!saInfo.ipVersion)
        {
            if (unlikely(saInfo.isNATTEnable))
                info->ipHdrLen = IPV4_HDR_LEN + UDPV4_HDR_LEN;
            else
                info->ipHdrLen = IPV4_HDR_LEN;
        }
        else
        {
            info->ipHdrLen = sizeof(struct ipv6_hdr);
        }
        if (unlikely(ADK_IPSEC_LOOKASIDE == info->operType))
        {
            /* proceed only if we have opData buffer
             *                         */
            info->opData = saInfo.lkasideInfo->cryptoOp;
            if (unlikely(NULL == info->opData))
            {
                ++globalStats->egressStats.ipsec_eg_global_la_no_buffer_drop;
                return ADK_E_DROP_TASK;
            }
        }
#if defined(IPSEC_ULI_SUB_BLOCKS)
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_RX_TS();
#endif
        if(likely(ADK_IPSEC_INLINE == info->operType))
        {
            finalPktLen = adk_ipsec_inline_prepare_crypto_op(info->taskInfo,
                    &saInfo,
                    info->version,
                    info->innerPktLen);
        }
        else
        {
            finalPktLen = adk_ipsec_outbound_la_prepare_crypto_op(info->taskInfo, info->opData,
                    &saInfo,
                    info->ipHdrLen, info->version,
                    info->innerPktLen);
        }
#if defined(IPSEC_ULI_SUB_BLOCKS)
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PREPARE_CRYPTO_TX_TS();
#endif
        if (unlikely(0 == finalPktLen))
        {
            ++globalStats->egressStats.ipsec_eg_global_pad_alloc_fail_drop;
            return ADK_E_DROP_TASK;
        }
#if defined(IPSEC_ULI_SUB_BLOCKS)
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_RX_TS();
#endif

        saInfo.bytesUsed += finalPktLen;
        /* modify outer IP header before sending packet for encryption */
        /* now fix the total length field of outer ip */
        if (!saInfo.ipVersion)
        {
            struct ipv4_hdr *outerIpv4 = NULL;
            outerIpv4 = rte_pktmbuf_mtod(pktMbuf, struct ipv4_hdr *);
            /*Set the IP Id*/
            ipId = ( (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq >= adk_ipsec_IdSeqRange[info->coreIndex].max) ?
                    (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq = adk_ipsec_IdSeqRange[info->coreIndex].min) :
                    (adk_ipsec_IdSeqDB[info->coreIndex].ipIdSeq++) );
            outerIpv4->packet_id = rte_cpu_to_be_16(ipId);
            outerIpv4->total_length = adk_htons(finalPktLen+info->ipHdrLen+
                    saInfo.digestLen);
            outerIpv4->hdr_checksum = 0;
            if (unlikely(saInfo.isNATTEnable))
            {
                struct udp_hdr  *outerUdp = NULL;
                outerUdp = (struct udp_hdr*)(outerIpv4 + 1);
                outerUdp->dgram_len = adk_htons(finalPktLen + UDPV4_HDR_LEN
                        + saInfo.digestLen);
                /* as per RFC 3948, section 2.1
                 *                              * the IPv4 UDP Checksum SHOULD be transmitted as a zero value
                 *                                                           */
                outerUdp->dgram_cksum = 0;
                pktMbuf->l2_len = UDPV4_HDR_LEN;
            }
            if (likely(ADK_IPSEC_INLINE == info->operType))
            {
                pktMbuf->outer_l3_len = sizeof (struct ipv4_hdr);
                pktMbuf->l4_len = UDPV4_HDR_LEN;
                pktMbuf->ol_flags |= (PKT_TX_OUTER_IP_CKSUM | PKT_TX_OUTER_IPV4 | PKT_TX_TUNNEL_IPIP);
            }
            else
            {
                pktMbuf->ol_flags |= PKT_TX_IPV4 | PKT_TX_IP_CKSUM;
                pktMbuf->l3_len = sizeof (struct ipv4_hdr);
            }
        }
        else
        {
            struct ipv6_hdr *outerIpv6 = NULL;
            if (likely(ADK_IPSEC_INLINE == info->operType))
            {
                pktMbuf->outer_l3_len = sizeof (struct ipv6_hdr);
                pktMbuf->l4_len = UDPV4_HDR_LEN;
                outerIpv6 = rte_pktmbuf_mtod(pktMbuf, struct ipv6_hdr *);
                outerIpv6->payload_len = adk_htons(finalPktLen +
                        saInfo.digestLen);
                if (pktMbuf->ol_flags & PKT_TX_UDP_CKSUM || pktMbuf->ol_flags & PKT_TX_IPV4)
                {
                    pktMbuf->ol_flags |= (PKT_TX_OUTER_IPV6 | PKT_TX_TUNNEL_IPIP);
                }
            }
            else
            {
                outerIpv6 = rte_pktmbuf_mtod(pktMbuf, struct ipv6_hdr *);
                outerIpv6->payload_len = adk_htons(finalPktLen +
                        saInfo.digestLen);
                pktMbuf->ol_flags &= ~PKT_TX_IPV6;
            }
        }
        /*  Increment stats and counters as appropriate. Note that
         *  this is common for both inline and lookaside
         */
        INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, info->version, ipsec_eg_num_pdus_sent_for_outbound_processing);
        INCREMENT_EGRESS_UTILITY_COUNTER(info->coreIndex, info->version, ipsec_eg_tx_pkts);
        if (unlikely((!isFastPath) && scratch_pad->flow_cache_entry))
        {
            ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad->flow_cache_entry->ol_flags, pktMbuf->ol_flags);
        }
        
        adkMetaData->innerIPVersion = info->version;
        adkMetaData->innerIPHdrOffset = info->innerIPHdrOffset;

        if (unlikely(ADK_IPSEC_LOOKASIDE == info->operType))
        {
            ipsec_dp_outbound_meta_data_t      *ipsecMetaData = NULL;
            ipsecMetaData = (ipsec_dp_outbound_meta_data_t *)&(adkMetaData->imc_param_start);
            ipsecMetaData = (ipsec_dp_outbound_meta_data_t *)&(adkMetaData->imc_param_start);
            ipsecMetaData->path = info->pktOrigin; 
            ipsecMetaData->saId = info->saId;
            ipsecMetaData->dscpVal_version = saInfo.ipVersion ? ADK_IPSEC_IPV6: ADK_IPSEC_IPV4;
            ADK_METADATA_PARAM((adk_pktio_task_t *)(info->taskInfo), next_frag) = NULL;

            union rte_event_crypto_metadata cryptoMetaData = { 0 };
            cryptoMetaData.request_info.cdev_id = saInfo.lkasideInfo->cryptoDevId;
            cryptoMetaData.request_info.queue_pair_id = saInfo.lkasideInfo->cryptoQP;
            adk_pal_dp_stage_info_t *stageInfo = NULL;
            adk_pal_dp_stage_info_get(ADK_IPSEC_ULI_FROM_CRYPTO_FN_ID, &stageInfo);
            cryptoMetaData.response_info.queue_id = stageInfo->event_q_id;
            cryptoMetaData.response_info.sub_event_type = ADK_IPSEC_ULI_FROM_CRYPTO_FN_ID;

            cryptoMetaData.response_info.flow_id = info->saId;
            ret = rte_cryptodev_sym_session_set_user_data( saInfo.lkasideInfo->cryptoSession,
                    (void *)&cryptoMetaData,
                    sizeof(cryptoMetaData));
            if (unlikely(ret != 0))
            {
                printf( "Failed to set crypto BD metadata\n");
                return ADK_E_DROP_TASK;
            }

            ret = adk_pal_dp_crypto_send(saInfo.lkasideInfo->cryptoDevId,
                    ADK_IPSEC_ULI_FROM_CRYPTO_FN_ID,
                    0, /* priority */
                    info->taskInfo,
                    info->opData,
                    ADK_PAL_DP_BATCH_CB_FN_IPSEC,
                    NULL
                    );

            if (unlikely(ADK_E_TASK_CONSUMED != ret))
            {
                return ADK_E_DROP_TASK;
            }

            saInfo.inFlightCnt++;
            ++adk_ipsec_dp_tbl_gbl_ptr->SAUtilityCnt[coreIndex][info->saId].egSaUtilityCounters.ipsec_eg_sa_inflight_pkt;
        }
        else
        {
            if (likely(saInfo.secSession))
            {
#if RDK_REL_MAJOR > 14
                struct ice_dsi_ipsec_metadata iceDsiMdata = { 0 };
                //iceDsiMdata.esn_hi = adk_ipsec_dp_tbl_gbl_ptr->eSATbl[saId].seqNumber >> 32;
                iceDsiMdata.l4_paylen = info->innerPktLen - pktMbuf->l3_len - pktMbuf->l4_len;
                /* This field is not used by CPM in egress path,
                 * so, we won't set anything
                 * iceDsiMdata.trailer_len = NOTHING
                 */
                rte_security_set_pkt_metadata(adk_ipsec_dp_tbl_gbl_ptr->secCtxDP,
                        saInfo.secSession,
                        pktMbuf, (void *)&iceDsiMdata);
#else
                rte_security_set_pkt_metadata(adk_ipsec_dp_tbl_gbl_ptr->secCtxDP,
                        saInfo.secSession,
                        pktMbuf, NULL);
#endif
            }
            else
            {
                ++globalStats->egressStats.ipsec_eg_global_no_security_session_drop;
                return ADK_E_DROP_TASK;
            }

            adkMetaData->exception_domain = ADK_FP_ORIGIN_DOMAIN;

            if (!saInfo.ipVersion)
            {
                adkMetaData->l4_offset = sizeof (struct ipv4_hdr);
                adk_ip_dp_uli_imc_data_t *ipUliImc = NULL;
                ipUliImc = (adk_ip_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
                ipUliImc->frag_count = 1;
                ipUliImc->sender = IP_ULI_SENDER_IPSEC;
                ipUliImc->pktOrigin = info->pktOrigin;
                ipUliImc->do_not_post_frag = TRUE;
                ipUliImc->inner_l4_protocol = info->nextHdr;
                info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
            }
            else
            {
                adkMetaData->l4_offset = sizeof (struct ipv6_hdr);
                adk_ipv6_dp_uli_imc_data_t *ipv6_imc = NULL;
                ipv6_imc = (adk_ipv6_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
                ipv6_imc->frag_count = 1;
                ipv6_imc->sender = IP6_ULI_SENDER_IPSEC;
                ipv6_imc->pktOrigin = info->pktOrigin;
                ipv6_imc->do_not_post_frag = TRUE;
                ipv6_imc->inner_l4_protocol = info->nextHdr;
                info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
            }
            ++adk_ipsec_dp_tbl_gbl_ptr->egSAStats[info->coreIndex][info->saId].ipsec_eg_sa_tot_out_pkts;
        }
    }
    else
    {
        return ret;
    }
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_AFTER_CRYPTO_TX_TS();
#endif
    return ret;
}

/* This function would be invoked in adk_ipsec_dp_pkt_proc_uli_single
 * and adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single when
 * policy action is Drop.
 */
inline static adk_status_t
adk_ipsec_dp_pkt_proc_action_drop (adk_ipsec_dp_uli_common_params_t *info)
{
    adk_ipsec_stat_t *globalStats = NULL;
    adk_status_t ret = ADK_SUCCESS;
    adk_ipsec_audit_event_t auditEventData = {0};
    if (ADK_IPSEC_IPV4 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
    }
    else if (ADK_IPSEC_IPV6 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
    }
    globalStats->egressStats.ipsec_eg_global_spd_pol_discard_drop++;
    ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_EG_SPD_OUT_LKUP_DISCARD,
                                        &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
    if(unlikely(ADK_E_INVALID_HANDLE == ret))
    {
        ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
    }
    return ret;
}

/* This function would be invoked in adk_ipsec_dp_pkt_proc_uli_single
 * and adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single when
 * policy action is Bypass.
 */
inline static void
adk_ipsec_dp_pkt_proc_action_bypass (adk_ipsec_dp_uli_common_params_t *info)
{
    adk_ipsec_stat_t *globalStats = NULL;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    int coreIndex = info->coreIndex;
    if (ADK_IPSEC_IPV4 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
    }
    else if (ADK_IPSEC_IPV6 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
    }
    adkMetaData->innerIPVersion = info->version;
    adkMetaData->innerIPHdrOffset = 0;
    adkMetaData->saID = ADK_IPSEC_INVALID_SA_ID;

    ++globalStats->egressStats.ipsec_eg_global_spd_pol_bypassCnt;
    if (ADK_IPSEC_IPV4 == info->version)
    {
        adk_ip_dp_uli_imc_data_t *ipUliImc = NULL;
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].egressUtilityCounters.ipsec_eg_tx_pkts);
        ipUliImc = (adk_ip_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
        ipUliImc->frag_count = 1;
        ipUliImc->sender = IP_ULI_SENDER_IPSEC;
        ipUliImc->pktOrigin = info->pktOrigin;
        ipUliImc->do_not_post_frag = FALSE;
        ipUliImc->inner_l4_protocol = 0xFF;
        info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
    }
    else
    {
        adk_ipv6_dp_uli_imc_data_t *ipv6_imc = NULL;
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].egressUtilityCounters.ipsec_eg_tx_pkts);
        ipv6_imc = (adk_ipv6_dp_uli_imc_data_t *)(&(adkMetaData->imc_param_start));
        ipv6_imc->frag_count = INVALID_FRAG_COUNT;
        ipv6_imc->sender = IP6_ULI_SENDER_IPSEC;
        ipv6_imc->pktOrigin = info->pktOrigin;
        ipv6_imc->do_not_post_frag = FALSE;
        ipv6_imc->inner_l4_protocol = 0xFF;
        info->destFunc = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
    }
}

static adk_status_t
adk_ipsec_dp_pkt_proc_uli_single (adk_ipsec_dp_uli_common_params_t *info)
{
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_RX_TS();
#endif
#if !defined(IPSEC_ULI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_RX_TS();
#endif
    adk_status_t      ret = ADK_SUCCESS;
    struct rte_mbuf   *pktMbuf = (struct rte_mbuf *)info->taskInfo;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    adk_ipsec_dp_uli_imc_param_t  *ipsecUliImc = (adk_ipsec_dp_uli_imc_param_t *)
                                                  &(adkMetaData->imc_param_start);

    uint8_t     *pdu;
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    struct udp_hdr  *udpHdr = NULL;
    adk_ipsec_spd_sel_t   spdPtn = {0};
    info->spdPtn = &spdPtn;
    adk_ipsec_stat_t      *globalStats;
    info->pktOrigin = ipsecUliImc->pktOrigin;
    info->operType = ADK_IPSEC_INLINE;

#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_TX_TS();
#endif
    ADK_PDU_PTR_GET(info->taskInfo, pdu, uint8_t *);
    ipsec_scratch_pad = ADK_SIF_DP_GET_SCRATCHPAD_PTR(info->coreIndex);
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_RX_TS();
#endif

    info->version = (*(uint8_t *)(pdu + adkMetaData->ip_offset)) >> 4;
    if (ADK_IPSEC_IPV4 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[info->coreIndex].egressUtilityCounters.ipsec_eg_total_pdu_recv_cnt);
        ipv4Hdr = (struct ipv4_hdr *)(pdu + adkMetaData->ip_offset);
        spdPtn.version = ADK_IPSEC_IPV4;
        spdPtn.sip_addr[0] = ipv4Hdr->src_addr;
        spdPtn.dip_addr[0] = ipv4Hdr->dst_addr;
        spdPtn.dscpVal = (ipv4Hdr->type_of_service >> 2) & 0x3F;
        /* this is required to be done for DSCP and protocol fields,
         * because in reality these are 8 bit fields!!
         */
        spdPtn.dscpVal = rte_cpu_to_be_16(spdPtn.dscpVal);
        spdPtn.protocol = rte_cpu_to_be_16(ipv4Hdr->next_proto_id);
        info->nextHdr = ipv4Hdr->next_proto_id;
        if ((IPPROTO_UDP == info->nextHdr) ||
            (IPPROTO_TCP == info->nextHdr) ||
            (IPPROTO_SCTP == info->nextHdr))
        {
            udpHdr = (struct udp_hdr *)(ipv4Hdr + 1);
            spdPtn.sport = udpHdr->src_port;
            spdPtn.dport = udpHdr->dst_port;
            if (unlikely((ADK_IP_PROTOCOL_UDP != info->nextHdr) || (IP_ULI_SENDER_IPLLI == info->pktOrigin)))
            {
                /* If nextHdr is not UDP or if it is forwarded packet */
                pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
            }
        }
        else if(IPPROTO_ICMP == info->nextHdr)
        {
            struct icmp_hdr *icmpHdr = (struct icmp_hdr *)(ipv4Hdr + 1);
            spdPtn.sport = icmpHdr->icmp_type;
            spdPtn.dport = icmpHdr->icmp_code;
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
        else
        {
            spdPtn.sport = 0;
            spdPtn.dport = 0;
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }

        if (IP_ULI_SENDER_IPLLI == info->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }
    }
    else if(ADK_IPSEC_IPV6 == info->version)
    {
        globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[info->coreIndex].egressUtilityCounters.ipsec_eg_total_pdu_recv_cnt);
        ipv6Hdr = (struct ipv6_hdr *)(pdu + adkMetaData->ip_offset);
        spdPtn.version = ADK_IPSEC_IPV6;
        rte_memcpy(spdPtn.sip_addr, ipv6Hdr->src_addr, 16);
        rte_memcpy(spdPtn.dip_addr, ipv6Hdr->dst_addr, 16);
        /* Extracting the 6-bit DSCP value from 8-bit value also containing ECN
         * bits. Bits 4 - 9 (from the left, starting with 0) represent the DSCP
         * value.
         * Conversion to 16 bit is required to be done for DSCP and protocol fields,
         * because in reality these are 8 bit fields!!
         */
        spdPtn.dscpVal = rte_cpu_to_be_16((ntohl(ipv6Hdr->vtc_flow) >> ADK_IPSEC_TC_OFFSET) & 0x3F);
        info->nextHdr = ipsecUliImc->ipv6_nextHdr;
        spdPtn.protocol = rte_cpu_to_be_16(info->nextHdr);
        if((IPPROTO_TCP == info->nextHdr) ||
           (IPPROTO_UDP == info->nextHdr) ||
           (IPPROTO_SCTP == info->nextHdr))
        {
            udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
            spdPtn.sport = udpHdr->src_port;
            spdPtn.dport = udpHdr->dst_port;
            if (unlikely((ADK_IP_PROTOCOL_UDP != info->nextHdr) || (IP6_ULI_SENDER_IP6LLI == info->pktOrigin)))
            {
                /* If nextHdr is not UDP or if it is forwarded packet */
                pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
            }
        }
        else if(IPPROTO_ICMPV6 == info->nextHdr)
        {
            struct icmp_hdr *icmpHdr = (struct icmp_hdr *)(pdu + adkMetaData->l4_offset);
            spdPtn.sport = icmpHdr->icmp_type;
            spdPtn.dport = icmpHdr->icmp_code;
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
        else
        {
            spdPtn.sport = 0;
            spdPtn.dport = 0;
            pktMbuf->ol_flags &= ~PKT_TX_UDP_CKSUM;
        }
        if(IP6_ULI_SENDER_IP6LLI == info->pktOrigin)
        {
            INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[info->coreIndex].egressUtilityCounters.ipsec_eg_fwd_pkts);
        }
    }
    else
    {
        /* IPv4/v6 header validations should be done in IP stack. Ideally, control should not reach here. */
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid IP header version. Dropping the packet");
        return ADK_E_DROP_TASK;
    }

#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_TX_TS();
#endif
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_RX_TS();
#endif
    ret = adk_ipsec_spd_lkup(&spdPtn, &info->polIndex);
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_SPD_LKUP_TX_TS();
#endif
    if (likely(ipsec_scratch_pad->flow_cache_entry))
    {
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->polIndex , info->polIndex);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->polAction , adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].action);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->eSAstate, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].eSAstate);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->SAId, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].eSAIdx);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->preFragEnabled, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].preFragEnable);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->pathMtu, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].pathMTU);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->configDF, adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].configDF);
        ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->useSingleSAInPrefrag, 
                adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].useSingleSAInPrefrag);
    }
    if (unlikely(ret != ADK_SUCCESS))
    {
        adk_ipsec_audit_event_t auditEventData = {0};

        globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop++;
        ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_EG_SPD_OUT_LKUP_DISCARD,
                                            &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
        }
        return ret;
    }
    
    if (unlikely(info->polIndex == 0))
    {
        adk_ipsec_audit_event_t auditEventData = {0};

        globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop++;
        ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_EG_SPD_OUT_LKUP_DISCARD,
                                            &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
        }
        return ret;
    }
    else
    {
        /* first get all the required indices
         */

        switch(adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[info->polIndex].action)
        {
            case ADK_IPSEC_SP_ACTION_IPSEC:
                ret = adk_ipsec_dp_pkt_proc_action_ipsec(info);
                break;
            
            case ADK_IPSEC_SP_ACTION_DROP:
                return adk_ipsec_dp_pkt_proc_action_drop(info);
            
            case ADK_IPSEC_SP_ACTION_BYPASS:
                /*TODO: This can be done later in Lower layer modules such as Ethernet*/
                if (likely(ipsec_scratch_pad->flow_cache_entry))
                    ADK_SYS_DP_FLOW_CACHE_SET_PARAM(ipsec_scratch_pad->flow_cache_entry->ol_flags, pktMbuf->ol_flags);
                adk_ipsec_dp_pkt_proc_action_bypass(info);
                break;
            default:
                ++globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop;
                return ADK_E_DROP_TASK;
        }
    }
#if !defined(IPSEC_ULI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_TX_TS();
#endif
    return ret;
}


adk_status_t
adk_ipsec_dp_pkt_proc_uli_flow_cache_pkt_path_single (adk_ipsec_dp_uli_common_params_t *info)
{
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_RX_TS();
#endif
#if !defined(IPSEC_ULI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_RX_TS();
#endif
    adk_status_t      ret = ADK_SUCCESS;
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(info->taskInfo);
    adk_ipsec_dp_uli_imc_param_t  *ipsecUliImc = (adk_ipsec_dp_uli_imc_param_t *)
                                                  &(adkMetaData->imc_param_start);
    adk_sif_dp_scratchpad_space_t scratch_pad;
    uint8_t     *pdu;
    info->pktOrigin = ipsecUliImc->pktOrigin;
    info->operType = ADK_IPSEC_INLINE;
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_VARIABLES_INIT_TX_TS();
#endif
    ADK_PDU_PTR_GET(info->taskInfo, pdu, uint8_t *);
    scratch_pad = *(ADK_SIF_DP_GET_SCRATCHPAD_PTR(info->coreIndex));

    adk_sys_dp_scratchpad_dump(&scratch_pad);
    info->version = scratch_pad.ip_version;
    info->nextHdr = scratch_pad.flow_cache_entry->nextHdr;
    info->polIndex = scratch_pad.flow_cache_entry->polIndex;
#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_RX_TS();
#endif

    if (ADK_IPSEC_IPV4 == scratch_pad.ip_version)
    {
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[info->coreIndex].egressUtilityCounters.ipsec_eg_total_pdu_recv_cnt);
    }
    else if(ADK_IPSEC_IPV6 == scratch_pad.ip_version)
    {
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[info->coreIndex].egressUtilityCounters.ipsec_eg_total_pdu_recv_cnt);
    }
    else
    {
        /* IPv4/v6 header validations should be done in IP stack. Ideally, control should not reach here. */
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid IP header version. Dropping the packet");
        return ADK_E_DROP_TASK;
    }

#if defined(IPSEC_ULI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_PARSE_PKT_TX_TS();
#endif
    if (likely(scratch_pad.flow_cache_entry->polIndex))
    {
        /* first get all the required indices
         */

        switch(scratch_pad.flow_cache_entry->polAction)
        {
            case ADK_IPSEC_SP_ACTION_IPSEC:
                ret = adk_ipsec_dp_pkt_proc_action_ipsec(info);
                break;
            
            case ADK_IPSEC_SP_ACTION_DROP:
                return adk_ipsec_dp_pkt_proc_action_drop(info);
                
            case ADK_IPSEC_SP_ACTION_BYPASS:
                adk_ipsec_dp_pkt_proc_action_bypass(info);
                break;
            default:
                return ADK_E_DROP_TASK;
        }
    }
    else
    {
        adk_ipsec_audit_event_t auditEventData = {0};
        adk_ipsec_stat_t      *globalStats = NULL;
        if (ADK_IPSEC_IPV4 == scratch_pad.ip_version)
        {
            globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[info->coreIndex];
        }
        else if(ADK_IPSEC_IPV6 == scratch_pad.ip_version)
        {
            globalStats = &adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[info->coreIndex];
        }
        globalStats->egressStats.ipsec_eg_global_spd_pol_default_discard_drop++;
        ret = adk_ipsec_dp_handle_exception(info->taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_EG_SPD_OUT_LKUP_DISCARD,
                                            &auditEventData, info->coreIndex, info->version, ADK_IPSEC_OUTBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            ++globalStats->egressStats.ipsec_eg_global_exception_not_registered;
        }
        return ret;
    }
#if !defined(IPSEC_ULI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_ULI_TX_TS();
#endif
    return ret;
}
