/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include <adk_sys.h>
#include <adk_pmu.h>
#include <adk_ipsec_struct.h>
#include <adk_ipsec_common.h>
#include <adk_ipsec_dp.h>

extern adk_ipsec_dp_table_info_t *adk_ipsec_dp_tbl_gbl_ptr;
/* in case of NAT-T, ipHdrLen indicates, IP hdr len as well as UDP hdr len
 */
inline uint32_t adk_ipsec_inline_prepare_crypto_op(void *mbuf,
                                     adk_ipsec_dp_eg_sa_t *saInfo,
                                     uint8_t ipVersion, uint32_t innerPktLen)
{
    struct rte_mbuf           *pktMbuf = (struct rte_mbuf *)mbuf;
    adk_pktio_task_t          *newMbuf = NULL;

    uint16_t  padLen = 0, dataLen;
    char      *padding;

    /* Padding needed for sending to cryptodev */
    dataLen = RTE_ALIGN_CEIL((innerPktLen)  + 2, saInfo->blockSize);
    padLen = dataLen  - innerPktLen;

    padding = rte_pktmbuf_append(pktMbuf, padLen + saInfo->digestLen);
    if (unlikely(NULL == padding))
    {
        newMbuf = (adk_pktio_task_t *)rte_pktmbuf_alloc(pktMbuf->pool);
        if (unlikely(NULL == newMbuf))
        {
            return 0;
        }
        ADK_PDU_PTR_GET(newMbuf, padding, char *);
        newMbuf->pkt_len = newMbuf->data_len = padLen + saInfo->digestLen;
        rte_pktmbuf_chain(pktMbuf, newMbuf);
    }
    if (padLen)
    {
        if (padLen > 2)
            rte_memcpy(padding, padValues, padLen - 2);
        padding[padLen - 2] = padLen - 2;
        if (ipVersion == ADK_IPSEC_IPV4)
            padding[padLen - 1] = 4;
        else
            padding[padLen - 1] = 41; /* IPv6 */
    }

    dataLen += ESPV4_HDR_LEN + saInfo->ivLen;
    return dataLen;
}

uint32_t adk_ipsec_inbound_la_prepare_crypto_op(void *mbuf, struct rte_crypto_op *opData,
                                                adk_ipsec_dp_ing_sa_t *saInfo,
                                                uint8_t ipHdrLen,
                                                uint32_t innerPktLen)
{
    struct rte_mbuf           *pktMbuf = (struct rte_mbuf *)mbuf;
    struct rte_mbuf           *lastSeg = NULL;
    struct rte_crypto_sym_op  *sym = NULL;

    uint16_t dataLen, dataOffset;
    uint8_t  *iv, *ivData;
    adk_ipsec_dp_lkaside_info_t *lkasideInfo = saInfo->lkasideInfo;

    /* in this case, innerPktLen is actually the entire packet len */
    dataLen = innerPktLen - ipHdrLen - ESPV4_HDR_LEN
              - saInfo->ivLen - saInfo->digestLen;

    if(unlikely(!opData))
    {
        return 0;
    }
    opData->type = RTE_CRYPTO_OP_TYPE_SYMMETRIC;
    opData->status = RTE_CRYPTO_OP_STATUS_NOT_PROCESSED;
    sym = opData->sym;
    rte_crypto_op_attach_sym_session (opData, lkasideInfo->cryptoSession);

    /* Ethernet header is part of packet and so we need to include l2_len to point to
     * correct offset.
     */
    dataOffset = pktMbuf->l2_len + ipHdrLen + ESPV4_HDR_LEN + saInfo->ivLen;
    iv = (uint8_t *)rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *,
            pktMbuf->l2_len + ipHdrLen + ESPV4_HDR_LEN);
    sym->auth.data.offset = pktMbuf->l2_len + ipHdrLen;
    ivData = rte_crypto_op_ctod_offset(opData, uint8_t *, IV_OFFSET);
    /* Copy IV at the end of crypto operation */
    rte_memcpy(ivData, iv, saInfo->ivLen);
    sym->cipher.data.offset = dataOffset;
    sym->cipher.data.length = dataLen;
    sym->auth.data.length = dataLen + ESPV4_HDR_LEN + saInfo->ivLen;
    /* NOTE in inbound case, innerPktLen indicates the entire IP
     * packet length that we received!
     * need multi segment handling here
     */
    if (pktMbuf->nb_segs == 1)
    {
        sym->auth.digest.data =
            rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *, innerPktLen + pktMbuf->l2_len - saInfo->digestLen);
        sym->auth.digest.phys_addr =
            rte_pktmbuf_mtophys_offset(pktMbuf, innerPktLen + pktMbuf->l2_len - saInfo->digestLen);
    }
    else
    {
        /* need to find some optimal way of doing this
         */
        lastSeg = rte_pktmbuf_lastseg(pktMbuf);
        if (likely(rte_pktmbuf_data_len(lastSeg) >= saInfo->digestLen))
        {
            /* thankfully, this segment has auth data
             */
            sym->auth.digest.data =
                rte_pktmbuf_mtod_offset(lastSeg, uint8_t *, rte_pktmbuf_data_len(lastSeg) - saInfo->digestLen);
            sym->auth.digest.phys_addr =
                rte_pktmbuf_mtophys_offset(lastSeg, rte_pktmbuf_data_len(lastSeg) - saInfo->digestLen);
        }
        else
        {
            /* the auth data is split across two segments! Does QAT support this?
             */
            return 0;
        }
    }
    sym->m_src = pktMbuf;
    lkasideInfo->cryptoOp = rte_crypto_op_alloc(lkasideInfo->cryptoPool,
            RTE_CRYPTO_OP_TYPE_SYMMETRIC);
    return sym->auth.data.length;
}

uint32_t adk_ipsec_outbound_la_prepare_crypto_op(void *mbuf, struct rte_crypto_op *opData,
                                     adk_ipsec_dp_eg_sa_t *saInfo,
                                     uint8_t ipHdrLen, uint8_t ipVersion,
                                     uint32_t innerPktLen)
{
    struct rte_mbuf           *pktMbuf = (struct rte_mbuf *)mbuf;
    struct rte_mbuf           *lastSeg = NULL;
    struct rte_crypto_sym_op  *sym = NULL;
    adk_pktio_task_t          *newMbuf = NULL;
    adk_ipsec_dp_lkaside_info_t *lkasideInfo = saInfo->lkasideInfo;
    uint16_t  padLen = 0, dataLen, dataOffset;
    char      *padding;
    uint8_t   *iv, *ivData;

    /* Padding needed for sending to cryptodev */
    dataLen = RTE_ALIGN_CEIL((innerPktLen)  + 2, saInfo->blockSize);
    padLen = dataLen  - innerPktLen;

    padding = rte_pktmbuf_append(pktMbuf, padLen + saInfo->digestLen);
    if (unlikely(NULL == padding))
    {
        newMbuf = (adk_pktio_task_t *)rte_pktmbuf_alloc(pktMbuf->pool);
        if (unlikely(NULL == newMbuf))
        {
            return 0;
        }
        ADK_PDU_PTR_GET(newMbuf, padding, char *);
        newMbuf->pkt_len = newMbuf->data_len = padLen + saInfo->digestLen;
        rte_pktmbuf_chain(pktMbuf, newMbuf);
    }
    if (padLen)
    {
        if (padLen > 2)
            rte_memcpy(padding, padValues, padLen - 2);
        padding[padLen - 2] = padLen - 2;
        if (ipVersion == ADK_IPSEC_IPV4)
            padding[padLen - 1] = 4;
        else
            padding[padLen - 1] = 41; /* IPv6 */
    }

    if(unlikely(!opData))
    {
        return 0;
    }
    opData->type = RTE_CRYPTO_OP_TYPE_SYMMETRIC;
    opData->status = RTE_CRYPTO_OP_STATUS_NOT_PROCESSED;
    sym = opData->sym;
    rte_crypto_op_attach_sym_session (opData, lkasideInfo->cryptoSession);

    /* offset at which data to be encrypted starts */
    dataOffset = ipHdrLen + ESPV4_HDR_LEN + saInfo->ivLen;
    iv = (uint8_t *)rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *, ipHdrLen + ESPV4_HDR_LEN);
    sym->auth.data.offset = ipHdrLen;
    ivData = rte_crypto_op_ctod_offset(opData, uint8_t *, IV_OFFSET);
    /* Copy IV at the end of crypto operation */
    rte_memcpy(ivData, iv, saInfo->ivLen);
    sym->cipher.data.offset = dataOffset;
    sym->cipher.data.length = dataLen;
    sym->auth.data.length = dataLen + ESPV4_HDR_LEN + saInfo->ivLen;
    if (unlikely(NULL == newMbuf))
    {
        if (pktMbuf->nb_segs == 1)
        {
            sym->auth.digest.data =
                rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *, innerPktLen + dataOffset + padLen);
            sym->auth.digest.phys_addr =
                rte_pktmbuf_mtophys_offset(pktMbuf, innerPktLen + dataOffset + padLen);
        }
        else
        {
            lastSeg = rte_pktmbuf_lastseg(pktMbuf);
            if (likely(rte_pktmbuf_data_len(lastSeg) >= saInfo->digestLen))
            {
                /* thankfully, this segment has auth data
                 */
                sym->auth.digest.data =
                    rte_pktmbuf_mtod_offset(lastSeg, uint8_t *, rte_pktmbuf_data_len(lastSeg) - saInfo->digestLen);
                sym->auth.digest.phys_addr =
                    rte_pktmbuf_mtophys_offset(lastSeg, rte_pktmbuf_data_len(lastSeg) - saInfo->digestLen);
            }
            else
            {
                /* the auth data is split across two segments! Does QAT support this?
                 */
                return 0;
            }
        }
    }
    else
    {
        sym->auth.digest.data =
            rte_pktmbuf_mtod_offset(newMbuf, uint8_t *, padLen);
        sym->auth.digest.phys_addr =
            rte_pktmbuf_mtophys_offset(newMbuf, padLen);
    }

    sym->m_src = pktMbuf;
    lkasideInfo->cryptoOp = rte_crypto_op_alloc(lkasideInfo->cryptoPool,
            RTE_CRYPTO_OP_TYPE_SYMMETRIC);
    return sym->auth.data.length;
}
