/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include <rte_eventdev.h>
#include <ice_dsi_ipsec.h>
#include <adk_hos_dp.h>
#include <adk_sys.h>
#include <adk_pmu.h>
#include <rte_bridge_crypto.h>
#include <adk_metadata.h>

#include <adk_ipsec_struct.h>
#include <adk_ipsec_common.h>
#include <adk_ipsec_dp.h>
#include "adk_ipsec_export.h"
#include "adk_ph_dev_dp.h"

#define ETH_FCS_LEN 4

extern adk_ipsec_dp_table_info_t *adk_ipsec_dp_tbl_gbl_ptr;
/* By default, IPSEC_LLI_SUB_BLOCKS would be commented.
 * If needed to take cycle count for sub blocks, please 
 * enable this macro.
 */
//#define IPSEC_LLI_SUB_BLOCKS
/* By default, Block Cycle count is for decrypted packet path and so, 
 * IPSEC_LLI_PLAIN_TEXT would be commented.
 * If needed to take cycle count for plain text flow, please 
 * enable this macro.
 */
//#define IPSEC_LLI_PLAIN_TEXT
/* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
    uint64_t ipsec_lli_rx_ts, ipsec_lli_tx_ts;
    uint64_t ipsec_post_crypto_lli_rx_ts, ipsec_post_crypto_lli_tx_ts;
    uint64_t ipsec_post_crypto_single_sa_lli_rx_ts, ipsec_post_crypto_single_sa_lli_tx_ts;
    uint64_t ipsec_lli_unexpected_rx_ts, ipsec_lli_unexpected_tx_ts;
    uint64_t ipsec_lli_policy_lkup_rx_ts, ipsec_lli_policy_lkup_tx_ts;
    uint64_t ipsec_lli_after_policy_lkup_rx_ts, ipsec_lli_after_policy_lkup_tx_ts;
    uint64_t ipsec_lli_process_inbound_rx_ts, ipsec_lli_process_inbound_tx_ts;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_lli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_lli_rx_ts, &ipsec_lli_tx_ts, "IPSEC LLI MODULE", PMU_IPSEC_LLI_MODULE)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_post_crypto_lli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_post_crypto_lli_rx_ts, &ipsec_post_crypto_lli_tx_ts, "IPSEC LLI POST CRYPTO MODULE", PMU_IPSEC_LLI_POST_CRYPTO_MODULE)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_post_crypto_single_sa_lli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_post_crypto_single_sa_lli_rx_ts, &ipsec_post_crypto_single_sa_lli_tx_ts, "IPSEC LLI POST CRYPTO SINGLE SA MODULE", PMU_IPSEC_LLI_POST_CRYPTO_SINGLE_SA_MODULE)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_lli_unexpected_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_lli_unexpected_rx_ts, &ipsec_lli_unexpected_tx_ts, "IPSEC LLI MODULE UNEXPECTED CYCLES IN PERFORMANCE PATH", PMU_IPSEC_LLI_MODULE_UNEXPECTED)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_lli_policy_lkup_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_lli_policy_lkup_rx_ts, &ipsec_lli_policy_lkup_tx_ts, "IPSEC LLI MODULE POLICY LKUP", PMU_IPSEC_LLI_MODULE_POLICY_LKUP)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_AFTER_POLICY_LKUP_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_lli_after_policy_lkup_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_AFTER_POLICY_LKUP_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_lli_after_policy_lkup_rx_ts, &ipsec_lli_after_policy_lkup_tx_ts, "IPSEC LLI MODULE AFTER POLICY LKUP", PMU_IPSEC_LLI_MODULE_AFTER_POLICY_LKUP)
    
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_PROCESS_INBOUND_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&ipsec_lli_process_inbound_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_PROCESS_INBOUND_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&ipsec_lli_process_inbound_rx_ts, &ipsec_lli_process_inbound_tx_ts, "IPSEC LLI MODULE PROCESS INBOUND", PMU_IPSEC_LLI_MODULE_PROCESS_INBOUND)
#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPETCED_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_TX_TS()
#endif

static inline adk_status_t adk_ipsec_raise_exception_null_spi_info(int coreIndex,
                                                                   adk_pktio_task_t *taskInfo,
                                                                   uint8_t version)
{
    uint16_t ret = ADK_SUCCESS;

    adk_ipsec_audit_event_t auditEventData = {0};
    if (ADK_IPSEC_IPV4 == version)
    {
        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_sa_not_present_drop++;
    }
    else
    {
        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_sa_not_present_drop++;
    }
    ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SA_NOT_PRESENT,
            &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
    if(unlikely(ADK_E_INVALID_HANDLE == ret))
    {
        if(ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
        }
    }
    return ADK_E_INTERNAL_ERROR;
}

static inline adk_status_t adk_ipsec_check_spi_config(int coreIndex,
                                               adk_pktio_task_t *taskInfo,
                                               uint16_t saID,
                                               uint32_t seqNumber,
                                               uint32_t packetLen,
                                               uint32_t ipHdrLen,
                                               uint8_t version)
{
    uint16_t ret = ADK_SUCCESS;
#define ingSpiInfo adk_ipsec_dp_tbl_gbl_ptr->iSATbl[saID]

   /* ARW check first! if it fails, drop the packet
     */
    if (likely(TRUE == ingSpiInfo.antiReplayEnable))
    {
        ret = adk_ipsec_arw_check(seqNumber, saID);
#if !defined(ADK_IGNORE_ARW_RESULT)
        if (unlikely((ADK_E_IPSEC_ARW_REPLAY == ret) || (ADK_E_IPSEC_ARW_OUTDATED_VALUE == ret)))
        {
            adk_ipsec_audit_event_t auditEventData = {0};
            if (ADK_IPSEC_IPV4 == version)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_seqno_check_fail_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_seqno_check_fail_drop++;
            }
            adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][saID].ipsec_ing_sa_seq_num_mismatch_drop++;
            ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SEQNO_CHECK_FAILURE, 
                                                &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                if(ADK_IPSEC_IPV4 == version)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
            }
            return ret;
        }
#endif
    }

    if (likely(ADK_IPSEC_TRUE == ingSpiInfo.lifetimesEnabled))
    {
        if (unlikely(ingSpiInfo.hardByteLifetime <= ingSpiInfo.bytesUsed))
        {
            adk_ipsec_sa_expiry_excep_data_t sa_expiry_excep_data = {0};
            if (ADK_IPSEC_IPV4 == version)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_sa_hard_life_expire_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_sa_hard_life_expire_drop++;
            }

            /*Revisit when IPsec audit event is implemented*/
            sa_expiry_excep_data.direction = ADK_IPSEC_INBOUND;
            sa_expiry_excep_data.whichExpiry = ADK_HARD_LIFE_EXPIRY;

            ret = adk_ipsec_dp_handle_exception(taskInfo,ADK_IPSEC_EXCEPTION_SA_EXPIRE,
                                                &sa_expiry_excep_data,coreIndex, version, ADK_IPSEC_INBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                if(ADK_IPSEC_IPV4 == version)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
            }
            return ret;
        }
        if (unlikely(ingSpiInfo.softByteLifetime <=
                ingSpiInfo.bytesUsed + (packetLen - ipHdrLen - ingSpiInfo.digestLen)))
        {
            /*Revisit when IPsec audit event is implemented*/
            /* Soft lifetime expiry or sampling : Need to clone the mbuf
            */
            struct rte_mbuf *pktClone = NULL;
            adk_ipsec_sa_expiry_excep_data_t sa_expiry_excep_data = {0};

            sa_expiry_excep_data.direction = ADK_IPSEC_INBOUND;
            sa_expiry_excep_data.whichExpiry = ADK_SOFT_LIFE_EXPIRY;
            pktClone = adk_ipsec_pktmbuf_clone((struct rte_mbuf *)(taskInfo),
                           ((struct rte_mbuf *)(taskInfo))->pool);

            if (unlikely(NULL == pktClone))
            {
                if(ADK_IPSEC_IPV4 == version)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_mbuf_clone_failed++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_mbuf_clone_failed++;
                }
            }
            else
            {
                ret = adk_ipsec_dp_handle_exception(pktClone,ADK_IPSEC_EXCEPTION_SA_EXPIRE,
                                                    &sa_expiry_excep_data,coreIndex, version, ADK_IPSEC_INBOUND, false);
                if(unlikely(ADK_E_INVALID_HANDLE == ret))
                {
                    adk_pal_dp_release_send(pktClone, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                    adk_pktio_task_buffer_free((adk_pktio_task_t *)(pktClone));
                    if(ADK_IPSEC_IPV4 == version)
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                    else
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                }
            }
        }
    }

    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipsec_dp_pkt_proc_lli()
 *
 * Structure of task params :
 *
 * pduParams[7] = pointer to pdu at the begining of inner IP header.
 *
 * Brief:
 * This function is the entry point for decrypting IP
 * packets . The function parses the  ESP header and performs SPI
 * to determine SA to be applied. Tunnel header and ESP
 * headers are removed from the packet.
 *
 * =============================================================
 */
adk_status_t
adk_ipsec_dp_pkt_proc_lli (void **taskList,
                           uint8_t task_count,
                           uint32_t task_mask,
                           __attribute__((unused)) void *ring,
                           __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
                           int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    uint8_t i, drop_count=0;
    uint32_t cur_bit, drop_mask=0;
    adk_pktio_task_t *taskInfo = NULL;
    uint8_t *pdu = NULL;

    for(i=0, cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(!(task_mask & cur_bit))
            continue;

        taskInfo = (adk_pktio_task_t *)taskList[i];

        ADK_PDU_PTR_GET(taskInfo, pdu, uint8_t *);
        /* Workaround to prefetch if the tail of the packet straddle cache boundary */
        rte_prefetch0(pdu + (taskInfo->pkt_len - 60));
        rte_prefetch0(pdu + (taskInfo->pkt_len + 60));

        ret = adk_ipsec_dp_pkt_proc_lli_single(taskList, coreIndex);
        if(unlikely((ADK_E_INTERNAL_ERROR == ret) ||
           (ADK_E_DROP_TASK == ret) ||
           (ADK_E_INVALID_HANDLE == ret)))
        {
            /*Free the task*/
            adk_pal_dp_release_send(taskList[0], ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
            adk_pktio_task_buffer_free (taskList[i]);
            drop_mask |= cur_bit;
            drop_count ++;
        }
        else if(unlikely(ADK_E_TASK_CONSUMED == ret))
        {
            /*Task consuled. Dont free the task*/
            drop_mask |= cur_bit;
            drop_count ++;
        }
    }
    task_count -= drop_count;

    return ret;
}

/* This function is used to fetch the L4 Src and Dst port information
 * from inner IPv4/IPv6 headers and pass it to the caller functions.
 * In case of UDP/TCP/SCTP protocols, Src and Dst ports will be reversed 
 * since the infomation would be used for Inbound SPD OUT Lkup.
 */
static inline void
adk_ipsec_get_l4_params_for_inbound_policy_lkup(uint8_t innerVersion, struct ipv4_hdr *innerIPv4, 
                                                uint8_t ipv6InnerNextHdr, uint8_t *pktStart,
                                                uint8_t innerHdrOffset, uint16_t *l4SrcPort,
                                                uint16_t *l4DstPort)
{
    struct udp_hdr *udpHdr = NULL;
    struct icmp_hdr *icmpHdr = NULL;
    if (ADK_IPSEC_IPV4 == innerVersion)
    {
        if ((IPPROTO_UDP == innerIPv4->next_proto_id) ||
            (IPPROTO_TCP == innerIPv4->next_proto_id) ||
            (IPPROTO_SCTP == innerIPv4->next_proto_id))
        {
            udpHdr = (struct udp_hdr *)(innerIPv4 + 1);
            *l4SrcPort = udpHdr->dst_port;
            *l4DstPort = udpHdr->src_port;
        }
        else if (IPPROTO_ICMP == innerIPv4->next_proto_id)
        {
            icmpHdr = (struct icmp_hdr *)(innerIPv4 + 1);
            *l4SrcPort = icmpHdr->icmp_type;
            *l4DstPort = icmpHdr->icmp_code;
        }
        else
        {
            *l4SrcPort = 0;
            *l4DstPort = 0;
        }
    }
    else
    {
        if ((IPPROTO_TCP == ipv6InnerNextHdr) ||
            (IPPROTO_UDP == ipv6InnerNextHdr) ||
            (IPPROTO_SCTP == ipv6InnerNextHdr))
        {
            udpHdr = (struct udp_hdr *)(pktStart + innerHdrOffset);
            *l4SrcPort = udpHdr->dst_port;
            *l4DstPort = udpHdr->src_port;
        }
        else if (IPPROTO_ICMPV6 == ipv6InnerNextHdr)
        {
            icmpHdr = (struct icmp_hdr *)(pktStart + innerHdrOffset);
            *l4SrcPort = icmpHdr->icmp_type;
            *l4DstPort = icmpHdr->icmp_code;
        }
        else
        {
            *l4SrcPort = 0;
            *l4DstPort = 0;
        }
    }
}

/* This function fills up the necessary information used in
 * DPDK ACL Lookup for inbound SPD OUT Tree. 
 * In case of non-initial frag in Single SA case, Src and Dst
 * port information won't be filled since they would be extracted
 * from adk_ipsec_get_initial_frag_info_or_buffer_non_initial_frags.
 * In other cases, L4 Src and Dst port information retrieved from 
 * adk_ipsec_get_l4_params_for_inbound_policy_lkup would be filled in SPD
 * Pattern.
 */
static inline void
adk_ipsec_create_inbound_spd_out_pattern(adk_ipsec_spd_sel_t *spdPtn, uint8_t innerVersion,
                                         struct ipv4_hdr *innerIPv4, struct ipv6_hdr *innerIPv6,
                                         uint8_t dscpVal, bool useSingleSaInPreFrag,
                                         bool isInitialFrag, uint8_t ipv6InnerNextHdr,
                                         uint16_t l4SrcPort, uint16_t l4DstPort)
{
    if (ADK_IPSEC_IPV4 == innerVersion)
    {
        spdPtn->version = 4;
        spdPtn->sip_addr[0] = innerIPv4->dst_addr;
        spdPtn->dip_addr[0] = innerIPv4->src_addr;
        /* this is required to be done for DSCP and protocol fields,
         * because in reality these are 8 bit fields!!
         */
        spdPtn->dscpVal = rte_cpu_to_be_16(dscpVal);
        spdPtn->protocol = rte_cpu_to_be_16(innerIPv4->next_proto_id);
    }
    else
    {
        spdPtn->version = 6;
        rte_memcpy(spdPtn->sip_addr, innerIPv6->dst_addr, 16);
        rte_memcpy(spdPtn->dip_addr, innerIPv6->src_addr, 16);
        /* this is required to be done for DSCP and protocol fields,
         * because in reality these are 8 bit fields!!
         */
        spdPtn->dscpVal = rte_cpu_to_be_16(dscpVal);
        spdPtn->protocol = rte_cpu_to_be_16(ipv6InnerNextHdr);
    }
    if (!useSingleSaInPreFrag || isInitialFrag)
    {
        /* We can directly fill the Src and Dst port here since 
         * L4 port information would be reversed already as part of
         * adk_ipsec_get_l4_params_for_inbound_policy_lkup
         */
        spdPtn->sport = l4SrcPort;
        spdPtn->dport = l4DstPort;
    }
}
/* TODO, reduce number of params for this function
 */
static adk_status_t inbound_policy_lkup(int coreIndex,
                                    struct rte_mbuf *pktMbuf,
                                    uint8_t *pktStart, uint8_t offset,
                                    uint8_t *innerVersion,
                                    uint16_t  *pktLen,
                                    uint32_t ecnFlag,
                                    uint8_t dscpVal,
                                    bool useSingleSaInPreFrag,
                                    uint16_t saId,
                                    uint16_t *polIndex,
                                    uint16_t *hashInfoIndex,
                                    uint8_t  *bucketInfoIndex,
                                    uint8_t  *initialFrag)
{
    uint32_t  ret;
    uint8_t nextHdr = 0, innerHdrOffset = 0;
    adk_ipsec_spd_sel_t spdPtn;
    bool frag_flag = FALSE, isInitialFrag = FALSE ;
    bool isSPDLookUpNeeded = FALSE ;
    adk_pktio_task_t *taskInfo = (adk_pktio_task_t *)pktMbuf;
    uint16_t l4SrcPort,l4DstPort;
    adk_dsi_rx_metadata_t *dsiRxMetaData = ADK_DSI_RX_METADATA_PTR(pktMbuf);
    struct ipv4_hdr *innerIPv4 = NULL;
    struct ipv6_hdr *innerIPv6 = NULL;

    if (unlikely(ADK_IPSEC_SA_LOOKASIDE == adk_ipsec_dp_tbl_gbl_ptr->iSATbl[saId].saMode))
    {
        isSPDLookUpNeeded = TRUE;
    }
    else
    {
        isSPDLookUpNeeded = adk_hos_is_spd_id_invalid(dsiRxMetaData);
    }
    *innerVersion = (*(uint8_t *)(pktStart + offset)) >> 4;

    if (ADK_IPSEC_IPV4 == *innerVersion)
    {
        innerIPv4 = (struct ipv4_hdr *)rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *, offset);
        if (unlikely(NULL == innerIPv4))
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_too_small_pkt_drop++;
            return ADK_E_DROP_TASK;
        }
        *pktLen += ntohs(innerIPv4->total_length);
        if (ecnFlag)
        {
            if (innerIPv4->type_of_service & ADK_IPSEC_IPTOS_ECN_MASK)
                innerIPv4->type_of_service |= ADK_IPSEC_IPTOS_ECN_CE;
        }
       /* When both Single SA and Pre-Frag are enabled */
        if (useSingleSaInPreFrag)
        {
            uint16_t frag_field , frag_offset;
            frag_field = rte_be_to_cpu_16(innerIPv4->fragment_offset);
            frag_offset = (uint16_t)(frag_field & IPV4_HDR_OFFSET_MASK);
            frag_flag = (uint16_t)(frag_field & IPV4_HDR_MF_FLAG);
            if(!frag_offset && frag_flag)
            {
                /* isInitialFrag would be set to True only for first fragment. */
                isInitialFrag = TRUE;
                /* frag_flag is set to TRUE since it is fragmented traffic */
                frag_flag = TRUE;
            }
            else if(frag_offset)
            {
                /* Last/Non-initial Fragment
                 * frag_flag is set to TRUE since it is fragmented traffic
                 */
                frag_flag = TRUE; 
            }
            else
            {
                /* Unfragmented Traffic */
                frag_flag = FALSE;
            }
        }
    }
    else
    {
        innerIPv6 = (struct ipv6_hdr *)rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *, offset);
        if (unlikely(NULL == innerIPv6))
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_too_small_pkt_drop++;
            return ADK_E_DROP_TASK;
        }
        *pktLen += ntohs(innerIPv6->payload_len) + sizeof(struct ipv6_hdr);
        nextHdr = innerIPv6->proto;
        innerHdrOffset = offset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {
            if (ADK_IPSEC_V6_NEXT_HDR_FRAG_HDR == nextHdr)
            {
                adk_ipsec_v6_frag_hdr_t *ipv6_frag_hdr = NULL;
                nextHdr = (*(uint8_t *)(pktStart + innerHdrOffset));
                ipv6_frag_hdr = (adk_ipsec_v6_frag_hdr_t *)(pktStart + innerHdrOffset);
                innerHdrOffset += sizeof(adk_ipsec_v6_frag_hdr_t);
                /* When both single sa and pre-frag are enabled. */
                if (useSingleSaInPreFrag)
                {
                    uint16_t frag_field, frag_offset;
                    frag_field = rte_be_to_cpu_16(ipv6_frag_hdr->fragOffset);
                    frag_offset = (uint16_t)(frag_field & 0xFFF8);
                    frag_flag = (uint16_t)(frag_field & 0x1);
                    if(!frag_offset && frag_flag)
                    {
                        /* isInitialFrag would be set to True only for first fragment. */
                        isInitialFrag = TRUE;
                        /* frag_flag is set to TRUE since it is fragmented traffic */
                        frag_flag = TRUE;
                    }
                    else if (frag_offset)
                    {
                        /* frag_flag is set to TRUE since it is fragmented traffic */
                        frag_flag = TRUE;
                        break;
                    }
                    else
                    {
                        /* Unfragmented Traffic */
                        frag_flag = FALSE;
                    }
                }
                else if(ipv6_frag_hdr->fragOffset)
                {
                    break;
                }
            }
            else
            {
                nextHdr = (*(uint8_t *)(pktStart + innerHdrOffset));
                innerHdrOffset += ((*(((uint8_t *)pktStart) + innerHdrOffset + 1))<<3) + 8;
            }
        }
        if (ecnFlag)
        {
            if ((ntohl(innerIPv6->vtc_flow) >> 20) & ADK_IPSEC_IPTOS_ECN_MASK)
                innerIPv6->vtc_flow = htonl(ntohl(innerIPv6->vtc_flow) | ADK_IPSEC_IPTOS_ECN_CE << 20);
        }
    }
    /* If pre-frag and single SA case are enabled and if it is fragmented packet */
    uint16_t resIndex = 0;
    uint8_t bucketIndex = 0;
    if (useSingleSaInPreFrag && frag_flag)
    {
        uint32_t preFragBufferTimeOutVal = adk_ipsec_dp_tbl_gbl_ptr->preFragBufferTimeOutVal;
        uint32_t hashIndex;
        
        if (ADK_IPSEC_IPV4 == *innerVersion)
        {

            adk_ip_dp_get_reassembly_hash_row(&pktMbuf, offset, &hashIndex, &resIndex);
            adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, resIndex);
            /* Calling IP function for this fragment to get the resIndex and bucket index*/
            ret = adk_ip_dp_ipsec_start_reassembly(coreIndex, &taskInfo, hashIndex, resIndex, offset,
                                                   &adk_ipsec_clear_initial_frag_buffered_pkts_hash_entry,
                                                   preFragBufferTimeOutVal, &bucketIndex);
        }
        else
        {
            /* Calling IPv6 function for this fragment to get the resIndex and bucket index*/
            ret = adk_ipv6_dp_ipsec_start_reassembly(coreIndex, &taskInfo, &resIndex, offset,
                                                     &adk_ipsec_ipv6_clear_initial_frag_buffered_pkts_hash_entry,
                                                     &bucketIndex, preFragBufferTimeOutVal);
        }    
        if (unlikely(ADK_SUCCESS != ret))
        {
            if (ADK_IPSEC_IPV4 == *innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_reassembly_fail_drop++;
                adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, resIndex);
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_reassembly_fail_drop++;
            }
            return ret;
        }

        *hashInfoIndex = resIndex;
        *bucketInfoIndex = bucketIndex;
        *initialFrag = isInitialFrag;

        if (!isInitialFrag)
        {
            if (ADK_IPSEC_IPV6 == *innerVersion)
                adk_ipv6_dp_acquire_reassembly_lock(resIndex);

            ret = adk_ipsec_get_initial_frag_info_or_buffer_non_initial_frags(coreIndex, *innerVersion, saId, taskInfo, 
                                                                              *pktLen, &spdPtn, resIndex,
                                                                              bucketIndex, &isSPDLookUpNeeded);
            if (ADK_IPSEC_IPV6 == *innerVersion)
                adk_ipv6_dp_release_reassembly_lock(resIndex);

            if ((ADK_SUCCESS != ret) || (!isSPDLookUpNeeded))
            {
                if (ADK_IPSEC_IPV4 == *innerVersion)
                    adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, resIndex);
                /* If the non-initial fragment is received after initial fragment, this case may hit and 
                 * here, policy lookup is not needed and just SA validation is enough for this non-initial fragment.
                 */
                *polIndex = ADK_IPSEC_INVALID_SPD_ID;
                return ret;
            }
        }

        if (ADK_IPSEC_IPV4 == *innerVersion)
            adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, resIndex);

    }
    if (likely(!isSPDLookUpNeeded))
    {
        *polIndex = adk_hos_get_spd_id(dsiRxMetaData);
    }
    else if (!useSingleSaInPreFrag ||
             isInitialFrag ||
             isSPDLookUpNeeded)
    {
#if defined(IPSEC_LLI_SUB_BLOCKS)        
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_RX_TS();
#endif
        /* Get L4 params for SPD Ptn */
        adk_ipsec_get_l4_params_for_inbound_policy_lkup(*innerVersion, innerIPv4, nextHdr, 
                                                        pktStart, innerHdrOffset, &l4SrcPort, &l4DstPort);
        /* Create Inbound SPD Out Ptn */
        adk_ipsec_create_inbound_spd_out_pattern(&spdPtn, *innerVersion, innerIPv4, innerIPv6, dscpVal,
                                                 useSingleSaInPreFrag, isInitialFrag, nextHdr,
                                                 l4SrcPort, l4DstPort);
        ret = adk_ipsec_spd_lkup(&spdPtn, polIndex);
#if defined(IPSEC_LLI_SUB_BLOCKS)        
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_TX_TS();
#endif
        if (unlikely(ret != ADK_SUCCESS))
        {
            if (ADK_IPSEC_IPV4 == *innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_out_default_discard_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_out_default_discard_drop++;
            }
            adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][saId].ipsec_ing_sa_traffic_sel_mismatch_drop++;
            return ADK_E_INTERNAL_ERROR;
        }
    }
    if (unlikely(0 == *polIndex))
    {
        /* ICMP transit processing here
         */
//TODO: ICMP inbound and outbound transits are not supported currently and the support will be added later.
#if 0
        ret = adk_ipsec_inbound_icmp_transit(taskInfo, pktStart+offset, *innerVersion, polIndex);
        if (ADK_E_IPSEC_POLICY_NOT_FOUND == ret)
        {
#endif
            if (ADK_IPSEC_IPV4 == *innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_out_default_discard_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_out_default_discard_drop++;
            }
            adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][saId].ipsec_ing_sa_traffic_sel_mismatch_drop++;
            return ADK_E_DROP_TASK;
//TODO: ICMP inbound and outbound transits are not supported currently and the support will be added later.
#if 0
        }
        else if (ADK_E_DROP_TASK == ret)
        {
            return ADK_IPSEC_INVALID_SPD_ID;
        }
#endif
    }
    if(isInitialFrag)
    {
        if (ADK_IPSEC_IPV4 == *innerVersion)
            adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, resIndex);
        else
            adk_ipv6_dp_acquire_reassembly_lock(resIndex);
        /* Get L4 params for filling Initial Frag Info*/
        adk_ipsec_get_l4_params_for_inbound_policy_lkup(*innerVersion, innerIPv4, nextHdr, 
                                                        pktStart, innerHdrOffset, &l4SrcPort, &l4DstPort);
        /* This function is called only for first fragment if preFrag is enabled and useSingleSAInPrefrag is set. */
        adk_ipsec_add_initial_frag_info(*innerVersion, saId, resIndex, bucketIndex, l4SrcPort, l4DstPort);
        if (ADK_IPSEC_IPV4 == *innerVersion)
            adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, resIndex);
        else
            adk_ipv6_dp_release_reassembly_lock(resIndex);
    }
    return ADK_SUCCESS;
}

static inline struct rte_mbuf *pktmbuf_secondlast_seg(struct rte_mbuf *pktMbuf)
{
    struct rte_mbuf *ret = pktMbuf;
    while(ret->next && ret->next->next)
        ret = ret->next;
    return ret;
}

/* This function is similar to process_inbound_pkt function.
 * Here, for all the buffered fragments, the following would be done,
 * 1. SA validation (only for non-initial fragments in the buffered packet chain since SA validation for initial fragment would have been done 
 *    already before calling this function)
 * 2. Removes outer IP and ESP header.
 * 3. Validates and removes padding.
 * 4. Send the buffered packet chain to IP module.
 */
static adk_status_t adk_ipsec_process_buffered_pkts_single_sa(int coreIndex, uint16_t hashIndex, 
                                                              uint8_t bucketIndex, bool antiReplayEnable,
                                                              uint64_t seqNum, uint8_t ivLen,
                                                              uint8_t innerVersion)
{
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_RX_TS();
#endif
    void *taskList = NULL;
    struct rte_mbuf *pktMbuf = NULL, *lastSeg = NULL, *secLastSeg = NULL;
    adk_metadata_t    *adkMetaData = NULL;
    adk_pktio_task_t *cur_task = NULL, *next_task = NULL, *prev_task = NULL;
    uint8_t *pktStart, *nextHdr;
    uint8_t loop = 0;
    uint8_t *padLen = NULL, *padBytes = NULL, offset = 0;
    uint16_t pktLen = 0, non_frag_saId = 0;
    uint32_t ret;
    uint8_t fn_id = 0, outerVersion = 0;
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    struct ipv6_hdr *outerIPv6 = NULL;
    uint8_t digestLen = 0;

    if (ADK_IPSEC_IPV4 == innerVersion)
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    else
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];

    digestLen = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[hashInfo->intial_frag_SA_ID].digestLen;
    cur_task = hashInfo->buffered_pkt_task_chain;
    for (loop = 1; (loop <= hashInfo->numBufferedFrags) && (cur_task); loop++)
    {
         pktMbuf = (struct rte_mbuf *)cur_task;
         adkMetaData = ADK_METADATA_PTR(cur_task);
         pktStart = rte_pktmbuf_mtod(pktMbuf, uint8_t *);
         outerVersion = adkMetaData->outerIPVersion;
         if (ADK_IPSEC_IPV6 == outerVersion)
             outerIPv6 = (struct ipv6_hdr *)(pktStart + adkMetaData->ip_offset);
         if (loop != hashInfo->numBufferedFrags)
         {
             non_frag_saId = adkMetaData->ts_correlator & 0xFFFF;
             if (unlikely(non_frag_saId != hashInfo->intial_frag_SA_ID))
             {
                 if (ADK_IPSEC_IPV4 == innerVersion)
                 {
                     adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
                 }
                 else
                 {
                     adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
                 }
                 //TODO: Exception need to be taken care for invalid SA for non_initial_frag
                 next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
                 adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                 adk_pktio_task_buffer_free (cur_task);
                 cur_task = next_task;
                 if (loop == 1)
                 {
                     hashInfo->buffered_pkt_task_chain = cur_task;
                 }
                 else
                 {
                     ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
                 }
                 continue;
             }
         }
         if(ADK_IPSEC_IPV6 == outerVersion)
         {
             if(ETH_FCS_LEN == (rte_pktmbuf_pkt_len(pktMbuf) - (ntohs(outerIPv6->payload_len) + sizeof(struct ipv6_hdr) + cur_task->l2_len)))
             {
                 rte_pktmbuf_trim(pktMbuf, ETH_FCS_LEN);
             }
         }
         /* handle multi-segment case here
         */
         if (unlikely(pktMbuf->nb_segs > 1))
         {
             /* need to find some optimal way of doing this
             */
             lastSeg = rte_pktmbuf_lastseg(pktMbuf);
             if (rte_pktmbuf_data_len(lastSeg) > digestLen + 2)
             {
                 /* thankfully, this segment has padding and auth data
                 */
                 nextHdr = rte_pktmbuf_mtod_offset(lastSeg, uint8_t*,
                               rte_pktmbuf_data_len(lastSeg)
                               - digestLen - 1);
                 /* set the padding pointer
                 */
                 padLen = rte_pktmbuf_mtod_offset(lastSeg, uint8_t *,
                              rte_pktmbuf_data_len(lastSeg)
                              - digestLen - 2);
             }
             else if (rte_pktmbuf_data_len(lastSeg) == digestLen)
             {
                 /* this segment has only the auth data,
                  * so padding must be in the segment before this
                  */
                 secLastSeg = pktmbuf_secondlast_seg(pktMbuf);
                 nextHdr = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t*,
                               rte_pktmbuf_data_len(secLastSeg) - 1);
                 /* set the padding pointer
                 */
                 padLen = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t *,
                              rte_pktmbuf_data_len(secLastSeg) - 2);
             }
             else
             {
                 /* worst case, auth data is split across two segments
                 */
                 secLastSeg = pktmbuf_secondlast_seg(pktMbuf);
                 nextHdr = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t*,
                               rte_pktmbuf_data_len(secLastSeg)
                               - (digestLen
                               - rte_pktmbuf_data_len(lastSeg)) - 1);
                 /* set the padding pointer
                 */
                 padLen = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t *,
                              rte_pktmbuf_data_len(secLastSeg)
                              - (digestLen
                              - rte_pktmbuf_data_len(lastSeg)) - 2);
                 /*Freeing the last segment as it contains only the partial digest data */
                 rte_pktmbuf_free_seg(lastSeg);
                 lastSeg = NULL;
                 pktMbuf->nb_segs -= 1;
                 secLastSeg->next = NULL;
             }
         }
         else /* only one segment case */
         {
             nextHdr = rte_pktmbuf_mtod_offset(pktMbuf, uint8_t*,
                           rte_pktmbuf_pkt_len(pktMbuf)
                           - digestLen - 1);
             /* set the padding pointer
             */
             padLen = rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *,
                          rte_pktmbuf_pkt_len(pktMbuf)
                          - digestLen - 2);
         }
         if (unlikely(ADK_IPSEC_DUMMY_HDR == *nextHdr))
         {
             if (ADK_IPSEC_IPV4 == innerVersion)
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_dummy_pkt_drop++;
             }
             else
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_dummy_pkt_drop++;
             }
             next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
             adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
             adk_pktio_task_buffer_free (cur_task);
             cur_task = next_task;
             if (loop == 1)
             {
                 hashInfo->buffered_pkt_task_chain = cur_task;
             }
             else
             {
                 ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
             }
             continue;
         }
         
         offset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ivLen;
         pktLen = adkMetaData->ts_correlator >> 16;
         
         /* if total_pkt_len > ipPktLen+padLen+digestLen+2;
          * drop packet as this could be TFC padded packet
          */
         pktLen += *padLen + 2 + digestLen;

         if (unlikely((rte_pktmbuf_pkt_len(pktMbuf) - offset) > pktLen))
         {
             if (ADK_IPSEC_IPV4 == innerVersion)
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_tfc_pad_drop++;
             }
             else
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_tfc_pad_drop++;
             }
             next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
             adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
             adk_pktio_task_buffer_free (cur_task);
             cur_task = next_task;
             if (loop == 1)
             {
                 hashInfo->buffered_pkt_task_chain = cur_task;
             }
             else
             {
                 ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
             }
             continue;
         }
         /* now verify the padding
         */
         padBytes = padLen - *padLen;
         if (unlikely(ADK_MEMCMP(padBytes, padValues, *padLen)))
         {
             next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
             adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
             adk_pktio_task_buffer_free (cur_task);
             cur_task = next_task;
             if (loop == 1)
             {
                 hashInfo->buffered_pkt_task_chain = cur_task;
             }
             else
             {
                 ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
             }
             continue;
         }
         if (rte_pktmbuf_trim(pktMbuf, *padLen + 2 + digestLen))
         {
             if (ADK_IPSEC_IPV4 == innerVersion)
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pad_removal_fail_drop++;
             }
             else
             {
                 adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pad_removal_fail_drop++;
             }
             next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
             adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
             adk_pktio_task_buffer_free (cur_task);
             cur_task = next_task;
             if (loop == 1)
             {
                 hashInfo->buffered_pkt_task_chain = cur_task;
             }
             else
             {
                 ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
             }
             continue;
         }

         if (likely(antiReplayEnable))
         {
             /* If we have reached till here, it means that in both the modes,
              * ARW check is already done, so just adjust the window
              */
             ret = adk_ipsec_arw_update(seqNum, hashInfo->intial_frag_SA_ID);
             if (unlikely((ADK_E_IPSEC_ARW_REPLAY == ret) || (ADK_E_IPSEC_ARW_OUTDATED_VALUE == ret)))
             {
                 /* drop and raise and audit event
                 */
                 if (ADK_IPSEC_IPV4 == innerVersion)
                 {
                     adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_arw_update_fail_drop++;
                 }
                 else
                 {
                     adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_arw_update_fail_drop++;
                 }
                 next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
                 adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
                 adk_pktio_task_buffer_free (cur_task);
                 cur_task = next_task;
                 if (loop == 1)
                 {
                     hashInfo->buffered_pkt_task_chain = cur_task;
                 }
                 else
                 {
                     ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)cur_task;
                 }
                 continue;
             }
         }

         if (ADK_IPSEC_IPV4 == innerVersion)
         {
             /* this counter here is only to ensure backward compatibility.
              * In mixed mode, this can cause confusion!
              */
             INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].ingressUtilityCounters.ipsec_ing_num_pdus_rcvd_after_inbound_processing);
             adk_ip_dp_lli_imc_data_t *ip_imc = NULL;
             ip_imc = (adk_ip_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
             ip_imc->ipsec_data = IPLLI_IMC_IPSEC_INNER_INLINE;
         }
         else
         {
             /* this counter here is only to ensure backward compatibility.
              * In mixed mode, this can cause confusion!
              */
             INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].ingressUtilityCounters.ipsec_ing_num_pdus_rcvd_after_inbound_processing);
             adk_ipv6_dp_lli_imc_data_t *ipv6_imc = NULL;
             ipv6_imc = (adk_ipv6_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
             ipv6_imc->ipsec_data =  IPLLI_IMC_IPSEC_INNER_INLINE;
         }

         /* TODO : Rewrite the IMC params that came from IP module before sending it back,
          * along with other params that IP may need.
          */
         adkMetaData->ip_offset = offset;
         next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
         prev_task = cur_task; 
         cur_task = next_task;
    }
    if (ADK_IPSEC_IPV4 == innerVersion)
    {
        fn_id = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    }
    else
    {
        fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    }
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_SINGLE_SA_LLI_TX_TS();  
#endif
    if (likely(adkDestEntryFunction_g[fn_id]))
    {
        taskList = (void *)hashInfo->buffered_pkt_task_chain;
        hashInfo->buffered_pkt_task_chain = NULL;
        hashInfo->numBufferedFrags = 0;
        if (ADK_IPSEC_IPV4 == innerVersion)
            adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, hashIndex);
        else
            adk_ipv6_dp_release_reassembly_lock(hashIndex);
        ret = adk_pal_dp_stage_send((void **)&taskList, 1, 1, fn_id, 0x00,
                0x00, ADK_PAL_DP_BATCH_CB_FN_IPSEC, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
        if (unlikely(ret == ADK_E_INTERNAL_ERROR))
        {
            if (ADK_IPSEC_IPV4 == innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
            }
            return ADK_E_INTERNAL_ERROR;
        }
    }
    else
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
            adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, hashIndex);
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
            adk_ipv6_dp_release_reassembly_lock(hashIndex);
        }
        return ADK_E_INTERNAL_ERROR;
    }

    return ADK_SUCCESS;
}

/* This function does following things
 * 1. Call policy lkup
 * 2. validates the ingress SA
 * 3. Validates and removes padding
 * 5. send packet to next module
 */
static adk_status_t process_inbound_pkt(void **taskList, int coreIndex,
                                        struct rte_mbuf   *pktMbuf,
                                        uint8_t operType,
                                        ipsec_dp_inbound_meta_data_t *ipsecMetaData)
{
   
    adk_pktio_task_t  *taskInfo = (adk_pktio_task_t *)taskList[0];
    adk_metadata_t    *adkMetaData = ADK_METADATA_PTR(taskInfo);
    uint8_t *pktStart, *nextHdr;
    uint8_t innerVersion = 0;
    uint8_t *padLen, *padBytes, offset = 0;
    uint16_t polIndex = 0, pktLen = 0;
    uint32_t ret;
    uint8_t fn_id = 0;
    uint8_t isInitialFrag = 0, singleSABucketIndex = 0;
    uint16_t singleSAHashIndex = 0;
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    uint8_t digestLen = ipsecMetaData->digestLen;
    pktStart = rte_pktmbuf_mtod(pktMbuf, uint8_t *);

    /* this will work only if "l4_offset" is set correctly
     * for IPv6, IPv6 module needs to ensure that extension headers
     * are accounted for to calculate l4_offset field
     * in case of IPv4+NAT-T, it correctly reflects IP hdr len
     * and UDP hdr len
     */
    offset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ipsecMetaData->ivLen;

#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_PROCESS_INBOUND_LLI_TX_TS();
#endif
    /* this function extracts necessary fields, performs policy
     * lookup
     */
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_RX_TS();
#endif
    ret = inbound_policy_lkup(coreIndex, pktMbuf, pktStart, offset, &innerVersion,
                              &pktLen, ipsecMetaData->ecnFlag,
                              ipsecMetaData->dscpVal_version,
                              ipsecMetaData->useSingleSaInPreFrag,
                              ipsecMetaData->saId, &polIndex,
                              &singleSAHashIndex, &singleSABucketIndex,
                              &isInitialFrag);
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_POLICY_LKUP_TX_TS();
#endif
    if (unlikely((ADK_E_INTERNAL_ERROR == ret) || (ADK_E_DROP_TASK == ret)))
    {
        adk_ipsec_audit_event_t auditEventData = {0};
        /* Storing Ingress SA Handle */
        auditEventData.event.ipsecIngSpdOutLkupDiscardEvent.data_from_expected_ingress_sa = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].iSAHandle;
        /* Storing Outer IP version */
        auditEventData.event.ipsecIngSpdOutLkupDiscardEvent.ipVersionSA = adkMetaData->outerIPVersion;
        /* Since we know that secProtocol is ESP, here we are filling inner IP header offset in the place
         * of secProtocol. 
         * innerIPHeaderOffset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ipsecMetaData->ivLen
         * Once we extracted the inner IP header in CP using this offset,
         * we can then populate ESP protocol value in the place of secProtocol in Audit event data structure.
         */
        auditEventData.event.ipsecIngSpdOutLkupDiscardEvent.secProtocol = offset;
        ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SPP_SPD_OUT_LKUP_DISCARD,
                                            &auditEventData, coreIndex, adkMetaData->outerIPVersion,
                                            ADK_IPSEC_INBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            if(ADK_IPSEC_IPV4 == adkMetaData->outerIPVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
        }
        return ret;
    }
    else if (unlikely(ADK_SUCCESS != ret))
    {
        return ret;
    }
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_AFTER_POLICY_LKUP_RX_TS();
#endif
    /* polIndex would be set to ADK_IPSEC_INVALID_SPD_ID in
     * inbound_policy_lkup only for non-initial fragment and when SPD lookup is
     * not needed. SPD lookup would not be needed only when initial frament was
     * already visited and policy version for both initial and current non-initial fragment
     * are same.
     * In this case, SA validation for
     * non-initial fragment can be done just by comapring with initial fragment
     * SA ID.
     */
    if (likely((adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[polIndex].action == ADK_IPSEC_SP_ACTION_IPSEC) && (ADK_IPSEC_INVALID_SPD_ID != polIndex)))
    {
        if (unlikely(adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[polIndex].spIDEx
                    != adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].spIDEx))
        {
            if (ADK_IPSEC_IPV4 == innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
            }
            if (isInitialFrag)
            {
                if (ADK_IPSEC_IPV4 == innerVersion)
                {
                    adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
                    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
                }
                else
                {
                    adk_ipv6_dp_acquire_reassembly_lock(singleSAHashIndex);
                    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
                }
                hashInfo->initial_frag_visited = FALSE;
                if (ADK_IPSEC_IPV4 == innerVersion)
                    adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
                else
                    adk_ipv6_dp_release_reassembly_lock(singleSAHashIndex);
            }
            adk_ipsec_audit_event_t auditEventData = {0};
            /* Storing Ingress SA Handle */
            auditEventData.event.ipsecIngSaMismatchEvent.data_from_expected_ingress_sa = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].iSAHandle;
            /* Storing Outer IP version */
            auditEventData.event.ipsecIngSaMismatchEvent.ipVersionSA = adkMetaData->outerIPVersion;
            /* Since we know that secProtocol is ESP, here we are filling inner IP header offset in the place
             * of secProtocol. 
             * innerIPHeaderOffset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ipsecMetaData->ivLen
             * Once we extracted the inner IP header in CP using this offset,
             * we can then populate ESP protocol value in the place of secProtocol in Audit event data structure.
             */
            auditEventData.event.ipsecIngSaMismatchEvent.secProtocol = offset;
            ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SA_MISMATCH,
                                                &auditEventData, coreIndex, adkMetaData->outerIPVersion,
                                                ADK_IPSEC_INBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                if(ADK_IPSEC_IPV4 == adkMetaData->outerIPVersion)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
            }
            return ret;
        }
    }
    else if(ipsecMetaData->useSingleSaInPreFrag &&
           (ADK_IPSEC_INVALID_SPD_ID == polIndex))
    {
       if (ADK_IPSEC_IPV4 == innerVersion)
       {
           adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
           hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
       }
       else
       {
           adk_ipv6_dp_acquire_reassembly_lock(singleSAHashIndex);
           hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
       }
       if (unlikely(ipsecMetaData->saId != hashInfo->intial_frag_SA_ID))
       {
           if (ADK_IPSEC_IPV4 == innerVersion)
           {
               adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
           }
           else
           {
               adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_sa_validation_fail_drop++;
           }
           if (isInitialFrag)
           {
               if (ADK_IPSEC_IPV4 == innerVersion)
               {
                   adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
                   hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
               }
               else
               {
                   adk_ipv6_dp_acquire_reassembly_lock(singleSAHashIndex);
                   hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
               }
               hashInfo->initial_frag_visited = FALSE;
               if (ADK_IPSEC_IPV4 == innerVersion)
                   adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
               else
                   adk_ipv6_dp_release_reassembly_lock(singleSAHashIndex);
           }
           adk_ipsec_audit_event_t auditEventData = {0};
           /* Storing Ingress SA Handle */
           auditEventData.event.ipsecIngSaMismatchEvent.data_from_expected_ingress_sa = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].iSAHandle;
           /* Storing Outer IP version */
           auditEventData.event.ipsecIngSaMismatchEvent.ipVersionSA = adkMetaData->outerIPVersion;
           /* Since we know that secProtocol is ESP, here we are filling inner IP header offset in the place
            * of secProtocol. 
            * innerIPHeaderOffset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ipsecMetaData->ivLen
            * Once we extracted the inner IP header in CP using this offset,
            * we can then populate ESP protocol value in the place of secProtocol in Audit event data structure.
            */
           auditEventData.event.ipsecIngSaMismatchEvent.secProtocol = offset;
           ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SA_MISMATCH,
                                               &auditEventData, coreIndex, adkMetaData->outerIPVersion,
                                               ADK_IPSEC_INBOUND, false);
           if (likely(ADK_E_TASK_CONSUMED == ret))
           {
               INCREMENT_INGRESS_UTILITY_COUNTER(adkMetaData->outerIPVersion, ipsec_ing_exception_sent);
           }
           else if(unlikely(ADK_E_INVALID_HANDLE == ret))
           {
               if(ADK_IPSEC_IPV4 == adkMetaData->outerIPVersion)
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
               }
               else
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
               }
           }
           else
           {
               if(ADK_IPSEC_IPV4 == adkMetaData->outerIPVersion)
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_send_failure_drop++;
               }
               else
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_send_failure_drop++;
               }
           }
           return ret;
       }
       if (ADK_IPSEC_IPV4 == innerVersion)
           adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
       else
           adk_ipv6_dp_release_reassembly_lock(singleSAHashIndex);
    }
    else
    {
        if (unlikely(adk_ipsec_dp_tbl_gbl_ptr->polSAInfo[polIndex].action != ADK_IPSEC_SP_ACTION_IPSEC))
        {
            adk_ipsec_audit_event_t auditEventData = {0};
            if (innerVersion == ADK_IPSEC_IPV4)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_out_not_ipsec_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_out_not_ipsec_drop++;
            }
            if (isInitialFrag)
            {
                if (ADK_IPSEC_IPV4 == innerVersion)
                {
                    adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
                    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
                }
                else
                {
                    adk_ipv6_dp_acquire_reassembly_lock(singleSAHashIndex);
                    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
                }
                hashInfo->initial_frag_visited = FALSE;
                if (ADK_IPSEC_IPV4 == innerVersion)
                    adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
                else
                    adk_ipv6_dp_release_reassembly_lock(singleSAHashIndex);
            }
            /* Storing Ingress SA Handle */
            auditEventData.event.ipsecIngSpdOutLkupNoIpsecPolicyEvent.data_from_expected_ingress_sa = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].iSAHandle;
            /* Storing Outer IP version */
            auditEventData.event.ipsecIngSpdOutLkupNoIpsecPolicyEvent.ipVersionSA = adkMetaData->outerIPVersion;
            /* Since we know that secProtocol is ESP, here we are filling inner IP header offset in the place
             * of secProtocol. 
             * innerIPHeaderOffset = adkMetaData->l4_offset + ESPV4_HDR_LEN + ipsecMetaData->ivLen
             * Once we extracted the inner IP header in CP using this offset,
             * we can then populate ESP protocol value in the place of secProtocol in Audit event data structure.
             */
            auditEventData.event.ipsecIngSpdOutLkupNoIpsecPolicyEvent.secProtocol = offset;
            ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SPP_SPD_OUT_LKUP_NO_IPSEC_DROP,
                                                &auditEventData, coreIndex, adkMetaData->outerIPVersion, ADK_IPSEC_INBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                if(ADK_IPSEC_IPV4 == adkMetaData->outerIPVersion)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
            }
            return ret;
        }
    }
    /* In case of useSingleSaInPreFrag, when the current packet is
     * initial fragment and if there are any buffered packets,
     * adk_ipsec_process_buffered_pkts_single_sa is called to
     * process the buffered packets and send to IP module.
     */
    if (isInitialFrag)
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {   
            adk_ip_dp_acquire_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
            hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
        }
        else
        {
            adk_ipv6_dp_acquire_reassembly_lock(singleSAHashIndex);
            hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[singleSAHashIndex].bucket[singleSABucketIndex];
        }
        if (hashInfo->numBufferedFrags > 0)
        {
            /* First buffer the initial fragment at the end of the fragment chain */
            adk_ipsec_buffer_initial_frag(coreIndex, innerVersion, singleSAHashIndex, singleSABucketIndex, pktLen, 
                                          ipsecMetaData->saId, taskInfo);
            ret = adk_ipsec_process_buffered_pkts_single_sa(coreIndex, singleSAHashIndex, singleSABucketIndex, 
                                                            ipsecMetaData->antiReplayEnable,
                                                            ipsecMetaData->seqNumber, ipsecMetaData->ivLen,
                                                            innerVersion);
            if (unlikely(ADK_SUCCESS != ret))
            {
                if (innerVersion == ADK_IPSEC_IPV4)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_buffered_single_sa_pkt_process_fail_drop++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_buffered_single_sa_pkt_process_fail_drop++;
                }
            }
            return ret;
        }
        if (ADK_IPSEC_IPV4 == innerVersion)
            adk_ip_dp_release_reassembly_lock_non_atomic(pktMbuf, singleSAHashIndex);
        else
            adk_ipv6_dp_release_reassembly_lock(singleSAHashIndex);
    }

    if (unlikely(operType == ADK_IPSEC_LOOKASIDE))
    {
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[ipsecMetaData->saId].inFlightCnt--;
    }
    
    /* handle multi-segment case here
     */
    if (unlikely(pktMbuf->nb_segs > 1))
    {
	struct rte_mbuf   *lastSeg = NULL, *secLastSeg = NULL;
        /* need to find some optimal way of doing this
         */
        lastSeg = rte_pktmbuf_lastseg(pktMbuf);
        if (rte_pktmbuf_data_len(lastSeg) > digestLen + 2)
        {
            /* thankfully, this segment has padding and auth data
             */
            nextHdr = rte_pktmbuf_mtod_offset(lastSeg, uint8_t*,
                                              rte_pktmbuf_data_len(lastSeg)
                                              - digestLen - 1);
            /* set the padding pointer
             */
            padLen = rte_pktmbuf_mtod_offset(lastSeg, uint8_t *,
                                             rte_pktmbuf_data_len(lastSeg)
                                             - digestLen - 2);
        }
        else if (rte_pktmbuf_data_len(lastSeg) == digestLen)
        {
            /* this segment has only the auth data,
             * so padding must be in the segment before this
             */
            secLastSeg = pktmbuf_secondlast_seg(pktMbuf);
            nextHdr = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t*,
                                              rte_pktmbuf_data_len(secLastSeg) - 1);
            /* set the padding pointer
             */
            padLen = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t *,
                                             rte_pktmbuf_data_len(secLastSeg) - 2);
        }
        else
        {
            /* worst case, auth data is split across two segments
             */
            secLastSeg = pktmbuf_secondlast_seg(pktMbuf);
            nextHdr = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t*,
                                rte_pktmbuf_data_len(secLastSeg)
                                - (digestLen
                                - rte_pktmbuf_data_len(lastSeg)) - 1);
            /* set the padding pointer
             */
            padLen = rte_pktmbuf_mtod_offset(secLastSeg, uint8_t *,
                                rte_pktmbuf_data_len(secLastSeg)
                                - (digestLen
                                   - rte_pktmbuf_data_len(lastSeg)) - 2);
            /*Freeing the last segment as it contains only the partial digest data */
            rte_pktmbuf_free_seg(lastSeg);
            lastSeg = NULL;
            pktMbuf->nb_segs -= 1;
            secLastSeg->next = NULL;
        }
    }
    else /* only one segment case */
    {
        nextHdr = rte_pktmbuf_mtod_offset(pktMbuf, uint8_t*,
                rte_pktmbuf_pkt_len(pktMbuf)
                - digestLen - 1);
        /* set the padding pointer
         */
        padLen = rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *,
                rte_pktmbuf_pkt_len(pktMbuf)
                - digestLen - 2);
    }
    if (unlikely(ADK_IPSEC_DUMMY_HDR == *nextHdr))
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_dummy_pkt_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_dummy_pkt_drop++;
        }
        return ADK_E_DROP_TASK;
    }

    /* if total_pkt_len > ipPktLen+padLen+digestLen+2;
     * drop packet as this could be TFC padded packet
     */
    pktLen += *padLen + 2 + digestLen;

    if (unlikely((rte_pktmbuf_pkt_len(pktMbuf) - offset) > pktLen))
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_tfc_pad_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_tfc_pad_drop++;
        }
        return (ADK_E_DROP_TASK);
    }
    /* now verify the padding
     */
    padBytes = padLen - *padLen;
    if (unlikely(ADK_MEMCMP(padBytes, padValues, *padLen)))
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pad_verification_fail_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pad_verification_fail_drop++;
        }
        return ADK_E_DROP_TASK;
    }
    if (rte_pktmbuf_trim(pktMbuf, *padLen + 2 + digestLen))
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pad_removal_fail_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pad_removal_fail_drop++;
        }
        return ADK_E_DROP_TASK;
    }
    if (likely(ipsecMetaData->antiReplayEnable))
    {
        /* If we have reached till here, it means that in both the modes,
         * ARW check is already done, so just adjust the window
         */
        ret = adk_ipsec_arw_update(ipsecMetaData->seqNumber, ipsecMetaData->saId);
        if (unlikely((ADK_E_IPSEC_ARW_REPLAY == ret) || (ADK_E_IPSEC_ARW_OUTDATED_VALUE == ret)))
        {
            /* drop and raise and audit event
             */
            if (ADK_IPSEC_IPV4 == innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_arw_update_fail_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_arw_update_fail_drop++;
            }
            return ADK_E_DROP_TASK;
        }
    }

    if (ADK_IPSEC_IPV4 == innerVersion)
    {
        /* this counter here is only to ensure backward compatibility.
         * In mixed mode, this can cause confusion!
         */
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].ingressUtilityCounters.ipsec_ing_num_pdus_rcvd_after_inbound_processing);
        adk_ip_dp_lli_imc_data_t *ip_imc = NULL;
        ip_imc = (adk_ip_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
        if(operType != ADK_IPSEC_LOOKASIDE)
            ip_imc->ipsec_data = IPLLI_IMC_IPSEC_INNER_INLINE;
        else
            ip_imc->ipsec_data = IPLLI_IMC_IPSEC_INNER_LOOKASIDE;

        fn_id = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    }
    else
    {
        /* this counter here is only to ensure backward compatibility.
         * In mixed mode, this can cause confusion!
         */
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].ingressUtilityCounters.ipsec_ing_num_pdus_rcvd_after_inbound_processing);
        adk_ipv6_dp_lli_imc_data_t *ipv6_imc = NULL;
        ipv6_imc = (adk_ipv6_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
        ipv6_imc->ipsec_data = IPLLI_IMC_IPSEC_INNER_INLINE;
        fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    }

    /* TODO : Rewrite the IMC params that came from IP module before sending it back,
     * along with other params that IP may need.
     */
    adkMetaData->ip_offset = offset;
#if defined(IPSEC_LLI_SUB_BLOCKS)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_AFTER_POLICY_LKUP_TX_TS();
#endif
#if !defined(IPSEC_LLI_SUB_BLOCKS) && !defined(IPSEC_LLI_PLAIN_TEXT)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_TX_TS();
#endif   
    if (likely(adkDestEntryFunction_g[fn_id]))
    {
        ret = adk_pal_dp_stage_send((void **)taskList, 1, 1, fn_id, 0x00,
                0x00, ADK_PAL_DP_BATCH_CB_FN_IPSEC, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
        if (unlikely(ret == ADK_E_INTERNAL_ERROR))
        {
            if (ADK_IPSEC_IPV4 == innerVersion)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
            }
            return ADK_E_DROP_TASK;
        }
    }
    else
    {
        if (ADK_IPSEC_IPV4 == innerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
        }
        return ADK_E_DROP_TASK;
    }

    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipsec_dp_pkt_proc_lli_from_crypto ()
 *
 * Structure of task params :
 *
 * pduParams[7]
 *
 * Brief:
 *
 * =============================================================
 */
adk_status_t
adk_ipsec_dp_pkt_proc_lli_from_crypto (void **taskList, __attribute__((unused))uint8_t task_count,
                                       __attribute__((unused))uint32_t task_mask,
                                       __attribute__((unused)) void *ring,
                                       __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry,
                                       int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;

    adk_pktio_task_t  *taskInfo = NULL;
    struct rte_mbuf   *pktMbuf = NULL;
    adk_metadata_t    *adkMetaData = NULL;
    struct rte_crypto_op *op_data = NULL;
    ipsec_dp_inbound_meta_data_t *ipsecMetaData = NULL;
    uint8_t cryptoStatus;

    op_data = (struct rte_crypto_op *)taskList[0];
    pktMbuf = (struct rte_mbuf*)op_data->sym->m_src;
    taskInfo = (adk_pktio_task_t*)(pktMbuf);
    adkMetaData = ADK_METADATA_PTR(taskInfo);
    ipsecMetaData = (ipsec_dp_inbound_meta_data_t *)&(adkMetaData->imc_param_start);
    cryptoStatus = op_data->status;
    rte_crypto_op_free(op_data);
    uint8_t  outerVersion = 0, *pktStart;

    if (unlikely(RTE_CRYPTO_OP_STATUS_SUCCESS != cryptoStatus))
    {
        /* hopefully, exceptional cases, so it is ok to parse necessary
         * packet here
         */
        adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][ipsecMetaData->saId].ipsec_ing_sa_esp_cryptographic_error++;
        if (RTE_CRYPTO_OP_STATUS_AUTH_FAILED == cryptoStatus)
        {
            adk_ipsec_audit_event_t auditEventData = {0};
            pktStart = rte_pktmbuf_mtod(pktMbuf, uint8_t *);
            outerVersion = (*(uint8_t *)(pktStart + adkMetaData->ip_offset)) >> 4;
            if (ADK_IPSEC_IPV4 == outerVersion)
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_auth_fail_drop++;
            else
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_auth_fail_drop++;
            ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_AUTH_CHECK_FAILURE, 
                                                &auditEventData, coreIndex, adkMetaData->outerIPVersion, ADK_IPSEC_INBOUND, false);
            if(unlikely(ADK_E_INVALID_HANDLE == ret))
            {
                if(ADK_IPSEC_IPV4 == outerVersion)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                }
            }
            return ret;
        }
        
        adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (taskInfo);
        return ADK_E_DROP_TASK;
    }

    ret = process_inbound_pkt((void**)&taskInfo, coreIndex, pktMbuf,
                              ADK_IPSEC_LOOKASIDE,
                              ipsecMetaData);
    if(unlikely(ADK_E_INTERNAL_ERROR == ret) || (ADK_E_DROP_TASK == ret))
    {
        if (ADK_IPSEC_IPV4 == outerVersion)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pkt_process_fail_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pkt_process_fail_drop++;
        }
		/*Free the task*/
        adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (taskInfo);
    }
    return ret;
}

static inline void map_hw_sa_to_adk_sa(const uint32_t *hwSAId, uint16_t *adkSAId)
{
    uint8_t i;
    uint32_t  hashVal;
    adk_ipsec_hw_sa_map_t *hwIdMapEntry;

    hashVal = *hwSAId & ADK_IPSEC_HWSA_MAP_HASH_MASK;

    for(i=0; i< ADK_IPSEC_HWSA_MAP_MAX_COLLISION; i++)
    {
        hwIdMapEntry = &adk_ipsec_dp_tbl_gbl_ptr->hwSAIdMap[hashVal][i];
        if (likely((hwIdMapEntry->isValid) &&
            (hwIdMapEntry->hwSAId == *hwSAId)))
        {
            *adkSAId = hwIdMapEntry->adkSAId;
            return;
        }
    }
}

static inline adk_status_t
adk_ipsec_create_spd_in_pattern_and_do_lkup(void **taskList, int coreIndex, uint8_t version, struct ipv4_hdr *ipv4Hdr, 
                                            struct ipv6_hdr *ipv6Hdr, struct udp_hdr *udpHdr, uint16_t *polIndex,
                                            uint8_t ipv6_nextHdr, uint16_t l4_offset)
{
    adk_ipsec_spd_in_sel_t spdPtn = {0};
    struct icmp_hdr *icmpHdr = NULL;
    uint32_t ret = 0;
    adk_pktio_task_t *taskInfo = (adk_pktio_task_t *)taskList[0];
    uint8_t *pdu = NULL;
    ADK_PDU_PTR_GET(taskInfo, pdu, uint8_t *);

    if (ADK_IPSEC_IPV4 == version)
    {
        if (unlikely(NULL == ipv4Hdr))
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_too_small_pkt_drop++;
            return ADK_E_DROP_TASK;
        }
        spdPtn.version = 4;
        spdPtn.sip_addr[0] = ipv4Hdr->src_addr;
        spdPtn.dip_addr[0] = ipv4Hdr->dst_addr;
        spdPtn.protocol = rte_cpu_to_be_32(ipv4Hdr->next_proto_id);
        if ((IPPROTO_UDP == ipv4Hdr->next_proto_id) ||
            (IPPROTO_TCP == ipv4Hdr->next_proto_id) ||
            (IPPROTO_SCTP == ipv4Hdr->next_proto_id))
        {
            *udpHdr = *(struct udp_hdr *)(ipv4Hdr + 1);
            spdPtn.sport = udpHdr->src_port;
            spdPtn.dport = udpHdr->dst_port;
        }
        else if(IPPROTO_ICMP == ipv4Hdr->next_proto_id)
        {
            icmpHdr = (struct icmp_hdr *)(ipv4Hdr + 1);
            spdPtn.sport = icmpHdr->icmp_type;
            spdPtn.dport = icmpHdr->icmp_code;
        }
        else
        {
            spdPtn.sport = 0;
            spdPtn.dport = 0;
        }
    }
    else if (ADK_IPSEC_IPV6 == version)
    {
        if (unlikely(NULL == ipv6Hdr))
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_too_small_pkt_drop++;
            return ADK_E_DROP_TASK;
        }
        spdPtn.version = 6;
        rte_memcpy(spdPtn.sip_addr, ipv6Hdr->src_addr, 16);
        rte_memcpy(spdPtn.dip_addr, ipv6Hdr->dst_addr, 16);
        spdPtn.protocol = rte_cpu_to_be_32(ipv6_nextHdr);
        if (ADK_IPSEC_V6_NEXT_HDR_FRAG_HDR == ipv6_nextHdr)
        {
            spdPtn.sport = spdPtn.dport = 0;
        }
        else if ((IPPROTO_UDP == ipv6_nextHdr) ||
                (IPPROTO_TCP == ipv6_nextHdr) ||
                (IPPROTO_SCTP == ipv6_nextHdr))
        {
            *udpHdr = *(struct udp_hdr *)(pdu + l4_offset);
            spdPtn.sport = udpHdr->src_port;
            spdPtn.dport = udpHdr->dst_port;
        }
        else if(IPPROTO_ICMPV6 == ipv6_nextHdr)
        {
            icmpHdr = (struct icmp_hdr *)(pdu + l4_offset);
            spdPtn.sport = icmpHdr->icmp_type;
            spdPtn.dport = icmpHdr->icmp_code;
        }
        else
        {
            spdPtn.sport = 0;
            spdPtn.dport = 0;
        }
    }
    else
    {
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid IP header version. Dropping the packet");
        return ADK_E_DROP_TASK;
    }
    ret = adk_ipsec_spd_in_lkup(&spdPtn, polIndex);
    if (unlikely(ret != ADK_SUCCESS))
    {
        adk_ipsec_audit_event_t auditEventData = {0};
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }
        else 
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }
        ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SPD_IN_LKUP_DISCARD,
                                            &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            if(ADK_IPSEC_IPV4 == version)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
        }
    }
    return ret;
}

static adk_status_t
process_lcl_plain_text_and_fwd_pkt(void **taskList, int coreIndex, uint8_t version, struct ipv4_hdr *ipv4Hdr,
                                   struct ipv6_hdr *ipv6Hdr, uint8_t isLocal, uint8_t *isNatPkt)
{
 
    uint32_t ret = 0;
    struct udp_hdr udpHdr = {0};
    uint16_t polIndex = ADK_IPSEC_INVALID_SPD_ID;
    uint8_t fn_id = 0;
    adk_pktio_task_t *taskInfo = (adk_pktio_task_t *)taskList[0];
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(taskInfo);
    adk_ipsec_dp_lli_imc_param_t *ipsecLliImc = (adk_ipsec_dp_lli_imc_param_t *)
                                                 &(adkMetaData->imc_param_start);

    uint8_t pktOrigin = ipsecLliImc->pktOrigin;
    
    adk_dsi_rx_metadata_t *dsiRxMetaData = ADK_DSI_RX_METADATA_PTR(taskInfo);
    if (likely(FALSE == adk_hos_is_spd_id_invalid(dsiRxMetaData)))
    {
        polIndex = adk_hos_get_spd_id(dsiRxMetaData);
        if ((ADK_IPSEC_IPV4 == version) && (IPPROTO_UDP == ipv4Hdr->next_proto_id))
        {
            udpHdr = *(struct udp_hdr *)(ipv4Hdr + 1); 
        }
    }
    else
    {
#if defined(IPSEC_LLI_SUB_BLOCKS)        
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_RX_TS();
#endif
        ret = adk_ipsec_create_spd_in_pattern_and_do_lkup(taskList, coreIndex, version, ipv4Hdr,
                                                          ipv6Hdr, &udpHdr, &polIndex,
                                                          ipsecLliImc->ipv6_nextHdr, adkMetaData->l4_offset);
#if defined(IPSEC_LLI_SUB_BLOCKS)        
        ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_UNEXPECTED_TX_TS();
#endif
        if(unlikely(ADK_SUCCESS != ret))
        {
            if (ADK_IPSEC_IPV4 == version)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_lookup_fail_drop++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_lookup_fail_drop++;
            }
           return ret;
        }
    }
    if (unlikely(0 == polIndex))
    {
        adk_ipsec_audit_event_t auditEventData = {0};
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }
        else 
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }
        ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_SPD_IN_LKUP_DISCARD,
                                            &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
        if(unlikely(ADK_E_INVALID_HANDLE == ret))
        {
            if(ADK_IPSEC_IPV4 == version)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
            else
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
            }
        }
        return ret;
    }
    if(unlikely(ADK_IPSEC_SP_ACTION_DROP == adk_ipsec_dp_tbl_gbl_ptr->spdInAction[polIndex]))
    {
       if (ADK_IPSEC_IPV4 == version)
       {
           adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_pol_discard_drop++;
       }
       else
       {
           adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_pol_discard_drop++;
       }
       return ADK_E_DROP_TASK;
    }
    else if (likely(ADK_IPSEC_SP_ACTION_BYPASS == adk_ipsec_dp_tbl_gbl_ptr->spdInAction[polIndex]))
    {
       /* Action is bypass */
       if (ADK_IPSEC_IPV4 == version)
       {
           ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_pol_bypass_count;
           if (likely((isLocal) && (ADK_IPSEC_NATT_PORT != rte_cpu_to_be_16(udpHdr.dst_port))))
           {
               adk_ip_dp_lli_imc_data_t *ip_imc = NULL;
               ip_imc = (adk_ip_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
               ip_imc->ipsec_data = IPLLI_IMC_IPSEC_OUTER;
               fn_id = adkModuleMgrCpuServiceFn_g[ADK_IP_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
           }
           else if (!isLocal)
           {
               adk_ipsec_dp_uli_imc_param_t *ipsecUliImc = NULL;
               ipsecUliImc = (adk_ipsec_dp_uli_imc_param_t *)&(adkMetaData->imc_param_start);
               ipsecUliImc->pktOrigin = pktOrigin;
               fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPSEC_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];	   
           }
           else
           {
              /* Local and NAT-T packet */
              *isNatPkt = 1;
              return ADK_SUCCESS;   
           }
       }
       else
       {
          ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_pol_bypass_count;
          if (likely(isLocal))
          {
              adk_ipv6_dp_lli_imc_data_t *ipv6_imc = NULL;
              ipv6_imc = (adk_ipv6_dp_lli_imc_data_t *)(&(adkMetaData->imc_param_start));
              ipv6_imc->ipsec_data = IPLLI_IMC_IPSEC_OUTER;
              fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPV6_MODULE_ID].cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
          }
          else
          {
              adk_ipsec_dp_uli_imc_param_t *ipsecUliImc = NULL; 
              ipsecUliImc = (adk_ipsec_dp_uli_imc_param_t *)(&(adkMetaData->imc_param_start));
              ipsecUliImc->pktOrigin = pktOrigin;
              ipsecUliImc->ipv6_nextHdr = ipsecLliImc->ipv6_nextHdr;
              fn_id = adkModuleMgrCpuServiceFn_g[ADK_IPSEC_MODULE_ID].cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
          }
       }
#if !defined(IPSEC_LLI_SUB_BLOCKS) && defined(IPSEC_LLI_PLAIN_TEXT)       
       ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_TX_TS();
#endif
       if (likely(adkDestEntryFunction_g[fn_id]))
       {
           ret = adk_pal_dp_stage_send((void **)taskList, 1, 1, fn_id, 0x00,
                   0x00, ADK_PAL_DP_BATCH_CB_FN_IPSEC, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
           if (unlikely(ret == ADK_E_INTERNAL_ERROR))
           {
               if (ADK_IPSEC_IPV4 == version)
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
               }
               else
               {
                   adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_send_to_next_stage_failed++;
               }
               return ADK_E_DROP_TASK;
           }
       }
       else
       {
           if (ADK_IPSEC_IPV4 == version)
           {
               adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
           }
           else
           {
               adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_invalid_dst_fn_id_drop++;
           }
           return ADK_E_DROP_TASK;
       }
    }
    else
    {
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_spd_in_default_discard_drop++;
        }

        return ADK_E_DROP_TASK;
    }
    return ADK_SUCCESS;
}

/* This function gets called for decrypting packets in both
 * look aside as well as inline mode.
 * In either modes, basic packet validations and IPsec specific
 * validations like life time are done. After that, if the packet
 * needs to be processed "inline", then a function gets called that
 * does same functionality as "from_crypto", else, IPsecMetaData
 * is populated and packet is sent to crypto BD for decryption
 */
adk_status_t
adk_ipsec_dp_pkt_proc_lli_single (void **taskList, int coreIndex)
{
#if !defined(IPSEC_LLI_SUB_BLOCKS) && !defined(IPSEC_LLI_PLAIN_TEXT)    
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_POST_CRYPTO_LLI_RX_TS();
#endif
#if !defined(IPSEC_LLI_SUB_BLOCKS) && defined(IPSEC_LLI_PLAIN_TEXT)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_LLI_RX_TS();
#endif
    adk_status_t ret = ADK_SUCCESS;

    adk_pktio_task_t  *taskInfo = (adk_pktio_task_t *)taskList[0];
    struct rte_mbuf   *pktMbuf = (struct rte_mbuf *)taskInfo;
    uint8_t *pdu;

    adk_metadata_t            *adkMetaData = ADK_METADATA_PTR(taskInfo);

    struct ipv4_hdr      *ipv4Hdr = NULL;
    struct ipv6_hdr      *ipv6Hdr = NULL;
    struct esp_header    *espHdr = NULL;

    uint8_t   version, ipHdrLen = 0, ecnFlag = 0;
    uint16_t  frag_field = 0, frag_offset = 0;
    bool frag_flag = FALSE;
    uint32_t  packetLen = 0;

    uint8_t   operType = ADK_IPSEC_INLINE;
    uint8_t isNatPkt = 0;
    uint8_t isLocal = ((adk_ipsec_dp_lli_imc_param_t *)
                    &(adkMetaData->imc_param_start))->isLocal;
    adk_dsi_rx_metadata_t     *dsiRxMetaData = NULL;
    uint8_t nextHdr = 0;

    pdu = rte_pktmbuf_mtod(pktMbuf, uint8_t *);
    dsiRxMetaData = ADK_DSI_RX_METADATA_PTR(taskInfo);
    if (unlikely(NULL == pdu))
    {
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid PDU poniter. Dropping the packet");
        return ADK_E_DROP_TASK;
    }
    version = adkMetaData->outerIPVersion;
#if defined(IPSEC_LLI_SUB_BLOCKS)
    ADK_DP_MEASURE_BLOCK_CYCLE_IPSEC_PROCESS_INBOUND_LLI_RX_TS();
#endif
    if (ADK_IPSEC_IPV4 == version)
    {
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].ingressUtilityCounters.ipsec_ing_total_pdu_recv_cnt);
        ipv4Hdr = (struct ipv4_hdr *)(pdu + adkMetaData->ip_offset);
        ecnFlag = ((ipv4Hdr->type_of_service & ADK_IPSEC_IPTOS_ECN_CE) == ADK_IPSEC_IPTOS_ECN_CE);
        ipHdrLen = IPV4_HDR_LEN;
        packetLen = ntohs(ipv4Hdr->total_length);
        nextHdr = ipv4Hdr->next_proto_id;
        if (likely(IPPROTO_ESP == nextHdr))
        {
            espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
        }
    }
    else if (ADK_IPSEC_IPV6 == version)
    {
        adk_ipsec_dp_lli_imc_param_t *ipsecLliImc = NULL;
        ipsecLliImc = (adk_ipsec_dp_lli_imc_param_t *)(&(adkMetaData->imc_param_start));
        nextHdr = ipsecLliImc->ipv6_nextHdr;
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].ingressUtilityCounters.ipsec_ing_total_pdu_recv_cnt);
        ipv6Hdr = (struct ipv6_hdr *)(pdu + adkMetaData->ip_offset);
        ecnFlag = ntohl(ipv6Hdr->vtc_flow) >> 20;
        ecnFlag = ((ecnFlag & ADK_IPSEC_IPTOS_ECN_CE) == ADK_IPSEC_IPTOS_ECN_CE);
        ipHdrLen = sizeof(struct ipv6_hdr);
        packetLen = ntohs(ipv6Hdr->payload_len) + sizeof(struct ipv6_hdr);
        if(ETH_FCS_LEN == (rte_pktmbuf_pkt_len(pktMbuf) - (packetLen + taskInfo->l2_len)))
        {
            rte_pktmbuf_trim(pktMbuf, ETH_FCS_LEN);
        }
        if(likely(IPPROTO_ESP == nextHdr))
        {
            espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
        }
    }
    else
    {
        /* IPv4/v6 header validations should be done in IP stack. Ideally, control should not reach here. */
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Invalid IP header version. Dropping the packet");
        return ADK_E_DROP_TASK;
    }
    
    adk_ipsec_dp_ing_sa_t        *spiInfo = NULL;
    if(likely(1 == dsiRxMetaData->ipsec_offloaded))
    {
        /* Inline decryption is successful and no error code is returned from CPM */
        if (likely(0 == dsiRxMetaData->ipsec_status))
        {
            INCREMENT_INGRESS_UTILITY_COUNTER(version, ipsec_ing_inline_decrypted);
            /* now first or all map the sa_idx to the one that we
             * understand!
             */
            uint8_t   offset; 
            uint16_t  adkSAId = 0;
            map_hw_sa_to_adk_sa(&dsiRxMetaData->sa_idx, &adkSAId);
            spiInfo = &adk_ipsec_dp_tbl_gbl_ptr->iSATbl[adkSAId];
            if (unlikely(NULL == spiInfo))
            {
                return adk_ipsec_raise_exception_null_spi_info(coreIndex, 
                                                               taskInfo, version);
            }
            offset = adkMetaData->l4_offset + ESPV4_HDR_LEN + spiInfo->ivLen;
            rte_prefetch0(rte_pktmbuf_mtod_offset(pktMbuf, uint8_t *,
                        rte_pktmbuf_pkt_len(pktMbuf)
                        - spiInfo->digestLen - 2));
            /*Prefetch Inner header's cache lines */
            rte_prefetch0(pdu + offset);
            rte_prefetch0((pdu + offset) + 64);
            /* in this function, hard/soft lifetime is validated, so ensure to send
             * only the length that got decrypted!
             */
            ret = adk_ipsec_check_spi_config(coreIndex, taskInfo, adkSAId,
                    rte_be_to_cpu_32(espHdr->seqNum),
                    packetLen,
                    ipHdrLen,
                    version);
           
            if (unlikely(ADK_SUCCESS != ret))
            {
                return ret;
            }
            spiInfo->bytesUsed += packetLen - ipHdrLen - spiInfo->digestLen;
            ++(adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][spiInfo->iSAIndex].ipsec_ing_sa_tot_in_pkts);
            
            ipsec_dp_inbound_meta_data_t  inlineMetaData;
            inlineMetaData.saId = spiInfo->iSAIndex;
            inlineMetaData.saHandle = spiInfo->iSAHandle;
            inlineMetaData.dscpVal_version = spiInfo->dscpVal;
            inlineMetaData.antiReplayEnable = spiInfo->antiReplayEnable;
            inlineMetaData.digestLen = spiInfo->digestLen;
            inlineMetaData.ivLen = spiInfo->ivLen;
            inlineMetaData.useSingleSaInPreFrag = spiInfo->useSingleSaInPreFrag;
            inlineMetaData.seqNumber = rte_be_to_cpu_32(espHdr->seqNum);
            inlineMetaData.ecnFlag = ecnFlag;
            if (unlikely(spiInfo->saMode == ADK_IPSEC_SA_LOOKASIDE))
                operType = ADK_IPSEC_LOOKASIDE;
            ret = process_inbound_pkt(taskList, coreIndex, pktMbuf,
                    operType,
                    &inlineMetaData);
            if (unlikely((ADK_E_DROP_TASK == ret) || (ADK_E_INTERNAL_ERROR == ret)))
            {
                if (ADK_IPSEC_IPV4 == version)
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pkt_process_fail_drop++;
                }
                else
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pkt_process_fail_drop++;
                }
            }
            return ret;
        }
        else
        {
            /* handle various error codes that CPM may return
             * here, only if is really not decrypted, we attempt lookaside
             */
            if (likely((ADK_IPSEC_CPM_ERROR_SA_MISS == dsiRxMetaData->ipsec_status) ||
                       (ADK_IPSEC_CPM_ERROR_NOT_DECRYPTED == dsiRxMetaData->ipsec_status)))
            {
                /* maybe this needs lookaside? */
                operType = ADK_IPSEC_LOOKASIDE;
                /* perform SPI lookup */
                spiInfo = adk_ipsec_spi_lkup(espHdr->spi);
            }
            else
            {
                /* for these errors, drop the packet and raise event
                 * if required -
                 * ADK_IPSEC_CPM_ERROR_ICV_FAILED
                 * ADK_IPSEC_CPM_ERROR_WRONG_LEN
                 * ADK_IPSEC_CPM_ERROR_MISC
                 * hopefully, exceptional cases
                 */
                uint16_t  adkSAId = 0;
                map_hw_sa_to_adk_sa(&dsiRxMetaData->sa_idx, &adkSAId);
                adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][adkSAId].ipsec_ing_sa_esp_cryptographic_error++;
                if (likely(ADK_IPSEC_CPM_ERROR_ICV_FAILED == dsiRxMetaData->ipsec_status))
                {
                    adk_ipsec_audit_event_t auditEventData = {0};
                    if (ADK_IPSEC_IPV4 == version)
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_auth_fail_drop++;
                    else
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_auth_fail_drop++;
                    ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_AUTH_CHECK_FAILURE, 
                                                        &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
                    if(unlikely(ADK_E_INVALID_HANDLE == ret))
                    {
                        if(ADK_IPSEC_IPV4 == version)
                        {
                            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                        }
                        else
                        {
                            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                        }
                    }
                    return ret;
                }
                return ADK_E_DROP_TASK;
            }
        }
    }
    else
    {
        if (ADK_IPSEC_IPV4 == version)
        {
            frag_field = rte_cpu_to_be_16(ipv4Hdr->fragment_offset);
            frag_offset = (uint16_t)(frag_field & IPV4_HDR_OFFSET_MASK);
            frag_flag = (uint16_t)(frag_field & IPV4_HDR_MF_FLAG);
            /* Raising the IPsec exception only when the received PDU is locally terminated and fragmented packet */
            if (unlikely(isLocal && ((frag_offset) || (!frag_offset && frag_flag))))
            {
                adk_ipsec_audit_event_t auditEventData = {0};
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_ip_fragment_drop++;
                ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_IP_FRAGMENT_DROP, 
                        &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
                if(unlikely(ADK_E_INVALID_HANDLE == ret))
                {
                    if(ADK_IPSEC_IPV4 == version)
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                    else
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                }
                return ret;
            }
            if ((isLocal && (IPPROTO_ESP != nextHdr)) ||
                (!isLocal))
            {
                /* process_lcl_plain_text_and_fwd_pkt is called for local plain text and forwarded traffic */
                ret = process_lcl_plain_text_and_fwd_pkt (taskList, coreIndex, version, ipv4Hdr, NULL, isLocal, &isNatPkt);    
                if (likely(!isNatPkt))
                {
                    return ret;
                }
            }
            if (IPPROTO_UDP == nextHdr)
            {
                struct udp_hdr       *udpHdr  = NULL;
                udpHdr = (struct udp_hdr *)(pdu + adkMetaData->l4_offset);
                if (unlikely(ADK_IPSEC_NATT_PORT != rte_cpu_to_be_16(udpHdr->dst_port)))
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_unknown_natt_pdu_drop++;
                    return ADK_E_DROP_TASK;
                }
                /* if this is keepalive, drop it */
                if (unlikely(ADK_IPSEC_NATT_KEEPALIVE_SIZE == ADK_TASK_PDU_SIZE(pktMbuf)))
                {
                    adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_keepalive_pdu_drop++;
                    return ADK_E_DROP_TASK;
                }
                espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset + UDPV4_HDR_LEN);
                /* update ipHdrLen as we rely on this to decide IV location
                 * and cipher sizes etc
                 */
                ipHdrLen += UDPV4_HDR_LEN;
                /* lets also chang the l4_offset to reflect this */
                adkMetaData->l4_offset += UDPV4_HDR_LEN;
            }
            else if (IPPROTO_ESP != nextHdr)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_unsupported_l4_proto_drop++;
                return ADK_E_DROP_TASK;
            }
        }
        else if (ADK_IPSEC_IPV6 == version)
        {
           /* Raising the IPsec exception only when the received PDU is locally terminated and fragmented packet */ 
            if (unlikely(isLocal && (ADK_IPSEC_V6_NEXT_HDR_FRAG_HDR == nextHdr)))
            {
                adk_ipsec_audit_event_t auditEventData = {0};
                ret = adk_ipsec_dp_handle_exception(taskInfo, ADK_IPSEC_EXCEPTION_AUDIT_ING_IP_FRAGMENT_DROP,
                        &auditEventData, coreIndex, version, ADK_IPSEC_INBOUND, false);
                if(unlikely(ADK_E_INVALID_HANDLE == ret))
                {
                    if(ADK_IPSEC_IPV4 == version)
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                    else
                    {
                        adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_not_registered++;
                    }
                }
                return ret;
            }
            if ((isLocal && (nextHdr != IPPROTO_ESP)) ||
                (!isLocal))
            {
                /* process_lcl_plain_text_and_fwd_pkt is called for local plain text and forwarded traffic */
                ret = process_lcl_plain_text_and_fwd_pkt (taskList, coreIndex, version, NULL, ipv6Hdr, isLocal, &isNatPkt);
                return ret;
            }
            if (IPPROTO_ESP != nextHdr)
            {
                adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_unsupported_l4_proto_drop++;
            }
        }

        /* Inline decryption failed, trying for lookaside decryption */
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_inline_decryption_failed++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_inline_decryption_failed++;
        }
        operType = ADK_IPSEC_LOOKASIDE;
        /* perform SPI lookup */
        spiInfo = adk_ipsec_spi_lkup(espHdr->spi);
    }
    
    if (unlikely(NULL == spiInfo))
    {
        return adk_ipsec_raise_exception_null_spi_info(coreIndex, 
                                                       taskInfo, version);
    }
    
    ret = adk_ipsec_check_spi_config(coreIndex, taskInfo, spiInfo->iSAIndex,
                                     rte_be_to_cpu_32(espHdr->seqNum),
                                     packetLen,
                                     ipHdrLen,
                                     version);
    if (unlikely(ADK_SUCCESS != ret))
    {
        return ret;
    }
    ipsec_dp_inbound_meta_data_t  *ipsecMetaData = NULL;
    ipsecMetaData = (ipsec_dp_inbound_meta_data_t *)&(adkMetaData->imc_param_start);
    struct rte_crypto_op        *opData;
    opData = spiInfo->lkasideInfo->cryptoOp;
    if (unlikely(NULL == opData))
    {
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_la_no_buffer_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_la_no_buffer_drop++;
        }
        return ADK_E_DROP_TASK;
    }
    ipsecMetaData->saId = spiInfo->iSAIndex;
    ipsecMetaData->saHandle = spiInfo->iSAHandle;
    ipsecMetaData->dscpVal_version = spiInfo->dscpVal;
    ipsecMetaData->antiReplayEnable = spiInfo->antiReplayEnable;
    ipsecMetaData->digestLen = spiInfo->digestLen;
    ipsecMetaData->ivLen = spiInfo->ivLen;
    ipsecMetaData->seqNumber = rte_be_to_cpu_32(espHdr->seqNum);
    ipsecMetaData->ecnFlag = ecnFlag;
    ++(adk_ipsec_dp_tbl_gbl_ptr->ingSAStats[coreIndex][spiInfo->iSAIndex].ipsec_ing_sa_tot_in_pkts);
 
    union rte_event_crypto_metadata cryptoMetaData = { 0 };
    cryptoMetaData.request_info.cdev_id = spiInfo->lkasideInfo->cryptoDevId;
    cryptoMetaData.request_info.queue_pair_id = spiInfo->lkasideInfo->cryptoQP;
    adk_pal_dp_stage_info_t *stageInfo = NULL;
    adk_pal_dp_stage_info_get(ADK_IPSEC_LLI_FROM_CRYPTO_FN_ID, &stageInfo);
    cryptoMetaData.response_info.queue_id = stageInfo->event_q_id;
    cryptoMetaData.response_info.sub_event_type = ADK_IPSEC_LLI_FROM_CRYPTO_FN_ID;

    cryptoMetaData.response_info.flow_id = spiInfo->iSAIndex;
    ret = rte_cryptodev_sym_session_set_user_data(spiInfo->lkasideInfo->cryptoSession,
            (void *)&cryptoMetaData,
            sizeof(cryptoMetaData));
    if (unlikely(ret != 0))
    {
        printf( "Failed to set crypto BD metadata\n");
        return ADK_E_DROP_TASK;
    }

    uint32_t bytesDecrypted = 0;
    bytesDecrypted = adk_ipsec_inbound_la_prepare_crypto_op(taskInfo, opData,
                                                            spiInfo,
                                                            ipHdrLen,
                                                            packetLen);
    if (unlikely(0 == bytesDecrypted))
    {
        if (ADK_IPSEC_IPV4 == version)
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_pad_alloc_fail_drop++;
        }
        else
        {
            adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_pad_alloc_fail_drop++;
        }
        return ADK_E_DROP_TASK;
    }

    spiInfo->bytesUsed += bytesDecrypted;
    if (ADK_IPSEC_IPV4 == version)
    {
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].ingressUtilityCounters.ipsec_ing_la_num_pdus_sent_for_inbound_processing);
    }
    else
    {
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].ingressUtilityCounters.ipsec_ing_la_num_pdus_sent_for_inbound_processing);
    }

    spiInfo->inFlightCnt++;
    adk_ipsec_dp_tbl_gbl_ptr->SAUtilityCnt[coreIndex][spiInfo->iSAIndex].ingSaUtilityCounters.ipsec_ing_sa_in_flight_pkt_count++;
    /* if we have reached till here, it means this is lookaside for sure
     */
     ret = adk_pal_dp_crypto_send(spiInfo->lkasideInfo->cryptoDevId,
                                 ADK_IPSEC_LLI_FROM_CRYPTO_FN_ID,
                                 0, /* priority */
                                 taskInfo,
                                 opData,
                                 ADK_PAL_DP_BATCH_CB_FN_IPSEC,
                                 NULL);

    if (unlikely(ADK_E_TASK_CONSUMED != ret))
    {
        return ADK_E_INTERNAL_ERROR;
    }
    return ADK_SUCCESS;
}

/* This is the callback function invoked by IP in single SA case when timer expires or reassembly completed.
 * This function will delete all the hash context stored in the particular hash index.
 */
uint16_t
adk_ipsec_clear_initial_frag_buffered_pkts_hash_entry(uint16_t hashIndex, uint8_t bucketIndex)
{
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    uint16_t drop_count=0;
    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    adk_pktio_task_t *cur_task = NULL, *next_task = NULL;
    cur_task = hashInfo->buffered_pkt_task_chain;
    while(cur_task)
    {
        next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
        adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (cur_task);
        drop_count++;
        cur_task = next_task;
    }
    *hashInfo = (adk_ipsec_initial_frag_buffered_pkts_info_t){0};
    hashInfo->buffered_pkt_task_chain = NULL;
    return drop_count;
}

/* This is the callback function invoked by IPv6 in single SA case when timer expires or reassembly completed.
 * This function will delete all the hash context stored in the particular hash index.
 */
uint16_t
adk_ipsec_ipv6_clear_initial_frag_buffered_pkts_hash_entry(uint16_t hashIndex, uint8_t bucketIndex)
{
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    uint16_t drop_count=0;
    hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    adk_pktio_task_t *cur_task = NULL, *next_task = NULL;
    cur_task = hashInfo->buffered_pkt_task_chain;
    while(cur_task)
    {
        next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
        adk_pal_dp_release_send(cur_task, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        adk_pktio_task_buffer_free (cur_task);
        drop_count++;
        cur_task = next_task;
    }
    *hashInfo = (adk_ipsec_initial_frag_buffered_pkts_info_t){0};
    hashInfo->buffered_pkt_task_chain = NULL;
    return drop_count;
}

