/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include <rte_bridge_crypto.h>
#include <math.h>
#include <adk_sys.h>
#include <adk_pmu.h>
#include <adk_metadata.h>
#include "adk_ph_dev_dp.h"
#include "adk_shm.h"
#include <adk_hash.h>

#include <adk_ipsec.h>
#include <adk_ipsec_common.h>
#include <adk_ipsec_dp.h>
#include "adk_ipsec_export.h"

adk_ipsec_dp_table_info_t *adk_ipsec_dp_tbl_gbl_ptr;
ipsecIdSeqDB_t adk_ipsec_IdSeqDB[ADK_MAX_NUM_CORES];
ipsecIdSeqRange_t adk_ipsec_IdSeqRange[ADK_MAX_NUM_CORES];

adk_status_t adk_ipsec_dp_init(void)
{
    adk_status_t retValue = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;
    uint32_t i, *workerList=NULL, numCores=0;
    adk_sys_resource_info_t         *cdevInfoList;
    adk_sys_rsrc_crypto_dev_info_t  *cryptoDevInfo;

    uint8_t   numDevs;
    char poolName[RTE_MEMPOOL_NAMESIZE];
    uint16_t  maxSessSize = 0, socketId = 0; 
    struct rte_mempool *sess_mp = NULL;

    retValue = adk_sys_config_crypto_dev_info_get (&cdevInfoList,
                                                   &numDevs);
    if (retValue != ADK_IPSEC_SUCCESS)
    {
        ADK_FORCE_LOG_ERROR ("[%s()] Cryptodev resource get function failed\n",
                __func__);
        return ADK_E_INTERNAL_ERROR;
    }

    /* Lookup the shared memory that gets created by control plane IP
     * module initialization */
    mz = rte_memzone_lookup(ADK_IPSEC_SHM_MEMZONE_NAME);
    if (mz == NULL)
    {
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Error: Failed to lookup IPsec shared memory");
        return ADK_DP_E_ERROR;
    }

    adk_ipsec_dp_tbl_gbl_ptr = (adk_ipsec_dp_table_info_t *)mz->addr;
    if (adk_ipsec_dp_tbl_gbl_ptr->cookie != ADK_IPSEC_MODULE_COOKIE)
    {
        ADK_IPSEC_DP_LOG_FATAL_ERROR("Error: Module initialization not completed");
        return ADK_DP_E_ERROR;
    }
    /* Initialize IP Sequence Number Range per DP core */
    workerList = adk_ipsec_dp_tbl_gbl_ptr->workerCoreList;
    numCores = adk_ipsec_dp_tbl_gbl_ptr->numCores;
    i = 0;
    adk_ipsec_IdSeqRange[workerList[i]].min = 0;
    adk_ipsec_IdSeqRange[workerList[i++]].max =
                         (pow(2,16) / numCores)-1;
    for (; i < numCores; i++)
    {
        adk_ipsec_IdSeqRange[workerList[i]].min =
            adk_ipsec_IdSeqRange[workerList[i - 1]].max + 1;
        adk_ipsec_IdSeqRange[workerList[i]].max =
            adk_ipsec_IdSeqRange[workerList[i - 1]].max +
            (pow(2, 16)/ numCores);
    }

    for (i = 0; i < numCores; i++)
    {
        adk_ipsec_IdSeqDB[workerList[i]].ipIdSeq =
            adk_ipsec_IdSeqRange[workerList[i]].min;
    }
    /* Initialize the LLI/ULI function pointers used in the data plane
     * processing during bind */
    adkDestEntryFunction_g[ADK_IPSEC_ULI_FN_ID] = adk_ipsec_dp_pkt_proc_uli;
    adkDestEntryFunction_g[ADK_IPSEC_ULI_FROM_CRYPTO_FN_ID] =
                                             adk_ipsec_dp_pkt_proc_uli_from_crypto;
    adkDestEntryFunction_g[ADK_IPSEC_LLI_FN_ID] = adk_ipsec_dp_pkt_proc_lli;
    adkDestEntryFunction_g[ADK_IPSEC_LLI_FROM_CRYPTO_FN_ID] =
                                             adk_ipsec_dp_pkt_proc_lli_from_crypto;
    if ((numDevs != 0) && (cdevInfoList != NULL))
    {
        for (i = 0; i < numDevs; i++)
        {
            cryptoDevInfo = (adk_sys_rsrc_crypto_dev_info_t *)cdevInfoList[i].crypto_dev_info_p;

            if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_INLINE)
            {
                adk_ipsec_dp_tbl_gbl_ptr->secCtxDP = (struct rte_security_ctx *)rte_eth_dev_get_sec_ctx(cryptoDevInfo->dev_id);
                if(adk_ipsec_dp_tbl_gbl_ptr->secCtxDP == NULL)
                {
                    ADK_IPSEC_DP_LOG_DEBUG("No inline device found! \n");
                }
            }
            else if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
            {
                if (cryptoDevInfo->num_sas != 0)
                {
                    /* now allocate sess pool for this device!
                     */
                    socketId = rte_cryptodev_socket_id(cryptoDevInfo->dev_id);
                    snprintf(poolName, RTE_MEMPOOL_NAMESIZE, "ipsec_sess_mp_%u", cryptoDevInfo->dev_id);
#if ADK_DPDK_VERSION >= 18050016
                    maxSessSize = rte_cryptodev_sym_get_private_session_size(cryptoDevInfo->dev_id);
#else
                    maxSessSize = rte_cryptodev_get_private_session_size(cryptoDevInfo->dev_id);
#endif
                    /*
                     * Create session mempool, with two objects per session,
                     * one for the session header and another one for the
                     * private session data for the crypto device.
                     */
                    sess_mp = rte_mempool_create(poolName,
                                ADK_IPSEC_NUM_OF_SESSIONS*2,
                                maxSessSize,
                                0, 0, NULL, NULL, NULL, NULL,
                                socketId, 0);
                    if (sess_mp == NULL)
                    {
                        ADK_IPSEC_DP_LOG_FATAL_ERROR(
                            "IPSEC: Failed to allocate memory\n");
                        return ADK_DP_E_ERROR;
                    }
                }
            }
        }
    }
    
    return retValue;
}

uint16_t
adk_ipsec_dp_egress_tx_cb(
        uint16_t port_id,
        uint16_t queue,
        struct rte_mbuf *pkts[],
        uint16_t nb_pkts,
        void *user_param)
{
    RTE_SET_USED(port_id);
    RTE_SET_USED(queue);
    RTE_SET_USED(user_param);
    uint8_t *pdu;
    uint64_t saSeqNum;
    adk_metadata_t *adkMetaData = NULL;
    struct esp_header *espHdr = NULL;
    int p = 0;
    adk_ipsec_dp_eg_sa_t *saInfo = NULL;
    uint16_t saID;
    for(p=0; p<nb_pkts && pkts[p]; ++p)
    {
        adkMetaData = ADK_METADATA_PTR(pkts[p]);
        ADK_PDU_PTR_GET (pkts[p], pdu, uint8_t *);
        saID = adkMetaData->saID;
        if (ADK_IPSEC_INVALID_SA_ID != saID)
        {
            saInfo = &adk_ipsec_dp_tbl_gbl_ptr->eSATbl[saID];
            espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
            espHdr->seqNum = (uint32_t) (saInfo->seqNumber++ & 0xffffffff);
            espHdr->seqNum = adk_htonl(espHdr->seqNum);
            switch (saInfo->encr)
            {
            case ADK_IPSEC_ENCR_AES_GCM_ICV8:
            case ADK_IPSEC_ENCR_AES_GCM_ICV12:
            case ADK_IPSEC_ENCR_AES_GCM_ICV16:
            case ADK_IPSEC_ENCR_AES_CCM_ICV8:
            case ADK_IPSEC_ENCR_AES_CCM_ICV12:
            case ADK_IPSEC_ENCR_AES_CCM_ICV16:
            case ADK_IPSEC_ENCR_AES_CTR:
            case ADK_IPSEC_ENCR_NULL_GMAC:
                saSeqNum = rte_cpu_to_be_64(saInfo->seqNumber);
                rte_memcpy(saInfo->ivBuf, &saSeqNum, sizeof(saSeqNum));
                break;
            }
        }
    }
    return nb_pkts;
}

inline adk_status_t adk_ipsec_spd_lkup(adk_ipsec_spd_sel_t *ptn,
                                       uint16_t *polIndex)
{
    adk_status_t retValue = 0;
    const uint8_t * data[] = {(uint8_t *)ptn};
    uint8_t spdTblIdx = adk_ipsec_dp_tbl_gbl_ptr->spdTblIdx;
    uint32_t  dpdkPolId = 0;

    if (unlikely(0 == adk_ipsec_dp_tbl_gbl_ptr->numPolicies[spdTblIdx]))
    {
        *polIndex = 0;
        return ADK_E_IPSEC_POLICY_NOT_FOUND;
    }
    retValue = rte_acl_classify_alg (
                    (struct rte_acl_ctx *)adk_ipsec_dp_tbl_gbl_ptr->spdTbl[spdTblIdx],
                    data,
                    &dpdkPolId,
                    1,
                    1,
                    RTE_ACL_CLASSIFY_SSE);
    *polIndex = (uint16_t)dpdkPolId;
    return retValue;
}

inline adk_status_t adk_ipsec_spd_in_lkup(adk_ipsec_spd_in_sel_t *ptn,
                                   uint16_t *polIndex)
{
    adk_status_t retValue = ADK_SUCCESS;
    const uint8_t * data[] = {(uint8_t *)ptn};
    uint8_t spdTblIdx = adk_ipsec_dp_tbl_gbl_ptr->spdInTblIdx;
    uint32_t  dpdkPolId = 0;

    if (unlikely(0 == adk_ipsec_dp_tbl_gbl_ptr->numInPolicies[spdTblIdx]))
    {
        *polIndex = 0;
        return ADK_E_IPSEC_POLICY_NOT_FOUND;
    }
    retValue = rte_acl_classify_alg (
                    (struct rte_acl_ctx *)adk_ipsec_dp_tbl_gbl_ptr->spdInTbl[spdTblIdx],
                    data,
                    &dpdkPolId,
                    1,
                    1,
                    RTE_ACL_CLASSIFY_SSE);
    *polIndex = (uint16_t)dpdkPolId;
    return retValue;
}

inline adk_ipsec_dp_ing_sa_t *adk_ipsec_spi_lkup(uint32_t spi)
{
    uint32_t hashVal, i, idx;
    adk_ipsec_dp_ing_sa_t *spiInfo = NULL;

    hashVal = SISD_CRC_HASH(0, spi);
    hashVal &=  ADK_IPSEC_SPI_TBL_HASH_MASK;

    for (i=0; i< ADK_IPSEC_SPI_TBL_MAX_COLLISION; i++)
    {
        if (adk_ipsec_dp_tbl_gbl_ptr->spiHashTbl[hashVal][i].isValid == TRUE)
        {
            idx = adk_ipsec_dp_tbl_gbl_ptr->spiHashTbl[hashVal][i].idx;
            spiInfo = &adk_ipsec_dp_tbl_gbl_ptr->iSATbl[idx];
            if (spiInfo->spi == spi)
                return spiInfo;
        }
    }
    return spiInfo;
}

/* This function does the following for non-initial fragments in single SA use case,
 * 1. If the initial fragment was already visited and there is no recent policy update, then SPD lookup is not needed.
 * 2. If the initial fragment was already visited and there was recent policy update happened, then SP and DP from initial fragment
 *    would be returned and policy lookup will be happened.
 * 3. If the initial fragment was not visited, then it would be buffered in the hash table at hashIndex given by IP.
 */
inline adk_status_t
adk_ipsec_get_initial_frag_info_or_buffer_non_initial_frags(int coreIndex, uint8_t innerVersion, 
                                                            uint16_t cur_saId, adk_pktio_task_t *taskInfo,
                                                            uint16_t pktLen, adk_ipsec_spd_sel_t *spdPtn, 
                                                            uint16_t ipHashIndex, uint8_t ipBucketIndex,
                                                            bool *isSPDLookUpNeeded)
{
    RTE_SET_USED(coreIndex);
    uint8_t cur_pol_ver = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[cur_saId].policy_version;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(taskInfo);
    adk_pktio_task_t *cur_task = NULL, *prev_task = NULL;
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    if (ADK_IPSEC_IPV4 == innerVersion)
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[ipHashIndex].bucket[ipBucketIndex];
    else
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[ipHashIndex].bucket[ipBucketIndex];

    if (hashInfo->initial_frag_visited)
    {
        if (unlikely(cur_pol_ver > hashInfo->initial_frag_policy_version))
        {
            spdPtn->sport = hashInfo->initial_frag_SP;
            spdPtn->dport = hashInfo->initial_frag_DP;
            *isSPDLookUpNeeded = TRUE;
            return ADK_SUCCESS;
        }
        else
        {
           *isSPDLookUpNeeded = FALSE;
           return ADK_SUCCESS;
        }
    }
    else if(!hashInfo->initial_frag_visited)
    {
        adkMetaData->ts_correlator = (pktLen << 16) | cur_saId;
        if (hashInfo->buffered_pkt_task_chain == NULL)
        {
            hashInfo->buffered_pkt_task_chain = taskInfo;
        }
        else
        {
            cur_task = hashInfo->buffered_pkt_task_chain;
            while(cur_task)
            {
                  prev_task = cur_task;
                  cur_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
            }
            ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)taskInfo;
        }
        hashInfo->numBufferedFrags++;
        INCREMENT_INGRESS_UTILITY_COUNTER(innerVersion, ipsec_ing_total_buffered_frags);
        adk_pal_dp_release_send(taskInfo, ADK_PAL_DP_BATCH_CB_FN_IPSEC, NULL);
        ADK_METADATA_PARAM(taskInfo, task_type) = ADK_TASK_TYPE_NEW;
        return ADK_E_TASK_CONSUMED;
    }
    
    return ADK_E_INTERNAL_ERROR;
}

/* This function updates the initial fragment information in the hash table at hash index given by IP.
 */
inline void
adk_ipsec_add_initial_frag_info(uint8_t innerVersion, uint16_t initial_frag_sa_id, uint16_t hashIndex,
                                uint8_t bucketIndex, uint16_t l4SrcPort, uint16_t l4DstPort)
{
    uint8_t initial_pol_ver = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[initial_frag_sa_id].policy_version;
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    if (ADK_IPSEC_IPV4 == innerVersion)
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    else
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    hashInfo->initial_frag_visited = TRUE;
    hashInfo->initial_frag_policy_version = initial_pol_ver;
    hashInfo->intial_frag_SA_ID = initial_frag_sa_id;
    hashInfo->initial_frag_SP = l4SrcPort;
    hashInfo->initial_frag_DP = l4DstPort;
    return;
}

/* This function adds the initial fragment at the last of the buffered_pkt_task_chain.
 */
inline void
adk_ipsec_buffer_initial_frag(int coreIndex, uint8_t innerVersion, uint16_t hashIndex, uint8_t bucketIndex, uint16_t pktLen,
                              uint16_t saId, adk_pktio_task_t *taskInfo)
{
    RTE_SET_USED(coreIndex);
    adk_ipsec_initial_frag_buffered_pkts_info_t *hashInfo = NULL;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(taskInfo);
    adk_pktio_task_t *cur_task = NULL, *prev_task = NULL;
    if (ADK_IPSEC_IPV4 == innerVersion)
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecInitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];
    else
        hashInfo = &adk_ipsec_dp_tbl_gbl_ptr->ipsecIPv6InitialFragInfoBufferedPktsHashTbl[hashIndex].bucket[bucketIndex];

    adkMetaData->ts_correlator = (pktLen << 16) | saId;
    cur_task = hashInfo->buffered_pkt_task_chain;
    while(cur_task)
    {
        prev_task = cur_task;
        cur_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(cur_task, next_frag);
    }
    ADK_METADATA_PTR(prev_task)->next_frag = (struct rte_mbuf *)taskInfo;
    hashInfo->numBufferedFrags++;
    INCREMENT_INGRESS_UTILITY_COUNTER(innerVersion, ipsec_ing_total_buffered_frags);
    return;
}

/* Clone packet */
struct rte_mbuf* adk_ipsec_pktmbuf_clone(struct rte_mbuf * inputPkt,
                            struct rte_mempool * pool)
{
    struct rte_mbuf *pkt_clone, *pkt_segment, *pkt, *prev;
    uint8_t nb_segs;
    adk_metadata_t *pktMd;

    pkt = inputPkt;
    pktMd = ADK_METADATA_PTR(pkt);
    nb_segs = pkt->nb_segs;
    pkt_clone = rte_pktmbuf_alloc(pool);
    if (unlikely(pkt_clone == NULL))
    {
        return NULL;
    }

    pkt_clone->nb_segs = pkt->nb_segs;
    pkt_clone->pkt_len = pkt->pkt_len;
    pkt_clone->port = pkt->port;
    pkt_clone->timestamp = pkt->timestamp;

    pkt_segment = pkt_clone;
    prev = NULL;

    /*copy ADK metadata for the first segment*/
    adk_metadata_t *adk_md_cap = ADK_METADATA_PTR(pkt_clone);
    rte_memcpy((void *)adk_md_cap,(void *)pktMd,
                offsetof(adk_metadata_t,imc_param_start));
    adk_md_cap->ip_offset = pktMd->ip_offset;
    adk_md_cap->l4_offset = pktMd->l4_offset;
    adk_md_cap->input_intf_id = pktMd->input_intf_id;
    adk_md_cap->task_type = (uint8_t)ADK_TASK_TYPE_NEW;

    /*copy all segments payload*/
    while(nb_segs)
    {
        rte_memcpy(rte_pktmbuf_mtod(pkt_segment, void *),
                rte_pktmbuf_mtod(pkt, void *), rte_pktmbuf_data_len(pkt));
        pkt_segment->data_len = rte_pktmbuf_data_len(pkt);
        nb_segs--;
        if(nb_segs)
        {
            prev = pkt_segment;
            pkt_segment->next = NULL;
            pkt_segment = rte_pktmbuf_alloc(pool);
            if(pkt_segment == NULL)
            {
                ADK_DP_LOG_ERROR(
                    ADK_IPSEC_MODULE_ID,
                    "Mbuf allocation failed for Pkt clone. Dropping packet!");
                break;
            }
            prev->next = pkt_segment;
            pkt= pkt->next;
        }
    }

    if(unlikely(pkt_segment == NULL))
    {
        pkt = pkt_clone;
        while(pkt != NULL)
        {
            prev= pkt->next;
            rte_pktmbuf_free(pkt); /* free all allocated segments */
            pkt = prev;
        }
        return NULL;
    }
    __rte_mbuf_sanity_check(pkt_clone, 1);
    return pkt_clone;
}

adk_status_t
adk_ipsec_dp_handle_exception(
    adk_pktio_task_t *phTask,
    uint8_t type,
    void *exceptionData,
    int coreIndex,
    uint8_t version,
    uint8_t direction,
    uint8_t isFrag)
{
    RTE_SET_USED(coreIndex);
    adk_pktio_task_t *next_task;
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipsec_dp_exception_handler_t  *exceptionHdl;
    adk_ph_dev_dp_metadata_t          *phMeta_p;
    adk_ipsec_ph_dp_cb_params_t *cbInfo = &adk_ipsec_dp_tbl_gbl_ptr->cbParams[coreIndex][adk_ipsec_dp_tbl_gbl_ptr->cbParamIndex];

    exceptionHdl =  adk_ipsec_dp_tbl_gbl_ptr->ipsecExceptionHandlerTble[type - 1];

    if (unlikely(!exceptionHdl))
    {
        return ADK_E_INVALID_HANDLE;
    }
    while(phTask)
    {
        next_task = (adk_pktio_task_t *)ADK_METADATA_PARAM(phTask, next_frag);

        /* Update PH metadata */
        phMeta_p = (adk_ph_dev_dp_metadata_t *)
            &(ADK_METADATA_PARAM(phTask,imc_param_start));
        phMeta_p->policerFlowId = exceptionHdl->policerId;

        ADK_MD_SET_ADK_DEV_ID(phTask, exceptionHdl->devId);
        ADK_MD_SET_DEV_TX_QUEUE_ID(phTask, exceptionHdl->queueId);
        phMeta_p->priority = exceptionHdl->priority;
        phMeta_p->cbFnId = exceptionHdl->cbFnId;
        phMeta_p->moduleId = ADK_IPSEC_MODULE_ID;

        adk_ipsec_dp_exception_data_t *ipsec_metadata_p;
        ipsec_metadata_p = (adk_ipsec_dp_exception_data_t *)&phMeta_p->imc_param_start;
        ADK_METADATA_PARAM(phTask, pdu_flow_id) = 0;
      
        cbInfo->version = version;
        cbInfo->direction = direction;
        cbInfo->phDevId = exceptionHdl->devId;
        cbInfo->phQueueId = exceptionHdl->queueId;

        switch (type)
        {
            case ADK_IPSEC_EXCEPTION_SA_ACQUIRE:
                ipsec_metadata_p->exceptionType = type;
                rte_memcpy(&ipsec_metadata_p->exceptionData.saAquireData, exceptionData, sizeof(adk_ipsec_sa_aquire_excep_data_t));
            break;
            case ADK_IPSEC_EXCEPTION_SA_EXPIRE:
                ipsec_metadata_p->exceptionType = type;
                rte_memcpy(&ipsec_metadata_p->exceptionData.saExpiryData, exceptionData, sizeof(adk_ipsec_sa_expiry_excep_data_t));
            break;
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_IP_FRAGMENT_DROP:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SA_NOT_PRESENT:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SEQNO_CHECK_FAILURE:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_AUTH_CHECK_FAILURE:
            case ADK_IPSEC_EXCEPTION_AUDIT_EG_SEQNO_OVERFLOW:
            case ADK_IPSEC_EXCEPTION_AUDIT_EG_SPD_OUT_LKUP_DISCARD:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SPD_IN_LKUP_DISCARD:
            case ADK_IPSEC_EXCEPTION_NO_POLICY_2ND_FRAGMENT_2SA_PREFRAG:
            case ADK_IPSEC_EXCEPTION_EG_PREFRAG_REQD_BUT_DF_BIT_SET:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SA_MISMATCH:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SPP_SPD_OUT_LKUP_DISCARD:
            case ADK_IPSEC_EXCEPTION_AUDIT_ING_SPP_SPD_OUT_LKUP_NO_IPSEC_DROP:
                ipsec_metadata_p->exceptionType = type;
                rte_memcpy(&ipsec_metadata_p->exceptionData.auditEventData, exceptionData, sizeof(adk_ipsec_audit_event_t));
            break;
            default :
                return ADK_E_DROP_TASK;
        }
        retValue = adk_ph_dev_dp_to_cp_send ((void *)phTask,ADK_PAL_DP_BATCH_CB_FN_IPSEC, cbInfo);
        if (retValue != ADK_SUCCESS)
        {
            return ADK_E_DROP_TASK;
        }
        if(!isFrag)
        {
            adk_ipsec_dp_tbl_gbl_ptr->cbParamIndex = (adk_ipsec_dp_tbl_gbl_ptr->cbParamIndex + 1) % ADK_PAL_MAX_ENQUEUE_SIZE;
        }
        phTask = next_task;
    }
    return ADK_E_TASK_CONSUMED;
}

static int set_bit_range_to_0(unsigned* A, unsigned l, unsigned h)
{
    unsigned i;
    unsigned l_bit, l_word;
    unsigned h_bit, h_word;
    unsigned l_bit_mask, h_bit_mask;

    if (l > h)
    {
        return 1;
    }

    l_bit = l % 32;
    l_word = l / 32;
    h_bit = h % 32;
    h_word = h / 32;

    l_bit_mask = (unsigned)((1 << l_bit) - 1); // 2 -> 0x03
    if (h_bit == 31)
    {
        h_bit_mask = 0;
    }
    else
    {
        h_bit_mask = ~((1 << (h_bit+1)) - 1); // 2 -> 0xfffffff8
    }

    if (l_word == h_word)
    {
        A[l_word] &= (l_bit_mask | h_bit_mask);
    }
    else
    {
        A[l_word] &= l_bit_mask;
        A[h_word] &= h_bit_mask;
        if (h_word - l_word > 1)
            for (i = l_word + 1; i < h_word; ++ i)
                A[i] = 0;
    }

    return 0;
}

static int set_bit_to_1(unsigned* A, unsigned b)
{
    unsigned b_bit, b_word;

    b_bit = b % 32;
    b_word = b / 32;

    A[b_word] |= (unsigned)(1 << b_bit);

    return 0;
}

static int get_bit_value(unsigned* A, unsigned b)
{
    unsigned b_bit, b_word;

    b_bit = b % 32;
    b_word = b / 32;

    if (A[b_word] & ((unsigned)(1 << b_bit)))
        return 1;
    else
        return 0;
}

/* This function is copied from the ZN ARW core module
 * implemented in the SPP
 *
 * Return values:
 * -1 - deffective window parameters
 *  0 - not replay
 *  1 - replay
 *  2 - outdated value (considered replay)
 *
 *  Refer to Zinc_ARW_core_v0.16.docx for more details
 */
inline int adk_ipsec_arw_check(
        unsigned long long x,
        uint16_t iSAIndex)
{
    unsigned long long B;
    unsigned W;
    unsigned* X;
    unsigned X_head;
    unsigned X_size;

    W = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_size;
    B = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_position;
    X = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_buffer;
    X_head = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_head;
    X_size = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_size;

    /* Checking window parameters
     */
    if (unlikely((W == 0) ||
            (X_size > W) ||
            ((X_size > 0) && (X_head >= W)))
       )
    {
        return ADK_E_IPSEC_ARW_INVALID_PARAMS;
    }

    if (unlikely(X_size == 0)) /* Empty window - initialization */
    {
        set_bit_to_1(X, 0);
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_position = x;
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_size = 1;
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_head = 0;
        return ADK_E_IPSEC_ARW_NO_REPLAY;
    }

    if ( x + (W - 1) < B) /* Outdated case */
    {
        /* result = 2;
         */
        return ADK_E_IPSEC_ARW_OUTDATED_VALUE;
    }
    else if (x == B)  /* Window upper bound - replay */
    {
        /* result = 1;
         */
        return ADK_E_IPSEC_ARW_REPLAY;
    }
    else if (x < B) /* Inside window */
    {
        unsigned s;
        s = (unsigned)(B - x);
        if (s >= X_size) /* Out of active window - expand window down */
        {
            return ADK_E_IPSEC_ARW_NO_REPLAY;
        }
        else /* Inside active window - do check */
        {
            int result;
            unsigned x_addr;
            x_addr = (X_head >= s) ? (X_head - s) : (X_head + (W - s));
            result = get_bit_value(X, x_addr);
            if (!result)
            {
                return ADK_E_IPSEC_ARW_NO_REPLAY;
            }
            else
            {
                return ADK_E_IPSEC_ARW_REPLAY;
            }
        }
    }
    else /* Ahead of window - move window up */
    {
            return ADK_E_IPSEC_ARW_NO_REPLAY;
    }

    return ADK_E_IPSEC_ARW_NO_REPLAY;
}

/* This function is copied from the ZN ARW core module
 * implemented in the SPP
 *
 * Return values:
 * -1 - deffective window parameters
 *  0 - not replay
 *  1 - replay
 *  2 - outdated value (considered replay)
 *
 *  Refer to Zinc_ARW_core_v0.16.docx for more details
 */
inline int adk_ipsec_arw_update(
        unsigned long long x,
        uint16_t iSAIndex)
{
    int result;

    // Short names for formulas
    unsigned long long B;
    unsigned W;
    unsigned* X;
    unsigned X_head;
    unsigned X_size;

    unsigned s;
    unsigned X_head_new;
    unsigned X_size_new;
    unsigned X_tail, X_tail_new;
    unsigned x_addr;

    W = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_size;
    B = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_position;
    X = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_buffer;
    X_head = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_head;
    X_size = adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_size;

    /* Checking window parameters
     */
    if (unlikely((W == 0) ||
            (X_size > W) ||
            ((X_size > 0) && (X_head >= W)))
       )
    {
        return ADK_E_IPSEC_ARW_INVALID_PARAMS;
    }

    if (x < B) /* Inside window */
    {
        s = (unsigned)(B - x);
        if (s >= X_size) /* Out of active window - expand window down */
        {
            result = 0;
            X_size_new = s + 1;
            X_tail =
                (X_head >= (X_size - 1)) ? (X_head - (X_size - 1)) : (X_head + (W - (X_size - 1)));
            X_tail_new =
                ((X_head >= (X_size_new - 1)) ? (X_head - (X_size_new - 1)) : (X_head + (W - (X_size_new - 1))));
            /* X_tail can not be equal X_tail_new:
             * it means X_tail = X_tail_new (mod W) which is controversive
             */
            if (X_tail > X_tail_new) /* no wrap, X_tail > X_tail_new >= 0 */
            {
                set_bit_range_to_0(X, X_tail_new, X_tail - 1);
            }
            else /* wrap */
            {
                if (X_tail > 0)
                {
                    set_bit_range_to_0(X, 0, X_tail - 1);
                }
                set_bit_range_to_0(X, X_tail_new, W - 1);
            }
            set_bit_to_1(X, X_tail_new);
            adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_size = X_size_new;
        }
        else /* Inside active window - do check */
        {
            x_addr = (X_head >= s) ? (X_head - s) : (X_head + (W - s));
            result = get_bit_value(X, x_addr);
            if (!result)
            {
                set_bit_to_1(X, x_addr);
            }
        }
    }
    else /* Ahead of window - move window up */
    {
        result = 0;
        if (x >= B + W) /* long jump - no drag */
        {
            X_size_new = 1;
            X_head_new = X_head;
            set_bit_to_1(X, X_head);
        }
        else /* shot jump - do drag */
        {
            s = (unsigned)(x - B);
            X_size_new = (X_size + s >= W) ? (W) : (X_size + s);
            if (X_head + s >= W) /* wrap */
            {
                X_head_new = X_head + s - W;
                if (X_head < W - 1)
                {
                    set_bit_range_to_0(X, X_head + 1, W - 1);
                }
                set_bit_range_to_0(X, 0, X_head_new);
            }
            else /* no wrap */
            {
                X_head_new = X_head + s;
                set_bit_range_to_0(X, X_head + 1, X_head_new);
            }
            set_bit_to_1(X, X_head_new);
        }
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].window_position = x;
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_size = X_size_new;
        adk_ipsec_dp_tbl_gbl_ptr->iSATbl[iSAIndex].buffer_head = X_head_new;
    }

    return result;
}

void adk_ipsec_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess)
{
    RTE_SET_USED(dpdkMbuf);
    /*Current worker core id */
    uint32_t coreIndex = ADK_LCORE_INDEX;
    debug_print("adk_IPsec_dp_pal_batch_cb_fn %d\n",cbEventType);
    switch(cbEventType)
    {
        case ADK_PAL_DP_BATCH_CB_EVENT_PH:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_PH_POLICER:
            if(cbParam != NULL)
            {
                adk_ipsec_ph_dp_cb_params_t  *cbInfo = cbParam;
                if(sendSuccess)
                {
                    if(ADK_IPSEC_INBOUND == cbInfo->direction)
                    {
                        INCREMENT_INGRESS_UTILITY_COUNTER(cbInfo->version, ipsec_ing_exception_sent);
                    }
                    else
                    {
                        INCREMENT_EGRESS_UTILITY_COUNTER(coreIndex, cbInfo->version, ipsec_eg_exception_sent);
                    }
                }
                else
                {
                    if(ADK_IPSEC_INBOUND == cbInfo->direction)
                    {
                        if(ADK_IPSEC_IPV4 == cbInfo->version)
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_exception_send_failure_drop;
                        else
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_exception_send_failure_drop;
                    }
                    else
                    {
                        if(ADK_IPSEC_IPV4 == cbInfo->version)
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].egressStats.ipsec_eg_global_exception_send_failure_drop;
                        else
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].egressStats.ipsec_eg_global_exception_send_failure_drop;
                    }
                    adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                }
                adk_ph_dev_pal_batch_handle_return(cbInfo->phDevId, cbInfo->phQueueId,
                        sendSuccess, cbEventType);
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO:
            if(cbParam != NULL)
            {
                adk_ipsec_ph_dp_cb_params_t  *cbInfo = cbParam;
                if(!sendSuccess)
                {
                    if(ADK_IPSEC_INBOUND == cbInfo->direction)
                    {
                        if(ADK_IPSEC_IPV4 == cbInfo->version)
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].ingressStats.ipsec_ing_global_la_send_to_crypto_failed;
                        else
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].ingressStats.ipsec_ing_global_la_send_to_crypto_failed;
                    }
                    else
                    {
                        if(ADK_IPSEC_IPV4 == cbInfo->version)
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV4[coreIndex].egressStats.ipsec_eg_global_la_send_to_crypto_failed;
                        else
                            ++adk_ipsec_dp_tbl_gbl_ptr->ipsecGblStatsV6[coreIndex].egressStats.ipsec_eg_global_la_send_to_crypto_failed;
                    }
                }
                adk_ph_dev_pal_batch_handle_return(cbInfo->phDevId, cbInfo->phQueueId,
                        sendSuccess, cbEventType);
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_NIC:
        case ADK_PAL_DP_BATCH_CB_EVENT_NETD:
        case ADK_PAL_DP_BATCH_CB_EVENT_TIMER:
        case ADK_PAL_DP_BATCH_CB_EVENT_EM:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER:
            break;
        default:
            break;
    }
    return;
}
