/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_ipsec_common.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief
 *
 * IPsec module's internal structure file.
 *
 ***********************************************************************/
#ifndef _ADK_IPSEC_COMMON_H_
#define _ADK_IPSEC_COMMON_H_

#include <adk_ipsec_utility_counters.h>
#include <adk_ipsec_export.h>

#define IPV4_ADDR_WIDTH   32
#define IPV6_ADDR_WIDTH   128

#define ACTIVE_BACKUP       2

#define ADK_IPSEC_MAX_NUM_SPD_RULES 1024
#define ADK_IPSEC_MAX_NUM_SPD_IN_RULES 1024 

#define ADK_IPSEC_SPI_TBL_HASH_WIDTH    12 /* review this? */
#define ADK_IPSEC_SPI_TBL_HASH_MAX      (1<<ADK_IPSEC_SPI_TBL_HASH_WIDTH)
#define ADK_IPSEC_SPI_TBL_HASH_MASK     (ADK_IPSEC_SPI_TBL_HASH_MAX - 1)
#define ADK_IPSEC_SPI_TBL_MAX_COLLISION 8

#define ADK_IPSEC_HWSA_MAP_HASH_WIDTH    12
#define ADK_IPSEC_HWSA_MAP_HASH_MAX      (1<<ADK_IPSEC_HWSA_MAP_HASH_WIDTH)
#define ADK_IPSEC_HWSA_MAP_HASH_MASK     (ADK_IPSEC_HWSA_MAP_HASH_MAX - 1)
#define ADK_IPSEC_HWSA_MAP_MAX_COLLISION 2

/* Keeping the same width for ADK_IPSEC_INITIAL_FRAG_INFO_TBL as like the IP reassembly
 * table width
 */
#define ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_WIDTH    14
#define ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_MAX      (1<<ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_WIDTH)
#define ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_MASK     (ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_MAX - 1)
#define ADK_IPSEC_SINGLE_SA_MAX_COLLISION             8
#define ADK_IPSEC_IPV6_INITIAL_FRAG_INFO_TBL_HASH_MAX 2048

#define INVALID_SA  0
#define VALID_SA  1

#define ESP_HDR_LEN           8

#define IPSEC_MAX_COLLISION   8
#define IPSEC_HASH_MAX        ADK_IPSEC_NUM_OF_SA

#define PROTO_TCP           0x06
#define PROTO_SCTP          0x84
#define PROTO_UDP           0x11
#define PROTO_ESP           0x32
#define PROTO_AH            0x33
#define PROTO_ICMP          0x01
#define PROTO_ICMPV6	    0x3A

#define MAX_SEQ_NUMBER 0xFFFFFFFF
/* Exception should be raised when sequence number reaches 
 * 75% of maximum sequence number value
 */
#define MAX_SEQ_NUMBER_EXCEPTION (MAX_SEQ_NUMBER * 75)/100
#define ADK_IPSEC_SHM_MEMZONE_NAME "adk_ipsec_shm_zone"
#define ADK_IPSEC_MAX_CRYPTO_DEV  32

#define ADK_IPSEC_IPTOS_ECN_CE  0x3
#define ADK_IPSEC_IPTOS_ECN_MASK  0xFF

#define CHK_ARW_WIN 0
#define UPD_ARW_WIN 1

#define IV_OFFSET   sizeof(struct rte_crypto_op) + sizeof(struct rte_crypto_sym_op)
#define ADK_IPSEC_NUM_OF_SESSIONS 2048

#define ADK_IPSEC_SA_INLINE     0x0
#define ADK_IPSEC_SA_LOOKASIDE  0x1
#define ADK_IPSEC_SA_BOTH       0x2

#define ADK_IPSEC_SPD_MAX_PRIORITY 0xFFFF

/* this needs to be fine tuned. It indicates
 * for how long we want to keep an SA alive even
 * when its "inFlightCnt" is not changing
 * This is in seconds!
 */
#define ADK_IPSEC_STALE_SA_TOLERANCE 5

/* This is merely an estimation of CPM packet
 * processing latency, in microseconds. Update
 * this if we get better estimates.
 * This is used to delay deletion of SA even when
 * inFlightCount has reached 0
 */
#define ADK_IPSEC_CPM_LATENCY 100

typedef struct adk_ipsec_spd_sel_s
{
    uint8_t   version;
    uint32_t  sip_addr[4];
    uint32_t  dip_addr[4];
    uint16_t  dscpVal;
    uint16_t  protocol;
    uint16_t  sport;
    uint16_t  dport;
} adk_ipsec_spd_sel_t __rte_cache_aligned;

typedef struct adk_ipsec_spd_in_sel_s
{
    uint8_t   version;
    uint32_t  sip_addr[4];
    uint32_t  dip_addr[4];
    uint32_t  protocol;
    uint16_t  sport;
    uint16_t  dport;
} adk_ipsec_spd_in_sel_t __rte_cache_aligned;


/*!
 * ===============================================================
 * @struct adk_ipsec_crypto_key_t "adk_ipsec_internal.h"
 *
 * @brief
 *
 * This data structure is a utility structure that defines the general
 * attributes of a key
 *
 * ===============================================================
 */
typedef struct adk_ipsec_crypto_key_s
{
    uint8_t *buf;
    uint32_t len;
    phys_addr_t phys_addr;
} adk_ipsec_crypto_key_t __rte_cache_aligned;

/*!
 * ==================================================================
 * @internal
 * @struct adk_ipsec_sa_info_t "adk_ipsec_internal.h"
 *
 * @brief
 *
 * Contains information on an SA granularity, inclusive of the
 * attributes needed to interface with the hardware. These
 * cryptodev specific attributes are not on a global scale and
 * are descriptive per-SA.
 *
 * ==================================================================
 */

/* this is a "spil-over" DS, primarily for holding
 * lookaside specific information, but also contains
 * those fields that can't fit in main SA DS and
 * require in DP processing in slow path
 */
typedef struct adk_ipsec_dp_lkaside_info_s
{
    uint8_t   cryptoDevId;
    uint8_t   cryptoQP;
    struct rte_cryptodev_sym_session *cryptoSession;
    struct rte_mempool               *cryptoPool;
    struct rte_crypto_op             *cryptoOp;
    rte_atomic32_t  seqNumber;
    uint16_t  NATTSrcPort;          
    uint16_t  NATTDstPort;
    uint32_t  NATTSrcAddr;
    uint32_t  NATTDstAddr;
}__attribute__ ((aligned))adk_ipsec_dp_lkaside_info_t;

typedef struct adk_ipsec_dp_eg_sa_s
{
    uint32_t    saSIP[4];
    uint32_t    saDIP[4];
    uint8_t     saMode:2;
    /* If ipVersion is 0, it is IPv4.
     * Else, it is IPv6.
     */
    uint8_t     ipVersion:1; 
    uint8_t     isNATTEnable:1;
    uint8_t     seqRollover:1;
    uint8_t     extSeqNum:1;
    uint8_t     lifetimesEnabled:1;
    uint8_t     reserved:1;
    struct rte_security_session *secSession;
    uint8_t     ivLen;
    uint8_t     *ivBuf;
    uint8_t     blockSize;
    uint8_t     digestLen;
    uint64_t    bytesUsed;
    uint64_t    seqNumber;
    uint8_t     ttl_hopLimit;
    uint8_t     encr;
    uint32_t    spi;
    adk_ipsec_dp_lkaside_info_t *lkasideInfo;
    uint16_t    inFlightCnt;
    uint32_t    egressStatsId;
    uint64_t    hardByteLifetime;
    uint64_t    softByteLifetime;
    uint32_t    flow_label_value;
} adk_ipsec_dp_eg_sa_t __rte_cache_aligned;

typedef struct adk_ipsec_dp_ing_sa_s
{
    uint16_t    iSAIndex;
    uint16_t    spIDEx;
    unsigned    window_size;
    unsigned    long long window_position;     /* window upper bit */
    unsigned    buffer_head;                   /* cyclic bufer lower bound */
    unsigned    buffer_size;                   /* current bufer size */
    unsigned    *window_buffer;                /* window bitmap memory */
    uint8_t     isValid:1;
    uint8_t     useSingleSaInPreFrag:1;
    uint8_t     antiReplayEnable:1;
    uint8_t     lifetimesEnabled:1;
    uint8_t     saMode:2;
    uint8_t     reserved:2;
    uint8_t     ivLen;
    uint8_t     digestLen;
    uint64_t    bytesUsed;
    uint8_t     dscpVal;
    uint32_t    hwSAId;
    adk_ipsec_dp_lkaside_info_t *lkasideInfo;
    uint32_t    spi;
    uint16_t    inFlightCnt;
    uint64_t    hardByteLifetime;
    uint64_t    softByteLifetime;
    adk_handle_t iSAHandle;
    uint8_t     policy_version;
    struct rte_security_session *secSession;
} adk_ipsec_dp_ing_sa_t __rte_cache_aligned;

typedef struct adk_sec_sec_pol_sainfo_s
{
    uint16_t  spIDEx;
    uint8_t   action:2;
    uint8_t   configDF:2;
    uint8_t   useSingleSAInPrefrag:1;
    uint8_t   preFragEnable:1;
    uint8_t   tunnelMode:1;
    uint8_t   espProto:1;
    uint8_t   eSAstate;
    uint16_t  eSAIdx;
    uint16_t  pathMTU;
    uint32_t  saNotFoundErrDrop;
} __attribute__ ((aligned)) adk_ipsec_sec_pol_sainfo_t;

typedef struct adk_ipsec_spi_hash_s
{
    bool      isValid;
    uint32_t  idx;
} __attribute__ ((aligned)) adk_ipsec_spi_hash_t;

struct esp_header
{
    uint32_t spi;
    uint32_t seqNum;
} __attribute__ ((packed));

typedef struct adk_ipsec_dp_exception_handler_s {
    uint8_t devId;
    uint16_t queueId;
    uint8_t cbFnId;
    uint32_t policerId;
    uint32_t statsId;
    uint8_t  priority;
    bool enableFlag;
} adk_ipsec_dp_exception_handler_t __rte_cache_aligned;

typedef struct adk_ipsec_dp_exception_data_s {
   adk_ipsec_exception_type_t exceptionType;
   union {
        adk_ipsec_sa_aquire_excep_data_t saAquireData;
        adk_ipsec_sa_expiry_excep_data_t saExpiryData;
        adk_ipsec_audit_event_t          auditEventData; 
   }exceptionData;
} adk_ipsec_dp_exception_data_t __rte_cache_aligned;

typedef struct adk_ipsec_crypto_dev_map_s
{
    bool      isValid;
    uint8_t   devId;
    uint16_t  numSAsAllowed;
    uint16_t  numSAsUsing;
    struct rte_mempool  *sessPool;
} __attribute__ ((aligned)) adk_ipsec_crypto_dev_map_t;

typedef struct adk_ipsec_intial_frag_buffered_pkts_info_s
{
    bool             initial_frag_visited;
    uint8_t          initial_frag_policy_version;
    uint16_t         intial_frag_SA_ID;
    uint16_t         initial_frag_SP;
    uint16_t         initial_frag_DP;
    adk_pktio_task_t *buffered_pkt_task_chain;
    uint8_t          numBufferedFrags;
} __attribute__ ((aligned)) adk_ipsec_initial_frag_buffered_pkts_info_t;

typedef struct adk_ipsec_single_sa_bucket_info_s
{
    /* In IP, total number of buckets is 8 for reassembly hash table to 
     * handle the collision. Same number of buckets is maintained in
     * IPsec hash table for Single SA use case when pre-frag is enabled.
     */
    adk_ipsec_initial_frag_buffered_pkts_info_t bucket[ADK_IPSEC_SINGLE_SA_MAX_COLLISION];
}adk_ipsec_single_sa_bucket_info_t __rte_cache_aligned;

typedef struct adk_ipsec_hw_sa_map_s
{
    uint32_t  hwSAId;
    uint16_t  adkSAId;
    bool      isValid;
} __attribute__ ((aligned)) adk_ipsec_hw_sa_map_t;

typedef struct adk_ipsec_dp_table_info_s
{
    uint32_t cookie;
    /* RTE ACL context for SPD OUT table */
    struct rte_acl_ctx          *spdTbl[ACTIVE_BACKUP];
    /* RTE ACL context for SPD IN table */
    struct rte_acl_ctx          *spdInTbl[ACTIVE_BACKUP];
    /* Total number of SPD OUT Policies */
    uint16_t                    numPolicies[ACTIVE_BACKUP];
    /* Total number of SPD IN Policies */
    uint16_t                    numInPolicies[ACTIVE_BACKUP];
    /* Active table index for SPD OUT table */
    uint8_t                     spdTblIdx;
    /* Active table index for SPD IN table */
    uint8_t                     spdInTblIdx;
    adk_ipsec_sec_pol_sainfo_t  polSAInfo[ADK_IPSEC_MAX_NUM_SPD_RULES];
    adk_ipsec_pol_action_t      spdInAction[ADK_IPSEC_MAX_NUM_SPD_IN_RULES];
    adk_ipsec_dp_eg_sa_t        eSATbl[ADK_IPSEC_NUM_OF_SA];
    adk_ipsec_eg_sa_stat_t      egSAStats[ADK_MAX_NUM_CORES][ADK_IPSEC_NUM_OF_SA];
    adk_ipsec_dp_ing_sa_t       iSATbl[ADK_IPSEC_NUM_OF_SA];
    adk_ipsec_ing_sa_stat_t     ingSAStats[ADK_MAX_NUM_CORES][ADK_IPSEC_NUM_OF_SA];
    adk_ipsec_spi_hash_t        spiHashTbl[ADK_IPSEC_SPI_TBL_HASH_MAX][ADK_IPSEC_SPI_TBL_MAX_COLLISION];
    adk_ipsec_hw_sa_map_t       hwSAIdMap[ADK_IPSEC_HWSA_MAP_HASH_MAX][ADK_IPSEC_HWSA_MAP_MAX_COLLISION];
    adk_ipsec_single_sa_bucket_info_t ipsecInitialFragInfoBufferedPktsHashTbl[ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_MAX];
    adk_ipsec_single_sa_bucket_info_t ipsecIPv6InitialFragInfoBufferedPktsHashTbl[ADK_IPSEC_IPV6_INITIAL_FRAG_INFO_TBL_HASH_MAX];
    /* this is only for devices that would get used for lookaside
     * processing
     */
    adk_ipsec_crypto_dev_map_t  cryptoDevMap[ADK_IPSEC_MAX_CRYPTO_DEV];
    /* there is only device that can process inline traffic!
     */
    struct rte_mempool          *inlineSessPool;
    /* Pointer to the security context and set of security operations for
     * the ETH device created in primary process
     */
    struct rte_security_ctx     *secCtxDP;
    adk_ipsec_dp_exception_handler_t *ipsecExceptionHandlerTble[ADK_IPSEC_EXCEPTION_MAX];
    adk_ipsec_stat_t            ipsecGblStatsV4[ADK_MAX_NUM_CORES] __rte_cache_aligned;
    adk_ipsec_stat_t            ipsecGblStatsV6[ADK_MAX_NUM_CORES] __rte_cache_aligned;
    uint32_t                    workerCoreList[ADK_MAX_NUM_CORES] __rte_cache_aligned;
    uint32_t                    numCores;
    /* Pre-Frag Buffering Timeout value used if inner packets are forwarded fragments */
    uint32_t                    preFragBufferTimeOutVal;
    bool                        isUtilityCntEnabled;
    adk_ipsec_utility_counters_t            ipsecUtilityCntV4[ADK_MAX_NUM_CORES] __rte_cache_aligned;
    adk_ipsec_utility_counters_t            ipsecUtilityCntV6[ADK_MAX_NUM_CORES] __rte_cache_aligned;
    adk_ipsec_sa_utility_counters_t         SAUtilityCnt[ADK_MAX_NUM_CORES][ADK_IPSEC_NUM_OF_SA] __rte_cache_aligned;
    /* This will hold the information required by the batch mode callback function to increment counter */
    adk_ipsec_ph_dp_cb_params_t             cbParams[ADK_MAX_NUM_CORES][ADK_PAL_MAX_ENQUEUE_SIZE];
    /* This will indicate which cbParams index should be used for storing callback params */
    uint8_t                     cbParamIndex;
} adk_ipsec_dp_table_info_t __rte_cache_aligned;

#endif
