/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_ipsec_cp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief
 *
 * IPsec module's internal structure file.
 *
 ***********************************************************************/
#ifndef _ADK_IPSEC_CP_H_
#define _ADK_IPSEC_CP_H_

#include "adk_cp_acl_list.h"
#include "adk_hos.h"

#define ADK_IPSEC_LOG_LEVEL_GET(mod_handle)  \
    (((ipsec_module_info_t *)(ADK_MODULE_INFO_PTR_GET(mod_handle)))->logLevel)

#define ADK_IPSEC_LOG_FATAL(mod_handle, err, fmt, args...)\
    do {\
        if ((ADK_IPSEC_LOG_LEVEL_GET(mod_handle)\
                    & ADK_LOG_LEVEL_FATAL) == ADK_LOG_LEVEL_FATAL) {\
            ADK_ERROR_WRITE("[ADK_IPSEC_FATAL: %s:%d] %s ",\
                    __FILE__, __LINE__,\
                    adk_sys_status_get(\
                        ADK_MODULE_SYS_HANDLE_GET (mod_handle), err)); \
            ADK_ERROR_WRITE(fmt, ##args);   \
        }\
    }while(0);

#define ADK_IPSEC_LOG_ERROR(mod_handle, err, fmt, args...) \
    do {\
        if ((ADK_IPSEC_LOG_LEVEL_GET(mod_handle)\
                    & ADK_LOG_LEVEL_ERROR) == ADK_LOG_LEVEL_ERROR) { \
            ADK_ERROR_WRITE("[ADK_IPSEC_ERROR: %s:%d] %s ", __FILE__, __LINE__, \
                    adk_sys_status_get(\
                        ADK_MODULE_SYS_HANDLE_GET (mod_handle), err)); \
            ADK_ERROR_WRITE(fmt, ##args);   \
        }\
    }while(0);

#define ADK_IPSEC_LOG_WARNING(mod_handle, fmt, args...) \
    do {\
        if ((ADK_IPSEC_LOG_LEVEL_GET(mod_handle)\
                    & ADK_LOG_LEVEL_WARNING) == ADK_LOG_LEVEL_WARNING) { \
            ADK_ERROR_WRITE("[ADK_IPSEC_WARNING: %s:%d] ", __FILE__, __LINE__); \
            ADK_ERROR_WRITE(fmt, ##args);   \
        }\
    }while(0);

#define ADK_IPSEC_LOG_DEBUG(mod_handle, fmt, args...) \
    do {\
        if ((ADK_IPSEC_LOG_LEVEL_GET(mod_handle)\
                    & ADK_LOG_LEVEL_DEBUG) == ADK_LOG_LEVEL_DEBUG) { \
            ADK_ERROR_WRITE("[ADK_IPSEC_DEBUG %s:%d] ", __FILE__, __LINE__); \
            ADK_ERROR_WRITE(fmt, ##args);   \
        }\
    }while(0);

#define ADK_IPSEC_LOG_INFO(mod_handle, fmt, args...) \
    do {\
        if ((ADK_IPSEC_LOG_LEVEL_GET(mod_handle)\
                    & ADK_LOG_LEVEL_INFO) == ADK_LOG_LEVEL_INFO) { \
            ADK_ERROR_WRITE("[ADK_IPSEC_INFO %s:%d] ", __FILE__, __LINE__); \
            ADK_ERROR_WRITE(fmt, ##args);   \
        }\
    }while(0);

#define ADK_IPSEC_LOG_FORCE(fmt, args...) \
    ADK_FORCE_LOG_ERROR(fmt, ##args);

#define ADK_IPSEC_LOG_FUNC_ENTRY(mod_handle) \
        ADK_LOG_FUNC_ENTRY(sysHandle, mod_handle)

#define ADK_IPSEC_LOG_FUNC_EXIT(mod_handle) \
        ADK_LOG_FUNC_EXIT(sysHandle, mod_handle)

#define ADK_RETURN_ERROR_SET(error_type) \
      ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);  \
      return ADK_ERROR_SEV_LEVEL_SET(error_type)

#define ADK_IPSEC_NON_ZERO_CNTR_WRITE(fmt, cntr) \
    if (cntr != 0) { \
        ADK_INFO_WRITE(fmt, cntr) \
    }

#define mVerifyArguments(ipsecModuleInfo_p,ipsecHandle,_ret)\
    {\
        if ( ipsecModuleInfo_p  == NULL )\
        {\
            ADK_LOG_ERROR(NULL, ipsecHandle, \
                    "IPSEC: ipsecModuleInfo pointer is invalid\n");\
            ADK_RELEASE_MODULE_LOCK(ipsecHandle, (_ret));\
            ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_MODULE_HANDLE);\
        }\
        if(ipsecModuleInfo_p->cookie != ADK_IPSEC_MODULE_COOKIE)\
        {\
            ADK_LOG_ERROR(NULL, ipsecHandle, \
                    "IPSEC: module handle cookie check failed\n");\
            ADK_RELEASE_MODULE_LOCK(ipsecHandle, (_ret));\
            ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_MODULE_HANDLE);\
        }\
    }

#define mFindIdx(cur, new, old)\
    {\
        old = cur;\
        if (cur == 0)\
            new = 1;\
        else new = 0;\
    }

#define LOGIPsecModuleInfo(x,y)\
    do {\
        ADK_IPSEC_LOG_INFO(ipsecHandle, "%s sysHandle: %p, ipsecHandle: %p,\n\
                policyHdl: %p, policyInHdl: %p\n\
                iSASetHdl: %p, eSASetHdl: %p\n\
                cpSpdAclList: %p, cpSpdInAclList: %p\n\
                numSpdRules: 0x%x, numSpdInRules: 0x%x\n\
                dpTbl: %p, numStaleEntries = %u\n\
                numCryptoDevices: %u\n",\
                y, (adk_sys_handle_t *)sysHandle, (adk_module_handle_t *)ipsecHandle,\
                x->policyHdl, x->policyInHdl,\
                x->iSASetHdl, x->eSASetHdl, x->cpSpdAclList,\
                x->cpSpdInAclList, x->numSpdRules, x->numSpdInRules,\
                x->dpTbl, x->numStaleEntries, x->numCryptoDevices);\
    }while(0);

#define LOGSecPolAttr(x,y)\
    do {\
        ADK_IPSEC_LOG_INFO(ipsecHandle, "%s action: 0x%x,\n\
                configDF: 0x%x, preFragEnable: 0x%x,\n\
                tunnelMode: 0x%x, espProto: 0x%x\n\
                useSingleSAInPrefrag: 0x%x, pathMTU: 0x%x, saNotFoundErrDrop: 0x%x,\n\
                spIDEx: 0x%x\n",\
                y,x.action, x.configDF, x.preFragEnable,\
                x.tunnelMode, x.espProto, x.useSingleSAInPrefrag,\
                x.pathMTU, x.saNotFoundErrDrop, x.spIDEx);\
    }while(0);

#define LOGSecPolUpdateAttr(x,y)\
    do {\
        ADK_IPSEC_LOG_INFO(ipsecHandle, "%s action: 0x%x,\n\
                configDF: 0x%x, preFragEnable: 0x%x,\n\
                tunnelMode: 0x%x, espProto: 0x%x\n\
                useSingleSAInPrefrag: 0x%x, pathMTU: 0x%x, saNotFoundErrDrop: 0x%x,\n\
                spIDEx: 0x%x\n",\
                y,x->action, x->configDF, x->preFragEnable,\
                x->tunnelMode, x->espProto, x->useSingleSAInPrefrag,\
                x->pathMTU, x->saNotFoundErrDrop, x->spIDEx);\
    }while(0);

#define LOGSecAsc(x,y)\
    do {\
        if(ADK_IPSEC_IPV4 == x->ipVersion) {\
            ADK_IPSEC_LOG_INFO(ipsecHandle, "%s secAsc_p->tunnelSrcIP: 0x%x,tunnelDstIP: 0x%x,\n\
                    SASPPContext.direction: 0x%x, mode: 0x%x, protocol: 0x%x, spi: 0x%x,\n\
                    isNATTEnable: 0x%x, NATTSrcPort: 0x%x\n",\
                    y,x->tunAddr.ipv4TunAddr.tunnelSrcIP, x->tunAddr.ipv4TunAddr.tunnelDstIP,\
                    x->SASPPContext.direction, x->SASPPContext.mode,\
                    x->SASPPContext.protocol, x->SASPPContext.spi,\
                    x->isNATTEnable, x->NATTContext.NATTSrcPort);\
        }\
        else if(ADK_IPSEC_IPV6 == x->ipVersion) {\
            ADK_IPSEC_LOG_INFO(ipsecHandle, "%s secAsc_p->tunnelSrcIP: 0x%x%x%x%x,tunnelDstIP: 0x%x%x%x%x,\n\
                    SASPPContext.direction: 0x%x, mode: 0x%x, protocol: 0x%x, spi: 0x%x,\n\
                    isNATTEnable: 0x%x, NATTSrcPort: 0x%x\n",\
                    y,x->tunAddr.ipv6TunAddr.tunnelSrcIP[0], x->tunAddr.ipv6TunAddr.tunnelSrcIP[1],\
                    x->tunAddr.ipv6TunAddr.tunnelSrcIP[2], x->tunAddr.ipv6TunAddr.tunnelSrcIP[3], \
                    x->tunAddr.ipv6TunAddr.tunnelDstIP[0], x->tunAddr.ipv6TunAddr.tunnelDstIP[1],\
                    x->tunAddr.ipv6TunAddr.tunnelDstIP[2], x->tunAddr.ipv6TunAddr.tunnelDstIP[3], \
                    x->SASPPContext.direction, x->SASPPContext.mode,\
                    x->SASPPContext.protocol, x->SASPPContext.spi,\
                    x->isNATTEnable, x->NATTContext.NATTSrcPort);\
        }\
    }while(0);

#define LOGSecAscUpdate(x,y)\
    do {\
        ADK_IPSEC_LOG_INFO(ipsecHandle, "%s SASPPContext.direction: 0x%x, mode: 0x%x, protocol: 0x%x, spi: 0x%x\n",\
                y,x->SASPPContext.direction, x->SASPPContext.mode,\
                x->SASPPContext.protocol, x->SASPPContext.spi);\
    }while(0);

#define ADK_IPSEC_BUILD_CP_DP_POL 0x01
#define ADK_IPSEC_BUILD_DP_POL    0x02

/* This is just to ensure that the same precedence value
 * is not assigned to the policies across multiple rulesets
 * This is required because HLP maintains a flat space for
 * policies across rulesets and the enable/disable of
 * policy works on precedence values!
 */
#define ADK_IPSEC_POLICY_GAP 100

/* Total of 2000 SAs supported, leaving plenty of room in 16-bit
 * SA index. Using the MSB of the 16-bit value for direction.
 * For outbound, bit 15 is 0, and for inbound, bit 15 is 1.
 */
#define ADK_IPSEC_OUTBOUND_SA_MASK  0x0
#define ADK_IPSEC_INBOUND_SA_MASK 0x8000
#define ADK_IPSEC_SA_DIR_SHIFT  15
#define ADK_IPSEC_DELETED_SA_MASK   0x4000

#define ADK_IPSEC_MAXIMUM_IV_LENGTH 16

#define ADK_IPSEC_PREFRAG_BUFFER_TIMEOUT_MILLISECONDS 100

/* Obj Types for dynamic memory */
typedef enum adk_ipsec_obj_s {
    ADK_IPSEC_MOD_INFO_OBJ,
    ADK_IPSEC_SPD_HOS_RULE_OBJ,
    ADK_IPSEC_SPD_RULESET_OBJ,
    ADK_IPSEC_SPD_IN_RULESET_OBJ,
    ADK_IPSEC_RULES_PER_SET_OBJ,
    ADK_IPSEC_IN_RULES_PER_SET_OBJ,
    ADK_IPSEC_ING_NUM_SA_OBJ,
    ADK_IPSEC_EG_NUM_SA_OBJ,
    ADK_IPSEC_ING_FLOW_OBJ,
    ADK_IPSEC_EG_FLOW_OBJ,
    ADK_IPSEC_CRYPTO_INFO_OBJ,
    ADK_IPSEC_QPAIR_INFO_OBJ,
    ADK_IPSEC_SP_OBJ,
    ADK_IPSEC_SP_IN_OBJ,
    ADK_IPSEC_SA_ID_OBJ,
    ADK_IPSEC_TUN_FLOW_OBJ,
    ADK_IPSEC_FLOW_INFO_OBJ,
    ADK_IPSEC_EXC_OBJ,
    ADK_IPSEC_KEY_OBJ,
    ADK_IPSEC_ARW_OBJ,
    ADK_IPSEC_HOS_RANGE_MASK_OBJ,
    ADK_IPSEC_HOS_RULE_OBJ,
    ADK_IPSEC_DP_TBL_OBJ
} adk_ipsec_obj_t;

enum
{
    VERSION_FIELD_IPSEC,
    SRC_IP_FIELD_IPSEC_IPV4_IPV6_0,
    SRC_IP_FIELD_IPSEC_IPV4_IPV6_1,
    SRC_IP_FIELD_IPSEC_IPV4_IPV6_2,
    SRC_IP_FIELD_IPSEC_IPV4_IPV6_3,
    DST_IP_FIELD_IPSEC_IPV4_IPV6_0,
    DST_IP_FIELD_IPSEC_IPV4_IPV6_1,
    DST_IP_FIELD_IPSEC_IPV4_IPV6_2,
    DST_IP_FIELD_IPSEC_IPV4_IPV6_3,
    DSCP_FIELD_IPSEC,
    PROTO_FIELD_IPSEC,
    SP_FIELD_IPSEC,
    DP_FIELD_IPSEC,
    NUM_FIELDS_IPSEC
};

/* Lookup Fields defined for SPD IN ACL */
enum
{
    VERSION_FIELD_IPSEC_IN,
    SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0,
    SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1,
    SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2,
    SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3,
    DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0,
    DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1,
    DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2,
    DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3,
    PROTO_FIELD_IPSEC_IN,
    SP_FIELD_IPSEC_IN,
    DP_FIELD_IPSEC_IN,
    NUM_FIELDS_IPSEC_IN
};

enum
{
    VERSION_INPUT_IDX_IPSEC,
    SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_0,
    SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_1,
    SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_2,
    SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_3,
    DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_0,
    DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_1,
    DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_2,
    DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_3,
    DSCP_PROTO_INPUT_IDX_IPSEC,
    SP_DP_INPUT_IDX_IPSEC
};

enum
{
    VERSION_INPUT_IDX_IPSEC_IN,
    SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_0,
    SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_1,
    SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_2,
    SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_3,
    DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_0,
    DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_1,
    DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_2,
    DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_3,
    PROTO_INPUT_IDX_IPSEC_IN,
    SP_DP_INPUT_IDX_IPSEC_IN
};

typedef enum adk_ipsec_sa_dir_s
{
    ADK_IPSEC_SA_UL,
    ADK_IPSEC_SA_DL
} adk_ipsec_sa_dir;

/* Flow Cache Version status */
typedef enum adk_ipsec_fc_ver_status_s
{
    ADK_IPSEC_FC_VER_UPDATED,
    ADK_IPSEC_FC_VER_TO_BE_UPDATED
} adk_ipsec_fc_ver_status_t;

/*!
 * =============================================================
 * @internal
 * @struct ipsec_module_info_t "adk_ipsec_internal.h"
 *
 * @brief
 *
 * global state information for the IPsec module. state information
 * for IPsec module's objects are stored in this data tsructure.
 * =============================================================
 */
typedef struct ipsec_module_info_s
{
    uint32_t    cookie;

    adk_rm_hdl_t    policyHdl;
    adk_rm_hdl_t    policyInHdl;
    adk_rm_hdl_t    iSASetHdl;
    adk_rm_hdl_t    eSASetHdl;

    /* CP last list */
    adk_acl_handle_t cpSpdAclList;
    uint32_t numSpdRules;
    /* CP last list */
    adk_acl_handle_t cpSpdInAclList;
    uint32_t numSpdInRules;

    /* data path tables & structures */
    adk_ipsec_dp_table_info_t *dpTbl;

    /* Keeping track of stale entries for subsequent deletion */
    uint16_t   staleSaEntries[ADK_IPSEC_NUM_OF_SA];
    uint16_t   numStaleEntries;

    /* crypto dev info, only for lookaside */
    uint8_t   numCryptoDevices;
    adk_sys_rsrc_crypto_dev_info_t *cryptoDev[ADK_IPSEC_MAX_CRYPTO_DEV];
    /* Pointer to the security context and set of security operations for
     * the ETH device created in secondary process
     */
    struct rte_security_ctx     *secCtxCP;
    uint8_t                     logLevel;
    /* By default, ipsec_fc_ver_status would be in ADK_IPSEC_FC_VER_UPDATED.
     * When application deletes a rule, it would be changed to 
     * ADK_IPSEC_FC_VER_TO_BE_UPDATED.
     */
    adk_ipsec_fc_ver_status_t   ipsec_fc_ver_status;
    /* total number of ingress SAs */
    uint16_t   numIngSAEntries;
    /* total number of egress SAs */
    uint16_t   numEgSAEntries;
    /* Egress SA config */
    adk_ipsec_sa_t eSAConfig[ADK_IPSEC_NUM_OF_SA];
    /* Ingress SA config */
    adk_ipsec_sa_t iSAConfig[ADK_IPSEC_NUM_OF_SA];
} ipsec_module_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ipsec_spd_rule_set_info_t adk_ipsec_internal.h "adk_ipsec_internal.h"
 *
 * @brief
 *
 * SPD ruleset id info.
 * =============================================================
 */
typedef struct ipsec_spd_rule_set_idx_s
{
    uint32_t index;
} ipsec_spd_rule_set_idx_t;

/*!
 * =============================================================
 * @internal
 * @struct ipsec_spd_pol_info_t adk_ipsec_internal.h "adk_ipsec_internal.h"
 *
 * =============================================================
 */
typedef struct ipsec_spd_pol_info_s
{
    uint16_t setId;
    uint16_t index;
} ipsec_spd_pol_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ipsec_sa_idx_t adk_ipsec_internal.h "adk_ipsec_internal.h"
 *
 * =============================================================
 */
typedef struct ipsec_sa_idx_s
{
    uint32_t index;
} ipsec_sa_idx_t;
/*!
 * =============================================================
 * @internal
 * @struct ipsec_tunnel_flow_idx_t adk_ipsec_internal.h "adk_ipsec_internal.h"
 *
 * =============================================================
 */
typedef struct ipsec_tunnel_flow_idx_s
{
    uint32_t index;
} ipsec_tunnel_flow_idx_t;

/* Structures from adk_ipsec_spd.h */
typedef struct _adk_ipsec_spd_rule_s
{
    uint8_t   version;
    uint8_t   proto_Begin;
    uint8_t   proto_End;
    uint32_t  srcAddr_Begin[4];
    uint32_t  srcAddr_End[4];
    uint32_t  dstAddr_Begin[4];
    uint32_t  dstAddr_End[4];
    uint16_t  srcPort_Begin;
    uint16_t  srcPort_End;
    uint16_t  dstPort_Begin;
    uint16_t  dstPort_End;
    uint8_t   dscpVal_Begin;
    uint8_t   dscpVal_End;
} adk_ipsec_spd_rule_t;

typedef struct _adk_ipsec_spd_in_rule_s
{
    uint8_t   version;
    uint8_t   proto_Begin;
    uint8_t   proto_End;
    uint32_t  srcAddr_Begin[4];
    uint32_t  srcAddr_End[4];
    uint32_t  dstAddr_Begin[4];
    uint32_t  dstAddr_End[4];
    uint16_t  srcPort_Begin;
    uint16_t  srcPort_End;
    uint16_t  dstPort_Begin;
    uint16_t  dstPort_End;
} adk_ipsec_spd_in_rule_t;


typedef adk_flow_handle_t adk_ipsec_spd_val_t;

typedef void adk_ipsec_spd_t;

typedef struct adk_ipsec_ipv4_address_info_s
{
    uint32_t  addr;
    uint32_t  mask;
}adk_ipsec_ipv4_address_info_t;

typedef struct adk_ipsec_ipv6_address_info_s
{
    uint32_t  addr[4];
    uint32_t  mask[4];
}adk_ipsec_ipv6_address_info_t;

RTE_ACL_RULE_DEF(adk_ipsec_acl_rule, NUM_FIELDS_IPSEC);
RTE_ACL_RULE_DEF(adk_ipsec_in_acl_rule, NUM_FIELDS_IPSEC_IN);

/*!
 * ================================================================
 * @struct ipsec_sec_pol_info_s "adk_ipsec_cp.h" 
 *
 * @brief
 *
 * The internal data structure defines security policy selector and
 * it's attributes for SDP OUT
 * 
 *
 * ================================================================
 */
typedef struct ipsec_sec_pol_info_s {
    uint32_t policyResourceID;
    struct adk_ipsec_acl_rule dpdkAclRule;
    adk_acl_rule_t cpRule;
} ipsec_sec_pol_info_t; /* end of struct adk_ipsec_sec_pol_t */

/*!
 * ================================================================
 * @struct ipsec_sec_pol_in_info_s "adk_ipsec_cp.h" 
 *
 * @brief
 *
 * The internal data structure defines security policy selector and
 * it's attributes for SDP IN
 * 
 *
 * ================================================================
 */
typedef struct ipsec_sec_pol_in_info_s {
    uint32_t policyResourceID;
    struct adk_ipsec_in_acl_rule dpdkAclInRule;
    adk_acl_rule_t cpRule;
} ipsec_sec_pol_in_info_t; /* end of struct adk_ipsec_sec_pol_t */

adk_ipsec_spd_t *adk_ipsec_spd_create (adk_sys_handle_t sysHandle,
                                       adk_module_handle_t ipsecHandle,
                                       uint8_t tblIdx);

adk_ipsec_spd_t *adk_ipsec_spd_in_create (adk_sys_handle_t sysHandle,
                                          adk_module_handle_t ipsecHandle,
                                          uint8_t tblIdx);
adk_status_t
ipsec_spd_convert_to_hos_acls(adk_module_handle_t ipsecHandle,
                        adk_ipsec_sec_pol_t *secPol_p,
                        adk_ipsec_spd_rule_t *spdRule_p,
                        uint32_t polHdlIdx,
                        uint16_t *num_hos_rules_p,
                        adk_hos_spd_tbl_rule_attr_t **rule_attr);
void
ipsec_spd_in_policy_ptn_create(adk_ipsec_spd_in_rule_t *ptn,
                                           adk_ipsec_sec_pol_in_sel_t *secPolSel);
adk_status_t
ipsec_spd_in_convert_to_hos_acls(adk_module_handle_t ipsecHandle,
                        adk_ipsec_sec_pol_in_t *secPol_p,
                        adk_ipsec_spd_in_rule_t *spdRule_p,
                        uint32_t polHdlIdx,
                        uint16_t *num_hos_rules_p,
                        adk_hos_spd_tbl_rule_attr_t **rule_attr);
void
ipsec_spd_convert_to_dpdk_acl (adk_module_handle_t ipsecHandle,
                        adk_ipsec_spd_rule_t *spdRulePtn,
                        uint32_t policyHdl,
                        struct adk_ipsec_acl_rule *dpdkAclRule);
void
ipsec_spd_in_convert_to_dpdk_acl (adk_module_handle_t ipsecHandle,
                        adk_ipsec_spd_in_rule_t *spdRulePtn,
                        uint32_t policyHdl,
                        struct adk_ipsec_in_acl_rule *dpdkAclRule);
adk_status_t
adk_ipsec_get_cp_spd_rule_idx(ipsec_module_info_t *ipsecModuleInfo_p,
                              adk_ipsec_sec_pol_sel_lkup_t *secPolSelLkup_p,
                              uint16_t numOfRules, uint16_t *polIdx,
                              ipsec_sec_pol_info_t **ipsecSecPol);

adk_status_t
adk_ipsec_get_cp_spd_in_rule_idx(ipsec_module_info_t *ipsecModuleInfo_p,
                                 adk_ipsec_sec_pol_sel_lkup_t *secPolSelLkup_p,
                                 uint16_t numOfRules, uint16_t *polIdx,
                                 ipsec_sec_pol_in_info_t **ipsecSecPol);

void adk_ipsec_get_spd_rule(adk_ipsec_sec_pol_sel_t *secPolSel,
                            struct adk_ipsec_acl_rule *dpdkAclRule);

void adk_ipsec_get_spd_in_rule(adk_ipsec_sec_pol_in_sel_t *secPolSel,
                               struct adk_ipsec_in_acl_rule *dpdkAclRule);

void ipsec_policy_ptn_create(adk_ipsec_spd_rule_t *ptn,
                             adk_ipsec_sec_pol_sel_t *pol);
adk_ipsec_return_code_t
adk_ipsec_spd_rule_add (adk_sys_handle_t  sysHandle,
                        adk_module_handle_t    ipsecHandle,
                        ipsec_module_info_t *ipsecModuleInfo_p,
                        adk_ipsec_sec_pol_t *secPol_p,
                        uint32_t policyHdl,
                        uint32_t polIndex,
                        bool polByIndex);

adk_ipsec_return_code_t
adk_ipsec_spd_in_rule_add (adk_sys_handle_t  sysHandle,
                           adk_module_handle_t    ipsecHandle,
                           ipsec_module_info_t *ipsecModuleInfo_p,
                           adk_ipsec_sec_pol_in_t *secPol_p,
                           uint32_t policyHdl,
                           uint16_t polIndex,
                           bool polByIndex);

void adk_ipsec_pre_spd_commit (adk_sys_handle_t sysHandle,
                          adk_module_handle_t ipsecHandle,
                          ipsec_module_info_t *ipsecModuleInfo_p,
                          uint32_t setId);

int adk_ipsec_spd_commit (adk_sys_handle_t sysHandle,
                          adk_module_handle_t ipsecHandle,
                          ipsec_module_info_t *ipsecModuleInfo_p);

int adk_ipsec_spd_in_commit (adk_sys_handle_t sysHandle,
                             adk_module_handle_t ipsecHandle,
                             ipsec_module_info_t *ipsecModuleInfo_p);

adk_status_t adk_ipsec_add_eSA(adk_sys_handle_t sysHandle,
                               adk_module_handle_t ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               uint32_t *saIndex,
                               adk_ipsec_sa_t *saConfig,
                               uint8_t  dscpVal);

adk_status_t adk_ipsec_update_eSA(ipsec_module_info_t *,
                                  adk_ipsec_upd_sa_t *newInfo,
                                  adk_handle_t saIdx,
                                  adk_interface_handle_t ipTunIf,
                                  uint8_t dscpVal);

adk_status_t adk_ipsec_stop_eSA(adk_module_handle_t ipsecHandle,
                        ipsec_module_info_t *ipsecModuleInfo_p,
                        uint32_t            saIdx,
                        uint8_t             numPol,
                        uint16_t            startPolIndex);

adk_status_t adk_ipsec_del_eSA(adk_sys_handle_t sysHandle,
                               adk_module_handle_t ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               uint32_t            saIdx);

adk_status_t
adk_ipsec_add_iSA(adk_sys_handle_t sysHandle,
                  adk_module_handle_t ipsecHandle,
                  ipsec_module_info_t *ipsecModuleInfo_p,
                  uint32_t *saIndex,
                  adk_ipsec_sa_t *saConfig,
                  uint8_t dscpVal);

void adk_ipsec_invalidate_iSA(adk_sys_handle_t sysHandle,
                        adk_module_handle_t ipsecHandle,
                        ipsec_module_info_t *ipsecModuleInfo_p,
                        uint32_t            saIdx);

adk_status_t  adk_ipsec_del_iSA(adk_sys_handle_t    sysHandle,
                        adk_module_handle_t ipsecHandle,
                        ipsec_module_info_t *ipsecModuleInfo_p,
                        uint32_t            saIdx);

adk_status_t adk_ipsec_spd_cp_create (adk_module_handle_t ipsecHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p);
adk_status_t adk_ipsec_spd_cp_delete (adk_module_handle_t ipsecHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p);

adk_status_t adk_ipsec_dp_table_init (adk_module_handle_t ipsecHandle,
                                      adk_sys_handle_t sysHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p);

adk_status_t adk_ipsec_dp_table_shutdown (adk_module_handle_t ipsecHandle,
                                          adk_sys_handle_t sysHandle,
                                          ipsec_module_info_t *ipsecModuleInfo_p);

void adk_ipsec_ipsec_policy_ptn_create(adk_ipsec_spd_rule_t *ptn,
                                       adk_ipsec_sec_pol_t *pol);

adk_status_t adk_ipsec_spd_rule_del(adk_sys_handle_t sysHandle,
                                    adk_module_handle_t ipsecHandle,
                                    ipsec_module_info_t *ipsecModuleInfo_p,
                                    uint32_t idx);

adk_status_t adk_ipsec_spd_in_rule_del(adk_sys_handle_t sysHandle,
                                       adk_module_handle_t ipsecHandle,
                                       ipsec_module_info_t *ipsecModuleInfo_p,
                                       uint16_t idx);

adk_status_t adk_ipsec_dp_handle_exception (adk_pktio_task_t *ph_task,
                                            uint8_t type, uint8_t direction,
                                            uint32_t polIndex, bool isHardLifeExpiry,
                                            uint8_t version, uint32_t spi);

adk_status_t adk_ipsec_crypto_dev_init(adk_sys_handle_t sysHandle,
                                       adk_module_handle_t ipsecHandle,
                                       ipsec_module_info_t *ipsecModuleInfo_p);

void adk_ipsec_dp_policy_attributes (adk_ipsec_sec_pol_sainfo_t *dpPolAttr_p,
                                     adk_ipsec_sec_pol_attr_t *secPolAttr_p,
                                     bool update);
adk_ipsec_return_code_t
adk_ipsec_verify_add_sp_input_args(adk_module_handle_t ipsecHandle,
                                   ipsec_module_info_t *ipsecModuleInfo_p,
                                   adk_ipsec_sec_pol_t  *secPol_p);

adk_ipsec_return_code_t
adk_ipsec_verify_add_sp_in_input_args(adk_module_handle_t ipsecHandle,
                                      adk_ipsec_sec_pol_in_t *secPol_p);

adk_ipsec_return_code_t
adk_ipsec_verify_upd_sp_input_args(adk_module_handle_t ipsecHandle,
                                   adk_ipsec_sec_pol_attr_t  *secPolAttr_p);

adk_ipsec_return_code_t
adk_ipsec_verify_add_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_sa_t		*secAsc_p,
                               adk_handle_t         *saNsHandle,
                               uint8_t              numPol,
                               uint16_t             startPolIndex);

adk_ipsec_return_code_t
adk_ipsec_verify_upd_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_upd_sa_t  *secAsc_p,
                               adk_handle_t             saNsHandle,
                               uint8_t              numPol,
                               uint16_t             startPolIndex);

adk_ipsec_return_code_t
adk_ipsec_verify_del_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_sa_direction_t	saDir,
                               uint8_t              numPol,
                               uint16_t             startPolIndex,
                               adk_handle_t         saNsHandle);

adk_ipsec_return_code_t
adk_ipsec_verify_get_sa_input_args(adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_handle_t         saNsHandle,
                               adk_ipsec_sa_direction_t saDirection);

adk_status_t
adk_ipsec_rsrc_notify_fn_cbfn (adk_module_handle_t ipsecHandle,
                               adk_sys_rsrc_oper_t oper,
                               adk_sys_resource_type_t type,
                               adk_sys_resource_info_t rsrc);

adk_status_t adk_ipsec_delete_stale_entries (adk_sys_handle_t sysHandle,
                                     adk_module_handle_t ipsecHandle,
                                     ipsec_module_info_t *ipsecModuleInfo_p);

void adk_ipsec_reset_unused_audit_fields (adk_ipsec_audit_event_t *auditEvent_p);

void adk_ipsec_extract_expiry_exception_data (adk_pktio_task_t *task,
                                              adk_ipsec_sa_expiry_excep_data_t *saExpiryData);

void adk_ipsec_extract_headers_sa_message (adk_pktio_task_t *task,
                                           adk_ipsec_sa_msg_t *saMessage_p);

void adk_ipsec_fill_encrypted_pkt_audit_info (adk_pktio_task_t *task,
                                              adk_ipsec_audit_event_encr_pkt_data_t *auditEventData_p);

void adk_ipsec_fill_before_encr_pkt_audit_info(adk_pktio_task_t *task,
                                               adk_ipsec_audit_event_before_encr_pkt_data_t *auditEventData_p);

void adk_ipsec_fill_ing_sa_validation_audit_info(adk_pktio_task_t *task,
                                                 adk_ipsec_audit_event_ing_sa_validation_t *auditEventSAData_p);
adk_status_t
adk_ipsec_ph_rx_parse (void *task, void **dp_message, adk_module_handle_t ipsecHandle);

#endif
