/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_ipsec_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief
 *
 * IPsec module's internal structure file.
 *
 ***********************************************************************/
#ifndef _ADK_IPSEC_DP_H_
#define _ADK_IPSEC_DP_H_

#include "adk_ip_dp_export.h"
#include "adk_ipv6_dp_export.h"

#define ADK_IPSEC_DP_LOG_FATAL_ERROR(__msg) \
            ADK_DP_LOG_FATAL_ERROR(ADK_IPSEC_MODULE_ID, __msg)

#define ADK_IPSEC_DP_LOG_DEBUG debug_print

#ifdef ADK_UTILITY_COUNTER_ENABLE 
#define INCREMENT_UTILITY_COUNTER(counter) \
    do  \
    {       \
        if(unlikely(adk_ipsec_dp_tbl_gbl_ptr->isUtilityCntEnabled))  \
        {  \
            ++counter;  \
        }  \
    }while(0);

#define INCREMENT_EGRESS_UTILITY_COUNTER(coreIndex, version, counter) \
    if (ADK_IPSEC_IPV4 == version) \
    { \
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].egressUtilityCounters.counter); \
    } \
    else \
    { \
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].egressUtilityCounters.counter); \
    }

#define INCREMENT_INGRESS_UTILITY_COUNTER(version, counter) \
    if (ADK_IPSEC_IPV4 == version) \
    { \
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV4[coreIndex].ingressUtilityCounters.counter); \
    } \
    else \
    { \
        INCREMENT_UTILITY_COUNTER(adk_ipsec_dp_tbl_gbl_ptr->ipsecUtilityCntV6[coreIndex].ingressUtilityCounters.counter); \
    }
#else
#define INCREMENT_UTILITY_COUNTER(counter) 
#define INCREMENT_EGRESS_UTILITY_COUNTER(version, counter)
#define INCREMENT_INGRESS_UTILITY_COUNTER(version, counter)
#endif


#define IPV4DEFTTL  64
#define IPV6DEFTTL  255

#define ADK_IPSEC_DP_DEBUG_PRINT  debug_print

#define ADK_IPSEC_DUMMY_HDR 59

#define IS_AUDIT_EXCEPT_SET(flag, pos) ((flag >> pos) & 0x1)

/* This mask defines the number of bits the rule set ID needs to be
 * shifted by (to pass the policy index in case of audit events).
 * Since 700 rulesets need to be supported as per requirement, 10 bits
 * are needed to store the set ID. Using the remaining 22 bits for
 * number of rules per set.
 */
#define IPSEC_RULES_PER_SET_SHIFT_MASK      22

#define IPSEC_ICMP_MSGTYPE_ECHO_REP         0
#define IPSEC_ICMP_MSGTYPE_DEST_UNREACHBLE  3
#define IPSEC_ICMP_MSGTYPE_SRC_QUENCH       4
#define IPSEC_ICMP_MSGTYPE_REDIRECT         5
#define IPSEC_ICMP_MSGTYPE_ECHO_REQ         8
#define IPSEC_ICMP_MSGTYPE_TIME_EXCEEDED    11
#define IPSEC_ICMP_MSGTYPE_PARAM_PROB       12
#define IPSEC_ICMP_MSGTYPE_TIMESTAMP_REQ    13
#define IPSEC_ICMP_MSGTYPE_TIMESTAMP_REP    14
#define IPSEC_ICMP_MSGTYPE_INFO_REQ         15
#define IPSEC_ICMP_MSGTYPE_INFO_REP         16
#define IPSEC_ICMPV6_MSGTYPE_DEST_UNREACHBLE 1
#define IPSEC_ICMPV6_MSGTYPE_TOO_BIG         2
#define IPSEC_ICMPV6_MSGTYPE_TIME_EXCEEDED   3
#define IPSEC_ICMPV6_MSGTYPE_PARAM_PROB      4

#define ADK_IPSEC_ICMP_TYPE_NOT_ALLOWED 1

#define ADK_IPSEC_DEF_IPV6_VTC_FLOW 0x60000000
#define ADK_IPSEC_TC_OFFSET 22

#define ADK_IPSEC_INLINE    0x0
#define ADK_IPSEC_LOOKASIDE 0x1
#define ADK_IPSEC_INVALID_SPD_ID  0xFFFF

/* Errors returnd by CPM - ideally, it shouldn't
 * be here! There should be some common place where
 * these error codes are defined. Not sure if there
 * is one, so defining it here.
 * This should reflect CPM:HAS Table 26-2 ONPI Metadata Generated by CPM
 */
#define ADK_IPSEC_CPM_ERROR_SA_MISS       0x1
#define ADK_IPSEC_CPM_ERROR_NOT_DECRYPTED 0x2
#define ADK_IPSEC_CPM_ERROR_ICV_FAILED    0x3
#define ADK_IPSEC_CPM_ERROR_WRONG_LEN     0x4
#define ADK_IPSEC_CPM_ERROR_MISC          0xF


/* TODO, if this is defined somewhere else, remove it from here */
#define IPV4_HDR_LEN  20
#define IPV6_HDR_LEN  40
#define UDPV4_HDR_LEN 8
#define ESPV4_HDR_LEN 8
#define ADK_IPSEC_NATT_PORT 4500
#define ADK_IPSEC_NATT_KEEPALIVE_SIZE IPV4_HDR_LEN+UDPV4_HDR_LEN+1

/* Various extension headers in IPv6, remove this if
 * we find some other common header file where the
 * same headers are defined. TODO
 * below is the same order in which the extension headers
 * are supposed to appear in IPv6 packet
 */

/* we actually care only about the next header value and header length
 * so, we don't need all those headers! This is just for reference,
 * we can just extract the first 8 bits in each header and depending on the
 * value for next header, can decide parsing
 */
#define ADK_IPSEC_V6_NEXT_HDR_HOP_HOP   0
#define ADK_IPSEC_V6_NEXT_HDR_DST_OPTS  60
#define ADK_IPSEC_V6_NEXT_HDR_RT_HDR    43
#define ADK_IPSEC_V6_NEXT_HDR_FRAG_HDR  44
#define ADK_IPSEC_V6_NEXT_HDR_NO_HDR    59
#define IPSEC_MAX_PAD_SIZE    UINT8_MAX

typedef struct adk_ipsec_v6_frag_hdr_s
{
    uint8_t   nextHdr;
    uint8_t   resvd;
    uint16_t  fragOffset;
    uint32_t  fragId;
} adk_ipsec_v6_frag_hdr_t __rte_cache_aligned;

typedef struct ipsec_dp_outbound_meta_data_s
{
    /* It is important that the first few
     * members of this DS match exactly with
     * that of IMC DS for IPv4 and IPv6.
     * That way, all unncessary copying can
     * be avoided.
     * path is not used by IPsec anywhere,
     * having this just simplifies passing
     * metadata across QAT stage and then
     * to IP modules thats all
     */
    uint8_t   path;
    uint32_t  vrdId;
    uint16_t  outerPathMtu;
    uint16_t  tunnelId;
    uint8_t   laFlag;
    uint8_t   laFlagVersion;
    uint8_t   dscpVal_version;
    uint32_t  saId;
    adk_handle_t saHandle;
} ipsec_dp_outbound_meta_data_t __rte_cache_aligned;

typedef struct ipsec_dp_inbound_meta_data_s
{
    bool      antiReplayEnable;
    uint8_t   ivLen;
    uint8_t   digestLen;
    uint8_t   dscpVal_version;
    uint32_t  saId;
    uint64_t  seqNumber;
    adk_handle_t saHandle;
    bool      useSingleSaInPreFrag;
    uint8_t   ecnFlag;
} ipsec_dp_inbound_meta_data_t __rte_cache_aligned;

typedef struct _ipsecIdSeqRange_t
{
    uint16_t min;
    uint16_t max;
} __attribute__ ((aligned)) ipsecIdSeqRange_t;

typedef struct _ipsecIdSeqDB_t
{
    uint16_t ipIdSeq;
} __attribute__ ((aligned)) ipsecIdSeqDB_t;

#define NO_FRAGS  TRUE
#define MORE_FRAGS  FALSE

typedef struct adk_ipsec_cache_frag_info_s
{
    uint8_t   operType;
    uint8_t   innerVersion;
    uint32_t  unrollPolIndex;
    uint32_t  saId;
    uint32_t  ivLen;
    union
    {
        struct ipv4_hdr ipv4Hdr;
        struct ipv6_hdr ipv6Hdr;
    }outerHeader;
    adk_pal_dp_stage_info_t     *stageInfo;
    union
    {
        adk_ip_dp_uli_imc_data_t    ipv4Imc;
        adk_ipv6_dp_uli_imc_data_t  ipv6Imc;
    }ipImc;
    adk_ipsec_dp_eg_sa_t   *saInfo;
} adk_ipsec_cache_frag_info_t;

/* adk_ipsec_dp_uli_common_params_t consists of common params
 * used across multiple functions in IPsec DP ULI.
 */
typedef struct adk_ipsec_uli_common_params_s
{
    int coreIndex;
    adk_pktio_task_t *taskInfo;
    uint8_t version:3;
    uint8_t saDir:1;
    uint8_t singleSa:1;
    uint8_t configDF:2;
    uint8_t reserved:1;
    uint8_t operType;
    uint16_t polIndex;
    uint16_t saId;
    uint8_t nextHdr;
    uint8_t destFunc;
    uint8_t pktOrigin;
    uint16_t frag_mtu;
    uint16_t innerPktLen;
    uint8_t innerIPHdrOffset;
    uint8_t ipHdrLen;
    uint16_t nonInitialPolIndex;
    uint16_t nonInitialSaId;
    adk_ipsec_spd_sel_t *spdPtn;
    struct rte_crypto_op *opData;
} adk_ipsec_dp_uli_common_params_t __rte_cache_aligned;

static const uint8_t padValues[IPSEC_MAX_PAD_SIZE] = {
    1, 2, 3, 4, 5, 6, 7, 8,
    9, 10, 11, 12, 13, 14, 15, 16,
    17, 18, 19, 20, 21, 22, 23, 24,
    25, 26, 27, 28, 29, 30, 31, 32,
    33, 34, 35, 36, 37, 38, 39, 40,
    41, 42, 43, 44, 45, 46, 47, 48,
    49, 50, 51, 52, 53, 54, 55, 56,
    57, 58, 59, 60, 61, 62, 63, 64,
    65, 66, 67, 68, 69, 70, 71, 72,
    73, 74, 75, 76, 77, 78, 79, 80,
    81, 82, 83, 84, 85, 86, 87, 88,
    89, 90, 91, 92, 93, 94, 95, 96,
    97, 98, 99, 100, 101, 102, 103, 104,
    105, 106, 107, 108, 109, 110, 111, 112,
    113, 114, 115, 116, 117, 118, 119, 120,
    121, 122, 123, 124, 125, 126, 127, 128,
    129, 130, 131, 132, 133, 134, 135, 136,
    137, 138, 139, 140, 141, 142, 143, 144,
    145, 146, 147, 148, 149, 150, 151, 152,
    153, 154, 155, 156, 157, 158, 159, 160,
    161, 162, 163, 164, 165, 166, 167, 168,
    169, 170, 171, 172, 173, 174, 175, 176,
    177, 178, 179, 180, 181, 182, 183, 184,
    185, 186, 187, 188, 189, 190, 191, 192,
    193, 194, 195, 196, 197, 198, 199, 200,
    201, 202, 203, 204, 205, 206, 207, 208,
    209, 210, 211, 212, 213, 214, 215, 216,
    217, 218, 219, 220, 221, 222, 223, 224,
    225, 226, 227, 228, 229, 230, 231, 232,
    233, 234, 235, 236, 237, 238, 239, 240,
    241, 242, 243, 244, 245, 246, 247, 248,
    249, 250, 251, 252, 253, 254, 255,
};

adk_status_t adk_ipsec_dp_pkt_proc_uli_from_crypto (void **,
                                                    uint8_t ,
                                                    uint32_t,
                                                    __attribute__((unused)) void *,
                                                    __attribute__((unused)) adk_sif_cfg_pipeline_info_t *,
                                                    int coreIndex);

adk_status_t
adk_ipsec_dp_pkt_proc_lli_single (void **taskList, int coreIndex);

adk_status_t
adk_ipsec_dp_pkt_proc_lli_from_crypto (void **, uint8_t ,
                                       uint32_t , __attribute__((unused)) void *,
                                      __attribute__((unused)) adk_sif_cfg_pipeline_info_t *,
                                      int coreIndex);

adk_status_t adk_ipsec_prepare_proto_op(int coreIndex, void *pkt, adk_metadata_t *adkMetaData,
                                        adk_ipsec_dp_eg_sa_t *saInfo,
                                        uint8_t innerVersion,
                                        uint32_t packetLen,
                                        uint32_t saId,
                                        adk_ipsec_dont_frag_config_t dfConfig,
                                        adk_ipsec_stat_t *globalStats,
                                        uint16_t *innerIPHdrOffset,
                                        uint8_t isFrag);

uint32_t adk_ipsec_inline_prepare_crypto_op(void *mbuf,
                                     adk_ipsec_dp_eg_sa_t *saInfo,
                                     uint8_t ipVersion, uint32_t innerPktLen);

uint32_t adk_ipsec_inbound_la_prepare_crypto_op(void *mbuf, struct rte_crypto_op *opData,
                                                adk_ipsec_dp_ing_sa_t *saInfo,
                                                uint8_t ipHdrLen,
                                                uint32_t innerPktLen);

uint32_t adk_ipsec_outbound_la_prepare_crypto_op(void *mbuf, struct rte_crypto_op *opData,
                                     adk_ipsec_dp_eg_sa_t *saInfo,
                                     uint8_t ipHdrLen, uint8_t ipVersion,
                                     uint32_t innerPktLen);

adk_dp_status_t adk_ipsec_dp_spi_get(uint32_t spi, uint32_t *sa_info);

adk_ipsec_dp_ing_sa_t *adk_ipsec_spi_lkup(uint32_t );

void prepare_packet(struct rte_mbuf *, uint8_t );

adk_status_t adk_ipsec_spd_lkup(adk_ipsec_spd_sel_t *, uint16_t *);

adk_status_t adk_ipsec_spd_in_lkup(adk_ipsec_spd_in_sel_t *, uint16_t *);

adk_status_t
adk_ipsec_get_initial_frag_info_or_buffer_non_initial_frags(int coreIndex, uint8_t innerVersion,
                                                            uint16_t cur_saId, adk_pktio_task_t *taskInfo,
                                                            uint16_t pktLen, adk_ipsec_spd_sel_t *spdPtn, 
                                                            uint16_t ipHashIndex, uint8_t ipBucketIndex,
                                                            bool *isSPDLookUpNeeded);

void
adk_ipsec_add_initial_frag_info(uint8_t innerVersion, uint16_t initial_frag_sa_id, uint16_t hashIndex, uint8_t bucketIndex,
    uint16_t l4SrcPort, uint16_t l4DstPort);

void
adk_ipsec_buffer_initial_frag(int coreIndex, uint8_t innerVersion, uint16_t hashIndex, uint8_t bucketIndex, uint16_t pktLen,
                              uint16_t saId, adk_pktio_task_t *taskInfo);

adk_status_t
adk_ipsec_dp_handle_exception(
    adk_pktio_task_t *ph_task,
    uint8_t type,
    void *exception_data,
    int coreIndex,
    uint8_t version,
    uint8_t direction,
    uint8_t isFrag);

int adk_ipsec_arw_check(
        unsigned long long sequence_number,
        uint16_t iSAIndex
        );

int adk_ipsec_arw_update(
        unsigned long long sequence_number,
        uint16_t iSAIndex
        );

adk_status_t
adk_ipsec_outbound_icmp_transit(int coreIndex, adk_pktio_task_t  *taskInfo,
                                uint8_t version,
                                adk_ipsec_cache_frag_info_t *cacheInfo,
                                uint16_t *destFunc,
                                uint8_t *operType,
                                bool fragFlag);

adk_status_t
adk_ipsec_inbound_icmp_transit(void *taskList, uint8_t *pdu,
                               uint8_t version,
                               uint16_t *polIndex);

struct rte_mbuf* adk_ipsec_pktmbuf_clone(struct rte_mbuf * inputPkt,
                            struct rte_mempool * pool);


#endif
