/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipsec_export.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipsec  IPSEC Module
 *
 * @{
 *
 * @brief
 *
 * This file contains all ADK API function definitions which can be used by IP module
 * for inter module optimisations
 *
 ***********************************************************************/

#ifndef _ADK_IPSEC_EXPORT_H_
#define _ADK_IPSEC_EXPORT_H_

#define ADK_IPSEC_NUM_OF_SA   2000
#define ADK_IPSEC_INVALID_SA_ID (ADK_IPSEC_NUM_OF_SA + 1)

typedef struct adk_ipsec_dp_uli_imc_param_s
{
    /* TODO: need to revisit during flow cache */ 
    uint8_t   la_flag;
    uint8_t   la_flag_version;
    uint16_t  srcPort; /* This can serve as ICMP type as well */
    uint16_t  dstPort; /* This can serve as ICMP code as well */
    uint8_t   protocol;
    bool      ipv6FragFlag;
    uint8_t   pktOrigin;
    uint8_t   ipv6_nextHdr;
    } __attribute__ ((aligned(ADK_METADATA_ALIGN))) adk_ipsec_dp_uli_imc_param_t;

/*!
 * =============================================================
 * @struct adk_ip_dp_lli_imc_data_t
 *
 * @brief
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPsec module LLI for the purpose of packet processing.
 *
 * =============================================================
 */

typedef struct adk_ipsec_dp_lli_imc_param_s
{
    uint8_t   isLocal;
    uint8_t   pktOrigin;
    uint8_t   ipv6_nextHdr;
} __attribute__ ((aligned(ADK_METADATA_ALIGN))) adk_ipsec_dp_lli_imc_param_t;


/*
 * These functions are used by datapath(inter module functions), shall not be used by application
 */

adk_status_t
adk_ipsec_dp_pkt_proc_uli (void **, uint8_t , uint32_t ,
                           void *, adk_sif_cfg_pipeline_info_t *,
                           int coreIndex);

adk_status_t
adk_ipsec_dp_pkt_proc_lli (void **, uint8_t , uint32_t ,
                           void *, adk_sif_cfg_pipeline_info_t *,
                           int coreIndex);
uint16_t
adk_ipsec_clear_initial_frag_buffered_pkts_hash_entry(uint16_t hashIndex, uint8_t bucketIndex);

uint16_t
adk_ipsec_ipv6_clear_initial_frag_buffered_pkts_hash_entry(uint16_t hashIndex, uint8_t bucketIndex);

void adk_ipsec_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#endif
/*!  end addtogroup _ipsec  IPSEC Module
 * @}
 */
