/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipv6.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _ipv6  IPv6 Module
 * @{
 *
 * @brief  
 *
 * This file contains all the external definitions (except constants)
 * required to use the IPV6 module. This file can be included by the
 * application.
 *
 ***********************************************************************/
#ifndef _ADK_IPV6_H_
#define _ADK_IPV6_H_

#include "adk_ph_dev.h"
#include "adk_domain.h"
#include "adk_ipv6_const.h"
#include "adk_qos_include.h"
#include "adk_hos.h"

/*!
 * =============================================================
 * @enum adk_ipv6_exception_type_t
 *
 * @brief 
 *
 * This enumeration defines all exceptions generated by IPV6
 * module.
 *
 * =============================================================
 */
typedef enum adk_ipv6_exception_type_e
{
    /*! The route is not found for the destination IP in the
     *  route_lookup_tree.
     */
    ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND = 0,
    /*! A parsing issue was encountered with the PDU while
     *  parsing the PDU.
     */
    ADK_IPV6_EXCEPTION_PARAM_PROBLEM = 1,
    /*! The hop limit of the PDU has become 0 */
    ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO = 2,
    /*! The size of the PDU is more than the MTU configured
     *  and the PDU cannot be fragmented because of
     *  configured parameters.
     */
    ADK_IPV6_EXCEPTION_MTU_VIOLATION = 3,
    /*! Reassembly timer expired */
    ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED = 4,
    /*! Fragment overlaps with existing fragments */
    ADK_IPV6_EXCEPTION_FRAG_OVERLAP = 5,
    /*! Extension header occurred more than once in packet */
    ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE = 6,
    /*! Forwarding is disabled on this interface*/
    ADK_IPV6_INTERFACE_FORWARDING_DISABLE = 7,
    /*! Incorrect ip version in ipv6 header */
    ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION = 8,
    /*! Unsupported L4 protocol*/
    ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO = 9,
    /*! Invalid Exception */
    ADK_IPV6_EXCEPTION_INVALID = 10,
    /*! UDP ingress checsum invalid exception
     */
    ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID = 11,
    /*! The PDU has more than 8 extension headers */
    ADK_IPV6_EXCEPTION_TOO_MANY_EXTENSION_HEADERS,
    /*! Resolve the layer-2 address of the next-hop. The
     *  next-hop ID is passed in the task parameters.
     */
    ADK_IPV6_EXCEPTION_RESOLVE_NEXTHOP_L2_ADDRESS,
    /*! Resolve the layer-2 address of the destination
     *  IPv6 address in the exception PDU.
     */
    ADK_IPV6_EXCEPTION_RESOLVE_DESTINATION_L2_ADDRESS,
    /*! Flow is not found in egress_flow_attr_tree */
    ADK_IPV6_EXCEPTION_UNKNOWN_EGRESS_ASSOCIATION,
    /*! The IPv6 ICMP is not translated, hence it is sent
     *  for exception to be handled.
     */
    ADK_IPV6_EXCEPTION_IPV6_ICMP,
    /*! Multicast forwrding table look up failed, hence
     * sent for exception to be handled
     */
    ADK_IPV6_MULTICAST_FWD_LOOKUP_FAIL,
    /*! Multicast RPF check failed, hence
     * sent for exception to be handled
     */
    ADK_IPV6_MULTICAST_RPF_CHECK_FAIL,
    /*! Illegal IPv6 Src Addr Received for Translation
     */
    ADK_IPV6_ILLEGAL_SRC_ADDR_TRANSLATE_RECEIVED,
    /*! UDP ingress length invalid exception 
     */
    ADK_IPV6_UDP_INGRESS_LENGTH_INVALID,
    /*! DEST MTU table lookup failed exception 
     */
    ADK_IPV6_EXCEPTION_DST_MTU_FAIL,
    /*! Local PTB message generated when Post frgmentation is detetected
     */
    ADK_IPV6_EXCEPTION_LOCAL_PTB,
    /*! The maximun exception type value*/
    ADK_IPV6_EXCEPTION_MAX
} adk_ipv6_exception_type_t;   /* ------  end of enum adk_ipv6_exception_type_t  ------ */

/*!
 * =============================================================
 * @enum adk_ipv6_obj_t
 *
 * @brief 
 *
 * This enumeration defines all object types for dynamic memory 
 * of ipv6.
 *
 * =============================================================
 */
typedef enum adk_ipv6_obj_s {
    ADK_IPV6_MOD_INFO_OBJ,
    ADK_IPV6_VIRT_IF_RES_OBJ,
    ADK_IPV6_ACL_RES_OBJ,
    ADK_IPV6_ACL_STATS_OBJ,
    ADK_IPV6_PBR_RULESETS_OBJ,
    ADK_IPV6_PBR_MAX_RULES_OBJ,
    ADK_IPV6_FILT_RULE_SET_OBJ,
    ADK_IPV6_FILT_HANDLE_OBJ,
    ADK_IPV6_SPORT_MASK_OBJ,
    ADK_IPV6_DPORT_MASK_OBJ,
    ADK_IPV6_PROTO_MASK_OBJ,
    ADK_IPV6_DSCP_MASK_OBJ,
    ADK_IPV6_RULE_ATTRIB_OBJ,
    ADK_IPV6_ING_ASSOC_OBJ,
    ADK_IPV6_EG_ASSOC_OBJ,
    ADK_IPV6_INTF_OBJ,
    ADK_IPV6_TUN_INTF_OBJ,
    ADK_IPV6_VIRT_INTF_OBJ,
    ADK_IPV6_TBL_DUMP_OBJ,
    ADK_IPV6_LPM_ADD_OBJ,
    ADK_IPV6_PKT_CAP_OBJ,
    ADK_IPV6_NH_ADD_OBJ,
    ADK_IPV6_CPU_NH_ADD_OBJ,
    ADK_IPV6_DST_ADD_OBJ,
    ADK_IPV6_ROUTE_ENTRY_OBJ,
    ADK_IPV6_SIP_ENTRY_OBJ,
    ADK_IPV6_DST_MTU_TBL_OBJ,
    ADK_IPV6_DST_MTU_HASHTBL_OBJ,
    ADK_IPV6_VRD_OBJ,
    ADK_IPV6_DP_TBL_OBJ,
    ADK_IPV6_DP_LPM_TBL_OBJ,
    ADK_IPV6_DP_PEND_LIST_OBJ,
    ADK_IPV6_OFL_ADD_OBJ,
    ADK_IPV6_IN_FLOW_TBL_OBJ,
    ADK_IPV6_HOS_RULE_OBJ,
    ADK_IPV6_ING_CLASSIFIER_INFO_OBJ,
    ADK_IPV6_ING_CLASSIFIER_RULE_INFO_OBJ,
    ADK_IPV6_ING_CLASSIFIER_RULE_ACT_OBJ,
    ADK_IPV6_ADDR_STATS_OBJ,
    ADK_IPV6_ING_MGROUP_OBJ,
    ADK_IPV6_IF_HANDLE_OBJ,
    ADK_IPV6_EGRESS_QOS_CLASSIFIER_INFO_OBJ
} adk_ipv6_obj_t;
/*!
 * =============================================================
 * @enum adk_ipv6_exception_handler_loc_t
 *
 * @brief 
 *
 * This enumeration defines the exception handler locations as
 * CPU or MPP.
 *
 * =============================================================
 */
typedef enum adk_ipv6_exception_handler_loc_e
{
    /*! The exception handler is on the CPU via PH. */
    ADK_IPV6_EXCEPTION_LOC_CPU_PH=1 ,
    /*! The exception handler is on the CPU via Netdriver. */
#ifdef ADK_QOS_GEN3_INTERFACE
    ADK_IPV6_EXCEPTION_LOC_CPU_SP_STACK=2,
#else
    ADK_IPV6_EXCEPTION_LOC_CPU_NOPH,
#endif
    /*! The exception handler is on MPP */
    ADK_IPV6_EXCEPTION_LOC_MPP

} adk_ipv6_exception_handler_loc_t;   /* ------  end of enum adk_ipv6_exception_handler_loc_t  ------ */

/*!
 * =============================================================
 * @enum adk_ipv6_status_t
 *
 * @brief 
 *
 * This enumeration defines all error codes returned by IPV6
 * module external APIs.
 *
 * =============================================================
 */
typedef enum adk_ipv6_status_e {
    /*! Successfully completed requested operation */
    ADK_IPV6_SUCCESS = ADK_SUCCESS,
    /*! A generic failure, for the default case in mapping the
     *  unknown RTE errors.
     */
    ADK_IPV6_E_GENERIC_FAILURE = ((ADK_IPV6_MODULE_ID << 24) + 1),
    /*! Unable to reset the state engine memory for statistics */
    ADK_IPV6_E_UNABLE_TO_RESET_STATS,
    /*! Invalid input parameter*/
    ADK_IPV6_E_INVALID_PARAMETER,
    /*! Failed to allocate memory for request operation*/
    ADK_IPV6_E_MEM_UNAVAILABLE,
    /*! Failed to allocate resource for request operation*/
    ADK_IPV6_E_RSRC_UNAVAILABLE,
    /*! Failed to allocate filter state engine memory block */
    ADK_IPV6_E_FILTER_STATE_MEMORY_UNAVAILABLE,
    /*! Failed to allocate dip tree state engine memory block */
    ADK_IPV6_E_DIP_STATE_MEMORY_UNAVAILABLE,
   /*! Failed to allocate sip tree state engine memory block */
    ADK_IPV6_E_SIP_STATE_MEMORY_UNAVAILABLE,
    /*! Failed to program MPP object (tree, hash, SE memory) */
    ADK_IPV6_E_MPP_PROGRAMMING_FAILURE,
    /*! Failed to read stats from state engine memory */
    ADK_IPV6_E_STATS_READ_FAILURE,
    /*! Invalid Layer-2 interface passed as input */
    ADK_IPV6_E_INVALID_L2_INTERFACE,
    /*! NULL pointer input is receceived */
    ADK_IPV6_E_NULL_PTR,
    /*! Invalid input filter pattern combination */
    ADK_IPV6_E_INVALID_INPUT_FILTER_COMBINATION,
    /*! Interface lookup tree lookup failed */
    ADK_IPV6_E_INTERFACE_LOOKUP_TREE_FAIL,
    /*! Failed to fetch filter rules for the interface */
    ADK_IPV6_E_FAILED_TO_GET_NUM_RULES,
    /*! Outof bound filter index is specified */
    ADK_IPV6_E_INVALID_FILTER_INDEX,
    /*! Filtering is already enabled on the interface */
    ADK_IPV6_E_FILTERING_ALREADY_ENABLED,
    /*! No namespace allocated to store the interface statistics */
    ADK_IPV6_E_INTERFACE_STAT_NULL_NS,
    /*! Failed to read/write the interface statistics */
    ADK_IPV6_E_INTERFACE_STAT_SE_FAILURE,
    /*! Maximum number of classes that can be configured in single
     * API call is exceeded.
     */
    ADK_IPV6_E_MAX_CLASS_CONFIG_EXCEEDED,
    /*! Inconsistent state between control API and fastpath */
    ADK_IPV6_E_INCONSISTENT_STATE,
    /*! Invalid IP Qos profile type information. */
    ADK_IPV6_E_INVALID_QOS_PROFILE_TYPE,
    /*! Invalid DSCP profile handle or DSCL class type spefiied. */
    ADK_IPV6_E_INVALID_DSCP_PROFILE,
    /*! committed (rate or burst) value is bigger than peak value
     * (rate or burst) for policing flow configuration.
     */
    ADK_IPV6_E_INCONSISTENT_RATES,
   /*! Ruleset trying to delete is not empty
     */
    ADK_IPV6_E_RULE_SET_NOT_EMPTY,
  /*!Invalid config
     */
    ADK_IPV6_E_INVALID_CONFIG,
    /*!
      * Invalid IP host or interface address
     */
    ADK_IPV6_E_INVALID_IP_ADDRESS,
    /*!
     * Invalid SRC or DST IPV6 address prefix
     */
    ADK_IPV6_E_INVALID_ADDR_PREFIX,
    /*!
     * Invalid IP Interface
     */
    ADK_IPV6_E_INVALID_IP_INTERFACE,
    /* Invalid IP Exception type */
    ADK_IPV6_E_INVALID_EXCEPTION_TYPE

} adk_ipv6_status_t;   /* ------  end of enum adk_ipv6_status_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_addr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The ipv6 address structure.
 *
 * This structure holds the IPv6 address. The lower bytes of
 * the IPv6 address are stored starting in ipv6Addr[0].
 *
 * =============================================================
 */
typedef struct adk_ipv6_addr_s
{
    /*! The array of 32 bit words containing the IPv6 address.
     *  The higher bytes of the IPv6 address are stored starting
     *  in ipv6Addr[0].
     */
    uint32_t addr[ADK_IPV6_PDU_ADDR_SIZE_WORDS];
} adk_ipv6_addr_t;   /* ------  end of struct adk_ipv6_addr_t  ------ */

/*!
 * adk_ipv6_pbr_policy_set_handle_t
 * IPv6 PBR Policy Set handle. This handle is returned at the time of
 * IPv6 PBR Policy Set creation time.
 */
typedef adk_handle_t adk_ipv6_pbr_policy_set_handle_t;

/*!
 * =============================================================
 * @struct adk_ipv6_addr_prefix_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The pbr lookup input structure.
 *
 * This structure contains the ipv6 address and its prefix length and 
 * it is used in pbr lookup input pattern.
 *
 * =============================================================
 */
typedef struct adk_ipv6_addr_prefix_s
{
    /*! The IPv6 address */
    adk_ipv6_addr_t Ipv6Addr;
    /*! The prefix length */
    uint8_t prefixLength;
} adk_ipv6_addr_prefix_t;   /* ------  end of struct adk_ipv6_addr_prefix_t  ------ */

/*!
 * adk_ipv6_qos_policing_flow_handle_t
 * IPv6 Policing parameters flow handle definition.
 */
typedef adk_handle_t adk_ipv6_qos_policing_flow_handle_t;
/*!
 * adk_ipv6_filter_rule_set_handle_t
 * IPv6 filter rule-set handle. This handle is returned at the time of 
 * IPv6 filter rule-set creation.
 */
typedef adk_handle_t adk_ipv6_filter_rule_set_handle_t;

/*!
 * adk_ipv6_teredo_client_addr_t
 * IPv6 teredo client address required to be added into
 * teredo client trusted peer list.
 */
typedef adk_ipv6_addr_t adk_ipv6_teredo_client_addr_t;

/*!
 * adk_ipv6_teredo_client_addr_t
 * IPv6 Dscp Group Hanlde definition
 */
typedef adk_handle_t adk_dscp_group_handle_t;


typedef adk_flow_handle_t adk_ipv6_ingress_classifier_rule_handle_t;
/*!
 * =============================================================
 * @typedef adk_ipv6_vrd_handle_t
 *
 * @brief
 *
 * IPv6 Virtual Routing Instance handle
 *
 * =============================================================
 */
typedef adk_handle_t adk_ipv6_vrd_handle_t;

/*!
 * =============================================================
 * @struct adk_ipv6_vrd_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create IPv6 virtual routing domain.
 *
 * =============================================================
 */
typedef struct adk_ipv6_vrd_s {
    /*! Virtual routing domain id.
     * It is application's responsibility to assign a unique ID
     * in the range 0 to (ADK_IPV6_VRD_ID_MAX - 1) except the ID
     * reserved for the default VRD i.e. ADK_IPV6_DEFAULT_VRD_ID.
     */
    uint32_t  vrd_id;

} adk_ipv6_vrd_t;


/*!
 * =============================================================
 * @struct adk_ipv6_ph_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure contains all elements that are required for
 * sending packets to ADK PDU handler.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ph_attr_s {
    uint8_t     devId;
    uint16_t    queueId;
    /*!
     * phBindHandle and cbFnId is valid only if the path is
     * ADK_IPV6_PROCESSING_PATH_CPU_PH. The application is expected to 
     * expceted to create the bind handle and
     * register the call back function with the PDU handler
     * before calling register API (reverse process is applicable
     * to unregister API). call back function ID is required if
     * the PDU needs to be multiplexed to a function based on 
     * call back function ID.
     */
    adk_ph_pipeline_bind_handle_t  phBindHandle;
    /*!
     * optional call back function as discussed above.
     */
    uint8_t                    cbFnId;
} adk_ipv6_ph_attr_t;   /* ------  end of struct adk_ipv6_ph_attr_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_nca_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure contains all elements that are required for
 * sending packets to ADK NCA.
 *
 * =============================================================
 */
#ifdef ADK_QOS_GEN3_INTERFACE
typedef struct adk_ipv6_nca_attr_s {
        uint32_t                    dev_net_id;
        /* For future use */
        uint32_t                    queue_id;
} adk_ipv6_nca_attr_t;   /* ------  end of struct adk_ipv6_nca_attr_t  ------ */
#endif

/*!
 * =============================================================
 *
 * The function signature for the callback function that gets invoked in
 * datapath when a matching flow is identified in IPv6 ingress association with
 * action ADK_IPV6_INGRESS_ATTR_INVOKE_CB
 *
 * =============================================================
 */
typedef void (*adk_ipv6_ingress_association_recv_cb_fn_t) (int32_t id,
                                                         void *packet,
                                                         void *user_data);
/*!
 * =============================================================
 * @struct adk_ipv6_ing_assoc_if_filter_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This structure contains all elements that are required for
 * interface based filtering of terminating packets.
 * =============================================================
 */
typedef struct adk_ipv6_ing_assoc_if_filter_info_s{
    bool is_filter_enabled;
    uint32_t interface_id;
}adk_ipv6_ing_assoc_if_filter_info_t;

/*!
 * =============================================================
 * @struct adk_ipv6_invoke_cb_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This structure contains all elements that are required for
 * invoking cb function in datapath from IPv6 module.
 *
 * =============================================================
 */
typedef struct adk_ipv6_invoke_cb_attr_s
{
    /*!  
     * The callback function that gets invoked in datapath. It is assumed that
     * the callback function registered is in the context of datapath process.
     */
    adk_ipv6_ingress_association_recv_cb_fn_t recvCb;
    /*!      
     * An integer type id (eg:- socket id) registered along with the cb function
     * and will be send back in the callback function.
     */
    int32_t id;
    /*!
     * A use data registered along with the cb function and will be send back in the
     * callback function.
     */
    void *user_data;
    /*!
     *interface based filtering info for multicast FPS sockets.
     */
    adk_ipv6_ing_assoc_if_filter_info_t if_filter_info;
} adk_ipv6_invoke_cb_attr_t;

/*!
 * =============================================================
 * @struct adk_ipv6_ingress_association_statistics_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The flow statistics structure.
 *
 * This structure contains the statistics that belong
 * to an ingress flow.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ingress_flow_statistics_s
{
    /*! The number of incoming PDUs */
    uint32_t incoming;
    /*! The number of PDUs sent to CPU as the flow attribute
     *  was configured to send the packets to CPU.
     */
    uint32_t sentToCpu;
}__attribute__((aligned(64))) adk_ipv6_ingress_association_statistics_t;   /* ------  end of struct adk_ipv6_ingress_association_statistics_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_egress_association_statistics_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The flow statistics structure.
 *
 * This structure contains the statistics that belong
 * to an egress flow.
 *
 * =============================================================
 */
typedef struct adk_ipv6_egress_flow_statistics_s
{
    /*! The number of incoming PDUs */
    uint32_t incoming;
}__attribute__((aligned(64))) adk_ipv6_egress_association_statistics_t;   /* ------  end of struct adk_ipv6_egress_association_statistics_t  ------ */

/*!
 * =============================================================
 * @internal
 * @enum adk_ipv6_next_hop_l2_state_t
 *
 * @brief The address resolution state of the next hop.
 *
 * The address resolution state of the next hop node.
 *
 * =============================================================
 */
typedef enum adk_ipv6_next_hop_l2_state_e
{
    /*! The L2 address of the next hop is NOT resolved */
    ADK_IPV6_NEXT_HOP_L2_STATE_INCOMPLETE,
    /*! The L2 address of the next hop is resolved */
    ADK_IPV6_NEXT_HOP_L2_STATE_COMPLETE
} adk_ipv6_next_hop_l2_state_t;   /* ------  end of enum adk_ipv6_next_hop_l2_state_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_next_hop_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The IPv6 next hop structure.
 *
 * This structure contains the next hop attributes.
 *
 * =============================================================
 */
typedef struct adk_ipv6_next_hop_attr_s
{
    /*! Indicates next hop’s address. */
    adk_ipv6_addr_t ipv6Addr_p;

    /*! Egress Ipv6 interface handle.
     */
    adk_interface_handle_t interfaceHandle;
} adk_ipv6_next_hop_attr_t;   /* ------  end of struct adk_ipv6_next_hop_attr_t  ------ */


/*!
 * =============================================================
 * @struct adk_ipv6_onlink_destination_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The attributes of the on link node.
 *
 * This contains the attributes of the node on the attached
 * link.
 *
 * =============================================================
 */
typedef struct adk_ipv6_onlink_destination_attr_s
{
    /*! Ipv6 Egress interface handle.
     */
    adk_interface_handle_t interfaceHandle;
} adk_ipv6_onlink_destination_attr_t;   /* ------  end of struct adk_ipv6_onlink_destination_attr_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_onlink_destination_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The on link destination information.
 *
 * This structure contains the information about a node attached
 * to the same link as this node.
 *
 * =============================================================
 */
typedef struct adk_ipv6_onlink_destination_info_s
{
   /*! ipv6VrdfHandle provides the handle for the Routing Domain.
     */
    adk_ipv6_vrd_handle_t ipv6VrdHandle;
    /*! The IPv6 address of the node attached on the same link.
     *  Note that this is the complete 128 bit address.
     */
    adk_ipv6_addr_t onLinkDestinationIpv6Addr;
    /*! The attributes of the destination. */
    adk_ipv6_onlink_destination_attr_t onLinkDestinationAttr;
} adk_ipv6_onlink_destination_info_t;   /* ------  end of struct adk_ipv6_onlink_destination_info_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_route_rules_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The route lookup input structure.
 *
 * This structure contains the parameters for the route lookup
 * input pattern.
 *
 * =============================================================
 */
typedef struct adk_ipv6_route_rules_s
{
    /*! The destination IPv6 address */
    adk_ipv6_addr_t dstIpv6Addr;
    /*! The prefix length */
    uint8_t prefixLength;
} adk_ipv6_route_rules_t;   /* ------  end of struct adk_ipv6_route_rules_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_route_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The route lookup attributes structure.
 *
 * This structure contains the parameters for the route lookup
 * output data.
 *
 * =============================================================
 */
typedef struct adk_ipv6_route_attr_s
{
    union {
	    /*! The handle of the next hop to which PDUs with the given
	     *  prefix are forwarded. This is not needed if the
	     *  destination is on the attached link.
	     *  The given handle is attached by default to this route, 
	     *  if Multipath Routing is enabled. Application is expected
	     *  to detach this next hop from route handle before deleting
	     *  the route.
	     */
	    adk_flow_handle_t nextHopHandle;

	    /*! If the route destination is ONLINK, (onLink = TRUE),
	     *  this field will take interface handle of the associated 
	     *  interface as input, It is a mandatory input for onlink scenario.
	     *  Not valid if the ONLINK is set to false (onlink = FALSE)
	     */  
	    adk_interface_handle_t interfaceHandle;
    }u_hdl;

    /*! Indicates if the destination is on the attached link.
     *  If this is TRUE, the nextHopHandle is ignored.
     *  ignored, if Multipath Routing is enabled
     */
    bool onLink;
    /*! Indicates the route MTU*/
    uint16_t routeMTU;
} adk_ipv6_route_attr_t;   /* ------  end of struct adk_ipv6_route_attr_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_route_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The IPv6 route information structure.
 *
 * This structure contains the information about the route
 * to add.
 *
 * =============================================================
 */
typedef struct adk_ipv6_route_info_s
{
    /*! The route lookup input parameters */
    adk_ipv6_route_rules_t ipv6RouteRules;
    /*! The route lookup output parameters */
    adk_ipv6_route_attr_t ipv6RouteAttr;
} adk_ipv6_route_info_t;   /* ------  end of struct adk_ipv6_route_info_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_range_t
 *
 * @brief 
 *
 * The IPv6 range data type is used to specify a contiguous range 
 * of values e.g. range of L4 port numbers or range of IP protocol
 * types, etc. This type of data type is used in PBR configuration.
 *
 * =============================================================
 */
typedef struct adk_ipv6_range_s {
    uint32_t begin;
    uint32_t end;
} adk_ipv6_range_t;

/*!
 * =============================================================
 * @enum adk_ipv6_pbr_field_mask_t
 *
 * @brief 
 *
 * This enum contains the bit mask for various fields used as input
 * for Policy Based Routing
 *
 * =============================================================
 */
typedef enum adk_ipv6_pbr_field_mask_s {
    /*!  IPV6 source IP address mask */
    ADK_IPV6_PBR_SRC_ADDR_MASK = 0x1,
} adk_ipv6_pbr_field_mask_t;


/*!
 * =============================================================
 * @enum adk_ipv6_frag_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * Defines the packet type w.r.t. to fragmentation. All combinations are
 * provided so that application does not have to add multiple rules for the same
 * flow.
 *
 * =============================================================
 */
typedef enum adk_ipv6_frag_e {
    /*! Unfragment packet */
    ADK_IPV6_UNFRAGMENTED_PKT = 0,
    /*! Either Unfragment packet or first fragment. This has L4 fields available
     * in the packet */
    ADK_IPV6_L4_AVAILABLE_PKT,
    /*! A fragmented packet which can be first (offset = 0) or a non-first
     * fragment (offset != 0) */
    ADK_IPV6_FRAGMENTED_PKT,
    /*! First fragment of a packet (offset = 0) */
    ADK_IPV6_FIRST_FRAGMENT,
    /*! Non first fragment of a packet (offset != 0) */
    ADK_IPV6_NON_FIRST_FRAGMENT
} adk_ipv6_frag_t;


/*!
 * =============================================================
 * @struct adk_ipv6_pbr_rule_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * Definition of adk_ipv6_pbr_rule_t
 *
 * This data structure is used as input for Policy based Routing. Since Policy
 * based routing can take arbitrary fields into account for lookup, all fields
 * in here except mask field are optional. The mask field indicates which field 
 * is specified and which are not. If a field is indicated to be valid by
 * setting corresponding bit in the bitmask, then a valid value for that field 
 * must be specified.
 * =============================================================
 */
typedef struct adk_ipv6_pbr_rule_s {
    /*! 
     * IP PBR rule selection mask. Logical OR of bit masks as defined in 
     * adk_ipv6_pbr_field_mask_t data structure
     */
    uint16_t PBR_RuleMask;
    /*!
     * srcAddrInfo gives the source IP address for this association. 
     * The source IP address can also be partially or fully specified 
     * using the prefix length.
     */
    adk_ipv6_addr_prefix_t srcAddrInfo;
} adk_ipv6_pbr_rule_t;

/*!
 * =============================================================
 * @struct adk_ipv6_pbr_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This data structure contains information that are required to 
 * update IP Policy Based Routing (PBR) table. 
 * =============================================================
 */
typedef struct adk_ipv6_pbr_action_s {
    uint8_t routeTableId;
} adk_ipv6_pbr_action_t;
/*!
 * =============================================================
 * @struct adk_ipv6_pbr_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This data structure contains information that are required to 
 * update IP Policy Based Routing (PBR) table. It is used as input 
 * parameter to adk_ipv6_pbr_entry_create and adk_ipv6_pbr_entry_delete APIs.
 *
 * =============================================================
 */
typedef struct adk_ipv6_pbr_info_s {
    /*!
     * PBR Rule coontains the input for the entire rule set to be 
     * created/deleted
     */
    adk_ipv6_pbr_rule_t pbrRule;
    /*!
     * PBR action coontains the output action for the entire rule set to be 
     * created/deleted
     */
    adk_ipv6_pbr_action_t pbrAction;
} adk_ipv6_pbr_info_t;

/*!
 * =============================================================
 * @struct adk_ipv6_pbr_rule_stats_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * Statistics for an IPv6 PBR rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_pbr_rule_stats_s {
    /*! total number of packets that matched the rule */
    uint32_t matchedPkts;

    /* Reserved space - do not use */
    uint32_t reserved_pad[3];
} adk_ipv6_pbr_rule_stats_t;

/*!
 * =============================================================
 * @struct adk_ipv6_egress_association_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The egress association attributes structure.
 *
 * This structure holds the attributes of the egress
 * association. The fields are arranged to help in the SED
 * script.
 *
 * =============================================================
 */
typedef struct adk_ipv6_egress_association_attr_s
{
    /* The mask to choose the optional parameters.
     * Each bit chooses a parameter.
     * The bits are defined as follows:
     * bit 0   - ADK_IPV6_EGRESS_RULE_USE_VRD_ID
     *           1: Use VRD Id, 0: Use VRD Handle
     * bit 1-7 - Reserved
     *
     */
    uint8_t rulesMask;
    /*! 
     *  ipv6VrdHandle provides the handle for the Routing Domain.
     *  If ADK_IPV6_EGRESS_RULE_USE_VRD_ID is not set in rulesMask, ipv6VrdHandle
     *  will be used. Else ipv6VrdId will be used.
     */
    adk_ipv6_vrd_handle_t ipv6VrdHandle;
    /*! ipv6VrdId provides the Id for the Routing Domain
     *  If ADK_IPV6_EGRESS_RULE_USE_VRD_ID is set in rulesMask, ipv6VrdId will be
     *  used. Else ipv6VrdHandle will be used.
     */
    uint32_t ipv6VrdId;
    /*! The source IPv6 address of the IPv6 header */
    adk_ipv6_addr_t srcIpv6Addr;
    /*! The destination IPv6 address of the IPv6 header */
    adk_ipv6_addr_t dstIpv6Addr;
    /*! The hop limit value of the IPv6 header */
    uint8_t hopLimit;
    /*! The dscp value */
    uint8_t dscp;
    /*! The flow label of the IPv6 header */
    uint32_t flowLabel;
    /*! The Next Header value */
    uint8_t nextHdr;
    /*! The MTU of this association */
    uint16_t pathMtu;
    /*! The source UDP port. */
    uint16_t srcUDPPort;
    /*! The destination UDP port. */
    uint16_t dstUDPPort;

    /*! The action flags indicating any special actions to take
     *  for this flow.
     *  Bit 0 - Encapsulation only
     *  Bit 1 - ISATAP tunneling
     *  Bit 2 - 6TO4 tunneling
     *  Bit 3 - Teredo tunneling
     *  Bit 4 - Link Local
     *  Bit 5 - ICMP Handling Exception (Add frag Hdr)
     *  Bit 6 - Create UDP header. It will be used 
     *          only when IPv6 header is not present.
     */
    uint8_t actionFlags;
} __attribute__ ((packed)) adk_ipv6_egress_association_attr_t;   /* ------  end of struct adk_ipv6_egress_association_attr_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_global_statistics_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The common statistics structure.
 *
 * This structure contains the statistics that do not belong
 * to any flow.
 *
 * =============================================================
 */
typedef struct adk_ipv6_common_statistics_s
{
    /*! Counts the PDUs for which the reassembly timer expired */
    uint32_t ipv6ReassemblyTimedoutPkts;

    /*! Counts the PDUs (locally terminated) for which no ingress
     * classifier rule was found.
     * This is equivalent to ipv6RmDroppedPacketsFiltering
     */
    uint32_t ipv6ClassifierDefaultDroppedPkts;

    /*! Counts the number of times the reassembly context
     *  allocation failed.
     */
    uint32_t ipv6ReassemblyError;

    /* Counts total number of incoming PDUs which are crossed
      * rate limiting value.
      * This is equivalent to ipv6RmDroppedPacketsRateLimiting
      */
    uint32_t ipv6PolicerDrop;

    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size */
    uint32_t ipv6TooBigPackets;

    /*! Counts the PDUs for which SIP lookup is failed
     * because of IP is not configured on any interface
     * */
    uint32_t ipv6OutSrcIpInvalid;

    /*! Counts the PDU's discarded because of no route found to 
     * transmit the packet 
     * */
    uint32_t ipv6RouteNotFoundPkts;

    /*! Counts the PDU's discarded because of no next hop
     *  entry present, next hop entry lookup failed.
     * */
    uint32_t ipv6NoNextHopEntryDrop;

    /*! Number of packets dropped because of error while allocating Mbuf for fragments */
    uint32_t ipv6MbufAllocFailures;

    /*! Number of reassembly timer create failures */
    uint32_t ipv6ReassTimerCreateFailures;

    /*! Number of packets dropped because of packets received on invalid interface */
    uint32_t ipv6InvalidIfFail;
    /*! Generic drop counter due to mismatched configuration */
    uint32_t ipv6GenericDroppedPkts;

} __attribute__((aligned(64)))adk_ipv6_global_statistics_t;   /* ------  end of struct adk_ipv6_global_statistics_t  ------ */

typedef adk_qos_ingress_stats_t adk_ipv6_ingress_qos_stats_t;

/*!
 * =============================================================
 * @struct adk_ipv6_interface_statistics_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The interface statistics structure.
 *
 * This structure contains interface specific statistics 
 *
 * =============================================================
 */
typedef struct adk_ipv6_interface_statistics_s
{
    /*! The number of IPv6 Multicast packets or fragments received at the IPv6
     *  network interface. Fragments are counted as individual packets.
     */
    uint64_t ipv6IfInMCastPackets;
    /*! The total number of bytes received in IPv6 Multicast packets or fragments
     *  at IPv6 network interface. Fragments are counted as individual packets.
     */
    uint64_t ipv6IfInMCastOctets;
    /*! The number of IPv6 Multicast packets or fragments transmitted at the IPv6
     *  network interface. Fragments are counted as individual packets.
     */
    uint64_t ipv6IfOutMCastPackets;
    /*! The total number of bytes  transmitted in IPv6 Multicast packets or fragments
     *  at the IPv6 network interface. Fragments are counted as individual packets.
     */
    uint64_t ipv6IfOutMCastOctets;
    /*! The total number of Ipv6 Multicast packets or fragments which are received
     *  on a particular interface are dropped due to invalid Multicast mac
     *  or invalid Multicast group.
     */
    uint32_t ipv6IfInDiscardedMCastPackets;
    /*! The total number of Ipv6 Multicast octets or fragments which are received 
     *  on a particular interface are dropped due to invalid Multicast mac
     *  or invalid Multicast group.
     */
    uint32_t ipv6IfInDiscardedMCastOctets;

    /*! Total number of received IPv6 packets. IPv6 fragments are
     * counted as individual packets. */
    uint64_t ipv6IfInPackets;

    /*! Total number of transmitted IPv6 packets. IPv6 fragments are
     * counted as individual packets. */
    uint64_t ipv6IfOutPackets;

    /*! Total number of received bytes. This counter is incremented before IPv6
     * reassembly (of IPv6 fragments). The counter is incremented using IPv6 total
     * length field in IPv6 header. For encrypted packets, IPv6 total length field
     * of outer IPv6 header is always used for incrementing this counter. */
    uint64_t ipv6IfInOctets;

    /*! Total number of transmitted bytes. This counter is incremented
     * after IPv6 fragmentation, QoS shaping and scheduling. */
    uint64_t ipv6IfOutOctets;

   /*! Total number of IPv6 packets that had to be fragmented */
    uint64_t ipv6IfOutFragPackets;

    /*! Total number of IPv6 fragments transmitted */
    uint64_t ipv6IfOutFragments;

    /*! Total number of received IPv6 Errors*/
    uint32_t ipv6IfInErrors;

   /*!Total number of output IPv6 packets that are discarded at an interface*/
    uint32_t ipv6IfOutDroppedPackets;

   /*!Total number of output datagram bytes that are discarded at an interface*/
    uint64_t ipv6IfOutDroppedOctets;

     /* Number of packets dropped because of fragment length is not multiples of eight */
     uint32_t ipv6IfNotMulOfEightFragLen;

    /* Number of packets dropped because of invalid Reassembly session id */
    uint32_t ipv6IfInvalidReasmSessionId;

    /* Number of packets dropped because of received reassembled packet offset 
     * overlapping previous fragment  offset
     */
    uint32_t ipv6IfReasmOverlapFrag;

    /* Number of packets dropped because of more fragments received
     * than expected fragment count 
     */
    uint32_t ipv6IfTooManyFragsDropCount;

    /* Number of packets dropped because received reassembled fragment
     * length is more than expected reassembly length
     */
    uint32_t ipv6IfExpectedReasmLengthExceed;

    /*! Number of non local packets dropped due to forwarding 
     * is not enabled
     */
    uint32_t ipv6IfForwardingDisableDrop;

    /*! Number of L4 header truncated PDU*/
    uint32_t  ipv6IfL4Hdrtruncated;

    /*! Number of locally terminating packets dropped because of
     *  carrying incorrect L4 Protocol checksum.
     */
    uint32_t ipv6IfUDPCksumErrors;

    /*! Number of locally terminating packets dropped because of
     *  carrying incorrect ICMP Protocol checksum.
     */
    uint32_t ipv6IfICMPCksumErrors;

    /*! Number of locally terminated packets dropped due to
     * unsupported protocol
     */
    uint32_t ipv6IfUnsupportedProtoDrops;

    /*! Number of received packets dropped, due to packets payload
     *  length got truncated.
     */
    uint32_t ipv6IfStatsInTruncatedPkts;

    /*! Number of packets dropped due to IPv6 QoS policy lookup failure
        or no default queue configured when QoS is disabled */
    uint32_t ipv6IfQoSLookupFailed;

    /*! Rx QoS Statistics for PHB */
    adk_ipv6_ingress_qos_stats_t ipv6ifRxQoSstats[ADK_QOS_MAX_PHB_CLASS_IDX];

} __attribute__((aligned(64)))adk_ipv6_interface_statistics_t;   /* ------  end of struct adk_ipv6_interface_statistics_t  ------ */


/*!
 * =============================================================
 * @struct adk_ipv6_address_stats_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * Structure for maintaining IPv6 address stats
 * =============================================================
 */
typedef struct adk_ipv6_address_stats_s {

    /*! total number of received octets */
    uint64_t ipv6InOctets;

    /*! total number of received packets */
    uint64_t ipv6InPackets;

    /*! total number of transmitted octets */
    uint64_t ipv6OutOctets;

    /*! total number of transmitted packets */
    uint64_t ipv6OutPackets;

    /*! total number of reassembled packets */
    uint64_t ipv6InReassembledPackets;

    /*! total number of received packets with errors */
    uint32_t ipv6InErrors;

} adk_ipv6_address_stats_t;

typedef struct adk_ipv6_l4counter_statistics_s
{
    /*! Total number of TCP protocol IPv6 packets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6TcpInPackets;
    /*! Total number of TCP protocol IPv6 octets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6TcpInOctets;
    /*! Total number of TCP protocol IPv6 packets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6TcpOutPackets;
    /*! Total number of TCP protocol IPv6 octets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6TcpOutOctets;
    /*! Total number of SCTP protocol IPv6 packets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6SctpInPackets;
    /*! Total number of SCTP protocol IPv6 octets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6SctpInOctets;
    /*! Total number of SCTP protocol IPv6 packets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6SctpOutPackets;
    /*! Total number of SCTP protocol IPv6 octets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6SctpOutOctets;
}adk_ipv6_l4counter_statistics_t;

/*!
 * =============================================================
 * @union adk_ipv6_exception_handler_fn_u adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The exception handler function information.
 *
 * This structure contains the MPP exception handler function
 * name and the CPU exception handler function information.
 *
 * =============================================================
 */
typedef union adk_ipv6_exception_handler_fn
{
        /*!
         * PH specific attribute, valid if the pduPath is
         * ADK_IPV6_EXCEPTION_LOC_CPU_PH.
         */
        adk_ipv6_ph_attr_t phAttr;
        /*!
         * ncaQid, NCA qid on which the packet is forwarded to
         * CPU. The value of this field depends on the ncaQids
         * configured in ACP configuration XML.
         * ADK does not perform any validation on this valid. It is up to
         * the application to provide appropriate value based on chip
         * configuration.
         */
#ifdef ADK_QOS_GEN3_INTERFACE
        adk_ipv6_nca_attr_t ncaAttr;
#else
        uint32_t ncaQid;
#endif
       /*! The name of the MPP exception function. */
    char *mppFn_p;
} adk_ipv6_exception_handler_fn_u;   /* ------  end of struct adk_ipv6_exception_handler_fn_u  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_exception_handler_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The exception handler information.
 *
 * This structure holds the information about the exception
 * handler for registration.
 *
 * =============================================================
 */
typedef struct adk_ipv6_exception_handler_s
{
    /*! The type of the exception for which the exception
     *  handler is being registered.
     */
    adk_ipv6_exception_type_t exceptionType;

    uint8_t devId;
    uint16_t queueId;
    uint8_t priority;
    adk_ph_dev_cbfn_handle_t cbFnHandle;
    adk_handle_t policer_handle;

} adk_ipv6_exception_handler_t;   /* ------  end of struct adk_ipv6_exception_handler_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_cpu_recv_param_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure defines the parameters passed by IPv6 data plane software to
 * CPU when a packet is sent to CPU via PHV1 due to exception or due to flow
 * configuration.
 *
 * =============================================================
 */
typedef struct adk_ipv6_cpu_recv_param_s
{
    /*! 
     * EIOA port number on which packet was received 
     */
    adk_port_number_t                 port;
    /*! 
     * exception type when packet is sent to CPU due to some exception. If
     * packet is not sent due to exception but because of some explicit flow
     * configuration, then this is set to 0xFF indicating no exception.
     */
    adk_ipv6_exception_type_t         exception_type;
    /*! 
     * the L2 interface ID on which the packet was received
     */
    uint32_t                    interface_id;
    /*!
     * the application specified 24-bit context id upon packet capture point configuration, if
     * packet is received due to explicit packet capture configuration. This is set to 0
     * if packet arrives due to some exception. 
     */
    uint32_t context;

    /*!
     * MTU value which resulted in post fragmentation is carried in Local PTB exception
     */
    uint32_t                    mtu;

     /*!
     * the 32-bit MSB of EIOA timestamp upon arrival of the packet in the 
     * system.
     */
    uint32_t                    timestamp_msb;
    /*!
     * the 32-bit LSB of EIOA timestamp upon arrival of the packet in the 
     * system.
     */
    uint32_t                    timestamp_lsb;
    /*!
     * the IPv6 header offset 
     */
    uint8_t                     ipv6_header_offset;
   /*!
     * the Vitual routing domain id (VRD-ID)
     */
    uint8_t                     vrd_id;
    /*!
     * The linux interface ID
     */
    uint16_t                    linux_intf_id;

    /*! Exception type related data
     */
     union adk_ipv6_exception_type_data{
     /*! 
     * Offset where extn header exception happened 
     */
        uint16_t		param_problem_offset; 

      }exceptionType_data_u;
} adk_ipv6_cpu_recv_param_t;

typedef struct adk_ipv6_udp_chksum_invalid_s
{
    uint16_t l4_offset;
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_udp_chksum_invalid_t;

typedef struct adk_ipv6_noroute_exception_param_s
{
    uint8_t routeTableId;
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_noroute_exception_param_t;


typedef struct adk_ipv6_reass_timer_exceeded_s
{
    uint32_t fragmentId;
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_reass_timer_exceeded_t;

typedef struct adk_ipv6_ext_hdr_twice_exception_data_s
{
    /*! Offset of next header value where repeat extension header encountered.
     * This is from start of IPv6 header */
   uint8_t repeatExtHdrNextHdrOffset;
    /*! Indicates repeated extension header type value */
   uint8_t nextHdrValue;
   adk_interface_handle_t ifaceHandle;
}adk_ipv6_ext_hdr_twice_exception_data_t;


typedef struct adk_ipv6_hop_limit_zero_exception_param_s {
    adk_interface_handle_t ifaceHandle;
} adk_ipv6_hop_limit_zero_exception_param_t;

typedef struct adk_ipv6_unsup_proto_exception_param_s
{
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_unsup_proto_exception_param_t;

typedef struct adk_ipv6_fwding_disabled_exception_param_s
{
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_fwding_disabled_exception_param_t;

typedef struct adk_ipv6_invalid_version_param_s
{
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_invalid_version_param_t;

typedef struct adk_ipv6_overlap_frag_param_s
{
    uint32_t fragmentId;
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_overlap_frag_param_t;

typedef struct adk_ipv6_mtu_violation_param_s
{
    uint16_t mtu;
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_mtu_violation_param_t;

typedef enum adk_ipv6_exception_parameter_problem_type_e
{
    /*! IPv6 hop by hop extension header is not the first extension header*/
    ADK_IPV6_HOP_BY_HOP_EXTENSION_NOT_FIRST_EXTENSION_HEADER = 0x1,
    /*! IPv6 hop by hop extension header encountered for non ICMPv6 packet */
    ADK_IPV6_HOP_BY_HOP_EXTENSION_HEADER_FOR_NON_ICMPV6_PROTOCOL = 0x2,
    /*! IPv6 header payload length field issue in reassembly */
    ADK_IPV6_HEADER_PAYLOAD_LENGTH_INCORRECT = 0x3,
    /*! IPv6 routing header has non-zero segments left */
    ADK_IPV6_EXCEPTION_SEGMENTS_LEFT_NON_ZERO = 0x4,
    /*! IPv6 Max Reassembly PDU size reached*/
    ADK_IPV6_REASSEMBLY_MAX_PDU_SIZE_REACHED = 0x5
} adk_ipv6_exception_parameter_problem_type_t;

typedef struct adk_ipv6_param_problem_exception_data_s
{
    /*! Offset of parameter problem in the packet. This offset
     * is from start of IPv6 header */
    uint8_t paramProblemOffset;
    /*! Indicates the parameter problem type */
    adk_ipv6_exception_parameter_problem_type_t paramProbType;
    /*! Interface Handle */
    adk_interface_handle_t ifaceHandle;
}adk_ipv6_param_problem_exception_data_t; /* end of struct adk_ipv6_param_problem_exception_data_t */


/*!
 * =============================================================
 * @struct adk_ipv6_cpu_recv_param_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure defines the parameters passed by IPv6 data plane software to
 * CPU when a packet is sent to CPU via PHV3 due to exception or due to flow
 * configuration.
 *
 * =============================================================
 */
typedef struct adk_ipv6_exception_task_param_s
{
    /*! Indicates the received exception type */
    adk_ipv6_exception_type_t       exception_type;

    /* rte mbuf structure pointer */
    struct rte_mbuf *mbuf;

   /*ipv6 header offset */
    uint16_t ipv6Offset;

    union
    {
        /*! L4_offset for exception type
         * ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID
         */
        adk_ipv6_udp_chksum_invalid_t udpChksumInvalid;
        /*! Route table Id for exception type
         * ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND and
         */
        adk_ipv6_noroute_exception_param_t  routeNotFound;
        /*! Fragment Id for exception type
         * ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED
         */
        adk_ipv6_reass_timer_exceeded_t  reassTimerExceed;
        /*! Information received for exception type
         * ADK_IPV6_EXCEPTION_PARAM_PROBLEM.
         */
        adk_ipv6_param_problem_exception_data_t paramProbData;
        /*! Information received for exception type
         * ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO
         */
         adk_ipv6_unsup_proto_exception_param_t unsupProto;
        /* Information received for exception type
         * ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO
         */
        adk_ipv6_hop_limit_zero_exception_param_t hopLimitZero;
        /* Information received for exception type
         * ADK_IPV6_INTERFACE_FORWARDING_DISABLE
         */
        adk_ipv6_fwding_disabled_exception_param_t fwdDisable;
        /* Information received for exception type
         * ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION
         */
        adk_ipv6_invalid_version_param_t  invalidVersion;
        /* ADK_IPV6_EXCEPTION_FRAG_OVERLAP */
        adk_ipv6_overlap_frag_param_t overlapFragment;
        /* ADK_IPV6_EXCEPTION_MTU_VIOLATION */
        adk_ipv6_mtu_violation_param_t mtuViolation;
        /* ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE */
        adk_ipv6_ext_hdr_twice_exception_data_t duplicateExtHeader;
    }exception_specific_data;

} adk_ipv6_exception_task_param_t;


/*
 * IP Filter (a.k.a access list) specific definitions. 
 */

/*!
 * =============================================================
 * @enum adk_ipv6_filter_rule_mask_t
 * 
 * @brief 
 * IPv6 filter rule mask definitions. This definition should be 
 * used to set rulesMask field in adk_ipv6_filter_rules_t data 
 * structure. When rule mask is set then value for that field
 * is treated as valid else it is treated as wildcard.
 *
 * =============================================================
 */
typedef enum adk_ipv6_filter_rule_mask_e {
    /*! IPv6 filter dscp */
    ADK_IPV6_FILTER_DSCP_MASK  = 0x0002,
    /*! This Mask is deprecated & will be removed in future
        IPv6 filter IPv6 flow label option */
    ADK_IPV6_FILTER_FLOW_LABEL_MASK  = 0x0004,
    /*! IPv6 filter destination address */
    ADK_IPV6_FILTER_DEST_ADDR_MASK  = 0x0008,
    /*! IPv6 filter source address */
    ADK_IPV6_FILTER_SRC_ADDR_MASK    = 0x0010,
    /*! This Mask is deprecated & will be removed in future
        IPv6 filter destination option type */
     ADK_IPV6_FILTER_DST_OPT_TYPE_MASK = 0x0020,
    /*! This Mask is deprecated & will be removed in future
        IPv6 filter routing type */
    ADK_IPV6_FILTER_ROUTING_TYPE_MASK      = 0x0040,
    /*! IPv6 filter IPv6 fragment option */
    ADK_IPV6_FILTER_FRAGMENT_MASK= 0x0080,
    /*! This Mask is deprecated & will be removed in future
        IPv6 Mobility type */
    ADK_IPV6_FILTER_MOBILITY_TYPE_MASK= 0x100,
    /*! IPv6 filter protocol type */
    ADK_IPV6_FILTER_PROTO_MASK= 0x200,
    /*! IPv6 filter proto info */
    ADK_IPV6_FILTER_PROTO_INFO_MASK = 0x400,

    /* ICMP */
    /*!ICMP message type  */
    IPV6_ICMP_TYPE_MASK        = 0x800,
    /*!ICMP message code */
    IPV6_ICMP_CODE_MASK        = 0x1000,


    /* TCP */
    /*!TCP source port */
    IPV6_TCP_SRC_PORT_MASK     = 0x800,
    /*!TCP destination port */
    IPV6_TCP_DEST_PORT_MASK    = 0x1000,

    /* UDP */
    /*!UDP source port */
    IPV6_UDP_SRC_PORT_MASK     = 0x800,
    /*!UDP destination port */
    IPV6_UDP_DEST_PORT_MASK    = 0x1000,

    /* SCTP */
    /*!SCTP source port */
    IPV6_SCTP_SRC_PORT_MASK    = 0x800,
    /*!SCTP destination port */
    IPV6_SCTP_DEST_PORT_MASK   = 0x1000
               
} adk_ipv6_filter_rule_mask_t;   /* ------  end of enum adk_ipv6_filter_rule_mask_t  ------ */
/*!
 * =============================================================
 * @enum adk_ipv6_filter_direction_t
 *
 * @brief 
 *
 * defines the direction in which filter is applied to an inteface.
 *
 * =============================================================
 */
typedef enum adk_ipv6_filter_direction_e {
    /*!< filter is applied during receive processing  */
    ADK_IPV6_FILTER_IN=1,
    /*!< filter is applied during transmit processing  */
    ADK_IPV6_FILTER_OUT
} adk_ipv6_filter_direction_t;   /* ------  end of enum adk_ipv6_filter_direction_t  ------ */


/*!
 * =============================================================
 * @enum adk_ipv6_direction_t
 *
 * @brief
 *
 * defines the direction in which profile id is applied to an inteface.
 *
 * =============================================================
 */
typedef enum adk_ipv6_direction_s {
    	ADK_IPV6_DIR_INGRESS = 0,
        ADK_IPV6_DIR_EGRESS,
        ADK_IPV6_DIR_MAX
} adk_ipv6_direction_t;



/*!
 * =============================================================
 * @enum adk_ipv6_protocol_t
 *
 * @brief 
 *
 * defines well known protocol fileds.
 *
 * =============================================================
 */
typedef enum adk_ipv6_protocol_e {
    /*! ICMPV6 protocol type */
    ADK_IPV6_PROTOCOL_ICMPV6=58,
    /*! TCP protocol type */
    ADK_IPV6_PROTOCOL_TCP=6,
    /*! UDP protocol type */
    ADK_IPV6_PROTOCOL_UDP=17,
    /*! ESP protocol type */
    ADK_IPV6_PROTOCOL_ESP=50,
    /*! AH protocol type */
    ADK_IPV6_PROTOCOL_AH=51,
    /*! SCTP protocol type */
    ADK_IPV6_PROTOCOL_SCTP=132,
    /*! UDP lite protocol type */
    ADK_IPV6_PROTOCOL_UDP_LITE=136
} adk_ipv6_protocol_t;   /* ------  end of enum adk_ipv6_protocol_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_icmpv6_filter_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * Defines ICMPv6 specific fields in the filter rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_icmpv6_filter_info_s{
    /*! ICMPv6 packet message type */
    adk_ipv6_range_t type;
    /*! ICMPv6 packet message code */
    adk_ipv6_range_t code;
} adk_ipv6_icmpv6_filter_info_t;   /* ----  end of struct adk_ipv6_icmpv6_filter_info_t  ---- */
/*!
 * =============================================================
 * @struct adk_ipv6_tcp_filter_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * Defines TCP specific fields in the filter rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_tcp_filter_info_s {
    /*! TCP source port number */
    adk_ipv6_range_t srcPort;
    /*! TCP destination port number */
    adk_ipv6_range_t destPort;
} adk_ipv6_tcp_filter_info_t;   /* ------  end of struct adk_ipv6_tcp_filter_info_t  ---- */

/*!
 * =============================================================
 * @struct adk_ipv6_udp_filter_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * Defines UDP specific fields in the filter rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_udp_filter_info_s {
    /*! UDP source port number */
    adk_ipv6_range_t srcPort;
    /*! UDP destination port number */
    adk_ipv6_range_t destPort;
} adk_ipv6_udp_filter_info_t;   /* ----  end of struct adk_ipv6_udp_filter_info_t  ---- */

/*!
 * =============================================================
 * @struct adk_ipv6_sctp_filter_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * Defines SCTP specific fields in the filter rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_sctp_filter_info_s {
    /*! SCTP source port number */
    adk_ipv6_range_t srcPort;
    /*! SCTP destination port number */
    adk_ipv6_range_t destPort;
} adk_ipv6_sctp_filter_info_t;   /* ----  end of struct adk_ipv6_sctp_filter_info_t  ---- */



/*!
 * =============================================================
 * @struct adk_ipv6_filter_rules_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure contains parameters that defines a filter rule. 
 *
 * =============================================================
 */
typedef struct adk_ipv6_filter_rules_s {
    /*!
     * Filter rule mask identifies the valid parameters in this
     * data structure.
     */
    adk_ipv6_filter_rule_mask_t ruleMask;
    /*!
     * IPv6 rule set id. filter rule is applied on this 
     * This parameter is mandatory for filter addition.
     */
    adk_ipv6_filter_rule_set_handle_t filterRuleSetHandle;
    /*!
     * identifies IPv6 protocol type. Some standard  protocol fields are
     * defined in enum adk_ipv6_protocol_t. User can specify one of these defined
     * value or any value between 0 and 255.
     */
    adk_ipv6_range_t protocol;
    /*!
     * destination IPv6 address. This field can be a fully specified
     * address too. This field is matched against the destion IPv6 address 
     * in IPv6 header.
     */
    adk_ipv6_addr_prefix_t dstIPv6Addr;
    /*!
     * source IPv6 address. This field can be a fully specified
     * address too. This field is matched against the source IPv6 address 
     * in IPv6 header.
     */
    adk_ipv6_addr_prefix_t srcIPv6Addr;
     /*!
     * This field identifies whether to apply the filter to only unfragmented
     * packets, only L4 protocol available packets, only fragmented packets, 
     * only first fragment (fragoffset = 0)
     * or only non-first fragment (fragoffset != 0).
     */
    adk_ipv6_frag_t frag_type;
    /*!
     * The value provided in the dscp field is matched against the dscp
     * field in the IPv6 header.
     */
    adk_ipv6_range_t dscp;
    /*! This field is deprecated & will be removed in future
     * The value provided in the flowLablel field is matched against the Flow Lable
     * field in the IPv6 header.
     */
    adk_ipv6_range_t flowLabel;
    /*! This field is deprecated & will be removed in future
     * The value provided in the dstOptionType field is matched against field from
     * extension header destination option.
     */
    adk_ipv6_range_t dstOptionType;
    /*! This field is deprecated & will be removed in future
     * The value provided in the routingType field is matched against field from
     * extension header routing header.
     */
    adk_ipv6_range_t routingType;
    /*! This field is deprecated & will be removed in future
     * The value provided in the mobilityType field is matched against field from
     * extension header mobility header. 
     */
    adk_ipv6_range_t mobilityType;

    /*!
     * protocol specific filter info. Only protocols defined in adk_ipv6_protocol_t
     * enum is supported now.
     */
    union adk_ipv6_proto {
        /*! icmpv6 specific filter fields */
        adk_ipv6_icmpv6_filter_info_t icmpv6;
        /*! tcp specific filter fields */
        adk_ipv6_tcp_filter_info_t  tcp;
        /*! udp specific filter fields */
        adk_ipv6_udp_filter_info_t  udp;
        /*! sctp specific filter fields */
        adk_ipv6_sctp_filter_info_t sctp;
    } protoInfo_u;
} adk_ipv6_filter_rules_t;   /* ----  end of struct adk_ipv6_filter_rules_t  ---- */

/*!
 * =============================================================
 * @struct adk_ipv6_filter_rule_stats_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * Statistics for an IPv6 filter rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_filter_rule_stats_s {
    /*! total number of packets that matched the rule */
    uint32_t matchedPkts;
} adk_ipv6_filter_rule_stats_t;

/*!
 * =============================================================
 * @enum adk_ipv6_filter_action_type_t
 *
 * @brief 
 *
 * IP filtering action types. 
 *
 * =============================================================
 */
typedef enum adk_ipv6_filter_action_type_e {
    /*!< Drop this packet immediately. A global stats counter will
     * be incremented by default. ACL specific counter will be 
     * incremented if enabled using enableCounter field in 
     * adk_ipv6_filter_action_t data structure.*/
    ADK_IPV6_FILTER_ACTION_DENY = 1,
    /*!< Forward to another interface or an upper layer
     * module. Global counter and optional per ACL counters are updated
     * before forwarding.
     */
    ADK_IPV6_FILTER_ACTION_ACCEPT,
    /*!< Forward to a program running on host processor. Appropriate
     * counters are incrememnted before forwarding the packet to CPU.
     * This ACTION is not a requirement from customer & hence not supporting
     */
    ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU
} adk_ipv6_filter_action_type_t;   /* ------  end of enum adk_ipv6_filter_action_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_ipv6_processing_path_t
 *
 * @brief 
 *
 * Defines the supported IPv6 processing locations.
 *
 * =============================================================
 */ 
typedef enum adk_ipv6_processing_path_e {
    /*! IPv6 Packet processing path = CPU via PDU handler*/
    ADK_IPV6_PROCESSING_PATH_CPU_PH=1,
    /*! IPv6 Packet processing path = CPU via non PDU handler path*/
#ifdef ADK_QOS_GEN3_INTERFACE
    ADK_IPV6_PROCESSING_PATH_CPU_SP_STACK=2,
#else
    ADK_IPV6_PROCESSING_PATH_CPU_NOPH=2,
#endif
    /*! IPv6 packet processing path = MPP 
     * This Path is not valid in Gen3 */
    ADK_IPV6_PROCESSING_PATH_MPP
} adk_ipv6_processing_path_t;



/*!
 * =============================================================
 * @struct adk_ipv6_filter_action_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * IPv6 filter action. This action (set of actions) is applied to 
 * the packet when the packet fields match the configured filter
 * rule.
 *
 * =============================================================
 */
typedef struct adk_ipv6_filter_action_s {
    /*! action->  ADK_IPV6_FILTER_ACTION_DENY/ACCEPT/FORWARD_TO_CPU*/
    adk_ipv6_filter_action_type_t action;
    /*! determines whether to increment counter on filter match */
    bool              enableCounter;
    /*! This field is deprecated & will be removed in future
        determines whether to enable QOS policing on filter match */
    bool              enablePolicing;
    /*! This field is deprecated & will be removed in future
     * policingHandle, This field determines the policing profile
     * handle. Relevant only if enablePolicing flag is set to
     * TRUE. TODO. We should change this based on how policing
     * profile creation evolves.
     */
    adk_ipv6_qos_policing_flow_handle_t policingHandle;
    
    /*!
     * cpuPath, determines the path taken by the PDU to reach
     * CPU. possible values are ADK_IPV6_PROCESSING_PATH_CPU_PH or
     * ADK_IPV6_PROCESSING_PATH_CPU_SP_STACK.
     */
    adk_ipv6_processing_path_t cpuPath;
    union
    {
        /*!
         * phAttr, defines the PDU handler specific attributes
         * if cpuPath is ADK_IPV6_PROCESSING_PATH_CPU_PH.
         */
        adk_ipv6_ph_attr_t phAttr;
        /*!
         * ncaQid, defines the NCA qid to which packet is forwarded.
         * Pipeline used to transfer PDU ph_mpp_cpu.
         */
        uint32_t ncaQid;
    } cpuPath_u;
    /*
     * Optional MTM Queue to be used if packet is being sent to CPU via PH or
     * NCA. function. If set to 0, then this MTM configuration is not used.
     */
    adk_handle_t mtmQueueHandle;
    /*! This field is deprecated & will be removed in future
     * enableDscpMapping. This field is of type bool and determines
     * whether the we have to mark the packets that matches this
     * ACL with a new DSCP value. possible values TRUE or FALSE.
     */
    bool  enableDscpMapping;
    /*! This field is deprecated & will be removed in future
     * newDscp. This field defines the new value of DSCP field.
     * This field is meaningful only if enableDscpMapping flag is set to
     * TRUE. newDscp field replaces the DSCP bits in the Traffic Class field.
     */
    uint8_t newDscp;

} adk_ipv6_filter_action_t;   /* --  end of struct adk_ipv6_filter_action_t  --- */

/*!
 * =============================================================
 * @struct adk_ipv6_filter_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure contains parameters that defines a filter rule
 * and action associated with the filter. This structure is used
 * to get the filter rule list
 *
 * =============================================================
 */
typedef struct adk_ipv6_filter_s {
    adk_ipv6_filter_rules_t  rules;
    adk_ipv6_filter_action_t action;
}adk_ipv6_filter_t;    /* ----  end of struct adk_ipv6_filter_t ---- */
/*!
 * =============================================================
 * @enum adk_ipv6_filter_action_type_t
 *
 * @brief 
 *
 * IPv6 address validation  action types. 
 *
 * =============================================================
 */
typedef enum adk_ipv6_addr_val_flag_type_e{

    /*!< flag to indicate that input is ipv6 source address
     */ 
     ADK_IPV6_ADDRESS_SOURCE= 0,
    /*!< flag to indicate that input is ipv6 destination address
     */ 
     ADK_IPV6_ADDRESS_DEST= 1
}adk_ipv6_addr_val_flag_type_t;  /* ------  end of enum adk_ipv6_addr_val_flag_type_t  ------ */
/*!
 * =============================================================
 * @enum adk_ipv6_filter_action_type_t
 *
 * @brief 
 *
 * IPv6 address validation  action types. 
 *
 * =============================================================
 */
typedef enum adk_ipv6_addr_val_action_type_e{

     /*!< Allow this packet for forwarding*/
    ADK_IPV6_ACTION_ALLOW = 0,
     /*!< Drop this packet immediately.*/
    ADK_IPV6_ACTION_DROP = 2,
    /*!< Drop this packet immediately and count */
    ADK_IPV6_ACTION_COUNT_AND_DROP= 4,
    /*!< Forward to a program running on host processor.
     *   This packet is not counted
     */
    ADK_IPV6_ACTION_FORWARD_TO_CPU=6
}adk_ipv6_addr_val_action_type_t;   /* ------  end of enum adk_ipv6_addr_val_action_type_t  ------ */


/*!
 * =============================================================
 * @struct adk_ipv6_address_val_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief 
 *
 * This structure contains parameters for ipv6 address validation. 
 *
 * =============================================================
 */
typedef struct adk_ipv6_address_val_info_s {

    /*! ipv6VrdfHandle provides the handle for the Routing Domain.
     */
    adk_ipv6_vrd_handle_t ipv6VrdHandle;
   /*!ipv6_address_flag Flag to indicate if the address being
    * passed is src address or destination address
    */
    adk_ipv6_addr_val_flag_type_t ipv6_address_flag;
    /*
     applyOnlyToForwarded - Flag indicating whether the source address validation 
     result will be applied only to forwarding traffic or not
     This flag is not applicable for destination ip address 
     */
     bool applyOnlyToForwarded;
  /*!
     * ipv6AddrPrefix gives ipv6 address and prefix. Using the prefix length, 
     * the ipv6address can be fully or partially specified. For example, 
     * specifying prefix length as 0, IPv6 address is 
     * completely wildcarded
     */
    adk_ipv6_addr_prefix_t ipv6AddrPrefix;

   /*!
     * action. ADK_IPV6_ACTION_DENY/ACCEPT/FORWARD_TO_CPU/ALLOW
     */
      adk_ipv6_addr_val_action_type_t action;
   /*!
     * cpuPath, determines the path taken by the PDU to reach
     * CPU. possible values are ADK_IPV6_PROCESSING_PATH_CPU_PH or
     * ADK_IPV6_PROCESSING_PATH_CPU_SP_STACK.
     */
    adk_ipv6_processing_path_t cpuPath;

union
    {
        /*!
         * phAttr, defines the PDU handler specific attributes
         * if cpuPath is ADK_IPV6_PROCESSING_PATH_CPU_PH.
         */
        adk_ipv6_ph_attr_t phAttr;
        /*!
         * ncaQid, defines the NCA qid to which packet is forwarded.
         * Pipeline used to transfer PDU ph_mpp_cpu.
         */
#ifdef ADK_QOS_GEN3_INTERFACE
        adk_ipv6_nca_attr_t ncaAttr;
#else
        uint32_t ncaQid;
#endif
    } cpuPath_u;
      
  /*
     * Optional MTM Queue to be used if packet is being sent to CPU via PH or
     * NCA. function. If set to 0, then this MTM configuration is not used.
     */
#ifndef ADK_QOS_GEN3_INTERFACE
    adk_handle_t mtmQueueHandle;
#else
    adk_handle_t rl_queue_handle;
#endif

}adk_ipv6_address_val_info_t;
/*!
 * adk_ipv6_acl_rule_handle_t
 * IPv6 ACL rule handle definition.
 */
typedef adk_flow_handle_t adk_ipv6_acl_rule_handle_t;

/*!
 * adk_ipv6_qos_police_attr_t
 * IPv6 QOS policy attributes, these bits define the nature of policing
 */
typedef struct adk_ipv6_qos_police_attr_s {
    
    uint8_t policeMech:2;
    uint8_t colorAware:1;
    uint8_t useLocally:1;
    uint8_t markDSCP:1;
    uint8_t markTOS:1;
    uint8_t reserved:2;
    
}adk_ipv6_qos_police_attr_t;

/*!
 * =============================================================
 * @enum adk_ipv6_qos_flow_handle_type_t
 *
 * @brief
 *
 * IPv6 Qos flow handle type. The type determines whether QoS
 * profile is applied with ingress flow OR ACL rule.
 *
 * =============================================================
 */
typedef enum adk_ipv6_qos_flow_handle_type_e {
    IPV6_QOS_INGRESS_FLOW_HANDLE = 1,
    IPV6_QOS_ACL_RULE_FLOW_HANDLE
} adk_ipv6_qos_flow_handle_type_t;   /* ------  end of enum adk_ipv6_qos_flow_handle_type_t  ------ */

/*!
* =============================================================
* @struct adk_ipv6_qos_classifier_rule_t
*
* @brief
*
* This data structure contains IPv6 Egress QoS lookup parameters
*
* =============================================================
*/
typedef struct adk_ipv6_qos_classifier_rule_s {
    /*!
     * QoS classifier rule mask. It identifies the valid
     * parameters in this data structure.
     */
    adk_qos_classifier_field_mask_t ruleMask;

   /*!
    * source IP address. This field can be IPv4 or IPv6 address.
    * This field can be a fully specified address or prefix.
    * This field is matched against the source IP address
    * in IPv4 or IPv6 header.
    */
    adk_ip_address_t srcAddr;

   /*!
    * destination IP address. This field can be IPv4 or IPv6 address.
    * This field can be a fully specified address or prefix.
    * This field is matched against the destination IP address
    * in IPv4 or IPv6 header.
    */
    adk_ip_address_t destAddr;
    /*!
     * 8-bit IP protocol type field.
     */
    uint8_t protocol;

    /*!
     * DSCP value for the packet. This value
     * can come from the packet header or from the mapping
     * policies that the user set up for this type of packet.
     * Note: It is expected in lower 6 bits.
     */
    uint8_t dscp;
} adk_ipv6_qos_classifier_rule_t;   /* ------  end of struct adk_ipv6_qos_classifier_rule_t ------ */

/*!
* =============================================================
* @struct adk_ipv6_qos_classifier_action_t
*
* @brief
*
* This data structure contains IPv6 Egress QoS action parameters
*
* =============================================================
*/
typedef struct adk_ipv6_qos_classifier_action_s {
    /*!
     * queueId for the packets.
     * User is expected to program the queue before calling this API.
     */
    adk_qos_queue_id_t queueId;
    /*!
     * Packet color for the packets matching the rule.
     * User is expected to program this value before calling this API.
     * If queue CMAN type is not WRED then this value is ignored. 
     */
     enum rte_tm_color pktColor;
} adk_ipv6_qos_classifier_action_t;   /* ------  end of struct adk_ipv6_qos_classifier_action_t------ */

/*!
* =============================================================
* @struct adk_ipv6_qos_info_t
*
* @brief
*
* This data structure contains IPv6 Egress QoS related parameters
*
* =============================================================
*/
typedef struct adk_ipv6_qos_info_s {
    /*QoS Classifier rule parameters*/
    adk_ipv6_qos_classifier_rule_t qosRule;
    /*QoS Classifier action parameters*/
    adk_ipv6_qos_classifier_action_t qosAction;
} adk_ipv6_qos_info_t;   /* ------  end of struct adk_ipv6_qos_info_t  ------ */

/*!
 * =============================================================
 * @struct adk_ipv6_diffserv_map_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * Diffserv (DSCP) mapping table
 *
 * =============================================================
 */
typedef struct adk_ipv6_diffserv_map_s {
    uint8_t inDscp;
    uint8_t mappedDscp;
} adk_ipv6_diffserv_map_t;   /* ------  end of struct adk_ipv6_diffserv_map_t  ------ */


/*!
* =============================================================
* @enum adk_ipv6_multicast_group_action_t
* @brief
* This enum defines the supported actions for a given multicast group
*
* =============================================================
*/
typedef enum adk_ipv6_multicast_group_action_e {
    /*! Indicates group for locally terminated packets */
    ADK_IPV6_MCGROUP_ACTION_ISLOCAL = 1,
    ADK_IPV6_MCGROUP_ACTION_MAX,
} adk_ipv6_multicast_group_action_t;

/*!
 * =============================================================
 * @struct adk_ipv6_multicast_group_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create IPv6 multicast group
 *
 * =============================================================
 */
typedef struct adk_ipv6_multicast_group_s {

    /*! IPv6 multicast group destination address */
    adk_ipv6_addr_t   dstAddr;
    /*! IPv6 source address */
    adk_ipv6_addr_t   srcAddr;
    /*! Indicates if IPv6 source address is to be
     *  used in multicast group entry
     */
    bool       useSrcAddr;
    /*! Incoming Layer-2 interface handle. */
    adk_interface_handle_t l2InterfaceHandle;
    /*! multicast group action */
    adk_ipv6_multicast_group_action_t  mcGrpAction;
 
}adk_ipv6_multicast_group_t; /* -- end of struct adk_ipv6_multicast_group_t - */



/*!
 * =============================================================
 * @struct adk_ipv6_multicast_flow_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create IPv6 multicast flow
 *
 * =============================================================
 */
typedef struct adk_ipv6_multicast_flow_s {
    /*! IPv6 multicast group handle */
    adk_flow_handle_t mcGrpHdl;
    /*! MTM Queue Handle */
    adk_handle_t queueHandle;
    /*! PH Handle to be used for sending to CPU */
    adk_ph_handle_t phHandle;
    /*! NCA Q id to be used for sending to CPU */
    uint32_t    ncaQid;
    /*! Outgoing interface handle */
    adk_interface_handle_t intHandle;
    /*! Call back function Id */
    uint8_t     cbFnId;
    /*! If set, indiactes packet needs to be sent to CPU */
    uint8_t     sendToCPU:1;
    /*! If set, indiactes packet needs to be sent to ethern module */
    uint8_t     sendToEth:1;
    /*! If set, indiactes IPv6 header not present, egFlowHandle 
     * will be used for such a case */
    uint8_t     hdrNotPrsnt:1;
    /*! If set, indicates that reassembly will take place 
     * for fragmented PDU's. It will be done only for non
     * sendToEth PDU's */
    uint8_t     allowReassembly:1;
    /*! If set, indiactes packet needs to be sent to CPU using PH */
    uint8_t     usePH:1;
    /*! If set, indiactes packet needs to be sent to CPU 
        ignoring the checksum validation */
    uint8_t     ignoreChecksum:1;
    /*! Not getting used as of now */
    uint8_t     unused:2;
}adk_ipv6_multicast_flow_t; /* -- end of struct adk_ipv6_multicast_flow_t - */

/*!
 * =============================================================
 * @struct adk_ipv6_multicast_group_stat_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains multicast group statistics
 *
 * =============================================================
 */
typedef struct adk_ipv6_multicast_group_stat_s {
    /*! total ingress bytes receive */
    uint32_t      byteLLIRcvCount;
    /*! total ingress packet receive */
    uint32_t      pktLLIRcvCount;
    /*! total egress bytes receive */
    uint32_t      byteULIRcvCount;
    /*! total egress packet receive */
    uint32_t      pktULIRcvCount;
    /*! total forwarded bytes */
    uint32_t      byteFwdCount;
    /*! total forwarded packets */
    uint32_t      pktFwdCount;
    /*! total bytes sent to CPU using PH */
    uint32_t      byteSendToPHCount;
    /*! total packets sent to CPU using PH */
    uint32_t      pktSendToPHCount;

}adk_ipv6_multicast_group_stat_t; /* -- end of struct adk_ipv6_multicast_group_stat_t - */

/*!
 * =============================================================
 * @struct adk_ipv6_interface_ingress_dscp_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to config qos for IP interface.
 *
 * =============================================================
 */
typedef struct adk_ipv6_interface_ingress_dscp_s {
    /*!
     * newDscp. This field defines the new value of DSCP field.
     * This field is meaningful only if trustedPort flag is set to
     * FALSE or enableRemarking is set to true. 
     * newDscp field replaces the DSCP bits in the Traffic Class field.
     */
    uint8_t newDscp;
    /*!
     * incomingDscp. This field defines the incoming value of DSCP field.
     * This field is meaningful only if enableRemarking is set to true.
     */
    uint8_t incomingDscp;
    /*!
     * isIncomingDscpValid. This field defines whether the incoming DSCP 
     * must be considered for DSCP Remarking
     */
    bool incomingDscpValid;
} adk_ipv6_interface_ingress_dscp_t;

/*!
 *=============================================================
 *@enum adk_ipv6_interface_attr_mask_t "adk_ipv6.h"
 * @brief
 * This enumeration defines the supported attributes of an IPv6 interface 
 */
typedef enum adk_ipv6_interface_attr_mask_s {
	/*!< To enable IP QoS */
    ADK_IPV6_INTERFACE_QOS_ENABLE = 0x01,
	/*!< To enable Forwarding for an IPv6 interface */
    ADK_IPV6_INTERFACE_FORWARDING_ENABLE = 0x02
}adk_ipv6_interface_attr_mask_t;

/*!
 * =============================================================
 * @struct adk_ipv6_interface_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create IP interface.
 *
 * =============================================================
 */
typedef struct adk_ipv6_interface_s {
    /*! 
     *  Layer-2 interface handle. E.g. This could be ethernet
     *  interface handle or VLAN based sub-interface handle.
     */
    adk_interface_handle_t layer2InterfaceHandle;
    
    /*! 
     * Address prefix for this IP interface. This is an Optional Param*/
    adk_ipv6_addr_prefix_t ipv6AddrPrefix;
    /*!
     * Interface MTU value. The range of values supported for MTU
     * configuration is between 1280 and 65,535 bytes. If specified as 0,
     * then default MTU value (1280) is used.
     */
    uint32_t mtu;
    /*Attributes of this Interface*/
    adk_ipv6_interface_attr_mask_t mask; 
 } adk_ipv6_interface_t;

/*!
 * =============================================================
 * @struct adk_ipv6_virtual_interface_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure contains parameters that are required
 * to create virtual IP interface.
 *
 * =============================================================
 */
typedef struct adk_ipv6_virtual_interface_s {
    
    /*! 
     *  For future use. 
     *  Virtual Routing Instance handle, to which this interface
     *  is associated. Traffic received from this interface will be 
     *  routed with in the given routing domain.
     */
    adk_ipv6_vrd_handle_t ipv6VrdHandle;
    /*! 
     * Address prefix for this IP interface. This will add entry
     * to the routing table for locally terminating traffic */
    adk_ipv6_addr_prefix_t ipv6AddrPrefix;
    /*! The ingress association lookup for locally destined or locally
     * terminated packets. Note that, all packets which are locally destined
     * (i.e. having destination IP address same as the interface address above)
     * are always subject to ingress association lookup. With this association
     * mask, appropriate fields can be selected for association lookup for
     * such packets. The selected fields can still be wild carded by specifying
     * them as 0 while creating the ingress association flow. This field,
     * if passed as 0, then default mask equal to ADK_IPV6_INGRESS_ASSOC_PROTOCOL_MASK
     * is used
     * Link Local IP address will be shared among multiple interfaces.
     * In that case, this field will be overwritten with the latest 
     * created interface information
     */
    uint16_t ingressAssociationRulesMask;
    /*!
     * Flag to enable re-assmebly for locally destined packets 
     * Link Local IP address will be shared among multiple interfaces.
     * In that case, this flag will be overwritten with the latest 
     * created interface information
     */
    bool enableReassembly;
} adk_ipv6_virtual_interface_t;
/*!
 * =============================================================
 * @struct adk_ipv6_interface_ingress_policing_attr_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This data structure is used for configuring IPv6 interface level 
 * ingress policing policy. This consists of a flag to enable or 
 * disable policing and the policing flow handle (when policing is 
 * enabled).
 *
 * =============================================================
 */
typedef struct adk_ipv6_interface_ingress_policing_attr_s {
    /*!
     * enablePolicing. This enable/disable policing of traffic received on
     * this interface. Possible values are TRUE or FALSE.
     */
    bool enablePolicing;
    /*!
     * policingHandle. This configures the policing flow handle
     * to be used for policing traffic received on this interface. This is
     * valid only if enablePolicing is set to TRUE. Application must create
     * the policing flow using adk_ip_qos_policing_flow_create API before
     * enabling policing for an interface.
     */
     adk_ipv6_qos_policing_flow_handle_t policingHandle;
} adk_ipv6_interface_ingress_policing_attr_t;

typedef adk_qos_ingress_info_t adk_ipv6_ingress_qos_info_t;

/*!
 * =============================================================
 * @enum adk_ipv6_table_group_t
 *
 * @brief
 *
 * This enumeration defines different group of tables that can be
 * dumped in user readable csv format.
 *
 * =============================================================
 */
typedef enum adk_ipv6_table_group_s {
    /*! Non inclusive lower bound on table group type */
    ADK_IPV6_TABLES_NONE = 0,
    /*! Interface related tables */
    ADK_IPV6_TABLES_INTF_ATTR = (1 << 0),
    /*! Routing related table */
    ADK_IPV6_TABLES_ROUTING = (1 << 1),
    /*! Ingress association flow specific tables */
    ADK_IPV6_TABLES_INGRESS_ASSOC = (1 << 2),
    /*! Next hop egress flows specific tables */
    ADK_IPV6_TABLES_NEXT_HOP = (1 << 3),
    /*! Egress association flow specific tables */
    ADK_IPV6_TABLES_EGRESS_ASSOC = (1 << 4),
    /*! IPV6 tunnel  related tables */
    ADK_IPV6_TABLES_TUNNEL = (1 << 5),
    /*! IPV6 routing domain related tables */
    ADK_IPV6_TABLES_VRD = (1 << 6),
    /*! IPV6 flow binding related tables */
    ADK_IPV6_TABLES_BIND = (1 << 7),
    /*! IPV6 QoS related tables */
    ADK_IPV6_TABLES_QOS = (1 << 8),
    /*! IPV6 exceptions related tables */
    ADK_IPV6_TABLES_EXCEPTIONS = (1 << 9),
    /*! IPV6 filters related tables */
    ADK_IPV6_TABLES_FILTERS = (1 << 10),
    /*! IPV6 Multicast related tables */
    ADK_IPV6_TABLES_MULTICAST = (1 << 11),
    /*! IPV6 filters related tables */
    ADK_IPV6_TABLES_DEST_MTU = (1 << 12),
    /*! IPV6 Reassembly related tables */
    ADK_IPV6_TABLES_REASSEMBLY = (1 << 13),
    /*! IPV6 Flow cache tables dumps */
    ADK_IPV6_TABLES_FLOWCACHE = (1 << 14),
    /*! Inclusive upper bound on table group type for all tables */
    ADK_IPV6_TABLES_ALL = ((1 << 15) - 1)
} adk_ipv6_table_group_t;

typedef struct adk_ipv6_tree_input_s
{
  char           Name[40];
  uint32_t   Size;
  uint32_t   Offset;

}adk_ipv6_tree_input_t;


typedef struct adk_ipv6_tree_output_s
{
   char           Name[30];
   uint32_t   Type;
   uint32_t   Size;
   uint32_t   Offset;

}adk_ipv6_tree_output_t;


typedef struct adk_ipv6_tree_conf_s
{  
   char           treeName[50];
   uint32_t   treeType;
   adk_ipv6_table_group_t   group;
   bool     defaultEntryPrinted;
   uint32_t   defaultDataBit;
   bool     splDftEntPrinted;

   uint16_t   inputCount;
   uint32_t   totalInputSize;
   uint32_t   outputRCount;
   uint32_t   outputVCount;
  
   adk_ipv6_tree_input_t    inputElement[ADK_IPV6_TREE_INPUT_COUNT_MAX];
   adk_ipv6_tree_output_t   outputRElement[ADK_IPV6_TREE_REAL_OUTPUT_COUNT_MAX];
   adk_ipv6_tree_output_t   outputVElement[ADK_IPV6_TREE_VIRTUAL_OUTPUT_COUNT_MAX];

}adk_ipv6_tree_conf_t; 


typedef struct adk_ipv6_packet_capture_info_s
{
    adk_interface_handle_t   ipv6IfHandle;
    adk_ipv6_direction_t     direction;
    adk_handle_t             mtmQueueHandle;
    adk_ipv6_ph_attr_t       phAttr;
    uint32_t                 context;
}adk_ipv6_pktcap_info_t;

typedef struct adk_ipv6_pktcap_status_s
{
    bool is_enabled;
} adk_ipv6_pktcap_status_t;


typedef struct adk_ipv6_ingress_classifier_stats_s {
   /*! total number of packets received from lower layer */
   uint32_t rxPkts;
   /*! total number of packets transmitted to upper layer or IPv6 ULI*/
   uint32_t txPkts;
   /*! total number of packets dropped due to policer
    *This is equivalent to ipRmDroppedPacketsRateLimitingPerRule
    */
   uint32_t ipv6PolicerDroppedPkts;
   /*! Total number of packets dropped for which ingress
     * classifier rule with DROP rule was found.
     */
   uint32_t ipv6ClassifierDroppedPkts;

} adk_ipv6_ingress_classifier_stats_t;

/*! Array to store the hos rule Ids for all protocols*/
extern adk_hos_rule_id_t adk_ipv6_supp_proto_ruleids[ADK_IPV6_MAX_SUPPORTED_PROTOCOLS];

/*!
 * @addtogroup _ipv6Functions  IPv6 Functions
 * @{
 */
/*!
 * The API function declarations.
 */
adk_status_t adk_ipv6_init (adk_module_handle_t ipv6Handle);

adk_dp_status_t adk_ipv6_dp_init (
                void);

adk_status_t adk_ipv6_dp_fn_id_init(void);

adk_status_t adk_ipv6_onlink_destination_add (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_onlink_destination_info_t *onLinkDestinationInfo_p,
    adk_flow_handle_t *onLinkDestinationHandle_p);

adk_status_t adk_ipv6_onlink_destination_modify (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_onlink_destination_attr_t *onLinkDestinationAttr_p,
    adk_flow_handle_t onLinkDestinationHandle);

adk_status_t adk_ipv6_onlink_destination_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t onLinkDestinationHandle);

adk_status_t adk_ipv6_next_hop_add (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_next_hop_attr_t *nextHopAttr_p,
    adk_flow_handle_t *nextHopHandle_p);

adk_status_t adk_ipv6_next_hop_modify (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_next_hop_attr_t *nextHopAttr_p,
    adk_flow_handle_t nextHopHandle);

adk_status_t adk_ipv6_next_hop_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t nextHopHandle);

adk_status_t adk_ipv6_route_add (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_route_info_t *routeInfo_p,
    uint8_t routeTableId);

adk_status_t adk_ipv6_route_delete (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_route_rules_t ipv6RouteRules,
    uint8_t routeTableId);

adk_status_t adk_ipv6_get_routes_all (
    adk_module_handle_t    ipv6Handle,
    uint8_t                ipv6RouteTblId,
    uint32_t               *numIpv6Routes,
    adk_ipv6_route_info_t  **ipv6Route_p);

adk_status_t adk_ipv6_log_level_set(
    adk_module_handle_t ipv6Handle,
    adk_log_level_t log_level);

adk_status_t adk_ipv6_exception_handler_register (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_exception_handler_t *exceptionHandler_p);

adk_status_t adk_ipv6_exception_handler_unregister (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_exception_handler_t *exceptionHandler_p);

adk_status_t adk_ipv6_global_statistics_get (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_global_statistics_t *ipv6GlobalStats_p);

adk_status_t adk_ipv6_address_stats_get (
		adk_module_handle_t ipv6Handle,
		adk_ipv6_addr_prefix_t ipv6Address,
		adk_ipv6_address_stats_t *stats_p);

adk_status_t adk_ipv6_l4counter_statistics_get (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_l4counter_statistics_t *ipv6L4CounterStats_p);



#ifdef ADK_DEBUG
adk_status_t
adk_ipv6_global_statistics_reset (
    adk_module_handle_t ipv6Handle);
#endif
adk_status_t
adk_ipv6_address_validate_add(
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_address_val_info_t *ipv6AddressValInfo_p
    );
adk_status_t
adk_ipv6_address_validate_remove(
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_address_val_info_t *ipv6AddressValInfo_p
    );
adk_status_t adk_ipv6_interface_filter_rule_set_map(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    adk_ipv6_filter_direction_t direction,
    adk_ipv6_filter_rule_set_handle_t filterRuleSetHandle);

adk_status_t  
adk_ipv6_get_num_pbr_rules ( 
        adk_module_handle_t __attribute__ ((unused))ipv6Handle,
        adk_ipv6_pbr_policy_set_handle_t __attribute__ ((unused))pbrRuleSetHandle,
        uint32_t __attribute__ ((unused))*numOfRules_p);

adk_status_t  
adk_ipv6_get_pbr_rules ( 
        adk_module_handle_t __attribute__ ((unused))ipv6Handle,
        adk_ipv6_pbr_policy_set_handle_t __attribute__ ((unused))pbrRuleSetHandle,
        uint32_t __attribute__ ((unused))numOfRules,
        adk_ipv6_pbr_info_t __attribute__ ((unused))*pbr_enty_p);

adk_status_t 
adk_ipv6_interface_statistics_get (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t interfaceHandle,
        adk_ipv6_interface_statistics_t *ipv6InterfaceStats_p);

#ifdef ADK_DEBUG
adk_status_t 
adk_ipv6_interface_statistics_reset ( 
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t interfaceHandle);
#endif

adk_status_t adk_ipv6_qos_policy_add(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t ruleIndex,
        const adk_ipv6_qos_info_t *qosInfo_p);

adk_status_t adk_ipv6_qos_policy_delete(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t ruleIndex);

adk_status_t adk_ipv6_qos_policy_build(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle);

adk_status_t adk_ipv6_get_qos_policies_all(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t *numQosPolicies,
        adk_ipv6_qos_info_t **ipv6QosPol_p);

adk_status_t
adk_ipv6_multicast_group_add (
        adk_module_handle_t ipv6Handle,
        adk_ipv6_multicast_group_t *mcGrp_p,
        adk_flow_handle_t *mcGrpHdl_p);

adk_status_t
adk_ipv6_multicast_group_del (
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t mcGrpHdl);

adk_status_t
adk_ipv6_multicast_flow_add (
        adk_module_handle_t ipv6Handle,
        const adk_ipv6_multicast_flow_t *mcFlow_p,
        adk_flow_handle_t *mcFlowHdl_p);

adk_status_t
adk_ipv6_multicast_flow_del (
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t mcFlowHdl);

adk_status_t
adk_ipv6_multicast_group_stat_get (
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t mcGrpHdl,
        adk_ipv6_multicast_group_stat_t *mcGrpStat_p);

adk_status_t
adk_ipv6_multicast_group_statistics_print (
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t mcGrpHdl);

#ifdef ADK_DEBUG
adk_status_t
adk_ipv6_multicast_group_stat_reset (
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t mcGrpHdl);
#endif

adk_status_t
adk_ipv6_multicast_rpf_check_enable (
        adk_module_handle_t ipv6Handle,
	adk_ipv6_vrd_handle_t ipv6VrdHandle);

adk_status_t
adk_ipv6_multicast_rpf_check_disable (
        adk_module_handle_t ipv6Handle,
	adk_ipv6_vrd_handle_t ipv6VrdHandle);

adk_status_t  adk_ipv6_attach_next_hop_to_route(
        adk_module_handle_t ipv6Handle,
        adk_ipv6_route_rules_t ipv6RouteRules,
        adk_flow_handle_t nextHopHandle);

adk_status_t  adk_ipv6_detach_next_hop_from_route(
        adk_module_handle_t ipv6Handle,
        adk_ipv6_route_rules_t ipv6RouteRules,
        adk_flow_handle_t nextHopHandle);

adk_status_t  adk_ipv6_attach_next_hop_to_pbr_route(
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t multiPathRouteHandle,
        adk_flow_handle_t nextHopHandle);

adk_status_t  adk_ipv6_detach_next_hop_from_pbr_route(
        adk_module_handle_t ipv6Handle,
        adk_flow_handle_t multiPathRouteHandle,
        adk_flow_handle_t nextHopHandle);

adk_status_t  adk_ipv6_service_function_update (adk_module_handle_t ipv6Handle);

adk_status_t adk_ph_ipv6_rx_parse (
              void *task,
              void **dp_message);

adk_status_t
adk_ipv6_pbr_configure_default_route_table (
        adk_module_handle_t ipv6Handle,
        uint8_t  defRtTbl);

adk_status_t
adk_ipv6_pbr_add_policy (
        adk_module_handle_t ipv6Handle,
        uint32_t ruleIndex,
        adk_ipv6_pbr_info_t *pbrInfo_p);

adk_status_t
adk_ipv6_pbr_del_policy (
        adk_module_handle_t ipv6Handle,
	uint32_t  ruleIndex);

adk_status_t adk_ipv6_pbr_commit (
    adk_module_handle_t ipv6Handle);

adk_status_t
adk_ipv6_get_pbr_policies_all(
        adk_module_handle_t ipv6Handle,
        uint32_t *numPbrPolicies,
        adk_ipv6_pbr_info_t **ipv6PbrPol_p);

adk_status_t adk_ipv6_pbr_rule_stats_get (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t pbrRuleHandle,
    adk_ipv6_pbr_rule_stats_t *ipv6PbrRuleStats_p);

adk_status_t adk_ipv6_pbr_rule_stats_print (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t pbrRuleHandle);

adk_status_t
adk_ipv6_interface_create(
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_interface_t  *ipv6If_p,
    adk_interface_handle_t *ipv6IfHandle);

adk_status_t
adk_ipv6_interface_delete(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle);

adk_status_t
adk_ipv6_virtual_ip_addr_add (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_addr_t *ipv6Addr_p);

adk_status_t
adk_ipv6_virtual_ip_addr_del (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_addr_t *ipv6Addr_p);

adk_status_t adk_ipv6_interface_addr_add(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    const adk_ipv6_addr_prefix_t *ipAddrPrefix);

adk_status_t adk_ipv6_interface_addr_rem(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    const adk_ipv6_addr_prefix_t *ipAddrPrefix);

adk_status_t adk_ipv6_sse_ip_addr_add (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_addr_prefix_t ipv6AddrPrefix);

adk_status_t adk_ipv6_sse_ip_addr_del(
    adk_module_handle_t ipv6Handle,
    adk_ipv6_addr_prefix_t ipv6AddrPrefix);


adk_status_t 
adk_ipv6_interface_priority_map_profile_config(
    adk_module_handle_t	ipv6Handle,
    adk_interface_handle_t 	ipv6IfHandle,
    adk_ipv6_direction_t 	direction,
    uint32_t 		profile_id);


adk_status_t adk_ipv6_interface_ingress_policing_config(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    const adk_ipv6_interface_ingress_policing_attr_t *ipv6IfPoliceAttr_p);

adk_status_t adk_ipv6_interface_mtu_set(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    uint32_t mtu);

/*!
 * * API to dynamically update outer path mtu for the tunnel interface.
 * */

/* this API shall be removed after migrating to new API */
adk_status_t
adk_ipv6_tunnel_interface_outer_pmtu_set(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6TunIfHandle,
    uint32_t outerPmtu);

adk_status_t adk_ipv6_dscp_pcp_map_add(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    uint8_t dscp,
    uint8_t pcp);

adk_status_t adk_ipv6_dscp_pcp_map_delete(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    uint8_t dscp);

adk_status_t
adk_ipv6_checksum_validation_enable (
        adk_module_handle_t ipv6Handle);


adk_status_t
adk_ipv6_checksum_validation_disable (
        adk_module_handle_t ipv6Handle);


adk_status_t
adk_ipv6_interface_statistics_print(
    adk_module_handle_t ipv6Handle,
                             adk_interface_handle_t ifHandle);


adk_status_t
adk_ipv6_packet_capture_start(
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_pktcap_info_t   *ipv6PktCapAttr,
    adk_flow_handle_t              *ipv6PktCapHandle_p);

adk_status_t 
adk_ipv6_packet_capture_stop(
                             adk_module_handle_t ipv6Handle,
    adk_flow_handle_t    ipv6PktCapHandle);


adk_status_t 
adk_ipv6_packet_capture_status(
        adk_module_handle_t ipv6Handle,
    adk_flow_handle_t    ipv6PktCapHandle,
    adk_ipv6_pktcap_status_t  *status);


adk_status_t 
adk_ipv6_packet_capture_enable(
                             adk_module_handle_t ipv6Handle,
    adk_flow_handle_t    ipv6PktCapHandle);



adk_status_t adk_ipv6_vrd_create(
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_vrd_t  *vrdAttr_p,
    adk_ipv6_vrd_handle_t *ipv6VrdHandle);

adk_status_t adk_ipv6_vrd_delete(
    adk_module_handle_t ipv6Handle,
    adk_ipv6_vrd_handle_t ipv6VrdHandle);


adk_status_t adk_ipv6_vrd_mode(
    adk_module_handle_t ipv6Handle,
    adk_ipv6_vrd_handle_t ipv6VrdHandle,
    bool host_mode);

adk_status_t adk_ipv6_vrd_handle_get(
    adk_module_handle_t ipv6Handle,
    uint32_t vrd_id,
    adk_ipv6_vrd_handle_t *ipv6VrdHandle_p);


adk_status_t 
adk_ipv6_global_statistics_print( adk_module_handle_t ipv6Handle );

adk_status_t adk_ipv6_fpl_trees_dump(adk_module_handle_t ipv6Handle, 
                                     char                *treeName,
                                     uint32_t        treeType);


adk_status_t
adk_ipv6_tables_dump(adk_module_handle_t ipv6_mod_handle,
                     adk_ipv6_tree_conf_t *ipv6TreeConf_p);
#ifdef ADK_DEBUG
adk_status_t 
adk_ipv6_dscp_group_stats_reset ( 
        adk_module_handle_t ipv6Handle,
        adk_dscp_group_handle_t dscpGrpHandle);
#endif
/*!
 * =============================================================
 * @internal
 * @fn adk_ipv6_get_imc
 * =============================================================
 */
adk_status_t
adk_ipv6_get_imc (adk_module_handle_t ipv6Handle,
                void *bindInfo_p,
                uint32_t *imcFnId);

/*!
 * =============================================================
 * @internal
 * @fn adk_ipv6_get_entry_func
 * =============================================================
 */
adk_status_t
adk_ipv6_get_entry_func(adk_module_handle_t ipv6Handle,
                        void *bindInfo_p,
                        uint32_t *entry_fn_id);

adk_status_t   
adk_ipv6_destination_mtu_add(
                adk_module_handle_t ipv6Handle,
                uint32_t vrd_id,
                adk_ipv6_addr_t dipAddr,
                uint16_t mtu);

adk_status_t   
adk_ipv6_destination_mtu_modify(
                adk_module_handle_t ipv6Handle,
                uint32_t vrd_id,
                adk_ipv6_addr_t dipAddr,
                uint16_t mtu);

adk_status_t 
adk_ipv6_destination_mtu_rem(
                adk_module_handle_t ipv6Handle,
                uint32_t vrd_id,
                adk_ipv6_addr_t dipAddr);

adk_status_t 
adk_ipv6_flow_cache_statistics_print( adk_module_handle_t ipv6Handle );

adk_status_t 
adk_ipv6_flow_cache_wcore_statistics_print( adk_module_handle_t ipv6Handle );


bool
adk_ipv6_compare_address(adk_ipv6_addr_prefix_t address1,
		                         adk_ipv6_addr_prefix_t address2);

bool
adk_ipv6_address_validate(adk_module_handle_t ipv6Handle, adk_ipv6_addr_prefix_t address);

adk_status_t adk_ipv6_reassembly_timeout_config (adk_module_handle_t ipv6Handle,
					        uint32_t timeval,
					        uint8_t percentChangeInTimeout,
					        uint32_t minTimeout);
adk_status_t
adk_ipv6_udp_port_exempt_zero_checksum_add(
                                    adk_module_handle_t ipv6Handle,
                                    uint16_t udpPort);

adk_status_t
adk_ipv6_udp_port_exempt_zero_checksum_delete(
                                adk_module_handle_t ipv6Handle,
                                uint16_t udpPort);

adk_status_t adk_ipv6_configure_supported_protocol(
		    adk_module_handle_t ipv6Handle,
		           uint8_t protocol);

adk_status_t adk_ipv6_remove_supported_protocol (
	            adk_module_handle_t ipv6Handle,
	                   uint8_t protocol);

adk_status_t adk_ipv6_ingress_qos_policy_add (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        adk_ipv6_ingress_qos_info_t *qosInfo_p);

adk_status_t adk_ipv6_ingress_qos_policy_delete (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        adk_ipv6_ingress_qos_info_t *qosInfo_p);

#endif /* End of _ADK_IPV6_H_ */
 
/*! end addtogroup _ipv6Functions
 * @}
 *
 * end addtogroup _ipv6
 * @}
 */
