/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipv6_const.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the constants required to use the IPV6 module.
 * This file can be included by the application.
 *
 ***********************************************************************/
#ifndef _ADK_IPV6_CONST_H_
#define _ADK_IPV6_CONST_H_

#include "adk_config_overrides.h"

/*!
 * IPV6 PDU related constants.
 */
#define ADK_IPV6_PDU_HEADER_LENGTH_BYTES 40
#define ADK_IPV6_PDU_VER_LENGTH 4
#define ADK_IPV6_PDU_TRAFFIC_CLASS_LENGTH 8
#define ADK_IPV6_PDU_DSCP_LENGTH 6
#define ADK_IPV6_PDU_FLOW_LABEL_LENGTH 20
#define ADK_IPV6_PDU_PAYLOAD_LENGTH_LENGTH 16
#define ADK_IPV6_PDU_NEXT_HDR_LENGTH 8
#define ADK_IPV6_UDP_INGRESS_FLOW_LENGTH 32
#define ADK_IPV6_PDU_HOP_LIMIT_LENGTH 8
#define ADK_IPV6_PDU_ADDR_LENGTH 128
#define ADK_IPV6_PDU_L4_PORT_LENGTH 16
#define ADK_IPV6_PDU_UDP_SRC_PORT_LENGTH 16
#define ADK_IPV6_PDU_UDP_DST_PORT_LENGTH 16
#define ADK_IPV6_PDU_ADDR_WORD_SIZE_BITS 32
#define ADK_IPV6_PDU_ADDR_SIZE_WORDS 4
#define ADK_IPV6_PDU_ADDR_SIZE_BYTES 16
#define ADK_IPV6_PDU_L4_PORT_SIZE_BYTES 2
#define ADK_IPV6_PDU_HDR_EXT_LENGTH 8
#define ADK_IPV6_PDU_VER_VALUE 6
#define ADK_IPV6_MIN_ALLOWED_MTU 1280
#define ADK_IPV6_PDU_SOURCE_ADDR_OFFSET_IN_BYTES 8
#define ADK_IPV6_PDU_DESTINATION_ADDR_OFFSET_IN_BYTES 24
#define ADK_IPV6_L2_FLOW_ID_SIZE 24
#define ADK_IPV6_FILTER_RULESET_ID_LENGTH 10
#define ADK_IPV6_INGRESS_ASSOC_RULES_MASK_WIDTH 8
#define ADK_IPV6_ADDR_WILDCARD_PAD_LENGTH 4

#define ADK_IPV6_VLAN_HDR_LEN        4
#define ADK_IPV6_ETH_HDR_LEN         14
#define ADK_IPV6_ICMPV6_HDR_LEN      4
#define ADK_IPV6_UDP_HDR_LEN         8
#define ADK_IPV6_UDP_LITE_HDR_LEN    8
#define ADK_IPV6_TCP_HDR_LEN         20
#define ADK_IPV6_SCTP_HDR_LEN        12
#define ADK_IPV6_HDR_LEN             40
#define ADK_IPV6_PLUS_UDP_HDR_LEN    48
#define ADK_IPV6_FRAG_HDR_LEN        8
#define ADK_IPV6_ICMPV6_CODE_LEN     1
#define ADK_IPV6_ICMPV6_TYPE_LEN     1
#define IPV6_ETH_MIN_PKT_LEN         60

/*! Width of the routing domain id in number of bits. This must be set a value
 * within the range of [1  6] bits. This must not be set to any other value 
 * other than the given range as that would require fast path code changes */
#define ADK_IPV6_VRD_ID_WIDTH                     6
 
#define ADK_IPV6_VRD_ID_WIDTH_ALIGN               2
/*! Maximum number of virtual routing instances. This is derived from the
  The range of VRD-IDs would be [0  ADK_IPV6_VRD_ID_MAX-1]*/
#define ADK_IPV6_VRD_ID_MAX                      (1 << ADK_IPV6_VRD_ID_WIDTH )

/*! Default virtual routing domain ID. This is used for packets which are not
 * assigned to any VR domain. This must be set to a value with in the range 
 * of [0   ADK_IPV6_VRD_ID_MAX-1] only.*/
#define ADK_IPV6_DEFAULT_VRD_ID                  (ADK_IPV6_VRD_ID_MAX - 1)

/*! Max IPV6 Routing tables supported by ADK is 201 */
#define ADK_IPV6_MAX_ROUTING_TABLES 200

/*! MAX IPV6 Route Entries supported by ADK is 1024 */
#define ADK_IPV6_MAX_ROUTE_ENTRIES 1024

/*! Default IPV6 Routing table. */
#define ADK_IPV6_LOCAL_ROUTING_TABLE ADK_IPV6_MAX_ROUTING_TABLES

/* The extension header type values. The extension header
 * values from 0 - 140 are assigned, 141 - 252 are unassigned,
 * 253 - 254 are for experimentation and testing, 255 is
 * reserved.
 */
#define ADK_IPV6_NEXT_HDR_NO_NEXT_HDR 59
#define ADK_IPV6_NEXT_HDR_HOP_BY_HOP 0
#define ADK_IPV6_NEXT_HDR_ICMP 58
#define ADK_IPV6_NEXT_HDR_TCP 6
#define ADK_IPV6_NEXT_HDR_SCTP 132
#define ADK_IPV6_NEXT_HDR_UDP 17
#define ADK_IPV6_NEXT_HDR_ROUTING 43
#define ADK_IPV6_NEXT_HDR_FRAGMENT 44
#define ADK_IPV6_NEXT_HDR_DST_OPTIONS 60
#define ADK_IPV6_NEXT_HDR_MOB_TYPE 135
#define ADK_IPV6_NEXT_HDR_AH 51
#define ADK_IPV6_NEXT_HDR_ESP 50
#define ADK_IPV6_NEXT_HDR_UNASSIGNED_MIN 141
#define ADK_IPV6_NEXT_HDR_UNASSIGNED_MAX 255

/*!
 * IPv6 fragment header field constants.
 */
#define ADK_IPV6_FRAG_HDR_RES1_LENGTH 8
#define ADK_IPV6_FRAG_HDR_OFFSET_LENGTH 13
#define ADK_IPV6_FRAG_HDR_RES2_LENGTH 2
#define ADK_IPV6_FRAG_HDR_MORE_FLAG_LENGTH 1
#define ADK_IPV6_FRAG_HDR_FLAG_LENGTH 3
#define ADK_IPV6_FRAG_HDR_ID_LENGTH 32
#define ADK_IPV6_FRAG_HDR_SIZE_BYTES 8
#define ADK_IPV6_FRAG_HDR_SIZE_BYTES_EXCLUDING_NXTHDR 7
/* The max size supported for the unfragmentable part of
 * PDU is 128 bytes. This is due to the SED PDU block size
 * of 128.
 */
#define ADK_IPV6_FRAG_HDR_UNFRAG_PDU_SIZE_SUPPORTED 128

/*!
 * IPv6 routing header field constants.
 */
#define ADK_IPV6_ROUTING_HDR_TYPE_LENGTH 8
#define ADK_IPV6_ROUTING_HDR_SEGMENTS_LEFT_LENGTH 8

/*!
 * Maximum IPv6 extension headers supported in ADK.
 * shall not be set to zero. value should be atleast 1.
 */
#define ADK_IPV6_MAX_EXTNS_SUPPORTED 8

/*!
 * Maximum number of IP tunnel interface supported by IPv6 module
 */
#define ADK_IPV6_TUNNEL_INTF_MIN    ADK_IPV6_NO_OF_IPV6_INTERFACES
#define ADK_IPV6_TUNNEL_INTF_MAX    256
#define ADK_IPV6_TUNNEL_IF_ID_WIDTH 12
#define ADK_IPV6_INTF_ADDR_MAX      201
#define ADK_IPV6_SSE_ADDR_MAX       1000
#define IPV6_SEC_ADDR_MAX (ADK_IPV6_INTF_ADDR_MAX - 1)
#define ADK_IPV6_VIRTUAL_ADDR_MAX 200
#define ADK_IPV6_INTERFACES_MAX 201
/*! Max IPv6 PBR rules supported by ADK is 201 */
#define ADK_IPV6_PBR_RULES_MAX 200

#ifndef ADK_MIN_IPV6_TUNNEL_MTU
#define ADK_MIN_IPV6_TUNNEL_MTU 1280  /* Default min value */
#endif

/*! Number of DSCP Group per interfaces */ 
#define ADK_IPV6_MAX_DSCP_GROUP_PER_IF  8

/* Number of Link Local IPv6 Address can be added in system */
#define ADK_IPV6_MAX_LINK_LOCAL_ADDR    256

/*! Sizing Params related definitions */
/*! Number of reassemblies*/
#define ADK_IPV6_NO_OF_REASSEMBLIES             2000
#define ADK_IPV6_REASSEMBLY_HASH_ENTRIES        2048
#define ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES    8
#define ADK_IPV6_MAX_REASSEMBLY_CONTEXT ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES * ADK_IPV6_REASSEMBLY_HASH_ENTRIES
/*! Number of next hops */
#define ADK_IPV6_NO_OF_NEXTHOPS                 1024
/*! Number of ACLs and number of entries in each ACL */
#define ADK_IPV6_NO_OF_ACL_ENTRIES              640
/*! Number of IPv6 addresses */
#define ADK_IPV6_NO_OF_IPV6_ADDRESS             256
/*! Number of IPv6 interfaces */
#define ADK_IPV6_NO_OF_IPV6_INTERFACES          256 
/*! Number of IPv6 Tunnel interfaces */
#define ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES   256 
/*! Number of IPv6 Virtual interfaces */
#define ADK_IPV6_NO_OF_IPV6_VIRTUAL_INTERFACES  256 
/*! Number of DSCP per interfaces (Assume)*/
#define ADK_IPV6_NO_OF_IPV6_DSCP_PER_IF         8
/*! Number of ingress flows */
#define ADK_IPV6_NO_OF_INGRESS_FLOW             1024
/*! Number of Mcast Groups */
#define ADK_IPV6_NO_OF_MGROUPS             1024
/*! Number of egress flows */
#define ADK_IPV6_NO_OF_EGRESS_FLOW              1024
/*! Number of Tunnel interfaces (Derived based on IPsec SPD rulesets) */
#define ADK_IPV6_NO_OF_IPV6_TNL_INTERFACES      270 
/*! Number of PBRs and number of entries in each PBR */
#define ADK_IPV6_NO_OF_PBR_ENTRIES              640
/*! Number of PBRs and number of entries in each PBR */
#define ADK_IPV6_NO_OF_SIP_ENTRIES              256
/*! Number of routes */
#define ADK_IPV6_NO_OF_ROUTES                   (1600 + ADK_IPV6_NO_OF_IPV6_INTERFACES + \
                                                        ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES + \
                                                        ADK_IPV6_NO_OF_IPV6_VIRTUAL_INTERFACES + \
                                                        (ADK_IPV6_VRD_ID_MAX*3))
/*! Number of onlink entries in system */
#define ADK_IPV6_NO_OF_ONLINK_ENTRIES           1024
#define ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES       8
/*! Number of destination MTU table entries in system */
#define ADK_IPV6_NO_OF_DEST_MTU_ENTRIES         1024
#define ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES     8
#define ADK_IPV6_MAX_DYNAMIC_DEST_MTU_TABLE     10
#define ADK_IPV6_MAX_REASS_ELE                  64

#define ADK_IPV6_MAX_TIMEOUT  2000000
#define ADK_IPV6_MAX_PERC_CHANGE_TIMEOUT 100
#define ADK_IPV6_MIN_TIMEOUT_VALUE 100

#define ADK_IPV6_REASSEMBLY_HASH_MASK           (ADK_IPV6_REASSEMBLY_HASH_ENTRIES - 1)
#define ADK_IPV6_ONLINK_HASH_MASK               (ADK_IPV6_NO_OF_ONLINK_ENTRIES-1)
#define ADK_IPV6_DEST_MTU_HASH_MASK             (ADK_IPV6_NO_OF_DEST_MTU_ENTRIES-1)

/* Number of Flowcache entries */
#define ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES   4
#define ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH         11
#define ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES       (1<<ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) /* shall be power of 2 */
#define ADK_IPV6_FLOW_CACHE_AGING_THRESHHOLD    (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>2) /* number of free entries left */
#define ADK_IPV6_FLOW_CACHE_AGING_TIMER         30000 /* in msec */
#define ADK_IPV6_FLOW_CACHE_AGING_ITER          4 /* 4 iterations to complete entire FLOW cache traversing */
#define ADK_IPV6_FLOW_CACHE_AGING_SEG_SIZE      (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>8) /* */

/*! Number of DSCP * interfaces combination */
#define ADK_IPV6_NO_OF_IPV6_DSCP_INTERFACES ADK_IPV6_NO_OF_IPV6_INTERFACES * ADK_IPV6_NO_OF_IPV6_DSCP_PER_IF

/*!
 * IPv6 header field constants for ACL
 */
#define ADK_IPV6_DST_OPT_HDR_TYPE_LENGTH 8
#define ADK_IPV6_MOBILITY_HDR_TYPE_LENGTH 8
#define ADK_IPV6_PROTO_INFO1_LENGTH 16
#define ADK_IPV6_PROTO_INFO2_LENGTH 16
#define ADK_IPV6_PROTO_INFO3_LENGTH 8
#define ADK_IPV6_INTERFACE_ID_LENGTH 8
#define ADK_IPV6_FRAG_LENGTH  2
#define ADK_IPV6_PROTOCOL_LENGTH 8

#define ADK_IPV6_NEXT_HDR_ICMPV6     58
#define ADK_IPV6_NEXT_HDR_UDP_LITE   136
#define ADK_IPV6_NEXT_HDR_SCTP       132

#define ADK_IPV6_FILTER_DEFAULT_RULE_DENY_PDU    0
#define ADK_IPV6_FILTER_DENY_PDU            1
#define ADK_IPV6_FILTER_ACCEPT_PDU          2
#define ADK_IPV6_FILTER_FORWARD_PDU_TO_CPU  3

#define ADK_IPV6_FILTER_PH_INFO_OFFSET      0
#define ADK_IPV6_FILTER_PH_INFO1_OFFSET     4 
#define ADK_IPV6_FILTER_NCAQ_INFO_OFFSET    4
#define ADK_IPV6_FILTER_RULE_MASK_PH_INFO1_AND_NCAQ_INFO_OFFSET    4
#define ADK_IPV6_FILTER_RULE_SET_HDL_OFFSET    8
#define ADK_IPV6_FILTER_QOS_FLOW_OFFSET    12
#define ADK_IPV6_FILTER_COUNTER_OFFSET    16
#define ADK_IPV6_FILTER_MTM_QUEUE_HANDLE   20

#define ADK_IPV6_PBR_RULE_SETS_MAX 32
#define ADK_IPV6_PBR_RULE_SETS_MASK ((ADK_IPV6_PBR_RULE_SETS_MAX)-1)

#define ADK_IPV6_FILTER_RULE_SETS_MAX 1023
#define ADK_IPV6_INGRESS_FLOW_DSCP_FLOW_OFFSET    8
#define ADK_IPV6_INGRESS_FLOW_TUNNEL_ID_OFFSET    12
#define ADK_IPV6_INGRESS_FLOW_SIP_MSB_OFFSET      16
#define ADK_IPV6_INGRESS_FLOW_SIP_LSB_OFFSET      24
#define ADK_IPV6_INGRESS_FLOW_SRC_PORT_OFFSET     32

/*! IPv6 Egress Interface Counter offset
 */
#define ADK_IPV6_INTERFACE_DROPPED_OCTECT_OFFSET   0
#define ADK_IPV6_INTERFACE_DROPPED_PDU_OFFSET      8
#define ADK_IPV6_INTERFACE_TX_PDU_OFFSET           12
#define ADK_IPV6_INTERFACE_TX_OCTECT_OFFSET        16
#define ADK_IPV6_INTERFACE_MC_TX_PDU_OFFSET        24

/*! IPv6 teredo client related constants
 */
#define ADK_IPV6_TEREDO_CLIENT_TRUSTED_ENTRY 0x4
#define ADK_IPV6_TEREDO_CLIENT_TRUSTED_ENTRY_LOC 0x2
#define ADK_IPV6_TEREDO_CLIENT_ENTRY_EXISTS 0x1
#define ADK_IPV6_TEREDO_CLIENT_DECAP_ENTRY  0x0
#define ADK_IPV6_TEREDO_CLIENT_DECAP_ENTRY_NS_OFFSET0  0
#define ADK_IPV6_TEREDO_CLIENT_DECAP_ENTRY_NS_OFFSET1  4
#define ADK_IPV6_TEREDO_CLIENT_ENCAP_ENTRY  0x1
#define ADK_IPV6_TEREDO_CLIENT_ENCAP_ENTRY_NS_OFFSET0  8
#define ADK_IPV6_TEREDO_CLIENT_ENCAP_ENTRY_NS_OFFSET1  12

/*!
 * IPv6 constants for flags field of IPv6 ULI
 */
#define ADK_IPV6_HDR_PRESENT 0x1
#define ADK_IPV6_EGRESS_FLOW_ID_PRESENT 0x2
#define ADK_IPV6_FRAGMENTATION_NOT_ALLOWED 0x4
#define ADK_IPV6_PRIORITY_BITS_VALID 0x8
#define ADK_IPV6_DO_NOT_ADD_CHECKSUM 0x10
#define ADK_IPV6_MPP_ULI_FN "adk_ipv6.mpp_uli_indirect_bind_fn"

/*! 
 * IPv6 feild constants for QOS
 */
#define ADK_IPV6_QOS_MAX_CLASSES_PER_API                 15
#define ADK_IPV6_DSCP_WIDTH                              8
#define ADK_IPV6_DSCP_PCP_WIDTH                          11
#define ADK_IPV6_DSCP_PCP_DEF_WIDTH                      12
#define ADK_IPV6_QOS_QID_LENGTH                          24

#define ADK_IPV6_QOS_TREE_WIDTH                     32
#define ADK_IPV6_QOS_TREE                           "adk_ipv6.ipv6_qos_tree"
#define ADK_IPV6_QOS_TREE_REAL_DEFAULT_ENTRY        0xFFFFFF
#define ADK_IPV6_QOS_TREE_VIRTUAL_DEFAULT_ENTRY     0

/*! 
 * IPv6 feild constants for Egress Intf DSCP Lookup
 */
#define ADK_IPV6_EGRESS_IF_DSCP_LOOKUP_TREE_WIDTH                     32
#define ADK_IPV6_EGRESS_IF_DSCP_LOOKUP_TREE                           "adk_ipv6.ipv6_egress_interface_dscp_lookup_tree"
#define ADK_IPV6_EGRESS_IF_DSCP_LOOKUP_TREE_REAL_DEFAULT_ENTRY        NCP_NS_BASEID_ipv6_interface_stats
#define ADK_IPV6_EGRESS_IF_DSCP_LOOKUP_TREE_VIRTUAL_DEFAULT_ENTRY     0x0 

/*! 
 * IPv6 feild constants for DSCP Remarking and Policing
 */
#define ADK_IPV6_IF_DSCP_POLICE_LOOKUP_TREE_WIDTH   32
#define ADK_IPV6_IF_DSCP_POLICE_LOOKUP_TREE         "adk_ipv6.ipv6_if_dscp_police_lookup"
#define ADK_IPV6_IF_DSCP_POLICE_LOOKUP_TREE_DEF     0x0
#define IPV6_MAX_DSCP_VALUE                         0x3F
#define ADK_IPV6_DSCP_POLICE_TREE_REAL_DEFAULT_ENTRY     0x0
#define ADK_IPV6_INTERFACE_STAT_INDEX_WIDTH         24

/*! Filter tree to lookup match the pattern in ingress
 *  and egress direction. Only one tree maintained for this purpose.
 */  
#define ADK_IPV6_FILTER_RULES_OVTREESET \
    "adk_ipv6.ipv6_filter_rules_ovtreeset"

/*! Tree maintained to check if filtering is enabled 
 *  on the interface in ingress direction
 */
#define ADK_IPV6_INTERFACE_LOOKUP_TREE \
    "adk_ipv6.ipv6_interface_lookup_tree"
#define ADK_IPV6_INTERFACE_LOOKUP_TREE_WIDTH  24
#define ADK_IPV6_INTERFACE_LOOKUP_DEFAULT_FN   "adk_ipv6.ipv6_invalid_interface"
#define ADK_IPV6_INTERFACE_DEFAULT_MTU        1280
#define ADK_IPV6_INTERFACE_MIN_MTU            1280
#define ADK_IPV6_INTERFACE_MAX_MTU            32767
#define ADK_IPV6_INTERFACE_MTU_MASK           0x7FFF
#define ADK_IPV6_INTERFACE_LOCAL_PTB_LOC      15  
/*! 
 * IPv6 feild constants for Tunnel interface Id Lookup 
 */
#define ADK_IPV6_TUNNEL_IF_ID_LOOKUP_TREE        "adk_ipv6.ipv6_tunnel_if_id_lookup"
#define ADK_IPV6_TUNNEL_IF_ID_LOOKUP_TREE_WIDTH  ADK_IPV6_TUNNEL_IF_ID_WIDTH
#define ADK_IPV6_TUNNEL_IF_ID_LOOKUP_DEFAULT_FN  "adk_ipv6.unknown_tunnel_if_id"

/*! 
 * IPv6 feild constants for CPU NH Lookup 
 */
#define ADK_IPV6_CPU_NH_LOOKUP_TREE                 "adk_ipv6.cpu_nh_lookup"
#define ADK_IPV6_CPU_NH_LOOKUP_TREE_WIDTH           ADK_IPV6_INTERFACE_ID_LENGTH
#define ADK_IPV6_CPU_NH_LOOKUP_TREE_REAL_DEFAULT_ENTRY      0
#define ADK_IPV6_CPU_NH_LOOKUP_TREE_VIRTUAL_DEFAULT_ENTRY   0

/*! Tree maintained to check if filtering is enabled 
 *  on the interface in egress direction and get rule set id 
 */
#define ADK_IPV6_EGRESS_INTERFACE_LOOKUP_TREE \
    "adk_ipv6.ipv6_egress_interface_lookup_tree"
#define ADK_IPV6_EGRESS_INTERFACE_LOOKUP_TREE_WIDTH           24
#define ADK_IPV6_EGRESS_INTERFACE_EGRESS_FILTER_RULE_SET_ID   0
#define ADK_IPV6_EGRESS_INTERFACE_ENABLE_EGRESS_FILTERING     10
#define ADK_IPV6_EGRESS_INTERFACE_VRD_ID_OFFSET               11
#define ADK_IPV6_EGRESS_INTERFACE_STATUS_DISABLED             17

#define ADK_IPV6_EGRESS_INTERFACE_QOS_LOC                     16
#define ADK_IPV6_EGRESS_INTERFACE_MTU_SIZE                    16


/*! Tree maintained to perform policy based routing 
 *  in ingress/egress direction
 */
#define ADK_IPV6_PBR_LOOKUP_TREE \
    "adk_ipv6.ipv6_pbr_lookup_tree"
#define ADK_IPV6_PBR_INGRESS_LOOKUP_ENABLE  0x4 
#define ADK_IPV6_PBR_EGRESS_LOOKUP_ENABLE   0x8

#define ADK_IPV6_EGRESS_PBR_STATE_TREE \
    "adk_ipv6.ipv6_egress_pbr_state_tree"
#define ADK_IPV6_EGRESS_PBR_ENABLED_FN \
    "adk_ipv6.ipv6_egress_pbr_lookup"
/* Number of fields in the PBR lookup */
#define IPV6_NUM_PBR_INPUT_FIELDS    9

/*!
 * IPv4 header field constants for translation.
 */
#define ADK_IPV6_TRANSLATE_IPV4_HDR_LENGTH_BYTES 20
#define ADK_IPV6_TRANSLATE_IPV4_HDR_VER 4
#define ADK_IPV6_TRANSLATE_IPV4_HDR_VER_LENGTH 4
#define ADK_IPV6_TRANSLATE_IPV4_HDR_IHL 5
#define ADK_IPV6_TRANSLATE_IPV4_HDR_IHL_LENGTH 4
#define ADK_IPV6_TRANSLATE_IPV4_HDR_TOS_LENGTH 8
#define ADK_IPV6_TRANSLATE_IPV4_HDR_TOTAL_LENGTH_LENGTH 16
#define ADK_IPV6_TRANSLATE_IPV4_HDR_ID_LENGTH 16
#define ADK_IPV6_TRANSLATE_IPV4_HDR_FLAG_LENGTH 3
#define ADK_IPV6_TRANSLATE_IPV4_HDR_FRAG_OFFSET_LENGTH 13
#define ADK_IPV6_TRANSLATE_IPV4_HDR_TTL_LENGTH 8
#define ADK_IPV6_TRANSLATE_IPV4_HDR_PROTOCOL_LENGTH 8
#define ADK_IPV6_TRANSLATE_IPV4_HDR_CHECKSUM 0
#define ADK_IPV6_TRANSLATE_IPV4_HDR_CHECKSUM_LENGTH 16
#define ADK_IPV6_TRANSLATE_IPV4_HDR_IP_ADDR_LENGTH 32


/*! The tree function names and input pattern widths (bits) */
/*! Exception lookup tree to look up the exception handler
 *  function.
 */
#define ADK_IPV6_EXCEPTION_HANDLER_TREE \
            "adk_ipv6.exception_handler_tree"
#define ADK_IPV6_EXCEPTION_HANDLER_STAGE_WIDTH 2
#define ADK_IPV6_EXCEPTION_HANDLER_EXCEPTION_TYPE_WIDTH 5
#define ADK_IPV6_EXCEPTION_HANDLER_EXCEPTION_DOMAIN_WIDTH 2
#define ADK_IPV6_EXCEPTION_HANDLER_TREE_WIDTH \
    (ADK_IPV6_EXCEPTION_HANDLER_STAGE_WIDTH + \
     ADK_IPV6_EXCEPTION_HANDLER_EXCEPTION_TYPE_WIDTH + \
     ADK_IPV6_EXCEPTION_HANDLER_EXCEPTION_DOMAIN_WIDTH)
#define ADK_IPV6_EXCEPTION_HANDLER_CB_FN_ID_SIZE 8
#define ADK_IPV6_EXCEPTION_HANDLER_MTM_QUEUE_ID_SIZE 24
#define ADK_IPV6_EXCEPTION_HANDLER_BIND_HANDLE_SIZE1 16
#define ADK_IPV6_EXCEPTION_HANDLER_BIND_HANDLE_MASK1 0xffff
#define ADK_IPV6_EXCEPTION_HANDLER_BIND_HANDLE_MASK2 0xffff0000
#define ADK_IPV6_EXCEPTION_HANDLER_STAGE1_ID 0
#define ADK_IPV6_EXCEPTION_HANDLER_STAGE2_ID 1
#define ADK_IPV6_EXCEPTION_HANDLER_STAGE3_ID 2
/*! The default exception handler function, used when no handler
 *  is registered.
 */
#define ADK_IPV6_DEFAULT_EXCEPTION_HANDLER \
            "adk_ipv6.default_exception_handler"
/*! The function stored in the tree for the first stage when the
 *  user registered exception handler function is a MPP based
 *  function.
 */
#define ADK_IPV6_FIRST_STAGE_MPP_EXCEPTION_FN \
            "adk_ipv6.first_stage_mpp_exception_fn"

/*! The teredo lookup tree to get the action on incoming
 *  teredo PDU. The input is teredo client IPv6 address
 *  from the PDU.
 */
#define ADK_IPV6_TEREDO_CLIENT_LOOKUP_TREE "adk_ipv6.teredo_client_lookup_tree"
#define ADK_IPV6_TEREDO_CLIENT_LOOKUP_TREE_DEFAULT_ENTRY 0x0
#define ADK_IPV6_TEREDO_CLIENT_LOOKUP_TREE_WIDTH (ADK_IPV6_VRD_ID_WIDTH + ADK_IPV6_VRD_ID_WIDTH_ALIGN + ADK_IPV6_PDU_ADDR_LENGTH)


/*! The route lookup tree to get the actions incoming PDU. The
 *  input is the destination IPv6 address.
 */
#define ADK_IPV6_ROUTE_LOOKUP_TREE "adk_ipv6.route_lookup_tree"
#define ADK_IPV6_ROUTE_LOOKUP_TREE_DEFAULT_ENTRY 0x0
#define ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH  (ADK_IPV6_VRD_ID_WIDTH + ADK_IPV6_VRD_ID_WIDTH_ALIGN + ADK_IPV6_PDU_ADDR_LENGTH)
/*! Route lookup actions and locations */
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_SIZE 8
#define ADK_IPV6_ROUTE_LOOKUP_PARAM_SELECTION_SIZE 8
#define ADK_IPV6_ROUTE_LOOKUP_PARAM_SELECTION_LOC \
        ADK_IPV6_ROUTE_LOOKUP_ACTION_SIZE
#define ADK_IPV6_ROUTE_LOOKUP_PARAM_SELECTION_MASK 0xff00
#define ADK_IPV6_NULL_ROUTE_FN   "adk_ipv6.null_route_count_and_drop"

/*! Multipath Route Lookup Tree Definition*/
#define ADK_IPV6_MULTIPATH_ROUTE_LOOKUP_TREE "adk_ipv6.ipv6_multiple_nexthop_tree"
#define ADK_IPV6_MULTIPATH_ROUTE_LOOKUP_TREE_DEFAULT_ENTRY 0x0
#define ADK_IPV6_MULTIPATH_ROUTE_LOOKUP_TREE_WIDTH (ADK_IPV6_MULTIPATH_FLOW_ID_SIZE + ADK_IPV6_MULTIPATH_HOP_INDEX_WIDTH)
#define ADK_IPV6_MULTIPATH_FLOW_ID_SIZE 24
#define ADK_IPV6_MULTIPATH_HOP_INDEX_WIDTH 4
#define ADK_IPV6_MULTIPATH_ROUTE_ENABLE 0x1

/*! The ingress flow dip to get ingress flow action for incoming PDU.
 *  Input to this tree is the destination IPv6 address.
 */
#define ADK_IPV6_INRESS_FLOW_SIP_TREE "adk_ipv6.ingress_flow_sip_tree"
#define ADK_IPV6_INGRESS_FLOW_SIP_TREE_DEFAULT_ENTRY 0x0
#define ADK_IPV6_INGRESS_FLOW_SIP_TREE_WIDTH  ((ADK_IPV6_VRD_ID_WIDTH+ADK_IPV6_VRD_ID_WIDTH_ALIGN)+ADK_IPV6_PDU_ADDR_LENGTH)
#define ADK_IPV6_INGRESS_FLOW_SIP_FOUND 0x1

/*Ingres Assoc trees*/
#define ADK_IPV6_INGRESS_FLOW_RESTART_FEATURE
#define ADK_IPV6_INGRESS_ASSOC_FLOW_MAX 70000
#define ADK_IPV6_INGRESS_ASSOC_FLOW_TREE "adk_ipv6.ipv6_ingress_assoc_flow_lookup"
#define ADK_IPV6_INGRESS_ASSOC_ICMP_FLOW_TREE "adk_ipv6.ipv6_ingress_assoc_icmp_flow_lookup"
#define ADK_IPV6_INGRESS_ASSOC_FLOW_TREE_WIDTH ( ADK_IPV6_INGRESS_ASSOC_RULES_MASK_WIDTH + \
                                                         ADK_IPV6_INTERFACE_ID_LENGTH + \
                                                         /*ADK_IPV6_PDU_FLOW_LABEL_LENGTH*/ 24 + \
                                                         ADK_IPV6_PDU_NEXT_HDR_LENGTH + \
                                                         ADK_IPV6_PDU_L4_PORT_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH + \
                                                         ADK_IPV6_PDU_L4_PORT_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH)

#define ADK_IPV6_INGRESS_MGROUP_FLOW_TREE "adk_ipv6.ipv6_ingress_mgroup_flow_lookup"
#define ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH ( ADK_IPV6_INTERFACE_ID_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH)

#define ADK_IPV6_INGRESS_ASSOC_FLOW_2_TREE "adk_ipv6.ipv6_ingress_assoc_flow_2_lookup"
#define ADK_IPV6_INGRESS_ASSOC_FLOW_2_TREE_WIDTH (ADK_IPV6_FLOW_ID_SIZE+ \
                                                  +ADK_IPV6_PDU_L4_PORT_LENGTH \
                                                  +ADK_IPV6_PDU_ADDR_LENGTH)

#define ADK_IPV6_INGRESS_ASSOC_FLOW_RESTART_TREE_WIDTH ( ADK_IPV6_INGRESS_ASSOC_RULES_MASK_WIDTH + \
                                                         ADK_IPV6_INTERFACE_ID_LENGTH + \
                                                         ADK_IPV6_PDU_FLOW_LABEL_LENGTH + \
                                                         ADK_IPV6_PDU_NEXT_HDR_LENGTH + \
                                                         ADK_IPV6_PDU_L4_PORT_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH + \
                                                         ADK_IPV6_PDU_L4_PORT_LENGTH + \
                                                         ADK_IPV6_PDU_ADDR_LENGTH)




/*! Route TREE action fields are defined below
 */
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD                    0x1
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_ONLINK		    0x2
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_MUTLIHOP   	    0x3
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE		    0x4
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE	    0x5
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_DROP		    0x6
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_DROP_WITH_COUNT        0x7
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_STH_VIA_PH		    0x8
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_STH_VIA_NCA	    0x9
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_FALLBACK    	    0xA
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_NULL_ROUTE             0xB
#define ADK_IPV6_ROUTE_LOOKUP_ROUTE_FOUND                   0xC
#define ADK_IPV6_ROUTE_LOOKUP_ACTION_LOC		    0
#define ADK_IPV6_ROUTE_LOOKUP_TERMINATE_MASK_LOC            4
#define ADK_IPV6_ROUTE_LOOKUP_COMP_DESTID_LOC               12
#define ADK_IPV6_ROUTE_LOOKUP_REASSEMBLE_LOC                20


/*! PBR TREE action fields are defined below
 */
#define ADK_IPV6_PBR_ROUTE_LOOKUP_FAILED     ADK_IPV6_ROUTE_LOOKUP_ACTION_FALLBACK

/*! Ingress flow Lookup actions fields are defined below 
 */
#define ADK_IPV6_INGRESS_FLOW_FOUND 0x1
#define ADK_IPV6_INGRESS_FLOW_FOUND_LOC 0
#define ADK_IPV6_INGRESS_FLOW_TRANSLATE_TO_IPV4 0x8
#define ADK_IPV6_INGRESS_FLOW_TRANSLATE_TO_IPV4_LOC 3
#define ADK_IPV6_INGRESS_FLOW_UDP_INGRESS_FLOW_ENABLE 0x10
#define ADK_IPV6_INGRESS_FLOW_UDP_INGRESS_FLOW_ENABLE_LOC 4
#define ADK_IPV6_INGRESS_FLOW_SEND_USING_PH 0x20
#define ADK_IPV6_INGRESS_FLOW_SEND_USING_PH_LOC 5
#define ADK_IPV6_INGRESS_FLOW_SEND_USING_NCA_QID 0x40
#define ADK_IPV6_INGRESS_FLOW_SEND_USING_NCA_QID_LOC 6
#define ADK_IPV6_INGRESS_FLOW_IS_LOCAL 0x80
#define ADK_IPV6_INGRESS_FLOW_IS_LOCAL_LOC 7
#define ADK_IPV6_INGRESS_FLOW_ING_FLOW_FWD 0x100
#define ADK_IPV6_INGRESS_FLOW_ING_FLOW_FWD_LOC 8
#define ADK_IPV6_INGRESS_FLOW_QOS_POLICING_ENABLE 0x10000
#define ADK_IPV6_INGRESS_FLOW_QOS_POLICING_ENABLE_LOC 16
#define ADK_IPV6_INGRESS_FLOW_ISATAP_TUNNEL     0x20000
#define ADK_IPV6_INGRESS_FLOW_ISATAP_TUNNEL_LOC 17
#define ADK_IPV6_INGRESS_FLOW_6TO4_TUNNEL     0x40000
#define ADK_IPV6_INGRESS_FLOW_6TO4_TUNNEL_LOC 18
#define ADK_IPV6_INGRESS_FLOW_TEREDO_TUNNEL_ENCAP     0x80000
#define ADK_IPV6_INGRESS_FLOW_TEREDO_TUNNEL_ENCAP_LOC 19
#define ADK_IPV6_INGRESS_FLOW_LINKLOCAL     0x100000
#define ADK_IPV6_INGRESS_FLOW_LINKLOCAL_LOC 20
#define ADK_IPV6_INGRESS_FLOW_TEREDO_DECAP 0x400000
#define ADK_IPV6_INGRESS_FLOW_TEREDO_DECAP_LOC 22
#define ADK_IPV6_INGRESS_FLOW_IGNORE_CHECKSUM 0x800000
#define ADK_IPV6_INGRESS_FLOW_IGNORE_CHECKSUM_LOC 23

/*! Ingress Checksum Validation configuration tree */
#define ADK_IPV6_CHECKSUM_VALIDATE_TREE            "adk_ipv6.ingress_validate_l4Chksum" 
#define ADK_IPV6_CHECKSUM_VALIDATION_ENABLE_FN_ID  "adk_ipv6.Invalid_checksum"
#define ADK_IPV6_CHECKSUM_VALIDATION_DISABLE_FN_ID "adk_ipv6.Valid_checksum"
#define ADK_IPV6_CHECKSUM_VALIDATE_TREE_WIDTH      2 

/*! Ingress flow lookup other configs and locations */
#define ADK_IPV6_INGRESS_FLOW_OTHER_CONFIG_MASK 0xff00
#define ADK_IPV6_INGRESS_FLOW_OTHER_CONFIG_SET_TOS_TO_ZERO 0x1
#define ADK_IPV6_INGRESS_FLOW_OTHER_CONFIG_SET_TOS_TO_ZERO_LOC 0x8

/*! Destination lookup tree to get the corresponding L2 flow ID */
#define ADK_IPV6_ON_LINK_DST_TREE "adk_ipv6.on_link_destination_tree"
#define ADK_IPV6_ON_LINK_DST_TREE_WIDTH ((ADK_IPV6_VRD_ID_WIDTH+ADK_IPV6_VRD_ID_WIDTH_ALIGN) + ADK_IPV6_PDU_ADDR_LENGTH)
#define ADK_IPV6_ON_LINK_DST_FLAG_DST_NOT_FOUND 0x0
#define ADK_IPV6_ON_LINK_DST_FLAG_DST_FOUND 0x1
#define ADK_IPV6_ON_LINK_DST_FLAG_LOC 16

/*! Route and advance lookup attribute tree. The input for this
 *  tree is the flow ID returned from the route_lookup_tree or
 *  advance_lookup_tree.
 */
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_TREE1 \
    "adk_ipv6.route_and_advance_lookup_attr_tree1"
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_TREE_DEFAULT_FN \
    "adk_ipv6.route_and_advance_lookup_attr_tree_default_fn"
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_TREE_WIDTH 24
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_FLAGS_SIZE 1
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_FLAGS_LOC  0
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_MULTIPATH_LOC  1
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_MULTIPATH_SIZE 1
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_CB_FN_ID_SIZE 8
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_CB_FN_ID_LOC 0
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_SIZE1 16
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_MASK1 0xffff
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_LOC1 8
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_SIZE2 16
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_MASK2 0xffff0000
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_BIND_HANDLE_LOC2 0

#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_TREE2 \
    "adk_ipv6.route_and_advance_lookup_attr_tree2"
#define ADK_IPV6_ROUTE_AND_ADVANCE_LOOKUP_ATTR_MTM_Q_ID_SIZE 24

/*! Tree - Pattern FlowId:24,Network Specific Add for translation:6+128,
 *  return Real Handle:Prefixwidth:24
 */
#define ADK_IPV6_TRANSLATE_SIP_PREFIX_MATCH_LOOKUP_TREE \
    "adk_ipv6.ipv6_translate_sip_prefix_match_tree"
#define ADK_IPV6_TRANSLATE_SIP_PREFIX_MATCH_LOOKUP_TREE_SIZE \
	  (ADK_IPV6_L2_FLOW_ID_SIZE + ADK_IPV6_VRD_ID_WIDTH + ADK_IPV6_VRD_ID_WIDTH_ALIGN\
         ADK_IPV6_PDU_ADDR_LENGTH)
#define ADK_IPV6_TRANSLATE_SIP_PREFIX_MATCH_TREE_DEFAULT_ENTRY 0x0


/*! Reassembly hash tree constants. The number of entries
 *  should be the same as in the corresponding name space.
 */
#define ADK_IPV6_REASSEMBLY_HASH_TREE_ID 3050
#define ADK_IPV6_REASSEMBLY_HASH_TREE_NUM_ENTRIES ADK_IPV6_NO_OF_REASSEMBLIES

/*! Egress flow attribute tree to get the egress flow attributes.
 *  The input to this tree is the IPv6 egress flow ID.
 */
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE "adk_ipv6.egress_flow_attr_tree"
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_DEFAULT_FN \
    "adk_ipv6.egress_flow_attr_tree_default_fn"
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_WIDTH 24
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_MTU_SIZE 16
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_MTU_LOC 0
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_FLAGS_SIZE 8
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_FLAGS_LOC 16
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_ENCAP_LOC 16
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_ISATAP_TUNNEL_LOC 17
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_6TO4_TUNNEL_LOC 18
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_TEREDO_TUNNEL_LOC 19
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_LINKLOCAL_LOC 20
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_ADD_FRAG_HDR_LOC 21
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_UDP_HDR_CREATE_LOC 22
#define ADK_IPV6_EGRESS_FLOW_ATTR_TREE_ACTION_TRANSLATE_TO_V4_LOC 23

/*! Egress flow attribute tree to get the UDP egress flow attributes.
 *  The input to this tree is the IPv6 egress flow ID.
 */
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE "adk_ipv6.egress_udp_flow_attr_tree"
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_DEFAULT_FN \
    "adk_ipv6.egress_udp_flow_attr_tree_default_fn"
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_WIDTH 24
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_PRIORITY_LOC 16
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_PRIORITY_SIZE 8
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_PRIORITY_VALID_LOC 16
#define ADK_IPV6_UDP_EGRESS_FLOW_ATTR_TREE_PRIORITY_VALID_SIZE 1

/*! Egress flow destination lookup tree to get the destination
 *  IPv6 address of the flow.
 */
#define ADK_IPV6_EGRESS_FLOW_DST_LOOKUP_TREE1 \
    "adk_ipv6.egress_flow_dst_lookup_tree1"
#define ADK_IPV6_EGRESS_FLOW_DST_LOOKUP_TREE2 \
    "adk_ipv6.egress_flow_dst_lookup_tree2"
#define ADK_IPV6_EGRESS_FLOW_DST_LOOKUP_TREE3 \
    "adk_ipv6.egress_flow_dst_lookup_tree3"
#define ADK_IPV6_EGRESS_FLOW_DST_LOOKUP_TREE_DEFAULT_FN \
    "adk_ipv6.egress_flow_dst_lookup_tree_default_fn"
#define ADK_IPV6_EGRESS_FLOW_DST_LOOKUP_TREE_WIDTH 24

/*! Fragment header ID index tree to get the index to
 *  SE memory containing the ID.
 */
#define ADK_IPV6_EGRESS_FRAGMENT_ID_INDEX_TREE \
            "adk_ipv6.egress_fragment_id_tree"
#define ADK_IPV6_EGRESS_FRAGMENT_ID_DEFAULT_ENTRY 1
#define ADK_IPV6_EGRESS_FRAGMENT_ID_INDEX_TREE_WIDTH 1

/*! Global statistics' index tree to get the global statistics index. */
#define ADK_IPV6_GLOBAL_STATISTICS_INDEX_TREE \
            "adk_ipv6.global_statistics_index_tree"
#define ADK_IPV6_GLOBAL_STATISTICS_DEFAULT_ENTRY 1
#define ADK_IPV6_GLOBAL_STATISTICS_INDEX_TREE_WIDTH 1

/*! Ingress attribute send using PH */
#define ADK_IPV6_INGRESS_ATTR_SEND_USING_PH 0x1
/*! Ingress attribute Translate */
#define ADK_IPV6_INGRESS_ATTR_TRANSLATE_TO_IPV4 0x4
/*! Ingress attribute Forward the PDU */
#define ADK_IPV6_INGRESS_ATTR_FORWARD_PDU 0x8
/*! Ingress attribute ISATAP tunnel processing */
#define ADK_IPV6_INGRESS_ATTR_ISATAP_TUNNELING  0x10
/*! Ingress attribute 6to4 tunnel processing */
#define ADK_IPV6_INGRESS_ATTR_6TO4_TUNNELING    0x20
/*! Ingress attribute send using nca Qid */
#define ADK_IPV6_INGRESS_ATTR_SEND_USING_NCA_QID 0x80
/*! Ingress attribute teredo tunnel encap processing */
#define ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_ENCAP 0x100
/*! Ingress attribute teredo tunnel decap processing */
#define ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_DECAP 0x200
/*! Ingress attribute ignore checksum validation */
#define ADK_IPV6_INGRESS_ATTR_IGNORE_CHECKSUM 0x400
/*! Ingress attribute to send packets to fps via 
 * registered callback function*/
#define ADK_IPV6_INGRESS_ATTR_INVOKE_CB 0x800



/*! Use VRD Id field insted of VRD Handle */
#define ADK_IPV6_EGRESS_RULE_USE_VRD_ID 0x1

/*! Egress attribute, fwd_to ethernet only */
#define ADK_IPV6_EGRESS_ATTR_ACTION_FWD_TO_ETH 0x0
/*! Egress attribute, encapsulation only */
#define ADK_IPV6_EGRESS_ATTR_ACTION_ENCAP 0x1
/*! Egress attribute, ISATAP tunnel processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_ISATAP_TUNNEL  0x2
/*! Egress attribute, 6to4 tunnel processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_6TO4_TUNNEL    0x4
/*! Egress attribute, teredo tunnel processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_TEREDO_TUNNEL    0x8
/*! Egress attribute, link local packet processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_LINKLOCAL 0x10
/*! Egress attribute, ICMP handling for translation packet processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_ADD_FRAG_HDR 0x20
/*! Egress attribute, UDP header create processing */
#define ADK_IPV6_EGRESS_ATTR_ACTION_ADD_UDP_HDR 0x40
/*! Egress attribute, translation v4 to v6*/
#define ADK_IPV6_EGRESS_ATTR_ACTION_TRANSLATE_TO_IPV4 0x80

/* Route look up Specific Definitions*/
#define ADK_IPV6_ROUTE_LOOKUP_INVALID_DEST_DROP_FN_ID "adk_ipv6.route_lookup_invalid_dest_addr_drop"
#define ADK_IPV6_EGRESS_ROUTE_LOOKUP_MULTICAST_FWD_FN_ID "adk_ipv6.adk_ipv6_multicast_egress_forwarding"

/*! IPv6 multicast specific definitions */
#define ADK_IPV6_LINK_LOCAL_ADDR_CHECK 0xff02
#define ADK_IPV6_NUM_MCAST_GROUPS 0x400
#define ADK_IPV6_MULTICAST_DIP_ONLY_FORWARD_TREE      "adk_ipv6.ipv6_multicast_dip_only_forward_tree"
#define ADK_IPV6_MULTICAST_DIP_ONLY_FORWARD_TREE_WIDTH     134
#define ADK_IPV6_PDU_ADDR_LINK_LOCAL_PREFIX_LENGTH 10
#define ADK_IPV6_PDU_ADDR_MULTICAST_PREFIX_LENGTH 8

#define ADK_IPV6_MULTICAST_FORWARD_TREE      "adk_ipv6.ipv6_multicast_forward_tree"
#define ADK_IPV6_MULTICAST_FORWARD_TREE_WIDTH     262
#define ADK_IPV6_MULTICAST_EG_FLOW_TREE      "adk_ipv6.ipv6_multicast_eg_flow_tree"
#define ADK_IPV6_MULTICAST_EG_FLOW_TREE_WIDTH     24
#define ADK_IPV6_MULTICAST_RPF_CHECK_ENABLE_FN_ID "adk_ipv6.adk_ipv6_multicast_forwarding_with_rpf_check"
#define ADK_IPV6_MULTICAST_RPF_CHECK_DISABLE_FN_ID "adk_ipv6.adk_ipv6_multicast_forwarding"
#define ADK_IPV6_MULTICAST_SEND_TO_ETH_BIT_LOC 0
#define ADK_IPV6_MULTICAST_SEND_TO_CPU_BIT_LOC 1
#define ADK_IPV6_MULTICAST_ALLOW_REASSEMZBLY_BIT_LOC 2
#define ADK_IPV6_MULTICAST_USE_PH_BIT_LOC 3
#define ADK_IPV6_MULTICAST_IGNORE_CHECKSUM_BIT_LOC 4
#define ADK_IPV6_MULTICAST_LOCALLY_GENERATED_PDU_BIT_LOC 24
#define ADK_IPV6_MULTICAST_CHECKSUM_RESULT_BIT_LOC 25
#define ADK_IPV6_MULTICAST_PDU_LENGTH_LOC 16
#define ADK_IPV6_MULTICAST_PDU_OFFSET_LOC 0
#define ADK_IPV6_MULTICAST_L2_INTERFACE_ID_LOC 0
#define ADK_IPV6_MULTICAST_FLOW_ID_LOC 8
#define ADK_IPV6_MULTICAST_GROUP_ID_LOC 0
#define ADK_IPV6_MULTICAST_PH_HANDLE_LOC 0
#define ADK_IPV6_MULTICAST_CB_FN_ID_LOC 24
#define ADK_IPV6_MULTICAST_NCA_QID_LOC 0
#define ADK_IPV6_MULTICAST_L4_CHKSUM_RESULT_LOC 8

#define ADK_IPV6_MULTICAST_DEFAULT_ENTRY_SIZE 8
#define ADK_IPV6_MULTICAST_DEFAULT_ENTRY 0xff


/*! The flow function names */
/*! The LLI direct bind function */
#define ADK_IPV6_MPP_LLI_DIRECT_BIND_FN "adk_ipv6.mpp_lli_direct_bind_fn"

/*! The ULI direct bind function */
#define ADK_IPV6_MPP_ULI_DIRECT_BIND_FN "adk_ipv6.mpp_uli_direct_bind_fn"

/*! The ULI in-direct bind function */
#define ADK_IPV6_MPP_ULI_INDIRECT_BIND_FN "adk_ipv6.mpp_uli_indirect_bind_fn"

/*! The CPU injectected pkt to IPv6 tunnel interface entry function*/
#define ADK_IPV6_PROCESS_CPU_PKTS_ENTRY_FN "adk_ipv6.ipv6_process_cpu_pkts"

/*! The CPU injectected pkt to IPv6oE interface entry function*/
#define ADK_IPV6_PROCESS_CPU_ETH_PKTS_ENTRY_FN "adk_ipv6.ipv6_process_cpu_eth_pkts"

/*! Exception/PH task related definitions. NOTE: that all the
 *  parameters may not be present always. Only iParams 1, 2,
 *  3, 4 and 5 are available. Rest are reserved for bind info
 *  or meta data.
 */
/*! Default Execption type for all - PH path send to host PDU */
#define ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE   0xFF
/*! Flow ID of the PDU, iParam1:24[0] */
#define ADK_IPV6_EXCEPTION_FLOW_ID_OFFSET 13
#define ADK_IPV6_EXCEPTION_FLOW_ID_SIZE 24
#define ADK_IPV6_EXCEPTION_FLOW_ID_SIZE_BYTES 3
/*! Length of the upper layer PDU as seen by IPv6, iParam2:16[16] */
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_LENGTH_OFFSET 17
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_LENGTH_SIZE 16
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_LENGTH_SIZE_BYTES 2
/*! Flags the PDU, iParam2:8[8] */
#define ADK_IPV6_EXCEPTION_FLAGS_OFFSET 10
#define ADK_IPV6_EXCEPTION_FLAGS_SIZE 8
#define ADK_IPV6_EXCEPTION_FLAGS_SIZE_BYTES 1
/*! Priority bits of the PDU, iParam2:8[0] */
#define ADK_IPV6_EXCEPTION_PRIORITY_OFFSET 11
#define ADK_IPV6_EXCEPTION_PRIORITY_SIZE 8
#define ADK_IPV6_EXCEPTION_PRIORITY_SIZE_BYTES 1
/*! Length of IPv6 PDU, iParam3:16[16] */
#define ADK_IPV6_EXCEPTION_IPV6_PDU_LENGTH_OFFSET 13
#define ADK_IPV6_EXCEPTION_IPV6_PDU_LENGTH_SIZE 16
#define ADK_IPV6_EXCEPTION_IPV6_PDU_LENGTH_SIZE_BYTES 2
/*! Offset of the upper layer PDU as seen by IPv6, iParam3:16[0] */
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_OFFSET_OFFSET 19
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_OFFSET_SIZE 8
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PDU_OFFSET_SIZE_BYTES 1
/*! Offset of the IPv6 header, iParam4:16[16] */
#define ADK_IPV6_EXCEPTION_IPV6_PDU_OFFSET_OFFSET 16
#define ADK_IPV6_EXCEPTION_IPV6_PDU_OFFSET_SIZE 8
#define ADK_IPV6_EXCEPTION_IPV6_PDU_OFFSET_SIZE_BYTES 1
/*! Offset of the L2 interface ID, iParam4:16[0] */
#define ADK_IPV6_EXCEPTION_L2_INTERFACE_ID_OFFSET 18
#define ADK_IPV6_EXCEPTION_L2_INTERFACE_ID_SIZE 16
#define ADK_IPV6_EXCEPTION_L2_INTERFACE_ID_SIZE_BYTES 2
/*! L4 protocol ID, iParam5:8[24] */
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PROTOCOL_ID_OFFSET 24
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PROTOCOL_ID_SIZE 8
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PROTOCOL_ID_SIZE_BYTES 1
/*! L2 association ID, iParam5:24[0] */
#define ADK_IPV6_EXCEPTION_L2_ASSOC_ID_OFFSET 17
#define ADK_IPV6_EXCEPTION_L2_ASSOC_ID_SIZE 24
#define ADK_IPV6_EXCEPTION_L2_ASSOC_ID_SIZE_BYTES 3

/*! Offset for the Upper layer Protocol in iParam1*/
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PROTOCOL_ID_FLOW_ID_OFFSET 20
#define ADK_IPV6_EXCEPTION_UPPER_LAYER_PROTOCOL_ID_BYTE_OFFSET 24
/*! MSB timestamp offset in iParam4*/
#define ADK_IPV6_MSB_TIMESTAMP_BYTE_OFFSET 0

/*! LSB timestamp offset in iParam5*/
#define ADK_IPV6_LSB_TIMESTAMP_BYTE_OFFSET 4

/*! Number of bytes for the MSB and LSB timestamp */
#define ADK_IPV6_TIMESTAMP_SIZE_BYTES 4


/*! Offset of the parameter which resulted in parameter problem
 *  in the task parameter, iParam3:16[0]
 */
#define ADK_IPV6_EXCEPTION_PARAM_PROBLEM_OFFSET 13
#define ADK_IPV6_EXCEPTION_PARAM_PROBLEM_OFFSET_SIZE 16
#define ADK_IPV6_EXCEPTION_PARAM_PROBLEM_OFFSET_SIZE_BYTES 2

/*! Generic constants */
#define ADK_IPV6_FLOW_ID_SIZE 24
#define ADK_IPV6_INVALID_MTM_QUEUE_ID 0xffffff
#define ADK_IPV6_INVALID_ULI_VRD_ID 0

/*! Vrd Host Mode Tree Macros */
#define ADK_IPV6_VRD_HOST_MODE_TREE  "adk_ipv6.ipv6_vrd_host_mode_tree"
#define ADK_IPV6_VRD_HOST_MODE_FN_ID "adk_ipv6.ipv6_vrd_host_mode_fn"
#define ADK_IPV6_RETURN_FN "adk_ipv6.ipv6_return"

/*! Custom Bind specific constants*/
#define ADK_IPV6_INGRESS_PROCESSING   0x1
#define ADK_IPV6_EGRESS_PROCESSING    0x0

/*! Maximum size of the reassembled PDU */
#define ADK_IPV6_REASSEMBLY_MAX_PDU_SIZE 64000

/*For testing on ASE kept less timeout*/
#ifndef ADK_IPV6_REASSEMBLY_TIMER_VALUE
#define ADK_IPV6_REASSEMBLY_TIMER_VALUE 0x000186A0  /* 100 milli seconds */
#endif


/* Used for purging the NextHop Handle binded to the Route Handle in case of Multipath Routing */
#define ADK_IPV6_DELETE_HASH_ENTRY_TIMEOUT_VALUE 0x0FFF3C /* 60sec */

/* IPv6 Translation definitions */
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_96  96
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_64 64
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_56 56
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_48 48 
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_40 40 
#define ADK_IPV6_TRANSLATION_PREFIX_VALUE_32 32

#define ADK_IPV6_NEXTHOP_MAX_WEIGHT 10
#define ADK_IPV6_MAX_NEXT_HOP_SUPPORTED 16

/* TREE TYPES */
#define ADK_IPV6_LPM_TREE    0
#define ADK_IPV6_OTREE       1
#define ADK_IPV6_OVTREESET   2


#define ADK_IPV6_TREE_REAL_OUTPUT_COUNT_MAX    10
#define ADK_IPV6_TREE_VIRTUAL_OUTPUT_COUNT_MAX 10
#define ADK_IPV6_TREE_INPUT_COUNT_MAX          10

#define ADK_IPV6_TREE_COUNT   65 


#define ADK_IPV6_TREE_OUTPUT_TYPE_VALUE        0
#define ADK_IPV6_TREE_OUTPUT_TYPE_ATTRIBUTE    1
#define ADK_IPV6_TREE_OUTPUT_TYPE_V6ADDRESS    2
#define ADK_IPV6_TREE_OUTPUT_TYPE_ATTR_NEG     3
#define ADK_IPV6_TREE_OUTPUT_TYPE_SIP_DIP      4


#define NCP_VP_FLOWID_MPP_netdriver_rx_rate_unlimited         0
#define NCP_VP_FLOWID_MPP_netdriver_rx_rate_limited           1


/* L3 Qos DSCP REmarking feature related Macros */
#define ADK_IPV6_PROFILE_ID_WIDTH                     5
#define ADK_IPV6_DSCP_MAP_FLAG                        0
#define ADK_IPV6_DSCP_MAP_PROFILE_ID_MAX              ((1 << ADK_IPV6_PROFILE_ID_WIDTH) - 1)



/* #ifdef ADK_XEON2_PROFILE_1 */
/* Common definitions for FLEXENB_PROFILEs */

/*! The lower inclusive bound of UDP ports which are reserved for compressed
 * ingress association flow lookup */
#ifdef ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MIN
#define ADK_IPV6_COMP_ING_ASSOC_FLOW_UDP_PORT_MIN    ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MIN
#else
#define ADK_IPV6_COMP_ING_ASSOC_FLOW_UDP_PORT_MIN   35000
#endif

/*! The upper inclusive bound of UDP ports which are reserved for compressed
 * ingress association flow lookup */
#ifdef ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MAX
#define ADK_IPV6_COMP_ING_ASSOC_FLOW_UDP_PORT_MAX    ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MAX
#else
#define ADK_IPV6_COMP_ING_ASSOC_FLOW_UDP_PORT_MAX    61000
#endif

/*IPv6 max supported protocols */
#define ADK_IPV6_MAX_SUPPORTED_PROTOCOLS 255

/* ipv6_comp_ingress_assoc_flow_lookup */
#define ADK_IPV6_COMP_ING_ASSOC_NUM_FIELDS    2
#define ADK_IPV6_COMP_ADDR_WIDTH              8
#define ADK_IPV6_L4PORT_WIDTH                 16
#define ADK_IPV6_COMP_ING_ASSOC_TREE_WIDTH    (ADK_IPV6_L4PORT_WIDTH+ ADK_IPV6_COMP_ADDR_WIDTH)
#define ADK_IPV6_MAX_COMP_IP_ADDRESS          256
#define ADK_IPV6_INGRESS_COMP_ASSOC_FLOW_TREE "adk_ipv6.ipv6_comp_ingress_assoc_flow_lookup"
/* #endif  / *  ADK_XEON2_PROFILE_1 * / */

/* Default qid */
#define IPV6_INVALID_QOS_QID                 0xFFFF

/*! MAX number of rules for IPv6 Egress QOS classifier */
#define ADK_IPV6_EGRESS_QOS_CLASSIFIER_MAX_RULES 128

#endif /* End of _ADK_IPV6_CONST_H_ */
 
/*! end addtogroup _ipv6
 * @}
 */
