#ifndef _ADK_IPV6_ING_CLASSIFIER_H_
#define _ADK_IPV6_ING_CLASSIFIER_H

#define ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES 1024
#define ADK_IPV6_CLASSIFIER_RULE_MAX_PRIORITY 0xFFFF
#define ADK_IPV6_FLOW_COOKIE    0xAD419F70

typedef enum {

	/*! IP protocol type mask */
	ADK_IPV6_ING_CLASSIFIER_PROTO_MASK     = 0x01,
	/*! Destination address mask */
	ADK_IPV6_ING_CLASSIFIER_DEST_ADDR_MASK = 0x02,
	/*! Source address mask */
	ADK_IPV6_ING_CLASSIFIER_SRC_ADDR_MASK  = 0x04,
	/*! Interface Handle mask */
	ADK_IPV6_ING_CLASSIFIER_INTF_HANDLE_MASK = 0x08,
	/*! Source Port mask */
	ADK_IPV6_ING_CLASSIFIER_SRC_PORT_MASK = 0x10,
	/*! Destination Port */
	ADK_IPV6_ING_CLASSIFIER_DEST_PORT_MASK  = 0x20,
	/*! ICMP Type MASK */
	ADK_IPV6_ING_CLASSIFIER_ICMP_TYPE_MASK  = 0x40,
	/*! ICMP Code MASK */
	ADK_IPV6_ING_CLASSIFIER_ICMP_CODE_MASK  = 0x80

} adk_ipv6_ing_classifier_rulemask_e;

typedef struct adk_ipv6_l4_port_info_s {
	adk_ipv6_range_t srcPort;
	adk_ipv6_range_t destPort;
} adk_ipv6_l4_port_info_t;

typedef struct adk_icmpv6_filter_info_s {
	adk_ipv6_range_t type;
	adk_ipv6_range_t code;
} adk_ipv6_icmp_filter_info_t;

typedef adk_ipv6_l4_port_info_t adk_ipv6_udp_port_info_t;
typedef adk_ipv6_l4_port_info_t adk_ipv6_tcp_port_info_t;
typedef adk_ipv6_l4_port_info_t adk_ipv6_sctp_port_info_t;

typedef struct {
	adk_ipv6_ing_classifier_rulemask_e ruleMask;
	uint8_t protocol;
	adk_ipv6_addr_prefix_t destAddr;
	adk_ipv6_addr_prefix_t srcAddr;
	adk_interface_handle_t ipv6IntfHandle;
	union {
		adk_ipv6_icmp_filter_info_t icmp;
		adk_ipv6_tcp_port_info_t tcp;
		adk_ipv6_udp_port_info_t udp;
		adk_ipv6_sctp_port_info_t sctp;
	} protoInfo_u;
} adk_ipv6_ingress_classifier_rule_t;

typedef enum {
	/*! Drop this packet immediately. A global stats counter will
	 *      * be incremented by default.
	 *           */
	ADK_IPV6_ING_CLASSIFIER_ACTION_DROP = 1,
	/*! Forward to a program running on host processor */
	ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_HOST = 2,
	/*! Forward to a EVENT OBJECT running in EVENT MACHINE */
	ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_EO = 3
} adk_ipv6_ingress_classifier_action_type_e;


typedef struct {
	adk_ipv6_ingress_classifier_action_type_e action;
	bool enablePolicing;
	adk_flow_handle_t policingHandle;
	uint32_t  eventMachineQueueId;
} adk_ipv6_ingress_classifier_action_t;

typedef struct adk_ipv6_dp_ing_classifier_action_info_s{ 
    adk_ipv6_ingress_classifier_action_type_e action;
    bool policingEnabled;
    uint32_t policerFlowId;
    uint32_t eventMachineQueueId;
}adk_ipv6_dp_ing_classifier_action_info_t;

typedef struct {
    /*! IPv6 ingress classifier rule */
    adk_ipv6_ingress_classifier_rule_t    ipv6Rules;
    /*! Ipv6 ingress classifier attributes */
    adk_ipv6_ingress_classifier_action_t  ipv6Attr;
} adk_ipv6_ingress_classifier_info_t;

adk_status_t adk_ipv6_ingress_classifier_rule_add(adk_module_handle_t ipv6Handle,
		const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
		const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p,
		const uint16_t ruleIndex);

adk_status_t adk_ipv6_ingress_classifier_rule_delete(adk_module_handle_t ipv6Handle,
		                const uint16_t ruleIndex);

adk_status_t adk_ipv6_ingress_classifier_build(adk_module_handle_t ipv6Handle);

adk_status_t adk_ipv6_ingress_classifier_stats_get(adk_module_handle_t  ipv6Handle,
		const uint16_t ruleIndex,
		adk_ipv6_ingress_classifier_stats_t  *ipv6Stats_p);

adk_status_t adk_ipv6_classifier_add_validate_arg(
        adk_module_handle_t ipv6Handle,
        const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
        const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p);

adk_status_t
adk_ipv6_get_ingress_classifier_all(
        adk_module_handle_t ipv6Handle,
        uint32_t *numClassifiers,
        adk_ipv6_ingress_classifier_info_t **ingressClassifier_p);
#endif
