/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipv6_flow.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6Functions  IPv6 Functions
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the APIs provided by the IPV6 module.
 *
 ***********************************************************************/
/* Infrastructure header files */
#include "adk_include.h"
#include "adk_ph_dev.h"
#include "acp_gen_interface.h"
#include "adk_utils_tree_update.h"
#include <netinet/in.h>
#include <arpa/inet.h>

#include "adk_shm.h"

/* IPV6 external header files */
#include "adk_ipv6.h"
#include "adk_ipv6_const.h"

/* IPV6 internal header files */
#include "adk_cp_acl_list.h"

#include "adk_ipv6_internal.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_export.h"
#include "adk_ph_dev_dp.h"
#include "adk_qos_include_internal.h"
#include "adk_ethernet.h"
#include "adk_ethernet_export.h"
#define ADK_CONVERT_TO_V6_VRD__HANDLE(__DATA) ((adk_ipv6_vrd_handle_t)__DATA)
#define ADK_CONVERT_TO_V6_FLOW__HANDLE(__DATA) ((adk_flow_handle_t)__DATA)

extern uint32_t ip_bit_mask[];

enum {
    IPV6_ING_CLASSIFIER_PROTO_FIELD,
    IPV6_ING_CLASSIFIER_SRC0_FIELD,
    IPV6_ING_CLASSIFIER_SRC1_FIELD,
    IPV6_ING_CLASSIFIER_SRC2_FIELD,
    IPV6_ING_CLASSIFIER_SRC3_FIELD,
    IPV6_ING_CLASSIFIER_DST0_FIELD,
    IPV6_ING_CLASSIFIER_DST1_FIELD,
    IPV6_ING_CLASSIFIER_DST2_FIELD,
    IPV6_ING_CLASSIFIER_DST3_FIELD,
    IPV6_ING_CLASSIFIER_SP_FIELD,
    IPV6_ING_CLASSIFIER_DP_FIELD,
    IPV6_ING_CLASSIFIER_INTF_FIELD_HDL,
    ADK_IPV6_ING_CLASSIFIER_NUM_FIELDS
};

enum {
    IPV6_ING_CLASSIFIER_PROTO_INPUT,
    IPV6_ING_CLASSIFIER_SRC0_INPUT,
    IPV6_ING_CLASSIFIER_SRC1_INPUT,	
    IPV6_ING_CLASSIFIER_SRC2_INPUT,
    IPV6_ING_CLASSIFIER_SRC3_INPUT,
    IPV6_ING_CLASSIFIER_DST0_INPUT,
    IPV6_ING_CLASSIFIER_DST1_INPUT,
    IPV6_ING_CLASSIFIER_DST2_INPUT,
    IPV6_ING_CLASSIFIER_DST3_INPUT,
    IPV6_ING_CLASSIFIER_L4PORTS_INPUT,
    IPV6_ING_CLASSIFIER_INTF_INPUT
};

RTE_ACL_RULE_DEF(adk_ipv6_ingress_classifier_rule_rte, ADK_IPV6_ING_CLASSIFIER_NUM_FIELDS);

static  struct  rte_acl_field_def adk_ipv6_ing_classifier_def[ADK_IPV6_ING_CLASSIFIER_NUM_FIELDS]  = {
    {
        .type = RTE_ACL_FIELD_TYPE_RANGE,
        .size = sizeof(uint8_t),
        .field_index  = IPV6_ING_CLASSIFIER_PROTO_FIELD,
        .input_index  = IPV6_ING_CLASSIFIER_PROTO_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, proto),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_SRC0_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_SRC0_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, sip0),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_SRC1_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_SRC1_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, sip1),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_SRC2_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_SRC2_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, sip2),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_SRC3_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_SRC3_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, sip3),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_DST0_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_DST0_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, dip0),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_DST1_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_DST1_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, dip1),
    },{
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_DST2_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_DST2_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, dip2),
    },{
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_DST3_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_DST3_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, dip3),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_RANGE,
        .size = sizeof(uint16_t),
        .field_index = IPV6_ING_CLASSIFIER_SP_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_L4PORTS_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, sPort),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_RANGE,
        .size = sizeof(uint16_t),
        .field_index = IPV6_ING_CLASSIFIER_DP_FIELD,
        .input_index = IPV6_ING_CLASSIFIER_L4PORTS_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, dPort),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_RANGE,
        .size = sizeof(uint32_t),
        .field_index = IPV6_ING_CLASSIFIER_INTF_FIELD_HDL,
        .input_index = IPV6_ING_CLASSIFIER_INTF_INPUT,
        .offset = offsetof(adk_ipv6_ing_classifier_ptn_t, ipIntfHandle),
    },
};

adk_status_t adk_ipv6_ing_classifier_hw_rule (
        adk_module_handle_t ipv6Handle,
        uint32_t precedence,
        uint32_t ingClassifierFlowId,
        const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
        const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p,
        uint8_t treeIndex
        )
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    struct adk_ipv6_ingress_classifier_rule_rte v = {0};
    adk_ipv6_dp_ing_classifier_action_info_t *classifierResult_p;
    adk_sys_handle_t sys_handle;
    uint32_t depth = 0;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    /*****   Build the Individual Patterns ******/

    /* Set the result before programming ID in dp
     * * set all the fields as is -
     * * control the access to field from DP.
     * * for e.g. only netd path supports policing as of now,
     * * no check required here - In DP it will access policingHandle only if
     * * action is send to netd
     * * Same principle applies for other fields
     * */
    classifierResult_p = &(ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierActionDp_p[treeIndex][ingClassifierFlowId]);
    classifierResult_p->action = ipv6IngClassifierAction_p->action;
    classifierResult_p->policingEnabled = ipv6IngClassifierAction_p->enablePolicing;
    classifierResult_p->eventMachineQueueId = ipv6IngClassifierAction_p->eventMachineQueueId;
    if (ipv6IngClassifierAction_p->enablePolicing) {
        retValue = adk_qos_policing_flow_id_get(sys_handle, ipv6IngClassifierAction_p->policingHandle, &classifierResult_p->policerFlowId);
        if (ADK_SUCCESS != retValue) {
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid policer handle\n");
            return retValue;
        }
    }else{
        classifierResult_p->policerFlowId = ADK_QOS_INVALID_POLICER_FLOW_ID;
    }
    ADK_IPV6_LOG_INFO(ipv6Handle, "\tclassifier - building rule: FlowId=%d, treeIndex=%d\n", ingClassifierFlowId, treeIndex);
    ADK_IPV6_LOG_INFO(ipv6Handle, "\t\tsip=%x %x %x %x  prefix=%d\n", ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[0],
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[1], ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[2],
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[3], ipv6IngClassifierRule_p->srcAddr.prefixLength);
    ADK_IPV6_LOG_INFO(ipv6Handle, "\t\tdip=%x %x %x %x prefix=%d\n", ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[0],
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[1], ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[2],
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[3], ipv6IngClassifierRule_p->destAddr.prefixLength);

    if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_PROTO_MASK)
    {
        ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tproto=%d\n", ipv6IngClassifierRule_p->protocol);
        /* Set RTE_ACL structure */
        v.field[IPV6_ING_CLASSIFIER_PROTO_FIELD].value.u8 =
            ipv6IngClassifierRule_p->protocol;
        v.field[IPV6_ING_CLASSIFIER_PROTO_FIELD].mask_range.u8  =
            ipv6IngClassifierRule_p->protocol;
        /* Set protocol specific port values */
        switch(ipv6IngClassifierRule_p->protocol)
        {
        case ADK_IPV6_PROTOCOL_ICMPV6:
            if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_TYPE_MASK)
            {
                ADK_IPV6_LOG_INFO(ipv6Handle,"\t\ticmpType=%d-%d\n",
                        ipv6IngClassifierRule_p->protoInfo_u.icmp.type.begin,
                        ipv6IngClassifierRule_p->protoInfo_u.icmp.type.end);
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.icmp.type.begin;
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.icmp.type.end;
            }
            else
            {
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 = 0;
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 = 0xFFFF;
            }

            if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_CODE_MASK)
            {
                ADK_IPV6_LOG_INFO(ipv6Handle,"\t\ticmpCode=%d-%d\n",
                        ipv6IngClassifierRule_p->protoInfo_u.icmp.code.begin,
                        ipv6IngClassifierRule_p->protoInfo_u.icmp.code.end);
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.icmp.code.begin;
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.icmp.code.end;
            }
            else
            {
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 = 0;
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 = 0xFFFF;
            }
            break;

        case ADK_IPV6_PROTOCOL_TCP:
        case ADK_IPV6_PROTOCOL_UDP:
        case ADK_IPV6_PROTOCOL_SCTP:
            if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_PORT_MASK)
            {
                ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tsPort=%d-%d\n",
                        ipv6IngClassifierRule_p->protoInfo_u.udp.srcPort.begin,
                        ipv6IngClassifierRule_p->protoInfo_u.udp.srcPort.end);
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.udp.srcPort.begin;
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.udp.srcPort.end;
            }
            else
            {
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 = 0;
                v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 = 0xFFFF;
            }
            if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_PORT_MASK)
            {
                ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tdPort=%d-%d\n",
                        ipv6IngClassifierRule_p->protoInfo_u.udp.destPort.begin,
                        ipv6IngClassifierRule_p->protoInfo_u.udp.destPort.end);
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.udp.destPort.begin;
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 =
                    ipv6IngClassifierRule_p->protoInfo_u.udp.destPort.end;
            }
            else
            {
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 = 0;
                v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 = 0xFFFF;
            }
            break;
        default:
            /* Set ANY for SP and DP */
            v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 = 0;
            v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 = 0xFFFF;
            v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 = 0;
            v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 = 0xFFFF;
            break;
        }
    }
    else
    {
        /* Set ANY for protocol */
        v.field[IPV6_ING_CLASSIFIER_PROTO_FIELD].value.u8 = 0;
        v.field[IPV6_ING_CLASSIFIER_PROTO_FIELD].mask_range.u8  = 0xFF;
        /* Set ANY for SP and DP */
        v.field[IPV6_ING_CLASSIFIER_SP_FIELD].value.u16 = 0;
        v.field[IPV6_ING_CLASSIFIER_SP_FIELD].mask_range.u16 = 0xFFFF;
        v.field[IPV6_ING_CLASSIFIER_DP_FIELD].value.u16 = 0;
        v.field[IPV6_ING_CLASSIFIER_DP_FIELD].mask_range.u16 = 0xFFFF;
    }
    /* Set Source address */
    if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_ADDR_MASK)
    {
        ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tsip=%x %x %x %x  prefix=%d\n", ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[0],ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[1], ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[2], ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[3], ipv6IngClassifierRule_p->srcAddr.prefixLength);

        v.field[IPV6_ING_CLASSIFIER_SRC0_FIELD].value.u32 =
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[0];
        v.field[IPV6_ING_CLASSIFIER_SRC1_FIELD].value.u32 =
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[1];
        v.field[IPV6_ING_CLASSIFIER_SRC2_FIELD].value.u32 =
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[2];
        v.field[IPV6_ING_CLASSIFIER_SRC3_FIELD].value.u32 =
            ipv6IngClassifierRule_p->srcAddr.Ipv6Addr.addr[3];


        depth = ipv6IngClassifierRule_p->srcAddr.prefixLength;
        v.field[IPV6_ING_CLASSIFIER_SRC0_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_SRC1_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_SRC2_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_SRC3_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
    }
    else
    {
        /* Set ANY */
        v.field[IPV6_ING_CLASSIFIER_SRC0_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_SRC1_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_SRC2_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_SRC3_FIELD].value.u32 = 0;

        v.field[IPV6_ING_CLASSIFIER_SRC0_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_SRC1_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_SRC2_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_SRC3_FIELD].mask_range.u32  = 0xFFFFFFFF;
    }
    /* Set Destination address */
    if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_ADDR_MASK)
    {
        ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tdip=%x %x %x %x prefix=%d\n", ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[0],ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[1], ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[2], ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[3], ipv6IngClassifierRule_p->destAddr.prefixLength);

        v.field[IPV6_ING_CLASSIFIER_DST0_FIELD].value.u32 =
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[0];
        v.field[IPV6_ING_CLASSIFIER_DST1_FIELD].value.u32 =
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[1];
        v.field[IPV6_ING_CLASSIFIER_DST2_FIELD].value.u32 =
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[2];
        v.field[IPV6_ING_CLASSIFIER_DST3_FIELD].value.u32 =
            ipv6IngClassifierRule_p->destAddr.Ipv6Addr.addr[3];


        depth = ipv6IngClassifierRule_p->destAddr.prefixLength;
        v.field[IPV6_ING_CLASSIFIER_DST0_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_DST1_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_DST2_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
        v.field[IPV6_ING_CLASSIFIER_DST3_FIELD].mask_range.u32  = (depth > 32) ? 32 : depth;
        depth = (depth > 32) ? (depth - 32) : 0;
    }
    else
    { /* SET ANY */
        v.field[IPV6_ING_CLASSIFIER_DST0_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_DST1_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_DST2_FIELD].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_DST3_FIELD].value.u32 = 0;

        v.field[IPV6_ING_CLASSIFIER_DST0_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_DST1_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_DST2_FIELD].mask_range.u32  = 0xFFFFFFFF;
        v.field[IPV6_ING_CLASSIFIER_DST3_FIELD].mask_range.u32  = 0xFFFFFFFF;
    }

    /* Set Interface handle */
    if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_INTF_HANDLE_MASK)
    {
        ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tifaceHd=%x\n", (uint32_t)(size_t) ADK_SYS_INTERFACE_ID_GET(ipv6IngClassifierRule_p->ipv6IntfHandle));
        /* Store ID instead of handle */
        v.field[IPV6_ING_CLASSIFIER_INTF_FIELD_HDL].value.u32 =
            (uint32_t)(size_t) ADK_SYS_INTERFACE_ID_GET(ipv6IngClassifierRule_p->ipv6IntfHandle);
        v.field[IPV6_ING_CLASSIFIER_INTF_FIELD_HDL].mask_range.u32  =
            (uint32_t)(size_t) ADK_SYS_INTERFACE_ID_GET(ipv6IngClassifierRule_p->ipv6IntfHandle);
    }
    else
    {
        /* Set ANY */
        v.field[IPV6_ING_CLASSIFIER_INTF_FIELD_HDL].value.u32 = 0;
        v.field[IPV6_ING_CLASSIFIER_INTF_FIELD_HDL].mask_range.u32  = 0xFFFFFFFF;
    }
    ADK_IPV6_LOG_INFO(ipv6Handle,"\t\taction=%d\n", ipv6IngClassifierAction_p->action);
    v.data.category_mask = 1;
    v.data.priority = precedence;
    v.data.userdata = ingClassifierFlowId;
    if (rte_acl_add_rules(
                ((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p)->ipv6IngClassifierCtx[treeIndex],
                (struct rte_acl_rule *)&v, 1) < 0)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Error inserting ACL rule in IP\n");
        return ADK_E_OTREE_INSERT_FAILED ;
    }

    ADK_RETURN(retValue);
}

adk_status_t adk_ipv6_ing_classifer_build(adk_module_handle_t ipv6Handle, adk_ipv6_ing_classifier_ctx_t *ctx)
{
    struct rte_acl_config cfg;
    int ret=0;

    ADK_MEMSET(&cfg, 0, sizeof(cfg));
    cfg.num_fields = ADK_IPV6_ING_CLASSIFIER_NUM_FIELDS;
    ADK_MEMCPY(&cfg.defs, adk_ipv6_ing_classifier_def, sizeof(adk_ipv6_ing_classifier_def));
    cfg.num_categories = 1;
    cfg.max_size = 0;

    ADK_IPV6_LOG_INFO(ipv6Handle, "rte_acl_build : building IPv6 Ingress Classifier context\n");

    ret = rte_acl_build(ctx, &cfg);
    if (ret != 0)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, ret, "rte_acl_build failed\n");
        return ADK_E_OTREE_BUILD_FAILED;
    }

    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * @internal
 * adk_ipv6_ingress_convert_to_hos_rule
 * @param[in,out]
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval
 *
 * @brief
 *
 * Internal support function for ACL rule conversion.
 *
 * =============================================================
*/
static
adk_status_t adk_ipv6_ingress_convert_to_hos_rule(adk_module_handle_t ipv6Handle,
        uint32_t ingClassifierFlowId,
        const adk_ipv6_ingress_classifier_rule_t *ipIngClassifierRule_p,
        const adk_ipv6_ingress_classifier_action_t *ipIngClassifierAction_p,
        uint16_t *num_hos_rules_p,
        adk_hos_ip_classifier_tbl_rule_attr_t **rule_attr)
{
    uint32_t sport_begin = 0, sport_end = 0xFFFF;
    uint32_t dport_begin = 0, dport_end = 0xFFFF;
    adk_acl_range_bitmask_t *sport_bitmask_p = NULL, *dport_bitmask_p = NULL;
    adk_acl_range_bitmask_t *tmp_sport_p, *tmp_dport_p;
    int sport_rule_count=1, dport_rule_count=1;
    int total_rule_count, loop1, loop2;
    int rule_count;
    adk_status_t retValue= ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    uint8_t depth=0;
    adk_module_handle_t eth_mod_handle;

    RTE_SET_USED(ipIngClassifierAction_p);

    if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_PROTO_MASK)
    {
        /* All Known protocol specific information is processed here */
        switch (ipIngClassifierRule_p->protocol)
        {
        case ADK_IPV6_PROTOCOL_ICMPV6:
            if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_TYPE_MASK) {
                sport_begin = ipIngClassifierRule_p->protoInfo_u.icmp.type.begin;
                sport_end = ipIngClassifierRule_p->protoInfo_u.icmp.type.end;
            }
            if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_CODE_MASK) {
                dport_begin = ipIngClassifierRule_p->protoInfo_u.icmp.code.begin;
                dport_end = ipIngClassifierRule_p->protoInfo_u.icmp.code.end;
            }
            break;

        case ADK_IPV6_PROTOCOL_TCP:
        case ADK_IPV6_PROTOCOL_SCTP:
        case ADK_IPV6_PROTOCOL_UDP:
            if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_PORT_MASK) {
                sport_begin = ipIngClassifierRule_p->protoInfo_u.udp.srcPort.begin;
                sport_end = ipIngClassifierRule_p->protoInfo_u.udp.srcPort.end;
            }
            if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_PORT_MASK) {
                dport_begin = ipIngClassifierRule_p->protoInfo_u.udp.destPort.begin;
                dport_end = ipIngClassifierRule_p->protoInfo_u.udp.destPort.end;
            }
            break;

        default:
            break;
        }               /* -----  end switch  ----- */
    }

    if ((ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_PORT_MASK) ||
        (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_TYPE_MASK))
    {
        /*Allocate memory for storing bitmask and Range info*/
        sport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
			 ADK_IPV6_HOS_RULE_OBJ,
			 1,sizeof(adk_acl_range_bitmask_t )*
                            ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if(!sport_bitmask_p)
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
        }

        sport_rule_count = adk_acl_convert_range_bitmask(sport_begin,sport_end, 16, sport_bitmask_p);

        if(sport_rule_count <= 0)
        {
            ADK_IPV6_LOG_INFO(ipv6Handle,"Rule splitting not possible for sport\n");
            retValue = ADK_DP_E_ERROR;
            goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
        }
    }

    if ((ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_PORT_MASK) ||
        (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_CODE_MASK))
    {
        /*Allocate memory for storing bitmask and Range info*/
        dport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
			 ADK_IPV6_HOS_RULE_OBJ,
			 1,sizeof(adk_acl_range_bitmask_t )*
                            ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if(!dport_bitmask_p)
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
        }

        dport_rule_count = adk_acl_convert_range_bitmask(dport_begin, dport_end, 16, dport_bitmask_p);
        if(dport_rule_count <= 0)
        {
            ADK_IPV6_LOG_INFO(ipv6Handle, "Rule splitting not possible for dport\n");
            retValue = ADK_DP_E_ERROR;
            goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
        }
    }

    total_rule_count = sport_rule_count * dport_rule_count;
 
    ADK_IPV6_LOG_INFO(ipv6Handle, "sport_rule_count: %d, dport_rule_count: %d \n",
            sport_rule_count, dport_rule_count);

    if(total_rule_count > ADK_ACL_MAX_HOS_RULES_PER_RULE)
    {
        ADK_IPV6_LOG_INFO(ipv6Handle, "Rule splitting crossing limits for hos rule count \n");
        retValue = ADK_DP_E_ERROR;
        goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
    }

    for(loop1 =0; loop1< total_rule_count; loop1++)
    {
        rule_attr[loop1] = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_HOS_RULE_OBJ,
				 1, sizeof(adk_hos_ip_classifier_tbl_rule_attr_t));
        if(!rule_attr[loop1])
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
        }
    }

    *num_hos_rules_p = total_rule_count;  //Update hos Rule count

    tmp_dport_p = dport_bitmask_p;
    tmp_sport_p = sport_bitmask_p;

    rule_count = 0;
    adk_hos_ip_classifier_tbl_rule_attr_t *hos_rule = rule_attr[rule_count];

    for(loop1 = 0; loop1 < sport_rule_count; loop1++)
    {
        for(loop2=0; loop2 < dport_rule_count; loop2++)
        {
                hos_rule->is_ipv6 = TRUE;
#if 0
                hos_rule->precedence = precedence;
#endif
                hos_rule->is_disabled = false;
                hos_rule->rule_id = ingClassifierFlowId;

                if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_INTF_HANDLE_MASK)
                {
                    uint32_t interface_id = ADK_SYS_INTERFACE_ID_GET(ipIngClassifierRule_p->ipv6IntfHandle);
                    retValue = adk_module_handle_get(sys_handle, ADK_ETHERNET_MODULE_ID,
                                    adk_ethernet_service_function_update, &eth_mod_handle);

                    if(ADK_SUCCESS != retValue)
                    {
                        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue, "Ethernet module get handle failed\n");
                        retValue = ADK_E_MODULE_NOT_REGISTERED;
                        goto _adk_ipv6_ingress_convert_to_hos_rule_exit;
                    }
                    hos_rule->vlan_id = ADK_HOS_ANY_VLAN;
                    adk_eth_intf_get_vlan(eth_mod_handle, interface_id, &hos_rule->vlan_id);
                    hos_rule->useVlan = true;
                }

                if ((ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_PORT_MASK) ||
                    (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_TYPE_MASK))
                {
                    hos_rule->l4_info1 = tmp_sport_p->val;
                    hos_rule->l4_info1_mask = tmp_sport_p->mask;
                }
                if ((ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_PORT_MASK) ||
                    (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_ICMP_CODE_MASK))
                {
                    hos_rule->l4_info2 = tmp_dport_p->val;
                    hos_rule->l4_info2_mask = tmp_dport_p->mask;
                }
                if (ipIngClassifierRule_p->ruleMask &
                        ADK_IPV6_ING_CLASSIFIER_SRC_ADDR_MASK)
                {
                    hos_rule->sip_addr[0] =
                        adk_htonl(ipIngClassifierRule_p->srcAddr.Ipv6Addr.addr[0]);
                    hos_rule->sip_addr[1] =
                        adk_htonl(ipIngClassifierRule_p->srcAddr.Ipv6Addr.addr[1]);
                    hos_rule->sip_addr[2] =
                        adk_htonl(ipIngClassifierRule_p->srcAddr.Ipv6Addr.addr[2]);
                    hos_rule->sip_addr[3] =
                        adk_htonl(ipIngClassifierRule_p->srcAddr.Ipv6Addr.addr[3]);

                    depth = ipIngClassifierRule_p->srcAddr.prefixLength;
                    hos_rule->sip_addr_mask[0] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->sip_addr_mask[1] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->sip_addr_mask[2] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->sip_addr_mask[3] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                }

                if (ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_ADDR_MASK)
                {
                    hos_rule->dip_addr[0] =
                        adk_htonl(ipIngClassifierRule_p->destAddr.Ipv6Addr.addr[0]);   
                    hos_rule->dip_addr[1] =
                        adk_htonl(ipIngClassifierRule_p->destAddr.Ipv6Addr.addr[1]);
                    hos_rule->dip_addr[2] =
                        adk_htonl(ipIngClassifierRule_p->destAddr.Ipv6Addr.addr[2]);
                    hos_rule->dip_addr[3] =
                        adk_htonl(ipIngClassifierRule_p->destAddr.Ipv6Addr.addr[3]);

                    depth = ipIngClassifierRule_p->destAddr.prefixLength;
                    hos_rule->dip_addr_mask[0] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->dip_addr_mask[1] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->dip_addr_mask[2] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                    depth = (depth > 32) ? (depth - 32) : 0;
                    hos_rule->dip_addr_mask[3] = adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
                }

                if(ipIngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_PROTO_MASK)
                {
                    hos_rule->proto = ipIngClassifierRule_p->protocol;
                    hos_rule->proto_mask = 0xff;
                }

                rule_count++;
                hos_rule = rule_attr[rule_count]; //Move to next hos rule
            tmp_dport_p +=1; // Move to next dport mask
        } /* Dport loop ends*/
        tmp_sport_p +=1;    //move to next sport mask
        tmp_dport_p = dport_bitmask_p; //dport start
    }/*Sport loop ends*/

_adk_ipv6_ingress_convert_to_hos_rule_exit:
    if(dport_bitmask_p)
        ADK_NVM_FREE(dport_bitmask_p);
    if(sport_bitmask_p)
        ADK_NVM_FREE(sport_bitmask_p);
    return retValue;
}

adk_status_t adk_ipv6_ingress_classifier_create(
        adk_module_handle_t ipv6Handle,
        adk_ipv6_ing_classifer_info_t  **ingClassifierInfo_pp)
{
    struct rte_acl_param prm;
    uint8_t   i;
    uint16_t  actionIndexDp;
    char name[50];
    adk_status_t    retValue;
    adk_acl_info_t cpListInfo={0};
    adk_ipv6_ingress_classifier_rule_t ipv6IngClassifierRule;
    adk_ipv6_ingress_classifier_action_t ipv6IngClassifierAction;
#define ACTIVE_TREE_INDEX_INIT 0
#define SHADOW_TREE_INDEX_INIT 1

    adk_ipv6_ing_classifer_info_t *ingClassifierInfo_p = NULL;
    adk_ipv6_info_t* ipv6ModuleInfo_p =
        (adk_ipv6_info_t *) ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ingClassifierInfo_p = (adk_ipv6_ing_classifer_info_t *)
        ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_ING_CLASSIFIER_INFO_OBJ, 1,
                sizeof(adk_ipv6_ing_classifer_info_t));

    if (ingClassifierInfo_p == NULL)
    {
        ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_MEM_UNAVAILABLE,
                "Memory allocation failed\n");
        return ADK_E_MEM_UNAVAILABLE;
    }
    *ingClassifierInfo_pp = ingClassifierInfo_p;

    prm.socket_id = SOCKET_ID_ANY;
    prm.rule_size = RTE_ACL_RULE_SZ(ADK_IPV6_ING_CLASSIFIER_NUM_FIELDS);
    prm.max_rule_num = ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES + 1;

    actionIndexDp = adk_rm_resource_reserve(ipv6ModuleInfo_p->ipv6IngClassifierRuleIDHandle);
    if (1 != actionIndexDp)
    {
        ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_RESOURCE_UNAVAILABLE,
                "Could not get resource id ING ORDERED CLASSIFIER using rte_acl_create!\n");
        ADK_NVM_FREE(ingClassifierInfo_p);
        return ADK_E_RESOURCE_UNAVAILABLE;
    }
    for (i=0; i<2; i ++)
    {
        /* Initialize */
        ingClassifierInfo_p->ipv6IngClassifierCtx[i] = NULL;

        snprintf(name, sizeof(name), "ADK_IPV6_INGRESS_ORDERED_CLASSIFIER_%1d",i);
        prm.name = name;
        ADK_IPV6_LOG_INFO(ipv6Handle, "rte_acl_create : creating new Ing Ordered classifier context: %s\n", prm.name);

        ingClassifierInfo_p->ipv6IngClassifierCtx[i] =
            (adk_ipv6_ing_classifier_ctx_t*) rte_acl_create(&prm);
        if (NULL == ingClassifierInfo_p->ipv6IngClassifierCtx[i])
        {
            if (1 == i)
            {
                ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_MEM_UNAVAILABLE,
                        "Could not create ING ORDERED CLASSIFIER using rte_acl_create!\n");
                /* free previous one */
                rte_acl_free(ingClassifierInfo_p->ipv6IngClassifierCtx[0]);
            }
            return ADK_E_MEM_UNAVAILABLE;
        }
    }
    ingClassifierInfo_p->aclTreeIndex = ACTIVE_TREE_INDEX_INIT;
    cpListInfo.user = ADK_ACL_IPV6_ING_CLASSIFIER;
    if(ADK_SUCCESS != adk_acl_create(&(ingClassifierInfo_p->cpList),
                &cpListInfo))
    {
        retValue = ADK_E_MEM_UNAVAILABLE;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not allocate CP list for IP ING CLASSIFIER!\n");
        goto free_rte_acl_ctx;
    }
    /*
     *      * Set first default Rule as ANY and action as DENY
     *           * it goes with highest priority
     *                */
    /* Set Rule mask as zero, indicating no mask */
    ipv6IngClassifierRule.ruleMask = 0;
    memset(&ipv6IngClassifierAction, 0x0, sizeof(ipv6IngClassifierAction));
    ipv6IngClassifierAction.action = ADK_IPV6_ING_CLASSIFIER_ACTION_DROP;
    if(ADK_SUCCESS != adk_ipv6_ing_classifier_hw_rule (
                ipv6Handle,
                ADK_IPV6_CLASSIFIER_RULE_MAX_PRIORITY,
                actionIndexDp,
                &ipv6IngClassifierRule,
                &ipv6IngClassifierAction,
                ACTIVE_TREE_INDEX_INIT
                ))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not add default DENY rule to IP ING CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }
    /* Build */
    if( ADK_SUCCESS != adk_ipv6_ing_classifer_build(ipv6Handle,
                ingClassifierInfo_p->ipv6IngClassifierCtx[ACTIVE_TREE_INDEX_INIT]))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not BUILD default DENY rule to IP ING CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }

    ipv6IngClassifierRule.ruleMask = 0;
    memset(&ipv6IngClassifierAction, 0x0, sizeof(ipv6IngClassifierAction));
    ipv6IngClassifierAction.action = ADK_IPV6_ING_CLASSIFIER_ACTION_DROP;

    if(ADK_SUCCESS != adk_ipv6_ing_classifier_hw_rule (
                ipv6Handle,
                ADK_IPV6_CLASSIFIER_RULE_MAX_PRIORITY,
                actionIndexDp,
                &ipv6IngClassifierRule,
                &ipv6IngClassifierAction,
                SHADOW_TREE_INDEX_INIT
                ))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not add default DENY rule to IP ING CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }
    /* Build */
    if( ADK_SUCCESS != adk_ipv6_ing_classifer_build(ipv6Handle,
                ingClassifierInfo_p->ipv6IngClassifierCtx[SHADOW_TREE_INDEX_INIT]))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not BUILD default DENY rule to IP ING CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }
    return ADK_SUCCESS;

free_rte_acl_ctx:
    adk_rm_resource_free(ipv6ModuleInfo_p->ipv6IngClassifierRuleIDHandle,
            actionIndexDp);
    rte_acl_free(ingClassifierInfo_p->ipv6IngClassifierCtx[0]);
    rte_acl_free(ingClassifierInfo_p->ipv6IngClassifierCtx[1]);
    return retValue;
} /* adk_ipv6_ingress_classifier_create */


/*!
* =============================================================
* adk_ipv6_ingress_classifier_rule_add
* @param[in]    ipv6Handle – Ipv6 module handle.
*
* @param[in]    ruleIndex - index at which the rule will be inserted
*
* @param[in]    ipv6IngClassifierRule_p  - pointer to adk_ipv6_ingress_classifier_rule_t 
* data structure.
*
* @param[in]    ipv6IngClassifierAction_p  - pointer to adk_ipv6_ingress_classifier_action_t
* data structure.
*
* @retval ADK_SUCCESS Successful completion
* @retval ADK_E_INVALID_PARAMETER -  one of the direct arguments
* to the API call is invalid. Possibly rule index specified is either
* out of sequence or there is no filter entry at index 0.
*
* @brief
* This API validates the input parameter and adds a new rule to the ipv6 ingress
* association ordered tree. Note that the rule does not get reflected in
* data plane until the adk_ipv6_ingress_classifier_build() API is called for this.
*
* =============================================================
*/

adk_status_t adk_ipv6_ingress_classifier_rule_add(adk_module_handle_t ipv6Handle,
        const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
        const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p,
        const uint16_t ruleIndex)

{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_ipv6_flow_info_t *myFlowInfo_p;
    adk_sys_handle_t sys_handle;
    adk_acl_rule_t *cpRule_p;
    size_t ingAssocRuleId = ADK_RM_RSRC_INVALID;
    adk_ipv6_ingress_classifier_rule_handle_t ipv6IngClassifierFlowHandle_p = 0;
    uint8_t i =0;
    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if (ipv6ModuleInfo_p->maxIpv6ClassifierRules >= ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue," Max Ingress classifier rules limit reached:%d\n",
                ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES);
        goto release_lock_and_return;
    }

    /* Validate input args */
    retValue = adk_ipv6_classifier_add_validate_arg( ipv6Handle, ipv6IngClassifierRule_p,
            ipv6IngClassifierAction_p);

    if (ADK_SUCCESS != retValue)
        goto release_lock_and_return;

    ingAssocRuleId = adk_rm_resource_reserve(
            ipv6ModuleInfo_p->ipv6IngClassifierRuleIDHandle);
    if (ADK_RM_RSRC_INVALID == ingAssocRuleId)
    {
        goto release_lock_and_return;
    }
    /* Clearing to indicate no handle is allocated yet */
    ipv6IngClassifierFlowHandle_p = 0x0;
    /*Set the flow stats to 0*/
    for(i=0; i <ADK_MAX_NUM_CORES; ++i)
    {
        (ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats)[i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ingAssocRuleId].rxPkts = 0;
        (ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats)[i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ingAssocRuleId].txPkts = 0;
        (ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats)[i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ingAssocRuleId].policerDropPkts = 0;
    }

    /* Allocate flow handle */
    ADK_SYS_FLOW_HANDLE_ALLOC(ipv6Handle, ADK_IPV6_ING_CLASSIFIER_RULE_INFO_OBJ,
            sizeof(adk_ipv6_flow_info_t),
            &ipv6IngClassifierFlowHandle_p);

    /************/
    if (ipv6IngClassifierFlowHandle_p == NULL)
    {
        goto release_lock_and_return;
    }
    else
    {
        myFlowInfo_p = (adk_ipv6_flow_info_t *)
            ADK_SYS_FLOW_ATTR_PTR_GET(ipv6IngClassifierFlowHandle_p);
    }

    if (myFlowInfo_p == NULL)
    {
        goto release_lock_and_return;
    }
    myFlowInfo_p->cookie = ADK_IPV6_FLOW_COOKIE;
    myFlowInfo_p->type = ADK_IPV6_FLOW_TYPE_INGRESS_CLASSIFIER_FLOW;

    ADK_SYS_FLOWID_SET(ipv6IngClassifierFlowHandle_p, ingAssocRuleId);

    ADK_MEMCPY((void *) &(myFlowInfo_p->ingClassifierFlow.rule),
            (const void *) ipv6IngClassifierRule_p,
            sizeof(adk_ipv6_ingress_classifier_rule_t));
    ADK_MEMCPY((void *) &(myFlowInfo_p->ingClassifierFlow.action),
            (const void *) ipv6IngClassifierAction_p,
            sizeof(adk_ipv6_ingress_classifier_action_t));
    /*
     *      * Add rule to ADK CP rule list.
     *           * The rules will be added during the next commit.
     *                */
    cpRule_p = &(myFlowInfo_p->ingClassifierFlow.cp_rule);
    cpRule_p->user_data = (void *)(ipv6IngClassifierFlowHandle_p);

    /* Convert this rule to a expanded set of hos rules */
    adk_hos_ip_classifier_tbl_rule_attr_t *hos_rule[ADK_ACL_MAX_HOS_RULES_PER_RULE];

    retValue = adk_ipv6_ingress_convert_to_hos_rule(ipv6Handle, ingAssocRuleId,
            ipv6IngClassifierRule_p, ipv6IngClassifierAction_p,
            &(cpRule_p->num_hos_rules), hos_rule);
    if (retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Failed to convert ingress classifier"
                " rule to HOS rule\n");
        goto release_lock_and_return;
    }

    for(i = 0; i< cpRule_p->num_hos_rules; i++)
    {
        cpRule_p->hos_rule_attr[i] = (adk_hos_rule_attr_t *)hos_rule[i];
    }

    /************/
    retValue = adk_acl_rule_add(
            ((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p)->cpList,
            ruleIndex,
            cpRule_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Failed to add the rule in CP list\n");
        goto release_lock_and_return;
    }
    ipv6ModuleInfo_p->maxIpv6ClassifierRules++;
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

    return ADK_SUCCESS;

release_lock_and_return:
    /* Upon error, release the reserved resources */
    if (ADK_RM_RSRC_INVALID != ingAssocRuleId)
    {
        adk_rm_resource_free(ipv6ModuleInfo_p->ipv6IngClassifierRuleIDHandle, ingAssocRuleId);
    }

    if (retValue != ADK_SUCCESS)
    {
        if (ipv6IngClassifierFlowHandle_p)
        {
            ADK_SYS_FLOW_HANDLE_FREE(ipv6IngClassifierFlowHandle_p);
        }
    }

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);
}

/*!
* =============================================================
* adk_ipv6_ingress_classifier_build
* @param[in]    ipv6Handle – Ipv6 module handle.
*
* @brief
* This API builds ipv6 ingress association ordered tree, which will be actually looked up by Data plane.
*/

adk_status_t adk_ipv6_ingress_classifier_build(adk_module_handle_t ipv6Handle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_ipv6_ing_classifer_info_t  *ingClassifierInfo_p = NULL;
    adk_sys_handle_t sys_handle;
    uint8_t activeTreeId, shadowTreeId;
    uint16_t ruleIndex;
    adk_ipv6_flow_info_t *myFlowInfo_p;
    size_t ingAssocRuleId;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    adk_acl_rule_t cpRule;
    adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p;
    adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }
    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    ingClassifierInfo_p = (adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p;
    /*
     *      * Clear the Ruleset & add the rules from the ADK CP rule list.
     *           * Then commit the ruleset.
     *                */
    if(ingClassifierInfo_p->aclTreeIndex)
    {
        activeTreeId = 1;
        shadowTreeId = 0;
    }
    else
    {
        activeTreeId = 0;
        shadowTreeId = 1;
    }
    /* Only update Shadow tree. Active tree update is not really required
     *      * Next build will atomatically update active now which will be shadow then
     *           */
    rte_acl_reset_rules(ingClassifierInfo_p->ipv6IngClassifierCtx[shadowTreeId]);
    /*Update CP version id for VRDs*/
    //	for(i=0;i<ADK_IPV6_VRD_ID_MAX;++i)
    //		(ipv6ModuleInfo_p->adk_ipv6_dp_tables_t->ip_cp_version_id[i])++;

    adk_acl_get_reset(ingClassifierInfo_p->cpList);
    ruleIndex = 0;


    while(ADK_SUCCESS == adk_acl_next_rule_get(
                ingClassifierInfo_p->cpList, &cpRule))
    {
        myFlowInfo_p = (adk_ipv6_flow_info_t *)
            ADK_SYS_FLOW_ATTR_PTR_GET(cpRule.user_data);
        ipv6IngClassifierRule_p = &(myFlowInfo_p->ingClassifierFlow.rule);
        ipv6IngClassifierAction_p = &(myFlowInfo_p->ingClassifierFlow.action);
        ingAssocRuleId = ADK_SYS_FLOWID_GET(cpRule.user_data);
        /*TODO: enhance this call to perform bulk add*/
        if( ADK_SUCCESS != adk_ipv6_ing_classifier_hw_rule(
                    ipv6Handle,
                    (ADK_IPV6_CLASSIFIER_RULE_MAX_PRIORITY - ruleIndex),
                    ingAssocRuleId,
                    ipv6IngClassifierRule_p,
                    ipv6IngClassifierAction_p,
                    shadowTreeId))
        {
            retValue = ADK_E_HW_PROGRAMMING_ERROR;
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Failed to add the rule in hardware\n");
            goto release_lock_and_return;
        }
        ruleIndex++;
    }

    /* Add ANY - DENY rule at the end */
    /* Set Rule mask as zero, indicating no mask */
    adk_ipv6_ingress_classifier_rule_t ipv6IngClassifierRule = {0};
    adk_ipv6_ingress_classifier_action_t ipv6IngClassifierAction = {0};
    ipv6IngClassifierRule.ruleMask = 0;
    ipv6IngClassifierAction.action = ADK_IPV6_ING_CLASSIFIER_ACTION_DROP;
    if(ADK_SUCCESS != adk_ipv6_ing_classifier_hw_rule (
                ipv6Handle,
                (ADK_IPV6_CLASSIFIER_RULE_MAX_PRIORITY - ruleIndex),
                1, /* zero id reserved for default rule */
                &ipv6IngClassifierRule,
                &ipv6IngClassifierAction,
                shadowTreeId
                ))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                "Could not add default DENY rule to IP ING CLASSIFIER!\n");
        goto release_lock_and_return;
    }

    /*Update shadow copy*/

    retValue = adk_ipv6_ing_classifer_build(ipv6Handle,
            ingClassifierInfo_p->ipv6IngClassifierCtx[shadowTreeId]);
    if(ADK_SUCCESS !=  retValue)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Error: Could not build IPv6 Ingress CLASSIFEER...\n");
        goto release_lock_and_return;
    }
    rte_acl_dump(ingClassifierInfo_p->ipv6IngClassifierCtx[0]);
    rte_acl_dump(ingClassifierInfo_p->ipv6IngClassifierCtx[1]);
    /* Toggle the index */
    ingClassifierInfo_p->aclTreeIndex = shadowTreeId;
    /* RCU sync */
    adk_rcu_synchronize();
    /* CP ACL List HOS Commit */
    retValue = adk_acl_hos_commit(ingClassifierInfo_p->cpList);
    /* Table full is not an error, CPU rule will handle the packet */
    if(retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Error: Could not commit in HOS...\n");
        retValue = ADK_E_INTERNAL_ERROR;
        goto release_lock_and_return;
    }
release_lock_and_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);
    return retValue;

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);
    ADK_RETURN(retValue);
}

/*!
* =============================================================
* adk_ipv6_ingress_classifier_rule_delete
* @param[in]    ipv6Handle – Ipv6 module handle.
*
* @param[in]    ruleIndex - index at which the rule will be inserted
*
* @retval ADK_SUCCESS Successful completion
*
* @retval ADK_E_INVALID_PARAMETER -  one of the direct arguments
* to the API call is invalid. Possibly rule index specified is either
* out of sequence or there is no filter entry at index 0.
*
* @brief
* This API validates the input parameters and deletes the rule at the given
* index from the ipv6 ingress association ordered tree.
* Note that the rule update does not get reflected in
* data plane until the adk_ipv6_ingress_classifier_build() API is called for this.
*
* =============================================================
*/

adk_status_t adk_ipv6_ingress_classifier_rule_delete(adk_module_handle_t ipv6Handle,
        const uint16_t ruleIndex)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    adk_ipv6_ing_classifer_info_t* ingClassifierInfo_p = NULL;
    uint16_t index=0;
    adk_acl_rule_t cpRule;
    size_t ingAssocRuleId = ADK_RM_RSRC_INVALID;
    adk_ipv6_ingress_classifier_rule_handle_t ipv6IngClassifierFlowHandle=0;


    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    /***TO DO: verify ***/
    ingClassifierInfo_p =  ((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p);

    adk_acl_get_reset(ingClassifierInfo_p->cpList);
    index=0;
    while(ADK_SUCCESS == adk_acl_next_rule_get(
                ingClassifierInfo_p->cpList, &cpRule))
    {
        if(index==ruleIndex)
        {
            ingAssocRuleId = ADK_SYS_FLOWID_GET(cpRule.user_data);
            ipv6IngClassifierFlowHandle = (adk_ipv6_ingress_classifier_rule_handle_t)cpRule.user_data;
            break;
        }
        index++;
    }

    if (ADK_RM_RSRC_INVALID != ingAssocRuleId)
    {
        adk_rm_resource_free(ipv6ModuleInfo_p->ipv6IngClassifierRuleIDHandle, ingAssocRuleId);
    }

    if (ipv6IngClassifierFlowHandle)
        ADK_SYS_FLOW_HANDLE_FREE(ipv6IngClassifierFlowHandle);

    /*** ***/


    retValue = adk_acl_rule_delete(ingClassifierInfo_p->cpList,
            ruleIndex);
    if (retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Failed to delete the rule from CP list\n");
        goto release_lock_rule_delete;
    }
    ipv6ModuleInfo_p->maxIpv6ClassifierRules--;
release_lock_rule_delete:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);

}

/*!
 * =============================================================
 * adk_ipv6_classifier_add_validate_arg
 * validates the input parameter for ingress Classifier rule add
 * =============================================================
 */
adk_status_t adk_ipv6_classifier_add_validate_arg(
        adk_module_handle_t ipv6Handle,
        const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
        const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p)
{
    adk_status_t    retValue;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    uint32_t    policerFlowId = 0;

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if (!ipv6IngClassifierRule_p || !ipv6IngClassifierAction_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid NULL input parameter\n");
        goto return_with_error;
    }

    if (ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_DEST_ADDR_MASK)
    {
        if((ipv6IngClassifierRule_p->destAddr.prefixLength == 0) ||
                (ipv6IngClassifierRule_p->destAddr.prefixLength > ADK_IPV6_PDU_ADDR_LENGTH))
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid Destination IP address prefix Length\n");
            goto return_with_error;
        }
    }

    if ((ipv6IngClassifierRule_p->ruleMask & ADK_IPV6_ING_CLASSIFIER_SRC_ADDR_MASK) &&
            ((ipv6IngClassifierRule_p->srcAddr.prefixLength == 0) ||
             (ipv6IngClassifierRule_p->srcAddr.prefixLength > ADK_IPV6_PDU_ADDR_LENGTH)))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid Source IP address prefix Length: %d\n", ipv6IngClassifierRule_p->srcAddr.prefixLength);
        goto return_with_error;
    }
    if (ipv6IngClassifierAction_p->enablePolicing) {
        retValue = adk_qos_policing_flow_id_get(sys_handle, ipv6IngClassifierAction_p->policingHandle, &policerFlowId);
        if (ADK_SUCCESS != retValue) {
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid policer handle\n");
            return retValue;
        }
        /*The above function just validates the policing Handle and return the policerFlowId
          Since, policerFlowId is not really required in this function, below code just eliminate warning */
        (void)policerFlowId;
    }
    

    switch (ipv6IngClassifierAction_p->action)
    {
    case ADK_IPV6_ING_CLASSIFIER_ACTION_DROP:

    /*! Forward to a program running on host processor */
    case ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_HOST:

    /*! Forward to a EVENT OBJECT running in EVENT MACHINE */
    case ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_EO:
        break;
    default:
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid ACTION input parameter\n");
        goto return_with_error;
    }
    return ADK_SUCCESS;
return_with_error:
    return retValue;
} /* adk_ipv6_classifier_add_validate_arg */

/*!
* =============================================================
* adk_ipv6_ingress_classifier_stats_get
* @param[in]   ipv6Handle  - IPv6 module handle.
* @param[in]   ruleIndex –  Rule index to get stats.
* @param[out]  ipv6Stats_p    - pointer to adk_ipv6_ingress_classifier_stats_t
* data structure.
*
* @retval ADK_SUCCESS Successful completion
* @retval ADK_E_INVALID_PARAMETER  invalid input parameter.
*
* @brief
* This API read the IPv6 classifier specific statistics
* information from the IPv6 DP table memory and copy them to
* ipv6Stats_p data structure.
*
* =============================================================
*/

adk_status_t
adk_ipv6_ingress_classifier_stats_get(
        adk_module_handle_t  ipv6Handle,
        const uint16_t ruleIndex,
        adk_ipv6_ingress_classifier_stats_t  *ipv6Stats_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_ipv6_flow_info_t          *flow_info_p;
    adk_sys_handle_t        sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    int i;
    uint16_t index =0;
    adk_ipv6_ing_classifer_info_t  *ingClassifierInfo_p=NULL;
    adk_acl_rule_t cpRule_p;
    size_t ingAssocRuleId = ADK_RM_RSRC_INVALID;
    adk_ipv6_ingress_classifier_rule_handle_t ipv6IngClassifierRule_p = NULL;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        return (retValue);
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    ingClassifierInfo_p = ((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p);

    adk_acl_get_reset(ingClassifierInfo_p->cpList);
    while(ADK_SUCCESS == adk_acl_next_rule_get(ingClassifierInfo_p->cpList, &cpRule_p))
    {
        if(index == ruleIndex)
        {
            ingAssocRuleId = ADK_SYS_FLOWID_GET(cpRule_p.user_data);
            ipv6IngClassifierRule_p =(adk_ipv6_ingress_classifier_rule_handle_t )cpRule_p.user_data;
            break;
        }
        index++;
    }

    if (!ADK_SYS_VALID_FLOW_HANDLE(ipv6IngClassifierRule_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid Index %d\n", ruleIndex);
        goto _adk_ipv6_ingress_classifier_stats_get_exit;
    }

    flow_info_p = (adk_ipv6_flow_info_t *)
        ADK_SYS_FLOW_ATTR_PTR_GET(ipv6IngClassifierRule_p);

    if ((flow_info_p->cookie != ADK_IPV6_FLOW_COOKIE) ||
            (flow_info_p->type != ADK_IPV6_FLOW_TYPE_INGRESS_CLASSIFIER_FLOW))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid ruleIndex %d\n", ruleIndex);
        goto _adk_ipv6_ingress_classifier_stats_get_exit;
    }

    ipv6Stats_p->rxPkts = 0;
    ipv6Stats_p->txPkts = 0;
    ipv6Stats_p->ipv6PolicerDroppedPkts  = 0;
    ipv6Stats_p->ipv6ClassifierDroppedPkts = 0;
    for(i=0; i<ADK_MAX_NUM_CORES; ++i)
    {
        ipv6Stats_p->rxPkts +=
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats[
            i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ADK_SYS_FLOWID_GET(ipv6IngClassifierRule_p)].rxPkts;
        ipv6Stats_p->txPkts +=
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats[
            i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ADK_SYS_FLOWID_GET(ipv6IngClassifierRule_p)].txPkts;
        ipv6Stats_p->ipv6PolicerDroppedPkts +=
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats[
            i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ADK_SYS_FLOWID_GET(ipv6IngClassifierRule_p)].policerDropPkts;
        ipv6Stats_p->ipv6ClassifierDroppedPkts +=
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6IngFlowStats[
            i*ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS+ADK_SYS_FLOWID_GET(ipv6IngClassifierRule_p)].ipv6ClassifierDroppedPkts;
    }


_adk_ipv6_ingress_classifier_stats_get_exit:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);
}

/*!
* ==========================================================================
*  adk_ipv6_get_ingress_classifier_all
*
* @param[in]    ipv6Handle - IPV6 module handle.
*
* @param[out]   *numClassifiers - Number of ingress classifier rules
*
* @param[out]   **ingressClassifier_p - List of ingress classifier rules
*
* @retval ADK_SUCCESS Successful completion
*
* @retval ADK_E_INVALID_PARAMETER  Invalid input parameters
*
* @retval ADK_E_INVALID_HANDLE   Ipv6 Module is invalid.
*
* @brief
*
* This API gets the ingress classifier
*
* @details
*
* This API gets all the ingress classifier rules along with the count of them.
* The array index of a rule represents its index in the tree.
* Application is expected to free ingressClassifier_p pointer once it is used.
* ===========================================================================
*/
adk_status_t
adk_ipv6_get_ingress_classifier_all(
        adk_module_handle_t           ipv6Handle,
        uint32_t                      *numClassifiers,
        adk_ipv6_ingress_classifier_info_t **ingressClassifier_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    adk_acl_rule_t cpRule;
    adk_ipv6_flow_info_t *classifierInfo = NULL;
    adk_ipv6_ingress_classifier_info_t *ingressClassifier = NULL;
    uint32_t num_nodes;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);
    if ((numClassifiers == NULL) || (ingressClassifier_p == NULL))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue, "Invalid input parameter %"ADKPRIh" %"ADKPRIh"\n",
                            numClassifiers, ingressClassifier_p);
        goto adk_ipv6_get_ingress_classifier_return;
    }

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    adk_acl_get_num_nodes(((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p)->cpList,
                         &num_nodes);

    ingressClassifier = (adk_ipv6_ingress_classifier_info_t *)
        ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_ING_CLASSIFIER_INFO_OBJ, num_nodes, sizeof(adk_ipv6_ingress_classifier_info_t));

    adk_acl_get_reset(((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p)->cpList);
    for (uint32_t i = 0; i < num_nodes; i++)
    {
        retValue = adk_acl_next_rule_get(((adk_ipv6_ing_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->ingClassifierInfo_p)->cpList,
                                 &cpRule);
       if (retValue != ADK_SUCCESS)
       {
           retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_LIST_NOT_FOUND);
           ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                               "Ingress classifier rule is not found in CP ACL list\n");
           goto adk_ipv6_get_ingress_classifier_return;
       }
       classifierInfo = (adk_ipv6_flow_info_t *)
            ADK_SYS_FLOW_ATTR_PTR_GET(cpRule.user_data);

       if ((classifierInfo != NULL) && (ingressClassifier != NULL))
       {
           ADK_MEMCPY(&((ingressClassifier+i)->ipv6Rules), &classifierInfo->ingClassifierFlow.rule, sizeof(adk_ipv6_ingress_classifier_rule_t));
           ADK_MEMCPY(&((ingressClassifier+i)->ipv6Attr), &classifierInfo->ingClassifierFlow.action, sizeof(adk_ipv6_ingress_classifier_action_t));
       }
    }
    *ingressClassifier_p = ingressClassifier;
    *numClassifiers = num_nodes;

adk_ipv6_get_ingress_classifier_return:
ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

    ADK_RETURN(retValue);
}

