/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @file  adk_ipv6_internal.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6Functions  IPV6 Module
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the internal functions provided by the IPV6
 * module. None of the functions from this file should be called by
 * other modules.
 *
 ***********************************************************************/
/* Infrastructure header files */
#include "adk_include.h"
#include "adk_ph_dev.h"

/* IPV6 external header files */
#include "adk_ipv6.h"
#include "adk_ipv6_const.h"

/* IPV6 internal header files */
#include "adk_ipv6_internal.h"

/*
 * Internal helper function to validate the interface handle 
 */
bool 
adk_ipv6_interface_valid_handle(adk_interface_handle_t ipv6IfHandle)
{
    ipv6_interface_info_t *if_pvt_info;

    if (!ADK_SYS_VALID_INTERFACE_HANDLE(ipv6IfHandle))
        return FALSE;

    if_pvt_info = (ipv6_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(ipv6IfHandle);

    if (if_pvt_info == NULL)
        return FALSE;

    if (if_pvt_info->cookie != ADK_IPV6_INTF_COOKIE)
        return FALSE;

    return TRUE;
}

/*
 * Internal helper function to validate the packet capture handle
 */
bool
adk_ipv6_if_pktcap_valid_handle(adk_flow_handle_t ipv6IfPktCapHandle)
{
    adk_ipv6_pktcap_internal_info_t *if_pvt_info;

    if (!ADK_SYS_VALID_FLOW_HANDLE(ipv6IfPktCapHandle))
        return FALSE;

    if_pvt_info = (adk_ipv6_pktcap_internal_info_t *)
        ADK_SYS_FLOW_ATTR_PTR_GET(ipv6IfPktCapHandle);

    if (if_pvt_info == NULL)
        return FALSE;

    if (if_pvt_info->cookie != ADK_IPV6_PACKET_CAPTURE_COOKIE)
        return FALSE;
    return TRUE;
}


/*
 * Internal helper function to validate the dscp group handle 
 */
bool 
adk_ipv6_dscp_group_valid_handle(adk_dscp_group_handle_t dscpGrpHandle)
{
    ipv6_if_dscp_grp_info_t *dscpgroup_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(dscpGrpHandle))
        return FALSE;

    dscpgroup_p = (ipv6_if_dscp_grp_info_t *)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (dscpGrpHandle);

    if (dscpgroup_p == NULL)
        return FALSE;

    if (dscpgroup_p->cookie != ADK_IPV6_DSCP_GROUP_COOKIE)
        return FALSE;

    return TRUE;
}

/*
 * Internal helper function to validate the pbr handle 
 */
bool 
adk_ipv6_pbr_valid_handle( adk_flow_handle_t  ruleHandle)
{
    adk_ipv6_pbr_Route_attr_t *pbrInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(ruleHandle))
        return FALSE;

    pbrInfo_p = ( adk_ipv6_pbr_Route_attr_t*)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (ruleHandle);

    if (pbrInfo_p == NULL)
        return FALSE;

    if (pbrInfo_p->cookie != ADK_IPV6_PBR_COOKIE)
        return FALSE;

    return TRUE;
}

/*
 * Internal helper function to validate the route handle 
 */
bool 
adk_ipv6_route_valid_handle( adk_flow_handle_t  routeHandle)
{
    adk_ipv6_multipathRoute_attr_t *routeInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(routeHandle))
        return FALSE;

    routeInfo_p = ( adk_ipv6_multipathRoute_attr_t*)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (routeHandle);

    if (routeInfo_p == NULL)
        return FALSE;

    if (routeInfo_p->cookie != ADK_IPV6_ROUTE_COOKIE)
        return FALSE;

    return TRUE;
}
/*
 * Internal helper function to validate the next hop handle 
 */
bool 
adk_ipv6_nexthop_valid_handle( adk_flow_handle_t  nexthopHandle)
{
    adk_ipv6_flow_info_t *ipv6FlowInfo_p = NULL;
    adk_ipv6_next_hop_info_t *nexthopInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(nexthopHandle))
        return FALSE;

    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (nexthopHandle);
    nexthopInfo_p = &ipv6FlowInfo_p->next_hop_info;

    if (nexthopInfo_p == NULL)
        return FALSE;

    if (nexthopInfo_p->cookie != ADK_IPV6_NEXTHOP_COOKIE)
        return FALSE;

    return TRUE;
}
/*
 * Internal helper function to validate the multicast group handle 
 */
bool 
adk_ipv6_multicast_group_valid_handle( adk_flow_handle_t  multicast_groupHandle)
{
    adk_ipv6_multicast_group_info_t *multicast_groupInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(multicast_groupHandle))
        return FALSE;

    multicast_groupInfo_p = ( adk_ipv6_multicast_group_info_t*)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (multicast_groupHandle);

    if (multicast_groupInfo_p == NULL)
        return FALSE;

    if (multicast_groupInfo_p->cookie != ADK_IPV6_MULTICAST_GROUP_COOKIE)
        return FALSE;

    return TRUE;
}
/*
 * Internal helper function to validate the multicast flow handle 
 */
bool 
adk_ipv6_multicast_flow_valid_handle( adk_flow_handle_t  multicast_flowHandle)
{
    adk_ipv6_multicast_flow_info_t *multicast_flowInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(multicast_flowHandle))
        return FALSE;

    multicast_flowInfo_p = ( adk_ipv6_multicast_flow_info_t*)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (multicast_flowHandle);

    if (multicast_flowInfo_p == NULL)
        return FALSE;

    if (multicast_flowInfo_p->cookie != ADK_IPV6_MULTICAST_FLOW_COOKIE)
        return FALSE;

    return TRUE;
}
/*
 * Internal helper function to validate the onlink handle 
 */
bool 
adk_ipv6_onlink_valid_handle( adk_flow_handle_t  onlinkHandle)
{
    adk_ipv6_flow_info_t    *ipv6FlowInfo_p;
    adk_ipv6_onlink_destination_internal_t *onlinkInfo_p;

    if (!ADK_SYS_VALID_FLOW_HANDLE(onlinkHandle))
        return FALSE;

    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
	    		ADK_SYS_FLOW_ATTR_PTR_GET (onlinkHandle);

    if (ipv6FlowInfo_p == NULL)
        return FALSE;

    onlinkInfo_p = &ipv6FlowInfo_p->onlink_info;

    if (onlinkInfo_p->cookie != ADK_IPV6_ONLINK_COOKIE)
        return FALSE;

    return TRUE;
}
/*
 * Internal helper function to validate ipv6 module handle
 */
bool 
adk_ipv6_valid_module_handle(adk_module_handle_t ipv6Handle)
{
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    
    ipv6ModuleInfo_p = (adk_ipv6_info_t *)ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    
    if(ipv6ModuleInfo_p == NULL)
       return FALSE;
  
    if(ipv6ModuleInfo_p->cookie != ADK_IPV6_MODULE_COOKIE)
       return FALSE;

    return TRUE;
}

bool adk_ipv6_valid_handle(adk_module_handle_t ipv6Handle)
{   
	adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;

	if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
		return FALSE;

	ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

	if (ipv6ModuleInfo_p == NULL)
		return FALSE;

	if (ipv6ModuleInfo_p->cookie != ADK_IPV6_MODULE_COOKIE)
		return FALSE;

	return TRUE;
}   

	void
adk_ipv6_mod_id_str_print(uint32_t dest_mod_id)
{
    switch(dest_mod_id)
    {
        case ADK_SIF_CFG_PIPELINE_MODULE_ETHERNET_ULI:
            ADK_TABLE_ENTRY_WRITE("|%s", "ADK_SIF_CFG_PIPELINE_MODULE_ETHERNET_ULI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_GTPU_LLI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_GTPU_LLI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_GTPU_ULI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_GTPU_ULI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_BFD_LLI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_BFD_LLI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_LLI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_LLI");
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_ULI:
            ADK_TABLE_ENTRY_WRITE("|%s","ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_ULI");
            break;
        default:
            ADK_TABLE_ENTRY_WRITE("|%02d",dest_mod_id)
                break;
    }
    return;
}


adk_status_t
adk_ipv6_if_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    int i;
    adk_ipv6_dp_interface_entry_t *if_entry = ipv6_tables->ipv6_dp_if_table.tbl;
    char if_type[3][100] = {{"IPV6_INTERFACE_TYPE_ETH"},
        {"IPV6_INTERFACE_TYPE_TUNNEL"},
        {"IPV6_INTERFACE_TYPE_VIRTUAL"}};

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Interface Table \n");

    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|enable_policing");
    ADK_TABLE_ENTRY_WRITE("|enable_pbr");
    ADK_TABLE_ENTRY_WRITE("|enable_ingress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_egress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_remark");
    ADK_TABLE_ENTRY_WRITE("|enable_qos");
    ADK_TABLE_ENTRY_WRITE("|enable_reassembly");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|interface_id"); 
    ADK_TABLE_ENTRY_WRITE("|outer_vrd_id");
    ADK_TABLE_ENTRY_WRITE("|nexthop_id");
    ADK_TABLE_ENTRY_WRITE("|dscp_profile_id");
    ADK_TABLE_ENTRY_WRITE("|police_flow_id");
    ADK_TABLE_ENTRY_WRITE("|if_type");
    ADK_TABLE_ENTRY_WRITE("|ingress_stats_id");
    ADK_TABLE_ENTRY_WRITE("|egress_stats_id");
    ADK_TABLE_ENTRY_WRITE("|ingress_assoc_mask");
    ADK_TABLE_ENTRY_WRITE("|mtu");
    ADK_TABLE_ENTRY_WRITE("|local_fdbk");
    ADK_TABLE_ENTRY_WRITE("|pmtu");
    ADK_TABLE_ENTRY_WRITE("|la_flag");
    ADK_TABLE_ENTRY_WRITE("|la_flag_ver");
    ADK_TABLE_ENTRY_WRITE("|ingress_ruleset");
    ADK_TABLE_ENTRY_WRITE("|egress_ruleset");
    ADK_TABLE_ENTRY_WRITE("|ipv6_addr");
    ADK_TABLE_ENTRY_WRITE("|ipv6_addr_prefix");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap flag");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap state");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap context");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap devId");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap QId");
    ADK_TABLE_ENTRY_WRITE("|ing pkt_cap cbId");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap flag");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap state");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap context");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap devId");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap QId");
    ADK_TABLE_ENTRY_WRITE("|eg pkt_cap cbId");
    ADK_TABLE_ENTRY_WRITE(" \n ");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<(ADK_IPV6_NO_OF_IPV6_INTERFACES + ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES); i++)
    {
        if( !if_entry[i].is_disabled ) 
        {
            ADK_TABLE_ENTRY_WRITE("%04d#", i);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_policing);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_pbr);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_ingress_acl);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_egress_acl);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_remark);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_qos);
            ADK_TABLE_ENTRY_WRITE("|%01x", if_entry[i].enable_reassembly);
            ADK_TABLE_ENTRY_WRITE("|%03d", if_entry[i].vrd_id);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].interface_id); 
            ADK_TABLE_ENTRY_WRITE("|%03d", if_entry[i].outer_vrd_id);
            ADK_TABLE_ENTRY_WRITE("|%03d", if_entry[i].nexthop_id);
            ADK_TABLE_ENTRY_WRITE("|%03d", if_entry[i].dscp_profile_id);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].police_flow_id);
            ADK_TABLE_ENTRY_WRITE("|%s", if_type[if_entry[i].if_type]);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].ingress_stats_id);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].egress_stats_id);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].ingress_assoc_mask);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].mtu);
            if(if_entry[i].pmtu&0x8000)
            {
                ADK_TABLE_ENTRY_WRITE("|TRUE");
            }
            else
            {
                ADK_TABLE_ENTRY_WRITE("|FALSE");
            }
            ADK_TABLE_ENTRY_WRITE("|%05d", (if_entry[i].pmtu&ADK_IPV6_INTERFACE_MTU_MASK));
            ADK_TABLE_ENTRY_WRITE("|%s",(if_entry[i].la_flag)?"TRUE":"FALSE");
            ADK_TABLE_ENTRY_WRITE("|%02d", if_entry[i].la_flag_version);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].ingress_ruleset);
            ADK_TABLE_ENTRY_WRITE("|%05d", if_entry[i].egress_ruleset);
            ADK_TABLE_ENTRY_WRITE("|%08x %08x %08x %08x", 
                    if_entry[i].ipv6_addr_prefix.Ipv6Addr.addr[0], 
                    if_entry[i].ipv6_addr_prefix.Ipv6Addr.addr[1], 
                    if_entry[i].ipv6_addr_prefix.Ipv6Addr.addr[2], 
                    if_entry[i].ipv6_addr_prefix.Ipv6Addr.addr[3]);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ipv6_addr_prefix.prefixLength);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.is_enabled);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.enqueue_status);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.context);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.devId);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.queueId);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].ing_pkt_capture_info.cbFnId);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.is_enabled);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.enqueue_status);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.context);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.devId);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.queueId);
            ADK_TABLE_ENTRY_WRITE("|%02d",
                    if_entry[i].eg_pkt_capture_info.cbFnId);
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}


void adk_ipv6_route_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value)
{
    adk_ipv6_dp_route_entry_t *route_entry = (adk_ipv6_dp_route_entry_t *)(void *)value;
    int i;
    char route_action[20][100] = {{"ADK_IPV6_ROUTE_LOOKUP_ROUTE_FOUND"},    
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD"},  
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_ONLINK"},  
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_MUTLIHOP"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE"},  
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_DROP"},  
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_DROP_WITH_COUNT"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_STH_VIA_PH"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_STH_VIA_NCA"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_FALLBACK"},  
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_NULL_ROUTE"}, 
                                  {"ADK_IPV6_ROUTE_LOOKUP_ACTION_LOC"}};

    ADK_TABLE_ENTRY_WRITE("|%02d|", key[0]);
    ADK_TABLE_ENTRY_WRITE("|");
    for(i=1; i<=16; i++)
    ADK_TABLE_ENTRY_WRITE("%02x",key[i]);
    ADK_TABLE_ENTRY_WRITE("|%02d",depth);
    ADK_TABLE_ENTRY_WRITE("|%01d",route_entry->enable_ingress_assoc);
    ADK_TABLE_ENTRY_WRITE("|%01d",route_entry->on_link);
    ADK_TABLE_ENTRY_WRITE("|%01d",route_entry->null_route);
    ADK_TABLE_ENTRY_WRITE("|%01d",route_entry->reassembly_flag);
    ADK_TABLE_ENTRY_WRITE("|%s",route_action[route_entry->action]);
    ADK_TABLE_ENTRY_WRITE("|%02x",route_entry->ingress_assoc_rulemask);
    ADK_TABLE_ENTRY_WRITE("|%05d",route_entry->nexthop_id);
    ADK_TABLE_ENTRY_WRITE("|%03d",route_entry->callback_fun_id);
    ADK_TABLE_ENTRY_WRITE("|%03d",route_entry->virtual_if_stats_id);
    ADK_TABLE_ENTRY_WRITE("|%"ADKPRIh"", route_entry->ph_bind_handle);
    ADK_TABLE_ENTRY_WRITE("\n");

    return;
}


adk_status_t
adk_ipv6_route_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Route Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|VRDID");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|MASK");
    ADK_TABLE_ENTRY_WRITE("|enable_ingress_assoc");
    ADK_TABLE_ENTRY_WRITE("|on_link");
    ADK_TABLE_ENTRY_WRITE("|null_route");
    ADK_TABLE_ENTRY_WRITE("|reassembly_flag");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|ingress_assoc_rulemask");
    ADK_TABLE_ENTRY_WRITE("|nexthop/interface id");
    ADK_TABLE_ENTRY_WRITE("|callback_fun_id");
    ADK_TABLE_ENTRY_WRITE("|virtual if id");
    ADK_TABLE_ENTRY_WRITE("|ph_bind_handle \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    adk_lpm_dump(ipv6_tables->ipv6_dp_route_table.lpm_tbl.lpm,
                    adk_ipv6_route_dump_cb_fn, 
                    (void *) ipv6_tables);
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_pbr_tbl_dump(adk_module_handle_t ipv6Handle)
{
    uint32_t Ruleset = 0, i = 0, idx = 0, numrules = 0;
    adk_ipv6_pbr_policy_set_handle_t ruleSetHandles = NULL;
    adk_ipv6_pbr_info_t *pbrEntry = NULL;
    adk_ipv6_info_t *ipv6ModuleInfo_p;

    ipv6ModuleInfo_p = (adk_ipv6_info_t*)ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 PBR Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|Index");
    ADK_TABLE_ENTRY_WRITE("|ruleMask");
    ADK_TABLE_ENTRY_WRITE("|Pcp");
    ADK_TABLE_ENTRY_WRITE("|Protocol");
    ADK_TABLE_ENTRY_WRITE("|Dscp");
    ADK_TABLE_ENTRY_WRITE("|SIP");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|SrcPort");
    ADK_TABLE_ENTRY_WRITE("|DstPort");
    ADK_TABLE_ENTRY_WRITE("|Frag Type");
    ADK_TABLE_ENTRY_WRITE("|ingAssoc Rulemask");
    ADK_TABLE_ENTRY_WRITE("|isReassFlag");
    ADK_TABLE_ENTRY_WRITE("|Action");

    ADK_TABLE_ENTRY_WRITE("\n====================================================== \n");
    for ( Ruleset = 0; Ruleset < ADK_IPV6_PBR_RULE_SETS_MAX; Ruleset++)
    {
        ruleSetHandles = ipv6ModuleInfo_p->policySetHandles[Ruleset];
        if ( ruleSetHandles != NULL)
        {
           if ( ADK_SUCCESS != adk_ipv6_get_num_pbr_rules(ipv6Handle, ruleSetHandles, &numrules))
           {
               continue;
           }
           if ( numrules != 0)
           {
               pbrEntry =  (adk_ipv6_pbr_info_t *) ADK_NVM_CALLOC (ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_TBL_DUMP_OBJ, numrules, sizeof(adk_ipv6_pbr_info_t));
               if ( pbrEntry == NULL)
               {
                   continue;
               }
               if ( ADK_SUCCESS != adk_ipv6_get_pbr_rules(ipv6Handle, ruleSetHandles, numrules, pbrEntry))
               {
                   ADK_NVM_FREE(pbrEntry);
                   continue;
               }
               ADK_TABLE_ENTRY_WRITE("Number of Rules - %05d\n", numrules);
               for ( idx = 0; idx < numrules; idx++)
               {
                   ADK_TABLE_ENTRY_WRITE("|%05d",idx);
                   ADK_TABLE_ENTRY_WRITE("|%04x",pbrEntry[idx].pbrRule.PBR_RuleMask);
                  if(pbrEntry[idx].pbrRule.PBR_RuleMask & ADK_IPV6_PBR_SRC_ADDR_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|");
                       for(i=0; i<4; i++)
                       {
                           ADK_TABLE_ENTRY_WRITE("%08x", pbrEntry[idx].pbrRule.srcAddrInfo.Ipv6Addr.addr[i]);
                       }
                       ADK_TABLE_ENTRY_WRITE("/%03d", pbrEntry[idx].pbrRule.srcAddrInfo.prefixLength);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }
               }
               ADK_NVM_FREE(pbrEntry);
           }
        }
    }
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_reass_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    adk_ipv6_dp_reassembly_entry_t *reass_entry = NULL;
    uint32_t i, k/*, idx*/;

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 REASSEMBLY TABLE \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|nb_frags");
    ADK_TABLE_ENTRY_WRITE("|frag_size");
    ADK_TABLE_ENTRY_WRITE("|total_size");
    ADK_TABLE_ENTRY_WRITE("|if_id");
    ADK_TABLE_ENTRY_WRITE("|err_detected");
    ADK_TABLE_ENTRY_WRITE("|frag_idx");
    ADK_TABLE_ENTRY_WRITE("|collosion_idx");
    ADK_TABLE_ENTRY_WRITE("|is_reassembly_success");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|hash_idx");
    ADK_TABLE_ENTRY_WRITE("|src addr");
    ADK_TABLE_ENTRY_WRITE("|dst addr");
    ADK_TABLE_ENTRY_WRITE("|frag id");
    ADK_TABLE_ENTRY_WRITE("|frags\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<ADK_IPV6_REASSEMBLY_HASH_ENTRIES*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES; i++)
    {
        if((ipv6_tables->ipv6_dp_reassembly_table.hash_tbl[i].is_enabled) &&
            (i&(ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES-1)))
            {
                /* idx = ipv6_tables->ipv6_dp_reassembly_table.hash_tbl[i][j].index; */
                reass_entry = ipv6_tables->ipv6_dp_reassembly_table.hash_tbl[i].reass_entry;

                ADK_TABLE_ENTRY_WRITE("|%03d|", reass_entry->nb_frags);       
                ADK_TABLE_ENTRY_WRITE("|%05d|", reass_entry->frag_size);      
                ADK_TABLE_ENTRY_WRITE("|%05d|", reass_entry->total_size);     
                ADK_TABLE_ENTRY_WRITE("|%03d|", reass_entry->if_id);          
                ADK_TABLE_ENTRY_WRITE("|%01d|", reass_entry->err_detected);   
                ADK_TABLE_ENTRY_WRITE("|%03d|", reass_entry->frag_idx);       
                ADK_TABLE_ENTRY_WRITE("|%03d|", reass_entry->collosion_idx);  
                ADK_TABLE_ENTRY_WRITE("|%01d|", reass_entry->is_reassembly_success);
                ADK_TABLE_ENTRY_WRITE("|%03d|", reass_entry->vrd_id);         
                ADK_TABLE_ENTRY_WRITE("|%04d|", reass_entry->hash_idx);      
                for(k=0; k<ADK_IPV6_PDU_ADDR_SIZE_BYTES; k++)
                    ADK_TABLE_ENTRY_WRITE("%02x", 
                            ((uint8_t *) (&reass_entry->reass_tuple.src_addr))[k]);
                ADK_TABLE_ENTRY_WRITE("|");
                for(k=0; k<ADK_IPV6_PDU_ADDR_SIZE_BYTES; k++)
                    ADK_TABLE_ENTRY_WRITE("%02x", 
                            ((uint8_t *) (&reass_entry->reass_tuple.dst_addr))[k])
                ADK_TABLE_ENTRY_WRITE("|");
            ADK_TABLE_ENTRY_WRITE("|%010u|", reass_entry->reass_tuple.frag_id); 
                for(k=0; k<reass_entry->nb_frags; k++)
                ADK_TABLE_ENTRY_WRITE("|%p", reass_entry->fg[k].task);   
                ADK_TABLE_ENTRY_WRITE("\n");
            }
        }
    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ipv6_onlink_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    adk_ipv6_dp_onlink_dest_entry_t *onlink_entry = NULL;
    adk_ipv6_dp_overflow_node_t *node;
    uint32_t i, k, idx;

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Onlink Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|VRDID");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|Hash bucket index");
    ADK_TABLE_ENTRY_WRITE("|collision index");
    ADK_TABLE_ENTRY_WRITE("|hash value");
    ADK_TABLE_ENTRY_WRITE("|table index");
    ADK_TABLE_ENTRY_WRITE("|is valid entry");
    ADK_TABLE_ENTRY_WRITE("|interface id \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<ADK_IPV6_NO_OF_ONLINK_ENTRIES*ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES; i++)
    {
        if(ipv6_tables->ipv6_dp_onlink_dest_table.hash_tbl[i].is_enabled)
        {
            idx = ipv6_tables->ipv6_dp_onlink_dest_table.hash_tbl[i].index;
            onlink_entry = &ipv6_tables->ipv6_dp_onlink_dest_table.tbl[idx];
            ADK_TABLE_ENTRY_WRITE("|%03d|", onlink_entry->vrd_id);
            for(k=0; k<ADK_IPV6_PDU_ADDR_SIZE_WORDS; k++)
                ADK_TABLE_ENTRY_WRITE("%08x", ntohl(onlink_entry->ipv6_addr.addr[k]));
            ADK_TABLE_ENTRY_WRITE("|%05d", i%ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES);
            ADK_TABLE_ENTRY_WRITE("|%01d", i/ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES);
            ADK_TABLE_ENTRY_WRITE("|%08x", ipv6_tables->ipv6_dp_onlink_dest_table.hash_tbl[i].hash_value);
            ADK_TABLE_ENTRY_WRITE("|%05d", idx);
            ADK_TABLE_ENTRY_WRITE("|%01d", onlink_entry->is_enabled);
            ADK_TABLE_ENTRY_WRITE("|%05d", onlink_entry->interface_id);
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    node = ipv6_tables->ipv6_dp_onlink_dest_table.overflow_list.head;

    while(node)
    {
        onlink_entry = &ipv6_tables->ipv6_dp_onlink_dest_table.tbl[node->idx];
        ADK_TABLE_ENTRY_WRITE("|%03d|", onlink_entry->vrd_id);
        for(k=0; k<ADK_IPV6_PDU_ADDR_SIZE_WORDS; k++)
            ADK_TABLE_ENTRY_WRITE("%08x", ntohl(onlink_entry->ipv6_addr.addr[k]));
        ADK_TABLE_ENTRY_WRITE("|%05d", 0);
        ADK_TABLE_ENTRY_WRITE("|%01d", 0);
        ADK_TABLE_ENTRY_WRITE("|%08x", 0);
        ADK_TABLE_ENTRY_WRITE("|%05d", node->idx);
        ADK_TABLE_ENTRY_WRITE("|%01d", onlink_entry->is_enabled);
        ADK_TABLE_ENTRY_WRITE("|%05d", onlink_entry->interface_id);
        ADK_TABLE_ENTRY_WRITE("\n");
        node = node->next;
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_dest_mtu_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_entry = NULL;
    adk_ipv6_dp_dest_mtu_table_t *dst_mtu_tbl;
    uint32_t i, k, idx;

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Destination MTU Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|VRDID");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|MTU \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    dst_mtu_tbl = &ipv6_tables->ipv6_dp_dest_mtu_tbl_list.tbl_head;
    while( dst_mtu_tbl!= NULL)
    {
        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_ENTRIES*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++)
        {
            if(dst_mtu_tbl->hash_tbl[i].is_enabled)
            {
                idx = dst_mtu_tbl->hash_tbl[i].index;
                dest_mtu_entry = &(dst_mtu_tbl->tbl[idx]);
                ADK_TABLE_ENTRY_WRITE("|%03d|", dest_mtu_entry->vrd_id);
                for(k=0; k<ADK_IPV6_PDU_ADDR_SIZE_WORDS; k++)
                    ADK_TABLE_ENTRY_WRITE("%08x", ntohl(dest_mtu_entry->ipv6_addr.addr[k]));
                ADK_TABLE_ENTRY_WRITE("|%05d", dest_mtu_entry->mtu);
                ADK_TABLE_ENTRY_WRITE("\n");
            }
        }
        dst_mtu_tbl = dst_mtu_tbl->next;
    }
    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}




void adk_ipv6_ingress_assoc_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value)
{
    adk_ipv6_dp_tables_t  *ipv6_tables = (adk_ipv6_dp_tables_t *)rock;
    adk_ipv6_dp_ingress_assoc_entry_t *ingress_assoc = ipv6_tables->ipv6_dp_ingress_assoc_table.tbl;
    uint8_t ruleMask = key[0], index = 0, i, nh_header = 0, tmp_idx;
    intptr_t idx = (intptr_t)(void *)value;

    ADK_TABLE_ENTRY_WRITE("|%05d", depth);
    ADK_TABLE_ENTRY_WRITE("|%02x", key[index]);
    index++;

    if(ruleMask & IPV6_INGRESS_ASSOC_BIT_VRD)
    {
        ADK_TABLE_ENTRY_WRITE("|%02x", key[index]);
        index++;
    }
    else
    {
        ADK_TABLE_ENTRY_WRITE("|*");
    }

    if(ruleMask & IPV6_INGRESS_ASSOC_BIT_IF)
    {
        ADK_TABLE_ENTRY_WRITE("|%05d", *((uint16_t *)&key[index]));
        index += 2;
    }
    else
    {
        ADK_TABLE_ENTRY_WRITE("|*");
    }

    if(ruleMask & IPV6_INGRESS_ASSOC_BIT_FLOW_LABEL)
    {
        ADK_TABLE_ENTRY_WRITE("|%02x%02x%02x", key[index],key[index+1],key[index+2]);
        index += 3;
    }
    else
    {
        ADK_TABLE_ENTRY_WRITE("|*");
    }
    
    if(ruleMask & IPV6_INGRESS_ASSOC_BIT_PROTO)
    {
        ADK_TABLE_ENTRY_WRITE("|%02x", key[index]);
        nh_header = key[index];
        index += 1;
    }
    else
    {
        ADK_TABLE_ENTRY_WRITE("|*");
    }

    if(nh_header == ADK_IPV6_PROTOCOL_ICMPV6)
    {
        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
        {
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO1)
                tmp_idx += ADK_IPV6_ICMPV6_CODE_LEN;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                tmp_idx += ADK_IPV6_ICMPV6_TYPE_LEN;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

            ADK_TABLE_ENTRY_WRITE("|");
            for(i=0; i<16; i++)
            {
                ADK_TABLE_ENTRY_WRITE("%02x", key[index+i+tmp_idx]);
            }
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
        {
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                tmp_idx += ADK_IPV6_ICMPV6_TYPE_LEN;    

            ADK_TABLE_ENTRY_WRITE("|");
            for(i=0; i<16; i++)
            {
                ADK_TABLE_ENTRY_WRITE("%02x", key[index+i+tmp_idx]);
            }
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO1)
        {
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                tmp_idx += ADK_IPV6_ICMPV6_TYPE_LEN;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

            ADK_TABLE_ENTRY_WRITE("|%02x", key[index+tmp_idx]);
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
        {
            ADK_TABLE_ENTRY_WRITE("|%02x", key[index]);
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

    }
    else
    {
        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
        {
            ADK_TABLE_ENTRY_WRITE("|");
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                tmp_idx += ADK_IPV6_PDU_L4_PORT_SIZE_BYTES;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

            for(i=0; i<16; i++)
            {
                ADK_TABLE_ENTRY_WRITE("%02x", key[index+i+tmp_idx]);
            }
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
        {
            ADK_TABLE_ENTRY_WRITE("|");
            for(i=0; i<16; i++)
            {
                ADK_TABLE_ENTRY_WRITE("%02x", key[index+i]);
            }
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO1)
        {
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                tmp_idx += ADK_IPV6_PDU_L4_PORT_SIZE_BYTES;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

            ADK_TABLE_ENTRY_WRITE("|%04x", ntohs(*(uint16_t *)&key[index+tmp_idx]));
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }

        if(ruleMask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
        {
            tmp_idx = 0;
            if(ruleMask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
                tmp_idx += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

            ADK_TABLE_ENTRY_WRITE("|%04x", ntohs(*(uint16_t *)&key[index+tmp_idx]));
        }
        else
        {
            ADK_TABLE_ENTRY_WRITE("|*");
        }
    }

    ADK_TABLE_ENTRY_WRITE("|");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_SEND_USING_PH)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_SEND_USING_PH");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_TRANSLATE_TO_IPV4)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_TRANSLATE_TO_IPV4");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_FORWARD_PDU)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_FORWARD_PDU");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_ISATAP_TUNNELING)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_ISATAP_TUNNELING");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_6TO4_TUNNELING)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_6TO4_TUNNELING");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_SEND_USING_NCA_QID)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_SEND_USING_NCA_QID");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_ENCAP)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_ENCAP");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_DECAP)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_TEREDO_TUNNELING_DECAP");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_IGNORE_CHECKSUM)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_IGNORE_CHECKSUM");
    if(ingress_assoc[idx].action & ADK_IPV6_INGRESS_ATTR_INVOKE_CB)
        ADK_TABLE_ENTRY_WRITE("ADK_IPV6_INGRESS_ATTR_INVOKE_CB");
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_stats_id);
    ADK_TABLE_ENTRY_WRITE("|%03d",ingress_assoc[idx].callback_fun_id);
    ADK_TABLE_ENTRY_WRITE("|%"ADKPRIh"", ingress_assoc[idx].ph_bind_handle);
    ADK_TABLE_ENTRY_WRITE("|%p",ingress_assoc[idx].invoke_cb_attr.recvCb);
    ADK_TABLE_ENTRY_WRITE("|%02d",ingress_assoc[idx].invoke_cb_attr.id);
    ADK_TABLE_ENTRY_WRITE("|%s",
            ((ingress_assoc[idx].invoke_cb_attr.if_filter_info.is_filter_enabled)? 
             "TRUE":"FALSE"));
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].invoke_cb_attr.if_filter_info.interface_id);
    adk_ipv6_mod_id_str_print(ingress_assoc[idx].flow_info.flow[0].destModId);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_info.flow[0].destFlow);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_info.flow[0].stageInfo.lb_type);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_info.flow[0].stageInfo.fn_id);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_info.flow[0].stageInfo.priority);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].flow_info.flow[0].stageInfo.event_q_id);
    ADK_TABLE_ENTRY_WRITE("|%05d",ingress_assoc[idx].tunnel_if_id);
    ADK_TABLE_ENTRY_WRITE("\n");

    return;
}


adk_status_t
adk_ipv6_ingress_assoc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Ingress Association Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|TreeWidth");
    ADK_TABLE_ENTRY_WRITE("|ruleMask");
    ADK_TABLE_ENTRY_WRITE("|VRDID");
    ADK_TABLE_ENTRY_WRITE("|IFID");
    ADK_TABLE_ENTRY_WRITE("|FlowLablel");
    ADK_TABLE_ENTRY_WRITE("|nextHeader");
    ADK_TABLE_ENTRY_WRITE("|SIP");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|SrcPort/icmpCode");
    ADK_TABLE_ENTRY_WRITE("|DstPort/icmpType");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|flow stats id");
    ADK_TABLE_ENTRY_WRITE("|PH cb_fun_id");
    ADK_TABLE_ENTRY_WRITE("|PH bind_handle");
    ADK_TABLE_ENTRY_WRITE("|cb func ");
    ADK_TABLE_ENTRY_WRITE("|socket id");
    ADK_TABLE_ENTRY_WRITE("|isIfFltr");
    ADK_TABLE_ENTRY_WRITE("|IfId");
    ADK_TABLE_ENTRY_WRITE("|dest id");
    ADK_TABLE_ENTRY_WRITE("|dest Flow id");
    ADK_TABLE_ENTRY_WRITE("|LB Type ");
    ADK_TABLE_ENTRY_WRITE("|dest Func ");
    ADK_TABLE_ENTRY_WRITE("|priority ");
    ADK_TABLE_ENTRY_WRITE("|evet Qid ");
    ADK_TABLE_ENTRY_WRITE("|Tunnel If Id \n");

    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    adk_lpm_dump(ipv6_tables->ipv6_dp_ingress_assoc_table.lpm_tbl.lpm,
                    adk_ipv6_ingress_assoc_dump_cb_fn, 
                    (void *) ipv6_tables);
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ipv6_filter_tbl_dump(adk_module_handle_t ipv6Handle)
{
    uint32_t Ruleset = 0, i = 0, idx = 0, numrules = 0;
    adk_ipv6_filter_rule_set_handle_t ruleSetHandles = NULL;
    adk_ipv6_filter_t *filterEntry = NULL;
    adk_ipv6_info_t *ipv6ModuleInfo_p;

    ipv6ModuleInfo_p = (adk_ipv6_info_t*)ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Filter Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|Index");
    ADK_TABLE_ENTRY_WRITE("|ruleMask");
    ADK_TABLE_ENTRY_WRITE("|Protocol");
    ADK_TABLE_ENTRY_WRITE("|Dscp");
    ADK_TABLE_ENTRY_WRITE("|SIP");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|SrcPort/icmpType");
    ADK_TABLE_ENTRY_WRITE("|DstPort/icmpCode");
    ADK_TABLE_ENTRY_WRITE("|Frag Type");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|enable counter");
    ADK_TABLE_ENTRY_WRITE("|cpuPath");
    ADK_TABLE_ENTRY_WRITE("|PH devId");
    ADK_TABLE_ENTRY_WRITE("|PH queueid");
    ADK_TABLE_ENTRY_WRITE("|PH cb_fun_id");
    ADK_TABLE_ENTRY_WRITE("|PH bind_handle");

    ADK_TABLE_ENTRY_WRITE("\n====================================================== \n");
    for ( Ruleset = 0; Ruleset < ADK_IPV6_FILTER_RULE_SETS_MAX; Ruleset++)
    {
        ruleSetHandles = ipv6ModuleInfo_p->ruleSetHandles[Ruleset];
        if ( ruleSetHandles != NULL)
        {
           if ( ADK_SUCCESS != adk_ipv6_internal_get_num_filter_rules(ipv6Handle, ruleSetHandles, &numrules))
           {
               continue;
           }
           if ( numrules != 0)
           {
               filterEntry =  (adk_ipv6_filter_t *) ADK_NVM_CALLOC (ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_TBL_DUMP_OBJ,  numrules, sizeof(adk_ipv6_filter_t));
               if ( filterEntry == NULL)
               {
                   continue;
               }
               if ( ADK_SUCCESS != adk_ipv6_internal_get_filter_rules(ipv6Handle, ruleSetHandles, numrules, filterEntry))
               {
                   ADK_NVM_FREE(filterEntry);
                   continue;
               }
               ADK_TABLE_ENTRY_WRITE("Number of Rules - %05d\n", numrules);
               for ( idx = 0; idx < numrules; idx++)
               {
                   ADK_TABLE_ENTRY_WRITE("|%05d",idx);
                   ADK_TABLE_ENTRY_WRITE("|%04x",filterEntry[idx].rules.ruleMask);
                   if ( filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_PROTO_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|%02x-%02x",filterEntry[idx].rules.protocol.begin, 
                                                          filterEntry[idx].rules.protocol.end);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }
                   if ( filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_DSCP_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|%02x-%02x",filterEntry[idx].rules.dscp.begin,
                                                          filterEntry[idx].rules.dscp.end);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }
                   if(filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_SRC_ADDR_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|");
                       for(i=0; i<4; i++)
                       {
                           ADK_TABLE_ENTRY_WRITE("%08x", filterEntry[idx].rules.srcIPv6Addr.Ipv6Addr.addr[i]);
                       }
                       ADK_TABLE_ENTRY_WRITE("/%03d", filterEntry[idx].rules.srcIPv6Addr.prefixLength);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }
                   if(filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_DEST_ADDR_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|");
                       for(i=0; i<4; i++)
                       {
                           ADK_TABLE_ENTRY_WRITE("%08x", filterEntry[idx].rules.dstIPv6Addr.Ipv6Addr.addr[i]);
                       }
                       ADK_TABLE_ENTRY_WRITE("/%03d", filterEntry[idx].rules.dstIPv6Addr.prefixLength);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }

                   if ( filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_PROTO_INFO_MASK)
                   {
                       if ( filterEntry[idx].rules.ruleMask & IPV6_ICMP_TYPE_MASK)
                       {
                           ADK_TABLE_ENTRY_WRITE("|%04x-%04x",filterEntry[idx].rules.protoInfo_u.icmpv6.type.begin,
                                                              filterEntry[idx].rules.protoInfo_u.icmpv6.type.end);
                       }
                       else
                       {
                           ADK_TABLE_ENTRY_WRITE("|*");
                       }
                       if ( filterEntry[idx].rules.ruleMask & IPV6_ICMP_CODE_MASK)
                       {
                           ADK_TABLE_ENTRY_WRITE("|%04x-%04x",filterEntry[idx].rules.protoInfo_u.icmpv6.code.begin,
                                                              filterEntry[idx].rules.protoInfo_u.icmpv6.code.end);
                       }
                       else
                       {
                           ADK_TABLE_ENTRY_WRITE("|*");
                       }
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*|*");
                   }
                   if ( filterEntry[idx].rules.ruleMask & ADK_IPV6_FILTER_FRAGMENT_MASK)
                   {
                       ADK_TABLE_ENTRY_WRITE("|%02x",filterEntry[idx].rules.frag_type);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*");
                   }

                   if ( filterEntry[idx].action.action == ADK_IPV6_FILTER_ACTION_ACCEPT)
                       ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_FILTER_ACTION_ACCEPT");
                   if ( filterEntry[idx].action.action == ADK_IPV6_FILTER_ACTION_DENY)
                       ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_FILTER_ACTION_DENY");
                   if ( filterEntry[idx].action.action == ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU)
                       ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU");
                   ADK_TABLE_ENTRY_WRITE("|%01d",filterEntry[idx].action.enableCounter);
                   if ( filterEntry[idx].action.action == ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU)
                   {
                       if ( filterEntry[idx].action.cpuPath & ADK_IPV6_PROCESSING_PATH_CPU_PH)
                           ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_PROCESSING_PATH_CPU_PH");
#ifdef ADK_QOS_GEN3_INTERFACE
                       if ( filterEntry[idx].action.cpuPath & ADK_IPV6_PROCESSING_PATH_CPU_SP_STACK)
                           ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_PROCESSING_PATH_CPU_SP_STACK");
#else
                       if ( filterEntry[idx].action.cpuPath & ADK_IPV6_PROCESSING_PATH_CPU_NOPH)
                           ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_PROCESSING_PATH_CPU_NOPH");
#endif
                       if ( filterEntry[idx].action.cpuPath & ADK_IPV6_PROCESSING_PATH_MPP)
                           ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_PROCESSING_PATH_MPP");
                       ADK_TABLE_ENTRY_WRITE("|%05d",filterEntry[idx].action.cpuPath_u.phAttr.devId);
                       ADK_TABLE_ENTRY_WRITE("|%05d",filterEntry[idx].action.cpuPath_u.phAttr.queueId);
                       ADK_TABLE_ENTRY_WRITE("|%"ADKPRIh"",filterEntry[idx].action.cpuPath_u.phAttr.phBindHandle);
                       ADK_TABLE_ENTRY_WRITE("|%08x\n",filterEntry[idx].action.cpuPath_u.phAttr.cbFnId);
                   }
                   else
                   {
                       ADK_TABLE_ENTRY_WRITE("|*|*|*|*|*\n");
                   }
                   
               }
               ADK_NVM_FREE(filterEntry);
           }
        }
    }
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}


adk_status_t
adk_ipv6_nexthop_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    int i;
    adk_ipv6_dp_nexthop_entry_t *nexthop_entry = ipv6_tables->ipv6_dp_nexthop_table.tbl;

    
    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Nexthop Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|nexthop id");
    ADK_TABLE_ENTRY_WRITE("|l2 resolved");
    ADK_TABLE_ENTRY_WRITE("|interface id");
    ADK_TABLE_ENTRY_WRITE("|dest id");
    ADK_TABLE_ENTRY_WRITE("|dest Flow id");
    ADK_TABLE_ENTRY_WRITE("|lb type");
    ADK_TABLE_ENTRY_WRITE("|function id");
    ADK_TABLE_ENTRY_WRITE("|priority");
    ADK_TABLE_ENTRY_WRITE("|event qid \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    for(i=0; i<ADK_IPV6_NO_OF_NEXTHOPS; i++)
    {
        if(nexthop_entry[i].is_enabled)
        {
            ADK_TABLE_ENTRY_WRITE("|%05d",i);
            if(nexthop_entry[i].is_l2_state_resolved)
            {
                ADK_TABLE_ENTRY_WRITE("|L2 RESOLVED");
            }
            else
            {
                ADK_TABLE_ENTRY_WRITE("|L2 NOT RESOLVED");
            }
            ADK_TABLE_ENTRY_WRITE("|%05x",nexthop_entry[i].interface_id);
            adk_ipv6_mod_id_str_print(nexthop_entry[i].flow_info.flow[0].destModId);
            ADK_TABLE_ENTRY_WRITE("|%05d",nexthop_entry[i].flow_info.flow[0].destFlow);
            ADK_TABLE_ENTRY_WRITE("|%03d",nexthop_entry[i].flow_info.flow[0].stageInfo.lb_type);
            ADK_TABLE_ENTRY_WRITE("|%03d",nexthop_entry[i].flow_info.flow[0].stageInfo.fn_id);
            ADK_TABLE_ENTRY_WRITE("|%03d",nexthop_entry[i].flow_info.flow[0].stageInfo.priority);
            ADK_TABLE_ENTRY_WRITE("|%03d",nexthop_entry[i].flow_info.flow[0].stageInfo.event_q_id);

            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_egress_assoc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    int i, j;
    adk_ipv6_dp_egress_flow_attr_entry_t *egress_assoc = ipv6_tables->ipv6_dp_egress_flow_attr_table.tbl;
   

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Egress association Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|nexthop flowid");
    ADK_TABLE_ENTRY_WRITE("|priority_flag");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|action_flags");
    ADK_TABLE_ENTRY_WRITE("|mtu");
    ADK_TABLE_ENTRY_WRITE("|flow_stats_id");
    ADK_TABLE_ENTRY_WRITE("|ver_tc_flabel");
    ADK_TABLE_ENTRY_WRITE("|len_nh_hlimit");
    ADK_TABLE_ENTRY_WRITE("|src_addr");
    ADK_TABLE_ENTRY_WRITE("|dst_addr");
    ADK_TABLE_ENTRY_WRITE("|src_port");
    ADK_TABLE_ENTRY_WRITE("|dst_port \n");
    ADK_TABLE_ENTRY_WRITE(" === =================================================== \n");
                                







    for(i=0; i<ADK_IPV6_NO_OF_EGRESS_FLOW; i++)
    {
        if(egress_assoc[i].is_enabled)
        {
            ADK_TABLE_ENTRY_WRITE("|%05d",i);
            ADK_TABLE_ENTRY_WRITE("|%05d",egress_assoc[i].priority_flag);
            ADK_TABLE_ENTRY_WRITE("|%05d",egress_assoc[i].vrd_id);
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_FWD_TO_ETH)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_FWD_TO_ETH");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_ENCAP)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_ENCAP");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_ISATAP_TUNNEL)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_ISATAP_TUNNEL");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_6TO4_TUNNEL)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_6TO4_TUNNEL");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_TEREDO_TUNNEL)    
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_TEREDO_TUNNEL");    
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_LINKLOCAL)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_LINKLOCAL");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_ADD_FRAG_HDR)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTI ON_ADD_FRAG_HDR");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_ADD_UDP_HDR)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTION_ADD_UDP_HDR");
            if(egress_assoc[i].action_flags & ADK_IPV6_EGRESS_ATTR_ACTION_TRANSLATE_TO_IPV4)
                ADK_TABLE_ENTRY_WRITE("|ADK_IPV6_EGRESS_ATTR_ACTION_TRANSLATE_TO_IPV4");
            ADK_TABLE_ENTRY_WRITE("|%05d",egress_assoc[i].mtu);
            ADK_TABLE_ENTRY_WRITE("|%05d",egress_assoc[i].flow_stats_id);
            ADK_TABLE_ENTRY_WRITE("|%08x",ntohl(egress_assoc[i].ver_tc_flabel));
            ADK_TABLE_ENTRY_WRITE("|%08x",ntohl(egress_assoc[i].len_nh_hlimit));
            ADK_TABLE_ENTRY_WRITE("|"); 
            for(j=0; j<ADK_IPV6_PDU_ADDR_SIZE_WORDS; j++)
                ADK_TABLE_ENTRY_WRITE("%08x",ntohl(egress_assoc[i].src_addr.addr[j]));
            ADK_TABLE_ENTRY_WRITE("|"); 
            for(j=0; j<ADK_IPV6_PDU_ADDR_SIZE_WORDS; j++)
                ADK_TABLE_ENTRY_WRITE("%08x",ntohl(egress_assoc[i].dst_addr.addr[j]));
            ADK_TABLE_ENTRY_WRITE("|%05d",ntohs(egress_assoc[i].src_port));
            ADK_TABLE_ENTRY_WRITE("|%05d",ntohs(egress_assoc[i].dst_port));
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_vrd_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    int i;
    adk_ipv6_dp_vrd_host_mode_entry_t *vrd_tbl = ipv6_tables->ipv6_dp_vrd_table.tbl ;
   

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 VRD Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|VRD ID");
    ADK_TABLE_ENTRY_WRITE("|enable routing \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<ADK_IPV6_VRD_ID_MAX; i++)
    {
        if(vrd_tbl[i].is_enabled)
        {
            ADK_TABLE_ENTRY_WRITE("|%02d",i);
            ADK_TABLE_ENTRY_WRITE("|%05d",vrd_tbl[i].enable_routing);
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_excep_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    int i;
    adk_ipv6_dp_excep_entry_t *excep_tbl = ipv6_tables->ipv6_dp_excep_table.tbl;
    char excep_path[4][100] = { 
        {"INVALID_PATH"},
        {"ADK_IPV6_EXCEPTION_LOC_CPU_PH"},
#ifdef ADK_QOS_GEN3_INTERFACE
        {"ADK_IPV6_EXCEPTION_LOC_CPU_SP_STACK"},
#else
        {"ADK_IPV6_EXCEPTION_LOC_CPU_NOPH"},
#endif
        {"ADK_IPV6_EXCEPTION_LOC_MPP"}};
    
    char domain[4][100] = {
        {"ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN"},
        {"ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN"},
        {"ADK_IPV6_EXCEPTION_OS_ORIGIN_DOMAIN"}
    };

    char excep_type[23][100] = {
        {"ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND"},
        {"ADK_IPV6_EXCEPTION_PARAM_PROBLEM"},
        {"ADK_IPV6_EXCEPTION_TOO_MANY_EXTENSION_HEADERS"},
        {"ADK_IPV6_EXCEPTION_RESOLVE_NEXTHOP_L2_ADDRESS"},
        {"ADK_IPV6_EXCEPTION_RESOLVE_DESTINATION_L2_ADDRESS"},
        {"ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO"},
        {"ADK_IPV6_EXCEPTION_MTU_VIOLATION"},
        {"ADK_IPV6_EXCEPTION_UNKNOWN_EGRESS_ASSOCIATION"},
        {"ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED"},
        {"ADK_IPV6_EXCEPTION_FRAG_OVERLAP"},
        {"ADK_IPV6_EXCEPTION_SEGMENTS_LEFT_NON_ZERO"},
        {"ADK_IPV6_EXCEPTION_IPV6_ICMP"},
        {"ADK_IPV6_MULTICAST_FWD_LOOKUP_FAIL"},
        {"ADK_IPV6_MULTICAST_RPF_CHECK_FAIL"},
        {"ADK_IPV6_REASSEMBLY_MAX_PDU_SIZE_REACHED"},
        {"ADK_IPV6_ILLEGAL_SRC_ADDR_TRANSLATE_RECEIVED"},
        {"ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID"},
        {"ADK_IPV6_UDP_INGRESS_LENGTH_INVALID"},
        {"ADK_IPV6_EXCEPTION_DST_MTU_FAIL"},
    };


    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Exception Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|Domain");
    ADK_TABLE_ENTRY_WRITE("|Exception");
    ADK_TABLE_ENTRY_WRITE("|clbk fun id");
    ADK_TABLE_ENTRY_WRITE("|exception path");
    ADK_TABLE_ENTRY_WRITE("|PH bind handle \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<(ADK_IPV6_EXCEPTION_DOMAIN_MAX*ADK_IPV6_EXCEPTION_MAX); i++)
    {
        if(excep_tbl[i].is_enable)
        {
            ADK_TABLE_ENTRY_WRITE("|%s",domain[i/ADK_IPV6_EXCEPTION_MAX]);
            ADK_TABLE_ENTRY_WRITE("|%s",excep_type[i%ADK_IPV6_EXCEPTION_MAX]);
            ADK_TABLE_ENTRY_WRITE("|%03d",excep_tbl[i].callback_fun_id);
            ADK_TABLE_ENTRY_WRITE("|%s",excep_path[excep_tbl[i].exception_path]);
            ADK_TABLE_ENTRY_WRITE("|%"ADKPRIh"",excep_tbl[i].ph_bind_handle);
            ADK_TABLE_ENTRY_WRITE("\n");
        }
    }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}


void adk_ipv6_ingress_sip_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value)
{
    adk_ipv6_dp_ingress_flow_sip_entry_t *sip_validate_entry = ( adk_ipv6_dp_ingress_flow_sip_entry_t *)(void *)value;
    int i;
    char sip_cpu_path[3][20] = {{"NA"}, {"PH_Path"}, {"Non_PH_Path"}};
    char sip_action[10][100] = {
        {"ADK_IPV6_ACTION_ALLOW"},
        {"INVALD_ACTION"},
        {"ADK_IPV6_ACTION_DROP"},
        {"INVALID_ACTION"},
        {"ADK_IPV6_ACTION_COUNT_AND_DROP"},
        {"INVALID_ACTION"},
        {"ADK_IPV6_ACTION_FORWARD_TO_CPU"}
    };
   
    ADK_TABLE_ENTRY_WRITE("|%02d|", key[0]);
    ADK_TABLE_ENTRY_WRITE("|");
    for(i=1; i<=16; i++)
    ADK_TABLE_ENTRY_WRITE("%02x",key[i]);
    ADK_TABLE_ENTRY_WRITE("|%02d",depth);
    ADK_TABLE_ENTRY_WRITE("|%01d",sip_validate_entry->apply_to_fwd_traffic);
    ADK_TABLE_ENTRY_WRITE("|%s",sip_action[sip_validate_entry->action]);
    ADK_TABLE_ENTRY_WRITE("|%s",sip_cpu_path[sip_validate_entry->cpuPath]);
    ADK_TABLE_ENTRY_WRITE("|%02d",sip_validate_entry->cbFnId);
    ADK_TABLE_ENTRY_WRITE("|%"ADKPRIh"", sip_validate_entry->ph_bind_handle);
    ADK_TABLE_ENTRY_WRITE("\n");

    return;
}


adk_status_t
adk_ipv6_ingress_sip_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    ADK_TABLE_ENTRY_WRITE(" \t IPv6 Ingress SIP Address Validation Table \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|VRDID");
    ADK_TABLE_ENTRY_WRITE("|DIP");
    ADK_TABLE_ENTRY_WRITE("|MASK");
    ADK_TABLE_ENTRY_WRITE("|apply to fwd traffic");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|CPU Path");
    ADK_TABLE_ENTRY_WRITE("|callback_fun_id");
    ADK_TABLE_ENTRY_WRITE("|ph_bind_handle \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    adk_lpm_dump(ipv6_tables->ipv6_dp_ingress_flow_sip_table.lpm_tbl.lpm,
                    adk_ipv6_ingress_sip_dump_cb_fn,
                    (void *) ipv6_tables);
    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_fc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables)
{
    adk_ipv6_dp_lli_flow_cache_entry_t *lli_fc_entry = NULL;
    adk_ipv6_dp_uli_flow_cache_entry_t *uli_fc_entry = NULL;
    uint32_t i, j, l, idx;
    char lli_fc_action[10][100] = {
        {"INVALID_ACTION"},
        {"ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND"},
        {"ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND"},
        {"ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_BIND"},
        {"ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE"},
        {"ADK_IPV6_DP_LLI_FLOW_CACHE_FRAGMENT_TERMINATE"}
    };

    char uli_fc_action[10][100] = {
        {"INVALID_ACTION"},
        {"ADK_IPV6_DP_ULI_FLOW_CACHE_FWD_BIND"}, 
        {"ADK_IPV6_DP_ULI_FLOW_CACHE_EGRESS_BIND"},
    };

    char fc_type[10][100] = {
        {"FLOW_CACHE_TYPE_5TUPLE"},
        {"FLOW_CACHE_TYPE_5TUPLE_FRAG"},
        {"FLOW_CACHE_TYPE_3TUPLE"},
        {"FLOW_CACHE_TYPE_3TUPLE_FRAG"}
    };
    char if_type[3][100] = {
        {"IPV6_INTERFACE_TYPE_ETH"},
        {"IPV6_INTERFACE_TYPE_TUNNEL"},
        {"IPV6_INTERFACE_TYPE_VIRTUAL"}};

    ADK_TABLE_ENTRY_WRITE(" \t IPv6 LLI FLOW CACHE TABLE DUMP \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE(" IPV6 LLI ROUTE FLOW CACHE ENTRIES \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|LCORE IDX");
    ADK_TABLE_ENTRY_WRITE("|FC index");
    ADK_TABLE_ENTRY_WRITE("|enable_qos");
    ADK_TABLE_ENTRY_WRITE("|reassembly_flag");
    ADK_TABLE_ENTRY_WRITE("|enable_ingress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_egress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_policing");
    ADK_TABLE_ENTRY_WRITE("|ing_assoc_lkup_done");
    ADK_TABLE_ENTRY_WRITE("|eg_pktcap_enabled");
    ADK_TABLE_ENTRY_WRITE("|version_id");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|next_hdr");
    ADK_TABLE_ENTRY_WRITE("|fc_type");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|ing_ruleset_id");
    ADK_TABLE_ENTRY_WRITE("|src_addr");
    ADK_TABLE_ENTRY_WRITE("|dst_addr");
    ADK_TABLE_ENTRY_WRITE("|src_port");
    ADK_TABLE_ENTRY_WRITE("|dst_port");
    ADK_TABLE_ENTRY_WRITE("|LB Type ");
    ADK_TABLE_ENTRY_WRITE("|destFunc ");
    ADK_TABLE_ENTRY_WRITE("|priority ");
    ADK_TABLE_ENTRY_WRITE("|event Qid ");
    ADK_TABLE_ENTRY_WRITE("|destFlowId");
    ADK_TABLE_ENTRY_WRITE("|dest_module_id");
    ADK_TABLE_ENTRY_WRITE("|egress_if_id");
    ADK_TABLE_ENTRY_WRITE("|mtu");
    ADK_TABLE_ENTRY_WRITE("|egress_if_type");
    ADK_TABLE_ENTRY_WRITE("|ipsec_info");

    ADK_TABLE_ENTRY_WRITE(" \n====================================================== \n");
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        for(j=0; j<(ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES); j++)
            {
            if(ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].is_enabled)
                {
                idx = ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].index;

                    lli_fc_entry = &ipv6_tables->ipv6_dp_lli_flow_cache_table[i].tbl[idx];

                    if((lli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND) ||
                            (lli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND))
                    {
                        ADK_TABLE_ENTRY_WRITE("|%05d", i);      
                        ADK_TABLE_ENTRY_WRITE("|%05d", idx);      
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_qos);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->reassembly_flag);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_ingress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_egress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_policing);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->ing_assoc_lkup_done);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->eg_pktcap_enabled);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->version_id);
                        ADK_TABLE_ENTRY_WRITE("|%s", lli_fc_action[lli_fc_entry->action]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->next_hdr);
                        ADK_TABLE_ENTRY_WRITE("|%s", fc_type[lli_fc_entry->fc_type]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->vrd_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d|", lli_fc_entry->ing_ruleset_id);
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->src_addr))[l]);
                        ADK_TABLE_ENTRY_WRITE("|");
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->dst_addr))[l])
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->src_port));
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->dst_port));
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.lb_type);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.fn_id);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.priority);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.event_q_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->destFlowId);
                        adk_ipv6_mod_id_str_print(lli_fc_entry->dest_module_id);
                        ADK_TABLE_ENTRY_WRITE("|%04d", lli_fc_entry->egress_bind_attr.egress_if_id );
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->egress_bind_attr.mtu);
                        ADK_TABLE_ENTRY_WRITE("|%s", if_type[lli_fc_entry->egress_bind_attr.egress_if_type]);
                        ADK_TABLE_ENTRY_WRITE("|%p", lli_fc_entry->egress_bind_attr.ipsec_info);
                        ADK_TABLE_ENTRY_WRITE("\n");
                    }
                }
            }
        }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE(" IPV6 LLI TERMINATE FLOW CACHE ENTRIES \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|LCORE IDX");
    ADK_TABLE_ENTRY_WRITE("|FC index");
    ADK_TABLE_ENTRY_WRITE("|enable_qos");
    ADK_TABLE_ENTRY_WRITE("|reassembly_flag");
    ADK_TABLE_ENTRY_WRITE("|enable_ingress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_egress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_policing");
    ADK_TABLE_ENTRY_WRITE("|ing_assoc_lkup_done");
    ADK_TABLE_ENTRY_WRITE("|eg_pktcap_enabled");
    ADK_TABLE_ENTRY_WRITE("|version_id");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|next_hdr");
    ADK_TABLE_ENTRY_WRITE("|fc_type");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|ing_ruleset_id");
    ADK_TABLE_ENTRY_WRITE("|src_addr");
    ADK_TABLE_ENTRY_WRITE("|dst_addr");
    ADK_TABLE_ENTRY_WRITE("|src_port");
    ADK_TABLE_ENTRY_WRITE("|dst_port");
    ADK_TABLE_ENTRY_WRITE("|LB Type");
    ADK_TABLE_ENTRY_WRITE("|destFunc");
    ADK_TABLE_ENTRY_WRITE("|priority");
    ADK_TABLE_ENTRY_WRITE("|event Qid");
    ADK_TABLE_ENTRY_WRITE("|destFlowId");
    ADK_TABLE_ENTRY_WRITE("|dest_module_id");
    ADK_TABLE_ENTRY_WRITE("|ingress_flow_statsid; ");
    ADK_TABLE_ENTRY_WRITE("|vif_flag");
    ADK_TABLE_ENTRY_WRITE("|vif_stats_id");
    ADK_TABLE_ENTRY_WRITE("|cbAddr");
    ADK_TABLE_ENTRY_WRITE("|socket_id");
    ADK_TABLE_ENTRY_WRITE("|user_data");

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        for(j=0; j<(ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES); j++)
            {
            if(ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].is_enabled)
                {
                idx = ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].index;
                    lli_fc_entry = &ipv6_tables->ipv6_dp_lli_flow_cache_table[i].tbl[idx];

                    if((lli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_BIND) ||
                            (lli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE))
                    {
                        ADK_TABLE_ENTRY_WRITE("|%05d", i);      
                        ADK_TABLE_ENTRY_WRITE("|%05d", idx);      
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_qos);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->reassembly_flag);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_ingress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_egress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_policing);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->ing_assoc_lkup_done);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->eg_pktcap_enabled);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->version_id);
                        ADK_TABLE_ENTRY_WRITE("|%s", lli_fc_action[lli_fc_entry->action]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->next_hdr);
                        ADK_TABLE_ENTRY_WRITE("|%s", fc_type[lli_fc_entry->fc_type]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->vrd_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d|", lli_fc_entry->ing_ruleset_id);
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->src_addr))[l]);
                        ADK_TABLE_ENTRY_WRITE("|");
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->dst_addr))[l])
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->src_port));
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->dst_port));
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.lb_type);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.fn_id);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.priority);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.event_q_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->destFlowId);
                        adk_ipv6_mod_id_str_print(lli_fc_entry->dest_module_id);
                        ADK_TABLE_ENTRY_WRITE("|%04d", lli_fc_entry->ingress_term_attr.ingress_flow_statsid);
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->ingress_term_attr.vif_flag);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->ingress_term_attr.vif_stats_id);
                        ADK_TABLE_ENTRY_WRITE("|%p"  , lli_fc_entry->ingress_term_attr.invoke_cb_attr.recvCb);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->ingress_term_attr.invoke_cb_attr.id);
                        ADK_TABLE_ENTRY_WRITE("|%p"  , lli_fc_entry->ingress_term_attr.invoke_cb_attr.user_data);
                        ADK_TABLE_ENTRY_WRITE("\n");
                    }
                }
            }
        }

    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE(" IPV6 LLI DEFRAG FLOW CACHE ENTRIES \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|LCORE IDX");
    ADK_TABLE_ENTRY_WRITE("|FC index");
    ADK_TABLE_ENTRY_WRITE("|enable_qos");
    ADK_TABLE_ENTRY_WRITE("|reassembly_flag");
    ADK_TABLE_ENTRY_WRITE("|enable_ingress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_egress_acl");
    ADK_TABLE_ENTRY_WRITE("|enable_policing");
    ADK_TABLE_ENTRY_WRITE("|ing_assoc_lkup_done");
    ADK_TABLE_ENTRY_WRITE("|eg_pktcap_enabled");
    ADK_TABLE_ENTRY_WRITE("|version_id");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|next_hdr");
    ADK_TABLE_ENTRY_WRITE("|fc_type");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|ing_ruleset_id");
    ADK_TABLE_ENTRY_WRITE("|src_addr");
    ADK_TABLE_ENTRY_WRITE("|dst_addr");
    ADK_TABLE_ENTRY_WRITE("|src_port");
    ADK_TABLE_ENTRY_WRITE("|dst_port");
    ADK_TABLE_ENTRY_WRITE("|LB Type");
    ADK_TABLE_ENTRY_WRITE("|destFunc");
    ADK_TABLE_ENTRY_WRITE("|priority");
    ADK_TABLE_ENTRY_WRITE("|event Qid");
    ADK_TABLE_ENTRY_WRITE("|destFlowId");
    ADK_TABLE_ENTRY_WRITE("|dest_module_id");
    ADK_TABLE_ENTRY_WRITE("|ingress_flow_statsid; ");
    ADK_TABLE_ENTRY_WRITE("|vif_flag");
    ADK_TABLE_ENTRY_WRITE("|vif_stats_id");
    ADK_TABLE_ENTRY_WRITE("|cbAddr");
    ADK_TABLE_ENTRY_WRITE("|socket_id");
    ADK_TABLE_ENTRY_WRITE("|user_data");

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        for(j=0; j<(ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES); j++)
        {
            if(ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].is_enabled)
            {
                idx = ipv6_tables->ipv6_dp_lli_flow_cache_table[i].hash_tbl[j].index;

                    lli_fc_entry = &ipv6_tables->ipv6_dp_lli_flow_cache_table[i].tbl[idx];

                    if((lli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_FRAGMENT_TERMINATE))
                    {
                        ADK_TABLE_ENTRY_WRITE("|%05d", i);      
                        ADK_TABLE_ENTRY_WRITE("|%05d", idx);      
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_qos);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->reassembly_flag);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_ingress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_egress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->enable_policing);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->ing_assoc_lkup_done);
                        ADK_TABLE_ENTRY_WRITE("|%01d", lli_fc_entry->eg_pktcap_enabled);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->version_id);
                        ADK_TABLE_ENTRY_WRITE("|%s", lli_fc_action[lli_fc_entry->action]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->next_hdr);
                        ADK_TABLE_ENTRY_WRITE("|%s", fc_type[lli_fc_entry->fc_type]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->vrd_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d|", lli_fc_entry->ing_ruleset_id);
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->src_addr))[l]);
                        ADK_TABLE_ENTRY_WRITE("|");
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&lli_fc_entry->dst_addr))[l])
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->src_port));
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(lli_fc_entry->dst_port));
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.lb_type);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.fn_id);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.priority);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->stageInfo.event_q_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->destFlowId);
                        adk_ipv6_mod_id_str_print(lli_fc_entry->dest_module_id);
                        ADK_TABLE_ENTRY_WRITE("|%04d", lli_fc_entry->ingress_frag_term_attr.ingress_assoc_rulemask);
                        ADK_TABLE_ENTRY_WRITE("|%05d", lli_fc_entry->ingress_frag_term_attr.vif_flag);
                        ADK_TABLE_ENTRY_WRITE("|%02d", lli_fc_entry->ingress_frag_term_attr.vif_stats_id);
                        ADK_TABLE_ENTRY_WRITE("|%p"  , lli_fc_entry->ingress_frag_term_attr.invoke_cb_attr.recvCb);
                        ADK_TABLE_ENTRY_WRITE("|%03d", lli_fc_entry->ingress_frag_term_attr.invoke_cb_attr.id);
                        ADK_TABLE_ENTRY_WRITE("|%p"  , lli_fc_entry->ingress_frag_term_attr.invoke_cb_attr.user_data);
                        ADK_TABLE_ENTRY_WRITE("\n");
                    }
                }
            }
        }
    ADK_TABLE_ENTRY_WRITE("\n");

    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE(" \t IPv6 ULI FLOW CACHE TABLE DUMP \n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    ADK_TABLE_ENTRY_WRITE("|LCORE IDX");
    ADK_TABLE_ENTRY_WRITE("|FC index");
    ADK_TABLE_ENTRY_WRITE("|enable_qos");
    ADK_TABLE_ENTRY_WRITE("|enable_egress_acl");
    ADK_TABLE_ENTRY_WRITE("|mtu_excp");
    ADK_TABLE_ENTRY_WRITE("|eg_pktcap_enabled");
    ADK_TABLE_ENTRY_WRITE("|version_id");
    ADK_TABLE_ENTRY_WRITE("|action");
    ADK_TABLE_ENTRY_WRITE("|next_hdr");
    ADK_TABLE_ENTRY_WRITE("|out_acl_statsid");
    ADK_TABLE_ENTRY_WRITE("|fc_type");
    ADK_TABLE_ENTRY_WRITE("|vrd_id");
    ADK_TABLE_ENTRY_WRITE("|src_addr");
    ADK_TABLE_ENTRY_WRITE("|dst_addr");
    ADK_TABLE_ENTRY_WRITE("|src_port");
    ADK_TABLE_ENTRY_WRITE("|dst_port");
    ADK_TABLE_ENTRY_WRITE("|egress_if_id");
    ADK_TABLE_ENTRY_WRITE("|egress_if_type");
    ADK_TABLE_ENTRY_WRITE("|mtu");
    ADK_TABLE_ENTRY_WRITE("|ipsec_info");
    ADK_TABLE_ENTRY_WRITE("|LB Type");
    ADK_TABLE_ENTRY_WRITE("|destFunc");
    ADK_TABLE_ENTRY_WRITE("|priority");
    ADK_TABLE_ENTRY_WRITE("|event Qid");
    ADK_TABLE_ENTRY_WRITE("|destFlowId");
    ADK_TABLE_ENTRY_WRITE("|dest_module_id");
    ADK_TABLE_ENTRY_WRITE(" \n====================================================== \n");
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        for(j=0; j<(ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES); j++)
        {
            if(ipv6_tables->ipv6_dp_uli_flow_cache_table[i].hash_tbl[j].is_enabled)
            {
                idx = ipv6_tables->ipv6_dp_uli_flow_cache_table[i].hash_tbl[j].index;

                    uli_fc_entry = &ipv6_tables->ipv6_dp_uli_flow_cache_table[i].tbl[idx];

                    if((uli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND) ||
                            (uli_fc_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND))
                    {
                        ADK_TABLE_ENTRY_WRITE("|%05d", i);      
                        ADK_TABLE_ENTRY_WRITE("|%05d", idx);      
                        ADK_TABLE_ENTRY_WRITE("|%01d", uli_fc_entry->enable_qos);
                        ADK_TABLE_ENTRY_WRITE("|%01d", uli_fc_entry->enable_egress_acl);
                        ADK_TABLE_ENTRY_WRITE("|%01d", uli_fc_entry->mtu_excp);
                        ADK_TABLE_ENTRY_WRITE("|%01d", uli_fc_entry->eg_pktcap_enabled);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->version_id);
                        ADK_TABLE_ENTRY_WRITE("|%s", uli_fc_action[uli_fc_entry->action]);
                        ADK_TABLE_ENTRY_WRITE("|%02d", uli_fc_entry->next_hdr);
                        ADK_TABLE_ENTRY_WRITE("|%04d", uli_fc_entry->out_acl_statsid);
                        ADK_TABLE_ENTRY_WRITE("|%s", fc_type[uli_fc_entry->fc_type]);
                        ADK_TABLE_ENTRY_WRITE("|%02d|", uli_fc_entry->vrd_id);
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&uli_fc_entry->src_addr))[l]);
                        ADK_TABLE_ENTRY_WRITE("|");
                        for(l=0; l<ADK_IPV6_PDU_ADDR_SIZE_BYTES; l++)
                            ADK_TABLE_ENTRY_WRITE("%02x", 
                                    ((uint8_t *) (&uli_fc_entry->dst_addr))[l])
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(uli_fc_entry->src_port));
                        ADK_TABLE_ENTRY_WRITE("|%05d", ntohs(uli_fc_entry->dst_port));
                        ADK_TABLE_ENTRY_WRITE("|%04d", uli_fc_entry->egress_if_id);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->egress_if_type);
                        ADK_TABLE_ENTRY_WRITE("|%05d", uli_fc_entry->mtu);
                        ADK_TABLE_ENTRY_WRITE("|%p"  , uli_fc_entry->ipsec_info);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->stageInfo.lb_type);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->stageInfo.fn_id);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->stageInfo.priority);
                        ADK_TABLE_ENTRY_WRITE("|%03d", uli_fc_entry->stageInfo.event_q_id);
                        ADK_TABLE_ENTRY_WRITE("|%05d", uli_fc_entry->destFlowId);
                        adk_ipv6_mod_id_str_print(uli_fc_entry->dest_module_id);
                        ADK_TABLE_ENTRY_WRITE("\n");
                    }
                }
            }
        }

    ADK_TABLE_ENTRY_WRITE("\n");
    ADK_TABLE_ENTRY_WRITE(" ====================================================== \n");
    return ADK_SUCCESS;
}


adk_status_t  
adk_ipv6_internal_get_num_filter_rules ( 
        adk_module_handle_t __attribute__ ((unused))ipv6Handle,
        adk_ipv6_filter_rule_set_handle_t __attribute__ ((unused))filterRuleSetHandle,
        uint32_t __attribute__ ((unused))*numOfRules_p)
{
    adk_status_t retValue = ADK_IPV6_SUCCESS;
#if ADK_IPV6_ACL_ENABLE
    uint32_t numRules = 0, Ruleset = 0;
    adk_sys_handle_t sysHandle;
    adk_cp_acl_list_rule_t cp_rule;
    ipv6_filter_rule_set_info_t *filter_info_p;
    adk_ipv6_info_t       	*ipv6ModuleInfo_p=NULL;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    
    /*Check if valid input interface input is given*/
    if ( filterRuleSetHandle )
    {
        if (!adk_ipv6_valid_rule_set_handle(filterRuleSetHandle)) 
        {
            ADK_LOG_ERROR(sysHandle, ipv6Handle, "Invalid IPv6 filter RulesetHandle specified\n");
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
            goto _adk_ipv6_internal_get_filter_rules_return;
        }
        filter_info_p = (ipv6_filter_rule_set_info_t *)(filterRuleSetHandle);

        adk_cp_acl_list_get_reset(filter_info_p->cp_list);
        while( ADK_SUCCESS == adk_cp_acl_list_next_rule_get(
                                    filter_info_p->cp_list, &cp_rule))
        {
            numRules++;
        }
    }
    else
    {
        for ( Ruleset = 0; Ruleset < ADK_IPV6_FILTER_RULE_SETS_MAX; Ruleset++)
        {
            filterRuleSetHandle = ipv6ModuleInfo_p->ruleSetHandles[Ruleset];
            if ( filterRuleSetHandle )
            {
                if (!adk_ipv6_valid_rule_set_handle(filterRuleSetHandle)) 
                {
                    ADK_LOG_ERROR(sysHandle, ipv6Handle, "Invalid IPv6 filter RulesetHandle specified\n");
                    retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
                    goto _adk_ipv6_internal_get_filter_rules_return;
                }
                filter_info_p = (ipv6_filter_rule_set_info_t *)(filterRuleSetHandle);

                adk_cp_acl_list_get_reset(filter_info_p->cp_list);
                while( ADK_SUCCESS == adk_cp_acl_list_next_rule_get(
                                            filter_info_p->cp_list, &cp_rule))
                {
                    numRules++;
                }
            }
        }
    }
    
    *numOfRules_p = numRules;
_adk_ipv6_internal_get_filter_rules_return:
#endif
    ADK_RETURN ( retValue );
}

adk_status_t  
adk_ipv6_internal_get_filter_rules ( 
        adk_module_handle_t __attribute__ ((unused))ipv6Handle,
        adk_ipv6_filter_rule_set_handle_t __attribute__ ((unused))filterRuleSetHandle,
        uint32_t __attribute__ ((unused))numOfRules,
        adk_ipv6_filter_t __attribute__ ((unused))*filter_p)
{
    adk_status_t retValue = ADK_IPV6_SUCCESS;
#if ADK_IPV6_ACL_ENABLE
    uint32_t numRules = 0, Ruleset = 0;
    adk_sys_handle_t sysHandle;
    adk_cp_acl_list_rule_t cp_rule;
    adk_ipv6_filter_rule_handle_info_t *rule_handle;
    ipv6_filter_rule_set_info_t *filter_info_p;
    adk_ipv6_info_t       	*ipv6ModuleInfo_p=NULL;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);


    /*Check if valid input interface input is given*/
    if ( filterRuleSetHandle)
    {
        if (!adk_ipv6_valid_rule_set_handle(filterRuleSetHandle)) 
        {
            ADK_LOG_ERROR(sysHandle, ipv6Handle, "Invalid IPv6 filter RulesetHandle specified\n");
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
            goto _adk_ipv6_get_filter_rules_return;
        }
        filter_info_p = (ipv6_filter_rule_set_info_t *)(filterRuleSetHandle);
        adk_cp_acl_list_get_reset(filter_info_p->cp_list);
        while( (ADK_SUCCESS == adk_cp_acl_list_next_rule_get(
                                    filter_info_p->cp_list, &cp_rule)) &&
               (numRules < numOfRules))
        {
            rule_handle = (adk_ipv6_filter_rule_handle_info_t *) cp_rule.user_data;
            ADK_MEMCPY(&(filter_p[numRules].rules), &(rule_handle->filterRules), sizeof(adk_ipv6_filter_rules_t));
            ADK_MEMCPY(&(filter_p[numRules].action), &(rule_handle->filterActions), sizeof(adk_ipv6_filter_action_t)); 
            numRules++;
        }
    }
    else
    {
        for ( Ruleset = 0; Ruleset < ADK_IPV6_FILTER_RULE_SETS_MAX; Ruleset++)
        {
            filterRuleSetHandle = ipv6ModuleInfo_p->ruleSetHandles[Ruleset];
            if ( filterRuleSetHandle)
            {
                if (!adk_ipv6_valid_rule_set_handle(filterRuleSetHandle)) 
                {
                    ADK_LOG_ERROR(sysHandle, ipv6Handle, "Invalid IPv6 filter RulesetHandle specified\n");
                    retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
                    goto _adk_ipv6_get_filter_rules_return;
                }
                filter_info_p = (ipv6_filter_rule_set_info_t *)(filterRuleSetHandle);
                adk_cp_acl_list_get_reset(filter_info_p->cp_list);
                while( (ADK_SUCCESS == adk_cp_acl_list_next_rule_get(
                                            filter_info_p->cp_list, &cp_rule)) &&
                       (numRules < numOfRules))
                {
                    rule_handle = (adk_ipv6_filter_rule_handle_info_t *) cp_rule.user_data;
                    ADK_MEMCPY(&(filter_p[numRules].rules), &(rule_handle->filterRules), sizeof(adk_ipv6_filter_rules_t));
                    ADK_MEMCPY(&(filter_p[numRules].action), &(rule_handle->filterActions), sizeof(adk_ipv6_filter_action_t)); 
                    numRules++;
                }
            }
            
        }
    }
    
_adk_ipv6_get_filter_rules_return:
#endif
    ADK_RETURN ( retValue );
}

adk_status_t
adk_ipv6_dump_tables(adk_module_handle_t ipv6Handle, 
        adk_ipv6_tree_conf_t *ipv6TreeConf_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ipv6ModuleInfo_p = (adk_ipv6_info_t*)ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if (!ipv6ModuleInfo_p) 
    {
        ADK_FORCE_LOG_ERROR( "Module handle is NULL \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        goto adk_ipv6_dump_tables_return;
    }

    switch(ipv6TreeConf_p->group)
    {
        case ADK_IPV6_TABLES_INTF_ATTR:
            adk_ipv6_if_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_ROUTING:
            adk_ipv6_route_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_pbr_tbl_dump(ipv6Handle);
            break;

        case ADK_IPV6_TABLES_INGRESS_ASSOC:
            adk_ipv6_ingress_assoc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_FILTERS:
            adk_ipv6_filter_tbl_dump(ipv6Handle);
            break;

        case ADK_IPV6_TABLES_NEXT_HOP:
            adk_ipv6_nexthop_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_onlink_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_EGRESS_ASSOC:
            adk_ipv6_egress_assoc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_VRD:
            adk_ipv6_vrd_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_EXCEPTIONS:
            adk_ipv6_excep_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;
        
        case ADK_IPV6_TABLES_DEST_MTU:
            adk_ipv6_dest_mtu_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_REASSEMBLY:
            adk_ipv6_reass_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_FLOWCACHE:
            adk_ipv6_fc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            break;

        case ADK_IPV6_TABLES_ALL:
            adk_ipv6_if_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_route_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_onlink_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_ingress_assoc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_nexthop_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_egress_assoc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_vrd_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_excep_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_ingress_sip_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_dest_mtu_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_reass_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_fc_tbl_dump(ipv6ModuleInfo_p->adk_ipv6_dp_tables);
            adk_ipv6_filter_tbl_dump(ipv6Handle);
            break;

        default:
        ADK_FORCE_LOG_ERROR( "Tree group not supported \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        goto adk_ipv6_dump_tables_return;
    }

adk_ipv6_dump_tables_return:
    return retValue;

}

adk_status_t
adk_ipv6_lpm_add(adk_module_handle_t ipv6Handle,
        const char *table_name,
        adk_ipv6_dp_lpm_table_t *lpm_tbl,
        uint8_t *input_ptn,
        uint16_t size,
        adk_handle_t index,
        adk_handle_t *old_index)
{
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_LOG_INFO(sys_handle, ipv6Handle,"n");
    /*check if the entry was already present in the LPM table */ 
    if( adk_lpm_is_rule_present(lpm_tbl->lpm, 
            input_ptn,
            size,
            old_index) == TRUE)
    {
        ADK_LOG_WARNING(sys_handle, ipv6Handle,
                "Entry present, overwriting an existing entry %s \n", table_name);

    }

    /* update the lpm */ 
    if(adk_lpm_add(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_LPM_ADD_OBJ,
		lpm_tbl->lpm,  
                input_ptn, 
                size, 
                index)) 
    { 
        /* release the reserved index */ 
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "Error while adding LPM[Shadow] entry  %s n", table_name);
        return ADK_E_INTERNAL_ERROR; 
    }

    return ADK_SUCCESS;
}

adk_status_t
adk_ipv6_lpm_delete(adk_module_handle_t ipv6Handle,
        uint8_t *table_name,
        adk_ipv6_dp_lpm_table_t *lpm_tbl,
        uint8_t *input_ptn,
        uint16_t size)
{
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_LOG_INFO(sys_handle, ipv6Handle,"n"); 
    /* update the lpm */ 
    if(adk_lpm_delete(lpm_tbl->lpm,  
                input_ptn, 
                size)) 
    { 
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "Error while removing LPM[Shadow] entry  %s n", table_name);
        return ADK_DP_E_ERROR; 
    } 

    return ADK_SUCCESS;
}

void adk_ipv6_dp_write_flow_info(adk_ipv6_dest_flow_info_t src,
        adk_ipv6_dp_flow_info_t **dest)
{
    uint8_t idx = (*dest)->idx ^ 1;

    (*dest)->flow[idx].destFlow = src.destFlow;
    (*dest)->flow[idx].destModId = src.destModId;
    (*dest)->flow[idx].stageInfo = src.stageInfo;

    (*dest)->idx = idx;

    adk_rcu_synchronize();
    
    idx = (*dest)->idx ^ 1;

    (*dest)->flow[idx].destFlow = src.destFlow;
    (*dest)->flow[idx].destModId = src.destModId;
    (*dest)->flow[idx].stageInfo = src.stageInfo;
 
    return;
}

/*!
 * =============================================================
 *
 * adk_ipv6_log_level_set
 *
 * @param[in]  ipv6Handle   IPv6 Module Handle
 * @param[in]  log_level        Logical OR of different log types
 *
 * @retval ADK_SUCCESS          Successful completion
 * @retval ADK_E_INVALID_HANDLE Invalid IPv6 module handle parameter
 *
 * @brief  Function to set the log-level for IPv6 module
 *
 * This function is used to set the log-level for IPv6 module. This
 * completely overwrites the existing log-level setting.
 *
 * =============================================================
 */
adk_status_t adk_ipv6_log_level_set(adk_module_handle_t ipv6Handle,
                                  adk_log_level_t log_level)
{
    adk_status_t                 ret = ADK_SUCCESS;
    adk_sys_handle_t             sys_handle;
    adk_ipv6_info_t              *ipv6ModuleInfo_p = NULL;

    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n",
                           ipv6Handle);
        ADK_IPV6_LOG_FORCE("Failed to set IPv6 log-level 0x%x\n",
                           log_level);
        return(ADK_E_INVALID_HANDLE);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if (ipv6ModuleInfo_p == NULL)
    {
        ret = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, ret, "Module info is NULL\n");
        goto ipv6_log_level_release;
    }

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, ret);

    ipv6ModuleInfo_p->adk_ipv6_dp_tables->logLevel = log_level;


    ADK_IPV6_LOG_INFO(ipv6Handle,
                      "IPv6 module log-level set to %d\n", ipv6ModuleInfo_p->adk_ipv6_dp_tables->logLevel);

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, ret);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

ipv6_log_level_release:
    return(ret);
}

#ifdef ADK_HOS_OLD_IMPL

adk_status_t
adk_ipv6_tnl_intf_acl_config(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle, 
    bool enable_acl)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    ipv6_interface_info_t *if_pvt_info = NULL;
    adk_hos_inner_ip_tbl_rule_attr_t inner_ip_rule;
    uint32_t tunnel_if_id, vrd_id;

    /*! Check if module handle is valid */
    if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        ADK_FORCE_LOG_ERROR( "Invalid module handle\n");
        ret = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        return (ret);
    }
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    if (!adk_ipv6_interface_valid_handle(ipv6IfHandle))
    {
        ret = ADK_E_INVALID_HANDLE;
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                          "Invalid Interface handle %"ADKPRIh "\n",
                          ipv6IfHandle);
        goto _adk_ipv6_intf_acl_config_exit;
    }

    if_pvt_info = (ipv6_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(ipv6IfHandle);
    
    if(!if_pvt_info->ipv6InnerIpTblInfo.spd_valid)
    {
        ADK_LOG_WARNING(sys_handle, ipv6Handle,
                "inner IP table is not yet added, "
                "ruleset handle will be updated during innerIP table creation \n");
        goto _adk_ipv6_intf_acl_config_exit;
    }
    /* add an entry to inner IP table */
    tunnel_if_id = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    ADK_MEMSET(&inner_ip_rule, 0,
            sizeof(adk_hos_inner_ip_tbl_rule_attr_t));
    inner_ip_rule.is_ipv6 = TRUE;
    inner_ip_rule.proto = if_pvt_info->ipv6InnerIpTblInfo.next_header;
    ADK_MEMCPY((uint8_t *)(&inner_ip_rule.sip_addr[0]), 
            (uint8_t *) (&if_pvt_info->ipv6InnerIpTblInfo.src_addr[0]), 16);
    ADK_MEMCPY((uint8_t *)(&inner_ip_rule.dip_addr[0]), 
            (uint8_t *) (&if_pvt_info->ipv6InnerIpTblInfo.dst_addr[0]), 16);
    if(!(adk_ipv6_vrd_valid_handle(if_pvt_info->if_info_u.tunnelIf.outerIpv6VrdHandle,
                    &vrd_id)))
    {
        ADK_LOG_WARNING(sys_handle, ipv6Handle, 
                "invalid outer VRD handlle, unable to add inner IP rule \n");
        return ret;
    }

    inner_ip_rule.vrd_id = vrd_id;
    if(!(adk_ipv6_vrd_valid_handle(if_pvt_info->if_info_u.tunnelIf.innerIpv6VrdHandle,
                    &vrd_id)))
    {
        ADK_LOG_WARNING(sys_handle, ipv6Handle, 
                "invalid inner VRD handlle, unable to add inner IP rule \n");
        return ADK_E_INVALID_PARAMETER;
    }
    inner_ip_rule.inner_vrd_id = vrd_id;
    inner_ip_rule.inner_ip_id  = tunnel_if_id;

    if(enable_acl && if_pvt_info->if_info_u.tunnelIf.enableIngAcl)
    {
        inner_ip_rule.acl_enabled  = TRUE;
        inner_ip_rule.ip_acl_id=((ipv6_filter_rule_set_info_t *)if_pvt_info->ingressRuleSetHandle)->ipv6_filer_rule_set_id;
    }

    inner_ip_rule.spd_valid    = if_pvt_info->ipv6InnerIpTblInfo.spd_valid;
    inner_ip_rule.spd_id       = if_pvt_info->ipv6InnerIpTblInfo.spd_id; 

    ret = adk_hos_rule_update(ADK_HOS_INNER_IP_TBL, if_pvt_info->ipv6HosInnerIpTblRuleHandle,
            &inner_ip_rule);
    if(ret != ADK_SUCCESS)
    {
        ADK_LOG_WARNING(sys_handle, ipv6Handle, 
                "unable to update inner IP TBL rules,  error %x \n", ret); 
        return ADK_E_INTERNAL_ERROR;
    }
_adk_ipv6_intf_acl_config_exit:
    return (ret);
}

#endif
 /*!
 * end addtogroup _ipv6Functions  IPV6 Module
 * @}
 */
