/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipv6_qos.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6Functions  IPv6 Functions
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the APIs provided by the IPV6 module for QOS.
 *
 ***********************************************************************/
 
/* Infrastructure header files */
#include "adk_include.h"

/* IPV6 external header files */
#include "adk_ipv6.h"
#include "adk_ipv6_const.h"
#include "adk_cp_acl_list.h"

/* IPV6 internal header files */
#include "adk_ipv6_internal.h"
#include "adk_ipv6_qos_classifier.h"

/* ********************************** */
/* TODO: All stub code to be replace! */
/* ********************************** */

static
adk_status_t adk_ipv6_qos_classifer_validate_arg(
        adk_module_handle_t ipv6Handle,
        const adk_ipv6_qos_classifier_rule_t *ipv6EgressQoSClassifierRule_p,
        const adk_ipv6_qos_classifier_action_t *ipv6EgressQoSClassifierAction_p);

static 
adk_status_t
adk_ipv6_egress_qos_classifier_hw_rule (
    adk_module_handle_t ipv6Handle,
    uint32_t ifId,
    uint32_t precedence,
    uint32_t egressQoSClassifierFlowId,
    const adk_ipv6_qos_info_t *ipv6EgressQoSClassifierInfo_p,
    uint8_t treeIndex);

static 
adk_status_t adk_ipv6_qos_policy_build_internal(adk_module_handle_t ipv6Handle, adk_ipv6_egress_qos_classifier_ctx_t *ctx);

static adk_status_t
adk_ipv6_qos_policy_delete_all(
        adk_module_handle_t ipv6Handle,
        uint32_t            ifId);

/***************** RTE ACL DEFINITION *******************
 */
enum {
    IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD,
    IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD, /*use 32 bits space for DSCP lookup*/
    ADK_IPV6_EGRESS_QOS_CLASSIFIER_NUM_FIELDS
};

enum {
    IPV6_EGRESS_QOS_CLASSIFIER_PROTO_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC0_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC1_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC2_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_SRC3_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST0_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST1_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST2_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_DEST3_INPUT,
    IPV6_EGRESS_QOS_CLASSIFIER_DSCP_INPUT /*use 32 bits space for DSCP lookup*/
};


RTE_ACL_RULE_DEF(adk_ipv6_egress_qos_classifier_rule_rte, ADK_IPV6_EGRESS_QOS_CLASSIFIER_NUM_FIELDS);

static  struct  rte_acl_field_def adk_ipv6_egress_qos_classifier_def[ADK_IPV6_EGRESS_QOS_CLASSIFIER_NUM_FIELDS]  = {
    {
        .type = RTE_ACL_FIELD_TYPE_RANGE,
        .size = sizeof(uint8_t),
        .field_index  = IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD,
        .input_index  = IPV6_EGRESS_QOS_CLASSIFIER_PROTO_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, proto),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC0_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, sip0),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC1_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, sip1),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC2_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, sip2),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_SRC3_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, sip3),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST0_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, dip0),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST1_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, dip1),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST2_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, dip2),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_DEST3_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, dip3),
    },
    {
        .type = RTE_ACL_FIELD_TYPE_MASK,
        .size = sizeof(uint32_t),
        .field_index = IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD,
        .input_index = IPV6_EGRESS_QOS_CLASSIFIER_DSCP_INPUT,
        .offset = offsetof(adk_ipv6_egress_qos_classifier_ptn_t, dscp),
    },
};

/*!
 * =============================================================
 * @internal
 * adk_ipv6_egress_qos_classifier_hw_rule
 * @param[in,out]
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval
 *
 * @brief
 *
 * Internal support function for ACL rule addition.
 *
 * =============================================================
 */
static 
adk_status_t
adk_ipv6_egress_qos_classifier_hw_rule (
    adk_module_handle_t ipv6Handle,
    uint32_t ifId, 
    uint32_t precedence,
    uint32_t egressQoSClassifierFlowId,
    const adk_ipv6_qos_info_t *ipv6EgressQoSClassifierInfo_p,
    uint8_t treeIndex
    )
{
    adk_status_t retValue = ADK_SUCCESS;
    int ret = 0;
    adk_ipv6_info_t *ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    struct adk_ipv6_egress_qos_classifier_rule_rte rule = {0} ;
    adk_ipv6_dp_egress_qos_classifier_action_info_t *classifierResult_p;
    uint8_t prefixWidth = 0;

    /*****   Build the Individual Patterns ******/

    classifierResult_p = &(ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierActionDp_p[ifId][treeIndex][egressQoSClassifierFlowId]);
    classifierResult_p->queueId = ipv6EgressQoSClassifierInfo_p->qosAction.queueId;
    classifierResult_p->pktColor = ipv6EgressQoSClassifierInfo_p->qosAction.pktColor;
    ADK_IPV6_LOG_INFO(ipv6Handle,"\tclassifier - building rule: treeIndex=%d\n", treeIndex);

    /* Set protocol and corresponding port information */
    if (ipv6EgressQoSClassifierInfo_p->qosRule.ruleMask & ADK_QOS_CLASSIFIER_PROTOCOL_TYPE_MASK)
    {
    	ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tproto=%d\n", ipv6EgressQoSClassifierInfo_p->qosRule.protocol);
        /* Set RTE_ACL structure */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD].value.u8 =
            ipv6EgressQoSClassifierInfo_p->qosRule.protocol;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD].mask_range.u8  =
            ipv6EgressQoSClassifierInfo_p->qosRule.protocol;
    }
    else
    {
        /* Set ANY for protocol */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD].value.u8 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_PROTO_FIELD].mask_range.u8  = 0xFF;
    }
    /* Set Source address */
    if (ipv6EgressQoSClassifierInfo_p->qosRule.ruleMask & ADK_QOS_CLASSIFIER_SRC_ADDR_MASK)
    {
        if (ADK_IPV4_ADDRESS == ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.type) {
            ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tsip=0x%x/%d\n", ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v4Addr.addr,
                                                            ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v4Addr.prefixWidth);
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].value.u32 =
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v4Addr.addr;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].mask_range.u32  =
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v4Addr.prefixWidth;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].mask_range.u32 = 0xFFFFFFFF;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].mask_range.u32 = 0xFFFFFFFF;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].mask_range.u32 = 0xFFFFFFFF;
        }else {
            ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tsip=0x%x:0x%x:0x%x:0x%x /%d\n",
                                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[0],
                                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[1],
                                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[2],
                                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[3],
                                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.prefixWidth);
            prefixWidth = ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.prefixWidth;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].value.u32 =
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[0];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[1];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[2];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.srcAddr.addr_u.v6Addr.addr[3];
        
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].mask_range.u32 = (prefixWidth> 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
        }
    }
    else
    {
        /* Set ANY */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC0_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC1_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC2_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_SRC3_FIELD].mask_range.u32  = 0xFFFFFFFF;
    }

    /* Set Destination address */
    if (ipv6EgressQoSClassifierInfo_p->qosRule.ruleMask & ADK_QOS_CLASSIFIER_DEST_ADDR_MASK)
    {
        if (ADK_IPV4_ADDRESS == ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.type) {
            ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tdip=0x%x/%d\n",
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v4Addr.addr,
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v4Addr.prefixWidth);
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].value.u32 =
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v4Addr.addr;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].mask_range.u32  =
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v4Addr.prefixWidth;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].mask_range.u32 = 0xFFFFFFFF;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].mask_range.u32 = 0xFFFFFFFF;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].value.u32 = 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].mask_range.u32 = 0xFFFFFFFF;
        }else {
            ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tdip=0x%x:0x%x:0x%x:0x%x /%d\n",
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[0],
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[1],
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[2],
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[3],
                                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.prefixWidth);
            prefixWidth = ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.prefixWidth;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].value.u32 =
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[0];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[1];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[2];
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].value.u32 = 
                ipv6EgressQoSClassifierInfo_p->qosRule.destAddr.addr_u.v6Addr.addr[3];
        
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
            prefixWidth = (prefixWidth > 32) ? (prefixWidth - 32) : 0;
            rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].mask_range.u32 = (prefixWidth > 32) ? 32 : prefixWidth;
        }
    }
    else
    {
        /* Set ANY */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST0_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST1_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST2_FIELD].mask_range.u32  = 0xFFFFFFFF;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DEST3_FIELD].mask_range.u32  = 0xFFFFFFFF;
    }

    /* Set Interface handle */
    if (ipv6EgressQoSClassifierInfo_p->qosRule.ruleMask & ADK_QOS_CLASSIFIER_DSCP_MASK)
    {
    	ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tdscp=0x%x\n", ipv6EgressQoSClassifierInfo_p->qosRule.dscp);
        /* Store ID instead of handle */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD].value.u32 = adk_ntohl(ipv6EgressQoSClassifierInfo_p->qosRule.dscp);
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD].mask_range.u32  = 8;
    }
    else
    {
        /* Set ANY */
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD].value.u32 = 0;
        rule.field[IPV6_EGRESS_QOS_CLASSIFIER_DSCP_FIELD].mask_range.u32  = 0;
    }

    ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tqueueId=%d\n", ipv6EgressQoSClassifierInfo_p->qosAction.queueId);
    ADK_IPV6_LOG_INFO(ipv6Handle,"\t\tpktColor=%d\n", ipv6EgressQoSClassifierInfo_p->qosAction.pktColor);
    rule.data.category_mask = 1;
    rule.data.priority = precedence;
    /*Pack the action in userdata field itself*/
    rule.data.userdata = egressQoSClassifierFlowId;

    ret = rte_acl_add_rules(
            ((adk_ipv6_egress_qos_classifer_info_t*) ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId])->ipEgressQoSClassifierCtx[treeIndex],
            (struct rte_acl_rule *)&rule, 1);
    if (ret< 0)
    {
        retValue = ADK_E_INTERNAL_ERROR;
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Error inserting ACL rule in IP\n");
        return ADK_E_OTREE_INSERT_FAILED ;
    }

    /* */
    ADK_RETURN(retValue);

}/* adk_ip_egress_qos_classifier_hw_rule */

/*!
 * =============================================================
 * adk_ipv6_qos_policy_add
 * @param[in] ipv6Handle   - ADK IPv6 module handle.
 *
 * @param[in] ipv6IfHandle - ADK IPv6 interface handle.
 *
 * @param[in] ruleIndex    - rule index
 *
 * @param[in] qosInfo_p    - pointer to data structure
 *                           adk_ipv6_qos_info_t.
 *
 * @retval ADK_IPV6_SUCCESS Successfully added QoS policy.
 *
 * @retval ADK_E_INVALID_HANDLE invalid module or interface handle. 
 *
 * @retval ADK_E_INVALID_PARAMETER invalid parameter. 
 *
 * @retval ADK_IPV6_E_INVALID_ADDR_PREFIX invalid Address prefix.
 *
 * @brief
 * This API adds the egress QoS classifier rule on a given 
 * IPv6 interface.
 * =============================================================
 */
adk_status_t
adk_ipv6_qos_policy_add(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t ruleIndex,
        const adk_ipv6_qos_info_t *qosInfo_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    uint32_t ifId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);
    adk_sys_handle_t sys_handle;
    adk_acl_rule_t *cpRule_p;
    size_t classifierRuleId = ADK_RM_RSRC_INVALID;
    adk_flow_handle_t flowHandle = 0;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p = NULL;
    uint32_t l2IfId = 0; 

    if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        return(ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE));
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        goto _adk_ipv6_qos_policy_add_exit;
    }

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if (!adk_ipv6_interface_valid_handle(ipv6IfHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Invalid Interface Handle type\n");
        goto _adk_ipv6_qos_policy_add_exit;
    }

    if (NULL == qosInfo_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid input parameter %"ADKPRIh"\n", qosInfo_p);
        goto _adk_ipv6_qos_policy_add_exit;
    }

    l2IfId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    /* Validate input args */
    retValue = adk_ipv6_qos_classifer_validate_arg( ipv6Handle, &qosInfo_p->qosRule, &qosInfo_p->qosAction);

    if (ADK_SUCCESS != retValue)
        goto _adk_ipv6_qos_policy_add_exit;

    classifierRuleId = adk_rm_resource_reserve(
                               ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId]);
    if (ADK_RM_RSRC_INVALID == classifierRuleId)
    {
        goto _adk_ipv6_qos_policy_add_exit;
    }
    /* Clearing to indicate no handle is allocated yet */
    flowHandle = 0x0;

    /* Allocate flow handle */
    ADK_SYS_FLOW_HANDLE_ALLOC (ipv6Handle, ADK_IPV6_EGRESS_QOS_CLASSIFIER_INFO_OBJ,
        sizeof(adk_ipv6_flow_info_t),
        &flowHandle);
    if (flowHandle == (adk_flow_handle_t)NULL) 
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, "Failed to allocate memory "
            "for the IPV6 flow handle\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);

        goto _adk_ipv6_qos_policy_add_exit;
    }

    /************/
    if(!(ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
                ADK_SYS_FLOW_ATTR_PTR_GET(flowHandle)))
    {
        retValue=ADK_E_INTERNAL_ERROR;
        goto _adk_ipv6_qos_policy_add_exit;
    }

    ipv6FlowInfo_p->cookie = ADK_IPV6_FLOW_COOKIE;
    ipv6FlowInfo_p->type = ADK_IPV6_EGRESS_QOS_CLASSIFIER_FLOW;
    ADK_SYS_FLOWID_SET(flowHandle, classifierRuleId);

    ADK_MEMCPY((void *) &(ipv6FlowInfo_p->egressQoSClassifierFlow.flow_info),
               (const void *) qosInfo_p,
               sizeof(adk_ipv6_qos_info_t));
    /*
     * Add rule to ADK CP rule list.
     * The rules will be added during the next commit.
     */
    cpRule_p = &(ipv6FlowInfo_p->egressQoSClassifierFlow.cp_rule);
    cpRule_p->user_data = (void *)(flowHandle);   

    retValue = adk_acl_rule_add(
            ((adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId])->cpList,
            ruleIndex,
            cpRule_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Failed to add the rule in CP list\n");
        goto _adk_ipv6_qos_policy_add_exit;
    }

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);
    return ADK_SUCCESS;

_adk_ipv6_qos_policy_add_exit:
	/* Upon error, release the reserved resources */
	if (ADK_RM_RSRC_INVALID != classifierRuleId)
	{
		adk_rm_resource_free(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId], classifierRuleId);
	}

    if (retValue != ADK_SUCCESS) {
        if (flowHandle)
            ADK_SYS_FLOW_HANDLE_FREE(flowHandle);
    }

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);
}

/*!
 * =============================================================
 * adk_ipv6_qos_policy_delete
 * @param[in] ipv6Handle   - ADK IPv6 module handle.
 *
 * @param[in] ipv6IfHandle - ADK IPv6 interface handle.
 *
 * @param[in] ruleIndex    - rule index. 
 *
 * @retval ADK_IPV6_SUCCESS Successfully added QoS policy.
 *
 * @retval ADK_E_INVALID_HANDLE invalid module or interface handle. 
 *
 * @retval ADK_E_INVALID_PARAMETER invalid parameter. 
 *
 * @brief
 * This API deletes the egress QoS classifier rule on a given 
 * IPv6 interface.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_qos_policy_delete (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t ruleIndex)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    adk_ipv6_egress_qos_classifer_info_t* egressQoSClassifierInfo_p = NULL;
    uint16_t index=0;
    adk_acl_rule_t cpRule;
    size_t classifierRuleId = ADK_RM_RSRC_INVALID;
    adk_flow_handle_t flowHandle = 0;
    uint32_t ifId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    egressQoSClassifierInfo_p =  ((adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId]);

    adk_acl_get_reset(egressQoSClassifierInfo_p->cpList);
    index=0;
    while(ADK_SUCCESS == adk_acl_next_rule_get(
    		egressQoSClassifierInfo_p->cpList, &cpRule))
    {
    	if(index==ruleIndex)
    	{
    		classifierRuleId = ADK_SYS_FLOWID_GET(cpRule.user_data);
    		flowHandle = (adk_flow_handle_t)cpRule.user_data;
    		break;
    	}
    	index++;
    }

    if (ADK_RM_RSRC_INVALID != classifierRuleId)
    {
    	adk_rm_resource_free(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId], classifierRuleId);
    }

    if (flowHandle)
    	ADK_SYS_FLOW_HANDLE_FREE(flowHandle);

    retValue = adk_acl_rule_delete(egressQoSClassifierInfo_p->cpList,
                                           ruleIndex);
    if (retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Failed to delete the rule from CP list\n");
        goto _adk_ipv6_qos_policy_delete_exit;
    }
_adk_ipv6_qos_policy_delete_exit:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);
    return ADK_SUCCESS;
}

/*!
 *=============================================================
 * adk_ipv6_ingress_qos_policy_add 
 *
 * @param[in] ipv6Handle   - ADK IPv6 module handle.
 *
 * @param[in] ipv6IfHandle - ADK IPv6 interface handle.
 *
 * @param[in] qosInfo_p    - pointer to data structure
 *                           adk_ipv6_ingress_qos_info_t.
 *
 * @retval ADK_IPV6_SUCCESS Successfully added QoS policy.
 *
 * @retval ADK_E_INVALID_HANDLE - invalid module handle or
 * interface handle.
 *
 * @retval ADK_E_INVALID_PARAMETER - invalid parameter.
 *
 * @brief
 *
 * This API adds the IP ingress Qos classifier policy rule.
 * =============================================================
 */
adk_status_t adk_ipv6_ingress_qos_policy_add (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        adk_ipv6_ingress_qos_info_t *qosInfo_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    uint32_t l2IfId;
    adk_ipv6_dp_interface_entry_t *dp_entry;
    adk_sys_handle_t sys_handle;

    if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        return(ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE));
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        goto _adk_ipv6_ingress_qos_policy_add_exit;
    }

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if (!adk_ipv6_interface_valid_handle(ipv6IfHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Invalid Interface Handle type\n");
        goto _adk_ipv6_ingress_qos_policy_add_exit;
    }

    if (NULL == qosInfo_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid input parameter %"ADKPRIh"\n", qosInfo_p);
        goto _adk_ipv6_ingress_qos_policy_add_exit;
    }

    l2IfId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    /*Update dp table*/
    dp_entry = &(ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[l2IfId]);
    dp_entry->dscpToPhbClass[qosInfo_p->dscp] = qosInfo_p->phbClass;
    ADK_IPV6_LOG_INFO(ipv6Handle,"\t\t dscp:0x%x, phbClass:%u\n", qosInfo_p->dscp, qosInfo_p->phbClass);

_adk_ipv6_ingress_qos_policy_add_exit:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN(retValue);
}

/*!
 *=============================================================
 * adk_ipv6_ingress_qos_policy_delete 
 *
 * @param[in] ipv6Handle   - ADK IPv6 module handle.
 *
 * @param[in] ipv6IfHandle - ADK IPv6 interface handle.
 *
 * @param[in] qosInfo_p    - pointer to data structure
 *                           adk_ipv6_ingress_qos_info_t.
 *
 * @retval ADK_IPV6_SUCCESS Successfully added QoS policy.
 *
 * @retval ADK_E_INVALID_HANDLE - invalid module handle or
 * interface handle.
 *
 * @retval ADK_E_INVALID_PARAMETER - invalid module handle or
 * qosInfo_p.
 *
 * @brief
 *
 * This API deletes the IP ingress Qos classifier policy rule.
 * =============================================================
 */
adk_status_t adk_ipv6_ingress_qos_policy_delete (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        adk_ipv6_ingress_qos_info_t *qosInfo_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    uint32_t l2IfId;
    adk_ipv6_dp_interface_entry_t *dp_entry;
    adk_sys_handle_t sys_handle;

    if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        return(ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE));
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        goto _adk_ipv6_ingress_qos_policy_delete_exit;
    }

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if (!adk_ipv6_interface_valid_handle(ipv6IfHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Invalid Interface Handle type\n");
        goto _adk_ipv6_ingress_qos_policy_delete_exit;
    }

    if (NULL == qosInfo_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid input parameter %"ADKPRIh"\n", qosInfo_p);
        goto _adk_ipv6_ingress_qos_policy_delete_exit;
    }

    l2IfId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    /*Update dp table*/
    dp_entry = &(ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[l2IfId]);
    dp_entry->dscpToPhbClass[qosInfo_p->dscp] = ADK_QOS_DS_PHB_CLASS_NONE;

_adk_ipv6_ingress_qos_policy_delete_exit:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN(retValue);
}

/**
 *=============================================================
 * @internal 
 * adk_ipv6_qos_policy_build_internal 
 * @param[in]  ipv6Handle - IPv6 module handle.
 * @param[in]  ctx - IPv6 egress QoS classifer Context
 *
 * @retval ADK_SUCCESS Successfully built ingress classifer context.
 *
 * @retval ADK_E_OTREE_BUILD_FAILED - Build Failed
 *
 * @brief
 *
 * This function builds IPv6 QoS classifier ordered tree
 * =============================================================
 */
static 
adk_status_t adk_ipv6_qos_policy_build_internal(adk_module_handle_t ipv6Handle, adk_ipv6_egress_qos_classifier_ctx_t *ctx)
{
    struct rte_acl_config cfg;
    int ret=0;

    adk_sys_handle_t sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_MEMSET(&cfg, 0, sizeof(cfg));
    cfg.num_fields = ADK_IPV6_EGRESS_QOS_CLASSIFIER_NUM_FIELDS;
    ADK_MEMCPY(&cfg.defs, adk_ipv6_egress_qos_classifier_def, sizeof(adk_ipv6_egress_qos_classifier_def));
    cfg.num_categories = 1;
    cfg.max_size = 0;

    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_build", "building IPv6 Egress QoS Classifier context\n");

    ret = rte_acl_build(ctx, &cfg);
    if (ret != 0)
    {
    	ADK_IPV6_LOG_ERROR(ipv6Handle, ret, "rte_acl_build failed\n");
        return ADK_E_OTREE_BUILD_FAILED;
    }

    return ADK_SUCCESS;
} /* adk_ipv6_qos_policy_build_internal */

/**
 *=============================================================
 * adk_ipv6_qos_policy_build 
 *
 * @param[in]  ipv6Handle   - ADK IPv6 module handle.
 *
 * @param[in]  ipv6IfHandle - ADK IPv6 interface handle. 
 *
 * @retval ADK_SUCCESS Successfully built ingress classifer context.
 *
 * @retval ADK_E_HW_PROGRAMMING_ERROR - Failed to add rule to hardware. 
 *
 * @retval ADK_E_OTREE_BUILD_FAILED - Build Failed
 *
 * @brief
 *
 * This function builds IPv6 QoS classifier ordered tree.
 * =============================================================
 */
adk_status_t adk_ipv6_qos_policy_build(adk_module_handle_t ipv6Handle, adk_interface_handle_t ipv6IfHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_ipv6_egress_qos_classifer_info_t  *egressQoSClassifierInfo_p = NULL;
    adk_sys_handle_t sys_handle;
    uint8_t activeTreeId, shadowTreeId;
    uint16_t ruleIndex;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p;
    size_t classifierRuleId;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    adk_acl_rule_t cpRule;
    adk_ipv6_qos_info_t *qosInfo_p;
    uint32_t ifId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    egressQoSClassifierInfo_p = (adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId];
    /*
     * Clear the Ruleset & add the rules from the ADK CP rule list.
     * Then commit the ruleset.
     */
    if(egressQoSClassifierInfo_p->aclTreeIndex)
    {
        activeTreeId = 1;
        shadowTreeId = 0;
    }
    else
    {
        activeTreeId = 0;
        shadowTreeId = 1;
    }
    /* Only update Shadow tree. Active tree update is not really required
     * Next build will atomatically update active now which will be shadow then
     */
    rte_acl_reset_rules(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[shadowTreeId]);
    /* Update Active copy. Get the rules from CP list & add them to S/W ACL*/
    adk_acl_get_reset(egressQoSClassifierInfo_p->cpList);
    ruleIndex = 0;

    while(ADK_SUCCESS == adk_acl_next_rule_get(
                egressQoSClassifierInfo_p->cpList, &cpRule))
    {
        ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
            ADK_SYS_FLOW_ATTR_PTR_GET(cpRule.user_data);
        qosInfo_p = &(ipv6FlowInfo_p->egressQoSClassifierFlow.flow_info);
        classifierRuleId = ADK_SYS_FLOWID_GET(cpRule.user_data);
        if( ADK_SUCCESS != adk_ipv6_egress_qos_classifier_hw_rule(
                ipv6Handle,
                ifId,
                (ADK_IPV6_EGRESS_QOS_CLASSIFIER_RULE_MAX_PRIORITY - ruleIndex),
                classifierRuleId,
                qosInfo_p,
                shadowTreeId))
        {
            retValue = ADK_E_HW_PROGRAMMING_ERROR;
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Failed to add the rule in hardware\n");
            goto release_lock_and_return;
        }
        ruleIndex++;
    }

    /* Add Default rule at the end */
    /* Set Rule mask as zero, indicating no mask */
    adk_ipv6_qos_info_t ipQoSDefaultRule = {0};
    ipQoSDefaultRule.qosAction.queueId =  ADK_QOS_INVALID_QUEUE_ID;
    ipQoSDefaultRule.qosAction.pktColor = RTE_TM_GREEN;

    if(ADK_SUCCESS != adk_ipv6_egress_qos_classifier_hw_rule(
            ipv6Handle,
            ifId,
			(ADK_IPV6_EGRESS_QOS_CLASSIFIER_RULE_MAX_PRIORITY - ruleIndex),
            0, /* zero id reserved for default rule */
            &ipQoSDefaultRule,
            shadowTreeId
            ))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                           "Could not add default rule to IPv6 QOS CLASSIFIER!\n");
        goto release_lock_and_return;
    }

    /*Update shadow copy*/
    retValue = adk_ipv6_qos_policy_build_internal(ipv6Handle,
            egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[shadowTreeId]);
    if(ADK_SUCCESS !=  retValue)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Error: Could not build IP Ingress CLASSIFEER...\n");
        goto release_lock_and_return;
    }
    /* Toggle the index */
    egressQoSClassifierInfo_p->aclTreeIndex = shadowTreeId;
    /* RCU sync */
    adk_rcu_synchronize();
    /* CP ACL List update to re-arrange the List
     * To be called only if adk_cp_acl_list_hos_commit is not invoked*/
    retValue = adk_acl_sw_only_commit(egressQoSClassifierInfo_p->cpList);
    if(retValue != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Error: Could not re-arrange CP ACL List...\n");
        retValue = ADK_E_INTERNAL_ERROR;
        goto release_lock_and_return;
    }

release_lock_and_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);
    return retValue;

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);
    ADK_RETURN(retValue);
} /* adk_ip_qos_policy_build*/

/**
 *=============================================================
 * @internal
 * adk_ipv6_egress_qos_classifier_create
 * @param[in]  ipv6Handle - IPv6 module handle.
 *
 * @param[out] egressQoSClassifierInfo_p - Address of pointer to context which is 
               populated and returned by this API
 *
 * @retval ADK_SUCCESS Successfully created ingress classifer context.
 *
 * @retval ADK_E_MEM_UNAVAILABLE Failed to create as memory not available
 * @brief
 *
 * This function creates egress QoS classifier ordered tree
 * This is called at initialization time where validation of params
 * is already done.
 * =============================================================
 */
adk_status_t adk_ipv6_egress_qos_classifier_create(
        adk_module_handle_t ipv6Handle,
        uint32_t ifId,
        adk_ipv6_egress_qos_classifer_info_t  **ipv6EgressQoSClassifierInfo_p)
{
    struct rte_acl_param prm;
    uint8_t   i;
    uint16_t  actionIndexDp = 0xffff;
    char name[50];
    adk_status_t    retValue;
    adk_sys_handle_t sys_handle;
    adk_acl_info_t cpListInfo={0};
    adk_ipv6_qos_info_t qosInfo;
    adk_ipv6_egress_qos_classifer_info_t *egressQoSClassifierInfo_p = NULL;

    adk_ipv6_info_t * ipv6ModuleInfo_p =
             (adk_ipv6_info_t *) ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    /*Allocate egress QoS classifier structure*/
    egressQoSClassifierInfo_p = (adk_ipv6_egress_qos_classifer_info_t*)
        ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_EGRESS_QOS_CLASSIFIER_INFO_OBJ, 1,
			 sizeof(adk_ipv6_egress_qos_classifer_info_t));

    if (egressQoSClassifierInfo_p == NULL)
    {
        ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_MEM_UNAVAILABLE,
                           "Memory allocation failed\n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    *ipv6EgressQoSClassifierInfo_p = egressQoSClassifierInfo_p;

    prm.socket_id = SOCKET_ID_ANY;
    prm.rule_size = RTE_ACL_RULE_SZ(ADK_IPV6_EGRESS_QOS_CLASSIFIER_NUM_FIELDS);
    prm.max_rule_num = ADK_IPV6_EGRESS_QOS_CLASSIFIER_MAX_RULES + 1;

    /*Reserve 0th id for the default rule*/ 
    if (!adk_rm_resource_reserve_fixed(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId], 0)) {
        ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_RESOURCE_UNAVAILABLE,
                "Could not get resource id for egress QoS ORDERED CLASSIFIER using rte_acl_create!\n");
        return ADK_E_RESOURCE_UNAVAILABLE;
    }

    /*Default rule is added at Index 0 */
    actionIndexDp = 0;

    for (i=0; i<2; i ++)
    {
        /* Initialize */
        egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i] = NULL;
      
        snprintf(name, sizeof(name), "ADK_IPV6_EGRESS_QOS_CLASSIFIER_%u_%1d", ifId, i);
        prm.name = name;
        ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_create",
                    "creating new Egress QOS Ordered classifier context: %s\n", prm.name);

        /*Allocate a new ACL context*/
        egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i] =
            (adk_ipv6_egress_qos_classifier_ctx_t*) rte_acl_create(&prm);
        if (NULL == egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i])
        {
            if (1 == i)
            {
                ADK_IPV6_LOG_FATAL(ipv6Handle, ADK_E_MEM_UNAVAILABLE,
                        "Could not create EGRESS QOS ORDERED CLASSIFIER using rte_acl_create!\n");
                /* free previous one */
                rte_acl_free(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[0]);
            }
            return ADK_E_MEM_UNAVAILABLE;
        }
    }

    egressQoSClassifierInfo_p->aclTreeIndex = ADK_IPV6_QOS_ACTIVE_TREE_INDEX_INIT;
    cpListInfo.user = ADK_ACL_IPV6_EGRESS_QOS_CLASSIFIER;

    /*Create a control plane list for this egress*/
    if(ADK_SUCCESS != adk_acl_create(&(egressQoSClassifierInfo_p->cpList),
                                                &cpListInfo))
    {
        retValue = ADK_E_MEM_UNAVAILABLE;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                           "Could not allocate CP list for IP EGRESS QOS CLASSIFIER!\n");
        goto free_rte_acl_ctx;
    }

    /* 
     * Set first default Rule as ANY 
     */

    memset(&qosInfo, 0x0, sizeof(adk_ipv6_qos_info_t));
    qosInfo.qosRule.ruleMask = 0;
    qosInfo.qosAction.queueId = ADK_QOS_INVALID_QUEUE_ID;
    qosInfo.qosAction.pktColor = RTE_TM_GREEN;

    if(ADK_SUCCESS != adk_ipv6_egress_qos_classifier_hw_rule(
            ipv6Handle,
            ifId,
            ADK_IPV6_EGRESS_QOS_CLASSIFIER_RULE_MAX_PRIORITY,
            actionIndexDp,
            &qosInfo,
			ADK_IPV6_QOS_ACTIVE_TREE_INDEX_INIT
            ))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                           "Could not add default DENY rule to IP ING CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }
    /* Build */
    if( ADK_SUCCESS != adk_ipv6_qos_policy_build_internal(ipv6Handle,
                egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[ADK_IPV6_QOS_ACTIVE_TREE_INDEX_INIT]))
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                           "Could not BUILD default rule to IP QOS EGRESS CLASSIFIER active tree!\n");
        goto free_rte_acl_ctx;
    }

    if(ADK_SUCCESS != adk_ipv6_egress_qos_classifier_hw_rule(
    		ipv6Handle,
            ifId,
			ADK_IPV6_EGRESS_QOS_CLASSIFIER_RULE_MAX_PRIORITY,
			actionIndexDp,
			&qosInfo,
		    ADK_IPV6_QOS_SHADOW_TREE_INDEX_INIT	
    ))
    {
    	retValue = ADK_E_HW_PROGRAMMING_ERROR;
    	ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
    			"Could not add default DENY rule to IP ING CLASSIFIER shadow tree!\n");
    	goto free_rte_acl_ctx;
    }
    /* Build */
    if( ADK_SUCCESS != adk_ipv6_qos_policy_build_internal(ipv6Handle,
    		egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[ADK_IPV6_QOS_SHADOW_TREE_INDEX_INIT]))
    {
    	retValue = ADK_E_HW_PROGRAMMING_ERROR;
    	ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
    			"Could not BUILD default rule to IP QOS EGRESS CLASSIFIER shadow tree!\n");
    	goto free_rte_acl_ctx;
    }
    return ADK_SUCCESS;

free_rte_acl_ctx:
    adk_rm_resource_free(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId],
            actionIndexDp);
    rte_acl_free(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[0]);
    rte_acl_free(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[1]);
    return retValue;
} /* adk_ipv6_egress_qos_classifier_create*/

/**
 *=============================================================
 * adk_ipv6_egress_qos_classifier_delete 
 * @param[in]  ipv6Handle - IPv6 module handle.
 *
 * @param[out] ctx - pointer to context to be deleted
 *
 * @retval ADK_SUCCESS Successfully deleted ingress classifer context.
 *
 * @brief
 *
 * This function deletes ingrees classifer ordered tree
 * =============================================================
 */
adk_status_t adk_ipv6_egress_qos_classifier_delete(
        adk_module_handle_t ipv6Handle,
        uint32_t ifId)
{
    uint8_t   i;
    adk_status_t    retValue;
    adk_sys_handle_t sys_handle;
    adk_ipv6_egress_qos_classifer_info_t *egressQoSClassifierInfo_p = NULL;

    adk_ipv6_info_t * ipv6ModuleInfo_p =
             (adk_ipv6_info_t *) ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    egressQoSClassifierInfo_p =  ((adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId]);

    if (egressQoSClassifierInfo_p) {

        /*Delete all the rules from the table and delete both ACL context*/

        retValue = adk_ipv6_qos_policy_delete_all(ipv6Handle, ifId);

        if (retValue != ADK_SUCCESS)
        {
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Failed to delete all rules from CP list \n");
            goto _adk_ipv6_egress_qos_classifier_delete_exit;
        }
        for (i=0; i<2; i ++)
        {
            /*Delete ACL context*/
            if (egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i]) {
                rte_acl_reset(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i]);
                rte_acl_free(egressQoSClassifierInfo_p->ipEgressQoSClassifierCtx[i]);
            }
        }

        /*Free egress QoS classifier structure*/
        ADK_NVM_FREE(egressQoSClassifierInfo_p);
    }

    return ADK_SUCCESS;

_adk_ipv6_egress_qos_classifier_delete_exit:
    return retValue;
} /* adk_ipv6_egress_qos_classifier_delete*/


/*!
 * =============================================================
 * adk_ipv6_qos_classifer_validate_arg
 * validates the input parameter for IPv6 QOS Classifier rule add 
 * =============================================================
 */
static
adk_status_t adk_ipv6_qos_classifer_validate_arg(
        adk_module_handle_t ipv6Handle,
        const adk_ipv6_qos_classifier_rule_t *ipv6EgressQoSClassifierRule_p,
        const adk_ipv6_qos_classifier_action_t *ipv6EgressQoSClassifierAction_p)
{
    adk_status_t    retValue;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if (!ipv6EgressQoSClassifierRule_p) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid NULL input parameter\n");
        goto return_with_error;
    }

    if (ipv6EgressQoSClassifierRule_p->ruleMask & ADK_QOS_CLASSIFIER_DEST_ADDR_MASK) {
        if ((ADK_IPV4_ADDRESS == ipv6EgressQoSClassifierRule_p->destAddr.type) &&
            (ipv6EgressQoSClassifierRule_p->destAddr.addr_u.v4Addr.prefixWidth > ADK_IPV4_MAX_PREFIX_WIDTH))
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid Destination IP address prefix width\n");
            goto return_with_error;
        } else {
            if (ipv6EgressQoSClassifierRule_p->destAddr.addr_u.v6Addr.prefixWidth > ADK_IPV6_MAX_PREFIX_WIDTH) {
                retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
                ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                        "Invalid Destination IP address prefix width\n");
                goto return_with_error;
            }
        }
    }

    if (ipv6EgressQoSClassifierRule_p->ruleMask & ADK_QOS_CLASSIFIER_SRC_ADDR_MASK) {
        if ((ADK_IPV4_ADDRESS == ipv6EgressQoSClassifierRule_p->srcAddr.type) &&
            (ipv6EgressQoSClassifierRule_p->srcAddr.addr_u.v4Addr.prefixWidth > ADK_IPV4_MAX_PREFIX_WIDTH))
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid Source IP address prefix width: %d\n", ipv6EgressQoSClassifierRule_p->srcAddr.addr_u.v4Addr.prefixWidth);
            goto return_with_error;
        } else {
            if (ipv6EgressQoSClassifierRule_p->srcAddr.addr_u.v6Addr.prefixWidth > ADK_IPV6_MAX_PREFIX_WIDTH) {
                retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_ADDR_PREFIX);
                ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                        "Invalid Destination IP address prefix width\n");
                goto return_with_error;
            }
        }
    }

    if ((ipv6EgressQoSClassifierRule_p->ruleMask & ADK_QOS_CLASSIFIER_DEST_ADDR_MASK) && 
        (ipv6EgressQoSClassifierRule_p->ruleMask & ADK_QOS_CLASSIFIER_DEST_ADDR_MASK) && 
        (ipv6EgressQoSClassifierRule_p->destAddr.type != ipv6EgressQoSClassifierRule_p->srcAddr.type)) {
            retValue = ADK_IPV6_E_INVALID_IP_ADDRESS;
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "IP address type mismatch : %u\n", ipv6EgressQoSClassifierRule_p->destAddr.type);
            goto return_with_error;
        }

    if (ipv6EgressQoSClassifierRule_p->dscp >= IPV6_NUM_DSCP_VALUES) {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid dscp value: %u\n", ipv6EgressQoSClassifierRule_p->dscp);
            goto return_with_error;
    }

    if (ADK_QOS_INVALID_QUEUE_ID == ipv6EgressQoSClassifierAction_p->queueId) {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Invalid queueId : %u\n", ipv6EgressQoSClassifierAction_p->queueId);
            goto return_with_error;
    }
    switch (ipv6EgressQoSClassifierAction_p->pktColor)
    {
    case RTE_TM_GREEN:
    case RTE_TM_YELLOW:
    case RTE_TM_RED:
         break;
    default:
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                "Invalid packet color \n");
        goto return_with_error;
    }
    return ADK_SUCCESS;
return_with_error:
    return retValue;
} /* adk_ip_qos_classifer_validate_arg*/

static adk_status_t
adk_ipv6_qos_policy_delete_all(
        adk_module_handle_t ipv6Handle,
        uint32_t            ifId)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    adk_ipv6_egress_qos_classifer_info_t* egressQoSClassifierInfo_p = NULL;
    uint16_t index=0;
    adk_acl_rule_t cpRule;
    size_t classifierRuleId = ADK_RM_RSRC_INVALID;
    adk_flow_handle_t flowHandle = 0;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    egressQoSClassifierInfo_p =  ((adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId]);

    adk_acl_get_reset(egressQoSClassifierInfo_p->cpList);

    index=0;

    while(ADK_SUCCESS == adk_acl_next_rule_get(
    		egressQoSClassifierInfo_p->cpList, &cpRule))
    {
        if (cpRule.user_data == NULL)
        {
            goto _adk_ipv6_qos_policy_delete_all_exit;
        }
        classifierRuleId = ADK_SYS_FLOWID_GET(cpRule.user_data);
        flowHandle = (adk_flow_handle_t)cpRule.user_data;
        if (ADK_RM_RSRC_INVALID != classifierRuleId)
        {
            adk_rm_resource_free(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId], classifierRuleId);
        }
        if (flowHandle)
            ADK_SYS_FLOW_HANDLE_FREE(flowHandle);

        retValue = adk_acl_rule_delete(egressQoSClassifierInfo_p->cpList, index);
        if (retValue != ADK_SUCCESS)
        {
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                    "Failed to delete the rule from CP list\n");
            goto _adk_ipv6_qos_policy_delete_all_exit;
        }
    	index++;
    }

    /*Delete the control plane list*/
    if(ADK_SUCCESS != adk_acl_free(egressQoSClassifierInfo_p->cpList))
    {
        retValue = ADK_E_MEM_UNAVAILABLE;
        ADK_IPV6_LOG_FATAL(ipv6Handle, retValue,
                           "Could not allocate CP list for IP EGRESS QOS CLASSIFIER!\n");
        goto _adk_ipv6_qos_policy_delete_all_exit;
    }

    /*Free 0th id for the default rule*/ 
    adk_rm_resource_free(ipv6ModuleInfo_p->ipv6EgressQoSClassifierRuleIDHandle[ifId], 0);

_adk_ipv6_qos_policy_delete_all_exit:
    ADK_IPV6_LOG_FUNC_EXIT(ipv6Handle);

    ADK_RETURN(retValue);
    return ADK_SUCCESS;
}

/*!
* ====================================================================
*  adk_ipv6_get_qos_policies_all
*
* @param[in]    ipv6Handle - IPV6 module handle.
*
* @param[out]   *numQosPolicies - Number of qos policies
*
* @param[out]   **ipv6QosPol_p - List of qos policies
*
* @retval ADK_SUCCESS Successful completion
*
* @retval ADK_E_INVALID_PARAMETER  Invalid input parameters
*
* @retval ADK_E_INVALID_HANDLE   Ipv6 Module is invalid.
*
* @brief
*
* This API gets the QOS policies.
*
* @details
*
* This API gets all the qos policies along with the count of them.
* The array index of a rule represents its index in the tree.
* Application is expected to free ipv6QosPol_p pointer once it is used.
* =====================================================================
*/
adk_status_t
adk_ipv6_get_qos_policies_all(
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        uint32_t *numQosPolicies,
        adk_ipv6_qos_info_t **ipv6QosPol_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_acl_rule_t cpRule;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p = NULL;
    adk_ipv6_qos_info_t *ipv6QosPol = NULL;
    adk_ipv6_egress_qos_classifer_info_t  *egressQoSClassifierInfo_p = NULL;
    uint32_t ifId, num_nodes;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    if (!adk_ipv6_interface_valid_handle(ipv6IfHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Invalid Interface Handle type\n");
        goto adk_ipv6_get_qos_policies_return;
    }
    if ((numQosPolicies == NULL) || (ipv6QosPol_p == NULL))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue, "Invalid input parameter %"ADKPRIh" %"ADKPRIh"\n",
                            numQosPolicies, ipv6QosPol_p);
        goto adk_ipv6_get_qos_policies_return;
    }
    ifId = ADK_SYS_INTERFACE_ID_GET(ipv6IfHandle);

    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    egressQoSClassifierInfo_p = (adk_ipv6_egress_qos_classifer_info_t*)ipv6ModuleInfo_p->adk_ipv6_dp_tables->egressQoSClassifierInfo_p[ifId];

    adk_acl_get_num_nodes(egressQoSClassifierInfo_p->cpList, &num_nodes);

    ipv6QosPol = (adk_ipv6_qos_info_t*)
        ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_EGRESS_QOS_CLASSIFIER_INFO_OBJ, num_nodes, sizeof(adk_ipv6_qos_info_t));

    adk_acl_get_reset(egressQoSClassifierInfo_p->cpList);

    for (uint32_t i=0; i<num_nodes; i++)
    {
        retValue = adk_acl_next_rule_get(egressQoSClassifierInfo_p->cpList, &cpRule);
        if (retValue != ADK_SUCCESS)
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_LIST_NOT_FOUND);
            ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                               "Qos Policy is not found in CP ACL list\n");
            goto adk_ipv6_get_qos_policies_return;
        }
        ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
            ADK_SYS_FLOW_ATTR_PTR_GET(cpRule.user_data);

        if ((ipv6FlowInfo_p != NULL) && (ipv6QosPol != NULL))
        {
            ADK_MEMCPY(&((ipv6QosPol+i)->qosRule), &ipv6FlowInfo_p->egressQoSClassifierFlow.flow_info.qosRule, sizeof(adk_ipv6_qos_classifier_rule_t));
            ADK_MEMCPY(&((ipv6QosPol+i)->qosAction), &ipv6FlowInfo_p->egressQoSClassifierFlow.flow_info.qosAction, sizeof(adk_ipv6_qos_classifier_action_t));
        }
    }
    *ipv6QosPol_p = ipv6QosPol;
    *numQosPolicies = num_nodes;

adk_ipv6_get_qos_policies_return:
ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

    ADK_RETURN(retValue);
}
/*! end addtogroup _ipv6
 * @}
 */
