/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ipv6_route.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6Functions  IPv6 Functions
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the APIs provided by the IPV6 module.
 *
 ***********************************************************************/
/* Infrastructure header files */
#include "adk_include.h"
#include "adk_ph_dev.h"
#include "acp_gen_interface.h"
#include "adk_utils_tree_update.h"
#include <netinet/in.h>
#include <arpa/inet.h>

#include "adk_shm.h"

/* IPV6 external header files */
#include "adk_ipv6.h"
#include "adk_ipv6_const.h"

/* IPV6 internal header files */
#include "adk_ipv6_internal.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_export.h"
#include "adk_ph_dev_dp.h"
#define ADK_CONVERT_TO_V6_VRD__HANDLE(__DATA) ((adk_ipv6_vrd_handle_t)__DATA)
#define ADK_CONVERT_TO_V6_FLOW__HANDLE(__DATA) ((adk_flow_handle_t)__DATA)

adk_status_t
adk_ipv6_internal_onlink_destination_add (
    adk_module_handle_t                 ipv6Handle,
    const adk_ipv6_onlink_destination_info_t  *onLinkDestinationInfo_p,
    uint32_t                        prefixLength,
    uint32_t                        vrd_id,
    adk_flow_handle_t                   **onLinkDestinationHandle_p,
    uint16_t                        *flowId);

adk_status_t
adk_ipv6_internal_onlink_destination_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t onLinkDestinationHandle, 
    adk_ipv6_addr_t     onLinkDestinationIpv6Addr,
    uint32_t        vrd_id);

static inline uint16_t adk_ipv6_route_find_cp_node(
        adk_ipv6_info_t *ipv6ModuleInfo_p,
        adk_acl_rule_t *route_p,
        uint8_t ipv6RouteTableId,
        const adk_ipv6_route_info_t *ipv6RouteInfo_p)
{
    uint16_t index = 0;
    bool found = false;
    adk_acl_rule_t route;

    if (ipv6ModuleInfo_p)
        adk_acl_get_reset(ipv6ModuleInfo_p->cp_list);

    while(ADK_SUCCESS == adk_acl_next_rule_get(
            ipv6ModuleInfo_p->cp_list, &route))
    {
        if( (ipv6RouteInfo_p->ipv6RouteRules.dstIpv6Addr.addr[0] == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteRules.dstIpv6Addr.addr[0]) &&
            (ipv6RouteInfo_p->ipv6RouteRules.dstIpv6Addr.addr[1] == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteRules.dstIpv6Addr.addr[1]) &&
            (ipv6RouteInfo_p->ipv6RouteRules.dstIpv6Addr.addr[2] == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteRules.dstIpv6Addr.addr[2]) &&
            (ipv6RouteInfo_p->ipv6RouteRules.dstIpv6Addr.addr[3] == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteRules.dstIpv6Addr.addr[3]) &&
            (ipv6RouteInfo_p->ipv6RouteRules.prefixLength == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteRules.prefixLength) &&
            (ipv6RouteTableId == ((adk_ipv6_route_all_t *)route.user_data)->ipv6RouteTableId)
        )
        {
            found = true;
            *route_p = route;
            break;
        }
        index++;
    }

    if(!found)
        index = 0xFFFF;

    return index;
}

static inline adk_status_t adk_ipv6_route_update_cp_list(
        bool addOperation,
        adk_module_handle_t ipv6Handle,
        adk_status_t dp_st,
        uint8_t  ipv6RouteTableId,
        const adk_ipv6_route_info_t *ipv6Route_p)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t  *ipv6ModuleInfo_p = NULL;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    adk_status_t ret = ADK_SUCCESS;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        ret = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto __adk_ipv6_route_update_cp_list_exit;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if(addOperation)
    { /* add or update existing route */
        if(dp_st == ADK_SUCCESS)
        { /* New route was added to DP successfully. So, add new node here */
            adk_acl_rule_t route;
            route.user_data = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_ROUTE_ENTRY_OBJ, 1,
                    sizeof(adk_ipv6_route_all_t));
            if(NULL == route.user_data)
            {
                ADK_IPV6_LOG_ERROR(ipv6Handle, ADK_E_MEM_UNAVAILABLE,
                        "Memory allocation failed\n");
                ret = ADK_E_MEM_UNAVAILABLE;
                goto __adk_ipv6_route_update_cp_list_exit;
            }

            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteRules, &ipv6Route_p->ipv6RouteRules, sizeof(adk_ipv6_route_rules_t));
            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteAttr, &ipv6Route_p->ipv6RouteAttr, sizeof(adk_ipv6_route_attr_t));
            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteTableId, &ipv6RouteTableId, sizeof(uint8_t));

            if(ADK_SUCCESS != adk_acl_rule_add(ipv6ModuleInfo_p->cp_list, 0, &route))
            {
                ADK_IPV6_LOG_ERROR(ipv6Handle, ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR),
                        "Error inserting ACL rule in IP route update\n");
                ret = ADK_E_OTREE_INSERT_FAILED;
                goto __adk_ipv6_route_update_cp_list_exit;
            }
            goto __adk_ipv6_route_update_cp_list_exit;
        }
        else if( ADK_E_ENTRY_OVERWRITTEN == dp_st)
        { /* Existing route was updated, now update CP list */
            adk_acl_rule_t route;
            uint16_t index = adk_ipv6_route_find_cp_node(ipv6ModuleInfo_p, &route, ipv6RouteTableId, ipv6Route_p);
            if(0xFFFF == index)
            {
                /* somethings' not right. Entry updated in DP but not found in CP list */
                ADK_IPV6_LOG_ERROR(ipv6Handle, ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR),
                        "Error in overwriting route rule in CP List, route entry present in DP is not present in CP list\n");
                ret = ADK_E_INTERNAL_ERROR;
                goto __adk_ipv6_route_update_cp_list_exit;
            }

            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteRules, &ipv6Route_p->ipv6RouteRules, sizeof(adk_ipv6_route_rules_t));
            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteAttr, &ipv6Route_p->ipv6RouteAttr, sizeof(adk_ipv6_route_attr_t));
            ADK_MEMCPY(&(*(adk_ipv6_route_all_t *)route.user_data).ipv6RouteTableId, &ipv6RouteTableId, sizeof(uint8_t));
            goto __adk_ipv6_route_update_cp_list_exit;
        }
    }
    else
    { /* delete route */
        adk_acl_rule_t route;
        uint16_t index = adk_ipv6_route_find_cp_node(ipv6ModuleInfo_p, &route, ipv6RouteTableId, ipv6Route_p);
        if(0xFFFF == index)
        {
            /* somethings' not right. Entry updated in DP but not found in CP list */
            ADK_IPV6_LOG_ERROR(ipv6Handle, ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR),
                    "Error in deleting route rule in CP List, route entry present in DP is not present in CP list\n");
            ret = ADK_E_INTERNAL_ERROR;
            goto __adk_ipv6_route_update_cp_list_exit;
        }

        if(ADK_SUCCESS != adk_acl_rule_delete(ipv6ModuleInfo_p->cp_list, index))
        {
            ADK_IPV6_LOG_ERROR(ipv6Handle, ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR),
                    "Failed to delete route entry from CP list\n");
            ret = ADK_E_INTERNAL_ERROR;
            goto __adk_ipv6_route_update_cp_list_exit;
        }
    }

__adk_ipv6_route_update_cp_list_exit:
    return ret;
}

/*!
 * =============================================================
 * adk_ipv6_nexthop_add
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in] nexthopAttr_p The nexthop attributes.
 * @param[out] nextHopHandle_p The next hop handle to be returned
 * to the caller.
 *
 * @retval ADK_SUCCESS Successfully added next hop.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Adds a next hop.
 *
 * This function adds a next hop entry into the
 * route_and_adv_lookup_attr_tree. It returns the next hop
 * handle of type adk_flow_handle_t. The application has to
 * maintain the required mapping of the next hop handle to
 * the next hop IPv6 address. There is no check done for
 * duplicate next hops as the IPv6 module does not have enough
 * information, hence it is the responsibility of the
 * application to make this check. The application (neighbor
 * discovery module) should use the next hop handle while configuring
 * route using adk_ipv6_add_route() *
 * =============================================================
 */
adk_status_t
adk_ipv6_next_hop_add (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_next_hop_attr_t *nextHopAttr_p,
    adk_flow_handle_t *nextHopHandle_p)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    adk_flow_handle_t ipv6AssociationHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p;
    adk_ipv6_next_hop_info_t *tempNextHopAttr_p;
    uint16_t flowId = 0;
    adk_ipv6_dp_nexthop_entry_t nexthop_entry;
    ipv6_interface_info_t *if_pvt_info;

    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_nexthop_add_null_handle_return;
    }

    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_nexthop_add_null_handle_return;
    }
 
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_LOG_FUNC_ENTRY(sys_handle, ipv6Handle);

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, 
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_nexthop_add_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if ((nextHopHandle_p == NULL) || (nextHopAttr_p == NULL))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_nexthop_add_return;
    }
   
   /*! Check if valid interface handle given as input */
    if ((!nextHopAttr_p->interfaceHandle) || 
       (!adk_ipv6_interface_valid_handle(nextHopAttr_p->interfaceHandle)))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid IPv6 interface specified\n");
        goto adk_ipv6_nexthop_add_return;
    }

    ADK_SYS_FLOW_HANDLE_ALLOC (ipv6Handle, ADK_IPV6_NH_ADD_OBJ,
        sizeof(adk_ipv6_flow_info_t),
        &ipv6AssociationHandle);
    if (ipv6AssociationHandle == (adk_flow_handle_t)NULL) 
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Failed to allocate memory "
            "for the nexthop handle\n");

        goto adk_ipv6_nexthop_add_return;
    }
    ADK_LOG_INFO(sys_handle, ipv6Handle, "Successfully allocated  memory "
            "for IPV6 flow handle: 0x%"ADKPRIh"\n",ipv6AssociationHandle);

    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
                        ADK_SYS_FLOW_ATTR_PTR_GET (ipv6AssociationHandle);
    ipv6FlowInfo_p->type = ADK_IPV6_NEXT_HOP_FLOW;

    tempNextHopAttr_p = &ipv6FlowInfo_p->next_hop_info;
    tempNextHopAttr_p->cookie = ADK_IPV6_NEXTHOP_COOKIE;

    /*reset the nexthop entry */
    ADK_MEMSET(&nexthop_entry, 0, sizeof(adk_ipv6_dp_nexthop_entry_t));
    
    nexthop_entry.interface_id = ADK_SYS_INTERFACE_ID_GET(nextHopAttr_p->interfaceHandle);
    ADK_MEMCPY(nexthop_entry.nextHopIpv6Addr.addr, nextHopAttr_p->ipv6Addr_p.addr, sizeof(adk_ipv6_addr_t));
    /*update the datapath table */
    retValue1 = adk_ipv6_dp_nexthop_entry_add(ipv6Handle,
                                &nexthop_entry,
                                &flowId);
    if(retValue1 != ADK_SUCCESS)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Failed to allocate memory "
            "for the nexthop handle\n");

        goto flow_id_alloc_failed;

    }
    adk_sys_flow_cache_update_version(sys_handle);
    if_pvt_info = (ipv6_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(nextHopAttr_p->interfaceHandle);

    if ( if_pvt_info->ifType == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        retValue1 = adk_ipv6_dp_tunnel_interface_entry_nexthop_set(ipv6Handle,
                                                       if_pvt_info,
                                                       flowId);
        if (retValue1 != ADK_SUCCESS)
        {
            ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                               "Failed to configure the nexthop to tunnel interface\n");
            goto flow_id_alloc_failed;
        }
    }


    ADK_SYS_FLOWID_SET(ipv6AssociationHandle, flowId);

    /*! Copy the required parameters from the attributes to flow */
    ADK_MEMCPY (
        &(tempNextHopAttr_p->nextHopAttr),
        nextHopAttr_p,
        sizeof (adk_ipv6_next_hop_attr_t));

    *nextHopHandle_p = ipv6AssociationHandle;

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);

    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);
    ADK_RETURN( retValue1 );

flow_id_alloc_failed:
    ADK_SYS_FLOW_HANDLE_FREE (ipv6AssociationHandle);

adk_ipv6_nexthop_add_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

adk_ipv6_nexthop_add_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );
}    /* -----  end of function adk_ipv6_nexthop_add  ----- */

/*!
 * =============================================================
 * adk_ipv6_next_hop_modify
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in] nexthopAttr_p The nexthop attributes.
 * @param[out] nextHopHandle The next hop handle to be
 * modified.
 *
 * @retval ADK_SUCCESS Successfully modified next hop.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Modifies the attributes of a next hop.
 *
 * This API modifies the l2 resolved flag attributes of an existing next hop.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_next_hop_modify (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_next_hop_attr_t *nextHopAttr_p,
    adk_flow_handle_t nextHopHandle)
{

    adk_status_t retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_next_hop_info_t *tempNextHopAttr_p;
    adk_ipv6_dp_nexthop_entry_t nexthop_entry;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p;
    uint32_t flowId = 0;
    
    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_next_hop_modify_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_next_hop_modify_null_handle_return;
    }
 
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_next_hop_modify_return;
    }

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if ( !adk_ipv6_nexthop_valid_handle (nextHopHandle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid flow handle\n");

        goto adk_ipv6_next_hop_modify_return;
    }
    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
                        ADK_SYS_FLOW_ATTR_PTR_GET (nextHopHandle);

    tempNextHopAttr_p = &ipv6FlowInfo_p->next_hop_info;

    if ((nextHopAttr_p == NULL) || (tempNextHopAttr_p == NULL))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_next_hop_modify_return;
    }
   
#if 0
    if ( nextHopAttr_p->weight >= ADK_IPV6_NEXTHOP_MAX_WEIGHT )
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, " Weight for Next Hop assined is "
            "greater than Maximum allowed \n");

        goto adk_ipv6_next_hop_modify_return;
    }
#endif

    flowId = ADK_SYS_FLOWID_GET (nextHopHandle);
   
    /*! Check if valid interface handle given as input */
    if ((!nextHopAttr_p->interfaceHandle) ||
       (!adk_ipv6_interface_valid_handle(nextHopAttr_p->interfaceHandle))) 
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid IPv6 interface specified\n");
        goto adk_ipv6_next_hop_modify_return;
    }

    ADK_MEMSET(&nexthop_entry, 0, sizeof(nexthop_entry));

    nexthop_entry.interface_id =  ADK_SYS_INTERFACE_ID_GET(nextHopAttr_p->interfaceHandle);
    ADK_MEMCPY(nexthop_entry.nextHopIpv6Addr.addr, nextHopAttr_p->ipv6Addr_p.addr, sizeof(adk_ipv6_addr_t));

    /* Modify the Flow */
    if(adk_ipv6_dp_nexthop_entry_modify(ipv6Handle,
                                &nexthop_entry,
                                flowId))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "unable to update fastpath table\n");
        goto adk_ipv6_next_hop_modify_return;
    } 
    adk_sys_flow_cache_update_version(sysHandle);
    /*! Copy the required parameters from the attributes to flow */
    ADK_MEMCPY (
        &(tempNextHopAttr_p->nextHopAttr),
        nextHopAttr_p,
        sizeof (adk_ipv6_next_hop_attr_t));

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);

    ADK_RETURN( retValue1 );

adk_ipv6_next_hop_modify_return:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
adk_ipv6_next_hop_modify_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );

}    /* -----  end of function adk_ipv6_next_hop_modify  ----- */

/*!
 * =============================================================
 * adk_ipv6_next_hop_delete
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[out] nextHopHandle The next hop handle to be
 * deleted.
 *
 * @retval ADK_SUCCESS Successfully deleted the next hop.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Deletes a next hop.
 *
 * This API deletes the an existing next hop.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_next_hop_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t nextHopHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p;
    adk_ipv6_next_hop_info_t *tempNextHopAttr_p;
    uint32_t flowId;
    ipv6_interface_info_t *if_pvt_info;

    if (!ipv6Handle)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_next_hop_delete_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue, "Invalid module handle \n");
       goto adk_ipv6_next_hop_delete_null_handle_return;
    }
 
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue,
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_next_hop_delete_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if ( !adk_ipv6_nexthop_valid_handle (nextHopHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue, "Invalid flow handle\n");

        goto adk_ipv6_next_hop_delete_return;
    }
    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
                        ADK_SYS_FLOW_ATTR_PTR_GET (nextHopHandle);

    tempNextHopAttr_p = &ipv6FlowInfo_p->next_hop_info;
    if (tempNextHopAttr_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue, "Invalid argument\n");

        goto adk_ipv6_next_hop_delete_return;
    }

    flowId = ADK_SYS_FLOWID_GET (nextHopHandle);

    if_pvt_info = (ipv6_interface_info_t *)
                  ADK_SYS_INTERFACE_ATTR_PTR_GET(tempNextHopAttr_p->nextHopAttr.interfaceHandle);

    if ( if_pvt_info->ifType == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        retValue = adk_ipv6_dp_tunnel_interface_entry_nexthop_set(ipv6Handle,
                                                       if_pvt_info,
                                                       0);
        if (retValue != ADK_SUCCESS)
        {
            ADK_IPV6_LOG_ERROR(ipv6Handle,retValue,
                               "Failed to remove the nexthop from tunnel interface\n");
            goto adk_ipv6_next_hop_delete_return;
        }
    }

    /*update the datapath table */
    retValue = adk_ipv6_dp_nexthop_entry_rem(ipv6Handle,
                                (uint16_t)flowId);
    if(retValue != ADK_SUCCESS)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue, "Failed to allocate memory "
            "for the nexthop handle\n");

        goto adk_ipv6_next_hop_delete_return;
    }
    adk_sys_flow_cache_update_version(sysHandle);
    ADK_LOG_INFO(sysHandle, ipv6Handle, "freeing  memory "
            "for IPV6 flow handle: 0x%"ADKPRIh"\n",nextHopHandle);
    ADK_SYS_FLOW_HANDLE_FREE (nextHopHandle);

adk_ipv6_next_hop_delete_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);
adk_ipv6_next_hop_delete_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_ipv6_next_hop_delete  ----- */

adk_status_t
adk_ipv6_internal_onlink_destination_add (
    adk_module_handle_t 		ipv6Handle,
    const adk_ipv6_onlink_destination_info_t  *onLinkDestinationInfo_p,
    uint32_t 			prefixLength,
    uint32_t                        vrd_id,
    adk_flow_handle_t 		        **onLinkDestinationHandle_p,
    uint16_t            *flowId)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    adk_dp_status_t dpRetVal = ADK_SUCCESS;
    adk_sys_handle_t sys_handle;
    adk_flow_handle_t ipv6AssociationHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_onlink_destination_internal_t *tempOnLinkDestinationInfo_p;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p = NULL;
    adk_ipv6_dp_onlink_dest_entry_t  onlink_dest_entry;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    
    /*! Allocate the memory for the IPV6 flow handle */
    ADK_SYS_FLOW_HANDLE_ALLOC (ipv6Handle, ADK_IPV6_DST_ADD_OBJ,
        sizeof(adk_ipv6_flow_info_t),
        &ipv6AssociationHandle);
    if (ipv6AssociationHandle == (adk_flow_handle_t)NULL) 
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Failed to allocate memory "
            "for the on link destination handle\n");

        goto adk_ipv6_onlink_destination_add_return;
    }
    ADK_LOG_INFO(sys_handle, ipv6Handle, "Successfully allocated  memory "
            "for IPV6 flow handle: 0x%"ADKPRIh"\n",ipv6AssociationHandle);
    
    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *)
        ADK_SYS_FLOW_ATTR_PTR_GET (ipv6AssociationHandle);

    ipv6FlowInfo_p->type = ADK_IPV6_ONLINK_FLOW;

    tempOnLinkDestinationInfo_p = &ipv6FlowInfo_p->onlink_info;
    tempOnLinkDestinationInfo_p->cookie = ADK_IPV6_ONLINK_COOKIE;

    ADK_MEMSET(&onlink_dest_entry, 0, sizeof(adk_ipv6_dp_onlink_dest_entry_t));

    onlink_dest_entry.interface_id = ADK_SYS_INTERFACE_ID_GET(
                onLinkDestinationInfo_p->onLinkDestinationAttr.interfaceHandle);
    onlink_dest_entry.vrd_id = vrd_id;
    ADK_MEMCPY(&onlink_dest_entry.ipv6_addr.addr,
                &onLinkDestinationInfo_p->onLinkDestinationIpv6Addr, 
                sizeof(adk_ipv6_addr_t));

    /*! add onlink dest entry */
    dpRetVal = adk_ipv6_dp_onlink_dest_entry_add(ipv6Handle, 
                                            (adk_ipv6_onlink_destination_info_t *) onLinkDestinationInfo_p, 
                                            vrd_id,
                                            &onlink_dest_entry,
                                            flowId,
                                            &tempOnLinkDestinationInfo_p->hash_bucket_index);
    if(dpRetVal != ADK_SUCCESS)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
         ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Failed to add the entry "
            "for the on link destination \n");

        goto adk_ipv6_onlink_destination_add_fail;
    }

    /*! Copy the required parameters from the attributes to flow */
    ADK_MEMCPY (
        &(tempOnLinkDestinationInfo_p->onlinkInfo),
        onLinkDestinationInfo_p,
        sizeof (adk_ipv6_onlink_destination_info_t));

    **onLinkDestinationHandle_p = (adk_flow_handle_t )ipv6AssociationHandle;

    if( prefixLength != 128)
    {
        ADK_SYS_FLOW_HANDLE_FREE (ipv6AssociationHandle);
    }
    
    ADK_RETURN( retValue1 );

adk_ipv6_onlink_destination_add_fail:
    ADK_SYS_FLOW_HANDLE_FREE (ipv6AssociationHandle);

adk_ipv6_onlink_destination_add_return:
    ADK_RETURN( retValue1 );

}


/*!
 * =============================================================
 * adk_ipv6_onlink_destination_add
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in] onLinkDestinationInfo_p This structure contains
 * the destination information.
 * @param[out] onLinkDestinationHandle_p The handle to the
 * destination to be returned to the caller.
 *
 * @retval ADK_SUCCESS On successful API call.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Adds a node on the attached link to the
 * onlink_destination_tree.
 *
 * This API adds an IPv6 node on the attached link to the
 * onlink_destination_tree. The on-link destination handle
 * returned should be used to bind with the layer-2 egress
 * association handle when the layer-2 address of the
 * destination is resolved. Note that this API does not take
 * the address resolution state as input as the destination
 * need not be added in advance.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_onlink_destination_add (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_onlink_destination_info_t *onLinkDestinationInfo_p,
    adk_flow_handle_t *onLinkDestinationHandle_p)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    uint32_t vrd_id = 0, prefixLength = 128;
    adk_sys_handle_t sys_handle;
    uint16_t flowId;

    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_onlink_destination_add_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_onlink_destination_add_null_handle_return;
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_LOG_FUNC_ENTRY(sys_handle, ipv6Handle);
 
    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_onlink_destination_add_return;
    }

    if ((onLinkDestinationInfo_p == NULL) ||
        (onLinkDestinationHandle_p == NULL))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_onlink_destination_add_return;
    }

    if ((!onLinkDestinationInfo_p->onLinkDestinationAttr.interfaceHandle) ||
        (!adk_ipv6_interface_valid_handle (onLinkDestinationInfo_p->onLinkDestinationAttr.interfaceHandle)))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid interface handle\n");

        goto adk_ipv6_onlink_destination_add_return;
    }

   /*Validate Routing Domain */
    if (!adk_ipv6_vrd_valid_handle(onLinkDestinationInfo_p->ipv6VrdHandle, &vrd_id))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,"Invalid IPv6 routing domain  handle");
        goto adk_ipv6_onlink_destination_add_return;
    }

    retValue1 = adk_ipv6_internal_onlink_destination_add(ipv6Handle,
            (adk_ipv6_onlink_destination_info_t *) onLinkDestinationInfo_p,
            prefixLength,
            vrd_id,
            &onLinkDestinationHandle_p,
            &flowId);
    if(retValue1 != ADK_SUCCESS)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "failed to add onlink entry \n");
        goto adk_ipv6_onlink_destination_add_return;
    }

    ADK_SYS_FLOWID_SET((*onLinkDestinationHandle_p), flowId);


adk_ipv6_onlink_destination_add_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

adk_ipv6_onlink_destination_add_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );
}    /* -----  end of function adk_ipv6_internal_onlink_destination_add  ----- */

/*!
 * =============================================================
 * adk_ipv6_onlink_destination_modify
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in] onLinkDestinationAttr_p This structure contains
 * the destination attributes.
 * @param[out] onLinkDestinationHandle The handle to the
 * destination to be modified.
 *
 * @retval ADK_SUCCESS On successful API call.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Modifies a node on the attached link to the
 * onlink_destination_tree.
 *
 * This API modifies an IPv6 node on the attached link to the
 * onlink_destination_tree.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_onlink_destination_modify (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_onlink_destination_attr_t *onLinkDestinationAttr_p,
    adk_flow_handle_t onLinkDestinationHandle)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_onlink_destination_internal_t *tempOnLinkDestinationInfo_p;
    adk_ipv6_dp_onlink_dest_entry_t onlink_dest_entry;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p;
    uint32_t vrd_id = 0;
    uint16_t flowId;

    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_onlink_destination_modify_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_onlink_destination_modify_null_handle_return;
    }
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
 

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_onlink_destination_modify_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);


    if (onLinkDestinationAttr_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "onLinkDestinationAttr_p is NULL \n");
        goto adk_ipv6_onlink_destination_modify_return;
    }


    if ( !adk_ipv6_onlink_valid_handle(onLinkDestinationHandle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid flow handle\n");

        goto adk_ipv6_onlink_destination_modify_return;
    }

    if((!onLinkDestinationAttr_p->interfaceHandle) || 
       (!adk_ipv6_interface_valid_handle (onLinkDestinationAttr_p->interfaceHandle)))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid interface handle\n");

        goto adk_ipv6_onlink_destination_modify_return;
    }

    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *) ADK_SYS_FLOW_ATTR_PTR_GET(onLinkDestinationHandle);
    tempOnLinkDestinationInfo_p = &ipv6FlowInfo_p->onlink_info;


    if (tempOnLinkDestinationInfo_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_onlink_destination_modify_return;
    }
   /*Validate Routing Domain */
    if (!adk_ipv6_vrd_valid_handle(tempOnLinkDestinationInfo_p->onlinkInfo.ipv6VrdHandle, &vrd_id))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,"Invalid IPv6 routing domain  handle");
        goto adk_ipv6_onlink_destination_modify_return;
    }

    ADK_MEMSET(&onlink_dest_entry, 0, sizeof(adk_ipv6_dp_onlink_dest_entry_t));

    onlink_dest_entry.interface_id = ADK_SYS_INTERFACE_ID_GET(
                onLinkDestinationAttr_p->interfaceHandle);
    onlink_dest_entry.vrd_id = vrd_id;
    ADK_MEMCPY(&onlink_dest_entry.ipv6_addr.addr,
                &tempOnLinkDestinationInfo_p->onlinkInfo.onLinkDestinationIpv6Addr, 
                sizeof(adk_ipv6_addr_t));

    flowId = ADK_SYS_FLOWID_GET(onLinkDestinationHandle);

    if(adk_ipv6_dp_onlink_dest_entry_modify(ipv6Handle,
                            vrd_id, 
                            &onlink_dest_entry,
                            flowId))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,"unable to update fastpath table\n");
        goto adk_ipv6_onlink_destination_modify_return;
    }

    /*! Copy the required parameters from the attributes to flow */
    ADK_MEMCPY (
            &tempOnLinkDestinationInfo_p->onlinkInfo.onLinkDestinationAttr,
            onLinkDestinationAttr_p,
            sizeof (adk_ipv6_onlink_destination_attr_t));

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);

    ADK_RETURN( retValue1 );

adk_ipv6_onlink_destination_modify_return:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
adk_ipv6_onlink_destination_modify_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );
}    /* -----  end of function adk_ipv6_onlink_destination_modify  ----- */

adk_status_t
adk_ipv6_internal_onlink_destination_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t onLinkDestinationHandle, 
    adk_ipv6_addr_t onLinkDestinationIpv6Addr, 
    uint32_t vrd_id)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_onlink_destination_internal_t *tempOnLinkDestinationInfo_p;
    adk_ipv6_flow_info_t *ipv6FlowInfo_p = NULL;
    adk_dp_status_t dpRetValue=ADK_SUCCESS;
    uint32_t flowId;
    
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ipv6FlowInfo_p = (adk_ipv6_flow_info_t *) ADK_SYS_FLOW_ATTR_PTR_GET(onLinkDestinationHandle);
    tempOnLinkDestinationInfo_p = &ipv6FlowInfo_p->onlink_info;
    flowId = ADK_SYS_FLOWID_GET(onLinkDestinationHandle);

    dpRetValue = adk_ipv6_dp_onlink_dest_entry_rem(ipv6Handle,
            vrd_id,
            (uint8_t *)&onLinkDestinationIpv6Addr.addr[0],
            (uint16_t) flowId,
            tempOnLinkDestinationInfo_p->hash_bucket_index);
    if(dpRetValue != ADK_SUCCESS)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "failed to rem onlink entry \n");
        goto adk_ipv6_onlink_destination_rem_return;
    }

adk_ipv6_onlink_destination_rem_return:
    ADK_SYS_FLOW_HANDLE_FREE(onLinkDestinationHandle);
    ADK_RETURN( retValue1);
}    /* -----  end of function adk_ipv6_internal_onlink_destination_delete  ----- */
#if 0
/*!
 * =============================================================
 * adk_ipv6_onlink_destination_delete
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[out] onLinkDestinationHandle The handle to the
 * destination to be deleted.
 *
 * @retval ADK_SUCCESS Successfully deleted the onlink destination.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Deletes a node on the attached link to the
 * onlink_destination_tree.
 *
 * This API deletes an IPv6 node on the attached link to the
 * onlink_destination_tree.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_onlink_destination_delete (
    adk_module_handle_t ipv6Handle,
    adk_flow_handle_t onLinkDestinationHandle)
{
    adk_status_t retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    adk_ipv6_onlink_destination_internal_t *tempOnLinkDestinationInfo_p;
    uint32_t vrd_id = 0;
    adk_ipv6_flow_info_t *ipv6flowinfo_p = NULL;

    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Module handle is NULL \n");
        goto adk_ipv6_onlink_destination_delete_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_onlink_destination_delete_null_handle_return;
    }
 
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, 
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_onlink_destination_delete_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);

    if ( !adk_ipv6_onlink_valid_handle(onLinkDestinationHandle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid flow handle\n");

        goto adk_ipv6_onlink_destination_delete_return;
    }
        
    ipv6flowinfo_p = (adk_ipv6_flow_info_t *)
        ADK_SYS_FLOW_ATTR_PTR_GET (onLinkDestinationHandle);

    tempOnLinkDestinationInfo_p = &ipv6flowinfo_p->onlink_info;
        
    if (tempOnLinkDestinationInfo_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_onlink_destination_delete_return;
    }

    /*Validate Routing Domain */
    if (!adk_ipv6_vrd_valid_handle(tempOnLinkDestinationInfo_p->onlinkInfo.ipv6VrdHandle, 
                &vrd_id))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,"Invalid IPv6 routing domain  handle");
        goto adk_ipv6_onlink_destination_delete_return;
    }
    
    retValue1 = adk_ipv6_internal_onlink_destination_delete(ipv6Handle,
            onLinkDestinationHandle,
            tempOnLinkDestinationInfo_p->onlinkInfo.onLinkDestinationIpv6Addr,
            vrd_id);
    if(retValue1 != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "error while deleting onlink entry \n");
        goto adk_ipv6_onlink_destination_delete_return;
    }

adk_ipv6_onlink_destination_delete_return:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
adk_ipv6_onlink_destination_delete_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );
}    /* -----  end of function adk_ipv6_onlink_destination_delete  ----- */
#endif

/*!
 * =============================================================
 * adk_ipv6_route_add
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in,out] routeInfo_p This structure contains the
 * information about the route to add.
 * @param[in] Route Table Id
 * @retval ADK_SUCCESS Successfully route added.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  This API adds a route to forward the PDUs.
 *
 * This API adds the route to the ingress and egress routing 
 * table.
 *
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_route_add (
        adk_module_handle_t          ipv6Handle,
        adk_ipv6_route_info_t        *routeInfo_p,
        uint8_t routeTableId)
{
    adk_status_t                    retValue1 = ADK_SUCCESS;
    adk_status_t                    dpRetValue = ADK_SUCCESS;
    adk_sys_handle_t                sys_handle;
    adk_ipv6_info_t                 *ipv6ModuleInfo_p = NULL;
    adk_ipv6_dp_route_entry_t       dpRouteEntry;
    adk_ipv6_addr_prefix_t          ipv6_addr_info;
    adk_interface_handle_t          outputIf;

    ADK_MEMSET(&dpRouteEntry, 0, sizeof(adk_ipv6_dp_route_entry_t));

    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_route_add_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
        goto adk_ipv6_route_add_null_handle_return;
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_LOG_FUNC_ENTRY(sys_handle, ipv6Handle);

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);

    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, 
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_route_add_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if (ipv6ModuleInfo_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue1, "Module info is NULL\n");
        goto adk_ipv6_route_add_return;
    }

    if (routeInfo_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid argument\n");

        goto adk_ipv6_route_add_return;
    }

    if (ipv6ModuleInfo_p->maxIpv6Routes >= ADK_IPV6_MAX_ROUTE_ENTRIES)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1," Max Route Entries limit reached:%d\n",
                ADK_IPV6_MAX_ROUTE_ENTRIES);
        goto adk_ipv6_route_add_return;
    }

    if (routeTableId > ADK_IPV6_MAX_ROUTING_TABLES)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1," Max Route Table Id's limit reached :%d\n",
                ADK_IPV6_MAX_ROUTING_TABLES);
        goto adk_ipv6_route_add_null_handle_return;
    }

    if(!routeInfo_p->ipv6RouteAttr.onLink)
    {
         if ( ! adk_ipv6_nexthop_valid_handle(routeInfo_p->ipv6RouteAttr.u_hdl.nextHopHandle))
         {
             retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
             ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid flow handle\n");

             goto adk_ipv6_route_add_return;
         }
    }

    if(routeInfo_p->ipv6RouteAttr.u_hdl.interfaceHandle)
    {
        outputIf = (routeInfo_p->ipv6RouteAttr.u_hdl.interfaceHandle);
    }
    else
    {
        outputIf = (routeInfo_p->ipv6RouteAttr.u_hdl.nextHopHandle);
    }

    if((routeInfo_p->ipv6RouteAttr.onLink == TRUE) &&
            (!adk_ipv6_interface_valid_handle(outputIf)))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid interface handle for onlink destination\n");

        goto adk_ipv6_route_add_return;

    }
  
    {
        dpRouteEntry.action = ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
        if (routeInfo_p->ipv6RouteAttr.onLink == TRUE)
        {
            dpRouteEntry.on_link = TRUE;
            dpRouteEntry.action = ADK_IPV6_ROUTE_LOOKUP_ACTION_ONLINK;
            dpRouteEntry.interface_id = ADK_SYS_INTERFACE_ID_GET(outputIf);
	    ADK_MEMCPY(dpRouteEntry.dstIpv6Addr.addr, routeInfo_p->ipv6RouteRules.dstIpv6Addr.addr, sizeof(adk_ipv6_addr_t));
        }
        else 
        {
	   dpRouteEntry.on_link = FALSE;
	   dpRouteEntry.action = ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
           dpRouteEntry.nexthop_id = ADK_SYS_FLOWID_GET (routeInfo_p->ipv6RouteAttr.u_hdl.nextHopHandle);
        }
	dpRouteEntry.routeMtu = routeInfo_p->ipv6RouteAttr.routeMTU;
    }

    ADK_MEMCPY(&ipv6_addr_info.Ipv6Addr.addr[0], 
            &routeInfo_p->ipv6RouteRules.dstIpv6Addr.addr[0], 16);
    ipv6_addr_info.prefixLength = routeInfo_p->ipv6RouteRules.prefixLength; 

    /*! Add this route to the route_lookup_tree. */
    dpRetValue = adk_ipv6_dp_route_entry_add(ipv6Handle, routeTableId/*vrd_id*/, ipv6_addr_info, 
                                            &dpRouteEntry, FALSE);
    if((dpRetValue != ADK_SUCCESS) && (dpRetValue != ADK_E_ENTRY_OVERWRITTEN))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR); 
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                "Failed to add route entry \n");
        goto adk_ipv6_route_add_return;
    }

    if (ADK_SUCCESS != adk_ipv6_route_update_cp_list(true, ipv6Handle, dpRetValue, routeTableId, routeInfo_p))
    {
        adk_ipv6_dp_route_entry_rem(ipv6Handle, routeTableId, ipv6_addr_info, FALSE);

        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                "Failed to add route entry \n");
        goto adk_ipv6_route_add_return;
    }

    ipv6ModuleInfo_p->maxIpv6Routes++;
    adk_sys_flow_cache_update_version(sys_handle);

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);
    ADK_RETURN( retValue1 );

adk_ipv6_route_add_return:

    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

adk_ipv6_route_add_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );
}    /* -----  end of function adk_ipv6_route_add  ----- */

/*!
 * =============================================================
 * adk_ipv6_route_delete
 * @param[in] ipv6Handle IPV6 module handle. Returned at the
 * time of IPV6 module's registration with the module manager.
 * @param[in] RouteRules This structure contains the
 * information about the route Attributes to be deleted
 * @param[in] Route table ID
 *
 * @retval ADK_SUCCESS Successfully route deleted.
 * @retval ADK_E_INCONSISTENT_STATE The IPV6 module info pointer
 * in the module handle is NULL.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL.
 * @retval ADK_E_MEM_UNAVAILABLE There is not enough memory
 * available.
 *
 * @brief  Deletes a route from the ingress and egress routing 
 * table.
 *
 * This API deletes the route from the routing table.
 *
 * =============================================================
 */
adk_status_t
adk_ipv6_route_delete (
    adk_module_handle_t ipv6Handle,
    const adk_ipv6_route_rules_t ipv6RouteRules,
    uint8_t routeTableId)
{
    adk_status_t 	 retValue1 = ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipv6_info_t  *ipv6ModuleInfo_p = NULL;
    adk_ipv6_addr_prefix_t ipv6_addr_info;
    adk_ipv6_route_info_t routeInfo_p;
    
    if (!ipv6Handle)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_FORCE("Invalid module handle %"ADKPRIh"\n", ipv6Handle);
        goto adk_ipv6_route_del_null_handle_return;
    }

    /*! Validate some input parameters */
    /* validate IPv6 Module Handle*/
    if(!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
       retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
       ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1, "Invalid module handle \n");
       goto adk_ipv6_route_del_null_handle_return;
    }
 
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 

    /*! Aquire module lock */
    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue1);
   
    /*Validate Ipv6 private data and cookie */
    if(!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                           "Module Info NULL or cookie mismatch \n");
        goto adk_ipv6_route_del_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if (ipv6ModuleInfo_p == NULL)
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue1, "Module info is NULL\n");
        goto adk_ipv6_route_del_return;
    }
    
    ADK_MEMCPY(&ipv6_addr_info.Ipv6Addr.addr[0], 
            &ipv6RouteRules.dstIpv6Addr.addr[0], 16);
    ipv6_addr_info.prefixLength = ipv6RouteRules.prefixLength;

    /* Copying IP to get API structure */
    ADK_MEMCPY(&routeInfo_p.ipv6RouteRules.dstIpv6Addr.addr[0],
            &ipv6RouteRules.dstIpv6Addr.addr[0], 16);
    routeInfo_p.ipv6RouteRules.prefixLength = ipv6RouteRules.prefixLength;

    retValue1 = adk_ipv6_dp_route_entry_rem(ipv6Handle,
                routeTableId/*vrd_id*/, ipv6_addr_info, FALSE);
    
    if(retValue1 != ADK_SUCCESS)
    {
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,"Error while removing route entry \n");
        goto adk_ipv6_route_del_return;
    }
    else if (ADK_SUCCESS != adk_ipv6_route_update_cp_list(false, ipv6Handle, retValue1, routeTableId, &routeInfo_p))
    {
        retValue1 = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_IPV6_LOG_ERROR(ipv6Handle,retValue1,
                "Failed to remove route entry from cp list\n");
        goto adk_ipv6_route_del_return;
    }
    ipv6ModuleInfo_p->maxIpv6Routes--;
    adk_sys_flow_cache_update_version(sysHandle);

adk_ipv6_route_del_return:
    ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue1);
adk_ipv6_route_del_null_handle_return:

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN( retValue1 );

}    /* -----  end of function adk_ipv6_route_delete  ----- */

/*!
* =============================================================================
*  adk_ipv6_get_routes_all
*
* @param[in]    ipv6Handle - IPV6 module handle.
*
* @param[out]   *numIpv6Routes - Number of routes added
*
* @param[out]   **ipv6Route_p - List of route entries
*
* @retval ADK_SUCCESS Successful completion
*
* @retval ADK_E_INVALID_PARAMETER  Invalid input parameters
*
* @retval ADK_E_INVALID_HANDLE   Ipv6 Module is invalid.
*
* @brief
*
* This API gets the route entries from routing table.
*
* @details
*
* This API gets all the route entries along with the count of them for a requested
* route table Id the array index of a rule represents its index in the tree.
* Application is expected to free *ipv6Route_p pointer once it is used.
* ================================================================================
*/
adk_status_t adk_ipv6_get_routes_all (
    adk_module_handle_t    ipv6Handle,
    uint8_t                ipv6RouteTblId,
    uint32_t               *numIpv6Routes,
    adk_ipv6_route_info_t  **ipv6Route_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_ipv6_info_t *ipv6ModuleInfo_p;
    adk_sys_handle_t sys_handle;
    adk_acl_rule_t cpRule;
    adk_ipv6_route_info_t *routeInfo = NULL;
    adk_ipv6_route_all_t *ipv6FlowInfo_p = NULL;
    uint32_t num_nodes = 0;

    /* Handle validations */
    if (!adk_ipv6_valid_handle(ipv6Handle))
    {
        ADK_IPV6_LOG_FORCE("Invalid IPv6 module handle %"ADKPRIh"\n", ipv6Handle);
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        return (retValue);
    }
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_IPV6_LOG_FUNC_ENTRY(ipv6Handle);

    ADK_ACQUIRE_MODULE_LOCK(ipv6Handle, retValue);
    if ((numIpv6Routes == NULL) || (ipv6Route_p == NULL))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue, "Invalid input parameter %"ADKPRIh" %"ADKPRIh"\n",
                            numIpv6Routes, ipv6Route_p);
        goto adk_ipv6_get_routes_return;
    }
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipv6Handle);
    if (!ipv6ModuleInfo_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                           "Invalid IPv6 module info\n");
        goto adk_ipv6_get_routes_return;
    }

    adk_acl_get_reset(ipv6ModuleInfo_p->cp_list);

    while(ADK_SUCCESS == adk_acl_next_rule_get(
             ipv6ModuleInfo_p->cp_list, &cpRule))
    {
        ipv6FlowInfo_p = (adk_ipv6_route_all_t *)cpRule.user_data;
        if ((ipv6FlowInfo_p != NULL) && (ipv6RouteTblId == ipv6FlowInfo_p->ipv6RouteTableId))
        {
            num_nodes++;
        }
    }
    if (0 == num_nodes)
        return ADK_SUCCESS;

    /* Allocate memory */
    routeInfo = (adk_ipv6_route_info_t *)
        ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_ROUTE_ENTRY_OBJ, num_nodes, sizeof(adk_ipv6_route_info_t));

    adk_acl_get_reset(ipv6ModuleInfo_p->cp_list);
    for (uint32_t i = 0; i < num_nodes; i++)
    {
        retValue = adk_acl_next_rule_get(ipv6ModuleInfo_p->cp_list,
                                 &cpRule);
       if (retValue != ADK_SUCCESS)
       {
           retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_LIST_NOT_FOUND);
           ADK_IPV6_LOG_ERROR(ipv6Handle, retValue,
                               "Route entry is not found in CP ACL list\n");
           goto adk_ipv6_get_routes_return;
       }
       ipv6FlowInfo_p = (adk_ipv6_route_all_t *)cpRule.user_data;

       if ((ipv6FlowInfo_p != NULL) && (routeInfo != NULL))
       {
           if (ipv6RouteTblId == ipv6FlowInfo_p->ipv6RouteTableId)
           {
               ADK_MEMCPY(&((routeInfo+i)->ipv6RouteRules), &ipv6FlowInfo_p->ipv6RouteRules, sizeof(adk_ipv6_route_rules_t));
               ADK_MEMCPY(&((routeInfo+i)->ipv6RouteAttr), &ipv6FlowInfo_p->ipv6RouteAttr, sizeof(adk_ipv6_route_attr_t));
           }
       }
    }
    *ipv6Route_p = routeInfo;
    *numIpv6Routes = num_nodes;

adk_ipv6_get_routes_return:
ADK_RELEASE_MODULE_LOCK(ipv6Handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_LOG_FUNC_EXIT(sys_handle, ipv6Handle);

    ADK_RETURN(retValue);
}/* End of function adk_ipv6_get_routes_all */

 /*!
 * end addtogroup _ipv6Functions  IPv6 Functions
 * @}
 */
