/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_ipv6_tables.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6Functions  IPv6 Module
 *
 * @{
 *
 * @brief   This file has all the low level functions which is used by 
 * control plane software to add/modify/remove the fast-path tables.
 *
 *  **********************************************************************/
#include <adk_include.h>
#include <adk_ipv6_tables.h>
#include <adk_ipv6_acl.h>
#include "adk_rcu.h"
#include "adk_rcu_distributed_util.h"

extern uint32_t ip_bit_mask[];

adk_dp_status_t 
adk_ipv6_dp_interface_entry_add(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t __attribute__((unused))dscp)
{
    uint8_t i;
    uint16_t idx, next_idx;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    
    /* reset the IPv6 if DB, retain the indexes */
    idx = ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id].idx;
    next_idx = ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id].next_idx;

    ADK_MEMSET((uint8_t *)&ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id], 0,
                sizeof(adk_ipv6_dp_interface_entry_t));   
  
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id].idx = idx;
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id].next_idx = next_idx;

    if((ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH) || 
            (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL))
    {
        /* reset interface in & out stats */
        for(i=0; i<ADK_MAX_NUM_CORES; i++)
        {
            ADK_MEMSET(&ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[i][ipv6IfInfo_p->stats_id], 0,
                    sizeof(adk_ipv6_dp_if_stats_entry_t));
        }
    }

    /* update IPv6 interface info */
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];

    if_tbl->if_type             = ipv6IfInfo_p->ifType;
    if(ipv6IfInfo_p->policingHandle)
        if_tbl->police_flow_id      = ADK_SYS_FLOWID_GET(ipv6IfInfo_p->policingHandle);

    if_tbl->ingress_stats_id    = ipv6IfInfo_p->stats_id;
    if_tbl->egress_stats_id     = ipv6IfInfo_p->EgrStatsId;
    if_tbl->sys_egress_stats_id = ipv6IfInfo_p->SysEgrStatsId;

    /*Initialize dscp to phb table*/
    for (i = 0; i < IPV6_MAX_DSCP_VALUE; i++) {
        if_tbl->dscpToPhbClass[i] = ADK_QOS_DS_PHB_CLASS_NONE;
    } 

    if(ipv6IfInfo_p->ingressRuleSetHandle)
        if_tbl->ingress_ruleset     = ADK_SYS_FLOWID_GET(ipv6IfInfo_p->ingressRuleSetHandle);

    if(ipv6IfInfo_p->egressRuleSetHandle)
        if_tbl->egress_ruleset      = ADK_SYS_FLOWID_GET(ipv6IfInfo_p->egressRuleSetHandle); 
 
    if(ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH)
    {
        if_tbl->interfaceAttr       = ipv6IfInfo_p->interfaceAttr; 
        if_tbl->vrd_id = ADK_IPV6_DEFAULT_VRD_ID;
        if_tbl->interface_id        = ADK_SYS_INTERFACE_ID_GET(
                ipv6IfInfo_p->if_info_u.regularIf.layer2InterfaceHandle); 
        if_tbl->mtu                 = ipv6IfInfo_p->if_info_u.regularIf.mtu;
        ADK_MEMCPY(&if_tbl->ipv6_addr_prefix, 
                        &ipv6IfInfo_p->if_info_u.regularIf.ipv6AddrPrefix, 
                        sizeof(adk_ipv6_addr_prefix_t));

        ADK_LOG_INFO(sys_handle, ipv6Handle,
                "added IPv6 intreface details at index %d\n",ipv6IfInfo_p->stats_id);
        ADK_LOG_INFO(sys_handle, ipv6Handle,
                "IPv6 intreface address added %08x %08x %08x %08x \n",if_tbl->ipv6_addr_prefix.Ipv6Addr.addr[0],
                if_tbl->ipv6_addr_prefix.Ipv6Addr.addr[1],
                if_tbl->ipv6_addr_prefix.Ipv6Addr.addr[2],
                if_tbl->ipv6_addr_prefix.Ipv6Addr.addr[3]);
    }
    else 
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle,"invalid interface type \n");
        return ADK_DP_E_ERROR;
    }
         
    return ADK_SUCCESS;
}


adk_dp_status_t 
adk_ipv6_dp_interface_entry_status_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                bool is_disabled)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if_tbl->is_disabled = is_disabled;

    return retValue;
}

void
adk_ipv6_dp_filter_ruleset_config(adk_ipv6_info_t *ipv6ModuleInfo_p,
                                  uint32_t        filterRuleSetId,
                                  adk_ipv6_dp_acl_tree_t  *ipv6_acl_tree_0,
                                  adk_ipv6_dp_acl_tree_t  *ipv6_acl_tree_1)
                                  
{
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[0][filterRuleSetId] 
        = ipv6_acl_tree_0;
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[1][filterRuleSetId] 
        = ipv6_acl_tree_1;
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_tree_index = !(ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_tree_index);
}

void 
adk_ipv6_dp_filter_ruleset_delete( adk_module_handle_t ipv6Handle,
                                   uint32_t        filterRuleSetId)
{
    adk_ipv6_info_t *ipv6ModuleInfo_p = NULL;
    uint8_t old_index, new_index;

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    
    old_index = ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_tree_index;
    new_index = !(old_index);

    /* Free the shadow copy */
    adk_ipv6_acl_ruleset_free(ipv6Handle,
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[new_index][filterRuleSetId]);
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[new_index][filterRuleSetId] = 0;
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_tree_index = new_index;

    /* Wait for readers to complete */
    adk_rcu_synchronize();

    /* Free the active copy */
    adk_ipv6_acl_ruleset_free(ipv6Handle,
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[old_index][filterRuleSetId]);
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[old_index][filterRuleSetId] = 0;
}

adk_dp_status_t 
adk_ipv6_dp_interface_entry_filter_config(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                bool enable)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH) || 
         (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL))
    {
        if ( direction == ADK_IPV6_FILTER_IN)
        {
            if_tbl->enable_ingress_acl  = enable;
        }
        else
        {
            if_tbl->enable_egress_acl  = enable;
        }

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}
adk_dp_status_t 
adk_ipv6_dp_interface_entry_filter_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                uint32_t filterRuleSetId)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH) ||
         (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL))
    {
        if ( direction == ADK_IPV6_FILTER_IN)
        {
            if_tbl->ingress_ruleset  = filterRuleSetId;
        }
        else
        {
            if_tbl->egress_ruleset  = filterRuleSetId;
        }

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}

adk_dp_status_t 
adk_ipv6_dp_interface_entry_filter_get(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                uint32_t *filterRuleSetId)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH) ||
         (ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL))
    {
        if ( direction == ADK_IPV6_FILTER_IN)
        {
            *filterRuleSetId = if_tbl->ingress_ruleset;
        }
        else
        {
            *filterRuleSetId = if_tbl->egress_ruleset;
        }
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}


adk_dp_status_t 
adk_ipv6_dp_interface_entry_mtu_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t mtu)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH)
    {
        if_tbl->mtu = mtu;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}

adk_dp_status_t 
adk_ipv6_dp_interface_entry_profile_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t profileId)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if(ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH)
    {
        if_tbl->dscp_profile_id = profileId;
        if_tbl->enable_remark = ipv6IfInfo_p->enableL3Remark;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}


adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_mtu_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t mtu, uint16_t pmtu)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        if_tbl->mtu = mtu;
        /*update the localFdbkFlag */
        if(mtu == ADK_MIN_IPV6_TUNNEL_MTU)
        {
            if_tbl->pmtu = pmtu;
        }
        else
        {
            if_tbl->pmtu = (1<<ADK_IPV6_INTERFACE_LOCAL_PTB_LOC)|pmtu;
        }

        /*Update the possibility of post Frag & corresponding version */
        if_tbl->la_flag_version++;
        if((mtu == ADK_MIN_IPV6_TUNNEL_MTU) || (pmtu == 0))
        {
            if_tbl->la_flag = TRUE;
        }
        else
        {
            if_tbl->la_flag = FALSE;
        }

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}



adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_nexthop_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t nexthop_id)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        if_tbl->nexthop_id = nexthop_id;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}

adk_dp_status_t 
adk_ipv6_dp_interface_entry_dscp_pcp_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t dscp,
                uint8_t pcp)
{
    adk_dp_status_t retValue = ADK_SUCCESS;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    /* update IPv6 interface info atomicaly*/
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[ipv6IfInfo_p->stats_id];
    if ( ipv6IfInfo_p->ifType == IPV6_INTERFACE_TYPE_ETH)
    {
        if_tbl->dscp_pcp[dscp] = pcp;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;

    }
    else
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "iftype not yet handled \n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    return retValue;
}

 

adk_dp_status_t 
adk_ipv6_dp_interface_entry_rem(adk_module_handle_t ipv6Handle, 
                uint8_t __attribute__((unused)) dscp,
                uint16_t if_id)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    uint8_t vrd_id;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
 
    /* disable the interface : do not reset other details 
     * Shall need to wait for RCU sync to reset other details. */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].is_disabled = TRUE;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Removing intreface details at index %d\n",if_id);
    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "IPv6 intreface address added %08x %08x %08x %08x \n",
     ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].ipv6_addr_prefix.Ipv6Addr.addr[0],
     ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].ipv6_addr_prefix.Ipv6Addr.addr[1],
     ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].ipv6_addr_prefix.Ipv6Addr.addr[2],
     ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].ipv6_addr_prefix.Ipv6Addr.addr[3]);

    /* Temporary Fix: wait for RCU sync time & return.
     * waiting for RCU synchronisation to pass the active traffic if any */
    adk_rcu_synchronize();

    vrd_id = ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].vrd_id;
    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_dp_pkt_capture_info_t *pkt_capture_info_p)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        /* if capture is already enabled, 
         * then update the flag first(capture stop) & update the context */
        if(if_tbl->ing_pkt_capture_info.is_enabled)
        {
            if_tbl->ing_pkt_capture_info.is_enabled = pkt_capture_info_p->is_enabled; 
            if_tbl->ing_pkt_capture_info.enqueue_status = pkt_capture_info_p->is_enabled; 
            if_tbl->ing_pkt_capture_info.context = pkt_capture_info_p->context;
            if_tbl->ing_pkt_capture_info.devId = pkt_capture_info_p->devId; 
            if_tbl->ing_pkt_capture_info.cbFnId = pkt_capture_info_p->cbFnId;
            if_tbl->ing_pkt_capture_info.queueId = pkt_capture_info_p->queueId;
        }
        /* if capture is not enabled,
         * then update the context & enable the capture flag */
        else
        {
            if_tbl->ing_pkt_capture_info.context = pkt_capture_info_p->context;
            if_tbl->ing_pkt_capture_info.enqueue_status = pkt_capture_info_p->is_enabled; 
            if_tbl->ing_pkt_capture_info.is_enabled = pkt_capture_info_p->is_enabled;
            if_tbl->ing_pkt_capture_info.devId = pkt_capture_info_p->devId; 
            if_tbl->ing_pkt_capture_info.cbFnId = pkt_capture_info_p->cbFnId;
            if_tbl->ing_pkt_capture_info.queueId = pkt_capture_info_p->queueId;
        }

        ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Updating ingress capture point for if idx %d context %d\n"
                ,if_id, pkt_capture_info_p->context);

        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_eg_pkt_capture_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_dp_pkt_capture_info_t *pkt_capture_info_p)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        /* if capture is already enabled, 
         * then update the flag first(capture stop) & update the context */
        if(if_tbl->eg_pkt_capture_info.is_enabled)
        {
            if_tbl->eg_pkt_capture_info.is_enabled = pkt_capture_info_p->is_enabled; 
            if_tbl->eg_pkt_capture_info.enqueue_status = pkt_capture_info_p->is_enabled; 
            if_tbl->eg_pkt_capture_info.context = pkt_capture_info_p->context;
            if_tbl->eg_pkt_capture_info.devId = pkt_capture_info_p->devId; 
            if_tbl->eg_pkt_capture_info.cbFnId = pkt_capture_info_p->cbFnId;
            if_tbl->eg_pkt_capture_info.queueId = pkt_capture_info_p->queueId;
        }
        /* if capture is not enabled,
         * then update the context & enable the capture flag */
        else
        {
            if_tbl->eg_pkt_capture_info.context = pkt_capture_info_p->context;
            if_tbl->eg_pkt_capture_info.enqueue_status = pkt_capture_info_p->is_enabled; 
            if_tbl->eg_pkt_capture_info.is_enabled = pkt_capture_info_p->is_enabled; 
            if_tbl->eg_pkt_capture_info.devId = pkt_capture_info_p->devId; 
            if_tbl->eg_pkt_capture_info.cbFnId = pkt_capture_info_p->cbFnId;
            if_tbl->eg_pkt_capture_info.queueId = pkt_capture_info_p->queueId;
        }

        ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Updating ingress capture point for if idx %d context %d\n"
                ,if_id, pkt_capture_info_p->context);

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;

        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}


adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_status_get(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_pktcap_status_t **ipv6_pktcap_status)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        (*ipv6_pktcap_status)->is_enabled = if_tbl->ing_pkt_capture_info.enqueue_status;
        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}

adk_dp_status_t
adk_ipv6_dp_eg_pkt_capture_status_get(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_pktcap_status_t **ipv6_pktcap_status)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        (*ipv6_pktcap_status)->is_enabled = if_tbl->eg_pkt_capture_info.enqueue_status;
        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}

adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_status_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        /* update the enqueue status */
        if_tbl->ing_pkt_capture_info.enqueue_status = TRUE;
        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}

adk_dp_status_t
adk_ipv6_dp_eg_pkt_capture_status_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id)
{
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];

    /*check if the if is active */
    if(if_tbl->is_disabled != TRUE)
    {
        /* update the enqueue status */
        if_tbl->eg_pkt_capture_info.enqueue_status = TRUE;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;
        return ADK_SUCCESS;
    }
    
    ADK_LOG_ERROR(sys_handle, ipv6Handle,
            "Invalid interface, interface is disabled \n");
    
    return ADK_E_INTERNAL_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_add(adk_module_handle_t __attribute__((unused))  ipv6Handle,
                uint16_t __attribute__((unused)) tunnel_if_id,
                uint16_t __attribute__((unused)) adk_if_id)
{



    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_tunnel_interface_entry_rem(adk_module_handle_t __attribute__((unused)) ipv6Handle,
                uint16_t __attribute__((unused)) tunnel_if_id)
{



    return ADK_SUCCESS;
}

void 
adk_ipv6_dp_route_key_generate(uint8_t *key, uint8_t vrd_id, uint8_t *addr)
{
    key[0] = vrd_id;
    ADK_MEMCPY(&key[1], &addr[0], sizeof(adk_ipv6_addr_t));
    
    return;
}


adk_dp_status_t 
adk_ipv6_dp_route_entry_add(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6_addr_info,
                            adk_ipv6_dp_route_entry_t *route_entry,
                            bool inner_ipterm_only)
{
    adk_status_t ret = ADK_SUCCESS;
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1], depth=0;
    uint32_t size;
    adk_ipv6_addr_prefix_t ipv6_addr_nw;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    adk_ipv6_dp_route_entry_t *ipv6_route_entry = NULL;
    adk_ipv6_dp_route_table_t *ipv6_route_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_route_table;
    adk_handle_t old_lpm_value = ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID);

    RTE_SET_USED(inner_ipterm_only);

    /* Clear pending List if distributed RCU is success */
   ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(ipv6_route_tbl->pending_list, 
                                            ipv6_route_tbl->list_count, 
                                            adk_rcu_shm_free_cb_fn, NULL);

    /* allocate route index*/
    ipv6_route_entry = (adk_ipv6_dp_route_entry_t *) 
                     ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
			 ADK_IPV6_ROUTE_ENTRY_OBJ,
			 1, sizeof(adk_ipv6_dp_route_entry_t), 0);

    if ( ipv6_route_entry == NULL)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "Memory allocation for route table failed \n");
        ret = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        return ret;
    }

    /* update the route action table */
    ADK_MEMCPY(ipv6_route_entry,
            route_entry, sizeof(adk_ipv6_dp_route_entry_t));

    ipv6_route_entry->is_enabled = TRUE;

    ipv6_addr_nw.Ipv6Addr.addr[0] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[0]);
    ipv6_addr_nw.Ipv6Addr.addr[1] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[1]);
    ipv6_addr_nw.Ipv6Addr.addr[2] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[2]);
    ipv6_addr_nw.Ipv6Addr.addr[3] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[3]);
   

    /* Key for route LPM lookup */
    adk_ipv6_dp_route_key_generate(input_ptn, 
                                        vrd_id, 
                                        (uint8_t *)&ipv6_addr_nw.Ipv6Addr.addr[0]);
    size = ADK_IPV6_VRD_ID_WIDTH + 2 + ipv6_addr_info.prefixLength; 

    /* add LPM entry - Macro updates both shadow & active copies */
    ret = adk_ipv6_lpm_add(ipv6Handle,
            ADK_IPV6_ROUTE_LOOKUP_TREE,
            &ipv6_route_tbl->lpm_tbl,
            input_ptn,
            size,
            ADK_CONVERT_TO_LPM_DATA(ipv6_route_entry),
            &old_lpm_value);
     if(ret != ADK_SUCCESS)
     {
         ADK_DP_SHM_FREE(ipv6_route_entry);
         return ret;
     }

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

     ADK_LOG_INFO(sys_handle, ipv6Handle,
                "added IPv6 route entry with VRD %d IPv6 adddress %08x %08x %08x %08x \n",vrd_id,
     ipv6_addr_info.Ipv6Addr.addr[0],
     ipv6_addr_info.Ipv6Addr.addr[1],
     ipv6_addr_info.Ipv6Addr.addr[2],
     ipv6_addr_info.Ipv6Addr.addr[3]);

     /* Add HOS rule to terminate the IPv6 frames*/
     if((ipv6_route_entry->action == ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE) ||
         (ipv6_route_entry->action == ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE))
     {
         /* Add entry for Plaintext or outer IP termination */
         adk_hos_ip_term_tbl_rule_attr_t hos_ip_term_rule;
         ADK_MEMSET(&hos_ip_term_rule, 0, sizeof(adk_hos_ip_term_tbl_rule_attr_t));
         hos_ip_term_rule.precedence = -1;
         hos_ip_term_rule.is_ipv6    = TRUE;
         hos_ip_term_rule.is_ip_term = TRUE;
         hos_ip_term_rule.pkt_type = ADK_HOS_L2_TERMINATING;
         hos_ip_term_rule.is_mcast = false;
         hos_ip_term_rule.is_bcast = false;
         hos_ip_term_rule.ip_addr_stats_id = ipv6_route_entry->ipv6_addr_stats_id;
         
         depth = ipv6_addr_info.prefixLength;
         hos_ip_term_rule.ip_addr_mask[0]=  adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
         depth = (depth > 32) ? (depth - 32) : 0;
         hos_ip_term_rule.ip_addr_mask[1]=  adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
         depth = (depth > 32) ? (depth - 32) : 0;
         hos_ip_term_rule.ip_addr_mask[2]=  adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);
         depth = (depth > 32) ? (depth - 32) : 0;
         hos_ip_term_rule.ip_addr_mask[3]=  adk_htonl(ip_bit_mask[(depth > 32) ?  32: depth]);

         hos_ip_term_rule.ip_addr[0] = ipv6_addr_nw.Ipv6Addr.addr[0];
         hos_ip_term_rule.ip_addr[1] = ipv6_addr_nw.Ipv6Addr.addr[1];
         hos_ip_term_rule.ip_addr[2] = ipv6_addr_nw.Ipv6Addr.addr[2];
         hos_ip_term_rule.ip_addr[3] = ipv6_addr_nw.Ipv6Addr.addr[3];

         if(!inner_ipterm_only)
         {
             ret = adk_hos_rule_add(ADK_HOS_IP_TERM_TBL,
                     &hos_ip_term_rule, 
                     &ipv6_route_entry->hos_ipterm_rulehdl);
             if (ret != ADK_SUCCESS)
             {
                 ADK_LOG_WARNING( sys_handle, ipv6Handle,
                         "Failed to add HOS IP TERM Table entry \n");
             }
         }

#ifdef ADK_HOS_OLD_IMPL
         /* Add entry for inner IP termination */
         ret = adk_hos_rule_add(ADK_HOS_INNER_IP_TERM_TBL,
                 &hos_ip_term_rule, 
                 &ipv6_route_entry->hos_inner_ipterm_rulehdl);
         if (ret != ADK_SUCCESS)
         {
             ADK_LOG_WARNING( sys_handle, ipv6Handle,
                     "Failed to add HOS INNER IP TERM Table entry \n");
         }
#endif
     }
     /* release the old index if overwritten */
     if(old_lpm_value != ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID))
     {
         ADK_LOG_INFO(sys_handle, ipv6Handle,
                 "entry was overwritten, releasing the"
                 "old resources %"ADKPRIh"\n", old_lpm_value);
         ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST(ADK_IPV6_MODULE_ID,
                                                  ADK_IPV6_DP_PEND_LIST_OBJ,
                                                  ipv6_route_tbl->pending_list,
                                                  old_lpm_value,
                                                  ipv6_route_tbl->list_count,
                                                  ADK_LPM_DEFAULT_PENDING_SIZE,
                                                  adk_rcu_shm_free_cb_fn, NULL);
         return ADK_E_ENTRY_OVERWRITTEN;
     }
     return ADK_SUCCESS;
}


adk_dp_status_t 
adk_ipv6_dp_route_entry_update_rule_mask(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t *ipv6AddrPrefix,
                            uint16_t rule_mask, bool inner_ipterm_only)
{
    adk_sys_handle_t sys_handle;
    adk_status_t ret;
    uint8_t ipv6_addr[16];
    uint32_t size;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1];
    adk_handle_t ipv6_route_entry;
    adk_ipv6_dp_route_table_t *ipv6_route_tbl = NULL;

    RTE_SET_USED(inner_ipterm_only);

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    ipv6_route_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_route_table;

    /* Clear pending List if distributed RCU is success */
   ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(ipv6_route_tbl->pending_list, 
                                            ipv6_route_tbl->list_count, 
                                            adk_rcu_shm_free_cb_fn, NULL);

    *((uint32_t *)&ipv6_addr[0]) = adk_htonl(ipv6AddrPrefix->Ipv6Addr.addr[0]);
    *((uint32_t *)&ipv6_addr[4]) = adk_htonl(ipv6AddrPrefix->Ipv6Addr.addr[1]);
    *((uint32_t *)&ipv6_addr[8]) = adk_htonl(ipv6AddrPrefix->Ipv6Addr.addr[2]);
    *((uint32_t *)&ipv6_addr[12]) = adk_htonl(ipv6AddrPrefix->Ipv6Addr.addr[3]);


    /* Key for route LPM lookup */
    adk_ipv6_dp_route_key_generate(input_ptn, 
            vrd_id, 
            ipv6_addr);

    size = ADK_IPV6_VRD_ID_WIDTH + 2 + ipv6AddrPrefix->prefixLength; 
    ADK_IPV6_CP_LPM_LOOKUP(ipv6_route_tbl->lpm_tbl, input_ptn, size, ipv6_route_entry); 

    if(!((adk_ipv6_dp_route_entry_t*)ipv6_route_entry)->is_enabled)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                 "error while updating ingress association mask" 
                 "route entry is not present \n");
        return ADK_E_INTERNAL_ERROR;   
    }

    /*atomically update ingress assocition rulemask */
    ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->ingress_assoc_rulemask = rule_mask;

     /* Add HOS rule to terminate the IPv6 frames*/
     if((((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->action == 
                 ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE) ||
         (((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->action == 
                 ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE))
     {
         adk_hos_ip_term_tbl_rule_attr_t hos_ip_term_rule;
         ADK_MEMSET(&hos_ip_term_rule, 0, sizeof(adk_hos_ip_term_tbl_rule_attr_t));
         hos_ip_term_rule.precedence = -1;
         hos_ip_term_rule.is_ipv6    = TRUE;
         hos_ip_term_rule.is_ip_term = TRUE;
         hos_ip_term_rule.pkt_type = ADK_HOS_L2_TERMINATING;
         hos_ip_term_rule.is_mcast = false;
         hos_ip_term_rule.is_bcast = false;
         hos_ip_term_rule.ip_addr_stats_id = ((adk_ipv6_dp_route_entry_t*)ipv6_route_entry)->ipv6_addr_stats_id;
         hos_ip_term_rule.ip_addr_mask[0]= adk_htonl(0xFFFFFFFF);
         hos_ip_term_rule.ip_addr_mask[1]= adk_htonl(0xFFFFFFFF);
         hos_ip_term_rule.ip_addr_mask[2]= adk_htonl(0xFFFFFFFF);
         hos_ip_term_rule.ip_addr_mask[3]= adk_htonl(0xFFFFFFFF);
         hos_ip_term_rule.ip_addr[0] = *((uint32_t *)&ipv6_addr[0]); 
         hos_ip_term_rule.ip_addr[1] = *((uint32_t *)&ipv6_addr[4]);
         hos_ip_term_rule.ip_addr[2] = *((uint32_t *)&ipv6_addr[8]);
         hos_ip_term_rule.ip_addr[3] = *((uint32_t *)&ipv6_addr[12]);

         if(!inner_ipterm_only)
         {
             ret = adk_hos_rule_update(ADK_HOS_IP_TERM_TBL,
                     ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->hos_ipterm_rulehdl,
                     &hos_ip_term_rule);
             if (ret != ADK_SUCCESS)
             {
                 ADK_LOG_WARNING( sys_handle, ipv6Handle,
                         "Failed to update HOS IP TERM Table entry \n");
             }
         }

#ifdef ADK_HOS_OLD_IMPL
         ret = adk_hos_rule_update(ADK_HOS_INNER_IP_TERM_TBL,
                 ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->hos_inner_ipterm_rulehdl,
                 &hos_ip_term_rule);
         if (ret != ADK_SUCCESS)
         {
             ADK_LOG_WARNING( sys_handle, ipv6Handle,
                     "Failed to update HOS INNER IP TERM Table entry \n");
         }
#endif
     }

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "updated ingress association mask successfully 0x%04x \n ", rule_mask);

    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_route_entry_rem(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6_addr_info,
                            bool inner_ipterm_only)
{
    adk_status_t ret = ADK_SUCCESS;
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1];
    uint32_t size;
    adk_handle_t ipv6_route_entry;
    adk_ipv6_addr_prefix_t ipv6_addr_nw;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_route_table_t *ipv6_route_tbl = NULL;

    RTE_SET_USED(inner_ipterm_only);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    ipv6_route_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_route_table;


    /* Clear pending List if distributed RCU is success */
   ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(ipv6_route_tbl->pending_list, 
                                            ipv6_route_tbl->list_count,
                                            adk_rcu_shm_free_cb_fn, NULL);


    ipv6_addr_nw.Ipv6Addr.addr[0] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[0]);
    ipv6_addr_nw.Ipv6Addr.addr[1] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[1]);
    ipv6_addr_nw.Ipv6Addr.addr[2] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[2]);
    ipv6_addr_nw.Ipv6Addr.addr[3] = adk_htonl(ipv6_addr_info.Ipv6Addr.addr[3]);

    /* Key for route LPM lookup */
    adk_ipv6_dp_route_key_generate(input_ptn, 
                                        vrd_id, 
                                        (uint8_t *)&ipv6_addr_nw.Ipv6Addr.addr[0]);
    size = ADK_IPV6_VRD_ID_WIDTH + 2 + ipv6_addr_info.prefixLength; 

    /* disable the entry immediatly */
    ADK_IPV6_CP_LPM_LOOKUP(ipv6_route_tbl->lpm_tbl, input_ptn, size, ipv6_route_entry);

     /* Remove HOS rule to terminate the IPv6 frames*/
     if((((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->action == 
                 ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE) ||
         (((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->action == 
          ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE))
     {
         /* Rem HOS IPv6 terminate rule */
         if(!inner_ipterm_only)
         {
             ret = adk_hos_rule_delete(ADK_HOS_IP_TERM_TBL,
                     ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->hos_ipterm_rulehdl);
             if (ret != ADK_SUCCESS)
             {
                 ADK_LOG_WARNING( sys_handle, ipv6Handle,
                         "Failed to delete HOS IP TERM Table entry \n");
             }
         }
#ifdef ADK_HOS_OLD_IMPL
         /* Rem HOS inner IPv6 terminate rule */
         ret = adk_hos_rule_delete(ADK_HOS_INNER_IP_TERM_TBL,
                 ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->hos_inner_ipterm_rulehdl);
         if (ret != ADK_SUCCESS)
         {
             ADK_LOG_WARNING( sys_handle, ipv6Handle,
                     "Failed to delete HOS INNER IP TERM Table entry \n");
         }
#endif
     }

    ((adk_ipv6_dp_route_entry_t *)ipv6_route_entry)->is_enabled = FALSE;

    /* remove LPM entry - Macro updates both shadow & active copies */
    ADK_IPV6_DP_LPM_REM(ipv6_route_tbl->lpm_tbl,
            input_ptn, size);
     
    /* releases the indexs to pending list, 
     * after the rcu sync index will return back to free list 
     */ 
    ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST(ADK_IPV6_MODULE_ID,
                                             ADK_IPV6_DP_PEND_LIST_OBJ,
                                             ipv6_route_tbl->pending_list, 
                                             ipv6_route_entry, 
                                             ipv6_route_tbl->list_count,
                                             ADK_LPM_DEFAULT_PENDING_SIZE,
                                             adk_rcu_shm_free_cb_fn, NULL);

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "removed IPv6 route entry with VRD %d IPv6 adddress %08x %08x %08x %08x \n",vrd_id,
     ipv6_addr_info.Ipv6Addr.addr[0],
     ipv6_addr_info.Ipv6Addr.addr[1],
     ipv6_addr_info.Ipv6Addr.addr[2],
     ipv6_addr_info.Ipv6Addr.addr[3]);


    return ADK_SUCCESS;
}


adk_dp_status_t
adk_ipv6_dp_pbr_rule_entry_add(adk_module_handle_t __attribute__((unused)) ipv6Handle,
                                uint32_t __attribute__((unused)) pbr_rule_index,
                                adk_ipv6_pbr_info_t __attribute__((unused)) *pbr_rule)
{



    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_pbr_rule_entry_rem(adk_module_handle_t __attribute__((unused)) ipv6Handle,
                                uint32_t __attribute__((unused)) pbr_rule_index)
{



    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_pbr_state_set(adk_module_handle_t __attribute__((unused)) ipv6Handle,
                        bool __attribute__((unused)) pbr_flag)
{



    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_add(adk_module_handle_t ipv6Handle,
                                adk_ipv6_dp_nexthop_entry_t *nexthop_entry,
                                uint16_t *index)
{
    adk_ipv6_dp_nexthop_table_t *nexthop_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[nexthop_entry->interface_id];

    nexthop_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_nexthop_table;

    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(nexthop_tbl); 

    /*! Allocate the memory for the IPV6 flow handle */
    ADK_IPV6_DP_RSRV_IDX(nexthop_tbl->free_list, nexthop_tbl->tbl, *index);

    nexthop_entry->idx = nexthop_tbl->tbl[*index].idx;
    nexthop_entry->next_idx = nexthop_tbl->tbl[*index].next_idx; 

    ADK_MEMCPY(&nexthop_tbl->tbl[*index],
            nexthop_entry, 
            sizeof(adk_ipv6_dp_nexthop_entry_t));

    nexthop_tbl->tbl[*index].is_enabled = TRUE;

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "added nextohop details at index %d\n",*index);

    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_modify(adk_module_handle_t ipv6Handle,
                                adk_ipv6_dp_nexthop_entry_t *nexthop_entry,
                                uint16_t index)
{
    adk_ipv6_dp_nexthop_table_t *nexthop_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[nexthop_entry->interface_id];

    nexthop_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_nexthop_table;

    if(nexthop_tbl->tbl[index].is_enabled == TRUE)
    {
	nexthop_tbl->tbl[index].is_l2_state_resolved = nexthop_entry->is_l2_state_resolved;
	ADK_MEMCPY(&nexthop_tbl->tbl[index],
			    nexthop_entry,
			    sizeof(adk_ipv6_dp_nexthop_entry_t));
	nexthop_tbl->tbl[index].is_enabled = TRUE;
    
        ADK_LOG_INFO(sys_handle, ipv6Handle,
                "updated nextohop details at index %d\n",index);

        return ADK_SUCCESS;
    }

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;

    ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "Fast path entry is not valid at index %d\n",index);

    return ADK_DP_E_ERROR;
}

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_rem(adk_module_handle_t ipv6Handle,
                                uint16_t  index)
{
    adk_ipv6_dp_nexthop_table_t *nexthop_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_interface_entry_t *if_tbl;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    nexthop_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_nexthop_table;

    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(nexthop_tbl); 

    if_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[nexthop_tbl->tbl[index].interface_id];

    nexthop_tbl->tbl[index].is_enabled = FALSE;

    ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(nexthop_tbl->pending_list, 
            nexthop_tbl->free_list, 
            nexthop_tbl->tbl, 
            index);

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[if_tbl->vrd_id]++;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "removed nexthop details at index %d\n",index);
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_ingress_association_key_create(uint8_t *key,
                        uint8_t mask,
                        uint8_t vrd_id,
                        uint16_t interface_id,
                        uint32_t flow_label,
                        uint8_t next_header,
                        uint32_t * src_ipv6_addr,
                        uint8_t src_prefix_length,
                        uint32_t * dst_ipv6_addr,
                        uint8_t dst_prefix_length,
                        uint16_t src_port,
                        uint16_t dst_port,
                        uint8_t icmp_type,
                        uint8_t icmp_code,
                        bool icmpCodeSpecified,
                        uint16_t *size)
{
    *size=0;

    key[*size] = mask;
    (*size) += 1;

    if(mask & IPV6_INGRESS_ASSOC_BIT_VRD)
    {
        key[*size] = vrd_id;
        (*size) += 1;
    }
    else if((mask & IPV6_INGRESS_ASSOC_BIT_IF))
   {
        *((uint16_t *)&key[*size]) = interface_id;
        (*size) += 2;
    }

    if((mask & IPV6_INGRESS_ASSOC_BIT_FLOW_LABEL)
        && (flow_label))
    {
        key[*size]     = (flow_label>>16)&0xFF;
        key[(*size)+1] = (flow_label>>8)&0xFF;
        key[(*size)+2] = (flow_label)&0xFF;
        (*size) += 3;
    }
    if((mask & IPV6_INGRESS_ASSOC_BIT_PROTO) &&
        (next_header))
    {
        *((uint16_t *)&key[*size]) = next_header;
        (*size) += 1;

        /*check for L4 fields - ICMP case */
        if(next_header == ADK_IPV6_NEXT_HDR_ICMPV6)
        {

            if((mask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                    &&(icmp_type))
            {
                key[*size] = icmp_type;
                (*size) += 1;
            }
            if(mask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&dst_ipv6_addr[0],
                        (dst_prefix_length>>3));
                (*size) += (dst_prefix_length>>3);
            }

            if((mask & IPV6_INGRESS_ASSOC_BIT_L4INFO1)
                    &&(icmpCodeSpecified))
            {
                key[*size] = icmp_code;
                (*size) += 1;
            }
            if(mask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&src_ipv6_addr[0],
                        (src_prefix_length>>3));
                (*size) += (src_prefix_length>>3);
            }
        }
        /*check for L4 fields - UDP/TCP/SCTP case */
        else if((next_header == ADK_IPV6_NEXT_HDR_UDP) ||
                (next_header == ADK_IPV6_NEXT_HDR_UDP_LITE) ||
                (next_header == ADK_IPV6_NEXT_HDR_SCTP) ||
                (next_header == ADK_IPV6_NEXT_HDR_TCP))
        {
            if(mask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&dst_ipv6_addr[0],
                        (dst_prefix_length>>3));
                (*size) += (dst_prefix_length>>3);
            }
            if((mask & IPV6_INGRESS_ASSOC_BIT_L4INFO2)
                    &&(dst_port))
            {
                *((uint16_t *)&key[*size]) = dst_port;
                (*size) += 2;
            }

            if(mask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&src_ipv6_addr[0],
                        (src_prefix_length>>3));
                (*size) += (src_prefix_length>>3);
            }
            if((mask & IPV6_INGRESS_ASSOC_BIT_L4INFO1)
                    &&(src_port))
            {
                *((uint16_t *)&key[*size]) = src_port;
                (*size) += 2;
            }
        }
        /*non UDP/TCP/SCTP/ICMP case */
        else
        {
            if(mask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&dst_ipv6_addr[0],
                        (dst_prefix_length>>3));
                (*size) += (dst_prefix_length>>3);
            }
            if(mask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
            {
                rte_memcpy(&key[*size],(uint8_t *)&src_ipv6_addr[0],
                        (src_prefix_length>>3));
                (*size) += (src_prefix_length>>3);
            }

        }
    }
    /* proto is not enabled but address can be enabled */
    else
    {
        if(mask & IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)
        {
            rte_memcpy(&key[*size],(uint8_t *)&dst_ipv6_addr[0],
                    (dst_prefix_length>>3));
            (*size) += (dst_prefix_length>>3);
        }
        if(mask & IPV6_INGRESS_ASSOC_BIT_SRC_ADDR)
        {
            rte_memcpy(&key[*size],(uint8_t *)&src_ipv6_addr[0],
                    (src_prefix_length>>3));
            (*size) += (src_prefix_length>>3);
        }
    }

    /* update the size to bits */
    (*size) = (*size)<<3;

    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_key_create(uint8_t *key, 
                        uint8_t interface_id,
                        uint32_t * dst_ipv6_addr,
                        uint32_t * src_ipv6_addr, 
			bool useSrcAddr,
			uint16_t *size)
{
    *size=0;

    key[*size] = interface_id;
    (*size) += 1;

    rte_memcpy(&key[*size],(uint8_t *)&dst_ipv6_addr[0], ADK_IPV6_PDU_ADDR_SIZE_BYTES); 
    (*size) += ADK_IPV6_PDU_ADDR_SIZE_BYTES;

    if(useSrcAddr)
    {
	rte_memcpy(&key[*size],(uint8_t *)&src_ipv6_addr[0], ADK_IPV6_PDU_ADDR_SIZE_BYTES);
	(*size) += ADK_IPV6_PDU_ADDR_SIZE_BYTES;
    }

    /* update the size to bits */
    (*size) = (*size)<<3;

    return ADK_SUCCESS; 
}

adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_entry_add(adk_module_handle_t ipv6Handle,
                            adk_ipv6_multicast_group_t  *mGrp_p,
                            adk_ipv6_dp_ingress_mgroup_entry_t *ingress_mgroup_entry,
                            uint16_t *index)
{
    uint32_t vrd_id = 0;
    uint16_t size = 0;
    uint8_t key[(ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH>>3)+1];
    adk_ipv6_dp_ingress_mgroup_table_t *ingress_mgroup_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_addr_t nw_srcIpv6Addr, nw_dstIpv6Addr;
    adk_handle_t old_lpm_value = ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ingress_mgroup_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_ingress_mgroup_table;
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(ingress_mgroup_tbl);

    /*! Allocate an index */
    ADK_IPV6_DP_RSRV_IDX(ingress_mgroup_tbl->free_list, ingress_mgroup_tbl->tbl, *index);

    ingress_mgroup_entry->idx = ingress_mgroup_tbl->tbl[*index].idx;
    ingress_mgroup_entry->next_idx = ingress_mgroup_tbl->tbl[*index].next_idx;

    /* update the ingress association action table */
    ADK_MEMCPY(&ingress_mgroup_tbl->tbl[*index],
            ingress_mgroup_entry,
            sizeof(adk_ipv6_dp_ingress_mgroup_entry_t));
	
    ingress_mgroup_tbl->tbl[*index].group_stats_id = *index;
    ingress_mgroup_tbl->tbl[*index].is_enabled = TRUE;

    nw_srcIpv6Addr.addr[0] = 
        adk_htonl(mGrp_p->srcAddr.addr[0]);
    nw_srcIpv6Addr.addr[1] = 
        adk_htonl(mGrp_p->srcAddr.addr[1]);
    nw_srcIpv6Addr.addr[2] = 
        adk_htonl(mGrp_p->srcAddr.addr[2]);
    nw_srcIpv6Addr.addr[3] = 
        adk_htonl(mGrp_p->srcAddr.addr[3]);

    nw_dstIpv6Addr.addr[0] = 
        adk_htonl(mGrp_p->dstAddr.addr[0]);
    nw_dstIpv6Addr.addr[1] = 
        adk_htonl(mGrp_p->dstAddr.addr[1]);
    nw_dstIpv6Addr.addr[2] = 
        adk_htonl(mGrp_p->dstAddr.addr[2]);
    nw_dstIpv6Addr.addr[3] = 
        adk_htonl(mGrp_p->dstAddr.addr[3]);


    /* create input patern to LPM */
    adk_ipv6_dp_ingress_mgroup_key_create(key, 
                           ADK_SYS_INTERFACE_ID_GET(mGrp_p->l2InterfaceHandle),
                           nw_dstIpv6Addr.addr,
                           nw_srcIpv6Addr.addr, 
			   mGrp_p->useSrcAddr,
			   &size);
    /* create input patern to LPM */
    /* add LPM entry */
    adk_ipv6_lpm_add(ipv6Handle,
                ADK_IPV6_INGRESS_MGROUP_FLOW_TREE,
                &ingress_mgroup_tbl->lpm_tbl,
                key, 
                size,
                (adk_handle_t) ((size_t)*index),
                &old_lpm_value);

     /* release the old index if overwritten */
     if(old_lpm_value != ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID))
     {
         ADK_LOG_INFO(sys_handle, ipv6Handle,
                 "entry was overwritten, releasing the old index"
                 "old resources %d\n", (uint32_t)(size_t)old_lpm_value);
         ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(ingress_mgroup_tbl->pending_list,
                 ingress_mgroup_tbl->free_list,
                 ingress_mgroup_tbl->tbl,
                 (uint32_t)(size_t)old_lpm_value);
     }

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_entry_rem(adk_module_handle_t ipv6Handle,
                            adk_ipv6_multicast_group_t *mGrp_p,
                            uint16_t index)
{
    uint32_t vrd_id = 0;
    uint16_t size = 0;
    uint8_t key[(ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH>>3)+1];
    adk_ipv6_dp_ingress_mgroup_table_t *ingress_mgroup_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_addr_t nw_srcIpv6Addr, nw_dstIpv6Addr;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ingress_mgroup_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_ingress_mgroup_table;
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(ingress_mgroup_tbl);

    /* disable the entry immediatly */ 
    ingress_mgroup_tbl->tbl[index].is_enabled = FALSE;

    nw_srcIpv6Addr.addr[0] = 
        adk_htonl(mGrp_p->srcAddr.addr[0]);
    nw_srcIpv6Addr.addr[1] = 
        adk_htonl(mGrp_p->srcAddr.addr[1]);
    nw_srcIpv6Addr.addr[2] = 
        adk_htonl(mGrp_p->srcAddr.addr[2]);
    nw_srcIpv6Addr.addr[3] = 
        adk_htonl(mGrp_p->srcAddr.addr[3]);

    nw_dstIpv6Addr.addr[0] = 
        adk_htonl(mGrp_p->dstAddr.addr[0]);
    nw_dstIpv6Addr.addr[1] = 
        adk_htonl(mGrp_p->dstAddr.addr[1]);
    nw_dstIpv6Addr.addr[2] = 
        adk_htonl(mGrp_p->dstAddr.addr[2]);
    nw_dstIpv6Addr.addr[3] = 
        adk_htonl(mGrp_p->dstAddr.addr[3]);


    /* create input patern to LPM */
    adk_ipv6_dp_ingress_mgroup_key_create(key, 
                           ADK_SYS_INTERFACE_ID_GET(mGrp_p->l2InterfaceHandle),
                           nw_dstIpv6Addr.addr,
                           nw_srcIpv6Addr.addr,
			   mGrp_p->useSrcAddr,
			   &size);

    /* remove from LPM entry */
    ADK_IPV6_DP_LPM_REM(ingress_mgroup_tbl->lpm_tbl,
            key,
            size);

    /* move the ingress association table entry to pending list */
    ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(ingress_mgroup_tbl->pending_list,
            ingress_mgroup_tbl->free_list,
            ingress_mgroup_tbl->tbl,
            index);

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;
    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_key_generate(uint8_t vrd_id,
        uint32_t *ipv6_addr,
        uint32_t *key,   
        uint16_t *key1,
        uint16_t *key2)
{
    uint32_t    init_value = 0;
    init_value = _mm_crc32_u32(init_value, (uint32_t)vrd_id);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[0]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[1]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[2]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[3]);
    *key = init_value;
    *key1 = init_value & ADK_IPV6_ONLINK_HASH_MASK;
    *key2 = (init_value>>16) & ADK_IPV6_ONLINK_HASH_MASK;

    return(ADK_SUCCESS);
}

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_add(adk_module_handle_t ipv6Handle,
                            adk_ipv6_onlink_destination_info_t __attribute__((unused)) *onLinkDestinationInfo_p,
                            uint8_t vrd_id, 
                            adk_ipv6_dp_onlink_dest_entry_t  *onlink_dest_entry,
                            uint16_t *index,
                            uint16_t *hash_bucket_index)
{
    adk_ipv6_dp_onlink_dest_hash_entry_t  *onlink_dest_hash_entry = NULL;
    uint16_t  first_key, second_key, i;
    uint32_t key;
    adk_ipv6_dp_onlink_dest_table_t *onlink_dst_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    onlink_dst_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_onlink_dest_table;

    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(onlink_dst_tbl); 
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(onlink_dst_tbl); 

    /* reserve onlink index */
    ADK_IPV6_DP_RSRV_IDX(onlink_dst_tbl->free_list, onlink_dst_tbl->tbl, *index);
  
    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Adding Onlink destination entry at %d \n",*index);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
               onlink_dest_entry->ipv6_addr.addr[0],
               onlink_dest_entry->ipv6_addr.addr[1],
               onlink_dest_entry->ipv6_addr.addr[2],
               onlink_dest_entry->ipv6_addr.addr[3]);

    onlink_dest_entry->ipv6_addr.addr[0] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[0]); 
    onlink_dest_entry->ipv6_addr.addr[1] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[1]); 
    onlink_dest_entry->ipv6_addr.addr[2] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[2]); 
    onlink_dest_entry->ipv6_addr.addr[3] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[3]); 
    onlink_dest_entry->idx = onlink_dst_tbl->tbl[*index].idx; 
    onlink_dest_entry->next_idx =  onlink_dst_tbl->tbl[*index].next_idx;

    ADK_MEMCPY(&onlink_dst_tbl->tbl[*index], 
            onlink_dest_entry, 
            sizeof(adk_ipv6_dp_onlink_dest_entry_t));

    onlink_dst_tbl->tbl[*index].is_enabled = TRUE;

    /* Generate key for onlink dest table lookup */
    adk_ipv6_dp_onlink_dest_key_generate(vrd_id,
                                &onlink_dest_entry->ipv6_addr.addr[0],
                                &key,
                                &first_key, 
                                &second_key);

    /* update the Hash entry, TODO:handle collision entry*/
    first_key = first_key*ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES;
    onlink_dest_hash_entry = &onlink_dst_tbl->hash_tbl[(first_key)];

    for(i=0; i<ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES; i++, onlink_dest_hash_entry++)
    {
        /* search for free entry */
        if(!onlink_dest_hash_entry->is_enabled)
        {
            onlink_dest_hash_entry->index       = *index;
            onlink_dest_hash_entry->hash_value  = key;
            onlink_dest_hash_entry->is_enabled  = TRUE;
            *hash_bucket_index = first_key + i;

            /* update the dp tables version */
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

            return(ADK_SUCCESS);
        }
    }

    second_key = second_key*ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES;
    onlink_dest_hash_entry = &onlink_dst_tbl->hash_tbl[(second_key)];

    for(i=0; i<ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES; i++, onlink_dest_hash_entry++)
    {
        /* search for free entry */
        if(!onlink_dest_hash_entry->is_enabled)
        {
            onlink_dest_hash_entry->index       = *index;
            onlink_dest_hash_entry->hash_value  = key;
            onlink_dest_hash_entry->is_enabled  = TRUE;
            *hash_bucket_index = second_key + i;

            /* update the dp tables version */
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

            return(ADK_SUCCESS);
        }
    }

    /* add to overflow list */
    ADK_IPV6_DP_ADD_OVERFLOW_ENTRY(onlink_dst_tbl->overflow_list,
            onlink_dst_tbl->free_list, onlink_dst_tbl->tbl, *index);

    return(ADK_SUCCESS);
}


adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_modify(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id, 
                            adk_ipv6_dp_onlink_dest_entry_t  *onlink_dest_entry,
                            uint16_t index)
{
    
    adk_ipv6_dp_onlink_dest_table_t *onlink_dst_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    onlink_dst_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_onlink_dest_table;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Updating Onlink destination entry at %d \n",index);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
               onlink_dest_entry->ipv6_addr.addr[0],
               onlink_dest_entry->ipv6_addr.addr[1],
               onlink_dest_entry->ipv6_addr.addr[2],
               onlink_dest_entry->ipv6_addr.addr[3]);

    onlink_dest_entry->ipv6_addr.addr[0] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[0]); 
    onlink_dest_entry->ipv6_addr.addr[1] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[1]); 
    onlink_dest_entry->ipv6_addr.addr[2] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[2]); 
    onlink_dest_entry->ipv6_addr.addr[3] = adk_htonl(onlink_dest_entry->ipv6_addr.addr[3]); 

    if(onlink_dst_tbl->tbl[index].is_enabled == TRUE)
    {
            if((vrd_id == (onlink_dest_entry)->vrd_id) && 
               (*((uint64_t *)&onlink_dest_entry->ipv6_addr.addr[0]) == 
                    *((uint64_t *)&( onlink_dst_tbl->tbl[index].ipv6_addr.addr[0]))) &&
               (*((uint64_t *)&onlink_dest_entry->ipv6_addr.addr[2]) == 
                    *((uint64_t *)&((onlink_dst_tbl->tbl[index].ipv6_addr.addr[2]))))
              )
            {
                /* update the interface handle */
                onlink_dst_tbl->tbl[index].interface_id = onlink_dest_entry->interface_id;
            
                /* update the dp tables version */
                ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

                return ADK_SUCCESS;
            }
            else
            {
                ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "flow id & onlink address are not matching \n");
                return(ADK_DP_E_ERROR);
            }
    }
    return(ADK_DP_E_ERROR);
}

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_rem(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id, 
                            uint8_t *onLinkDestinationIpv6Addr, 
                            uint16_t tbl_index,
                            uint16_t hash_bucket_idx)
{
    adk_ipv6_dp_onlink_dest_table_t *onlink_dst_tbl;
    adk_ipv6_dp_onlink_dest_entry_t *onlink_dest_entry;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    onlink_dst_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_onlink_dest_table;
    
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(onlink_dst_tbl); 
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(onlink_dst_tbl); 

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "removing Onlink destination entry at %d \n",tbl_index);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
              *((uint32_t *)&onLinkDestinationIpv6Addr[0]),
              *((uint32_t *)&onLinkDestinationIpv6Addr[4]),
              *((uint32_t *)&onLinkDestinationIpv6Addr[8]),
              *((uint32_t *)&onLinkDestinationIpv6Addr[12]));


    *((uint32_t *)&onLinkDestinationIpv6Addr[0]) = 
                            adk_htonl(*((uint32_t *)&onLinkDestinationIpv6Addr[0]));
    *((uint32_t *)&onLinkDestinationIpv6Addr[4]) = 
                            adk_htonl(*((uint32_t *)&onLinkDestinationIpv6Addr[4]));
    *((uint32_t *)&onLinkDestinationIpv6Addr[8]) = 
                            adk_htonl(*((uint32_t *)&onLinkDestinationIpv6Addr[8]));
    *((uint32_t *)&onLinkDestinationIpv6Addr[12]) = 
                            adk_htonl(*((uint32_t *)&onLinkDestinationIpv6Addr[12]));

    if(onlink_dst_tbl->hash_tbl[hash_bucket_idx].index == tbl_index)
    {
        onlink_dest_entry = &onlink_dst_tbl->tbl[tbl_index];

        if((vrd_id == (onlink_dest_entry)->vrd_id) && 
                (*((uint64_t *)&onLinkDestinationIpv6Addr[0]) == 
                 *((uint64_t *)&((onlink_dest_entry)->ipv6_addr.addr[0]))) &&
                (*((uint64_t *)&onLinkDestinationIpv6Addr[8]) == 
                 *((uint64_t *)&((onlink_dest_entry)->ipv6_addr.addr[2]))))
        {
            /* reset the hash entry */
            *((uint64_t *) &onlink_dst_tbl->hash_tbl[hash_bucket_idx]) = 0;

            /* move the onlink table entry to pending list */
            ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(onlink_dst_tbl->pending_list,
                    onlink_dst_tbl->free_list,
                    onlink_dst_tbl->tbl,
                    tbl_index);

            /* update the dp tables version */
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

            return ADK_SUCCESS;
        }
    }

    /* check in overflow list */
    ADK_IPV6_DP_RELEASE_OVERFLOW_ENTRY(onlink_dst_tbl->overflow_list, 
            onlink_dst_tbl->overflow_pending_list, onlink_dst_tbl->pending_list, 
            onlink_dst_tbl->free_list, onlink_dst_tbl->tbl, tbl_index);

    return ADK_DP_E_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_egress_flow_attr_entry_add(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_dp_egress_flow_attr_entry_t *egress_flow_attr_entry,
                            uint16_t *index)
{
    adk_ipv6_dp_egress_flow_attr_table_t *egress_flow_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    uint8_t next_hdr;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    egress_flow_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_egress_flow_attr_table;

    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(egress_flow_tbl); 

    ADK_IPV6_DP_RSRV_IDX(egress_flow_tbl->free_list, egress_flow_tbl->tbl, *index);
    
    next_hdr = (egress_flow_attr_entry->len_nh_hlimit>>8) &0xFF;
    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Adding egress flow entry at %d \n",*index);

    egress_flow_attr_entry->ver_tc_flabel = adk_htonl(egress_flow_attr_entry->ver_tc_flabel);
    egress_flow_attr_entry->len_nh_hlimit = adk_htonl(egress_flow_attr_entry->len_nh_hlimit);

    egress_flow_attr_entry->dst_addr.addr[0] = 
        adk_htonl(egress_flow_attr_entry->dst_addr.addr[0]);
    egress_flow_attr_entry->dst_addr.addr[1] = 
        adk_htonl(egress_flow_attr_entry->dst_addr.addr[1]);
    egress_flow_attr_entry->dst_addr.addr[2] = 
        adk_htonl(egress_flow_attr_entry->dst_addr.addr[2]);
    egress_flow_attr_entry->dst_addr.addr[3] = 
        adk_htonl(egress_flow_attr_entry->dst_addr.addr[3]);

    egress_flow_attr_entry->src_addr.addr[0] = 
        adk_htonl(egress_flow_attr_entry->src_addr.addr[0]);
    egress_flow_attr_entry->src_addr.addr[1] = 
        adk_htonl(egress_flow_attr_entry->src_addr.addr[1]);
    egress_flow_attr_entry->src_addr.addr[2] = 
        adk_htonl(egress_flow_attr_entry->src_addr.addr[2]);
    egress_flow_attr_entry->src_addr.addr[3] = 
        adk_htonl(egress_flow_attr_entry->src_addr.addr[3]);

    egress_flow_attr_entry->dst_port = adk_htons(egress_flow_attr_entry->dst_port);
    egress_flow_attr_entry->src_port = adk_htons(egress_flow_attr_entry->src_port);

    egress_flow_attr_entry->idx = egress_flow_tbl->tbl[*index].idx;
    egress_flow_attr_entry->next_idx = egress_flow_tbl->tbl[*index].next_idx;

    /*Update DP entry */
    ADK_MEMCPY(&egress_flow_tbl->tbl[*index], 
            egress_flow_attr_entry, 
            sizeof(adk_ipv6_dp_egress_flow_attr_entry_t));

#ifdef ADK_IPV6_FLOW_CACHE
    /* Update the Hash value */
    if((next_hdr == ADK_IPV6_NEXT_HDR_UDP) || 
            (next_hdr == ADK_IPV6_NEXT_HDR_UDP_LITE) ||
            (next_hdr == ADK_IPV6_NEXT_HDR_TCP) || 
            (next_hdr == ADK_IPV6_NEXT_HDR_SCTP))
    {
        adk_ipv6_dp_5tuple_uli_flow_cache_key_generate(
            (void *) &egress_flow_attr_entry->ver_tc_flabel,
            next_hdr,
            egress_flow_attr_entry->src_port,
            egress_flow_attr_entry->dst_port,
            egress_flow_attr_entry->vrd_id,
            &egress_flow_attr_entry->hash_value);
    }
    else
    {
        adk_ipv6_dp_3tuple_uli_flow_cache_key_generate(
            (void *) &egress_flow_attr_entry->ver_tc_flabel,
            next_hdr,
            egress_flow_attr_entry->vrd_id,
            &egress_flow_attr_entry->hash_value);
    }
#endif

    egress_flow_tbl->tbl[*index].is_enabled = TRUE;
    egress_flow_tbl->tbl[*index].flow_stats_id = *index;

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[egress_flow_attr_entry->vrd_id]++;

    return(ADK_SUCCESS);
}


adk_dp_status_t 
adk_ipv6_dp_egress_flow_attr_entry_modify(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_dp_egress_flow_attr_entry_t *egress_flow_attr_entry,
                            uint16_t index)
{
    adk_ipv6_dp_egress_flow_attr_table_t *egress_flow_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    egress_flow_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_egress_flow_attr_table;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "Adding egress flow entry at %d \n",index);

    if(egress_flow_tbl->tbl[index].is_enabled == TRUE)
    {
        egress_flow_tbl->tbl[index].mtu = egress_flow_attr_entry->mtu;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[egress_flow_attr_entry->vrd_id]++;

        return(ADK_SUCCESS);
    }

    return ADK_DP_E_ERROR;
}


adk_dp_status_t
adk_ipv6_dp_egress_flow_attr_entry_rem(adk_module_handle_t ipv6Handle,
                                uint16_t tbl_idx)
{
    adk_ipv6_dp_egress_flow_attr_table_t *egress_flow_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    egress_flow_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_egress_flow_attr_table;

    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(egress_flow_tbl); 

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "removing egress flow entry at %d \n", tbl_idx);
    egress_flow_tbl->tbl[tbl_idx].is_enabled = FALSE;
    ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(egress_flow_tbl->pending_list, 
            egress_flow_tbl->free_list, 
            egress_flow_tbl->tbl, 
            tbl_idx);

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[egress_flow_tbl->tbl[tbl_idx].vrd_id]++;

    return ADK_SUCCESS;
}



void 
adk_ipv6_dp_ingress_flow_sip_key_generate(uint8_t *key, uint8_t vrd_id, uint8_t *addr)
{
    key[0] = vrd_id;
    ADK_MEMCPY(&key[1], &addr[0], sizeof(adk_ipv6_addr_t));
    
    return;
}


adk_dp_status_t 
adk_ipv6_dp_ingress_flow_sip_entry_add(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6AddrPrefix,
                            adk_ipv6_dp_ingress_flow_sip_entry_t *ingress_flow_sip_entry)
{
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1];
    uint32_t size;
    adk_ipv6_addr_prefix_t ipv6_src_nw;
    adk_ipv6_dp_ingress_flow_sip_table_t *sip_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_ingress_flow_sip_entry_t *sip_entry = NULL;
    adk_handle_t old_lpm_value = ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID);

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    sip_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_ingress_flow_sip_table;

    /* Clear pending List if distributed RCU is success */
    ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(sip_tbl->pending_list, 
                                           sip_tbl->list_count,
                                           adk_rcu_shm_free_cb_fn, NULL);

    sip_entry = (adk_ipv6_dp_ingress_flow_sip_entry_t *) 
              ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
		 ADK_IPV6_SIP_ENTRY_OBJ,
		 1 , sizeof(adk_ipv6_dp_ingress_flow_sip_entry_t), 0);
    if ( sip_entry == NULL)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                      "Memory allocation for sip table failed \n");
        return  ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
    }

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
              ipv6AddrPrefix.Ipv6Addr.addr[0],
              ipv6AddrPrefix.Ipv6Addr.addr[1],
              ipv6AddrPrefix.Ipv6Addr.addr[2],
              ipv6AddrPrefix.Ipv6Addr.addr[3]);



    /* update the route action table */
    ADK_MEMCPY(sip_entry, 
            ingress_flow_sip_entry, 
            sizeof(adk_ipv6_dp_ingress_flow_sip_entry_t));

    sip_entry->is_enabled = TRUE;

    ipv6_src_nw.Ipv6Addr.addr[0] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[0]); 
    ipv6_src_nw.Ipv6Addr.addr[1] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[1]); 
    ipv6_src_nw.Ipv6Addr.addr[2] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[2]); 
    ipv6_src_nw.Ipv6Addr.addr[3] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[3]); 

    /* Key for route LPM lookup */
    adk_ipv6_dp_ingress_flow_sip_key_generate(input_ptn, 
                                              vrd_id, 
                                              (uint8_t *)&ipv6_src_nw.Ipv6Addr.addr[0]);
    /* add LPM entry */
    size = ADK_IPV6_VRD_ID_WIDTH + 2 + ipv6AddrPrefix.prefixLength; 

    adk_ipv6_lpm_add(ipv6Handle,
                ADK_IPV6_INRESS_FLOW_SIP_TREE,                  
                &sip_tbl->lpm_tbl,
                input_ptn, 
                size, 
                ADK_CONVERT_TO_LPM_DATA(sip_entry),
                &old_lpm_value);
     /* release the old index if overwritten */
     if(old_lpm_value != ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID))
     {
         ADK_LOG_INFO(sys_handle, ipv6Handle,
                 "entry was overwritten, releasing the" 
                 "old resources %"ADKPRIh"\n", old_lpm_value);
         ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST(ADK_IPV6_MODULE_ID,
                                                  ADK_IPV6_DP_PEND_LIST_OBJ,
                                                  sip_tbl->pending_list, 
                                                  old_lpm_value, 
                                                  sip_tbl->list_count,
                                                  ADK_LPM_DEFAULT_PENDING_SIZE,
                                                  adk_rcu_shm_free_cb_fn, NULL);
     }

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_ingress_flow_sip_entry_rem(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6AddrPrefix)
{
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1];
    uint32_t size;
    adk_handle_t sip_entry;
    adk_ipv6_addr_prefix_t ipv6_src_nw;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;
    adk_ipv6_dp_ingress_flow_sip_table_t *sip_tbl;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    sip_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_ingress_flow_sip_table;

    /* Clear pending List if distributed RCU is success */
    ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(sip_tbl->pending_list, 
                                           sip_tbl->list_count,
                                           adk_rcu_shm_free_cb_fn, NULL);
  

    ipv6_src_nw.Ipv6Addr.addr[0] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[0]); 
    ipv6_src_nw.Ipv6Addr.addr[1] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[1]); 
    ipv6_src_nw.Ipv6Addr.addr[2] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[2]); 
    ipv6_src_nw.Ipv6Addr.addr[3] = adk_htonl(ipv6AddrPrefix.Ipv6Addr.addr[3]); 

    /* Key for route LPM lookup */
    adk_ipv6_dp_ingress_flow_sip_key_generate(input_ptn, 
                                              vrd_id, 
                                              (uint8_t *)&ipv6_src_nw.Ipv6Addr.addr[0]);

    /* remove LPM entry - Macro updates both shadow & active copies */
    size = ADK_IPV6_VRD_ID_WIDTH + 2 + ipv6AddrPrefix.prefixLength; 

    ADK_IPV6_CP_LPM_LOOKUP(sip_tbl->lpm_tbl, input_ptn, size, sip_entry);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "removing SIP validation entry at 0x%"ADKPRIh" \n",sip_entry);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
              ipv6AddrPrefix.Ipv6Addr.addr[0],
              ipv6AddrPrefix.Ipv6Addr.addr[1],
              ipv6AddrPrefix.Ipv6Addr.addr[2],
              ipv6AddrPrefix.Ipv6Addr.addr[3]);


    /* disable the entry immediatly */
    ((adk_ipv6_dp_ingress_flow_sip_entry_t *) sip_entry)->is_enabled = FALSE;

    ADK_IPV6_DP_LPM_REM(sip_tbl->lpm_tbl,
            input_ptn,
            size);
    /* 
     * releases the indexs to pending list, 
     * after the rcu sync index will return back to free list 
     */
    ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST(ADK_IPV6_MODULE_ID,
                                             ADK_IPV6_DP_PEND_LIST_OBJ,
                                             sip_tbl->pending_list, 
                                             sip_entry, 
                                             sip_tbl->list_count,
                                             ADK_LPM_DEFAULT_PENDING_SIZE,
                                             adk_rcu_shm_free_cb_fn, NULL);

    /* update the dp tables version */
    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

    return ADK_SUCCESS;
}


adk_dp_status_t
adk_ipv6_dp_excep_entry_add(adk_module_handle_t ipv6Handle,
                            adk_ipv6_exception_type_t exception,
                            adk_ipv6_dp_excep_entry_t *except_entry)
{
    adk_ipv6_dp_excep_table_t *except_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "enabling ipv6 exception exception %d \n",exception);

    except_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_excep_table;

    ADK_MEMSET(&except_tbl->tbl[exception],
            0 ,
            sizeof(adk_ipv6_dp_excep_entry_t));

    rte_memcpy(&except_tbl->tbl[exception],
            except_entry,
            sizeof(adk_ipv6_dp_excep_entry_t));

    except_tbl->tbl[exception].is_enable = TRUE;

    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_excep_entry_rem(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_exception_type_t exception)
{
    adk_ipv6_dp_excep_table_t *except_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 

    except_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_excep_table;

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "disabled ipv6 exception %d \n",exception);
    except_tbl->tbl[exception].is_enable = FALSE;

    /* wait for RCU sync time */
    adk_rcu_synchronize();

    return ADK_SUCCESS;
}

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_key_generate(uint8_t vrd_id,
        uint32_t *ipv6_addr,
        uint32_t *key,   
        uint16_t *key1,
        uint16_t *key2)
{
    uint32_t    init_value = 0;
    init_value = _mm_crc32_u32(init_value, (uint32_t)vrd_id);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[0]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[1]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[2]);
    init_value = _mm_crc32_u32(init_value, ipv6_addr[3]);
    *key = init_value;
    *key1 = init_value & ADK_IPV6_DEST_MTU_HASH_MASK;
    *key2 = (init_value>>16) & ADK_IPV6_DEST_MTU_HASH_MASK;

    return(ADK_SUCCESS);
}

                      

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_add(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_entry,
                            uint16_t *index)
{
    
    adk_ipv6_dp_dest_mtu_hash_entry_t  *dest_mtu_hash_entry = NULL;
    adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_lkup_entry = NULL;
    uint16_t  first_key, second_key, i;
    uint32_t key;
    adk_ipv6_dp_dest_mtu_table_t *dst_mtu_tbl, *prev_dst_mtu_tbl;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
               dest_mtu_entry->ipv6_addr.addr[0],
               dest_mtu_entry->ipv6_addr.addr[1],
               dest_mtu_entry->ipv6_addr.addr[2],
               dest_mtu_entry->ipv6_addr.addr[3]);

    dest_mtu_entry->ipv6_addr.addr[0] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[0]); 
    dest_mtu_entry->ipv6_addr.addr[1] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[1]); 
    dest_mtu_entry->ipv6_addr.addr[2] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[2]); 
    dest_mtu_entry->ipv6_addr.addr[3] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[3]); 

    /* 
     * check if entry already present in the DEST MTU table 
     * Then update the mtu, else add an entry 
     */
    if(adk_ipv6_dp_dest_mtu_entry_lookup(vrd_id, 
                (uint8_t *)&dest_mtu_entry->ipv6_addr.addr[0],
                ipv6ModuleInfo_p->adk_ipv6_dp_tables,
                &dest_mtu_lkup_entry) == ADK_SUCCESS)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, 
                "Entry already exist !!! \n");
        return ADK_DP_E_ERROR;
    }

    dst_mtu_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.tbl_head;

    while( dst_mtu_tbl != NULL)
    {
        ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(dst_mtu_tbl);

        if ( dst_mtu_tbl->free_list.count )
        {
            /* Reserve dest mtu index */
            ADK_IPV6_DP_RSRV_IDX(dst_mtu_tbl->free_list, dst_mtu_tbl->tbl, *index);
        }
        else
        {
            if ( dst_mtu_tbl->next == NULL)
            {
                if ( ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.num_table > ADK_IPV6_MAX_DYNAMIC_DEST_MTU_TABLE) 
                {
                    ADK_LOG_ERROR(sys_handle, ipv6Handle,
                    "\nDestination mtu Table Max capacity achieved \n");
                    return ADK_E_MEM_UNAVAILABLE;
                }

                dst_mtu_tbl->next = (adk_ipv6_dp_dest_mtu_table_t *) ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_DST_MTU_TBL_OBJ, 1, 
                        sizeof(adk_ipv6_dp_dest_mtu_table_t), 0);
                if(dst_mtu_tbl->next == NULL)
                {
                    ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                    return ADK_DP_E_ERROR; 
                }
                prev_dst_mtu_tbl = dst_mtu_tbl;
                dst_mtu_tbl = dst_mtu_tbl->next; 

                /* IPV6 Destination Address MTU table */
                dst_mtu_tbl->hash_tbl = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_DST_MTU_HASHTBL_OBJ, 
                        (ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES*ADK_IPV6_NO_OF_DEST_MTU_ENTRIES),
                         sizeof(adk_ipv6_dp_dest_mtu_hash_entry_t), 0);
                if(dst_mtu_tbl->hash_tbl == NULL)
                {
                    prev_dst_mtu_tbl->next = NULL;
                    ADK_DP_SHM_FREE(dst_mtu_tbl);
                    ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                    return ADK_DP_E_ERROR; 
                }

                dst_mtu_tbl->tbl = ADK_DP_SHM_CALLOC(
			ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_DST_MTU_TBL_OBJ, 
                        (ADK_IPV6_NO_OF_DEST_MTU_ENTRIES),
                         sizeof(adk_ipv6_dp_dest_mtu_hash_entry_t), 0);
                if(dst_mtu_tbl->tbl == NULL)
                {
                    ADK_DP_SHM_FREE(dst_mtu_tbl->hash_tbl);
                    prev_dst_mtu_tbl->next = NULL;
                    ADK_DP_SHM_FREE(dst_mtu_tbl);
                    ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                    return ADK_DP_E_ERROR; 
                }

                ADK_IPV6_DP_TABLE_IDX_INIT((*dst_mtu_tbl),
                        ADK_IPV6_NO_OF_DEST_MTU_ENTRIES);

                ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.num_table++;
            }
            else
            {
                dst_mtu_tbl = dst_mtu_tbl->next;
            }
            continue;
        }
  
        ADK_LOG_INFO(sys_handle, ipv6Handle,
                    "Adding destination mtu entry at %d \n",*index);

        dest_mtu_entry->idx      = dst_mtu_tbl->tbl[*index].idx; 
        dest_mtu_entry->next_idx = dst_mtu_tbl->tbl[*index].next_idx;

        ADK_MEMCPY(&dst_mtu_tbl->tbl[*index], 
                dest_mtu_entry, 
                sizeof(adk_ipv6_dp_dest_mtu_entry_t));

        dst_mtu_tbl->tbl[*index].is_enabled = TRUE;

        /* Generate key for dest mtu table lookup */
        adk_ipv6_dp_dest_mtu_key_generate(vrd_id,
                                    &dest_mtu_entry->ipv6_addr.addr[0],
                                    &key,
                                    &first_key, 
                                    &second_key);

        /* update the Hash entry, TODO:handle collision entry*/
        first_key = first_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;
        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(first_key)];

        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            /* search for free entry */
            if(!dest_mtu_hash_entry->is_enabled)
            {
                dest_mtu_hash_entry->index       = *index;
                dest_mtu_hash_entry->hash_value  = key;
                dest_mtu_hash_entry->is_enabled  = TRUE;

                /* update the dp tables version */
                ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

                return(ADK_SUCCESS);
            }
        }

        second_key = second_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;
        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(second_key)];

        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            /* search for free entry */
            if(!dest_mtu_hash_entry->is_enabled)
            {
                dest_mtu_hash_entry->index       = *index;
                dest_mtu_hash_entry->hash_value  = key;
                dest_mtu_hash_entry->is_enabled  = TRUE;

                /* update the dp tables version */
                ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

                return(ADK_SUCCESS);
            }
        }

        /* Free entry Not found - proceeding for next table */ 
        /* Relese the allocated index as hash entry not found */
        ADK_IPV6_DP_RELEASE_IDX(dst_mtu_tbl->pending_list, dst_mtu_tbl->free_list, dst_mtu_tbl->tbl, *index);

        if ( dst_mtu_tbl->next == NULL)
        {
            if ( ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.num_table > ADK_IPV6_MAX_DYNAMIC_DEST_MTU_TABLE) 
            {
                ADK_LOG_ERROR(sys_handle, ipv6Handle,
                        "\nDestination mtu Table Max capacity achieved \n");
                return ADK_E_MEM_UNAVAILABLE;
            }

            dst_mtu_tbl->next = (adk_ipv6_dp_dest_mtu_table_t *) ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_DST_MTU_TBL_OBJ, 1, 
                    sizeof(adk_ipv6_dp_dest_mtu_table_t), 0);
            if(dst_mtu_tbl->next == NULL)
            {
                ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                return ADK_DP_E_ERROR; 
            }

            prev_dst_mtu_tbl = dst_mtu_tbl;
            dst_mtu_tbl = dst_mtu_tbl->next; 

            /* IPV6 Destination Address MTU table */
            dst_mtu_tbl->hash_tbl = ADK_DP_SHM_CALLOC(
			ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_DST_MTU_HASHTBL_OBJ,
                    (ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES*ADK_IPV6_NO_OF_DEST_MTU_ENTRIES),
                    sizeof(adk_ipv6_dp_dest_mtu_hash_entry_t), 0);
            if(dst_mtu_tbl->hash_tbl == NULL)
            {
                prev_dst_mtu_tbl->next = NULL;
                ADK_DP_SHM_FREE(dst_mtu_tbl);
                ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                return ADK_DP_E_ERROR; 
            }

            dst_mtu_tbl->tbl = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
                    ADK_IPV6_DST_MTU_TBL_OBJ, (ADK_IPV6_NO_OF_DEST_MTU_ENTRIES),
                    sizeof(adk_ipv6_dp_dest_mtu_hash_entry_t), 0);
            if(dst_mtu_tbl->tbl == NULL)
            {
                ADK_DP_SHM_FREE(dst_mtu_tbl->hash_tbl);
                prev_dst_mtu_tbl->next = NULL;
                ADK_DP_SHM_FREE(dst_mtu_tbl);
                ADK_LOG_ERROR(sys_handle, ipv6Handle, "error while allocating IPV6 DP DestMTU ext Table \n"); 
                return ADK_DP_E_ERROR; 
            }

            ADK_IPV6_DP_TABLE_IDX_INIT((*dst_mtu_tbl),
                    ADK_IPV6_NO_OF_DEST_MTU_ENTRIES);
            ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.num_table++;

        }
        else
        {
            dst_mtu_tbl = dst_mtu_tbl->next;
        }
    }
    return(ADK_DP_E_ERROR);
}


adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_modify(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_entry)
{
    
    adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_lkup_entry = NULL;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
               dest_mtu_entry->ipv6_addr.addr[0],
               dest_mtu_entry->ipv6_addr.addr[1],
               dest_mtu_entry->ipv6_addr.addr[2],
               dest_mtu_entry->ipv6_addr.addr[3]);

    dest_mtu_entry->ipv6_addr.addr[0] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[0]); 
    dest_mtu_entry->ipv6_addr.addr[1] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[1]); 
    dest_mtu_entry->ipv6_addr.addr[2] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[2]); 
    dest_mtu_entry->ipv6_addr.addr[3] = adk_htonl(dest_mtu_entry->ipv6_addr.addr[3]); 

    /* 
     * check if entry already present in the DEST MTU table 
     * Then update the mtu, else add an entry 
     */
    if(adk_ipv6_dp_dest_mtu_entry_lookup(vrd_id, 
                (uint8_t *)&dest_mtu_entry->ipv6_addr.addr[0],
                ipv6ModuleInfo_p->adk_ipv6_dp_tables,
                &dest_mtu_lkup_entry) == ADK_SUCCESS)
    {
        dest_mtu_lkup_entry->mtu = dest_mtu_entry->mtu;

        /* update the dp tables version */
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

        return ADK_SUCCESS;
    }

    return(ADK_DP_E_ERROR);
}

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_lookup(uint8_t vrd_id, 
                            uint8_t *ipv6_addr,
                            adk_ipv6_dp_tables_t *dp_tables,
                            adk_ipv6_dp_dest_mtu_entry_t  **dst_mtu_entry)
{
    uint32_t key;
    uint16_t first_key, second_key, i;
    adk_ipv6_dp_dest_mtu_hash_entry_t   *dest_mtu_hash_entry = NULL;
    adk_ipv6_dp_dest_mtu_table_t        *dst_mtu_tbl;


    /* Generate key for dest mtu table lookup */
    adk_ipv6_dp_dest_mtu_key_generate(vrd_id,
                                (uint32_t *)ipv6_addr,
                                &key,
                                &first_key, 
                                &second_key);
    first_key = first_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;
    second_key = second_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;

    dst_mtu_tbl = &dp_tables->ipv6_dp_dest_mtu_tbl_list.tbl_head;
    while( dst_mtu_tbl != NULL) 
    {
    
        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(first_key)];
        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            if((dest_mtu_hash_entry->hash_value == key))
            {
                /*compare vrdid & destination address */
                *dst_mtu_entry = &dst_mtu_tbl->tbl[dest_mtu_hash_entry->index];
    
                if((vrd_id == (*dst_mtu_entry)->vrd_id) && 
                   (*((uint64_t *)&ipv6_addr[0]) == 
                        *((uint64_t *)&((*dst_mtu_entry)->ipv6_addr.addr[0]))) &&
                   (*((uint64_t *)&ipv6_addr[8]) == 
                        *((uint64_t *)&((*dst_mtu_entry)->ipv6_addr.addr[2]))))
                {
                    return(ADK_SUCCESS);
                }
            }
        }
        
        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(second_key)];
        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            if((dest_mtu_hash_entry->hash_value == key))
            {
                /*compare vrdid & destination address */
                *dst_mtu_entry = &dst_mtu_tbl->tbl[dest_mtu_hash_entry->index];
    
                if((vrd_id == (*dst_mtu_entry)->vrd_id) && 
                   (*((uint64_t *)&ipv6_addr[0]) == 
                        *((uint64_t *)&((*dst_mtu_entry)->ipv6_addr.addr[0]))) &&
                   (*((uint64_t *)&ipv6_addr[8]) == 
                        *((uint64_t *)&((*dst_mtu_entry)->ipv6_addr.addr[2]))))
                {
                    return(ADK_SUCCESS);
                }
            }
        }
        dst_mtu_tbl = dst_mtu_tbl->next;
    }
    /*ADK_IPV6_DP_LOG_INFO("dest mtu entry not found \n");*/
    *dst_mtu_entry = NULL;

    return ADK_DP_E_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_rem(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id, 
                            uint8_t *ipv6_addr)
{
    uint32_t key;
    uint16_t first_key, second_key, i, tbl_index;
    adk_ipv6_dp_dest_mtu_table_t *dst_mtu_tbl;
    adk_ipv6_dp_dest_mtu_table_t *prev_dst_mtu_tbl = NULL;
    adk_ipv6_dp_dest_mtu_entry_t *dst_mtu_entry;
    adk_ipv6_dp_dest_mtu_hash_entry_t   *dest_mtu_hash_entry = NULL;
    adk_sys_handle_t sys_handle;
    adk_ipv6_info_t *ipv6ModuleInfo_p= NULL;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);

    dst_mtu_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.tbl_head;
    
    ADK_LOG_INFO(sys_handle, ipv6Handle,
                "vrd id %d IPv6 address %08x %08x %08x %08x \n",vrd_id,
              *((uint32_t *)&ipv6_addr[0]),
              *((uint32_t *)&ipv6_addr[4]),
              *((uint32_t *)&ipv6_addr[8]),
              *((uint32_t *)&ipv6_addr[12]));

    *((uint32_t *)&ipv6_addr[0]) = 
                            adk_htonl(*((uint32_t *)&ipv6_addr[0]));
    *((uint32_t *)&ipv6_addr[4]) = 
                            adk_htonl(*((uint32_t *)&ipv6_addr[4]));
    *((uint32_t *)&ipv6_addr[8]) = 
                            adk_htonl(*((uint32_t *)&ipv6_addr[8]));
    *((uint32_t *)&ipv6_addr[12]) = 
                            adk_htonl(*((uint32_t *)&ipv6_addr[12]));

    /* Generate key for dest mtu table lookup */
    adk_ipv6_dp_dest_mtu_key_generate(vrd_id,
                                (uint32_t *)ipv6_addr,
                                &key,
                                &first_key, 
                                &second_key);

    first_key = first_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;

  
    ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(dst_mtu_tbl);
    prev_dst_mtu_tbl = dst_mtu_tbl;
    dst_mtu_tbl = dst_mtu_tbl->next;

    while(dst_mtu_tbl != NULL)
    {
        ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(dst_mtu_tbl);

        if ( dst_mtu_tbl->free_list.count == ADK_IPV6_NO_OF_DEST_MTU_ENTRIES)
        {
             prev_dst_mtu_tbl->next = dst_mtu_tbl->next;

             /* IPv6 destination address MTU table release*/
             ADK_IPV6_DP_TABLE_FREE(dst_mtu_tbl->hash_tbl);
             ADK_IPV6_DP_TABLE_FREE(dst_mtu_tbl->tbl);
             ADK_IPV6_DP_TABLE_FREE(dst_mtu_tbl);
             dst_mtu_tbl = prev_dst_mtu_tbl->next;
             ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.num_table--;
        }
        else
        {
            prev_dst_mtu_tbl = dst_mtu_tbl;
            dst_mtu_tbl = dst_mtu_tbl->next;
        }
    }

    dst_mtu_tbl = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_dest_mtu_tbl_list.tbl_head;
    while(dst_mtu_tbl != NULL)
    {

        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(first_key)];
        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            if((dest_mtu_hash_entry->hash_value == key))
            {
                /* compare vrdid & destination address */
                dst_mtu_entry = &dst_mtu_tbl->tbl[dest_mtu_hash_entry->index];

                if((vrd_id == (dst_mtu_entry)->vrd_id) && 
                   (*((uint64_t *)&ipv6_addr[0]) == 
                        *((uint64_t *)&((dst_mtu_entry)->ipv6_addr.addr[0]))) &&
                   (*((uint64_t *)&ipv6_addr[8]) == 
                        *((uint64_t *)&((dst_mtu_entry)->ipv6_addr.addr[2]))))
                {
                    /* reset the hash entry */
                    tbl_index = dest_mtu_hash_entry->index;
                    *((uint64_t *)dest_mtu_hash_entry) = 0;

                    /* move the dest mtu table entry to pending list */
                    ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(dst_mtu_tbl->pending_list,
                            dst_mtu_tbl->free_list,
                            dst_mtu_tbl->tbl,
                            tbl_index);

                    dst_mtu_entry->is_enabled = FALSE;

                    /* update the dp tables version */
                    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

                    return(ADK_SUCCESS);
                }
            }
        }
        
        second_key = second_key*ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES;
        dest_mtu_hash_entry = &dst_mtu_tbl->hash_tbl[(second_key)];
        for(i=0; i<ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES; i++, dest_mtu_hash_entry++)
        {
            if((dest_mtu_hash_entry->hash_value == key))
            {
                /*compare vrdid & destination address */
                dst_mtu_entry = &dst_mtu_tbl->tbl[dest_mtu_hash_entry->index];

                if((vrd_id == (dst_mtu_entry)->vrd_id) && 
                   (*((uint64_t *)&ipv6_addr[0]) == 
                        *((uint64_t *)&((dst_mtu_entry)->ipv6_addr.addr[0]))) &&
                   (*((uint64_t *)&ipv6_addr[8]) == 
                        *((uint64_t *)&((dst_mtu_entry)->ipv6_addr.addr[2]))))
                {
                    /* reset the hash entry */
                    tbl_index = dest_mtu_hash_entry->index;
                    *((uint64_t *)dest_mtu_hash_entry) = 0;

                    /* move the dest mtu table entry to pending list */
                    ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(dst_mtu_tbl->pending_list,
                            dst_mtu_tbl->free_list,
                            dst_mtu_tbl->tbl,
                            tbl_index);
                    
                    dst_mtu_entry->is_enabled = FALSE;

                    /* update the dp tables version */
                    ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++;

                    return(ADK_SUCCESS);
                }
            }
        }
        dst_mtu_tbl = dst_mtu_tbl->next;
    }
    return ADK_DP_E_ERROR;
}


struct rte_acl_field_def adk_ipv6_acl_defs[NUM_FIELDS_IPV6] = {
        {
                .type = RTE_ACL_FIELD_TYPE_RANGE,
                .size = sizeof(uint8_t),
                .field_index = PROTO_FIELD_IPV6,
                .input_index = PROTO_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, proto),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = SRC1_FIELD_IPV6,
                .input_index = SRC1_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_src1),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = SRC2_FIELD_IPV6,
                .input_index = SRC2_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_src2),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = SRC3_FIELD_IPV6,
                .input_index = SRC3_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_src3),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = SRC4_FIELD_IPV6,
                .input_index = SRC4_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_src4),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = DST1_FIELD_IPV6,
                .input_index = DST1_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_dst1),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = DST2_FIELD_IPV6,
                .input_index = DST2_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_dst2),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = DST3_FIELD_IPV6,
                .input_index = DST3_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_dst3),
        },       
        {
                .type = RTE_ACL_FIELD_TYPE_MASK,
                .size = sizeof(uint32_t),
                .field_index = DST4_FIELD_IPV6,
                .input_index = DST4_FIELD_IPV6,
                .offset = offsetof(struct ipv6_7tuple, ip_dst4),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_RANGE,
                .size = sizeof(uint16_t),
                .field_index = SRCP_FIELD_IPV6,
                .input_index = L4PORTS_INPUT_IPV6,
                .offset = offsetof(struct ipv6_7tuple, port_src),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_RANGE,
                .size = sizeof(uint16_t),
                .field_index = DSTP_FIELD_IPV6,
                .input_index = L4PORTS_INPUT_IPV6,
                .offset = offsetof(struct ipv6_7tuple, port_dst),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_RANGE,
                .size = sizeof(uint16_t),
                .field_index = DSCP_FIELD_IPV6,
                .input_index = DSCP_FRAG_INPUT_IPV6,
                .offset = offsetof(struct ipv6_7tuple, dscp),
        },
        {
                .type = RTE_ACL_FIELD_TYPE_RANGE,
                .size = sizeof(uint16_t),
                .field_index = FRAG_FIELD_IPV6,
                .input_index = DSCP_FRAG_INPUT_IPV6,
                .offset = offsetof(struct ipv6_7tuple, frag_type),
        },
};

RTE_ACL_RULE_DEF(ipv6_acl_rule, NUM_FIELDS_IPV6);

adk_ipv6_dp_acl_tree_t* adk_ipv6_acl_create(adk_module_handle_t ipv6Handle, 
                            uint16_t ruleSetId, uint8_t index)
{
    /*int ret;*/
    adk_sys_handle_t sys_handle;
    char name[30];
    struct rte_acl_ctx * ctx=NULL;
    struct rte_acl_param prm;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle); 
    
    snprintf(name, sizeof(name), "ADK_IPV6_ACL%3d_%1d", ruleSetId, index);
    prm.name = name;
    prm.socket_id = SOCKET_ID_ANY;
    prm.rule_size = RTE_ACL_RULE_SZ(NUM_FIELDS_IPV6);
    prm.max_rule_num = IPV6_ACL_MAX_RULES_PER_SET;


    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_create", "creating new ACL context: %s\n", prm.name);

    ctx = rte_acl_create(&prm);
    if (ctx == NULL)
	ADK_LOG_ERROR(sys_handle, ipv6Handle,"\nFailed to create ACL context!\n");
#if 0
    ret = rte_acl_set_ctx_classify(ctx, RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "\nFailed to setup %d method for ACL context\n", RTE_ACL_CLASSIFY_SCALAR);
    }
#endif

    return((adk_ipv6_dp_acl_tree_t *)ctx);

}

adk_status_t adk_ipv6_acl_ruleset_free(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_dp_acl_tree_t *acl_tree)
{
    adk_sys_handle_t sys_handle; 
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);                         

    ADK_LOG_INFO(sys_handle, ipv6Handle, "freeing IPv6 acl ruleset \n");

    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_free", "Deleting RTE ACL context \n");
    rte_acl_free((struct rte_acl_ctx *)acl_tree);
    return ADK_SUCCESS;
}

adk_status_t adk_ipv6_acl_rule_add(adk_module_handle_t ipv6Handle, 
            adk_ipv6_dp_acl_tree_t *acl_tree,
		 	adk_ipv6_dp_acl_rule_t *aclrule,
		 	adk_ipv6_acl_val_t  acl_val,
		 	uint32_t index)
{
    adk_sys_handle_t sys_handle; 
    int rc, i;
    struct ipv6_acl_rule v;

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);                         

    ADK_MEMSET(&v, 0, sizeof(v));

    for(i=0; i<4; i++ )
    {
        v.field[SRC1_FIELD_IPV6+i].value.u32 = aclrule->sip_addr[i];
        if( ((i+1)* sizeof(uint32_t) * CHAR_BIT) <= aclrule->sip_mask)
        {    
            v.field[SRC1_FIELD_IPV6+i].mask_range.u32 = sizeof(uint32_t) * CHAR_BIT;
        }
        else
        {
            if( ((i)* sizeof(uint32_t) * CHAR_BIT) < aclrule->sip_mask)
            {
                v.field[SRC1_FIELD_IPV6+i].mask_range.u32 = 
                    aclrule->sip_mask-(i* sizeof(uint32_t)* CHAR_BIT);
            }
            else
            {
                v.field[SRC1_FIELD_IPV6+i].mask_range.u32 = 0;
            }
        }
    }


    for(i=0; i<4; i++ )
    {
        v.field[DST1_FIELD_IPV6+i].value.u32 = aclrule->dip_addr[i];
        if( ((i+1)* sizeof(uint32_t) * CHAR_BIT) <= aclrule->dip_mask)
        {    
            v.field[DST1_FIELD_IPV6+i].mask_range.u32 = sizeof(uint32_t) * CHAR_BIT;
        }
        else
        {
            if( ((i)* sizeof(uint32_t) * CHAR_BIT) < aclrule->dip_mask)
            {
                v.field[DST1_FIELD_IPV6+i].mask_range.u32 = aclrule->dip_mask-((i)* 
                                                            sizeof(uint32_t)*CHAR_BIT);
            }
            else
            {
                v.field[DST1_FIELD_IPV6+i].mask_range.u32 = 0; 
            }
        }
    }

    v.field[SRCP_FIELD_IPV6].value.u16 = aclrule->sport_begin;
    v.field[SRCP_FIELD_IPV6].mask_range.u16 = aclrule->sport_end;

    v.field[DSTP_FIELD_IPV6].value.u16 = aclrule->dport_begin;
    v.field[DSTP_FIELD_IPV6].mask_range.u16 = aclrule->dport_end;

    v.field[PROTO_FIELD_IPV6].value.u8 = aclrule->proto_begin;
    v.field[PROTO_FIELD_IPV6].mask_range.u8  = aclrule->proto_end;

    v.field[DSCP_FIELD_IPV6].value.u16 = aclrule->dscp_begin;
    v.field[DSCP_FIELD_IPV6].mask_range.u16  = aclrule->dscp_end;

    v.field[FRAG_FIELD_IPV6].value.u16 = aclrule->frag_type_begin;
    v.field[FRAG_FIELD_IPV6].mask_range.u16  = aclrule->frag_type_end;

    v.data.category_mask = 1 /*LEN2MASK(RTE_ACL_MAX_CATEGORIES)*/;
    v.data.priority = index;
    v.data.userdata = acl_val;

    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_add_rules", "Adding ACL rule with index %d \n", index);
    rc = rte_acl_add_rules((struct rte_acl_ctx *)acl_tree, (struct rte_acl_rule *)&v, 1);
    if (rc != 0) {
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
                "\nFailed to add rules into ACL context, error code: %s\n", rte_strerror(rc));
        return ADK_E_INTERNAL_ERROR;
    }

    return ADK_SUCCESS;
}

adk_status_t adk_ipv6_acl_ruleset_commit(adk_module_handle_t ipv6Handle,
        adk_ipv6_dp_acl_tree_t *acl_tree)
{
    adk_sys_handle_t sys_handle;
    struct rte_acl_config cfg;
    int ret;
    
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);                         

    ADK_MEMSET(&cfg, 0, sizeof(cfg));
    cfg.num_fields = RTE_DIM(adk_ipv6_acl_defs);
    ADK_MEMCPY(&cfg.defs, adk_ipv6_acl_defs, sizeof(adk_ipv6_acl_defs));
    cfg.num_categories = 1;
    cfg.max_size = 0;

    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_build", "Building ACL context \n");
    ret = rte_acl_build((struct rte_acl_ctx *)acl_tree, &cfg);
    if (ret != 0)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle,
        "\nFailed to build ACL context. Error:%s\n",rte_strerror(ret));
        return ADK_E_INTERNAL_ERROR;
    }
    return ADK_SUCCESS;

}

void adk_ipv6_acl_ruleset_clear(adk_module_handle_t ipv6Handle,
        adk_ipv6_dp_acl_tree_t *acl_tree)
{
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);                         

    ADK_LOG_INFO(sys_handle, ipv6Handle, "IPV6 ACL reset \n");
    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_reset_rules", "Remove all the rules of ACL context \n");
    /* Remove all the rules. Does not clear the internal run time memory */
    rte_acl_reset_rules((struct rte_acl_ctx *)acl_tree);
}

void adk_print_ipv6_acl_list(adk_module_handle_t ipv6Handle,
            adk_ipv6_dp_acl_tree_t *acl_tree)
{
    adk_sys_handle_t sys_handle;
    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);                         

    ADK_LOG_INFO(sys_handle, ipv6Handle, "\nprint_ip_acl_list called for ACL:%p\n", acl_tree);
        
    ADK_IPV6_LOG_EXT_API(ipv6Handle, "rte_acl_dump", "Dump ACL context info\n");
    rte_acl_dump((struct rte_acl_ctx *)acl_tree);
}


/*!
 * =============================================================
 * adk_ipv6_flow_cahce_statistics_get
 * @param[in]    ipv6Handle - IPv6 module handle.
 *
 *
 * @retval ADK_IPV6_SUCCESS Successful completion
 * 
 *
 * @brief  
 * This API gets the flow cache statistics 
 *
 * =============================================================
 */
adk_status_t 
adk_ipv6_flow_cache_statistics_get( adk_module_handle_t ipv6Handle ,
        adk_ipv6_dp_flow_cache_stats_t *lli_fc_stats,
        adk_ipv6_dp_flow_cache_stats_t *uli_fc_stats)
{
    adk_status_t                     retValue = ADK_IPV6_SUCCESS;
    uint32_t                         i = 0;
    adk_sys_handle_t                 sys_handle;
    adk_ipv6_info_t                  *ipv6ModuleInfo_p = NULL;
    adk_ipv6_dp_flow_cache_stats_t   *lli_stats;
    adk_ipv6_dp_flow_cache_stats_t   *uli_stats;

    /* Handle validations */
    if (!ipv6Handle)
    {
        ADK_FORCE_LOG_ERROR( "Module handle is NULL\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        return (retValue);
    }

     /*! Check if module handle is valid */
    if (!ADK_MODULE_HANDLE_VALIDATE(ipv6Handle))
    {
        ADK_FORCE_LOG_ERROR( "Invalid module handle\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        return (retValue);
    }

    if (!adk_ipv6_valid_module_handle(ipv6Handle))
    {
        ADK_FORCE_LOG_ERROR( "Module Info is NULL or cookie mismatches\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        return (retValue);
    }

    sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
    ADK_LOG_FUNC_ENTRY(sys_handle, ipv6Handle);

    /*Check if IPv6 module info is not NULL*/
    if ( !lli_fc_stats || !uli_fc_stats)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, "Invalid input parameter\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_IPV6_E_INVALID_PARAMETER);
        return (retValue);
    }
    
    ipv6ModuleInfo_p = ADK_MODULE_INFO_PTR_GET_INTERNAL(ipv6Handle);
    if (ipv6ModuleInfo_p == NULL)
    {
        ADK_LOG_ERROR(sys_handle, ipv6Handle, "Module info is NULL\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE);
        return (retValue);
    }

    ADK_MEMSET(lli_fc_stats, 0, sizeof(adk_ipv6_dp_flow_cache_stats_t));
    ADK_MEMSET(uli_fc_stats, 0, sizeof(adk_ipv6_dp_flow_cache_stats_t));

    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        lli_stats = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[i].lli_fc_stats; 
        lli_fc_stats->fc_route_hit     += lli_stats->fc_route_hit;
        lli_fc_stats->fc_term_hit      += lli_stats->fc_term_hit;        
        lli_fc_stats->fc_frag_term_hit += lli_stats->fc_frag_term_hit;
        lli_fc_stats->fc_bind_term_hit += lli_stats->fc_bind_term_hit;
        lli_fc_stats->fc_miss          += lli_stats->fc_miss;
        lli_fc_stats->fc_flows         += lli_stats->fc_flows;
        lli_fc_stats->fc_cleared       += lli_stats->fc_cleared;

        uli_stats = &ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[i].uli_fc_stats; 
        uli_fc_stats->fc_route_hit     += uli_stats->fc_route_hit;
        uli_fc_stats->fc_term_hit      += uli_stats->fc_term_hit;        
        uli_fc_stats->fc_frag_term_hit += uli_stats->fc_frag_term_hit;
        uli_fc_stats->fc_bind_term_hit += uli_stats->fc_bind_term_hit;
        uli_fc_stats->fc_miss          += uli_stats->fc_miss;
        uli_fc_stats->fc_flows         += uli_stats->fc_flows;
        uli_fc_stats->fc_cleared       += uli_stats->fc_cleared;
    }

    return retValue;
}

#if 1
adk_dp_status_t
adk_ipv6_cp_ingress_mgroup_entry_lookup(adk_module_handle_t ipv6Handle,
		adk_ipv6_dp_ingress_mgroup_table_t *ingress_mgroup_tbl,
		adk_ipv6_multicast_group_t *mcGrp_p,
		adk_handle_t *mcast_index)
{
	adk_ipv6_addr_t nw_srcIpv6Addr, nw_dstIpv6Addr;
	uint16_t size;
	uint8_t key[(ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH>>3)+1];
        adk_sys_handle_t sys_handle;

	sys_handle = ADK_MODULE_SYS_HANDLE_GET(ipv6Handle);
	nw_srcIpv6Addr.addr[0] = adk_htonl(mcGrp_p->srcAddr.addr[0]);
	nw_srcIpv6Addr.addr[1] = adk_htonl(mcGrp_p->srcAddr.addr[1]);
	nw_srcIpv6Addr.addr[2] = adk_htonl(mcGrp_p->srcAddr.addr[2]);
	nw_srcIpv6Addr.addr[3] = adk_htonl(mcGrp_p->srcAddr.addr[3]);

	nw_dstIpv6Addr.addr[0] = adk_htonl(mcGrp_p->dstAddr.addr[0]);
	nw_dstIpv6Addr.addr[1] = adk_htonl(mcGrp_p->dstAddr.addr[1]);
	nw_dstIpv6Addr.addr[2] = adk_htonl(mcGrp_p->dstAddr.addr[2]);
	nw_dstIpv6Addr.addr[3] = adk_htonl(mcGrp_p->dstAddr.addr[3]);

	/* create input patern to LPM */
	adk_ipv6_dp_ingress_mgroup_key_create(key,
			ADK_SYS_INTERFACE_ID_GET(mcGrp_p->l2InterfaceHandle),
			nw_dstIpv6Addr.addr,
			nw_srcIpv6Addr.addr,
			mcGrp_p->useSrcAddr,
			&size);
	ADK_IPV6_CP_LPM_LOOKUP(ingress_mgroup_tbl->lpm_tbl, key, size, *mcast_index);
	return ADK_SUCCESS;
}
#endif

 /*!
 * end addtogroup _ipv6Functions  IPv6 Module
 * @}
 */
