/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_internal.h"
#if 1
#ifdef ADK_RCU
#include "adk_rcu.h"
#endif
#ifdef ADK_TIMER
#include "adk_timer.h"
#endif
#include "adk_hash.h"
#include "adk_sys_dpdk.h"
#include <adk_cksum.h>
#include "adk_ipv6_dp_export.h"
#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#ifdef ADK_DEVICE_SNR
#include <adk_hos_dp.h>
#endif
#include "adk_ph_dev_dp.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_reassembly.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_flow_cache_lli.h"
#include "adk_ipv6_dp_flow_cache_uli.h"
#endif
/* global fast-path database pointer */
adk_ipv6_dp_tables_t    *adk_ipv6_dp_tables;
uint32_t adk_ipv6_global_frag_id[ADK_MAX_NUM_CORES];
uint32_t pbr_policy_set_id;

#ifdef BLOCK_COUNT_ENABLED
        uint64_t ipv6_lli_rx_p1_ts, ipv6_lli_tx_p1_ts;
        uint64_t ipv6_lli_rx_p2_ts, ipv6_lli_tx_p2_ts;
        uint64_t ipv6_uli_rx_p1_ts, ipv6_uli_tx_p1_ts;
        uint64_t ipv6_uli_rx_p2_ts, ipv6_uli_tx_p2_ts;
        uint64_t ipv6_uli_rx_ex_ts, ipv6_uli_tx_ex_ts;
#endif
 
uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES] =
      {0, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, 
       IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID,
       1, IPV6_INVALID_DSCP_ID, 2, IPV6_INVALID_DSCP_ID, 
       3, IPV6_INVALID_DSCP_ID, 4, IPV6_INVALID_DSCP_ID,
       5, IPV6_INVALID_DSCP_ID, 6, IPV6_INVALID_DSCP_ID, 
       7, IPV6_INVALID_DSCP_ID, 8, IPV6_INVALID_DSCP_ID,
       9, IPV6_INVALID_DSCP_ID, 10, IPV6_INVALID_DSCP_ID, 
       11, IPV6_INVALID_DSCP_ID, 12, IPV6_INVALID_DSCP_ID,
       13, IPV6_INVALID_DSCP_ID, 14, IPV6_INVALID_DSCP_ID, 
       15, IPV6_INVALID_DSCP_ID, 16, IPV6_INVALID_DSCP_ID,
       17, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, 
       IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, 18, IPV6_INVALID_DSCP_ID,
       19, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, 
       IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID,
       20, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, 
       IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID, IPV6_INVALID_DSCP_ID
      };

/*!
 * =============================================================
 * @internal
 *
 * adk_ipv6_dp_fn_id_init
 *
 * @brief
 *
 * This function initializes the fast-path functions
 * =============================================================
 */

/* DP init */
adk_status_t adk_ipv6_dp_fn_id_init(void)
{
    adkDestEntryFunction_g[ADK_IPV6_LLI_FN_ID]       = adk_ipv6_dp_pkt_proc_lli;
    adkDestEntryFunction_g[ADK_IPV6_ULI_FN_ID]       = adk_ipv6_dp_pkt_proc_uli;
    adkDestEntryFunction_g[ADK_IPV6_LLI_REASSEMBLY_FN_ID]  = adk_ipv6_dp_reassembly;
    adkDestEntryFunction_g[ADK_IPV6_LLI_REASS_TIMER_FN_ID] = adk_ipv6_dp_reassembly_timeout_proc;
    adkDestEntryFunction_g[ADK_IPV6_NETD_FN_ID]      = adk_ipv6_dp_pkt_proc_netd; 
    adkDestEntryFunction_g[ADK_IPV6_PACKET_CAPTURE_FN_ID] = adk_ipv6_dp_pkt_proc_packet_capture;  
                
    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_init
 *
 * @retval ADK_SUCCESS Successfully initialized IPV6 module.
 * @retval ADK_E_INVALID_PARAMETER The input parameters are
 * NULL
 *
 * @brief  Initialises the IPV6 module.
 *
 * This function initializes the ADK IPV6 datapath function indexes 
 * & global database.
 * 
 * =============================================================
 */

adk_status_t
adk_ipv6_dp_init(void)
{ 
    adk_status_t ret = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;

    /* Lookup the shared memory that gets created by control plane IPv6
     * module initialization */
    mz = rte_memzone_lookup(ADK_IPV6_SHM_MEMZONE_NAME);
    if (mz == NULL)
    {
        ADK_IPV6_DP_LOG_FATAL_ERROR( "Failed to lookup IPv6 shared memory \n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    /*update the global database ptrs */
    adk_ipv6_dp_tables = (adk_ipv6_dp_tables_t *) mz->addr;

    /* Function ID registration */
    ret = adk_ipv6_dp_fn_id_init();

    return ret;
}

/*!
 * =============================================================
 * adk_ipv6_dp_table_init()
 *
 * This function initializes the tables as needed for runtime
 * IPV6 operation.
 *
 * =============================================================
 */
adk_dp_status_t adk_ipv6_dp_table_init (
    adk_module_handle_t ipv6Handle,
    adk_ipv6_dp_tables_t *dp_pvt_p)
{
    uint16_t i, idx;
    adk_status_t ret=ADK_SUCCESS;
    /*! IPv6 interface table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_if_table.tbl,
                                    (adk_ipv6_dp_interface_entry_t *),
                                    sizeof(adk_ipv6_dp_interface_entry_t),
                                    ADK_IPV6_NO_OF_IPV6_INTERFACES + 
                                    ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_if_table, 
            ADK_IPV6_NO_OF_IPV6_INTERFACES+ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES);

/* TODO:: Need to enable based on some trigger or during IPv6 interface creation .
 * Right now Enabling IPv6 by default by setting it to FALSE. 
 */
    for(i=0; i<(ADK_IPV6_NO_OF_IPV6_INTERFACES+ ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES); i++)
    {
        dp_pvt_p->ipv6_dp_if_table.tbl[i].is_disabled = FALSE;
    }

    /*! Initialize ip-addr stats table */
    dp_pvt_p->ipv6AddrStatsTble = (adk_ipv6_dp_ip_addr_stats_t**)
	    ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),ADK_IPV6_ADDR_STATS_OBJ,1,ADK_MAX_NUM_CORES * sizeof(adk_ipv6_dp_ip_addr_stats_t), 0);
    if (dp_pvt_p->ipv6AddrStatsTble == NULL)
    {
	    return ADK_E_INTERNAL_ERROR;
    }


    /*! IPv6 statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {

        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_if_table.stats_tbl[i],
                (adk_ipv6_dp_if_stats_entry_t *),
                sizeof(adk_ipv6_dp_if_stats_entry_t),
                ADK_IPV6_NO_OF_IPV6_INTERFACES + ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES);

        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_global_stats_table[i],
                (adk_ipv6_dp_global_statistics_t *),
                sizeof(adk_ipv6_dp_global_statistics_t),
                1);
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_utility_counters[i],
                (ipv6_utility_counters_t *),
                sizeof(ipv6_utility_counters_t),
                1);
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_virtual_if_stats_table[i],
                (adk_ipv6_dp_virtual_if_statistics_t *),
                sizeof(adk_ipv6_dp_virtual_if_statistics_t),
                ADK_IPV6_NO_OF_IPV6_VIRTUAL_INTERFACES);

        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_l4counter_stats_table[i],
                (adk_ipv6_dp_l4counter_statistics_t *),
                sizeof(adk_ipv6_dp_l4counter_statistics_t),
                1);

        dp_pvt_p->ipv6AddrStatsTble[i]=(adk_ipv6_dp_ip_addr_stats_t*)
        ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),ADK_IPV6_ADDR_STATS_OBJ, 1,
        ADK_IPV6_INTF_ADDR_MAX * ADK_DP_MAX_IPV6_STATS *
        sizeof(adk_ipv6_dp_ip_addr_stats_t), 0);
        if (dp_pvt_p->ipv6AddrStatsTble[i] == NULL)
        {
            return ADK_E_INTERNAL_ERROR;
        }
    }

    dp_pvt_p->ipv6IfaceHandles = (adk_interface_handle_t *)
            ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_IF_HANDLE_OBJ, 1,
            (ADK_IPV6_NO_OF_IPV6_INTERFACES + ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES) *
            sizeof(adk_interface_handle_t), 0);
    if (dp_pvt_p->ipv6IfaceHandles == NULL)
    {
        return ADK_E_INTERNAL_ERROR;
    }

    /*! IPv6 Tunnel interface lookup table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_tnl_if_table.tbl,
            (adk_ipv6_dp_tunnel_interface_lookup_entry_t *),
            sizeof(adk_ipv6_dp_tunnel_interface_lookup_entry_t),
            ADK_IPV6_NO_OF_IPV6_TNL_INTERFACES);

    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        adk_ipv6_global_frag_id[i] = i << 24;
    }


    /*! IPv6 route table */
    ADK_IPV6_DP_LPM_TABLE_ALLOC(dp_pvt_p->ipv6_dp_route_table.lpm_tbl.lpm,
            ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH);

    /* IPV6 ACL table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_acl_table.acl_ruleset_tbl[0],
            (adk_ipv6_dp_acl_tree_t **),
            sizeof(adk_ipv6_dp_acl_tree_t*),
            ADK_IPV6_FILTER_RULE_SETS_MAX + 1);

    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_acl_table.acl_ruleset_tbl[1],
            (adk_ipv6_dp_acl_tree_t **),
            sizeof(adk_ipv6_dp_acl_tree_t*),
            ADK_IPV6_FILTER_RULE_SETS_MAX + 1);


    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_acl_table.tbl[0],
            (adk_ipv6_dp_acl_action_entry_t *),
            sizeof(adk_ipv6_dp_acl_action_entry_t),
            ADK_IPV6_NO_OF_ACL_ENTRIES + 1);
    
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_acl_table.tbl[1],
            (adk_ipv6_dp_acl_action_entry_t *),
            sizeof(adk_ipv6_dp_acl_action_entry_t),
            ADK_IPV6_NO_OF_ACL_ENTRIES + 1);
    
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_acl_table.acl_stats_tbl,
            (uint32_t *),
            sizeof(uint32_t),
            ADK_MAX_NUM_CORES * (ADK_IPV6_NO_OF_ACL_ENTRIES + 1)); 
   
    /* IPV6 PBR table */
     ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_pbr_table.tbl,
             (adk_ipv6_dp_pbr_action_entry_t *),
             sizeof(adk_ipv6_dp_pbr_action_entry_t),
             IPV6_PBR_MAX_NUM_RULES+1);

 
    /* IPV6 PBR rule statistics table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_pbr_table.pbrRuleStats,
            (uint32_t *),
            sizeof(uint32_t),
            ADK_MAX_NUM_CORES *(IPV6_PBR_MAX_NUM_RULES+1));

    /* IPV6 PBR state table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_pbr_state_table.tbl,
            (adk_ipv6_dp_pbr_state_t *),
            sizeof(adk_ipv6_dp_pbr_state_t),
            1);


    /* IPV6 VRD to PBR ruleset map table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_pbr_ruleset_map_table,
            (adk_ipv6_dp_vrd_pbr_ruleset_map_table_t *),
            sizeof(adk_ipv6_dp_vrd_pbr_ruleset_map_table_t),
            ADK_IPV6_VRD_ID_MAX);

    /* IPV6 teredo client table */
    ADK_IPV6_DP_LPM_TABLE_ALLOC(dp_pvt_p->ipv6_dp_teredo_table.lpm_tbl.lpm,
                ADK_IPV6_TEREDO_CLIENT_LOOKUP_TREE_WIDTH);

    /* IPV6 nexthop table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_nexthop_table.tbl,
            (adk_ipv6_dp_nexthop_entry_t *),
            sizeof(adk_ipv6_dp_nexthop_entry_t),
            ADK_IPV6_NO_OF_NEXTHOPS);


    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_nexthop_table,
            ADK_IPV6_NO_OF_NEXTHOPS);

    /* IPV6 Onlink table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_onlink_dest_table.hash_tbl,
            (adk_ipv6_dp_onlink_dest_hash_entry_t *),
            sizeof(adk_ipv6_dp_onlink_dest_hash_entry_t),
            (ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES*ADK_IPV6_NO_OF_ONLINK_ENTRIES));

    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_onlink_dest_table.tbl,
            (adk_ipv6_dp_onlink_dest_entry_t *),
            sizeof(adk_ipv6_dp_onlink_dest_entry_t),
            ADK_IPV6_NO_OF_ONLINK_ENTRIES);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_onlink_dest_table,
            ADK_IPV6_NO_OF_ONLINK_ENTRIES);

    dp_pvt_p->ipv6_dp_onlink_dest_table.overflow_list.head = NULL;
    dp_pvt_p->ipv6_dp_onlink_dest_table.overflow_list.count = 0;
    dp_pvt_p->ipv6_dp_onlink_dest_table.overflow_pending_list.head = NULL;
    dp_pvt_p->ipv6_dp_onlink_dest_table.overflow_pending_list.count = 0;

    /*IPV6 Reassembly table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_reassembly_table.hash_tbl,
            (adk_ipv6_dp_reassembly_hash_entry_t *),
            sizeof(adk_ipv6_dp_reassembly_hash_entry_t),
            ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES*ADK_IPV6_REASSEMBLY_HASH_ENTRIES);

    for(i=0; i<ADK_IPV6_REASSEMBLY_HASH_ENTRIES; i++)
    {
        idx = i*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;
        /* reassembly hash id based lock */
        rte_spinlock_init((rte_spinlock_t *)&dp_pvt_p->ipv6_dp_reassembly_table.hash_tbl[idx]);
    }
    dp_pvt_p->ipv6_dp_reassembly_table.tbl =
            rte_mempool_create(ADK_IPV6_DP_REASM_SESSION_POOL_TBL_NAME,
            ADK_IPV6_REASSEMBLY_MAX_SESSION,
            sizeof(adk_ipv6_dp_reassembly_entry_t),
            ADK_IPV6_DP_REASM_SESSION_POOL_CACHE_SIZE,
            0, NULL, NULL, NULL, NULL, rte_socket_id(), 0);
    if(dp_pvt_p->ipv6_dp_reassembly_table.tbl == NULL)
    {
        return ADK_E_MEM_UNAVAILABLE;
    }

    dp_pvt_p->reasmMinTout= IPV6_REASSEMBLY_TIMEOUT;
    dp_pvt_p->reasmTout   = IPV6_REASSEMBLY_TIMEOUT;
    dp_pvt_p->reasmStage1 = IPV6_REASSEMBLY_TIMEOUT;
    dp_pvt_p->reasmStage2 = IPV6_REASSEMBLY_TIMEOUT;
    dp_pvt_p->reasmStage3 = IPV6_REASSEMBLY_TIMEOUT;
    dp_pvt_p->reasmStage4 = IPV6_REASSEMBLY_TIMEOUT;

    /* By default, none of the protocols are supported during init*/
    ADK_MEMSET(&dp_pvt_p->allowedL4protos, 0x0, sizeof(dp_pvt_p->allowedL4protos));

    /* By default, none of the UDP ports are exempted from checksum 0 during init*/
    ADK_MEMSET(&dp_pvt_p->allowedUdpPorts, 0x0, sizeof(dp_pvt_p->allowedUdpPorts));

    /* IPv6 Egress attribute table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_egress_flow_attr_table.tbl,
            (adk_ipv6_dp_egress_flow_attr_entry_t *),
            sizeof(adk_ipv6_dp_egress_flow_attr_entry_t),
            ADK_IPV6_NO_OF_EGRESS_FLOW);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_egress_flow_attr_table,
            ADK_IPV6_NO_OF_EGRESS_FLOW);

    /*! IPv6 Egress association statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_egress_flow_attr_table.stats_tbl[i],
                (adk_ipv6_dp_egress_assoc_stats_entry_t *),
                sizeof(adk_ipv6_dp_egress_assoc_stats_entry_t),
                ADK_IPV6_NO_OF_EGRESS_FLOW);
    }

    /* IPV6 ingress Mcast Group table */
    ADK_IPV6_DP_LPM_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_mgroup_table.lpm_tbl.lpm,
            ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH);

    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_mgroup_table.tbl,
            (adk_ipv6_dp_ingress_mgroup_entry_t *),
            sizeof(adk_ipv6_dp_ingress_mgroup_entry_t),
            ADK_IPV6_NO_OF_MGROUPS);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_ingress_mgroup_table,
            ADK_IPV6_NO_OF_MGROUPS);

    /* IPV6 ingress association table */
    ADK_IPV6_DP_LPM_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_assoc_table.lpm_tbl.lpm,
            ADK_IPV6_INGRESS_ASSOC_FLOW_TREE_WIDTH);

    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_assoc_table.tbl,
            (adk_ipv6_dp_ingress_assoc_entry_t *),
            sizeof(adk_ipv6_dp_ingress_assoc_entry_t),
            ADK_IPV6_NO_OF_INGRESS_FLOW);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_ingress_assoc_table,
            ADK_IPV6_NO_OF_INGRESS_FLOW);

    /*! IPv6 ingress association statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_assoc_table.stats_tbl[i],
                (adk_ipv6_dp_ing_assoc_stats_entry_t *),
                sizeof(adk_ipv6_dp_ing_assoc_stats_entry_t),
                ADK_IPV6_NO_OF_INGRESS_FLOW);
    }

    /* IPV6 SIP table */
    ADK_IPV6_DP_LPM_TABLE_ALLOC(dp_pvt_p->ipv6_dp_ingress_flow_sip_table.lpm_tbl.lpm,
            ADK_IPV6_INGRESS_FLOW_SIP_TREE_WIDTH);


    /* IPv6 vrd table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_vrd_table.tbl,
            (adk_ipv6_dp_vrd_host_mode_entry_t *),
            sizeof(adk_ipv6_dp_vrd_host_mode_entry_t),
            ADK_IPV6_VRD_ID_MAX);

    /* IPv6 Exception table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_excep_table.tbl,
            (adk_ipv6_dp_excep_entry_t *),
            sizeof(adk_ipv6_dp_excep_entry_t),
            (ADK_IPV6_EXCEPTION_MAX * ADK_IPV6_EXCEPTION_DOMAIN_MAX));


    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_excep_table,
            (ADK_IPV6_EXCEPTION_MAX * ADK_IPV6_EXCEPTION_DOMAIN_MAX));

    /* IPV6 Destination Address MTU table */
    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head.hash_tbl,
            (adk_ipv6_dp_dest_mtu_hash_entry_t *),
            sizeof(adk_ipv6_dp_dest_mtu_hash_entry_t),
            (ADK_IPV6_NO_OF_DEST_MTU_COL_ENTRIES*ADK_IPV6_NO_OF_DEST_MTU_ENTRIES));

    ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head.tbl,
            (adk_ipv6_dp_dest_mtu_entry_t *),
            sizeof(adk_ipv6_dp_dest_mtu_entry_t),
            ADK_IPV6_NO_OF_DEST_MTU_ENTRIES);

    ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head,
            ADK_IPV6_NO_OF_DEST_MTU_ENTRIES);
    dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.num_table = 1;

#ifdef ADK_IPV6_FLOW_CACHE
    /* Flow cache tables */
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        /*LLI flow cache tables */
        /* allocate memory for Hash entries */
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_lli_flow_cache_table[i].hash_tbl,
                (adk_ipv6_dp_flow_cache_hash_entry_t *),
                sizeof(adk_ipv6_dp_flow_cache_hash_entry_t),
                (ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES *
                 ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES));

        /* allocate memory for flow cache table */
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_lli_flow_cache_table[i].tbl,
                (adk_ipv6_dp_lli_flow_cache_entry_t *),
                sizeof(adk_ipv6_dp_lli_flow_cache_entry_t),
                ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES);

        /* initialise the indices */
        ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_lli_flow_cache_table[i],
                ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES);

        /* ULI flow cache tables */
        /* allocate memory for Hash entries */
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_uli_flow_cache_table[i].hash_tbl,
                (adk_ipv6_dp_flow_cache_hash_entry_t *),
                sizeof(adk_ipv6_dp_flow_cache_hash_entry_t),
                (ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES*ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES));

        /* allocate memory for flow cache table */
        ADK_IPV6_DP_TABLE_ALLOC(dp_pvt_p->ipv6_dp_uli_flow_cache_table[i].tbl,
                (adk_ipv6_dp_uli_flow_cache_entry_t *),
                sizeof(adk_ipv6_dp_uli_flow_cache_entry_t),
                ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES);

        /* initialise the indices */
        ADK_IPV6_DP_TABLE_IDX_INIT(dp_pvt_p->ipv6_dp_uli_flow_cache_table[i],
                ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES);
    }
#endif
    /* Allocate & initailise the Ing Assoc Otree actions table shadow and active copies */
    dp_pvt_p->ipv6IngFlowStats = ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_IN_FLOW_TBL_OBJ, 1,
		    (ADK_MAX_NUM_CORES * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS *
		     sizeof(adk_ipv6_dp_ing_flow_stats_t)), 0);
    if(!dp_pvt_p->ipv6IngFlowStats)
    {
	    return ADK_E_INTERNAL_ERROR;
    }

    dp_pvt_p->ingClassifierActionDp_p[0] =
	    (adk_ipv6_dp_ing_classifier_action_info_t *)
	    ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
			    ADK_IPV6_ING_CLASSIFIER_RULE_ACT_OBJ, 1,
			    ((ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES + 1) *
			     sizeof(adk_ipv6_dp_ing_classifier_action_info_t)),
			    0);
    if(!dp_pvt_p->ingClassifierActionDp_p[0])
    {
	    return ADK_E_INTERNAL_ERROR;
    }
    dp_pvt_p->ingClassifierActionDp_p[1] =
	    (adk_ipv6_dp_ing_classifier_action_info_t *)
	    ADK_DP_SHM_CALLOC(ADK_MODULE_MODULEID_GET(ipv6Handle),
			    ADK_IPV6_ING_CLASSIFIER_RULE_ACT_OBJ, 1,
			    ((ADK_IPV6_INGRESS_CLASSIFIER_MAX_RULES + 1) *
			     sizeof(adk_ipv6_dp_ing_classifier_action_info_t)),
			    0);
    if(!dp_pvt_p->ingClassifierActionDp_p[1])
    {   
	    return ADK_E_INTERNAL_ERROR;
    }

    /* Allocate and initialize ingress association otree table (ingAssocOtreeInfo_p)
     *      * Note that ingClassifierActionDp_p must be initialized before calling this.
     *           */
    ret = adk_ipv6_ingress_classifier_create(ipv6Handle, &dp_pvt_p->ingClassifierInfo_p);
    if (ADK_SUCCESS != ret)
    {   
	    return ADK_E_INTERNAL_ERROR;
    }

    /*Single PBR rule set for all the interfaces*/
    ret = adk_ipv6_pbr_policy_set_create(ipv6Handle, &dp_pvt_p->adk_ipv6_pbr_policy_set_handle_t, &pbr_policy_set_id); 
    if (ADK_SUCCESS != ret)
    {   
	    ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 PBR ruleset creation failure\n");
	    return ADK_E_INTERNAL_ERROR;
    }
    /*Configure default routing table to 200 during init */
    dp_pvt_p->defaultRouteTable = ADK_IPV6_LOCAL_ROUTING_TABLE;

    ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 initialisation success \n");
    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_table_release()
 *
 * This function initializes the tables as needed for runtime
 * IPV6 operation.
 *
 * =============================================================
 */
adk_dp_status_t adk_ipv6_dp_table_release(
    adk_module_handle_t ipv6Handle,
    adk_ipv6_dp_tables_t *dp_pvt_p)
{
    adk_status_t ret = ADK_SUCCESS;
    uint16_t i;
    adk_ipv6_dp_dest_mtu_table_t        *dst_mtu_tbl, *temp_dst_mtu_tbl;
    adk_ipv6_dp_onlink_dest_table_t *onlink_dst_tbl;
    /* 
     * release resources of IPv6 DP tables 
     */

    /*! IPv6 interface table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_if_table.tbl);

    /*! IPv6 statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_if_table.stats_tbl[i]);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_global_stats_table[i]);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_utility_counters[i]);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_virtual_if_stats_table[i]);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_l4counter_stats_table[i]);
    }

    /*! IPv6 Tunnel interface lookup table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_tnl_if_table.tbl);

    /*! IPv6 route table */
    adk_rcu_distributed_wait();
    ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(dp_pvt_p->ipv6_dp_route_table.pending_list, 
                                           dp_pvt_p->ipv6_dp_route_table.list_count,
                                           adk_rcu_shm_free_cb_fn, NULL);
    ADK_IPV6_DP_LPM_TABLE_FREE(dp_pvt_p->ipv6_dp_route_table.lpm_tbl.lpm);

    /* IPV6 ACL table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_acl_table.acl_ruleset_tbl[0]);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_acl_table.acl_ruleset_tbl[1]);

    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_acl_table.tbl[0]);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_acl_table.tbl[1]);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_acl_table.acl_stats_tbl);
    
    /* IPV6 PBR table
     * TBD for DPDK PBR resources */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_pbr_table.tbl);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_pbr_table.pbrRuleStats);

    /* IPV6 PBR state table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_pbr_state_table.tbl);

    /* IPV6 VRD to PBR ruleset id table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_pbr_ruleset_map_table);

    /* IPV6 teredo client table */
    ADK_IPV6_DP_LPM_TABLE_FREE(dp_pvt_p->ipv6_dp_teredo_table.lpm_tbl.lpm);

    /* IPV6 nexthop table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_nexthop_table.tbl);

    /* IPV6 Onlink table */
     /* RCU sync invoked while deleting route table pending list */
    onlink_dst_tbl = &(dp_pvt_p->ipv6_dp_onlink_dest_table);
    ADK_IPV6_DP_CLEANUP_OVERFLOW_TBL(onlink_dst_tbl);

    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_onlink_dest_table.hash_tbl);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_onlink_dest_table.tbl);

#if 0
    /* TBD: rte_mempool_create for ipv6_dp_reassembly_table.tbl is done in DP
     * But the rte_mempool_free is trying to be done from CP. Since the rte memory is not shared between
     * CP and DP, its leading to crash. Disabling it to avoid the crash.
     * Have to be looked at later.
     */
    /*IPV6 Reassembly table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_reassembly_table.hash_tbl);
    rte_mempool_free((struct rte_mempool *)dp_pvt_p->ipv6_dp_reassembly_table.tbl);
#endif
    /* IPv6 Egress attribute table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_egress_flow_attr_table.tbl);

    /*! IPv6 Egress association statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_egress_flow_attr_table.stats_tbl[i]);
    }
  
    /* IPv6 multicast group table  */
    ADK_IPV6_DP_LPM_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_mgroup_table.lpm_tbl.lpm);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_mgroup_table.tbl);

    /* IPV6 ingress association table */
    ADK_IPV6_DP_LPM_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_assoc_table.lpm_tbl.lpm);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_assoc_table.tbl);

    /*! IPv6 ingress association statistics table */
    for(i = 0; i<ADK_MAX_NUM_CORES; i++)
    {
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_assoc_table.stats_tbl[i]);
    }
    /*! Releasing ipv6 Address stats table*/
    for (i=0; i<ADK_MAX_NUM_CORES; i++)
    {
	    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6AddrStatsTble[i]);
    }
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6AddrStatsTble);

    /*! Releasing interface handling table  */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6IfaceHandles);

    /* IPV6 SIP table */
    /* RCU sync invoked while deleting route table pending list */
    ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(dp_pvt_p->ipv6_dp_ingress_flow_sip_table.pending_list,
                                           dp_pvt_p->ipv6_dp_ingress_flow_sip_table.list_count,
                                           adk_rcu_shm_free_cb_fn, NULL);
    ADK_IPV6_DP_LPM_TABLE_FREE(dp_pvt_p->ipv6_dp_ingress_flow_sip_table.lpm_tbl.lpm);

    /* IPv6 vrd table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_vrd_table.tbl);

    /* IPv6 Exception table */
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_excep_table.tbl);
   
    /* IPv6 destination address MTU table */

    dst_mtu_tbl = dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head.next;
    while ( dst_mtu_tbl != NULL)
    {
        ADK_IPV6_DP_TABLE_FREE(dst_mtu_tbl->hash_tbl);
        ADK_IPV6_DP_TABLE_FREE(dst_mtu_tbl->tbl);
        temp_dst_mtu_tbl = dst_mtu_tbl;
        dst_mtu_tbl = dst_mtu_tbl->next;
        ADK_IPV6_DP_TABLE_FREE(temp_dst_mtu_tbl);
    }

    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head.hash_tbl);
    ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_dest_mtu_tbl_list.tbl_head.tbl);

    /*Reset the allowed proto list*/
    ADK_MEMSET(&dp_pvt_p->allowedL4protos, 0x0, sizeof(dp_pvt_p->allowedL4protos));
    /*Reset udp ports */
    ADK_MEMSET(&dp_pvt_p->allowedUdpPorts, 0x0, sizeof(dp_pvt_p->allowedUdpPorts));

    ret = adk_ipv6_pbr_policy_set_delete(ipv6Handle, dp_pvt_p->adk_ipv6_pbr_policy_set_handle_t);
    if (ADK_SUCCESS != ret)
    {
            ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 PBR ruleset creation failure\n");
            return ADK_E_INTERNAL_ERROR;
    }

#ifdef ADK_IPV6_FLOW_CACHE
    /* Flow cache tables */
    for(i=0; i<ADK_MAX_NUM_CORES; i++)
    {
        /*LLI flow cache tables */
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_lli_flow_cache_table[i].hash_tbl);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_lli_flow_cache_table[i].tbl);

        /* ULI flow cache tables */
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_uli_flow_cache_table[i].hash_tbl);
        ADK_IPV6_DP_TABLE_FREE(dp_pvt_p->ipv6_dp_uli_flow_cache_table[i].tbl);
    }
#endif

    ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 tables release success \n");
    return ADK_SUCCESS;
}

void
adk_ipv6_dp_flow_cache_monitor(void)
{
#ifdef ADK_IPV6_FLOW_CACHE
    uint8_t coreIndex = ADK_LCORE_INDEX;    
    uint16_t i,j, fc_index, prev_fc_idx, max_iter;
    adk_ipv6_dp_flow_cache_aging_info_t *lli_fc_age_flags;
    adk_ipv6_dp_lli_flow_cache_entry_t  *lli_fc_entry;
    adk_ipv6_dp_flow_cache_aging_info_t *uli_fc_age_flags;
    adk_ipv6_dp_uli_flow_cache_entry_t  *uli_fc_entry;

    if(adk_ipv6_dp_tables == NULL)
        return;

    lli_fc_age_flags = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].age_info;
    lli_fc_age_flags->time_in_msec++;
    if((lli_fc_age_flags->time_in_msec >= ADK_IPV6_FLOW_CACHE_AGING_TIMER) || 
           (!adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].free_list.count && 
                (!(lli_fc_age_flags->time_in_msec & 0xFF))))
    {
        lli_fc_age_flags->time_in_msec = 0;
        /* check threshhold for aging,
         * if occupancy is greater than 75% then check for clean up. */
        if(adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].free_list.count <=
                (ADK_IPV6_FLOW_CACHE_AGING_THRESHHOLD))
        {
            /* LLI Flow cache monitor */
            prev_fc_idx = lli_fc_age_flags->prev_fc_idx;
            max_iter = prev_fc_idx + (ADK_IPV6_FLOW_CACHE_AGING_SEG_SIZE);
            for(i=prev_fc_idx; i<max_iter; i++)
            {
                /* scan active flows */
                if(lli_fc_age_flags->active_flows[i])
                {
                    /* TODO : optimise with BFS instruction */
                    for(j = 0; j<64; j++)
                    {
                        if(lli_fc_age_flags->active_flows[i] & (1ll<<j))
                        {
                            /* check for aging_bitmap */
                            if(!(lli_fc_age_flags->aging_bitmap[i] & (1ll<<j)))
                            {
                                fc_index = (i<<6)+j;
                                /* release the flow cache entry, it is aged out */
                                lli_fc_entry = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].tbl[fc_index];
                                if(lli_fc_entry->is_enabled)
                                {
                                    adk_ipv6_dp_lli_flow_cache_entry_release(lli_fc_entry);
                                }
                                else
                                {
                                    ADK_IPV6_DP_LOG_INFO("LLI FC entry is not active !!! \n");
                                }
                            }
                        }
                    }
                    lli_fc_age_flags->aging_bitmap[i] = 0;
                }
            }

            lli_fc_age_flags->prev_fc_idx = i;
            if(i >= (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>6))
                lli_fc_age_flags->prev_fc_idx = 0;
        }
/* shall be enabled when LLI & ULI 
 * FC tables are run at diffrent epochs */
#if 0 
    }

    /* ULI Flow cache monitor, can be used when we are using*/
    uli_fc_age_flags = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].age_info;
    uli_fc_age_flags->time_in_msec++;
    if(uli_fc_age_flags->time_in_msec >= ADK_IPV6_FLOW_CACHE_AGING_TIMER)
    {
        uli_fc_age_flags->time_in_msec = 0;
#endif

        if(adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].free_list.count <=
                (ADK_IPV6_FLOW_CACHE_AGING_THRESHHOLD))
        {
            uli_fc_age_flags = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].age_info;
            prev_fc_idx = uli_fc_age_flags->prev_fc_idx;
            max_iter = prev_fc_idx + (ADK_IPV6_FLOW_CACHE_AGING_SEG_SIZE);
            for(i=prev_fc_idx; i<max_iter; i++)
            {
                /* scan active flows */
                if(uli_fc_age_flags->active_flows[i])
                {
                    /* TODO : optimise with BFS instruction */
                    for(j = 0; j<64; j++)
                    {
                        if(uli_fc_age_flags->active_flows[i] & (1ll<<j))
                        {
                            /* check for aging_bitmap */
                            if(!(uli_fc_age_flags->aging_bitmap[i] & (1ll<<j)))
                            {
                                fc_index = (i<<6)+j;
                                /* release the flow cache entry, it is aged out */
                                uli_fc_entry = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].tbl[fc_index];
                                if(uli_fc_entry->is_enabled)
                                {
                                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_fc_entry);
                                }
                                else
                                {
                                    ADK_IPV6_DP_LOG_INFO(
                                            "ULI FC entry is not active !!! \n");
                                }
                            }
                        }
                    }
                    uli_fc_age_flags->aging_bitmap[i] = 0;
                }
            }

            uli_fc_age_flags->prev_fc_idx = i;
            if(i >= (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>6)) 
                uli_fc_age_flags->prev_fc_idx = 0;
        }
    }

#endif
    return;
}

uint32_t
adk_icmpv6_phdr_cksum(adk_ipv6_hdr_t  *ipv6_hdr,
             uint32_t ipv6_len)
{
    uint32_t psd_cksum;
    struct ipv6_psd_header {
        uint32_t src_addr[4];   /* IP address of source host. */
        uint32_t dst_addr[4];   /* IP address of destination host. */
        uint32_t len;           /* L4 length. */
        uint16_t zero1;        /* zero. */
        uint8_t  zero2;         /* zero. */
        uint8_t  proto;         /* L4 protocol type. */
    } psd_hdr;

    rte_memcpy(psd_hdr.src_addr, ipv6_hdr->src_addr, 16);
    rte_memcpy(psd_hdr.dst_addr, ipv6_hdr->dst_addr, 16);
    psd_hdr.zero1 = 0;
    psd_hdr.zero2 = 0;
    psd_hdr.proto = 0x3a;
    psd_hdr.len = rte_cpu_to_be_32(ipv6_len);
    psd_cksum = adk_raw_cksum((void *)&psd_hdr, sizeof(psd_hdr));

    return(psd_cksum);
}


uint16_t
adk_ipv6_icmp6_checksum(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr, uint16_t ip_offset)
{
    uint32_t cksum=0;
    uint16_t ipv6_len, total_len=0, len;
    uint16_t ext_hdr_len = 0;
    uint8_t *buff, nb_segs = task->nb_segs;
    bool odd_len = FALSE;
    ext_hdr_len = l4_offset - ADK_IPV6_PDU_HEADER_LENGTH_BYTES - ip_offset;
    uint32_t len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = ((len_nh_hlimit>>16) - ext_hdr_len);
    len  = task->data_len - l4_offset;
    buff = rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);

    if(nb_segs > 1)
    {
        while((total_len+len) <ipv6_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len-1)&1;
            }
            else
            {
                cksum += (adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len)&1;
            }
            total_len += len;

            task = (adk_pktio_task_t *)task->next;
            len  = task->data_len;
            buff = rte_pktmbuf_mtod(task, uint8_t *);
        }
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)(buff), (ipv6_len-total_len)));
        }
        else
        {
            cksum += (adk_raw_cksum((void *)(buff), (ipv6_len-total_len)));
        }
    }
    else
    {
        cksum = adk_raw_cksum((void *)(buff), ipv6_len);
    }

    cksum += adk_icmpv6_phdr_cksum((void *)ipv6_hdr, ipv6_len);
    cksum = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);
    cksum = (~cksum) & 0xffff;
    return cksum;
}

uint32_t 
adk_ipv6_phdr_cksum(adk_ipv6_hdr_t  *ipv6_hdr,
        uint16_t udp_len)
{
    uint32_t psd_cksum;
    struct ipv6_psd_header {
        uint32_t src_addr[4];   /* IP address of source host. */
        uint32_t dst_addr[4];   /* IP address of destination host. */
        uint32_t len;           /* L4 length. */
        uint16_t zero1;        /* zero. */
        uint8_t  zero2;         /* zero. */
        uint8_t  proto;         /* L4 protocol type. */
    } psd_hdr;

    rte_memcpy(psd_hdr.src_addr, ipv6_hdr->src_addr, 16);
    rte_memcpy(psd_hdr.dst_addr, ipv6_hdr->dst_addr, 16);
    psd_hdr.zero1 = 0;
    psd_hdr.zero2 = 0;
    psd_hdr.proto = 0x11;
    psd_hdr.len = rte_cpu_to_be_32(udp_len);
    psd_cksum = adk_raw_cksum((void *)&psd_hdr, sizeof(psd_hdr));

    return(psd_cksum);
}

uint16_t
adk_ipv6_udp_checksum_validation(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr)
{
    uint32_t cksum=0;
    uint16_t udp_len, total_len=0, len;
    uint8_t *buff, nb_segs = task->nb_segs;
    bool odd_len = FALSE;
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
    struct udp_hdr *udpHdr = (struct udp_hdr *)rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);
    udp_len = adk_ntohs(udpHdr->dgram_len);
    len  = task->data_len - l4_offset;

    if(nb_segs == 1)
    {
        buff = rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);
    }
    else
    {
        /*Partial cksum available for the head fragment*/
        ADK_IPV6_DP_LOG_DEBUG("dsi_md_p->checksum=%d\n", dsi_md_p->checksum);
        cksum = rte_cpu_to_be_16(dsi_md_p->checksum) + (udpHdr->dgram_cksum);
        total_len += len;
        /*jump to next fragment/segment*/
        task = (adk_pktio_task_t *)(task->next);
        buff = rte_pktmbuf_mtod(task, uint8_t *);
        len = task->data_len;
    }

    if(nb_segs > 1)
    {
        while((total_len+len) <udp_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len-1)&1;
            }
            else
            {
                cksum += (adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len)&1;
            }
            total_len += len;

            task = (adk_pktio_task_t *)task->next;
            len  = task->data_len;
            buff = rte_pktmbuf_mtod(task, uint8_t *);
        }
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
        else
        {
            cksum += (adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
    }
    else
    {
        cksum = adk_raw_cksum((void *)(buff), udp_len);
    }

    cksum += adk_ipv6_phdr_cksum((void *)ipv6_hdr, udp_len);
    cksum = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);
    cksum = (~cksum) & 0xffff;
    return cksum;
}

uint16_t
adk_ipv6_udp_checksum_generation(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr)
{
    uint32_t cksum=0;
    uint16_t udp_len, total_len=0, len;
    uint8_t *buff, nb_segs = task->nb_segs;
    bool odd_len = FALSE;
    struct udp_hdr *udpHdr = (struct udp_hdr *)rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);
    udp_len = adk_ntohs(udpHdr->dgram_len);
    len  = task->data_len - l4_offset;
    buff = rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);

    if(nb_segs > 1)
    {
        while((total_len+len) <udp_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len-1)&1;
            }
            else
            {
                cksum += (adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len)&1;
            }
            total_len += len;

            task = (adk_pktio_task_t *)task->next;
            len  = task->data_len;
            buff = rte_pktmbuf_mtod(task, uint8_t *);
        }
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
        else
        {
            cksum += (adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
    }
    else
    {
        cksum = adk_raw_cksum((void *)(buff), udp_len);
    }

    cksum += adk_ipv6_phdr_cksum((void *)ipv6_hdr, udp_len);
    cksum = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);
    cksum = (~cksum) & 0xffff;
    return cksum;
}

adk_status_t adk_ipv6_dp_chksum_validate(adk_pktio_task_t *task,  
        adk_ipv6_frag_t frag_flag, uint8_t next_hdr, uint16_t l4_offset, uint16_t dst_port,
        adk_ipv6_hdr_t *ipv6_hdr, adk_ipv6_dp_if_stats_entry_t *if_stats,
        adk_ipv6_dp_interface_entry_t *ipv6_if_entry, bool is_reassembled) 
{
    adk_status_t ret = ADK_SUCCESS;
	adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    uint16_t pdu_length = 0, dest_port = 0, calc_cksum = 0;
    bool chksum_error = false;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_ip_addr_stats_t *ipv6_addr_stats = NULL;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p = NULL;
    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;

    if (adk_int_md_p->is_outer_ip)
    {
        ipv6_addr_stats = ipv6_if_entry->ipv6_addr_stats;
    }
    else
    {
        ipv6_addr_stats = adk_int_md_p->ipv6_addr_stats;
    }

    pdu_length = (task->pkt_len - adk_md_p->ip_offset);
   
	if (ADK_IPV6_NEXT_HDR_UDP == next_hdr)
	{    
		/* validate L4 header length */
		if((pdu_length - l4_offset) != 
				adk_ntohs(*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+4))))
		{    
			ADK_IPV6_DP_LOG_ERROR("invalid L4 length dropping the packet, "
					"received len %d L4 hdr len %d \n", (pdu_length - l4_offset),
					adk_ntohs(*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+4))));

			if((pdu_length-l4_offset) < ADK_IPV6_UDP_HDR_LEN)
			{    
				if (adk_int_md_p->is_outer_ip)
					INC_NON_NULL(if_stats, ipv6IfL4Hdrtruncated, 1);
				INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
			}    
			return(ADK_DP_E_ERROR);
		}    

		if(ADK_SYS_UDP_VALIDATE_FLAG_GET &&
				(frag_flag == ADK_IPV6_UNFRAGMENTED_PKT))
		{    
			/*Zero Checksum check for configured ports*/
			dest_port = rte_be_to_cpu_16(dst_port);
			if((!ADK_IPV6_GET_UDP_CHECKSUM(ipv6_hdr, l4_offset)) &&
					((adk_ipv6_dp_tables->allowedUdpPorts[dest_port/64] &
					  ((uint64_t)1 << (dest_port) % 64))))
			{    
				ADK_IPV6_DP_LOG_DEBUG("zero checksum allowed for the UDP port 0x%x\n", dest_port);
			}    

			if (ADK_IPV6_GET_UDP_CHECKSUM(ipv6_hdr, l4_offset))
			{    
				/* verify udp checksum */
				if(is_reassembled || unlikely(adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(task))))
				{    
					/*Checksum validation*/
					calc_cksum = adk_ipv6_udp_checksum_validation(task, (l4_offset+adk_md_p->ip_offset), ipv6_hdr);
				}    

				if (calc_cksum)
				{    
					chksum_error = true;
				}
			}
			else
			{
				chksum_error = true;
			}
			if(unlikely(chksum_error))
			{
				if (adk_int_md_p->is_outer_ip)
					INC_NON_NULL(if_stats, ipv6IfUDPCksumErrors, 1);
				INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
				ADK_IPV6_DP_LOG_ERROR("invalid UDP Checksum, dropping the"
						" received packet. rxed checksum %x  Calculated UDP checksum %x \n",
						ADK_IPV6_GET_UDP_CHECKSUM(ipv6_hdr, l4_offset),
						calc_cksum);

				/* raise an exception  */
				ret = adk_dp_send_ipv6_exception(task, 0, task->port, adk_md_p->input_intf_id,
						adk_md_p->ip_offset, 0, l4_offset, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
						ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID, coreIndex, 0);
				if(unlikely(ret != ADK_SUCCESS))
				{
					ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
							"ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID\n");
					return(ADK_DP_E_ERROR);
				}
				return(ADK_E_TASK_CONSUMED);
			}
		}
	}

	/*N O N - U D P termination*/
	/* Below line will be enabled, once/if HLP offload is supported for IP proto
	 * else if (adk_hos_is_ipproto_invalid(ADK_DSI_RX_METADATA_PTR(task)))
	 */
	else
	{
		// ADK_IPV6_DP_LOG_DEBUG("IPv6 SupportedProto HW lookup failed. Falling back to SW lookup\n");
		/* Check if its a supported l4 protocol */
		if( !(adk_ipv6_dp_tables->allowedL4protos[next_hdr/64] & ((uint64_t)1<< (next_hdr % 64))) )
		{
			ADK_IPV6_DP_LOG_ERROR("Dropping packet due to Unsupported protocol : 0x%x\n", next_hdr);
			if (adk_int_md_p->is_outer_ip)
				INC_NON_NULL(if_stats, ipv6IfUnsupportedProtoDrops, 1);
			INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
			/*TODO: Raise an exception here */
			ret = adk_dp_send_ipv6_exception(task, 0, task->port, adk_md_p->input_intf_id,
					adk_md_p->ip_offset, 0, l4_offset, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
					ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO, coreIndex, 0);
			if(unlikely(ret != ADK_SUCCESS))
			{
				ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
						"ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO\n");
				return(ADK_DP_E_ERROR);
			}
			return (ADK_E_TASK_CONSUMED);
		}
		/*ICMPv6 termination*/
		/*TODO: Below block to be removed, iff ICMP l4 validation is done by HLP
		 *Post testing this will be removed
		 */
		else if (next_hdr == ADK_IPV6_NEXT_HDR_ICMP &&
				unlikely(adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(task))))
		{
			calc_cksum = adk_ipv6_icmp6_checksum(task, (l4_offset+adk_md_p->ip_offset), ipv6_hdr,  adk_md_p->ip_offset);
			if (calc_cksum)
			{
				if (adk_int_md_p->is_outer_ip)
					INC_NON_NULL(if_stats, ipv6IfICMPCksumErrors, 1);
				if ((adk_htonl(ipv6_hdr->dst_addr[0])>>24)!= 0xff)
				{
					INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
				}
				ADK_IPV6_DP_LOG_ERROR("ICMP6 checksum validation failed,"
						"calculated checksum is %x\n", calc_cksum);
				return(ADK_DP_E_ERROR);
			}
		}
		/* L4 length validation */
		if(adk_validate_l4_length(next_hdr, (pdu_length - l4_offset)) != ADK_SUCCESS)
		{
			if (adk_int_md_p->is_outer_ip)
				INC_NON_NULL(if_stats, ipv6IfL4Hdrtruncated, 1);

			INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);

			ADK_IPV6_DP_LOG_ERROR("error: L4 Header truncated packet "
					"received\n");
			return(ADK_DP_E_ERROR);
		}
	}
	return ret;
}

void adk_ipv6_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess)
{
    /*Current worker core id */
    uint32_t coreIndex = ADK_LCORE_INDEX;
    debug_print("adk_ipv6_dp_pal_batch_cb_fn %d\n",cbEventType);

    switch(cbEventType)
    {
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER:
        case ADK_PAL_DP_BATCH_CB_EVENT_EM:
            if(sendSuccess)
            {
                uint32_t flow_id = (uintptr_t)cbParam;
                adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].txPkts++;
                ADK_IPV6_DP_LOG_DEBUG("\nadk_em_send_to_eo() success!\n");
                return;
            }
            else
            {
                ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Failed to enqueue to EM LLI stage");
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                return;
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_NETD:
            if(sendSuccess)
            {
                ADK_IPV6_DP_LOG_DEBUG("\nadk_sys_dp_pkt_send_to_linux success!\n");
                return;
            }
            else
            {
                ADK_IPV6_DP_LOG_ERROR(
                       "IP Ingress association failed to send packet to netdriver.");
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                return;
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_PH:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_PH_POLICER:
            if(cbParam != NULL)
            {   
                ADK_IPV6_DP_LOG_DEBUG("ADK_PAL_DP_BATCH_CB_EVENT_PH POLICER in adk_ipv6_dp_pal_batch_cb_fn\n");
                adk_ipv6_dp_excep_entry_t *exceptionHdl = (adk_ipv6_dp_excep_entry_t *)cbParam;
                adk_ph_dev_pal_batch_handle_return(exceptionHdl->devId, exceptionHdl->queueId,
                        sendSuccess, cbEventType);
                return;
            }
            if(!sendSuccess)
            {   
                adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
                return;
            }
            break;
        case ADK_PAL_DP_BATCH_CB_EVENT_NIC:
        case ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER:
        case ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO:
        case ADK_PAL_DP_BATCH_CB_EVENT_TIMER:
        default:
            break;
    }
}
