/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ph_dev_dp.h"

#include "adk_ipv6_dp_exception.h"

adk_dp_status_t 
adk_dp_send_ipv6_exception(adk_pktio_task_t *task, uint8_t vrd_id, uint8_t port, uint16_t if_id,
                    uint16_t ipv6_offset, uint16_t  flow_id, uint16_t parse_err_offset,
                    uint8_t __attribute__((unused)) domain, uint8_t exception_type, int coreIndex, uint16_t mtu)
{
    adk_ipv6_dp_excep_entry_t *excep_entry = NULL;
    adk_dp_status_t ret = ADK_SUCCESS;
    adk_metadata_t *adk_md_p;
    (void)mtu;
    (void)coreIndex;

    RTE_SET_USED(port);
    RTE_SET_USED(if_id);
    RTE_SET_USED(ipv6_offset);
    RTE_SET_USED(flow_id);

    ret = adk_ipv6_dp_excep_entry_lookup(exception_type, &excep_entry);
    if(ret  != ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame dropped as default action for exception \n");
        return ADK_DP_E_ERROR;
    }

    adk_md_p = ADK_METADATA_PTR(task);

    ADK_IPV6_FILL_PH_DEV_PARAMS(task, excep_entry->devId, excep_entry->queueId, 0,
            excep_entry->callback_fun_id, excep_entry->policerId, exception_type, vrd_id, ipv6_offset, 0, mtu);

    adk_ph_dev_dp_metadata_t *phMeta_p;
    adk_ipv6_ph_dev_rx_param_t *ipv6_ph_params;
    phMeta_p = (adk_ph_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,imc_param_start));
    ipv6_ph_params = (adk_ipv6_ph_dev_rx_param_t *)&phMeta_p->imc_param_start;

    switch(ipv6_ph_params->exception_type)
    {
    case ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND:
        ipv6_ph_params->exception_specific_data.routeNotFound.routeTableId = vrd_id;
        ipv6_ph_params->exception_specific_data.routeNotFound.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID:
        ipv6_ph_params->exception_specific_data.udpChksumInvalid.l4_offset = parse_err_offset;
        ipv6_ph_params->exception_specific_data.udpChksumInvalid.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO:
        ipv6_ph_params->exception_specific_data.unsupProto.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO:
        ipv6_ph_params->exception_specific_data.hopLimitZero.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_INTERFACE_FORWARDING_DISABLE:
        ipv6_ph_params->exception_specific_data.fwdDisable.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION:
        ipv6_ph_params->exception_specific_data.invalidVersion.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_FRAG_OVERLAP:
        ipv6_ph_params->exception_specific_data.overlapFragment.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED:
        ipv6_ph_params->exception_specific_data.reassTimerExceed.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_MTU_VIOLATION:
        ipv6_ph_params->exception_specific_data.mtuViolation.mtu = mtu;
        ipv6_ph_params->exception_specific_data.mtuViolation.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE:
        ipv6_ph_params->exception_specific_data.duplicateExtHeader.repeatExtHdrNextHdrOffset = parse_err_offset;
        ipv6_ph_params->exception_specific_data.duplicateExtHeader.nextHdrValue = ADK_IPV6_NEXT_HDR_HOP_BY_HOP;
        ipv6_ph_params->exception_specific_data.duplicateExtHeader.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    case ADK_IPV6_EXCEPTION_PARAM_PROBLEM:
        ipv6_ph_params->exception_specific_data.paramProbData.paramProblemOffset = parse_err_offset;
        ipv6_ph_params->exception_specific_data.paramProbData.ifaceHandle = adk_ipv6_dp_tables->ipv6IfaceHandles[if_id];
        break;
    default:
        ADK_IPV6_DP_LOG_INFO("Ipv6 Invalid Exception type \n");
    }
    ret = adk_ph_dev_dp_to_cp_send((void *)task, ADK_PAL_DP_BATCH_CB_FN_IPV6, excep_entry);

    if(ret != ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_INFO_1D(
                "failed to send excep(%d) to PH, Dropping packet \n"
                ,exception_type);
        return ret;
    }

    ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via PH \n");
    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_send_clone_exception(struct rte_mbuf *task, uint8_t vrd_id,
        uint8_t domain, uint8_t exception_type, int coreIndex, uint16_t mtu)
{
    /* send local Fdbk */
    adk_pktio_task_t *except_pkt = NULL;
    uint8_t *excep_pdu, *ipv6_hdr, *pdu;
    adk_metadata_t *adk_md_p, *excep_md_p;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
 
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    except_pkt = (adk_pktio_task_t *) rte_pktmbuf_alloc(task->pool);

    if(except_pkt != NULL)
    {
        excep_md_p = ADK_METADATA_PTR(except_pkt);
        adk_md_p = ADK_METADATA_PTR(task);
        /*copy the meta data*/
        rte_memcpy(excep_md_p, adk_md_p, sizeof(adk_metadata_t));
        excep_md_p->next_frag = NULL;
        excep_md_p->task_type = ADK_TASK_TYPE_NEW;
        ADK_PDU_HEAD_PUSH(except_pkt, pdu, 
                (uint8_t *), (excep_md_p->ip_offset+sizeof(adk_ipv6_hdr_t)));

        ADK_PDU_OFFSET_PTR_GET(except_pkt, excep_pdu, uint8_t *, excep_md_p->ip_offset);
        ADK_PDU_OFFSET_PTR_GET(task, ipv6_hdr, uint8_t *, adk_md_p->ip_offset);
        /*copy the ipv6 header */
        rte_memcpy(excep_pdu, ipv6_hdr, sizeof(adk_ipv6_hdr_t)); 

        except_pkt->port = task->port;

        /* raise an exception & forward the packet as well */
        if((adk_dp_send_ipv6_exception(except_pkt, vrd_id, 0, 0, 0, 0, 0,
                        domain, exception_type, coreIndex, mtu)) != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                    "ADK_IPV6_EXCEPTION_DST_MTU_FAIL\n");
            rte_pktmbuf_free((struct rte_mbuf *)except_pkt);
        }
    }
    else
    {
        ipv6_global_stats->ipv6MbufAllocFailures++;
        ADK_IPV6_DP_LOG_INFO_DEBUG("error: unable to allocate pktmbuf for"
                "dest MTU\n");
    }

    return ADK_SUCCESS;
}


adk_dp_status_t
adk_ipv6_dp_handle_sip_excep(adk_ipv6_dp_ingress_flow_sip_entry_t *ingress_flow_sip_entry,
        adk_pktio_task_t *task, uint8_t vrd_id, adk_ipv6_dp_if_stats_entry_t *if_in_stat)
{
    adk_dp_status_t ret;
    adk_metadata_t *adk_md_p;
    adk_md_p = ADK_METADATA_PTR(task);
    (void)vrd_id;

    /* check SIP lookup action */
    switch (ingress_flow_sip_entry->action)
    {
        case ADK_IPV6_ACTION_DROP: 

            ADK_IPV6_DP_LOG_INFO("error: IPV6 SIP is blocked \n");
            IPV6_DP_DROP_TASK(task);
            return ADK_SUCCESS;

        case ADK_IPV6_ACTION_COUNT_AND_DROP:

            /* increment statistics */
            if_in_stat->ipv6IfStatsInSrcAddrErrors++;

            /*drop the packet */ 
            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 SIP is blocked \n");
            IPV6_DP_DROP_TASK(task);
            return ADK_SUCCESS;

        case ADK_IPV6_ACTION_FORWARD_TO_CPU:

            /* increment the statistics */
            if_in_stat->ipv6IfStatsInDelivers++;

            /* send the packet to CPU */
            if(ingress_flow_sip_entry->cpuPath ==  ADK_IPV6_PROCESSING_PATH_CPU_PH)
            {
                ADK_IPV6_FILL_PH_DEV_PARAMS(task, ingress_flow_sip_entry->devId, 
                        ingress_flow_sip_entry->queueId, 0, ingress_flow_sip_entry->cbFnId,
                        ingress_flow_sip_entry->policerId, ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id, 0, 0, 0);

                ret = adk_ph_dev_dp_to_cp_send((void *)task, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
                if(ret != ADK_SUCCESS)
                {
                    ADK_IPV6_DP_LOG_INFO(
                            "failed to send packet to PH Dropping packet.\n");
                    IPV6_DP_DROP_TASK(task);
                    return(ADK_E_INTERNAL_ERROR);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via PH \n");
            }
            else
            {
                /* send packet to CPU via netD */
                /* move the buffer pointers to l2 header */
                //ADK_PDU_HEAD_PUSH(task, pdu, (uint8_t *), ADK_TASK_USER_PARAMS(task, 3));
                //ADK_TASK_USER_PARAMS(task, 7) = ADK_CONVERT_TO_TPARAMS(pdu);

                ADK_IPV6_FILL_SEND_TO_CPU_PARAMS( task, ingress_flow_sip_entry);

                /* send to netD */
                ret = adk_pal_dp_netd_send(task,ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
                if(ret != ADK_E_TASK_CONSUMED)
                {
                    /*increment the statistics */
                    IPV6_DP_DROP_TASK(task);
                    return(ADK_E_INTERNAL_ERROR);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via netD \n");
            }
           
            /* mark the packet as consumed */
            return (ADK_E_TASK_CONSUMED);

        default:
            ADK_IPV6_DP_LOG_INFO("error: invalid SIP action \n");
            IPV6_DP_DROP_TASK(task);
            return ADK_SUCCESS;
    }       

    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_handle_in_acl_excep(adk_ipv6_dp_acl_action_entry_t acl_action_tbl, 
        adk_pktio_task_t __attribute__((unused))*task, uint8_t __attribute__((unused))vrd_id, 
        adk_ipv6_dp_if_stats_entry_t  *if_stats)
{
    (void)if_stats;

    /* check ACL lookup action */
    switch (acl_action_tbl.action)
    {
        case ADK_IPV6_FILTER_ACTION_DENY:
            
            ADK_IPV6_DP_LOG_INFO("error: IPV6 ACL drop action \n");

            return ADK_E_DROP_TASK;
/* IPv6 ACL : send to host path is not supported in G3 */
#if 0 
    adk_dp_status_t ret;
    adk_metadata_t *adk_md_p;

    adk_md_p = ADK_METADATA_PTR(task);

        case ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU:

            if(acl_action_tbl.ph_bind_handle)
            {
                if(acl_action_tbl.ph_bind_handle == NULL)
                {
                    ADK_IPV6_FILL_PH_DEV_PARAMS(task, acl_action_tbl.devId, acl_action_tbl.queueId, 
                            0, acl_action_tbl.callback_fun_id, UINT_MAX, 
                            ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id, 0, 0, 0);
                    ret = adk_ph_dev_dp_to_cp_send((void *)task);
                }
                else
                {
                    ADK_IPV6_FILL_PH_PARAMS(task, task->port, adk_md_p->input_intf_id, 
                            ADK_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id, 
                            adk_md_p->ip_offset, 0, 0);


                    ret = adk_ph_dp_cpu_send((void *)task, acl_action_tbl.ph_bind_handle,
                            acl_action_tbl.callback_fun_id);
                }
                if(ret != ADK_SUCCESS)
                {
                    ADK_IPV6_DP_LOG_INFO(
                            "failed to send packet to PH Dropping packet.\n");
                    return(ret);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via PH \n");
            }
            else
            {
                ADK_IPV6_FILL_SEND_TO_CPU_PARAMS( task, acl_entry);

                /* send to netD */
                ret = adk_sys_dp_pkt_send_to_linux(task);
                if(ret != ADK_SUCCESS)
                {
                    /*increment the statistics */
                    return(ret);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via netD \n");
            }
            /* mark the packet as consumed */
            return ADK_E_TASK_CONSUMED;
#endif
        default:
            ADK_IPV6_DP_LOG_INFO("error: invalid ACL action \n");
            return(ADK_E_INTERNAL_ERROR);
    }

    return ADK_SUCCESS;
}

adk_dp_status_t
adk_ipv6_dp_handle_out_acl_excep(adk_ipv6_dp_acl_action_entry_t acl_action_tbl,
        adk_pktio_task_t __attribute__((unused))*task, uint8_t __attribute__((unused))vrd_id, 
        uint8_t  __attribute__((unused))port, uint16_t  __attribute__((unused))if_id, 
        uint16_t __attribute__((unused))ipv6_offset, uint8_t  __attribute__((unused))domain, adk_ipv6_dp_if_stats_entry_t *if_stats)
{

    (void)if_stats;

    /* check SIP lookup action */
    switch (acl_action_tbl.action)
    {
        case ADK_IPV6_FILTER_ACTION_DENY:

            ADK_IPV6_DP_LOG_INFO("error: IPV6 ACL drop action \n");

            return ADK_E_DROP_TASK;
/* IPv6 ACL : send to host path is not supported */
#if 0
    adk_dp_status_t ret;

        case ADK_IPV6_FILTER_ACTION_FORWARD_TO_CPU:

            if(acl_action_tbl.ph_bind_handle)
            {
                if(acl_action_tbl.ph_bind_handle == NULL)
                {
                    ADK_IPV6_FILL_PH_DEV_PARAMS(task, acl_action_tbl.devId, acl_action_tbl.queueId,
                            0, acl_action_tbl.callback_fun_id, UINT_MAX,
                            ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id, 0, 0, 0);

                    ret = adk_ph_dev_dp_to_cp_send((void *)task);
                }
                else
                {
                    ADK_IPV6_FILL_PH_PARAMS(task, port, if_id, domain, 
                            ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id, ipv6_offset, 0, 0);

                    ret = adk_ph_dp_cpu_send((void *)task, acl_action_tbl.ph_bind_handle,
                            acl_action_tbl.callback_fun_id);
                }
                if(ret != ADK_SUCCESS)
                {
                    ADK_IPV6_DP_LOG_INFO(
                            "failed to send packet to PH Dropping packet.\n");
                    return(ret);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via PH \n");
            }
            else
            {
                ADK_IPV6_FILL_SEND_TO_CPU_PARAMS( task, acl_entry);

                /* send packet to CPU via netD */
                ret = adk_sys_dp_pkt_send_to_linux(task);
                if(ret != ADK_SUCCESS)
                {
                    /*increment the statistics */
                    return(ret);
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("IPv6 frame terminated to CPU via netD \n");
            }

            /* mark the packet as consumed */
            return ADK_E_TASK_CONSUMED;
#endif
        default:
            ADK_IPV6_DP_LOG_INFO("error: invalid ACL action \n");
            return ADK_E_INTERNAL_ERROR;
    }       

    return ADK_SUCCESS;
}



