/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_lli.h"
#include "adk_ipv6_dp_flow_cache_lli.h"
#include "adk_ipv6_dp_acl.h"

extern uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES];

#ifdef ADK_IPV6_FLOW_CACHE
adk_dp_status_t 
adk_ipv6_dp_lli_flow_cache_entry_alloc(
        adk_ipv6_dp_flow_cache_hash_entry_t **lli_flow_cache_hash_entry,
        adk_ipv6_dp_lli_flow_cache_entry_t **flow_cache_entry,
        uint32_t hash_value)
{
    adk_ipv6_dp_lli_flow_cache_table_t *lli_flow_cache_table;
    uint16_t first_key, second_key, i, index;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    /* hash lookup */
    lli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex];
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    *lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[first_key];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, (*lli_flow_cache_hash_entry)++)
    {
        if(!(*lli_flow_cache_hash_entry)->is_enabled)
        {
            ADK_IPV6_DP_FC_RSRV_IDX(lli_flow_cache_table->free_list,
                    lli_flow_cache_table->tbl,
                    index);
            *flow_cache_entry = &lli_flow_cache_table->tbl[index];
            (*lli_flow_cache_hash_entry)->index = index;
            
            /* update the Flow cache hash info */
            (*flow_cache_entry)->hash_idx = first_key+i;

            /* set active & aging flags*/
            adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index);
            adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.active_flows, index);
            ADK_IPV6_DP_LOG_INFO_DEBUG("LLI:sucessfully allocated CoreIndex %d FC index %d !!!\n",
                    coreIndex, index);

            /* reset ingress & egress DSCP based info */
            ADK_MEMSET((void *)(&((*flow_cache_entry)->in_dscp_lkup[0])), 0, 
                    sizeof((*flow_cache_entry)->in_dscp_lkup));
            ADK_MEMSET((void *)(&((*flow_cache_entry)->eg_dscp_lkup[0])), 0, 
                    sizeof((*flow_cache_entry)->eg_dscp_lkup));

            return ADK_SUCCESS;
        }
    }

    second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) & 
        (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    *lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[second_key];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, (*lli_flow_cache_hash_entry)++)
    {
        if(!(*lli_flow_cache_hash_entry)->is_enabled)
        {
            ADK_IPV6_DP_FC_RSRV_IDX(lli_flow_cache_table->free_list,
                    lli_flow_cache_table->tbl,
                    index);
            *flow_cache_entry = &lli_flow_cache_table->tbl[index];
            (*lli_flow_cache_hash_entry)->index = index;

            /* update the Flow cache hash info */
            (*flow_cache_entry)->hash_idx = second_key+i;

            /* set active & aging flag */
            adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index);
            adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.active_flows, index);
            ADK_IPV6_DP_LOG_INFO_DEBUG("LLI:sucessfully allocated CoreIndex %d FC index %d !!!\n", 
                    coreIndex, index);

            /* reset ingress & egress DSCP based info */
            ADK_MEMSET((void *)(&((*flow_cache_entry)->in_dscp_lkup[0])), 0, 
                    sizeof((*flow_cache_entry)->in_dscp_lkup));
            ADK_MEMSET((void *)(&((*flow_cache_entry)->eg_dscp_lkup[0])), 0, 
                    sizeof((*flow_cache_entry)->eg_dscp_lkup));
            return ADK_SUCCESS;
        }
    }
    ADK_IPV6_DP_LOG_INFO_DEBUG("LLI flow-cache entry FULL !!!\n");
    return ADK_DP_E_ERROR;
}

adk_status_t 
adk_ipv6_dp_lli_flow_cache_entry_release(adk_ipv6_dp_lli_flow_cache_entry_t *lli_fc_entry)
{
    uint16_t hash_idx;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_flow_cache_hash_entry_t *fc_hash_entry;
    adk_ipv6_dp_lli_flow_cache_table_t  *lli_fc_table;
    

    /* release FC Hash entry */
    hash_idx = (lli_fc_entry->hash_idx);
    fc_hash_entry = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].hash_tbl[hash_idx];
    *((uint64_t *)fc_hash_entry) = 0;
    
    /* mark the FC entry disabled */
    lli_fc_entry->is_enabled = FALSE;
    
    /* release the FC entry index */
    lli_fc_table = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex];
    ADK_IPV6_DP_FC_RELEASE_IDX(lli_fc_table->free_list,
            lli_fc_table->tbl,
            lli_fc_entry->idx);

    /* reset active flag */
    adk_ipv6_dp_fc_bitmap_reset(lli_fc_table->age_info.active_flows, lli_fc_entry->idx);

    ADK_IPV6_DP_LOG_INFO_DEBUG("successfully released the aged out lcoreIndex %d FC entry %d \n", 
            coreIndex, lli_fc_entry->idx);

    return(ADK_SUCCESS);
}

static inline void 
adk_ipv6_dp_3tuple_lli_flow_cache_key_generate(adk_ipv6_hdr_t *ipv6_hdr, 
        uint8_t __attribute__((unused)) next_hdr, uint8_t vrd_id, uint16_t filter_ruleset_id,
        uint16_t *first_key, uint16_t *second_key, uint32_t *hash_key)
{
    uint64_t hash_value = 0;
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                                 ((((uint64_t)filter_ruleset_id)<<16)|
                                  (vrd_id<<8)));

    *hash_key = hash_value&0xFFFFFFFF;
    *first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    *second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) & 
                   (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    return;
}

static inline adk_dp_status_t 
adk_ipv6_dp_3tuple_lli_flow_cache_lookup(adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
                    adk_ipv6_hdr_t *ipv6_hdr, uint8_t next_hdr, uint8_t fc_type,
                    adk_ipv6_dp_lli_flow_cache_entry_t **flow_cache_entry, uint32_t *hash_value)
{
    uint32_t hash_key;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t first_key, second_key, i, index;
    adk_ipv6_dp_flow_cache_hash_entry_t *lli_flow_cache_hash_entry;
    adk_ipv6_dp_lli_flow_cache_entry_t      *lli_flow_cache_entry;
    adk_ipv6_dp_lli_flow_cache_table_t      *lli_flow_cache_table;

    lli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex];
    /* generate hash key */
    adk_ipv6_dp_3tuple_lli_flow_cache_key_generate(ipv6_hdr, next_hdr, ipv6_if_entry->vrd_id,
        ipv6_if_entry->ingress_ruleset, &first_key, &second_key, &hash_key);

    *hash_value = hash_key;

    /* hash lookup */
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[first_key];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, lli_flow_cache_hash_entry++)
    {
        if((lli_flow_cache_hash_entry->hash_value == hash_key))
        {
            /*compare flow cache tuples */
            index = lli_flow_cache_hash_entry->index;
            lli_flow_cache_entry = &lli_flow_cache_table->tbl[index];
  
            if((fc_type == (lli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[2]))) &&
               (ipv6_if_entry->vrd_id == (lli_flow_cache_entry)->vrd_id) &&
               (next_hdr == ((lli_flow_cache_entry)->next_hdr)) &&
               (ipv6_if_entry->ingress_ruleset == (lli_flow_cache_entry)->ing_ruleset_id))
            {
                if((lli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[ipv6_if_entry->vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index);

                    *flow_cache_entry = lli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_LLI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");

                    /* release the fc entry */
                    adk_ipv6_dp_lli_flow_cache_entry_release(lli_flow_cache_entry);

                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
   
    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[second_key];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, lli_flow_cache_hash_entry++)
    {
        if((lli_flow_cache_hash_entry->hash_value == hash_key))
        {
            /*compare flow cache tuples */
            index = lli_flow_cache_hash_entry->index;
            lli_flow_cache_entry = &lli_flow_cache_table->tbl[index];
  
            if((fc_type == (lli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[2]))) &&
               (ipv6_if_entry->vrd_id == (lli_flow_cache_entry)->vrd_id) && 
               (next_hdr == ((lli_flow_cache_entry)->next_hdr)) &&
               (ipv6_if_entry->ingress_ruleset == (lli_flow_cache_entry)->ing_ruleset_id))
            {
                if((lli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[ipv6_if_entry->vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index); 
                    *flow_cache_entry = lli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_LLI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_lli_flow_cache_entry_release(lli_flow_cache_entry);
                   
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("flow-cache entry not found for 3 tuple LLI FC\n");
    *flow_cache_entry = NULL;
    return ADK_DP_E_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_lli_flow_cache_3tuple_proc(adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
        adk_ipv6_hdr_t *ipv6_hdr, uint8_t hlimit, uint8_t next_hdr, adk_ipv6_frag_t frag_flag, uint8_t excep,
        uint16_t l4_offset, uint16_t offset_to_frag, uint32_t frag_id, uint16_t __attribute__ ((unused))src_port,
        uint16_t __attribute__ ((unused))dst_port, adk_pktio_task_t *task,
        uint8_t coreIndex, void **ringHdl, uint8_t *dst_mod_id,
        adk_pal_dp_stage_info_t *stageInfo, uint32_t *hash_value)
{
    adk_dp_status_t ret;
    adk_ipv6_dp_lli_flow_cache_entry_t *flow_cache_entry;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;
    adk_ipv6_invoke_cb_attr_t *invoke_cb_attr;
    uint8_t ipv6_dscp, dscp_id, ipv6_pcp, imc_vrd_id, remarked_dscp, profile_id, fc_type;
    adk_ipv6_dp_ing_assoc_stats_entry_t *ingress_stats;
    uint16_t eg_acl_stats_id = 0, in_acl_stats_id = 0, eg_if_id, stats_id, ipv6_len, pdu_length;
    uint32_t ver_tc_flabel;
    uint32_t len_nh_hlimit;
#if ADK_IPV6_ACL_ENABLE
    uint8_t remarked_dscp_id;
#endif
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    adk_ipv6_dp_lli_imc_data_t *ipv6_lli_imc_md_p; 
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    adk_sif_code_point_mapped_values_t ingress_remark_info={0,0};
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p;

    
    ipv6_lli_imc_md_p = (adk_ipv6_dp_lli_imc_data_t *) &adk_md_p->imc_param_start;
    ver_tc_flabel = adk_ntohl(ipv6_hdr->ver_tc_flabel);
    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_dscp = ((ver_tc_flabel&IPV6_HDR_DSCP_MASK)>> IPV6_HDR_DSCP_LOC);
    dscp_id = adk_ipv6_dscp_map[ipv6_dscp];
    pdu_length = (task->pkt_len - adk_md_p->ip_offset);

    /*
     * check for DSCP remarking 
     */
    remarked_dscp = ipv6_dscp;
    if(ipv6_if_entry->enable_remark)
    {
        profile_id = (ipv6_if_entry->dscp_profile_id)?
            ipv6_if_entry->dscp_profile_id:ipv6_lli_imc_md_p->profile_id;
        if(profile_id)
        {
            ret = adk_sif_dp_code_point_mapping_get(profile_id, 
                    ADK_SIF_DP_CODE_POINT_TYPE_DSCP, ipv6_dscp, &ingress_remark_info);
            if((ret == ADK_SUCCESS) && 
               (ingress_remark_info.dscpOut & ADK_DSCP_OUTPUT_VALID_MASK_SET))
            {
                remarked_dscp = (ingress_remark_info.dscpOut &
                                 ADK_DSCP_OUTPUT_VALUE_MASK);
            }
        }
    }
    fc_type = (frag_flag == ADK_IPV6_UNFRAGMENTED_PKT)? ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_3TUPLE:
        ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_3TUPLE_FRAGMENT;
    ret = adk_ipv6_dp_3tuple_lli_flow_cache_lookup(ipv6_if_entry,
                    ipv6_hdr,
                    next_hdr,
                    fc_type,
                    &flow_cache_entry,
                    hash_value);

    if(ret == ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: Flow cache match with action %d \n ",
                flow_cache_entry->action);
        if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->ingress_stats_id];
        switch(flow_cache_entry->action)
        {
            case ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND:
            case ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND: 
                {
                    ADK_IPV6_LLI_FC_INCR(fc_route_hit);
                    /* Hoplimit validation */
                    if((hlimit) <= 1)
                    {
                        /* incerment stats */
                        if_stats->ipv6IfStatsInHdrErrors++;

                        /*raise an exception */
                        ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port, 
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO,
                                coreIndex, 0); 
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                    "ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO \n");
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                            return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }
                    else
                    {
                        ipv6_hdr->len_nh_hlimit -= adk_htonl(1);
                    }

                    /*Update ingress statistics*/
                    if_stats->ipv6IfStatsInForwDatagrams++;

                    /*update egress_interface stats */
                    eg_if_id = flow_cache_entry->fwd_bind_attr.egress_if_id;
                    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][eg_if_id];
                    if_stats->ipv6IfStatsOutForwDatagrams++;

#ifdef ADK_HOS_OLD_IMPL
                    adk_dsi_rx_metadata_t *dsi_md_p;
                    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                    if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                    {
                        if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                                (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                        {
                            in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                    & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                            if( in_acl_stats_id != 0)
                                 adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                        }
#if ADK_IPV6_ACL_ENABLE
                        else if (flow_cache_entry->enable_ingress_acl)
                        {
                             ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                                           ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                           &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                             if ( ret != ADK_SUCCESS)
                             {
                                 if ( ret != ADK_E_TASK_CONSUMED)
                                 { 
                                     IPV6_DP_DROP_TASK(task);
                                 }
                                 if ( ret == ADK_E_INTERNAL_ERROR)
                                 {
                                     ipv6_global_stats->ipv6AclFail++;
                                 } 
                                 return ADK_IPV6_FLOW_CACHE_CONSUME;
                             }
                             if ( dscp_id != IPV6_INVALID_DSCP_ID)
                             {
                                 flow_cache_entry->in_dscp_lkup[dscp_id] =
                                     (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                             }
                        }
#endif 
                    }
                    /*check the MTU */
                    ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (len_nh_hlimit >> 16);
                    if(flow_cache_entry->fwd_bind_attr.mtu < ipv6_len)
                    {
                        /*raise an exception */
                        ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                                adk_md_p->input_intf_id,adk_md_p->ip_offset, 0, 0,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_MTU_VIOLATION,
                                coreIndex, 0); 
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                    "ADK_IPV6_EXCEPTION_MTU_VIOLATION \n");
                           
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }

                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }

                    /* get the egress interface entry */
                    ipv6_if_entry = &adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[eg_if_id];

                    /* egress packet Capture */
                    if(flow_cache_entry->eg_pktcap_enabled)
                    {
                        adk_ipv6_dp_packet_capture(task, ipv6_if_entry->vrd_id, 
                                &ipv6_if_entry->eg_pkt_capture_info, coreIndex);
                    }


                    /* increment the statistics */
                    if ((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24)== 0xFF)
                    {
                        if_stats->ipv6IfStatsOutMcastPkts++;
                        if_stats->ipv6IfStatsOutMcastOctets += pdu_length;
                    }
                    else
                    {
                        if_stats->ipv6IfStatsOutUcastPkts++;
                    }
                    if_stats->ipv6IfStatsHCOutOctets += pdu_length; 

                    if( ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND == flow_cache_entry->action) 
                    {
                        imc_vrd_id = flow_cache_entry->egress_bind_attr.outer_vrd_id;
                    }
                    else
                    {
                        imc_vrd_id = (uint8_t)flow_cache_entry->vrd_id;
                    }

                    /* get the remarked pcp*/
                    ipv6_pcp = ipv6_if_entry->dscp_pcp[remarked_dscp];

#if ADK_IPV6_ACL_ENABLE
                    remarked_dscp_id = adk_ipv6_dscp_map[remarked_dscp];

                    /* fill the IMC parameters */
                    if((remarked_dscp_id != IPV6_INVALID_DSCP_ID) &&
                            (flow_cache_entry->eg_dscp_lkup[remarked_dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                    {
                        eg_acl_stats_id = flow_cache_entry->eg_dscp_lkup[remarked_dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK;
                        if( eg_acl_stats_id != 0)
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+eg_acl_stats_id]++;
                    }
                    else
                    {
                        if (flow_cache_entry->enable_egress_acl)
                        {
                             ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                                           ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                           &eg_acl_stats_id, FALSE, ADK_TN_PORT_DOMAIN, coreIndex);
                             if ( ret != ADK_SUCCESS)
                             {
                                 if ( ret != ADK_E_TASK_CONSUMED)
                                 { 
                                     IPV6_DP_DROP_TASK(task);
                                 }
                                 if ( ret == ADK_E_INTERNAL_ERROR)
                                 {
                                     ipv6_global_stats->ipv6AclFail++;
                                 } 
                                 return ADK_IPV6_FLOW_CACHE_CONSUME;
                             }
                        }
                        if (remarked_dscp_id != IPV6_INVALID_DSCP_ID)
                        {
                             flow_cache_entry->eg_dscp_lkup[remarked_dscp_id] =
                                 (IPV6_DSCP_LKUP_ENABLE_MASK|
                                  (eg_acl_stats_id));
                        } 
                    }
#endif 
                    /* update the egress stats Id */
                    adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].flags = 
                        ADK_SYS_EGRESS_STATS_MDATA_FLAG_BYTES_CNTR;
                    adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].stats_id = 
                        ipv6_if_entry->sys_egress_stats_id;

                    /* remark the outgoing packet */ 
                    ipv6_hdr->ver_tc_flabel = adk_htonl((ver_tc_flabel & (~IPV6_HDR_DSCP_MASK))|
                            (remarked_dscp<<IPV6_HDR_DSCP_LOC));

                    adk_ipv6_dp_lli_imc_fill(adk_md_p, flow_cache_entry->dest_module_id,
                            remarked_dscp, ipv6_pcp, imc_vrd_id,
                            (len_nh_hlimit&0xFF), 
                            flow_cache_entry->fwd_bind_attr.egress_if_id, if_stats, 
                            next_hdr, src_port, dst_port, frag_flag);

                    /* update the dest funcion/flow details */ 
                    adk_md_p->func_idx  = flow_cache_entry->stageInfo.fn_id;
                    *dst_mod_id         = flow_cache_entry->dest_module_id;
                    rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));
                    adk_md_p->pdu_flow_id       = flow_cache_entry->destFlowId;
                    adk_md_p->proc_stage_idx    = 0;
                    *ringHdl = NULL;

                    ADK_IPV6_DP_LOG_INFO_DEBUG("FC: egress bind info returned \n");
                    return ADK_IPV6_FLOW_CACHE_DONE;
                }
            case ADK_IPV6_DP_LLI_FLOW_CACHE_FRAGMENT_TERMINATE:
                {
                    ADK_IPV6_LLI_FC_INCR(fc_frag_term_hit);
#ifdef ADK_HOS_OLD_IMPL
                    adk_dsi_rx_metadata_t *dsi_md_p;
                    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                    if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                    {

                    if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                            (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                    {
                        in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                        if( in_acl_stats_id != 0)
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                    }
#if ADK_IPV6_ACL_ENABLE
                    else if (flow_cache_entry->enable_ingress_acl)
                    {
                         ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                                       ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                       &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                         if ( ret != ADK_SUCCESS)
                         {
                             if ( ret != ADK_E_TASK_CONSUMED)
                             { 
                                 IPV6_DP_DROP_TASK(task);
                             }
                             if ( ret == ADK_E_INTERNAL_ERROR)
                             {
                                 ipv6_global_stats->ipv6AclFail++;
                             } 
                             return ADK_IPV6_FLOW_CACHE_CONSUME;
                         }
                         if ( dscp_id != IPV6_INVALID_DSCP_ID)
                         {
                             flow_cache_entry->in_dscp_lkup[dscp_id] =
                                 (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                         }
                    }
#endif 
                    }      
              /* check if extension header parsing errors */
                    if(excep)
                    {
                        if(excep == ADK_IPV6_HDR_ERR_EXCEP)
                        {
                            if_stats->ipv6IfStatsInHdrErrors++;
                            IPV6_DP_DROP_TASK(task);
                            return ADK_E_TASK_CONSUMED;
                        }

                        /* raise an exception  */
                        ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                                    "%d \n", excep);
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }

                    /* do reassembly if fragmented */
                    else if(frag_flag != ADK_IPV6_UNFRAGMENTED_PKT)   
                    {
                        /* increment packet req for reassembly */
                        if(flow_cache_entry->ingress_frag_term_attr.vif_flag)
                        {
                            stats_id = flow_cache_entry->ingress_frag_term_attr.vif_stats_id;
                            if_stats = (adk_ipv6_dp_if_stats_entry_t *)&
                                adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
                        }

                        if_stats->ipv6IfStatsReasmReqds++;

                        /* reassembly metadata */
                        adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
                            (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
                        reass_md_p->frag_id = frag_id;
                        reass_md_p->rulemask = 
                            flow_cache_entry->ingress_frag_term_attr.ingress_assoc_rulemask;
                        reass_md_p->offset_to_frag = offset_to_frag;
                        reass_md_p->if_stats = if_stats;    
#ifdef ADK_IPV6_FLOW_CACHE
                        reass_md_p->hash_value = *hash_value;
                        reass_md_p->fc_hit = TRUE;
                        reass_md_p->next_hdr = next_hdr;
#if ADK_IPV6_ACL_ENABLE
                        /* ACL stats id & PCP mapping update */
                        reass_md_p->in_acl_stats_id = in_acl_stats_id;
#endif
#endif

#if defined(ADK_PAL_DYNAMIC_PPL)
                        ret = adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1, 1, 
                                &adk_ipv6_dp_tables->ipv6_dp_reassmbly_stage_info,
                                (frag_id & ADK_IPV6_REASSEMBLY_HASH_MASK), 0 , ADK_TASK_TYPE_FWD);
                        if(ret != 1)
                        {
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;
#else 
                        ret = adk_ipv6_dp_reassembly((void **)task, 1, 1, NULL, NULL, coreIndex);
                        if(ret == ADK_E_TASK_CONSUMED)
                            return ADK_IPV6_FLOW_CACHE_CONSUME;
#endif
                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: fragment sent for reassembly processing\n");
                    }

                    return (ADK_IPV6_FLOW_CACHE_SLOW_PATH);
                }
            case ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_BIND:
            case ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE:
                {
                    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
                    /* NO in-ACL LKUP for reassembled packets */
                    if(!adk_int_md_p->is_reassembled)
                    {
#ifdef ADK_HOS_OLD_IMPL
                        adk_dsi_rx_metadata_t *dsi_md_p;
                        dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                        if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                        {
                            if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                                    (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                            {
                                in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                    & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                                if( in_acl_stats_id != 0)
                                    adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                            }
#if ADK_IPV6_ACL_ENABLE
                            else if (flow_cache_entry->enable_ingress_acl)
                            {
                                ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                        ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                        &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                                if ( ret != ADK_SUCCESS)
                                {
                                    if ( ret != ADK_E_TASK_CONSUMED)
                                    { 
                                        IPV6_DP_DROP_TASK(task);
                                    }
                                    if ( ret == ADK_E_INTERNAL_ERROR)
                                    {
                                        ipv6_global_stats->ipv6AclFail++;
                                    } 
                                    return ADK_IPV6_FLOW_CACHE_CONSUME;
                                }
                                if ( dscp_id != IPV6_INVALID_DSCP_ID)
                                {
                                    flow_cache_entry->in_dscp_lkup[dscp_id] =
                                        (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                                }

                            }
#endif 
                        }
                    }
                    /* check if extension header parsing errors */
                    if(excep)
                    {
                        if(excep == ADK_IPV6_HDR_ERR_EXCEP)
                        {
                            if_stats->ipv6IfStatsInHdrErrors++;
                            IPV6_DP_DROP_TASK(task);
                            return ADK_E_TASK_CONSUMED;
                        }

                        /* raise an exception  */
                        ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port, 
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset, 
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                                    "%d \n", excep);
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;

                    }

                    /* Not included in the 3 tuple hash comparision because the 
                     * fragmented packet next header may not be same for all the fragments */
                    if(flow_cache_entry->next_hdr !=  next_hdr)
                    {
                        return (ADK_IPV6_FLOW_CACHE_SLOW_PATH);
                    }

                    /* Update statistics */
                    stats_id = flow_cache_entry->ingress_bind_attr.ingress_flow_statsid;
                    ingress_stats = &adk_ipv6_dp_tables->ipv6_dp_ingress_assoc_table.stats_tbl[coreIndex][stats_id];
                    ingress_stats->incoming++;
                    
                    /*check if vif flag enabled */
                    if(flow_cache_entry->ingress_bind_attr.vif_flag)
                    {
                        stats_id = flow_cache_entry->ingress_bind_attr.vif_stats_id;
                        if_stats = (adk_ipv6_dp_if_stats_entry_t *)&
                            adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
                    }

                    if( flow_cache_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE)
                    {
                        ADK_IPV6_LLI_FC_INCR(fc_term_hit);
                        invoke_cb_attr = &flow_cache_entry->ingress_term_attr.invoke_cb_attr;

                        /*check for interface filtering */
                        if(invoke_cb_attr->if_filter_info.is_filter_enabled)
                        {
                            if(invoke_cb_attr->if_filter_info.interface_id != 
                                ipv6_if_entry->ingress_stats_id)
                            {
                                ipv6_global_stats->ifFilterDrops++;
                                return ADK_IPV6_FLOW_CACHE_ERROR;
                            }
                        }

                        /* increment stats */
                        if_stats->ipv6IfStatsInDelivers++;

#ifdef ADK_PAL_CYCLE_MEASUREMENT
                        /*tx time stamp and generate the time/cycle diff*/
                        adk_sys_dp_proc_cycle_measure(task);
#endif
                        invoke_cb_attr->recvCb(invoke_cb_attr->id, task,
                                    invoke_cb_attr->user_data);
                        
                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: 3 tuple FC flow packet sent to FPS \n");
                        return (ADK_IPV6_FLOW_CACHE_CONSUME);
                    }
                    else
                    {
                        ADK_IPV6_LLI_FC_INCR(fc_bind_term_hit);
                        /* fill the IMC parameters */
                        adk_ipv6_dp_lli_imc_fill(adk_md_p, flow_cache_entry->dest_module_id,
                                remarked_dscp, 0, (uint8_t)flow_cache_entry->vrd_id,
                                (len_nh_hlimit&0xFF),
                                flow_cache_entry->ingress_bind_attr.tunnel_if_id, if_stats,
                                next_hdr, src_port, dst_port, frag_flag);

                        /* update the dest funcion/flow details */ 
                        adk_md_p->func_idx  = flow_cache_entry->stageInfo.fn_id;
                        *dst_mod_id         = flow_cache_entry->dest_module_id;
                        rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));
                        adk_md_p->pdu_flow_id    = flow_cache_entry->destFlowId;
                        adk_md_p->proc_stage_idx = 0;
                        *ringHdl = NULL;
                        
                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: 3 tuple FC returned ingress bind info \n");
                        return ADK_IPV6_FLOW_CACHE_DONE;        
                    }
                }
            default:
                {
                    ADK_IPV6_DP_LOG_INFO_DEBUG("FC: invalid 3tuple FC action \n");
                    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
                }
        }
    }

    ADK_IPV6_LLI_FC_INCR(fc_miss);
    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}

static inline void 
adk_ipv6_dp_5tuple_flow_cache_key_generate(adk_ipv6_hdr_t *ipv6_hdr,
        uint8_t next_hdr, uint16_t src_port, uint16_t dst_port, uint8_t vrd_id, 
        uint16_t filter_ruleset_id, uint16_t *first_key, uint16_t *second_key, uint32_t *hash_key)
{
    uint64_t hash_value = 0;

    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                                 (((uint64_t)src_port <<48)|
                                  ((uint64_t)dst_port <<32)|
                                  ((uint64_t)filter_ruleset_id<<16)|
                                  ((uint64_t)vrd_id<<8)|
                                  next_hdr));

    *hash_key = hash_value & 0xFFFFFFFF;
    *first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    *second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) & 
                   (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    return;
}


static inline adk_dp_status_t 
adk_ipv6_dp_5tuple_lli_flow_cache_lookup(adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
                   adk_ipv6_hdr_t *ipv6_hdr, uint8_t next_hdr, uint8_t fc_type, uint16_t src_port, uint16_t dst_port,
                   adk_ipv6_dp_lli_flow_cache_entry_t **flow_cache_entry, uint32_t *hash_value)
{
    uint32_t hash_key;
    uint16_t first_key, second_key, i, index;
    uint8_t  coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_flow_cache_hash_entry_t *lli_flow_cache_hash_entry;
    adk_ipv6_dp_lli_flow_cache_entry_t      *lli_flow_cache_entry;
    adk_ipv6_dp_lli_flow_cache_table_t      *lli_flow_cache_table;

    lli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex];

    /* generate hash key */
    adk_ipv6_dp_5tuple_flow_cache_key_generate(ipv6_hdr, next_hdr, src_port, dst_port, 
         ipv6_if_entry->vrd_id, ipv6_if_entry->ingress_ruleset, &first_key, &second_key, &hash_key);

    *hash_value = hash_key;

    /* hash lookup */
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[first_key];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, lli_flow_cache_hash_entry++)
    {
        if((lli_flow_cache_hash_entry->hash_value == hash_key))
        {
            /*compare flow cache tuples */
            index = lli_flow_cache_hash_entry->index;
            lli_flow_cache_entry = &lli_flow_cache_table->tbl[index];

            if((fc_type == (lli_flow_cache_entry)->fc_type) &&
                    (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                        *((uint64_t *)&((lli_flow_cache_entry)->src_addr[0]))) &&
                    (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                     *((uint64_t *)&((lli_flow_cache_entry)->src_addr[2]))) &&
                    (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                     *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[0]))) &&
                    (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                     *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[2]))) &&
                    (src_port == (lli_flow_cache_entry)->src_port) &&
                    (dst_port == (lli_flow_cache_entry)->dst_port) &&
                    (next_hdr == (lli_flow_cache_entry)->next_hdr) &&
                    (ipv6_if_entry->vrd_id == (lli_flow_cache_entry)->vrd_id) &&
                    (ipv6_if_entry->ingress_ruleset == (lli_flow_cache_entry)->ing_ruleset_id))
            {
                if((lli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[ipv6_if_entry->vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index);
                    *flow_cache_entry = lli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_LLI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_lli_flow_cache_entry_release(lli_flow_cache_entry);
                   
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
   
    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    lli_flow_cache_hash_entry = &lli_flow_cache_table->hash_tbl[second_key];   
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, lli_flow_cache_hash_entry++)
    {
        if((lli_flow_cache_hash_entry->hash_value == hash_key))
        {
            /*compare flow cache tuples */
            index = lli_flow_cache_hash_entry->index;
            lli_flow_cache_entry = &lli_flow_cache_table->tbl[index];
  
            if((fc_type == (lli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((lli_flow_cache_entry)->dst_addr[2]))) &&
               (src_port == (lli_flow_cache_entry)->src_port) &&
               (dst_port == (lli_flow_cache_entry)->dst_port) &&
               (next_hdr == (lli_flow_cache_entry)->next_hdr) &&
               (ipv6_if_entry->vrd_id == (lli_flow_cache_entry)->vrd_id) && 
               (ipv6_if_entry->ingress_ruleset == (lli_flow_cache_entry)->ing_ruleset_id))
            {
                if((lli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[ipv6_if_entry->vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(lli_flow_cache_table->age_info.aging_bitmap, index);
                    *flow_cache_entry = lli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_LLI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_lli_flow_cache_entry_release(lli_flow_cache_entry);
                   
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("flow-cache entry not found for 5 tuple LLI FC\n");
    *flow_cache_entry = NULL;
    return ADK_DP_E_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_lli_flow_cache_5tuple_proc(adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
                    adk_ipv6_hdr_t *ipv6_hdr, uint8_t hlimit, uint8_t next_hdr, 
                    adk_ipv6_frag_t frag_flag, uint16_t src_port,
                    uint16_t dst_port, uint8_t excep, uint16_t l4_offset, 
                    uint16_t offset_to_frag, uint32_t frag_id, adk_pktio_task_t *task, 
                    uint8_t coreIndex, void **ringHdl, uint8_t *dst_mod_id, 
                    adk_pal_dp_stage_info_t *stageInfo, uint32_t *hash_value)
{
    adk_dp_status_t ret;
    adk_ipv6_dp_lli_flow_cache_entry_t *flow_cache_entry;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR((task));
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats;
    adk_ipv6_dp_ing_assoc_stats_entry_t *ingress_stats;
    adk_ipv6_invoke_cb_attr_t *invoke_cb_attr;
    uint8_t ipv6_dscp, dscp_id, ipv6_pcp, imc_vrd_id, remarked_dscp, profile_id, fc_type; 
    uint16_t pdu_length, eg_if_id, stats_id, ipv6_len;
    uint16_t eg_acl_stats_id = 0, in_acl_stats_id = 0;
    uint32_t ver_tc_flabel;
    adk_ipv6_dp_lli_imc_data_t *ipv6_lli_imc_md_p; 
    adk_sif_code_point_mapped_values_t ingress_remark_info={0,0};
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p;
    uint32_t  len_nh_hlimit;

#if ADK_IPV6_ACL_ENABLE
    uint8_t remarked_dscp_id;
#endif
#ifdef UDP_CHECKSUM_ENABLED
    uint16_t calc_cksum;
#endif
    /* 
     * Check for Policing remark
     */
    if(ipv6_if_entry->enable_policing)
    {
        ADK_IPV6_DP_LOG_INFO("IPv6 policing is not enabled yet \n ");
        return ADK_IPV6_FLOW_CACHE_ERROR;
    }

    ipv6_lli_imc_md_p = (adk_ipv6_dp_lli_imc_data_t *) &adk_md_p->imc_param_start;
    ver_tc_flabel = adk_ntohl(ipv6_hdr->ver_tc_flabel);
    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_dscp = ((ver_tc_flabel&IPV6_HDR_DSCP_MASK)>>IPV6_HDR_DSCP_LOC);
    dscp_id = adk_ipv6_dscp_map[ipv6_dscp];
    pdu_length = (task->pkt_len - adk_md_p->ip_offset);    
    /*
     * check for DSCP remarking 
     */
    remarked_dscp = ipv6_dscp;
    if(ipv6_if_entry->enable_remark)
    {
        profile_id = (ipv6_if_entry->dscp_profile_id)?
            ipv6_if_entry->dscp_profile_id:ipv6_lli_imc_md_p->profile_id;
        if(profile_id)
        {
            ret = adk_sif_dp_code_point_mapping_get(profile_id, 
                    ADK_SIF_DP_CODE_POINT_TYPE_DSCP, ipv6_dscp, &ingress_remark_info);
            if((ret == ADK_SUCCESS) && 
               (ingress_remark_info.dscpOut & ADK_DSCP_OUTPUT_VALID_MASK_SET))
            {
                remarked_dscp = (ingress_remark_info.dscpOut &
                                 ADK_DSCP_OUTPUT_VALUE_MASK);
            }
        }
    }

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    fc_type = (frag_flag == ADK_IPV6_UNFRAGMENTED_PKT)? ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_5TUPLE:
        ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_5TUPLE_FRAGMENT;
    ret = adk_ipv6_dp_5tuple_lli_flow_cache_lookup(ipv6_if_entry, ipv6_hdr, next_hdr, fc_type, 
            src_port, dst_port, &flow_cache_entry, hash_value);
    if(ret == ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: Flow cache match with action %d \n ", 
                flow_cache_entry->action);
        if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->ingress_stats_id];
        switch(flow_cache_entry->action)
        {
            case ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND:
            case ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND: 
                {
                    ADK_IPV6_LLI_FC_INCR(fc_route_hit);
                    eg_if_id = flow_cache_entry->fwd_bind_attr.egress_if_id;
                    /* Hoplimit validation */
                    if((hlimit) <= 1)
                    {
                        /* incerment stats */
                        if_stats->ipv6IfStatsInHdrErrors++;

                        /*raise an exception */
                        ret = adk_dp_send_ipv6_exception((task), ipv6_if_entry->vrd_id,(task)->port,
                               adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, 
                               ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO,
                                coreIndex, 0); 
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                    "ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO \n");
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }
                    else
                    {
                        ipv6_hdr->len_nh_hlimit -= adk_htonl(1);
                    }

                    /*Update ingress statistics*/
                    if_stats->ipv6IfStatsInForwDatagrams++;

                    /*update egress_interface stats */
                    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][eg_if_id];
                    if_stats->ipv6IfStatsOutForwDatagrams++;

#ifdef ADK_HOS_OLD_IMPL 
                    adk_dsi_rx_metadata_t *dsi_md_p;
                    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                    if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                    {
                    if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                            (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                    {
                        in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                        if( in_acl_stats_id != 0)
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                    }
#if ADK_IPV6_ACL_ENABLE
                    else if (flow_cache_entry->enable_ingress_acl)
                    {
                         ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                                       ipv6_dscp, frag_flag, ipv6_hdr, src_port, dst_port,  ipv6_if_entry->vrd_id, 
                                                       &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                         if ( ret != ADK_SUCCESS)
                         {
                             if ( ret != ADK_E_TASK_CONSUMED)
                             { 
                                 IPV6_DP_DROP_TASK(task);
                             }
                             if ( ret == ADK_E_INTERNAL_ERROR)
                             {
                                 ipv6_global_stats->ipv6AclFail++;
                             } 
                             return ADK_IPV6_FLOW_CACHE_CONSUME;
                         }
                         if ( dscp_id != IPV6_INVALID_DSCP_ID)
                         {
                             flow_cache_entry->in_dscp_lkup[dscp_id] =
                                 (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                         }

                    }
#endif 
                    }
                    /*check the MTU */
                    ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (len_nh_hlimit >> 16);
                    if(flow_cache_entry->fwd_bind_attr.mtu < ipv6_len)
                    {
                        /*raise an exception */
                        ret = adk_dp_send_ipv6_exception((task), ipv6_if_entry->vrd_id,(task)->port,
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_MTU_VIOLATION,
                                coreIndex, 0); 
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                    "ADK_IPV6_EXCEPTION_MTU_VIOLATION \n");
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }

                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }

                    /* get the egress if entry */
                    ipv6_if_entry = &adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[eg_if_id];
                    /* egress packet Capture */
                    if(flow_cache_entry->eg_pktcap_enabled)
                    {
                        adk_ipv6_dp_packet_capture(task, ipv6_if_entry->vrd_id, 
                                &ipv6_if_entry->eg_pkt_capture_info, coreIndex);
                    }

                    /* increment the statistics */
                    if((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xFF)
                    {
                        if_stats->ipv6IfStatsOutMcastPkts++;
                        if_stats->ipv6IfStatsOutMcastOctets += pdu_length;
                    }
                    else
                    {
                        if_stats->ipv6IfStatsOutUcastPkts++;
                    }
                    if_stats->ipv6IfStatsHCOutOctets += pdu_length; 

                    if( ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND == flow_cache_entry->action)
                    {
                        imc_vrd_id = flow_cache_entry->egress_bind_attr.outer_vrd_id;
                    }
                    else
                    {
                        imc_vrd_id = (uint8_t)flow_cache_entry->vrd_id;
                    }

                    /* get the remarked dscp to pcp map */
                    ipv6_pcp = ipv6_if_entry->dscp_pcp[remarked_dscp];

#if ADK_IPV6_ACL_ENABLE
                    remarked_dscp_id = adk_ipv6_dscp_map[remarked_dscp];

                    /* fill the IMC parameters */
                    if((remarked_dscp_id != IPV6_INVALID_DSCP_ID) &&
                            (flow_cache_entry->eg_dscp_lkup[remarked_dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                    {
                        eg_acl_stats_id = flow_cache_entry->eg_dscp_lkup[remarked_dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK;
                        if( eg_acl_stats_id != 0)
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+eg_acl_stats_id]++;
                    }
                    else
                    {
                        /* Do ACL lookup */
                        if (flow_cache_entry->enable_egress_acl)
                        {
                             ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry,next_hdr,
                                                           remarked_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                           &eg_acl_stats_id, FALSE, ADK_TN_PORT_DOMAIN, coreIndex);
                             if ( ret != ADK_SUCCESS)
                             {
                                 if ( ret != ADK_E_TASK_CONSUMED)
                                 { 
                                     IPV6_DP_DROP_TASK(task);
                                 }
                                 if ( ret == ADK_E_INTERNAL_ERROR)
                                 {
                                     ipv6_global_stats->ipv6AclFail++;
                                 } 
                                 return ADK_IPV6_FLOW_CACHE_CONSUME;
                             }

                        }
                        if (remarked_dscp_id != IPV6_INVALID_DSCP_ID)
                        {
                             flow_cache_entry->eg_dscp_lkup[remarked_dscp_id] =
                                 (IPV6_DSCP_LKUP_ENABLE_MASK|
                                  (eg_acl_stats_id));
                        } 
                    }
#endif 

                    /* update the egress stats Id */
                    adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].flags = 
                        ADK_SYS_EGRESS_STATS_MDATA_FLAG_BYTES_CNTR;
                    adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].stats_id = 
                        ipv6_if_entry->sys_egress_stats_id;

                    /* remark the outgoing packet */ 
                    ipv6_hdr->ver_tc_flabel = adk_htonl((ver_tc_flabel & (~IPV6_HDR_DSCP_MASK))|
                            (remarked_dscp<<IPV6_HDR_DSCP_LOC));


                    adk_ipv6_dp_lli_imc_fill(adk_md_p, flow_cache_entry->dest_module_id,
                            remarked_dscp, ipv6_pcp, imc_vrd_id,
                            ((adk_ntohl(ipv6_hdr->len_nh_hlimit))&0xFF), 
                            flow_cache_entry->fwd_bind_attr.egress_if_id, if_stats,
                            next_hdr, src_port, dst_port, frag_flag);

                    /* update the dest funcion/flow details */ 
                    adk_md_p->func_idx  = flow_cache_entry->stageInfo.fn_id;
                    *dst_mod_id         = flow_cache_entry->dest_module_id;
                    rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));
                    adk_md_p->pdu_flow_id    = flow_cache_entry->destFlowId;
                    adk_md_p->proc_stage_idx = 0;
                    *ringHdl = NULL;
                    
                    ADK_IPV6_DP_LOG_INFO_DEBUG("FC: egress bind info returned \n");

                    return ADK_IPV6_FLOW_CACHE_DONE;
                }
            case ADK_IPV6_DP_LLI_FLOW_CACHE_FRAGMENT_TERMINATE:
                {
                    ADK_IPV6_LLI_FC_INCR(fc_frag_term_hit);
#ifdef ADK_HOS_OLD_IMPL 
                    adk_dsi_rx_metadata_t *dsi_md_p;
                    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                    if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                    {

                    if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                            (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                    {
                        in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                        if( in_acl_stats_id != 0)
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                    }
#if ADK_IPV6_ACL_ENABLE
                    else if (flow_cache_entry->enable_ingress_acl)
                    {
                         ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry, next_hdr,
                                                       ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                       &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                         if ( ret != ADK_SUCCESS)
                         {
                             if ( ret != ADK_E_TASK_CONSUMED)
                             { 
                                 IPV6_DP_DROP_TASK(task);
                             }
                             if ( ret == ADK_E_INTERNAL_ERROR)
                             {
                                 ipv6_global_stats->ipv6AclFail++;
                             } 
                             return ADK_IPV6_FLOW_CACHE_CONSUME;
                         }
                         if ( dscp_id != IPV6_INVALID_DSCP_ID)
                         {
                             flow_cache_entry->in_dscp_lkup[dscp_id] =
                                 (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                         }
                    }
#endif 
                    }      
        
                    /* check if extension header parsing errors */
                    if(excep)
                    {
                        if(excep == ADK_IPV6_HDR_ERR_EXCEP)
                        {
                            if_stats->ipv6IfStatsInHdrErrors++;
                            IPV6_DP_DROP_TASK(task);
                            return ADK_E_TASK_CONSUMED;
                        }

                        /* raise an exception  */
                        ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                                    "%d \n", excep);
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }

                    /* do reassembly if fragmented */
                    else if(frag_flag != ADK_IPV6_UNFRAGMENTED_PKT)   
                    {
                        /* increment packet req for reassembly */
                        if(flow_cache_entry->ingress_frag_term_attr.vif_flag)
                        {
                            stats_id = flow_cache_entry->ingress_frag_term_attr.vif_stats_id;
                            if_stats = (adk_ipv6_dp_if_stats_entry_t *)&
                                adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
                        }

                        if_stats->ipv6IfStatsReasmReqds++;

                        /* reassembly metadata */
                        adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
                            (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
                        reass_md_p->frag_id = frag_id;
                        reass_md_p->rulemask = 
                            flow_cache_entry->ingress_frag_term_attr.ingress_assoc_rulemask;
                        reass_md_p->offset_to_frag = offset_to_frag;
                        reass_md_p->if_stats = if_stats;    
#ifdef ADK_IPV6_FLOW_CACHE
                        reass_md_p->hash_value = *hash_value;
                        reass_md_p->fc_hit = TRUE;
                        reass_md_p->next_hdr = next_hdr;
                        reass_md_p->src_port = src_port;
                        reass_md_p->dst_port = dst_port;
#if ADK_IPV6_ACL_ENABLE
                        /* ACL stats id & PCP mapping update */
                        reass_md_p->in_acl_stats_id = in_acl_stats_id;
#endif
#endif

#if defined(ADK_PAL_DYNAMIC_PPL)
                        ret = adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1, 1, 
                                &adk_ipv6_dp_tables->ipv6_dp_reassmbly_stage_info,
                                (frag_id & ADK_IPV6_REASSEMBLY_HASH_MASK), 0 , ADK_TASK_TYPE_FWD);
                        if(ret != 1)
                        {
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                        return ADK_IPV6_FLOW_CACHE_CONSUME;
#else 
                        ret = adk_ipv6_dp_reassembly((void **)task, 1, 1, NULL, NULL, coreIndex);
                        if(ret == ADK_E_TASK_CONSUMED)
                            return ADK_IPV6_FLOW_CACHE_CONSUME;
#endif
                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: fragment sent for reassembly processing\n");
                    }

                    return (ADK_IPV6_FLOW_CACHE_SLOW_PATH);
                }
            case ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_BIND:
            case ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE:
                {
                    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
                    /* NO in-ACL LKUP for reassembled packets */
                    if(!adk_int_md_p->is_reassembled)
                    {
#ifdef ADK_HOS_OLD_IMPL
                        adk_dsi_rx_metadata_t *dsi_md_p;
                        dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
                        if(!adk_hos_is_acl_valid(dsi_md_p))
#endif
                        {
                            if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                                    (flow_cache_entry->in_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                            {
                                in_acl_stats_id = flow_cache_entry->in_dscp_lkup[dscp_id]
                                    & IPV6_DSCP_LKUP_ACL_STATSID_MASK; 
                                if( in_acl_stats_id != 0)
                                    adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+in_acl_stats_id]++;
                            }
#if ADK_IPV6_ACL_ENABLE
                            else if (flow_cache_entry->enable_ingress_acl)
                            {
                                ret = adk_ipv6_acl_lookup_func( task, ipv6_if_entry, next_hdr,
                                        ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port,ipv6_if_entry->vrd_id, 
                                        &in_acl_stats_id, TRUE, ADK_TN_PORT_DOMAIN, coreIndex);
                                if ( ret  != ADK_SUCCESS)
                                {
                                    if ( ret != ADK_E_TASK_CONSUMED)
                                    { 
                                        IPV6_DP_DROP_TASK(task);
                                    }
                                    if ( ret == ADK_E_INTERNAL_ERROR)
                                    {
                                        ipv6_global_stats->ipv6AclFail++;
                                    } 
                                    return ADK_IPV6_FLOW_CACHE_CONSUME;
                                }
                                if ( dscp_id != IPV6_INVALID_DSCP_ID)
                                {
                                    flow_cache_entry->in_dscp_lkup[dscp_id] =
                                        (IPV6_DSCP_LKUP_ENABLE_MASK|(in_acl_stats_id));
                                }

                            }
#endif 
                        }
                    }
                    /* check if extension header parsing errors */
                    if(excep)
                    {
                        if(excep == ADK_IPV6_HDR_ERR_EXCEP)
                        {
                            if_stats->ipv6IfStatsInHdrErrors++;
                            IPV6_DP_DROP_TASK(task);
                            return ADK_E_TASK_CONSUMED;
                        }

                        /* raise an exception  */
                        ret = adk_dp_send_ipv6_exception((task), ipv6_if_entry->vrd_id,(task)->port,
                                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                                ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                        if(ret != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                                    "%d \n", excep);
                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }

                        return ADK_IPV6_FLOW_CACHE_CONSUME;
                    }

                    /* Update statistics */
                    stats_id  = flow_cache_entry->ingress_bind_attr.ingress_flow_statsid;
                    ingress_stats = &adk_ipv6_dp_tables->ipv6_dp_ingress_assoc_table.stats_tbl[coreIndex][stats_id];
                    ingress_stats->incoming++;

                    /* validate L4 header */
                    if(next_hdr == ADK_IPV6_NEXT_HDR_UDP)
                    {
                        /* validate L4 header length */
                        if((pdu_length - l4_offset) != 
                                adk_ntohs(*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+4))))
                        {
                            ADK_IPV6_DP_LOG_INFO_2D_DEBUG("invalid L4 length dropping the packet, " 
                                    "received len %d l4 hdr len %d \n", (pdu_length - l4_offset),
                                    adk_ntohs(*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+4))));
                            if((pdu_length-l4_offset) < ADK_IPV6_UDP_HDR_LEN)
                            {
                                if_stats->ipv6IfL4Hdrtruncated++;
                            }

                            /* raise an exception  */
                            ret = adk_dp_send_ipv6_exception((task), ipv6_if_entry->vrd_id, 
                                    (task)->port, adk_md_p->input_intf_id, adk_md_p->ip_offset, 0,0,
                                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                                    ADK_IPV6_UDP_INGRESS_LENGTH_INVALID, coreIndex, 0);
                            if(ret != ADK_SUCCESS)
                            {
                                ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                        "ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID\n");
                                return ADK_IPV6_FLOW_CACHE_ERROR;
                            }

                            return ADK_IPV6_FLOW_CACHE_CONSUME;
                        }
                        /* verify udp checksum */
#ifdef UDP_CHECKSUM_ENABLED
#ifdef ADK_HOS_OLD_IMPL
                        calc_cksum = adk_ipv6_udp_checksum_generation_verify((task), 
                                (l4_offset+adk_md_p->ip_offset),
                                ipv6_hdr);
#else
                        calc_cksum = adk_ipv6_udp_checksum_generation((task), 
                                (l4_offset+adk_md_p->ip_offset),
                                ipv6_hdr);
#endif
                        if (calc_cksum)
                        {
                            ADK_IPV6_DP_LOG_INFO_2D_DEBUG("invalid UDP Checksum, dropping the" 
                                    " received packet. rxed checksum %x  Calculated UDP checksum %x \n", 
                                    (*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+6))),
                                    calc_cksum);

                            /* raise an exception  */
                            ret = adk_dp_send_ipv6_exception((task), ipv6_if_entry->vrd_id, 
                                    (task)->port, adk_md_p->input_intf_id, adk_md_p->ip_offset,0,0,
                                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                                    ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID, coreIndex, 0);
                            if(ret != ADK_SUCCESS)
                            {
                                ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                        "ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID\n");
                                return ADK_IPV6_FLOW_CACHE_ERROR;
                            }
                            return ADK_IPV6_FLOW_CACHE_CONSUME;
                        }
#endif
                    }
                    else
                    {
                        /* L4 length validation */
                        if(adk_validate_l4_length(next_hdr, (pdu_length - l4_offset)) != ADK_SUCCESS)
                        {
                            /* increment global stats */
                            if_stats->ipv6IfL4Hdrtruncated++;

                            return ADK_IPV6_FLOW_CACHE_ERROR;
                        }
                    }

                    if(flow_cache_entry->ingress_term_attr.vif_flag)
                    {
                        stats_id = flow_cache_entry->ingress_term_attr.vif_stats_id;
                        if_stats = (adk_ipv6_dp_if_stats_entry_t *)&
                            adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
                    }

                    if( flow_cache_entry->action == ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE)
                    {
                        ADK_IPV6_LLI_FC_INCR(fc_term_hit);
                        invoke_cb_attr = &flow_cache_entry->ingress_term_attr.invoke_cb_attr;

                        /*check for interface filtering */
                        if(invoke_cb_attr->if_filter_info.is_filter_enabled)
                        {
                            if(invoke_cb_attr->if_filter_info.interface_id != 
                                ipv6_if_entry->ingress_stats_id)
                            {
                                ipv6_global_stats->ifFilterDrops++;
                                return ADK_IPV6_FLOW_CACHE_ERROR;
                            }
                        }

                        /* increment stats */
                        if_stats->ipv6IfStatsInDelivers++;

#ifdef ADK_PAL_CYCLE_MEASUREMENT
                        /*tx time stamp and generate the time/cycle diff*/
                        adk_sys_dp_proc_cycle_measure(task);
#endif

                        invoke_cb_attr->recvCb(invoke_cb_attr->id, (task),
                                    invoke_cb_attr->user_data);

                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: sent packet to fps clbk \n "); 
                        return (ADK_IPV6_FLOW_CACHE_CONSUME);
                    }
                    else
                    {
                        ADK_IPV6_LLI_FC_INCR(fc_bind_term_hit);

                        /* fill the IMC parameters */
                        adk_ipv6_dp_lli_imc_fill(adk_md_p, flow_cache_entry->dest_module_id,
                                remarked_dscp, 0, (uint8_t)flow_cache_entry->vrd_id,
                                (len_nh_hlimit&0xFF),
                                flow_cache_entry->ingress_bind_attr.tunnel_if_id, if_stats,
                                next_hdr, src_port, dst_port, frag_flag);

                        /* update the dest funcion/flow details */ 
                        adk_md_p->func_idx  = flow_cache_entry->stageInfo.fn_id;
                        *dst_mod_id         = flow_cache_entry->dest_module_id;
                        rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));
                        adk_md_p->pdu_flow_id    = flow_cache_entry->destFlowId;
                        adk_md_p->proc_stage_idx = 0;
                        *ringHdl = NULL;

                        ADK_IPV6_DP_LOG_INFO_DEBUG("FC: ingress bind info returned \n");
                        return ADK_IPV6_FLOW_CACHE_DONE;        
                    }
                }
            default:
                {
                    ADK_IPV6_DP_LOG_INFO_DEBUG("FC: invalid 5tuple FC action \n");
                    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
                }
        }
    }
    ADK_IPV6_LLI_FC_INCR(fc_miss);
    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}

#endif

