/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_export.h"
#include "adk_ipv6_dp_flow_cache_uli.h"

#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif

extern uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES];

#ifdef ADK_IPV6_FLOW_CACHE
adk_dp_status_t 
adk_ipv6_dp_uli_flow_cache_entry_alloc(
        adk_ipv6_dp_flow_cache_hash_entry_t **uli_flow_cache_hash_entry,
        adk_ipv6_dp_uli_flow_cache_entry_t **flow_cache_entry,
        uint32_t hash_value)
{
    adk_ipv6_dp_uli_flow_cache_table_t *uli_flow_cache_table;
    uint16_t first_key, second_key, i, index;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    /* hash lookup */
    uli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex];
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    *uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[first_key];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, (*uli_flow_cache_hash_entry)++)
    {
        if(!(*uli_flow_cache_hash_entry)->is_enabled)
        {
            ADK_IPV6_DP_FC_RSRV_IDX(uli_flow_cache_table->free_list,
                    uli_flow_cache_table->tbl,
                    index);
            *flow_cache_entry = &uli_flow_cache_table->tbl[index];
            (*uli_flow_cache_hash_entry)->index = index;

            /* update the Flow cache hash info */
            (*flow_cache_entry)->hash_idx = first_key+i;

            /* set active & aging flag */
            adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
            adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.active_flows, index);

            /* reset egress DSCP based info */
            ADK_MEMSET((*flow_cache_entry)->eg_dscp_lkup, 0, 
                    (sizeof(uint16_t)* IPV6_MAX_DSCP_IDS));

            return ADK_SUCCESS;
        }
    }

    second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) & 
        (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);

    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    *uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[second_key];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, (*uli_flow_cache_hash_entry)++)
    {
        if(!(*uli_flow_cache_hash_entry)->is_enabled)
        {
            ADK_IPV6_DP_FC_RSRV_IDX(uli_flow_cache_table->free_list,
                    uli_flow_cache_table->tbl,
                    index);
            *flow_cache_entry = &uli_flow_cache_table->tbl[index];
            (*uli_flow_cache_hash_entry)->index = index;

            /* update the Flow cache hash info */
            (*flow_cache_entry)->hash_idx = second_key+i;
            
            /* set active & aging flag */
            adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
            adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.active_flows, index);

            /* reset egress DSCP based info */
            ADK_MEMSET((*flow_cache_entry)->eg_dscp_lkup, 0, 
                    (sizeof(uint16_t)* IPV6_MAX_DSCP_IDS));

            return ADK_SUCCESS;
        }
    }

    ADK_IPV6_DP_LOG_INFO_DEBUG("ULI flow-cache entry FULL !!!\n");
    return ADK_DP_E_ERROR;
}

adk_status_t 
adk_ipv6_dp_uli_flow_cache_entry_release(adk_ipv6_dp_uli_flow_cache_entry_t *uli_fc_entry)
{
    uint16_t hash_idx;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_flow_cache_hash_entry_t *fc_hash_entry;
    adk_ipv6_dp_uli_flow_cache_table_t  *uli_fc_table;
    

    /* release FC Hash entry */
    hash_idx = (uli_fc_entry->hash_idx); 
    fc_hash_entry = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].hash_tbl[hash_idx];
    *((uint64_t *)fc_hash_entry) = 0;
    
    /* mark the FC entry disabled */
    uli_fc_entry->is_enabled = FALSE;
    
    /* release the FC entry index */
    uli_fc_table = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex];
    ADK_IPV6_DP_FC_RELEASE_IDX(uli_fc_table->free_list,
            uli_fc_table->tbl,
            uli_fc_entry->idx);

    /* reset active flag */
    adk_ipv6_dp_fc_bitmap_reset(uli_fc_table->age_info.active_flows, uli_fc_entry->idx);
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("successfully released the aged out FC entry %d \n",
            uli_fc_entry->idx);

    return(ADK_SUCCESS);
}

static inline adk_dp_status_t
adk_ipv6_dp_3tuple_uli_flow_cache_lookup(adk_ipv6_hdr_t *ipv6_hdr, 
                    uint8_t __attribute__ ((unused)) next_hdr,
                    uint8_t vrd_id,
                    adk_ipv6_dp_uli_flow_cache_entry_t **flow_cache_entry,
                    uint32_t hash_value)
{
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t i, first_key, second_key, index;
    adk_ipv6_dp_flow_cache_hash_entry_t     *uli_flow_cache_hash_entry;
    adk_ipv6_dp_uli_flow_cache_entry_t      *uli_flow_cache_entry;
    adk_ipv6_dp_uli_flow_cache_table_t      *uli_flow_cache_table;

    uli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex];

    first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    /* hash lookup */
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(first_key)];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_3TUPLE == (uli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[2]))) &&
               (vrd_id == (uli_flow_cache_entry)->vrd_id))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);

                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                                        
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) &  
        (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);         

    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(second_key)];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_3TUPLE == (uli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[2]))) &&
               (vrd_id == (uli_flow_cache_entry)->vrd_id))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
                    
                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                    
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("flow-cache entry not found for 3 tuple ULI FC\n");
    *flow_cache_entry = NULL;
    return ADK_DP_E_ERROR;
}

static inline adk_dp_status_t
adk_ipv6_dp_1tuple_uli_flow_cache_lookup(uint32_t id, 
                    uint8_t vrd_id,
                    adk_ipv6_dp_uli_flow_cache_entry_t **flow_cache_entry,
                    uint32_t hash_value)
{
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t i, first_key, second_key, index;
    adk_ipv6_dp_flow_cache_hash_entry_t     *uli_flow_cache_hash_entry;
    adk_ipv6_dp_uli_flow_cache_entry_t      *uli_flow_cache_entry;
    adk_ipv6_dp_uli_flow_cache_table_t      *uli_flow_cache_table;

    uli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex];

    first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    /* hash lookup */
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(first_key)];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_1TUPLE == (uli_flow_cache_entry)->fc_type))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id] && 
                        (uli_flow_cache_entry->tunnel_if_id == id)) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);

                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                                        
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) &  
        (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);         

    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(second_key)];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_1TUPLE == (uli_flow_cache_entry)->fc_type))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id] && 
                        (uli_flow_cache_entry->tunnel_if_id == id)) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
                    
                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                    
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("flow-cache entry not found for 3 tuple ULI FC\n");
    *flow_cache_entry = NULL;
    return ADK_DP_E_ERROR;
}


static inline adk_status_t
adk_ipv6_dp_uli_flow_cache_action(adk_pktio_task_t **task,
                    adk_ipv6_hdr_t *ipv6_hdr, adk_ipv6_frag_t frag_flag, 
                    uint8_t next_hdr, uint16_t src_port, uint16_t dst_port, 
                    adk_ipv6_dp_uli_flow_cache_entry_t *flow_cache_entry,
                    uint8_t *dst_mod_id, adk_pal_dp_stage_info_t *stageInfo)
{

    adk_metadata_t *adk_md_p = ADK_METADATA_PTR((*task));
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry;
    uint32_t fragOctets = 0;
    uint16_t eg_if_id, ipv6_len, destFunc, mtu;
    uint16_t eg_acl_stats_id = 0, stats_id, tunnelIfId =0, laFlagVersion = 0;
    uint8_t  num_frags = 0, coreIndex = ADK_LCORE_INDEX, ipv6_pcp, ipv6_dscp, dscp_id;
    bool localFdBk = FALSE, laFlag = FALSE;
    adk_ipv6_dp_uli_imc_path_t path;
    adk_ipv6_dp_frag_error_t frag_status = ADK_IPV6_DP_FRAG_NOT_REQ; 
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    adk_ipv6_dp_uli_imc_data_t *ipv6_md_p;
    adk_ethernet_dp_uli_imc_param_t *eth_uli_md_p;
#ifdef ADK_MODULE_IPSEC_ENABLED
    adk_ipsec_dp_uli_imc_param_t  *ipsec_md_p; 
#else                               
    RTE_SET_USED(src_port);         
    RTE_SET_USED(dst_port);         
#endif
  
#if ADK_IPV6_ACL_ENABLE
    adk_status_t ret = ADK_SUCCESS;
    uint32_t len_nh_hlimit;
#endif

    ADK_IPV6_DP_LOG_INFO_DEBUG("FC: Flow cache match with action %d \n ",
            flow_cache_entry->action);
    eg_if_id = flow_cache_entry->egress_if_id;

    switch(flow_cache_entry->action)
    {
        case ADK_IPV6_DP_ULI_FLOW_CACHE_FWD_BIND:
        case ADK_IPV6_DP_ULI_FLOW_CACHE_EGRESS_BIND: 
            {
                ipv6_if_entry = &adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[eg_if_id];
                if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][eg_if_id];
                if_stats->ipv6IfStatsOutRequests++;

                if(flow_cache_entry->mtu_excp)
                {
                    if(adk_ipv6_dp_send_clone_exception((*task), flow_cache_entry->vrd_id,
                                ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN,
                                ADK_IPV6_EXCEPTION_DST_MTU_FAIL, coreIndex, ipv6_if_entry->mtu) != ADK_SUCCESS)
                    {                 

                        ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                "ADK_IPV6_EXCEPTION_DST_MTU_FAIL\n");
                    }
                }

                ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
                dscp_id = adk_ipv6_dscp_map[ipv6_dscp];
                if((dscp_id != IPV6_INVALID_DSCP_ID) &&
                        (flow_cache_entry->eg_dscp_lkup[dscp_id] & IPV6_DSCP_LKUP_ENABLE_MASK))
                {
                    ipv6_pcp = (flow_cache_entry->eg_dscp_lkup[dscp_id] & 
                            IPV6_DSCP_LKUP_PCP_MASK)>>12;
                    eg_acl_stats_id = flow_cache_entry->eg_dscp_lkup[dscp_id]
                                & IPV6_DSCP_LKUP_ACL_STATSID_MASK;
                        if( eg_acl_stats_id != 0)
                        {
                             stats_id = (coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+eg_acl_stats_id; 
                             adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[stats_id]++;
                        }
                }
                else
                {
                    ipv6_pcp = ipv6_if_entry->dscp_pcp[ipv6_dscp];
#if ADK_IPV6_ACL_ENABLE
                    if (flow_cache_entry->enable_egress_acl)
                    {
                         len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);

                         ret = adk_ipv6_acl_lookup_func( *task, ipv6_if_entry, next_hdr,
                                                       ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, ipv6_if_entry->vrd_id, 
                                                       &eg_acl_stats_id, FALSE, ADK_TN_PORT_DOMAIN, coreIndex);
                         if ( ret != ADK_SUCCESS)
                         {
                             if ( ret != ADK_E_TASK_CONSUMED)
                             { 
                                 IPV6_DP_DROP_TASK(*task);
                             }
                             if ( ret == ADK_E_INTERNAL_ERROR)
                             {
                                 ipv6_global_stats->ipv6AclFail++;
                             } 
                             return ADK_IPV6_FLOW_CACHE_CONSUME;
                         }

                    }
                    if (dscp_id != IPV6_INVALID_DSCP_ID)
                    {
                         flow_cache_entry->eg_dscp_lkup[dscp_id] =
                             (IPV6_DSCP_LKUP_ENABLE_MASK|
                              (ipv6_pcp << IPV6_DSCP_LKUP_PCP_POS)|
                              (eg_acl_stats_id));
                    } 
#endif 
                }

                destFunc = flow_cache_entry->stageInfo.fn_id;
                *dst_mod_id =flow_cache_entry->dest_module_id;
                rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));

                /* consider per packet MTU for frag */ 
                mtu = flow_cache_entry->mtu;
                ipv6_md_p = (adk_ipv6_dp_uli_imc_data_t *)&adk_md_p->imc_param_start;
                path = ipv6_md_p->path;
                if(path == ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR)
                {
                    if(ipv6_md_p->withHdrInfo.outerPathMtuInfo)
                    {
                        if(ipv6_md_p->withHdrInfo.outerPathMtuInfo & ADK_IPV6_INTERFACE_MTU_MASK)
                            mtu = (ipv6_md_p->withHdrInfo.outerPathMtuInfo &
                                    ADK_IPV6_INTERFACE_MTU_MASK);
                        localFdBk = (ipv6_md_p->withHdrInfo.outerPathMtuInfo>>15);

                    }
                    tunnelIfId = ipv6_md_p->withHdrInfo.tunnelIfId;
                    laFlag = ipv6_md_p->withHdrInfo.laFlag;
                    laFlagVersion = ipv6_md_p->withHdrInfo.laFlagVersion;
                }

                if((adkDestEntryFunction_g[destFunc]!= NULL) && (*dst_mod_id))
                {
                    if(flow_cache_entry->egress_if_type == IPV6_INTERFACE_TYPE_ETH)
                    {
                        eth_uli_md_p = (adk_ethernet_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
                        /* update the egress stats Id */
                        adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].flags = 
                            ADK_SYS_EGRESS_STATS_MDATA_FLAG_BYTES_CNTR;
                        adk_md_p->egress_stats[ADK_SYS_EGRESS_STATS_MDATA_IP_LOC].stats_id = 
                            ipv6_if_entry->sys_egress_stats_id;

                        /* update the ethernet imc */ 
                        eth_uli_md_p->dscp = ipv6_dscp;
                        eth_uli_md_p->pcp  = ipv6_pcp;
                        eth_uli_md_p->is_unfrag_first_frag =((frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)||
                                (frag_flag==ADK_IPV6_FIRST_FRAGMENT))? TRUE: FALSE;
                        eth_uli_md_p->l4_proto = next_hdr; 
                        ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to Ethernet \n");
                        ADK_IPV6_DP_LOG_INFO_DEBUG("dscp %d \n", eth_uli_md_p->dscp);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("pcp %d \n", eth_uli_md_p->pcp);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("is_unfrag_first_frag %d \n", eth_uli_md_p->is_unfrag_first_frag);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("l4_proto %d \n", eth_uli_md_p->l4_proto);
                    }/* IPSEC */
#ifdef ADK_MODULE_IPSEC_ENABLED
                    else if(flow_cache_entry->egress_if_type == IPV6_INTERFACE_TYPE_TUNNEL)
                    {
                        ipsec_md_p = (adk_ipsec_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
                        ipsec_md_p->ip_vrdid = flow_cache_entry->outer_vrd_id;
                        ipsec_md_p->outerPathMtu = ipv6_if_entry->pmtu;
                        ipsec_md_p->protocol = next_hdr;
                        ipsec_md_p->la_flag = ipv6_if_entry->la_flag;
                        ipsec_md_p->la_flag_version = ipv6_if_entry->la_flag_version;
                        ipsec_md_p->tunnel_id = ipv6_if_entry->ingress_stats_id;
                        ipsec_md_p->srcPort = src_port;
                        ipsec_md_p->dstPort = dst_port;
                        ipsec_md_p->ipv6FragFlag = (frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)? FALSE: TRUE;
                        ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to IPsec \n");
                        ADK_IPV6_DP_LOG_INFO_DEBUG("ip_vrdid %d \n",ipsec_md_p->ip_vrdid);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("outerPathMtu %d \n", ipsec_md_p->outerPathMtu);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("la_flag %d \n", ipsec_md_p->la_flag);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("la_flag_version %d \n", ipsec_md_p->la_flag_version);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("tunnel_id %d \n", ipsec_md_p->tunnel_id);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("protocol %d \n", ipsec_md_p->protocol);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("ipv6FragFlag %d \n", ipsec_md_p->ipv6FragFlag);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("srcPort %d \n", ipsec_md_p->srcPort);
                        ADK_IPV6_DP_LOG_INFO_DEBUG("dstPort %d \n", ipsec_md_p->dstPort);
                    }
#endif
                }

                ipv6_len = adk_ntohl(ipv6_hdr->len_nh_hlimit)>>16;
                ipv6_len += ADK_IPV6_PDU_HEADER_LENGTH_BYTES;

                /* Update the Function & PDU Flow Id, so that it will reflect in 
                 * Fragments */
                adk_md_p->func_idx          = flow_cache_entry->stageInfo.fn_id;
                adk_md_p->pdu_flow_id       = flow_cache_entry->destFlowId;
                adk_md_p->proc_stage_idx    = 0;

                if(mtu < ipv6_len)
                {
                    if(path == ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR)
                    {
                        /* extract the tunnel if info */
                        adk_ipv6_dp_interface_entry_t *tnl_if_entry;
                        ret = adk_ipv6_dp_interface_entry_lookup(
                                tunnelIfId, 
                                0, 
                                &tnl_if_entry);
                        if(ret == ADK_SUCCESS)
                        {
                            if((!laFlag) && 
                                    (!tnl_if_entry->la_flag) && 
                                    (tnl_if_entry->la_flag_version == laFlagVersion))
                            {
                                tnl_if_entry->la_flag = 1;
                            }
                        }
                        else
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("invalid tunnel interface id \n");
                        }
                    }

                    /* do not send local feedback if dest mtu exception is already raised */
                    if(localFdBk && (!flow_cache_entry->mtu_excp))
                    {
                        if(adk_ipv6_dp_send_clone_exception((*task), flow_cache_entry->vrd_id,
                                    ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN,
                                    ADK_IPV6_EXCEPTION_LOCAL_PTB, coreIndex, mtu) != ADK_SUCCESS)
                        {
                            ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                                    "ADK_IPV6_EXCEPTION_LOCAL_PTB\n");
                        }
                    }

                    if_stats->ipv6IfStatsOutFragReqds++;                

                    frag_status = adk_ipv6_dp_fragment(task, coreIndex, mtu, 
                            &num_frags, &fragOctets, ipv6_if_entry->if_type);
                    if ( frag_status == ADK_IPV6_DP_FRAG_OK )
                    {
                        if_stats->ipv6IfStatsOutFragOKs += num_frags;
                    }
                    else
                    {
                        if_stats->ipv6IfStatsOutFragFails++;
                        ADK_IPV6_DP_LOG_INFO_DEBUG("error: FC Fragmentation failed \n ");
                        return ADK_IPV6_FLOW_CACHE_ERROR;
                    }
                    /* after framentation retry the new ipv6_hdr offset */
                    adk_md_p = ADK_METADATA_PTR((*task));
                    ADK_PDU_OFFSET_PTR_GET((*task), ipv6_hdr, 
                            adk_ipv6_hdr_t *, adk_md_p->ip_offset);
                }

                /* packet capture */
                if(flow_cache_entry->eg_pktcap_enabled)
                {
                    adk_ipv6_dp_packet_capture((*task), ipv6_if_entry->vrd_id,
                            &ipv6_if_entry->eg_pkt_capture_info, coreIndex);
                }

                /* increment the statistics */
                if ( frag_status == ADK_IPV6_DP_FRAG_OK)
                {
                    if((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xFF)
                    {
                        if_stats->ipv6IfStatsOutMcastPkts += num_frags;;
                        if_stats->ipv6IfStatsOutMcastOctets += fragOctets;
                    }
                    else
                    {                   
                        if_stats->ipv6IfStatsOutUcastPkts += num_frags;
                    }
                    if_stats->ipv6IfStatsHCOutOctets += fragOctets;
                }
                else
                {
                    /* increment the statistics */
                    if((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xFF)
                    {
                        if_stats->ipv6IfStatsOutMcastPkts++;
                        if_stats->ipv6IfStatsOutMcastOctets += ipv6_len;
                    }
                    else
                    {
                        if_stats->ipv6IfStatsOutUcastPkts++;
                    }
                    /* TODO: remove if any meta headers */
                    if_stats->ipv6IfStatsHCOutOctets += ipv6_len; 
                }

                /* update the dest funcion/flow details */ 
                *dst_mod_id         = flow_cache_entry->dest_module_id;
                rte_memcpy(stageInfo, &flow_cache_entry->stageInfo, sizeof(adk_pal_dp_stage_info_t));

                ADK_IPV6_DP_LOG_INFO_DEBUG("FC: egress bind info returned \n");
                return ADK_IPV6_FLOW_CACHE_DONE;
            }
        default:
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG("FC: invalid 3tuple FC action \n");
                return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
            }
    }
    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}

adk_dp_status_t 
adk_ipv6_dp_uli_flow_cache_1tuple_proc(adk_ipv6_hdr_t *ipv6_hdr, uint8_t next_hdr,
        adk_ipv6_frag_t frag_flag, uint16_t tunnel_if_id, uint8_t vrd_id, 
        adk_pktio_task_t **task, uint8_t *dst_mod_id, 
        adk_pal_dp_stage_info_t *stageInfo, uint32_t hash_value)
{
    adk_dp_status_t ret;
    adk_ipv6_dp_uli_flow_cache_entry_t *flow_cache_entry;

    ret = adk_ipv6_dp_1tuple_uli_flow_cache_lookup(tunnel_if_id, vrd_id, &flow_cache_entry,
                    hash_value);
    if(ret == ADK_SUCCESS)
    {
        return(adk_ipv6_dp_uli_flow_cache_action(task, ipv6_hdr, frag_flag, next_hdr, 
                    0, 0, flow_cache_entry, dst_mod_id, stageInfo));
    }

    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}
adk_dp_status_t 
adk_ipv6_dp_uli_flow_cache_3tuple_proc(adk_ipv6_hdr_t *ipv6_hdr, uint8_t next_hdr,
        adk_ipv6_frag_t frag_flag, uint16_t __attribute__ ((unused))src_port, 
        __attribute__ ((unused))uint16_t dst_port,  uint8_t vrd_id, 
        adk_pktio_task_t **task, uint8_t *dst_mod_id, 
        adk_pal_dp_stage_info_t *stageInfo, uint32_t hash_value)
{
    adk_dp_status_t ret;
    adk_ipv6_dp_uli_flow_cache_entry_t *flow_cache_entry;

    ret = adk_ipv6_dp_3tuple_uli_flow_cache_lookup(ipv6_hdr, next_hdr, vrd_id, &flow_cache_entry,
                    hash_value);

    if(ret == ADK_SUCCESS)
    {
        return(adk_ipv6_dp_uli_flow_cache_action(task, ipv6_hdr, frag_flag, next_hdr, 
                    src_port, dst_port, flow_cache_entry, dst_mod_id, stageInfo));
    }

    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}

static inline adk_dp_status_t
adk_ipv6_dp_5tuple_uli_flow_cache_lookup(adk_ipv6_hdr_t *ipv6_hdr, uint8_t next_hdr,
        uint16_t src_port, uint16_t dst_port, uint8_t vrd_id, 
        adk_ipv6_dp_uli_flow_cache_entry_t **flow_cache_entry, uint32_t hash_value)
{
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t i, first_key, second_key, index;
    adk_ipv6_dp_flow_cache_hash_entry_t     *uli_flow_cache_hash_entry;
    adk_ipv6_dp_uli_flow_cache_entry_t      *uli_flow_cache_entry;
    adk_ipv6_dp_uli_flow_cache_table_t      *uli_flow_cache_table;

    uli_flow_cache_table = &adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex];

    first_key = hash_value & (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);
    /* hash lookup */
    first_key = first_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(first_key)];
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_5TUPLE == (uli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[2]))) &&
               (src_port == (uli_flow_cache_entry)->src_port) &&
               (dst_port == (uli_flow_cache_entry)->dst_port) &&
               (next_hdr == (uli_flow_cache_entry)->next_hdr) &&
               (vrd_id == (uli_flow_cache_entry)->vrd_id))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
                    
                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                    
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }

    second_key = (hash_value >> ADK_IPV6_NO_OF_FLOW_CACHE_WIDTH) & 
                   (ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES - 1);

    second_key = second_key*ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES;
    uli_flow_cache_hash_entry = &uli_flow_cache_table->hash_tbl[(second_key)];        
    for(i=0; i<ADK_IPV6_NO_OF_FLOW_CACHE_COL_ENTRIES; i++, uli_flow_cache_hash_entry++)
    {
        if((uli_flow_cache_hash_entry->hash_value == hash_value))
        {
            /*compare flow cache tuples */
            index = uli_flow_cache_hash_entry->index;
            uli_flow_cache_entry = &uli_flow_cache_table->tbl[index];
  
            if((ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_5TUPLE == (uli_flow_cache_entry)->fc_type) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->src_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->src_addr[2]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[0]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[0]))) &&
               (*((uint64_t *)&ipv6_hdr->dst_addr[2]) == 
                    *((uint64_t *)&((uli_flow_cache_entry)->dst_addr[2]))) &&
               (src_port == (uli_flow_cache_entry)->src_port) &&
               (dst_port == (uli_flow_cache_entry)->dst_port) &&
               (next_hdr == (uli_flow_cache_entry)->next_hdr) &&
               (vrd_id == (uli_flow_cache_entry)->vrd_id))
            {
                if((uli_flow_cache_entry)->version_id == 
                        adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]) 
                {
                    /* set aging flag */
                    adk_ipv6_dp_fc_bitmap_set(uli_flow_cache_table->age_info.aging_bitmap, index);
                    
                    *flow_cache_entry = uli_flow_cache_entry;
                    return(ADK_SUCCESS);
                }
                else
                {
                    ADK_IPV6_ULI_FC_INCR(fc_cleared);
                    ADK_IPV6_DP_LOG_INFO_DEBUG("Exact match found, but stale entry !!! \n");
                    
                    /* release the fc entry */
                    adk_ipv6_dp_uli_flow_cache_entry_release(uli_flow_cache_entry);
                   
                    *flow_cache_entry = NULL;
                    return ADK_DP_E_ERROR;
                }
            }
        }
    }
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("flow-cache entry not found for 5tuple ULI FC\n");
    *flow_cache_entry = NULL;
    return ADK_DP_E_ERROR;
}

adk_dp_status_t 
adk_ipv6_dp_uli_flow_cache_5tuple_proc(adk_ipv6_hdr_t *ipv6_hdr, 
        uint8_t next_hdr, adk_ipv6_frag_t frag_flag, uint16_t src_port,
        uint16_t dst_port, uint8_t vrd_id, adk_pktio_task_t **task, 
        uint8_t *dst_mod_id, adk_pal_dp_stage_info_t *stageInfo, uint32_t hash_value)
{
    adk_dp_status_t ret;
    adk_ipv6_dp_uli_flow_cache_entry_t *flow_cache_entry;

    ret = adk_ipv6_dp_5tuple_uli_flow_cache_lookup(ipv6_hdr, next_hdr, src_port, dst_port, vrd_id, 
            &flow_cache_entry, hash_value);
    if(ret == ADK_SUCCESS)
    {
        return(adk_ipv6_dp_uli_flow_cache_action(task, ipv6_hdr, frag_flag, next_hdr, src_port, 
                    dst_port, flow_cache_entry, dst_mod_id, stageInfo));
    }
    return ADK_IPV6_FLOW_CACHE_SLOW_PATH;        
}
#endif

