/*
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#include "adk_ipv6_dp_frag.h"
#include "adk_eth_constants.h"
/*
 * IPV6 Fragmentation Id 
 */ 

extern uint32_t adk_ipv6_global_frag_id[ADK_MAX_NUM_CORES];
extern struct rte_mempool *adkPktmbufPool_g;
extern struct rte_mempool *pktmbuf_pool_indirect_g;
   
static inline adk_dp_status_t 
adk_ipv6_find_non_fragmentable_part(uint8_t next_header, uint8_t *pdu, uint8_t *offset, 
        uint8_t *nexthdr_offset, uint8_t *nexthdr)
{

    uint8_t unfrag_offset = 0, dstopt_offset = 0;
    bool first_header = TRUE, dstopt_encounted = FALSE, routeHdr_enc = FALSE, offset_found = FALSE;

    pdu += 40;
    unfrag_offset = 40;
    *nexthdr_offset = 6;
    do{
        switch ( next_header)
        {
           case ADK_IPV6_NEXT_HDR_HOP_BY_HOP:
 
               if (first_header == TRUE)
               {
                   next_header = pdu[0];
                   unfrag_offset += ((pdu[1]<<3) + 8);
                   pdu = pdu + ((pdu[1]<<3) + 8);
                   *nexthdr_offset = 40;
               }
               else 
               {
                   ADK_IPV6_DP_LOG_INFO(
                           "Hop-by-Hop extension header is not the first extension header \n");
                   return ADK_DP_E_ERROR; 
               }
               break;

           case ADK_IPV6_NEXT_HDR_DST_OPTIONS:

               if ( routeHdr_enc == TRUE )
               {
                   *offset = unfrag_offset;
                   return ADK_SUCCESS;
               }
               else
               {
                   next_header = pdu[0];
                   dstopt_offset = unfrag_offset;
                   dstopt_encounted = TRUE;
                   unfrag_offset += ((pdu[1]<<3) + 8);
                   pdu = pdu + ((pdu[1]<<3) + 8);
               }
               break;

           case ADK_IPV6_NEXT_HDR_ROUTING:
              
               next_header = pdu[0];
               *nexthdr_offset = unfrag_offset;
               routeHdr_enc = TRUE;
               unfrag_offset += ((pdu[1]<<3) + 8);
               pdu = pdu + ((pdu[1]<<3) + 8);
               dstopt_encounted = FALSE;
               break;

           default:
               if ( dstopt_encounted == TRUE)
               {
                   *nexthdr = ADK_IPV6_NEXT_HDR_DST_OPTIONS; 
                   *offset = dstopt_offset;
               }
               else
               {
                   *nexthdr = next_header; 
                   *offset = unfrag_offset;
               }
               offset_found = TRUE;
               break;       
        }
        first_header = FALSE;
    }while(offset_found != TRUE);

    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_fragment()
 *
 * IPv6 DP function to process fragmentation
 * =============================================================
 */
adk_ipv6_dp_frag_error_t 
adk_ipv6_dp_fragment(adk_pktio_task_t **task_out_p, int coreIndex, uint32_t mtu, 
        uint8_t *num_frags, uint32_t *fragOctets, adk_ipv6_interface_type_t if_type)
{

    adk_ipv6_hdr_t  *ipv6_hdr = NULL;
    adk_ipv6_extension_fragment_t *frag_hdr = NULL;
    int8_t frag_count = 1, first_frag_nb_segs = 0;
    uint8_t  *pdu = NULL, *frag = NULL;
    uint8_t  fragment_offset = 0, nexthdr_offset = 0, nexthdr =  0, total_frags = 0;
    uint16_t frag_offset = 0, ipv6_len = 0, payload_len = 0, frag_length = 0;
    uint32_t in_seg_data_pos = 0, more_in_segs, more_out_segs, frag_size = 0, len = 0;
    uint32_t first_frag_pkt_len = 0, first_frag_data_len = 0;
    struct rte_mbuf *in_seg = NULL;
    /*ToDo: To test the cycle count impact if any due to this change. If yes, need to create
     * mempool and to allocate the pointer from mempool*/
    struct rte_mbuf *task_frags[8*ADK_DP_TASK_MULTIPLY_MAX];
    struct rte_mbuf *task_incoming, *temp_seg, *in_seg_next;
    adk_pktio_task_t *taskout_next_p = NULL, *task_next = NULL;
    struct rte_mbuf *out_pkt = NULL, *out_seg = NULL, *out_seg_prev = NULL;
    adk_metadata_t *adk_md_p, *chain_md_p;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_utility_counters_t         *ipv6_utility_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
    adk_ipv6_dp_uli_imc_data_t *ipv6_imc = NULL;
#ifdef ADK_DPDK_IPV6_FRAG
    struct rte_mempool *pool_indirect_p;
#endif
    adk_md_p = ADK_METADATA_PTR(*task_out_p);
    ipv6_imc = (adk_ipv6_dp_uli_imc_data_t *)(&(adk_md_p->imc_param_start));

#ifdef ADK_MODULE_IPSEC_ENABLED
    adk_ipsec_dp_uli_imc_param_t *ipsec_md_p;
    if(if_type == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        ipsec_md_p = (adk_ipsec_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
        ipsec_md_p->ipv6FragFlag = TRUE;
        ADK_IPV6_DP_LOG_INFO_DEBUG("updating fragFlag = %d \n", ipsec_md_p->ipv6FragFlag);
    }
#endif

     /* Strip-off the ETH/VLAN header from task if present*/
    ADK_PDU_HEAD_PULL((*task_out_p), pdu, (uint8_t *), ADK_METADATA_PARAM((*task_out_p), ip_offset));
    ipv6_hdr = (adk_ipv6_hdr_t *)pdu;
    adk_md_p->ip_offset = 0;
    if (unlikely(ipv6_hdr == NULL))
    {
        ADK_IPV6_DP_LOG_ERROR("Err: ipv6 header pointer updated as null \n");
        return ADK_E_INVALID_PARAMETER; 
    }

    /* Find the unfragmentable portion of the incoming packet */
    if ( adk_ipv6_find_non_fragmentable_part( ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8) & 0xFF), 
                                         (uint8_t *)ipv6_hdr, &fragment_offset, &nexthdr_offset, 
                                         &nexthdr) != ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_INFO("Err: HOP by HOP is not first ext header \n");
        return ADK_IPV6_DP_FRAG_NON_FIRST_HOPBYHOP;
    }

    frag_size = ((mtu - ADK_IPV6_FRAG_HDR_LEN - fragment_offset)/8 ) * 8;
    frag_offset = frag_size;
 

    /*
     * Add space for IPv6 Fragmentation header 
     */
    ADK_PDU_HEAD_PUSH(*task_out_p, pdu, (uint8_t *), ADK_IPV6_FRAG_HDR_LEN);
    if(pdu == NULL)
    {
        if(IP6_ULI_SENDER_IPSEC != ipv6_imc->sender)
        {
            ipv6_global_stats->hdrmNotPresent++;
        }
        ADK_IPV6_DP_LOG_INFO_DEBUG("error: Headroom not available"
                " dropping the packet \n");
        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxhdrRoomNotAvail);
        return ADK_IPV6_DP_FRAG_MEM_ALLOC_FAILED;
    }
    for ( len = 0; len < fragment_offset; len++)
    {
        pdu[len] = pdu[(len + ADK_IPV6_FRAG_HDR_LEN)];
    }

    
    /* Update the IPv6 header details for First Fragment */
    ipv6_hdr = (adk_ipv6_hdr_t *)pdu;
    ipv6_len = (adk_ntohl(ipv6_hdr->len_nh_hlimit) >>16) + ADK_IPV6_HDR_LEN;
    payload_len = ipv6_len - fragment_offset;
   
    ipv6_hdr->len_nh_hlimit = (adk_ntohl(ipv6_hdr->len_nh_hlimit) & 0xFFFF) | 
                              ((fragment_offset + frag_size + ADK_IPV6_FRAG_HDR_LEN - ADK_IPV6_HDR_LEN)<< 16);
    adk_md_p->outerIpPktLen = (ipv6_hdr->len_nh_hlimit >>16) + ADK_IPV6_HDR_LEN;
    ipv6_hdr->len_nh_hlimit = adk_htonl(ipv6_hdr->len_nh_hlimit);

    pdu[nexthdr_offset] = ADK_IPV6_NEXT_HDR_FRAGMENT; 
    frag_hdr =  (adk_ipv6_extension_fragment_t *)(pdu + fragment_offset);

    /* Allocate Fragment Id */
    adk_ipv6_global_frag_id[coreIndex]++;
    if ( adk_ipv6_global_frag_id[coreIndex] == (uint32_t) ((coreIndex << 24) | 0xFFFFFF))
    {
        adk_ipv6_global_frag_id[coreIndex] = (coreIndex << 24);
    }

    frag_hdr->next_header = nexthdr;
    frag_hdr->frag_data = adk_htons(0x1); /*offset zero and more flag true */
    frag_hdr->id = adk_htonl(adk_ipv6_global_frag_id[coreIndex]);

   
    /* Update the First Fragment Pktlen & Datalen  */ 
    in_seg = (struct rte_mbuf *)*task_out_p;
    task_incoming = (struct rte_mbuf *)*task_out_p;

    total_frags = (payload_len % frag_size)?((payload_len / frag_size)+1):(payload_len / frag_size);
    *num_frags = total_frags;  
    in_seg_data_pos = fragment_offset + ADK_IPV6_FRAG_HDR_LEN;
    more_in_segs = 1;

    /*Max Supported frags is 64. Considering 65K payload and minMTU as 1280, we can have max 52 frags.
     * Keeping some more buffer and restricting it to 64*/
    if (total_frags > (8*ADK_DP_TASK_MULTIPLY_MAX))
    {
        if(IP6_ULI_SENDER_IPSEC != ipv6_imc->sender)
        {
            ipv6_global_stats->ipv6OutFragLimitExceed++;
        }
        ADK_IPV6_DP_LOG_INFO_1D("Err: No of Fragments(%d) exceeded Max (16) Supported\n", total_frags);
        return ADK_IPV6_DP_FRAG_LIMIT_EXCEED;
    }

    /* Collecting Data to Convert Incoming PDU as First Fragment */
    (*fragOctets) = (*fragOctets) + (fragment_offset + frag_size + ADK_IPV6_FRAG_HDR_LEN);

    more_out_segs = 1;
    first_frag_pkt_len = fragment_offset + ADK_IPV6_FRAG_HDR_LEN;
    frag_length = frag_size + first_frag_pkt_len;

    while (likely(more_out_segs && more_in_segs)) {

        len = frag_length - first_frag_pkt_len;
        if (len > (in_seg->data_len - in_seg_data_pos)) {
            len = in_seg->data_len - in_seg_data_pos;
        }
        first_frag_nb_segs++;
        in_seg_data_pos += len;
        first_frag_pkt_len += len;

        /* check whether the current fragment data is copied */
        if (likely(first_frag_pkt_len >= frag_length)) {
            more_out_segs = 0;
            first_frag_data_len = in_seg_data_pos;
        }

        /* Check for segments and copy data from segments  */
        if (unlikely(in_seg_data_pos == in_seg->data_len)) {
            in_seg = in_seg->next;

            if (unlikely(in_seg == NULL)) {
                more_in_segs = 0;
                first_frag_data_len = in_seg_data_pos;
            }
            in_seg_data_pos = 0;
        }
    }
    task_frags[0] = task_incoming;

    more_in_segs = 1;
    while(frag_count < total_frags)
    {
         /* Allocate direct buffer for Fragment packet headers*/
        out_pkt = rte_pktmbuf_alloc((*task_out_p)->pool);
        if (unlikely(out_pkt == NULL)) {
            frag_count--;
            while (frag_count > 0)
            {
                rte_pktmbuf_free(task_frags[frag_count]);
                frag_count--;
            }
            ADK_IPV6_DP_LOG_INFO_DEBUG("Err: unable to allocate mbufs for frag\n");
            if(IP6_ULI_SENDER_IPSEC != ipv6_imc->sender)
            {
                ipv6_global_stats->ipv6MbufAllocFailures++;
            }
            return ADK_IPV6_DP_FRAG_MEM_ALLOC_FAILED; 
        }
        adk_pkt_dsi_tx_md_reset( out_pkt); 
        out_pkt->hash.usr = task_incoming->hash.usr;
        out_pkt->priv_size = task_incoming->priv_size;
        out_pkt->udata64 =  task_incoming->udata64;
    
        /* Copy the Header from the first fragment and Update only Offset fields */
        frag = rte_pktmbuf_mtod(out_pkt, uint8_t*); 
        adk_md_p = ADK_METADATA_PTR(*task_out_p);
        ADK_PDU_OFFSET_PTR_GET(*task_out_p, pdu, uint8_t *, adk_md_p->ip_offset);

        rte_memcpy( frag, pdu, fragment_offset + ADK_IPV6_FRAG_HDR_LEN);

        frag_hdr =  (adk_ipv6_extension_fragment_t *)(frag + fragment_offset);

        if ( (frag_count+1) == total_frags)
        {
            frag_hdr->frag_data = adk_htons(((frag_offset/8) << 3));
        }
        else 
        {
            frag_hdr->frag_data = adk_htons(((frag_offset/8) << 3) | 0x1);
        }

        if ( (frag_offset+frag_size) > payload_len )
        {
            frag_size = payload_len - frag_offset; 
        }

        *fragOctets = (*fragOctets)+ (fragment_offset + frag_size + ADK_IPV6_FRAG_HDR_LEN); 
        ((adk_ipv6_hdr_t*)frag)->len_nh_hlimit = (adk_ntohl(ipv6_hdr->len_nh_hlimit) & 0xFFFF) | 
                                                 ((fragment_offset + frag_size + ADK_IPV6_FRAG_HDR_LEN - ADK_IPV6_HDR_LEN)<< 16);
        ((adk_ipv6_hdr_t*)frag)->len_nh_hlimit = adk_htonl(((adk_ipv6_hdr_t*)frag)->len_nh_hlimit );
	frag_offset += frag_size;

        /* Update the Fragment Data length and packet length */
        out_pkt->data_len = fragment_offset + ADK_IPV6_FRAG_HDR_LEN;
        out_pkt->pkt_len  = fragment_offset + ADK_IPV6_FRAG_HDR_LEN;
        frag_length = frag_size +  out_pkt->pkt_len;
            
        out_seg_prev = out_pkt;
        more_out_segs = 1;
        while (likely(more_out_segs && more_in_segs)) {


#ifndef ADK_DPDK_IPV6_FRAG
            /* Allocate direct buffer for payload of fragments &
               Link it with header buffer of the fragment */
            out_seg = rte_pktmbuf_alloc((*task_out_p)->pool);
            if (unlikely(out_seg == NULL)) {
                rte_pktmbuf_free(out_pkt);

                frag_count--;
                while (frag_count > 0)
                {
                    rte_pktmbuf_free(task_frags[frag_count]);
                    frag_count--;
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG("Err: unable to allocate mbufs for fragments \n");
                if(IP6_ULI_SENDER_IPSEC != ipv6_imc->sender)
                {
                    ipv6_global_stats->ipv6MbufAllocFailures++;
                }
                return ADK_IPV6_DP_FRAG_MEM_ALLOC_FAILED; 
            }
            adk_pkt_dsi_tx_md_reset( out_seg); 
            out_seg->hash.usr = task_incoming->hash.usr;
            out_seg->priv_size = task_incoming->priv_size;
            out_seg->udata64 =  task_incoming->udata64;
#else
            /* Allocate indirect buffer */

            adk_sys_config_ip_frag_mempool_get((void **) &pool_indirect_p);

            out_seg = rte_pktmbuf_alloc(pool_indirect_p);
            if (unlikely(out_seg == NULL)) {
                rte_pktmbuf_free(out_pkt);
                frag_count--;
                while (frag_count > 0)
                {
                    rte_pktmbuf_free(task_frags[frag_count]);
                    frag_count--;
                }
                ADK_IPV6_DP_LOG_INFO_DEBUG(
                        "Err: unable to allocate indirect mbufs for fragments \n");
                if(IP6_ULI_SENDER_IPSEC != ipv6_imc->sender)
                {
                    ipv6_global_stats->ipv6MbufAllocFailures++;
                }
                return ADK_IPV6_DP_FRAG_MEM_ALLOC_FAILED; 
            }
            adk_pkt_dsi_tx_md_reset( out_seg); 
            out_seg->hash.usr = task_incoming->hash.usr;
            out_seg->udata64 =  task_incoming->udata64;
#endif    

            out_seg_prev->next = out_seg;
            out_seg_prev = out_seg;

#ifdef ADK_DPDK_IPV6_FRAG
            /* Prepare indirect buffer */
            rte_pktmbuf_attach(out_seg, in_seg);
#endif

            len = frag_length - out_pkt->pkt_len;
            if (len > (in_seg->data_len - in_seg_data_pos)) {
                len = in_seg->data_len - in_seg_data_pos;
            }

#ifndef ADK_DPDK_IPV6_FRAG
            /* Copy the fragment payload portion to new buffer */ 
            frag = rte_pktmbuf_mtod(out_seg, uint8_t*); 
            pdu = rte_pktmbuf_mtod(in_seg, uint8_t*); 
            rte_memcpy( frag, pdu+in_seg_data_pos, len);
#else
            out_seg->data_off = in_seg->data_off + in_seg_data_pos;        
#endif

            out_seg->data_len = (uint16_t)len;
            out_pkt->pkt_len = (uint16_t)(len +
                out_pkt->pkt_len);
            out_pkt->nb_segs += 1;
            in_seg_data_pos += len;

            /* check whether the current fragment data is copied */
            if (unlikely(out_pkt->pkt_len >= frag_length)) {
                more_out_segs = 0;
            }

            /* Check for segments and copy data from segments  */
            if (unlikely(in_seg_data_pos == in_seg->data_len)) {
                in_seg = in_seg->next;
                in_seg_data_pos = 0;

                if (unlikely(in_seg == NULL)) {
                    more_in_segs = 0;
                }
            }
         }

         task_frags[frag_count] = out_pkt;
         frag_count++;
    }

    /* Update incoming packet data fields to convert as first frag  */
    task_incoming->pkt_len = (uint16_t)first_frag_pkt_len; 
    task_incoming->nb_segs = (uint16_t)first_frag_nb_segs;
    in_seg = task_incoming;

    while( first_frag_nb_segs > 1)
    {
        in_seg = in_seg->next;
        first_frag_nb_segs--;
    }
    in_seg->data_len = (uint16_t)first_frag_data_len;
    
    
    /* Free the unused segments of received pdu */   
    temp_seg = in_seg;
    in_seg = in_seg->next;
    while ( in_seg != NULL)
    {    
         in_seg_next = in_seg;
         in_seg = in_seg->next;        
         rte_pktmbuf_free_seg(in_seg_next);
    }
    temp_seg->next = NULL;


    /* Update the Meta Data for the created frags */
    frag_count = 0;
    *task_out_p = (adk_pktio_task_t *)task_frags[0];

    while(frag_count < total_frags)
    {
        /*Arange the pointers*/
        taskout_next_p = (adk_pktio_task_t *)task_frags[frag_count];

        chain_md_p = ADK_METADATA_PTR(task_frags[frag_count]);

        if ( frag_count != 0)
        {
            task_frags[frag_count]->ol_flags = task_frags[0]->ol_flags;
            task_frags[frag_count]->l3_len = sizeof (struct ipv6_hdr);
            rte_memcpy(chain_md_p, adk_md_p, (sizeof(adk_metadata_t)));
            ADK_PDU_OFFSET_PTR_GET(task_frags[frag_count], ipv6_hdr,adk_ipv6_hdr_t *,adk_md_p->ip_offset );
            ipv6_len = ((adk_ntohl(ipv6_hdr->len_nh_hlimit))>>16) + ADK_IPV6_HDR_LEN;

            if (frag_count == (total_frags -1))
            {
                chain_md_p->outerIpPktLen = ipv6_len;
            }
            (taskout_next_p)->port = (*task_out_p)->port;
        }
        frag_count++;
        if (frag_count < total_frags)
        {   
            task_next = (adk_pktio_task_t *)task_frags[frag_count];
            if (frag_count == 1)
            {
                chain_md_p->task_type = (uint8_t)ADK_TASK_TYPE_FWD;
            }
            else
            {
                chain_md_p->task_type = (uint8_t)ADK_TASK_TYPE_NEW;
            }
            chain_md_p->next_frag = (struct rte_mbuf *)task_next;
        }
        else
        {
            chain_md_p->task_type = (uint8_t)ADK_TASK_TYPE_NEW;
            chain_md_p->next_frag = NULL;
        }
    }

#ifndef ADK_DPDK_IPV6_FRAG
    /* Release the incoming packet 
    rte_pktmbuf_free(task_incoming);*/
#endif
return ADK_IPV6_DP_FRAG_OK; 
}


