/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ph_dev_dp.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_acl.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_flow_cache_lli.h"

#ifdef ADK_HOS_OLD_IMPL
#include "adk_ipv6_dp_hw_lli.h"
#include <adk_hos_dp.h>

#include "adk_ipv6_dp_lli.h"

inline adk_status_t
adk_ipv6_hw_offload_proc(struct rte_mbuf *task, adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
        adk_ipv6_hdr_t *ipv6_hdr, uint16_t src_port, uint16_t dst_port,
        uint8_t next_hdr, adk_ipv6_frag_t frag_flag, bool l4_port_present,  uint16_t offset_to_frag, 
        uint32_t frag_id, uint8_t excep)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_dsi_rx_metadata_t *dsi_md_p;
    adk_ipv6_dp_if_stats_entry_t *if_stats;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t rule_mask, stats_id;
    adk_ipv6_dest_flow_info_t flow_info;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_route_entry_t *route_entry = NULL;    
    adk_ipv6_dp_ingress_flow_sip_entry_t *ingress_flow_sip_entry = NULL;

    RTE_SET_USED(l4_port_present);

    adk_md_p = ADK_METADATA_PTR(task);
    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
    stats_id = ipv6_if_entry->ingress_stats_id;
    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][stats_id];
    /* check for bad IP header */
    if(adk_hos_is_ip_hdr_bad(dsi_md_p))
    {
        ADK_IPV6_DP_LOG_INFO_DEBUG("IP header validation failed \n");
        IPV6_DP_DROP_TASK(task);
        return ADK_E_TASK_CONSUMED;
    }

    ADK_IPV6_DP_LOG_INFO_DEBUG(" is acl valid    -   %d \n",adk_hos_is_acl_valid(dsi_md_p));
    ADK_IPV6_DP_LOG_INFO_DEBUG(" acl allow       -   %d \n",adk_hos_is_acl_allow(dsi_md_p));
    ADK_IPV6_DP_LOG_INFO_DEBUG(" is l3 term      -   %d \n",adk_hos_is_l3_term(dsi_md_p));
    ADK_IPV6_DP_LOG_INFO_DEBUG(" is fps destined -   %d \n",adk_hos_is_fps_destined(dsi_md_p));
    ADK_IPV6_DP_LOG_INFO_DEBUG(" is vip destined -   %d \n",adk_hos_is_vip_destined(dsi_md_p));
    ADK_IPV6_DP_LOG_INFO_DEBUG(" ipsec_offloaded -   %d \n",dsi_md_p->ipsec_offloaded);
    ADK_IPV6_DP_LOG_INFO_DEBUG(" is inner vip destined - %d \n",adk_hos_is_inner_vip_destined(dsi_md_p)); 

    /* 
     * outer header processing 
     */
    if(dsi_md_p->ipsec_offloaded && 
            ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_ETH)
    {
        /* Since IPsec offload done, need not to check ACL & reassembly */

        /* check if extension header parsing errors */
        if(excep)
        {
            if(excep == ADK_IPV6_HDR_ERR_EXCEP)
            {
                INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
                IPV6_DP_DROP_TASK(task);
                return ADK_E_TASK_CONSUMED;
            }

            /* raise an exception  */
            ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, adk_md_p->l4_offset, 
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
            if(ret != ADK_SUCCESS)
            {
                ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                        "%d \n", excep);
                IPV6_DP_DROP_TASK(task);
            }
            return ADK_E_TASK_CONSUMED;
        }

        /* assuming default rule mask for terminated traffic */ 
        rule_mask = IPV6_INGRESS_ASSOC_DEFAULT_MASK;

        /* check for default rulemask/vif termination */
        if((!(adk_hos_is_fps_destined(dsi_md_p))) || 
                (adk_hos_is_vip_destined(dsi_md_p)))
        {
            ret = adk_ipv6_dp_route_entry_lookup(ipv6_if_entry->vrd_id,
                    (uint8_t *) &ipv6_hdr->dst_addr[0], &route_entry); 
            if(ret != ADK_SUCCESS)
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG("route entry not found, "
                        "fallback to slow path to raise excep \n");
                return ADK_E_UNSUPPORTED;
            }

            rule_mask = route_entry->ingress_assoc_rulemask; 
            if ( ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE == route_entry->action)
            {
                stats_id = route_entry->virtual_if_stats_id;
                if_stats = (adk_ipv6_dp_if_stats_entry_t *)
                    &adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
            }
        }

        /* terminate packet to destination module */
        ret = adk_ipv6_dp_terminate(task, rule_mask, ipv6_if_entry, if_stats, ipv6_hdr, 
                frag_flag, next_hdr, (adk_md_p->l4_offset-adk_md_p->ip_offset), 
                src_port, dst_port);
        if(ret == ADK_E_TASK_CONSUMED)
        {
            return ret;
        }
        else if( ret == ADK_SUCCESS)
        {
            /* send the packet to destination */
            if( adkDestEntryFunction_g[flow_info.stageInfo.fn_id])
            {
                /*update the destFlow */
                adk_md_p->pdu_flow_id = flow_info.destFlow;

                ret  = adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1,1,
                        &flow_info.stageInfo, ADK_METADATA_PARAM(task, lock_id) ,
                        0 , ADK_TASK_TYPE_FWD);
                if(ret != 1)
                {
                    IPV6_DP_DROP_TASK(task);
                }
            }
            ret = ADK_E_TASK_CONSUMED;
        }
        else
        {
            IPV6_DP_DROP_TASK(task);
            ret = ADK_E_TASK_CONSUMED;
        }
    }

    /*
     * inner header / Plain text termination processing
     */
    else if(((!ipv6_if_entry->enable_ingress_acl) ||
                (adk_hos_is_acl_valid(dsi_md_p) && adk_hos_is_acl_allow(dsi_md_p)))&&
            (adk_hos_is_l3_term(dsi_md_p)))
    {
        /* check if it is received from Tunnel & SPD is not valid */
        if((ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_TUNNEL) && 
                (!adk_hos_is_spd_id_valid(dsi_md_p)))
        {
                ADK_IPV6_DP_LOG_INFO_DEBUG("fallback to normal processing " 
                        "HOS SPD lookup is not success \n");
                return ADK_E_UNSUPPORTED;
        }
        /* 
         * SIP validation 
         */
        ret = adk_ipv6_dp_ingress_flow_sip_entry_lookup(ipv6_if_entry->vrd_id,
                (uint8_t *) &ipv6_hdr->src_addr[0], &ingress_flow_sip_entry);
        /*
         * check the SIP validation if the lookup is successful, action is not equal to allow.
         * then drop/send packet to host 
         * if the apply to fwd flag is enabled then take action after route lookup 
         */
        if(ret == ADK_SUCCESS)  
        {
            if(!(ingress_flow_sip_entry->apply_to_fwd_traffic))
            {
                if( ingress_flow_sip_entry->action != ADK_IPV6_ACTION_ALLOW )
                {
                    adk_ipv6_dp_handle_sip_excep(ingress_flow_sip_entry, task,
                            ipv6_if_entry->vrd_id, if_stats);
                    return ADK_E_TASK_CONSUMED;
                }
            }
        }

        /* assuming default rule mask for terminated traffic */ 
        rule_mask = IPV6_INGRESS_ASSOC_DEFAULT_MASK; 

        /* check for default rulemask/vif termination */
        if((!(adk_hos_is_fps_destined(dsi_md_p))) || 
           ((adk_hos_is_inner_vip_destined(dsi_md_p))&& 
             (ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_TUNNEL))||
           ((adk_hos_is_vip_destined(dsi_md_p))&&
             (ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_ETH)))
        {
            ret = adk_ipv6_dp_route_entry_lookup(ipv6_if_entry->vrd_id,
                    (uint8_t *) &ipv6_hdr->dst_addr[0], &route_entry); 
            if(ret != ADK_SUCCESS)
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG("route entry not found, "
                        "fallback to slow path to raise excep \n");
                return ADK_E_UNSUPPORTED;
            }

            if(!((route_entry->action == ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE) ||
                (route_entry->action == ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE)))
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG("inner header is not terminated \n");
                return ADK_E_UNSUPPORTED;
            }

            rule_mask = route_entry->ingress_assoc_rulemask; 
            if ( ADK_IPV6_ROUTE_LOOKUP_ACTION_VIF_TERMINATE == route_entry->action)
            {
                stats_id = route_entry->virtual_if_stats_id;
                if_stats = (adk_ipv6_dp_if_stats_entry_t *)
                    &adk_ipv6_dp_tables->ipv6_dp_virtual_if_stats_table[coreIndex][stats_id]; 
            }
        }

        /* check if extension header parsing errors */
        if(excep)
        {
            /* raise an exception  */
            ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, adk_md_p->l4_offset, 
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
            if(ret != ADK_SUCCESS)
            {
                ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
                        "%d \n", excep);
                IPV6_DP_DROP_TASK(task);
            }
            return ADK_E_TASK_CONSUMED;
        }

        /* check if packet is fragmented & need reassembly 
         * (TODO refer reassembly flag from route lookup) */
        if(ipv6_if_entry->enable_reassembly && 
                (frag_flag != ADK_IPV6_UNFRAGMENTED_PKT))
        {
            /* increment packet req for reassembly */
            INC_NON_NULL(if_stats, ipv6IfStatsReasmReqds, 1);

            /* reassembly metadata */
            adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
                (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
            reass_md_p->frag_id = frag_id;
            reass_md_p->rulemask = rule_mask;
            reass_md_p->offset_to_frag = offset_to_frag;
            reass_md_p->if_stats = if_stats;    

            ret = adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1, 1, 
                    &(adk_ipv6_dp_tables->ipv6_dp_reassmbly_stage_info), 
                    (frag_id & ADK_IPV6_REASSEMBLY_HASH_MASK), 0, ADK_TASK_TYPE_FWD);
            if(ret != 1)
            {
                IPV6_DP_DROP_TASK(task);
            }
            return ADK_E_TASK_CONSUMED;

            /* update the reassembled task metadata address*/
            adk_md_p = ADK_METADATA_PTR(task);
        }

        /* terminate packet to destination module */
        ret = adk_ipv6_dp_terminate(task, rule_mask, ipv6_if_entry, if_stats, ipv6_hdr, 
                frag_flag, next_hdr, (adk_md_p->l4_offset-adk_md_p->ip_offset), 
                src_port, dst_port);
        if(ret == ADK_E_TASK_CONSUMED)
        {
            return ret;
        }
        else if( ret == ADK_SUCCESS)
        {
            /* send the packet to destination */
            if(adkDestEntryFunction_g[flow_info.stageInfo.fn_id])
            {
                adk_md_p->pdu_flow_id = flow_info.destFlow;
                ret  = adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1,1,
                        &flow_info.stageInfo, ADK_METADATA_PARAM(task, lock_id) ,
                        0 , ADK_TASK_TYPE_FWD);
                if(ret != 1)
                {
                    IPV6_DP_DROP_TASK(task);
                }
            }
            ret = ADK_E_TASK_CONSUMED;
        }
        else
        {
            if(ADK_SUCCESS != ret)
            {
                IPV6_DP_DROP_TASK(task);
            }
            ret = ADK_E_TASK_CONSUMED;
        }
    }
    else if(adk_hos_is_acl_valid(dsi_md_p) && (!adk_hos_is_acl_allow(dsi_md_p)))
    {
        ADK_IPV6_DP_LOG_INFO("error: IPV6 ACL drop action \n");

        IPV6_DP_DROP_TASK(task);
        ret = ADK_E_TASK_CONSUMED;
    }
    return ret;
}
#endif
