/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ph_dev_dp.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_acl.h"
#include "adk_ipv6_dp_pbr.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_flow_cache_lli.h"
#include "adk_sif_dp.h"
#include "adk_ipv6_utility_counters.h"

#ifdef ADK_DEVICE_SNR
#include <adk_hos_dp.h>
#include "adk_ipv6_dp_hw_lli.h"
#endif


/* global fast-path database pointer */
extern adk_ipv6_dp_tables_t *adk_ipv6_dp_tables;
extern uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES];

adk_status_t
adk_ipv6_dp_terminate(adk_pktio_task_t *task, uint8_t rulemask,
        adk_ipv6_dp_interface_entry_t *ipv6_if_entry,  adk_ipv6_dp_if_stats_entry_t *if_stats,
        adk_ipv6_hdr_t *ipv6_hdr, adk_ipv6_frag_t frag_flag, uint8_t next_hdr, uint16_t l4_offset,
        uint16_t src_port, uint16_t dst_port)
{
    uint32_t len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    uint8_t coreIndex = ADK_LCORE_INDEX;
    adk_status_t ret = ADK_SUCCESS;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    uint16_t ipv6_len = 0;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    RTE_SET_USED(rulemask);
    RTE_SET_USED(ipv6_if_entry);
    RTE_SET_USED(if_stats);
    RTE_SET_USED(frag_flag);
    RTE_SET_USED(l4_offset);

    ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (len_nh_hlimit>>16);
    /* do IPv6 ingress classification lookup */
    ret = adk_ipv6_dp_ingress_classification_entry_lookup(task, adk_md_p->input_intf_id, next_hdr, ipv6_hdr->src_addr, 
            ipv6_hdr->dst_addr, src_port, dst_port, ipv6_len, l4_offset);
    if(ret != ADK_E_TASK_CONSUMED)
    {
        /* drop the packet */
        ADK_IPV6_DP_LOG_ERROR("error: while IPv6 Ingress classification lookup \n");
        return (ADK_DP_E_ERROR);
    }
    else
    {
        return ADK_E_TASK_CONSUMED;
    }
    return ret;
}

void
adk_ipv6_increment_rx_l4_stats(uint8_t next_hdr, uint16_t ipv6_len, int coreIndex)
{
    adk_ipv6_dp_l4counter_statistics_t   *ipv6_l4proto_stats = NULL;
    ipv6_l4proto_stats = adk_ipv6_dp_tables->ipv6_dp_l4counter_stats_table[coreIndex];

    if (next_hdr == ADK_IPV6_NEXT_HDR_TCP)
    {
        INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpInPackets, 1);
        INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpInOctets, ipv6_len);
    }
    else if (next_hdr == ADK_IPV6_NEXT_HDR_SCTP)
    {
        INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpInPackets, 1);
        INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpInOctets, ipv6_len);
    }
}

uint32_t adk_ipv6_compare_multicast_mac(uint8_t *calc_dmac, uint8_t *dst_mac)
{
    uint32_t ret = ADK_SUCCESS;

    for (uint8_t i =0; i<=5; i++)
    {
        if (calc_dmac[i] != dst_mac[i])
        {
            ret = ADK_DP_E_ERROR;
        }
    }
    return ret;
}
void adk_ipv6_compute_multicast_dest_mac(uint32_t dst_address, uint8_t *calc_dmac)
{
    calc_dmac[5] = (dst_address & 0xff);
    calc_dmac[4] = (dst_address >> 8 & 0xff);
    calc_dmac[3] = (dst_address >> 16 & 0xff);
    calc_dmac[2] = (dst_address >> 24 & 0xff);
    calc_dmac[1] = 0x33;
    calc_dmac[0] = 0x33;
}

adk_status_t
adk_ipv6_dp_validate_multicast_mac( uint32_t *dst_ip, void *pduData_p)
{
    uint32_t ret = ADK_SUCCESS;
    uint8_t calc_dmac[6], dst_mac[6];
    uint32_t dst_address = adk_ntohl(dst_ip[3]);
    uint8_t *pdu;
    pdu = (uint8_t *) pduData_p;

    ADK_IPV6_COMPUTE_MULTICAST_DEST_MAC(dst_address, calc_dmac);
    rte_memcpy(dst_mac,&pdu[0],(sizeof(uint8_t) * 6));
    ret = ADK_IPV6_COMPARE_MULTICAST_MAC(calc_dmac, dst_mac);
    return ret;
}


static inline int send_to_ipv6_uli(void **task_p, uint32_t task_mask,
                                      uint8_t task_count)
{
    uint8_t ret = ADK_E_DROP_TASK;
    uint16_t destFunc = ADK_IPV6_ULI_FN_ID;
    adk_pktio_task_t **task;

    task = (adk_pktio_task_t **) task_p;
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task[0]);
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *)&(adk_md_p->imc_param_start);
    adk_ipv6_uli_md_p->sender = IP6_ULI_SENDER_IP6LLI;
    adk_ipv6_uli_md_p->pktOrigin = IP6_ULI_SENDER_IP6LLI;

	if (adkDestEntryFunction_g[destFunc])
	{   
		ret = adk_pal_dp_stage_send((void **)task_p, task_count, task_mask, destFunc, 0x00,
				0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
		if(ret == ADK_E_INTERNAL_ERROR)
		{  
			ret = ADK_DP_E_ERROR;
			return ret;
		}
	}

    return ret;
}

static inline int send_to_ipsec_lli(void **task_p, uint8_t next_hdr,  uint32_t task_mask,
        uint8_t task_count, uint8_t action)
{
    uint8_t ret = ADK_E_DROP_TASK;
    uint16_t destFunc = ADK_IPSEC_LLI_FN_ID;
    adk_pktio_task_t **task = (adk_pktio_task_t **) task_p;
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task[0]);
    adk_ipsec_dp_lli_imc_param_t * ipseclliImc = (adk_ipsec_dp_lli_imc_param_t *)(&(adk_md_p->imc_param_start));
    action == ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD ? (ipseclliImc->isLocal = false):(ipseclliImc->isLocal = true);
    ipseclliImc->ipv6_nextHdr = next_hdr;
    ipseclliImc->pktOrigin = IP6_ULI_SENDER_IP6LLI;
    /*l4_offset is already updated in the ADK metadata*/
    ADK_IPV6_DP_LOG_DEBUG("l4offset: %u\n", adk_md_p->l4_offset);

    ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS1_TS();
   
    if (adkDestEntryFunction_g[destFunc])
    {   
        ret = adk_pal_dp_stage_send((void **)task_p, task_count, task_mask, destFunc, 0x00,
                0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
        if(ret == ADK_E_INTERNAL_ERROR)
        {   
            ret = ADK_DP_E_ERROR;
            return ret;
        }
    }

    ADK_IPV6_DP_LOG_DEBUG("imcForIpsecLLI: isLocal=%d", ipseclliImc->isLocal);
    return ret;
}

static uint8_t adk_validate_ipv6_header(adk_pktio_task_t  *task, uint16_t pdu_length, 
                        uint16_t ipv6_len, uint32_t ver_tc_flabel,
                        adk_ipv6_dp_if_stats_entry_t *if_stats, uint16_t if_id, 
                        uint8_t ip_offset, adk_ipv6_hdr_t *ipv6_hdr, bool is_unicast, int coreIndex
#ifdef ADK_UTILITY_COUNTER_ENABLE
                  , ipv6_utility_counters_t *ipv6_utility_stats
#endif
        )
{
    uint8_t ret = ADK_SUCCESS;

    if (pdu_length < ADK_IPV6_HDR_LEN)
    {
        ADK_IPV6_DP_LOG_ERROR(
                "error: Too short to contain a valid IPv6 header %d \n", (ver_tc_flabel>> 28));

        /* increment the header errors */
        INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);

        ret = ADK_E_DROP_TASK;
    }

    /* Check for IPv6 version */
    if((ver_tc_flabel>> 28)  != ADK_IPV6_PDU_VER_VALUE)
    {
        ADK_IPV6_DP_LOG_ERROR(
                "error: invalid ipv6 version %d \n", (ver_tc_flabel>> 28));

        /* increment the header errors */
        /*TODO: Check if we need new counter. Gen-2 had a verion mismatch counter */
        INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidVersion);

        /*raise an exception */
        ret = adk_dp_send_ipv6_exception(task, 0, task->port,
                if_id, ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION, coreIndex, 0);
        if(unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                    " ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION \n");
            ret = ADK_E_DROP_TASK;
        }
        ret = ADK_E_TASK_CONSUMED;
    }

    if((ipv6_len != pdu_length))
    {
        /* check if it is truncated pdu */
        if(pdu_length < ipv6_len)
        {
            ADK_IPV6_DP_LOG_ERROR("error: truncated PDU \n");

            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);

            if ( (adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xFF)
            {
                INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, (ipv6_len-pdu_length));
            }
            else
            {
                INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, pdu_length);
            }
            INC_NON_NULL(if_stats, ipv6IfStatsInTruncatedPkts, 1);
            ret = ADK_E_DROP_TASK;
        }
    }
    
    if(is_unicast)
    {
        uint8_t *ethHdr;
        ADK_PDU_PTR_GET(task, ethHdr, uint8_t *);
        if(unlikely(ethHdr[0] & 0x1))
        {
            ADK_IPV6_DP_LOG_ERROR(
                    "error: PDU contains unicast DIP with multicast L2\n");

            /* increment the header errors */
            INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxUcastDstIpMcastDstMacDrop);
            ret = ADK_E_DROP_TASK; 
        }
    }

    return ret;
}

static uint8_t adk_ipv6_dp_validate_interface(adk_metadata_t *adk_md_p,
         int coreIndex, adk_ipv6_dp_if_stats_entry_t  **if_stats,
         adk_ipv6_dp_interface_entry_t    **ipv6_if_entry)
{
    adk_status_t ret = ADK_DP_E_ERROR;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    *ipv6_if_entry = &adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[adk_md_p->input_intf_id];

    if (likely(ipv6_if_entry))
    {
        *if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][(*ipv6_if_entry)->ingress_stats_id];
        ret = ADK_SUCCESS;
    }
    else
    {
        /* incr the gbl count */
        INC_NON_NULL(ipv6_global_stats, InvalidIfFail, 1);
    }
    ADK_IPV6_DP_LOG_DEBUG("received on interface id %d \n", adk_md_p->input_intf_id);
    return ret;
}

static uint8_t adk_ipv6_outer_hdr_esp_processing(adk_pktio_task_t *task, 
        uint16_t ipv6_len, adk_ipv6_dp_ip_addr_stats_t *ipv6_addr_stats,
        uint8_t ipsec_data, adk_ipv6_hdr_t  *ipv6_hdr, uint16_t pdu_length,
        adk_metadata_t *adk_md_p, uint32_t task_mask, uint8_t task_count,
        int coreIndex)
{
    adk_status_t ret = ADK_DP_E_ERROR;
    uint32_t frag_id =0, len_nh_hlimit = 0;
    uint16_t l4_offset = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0;
    uint8_t next_hdr=0, excep = 0;
    uint8_t action=ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
    uint8_t ipv6_addr_stats_id = 0;
    bool l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_qos_ds_phb_class_t   phb_class;
    uint8_t ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) & IPV6_HDR_DSCP_MASK) >> IPV6_HDR_DSCP_LOC;
    adk_ipv6_dp_route_entry_t  *route_entry = NULL;
    adk_ipv6_dp_interface_entry_t    *ipv6_if_entry = NULL;
    adk_ipv6_dp_if_stats_entry_t     *if_stats = NULL;
    adk_ipv6_dp_global_statistics_t  *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    RTE_SET_USED(ipsec_data);

    if (unlikely(adk_ipv6_dp_validate_interface(adk_md_p, coreIndex, &if_stats, &ipv6_if_entry)) != ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
        goto adk_ipv6_outer_hdr_esp_processing_return;
    }
    /* extract the L4 fields */
    l4_offset = adk_ipv6_parse_l4_ports(pdu_length, ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF), (uint8_t *)ipv6_hdr, 
            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present, &offset_to_frag,
            &frag_id, &excep);

    /* update the adk metadata l4 offset, relative to mbuf data offset */
    adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;
    ADK_METADATA_PARAM(task, outerIpStatsId) = ADK_SYS_INVALID_MDATA_ID;
    ADK_METADATA_PARAM(task, outerIPVersion)  = ADK_IPV6_VER;

    if (ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_QOS_ENABLE)
    {
        phb_class = ipv6_if_entry->dscpToPhbClass[ipv6_dscp];
        if (ADK_QOS_DS_PHB_CLASS_NONE != phb_class) 
        {
            /*Increment RX QoS statistics for PHB*/
            INC_NON_NULL(if_stats, ipv6ifRxQoSstats[phb_class].rxPkts, 1);
            INC_NON_NULL(if_stats, ipv6ifRxQoSstats[phb_class].rxBytes, ipv6_len);
        }
    }
    ret = adk_ipv6_dp_route_entry_lookup(ADK_IPV6_LOCAL_ROUTING_TABLE,
            (uint8_t *) &ipv6_hdr->dst_addr[0], &route_entry);
    if (likely(route_entry != NULL))
    {
        ipv6_addr_stats_id = route_entry->ipv6_addr_stats_id;
        action = ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE;
        ADK_IPV6_DP_LOG_DEBUG("IPv6 route action %d \n",action);
    }
    else
    {
        action = ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
        INC_NON_NULL(if_stats, ipv6IfStatsInForwDatagrams, 1);
    }

    ret = adk_validate_ipv6_header(task, pdu_length, ipv6_len, adk_ntohl(ipv6_hdr->ver_tc_flabel), if_stats,
            adk_md_p->input_intf_id, adk_md_p->ip_offset, ipv6_hdr, 1, coreIndex
#ifdef ADK_UTILITY_COUNTER_ENABLE
            , ipv6_utility_stats 
#endif
            );
    if (unlikely(ret != ADK_SUCCESS))
        goto adk_ipv6_outer_hdr_esp_processing_return;
    
    if (ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE == action)
    {
        /*! Get address stats */
        ipv6_addr_stats = &(adk_ipv6_dp_tables->ipv6AddrStatsTble[coreIndex][ipv6_addr_stats_id]);
        ipv6_if_entry->ipv6_addr_stats = ipv6_addr_stats;

        /* Increment interface and address stats */
        INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);

        INC_NON_NULL(ipv6_addr_stats, ipv6InPackets, 1);
        INC_NON_NULL(ipv6_addr_stats, ipv6InOctets, ipv6_len);

        if(!ADK_SYS_IPSEC_BYPASS_GET)
        {
            /* setting outerIpStatsId in adk metadata to outer dip stats id. This will be used in v4 as well as v6
             * to compare against inner ip stats id
             */
            ADK_METADATA_PARAM(task, outerIpStatsId) = ipv6_addr_stats_id;

            /* Received from ETH LLI and ipsec bypass is FALSE, Hence sending to IPSec LLI */;
            ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktSendToIpsecLli);
            ret = send_to_ipsec_lli((void **) &task, next_hdr, task_mask, task_count,
                    ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE);
            goto adk_ipv6_outer_hdr_esp_processing_return;
        }
        else
        {
            ADK_IPV6_DP_LOG_ERROR("IP LLI: IPsec is not enabled. Hence dropping");
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_outer_hdr_esp_processing_return;
        }
// TODO: Decrypted packets always need to be sent to IPSec. Dont see any use-case, 
// where decrypted packets are sent directly for Termination. Hence commenting it out for now.
#if 0
        adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
        adk_int_md_p->is_outer_ip = true;
        ret = adk_ipv6_dp_terminate(task, 0, ipv6_if_entry, if_stats,
                ipv6_hdr, frag_flag, next_hdr, l4_offset, src_port, dst_port);
        goto adk_ipv6_outer_hdr_esp_processing_return;
#endif

    }
    else // if (ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD == action)
    {
        if (ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_FORWARDING_ENABLE)
        {
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            /* Hoplimit validation */
            if(unlikely((adk_ntohl(ipv6_hdr->len_nh_hlimit) & 0xFF) <= 1))
            {
                INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxHopLimitZero);

                /*raise an exception */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO, coreIndex, 0);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                            "ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO \n");
                    ret = ADK_E_DROP_TASK;
                    goto adk_ipv6_outer_hdr_esp_processing_return;
                }
                ret = ADK_E_TASK_CONSUMED;
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }
            else
            {
                len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
                len_nh_hlimit--;
                ipv6_hdr->len_nh_hlimit = adk_htonl(len_nh_hlimit);
            }
            if (!ADK_SYS_IPSEC_BYPASS_GET)
            {
                /* Received non-locally terminated packet from ETH LLI and ipsec bypass is FALSE,
                 * Hence sending to IPSec LLI
                 */
                ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktSendToIpsecLli);
                ret = send_to_ipsec_lli((void **)&task, next_hdr, task_mask, task_count, action);
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }
            else
            {
                ADK_IPV6_DP_LOG_DEBUG("Forwarding to IPv6 ULI\n");
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktFwdToIpv6Uli);
                ret = send_to_ipv6_uli((void **)&task, task_mask, task_count);
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }
        }
        else
        {
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            INC_NON_NULL(if_stats, ipv6IfForwardingDisableDrop, 1);
            ADK_IPV6_DP_LOG_ERROR("Forwarding is disabled \n");

            /*raise an exception */
            ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                    ADK_IPV6_INTERFACE_FORWARDING_DISABLE, coreIndex, 0);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                        "ADK_IPV6_INTERFACE_FORWARDING_DISABLE \n");
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }
            ret = ADK_E_TASK_CONSUMED;
            goto adk_ipv6_outer_hdr_esp_processing_return;
        }
    }

adk_ipv6_outer_hdr_esp_processing_return:
    return ret;
}

static uint8_t adk_ipv6_inner_hdr_esp_processing( adk_pktio_task_t *task,
        adk_ipv6_dp_interface_entry_t *ipv6_if_entry, uint16_t ipv6_len,
        adk_ipv6_dp_if_stats_entry_t  *if_stats, adk_ipv6_dp_ip_addr_stats_t *ipv6_addr_stats,
        uint8_t ipsec_data, adk_ipv6_hdr_t  *ipv6_hdr,
        uint16_t pdu_length, adk_metadata_t *adk_md_p, int coreIndex)
{
    adk_status_t ret = ADK_DP_E_ERROR;
    uint32_t frag_id =0;
    uint16_t l4_offset = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0;
    uint8_t next_hdr=0, excep = 0;
    uint8_t ipv6_addr_stats_id = 0;
    bool l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ph_dev_dp_metadata_t  *phMeta_p = NULL;
    adk_ipv6_ph_dev_rx_param_t  *ipv6_ph_params = NULL;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p = NULL;
    
    /* extract the L4 fields */
    l4_offset = adk_ipv6_parse_l4_ports(pdu_length, ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF), (uint8_t *)ipv6_hdr,
            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present, &offset_to_frag,
            &frag_id, &excep);

    /* update the adk metadata l4 offset, relative to mbuf data offset */
    adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;

    /*! Get address stats */
    ipv6_addr_stats_id = adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(task));
    ipv6_addr_stats = &(adk_ipv6_dp_tables->ipv6AddrStatsTble[coreIndex][ipv6_addr_stats_id]);

    /* check if extension header parsing errors */
    if(excep)
    {
        INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);

        if(excep == ADK_IPV6_HDR_ERR_EXCEP)
        {
            ret = ADK_DP_E_ERROR;
            goto adk_ipv6_inner_hdr_esp_processing_return;
        }

        switch(excep)
        {
        case ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE:
            /* raise an exception  */
            ret = adk_dp_send_ipv6_exception(task, 0, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                        "%d \n", excep);
                ret = ADK_E_DROP_TASK;
            }
            goto adk_ipv6_inner_hdr_esp_processing_return;
            break;
        case ADK_IPV6_HOP_BY_HOP_EXTENSION_NOT_FIRST_EXTENSION_HEADER:
        case ADK_IPV6_EXCEPTION_SEGMENTS_LEFT_NON_ZERO:
        case ADK_IPV6_HOP_BY_HOP_EXTENSION_HEADER_FOR_NON_ICMPV6_PROTOCOL:
            phMeta_p = (adk_ph_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,imc_param_start));
            ipv6_ph_params = (adk_ipv6_ph_dev_rx_param_t *)&phMeta_p->imc_param_start;
            ipv6_ph_params->exception_specific_data.paramProbData.paramProbType = excep;

            /* raise an exception  */
            ret = adk_dp_send_ipv6_exception(task, 0, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_PARAM_PROBLEM, coreIndex, 0);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception ADK_IPV6_EXCEPTION_PARAM_PROBLEM \n");
                ret = ADK_E_DROP_TASK;
            }
            goto adk_ipv6_inner_hdr_esp_processing_return;
            break;
        }

    }
    /* do reassembly if fragmented */
    if (frag_flag != ADK_IPV6_UNFRAGMENTED_PKT)
    {

        /* reassembly metadata */
        adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
            (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
        reass_md_p->frag_id = frag_id;
        reass_md_p->rulemask = 0;
        reass_md_p->offset_to_frag = offset_to_frag;
        reass_md_p->if_stats = if_stats;
        reass_md_p->ipv6_addr_stats = ipv6_addr_stats;
        reass_md_p->ipsec_data = ipsec_data;
        reass_md_p->is_outer_ip = false;
#ifdef ADK_IPV6_FLOW_CACHE
        reass_md_p->l4_port_present = l4_port_present;
        reass_md_p->vif_flag = vif_flag;
        reass_md_p->vif_stats_id = 0;

        reass_md_p->fc_hit = FALSE;
        reass_md_p->hash_value = hash_value;
        reass_md_p->in_acl_stats_id = in_acl_stats_id;
        reass_md_p->next_hdr = next_hdr;
        reass_md_p->src_port = src_port;
        reass_md_p->dst_port = dst_port;
#endif
#if defined(ADK_PAL_DYNAMIC_PPL)
		if (adkDestEntryFunction_g[ADK_IPV6_LLI_REASSEMBLY_FN_ID])
		{   
			ret = adk_pal_dp_stage_send((void **)&task, 1, 0x1, ADK_IPV6_LLI_REASSEMBLY_FN_ID, 0x00,
					0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
			if(ret == ADK_E_INTERNAL_ERROR)
			{   
				ret = ADK_DP_E_ERROR;
			}
            goto adk_ipv6_inner_hdr_esp_processing_return;
		}
#else 
        ret = adk_ipv6_dp_reassembly((void **)&task, 1, 1, NULL, NULL, coreIndex);
        if(likely(ret == ADK_E_TASK_CONSUMED))
        {
            goto adk_ipv6_inner_hdr_esp_processing_return;
        }
        else
        {
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_inner_hdr_esp_processing_return;
        }
#endif

    }

    if (((adk_md_p->outerIPVersion == ADK_IPV6_VER) &&
            (ipv6_addr_stats_id != adk_md_p->outerIpStatsId)) ||
            (adk_md_p->outerIPVersion == ADK_IPV4_VER))
    {
        INC_NON_NULL(ipv6_addr_stats, ipv6InPackets, 1);
        INC_NON_NULL(ipv6_addr_stats, ipv6InOctets, ipv6_len);
    }

    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
    adk_int_md_p->is_outer_ip = false;
    adk_int_md_p->ipv6_addr_stats = ipv6_addr_stats;

    ret = adk_ipv6_dp_chksum_validate(task, frag_flag, next_hdr, l4_offset, dst_port,
            ipv6_hdr, if_stats, ipv6_if_entry, 0);
    if (unlikely((ret == ADK_DP_E_ERROR) || (ret == ADK_E_TASK_CONSUMED) ))
    {
        ret = (ret == ADK_DP_E_ERROR) ? ADK_E_DROP_TASK : ADK_E_TASK_CONSUMED;
        goto adk_ipv6_inner_hdr_esp_processing_return;
    }
    ret = adk_ipv6_dp_terminate(task, 0, ipv6_if_entry, if_stats,
            ipv6_hdr, frag_flag, next_hdr, l4_offset, src_port, dst_port);
    if(likely(ret == ADK_E_TASK_CONSUMED))
    {
        goto adk_ipv6_inner_hdr_esp_processing_return;
    }
    else
    {
        ret = ADK_E_DROP_TASK;
        goto adk_ipv6_inner_hdr_esp_processing_return;
    }
adk_ipv6_inner_hdr_esp_processing_return:
    return ret;
}

static uint8_t adk_ipv6_outer_hdr_plain_processing(adk_pktio_task_t *task,
        uint16_t ipv6_len, adk_ipv6_dp_ip_addr_stats_t *ipv6_addr_stats,
        uint8_t ipsec_data, adk_ipv6_hdr_t  *ipv6_hdr, uint16_t pdu_length,
        adk_metadata_t *adk_md_p, adk_ipv6_dp_global_statistics_t *ipv6_global_stats,
        uint32_t task_mask, uint8_t task_count, int coreIndex)
{
    adk_status_t ret = ADK_DP_E_ERROR;
    uint32_t frag_id =0, len_nh_hlimit = 0;
    uint16_t l4_offset = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0;
    uint8_t next_hdr=0, excep = 0;
    uint8_t action=ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
    uint8_t ipv6_addr_stats_id = 0;
    bool l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_qos_ds_phb_class_t   phb_class;
    adk_ph_dev_dp_metadata_t  *phMeta_p = NULL;
    adk_ipv6_ph_dev_rx_param_t  *ipv6_ph_params = NULL;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p = NULL;
    adk_ipv6_dp_if_stats_entry_t     *if_stats = NULL;
    adk_ipv6_dp_interface_entry_t        *ipv6_if_entry = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    uint8_t ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) & IPV6_HDR_DSCP_MASK) >> IPV6_HDR_DSCP_LOC;

    if (unlikely(adk_ipv6_dp_validate_interface(adk_md_p, coreIndex, &if_stats, &ipv6_if_entry)) != ADK_SUCCESS)
    {
        ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
        goto adk_ipv6_outer_hdr_plain_processing_return;
    }
    /* extract the L4 fields */
    l4_offset = adk_ipv6_parse_l4_ports(pdu_length, ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF), (uint8_t *)ipv6_hdr,
            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present, &offset_to_frag,
            &frag_id, &excep);

    /* update the adk metadata l4 offset, relative to mbuf data offset */
    adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;
    ADK_METADATA_PARAM(task, outerIpStatsId) = ADK_SYS_INVALID_MDATA_ID;
    ADK_METADATA_PARAM(task, outerIPVersion)  = ADK_IPV6_VER;


    if (ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_QOS_ENABLE)
    {
        phb_class = ipv6_if_entry->dscpToPhbClass[ipv6_dscp];
        if (ADK_QOS_DS_PHB_CLASS_NONE != phb_class) 
        {
            /*Increment RX QoS statistics for PHB*/
            INC_NON_NULL(if_stats, ipv6ifRxQoSstats[phb_class].rxPkts, 1);
            INC_NON_NULL(if_stats, ipv6ifRxQoSstats[phb_class].rxBytes, ipv6_len);
        }
    }

    if (TRUE == adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(task)))
    {
        action = ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE;
    }
    /*Commenting the below check, as HOS offload support is not in place for outer IP header validation
     *Will enable back, once we have right design in place*/ 
    //if((unlikely(adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(task[i]))))
    if(1)
    {
        //Since ip header is invalid, processing further to handle counter statistics
        ret = adk_validate_ipv6_header(task, pdu_length, ipv6_len, adk_ntohl(ipv6_hdr->ver_tc_flabel), if_stats, 
                adk_md_p->input_intf_id, adk_md_p->ip_offset, ipv6_hdr, 1, coreIndex
#ifdef ADK_UTILITY_COUNTER_ENABLE
                , ipv6_utility_stats
#endif
                );
        if (ret != ADK_SUCCESS)
            goto adk_ipv6_outer_hdr_plain_processing_return;
    }
    if(ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE == action)
    {
        /*! Get address stats */
        ipv6_addr_stats_id = adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(task));
        ipv6_addr_stats = &(adk_ipv6_dp_tables->ipv6AddrStatsTble[coreIndex][ipv6_addr_stats_id]);
        ipv6_if_entry->ipv6_addr_stats = ipv6_addr_stats;

        /* check if extension header parsing errors */
        if(excep)
        {
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
            INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
            if(excep == ADK_IPV6_HDR_ERR_EXCEP)
            {
                ret = ADK_DP_E_ERROR;
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }

            switch(excep)
            {
            case ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE:
                /* raise an exception  */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                        adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                        ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                            "%d \n", excep);
                    ret = ADK_E_DROP_TASK;
                }
                goto adk_ipv6_outer_hdr_plain_processing_return;
                break;
            case ADK_IPV6_HOP_BY_HOP_EXTENSION_NOT_FIRST_EXTENSION_HEADER:
            case ADK_IPV6_EXCEPTION_SEGMENTS_LEFT_NON_ZERO:
            case ADK_IPV6_HOP_BY_HOP_EXTENSION_HEADER_FOR_NON_ICMPV6_PROTOCOL:
                phMeta_p = (adk_ph_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,imc_param_start));
                ipv6_ph_params = (adk_ipv6_ph_dev_rx_param_t *)&phMeta_p->imc_param_start;
                ipv6_ph_params->exception_specific_data.paramProbData.paramProbType = excep;

                /* raise an exception  */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                        adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset,
                        ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_PARAM_PROBLEM, coreIndex, 0);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception ADK_IPV6_EXCEPTION_PARAM_PROBLEM \n");
                    ret = ADK_E_DROP_TASK;
                }
                goto adk_ipv6_outer_hdr_plain_processing_return;
                break;
            }
        }
        /* do reassembly if fragmented */
        if (frag_flag != ADK_IPV6_UNFRAGMENTED_PKT)
        {
            /* increment packet req for reassembly */
            INC_NON_NULL(if_stats, ipv6IfStatsReasmReqds, 1);

            /* reassembly metadata */
            adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
                (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
            reass_md_p->frag_id = frag_id;
            reass_md_p->rulemask = 0;
            reass_md_p->offset_to_frag = offset_to_frag;
            reass_md_p->if_stats = if_stats;
            reass_md_p->ipv6_addr_stats = ipv6_addr_stats;
            reass_md_p->ipsec_data = ipsec_data;
            reass_md_p->is_outer_ip = true;
#if defined(ADK_PAL_DYNAMIC_PPL)
            if (adkDestEntryFunction_g[ADK_IPV6_LLI_REASSEMBLY_FN_ID])
            {   
                ret = adk_pal_dp_stage_send((void **)&task, task_count, task_mask, ADK_IPV6_LLI_REASSEMBLY_FN_ID, 0x00,
                        0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
                if(ret == ADK_E_INTERNAL_ERROR)
                {   
                    ret = ADK_DP_E_ERROR;
                }
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
#else 
            ret = adk_ipv6_dp_reassembly((void **)&task, 1, 1, NULL, NULL, coreIndex);
            if(likely(ret == ADK_E_TASK_CONSUMED))
            {
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
            else
            {
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
#endif
        }
        else 
        {
            l4_offset = adk_ipv6_parse_header_exception(
                    ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF), (uint8_t *)ipv6_hdr, &excep);
            /* check if extension header parsing errors */
            if(unlikely(excep))
            {
                /* raise an exception  */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                        adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset, 
                        ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
                if(ret != ADK_SUCCESS)
                {
                    INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                            "%d \n", excep);
                    ret = ADK_E_DROP_TASK;
                    goto adk_ipv6_outer_hdr_plain_processing_return;
                }
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
        }
        /* Increment interface and address stats */
        INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);

        INC_NON_NULL(ipv6_addr_stats, ipv6InPackets, 1);
        INC_NON_NULL(ipv6_addr_stats, ipv6InOctets, ipv6_len);
        if(!ADK_SYS_IPSEC_BYPASS_GET && ipsec_data == IPLLI_IMC_ETH_OUTER)
        {
            /* setting outerIpStatsId in adk metadata to outer dip stats id. This will be used in v4 as well as v6
             * to compare against inner ip stats id
             */
            ADK_METADATA_PARAM(task, outerIpStatsId) = ipv6_addr_stats_id;

            /* Received from ETH LLI and ipsec bypass is FALSE, Hence sending to IPSec LLI */;
            ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktSendToIpsecLli);
            ret = send_to_ipsec_lli((void **)&task, next_hdr, task_mask, task_count,
                    ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE);
            goto adk_ipv6_outer_hdr_plain_processing_return;
        }
        adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
        adk_int_md_p->is_outer_ip = true;
        ret = adk_ipv6_dp_chksum_validate(task, frag_flag, next_hdr, l4_offset, dst_port,
                ipv6_hdr, if_stats, ipv6_if_entry, 0);
        if (unlikely((ret == ADK_DP_E_ERROR) || (ret == ADK_E_TASK_CONSUMED) ))
        {
            ret = (ret == ADK_DP_E_ERROR) ? ADK_E_DROP_TASK : ADK_E_TASK_CONSUMED;
            goto adk_ipv6_outer_hdr_plain_processing_return;
        }
        ret = adk_ipv6_dp_terminate(task, 0, ipv6_if_entry, if_stats,
                ipv6_hdr, frag_flag, next_hdr, l4_offset, src_port, dst_port);
        if(likely(ret == ADK_E_TASK_CONSUMED))
        {
            goto adk_ipv6_outer_hdr_plain_processing_return;
        }
        else
        {
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_outer_hdr_plain_processing_return;
        }
    }
    else if (ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD == action)
    {
        if (ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_FORWARDING_ENABLE)
        {
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            /* Hoplimit validation */
            if(unlikely((adk_ntohl(ipv6_hdr->len_nh_hlimit) & 0xFF) <= 1))
            {   
                INC_NON_NULL(if_stats, ipv6IfStatsInHdrErrors, 1);
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxHopLimitZero);
                
                /*raise an exception */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO, coreIndex, 0);
                if(unlikely(ret != ADK_SUCCESS))
                {   
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                            "ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO \n");
                    ret = ADK_E_DROP_TASK;
                    goto adk_ipv6_outer_hdr_plain_processing_return;
                }
                ret = ADK_E_TASK_CONSUMED;
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
            else
            {
                len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
                len_nh_hlimit--;
                ipv6_hdr->len_nh_hlimit = adk_htonl(len_nh_hlimit);
            }
            if (!ADK_SYS_IPSEC_BYPASS_GET)
            {
                /* Received non-locally terminated packet from ETH LLI and ipsec bypass is FALSE,
                 * Hence sending to IPSec LLI 
                 */
                ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktSendToIpsecLli);
                ret = send_to_ipsec_lli((void **)&task, next_hdr, task_mask, task_count, action);
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
            else
            {
                ADK_IPV6_DP_LOG_DEBUG("Forwarding to IPv6 ULI\n");
                INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktFwdToIpv6Uli);
                ret = send_to_ipv6_uli((void **)&task, task_mask, task_count); 
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
        }
        else
        {
            INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
            INC_NON_NULL(if_stats, ipv6IfForwardingDisableDrop, 1);
            ADK_IPV6_DP_LOG_ERROR("Forwarding is disabled \n");

            /*raise an exception */
            ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                    adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                    ADK_IPV6_INTERFACE_FORWARDING_DISABLE, coreIndex, 0);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                        "ADK_IPV6_INTERFACE_FORWARDING_DISABLE \n");
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_plain_processing_return;
            }
            ret = ADK_E_TASK_CONSUMED;
            goto adk_ipv6_outer_hdr_plain_processing_return;
        }
    }
    else
    {
        /*drop the packet */ 
        INC_NON_NULL(ipv6_global_stats, invalidRouteAction, 1);
        ADK_IPV6_DP_LOG_ERROR(
                "error: Invalid IPV6 DIP lookup action \n");
        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidRouteAction);
        ret = ADK_E_DROP_TASK;
        goto adk_ipv6_outer_hdr_plain_processing_return;
    }
    adk_ipv6_outer_hdr_plain_processing_return:
    return ret;
}

/*!
 * =============================================================
 * adk_ipv6_dp_pkt_proc_lli()
 *
 * structure of task params :
 *
 * taskparams[0] : upper 8-bits indicate eioa port number.
 * taskparams[1] : interface (or sub interface) handle.
 * taskparams[2] : ipsec data if received on tunnel interface
 *                 (from ipsec after decryption).
 * taskparams[3] : l2 header size.
 * taskparams[7] : pdu buffer pointing to ip header.
 *
 * brief:
 * this function is an interface to ethernet module and performs
 * following:
 *
 * a. route lookup.
 * b. acl lookup.
 * c. interface-level policing.
 * d. ingress association/advance lookup.
 * e. ingress classifier lookup
 * f. reassembly of fragmented packets.
 *
 * this function binds to following modules depending on packet processing:
 * 1. binds to ipsec lli - adk_ipsec_dp_pkt_proc_lli for decryption of esp
 * packet received from tn port.
 * 2. binds to gtpu lli - adk_gtpu_dp_pkt_proc_lli for termination of inner
 * packet received on ipsec tunnel.
 * 3. binds to ethernet uli - adk_eth_dp_pkt_proc_uli for normal ipv6
 * forwarding (tn->wifi, wifi->tn).
 *
 * =============================================================
 */
adk_dp_status_t
adk_ipv6_dp_pkt_proc_lli(void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask, __attribute__((unused)) void *ring,
                        __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        int coreIndex)
{

    adk_status_t ret = ADK_DP_E_ERROR;
    uint32_t cur_bit, i, frag_id =0;    
    uint16_t ipv6_len, pdu_length, l4_offset = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0;   
    uint8_t next_hdr=0, excep = 0;
    bool multicast_pkt = FALSE;void *pduData_p;
    bool l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_pktio_task_t **task, *next_frag;
    adk_ipv6_dp_interface_entry_t        *ipv6_if_entry = NULL;
    adk_ipv6_dp_if_stats_entry_t         *if_stats = NULL;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    adk_ipv6_dp_ip_addr_stats_t          *ipv6_addr_stats = NULL;
    uint8_t ipsec_data = 0; 
    adk_ipv6_hdr_t  *ipv6_hdr;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p = NULL;

    task = (adk_pktio_task_t **) task_p;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
#ifdef ADK_UTILITY_COUNTER_ENABLE
    ipv6_utility_counters_t  *ipv6_utility_stats = 
         adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
#endif

    ADK_IPV6_DP_LOG_DEBUG("received %d IPv6 frames in LLI \n", task_count);

    for(i=0, cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(!(task_mask & cur_bit))
            continue;
        while(task[i])
        {
            adk_md_p =  ADK_METADATA_PTR(task[i]);
            next_frag = adk_md_p->next_frag;
            ipsec_data = ((adk_ipv6_dp_lli_imc_data_t *) &(adk_md_p->imc_param_start))->ipsec_data;

            if (IPLLI_IMC_ETH_OUTER == ipsec_data)
            {
               ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS1_TS();
            }
            else
            {
               ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS2_TS();
            }
            
            ADK_PDU_OFFSET_PTR_GET(task[i], ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);
            ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (adk_ntohl(ipv6_hdr->len_nh_hlimit) >> 16);
            pdu_length = (task[i]->pkt_len - adk_md_p->ip_offset);

#if defined(ADK_DEBUG)
            ADK_IPV6_DP_LOG_INFO("IPv6 LLI_single entry::\n");

            ADK_IPV6_DP_LOG_INFO("ipIfId=0x%x, ip-offset=%d\n",
                    ADK_METADATA_PTR(task[i])->input_intf_id, ADK_MD_GET_IP_OFF(task[i]));

            ADK_IPV6_DP_LOG_INFO("ip-lli-imc::ipsec-data=0x%02x \n", ipsec_data);

            ADK_IPV6_DP_LOG_INFO("\n \tis-inline-decrypted=%u,\n \t is-L3-terminating=%u,\n \t is-classifier-invalid=%u,"
                    "\n \t classifier-rule-id=%u,\n \t is-ip-hdr-bad=%u,\n \t is-l4-cksum-err=%u,"
                    "\n \t is_ip_bcast=%u, \n \t is_ip_mcast=%u, \n \t is_ip_stats_invalid=%u, \n \t get_ip_stats_id=%u\n",
                    adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_l3_term(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_ip_hdr_bad(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_l4_chksum_err(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_ip_bcast(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_ip_mcast(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_is_ip_stats_invalid(ADK_DSI_RX_METADATA_PTR(task[i])),
                    adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(task[i]))
                    );

            if(!adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(task[i])))
            {
                uint32_t flow_id = adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(task[i]));
                ADK_IPV6_DP_LOG_INFO("\n \t classifier-action=%u\n",
                        (adk_ipv6_dp_tables->ingClassifierActionDp_p[adk_ipv6_dp_tables->ingClassifierInfo_p->aclTreeIndex][flow_id]).action);
            }
#endif
            /***** UNICAST PACKET HANDLING ******/
            if ((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) != 0xFF)
            {
                /*Since QAT was able to decrypt successfully, we assume following things
                 * 1. IP header is valid. So SW validation is not needed.
                 * 2. Outer IP is a terminating packet
                 */ 
                if(adk_hos_is_inline_decrypted(ADK_DSI_RX_METADATA_PTR(task[i])) && IPLLI_IMC_ETH_OUTER == ipsec_data)
                {
                    ret = adk_ipv6_outer_hdr_esp_processing(task[i], ipv6_len, ipv6_addr_stats,
                            ipsec_data, ipv6_hdr, pdu_length, adk_md_p, task_mask, task_count,
                            coreIndex);
                }
                else if (IPLLI_IMC_IPSEC_INNER_INLINE == ipsec_data)
                {
                    ret = adk_ipv6_inner_hdr_esp_processing(task[i], ipv6_if_entry, ipv6_len,
                            if_stats, ipv6_addr_stats, ipsec_data, ipv6_hdr, pdu_length, adk_md_p,
                            coreIndex);
                }
                else if (IPLLI_IMC_IPSEC_OUTER == ipsec_data)
                {
                    /* extract the L4 port */
                    l4_offset = adk_ipv6_parse_l4_ports(pdu_length, 
                            ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF), (uint8_t *)ipv6_hdr, 
                            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present, &offset_to_frag,
                            &frag_id, &excep);

                    if (unlikely(adk_ipv6_dp_validate_interface(adk_md_p, coreIndex, &if_stats, &ipv6_if_entry)) != ADK_SUCCESS)
                    {
                        ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }
                    /* update the adk metadata l4 offset, relative to mbuf data offset */
                    adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;
                    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
                    adk_int_md_p->is_outer_ip = true;

                    ret = adk_ipv6_dp_chksum_validate(task[i], frag_flag, next_hdr, l4_offset, dst_port,
                                                         ipv6_hdr, if_stats, ipv6_if_entry, 0);
                    if (unlikely((ret == ADK_DP_E_ERROR) || (ret == ADK_E_TASK_CONSUMED) ))
                    {
                        ret = (ret == ADK_DP_E_ERROR) ? ADK_E_DROP_TASK : ADK_E_TASK_CONSUMED;
                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }
                    ret = adk_ipv6_dp_terminate(task[i], 0, ipv6_if_entry, if_stats, 
                            ipv6_hdr, frag_flag, next_hdr, l4_offset, src_port, dst_port);
                    if(likely(ret == ADK_E_TASK_CONSUMED))
                    {
                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }
                    else
                    {
                        ret = ADK_E_DROP_TASK;
                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }
                }
                else 
                {
                    ret = adk_ipv6_outer_hdr_plain_processing(task[i], ipv6_len, ipv6_addr_stats,
                            ipsec_data, ipv6_hdr, pdu_length, adk_md_p, ipv6_global_stats, task_mask,
                            task_count, coreIndex);
                }

            }
            /***** MULTICAST PACKET HANDLING ******/
            else
            {
                ret = adk_validate_ipv6_header(task[i], pdu_length, ipv6_len, adk_ntohl(ipv6_hdr->ver_tc_flabel), if_stats,
                        adk_md_p->input_intf_id, adk_md_p->ip_offset, ipv6_hdr, 0, coreIndex
#ifdef ADK_UTILITY_COUNTER_ENABLE
                        , ipv6_utility_stats
#endif
                        );
                if (ret != ADK_SUCCESS)
                    goto adk_ipv6_dp_pkt_proc_lli_return;

                /* extract the L4 port */
                l4_offset = adk_ipv6_parse_l4_ports(pdu_length, ((adk_ntohl(ipv6_hdr->len_nh_hlimit)>>8)&0xFF),
                        (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag,
                        &l4_port_present, &offset_to_frag, &frag_id, &excep);

                /* update the adk metadata l4 offset, relative to mbuf data offset */
                adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;

                if (unlikely(adk_ipv6_dp_validate_interface(adk_md_p, coreIndex, &if_stats, &ipv6_if_entry)) != ADK_SUCCESS)
                {
                    ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
                    goto adk_ipv6_dp_pkt_proc_lli_return;
                }

                /*Check scope of a multicast packet, if scope is zero drop the packet */
                if (((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 16) & 0x000f) != 0x0000)
                {
                    multicast_pkt = TRUE;
                    /*The multicast packets also need to go to IPsec for IPsec policy lookup.
                     *The packet continues its path only if IPsec SPD allows it
                     */
                    ipsec_data = ((adk_ipv6_dp_lli_imc_data_t *) &(adk_md_p->imc_param_start))->ipsec_data;
                    if(!ADK_SYS_IPSEC_BYPASS_GET && ipsec_data == IPLLI_IMC_ETH_OUTER)
                    {
                        INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1);
                        INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxPktSendToIpsecLli);

                        ADK_METADATA_PARAM(task[i], outerIPVersion)  = ADK_IPV6_VER;
                        /* Received from ETH LLI and ipsec bypass is FALSE, Hence sending to IPSec LLI */;
                        ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
                        ret = send_to_ipsec_lli(task_p, next_hdr, task_mask, task_count,
                                ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE);

                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }

                    /*Check if this multicast IP is part of Multicast group.
                     *If yes, send to host/netdriver, else drop.
                     */
                    adk_ipv6_dp_ingress_mgroup_entry_t *ingress_mgroup_entry = NULL;
                    ret = adk_ipv6_dp_ingress_mgroup_entry_lookup(adk_md_p->input_intf_id, ipv6_hdr->src_addr,
                            ipv6_hdr->dst_addr, &ingress_mgroup_entry);
                    if(unlikely(ret != ADK_SUCCESS))
                    {
                        /* increment the stats */
                        /* drop the packet */
                        if (ipsec_data == IPLLI_IMC_ETH_OUTER)
                        {
                            INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1)
                            INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                        }
                        INC_NON_NULL(if_stats, ipv6IfStatsInMcastErrors, 1);
                        INC_NON_NULL(if_stats, ipv6IfInDiscardedMCastOctets, ipv6_len);
                        ADK_IPV6_DP_LOG_ERROR("error: while IPv6 Ingress Mcast Group lookup \n");
                        goto adk_ipv6_dp_pkt_proc_lli_return;
                    }
                    else
                    {
                        /*Get the pointer which points ethernet header */
                        ADK_PDU_PTR_GET(*task, pduData_p, void *);

                        ret = adk_ipv6_dp_validate_multicast_mac(ipv6_hdr->dst_addr, pduData_p);
                        if (unlikely(ret != ADK_SUCCESS))
                        {
                            if (ipsec_data == IPLLI_IMC_ETH_OUTER)
                            {
                                INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1);
                                INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                            }
                            INC_NON_NULL(if_stats, ipv6IfStatsInMcastErrors, 1);
                            INC_NON_NULL(if_stats, ipv6IfInDiscardedMCastOctets, ipv6_len);
                            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidMcastMac);

                            ADK_IPV6_DP_LOG_ERROR("error: Mac address is not a multicast mac address \n");
                            ret = ADK_E_DROP_TASK;
                            goto adk_ipv6_dp_pkt_proc_lli_return;
                        }
                        if(likely(ADK_IPV6_MCGROUP_ACTION_ISLOCAL == ingress_mgroup_entry->action))
                        {
                            if (ipsec_data == IPLLI_IMC_ETH_OUTER)
                            {
                                INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1);
                                INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                            }
                            ADK_IPV6_DP_LOG_DEBUG("Mcast group Lookup success: action is : %u\n", ingress_mgroup_entry->action);

                            adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
                            adk_int_md_p->is_outer_ip = true;
                            ret = adk_ipv6_dp_terminate(task[i], 0, ipv6_if_entry, if_stats, 
                                  ipv6_hdr, frag_flag, next_hdr, l4_offset, src_port, dst_port);
                            if(likely(ret == ADK_E_TASK_CONSUMED))
                            {
                                goto adk_ipv6_dp_pkt_proc_lli_return; 
                            }
                            else
                            {
                                ret = ADK_E_DROP_TASK;
                                goto adk_ipv6_dp_pkt_proc_lli_return;
                            }
                        }
                        else
                        {
                            INC_NON_NULL(if_stats, ipv6IfStatsInMcastErrors, 1);
                            INC_NON_NULL(if_stats, ipv6IfInDiscardedMCastOctets, ipv6_len);
                            if (ipsec_data == IPLLI_IMC_ETH_OUTER)
                            {
                                INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1);
                                INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                            }
                            ADK_IPV6_DP_LOG_ERROR("Invalid Mcast group action\n");
                            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidMcastGroup);
                            ret = ADK_E_DROP_TASK;
                            goto adk_ipv6_dp_pkt_proc_lli_return;
                        }

                    }
                }
                else
                {
                    /* Scope of multicast packet is zero, so dropping the packet */
                    INC_NON_NULL(if_stats, ipv6IfStatsInMcastPkts, 1);
                    INC_NON_NULL(if_stats, ipv6IfStatsHCInMcastOctets, ipv6_len);
                    INC_NON_NULL(if_stats, ipv6IfStatsInMcastErrors, 1);
                    INC_NON_NULL(if_stats, ipv6IfInDiscardedMCastOctets, ipv6_len);

                    INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidMcastScope);
                    ADK_IPV6_DP_LOG_ERROR("Error: Scope of multicast packet is zero \n");
                    ret = ADK_DP_E_ERROR;
                    goto adk_ipv6_dp_pkt_proc_lli_return;
                }
            }
adk_ipv6_dp_pkt_proc_lli_return:
            if(likely((ADK_E_TASK_CONSUMED == ret) || (ADK_SUCCESS == ret )))
            {
                task[i] = next_frag;
                continue;
            }
            else
            {
                adk_pal_dp_release_send(task[i], ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
                IPV6_DP_DROP_TASK(task[i]);
                return ret;
            }
        }
    }
    return ADK_SUCCESS;
}

adk_dp_status_t adk_ipv6_dp_ing_classifier_lookup(adk_ipv6_ing_classifier_ctx_t *ipv6IngClassifierCtx_p,
                adk_ipv6_ing_classifier_ptn_t  *ingClassifierPtn,
                uint32_t  *result_p)
{
        int ret;
        const uint8_t * data[] = {(uint8_t *)ingClassifierPtn};

        ret = rte_acl_classify_alg((struct rte_acl_ctx *)ipv6IngClassifierCtx_p, data,
                        result_p, 1, 1,
                        RTE_ACL_CLASSIFY_SSE);
        if (unlikely(ret != 0))    {
                ADK_IPV6_DP_LOG_ERROR("Error in rte_acl_classify_alg for IPv6 Ingress Classifier: %d", ret);
                return ADK_E_INTERNAL_ERROR;
        }
        return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_pkt_proc_police()
 *
 * structure of imc task params :
 *
 *
 * brief:
 * This is an entry fcuntion for IPv6 policer function.
 * does following:
 *
 * =============================================================
 */
adk_status_t 
adk_ipv6_dp_pkt_proc_police(void  **task_p,
                            uint8_t __attribute__((unused)) task_count,
                            uint32_t __attribute__((unused)) task_mask,
                            void __attribute__((unused)) *ring,
                            adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                            __attribute__((unused)) int coreIndex)
{
    adk_pktio_task_t *task = (adk_pktio_task_t*) *task_p;
    adk_ipv6_hdr_t  *ipv6_hdr;
    adk_metadata_t *adk_md_p=ADK_METADATA_PTR(task);
    uint8_t policeColor = ADK_QOS_DP_GREEN;
    adk_ipv6_dp_policer_meta_data_t *ipv6_policer_md_p = NULL;
    uint16_t ipv6_len = 0;
    uint8_t flow_id;
    uint32_t ingressIfId;
    adk_pal_dp_stage_info_t *em_lli_stage_info_p = NULL;
    uint8_t tasks_sent, fn_id;
    adk_dp_status_t dp_st;

    ipv6_policer_md_p = (adk_ipv6_dp_policer_meta_data_t *)&(ADK_METADATA_PARAM(task_p, imc_param_start));

    /* Get the ingress if id from l2 if handle */
    ingressIfId = adk_md_p->input_intf_id;

    ADK_PDU_OFFSET_PTR_GET(task_p, ipv6_hdr, adk_ipv6_hdr_t *,
                           adk_md_p->ip_offset);


    if(unlikely( ADK_SUCCESS !=
                adk_qos_dp_policing_color_blind(
                    ipv6_policer_md_p->ing_action.policerFlowId,
                    ipv6_len, &policeColor) ) )
    {
        /*Free the task*/
        adk_pktio_task_buffer_free(task);
        return ADK_SUCCESS;
    }

    flow_id = ipv6_policer_md_p->ingress_classifier_flow_id;
    /* check policing color */
    if(unlikely(ADK_QOS_DP_RED == policeColor))
    {
        /* Dropping the packet */
        adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex]->ipv6RateLimitingDrops++;
        adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].policerDropPkts++;
        adk_pktio_task_buffer_free(task);
        return ADK_SUCCESS;
    }

    if(ADK_METADATA_PARAM(task, event_sched_type) == RTE_SCHED_TYPE_ORDERED)
    {
        fn_id = ADK_SIF_EM_LLI_FN_ID;
    }
    else
    {
        fn_id = ADK_SIF_EM_LLI_DIRECT_FN_ID;
    }

    adk_sif_em_dp_lli_meta_data_t *em_lli_md_p = (adk_sif_em_dp_lli_meta_data_t *)&(ADK_METADATA_PARAM(task,
                imc_param_start));
    em_lli_md_p->eventMachineQueueId = ipv6_policer_md_p->ing_action.eventMachineQueueId;
    /*Get the stage info from PAL*/
    dp_st = adk_pal_dp_stage_info_get(fn_id,
            &em_lli_stage_info_p);
    if(dp_st != ADK_SUCCESS)
    {
        adk_pktio_task_buffer_free(task);
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Failed to get EM LLI stage from PAL");
        return ADK_E_INTERNAL_ERROR;
    }

    tasks_sent = adk_pal_dp_next_stage_event_handle_with_stage_info(
            (void **)&task, 1, 0x1,
            em_lli_stage_info_p, em_lli_md_p->eventMachineQueueId,
            0x00, ADK_METADATA_PARAM(task, task_type));
    if(tasks_sent == 0)
    {
        adk_pktio_task_buffer_free(task);
        ADK_DP_LOG_ERROR(ADK_ETHERNET_MODULE_ID,"Failed to enqueue to EM LLI stage");
        return ADK_E_INTERNAL_ERROR;
    }
    ADK_IPV6_DP_LOG_DEBUG("\nadk_em_send_to_eo() success!\n");
    adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].txPkts++;
    /* change return value, when this function is staged */
    return ADK_E_TASK_CONSUMED;
}

void adk_ipv6_dp_slowpath_policer_stats(void *data)
{
    adk_slowpath_user_param_t *params = NULL;
    if(!data)
    {
        ADK_IPV6_DP_LOG_ERROR("\nInvalid slow path parameters\n");
        return;
    }
    params = (adk_slowpath_user_param_t *)data;
    if((((adk_slowpath_user_param_t *)params)->policeFlowId != UINT_MAX))
    {
        if(params->pkt_color == ADK_QOS_DP_GREEN)
        {
            adk_ipv6_dp_tables->ipv6IngFlowStats[
                ADK_LCORE_INDEX * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + (params->flow_id)].txPkts++;

            adk_ipv6_increment_rx_l4_stats(params->l4_protocol, params->l4_len, ADK_LCORE_INDEX);
        }
        else
        {
            adk_ipv6_dp_tables->ipv6IngFlowStats[
                ADK_LCORE_INDEX * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + (params->flow_id)].policerDropPkts++;

            adk_ipv6_dp_tables->ipv6_dp_global_stats_table[ADK_LCORE_INDEX]->ipv6RateLimitingDrops++;
        }
    }
    else
    {
        adk_ipv6_dp_tables->ipv6IngFlowStats[
            ADK_LCORE_INDEX * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + (params->flow_id)].txPkts++;

        adk_ipv6_increment_rx_l4_stats(params->l4_protocol, params->l4_len, ADK_LCORE_INDEX);
    }
}
