/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ph_dev_dp.h"
#include "adk_ipv6_dp_pkt_capture.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_acl.h"
#include "adk_ipv6_dp_pbr.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_flow_cache_lli.h"

#ifdef ADK_DEVICE_SNR
#include <adk_hos_dp.h>
#include "adk_ipv6_dp_hw_lli.h"
#endif

   
void 
adk_ipv6_dp_packet_capture(adk_pktio_task_t *task,
        uint8_t vrd_id,
        adk_ipv6_dp_pkt_capture_info_t *capture_info,
        uint8_t __attribute__((unused)) coreIndex)
{

    RTE_SET_USED(task);
    RTE_SET_USED(vrd_id);
    RTE_SET_USED(capture_info);
    RTE_SET_USED(coreIndex);

    struct rte_mbuf *capture_mbuf = NULL, *new_mbuf, *next_mbuf, *prev_mbuf = NULL, *chain_mbuf = NULL;
    adk_metadata_t *capture_md_p, *adk_md_p;
    uint8_t *capture_pdu, *pdu, i, nb_segs;
    adk_status_t ret = ADK_SUCCESS;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
 
    if(capture_info->enqueue_status)
    {
        /* allocate mbuf */
        while(task)
        {
            new_mbuf = rte_pktmbuf_alloc(((struct rte_mbuf *)task)->pool);
            if(new_mbuf != NULL)
            {
                /*update the mbuf ptr to send */
                if(prev_mbuf == NULL)
                {
                    capture_mbuf=new_mbuf;
                }
                else
                {
                    (ADK_METADATA_PTR(prev_mbuf))->next_frag = new_mbuf;
                }

                /* update the mbuf */
                new_mbuf->port = task->port;
                new_mbuf->pkt_len = task->pkt_len;
                new_mbuf->data_len = task->data_len;
                new_mbuf->nb_segs = task->nb_segs;

                /*copy ADK meta data*/
                capture_md_p = ADK_METADATA_PTR(new_mbuf);
                adk_md_p = ADK_METADATA_PTR(task);
                rte_memcpy(capture_md_p, adk_md_p, 
                        (sizeof(adk_metadata_t)-sizeof(uintptr_t)));
                /* Setting the task type as New for captured packet */
                capture_md_p->task_type = ADK_TASK_TYPE_NEW;

                /* shall be zero, becuase the new mbuf shall be allocated 
                 * & initialised for the same */
                capture_md_p->next_frag = NULL;

                /*copy the DATA */
                ADK_PDU_PTR_GET(new_mbuf, capture_pdu, uint8_t *);
                ADK_PDU_PTR_GET(task, pdu, uint8_t *);
                rte_memcpy(capture_pdu, pdu, task->data_len); 

                /* check for multi segemented buffer */
                if(task->nb_segs > 1)
                {
                    prev_mbuf = new_mbuf;
                    nb_segs = task->nb_segs;
                    for(i=1; i <nb_segs; i++)
                    {
                        /* allocate mbuf */
                        next_mbuf = rte_pktmbuf_alloc(((struct rte_mbuf *)task)->pool);
                        task = task->next;
                        if(next_mbuf != NULL)
                        {
                            /* update the mbuf */
                            next_mbuf->pkt_len = task->pkt_len;
                            next_mbuf->data_len = task->data_len;

                            /* copy ADK meta data*/
                            rte_memcpy(ADK_METADATA_PTR(next_mbuf),
                                    ADK_METADATA_PTR(task),
                                    (sizeof(adk_metadata_t)-sizeof(uintptr_t)));
                            capture_md_p->next_frag = NULL;

                            /* copy the DATA */
                            ADK_PDU_PTR_GET(next_mbuf, capture_pdu, uint8_t *);
                            ADK_PDU_PTR_GET(task, pdu, uint8_t *);
                            rte_memcpy(capture_pdu, pdu, task->data_len);

                            /* mbuf chaining */
                            prev_mbuf->next = next_mbuf;
                            prev_mbuf = next_mbuf;
                        }
                    }
                }
                /* ADK chaining: parse next fragment */
                task = adk_md_p->next_frag;
                prev_mbuf = new_mbuf;
            }
            else
            {
                ipv6_global_stats->ipv6MbufAllocFailures++;
                ADK_IPV6_DP_LOG_INFO_DEBUG(
                        "error: unable to allocate pktmbuf for pkt capture \n");
                return;
            }
        }

        while(capture_mbuf != NULL)
        {
            adk_md_p = ADK_METADATA_PTR(capture_mbuf);
            chain_mbuf = adk_md_p->next_frag;

            ADK_IPV6_FILL_PH_DEV_PARAMS(capture_mbuf, capture_info->devId,
                capture_info->queueId, 0, capture_info->cbFnId, UINT_MAX,
                ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, vrd_id,
                0, capture_info->context, 0);

            ret = adk_ph_dev_dp_to_cp_send((void *)capture_mbuf, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
            /* raise an exception & forward the packet as well */
            if(ret != ADK_SUCCESS)
            {
                capture_info->enqueue_status = FALSE;
                ADK_IPV6_DP_LOG_INFO(
                        "error: IPV6 failed to send sampling packet\n");
                rte_pktmbuf_free((struct rte_mbuf *)capture_mbuf);
            }
            capture_mbuf = chain_mbuf;
        }
    }

    return;
}

adk_dp_status_t
adk_ipv6_dp_pkt_proc_packet_capture(void **task_p, uint8_t task_count, uint32_t task_mask, 
        void *ring, adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    RTE_SET_USED(task_count);
    RTE_SET_USED(ring);
    RTE_SET_USED(next_stage_entry);

#ifdef ADK_DEVICE_SNR
    uint32_t i, cur_bit;

    adk_ipv6_dp_interface_entry_t   *ipv6_if_entry = NULL;
    adk_dsi_rx_metadata_t           *dsi_md_p;
    adk_metadata_t                  *adk_md_p;
    adk_ipv6_dp_pkt_capture_info_t  *capture_info;
    adk_pktio_task_t                **task;
    adk_ipv6_dp_global_statistics_t    *ipv6_global_stats = NULL;
    uint16_t                        *ether_type;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    task = (adk_pktio_task_t **) task_p;
    
    ADK_IPV6_DP_LOG_INFO_DEBUG("Received packets at IPv6 cature function \n");

    for(i=0, cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(!(task_mask & cur_bit))
            continue;

        dsi_md_p = ADK_DSI_RX_METADATA_PTR(task[i]);
        adk_md_p = ADK_METADATA_PTR(task[i]);
        
        /* get the packet capture attributes */
        ret = adk_ipv6_dp_interface_entry_lookup((dsi_md_p->outer_id &
                    ADK_HOS_OUTER_IP_ID_MASK), 0,
                &ipv6_if_entry);
        if(ret != ADK_SUCCESS)
        {
            IPV6_DP_DROP_TASK(task[i]);
            ipv6_global_stats->ipv6HwPktCapDrops++;
            continue;
        }

#ifdef ADK_HOS_OLD_IMPL
        if(adk_hos_is_pcap_egress(dsi_md_p))
        {
            adk_md_p->exception_domain = ADK_FP_ORIGIN_DOMAIN;
            capture_info = &ipv6_if_entry->eg_pkt_capture_info;
        }
        else
        {
            capture_info = &ipv6_if_entry->ing_pkt_capture_info;
        }
#else
            capture_info = &ipv6_if_entry->ing_pkt_capture_info;
#endif        
        
        /* update the IP offset - double tagged packets are not parsed */
        ADK_PDU_OFFSET_PTR_GET(task[i], ether_type, uint16_t *, 12);
        adk_md_p->ip_offset =(*ether_type == ETHER_TYPE_VLAN)?18:14;

        if(capture_info->enqueue_status)
        {
            /* send captured packet to control thread */
            ADK_IPV6_FILL_PH_DEV_PARAMS(task[i], capture_info->devId,
                    capture_info->queueId, 0, capture_info->cbFnId, UINT_MAX,
                    ADK_IPV6_EXCEPTION_PH_CPU_PATH_TYPE, ipv6_if_entry->vrd_id,
                    0, capture_info->context, 0);

            ret = adk_ph_dev_dp_to_cp_send((void *)task[i], ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
            /* raise an exception & forward the packet as well */
            if(ret != ADK_SUCCESS)
            {
                capture_info->enqueue_status = FALSE;
                ADK_IPV6_DP_LOG_INFO(
                        "error: IPV6 failed to send sampling packet\n");
                IPV6_DP_DROP_TASK(task[i]);
                continue;
            }
            ADK_IPV6_DP_LOG_INFO_DEBUG("Sent captured packets to PH \n");
        }
        else
        {
            ADK_IPV6_DP_LOG_INFO(
                    "error: IPV6 packet capture enqueue status disabled \n");
            IPV6_DP_DROP_TASK(task[i]);
            ipv6_global_stats->ipv6HwPktCapDrops++;
            continue;
        }
    }
#else
    RTE_SET_USED(task_p);
    RTE_SET_USED(task_mask);
    RTE_SET_USED(coreIndex);
#endif
    return ret; 
}

