/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_lli.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_flow_cache_lli.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_reassembly.h"
#include "adk_ipv6_dp_pbr.h"
#include "adk_hos_conf.h"

extern uint32_t pbr_policy_set_id;
static uint32_t activeReasmContexts;
extern uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES];
static inline adk_dp_status_t
adk_ipv6_dp_reassembly_key_create( uint32_t __attribute__((unused)) *src_addr,
        uint32_t __attribute__((unused))*dst_addr, uint32_t frag_id,
        uint32_t *key)
{
    /* as the entries are added & looked by datapath,
     * hash function is simplified.
     * Need to update if required later 
     */ 
    *key = adk_ntohl(frag_id);

    return ADK_SUCCESS;
}
 
static inline adk_dp_status_t
adk_ipv6_dp_cmp_frag_tuples(adk_ipv6_dp_reassembly_entry_t *reass_entry,
        uint64_t *src_addr, uint64_t *dst_addr, uint32_t frag_id)
{
    adk_dp_status_t ret = ADK_SUCCESS;

    ret = frag_id ^ reass_entry->reass_tuple.frag_id;
    ret += ((reass_entry->reass_tuple.src_addr[0]) ^ (src_addr[0])); 
    ret += ((reass_entry->reass_tuple.src_addr[1]) ^ (src_addr[1])); 
    ret += ((reass_entry->reass_tuple.dst_addr[0]) ^ (dst_addr[0])); 
    ret += ((reass_entry->reass_tuple.dst_addr[1]) ^ (dst_addr[1])); 

    return ret;
}

static inline void
adk_ipv6_dp_reset_reass_entry(adk_ipv6_dp_reassembly_entry_t **reass_entry)
{
    (*reass_entry)->nb_frags = 0;
    (*reass_entry)->total_size = UINT16_MAX;
    (*reass_entry)->frag_size = 0;
    (*reass_entry)->err_detected = 0;
    (*reass_entry)->is_reassembly_success = 0;
    (*reass_entry)->frag_idx = ADK_IPV6_MIN_FRAG_IDX;
    (*reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task = NULL;
    (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].task = NULL;
    (*reass_entry)->fg[2].task = NULL;
    (*reass_entry)->fg[3].task = NULL;
}

static inline void 
adk_ipv6_dp_reset_reass_hash_entry(adk_ipv6_dp_reassembly_hash_entry_t **reass_hash_entry)
{
    ADK_MEMSET((uint8_t *)(*reass_hash_entry), 0, sizeof(adk_ipv6_dp_reassembly_hash_entry_t));
    return;
}

/* 
 * This function releases all the stored buffers 
 */
static inline adk_dp_status_t
adk_ipv6_release_frag(adk_ipv6_dp_reassembly_entry_t **reass_entry)
{
    int i;
    for(i=0; i<(*reass_entry)->frag_idx; i++)
    {
        if((*reass_entry)->fg[i].task)
        {
            rte_pktmbuf_free((struct rte_mbuf*)((*reass_entry)->fg[i].task));
        }
    }
    return ADK_SUCCESS;
}

#ifdef ADK_TIMER
static void 
adk_ipv6_dp_reassembly_cleanup(uint16_t hash_id, uint8_t collision_id, uint8_t reass_ver_id,
                                    adk_ipv6_dp_if_stats_entry_t  *if_stats)
{
    adk_status_t ret;
    uint16_t lock_idx, idx;
    adk_ipv6_dp_reassembly_hash_entry_t *reass_hash_entry;
    adk_ipv6_dp_reassembly_entry_t *reass_entry;
    struct rte_mbuf *excep_pkt;
    adk_metadata_t *adk_md_p;
    uint8_t coreIndex = ADK_LCORE_INDEX;

    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    lock_idx = hash_id * ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;
    idx = lock_idx + collision_id;

   /* Acquire Hash id based Lock */
    rte_spinlock_lock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    /* get the hash entry */
    reass_hash_entry = &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[idx]; 
    reass_entry = reass_hash_entry->reass_entry;
    ADK_IPV6_DP_LOG_INFO_DEBUG("timeout frag index %d \n", reass_entry->frag_idx);
 
    /* Add validation checks here */
    /* (Race condition of reassembly timeout & reassembly success)
     * check if the Reassembly hash entry is valid.
     * check if the reassembly was successfull & it is freed before timeout cleanup */
    if( !reass_hash_entry->is_enabled || reass_entry == NULL)
    {
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);
        return;
    }
 
    if(reass_ver_id != reass_entry->reass_ver_id)
    {
        ipv6_global_stats->ipv6ReassTimedVerMismatch++;
        rte_spinlock_unlock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);
        return;
    }

    /*update the reassembly failure count */
    INC_NON_NULL(if_stats, ipv6IfStatsReasmFails, 1);

    /*check if first frag received, 
     * then raise an exception else use some other fragment */ 
    if((reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task)
    {
        excep_pkt = (reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task;
        (reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task = NULL;

        /* raise an exception here */
        adk_md_p = ADK_METADATA_PTR(excep_pkt);
        ret = adk_dp_send_ipv6_exception(excep_pkt, (reass_entry)->vrd_id, 
                ((adk_pktio_task_t *)excep_pkt)->port, (reass_entry)->if_id, adk_md_p->ip_offset,
                0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED,
                ADK_LCORE_INDEX, 0);
        if(ret != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_INFO_DEBUG( 
                    "Error while raising IPv6 reassembly timer exception \n");
            
            /*release the task */
            rte_pktmbuf_free((struct rte_mbuf*)excep_pkt);
        }
    }

    /* reset the ipv6 reassembly entry */
    /* release the fragments */
    adk_ipv6_release_frag(&reass_entry);

    /* reset the reassembly entry */
    /* reset the hash index */
    reass_entry->collosion_idx = 0;
    reass_entry->hash_idx      = 0; 
    adk_ipv6_dp_reset_reass_entry(&reass_entry);

    /* reset the hash entry */
    adk_ipv6_dp_reset_reass_hash_entry(&reass_hash_entry);
    
    /* release the reass entry to pool */
    rte_mempool_put(adk_ipv6_dp_tables->ipv6_dp_reassembly_table.tbl, reass_entry);

    __atomic_fetch_sub(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);

    rte_spinlock_unlock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    return;
}
#endif

/* The total number of reassembly contexts, is divided into four stages.
* Stage 1: supports timeout value timeval : for contexts occupancy of fist 1/3rd of total contexts
* Stage 2: supports timeout value as percentageChangeInTimeout of timeval: for contexts occupancy of next 1/6th of total contexts
* stage 3: supports timeout value as percentageChangeInTimeout of stage 2 timeout value: for contexts occupancy of next 1/6th of total contexts
* stage 4: supports timeout value minTimeVal : for contexts occupancy of remaining contexts
* ******************************************************************************************************
* Example:  If the configuration is like : adk_ipv6_reassembly_timeout_config(ip_handle, 10000, 20, 100)
* And we will take number of Max available contexts as 16000 for ease of computation (actual value is 16384)
* stage 1: First 5333 (16000/3) contexts will be configured with reassembly timer of 10000 milliseconds
* stage 2: Next 2666 (16000/6) contexts will be configured with reassembly timer of 8000 milliseconds  (10000 * 80%)
* stage 3: Next 2666 (16000/6) contexts will be configured with reassembly timer of 6400 milliseconds  (8000 * 80%)
* stage 4: Last 5335 contexts will be configured with the reassembly timer of 100 milliseconds.
* ******************************************************************************************************
*/
static inline void adk_ipv6_dp_get_staggered_timeout_value(adk_timer_info_t *timeval)
{
    if(activeReasmContexts < ADK_IPV6_MAX_REASSEMBLY_CONTEXT/3)
        timeval->interval_milli_sec = adk_ipv6_dp_tables->reasmStage1;
    else if(activeReasmContexts < ADK_IPV6_MAX_REASSEMBLY_CONTEXT/2)
        timeval->interval_milli_sec = adk_ipv6_dp_tables->reasmStage2;
    else if(activeReasmContexts < (ADK_IPV6_MAX_REASSEMBLY_CONTEXT*2)/3)
        timeval->interval_milli_sec = adk_ipv6_dp_tables->reasmStage3;
    else
        timeval->interval_milli_sec = adk_ipv6_dp_tables->reasmStage4;
}

static void 
adk_ipv6_dp_reassembly_timeout(void *timer_cb_info,
                   __attribute__((unused)) adk_timer_event_type_t event_type)
{
    adk_metadata_t *timer_stage_md_p;
    adk_ipv6_dp_reass_stage_meta_data_t *reass_stage_imc_md = NULL;
    adk_ipv6_dp_reassembly_entry_t *reass_entry = NULL;
    adk_ipv6_dp_reassembly_hash_entry_t *reass_hash_entry = NULL;
    struct rte_mbuf *task = NULL;
    struct rte_mempool *pktpool = NULL;
    uint16_t hash_id, lock_idx, idx, queue_id, flow_id;
    uint8_t reass_ver_id, collision_id, coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;

    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    /*check if the event type is TIMEOUT, then delete the timer mbuf */
    if(event_type == ADK_TIMER_EVENT_TYPE_DELETE)
    {
        return;
    }


    /* event type is ADK_TIMER_EVENT_TYPE_TIMEOUT */
    reass_ver_id = ((size_t)timer_cb_info)>>16;
    hash_id     = (((size_t)timer_cb_info)>>3) & ADK_IPV6_REASSEMBLY_HASH_MASK;
    collision_id = (((size_t)timer_cb_info)&0x7);
    lock_idx = hash_id*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;
    idx = lock_idx + collision_id;


    rte_spinlock_lock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    reass_hash_entry = &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[idx];
    reass_entry = reass_hash_entry->reass_entry;

    /* (Race condition of reassembly timeout & reassembly success)
     * check if the Reassembly hash entry is valid.
     * check if the reassembly was successfull & it is freed before timeout cleanup */
    if( !reass_hash_entry->is_enabled || reass_entry == NULL)
    {
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);
        return;
    }

    if(reass_entry->ipsecCb)
    {
        ipv6_global_stats->ipv6ReassTimedContextFailures = reass_entry->ipsecCb(lock_idx, collision_id);
    }
    else
    {
        ipv6_global_stats->ipv6ReassTimedContextFailures++;
    }

    if(reass_ver_id != reass_entry->reass_ver_id)
    {
        ipv6_global_stats->ipv6ReassTimedVerMismatch++;
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);
        return;
    }
    if_stats = reass_entry->if_stats;
    queue_id = reass_entry->queue_id;
    flow_id = reass_entry->flow_id;

    rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    adk_sys_config_pktio_mempool_get((void **)&pktpool);
    if(pktpool == NULL)
    {
        ADK_IPV6_DP_LOG_INFO(
                "Error unable to get pktio mempool !!!\n");
        adk_ipv6_dp_reassembly_cleanup(hash_id, collision_id, reass_ver_id, if_stats);
        return;
    }

    task = rte_pktmbuf_alloc(pktpool);
    if (unlikely(task == NULL)) {
        ADK_IPV6_DP_LOG_INFO_DEBUG(
                "Error whle allocating mbuf for Reassembly timer!!!\n");
        adk_ipv6_dp_reassembly_cleanup(hash_id, collision_id, reass_ver_id, if_stats);
        ipv6_global_stats->ipv6MbufAllocFailures++;
        return;
    }

    /* update the Queue Id, Flow Id, & Reassembly Indexes */
    timer_stage_md_p = ADK_METADATA_PTR(task);

    reass_stage_imc_md = (adk_ipv6_dp_reass_stage_meta_data_t *)
        &timer_stage_md_p->imc_param_start;

    reass_stage_imc_md->queue_id  = queue_id;
    reass_stage_imc_md->flow_id   = flow_id;
    reass_stage_imc_md->hash_id   = hash_id;
    reass_stage_imc_md->if_stats  = if_stats;
    reass_stage_imc_md->collision_id = collision_id;
    reass_stage_imc_md->reass_ver_id = reass_ver_id;

    /* post an event to SQM for serialising the timeout event*/
#if defined(ADK_PAL_DYNAMIC_PPL)
    uint32_t ret = 0;
    ret = adk_pal_dp_stage_send((void **)&task, 1, 0x1, ADK_IPV6_LLI_REASS_TIMER_FN_ID, 0x00,
                                0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
   if(ret == ADK_E_INTERNAL_ERROR)
    {
        ipv6_global_stats->ipv6ReassTimedEnqueueFailures++;
        adk_pal_dp_release_send(task, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
        rte_pktmbuf_free(task);
        adk_ipv6_dp_reassembly_cleanup(hash_id, collision_id, reass_ver_id, if_stats);
        return ;
    }
#else
    /*upon failure/success timout function releases the timer mbuf */
    adk_ipv6_dp_reassembly_timeout_proc((void **)&task, 1, 0x1, NULL, NULL,
                                   coreIndex);
#endif

    return;
}
/*
 * This function checks for the re-assembly tuple match & return the reassembly table
 * if the entry not present, 
 *  New entry will be created & returned.
 * 
 * function returns an error if the reassembly collisions are reached max.
 *
 */ 
static inline bool 
adk_ipv6_dp_reassembly_hash_lookup(struct rte_mbuf *task,
        uint32_t *src_addr, uint32_t *dst_addr,
        uint32_t frag_id, uint32_t key, uint16_t if_id,
        uint8_t  *excep, adk_ipv6_dp_reassembly_entry_t **reass_entry,
        adk_ipv6_dp_reassembly_hash_entry_t **reass_hash_entry,
        uint8_t  vrd_id, adk_ipv6_dp_if_stats_entry_t *if_stats,
        uint8_t __attribute__((unused)) coreIndex,
	clear_ipsec_ctx_for_hash ipsecCb,
        uint8_t packetType,
        uint32_t preFragBufferTimeOutVal)
{
    uint8_t i, free_hash_entry = UINT8_MAX;
    uint16_t hash_idx, idx, lock_idx;
    size_t timer_cb_info;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
    adk_timer_info_t timer_info = {IPV6_REASSEMBLY_TIMEOUT};
    int ret;
#endif
#endif

    hash_idx = (key&ADK_IPV6_REASSEMBLY_HASH_MASK);
    lock_idx = hash_idx*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;
    idx = lock_idx + 1; /* 0th position reserved for Lock */

    (*reass_hash_entry) = &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[idx];

    for(i=1; i<(ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES); (*reass_hash_entry)++, i++)
    {
        if((((*reass_hash_entry)->hash_value == key) && (*reass_hash_entry)->first_frag))
        {
            *reass_entry = (*reass_hash_entry)->reass_entry;
            /* check for re-assembly tuples */
            if(!(adk_ipv6_dp_cmp_frag_tuples((*reass_entry), 
                            (uint64_t *)&src_addr[0], 
                            (uint64_t *)&dst_addr[0], 
                            frag_id)))
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG("reaassembly tuples matched \n");
                return TRUE;
            }
        }
        else
        {
            if((free_hash_entry > i) && (!(*reass_hash_entry)->is_enabled)) 
                free_hash_entry = i; 
        }
    }

    /*if entry not found 
       - insert hash entry if not full
       - else return error 
    */
    ADK_IPV6_DP_LOG_INFO_DEBUG("Entry not found!!! "
            "Adding new reass hash entry for frag_id %d \n", frag_id);

    if(i == ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES)
    {
        if(free_hash_entry != UINT8_MAX)
        {
           
            if(rte_mempool_get(adk_ipv6_dp_tables->ipv6_dp_reassembly_table.tbl, (void **)reass_entry))
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG(
                        "Reassembly hash table full !!! \n");
                *excep = ADK_IPV6_REASS_LIST_FULL;
                ipv6_global_stats->ipv6ReassTableFullFailures++;
                return FALSE;
            }

#ifdef ADK_TIMER
#ifdef ADK_DP_MAINTENANCE
            (*reass_entry)->queue_id = ADK_METADATA_PARAM(task, event_queue_id);  
            (*reass_entry)->flow_id  = ADK_METADATA_PARAM(task, lock_id); /*(frag_id & ADK_IPV6_REASSEMBLY_HASH_MASK)*/;
            (*reass_entry)->if_stats = if_stats;
	    (*reass_entry)->ipsecCb = ipsecCb;
 
            /* start the reassembly timer */
            if(packetType == ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD)
                timer_info.interval_milli_sec = preFragBufferTimeOutVal; /*Use IpSec timeout value*/
            else
                adk_ipv6_dp_get_staggered_timeout_value(&timer_info); /*Use IPv6 reassembly timeout value*/
 
            timer_cb_info = ((((*reass_entry)->reass_ver_id+1)<<16)|
                               (hash_idx<<3)|
                               free_hash_entry);
            ret = adk_timer_create(/*coreIndex*/ ADK_LCORE_INDEX, 
                    timer_info, 
                    adk_ipv6_dp_reassembly_timeout, 
                    (void *)timer_cb_info,
                    &((*reass_entry)->timer_hdl));    
            if(!ret)
            {
                ADK_IPV6_DP_LOG_INFO_DEBUG(
                        "Error whle registering Reassembly timer!!! \n");
                *excep = ADK_IPV6_REASS_TIMER_CREATE_ERR;
                ipv6_global_stats->ipv6ReassTimerCreateFailures++; 
                return FALSE;
            }
#endif
#endif
            (*reass_entry)->reass_ver_id++;

            /* update the hash table */
            (*reass_hash_entry) =
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx+free_hash_entry];
            (*reass_hash_entry)->hash_value = key;
	    (*reass_hash_entry)->first_frag = 1;
            (*reass_hash_entry)->is_enabled = TRUE;
            (*reass_hash_entry)->reass_entry = (*reass_entry);

            /* update the re-assembly table */
            adk_ipv6_dp_reset_reass_entry(reass_entry);

            (*reass_entry)->hash_idx = hash_idx;
            (*reass_entry)->collosion_idx = free_hash_entry;
            rte_memcpy((*reass_entry)->reass_tuple.src_addr, src_addr, sizeof(adk_ipv6_addr_t));
            rte_memcpy((*reass_entry)->reass_tuple.dst_addr, dst_addr, sizeof(adk_ipv6_addr_t));
            (*reass_entry)->reass_tuple.frag_id = frag_id;
            (*reass_entry)->if_id = if_id;
            (*reass_entry)->vrd_id = vrd_id;

	     __atomic_fetch_add(&activeReasmContexts, 1, __ATOMIC_SEQ_CST);
 
            return TRUE;
        }
        else
        {
            ADK_IPV6_DP_LOG_INFO_DEBUG(
                    "Reassembly hash table full !!! \n");
            *excep = ADK_IPV6_REASS_LIST_FULL;
            INC_NON_NULL(ipv6_global_stats, ipv6ReassTableFullFailures, 1);
            return FALSE;
        }
    }

    return FALSE;
}

/* 
 * This function does the basic fragment header validation 
 * & inserts the fragment for re-assembly 
 * 
 * can use the hole algorithm to insert fragments 
 */
static inline adk_dp_status_t
adk_ipv6_dp_insert_frag( adk_ipv6_dp_reassembly_entry_t **reass_entry,
        adk_pktio_task_t *task,
        uint16_t         offset_to_frag,
        uint8_t          *excep)
{
    uint32_t idx, expected_res_len;
    uint16_t len, ofs;
    uint8_t i;
    adk_ipv6_hdr_t *ipv6_hdr;
    adk_metadata_t *adk_md_p;
    adk_ph_dev_dp_metadata_t *phMeta_p;
    adk_ipv6_ph_dev_rx_param_t *ipv6_ph_params;
    adk_ipv6_dp_if_stats_entry_t  *if_stats = (*reass_entry)->if_stats;

    adk_md_p =  ADK_METADATA_PTR(task);

    ADK_PDU_OFFSET_PTR_GET(task, ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);

    adk_ipv6_frag_hdr_t *frag_hdr = (adk_ipv6_frag_hdr_t*) ((uint8_t *) ipv6_hdr + offset_to_frag);

    phMeta_p = (adk_ph_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task,imc_param_start));
    ipv6_ph_params = (adk_ipv6_ph_dev_rx_param_t *)&phMeta_p->imc_param_start;
    ipv6_ph_params->exception_specific_data.overlapFragment.fragmentId = adk_ntohl(frag_hdr->frag_id);

    /* check if reassembly session is not valid & does all the subsequen packets */
    if((*reass_entry)->err_detected)
    {
        ADK_IPV6_DP_LOG_INFO_DEBUG(
                "invalid reassembly session, drop the fragment \n");
        INC_NON_NULL(if_stats, ipv6IfInvalidReasmSessionId, 1);
        return ADK_DP_E_ERROR;
    }

    /* len of the fragment payload */
    len = (((adk_ntohl(ipv6_hdr->len_nh_hlimit))>>16) + sizeof(adk_ipv6_hdr_t)) - 
        (offset_to_frag + sizeof(adk_ipv6_frag_hdr_t));
    ofs = (adk_ntohl(frag_hdr->nh_ofs_m))&0xFFF8;

    /* update the total fragments len */
    (*reass_entry)->frag_size += len;

    /*update the expected total reassembly length */
    expected_res_len = ofs + len;

    if(expected_res_len > UINT16_MAX)
    {
        *excep = ADK_IPV6_REASS_MAX_PDU_SIZE_REACHED;
        ipv6_ph_params->exception_specific_data.paramProbData.paramProbType = ADK_IPV6_REASSEMBLY_MAX_PDU_SIZE_REACHED;
        INC_NON_NULL(if_stats, ipv6IfExpectedReasmLengthExceed, 1);
        return ADK_DP_E_ERROR;
    }

    /* check if first fragment */
    if(!ofs)
    {
        ADK_IPV6_DP_LOG_DEBUG("received first frag \n");
        /* check if the len is multiple of 8 */
        if(len&0x7)
        {
            INC_NON_NULL(if_stats, ipv6IfNotMulOfEightFragLen, 1);
            *excep = ADK_IPV6_REASS_NOT_MULTIPLE_OF8;
            ipv6_ph_params->exception_specific_data.paramProbData.paramProbType = ADK_IPV6_HEADER_PAYLOAD_LENGTH_INCORRECT;
            ADK_IPV6_DP_LOG_DEBUG(
                    "initial fragment length is not multiple of 8, dropping the frag \n");
            return ADK_DP_E_ERROR;
        }

        (*reass_entry)->ext_hdr_len = offset_to_frag - ADK_IPV6_HDR_LEN;
        idx = ((*reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task == NULL)? 
            ADK_IPV6_FIRST_FRAG_IDX : UINT32_MAX;
    }
    /*check if last fragment */
    else if(!(frag_hdr->nh_ofs_m & adk_htonl(1)))
    {
        ADK_IPV6_DP_LOG_DEBUG("received last frag \n");
        idx = ((*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].task == NULL)? 
            ADK_IPV6_LAST_FRAG_IDX:UINT32_MAX;

        (*reass_entry)->total_size = expected_res_len; 
    }
    /* insert intermediate fragments */
    else 
    {
        ADK_IPV6_DP_LOG_DEBUG("received intermediate frag \n");

        /* check if the len is multiple of 8 */
        if(len&0x7)
        {
            INC_NON_NULL(if_stats, ipv6IfNotMulOfEightFragLen, 1);
            *excep = ADK_IPV6_REASS_NOT_MULTIPLE_OF8;
            ipv6_ph_params->exception_specific_data.paramProbData.paramProbType = ADK_IPV6_HEADER_PAYLOAD_LENGTH_INCORRECT;
            ADK_IPV6_DP_LOG_DEBUG(
                    "intermediate fragment length is not multiple of 8, dropping the frag \n");
            return ADK_DP_E_ERROR;
        }

        idx = (*reass_entry)->frag_idx;
        if(idx < (sizeof((*reass_entry)->fg)/sizeof((*reass_entry)->fg[0])))
            (*reass_entry)->frag_idx++;
    }

    /*check if the received fraglen is greater than expected reassembled len */
    if((*reass_entry)->frag_size > (*reass_entry)->total_size)
    {
        INC_NON_NULL(if_stats, ipv6IfExpectedReasmLengthExceed, 1);
        *excep = ADK_IPV6_REASS_OVERLAP_FRAG;
        return ADK_DP_E_ERROR;
    }
    /* check if frags are duplicated or overlapped */
    for(i=ADK_IPV6_FIRST_FRAG_IDX; i<idx; i++)
    {
        if((*reass_entry)->fg[i].task)
        {
            if((*reass_entry)->fg[i].ofs == ofs)
            {
                INC_NON_NULL(if_stats, ipv6IfReasmOverlapFrag, 1);
                *excep = ADK_IPV6_REASS_DUP_FRAG;
                (*reass_entry)->err_detected = TRUE;
                return ADK_DP_E_ERROR;
            }
            else if(((*reass_entry)->fg[i].ofs > ofs) &&
                    (((*reass_entry)->fg[i].ofs + (*reass_entry)->fg[i].len) < ofs))
            {
                INC_NON_NULL(if_stats, ipv6IfReasmOverlapFrag, 1);
                *excep = ADK_IPV6_REASS_OVERLAP_FRAG;
                (*reass_entry)->err_detected = TRUE;
                return ADK_DP_E_ERROR;
            }
        }
    }
    /* check if holes are full */
    if(idx >= (sizeof((*reass_entry)->fg)/sizeof((*reass_entry)->fg[0])))
    {
        INC_NON_NULL(if_stats, ipv6IfTooManyFragsDropCount, 1);
        ADK_IPV6_DP_LOG_WARNING("received more fragments than expected "
                "dropping the frag. subsequent fragments will be droped \n");

        *excep = (idx != UINT32_MAX)? 
            ADK_IPV6_REASS_NO_FREE_HOLES:ADK_IPV6_REASS_OVERLAP_FRAG;

        return ADK_DP_E_ERROR;
    }

    (*reass_entry)->nb_frags++;
    (*reass_entry)->fg[idx].ofs = ofs;
    (*reass_entry)->fg[idx].len = len;
    (*reass_entry)->fg[idx].task = task;

    return ADK_SUCCESS;
}

static inline adk_dp_status_t 
adk_ipv6_frag_hdr_remove(adk_pktio_task_t *task, 
        uint8_t *start)
{
    uint8_t *src_pdu, *dst_pdu, *next_header;
    adk_metadata_t *adk_md_p;
    adk_md_p = ADK_METADATA_PTR(task);
    
    src_pdu = start + adk_md_p->ip_offset;
    next_header = &src_pdu[6];

    src_pdu = src_pdu + ADK_IPV6_HDR_LEN;

    /*find the extension header previous to frag header 
     * TODO - Can be optimised from initial parser */
    while((*next_header) != ADK_IPV6_NEXT_HDR_FRAGMENT)
    {   
        next_header = &src_pdu[0];
        src_pdu += (src_pdu[1]<<2) + 8/* initial 8 octets */;
    }

    /* update the prev extension next header value */
    *next_header = src_pdu[0];

    dst_pdu = src_pdu;
    src_pdu -= 8;
    while(src_pdu >= start)
    {
        /**((uint64_t*)&dst_pdu[0]) = *((uint64_t*)&src_pdu[0]);*/
        rte_memcpy((&dst_pdu[0]) ,&src_pdu[0], 8);
        src_pdu -= 8;
        dst_pdu -= 8;
    }
    *((uint64_t*)&dst_pdu[0]) = *((uint64_t*)&src_pdu[0]);

    /* udpdate the task header start */
    ADK_PDU_HEAD_PULL(task, start, (uint8_t *), sizeof(adk_ipv6_frag_hdr_t));

    return (ADK_SUCCESS);
}


static inline adk_dp_status_t
adk_ipv6_dp_do_reassembly(adk_ipv6_dp_reassembly_entry_t **reass_entry,
        adk_ipv6_dp_reassembly_hash_entry_t __attribute__((unused))**reass_hash_entry,
        adk_pktio_task_t **reass_mbuf)
{
    int i=ADK_IPV6_LAST_FRAG_IDX;
    uint8_t *pdu;
    uint16_t reass_pdu_len, initial_frag_len, tail_ofs, tail_len, l3_offset;
    adk_ipv6_hdr_t *ipv6_hdr = NULL;
    adk_pktio_task_t *tail_task, *prev_task;
    adk_metadata_t *adk_md_p;

    /* check offsets & do the re-assembly 
     * start from last fragment */
    initial_frag_len = (*reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].len; 
    tail_ofs = (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].ofs;
    tail_len = (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].len;
    tail_task = (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].task;
    reass_pdu_len = (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].ofs+
        (*reass_entry)->fg[ADK_IPV6_LAST_FRAG_IDX].len;
    
    while(tail_ofs != initial_frag_len)
    {
        prev_task = tail_task;

        for(i= ((*reass_entry)->frag_idx - 1); 
                i != ADK_IPV6_FIRST_FRAG_IDX && tail_ofs != initial_frag_len;
                i--)
        {
            /*check for the previous frag */
            if(((*reass_entry)->fg[i].ofs + (*reass_entry)->fg[i].len)== tail_ofs)
            {
                /* remove the ipv6 & extension headers if present 
                 * - not necessary to update the fragment adk metadata*/
                l3_offset = ADK_TASK_PDU_SIZE(tail_task)-tail_len;
                ADK_PDU_HEAD_PULL(tail_task, pdu, (uint8_t *), l3_offset);

                /* chain the task */
                ADK_TASK_CHAIN((*reass_entry)->fg[i].task, tail_task);
                tail_task = (*reass_entry)->fg[i].task;
                tail_ofs = (*reass_entry)->fg[i].ofs;
                tail_len += (*reass_entry)->fg[i].len;
            }
        }

        /* check if previous task did not present */
        if(tail_task == prev_task)
        {
            ADK_IPV6_DP_LOG_INFO_DEBUG("Hole identified during reassembly "
                    "either duplicate or overlap packets received \n");

            (*reass_entry)->err_detected = TRUE;

            return ADK_DP_E_ERROR;
        }
    }
    /* chain the initial fragment */
    l3_offset = ADK_TASK_PDU_SIZE(tail_task) - tail_len;
    ADK_PDU_HEAD_PULL(tail_task, pdu, (uint8_t *), l3_offset);
    /*chain the task */
    ADK_TASK_CHAIN((*reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task, tail_task);
    (*reass_mbuf) = (*reass_entry)->fg[ADK_IPV6_FIRST_FRAG_IDX].task;

    /* update the reassembled len */
    adk_md_p = ADK_METADATA_PTR(*reass_mbuf);
    ADK_PDU_OFFSET_PTR_GET((*reass_mbuf), ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);
    ipv6_hdr->len_nh_hlimit &= adk_htonl(0xFFFF);
    ipv6_hdr->len_nh_hlimit |= adk_htonl((reass_pdu_len+(*reass_entry)->ext_hdr_len)<<16);


    /* remove fragment ext header */
    adk_ipv6_frag_hdr_remove((*reass_mbuf), (((uint8_t *)ipv6_hdr)-adk_md_p->ip_offset)); 
   
    (*reass_entry)->is_reassembly_success = TRUE;


    return ADK_SUCCESS;
}



adk_dp_status_t
adk_ipv6_dp_reassembly_proc( uint32_t *src_addr, uint32_t *dst_addr, uint16_t if_id,
        uint8_t  *excep, adk_ipv6_dp_reass_meta_data_t *reass_md_p,
        adk_pktio_task_t **mbuf, uint8_t  coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t key, frag_offset, frag_id = reass_md_p->frag_id;
    uint16_t hash_idx, offset_to_frag = reass_md_p->offset_to_frag, lock_idx;
    bool is_present;
    adk_pktio_task_t *reass_mbuf = NULL;
    adk_ipv6_dp_reassembly_entry_t *reass_entry = NULL;
    adk_ipv6_dp_reassembly_hash_entry_t *reass_hash_entry = NULL;
    adk_ipv6_hdr_t *ipv6_hdr;
    adk_ipv6_frag_hdr_t *frag_hdr;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_if_stats_entry_t  *if_stats = reass_md_p->if_stats;
    adk_ipv6_dp_ip_addr_stats_t   *ipv6_addr_stats = reass_md_p->ipv6_addr_stats; 
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
#ifdef ADK_IPV6_FLOW_CACHE
    uint32_t fc_hash_value = reass_md_p->hash_value;
    uint16_t ing_assoc_rule_mask = reass_md_p->rulemask;
    uint16_t vif_stats_id = reass_md_p->vif_stats_id;
    uint16_t src_port = reass_md_p->src_port, dst_port = reass_md_p->dst_port;
    uint8_t fc_hit = reass_md_p->fc_hit;
    uint8_t vif_flag = reass_md_p->vif_flag;
    uint8_t next_hdr = reass_md_p->next_hdr;
#if ADK_IPV6_ACL_ENABLE
    uint16_t ing_acl_statsid = reass_md_p->in_acl_stats_id;
    uint8_t dscp_id, ipv6_dscp;
#endif
#ifdef ADK_DEVICE_SNR
    adk_dsi_rx_metadata_t *dsi_md_p;
    dsi_md_p = ADK_DSI_RX_METADATA_PTR((*mbuf));
#endif
#endif
#if defined (ADK_DEVICE_SNR) && defined(UDP_CHECKSUM_ENABLED)
    uint16_t initial_frag_len;
#endif
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p;

    /*check for atomic fragments */
    adk_md_p = ADK_METADATA_PTR(*mbuf);
    ADK_PDU_OFFSET_PTR_GET((*mbuf), ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);
    frag_hdr = (adk_ipv6_frag_hdr_t*) ((uint8_t *) ipv6_hdr + offset_to_frag); 
    
    frag_offset = adk_ntohl(frag_hdr->nh_ofs_m)&0xFFF9; 

    if(!frag_offset)
    {
        /* update the reassembled len */
        ipv6_hdr->len_nh_hlimit -= adk_htonl(0x80000);
        /* remove fragment ext header */
        adk_ipv6_frag_hdr_remove((*mbuf), (((uint8_t *)ipv6_hdr)-adk_md_p->ip_offset));

        return ADK_SUCCESS;
    }

    /* non Atomic fragments */
    /* generate reassembly hash key */
    adk_ipv6_dp_reassembly_key_create(src_addr,
            dst_addr, 
            frag_id, 
            &key);

    hash_idx = key&ADK_IPV6_REASSEMBLY_HASH_MASK;
    lock_idx = hash_idx*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;

    /* Acquire Hash id based Lock */
    rte_spinlock_lock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    /* check if Reassembly instance present */
    is_present = adk_ipv6_dp_reassembly_hash_lookup((*mbuf), src_addr, 
                   dst_addr,   frag_id, key,  if_id,
                   excep, &reass_entry, &reass_hash_entry, 
                   0, if_stats, coreIndex, NULL, 0, 0);

    if(!is_present)
    {
        /* Release Hash id based Lock */
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

        ADK_IPV6_DP_LOG_INFO_DEBUG("reassembly hash lookup fails \n");
        return ADK_E_INTERNAL_ERROR;
    }

    reass_entry->if_stats = reass_md_p->if_stats;
    /* insert fragment - hole algorithm */
    ret = adk_ipv6_dp_insert_frag(&reass_entry, *mbuf, offset_to_frag, excep);
    if(ret)
    {
        INC_NON_NULL(ipv6_addr_stats, ipv6InErrors, 1);
        /* Release Hash id based Lock */
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

        ADK_IPV6_DP_LOG_ERROR("Error while inserting fragment \n");
        return ADK_E_INTERNAL_ERROR;
    }

    if(reass_entry->frag_size <= reass_entry->total_size)
    {
        if(reass_entry->frag_size < reass_entry->total_size)
        {
            ADK_MEASURE_CYCLE_DIFF(&(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE]), 1);
            ADK_MEASURE_CYCLE_CHECK_EXIT(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE], coreIndex, 
                    ADK_TERMINATION_STAGE);

            ADK_IPV6_DP_LOG_DEBUG("all fragments are not received. "
                    "wait for other fragments\n");
            /* Release Hash id based Lock */
            rte_spinlock_unlock((rte_spinlock_t *)
                    &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

            return ADK_E_TASK_CONSUMED;
        }
        else
        {
            initial_frag_len = reass_entry->fg[ADK_IPV6_FIRST_FRAG_IDX].len;
            /* check for reassemble */
            ret = adk_ipv6_dp_do_reassembly(&reass_entry,
                    &reass_hash_entry,
                    &reass_mbuf);

            /* reassembly is not succesfull, return task consumed. 
             * because fragment is already inserted - 
             * timeout shall release all the stored fragments */
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("Reassembly error \n");
                ret = ADK_E_TASK_CONSUMED;
            }

            if(likely(ret == ADK_SUCCESS))
            {
                /* return reassembled task */
                *mbuf = reass_mbuf;
                adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) 
                    &ADK_METADATA_PARAM(reass_mbuf, imc_param_start);
                adk_int_md_p->is_reassembled = TRUE;
                adk_int_md_p->initial_frag_len = initial_frag_len;
                /* reset the Reass entry */
                adk_ipv6_dp_reset_reass_entry(&reass_entry);

                /* reset the hash entry */
                adk_ipv6_dp_reset_reass_hash_entry(&reass_hash_entry);

                /* increse the reassembly version */
                reass_entry->reass_ver_id++;

#ifdef ADK_DP_MAINTENANCE
#ifdef ADK_TIMER
                /* Delete the reassembly timer, 
                 * as reassembly is successfull */
                adk_timer_delete(coreIndex, reass_entry->timer_hdl);
                rte_mempool_put(adk_ipv6_dp_tables->ipv6_dp_reassembly_table.tbl,
                        reass_entry);
#endif
#endif
            }

            /* Release Hash id based Lock */
            rte_spinlock_unlock((rte_spinlock_t *)
                    &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

            return ret;
        }
    }
    else
    {
        /* reassembly is not succesfull, return task consumed. 
         * because fragment is already inserted - 
         * timeout shall release all the stored fragments */
        reass_entry->err_detected = TRUE;
        ret = ADK_E_TASK_CONSUMED;
        INC_NON_NULL(ipv6_global_stats, ipv6ReassErrContextFailures, 1);
    }

    /* 
     * if the reassembly is not successfull, ADK is not clearing 
     * the re-assembly session, becuase it has to drop the  
     * subsequent fragments received untill re-assembly session timeout.
     */

    /* Release Hash id based Lock */
    rte_spinlock_unlock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    return ret;
}


#ifdef ADK_TIMER
/*!
 * =============================================================
 * adk_ipv6_dp_reassembly_timeout_proc()
 *
 * structure of imc task params :
 *
 *
 * brief:
 * This is an entry fcuntion for IPv6 reassembly timer expiry function.
 * does cleaup the reassembly session databases & 
 * raise an reassembly timeout exception to application.
 *
 * =============================================================
 */

adk_status_t 
adk_ipv6_dp_reassembly_timeout_proc(void  **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex)
{
    uint8_t reass_ver_id, collision_id;
    uint16_t hash_id;
    adk_metadata_t *timer_stage_md_p;
    adk_ipv6_dp_reass_stage_meta_data_t *reass_stage_imc_md;
    adk_ipv6_dp_if_stats_entry_t        *if_stats = NULL;
    ADK_IPV6_DP_LOG_INFO_DEBUG("enter timeout \n");

    /* extract the hash/reassembly index */
    timer_stage_md_p = ADK_METADATA_PTR(task_p[0]);
    reass_stage_imc_md = (adk_ipv6_dp_reass_stage_meta_data_t *) 
        &timer_stage_md_p->imc_param_start;

    reass_ver_id = reass_stage_imc_md->reass_ver_id;
    if_stats = reass_stage_imc_md->if_stats;
    hash_id = reass_stage_imc_md->hash_id; 
    collision_id = reass_stage_imc_md->collision_id;

    /* release the timer task */
    rte_pktmbuf_free((struct rte_mbuf*) task_p[0]);

    /* Reassembly Cleanup */
    adk_ipv6_dp_reassembly_cleanup(hash_id, collision_id, reass_ver_id, if_stats);
       
    return ADK_SUCCESS;
}
#endif

static adk_dp_status_t
adk_ipv6_reassembly_excep(uint8_t excep, adk_pktio_task_t *task, uint8_t vrd_id,
        uint8_t port, uint16_t if_id, uint16_t ipv6_offset, int coreIndex)
{

    adk_dp_status_t ret;
    switch(excep)
    {
        case ADK_IPV6_REASS_LIST_FULL:
            ADK_IPV6_DP_LOG_INFO_DEBUG("List is full, dropping the packet. \n");
            return ADK_E_INTERNAL_ERROR; 

        case ADK_IPV6_REASS_NOT_MULTIPLE_OF8:
            ADK_IPV6_DP_LOG_INFO_DEBUG(
                    "Received fragment size is not multiple of 8, sending exception. \n");

            ret = adk_dp_send_ipv6_exception(task, vrd_id, port, if_id, ipv6_offset, 0, 0,
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_PARAM_PROBLEM, coreIndex, 0);
            return ret;

        case ADK_IPV6_REASS_DUP_FRAG:
        case ADK_IPV6_REASS_OVERLAP_FRAG:
            ADK_IPV6_DP_LOG_INFO_DEBUG(
                    "Duplicate/overlap fragment received, Sending exception. \n");

            ret = adk_dp_send_ipv6_exception(task, vrd_id, port, if_id, ipv6_offset, 0, 0, 
                    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_FRAG_OVERLAP, coreIndex, 0);
            return ret;

       case ADK_IPV6_REASS_NO_FREE_HOLES:
            ADK_IPV6_DP_LOG_INFO_DEBUG(
                    "no free holes, dropping the packet. \n");
            return ADK_E_INTERNAL_ERROR;

       case ADK_IPV6_REASS_MAX_PDU_SIZE_REACHED:
            ADK_IPV6_DP_LOG_INFO_DEBUG(
                    "MAX PDU size reached, Sending exception to host.. \n");

            ret = adk_dp_send_ipv6_exception(task, vrd_id, port, if_id, ipv6_offset, 0, 0,
                   ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, ADK_IPV6_EXCEPTION_PARAM_PROBLEM,
                    coreIndex, 0);

            return ret;


       default:
            return ADK_E_INTERNAL_ERROR;
    }

    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_reassembly()
 *
 * structure of imc task params :
 *
 *
 * brief:
 * This is an entry fcuntion for IPv6 reassembly function.
 * does following:
 *
 * a. reassembly of fragmented packets.
 * b. ingress association lookup.
 * c. This function always return "task Consumed"
 *      in case of error packets are dropped/exceptions are raised.
 *      in case of bind, packets are sent to destination module.
 * =============================================================
 */
adk_status_t 
adk_ipv6_dp_reassembly(void  **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t *task = (adk_pktio_task_t*) *task_p;
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry = NULL;
    adk_ipv6_dp_if_stats_entry_t *if_stats;
    adk_ipv6_hdr_t  *ipv6_hdr;
    adk_ipv6_dp_ip_addr_stats_t          *ipv6_addr_stats = NULL;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    uint32_t len_nh_hlimit, frag_id = 0, hash_value;
    uint16_t if_id = 0, src_port=0, dst_port =0, offset_to_frag = 0, ipv6_len, l4_offset;
    uint8_t excep, next_hdr=0, fc_hit;
    bool  l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_metadata_t *adk_md_p=ADK_METADATA_PTR(task);
    adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
    (adk_ipv6_dp_reass_meta_data_t *) &adk_md_p->imc_param_start;
    uint8_t ipv6_addr_stats_id = 0;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p = NULL;
    adk_dsi_rx_metadata_t *dsi_md_p = NULL;

#ifdef ADK_IPV6_FLOW_CACHE
    adk_status_t hash_ret;
    void *ringHdl;
    uint8_t dst_mod_id;
#ifdef ADK_DEVICE_SNR
    adk_dsi_rx_metadata_t *dsi_md_p;
    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
#endif
#endif

    /*Copying the address stats*/
    ipv6_addr_stats_id = adk_hos_get_ip_stats_id(ADK_DSI_RX_METADATA_PTR(task));
    ipv6_addr_stats = reass_md_p->ipv6_addr_stats;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    if (reass_md_p->is_outer_ip)
    {
        if_id = adk_md_p->input_intf_id;
        ret = adk_ipv6_dp_interface_entry_lookup(if_id, 0, &ipv6_if_entry);
        if(ret != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_INFO("error while retriving interface attributes \n");
            INC_NON_NULL(ipv6_global_stats, InvalidIfFail, 1);
            goto adk_ipv6_dp_reassembly_return;
        }
    }
    if_stats = reass_md_p->if_stats;
    hash_value = reass_md_p->hash_value;
    fc_hit = reass_md_p->fc_hit;

    ADK_PDU_OFFSET_PTR_GET(task, ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);
    excep = ADK_IPV6_REASS_INVALID_EXCEP;

    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (len_nh_hlimit >> 16);

    if (reass_md_p->is_outer_ip)
    {
        INC_NON_NULL(if_stats, ipv6IfStatsInReceives, 1);
        INC_NON_NULL(if_stats, ipv6IfStatsHCInOctets, ipv6_len);
    }

    /* Incrementing address stats */
    if (((adk_md_p->outerIPVersion == ADK_IPV6_VER) &&
            (ipv6_addr_stats_id != adk_md_p->outerIpStatsId)) ||
            (adk_md_p->outerIPVersion == ADK_IPV4_VER))
    {
        INC_NON_NULL(ipv6_addr_stats, ipv6InPackets, 1);
        INC_NON_NULL(ipv6_addr_stats, ipv6InOctets, ipv6_len);
    }
    ret = adk_ipv6_dp_reassembly_proc(ipv6_hdr->src_addr, ipv6_hdr->dst_addr,
            adk_md_p->input_intf_id, &excep, reass_md_p, &task, coreIndex);
    if(likely(ret == ADK_E_TASK_CONSUMED))
    {
        adk_pal_dp_release_send(task, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
        ADK_METADATA_PARAM(task, task_type) = ADK_TASK_TYPE_NEW;
        goto adk_ipv6_dp_reassembly_return;
    }

    /*The task below is not the last frag recevied. Instead a new reassembled mbuf 
     * is created. Hence setting task_type as FWD*/
    ADK_METADATA_PARAM(task, task_type) = ADK_TASK_TYPE_FWD;

    /*get the reassembed metadata ptr */
    adk_md_p = ADK_METADATA_PTR(task);

    if(unlikely(ret != ADK_SUCCESS))
    {
        ADK_IPV6_DP_LOG_ERROR("Error while reassembling \n");

        /* increment packet fail counter */
        if (reass_md_p->is_outer_ip)
            INC_NON_NULL(if_stats, ipv6IfStatsReasmFails, 1);

        /* send exception to host */
        ret = adk_ipv6_reassembly_excep(excep, task, 0, task->port,
                if_id, adk_md_p->ip_offset, coreIndex);
        if(unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error while sending reassembly exception \n");
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_dp_reassembly_return;
        }
        ret = ADK_E_TASK_CONSUMED;
        goto adk_ipv6_dp_reassembly_return;
    }

    /* Reassembly is successfull extract L4 ports*/
    ADK_PDU_OFFSET_PTR_GET(task, ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);
    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);

    excep = 0;
    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8) & 0xFF), (uint8_t *)ipv6_hdr, 
            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);
    /* update the adk metadata l4 offset, relative to mbuf data offset */
    adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;

    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *) &adk_md_p->imc_param_start;
    adk_int_md_p->is_outer_ip = reass_md_p->is_outer_ip;
    adk_int_md_p->ipv6_addr_stats = reass_md_p->ipv6_addr_stats;
    /* After Reassembly successful we need to generate checksum again and validate
     * So explicitly updating HOS l4-chksum-flag to HOS_FLAG_L4_CHKSUM_ERR
     * */
    dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
    dsi_md_p->flags |= HOS_FLAG_L4_CHKSUM_ERR;

    ret = adk_ipv6_dp_chksum_validate(task, frag_flag, next_hdr, l4_offset, dst_port,
            ipv6_hdr, if_stats, ipv6_if_entry, 1);
    if (unlikely((ret == ADK_DP_E_ERROR) || (ret == ADK_E_TASK_CONSUMED) ))
    {
        goto adk_ipv6_dp_reassembly_return;
    }

    ((adk_ipv6_dp_lli_imc_data_t *) &(adk_md_p->imc_param_start))->ipsec_data = reass_md_p->ipsec_data;

    /* increment packet req for reassembly */
    if (reass_md_p->is_outer_ip)
        INC_NON_NULL(if_stats, ipv6IfStatsReasmOKs, 1);

    INC_NON_NULL(ipv6_addr_stats, ipv6InReassembledPackets, 1);

    /*Once reassembly done, check if ipsec bypass is enabled/disabled and act accordingly*/
    if(!ADK_SYS_IPSEC_BYPASS_GET && (reass_md_p->ipsec_data == IPSEC_NOT_VISITED))
    {
       /* Received from ETH LLI and ipsec bypass is FALSE, Hence sending to IPSec LLI */;
       ADK_IPV6_DP_LOG_DEBUG("IP LLI: bypass=0, ipsec not visited. sending-to IPSec LLI\n");
       adk_ipsec_dp_lli_imc_param_t * ipseclliImc = (adk_ipsec_dp_lli_imc_param_t *)(&(adk_md_p->imc_param_start));
       ipseclliImc->isLocal = true;
       ipseclliImc->pktOrigin = IP6_ULI_SENDER_IP6LLI;
       ipseclliImc->ipv6_nextHdr = next_hdr;
       ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS1_TS();

       if (adkDestEntryFunction_g[ADK_IPSEC_LLI_FN_ID])
       {
           ret = adk_pal_dp_stage_send((void **)&task, 1, 0x1, ADK_IPSEC_LLI_FN_ID, 0x00,
                   0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT, NULL);
           if(ret == ADK_E_INTERNAL_ERROR)
           {
               ret = ADK_DP_E_ERROR;
               goto adk_ipv6_dp_reassembly_return;
           }
       }
       ADK_IPV6_DP_LOG_DEBUG("imcForIpsecLLI: isLocal=%d, l4offset: %u\n", ipseclliImc->isLocal, adk_md_p->l4_offset);
       ret = ADK_E_TASK_CONSUMED;
       goto adk_ipv6_dp_reassembly_return;
    }

    /* check for extension header parsing errors */
    if(unlikely(excep))
    {
        /* raise an exception  */
        ret = adk_dp_send_ipv6_exception(task, 0, task->port, if_id,
                adk_md_p->ip_offset, 0, l4_offset, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                excep, coreIndex, 0);
        if(ret != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception" 
                    "%d \n", excep);
            ret = ADK_DP_E_ERROR;
            goto adk_ipv6_dp_reassembly_return;
        }
        ret = ADK_E_TASK_CONSUMED;
        goto adk_ipv6_dp_reassembly_return;
    }
    /* ingress_classifier lookup post reassembly*/
    ret = adk_ipv6_dp_terminate(task, reass_md_p->rulemask, ipv6_if_entry, if_stats, ipv6_hdr,
            frag_flag, next_hdr, l4_offset, src_port, dst_port);
    if(likely(ret == ADK_E_TASK_CONSUMED))
    {
        goto adk_ipv6_dp_reassembly_return;
    }
    else
    {
        ret = ADK_DP_E_ERROR;
        goto adk_ipv6_dp_reassembly_return;
    }
adk_ipv6_dp_reassembly_return:
    if ((ret != ADK_E_TASK_CONSUMED ) && (ret != ADK_SUCCESS))
    {
        IPV6_DP_DROP_TASK(task);
    }

    return ret;
}

inline adk_status_t adk_ipv6_dp_ipsec_start_reassembly(uint16_t coreIndex,
        adk_pktio_task_t **ph_task_p,
        uint16_t *res_index,
        uint16_t ipv6_offset/*Offset to inner IP*/,
        clear_ipsec_ctx_for_hash ipsecCb,
        uint8_t *bucket_index,
        uint32_t preFragBufferTimeOutVal 
    )
{
    adk_status_t ret = ADK_SUCCESS;
    //struct ipv6_hdr *ipv6hdr;
    adk_ipv6_hdr_t *ipv6hdr;
    adk_pktio_task_t *ph_task = *ph_task_p;
    adk_ipv6_dp_reassembly_entry_t *reass_entry = NULL;
    adk_ipv6_dp_reassembly_hash_entry_t *reass_hash_entry = NULL;
    adk_ipv6_dp_if_stats_entry_t         *if_stats = NULL;
    uint16_t if_id=0;
    uint32_t key = 0;
    uint16_t hash_idx=0, lock_idx=0;
    uint32_t frag_id=0;
    uint8_t excep=0;
    uint32_t srcaddr[4], destaddr[4];
    bool is_present=0, l4_port_present=0;
    adk_ipv6_dp_route_entry_t *route_entry = NULL;
    uint32_t ver_tc_flabel, len_nh_hlimit;
    uint16_t ipv6_len, pdu_length, l4_offset = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0;
    uint8_t next_hdr=0, action =0;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ipv6_dp_pbr_action_entry_t       *pbr_action = NULL;
    adk_ipv6_pbr_val_t pbr_val;
    bool defTblLookup = TRUE;
    uint8_t route_table;

    adk_metadata_t *adk_md_p=ADK_METADATA_PTR(ph_task);
    ADK_PDU_OFFSET_PTR_GET(ph_task, ipv6hdr, struct adk_ipv6_hdr_s *, ipv6_offset);

    srcaddr[0]=adk_ntohl(ipv6hdr->src_addr[0]);
    srcaddr[1]=adk_ntohl(ipv6hdr->src_addr[1]);
    srcaddr[2]=adk_ntohl(ipv6hdr->src_addr[2]);
    srcaddr[3]=adk_ntohl(ipv6hdr->src_addr[3]);

    destaddr[0]=adk_ntohl(ipv6hdr->dst_addr[0]);
    destaddr[1]=adk_ntohl(ipv6hdr->dst_addr[1]);
    destaddr[2]=adk_ntohl(ipv6hdr->dst_addr[2]);
    destaddr[3]=adk_ntohl(ipv6hdr->dst_addr[3]);

    ret = adk_ipv6_dp_egress_pbr(pbr_policy_set_id, 0, next_hdr,
            ipv6hdr, src_port, dst_port, frag_flag, &pbr_val);
    if(likely(ret == ADK_SUCCESS)) /* PBR lookup successful, get the route table ID */
    {
        pbr_action = (adk_ipv6_dp_pbr_action_entry_t *)
                     &adk_ipv6_dp_tables->ipv6_dp_pbr_table.tbl[(uintptr_t)pbr_val];
        (adk_ipv6_dp_tables->ipv6_dp_pbr_table.pbrRuleStats[coreIndex*IPV6_PBR_MAX_NUM_RULES +
               pbr_action->stats_id - 1])++;
        /*PBR lookup success, do DIP lookup in specific table */
        route_table = pbr_action->routeTableId;
        ADK_IPV6_DP_LOG_DEBUG("### PBR lookup success. routeTableId : %u\n", route_table);

        ret = adk_ipv6_dp_route_entry_lookup(route_table, (uint8_t *) &ipv6hdr->dst_addr[0],
                                     &route_entry);
        if(ret == ADK_SUCCESS) /* DIP based lookup successful */
        {
            defTblLookup = FALSE;
            action = route_entry->action;
            ADK_IPV6_DP_LOG_DEBUG("IPv6 route action %d \n",route_entry->action);
        }
    }
    if (defTblLookup == TRUE)
    {
        /* Either PBR lookup has failed or route lookup in PBR yielded route table has failed
         * In both cases, do DIP lookup in Default routing table 
         */
        route_table = adk_ipv6_dp_tables->defaultRouteTable;
        ret = adk_ipv6_dp_route_entry_lookup(route_table, (uint8_t *) &ipv6hdr->dst_addr[0],
                                     &route_entry);

        if(ret == ADK_SUCCESS) /* DIP based lookup successful */
        {
            action = route_entry->action;
            ADK_IPV6_DP_LOG_DEBUG("IPv6 route action %d \n",route_entry->action);
        }
        else /* DIP based lookup fails*/
        {
             ADK_IPV6_DP_LOG_WARNING("DIP lookup failed for Inner IPv6 address %x %x %x %x" 
                         "Must be a forwarding traffic\n", destaddr[0], destaddr[1], destaddr[2], destaddr[3]);
             action = ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
        }
    }

    //to fetch the frag_id from task
    ver_tc_flabel = adk_ntohl(ipv6hdr->ver_tc_flabel);
    len_nh_hlimit = adk_ntohl(ipv6hdr->len_nh_hlimit);
    ipv6_len = ADK_IPV6_PDU_HEADER_LENGTH_BYTES + (len_nh_hlimit >> 16);
    pdu_length = (ph_task->pkt_len - adk_md_p->ip_offset);

    l4_offset = adk_ipv6_parse_l4_ports(pdu_length, ((len_nh_hlimit>>8)&0xFF), (uint8_t *)ipv6hdr,
                &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present, &offset_to_frag,
                &frag_id, &excep);
    
    adk_ipv6_dp_reassembly_key_create(srcaddr,
                                      destaddr,
                                      frag_id,
                                      &key);
    hash_idx = key&ADK_IPV6_REASSEMBLY_HASH_MASK;
    lock_idx = hash_idx*ADK_IPV6_REASSEMBLY_HASH_COL_ENTRIES;
    ADK_IPV6_DP_LOG_DEBUG("### hashidx: 0x%x lockidx: 0x%x Frag ID: 0x%x srcIP:%x %x %x %x dstIP: %x %x %x %x\n",
		    hash_idx, lock_idx, key, srcaddr[0], srcaddr[1], srcaddr[2], srcaddr[3], destaddr[0], destaddr[1], destaddr[2], destaddr[3]);

    /* Acquire Hash id based Lock */
    rte_spinlock_lock((rte_spinlock_t *)
            &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    /* check if Reassembly instance present */
    is_present = adk_ipv6_dp_reassembly_hash_lookup(ph_task, srcaddr,
                   destaddr, frag_id, key, if_id,
                   &excep, &reass_entry, &reass_hash_entry,
                   0/*vrd_id*/, if_stats, coreIndex, ipsecCb, action, preFragBufferTimeOutVal);
    if(!is_present)
    {
        /* Release Hash id based Lock */
        rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

        ADK_IPV6_DP_LOG_INFO_DEBUG("reassembly hash lookup fails \n");
        return ADK_E_INTERNAL_ERROR;
    }
    *res_index = lock_idx ; //+ reass_entry->collosion_idx;
    *bucket_index = reass_entry->collosion_idx; //8; //lock_idx + 1;
    ADK_IPV6_DP_LOG_DEBUG("### res_index: 0x%x bucket_index: 0x%x\n", *res_index, *bucket_index);
    rte_spinlock_unlock((rte_spinlock_t *)
                &adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

    return ADK_SUCCESS;
}

/*The below acquire and release lock wrappers are introduced 
 * so that IPSEC can use them*/
inline void adk_ipv6_dp_acquire_reassembly_lock(
		        uint16_t lock_idx)
{
    rte_spinlock_lock((rte_spinlock_t *)
			&adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);

}

inline void adk_ipv6_dp_release_reassembly_lock(
		        uint16_t lock_idx)
{
    rte_spinlock_unlock((rte_spinlock_t *)
			&adk_ipv6_dp_tables->ipv6_dp_reassembly_table.hash_tbl[lock_idx]);
}
