/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_dp_tables.h"
#include "adk_ipv6_dp_common.h"
#include "adk_ipv6_dp_exception.h"
#include "adk_ipv6_dp_frag.h"
#include "adk_ipv6_dp_flow_cache_uli.h"
#include "adk_ipv6_dp_acl.h"
#include "adk_ipv6_utility_counters.h"
#include "adk_nd_common.h"
#include "adk_ipv6_dp_pbr.h"
#include "adk_ipv6_dp_export.h"
#include <adk_ethernet_export.h>
#include "adk_sif_dp.h"
#include "adk_sys.h"
#include "adk_sys_dp_flow_cache_uli_func.h"

#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#ifdef ADK_DEVICE_SNR
#include <adk_hos_dp.h>
#endif

#define ADK_IPV6_INVALID_NH_INTF_ID 0xFFFF
#define ADK_IPV6_INVALID_ROUTE_TABLE_ID 0xFFFF

#define UPDATE_FC(p, m) do{\
        if((p == IP6_ULI_SENDER_EO) && \
                scratch_pad_ipv6->flow_cache_entry)\
        {\
          m;\
        }\
}while(0);

typedef struct {
    uint32_t frag_id;
    uint32_t l4_offset;
    uint16_t src_port;
    uint16_t dst_port;
    uint16_t offset_to_frag;
    uint8_t next_hdr; 
    uint8_t excep;
    adk_ipv6_frag_t frag_flag;
    bool l4_port_present;
} ipv6_l4_info;

/* global fast-path database pointer */
extern adk_ipv6_dp_tables_t    *adk_ipv6_dp_tables;
extern uint8_t adk_ipv6_dscp_map[IPV6_MAX_DSCP_VALUES];
extern struct rte_mempool *adkPktmbufPool_g;
extern struct rte_mempool *pktmbuf_pool_indirect_g;
extern uint32_t pbr_policy_set_id;
 __thread adk_sif_dp_scratchpad_space_t *scratch_pad_ipv6 = NULL;
static adk_status_t
adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass1(struct rte_mbuf *task, ipv6_l4_info *l4_info,
                                                          uint16_t *destFunc);
static adk_status_t
adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass2(struct rte_mbuf *task, int coreIndex,
                                                          uint16_t *destFunc);

static uint8_t  adk_ipv6_inner_hdr_processing(struct rte_mbuf *task, int coreIndex, 
                                               uint16_t *destFunc);

static uint8_t  adk_ipv6_outer_hdr_esp_processing(struct rte_mbuf *task, int coreIndex,
                                               uint16_t *destFunc);

static uint8_t adk_ipv6_dp_pkt_proc_uli_slow_path(struct rte_mbuf *task, int coreIndex, 
                                               uint16_t *destFunc);

static int is_global_unicast(adk_ipv6_hdr_t *ipv6_hdr);

void fetch_rx_cycle_count(uint8_t path);
void fetch_tx_cycle_count(uint8_t path);
void adk_ipv6_increment_tx_l4_stats(uint8_t next_hdr, uint8_t path, uint8_t pktOrigin,
                                 uint16_t ipv6_len, int coreIndex);

/* Function process single packet & return status.
 * Packet is not dropped inside this function */
static inline adk_dp_status_t 
adk_ipv6_dp_proc_cpu_path(void **task_p, uint16_t if_id, int coreIndex, 
        adk_pal_dp_stage_info_t *stageInfo)
{
    adk_pktio_task_t *task_out =  *(adk_pktio_task_t**) task_p;
    adk_status_t ret = ADK_SUCCESS;
    uint32_t len_nh_hlimit;
    uint16_t  ipv6_len = 0;
    uint8_t  ipv6_dscp = 0, ipv6_pcp = 0, *pdu = NULL;
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry = NULL;
    adk_ipv6_hdr_t  *ipv6_hdr;
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats = NULL;
    adk_metadata_t *adk_md_p;
    uint8_t next_hdr = 0,  excep;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag = 0, l4_offset;
    uint32_t frag_id;
    bool  l4_port_present = FALSE; 
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ethernet_dp_uli_imc_param_t *eth_uli_md_p;
#if ADK_IPV6_ACL_ENABLE
    uint16_t eg_acl_stats_id = 0;
#endif
#ifdef ADK_MODULE_IPSEC_ENABLED
    adk_ipv6_dp_nexthop_entry_t *nexthop_entry = 0;
    adk_ipv6_dest_flow_info_t flow_info={0,};
    adk_ipsec_dp_uli_imc_param_t  *ipsec_md_p = NULL; 
#endif
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];

    adk_md_p =  ADK_METADATA_PTR(task_out);
    ADK_PDU_PTR_GET(task_out, pdu, uint8_t *);

    /* Egress interface lookup */
    ret = adk_ipv6_dp_interface_entry_lookup(if_id, 
            0, 
            &ipv6_if_entry);
    if(unlikely(ret != ADK_SUCCESS))
    {
        ADK_IPV6_DP_LOG_DEBUG(
                "error while retriving interface attributes,"
                " ifid %d \n", if_id);
        /* incr the gbl count */
        ipv6_global_stats->InvalidIfFail++;
        return ret;
    }
    if((ipv6_if_entry->if_type != IPV6_INTERFACE_TYPE_ETH) &&
       (ipv6_if_entry->if_type != IPV6_INTERFACE_TYPE_TUNNEL))
    {
        ADK_IPV6_DP_LOG_DEBUG("Invalid IPv6 interface is specified \n");
        /* incr the gbl count */
        ipv6_global_stats->InvalidIfFail++;
        return ADK_DP_E_ERROR;
    }
    if(ipv6_if_entry->is_disabled == TRUE)
    {
        ADK_IPV6_DP_LOG_DEBUG("IPv6 interface is disabled \n");
    
        /* incr the gbl count */
        ipv6_global_stats->InvalidIfFail++;
        return ADK_DP_E_ERROR;
    }
    
    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->egress_stats_id];
    if_stats->ipv6IfStatsOutRequests++;
    

    /*Get the destAddr*/
    ipv6_hdr = (adk_ipv6_hdr_t *)(pdu + adk_md_p->ip_offset);
    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = (len_nh_hlimit >>16) + ADK_IPV6_HDR_LEN;
    
    /* Egress ACL & QoS*/
    /* extract the L4 port */
    l4_offset = adk_ipv6_parse_l4_ports((task_out->pkt_len - adk_md_p->ip_offset), 
            ((len_nh_hlimit>>8) & 0xFF), (uint8_t *)ipv6_hdr, 
            &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);


    /* Egress ACL lookup */
    if(ipv6_if_entry->enable_egress_acl)
    {
#if ADK_IPV6_ACL_ENABLE
        ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
        ret = adk_ipv6_acl_lookup_func( task_out, ipv6_if_entry, next_hdr,
                                      ipv6_dscp, frag_flag, ipv6_hdr,src_port, dst_port, 
                                      ipv6_if_entry->vrd_id, &eg_acl_stats_id, FALSE,
                                      ADK_OS_ORIGIN_DOMAIN, coreIndex);
        if ( ret != ADK_SUCCESS)
        {
            if ( ret == ADK_E_INTERNAL_ERROR)
            {
                ipv6_global_stats->ipv6AclFail++;
            } 
            return (ret); 
        }
#endif
    }

    /* increment the statistics */
    if((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xFF)
    {
        if_stats->ipv6IfStatsOutMcastPkts++;
        if_stats->ipv6IfStatsOutMcastOctets += ipv6_len;
    }
        
    if ( ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_ETH)
    {
        ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
        ipv6_pcp = ipv6_if_entry->dscp_pcp[ipv6_dscp] & 0x0F;
        eth_uli_md_p = (adk_ethernet_dp_uli_imc_param_t *)&adk_md_p->imc_param_start;
        eth_uli_md_p->dscp = ipv6_dscp;
        eth_uli_md_p->pcp  = ipv6_pcp;
        eth_uli_md_p->is_unfrag_first_frag =((frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)||
                (frag_flag==ADK_IPV6_FIRST_FRAGMENT))? TRUE: FALSE;
        eth_uli_md_p->l4_proto = next_hdr;

        eth_uli_md_p->next_hop.is_ipv6 = true;

        eth_uli_md_p->next_hop.u.ipv6_addr.addr[0] = ipv6_hdr->dst_addr[0];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[1] = ipv6_hdr->dst_addr[1];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[2] = ipv6_hdr->dst_addr[2];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[3] = ipv6_hdr->dst_addr[3];

        eth_uli_md_p->output_intf_id = if_id;

        ADK_IPV6_DP_LOG_DEBUG("IMC info to Ethernet \n");
        ADK_IPV6_DP_LOG_DEBUG("dscp %d \n", eth_uli_md_p->dscp);
        ADK_IPV6_DP_LOG_DEBUG("pcp %d \n", eth_uli_md_p->pcp);
        ADK_IPV6_DP_LOG_DEBUG("is_unfrag_first_frag %d \n", eth_uli_md_p->is_unfrag_first_frag);
        ADK_IPV6_DP_LOG_DEBUG("l4_proto %d \n", eth_uli_md_p->l4_proto);
    }
#ifdef ADK_MODULE_IPSEC_ENABLED
    /* IPSEC */
    else if(ipv6_if_entry->if_type == IPV6_INTERFACE_TYPE_TUNNEL)
    {
        /* Nexthop lookup */
        ret = adk_ipv6_dp_nexthop_entry_lookup(ipv6_if_entry->nexthop_id,
                &nexthop_entry);
        if(ret != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_DEBUG("error: nexthop lookup failed \n");
            return ADK_DP_E_ERROR;
        }

        /* update the adk metadata l4 offset, relative to mbuf data offset */
        adk_md_p->l4_offset = l4_offset+adk_md_p->ip_offset;

        /* read dest info atmoically */
        adk_ipv6_dp_read_flow_info(nexthop_entry->flow_info,
                &flow_info);
        
        adk_md_p->pdu_flow_id = flow_info.destFlow;   

        /* Get the pipeline info */
        ADK_MEMCPY(stageInfo, &flow_info.stageInfo, sizeof(adk_pal_dp_stage_info_t));

        /* IPsec IMC fields */
        ipsec_md_p = (adk_ipsec_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
        ipsec_md_p->protocol = next_hdr;
        ipsec_md_p->ipv6FragFlag = (frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)? FALSE: TRUE;
        ipsec_md_p->srcPort = src_port;
        ipsec_md_p->dstPort = dst_port;
        ADK_IPV6_DP_LOG_DEBUG("IMC info to IPsec \n");
        ADK_IPV6_DP_LOG_DEBUG("protocol %d \n", ipsec_md_p->protocol);
        ADK_IPV6_DP_LOG_DEBUG("ipv6FragFlag %d \n", ipsec_md_p->ipv6FragFlag);
        ADK_IPV6_DP_LOG_DEBUG("srcPort %d \n", ipsec_md_p->srcPort);
        ADK_IPV6_DP_LOG_DEBUG("dstPort %d \n", ipsec_md_p->dstPort);
    }
#endif
    
    return ADK_SUCCESS;
}


static inline adk_dp_status_t 
adk_dp_send_ipv6_uli_exception(adk_pktio_task_t *task, uint8_t vrd_id, uint8_t port, uint16_t if_id,
                    uint16_t ipv6_offset, uint16_t  flow_id, uint16_t parse_err_offset,
                    uint8_t domain, uint8_t exception_type, int coreIndex, uint16_t mtu, 
                    adk_ipv6_hdr_t *ipv6_hdr, bool l4_cksum_present)
{
  RTE_SET_USED(ipv6_hdr);
  RTE_SET_USED(l4_cksum_present);
  
#ifdef UDP_CHECKSUM_ENABLED
#ifdef ADK_DEVICE_SNR
    uint16_t *tmp_16_p;

    if(!l4_cksum_present)
    {
        /*L4 checksum was offloaded to h/w.
         * While raising exception calculate UDP checksum & send it.
         */
        tmp_16_p = (uint16_t *)(void *)(((uint8_t *)ipv6_hdr) + ADK_IPV6_HDR_LEN + 6);
        *tmp_16_p = 0;
        /* UDP checksum update */
        *tmp_16_p = (adk_ipv6_udp_checksum_generation(task,
                    ADK_IPV6_HDR_LEN,
                    ipv6_hdr));
        if(!(*tmp_16_p))
        {
            *tmp_16_p = 0xFFFF;
        }
    }
#endif
#endif
    return(adk_dp_send_ipv6_exception(task,vrd_id, port, if_id, ipv6_offset, flow_id, parse_err_offset,
                domain, exception_type, coreIndex, mtu));
}

adk_dp_status_t adk_ipv6_dp_egress_qos_classifier_lookup(adk_ipv6_egress_qos_classifier_ctx_t *egressQoSClassifierCtx_p,
                adk_ipv6_egress_qos_classifier_ptn_t *egressQoSClassifierPtn,
                uint32_t  *result_p)
{
        int ret;
        const uint8_t * data[] = {(uint8_t *)egressQoSClassifierPtn};

        ret = rte_acl_classify_alg((struct rte_acl_ctx *)egressQoSClassifierCtx_p, data,
                        result_p, 1, 1,
                        RTE_ACL_CLASSIFY_SSE);
        if (ret != 0)    {
                ADK_IPV6_DP_LOG_INFO_1D("Error in rte_acl_classify_alg for IPv6 egress QoS Classifier: %d", ret);
                return ADK_E_INTERNAL_ERROR;
        }
        return ADK_SUCCESS;
}

static inline adk_dp_status_t
adk_ipv6_dp_qos_classification(struct rte_mbuf *task,
                               adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
                               adk_ipv6_hdr_t  *ipv6_hdr,
                               uint8_t next_hdr,
                               uint32_t interface_id)
{
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_egress_qos_classifier_ptn_t qosClassifierPtn; 
    uint32_t qosClassifierFlowId = 0;
    uint8_t aclTreeIndex = 0;
    struct ipv4_hdr *innerIPv4Hdr = NULL;
    adk_ipv6_hdr_t  *innerIPv6Hdr = NULL;
    adk_ipv6_dp_egress_qos_classifier_action_info_t *qosClassifierAction_p = NULL;
    uint8_t *pdu = NULL; 
    uint8_t ipv6_dscp = 0;
    uint8_t inner_l4_proto = 0;

    ADK_PDU_PTR_GET(task, pdu, uint8_t *);
    /*
     * IP Qos Lookup for link scope packets
     */
    if(likely(ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_QOS_ENABLE))
    {
        /*Check metadata parameters received from IPsec
          If innerIPVersion is ZERO then it is a plain text packet
          in presence of non-ZERO value it could be either v4/v6
          process accordingly*/
        if (adk_md_p->innerIPVersion && adk_md_p->innerIPHdrOffset) {
            inner_l4_proto = (((adk_ipv6_dp_uli_imc_data_t *)&adk_md_p->imc_param_start)->inner_l4_protocol);
            if (ADK_IPV4_VER == adk_md_p->innerIPVersion) {
                innerIPv4Hdr = (struct ipv4_hdr *)(pdu + ADK_MD_GET_INNER_IP_OFF(task));
                qosClassifierPtn.proto = inner_l4_proto;
                qosClassifierPtn.dscp = (innerIPv4Hdr->type_of_service>>2);
                qosClassifierPtn.sip0 = innerIPv4Hdr->src_addr;
                qosClassifierPtn.sip1 = 0;
                qosClassifierPtn.sip2 = 0;
                qosClassifierPtn.sip3 = 0;
                qosClassifierPtn.dip0 = innerIPv4Hdr->dst_addr;
                qosClassifierPtn.dip1 = 0;
                qosClassifierPtn.dip2 = 0;
                qosClassifierPtn.dip3 = 0;
            }else if (ADK_IPV6_VER == adk_md_p->innerIPVersion) {
                innerIPv6Hdr = (adk_ipv6_hdr_t*)(pdu + ADK_MD_GET_INNER_IP_OFF(task)); 
                qosClassifierPtn.proto = inner_l4_proto;
                qosClassifierPtn.dscp = (adk_ntohl(innerIPv6Hdr->ver_tc_flabel) >> 22) & 0x3F;
                qosClassifierPtn.sip0 = innerIPv6Hdr->src_addr[0];
                qosClassifierPtn.sip1 = innerIPv6Hdr->src_addr[1];
                qosClassifierPtn.sip2 = innerIPv6Hdr->src_addr[2];
                qosClassifierPtn.sip3 = innerIPv6Hdr->src_addr[3];
                qosClassifierPtn.dip0 = innerIPv6Hdr->dst_addr[0];
                qosClassifierPtn.dip1 = innerIPv6Hdr->dst_addr[1];
                qosClassifierPtn.dip2 = innerIPv6Hdr->dst_addr[2];
                qosClassifierPtn.dip3 = innerIPv6Hdr->dst_addr[3];
            }
        } else {
            qosClassifierPtn.proto = next_hdr;
            ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
            qosClassifierPtn.dscp = ipv6_dscp;
            qosClassifierPtn.sip0 = ipv6_hdr->src_addr[0];
            qosClassifierPtn.sip1 = ipv6_hdr->src_addr[1];
            qosClassifierPtn.sip2 = ipv6_hdr->src_addr[2];
            qosClassifierPtn.sip3 = ipv6_hdr->src_addr[3];
            qosClassifierPtn.dip0 = ipv6_hdr->dst_addr[0];
            qosClassifierPtn.dip1 = ipv6_hdr->dst_addr[1];
            qosClassifierPtn.dip2 = ipv6_hdr->dst_addr[2];
            qosClassifierPtn.dip3 = ipv6_hdr->dst_addr[3];
        }
        aclTreeIndex = adk_ipv6_dp_tables->egressQoSClassifierInfo_p[interface_id]->aclTreeIndex;
        adk_ipv6_dp_egress_qos_classifier_lookup(
                adk_ipv6_dp_tables->egressQoSClassifierInfo_p[interface_id]->ipEgressQoSClassifierCtx[aclTreeIndex],
                &qosClassifierPtn, &qosClassifierFlowId);
        /*Fill up the transmit queue in ADK metadata*/
        qosClassifierAction_p = &(adk_ipv6_dp_tables->egressQoSClassifierActionDp_p[interface_id][aclTreeIndex][qosClassifierFlowId]);
        ADK_METADATA_PARAM(task, dev_tx_queue_id) = qosClassifierAction_p->queueId;
        ADK_METADATA_PARAM(task, pktColor) = qosClassifierAction_p->pktColor;
    }else {
        ADK_METADATA_PARAM(task, dev_tx_queue_id) = adk_qos_dp_get_default_tx_queue();
    }

    return ADK_SUCCESS;
}

static adk_dp_status_t  check_for_fragmentation(uint16_t mtu, uint32_t ipv6_len, uint16_t next_hdr, uint16_t l4_offset, 
                                           uint8_t pktOrigin, adk_ipv6_dp_interface_entry_t *ipv6_if_entry, int coreIndex, 
                                           struct rte_mbuf *task, uint8_t *total_frags, adk_ipv6_hdr_t **ipv6_hdr)
{
    adk_metadata_t *adk_md_p;
    uint16_t udp_len = 0;
    uint32_t cksum = 0;
    uint16_t *tmp_16_p =NULL;
    adk_status_t ret = ADK_SUCCESS;
    uint32_t fragOctets = 0;
    adk_ipv6_dp_frag_error_t  frag_status = ADK_IPV6_DP_FRAG_NOT_REQ; 
    adk_ipv6_dp_if_stats_entry_t  *if_stats = NULL;
    uint8_t num_frags = 1;
    bool l4_cksum_present = FALSE;

    adk_md_p =  ADK_METADATA_PTR(task);

    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->egress_stats_id];
    
    if (mtu < ipv6_len)
    {
        /* check if next header is UDP, then update the checksum */
        if(next_hdr == ADK_IPV6_PROTOCOL_UDP)
        {
            /*Update UDP checksum  */
            tmp_16_p = (uint16_t *)(void *)(((uint8_t *)*ipv6_hdr) + l4_offset + 6);
            *tmp_16_p = 0;

            if(task->ol_flags & PKT_TX_UDP_CKSUM)
            {
                l4_cksum_present = TRUE;
                *tmp_16_p = (adk_ipv6_udp_checksum_generation(task, adk_md_p->l4_offset, *ipv6_hdr));
                if(!(*tmp_16_p))
                {
                    *tmp_16_p = 0xFFFF;
                }
                task->ol_flags &= ~PKT_TX_UDP_CKSUM;
                task->ol_flags &= ~PKT_TX_IPV6;
            }
            else
            {
                /*ALready set the checksum to 0 above*/
            }
        }
        else if(1 == ADK_SYS_IPSEC_BYPASS_GET)
        {
            /* We are now using the UDP_CKSUM bit/s in ol_flags for internal use in ip and ipsec. This needs to be reset when we don't want hw to calculate udp cksum */
            task->ol_flags &= ~PKT_TX_UDP_CKSUM;
            task->ol_flags &= ~PKT_TX_IPV6;
        }
        /* else - IPSec would have reset the flag if inner packet was non-udp */

        INC_NON_NULL(if_stats, ipv6IfStatsOutFragReqds, 1);
        frag_status = adk_ipv6_dp_fragment(&task, coreIndex, mtu,
                &num_frags, &fragOctets, ipv6_if_entry->if_type);
        if (likely(frag_status == ADK_IPV6_DP_FRAG_OK))
        {
            INC_NON_NULL(if_stats, ipv6IfStatsOutFragOKs,  num_frags);
        }
        else
        {
            INC_NON_NULL(if_stats, ipv6IfStatsOutFragFails, 1);
            INC_NON_NULL(if_stats, ipv6IfStatsOutDroppedOctets, fragOctets);
            ADK_IPV6_DP_LOG_ERROR("error: Fragmentation failed \n");
            return ADK_E_DROP_TASK;
        }

        /* after framentation retry the new ipv6_hdr offset */
        ADK_PDU_OFFSET_PTR_GET(task, *ipv6_hdr, 
                adk_ipv6_hdr_t *, adk_md_p->ip_offset);
    }

    /*Checksum generation should not be done for forwarded packet*/
    else if ((IP6_ULI_SENDER_EO == pktOrigin) || (IP6_ULI_SENDER_NETD == pktOrigin)) 
    {
        if(next_hdr == ADK_IPV6_PROTOCOL_UDP)
        {
            udp_len =  adk_ntohs(*(uint16_t *)(((uint8_t *)*ipv6_hdr) + l4_offset + 4));
            /*Update UDP checksum  */
            tmp_16_p = (uint16_t *)(void *)(((uint8_t *)*ipv6_hdr) + l4_offset + 6);
            *tmp_16_p = 0;

            if(task->ol_flags & PKT_TX_UDP_CKSUM)
            {
#ifdef UDP_CHECKSUM_ENABLED
                /* offload udp checksum calculation. */
                task->l3_len = l4_offset;
                task->l4_len = ADK_IPV6_UDP_HDR_LEN;

                task->ol_flags |= PKT_TX_UDP_CKSUM;
                task->ol_flags |= PKT_TX_IPV6;
                /*If extension header is present in packet, then rte pseudo header doesnt support.
                 * Hence we are removing extension header, before passing to pseudo header
                 */
                cksum = adk_ipv6_phdr_cksum((void *)*ipv6_hdr, udp_len);
                *tmp_16_p = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);

                ADK_IPV6_DP_LOG_DEBUG("pseudo UDP Checksum (non-fragmented packet, cksum-calculation offloaded)=%x\n", *tmp_16_p);
#else
                *tmp_16_p = (adk_ipv6_udp_checksum_generation(task, adk_md_p->l4_offset, *ipv6_hdr));
                if(!(*tmp_16_p))
                {
                    *tmp_16_p = 0xFFFF;
                }
                task->ol_flags &= ~PKT_TX_UDP_CKSUM;
                task->ol_flags &= ~PKT_TX_IPV6;
#endif
            } 
            else
            {
                /*Checksum already set to 0 above*/
            }

        }
        else if(1 == ADK_SYS_IPSEC_BYPASS_GET)
        {
            /* We are now using the UDP_CKSUM bit/s in ol_flags for internal use in ip and ipsec. This needs to be reset when we don't want hw to calculate udp cksum */
            task->ol_flags &= ~PKT_TX_UDP_CKSUM;
            task->ol_flags &= ~PKT_TX_IPV6;
        }
        /* else - IPSec would have reset the flag if inner packet was non-udp */

    }
    *total_frags = num_frags;
    return ret;
}

static inline void set_ethuli_params(struct rte_mbuf *task, uint8_t ipv6_dscp, uint8_t ipv6_pcp, 
                                        bool onlink_flag, uint16_t if_id, adk_ipv6_hdr_t  *ipv6_hdr,  adk_ipv6_frag_t frag_flag, 
                                        uint8_t next_hdr, adk_ipv6_dp_nexthop_entry_t *nexthop_entry, uint8_t num_frags,
                                        uint16_t *destFunc)
{
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task);
    adk_ethernet_dp_uli_imc_param_t *eth_uli_md_p = NULL;
    eth_uli_md_p = (adk_ethernet_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
    eth_uli_md_p->dscp = ipv6_dscp;
    eth_uli_md_p->pcp  = ipv6_pcp;
    eth_uli_md_p->is_unfrag_first_frag =((frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)||
            (frag_flag==ADK_IPV6_FIRST_FRAGMENT))? TRUE: FALSE;
    eth_uli_md_p->l4_proto = next_hdr;
    eth_uli_md_p->next_hop.is_ipv6 = true;
    eth_uli_md_p->output_intf_id = if_id;
    eth_uli_md_p->frag_count = num_frags;

    if(onlink_flag)
    {
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[0] = ipv6_hdr->dst_addr[0];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[1] = ipv6_hdr->dst_addr[1];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[2] = ipv6_hdr->dst_addr[2];
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[3] = ipv6_hdr->dst_addr[3];
    }
    else if(nexthop_entry != NULL)
    {
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[0] = adk_ntohl(nexthop_entry->nextHopIpv6Addr.addr[0]);
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[1] = adk_ntohl(nexthop_entry->nextHopIpv6Addr.addr[1]);
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[2] = adk_ntohl(nexthop_entry->nextHopIpv6Addr.addr[2]);
        eth_uli_md_p->next_hop.u.ipv6_addr.addr[3] = adk_ntohl(nexthop_entry->nextHopIpv6Addr.addr[3]);
    }

    *destFunc = ADK_ETH_ULI_FN_ID;

    ADK_IPV6_DP_LOG_DEBUG("IMC info to Ethernet \n");
    ADK_IPV6_DP_LOG_DEBUG("dscp %d \n", eth_uli_md_p->dscp);
    ADK_IPV6_DP_LOG_DEBUG("pcp %d \n", eth_uli_md_p->pcp);
    ADK_IPV6_DP_LOG_DEBUG("is_unfrag_first_frag %d \n", eth_uli_md_p->is_unfrag_first_frag);
    ADK_IPV6_DP_LOG_DEBUG("l4_proto %d \n", eth_uli_md_p->l4_proto);
    ADK_IPV6_DP_LOG_DEBUG("frag_count %d \n", eth_uli_md_p->frag_count);
    ADK_IPV6_DP_LOG_DEBUG("dstAddr[0] %x\n", eth_uli_md_p->next_hop.u.ipv6_addr.addr[0]);
    ADK_IPV6_DP_LOG_DEBUG("dstAddr[1] %x\n", eth_uli_md_p->next_hop.u.ipv6_addr.addr[1]);
    ADK_IPV6_DP_LOG_DEBUG("dstAddr[2] %x\n", eth_uli_md_p->next_hop.u.ipv6_addr.addr[2]);
    ADK_IPV6_DP_LOG_DEBUG("dstAddr[3] %x\n", eth_uli_md_p->next_hop.u.ipv6_addr.addr[3]);
}

static inline void set_ipseculi_params(struct rte_mbuf *task, uint8_t next_hdr, uint8_t l4_offset,
        uint8_t path, uint16_t *destFunc)
{
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task);
    task->l3_len = l4_offset;
    task->l4_len = ADK_IPV6_UDP_HDR_LEN;

    ADK_IPV6_DP_LOG_DEBUG("IPV6 ULI: sending packet to IPsec\n");
    adk_ipsec_dp_uli_imc_param_t  *ipsecUliImc = (adk_ipsec_dp_uli_imc_param_t *)
        &(adk_md_p->imc_param_start);
    ipsecUliImc->pktOrigin = path;
    ipsecUliImc->ipv6_nextHdr = next_hdr;
    *destFunc = ADK_IPSEC_ULI_FN_ID;
}


static uint8_t  adk_ipv6_flow_cache_lookup( struct rte_mbuf *task, ipv6_l4_info *l4_info,
        adk_ipv6_hdr_t  *ipv6_hdr, int coreIndex)
{

    adk_status_t ret = ADK_SUCCESS;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p;
    uint32_t  hash_key;

    adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;
    scratch_pad_ipv6 = ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreIndex);
    scratch_pad_ipv6->ip_version = ADK_IPV6_VER;

    if ((IP6_ULI_SENDER_IPSEC != adk_ipv6_uli_md_p->sender))// && likely(l4_info->next_hdr == ADK_IPV6_PROTOCOL_UDP)) 
    {
        ret = adk_sys_dp_uli_flow_cache_get((adk_ip_hdr_t *)ipv6_hdr,
                ADK_IPV6,
                l4_info->next_hdr,
                l4_info->src_port,
                l4_info->dst_port,
                &scratch_pad_ipv6->flow_cache_entry,
                &hash_key);
        if (ADK_E_FLOW_CACHE_ALLOC_FAILED == ret) 
        {
            /*Flow cache allocation failed, take slow path*/
        }
        UPDATE_FC(adk_ipv6_uli_md_p->pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->nextHdr , l4_info->next_hdr));
        if (ADK_E_FLOW_CACHE_PKT_FAST_PATH == ret) 
        {
            /*Active Flow cache .. go for cached action processing*/
            scratch_pad_ipv6->pktPath = ADK_E_FLOW_CACHE_PKT_FAST_PATH;
            ret =  ADK_SUCCESS;
        } 
        else 
        {
            scratch_pad_ipv6->pktPath = ADK_E_FLOW_CACHE_PKT_SLOW_PATH;
            ret =  ADK_SUCCESS;
        }
    }

    return ret;
}
void adk_ipv6_increment_tx_l4_stats(uint8_t next_hdr, uint8_t path, uint8_t pktOrigin,
                                 uint16_t ipv6_len, int coreIndex)
{
    adk_ipv6_dp_l4counter_statistics_t   *ipv6_l4proto_stats = NULL;
    ipv6_l4proto_stats = adk_ipv6_dp_tables->ipv6_dp_l4counter_stats_table[coreIndex];

    if (pktOrigin == IP6_ULI_SENDER_NETD)
    {
        if (likely((next_hdr == ADK_IPV6_PROTOCOL_TCP) && (IP6_ULI_SENDER_IPSEC != path)))
        {
            INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpOutPackets, 1);
            INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpOutOctets, ipv6_len);
        }
        else if (likely((next_hdr == ADK_IPV6_PROTOCOL_SCTP) && (IP6_ULI_SENDER_IPSEC != path)))
        {
            INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpOutPackets, 1);
            INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpOutOctets, ipv6_len);
        }
    }
    else if (pktOrigin == IP6_ULI_SENDER_EO)
    {
        if (unlikely((next_hdr == ADK_IPV6_PROTOCOL_TCP) && (IP6_ULI_SENDER_IPSEC != path)))
        {
            INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpOutPackets, 1);
            INC_NON_NULL(ipv6_l4proto_stats, ipv6TcpOutOctets, ipv6_len);
        }
        else if (unlikely((next_hdr == ADK_IPV6_PROTOCOL_SCTP) && (IP6_ULI_SENDER_IPSEC != path)))
        {
            INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpOutPackets, 1);
            INC_NON_NULL(ipv6_l4proto_stats, ipv6SctpOutOctets, ipv6_len);
        }
    }
}
static uint8_t  adk_ipv6_outer_hdr_esp_processing( struct rte_mbuf *task, int coreIndex, uint16_t  *destFunc)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t frag_id, len_nh_hlimit=0;
    uint16_t if_id = 0, src_port = 0, dst_port = 0, ipv6_len = 0, offset_to_frag;
    uint16_t mtu = 0;
    uint8_t next_hdr = 0, num_frags = 1, excep, ipv6_dscp = 0, ipv6_pcp = 0;
    uint8_t *pdu = NULL, vrd_id=ADK_IPV6_DEFAULT_VRD_ID; 
    bool l4_port_present = FALSE, l4_cksum_present = TRUE;
    uint8_t  path = 0, pktOrigin = 0, total_frags = 1;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry = NULL;
    adk_ipv6_dp_route_entry_t* route_entry = NULL;
    adk_ipv6_dp_route_entry_t *sip_entry = NULL;
    adk_ipv6_dp_nexthop_entry_t *nexthop_entry = NULL;
    adk_ipv6_hdr_t  *ipv6_hdr = NULL;
    adk_ipv6_dp_if_stats_entry_t         *if_stats = NULL;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    ipv6_utility_counters_t              *ipv6_utility_stats = NULL;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p;
    uint8_t action=ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD;
    adk_ipv6_pbr_val_t pbr_val;
    adk_dp_status_t dp_st;
    adk_ipv6_dp_pbr_action_entry_t       *pbr_action = NULL;
    bool defTblLookup = TRUE;
    uint16_t l4_offset;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
    adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;

    path = adk_ipv6_uli_md_p->sender;
    pktOrigin = adk_ipv6_uli_md_p->pktOrigin;
    scratch_pad_ipv6 = ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreIndex);


    ADK_PDU_PTR_GET(task, pdu, uint8_t *);
    INC_NON_NULL(ipv6_global_stats, uliPdus, 1);
    vrd_id = ADK_IPV6_LOCAL_ROUTING_TABLE;
    ADK_PDU_OFFSET_PTR_GET(task,
            ipv6_hdr,
            adk_ipv6_hdr_t *,
            adk_md_p->ip_offset);

    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);

    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8)&0xFF), 
            (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);
    adk_md_p->l4_offset = l4_offset + adk_md_p->ip_offset;

    ipv6_len = (len_nh_hlimit>>16) + ADK_IPV6_HDR_LEN;

    /*
     * Check if PBR is enabled for the interface(internally maintained flag, not a user configurable one)
     * If yes, do PBR lookup. If PBR lookup fails, fall-back on DIP based lookup
     */ 
    defTblLookup = TRUE;
    ret = adk_ipv6_dp_egress_pbr(pbr_policy_set_id, 0, next_hdr,
            ipv6_hdr, src_port, dst_port, frag_flag, &pbr_val);
    if(likely(ret == ADK_SUCCESS)) /* PBR lookup successful, get the route table ID */
    {
        pbr_action = (adk_ipv6_dp_pbr_action_entry_t *)
            &adk_ipv6_dp_tables->ipv6_dp_pbr_table.tbl[(uintptr_t)pbr_val];
        (adk_ipv6_dp_tables->ipv6_dp_pbr_table.pbrRuleStats[coreIndex*IPV6_PBR_MAX_RULES_PER_SET +
         pbr_action->stats_id - 1])++;
        /*PBR lookup success, do DIP lookup in specific table */
        vrd_id = pbr_action->routeTableId;
        ADK_IPV6_DP_LOG_DEBUG("### PBR lookup success. routeTableId : %u\n", vrd_id);
        UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->routeTblId, vrd_id));

        dp_st = adk_ipv6_dp_route_entry_lookup(vrd_id, (uint8_t *) &ipv6_hdr->dst_addr[0],
                &route_entry); 
        if(likely(dp_st == ADK_SUCCESS)) /* DIP based lookup successful */
        {
            defTblLookup = FALSE;
            action = route_entry->action;
            ADK_IPV6_DP_LOG_DEBUG("IPv6 route action %d \n",route_entry->action);
        }
    }
    if (likely(defTblLookup == TRUE))
    {
        /* Either PBR lookup has failed or route lookup in PBR yielded route table has failed
         * In both cases, do DIP lookup in Default routing table 
         */
        vrd_id = adk_ipv6_dp_tables->defaultRouteTable;
        UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->routeTblId , vrd_id));
        dp_st = adk_ipv6_dp_route_entry_lookup(vrd_id, (uint8_t *) &ipv6_hdr->dst_addr[0],
                &route_entry); 

        if(likely(dp_st == ADK_SUCCESS)) /* DIP based lookup successful */
        {
            action = route_entry->action;
            ADK_IPV6_DP_LOG_DEBUG("IPv6 route action %d \n",route_entry->action);
        }
        else /* DIP based lookup fails*/
        {
            ADK_IPV6_DP_LOG_ERROR("error: IPV6 route lookup failed %08x %08x"
                    , adk_ntohl(ipv6_hdr->dst_addr[0]), adk_ntohl(ipv6_hdr->dst_addr[1]));
            ADK_IPV6_DP_LOG_ERROR(" %08x %08x \n",
                    adk_ntohl(ipv6_hdr->dst_addr[2]), adk_ntohl(ipv6_hdr->dst_addr[3]));
            INC_NON_NULL(ipv6_global_stats, routeNotFoundDrop, 1);

            /* raise an exception "Route not found" */
            ret = adk_dp_send_ipv6_uli_exception(task, vrd_id, 0, 0, adk_md_p->ip_offset, 0, 0,
                    ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN, ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND, 
                    coreIndex, 0, ipv6_hdr, l4_cksum_present);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                        "ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND \n");
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }
            return ADK_E_TASK_CONSUMED;
        }
    }
    /* Below function will do lookup for SIP, and lookup will be success for
     * locally originated packets, for forwarding packet lookup will fail, so need to
     * take care for forwarding packets 
     */
    if ((IP6_ULI_SENDER_NETD == pktOrigin) || (IP6_ULI_SENDER_EO == pktOrigin))
    {
        ret = adk_ipv6_dp_route_entry_lookup(ADK_IPV6_LOCAL_ROUTING_TABLE, (uint8_t *) &ipv6_hdr->src_addr[0], &sip_entry);

        if (unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error: IPV6 Source IP lookup failed, so dropping the packet %08x %08x"
                    , adk_ntohl(ipv6_hdr->src_addr[0]), adk_ntohl(ipv6_hdr->src_addr[1]));
            ADK_IPV6_DP_LOG_ERROR(" %08x %08x \n",
                    adk_ntohl(ipv6_hdr->src_addr[2]), adk_ntohl(ipv6_hdr->src_addr[3]));

            INC_NON_NULL(ipv6_global_stats, ipv6OutSrcIpInvalid, 1);
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_outer_hdr_esp_processing_return;
        }
        adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id =
            sip_entry->ipv6_addr_stats_id;
        adk_md_p->outerIPVersion = ADK_IPV6_PDU_VER_VALUE;
        UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->outerSipStatsId, sip_entry->ipv6_addr_stats_id));
        /* This is V6-V6 Or V6-V4 case. Check if both inner and outer IP are same. We dont want 
         * to increment the address stats twice in that case. Update either the Outer or the Inner IP
         * stats ID. Hence the check
         */
        if ((adk_md_p->innerIPVersion == ADK_IPV6_PDU_VER_VALUE) &&
                (adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id
                 == sip_entry->ipv6_addr_stats_id))
        {
            adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID;
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->innerSipStatsId, (uint16_t)ADK_SYS_INVALID_MDATA_ID));
        }
        else
        {
            /*Dont update the statsId for Inner IP , Could be that inner/outer IP are same Or
             *V4 woould have updated the Inner IP stats ID(V6 over V4 case)
             */
        }
    }

    switch(action)
    {
        case ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD:
        case ADK_IPV6_ROUTE_LOOKUP_ACTION_ONLINK:


            if(route_entry->action == ADK_IPV6_ROUTE_LOOKUP_ACTION_FWD)
            {
                /* Nexthop lookup */
                ret = adk_ipv6_dp_nexthop_entry_lookup(route_entry->nexthop_id,
                        &nexthop_entry);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    ADK_IPV6_DP_LOG_ERROR("error: nexthop lookup failed for id %d\n",
                            route_entry->nexthop_id);
                    INC_NON_NULL(ipv6_global_stats,noNextHopEntryDrop, 1);
                    /* raise an exception "L2 not resolved" */
                    ret = adk_dp_send_ipv6_uli_exception(task, vrd_id, 0, 0, adk_md_p->ip_offset, 0, 0,
                            ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN,
                            ADK_IPV6_EXCEPTION_RESOLVE_DESTINATION_L2_ADDRESS, coreIndex, 0,
                            ipv6_hdr, l4_cksum_present);
                    if(unlikely(ret != ADK_SUCCESS))
                    {
                        ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception" 
                                "ADK_IPV6_EXCEPTION_RESOLVE_DESTINATION_L2_ADDRESS\n");
                    }
                    ret=ADK_E_DROP_TASK;
                    goto adk_ipv6_outer_hdr_esp_processing_return;
                }
                if_id = nexthop_entry->interface_id;
                UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->nextHop , route_entry->nexthop_id));
                UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->egressIntfId, ADK_IPV6_INVALID_NH_INTF_ID));
            }
            else
            {
                /* For a Onlink case , get the output interface from route DP struct */
                if_id = route_entry->interface_id;
                UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->egressIntfId, if_id));
                UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->nextHop, ADK_IPV6_INVALID_NH_INTF_ID));
            }
            /* Egress interface lookup */
            ret = adk_ipv6_dp_interface_entry_lookup(if_id, 
                    0, 
                    &ipv6_if_entry);
            if(unlikely(ret != ADK_SUCCESS))
            {
                ADK_IPV6_DP_LOG_ERROR("Error while retriving egress interface attributes \n");
                /* incr the gbl count */
                INC_NON_NULL(ipv6_global_stats, InvalidIfFail, 1);
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }

            if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->egress_stats_id];

            /*Fetch the routeMTU from dp table*/
            mtu = route_entry->routeMtu;
            /* if route MTU is non-zero, then it has higher precedence over 
             * interface MTU, else interface MTU will be used for frag 
             */
            if(!mtu)
            {
                mtu = ipv6_if_entry->mtu;
            }

            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->routeMtu, mtu));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->interfaceStatsId, ipv6_if_entry->egress_stats_id));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->isQoSEnabled, (ipv6_if_entry->interfaceAttr & ADK_IPV6_INTERFACE_QOS_ENABLE)));

            if (unlikely((mtu < ipv6_len) && ((IP6_ULI_SENDER_IP6LLI == pktOrigin))))
            {
                /*Frag is not done in forwarding case for IPv6
                 * Drop the packet and raise an exception
                 */
                ADK_IPV6_DP_LOG_DEBUG("Packet too big drop\n");
                ipv6_global_stats->ipv6FwdMtuExceed++ ;
                /*raise an exception */
                ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                        if_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                        ADK_IPV6_EXCEPTION_MTU_VIOLATION, coreIndex, mtu);
                if(unlikely(ret != ADK_SUCCESS))
                {
                    ADK_IPV6_DP_LOG_ERROR("error: IPV6 failed to send exception"
                            "ADK_IPV6_EXCEPTION_MTU_VIOLATION \n");
                }
                ret = ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_esp_processing_return;

            }

            ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
            /*Flags set by Netdriver for PTP traffic requiring egress timestamp*/
            if (unlikely((IP6_ULI_SENDER_NETD == pktOrigin) && 
                        ((ADK_MD_GET_TS_MSG_TYPE(task) == ADK_NETD_TS_CAPTURE_IDX_ALLOC) ||
                         (ADK_MD_GET_TS_MSG_TYPE(task) == ADK_NETD_TS_CAPTURE)))) {
                /*PTP queue lookup*/
                ADK_METADATA_PARAM(task, dev_tx_queue_id) = adk_qos_dp_get_ptp_queue();
            }else{
                adk_ipv6_dp_qos_classification(task, ipv6_if_entry, ipv6_hdr, next_hdr, if_id);
            }

            if (ADK_QOS_INVALID_QUEUE_ID == adk_md_p->dev_tx_queue_id) {
                INC_NON_NULL(if_stats, ipv6IfQoSLookupFailed, 1);
                ret=ADK_E_DROP_TASK;
                goto adk_ipv6_outer_hdr_esp_processing_return;
            }

            bool isPHBQueue = false;
            uint32_t qosStatsId;
            /*Find the stats id for QoS queue*/
            qosStatsId = adk_qos_dp_get_queue_stats_id(adk_md_p->dev_tx_queue_id, &isPHBQueue);

            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->dev_tx_queue_id, adk_md_p->dev_tx_queue_id));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->pktColor, adk_md_p->pktColor));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->qosStatsId, qosStatsId));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->isPHBQueue, isPHBQueue));

            adk_md_p->qosStatsId = qosStatsId;
            adk_md_p->isPHBQueue = isPHBQueue;
            adk_md_p->isIpPkt = true;

            /* update the egress stats Id */
            adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id =
                ipv6_if_entry->egress_stats_id;
            adk_md_p->outerIPVersion = ADK_IPV6_PDU_VER_VALUE;
            adk_md_p->outerIpPktLen = ipv6_len;

            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxSendToEthUli);
            check_for_fragmentation(mtu, ipv6_len, next_hdr, l4_offset, pktOrigin, ipv6_if_entry, coreIndex, task, &total_frags, &ipv6_hdr);

            if(ADK_SYS_IPSEC_BYPASS_GET || (INVALID_FRAG_COUNT == adk_ipv6_uli_md_p->frag_count))
                num_frags = total_frags;
            else
                num_frags = adk_ipv6_uli_md_p->frag_count;

            set_ethuli_params(task, ipv6_dscp, ipv6_pcp, route_entry->on_link, if_id,
                                 ipv6_hdr, frag_flag, next_hdr, nexthop_entry, num_frags, destFunc);
            goto adk_ipv6_outer_hdr_esp_processing_return;
            break;

        case ADK_IPV6_ROUTE_LOOKUP_ACTION_DROP:

            ret = ADK_E_DROP_TASK;
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxRouteActionDrop);
            goto adk_ipv6_outer_hdr_esp_processing_return;
            break;

        default:
            /*drop the packet */ 
            ADK_IPV6_DP_LOG_ERROR("error: Invalid IPV6 route lookup action \n");

            INC_NON_NULL(ipv6_global_stats,invalidRouteAction, 1);
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxInvalidRouteAction);
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_outer_hdr_esp_processing_return;
            break;
    }

adk_ipv6_outer_hdr_esp_processing_return:
    return ret;
}

void fetch_rx_cycle_count(uint8_t path)
{
	if (IP6_ULI_SENDER_IPSEC != path)
	{
		ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS1_TS();
	}
	else
	{
		ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS2_TS();
	}
}

void fetch_tx_cycle_count(uint8_t path)
{
	if (IP6_ULI_SENDER_IPSEC != path)
	{
		ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS1_TS();
	}
	else
	{
		ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS2_TS();
	}
}


static uint8_t adk_ipv6_dp_pkt_proc_uli_slow_path(struct rte_mbuf *task,
                         int coreIndex, uint16_t *destFunc) 
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t frag_id=0, len_nh_hlimit=0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag=0, ipv6_len = 0;
    uint8_t next_hdr = 0, excep, ipv6_dscp = 0, ipv6_pcp = 0;
    bool l4_port_present = FALSE;
    uint8_t  path = 0, pktOrigin = 0;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry = NULL;
    adk_ipv6_hdr_t  *ipv6_hdr = NULL;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    ipv6_utility_counters_t              *ipv6_utility_stats = NULL;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p;
    adk_ethernet_dp_uli_imc_param_t *eth_uli_md_p = NULL;
    uint32_t interface_id = 0;
    uint16_t l4_offset;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
    adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;
    path = adk_ipv6_uli_md_p->sender;
    pktOrigin = adk_ipv6_uli_md_p->pktOrigin;

    ADK_IPV6_DP_LOG_DEBUG("Received packet at IPv6 ULI SLOWPATH %d and Origin %d\n",path, pktOrigin);

    ADK_PDU_OFFSET_PTR_GET(task,
            ipv6_hdr,
            adk_ipv6_hdr_t *,
            adk_md_p->ip_offset);

    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = (len_nh_hlimit>>16) + ADK_IPV6_HDR_LEN;

    /* Removing the FCS bits */
    if (task->pkt_len > ipv6_len)
    {
        ret = rte_pktmbuf_trim(task, (task->pkt_len - ipv6_len));
    }

    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8)&0xFF),
            (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);
    adk_md_p->l4_offset = l4_offset + adk_md_p->ip_offset;
    adk_md_p->innerIpPktLen = ipv6_len;

    if ((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xff) {
        adk_md_p->isIpMcastPkt = true;
    }else {
        adk_md_p->isIpMcastPkt = false;
    } 

    /*Increment L4 Egress stats*/
    adk_ipv6_increment_tx_l4_stats(next_hdr, path, pktOrigin, (ipv6_len-l4_offset), coreIndex);

    if(likely(is_global_unicast(ipv6_hdr) == ADK_SUCCESS))
    {
        if ((IP6_ULI_SENDER_IPSEC != path) && (!ADK_SYS_IPSEC_BYPASS_GET))
        {
            ret = adk_ipv6_inner_hdr_processing(task, coreIndex, destFunc);
        }
        else
        {
            ret = adk_ipv6_outer_hdr_esp_processing(task, coreIndex, destFunc);
        }
    }
    else
    {
        if (!ADK_SYS_IPSEC_BYPASS_GET && (IP6_ULI_SENDER_IPSEC != path))
        {
            set_ipseculi_params(task, next_hdr, l4_offset, path, destFunc);
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxSendToIpsecUli);
            ret=ADK_SUCCESS;
            goto adk_ipv6_dp_pkt_proc_uli_slow_path_return;
        }

        if(adk_eth_dp_interface_id_get(task,&interface_id) != ADK_SUCCESS)
        {
            ADK_IPV6_DP_LOG_ERROR("Invalid interface received for Multicast/link-local packet\n");
            /*TODO: Need to add this counter to CP as well, after approval */
            INC_NON_NULL(ipv6_global_stats, InvalidIfFail, 1);
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_dp_pkt_proc_uli_slow_path_return;
        }

        /* Egress interface lookup */
        ret = adk_ipv6_dp_interface_entry_lookup(interface_id,
                0,
                &ipv6_if_entry);
        if(unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
            /* incr the gbl count */
            INC_NON_NULL(ipv6_global_stats, InvalidIfFail, 1);
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_dp_pkt_proc_uli_slow_path_return;
        }

        ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
        ipv6_pcp = ipv6_if_entry->dscp_pcp[ipv6_dscp] & 0x0F;
        eth_uli_md_p = (adk_ethernet_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
        eth_uli_md_p->dscp  = ipv6_dscp;
        eth_uli_md_p->pcp  = ipv6_pcp;
        /*Setting the next stage as ADK_ETH_ULI_NETD_BRIDGE_FN_ID , which will not do Eth encapsulation
         * or any lookups. It will retain the Eth header of the packet comming from Linux and send out via
         * HLP 20 as it is. This is ONLY for mutlicast/link-local packets, as it will be originated only
         * from Linux and not from EO  */
        *destFunc = ADK_ETH_ULI_NETD_BRIDGE_FN_ID;

        ADK_IPV6_DP_LOG_DEBUG("Multicast/link-local packets will be sent on interface %d\n",interface_id);
        adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id =
            ipv6_if_entry->egress_stats_id;
        adk_md_p->outerIPVersion = ADK_IPV6_PDU_VER_VALUE;
        adk_md_p->outerIpPktLen = ipv6_len;
        task->ol_flags &= ~PKT_TX_UDP_CKSUM;
        task->ol_flags &= ~PKT_TX_IPV6;
        adk_ipv6_dp_qos_classification(task, ipv6_if_entry, ipv6_hdr, next_hdr, interface_id);
        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxSendToEthUli);
        return ADK_SUCCESS;
    }
adk_ipv6_dp_pkt_proc_uli_slow_path_return:
    return ret;
}

static uint8_t  adk_ipv6_inner_hdr_processing( struct rte_mbuf *task, int coreIndex, uint16_t *destFunc)
{
    adk_status_t ret = ADK_SUCCESS;
    uint8_t  path = 0, pktOrigin = 0;
    adk_ipv6_dp_route_entry_t *inner_sip_entry = NULL;
    adk_ipv6_hdr_t  *ipv6_hdr = NULL;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    ipv6_utility_counters_t              *ipv6_utility_stats = NULL;
    uint32_t frag_id, len_nh_hlimit=0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag;
    uint8_t next_hdr = 0, excep, l4_offset;
    bool l4_port_present = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
    adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;
    path = adk_ipv6_uli_md_p->sender;
    pktOrigin = adk_ipv6_uli_md_p->pktOrigin;

    ADK_PDU_OFFSET_PTR_GET(task,
            ipv6_hdr,
            adk_ipv6_hdr_t *,
            adk_md_p->ip_offset);


    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);

    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8)&0xFF),
            (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);
    adk_md_p->l4_offset = l4_offset + adk_md_p->ip_offset;

    if (((IP6_ULI_SENDER_NETD == pktOrigin) || (IP6_ULI_SENDER_EO == pktOrigin))
            && IP6_ULI_SENDER_IPSEC != path)
    {
        /*Updating the statsID for Inner IP, before sending to IPSec ULI for encr*/
        ret = adk_ipv6_dp_route_entry_lookup(ADK_IPV6_LOCAL_ROUTING_TABLE, (uint8_t *) &ipv6_hdr->src_addr[0], &inner_sip_entry);

        if (unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error: IPV6 Source IP lookup failed, so dropping the packet %08x %08x"
                    , adk_ntohl(ipv6_hdr->src_addr[0]), adk_ntohl(ipv6_hdr->src_addr[1]));
            ADK_IPV6_DP_LOG_ERROR(" %08x %08x \n",
                    adk_ntohl(ipv6_hdr->src_addr[2]), adk_ntohl(ipv6_hdr->src_addr[3]));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->innerSipStatsId, (uint16_t)ADK_SYS_INVALID_MDATA_ID));
            UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->isOriginated, true));

            INC_NON_NULL(ipv6_global_stats, ipv6OutSrcIpInvalid, 1);
            ret = ADK_E_DROP_TASK;
            goto adk_ipv6_inner_hdr_processing_return;
        }
        adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id =
            inner_sip_entry->ipv6_addr_stats_id;
        adk_md_p->innerIPVersion = ADK_IPV6_PDU_VER_VALUE;

        UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->innerSipStatsId, inner_sip_entry->ipv6_addr_stats_id));
        UPDATE_FC(pktOrigin, ADK_SYS_DP_FLOW_CACHE_SET_PARAM(scratch_pad_ipv6->flow_cache_entry->isOriginated, true));
    }

    /* Added check to ensure sender is EO or Netd. Forwarding traffic
     * and IPSec ULI packet shouldnt be sent to IPsec again
     */
    if (!ADK_SYS_IPSEC_BYPASS_GET && (IP6_ULI_SENDER_IPSEC != path))
    {
        set_ipseculi_params(task, next_hdr, l4_offset, path, destFunc);
        INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6EgressUtilityCounters.ipv6TxSendToIpsecUli);
        ret=ADK_SUCCESS;
        goto adk_ipv6_inner_hdr_processing_return;
    }

    adk_ipv6_inner_hdr_processing_return:
    return ret;
}

static int is_global_unicast(adk_ipv6_hdr_t *ipv6_hdr)
{
    uint8_t ret = ADK_SUCCESS;
    if(adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24 == 0xff ||
            adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24 == 0xfe)
        ret = ADK_E_INTERNAL_ERROR;
    return ret;
}

static inline adk_dp_status_t
adk_ipv6_dp_pkt_proc_uli_single(struct rte_mbuf *task,
                        int coreIndex,
                        uint16_t *destFunc)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t frag_id=0, len_nh_hlimit=0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag=0;
    uint8_t next_hdr = 0, excep;
    bool l4_port_present = FALSE;
    uint8_t  path = 0, pktOrigin = 0;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    adk_ipv6_hdr_t  *ipv6_hdr = NULL;
    adk_ipv6_dp_global_statistics_t      *ipv6_global_stats = NULL;
    ipv6_utility_counters_t              *ipv6_utility_stats = NULL;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p;
    uint16_t l4_offset, ipv6_len = 0;
    ipv6_l4_info l4_info;

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];
    adk_md_p =  ADK_METADATA_PTR(task);
    adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;
    path = adk_ipv6_uli_md_p->sender;
    pktOrigin = adk_ipv6_uli_md_p->pktOrigin;

    ADK_IPV6_DP_LOG_DEBUG("Received packet at IPv6 ULI PATH %d and Origin %d\n",path, pktOrigin);

    fetch_rx_cycle_count(path);

    INC_NON_NULL(ipv6_global_stats, uliPdus, 1);
    ADK_PDU_OFFSET_PTR_GET(task,
            ipv6_hdr,
            adk_ipv6_hdr_t *,
            adk_md_p->ip_offset);

    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = (len_nh_hlimit >>16) + ADK_IPV6_HDR_LEN;
    adk_md_p->innerIpPktLen = ipv6_len;

    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8)&0xFF), 
            (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);

    adk_md_p->l4_offset = l4_offset + adk_md_p->ip_offset;
    l4_info.frag_id = frag_id;
    l4_info.src_port = src_port;
    l4_info.dst_port = dst_port;
    l4_info.next_hdr = next_hdr;
    l4_info.frag_flag = frag_flag;
    l4_info.l4_port_present = l4_port_present;
    l4_info.offset_to_frag = offset_to_frag;
    l4_info.l4_offset = l4_offset;

    ret = adk_ipv6_flow_cache_lookup(task, &l4_info, ipv6_hdr, coreIndex);

    if (likely(scratch_pad_ipv6->pktPath == ADK_E_FLOW_CACHE_PKT_FAST_PATH) &&
            (ADK_SUCCESS == ret))
    {
        /*Increment L4 Egress stats*/
        adk_ipv6_increment_tx_l4_stats(next_hdr, path, pktOrigin, (ipv6_len-l4_offset), coreIndex);

        if (likely(!ADK_SYS_IPSEC_BYPASS_GET))
        {
            ret = adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass1(task, &l4_info, destFunc);
        }
        else
        {
            ret = adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass2(task, coreIndex, destFunc);
        }
    }
    else if(scratch_pad_ipv6->pktPath == ADK_E_FLOW_CACHE_PKT_SLOW_PATH)
    {
        ret = adk_ipv6_dp_pkt_proc_uli_slow_path(task, coreIndex, destFunc);
    }

    fetch_tx_cycle_count(path);
    return ret;
}

/*!
 * =============================================================
 * adk_ipv6_dp_pkt_proc_netd()
 *
 * structure of imc task params :
 *
 *
 * brief:
 * This is an entry fcuntion for processing Netdriver packets
 * does following:
 *
 * a. Statistics update
 * b. Egress ACL
 * c. Egress QOS
 * d. This function always return "task Consumed"
 *      in case of error packets are dropped/exceptions are raised.
 *      in case of bind, packets are sent to destination module.
 * =============================================================
 */
adk_status_t 
adk_ipv6_dp_pkt_proc_netd(void  **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t *task_out;
    adk_metadata_t *adk_md_p;
    task_out = (adk_pktio_task_t *)(*task_p);

    if (0 != ADK_MD_GET_IP_OFF(task_out))
    {
        uint8_t * pdu;
        ADK_PDU_PTR_GET(task_out, pdu, uint8_t *);
        ADK_PDU_HEAD_PULL(task_out, pdu, (uint8_t *), ADK_MD_GET_IP_OFF(task_out));
        ADK_MD_SET_IP_OFF(task_out, 0);
    }
    /* If origin=netd and global-flag=true -> generate udp checksum*/
    if(ADK_SYS_UDP_GEN_CONFIG_GET)
    {
        task_out->ol_flags |= PKT_TX_UDP_CKSUM;
    }
    else  /*Explicitly disabling CKSUM HW offload */
    {
        task_out->ol_flags &= ~PKT_TX_UDP_CKSUM;
    }
    task_out->ol_flags |= PKT_TX_IPV6;

    adk_md_p = ADK_METADATA_PTR(task_out);
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID; 
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID; 
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID;
    adk_md_p->innerIPVersion = 0;
    adk_md_p->saID = ADK_IPSEC_INVALID_SA_ID;

    adk_ipv6_dp_uli_imc_data_t *ipv6_imc_data_p = (adk_ipv6_dp_uli_imc_data_t *)&(adk_md_p->imc_param_start);
    memset(ipv6_imc_data_p, 0, sizeof(adk_ipv6_dp_uli_imc_data_t));
    ipv6_imc_data_p->sender = IP6_ULI_SENDER_NETD;
    ipv6_imc_data_p->pktOrigin = IP6_ULI_SENDER_NETD;
    coreIndex = ADK_LCORE_INDEX;

    ret = adk_ipv6_dp_pkt_proc_uli(task_p, 0, 1, NULL, NULL, coreIndex);
    if(unlikely((ADK_E_INTERNAL_ERROR == ret)
                         ||(ADK_E_DROP_TASK == ret)))
    {
        /* Free the task */
        adk_pktio_task_buffer_free((adk_pktio_task_t *)(task_p));
    }
    return ret;
}

/*!
 * =============================================================
 * adk_ipv6_dp_pkt_proc_uli()
 *
 *
 * Structure of task params :
 *
 * taskParams[0]  Reserved set to 0.
 * 
 * taskParams[1]  Flags
 *                :1[0]  1 -> ip header present in pdu
 *                :1[1]  1 -> is destination id/encap header info passed 
 *                              from higher layers
 *                :1[2]  0 -> Use destination flow id for header
 *                            encapsulation.
 *                       1 -> Use encap header info  for header
 *                            encapsulation.
 *                :1[3]  1 -> Priority bits are valid
 *                :30[2] Not used.       
 * 
 * taskParams[2]  Valid only if IP header to be added.
 *                :8[0]  IP ToS   - IPv6 Header Traffic Class
 *                          (used only when header ipv6 header encapsulation required).
 *                :8[24] Hoplimit - IPv6 Header Hoplimit
 *                          (if set to zero, default HOPLIMIT value used)
 *                :16[16] - The MTU for this PDU. If there is no MTU it should be
 *                          set to 0.
 * taskParams[6]  Next task buffer if any. Else set to 0.
 * 
 * taskParams[7]  Pdu buffer pointing to ip header if ip header present
 *                Else pointing to UDP payload (Eg GTP-u).
 *
 * taskParams[10] Destination flow ID.
 *                or pointer to encap info
 *
 *
 * Brief:
 * This function is an interface to upper layer modules and performs
 * following:
 *
 * a. Route Lookup.
 * b. Egress ACL lookup.
 * c. Fragmentation of MTU exceeded packets.
 *
 * This function binds to following modules depending on packet processing:
 * 1. Binds to Ethernet ULI - adk_eth_dp_pkt_proc_uli for normal IPv6
 * forwarding (TN->WIFI, WIFI->TN).
 *
 ==========================================================================*/

adk_dp_status_t
adk_ipv6_dp_pkt_proc_uli(void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    struct rte_mbuf *task, *next_task;
    uint32_t i, cur_bit;
    uint8_t path = 0, pktOrigin = 0;
    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_uli_imc_data_t *adk_ipv6_uli_md_p = NULL;
    uint16_t destFunc = 0;
  
    /* Get the pipeline info */
    for(i=0, cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(!(task_mask & cur_bit))
            continue;

        task = task_p[i];
        adk_md_p =  ADK_METADATA_PTR(task);
        adk_ipv6_uli_md_p = (adk_ipv6_dp_uli_imc_data_t *) &adk_md_p->imc_param_start;
        path = adk_ipv6_uli_md_p->sender;
        pktOrigin = adk_ipv6_uli_md_p->pktOrigin;

        if (path != IP6_ULI_SENDER_IPSEC)
        {
            (ADK_METADATA_PTR(task)->next_frag) = NULL;
        }

        while(task)
        {
            adk_md_p =  ADK_METADATA_PTR(task);
            next_task = adk_md_p->next_frag;

            if (likely(pktOrigin == IP6_ULI_SENDER_EO))
            {
                if (ADK_E_FLOW_CACHE_PKT_FAST_PATH == (ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreIndex))->pktPath) 
                {
                    ret = adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass2(task, coreIndex, &destFunc);
                }
                else
                {
                    ret = adk_ipv6_dp_pkt_proc_uli_single(task, coreIndex, &destFunc);
                }
            }
            else
            {
                ret = adk_ipv6_dp_pkt_proc_uli_slow_path(task, coreIndex, &destFunc);
            }

            if(likely(ret == ADK_SUCCESS))
            {
                task=next_task;
            }
            else
            {
                if(ADK_E_TASK_CONSUMED == ret)
                {
                    ADK_IPV6_DP_LOG_DEBUG("packet consumed \n");
                    task=next_task;
                }
                else
                {
                    ADK_IPV6_DP_LOG_DEBUG("packet dropped \n");
                    adk_pal_dp_release_send(task, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
                    IPV6_DP_DROP_TASK(task);
                    task=next_task;
                    return ret;
                }
            }
        }

		if (adkDestEntryFunction_g[destFunc])
		{
			ret = adk_pal_dp_stage_send((void **)task_p, task_count, task_mask, destFunc, 0x00,
					0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_MAX, NULL);
			if(ret == ADK_E_INTERNAL_ERROR)
			{
                adk_pal_dp_release_send((adk_pktio_task_t *)task_p[0], ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL);
				adk_pktio_task_buffer_free((adk_pktio_task_t *)(task_p[0]));
				ret = ADK_DP_E_ERROR;
				return ret;
			}
		}
	}
    return ADK_SUCCESS;
}

 adk_status_t adk_ipv6_dp_pkt_proc_em_uli(void ** task)
 {
    ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_EXTRA_TS();
    rte_prefetch0(rte_pktmbuf_mtod_offset((adk_pktio_task_t *)(*task), uint8_t *, 0));
    rte_prefetch0(rte_pktmbuf_mtod_offset((adk_pktio_task_t *)(*task), uint8_t *, 64)); 
    int coreIndex = ADK_LCORE_INDEX;
    adk_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t *task_out;
    adk_metadata_t *adk_md_p;
    task_out = (adk_pktio_task_t *)(*task);
    adk_md_p = ADK_METADATA_PTR(task_out);

    ADK_MD_GET_TS_MSG_TYPE(task_out) = ADK_NETD_TS_NONE;
    ADK_METADATA_PARAM(task_out, linux_intf_id) = 0;
    ADK_METADATA_PARAM(task_out, ts_msg_type) = 0;
    ADK_METADATA_PARAM(task_out, ts_correlator) = 0;
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID; 
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID; 
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id = ADK_SYS_INVALID_MDATA_ID;
    adk_md_p->innerIPVersion = 0;
    adk_md_p->ip_offset = 0;
    adk_md_p->saID = ADK_IPSEC_INVALID_SA_ID;

   
    adk_ipv6_dp_uli_imc_data_t *ipv6_imc_data_p = (adk_ipv6_dp_uli_imc_data_t *)&(adk_md_p->imc_param_start);  
    memset(ipv6_imc_data_p, 0, sizeof(adk_ipv6_dp_uli_imc_data_t));
    ipv6_imc_data_p->sender = IP6_ULI_SENDER_EO;
    ipv6_imc_data_p->pktOrigin = IP6_ULI_SENDER_EO;
    /*If origin=eo and per-packet-flag=true -> generate udp checksum*/
    if(ADK_EM_UDP_GEN_CONFIG_GET(task_out))
    {
        task_out->ol_flags |= PKT_TX_UDP_CKSUM;
    }
    else  /*Explicitly disabling CKSUM HW offload */
    {
        task_out->ol_flags &= ~PKT_TX_UDP_CKSUM;
    }
    task_out->ol_flags |= PKT_TX_IPV6;
    ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_EXTRA_TS();
    ret = adk_ipv6_dp_pkt_proc_uli(task, 0, 1, NULL, NULL, coreIndex); 
    if(unlikely((ADK_E_INTERNAL_ERROR == ret)
                         ||(ADK_E_DROP_TASK == ret)))
    {
        /* Free the task */
        adk_pktio_task_buffer_free((adk_pktio_task_t *)(task));
    }
    return ret;
 }

void adk_ipv6_dp_egress_stats(struct rte_mbuf *pkt, void *data) 
{
    adk_metadata_t * adk_md_p = ADK_METADATA_PTR(pkt);
    adk_ipv6_dp_if_stats_entry_t  *if_stats = NULL;
    adk_ipv6_dp_ip_addr_stats_t  *inner_ipv6_addr_stats = NULL;
    adk_ipv6_dp_ip_addr_stats_t  *outer_ipv6_addr_stats = NULL;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    uint16_t egress_stats_id=0, outer_addr_stats_id=0, inner_addr_stats_id=0;
    adk_egress_user_param_t *params = (adk_egress_user_param_t *)data;
    bool isPktDropped;

    /*E N Q U E U E   S T A T S*/
    if (TRUE != params->xmit_cb)
    {
        isPktDropped = (rte_pktmbuf_tx_enqueue_status(pkt) == RTE_MBUF_TX_ENQUEUE_SUCCESS) ? false:true;

        ADK_IPV6_DP_LOG_DEBUG("### data cb: %u stats_ID %u\n", params->xmit_cb, params->stats_mask);

        egress_stats_id = adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id;
        inner_addr_stats_id = adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id;
        outer_addr_stats_id = adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id;

        if (params->stats_mask & ADK_STATS_IP_INTERFACE_MASK)
        {
            if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][egress_stats_id];
            if(NULL == if_stats)
            {
                ADK_IPV6_DP_LOG_ERROR("Invalid Iface Stats ID \n");
                return ;
            }
            if (!isPktDropped){
                if (adk_md_p->isIpMcastPkt) { 
                    if_stats->ipv6IfStatsOutMcastPkts++;
                    if_stats->ipv6IfStatsOutMcastOctets += adk_md_p->outerIpPktLen;
                }else{
                    if_stats->ipv6IfStatsOutUcastPkts++;
                    if_stats->ipv6IfStatsHCOutOctets += adk_md_p->outerIpPktLen;
                }
            }else{
                if_stats->ipv6IfStatsOutDiscards++;
                if_stats->ipv6IfStatsOutDroppedOctets += adk_md_p->outerIpPktLen;
            }
            
        }
        if (params->stats_mask & ADK_STATS_INNER_IP_ADDRESS_MASK)
        {
            inner_ipv6_addr_stats = &(adk_ipv6_dp_tables->ipv6AddrStatsTble[coreIndex][inner_addr_stats_id]);
            if(NULL == inner_ipv6_addr_stats)
            {
                ADK_IPV6_DP_LOG_ERROR("Invalid inner IP address Stats ID \n");
                return ;
            }
            if (!isPktDropped) {
                inner_ipv6_addr_stats->ipv6OutPackets++;
                inner_ipv6_addr_stats->ipv6OutOctets += adk_md_p->innerIpPktLen;
            }
        }
        if (params->stats_mask & ADK_STATS_OUTER_IP_ADDRESS_MASK)
        {
            outer_ipv6_addr_stats = &(adk_ipv6_dp_tables->ipv6AddrStatsTble[coreIndex][outer_addr_stats_id]);
            if(NULL == outer_ipv6_addr_stats)
            {
                ADK_IPV6_DP_LOG_ERROR("Invalid outer IP address Stats ID \n");
                return ;
            }
            if (!isPktDropped) {
                outer_ipv6_addr_stats->ipv6OutPackets++;
                outer_ipv6_addr_stats->ipv6OutOctets += adk_md_p->outerIpPktLen;
            }
        }
    }
    /*T R A N S M I T   S T A T S*/
    else
    {
        if(rte_pktmbuf_tx_cmpltn_status(pkt) == RTE_MBUF_TX_CMPLTN_SUCCESS)
        {
            /*Transmit queue might be removed later. Hence incrementing the 
             *TX packets/octets only in the Enqueue successfull case
             */
            ADK_IPV6_DP_LOG_DEBUG("### Xmit Success\n");

        }
        else
        {
            ADK_IPV6_DP_LOG_DEBUG("### Xmit Drop\n");
            if(if_stats)
            {
                if_stats->ipv6IfStatsOutDiscards++;
                if_stats->ipv6IfStatsOutDroppedOctets += adk_md_p->outerIpPktLen;
            }
            //No address drop counter support yet
        } 
    }
    return;
}

static adk_status_t  adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass1(struct rte_mbuf *task, ipv6_l4_info *l4_info,
                                 uint16_t *destFunc)
{
    adk_status_t ret = ADK_SUCCESS;
    ADK_IPV6_DP_LOG_DEBUG("IPv6_ULI_FC entry FASTPATH pass1 ..\n");
    adk_ipv6_dp_uli_imc_data_t *ipv6uliImc = (adk_ipv6_dp_uli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(task));
    if (l4_info)
    {
        set_ipseculi_params(task, l4_info->next_hdr, l4_info->l4_offset, ipv6uliImc->sender, destFunc);
    }
    return ret;
}

static adk_status_t  adk_ipv6_dp_pkt_proc_uli_flow_cache_pkt_path_single_pass2(struct rte_mbuf *task,
                                 int coreIndex, uint16_t *destFunc)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_ipv6_dp_uli_imc_data_t *ipv6uliImc = (adk_ipv6_dp_uli_imc_data_t *)&(ADK_MD_GET_IMC_PARAM_START(task));
    adk_ipv6_dp_interface_entry_t *ipv6_if_entry = NULL;
    adk_ipv6_dp_nexthop_entry_t *nexthop_entry=NULL;
    adk_ipv6_dp_if_stats_entry_t *if_stats = NULL;
    uint32_t egressIfId, nextHopId;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    adk_ipv6_hdr_t *ipv6_hdr = NULL;
    uint32_t frag_id, len_nh_hlimit=0, mtu = 0, ipv6_len = 0;
    uint16_t src_port = 0, dst_port = 0, offset_to_frag, if_id;
    uint8_t next_hdr = 0, excep, ipv6_dscp = 0, ipv6_pcp = 0, l4_offset=0;
    bool  l4_port_present = FALSE, is_onlink = FALSE;
    adk_ipv6_frag_t frag_flag = ADK_IPV6_UNFRAGMENTED_PKT;
    uint8_t pktOrigin, sender;
    uint8_t total_frags = 1, num_frags = 0;

    pktOrigin = ipv6uliImc->pktOrigin;
    sender = ipv6uliImc->sender;

    ADK_IPV6_DP_LOG_DEBUG("IPv6_ULI_FC entry FASTPATH pass2 ..\n");

    fetch_rx_cycle_count(sender);

    ADK_PDU_OFFSET_PTR_GET(task,
            ipv6_hdr,
            adk_ipv6_hdr_t *,
            adk_md_p->ip_offset);

    len_nh_hlimit = adk_ntohl(ipv6_hdr->len_nh_hlimit);
    ipv6_len = (len_nh_hlimit >>16) + ADK_IPV6_HDR_LEN;

    l4_offset = adk_ipv6_parse_l4_ports((task->pkt_len - adk_md_p->ip_offset), ((len_nh_hlimit>>8)&0xFF),
            (uint8_t *)ipv6_hdr, &src_port, &dst_port, &next_hdr, &frag_flag, &l4_port_present,
            &offset_to_frag, &frag_id, &excep);

    scratch_pad_ipv6 = ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreIndex);

    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id = scratch_pad_ipv6->flow_cache_entry->innerSipStatsId;
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id = scratch_pad_ipv6->flow_cache_entry->outerSipStatsId;

    ADK_IPV6_DP_LOG_DEBUG("v6 innerIpStatsId(%d)=%d\n", ADK_STATS_MDATA_ID_INNER_IP_STATS, adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INNER_IP_STATS].stats_id);
    ADK_IPV6_DP_LOG_DEBUG("v6 outerIpStatsId(%d)=%d\n", ADK_STATS_MDATA_ID_OUTER_IP_STATS, adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_OUTER_IP_STATS].stats_id);


    egressIfId = scratch_pad_ipv6->flow_cache_entry->egressIntfId;
    mtu = scratch_pad_ipv6->flow_cache_entry->routeMtu;
    nextHopId = scratch_pad_ipv6->flow_cache_entry->nextHop;

    adk_md_p->dev_tx_queue_id = scratch_pad_ipv6->flow_cache_entry->dev_tx_queue_id;
    adk_md_p->pktColor = scratch_pad_ipv6->flow_cache_entry->pktColor;
    adk_md_p->qosStatsId = scratch_pad_ipv6->flow_cache_entry->qosStatsId;
    adk_md_p->isPHBQueue = scratch_pad_ipv6->flow_cache_entry->isPHBQueue;
    adk_md_p->isIpPkt = true;

    if ((adk_ntohl(ipv6_hdr->dst_addr[0]) >> 24) == 0xff) {
        adk_md_p->isIpMcastPkt = true;
    }else {
        adk_md_p->isIpMcastPkt = false;
    } 

    if (ADK_IPV6_INVALID_NH_INTF_ID == nextHopId)
    {
        if_id = egressIfId;
        is_onlink = TRUE;
    }
    else
    {
        /*NextHop case*/
        ret = adk_ipv6_dp_nexthop_entry_lookup(nextHopId, &nexthop_entry);
        if(unlikely(ret != ADK_SUCCESS))
        {
            ADK_IPV6_DP_LOG_ERROR("error: nexthop lookup failed for id %d\n", nextHopId);
            ret = ADK_E_DROP_TASK;
            goto flow_cache_uli_single_exit;
        }
        if_id = nexthop_entry->interface_id;
        is_onlink = FALSE;
    }
    ret = adk_ipv6_dp_interface_entry_lookup(if_id, 0, &ipv6_if_entry);
    if(unlikely(ret != ADK_SUCCESS))
    {
        ADK_IPV6_DP_LOG_ERROR("error while retriving interface attributes \n");
        ret = ADK_E_DROP_TASK;
        goto flow_cache_uli_single_exit;
    }

    if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->egress_stats_id];

    if (ADK_QOS_INVALID_QUEUE_ID == adk_md_p->dev_tx_queue_id) {
        ADK_IPV6_DP_LOG_DEBUG("IP ULI: Invalid QoS queue id. Dropping the packet.");
        if_stats->ipv6IfQoSLookupFailed++;
        ret = ADK_E_DROP_TASK;
        goto flow_cache_uli_single_exit;
    }

    /* update the egress stats Id */
    adk_md_p->egress_mdata_stats[ADK_STATS_MDATA_ID_INTERFACE_STATS].stats_id =
    scratch_pad_ipv6->flow_cache_entry->interfaceStatsId;

    adk_md_p->outerIPVersion = ADK_IPV6_PDU_VER_VALUE;
    adk_md_p->outerIpPktLen = ipv6_len;

    ipv6_dscp = (adk_ntohl(ipv6_hdr->ver_tc_flabel) >> 22) & 0x3F;
    ipv6_pcp = ipv6_if_entry->dscp_pcp[ipv6_dscp] & 0x0F;

    /*F R A G M E N T A T I O N*/
    check_for_fragmentation(mtu, ipv6_len, next_hdr, l4_offset, pktOrigin, ipv6_if_entry, coreIndex, task, &total_frags, &ipv6_hdr);

    if(ADK_SYS_IPSEC_BYPASS_GET || (INVALID_FRAG_COUNT == ipv6uliImc->frag_count))
        num_frags = total_frags;
    else
        num_frags = ipv6uliImc->frag_count;

    set_ethuli_params(task, ipv6_dscp, ipv6_pcp, is_onlink, if_id,
            ipv6_hdr, frag_flag, next_hdr, nexthop_entry, num_frags, destFunc);

    flow_cache_uli_single_exit:
    fetch_tx_cycle_count(sender);
    return ret;
}
