/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IPV6_ACL_H__
#define __ADK_IPV6_ACL_H__
/* #include <adk_include.h> */
#include <adk_lpm.h>
#include <time.h>
#include <adk_pmu.h>
#include <adk_pktio.h>




typedef adk_handle_t /*void*/ adk_ipv6_dp_acl_tree_t;
typedef uint32_t adk_ipv6_acl_val_t;

/*
 * IPv6 ACL Table 
 *
 * 5 tuples, dscp & frag flag is used as input DPDK ACL tree. 
 * DPDK input ACL rule struct : adk_ipv6_dp_acl_rule_t
 * Fastpath lookup struct : adk_ipv6_dp_acl_ptn_t
 * ACL output struct : adk_ipv6_dp_acl_action_t    
 * 
 * Index shall be returned from ACL lookup & 
 * it is used to get the action for the lookup
 */

#define IPV6_ACL_MAX_RULES_PER_SET  66
#define IPV6_ACL_MAX_NUM_RULES      256
#define IPV6_ACL_LKUP_MAX_NUM_PKT   8

#define ADK_IPV6_MAX_FILTER_RULE_PRIORITY 0xFFFF

typedef struct _adk_ipv6_dp_acl_rule_s
{
    uint32_t    sip_addr[4];
    uint8_t     sip_mask;
    uint32_t    dip_addr[4];
    uint8_t     dip_mask;
    uint16_t    sport_begin;
    uint16_t    sport_end;
    uint16_t    dport_begin;
    uint16_t    dport_end;
    uint8_t     proto_begin;
    uint8_t     proto_end;
    uint16_t    dscp_begin;
    uint16_t    dscp_end;
    uint16_t    frag_type_begin;
    uint16_t    frag_type_end;
} adk_ipv6_dp_acl_rule_t;

typedef struct __attribute__((__packed__))  _adk_ipv6_dp_acl_ptn_s
{
    uint8_t proto;
    uint32_t    sip_addr[4];
    uint32_t    dip_addr[4];
    uint16_t    srcPort;
    uint16_t    destPort;
    uint16_t    dscp;
    uint16_t    frag_type;
}adk_ipv6_dp_acl_ptn_t;


typedef struct adk_ipv6_dp_acl_action_entry_s
{
    uint8_t     is_enabled:1;
    uint8_t     action:5;
    uint8_t     enable_counter:1;
    uint8_t     enable_policing:1;
    uint8_t     callback_fun_id;
    uint16_t    acl_stats_id;
    adk_ph_pipeline_bind_handle_t    ph_bind_handle;
    uint32_t    qid;
    uint32_t    policing_flow_id;
    bool        cpu_path;

    /* add rule specific stats required here */
    uint8_t     devId;
    uint16_t    queueId;
}adk_ipv6_dp_acl_action_entry_t;

adk_ipv6_dp_acl_tree_t *adk_ipv6_acl_create(adk_module_handle_t ipv6Handle,
                            uint16_t ruleSetId, uint8_t index);

adk_status_t adk_ipv6_acl_ruleset_free(adk_module_handle_t ipv6Handle,
                            adk_ipv6_dp_acl_tree_t *acl_tree);

adk_status_t adk_ipv6_acl_rule_add(adk_module_handle_t ipv6Handle,
            adk_ipv6_dp_acl_tree_t *acl_tree,
		 	adk_ipv6_dp_acl_rule_t *acl_rule,
		 	adk_ipv6_acl_val_t  acl_val,
		 	uint32_t index);

adk_status_t adk_ipv6_acl_lookup(adk_ipv6_dp_acl_tree_t *acl_tree,
		      adk_ipv6_dp_acl_ptn_t  *acl_ptn,
		      adk_ipv6_acl_val_t  *acl_val);

adk_status_t adk_ipv6_acl_ruleset_commit(adk_module_handle_t ipv6Handle,
            adk_ipv6_dp_acl_tree_t *acl_tree);

void adk_ipv6_acl_ruleset_clear(adk_module_handle_t ipv6Handle,
            adk_ipv6_dp_acl_tree_t *acl_tree);

void adk_print_ipv6_acl_list(adk_module_handle_t ipv6Handle,
            adk_ipv6_dp_acl_tree_t *acl_tree);

#endif

