/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_ipv6_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief   This file has all the macros and data types which are shared by
 *          control plane and data plane software.
 *
 *  **********************************************************************/

#ifndef _ADK_IPV6_DP_H_
#define _ADK_IPV6_DP_H_


#ifdef __cplusplus
extern "C" {
#endif

#include "adk_include.h"
#include "adk_sys_dpdk.h"
#include "adk_hos.h"
#include "adk_ph_dev.h"
#include <adk_ipv6.h>
#include <adk_ipv6_acl.h>
#include <adk_ipv6_ing_classifier.h>
#include "adk_ipv6_utility_counters.h"
#include "adk_cp_acl_list.h"
#include "adk_rcu_distributed_util.h"
//#include <adk_ipv6_dp_export.h>
#include "adk_lpm.h"
#include <time.h>
#include "adk_hash.h"
#include <adk_pmu.h>
#include <adk_pktio.h>
#include <adk_timer.h>
#include <adk_ipv6_dp_export.h>
#include <adk_ipv6_qos_classifier.h>

#define IPV6_HDR_DSCP_MASK                      0x0FC00000
#define IPV6_HDR_DSCP_LOC                       22
#define ADK_IPV6_DP_REASM_SESSION_POOL_TBL_NAME "adk_ipv6_reas_tbl"
#define ADK_IPV6_DP_REASM_SESSION_POOL_CACHE_SIZE 16
#define ADK_IPV6_REASSEMBLY_MAX_SESSION 2048 + (ADK_MAX_NUM_CORES \
                                *ADK_IPV6_DP_REASM_SESSION_POOL_CACHE_SIZE)
#define ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS 1024
#undef ADK_UTILITY_COUNTER_ENABLE
/*! Maximum number of IP addresses than can be set for an IP interface */
#define ADK_IPV6_INTF_ADDR_MAX                    201

/*! The maximum number of IPv6 interfaces over Ethernet. This includes
 * interfaces over PHY ports, VLAN sub-interfaces ,Ethernet SVIF (BVI) & VIP*/
#define ADK_IPV6_NUM_IPOE_INTERFACES_MAX             (128+600)
/*! Maximum number of IP tunnel interface supported by IPv6 module */
//#define ADK_IPV6_TUNNEL_INTF_MAX                  728
#define ADK_DP_MAX_IPV6_INTERFACE (ADK_IPV6_NUM_IPOE_INTERFACES_MAX + ADK_IPV6_TUNNEL_INTF_MAX)
#define ADK_DP_MAX_IPV6_STATS ADK_DP_MAX_IPV6_INTERFACE
/********************************************************************************
 *                  IPv6 Shared memory details     
 ********************************************************************************/
#define ADK_IPV6_SHM_MEMZONE_NAME "adk_ipv6_shm_zone"

/********************************************************************************
 *                  IPv6 FLOW cache DSCP lookup details 
 ********************************************************************************/
#define IPV6_MAX_DSCP_VALUES                64
#define IPV6_MAX_DSCP_IDS                   22
#define IPV6_INVALID_DSCP_ID                21 /*0 - 20 valid ids*/
#define IPV6_DSCP_LKUP_ACL_STATSID_WIDTH    10
#define IPV6_DSCP_LKUP_ENABLE_MASK          0x8000
#define IPV6_DSCP_LKUP_PCP_MASK             0x7000
#define IPV6_DSCP_LKUP_ACL_STATSID_MASK     0x03FF
#define IPV6_DSCP_LKUP_ENABLE_POS           15
#define IPV6_DSCP_LKUP_PCP_POS              12
#define IPV6_DSCP_LKUP_ACL_STATSID_POS      0
#define IPV6_TUNNEL_IF_ID_MIN               512

#define ADK_CONVERT_TO_LPM_DATA(__DATA) ((adk_handle_t)(uintptr_t)__DATA)

#define ADK_IPV6_TUNNEL_ID_NETD_TO_ADK(netd_id) \
    (netd_id - IPV6_TUNNEL_IF_ID_MIN + ADK_IPV6_NO_OF_IPV6_INTERFACES)
#define ADK_IPV6_TUNNEL_ID_ADK_TO_NETD(adk_id) \
    (adk_id - ADK_IPV6_NO_OF_IPV6_INTERFACES + IPV6_TUNNEL_IF_ID_MIN)

#define ADK_IPV6_DP_TABLE_ALLOC(tbl, entry_type, entry_size, num_entries)\
    do{ \
        tbl =  entry_type ADK_DP_SHM_CALLOC(ADK_IPV6_MODULE_ID, ADK_IPV6_DP_TBL_OBJ, num_entries, entry_size, 0); \
        if(tbl == NULL) \
        { \
            ADK_IPV6_DP_LOG_INFO_1S( "error while allocating"  \
                    " IPV6 DP table %s \n", #tbl); \
            return ADK_DP_E_ERROR; \
        } \
    }while(0);

#define ADK_IPV6_DP_TABLE_FREE(tbl)\
    do{ \
        ADK_DP_SHM_FREE(tbl); \
    }while(0);

#define ADK_IPV6_DP_LPM_TABLE_ALLOC(lpm_tbl, tree_width)\
    do{ \
        lpm_tbl =  adk_lpm_create(ADK_IPV6_MODULE_ID, ADK_IPV6_DP_LPM_TBL_OBJ, tree_width,\
                                  ADK_LPM_DEFAULT_PENDING_SIZE);\
        if(!lpm_tbl) \
        { \
            ADK_IPV6_DP_LOG_INFO_1S( "error while allocating"  \
                     " IPV6 DP LPM table %s \n", #lpm_tbl); \
            return ADK_DP_E_ERROR; \
        } \
    }while(0);

#define ADK_IPV6_DP_LPM_TABLE_FREE(lpm_handle)\
    do{ \
        adk_lpm_free(lpm_handle);\
    }while(0);

#define ADK_IPV6_DP_TABLE_IDX_INIT(dp_tbl, num_entries) \
    do { \
        uint16_t ne; \
        for(ne=0; ne<(num_entries-1); ne++) \
        { \
            dp_tbl.tbl[ne].idx = ne; \
            dp_tbl.tbl[ne].next_idx = ne+1; \
        } \
        dp_tbl.tbl[ne].idx = ne; \
        dp_tbl.tbl[ne].next_idx = UINT16_MAX; \
        dp_tbl.free_list.head_idx = 0; \
        dp_tbl.free_list.tail_idx = num_entries-1; \
        dp_tbl.free_list.count = num_entries; \
        dp_tbl.pending_list.head_idx = UINT16_MAX; \
        dp_tbl.pending_list.tail_idx = UINT16_MAX; \
        dp_tbl.pending_list.count = 0; \
    }while(0);

#define ADK_IPV6_DP_FREE_IDX(list, tbl, index) \
    do{ \
        if(!(list).count) \
        { \
            (list).head_idx = index; \
            (list).tail_idx = index; \
        } \
        else \
        { \
            tbl[(list).tail_idx].next_idx = tbl[(index)].idx; \
            (list).tail_idx = index; \
        } \
        tbl[(index)].next_idx = UINT16_MAX; \
        (list).count++; \
    }while(0);



#define ADK_IPV6_DP_RSRV_IDX(free_list, tbl, index) \
        do{ \
            if((free_list).count) \
            { \
                (index) = (free_list).head_idx; \
                (free_list).head_idx = tbl[(index)].next_idx; \
                (free_list).count--; \
            } \
            else \
            { \
                ADK_IPV6_DP_LOG_INFO_DEBUG( "Error: no free resources \n"); \
                return ADK_DP_E_ERROR; \
            } \
        }while(0);

#define ADK_IPV6_DP_FC_RSRV_IDX(free_list, tbl, index) \
        ADK_IPV6_DP_RSRV_IDX(free_list, tbl, index)


#define ADK_IPV6_DP_FC_RELEASE_IDX(free_list, tbl, index)\
        ADK_IPV6_DP_FREE_IDX(free_list, tbl, index)\


#define ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK(dp_tbl)\
    do{\
        if(((dp_tbl)->free_list.count==0) && ((dp_tbl)->pending_list.count!=0)) \
        { \
            adk_rcu_distributed_wait(); \
            ADK_IPV6_DP_CLEANUP_TBL((dp_tbl)->pending_list, \
                    (dp_tbl)->free_list, (dp_tbl)->tbl); \
        } \
        else if(adk_rcu_distributed_ver_verify() == ADK_SUCCESS) \
        { \
            ADK_IPV6_DP_CLEANUP_TBL((dp_tbl)->pending_list, \
                    (dp_tbl)->free_list, (dp_tbl)->tbl); \
        }\
    }while(0)

#define ADK_IPV6_DP_CLEANUP_OVERFLOW_TBL(dp_tbl) \
    do{ \
        adk_ipv6_dp_overflow_node_t *node, *next_node; \
        node = dp_tbl->overflow_pending_list.head; \
        while(node) \
        { \
            next_node = node->next; \
            ADK_DP_SHM_FREE(node); \
            node = next_node; \
        }\
        dp_tbl->overflow_pending_list.count = 0; \
        dp_tbl->overflow_pending_list.head = NULL; \
    }while(0)

#define ADK_IPV6_DP_RCU_DISTRIBUTED_CHECK_OVERFLOW_TBL(dp_tbl)\
    do{\
        if(((dp_tbl)->free_list.count==0) && ((dp_tbl)->overflow_pending_list.count!=0)) \
        { \
            adk_rcu_distributed_wait(); \
            ADK_IPV6_DP_CLEANUP_OVERFLOW_TBL(dp_tbl); \
        } \
        else if(adk_rcu_distributed_ver_verify() == ADK_SUCCESS) \
        { \
            ADK_IPV6_DP_CLEANUP_OVERFLOW_TBL(dp_tbl); \
        }\
    }while(0)


#define ADK_IPV6_DP_CLEANUP_TBL(pending_list, free_list, tbl)    \
    do{\
        if(pending_list.count) \
        {\
            if(!(free_list).count) \
            { \
                (free_list).head_idx = pending_list.head_idx; \
                (free_list).tail_idx = pending_list.tail_idx; \
            } \
            else \
            { \
                tbl[(free_list).tail_idx].next_idx = tbl[pending_list.head_idx].idx; \
                (free_list).tail_idx = pending_list.tail_idx; \
            } \
            (free_list).count += pending_list.count; \
            pending_list.count = 0; \
        }\
    }while(0);



#define ADK_IPV6_DP_RELEASE_IDX(pending_list, free_list, tbl, index) \
    do{ \
        ADK_IPV6_DP_FREE_IDX(pending_list, tbl, index) \
        adk_rcu_synchronize(); \
        ADK_IPV6_DP_CLEANUP_TBL(pending_list, free_list, tbl) \
    }while(0);

#define ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(pending_list, free_list, tbl, index) \
    do{ \
        ADK_IPV6_DP_FREE_IDX(pending_list, tbl, index) \
        adk_rcu_distributed_ver_update(); \
    }while(0);

#define ADK_IPV6_DP_ADD_OVERFLOW_ENTRY(overflow_list, \
        free_list, tbl, tbl_index)\
    do{ \
        adk_ipv6_dp_overflow_node_t *node = ADK_DP_SHM_CALLOC(ADK_IPV6_MODULE_ID, ADK_IPV6_OFL_ADD_OBJ, 1, sizeof(adk_ipv6_dp_overflow_node_t), 0); \
        if(node == NULL) \
        { \
            printf("unable to allocate memory \n"); \
            /* release the allocated index*/ \
            /* need not to wait for RCU, becuase this entry is not visible to fast-path yet */ \
            ADK_IPV6_DP_FREE_IDX(free_list, tbl, tbl_index); \
            return(ADK_DP_E_ERROR); \
        } \
        node->idx = tbl_index; \
        node->next = NULL; \
        if(overflow_list.head) \
        { \
            node->next = overflow_list.head; \
            overflow_list.head = node; \
        } \
        else \
        { \
            overflow_list.head = node; \
        } \
        overflow_list.count++; \
        /* update the dp tables version */ \
        ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++; \
    }while (0)

#define ADK_IPV6_DP_RELEASE_OVERFLOW_ENTRY(of_list, of_pending_list, \
        pending_list, free_list, tbl, tbl_index) \
    do{\
        /* check for ovrflow list */ \
        adk_ipv6_dp_overflow_node_t *node, *prev=NULL; \
        node = of_list.head; \
        while(node) \
        { \
            if( node->idx == tbl_index) \
            { \
                /* detatch ll-node from overflow_list */ \
                if(node == of_list.head) \
                { \
                    of_list.head = node->next; \
                } \
                else \
                { \
                    prev->next = node->next; \
                } \
                of_list.count--; \
                /* add ll-node to overflow pending list */ \
                node->next = of_pending_list.head; \
                of_pending_list.head = node; \
                of_pending_list.count++; \
                /* release the tbl_idx to pending list*/ \
                ADK_IPV6_DP_RELEASE_DISTRIBUTED_IDX(pending_list, \
                        free_list, \
                        tbl, \
                        tbl_index); \
                /* update the dp tables version */ \
                ipv6ModuleInfo_p->adk_ipv6_dp_tables->ipv6_fc_ver[vrd_id]++; \
                return ADK_SUCCESS; \
            } \
            prev = node; \
            node = node->next; \
        } \
    }while(0)

#define ADK_IPV6_DP_RELEASE_LPM_IDX(free_list, tbl, index) \
    ADK_IPV6_DP_FREE_IDX(free_list, tbl, index)

#define ADK_IPV6_DP_LPM_ADD(lpm_tbl, input_ptn, size, lpm_value) \
    do{ \
        ADK_LOG_INFO(sys_handle, ipv6Handle,"\n"); \
        /* update the shadow copy */ \
        if(adk_lpm_add(ADK_MODULE_MODULEID_GET(ipv6Handle), ADK_IPV6_LPM_ADD_OBJ,\
		    lpm_tbl.lpm,  \
                    input_ptn, \
                    size, \
                    lpm_value)) \
        { \
/*            ADK_IPV6_DP_RELEASE_LPM_IDX(free_list, tbl, lpm_value); */\
            ADK_LOG_ERROR(sys_handle, ipv6Handle,\
            "Error while adding LPM entry  %s \n", #lpm_tbl); \
            return ADK_DP_E_ERROR; \
        } \
    }while(0);

#define ADK_IPV6_CP_LPM_LOOKUP(lpm_tbl, input_ptn, size, index)\
    do{ \
        int lpm_ret;\
        lpm_ret = adk_lpm_is_rule_present(lpm_tbl.lpm, input_ptn, size, &(index)); \
        if(lpm_ret) \
        { \
            if(index == ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID)) \
            {\
                return ADK_DP_E_ERROR; \
            } \
        } \
        else \
        { \
            ADK_LOG_ERROR(sys_handle, ipv6Handle,\
            "Error while exact LPM entry lookup %s \n", #lpm_tbl); \
            return ADK_DP_E_ERROR; \
        } \
    }while(0);



#define ADK_IPV6_DP_LPM_LOOKUP(lpm_tbl, input_ptn, index)\
        do{ \
            index = adk_lpm_lookup(lpm_tbl.lpm, input_ptn); \
            if(index == ADK_CONVERT_TO_LPM_DATA(ADK_LPM_VALUE_INVALID)) \
            {\
                return ADK_DP_E_ERROR; \
            } \
        }while(0);

#define ADK_IPV6_DP_LPM_REM(lpm_tbl, input_ptn, size) \
        do{ \
            ADK_LOG_INFO(sys_handle, ipv6Handle,"\n"); \
            /* update the shadow copy */ \
            if(adk_lpm_delete(lpm_tbl.lpm,  \
                        input_ptn, \
                        size)) \
            { \
                ADK_LOG_ERROR(sys_handle, ipv6Handle,\
                "Error while removing LPM entry  %s \n", #lpm_tbl); \
                return ADK_DP_E_ERROR; \
            } \
        }while(0);

#define ADK_IPV6_GET_EXT_HDR_LEN(ext_hdr_len) \
    ((ext_hdr_len << 3) + 8/*initial 8 octets*/);

    /* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
    extern uint64_t ipv6_lli_rx_p1_ts, ipv6_lli_tx_p1_ts;
    extern uint64_t ipv6_lli_rx_p2_ts, ipv6_lli_tx_p2_ts;
    extern uint64_t ipv6_uli_rx_p1_ts, ipv6_uli_tx_p1_ts;
    extern uint64_t ipv6_uli_rx_p2_ts, ipv6_uli_tx_p2_ts;
    extern uint64_t ipv6_uli_rx_ex_ts, ipv6_uli_tx_ex_ts;

#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS1_TS() \
    adk_dp_measure_block_cycle_rx_ts(&ipv6_lli_rx_p1_ts)
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS1_TS() \
    adk_dp_measure_block_cycle_tx_ts(&ipv6_lli_rx_p1_ts, \
            &ipv6_lli_tx_p1_ts, "IPV6 LLI PASS1 MODULE", PMU_IPV6_LLI_PASS1_MODULE)

#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS2_TS() \
    adk_dp_measure_block_cycle_rx_ts(&ipv6_lli_rx_p2_ts)
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS2_TS() \
    adk_dp_measure_block_cycle_tx_ts(&ipv6_lli_rx_p2_ts, \
            &ipv6_lli_tx_p2_ts, "IPV6 LLI PASS2 MODULE", PMU_IPV6_LLI_PASS2_MODULE)


#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS1_TS() \
    adk_dp_measure_block_cycle_rx_ts(&ipv6_uli_rx_p1_ts)
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS1_TS() \
    adk_dp_measure_block_cycle_tx_ts(&ipv6_uli_rx_p1_ts, &ipv6_uli_tx_p1_ts, "IPV6 ULI PASS1 MODULE", \
            PMU_IPV6_ULI_PASS1_MODULE)

#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS2_TS() \
    adk_dp_measure_block_cycle_rx_ts(&ipv6_uli_rx_p2_ts)
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS2_TS() \
    adk_dp_measure_block_cycle_tx_ts(&ipv6_uli_rx_p2_ts, &ipv6_uli_tx_p2_ts, "IPV6 ULI PASS2 MODULE", \
            PMU_IPV6_ULI_PASS2_MODULE)

#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_EXTRA_TS()\
        adk_dp_measure_block_cycle_rx_ts(&ipv6_uli_rx_ex_ts)
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_EXTRA_TS()\
        adk_dp_measure_block_cycle_tx_ts(&ipv6_uli_rx_ex_ts, &ipv6_uli_tx_ex_ts, "IPV6 ULI EXTRA MODULE", \
            PMU_IPV6_ULI_EXTRA_MODULE)

#else
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS1_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS1_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_RX_PASS2_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS2_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS1_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS1_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_PASS2_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_PASS2_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_RX_EXTRA_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_ULI_TX_EXTRA_TS()
#endif


/********************************************************************************
 *                  IPv6 Fast path table structures                             *
 ********************************************************************************/
#define ADK_IPV6_LPM_SHADOW_COPIES  2
typedef struct adk_ipv6_ph_params_s
{
    uint8_t exception;
    uint8_t vrd_id;
    uint16_t param_problem_offset;
    uint16_t mtu;
}adk_ipv6_ph_params_t;

typedef struct adk_ipv6_dp_array_list_s
{
    uint16_t    head_idx;
    uint16_t    tail_idx;
    uint32_t    count;
}adk_ipv6_dp_array_list_t;


typedef struct adk_ipv6_dp_lpm_table_s
{
    adk_lpm_handle_t    lpm;
}adk_ipv6_dp_lpm_table_t;


/* ingress & egress interface table entry */

typedef struct adk_ipv6_dp_pkt_capture_info_s{
    uint8_t  is_enabled:1;
    uint8_t  enqueue_status:1;  /* set false if enqueue fails */
    uint8_t  reserved:6;

    /* Ph Callback Function Details 
     * Ring info */
    uint8_t devId;
    uint8_t cbFnId;
    uint16_t queueId;

    /* Packet capture instance */
    uint32_t context;
}adk_ipv6_dp_pkt_capture_info_t;

typedef struct adk_ipv6_dp_ip_addr_stats_s
{
    uint64_t ipv6InOctets;
    uint64_t ipv6InPackets;
    uint64_t ipv6OutOctets;
    uint64_t ipv6OutPackets;
    uint64_t ipv6InReassembledPackets;
    uint32_t ipv6InErrors;
}__attribute__((aligned(64))) adk_ipv6_dp_ip_addr_stats_t;


typedef struct adk_ipv6_dp_interface_info_s{
    uint16_t    idx;
    uint16_t    next_idx;
    uint8_t     is_disabled:1;
    uint8_t     enable_policing:1;
    uint8_t     enable_pbr:1;
    uint8_t     enable_ingress_acl:1;
    uint8_t     enable_egress_acl:1;
    uint8_t     enable_remark:1;
    uint8_t     enable_qos:1;
    uint8_t     enable_reassembly:1;
    uint8_t     vrd_id;
    uint16_t    interface_id;   /* interface id or tunnel interface id */
    uint16_t    nexthop_id;

    uint8_t     outer_vrd_id;   /* valid for tunnel interface type */
    uint8_t     dscp_profile_id;
    uint16_t    police_flow_id;

    adk_ipv6_addr_prefix_t      ipv6_addr_prefix;
    adk_ipv6_interface_type_t   if_type;

    uint16_t    ingress_stats_id;
    uint16_t    egress_stats_id;

    uint16_t    ipv6_addr_stats_id; /* For ipv6 primary address */

    adk_ipv6_addr_prefix_t    ipv6_prefix[ADK_IPV6_INTF_ADDR_MAX];
    uint16_t    ipv6_prefix_stats_id[ADK_IPV6_INTF_ADDR_MAX];
    adk_ipv6_dp_ip_addr_stats_t  *ipv6_addr_stats;

    uint16_t    ingress_assoc_mask;
    uint16_t    reserved;

    uint32_t    sys_egress_stats_id;

    /* do not change the mtu declaration 
     * optimised to set both of them Atomically */
    uint16_t    pmtu;
    uint16_t    mtu;

    uint8_t     la_flag;
    uint8_t     la_flag_version;
    uint16_t    ingress_ruleset;
    uint16_t    egress_ruleset;
    uint16_t    pbr_policy_set_id;

    adk_ipv6_dp_pkt_capture_info_t ing_pkt_capture_info;
    adk_ipv6_dp_pkt_capture_info_t eg_pkt_capture_info;
    uint8_t     interfaceAttr;
    uint8_t     dscp_pcp[/*IPV6_NUM_DSCP_VALUES*/64];
    adk_qos_ds_phb_class_t dscpToPhbClass[IPV6_MAX_DSCP_VALUES]; 
}__attribute__ ((aligned(64)))adk_ipv6_dp_interface_info_t;

typedef adk_ipv6_dp_interface_info_t adk_ipv6_dp_interface_entry_t;


/*!
 * ================================================================
 * @struct adk_ipv6_dp_address_stats_t adk_ipv6_dp.h "adk_ipv6_dp.h"
 *
 * @brief
 *
 * Structure for maintaining IPv6 address stats
 * ================================================================
 */
typedef struct adk_ipv6_dp_address_stats_s {

    /*! total number of received octets */
    uint64_t ipv6InOctets;

    /*! total number of received packets */
    uint64_t ipv6InPackets;

    /*! total number of transmitted octets */
    uint64_t ipv6OutOctets;

    /*! total number of transmitted packets */
    uint64_t ipv6OutPackets;

    /*! total number of reassembled packets */
    uint64_t ipv6InReassembledPackets;

    /*! total number of received packets with errors */
    uint32_t ipv6InErrors;

} adk_ipv6_dp_address_stats_t;


/*!
 * =============================================================
 * @struct adk_ipv6_dp_if_stats_entry_t adk_ipv6_dp.h "adk_ipv6_dp.h"
 *
 * @brief The interface statistics structure.
 *
 * This structure contains interface specific statistics
 *
 * =============================================================
 */
typedef struct adk_ipv6_dp_if_stats_entry_s
{
    /* The total number of datagrams successfully delivered to IP user-protocols
     * (including ICMP).
     * Compliance Level: This only accounts for packets sent to CPU or higher 
     * layer protocol via ingress association. For example, it does not account
     * for packets having IPv6 options or packets sent to application via ACL,
     * etc. */
     uint32_t ipv6IfStatsInDelivers;
    /* The number of IP fragments received that needed to be reassembled at this
     * interface. 
     * Compliance Level: This accounts for number of fragments re-directed for
     * re-assembly by IPv6 module.  
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsReasmReqds;
    /* The number of IP datagrams successfully reassembled. 
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsReasmOKs;
    /* The number of IP fragments received that needed to be reassembled at this
     * interface. 
     * Compliance Level: This accounts for number of fragments re-directed for
     * re-assembly by IPv6 module.  
     * Compliance Level: Supported */
    uint32_t ipv6IfStatsReasmFails;
    /* The total number of octets received in input IP datagrams, including
     * those received in error. 
     * Compliance Level: supported */
     uint64_t ipv6IfStatsHCInOctets;
    /* The total number of octets in IP datagrams delivered to the lower layers
     * for transmission
     * Compliance Level: Supported */
     uint64_t ipv6IfStatsHCOutOctets;
    /* The total number of input IP datagrams received on input IP interface,
     * including those received in error .
     * Compliance Level: Supported  */
     uint32_t ipv6IfStatsInReceives;
    /* Ingress packets that got forwarded.
     * Compliance Level: supported */
     uint32_t ipv6IfStatsInForwDatagrams;
    /* Egress packets that got forwarded
     * Compliance Level: supported */
     uint32_t ipv6IfStatsOutForwDatagrams;
    /* The total number of IP unicast datagrams transmitted out.
     * Compliance Level: The number of IP unicast datagrams delivered to lower
     * Ethernet layer for transmission.
     * All packets counted here may not be sent out in case they are dropped
     * in MTM as per Ethernet or IP QoS policy or due to missing configuration
     * in Ethernet */
     uint32_t ipv6IfStatsOutUcastPkts;
    /* The total number of IP datagrams that this entity supplied to the lower
     * layers for transmission (e.g. sent to Ethernet ).
     * Compliance Level: This accounts for all packets sent by IPv6 module to 
     * the Lower Layer */
     uint32_t ipv6IfStatsOutTransmits;
    /* The total number of IP datagrams that local IP user-protocols (including
     * ICMP) supplied to IP in requests for transmission.
     * Compliance Level: This accounts for all packets given by higher layer 
     * protocol modules including IPSec module or CPU to IPv4 module for 
     * transmission */
     uint32_t ipv6IfStatsOutRequests;
    /* The number of IP datagrams that have been successfully fragmented
     * Compliance Level: This accounts for packets fragmented by IPv4 module and
     * include both forwarded as well as locally originated packets. 
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsOutFragOKs;
    /* The number of IP datagrams that would require fragmentation in order to
     * be transmitted
     * Compliance Level: This accounts for all locally  originated packets, 
     * which had IP packet length more than MTU and thus 
     * needed to be fragmented. */
     uint32_t ipv6IfStatsOutFragReqds;
    /* Ingress packets with address errors
     * Compliance Level: supported */
    uint32_t ipv6IfStatsInAddrErrors;
    /* Packets dropped by ingress interface level policing, ingress ACL based
     * policing or policing via ingress association level filter.
     * Compliance level: Supported - This includes packets dropped by ingress interface
     * level policing, policing via ACL filter or ingress association flow level
     * policing (for locally terminating traffic) */
    uint32_t ipv6IfStatsInPoliceDropPkts;
    /* Ingress packets with source address errors
     * Compliance Level: supported */
    uint32_t ipv6IfStatsInSrcAddrErrors;
    /* The number of input IP datagrams discarded because the datagram frame
     * didn't carry enough data
     * Compliance Level: supported */
    uint32_t ipv6IfStatsInTruncatedPkts;
    /* The number of input IP datagrams discarded due to errors in their IP
     * headers.
     * Compliance Level: This accounts for packet which failed basic IPv6 header
     * validation in IPv6 module including version error and length error
     * Compliance Level: Supported */
    uint32_t ipv6IfStatsInHdrErrors;
    /* The number of input IP datagrams discarded because no route could be
     * found to transmit them to their destination
     * Compliance Level: Counts all ingress No Route Found PDUs and PDUs will be 
     * discarded/sent to cpu  based on exception handling configuration*/
    uint32_t ipv6IfStatsInNoRoutes;
    /* The number of input IP datagrams discarded due to a black hole route.
     * Compliance Level: Supported  */
    uint32_t ipv6IfStatsInBlackHoles;
    /* The number of IP multicast datagrams received
     * Compliance Level: Supported */
    uint32_t ipv6IfStatsInMcastPkts;
    /* The number of IP multicast datagrams dropped
     * as corresponding group is not created */
    uint32_t ipv6IfStatsInMcastErrors;
    /*! The total number of Ipv6 Multicast octets or fragments which are received 
     *  on a particular interface are dropped due to invalid Multicast mac
     *  or invalid Multicast group.
     */
    uint32_t ipv6IfInDiscardedMCastOctets;
    /* The number of IP multicast datagrams transmitted
     * Compliance Level: supported */
    uint32_t ipv6IfStatsOutMcastPkts;
    /* The total number of octets transmitted in IP multicast datagrams.
     * Compliance Level: Supported */
    uint64_t ipv6IfStatsOutMcastOctets;
    /* The total number of octets received in IP multicast datagrams
     * Compliance Level: supported */
    uint64_t ipv6IfStatsHCInMcastOctets;

    /* Total number of padding octets received in Ethernet frames.
     * Compliance level: This counter counts padding octets that are present in
     * the Ethernet frames at the end of the IPv6 packets. It does not include
     * Ethernet header or FCS. This counter will be 0 in case of IPv6 tunnel and
     * virtual interfaces */
     uint64_t ipv6IfStatsHCInEthPadOctets;
    /* Total number of octets sent as Ethernet frames.
     * Compliance level: This counter counts all bytes within the Ethernet
     * frame that are sent out of IPoE interfaces. This includes Ethernet
     * header/trailer, IP header and payload, Ethernet padding and FCS. It does
     * not count octets for frames that are dropped in MTM (as per L2 or L3
     * QoS). This counter will be 0 in case of IP tunnel and virtual interfaces.
     */
     uint64_t ipv6IfStatsHCOutEthOctets;
    /* The number of IP datagrams that have been discarded because 
     * they needed to be fragmented but could not be. 
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsOutFragFails;
    /* Egress packets (with no errors) that got discarded
     * Compliance Level: supported*/
     uint32_t ipv6IfStatsOutDiscards;
    /* Egress Multicast packets (with no errors) that got discarded
     * Compliance Level: Packets sent out but discarded by the MTM buffer
     * management policy as per L3 QoS policy. This counter will be 0 in case
     * of Ethernet level Traffic Manager */
     uint32_t ipv6IfStatsOutMcastDiscards;
    /* Egress Unicast packets (with no errors) that got discarded
     * Compliance Level: Packets sent out but discarded by the MTM buffer
     * management policy as per L3 QoS policy. This counter will be 0 in case
     * of Ethernet level Traffic Manager */
     uint32_t ipv6IfStatsOutUcastDiscards;
    /* Egress packets (with no errors) that got discarded after being delivered
     * to lower layer (Ethernet).
     * Compliance Level: Packets sent out but discarded by the MTM buffer
     * management policy as per L2 QoS policy. This counter will be 0 in case
     * of IP level Traffic Manager. This counter will also be 0 in case of BVI
     * interface with Ethernet level traffic Manager. */
     uint32_t ipv6IfStatsOutL2Discards;
    /* Ingress packets (with no errors) discarded
     * Compliance Level: Not supported and always set to 0 */
     uint32_t ipv6IfStatsInDiscards;
    /* The number of locally-addressed IP datagrams received successfully
     * but discarded because of an unknown or unsupported protocol. 
     * Compliance Level: Not supported and always returned as 0. Application
     * is expected to handle all such packets and count them. */
     uint32_t ipv6IfStatsInUnknownProtos;
    /* The number of IP broadcast datagrams transmitted
     * Compliance Level: Not supported and always set to 0 */
     uint32_t ipv6IfStatsOutBcastPkts;
    /* The number of IP broadcast datagrams received. 
     * Compliance Level: Not supported and always set to 0 */
     uint32_t ipv6IfStatsInBcastPkts;

     /*! Total number of Output packets Dropped */
     uint64_t ipv6IfStatsOutDroppedOctets;

     /* Number of packets dropped because of fragment length is not multiples of eight */
     uint32_t ipv6IfNotMulOfEightFragLen;

    /* Number of packets dropped because of invalid Reassembly session id */
    uint32_t ipv6IfInvalidReasmSessionId;

    /* Number of packets dropped because of received reassembled packet offset overlapping previous fragment  offset*/
    uint32_t ipv6IfReasmOverlapFrag;

    /* Number of packets dropped because of more fragments received than expected fragment count */
    uint32_t ipv6IfTooManyFragsDropCount;

    /* Number of packets dropped because received reassembled fragment length is more than expected reassembly length */
    uint32_t ipv6IfExpectedReasmLengthExceed;

    /*! Number of non local packets dropped due to forwarding
     * is not enabled
     */
    uint32_t ipv6IfForwardingDisableDrop;

    /*! Number of L4 header truncated PDU*/
    uint32_t  ipv6IfL4Hdrtruncated;

    /*! Number of locally terminating packets dropped because of
     *  carrying incorrect L4 Protocol checksum.
     */
    uint32_t ipv6IfUDPCksumErrors;

    /*! Number of locally terminating packets dropped because of
     *  carrying incorrect ICMP Protocol checksum.
     */
    uint32_t ipv6IfICMPCksumErrors;

    /*! Number of locally terminated packets dropped due to
     * unsupported protocol
     */
    uint32_t ipv6IfUnsupportedProtoDrops;

    /*! Number of packets dropped due to IPv6 QoS lookup failed
        or no default queue configured when QoS is disabled */ 
    uint32_t ipv6IfQoSLookupFailed;

    /*! Rx QoS Statistics for PHB */
    adk_ipv6_ingress_qos_stats_t ipv6ifRxQoSstats[ADK_QOS_MAX_PHB_CLASS_IDX];
} __attribute__((aligned(64)))adk_ipv6_dp_if_stats_entry_t;

/*!
 * =============================================================
 * @struct adk_ipv6_cpu_recv_param_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This structure defines the parameters passed by IPv6 data plane software to
 * CPU when a packet is sent to CPU via PHV3 due to exception or due to flow
 * configuration.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ph_dev_rx_param_s
{
    /*! Indicates the received exception type */
    adk_ipv6_exception_type_t       exception_type;

    /* rte mbuf structure pointer */
    struct rte_mbuf *mbuf;

   /*ipv6 header offset */
    uint16_t ipv6Offset;

    union
    {
        /*! L4_offset for exception type
         * ADK_IPV6_UDP_INGRESS_CHECKSUM_INVALID
         */
        adk_ipv6_udp_chksum_invalid_t udpChksumInvalid;
        /*! Route table Id for exception type
         * ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND and
         */
        adk_ipv6_noroute_exception_param_t  routeNotFound;
        /*! Fragment Id for exception type
         * ADK_IPV6_EXCEPTION_REASSEMBLY_TIMER_EXPIRED
         */
        adk_ipv6_reass_timer_exceeded_t  reassTimerExceed;
        /*! Information received for exception type
         * ADK_IPV6_EXCEPTION_PARAM_PROBLEM.
         */
        adk_ipv6_param_problem_exception_data_t paramProbData;
        /*! Information received for exception type
         * ADK_IPV6_EXCEPTION_UNSUPPORTED_L4_PROTO
         */
         adk_ipv6_unsup_proto_exception_param_t unsupProto;
        /* Information received for exception type
         * ADK_IPV6_EXCEPTION_HOP_LIMIT_ZERO
         */
        adk_ipv6_hop_limit_zero_exception_param_t hopLimitZero;
        /* Information received for exception type
         * ADK_IPV6_INTERFACE_FORWARDING_DISABLE
         */
        adk_ipv6_fwding_disabled_exception_param_t fwdDisable;
        /* Information received for exception type
         * ADK_IPV6_EXCEPTION_INCORRECT_IP_VERSION
         */
        adk_ipv6_invalid_version_param_t  invalidVersion;
        /* ADK_IPV6_EXCEPTION_FRAG_OVERLAP */
        adk_ipv6_overlap_frag_param_t overlapFragment;
        /* ADK_IPV6_EXCEPTION_MTU_VIOLATION */
        adk_ipv6_mtu_violation_param_t mtuViolation;
        /* ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE */
        adk_ipv6_ext_hdr_twice_exception_data_t duplicateExtHeader;
    }exception_specific_data;

} __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ipv6_ph_dev_rx_param_t;


/*!
 * =============================================================
 * @struct adk_ipv6_dp_global_statistics_t adk_ipv6_dp.h "adk_ipv6_dp.h"
 *
 * @brief The common statistics structure.
 *
 * This structure contains the statistics that do not belong
 * to any flow.
 *
 * =============================================================
 */
typedef struct adk_ipv6_dp_global_statistics_s
{
    /*! Counts the PDU's discarded because of no route found to 
     * transmit the packet 
     * */
    uint32_t routeNotFoundDrop;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t routeNotFoundCpuException;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the MPP.
     */
    uint32_t routeNotFoundMppException;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the CPU SP_STACK.
     */
    uint32_t routeNotFoundCpuNophException;

    /*! Counts the PDUs which were found to have issues
     *  while parsing which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t paramProblemDrop;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t paramProblemCpuException;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t paramProblemMppException;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t paramProblemCpuNophException;

    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which
     *  were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t tooManyEhDrop;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which were
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t tooManyEhCpuException;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which
     *  were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t tooManyEhMppException;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which were
     *  handled by an exception handler function residing
     *  on the CPU NO PH .
     */
    uint32_t tooManyEhCpuNophException;

    /*! Counts the PDU's discarded because of no next hop
     *  Entry present, next hop entry lookup failed.
     */
    uint32_t noNextHopEntryDrop;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t resolveNexthopL2AddrCpuException;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the MPP.
     */
    uint32_t resolveNexthopL2AddrMppException;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the CPU NO PH.
     */
    uint32_t resolveNexthopL2AddrCpuNophException;

    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were dropped
     *  as there is no exception handler registered for them.
     */
    uint32_t resolveDstL2AddrDrop;
    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t resolveDstL2AddrCpuException;
    /*! Counts the PDUs for which the L2 address of
     *  the destination is not resolved which were handled
     *  by an exception handler function residing on the MPP.
     */
    uint32_t resolveDstL2AddrMppException;
    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were handled by an
     *  exception handler function residing on the CPU NO PH.
     */
    uint32_t resolveDstL2AddrCpuNophException;

    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t hopLimitZeroDrop;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t hopLimitZeroCpuException;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t hopLimitZeroMppException;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t hopLimitZeroCpuNophException;

    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t mtuViolationDrop;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t mtuViolationCpuException;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t mtuViolationMppException;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t mtuViolationCpuNophException;

    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t unknownEgressFlowDrop;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t unknownEgressFlowCpuException;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t unknownEgressFlowMppException;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t unknownEgressFlowCpuNophException;

    /*! Counts the PDUs for which the reassembly timer expired
     *  which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t reassemblyTimerExpiryDrop;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t reassemblyTimerExpiryCpuException;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t reassemblyTimerExpiryMppException;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t reassemblyTimerExpiryCpuNophException;

    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were dropped as there is
     *  no exception handler registered for them.
     */
    uint32_t fragOverLapDrop;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t fragOverLapCpuException;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t fragOverLapMppException;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t fragOverLapCpuNophException;

    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were dropped as
     *  there is no exception handler registered for them.
     */
    uint32_t nonZeroSegmentsLeftDrop;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled by
     *  an exception handler function residing on the CPU PH.
     */
    uint32_t nonZeroSegmentsLeftCpuException;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled
     *  by an exception handler function residing on the MPP.
     */
    uint32_t nonZeroSegmentsLeftMppException;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled by
     *  an exception handler function residing on the CPU NO PH.
     */
    uint32_t nonZeroSegmentsLeftCpuNophException;

    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  dropped as there is no exception handler registered
     *  for them.
     */
    uint32_t translateIcmpIpv6Drop;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t translateIcmpIpv6CpuException;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the MPP.
     */
    uint32_t translateIcmpIpv6MppException;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the CPU NO PH .
     */
    uint32_t translateIcmpIpv6CpuNophException;

    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and  
     *  there was no exception handler registered for them.
     */
    uint32_t mcLkpFailDrop;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and  
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t mcLkpFailCpuException;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and  
     *  handled by an exception handler function residing
     *  on the MPP.
     */
    uint32_t mcLkpFailMppException;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and
     *  handled by an exception handler function residing
     *  on the CPU NO PH.
     */
    uint32_t mcLkpFailCpuNophException;

    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and there was no exception handler registered 
     *  for them.
     */
    uint32_t mcRPFCheckFailDrop;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t mcRPFCheckFailCpuException;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function 
     *  residing on the MPP.
     */
    uint32_t mcRPFCheckFailMppException;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t mcRPFCheckFailCpuNophException;

    /*! Counts the PDUs for which the reassembly max pdu length 
     *  reached which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t reassemblyfragMaxPduLengthDrop;
    /*! Counts the PDUs for which the reassembly max pdu length
     *  reached which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t reassemblyfragMaxPduLengthCpuException;
    /*! Counts the PDUs for which the reassembly  max pdu length 
     *  reached which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t reassemblyfragMaxPduLengthMppException;
    /*! Counts the PDUs for which the reassembly max pdu length 
     *  reached which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t reassemblyfragMaxPduLengthCpuNophException;

    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t illegalSrcAddrTranslateDrop;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t illegalSrcAddrTranslateDropCpuException;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t illegalSrcAddrTranslateDropMppException;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t illegalSrcAddrTranslateDropCpuNophException;

    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t UDPIngressChecksumInvalidDrop;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t UDPIngressChecksumInvalidDropCpuException;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t UDPIngressChecksumInvalidDropMppException;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t UDPIngressChecksumInvalidDropCpuNophException;

    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t UDPIngressLengthInvalidDrop;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t UDPIngressLengthInvalidDropCpuException;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t UDPIngressLengthInvalidDropMppException;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t UDPIngressLengthInvalidDropCpuNophException;

    /*! Counts the PDUs for which the Dest MTU Lookup not found 
     *  and are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t DestMtuLookupdDrop;
    /*! Counts the PDUs for which the Dest MTU Lookup not found
     *  and are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t DestMtuLookupdDropCpuException;
    /*! Counts the PDUs for which the Dest MTU Lookup not found
     *  and are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t DestMtuLookupdDropMppException;
    /*! Counts the PDUs for which the Dest MTU Lookup not found
     *  and are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t DestMtuLookupdDropCpuNophException;

    /*! Counts the PDUs for which the local PTB generated upon post fragmentation
     *  which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t localPTBDrop;
    /*! Counts the PDUs for which the local PTB generated upon post fragmentation
     *  which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t localPTBCpuException;
    /*! Counts the PDUs for which the local PTB generated upon post fragmentation
     *  which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t localPTBMppException;
    /*! Counts the PDUs for which the local PTB generated upon post fragmentation
     *  which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t localPTBCpuNophException;

    /*! Common statistics */
    /*! Counts the PDUs for which route action in
     *  route_lookup_tree is invalid, which were dropped.
     */
    uint32_t invalidRouteAction;
    /*! Counts the PDUs for which association attributes
     *  are not found.
     */
    uint32_t noAttr;

    /*! Number of ICMP PDUs with invalid type encountered during
     *  translation to IPv4.
     */
    uint32_t translateIcmpInvalidType;
    /*! Number of ICMP PDUs with type meant for just the next hop
     *  encountered during translation to IPv4.
     */
    uint32_t translateIcmpSingleHopType;
    /*! Number of PDUs dropped due to checksum failure for translation */
    uint32_t translateChecksumFailure;
    /*! Number of PDUs dropped due to WKP and Private SIP DIP */
    uint32_t translateWkpNonGlobalIp;
    /*! Number of ICMP PDUs dropped due to frag ICMP */
    uint32_t translateFragICMPDrop;
    /*! Number of PDUs dropped due to invalid extn Hdr combination */
    uint32_t translateInvalidExtnHdr;

    /*! Number of PDUs dropped due to QOS action drop */
    uint32_t  qosActionDropPdus;
    /*! Number of PDUs forwrded due to QOS action forward */
    uint32_t  qosActionFwdPdus;
    /*! Number of link local PDUs received in LLI */
    uint32_t  linkLocalLLIPdus;

    /*! Number of PDUs coming into the ULI */
    uint32_t uliPdus;
    /*! Number of PDUs dropped due to invalid destination address */
    uint32_t  forwardingInvalidDstAddr;
    /*! Number of PDUs for which QOS policy tree look up failed */
    uint32_t  qosPolicyLookupFailedPDUs;
    /*! Number of PDUs sent for ISATAP encapsulation */
    uint32_t  encapIsatapPdus;
    /*! Number of PDUs sent for 6to4  encapsulation */
    uint32_t  encap6to4Pdus;
    /*! Number of PDUs sent for teredo  encapsulation */
    uint32_t  encapTeredoPdus;
    /*! Number of Link-local multicast PDUs coming into ULI */
    uint32_t  linkLocalULIPdus;
    /*! Number of hashInsert update fails for multipath next hop*/
    uint32_t  multiPathHashInsertFail;
    /*! Number of hashDelete fails*/
    uint32_t  HashDeleteFail;
    /*! Number of Invalid interface PDU*/
    uint32_t  InvalidIfFail;
    /*! Total number of packets which needed to be terminated locally via
     * ingress association but dropped because no matching ingress
     * association flow was found */
    uint32_t  noIngressAssociation;
    /*! Generic drop counter due to mismatched configuration
     */
    uint32_t  ipv6GenericDroppedPkts;
    /*! Total number of locally terminating packet that got dropped due to
     * unsupported ingress association mask. This counter is expected to
     * always remain zero as unsupported ingress association masks are not
     * allowed to be configured by the IPv6 APIs. */
    uint32_t  unknownAssociationMaskCnt;
    /*! Number of reassembly timer create failures */
    uint32_t ipv6ReassTimerCreateFailures;
    /*! Reassembly table full hits*/
    uint32_t ipv6ReassTableFullFailures;
    /*! Reassembly Errored context failures */
    uint32_t ipv6ReassErrContextFailures;
    /*! Reassembly Timedout contexts */
    uint32_t ipv6ReassTimedContextFailures;
    /*! Reassembly Timedout context versrion mismatch due to reuse */
    uint32_t ipv6ReassTimedVerMismatch;
    /*! Reassembly Timedout en-queue fails due to congestion*/
    uint32_t ipv6ReassTimedEnqueueFailures;
    /*! Mbuf alloc failures in IPv6 module */
    uint32_t ipv6MbufAllocFailures;
    /*! Number of IPv6 LLI flow cache full hits */
    uint32_t ipv6LLIFcFull;
    /*! Number of IPv6 ULI flow cache full hits */
    uint32_t ipv6ULIFcFull;
    /*! Number of packet discards due to non-availbility of headroom */
    uint32_t hdrmNotPresent;
    /*! Number of packets dropped in ACL due to incorrect configuration */
    uint32_t ipv6AclFail;
    /*! Number of packets dropped due to interface filtering */
    uint32_t ifFilterDrops;
    /*! Number of packets dropped due to pre&postFrag for inline Egress IPsec processing */
    uint32_t prePostFragErrors;
    /*! Number of packets dropped due to pre&postFrag for inline Egress IPsec processing */
    uint32_t ipv6HwPktCapDrops;
    /*! Number of packets dropped due to fragmentation limit(Max frags 8) exceeded*/
    uint32_t ipv6OutFragLimitExceed;
    /*! Number of packets Dropped due to packet size exceeded MTU size */
    uint32_t ipv6FwdMtuExceed;
    /*! Counts the PDUs for which SIP lookup is failed
    * because of IP is not configured on any interface
    */
    uint32_t ipv6OutSrcIpInvalid;
    /*! Counts the PDUs which are dropped due to policing*/
    uint32_t ipv6RateLimitingDrops;
    /*! Counts the PDUs which are dropped due to errors in ingress direction*/
    uint32_t ipv6InError;

} __attribute__((aligned(64)))adk_ipv6_dp_global_statistics_t;

typedef struct adk_ipv6_dp_l4counter_statistics_s
{
    /*! Total number of TCP protocol IPv6 packets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6TcpInPackets;
    /*! Total number of TCP protocol IPv6 octets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6TcpInOctets;
    /*! Total number of TCP protocol IPv6 packets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6TcpOutPackets;
    /*! Total number of TCP protocol IPv6 octets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6TcpOutOctets;
    /*! Total number of SCTP protocol IPv6 packets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6SctpInPackets;
    /*! Total number of SCTP protocol IPv6 octets received.
     *  Fragments are not included as individual packets.
     */
    uint64_t ipv6SctpInOctets;
    /*! Total number of SCTP protocol IPv6 packets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6SctpOutPackets;
    /*! Total number of SCTP protocol IPv6 octets transmitted.
     *  Fragmented packets are not included as individual packets.
     */
    uint64_t ipv6SctpOutOctets;
}adk_ipv6_dp_l4counter_statistics_t;

/* Tunnel interface entry 
 *  
 * adk_interface_id - 
 *
 */

typedef struct adk_ipv6_dp_tunnel_interface_lookup_entry_s
{   
    uint16_t    is_enabled:1;
    uint16_t    reserved:15;
    uint16_t    adk_interface_id;
}__attribute__((aligned(4))) adk_ipv6_dp_tunnel_interface_lookup_entry_t;


/* Virtual interface stats entry
 *
 * stats entries -
 *
 */
typedef struct adk_ipv6_dp_virtual_if_statistics_s
{

    /* The total number of datagrams successfully delivered to IP user-protocols
     * (including ICMP). 
     * Compliance Level: This only accounts for packets sent to CPU or higher 
     * layer protocol via ingress association. For example, it does not account
     * for packets having IPv6 options or packets sent to application via ACL,
     * etc. */
     uint32_t ipv6IfStatsInDelivers;
    /* The number of IP fragments received that needed to be reassembled at this
     * interface. 
     * Compliance Level: This accounts for number of fragments re-directed for
     * re-assembly by IPv6 module.  
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsReasmReqds;
    /* The number of IP datagrams successfully reassembled. 
     * Compliance Level: Supported */
     uint32_t ipv6IfStatsReasmOKs;
    /* The number of IP fragments received that needed to be reassembled at this
     * interface. 
     * Compliance Level: This accounts for number of fragments re-directed for
     * re-assembly by IPv6 module.  
     * Compliance Level: Supported */
    uint32_t ipv6IfStatsReasmFails;

}__attribute__((aligned(4))) adk_ipv6_dp_virtual_if_statistics_t;

/* Route table entry 
 *
 * reassembly_flag      -        
 * comp_dest_id         -
 * action               -
 * nexthop_id          -
 *
 * Intial 64 bits are updated as part of PBR action modify atomically.
 * DO NOT change the oredering of the structure.
 * */
typedef struct adk_ipv6_dp_route_entry_s
{
#if 0
    uint16_t    idx;
    uint16_t    next_idx;
#endif
    uint8_t     is_enabled:1;
    uint8_t     reserved:3;
    uint8_t     enable_ingress_assoc:1;
    uint8_t     on_link:1;
    uint8_t     null_route:1;
    uint8_t     reassembly_flag:1;
    uint8_t     action;
    uint16_t    ingress_assoc_rulemask;

    union {
    uint16_t    nexthop_id; /* nexthopid or flowId */
    uint16_t    interface_id; /* used when onlink lookup enabled */
    };
    uint16_t    routeMtu;
    uint8_t     reserved1;
    uint8_t     callback_fun_id;
    uint32_t    virtual_if_stats_id;
    adk_ipv6_addr_t dstIpv6Addr;

    adk_ph_pipeline_bind_handle_t    ph_bind_handle;
    uint16_t            devId;
    uint16_t            queueId;
    uint32_t            statsId;
    uint16_t            ipv6_addr_stats_id;
    uint32_t            policerId;
    adk_hos_rule_id_t hos_ipterm_rulehdl; /* IP termination handle used at CP */
#ifdef ADK_HOS_OLD_IMPL
    adk_hos_rule_id_t hos_inner_ipterm_rulehdl; /* IP termination handle used at CP */
#endif
}__attribute__((aligned(8))) adk_ipv6_dp_route_entry_t;



/*
 * IPv6 PBR Table 
 *
 * 5 tuples, pbits, dscp & frag flag is used as input DPDK ACL tree. 
 * DPDK input PBR rule struct : adk_ipv6_dp_pbr_rule_t
 * Fastpath lookup struct : adk_ipv6_dp_pbr_ptn_t
 * PBR output struct : adk_ipv6_dp_pbr_action_t     
 *
 * Index shall be returned from ACL lookup & 
 * it is used to get the action for the lookup
 */

typedef adk_handle_t /*void*/ adk_ipv6_dp_pbr_tree_t;

typedef struct _adk_ipv6_dp_pbr_rule_s
{
    uint32_t    sip_addr[4];
    uint8_t     sip_mask;
    uint32_t    dip_addr[4];
    uint8_t     dip_mask;
    uint16_t    sport_begin;
    uint16_t    sport_end;
    uint16_t    dport_begin;
    uint16_t    dport_end;
    uint8_t     proto_begin;
    uint8_t     proto_end;
    uint16_t    dscp_begin;
    uint16_t    dscp_end;
    uint16_t    frag_type_begin;
    uint16_t    frag_type_end;
    uint8_t     pbits_begin;
    uint8_t     pbits_end;
} adk_ipv6_dp_pbr_rule_t;

typedef struct __attribute__((__packed__))  _adk_ipv6_dp_pbr_ptn_s
{
    uint8_t     pbits;
    uint8_t     proto;
    uint32_t    sip_addr[4];
    uint32_t    dip_addr[4];
    uint16_t    srcPort;
    uint16_t    destPort;
    uint16_t    dscp;
    uint16_t    frag_type;
}adk_ipv6_dp_pbr_ptn_t;


typedef struct adk_ipv6_dp_pbr_action_entry_s
{
    uint16_t stats_id;
    uint8_t useDipRoutingTable;
    uint8_t routeTableId;
    adk_ipv6_dp_route_entry_t route_entry;
}adk_ipv6_dp_pbr_action_entry_t;


/*
 * IPv6 egress PBR state
 *
 * Used to check if PBR is enabled in ULI path
 *
 * Single Entry for the entire system, can directly check this flag.
 *
 * pbr_enable           -
 *
 */
typedef struct adk_ipv6_dp_pbr_state_s
{
    bool            pbr_enable;
}adk_ipv6_dp_pbr_state_t;


/*
 * IPV6 Flow Info  entry (Bind Info)
 *
 * destModId            -
 * destFlow             -
 * destFunc             -
 */


typedef struct adk_ipv6_dest_flow_info_s 
{
    /*! dest Flow & destFunc for bind */
    uint32_t            destFlow;
    uint16_t            destModId;
    adk_pal_dp_stage_info_t stageInfo;
}adk_ipv6_dest_flow_info_t;

typedef struct adk_ipv6_dp_flow_info_s
{
    adk_ipv6_dest_flow_info_t    flow[2];
    uint8_t                 idx;
}adk_ipv6_dp_flow_info_t;


/*
 * IPV6 Nexthop Table entry (advanced route lookup)
 *
 * interface_id         -
 * destModId            -
 * destFlow             -
 * destFunc             - 
 */
typedef struct adk_ipv6_dp_nexthop_entry_s
{
    /*! dest Flow & destFunc for bind */
    adk_ipv6_dp_flow_info_t  flow_info;

    uint16_t            idx;
    uint16_t            next_idx;
    uint16_t            is_enabled:1;
    uint16_t            is_l2_state_resolved:1;
    uint16_t            reserved:14;
    uint16_t            interface_id;
    adk_ipv6_addr_t     nextHopIpv6Addr;
}__attribute__((aligned(32))) adk_ipv6_dp_nexthop_entry_t;


/*
 * IPv6 fragment extension header 
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |  Next Header  |   Reserved    |      Fragment Offset    |Res|M|
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                         Identification                        |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 */
typedef struct adk_ipv6_frag_hdr_s
{
    uint32_t nh_ofs_m;   
    uint32_t frag_id;
}adk_ipv6_frag_hdr_t;


/* 
 * IPV6 Reassembly tuples 
 *
 * Reassembly tuples considered for reassembly fow identififcation
 *  src_addr         - IPv6 source address
 *  dst_addr         - IPv6 destination address
 *  frag_id;         - fragmentation id 
 */
typedef struct adk_ipv6_reassembly_tuple_s
{
    uint64_t        src_addr[2];
    uint64_t        dst_addr[2];
    uint32_t        frag_id;
}adk_ipv6_reassembly_tuple_t;


typedef struct adk_ipv6_reassembly_frags_s
{
    uint16_t        ofs;
    uint16_t        len;
    adk_pktio_task_t *task;
}adk_ipv6_reassembly_frags_t;

/* 
 * IPV6 Reassembly metadata
 *
 * The data structure defines the reassembly timer stage metadata that is
 * passed along the timerout handler registrarion.
 *  queue_id         - queue id of the packet received
 *  flow_id          - flow id of the received packet
 *  if_stats         - if stats, required to increment during cleanup.
 */
typedef struct adk_ipv6_dp_reass_stage_meta_data_s
{
    uint16_t    queue_id;
    uint16_t    flow_id;
    uint16_t    hash_id;
    uint8_t     collision_id;
    uint8_t     reass_ver_id;
    adk_ipv6_dp_if_stats_entry_t *if_stats;
}adk_ipv6_dp_reass_stage_meta_data_t;

typedef struct adk_ipv6_dp_policer_meta_data_s
{
    adk_ipv6_dp_ing_classifier_action_info_t ing_action;
    uint32_t ingress_classifier_flow_id;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ipv6_dp_policer_meta_data_t;

/* 
 * IPV6 Reassembly Table Entry 
 *
 * Source IPv6 address, destination IPv6 address & fragment id 
 * is used as an input HASH table.
 * (index can be directly derived from fragment id).
 *
 * hash table lookup returns the index to access the following table.
 *
 *  idx                - index of the current reass entry
 *  next_idx           - next index to maintain the free list
 *  nb_frags           - number of fragments received 
 *  frag_size          - total fragmented packet size received
 *  total_size         - total reassembled size expected
 *  if_id              - interface on which fragment received
 *  err_detected       - flag used to identify that the fragments 
 *                          received should be dropped.
 *  frag_idx           - index for the frag array. 
 *  collosion_idx      - collision index of the reass session
 *  is_reassembly_success - if flag TRUE, indicates reassembly is success
 *  vrd_id             - virtual routing domain on which packets are recived 
 *  ext_hdr_len        - first fragment extension header length if any
 *  hash_idx           - hash index of the reassembly session.
 */ 
typedef struct adk_ipv6_dp_reassembly_entry_s
{
    uint16_t         idx;      
    uint16_t         next_idx;

    uint8_t          reass_ver_id;
    uint8_t          nb_frags;
    uint16_t         frag_size;
    
    uint16_t         total_size;
    uint16_t         if_id;
    
    uint8_t          err_detected;
    uint8_t          frag_idx;
    uint8_t          collosion_idx;
    uint8_t          is_reassembly_success;

    uint8_t          vrd_id;
    uint8_t          ext_hdr_len;
    uint16_t         hash_idx;

    uint16_t         queue_id;
    uint16_t         flow_id;
    adk_ipv6_dp_if_stats_entry_t *if_stats;

    adk_timer_hdl_t  timer_hdl;
    clear_ipsec_ctx_for_hash    ipsecCb;
    adk_ipv6_reassembly_tuple_t reass_tuple;
    adk_ipv6_reassembly_frags_t fg[ADK_IPV6_MAX_REASS_ELE];
}__attribute__ ((aligned(64))) adk_ipv6_dp_reassembly_entry_t;

/* 
 * IPV6 Reassembly Hash Table Entry 
 *
 * Hash table for reassembly table, 
 * index 0 is used for HASH ID based lock  
 *
 *  is_enabled         - is the entry used
 *  list_len           - number of collisioned entries
 *  index              - index to re-assembly context
 *  hash_value         - hash value
 */
typedef struct adk_ipv6_dp_reassembly_hash_entry_s
{
    uint8_t         is_enabled;
    uint8_t         list_len;
    uint16_t        index;
    uint32_t        hash_value;
    bool            first_frag;
    adk_ipv6_dp_reassembly_entry_t *reass_entry;
}__attribute__ ((aligned(8))) adk_ipv6_dp_reassembly_hash_entry_t;

/* 
 * IPV6 Reassembly Free Entry  
 * Free entry indexes are maintained in the list  
 */

typedef struct adk_ipv6_dp_reassembly_list_s
{
    uint16_t head_idx;
    uint16_t tail_idx;
    uint32_t count;
}adk_ipv6_dp_reassembly_list_t;

/* 
 * IPV6 egress flow attribute Entry 
 *
 * IPv6 egress flow id is used directly input to the egress attribute table
 * Table returns the UDP header fields
 *
 *  mtu                 -
 *  action              -
 *  vrd_id              -
 *  priority            -
 *  priority_flag       -
 *  src_port            -
 *  dst_port            -
 *  dst_addr            -
 *  src_addr            -
 */

typedef struct adk_ipv6_dp_egress_flow_attr_entry_s
{
    uint16_t    idx;
    uint16_t    next_idx;
    uint8_t     is_enabled:1;
    uint8_t     reserved:6;
    uint8_t     priority_flag:1;
    uint8_t     vrd_id;
    uint8_t     action_flags;
    uint16_t    mtu;
    uint16_t    flow_stats_id;

#ifdef ADK_IPV6_FLOW_CACHE
    uint16_t    first_key;
    uint16_t    second_key;
    uint32_t    hash_value;
#endif

    uint32_t    ver_tc_flabel;
    uint32_t    len_nh_hlimit;
    adk_ipv6_addr_t src_addr;
    adk_ipv6_addr_t dst_addr;
    uint16_t    src_port;
    uint16_t    dst_port;
}__attribute__ ((aligned(64))) adk_ipv6_dp_egress_flow_attr_entry_t;


/*
 * IPv6 Ingress association Entry
 *
 * Rule mask, vrd id, priority, priority flag, src port, 
 * dst port, dst addr, & src addr are the input to the LPM tree 
 * (need to have seperate LPM tree for ICMP pkts?) 
 *
 * LPM lookup returns the index to the following table
 *
 * action               - 
 * destModId            -
 * destFlow             -
 * destFunc             -
 */
typedef struct adk_ipv6_dp_ingress_assoc_entry_t
{
    /*! dest Flow & destFunc for bind */
    adk_ipv6_dp_flow_info_t  flow_info;

    uint16_t            idx;
    uint16_t            next_idx;
    uint8_t             is_enabled:1;
    uint8_t             res:7;
    /* CPU Path */
    uint8_t             callback_fun_id;
    uint16_t            action;

    uint16_t            flow_stats_id;

    uint16_t            tunnel_if_id;
    adk_ph_pipeline_bind_handle_t   ph_bind_handle;

    /* fps callback attributes */
    adk_ipv6_invoke_cb_attr_t invoke_cb_attr;
    uint16_t            devId;
    uint16_t            queueId;
    uint32_t            statsId;
    uint32_t            policerId;

}__attribute__ ((aligned(32))) adk_ipv6_dp_ingress_assoc_entry_t;

typedef struct adk_ipv6_dp_ingress_mgroup_entry_s
{
	uint16_t            idx;
	uint16_t            next_idx;
	uint8_t             is_enabled:1;
	uint8_t             group_stats_id;
	uint8_t             ref_count;
	uint16_t            action;
}__attribute__ ((aligned(32))) adk_ipv6_dp_ingress_mgroup_entry_t;


typedef struct adk_ipv6_dp_ingress_assoc_rule_s
{
    /* IPv6 ingress association rule selection mask.
     * Refer adk_ipv6_ingress_association_field_mask_t.
     */
    uint16_t rules_mask;
    
    /*! 
     * The IPv6 Interface handle.      
     * If passed as 0, the IP interface handle becomes a
     * "don't care" field.
     */
    uint16_t interface_id;
    
    /*! 
     * The flow label from the IPv6 header. If passed as 0, the flow label
     * field becomes a "don't care".
     */
    uint32_t flow_label;
 
    /*!
     * ipv6VrdId provides the Routing Domain ID. 
     */
    uint8_t vrd_id;
   
    /*!
     * 8-bit IP protocol type field. If passed as 0, the protocol
     * field becomes a "don't care".
     */
    uint8_t protocol_type;
    
    /*! 
     * L4 source port field.
     * If passed as 0, the field becomes a "don't care".
     */
    uint16_t src_port;
    
    /*! 
     * L4 destination port field.
     * If passed as 0, the field becomes a "don't care".
     */
    uint16_t dst_port;
    
    /*! The source IPv6 address.
     */
    adk_ipv6_addr_t srcIpv6Addr;
    
    /*! The destination IPv6 address.
     */
    adk_ipv6_addr_t dstIpv6Addr;
} adk_ipv6_dp_ingress_assoc_rule_t;   /* ------  end of struct adk_ipv6_dp_ingress_assoc_rule_t  ------ */


/*
 * IPv6 Ingress association statistics Entry 
 */
typedef adk_ipv6_ingress_association_statistics_t adk_ipv6_dp_ing_assoc_stats_entry_t; 

/*
 * IPv6 Ingress association statistics Entry 
 */
typedef adk_ipv6_egress_association_statistics_t adk_ipv6_dp_egress_assoc_stats_entry_t;

/*
 * IPv6 Ingress flow SIP table Entry  
 *
 * vrd id, IPv6 src addr are the input to the LPM tree
 * LPM lookup returns the index to the following table
 *
 * action               -
 * ph_handle            -
 * apply_to_fwd_traffic -
 * cbFnId               -
 */
typedef struct adk_ipv6_dp_ingress_flow_sip_entry_s
{
    uint16_t    idx;
    uint16_t    next_idx;
    /*! dest Flow & destFunc for bind */
    uint32_t    is_enabled:1;
    uint32_t    reserved:14;
    uint32_t    apply_to_fwd_traffic:1; /* check if this can be combined with action */
    uint32_t    cbFnId:8;
    uint32_t    action:8;
    adk_ph_pipeline_bind_handle_t    ph_bind_handle;
    uint32_t    qid;
    adk_ipv6_processing_path_t cpuPath;    
    uint16_t            devId;
    uint16_t            queueId;
    uint32_t            statsId;
    uint32_t            policerId;
}__attribute__ ((aligned(16))) adk_ipv6_dp_ingress_flow_sip_entry_t;


/*
 * IPv6 VRD Host Mode Table Entry 
 *
 * Vrd id is an input to the Table. 
 * table returns routing enabled or not at VRD level.
 *
 *  enable_routing      -
 *
 */
typedef struct adk_ipv6_dp_vrd_host_mode_entry_s
{
    uint8_t         is_enabled:1;
    uint8_t         enable_routing:1;
    uint8_t         reserved:6;
}adk_ipv6_dp_vrd_host_mode_entry_t;

/*
 * IPv6 Onlink destination Table hash Entry 
 *
 * Vrd id & complete ipv6 address are stored 
 * to verify the entry. 
 * 
 */
typedef struct adk_ipv6_dp_onlink_dest_hash_entry_s
{
    uint8_t     is_enabled;
    uint8_t     reserved;
    uint16_t    index;
    uint32_t    hash_value;
}__attribute__ ((aligned(8))) adk_ipv6_dp_onlink_dest_hash_entry_t;

/*
 * IPv6 Onlink destination Table Entry 
 *
 * Vrd id & complete ipv6 address are stored 
 * to verify the entry. 
 */
typedef struct adk_ipv6_dp_onlink_dest_entry_s
{
    /*! dest Flow & destFunc for bind */
    adk_ipv6_dp_flow_info_t  flow_info;

    uint16_t                idx;
    uint16_t                next_idx;
    uint8_t                 is_enabled:1;
    uint8_t                 reserved1:7;
    uint8_t                 vrd_id;
    uint16_t                reserved2;
    adk_ipv6_addr_t         ipv6_addr;
    uint16_t                interface_id;
}__attribute__ ((aligned(64))) adk_ipv6_dp_onlink_dest_entry_t;

typedef struct adk_ipv6_dp_overflow_node_s
{
    uint16_t idx;
    struct adk_ipv6_dp_overflow_node_s *next;
}adk_ipv6_dp_overflow_node_t;

typedef struct adk_ipv6_dp_linked_list_s
{
    adk_ipv6_dp_overflow_node_t *head;
    uint16_t count;
}adk_ipv6_dp_linked_list_t;

/*
 * IPv6 Exception Table Entry 
 * 
 * exception domain & type are 
 * input to the table.
 * 
 */
typedef struct adk_ipv6_dp_excep_entry_s
{
    uint16_t    idx;
    uint16_t    next_idx;
    uint8_t     is_enable:1;
    uint8_t     reserved:7;
    uint8_t     callback_fun_id;
    uint16_t    qid;
    adk_ipv6_exception_handler_loc_t exception_path;
    adk_ph_pipeline_bind_handle_t    ph_bind_handle;
    adk_ipv6_exception_type_t type;
    uint16_t    devId;
    uint16_t    queueId;
    uint32_t    statsId;
    uint32_t    policerId;
    uint8_t priority;
}__attribute__ ((aligned(16))) adk_ipv6_dp_excep_entry_t;

/*
 * IPv6 Destination MTU Table hash Entry 
 *
 * Vrd id & complete ipv6 address are stored 
 * to verify the entry. 
 */
typedef struct adk_ipv6_dp_dest_mtu_hash_entry_s
{
    uint8_t     is_enabled;
    uint8_t     reserved;
    uint16_t    index;
    uint32_t    hash_value;
}__attribute__ ((aligned(8))) adk_ipv6_dp_dest_mtu_hash_entry_t;

/*
 * IPv6 Destination MTU Table Entry 
 *
 * Vrd id & complete ipv6 address are stored 
 * to verify the entry. 
 */
typedef struct adk_ipv6_dp_dest_mtu_entry_s
{
    uint16_t                idx;
    uint16_t                next_idx;
    uint8_t                 is_enabled:1;
    uint8_t                 reserved1:7;
    uint8_t                 vrd_id;
    uint16_t                reserved2;
    adk_ipv6_addr_t         ipv6_addr;
    uint16_t                mtu;
}__attribute__ ((aligned(32))) adk_ipv6_dp_dest_mtu_entry_t;

/*
 * IPV6 Multicast Table
 *
 * IPv6 multicast Destination address & source address as input 
 * (need to check the LPM hash tables need to be used )
 *
 * output index can be used for following data rerive
 */ 
typedef struct adk_ipv6_dp_multicast_fwd_s{
    uint32_t     mcast_grp_id;
    uint32_t    mcast_flow_id;
}__attribute__((aligned(8))) adk_ipv6_dp_multicast_fwd_t;





/********************************************************************************
 *                  IPv6 Fast path table definitions                            *
 ********************************************************************************/

/*
 * IPV6 ingress interface Table 
 *
 * Input    : Interface id 
 * Output   : Interface attribute. check ipv6_interface_info_t for more details 
 * cpu nh table can be combined
 */ 
typedef struct adk_ipv6_dp_interface_table_s
{
    adk_ipv6_dp_interface_entry_t           *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    adk_ipv6_dp_if_stats_entry_t            *stats_tbl[ADK_MAX_NUM_CORES];
}adk_ipv6_dp_interface_table_t;




/*
 * IPV6 tunnel interface Lookup Table 
 * 
 * Input    : Tunnel interface id
 * Output   : adk interface id. ( Hash Table is required? ) 
 * cpu nh table can be combined
 */ 
typedef struct adk_ipv6_dp_tunnel_interface_lookup_table_s
{
    adk_ipv6_dp_tunnel_interface_lookup_entry_t   *tbl;
}adk_ipv6_dp_tunnel_interface_lookup_table_t;


typedef struct adk_ipv6_dp_route_pending_list_s
{
    adk_ipv6_dp_route_entry_t        *entry;
    struct adk_ipv6_dp_route_pending_list_s *next;
}adk_ipv6_dp_route_pending_list_t;


/*
 * IPv6 Route Table 
 *
 * Input    : VRD id & IPv6 dest address(can be wildcarded)
 * Output   : output from LPM table is used as an index to ipv6_route_table 
 *             Table entry hash action, Next hop id or flow id, flags etc...
 *
 * Note: adk_ipv6_dp_route_entry_t database lookup can be avoided 
 *              if we can fit the ipv6_route_lpm_table lookup result 
 *              within 32 bits Evaluating.
 *
 */
typedef struct adk_ipv6_dp_route_table_s
{
    adk_ipv6_dp_lpm_table_t             lpm_tbl;
    adk_rcu_pending_list_t              *pending_list;
    uint32_t                        list_count;

}adk_ipv6_dp_route_table_t;

/*
 * IPv6 ACL Table 
 *
 * Input  : 5 tuples, dscp & frag flag is used as input DPDK ACL tree. 
 *          DPDK input ACL rule struct : adk_ipv6_dp_acl_rule_t
 * Output : 
 * 
 *          ACL lookup (index) is used to access following IPV6 ACL 
 *          action table 
 */

/*
 * Rule and trace formats definitions.
 */

struct ipv6_7tuple {
        uint8_t  proto;
        uint32_t ip_src1;
        uint32_t ip_src2;
        uint32_t ip_src3;
        uint32_t ip_src4;
        uint32_t ip_dst1;
        uint32_t ip_dst2;
        uint32_t ip_dst3;
        uint32_t ip_dst4;
        uint16_t port_src;
        uint16_t port_dst;
        uint16_t  dscp;
        uint16_t  frag_type;
}__attribute__((__packed__));

enum {
        PROTO_FIELD_IPV6,
        SRC1_FIELD_IPV6,
        SRC2_FIELD_IPV6,
        SRC3_FIELD_IPV6,
        SRC4_FIELD_IPV6,
        DST1_FIELD_IPV6,
        DST2_FIELD_IPV6,
        DST3_FIELD_IPV6,
        DST4_FIELD_IPV6,
        SRCP_FIELD_IPV6,
        DSTP_FIELD_IPV6,
        DSCP_FIELD_IPV6,
        FRAG_FIELD_IPV6,
        NUM_FIELDS_IPV6
};

enum {
        PROTO_INPUT_IPV6,
        SRC1_INPUT_IPV6,
        SRC2_INPUT_IPV6,
        SRC3_INPUT_IPV6,
        SRC4_INPUT_IPV6,
        DST1_INPUT_IPV6,
        DST2_INPUT_IPV6,
        DST3_INPUT_IPV6,
        DST4_INPUT_IPV6,
        L4PORTS_INPUT_IPV6,
        DSCP_FRAG_INPUT_IPV6
};

typedef struct adk_ipv6_dp_acl_table_s
{
    adk_ipv6_dp_acl_tree_t                  **acl_ruleset_tbl[2];
    adk_ipv6_dp_acl_action_entry_t          *tbl[2];
    uint32_t *acl_stats_tbl;
    /* Index to identify current and shadow copy */
    uint8_t acl_tree_index;
}adk_ipv6_dp_acl_table_t;

/*
 * IPv6 PBR Table 
 *
 * Input  : 5 tuples, dscp, pbits & frag flag is used as input DPDK ACL tree. 
 *          DPDK input ACL rule struct : adk_ipv6_dp_pbr_rule_t
 * Output : 
 *          PBR lookup result (index) is used to access following PBR ACL 
 *          action table 
 */
typedef struct adk_ipv6_dp_pbr_table_s
{
    adk_ipv6_dp_pbr_action_entry_t  *tbl;
    uint32_t *pbrRuleStats;
}adk_ipv6_dp_pbr_table_t;

/*
 * IPv6 VRD to PBR rule set mapping table
 * Input : vrd
 * Output: Pbr enabled and pbr policy set id
*/
typedef struct adk_ipv6_dp_vrd_pbr_ruleset_map_table_s 
{
    uint8_t pbr_policy_set_id;
    bool pbr_enabled;
} adk_ipv6_dp_vrd_pbr_ruleset_map_table_t; 

/*
 * IPv6 PBR State Table 
 *
 * Input  : No input, directly access the global flag 
 * Output : Flag returned indicates if PBR is enabled in ULI direction. 
 */
typedef struct adk_ipv6_dp_pbr_state_table_s
{
    adk_ipv6_dp_pbr_state_t                 *tbl;
}adk_ipv6_dp_pbr_state_table_t;

/* 
 * IPV6 Teredo client Table
 *
 * Input  : VRDId & IPV6 address(can be wild carded) is used as input to LPM tree
 * Output : 8 bits  Action, 16 bits flowid
 *          value can be an index & shall be used to index the NH Table
 */
typedef struct adk_ipv6_dp_teredo_table_s
{
    adk_ipv6_dp_lpm_table_t             lpm_tbl;
}adk_ipv6_dp_teredo_table_t;

/*
 * IPV6 Nexthop Table (advanced route lookup)
 *
 * Input  : Nexthop id/IPV6 flow id is the input & it can be directly indexed.
 * Output : Output interface, mtu, destFunc, destination flow etc...
 *          Refer adk_ipv6_dp_nexthop_entry_t for complete information.
 */
typedef struct adk_ipv6_dp_nexthop_table_s
{
    adk_ipv6_dp_nexthop_entry_t         *tbl;
    adk_ipv6_dp_array_list_t            pending_list;
    adk_ipv6_dp_array_list_t            free_list;
}adk_ipv6_dp_nexthop_table_t;

/*
 * IPV6 onlink destination Table (Convert this to hash table. Dest IP is not wildcarded)
 *
 * Input  : VRD ID & IPV6 dest address(fullly specified) are used as input for LPM tree
 * Output : Table return index(24 bit) & 8 bit flag.
 *          Index can be directly used look at the adk_ipv6_dp_route_advance_info_t entry
 */
typedef struct adk_ipv6_dp_onlink_dest_table_s
{
    adk_ipv6_dp_onlink_dest_hash_entry_t    *hash_tbl;
    adk_ipv6_dp_onlink_dest_entry_t         *tbl;
    adk_ipv6_dp_linked_list_t               overflow_list;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_linked_list_t               overflow_pending_list;
    adk_ipv6_dp_array_list_t                free_list;
}adk_ipv6_dp_onlink_dest_table_t;
/* 
 * IPV6 Reassembly Table (HASH table is required) 
 *
 * Input  : Source IPv6 address, destination IPv6 address & fragment id is used as an 
 *              input HASH table.
 * Output : hash table lookup (index can be directly derived from fragment id)
 *              returns the index to access the following table.
 */
typedef struct adk_ipv6_dp_reassembly_table_s
{
    adk_ipv6_dp_reassembly_hash_entry_t *hash_tbl;
    struct rte_mempool                  *tbl;
    rte_spinlock_t                      lock; /*spinlock shall be optimised */
    adk_ipv6_dp_array_list_t            free_list;
    adk_ipv6_dp_array_list_t            pending_list;
}adk_ipv6_dp_reassembly_table_t;

/* 
 * IPV6 egress flow attribute Table 
 *
 * Input  : IPv6 egress flow id is used directly input to the egress attribute table
 * Output : Table returns the IP header, UDP header fields
 */
typedef struct adk_ipv6_dp_egress_flow_attr_table_s
{
    adk_ipv6_dp_egress_flow_attr_entry_t    *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    adk_ipv6_dp_egress_assoc_stats_entry_t  *stats_tbl[ADK_MAX_NUM_CORES];
}adk_ipv6_dp_egress_flow_attr_table_t;

/*
 * IPv6 Ingress association Table
 *
 * Input  : Rule mask, vrd id, priority, priority flag, src port, dst port, 
 *              dst addr, & src addr are the input to the LPM tree 
 *              (need to have seperate LPM tree for ICMP pkts?) 
 *
 * Output : action, dest Module, function etc... Refer adk_ipv6_dp_ingress_assoc_entry_t
 *          for more details.
 */
typedef struct adk_ipv6_dp_ingress_assoc_table_s
{
    adk_ipv6_dp_lpm_table_t                 lpm_tbl;
    adk_ipv6_dp_ingress_assoc_entry_t       *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    adk_ipv6_dp_ing_assoc_stats_entry_t     *stats_tbl[ADK_MAX_NUM_CORES];
}adk_ipv6_dp_ingress_assoc_table_t;

typedef struct adk_ipv6_dp_ingress_mgroup_table_s
{
    adk_ipv6_dp_lpm_table_t                 lpm_tbl;
    adk_ipv6_dp_ingress_mgroup_entry_t       *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
//    adk_ipv6_dp_ing_assoc_stats_entry_t     *stats_tbl[ADK_MAX_NUM_CORES];
}adk_ipv6_dp_ingress_mgroup_table_t;

typedef struct adk_ipv6_dp_ingflow_sip_pending_list_s
{
    adk_ipv6_dp_ingress_flow_sip_entry_t* entry;
    struct adk_ipv6_dp_ingflow_sip_pending_list_s* next;
}adk_ipv6_dp_ingflow_sip_pending_list_t; 

/*
 * IPv6 Ingress flow SIP table   
 *
 * Input  : Vrd id, IPv6 src addr are the input to the LPM tree
 * Output : action, PH handle, callback function, flags etc...
 *          refer adk_ipv6_dp_ingress_flow_sip_entry_t for more details 
 *
 *          (check if return value can be fit within 32 bit).
 */
typedef struct adk_ipv6_dp_ingress_flow_sip_table_s
{
    adk_ipv6_dp_lpm_table_t         lpm_tbl;
    adk_rcu_pending_list_t         *pending_list;
    uint32_t                        list_count;
}adk_ipv6_dp_ingress_flow_sip_table_t;



/*
 * IPv6 VRD Table
 * Input    : 
 * Output   : 
 */ 
typedef struct adk_ipv6_dp_vrd_table_s
{
    uint8_t     vrd_state[ADK_IPV6_VRD_ID_MAX];
    adk_ipv6_dp_vrd_host_mode_entry_t       *tbl;
}adk_ipv6_dp_vrd_table_t;

/*
 * IPv6 Exception Table
 *
 * Input    : 
 * Output   : 
 */ 
typedef struct adk_ipv6_dp_excep_table_s
{
    adk_ipv6_dp_excep_entry_t               *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
}adk_ipv6_dp_excep_table_t;

/*
 * IPV6 onlink destination Table (Convert this to hash table. Dest IP is not wildcarded)
 *
 * Input  : VRD ID & IPV6 dest address(fullly specified) are used as input for LPM tree
 * Output : Table return index(24 bit) & 8 bit flag.
 *          Index can be directly used look at the adk_ipv6_dp_route_advance_info_t entry
 */
typedef struct adk_ipv6_dp_dest_mtu_table_s
{
    struct adk_ipv6_dp_dest_mtu_table_s     *next;
    adk_ipv6_dp_dest_mtu_hash_entry_t       *hash_tbl;
    adk_ipv6_dp_dest_mtu_entry_t            *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    uint32_t                                num_entries;
}adk_ipv6_dp_dest_mtu_table_t;

typedef struct adk_ipv6_dp_dest_mtu_tbl_list_s
{
    adk_ipv6_dp_dest_mtu_table_t           tbl_head;
    uint32_t                               num_table;
}adk_ipv6_dp_dest_mtu_tbl_list_t;

/*
 * IPv6 M->M Bind info
 *
 * Input    : 
 * Output   : 
 */ 
typedef struct adk_ipv6_dp_bind_table_s
{
    uint8_t  is_enabled;
    adk_ipv6_dp_flow_info_t tbl;
}adk_ipv6_dp_bind_table_t;

typedef struct adk_ipv6_dp_flow_cache_aging_info_s{
    uint32_t time_in_msec;
    uint16_t prev_fc_idx;

    uint64_t active_flows[ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>6];
    uint64_t aging_bitmap[ADK_IPV6_NO_OF_FLOW_CACHE_ENTRIES>>6];
}adk_ipv6_dp_flow_cache_aging_info_t;

typedef struct adk_ipv6_dp_flow_cache_hash_entry_s
{
    uint32_t hash_value;
    uint32_t is_enabled:1;
    uint32_t reserved:15;
    uint32_t index:16;
}adk_ipv6_dp_flow_cache_hash_entry_t;


typedef enum adk_ipv6_dp_lli_flow_cache_actions_s
{
    ADK_IPV6_DP_LLI_FLOW_CACHE_FWD_BIND = 1, /* Egress ULI bind to lower layers - Eth */ 
    ADK_IPV6_DP_LLI_FLOW_CACHE_EGRESS_BIND, /* Egress ULI bind to higher layers - IPsec */ 
    ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_BIND, /* ingress LLI bind to higher layers - IPsec, GTPu BFD etc */
    ADK_IPV6_DP_LLI_FLOW_CACHE_INGRES_TERMINATE, /* FPS callback */
    ADK_IPV6_DP_LLI_FLOW_CACHE_FRAGMENT_TERMINATE, /* fragement termination */
    ADK_IPV6_DP_LLI_FLOW_CACHE_ACTION_MAX 
}adk_ipv6_dp_lli_flow_cache_actions_t;


typedef enum adk_ipv6_dp_lli_flow_cache_type_s
{
    ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_5TUPLE,
    ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_5TUPLE_FRAGMENT,
    ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_3TUPLE,
    ADK_IPV6_DP_LLI_FLOW_CACHE_TYPE_3TUPLE_FRAGMENT
}adk_ipv6_dp_lli_flow_cache_type_t;

/* Flows which binds to lower layer egress bind (Ex: Ethernet) */
typedef struct adk_ipv6_dp_lli_flow_cache_fwd_attr_s{
    uint16_t                egress_if_id;
    uint16_t                mtu;
    uint8_t                 egress_if_type;
}adk_ipv6_dp_lli_flow_cache_fwd_attr_t;


/* LLI flows which binds higher layer egress processing(Ex: IPsec) */
typedef struct adk_ipv6_dp_lli_flow_cache_egress_attr_s{
    uint16_t                egress_if_id;
    uint16_t                mtu;
    uint8_t                 egress_if_type;
    uint8_t                 outer_vrd_id;
    adk_handle_t            ipsec_info;    
}adk_ipv6_dp_lli_flow_cache_egress_attr_t;


/* LLI flows which binds higher layer ingress processing(Ex: IPsec/GTPU/TWAMP/BFD) */
typedef struct adk_ipv6_dp_lli_flow_cache_ingress_attr_s{
    uint16_t                ingress_flow_statsid;
    uint16_t                vif_flag:1;
    uint16_t                vif_stats_id:15;
    uint32_t                tunnel_if_id;
}adk_ipv6_dp_lli_flow_cache_ingress_attr_t;


/* LLI flows which terminates to FPS */
typedef struct adk_ipv6_dp_lli_flow_cache_ingress_term_attr_s{
    uint16_t                ingress_flow_statsid;
    uint16_t                vif_flag:1;
    uint16_t                vif_stats_id:15;
    adk_ipv6_invoke_cb_attr_t invoke_cb_attr;
}adk_ipv6_dp_lli_flow_cache_ingress_term_attr_t;

/* LLI flows which terminates to FPS */
typedef struct adk_ipv6_dp_lli_flow_cache_ingress_frag_term_attr_s{
    uint16_t                ingress_assoc_rulemask;
    uint16_t                vif_flag:1;
    uint16_t                vif_stats_id:15;
    adk_ipv6_invoke_cb_attr_t invoke_cb_attr;
}adk_ipv6_dp_lli_flow_cache_ingress_frag_term_attr_t;


/********************************************************************************
 *                  IPv6 Fast path flow cache table structures                             *
 ********************************************************************************/
typedef struct adk_ipv6_dp_lli_flow_cache_entry_s{
    uint32_t                idx:16;
    uint32_t                next_idx:16;

    uint8_t                 is_enabled:1;
    uint8_t                 enable_qos:1;
    uint8_t                 reassembly_flag:1;
    uint8_t                 enable_ingress_acl:1;
    uint8_t                 enable_egress_acl:1;
    uint8_t                 enable_policing:1;
    uint8_t                 ing_assoc_lkup_done:1;
    uint8_t                 eg_pktcap_enabled:1;
    uint8_t                 version_id:8;
    uint8_t                 action:8;
    uint8_t                 next_hdr:8;

    /* required for flow cache aging */
    uint8_t                 reserved;
    uint16_t                hash_idx;

    uint8_t                 fc_type;
    uint8_t                 vrd_id;
    uint16_t                ing_ruleset_id;

    uint32_t                src_addr[4];
    uint32_t                dst_addr[4];

    uint16_t                src_port;
    uint16_t                dst_port;

    uint32_t                destFlowId; 
  
    adk_pal_dp_stage_info_t    stageInfo; 
    uint16_t                dest_module_id;

    union {
        adk_ipv6_dp_lli_flow_cache_fwd_attr_t     fwd_bind_attr;
        adk_ipv6_dp_lli_flow_cache_egress_attr_t  egress_bind_attr;
        adk_ipv6_dp_lli_flow_cache_ingress_attr_t ingress_bind_attr;
        adk_ipv6_dp_lli_flow_cache_ingress_term_attr_t ingress_term_attr;
        adk_ipv6_dp_lli_flow_cache_ingress_frag_term_attr_t ingress_frag_term_attr;
    };

    /* for ACL lookup & dscp mapping */
    uint16_t                in_dscp_lkup[IPV6_MAX_DSCP_IDS];
    uint16_t                eg_dscp_lkup[IPV6_MAX_DSCP_IDS];
}__attribute__ ((aligned(64))) adk_ipv6_dp_lli_flow_cache_entry_t;


typedef enum adk_ipv6_dp_uli_flow_cache_actions_s
{
    ADK_IPV6_DP_ULI_FLOW_CACHE_FWD_BIND = 1, /* Egress ULI bind to lower layers - Eth */ 
    ADK_IPV6_DP_ULI_FLOW_CACHE_EGRESS_BIND, /* Egress ULI bind to higher layers - IPsec */
    ADK_IPV6_DP_ULI_FLOW_CACHE_MAX
}adk_ipv6_dp_uli_flow_cache_actions_t;

typedef enum adk_ipv6_dp_uli_flow_cache_type_s
{
    ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_5TUPLE,
    ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_3TUPLE,
    ADK_IPV6_DP_ULI_FLOW_CACHE_TYPE_1TUPLE
}adk_ipv6_dp_uli_flow_cache_type_t;

#if 0
typedef struct adk_ipv6_dp_uli_flow_cache_fwd_attr_s{
    uint16_t                egress_if_id;
    uint16_t                mtu;

}adk_ipv6_dp_uli_flow_cache_fwd_attr_t;

typedef struct adk_ipv6_dp_uli_flow_cache_egress_attr_s{
    uint16_t                egress_if_id;
    uint16_t                mtu;
    adk_handle_t            ipsec_info;
}adk_ipv6_dp_uli_flow_cache_egress_attr_t;
#endif

typedef struct adk_ipv6_dp_uli_flow_cache_entry_s{
    uint32_t                idx:16;
    uint32_t                next_idx:16;

    uint8_t                 is_enabled:1;
    uint8_t                 enable_qos:1;
    uint8_t                 enable_egress_acl:1;
    uint8_t                 mtu_excp:1;
    uint8_t                 eg_pktcap_enabled:1;
    uint8_t                 reserved:3;
    uint8_t                 version_id;
    uint8_t                 action;
    uint8_t                 next_hdr;

    uint16_t                out_acl_statsid;
    uint8_t                 fc_type;
    uint8_t                 vrd_id;

    uint8_t                 outer_vrd_id;
    /* required for flow cache aging */
    uint8_t                 reserved1;
    uint16_t                hash_idx;

    uint32_t                src_addr[4];
    uint32_t                dst_addr[4];

    uint16_t                src_port;
    uint16_t                dst_port;

    uint16_t                egress_if_id;
    uint8_t                 egress_if_type;
    uint8_t                 reserved_1;

    uint16_t                mtu;
    uint16_t                tunnel_if_id;

    adk_handle_t            ipsec_info;


    uint32_t                destFlowId; 
    adk_pal_dp_stage_info_t    stageInfo;
    uint16_t                dest_module_id;

    /* for ACL lookup & dscp mapping */
    uint16_t                eg_dscp_lkup[IPV6_MAX_DSCP_IDS];
}__attribute__ ((aligned(64))) adk_ipv6_dp_uli_flow_cache_entry_t;

typedef struct adk_ipv6_dp_flow_cache_stats_s{
    uint64_t                fc_route_hit;
    uint64_t                fc_term_hit;
    uint64_t                fc_frag_term_hit;
    uint64_t                fc_bind_term_hit;
    uint32_t                fc_cleared;
    uint32_t                fc_miss;
    uint32_t                fc_flows;
}__attribute__ ((aligned(64))) adk_ipv6_dp_flow_cache_stats_t;

/*
 * IPv6 fast-path flow-cache table
 */
typedef struct adk_ipv6_dp_lli_flow_cache_table_s
{
    adk_ipv6_dp_flow_cache_hash_entry_t     *hash_tbl;  
    adk_ipv6_dp_lli_flow_cache_entry_t      *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    adk_ipv6_dp_flow_cache_aging_info_t     age_info;
    adk_ipv6_dp_flow_cache_stats_t          lli_fc_stats;
}adk_ipv6_dp_lli_flow_cache_table_t;


typedef struct adk_ipv6_dp_uli_flow_cache_table_s
{
    adk_ipv6_dp_flow_cache_hash_entry_t     *hash_tbl;  
    adk_ipv6_dp_uli_flow_cache_entry_t      *tbl;
    adk_ipv6_dp_array_list_t                pending_list;
    adk_ipv6_dp_array_list_t                free_list;
    adk_ipv6_dp_flow_cache_aging_info_t     age_info;
    adk_ipv6_dp_flow_cache_stats_t          uli_fc_stats;
}adk_ipv6_dp_uli_flow_cache_table_t;

typedef struct adk_ipv6_dp_ing_flow_stats_s
{
    uint32_t rxPkts;
    uint32_t txPkts;
    uint32_t policerDropPkts;
    uint32_t ipv6ClassifierDroppedPkts;
}__attribute__((aligned(64))) adk_ipv6_dp_ing_flow_stats_t;

typedef void adk_ipv6_ing_classifier_ctx_t;

typedef struct {
        uint32_t cookie;
        /*Two copies: Active & Shadow*/
        adk_ipv6_ing_classifier_ctx_t *ipv6IngClassifierCtx[2];
        uint8_t aclTreeIndex;
        adk_acl_handle_t cpList;
} adk_ipv6_ing_classifer_info_t;

typedef struct ipv6_ing_classifier_flow_info_s {
        uint16_t precedence;
        adk_ipv6_ingress_classifier_rule_t rule;
        adk_ipv6_ingress_classifier_action_t action;
        adk_acl_rule_t cp_rule;
} ipv6_ing_classifier_flow_info_t;

typedef struct {
        uint8_t  proto;
        uint32_t sip0;
        uint32_t sip1;
        uint32_t sip2;
        uint32_t sip3;
        uint32_t dip0;
        uint32_t dip1;
        uint32_t dip2;
        uint32_t dip3;
        uint16_t sPort;
        uint16_t dPort;
        uint32_t ipIntfHandle;
} __attribute__((__packed__)) adk_ipv6_ing_classifier_ptn_t;

/*
 * IPv6 fast-path tables
 */
typedef struct adk_ipv6_dp_tables_s
{
    /*
     * IPv6 egress global stats Table
     */
    adk_ipv6_dp_global_statistics_t     *ipv6_dp_global_stats_table[ADK_MAX_NUM_CORES];

    adk_ipv6_dp_ip_addr_stats_t   **ipv6AddrStatsTble;

    /*
     * Utility counters
     */
     ipv6_utility_counters_t   *ipv6_utility_counters[ADK_MAX_NUM_CORES];

    /*
     * interface tables
     */
    adk_ipv6_dp_interface_table_t       ipv6_dp_if_table;
    /*
     * L4 counters table
     */
    adk_ipv6_dp_l4counter_statistics_t  *ipv6_dp_l4counter_stats_table[ADK_MAX_NUM_CORES];

    /*
     * Tunnel interface table
     */
    adk_ipv6_dp_tunnel_interface_lookup_table_t ipv6_dp_tnl_if_table;
    /*
     * IPv6 virtual interface stats Table
     */
    adk_ipv6_dp_virtual_if_statistics_t     *ipv6_dp_virtual_if_stats_table[ADK_MAX_NUM_CORES];

    /*
     * IPv6 route tables
     */
    adk_ipv6_dp_route_table_t           ipv6_dp_route_table;

    /* 
     * IPv6 nexthop tables
     */
    adk_ipv6_dp_nexthop_table_t         ipv6_dp_nexthop_table;

    /*
     * IPv6 nexthop tables
     */
    adk_ipv6_dp_onlink_dest_table_t     ipv6_dp_onlink_dest_table;

    /*
     * IPv6 reassembly Table
     */
    adk_ipv6_dp_reassembly_table_t      ipv6_dp_reassembly_table;

    /*
     * IPv6 Egress flow Table
     */
    adk_ipv6_dp_egress_flow_attr_table_t    ipv6_dp_egress_flow_attr_table;

    /*
     * IPv6 ingress association Table
     */
    adk_ipv6_dp_ingress_assoc_table_t   ipv6_dp_ingress_assoc_table;

    /*
     * Multicast Group table
     */
    adk_ipv6_dp_ingress_mgroup_table_t   ipv6_dp_ingress_mgroup_table;

    /*
     * IPv6 Address validation table
     */
    adk_ipv6_dp_ingress_flow_sip_table_t    ipv6_dp_ingress_flow_sip_table;

    /*
     * IPv6 VRD table
     */
    adk_ipv6_dp_vrd_table_t             ipv6_dp_vrd_table;

    /*
     * IPv6 Exception Table
     */
    adk_ipv6_dp_excep_table_t           ipv6_dp_excep_table;

    /*
     * IPv6 Destination MTU table
     */
    adk_ipv6_dp_dest_mtu_tbl_list_t     ipv6_dp_dest_mtu_tbl_list;
    /*
     * IPv6 ACL Table
     */
    adk_ipv6_dp_acl_table_t             ipv6_dp_acl_table;

    /*
     * IPV6 PBR Table
     */
    adk_ipv6_dp_pbr_table_t             ipv6_dp_pbr_table;

    /*
     * IPV6 PBR Table
     */
    adk_ipv6_dp_vrd_pbr_ruleset_map_table_t  *ipv6_dp_pbr_ruleset_map_table;

    /*
     *  IPV6 PBR policy set handles
     */
    adk_ipv6_pbr_policy_set_handle_t policySetHandles[ADK_IPV6_PBR_RULE_SETS_MAX];

    /*
     * IPv6 PBR State Table
     */
    adk_ipv6_dp_pbr_state_table_t       ipv6_dp_pbr_state_table;

    /*
     * IPv6 Teredo cli Table
     */
    adk_ipv6_dp_teredo_table_t          ipv6_dp_teredo_table;

    /*
     * IPv6 M->M Bind Table
     */
    adk_ipv6_dp_bind_table_t            ipv6_dp_bind_table;

    /*
     * IPv6 Reassembly Stage info
     */

    adk_pal_dp_stage_info_t             ipv6_dp_reassmbly_stage_info;

    /*
     * IPv6 Flow cache Table
     */
    uint8_t                             ipv6_fc_ver[ADK_IPV6_VRD_ID_MAX];
    adk_ipv6_dp_lli_flow_cache_table_t  ipv6_dp_lli_flow_cache_table[ADK_MAX_NUM_CORES];
    adk_ipv6_dp_uli_flow_cache_table_t  ipv6_dp_uli_flow_cache_table[ADK_MAX_NUM_CORES];
    uint8_t                             enable_hw_cap;
    /*To hold IPv6 ingress assoc stats and  otree information */
    adk_ipv6_dp_ing_flow_stats_t *ipv6IngFlowStats;
    adk_ipv6_ing_classifer_info_t *ingClassifierInfo_p;
    adk_ipv6_dp_ing_classifier_action_info_t *ingClassifierActionDp_p[2];
    /*Classifier tables are maintained for each IP interface separately*/
    adk_ipv6_egress_qos_classifer_info_t *egressQoSClassifierInfo_p[ADK_DP_MAX_IPV6_INTERFACE];
    adk_ipv6_dp_egress_qos_classifier_action_info_t *egressQoSClassifierActionDp_p[ADK_DP_MAX_IPV6_INTERFACE][2];
    adk_handle_t adk_ipv6_pbr_policy_set_handle_t;
    adk_interface_handle_t *ipv6IfaceHandles;
    /*global ipsec configuration parameter */
    uint8_t ipsec_bypass;
    /* global log level parameter */
    uint8_t logLevel;
    /* ipv6 utility counters flag */
    bool isIpv6UtilityCntEnabled;
    /*Staggered reassembly configuration parameters */
    uint32_t reasmTout;
    uint8_t  reasmPrcntToutChange;
    uint32_t reasmMinTout;
    uint32_t reasmStage1;
    uint32_t reasmStage2;
    uint32_t reasmStage3;
    uint32_t reasmStage4;
    uint64_t allowedL4protos[4];
    uint64_t allowedUdpPorts[1024];
    uint8_t  defaultRouteTable;
}adk_ipv6_dp_tables_t;

adk_status_t adk_ipv6_ingress_classifier_create(adk_module_handle_t ipv6Handle,
                                                adk_ipv6_ing_classifer_info_t **ingClassifierInfo_p);

adk_status_t adk_ipv6_ing_classifer_build(adk_module_handle_t ipv6Handle,
                adk_ipv6_ing_classifier_ctx_t *ctx);

adk_status_t adk_ipv6_ing_classifier_hw_rule (
                adk_module_handle_t ipv6Handle,
                uint32_t precedence,
                uint32_t ingClassifierFlowId,
                const adk_ipv6_ingress_classifier_rule_t *ipv6IngClassifierRule_p,
                const adk_ipv6_ingress_classifier_action_t *ipv6IngClassifierAction_p,
                uint8_t treeIndex);

adk_dp_status_t adk_ipv6_dp_ing_classifier_lookup(adk_ipv6_ing_classifier_ctx_t *ipv6IngClassifierCtx_p,
                        adk_ipv6_ing_classifier_ptn_t  *ingClassifierPtn,
			uint32_t  *result_p);

adk_status_t adk_ipv6_dp_ip_addr_stats_get (
                   adk_module_handle_t ipv6Handle,
                   uint16_t ipv6_addr_stats_id,
                   adk_ipv6_address_stats_t *stats_p);



adk_dp_status_t adk_ipv6_dp_table_init(adk_module_handle_t ipv6Handle, adk_ipv6_dp_tables_t *dp_pvt_p);

adk_dp_status_t adk_ipv6_dp_table_release(adk_module_handle_t ipv6Handle, adk_ipv6_dp_tables_t *dp_pvt_p);

adk_status_t adk_ipv6_pbr_config (
                   adk_module_handle_t ipv6Handle,
                   adk_interface_handle_t *ipv6IfHandle,
                   adk_ipv6_pbr_policy_set_handle_t pbrPolicySetHandle);
    
adk_status_t adk_ipv6_vrd_pbr_config (
                  adk_module_handle_t ipv6Handle,
                  uint32_t vrd_id,
                  adk_ipv6_pbr_policy_set_handle_t pbrPolicySetHandle);

adk_status_t adk_ipv6_pbr_policy_set_create (
                  adk_module_handle_t ipv6Handle,
                  adk_ipv6_pbr_policy_set_handle_t *pbrPolicySetHandle_p,
                  uint32_t *pbr_policy_set_id);

adk_status_t adk_ipv6_pbr_policy_set_delete (
                  adk_module_handle_t ipv6Handle,
                  adk_ipv6_pbr_policy_set_handle_t pbrPolicySetHandle);

static inline void adk_ipv6_dp_3tuple_uli_flow_cache_key_generate(void *l3_hdr, 
        uint8_t __attribute__((unused)) next_hdr, 
        uint8_t vrd_id,
        uint32_t *hash_key)
{
    adk_ipv6_hdr_t *ipv6_hdr = (adk_ipv6_hdr_t *) l3_hdr;
    uint64_t hash_value = 0;
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                                 ((vrd_id<<8)));

    *hash_key = hash_value&0xFFFFFFFF;
    return;
}
static inline void adk_ipv6_dp_5tuple_uli_flow_cache_key_generate(void *l3_hdr, 
        uint8_t  next_hdr,
        uint16_t src_port,
        uint16_t dst_port,
        uint8_t  vrd_id,
        uint32_t *hash_key)
{
    adk_ipv6_hdr_t *ipv6_hdr = (adk_ipv6_hdr_t *)l3_hdr;
    uint64_t hash_value = 0;
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->src_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[0]));
    hash_value = SISD_CRC_HASH64(hash_value,
                    *((uint64_t *) &ipv6_hdr->dst_addr[2]));
    hash_value = SISD_CRC_HASH64(hash_value,
                                 (((uint64_t)src_port <<48)|
                                  ((uint64_t)dst_port <<32)|
                                  ((uint64_t)vrd_id<<8)|
                                  next_hdr));
    *hash_key = hash_value&0xFFFFFFFF;

    return;
}

void adk_ipv6_dp_egress_stats(struct rte_mbuf *mbuf, void *data);
void adk_ipv6_dp_slowpath_policer_stats(void *data);
void adk_ipv6_increment_rx_l4_stats(uint8_t next_hdr, uint16_t ipv6_len, int coreIndex);
adk_status_t adk_ipv6_dp_chksum_validate(adk_pktio_task_t *task,  
        adk_ipv6_frag_t frag_flag, uint8_t next_hdr, uint16_t l4_offset, uint16_t dst_port,
        adk_ipv6_hdr_t *ipv6_hdr, adk_ipv6_dp_if_stats_entry_t *if_stats,
        adk_ipv6_dp_interface_entry_t *ipv6_if_entry, bool is_reassembled); 

#ifdef ADK_IPV6_FLOW_CACHE
static inline void adk_ipv6_dp_fc_bitmap_set(uint64_t *array, uint16_t index)
{
    uint64_t bit_set = 1;
    array[(index>>6)] |= (bit_set<<((index)&0x3F)); 
    return;
}
static inline void adk_ipv6_dp_fc_bitmap_reset(uint64_t *array, uint16_t index)
{
    uint64_t bit_reset = 1;
    array[(index>>6)] &= (~(bit_reset<<((index)&0x3F)));
    return;
}
#endif

#if 0 /* TBD: needs to be enabled once 
         the all PH exceptions are moved to PHv3 */
/* IMC Params shared between IPv6->PH path */
typedef struct adk_ipv6_dp_ph_imc_data_s
{
    uint8_t  port;
    uint8_t  exception_type;
    uint8_t  vrd_id;
    uint8_t  reserved;
    uint16_t if_id;
    uint16_t linux_if_id;
    uint16_t flow_id;
    uint16_t context;
    uint16_t param_problem_offset;
}adk_ipv6_dp_ph_imc_data_t;
#endif

#ifdef __cplusplus
}
#endif
#endif /* _ADK_IPV6_DP_H_ */ 
/*! end addtogroup _ipv6
 * @}
 */
