/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2017 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef _ADK_IPV6_DP_ACL_H_
#define _ADK_IPV6_DP_ACL_H_

#include <adk_ipv6.h>
#include <adk_ipv6_acl.h>



static inline adk_dp_status_t
adk_ipv6_acl_lookup_func( adk_pktio_task_t *task, 
                          adk_ipv6_dp_interface_entry_t *ipv6_if_entry, 
                          uint8_t proto, uint16_t dscp, 
                          adk_ipv6_frag_t frag_flag, 
                          adk_ipv6_hdr_t *ipv6_hdr, 
                          uint16_t src_port, uint16_t dst_port,
                          uint8_t  recv_vrd, 
                          uint16_t *stats_id,
                          bool      ingress,
                          uint8_t   domain,
                          int coreIndex)
{
    adk_dp_status_t ret;
    adk_ipv6_acl_val_t acl_index;
    adk_ipv6_dp_acl_ptn_t ipv6_acl_ptn;
    uint16_t ruleset;
    adk_metadata_t *adk_md_p = ADK_METADATA_PTR(task);
    uint8_t action, acl_tree_index;
    adk_ipv6_dp_if_stats_entry_t         *if_stats = NULL;

    acl_tree_index = adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_tree_index;

    if (ingress)
    {
        ruleset = ipv6_if_entry->ingress_ruleset; 
        if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->ingress_stats_id];
    }
    else
    {
        ruleset = ipv6_if_entry->egress_ruleset;
        if_stats = &adk_ipv6_dp_tables->ipv6_dp_if_table.stats_tbl[coreIndex][ipv6_if_entry->egress_stats_id];
    }

    if ( adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[acl_tree_index][ruleset] == NULL)
    {
        ADK_IPV6_DP_LOG_INFO("error: ACL Ruleset is not configured \n");
        return ADK_E_INTERNAL_ERROR;
    }


    ipv6_acl_ptn.proto  = proto;
    ipv6_acl_ptn.dscp   = dscp;
    ipv6_acl_ptn.frag_type = frag_flag;
    rte_memcpy(&ipv6_acl_ptn.sip_addr[0], &ipv6_hdr->src_addr[0], 
            (sizeof(adk_ipv6_addr_t) + sizeof(adk_ipv6_addr_t)));

    ipv6_acl_ptn.srcPort = src_port; 
    ipv6_acl_ptn.destPort = dst_port;

    if(adk_ipv6_acl_lookup(adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_ruleset_tbl[acl_tree_index][ruleset],
                &ipv6_acl_ptn, &acl_index))
    {
        ADK_IPV6_DP_LOG_INFO("error: while ACL lookup \n");
        return ADK_E_INTERNAL_ERROR;
    }

    action = adk_ipv6_dp_tables->ipv6_dp_acl_table.tbl[acl_tree_index][acl_index].action;
    if ( (acl_index) && (adk_ipv6_dp_tables->ipv6_dp_acl_table.tbl[acl_tree_index][acl_index].enable_counter == TRUE)) 
    {
        *stats_id = adk_ipv6_dp_tables->ipv6_dp_acl_table.tbl[acl_tree_index][acl_index].acl_stats_id;
        adk_ipv6_dp_tables->ipv6_dp_acl_table.acl_stats_tbl[(coreIndex*ADK_IPV6_NO_OF_ACL_ENTRIES)+(*stats_id)]++;
    }
    if ( action != ADK_IPV6_FILTER_ACTION_ACCEPT)
    {
        if(ingress)
        {
            ADK_IPV6_DP_LOG_INFO_DEBUG("ACL action is %d \n", action);
            ret = adk_ipv6_dp_handle_in_acl_excep(adk_ipv6_dp_tables->ipv6_dp_acl_table.tbl[acl_tree_index][acl_index],
                    task,
                    ipv6_if_entry->vrd_id,
                    if_stats); 
            if (ret != ADK_SUCCESS)
            {
                return ret;
            }
        } 
        else
        {
            ret = adk_ipv6_dp_handle_out_acl_excep(adk_ipv6_dp_tables->ipv6_dp_acl_table.tbl[acl_tree_index][acl_index],
                        task, recv_vrd, task->port, adk_md_p->input_intf_id,
                        adk_md_p->ip_offset,domain, if_stats);
            if ( ret != ADK_SUCCESS)
            {
                return ret;
            }
        }
        return ADK_E_TASK_CONSUMED; 
    }
    return ADK_SUCCESS;
}

#endif //_ADK_IPV6_DP_ACL_H_
