/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef _ADK_IPV6_DP_COMMON_H_
#define _ADK_IPV6_DP_COMMON_H_

#include <adk_cksum.h>
#include <adk_ipv6_dp_pkt_capture.h>
#include "adk_ph_dev_dp.h"
#include <adk_ethernet_export.h>
#ifdef ADK_MODULE_BFD_ENABLED
#include <adk_bfd_dp_export.h>
#endif
#ifdef ADK_MODULE_TWAMP_ENABLED
#include <adk_twamp_dp_export.h>
#endif
#ifdef ADK_MODULE_IPSEC_ENABLED
#include <adk_ipsec_export.h>
#endif
#ifdef ADK_DEVICE_SNR
#include <adk_hos_dp.h>
#endif


static inline void 
adk_ipv6_dp_lli_imc_fill(adk_metadata_t *adk_md_p, uint8_t dst_mod_id, 
        uint8_t dscp, uint8_t pcp, uint8_t __attribute__((unused)) vrd_id,
        uint8_t hop_limit, uint32_t __attribute__((unused)) tunnel_if_id, 
        adk_ipv6_dp_if_stats_entry_t *if_stats, uint8_t next_hdr,
        uint16_t src_port, uint16_t dst_port, adk_ipv6_frag_t frag_flag)
{
#ifndef ADK_MODULE_BFD_ENABLED
    RTE_SET_USED(hop_limit);
#endif
#ifndef ADK_MODULE_IPSEC_ENABLED
    RTE_SET_USED(vrd_id);
    RTE_SET_USED(tunnel_if_id);
    RTE_SET_USED(src_port);
    RTE_SET_USED(dst_port);
#endif

    /*set the IMC parameter */
    switch(dst_mod_id)
    {
        case ADK_SIF_CFG_PIPELINE_MODULE_ETHERNET_ULI:
            {
                adk_ethernet_dp_uli_imc_param_t *eth_md_p;
                eth_md_p = (adk_ethernet_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
                eth_md_p->dscp = dscp;            
                eth_md_p->pcp = pcp;          
                eth_md_p->is_unfrag_first_frag =((frag_flag==ADK_IPV6_UNFRAGMENTED_PKT)||
                        (frag_flag==ADK_IPV6_FIRST_FRAGMENT))? TRUE: FALSE;
                eth_md_p->l4_proto = next_hdr;

                ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to Ethernet \n");
                ADK_IPV6_DP_LOG_INFO_DEBUG("dscp %d \n", eth_md_p->dscp);
                ADK_IPV6_DP_LOG_INFO_DEBUG("pcp %d \n", eth_md_p->pcp);
                ADK_IPV6_DP_LOG_INFO_DEBUG("is_unfrag_first_frag %d \n", eth_md_p->is_unfrag_first_frag);
                ADK_IPV6_DP_LOG_INFO_DEBUG("l4_proto %d \n", eth_md_p->l4_proto);

            }
            break;

#ifdef ADK_MODULE_IPSEC_ENABLED
        case ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI:
            {
                if_stats->ipv6IfStatsInDelivers++;
                adk_md_p->input_intf_id = tunnel_if_id;
            }
            break;
        case ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI:
            {

                adk_status_t ret;
                adk_ipv6_dp_interface_entry_t *ipv6_if_entry;
                adk_ipsec_dp_uli_imc_param_t  *ipsec_md_p; 
                ipsec_md_p = (adk_ipsec_dp_uli_imc_param_t *) &adk_md_p->imc_param_start;
                /* send the outer PMTU info */
                ret = adk_ipv6_dp_interface_entry_lookup(tunnel_if_id, 0, &ipv6_if_entry);   
                if(ret == ADK_SUCCESS)
                {
                    ipsec_md_p->la_flag = ipv6_if_entry->la_flag;
                    ipsec_md_p->la_flag_version = ipv6_if_entry->la_flag_version;
                }
                else
                {
                    ADK_DP_LOG_INFO(ADK_IPV6_MODULE_ID, 
                            "Invalid tunnel id, pmtu info not "
                            "set from inner header processing \n");
                }

                ipsec_md_p->protocol = next_hdr;
                ipsec_md_p->srcPort = src_port;
                ipsec_md_p->dstPort = dst_port;
                ipsec_md_p->ipv6FragFlag = (frag_flag == ADK_IPV6_UNFRAGMENTED_PKT)? FALSE:TRUE;
                
                ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to IPsec \n");
                ADK_IPV6_DP_LOG_INFO_DEBUG("la_flag %d \n", ipsec_md_p->la_flag);
                ADK_IPV6_DP_LOG_INFO_DEBUG("la_flag_version %d \n", ipsec_md_p->la_flag_version);
                ADK_IPV6_DP_LOG_INFO_DEBUG("protocol %d \n", ipsec_md_p->protocol);
                ADK_IPV6_DP_LOG_INFO_DEBUG("ipv6FragFlag %d \n", ipsec_md_p->ipv6FragFlag);
                ADK_IPV6_DP_LOG_INFO_DEBUG("srcPort %d \n", ipsec_md_p->srcPort);
                ADK_IPV6_DP_LOG_INFO_DEBUG("dstPort %d \n", ipsec_md_p->dstPort);
            }
            break;  
#endif
#ifdef ADK_MODULE_TWAMP_ENABLED
        case ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_LLI:
            {
                if_stats->ipv6IfStatsInDelivers++;
                adk_twamp_dp_lli_imc_data_t *twamp_md_p;
                twamp_md_p = (adk_twamp_dp_lli_imc_data_t *) &adk_md_p->imc_param_start;
                twamp_md_p->path = ADK_TWAMP_DP_IPV6_LLI_PATH;
                twamp_md_p->ip_info_u.ipv6_encap_info_p.dscp = dscp; 
                twamp_md_p->ip_info_u.ipv6_encap_info_p.ipVrdId = vrd_id;

                ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to TWAMP \n");
                ADK_IPV6_DP_LOG_INFO_DEBUG("path %d\n", twamp_md_p->path);
                ADK_IPV6_DP_LOG_INFO_DEBUG("dscp %d\n", twamp_md_p->ip_info_u.ipv6_encap_info_p.dscp);
                ADK_IPV6_DP_LOG_INFO_DEBUG("vrd_id %d\n", twamp_md_p->ip_info_u.ipv6_encap_info_p.ipVrdId);
            }
            break;
#endif
#ifdef ADK_MODULE_BFD_ENABLED
        case ADK_SIF_CFG_PIPELINE_MODULE_BFD_LLI:
            {
                if_stats->ipv6IfStatsInDelivers++;
                adk_bfd_dp_lli_imc_data_t *bfd_md_p;
                bfd_md_p = (adk_bfd_dp_lli_imc_data_t *) &adk_md_p->imc_param_start;
                bfd_md_p->ttl = hop_limit;

                ADK_IPV6_DP_LOG_INFO_DEBUG("IMC info to BFD \n");
                ADK_IPV6_DP_LOG_INFO_DEBUG("ttl %d\n", bfd_md_p->ttl);
            }
            break;
#endif
    }

    return;
}

#ifdef ADK_HOS_OLD_IMPL
static inline uint16_t 
adk_ipv6_udp_checksum_generation_verify(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr)
{
    uint32_t cksum=0;
    uint16_t udp_len, total_len=0, len;
    uint8_t *buff, nb_segs = task->nb_segs;
    bool odd_len = FALSE;

    adk_metadata_t *adk_md_p;
    adk_ipv6_dp_internal_imc_data_t *adk_int_md_p;
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
   
    udp_len = task->pkt_len - l4_offset;
    len  = task->data_len - l4_offset;
    buff = rte_pktmbuf_mtod_offset(task, uint8_t *, l4_offset);

    adk_md_p = ADK_METADATA_PTR(task);
    adk_int_md_p = (adk_ipv6_dp_internal_imc_data_t *)&adk_md_p->imc_param_start;
    if( !adk_int_md_p->is_reassembled )
    {
        if(adk_hos_is_l4_chksum_err(dsi_md_p))
        {
            return 1;
        }
        return 0;
    }
    else
    {
        cksum = adk_htons(dsi_md_p->checksum);
        cksum += (*((uint16_t *) &buff[6]));

        /* parse initial fragment & update the len & buff address */
        total_len = adk_int_md_p->initial_frag_len;
        while(len<= adk_int_md_p->initial_frag_len)
        {
            task = (adk_pktio_task_t *)task->next;
            len  += task->data_len;
        }

        /* update the buff & segment length */
        buff = rte_pktmbuf_mtod(task, uint8_t *);
        len = task->data_len;
    }

    if(nb_segs > 1)
    {
        while((total_len+len) <udp_len)
        {
            if(odd_len)
            {
                cksum += rte_bswap16(adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len-1)&1;
            }
            else
            {
                cksum += (adk_raw_cksum((void *)(buff), len));
                odd_len = (task->data_len)&1;
            }
            total_len += len;

            task = (adk_pktio_task_t *)task->next;
            len  = task->data_len;
            buff = rte_pktmbuf_mtod(task, uint8_t *);
        }
        if(odd_len)
        {
            cksum += rte_bswap16(adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
        else
        {
            cksum += (adk_raw_cksum((void *)(buff), (udp_len-total_len)));
        }
    }
    else
    {
        cksum = adk_raw_cksum((void *)(buff), len);
    }

    cksum += rte_ipv6_phdr_cksum((void *)ipv6_hdr, task->ol_flags);
    cksum = ((cksum & 0xffff0000) >> 16) + (cksum & 0xffff);
    cksum = (~cksum) & 0xffff;

    return cksum;
}

static inline uint16_t 
adk_ipv6_hw_udp_checksum_gen(adk_pktio_task_t  *task,
              adk_ipv6_hdr_t  *ipv6_hdr)
{
    uint32_t cksum;
    task->ol_flags |= (PKT_TX_IPV6|PKT_TX_UDP_CKSUM);

    /* all encapsulated scenarios doesn't have extension header 
     * TODO need to update for frag scenario */
    task->l3_len = ADK_IPV6_HDR_LEN; 
    cksum = rte_ipv6_phdr_cksum((void *)ipv6_hdr, task->ol_flags);
    return cksum;
}
#endif

static inline adk_dp_status_t 
adk_validate_l4_length(uint8_t next_hdr, uint16_t length)
{

    switch(next_hdr)
    {
        case ADK_IPV6_PROTOCOL_ICMPV6:
            if(length < ADK_IPV6_ICMPV6_HDR_LEN)
                return ADK_E_INTERNAL_ERROR;
            break;

       case ADK_IPV6_PROTOCOL_SCTP:
            if(length < ADK_IPV6_SCTP_HDR_LEN)
                return ADK_E_INTERNAL_ERROR;
            break;

       case ADK_IPV6_PROTOCOL_TCP:
            if(length < ADK_IPV6_TCP_HDR_LEN)
                return ADK_E_INTERNAL_ERROR;
            break;

       case ADK_IPV6_PROTOCOL_UDP_LITE:
            if(length < ADK_IPV6_UDP_LITE_HDR_LEN)
                return ADK_E_INTERNAL_ERROR;
            break;

       default: 
            break;
    }

    return ADK_SUCCESS;
}

static inline uint8_t 
adk_ipv6_parse_l4_ports(uint16_t pdu_length, uint8_t next_header, uint8_t *pdu, uint16_t *src_port,
        uint16_t *dst_port, uint8_t *l4_proto, adk_ipv6_frag_t *frag_flag, bool *l4_port_present, 
        uint16_t *offset_to_frag, uint32_t *frag_id, uint8_t *parse_err)
{

    uint8_t l4_offset = 0, count = 0;
    uint16_t ext_hdr_len;
    uint8_t max_ext_supported = ADK_IPV6_MAX_EXTNS_SUPPORTED;

    if((next_header == ADK_IPV6_NEXT_HDR_UDP) || 
            (next_header == ADK_IPV6_NEXT_HDR_TCP) ||
            (next_header == ADK_IPV6_NEXT_HDR_UDP_LITE) ||
            (next_header == ADK_IPV6_NEXT_HDR_SCTP))
    {
        l4_offset = /*IPV6_BASIC_HDR_LEN*/ 40;

        *src_port = (*((uint16_t *)&pdu[40]));
        *dst_port = (*((uint16_t *)&pdu[42]));
        *l4_proto = next_header;
        *l4_port_present = TRUE;

        ADK_IPV6_DP_LOG_DEBUG("L4 proto %d L4 ports src:0x%x dest:0x%x \n",
                *l4_proto, *src_port, *dst_port);

        return l4_offset;
    }
   
    pdu += 40;
    l4_offset +=40;
    do{
        max_ext_supported--;
        switch (next_header)
        {
            case ADK_IPV6_NEXT_HDR_UDP:
            case ADK_IPV6_NEXT_HDR_TCP:
            case ADK_IPV6_NEXT_HDR_UDP_LITE:
            case ADK_IPV6_NEXT_HDR_SCTP:
                {
                    *src_port = (*((uint16_t *)&pdu[0]));
                    *dst_port = (*((uint16_t *)&pdu[2]));
                    if (((*frag_flag) == ADK_IPV6_UNFRAGMENTED_PKT) || 
                            ((*frag_flag) == ADK_IPV6_DP_FIRST_FRAGMENT))
                    {
                        *l4_port_present = TRUE;
                        *l4_proto = next_header; 
                    }
                    else
                    {
                        l4_offset = *offset_to_frag;
                    }

                    ADK_IPV6_DP_LOG_DEBUG("L4 proto %d L4 ports src:0x%x dest:0x%x \n",
                            *l4_proto, *src_port, *dst_port);
                    if (count)
                    {
                        *parse_err = ADK_IPV6_HOP_BY_HOP_EXTENSION_HEADER_FOR_NON_ICMPV6_PROTOCOL;
                    }
                    return l4_offset;
                }
            case ADK_IPV6_PROTOCOL_ICMPV6:
                {
                    *src_port = rte_cpu_to_be_16((uint16_t)pdu[0]);
                    *dst_port = rte_cpu_to_be_16((uint16_t)pdu[1]);
                    *l4_proto = next_header;

                    ADK_IPV6_DP_LOG_DEBUG("L4 proto %d L4 ports src:0x%x dest:0x%x \n",
                            *l4_proto, *src_port, *dst_port);
                    return l4_offset;
                }
            case ADK_IPV6_NEXT_HDR_HOP_BY_HOP:
                {
                    next_header = pdu[0];
                    ext_hdr_len = ADK_IPV6_GET_EXT_HDR_LEN(pdu[1]);
                    if((pdu_length-l4_offset)<ext_hdr_len)
                    {
                        *parse_err = ADK_IPV6_HDR_ERR_EXCEP;
                        return l4_offset;
                    }
                    if (count >= 1)
                    {
                        *parse_err = ADK_IPV6_EXCEPTION_EXT_HEADER_OCCURED_TWICE;
                        return l4_offset;
                    }
                    count++;
                    if(max_ext_supported != (ADK_IPV6_MAX_EXTNS_SUPPORTED - 1))
                    {
                        if ( *frag_flag != ADK_IPV6_DP_FIRST_FRAGMENT)
                        {
                            *parse_err = ADK_IPV6_HOP_BY_HOP_EXTENSION_NOT_FIRST_EXTENSION_HEADER;
                            return l4_offset;
                        }   
                    }

                    l4_offset += ext_hdr_len;/* initial 8 octets */
                    pdu += ext_hdr_len;/* initial 8 octets */

                    break;
                }
            case ADK_IPV6_NEXT_HDR_ROUTING:
                {
                    next_header = pdu[0];
                    /* check if segments left is non-zero, then raise an exception */
                    if(pdu[3])
                    {
                         if ( *frag_flag != ADK_IPV6_DP_FIRST_FRAGMENT)
                         {
                             *parse_err = ADK_IPV6_EXCEPTION_SEGMENTS_LEFT_NON_ZERO;
                         }
                    }
                    ext_hdr_len = ADK_IPV6_GET_EXT_HDR_LEN(pdu[1]);
                    if((pdu_length-l4_offset)<ext_hdr_len)
                    {
                        *parse_err = ADK_IPV6_HDR_ERR_EXCEP;
                        return l4_offset;
                    }

                    l4_offset += ext_hdr_len;/* initial 8 octets */
                    pdu += ext_hdr_len;/* initial 8 octets */
                    break;
                }
            case ADK_IPV6_NEXT_HDR_DST_OPTIONS:
            case ADK_IPV6_NEXT_HDR_MOB_TYPE:
                {
                    next_header = pdu[0];
                    ext_hdr_len = ADK_IPV6_GET_EXT_HDR_LEN(pdu[1]);
                    if((pdu_length-l4_offset)<ext_hdr_len)
                    {
                        *parse_err = ADK_IPV6_HDR_ERR_EXCEP;
                        return l4_offset;
                    }

                    l4_offset += ext_hdr_len;/* initial 8 octets */
                    pdu += ext_hdr_len;/* initial 8 octets */
                    break;
                }
            case ADK_IPV6_NEXT_HDR_FRAGMENT:
                {
                    *l4_proto = pdu[0];
                    *frag_id   = *((uint32_t *)&pdu[4]);
                    *offset_to_frag = l4_offset;
                    if(adk_ntohs(((*(uint16_t*) &pdu[2]))) &0xFFF8)
                    {
                        *frag_flag = ADK_IPV6_DP_NON_FIRST_FRAGMENT;
                        return (l4_offset);
                    }
                    else
                    {
                        *frag_flag = ADK_IPV6_DP_FIRST_FRAGMENT;
                        next_header = pdu[0];
                        pdu += 8;/* initial 8 octets */
                        l4_offset += 8/* initial 8 octets */;
                        break;
                    }
                }
            case ADK_IPV6_NEXT_HDR_ESP:
            case ADK_IPV6_NEXT_HDR_AH:
                {
                    /* payload is encrypted/Authenticated, 
                     * updated next header as ESP/AH & corresponding L4 offset */
                    *l4_proto = next_header;
                    if (count)
                    {
                        *parse_err = ADK_IPV6_HOP_BY_HOP_EXTENSION_HEADER_FOR_NON_ICMPV6_PROTOCOL;
                    }
                    return (l4_offset);
                }
            case ADK_IPV6_NEXT_HDR_NO_NEXT_HDR:
                {
                    ADK_IPV6_DP_LOG_DEBUG("no next header \n");
                    if ( *frag_flag != ADK_IPV6_DP_FIRST_FRAGMENT)
                    {
                        *parse_err = ADK_IPV6_EXCEPTION_PARAM_PROBLEM;
                    }
                    *l4_proto = next_header;
                    return (l4_offset);
                }
            default:
                {
                    ADK_IPV6_DP_LOG_WARNING("invalid next header \n");
                    *l4_proto = next_header;
                    return (l4_offset);
                }
        }
    }while(max_ext_supported);

    ADK_IPV6_DP_LOG_WARNING("too many extension headers. extension headers parsed %d \n", 
            (ADK_IPV6_MAX_EXTNS_SUPPORTED-max_ext_supported));

    if ( *frag_flag != ADK_IPV6_DP_FIRST_FRAGMENT)
    {
        *parse_err = ADK_IPV6_EXCEPTION_TOO_MANY_EXTENSION_HEADERS;
    }
    *l4_proto = next_header;
    return (l4_offset);
}
static inline uint8_t 
adk_ipv6_parse_header_exception(uint8_t next_header, uint8_t *pdu, uint8_t *parse_err)
{

    uint8_t l4_offset = 0;
    uint8_t max_ext_supported = ADK_IPV6_MAX_EXTNS_SUPPORTED;

    if((next_header == ADK_IPV6_NEXT_HDR_UDP) || 
            (next_header == ADK_IPV6_NEXT_HDR_TCP) ||
            (next_header == ADK_IPV6_NEXT_HDR_UDP_LITE) ||
            (next_header == ADK_IPV6_NEXT_HDR_SCTP))
    {
        l4_offset = /*IPV6_BASIC_HDR_LEN*/ 40;
        return l4_offset;
    }
   
    pdu += 40;
    l4_offset +=40;
    do{
        max_ext_supported--;

        switch (next_header)
        {
            case ADK_IPV6_NEXT_HDR_UDP:
            case ADK_IPV6_NEXT_HDR_TCP:
            case ADK_IPV6_NEXT_HDR_UDP_LITE:
            case ADK_IPV6_NEXT_HDR_SCTP:
            case ADK_IPV6_PROTOCOL_ICMPV6:
                {
                    return l4_offset;
                }
            case ADK_IPV6_NEXT_HDR_HOP_BY_HOP:
                {
                    next_header = pdu[0];

                    l4_offset += (pdu[1]<<2) + 8;/* initial 8 octets */
                    pdu = pdu + ((pdu[1]<<2) + 8);/* initial 8 octets */
                    if(max_ext_supported != (ADK_IPV6_MAX_EXTNS_SUPPORTED - 1))
                    {
                        *parse_err = ADK_IPV6_EXCEPTION_PARAM_PROBLEM;
                    }
                    break;
                }
            case ADK_IPV6_NEXT_HDR_ROUTING:
                {
                    next_header = pdu[0];

                    /* check if segments left is non-zero, then raise an exception */
                    if(pdu[3])
                    {
                        *parse_err = ADK_IPV6_EXCEPTION_PARAM_PROBLEM;
                    }

                    l4_offset += (pdu[1]<<2) + 8/* initial 8 octets */;
                    pdu = pdu + ((pdu[1]<<2) + 8);/* initial 8 octets */;
                    break;
                }
            case ADK_IPV6_NEXT_HDR_DST_OPTIONS:
            case ADK_IPV6_NEXT_HDR_MOB_TYPE:
                {
                    next_header = pdu[0];
                    l4_offset += (pdu[1]<<2) + 8/* initial 8 octets */;
                    pdu = pdu + ((pdu[1]<<2) + 8);/* initial 8 octets */;
                    break;
                }
            case ADK_IPV6_NEXT_HDR_FRAGMENT:
                {
                    if(adk_ntohs(((*(uint16_t*) &pdu[2]))) &0xFFF8)
                    {
                        return (l4_offset);
                    }
                    else
                    {
                        next_header = pdu[0];
                        pdu += 8;/* initial 8 octets */
                        l4_offset += 8/* initial 8 octets */;
                        break;
                    }
                }
            case ADK_IPV6_NEXT_HDR_ESP:
            case ADK_IPV6_NEXT_HDR_AH:
                {
                    /* payload is encrypted/Authenticated, 
                     * updated next header as ESP/AH & corresponding L4 offset */
                    return (l4_offset);
                }
            case ADK_IPV6_NEXT_HDR_NO_NEXT_HDR:
                {
                    ADK_IPV6_DP_LOG_INFO_DEBUG("no next header \n");
                    *parse_err = ADK_IPV6_EXCEPTION_PARAM_PROBLEM;
                    return (l4_offset);
                }
            default:
                {
                    ADK_IPV6_DP_LOG_INFO_DEBUG("invalid next header \n");
                    *parse_err = ADK_IPV6_EXCEPTION_PARAM_PROBLEM;
                    return (l4_offset);
                }
        }
    }while(max_ext_supported);


    ADK_IPV6_DP_LOG_INFO_DEBUG("too many extension headers. extension headers parsed %d \n", 
            (ADK_IPV6_MAX_EXTNS_SUPPORTED-max_ext_supported));

    *parse_err = ADK_IPV6_EXCEPTION_TOO_MANY_EXTENSION_HEADERS;
    return (l4_offset);
}

#endif //_ADK_IPV6_DP_COMMON_H_
