/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IPV6_DP_EXPORT_H__
#define __ADK_IPV6_DP_EXPORT_H__

typedef struct adk_ipv6_hdr_s{
    uint32_t    ver_tc_flabel;
    uint32_t    len_nh_hlimit;
    uint32_t    src_addr[4];
    uint32_t    dst_addr[4];
}adk_ipv6_hdr_t;

/*To identify the packet originated path*/
#define IP6_ULI_SENDER_EO    0x01
#define IP6_ULI_SENDER_NETD  0x02
#define IP6_ULI_SENDER_IPSEC 0x03
#define IP6_ULI_SENDER_IP6LLI 0x04

#define ADK_IPV6_GET_UDP_CHECKSUM(ipv6_hdr, l4_offset) \
                               (*((uint16_t *)(((uint8_t *)ipv6_hdr)+l4_offset+6)))
/*!
 * =============================================================
 * @struct adk_ipv6_dp_lli_imc_data_t adk_ipv6_dp_export.h "adk_ipv6_dp_export.h"
 *
 * @brief 
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPv4 module LLI for the purpose of packet processing.
 *
 * =============================================================
 */
typedef struct adk_ipv6_dp_lli_imc_data_s
{
     /* DSCP remapping profile ID received from Ethernet */
    uint8_t profile_id;
     /* PCP value received from Ethernet */
    uint8_t pcp;
    /* IPSec module should set this flag to one of the following values:
     * {IPSEC_DONE_PLAINTEXT_PACKET, IPSEC_DONE_PACKET_DECRYPTED}.
     * Other modules (e.g. ethernet) should set it to IPSEC_NOT_VISITED while calling IP LLI function
     */
    uint8_t ipsec_data; /* 1B */
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ipv6_dp_lli_imc_data_t;


/*!
 * =============================================================
 * @enum adk_ipv6_dp_uli_imc_path_t
 *
 * @brief
 *
 * This enumeration defines different injection paths in IP ULI.
 * =============================================================
 */
typedef enum adk_ipv6_dp_uli_imc_path_s {

    /*  Fast path injection with IP header present in pdu. */
    ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR = 0,
    /*  Fast path injection with partial IP header present in pdu(L3/L4 checksum needs to be updated. */
    ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR_UPDATE_CKSUM = 1,
    /*  Fast path injection & IP/UDP header to be encapsulated using the
     *  destination flow from bind */
    ADK_IPV6_DP_ULI_PATH_FPI_ENCAP_WITH_DSTFLOW = 2,
    /*  Fast path injection & IP/UDP header to be encapsulated using the
     *  adk_ipv6_dp_udp_encap_info_t
     *  This path will be deprecated */
    ADK_IPV6_DP_ULI_PATH_FPI_ENCAP_WITH_INFO = 3,
    /*  Linux injected path.*/
    ADK_IPV6_DP_ULI_PATH_LINUX_INJECTED = 4,
} __attribute__ ((__packed__)) adk_ipv6_dp_uli_imc_path_t;

/*!
 * =============================================================
 * @struct adk_ip_dp_uli_imc_with_hdr_info_t
 *
 * @brief
 *
 * The data structure defines the information associated with IP ULI injection
 * path ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR.
 * =============================================================
 */
typedef struct adk_ipv6_dp_uli_imc_with_hdr_info_s
{
    /* The Routing Domain ID
     */
    uint32_t ipVrdId;
    uint16_t outerPathMtuInfo;
    uint16_t tunnelIfId;
    uint16_t laFlag:1;
    uint16_t laFlagVersion:15;
}adk_ipv6_dp_uli_imc_with_hdr_info_t;

/*!
 * =============================================================
 * @struct adk_ipv6_dp_uli_imc_with_hdr_info_t
 *
 * @brief
 *
 * The data structure defines the information associated with IP ULI injection
 * path ADK_IPV6_DP_ULI_PATH_FPI_WITH_HDR_UPDATE.
 * =============================================================
 */
typedef struct adk_ipv6_dp_uli_imc_with_hdr_upt_info_s
{
    /* The Routing Domain ID
     */
    uint32_t ipVrdId;
}adk_ipv6_dp_uli_imc_with_hdr_upt_info_t;

/*!
 * =============================================================
 * @struct adk_ipv6_dp_uli_imc_with_hdr_info_t
 *
 * @brief
 *
 * The data structure defines the information associated with IP ULI injection
 * path ADK_IPV6_DP_ULI_PATH_FPI_ENCAP_WITH_DSTFLOW.
 * =============================================================
 */
typedef struct adk_ipv6_dp_uli_imc_encap_with_dstflow_info_s
{
    bool isPriBitsEnabled;
    /* IP TOS value to be used in case of IP header encapsulation by IP module.
     */
    uint8_t traffiClass;
}adk_ipv6_dp_uli_imc_encap_with_dstflow_info_t;

/*!
 * =============================================================
 * @enum adk_ipv6_interface_type_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This enumeration defines the different types of ipv6 interfaces.
 *
 * =============================================================
 */
typedef enum adk_ipv6_interface_type_e {
    /*!< IPv6 interface over Ethernet */
    IPV6_INTERFACE_TYPE_ETH = 0,
    /*!< IPv6 tunnel interface */
    IPV6_INTERFACE_TYPE_TUNNEL,
    /*!< IPv6 virtuall interface */
    IPV6_INTERFACE_TYPE_VIRTUAL
} adk_ipv6_interface_type_t;

/*!
 * =============================================================
 * @struct adk_ipv6_dp_uli_imc_data_t adk_ipv6_dp_export.h "adk_ipv6_dp_export.h"
 *
 * @brief 
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPv4 module ULI for the purpose of packet processing.
 *
 * =============================================================
 */
typedef struct adk_ipv6_dp_uli_imc_data_s
{
    /* To identify the Injection Path
     * Value can be : IP6_ULI_SENDER_EO, IP6_ULI_SENDER_NETD, IPV6_ULI_SENDER_IPSEC,
     * IP6_ULI_SENDER_IP6LLI */
    uint8_t sender;
    uint8_t pktOrigin;
    /* In outbound direction, if the IPsec SA mode configuration is inline, then
     * IPsec will be setting do_not_post_frag flag in IPv6 IMC because inline mode and
     * postFrag won’t go together. IPv6 can do fragmentation on the packet in ULI only
     * when this flag is not set. If do_not_post_frag flag is set and fragmentation has
     * to be done on packet in IPv6, then packet has to be dropped.
     */
    bool do_not_post_frag;
    /* L4 protocol of inner IPv6 header.
     * This data would be filled up by IPsec ULI.
     * IPv6 module should not use inner_l4_protocol if innerIPVersion in adk_metadata_t
     * is 0.
     */
    uint8_t inner_l4_protocol;

    /* Ethernet expects number of fragments created in its IMC.
     * In pre-frag case, this is generated by IPSec.
     * IPSec should set it in IPv6 IMC and IP will send it to ethernet
     */
    uint8_t frag_count;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ipv6_dp_uli_imc_data_t;

adk_status_t adk_ipv6_dp_pkt_proc_lli (void **task_p,
                      uint8_t task_count, uint32_t task_mask, void *ring, 
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

adk_status_t adk_ipv6_dp_pkt_proc_uli (void **task_p,
                      uint8_t task_count, uint32_t task_mask, void *ring,
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);


adk_status_t adk_ipv6_dp_pkt_proc_netd(void  **task_p,
                        uint8_t  task_count,
                        uint32_t  task_mask,
                        void  *ring,
                        adk_sif_cfg_pipeline_info_t  *next_stage_entry, 
                        int coreIndex);

adk_status_t adk_ipv6_dp_pkt_proc_packet_capture (void **task_p,
                      uint8_t task_count, uint32_t task_mask, void *ring, 
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

void
adk_ipv6_dp_flow_cache_monitor(void);

uint16_t       
adk_ipv6_udp_checksum_generation(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr);

uint16_t
adk_ipv6_udp_checksum_validation(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr);

uint32_t 
adk_ipv6_phdr_cksum(adk_ipv6_hdr_t  *ipv6_hdr,
		        uint16_t udp_len);

uint16_t
adk_ipv6_icmp6_checksum(adk_pktio_task_t  *task,
              uint16_t  l4_offset, adk_ipv6_hdr_t  *ipv6_hdr, uint16_t ip_offset);

uint32_t
adk_icmpv6_phdr_cksum(adk_ipv6_hdr_t  *ipv6_hdr,
                          uint32_t ipv6_len);

#ifdef UDP_CHECKSUM_STAGE_AFTER_BB_IC_MODULE
adk_status_t adk_udp_dp_pkt_proc_uli (void **task_p,
                        uint8_t task_count,
                        uint32_t task_mask,
                        __attribute__((unused)) void *ring,
                        __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        int coreIndex);

adk_status_t adk_udp_dp_pkt_proc_ring_uli (void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask,
                        void *ring,
                        adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);
#endif
adk_status_t adk_ipv6_dp_pkt_proc_em_uli(void ** task);

/* This callback will be called whenever timer started from IPSec is deleted.
 * res_index and bucket_index should be used to identify exact reassembly context in IPSec.
 * IP will acquire lock before calling this callback.
 */
typedef uint16_t (*clear_ipsec_ctx_for_hash)(uint16_t res_index, uint8_t bucket_index);

/* adk_ipv6_dp_ipsec_start_reassembly : This API will find the exact bucket for a given res_index and start the timer.
 * 
 * On success, IP will return the bucket index that should be used by ipsec for flow related metadata and parking the tasks.
 * If return value is not ADK_SUCCESS, bucket_index will be invalid and shouldn't not be used for any
 * further processing.
 */     
inline adk_status_t adk_ipv6_dp_ipsec_start_reassembly(uint16_t coreIndex,
        adk_pktio_task_t **ph_task_p,
        uint16_t *res_index,
        uint16_t ipv6_offset,
        clear_ipsec_ctx_for_hash ipsecCb,
        uint8_t * bucket_index,
        uint32_t preFragBufferTimeOutVal 
        );
/* adk_ipv6_dp_acquire_reassembly_lock : This API should be called to acquire the lock 
 */    
inline void adk_ipv6_dp_acquire_reassembly_lock(
        uint16_t res_index
);  
/* adk_ipv6_dp_release_reassembly_lock : This API should be called to release the lock 
 */ 
inline void adk_ipv6_dp_release_reassembly_lock(
        uint16_t res_index
);

void adk_ipv6_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#if 0
void adk_dummy_print(uint16_t hashIndex, uint8_t bucketIndex);
#endif

#endif /* __ADK_IPV6_DP_EXPORT_H__ */
