/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2017 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef _ADK_IPV6_DP_PBR_H_
#define _ADK_IPV6_DP_PBR_H_

#include <adk_ipv6.h>
#include <adk_ipv6_pbr_acl.h>
#include <adk_ipv6_dp_lli.h>
#include <adk_ipv6_dp_common.h>

static inline int adk_ipv6_dp_pbr_lookup(adk_ipv6_pbr_tree_t *pbr_tree,
		           adk_ipv6_pbr_ptn_t  *pbr_ptn,
		           adk_ipv6_pbr_val_t  *pbr_val)
{
    int ret;
    const uint8_t * data[] = {(uint8_t *)pbr_ptn};

    ret = rte_acl_classify_alg((struct rte_acl_ctx *)pbr_tree, data,
                               (uint32_t *) pbr_val, 1, 1,
                               RTE_ACL_CLASSIFY_SSE);
    if (ret != 0)
    {
        ADK_IPV6_DP_LOG_INFO_1D("Error in IPv6 PBR ACL classify: %d", ret);
        return ADK_E_INTERNAL_ERROR;
    }
    
    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_pbr()
 * 
 * Performs the pbr lookup & stores the action table entry 
 * index in the passed pointer.
 * The possible return values
 * TRUE: If PBR lookup successfull
 * FALSE: If PBR lookup fails and fall-back to DIP based lookup
 * =============================================================
 */
static inline adk_status_t adk_ipv6_dp_pbr(adk_pktio_task_t *task, uint8_t pcp, uint8_t protocol,
                     adk_ipv6_hdr_t *ipv6_hdr, uint16_t src_port, uint16_t dest_port,
                     uint8_t dscp, adk_ipv6_frag_t frag_flag, adk_ipv6_dp_if_stats_entry_t *if_stats,
                     adk_ipv6_dp_route_entry_t __attribute__((unused))**route_entry,
                     adk_ipv6_dp_interface_entry_t *ipv6_if_entry,
                     uint8_t excep, uint16_t l4_offset, uint32_t frag_id,
                     uint16_t offset_to_frag, int coreIndex)
{
    adk_status_t ret;
    adk_ipv6_pbr_ptn_t pbr_ptn;
    adk_ipv6_dp_pbr_action_entry_t       *pbr_action = NULL;
    adk_ipv6_pbr_val_t pbr_val =0;
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task);
    ipv6_pbr_rule_set_info_t *policy_info_p;

    /* PBR enabled, but ruleset is not mapped to interface */
    if(ipv6_if_entry->pbr_policy_set_id == 0)
    {
        ADK_IPV6_DP_LOG_INFO("PBR enabled, but ruleset not mapped!\n");

        if(unlikely(NULL == *route_entry))
        {
           ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 route lookup failed \n");

           /* increment the stats */
           if_stats->ipv6IfStatsInForwDatagrams++;
           if_stats->ipv6IfStatsInNoRoutes++;

           /* raise an exception "Route not found" */
           ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                            adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, 
                            ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                            ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND, coreIndex, 0); 
           if(ret != ADK_SUCCESS)
           {
              ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                             "ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND \n");
              return ADK_E_DROP_TASK;
           }
           return ADK_E_TASK_CONSUMED;
        }
        else
        {
            return ADK_SUCCESS; 
        }      
    }
    
    policy_info_p = (ipv6_pbr_rule_set_info_t *)
                    (adk_ipv6_dp_tables->policySetHandles[((ipv6_if_entry->pbr_policy_set_id)-1)&ADK_IPV6_PBR_RULE_SETS_MASK]);

    /* PBR enabled, but mapped ruleset to interface is invalid */
    if(!policy_info_p)
    {
        ADK_IPV6_DP_LOG_INFO("PBR enabled, but mapped to invalid ruleset!\n");
        if(unlikely(NULL == *route_entry))
        {
           ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 route lookup failed \n");

           /* increment the stats */
           if_stats->ipv6IfStatsInForwDatagrams++;
           if_stats->ipv6IfStatsInNoRoutes++;

           /* raise an exception "Route not found" */
           ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                            adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, 
                            ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                            ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND, coreIndex, 0); 
           if(ret != ADK_SUCCESS)
           {
              ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                             "ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND \n");
              return ADK_E_DROP_TASK;
           }
           return ADK_E_TASK_CONSUMED;
        }
        else
        {
            return ADK_SUCCESS; 
        }      
    }

    (void)pcp;
    (void)protocol;

    /* Copy source and destination IPv6 addresses together */
    rte_memcpy(&pbr_ptn.sip_addr[0], 
               &ipv6_hdr->src_addr[0], 
               (sizeof(adk_ipv6_addr_t) + sizeof(adk_ipv6_addr_t)));
    (void)src_port;
    (void)dest_port;
    (void)dscp;
    (void)frag_flag;
    /* lookup PBR ACL */
    ret = adk_ipv6_dp_pbr_lookup(policy_info_p->pbr_tree[policy_info_p->pbr_tree_index],
                            &pbr_ptn, &pbr_val);
    if(unlikely(!ret || !(pbr_val)))
    {
        ADK_IPV6_DP_LOG_DEBUG("Entry not found in PBR ACL tree, fall back on DIP based lookup!");
        /*PBR lookup failed, fall-back to DIP based lookup */
        if(unlikely(NULL == *route_entry))
        {
           ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 route lookup failed \n");

           /* increment the stats */
           if_stats->ipv6IfStatsInForwDatagrams++;
           if_stats->ipv6IfStatsInNoRoutes++;

           /* raise an exception "Route not found" */
           ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
                            adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, 
                            ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
                            ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND, coreIndex, 0); 
           if(ret != ADK_SUCCESS)
           {
              ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
                             "ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND \n");
              return ADK_E_DROP_TASK;
           }
           return ADK_E_TASK_CONSUMED;
       } 
       return ADK_SUCCESS; 
    } 

    /* PBR lookup successful */
    pbr_action = (adk_ipv6_dp_pbr_action_entry_t *)
                 &adk_ipv6_dp_tables->ipv6_dp_pbr_table.tbl[(uintptr_t)pbr_val];
    (adk_ipv6_dp_tables->ipv6_dp_pbr_table.pbrRuleStats[coreIndex*IPV6_PBR_MAX_RULES_PER_SET +
                 pbr_action->stats_id - 1])++;
    if(likely(!pbr_action->useDipRoutingTable))
    {
       /* Overwrite the route info found in IP DIP lookup */
       *route_entry = &(pbr_action->route_entry);
    }
    else if(unlikely(NULL == *route_entry))
    {
       ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 route lookup failed \n");

       /* increment the stats */
       if_stats->ipv6IfStatsInForwDatagrams++;
       if_stats->ipv6IfStatsInNoRoutes++;

       /* raise an exception "Route not found" */
       ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
    	        adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, 0, ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN,
	        ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND, coreIndex, 0); 
       if(ret != ADK_SUCCESS)
       {
           ADK_IPV6_DP_LOG_INFO_DEBUG("error: IPV6 failed to send exception" 
	    	     "ADK_IPV6_EXCEPTION_ROUTE_NOT_FOUND \n");
           return ADK_E_DROP_TASK;
       }
       return ADK_E_TASK_CONSUMED;
   } else { /* PBR action is use DIP routing, and routing look up is successful */
       return ADK_SUCCESS;
   }

   /* Check if PBR action is terminate */
   if(unlikely(ADK_IPV6_ROUTE_LOOKUP_ACTION_TERMINATE == pbr_action->route_entry.action))
   {
       /* check if extension header parsing errors */
       if(excep)
       {
            if(excep == ADK_IPV6_HDR_ERR_EXCEP)
            {
                if_stats->ipv6IfStatsInHdrErrors++;
                IPV6_DP_DROP_TASK(task);
                return ADK_E_TASK_CONSUMED;
            }

	   /* raise an exception  */
	   ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
		   adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset, 
		   ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
	   if(ret != ADK_SUCCESS)
	   {
	       ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
		       "%d \n", excep);
               return ADK_E_DROP_TASK;
	   }
	   /* update the count & mask */
           return ADK_E_TASK_CONSUMED;
	}

	/* do reassembly if fragmented */
	if((pbr_action->route_entry.reassembly_flag) && (frag_flag != ADK_IPV6_UNFRAGMENTED_PKT))
	{
	    /* increment packet req for reassembly */
	    if_stats->ipv6IfStatsReasmReqds++;

	    /* reassembly metadata */
	    adk_ipv6_dp_reass_meta_data_t *reass_md_p = 
	       (adk_ipv6_dp_reass_meta_data_t*) &adk_md_p->imc_param_start;
	    reass_md_p->frag_id = frag_id;
	    reass_md_p->rulemask = pbr_action->route_entry.ingress_assoc_rulemask;
	    reass_md_p->offset_to_frag = offset_to_frag;
	    reass_md_p->if_stats = if_stats; 
#ifdef ADK_IPV6_FLOW_CACHE
	    reass_md_p->vif_flag = 0;
	    reass_md_p->vif_stats_id = 0;
#endif
#if defined(ADK_PAL_DYNAMIC_PPL)
	    ret =adk_pal_dp_next_stage_event_handle_with_stage_info((void **)&task, 1, 1, 
		   &adk_ipv6_dp_tables->ipv6_dp_reassmbly_stage_info, 
		   (frag_id & ADK_IPV6_REASSEMBLY_HASH_MASK) , 0 , ADK_TASK_TYPE_FWD);
	   if(ret != 1)
	   {
               return ADK_E_DROP_TASK;
	   }

           return ADK_E_TASK_CONSUMED;
#else 
	   ret = adk_ipv6_dp_reassembly((void **)&task, 1, 1, NULL, NULL, coreIndex);

	   if(ret == ADK_E_TASK_CONSUMED)
	       return ret;
#endif
       }
       else if (frag_flag != ADK_IPV6_UNFRAGMENTED_PKT)
       {
	    l4_offset = adk_ipv6_parse_header_exception(protocol, (uint8_t *)ipv6_hdr, &excep);
	    /* check if extension header parsing errors */
	    if(excep)
	    {
		/* raise an exception  */
		ret = adk_dp_send_ipv6_exception(task, ipv6_if_entry->vrd_id, task->port,
			adk_md_p->input_intf_id, adk_md_p->ip_offset, 0, l4_offset, 
			ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN, excep, coreIndex, 0);
		if(ret != ADK_SUCCESS)
		{
		    ADK_IPV6_DP_LOG_INFO_1D_DEBUG("error: IPV6 failed to send exception" 
			    "%d \n", excep);
                    return ADK_E_DROP_TASK;
		}

		/* update the count & mask */
		return ADK_E_TASK_CONSUMED;
	    }
	}

	ret = adk_ipv6_dp_terminate(task, pbr_action->route_entry.ingress_assoc_rulemask, 
	      ipv6_if_entry, if_stats, ipv6_hdr, frag_flag, protocol, l4_offset, src_port, dest_port);        
        return ret;
   }


   return ADK_SUCCESS;
}

/*!
 * =============================================================
 * adk_ipv6_dp_egress_pbr()
 * 
 * Performs the pbr lookup & stores the action table entry 
 * index in the passed pointer.
 * The possible return values
 * ADK_SUCCESS: If PBR lookup successfull,
 * =============================================================
 */
static inline adk_status_t adk_ipv6_dp_egress_pbr(uint8_t policyset_id, uint8_t pcp, uint8_t protocol,
                     adk_ipv6_hdr_t *ipv6_hdr, uint16_t src_port, uint16_t dest_port,
                     adk_ipv6_frag_t frag_flag, adk_ipv6_pbr_val_t  *pbr_val 
                     )
{
    adk_status_t ret;
    adk_ipv6_pbr_ptn_t pbr_ptn;
    ipv6_pbr_rule_set_info_t *policy_info_p;

    /* 
     * PBR enabled, but ruleset is not mapped to interface 
     * Fall-back to DIP based lookup
    */
    if(policyset_id == 0)
    {
        ADK_IPV6_DP_LOG_INFO("PBR enabled, but ruleset not mapped!\n");
        return ADK_E_INTERNAL_ERROR;
    }
    policy_info_p = (ipv6_pbr_rule_set_info_t *)
                    (adk_ipv6_dp_tables->policySetHandles[((policyset_id-1)&ADK_IPV6_PBR_RULE_SETS_MASK)]);

    /*
     * PBR enabled, but mapped ruleset to interface is invalid 
     * Fall-back to DIP based lookup
    */
    if(!policy_info_p)
    {
        ADK_IPV6_DP_LOG_INFO("PBR enabled, but mapped to invalid ruleset!\n");
        return ADK_E_INTERNAL_ERROR;
    }

    (void)pcp;
    (void)protocol;
    (void)src_port;
    (void)dest_port;
    (void)frag_flag;

    rte_memcpy(&pbr_ptn.sip_addr[0], 
               &ipv6_hdr->src_addr[0], 
               sizeof(adk_ipv6_addr_t));

    /* lookup PBR ACL */
    ret = adk_ipv6_dp_pbr_lookup(policy_info_p->pbr_tree[policy_info_p->pbr_tree_index],
                            &pbr_ptn, pbr_val);

    ADK_IPV6_DP_LOG_DEBUG("ret: %x pbr_val: %x\n", ret, *pbr_val);

    if(unlikely((ret != ADK_SUCCESS) || !(*pbr_val)))
    {
        ADK_IPV6_DP_LOG_DEBUG("Entry not found in PBR ACL tree, fall back on DIP based lookup!");
        return ADK_E_INTERNAL_ERROR;
    } 

    return ADK_SUCCESS;
}

#endif //_ADK_IPV6_DP_PBR_H_
