/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_ipv6_dp_pvt.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief   This file has the data types and functions which are used only by
 *          data plane software.
 *
 *  **********************************************************************/

#ifndef _ADK_IPV6_DP_PVT_H_
#define _ADK_IPV6_DP_PVT_H_


#ifdef __cplusplus
extern "C" {
#endif

#include "adk_ipv6_dp.h"
#include "adk_ipv6_dp_export.h"

#define IPV6_REASSEMBLY_TIMEOUT          (ADK_IPV6_REASSEMBLY_TIMER_VALUE/1000)

#define IPV6_DP_DROP_TASK(mbuf) \
    do{ \
        rte_pktmbuf_free((struct rte_mbuf *)mbuf); \
        mbuf = NULL; \
    }while(0);

#define IPV6_DP_DROP_CHAINED_TASK(mbuf) \
    do{ \
        adk_metadata_t *_adk_md_p; \
        struct rte_mbuf *_next_mbuf; \
        while(mbuf) \
        { \
            _adk_md_p = ADK_METADATA_PTR(mbuf); \
            _next_mbuf = _adk_md_p->next_frag; \
            rte_pktmbuf_free((struct rte_mbuf *)mbuf); \
            mbuf = _next_mbuf; \
        } \
        mbuf = NULL; \
    }while(0);


#define ADK_IPV6_FILL_PH_DEV_PARAMS(task, devid, queueid, pri, cb, polFlowId, excep, vrdid, offset, \
        flow_id, _mtu) \
    do{ \
        adk_ph_dev_dp_metadata_t *phMeta_p; \
        adk_ipv6_ph_dev_rx_param_t *ipv6_ph_params; \
        phMeta_p = (adk_ph_dev_dp_metadata_t *) \
        &(ADK_METADATA_PARAM(task,imc_param_start)); \
        ADK_MD_SET_ADK_DEV_ID(task, devid); \
        ADK_MD_SET_DEV_TX_QUEUE_ID(task, queueid); \
        phMeta_p->priority = pri; \
        phMeta_p->cbFnId = cb; \
        phMeta_p->policerFlowId = polFlowId; \
        phMeta_p->moduleId = ADK_IPV6_MODULE_ID; \
        ipv6_ph_params = (adk_ipv6_ph_dev_rx_param_t *) \
        &phMeta_p->imc_param_start; \
        ADK_METADATA_PARAM(task, pdu_flow_id) = flow_id; \
        ipv6_ph_params->exception_type = excep; \
        ipv6_ph_params->ipv6Offset = offset; \
        ipv6_ph_params->mbuf = (struct rte_mbuf *)task; \
    }while(0);

#define ADK_IPV6_FILL_PH_PARAMS(task, port, if_id, domain, exception, vrd_id, offset, flow_id, mtu) \
    do{ \
        ADK_TASK_PH_PARAMS(task, 2) = (((size_t)port) <<24) | if_id; \
        ADK_TASK_PH_PARAMS(task, 3) = (((size_t)exception) <<24) | flow_id; \
        ADK_TASK_PH_PARAMS(task, 4) = (((size_t)offset) <<24) | (vrd_id<<16) | port; \
        ADK_TASK_PH_PARAMS(task, 5) = (mtu); \
        ADK_TASK_PH_PARAMS(task, 6) = (((size_t)domain) <<24); \
        ((adk_ph_pdu_buf_rx_t *)task)->pduSize      = ADK_TASK_PDU_SIZE(task);\
    }while(0);


#define ADK_IPV6_FILL_SEND_TO_CPU_PARAMS( task, entry) \
    do{ \
        adk_slowpath_dev_dp_metadata_t *spDevMd_p; \
        spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(task, \
                                                            imc_param_start)); \
        ADK_MD_SET_ADK_DEV_ID(task, entry->devId); \
        ADK_MD_SET_DEV_TX_QUEUE_ID(task, entry->queueId); \
        ADK_METADATA_PARAM(task, egress_stats)[ADK_SYS_EGRESS_STATS_MDATA_QUEUE_LOC].stats_info = \
                                                                    entry->statsId; \
        spDevMd_p->policerFlowId = entry->policerId; \
    }while(0);

#ifdef ADK_IPV6_FLOW_CACHE_STATS
#define ADK_IPV6_LLI_FC_INCR(cnt) \
   (adk_ipv6_dp_tables->ipv6_dp_lli_flow_cache_table[coreIndex].lli_fc_stats).cnt++; 

#define ADK_IPV6_ULI_FC_INCR(cnt) \
   (adk_ipv6_dp_tables->ipv6_dp_uli_flow_cache_table[coreIndex].uli_fc_stats).cnt++; 
#else
#define ADK_IPV6_LLI_FC_INCR(cnt) 
#define ADK_IPV6_ULI_FC_INCR(cnt) 
#endif

typedef enum adk_ipv6_dp_frag_idx_e{
    ADK_IPV6_LAST_FRAG_IDX,
    ADK_IPV6_FIRST_FRAG_IDX,
    ADK_IPV6_MIN_FRAG_IDX,
    ADK_IPV6_MAX_FRAG_IDX = ADK_IPV6_MAX_REASS_ELE
}adk_ipv6_dp_frag_idx_t;

typedef enum adk_ipv6_dp_reass_excep_e{
    ADK_IPV6_REASS_LIST_FULL,
    ADK_IPV6_REASS_TIMER_CREATE_ERR,
    ADK_IPV6_REASS_NOT_MULTIPLE_OF8,
    ADK_IPV6_REASS_DUP_FRAG,
    ADK_IPV6_REASS_OVERLAP_FRAG,
    ADK_IPV6_REASS_NO_FREE_HOLES,
    ADK_IPV6_REASS_MAX_PDU_SIZE_REACHED,
    ADK_IPV6_REASS_INVALID_EXCEP
}adk_ipv6_dp_reass_excep_t;

#define ADK_IPV6_HDR_ERR_EXCEP            ADK_IPV6_EXCEPTION_MAX

#define ADK_IPV6_TIMEOUT_HASH_BUCKET_MASK 0xFFFFFF
#define ADK_IPV6_TIMEOUT_HASH_ENTRY_MASK 0xFF

/* egress flow task param action flag masks */
#define ADK_IPV6_HEADER_ENCAP_FLAG_MASK   0x01
#define ADK_IPV6_DEST_INFO_PRESENT_MASK   0x02
#define ADK_IPV6_DEST_ID_MASK             0x04  
#define ADK_IPV6_PRI_BIT_MASK             0x08
/* #define ADK_IPV6_LINUX_INJECTED_MASK      0x10  */

/* IPv6 Flow cache return values */
#define ADK_IPV6_FLOW_CACHE_DONE            0
#define ADK_IPV6_FLOW_CACHE_SLOW_PATH       1
#define ADK_IPV6_FLOW_CACHE_CONSUME         2
#define ADK_IPV6_FLOW_CACHE_ERROR           4

/* IPv6 Fragment type */
#define ADK_IPV6_DP_FIRST_FRAGMENT 2
#define ADK_IPV6_DP_NON_FIRST_FRAGMENT 3


/********************************************************************************
 *                  IPv6 Fast path structures                                   *
 ********************************************************************************/
typedef struct adk_ipv6_extn_hdr_s{
    uint8_t    next_header;
    uint8_t    hdr_length;
}adk_ipv6_extn_hdr_t;

typedef struct adk_ipv6_extension_fragment_s {
    uint8_t next_header;            /**< Next header type */
    uint8_t reserved;               /**< Reserved */
    uint16_t frag_data;             /**< All fragmentation data */
    uint32_t id;                    /**< Packet ID */
}adk_ipv6_extension_fragment_t;


/* global fast-path database pointer */
extern adk_ipv6_dp_tables_t    *adk_ipv6_dp_tables;

extern uint32_t         ipv6_global_frag_id[ADK_MAX_NUM_CORES];


/*!
 * =============================================================
 * @struct adk_ipv6_dp_lli_imc_reass_data_t adk_ipv6_dp_export.h "adk_ipv6_dp_export.h"
 *
 * @brief 
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to IPv6 module LLI for the purpose of reassembly 
 * packet processing.
 *
 * =============================================================
 */
typedef struct adk_ipv6_dp_reass_meta_data_s
{
    adk_ipv6_dp_if_stats_entry_t         *if_stats;
    adk_ipv6_dp_ip_addr_stats_t          *ipv6_addr_stats;

    uint32_t    frag_id; /* 4B */
    uint16_t    offset_to_frag;
    uint16_t    in_acl_stats_id;
    
    uint8_t     rulemask;
    uint8_t     resrved:5;
    uint8_t     l4_port_present:1;
    uint8_t     fc_hit:1;   /* flag to indicate, fragments matches the FC */
    uint8_t     vif_flag:1;
    uint16_t    vif_stats_id;
    uint32_t    hash_value; /* flow cache hash value */
    
    uint16_t    src_port;
    uint16_t    dst_port;
    uint8_t     next_hdr;
    uint8_t     ipsec_data;
    bool        is_outer_ip;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_ipv6_dp_reass_meta_data_t;



typedef struct adk_ipv6_dp_internal_imc_data_s
{
    adk_ipv6_dp_lli_imc_data_t lli_imc_data;
    bool        is_reassembled;
    uint16_t    initial_frag_len;
    bool        is_outer_ip;
    adk_ipv6_dp_ip_addr_stats_t  *ipv6_addr_stats;
}adk_ipv6_dp_internal_imc_data_t;


/********************************************************************************
 *                  IPv6 Fast path function definitions                         *
 ********************************************************************************/
adk_dp_status_t
adk_ipv6_dp_pkt_proc_lli(void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t task_mask,
                        __attribute__((unused)) void *ring,
                        __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                        int coreIndex);

adk_dp_status_t
adk_ipv6_dp_pkt_proc_uli(void __attribute__((unused)) **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        int __attribute__((unused)) coreIndex);

adk_status_t 
adk_ipv6_dp_pkt_proc_netd(void  **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);

adk_status_t 
adk_ipv6_dp_reassembly_timeout_proc(void  **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);

adk_status_t 
adk_ipv6_dp_reassembly(void **task_p,
                        uint8_t __attribute__((unused)) task_count,
                        uint32_t __attribute__((unused)) task_mask,
                        void __attribute__((unused)) *ring,
                        adk_sif_cfg_pipeline_info_t __attribute__((unused)) *next_stage_entry, 
                        __attribute__((unused)) int coreIndex);

#ifdef __cplusplus
}
#endif
#endif /* _ADK_IPV6_DP_PVT_H_ */ 
/*! end addtogroup _ipv6
 * @}
 */
