/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_ipv6_dp_tables.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief   This file has the functions for fast-path tables lookup used 
 * by data plane software.
 *
 *  **********************************************************************/

#ifndef _ADK_IPV6_DP_TABLES_H_
#define _ADK_IPV6_DP_TABLES_H_

#include "adk_include.h"
#include "adk_ipv6_dp_pvt.h"
#include "adk_ipv6_tables.h"
#include "adk_ipv6_internal.h"
#include "adk_rcu.h"
#include "adk_ipv6_dp.h"
#include <adk_em.h>
#include <adk_qos_dp_export.h>
#include "adk_hos_dp.h"

static inline adk_dp_status_t 
adk_ipv6_dp_interface_entry_lookup(uint16_t if_id, 
                uint8_t __attribute__((unused)) dscp,
                adk_ipv6_dp_interface_entry_t **ipv6_if_entry)
{

    if(if_id > (ADK_IPV6_NO_OF_IPV6_INTERFACES + ADK_IPV6_NO_OF_IPV6_TUNNEL_INTERFACES))
    {
        return ADK_DP_E_ERROR;
    }

    if(!adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id].is_disabled)
    {
        *ipv6_if_entry = &adk_ipv6_dp_tables->ipv6_dp_if_table.tbl[if_id];
        return ADK_SUCCESS;
    }

    return ADK_DP_E_ERROR;
}

static inline adk_dp_status_t 
adk_ipv6_dp_route_entry_lookup(uint8_t vrd_id, 
                            uint8_t *ipv6_addr,
                            adk_ipv6_dp_route_entry_t** route_entry)
{
    uint8_t input_ptn[(ADK_IPV6_ROUTE_LOOKUP_TREE_WIDTH>>3)+1];
    adk_handle_t index;
    adk_ipv6_dp_route_table_t *ipv6_route_tbl = &adk_ipv6_dp_tables->ipv6_dp_route_table;

    /* Key for route LPM lookup */
    adk_ipv6_dp_route_key_generate(input_ptn, 
            vrd_id, 
            ipv6_addr);

    ADK_IPV6_DP_LPM_LOOKUP(ipv6_route_tbl->lpm_tbl, input_ptn, index); 

    if((index) && (((adk_ipv6_dp_route_entry_t *)index)->is_enabled))
    {
        *route_entry = (adk_ipv6_dp_route_entry_t *)index;
        return ADK_SUCCESS;
    }

    return ADK_DP_E_ERROR;
}

static inline adk_dp_status_t
adk_ipv6_dp_nexthop_entry_lookup(uint16_t nexthop_id,
                                adk_ipv6_dp_nexthop_entry_t **nexthop_entry)
{
    if(adk_ipv6_dp_tables->ipv6_dp_nexthop_table.tbl[nexthop_id].is_enabled)
    {
        *nexthop_entry = &adk_ipv6_dp_tables->ipv6_dp_nexthop_table.tbl[nexthop_id];
    return ADK_SUCCESS;
    }
    return ADK_DP_E_ERROR;
}

static inline adk_dp_status_t
adk_ipv6_dp_ingress_classification_entry_lookup(adk_pktio_task_t *task,
        uint16_t    interface_id,
        uint8_t     next_header,
        uint32_t    *src_ipv6_addr,
        uint32_t    *dst_ipv6_addr,
        uint16_t    src_port_icmp_type,
        uint16_t    dst_port_icmp_code,
        uint16_t    ipv6_len,
        uint16_t    l4_offset)
{
    adk_dp_status_t dp_st;
    uint32_t flow_id=0;
    adk_ipv6_ing_classifier_ptn_t  classifierRules;
    adk_ipv6_dp_ing_classifier_action_info_t *ingClassifierAction_p=NULL;
    uint8_t coreIndex = ADK_LCORE_INDEX;
    adk_ipv6_dp_global_statistics_t *ipv6_global_stats;
    ipv6_utility_counters_t         *ipv6_utility_stats;
    adk_ipv6_dp_policer_meta_data_t *ipv6_policer_md_p = NULL;
    adk_slowpath_dev_dp_metadata_t *spDevMd_p;
    adk_ipv6_hdr_t *ipv6_hdr = NULL;
    uint8_t ipsec_data = 0;
    adk_metadata_t *adk_md_p =  ADK_METADATA_PTR(task);

    ipsec_data = ((adk_ipv6_dp_lli_imc_data_t *) &(adk_md_p->imc_param_start))->ipsec_data;
    ADK_PDU_OFFSET_PTR_GET(task, ipv6_hdr, adk_ipv6_hdr_t *, adk_md_p->ip_offset);

    ipv6_global_stats = adk_ipv6_dp_tables->ipv6_dp_global_stats_table[coreIndex];
    ipv6_utility_stats = (ipv6_utility_counters_t *)adk_ipv6_dp_tables->ipv6_utility_counters[coreIndex];

    if(adk_hos_is_ip_classifier_invalid(ADK_DSI_RX_METADATA_PTR(task)))
    {
        ADK_IPV6_DP_LOG_DEBUG("IPv6 IngressClassifier HW lookup failed. Falling back to SW lookup\n");
        classifierRules.ipIntfHandle = adk_htonl(interface_id);
        classifierRules.proto = next_header;
        classifierRules.sip0 = src_ipv6_addr[0];
        classifierRules.sip1 = src_ipv6_addr[1];
        classifierRules.sip2 = src_ipv6_addr[2];
        classifierRules.sip3 = src_ipv6_addr[3];
        classifierRules.dip0 = dst_ipv6_addr[0];
        classifierRules.dip1 = dst_ipv6_addr[1];
        classifierRules.dip2 = dst_ipv6_addr[2];
        classifierRules.dip3 = dst_ipv6_addr[3];
        classifierRules.dPort = dst_port_icmp_code;
        classifierRules.sPort = src_port_icmp_type;

        dp_st = adk_ipv6_dp_ing_classifier_lookup(
                adk_ipv6_dp_tables->ingClassifierInfo_p->ipv6IngClassifierCtx[adk_ipv6_dp_tables->ingClassifierInfo_p->aclTreeIndex],
                &classifierRules, &flow_id);
        if(unlikely((dp_st != ADK_SUCCESS) || !flow_id))
        {
            ipv6_global_stats->noIngressAssociation++;
            ADK_IPV6_DP_LOG_INFO_DEBUG("Entry not found in Ingress Classifier tree\n!");
            return ADK_E_INTERNAL_ERROR;
        }
    }
    else
    {   /* Retrieve classifier action index and terminate the packet */
        ADK_IPV6_DP_LOG_DEBUG("IPv6 IngressClassifier HW lookup Success\n");
        flow_id = adk_hos_get_ip_classifier_rule_id(ADK_DSI_RX_METADATA_PTR(task));
    }

    uintptr_t cbParam = flow_id;
    ingClassifierAction_p =
        &(adk_ipv6_dp_tables->ingClassifierActionDp_p[adk_ipv6_dp_tables->ingClassifierInfo_p->aclTreeIndex][flow_id]);
    adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].rxPkts++;
    ADK_IPV6_DP_LOG_DEBUG("Classifier (TreeIndex=%d) returned:  flow_id=%d, action=%d\n", adk_ipv6_dp_tables->ingClassifierInfo_p->aclTreeIndex,
            flow_id, ingClassifierAction_p->action);
    ipv6_policer_md_p = (adk_ipv6_dp_policer_meta_data_t *)&(ADK_METADATA_PARAM(task, imc_param_start));
    ipv6_policer_md_p->ing_action = *ingClassifierAction_p;
    ipv6_policer_md_p->ingress_classifier_flow_id = flow_id;

    switch(ingClassifierAction_p->action)
    {
        case ADK_IPV6_ING_CLASSIFIER_ACTION_DROP:
            if (1 == flow_id)
            {
                ipv6_global_stats->noIngressAssociation++;
            }
            else
            {
                adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].ipv6ClassifierDroppedPkts++;
            }
            ADK_IPV6_DP_LOG_ERROR("Classifier action->DROP (FlowId=%d). dropping the packet\n", flow_id);
            return ADK_E_DROP_TASK;
        case ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_EO:

#ifdef ADK_QOS_GEN3_INTERFACE
            if (ingClassifierAction_p->policingEnabled) {
#if defined(ADK_PAL_DYNAMIC_PPL) && defined(IPV6_POLICER_EVENT_DEV_STAGE)
                adk_pal_dp_stage_info_t *policer_stage_info_p = NULL;
                uint16_t tmp_16;
                /*This is to eliminate compiler warning */
                /*Get the stage info from PAL*/
                dp_st = adk_pal_dp_stage_info_get(ADK_IPV6_POLICING_FN_ID,
                        &policer_stage_info_p);
                if(dp_st != ADK_SUCCESS)
                {
                    ADK_IPV6_DP_LOG_ERROR("Failed to get next stage info from PAL");
                    ipv6_global_stats->noIngressAssociation++;
                    INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxClassifierSendToEoFail);
                    return ADK_E_DROP_TASK;
                }

                tmp_16 = adk_pal_dp_next_stage_event_handle_with_stage_info(
                        (void **)&task, 1, 0x1,
                        policer_stage_info_p, ingClassifierAction_p->policerFlowId,
                        0x00, ADK_METADATA_PARAM(task, task_type));
                if(tmp_16 != 1)
                {
                    ADK_IPV6_DP_LOG_ERROR("Failed to enQ to IPv6 Policer stage");
                    ipv6_global_stats->ipv6InError++;
                    return ADK_E_DROP_TASK;
                }
                return ADK_E_TASK_CONSUMED;
#else
                uint8_t policeColor = ADK_QOS_DP_GREEN;
                if(unlikely( ADK_SUCCESS !=
                            adk_qos_dp_policing_color_blind(
                                ipv6_policer_md_p->ing_action.policerFlowId,
                                ipv6_len, &policeColor) ) )
                {
                    return ADK_E_INTERNAL_ERROR;
                }

                /* check policing color */
                if(unlikely(ADK_QOS_DP_RED == policeColor))
                {
                    adk_ipv6_dp_tables->ipv6_dp_global_stats_table[ADK_LCORE_INDEX]->ipv6RateLimitingDrops++;
                    adk_ipv6_dp_tables->ipv6IngFlowStats[coreIndex * ADK_DP_MAX_IPV6_ING_ASSOC_FLOWS + flow_id].policerDropPkts++;
                    /* Dropping the packet */
                    adk_pktio_task_buffer_free(task);
                    return ADK_E_TASK_CONSUMED;
                }
#endif
            }
#endif
            if(ADK_SYS_IPSEC_BYPASS_GET)
            {
               ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS1_TS();
            }
            else
            {
               ADK_DP_MEASURE_BLOCK_CYCLE_IPV6_LLI_TX_PASS2_TS();
            }
            /*This is needed only for ORDERED SHED*/
            if(ADK_METADATA_PARAM(task, event_sched_type) != RTE_SCHED_TYPE_ATOMIC)
            {   
                adk_sif_em_dp_lli_meta_data_t *em_lli_md_p = (adk_sif_em_dp_lli_meta_data_t *)&(ADK_METADATA_PARAM(task,
                            imc_param_start));
                em_lli_md_p->eventMachineQueueId = ingClassifierAction_p->eventMachineQueueId;
                dp_st = adk_pal_dp_stage_send((void **)&task, 1, 0x1, ADK_SIF_EM_LLI_FN_ID, em_lli_md_p->eventMachineQueueId,
                        0x00, ADK_PAL_DP_BATCH_CB_FN_IPV6, ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER, (void *)cbParam);
                /* incrementing L4 stats */
                adk_ipv6_increment_rx_l4_stats(next_header, ipv6_len - l4_offset, ADK_LCORE_INDEX);
            }
            else
            {
#ifdef ADK_HOS_DP_TEST
    /* Copy DSI VSI meta data from original location to post EM header */
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
    uint8_t* post_em_header_ptr = (uint8_t*)((uintptr_t)task + sizeof(struct rte_mbuf) + EM_HDR_SIZE);
    memcpy(post_em_header_ptr, dsi_md_p, sizeof(adk_dsi_rx_metadata_t));
#endif
                dp_st = adk_pal_dp_em_send(task, ingClassifierAction_p->eventMachineQueueId,
                        ADK_PAL_DP_BATCH_CB_FN_IPV6, (void *)cbParam);
                if (dp_st == ADK_E_TASK_CONSUMED)
                {
                    adk_ipv6_increment_rx_l4_stats(next_header, ipv6_len - l4_offset, ADK_LCORE_INDEX);
                }
            }

            return dp_st;

        case ADK_IPV6_ING_CLASSIFIER_ACTION_SEND_TO_HOST:
            {
                ADK_IPV6_DP_LOG_DEBUG("IPv6 Ingress classifier action STH via NetD!, linux_intf_id=%d\n", interface_id);
                /* Update metadata */
                spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)
                    &(ADK_METADATA_PARAM(task,imc_param_start));
                spDevMd_p->policerFlowId = ingClassifierAction_p->policerFlowId;
                spDevMd_p->flowId = flow_id;
                spDevMd_p->moduleId = ADK_IPV6_MODULE_ID;
                spDevMd_p->l4_len = ipv6_len - l4_offset;
                spDevMd_p->l4_protocol = next_header;
                /* Copy ethernet header in case of decryption */
                //if(IPSEC_DONE_PACKET_DECRYPTED == ipsec_data)
                if( (IPLLI_IMC_IPSEC_INNER_LOOKASIDE == ipsec_data) || (IPLLI_IMC_IPSEC_INNER_INLINE == ipsec_data) )
                {
#define ETHER_TYPE_IPV6       0x86DD
#define ETHER_TYPE_OFFSET(mbuf)     (mbuf->l2_len - 2)

                    uint8_t *ethHdr = NULL;
                    ADK_PDU_PTR_GET(task, ethHdr, uint8_t *);
                    *((uint16_t *)(ethHdr + ETHER_TYPE_OFFSET(task))) = htons(ETHER_TYPE_IPV6);

                    rte_memcpy( ((uint8_t *)ipv6_hdr) - task->l2_len, ethHdr, task->l2_len);

                    rte_pktmbuf_adj(task, ADK_MD_GET_IP_OFF(task) - task->l2_len);
                }
                if(unlikely(adk_pal_dp_netd_send((void *)task, ADK_PAL_DP_BATCH_CB_FN_IPV6, NULL) != ADK_E_TASK_CONSUMED))
                {
                    /*TODO: Utility counter need to be moved to callback*/
                    INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxCLassifierSendToHostFail);
                    return ADK_E_DROP_TASK;
                }
                else
                {
                    /*TODO: Utility counter need to be moved to callback*/
                    INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxClassifierSendToHost);
                }
                return ADK_E_TASK_CONSUMED;
            }
        default:
            ADK_IPV6_DP_LOG_ERROR("Classifier action didn't match any rule. Dropping the packet\n");
            ipv6_global_stats->ipv6GenericDroppedPkts++;
            INCREMENT_IPV6_UTILITY_COUNTER(ipv6_utility_stats->ipv6IngressUtilityCounters.ipv6RxInvalidClassifierAction);
            return ADK_E_DROP_TASK;
    }
    /* Not expected to come till here..drop it */
    return ADK_E_DROP_TASK;
}

static inline adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_entry_lookup(uint8_t    interface_id,
                        uint32_t    *src_ipv6_addr,
                        uint32_t    *dst_ipv6_addr,
                        adk_ipv6_dp_ingress_mgroup_entry_t **ingress_mgroup_entry)
{

    uint8_t key[(ADK_IPV6_INGRESS_MGROUP_FLOW_TREE_WIDTH>>3)+1];
    adk_handle_t index;
    adk_ipv6_dp_ingress_mgroup_table_t *ingress_assoc_tbl = &adk_ipv6_dp_tables->ipv6_dp_ingress_mgroup_table;
     uint16_t size;
    /* create input patern to LPM */
    adk_ipv6_dp_ingress_mgroup_key_create(key, 
                           interface_id,
                           dst_ipv6_addr,
                           src_ipv6_addr, 
			   1,
			   &size);


    ADK_IPV6_DP_LPM_LOOKUP(ingress_assoc_tbl->lpm_tbl, key, index);

    if(ingress_assoc_tbl->tbl[(size_t)index].is_enabled)
    {
        *ingress_mgroup_entry = &ingress_assoc_tbl->tbl[(size_t)index];
    }
        
    return ADK_SUCCESS;
}


static inline adk_dp_status_t 
adk_ipv6_dp_ingress_association_entry_lookup(uint8_t     mask,
                        uint8_t     vrd_id,
                        uint16_t    interface_id,
                        uint32_t    flow_label,        
                        uint8_t     next_header,
                        uint32_t    *src_ipv6_addr,
                        uint32_t    *dst_ipv6_addr,
                        uint16_t    src_port_icmp_type,
                        uint16_t    dst_por_icmp_code,
                        adk_ipv6_dp_ingress_assoc_entry_t **ingress_assoc_entry)
{

    uint8_t key[(ADK_IPV6_INGRESS_ASSOC_FLOW_TREE_WIDTH>>3)+1];
    uint16_t size;
    adk_handle_t index;
    adk_ipv6_dp_ingress_assoc_table_t *ingress_assoc_tbl = &adk_ipv6_dp_tables->ipv6_dp_ingress_assoc_table;

    /* create input patern to LPM */
    adk_ipv6_dp_ingress_association_key_create(key, 
                           mask,
                           vrd_id,
                           interface_id,
                           flow_label,
                           next_header,
                           src_ipv6_addr,
                           (ADK_IPV6_PDU_ADDR_LENGTH),
                           dst_ipv6_addr,
                           (ADK_IPV6_PDU_ADDR_LENGTH),
                           src_port_icmp_type,
                           dst_por_icmp_code,
                           (uint8_t) src_port_icmp_type,
                           (uint8_t) dst_por_icmp_code,
                           TRUE,
                           &size);


    ADK_IPV6_DP_LPM_LOOKUP(ingress_assoc_tbl->lpm_tbl, key, index);

    if(ingress_assoc_tbl->tbl[(size_t)index].is_enabled)
    {
        *ingress_assoc_entry = &ingress_assoc_tbl->tbl[(size_t)index];
    }
        
    return ADK_SUCCESS;
}

static inline adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_lookup(uint8_t vrd_id, 
                            uint8_t *onLinkDestinationIpv6Addr,
                            adk_ipv6_dp_onlink_dest_entry_t  **onlink_dest_entry)
{
    uint32_t    key;
    uint16_t first_key, second_key, i;
    adk_ipv6_dp_onlink_dest_hash_entry_t  *onlink_dest_hash_entry = NULL;
    adk_ipv6_dp_onlink_dest_table_t *onlink_dst_tbl;

    onlink_dst_tbl = &adk_ipv6_dp_tables->ipv6_dp_onlink_dest_table;

    /* Generate key for onlink dest table lookup */
    adk_ipv6_dp_onlink_dest_key_generate(vrd_id,
                                (uint32_t *)onLinkDestinationIpv6Addr,
                                &key,
                                &first_key, 
                                &second_key);

    first_key = first_key*ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES;
    onlink_dest_hash_entry = &onlink_dst_tbl->hash_tbl[(first_key)];
    for(i=0; i<ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES; i++, onlink_dest_hash_entry++)
    {
        if((onlink_dest_hash_entry->hash_value == key))
        {
            /*compare vrdid & destination address */
            *onlink_dest_entry = &onlink_dst_tbl->tbl[onlink_dest_hash_entry->index];

            if((vrd_id == (*onlink_dest_entry)->vrd_id) && 
               (*((uint64_t *)&onLinkDestinationIpv6Addr[0]) == 
                    *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[0]))) &&
               (*((uint64_t *)&onLinkDestinationIpv6Addr[8]) == 
                    *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[2]))))
            {
                return(ADK_SUCCESS);
            }
        }
    }
   
    second_key = second_key*ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES;
    onlink_dest_hash_entry = &onlink_dst_tbl->hash_tbl[(second_key)];
    for(i=0; i<ADK_IPV6_NO_OF_ONLINK_COL_ENTRIES; i++, onlink_dest_hash_entry++)
    {
        if((onlink_dest_hash_entry->hash_value == key))
        {
            /*compare vrdid & destination address */
            *onlink_dest_entry = &onlink_dst_tbl->tbl[onlink_dest_hash_entry->index];

            if((vrd_id == (*onlink_dest_entry)->vrd_id) && 
               (*((uint64_t *)&onLinkDestinationIpv6Addr[0]) == 
                    *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[0]))) &&
               (*((uint64_t *)&onLinkDestinationIpv6Addr[8]) == 
                    *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[2]))))
            {
                return(ADK_SUCCESS);
            }
        }
    }

    /* check in the overflow list */
    adk_ipv6_dp_overflow_node_t *node;
    node=onlink_dst_tbl->overflow_list.head;
    while(node)
    {
        /*compare vrdid & destination address */
        *onlink_dest_entry = &onlink_dst_tbl->tbl[node->idx];

        if((vrd_id == (*onlink_dest_entry)->vrd_id) && 
                (*((uint64_t *)&onLinkDestinationIpv6Addr[0]) == 
                 *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[0]))) &&
                (*((uint64_t *)&onLinkDestinationIpv6Addr[8]) == 
                 *((uint64_t *)&((*onlink_dest_entry)->ipv6_addr.addr[2]))))
        {
            return(ADK_SUCCESS);
        }
        node = node->next;
    }

    ADK_IPV6_DP_LOG_INFO("onlink entry not found \n");

    *onlink_dest_entry = NULL;

    return ADK_DP_E_ERROR;
}

static inline adk_dp_status_t
adk_ipv6_dp_egress_flow_attr_entry_lookup(uint16_t eg_flow_id,
                                adk_ipv6_dp_egress_flow_attr_entry_t **eg_flow_entry)
{
    adk_ipv6_dp_egress_flow_attr_table_t *egress_flow_tbl;
    egress_flow_tbl = &adk_ipv6_dp_tables->ipv6_dp_egress_flow_attr_table;

    if(egress_flow_tbl->tbl[eg_flow_id].is_enabled == TRUE)
    {
        *eg_flow_entry = &egress_flow_tbl->tbl[eg_flow_id]; 
        return ADK_SUCCESS;
    }

    return ADK_DP_E_ERROR;
}


static inline adk_dp_status_t 
adk_ipv6_dp_ingress_flow_sip_entry_lookup(uint8_t vrd_id, 
                            uint8_t *ipv6_addr,
                            adk_ipv6_dp_ingress_flow_sip_entry_t **ingress_flow_sip_entry)
{
    uint8_t input_ptn[(ADK_IPV6_INGRESS_FLOW_SIP_TREE_WIDTH>>3)+1];
    adk_handle_t index;
    adk_ipv6_dp_ingress_flow_sip_table_t *sip_tbl;
    sip_tbl = &adk_ipv6_dp_tables->ipv6_dp_ingress_flow_sip_table;

    /* Key for route LPM lookup */
    adk_ipv6_dp_ingress_flow_sip_key_generate(input_ptn, 
            vrd_id, 
            ipv6_addr);

    ADK_IPV6_DP_LPM_LOOKUP(sip_tbl->lpm_tbl, input_ptn, index);

    *ingress_flow_sip_entry = (adk_ipv6_dp_ingress_flow_sip_entry_t *) index;
    return ADK_SUCCESS;
}

static inline adk_dp_status_t 
adk_ipv6_dp_vrd_host_mode_entry_lookup(uint8_t vrd_id,
                         adk_ipv6_dp_vrd_host_mode_entry_t **ipv6_dp_vrd_host_mode_entry)
{
    
    if(adk_ipv6_dp_tables->ipv6_dp_vrd_table.tbl[vrd_id].is_enabled)
    {
        *ipv6_dp_vrd_host_mode_entry = &adk_ipv6_dp_tables->ipv6_dp_vrd_table.tbl[vrd_id];
        return ADK_SUCCESS;
    }

    return ADK_DP_E_ERROR;
}


static inline adk_dp_status_t
adk_ipv6_dp_excep_entry_lookup(uint8_t exception,
                            adk_ipv6_dp_excep_entry_t **except_entry)
{
    adk_ipv6_dp_excep_table_t *except_tbl;
    except_tbl = &adk_ipv6_dp_tables->ipv6_dp_excep_table;

    if(except_tbl->tbl[exception].is_enable == TRUE)
    {
        *except_entry = &except_tbl->tbl[exception];
        return ADK_SUCCESS;
    }
    return ADK_DP_E_ERROR;
}


static inline void
adk_ipv6_dp_read_flow_info(adk_ipv6_dp_flow_info_t src,
        adk_ipv6_dest_flow_info_t *dest)
{
    uint8_t idx = src.idx;

    dest->destFlow = src.flow[idx].destFlow;
    dest->destModId = src.flow[idx].destModId;
    ADK_MEMCPY(&dest->stageInfo, &src.flow[idx].stageInfo, sizeof(adk_pal_dp_stage_info_t));

    return;
}




#endif
/*! end addtogroup _ipv6
 * @}
 */
