/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @file  adk_ipv6_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the internal definitions required for the
 * IPV6 module. This file should not be included by the application.
 *
 ***********************************************************************/
#ifndef _ADK_IPV6_INTERNAL_H_
#define _ADK_IPV6_INTERNAL_H_

/* #include "adk_include.h" */
#include "adk_sys_types.h"
#include "adk_error.h"
#include "adk_list.h"
#include "adk_sif_cfg.h"
#include "adk_log_lib.h"
#include "adk_utils_tree_update.h"
#include "adk_ipv6.h"
#include "adk_ipv6_const.h"
#include "adk_rm.h"
#include "adk_lpm.h"
#include "adk_ipv6_dp.h"
#include "adk_ipv6_acl.h"
#include "adk_ipv6_pbr_acl.h"
#ifdef ADK_DEVICE_SNR
#include <adk_hos.h>
#endif
#include "adk_cp_acl_list.h"
#define ADK_IPV6_VERSION           6
#define ADK_IPV6_TUNNEL_IF_ID_MIN  512

#define IPV6_NUM_DSCP_VALUES        64
#define IPV6_NUM_PCP_VALUES          8
#define IPV6_DEFAULT_PCP             0

#define IPV6_INGRESS_ASSOC_BIT_VRD        0x1
#define IPV6_INGRESS_ASSOC_BIT_IF         0x2
#define IPV6_INGRESS_ASSOC_BIT_FLOW_LABEL 0x4
#define IPV6_INGRESS_ASSOC_BIT_PROTO      0x8
#define IPV6_INGRESS_ASSOC_BIT_L4INFO1    0x10
#define IPV6_INGRESS_ASSOC_BIT_L4INFO2    0x20
#define IPV6_INGRESS_ASSOC_BIT_SRC_ADDR   0x40
#define IPV6_INGRESS_ASSOC_BIT_DEST_ADDR  0x80

#define IPV6_INGRESS_ASSOC_DEFAULT_MASK  (IPV6_INGRESS_ASSOC_BIT_VRD | \
                                          IPV6_INGRESS_ASSOC_BIT_PROTO | \
                                          IPV6_INGRESS_ASSOC_BIT_L4INFO1 | \
                                          IPV6_INGRESS_ASSOC_BIT_L4INFO2 | \
                                          IPV6_INGRESS_ASSOC_BIT_SRC_ADDR | \
                                          IPV6_INGRESS_ASSOC_BIT_DEST_ADDR)

#define ADK_IPV6_EXCEED_ACTION_MARK_FOR_CONGESTION 0xc000
#define ADK_IPV6_VIOLATE_ACTION_MARK_FOR_CONGESTION 0xc000

#define ADK_IPV6_DEL_PBR_RULES_PENDING_MAX       512
/*! The exception handler is to drop the packet . */
#define    ADK_IPV6_EXCEPTION_LOC_DROP_OFFSET       0  
/*! The exception handler is on the CPU via PH. */
#define    ADK_IPV6_EXCEPTION_LOC_CPU_PH_OFFSET     1  
/*! The exception handler is on MPP */
#define    ADK_IPV6_EXCEPTION_LOC_MPP_OFFSET        2
/*! The exception handler is on the CPU via Netdriver. */
#define    ADK_IPV6_EXCEPTION_LOC_CPU_NOPH_OFFSET   3


#if 0
###/* No fo IPv6 Trees to be compacted Periodically */
####ifdef ADK_IPV6_COMPACT_UNLEARN_INGRESS_ASSOC
####define ADK_IPV6_NO_OF_TREES_COMPACT   3
####else
####define ADK_IPV6_NO_OF_TREES_COMPACT   4
####endif
#endif

#define ntohll(x) ( ( (uint64_t)(ntohl( (uint32_t)((x << 32) >> 32) )) << 32) |\
            ntohl( ((uint32_t)(x >> 32)) ) )

#define htonll(x) ntohll(x)

#if 0
#define ADK_IPV6_DP_LOG_ERROR debug_print
#endif
#define ADK_IPV6_DP_LOG_FATAL_ERROR(__msg) \
    ADK_DP_LOG_FATAL_ERROR(ADK_IPV6_MODULE_ID, __msg);
#define ADK_IPV6_DP_LOG_INFO_1S(__msg, __str) \
    ADK_DP_LOG_INFO_1S(ADK_IPV6_MODULE_ID, __msg, __str);
#define ADK_IPV6_DP_LOG_INFO_1D(__msg, __data) \
    ADK_DP_LOG_INFO_1D(ADK_IPV6_MODULE_ID, __msg, __data);
#define ADK_IPV6_DP_LOG_INFO_2D(__msg, __data1, __data2) \
    ADK_DP_LOG_INFO_2D(ADK_IPV6_MODULE_ID, __msg, __data1, __data2);
#define ADK_IPV6_DP_LOG_INFO_1D_DEBUG(__msg, __data) \
    ADK_DP_LOG_INFO_1D_DEBUG(ADK_IPV6_MODULE_ID, __msg, __data);
#define ADK_IPV6_DP_LOG_INFO_2D_DEBUG(__msg, __data1, __data2) \
    ADK_DP_LOG_INFO_2D_DEBUG(ADK_IPV6_MODULE_ID, __msg, __data1, __data2);

#define ADK_IPV6_LOG_LEVEL_GET(mod_handle) \
    ((adk_ipv6_info_t *)ADK_MODULE_INFO_PTR_GET(mod_handle))->adk_ipv6_dp_tables->logLevel

#define ADK_IPV6_LOG_EXT_API(mod_handle, api, fmt, args...) \
    ADK_LOG_EXT_API(sys_handle, mod_handle, api, fmt, ##args)

#define ADK_IPV6_DP_LOG_INFO(__msg, args...) \
    if ((adk_ipv6_dp_tables->logLevel & ADK_LOG_LEVEL_INFO )== ADK_LOG_LEVEL_INFO ){ \
        ADK_ERROR_WRITE("[ADK_IPV6_INFO_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(__msg, ##args);   \
    }

#define ADK_IPV6_DP_LOG_ERROR(__msg, args...) \
    if ((adk_ipv6_dp_tables->logLevel & ADK_LOG_LEVEL_ERROR )== ADK_LOG_LEVEL_ERROR ){ \
        ADK_ERROR_WRITE("[ADK_IPV6_ERROR_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(__msg, ##args);   \
    }

#define ADK_IPV6_DP_LOG_FATAL(__msg, args...) \
    if ((adk_ipv6_dp_tables->logLevel & ADK_LOG_LEVEL_FATAL )== ADK_LOG_LEVEL_FATAL ){ \
        ADK_ERROR_WRITE("[ADK_IPV6_FATAL_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(__msg, ##args);   \
    }

#define ADK_IPV6_DP_LOG_WARNING(__msg, args...) \
    if ((adk_ipv6_dp_tables->logLevel & ADK_LOG_LEVEL_WARNING )== ADK_LOG_LEVEL_WARNING ){ \
        ADK_ERROR_WRITE("[ADK_IPV6_WARNING_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(__msg, ##args);   \
    }

#define ADK_IPV6_DP_LOG_DEBUG(__msg, args...) \
    if ((adk_ipv6_dp_tables->logLevel & ADK_LOG_LEVEL_DEBUG )== ADK_LOG_LEVEL_DEBUG ){ \
        ADK_ERROR_WRITE("[ADK_IPV6_DEBUG_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(__msg, ##args);   \
    }

#define ADK_IPV6_LOG_ERROR(mod_handle, err, fmt, args...) \
    if (((ADK_IPV6_LOG_LEVEL_GET(mod_handle) & ADK_LOG_LEVEL_ERROR)  == ADK_LOG_LEVEL_ERROR )) { \
        ADK_ERROR_WRITE("[ADK_IPV6_ERROR_LOG: %s:%d] %s ", \
                        __FILE__, __LINE__, \
                        adk_sys_status_get(\
                            ADK_MODULE_SYS_HANDLE_GET (mod_handle), err)); \
        ADK_ERROR_WRITE(fmt, ##args);   \
    }

#define ADK_IPV6_LOG_FATAL(mod_handle, err, fmt, args...) \
    if (((ADK_IPV6_LOG_LEVEL_GET(mod_handle) & ADK_LOG_LEVEL_FATAL)  == ADK_LOG_LEVEL_FATAL )) { \
        ADK_ERROR_WRITE("[ADK_IPV6_FATAL_LOG: %s:%d] %s ", \
                        __FILE__, __LINE__, \
                        adk_sys_status_get(\
                            ADK_MODULE_SYS_HANDLE_GET (mod_handle), err)); \
        ADK_ERROR_WRITE(fmt, ##args);   \
    }

#define ADK_IPV6_LOG_WARNING(mod_handle, fmt, args...) \
    if (((ADK_IPV6_LOG_LEVEL_GET(mod_handle) & ADK_LOG_LEVEL_WARNING)  == ADK_LOG_LEVEL_WARNING )) { \
        ADK_ERROR_WRITE("[ADK_IPV6_WARNING_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(fmt, ##args);   \
    }

#define ADK_IPV6_LOG_INFO(mod_handle, fmt, args...) \
    if (((ADK_IPV6_LOG_LEVEL_GET(mod_handle) & ADK_LOG_LEVEL_INFO)  == ADK_LOG_LEVEL_INFO )) { \
        ADK_ERROR_WRITE("[ADK_IPV6_INFO_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(fmt, ##args);   \
    }

#define ADK_IPV6_LOG_DEBUG(mod_handle, fmt, args...) \
    if (((ADK_IPV6_LOG_LEVEL_GET(mod_handle) & ADK_LOG_LEVEL_DEBUG)  == ADK_LOG_LEVEL_DEBUG )) { \
        ADK_ERROR_WRITE("[ADK_IPV6_DEBUG_LOG: %s:%d] ", __FILE__, __LINE__); \
        ADK_ERROR_WRITE(fmt, ##args);   \
    }

#define ADK_IPV6_LOG_FORCE(fmt, args...) \
	            ADK_FORCE_LOG_ERROR(fmt, ##args);

#define ADK_IPV6_LOG_EXT_API(mod_handle, api, fmt, args...) \
	            ADK_LOG_EXT_API(sys_handle, mod_handle, api, fmt, ##args)

#define ADK_IPV6_LOG_FUNC_ENTRY(mod_handle) \
	            ADK_LOG_FUNC_ENTRY(sys_handle, mod_handle)

#define ADK_IPV6_LOG_FUNC_EXIT(mod_handle) \
	            ADK_LOG_FUNC_EXIT(sys_handle, mod_handle)

#define ADK_IPV6_DP_LOG_INFO_DEBUG    debug_print

#define ADK_IPV6_NON_ZERO_CNTR_WRITE(fmt, cntr) \
    if (cntr != 0) { \
        ADK_INFO_WRITE(fmt, cntr) \
    }

#define ADK_IPV6_COMPUTE_MULTICAST_DEST_MAC(dst_address, cal_dmac) \
        adk_ipv6_compute_multicast_dest_mac(dst_address, calc_dmac);

#define ADK_IPV6_COMPARE_MULTICAST_MAC(calc_dmac, dst_mac) \
        adk_ipv6_compare_multicast_mac(calc_dmac, dst_mac);

#ifdef ADK_UTILITY_COUNTER_ENABLE
#define INCREMENT_IPV6_UTILITY_COUNTER(counter) \
    do  \
    {       \
        if(adk_ipv6_dp_tables->isIpv6UtilityCntEnabled)  \
        {  \
            counter++;  \
        }  \
    }while(0);
#else
#define INCREMENT_IPV6_UTILITY_COUNTER(counter)
#endif
//#define ADK_IPV6_DP_PKT_DUMP(task, len) adk_pkt_dump(stdout, ((struct rte_mbuf *)task), len)
/*!
 * @enum adk_ipv6_cookie_values_t "adk_ipv6_internal.h"
 *
 * @brief The anumeration defines cookie values used in IPv6 ADK
 */
typedef enum adk_ipv6_cookie_values_e {
    
          /*! IPv6 Module cookie */
          ADK_IPV6_MODULE_COOKIE = ((ADK_IPV6_MODULE_ID << 24) + 1000),
          /*! IPv6 ingress flow cookie */
          ADK_IPV6_INGRESS_FLOW_COOKIE,
          /*! IPv6 egress flow cookie */
          ADK_IPV6_EGRESS_FLOW_COOKIE,
          /*! IPv6 Interface Cookie */
          ADK_IPV6_INTF_COOKIE,
          /*! Ipv6 Filter Rule Cookie */
          ADK_IPV6_RULESET_COOKIE,
          /*! Ipv6 VRD ID Cookie */
          ADK_IPV6_VRD_ID_COOKIE,
          /*! IPv6 Dscp group Cookie */
          ADK_IPV6_DSCP_GROUP_COOKIE,
          /*! IPv6 PBR Cookie */
          ADK_IPV6_PBR_COOKIE,
          /*! IPv6 ROUTE Cookie */
          ADK_IPV6_ROUTE_COOKIE,
          /*! IPv6 NEXT HOP Cookie */
          ADK_IPV6_NEXTHOP_COOKIE,
          /*! IPv6 CPU_NEXT HOP Cookie */
          ADK_IPV6_CPU_NEXTHOP_COOKIE,
          /*! IPv6 TEREDO CLIENT Cookie */
          ADK_IPV6_TEREDO_COOKIE,
          /*! IPv6 MULTICAST GROUP Cookie */
          ADK_IPV6_MULTICAST_GROUP_COOKIE,
          /*! IPv6 MULTICAST FLOW Cookie */
          ADK_IPV6_MULTICAST_FLOW_COOKIE,
          /*! IPv6 ONLINK DESTINATION Cookie */
          ADK_IPV6_ONLINK_COOKIE,
          /*! IPv6 Egress Translation handle Cookie */
          ADK_IPV6_EGR_TRANSLATE_COOKIE,
          /*! IPv6 Filter Rule handle Cookie */
          ADK_IPV6_FILTER_RULE_HANDLE_COOKIE,
          /*! IPv6 PBR Rule handle Cookie */
          ADK_IPV6_PBR_RULE_HANDLE_COOKIE,
          /*! Ipv6 PBR Rule Cookie */
          ADK_IPV6_PBR_RULESET_COOKIE,
          /*! Ipv6 Packet Capture point cookie */
          ADK_IPV6_PACKET_CAPTURE_COOKIE,
} adk_ipv6_cookie_values_t;

/*!
 * =============================================================
 * @struct adk_ipv6_linklocal_addr_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief The IPv6 Linklocal address databasestructure.
 *
 * This structure contains the interface link local ipv6 address
 *
 * =============================================================
 */
typedef struct adk_ipv6_linklocal_addr_s {
   /*! indiactes whether this location is validi & 
    * maintains the number of interface with same link local address
    */
   uint32_t count;
   /*! Virtual Routing Domain Id
    */
   uint32_t vrd_id;
   /*! IPv6 Address to the compression table
    */
   adk_ipv6_addr_t Ipv6Addr;
}adk_ipv6_linklocal_addr_t;

/*!
 * =============================================================
 * @struct adk_ipv6_comp_address_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief The IPv6 compression address structure.
 *
 * This structure contains compressed ipv6 address
 *
 * =============================================================
 */
typedef struct adk_ipv6_comp_address_s {
   /*! indiactes whether this location is valid 
    */
   bool validEntry;
   /*! Virtual Routing Domain Id 
    */
   uint32_t vrd_id;
   /*! IPv6 Address to the compression table
    */
   adk_ipv6_addr_t Ipv6Addr;
}adk_ipv6_comp_address_t;

/*!
 * =============================================================
 * @struct adk_ipv6_comp_action_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief enum for address compress action.
 *
 * This enum has action to manage address compression table
 *
 * =============================================================
 */
typedef enum adk_ipv6_comp_action_e {
    /*! Add entry to the compression table 
     *  and return Id
     */
   ADK_IPV6_COMPRESS_ADDRESS = 0,
   /*! Find and return already compressed address
    */
   ADK_IPV6_FIND_COMPRESSED_ID,
   /*! Remove entry from compress table
    */
   ADK_IPV6_REMOVE_COMPRESSION
}adk_ipv6_comp_action_t;

typedef struct {
    /*! The route lookup input parameters */
    adk_ipv6_route_rules_t ipv6RouteRules;
    /*! The route lookup output parameters */
   adk_ipv6_route_attr_t ipv6RouteAttr;
    /*! Route table Id */
    uint8_t ipv6RouteTableId;
}adk_ipv6_route_all_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief Holds the internal info of the IPV6 data structure.
 *
 * This structure holds the tree IDs, the function IDs and
 * other internal data required by the IPV6 module.
 *
 * =============================================================
 */
typedef struct adk_ipv6_info_s
{
    /*! The IPv6 module cookie */
    uint32_t cookie;
    /*! The MPP LLI function ID for direct bind */
    uint32_t mppLLI_directBindFnId;
    /*! The MPP ULI function ID for direct bind */
    uint32_t mppULI_directBindFnId;
    /*! The MPP ULI function ID for indirect bind */
    uint32_t mppULI_indirectBindFnId;
    /*! The ID for the entry function for packets injected from CPU
     *  to be sent out of on IPv6 tunnel interface.
     */
    uint32_t ipv6ProcessCpuPktsFnId;
    /*! The ID for the entry function for packets injected from CPU
     *  to be sent out of on IPv6oE interface.
     */
    uint32_t ipv6ProcessCpuEthPktsFnId;
    /*! The ID of the ipv6_tunnel_if_id_lookup tree. */
    uint32_t ipv6TunnelIfIdLookupTreeId;
    /*! The ID of the default function of ipv6_tunnel_if_id_lookup tree. */
    uint32_t ipv6TunnelIfIdLookupTreeDefaultFnId;
    /*! The ID of cpu nh lookup tree */
    uint32_t cpuNhLookupTreeId; 
    /*! The ID of the exception_handler_tree. */

    uint32_t exceptionHandlerTreeId;
    /*! The ID of the default exception handler function, used
     *  when no handler is registered.
     */
    uint32_t defaultExceptionHandlerFnId;
    /*! The ID of the function stored in the tree for the
     *  first stage when the user registered exception handler
     *  function is a MPP based function.
     */
    uint32_t firstStageMppExceptionFnId;
    /*! The ID of the ipv6_vrd_host_mode_tree. */
    uint32_t ipv6HostModeTreeId;
    /*! The ID of the ingress_flow_sip_tree. */
    uint32_t ingFlowSIPLookupTreeId;

    /*! The ID of the ipv6_ingress_assoc_flow_lookup Tree. */
    uint32_t ingAssocFlowLookupTreeId;
/* #ifdef ADK_XEON2_PROFILE_1 */
    /*! The ID of the ipv6_comp_ingress_assoc_flow_lookup */
    uint32_t ingAssocCompFlowLookupTreeId;
/* #endif */
    /*! The ID of the ipv6_ingress_assoc_icmp_flow_lookup Tree. */
    uint32_t ingAssocIcmpFlowLookupTreeId;
    /*! The ID of the ipv6_ingress_assoc_flow_lookup Tree 2. */
    uint32_t ingAssocFlow2LookupTreeId;
    /*! The ID of the ipv6_ingress_assoc_default function. */
    uint32_t ingAssocFlowDefaultFnId;

    /*! The ID of the multiPath route_lookup_tree. */
    uint32_t multiPathRouteTreeId;

    /*! The ID of the onlink_destination_tree. */
    uint32_t onLinkDestinationTreeId;

    /*! The ID of the ipv6_custom_bind_get_dest_flow Tree */
    uint32_t customBindGetDstFlowTreeId;

    /*! The ID of the route_and_adv_lookup_attr_tree1. */
    uint32_t routeAndAdvanceLookupAttrTree1Id;
    /*! The ID of the route_and_adv_lookup_attr_tree2. */
    uint32_t routeAndAdvanceLookupAttrTree2Id;
    /*! The ID of the default route_and_adv_lookup_attr_tree fn. */
    uint32_t routeAndAdvanceLookupAttrTreeDefaultFnId;

    /*! The ID of the egress_flow_attr_tree. */
    uint32_t egressFlowAttrTreeId;
    /*! The ID of the default egress_flow_attr_tree fn. */
    uint32_t egressFlowAttrTreeDefaultFnId;
    /*! The ID of the UDP egress_flow_attr_tree. */
    uint32_t egressUDPFlowAttrTreeId;
    /*! The ID of the default UDP egress_flow_attr_tree fn. */
    uint32_t egressUDPFlowAttrTreeDefaultFnId;

    /*! The ID of the egress_flow_dst_lookup_tree1[to 3] IDs.
     *  These trees are used to get the destination IPv6 address
     *  of a flow.
     */
    uint32_t egressFlowDstLookupTreeId[4];
    /*! The ID of the default fn for egress_flow_dst_lookup_tree*. */
    uint32_t egressFlowDstLookupTreeDefaultFnId;

    /*! The ID of the egress_fragment_id_tree. */
    uint32_t egressFragmentIdTreeId;

    /*! The ID of the teredo_client_lookup_tree. */
    uint32_t teredoClientTreeId;
    /*! The Id of IPV6 address Translation Look up Tree*/
    uint32_t addressTranslationLookupAttrTree;

    /*! The Id of IPV6 pbr lookup Tree */
    uint32_t pbrLookupTreeId; 

    /*! The Id of IPV6 egress pbr state Tree */
    uint32_t egressPbrStateTreeId; 

    /*! The Id of IPV6 checksum validate Tree */
    uint32_t checksumValidateTreeId;

    /*! The index allocated from the egressFragmentIdNsHandle. */
    uint32_t egressFragmentIdIndex;

    /*! The index allocated from the globalStatisticsNsHandle
     *  to hold the 256 byte global statistics.
     */
    uint32_t globalStatisticsIndex;

#if 0
    /*! The handle to the 16 byte name space entry. All the ingress
     *  flow IDs  allocated from this space are used as
     *  ingress association flow id .
     */
    rsrc_hdl_t ingressFlowStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. All the nexthop
     *  IDs allocated from this space are used as nexthop id for 
     *  routes , onlink destionations, cpu flow nexthops and 
     *  multicast flow id .
     */
    rsrc_hdl_t nextHopNsHandle;
    /*! The handle to the 16 byte name space entry. All the egress
     *  flow IDs are allocated from this space.
     */
    rsrc_hdl_t egressFlowStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. The fragment ID
     *  is stored in this name space.
     */
    rsrc_hdl_t egressFragmentIdNsHandle;
    /*! The handle to the 256 byte name space entry. There will
     *  be 2 indices allocated from this space and will be
     *  used to hold the global statistics. There are two seperate
     *  indices allocated as the system memory need not be
     *  contiguous for two contiguous indices.
     */
    rsrc_hdl_t globalStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. 
     *  This space will be used to hold the egress global statistics. 
     */
    rsrc_hdl_t globalEgressStatisticsNsHandle;

    /*! The teredo client namespace handle to hold
     *  teredo client status and information.
     */
    rsrc_hdl_t teredoClientNsHandle;
    /*! The teredo client namespace handle to hold
     *  teredo client status and information.
     */
    rsrc_hdl_t multipathRoutingNsHandle;
    /*! User Defined Namespace resource for ruleset ID */
    rsrc_hdl_t ipv6FilterRuleSetIDHandle;   
    /*! User Defined Namespace resource for interface statistics id */
    rsrc_hdl_t ipv6IfStatsIDHandle;
    /*! User Defined Namespace resource for filter statistics id */
    rsrc_hdl_t ipv6FilterRuleStatsHandle;
    /*! The ID of multicast group namespace */
    rsrc_hdl_t mcGrpNsId;
#else
    adk_rm_hdl_t ipv6IngClassifierRuleIDHandle;
    /*! The handle to the 16 byte name space entry. All the ingress
     *  flow IDs  allocated from this space are used as
     *  ingress association flow id .
     */
    adk_rm_hdl_t routeLookupRsrcHandle;
    /*! The handle to the 16 byte name space entry. All the ingress
     *  flow IDs  allocated from this space are used as
     *  ingress association flow id .
     */
    adk_rm_hdl_t ingressFlowStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. All the nexthop
     *  IDs allocated from this space are used as nexthop id for 
     *  routes , onlink destionations, cpu flow nexthops and 
     *  multicast flow id .
     */
    adk_rm_hdl_t nextHopNsHandle;
    /*! The handle to the 16 byte name space entry. All the egress
     *  flow IDs are allocated from this space.
     */
    adk_rm_hdl_t egressFlowStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. The fragment ID
     *  is stored in this name space.
     */
    adk_rm_hdl_t egressFragmentIdNsHandle;
    /*! The handle to the 256 byte name space entry. There will
     *  be 2 indices allocated from this space and will be
     *  used to hold the global statistics. There are two seperate
     *  indices allocated as the system memory need not be
     *  contiguous for two contiguous indices.
     */
    adk_rm_hdl_t globalStatisticsNsHandle;
    /*! The handle to the 16 byte name space entry. 
     *  This space will be used to hold the egress global statistics. 
     */
    adk_rm_hdl_t globalEgressStatisticsNsHandle;

    /*! The teredo client namespace handle to hold
     *  teredo client status and information.
     */
    adk_rm_hdl_t teredoClientNsHandle;
    /*! The teredo client namespace handle to hold
     *  teredo client status and information.
     */
    adk_rm_hdl_t multipathRoutingNsHandle;
    /*! User Defined Namespace resource for ruleset ID */
    adk_rm_hdl_t ipv6FilterRuleSetIDHandle;
    /*! User Defined Namespace resource for interface statistics id */
    adk_rm_hdl_t ipv6IfStatsIDHandle;
    /*! User Defined Namespace resource for filter statistics id */
    adk_rm_hdl_t ipv6FilterRuleStatsHandle;
    /*! User Defined Namespace resource for filter statistics id */
    adk_rm_hdl_t ipv6EgressIfDscpHandle;
    /*! User Defined Namespace resource for filter statistics id */
    adk_rm_hdl_t ipv6IfDscpHandle;
    /*! The ID of multicast group namespace */
    adk_rm_hdl_t mcGrpNsId;
#endif
    uint32_t   ipv6IfStatsID;

#if 0
    /*! The ID of filter lookup tree. */
    uint32_t filterRulesTreeId;
    /*! The ID of interface lookup tree */
    uint32_t interfaceLookupTreeId;
    /*! The ID of egress interface lookup tree */
    uint32_t egressInterfaceLookupTreeId;
    /*! The ID of egress interface dscp lookup tree */
    uint32_t egressIfDscpLookupTreeId;
    /*! The ID of QOS lookup tree */
    uint32_t qosTreeId;
    /*! The ID of next hop mac address tree */
    uint32_t nextHopMacAddrTreee;
    /*! The ID of multicast destination address tree */
    uint32_t mcFwdTreeId;
    /*! The ID of multicast destination address tree with only dip lookup*/
    uint32_t mcDipFwdTreeId;
    /*! The ID of multicast egress flow tree */
    uint32_t mcEgFlowTreeId;
    /*! The ID of interface dscp policing lookup tree */
    uint32_t ifDscpPoliceLookupTreeId; 

#endif
    /*! The ID of invalid interface id terminating funtion*/
    uint32_t ifInvalidFnId; 

    /*! The ID of ipv6 to ethernet ULI indirect IMC function */
    uint32_t ipv6ToEthULIIndirectIMCFnId; 
    /*! The ID of ipv6 to ethernet ULI direct IMC function */
    uint32_t ipv6ToEthULIDirectIMCFnId; 
    /*! The ID of ipv6 to ipsec ULI indirect IMC function */
    uint32_t ipv6ToIPsecULIIndirectIMCFnId;
    /*! The ID of ipv6 translation to ipv4 ULI indirect IMC function */
    uint32_t ipv6TranslateToIpv4ULIIndirectIMCFnId;
    /*! The ID of ipv6 to ipv4 ULI direct IMC function */
    uint32_t ipv6ToIpv4ULIDirectIMCFnId;
    /*! The ID of ipv6 to udp LLI direct IMC function */
    uint32_t ipv6ToUDPLLIDirectIMCFnId; 
    /*! The ID of ipv6 to udp ULI direct IMC function */
    uint32_t ipv6ToUDPULIDirectIMCFnId; 
    /*! The ID of ipv6 to IPsec LLI direct IMC function */
    uint32_t ipv6ToIPsecLLIDirectIMCFnId; 
    /*! The ID of ipv6 egress to IPsec ULI direct IMC function */
    uint32_t ipv6ToIPsecULIDirectIMCFnId; 
    /*! The ID of ipv6 to BFD LLI direct IMC function */
    uint32_t ipv6ToBFDLLIDirectIMCFnId; 
    /*! The ID of ipv6 LLI to ipv6 ULI Indirect IMC function */
    uint32_t ipv6LLIToIpv6ULIInDirectIMCFnId;
    /*! The ID of ipv6 LLI to gtpu LLI direct F2F IMC function */
    uint32_t ipv6ToGTPULLIDirectIMCFnId;
    /*! The ID of ipv6 LLI to gtpu LLI direct F2M IMC function */
    uint32_t ipv6ToGTPULLIDirectF2MIMCFnId;
    /*! The ID of ipv6 to BP-IWF ULI direct IMC function */
    uint32_t ipv6ToBpIwfULIDirectIMCFnId;
    /*! The ID of ipv6 to TWAMP direct IMC function */
    uint32_t ipv6ToTwampDirectIMCFnId;
    /*! Filter rule set handles */
    adk_ipv6_filter_rule_set_handle_t ruleSetHandles[ADK_IPV6_FILTER_RULE_SETS_MAX];
    /*! PBR rule set handles */
    adk_ipv6_pbr_policy_set_handle_t policySetHandles[ADK_IPV6_PBR_RULE_SETS_MAX];
    /* Database to store link local ipv6 address count */
    adk_ipv6_linklocal_addr_t linkLocalAddrDb[ADK_IPV6_MAX_LINK_LOCAL_ADDR];
    uint32_t  linkLocalAddrCount; 
/* #ifdef ADK_XEON2_PROFILE_1 */
    /*! Compressed IPv6 Addresses */
    adk_ipv6_comp_address_t  ipv6CompAddr[ADK_IPV6_MAX_COMP_IP_ADDRESS];
/* #endif */
    adk_ipv6_vrd_handle_t    vrdHandleList[ADK_IPV6_VRD_ID_MAX];
    uint32_t  tree_under_compaction;

    /*Entry function pointers in DP process*/
    uint16_t       cpuEntryFn_LLI[_DEST_FUNC_MAX];
    uint16_t       cpuEntryFn_ULI[_DEST_FUNC_MAX];

    adk_ipv6_dp_tables_t    *adk_ipv6_dp_tables;
    adk_rm_hdl_t            ipv6_dp_vrd_rsrc_id;
    adk_rm_hdl_t            ipv6_dp_virtual_if_rsrc_hdl;
    adk_rm_hdl_t            ipv6_dp_acl_rsrc_id;
    adk_rm_hdl_t            ipv6_dp_acl_stats_rsrc_id;
    adk_rm_hdl_t            ipv6PbrPolicySetIDHandle;  /* For PBR rule set IDs */
    adk_rm_hdl_t            ipv6PbrPolicyIDHandle; /* For PBR rule IDs */
    adk_rm_hdl_t            ipv6AddrStatsHandle;
    /* For IPv6 egress qos classifier rule IDs per interface */
    adk_rm_hdl_t            ipv6EgressQoSClassifierRuleIDHandle[ADK_DP_MAX_IPV6_INTERFACE];
    adk_ipv6_addr_prefix_t  ipv6_virtual_addr_prefix[ADK_IPV6_VIRTUAL_ADDR_MAX];
    uint16_t                ipv6_virtual_addr_stats_id[ADK_IPV6_VIRTUAL_ADDR_MAX];
    uint8_t                 numAddrIndex;
    adk_ipv6_addr_prefix_t  ipv6_sse_addr_prefix[ADK_IPV6_SSE_ADDR_MAX];
    uint16_t                ipv6_sse_addr_stats_id[ADK_IPV6_SSE_ADDR_MAX];
    uint8_t                 sseAddrIndex;

    uint32_t                pcap_id;
    uint32_t                mirror_id;
    bool                    enableHwCap;
    uint8_t                 logLevel;
    uint8_t                 totalIpv6Address;
    uint8_t                 maxIpv6Interfaces;
    uint8_t                 maxIpv6PbrRules;
    uint16_t                maxIpv6Routes;
    uint16_t                maxIpv6ClassifierRules;
    adk_acl_handle_t        cp_list;
#if 0
    adk_rm_hdl_t                            ipv6_dp_if_rsrc_id;
    adk_ipv6_dp_interface_entry_t           *ipv6_dp_if_table;
    adk_ipv6_dp_if_stats_entry_t            *ipv6_dp_if_stats_table[ADK_MAX_NUM_CORES];
    adk_ipv6_dp_global_statistics_t         *ipv6_dp_global_stats_table[ADK_MAX_NUM_CORES];
    adk_ipv6_dp_tunnel_interface_lookup_entry_t   *ipv6_dp_tnl_if_lookup_table;

    adk_rm_hdl_t                            ipv6_dp_route_rsrc_id;
    adk_lpm_handle_t                        ipv6_dp_route_lpm_table;
    adk_ipv6_dp_route_entry_t               *ipv6_dp_route_table;

    adk_rm_hdl_t                            ipv6_dp_acl_rsrc_id;
    adk_rm_hdl_t                            ipv6_dp_acl_stats_rsrc_id;
    adk_ipv6_dp_acl_action_entry_t          *ipv6_dp_acl_action_table;
    adk_ipv6_dp_acl_tree_t                  *ipv6_dp_acl_tree_table;

    adk_rm_hdl_t                            ipv6_dp_pbr_rsrc_id;
    adk_ipv6_dp_pbr_action_entry_t          *ipv6_dp_pbr_action_table;

    adk_ipv6_dp_pbr_state_t                 *ipv6_dp_pbr_state_table;
    adk_lpm_handle_t                        ipv6_dp_teredo_cli_lpm_table;

    adk_rm_hdl_t                            ipv6_dp_nexthop_rsrc_id;
    adk_ipv6_dp_nexthop_entry_t             *ipv6_dp_nexthop_table;

    adk_rm_hdl_t                            ipv6_dp_onlink_dest_rsrc_id;
    adk_ipv6_dp_onlink_dest_hash_entry_t    **ipv6_dp_onlink_dest_hash_table;
    adk_ipv6_dp_onlink_dest_entry_t         *ipv6_dp_onlink_dest_table;


    adk_rm_hdl_t                            ipv6_dp_reassembly_rsrc_id;
    adk_ipv6_dp_reassembly_hash_entry_t     **ipv6_dp_reassembly_hash_table;
    adk_ipv6_dp_reassembly_entry_t          *ipv6_dp_reassembly_table;
    adk_ipv6_dp_reassembly_list_t           *ipv6_dp_reassembly_list;
    adk_ipv6_dp_reassembly_lock_t           *ipv6_dp_reassembly_lock;

    adk_rm_hdl_t                            ipv6_dp_egress_flow_attr_rsrc_id;
    adk_ipv6_dp_egress_flow_attr_entry_t    *ipv6_dp_egress_flow_attr_table;
    adk_ipv6_dp_egress_assoc_stats_entry_t  *ipv6_dp_egress_assoc_stats_table[ADK_MAX_NUM_CORES];


    adk_rm_hdl_t                            ipv6_dp_ingress_assoc_rsrc_id;
    adk_lpm_handle_t                        ipv6_dp_ingress_assoc_lpm_table;
    adk_ipv6_dp_ingress_assoc_entry_t       *ipv6_dp_ingress_assoc_table;
    adk_ipv6_dp_ing_assoc_stats_entry_t     *ipv6_dp_ing_assoc_stats_table[ADK_MAX_NUM_CORES];

    adk_rm_hdl_t                            ipv6_dp_ingress_flow_sip_rsrc_id;
    adk_lpm_handle_t                        ipv6_dp_ingress_flow_sip_lpm_table;
    adk_ipv6_dp_ingress_flow_sip_entry_t    *ipv6_dp_ingress_flow_sip_table;

    adk_ipv6_dp_vrd_host_mode_entry_t       *ipv6_dp_vrd_host_mode_table;
    adk_ipv6_dp_excep_entry_t               *ipv6_dp_excep_table;
#endif
} adk_ipv6_info_t;   /* ------  end of struct adk_ipv6_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_ingress_flow_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief Holds the ingress flow rules and attributes.
 *
 * This structure holds the ingress flow rules and attributes.
 * It is stored in the flow handle.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ingress_flow_info_s
{
    /*! The IPv6 ingress flow cookie */
    uint32_t  cookie;
} adk_ipv6_ingress_flow_info_t;   /* ------  end of struct adk_ipv6_ingress_flow_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_ingress_mgroup_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief Holds the ingress mcast group rules and attributes.
 *
 * This structure holds the ingress multicast group rules and attributes.
 * It is stored in the flow handle.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ingress_mgroup_info_s
{
    /*! The Mcast Group info. */
    adk_ipv6_multicast_group_t mGrpInfo;
    /*! The IPv6 ingress flow cookie */
    uint32_t  cookie;
} adk_ipv6_ingress_mgroup_info_t;   /* ------  end of struct adk_ipv6_ingress_mgroup_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_qos_classifier_flow_info_t
 *
 * @brief
 *
 * Flow information for an IPv6 egress QOS classifier flow.
 * =============================================================
 */
typedef struct adk_ipv6_qos_classifier_flow_info_s {
    uint16_t precedence;
    adk_ipv6_qos_info_t flow_info;
    adk_acl_rule_t cp_rule;
} adk_ipv6_qos_classifier_flow_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_vrd_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * virtual routing domain information for a VRD handle.
 * =============================================================
 */
typedef struct adk_ipv6_vrd_info_s {
    uint32_t cookie;
    adk_ipv6_vrd_t vrdAttr;
} adk_ipv6_vrd_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_filter_rule_handle_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * virtual routing domain information for a VRD handle.
 * =============================================================
 */
typedef struct adk_ipv6_filter_rule_handle_info_s {
    uint32_t cookie;
    uint32_t filterStateId;
    uint32_t precedence;
    adk_ipv6_filter_rules_t  filterRules;
    adk_ipv6_filter_action_t filterActions;
    adk_acl_handle_t cp_list;
    adk_acl_rule_t cp_rule;
} adk_ipv6_filter_rule_handle_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_pbr_rule_handle_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * PBR rule information
 * =============================================================
 */
typedef struct adk_ipv6_pbr_rule_handle_info_s {
    uint32_t cookie;
    uint32_t filterStateId;
    uint32_t precedence;
    uint16_t rule_index; 
    adk_ipv6_pbr_rule_t  rule;
    adk_ipv6_pbr_action_t action;
    adk_acl_rule_t cp_rule;
} adk_ipv6_pbr_rule_handle_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_egress_flow_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief Holds the egress flow rules and attributes.
 *
 * This structure holds the egress flow rules and attributes.
 * It is stored in the flow handle.
 *
 * =============================================================
 */
typedef struct adk_ipv6_egress_flow_info_s
{
    /*! The flow info. */
    adk_ipv6_egress_association_attr_t ipv6Attr;
    /*! The IPv6 egress flow cookie */
    uint32_t cookie;
} adk_ipv6_egress_flow_info_t;   /* ------  end of struct adk_ipv6_egress_flow_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_ingress_flow_namespace_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief Holds the ingress flow stats and qos related info.
 *
 * This structure holds the ingress flow stats and new dscp value
 * with qos policy parameter flow index. it is stored in ingress
 * flow namespace memory.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ingress_flow_namespace_info_s
{
    /* DSCP info value */
    uint32_t      qosFlow;
    /* Tunnel Id */
    uint32_t      tunnelId;
    /* IPv6 Src Address for compressed ingress flow */
    adk_ipv6_addr_t srcIpv6Addr;
    /* IPv6 Src Port for compressed ingress flow */
    uint16_t srcPort;
    /* Unused space in namespace */
    uint8_t unused[30]; 
}  adk_ipv6_ingress_flow_namespace_info_t;   
/* ------  end of struct adk_ipv6_ingress_flow_namespace_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @enum adk_ipv6_ingress_flow_path_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief has enum to decide on ingress flow trees
 *
 * =============================================================
 */

typedef enum adk_ipv6_ingress_flow_path_e {
    /*! Learn only Non ICMP ingress association tree */
    ADK_IPV6_LEARN_ONLY_INGRESS_NON_ICMP_FLOW = 0,
    /*! Learn only ICMP ingress association tree */
    ADK_IPV6_LEARN_ONLY_INGRESS_ICMP_FLOW = 1,
    /*! Learn Both ICMP &  Non ICMP ingress association tree */
    ADK_IPV6_LEARN_BOTH_INGRESS_FLOW = 2,
}adk_ipv6_ingress_flow_path_t;


/*!
 * ==========================================================
 * @internal
 * @struct ipv6_filter_list_call_param_info
 *
 * @brief This structure is passed to call back function 
 * which list the all the filter rules
 *
 * =========================================================
 */
typedef struct ipv6_filter_list_call_param_info_s {
    uint32_t        ipv6Handle;
    uint32_t        numOfRulesExpected;
    uint32_t        numOfRulesRetrieved;
    adk_ipv6_filter_t   *filterList_p;
} ipv6_filter_list_call_param_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_ingress_global_statistics_t
 *
 * @brief The common statistics structure.
 *
 * This structure contains the statistics belong to ingress
 * global statistics.
 *
 * =============================================================
 */
typedef struct adk_ipv6_ingress_global_statistics_s
{
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were dropped as there is
     *  no exception handler registered for them.
     */
    uint32_t routeNotFoundDrop;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t routeNotFoundCpuException;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the MPP.
     */
    uint32_t routeNotFoundMppException;
    /*! Counts the PDUs for which no route was found in
     *  route_lookup_tree which were handled by an
     *  exception handler function residing on the CPU NOPH.
     */
    uint32_t routeNotFoundCpuNophException;

    /*! Counts the PDUs which were found to have issues
     *  while parsing which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t paramProblemDrop;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t paramProblemCpuException;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t paramProblemMppException;
    /*! Counts the PDUs which were found to have issues
     *  while parsing which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t paramProblemCpuNophException;

    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which
     *  were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t tooManyEhDrop;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which were
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t tooManyEhCpuException;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which
     *  were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t tooManyEhMppException;
    /*! Counts the PDUs which were found to have more
     *  than 8 extension headers while parsing which were
     *  handled by an exception handler function residing
     *  on the CPU NO PH .
     */
    uint32_t tooManyEhCpuNophException;

    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were dropped as there is
     *  no exception handler registered for them.
     */
    uint32_t resolveNexthopL2AddrDrop;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t resolveNexthopL2AddrCpuException;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the MPP.
     */
    uint32_t resolveNexthopL2AddrMppException;
    /*! Counts the PDUs for which the next hop L2 address
     *  is not resolved which were handled by an
     *  exception handler function residing on the CPU NO PH.
     */
    uint32_t resolveNexthopL2AddrCpuNophException;

    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were dropped
     *  as there is no exception handler registered for them.
     */
    uint32_t resolveDstL2AddrDrop;
    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were handled by an
     *  exception handler function residing on the CPU PH.
     */
    uint32_t resolveDstL2AddrCpuException;
    /*! Counts the PDUs for which the L2 address of
     *  the destination is not resolved which were handled
     *  by an exception handler function residing on the MPP.
     */
    uint32_t resolveDstL2AddrMppException;
    /*! Counts the PDUs for which the L2 address of the
     *  destination is not resolved which were handled by an
     *  exception handler function residing on the CPU NO PH.
     */
    uint32_t resolveDstL2AddrCpuNophException;

    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t hopLimitZeroDrop;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t hopLimitZeroCpuException;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t hopLimitZeroMppException;
    /*! Counts the PDUs for which the hop-limit has become 0
     *  which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t hopLimitZeroCpuNophException;

    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t mtuViolationDrop;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t mtuViolationCpuException;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t mtuViolationMppException;
    /*! Counts the PDUs for which the outgoing MTU is less
     *  than PDU size which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t mtuViolationCpuNophException;
    
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were dropped as there is no
     *  exception handler registered for them.
     */
    uint32_t unknownEgressFlowDrop;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t unknownEgressFlowCpuException;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t unknownEgressFlowMppException;
    /*! Counts the PDUs which do not have a flow in
     *  egress_flow_attr_tree which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t unknownEgressFlowCpuNophException;

    /*! Counts the PDUs for which the reassembly timer expired
     *  which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t reassemblyTimerExpiryDrop;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t reassemblyTimerExpiryCpuException;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t reassemblyTimerExpiryMppException;
    /*! Counts the PDUs for which the reassembly timer expired
     *  which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t reassemblyTimerExpiryCpuNophException;

    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were dropped as there is
     *  no exception handler registered for them.
     */
    uint32_t fragOverLapDrop;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the CPU PH.
     */
    uint32_t fragOverLapCpuException;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the MPP.
     */
    uint32_t fragOverLapMppException;
    /*! Counts the PDUs for which the current fragment overlapped
     *  the existing fragment which were handled by an exception
     *  handler function residing on the CPU NO PH.
     */
    uint32_t fragOverLapCpuNophException;
    
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were dropped as
     *  there is no exception handler registered for them.
     */
    uint32_t nonZeroSegmentsLeftDrop;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled by
     *  an exception handler function residing on the CPU PH.
     */
    uint32_t nonZeroSegmentsLeftCpuException;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled
     *  by an exception handler function residing on the MPP.
     */
    uint32_t nonZeroSegmentsLeftMppException;
    /*! Counts the PDUs for which the routing header has
     *  'segments left' as non-zero, which were handled by
     *  an exception handler function residing on the CPU NO PH.
     */
    uint32_t nonZeroSegmentsLeftCpuNophException;

    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  dropped as there is no exception handler registered
     *  for them.
     */
    uint32_t translateIcmpIpv6Drop;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t translateIcmpIpv6CpuException;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the MPP.
     */
    uint32_t translateIcmpIpv6MppException;
    /*! Counts the ICMP IPv6 translation PDUs, which were
     *  handled by an exception handler function residing
     *  on the CPU NO PH .
     */
    uint32_t translateIcmpIpv6CpuNophException;

    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and
     *  there was no exception handler registered for them.
     */
    uint32_t mcLkpFailDrop;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and
     *  handled by an exception handler function residing
     *  on the CPU PH.
     */
    uint32_t mcLkpFailCpuException;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and
     *  handled by an exception handler function residing
     *  on the MPP.
     */
    uint32_t mcLkpFailMppException;
    /*! Counts the multicast PDUs dropped due to multicast
     *  entry not present in multicast forwarding table and
     *  handled by an exception handler function residing
     *  on the CPU NO PH.
     */
    uint32_t mcLkpFailCpuNophException;

    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and there was no exception handler registered
     *  for them.
     */
    uint32_t mcRPFCheckFailDrop;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t mcRPFCheckFailCpuException;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t mcRPFCheckFailMppException;
    /*! Counts the multicast PDUs dropped due to RPF check
     *  fail and handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t mcRPFCheckFailCpuNophException;
    
    /*! Counts the PDUs for which the reassembly max pdu length 
     *  reached which were dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t reassemblyfragMaxPduLengthDrop;
    /*! Counts the PDUs for which the reassembly max pdu length 
     *  reached which were handled by an exception handler function
     *  residing on the CPU PH.
     */
    uint32_t reassemblyfragMaxPduLengthCpuException;
    /*! Counts the PDUs for which the reassembly  max pdu length 
     *  reached which were handled by an exception handler function
     *  residing on the MPP.
     */
    uint32_t reassemblyfragMaxPduLengthMppException;
    /*! Counts the PDUs for which the reassembly max pdu length 
     *  reached which were handled by an exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t reassemblyfragMaxPduLengthCpuNophException;

    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t illegalSrcAddrTranslateDrop;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t illegalSrcAddrTranslateDropCpuException;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t illegalSrcAddrTranslateDropMppException;
    /*! Counts the PDUs for which the illegal Src ipv6 recived for translation
     *   are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t illegalSrcAddrTranslateDropCpuNophException;
    
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t UDPIngressChecksumInvalidDrop;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t UDPIngressChecksumInvalidDropCpuException;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t UDPIngressChecksumInvalidDropMppException;
    /*! Counts the PDUs for which the UDP checksum is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t UDPIngressChecksumInvalidDropCpuNophException;

    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler
     *  registered for them.
     */
    uint32_t UDPIngressLengthInvalidDrop;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU PH.
     */
    uint32_t UDPIngressLengthInvalidDropCpuException;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the MPP.
     */
    uint32_t UDPIngressLengthInvalidDropMppException;
    /*! Counts the PDUs for which the UDP length is invalid
     *  and are dropped as there is no exception handler function
     *  residing on the CPU NO PH.
     */
    uint32_t UDPIngressLengthInvalidDropCpuNophException;

    /*! Common statistics */
    /*! Counts the invalid length PDUs which were dropped */
    uint32_t invalidPayloadLen;
    /*! Counts the PDUs for which route action in
     *  route_lookup_tree is invalid, which were dropped.
     */
    uint32_t invalidRouteAction;
    /*! Counts the PDUs for which association attributes
     *  are not found.
     */
    uint32_t noAttr;
    
    /*! Number of ICMP PDUs with invalid type encountered during
     *  translation to IPv4.
     */
    uint32_t translateIcmpInvalidType;
    /*! Number of ICMP PDUs with type meant for just the next hop
     *  encountered during translation to IPv4.
     */
    uint32_t translateIcmpSingleHopType;
    /*! Number of ICMP PDUs dropped due to checksum failure */
    uint32_t translateChecksumFailure;
    /*! Number of ICMP PDUs dropped due to WKP and Non-Global SIP DIP*/
    uint32_t translateWkpNonGlobalIp;
    /*! Number of ICMP PDUs dropped due to fragmented ICMP pdu received*/
    uint32_t translateFragICMPDrop;
    /*! Number of ICMP PDUs dropped due to invalid extn Hdr combination */
    uint32_t translateInvalidExtnHdr;
    
    
    /*! Number of PDUs dropped due to QOS action drop */
    uint32_t  qosActionDropPdus;
    /*! Number of PDUs forwrded due to QOS action forward */
    uint32_t  qosActionFwdPdus;
    
    /*! Number of link local LLI PDUs */
    uint32_t  linkLocalLLIPdus;
    /*! Number of hashInsert update fails for multipath next hop*/
    uint32_t  multiPathHashInsertFail;
    /*! Number of hashDelete fails*/
    uint32_t  HashDeleteFail;
    /*! Number of Invalid interface PDU*/
    uint32_t  InvalidIfFail;
    /*! Number of PDUs dropped due to no ingress assoc */
    uint32_t  noIngressAssociation;
    /*! Number of PDUs dropped due to invalid ingress assoc mask */
    uint32_t  unknownAssociationMaskCnt;

    /* Reserved, keeping the size of structure to 512 */
    uint8_t reserved[160];
} adk_ipv6_ingress_global_statistics_t;   /* ------  end of struct adk_ipv6_ingress_global_statistics_t  ------ */


/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_egress_global_statistics_t
 *
 * @brief The common statistics structure.
 *
 * This structure contains the statistics belong to ingress
 * global statistics.
 *
 * =============================================================
 */
typedef struct adk_ipv6_egress_global_statistics_s
{

    /*! Number of PDUs forwarded */
    uint32_t fwdPdus;
    /*! Number of PDUs coming into the ULI */
    uint32_t uliPdus;
    /*! Number of PDUs dropped due to invalid destination address */
    uint32_t  forwardingInvalidDstAddr;
    /*! Number of PDUs for which QOS policy tree look up failed */
    uint32_t  qosPolicyLookupFailedPDUs;
    /*! Number of PDUs sent for ISATAP encapsulation */
    uint32_t  encapIsatapPdus;
    /*! Number of PDUs sent for 6to4  encapsulation */
    uint32_t  encap6to4Pdus;
    /*! Number of PDUs sent for teredo  encapsulation */
    uint32_t  encapTeredoPdus;
    /*! Number of link local PDUs */
    uint32_t  linkLocalPdus;
    /* No more statistics can be added as we have ran out of
     * pre-allocated namespace entry size. No reserved bytes
     * left */
} adk_ipv6_egress_global_statistics_t;   /* ------  end of struct adk_ipv6_egress_global_statistics_t  ------ */

typedef struct adk_ipv6_multicast_flow_info_s {

    adk_ipv6_multicast_flow_t mcFlow;
    uint32_t          ingFlowId;
    /*! Indicate the route cookie 
     * ADK_IPV6_MULTICAST_FLOW_COOKIE
     */
    uint32_t cookie;

} adk_ipv6_multicast_flow_info_t;

typedef struct adk_ipv6_pktcap_internal_info_s
{
        /*! The flow info. */
        adk_ipv6_pktcap_info_t flowInfo;
        /*! The IPv6 ingress flow cookie */
        uint32_t  cookie;
} adk_ipv6_pktcap_internal_info_t;   /* ------  end of struct adk_ipv6_packet_capture_info_t  ------ */

/*! IPv6 inress flow apecific definitions */
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_VRD_ID  0
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_L2FLOW 1
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_FLOW_LABEL 2
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_PROTOCOL 3
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_UDP_SRC_PORT 4
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_UDP_DST_PORT 5
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_SRC_ADDR 6
#define ADK_IPV6_INGRESS_FLOW_ELEMENT_DST_ADDR 7

#define ADK_IPV6_INGRESS_FLOW_DATA_VRD_ID  0
#define ADK_IPV6_INGRESS_FLOW_DATA_INTERFACE 1
#define ADK_IPV6_INGRESS_FLOW_DATA_FLOW_LABEL 2
#define ADK_IPV6_INGRESS_FLOW_DATA_PROTOCOL 3
#define ADK_IPV6_INGRESS_FLOW_DATA_UDP_SRC_PORT 4
#define ADK_IPV6_INGRESS_FLOW_DATA_UDP_DST_PORT 5
#define ADK_IPV6_INGRESS_FLOW_DATA_SRC_ADDR 6
#define ADK_IPV6_INGRESS_FLOW_DATA_DST_ADDR 10
#define ADK_IPV6_INGRESS_FLOW_PATTERN_NUM_WORDS 14
#define ADK_IPV6_INGRESS_FLOW_NUM_PATTERNS 8

/*! IPv6 filter specific definitions*/
#define ADK_IPV6_IN_FILTER_TREE_NAME          "adk_ipv6.ipv6_filter_rules_ingress_tree"
#define ADK_IPV6_OUT_FILTER_TREE_NAME         "adk_ipv6.ipv6_filter_rules_egress_tree"
#define ADK_IPV6_FILTER_ELEMENT_L2IF          0
#define ADK_IPV6_FILTER_ELEMENT_DSCP          1
#define ADK_IPV6_FILTER_ELEMENT_FLOW_LABEL    2 
#define ADK_IPV6_FILTER_ELEMENT_SADDR         3
#define ADK_IPV6_FILTER_ELEMENT_DADDR         4
#define ADK_IPV6_FILTER_ELEMENT_DST_OPT_TYPE  5
#define ADK_IPV6_FILTER_ELEMENT_ROUTING_TYPE  6
#define ADK_IPV6_FILTER_ELEMENT_FRAG          7
#define ADK_IPV6_FILTER_ELEMENT_MOBILITY_TYPE 8
#define ADK_IPV6_FILTER_ELEMENT_PROTO         9 
#define ADK_IPV6_FILTER_ELEMENT_PROTO_INFO1   10
#define ADK_IPV6_FILTER_ELEMENT_PROTO_INFO2   11
#define ADK_IPV6_FILTER_ELEMENT_PROTO_INFO3   12

#define ADK_IPV6_FILTER_DATA_L2IF          0
#define ADK_IPV6_FILTER_DATA_DSCP          1
#define ADK_IPV6_FILTER_DATA_FLOW_LABEL    2 
#define ADK_IPV6_FILTER_DATA_SADDR         3
#define ADK_IPV6_FILTER_DATA_DADDR         7
#define ADK_IPV6_FILTER_DATA_DST_OPT_TYPE  11
#define ADK_IPV6_FILTER_DATA_ROUTING_TYPE  12
#define ADK_IPV6_FILTER_DATA_FRAG          13
#define ADK_IPV6_FILTER_DATA_MOBILITY_TYPE 14
#define ADK_IPV6_FILTER_DATA_PROTO         15 
#define ADK_IPV6_FILTER_DATA_PROTO_INFO1   16
#define ADK_IPV6_FILTER_DATA_PROTO_INFO2   17
#define ADK_IPV6_FILTER_DATA_PROTO_INFO3   18

#define ADK_IPV6_FILTER_NUM_ELEMENTS       18
#define ADK_IPV6_FILTER_NUM_ELEMENTS_TMP    12
#define ADK_IPV6_FILTER_NUM_PATTERNS          8
/* bit 6 is gettting used for stateful filtering*/
#define ADK_IPV6_FILTER_STATEFUL_FILTER_STATE_OFFSET      6
/* bit 5 is gettting used for enable DSCP map bit */
#define ADK_IPV6_FILTER_DSCP_MAP_STATE_OFFSET      5
/* bit 4 is getting used for QOS policing enable state */
#define ADK_IPV6_FILTER_POLICE_STATE_OFFSET 4
/* bit 3 is getting used for PH info */
#define ADK_IPV6_FILTER_CPU_NOPH_OFFSET     3
/* bit 2 is getting used for enabling counter */
#define ADK_IPV6_FILTER_CNTR_STATE_OFFSET   2

#define ADK_IPV6_FILTER_MAX_PARAM_SIZE      24
#define IPV6_IF_FILTER_RULE_SET_WIDTH 10


/*! The maximum number of ACL rules that can be deleted or modified in a 
 * given ACL rule set with commit pending */
#define ADK_IPV6_DEL_FILTER_RULES_PENDING_MAX    64
/* IPv6 QOS policing parmaters size */
#define IPV6_POLICE_PARAM_SIZE 64

/* Interface lookup tree action offset*/
#define ADK_IPV6_INTERFACE_ENABLE_INGRESS_FILTERING    0
#define ADK_IPV6_INTERFACE_ENABLE_PBR_INGRESS_LOOKUP   1 
#define ADK_IPV6_INTERFACE_ENABLE_L3_DSCP_REMARK       2 
#define ADK_IPV6_INTERFACE_ENABLE_POLICING             3
#define ADK_IPV6_INTERFACE_INGRESS_FILTER_RULE_SET_ID  8
#define ADK_IPV6_INTERFACE_VRD_ID_OFFSET	       18

/* Default rules present in the tree */
#define ADK_IPV6_DEFAULT_OTREESET_RULES 2

/*VRD ID related defs*/
#define ADK_IPV6_VRD_ID_MASK                    ((1 << ADK_IPV6_VRD_ID_WIDTH) - 1)


/*! Internal macro definitions */
/* Macro to compare IPv6 addresses */
#define ADK_IPV6_COMPARE_IPV6_ADDRS(__addr1, __addr2) \
    (((__addr1)[0] != (__addr2)[0]) || ((__addr1)[1] != (__addr2)[1]) || \
     ((__addr1)[2] != (__addr2)[2]) || ((__addr1)[3] != (__addr2)[3]))

/*! Macro to LOG ingress flow information */
#define ADK_IPV6_LOG_INGRESS_FLOW_INFO(\
            __sysHandle,\
            __ipv6Handle,\
            __inputInfo) \
    { \
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle),"Input Ingress Flow Information\n");\
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle), \
                "Rule mask value: %d, InterfaceHandle: %"ADKPRIh", Flow Label: %d, Protocol: %d\n",\
                __inputInfo->ipv6Rules.rulesMask,__inputInfo->ipv6Rules.interfaceHandle,\
                __inputInfo->ipv6Rules.flowLabel,__inputInfo->ipv6Rules.protocolType); \
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle), \
                "Destination address: 0x%x, 0x%x, 0x%x, 0x%x\n",\
                __inputInfo->ipv6Rules.dstIpv6Addr.addr[0],__inputInfo->ipv6Rules.dstIpv6Addr.addr[1],\
                __inputInfo->ipv6Rules.dstIpv6Addr.addr[2],__inputInfo->ipv6Rules.dstIpv6Addr.addr[3]);\
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle),\
                "Destination address prefix width: %d\n",\
                __inputInfo->ipv6Rules.dstPrefixLength);\
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle), \
                "Source address: 0x%x, 0x%x, 0x%x, 0x%x\n",\
                __inputInfo->ipv6Rules.srcIpv6Addr.addr[0],__inputInfo->ipv6Rules.srcIpv6Addr.addr[1],\
                __inputInfo->ipv6Rules.srcIpv6Addr.addr[2],__inputInfo->ipv6Rules.srcIpv6Addr.addr[3]);\
        ADK_LOG_INFO(__sysHandle, (__ipv6Handle),\
                "Source address prefix width: %d\n",\
                __inputInfo->ipv6Rules.srcPrefixLength);\
    }\



/*!
 * The internal function declarations.
 */

adk_status_t
adk_ipv6_config_tunnel_cpu_path(adk_module_handle_t ipv6Handle,
                                uint32_t adk_interface_id,
                                uint32_t cpu_interface_id,
                                bool addEntry);

void adk_ipv6_prefix_mask( uint32_t prefix, uint32_t *mask);

adk_status_t
adk_ipv6_interface_hw_rule (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        bool addEntry);
adk_status_t
adk_ipv6_egress_interface_hw_rule (
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        bool addEntry);
adk_status_t
adk_ipv6_interface_dscp_hw_build_pattern ( 
        adk_module_handle_t ipv6Handle,
        adk_interface_handle_t ipv6IfHandle,
        const adk_ipv6_interface_ingress_dscp_t *ipv6Conf_p,
        const uint32_t *policingHandle_p,
        bool addEntry);

bool 
adk_ipv6_interface_valid_handle(
        adk_interface_handle_t ipv6IfHandle);

bool 
adk_ipv6_dscp_group_valid_handle(
        adk_dscp_group_handle_t dscpGrpHandle);

bool 
adk_ipv6_pbr_valid_handle( 
        adk_flow_handle_t  ruleHandle);

bool 
adk_ipv6_route_valid_handle( 
        adk_flow_handle_t  routeHandle);

bool 
adk_ipv6_nexthop_valid_handle( 
        adk_flow_handle_t  nexthopHandle);

bool 
adk_ipv6_multicast_group_valid_handle( 
        adk_flow_handle_t  multicast_groupHandle);

bool 
adk_ipv6_multicast_flow_valid_handle( 
        adk_flow_handle_t  multicast_flowHandle);

bool 
adk_ipv6_onlink_valid_handle( 
        adk_flow_handle_t  onlinkHandle);
bool 
adk_ipv6_valid_module_handle(
        adk_module_handle_t ipv6Handle);

adk_status_t
adk_ipv6_interface_dscp_pcp_map_config(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t ipv6IfHandle,
    bool enable);
bool adk_ipv6_valid_rule_set_handle(
                adk_ipv6_filter_rule_set_handle_t filterRuleSetHandle);

bool adk_ipv6_valid_policy_set_handle(
    adk_ipv6_pbr_policy_set_handle_t pbrPolicySetHandle);

bool adk_ipv6_vrd_valid_handle(adk_ipv6_vrd_handle_t ipv6VrdHandle,
                                     uint32_t        *vrdId);

bool adk_ipv6_valid_handle(adk_module_handle_t ipv6Handle);

bool adk_ipv6_valid_pbr_rule_handle(
                adk_handle_t pbrRuleHandle);

adk_status_t adk_ipv6_shutdown (adk_module_handle_t ipv6Handle);


uint32_t adk_ipv6_extract_bits( uint32_t value, uint32_t Offset, uint32_t size);


adk_status_t 
adk_ipv6_pbr_print_rules (adk_module_handle_t ipv6Handle,
                          adk_ipv6_tree_conf_t *ipv6TreeConf_p);


bool
adk_ipv6_address_compression_manage(adk_module_handle_t ipv6Handle,
                        adk_ipv6_addr_t address,
                        uint32_t vrd_id, 
                        uint32_t *compDestId, 
                        adk_ipv6_comp_action_t action);

bool
adk_ipv6_if_pktcap_valid_handle(adk_flow_handle_t ipv6IfPktCapHandle);

void
adk_ipv6_mod_id_str_print(uint32_t dest_mod_id);

adk_status_t
adk_ipv6_if_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

void adk_ipv6_route_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value);

adk_status_t
adk_ipv6_route_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_pbr_tbl_dump(adk_module_handle_t ipv6Handle);

void adk_ipv6_ingress_assoc_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value);
adk_status_t
adk_ipv6_ingress_assoc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_filter_tbl_dump(adk_module_handle_t ipv6Handle);

adk_status_t
adk_ipv6_nexthop_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_egress_assoc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_vrd_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_excep_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_onlink_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_reass_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_ingress_sip_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

void adk_ipv6_ingress_sip_dump_cb_fn(adk_lpm_handle_t __attribute__((unused)) lpm, 
                          __attribute__((unused)) void *rock,
                           uint8_t *key, 
                           uint16_t depth, 
                           adk_handle_t value);
adk_status_t
adk_ipv6_dest_mtu_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_fc_tbl_dump(adk_ipv6_dp_tables_t  *ipv6_tables);

adk_status_t
adk_ipv6_dump_tables(adk_module_handle_t ipv6Handle, 
        adk_ipv6_tree_conf_t *ipv6TreeConf_p);


adk_status_t
adk_ipv6_lpm_add(adk_module_handle_t ipv6_handle,
        const char *table_name,
        adk_ipv6_dp_lpm_table_t *lpm_tbl,
        uint8_t *input_ptn,
        uint16_t size,
        adk_handle_t index,
        adk_handle_t *old_index);

adk_status_t
adk_ipv6_lpm_delete(adk_module_handle_t ipv6_handle,
        uint8_t *table_name,
        adk_ipv6_dp_lpm_table_t *lpm_tbl,
        uint8_t *input_ptn,
        uint16_t size); 


void adk_ipv6_dp_write_flow_info(adk_ipv6_dest_flow_info_t src,
        adk_ipv6_dp_flow_info_t **dest);

#ifdef ADK_DEVICE_SNR
adk_status_t
adk_ipv6_tnl_intf_acl_config(
    adk_module_handle_t ipv6Handle,
    adk_interface_handle_t if_handle,
    bool enable_acl);
#endif
bool adk_ipv6_valid_acl_rule_handle(
                adk_handle_t filterRuleHandle);

adk_status_t  
adk_ipv6_internal_get_filter_rules ( 
        adk_module_handle_t ipv6Handle,
        adk_ipv6_filter_rule_set_handle_t filterRuleSetHandle,
        uint32_t numOfRules,
        adk_ipv6_filter_t *filter_p);

adk_status_t  
adk_ipv6_internal_get_num_filter_rules ( 
        adk_module_handle_t ipv6Handle,
        adk_ipv6_filter_rule_set_handle_t filterRuleSetHandle,
        uint32_t *numOfRules_p);

/*!
 * The internal function declarations.
 * for Pbr Routing Structure.
 * This is stored in the routeHandle Flow Memory 
 * Used for adk_ipv6_pbr_entry_create and adk_ipv6_pbr_entry_delete APIs'
 */
typedef struct adk_ipv6_pbr_Route_attr_s
{
    /*! Indicate that Route being added is Multi-NextHop attached Route.
     *  action field should be set to ADK_IPV6_MULTIPATH_ROUTE_ENABLE,
     *  for Multipath Routing.
     */
    bool action;
    /*! Indicate the Max Hops configured coressponding to the Route Handle
     */
    uint32_t maxNextHopAttached;
    /*IPv6 Pbr Rule is Stored in the Flow Memory*/
    adk_ipv6_pbr_info_t pbrRoute;
    /*! Indicate the PBR cookie 
     * ADK_IPV6_PBR_COOKIE
     */
    uint32_t cookie;
} adk_ipv6_pbr_Route_attr_t; 

/*!
 * The internal function declarations.
 * for Multipath Routing Structure.
 * This is stored in the routeHandle Flow Memory 
 * Used for adk_ipv6_route_add and adk_ipv6_route_delete APIs'
 */
typedef struct adk_ipv6_multipathRoute_attr_s
{
    /*! Indicate that Route being added is Multi-NextHop attached Route.
     *  action field should be set to ADK_IPV6_MULTIPATH_ROUTE_ENABLE,
     *  for Multipath Routing.
     */
    bool action;
    /*! Indicate the Max Hops configured coressponding to the Route Handle
     */
    uint32_t maxNextHopAttached;
    /*IPv6 DIP and Prefix is Stored in the Flow Memory*/
    adk_ipv6_route_rules_t multipathRoute;
    /*! Indicate the route cookie 
     * ADK_IPV6_ROUTE_COOKIE
     */
    uint32_t cookie;
} adk_ipv6_multipathRoute_attr_t; 

 /*!
 * =============================================================
 * @struct adk_ipv6_multicast_group_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This internal data structure contains information of multicast_group that
 * needs to be added into multicast_group peer list.
 *
 * =============================================================
 */
typedef struct adk_ipv6_multicast_group_info_s {

    /*! information of multicast_group
     */ 
    adk_ipv6_multicast_group_t  groupInfo;
    /*! Indicate the multicast group cookie 
     * ADK_IPV6_MULTICAST_GROUP_COOKIE
     */
    uint32_t cookie;

}adk_ipv6_multicast_group_info_t; /* -- end of struct adk_ipv6_multicast_group_info_t - */

 /*!
 * =============================================================
 * @struct adk_ipv6_onlink_destination_internal_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief
 *
 * This internal data structure contains information of onlink destination that
 * needs to be added into Onlink list.
 *
 * =============================================================
 */
typedef struct adk_ipv6_onlink_destination_internal_s {

    /*! information of onlink
     */ 
    adk_ipv6_onlink_destination_info_t  onlinkInfo;
    /*! Indicate the onlink cookie 
     * ADK_IPV6_ONLINK_COOKIE
     */
    uint16_t hash_bucket_index;
    uint32_t cookie;

}adk_ipv6_onlink_destination_internal_t; /* -- end of struct adk_ipv6_onlink_destination_internal_t - */

/*!
 * The internal function declarations.
 * for Multipath Routing Structure.
 * This is stored in the Namespace of Multipath ROute Handle
 * Used for attach, detach APIs of multipath
 */
typedef struct adk_ipv6_multipathRouteNamespace_attr_s
{
    /*! Indicate the Max Hops configured coressponding to the Route Handle
     */
     uint8_t maxNextHopAttached;
    /*! Indicate the currentOffset to read in DP.
     */
     uint8_t currentOffsetOfNextHop;
    /*! Indicate the list of NextHop pointer attached to the Route Handle
     * this points to the Namespace memory
     */
     uint32_t nextHopHandle_p[ADK_IPV6_MAX_NEXT_HOP_SUPPORTED];
} adk_ipv6_multipathRouteNamespace_attr_t;


typedef struct adk_ipv6_nextHopHandle_ns_info_s
{
  /*! Indicates the Reference Count used to keep track of the Used nextHop Handle
   */
  uint8_t refCount;
  /*! Indicates the weight associated with the NextHopHandle
   * Weigths are expected to be in percentage intotal of 100.
   */
  uint8_t weight;
} adk_ipv6_nextHopHandle_ns_info_t;


/*!
 * =============================================================
 * @internal
 * @struct ipv6_interface_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Interface information for an IP interface handle.
 * =============================================================
 */
typedef struct ipv6_interface_dscp_grp_list_s {
    /*! Indicate the policing flow handle .
     */
    adk_dscp_group_handle_t          dscpGrpHandle;
    /*! Indicate the policing flow handle .
     */
    bool                       isValid;
}ipv6_if_dscp_grp_list_t;

#ifdef ADK_DEVICE_SNR
/*!
 * =============================================================
 * @internal
 * @struct ipv6_inner_ip_tbl_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Inner IP table info state for IP TNL if ACL mapping.
 * =============================================================
 */

typedef struct ipv6_inner_ip_tbl_info_s{
    uint32_t src_addr[4];
    uint32_t dst_addr[4];
    uint32_t spd_id;
    bool     spd_valid;
    uint8_t  next_header;
}ipv6_inner_ip_tbl_info_t;
#endif
/*!
 * =============================================================
 * @internal
 * @struct ipv6_interface_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Interface information for an IP interface handle.
 * =============================================================
 */
typedef struct ipv6_interface_info_s {
    /*! Indicate the interface cookie.
     */
    uint32_t cookie;
    /*! Indicate the ingress stat id
     */
    uint32_t stats_id;
    /*! Indicate the egress stat id . 
     *It can be a statistics id for tunnel interface 
     * and Mtm shared param for other interfaces 
     */
    uint32_t EgrStatsId;
    /*! Egress Sys egress stats id.
     * stats collected after QOS 
     */
    uint32_t SysEgrStatsId;
    /*! Indicate the policing flow handle .
     */
    adk_ipv6_qos_policing_flow_handle_t policingHandle;
    /*! Indicate the interface type
     */
    adk_ipv6_interface_type_t ifType;
    /*! Indicate the interface attributes.
     */
    union
    {
        /*! Indicate the regular interface attributes.
         */
        adk_ipv6_interface_t          regularIf;
        /*! Indicate the virtual interface attributes.
         */
        adk_ipv6_virtual_interface_t  virtualIf;
  
    }if_info_u;

    /*!
     * Enable L3 dscp remarking  
     */
    bool enableL3Remark;
    /*!
     * Packet capture is enalbed at interface level 
     */
    bool enableIngressPacketCapture;
    bool enableEgressPacketCapture;

    /*! Indicate the number of ip address attached to the IPv6 interface.
     */
    uint8_t numAddr;
    /*! Indicate the ip address attached to the IPv6 interface.
     */
    adk_ipv6_addr_prefix_t ipv6_prefix[IPV6_SEC_ADDR_MAX ];

    /* For ipv6 primary address */
    adk_ipv6_addr_prefix_t      ipv6_addr_prefix;
    /* ipv6 address stats id */
    uint16_t    ipv6_addr_stats_id;
    uint16_t    ipv6_prefix_stats_id[IPV6_SEC_ADDR_MAX];
    /*! Indicate the ingressfilter ruleset handle.
     */
    adk_ipv6_filter_rule_set_handle_t ingressRuleSetHandle;
    /*! Indicate the filter egress ruleset handle.
     */
    adk_ipv6_filter_rule_set_handle_t egressRuleSetHandle;
#ifdef ADK_HOS_OLD_IMPL
    /*! HOS handle for IP termination table.
     */
    adk_hos_rule_id_t ipv6HosTermRuleHandle[ADK_IPV6_INTF_ADDR_MAX];   
    /*! HOS handle for Inner IP table.
     */
    adk_hos_rule_id_t ipv6HosInnerIpTblRuleHandle;  
    /*! inner IP info state maintained for tunnel if ACL mapping.
     */
    ipv6_inner_ip_tbl_info_t ipv6InnerIpTblInfo;
#endif
    /*! Indicate the DSCP to PCP mapping for this ipv6 interface.
     */
    uint8_t dscp_pcp[IPV6_NUM_DSCP_VALUES];

    /*! Indicate the DSCP Profile Id of the interface 
     */
    uint32_t profile_Id;
    /*! Indicate the PBR Ruleset 
     */
    uint32_t pbr_policy_set_id;

    /*! Enable PBR flag
     */
    uint8_t enablePBR;

    /*! Indicate the list of dscp stats id.
     */
    uint32_t dscp_stats_id[IPV6_NUM_DSCP_VALUES];
    /*! Indicate the list of dscp group attached to the interface.
     */
    ipv6_if_dscp_grp_list_t   ifDscpGroupList[ADK_IPV6_MAX_DSCP_GROUP_PER_IF];
    /*! Indicate the stored statistics of deleted dscp groups.
     */
    adk_ipv6_interface_statistics_t storeDscpStats;
    /*! Indicates the interface attributes */
     uint8_t interfaceAttr;
 } __attribute__ ((aligned(64))) ipv6_interface_info_t;

#define ADK_IPV6_IF_TUNNEL_INFO_SET(ifInfo, pointer, value) \
     {\
         (ifInfo->if_info_u.tunnelIf.pointer) = value;\
     }\


#define ADK_IPV6_IF_INFO_SET(ifInfo, pointer, value) \
     if ( ifInfo->ifType == IPV6_INTERFACE_TYPE_ETH)\
     {\
         (ifInfo->if_info_u.regularIf.pointer) = value;\
     }\

#define ADK_IPV6_IF_INFO_GET(ifInfo, pointer, retval) \
     if ( ifInfo->ifType == IPV6_INTERFACE_TYPE_ETH)\
     {\
         retval = (ifInfo->if_info_u.regularIf.pointer);\
     }\
     else if (ifInfo->ifType == IPV6_INTERFACE_TYPE_TUNNEL)\
     {\
         retval =  (ifInfo->if_info_u.tunnelIf.pointer);\
     }\
/*!
 * =============================================================
 * @internal
 * @struct ipv6_pending_deleted_rule_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Information about the resources to be cleared for the deleted filter rules.
 * =============================================================
 */
typedef struct ipv6_pending_deleted_rule_info_s {
    uint32_t num_pending_deleted_rules;
    uint32_t pendingFilterStatsId[ADK_IPV6_DEL_FILTER_RULES_PENDING_MAX];
} ipv6_pending_deleted_rule_info_t;


/*!
 * =============================================================
 * @internal
 * @struct ipv6_filter_rule_set_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * ACL Rule set information.
 * =============================================================
 */
typedef struct ipv6_filter_rule_set_info_s {
    /*! Indicate the filter cookie.
     */
    uint32_t cookie;
    /*! Indicate the filter ruleset id.
     */
    uint32_t ipv6_filer_rule_set_id;
    /*! IPv6 ACL context 
     */
    adk_ipv6_dp_acl_tree_t  *ipv6_acl_tree[2];

    /* info of deleted rules before commit*/
    ipv6_pending_deleted_rule_info_t deleteInfo;

    /* IPv6 ACL rule list information */
    adk_acl_handle_t cp_list; 

    /* Index to identify current and shadow copy */
    uint8_t acl_tree_index;
   
    /* To Track if any update to rules */
    bool anyUpdate;

} ipv6_filter_rule_set_info_t;

 /*!
  * =============================================================
  * @internal
  * @struct ipv6_pbr_rule_set_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
  *
  * @brief
  *
  * PBR Rule set information.
  * =============================================================
 */
typedef struct ipv6_pbr_rule_set_info_s {
    /*! Indicate the pbr cookie.
     */
    uint32_t cookie;
    /*! Indicate the pbr ruleset id.
     */
    uint32_t ipv6_pbr_rule_set_id;

   /* info of deleted rules before commit*/
    uint32_t num_pending_rule_ids;
    uint32_t pendingRuleIds[ADK_IPV6_DEL_PBR_RULES_PENDING_MAX];

    /*Two copies: Active & Shadow*/
    adk_ipv6_pbr_tree_t *pbr_tree[2];
    uint8_t pbr_tree_index;
    adk_acl_handle_t cp_list;
} ipv6_pbr_rule_set_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ipv6_if_dscp_grp_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Interface information for an IP interface handle.
 * =============================================================
 */
typedef struct ipv6_if_dscp_grp_info_s {
    /*! Indicate the ipv6 interface handle.
     */
    adk_interface_handle_t ipv6IfHandle;
    /*! Indicate the interface ingress statistics id.
     */
    uint32_t stats_id;
    /*! Indicate the interface egress statistics id.
     *  It can be a statistics id for tunnel interface 
     *  and Mtm shared param for other interfaces 
     */
    uint32_t EgrStatsId;
    /*! Indicate the dscp group cookie 
     * ADK_IPV6_DSCP_GROUP_COOKIE
     */
    uint32_t cookie;
    
} ipv6_if_dscp_grp_info_t;

/*!
 * =============================================================
 * @struct adk_ipv6_next_hop_info_t adk_ipv6.h "adk_ipv6.h"
 *
 * @brief The IPv6 next hop structure.
 *
 * This structure contains the next hop attributes.
 *
 * =============================================================
 */
typedef struct adk_ipv6_next_hop_info_s
{
    /*! Next Hop attributes 
     */
    adk_ipv6_next_hop_attr_t nextHopAttr;
    /*! Indicate the next hop cookie 
     * ADK_IPV6_NEXTHOP_COOKIE
     */
    uint32_t cookie;
} adk_ipv6_next_hop_info_t;   /* ------  end of struct adk_ipv6_next_hop_info_t  ------ */


/*!
 * =============================================================
 * @struct adk_ipv6_tree_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief The IPv6 tree structure.
 *
 * This structure contains the tree attributes.
 *
 * =============================================================
 */
typedef struct adk_ipv6_tree_info_s
{
    char *treeName;
    /*! Indicate whether default entry is printed 
     */
    bool defaultEntry;
} adk_ipv6_tree_info_t;   /* ------  end of struct adk_ipv6_tree_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct ipv6_if_stat_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Interface stat information for an IP interface handle.
 * =============================================================
 */
typedef struct ipv6_if_stat_info_s {
    /*! Total Octects Received at the interface */
    uint64_t  ipv6IfStatsHCInOctets;

    /*! Number of Multicast Octects at the interface */
    uint64_t  ipv6IfStatsHCInMcastOctets;

    /*! Number of PDUs Received at the interface */
    uint32_t  InterfaceRxPdus;
    /*! Number of Multicast PDUs at the interface */
    uint32_t  InterfaceMulticastRxPdus;

    /*! Number of PDUs Delivered to CPU at the interface */
    uint32_t  ipv6IfStatsInDelivers;
    /*! Number of Multicast PDUs Tx at the interface */
    uint32_t  InterfaceMulticastTxPdus;

    /*! Number of PDUs Discarded at the interface due to address errors */
    uint32_t  ipv6IfStatsInAddrErrors;
    /*! Number of PDUs sent for reassembly at the interface */
    uint32_t  ipv6IfStatsReasmReqds;

    /*! Number of PDUs Received at the interface with header error */
    uint32_t  ipv6IfStatsInHdrErrors;
    /*! Number of Unicast PDUs Tx at the interface */
    uint32_t  InterfaceUnicastTxPdus;

    /*! Number of PDUs Received at the interface with not enough length */
    uint32_t  ipv6IfStatsInTruncatedPkts;
    /*! Number of PDUs successfully reassembled Pdus */
    uint32_t  ipv6IfStatsReasmOKs;

    /*! Number of Failed reassembly Pdus */
    uint32_t  ipv6IfStatsReasmFails;
    /*! Number of PDUs Forwarded which is received at this interface */
    uint32_t  ipv6IfStatsInForwDatagrams;

    /*! Number of PDUs Forwarded at the interface */
    uint32_t  ipv6IfStatsOutForwDatagrams;
    /*! Number of PDUs with ingress no route at the interface */
    uint32_t  ipv6IfStatsInNoRoutes;

    /*! Number of PDUs with black hole route at the interface */
    uint32_t  ipv6IfStatsInBlackHoles;
    /*! Number of PDUs with out request at the interface */
    uint32_t  ipv6IfStatsOutRequests;

    /*! Number of PDUs which requires Fragmentation at the interface */
    uint32_t  ipv6IfStatsOutFragReqds;
    /*! Number of PDUs failed fragmentation at the interface */
    uint32_t  ipv6IfStatsOutFragFails;

    /*! Number of Octets Transmitted out of IPv6 Module*/
    uint64_t  ipv6IfStatsOutOctets;

    /*! Number of Multicast Octets Transmitted out of IPv6 Module*/
    uint64_t  ipv6IfStatsMcastOutOctets;

    /*! Number of PDUs Discarded at the interface due to Source ddress errors */
    uint32_t  ipv6IfStatsInSrcAddrErrors;
   
    /*! Number of PDUs Discarded at the interface due to Egress ACL filtering */
    uint32_t  ipv6IfStatsOutAclDroppedPkts;
            
    /*! Number of PDUs Discarded at the interface due to Ingress Policing */
    uint32_t  ipv6IfStatsInPoliceDropPkts;

    uint32_t   reserved[1];
} __attribute__((__packed__)) ipv6_if_stat_info_t;

/*!
 * =============================================================
 * @internal
 * @struct ipv6_if_stat_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * Interface stat information for an IP interface handle.
 * =============================================================
 */
typedef struct ipv6_if_outstat_info_s {
    /*! Number of Multicast PDUs Dropped at queue */
    uint32_t IfMtmMcDropPdus;
    /*! Number of Broadcast pdus Dropped at queue */
    uint32_t IfMtmBcDropPdus;
    /*! Number of Unicast PDUs dropped at the interface */
    uint32_t IfMtmUcDropPdus;
    /*! Number of PDUs Transmitted at the interface */
    uint32_t IfMtmTrueTxPdus;
    /*! Number of Octects Transmitted at the interface */
    uint64_t IfMtmTrueTxOctects;
    /*! Ethernet octets transmitted */
    uint64_t ipIfTrueEthOctetsTx;
} __attribute__((__packed__))ipv6_if_outstat_info_t;


/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_flow_type_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * ipv6 flow type required for bind .
 * =============================================================
 */
typedef enum adk_ipv6_flow_type_e {
    ADK_IPV6_NEXT_HOP_FLOW,
    ADK_IPV6_CPU_NEXT_HOP_FLOW,
    ADK_IPV6_INGRESS_FLOW,
    ADK_IPV6_ONLINK_FLOW,
    ADK_IPV6_FILTER_INFO,
    ADK_IPV6_PBR_INFO,
    ADK_IPV6_EGRESS_FLOW,
    /*!< Ingress classifier ordered flows */
    ADK_IPV6_FLOW_TYPE_INGRESS_CLASSIFIER_FLOW,
    ADK_IPV6_INGRESS_MGROUP_FLOW,
    ADK_IPV6_EGRESS_QOS_CLASSIFIER_FLOW,
    ADK_IPV6_FLOW_TYPE_INVALID
}adk_ipv6_flow_type_t;


/*!
 * =============================================================
 * @internal
 * @struct ipv6_flow_info_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * ipv6 flow info required for bind .
 * =============================================================
 */
typedef struct adk_ipv6_flow_info_s {
    uint32_t cookie;
    adk_ipv6_flow_type_t type;
    union {
        adk_ipv6_next_hop_info_t            next_hop_info;
        adk_ipv6_ingress_flow_info_t        ingress_flow_info;
        adk_ipv6_onlink_destination_internal_t onlink_info;
        ipv6_filter_rule_set_info_t         filter_info;
        adk_ipv6_pbr_info_t                 pbr_info;  
        adk_ipv6_egress_flow_info_t         egress_flow_info;
        ipv6_ing_classifier_flow_info_t     ingClassifierFlow;
        adk_ipv6_ingress_mgroup_info_t      ingMgroupFlow;
        adk_ipv6_qos_classifier_flow_info_t egressQoSClassifierFlow;
    };

} adk_ipv6_flow_info_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_excep_domain_type_t adk_ipv6_internal.h "adk_ipv6_internal.h"
 *
 * @brief
 *
 * ipv6 exception domain enum definations.
 * =============================================================
 */
typedef enum adk_ipv6_excep_domain_e {
    ADK_IPV6_EXCEPTION_TN_PORT_DOMAIN = 0,
    ADK_IPV6_EXCEPTION_FP_ORIGIN_DOMAIN = 1,
    ADK_IPV6_EXCEPTION_OS_ORIGIN_DOMAIN = 2,
    ADK_IPV6_EXCEPTION_DOMAIN_MAX
}adk_ipv6_excep_domain_t;





#endif /* End of _ADK_IPV6_INTERNAL_H_ */
 
/*! end addtogroup _ipv6
 * @}
 */
