/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IP_PBR_ACL_H__
#define __ADK_IP_PBR_ACL_H__

#include <adk_hash.h>

#define IPV6_PBR_MAX_RULES_PER_SET    512

#define IPV6_PBR_MAX_NUM_RULES 2048   /* It should be multiple of 64 */

typedef uint32_t adk_ipv6_pbr_val_t;

#define PBR_LKUP_MAX_NUM_PKT 8

typedef void adk_ipv6_pbr_tree_t;

typedef struct _adk_ipv6_pbr_acl_rule_s
{
    uint32_t sip_addr[4];
    uint8_t sip_mask;
} adk_ipv6_pbr_acl_rule_t;


typedef struct __attribute__((__packed__))  _adk_ipv6_pbr_ptn_s
{
    /* first input field - always one byte long for ACL lookup. */
    uint8_t first_byte;
    uint32_t sip_addr[4];
}adk_ipv6_pbr_ptn_t;

adk_ipv6_pbr_tree_t *adk_ipv6_pbr_create(adk_sys_handle_t sys_handle, 
                                         adk_module_handle_t ipv6Handle, 
                                         uint16_t ruleSetId, uint8_t index);

int adk_ipv6_pbr_ruleset_free(adk_sys_handle_t sys_handle,
                              adk_module_handle_t ipv6Handle,
                              adk_ipv6_pbr_tree_t *pbr_tree);

int adk_ipv6_pbr_rule_add(adk_sys_handle_t sys_handle,
                        adk_module_handle_t ipv6Handle,
                        adk_ipv6_pbr_tree_t *pbr_tree,
		 	adk_ipv6_pbr_acl_rule_t *pbr_rule,
		 	adk_ipv6_pbr_val_t  pbr_val,
		 	uint32_t precedence);

int adk_ipv6_pbr_ruleset_commit(adk_sys_handle_t sys_handle,
                                adk_module_handle_t ipv6Handle,
                                adk_ipv6_pbr_tree_t *pbr_tree);

void adk_ipv6_pbr_ruleset_clear(adk_sys_handle_t sys_handle,
                                adk_module_handle_t ipv6Handle,
                                adk_ipv6_pbr_tree_t *pbr_tree);

#endif

