/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_IPV6_QOS_CLASSIFIER_H__
#define __ADK_IPV6_QOS_CLASSIFIER_H__

#define ADK_IPV6_EGRESS_QOS_CLASSIFIER_RULE_MAX_PRIORITY 0xFFFF

#define ADK_IPV6_QOS_ACTIVE_TREE_INDEX_INIT 0
#define ADK_IPV6_QOS_SHADOW_TREE_INDEX_INIT 1

#define ADK_IPV6_PKT_COLOR_POS 16

#define ADK_IPV4_MAX_PREFIX_WIDTH 32
#define ADK_IPV6_MAX_PREFIX_WIDTH 128

/*!
 * =============================================================
 * @internal
 * @struct adk_ipv6_egress_qos_classifier_info_t
 *
 * @brief
 *
 * IP egress QOS Classifier Tree information.
 * =============================================================
 */
typedef void adk_ipv6_egress_qos_classifier_ctx_t;

typedef struct {
    uint32_t cookie;
    /*Two copies: Active & Shadow*/
    adk_ipv6_egress_qos_classifier_ctx_t *ipEgressQoSClassifierCtx[2];
    uint8_t aclTreeIndex;
    adk_acl_handle_t cpList;
} adk_ipv6_egress_qos_classifer_info_t;

/* IPv6 Egress QOS Classifier Related Structures */

typedef struct {
    uint8_t  proto;
    uint32_t sip0;
    uint32_t sip1;
    uint32_t sip2;
    uint32_t sip3;
    uint32_t dip0;
    uint32_t dip1;
    uint32_t dip2;
    uint32_t dip3;
    uint32_t dscp; /*TODO: break this into 8 bit and 24 bit reserved space*/
} __attribute__((__packed__)) adk_ipv6_egress_qos_classifier_ptn_t;

/* IPv6 Egress QOS classification action*/
typedef struct adk_ipv6_dp_egress_qos_classifier_action_info_s{ 
    uint16_t queueId;
    uint8_t  pktColor;
}adk_ipv6_dp_egress_qos_classifier_action_info_t;

/* Ingress Classifier Ordered tree related declaration */
adk_status_t adk_ipv6_egress_qos_classifier_create(
        adk_module_handle_t ipHandle,
        uint32_t ifId,
        adk_ipv6_egress_qos_classifer_info_t  **egressQoSClassifierInfo_p);

adk_status_t adk_ipv6_egress_qos_classifier_delete(
        adk_module_handle_t ipHandle,
        uint32_t            ifId);

adk_status_t adk_ipv6_egress_qos_classifer_add_validate_arg(
        adk_module_handle_t ipHandle,
        const adk_ipv6_qos_info_t *ipEgressQoSClassifierRule_p);

adk_status_t adk_ipv6_egress_qos_classifer_build(adk_module_handle_t ipHandle,
        adk_ipv6_egress_qos_classifier_ctx_t *ctx);

adk_dp_status_t adk_ipv6_dp_egress_qos_classifier_lookup(
        adk_ipv6_egress_qos_classifier_ctx_t  *egressQoSClassifierCtx,
        adk_ipv6_egress_qos_classifier_ptn_t  *egressQoSClassifierPtn,
        uint32_t  *result_p);
#endif
