/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_ipv6_tables.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ipv6  IPv6 Module
 *
 * @{
 *
 * @brief   This file has the functions for fast-path tables lookup used 
 * by data plane software.
 *
 *  **********************************************************************/

#ifndef _ADK_IPV6_TABLES_H_
#define _ADK_IPV6_TABLES_H_

#include "adk_ipv6_internal.h"

adk_dp_status_t 
adk_ipv6_dp_interface_entry_add(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t dscp);

adk_dp_status_t 
adk_ipv6_dp_interface_entry_status_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                bool is_disabled);

adk_dp_status_t 
adk_ipv6_dp_interface_entry_filter_config(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                bool enable);

adk_dp_status_t
adk_ipv6_dp_interface_entry_filter_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                uint32_t filterRuleSetId);

adk_dp_status_t
adk_ipv6_dp_interface_entry_filter_get(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                adk_ipv6_filter_direction_t direction,
                uint32_t *filterRuleSetId);

adk_dp_status_t 
adk_ipv6_dp_interface_entry_mtu_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t mtu);

adk_dp_status_t 
adk_ipv6_dp_interface_entry_profile_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t profileId);

adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_mtu_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t mtu, uint16_t pmtu);
adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_nexthop_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint16_t nexthop_id);

adk_dp_status_t 
adk_ipv6_dp_interface_entry_dscp_pcp_set(adk_module_handle_t ipv6Handle,
                ipv6_interface_info_t *ipv6IfInfo_p,
                uint8_t dscp,
                uint8_t pcp);


adk_dp_status_t 
adk_ipv6_dp_interface_entry_rem(adk_module_handle_t ipv6Handle, 
                uint8_t dscp,
                uint16_t if_id);

adk_dp_status_t 
adk_ipv6_dp_tunnel_interface_entry_add(adk_module_handle_t ipv6Handle, 
                uint16_t tunnel_if_id,
                uint16_t adk_if_id);

adk_dp_status_t 
adk_ipv6_tunnel_interface_entry_rem(adk_module_handle_t ipv6Handle, 
                uint16_t tunnel_if_id);

adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_dp_pkt_capture_info_t *pkt_capture_info_p);

adk_dp_status_t
adk_ipv6_dp_eg_pkt_capture_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_dp_pkt_capture_info_t *pkt_capture_info_p);

adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_status_get(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_pktcap_status_t **ipv6_pktcap_status);

adk_dp_status_t
adk_ipv6_dp_eg_pkt_capture_status_get(adk_module_handle_t ipv6Handle,
                uint16_t if_id,
                adk_ipv6_pktcap_status_t **ipv6_pktcap_status);

adk_dp_status_t
adk_ipv6_dp_ing_pkt_capture_status_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id);

adk_dp_status_t
adk_ipv6_dp_eg_pkt_capture_status_set(adk_module_handle_t ipv6Handle,
                uint16_t if_id);

void 
adk_ipv6_dp_route_key_generate(uint8_t *key, uint8_t vrd_id, uint8_t *addr);

adk_dp_status_t 
adk_ipv6_dp_route_entry_add(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6_addr_info,
                            adk_ipv6_dp_route_entry_t *route_entry,
                            bool inner_ipterm_only);

adk_dp_status_t 
adk_ipv6_dp_route_entry_update_rule_mask(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t *ipv6_addr_info,
                            uint16_t rule_mask, bool inner_ipterm_only);

adk_dp_status_t 
adk_ipv6_dp_route_entry_rem(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6_addr_info,
                            bool inner_ipterm_only);

adk_dp_status_t
adk_ipv6_dp_pbr_rule_entry_add(adk_module_handle_t ipv6Handle,
                                uint32_t pbr_rule_index,
                                adk_ipv6_pbr_info_t *pbr_rule);

adk_dp_status_t
adk_ipv6_dp_pbr_rule_entry_rem(adk_module_handle_t ipv6Handle,
                                uint32_t pbr_rule_index);

adk_dp_status_t
adk_ipv6_dp_pbr_state_set(adk_module_handle_t ipv6Handle,
                        bool pbr_flag);

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_add(adk_module_handle_t ipv6Handle,
                                adk_ipv6_dp_nexthop_entry_t *nexthop_entry,
                                uint16_t *index);

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_modify(adk_module_handle_t ipv6Handle,
                                adk_ipv6_dp_nexthop_entry_t *nexthop_entry,
                                uint16_t index);

adk_dp_status_t
adk_ipv6_dp_nexthop_entry_rem(adk_module_handle_t ipv6Handle,
                                uint16_t index);

adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_key_create(uint8_t *key, 
                        uint8_t interface_id,
                        uint32_t * dst_ipv6_addr,
                        uint32_t * src_ipv6_addr, 
			bool useSrcAddr,
			uint16_t *size);

adk_dp_status_t 
adk_ipv6_dp_ingress_association_key_create(uint8_t *key, 
                        uint8_t mask,
                        uint8_t vrd_id,
                        uint16_t interface_id,
                        uint32_t flow_label,        
                        uint8_t next_header,
                        uint32_t * src_ipv6_addr,
                        uint8_t src_prefix_length,
                        uint32_t * dst_ipv6_addr,
                        uint8_t dst_prefix_length,
                        uint16_t src_port,
                        uint16_t dst_port,
                        uint8_t icmp_type,
                        uint8_t icmp_code,
                        bool icmpCodeSpecified,
                        uint16_t *size);


adk_dp_status_t 
adk_ipv6_dp_onlink_dest_key_generate(uint8_t vrd_id,
        uint32_t *ipv6_addr,
        uint32_t *key,   
        uint16_t *key1,
        uint16_t *key2);

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_add(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_onlink_destination_info_t *onLinkDestinationInfo_p,
                            uint8_t vrd_id, 
                            adk_ipv6_dp_onlink_dest_entry_t  *onlink_dest_entry,
                            uint16_t *index,
                            uint16_t *hash_bucket_index);

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_modify(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id, 
                            adk_ipv6_dp_onlink_dest_entry_t  *onlink_dest_entry,
                            uint16_t index);

adk_dp_status_t 
adk_ipv6_dp_onlink_dest_entry_rem(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id, 
                            uint8_t *onLinkDestinationIpv6Addr, 
                            uint16_t tbl_index,
                            uint16_t hash_bucket_index);

adk_dp_status_t 
adk_ipv6_dp_egress_flow_attr_entry_add(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_dp_egress_flow_attr_entry_t *egress_flow_attr_entry,
                            uint16_t *index);

adk_dp_status_t 
adk_ipv6_dp_egress_flow_attr_entry_modify(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_dp_egress_flow_attr_entry_t *egress_flow_attr_entry,
                            uint16_t index);

adk_dp_status_t
adk_ipv6_dp_egress_flow_attr_entry_rem(adk_module_handle_t ipv6Handle,
                                uint16_t tbl_idx);

void 
adk_ipv6_dp_ingress_flow_sip_key_generate(uint8_t *key, uint8_t vrd_id, uint8_t *addr);

adk_dp_status_t 
adk_ipv6_dp_ingress_flow_sip_entry_add(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6AddrPrefix,
                            adk_ipv6_dp_ingress_flow_sip_entry_t *ingress_flow_sip_entry);

adk_dp_status_t 
adk_ipv6_dp_ingress_flow_sip_entry_rem(adk_module_handle_t ipv6Handle, 
                            uint8_t vrd_id,
                            adk_ipv6_addr_prefix_t ipv6AddrPrefix);
adk_dp_status_t
adk_ipv6_dp_excep_entry_add(adk_module_handle_t ipv6Handle,
                            adk_ipv6_exception_type_t exception,
                            adk_ipv6_dp_excep_entry_t *except_entry);

adk_dp_status_t
adk_ipv6_dp_excep_entry_rem(adk_module_handle_t ipv6Handle, 
                            adk_ipv6_exception_type_t exception);

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_key_generate(uint8_t vrd_id,
        uint32_t *ipv6_addr,
        uint32_t *key,   
        uint16_t *key1,
        uint16_t *key2);

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_add(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_entry,
                            uint16_t *index);

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_modify(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id,
                            adk_ipv6_dp_dest_mtu_entry_t *dest_mtu_entry);

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_rem(adk_module_handle_t ipv6Handle,
                            uint8_t vrd_id, 
                            uint8_t *ipv6_addr);

adk_dp_status_t 
adk_ipv6_dp_dest_mtu_entry_lookup(uint8_t vrd_id, 
                            uint8_t *ipv6_addr,
                            adk_ipv6_dp_tables_t *ipv6_dp_tables, 
                            adk_ipv6_dp_dest_mtu_entry_t  **dst_mtu_entry);

adk_status_t 
adk_ipv6_flow_cache_statistics_get( adk_module_handle_t ipv6Handle ,
        adk_ipv6_dp_flow_cache_stats_t *lli_fc_stats,
        adk_ipv6_dp_flow_cache_stats_t *uli_fc_stats);

void 
adk_ipv6_dp_filter_ruleset_delete(adk_module_handle_t ipv6Handle,
                                  uint32_t        filterRuleSetId);
void
adk_ipv6_dp_filter_ruleset_config(adk_ipv6_info_t *ipv6ModuleInfo_p,
                                  uint32_t        filterRuleSetId,
                                  adk_ipv6_dp_acl_tree_t  *ipv6_acl_tree_0,
                                  adk_ipv6_dp_acl_tree_t  *ipv6_acl_tree_1);
adk_dp_status_t 
adk_ipv6_dp_ingress_mgroup_entry_add(adk_module_handle_t ipv6Handle,
                            adk_ipv6_multicast_group_t  *mGrp_p,
                            adk_ipv6_dp_ingress_mgroup_entry_t *ingress_mgroup_entry,
                            uint16_t *index);
adk_dp_status_t
adk_ipv6_dp_ingress_mgroup_entry_rem(adk_module_handle_t ipv6Handle,
		            adk_ipv6_multicast_group_t *mGrp_p,
			    uint16_t index);

adk_dp_status_t
adk_ipv6_cp_ingress_mgroup_entry_lookup(adk_module_handle_t ipv6Handle,
				adk_ipv6_dp_ingress_mgroup_table_t *ingress_mgroup_tbl,
			        adk_ipv6_multicast_group_t *mcGrp_p,	
				adk_handle_t *mcast_index);
#endif
/*! end addtogroup _ipv6
 * @}
 */
