/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @file  adk_twamp_const_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _twamp  TWAMP Module
 *
 * @{
 *
 * @brief
 *
 * The constants in this file are common to both Control code and data path
 * code.
 *
 ***********************************************************************/
#ifndef _ADK_TWAMP_CONST_INTERNAL_H_
#define _ADK_TWAMP_CONST_INTERNAL_H_

/*
 * TWAMP resource definitions.
 */

#define TWAMP_MAX_REFLECTORS_PPF_RESPONDER    400
#define TWAMP_MAX_PPF_RESPONDER               8
#define TWAMP_TOTAL_REFLECTORS                (TWAMP_MAX_PPF_RESPONDER * TWAMP_MAX_REFLECTORS_PPF_RESPONDER)

/*!
 * IP_DSCP_TTL offsets
 */
#define TWAMP_IP_DSCP_OFFSET                     0
#define TWAMP_IP_TTL_OFFSET                      8

/*!
 * TWAMP global stats offsets
 */
#define  TWAMP_GL_RX_PKTS_OFFSET                 0x00
#define  TWAMP_GL_TX_PKTS_OFFSET                 0x04
#define  TWAMP_GL_INVALID_RSPND_PKTS_OFFSET      0x08
#define  TWAMP_GL_INVALID_LEN_PACKETS_OFFSET     0x0C
#define  TWAMP_GL_HASH_ERR_OFFSET                0x10
#define  TWAMP_GL_NO_BIND_OFFSET                 0x14
#define  TWAMP_GL_HASH_DEL_FAILED_OFFSET         0x18
#define  TWAMP_GL_REFLCTR_PARENT_SEQ_MISMATCH    0x1C
#define  TWAMP_GL_RSPND_DISABLE_OFFSET           0x20
#define  TWAMP_GL_INVALID_IP_VERSION_OFFSET      0x24
#define  TWAMP_GL_MAX_OFFSET                     0x30

/*!
 * RCU pending list threshold for every table.
 * This threshold is intentionally set to 1 because of the fixed amount of
 * resources. Unlike malloc and free, the resources are obtained through the
 * resource manager which should not be depleted if the RCU pending list is not
 * flushed out.
 * Otherwise, allowed threshold must be added to the actual dimensioned values.
 * If the RCU pending list size needs to be set to 2 then this threshold value
 * of 2 must be added to the dimensions of Responder ID.
 */

#define ADK_TWAMP_RESPONDER_TBL_THRESHOLD 1

#endif /* End of  _ADK_TWAMP_CONST_INTERNAL_H_  */

/*! end addtogroup _twamp
 * @}
 */

