/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_TWAMP_DP_EXPORT_H__
#define __ADK_TWAMP_DP_EXPORT_H__

/*!
 * =============================================================
 * @struct adk_twamp_dp_ipv4_info_t adk_twamp_dp_export.h "adk_twamp_dp_export.h"
 *
 * @brief
 *
 * Definition of adk_twamp_dp_ipv4_info_t
 *
 * =============================================================
 */
typedef struct adk_twamp_dp_ipv4_info_s {
    /*!
     * ipVrdId provides the Routing Domain ID (in host order)
     */
    uint32_t ipVrdId;
    /*!
     * DSCP
     */
    uint8_t dscp;
} adk_twamp_dp_ipv4_info_t;

/*!
 * =============================================================
 * @struct adk_twamp_dp_ipv6_info_t adk_twamp_dp_export.h "adk_twamp_dp_export.h"
 *
 * @brief
 *
 * Definition of adk_twamp_dp_ipv6_info_t
 *
 * =============================================================
 */
typedef struct adk_twamp_dp_ipv6_info_s {
    /*!
     * ipVrdId provides the Routing Domain ID (in host order)
     */
    uint32_t ipVrdId;
    /*!
     * DSCP
     */
    uint8_t dscp;
} adk_twamp_dp_ipv6_info_t;

/*!
 * =============================================================
 * @enum adk_twamp_dp_lli_imc_path_t
 *
 * @brief
 *
 * This enumeration defines different paths in TWAMP LLI.
 * =============================================================
 */
typedef enum adk_twamp_dp_lli_imc_path_s {
    /*  Entry Function for IPv4 module */
    ADK_TWAMP_DP_IPV4_LLI_PATH = 0,
    /*  Entry Function for IPv6 module */
    ADK_TWAMP_DP_IPV6_LLI_PATH = 1
} __attribute__ ((__packed__)) adk_twamp_dp_lli_imc_path_t;

/*!
 * =============================================================
 * @struct adk_twamp_dp_lli_imc_data_t adk_twamp_dp_export.h "adk_twamp_dp_export.h"
 *
 * @brief
 *
 * The data structure defines the IMC metadata that is passed along the packet
 * from source module to TWAMP module LLI for the purpose of packet processing.
 *
 * =============================================================
 */
typedef struct adk_twamp_dp_lli_imc_data_s
{
    adk_twamp_dp_lli_imc_path_t path;
    union {
        adk_twamp_dp_ipv4_info_t ipv4_encap_info_p;
        adk_twamp_dp_ipv6_info_t ipv6_encap_info_p;
    }ip_info_u;
}  __attribute__((aligned(ADK_METADATA_ALIGN))) adk_twamp_dp_lli_imc_data_t;


/*
 * This processing function can be staged using PAL APIs. This function does the
 * complete TWAMP processing.
 */
adk_dp_status_t
adk_twamp_dp_pkt_proc_lli (void **task_p,
                     uint8_t task_count, uint32_t task_mask,
	             void *ring, adk_sif_cfg_pipeline_info_t *next_stage_entry, int coreIndex);

#endif /* __ADK_TWAMP_DP_EXPORT_H__ */
