/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_config_overrides.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @brief  System level ADK Configurations Override
 *
 * This file contains overriding configurations that are used by ADK. These
 * configurations span across SIF and all ADK modules. It does not have SIF
 * or ADK module level default configurations. Those default configurations
 * will be in the SIF and individual ADK module specific directories. But
 * those default configurations can be overriden by configurations which are
 * defined in this file. If a configuration is not defined in this file, then
 * ADK level defaults will continue to be used.
 *
 * This file is shipped empty in the package for the user. It is only meant
 * to be populated by the user to have overriding configurations, if and when
 * needed. The user is free to either populate the values in this file itself
 * or completely replace this file with its own file (having same file name).
 *
 * Note that, not all ADK configurations can be overriden. The ones which can
 * can be overriden are identified in this file within the comments. Since
 * this file is included in both control plane and data plane software, it
 * must be present in this directory for build to succeed.
 *
 * User can also have different overriding configurations based upon the
 * platform. For this, it can use the ADK #defines available for differnet
 * platforms e.g. ADK_XEON2, ADK_XEON, etc.
 *
 *  **********************************************************************/

#ifndef __ADK_CONFIG_OVERRIDES_H__
#define __ADK_CONFIG_OVERRIDES_H__

/*!
 *  ADK_IP_REASSEMBLY_TIMEOUT
 *
 *  Used by module: IPv4
 *
 *  Default value defined in: adk_ip_config.h
 *
 *  This controls the re-assembly timeout value used by IPv4 module when
 *  re-assembling fragments of a packet. It should be a 32-bit number providing
 *  timer duration in micro seconds. Note that internally this will get
 *  converted to milli seconds since the ADK timer framework granularity is
 *  milli second.
 *
 */



 /*
    Restricting UDP ports allocation within a range of 41000-61000 for UDP session (WCDMA-specific)
    This is required only on BB Platform
 */
#define ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MIN    41000
#define ADK_IP_COMP_ING_ASSOC_FLOW_UDP_PORT_MAX     61000

 /*
    Configuration file to be used if not specified in adk_sys_init(sysConf_p->userArgs_p)
 */
#define ADK_CONFIG_DEFAULT    "$ADK_CODE/config/adk_haswell_rtc.cfg"

/*ADK MetaData offsets*/
#define ADK_META_DATA_OFFSET    32

/* Ethernet Transmit BridgeDriver MetaData offset*/
#define ADK_ETH_BD_TX_METADATA_OFFSET    0

#endif /* __ADK_CONFIG_OVERRIDES_H__ */
