/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_nd_common.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL layer
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the internal definitions shared by ADK with
 * netdriver.
 *
 ***********************************************************************/

#ifndef _ADK_ND_COMMON_H_
#define _ADK_ND_COMMON_H_

#if 0
#ifdef __KERNEL__
#include <linux/if.h>
#define RTE_STD_C11
#else
#include <rte_common.h>
#endif
#endif

#define STRINGIFY(x) #x
#define TO_STRING(y) STRINGIFY(y)

#define ADK_NETD_FILE           "adk_nd"

/* ADK MD Version */
#define ADK_NETD_FP_INTF_VERSION_MAJOR   2
#define ADK_NETD_FP_INTF_VERSION_MINOR   0
#define ADK_NETD_FP_INTF_VERSION "FP_INTF-"TO_STRING(ADK_NETD_FP_INTF_VERSION_MAJOR)"."TO_STRING(ADK_NETD_FP_INTF_VERSION_MINOR)

#define MAX_ADK_NETD_DEVICES    20
#define ADK_NETD_NAMESIZE       32
#define ADK_NETD_MAX_IF_NAME_SZ 32

#define ADK_NETD_IOCTL_TEST    _IOWR(0, 1, int)
#define ADK_NETD_IOCTL_CREATE  _IOWR(0, 2, struct adk_netd_device_info)
#define ADK_NETD_IOCTL_CLEANUP _IOWR(0, 3, struct adk_netd_device_info)
#define ADK_NETD_IOCTL_GETINFO _IOWR(0, 4, struct adk_netd_device_info)
#define ADK_NETD_IOCTL_START   _IOWR(0, 5, int)

#define ADK_NETD_NUM_FIFO       9

enum adk_netd_dm_type {
	ADK_NETD_DM_SDM,
	ADK_NETD_DM_HQM
};

enum adk_netd_fifo_type {
	ADK_NETD_FIFO_TX,
	ADK_NETD_FIFO_RX,
	ADK_NETD_FIFO_FQ,
	ADK_NETD_FIFO_AQ
};

struct  adk_netd_fifo_t {
	enum adk_netd_fifo_type fifo_type;
	int fifo_queue_id;
	phys_addr_t fifo_addr;
	int burst_size;
	int fifo_size;
	int bm_queue_id;
	int priority;
};

struct  adk_netd_device_info {
	uint16_t    intf_ver_mj;
	uint16_t    intf_ver_mi;
	/* interface name */
	char name[ADK_NETD_MAX_IF_NAME_SZ];
	enum adk_netd_dm_type dm_type;
	uint16_t num_tx_queues;
	uint16_t num_rx_queues;
	/* mbuf size */
	uint16_t mbuf_size;

	/* NetDriver MD offset */
	uint16_t md_offset;

	/* Rx Thread polling status
	 * used in getting info
	 */
	uint8_t poll_status;
	/* core id for kthread */
	uint32_t core_id;

	__extension__
	/* kthread flag to force bind */
	uint8_t force_bind : 1;

	union {
		/* SDM/FIFO info*/
		struct {
			struct adk_netd_fifo_t tx_fifo[ADK_NETD_NUM_FIFO];
			struct adk_netd_fifo_t rx_fifo[ADK_NETD_NUM_FIFO];
			struct adk_netd_fifo_t fq_fifo;
			struct adk_netd_fifo_t aq_fifo;
		};

		struct {
			/* HQM/DM related info */
			uint32_t test;
		};
	};
};


#define ADK_NETD_MD_COOKIE	0xa5c3

typedef uint8_t u8;
typedef uint64_t u64;

#define ADK_NETD_MAX_PORT_GROUPS 5
#define ADK_NETD_MAX_TS_INDEX 64

enum adk_netd_ts_message_type {
	ADK_NETD_TS_NONE,
	ADK_NETD_TS_CAPTURE,
	ADK_NETD_TS_CAPTURE_IDX_ALLOC,
	ADK_NETD_TS_IDX_RELEASE,
	ADK_NETD_TS_REPLY,
	ADK_NETD_TS_TX,
	ADK_NETD_TS_IDX_PROVISION,
};

enum adk_netd_ts_status_code {
	ADK_NETD_TS_INDEX_ALLOC_SUCCESS = 1,
	ADK_NETD_TS_INDEX_ALLOC_FAIL,
	ADK_NETD_TS_TX_SUCCESS,
	ADK_NETD_TS_TX_FAIL,
	ADK_NETD_TS_TX_QUEUE_FAIL,
	ADK_NETD_TS_TIMEOUT,
};

struct adk_netd_ts_index_t {
    /* stores next free index*/
	u8 next_free_index;
	/* Allocation count*/
	u8 alloc_count;
	/* Max number of TS index per port group*/
	u8 max_ts_index;
	/* start index value*/
	u8 start_index;
	/* bitmask containing index allocation status*/
	u64 bit_mask;
	/* Lock for protection*/
	rte_spinlock_t lock;
};

/*
 * ADK Netdriver metadata
 */
struct adk_netd_pkt_md {
	uint16_t cookie;        /**< Netdriver cookie. */
	uint16_t intf_ver_mj;   /**< ADK FP interface version major */
	uint16_t intf_ver_mi;   /**< ADK FP interface version minor */
	uint8_t  nb_segs;       /**< Number of segments. */
	uint32_t pkt_len;       /**< Total pkt len: sum of all segment data_len. */
	uint16_t data_len;      /**< Amount of data in segment buffer. */
	uint16_t interface_id;  /**< physical/HLP port */
	uint16_t linux_intf_id; /**< linux interface id */
	void     *next;         /**< Next segment of the packet  */

	uint32_t correlator;    /**< correlator used in TS mgmt */
	uint8_t  msg_type;      /**< Messgae type. */
	uint8_t  ts_index;      /**< TS index. */
	uint64_t timestamp;     /**< Timestamp value */
};
#endif /* _ADK_ND_COMMON_H_ */
/*!  end addtogroup _pal  PAL layer
 * @}
 */
