/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_snr_profile_1.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL layer
 *
 * @{
 *
 * @brief  
 *  m-snr product specific pipeline definitions 
 *
 ***********************************************************************/
#ifndef _ADK_PAL_SNR_PPL_PROFILE_H_
#define _ADK_PAL_SNR_PPL_PROFILE_H_


/* For SNR product ADK pipelines are designed with below number of event queues.
 * It is required that application has to allocate these many number of 
 * event-queues for ADK.
 */
#define ADK_PAL_DEFAULT_NUM_EVQS   4

adk_pal_proc_group_info_t adk_proc_group_list[] = {
    /*  
     *  assumption is that all are Atomic Queues
     *
     *  Group-Id (KEY)       event-queue-id         pkt_pri/fn_pri  
     *  
     */
    {   ADK_PAL_PG_NW_NIC_ARRIVAL,  ADK_PAL_W_EVQ_INDX_0,  ADK_PAL_USE_PKT_PRI},
    {   ADK_PAL_PG_SP_ARRIVAL,      ADK_PAL_W_EVQ_INDX_0,  ADK_PAL_USE_PKT_PRI},
    {   ADK_PAL_PG_MULTI_STAGE_HI,  ADK_PAL_W_EVQ_INDX_1,  ADK_PAL_USE_STAGE_PRI},
    {   ADK_PAL_PG_MULTI_STAGE_MED, ADK_PAL_W_EVQ_INDX_2,  ADK_PAL_USE_STAGE_PRI},
    {   ADK_PAL_PG_MULTI_STAGE_LOW, ADK_PAL_W_EVQ_INDX_3,  ADK_PAL_USE_STAGE_PRI}};

    /*
     * List of staged processing funcitons for SNR TARGET
     */
adk_pal_proc_fn_id_e adk_stage_fn_list[] = { 
                ADK_ETH_LLI_FN_ID, 
                ADK_ETH_ULI_NETD_FN_ID, 
                ADK_ETH_ULI_NETD_BRIDGE_FN_ID,
                ADK_ETH_ULI_PKT_BUFFER_FN_ID,
#if 0
                ADK_IPV4_REASSEMBLE_FN_ID,
                ADK_IPV4_REASM_TOUT_FN_ID,
#endif
                ADK_IPV4_POLICING_FN_ID,
                ADK_IPV6_POLICING_FN_ID,
                ADK_IPSEC_LLI_FROM_CRYPTO_FN_ID,
                ADK_IPSEC_ULI_FROM_CRYPTO_FN_ID,
                ADK_PH_DEV_RATE_LIMIT_FN_ID,
                ADK_SIF_EM_LLI_FN_ID
                };

#endif /* End of _ADK_PAL_SNR_PPL_PROFILE_H_*/

/*! end addtogroup _pal
 * @}
 */
