/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __NCP_SAL_INTERNAL_H__
#define __NCP_SAL_INTERNAL_H__

#include <stdlib.h>

#include "ncp_internal.h"

#ifdef __cplusplus
extern "C" { 
#endif

#ifndef TRUE
#define TRUE    1
#endif

#ifndef FALSE
#define FALSE   0
#endif


typedef struct ncp_sem_st {
    pthread_mutex_t     mutex;
    pthread_mutexattr_t mutexAttr;
} ncp_sem_t;

ncp_st_t
ncp_sem_init(ncp_sem_t *sem, int32_t shared, uint32_t value);

ncp_st_t
ncp_sem_post(ncp_sem_t *sem);

ncp_st_t
ncp_sem_wait(ncp_sem_t *sem);

ncp_st_t
ncp_sem_destroy(ncp_sem_t *sem);

/*!
 * NVM Access interface 
 */
#define NCP_NVM_MAX_USER_POINTERS 11

#ifdef NCP_USE_NVM

#define NCP_NVM_COOKIE       (0x4E564D41)

#ifdef NCP_DEBUG
#define NCP_NVM_INIT_ERR(s) \
    do { \
        printf("%s, line = %d\n", __FILE__,__LINE__); \
        (s); \
    } while (0);
#else
#define NCP_NVM_INIT_ERR(s)
#endif

#define __ncp_malloc(size)              malloc(size)
#define __ncp_free(ptr)                 free(ptr)
#define __ncp_realloc(ptr, size)        realloc(ptr, size)
#define __ncp_calloc(no, size)          calloc(no, size)

#define __ncp_nvm_malloc(size)              rte_malloc(NULL, size, 0)
#define __ncp_nvm_free(ptr)                 rte_free((void *)ptr)
#define __ncp_nvm_realloc(ptr, size)        rte_realloc((void *)ptr, size, 0)
#define __ncp_nvm_calloc(no, size)          rte_calloc(NULL, no, size, 0)

#endif /* #ifdef NCP_USE_NVM */

typedef struct ncp_nvm_addr_s {
    char            *base;
    uint32_t     cookie;
    void            *rootPtr;
    void            *userPtr[NCP_NVM_MAX_USER_POINTERS];
    ncp_sem_t       nvmLock;
    bool      nvmActive;
} ncp_nvm_addr_t;


/* NVM stats */
typedef struct ncp_nvm_stats_s {
    uint32_t     bytes_total;
    uint32_t     chunks_used;
    uint32_t     bytes_used;
    uint32_t     chunks_free;
    uint32_t     bytes_free;
    uint32_t     nxt_avail_addr;
} ncp_nvm_stats_t;

/* NVM Access Interface Function Prototypes */
int   ncp_nvm_init(bool coldStart);
char * ncp_nvm_mmap(void);
int   ncp_nvm_munmap(void);
void  ncp_nvm_stats_get(ncp_nvm_stats_t *nvmStats);
void *ncp_nvm_ptr_get(void);
void  ncp_nvm_ptr_set(void * rootPtr);
void *ncp_nvm_user_ptr_get(int idx);
void  ncp_nvm_user_ptr_set(int idx, void * rootPtr);
bool ncp_nvm_is_enabled(void);
bool ncp_nvm_is_active(void);



void *ncp_malloc(size_t size);
void *ncp_realloc(void *ptr, size_t size);
void *ncp_calloc(uint32_t no, size_t size);
void  ncp_free(void *ptr);

/*
 * the nvm_*alloc functions always exist, but are defined 
 * differently depending on whether NCP_USE_NVM is defined.
 */
void *ncp_nvm_malloc(size_t size);
void *ncp_nvm_realloc(void *ptr, size_t size);
void *ncp_nvm_calloc(uint32_t no, size_t size);
void  ncp_nvm_free(void *ptr);


#ifdef __cplusplus
}
#endif

#endif /* __NCP_SAL_INTERNAL_H__ */
