/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __NCP_TASK_H__
#define __NCP_TASK_H__

#include  "ncp.h"

#define NCP_TASK_PARAM_SIZE_BYTES         32

typedef void *ncp_task_hdl_t;
typedef void *ncp_task_raw_recv_hdl_t;

typedef struct ncp_task_recv_buf_s 
{
            uint32_t            dataOffset;             /*!< Word 0: virtual address of received task data  */
            unsigned                launchId        :18;    /*!< Word 1: The ID of the lauching engine          */
            unsigned                reserved        :14;    /*!< reserved                                       */
            unsigned                dataSz          :16;    /*!< Word 2: The number of bytes of data in the received task */ 
            unsigned                TOID            :12;    /*!< The task order ID associated with this task    */ 
            unsigned                taskPrio        :3;     /*!< The priority of this task                      */                   
            unsigned                traceEn         :1;     /*!< debug bit to enable task tracing               */
            unsigned                destId_9_0      :10;    /*!< Word 3:The task destination instance ID, bits 0..9      */
            unsigned                debugFieldV     :2;     /*!< 00-None, 01-Debug For trace, 10-Debug For Performance, 11-reserved */
            unsigned                bufferSize      :2;     /*!< The logical buffer size indicator 00..11 for sizes 0..3 */
            unsigned                bufferSizeErr   :1;     /*!< Task Buffer Error Status - failure due to buffer error */
            unsigned                taskErr         :1;     /*!< Task Error Status - general                    */
            unsigned                templateId      :8;     /*!< The template ID associated with this task      */
            unsigned                QID             :8;     /*!< The ID of the NCA Queue this task was sent to  */                                                 
            uint8_t             parms[NCP_TASK_PARAM_SIZE_BYTES]; /*< Word 4..11: Task parameters */
} ncp_task_recv_buf_t;

typedef struct ncp_task_send_meta_s
{
    unsigned    flowId              :24;    /*!< Word 0: The FLOWID associated with this task transmission */
    unsigned    reserved0           :8;     /*!< reserved           */
    unsigned    debugData           :24;    /*!< 3 bytes of debug data, valid if debugFieldV below is 01 or 10 */
    unsigned    debugFieldV         :1;     /*!< 00-No debug, 01-Debug field for tracing, 10-debug field for performance, 11-reserved */ 
    unsigned    reserved3           :2;     /*!< reserved           */
    unsigned    priority            :3;     /*!< The priority associated with this task */
    unsigned    reserved2           :1;     /*!< reserved           */          
    unsigned    reserved1           :1;     /*!< reserved           */
    unsigned    reserved7           :6;     /*!< Word 2: reserved   */
    unsigned    reserved6           :5;     /*!< reserved           */
    unsigned    reserved5           :1;     /*!< reserved           */
    unsigned    reserved4           :4;     /*!< reserved           */
    unsigned    dataSize            :16;    /*!< size in bytes of task payload */
    unsigned    reserved8           :32;    /*!< Word 3: reserved           */       
    uint8_t taskParms[NCP_TASK_PARAM_SIZE_BYTES]; /*!< Word 4...11: The task parameters associated with this task */
    uint32_t reserved9_12       [4];    /*!< Word 12-15: reserved */
} ncp_task_send_meta_t;

typedef struct ncp_task_stats_s
{
    /* Task Stats - SUCCESS */
    uint32_t stat_RxTaskDispatched;             /*!< The total number of tasks successfully received using dispatch receive mode */
    uint32_t stat_RxTaskDirect;                 /*!< The total number of tasks successfully received using direct receive mode */ 
    uint32_t stat_TxTask;                       /*!< The total number of tasks successfully transmitted */
    uint32_t stat_TxTaskSegments;               /*!< The total number of tasks successfully transmitted using gather-mode DMA */
    uint32_t stat_TxTaskComp;                   /*!< The total number of task completions   successfully transmitted */    
    /* Task Stats - FAILURE */    
    uint32_t stat_RxDroppedNoBind;              /*!< The total number of tasks received and discarded due to no active VP binding */
    uint32_t stat_RxDroppedTaskErr;             /*!< The total number of tasks received and discarded due to a generic task error */
    uint32_t stat_RxDroppedBufferSizeErr;       /*!< The total number of tasks received and discarded due to a buffer size error */
    
    /* Queue State Statistics */
    uint32_t stat_RxQueueEmpty;                 /*!< Number of times Rx Queue was empty - wait for irq */
    uint32_t stat_RxQueueNotEmpty;              /*!< Number of times Rx Queue was not empty */
    uint32_t stat_TxQueueFull;                  /*!< Number of times Tx Queue was full - wait for irq */
    uint32_t stat_TxQueueNotFull;               /*!< Number of times Tx Queue was not full */
    uint32_t stat_CompQueueFull;                /*!< Number of times Comp Queue was full - wait for irq */
    uint32_t stat_CompQueueNotFull;             /*!< Number of times Comp Queue was not full */    
    
    /* API Stats - SUCCESS  */
    uint32_t stat_api_task_hdl_create_ok;       /*!< The total number of successful API invocations */
    uint32_t stat_api_task_hdl_remove_ok;       /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_queue_bind_ok;       /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_queue_unbind_ok;     /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_queue_vp_bind_ok;    /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_queue_vp_unbind_ok;  /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_ok;                  /*!< The total number of successful API invocations */
    uint32_t stat_api_recv_done_ok;             /*!< The total number of successful API invocations */
    uint32_t stat_api_send_ok;                  /*!< The total number of successful API invocations */
    uint32_t stat_api_send_segments_ok;         /*!< The total number of successful API invocations */
    uint32_t stat_api_task_buffer_alloc_ok;     /*!< The total number of successful API invocations */
    uint32_t stat_api_task_buffer_free_ok;      /*!< The total number of successful API invocations */
    uint32_t stat_api_send_mutex_disable_ok;    /*!< The total number of successful API invocations */
    /* API Stats - FAILURE  */    
    uint32_t stat_api_task_hdl_create_err;      /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_task_hdl_remove_err;      /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_queue_bind_err;      /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_queue_unbind_err;    /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_queue_vp_bind_err;   /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_queue_vp_unbind_err; /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_err;                 /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_recv_done_err;            /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_send_err;                 /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_send_segments_err;        /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_task_buffer_alloc_err;    /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_task_buffer_free_err;     /*!< The total number of API invocations that failed due to error(s) */
    uint32_t stat_api_send_mutex_disable_err;   /*!< The total number of API invocations that failed due to error(s) */
    /* User Mode Buffer Pool Statistics */
    uint32_t stat_upool_sz0_buffers_total;      /*!< The total number of size0 buffers in user mode pool */
    uint32_t stat_upool_sz1_buffers_total;      /*!< The total number of size1 buffers in user mode pool */
    uint32_t stat_upool_sz2_buffers_total;      /*!< The total number of size2 buffers in user mode pool */
    uint32_t stat_upool_sz3_buffers_total;      /*!< The total number of size3 buffers in user mode pool */    
    uint32_t stat_upool_sz0_buffers_free;       /*!< The current number of free size0 buffers in user mode pool */
    uint32_t stat_upool_sz1_buffers_free;       /*!< The current number of free size1 buffers in user mode pool */
    uint32_t stat_upool_sz2_buffers_free;       /*!< The current number of free size2 buffers in user mode pool */
    uint32_t stat_upool_sz3_buffers_free;       /*!< The current number of free size3 buffers in user mode pool */ 
    uint32_t stat_upool_sz0_buffers_low_water;  /*!< The lowest number of free size0 buffers in user mode pool */
    uint32_t stat_upool_sz1_buffers_low_water;  /*!< The lowest number of free size1 buffers in user mode pool */
    uint32_t stat_upool_sz2_buffers_low_water;  /*!< The lowest number of free size2 buffers in user mode pool */
    uint32_t stat_upool_sz3_buffers_low_water;  /*!< The lowest number of free size3 buffers in user mode pool */     
    /* Kernel Mode Buffer Pool Statistics */
    uint32_t stat_kpool_sz0_buffers_total;      /*!< The total number of size0 buffers in kernel mode pool */
    uint32_t stat_kpool_sz1_buffers_total;      /*!< The total number of size1 buffers in kernel mode pool */
    uint32_t stat_kpool_sz2_buffers_total;      /*!< The total number of size2 buffers in kernel mode pool */
    uint32_t stat_kpool_sz3_buffers_total;      /*!< The total number of size3 buffers in kernel mode pool */    
    uint32_t stat_kpool_sz0_buffers_free;       /*!< The current number of free size0 buffers in kernel mode pool */
    uint32_t stat_kpool_sz1_buffers_free;       /*!< The current number of free size1 buffers in kernel mode pool */
    uint32_t stat_kpool_sz2_buffers_free;       /*!< The current number of free size2 buffers in kernel mode pool */
    uint32_t stat_kpool_sz3_buffers_free;       /*!< The current number of free size3 buffers in kernel mode pool */ 
    uint32_t stat_kpool_sz0_buffers_low_water;  /*!< The lowest number of free size0 buffers in kernel mode pool */
    uint32_t stat_kpool_sz1_buffers_low_water;  /*!< The lowest number of free size1 buffers in kernel mode pool */
    uint32_t stat_kpool_sz2_buffers_low_water;  /*!< The lowest number of free size2 buffers in kernel mode pool */
    uint32_t stat_kpool_sz3_buffers_low_water;  /*!< The lowest number of free size3 buffers in kernel mode pool */
#if 0
} NCP_COMPAT_PACKED ncp_task_stats_t;
#else
} ncp_task_stats_t;
#endif

typedef ncp_st_t (*ncp_task_recv_fn_t)(
	uint8_t     recvQueueId,       /*!< The receive queue ID associated with this task,  as returned in the ncp_task_recv_queue_bind API */
	ncp_vp_hdl_t    vpHdl,             /*!< The VP (virtual pipeline) handle associated with the received task */
	uint8_t     engineSequenceId,  /* The engine sequeunce ID associated with the received task */
	void            *recvFuncArg,      /* The argument provided by the application to the ncp_task_recv_queue_vp_bind() API for this VP bind */
	void            *task,             /* Virtual address of task is returned here. 
	                                    * Note: legacy ncp_task_recv API returns ncaV1 header,  otherwise ncaV2 
	                                    */
	void                *taskData);    /* Virtual address of task Data is returned here        */
typedef void (*ncp_task_send_done_fn_t)(
    void *sendDoneFuncArg,
	void *txBuffer);

ncp_st_t
ncp_task_recv(
	ncp_task_hdl_t      taskHdl,
	uint8_t         *pRecvQueueId,
	ncp_vp_hdl_t        *pVpHdl,
	uint8_t         *pEngineSequenceId,
	ncp_task_recv_buf_t **task,
    void                **taskData,	
	void                **recvHandlerArg,
	bool          wait);
ncp_st_t
ncp_task_recv_queue_name_to_id(
    ncp_hdl_t ncpHdl,
    char *recvQueueName, 
    uint8_t *p_ncaQueueID);

ncp_st_t
ncp_task_recv_queue_get_tqs_id(
    ncp_hdl_t       ncpHdl, 
    uint8_t     ncaQueueID, 
    uint8_t     *p_tqsID);

ncp_st_t
ncp_vp_engine_seq_id_get(
    ncp_vp_hdl_t    vpHdl,
    const char      *engineSeqName,
    uint8_t     *engineSeqId);

#if 0
ncp_st_t
ncp_task_recv_queue_bind(
	ncp_task_hdl_t  taskHdl,
	uint8_t     ncaQueueId,
	uint8_t     weight,
	bool      shared,
	bool      fixedRecvQueueId,
	uint8_t     *recvQueueId);
#endif

ncp_st_t
ncp_task_recv_queue_vp_bind(
	ncp_task_hdl_t      taskHdl,
	uint8_t         recvQueueId,
	ncp_vp_hdl_t        vpHdl,
	uint8_t         engineSequenceId,
	bool          shared,
	ncp_task_recv_fn_t  recvFunc,
	void                *recvFuncArg);	 

ncp_st_t
ncp_task_recv_queue_vp_unbind(
	ncp_task_hdl_t  taskHdl,
	uint8_t     recvQueueId,
	ncp_vp_hdl_t    vpHdl,
	uint8_t     engineSequenceId);

ncp_st_t
ncp_task_recv_queue_bind(
	ncp_task_hdl_t  taskHdl,
	uint8_t     ncaQueueId,
	uint8_t     weight,
	bool      shared,
	bool      fixedRecvQueueId,
	uint8_t     *recvQueueId);

ncp_st_t
ncp_task_recv_queue_unbind(
	ncp_task_hdl_t  taskHdl,
	uint8_t     recvQueueId);

ncp_st_t
ncp_task_buffer_alloc (
	uint32_t    size,
	void            **taskAddr);

ncp_st_t
ncp_task_send (
	ncp_task_hdl_t          taskHdl,
	ncp_vp_hdl_t            vpHdl,
	ncp_task_send_meta_t    *taskMetaData,
	void                    *taskBuffer,
	void                    *taskPduData,
	bool              wait,
	ncp_task_send_done_fn_t sendDoneFn,
    void                    *sendDoneFuncArg);

ncp_st_t 
ncp_task_hdl_create(
    ncp_hdl_t       ncpHdl,
    uint8_t     threadQueueSetId,
    bool      shared,
    bool      orderedTaskCompletion,   
    ncp_task_hdl_t  *taskHdl);

ncp_st_t 
ncp_task_hdl_remove(
    ncp_task_hdl_t  taskHdl);

ncp_st_t
ncp_task_buffer_free (void *taskAddr);

ncp_st_t
ncp_task_v2_get_taskHdl(ncp_task_hdl_t *pTaskHdl);

#endif /* _NCP_TASK_H_ */

