/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "ncp_internal.h"


static const char *ncp_st_err_msg = "ERROR: Status Code Out of Bounds";



/* Internal function to locate the status code */

static const char*
find_status_code(ncp_st_t st)
{
    uint32_t sti = (uint32_t) st;
    if ( sti > NCP_ST_LAST_STATUS_CODE ) { 
        return NULL;
    }

    switch ( st )
    {

        case NCP_ST_SUCCESS:
            return "NCP_ST_SUCCESS";
        case NCP_ST_ERROR:
            return "NCP_ST_ERROR";
        case NCP_ST_NO_MEMORY:
            return "NCP_ST_NO_MEMORY";
        case NCP_ST_INTERNAL_ERROR:
            return "NCP_ST_INTERNAL_ERROR";
        case NCP_ST_INVALID_VALUE:
            return "NCP_ST_INVALID_VALUE";
        case NCP_ST_INVALID_HANDLE:
            return "NCP_ST_INVALID_HANDLE";
        case NCP_ST_HANDLE_IN_USE:
            return "NCP_ST_HANDLE_IN_USE";
        case NCP_ST_INVALID_PARAMETER:
            return "NCP_ST_INVALID_PARAMETER";
        case NCP_ST_NOT_IMPLEMENTED:
            return "NCP_ST_NOT_IMPLEMENTED";
        case NCP_ST_NVM_ERROR:
            return "NCP_ST_NVM_ERROR";
        case NCP_ST_EOF:
            return "NCP_ST_EOF";
        case NCP_ST_PARSE_ERROR:
            return "NCP_ST_PARSE_ERROR";
        case NCP_ST_API_NOT_SUPPORTED:
            return "NCP_ST_API_NOT_SUPPORTED";
        case NCP_ST_POLL_TIMEOUT:
            return "NCP_ST_POLL_TIMEOUT";
        case NCP_ST_REF_COUNT_MISMATCH:
            return "NCP_ST_REF_COUNT_MISMATCH";
        case NCP_ST_MAP_ERROR:
            return "NCP_ST_MAP_ERROR";
        case NCP_ST_MAP_INTERNAL_ERROR:
            return "NCP_ST_MAP_INTERNAL_ERROR";
        case NCP_ST_MAP_INVALID_MAP:
            return "NCP_ST_MAP_INVALID_MAP";
        case NCP_ST_MAP_KEY_NOT_EXIST:
            return "NCP_ST_MAP_KEY_NOT_EXIST";
        case NCP_ST_MAP_INVALID_TYPE:
            return "NCP_ST_MAP_INVALID_TYPE";
        case NCP_ST_MAP_INVALID_INDEX:
            return "NCP_ST_MAP_INVALID_INDEX";
        case NCP_ST_MAP_INVALID_SEARCH:
            return "NCP_ST_MAP_INVALID_SEARCH";
        case NCP_ST_MAP_CALLBACK_ERROR:
            return "NCP_ST_MAP_CALLBACK_ERROR";
        case NCP_ST_READFN_NULL:
            return "NCP_ST_READFN_NULL";
        case NCP_ST_INVALID_CFG_FILE:
            return "NCP_ST_INVALID_CFG_FILE";
        case NCP_ST_INVALID_INSTANCE:
            return "NCP_ST_INVALID_INSTANCE";
        case NCP_ST_INCOMPATIBLE_TARGET_DEVICE_REVISION:
            return "NCP_ST_INCOMPATIBLE_TARGET_DEVICE_REVISION";
        case NCP_ST_INVALID_DOMAIN_ID:
            return "NCP_ST_INVALID_DOMAIN_ID";
        case NCP_ST_SERVICE_UNAVAILABLE_ON_THIS_DOMAIN:
            return "NCP_ST_SERVICE_UNAVAILABLE_ON_THIS_DOMAIN";
        case NCP_ST_CONFIG_ABORTED:
            return "NCP_ST_CONFIG_ABORTED";
        case NCP_ST_NOT_CONFIGURED:
            return "NCP_ST_NOT_CONFIGURED";
        case NCP_ST_DOMAIN_NOT_CONFIGURED:
            return "NCP_ST_DOMAIN_NOT_CONFIGURED";
        case NCP_ST_PRIMARY_DOMAIN_NOT_CONFIGURED:
            return "NCP_ST_PRIMARY_DOMAIN_NOT_CONFIGURED";
        case NCP_ST_DOMAIN_NVM_BAD_ADDR:
            return "NCP_ST_DOMAIN_NVM_BAD_ADDR";
        case NCP_ST_DOMAIN_NVM_BAD_SIZE:
            return "NCP_ST_DOMAIN_NVM_BAD_SIZE";
        case NCP_ST_DOMAIN_NVM_ADDR_OVERLAPS_OS:
            return "NCP_ST_DOMAIN_NVM_ADDR_OVERLAPS_OS";
        case NCP_ST_DOMAIN_NVM_ADDR_MISMATCH:
            return "NCP_ST_DOMAIN_NVM_ADDR_MISMATCH";
        case NCP_ST_DOMAIN_NVM_SIZE_MISMATCH:
            return "NCP_ST_DOMAIN_NVM_SIZE_MISMATCH";
        case NCP_ST_DOMAIN_DMA_BAD_ADDR:
            return "NCP_ST_DOMAIN_DMA_BAD_ADDR";
        case NCP_ST_DOMAIN_DMA_BAD_SIZE:
            return "NCP_ST_DOMAIN_DMA_BAD_SIZE";
        case NCP_ST_DOMAIN_DMA_ADDR_OVERLAPS_OS:
            return "NCP_ST_DOMAIN_DMA_ADDR_OVERLAPS_OS";
        case NCP_ST_DOMAIN_DMA_ADDR_MISMATCH:
            return "NCP_ST_DOMAIN_DMA_ADDR_MISMATCH";
        case NCP_ST_DOMAIN_DMA_SIZE_MISMATCH:
            return "NCP_ST_DOMAIN_DMA_SIZE_MISMATCH";
        case NCP_ST_DOMAIN_RSVD_MEM_SIZE_MISMATCH:
            return "NCP_ST_DOMAIN_RSVD_MEM_SIZE_MISMATCH";
        case NCP_ST_HW_NOT_CONFIGURED:
            return "NCP_ST_HW_NOT_CONFIGURED";
        case NCP_ST_ALREADY_CONFIGURED:
            return "NCP_ST_ALREADY_CONFIGURED";
        case NCP_ST_UNKNOWN_RELEASE_ID:
            return "NCP_ST_UNKNOWN_RELEASE_ID";
        case NCP_ST_STATE_VERSION_MISMATCH:
            return "NCP_ST_STATE_VERSION_MISMATCH";
        case NCP_ST_STATE_INCONSISTENT:
            return "NCP_ST_STATE_INCONSISTENT";
        case NCP_ST_MUTEX_ATTR_INIT_ERROR:
            return "NCP_ST_MUTEX_ATTR_INIT_ERROR";
        case NCP_ST_MUTEX_SHARED_INIT_ERROR:
            return "NCP_ST_MUTEX_SHARED_INIT_ERROR";
        case NCP_ST_MUTEX_INIT_ERROR:
            return "NCP_ST_MUTEX_INIT_ERROR";
        case NCP_ST_MUTEX_WAIT_ERROR:
            return "NCP_ST_MUTEX_WAIT_ERROR";
        case NCP_ST_MUTEX_BUSY:
            return "NCP_ST_MUTEX_BUSY";
        case NCP_ST_MUTEX_TRYWAIT_ERROR:
            return "NCP_ST_MUTEX_TRYWAIT_ERROR";
        case NCP_ST_MUTEX_POST_ERROR:
            return "NCP_ST_MUTEX_POST_ERROR";
        case NCP_ST_MUTEX_DESTROY_ERROR:
            return "NCP_ST_MUTEX_DESTROY_ERROR";
        case NCP_ST_PTHREAD_ONCE_ERROR:
            return "NCP_ST_PTHREAD_ONCE_ERROR";
        case NCP_ST_PTHREAD_SETSPECIFIC_ERROR:
            return "NCP_ST_PTHREAD_SETSPECIFIC_ERROR";
        case NCP_ST_STRLIST_INV_LIST:
            return "NCP_ST_STRLIST_INV_LIST";
        case NCP_ST_STRLIST_INV_NODE:
            return "NCP_ST_STRLIST_INV_NODE";
        case NCP_ST_STRLIST_INV_VALUE:
            return "NCP_ST_STRLIST_INV_VALUE";
        case NCP_ST_STRLIST_INV_TYPE:
            return "NCP_ST_STRLIST_INV_TYPE";
        case NCP_ST_STRLIST_INV_ITER:
            return "NCP_ST_STRLIST_INV_ITER";
        case NCP_ST_STRLIST_INV_CONV:
            return "NCP_ST_STRLIST_INV_CONV";
        case NCP_ST_STRLIST_INV_RANGE:
            return "NCP_ST_STRLIST_INV_RANGE";
        case NCP_ST_STRLIST_ITEM_EXCEED:
            return "NCP_ST_STRLIST_ITEM_EXCEED";
        case NCP_ST_AGEN_INVALID_HDL:
            return "NCP_ST_AGEN_INVALID_HDL";
        case NCP_ST_AGEN_INVALID_INST:
            return "NCP_ST_AGEN_INVALID_INST";
        case NCP_ST_AGEN_INVALID_FUNC:
            return "NCP_ST_AGEN_INVALID_FUNC";
        case NCP_ST_AGEN_INVALID_OP:
            return "NCP_ST_AGEN_INVALID_OP";
        case NCP_ST_AGEN_INVALID_INDEX:
            return "NCP_ST_AGEN_INVALID_INDEX";
        case NCP_ST_AGEN_INVALID_PTNWIDTH:
            return "NCP_ST_AGEN_INVALID_PTNWIDTH";
        case NCP_ST_AGEN_VNUMS_EXHAUSTED:
            return "NCP_ST_AGEN_VNUMS_EXHAUSTED";
        case NCP_ST_AGEN_INVALID_XSIZE:
            return "NCP_ST_AGEN_INVALID_XSIZE";
        case NCP_ST_AGEN_INVALID_UXSIZE:
            return "NCP_ST_AGEN_INVALID_UXSIZE";
        case NCP_ST_AGEN_NOT_YET_DONE:
            return "NCP_ST_AGEN_NOT_YET_DONE";
        case NCP_ST_AGEN_INVALID_PROGRAM_IMAGE:
            return "NCP_ST_AGEN_INVALID_PROGRAM_IMAGE";
        case NCP_ST_AGEN_MEM_OVERFLOW:
            return "NCP_ST_AGEN_MEM_OVERFLOW";
        case NCP_ST_AGEN_INVALID_FPO_VERSION:
            return "NCP_ST_AGEN_INVALID_FPO_VERSION";
        case NCP_ST_AGEN_CB_ABORT:
            return "NCP_ST_AGEN_CB_ABORT";
        case NCP_ST_AGEN_INVALID_FPO_BUFFER:
            return "NCP_ST_AGEN_INVALID_FPO_BUFFER";
        case NCP_ST_AGEN_INVALID_FPO_FILE_COOKIE:
            return "NCP_ST_AGEN_INVALID_FPO_FILE_COOKIE";
        case NCP_ST_AGEN_WRITE_EXCEEDS_TREESIZE:
            return "NCP_ST_AGEN_WRITE_EXCEEDS_TREESIZE";
        case NCP_ST_AGEN_INVALID_TREE:
            return "NCP_ST_AGEN_INVALID_TREE";
        case NCP_ST_AGEN_INVALID_PTN:
            return "NCP_ST_AGEN_INVALID_PTN";
        case NCP_ST_AGEN_INVALID_ACTION:
            return "NCP_ST_AGEN_INVALID_ACTION";
        case NCP_ST_AGEN_INVALID_CHECKPOINT:
            return "NCP_ST_AGEN_INVALID_CHECKPOINT";
        case NCP_ST_AGEN_INVALID_CHECKPOINT_PROCESSOR:
            return "NCP_ST_AGEN_INVALID_CHECKPOINT_PROCESSOR";
        case NCP_ST_AGEN_CHECKPOINT_READ_ERROR:
            return "NCP_ST_AGEN_CHECKPOINT_READ_ERROR";
        case NCP_ST_AGEN_CHECKPOINT_WRITE_ERROR:
            return "NCP_ST_AGEN_CHECKPOINT_WRITE_ERROR";
        case NCP_ST_AGEN_SYMBOL_TABLE_NOT_LOADED:
            return "NCP_ST_AGEN_SYMBOL_TABLE_NOT_LOADED";
        case NCP_ST_AGEN_SYMBOL_NOT_FOUND:
            return "NCP_ST_AGEN_SYMBOL_NOT_FOUND";
        case NCP_ST_AGEN_INVALID_MODE:
            return "NCP_ST_AGEN_INVALID_MODE";
        case NCP_ST_AGEN_INVALID_MODULE_NAME:
            return "NCP_ST_AGEN_INVALID_MODULE_NAME";
        case NCP_ST_AGEN_NO_FLOW_MEMORY:
            return "NCP_ST_AGEN_NO_FLOW_MEMORY";
        case NCP_ST_AGEN_ISU_NOT_ENABLED:
            return "NCP_ST_AGEN_ISU_NOT_ENABLED";
        case NCP_ST_AGEN_ISU_RMEM_TOO_SMALL:
            return "NCP_ST_AGEN_ISU_RMEM_TOO_SMALL";
        case NCP_ST_AGEN_ISU_FMEM_TOO_SMALL:
            return "NCP_ST_AGEN_ISU_FMEM_TOO_SMALL";
        case NCP_ST_AGEN_ISU_NEW_FN_NAME_CONFLICT:
            return "NCP_ST_AGEN_ISU_NEW_FN_NAME_CONFLICT";
        case NCP_ST_AGEN_ISU_OLD_FN_NAME_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_NAME_MISMATCH";
        case NCP_ST_AGEN_ISU_OLD_FN_TYPE_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_TYPE_MISMATCH";
        case NCP_ST_AGEN_ISU_OLD_FN_WIDTH_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_WIDTH_MISMATCH";
        case NCP_ST_AGEN_ISU_OLD_FN_VSIZE_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_VSIZE_MISMATCH";
        case NCP_ST_AGEN_ISU_OLD_FN_SRCFNS_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_SRCFNS_MISMATCH";
        case NCP_ST_AGEN_ISU_OLD_FN_SWIDTHS_MISMATCH:
            return "NCP_ST_AGEN_ISU_OLD_FN_SWIDTHS_MISMATCH";
        case NCP_ST_AGEN_ISU_DEL_FN_CALLER_EXISTS:
            return "NCP_ST_AGEN_ISU_DEL_FN_CALLER_EXISTS";
        case NCP_ST_AGEN_ISU_VTREE_SRCFN_INVALID:
            return "NCP_ST_AGEN_ISU_VTREE_SRCFN_INVALID";
        case NCP_ST_AGEN_SCAN_REGEX_MISSING_PAREN:
            return "NCP_ST_AGEN_SCAN_REGEX_MISSING_PAREN";
        case NCP_ST_AGEN_SCAN_REGEX_MISSING_BRACE:
            return "NCP_ST_AGEN_SCAN_REGEX_MISSING_BRACE";
        case NCP_ST_AGEN_SCAN_REGEX_INVALID_INTERVAL:
            return "NCP_ST_AGEN_SCAN_REGEX_INVALID_INTERVAL";
        case NCP_ST_AGEN_SCAN_REGEX_MALFORMED_QUANTIFIER:
            return "NCP_ST_AGEN_SCAN_REGEX_MALFORMED_QUANTIFIER";
        case NCP_ST_AGEN_SCAN_REGEX_UNEXPECTED_TOKEN:
            return "NCP_ST_AGEN_SCAN_REGEX_UNEXPECTED_TOKEN";
        case NCP_ST_DEV_INVALID_DEVICE_NUMBER:
            return "NCP_ST_DEV_INVALID_DEVICE_NUMBER";
        case NCP_ST_DEV_DEVICE_NOT_FOUND:
            return "NCP_ST_DEV_DEVICE_NOT_FOUND";
        case NCP_ST_DEV_DEVICE_FILE_NOT_FOUND:
            return "NCP_ST_DEV_DEVICE_FILE_NOT_FOUND";
        case NCP_ST_DEV_DEVICE_CONFIG_ERROR:
            return "NCP_ST_DEV_DEVICE_CONFIG_ERROR";
        case NCP_ST_DEV_INVALID_IOCTL_COMMAND:
            return "NCP_ST_DEV_INVALID_IOCTL_COMMAND";
        case NCP_ST_DEV_INVALID_BIOS_COMMAND:
            return "NCP_ST_DEV_INVALID_BIOS_COMMAND";
        case NCP_ST_DEV_INVALID_HANDLE:
            return "NCP_ST_DEV_INVALID_HANDLE";
        case NCP_ST_DEV_INVALID_BUFFER:
            return "NCP_ST_DEV_INVALID_BUFFER";
        case NCP_ST_DEV_INVALID_FLAGS:
            return "NCP_ST_DEV_INVALID_FLAGS";
        case NCP_ST_DEV_INVALID_REGION_ID:
            return "NCP_ST_DEV_INVALID_REGION_ID";
        case NCP_ST_DEV_INVALID_ACCESS:
            return "NCP_ST_DEV_INVALID_ACCESS";
        case NCP_ST_DEV_INVALID_PAGE_ACCESS:
            return "NCP_ST_DEV_INVALID_PAGE_ACCESS";
        case NCP_ST_DEV_INVALID_WIDTH:
            return "NCP_ST_DEV_INVALID_WIDTH";
        case NCP_ST_DEV_INVALID_OFFSET:
            return "NCP_ST_DEV_INVALID_OFFSET";
        case NCP_ST_DEV_INVALID_COMMAND:
            return "NCP_ST_DEV_INVALID_COMMAND";
        case NCP_ST_DEV_UNALIGNED_OFFSET:
            return "NCP_ST_DEV_UNALIGNED_OFFSET";
        case NCP_ST_DEV_UNMAPPED_ERROR:
            return "NCP_ST_DEV_UNMAPPED_ERROR";
        case NCP_ST_DEV_NO_MEMORY:
            return "NCP_ST_DEV_NO_MEMORY";
        case NCP_ST_DEV_IO_ERROR:
            return "NCP_ST_DEV_IO_ERROR";
        case NCP_ST_DEV_PIO_FAILED:
            return "NCP_ST_DEV_PIO_FAILED";
        case NCP_ST_DEV_PIO_SW_TIMEOUT:
            return "NCP_ST_DEV_PIO_SW_TIMEOUT";
        case NCP_ST_DEV_PIO_TIMEOUT_ERR:
            return "NCP_ST_DEV_PIO_TIMEOUT_ERR";
        case NCP_ST_DEV_PIO_DECODE_ERR:
            return "NCP_ST_DEV_PIO_DECODE_ERR";
        case NCP_ST_DEV_PIO_NODE_ERR:
            return "NCP_ST_DEV_PIO_NODE_ERR";
        case NCP_ST_DEV_PIO_WRONG_ACK_ERR:
            return "NCP_ST_DEV_PIO_WRONG_ACK_ERR";
        case NCP_ST_DEV_PIO_ACK_TIMEOUT_ERR:
            return "NCP_ST_DEV_PIO_ACK_TIMEOUT_ERR";
        case NCP_ST_DEV_PIO_UNCLAIMED_HDR_ERR:
            return "NCP_ST_DEV_PIO_UNCLAIMED_HDR_ERR";
        case NCP_ST_DEV_PIO_RING0_PARITY_ERR:
            return "NCP_ST_DEV_PIO_RING0_PARITY_ERR";
        case NCP_ST_DEV_PIO_RING1_PARITY_ERR:
            return "NCP_ST_DEV_PIO_RING1_PARITY_ERR";
        case NCP_ST_DEV_PIO_RING2_PARITY_ERR:
            return "NCP_ST_DEV_PIO_RING2_PARITY_ERR";
        case NCP_ST_DEV_INTERRUPTED:
            return "NCP_ST_DEV_INTERRUPTED";
        case NCP_ST_DEV_WRONG_TARGET_HW:
            return "NCP_ST_DEV_WRONG_TARGET_HW";
        case NCP_ST_DEV_EVENT_TYPE_INVALID:
            return "NCP_ST_DEV_EVENT_TYPE_INVALID";
        case NCP_ST_DEV_EVENT_CBK_INVALID:
            return "NCP_ST_DEV_EVENT_CBK_INVALID";
        case NCP_ST_DEV_EVENT_HDL_INVALID:
            return "NCP_ST_DEV_EVENT_HDL_INVALID";
        case NCP_ST_DEV_EVENT_HANDLER_IN_USE:
            return "NCP_ST_DEV_EVENT_HANDLER_IN_USE";
        case NCP_ST_DEV_DMA_DISABLED:
            return "NCP_ST_DEV_DMA_DISABLED";
        case NCP_ST_DEV_PCQ_TIMEOUT:
            return "NCP_ST_DEV_PCQ_TIMEOUT";
        case NCP_ST_DEV_ISR_ALREADY_REGISTERED:
            return "NCP_ST_DEV_ISR_ALREADY_REGISTERED";
        case NCP_ST_DEV_ISR_NOT_REGISTERED:
            return "NCP_ST_DEV_ISR_NOT_REGISTERED";
        case NCP_ST_DEV_CANT_MAP_VIRQ:
            return "NCP_ST_DEV_CANT_MAP_VIRQ";
        case NCP_ST_DEV_CANT_SET_IRQ_TYPE:
            return "NCP_ST_DEV_CANT_SET_IRQ_TYPE";
        case NCP_ST_DEV_REQUEST_IRQ_ERR:
            return "NCP_ST_DEV_REQUEST_IRQ_ERR";
        case NCP_ST_DEV_DOMAIN_LOCK_TIMEOUT:
            return "NCP_ST_DEV_DOMAIN_LOCK_TIMEOUT";
        case NCP_ST_DEV_L2C_RESET_ERROR:
            return "NCP_ST_DEV_L2C_RESET_ERROR";
        case NCP_ST_DEV_L2C_INVALID_OFFSET:
            return "NCP_ST_DEV_L2C_INVALID_OFFSET";
        case NCP_ST_DEV_L2C_INVALID_REGIONID:
            return "NCP_ST_DEV_L2C_INVALID_REGIONID";
        case NCP_ST_DEV_L2C_INVALID_TARGET:
            return "NCP_ST_DEV_L2C_INVALID_TARGET";
        case NCP_ST_DEV_TRACE_INVALID_OFFSET:
            return "NCP_ST_DEV_TRACE_INVALID_OFFSET";
        case NCP_ST_DEV_TRACE_INVALID_REGIONID:
            return "NCP_ST_DEV_TRACE_INVALID_REGIONID";
        case NCP_ST_DEV_LBI_NOT_SUPPORTED:
            return "NCP_ST_DEV_LBI_NOT_SUPPORTED";
        case NCP_ST_DEV_LBI_ACCESS_TIMEOUT:
            return "NCP_ST_DEV_LBI_ACCESS_TIMEOUT";
        case NCP_ST_DEV_LBI_CS_INVALID:
            return "NCP_ST_DEV_LBI_CS_INVALID";
        case NCP_ST_DEV_LBI_CS_NOT_CONFIGURED:
            return "NCP_ST_DEV_LBI_CS_NOT_CONFIGURED";
        case NCP_ST_DEV_LBI_CS_DEV_WIDTH_INVALID:
            return "NCP_ST_DEV_LBI_CS_DEV_WIDTH_INVALID";
        case NCP_ST_DEV_LBI_GPIO_SEL_INVALID:
            return "NCP_ST_DEV_LBI_GPIO_SEL_INVALID";
        case NCP_ST_DEV_LBI_GPIO_MODE_INVALID:
            return "NCP_ST_DEV_LBI_GPIO_MODE_INVALID";
        case NCP_ST_DEV_LBI_GPIO_SEL_NOT_CONFIGURED:
            return "NCP_ST_DEV_LBI_GPIO_SEL_NOT_CONFIGURED";
        case NCP_ST_DEV_LBI_GPIO_IN_USE:
            return "NCP_ST_DEV_LBI_GPIO_IN_USE";
        case NCP_ST_DEV_LBI_INTR_SEL_INVALID:
            return "NCP_ST_DEV_LBI_INTR_SEL_INVALID";
        case NCP_ST_DEV_INVALID_VAT_POOL_ID:
            return "NCP_ST_DEV_INVALID_VAT_POOL_ID";
        case NCP_ST_DEV_INVALID_VAT_POOL_ENTRY:
            return "NCP_ST_DEV_INVALID_VAT_POOL_ENTRY";
        case NCP_ST_EDEV_INIT_FAILED:
            return "NCP_ST_EDEV_INIT_FAILED";
        case NCP_ST_EDEV_FINI_FAILED:
            return "NCP_ST_EDEV_FINI_FAILED";
        case NCP_ST_HWIO_TRACE_BUSY:
            return "NCP_ST_HWIO_TRACE_BUSY";
        case NCP_ST_DEV_APB2SER_INDIRECT_TIMEOUT:
            return "NCP_ST_DEV_APB2SER_INDIRECT_TIMEOUT";
        case NCP_ST_PLL_HANDLE_INVALID:
            return "NCP_ST_PLL_HANDLE_INVALID";
        case NCP_ST_PLL_SEM_INIT_ERROR:
            return "NCP_ST_PLL_SEM_INIT_ERROR";
        case NCP_ST_PLL_FREQUENCY_INVALID:
            return "NCP_ST_PLL_FREQUENCY_INVALID";
        case NCP_ST_PLL_OPERATION_NOT_SUPPORTED:
            return "NCP_ST_PLL_OPERATION_NOT_SUPPORTED";
        case NCP_ST_NS_INVALID_VALUE:
            return "NCP_ST_NS_INVALID_VALUE";
        case NCP_ST_NS_MEM_INIT_ERROR:
            return "NCP_ST_NS_MEM_INIT_ERROR";
        case NCP_ST_NS_ENGINE_UNKNOWN:
            return "NCP_ST_NS_ENGINE_UNKNOWN";
        case NCP_ST_NS_ENTRY_OUT_OF_RANGE:
            return "NCP_ST_NS_ENTRY_OUT_OF_RANGE";
        case NCP_ST_NS_BUFFER_TOO_SMALL:
            return "NCP_ST_NS_BUFFER_TOO_SMALL";
        case NCP_ST_NS_NO_NAMESPACES_DEFINED:
            return "NCP_ST_NS_NO_NAMESPACES_DEFINED";
        case NCP_ST_NS_ENTRY_ID_INVALID:
            return "NCP_ST_NS_ENTRY_ID_INVALID";
        case NCP_ST_NS_CALLBACK_MISSING:
            return "NCP_ST_NS_CALLBACK_MISSING";
        case NCP_ST_NS_CANNOT_HAVE_STATIC_ENTRIES:
            return "NCP_ST_NS_CANNOT_HAVE_STATIC_ENTRIES";
        case NCP_ST_NS_ENTRY_ID_TOO_BIG:
            return "NCP_ST_NS_ENTRY_ID_TOO_BIG";
        case NCP_ST_RSRC_INIT_FAILED:
            return "NCP_ST_RSRC_INIT_FAILED";
        case NCP_ST_RSRC_INSUFF_IDS_AVAIL:
            return "NCP_ST_RSRC_INSUFF_IDS_AVAIL";
        case NCP_ST_RSRC_INSUFF_IDS_AVAIL_IN_RANGE:
            return "NCP_ST_RSRC_INSUFF_IDS_AVAIL_IN_RANGE";
        case NCP_ST_RSRC_READ_ONLY:
            return "NCP_ST_RSRC_READ_ONLY";
        case NCP_ST_PAB_MEM_INIT_ERROR:
            return "NCP_ST_PAB_MEM_INIT_ERROR";
        case NCP_ST_PAB_INVALID_SHIFT_VALUE:
            return "NCP_ST_PAB_INVALID_SHIFT_VALUE";
        case NCP_ST_PAB_INVALID_VALUE:
            return "NCP_ST_PAB_INVALID_VALUE";
        case NCP_ST_PIC_INVALID_VALUE:
            return "NCP_ST_PIC_INVALID_VALUE";
        case NCP_ST_PKA_NOT_INSTALLED:
            return "NCP_ST_PKA_NOT_INSTALLED";
        case NCP_ST_PKA_NOT_AVAILABLE:
            return "NCP_ST_PKA_NOT_AVAILABLE";
        case NCP_ST_PKA_INVALID_VALUE:
            return "NCP_ST_PKA_INVALID_VALUE";
        case NCP_ST_PKA_INVALID_ARGUMENT_LENGTHS:
            return "NCP_ST_PKA_INVALID_ARGUMENT_LENGTHS";
        case NCP_ST_PKA_OPERATION_NOT_RUNNING:
            return "NCP_ST_PKA_OPERATION_NOT_RUNNING";
        case NCP_ST_PKA_OPERATION_IN_PROGRESS:
            return "NCP_ST_PKA_OPERATION_IN_PROGRESS";
        case NCP_ST_PKA_INSUFFICIENT_ARGUMENT_LENGTHS:
            return "NCP_ST_PKA_INSUFFICIENT_ARGUMENT_LENGTHS";
        case NCP_ST_TRNG_NOT_INSTALLED:
            return "NCP_ST_TRNG_NOT_INSTALLED";
        case NCP_ST_TRNG_INVALID_VALUE:
            return "NCP_ST_TRNG_INVALID_VALUE";
        case NCP_ST_TRNG_RESEED_TIMEOUT:
            return "NCP_ST_TRNG_RESEED_TIMEOUT";
        case NCP_ST_TRNG_BIST_ERROR:
            return "NCP_ST_TRNG_BIST_ERROR";
        case NCP_ST_TRNG_FIPS_OUT_ERROR:
            return "NCP_ST_TRNG_FIPS_OUT_ERROR";
        case NCP_ST_TRNG_STS_ERROR:
            return "NCP_ST_TRNG_STS_ERROR";
        case NCP_ST_TRNG_STS_GENERATE_ERROR:
            return "NCP_ST_TRNG_STS_GENERATE_ERROR";
        case NCP_ST_TRNG_STS_INVALID_DATA_ERROR:
            return "NCP_ST_TRNG_STS_INVALID_DATA_ERROR";
        case NCP_ST_TRNG_DOUT_VALUE_TIMEOUT:
            return "NCP_ST_TRNG_DOUT_VALUE_TIMEOUT";
        case NCP_ST_SPP_PACKET_DRAIN_FAILED:
            return "NCP_ST_SPP_PACKET_DRAIN_FAILED";
        case NCP_ST_SPP_NOT_CONFIGURED:
            return "NCP_ST_SPP_NOT_CONFIGURED";
        case NCP_ST_SPP_DEVICE_UNKNOWN:
            return "NCP_ST_SPP_DEVICE_UNKNOWN";
        case NCP_ST_SPP_FW_INV_SCRIPT_NAME:
            return "NCP_ST_SPP_FW_INV_SCRIPT_NAME";
        case NCP_ST_SPP_FW_INV_SCRIPT_ID:
            return "NCP_ST_SPP_FW_INV_SCRIPT_ID";
        case NCP_ST_SPP_FW_FILE_INVALID:
            return "NCP_ST_SPP_FW_FILE_INVALID";
        case NCP_ST_SPP_FIRMWARE_MISSING:
            return "NCP_ST_SPP_FIRMWARE_MISSING";
        case NCP_ST_SPP_CTX_SCRIPT_MISSING:
            return "NCP_ST_SPP_CTX_SCRIPT_MISSING";
        case NCP_ST_SPP_CTX_LIBRARY_MISSING:
            return "NCP_ST_SPP_CTX_LIBRARY_MISSING";
        case NCP_ST_SPP_CTX_DISABLED:
            return "NCP_ST_SPP_CTX_DISABLED";
        case NCP_ST_SPP_CTX_PROTO_INVALID:
            return "NCP_ST_SPP_CTX_PROTO_INVALID";
        case NCP_ST_SPP_CTX_NEED_4B_FRESH:
            return "NCP_ST_SPP_CTX_NEED_4B_FRESH";
        case NCP_ST_SPP_CTX_FRESH_NOT_NEEDED:
            return "NCP_ST_SPP_CTX_FRESH_NOT_NEEDED";
        case NCP_ST_SPP_CTX_AUTH_INVALID:
            return "NCP_ST_SPP_CTX_AUTH_INVALID";
        case NCP_ST_SPP_CTX_AUTH_MUST_BE_GMAC:
            return "NCP_ST_SPP_CTX_AUTH_MUST_BE_GMAC";
        case NCP_ST_SPP_CTX_AUTH_MUST_BE_NULL:
            return "NCP_ST_SPP_CTX_AUTH_MUST_BE_NULL";
        case NCP_ST_SPP_CTX_COMB_NOT_SUPPORTED:
            return "NCP_ST_SPP_CTX_COMB_NOT_SUPPORTED";
        case NCP_ST_SPP_CTX_AUTH_KEY_LEN_INVALID:
            return "NCP_ST_SPP_CTX_AUTH_KEY_LEN_INVALID";
        case NCP_ST_SPP_CTX_ENCR_NOT_SUPPORTED:
            return "NCP_ST_SPP_CTX_ENCR_NOT_SUPPORTED";
        case NCP_ST_SPP_CTX_ENCR_INVALID:
            return "NCP_ST_SPP_CTX_ENCR_INVALID";
        case NCP_ST_SPP_CTX_ENCR_MUST_BE_NULL_GMAC:
            return "NCP_ST_SPP_CTX_ENCR_MUST_BE_NULL_GMAC";
        case NCP_ST_SPP_CTX_ENCR_KEY_LEN_INVALID:
            return "NCP_ST_SPP_CTX_ENCR_KEY_LEN_INVALID";
        case NCP_ST_SPP_CTX_NEED_4B_SALT:
            return "NCP_ST_SPP_CTX_NEED_4B_SALT";
        case NCP_ST_SPP_CTX_NEED_3B_SALT:
            return "NCP_ST_SPP_CTX_NEED_3B_SALT";
        case NCP_ST_SPP_CTX_SALT_NOT_NEEDED:
            return "NCP_ST_SPP_CTX_SALT_NOT_NEEDED";
        case NCP_ST_SPP_CTX_NEED_ANTI_REPLAY_IN_ESN:
            return "NCP_ST_SPP_CTX_NEED_ANTI_REPLAY_IN_ESN";
        case NCP_ST_SPP_CTX_SALT_KEY_LEN_INVALID:
            return "NCP_ST_SPP_CTX_SALT_KEY_LEN_INVALID";
        case NCP_ST_SPP_CTX_AUTH_TAG_LEN_INVALID:
            return "NCP_ST_SPP_CTX_AUTH_TAG_LEN_INVALID";
        case NCP_ST_SPP_CTX_LIFETIME_INVALID:
            return "NCP_ST_SPP_CTX_LIFETIME_INVALID";
        case NCP_ST_SPP_CTX_ARW_SIZE_INVALID:
            return "NCP_ST_SPP_CTX_ARW_SIZE_INVALID";
        case NCP_ST_SPP_CTX_COUNTERS_MUST_BE_PRESENT:
            return "NCP_ST_SPP_CTX_COUNTERS_MUST_BE_PRESENT";
        case NCP_ST_SPP_CTX_IV_INVALID:
            return "NCP_ST_SPP_CTX_IV_INVALID";
        case NCP_ST_SPP_CTX_CIPHER_STATE_INVALID:
            return "NCP_ST_SPP_CTX_CIPHER_STATE_INVALID";
        case NCP_ST_SPP_TRUNCATED_HMAC_INVALID_FOR_SSL:
            return "NCP_ST_SPP_TRUNCATED_HMAC_INVALID_FOR_SSL";
        case NCP_ST_SPP_CTX_TOO_BIG:
            return "NCP_ST_SPP_CTX_TOO_BIG";
        case NCP_ST_SPP_CTX_INVALID:
            return "NCP_ST_SPP_CTX_INVALID";
        case NCP_ST_SPP_CTX_COUNTERS_MISSING:
            return "NCP_ST_SPP_CTX_COUNTERS_MISSING";
        case NCP_ST_SPP_CTX_ICV_LEN_INVALID:
            return "NCP_ST_SPP_CTX_ICV_LEN_INVALID";
        case NCP_ST_SPP_CTX_WORD3_MISSING_AIDSRC_SA:
            return "NCP_ST_SPP_CTX_WORD3_MISSING_AIDSRC_SA";
        case NCP_ST_SPP_CTX_AID_LEN_MAX_256_EXCEED:
            return "NCP_ST_SPP_CTX_AID_LEN_MAX_256_EXCEED";
        case NCP_ST_TIMER_INVALID_CLASS:
            return "NCP_ST_TIMER_INVALID_CLASS";
        case NCP_ST_TIMER_INVALID_INDEX:
            return "NCP_ST_TIMER_INVALID_INDEX";
        case NCP_ST_TIMER_INVALID_HANDLE:
            return "NCP_ST_TIMER_INVALID_HANDLE";
        case NCP_ST_TIMER_INVALID_QUEUE_ID:
            return "NCP_ST_TIMER_INVALID_QUEUE_ID";
        case NCP_ST_TIMER_INVALID_FLAGS:
            return "NCP_ST_TIMER_INVALID_FLAGS";
        case NCP_ST_TIMER_NO_INDEX:
            return "NCP_ST_TIMER_NO_INDEX";
        case NCP_ST_TIMER_INIT_FAILED:
            return "NCP_ST_TIMER_INIT_FAILED";
        case NCP_ST_TIMER_NO_VALID_VP:
            return "NCP_ST_TIMER_NO_VALID_VP";
        case NCP_ST_TIMER_NO_VALID_CLASS_HANDLER:
            return "NCP_ST_TIMER_NO_VALID_CLASS_HANDLER";
        case NCP_ST_TIMER_REF_STROBE_OUTPUT_NOT_ENABLED:
            return "NCP_ST_TIMER_REF_STROBE_OUTPUT_NOT_ENABLED";
        case NCP_ST_TIMER_SEM_INIT_ERROR:
            return "NCP_ST_TIMER_SEM_INIT_ERROR";
        case NCP_ST_TIMER_INVALID_REFCLK:
            return "NCP_ST_TIMER_INVALID_REFCLK";
        case NCP_ST_TIMER_CPU_TIMER_NOT_STOPPED:
            return "NCP_ST_TIMER_CPU_TIMER_NOT_STOPPED";
        case NCP_ST_TIMER_CPU_TIMER_NOT_STARTED:
            return "NCP_ST_TIMER_CPU_TIMER_NOT_STARTED";
        case NCP_ST_TIMER_CPU_TIMER_NOT_ALLOCATED:
            return "NCP_ST_TIMER_CPU_TIMER_NOT_ALLOCATED";
        case NCP_ST_SMON_INVALID_MODULE:
            return "NCP_ST_SMON_INVALID_MODULE";
        case NCP_ST_SMON_INVALID_SUBMODULE:
            return "NCP_ST_SMON_INVALID_SUBMODULE";
        case NCP_ST_SMON_INVALID_PRESCALE:
            return "NCP_ST_SMON_INVALID_PRESCALE";
        case NCP_ST_SMON_INVALID_FUNCTION:
            return "NCP_ST_SMON_INVALID_FUNCTION";
        case NCP_ST_SMON_INVALID_MODE:
            return "NCP_ST_SMON_INVALID_MODE";
        case NCP_ST_SMON_INVALID_MUX:
            return "NCP_ST_SMON_INVALID_MUX";
        case NCP_ST_SMON_INVALID_MUX_MODE:
            return "NCP_ST_SMON_INVALID_MUX_MODE";
        case NCP_ST_SMON_INVALID_PARAM:
            return "NCP_ST_SMON_INVALID_PARAM";
        case NCP_ST_MMB_INVALID_MEM_SIZE:
            return "NCP_ST_MMB_INVALID_MEM_SIZE";
        case NCP_ST_MMB_INVALID_THRESHOLD:
            return "NCP_ST_MMB_INVALID_THRESHOLD";
        case NCP_ST_MMB_INIT_FAILED:
            return "NCP_ST_MMB_INIT_FAILED";
        case NCP_ST_MME_INVALID_MPCQ_ID:
            return "NCP_ST_MME_INVALID_MPCQ_ID";
        case NCP_ST_MME_INVALID_MPCQ_ENTRY_SIZE:
            return "NCP_ST_MME_INVALID_MPCQ_ENTRY_SIZE";
        case NCP_ST_MME_INVALID_POOL_ID:
            return "NCP_ST_MME_INVALID_POOL_ID";
        case NCP_ST_MME_INVALID_BLOCK_SIZE:
            return "NCP_ST_MME_INVALID_BLOCK_SIZE";
        case NCP_ST_MME_INIT_FAILED:
            return "NCP_ST_MME_INIT_FAILED";
        case NCP_ST_MME_INVALID_HANDLE:
            return "NCP_ST_MME_INVALID_HANDLE";
        case NCP_ST_MME_CAT_INVALID_FORMAT:
            return "NCP_ST_MME_CAT_INVALID_FORMAT";
        case NCP_ST_MME_FLUSH_IFLS_NOT_STOPPED:
            return "NCP_ST_MME_FLUSH_IFLS_NOT_STOPPED";
        case NCP_ST_MME_FLUSH_MREF_NOT_STOPPED:
            return "NCP_ST_MME_FLUSH_MREF_NOT_STOPPED";
        case NCP_ST_MME_FLUSH_MPCQ_NOT_STOPPED:
            return "NCP_ST_MME_FLUSH_MPCQ_NOT_STOPPED";
        case NCP_ST_MME_INVALID_POOL_SIZE:
            return "NCP_ST_MME_INVALID_POOL_SIZE";
        case NCP_ST_PBM_INIT_FAILED:
            return "NCP_ST_PBM_INIT_FAILED";
        case NCP_ST_MTM_HANDLE_INVALID:
            return "NCP_ST_MTM_HANDLE_INVALID";
        case NCP_ST_MTM_SEM_INIT_ERROR:
            return "NCP_ST_MTM_SEM_INIT_ERROR";
        case NCP_ST_MTM_RATE_ROUNDING_ALG_INVALID:
            return "NCP_ST_MTM_RATE_ROUNDING_ALG_INVALID";
        case NCP_ST_MTM_SCHEDULING_MODE_INVALID:
            return "NCP_ST_MTM_SCHEDULING_MODE_INVALID";
        case NCP_ST_MTM_PRIORITY_LEVEL_IN_USE:
            return "NCP_ST_MTM_PRIORITY_LEVEL_IN_USE";
        case NCP_ST_MTM_MR_IN_STRICT_MODE:
            return "NCP_ST_MTM_MR_IN_STRICT_MODE";
        case NCP_ST_MTM_SHAPING_MODE_INVALID:
            return "NCP_ST_MTM_SHAPING_MODE_INVALID";
        case NCP_ST_MTM_SHAPING_RATES_INCOMPATIBLE:
            return "NCP_ST_MTM_SHAPING_RATES_INCOMPATIBLE";
        case NCP_ST_MTM_SHAPING_PEAK_LESS_THAN_SECONDARY:
            return "NCP_ST_MTM_SHAPING_PEAK_LESS_THAN_SECONDARY";
        case NCP_ST_MTM_SHAPING_BUCKET_LESS_THAN_MAX_PACKET:
            return "NCP_ST_MTM_SHAPING_BUCKET_LESS_THAN_MAX_PACKET";
        case NCP_ST_MTM_HW_SHAPER_NOT_IN_USE:
            return "NCP_ST_MTM_HW_SHAPER_NOT_IN_USE";
        case NCP_ST_MTM_TIME_SCALE_INVALID:
            return "NCP_ST_MTM_TIME_SCALE_INVALID";
        case NCP_ST_MTM_TIME_SCALE_INCOMPATIBLE:
            return "NCP_ST_MTM_TIME_SCALE_INCOMPATIBLE";
        case NCP_ST_MTM_SERVICE_QUANTUM_INVALID:
            return "NCP_ST_MTM_SERVICE_QUANTUM_INVALID";
        case NCP_ST_MTM_SERVICE_QUANTUM_NOT_APPLICABLE:
            return "NCP_ST_MTM_SERVICE_QUANTUM_NOT_APPLICABLE";
        case NCP_ST_MTM_PARAMETER_LENGTH_INVALID:
            return "NCP_ST_MTM_PARAMETER_LENGTH_INVALID";
        case NCP_ST_MTM_OFFSET_INVALID:
            return "NCP_ST_MTM_OFFSET_INVALID";
        case NCP_ST_MTM_PARENT_DOES_NOT_EXIST:
            return "NCP_ST_MTM_PARENT_DOES_NOT_EXIST";
        case NCP_ST_MTM_PARENT_ID_INVALID:
            return "NCP_ST_MTM_PARENT_ID_INVALID";
        case NCP_ST_MTM_PARENT_DRAINING:
            return "NCP_ST_MTM_PARENT_DRAINING";
        case NCP_ST_MTM_TOO_MANY_CHILDREN:
            return "NCP_ST_MTM_TOO_MANY_CHILDREN";
        case NCP_ST_MTM_PARENT_HIER_LEVEL_INCORRECT:
            return "NCP_ST_MTM_PARENT_HIER_LEVEL_INCORRECT";
        case NCP_ST_MTM_SCHEDULER_EXISTS:
            return "NCP_ST_MTM_SCHEDULER_EXISTS";
        case NCP_ST_MTM_SCHEDULER_DOES_NOT_EXIST:
            return "NCP_ST_MTM_SCHEDULER_DOES_NOT_EXIST";
        case NCP_ST_MTM_SCHEDULER_ID_INVALID:
            return "NCP_ST_MTM_SCHEDULER_ID_INVALID";
        case NCP_ST_MTM_SCHEDULER_ARB_MODE_INVALID:
            return "NCP_ST_MTM_SCHEDULER_ARB_MODE_INVALID";
        case NCP_ST_MTM_SCHEDULER_TS_SCRIPT_ID_INVALID:
            return "NCP_ST_MTM_SCHEDULER_TS_SCRIPT_ID_INVALID";
        case NCP_ST_MTM_SCHEDULER_TS_SCRIPT_DOES_NOT_EXIST:
            return "NCP_ST_MTM_SCHEDULER_TS_SCRIPT_DOES_NOT_EXIST";
        case NCP_ST_MTM_SCHEDULER_TS_SCRIPT_REQUIRED:
            return "NCP_ST_MTM_SCHEDULER_TS_SCRIPT_REQUIRED";
        case NCP_ST_MTM_SCHEDULER_TS_ALREADY_RUNNING:
            return "NCP_ST_MTM_SCHEDULER_TS_ALREADY_RUNNING";
        case NCP_ST_MTM_SCHEDULER_HAS_CHILDREN:
            return "NCP_ST_MTM_SCHEDULER_HAS_CHILDREN";
        case NCP_ST_MTM_SCHEDULER_WAIT_FOR_INACTIVE_TIMEOUT:
            return "NCP_ST_MTM_SCHEDULER_WAIT_FOR_INACTIVE_TIMEOUT";
        case NCP_ST_MTM_SCHEDULER_AVG_ERROR_INVALID:
            return "NCP_ST_MTM_SCHEDULER_AVG_ERROR_INVALID";
        case NCP_ST_MTM_QUEUE_EXISTS:
            return "NCP_ST_MTM_QUEUE_EXISTS";
        case NCP_ST_MTM_QUEUE_DOES_NOT_EXIST:
            return "NCP_ST_MTM_QUEUE_DOES_NOT_EXIST";
        case NCP_ST_MTM_QUEUE_ID_INVALID:
            return "NCP_ST_MTM_QUEUE_ID_INVALID";
        case NCP_ST_MTM_QUEUE_NO_TS_RUNNING:
            return "NCP_ST_MTM_QUEUE_NO_TS_RUNNING";
        case NCP_ST_MTM_QUEUE_TM_PARAMS_HIGHER_LEVEL_INVALID:
            return "NCP_ST_MTM_QUEUE_TM_PARAMS_HIGHER_LEVEL_INVALID";
        case NCP_ST_MTM_QUEUE_TS_PARAMS_HIGHER_LEVEL_INVALID:
            return "NCP_ST_MTM_QUEUE_TS_PARAMS_HIGHER_LEVEL_INVALID";
        case NCP_ST_MTM_QUEUE_TM_SCRIPT_ID_INVALID:
            return "NCP_ST_MTM_QUEUE_TM_SCRIPT_ID_INVALID";
        case NCP_ST_MTM_QUEUE_TM_SCRIPT_DOES_NOT_EXIST:
            return "NCP_ST_MTM_QUEUE_TM_SCRIPT_DOES_NOT_EXIST";
        case NCP_ST_MTM_NODE_ID_INVALID:
            return "NCP_ST_MTM_NODE_ID_INVALID";
        case NCP_ST_MTM_DRAIN_BUSY:
            return "NCP_ST_MTM_DRAIN_BUSY";
        case NCP_ST_MTM_DRAIN_ENQUEUE_STOP_TIMEOUT:
            return "NCP_ST_MTM_DRAIN_ENQUEUE_STOP_TIMEOUT";
        case NCP_ST_MTM_DRAIN_TIMEOUT:
            return "NCP_ST_MTM_DRAIN_TIMEOUT";
        case NCP_ST_MTM_NODE_DRAINING:
            return "NCP_ST_MTM_NODE_DRAINING";
        case NCP_ST_MTM_MCAST_NO_MEMORY:
            return "NCP_ST_MTM_MCAST_NO_MEMORY";
        case NCP_ST_MTM_MCAST_GROUP_DOES_NOT_EXIST:
            return "NCP_ST_MTM_MCAST_GROUP_DOES_NOT_EXIST";
        case NCP_ST_MTM_MCAST_FLOW_NOT_IN_GROUP:
            return "NCP_ST_MTM_MCAST_FLOW_NOT_IN_GROUP";
        case NCP_ST_MTM_TW_QUEUE_FIFO_INIT_FAILED:
            return "NCP_ST_MTM_TW_QUEUE_FIFO_INIT_FAILED";
        case NCP_ST_MTM_INTERNAL_SW_ERROR:
            return "NCP_ST_MTM_INTERNAL_SW_ERROR";
        case NCP_ST_MTM_BG_NODE_REM_ACTIVE:
            return "NCP_ST_MTM_BG_NODE_REM_ACTIVE";
        case NCP_ST_MTM_BG_NODE_REM_INACTIVE:
            return "NCP_ST_MTM_BG_NODE_REM_INACTIVE";
        case NCP_ST_MTM_BG_NODE_REM_SHUTTINGDOWN:
            return "NCP_ST_MTM_BG_NODE_REM_SHUTTINGDOWN";
        case NCP_ST_MTM_QUEUE_BG_REM_PENDING:
            return "NCP_ST_MTM_QUEUE_BG_REM_PENDING";
        case NCP_ST_MTM_SCHEDULER_BG_REM_PENDING:
            return "NCP_ST_MTM_SCHEDULER_BG_REM_PENDING";
        case NCP_ST_SE_INVALID_MLT_INDEX:
            return "NCP_ST_SE_INVALID_MLT_INDEX";
        case NCP_ST_SE_INVALID_MLT_REQUEST_SIZE:
            return "NCP_ST_SE_INVALID_MLT_REQUEST_SIZE";
        case NCP_ST_SE_INVALID_HANDLE:
            return "NCP_ST_SE_INVALID_HANDLE";
        case NCP_ST_SE_INIT_FAILED:
            return "NCP_ST_SE_INIT_FAILED";
        case NCP_ST_SE_INVALID_FEM_POINTER:
            return "NCP_ST_SE_INVALID_FEM_POINTER";
        case NCP_ST_SE_FEM_NO_EVENTS:
            return "NCP_ST_SE_FEM_NO_EVENTS";
        case NCP_ST_SE_FEM_NOT_ENABLED:
            return "NCP_ST_SE_FEM_NOT_ENABLED";
        case NCP_ST_SE_FEM_EVENT_QUEUE_FLUSHED:
            return "NCP_ST_SE_FEM_EVENT_QUEUE_FLUSHED";
        case NCP_ST_SE_SEM_INIT_ERROR:
            return "NCP_ST_SE_SEM_INIT_ERROR";
        case NCP_ST_SE_NS_NOT_CONTIGUOUS:
            return "NCP_ST_SE_NS_NOT_CONTIGUOUS";
        case NCP_ST_SED_INVALID_PARAM_INDEX:
            return "NCP_ST_SED_INVALID_PARAM_INDEX";
        case NCP_ST_SED_INVALID_PARAM_REQUEST_SIZE:
            return "NCP_ST_SED_INVALID_PARAM_REQUEST_SIZE";
        case NCP_ST_SED_INVALID_HANDLE:
            return "NCP_ST_SED_INVALID_HANDLE";
        case NCP_ST_SED_INVALID_PARAM:
            return "NCP_ST_SED_INVALID_PARAM";
        case NCP_ST_SED_INIT_FAILED:
            return "NCP_ST_SED_INIT_FAILED";
        case NCP_ST_SED_SEM_INIT_ERROR:
            return "NCP_ST_SED_SEM_INIT_ERROR";
        case NCP_ST_CE_INVALID_PARAM_SIZE:
            return "NCP_ST_CE_INVALID_PARAM_SIZE";
        case NCP_ST_CE_INCOMPATIBLE_SCRIPT:
            return "NCP_ST_CE_INCOMPATIBLE_SCRIPT";
        case NCP_ST_CE_INCOMPATIBLE_LIBRARY:
            return "NCP_ST_CE_INCOMPATIBLE_LIBRARY";
        case NCP_ST_CE_INVALID_NUM_ALUS:
            return "NCP_ST_CE_INVALID_NUM_ALUS";
        case NCP_ST_CE_INVALID_MEM_SIZE:
            return "NCP_ST_CE_INVALID_MEM_SIZE";
        case NCP_ST_CE_INVALID_LIBRARY_NAME:
            return "NCP_ST_CE_INVALID_LIBRARY_NAME";
        case NCP_ST_CE_INVALID_LIBRARY_FUNCTION:
            return "NCP_ST_CE_INVALID_LIBRARY_FUNCTION";
        case NCP_ST_CE_INVALID_REGION:
            return "NCP_ST_CE_INVALID_REGION";
        case NCP_ST_CE_LIBRARY_FUNCTION_NOT_FOUND:
            return "NCP_ST_CE_LIBRARY_FUNCTION_NOT_FOUND";
        case NCP_ST_CE_SCRIPT_NO_MEMORY:
            return "NCP_ST_CE_SCRIPT_NO_MEMORY";
        case NCP_ST_CE_SCRIPT_INVALID_ID:
            return "NCP_ST_CE_SCRIPT_INVALID_ID";
        case NCP_ST_CE_SCRIPT_INVALID_ADDRESS:
            return "NCP_ST_CE_SCRIPT_INVALID_ADDRESS";
        case NCP_ST_CE_SCRIPT_IN_USE:
            return "NCP_ST_CE_SCRIPT_IN_USE";
        case NCP_ST_CE_SCRIPT_NOT_LOADED:
            return "NCP_ST_CE_SCRIPT_NOT_LOADED";
        case NCP_ST_CE_SCRIPT_LOADED:
            return "NCP_ST_CE_SCRIPT_LOADED";
        case NCP_ST_CE_LOAD_FAILED:
            return "NCP_ST_CE_LOAD_FAILED";
        case NCP_ST_CE_INIT_FAILED:
            return "NCP_ST_CE_INIT_FAILED";
        case NCP_ST_MISCOMPARE:
            return "NCP_ST_MISCOMPARE";
        case NCP_ST_INVALID_REG_SIZE:
            return "NCP_ST_INVALID_REG_SIZE";
        case NCP_ST_COW_INVALID_ENGINE:
            return "NCP_ST_COW_INVALID_ENGINE";
        case NCP_ST_COW_INVALID_PRIORITY:
            return "NCP_ST_COW_INVALID_PRIORITY";
        case NCP_ST_COW_INVALID_QUEUE_PRIO_MAP:
            return "NCP_ST_COW_INVALID_QUEUE_PRIO_MAP";
        case NCP_ST_COW_INVALID_NUM_LOGICAL_QUEUES:
            return "NCP_ST_COW_INVALID_NUM_LOGICAL_QUEUES";
        case NCP_ST_COW_INVALID_THRES_GROUP_ID:
            return "NCP_ST_COW_INVALID_THRES_GROUP_ID";
        case NCP_ST_COW_INVALID_DEST_INST_RANGE:
            return "NCP_ST_COW_INVALID_DEST_INST_RANGE";
        case NCP_ST_COW_INVALID_DEST_INST:
            return "NCP_ST_COW_INVALID_DEST_INST";
        case NCP_ST_COW_INVALID_RED_SLOPE:
            return "NCP_ST_COW_INVALID_RED_SLOPE";
        case NCP_ST_COW_INVALID_GLOBAL_BUF_ALL_THRES:
            return "NCP_ST_COW_INVALID_GLOBAL_BUF_ALL_THRES";
        case NCP_ST_COW_INVALID_GLOBAL_BUF_RANDOM_THRES:
            return "NCP_ST_COW_INVALID_GLOBAL_BUF_RANDOM_THRES";
        case NCP_ST_COW_INVALID_THRES_GROUP_RANDOM_THRES:
            return "NCP_ST_COW_INVALID_THRES_GROUP_RANDOM_THRES";
        case NCP_ST_COW_INVALID_THRES_GROUP_ALL_THRES:
            return "NCP_ST_COW_INVALID_THRES_GROUP_ALL_THRES";
        case NCP_ST_COW_INVALID_BACKPRESSURE_THRES:
            return "NCP_ST_COW_INVALID_BACKPRESSURE_THRES";
        case NCP_ST_COW_INVALID_THRESH_UNIT:
            return "NCP_ST_COW_INVALID_THRESH_UNIT";
        case NCP_ST_COW_INVALID_BP_THRESH_HYST:
            return "NCP_ST_COW_INVALID_BP_THRESH_HYST";
        case NCP_ST_COW_INVALID_ARB_MODE:
            return "NCP_ST_COW_INVALID_ARB_MODE";
        case NCP_ST_COW_INVALID_LOAD_BALANCE_MODE:
            return "NCP_ST_COW_INVALID_LOAD_BALANCE_MODE";
        case NCP_ST_COW_INVALID_PARAM_DEF_BUFFER:
            return "NCP_ST_COW_INVALID_PARAM_DEF_BUFFER";
        case NCP_ST_COW_INVALID_PARAM_SEL_BUFFER:
            return "NCP_ST_COW_INVALID_PARAM_SEL_BUFFER";
        case NCP_ST_COW_INVALID_TEMPLATE_ID:
            return "NCP_ST_COW_INVALID_TEMPLATE_ID";
        case NCP_ST_COW_INVALID_PARAM_DEF_LEN:
            return "NCP_ST_COW_INVALID_PARAM_DEF_LEN";
        case NCP_ST_COW_INVALID_ENTRY_SIZE:
            return "NCP_ST_COW_INVALID_ENTRY_SIZE";
        case NCP_ST_COW_INVALID_VP_ID:
            return "NCP_ST_COW_INVALID_VP_ID";
        case NCP_ST_COW_INVALID_MAX_ENTRIES:
            return "NCP_ST_COW_INVALID_MAX_ENTRIES";
        case NCP_ST_COW_NOT_LAUNCH_ENGINE:
            return "NCP_ST_COW_NOT_LAUNCH_ENGINE";
        case NCP_ST_COW_INVALID_INPUT_QUEUE_WEIGHT:
            return "NCP_ST_COW_INVALID_INPUT_QUEUE_WEIGHT";
        case NCP_ST_COW_INVALID_OUT_TEMPLATE_BUFFER:
            return "NCP_ST_COW_INVALID_OUT_TEMPLATE_BUFFER";
        case NCP_ST_COW_NOT_TASK_LOG_DEBUG_MODE:
            return "NCP_ST_COW_NOT_TASK_LOG_DEBUG_MODE";
        case NCP_ST_COW_TASK_TRACE_ALREADY_ENABLED:
            return "NCP_ST_COW_TASK_TRACE_ALREADY_ENABLED";
        case NCP_ST_COW_TASK_TRACE_ALREADY_DISABLED:
            return "NCP_ST_COW_TASK_TRACE_ALREADY_DISABLED";
        case NCP_ST_COW_TASK_TRACE_TRIGGER_ALREADY_DISABLED:
            return "NCP_ST_COW_TASK_TRACE_TRIGGER_ALREADY_DISABLED";
        case NCP_ST_COW_TASK_TRACE_INVALID_PARAM_OFFSET:
            return "NCP_ST_COW_TASK_TRACE_INVALID_PARAM_OFFSET";
        case NCP_ST_COW_TCLD_BUF_INIT_FAILURE:
            return "NCP_ST_COW_TCLD_BUF_INIT_FAILURE";
        case NCP_ST_COW_TDGL_ONLINE_FAILURE:
            return "NCP_ST_COW_TDGL_ONLINE_FAILURE";
        case NCP_ST_COW_TIL_ONLINE_FAILURE:
            return "NCP_ST_COW_TIL_ONLINE_FAILURE";
        case NCP_ST_COW_MAX_BP_DESTS:
            return "NCP_ST_COW_MAX_BP_DESTS";
        case NCP_ST_COW_UNSUPPORTED_BP_TO_DISCARD_THRESH_GROUP_RATIO:
            return "NCP_ST_COW_UNSUPPORTED_BP_TO_DISCARD_THRESH_GROUP_RATIO";
        case NCP_ST_COW_INVALID_CHIP:
            return "NCP_ST_COW_INVALID_CHIP";
        case NCP_ST_COW_INVALID_DISCARD_STATS_BUFFER:
            return "NCP_ST_COW_INVALID_DISCARD_STATS_BUFFER";
        case NCP_ST_COW_INVALID_QUEUE_INDEX_MAP_BUFFER:
            return "NCP_ST_COW_INVALID_QUEUE_INDEX_MAP_BUFFER";
        case NCP_ST_COW_INVALID_QUEUE:
            return "NCP_ST_COW_INVALID_QUEUE";
        case NCP_ST_COW_INVALID_MODE_TASK_BUFFER:
            return "NCP_ST_COW_INVALID_MODE_TASK_BUFFER";
        case NCP_ST_COW_INVALID_NUM_ITEMS_BUFFER:
            return "NCP_ST_COW_INVALID_NUM_ITEMS_BUFFER";
        case NCP_ST_VPM_INVALID_VP_HDL:
            return "NCP_ST_VPM_INVALID_VP_HDL";
        case NCP_ST_VPM_INVALID_VP_NAME:
            return "NCP_ST_VPM_INVALID_VP_NAME";
        case NCP_ST_VPM_INVALID_ENGINE_SEQUENCE_NAME:
            return "NCP_ST_VPM_INVALID_ENGINE_SEQUENCE_NAME";
        case NCP_ST_VPM_C_COOKIE_MISMATCH:
            return "NCP_ST_VPM_C_COOKIE_MISMATCH";
        case NCP_ST_VPM_VP_NOT_FOUND:
            return "NCP_ST_VPM_VP_NOT_FOUND";
        case NCP_ST_VPM_INVALID_FLOW_DATA:
            return "NCP_ST_VPM_INVALID_FLOW_DATA";
        case NCP_ST_VPM_INVALID_FLOW_ID:
            return "NCP_ST_VPM_INVALID_FLOW_ID";
        case NCP_ST_VPM_FLOW_DATA_NOT_DEFINED:
            return "NCP_ST_VPM_FLOW_DATA_NOT_DEFINED";
        case NCP_ST_VPM_ENGINE_SEQUENCE_NOT_FOUND:
            return "NCP_ST_VPM_ENGINE_SEQUENCE_NOT_FOUND";
        case NCP_ST_VPM_ENGINE_NOT_FOUND:
            return "NCP_ST_VPM_ENGINE_NOT_FOUND";
        case NCP_ST_VPM_INVALID_LAUNCH_ENGINE:
            return "NCP_ST_VPM_INVALID_LAUNCH_ENGINE";
        case NCP_ST_VPM_LAUNCH_ENGINE_MISMATCH:
            return "NCP_ST_VPM_LAUNCH_ENGINE_MISMATCH";
        case NCP_ST_VPM_OUTPUT_FORMAT_INVALID:
            return "NCP_ST_VPM_OUTPUT_FORMAT_INVALID";
        case NCP_ST_VPM_INVALID_ENGINE:
            return "NCP_ST_VPM_INVALID_ENGINE";
        case NCP_ST_VPM_ENGINE_SEQUENCE_NOT_CPU_TERMINATING:
            return "NCP_ST_VPM_ENGINE_SEQUENCE_NOT_CPU_TERMINATING";
        case NCP_ST_VPM_VP_NOT_CPU_LAUNCH_ENGINE:
            return "NCP_ST_VPM_VP_NOT_CPU_LAUNCH_ENGINE";
        case NCP_ST_VPM_INVALID_FLOW_PARAM_TYPE:
            return "NCP_ST_VPM_INVALID_FLOW_PARAM_TYPE";
        case NCP_ST_VPM_INPUT_FORMAT_INVALID:
            return "NCP_ST_VPM_INPUT_FORMAT_INVALID";
        case NCP_ST_I2C_INVALID_HANDLE:
            return "NCP_ST_I2C_INVALID_HANDLE";
        case NCP_ST_I2C_INVALID_ARGUMENT:
            return "NCP_ST_I2C_INVALID_ARGUMENT";
        case NCP_ST_I2C_INVALID_NODE_ID:
            return "NCP_ST_I2C_INVALID_NODE_ID";
        case NCP_ST_I2C_ERROR_SEMAPHORE:
            return "NCP_ST_I2C_ERROR_SEMAPHORE";
        case NCP_ST_I2C_ERROR_MAST_XMT_CFG:
            return "NCP_ST_I2C_ERROR_MAST_XMT_CFG";
        case NCP_ST_I2C_ERROR_MAST_XMT_STAT:
            return "NCP_ST_I2C_ERROR_MAST_XMT_STAT";
        case NCP_ST_I2C_ERROR_MAST_RCV_CFG:
            return "NCP_ST_I2C_ERROR_MAST_RCV_CFG";
        case NCP_ST_I2C_ERROR_MAST_RCV_STAT:
            return "NCP_ST_I2C_ERROR_MAST_RCV_STAT";
        case NCP_ST_I2C_ERROR_READ_INCOMPLETE:
            return "NCP_ST_I2C_ERROR_READ_INCOMPLETE";
        case NCP_ST_I2C_ERROR_MASTER_STATUS:
            return "NCP_ST_I2C_ERROR_MASTER_STATUS";
        case NCP_ST_I2C_ERROR_TIMEOUT:
            return "NCP_ST_I2C_ERROR_TIMEOUT";
        case NCP_ST_I2C_ERROR_KERNEL_TRANSFER:
            return "NCP_ST_I2C_ERROR_KERNEL_TRANSFER";
        case NCP_ST_RDEV_RECV_FAILED:
            return "NCP_ST_RDEV_RECV_FAILED";
        case NCP_ST_RDEV_RECV_TIMEOUT:
            return "NCP_ST_RDEV_RECV_TIMEOUT";
        case NCP_ST_RDEV_RECV_SELECT_ERROR:
            return "NCP_ST_RDEV_RECV_SELECT_ERROR";
        case NCP_ST_RDEV_RECV_WRONG_SESSION:
            return "NCP_ST_RDEV_RECV_WRONG_SESSION";
        case NCP_ST_RDEV_RECV_WRONG_CONN:
            return "NCP_ST_RDEV_RECV_WRONG_CONN";
        case NCP_ST_RDEV_RECV_WRONG_MSG_TYPE:
            return "NCP_ST_RDEV_RECV_WRONG_MSG_TYPE";
        case NCP_ST_RDEV_RECV_SEQUENCE_ERROR:
            return "NCP_ST_RDEV_RECV_SEQUENCE_ERROR";
        case NCP_ST_RDEV_SERVER_DEV_OPEN_FAILED:
            return "NCP_ST_RDEV_SERVER_DEV_OPEN_FAILED";
        case NCP_ST_RDEV_SERVER_NOT_RESPONDING:
            return "NCP_ST_RDEV_SERVER_NOT_RESPONDING";
        case NCP_ST_RDEV_SERVER_NO_SESSIONS:
            return "NCP_ST_RDEV_SERVER_NO_SESSIONS";
        case NCP_ST_RDEV_SERVER_NO_MEMORY:
            return "NCP_ST_RDEV_SERVER_NO_MEMORY";
        case NCP_ST_RDEV_SERVER_THREAD_CREATE_FAILED:
            return "NCP_ST_RDEV_SERVER_THREAD_CREATE_FAILED";
        case NCP_ST_RDEV_SERVER_CONN_WAITING_FOR_EVENT:
            return "NCP_ST_RDEV_SERVER_CONN_WAITING_FOR_EVENT";
        case NCP_ST_RDEV_SERVER_NO_CONN_FOR_EVENT:
            return "NCP_ST_RDEV_SERVER_NO_CONN_FOR_EVENT";
        case NCP_ST_RDEV_SERVER_INVALID_MESSAGE:
            return "NCP_ST_RDEV_SERVER_INVALID_MESSAGE";
        case NCP_ST_RDEV_SERVER_API_NOT_IMPLEMENTED:
            return "NCP_ST_RDEV_SERVER_API_NOT_IMPLEMENTED";
        case NCP_ST_RDEV_SERVER_PREVIOUS_NON_ACK_WRITE_FAILED:
            return "NCP_ST_RDEV_SERVER_PREVIOUS_NON_ACK_WRITE_FAILED";
        case NCP_ST_RDEV_INVALID_DESCRIPTOR:
            return "NCP_ST_RDEV_INVALID_DESCRIPTOR";
        case NCP_ST_RDEV_INVALID_ENDPOINT:
            return "NCP_ST_RDEV_INVALID_ENDPOINT";
        case NCP_ST_RDEV_ADDR_IN_USE:
            return "NCP_ST_RDEV_ADDR_IN_USE";
        case NCP_ST_RDEV_THREAD_CREATE_FAILED:
            return "NCP_ST_RDEV_THREAD_CREATE_FAILED";
        case NCP_ST_RDEV_THREAD_DETACH_FAILED:
            return "NCP_ST_RDEV_THREAD_DETACH_FAILED";
        case NCP_ST_RDEV_SERVER_ADDR_NOT_FOUND:
            return "NCP_ST_RDEV_SERVER_ADDR_NOT_FOUND";
        case NCP_ST_RDEV_MAPPING_FILE_NOT_FOUND:
            return "NCP_ST_RDEV_MAPPING_FILE_NOT_FOUND";
        case NCP_ST_RDEV_INVALID_HANDLE:
            return "NCP_ST_RDEV_INVALID_HANDLE";
        case NCP_ST_RDEV_CONN_NOT_EXIST:
            return "NCP_ST_RDEV_CONN_NOT_EXIST";
        case NCP_ST_RDEV_CLIENT_HAS_NO_CONNECTIONS:
            return "NCP_ST_RDEV_CLIENT_HAS_NO_CONNECTIONS";
        case NCP_ST_RDEV_MSG_FRAG_SEQUENCE_ERR:
            return "NCP_ST_RDEV_MSG_FRAG_SEQUENCE_ERR";
        case NCP_ST_RDEV_MSG_SIZE_ERR:
            return "NCP_ST_RDEV_MSG_SIZE_ERR";
        case NCP_ST_RDEV_MSG_LENGTH_EXCEEDS_MAX:
            return "NCP_ST_RDEV_MSG_LENGTH_EXCEEDS_MAX";
        case NCP_ST_RDEV_MSG_EXCEEDS_MAX_PACKET_SIZE:
            return "NCP_ST_RDEV_MSG_EXCEEDS_MAX_PACKET_SIZE";
        case NCP_ST_RDEV_MSG_SEND_FAILED:
            return "NCP_ST_RDEV_MSG_SEND_FAILED";
        case NCP_ST_RDEV_MSG_RECV_FAILED:
            return "NCP_ST_RDEV_MSG_RECV_FAILED";
        case NCP_ST_RDEV_MSG_RECV_NOMSG:
            return "NCP_ST_RDEV_MSG_RECV_NOMSG";
        case NCP_ST_RDEV_SOCKET_ERROR:
            return "NCP_ST_RDEV_SOCKET_ERROR";
        case NCP_ST_RDEV_INPUT_TASK_GET_UNDEFINED:
            return "NCP_ST_RDEV_INPUT_TASK_GET_UNDEFINED";
        case NCP_ST_MPP_FBI_INIT_ERROR:
            return "NCP_ST_MPP_FBI_INIT_ERROR";
        case NCP_ST_MPP_HE_INIT_ERROR:
            return "NCP_ST_MPP_HE_INIT_ERROR";
        case NCP_ST_MPP_SEM_INIT_ERROR:
            return "NCP_ST_MPP_SEM_INIT_ERROR";
        case NCP_ST_MPP_PQM_INIT_ERROR:
            return "NCP_ST_MPP_PQM_INIT_ERROR";
        case NCP_ST_MPP_HE_NS_INIT_ERROR:
            return "NCP_ST_MPP_HE_NS_INIT_ERROR";
        case NCP_ST_MPP_ETAC_SW_ADDR_ERROR:
            return "NCP_ST_MPP_ETAC_SW_ADDR_ERROR";
        case NCP_ST_MPP_ETAC_BW_ADDR_ERROR:
            return "NCP_ST_MPP_ETAC_BW_ADDR_ERROR";
        case NCP_ST_MPP_ETAC_FW_ADDR_ERROR:
            return "NCP_ST_MPP_ETAC_FW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_INIT_ERROR:
            return "NCP_ST_TREE_PROG_INIT_ERROR";
        case NCP_ST_TREE_ROOT_INIT_ERROR:
            return "NCP_ST_TREE_ROOT_INIT_ERROR";
        case NCP_ST_TREE_PROG_HIGHBW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_HIGHBW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_MEDBW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_MEDBW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_LOWBW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_LOWBW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_HIGHFW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_HIGHFW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_MEDFW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_MEDFW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_LOWFW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_LOWFW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_HIGHSW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_HIGHSW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_MEDSW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_MEDSW_ADDR_ERROR";
        case NCP_ST_TREE_PROG_LOWSW_ADDR_ERROR:
            return "NCP_ST_TREE_PROG_LOWSW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_HIGHBW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_HIGHBW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_MEDBW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_MEDBW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_LOWBW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_LOWBW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_HIGHFW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_HIGHFW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_MEDFW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_MEDFW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_LOWFW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_LOWFW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_HIGHSW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_HIGHSW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_MEDSW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_MEDSW_ADDR_ERROR";
        case NCP_ST_TREE_ROOT_LOWSW_ADDR_ERROR:
            return "NCP_ST_TREE_ROOT_LOWSW_ADDR_ERROR";
        case NCP_ST_DPI_HFA_INVALID_HDL:
            return "NCP_ST_DPI_HFA_INVALID_HDL";
        case NCP_ST_DPI_HFA_INVALID_RST_MISSING_OYSTER_OR_V2RST:
            return "NCP_ST_DPI_HFA_INVALID_RST_MISSING_OYSTER_OR_V2RST";
        case NCP_ST_DPI_HFA_INVALID_RST_NUMBER_OF_SECTIONS_OOB:
            return "NCP_ST_DPI_HFA_INVALID_RST_NUMBER_OF_SECTIONS_OOB";
        case NCP_ST_DPI_INVALID_COMMAND:
            return "NCP_ST_DPI_INVALID_COMMAND";
        case NCP_ST_DPI_HFA_INVALID_RST_SECTION_TYPE:
            return "NCP_ST_DPI_HFA_INVALID_RST_SECTION_TYPE";
        case NCP_ST_DPI_INITIALIZE_ERROR:
            return "NCP_ST_DPI_INITIALIZE_ERROR";
        case NCP_ST_DPI_OUTPUT_OVERFLOW:
            return "NCP_ST_DPI_OUTPUT_OVERFLOW";
        case NCP_ST_DPI_SC_STACK_OVERFLOW:
            return "NCP_ST_DPI_SC_STACK_OVERFLOW";
        case NCP_ST_DPI_SC_STACK_UNDERFLOW:
            return "NCP_ST_DPI_SC_STACK_UNDERFLOW";
        case NCP_ST_DPI_JOB_TERMINATE:
            return "NCP_ST_DPI_JOB_TERMINATE";
        case NCP_ST_DPI_INVALID_INSTR:
            return "NCP_ST_DPI_INVALID_INSTR";
        case NCP_ST_DPI_BACKUP_ERROR:
            return "NCP_ST_DPI_BACKUP_ERROR";
        case NCP_ST_DPI_UNCORRECTED_PARITY_ERROR:
            return "NCP_ST_DPI_UNCORRECTED_PARITY_ERROR";
        case NCP_ST_DPI_LCB_ALLOCATION_ERROR:
            return "NCP_ST_DPI_LCB_ALLOCATION_ERROR";
        case NCP_ST_DPI_LCB_ALLOCATED:
            return "NCP_ST_DPI_LCB_ALLOCATED";
        case NCP_ST_DPI_INVALID_PARAMETER:
            return "NCP_ST_DPI_INVALID_PARAMETER";
        case NCP_ST_DPI_THREAD_INIT_ERROR:
            return "NCP_ST_DPI_THREAD_INIT_ERROR";
        case NCP_ST_DPI_MAX_THREADS_REACHED:
            return "NCP_ST_DPI_MAX_THREADS_REACHED";
        case NCP_ST_DPI_INVALID_THREAD_ID:
            return "NCP_ST_DPI_INVALID_THREAD_ID";
        case NCP_ST_DPI_INVALID_STREAM_HANDLE:
            return "NCP_ST_DPI_INVALID_STREAM_HANDLE";
        case NCP_ST_DPI_RULESET_LOAD_ERROR:
            return "NCP_ST_DPI_RULESET_LOAD_ERROR";
        case NCP_ST_DPI_SCAN_ERROR:
            return "NCP_ST_DPI_SCAN_ERROR";
        case NCP_ST_DPI_INVALID_FILE_FORMAT:
            return "NCP_ST_DPI_INVALID_FILE_FORMAT";
        case NCP_ST_DPI_INVALID_RULESET_BUFFER_FORMAT:
            return "NCP_ST_DPI_INVALID_RULESET_BUFFER_FORMAT";
        case NCP_ST_DPI_SCAN_TIMEOUT_ERROR:
            return "NCP_ST_DPI_SCAN_TIMEOUT_ERROR";
        case NCP_ST_DPI_SCAN_MODE_MISMATCH:
            return "NCP_ST_DPI_SCAN_MODE_MISMATCH";
        case NCP_ST_DPI_QUEUE_IS_FULL:
            return "NCP_ST_DPI_QUEUE_IS_FULL";
        case NCP_ST_DPI_BUILD_HW_TABLE_ERROR:
            return "NCP_ST_DPI_BUILD_HW_TABLE_ERROR";
        case NCP_ST_DPI_INSUFFICIENT_RESOURCES_ERROR:
            return "NCP_ST_DPI_INSUFFICIENT_RESOURCES_ERROR";
        case NCP_ST_DPI_LIBRARY_NOT_INITIALIZED:
            return "NCP_ST_DPI_LIBRARY_NOT_INITIALIZED";
        case NCP_ST_DPI_LIBRARY_ALREADY_INITIALIZED:
            return "NCP_ST_DPI_LIBRARY_ALREADY_INITIALIZED";
        case NCP_ST_DPI_NO_RULESET_LOADED:
            return "NCP_ST_DPI_NO_RULESET_LOADED";
        case NCP_ST_DPI_XLATE_OVERFLOW:
            return "NCP_ST_DPI_XLATE_OVERFLOW";
        case NCP_ST_DPI_FETCHED_TASK_BUSY:
            return "NCP_ST_DPI_FETCHED_TASK_BUSY";
        case NCP_ST_DPI_INVALID_HDL:
            return "NCP_ST_DPI_INVALID_HDL";
        case NCP_ST_DPI_INVALID_JOB_ID:
            return "NCP_ST_DPI_INVALID_JOB_ID";
        case NCP_ST_DPI_INVALID_BLUT_SECTION:
            return "NCP_ST_DPI_INVALID_BLUT_SECTION";
        case NCP_ST_DPI_INVALID_SMALL_CONTEXT_MAX:
            return "NCP_ST_DPI_INVALID_SMALL_CONTEXT_MAX";
        case NCP_ST_DPI_HFA_INVALID_RST_SRAM_OR_BLUT_MISSING:
            return "NCP_ST_DPI_HFA_INVALID_RST_SRAM_OR_BLUT_MISSING";
        case NCP_ST_DPI_HFA_DMA_BURST_NOT_ENABLED:
            return "NCP_ST_DPI_HFA_DMA_BURST_NOT_ENABLED";
        case NCP_ST_TASK_LITE_UBOOT_NO_ALLOCATOR_REGISTERED:
            return "NCP_ST_TASK_LITE_UBOOT_NO_ALLOCATOR_REGISTERED";
        case NCP_ST_TASK_LITE_NULL_SEND_DONE_FUNCTION:
            return "NCP_ST_TASK_LITE_NULL_SEND_DONE_FUNCTION";
        case NCP_ST_TASK_LITE_QS_NOT_IN_USE:
            return "NCP_ST_TASK_LITE_QS_NOT_IN_USE";
        case NCP_ST_TASK_LITE_QS_NOT_AVAILABLE:
            return "NCP_ST_TASK_LITE_QS_NOT_AVAILABLE";
        case NCP_ST_TASK_TDF_ZEROCOPY_AND_GATHERMODE_INVALID:
            return "NCP_ST_TASK_TDF_ZEROCOPY_AND_GATHERMODE_INVALID";
        case NCP_ST_TASK_SKB_COOKIE_VALIDATION_FAILED:
            return "NCP_ST_TASK_SKB_COOKIE_VALIDATION_FAILED";
        case NCP_ST_TASK_NO_SKB_MEM:
            return "NCP_ST_TASK_NO_SKB_MEM";
        case NCP_ST_TASK_NO_RXBUFF_MEM:
            return "NCP_ST_TASK_NO_RXBUFF_MEM";
        case NCP_ST_TASK_GSM_ALLOC_NOT_ALLOWED:
            return "NCP_ST_TASK_GSM_ALLOC_NOT_ALLOWED";
        case NCP_ST_TASK_MMAP_FAILED:
            return "NCP_ST_TASK_MMAP_FAILED";
        case NCP_ST_TASK_QUESET_NOT_IN_LOCAL_DOMAIN:
            return "NCP_ST_TASK_QUESET_NOT_IN_LOCAL_DOMAIN";
        case NCP_ST_TASK_NO_MAN_COMPLETION_OWED:
            return "NCP_ST_TASK_NO_MAN_COMPLETION_OWED";
        case NCP_ST_TASK_NULL_MAN_COMPLETION_LIST:
            return "NCP_ST_TASK_NULL_MAN_COMPLETION_LIST";
        case NCP_ST_TASK_RECEIVE_QUEUES_MUST_HAVE_SAME_SHARE_PERMISSION:
            return "NCP_ST_TASK_RECEIVE_QUEUES_MUST_HAVE_SAME_SHARE_PERMISSION";
        case NCP_ST_TASK_EXCLUSIVE_BINDING_NOT_AVAIL:
            return "NCP_ST_TASK_EXCLUSIVE_BINDING_NOT_AVAIL";
        case NCP_ST_TASK_SHARED_BINDING_NOT_AVAIL:
            return "NCP_ST_TASK_SHARED_BINDING_NOT_AVAIL";
        case NCP_ST_TASK_ORDERED_OUTPUT_MISMATCH:
            return "NCP_ST_TASK_ORDERED_OUTPUT_MISMATCH";
        case NCP_ST_TASK_SHARED_PIPELINES_NOT_SUPPORTED:
            return "NCP_ST_TASK_SHARED_PIPELINES_NOT_SUPPORTED";
        case NCP_ST_TASK_SHARED_RECEIVE_QUEUES_NOT_SUPPORTED:
            return "NCP_ST_TASK_SHARED_RECEIVE_QUEUES_NOT_SUPPORTED";
        case NCP_ST_TASK_EXCLUSIVE_VP_BINDING_NOT_AVAIL:
            return "NCP_ST_TASK_EXCLUSIVE_VP_BINDING_NOT_AVAIL";
        case NCP_ST_TASK_INVALID_NCA_INPUT_QUEUE:
            return "NCP_ST_TASK_INVALID_NCA_INPUT_QUEUE";
        case NCP_ST_TASK_INVALID_NCA_OUTPUT_QUEUE:
            return "NCP_ST_TASK_INVALID_NCA_OUTPUT_QUEUE";
        case NCP_ST_TASK_INVALID_RECEIVE_QUEUE:
            return "NCP_ST_TASK_INVALID_RECEIVE_QUEUE";
        case NCP_ST_TASK_SEND_QUEUE_FULL:
            return "NCP_ST_TASK_SEND_QUEUE_FULL";
        case NCP_ST_TASK_RECV_QUEUE_EMPTY:
            return "NCP_ST_TASK_RECV_QUEUE_EMPTY";
        case NCP_ST_TASK_INVALID_TASK_HANDLE:
            return "NCP_ST_TASK_INVALID_TASK_HANDLE";
        case NCP_ST_TASK_INVALID_VP_HANDLE:
            return "NCP_ST_TASK_INVALID_VP_HANDLE";
        case NCP_ST_TASK_INVALID_BIND_TABLE_ENTRY:
            return "NCP_ST_TASK_INVALID_BIND_TABLE_ENTRY";
        case NCP_ST_TASK_NO_FREE_DYNAMIC_QUEUE:
            return "NCP_ST_TASK_NO_FREE_DYNAMIC_QUEUE";
        case NCP_ST_TASK_NULL_TASK_IN_RECVDONE:
            return "NCP_ST_TASK_NULL_TASK_IN_RECVDONE";
        case NCP_ST_TASK_RX_TOO_BIG:
            return "NCP_ST_TASK_RX_TOO_BIG";
        case NCP_ST_TASK_BUFFER_POOL_NOT_FOUND:
            return "NCP_ST_TASK_BUFFER_POOL_NOT_FOUND";
        case NCP_ST_TASK_REQUESTED_BUFF_POOL_TOO_BIG:
            return "NCP_ST_TASK_REQUESTED_BUFF_POOL_TOO_BIG";
        case NCP_ST_TASK_QMAP_ERR_TOO_BIG:
            return "NCP_ST_TASK_QMAP_ERR_TOO_BIG";
        case NCP_ST_TASK_BAD_QTYPE_FOR_STATUS_READ:
            return "NCP_ST_TASK_BAD_QTYPE_FOR_STATUS_READ";
        case NCP_ST_TASK_PUT_NULL_TASK_COMPLETION_QUEUE:
            return "NCP_ST_TASK_PUT_NULL_TASK_COMPLETION_QUEUE";
        case NCP_ST_TASK_PUT_NULL_TASK_OUTPUT_QUEUE:
            return "NCP_ST_TASK_PUT_NULL_TASK_OUTPUT_QUEUE";
        case NCP_ST_TASK_INVALID_FIXED_QUEUEID:
            return "NCP_ST_TASK_INVALID_FIXED_QUEUEID";
        case NCP_ST_TASK_NULL_RECVQUEUEID_PTR:
            return "NCP_ST_TASK_NULL_RECVQUEUEID_PTR";
        case NCP_ST_TASK_HANDLE_HEAD_NULL:
            return "NCP_ST_TASK_HANDLE_HEAD_NULL";
        case NCP_ST_TASK_SEND_NULL_METADATA_PTR:
            return "NCP_ST_TASK_SEND_NULL_METADATA_PTR";
        case NCP_ST_TASK_SEND_NULL_TASK_PTR:
            return "NCP_ST_TASK_SEND_NULL_TASK_PTR";
        case NCP_ST_TASK_NO_GSM:
            return "NCP_ST_TASK_NO_GSM";
        case NCP_ST_TASK_BUFFSIZE_TOO_LARGE:
            return "NCP_ST_TASK_BUFFSIZE_TOO_LARGE";
        case NCP_ST_TASK_CLIENT_FREE_NULLPTR:
            return "NCP_ST_TASK_CLIENT_FREE_NULLPTR";
        case NCP_ST_TASK_CFG_TOO_MANY_TASK_MEMPOOLS:
            return "NCP_ST_TASK_CFG_TOO_MANY_TASK_MEMPOOLS";
        case NCP_ST_TASK_CFG_TOO_MANY_RX_MEMPOOLS:
            return "NCP_ST_TASK_CFG_TOO_MANY_RX_MEMPOOLS";
        case NCP_ST_TASK_NO_RXPOOL_DEFINED:
            return "NCP_ST_TASK_NO_RXPOOL_DEFINED";
        case NCP_ST_TASK_NO_WARM_RESTART_VP:
            return "NCP_ST_TASK_NO_WARM_RESTART_VP";
        case NCP_ST_TASK_NO_IPCQ_FOR_RXB_POOL_IN_WARM_RESTART:
            return "NCP_ST_TASK_NO_IPCQ_FOR_RXB_POOL_IN_WARM_RESTART";
        case NCP_ST_TASK_CFG_TOO_MANY_RXB_POOLS:
            return "NCP_ST_TASK_CFG_TOO_MANY_RXB_POOLS";
        case NCP_ST_TASK_CFG_TOO_MANY_QUEUE_GROUPS:
            return "NCP_ST_TASK_CFG_TOO_MANY_QUEUE_GROUPS";
        case NCP_ST_TASK_RECV_QUEUE_NOT_IN_QUEUESET:
            return "NCP_ST_TASK_RECV_QUEUE_NOT_IN_QUEUESET";
        case NCP_ST_TASK_RECV_ACCESS_MODE_NOT_SAME_AS_QUEUESET:
            return "NCP_ST_TASK_RECV_ACCESS_MODE_NOT_SAME_AS_QUEUESET";
        case NCP_ST_TASK_CFG_TOO_MANY_THREAD_QUEUESETS_PER_GROUP:
            return "NCP_ST_TASK_CFG_TOO_MANY_THREAD_QUEUESETS_PER_GROUP";
        case NCP_ST_TASK_BUFFSZ_ORDER_ERR:
            return "NCP_ST_TASK_BUFFSZ_ORDER_ERR";
        case NCP_ST_TASK_BUFFSZ_OUT_OF_BOUNDS:
            return "NCP_ST_TASK_BUFFSZ_OUT_OF_BOUNDS";
        case NCP_ST_TASK_NO_CACHE_ALLLIGNED_MEM:
            return "NCP_ST_TASK_NO_CACHE_ALLLIGNED_MEM";
        case NCP_ST_TASK_NO_MALLOC_MEM:
            return "NCP_ST_TASK_NO_MALLOC_MEM";
        case NCP_ST_TASK_NULL_RECV_QUEUE_PTR:
            return "NCP_ST_TASK_NULL_RECV_QUEUE_PTR";
        case NCP_ST_TASK_RECV_RET_SUCCESS_WITH_NULL_PTR:
            return "NCP_ST_TASK_RECV_RET_SUCCESS_WITH_NULL_PTR";
        case NCP_ST_TASK_NULL_PCQ_PRODUCER_PTR:
            return "NCP_ST_TASK_NULL_PCQ_PRODUCER_PTR";
        case NCP_ST_TASK_NULL_PCQ_CONSUMER_PTR:
            return "NCP_ST_TASK_NULL_PCQ_CONSUMER_PTR";
        case NCP_ST_TASK_NO_QUEUESET_IN_DOMAIN:
            return "NCP_ST_TASK_NO_QUEUESET_IN_DOMAIN";
        case NCP_ST_TASK_NO_QUEUESET_IN_GRP:
            return "NCP_ST_TASK_NO_QUEUESET_IN_GRP";
        case NCP_ST_TASK_NO_TASKMEMPOOL_DEFINED:
            return "NCP_ST_TASK_NO_TASKMEMPOOL_DEFINED";
        case NCP_ST_TASK_NO_RXB_POOL_DEFINED:
            return "NCP_ST_TASK_NO_RXB_POOL_DEFINED";
        case NCP_ST_TASK_BUFFPOOL_CORRUPTION:
            return "NCP_ST_TASK_BUFFPOOL_CORRUPTION";
        case NCP_ST_TASK_POOLALLOC_RET_NULL:
            return "NCP_ST_TASK_POOLALLOC_RET_NULL";
        case NCP_ST_TASK_POOLALLOC_RET_PHYSADDR_NULL:
            return "NCP_ST_TASK_POOLALLOC_RET_PHYSADDR_NULL";
        case NCP_ST_TASK_CORRUPT_NCA_INPUT_QUEUE:
            return "NCP_ST_TASK_CORRUPT_NCA_INPUT_QUEUE";
        case NCP_ST_TASK_REQUEST_FOR_ZERO_LENGTH_TASK_BUFFER:
            return "NCP_ST_TASK_REQUEST_FOR_ZERO_LENGTH_TASK_BUFFER";
        case NCP_ST_TASK_ALLOC_REQUIRES_POINTER:
            return "NCP_ST_TASK_ALLOC_REQUIRES_POINTER";
        case NCP_ST_TASK_DISPATCHED:
            return "NCP_ST_TASK_DISPATCHED";
        case NCP_ST_TASK_REQ_FOR_ZERO_LEN_CACHE_ALIGNED_MEM:
            return "NCP_ST_TASK_REQ_FOR_ZERO_LEN_CACHE_ALIGNED_MEM";
        case NCP_ST_TASK_NCP_DESTROY_HANDLE_INVALID:
            return "NCP_ST_TASK_NCP_DESTROY_HANDLE_INVALID";
        case NCP_ST_TASK_NO_SWQUEUE_MEM:
            return "NCP_ST_TASK_NO_SWQUEUE_MEM";
        case NCP_ST_TASK_HDL_CREATE_ERR:
            return "NCP_ST_TASK_HDL_CREATE_ERR";
        case NCP_ST_TASK_TLS_RETRIEVAL_ERR:
            return "NCP_ST_TASK_TLS_RETRIEVAL_ERR";
        case NCP_ST_TASK_ALLOC_SKB_FAILURE:
            return "NCP_ST_TASK_ALLOC_SKB_FAILURE";
        case NCP_ST_TASK_BAD_GROUP_FOR_IRQ:
            return "NCP_ST_TASK_BAD_GROUP_FOR_IRQ";
        case NCP_ST_TASK_NULL_L1_BINDP:
            return "NCP_ST_TASK_NULL_L1_BINDP";
        case NCP_ST_TASK_NO_MEM_L1BIND:
            return "NCP_ST_TASK_NO_MEM_L1BIND";
        case NCP_ST_TASK_NO_MEM_L2BIND:
            return "NCP_ST_TASK_NO_MEM_L2BIND";
        case NCP_ST_TASK_NO_BUFF_DESC:
            return "NCP_ST_TASK_NO_BUFF_DESC";
        case NCP_ST_INVALID_BUFF_FOR_TASK_FREE:
            return "NCP_ST_INVALID_BUFF_FOR_TASK_FREE";
        case NCP_ST_TASK_SEND_SEGMENTS_INVALID_DESC:
            return "NCP_ST_TASK_SEND_SEGMENTS_INVALID_DESC";
        case NCP_ST_TASK_SEND_SEGMENTS_INVALID_NUM_SEGS:
            return "NCP_ST_TASK_SEND_SEGMENTS_INVALID_NUM_SEGS";
        case NCP_ST_TASK_UNEXPECTED_QUEUE_TYPE:
            return "NCP_ST_TASK_UNEXPECTED_QUEUE_TYPE";
        case NCP_ST_TASK_SHUTDOWN:
            return "NCP_ST_TASK_SHUTDOWN";
        case NCP_ST_TASK_MISMATCH:
            return "NCP_ST_TASK_MISMATCH";
        case NCP_ST_TASK_NO_LCM_MEMORY:
            return "NCP_ST_TASK_NO_LCM_MEMORY";
        case NCP_ST_TASK_NO_GSM_MEMORY:
            return "NCP_ST_TASK_NO_GSM_MEMORY";
        case NCP_ST_TASK_NO_TASK_CONTROL_MEMORY:
            return "NCP_ST_TASK_NO_TASK_CONTROL_MEMORY";
        case NCP_ST_TASK_NO_THREADQUEUESET_TDF_SECTION:
            return "NCP_ST_TASK_NO_THREADQUEUESET_TDF_SECTION";
        case NCP_ST_TASK_NO_TASKSEND_TDF_SECTION:
            return "NCP_ST_TASK_NO_TASKSEND_TDF_SECTION";
        case NCP_ST_TASK_NO_TASKRECV_TDF_SECTION:
            return "NCP_ST_TASK_NO_TASKRECV_TDF_SECTION";
        case NCP_ST_TASK_NO_TASKRECVQUEUEBIND_TDF_SECTION:
            return "NCP_ST_TASK_NO_TASKRECVQUEUEBIND_TDF_SECTION";
        case NCP_ST_TASK_NO_TASKRECVQUEUEVPBIND_TDF_SECTION:
            return "NCP_ST_TASK_NO_TASKRECVQUEUEVPBIND_TDF_SECTION";
        case NCP_ST_TASK_RECEIVE_QUEUE_NOT_FOUND:
            return "NCP_ST_TASK_RECEIVE_QUEUE_NOT_FOUND";
        case NCP_ST_TASK_RETRY:
            return "NCP_ST_TASK_RETRY";
        case NCP_ST_TASK_IOCTL_FAILED:
            return "NCP_ST_TASK_IOCTL_FAILED";
        case NCP_ST_TASK_QUEUE_STUCK_IN_WARM_RESTART:
            return "NCP_ST_TASK_QUEUE_STUCK_IN_WARM_RESTART";
        case NCP_ST_TASK_WARM_RESTART_UNSUPPORTED_PLATFORM:
            return "NCP_ST_TASK_WARM_RESTART_UNSUPPORTED_PLATFORM";
        case NCP_ST_TASK_SENDDONE_FN_WITH_VP_MANAGED_POOL:
            return "NCP_ST_TASK_SENDDONE_FN_WITH_VP_MANAGED_POOL";
        case NCP_ST_TASK_NO_TQS_USES_POOL:
            return "NCP_ST_TASK_NO_TQS_USES_POOL";
        case NCP_ST_TASK_INVALID_NCA_TASK_COMPLETION_QUEUE:
            return "NCP_ST_TASK_INVALID_NCA_TASK_COMPLETION_QUEUE";
        case NCP_ST_TASK_TASK_COMPLETION_QUEUE_FULL:
            return "NCP_ST_TASK_TASK_COMPLETION_QUEUE_FULL";
        case NCP_ST_TASK_STATS_DISABLED:
            return "NCP_ST_TASK_STATS_DISABLED";
        case NCP_ST_TASK_MISSING_NCADOMAININFO_SECTION:
            return "NCP_ST_TASK_MISSING_NCADOMAININFO_SECTION";
        case NCP_ST_TASK_NULL_TASK:
            return "NCP_ST_TASK_NULL_TASK";
        case NCP_ST_TASK_NO_MME_ALLOCATOR:
            return "NCP_ST_TASK_NO_MME_ALLOCATOR";
        case NCP_ST_TASK_BUFF_POOL_OVERRUN:
            return "NCP_ST_TASK_BUFF_POOL_OVERRUN";
        case NCP_ST_TASK_CPU_POOL_IN_USE:
            return "NCP_ST_TASK_CPU_POOL_IN_USE";
        case NCP_ST_TASK_QUEUESET_NOT_SHARED:
            return "NCP_ST_TASK_QUEUESET_NOT_SHARED";
        case NCP_ST_TASK_ZERO_LEN_SEND_SEGMENT:
            return "NCP_ST_TASK_ZERO_LEN_SEND_SEGMENT";
        case NCP_ST_TASK_BAD_PTR_CNT:
            return "NCP_ST_TASK_BAD_PTR_CNT";
        case NCP_ST_TASK_BAD_SEND_META_DATA:
            return "NCP_ST_TASK_BAD_SEND_META_DATA";
        case NCP_ST_NCA_BP_DEST_BIT_INVALID:
            return "NCP_ST_NCA_BP_DEST_BIT_INVALID";
        case NCP_ST_NCA_BP_BUFFER_INVALID:
            return "NCP_ST_NCA_BP_BUFFER_INVALID";
        case NCP_ST_BP_INVALID_DEST_INDEX:
            return "NCP_ST_BP_INVALID_DEST_INDEX";
        case NCP_ST_BP_DEST_ID_ALREADY_SET:
            return "NCP_ST_BP_DEST_ID_ALREADY_SET";
        case NCP_ST_BP_DEST_ID_ALREADY_UNSET:
            return "NCP_ST_BP_DEST_ID_ALREADY_UNSET";
        case NCP_ST_BP_AGG_INVALID_INDEX:
            return "NCP_ST_BP_AGG_INVALID_INDEX";
        case NCP_ST_BP_INVALID_BUFFER:
            return "NCP_ST_BP_INVALID_BUFFER";
        case NCP_ST_BP_INVALID_BUFFER_SIZE:
            return "NCP_ST_BP_INVALID_BUFFER_SIZE";
        case NCP_ST_UNKNOWN_MEMORY_CONTROLLER:
            return "NCP_ST_UNKNOWN_MEMORY_CONTROLLER";
        case NCP_ST_SYSCACHE_INIT_TIMEOUT:
            return "NCP_ST_SYSCACHE_INIT_TIMEOUT";
        case NCP_ST_SYSCACHE_MEMINIT_TIMEOUT:
            return "NCP_ST_SYSCACHE_MEMINIT_TIMEOUT";
        case NCP_ST_SYSCACHE_INVALID_CONFIG:
            return "NCP_ST_SYSCACHE_INVALID_CONFIG";
        case NCP_ST_SYSCACHE_SYSMEM_TOO_SMALL:
            return "NCP_ST_SYSCACHE_SYSMEM_TOO_SMALL";
        case NCP_ST_TASK_BAD_TQS_NAME:
            return "NCP_ST_TASK_BAD_TQS_NAME";
        case NCP_ST_TASK_BAD_RXQ_NAME:
            return "NCP_ST_TASK_BAD_RXQ_NAME";
        case NCP_ST_TASK_RECVQUEUE_NAME_NOT_FOUND:
            return "NCP_ST_TASK_RECVQUEUE_NAME_NOT_FOUND";
        case NCP_ST_TASK_TQSNAME_NOT_FOUND:
            return "NCP_ST_TASK_TQSNAME_NOT_FOUND";
        case NCP_ST_TASK_TQSID_NOT_FOUND:
            return "NCP_ST_TASK_TQSID_NOT_FOUND";
        case NCP_ST_TASK_RECV_QUEUE_ID_NOT_FOUND:
            return "NCP_ST_TASK_RECV_QUEUE_ID_NOT_FOUND";
        case NCP_ST_TASK_SEND_DONE_RECURSION_DETECTED:
            return "NCP_ST_TASK_SEND_DONE_RECURSION_DETECTED";
        case NCP_ST_TASK_TQS_DISABLED:
            return "NCP_ST_TASK_TQS_DISABLED";
        case NCP_ST_TASK_UMODE_ONLY_API:
            return "NCP_ST_TASK_UMODE_ONLY_API";
        case NCP_ST_TASK_TQS_MODE_CONFLICT:
            return "NCP_ST_TASK_TQS_MODE_CONFLICT";
        case NCP_ST_TASK_UNEXPECTED_TASK_COMPLETION:
            return "NCP_ST_TASK_UNEXPECTED_TASK_COMPLETION";
        case NCP_ST_TASK_PCQ_GROUP_NOT_FOUND:
            return "NCP_ST_TASK_PCQ_GROUP_NOT_FOUND";
        case NCP_ST_TASK_LOGICAL_QUEUE_NOT_FOUND:
            return "NCP_ST_TASK_LOGICAL_QUEUE_NOT_FOUND";
        case NCP_ST_TASK_BAD_LOGICAL_QUEUE_NAME:
            return "NCP_ST_TASK_BAD_LOGICAL_QUEUE_NAME";
        case NCP_ST_TASK_LOGICAL_QUEUE_NAME_NOT_FOUND:
            return "NCP_ST_TASK_LOGICAL_QUEUE_NAME_NOT_FOUND";
        case NCP_ST_TASK_TBR_BUFFERS_OWNED_DURING_REMOVE:
            return "NCP_ST_TASK_TBR_BUFFERS_OWNED_DURING_REMOVE";
        case NCP_ST_TASK_WAIT_TIMEOUT:
            return "NCP_ST_TASK_WAIT_TIMEOUT";
        case NCP_ST_TASK_BAD_WFI_TARGET:
            return "NCP_ST_TASK_BAD_WFI_TARGET";
        case NCP_ST_TASK_INVALID_WFI_USE_CASE:
            return "NCP_ST_TASK_INVALID_WFI_USE_CASE";
        case NCP_ST_TASK_QUEUE_NOT_BOUND_TO_TASK_HANDLE:
            return "NCP_ST_TASK_QUEUE_NOT_BOUND_TO_TASK_HANDLE";
        case NCP_ST_TASK_SHARED_POOLS_NOT_SUPPORTED:
            return "NCP_ST_TASK_SHARED_POOLS_NOT_SUPPORTED";
        case NCP_ST_TASK_NO_TASK_HDL_USES_POOL:
            return "NCP_ST_TASK_NO_TASK_HDL_USES_POOL";
        case NCP_ST_TASK_POOL_IS_BEING_REINITIALIZED:
            return "NCP_ST_TASK_POOL_IS_BEING_REINITIALIZED";
        case NCP_ST_TASK_EXITING_WITH_COMPLETIONS_OWED:
            return "NCP_ST_TASK_EXITING_WITH_COMPLETIONS_OWED";
        case NCP_ST_TASK_PID_LOOKUP_FAILURE:
            return "NCP_ST_TASK_PID_LOOKUP_FAILURE";
        case NCP_ST_TASK_POOL_RESTART_NEEDED:
            return "NCP_ST_TASK_POOL_RESTART_NEEDED";
        case NCP_ST_TASK_NO_POOL_REINIT_REQUIRED:
            return "NCP_ST_TASK_NO_POOL_REINIT_REQUIRED";
        case NCP_ST_TASK_THREAD_ALREADY_WAITING:
            return "NCP_ST_TASK_THREAD_ALREADY_WAITING";
        case NCP_ST_SYSMEM_PHY_CPC_CAL_TIMEOUT:
            return "NCP_ST_SYSMEM_PHY_CPC_CAL_TIMEOUT";
        case NCP_ST_SYSMEM_PHY_TRAIN_TIMEOUT:
            return "NCP_ST_SYSMEM_PHY_TRAIN_TIMEOUT";
        case NCP_ST_SYSMEM_PHY_TRAIN_SW_TIMEOUT:
            return "NCP_ST_SYSMEM_PHY_TRAIN_SW_TIMEOUT";
        case NCP_ST_SYSMEM_PHY_WL_ERR:
            return "NCP_ST_SYSMEM_PHY_WL_ERR";
        case NCP_ST_SYSMEM_PHY_WL_DLY_ERR:
            return "NCP_ST_SYSMEM_PHY_WL_DLY_ERR";
        case NCP_ST_SYSMEM_PHY_ECC_WL_ERR:
            return "NCP_ST_SYSMEM_PHY_ECC_WL_ERR";
        case NCP_ST_SYSMEM_PHY_ECC_WL_DLY_ERR:
            return "NCP_ST_SYSMEM_PHY_ECC_WL_DLY_ERR";
        case NCP_ST_SYSMEM_PHY_RDFIFO_OPT_ERR:
            return "NCP_ST_SYSMEM_PHY_RDFIFO_OPT_ERR";
        case NCP_ST_SYSMEM_PHY_CPC_ERR:
            return "NCP_ST_SYSMEM_PHY_CPC_ERR";
        case NCP_ST_SYSMEM_PHY_RD_DLY_ERR:
            return "NCP_ST_SYSMEM_PHY_RD_DLY_ERR";
        case NCP_ST_SYSMEM_PHY_RD_CAL_ERR:
            return "NCP_ST_SYSMEM_PHY_RD_CAL_ERR";
        case NCP_ST_SYSMEM_PHY_WR_LVL_ERR:
            return "NCP_ST_SYSMEM_PHY_WR_LVL_ERR";
        case NCP_ST_SYSMEM_PHY_RD_LVL_ERR:
            return "NCP_ST_SYSMEM_PHY_RD_LVL_ERR";
        case NCP_ST_SYSMEM_PHY_GT_TRN_ERR:
            return "NCP_ST_SYSMEM_PHY_GT_TRN_ERR";
        case NCP_ST_SYSMEM_PHY_CMD_DLY_ERR:
            return "NCP_ST_SYSMEM_PHY_CMD_DLY_ERR";
        case NCP_ST_TREEMEM_PHY_CPC_CAL_TIMEOUT:
            return "NCP_ST_TREEMEM_PHY_CPC_CAL_TIMEOUT";
        case NCP_ST_TREEMEM_PHY_TRAIN_TIMEOUT:
            return "NCP_ST_TREEMEM_PHY_TRAIN_TIMEOUT";
        case NCP_ST_TREEMEM_PHY_TRAIN_SW_TIMEOUT:
            return "NCP_ST_TREEMEM_PHY_TRAIN_SW_TIMEOUT";
        case NCP_ST_TREEMEM_PHY_WL_ERR:
            return "NCP_ST_TREEMEM_PHY_WL_ERR";
        case NCP_ST_TREEMEM_PHY_WL_DLY_ERR:
            return "NCP_ST_TREEMEM_PHY_WL_DLY_ERR";
        case NCP_ST_TREEMEM_PHY_ECC_WL_ERR:
            return "NCP_ST_TREEMEM_PHY_ECC_WL_ERR";
        case NCP_ST_TREEMEM_PHY_ECC_WL_DLY_ERR:
            return "NCP_ST_TREEMEM_PHY_ECC_WL_DLY_ERR";
        case NCP_ST_TREEMEM_PHY_RDFIFO_OPT_ERR:
            return "NCP_ST_TREEMEM_PHY_RDFIFO_OPT_ERR";
        case NCP_ST_TREEMEM_PHY_CPC_ERR:
            return "NCP_ST_TREEMEM_PHY_CPC_ERR";
        case NCP_ST_TREEMEM_PHY_RD_DLY_ERR:
            return "NCP_ST_TREEMEM_PHY_RD_DLY_ERR";
        case NCP_ST_TREEMEM_PHY_RD_CAL_ERR:
            return "NCP_ST_TREEMEM_PHY_RD_CAL_ERR";
        case NCP_ST_TREEMEM_PHY_WR_LVL_ERR:
            return "NCP_ST_TREEMEM_PHY_WR_LVL_ERR";
        case NCP_ST_TREEMEM_PHY_RD_LVL_ERR:
            return "NCP_ST_TREEMEM_PHY_RD_LVL_ERR";
        case NCP_ST_TREEMEM_PHY_GT_TRN_ERR:
            return "NCP_ST_TREEMEM_PHY_GT_TRN_ERR";
        case NCP_ST_TREEMEM_PHY_CMD_DLY_ERR:
            return "NCP_ST_TREEMEM_PHY_CMD_DLY_ERR";
        case NCP_ST_TREEMEM_DDR_INIT_ERR:
            return "NCP_ST_TREEMEM_DDR_INIT_ERR";
        case NCP_ST_SYSMEM_INVALID_HANDLE:
            return "NCP_ST_SYSMEM_INVALID_HANDLE";
        case NCP_ST_SYSMEM_SEM_INIT_ERROR:
            return "NCP_ST_SYSMEM_SEM_INIT_ERROR";
        case NCP_ST_SYSMEM_INVALID_ID:
            return "NCP_ST_SYSMEM_INVALID_ID";
        case NCP_ST_SYSMEM_INVALID_TOPOLOGY:
            return "NCP_ST_SYSMEM_INVALID_TOPOLOGY";
        case NCP_ST_SYSMEM_LAYOUT_SIZE_EXCEEDS_PHYS_SIZE:
            return "NCP_ST_SYSMEM_LAYOUT_SIZE_EXCEEDS_PHYS_SIZE";
        case NCP_ST_SYSMEM_LAYOUT_OS_MEMORY_SIZE_MISMATCH:
            return "NCP_ST_SYSMEM_LAYOUT_OS_MEMORY_SIZE_MISMATCH";
        case NCP_ST_SYSMEM_LAYOUT_DMA_BUFFER_SIZE_MISMATCH:
            return "NCP_ST_SYSMEM_LAYOUT_DMA_BUFFER_SIZE_MISMATCH";
        case NCP_ST_SM_DIMM0_NOT_POPULATED:
            return "NCP_ST_SM_DIMM0_NOT_POPULATED";
        case NCP_ST_SM_DIMM1_NOT_POPULATED:
            return "NCP_ST_SM_DIMM1_NOT_POPULATED";
        case NCP_ST_SM_DIMM_SPD_MISMATCH:
            return "NCP_ST_SM_DIMM_SPD_MISMATCH";
        case NCP_ST_SM_DIMM_ECC_NOT_SUPPORTED:
            return "NCP_ST_SM_DIMM_ECC_NOT_SUPPORTED";
        case NCP_ST_SUBNET_SEM_INIT_ERROR:
            return "NCP_ST_SUBNET_SEM_INIT_ERROR";
        case NCP_ST_SUBNET_INVALID_PTN:
            return "NCP_ST_SUBNET_INVALID_PTN";
        case NCP_ST_SUBNET_INVALID_HDL:
            return "NCP_ST_SUBNET_INVALID_HDL";
        case NCP_ST_SUBNET_NO_MATCH:
            return "NCP_ST_SUBNET_NO_MATCH";
        case NCP_ST_SUBNET_INVALID_XSIZE:
            return "NCP_ST_SUBNET_INVALID_XSIZE";
        case NCP_ST_SUBNET_INVALID_BUCKET_IDX:
            return "NCP_ST_SUBNET_INVALID_BUCKET_IDX";
        case NCP_ST_SUBNET_INVALID_CHECKPOINT:
            return "NCP_ST_SUBNET_INVALID_CHECKPOINT";
        case NCP_ST_SUBNET_READ_ERROR:
            return "NCP_ST_SUBNET_READ_ERROR";
        case NCP_ST_SUBNET_WRITE_ERROR:
            return "NCP_ST_SUBNET_WRITE_ERROR";
        case NCP_ST_INVALID_TS_INDEX:
            return "NCP_ST_INVALID_TS_INDEX";
        case NCP_ST_INVALID_TS_VALUE:
            return "NCP_ST_INVALID_TS_VALUE";
        case NCP_ST_MAC_INVALID_SPEED:
            return "NCP_ST_MAC_INVALID_SPEED";
        case NCP_ST_MAC_INVALID_DUPLEX:
            return "NCP_ST_MAC_INVALID_DUPLEX";
        case NCP_ST_MAC_INVALID_MAC:
            return "NCP_ST_MAC_INVALID_MAC";
        case NCP_ST_EIOA_HANDLE_INVALID:
            return "NCP_ST_EIOA_HANDLE_INVALID";
        case NCP_ST_EIOA_SEM_INIT_ERROR:
            return "NCP_ST_EIOA_SEM_INIT_ERROR";
        case NCP_ST_EIOA_INVALID_TAG:
            return "NCP_ST_EIOA_INVALID_TAG";
        case NCP_ST_EIOA_INVALID_VLAN_TYPE:
            return "NCP_ST_EIOA_INVALID_VLAN_TYPE";
        case NCP_ST_EIOA_INVALID_BUFFER:
            return "NCP_ST_EIOA_INVALID_BUFFER";
        case NCP_ST_EIOA_INVALID_PORT_CONFIG:
            return "NCP_ST_EIOA_INVALID_PORT_CONFIG";
        case NCP_ST_EIOA_MDIO_STATUS_MSB_READ_ERROR:
            return "NCP_ST_EIOA_MDIO_STATUS_MSB_READ_ERROR";
        case NCP_ST_EIOA_MDIO_CONTROL_BUSY_READ_ERROR:
            return "NCP_ST_EIOA_MDIO_CONTROL_BUSY_READ_ERROR";
        case NCP_ST_EIOA_PORT_NOT_CONFIGURED:
            return "NCP_ST_EIOA_PORT_NOT_CONFIGURED";
        case NCP_ST_EIOA_MCAST_TBL_IDX_ERROR:
            return "NCP_ST_EIOA_MCAST_TBL_IDX_ERROR";
        case NCP_ST_EIOA_SPEED_NOT_SUPPORTED:
            return "NCP_ST_EIOA_SPEED_NOT_SUPPORTED";
        case NCP_ST_EIOA_FUNC_NOT_SUPPORTED:
            return "NCP_ST_EIOA_FUNC_NOT_SUPPORTED";
        case NCP_ST_EIOA_PORT_SHAPER_INVALID_VALUE:
            return "NCP_ST_EIOA_PORT_SHAPER_INVALID_VALUE";
        case NCP_ST_EIOA_PORT_SCHEDULER_INVALID_VALUE:
            return "NCP_ST_EIOA_PORT_SCHEDULER_INVALID_VALUE";
        case NCP_ST_EIOA_INVALID_EIOA:
            return "NCP_ST_EIOA_INVALID_EIOA";
        case NCP_ST_EIOA_MEM_INIT_ERROR:
            return "NCP_ST_EIOA_MEM_INIT_ERROR";
        case NCP_ST_EIOA_COS_INVALID:
            return "NCP_ST_EIOA_COS_INVALID";
        case NCP_ST_EIOA_DSCP_INVALID:
            return "NCP_ST_EIOA_DSCP_INVALID";
        case NCP_ST_EIOA_INVALID_VLAN_ID:
            return "NCP_ST_EIOA_INVALID_VLAN_ID";
        case NCP_ST_EIOA_VLAN_ID_ALREADY_DEFINED:
            return "NCP_ST_EIOA_VLAN_ID_ALREADY_DEFINED";
        case NCP_ST_EIOA_PORT_DEFAULT_VLAN_ID:
            return "NCP_ST_EIOA_PORT_DEFAULT_VLAN_ID";
        case NCP_ST_EIOA_INVALID_CVLAN_ID:
            return "NCP_ST_EIOA_INVALID_CVLAN_ID";
        case NCP_ST_EIOA_HSS_PHY_APB2SER_CMD_TIMEOUT:
            return "NCP_ST_EIOA_HSS_PHY_APB2SER_CMD_TIMEOUT";
        case NCP_ST_EIOA_HSS_PHY_APB2SER_AHBL_TIMEOUT:
            return "NCP_ST_EIOA_HSS_PHY_APB2SER_AHBL_TIMEOUT";
        case NCP_ST_EIOA_HSS_PHY_APB2SER_HRESP_ERR:
            return "NCP_ST_EIOA_HSS_PHY_APB2SER_HRESP_ERR";
        case NCP_ST_EIOA_HSS_PLL_ERR:
            return "NCP_ST_EIOA_HSS_PLL_ERR";
        case NCP_ST_EIOA_HSS_SERDES_APB2SER_CMD_TIMEOUT:
            return "NCP_ST_EIOA_HSS_SERDES_APB2SER_CMD_TIMEOUT";
        case NCP_ST_EIOA_POWER_NOT_ENABLED:
            return "NCP_ST_EIOA_POWER_NOT_ENABLED";
        case NCP_ST_EIOA_PORT_SWITCHOVER_COMBINATION_NOT_SUPPORTED:
            return "NCP_ST_EIOA_PORT_SWITCHOVER_COMBINATION_NOT_SUPPORTED";
        case NCP_ST_EIOA_PORT_SWITCHOVER_NOT_FIBER:
            return "NCP_ST_EIOA_PORT_SWITCHOVER_NOT_FIBER";
        case NCP_ST_EIOA_AUTO_NEGOTIATION_NOT_STARTED:
            return "NCP_ST_EIOA_AUTO_NEGOTIATION_NOT_STARTED";
        case NCP_ST_EIOA_PORT_SWITCHOVER_BAD_PARAMETERS:
            return "NCP_ST_EIOA_PORT_SWITCHOVER_BAD_PARAMETERS";
        case NCP_ST_EIOA_HSS_PHY_BAD_PARAMETERS:
            return "NCP_ST_EIOA_HSS_PHY_BAD_PARAMETERS";
        case NCP_ST_EIOA_INVALID_HANDLE:
            return "NCP_ST_EIOA_INVALID_HANDLE";
        case NCP_ST_EIOA_INVALID_PORT_HANDLE:
            return "NCP_ST_EIOA_INVALID_PORT_HANDLE";
        case NCP_ST_EIOA_PORT_INVALID_INTERFACE_TYPE:
            return "NCP_ST_EIOA_PORT_INVALID_INTERFACE_TYPE";
        case NCP_ST_EIOA_PORT_INVALID_PRIORITY:
            return "NCP_ST_EIOA_PORT_INVALID_PRIORITY";
        case NCP_ST_EIOA_INVALID_VIRTUAL_DOMAIN_ID:
            return "NCP_ST_EIOA_INVALID_VIRTUAL_DOMAIN_ID";
        case NCP_ST_EIOA_INVALID_MPLS_LABEL:
            return "NCP_ST_EIOA_INVALID_MPLS_LABEL";
        case NCP_ST_EIOA_SWITCH_HANDLE_INVALID:
            return "NCP_ST_EIOA_SWITCH_HANDLE_INVALID";
        case NCP_ST_EIOA_SWITCH_SEM_INIT_ERROR:
            return "NCP_ST_EIOA_SWITCH_SEM_INIT_ERROR";
        case NCP_ST_EIOA_SWITCH_ENTRY_DISABLED:
            return "NCP_ST_EIOA_SWITCH_ENTRY_DISABLED";
        case NCP_ST_EIOA_EPP_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_EPP_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_IPP_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_IPP_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_ACL_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_ACL_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_BRIDGE_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_BRIDGE_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_POLICER_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_POLICER_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_BRIDGE_SUPERVISOR_LEARN_TIMEOUT:
            return "NCP_ST_EIOA_BRIDGE_SUPERVISOR_LEARN_TIMEOUT";
        case NCP_ST_EIOA_BRIDGE_SUPERVISOR_LEARN_FAILED:
            return "NCP_ST_EIOA_BRIDGE_SUPERVISOR_LEARN_FAILED";
        case NCP_ST_EIOA_BRIDGE_AGE_TIMEOUT_CONFIG_ERROR:
            return "NCP_ST_EIOA_BRIDGE_AGE_TIMEOUT_CONFIG_ERROR";
        case NCP_ST_EIOA_BRIDGE_AGGREGATION_HASH_INVALID_IDX:
            return "NCP_ST_EIOA_BRIDGE_AGGREGATION_HASH_INVALID_IDX";
        case NCP_ST_EIOA_BRIDGE_INVALID_LOGICAL_PORT_ID:
            return "NCP_ST_EIOA_BRIDGE_INVALID_LOGICAL_PORT_ID";
        case NCP_ST_EIOA_BRIDGE_INVALID_AGE:
            return "NCP_ST_EIOA_BRIDGE_INVALID_AGE";
        case NCP_ST_EIOA_BRIDGE_INVALID_KEY_TYPE:
            return "NCP_ST_EIOA_BRIDGE_INVALID_KEY_TYPE";
        case NCP_ST_EIOA_BRIDGE_LEARNING_NOTIFICATION_DISABLED:
            return "NCP_ST_EIOA_BRIDGE_LEARNING_NOTIFICATION_DISABLED";
        case NCP_ST_EIOA_BRIDGE_SUPERVISOR_UNLEARN_TIMEOUT:
            return "NCP_ST_EIOA_BRIDGE_SUPERVISOR_UNLEARN_TIMEOUT";
        case NCP_ST_EIOA_BRIDGE_SUPERVISOR_UNLEARN_FAILED:
            return "NCP_ST_EIOA_BRIDGE_SUPERVISOR_UNLEARN_FAILED";
        case NCP_ST_EIOA_BRIDGE_AGE_HYSTERESIS_INVALID:
            return "NCP_ST_EIOA_BRIDGE_AGE_HYSTERESIS_INVALID";
        case NCP_ST_EIOA_POLICER_HANDLE_INVALID:
            return "NCP_ST_EIOA_POLICER_HANDLE_INVALID";
        case NCP_ST_EIOA_POLICER_GLOBAL_MAX_RATE_INVALID:
            return "NCP_ST_EIOA_POLICER_GLOBAL_MAX_RATE_INVALID";
        case NCP_ST_EIOA_POLICER_RATE_TOO_FAST:
            return "NCP_ST_EIOA_POLICER_RATE_TOO_FAST";
        case NCP_ST_EIOA_POLICER_RATE_TOO_SLOW:
            return "NCP_ST_EIOA_POLICER_RATE_TOO_SLOW";
        case NCP_ST_EIOA_POLICER_RATE_PEAK_LESS_THAN_COMMITTED:
            return "NCP_ST_EIOA_POLICER_RATE_PEAK_LESS_THAN_COMMITTED";
        case NCP_ST_EIOA_SWITCH_POLICER_MODE_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_MODE_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_ID_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_ID_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_MAX_BURST_SIZE_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_MAX_BURST_SIZE_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_VLAN_ID_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_VLAN_ID_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_ACL_ID_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_ACL_ID_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_COLOR_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_COLOR_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_COS_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_COS_INVALID";
        case NCP_ST_EIOA_SWITCH_POLICER_DSCP_INVALID:
            return "NCP_ST_EIOA_SWITCH_POLICER_DSCP_INVALID";
        case NCP_ST_EIOA_SWITCH_STATS_MEMORY_INIT_TIMEOUT:
            return "NCP_ST_EIOA_SWITCH_STATS_MEMORY_INIT_TIMEOUT";
        case NCP_ST_EIOA_SWITCH_STATS_COUNTER_NOT_DEFINED:
            return "NCP_ST_EIOA_SWITCH_STATS_COUNTER_NOT_DEFINED";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_DIRECTION:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_DIRECTION";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_PORT_GROUP:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_PORT_GROUP";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_DSCP_GROUP:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_DSCP_GROUP";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_VLAN_GROUP:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_VLAN_GROUP";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_DSCP:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_DSCP";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_VLAN:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_VLAN";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_COUNTER:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_COUNTER";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_PACKET_TYPE:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_PACKET_TYPE";
        case NCP_ST_TASK_GEN_RATE_LIMIT_CONFIG_ERROR:
            return "NCP_ST_TASK_GEN_RATE_LIMIT_CONFIG_ERROR";
        case NCP_ST_MAC_PHY_OUI_UNKNOWN:
            return "NCP_ST_MAC_PHY_OUI_UNKNOWN";
        case NCP_ST_MAC_PHY_OUI_FOUND:
            return "NCP_ST_MAC_PHY_OUI_FOUND";
        case NCP_ST_MAC_SPEED_REQUIRES_FIBER:
            return "NCP_ST_MAC_SPEED_REQUIRES_FIBER";
        case NCP_ST_EIOA_COMMON_SHARED_MAC_ADDR_INVALID_INDEX:
            return "NCP_ST_EIOA_COMMON_SHARED_MAC_ADDR_INVALID_INDEX";
        case NCP_ST_EIOA_COMMON_SHARED_MAC_ADDR_INVALID_MAC_ADDR:
            return "NCP_ST_EIOA_COMMON_SHARED_MAC_ADDR_INVALID_MAC_ADDR";
        case NCP_ST_EIOA_COMMON_TASK_PRIORITY_INVALID:
            return "NCP_ST_EIOA_COMMON_TASK_PRIORITY_INVALID";
        case NCP_ST_EIOA_COMMON_BRIDGING_VP_ID_INVALID:
            return "NCP_ST_EIOA_COMMON_BRIDGING_VP_ID_INVALID";
        case NCP_ST_EIOA_COMMON_VP_NOT_SET:
            return "NCP_ST_EIOA_COMMON_VP_NOT_SET";
        case NCP_ST_EIOA_COMMON_VP_TYPE_INVALID:
            return "NCP_ST_EIOA_COMMON_VP_TYPE_INVALID";
        case NCP_ST_EIOA_PORT_INGRESS_ETHERTYPE_IDX_INVALID:
            return "NCP_ST_EIOA_PORT_INGRESS_ETHERTYPE_IDX_INVALID";
        case NCP_ST_EIOA_PORT_VLANS_PRESENT:
            return "NCP_ST_EIOA_PORT_VLANS_PRESENT";
        case NCP_ST_EIOA_PORT_NON_QINQ:
            return "NCP_ST_EIOA_PORT_NON_QINQ";
        case NCP_ST_EIOA_PORT_VLAN_MAP_FULL:
            return "NCP_ST_EIOA_PORT_VLAN_MAP_FULL";
        case NCP_ST_EIOA_PORT_MACSEC_NOT_SUPPORTED:
            return "NCP_ST_EIOA_PORT_MACSEC_NOT_SUPPORTED";
        case NCP_ST_EIOA_PORT_MACSEC_TX_INVALID_SA:
            return "NCP_ST_EIOA_PORT_MACSEC_TX_INVALID_SA";
        case NCP_ST_EIOA_PORT_MACSEC_TX_INVALID_SC:
            return "NCP_ST_EIOA_PORT_MACSEC_TX_INVALID_SC";
        case NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_ACL_ID:
            return "NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_ACL_ID";
        case NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_RULE_ID:
            return "NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_RULE_ID";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_RULES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_RULES";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_IPv4_ADDRESSES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_IPv4_ADDRESSES";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_IP_ADDRESSES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_IP_ADDRESSES";
        case NCP_ST_EIOA_SWITCH_ACL_IPv6_ADDRESS_IN_IPv4_MODE:
            return "NCP_ST_EIOA_SWITCH_ACL_IPv6_ADDRESS_IN_IPv4_MODE";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_LAYER_4_PORTS:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_LAYER_4_PORTS";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MAC_ADDRESSES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MAC_ADDRESSES";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MAC_WILDCARD_ADDRESSES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MAC_WILDCARD_ADDRESSES";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MACSEC_ETHERTYPE_PROTOCOL_VALUES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_MACSEC_ETHERTYPE_PROTOCOL_VALUES";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_DSCP_COS_VALUES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_DSCP_COS_VALUES";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_COS:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_COS";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_DSCP:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_DSCP";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_VLAN_ID:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_VLAN_ID";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_POLICER_ID:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_POLICER_ID";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_POLICER_MAP:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_POLICER_MAP";
        case NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_ROUTES:
            return "NCP_ST_EIOA_SWITCH_ACL_TOO_MANY_ROUTES";
        case NCP_ST_EIOA_SWITCH_ACL_UNKNOWN_ROUTE:
            return "NCP_ST_EIOA_SWITCH_ACL_UNKNOWN_ROUTE";
        case NCP_ST_EIOA_SWITCH_ACL_ROUTE_INVALID_SUB_INTERFACE:
            return "NCP_ST_EIOA_SWITCH_ACL_ROUTE_INVALID_SUB_INTERFACE";
        case NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_PBR_ID:
            return "NCP_ST_EIOA_SWITCH_ACL_OUT_OF_RANGE_PBR_ID";
        case NCP_ST_EIOA_SWITCH_ACL_ACTION_ID_CLASH:
            return "NCP_ST_EIOA_SWITCH_ACL_ACTION_ID_CLASH";
        case NCP_ST_EIOA_SWITCH_ACL_INVALID_HANDLE:
            return "NCP_ST_EIOA_SWITCH_ACL_INVALID_HANDLE";
        case NCP_ST_EIOA_SWITCH_ACL_MAC_WILDCARD_ADDRESS_PARTIAL_OVERLAP:
            return "NCP_ST_EIOA_SWITCH_ACL_MAC_WILDCARD_ADDRESS_PARTIAL_OVERLAP";
        case NCP_ST_EIOA_SWITCH_BRIDGE_INVALID_HANDLE:
            return "NCP_ST_EIOA_SWITCH_BRIDGE_INVALID_HANDLE";
        case NCP_ST_EIOA_SWITCH_STATS_INVALID_HANDLE:
            return "NCP_ST_EIOA_SWITCH_STATS_INVALID_HANDLE";
        case NCP_ST_EIOA_SWITCH_BRIDGE_DESTINATION_VP_INVALID_NAME:
            return "NCP_ST_EIOA_SWITCH_BRIDGE_DESTINATION_VP_INVALID_NAME";
        case NCP_ST_EIOA_BRIDGE_DESTINATION_VP_DUPLICATE_NAME:
            return "NCP_ST_EIOA_BRIDGE_DESTINATION_VP_DUPLICATE_NAME";
        case NCP_ST_EIOA_INVALID_DOMAIN_ID:
            return "NCP_ST_EIOA_INVALID_DOMAIN_ID";
        case NCP_ST_EIOA_MAP_NOT_SET:
            return "NCP_ST_EIOA_MAP_NOT_SET";
        case NCP_ST_EIOA_VLAN_MAP_PRESENT:
            return "NCP_ST_EIOA_VLAN_MAP_PRESENT";
        case NCP_ST_EIOA_PAUSE_MODE_NOT_SUPPORTED:
            return "NCP_ST_EIOA_PAUSE_MODE_NOT_SUPPORTED";
        case NCP_ST_CAT_NOT_CONFIGURED:
            return "NCP_ST_CAT_NOT_CONFIGURED";
        case NCP_ST_CAT_NOT_ENABLED:
            return "NCP_ST_CAT_NOT_ENABLED";
        case NCP_ST_CAT_INSUFFICIENT_MEMORY:
            return "NCP_ST_CAT_INSUFFICIENT_MEMORY";
        case NCP_ST_CAT_MEMORY_SIZE_INVALID:
            return "NCP_ST_CAT_MEMORY_SIZE_INVALID";
        case NCP_ST_CAT_MEMORY_ADDR_INVALID:
            return "NCP_ST_CAT_MEMORY_ADDR_INVALID";
        case NCP_ST_CAT_MISSING_WRITEFN:
            return "NCP_ST_CAT_MISSING_WRITEFN";
        case NCP_ST_CAT_MISSING_READFN:
            return "NCP_ST_CAT_MISSING_READFN";
        case NCP_ST_CAT_DATA_BUFFER_INVALID:
            return "NCP_ST_CAT_DATA_BUFFER_INVALID";
        case NCP_ST_CAT_DATA_BUFSIZE_INVALID:
            return "NCP_ST_CAT_DATA_BUFSIZE_INVALID";
        case NCP_ST_CAT_INVALID_ITERATOR:
            return "NCP_ST_CAT_INVALID_ITERATOR";
        case NCP_ST_CAT_WRITE_ERROR:
            return "NCP_ST_CAT_WRITE_ERROR";
        case NCP_ST_CAT_READ_ERROR:
            return "NCP_ST_CAT_READ_ERROR";
        case NCP_ST_CAT_DATA_EOF:
            return "NCP_ST_CAT_DATA_EOF";
        case NCP_ST_CAT_MISMATCH_DATA_TRUNCATED:
            return "NCP_ST_CAT_MISMATCH_DATA_TRUNCATED";
        case NCP_ST_CAT_MISMATCH_DEVICE:
            return "NCP_ST_CAT_MISMATCH_DEVICE";
        case NCP_ST_CAT_MISMATCH_RTE_REVISION:
            return "NCP_ST_CAT_MISMATCH_RTE_REVISION";
        case NCP_ST_CAT_NO_RECOVERY_DATA:
            return "NCP_ST_CAT_NO_RECOVERY_DATA";
        case NCP_ST_CAT_CORRUPT_RECOVERY_DATA:
            return "NCP_ST_CAT_CORRUPT_RECOVERY_DATA";
        case NCP_ST_CAT_NO_PREVIOUS_WRITE:
            return "NCP_ST_CAT_NO_PREVIOUS_WRITE";
        case NCP_ST_CAT_RAW_DATA_POLL:
            return "NCP_ST_CAT_RAW_DATA_POLL";
        case NCP_ST_SAL_SSID_FAILURE:
            return "NCP_ST_SAL_SSID_FAILURE";
        case NCP_ST_SAL_CHDIR_FAILURE:
            return "NCP_ST_SAL_CHDIR_FAILURE";
        case NCP_ST_ALARM_NONE_PENDING:
            return "NCP_ST_ALARM_NONE_PENDING";
        case NCP_ST_ALARM_NOT_CONFIGURED:
            return "NCP_ST_ALARM_NOT_CONFIGURED";
        case NCP_ST_ALARM_NO_HANDLER_FOR_IRQ:
            return "NCP_ST_ALARM_NO_HANDLER_FOR_IRQ";
        case NCP_ST_ALARM_NO_SOURCE_FOR_IRQ:
            return "NCP_ST_ALARM_NO_SOURCE_FOR_IRQ";
        case NCP_ST_ALARM_INVALID_IRQ:
            return "NCP_ST_ALARM_INVALID_IRQ";
        case NCP_ST_TDMIOA_INVALID_RX_PREPEND_SIZE:
            return "NCP_ST_TDMIOA_INVALID_RX_PREPEND_SIZE";
        case NCP_ST_TDMIOA_INVALID_TX_PREPEND_SIZE:
            return "NCP_ST_TDMIOA_INVALID_TX_PREPEND_SIZE";
        case NCP_ST_PROFILER_ENTRIES_BUFFER_INVALID:
            return "NCP_ST_PROFILER_ENTRIES_BUFFER_INVALID";
        case NCP_ST_NRCP_INVALID_PARAM:
            return "NCP_ST_NRCP_INVALID_PARAM";
        case NCP_ST_NRCP_POLL_TIMEOUT:
            return "NCP_ST_NRCP_POLL_TIMEOUT";
        case NCP_ST_NRCP_NOT_SUPPORTED:
            return "NCP_ST_NRCP_NOT_SUPPORTED";
        case NCP_ST_ECM_NOT_SUPPORTED:
            return "NCP_ST_ECM_NOT_SUPPORTED";
        case NCP_ST_ECM_OFFSET_INVALID:
            return "NCP_ST_ECM_OFFSET_INVALID";
        case NCP_ST_ECM_VALUE_LENGTH_INVALID:
            return "NCP_ST_ECM_VALUE_LENGTH_INVALID";
        case NCP_ST_ECM_BUSY:
            return "NCP_ST_ECM_BUSY";
        case NCP_ST_ECM_FUSE_READ_TIMEOUT:
            return "NCP_ST_ECM_FUSE_READ_TIMEOUT";
        case NCP_ST_ECM_FUSE_WRITE_TIMEOUT:
            return "NCP_ST_ECM_FUSE_WRITE_TIMEOUT";
        case NCP_ST_ECM_FUSE_READ_ERROR:
            return "NCP_ST_ECM_FUSE_READ_ERROR";
        case NCP_ST_ECM_FUSE_WRITE_ERROR:
            return "NCP_ST_ECM_FUSE_WRITE_ERROR";
        case NCP_ST_ECM_READ_MISCOMPARE:
            return "NCP_ST_ECM_READ_MISCOMPARE";
        case NCP_ST_ECM_MARGIN_READ_MISCOMPARE:
            return "NCP_ST_ECM_MARGIN_READ_MISCOMPARE";
        case NCP_ST_LAST_STATUS_CODE:
            return "NCP_ST_LAST_STATUS_CODE";
        default:
            return NULL;
    } /* End of switch */
}

const char *
ncp_status_get(ncp_st_t st)       /* Status code */
{
    const char *code_name = find_status_code(st);

    if ( code_name == NULL ) {
        return ncp_st_err_msg;
    } else {
        return code_name;
    }
}

