/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <sys/types.h>
#include <unistd.h> 

#include "ncp_internal.h"

static ncp_sem_t *taskRxSem = NULL;

ncp_st_t 
ncp_task_hdl_create(
    ncp_hdl_t       ncpHdl,
    uint8_t     threadQueueSetId,
    bool      shared,
    bool      orderedTaskCompletion,   
    ncp_task_hdl_t  *taskHdl)
{
    ncp_st_t ncpStatus = NCP_ST_SUCCESS;

    (void)ncpHdl;
    (void)threadQueueSetId;
    (void)shared;
    (void)orderedTaskCompletion;
    *taskHdl = NULL;

    taskRxSem = ncp_nvm_malloc(sizeof(ncp_sem_t));
    if (taskRxSem == NULL)
    {
        ncpStatus = NCP_ST_NO_MEMORY;
        goto ncp_return;
    }
printf("%s(%d)-%s: pid=%d, tid=0x%08x, taskRxSem=%p\n", __FILE__,__LINE__,__func__,getpid(), (int)pthread_self(), taskRxSem);fflush(stdout);
    ncpStatus = ncp_sem_init(taskRxSem, TRUE, 0);
    if (ncpStatus != NCP_ST_SUCCESS)
    {
        printf("ERROR: ncp_sem_init failed in ncp_task_recv\n");
        goto ncp_return;
    }

NCP_RETURN_LABEL
    return ncpStatus;
}


ncp_st_t 
ncp_task_hdl_remove(
    ncp_task_hdl_t  taskHdl)
{
    (void)taskHdl;
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_recv_queue_bind(
	ncp_task_hdl_t taskHdl,
	uint8_t ncaQueueId,
	uint8_t weight,
	bool shared,
	bool fixedRecvQueueId,
	uint8_t *recvQueueId)
{
    (void)taskHdl;
    (void)ncaQueueId;
    (void)weight;
    (void)shared;
    (void)fixedRecvQueueId;
    (void)recvQueueId;
    return NCP_ST_SUCCESS;
}
ncp_st_t
ncp_task_recv_queue_unbind(
	ncp_task_hdl_t taskHdl,
	uint8_t recvQueueId)
{
    (void)taskHdl;
    (void)recvQueueId;
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_recv_queue_vp_bind(
	ncp_task_hdl_t taskHdl,
	uint8_t recvQueueId,
	ncp_vp_hdl_t vpHdl,
	uint8_t engineSequenceId,
	bool shared,
	ncp_task_recv_fn_t recvFunc,
	void *recvFuncArg)
{
    (void)taskHdl;
    (void)recvQueueId;
    (void)vpHdl;
    (void)engineSequenceId;
    (void)shared;
    (void)recvFunc;
    (void)recvFuncArg;
    return NCP_ST_SUCCESS;
}


ncp_st_t
ncp_task_recv_queue_vp_unbind(
	ncp_task_hdl_t taskHdl,
	uint8_t recvQueueId,
	ncp_vp_hdl_t vpHdl,
	uint8_t engineSequenceId)
{
    (void)taskHdl;
    (void)recvQueueId;
    (void)vpHdl;
    (void)engineSequenceId;
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_buffer_alloc (
	uint32_t size,
	void **taskAddr)
{
    (void)size;
    *taskAddr = ncp_malloc(size);
    if (*taskAddr) {
        return NCP_ST_SUCCESS;
    } else {
        return NCP_ST_NO_MEMORY;
    }
}

ncp_st_t
ncp_task_buffer_free (void *taskAddr)
{
    ncp_free(taskAddr);
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_recv(
	ncp_task_hdl_t      taskHdl,
	uint8_t         *pRecvQueueId,
	ncp_vp_hdl_t        *pVpHdl,
	uint8_t         *pEngineSequenceId,
	ncp_task_recv_buf_t **task,
    void                **taskData,	
	void                **recvHandlerArg,
	bool          wait)
{
    ncp_st_t ncpStatus = NCP_ST_SUCCESS;

    (void)taskHdl;
    (void)pRecvQueueId;
    (void)pVpHdl;
    (void)pEngineSequenceId;
    (void)task;
    (void)taskData;
    (void)recvHandlerArg;
    if (wait)
    {
        /* Wait until a packet is received */
        /* Wait until a packet is received */
#if 1
        /* TODO: Need to figure out why 1st call of ncp_sem_wait returns w/o ncp_sem_post */
        while (NCP_ST_SUCCESS == (ncpStatus = ncp_sem_wait(taskRxSem)))
        {
/* TODO: Should never happen as long as nobody does ncp_sem_post */
printf("%s(%d)-%s: pid=%d, tid=0x%08x, taskRxSem=%p: sleep(1)\n", __FILE__,__LINE__,__func__,getpid(), (int)pthread_self(), taskRxSem);fflush(stdout);
            sleep(1);
        }
#else
        ncpStatus = ncp_sem_wait(taskRxSem);
#endif
    } else {
        ncpStatus = NCP_ST_TASK_RECV_QUEUE_EMPTY;
    }
    return ncpStatus;
}

ncp_st_t
ncp_task_send (
	ncp_task_hdl_t taskHdl,
	ncp_vp_hdl_t vpHdl,
	ncp_task_send_meta_t *taskMetaData,
	void *taskBuffer,
	void *taskPduData,
	bool wait,
	ncp_task_send_done_fn_t sendDoneFn,
    void                   *sendDoneFuncArg)
{
    (void)taskHdl;
    (void)vpHdl;
    (void)taskMetaData;
    (void)taskBuffer;
    (void)taskPduData;
    (void)wait;
    (void)sendDoneFn;
    (void)sendDoneFuncArg;
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_recv_queue_name_to_id(ncp_hdl_t ncpHdl,
                           char *recvQueueName, 
                           uint8_t *p_ncaQueueID)
{
    (void)ncpHdl;
    (void)recvQueueName;
    *p_ncaQueueID = 0;
    return NCP_ST_SUCCESS;
}

ncp_st_t
ncp_task_recv_queue_get_tqs_id(ncp_hdl_t ncpHdl, 
                           uint8_t ncaQueueID, 
                           uint8_t *p_tqsID)
{
    (void)ncpHdl;
    (void)ncaQueueID;
    *p_tqsID = 0;
    return NCP_ST_SUCCESS;
}

