/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef _ncp_sorted_set_h
#define _ncp_sorted_set_h

#include "ncp_types.h"

#ifdef __cplusplus
extern "C" { 
#endif

typedef void* ncp_sorted_set_t;

typedef enum {
    NCP_SORTED_SET_CMP_EQUAL = 0,
    NCP_SORTED_SET_CMP_LEFT,
    NCP_SORTED_SET_CMP_RIGHT
}ncp_sorted_set_cmp_result_t;

typedef ncp_sorted_set_cmp_result_t (*ncp_sorted_set_element_compare_function_t)(void *akey,void *bkey);

typedef int (*ncp_sorted_set_element_function_t)(uint32_t n,void *key,void *value, void *args);

typedef int (*ncp_sorted_set_nodeDeleteFn_t)( void *key,void *value);

NCP_API ncp_sorted_set_t* ncp_sorted_set_create(
                    uint32_t mallocSizeBytes,
                    uint32_t numInitialNodes);

NCP_API void ncp_sorted_set_delete_nodes_and_set(
                    ncp_sorted_set_t *sorted_set,
                    ncp_sorted_set_nodeDeleteFn_t delFn);

NCP_API int ncp_sorted_set_iterate_through_set(
                    ncp_sorted_set_t *sorted_set,
                    ncp_sorted_set_element_function_t fn, 
                    void *args);

NCP_API void* ncp_sorted_set_get(
                    ncp_sorted_set_t *sorted_set,
                    void *key,
                    ncp_sorted_set_element_compare_function_t compareFn);

NCP_API void* ncp_sorted_set_get_n(
                    ncp_sorted_set_t *sorted_set,
                    uint32_t n);

NCP_API void* ncp_sorted_set_put_n(
                    ncp_sorted_set_t *sorted_set,
                    uint32_t n,
                    void *value);

NCP_API uint32_t ncp_sorted_set_size(
                    ncp_sorted_set_t *sorted_set);

NCP_API void* ncp_sorted_set_put(
                    ncp_sorted_set_t *sorted_set,
                    void *key,
                    ncp_sorted_set_element_compare_function_t compareFn,
                    void* value);

NCP_API void *ncp_sorted_set_remove(
                    ncp_sorted_set_t *sorted_set,
                    void *key,  
                    ncp_sorted_set_element_compare_function_t compareFn);


#ifdef __cplusplus
}
#endif

#endif /* _ncp_sorted_set_h */
