/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL Layer
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the external definitions required to use the
 * ADK Pipeline Abstraction layer. This file can be included by the
 * control plane of the application.
 *
 ***********************************************************************/
#ifndef _ADK_PAL_H_
#define _ADK_PAL_H_

#include <stdio.h>
#include <rte_eventdev.h>

#define ADK_RTE_ETH_BD_RX_METADATA_OFFSET 0
#define ADK_RTE_ETH_BD_TX_METADATA_OFFSET 0
#define ADK_RTE_CRYPTO_BD_TX_METADATA_OFFSET 0

/*
 * data structure to represent a worker to event-port-id mapping. 
 */
typedef struct adk_pal_worker_info_s {
    uint8_t     evdev_port_id; /*  event port id*/
    uint32_t    lcore_id;      /* worker core id*/
} adk_pal_worker_info_t;

/*
 * data structure to represent event-queue information
 */
typedef struct adk_pal_evq_info_s {
    uint8_t     event_queue_id; 
    /*  event queue id*/
    
    uint32_t    evq_cfg_flags;  
    /* Event queue configuration bitmap flags (defined as DPDK Event-Dev)
     * RTE_EVENT_QUEUE_CFG_ATOMIC_ONLY, RTE_EVENT_QUEUE_CFG_ORDERED_ONLY
     * RTE_EVENT_QUEUE_CFG_PARALLEL_ONLY*/
} adk_pal_evq_info_t;



/*
 * data structure to represent a bridge ethernet instance. 
 */
typedef struct adk_pal_bridge_eth_info_s {
    uint8_t     *eth_ports_list; /* list of ethernet ports*/
    uint8_t     num_ports;       /* number of ethernet ports*/
    uint8_t     eth_dev_rx_metadata_offset; /* metadata offset in receive direction */
    uint8_t     eth_dev_tx_metadata_offset; /* metadata offset in transmit direction */
    uint8_t     event_queue_id; /* event queue id that connects to bridge-eth-tx driver */
} adk_pal_bridge_eth_info_t;

#ifdef ADK_HOST_PATH_USE_NETD
/*
 * data structure to represent a bridge netdriver instance.
 */
typedef struct adk_pal_bridge_netd_info_s {
    uint8_t     *netd_ports_list; /* list of ethernet ports*/
    uint8_t     num_ports;       /* number of ethernet ports*/
    uint8_t     netd_metadata_offset; /* metadata offset */
    uint8_t     event_queue_id; /* event queue id that connects to bridge-eth-tx driver */
    uint8_t     reserved_cppi_bd_dev_id; /*CPPI bridge dev id to reach netdriver */
} adk_pal_bridge_netd_info_t;
#endif /* ADK_HOST_PATH_USE_NETD */

typedef struct adk_pal_bridge_crypto_info_s {
    uint8_t     crypto_dev_id;  /* Crypto device id*/
    uint8_t     crypto_bd_metadata_offset; /* bridge crypto metadata offset */
    uint8_t     event_queue_id; /* event queue id that connects to bridge-crypto driver */
} adk_pal_bridge_crypto_info_t;

typedef struct adk_pal_global_stats_s {
    /*utility counters*/
    uint64_t    arrival_evnts;      
    uint64_t    nic_arrival_evnts; 
    uint64_t    timer_arrival_events; 
    uint64_t    crypto_arrival_events;
    uint64_t    em_arrival_events;
    uint64_t    cpu_arrival_events;
#ifdef ADK_HOST_PATH_USE_NETD
    uint64_t    netd_arrival_events;
#endif
    uint64_t    pal_batched_em_events;
    uint64_t    pal_batched_nic_events;
    uint64_t    pal_batched_ph_events;
    uint64_t    pal_batched_netd_events;
    uint64_t    pal_batched_stage_events;
    uint64_t    pal_batched_crypto_events;
    uint64_t    pal_batched_release_events;
    uint64_t    pal_batched_timer_events;
    uint64_t    em_xmt_events;
    uint64_t    rte_xmt_events;
    /*Error counters*/
    uint64_t    pal_dispatcher_err_hdl;
    uint64_t    nic_arrival_null_mbuf_evnts_drop;
    uint64_t    em_xmt_fail_events; /*Number of events failed on em_send_multi for batch*/ 
    uint64_t    rte_xmt_fail_events; /*Number of events failed on rte_eventdev_enqueue_burst for batch*/
    uint64_t    batch_fail_safe_err; /* Error occured due to previos dequed packets not processed completely*/
    uint64_t    batch_fail_safe_drops; /*Number of packets dropped from the previos dequeue*/
}adk_pal_global_stats_t;

/*
 * API to register event-dev information with PAL.
 *
 * 1) Stores event-dev information internally
 * 2) Allocates event-queues to processing groups
 */
adk_status_t
adk_pal_register(
    adk_sys_handle_t    sys_handle,
    uint8_t             evdev_id,
    adk_pal_worker_info_t *w_qm_info,
    uint32_t            num_workers);

/*
 * API to map veth port ot nic port.
 */
adk_status_t
adk_pal_veth_port_to_nic_port_mapping (
    adk_sys_handle_t  sys_handle,
    uint32_t veth_port,
    uint32_t nic_port);

/*
 * API to register tun-dev/pmd with PAL 
 */
adk_status_t
adk_pal_tun_dev_register (
    adk_sys_handle_t  sys_handle,
    uint32_t tun_dev_id,
    uint32_t adk_tunnel_if_id);

/*
 * API to un-register a previously registred 
 * tun-dev/pmd with PAL 
 */
adk_status_t
adk_pal_tun_dev_unregister (
    adk_sys_handle_t  sys_handle,
    uint32_t tun_dev_id);

/*
 * API to add bridge ethernet to PAL
 */
adk_status_t
adk_pal_bridge_eth_add (
    adk_sys_handle_t            sys_handle,
    adk_pal_bridge_eth_info_t   *eth_bd_info_p);

#ifdef ADK_HOST_PATH_USE_NETD
/*
 * API to add netdriver info to PAL
 */
adk_status_t
adk_pal_bridge_netd_add (
    adk_sys_handle_t            sys_handle,
    adk_pal_bridge_netd_info_t   *netd_bd_info_p);

/*
 * API to add Linux dev to PAL
 */
adk_status_t
adk_pal_dev_net_add(
    adk_sys_handle_t __attribute__((unused))  sys_handle,
    uint8_t     event_queue_id,
    uint8_t     netd_metadata_offset);

#endif /* ADK_HOST_PATH_USE_NETD */

adk_status_t
adk_pal_bridge_crypto_add (
    adk_sys_handle_t  sys_handle,
    adk_pal_bridge_crypto_info_t   *bd_crypto_info_p);

/*
 * API to add a worker-to-event-port map to PAL.
 */
adk_status_t
adk_pal_worker_to_event_port_map_add(
    adk_sys_handle_t        sys_handle,
    adk_pal_worker_info_t   *winfo_p);

/*
 * API to remove previously added worker-to-event-port map from PAL.
 */
adk_status_t
adk_pal_worker_to_event_port_map_rem(
    adk_sys_handle_t      sys_handle,
    adk_pal_worker_info_t *winfo_p);

/*
 * port type 
 *
 * INTERNAL : TN traffic 
 * EXTERNAL : BB_CONFIG traffic
 *
 */
typedef enum{
    ADK_PAL_ETH_PORT_TYPE_EXTERNAL,
    ADK_PAL_ETH_PORT_TYPE_INTERNAL
} adk_pal_eth_port_type_e;

/*
 * API to register ethernet-port type
 */
adk_status_t
adk_pal_eth_port_type_register(
    uint8_t                     port_num, 
    adk_pal_eth_port_type_e     port_type);


/*
 *********************************************************
 * Dynamic pipeline related definitions
 *
 *********************************************************
 */
	
 /* 
 * ADK pipeline stage load balance type definition
 */
 typedef enum 
 {
    ADK_PAL_STAGE_LB_TYPE_ORDERED = RTE_SCHED_TYPE_ORDERED,
    ADK_PAL_STAGE_LB_TYPE_ATOMIC  = RTE_SCHED_TYPE_ATOMIC,
    ADK_PAL_STAGE_LB_TYPE_PARALLE = RTE_SCHED_TYPE_PARALLEL,
    ADK_PAL_STAGE_LB_TYPE_DIRECT,
 } adk_pal_stage_lb_type_e;

/* 
 * ADK pipeline stage entry type definition
 */
 typedef enum 
 {
    ADK_PAL_STAGE_ENTRY_TYPE_FN_CALL = 0,
    ADK_PAL_STAGE_ENTRY_TYPE_EV_Q,
    ADK_PAL_STAGE_ENTRY_TYPE_RING,
} adk_pal_stage_entry_type_e;

 /* 
 * ADK pipeline stage entry info
 */
 typedef struct 
 {
    adk_pal_stage_entry_type_e entry_type;
    union {
        struct{
	    uint8_t   evnt_q_id; /* Event-queue-id */
	    int8_t    stage_priority;
	};
	struct rte_ring *ring_buffer; /* Place Holder */
    };
 } adk_pal_stage_entry_info_t;
 

/* 
 * ADK_CP_EXTERNAL 
 * This API to be called by application / configuration layer. 
 * to change a stage's entry type
 */
adk_status_t 
adk_pal_stage_entry_update(const char * stage_name, 
			    adk_pal_stage_entry_info_t  *entry_info);

/* 
 * ADK_CP_EXTERNAL 
 * This API to be called by application / configuration layer. 
 * to upgrade a stage's load-balancing type
 */
adk_status_t 
adk_pal_stage_lb_update(const char * stage_name,
			adk_pal_stage_lb_type_e  new_lb_type);

/* 
 * ADK_CP_EXTERNAL 
 * This API to dump the current pipeline configuration.
 */
adk_status_t
adk_pal_dp_ppl_dump(void);

/*
 * API to dump PAL dataplane stats
 */
adk_status_t adk_pal_global_stats_get(adk_pal_global_stats_t *stats);

#ifdef ADK_DEBUG
adk_status_t
adk_pal_stats_print(void);

adk_status_t
adk_pal_stats_clear(void);
#endif
adk_status_t
adk_pal_set_utility_counters(bool utility_cnt_state);
#endif /* End of _ADK_PAL_H_ */

/*!  end addtogroup _pal
 * @}
 */
