/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_dp_export.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL Layer
 * @{
 *   @addtogroup _palFunctions
 *   @{
 *
 * @brief  
 *
 * This file contains all the external definitions required to use the
 * data plane part of the DPDK Abstraction layer. This file can be included
 * by the dataplane part of the application.
 *
 ***********************************************************************/
#ifndef _ADK_PAL_DP_EXPORT_H_
#define _ADK_PAL_DP_EXPORT_H_

#include <rte_eventdev.h>
#include <rte_crypto.h>

adk_status_t 
adk_pal_dp_init(void);

int
adk_pal_event_dispatcher (
    uint8_t  event_dev_id,
    uint8_t  event_port_id,
    struct rte_event events[], 
    int num,
    int rel_index,
    void *cb_data);

adk_status_t
adk_pal_dp_ph_send (
    uint8_t evd_queue_id,
    uint8_t sub_event_id,
    uint8_t priority,
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam);



/*
 * ADK_DP_INTERNAL
 * Data plane API to be called at the end of each stage. 
 * This function generates a event and passes that event to next stage.
 * 
 * Sets the "ev.event_type" to CPU->CPU/Device
 * next-stage-id is passed in the "ev.sub_event_type" field
 * 
 */
uint32_t
adk_pal_dp_next_stage_event_handle(void  **task, uint8_t task_count, 
                                   uint8_t task_mask, uint8_t  next_stage_id,
                                   uint32_t flow_id, uint8_t  pkt_priority,
                                   adk_task_type_e);

/*
 * ADK_DP_INTERNAL
 * Data plane API to be called at the end of each stage. 
 * This function generates a event and passes that event to next stage.
 * 
 * Sets the "ev.event_type" to CPU->CPU/Device
 * next-stage-id is passed in the "ev.sub_event_type" field
 * 
 */
uint32_t
adk_pal_dp_next_stage_event_handle_with_queue(void  **task, uint8_t task_count, 
                                   uint8_t task_mask, uint8_t  next_stage_id,
                                   uint8_t queue_id, uint32_t flow_id, 
                                   uint8_t  pkt_priority, adk_task_type_e);


/* 
 *  ADK_DP_INTERNAL
 */

uint32_t
adk_pal_dp_crypto_send (
    uint8_t    crypto_dev_id,
    uint32_t  dest_func_idx,
    uint8_t   priority,
    struct rte_mbuf *dpdk_mbuf_p,
    struct rte_crypto_op *op_data,
    uint8_t cbFn,
    void *cbParam);

uint32_t
adk_pal_dp_timer_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam);
/* 
 *  ADK_DP_INTERNAL
 * Wrapper APIs to get event-queue-id assigned to a given stage.
 * can be used for timer callback  to generate atomic events
 * 
 */ 
 adk_status_t
 adk_pal_dp_stage_evq_id_get(uint8_t  next_stage_id, uint8_t   *evnt_q_id);


 /* 
 *  ADK_DP_INTERNAL
 * Wrapper APIs to get load-balance type of a given stage.
 * can be used for timer callback  to generate atomic events
 * 
 */ 
 adk_status_t
 adk_pal_dp_stage_lb_type_get(uint8_t  next_stage_id, uint8_t   *lb_type);
 

#endif /* End of _ADK_PAL_DP_EXPORT_H_ */

/*! end addtogroup _palFunctions
 * @}
 *
 * end addtogroup _pal
 * @}
 */
