/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL Layer
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the ADK PAL internal definitions required to 
 * use the by other ADK modules for Pipeline Abstraction services. 
 * This file can be included by the ADK protocl modules.
 *
 ***********************************************************************/
#ifndef _ADK_PAL_INTERNAL_H_
#define _ADK_PAL_INTERNAL_H_

#include <stdio.h>

#include <adk_dpdk.h>
/* #include <adk_include.h> */
#include "adk_sif_constants.h"
#include "adk_pal_dp.h"
#include "adk_pal_dp_export.h"
#include "adk_metadata.h"
#include <adk_pmu.h>
#include <rte_eventdev.h>

#define ADK_PAL_PKT_TYPE_ALL  (0xFF)

/* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
    extern uint64_t pal_uli_rx_ts, pal_uli_tx_ts;
    extern uint64_t pal_lli_rx_ts, pal_lli_tx_ts;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&pal_lli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&pal_lli_rx_ts, &pal_lli_tx_ts, "PAL LLI MODULE", PMU_PAL_LLI_MODULE)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&pal_uli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&pal_uli_rx_ts, &pal_uli_tx_ts, "PAL ULI MODULE", PMU_PAL_ULI_MODULE)
#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_TX_TS()
#endif
    
    /* Macro definition for ADK overall cycle count */
#ifdef ADK_BLOCK_COUNT_ENABLED
    extern uint64_t adk_uli_rx_ts, adk_uli_tx_ts;
    extern uint64_t adk_lli_rx_ts, adk_lli_tx_ts;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_LLI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&adk_lli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_LLI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&adk_lli_rx_ts, &adk_lli_tx_ts, "ADK LLI", PMU_ADK_LLI_MODULE)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_RX_TS() adk_dp_measure_block_cycle_rx_ts(&adk_uli_rx_ts)
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_TX_TS() adk_dp_measure_block_cycle_tx_ts(&adk_uli_rx_ts, &adk_uli_tx_ts, "ADK ULI", PMU_ADK_ULI_MODULE)
#else
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_LLI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_LLI_TX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_RX_TS()
    #define ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_TX_TS()
#endif

  /*Measure end to end both lli/uli path cycles*/
#ifdef ADK_MEASURE_END_TO_END_CYCLES
    extern uint64_t g_adk_entry_ts[];
    extern uint64_t g_lli_rx_ts[], g_lli_tx_ts[];
    extern uint64_t g_uli_rx_ts[], g_uli_tx_ts[];
    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ADK_ENTRY_TS()      adk_dp_measure_block_cycle_rx_ts(&g_adk_entry_ts[ADK_LCORE_INDEX])
    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS_UPDATE()  g_lli_rx_ts[ADK_LCORE_INDEX] = g_adk_entry_ts[ADK_LCORE_INDEX]; \
												                      g_uli_rx_ts[ADK_LCORE_INDEX] = 0;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS()         adk_dp_measure_block_cycle_rx_ts(&g_lli_rx_ts[ADK_LCORE_INDEX])

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_INTER_TX_TS()   adk_dp_measure_block_cycle_tx_inter_ts(&g_lli_rx_ts[ADK_LCORE_INDEX], \
                                                                        &g_lli_tx_ts[ADK_LCORE_INDEX], PMU_END_TO_END_LLI_MODULE)

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_TX_TS()         if(g_lli_rx_ts[ADK_LCORE_INDEX] != 0 ) { \
	                                                                      adk_dp_measure_block_cycle_tx_ts(&g_lli_rx_ts[ADK_LCORE_INDEX],\
                                                                          &g_lli_tx_ts[ADK_LCORE_INDEX], \
                                                                          "END TO END LLI CYCLES", PMU_END_TO_END_LLI_MODULE); } \
                                                                      else \
                                                                           g_lli_tx_ts[ADK_LCORE_INDEX] = 0;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS_UPDATE()  g_uli_rx_ts[ADK_LCORE_INDEX] = g_adk_entry_ts[ADK_LCORE_INDEX]; \
												                      g_lli_rx_ts[ADK_LCORE_INDEX] = 0;

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS()         adk_dp_measure_block_cycle_rx_ts(&g_uli_rx_ts[ADK_LCORE_INDEX])
    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_INTER_TX_TS()   adk_dp_measure_block_cycle_tx_inter_ts(&g_uli_rx_ts[ADK_LCORE_INDEX], \
                                                                        &g_uli_tx_ts[ADK_LCORE_INDEX], PMU_END_TO_END_ULI_MODULE) 

    #define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_TX_TS()         if(g_uli_rx_ts[ADK_LCORE_INDEX] != 0 ) { \
	                                                                       adk_dp_measure_block_cycle_tx_ts(&g_uli_rx_ts[ADK_LCORE_INDEX],\
			                                                               &g_uli_tx_ts[ADK_LCORE_INDEX], "END TO END ULI CYCLES",\
			                                                               PMU_END_TO_END_ULI_MODULE); }\
                                                                      else \
                                                                           g_uli_tx_ts[ADK_LCORE_INDEX] = 0;
#else
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ADK_ENTRY_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS_UPDATE()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_INTER_TX_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_TX_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS_UPDATE()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_INTER_TX_TS()
#define ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_TX_TS()
#endif

#ifdef ADK_PAL_CYCLE_MEASUREMENT

#define ADK_PAL_DP_XSTATS_ENABLE    0x01
#define ADK_PAL_DP_XSTATS_DISABLE   0x00

extern uint64_t adk_pal_rx_ts[ADK_MAX_NUM_CORES];
extern uint64_t *min_cycles;
#endif


#define ADK_PAL_DP_PROC_FN_PTR_TBL_INIT_DONE 0x01
#define ADK_PAL_DP_PROC_FN_PTR_TBL_INIT_NOT_DONE 0x00

/*
 * API to initilize PAL sub-module. 
 * Applicaiton should not call this API directly. 
 * This API is called by ADK sys-init.
 */
adk_status_t
adk_pal_init(adk_sys_handle_t    sys_handle);

/*
 * Applicaiton should not call this API directly. 
 * API to retrive PAL handle 
 */
adk_status_t
adk_pal_handle_get(adk_sys_handle_t sys_handle);


adk_status_t adk_pal_rsrc_notify_fn_cbfn
            (adk_module_handle_t mod_handle, 
             adk_sys_rsrc_oper_t oper,
             adk_sys_resource_type_t type,
             adk_sys_resource_info_t rsrc);

/*
 * API to add a worker-to-event-port map to PAL.
 */
adk_status_t
adk_pal_worker_to_event_port_map_add(
    adk_sys_handle_t        sys_handle,
    adk_pal_worker_info_t   *winfo_p);

/*
 * API to remove previously added worker-to-event-port map from PAL.
 */
adk_status_t
adk_pal_worker_to_event_port_map_rem(
    adk_sys_handle_t      sys_handle,
    adk_pal_worker_info_t *winfo_p);


/*
 *********************************************************
 * Dynamic pipeline related definitions
 *
 *********************************************************
 */

typedef enum {
	ADK_PAL_PG_NW_NIC_ARRIVAL,
	ADK_PAL_PG_SP_ARRIVAL,
	ADK_PAL_PG_BB_NIC_ARRIVAL,
	ADK_PAL_PG_CRYPTO_COP_RESP_ARRIVAL,
	ADK_PAL_PG_TIMER_COP_RESP_ARRIVAL,
	ADK_PAL_PG_MULTI_STAGE_HI,
	ADK_PAL_PG_MULTI_STAGE_MED,
	ADK_PAL_PG_MULTI_STAGE_LOW,
	ADK_PAL_PG_NW_NIC_SEND,
	ADK_PAL_PG_BB_NIC_SEND,
	ADK_PAL_PG_CRYPTO_COP_REQST_SEND,
} adk_pal_proc_group_id_e;



typedef struct {
	char                                fn_name[256];
        uint8_t                             module_id;
        adk_pal_proc_fn_id_e                fn_id;
        adk_pal_stage_lb_type_e             lb_type;
        adk_pal_proc_group_id_e             group_id;
	uint8_t                             priority;
} adk_pal_proc_func_cp_info_t;

/*List of processing functions added by each module
 * Each module addes all of it's dataplane stageable processing functions
 * 
 * this is a control plane table
 */
extern adk_pal_proc_func_cp_info_t adk_proc_fn_list[ADK_MAX_PROC_FUNC_ID];

typedef struct {
    adk_pal_proc_group_id_e     group_id;
    uint8_t                     event_queue_index;
    uint8_t                     pri_type;
} adk_pal_proc_group_info_t;

/* 
 * Per product 
 *  > processing groups to event-queue assignment
 *  > list of stageable processing functions
 */
#define ADK_PAL_USE_PKT_PRI     1
#define ADK_PAL_USE_STAGE_PRI   2


#define ADK_PAL_W_EVQ_INDX_0   0
#define ADK_PAL_W_EVQ_INDX_1   1
#define ADK_PAL_W_EVQ_INDX_2   2
#define ADK_PAL_W_EVQ_INDX_3   3
#define ADK_PAL_W_EVQ_INDX_4   4
#define ADK_PAL_W_EVQ_INDX_5   5
#define ADK_PAL_W_EVQ_INDX_6   6
#define ADK_PAL_W_EVQ_INDX_7   7

 /*
 * Stage Entry function prototype. 
 */
typedef void (*adk_pal_stage_entry_fn)(void  **task, uint8_t task_count, uint32_t task_mask, void *appData);
 
/*
 * ADK_CP_INTERNAL 
 * This is called by modules to receive traffic from a eth-dev
 * 
 */
adk_status_t 
adk_pal_stage_info_get(uint8_t fn_id, adk_pal_stage_info_t **info_p);

/*
 * ADK_CP_INTERNAL 
 * This is called by modules to get the the stage information for a given
 * processing function
 * 
 */
adk_status_t 
adk_pal_dp_stage_info_get(uint8_t fn_id, adk_pal_dp_stage_info_t **info_p);

/*
 * ADK_CP_INTERNAL 
 * This is called by modules to receive traffic from a eth-dev
 * 
 */
adk_status_t 
adk_pal_eth_dev_subscribe(uint16_t eth_dev_id,
                          uint8_t pkt_type,
                          uint8_t fn_id);

/*
 * ADK_CP_INTERNAL 
 * This is called by modules to un-scbscribe traffic from a eth-dev/pkt-type
 * 
 */
adk_status_t 
adk_pal_eth_dev_unsubscribe(uint16_t eth_dev_id,
                          uint8_t pkt_type);

uint32_t
adk_pal_dequeue_burst (
    uint8_t lcore_id,
    struct rte_mbuf *dpdk_mbuf_p[],
    uint8_t sub_event_id[],
    uint8_t burst_size,
    bool release_send);

/*
 *********************************************************
 * Dataplane stats related APIs, used only from utils
 *********************************************************
 */

/*
 * API to dump PAL dataplane stats
 */
adk_status_t
adk_pal_dp_stats_print(void);

/*
 * API to enable the PAL dataplane extended stats collection
 */
adk_status_t
adk_pal_dp_xstats_enable(void);

/*
 * API to disable the PAL dataplane extended stats collection
 */
adk_status_t
adk_pal_dp_xstats_disable(void);


#endif /* End of _ADK_PAL_INTERNAL_H_ */

/*!
 * end addtogroup _pal
 * @}
 */
