/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_dp.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _palFunctions  PAL Functions
 *
 * @{
 *
 * @brief
 *
 * This file contains all the APIs provided by the Pipeline Abstraction Layer (PAL)
 * data plane.
 *
 ***********************************************************************/

#include <rte_eventdev.h>
#include <adk_dpdk.h>
#include <adk_include.h>
#include "adk_sif_dp.h"
#include "adk_sys_lcore.h"
#include "adk_pal_dp_export.h"
#include "adk_pal_dp.h"
#include "adk_ethernet_export.h"
#include "adk_metadata.h"
#include <rte_bridge_eth.h>
#include <rte_bridge_crypto.h>
#include <adk_pmu.h>
#include <adk_rcu.h>
#include "adk_ip_dp_export.h"
#include "adk_ipv6_dp_export.h"
#include "adk_eth_dp.h"
#include "adk_ipsec_export.h"
#include "adk_ph_dev_dp.h"
#include "adk_shm.h"
#include "pdk_include.h"
#ifdef ADK_HOST_PATH_USE_NETD
#include <rte_bridge_cppi.h>
#include "adk_nd_common.h"
#include "adk_dev_net.h"
#endif

/* IPv4 and IPv6 Entry Points*/
#define ADK_EVENT_DEV_QUEUE_IP 1
#define ADK_EVENT_DEV_QUEUE_IPV6 2

/* Macro definition for module level cycle count */
#ifdef BLOCK_COUNT_ENABLED
    uint64_t pal_uli_rx_ts, pal_uli_tx_ts;
    uint64_t pal_lli_rx_ts, pal_lli_tx_ts;
#endif

/* Macro definition for ADK level cycle count */
#ifdef ADK_BLOCK_COUNT_ENABLED
    uint64_t adk_uli_rx_ts, adk_uli_tx_ts;
    uint64_t adk_lli_rx_ts, adk_lli_tx_ts;
#endif

/* PAL info   */
adk_pal_dp_info_t  *adk_pal_dp_info_p;


#ifdef ADK_PAL_CYCLE_MEASUREMENT
extern uint64_t *adk_pal_dp_min_cycles;
#endif

__thread adk_pal_dp_batch_info_t *adk_pal_batch_info_p = NULL;
/* 
 *  ADK_PAL_DP_INTERNAL
 * 
 * This error handler function. to handle invalid dispather-index.
 *
 */ 
adk_status_t
adk_pal_dp_dispather_error_handler(void **task_p, uint8_t task_count,
                         uint32_t task_mask, 
                         void *ptr1, 
                         adk_sif_cfg_pipeline_info_t *ptr2,
                         int coreIndex);

static inline void * mbuf_va2pa(struct rte_mbuf *m)
{

    if(m) { 
        return ((void*)((uint64_t)m - ((uint64_t)m->buf_addr - (uint64_t)m->buf_physaddr)));
    } else {
        return NULL;
    }
}


static inline void adk_pal_batch_proc_pkts_validate(void);
#if 0
static void adk_netd_md_dump(FILE *fp, struct rte_mbuf *dpdk_mbuf_p)
{

    struct adk_netd_pkt_md  *netd_md_p = (struct adk_netd_pkt_md*)                 \
                        ((uint8_t *)dpdk_mbuf_p + sizeof(struct rte_mbuf) + \
                                sizeof(struct rte_bridge_cppi_tx_metadata));

    rte_pktmbuf_dump(fp, dpdk_mbuf_p, 128);
    fprintf(fp, " NetDriver MD Info \n");
    fprintf(fp, " cookie  = 0x%X \n", netd_md_p->cookie);
    fprintf(fp, " nb_segs = %d\n", netd_md_p->nb_segs);
    fprintf(fp, " pkt_len = %d\n", netd_md_p->pkt_len);
    fprintf(fp, " data_len = %d\n", netd_md_p->data_len);
    fprintf(fp, " interface_id = %d\n", netd_md_p->interface_id);
    fprintf(fp, " linux_intf_id = %d\n", netd_md_p->linux_intf_id);
    fprintf(fp, " next = %p\n", netd_md_p->next);
    fprintf(fp, " msg_type= %d\n", netd_md_p->msg_type);
    fprintf(fp, " ts_index= %d\n", netd_md_p->ts_index);
    fprintf(fp, " ts_index= %d\n", netd_md_p->ts_index);
    fprintf(fp, " intf_ver_mj = %d\n", netd_md_p->intf_ver_mj);
    fprintf(fp, " intf_ver_mi = %d\n", netd_md_p->intf_ver_mi);
    return ;
}
#endif

/*!
 * =============================================================
 * adk_pal_dp_init
 *
 * @retval ADK_SUCCESS Successfully initialized the internal structures.
 * @retval ADK_E_RESOURCE_UNAVAILABLE Unable to get the resource.
 *
 * @brief  initilizes ADK PAL in the data-plane context.
 *
 * This function initializes the PAL in dataplane context .
 *
 * =============================================================
 */
adk_status_t
adk_pal_dp_init(void)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_pal_shm_config_t *shm_config_p = NULL;
    uint16_t fn_index = 0;

    /* Allocated shared memory for IPC between primary and secondary process */
    retValue = adk_shm_lookup(ADK_PAL_SHM_MEMZONE_NAME, (adk_shm_handle_t **)&shm_config_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("adk_pal_handle_get : IPC shared memory lookup failed.\n");
        return (ADK_E_MEM_UNAVAILABLE);
    }


    /* Restore PAL info pointer*/
    adk_pal_dp_info_p = (adk_pal_dp_info_t *)shm_config_p->adk_pal_dp_handle;

#ifdef ADK_PAL_CYCLE_MEASUREMENT
    /* Restore instrumentation pointers */
    adk_pal_dp_min_cycles     = shm_config_p->min_cycles;
#endif

    /* Fill up the look-up with error handler. do it only very first time of
     * adk_pal_dp_init() call. 
     * each module fills the module entry-funcitons at corresponding index
     * during adk_<module>_dp_init call
     * */
    /* Index zero is reserved for error condition checking */
    if(adk_pal_dp_info_p->dp_proc_fn_ptr_table_state == 
       ADK_PAL_DP_PROC_FN_PTR_TBL_INIT_NOT_DONE) {
        adkDestEntryFunction_g[0] =  NULL;
        for(fn_index = 1; fn_index < ADK_MAX_DP_SERVICE_FN_ID; fn_index++)
        {
            adkDestEntryFunction_g[fn_index] =  adk_pal_dp_dispather_error_handler;
        }
        adk_pal_dp_info_p->dp_proc_fn_ptr_table_state =  
            ADK_PAL_DP_PROC_FN_PTR_TBL_INIT_DONE;
    }

    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_ETH] =  adk_eth_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_IP] =  adk_ip_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_IPV6] =  adk_ipv6_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_IPSEC] =  adk_ipsec_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_PH] =  adk_ph_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_EM] =  adk_em_dp_pal_batch_cb_fn;
    adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_SIF] =  adk_sif_dp_pal_batch_cb_fn;

    adk_pal_dp_info_p->isUtilityCntEnabled = true;
    ADK_RETURN( retValue );
}    /* -----  end of function adk_pal_dp_init----- */

/*!
 * =============================================================
 * adk_pal_event_dispatcher
 * @param[in] event_index  event-index de-queued from cq.
 * @param[in] event_p event received from the scheduler.
 * @param[in] cbData_p The call back data that was registered
 * when this function was registered with the dispatcher.
 *
 * @retval None
 *
 * @brief  ADK event dispatcher function. calls corresponding
 * ADK dataplane entry function based arrived event.
 *
 * =============================================================
 */

int
adk_pal_event_dispatcher (
    uint8_t  event_dev_id,
    uint8_t  event_port_id,
    struct rte_event events[],
    int num,
    int rel_index,
    void *cb_data)
{
    ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ADK_ENTRY_TS();
    struct rte_mbuf *dpdk_mbuf_p = NULL;
    uintptr_t   event_ptr        = 0;
    uintptr_t   adk_task_ptr     = 0;
    uint32_t coreId;
    int i;
    uint8_t    disp_index = ADK_PAL_ERR_HANDLER_FUNC_ID;

#ifdef ADK_PAL_CYCLE_MEASUREMENT
    uint64_t   now;
#endif

    ADK_DP_MEASURE_BLOCK_CYCLE_ADK_LLI_RX_TS();
    RTE_SET_USED(event_dev_id);
    RTE_SET_USED(event_port_id);
    RTE_SET_USED(rel_index);
    RTE_SET_USED(cb_data);

    /*Current worker core id */
    coreId = ADK_LCORE_INDEX;

    if (unlikely(adk_pal_dp_info_p == NULL)) {
        debug_print("\n[%s()] PAL Uninitialized!\n", __func__);
        return 1;
    }

#if ADK_PERFORMANCE_STATS
      /* Increment batching stats - batch size for each dequeue */
    adk_pal_dp_info_p->stats[coreId].batch_stats[num]+=1;
    /*Total number of dequeue */
    adk_pal_dp_info_p->stats[coreId].total_num_of_dequeue+=num;
    rte_event_queue_dispatcher_get_stats(coreId, &adk_pal_dp_info_p->stats[coreId].user_stats);
    if(adk_pal_dp_info_p->stats[coreId].start_loop_count == 0)
	adk_pal_dp_info_p->stats[coreId].start_loop_count = adk_pal_dp_info_p->stats[coreId].user_stats.loop_count;
#endif

    if (unlikely(num == 0)) {
        debug_print("\n[%s()] ADK Dispatcher called with zero events !\n", __func__);
        return 1;
    }

    adk_pal_batch_info_p = ADK_PAL_DP_GET_BATCH_INFO_PTR(coreId);

    if(adk_pal_batch_info_p->procPkts != 0 ||
        adk_pal_batch_info_p->rxPkts != 0)
    {
        debug_print("Calling adk_pal_dp_batch_send from adk_pal_Event_dispatcher"
                "adkEMEventIndex %d, adkRteEventIndex %d, procPkts %d \n",
                adk_pal_batch_info_p->adkEmEventIndex,
                adk_pal_batch_info_p->adkRteEventIndex,
                adk_pal_batch_info_p->procPkts);
        adk_pal_dp_em_event_handle_return(0);
        adk_pal_dp_rte_event_handle_return_fail(0, adk_pal_batch_info_p->adkRteEventIndex);
        adk_pal_dp_info_p->stats[coreId].pal_batch_fail_safe_err ++;
        adk_pal_dp_info_p->stats[coreId].pal_batch_fail_safe_drop += adk_pal_batch_info_p->adkRteEventIndex;
        adk_pal_dp_info_p->stats[coreId].pal_batch_fail_safe_drop += adk_pal_batch_info_p->adkEmEventIndex;
    }

    adk_pal_batch_info_p->rxPkts = 0;
    adk_pal_batch_info_p->procPkts = 0;
    adk_pal_batch_info_p->adkEmEventIndex = 0;
    adk_pal_batch_info_p->adkRteEventIndex = 0;
    adk_pal_batch_info_p->eventMachineQueueId = 0;

    INCREMENT_PAL_UTILITY_COUNTER_COUNT(adk_pal_dp_info_p->stats[coreId].arrival_evnts, num);
    adk_pal_batch_info_p->rxPkts = num;
    debug_print("packets  %d recevived on adk_pal_event_dispatcher\n",adk_pal_batch_info_p->rxPkts);
    
    for (i=0; i<num; i++)
    {
#ifdef ADK_MEASURE_END_TO_END_CYCLES
        if(i != 0)
        {
            ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ADK_ENTRY_TS();
        }
#endif

        struct rte_event    *event_p = &events[i];
        struct rte_crypto_op *crypto_op_data;

#ifdef ADK_MBUF_PREFETCH 
        if (((i+1) < num) && 
            (RTE_EVENT_TYPE_CPU == events[i+1].event_type)){
            rte_prefetch1((events[i+1].mbuf)->cacheline0);
            rte_prefetch1((events[i+1].mbuf)->cacheline1);
            rte_prefetch1((void *)ADK_METADATA_PTR(events[i+1].mbuf));
            rte_prefetch1((uint8_t *)ADK_METADATA_PTR(events[i+1].mbuf) + 64);
        }
#endif

#if defined(ADK_MEASURE_CYCLE)
        ADK_MEASURE_CYCLE_PAL_START(&(adkMeasCycle_g[coreId][ADK_TERMINATION_STAGE]), 
                                &(adkMeasCycle_g[coreId][ADK_ROUTING_STAGE]));
#endif

        adk_pal_dp_event_decode(event_p, &disp_index);

#ifdef ADK_PAL_CYCLE_MEASUREMENT
        now =  rte_rdtsc();
#endif
        switch(event_p->event_type)
        {
            case RTE_EVENT_TYPE_ETHDEV :

                if(!(event_p->mbuf)) {
                  /* debug_print("\n[%s()] event received from eth-BD with NULL mbuf\n",__func__); */
                    adk_pal_batch_info_p->procPkts++;
                    adk_pal_dp_info_p->stats[coreId].nic_arrival_null_mbuf_evnts_drop++;
                    adk_pal_batch_proc_pkts_validate();
                    continue;
                }
    
                ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_RX_TS();
                /* Get the mbuf pointer from event */
                dpdk_mbuf_p = event_p->mbuf;
                rte_prefetch0((void *)(dpdk_mbuf_p));
                rte_prefetch0((void *)ADK_DSI_RX_METADATA_PTR(dpdk_mbuf_p));
                rte_prefetch0((void *)ADK_METADATA_PTR(dpdk_mbuf_p));
                rte_prefetch0(rte_pktmbuf_mtod(dpdk_mbuf_p, void *));

#ifdef ADK_DEVICE_SNR
#ifdef ADK_PAL_CYCLE_MEASUREMENT
#ifdef ADK_PAL_CYCLE_MEASUREMENT_MBUF_UDATA
#error "mbuf->udata64 overlaps with Ingress TN TS fix"
#endif
                /* store RIMON ingress timestamp in udata64. later retrived in
                 * send to linux path and set in Netd.MD */
                dpdk_mbuf_p->udata64    = dpdk_mbuf_p->timestamp;
                dpdk_mbuf_p->timestamp  = now;
#endif
#endif


                if(unlikely(adk_pal_dp_eth_dev_decode(dpdk_mbuf_p)!=ADK_SUCCESS))
                {
                    disp_index = ADK_PAL_ERR_HANDLER_FUNC_ID;
                }
                adk_task_ptr = (uintptr_t )dpdk_mbuf_p;
                break;
            case RTE_EVENT_TYPE_CPU :
                dpdk_mbuf_p = event_p->mbuf;
                adk_task_ptr = (uintptr_t )dpdk_mbuf_p;
                break;
#if ADK_DPDK_VERSION >= 18050016
            case RTE_EVENT_TYPE_TIMER:
#else 
            case RTE_EVENT_TYPE_TIMERDEV:
#endif
                debug_print("Received Timer events\n");
                adk_pal_batch_info_p->procPkts++;
                /* Get the event pointer from event */
                event_ptr = (uintptr_t)event_p->event_ptr;
                INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[coreId].timer_arrival_events);
                adk_task_ptr = event_ptr;
                break;
#ifdef ADK_HOST_PATH_USE_NETD
            case RTE_EVENT_TYPE_CPPIDEV :
                /* Get the mbuf pointer from event */
                dpdk_mbuf_p = event_p->mbuf;
                rte_prefetch0((void *)ADK_METADATA_PTR(dpdk_mbuf_p));
                rte_prefetch0(rte_pktmbuf_mtod(dpdk_mbuf_p, void *));

                INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[coreId].netd_arrival_events);;
#ifdef ADK_PAL_CYCLE_MEASUREMENT
                dpdk_mbuf_p->timestamp  = now;
#endif
                adk_task_ptr = (uintptr_t )dpdk_mbuf_p;
                break;
#endif /* ADK_HOST_PATH_USE_NETD */
            case RTE_EVENT_TYPE_CRYPTODEV:
                /* Get the mbuf pointer from event */
                /* Initialize ADK meta data.  */
                /* adk_pkt_metadata_init((adk_pktio_task_t *) dpdk_mbuf_p, ADK_TN_PORT_DOMAIN, 0);*/

                /* as of now mbuf udata is been used to carry function-idex,
                 * need to be changed once crypto-bd enhancement is ready */
                disp_index = event_p->sub_event_type;
                INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[coreId].crypto_arrival_events);
                crypto_op_data = (struct rte_crypto_op *)event_p->event_ptr;
                dpdk_mbuf_p = crypto_op_data->sym->m_src;
                adk_task_ptr = (uintptr_t)event_p->event_ptr;;
                rte_prefetch0((void *)(dpdk_mbuf_p));
                rte_prefetch0((void *)ADK_METADATA_PTR(dpdk_mbuf_p));
                rte_prefetch0(rte_pktmbuf_mtod(dpdk_mbuf_p, void *));

                break;
            default:
                debug_print("\n[%s()] Unsupported event type %d \n",__func__, event_p->event_type );
                return 0;
                break;
        }

        if(dpdk_mbuf_p != NULL) { /* Timer Events may not allocate mbuf all the times */
            /* Fill in the Event Q and Lock ID from Event Struct to the ADK metadata */
            ADK_METADATA_PARAM(dpdk_mbuf_p, event_queue_id) = event_p->queue_id;
            ADK_METADATA_PARAM(dpdk_mbuf_p, event_sched_type) = event_p->sched_type;
            ADK_METADATA_PARAM(dpdk_mbuf_p, lock_id) = event_p->flow_id;
            ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) = ADK_TASK_TYPE_FWD;
            ADK_METADATA_PARAM(dpdk_mbuf_p, proc_stage_idx) = disp_index;
#ifdef ADK_PERFORMANCE_STATS
            for (uint8_t j = 0; j < ADK_MAX_NUM_OF_FLOW_SUPPORT; j ++) {
		if(adk_pal_dp_info_p->stats[coreId].num_flows[j].flowid == dpdk_mbuf_p->hash.rss) {
			adk_pal_dp_info_p->stats[coreId].num_flows[j].flow_count ++;
			break;
		} else if(adk_pal_dp_info_p->stats[coreId].num_flows[j].flowid == 0 ) {
			adk_pal_dp_info_p->stats[coreId].num_flows[j].flowid = dpdk_mbuf_p->hash.rss;
			adk_pal_dp_info_p->stats[coreId].num_flows[j].flow_count ++;
			break;
		}
	   }
#endif
#ifdef ADK_PAL_CYCLE_MEASUREMENT
#ifdef ADK_PAL_CYCLE_MEASUREMENT_MBUF_UDATA
            dpdk_mbuf_p->udata64 = now;
#else 
            ADK_METADATA_PARAM(dpdk_mbuf_p, stage_rx_ts) = now;
#endif
#endif
            /*Initialize the pktPath to ZERO */
            (ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreId))->pktPath = ADK_E_FLOW_CACHE_PKT_SLOW_PATH;
            (ADK_SIF_DP_GET_SCRATCHPAD_PTR(coreId))->flow_cache_entry = NULL;
        }

        ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_TX_TS();
        (adkDestEntryFunction_g[disp_index])((void **)&adk_task_ptr, 1, 0x1, NULL, NULL, coreId);
    }

    return 0;
}    /* -----  end of function adk_pal_event_dispatcher  ----- */


/*!
 * =============================================================
 * adk_pal_nic_entry_fn
 * @param[in] dpdk_mbuf_p mbuf received from ADK event dispatcher.
 * @param[in] coreId logical core id of the current worker.
 *
 * @retval None
 *
 * @brief  Hands over the event to the ethernet entry function.
 *
 * This function calls the ethernet entry function.
 * It assumes that the mbuf(packet) is created with priviate area of
 * atleast ADK_META_DATA_SIZE and ADK metadata offset starts
 * at ADK_META_DATA_OFFSET.
 *
 * =============================================================
 */
void
adk_pal_nic_entry_fn (
    struct rte_mbuf *dpdk_mbuf_p,
    uint32_t coreId)
{

    /*
     * Call the ethernet dataplane entry function.
     */
    ADK_DP_MEASURE_BLOCK_CYCLE_PAL_LLI_TX_TS();
    adkDestEntryFunction_g[ADK_ETH_LLI_FN_ID]((void **)&dpdk_mbuf_p, 1, 0x1, NULL, NULL, coreId);

    return;
}    /* -----  end of function adk_pal_nic_entry_fn  ----- */

/*!
 * =============================================================
 * adk_pal_veth_entry_fn
 * @param[in] dpdk_mbuf_p mbuf received from ADK event dispatcher.
 * @param[in] coreId logical core id of the current worker.
 *
 * @retval None
 *
 * @brief  Hands over the event to the ethernet entry function.
 * @brief  Hands over the mbuf(packet) to the vEth entry function.
 *
 * This function calls the vEth entry function.
 * It assumes that the mbuf(packet) is created with priviate area of
 * atleast ADK_META_DATA_SIZE and ADK metadata offset starts
 * at ADK_META_DATA_OFFSET.
 *
 * =============================================================
 */

void
adk_pal_veth_entry_fn (
    struct rte_mbuf *dpdk_mbuf_p,
    uint32_t coreId)
{

    /* Adjust the input port number vEth to NIC port mapping */

    dpdk_mbuf_p->port = adk_pal_dp_info_p->eth_dev_info[dpdk_mbuf_p->port].port_id;

    /*
     * Call the ethernet dataplane entry(Linux) function.
     */
    adkDestEntryFunction_g[ADK_ETH_ULI_NETD_FN_ID]((void **)&dpdk_mbuf_p, 1, 0x1, NULL, NULL, coreId);

    return;
}    /* -----  end of function adk_pal_veth_entry_fn  ----- */

/*!
 * =============================================================
 * adk_pal_nic_event_prepare
 * @param[in] out_port Output port on which the packet has to be
 * sent out.
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent out.
 * @param[in] ev rte_event that needs to be sent out.
 *
 * @retval ADK_E_INTERNAL_ERROR on error.
 * @retval ADK_SUCCESS on success.
 *
 * @brief   prepare the rte_event that will trasmitted on output port.
 *
 * This function prepare the event to be trasmittted on nic. 
 * If API returns ADK_E_INTERNAL_ERROR, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 * =============================================================
 */
adk_dp_status_t
adk_pal_nic_event_prepare(
        uint8_t   out_port,
        uint8_t pktColor,
        uint16_t dev_tx_queue_id,
        struct rte_mbuf *dpdk_mbuf_p,
        struct rte_event *ev)
{
    /* Array out of bound check */
    if( out_port >= ADK_PAL_MAX_NUM_NIC_PORTS )
    {
        return ADK_E_INTERNAL_ERROR;
    }

    ADK_PKTMBUF_COLOR(dpdk_mbuf_p) = pktColor;

    /* Get the packet out queue  and set in mbuf*/
    ev->queue_id = adk_pal_dp_info_p->eth_port_to_event_queue_id[out_port];
    ev->event_type     = RTE_EVENT_TYPE_CPU;
    ev->mbuf           = dpdk_mbuf_p;
    ev->priority       = 0;
    ev->sched_type     = RTE_SCHED_TYPE_ATOMIC;

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        ev->op             = RTE_EVENT_OP_FORWARD;
    } else
    {
        ev->op             = RTE_EVENT_OP_NEW;
    }
    {
        /*Fill EBD TX MD in mbuf */
        dpdk_mbuf_p->port = out_port;
        rte_pktmbuf_txq_id(dpdk_mbuf_p) = dev_tx_queue_id;
    }

#ifdef ADK_PAL_CYCLE_MEASUREMENT
    /*tx time stamp and generate the time/cycle diff*/
    adk_sys_dp_proc_cycle_measure(dpdk_mbuf_p);
#endif

    rte_bridge_eth_tx_event_meta_set(dpdk_mbuf_p, 0, NULL);
    return ADK_SUCCESS;
}

static inline void adk_pal_batch_rte_enqueue_size_validate(void)
{
    if(adk_pal_batch_info_p->adkRteEventIndex == ADK_PAL_MAX_ENQUEUE_SIZE)
    {
        adk_pal_dp_batch_send(0, adk_pal_batch_info_p->adkRteEventIndex);

        adk_pal_batch_info_p->adkRteEventIndex = 0;
    }
}

static inline void adk_pal_batch_rte_event_cb_update(
        uint8_t cbFn,
        uint8_t cbEventType,
        void *cbParam)
{
    adk_pal_batch_info_p->adkRteEventRet[adk_pal_batch_info_p->adkRteEventIndex].cbFn = cbFn;
    adk_pal_batch_info_p->adkRteEventRet[adk_pal_batch_info_p->adkRteEventIndex].cbEventType = cbEventType;
    adk_pal_batch_info_p->adkRteEventRet[adk_pal_batch_info_p->adkRteEventIndex].cbParam = cbParam;

    adk_pal_batch_info_p->adkRteEventIndex++;
}

static inline void adk_pal_batch_proc_pkts_validate(void)
{
    if(adk_pal_batch_info_p->rxPkts ==
            adk_pal_batch_info_p->procPkts)
    {
        adk_pal_dp_batch_send(adk_pal_batch_info_p->adkEmEventIndex, adk_pal_batch_info_p->adkRteEventIndex);

        adk_pal_batch_info_p->adkEmEventIndex = 0;
        adk_pal_batch_info_p->adkRteEventIndex = 0;
        adk_pal_batch_info_p->procPkts = 0;
        adk_pal_batch_info_p->rxPkts = 0;
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_TX_TS();
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_TX_TS();
    }
#ifdef ADK_MEASURE_END_TO_END_CYCLES
    else
    {
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_TX_TS();
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_TX_TS();
    }
#endif
}
/*!
 * =============================================================
 * adk_pal_dp_nic_send
 * @param[in] out_port Output port on which the packet has to be
 * sent out.
 * @param[in] queue_id transmit queue-id.
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent out.
 *
 * @retval Number of packets sent out.
 *
 * @brief  Sends the packet on the output port.
 *
 * This function sends the packet out on bridge-dev direct queue.
 * It returns the number of packets sent out. If the number of
 * packets sent out is not equal to 1, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_nic_send (
        uint8_t   out_port,
        uint8_t pktColor, 
        uint16_t  tx_queue_id,
        struct rte_mbuf *dpdk_mbuf_p,
        uint8_t cbFn,
        void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;
    struct rte_event *ev;

    adk_pal_batch_rte_enqueue_size_validate();

    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    ret = adk_pal_nic_event_prepare(out_port, pktColor, tx_queue_id, dpdk_mbuf_p, ev);
    if(ret != ADK_SUCCESS)
    {
        return ADK_E_INTERNAL_ERROR;
    }

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
    }

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_NIC, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_nic_events);
 
    adk_pal_batch_proc_pkts_validate();

    return ret;
}

/** Convert from EM event to env_mbuf */
static inline env_mbuf_t *
event_to_mbuf(em_event_t event)
{
    return (env_mbuf_t *)((uintptr_t)event - sizeof(env_mbuf_t));
}

void adk_pal_dp_em_event_handle_return(uint8_t retIndex)
{
    /*Handle Success Case*/
    for(int i = 0 ; i < retIndex; i ++)
    {
        debug_print("adk_pal_dp_em_event_handle_return success\n");
        debug_print("adk_pal_dp_em_event_handle_return cbEventType %d\n",adk_pal_batch_info_p->adkEmEventRet[i].cbEventType);
        debug_print("adk_pal_dp_em_event_handle_return cbFn %d\n",adk_pal_batch_info_p->adkEmEventRet[i].cbFn);
       adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[adk_pal_batch_info_p->adkEmEventRet[i].cbFn](
                            NULL,
                            adk_pal_batch_info_p->adkEmEventRet[i].cbEventType,
                            adk_pal_batch_info_p->adkEmEventRet[i].cbParam,
                            true);
    }
    /*Handle failure Case*/
    for(;retIndex < adk_pal_batch_info_p->adkEmEventIndex;retIndex++)
    {
        debug_print("adk_pal_dp_em_event_handle_return failure %d\n", retIndex);
        debug_print("adk_pal_dp_em_event_handle_return failure %d\n", adk_pal_batch_info_p->adkEmEventIndex);
        adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].em_xmt_fail_events++;
        adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[adk_pal_batch_info_p->adkEmEventRet[retIndex].cbFn](
                env_mbuf_to_rte_mbuf(event_to_mbuf(adk_pal_batch_info_p->adkEmEvent[retIndex])),
                adk_pal_batch_info_p->adkEmEventRet[retIndex].cbEventType,
                adk_pal_batch_info_p->adkEmEventRet[retIndex].cbParam,
                false);
    }
}

void adk_pal_dp_rte_event_handle_return_success(uint8_t eventIndex, uint8_t eventCount)
{
    /*Handle Success Case*/
    for(; eventIndex < eventCount; eventIndex++)
    {   
        debug_print("adk_pal_dp_rte_event_handle_return success eventIndex %d,eventCount %d\n",eventIndex,eventCount);
        debug_print("adk_pal_dp_rte_event_handle_return cbEventType %d\n",adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbEventType);
        debug_print("adk_pal_dp_rte_event_handle_return cbFn %d\n",adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbFn);
        adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbFn](
                NULL,
                adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbEventType,
                adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbParam,
                true);
    }
}

void adk_pal_dp_rte_event_handle_return_fail(uint8_t eventIndex, uint8_t eventCount)
{
    /*Handle Success Case*/
    for(; eventIndex < eventCount; eventIndex++)
    {
        debug_print("adk_pal_dp_rte_event_handle_return failure\n");
        adk_pal_dp_info_p->adk_pal_dp_batch_cb_fn_g[adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbFn](
                adk_pal_batch_info_p->adkRteEvents[eventIndex].mbuf,
                adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbEventType,
                adk_pal_batch_info_p->adkRteEventRet[eventIndex].cbParam,
                false);
    }
}

adk_status_t adk_pal_dp_batch_send(uint8_t emEventCount, uint8_t rteEventCount)
{
    adk_status_t retVal =  ADK_E_TASK_CONSUMED;
    uint32_t ret = 0;

    if(emEventCount)
    {
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_INTER_TX_TS();
        debug_print("\n calling em_send_multi for %d events \n",adk_pal_batch_info_p->adkEmEventIndex);
        ret = em_send_multi(adk_pal_batch_info_p->adkEmEvent, 
                emEventCount,
                adk_pal_batch_info_p->eventMachineQueueId);
        ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS(); 
        if (ret != adk_pal_batch_info_p->adkEmEventIndex) {
            ADK_DP_LOG_ERROR_2D(ADK_SYS_MODULE_ID,
                    "em_send_multi() failed! status: pkts send %d out of %d \n", ret, emEventCount);
        }
        INCREMENT_PAL_UTILITY_COUNTER_COUNT(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].em_xmt_events, ret);
        adk_pal_dp_em_event_handle_return(ret);
    }
    if(rteEventCount)
    {
        /* Send the packet out */
       uint8_t retry_count = ADK_PAL_BATCH_ENQUEUE_BURST_RETRY_COUNT; 
       uint8_t eventIndex = 0;
       debug_print("\n calling rte_event_enqueue_burst for %d events \n",adk_pal_batch_info_p->adkRteEventIndex);
       ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_INTER_TX_TS();
       do{
            ret = rte_event_enqueue_burst(adk_pal_dp_info_p->event_dev_id, 
                adk_pal_dp_info_p->evdev_port_ids[ADK_LCORE_INDEX],
                &adk_pal_batch_info_p->adkRteEvents[eventIndex],
                rteEventCount);
            ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS();
            if(ret == rteEventCount)
            {
                debug_print("enqueue %d event in batch out of %d events\n", ret, rteEventCount);
                INCREMENT_PAL_UTILITY_COUNTER_COUNT(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].rte_xmt_events, ret);
                adk_pal_dp_rte_event_handle_return_success(eventIndex, ret);
                retry_count = 0;
                return retVal;
            }
            else if((ret > 0) && (ret < rteEventCount))
            {
                debug_print("Failed to enqueue %d event in batch out of %d events\n", ret, rteEventCount);
                INCREMENT_PAL_UTILITY_COUNTER_COUNT(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].rte_xmt_events, ret);
                adk_pal_dp_rte_event_handle_return_success(eventIndex, ret);
                rteEventCount -= ret;
                eventIndex += ret;
                retry_count = ADK_PAL_BATCH_ENQUEUE_BURST_RETRY_COUNT;
            }
            else if(ret == 0 && (retry_count))
            {
               debug_print("Enqueue retry\n");
               retry_count--;
            }
            if(ret == 0 && (retry_count))
            {
                debug_print("Enqueue Failed\n");
                adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].rte_xmt_fail_events += rteEventCount;
                adk_pal_dp_rte_event_handle_return_fail(eventIndex, rteEventCount);
            }
            ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_INTER_TX_TS();
        }while(retry_count);
    }
    return retVal;
}

adk_status_t
adk_pal_dp_stage_send(
        void  **task,
        uint8_t task_count,
        uint8_t task_mask,
        uint8_t  dest_fn_id,
        uint32_t flow_id,
        uint8_t  pkt_priority,
        uint8_t cbFn,
        uint8_t cbEventType,
        void *cbParam)
{

    adk_status_t ret = ADK_E_TASK_CONSUMED;
    adk_pal_dp_stage_info_t *dp_stage_info_p;
    struct rte_event *ev;

    debug_print("adk_pal_dp_stage_send \n");
    dp_stage_info_p = &(adk_pal_dp_info_p->dp_stage_info[dest_fn_id]);
    if(unlikely(dp_stage_info_p == NULL))
    {
        ADK_DP_LOG_ERROR_1D(ADK_SYS_MODULE_ID,
                "could not get event Queue related stage info for FN_ID %d\n",dest_fn_id);
        return ADK_E_INTERNAL_ERROR;
    }

    if(dp_stage_info_p->event_q_id == -1) {
        (adkDestEntryFunction_g[dp_stage_info_p->fn_id])
            (task, task_count, task_mask, NULL, NULL, ADK_LCORE_INDEX);
        return ADK_SUCCESS;
    }

    adk_pal_batch_rte_enqueue_size_validate();

    /*encode next-stage-event */
    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    adk_pal_dp_event_encode(ev, dp_stage_info_p->event_q_id, pkt_priority, dp_stage_info_p);

    /* Get the packet out queue  and set in mbuf*/
    ev->queue_id       = dp_stage_info_p->event_q_id;
    ev->event_type     = RTE_EVENT_TYPE_CPU;
    ev->mbuf           = (struct rte_mbuf *)task[0];
    ev->priority       = pkt_priority;
    ev->flow_id        = flow_id;
    ev->sched_type     = dp_stage_info_p->lb_type;

    if(ADK_METADATA_PARAM(task[0], task_type) == ADK_TASK_TYPE_FWD)
    {
        ev->op             = RTE_EVENT_OP_FORWARD;
        adk_pal_batch_info_p->procPkts++;
    }
    else
    {
        ev->op             = RTE_EVENT_OP_NEW;
    }
    adk_pal_batch_rte_event_cb_update(cbFn, cbEventType, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_stage_events);
    adk_pal_batch_proc_pkts_validate();

    ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_TX_TS();
    ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_TX_TS();

    return ret;
}

/*!
 * =============================================================
 * adk_pal_dp_em_send
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent out.
 * @param[in] eventDev Queue ID.
 *
 * @retval ADK_SUCCESS.
 *
 * @brief  Sends the packets to EM.
 *
 *This function will be buffering the packets and send the to EM as batch
 * =============================================================
 */
adk_status_t
adk_pal_dp_em_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint32_t eventMachineQueueId,
    uint8_t cbFn,
    void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;

    if(((adk_pal_batch_info_p->eventMachineQueueId != 0) &&
        (adk_pal_batch_info_p->eventMachineQueueId != eventMachineQueueId)) ||
        (adk_pal_batch_info_p->adkEmEventIndex == ADK_PAL_MAX_ENQUEUE_SIZE))
    {
        debug_print("Event Machine QueueID is different\n");
        ret = adk_pal_dp_batch_send(adk_pal_batch_info_p->adkEmEventIndex, 0);

        adk_pal_batch_info_p->adkEmEventIndex = 0;
    }
    adk_pal_batch_info_p->eventMachineQueueId = eventMachineQueueId;

    adk_pal_batch_info_p->adkEmEvent[adk_pal_batch_info_p->adkEmEventIndex] = 
        em_rte_mbuf_to_event((struct rte_mbuf *)(dpdk_mbuf_p));

    adk_pal_batch_info_p->adkEmEventRet[adk_pal_batch_info_p->adkEmEventIndex].cbFn = cbFn;
    adk_pal_batch_info_p->adkEmEventRet[adk_pal_batch_info_p->adkEmEventIndex].cbEventType = ADK_PAL_DP_BATCH_CB_EVENT_EM;
    adk_pal_batch_info_p->adkEmEventRet[adk_pal_batch_info_p->adkEmEventIndex].cbParam = cbParam;
    
    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
    }
    adk_pal_batch_info_p->adkEmEventIndex++;

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_em_events);
    adk_pal_batch_proc_pkts_validate();

    return ret; 
}

#ifdef ADK_HOST_PATH_USE_NETD
uint32_t
adk_pal_netd_send_prepare (
    struct rte_mbuf *dpdk_mbuf_p, struct rte_event *ev)
{
    uint32_t numOfPktsPrepared = 0;
#if RDK_VERSION < 0x10E00
    struct rte_bridge_cppi_tx_metadata  *cppi_tx_params_p;
#endif
    uint16_t linux_intf_id = USHRT_MAX;
    /* adk_netd_metadata is at the beginning of payload */
    struct adk_netd_pkt_md *netd_md_p;
    struct adk_netd_pkt_md *t_netd_md_p;
    struct rte_mbuf *t_mbuf_p;

    if(!dpdk_mbuf_p) {
        debug_print("%s() mbuf null\n",__func__);
        goto __adk_pal_netd_send_prepare_invalid_param;
    }
    
    linux_intf_id = ADK_MD_GET_LINUX_INTF_ID(dpdk_mbuf_p);

    if(linux_intf_id >=  ADK_PAL_MAX_LINUX_INTF_ID) {
        debug_print("%s() Error: invalid linux intf id %u\n",
                    __func__, linux_intf_id);
        goto __adk_pal_netd_send_prepare_invalid_param;
    }

    if(adk_pal_dp_info_p->linux_intf_id_to_evnt_qid[linux_intf_id] == -1)
    {
        debug_print("%s() Error: event queue invalid \n", __func__);
        goto __adk_pal_netd_send_prepare_invalid_param;
    }

    /* Get the event queue corresponding to Netd port*/
    ev->queue_id       =
        adk_pal_dp_info_p->linux_intf_id_to_evnt_qid[linux_intf_id];
    ev->event_type     = RTE_EVENT_TYPE_CPU;
    ev->mbuf           = dpdk_mbuf_p;
    ev->priority       = 0;
    ev->sched_type     = RTE_SCHED_TYPE_ATOMIC;
    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD) {
        ev->op         = RTE_EVENT_OP_FORWARD;
    } else { 
        ev->op         = RTE_EVENT_OP_NEW;
    }

    /* Fill netd metadata in mbuf private area */
    ADK_NETD_METADATA_PTR(dpdk_mbuf_p, netd_md_p);
    debug_print("\nphy_addr = %lx, buf_addr=%p, mbuf=%p, data_offset=%d, netd_md_p=%p\n",
           dpdk_mbuf_p->buf_physaddr, dpdk_mbuf_p->buf_addr, dpdk_mbuf_p,
           dpdk_mbuf_p->data_off, (void *)netd_md_p);

    /* Fill up the netdriver metadata */
    netd_md_p->pkt_len = dpdk_mbuf_p->pkt_len;
    netd_md_p->data_len = dpdk_mbuf_p->data_len;
    netd_md_p->interface_id = ADK_PACKET_INPUT_PORT(dpdk_mbuf_p);
    netd_md_p->linux_intf_id = linux_intf_id;
    netd_md_p->nb_segs = 1;
    netd_md_p->cookie = ADK_NETD_MD_COOKIE;
    netd_md_p->intf_ver_mj = ADK_NETD_FP_INTF_VERSION_MAJOR;
    netd_md_p->intf_ver_mi = ADK_NETD_FP_INTF_VERSION_MINOR;

    netd_md_p->correlator     = ADK_MD_GET_TS_CORR(dpdk_mbuf_p);
    netd_md_p->msg_type       = ADK_MD_GET_TS_MSG_TYPE(dpdk_mbuf_p);
    netd_md_p->ts_index       = ADK_MD_GET_TS_INDEX(dpdk_mbuf_p);

#ifdef ADK_DEVICE_SNR 
    if (dpdk_mbuf_p->ol_flags & PKT_RX_TIMESTAMP)
        netd_md_p->timestamp      = ((dpdk_mbuf_p->udata64 << 1 ) | 0x1);
    else 
        netd_md_p->timestamp      = 0x00;
#else 
    netd_md_p->timestamp      = dpdk_mbuf_p->timestamp;
#endif
    if (dpdk_mbuf_p->next) {
        t_mbuf_p              = dpdk_mbuf_p->next;
        netd_md_p->next       = mbuf_va2pa(t_mbuf_p);
    } else {
        netd_md_p->next       = NULL;
        t_mbuf_p = NULL;
    }

    /* Fill up the CPPI BD metadata */
#if RDK_VERSION < 0x10E00
    cppi_tx_params_p = (struct rte_bridge_cppi_tx_metadata *)                 \
                          ((uint8_t *)dpdk_mbuf_p + sizeof(struct rte_mbuf) + \
                                ADK_CPPI_BD_MD_OFFSET);
#endif

#ifdef ADK_QOS_GEN3_INTERFACE
#if RDK_VERSION >= 0x10E00
    rte_pktmbuf_txq_id(dpdk_mbuf_p) = ADK_MD_GET_DEV_TX_QUEUE_ID(dpdk_mbuf_p);
    dpdk_mbuf_p->port = (ADK_MD_GET_ADK_DEV_ID(dpdk_mbuf_p)
                        - ADK_DEV_NET_ID_BASE
                        + ADK_PH_DEV_MAX_NUM_DEVICES);
#else
    cppi_tx_params_p->cppi_queue_id = ADK_MD_GET_DEV_TX_QUEUE_ID(dpdk_mbuf_p);
    cppi_tx_params_p->cppi_dev_id =  (ADK_MD_GET_ADK_DEV_ID(dpdk_mbuf_p)
                                     - ADK_DEV_NET_ID_BASE
                                     + ADK_PH_DEV_MAX_NUM_DEVICES);
#endif
#else
#if RDK_VERSION >= 0x10E00
    rte_pktmbuf_txq_id(dpdk_mbuf_p) = 0;
    dpdk_mbuf_p->port = ADK_PH_DEV_MAX_NUM_DEVICES;
#else
    cppi_tx_params_p->cppi_queue_id = 0;
    cppi_tx_params_p->cppi_dev_id = ADK_PH_DEV_MAX_NUM_DEVICES;
#endif
#endif

    /* to support multi-segment packets */
    while(t_mbuf_p) {
        ADK_NETD_METADATA_PTR(t_mbuf_p, t_netd_md_p);
        t_netd_md_p->data_len = t_mbuf_p->data_len;
        t_netd_md_p->cookie   = ADK_NETD_MD_COOKIE;
        t_mbuf_p              = t_mbuf_p->next;
        if (t_mbuf_p) {
            t_netd_md_p->next     =  mbuf_va2pa(t_mbuf_p);
        } else {
            t_netd_md_p->next     = NULL;
        }
        netd_md_p->nb_segs += 1;
    }

    debug_print("ADK: to netd: ev queue_id %u "
            "dpdk_mbuf_p %p  netd_md %p "
            "pkt_len %u data_len %u nb_segs %u ",
            ev->queue_id, dpdk_mbuf_p, netd_md_p, netd_md_p->pkt_len,
            netd_md_p->data_len, netd_md_p->nb_segs);
    numOfPktsPrepared++;
__adk_pal_netd_send_prepare_invalid_param:
    return numOfPktsPrepared;

}
/*!
 * =============================================================
 * adk_pal_netd_send
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent to
 * adk netdriver interface.
 *
 * @retval Number of packets sent out.
 *
 * @brief  Sends the packet on the adk netdriver interface.
 *
 * It is assumed that there is a adk netdriver interface created corresponding
 * to the input port. This function gets the input port from the DPDK
 * mbuf. It looks up the packet out queue corresponding to the
 * adk netdriver interface and sends out the packet on that queue.
 * It returns the number of packets sent out. If the number of
 * packets sent out is not equal to 1, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_netd_send(
        struct rte_mbuf *dpdk_mbuf_p,
        uint8_t cbFn,
        void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;
    uint32_t numOfPktsPrepared = 0;
    struct rte_event *ev;

    adk_pal_batch_rte_enqueue_size_validate();

    /*encode next-stage-event */
    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    numOfPktsPrepared = adk_pal_netd_send_prepare(dpdk_mbuf_p, ev);
    if (!numOfPktsPrepared)
    {
        return ADK_E_INTERNAL_ERROR;
    }
    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
    }

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_NETD, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_netd_events);

    adk_pal_batch_proc_pkts_validate();

    return ret;
}    /* -----  end of function adk_pal_netd_send  ----- */
#endif /* ADK_HOST_PATH_USE_NETD */

/*!
 * =============================================================
 * adk_pal_ph_send
 * @param[in] outPort Output port on which the packet has to be
 * sent out.
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent out.
 *
 * @retval Number of packets sent out.
 *
 * @brief  Sends the packet on the output port.
 *
 * This function sends the packet out on bridge-dev direct queue.
 * It returns the number of packets sent out. If the number of
 * packets sent out is not equal to 1, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 * =============================================================
 */
adk_status_t 
adk_pal_dp_ph_send (
    uint8_t evd_queue_id,
    uint8_t sub_event_id,
    uint8_t priority,
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;
    struct rte_event *ev;


    debug_print("adk_pal_dp_ph_send \n");
    adk_pal_batch_rte_enqueue_size_validate();

    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    /* Get the packet out queue  and set in mbuf*/
    ev->queue_id = evd_queue_id;
    ev->event_type     = RTE_EVENT_TYPE_CPU;
    ev->sub_event_type  = sub_event_id;
    ev->mbuf           = dpdk_mbuf_p;
    ev->priority       = priority;
    ev->sched_type     = RTE_SCHED_TYPE_ATOMIC;

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
        ev->op             = RTE_EVENT_OP_FORWARD;
    } else
    {
        ev->op             = RTE_EVENT_OP_NEW;
    }
    /* Set the flow ID for atomic queues */
    ev->flow_id = ADK_METADATA_PARAM(dpdk_mbuf_p, pdu_flow_id);

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_PH, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_ph_events);

    debug_print("adk_pal_dp_ph_send adkRteEventIndex %d,adk_pal_batch_info_p->procPkts %d \n",
                            adk_pal_batch_info_p->adkRteEventIndex,adk_pal_batch_info_p->procPkts);
    adk_pal_batch_proc_pkts_validate();

    return ret;
}    /* -----  end of function adk_pal_ph_send  ----- */

/*!
 * =============================================================
 * adk_pal_dp_release_send
 * @param[in] outPort Output port on which the packet has to be
 * sent out.
 * @param[in] dpdk_mbuf_p DPDK mbuf that needs to be sent out.
 *
 * @retval Number of packets sent out.
 *
 * @brief  Sends the packet on the output port.
 *
 * This function sends the packet out on bridge-dev direct queue.
 * It returns the number of packets sent out. If the number of
 * packets sent out is not equal to 1, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 *
 * This function sends the packet out on bridge-dev direct queue.
 * It returns the number of packets sent out. If the number of
 * packets sent out is not equal to 1, it is the responsibility
 * of the caller to free the packet and increment any statistics.
 *
 * =============================================================
 */
adk_status_t
adk_pal_dp_release_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam)
{
    adk_status_t ret = ADK_SUCCESS;
    struct rte_event *ev;

    debug_print("adk_pal_dp_release_send \n");
    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_NEW)
    {
        return ADK_SUCCESS;
    }
    adk_pal_batch_rte_enqueue_size_validate();

    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    /* Get the packet out queue  and set in mbuf*/
    ev->queue_id = ADK_METADATA_PARAM(dpdk_mbuf_p, event_queue_id);
    ev->event_type     = RTE_EVENT_TYPE_CPU;
    ev->sub_event_type  = 0;
    ev->mbuf           = NULL;
    ev->priority       = 0;
    ev->sched_type     = ADK_METADATA_PARAM(dpdk_mbuf_p, event_sched_type);

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
        ev->op             = RTE_EVENT_OP_RELEASE;
    } 
    /* Set the flow ID for atomic queues */
    ev->flow_id = ADK_METADATA_PARAM(dpdk_mbuf_p, pdu_flow_id);

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_RELEASE, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_release_events);
    
    adk_pal_batch_proc_pkts_validate();

    if(ret == ADK_E_TASK_CONSUMED)
        ret = ADK_SUCCESS;

    return ret;
}    /* -----  end of function adk_pal_ph_send  ----- */
#if 0
/*!
 * =============================================================
 * adk_pal_dequeue
 * @param[in] lcoreid logical core/ thread on where de-queue is happening
 * @param[in] *dpdk_mbuf_p[] arrary of DPDK mbuf pointer.
 * @param[in] *sub_event_id[] arrary of sub-event-ids 1-1 mapping with mbufs.
 * @param[in] burst_size  max burst size.  should be less than 32.
 *
 * @retval Number of packets de-queued.
 *
 * @brief  de-queues burst of packets from logical cores port.
 *
 * =============================================================
 */

uint32_t
adk_pal_dequeue_burst (
    uint8_t lcore_id,
    struct rte_mbuf *dpdk_mbuf_p[],
    uint8_t sub_event_id[],
    uint8_t burst_size)
{
    int32_t numOfPktsRecv = 0;
    struct rte_event ev[32];
    uint8_t index = 0;


    numOfPktsRecv = rte_event_dequeue_burst(adk_pal_dp_info_p->event_dev_id,
                                            adk_pal_dp_info_p->evdev_port_ids[lcore_id],
                                            ev, burst_size, false);
    if (numOfPktsRecv < 0)
    {
        debug_print("rte_event_dequeue_burst failed: %d\n", numOfPktsRecv);
    } else
    {
        if ( numOfPktsRecv > 32 )
        {
            debug_print("\n [%s()] un-supported de-queue burst size : %d\n", __func__, numOfPktsRecv);
            numOfPktsRecv = 0;
            return numOfPktsRecv;
        }
        for (index =0; index < numOfPktsRecv; index++)
        {
            dpdk_mbuf_p[index] = ev[index].mbuf;
            sub_event_id[index] = ev[index].sub_event_type;
        }
    }

    return numOfPktsRecv;
}    /* -----  end of function adk_pal_dequeue_burst ----- */
#endif

/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 * This API is internal to PAL layer ( used by adk_pal_dp_next_stage_event_handle API).
 * It is to encode event and relevant fields in event structure based on
 * ev-queue-id and next-stage-id.
 *
 * =============================================================
 */
adk_status_t
adk_pal_dp_event_encode(struct rte_event *event_p,
                        uint8_t   evnt_q_id,
                        uint8_t   priority,
                        adk_pal_dp_stage_info_t *stage_info_p)
{
    adk_status_t ret_value = ADK_SUCCESS;

    event_p->event_type     = RTE_EVENT_TYPE_CPU;
    event_p->queue_id       = evnt_q_id;
    event_p->priority       = priority;
    event_p->sub_event_type = stage_info_p->fn_id;

    if (stage_info_p->priority != -1) {
        event_p->priority       = stage_info_p->priority;
    }

    return (ret_value);

}    /* -----  end of function adk_pal_dp_event_encode----- */
/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 * This API is internal to PAL layer ( used by adk_pal_dp_event_dispatcher API).
 * It is to decodes dispatcher-index from event.
 * the dispatcher index is used to look up adk-event-dispatcher table.
 *
 * =============================================================
 */
adk_status_t
adk_pal_dp_event_decode(struct rte_event *event_p,
                        uint8_t   *dispatcher_index)
{

    adk_status_t ret_value = ADK_SUCCESS;
    struct rte_mbuf *dpdk_mbuf_p = event_p->mbuf;
    uint8_t pkt_type_tbl_index;
    uint8_t pkt_type = 0;

#ifdef ADK_HOST_PATH_USE_NETD
        struct rte_mbuf *t_mbuf_p = NULL;
        uint16_t fp_intf_ver_mj, fp_intf_ver_mi;
#endif

        *dispatcher_index = ADK_PAL_ERR_HANDLER_FUNC_ID;
        switch(event_p->event_type)
        {
            case RTE_EVENT_TYPE_ETHDEV :
                ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_RX_TS_UPDATE();
                if(!dpdk_mbuf_p) {
                   /* debug_print("\n[%s()] event received from eth-BD with NULL mbuf\n",__func__); */
                    ret_value = ADK_E_INVALID_PARAMETER;
                } else {

                    *dispatcher_index =  
                        adk_pal_dp_info_p->eth_dev_info[dpdk_mbuf_p->port].entry_fn_id;
                    pkt_type_tbl_index = 
                        adk_pal_dp_info_p->eth_dev_info[dpdk_mbuf_p->port].type_tbl_idx;
                    if(pkt_type_tbl_index !=
                       ADK_PAL_PKT_TYPE_TBL_ID_RSVD) {
#ifdef ADK_DEVICE_SNR
                        pkt_type = ADK_DSI_RX_METADATA_PARAM(dpdk_mbuf_p, packet_type);
#endif
                        *dispatcher_index =  adk_pal_dp_info_p->pkt_type_fn_id[pkt_type_tbl_index][pkt_type];
                    }
                }
                break;

#ifdef ADK_HOST_PATH_USE_NETD
            case RTE_EVENT_TYPE_CPPIDEV :
                /* Get the mbuf pointer from event */
                dpdk_mbuf_p = event_p->mbuf;
                if(!dpdk_mbuf_p) {
                    debug_print("\n[%s()] event received from CPPI-BD with NULL mbuf\n",__func__);
                    ret_value = ADK_E_INVALID_PARAMETER;
                } else {
                    uint16_t linux_intf_id;
                    struct adk_netd_pkt_md *netd_md_p;
                    ADK_NETD_METADATA_PTR(dpdk_mbuf_p, netd_md_p);
                    rte_prefetch0((void *)ADK_METADATA_PTR(dpdk_mbuf_p));
                    rte_prefetch0(rte_pktmbuf_mtod(dpdk_mbuf_p, void *));
                    rte_prefetch0((void *) netd_md_p);

                    if (netd_md_p->cookie != ADK_NETD_MD_COOKIE) {
                        ret_value = ADK_E_INVALID_PARAMETER;
                        return (ret_value);
                    }

                    fp_intf_ver_mj = netd_md_p->intf_ver_mj;
                    fp_intf_ver_mi = netd_md_p->intf_ver_mi;

                    dpdk_mbuf_p->pkt_len = netd_md_p->pkt_len;
                    dpdk_mbuf_p->data_len = netd_md_p->data_len;

                    debug_print("\n Pkt MD version %d.%d\n", fp_intf_ver_mj,fp_intf_ver_mi);
                    /* Code to handle Net MD mbuf chaining */
                    if((fp_intf_ver_mj >= ADK_NETD_FP_INTF_VERSION_MAJOR)
#if ADK_NETD_FP_INTF_VERSION_MINOR 
                            && (fp_intf_ver_mi >= ADK_NETD_FP_INTF_VERSION_MINOR)
#endif
                      ) {
                        dpdk_mbuf_p->next   = netd_md_p->next;
                        dpdk_mbuf_p->nb_segs= netd_md_p->nb_segs;
                        t_mbuf_p            = dpdk_mbuf_p->next;
                        while(t_mbuf_p) {
                            struct adk_netd_pkt_md *t_netd_md_p;
                            ADK_NETD_METADATA_PTR(t_mbuf_p, t_netd_md_p);
                            t_mbuf_p->data_len  =  t_netd_md_p->data_len;
                            t_mbuf_p->next      = t_netd_md_p->next;
                            t_mbuf_p            = t_netd_md_p->next;
                        }
                    }
                    dpdk_mbuf_p->port = netd_md_p->interface_id;

                    /* Initialize only ADK.MD as NETD.MD overlaps with DSI.TX.MD
                     * DSI.TX.MD will be rest after consuming NETD.MD
                     * The current assumtion is that ADK.NETD.MD size is <= 32
                     * Bytes; hence ADK.MD will not overlap with NETD.MD*/
                    adk_pkt_adk_md_reset((adk_pktio_task_t *) dpdk_mbuf_p, 
                                ADK_OS_ORIGIN_DOMAIN, 0, ADK_TASK_TYPE_FWD);

                    ADK_MD_SET_TS_CORR(dpdk_mbuf_p, netd_md_p->correlator);
                    ADK_MD_SET_TS_MSG_TYPE(dpdk_mbuf_p, netd_md_p->msg_type);
                    ADK_MD_SET_TS_INDEX(dpdk_mbuf_p, netd_md_p->ts_index);
                    ADK_MD_SET_LINUX_INTF_ID(dpdk_mbuf_p, netd_md_p->linux_intf_id);

                    debug_print("ADK: from netd: netd_md_p->linux_intf_id %u\n",
                            netd_md_p->linux_intf_id);

                    linux_intf_id = netd_md_p->linux_intf_id;
                    if (linux_intf_id < ADK_PAL_MAX_PHY_LINUX_INTF_ID) {
                        *dispatcher_index =  ADK_ETH_ULI_NETD_FN_ID;
                    } else if ((linux_intf_id >  ADK_PAL_MAX_PHY_LINUX_INTF_ID) && 
                             (linux_intf_id <=  ADK_PAL_MIN_TUN_LINUX_INTF_ID)) {
                        *dispatcher_index = ADK_SIF_BRIDGE_DEMUX_FN_ID;
                    } else if ((linux_intf_id >=  ADK_PAL_MIN_TUN_LINUX_INTF_ID) && 
                             (linux_intf_id <=  ADK_PAL_MAX_TUN_LINUX_INTF_ID)) {
                        *dispatcher_index =  ADK_SIF_TUND_IP_DEMUX_FN_ID;
                    } else  {
                        dpdk_mbuf_p = NULL;
                        ret_value = ADK_E_INVALID_PARAMETER;
                        return (ret_value);
                    }
#ifdef ADK_DEVICE_SNR
                    adk_pkt_dsi_tx_md_reset(dpdk_mbuf_p);
#endif
                }
                break;
#endif

            case RTE_EVENT_TYPE_CPU :
                ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_ULI_RX_TS_UPDATE();
                ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_RX_TS();
                /*This id for EM packets for IP entry point*/
                if (event_p->queue_id == ADK_EVENT_DEV_QUEUE_IP)
                {
                    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].em_arrival_events);
                    *dispatcher_index = ADK_SIF_EM_DEMUX_FN_ID;
                }
                else if (event_p->queue_id == ADK_EVENT_DEV_QUEUE_IPV6)
                {
                    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].em_arrival_events);
                    *dispatcher_index = ADK_SIF_EM_IPV6_DEMUX_FN_ID;
                }
                else /*Normal ADK packets*/
                {
                    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].cpu_arrival_events);
                    /*Decode based on sub-event (either ADK/EVDEV) */
                    *dispatcher_index = event_p->sub_event_type;
                }
                break;

#if ADK_DPDK_VERSION >= 18050016
            case RTE_EVENT_TYPE_TIMER:
#else 
            case RTE_EVENT_TYPE_TIMERDEV:
#endif
                *dispatcher_index = event_p->sub_event_type;
                break;
            case RTE_EVENT_TYPE_CRYPTODEV:
                *dispatcher_index = event_p->sub_event_type;
                break;
            default:
                debug_print("\n[ADK_PAL] Unsupported event type %d \n", event_p->event_type );
                ret_value = ADK_E_INVALID_PARAMETER;
                break;
        }
    return (ret_value);
}    /* -----  end of function adk_pal_dp_event_decode----- */


/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 * This API is internal to PAL layer ( used by adk_pal_dp_event_dispatcher API).
 * It is to decodes dispatcher-index from event.
 * the dispatcher index is used to look up adk-event-dispatcher table.
 *
 * =============================================================
 */
adk_status_t
adk_pal_dp_eth_dev_decode(struct rte_mbuf *dpdk_mbuf_p)
{

    adk_status_t ret_value = ADK_SUCCESS;
    uint8_t   eth_dev_type =  ADK_PAL_ETH_PORT_TYPE_INVALID;
    uint32_t coreId;
    adk_pal_dp_eth_dev_t *eth_dev_info_p = NULL;

    /*Current worker core id */
    coreId = ADK_LCORE_INDEX;

    rte_prefetch0((void *)(dpdk_mbuf_p));
    rte_prefetch0((void *)ADK_DSI_RX_METADATA_PTR(dpdk_mbuf_p));
    rte_prefetch0((void *)ADK_METADATA_PTR(dpdk_mbuf_p));
    rte_prefetch0(rte_pktmbuf_mtod(dpdk_mbuf_p, void *));

    eth_dev_info_p =  &(adk_pal_dp_info_p->eth_dev_info[dpdk_mbuf_p->port]);

    if(unlikely((!eth_dev_info_p))) {
        ret_value = ADK_E_INVALID_PARAMETER;
        return (ret_value);
    }

    eth_dev_type = eth_dev_info_p->dev_type;

    switch(eth_dev_type) {
        case ADK_PAL_ETH_PORT_TYPE_NIC:
        case ADK_PAL_ETH_PORT_TYPE_IC: {
            /* Initialize ADK meta data.  */
            adk_pkt_metadata_init((adk_pktio_task_t *) dpdk_mbuf_p, 
                                ADK_TN_PORT_DOMAIN, 0, ADK_TASK_TYPE_FWD);
            ADK_MD_SET_LINUX_INTF_ID(dpdk_mbuf_p, ADK_PACKET_INPUT_PORT(dpdk_mbuf_p));
            INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[coreId].nic_arrival_evnts);
        } break;
        default: {
            debug_print("\n[%s()] Unsupported PAL port type! %d\n",
                __func__, eth_dev_type);
            ret_value = ADK_E_INVALID_PARAMETER;
        } break;

    }

    return (ret_value);
}    /* -----  end of function adk_pal_dp_eth_dev_decode----- */


/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_next_stage_event_handle_with_queue(void  **task, uint8_t task_count,
                                   uint8_t task_mask, uint8_t  next_stage_id,
                                   uint8_t queue_id, uint32_t flow_id, 
                                   uint8_t  priority, adk_task_type_e type)
{
    uint32_t numOfPktsSent;
    uint32_t coreIndex = ADK_LCORE_INDEX;
    adk_pal_dp_stage_info_t *stage_info_p;
    RTE_SET_USED(task_count);
    RTE_SET_USED(task_mask);

    stage_info_p = &(adk_pal_dp_info_p->dp_stage_info[next_stage_id]);

    if(stage_info_p == NULL)
    {
        debug_print("worker %u thread failed to enqueue event\n",
               coreIndex);
        numOfPktsSent = 0;
        return numOfPktsSent;
    }

    {
        struct rte_event ev = {{0},{0}};
        /*encode next-stage-event */

        adk_pal_dp_event_encode(&ev, queue_id, priority, stage_info_p);

        /* Get the packet out queue  and set in mbuf*/
        ev.queue_id       = queue_id;
        ev.event_type     = RTE_EVENT_TYPE_CPU;
        ev.mbuf           = (struct rte_mbuf *)task[0];
        ev.priority       = priority;
        ev.flow_id        = flow_id;
        ev.sched_type     = stage_info_p->lb_type;
        if(type == ADK_TASK_TYPE_FWD)
        {
            ev.op             = RTE_EVENT_OP_FORWARD;
        } else
        {
            ev.op             = RTE_EVENT_OP_NEW;
        }

        ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_TX_TS();
        ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_TX_TS();
        /* Send the packet out */
        numOfPktsSent = rte_event_enqueue_burst(adk_pal_dp_info_p->event_dev_id, adk_pal_dp_info_p->evdev_port_ids[coreIndex], &ev, 1);
        if (numOfPktsSent != 1) {
            debug_print("[%s] worker %u thread failed to enqueue event\n",
                    __func__, coreIndex);
            numOfPktsSent = 0;
        }
    }
    return numOfPktsSent;

}    /* -----  end of function adk_pal_dp_next_stage_event_handle_with_queue ----- */

/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_next_stage_event_handle_with_stage_info(void  **task, uint8_t task_count,
                                   uint8_t task_mask, 
                                   adk_pal_dp_stage_info_t *dp_stage_info_p,
                                   uint32_t flow_id, uint8_t  priority,
                                   adk_task_type_e task_type)
{
    uint32_t numOfPktsSent;
    int err;
    uint32_t coreIndex = ADK_LCORE_INDEX;
    uint8_t  next_stage_id;
    adk_pal_stage_info_t *stage_info_p;

    if (task_count==0)
    {
        numOfPktsSent = 0;
        return numOfPktsSent;
    }

    if(dp_stage_info_p == NULL)
    {
        debug_print("worker %u thread, NULL stage info \n", coreIndex);
        numOfPktsSent = 0;
        return numOfPktsSent;
    }

    next_stage_id = dp_stage_info_p->fn_id;

    if(dp_stage_info_p->event_q_id == -1) {
        (adkDestEntryFunction_g[next_stage_id])
            (task, task_count, task_mask, NULL, NULL, coreIndex);
        numOfPktsSent = 1;
    } else if(dp_stage_info_p->event_q_id >= 0) {
        struct rte_event ev = {{0},{0}};
        /*encode next-stage-event */

        adk_pal_dp_event_encode(&ev, dp_stage_info_p->event_q_id,
                                priority, dp_stage_info_p);

        /* Get the packet out queue  and set in mbuf*/
        ev.queue_id       = dp_stage_info_p->event_q_id;
        ev.event_type     = RTE_EVENT_TYPE_CPU;
        ev.mbuf           = (struct rte_mbuf *)task[0];
        ev.priority       = priority;
        ev.flow_id        = flow_id;
        ev.sched_type     = dp_stage_info_p->lb_type;
        if (task_type == ADK_TASK_TYPE_FWD )
        {
            ev.op             = RTE_EVENT_OP_FORWARD;
        } else
        {
            ev.op             = RTE_EVENT_OP_NEW;
        }
        ADK_DP_MEASURE_BLOCK_CYCLE_PAL_ULI_TX_TS();
        ADK_DP_MEASURE_BLOCK_CYCLE_ADK_ULI_TX_TS();

#ifdef ADK_PAL_CYCLE_MEASUREMENT
            /*tx time stamp and generate the time/cycle diff*/
            adk_sys_dp_proc_cycle_measure(task[0]);
#endif

#if defined(ADK_MEASURE_CYCLE)
        ADK_MEASURE_CYCLE_DIFF(&(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE]), 1);
        ADK_MEASURE_CYCLE_CHECK_EXIT(adkMeasCycle_g[coreIndex][ADK_TERMINATION_STAGE], coreIndex, ADK_TERMINATION_STAGE);
#endif
            /* Send the packet out */
            numOfPktsSent = rte_event_enqueue_burst(adk_pal_dp_info_p->event_dev_id, adk_pal_dp_info_p->evdev_port_ids[coreIndex], &ev, 1);
            if (numOfPktsSent != 1) {
                debug_print("[%s] worker %u thread failed to enqueue event\n",
                        __func__, coreIndex);
                numOfPktsSent = 0;
            }
    } else {
        numOfPktsSent = 1;
        stage_info_p = adk_pal_dp_info_p->stage_info_p[next_stage_id];
        err = rte_ring_enqueue(stage_info_p->entry_info.ring_buffer, task[0]);
        if((err != 0) && (err != -EDQUOT)) {
            debug_print("\n[%s] ring enqueue failed %d \n", __func__, err);
            numOfPktsSent = 0;
        }
    }
    return numOfPktsSent;

}    /* -----  end of function adk_pal_dp_next_stage_event_handle_with_stage_info ----- */



/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_next_stage_event_handle(void  **task, uint8_t task_count,
                                   uint8_t task_mask, uint8_t  next_stage_id,
                                   uint32_t flow_id, uint8_t  priority,
                                   adk_task_type_e task_type)
{
    uint32_t numOfPktsSent;
    adk_pal_dp_stage_info_t  *dp_stage_info_p;

    if (task_count==0)
    {
        numOfPktsSent = 0;
        return numOfPktsSent;
    }

    dp_stage_info_p = &(adk_pal_dp_info_p->dp_stage_info[next_stage_id]);

    if(dp_stage_info_p == NULL)
    {
        debug_print("worker %u thread failed to enqueue event\n",
               ADK_LCORE_INDEX);
        numOfPktsSent = 0;
        return numOfPktsSent;
    }

    numOfPktsSent = adk_pal_dp_next_stage_event_handle_with_stage_info(task, 
                task_count, task_mask, dp_stage_info_p, flow_id, priority, task_type);
    return numOfPktsSent;
}    /* -----  end of function adk_pal_dp_next_stage_event_handle ----- */

/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */

adk_status_t
adk_pal_dp_dispather_error_handler(void **task_p, uint8_t task_count,
                         uint32_t task_mask,
                         void *ptr1,
                         adk_sif_cfg_pipeline_info_t *ptr2,
                         int coreIndex)
{

    adk_status_t ret = ADK_SUCCESS;
    uint8_t     i= 0;

    /* To avoid un-used variabel warning */
    RTE_SET_USED(ptr1);
    RTE_SET_USED(ptr2);
    RTE_SET_USED(task_p);
    RTE_SET_USED(task_count);
    RTE_SET_USED(task_mask);
    RTE_SET_USED(coreIndex);

    /* Need to add debug counters */

    debug_print("\n [%s()] called coreIndex: %d\n", __func__, coreIndex);

    /* Free up the mbufs */
    for(i=0;i<task_count;i++)
    {
        rte_pktmbuf_free(task_p[i]);
        adk_pal_dp_info_p->stats[coreIndex].pal_dispatcher_err_hdl++;
    }
    return(ret);
}

/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */

 adk_dp_status_t
 adk_pal_dp_stage_evq_id_get(uint8_t  next_stage_id, uint8_t   *evnt_q_id)
 {
    adk_dp_status_t ret = ADK_SUCCESS;
    adk_pal_stage_info_t *stage_info_p;

    stage_info_p = adk_pal_dp_info_p->stage_info_p[next_stage_id];

    if ((stage_info_p !=NULL) &&
        (stage_info_p->entry_info.entry_type == ADK_PAL_STAGE_ENTRY_TYPE_EV_Q)) {
        *evnt_q_id =  stage_info_p->entry_info.evnt_q_id;
    } else {
        debug_print("\n[%s()] invalid stage or entry-type \n", __func__);
        ret = ADK_DP_E_ERROR;
    }

    return(ret);

 }

/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */
uint32_t
adk_pal_dp_crypto_send (
    uint8_t    crypto_dev_id,
    uint32_t  dest_func_idx,
    uint8_t   priority,
    struct rte_mbuf *dpdk_mbuf_p,
    struct rte_crypto_op *op_data,
    uint8_t cbFn,
    void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;
    adk_pal_stage_info_t *stage_info_p;
    struct rte_event *ev;

    stage_info_p = adk_pal_dp_info_p->stage_info_p[dest_func_idx];
 
    /*Validate the destination stage information */
    if(stage_info_p->entry_info.entry_type != ADK_PAL_STAGE_ENTRY_TYPE_EV_Q )
    {
        debug_print("[%s()]destination stage funciton entry type validation failed\n"
               , __func__);
        return ADK_E_INTERNAL_ERROR;
    }

    adk_pal_batch_rte_enqueue_size_validate();

    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    /* Get the crypto-bd event-queue-id*/
    ev->queue_id       = adk_pal_dp_info_p->crypto_dev_to_event_queue_id[crypto_dev_id];
    ev->event_type     = RTE_EVENT_TYPE_CRYPTODEV;
    ev->sub_event_type = dest_func_idx,
    ev->priority       = priority;
    ev->sched_type     = stage_info_p->stage_lb_type;

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
        ev->op         = RTE_EVENT_OP_FORWARD;
    } else {
        ev->op         = RTE_EVENT_OP_NEW;
    }

    ev->event_ptr      = op_data;

#ifdef ADK_PAL_CYCLE_MEASUREMENT
    /*tx time stamp and generate the time/cycle diff*/
    adk_sys_dp_proc_cycle_measure(dpdk_mbuf_p);
#endif

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_crypto_events);

    adk_pal_batch_proc_pkts_validate();

    return ret;
}


/*
 * =============================================================
 *  ADK_DP_PAL_INTERNAL
 *
 *
 * =============================================================
 */
/*This function is just place holder to support timer event via batch mode
and not tested .currently RTE Timer is not used by any adk modules ,
when there is a need to use RTE Timer then this can be used*/

uint32_t
adk_pal_dp_timer_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam)
{
    adk_status_t ret = ADK_E_TASK_CONSUMED;
    struct rte_event *ev;
    
     adk_pal_batch_rte_enqueue_size_validate();

    ev = &adk_pal_batch_info_p->adkRteEvents[adk_pal_batch_info_p->adkRteEventIndex];

    /* Get the crypto-bd event-queue-id*/
    ev->queue_id       = ADK_METADATA_PARAM(dpdk_mbuf_p, event_queue_id);
    ev->event_type     = RTE_EVENT_TYPE_TIMER;
    ev->mbuf           = dpdk_mbuf_p;
    ev->sub_event_type = 0,
    ev->priority       = 0;
    ev->sched_type     = ADK_METADATA_PARAM(dpdk_mbuf_p, event_sched_type);

    if(ADK_METADATA_PARAM(dpdk_mbuf_p, task_type) == ADK_TASK_TYPE_FWD)
    {
        adk_pal_batch_info_p->procPkts++;
        ev->op         = RTE_EVENT_OP_FORWARD;
    } else {
        ev->op         = RTE_EVENT_OP_NEW;
    }

    adk_pal_batch_rte_event_cb_update(cbFn, ADK_PAL_DP_BATCH_CB_EVENT_TIMER, cbParam);

    INCREMENT_PAL_UTILITY_COUNTER(adk_pal_dp_info_p->stats[ADK_LCORE_INDEX].pal_batched_timer_events);

#ifdef ADK_PAL_CYCLE_MEASUREMENT
    /*tx time stamp and generate the time/cycle diff*/
    adk_sys_dp_proc_cycle_measure(dpdk_mbuf_p);
#endif
    adk_pal_batch_proc_pkts_validate();

    return ret;
}

/*! end addtogroup _pal
 * @}
 */
