/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_dp_internal.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _palFunctions  PAL
 *
 * @{
 *
 * @brief
 *
 * This file contains all the internal functions of QM Abstraction Layer (PAL)
 * data plane.
 *
 ***********************************************************************/

#include <adk_dpdk.h>
#include <adk_include.h>
#include "adk_pal_dp.h"
#include "adk_pal_dp_export.h"

/*!
 * =============================================================
 * @internal
 *
 * adk_pal_adk_meta_data_set
 * @param[in] dpdkMbuf_p DPDK mbuf.
 *
 * @retval None
 *
 * @brief  Sets the ADK meta data in the DPDK mbuf.
 *
 * This function sets the ADK meta data in the DPDK mbuf for the
 * packets received on the NIC port and for packets received on
 * vEth interface.
 *
 * =============================================================
 */
void
adk_pal_adk_meta_data_set (
    struct rte_mbuf *dpdkMbuf_p,
    adk_sif_domain_t exception_domain)
{
    void *pduData_p;

    /*
     * Initialize the ADK meta data.
     */
    /* Parameter 0 contains the input port */
    ADK_TASK_USER_PARAMS(dpdkMbuf_p, 0) = dpdkMbuf_p->port;

    /* Parameter 6 contains the pointer to the chained packet */
    ADK_TASK_USER_PARAMS(dpdkMbuf_p, 6) = 0;

    /* Parameter 7 contains the pointer to the packet data */
    pduData_p = rte_pktmbuf_mtod(dpdkMbuf_p, char *);
    ADK_TASK_USER_PARAMS(dpdkMbuf_p, 7) = (size_t) pduData_p;

    adk_pkt_metadata_init((adk_pktio_task_t *) dpdkMbuf_p, exception_domain,
                          0, ADK_TASK_TYPE_FWD);

    return;
}    /* -----  end of function adk_pal_adk_meta_data_set  ----- */
/*! end addtogroup _pal
 * @}
 */
