/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_cp_pvt.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL Layer
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the internal definitions required by the
 * control&dataplane plane parts of the Pipeline Abstraction layer. 
 * This file should not be included by the application or any 
 * other ADK modules.
 *
 ***********************************************************************/
#ifndef _ADK_PAL_PVT_H_
#define _ADK_PAL_PVT_H_

#include "adk_pal_dp.h"
#include "adk_rm.h"

#define ADK_PAL_NON_ZERO_CNTR_WRITE(fmt, cntr) \
    if (cntr != 0) { \
        ADK_INFO_WRITE(fmt, cntr) \
    }

/*!
 * =============================================================
 * @struct adk_pal_cp_info_t adk_pal_cp_pvt.h "adk_pal_cp_pvt.h"
 *
 * @brief Structure to store PAL control information.
 *
 * =============================================================
 */

typedef struct adk_pal_cp_info_s {

    /*ADK System handle */
    adk_sys_handle_t    sys_handle;

    adk_pal_dp_info_t *adk_pal_dp_info_p;

    uint64_t *adk_pal_cp_min_cycles;


    /* EVD instance  */
    uint8_t   event_dev_id;

    /* Ethernet Bridge Device () direct-queue-id*/
    uint8_t   eth_port_to_event_queue_id[ADK_PAL_MAX_NUM_NIC_PORTS];

    /* Event queues list */
    uint8_t   event_q_list[ADK_PAL_MAX_NUM_EVQS];
    uint8_t   num_ev_queues;

    /* crypto qp list */
    uint32_t  crypto_dev_to_event_queue_id[ADK_PAL_MAX_NUM_CRYPTO_DEV];

    /* Dynamic-pipeline  */
    adk_pal_stage_info_t  *stage_info_p[ADK_MAX_PROC_FUNC_ID];
    
    /* Timer-BD informaiton */
    uint8_t     adkTimerBdInstance_g;

    /*packet type table index */
    adk_rm_hdl_t    pkt_type_tbl_rm_handle;

    /* Flag to indicate VSI dev's  */
    bool        vsi_dev_list[ADK_PAL_MAX_NUM_NIC_PORTS];

} adk_pal_cp_info_t ;


/*
 * data structure to hold the stats of a processing stage
 * per lcore stats. @index (RTE_MAX_LCOR+1) over all stats
 */
typedef struct adk_pal_proc_fn_stats_s{
    uint32_t    avg_cycles[ADK_MAX_NUM_CORES + 1]; 
    uint32_t    min_latency[ADK_MAX_NUM_CORES + 1]; 
    uint32_t    max_latency[ADK_MAX_NUM_CORES + 1]; 
    uint32_t    avg_latency[ADK_MAX_NUM_CORES + 1]; 
} adk_pal_proc_fn_stats_t;

/*
 * Helper funciton to return processing-funciton-name of a 
 * given function-id
 */
char *adk_pal_proc_fn_id_to_name(adk_pal_proc_fn_id_e fn_id);

/*
 * Helper funciton to return processing-funciton-id of a 
 * given function-name
 */
adk_pal_proc_fn_id_e adk_pal_proc_fn_name_to_id(char *name);


#endif /* End of _ADK_PAL_PVT_H_ */

/*!
 * @} end addtogroup _pal
 */
