/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_pal_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _pal  PAL Layer
 *
 * @{
 *
 * @brief  
 *
 * This file contains all the internal definitions required by the
 * data plane part of the Pipeline Abstraction layer. This file should not be
 * included by the dataplane part of the application.
 *
 ***********************************************************************/
#ifndef _ADK_PAL_DP_H_
#define _ADK_PAL_DP_H_

#include <rte_eventdev.h>
#include <rte_event_queue_dispatcher.h>
/*#include <adk_dpdk.h> */
#include "adk_pal.h"
#include "adk_em.h"
#include <adk_pktio.h>

#define ADK_PAL_FN_ID_WIDTH 8
#define ADK_PAL_MAX_FN_IDS  256
#define ADK_PAL_MAX_SAMPLES 32

#define ADK_PAL_MAX_NUM_EVQS   8

#define ADK_PAL_PKT_TYPE_WIDTH (8)
#define ADK_PAL_PKT_TYPE_MAX   ( (1 << ADK_PAL_PKT_TYPE_WIDTH) - 1 )
/*#define ADK_PAL_PKT_TYPE_ALL   ADK_PAL_PKT_TYPE_MAX*/

#define ADK_PAL_PKT_TYPE_TBL_ID_WIDTH (4)
#define ADK_PAL_PKT_TYPE_TBL_IDS (4)
#define ADK_PAL_PKT_TYPE_TBL_ID_RSVD (0)

#define ADK_PAL_ETH_PORT_TYPE_NIC   (0x1)
#define ADK_PAL_ETH_PORT_TYPE_VETH  (0x2)
#define ADK_PAL_ETH_PORT_TYPE_IC    (0x3)
#define ADK_PAL_ETH_PORT_TYPE_TUND  (0x4)
#define ADK_PAL_ETH_PORT_TYPE_WIDTH (4)
#define ADK_PAL_ETH_PORT_TYPE_INVALID  ((1 << ADK_PAL_ETH_PORT_TYPE_WIDTH) - 1)


#define ADK_PAL_ETH_DEV_ID_WIDTH    (16)

#define ADK_PAL_MAX_NUM_NIC_PORTS     (RTE_MAX_ETHPORTS)
#define ADK_PAL_MAX_FLOATING_THREADS   (8) 
#define ADK_PAL_INVALID_EV_PORT_ID    (0xFF)
#define ADK_PAL_MAX_NUM_CRYPTO_DEV     (255)


#define ADK_PAL_N_PHY_LINUX_INTF_IDS  (256)
#define ADK_PAL_N_BVI_LINUX_INTF_IDS  (256)
#define ADK_PAL_N_TUN_LINUX_INTF_IDS  (1024)

#define ADK_PAL_N_LINUX_INTF_IDS      (ADK_PAL_N_PHY_LINUX_INTF_IDS + \
                                       ADK_PAL_N_BVI_LINUX_INTF_IDS +\
                                       ADK_PAL_N_TUN_LINUX_INTF_IDS )


#define ADK_PAL_TUNNEL_IF_BASE_ID     (ADK_PAL_N_PHY_LINUX_INTF_IDS +\
                                       ADK_PAL_N_BVI_LINUX_INTF_IDS)

#define ADK_PAL_MAX_TUNNEL_IFS        (ADK_PAL_N_TUN_LINUX_INTF_IDS)
#define ADK_PAL_MAX_TUNNEL_IF_ID     (ADK_PAL_TUNNEL_IF_BASE_ID + ADK_PAL_MAX_TUNNEL_IFS)

/* The MASK has to be relooked when ever man tunnel ids are increased  */
#define ADK_PAL_TUNNEL_IF_ID_MASK    (0x3FF)

#define ADK_PAL_MIN_LINUX_INTF_ID     (0)
#define ADK_PAL_MAX_LINUX_INTF_ID     (ADK_PAL_N_LINUX_INTF_IDS-1)

#define ADK_PAL_MIN_PHY_LINUX_INTF_ID  (0)

#define ADK_PAL_MAX_PHY_LINUX_INTF_ID  (ADK_PAL_MIN_PHY_LINUX_INTF_ID +\
                                       ADK_PAL_N_PHY_LINUX_INTF_IDS - 1)

#define ADK_PAL_MIN_TUN_LINUX_INTF_ID  (ADK_PAL_N_PHY_LINUX_INTF_IDS +\
                                       ADK_PAL_N_BVI_LINUX_INTF_IDS)

#define ADK_PAL_MAX_TUN_LINUX_INTF_ID  (ADK_PAL_MIN_TUN_LINUX_INTF_ID +\
                                       ADK_PAL_N_TUN_LINUX_INTF_IDS - 1)

#define ADK_PAL_DP_GET_BATCH_INFO_PTR(_coreId)  \
    (&adk_pal_dp_info_p->dp_batch_info[(_coreId)])

#define ADK_PAL_MAX_ENQUEUE_SIZE 32

#ifdef ADK_UTILITY_COUNTER_ENABLE
#define INCREMENT_PAL_UTILITY_COUNTER(counter) \
    do  \
    {       \
        if(adk_pal_dp_info_p->isUtilityCntEnabled)  \
        {  \
            counter++;  \
        }  \
    }while(0);
#define INCREMENT_PAL_UTILITY_COUNTER_COUNT(counter,count) \
    do  \
    {       \
        if(adk_pal_dp_info_p->isUtilityCntEnabled)  \
        {  \
            counter = counter + count;  \
        }  \
    }while(0);

#else
#define INCREMENT_PAL_UTILITY_COUNTER(counter)
#define INCREMENT_PAL_UTILITY_COUNTER_COUNT(counter,count)
#endif

/*!
 * =============================================================
 * @struct adk_pal_shm_config_t adk_pal_dp.h "adk_pal_dp.h"
 *
 * @brief Structure to pass information from primary to secondary
 * process .
 *
 * =============================================================
 */
typedef struct adk_pal_shm_config_t{
    /*!
     * ADK PAL Control Plane handle is stored in primary process and read in
     * secondary process.
     */
    adk_handle_t    adk_pal_cp_handle;

    /*!
     * ADK PAL Data Plane handle is stored in primary process and read in
     * secondary process.
     */
    adk_handle_t    adk_pal_dp_handle;

    /*!
     * PAL metrics pointers shared between ADK CP & DP
     */
    uint64_t        *min_cycles;

} adk_pal_shm_config_t __rte_cache_aligned;


/*!
 * Name of shared memory used in IPC between primary & secondary process
 */
#define ADK_PAL_SHM_MEMZONE_NAME "ADK_PAL_SHM_MEMZONE"

#define ADK_PAL_BATCH_ENQUEUE_BURST_RETRY_COUNT  2

/*
* Pipeline stage information stored in dataplane
*/
typedef struct adk_pal_stage_info_s{
    adk_pal_stage_lb_type_e  	stage_lb_type;
    uint8_t                         fn_id;
    adk_pal_stage_entry_info_t	entry_info;
} adk_pal_stage_info_t;

/*
* Pipeline stage information stored in dataplane
*/
typedef struct adk_pal_dp_stage_info_s{
    uint8_t lb_type     :2;
    int8_t  priority    :6;
    uint8_t fn_id       :8;
    int8_t  event_q_id  :8;
    uint8_t reserved    :8;
} adk_pal_dp_stage_info_t;


/*!
 * =============================================================
 * @struct adk_pal_dp_eth_dev_t adk_pal_dp.h "adk_pal_dp.h"
 *
 * @brief Structure to store eth-dev information
 *
 * =============================================================
 */
typedef struct adk_pal_dp_eth_dev_s{
    /** 32-bit in size */
    union {
        uint32_t        aggr_info;
        struct {
            uint32_t    entry_fn_id    : ADK_PAL_FN_ID_WIDTH;
            uint32_t    dev_type       : ADK_PAL_ETH_PORT_TYPE_WIDTH ;
            uint32_t    type_tbl_idx   : ADK_PAL_PKT_TYPE_TBL_ID_WIDTH;
            uint32_t    port_id        : ADK_PAL_ETH_DEV_ID_WIDTH;
        };
    };
} adk_pal_dp_eth_dev_t;


#ifdef ADK_PERFORMANCE_STATS
#define ADK_MAX_NUM_OF_FLOW_SUPPORT 20
typedef struct adk_pal_flows_stats_s {
	uint64_t flow_count;
        uint32_t flowid;
}adk_pal_flows_stats_t;
#endif
/*!
 * =============================================================
 * @struct adk_pal_dp_stats_t adk_pal_dp.h "adk_pal_dp.h"
 *
 * @brief Structure to store PAL dataplane stats
 *
 * =============================================================
 */

typedef struct adk_pal_dp_stats_s{
    uint64_t    arrival_evnts;
    uint64_t    nic_arrival_evnts;
    uint64_t    timer_arrival_events;
    uint64_t    crypto_arrival_events;
    uint64_t    em_arrival_events;
    uint64_t    cpu_arrival_events;
    uint64_t    pal_dispatcher_err_hdl;
    uint64_t    nic_arrival_null_mbuf_evnts_drop;
#ifdef ADK_HOST_PATH_USE_NETD
    uint64_t    netd_arrival_events;
#endif
#ifdef ADK_PERFORMANCE_STATS
    uint64_t    batch_stats[33];
    uint64_t    total_num_of_dequeue;
    uint64_t    start_loop_count;
    adk_pal_flows_stats_t num_flows[ADK_MAX_NUM_OF_FLOW_SUPPORT];
    struct rte_event_queue_dispatcher_stats user_stats;
#endif
    uint64_t    pal_batched_em_events;
    uint64_t    pal_batched_nic_events;
    uint64_t    pal_batched_ph_events;
    uint64_t    pal_batched_netd_events;
    uint64_t    pal_batched_stage_events;
    uint64_t    pal_batched_crypto_events;
    uint64_t    pal_batched_release_events;
    uint64_t    pal_batched_timer_events;
    uint64_t    em_xmt_events;
    uint64_t    em_xmt_fail_events;
    uint64_t    rte_xmt_events;
    uint64_t    rte_xmt_fail_events;
    uint64_t    pal_batch_fail_safe_err;
    uint64_t    pal_batch_fail_safe_drop;
}adk_pal_dp_stats_t __rte_cache_aligned;


/*!
 * =============================================================
 *   Callback Functions   {{{2
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 * =============================================================
 */

typedef void (*adk_pal_dp_batch_cb_fn) (
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

typedef enum adk_pal_dp_batch_cb_fn_s{
    ADK_PAL_DP_BATCH_CB_FN_ETH,
    ADK_PAL_DP_BATCH_CB_FN_IP,
    ADK_PAL_DP_BATCH_CB_FN_IPV6,
    ADK_PAL_DP_BATCH_CB_FN_IPSEC,
    ADK_PAL_DP_BATCH_CB_FN_PH,
    ADK_PAL_DP_BATCH_CB_FN_EM,
    ADK_PAL_DP_BATCH_CB_FN_SIF,
    ADK_PAL_DP_BATCH_CB_FN_MAX
}adk_pal_dp_batch_cb_fn_t;

typedef enum adk_pal_dp_batch_cb_event_s{
    ADK_PAL_DP_BATCH_CB_EVENT_EM,
    ADK_PAL_DP_BATCH_CB_EVENT_NIC,
    ADK_PAL_DP_BATCH_CB_EVENT_PH,
    ADK_PAL_DP_BATCH_CB_EVENT_NETD,
    ADK_PAL_DP_BATCH_CB_EVENT_STAGE_BUFFER,
    ADK_PAL_DP_BATCH_CB_EVENT_STAGE_PH_POLICER,
    ADK_PAL_DP_BATCH_CB_EVENT_STAGE_EM_ORDER,
    ADK_PAL_DP_BATCH_CB_EVENT_STAGE_ETH_NETD,
    ADK_PAL_DP_BATCH_CB_EVENT_STAGE_DIRECT,
    ADK_PAL_DP_BATCH_CB_EVENT_CRYPTO,
    ADK_PAL_DP_BATCH_CB_EVENT_TIMER,
    ADK_PAL_DP_BATCH_CB_EVENT_RELEASE,
    ADK_PAL_DP_BATCH_CB_EVENT_MAX,
}adk_pal_dp_batch_cb_event_t;


typedef struct adk_pal_dp_batch_cb_param_s
{
    uint8_t   cbFn:4;
    uint8_t   cbEventType:4;
    void*     cbParam; 
}adk_pal_dp_batch_cb_param_t; 

typedef struct adk_pal_dp_batch_info_s{
    uint8_t rxPkts; 
    uint8_t procPkts;
    uint8_t adkEmEventIndex;
    uint8_t adkRteEventIndex; 
    uint32_t eventMachineQueueId;
    em_event_t adkEmEvent[32]; /* 32 events are MAX dequed in rte_eventdev_dequeue_burst*/
    struct rte_event adkRteEvents[ 32 ];
    adk_pal_dp_batch_cb_param_t adkEmEventRet[32];
    adk_pal_dp_batch_cb_param_t adkRteEventRet[32];
}adk_pal_dp_batch_info_t;

/*!
 * =============================================================
 * @struct adk_pal_dp_info_t adk_pal_dp.h "adk_pal_dp.h"
 *
 * @brief Structure to store PAL dataplane information.
 *
 * =============================================================
 */

typedef struct adk_pal_dp_info_s {

    /*ADK System handle */
    adk_sys_handle_t    sys_handle;

    /* EVD instance  */
    uint8_t   event_dev_id;

    /* rx metadata offset of the ethernet device */
    uint32_t  nic_rx_md_offset;

    /* tx metadata offset of the ethernet device */
    uint32_t  nic_tx_md_offset;

#ifdef ADK_HOST_PATH_USE_NETD
    /* metadata offset of the netd device */
    uint32_t  netd_md_offset;
#endif

    /* metadata offset of the crypto bridge  device */
    uint32_t  crypto_bd_md_offset;

    /* Array to store eth-dev information */
    adk_pal_dp_eth_dev_t  eth_dev_info[ADK_PAL_MAX_NUM_NIC_PORTS];

    /* per packet type dataplane entry function-id*/
    uint8_t pkt_type_fn_id[ADK_PAL_PKT_TYPE_TBL_ID_WIDTH][ADK_PAL_PKT_TYPE_MAX];

    /* Ethernet Bridge Device () direct-queue-id*/
    uint8_t   eth_port_to_event_queue_id[ADK_PAL_MAX_NUM_NIC_PORTS];

#ifdef ADK_HOST_PATH_USE_NETD
    /* Ethernet Bridge Device () direct-queue-id*/
    int16_t   linux_intf_id_to_evnt_qid[ADK_PAL_N_LINUX_INTF_IDS];
#endif

    /* adk-tunnel-if-id to tun-dev-id mapping*/
    uint32_t  adk_tun_if_id_to_tundev[ADK_PAL_MAX_TUNNEL_IF_ID];

    /* EVD Producer port per worker thread*/
    uint8_t   evdev_port_ids[ADK_MAX_NUM_CORES + ADK_PAL_MAX_FLOATING_THREADS];

    /* Event queues list */
    uint8_t   event_q_list[ADK_PAL_MAX_NUM_EVQS];
    uint8_t   num_ev_queues;

    /* crypto qp list */
    uint32_t  crypto_dev_to_event_queue_id[ADK_PAL_MAX_NUM_CRYPTO_DEV];

    /* Dynamic-pipeline  */
    adk_pal_stage_info_t  *stage_info_p[ADK_MAX_PROC_FUNC_ID];

    /* Dynamic-pipeline  */
    adk_pal_dp_stage_info_t  dp_stage_info[ADK_MAX_PROC_FUNC_ID];

    /* PAL stats */
    adk_pal_dp_stats_t stats[ADK_MAX_NUM_CORES]; 

    /* Timer-BD informaiton */
    uint8_t     adkTimerBdInstance_g;

    /* Flag to indicate whether the dataplne entry function pointer table is
     * intilized or not */
    uint8_t     dp_proc_fn_ptr_table_state;

    /*Extended stats state
     * ADK_PAL_DP_XSTATS_DISABLE
     * ADK_PAL_DP_XSTATS_ENABLE 
     * */
    int16_t     xstats_state;

    adk_pal_dp_batch_info_t   dp_batch_info[ADK_MAX_NUM_CORES];
    adk_pal_dp_batch_cb_fn adk_pal_dp_batch_cb_fn_g[ADK_PAL_DP_BATCH_CB_FN_MAX];
    bool isUtilityCntEnabled;
} adk_pal_dp_info_t __rte_cache_aligned;


extern adk_pal_dp_info_t  *adk_pal_dp_info_p;

/* 
 *  ADK_DP_PAL_INTERNAL
 *
 * This API is internal to PAL layer ( used by adk_pal_dp_next_stage_event_handler API). 
 * It is to encode event and relevant fields in event structure based on ev-queue-id and next-stage-id.
 */ 
adk_status_t
adk_pal_dp_event_encode(struct rte_event *event, uint8_t   evnt_q_id, 
                        uint8_t priority, adk_pal_dp_stage_info_t *stage_info_p);
 
/* 
 *  ADK_PAL_DP_INTERNAL
 * 
 * This API is internal to PAL layer ( used by adk_pal_dp_event_dispatcher API).
 * It is to decodes dispatcher-index from event. 
 * the dispatcher index is used to look up adk-event-dispatcher table.
 */ 
adk_status_t
adk_pal_dp_event_decode(struct rte_event *event, uint8_t   *dispatcher_index);



/* 
 *  ADK_PAL_DP_INTERNAL
 * 
 * This function is internal to PAL layer. 
 * 1. It is to do any port mappings and update the mbuf header fields
 * 2. to init adk-metadata 
 */
adk_status_t
adk_pal_dp_eth_dev_decode(struct rte_mbuf *dpdk_mbuf_p);

void
adk_pal_nic_entry_fn (
    struct rte_mbuf *dpdkMbuf_p,
    uint32_t lcore_id);

void
adk_pal_veth_entry_fn (
    struct rte_mbuf *dpdkMbuf_p,
    uint32_t lcore_id);

adk_dp_status_t
adk_pal_nic_event_prepare(
        uint8_t   out_port,
        uint8_t pktColor,
        uint16_t dev_tx_queue_id,
        struct rte_mbuf *dpdk_mbuf_p,
        struct rte_event *ev);

#ifdef ADK_HOST_PATH_USE_NETD
uint32_t
adk_pal_netd_send_prepare (
    struct rte_mbuf *dpdk_mbuf_p, struct rte_event *ev);

adk_status_t
adk_pal_dp_netd_send(
        struct rte_mbuf *dpdk_mbuf_p,
        uint8_t cbFn,
        void *cbParam);
#endif /* ADK_HOST_PATH_USE_NETD */


/* ADK_DP_INTERNAL
 * Data plane API to be called at the end of each stage.
 * This function generates a event and passes that event to next stage.
 * Sets the "ev.event_type" to CPU->CPU/Device
 * next-stage-id is passed in the "ev.sub_event_type" field
 * 
 */
uint32_t
adk_pal_dp_next_stage_event_handle_with_stage_info(void  **task, uint8_t task_count, 
                                   uint8_t task_mask, adk_pal_dp_stage_info_t *stage_info_p,
                                   uint32_t flow_id, uint8_t  pkt_priority, adk_task_type_e);


adk_status_t adk_pal_dp_batch_send(uint8_t emEventCount, uint8_t rteEventCount);

adk_status_t
adk_pal_dp_stage_send(
        void  **task,
        uint8_t task_count,
        uint8_t task_mask,
        uint8_t  dest_fn_id,
        uint32_t flow_id,
        uint8_t  pkt_priority,
        uint8_t cbFn,
        uint8_t cbEventType,
        void *cbParam);

adk_status_t
adk_pal_dp_em_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint32_t eventMachineQueueId,
    uint8_t cbFn,
    void *cbParam);

uint32_t
adk_pal_dp_nic_send (
        uint8_t   out_port,
        uint8_t pktColor,
        uint16_t  tx_queue_id,
        struct rte_mbuf *dpdk_mbuf_p,
        uint8_t cbFn,
        void *cbParam);

adk_status_t
adk_pal_dp_release_send (
    struct rte_mbuf *dpdk_mbuf_p,
    uint8_t cbFn,
    void *cbParam);

void adk_pal_dp_em_event_handle_return(
    uint8_t retIndex);

void adk_pal_dp_rte_event_handle_return_success(uint8_t eventIndex, uint8_t eventCount);
void adk_pal_dp_rte_event_handle_return_fail(uint8_t eventIndex, uint8_t eventCount);

#endif /* End of _ADK_PAL_DP_H_ */

/*!
 * end addtogroup _pal
 * @}
 */
