/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ph_dev.h
 *
 *      $HeadURL: ncp_adk/code/ph/include/adk_ph_dev.h $
 *      $Revision: 366f377355b8b32e2740de4f058f578662bfab2c $
 *      $Date: Mon Oct 10 12:54:51 2016 -0500 $
 *
 * @addtogroup _ph  PH Module
 * @{
 *
 * @brief
 *
 * Contains the structure/union/macro definitions used by the PDU handler.
 * This file needs to be included by the user of the PDU handler.
 *
 * THIS FILE IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 *  **********************************************************************/
#include "adk_sys.h"
#ifndef __ADK_PH_DEV_H__
#define __ADK_PH_DEV_H__

#define ADK_PH_DEV_MAX_NUM_DEVICES      6
#define ADK_PH_DEV_MAX_NUM_QUEUES       6

#define ADK_EVENT_SIZE  1

#define ADK_PH_CBFN_MAX 64
#define ADK_PH_MODULE_ID_MAX 4           /*ETH,IP,IPV6,IPSEC*/
/*!
 *
 * The data structure declarations   {{{1
 */

/*!
 * The PDU handler's handle
 */
typedef adk_handle_t adk_ph_handle_t;

/*!
 * The bind handle
 */
typedef adk_handle_t adk_ph_pipeline_bind_handle_t;


/*!
 * =============================================================
 * @internal
 * @enum adk_ph_dev_interrupt_events_t   {{{2
 *
 * @brief Messages sent over file descriptor to wake up waiting threads
 *
 * =============================================================
 */
typedef enum adk_ph_dev_interrupt_events_e {
    ADK_PH_DEV_INTR_MSG_EVENT_SENT = 1 /*!<  */
} adk_ph_dev_interrupt_events_t;   /* ------  end of enum adk_ph_dev_interrupt_events_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_rx_queue_statistics_t adk_ph_dev.h "adk_ph_dev.h"   {{{2
 *
 * @brief Statistics for the PH Dev
 *
 *
 * =============================================================
 */
typedef struct adk_ph_dev_rx_queue_statistics_s {
    uint64_t    pdusEnqueued;
    uint64_t    pdusDequeued;
    uint64_t    pdusEnqDropped; /*! Deprecated */
    uint64_t    pdusDeqDropped;
    uint64_t    rxDisabled;
    uint64_t    bufferHighWatermark;
    uint64_t    bufferFull;
    uint64_t    congestStop;
    uint64_t    intrSent;
    uint64_t    intrSendFailed;
    uint64_t    pdusDroppedQosRed; /*! Deprecated */
    uint64_t    unknownType;
    uint64_t    palNextStageFailed;
} adk_ph_dev_rx_queue_statistics_t;   /* ------  end of struct adk_ph_dev_rx_queue_statistics_t  ------ */


/*!
 * =============================================================
 *
 * @def adk_ph_dev_handle_t   {{{2
 *
 * Handle indicating which PH device to use
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef adk_handle_t adk_ph_dev_handle_t;


/*!
 * =============================================================
 *
 * @def  adk_ph_dev_cbfn_handle_t
 *
 * Handle indicating which PH callback function to use
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */

typedef uint8_t adk_ph_dev_cbfn_handle_t;
/*!
 * =============================================================
 * @enum adk_ph_dev_device_type_t   {{{2
 *
 * @brief Type of device being added for PH
 *
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef enum {
    ADK_PH_DEV_NONE = 0,
    ADK_PH_DEV_TYPE_EVENTDEV,
} adk_ph_dev_device_type_t;


/*!
 * =============================================================
 *   Callback Functions   {{{2
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 * =============================================================
 */


typedef uint16_t (*adk_ph_dev_callback_fn) (
    uint8_t             moduleId,
    void               *dp_message,
    void               *userParam_p);

/*!
 * =============================================================
 *   Callback Functions   {{{2
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 * =============================================================
 */


typedef uint16_t (*adk_ph_dev_rx_parse_fn) (
    void               *task,
    void               **dp_message,
    adk_module_handle_t module_handle);

/*!
 * =============================================================
 *
 * @struct adk_ph_dev_rx_queue_info_s adk_ph_dev.h "adk_ph_dev.h"   {{{2
 *
 *
 * Structure for returning the info of a queue
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef struct adk_ph_dev_rx_queue_info_s {
    union {
        struct {
            /*
             * The Eventdev enqueue queue ID
             */
            uint16_t    queueId;
        } evd;
    } u;
} adk_ph_dev_rx_queue_info_t;


/*!
 * =============================================================
 *
 * @struct adk_ph_dev_rx_queue_config_t adk_ph_dev.h "adk_ph_dev.h"   {{{2
 *
 *
 * Structure for adding a queue
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef struct adk_ph_dev_rx_queue_config_s {
    union {
        struct {
            /*
             * The Eventdev enqueue queue ID
             */
            uint16_t    queueId;
        } evd;
    } u;
} adk_ph_dev_rx_queue_config_t;   /* ------  end of struct adk_ph_dev_rx_queue_config_t  ------ */


/*!
 * =============================================================
 * @struct adk_ph_dev_device_info_t  adk_ph.h "adk_ph.h"   {{{2
 *
 * @brief Structure for adding a queue to a queue group
 *
 * This structure is used by the adk_ph_dev_queue_group_create() and
 * adk_ph_dev_queue_add() functions to add the alloc/free/tx/rx queues.
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef struct adk_ph_dev_device_info_s {

    /*!
     *  Is the device currently in the active state
     */
    bool                            isActive;

    /*!
     *  Is the device's Rx currently in the enabled state
     */
    bool                            isRxEnabled;

    /*!
     * A new buffer will be created and the contents will be copied when true
     */
    bool    noZeroCopy;
    /*!
     * When true, if the ring buffer fill, the transmission will be disabled
     * for this device. The application must adk_ph_dev_queue_start() to begin
     * transmission again.
     */
    bool    congestionControlStop;
    /*!
     * When true control information will be written to the file descriptor
     * defined in controlFd
     */
    bool    interruptEnabled;

    /*!
     * If interruptEnabled is false, controlFd is not used
     */
    int     controlFd;

    /*!
     * The maximum number of Rx queues supported
     */
    uint8_t numRxQueues;

    /*!
     * There is one free/alloc pair per device. If the number of Rx 
     * queues is greater than 0, then freeQueue will be created.
     *
     * This needs to be a power of 2
     */
    uint32_t    freeRingSize;

    /*!
     * Information defining the transport device
     */
    adk_ph_dev_device_type_t    type;
    union {
        struct {
            /*!
             * Eventdev port to attach to
             */
            uint8_t     portId;
        } evd;
    } u;

} adk_ph_dev_device_info_t;   /* ------  end of struct adk_ph_dev_device_info_t  ------ */


/*!
 * =============================================================
 *
 * @struct adk_ph_dev_device_config_t  adk_ph.h "adk_ph.h"   {{{2
 *
 * @brief Structure for adding a queue to a queue group
 *
 * This structure is used by the adk_ph_dev_queue_group_create() and
 * adk_ph_dev_queue_add() functions to add the alloc/free/tx/rx queues.
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef struct adk_ph_dev_device_config_s {

    /*!
     * A new buffer will be created and the contents will be copied when true
     */
    bool    noZeroCopy;
    /*!
     * When true, if the ring buffer fill, the transmission will be disabled
     * for this device. The application must adk_ph_dev_queue_start() to begin
     * transmission again.
     */
    bool    congestionControlStop;
    /*!
     * When true control information will be written to the file descriptor
     * defined in controlFd
     */
    bool    interruptEnabled;

    /*!
     * If interruptEnabled is false, controlFd is not used
     */
    int     controlFd;

    /*!
     * The maximum number of Rx queues supported
     */
    uint8_t numRxQueues;

    /*!
     * There is one free/alloc pair per device. If the number of Rx
     * queues is greater than 0, then freeQueue will be created.
     *
     * If this is left at 0, an ADK default value will be used.
     *
     * This needs to be a power of 2
     */
    uint32_t    freeRingSize;

    /*!
     * Size of CPPI mbuf cache.
     *
     * If this is left at 0, an ADK default value will be used.
     *
     */
    uint32_t    mbuf_cache_sz;

    /*!
     * Information defining the transport device
     */
    adk_ph_dev_device_type_t    type;
    union {
        struct {
            /*!
             * Eventdev port to attach to
             */
            uint8_t     portId;
        } evd;
    } u;

    /*!
     * Flag for coalescing a multi-segment mbuf to a flat PDU buffer when Rx
     * callbacks are enabled.
     */
    bool    coalesceEnabled;   /*< flag to enable or disable coalescing */

    /*!
     * Size of buffer to create for coalescing chained PDUs to a flat PDU. If
     * this is 0 and coalesceEnabled is true, then the default ADK value
     * (ADK_PH_DEV_COALESCE_BUF_SIZE) will be used
     */
    uint32_t    coalesceBufSize;
} adk_ph_dev_device_config_t;   /* ------  end of struct adk_ph_dev_device_config_t  ------ */


/*!
 * =============================================================
 *
 * @struct adk_ph_dev_instance_config_t adk_ph_dev.h "adk_ph_dev.h"   {{{2
 *
 * @brief
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 *
 * =============================================================
 */
typedef struct adk_ph_dev_instance_config_s {
    /*!
     * CPPI instance ID that will be used to create the free and alloc queues
     */
    uint8_t     cppiInstanceId;
} adk_ph_dev_instance_config_t;   /* ------  end of struct adk_ph_dev_instance_config_t  ------ */

typedef struct adk_ph_dev_device_statistics_s {
    uint64_t                        freeQ_jobsEnqueued;
    uint64_t                        freeQ_jobsProcessed;
    uint64_t                        rateLimitingDrops;
} adk_ph_dev_device_statistics_t;

typedef struct adk_ph_dev_queue_statistics_s {
    int a;
} adk_ph_dev_queue_statistics_t;

/*
 * typedef struct adk_dev_tx_queue_info_s {
 *     int a;
 * } adk_dev_tx_queue_info_t;
 */

/*!
 * @addtogroup _phFunctions  PH Functions
 * @{
 *
 *
 * The API function declarations.   {{{1
 *
 * THIS IS PRELIMINARY AND IS SUBJECT TO CHANGE
 */


/*
 * This API is a global init for the PH instance. Each of the devices hang off
 * of this.
 */
adk_status_t adk_ph_dev_instance_create(       /*  {{{2  */
    adk_sys_handle_t        sysHandle,
    adk_ph_dev_instance_config_t *config_p);


adk_status_t adk_ph_dev_instance_get(       /*  {{{2  */
    adk_sys_handle_t        sysHandle);


adk_status_t adk_ph_dev_device_configure(       /*  {{{2  */
    const adk_ph_dev_device_config_t   *config_p,
    uint16_t                           *deviceId_p);


adk_status_t adk_ph_dev_device_start(       /*  {{{2  */
    uint16_t deviceId);


adk_status_t adk_ph_dev_device_stop(       /*  {{{2  */
    uint16_t deviceId);


adk_status_t adk_ph_dev_device_close(       /*  {{{2  */
    uint16_t deviceId);


adk_status_t adk_ph_dev_device_info_get(       /*  {{{2  */
    uint16_t                    deviceId,
    adk_ph_dev_device_info_t   *devInfo_p);


adk_status_t adk_ph_dev_device_stats_get(       /*  {{{2  */
    uint16_t                        deviceId,
    adk_ph_dev_device_statistics_t *stats);


adk_status_t adk_ph_dev_device_stats_print(       /*  {{{2  */
    uint16_t            deviceId);


adk_status_t
adk_ph_dev_callback_fn_add(
    adk_ph_dev_callback_fn       cbFn,
    void                        *userParam,
    adk_ph_dev_cbfn_handle_t    *cbFnHandle_p);

adk_status_t
adk_ph_dev_callback_fn_rem(
        adk_ph_dev_cbfn_handle_t    cbFnHandle_p);

adk_status_t
adk_ph_dev_rx_parse_register(
    adk_ph_dev_rx_parse_fn     parseFn,
    uint8_t                    moduleId);

adk_status_t
adk_ph_dev_rx_parse_unregister(
    uint8_t                    moduleId);

adk_status_t adk_ph_dev_rx_queue_setup(       /*  {{{2  */
    uint16_t                        deviceId,
    uint16_t                        rxQueueId,
    adk_ph_dev_rx_queue_config_t   *queueConf_p);


adk_status_t adk_ph_dev_rx_queue_remove(       /*  {{{2  */
    uint16_t                        deviceId,
    uint16_t                        rxQueueId);


adk_status_t adk_ph_dev_rx_queue_stats_get(       /*  {{{2  */
    uint16_t                            deviceId,
    uint16_t                            rxQueueId,
    adk_ph_dev_rx_queue_statistics_t    *queueStats_p);


adk_status_t adk_ph_dev_rx_queue_stats_print(       /*  {{{2  */
    uint16_t                        deviceId,
    uint16_t                        rxQueueId);


adk_status_t adk_ph_dev_rx_queue_info_get(       /*  {{{2  */
    uint16_t                    devId,
    uint16_t                    rxQueueId,
    adk_ph_dev_rx_queue_info_t *queueInfo_p);

uint16_t adk_ph_dev_rx_burst(       /*  {{{2  */
    uint16_t            devId,
    uint16_t            queueId,
    struct rte_mbuf   **rxPkts,
    const uint16_t      maxNumPkts);

adk_status_t adk_ph_dev_buffer_free(       /*  {{{2  */
    uint16_t            devId,
    struct rte_mbuf    *mbuf_p);


adk_status_t adk_ph_dev_dp_instance_init(       /*  {{{2  */
    void);

/*!
 *
 * Helper functions   {{{1
 */
uint16_t adk_ph_dev_cb_fn_id_get(       /*  {{{2  */
    struct rte_mbuf *mbuf_p);


int adk_ph_dev_pdu_offset(       /*  {{{2  */
    struct rte_mbuf *mbuf_p);


uint16_t adk_ph_dev_pdu_size(       /*  {{{2  */
    struct rte_mbuf *mbuf_p);

uint32_t adk_ph_dev_coalesce_pdu(       /*  {{{2  */
    struct rte_mbuf    *inMbuf_p,
    void               *outPdu_p,
    uint32_t            outPduSize);

#endif /*! end of __ADK_PH_DEV_H__ */

/*! end addtogroup _phFunctions }}}1
 * @}
 *
 * end addtogroup _ph
 * @}
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 *
 */
