/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ph_dev_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ph  PH Module
 *
 * @{
 *
 * @brief
 *
 * Contains the structure/union/macro definitions used by the PDU handler.
 * This file needs to be included by the user of the PDU handler.
 *
 *  **********************************************************************/
#ifndef __ADK_PH_DP_H__
#define __ADK_PH_DP_H__

#include "adk_sys.h"



/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_metadata_t  "adk_ph_dev_dp.h"
 *
 * @brief The structure to be filled out by the appliation when sending PDUs
 * to the host via PH
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_metadata_s {
    uint8_t     priority;
    uint8_t     cbFnId;
    uint8_t     moduleId;
    uint32_t    policerFlowId;
    void       *imc_param_start;
} adk_ph_dev_dp_metadata_t;



/*!
 * @addtogroup _phFunctions  PH DP Functions
 * @{
 */
/*!
 * The API function declarations.
 */

adk_status_t adk_ph_dev_dp_init(void);

adk_status_t adk_ph_dev_dp_to_cp_send(void *pdu_p, uint8_t CbFnBatch, void* CbParamBatch);

adk_status_t adk_ph_dev_staged_dp_to_cp_send(void *pdu_p);

void adk_ph_dev_pal_batch_handle_return(
    uint16_t devId,
    uint16_t queueId,
    uint8_t cbEventType,
    bool sendSuccess);

void adk_ph_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#endif /*! end of __ADK_PH_DP_H__ */

/*! end addtogroup _phFunctions
 * @}
 *
 * end addtogroup _ph
 * @}
 */
