/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ph_dev_dp_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ph  PH Module
 *
 * @{
 *
 * @brief
 *
 * Contains the structure/union/macro definitions used by the PDU handler.
 * This file should not be included by the PDU handler user.
 *
 *  **********************************************************************/
#ifndef __ADK_PH_DEV_DP_INTERNAL_H__
#define __ADK_PH_DEV_DP_INTERNAL_H__

#include <inttypes.h>
#include <stdbool.h>

#include "rte_ring.h"

#include "adk_ph_dev.h"

#define ADK_PH_DEV_DP_SHM_COOKIE        0x7cda0572
#define ADK_PH_DEV_DP_DEVICE_COOKIE     0x269fc6a2

#define ADK_PH_DEV_LOCK_SHARED          true
#define ADK_PH_DEV_LOCK_VALUE           1

#define ADK_PH_DEV_DEVICE_ID_BASE       (ADK_DEV_NET_ID_RSVD_MAX + 1)
#define ADK_PH_DEV_DEVICE_ID_MAX        (ADK_PH_DEV_DEVICE_ID_BASE + ADK_PH_DEV_MAX_NUM_DEVICES - 1)

#define ADK_PH_DEV_DP_SHM_MEMZONE_NAME "ADK_PH_DP_DEV_SHM_MEMZONE"

#define ADK_PH_DP_PTR_ALIGN             sizeof(void *)

#if 1
#define ADK_PH_DEV_CB_FN_ID_SET(pdu_p, cbFnId) \
    ADK_METADATA_PARAM((pdu_p), func_idx) = (cbFnId)
#else
#define ADK_PH_DEV_CBFNID_INDEX         1
#define ADK_PH_DEV_CB_FN_ID_SET(pdu_p, cbFnId) \
    ((adk_ph_pdu_buf_rx_t *)(pdu_p))->pduParams[ADK_PH_DEV_CBFNID_INDEX] = (cbFnId);
#endif

/* Data Structures {{{1 */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_stats_t adk_ph_dev_dp_internal.h "adk_ph_dev_dp_internal.h"
 *
 * @brief Statistics for the PH Dev
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_stats_s {
    /*Packets drop due to rate limiting*/
    uint64_t    rateLimitingDrops;
} __attribute__ ((aligned(64))) adk_ph_dev_dp_stats_t;   /* ------  end of struct adk_ph_dev_dp_queue_stats_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_queue_stats_t adk_ph_dev_dp_internal.h "adk_ph_dev_dp_internal.h"
 *
 * @brief Statistics for the PH Dev
 *
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_queue_stats_s {
    struct rx_s {
        uint64_t    pdusEnqueued;
        uint64_t    pdusDequeued;
        /* uint64_t    pdusEnqDropped; */
        uint64_t    pdusDeqDropped;
        uint64_t    rxDisabled;
        uint64_t    bufferHighWatermark;
        uint64_t    bufferFull;
        uint64_t    congestStop;
        uint64_t    intrSent;
        uint64_t    intrSendFailed;
        /* uint64_t    pdusDroppedQosRed; */
        uint64_t    unknownType;
        uint64_t    palNextStageFailed;
    } rx;
    struct tx_s {
    } tx;
} __attribute__ ((aligned(64))) adk_ph_dev_dp_queue_stats_t;   /* ------  end of struct adk_ph_dev_dp_queue_stats_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_queue_info_t adk_ph_dev_dp_internal.h "adk_ph_dev_dp_internal.h"
 *
 *
 * @brief Parameters for describing the queue
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_queue_info_s {
    union {
        struct {
            uint16_t            queueId;
            uint16_t            devId;
        } evd;
    } u;

    /*!
     * Statistics data structure
     */
    adk_ph_dev_dp_queue_stats_t    stats[ADK_MAX_NUM_CORES];

} __attribute__ ((aligned(64))) adk_ph_dev_dp_queue_info_t;   /* ------  end of struct adk_ph_dev_dp_queue_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_device_info_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"   {{{2
 *
 * @brief DP part of structure for an device instance
 *
 * DP device specific items. Only things in the data structure are what is
 * needed by the DP. The sister structure adk_ph_dev_t holds information
 * that is only needed by the CP.
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_device_info_s
{
    uint32_t                cookie;
    /*
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Internal state information
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     */
    /*
     * Flag indicating the DP is using the data structure. This will be
     * checked in the CP to ensure that freeing the structure won't cause the
     * DP to crash. The DP will check to see if the pointer to the struct is
     * not NULL and grab it if that is the case. It will then set this flag.
     * The CP will set the pointer to the struct to NULL, do some other work,
     * then check the flag. If it is cleared, it will free the struct.
     */
    bool    inUse;
    bool    isActive;
    bool    isRxEnabled;
    uint8_t palId;


    /*
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * User configuration information
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     *
     * Booleans are false by default
     *
     */
    /*!
     * A new buffer will be created and the contents will be copied when true
     */
    bool    noZeroCopy;
    /*!
     * When true, if the ring buffer fill, the transmission will be disabled
     * for this device. The application must adk_ph_dev_queue_start() to begin
     * transmission again.
     */
    bool    congestionControlStop;
    /*!
     * When true control infomation will be written to the file descriptor
     * defined in controlFd
     */
    bool    interruptEnabled;

    /*
     * If interruptEnabled is false, controlFd is not used
     */
    int     controlFd;

    /*!
     * Information defining the transport device
     *
     * We are using a union below so that if/when we add another transport
     * type, we don't need to update the data structure in a non-source
     * compatible way.
     */
    adk_ph_dev_device_type_t    type;
    union {
        struct {
            /*
             * Eventdev port to attach to
             */
            uint8_t     portId;
        } evd;
    } u;

    /*
     * The PH device can have multple queues per device. They are stored here
     */
    adk_ph_dev_dp_queue_info_t *rxQueues[ADK_PH_DEV_MAX_NUM_QUEUES];

    /*
     * The PH device statistics 
     */
    adk_ph_dev_dp_stats_t devStats;

} __attribute__ ((aligned(64))) adk_ph_dev_dp_device_info_t;   /* ------  end of struct adk_ph_dev_dp_device_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_dp_shm_info_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"   {{{2
 *
 * Structure in shared memory used as an anchor for PH DP information
 *
 * The CP will include a pointer to this structure so that it can update the
 * DP needed fields. This structure is to allow separation between the CP and
 * DP data structures.
 *
 * =============================================================
 */
typedef struct adk_ph_dev_dp_shm_info_s
{
    uint32_t                cookie;
    void                   *dpLock_p;

    adk_ph_dev_dp_device_info_t     *devices[ADK_PH_DEV_DEVICE_ID_MAX + 1];
} __attribute__ ((aligned(64))) adk_ph_dev_dp_shm_info_t;   /* ------  end of struct adk_ph_dev_dp_shm_info_t  ------ */


#endif /*! end of __ADK_PH_DEV_DP_INTERNAL_H__ */

/*! end addtogroup _ph }}}1
 * @}
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 *
 */
