/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * file  adk_ph_dev_dp_lttng_debug.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * brief
 *
 * This file contains definitions of tracepoint providers and events.
 *
 ***********************************************************************/

#include "adk_modules_enabled.h"
#if defined(ADK_MODULE_SIF_LTTNG_DP_ENABLED)

#ifndef ADK_DEBUG
#define tracepoint(...)
#else

#ifndef _TRACEPOINT_COM_INTEL_ADK_DP_PH_H

#  undef TRACEPOINT_PROVIDER
#  define TRACEPOINT_PROVIDER com_intel_adk_ph_dp

/*
 *  * include file (this files's name)
 *   */
#  undef TRACEPOINT_INCLUDE
#  define TRACEPOINT_INCLUDE "./adk_ph_dev_dp_lttng.h"

#endif /* ifndef _TRACE_POINT_COM_INTEL_ADK_DP_PH_H */

#if !defined(_TRACEPOINT_COM_INTEL_ADK_PH_DEV_DP_LTTNG_H) || defined(TRACEPOINT_HEADER_MULTI_READ)
#define _TRACEPOINT_COM_INTEL_ADK_PH_DEV_DP_LTTNG_H

#ifndef _TRACEPOINT_COM_INTEL_ADK_DP_PH_H

#  ifdef __cplusplus
extern "C" {
#  endif
#endif

#include <stdint.h>
#include <unistd.h>
#include <inttypes.h>

#include <lttng/tracepoint.h>

/*
 * LTTng tracepoints unique to PH
 */
TRACEPOINT_EVENT(com_intel_adk_ph_dp, \
                 intel_adk_ph_dp_input, \
                 TP_ARGS(const uint16_t,    devId, \
                         const uint16_t,    queueId, \
                         const uint8_t,     priority, \
                         const uint8_t,     cbFnId, \
                         void *,            pdu_p), \
                 TP_FIELDS( \
                            ctf_string(Module_Name, "adk_ph") \
                            ctf_string(File_Name, __FILE__) \
                            ctf_integer(int, Line_no, __LINE__) \
                            ctf_integer(uint16_t, devId, devId) \
                            ctf_integer(uint16_t, queueId, queueId) \
                            ctf_integer(uint8_t, priority, priority) \
                            ctf_integer(uint8_t, cbFnId, cbFnId) \
                            ctf_integer_hex(void *, pdu_p, pdu_p) \
                          ) \
                ) \
TRACEPOINT_LOGLEVEL(com_intel_adk_ph_dp, intel_adk_ph_dp_input, TRACE_DEBUG)

TRACEPOINT_EVENT(com_intel_adk_ph_dp, \
                 intel_adk_ph_dp_input_2, \
                 TP_ARGS(const char *,  message, \
                         const uint16_t,    queueId, \
                         const uint8_t,     cbFnId), \
                 TP_FIELDS( \
                            ctf_string(Module_Name, "adk_ph") \
                            ctf_string(File_Name, __FILE__) \
                            ctf_integer(int, Line_no, __LINE__) \
                            ctf_string(Message, message) \
                            ctf_integer(uint16_t, queueId, queueId) \
                            ctf_integer(uint8_t, cbFnId, cbFnId) \
                          ) \
                ) \
TRACEPOINT_LOGLEVEL(com_intel_adk_ph_dp, intel_adk_ph_dp_input_2, TRACE_DEBUG)

#endif /* _TRACEPOINT_COM_INTEL_ADK_PH_DEV_DP_LTTNG_H */

        /* This part must be outside ifdef protection */
#ifndef _TRACEPOINT_COM_INTEL_ADK_DP_PH_H
#  include <lttng/tracepoint-event.h>

#  ifdef __cplusplus
}
#  endif
#endif
#endif
#endif  /* #if defined(ADK_MODULE_SIF_LTTNG_DP_ENABLED) */
