/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ph_dev_export.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ph  PH Module
 *
 * @{
 *
 * @brief
 *
 * Contains the structure/union/macro definitions used by the PDU handler and
 * intended for use by the modules (not the customer)
 *
 *  **********************************************************************/
#ifndef __ADK_PH_DEV_EXPORT_H__
#define __ADK_PH_DEV_EXPORT_H__

#include "adk_sys.h"
#include "adk_metadata.h"

/*!
 * @addtogroup _phFunctions  PH Dev Functions
 * @{
 */
/*!
 * The API function declarations.       {{{1
 */

/*!
 * =============================================================
 * @internal
 * adk_ph_dev_tx_subevent_set       {{{2
 * @param[in]  pdu_p: pointer to the metadata to be updated
 * @param[in]  subEventId: value to write
 *
 * @retval  <NONE>
 *
 * @brief  Set the Subevent ID that will be used by the PAL in the DP to call
 * the appropriate function
 *
 *
 * =============================================================
 */
static inline void
adk_ph_dev_tx_subevent_set (
    struct rte_mbuf    *pdu_p,
    uint8_t             subEventId)
{
    ADK_METADATA_PARAM((pdu_p), func_idx) = (subEventId);
}    /* -----  end of function adk_ph_dev_tx_subevent_set  ----- */


uint16_t adk_ph_dev_tx_burst(       /*  {{{2  */
    uint16_t            deviceId,
    uint16_t            rxQueueId,
    struct rte_mbuf   **txPkts,
    const uint16_t      maxNumPkts);


#endif /*! end of __ADK_PH_DEV_EXPORT_H__ */

/*! end addtogroup _phFunctions       }}}1
 * @}
 *
 * end addtogroup _ph
 * @}
 */
