/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_ph_dev_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _ph  PH Module
 *
 * @{
 *
 * @brief
 *
 * Contains the structure/union/macro definitions used by the PDU handler.
 * This file should not be included by the PDU handler user.
 *
 *  **********************************************************************/
#ifndef __ADK_PH_DEV_INTERNAL_H__
#define __ADK_PH_DEV_INTERNAL_H__

#include "adk_ph_dev.h"
#include "adk_ph_dev_dp_internal.h"
#include <adk_sif_cfg_include.h>
#include <adk_platform.h>
#include <adk_module_mgr.h>
#include <adk_pal.h>
#include <adk_metadata.h>

/* true if x is a power of 2 */
#define POWER_OF_2(x) (0 == (((x)-1) & (x)))

#define ADK_PH_RTE_MAX_CPPI_INSTANCES   255
#define ADK_PH_DEV_MAX_RING_NAME_LEN    32

#define ADK_PH_DEV_CPPI_ALLOC_RING_SIZE 256
#define ADK_PH_DEV_CPPI_FREE_RING_SIZE  256
#define ADK_PH_DEV_CPPI_MBUF_CACHE_SIZE (ADK_PH_DEV_CPPI_ALLOC_RING_SIZE >> 4)

#define ADK_PH_DEV_SHM_COOKIE       0x39a7c13e
#define ADK_PH_DEV_DEVICE_COOKIE    0x44a10ed2

#define ADK_PH_DEV_SHM_MEMZONE_NAME "ADK_PH_DEV_SHM_MEMZONE"

#define ADK_PH_CPPI_FREE_QUEUE_ID   0
#define ADK_PH_CPPI_ALLOC_QUEUE_ID  1

#define ADK_PH_DEV_COALESCE_BUF_SIZE    16384

#define ADK_PH_DEV_SLEEP_TIME   25 /* 25 usec */
#define ADK_PH_DEV_SLEEP_LOOPS   40 /* total wait time of 1ms */
/*
 * Macro assumes that if the CP cookie is good, the DP cookie will also be
 * good, since the DP cookie is set before the CP cookie.
 */
#define ADK_PH_DEV_SHM_VALIDATE()  \
    if ((NULL == phShm_p) || \
        (ADK_PH_DEV_SHM_COOKIE != phShm_p->cookie)) \
    { \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_SYS_NOT_INITIALIZED); \
        ADK_FORCE_LOG_ERROR("PH shared memory not initialized\n"); \
        goto _return; \
    }

#define ADK_PH_DEV_DEVICE_VALIDATE() \
    if ((devId < ADK_PH_DEV_DEVICE_ID_BASE) || \
        (devId >= ADK_PH_DEV_DEVICE_ID_MAX)) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "Invalid devId (%d) valid range is %d to %d\n", \
                      devId, ADK_PH_DEV_DEVICE_ID_BASE, \
                      (ADK_PH_DEV_DEVICE_ID_MAX - 1)); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail; \
    } \
 \
    cpDev_p = phShm_p->devices[devId]; \
 \
    if ((NULL == cpDev_p) || \
        (ADK_PH_DEV_DEVICE_COOKIE != cpDev_p->cookie)) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "Device devId (%d) is NULL or invalid cookie\n", \
                      devId); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail; \
    }


#define ADK_PH_DEV_RX_QUEUE_VALIDATE() \
    if (queueId >= phShm_p->devices[devId]->numRxQueues) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "Invalid Rx queueId (%d) for devId (%d) max is %d\n", \
                      queueId, devId, phShm_p->devices[devId]->numRxQueues); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    } \
 \
    if (NULL == cpDev_p->rxQueues[queueId]) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "Rx queueId %d does not exists for device %d\n", \
                      queueId, devId); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    }

#define ADK_PH_DEV_DEVICE_ERROR_IF_ACTIVE_STATE_IS(__state__) \
    if ((__state__) == cpDev_p->isActive) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "devId (%d) is %s\n", devId, \
                      ((__state__ == false) ? "inactive" : "active")); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    }

/*
 * Validate that the DP has been setup
 */
#define ADK_PH_DEV_DP_SHM_VALIDATE()  \
    if ((NULL == phShm_p->dp_p) || \
        (ADK_PH_DEV_DP_SHM_COOKIE != phShm_p->dp_p->cookie)) \
    { \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_SYS_NOT_INITIALIZED); \
        ADK_FORCE_LOG_ERROR("PH DP shared memory not initialized\n"); \
        goto _return; \
    }

/*
 * Validate the DP device has been configured
 */
#define ADK_PH_DEV_DP_DEVICE_VALIDATE() \
    dpDev_p = phShm_p->dp_p->devices[devId]; \
 \
    if ((NULL == dpDev_p) || \
        (ADK_PH_DEV_DP_DEVICE_COOKIE != dpDev_p->cookie)) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "DP Device devId (%d) is NULL or invalid cookie\n", \
                      devId); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    }

/*
 * Validate the DP Queue has been configured
 */
#define ADK_PH_DEV_DP_RX_QUEUE_VALIDATE() \
    if (NULL == dpDev_p->rxQueues[queueId]) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "DP Rx queueId %d does not exists for device\n", \
                      queueId); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    }

    /*
     * Validate the DP and CP states are the same
     */
#define ADK_PH_DEV_CP_DP_STATE_VALIDATE() \
    if ((cpDev_p->isActive != dpDev_p->isActive) || \
        (cpDev_p->isRxEnabled != dpDev_p->isRxEnabled)) \
    { \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                "DP and CP state mismatch for device %d: CP:DP" \
                "isActive: %d:%d, isRxEnabled: %d:%d\n", \
                devId, \
                cpDev_p->isActive , dpDev_p->isActive, \
                cpDev_p->isRxEnabled, dpDev_p->isRxEnabled);\
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INCONSISTENT_STATE); \
        goto _parameter_validation_fail_need_unlock; \
    }


#define ADK_PH_DEV_RECHECK_CP_DEV_VALID() \
    if ((NULL == cpDev_p) || \
        (ADK_PH_DEV_DEVICE_COOKIE != cpDev_p->cookie)) \
    { \
        ADK_RELEASE_LOCK(cpDev_p->lock_p, retValue); \
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), \
                      "Device devId (%d) has been removed\n", \
                      devId); \
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER); \
        goto _parameter_validation_fail_need_unlock; \
    }


/* Data Structures {{{1 */



typedef struct adk_ph_dev_device_rx_stats_s {
    uint64_t    pdusDequeued;
    uint64_t    pdusDeqDropped;
} adk_ph_dev_rx_queue_stats_t;

typedef struct adk_ph_dev_queue_stats_s {
        adk_ph_dev_rx_queue_stats_t rx;
} adk_ph_dev_queue_stats_t;

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_queue_info_t adk_ph_dev.h "adk_ph_dev.h"
 *
 * @brief
 *
 * Detailed description starts here
 *
 * =============================================================
 */
typedef struct adk_ph_dev_queue_info_s {
    union {
        struct {
            uint16_t            queueId;
        } evd;
    } u;

    adk_ph_dev_queue_stats_t    stats;
} adk_ph_dev_queue_info_t;   /* ------  end of struct adk_ph_dev_queue_info_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_device_internal_info_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"   {{{2
 *
 * @brief CP part of structure for an device instance
 *
 * Device specific items. Some of the needed items are in the DP sister
 * structure adk_ph_dev_dp_device_t
 *
 * =============================================================
 */
typedef struct adk_ph_dev_device_internal_info_s {
    uint32_t    cookie;
    /*
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Internal state information
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     */
    bool        isActive;
    bool        isRxEnabled;
    uint32_t    palId;

    uint16_t    devId;
    uint16_t    cppiDeviceId;
    void       *lock_p;

    /*
     * free info
     *
     * Rings created for the CPPI
     *
     */
    struct rte_ring    *freeRing_p;

    /*
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * User configuration information
     * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     *
     * Booleans are false by default
     *
     */

    /*!
     * A new buffer will be created and the contents will be copied when true
     */
    bool    noZeroCopy;
    /*!
     * When true, if the ring buffer fill, the transmission will be disabled
     * for this device. The application must adk_ph_dev_queue_start() to begin
     * transmission again.
     */
    bool    congestionControlStop;
    /*!
     * When true control infomation will be written to the file descriptor
     * defined in controlFd
     */
    bool    interruptEnabled;

    /*
     * If interruptEnabled is false, controlFd is not used
     */
    int     controlFd;

    /*!
     * Information defining the transport device
     *
     * We are using a union below so that if/when we add another transport
     * type, we don't need to update the data structure in a non-source
     * compatible way.
     */
    adk_ph_dev_device_type_t    type;
    union {
        struct {
            /*
             * Eventdev port to attach to
             */
            uint8_t     portId;
        } evd;
    } u;

    /*!
     * The maximum number of Rx queues supported
     */
    uint8_t numRxQueues;
    /*
     * User configurable Free variables
     *
     * The set the size of the rings for the free. These must be
     * power-of-2. If this is 0, the a default ADK value will be used.
     */
    uint32_t            freeRingSize;

    adk_ph_dev_queue_info_t    *rxQueues[ADK_PH_DEV_MAX_NUM_QUEUES];

    /*!
     * Items for coalescing a multi-segment mbuf to a single mbuf. These
     * will only be used when Rx callback functions are enabled.
     */
    /*!
     * Flag indicating coalesce is enabled for the callback functions
     */
    bool        coalEnabled;
    /*!
     * Size of the buffer to copy PDUs to. If the application passes this as 0
     * and coalesce is enabled, the default ADk size
     * (ADK_PH_DEV_COALESCE_BUF_SIZE) will be used
     */
    uint32_t    coalBufSize;
    /*!
     * Data buffer where the PDU will be copied if we're doing a coalesce
     * before calling the callback function
     */
    uint8_t    *coalBuf_p;
} adk_ph_dev_device_internal_info_t;   /* ------  end of struct adk_ph_dev_device_internal_info_t  ------ */


/*!
 * =============================================================
 *
 * @struct adk_ph_dev_cb_fn_id_map_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"
 *
 * @brief Element for Array of callback functions
 *
 * =============================================================
 */
typedef struct adk_ph_dev_cb_fn_id_map_s {
    adk_ph_dev_callback_fn  cbFn;
    void   *cbParams;
} adk_ph_dev_cb_fn_id_map_t;


/*!
 * =============================================================
 *
 * @struct adk_ph_dev_parse_fn_map_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"
 *
 * @brief Element for Array of callback functions
 *
 * =============================================================
 */
typedef struct adk_ph_dev_parse_fn_map_t {
    adk_ph_dev_rx_parse_fn  parseFn;
    adk_module_handle_t     moduleHandle;
} adk_ph_dev_parse_fn_map_t;
/*!
 * =============================================================
 * @internal
 * @struct adk_ph_dev_shm_info_t adk_ph_dev_internal.h "adk_ph_dev_internal.h"   {{{2
 *
 * Structure in shared memory used as an anchor for PH information
 *
 *
 * =============================================================
 */
typedef struct adk_ph_dev_shm_info_s
{
    uint32_t                    cookie;
    uint8_t                     numDevices;
    adk_sys_handle_t            sysHandle;
    void                       *lock_p;
    uint16_t                    cppiInstanceId;

    /* CbFn Handle will be unique 8 bits value  Upper 2 bits indicates 
    EventType which will be 0 for PH, Lower 6 bits indicating cbFnID*/

    adk_ph_dev_cb_fn_id_map_t cbFnIdMap[ADK_PH_CBFN_MAX];

    adk_ph_dev_parse_fn_map_t   parseFnMap[ADK_PH_MODULE_ID_MAX+1];
    /*
     * Pointer to the DP shared memory zone
     */
    adk_ph_dev_dp_shm_info_t *dp_p;


    adk_ph_dev_device_internal_info_t    *devices[ADK_PH_DEV_DEVICE_ID_MAX + 1];
} adk_ph_dev_shm_info_t;   /* ------  end of struct adk_ph_dev_shm_info_t  ------ */



/*!
 * Internal function declarations.  {{{1
 */
adk_status_t adk_ph_dev_shm_create(
    adk_sys_handle_t                sysHandle,
    adk_ph_dev_instance_config_t   *conf_p,
    adk_ph_dev_shm_info_t         **phShm_p);

adk_status_t adk_ph_dev_shm_get(
    adk_sys_handle_t                sysHandle,
    adk_ph_dev_shm_info_t         **phShm_p);

uint16_t adk_ph_dev_next_dev_get(
    adk_ph_dev_shm_info_t  *phShm_p);

adk_status_t adk_ph_dev_cppi_device_get(
    adk_sys_handle_t    sysHandle,
    uint8_t             instanceId,
    uint16_t            deviceId);

adk_status_t adk_ph_dev_cppi_device_free(
    adk_sys_handle_t    sysHandle,
    uint8_t             instanceId,
    uint16_t            deviceId);

adk_status_t adk_ph_dev_ring_create(
    adk_sys_handle_t    sysHandle,
    uint16_t            deviceId,
    uint16_t            queueId,
    char                ringType,
    uint16_t            allocFreeRingSize,
    struct rte_ring   **ring_p);

void adk_ph_dev_ring_destroy(
    adk_sys_handle_t    sysHandle,
    struct rte_ring    *ring_p);

adk_status_t adk_ph_dev_dp_rx_queue_stats_get(
    adk_ph_dev_dp_queue_info_t     *queue_p,
    adk_ph_dev_dp_queue_stats_t    *stats_p);

#endif /*! end of __ADK_PH_DEV_INTERNAL_H__ */

/*! end addtogroup _ph }}}1
 * @}
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 *
 */
