/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_SIF_CFG_EAL_H__
#define __ADK_SIF_CFG_EAL_H__

#define ADK_SIF_CFG_EAL_NUM_ARGS 9

typedef enum adk_sif_cfg_eal_process_type_e{
    ADK_SIF_CFG_EAL_PROCESS_TYPE_AUTO = -1,   /* Auto detection of primary or secondary process */
    ADK_SIF_CFG_EAL_PROCESS_TYPE_PRIMARY = 0, /* Primary process - Default */
    ADK_SIF_CFG_EAL_PROCESS_TYPE_SECONDARY,   /* Secondary process */
    ADK_SIF_CFG_EAL_PROCESS_TYPE_INVALID,
}adk_sif_cfg_eal_process_type_t;

typedef struct adk_sif_cfg_eal_info_s{
    uint8_t     valid;
    /* Internal tables */
    uint32_t    master_lcore;                                 /* Control logical core */
    uint32_t    worker_lcores[ADK_SIF_CFG_MAX_WORKER_LCORES];        /* List of DP logical cores */
    int32_t     worker_lcore_scoket[ADK_SIF_CFG_MAX_WORKER_LCORES];  /* Socket ID of DP logical cores */
    uint16_t    mem_channels;                                 /* Number of memory channels */

    /* Occupancy of internal tables */
    uint32_t    num_worker_lcores;    
    uint32_t    process_type;                                 /* Process type - primary/secondary/auto */
}adk_sif_cfg_eal_info_t;

typedef struct adk_sif_cfg_eal_info_tbl_s{
    adk_sif_cfg_eal_info_t      eal_info[ADK_SIF_CFG_MAX_EAL];
    uint32_t                    size;
    uint32_t                    cur;
}adk_sif_cfg_eal_info_tbl_t;

/* Function prototypes */
adk_status_t
adk_sif_cfg_eal_init(adk_sif_cfg_info_t *cfg_info);

adk_status_t
adk_sif_cfg_eal_configure(adk_sif_cfg_info_t *cfg_info_hdl);

adk_status_t
adk_sif_cfg_eal_create(adk_sif_cfg_info_t *cfg_info_hdl,
                       adk_cfg_eal_params_t *params,
                       adk_sif_cfg_eal_info_t **eal_info);

adk_status_t
adk_sif_cfg_eal_info_alloc(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                           adk_sif_cfg_eal_info_t **info);

adk_status_t
adk_sif_cfg_eal_info_free(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                          adk_sif_cfg_eal_info_t *info);

adk_status_t
adk_sif_cfg_eal_info_rewind(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl);

adk_status_t
adk_sif_cfg_eal_info_set(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                         adk_cfg_eal_params_t *params,
                         adk_sif_cfg_eal_info_t **info);

adk_status_t
adk_sif_cfg_eal_info_get(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                         adk_sif_cfg_eal_info_t **info);

adk_status_t
adk_sif_cfg_eal_lcore_check(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                            uint32_t lcore_id);

adk_status_t
adk_sif_cfg_eal_worker_socket_check(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                                    int32_t socket_id);


#endif
