/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_SIF_CFG_PIPELINE_H__
#define __ADK_SIF_CFG_PIPELINE_H__

#include "adk_sif_constants.h"
#include "adk_sif_cfg_pipeline_common.h"
#include "adk_cfg_pipeline.h"
#include "adk_sif_cfg.h"

typedef enum adk_sif_cfg_pipeline_bind_type_e{
    _PIPELINE_BIND = 0,
    _PORT_RX_BIND,
    _RING_RX_BIND,
    _PORT_TX_BIND,
    _RING_TX_BIND
}adk_sif_cfg_pipeline_bind_type_t;

typedef struct adk_sif_cfg_pipeline_entry_module_s{
    adk_sif_cfg_pipeline_port_q_t       port_q_in[ADK_SIF_CFG_MAX_PORT];
    adk_sif_cfg_pipeline_ring_q_t       ring_q_in[ADK_SIF_CFG_MAX_RING_QUEUES];
    adk_sif_cfg_pipeline_buff_t         buff_in[ADK_SIF_CFG_MAX_PKT_BUFFS];
    bool                          qat_rx;
    
    /* Occupancy of internal tables */
    uint32_t                            num_port_q_in;
    uint32_t                            num_ring_q_in;
    uint32_t                            num_buff_in;
}adk_sif_cfg_pipeline_entry_module_t;

typedef struct adk_sif_cfg_pipeline_dest_module_s{
    //adk_sif_cfg_pipeline_modules_t      mod;
    adk_sif_cfg_pipeline_modules_t      pktio;
    uint32_t                            func_idx;
    void                                *ring;
    uint32_t                            ring_id;
    uint8_t                             valid;
    struct adk_sif_cfg_pipeline_info_s  *next_entry;
}adk_sif_cfg_pipeline_dest_module_t;

typedef struct adk_sif_cfg_pipeline_exit_module_s{
    adk_sif_cfg_pipeline_dest_module_t  dest_module[ADK_SIF_CFG_PIPELINE_MODULE_MAX];
    adk_sif_cfg_pipeline_stage_t        stage;
    bool                          qat_tx;
    uint8_t                             valid;
}adk_sif_cfg_pipeline_exit_module_t;

typedef struct adk_sif_cfg_pipeline_info_s{
    /* Configuration info table */
    adk_sif_cfg_pipeline_exit_module_t  exit_module[ADK_SIF_CFG_PIPELINE_MODULE_MAX];
    adk_sif_cfg_pipeline_entry_module_t entry_module;  
    uint8_t                             valid;
}adk_sif_cfg_pipeline_info_t;

typedef struct adk_sif_cfg_pipeline_info_tbl_s{
    adk_sif_cfg_pipeline_info_t         pipeline_info[ADK_MAX_NUM_CORES][ADK_SIF_CFG_PIPELINE_MODULE_MAX];
    uint32_t                            size;
    uint32_t                            cur;
}adk_sif_cfg_pipeline_info_tbl_t;

typedef struct adk_sif_cfg_pipeline_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *src_info;
    adk_sif_cfg_pipeline_info_t         *dst_info;
}adk_sif_cfg_pipeline_bind_info_t;

typedef struct adk_sif_cfg_pipeline_port_rx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_port_rx_bind_info_t;

typedef struct adk_sif_cfg_pipeline_ring_rx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_ring_rx_bind_info_t;

typedef struct adk_sif_cfg_pipeline_buff_rx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_buff_rx_bind_info_t;

typedef struct adk_sif_cfg_pipeline_port_tx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_port_tx_bind_info_t;

typedef struct adk_sif_cfg_pipeline_ring_tx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_ring_tx_bind_info_t;

typedef struct adk_sif_cfg_pipeline_buff_tx_bind_info_s{
    adk_sif_cfg_pipeline_info_t         *info;
}adk_sif_cfg_pipeline_buff_tx_bind_info_t;

typedef struct adk_lcore_input_receive_types_s {
    bool port;
    bool ring;
    bool buff;
    bool all;
} adk_lcore_input_receive_types_t;

typedef struct adk_lcore_input_receive_s {
    adk_lcore_input_receive_types_t types;
    adk_sif_cfg_pipeline_port_q_t portq_in[ADK_SIF_CFG_MAX_PORT];
    adk_sif_cfg_pipeline_ring_q_t ringq_in[ADK_SIF_CFG_MAX_RING_QUEUES];
    adk_sif_cfg_pipeline_buff_t   buffs_in[ADK_SIF_CFG_MAX_PKT_BUFFS];
    uint32_t num_portq_in;
    uint32_t num_ringq_in;
    uint32_t num_buffs_in;
    bool qat_rx_decrypt;
    bool qat_rx_encrypt;
} adk_lcore_input_receive_t;

extern adk_sif_cfg_pipeline_info_tbl_t  *adkPipelineInfoTbl_g;

/* Function prototypes */
adk_status_t
adk_sif_cfg_pipeline_init(adk_sif_cfg_info_t *cfg_info);

adk_status_t
adk_sif_cfg_pipeline_configure(adk_sif_cfg_info_t *cfg_info_hdl);

adk_status_t
adk_sif_cfg_pipeline_create(adk_sif_cfg_info_t *cfg_info_hdl,
                            adk_cfg_pipeline_params_t *params,
                            adk_sif_cfg_pipeline_info_t **pipeline_info);
adk_status_t
adk_sif_cfg_pipeline_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                          adk_cfg_pipeline_params_t *src_params,
                          adk_cfg_pipeline_params_t *dest_params,
                          adk_sif_cfg_pipeline_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_port_rx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_port_rx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_ring_rx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_ring_rx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_buff_rx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_buff_rx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_qat_rx_bind (
    adk_sif_cfg_info_t *cfg_info_hdl,
    adk_cfg_pipeline_params_t *params,
    adk_sif_cfg_pipeline_ring_rx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_port_tx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_port_tx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_ring_tx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_ring_tx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_buff_tx_bind(adk_sif_cfg_info_t *cfg_info_hdl,
                                  adk_cfg_pipeline_params_t *params,
                                  adk_sif_cfg_pipeline_buff_tx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_qat_tx_bind (
    adk_sif_cfg_info_t *cfg_info_hdl,
    adk_cfg_pipeline_params_t *params,
    adk_sif_cfg_pipeline_ring_tx_bind_info_t *bind_info);
adk_status_t
adk_sif_cfg_pipeline_info_entry_module_set(adk_sif_cfg_info_t *cfg_info_hdl,
                                           adk_cfg_pipeline_params_t *params,
                                           adk_sif_cfg_pipeline_info_t **info);
adk_status_t
adk_sif_cfg_pipeline_info_exit_module_set(adk_sif_cfg_info_t *cfg_info_hdl,
                                          adk_cfg_pipeline_params_t *src_params,
                                          adk_cfg_pipeline_params_t *dest_params,
                                          adk_sif_cfg_pipeline_info_t *dest_info,
                                          adk_sif_cfg_pipeline_info_t **src_info);
adk_status_t
adk_sif_cfg_pipeline_info_tx_exit_module_set(adk_sif_cfg_info_t *cfg_info_hdl, 
                                             adk_cfg_pipeline_params_t *params, 
                                             adk_sif_cfg_pipeline_info_t **info);
adk_status_t
adk_sif_cfg_pipeline_info_alloc(adk_sif_cfg_pipeline_info_tbl_t *pipeline_info_hdl,
                                uint32_t lcore_id,
                                uint32_t entry_mod,
                                adk_sif_cfg_pipeline_info_t **info);
adk_status_t
adk_sif_cfg_pipeline_info_free(adk_sif_cfg_pipeline_info_tbl_t *pipeline_info_hdl,
                               adk_sif_cfg_pipeline_info_t *info);
adk_status_t
adk_sif_cfg_pipeline_info_rewind(adk_sif_cfg_pipeline_info_tbl_t *pipeline_info_hdl);

adk_status_t
adk_sif_cfg_pipeline_info_port_q_set(adk_sif_cfg_info_t *cfg_info_hdl,
                                     adk_sif_cfg_pipeline_port_q_t *params_pq,
                                     uint32_t num_params_pq,
                                     uint16_t entry_func,
                                     void *entry_mod_hdl,
                                     adk_sif_cfg_pipeline_port_q_t *info_pq,
                                     uint32_t *num_info_pq);
adk_status_t
adk_sif_cfg_pipeline_info_ring_q_set(adk_sif_cfg_info_t *cfg_info_hdl,
                                     adk_sif_cfg_pipeline_ring_q_t *params_rq,
                                     uint32_t num_params_rq,
                                     uint16_t entry_func,
                                     void *entry_mod_hdl,
                                     adk_sif_cfg_pipeline_ring_q_t *info_rq,
                                     uint32_t *num_info_rq);
adk_status_t
adk_sif_cfg_pipeline_info_buff_set(adk_sif_cfg_info_t *cfg_info_hdl,
                                     adk_sif_cfg_pipeline_buff_t *params_rq,
                                     uint32_t num_params_rq,
                                     uint16_t entry_func,
                                     adk_sif_cfg_pipeline_buff_t *info_rq,
                                     uint32_t *num_info_rq);
adk_status_t
adk_sif_cfg_pipeline_info_port_idx_get(const uint8_t portid,
                                       adk_sif_cfg_pipeline_port_q_t *p,
                                       uint32_t *n,
                                       uint32_t *idx);
adk_status_t
adk_sif_cfg_pipeline_info_queue_idx_get(const uint16_t queueid,
                                        uint16_t *q,
                                        uint32_t *n,
                                        uint32_t *idx);
adk_status_t
adk_sif_cfg_pipeline_info_ring_idx_get(const uint16_t ringid,
                                       adk_sif_cfg_pipeline_ring_q_t *r,
                                       uint32_t *n,
                                       uint32_t *idx);
adk_status_t
adk_sif_cfg_pipeline_info_buff_idx_get(const uint16_t buffid,
                                       adk_sif_cfg_pipeline_buff_t *r,
                                       uint32_t *n,
                                       uint32_t *idx);
adk_status_t
adk_sif_cfg_pipeline_info_get(adk_sif_cfg_pipeline_info_tbl_t *pipeline_info_hdl,
                              uint32_t lcore_id,
                              uint32_t entry_mod,
                              adk_sif_cfg_pipeline_info_t **info);
adk_status_t
adk_sif_cfg_pipeline_info_dest_get(adk_sif_cfg_pipeline_dest_module_t  *dest_module);

adk_status_t
adk_sif_cfg_pipeline_lcore_portq_ringq_in_get(int32_t coreIndex, adk_sif_cfg_pipeline_port_q_t *portq_in, uint32_t *num_portq_in,
                                              adk_sif_cfg_pipeline_ring_q_t *ringq_in, uint32_t *num_ringq_in,
                                              bool *qat_rx_decrypt, bool *qat_rx_encrypt);
adk_status_t
adk_sif_cfg_pipeline_lcore_entry_dump(uint32_t coreIndex, adk_lcore_input_receive_t *lcore_receive);

adk_status_t
adk_sif_cfg_pipeline_lcore_entry_in_get(uint32_t coreIndex, adk_lcore_input_receive_t *lcore_receive);

adk_status_t
adk_sif_cfg_pipeline_info_dump(adk_sif_cfg_pipeline_info_tbl_t *pipeline_info_hdl);

adk_status_t
adk_sif_cfg_pipeline_module_name_get(adk_sif_cfg_pipeline_modules_t mod, char *name);

adk_status_t
adk_sif_cfg_pipeline_stage_name_get(adk_sif_cfg_pipeline_stage_t stage, char *name);

adk_status_t
adk_sif_cfg_pipeline_entry_func_name_get(uint16_t entry_func, char *name);

#endif
