/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_SIF_CFG_PIPELINE_COMMON_H__
#define __ADK_SIF_CFG_PIPELINE_COMMON_H__

#include "adk_sif_cfg_common.h"


typedef enum adk_sif_cfg_pipeline_stage_e{
    ADK_SIF_CFG_PIPELINE_STAGE_MIN = 0,
    ADK_SIF_CFG_PIPELINE_STAGE_INITIATOR,
    ADK_SIF_CFG_PIPELINE_STAGE_INTERMEDIATE,
    ADK_SIF_CFG_PIPELINE_STAGE_TERMINATOR,
    ADK_SIF_CFG_PIPELINE_STAGE_INITIATOR_TERMINATOR,
    ADK_SIF_CFG_PIPELINE_STAGE_MAX
}adk_sif_cfg_pipeline_stage_t;

typedef enum adk_sif_cfg_pipeline_modules_e{
    ADK_SIF_CFG_PIPELINE_MODULE_MIN = 0,
    ADK_SIF_CFG_PIPELINE_MODULE_PORT_RX,
    ADK_SIF_CFG_PIPELINE_MODULE_PORT_TX,
    ADK_SIF_CFG_PIPELINE_MODULE_RING_RX,
    ADK_SIF_CFG_PIPELINE_MODULE_RING_TX,
    ADK_SIF_CFG_PIPELINE_MODULE_PKTIO_RX,
    ADK_SIF_CFG_PIPELINE_MODULE_PKTIO_TX,
    ADK_SIF_CFG_PIPELINE_MODULE_QAT_RX,
    ADK_SIF_CFG_PIPELINE_MODULE_QAT_TX,
    ADK_SIF_CFG_PIPELINE_MODULE_BUFF_RX,
    ADK_SIF_CFG_PIPELINE_MODULE_BUFF_TX,
    ADK_SIF_CFG_PIPELINE_MODULE_ETHERNET_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_ETHERNET_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPV4_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPV4_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPV6_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPV6_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI_TO_CRYPTO,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_LLI_FROM_CRYPTO,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_L4_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_L4_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI_TO_CRYPTO,
    ADK_SIF_CFG_PIPELINE_MODULE_IPSEC_ULI_FROM_CRYPTO,
    ADK_SIF_CFG_PIPELINE_MODULE_GTPU_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_GTPU_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_PH_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_PH_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_BFD_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_BFD_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_TWAMP_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_ETH_OAM_LLI,
    ADK_SIF_CFG_PIPELINE_MODULE_ETH_OAM_ULI,
    ADK_SIF_CFG_PIPELINE_MODULE_IGNORE,
    ADK_SIF_CFG_PIPELINE_MODULE_MAX
}adk_sif_cfg_pipeline_modules_t;

typedef enum adk_cfg_entry_func_e{
    __FN_ETH_DP_PROCESS_PKT = 0,
#if 0
    __FN_PKTIO_RX_DP_PROCESS_PKT,
#endif
    ADK_SIF_CFG_PIPELINE_ENTRY_FUNC_MAX
}adk_sif_cfg_pipeline_entry_func_t;

typedef enum adk_sif_cfg_pipeline_module_idx_s{
    _RX_IDX = 0,
    _ENTRY_MOD_IDX,
    _EXIT_MOD_IDX,
    _TX_IDX
}adk_sif_cfg_pipeline_module_idx_t;

typedef enum adk_sif_cfg_pipeline_cb_func_s{
    ADK_SIF_CFG_PIPELINE_CB_FUNC_DEFAULT = 0,
    ADK_SIF_CFG_PIPELINE_CB_FUNC_RING    = 1
}adk_sif_cfg_pipeline_cb_func_t;

typedef enum adk_sif_cfg_pipeline_port_type_e{
    ADK_SIF_CFG_PIPELINE_PORT_TYPE_MIN = 0,
    ADK_SIF_CFG_PIPELINE_PORT_TYPE_DPDK_PORT,
    ADK_SIF_CFG_PIPELINE_PORT_TYPE_DPDK_RING,
    ADK_SIF_CFG_PIPELINE_PORT_TYPE_DPDK_BUFF,
    ADK_SIF_CFG_PIPELINE_PORT_TYPE_MAX
}adk_sif_cfg_pipeline_port_type_t;

typedef adk_status_t (*adk_sif_cfg_pipeline_entry_func_f)(void **task_p, uint8_t task_count,
                                           uint32_t task_mask, int coreIndex);

typedef struct adk_sif_cfg_pipeline_port_q_s{
    uint32_t                            type;
    uint8_t                             flags;
    uint8_t                             portid;
    uint16_t                            queueid[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t                            weight[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint32_t                            burst[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t                            entry_func[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    void                                *entry_mod_hdl[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint32_t                            num_queues;
}adk_sif_cfg_pipeline_port_q_t;

typedef struct adk_sif_cfg_pipeline_ring_q_s{
    uint32_t                            type;
    uint8_t                             flags;
    uint16_t                            ringid;
    struct rte_ring                     *ringhdl;
    uint16_t                            entry_func;
    void                                *entry_mod_hdl;
}adk_sif_cfg_pipeline_ring_q_t;

typedef struct adk_sif_cfg_pipeline_buff_s{
    uint16_t                            buff_id;
    uint16_t                            entry_func;
    uint16_t                            transmit;
    uint16_t                            weight;
}adk_sif_cfg_pipeline_buff_t;

extern const uint16_t adkEntryFunctions_g[];

#endif
