/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* CFG_VALUE_LEN defined by DPDK is not large enough to hold the full driver path */
#define ADK_CFG_VALUE_LEN  (2 * CFG_VALUE_LEN)

/* Table to store EAL configuration information */
adk_sif_cfg_eal_info_tbl_t adkEalInfoTbl_g;


adk_status_t
adk_sif_cfg_eal_init(adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_info->eal_info_tbl = (void *)&adkEalInfoTbl_g;

    return ret;
}

adk_status_t
adk_sif_cfg_eal_configure(adk_sif_cfg_info_t *cfg_info_hdl)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_params_t *cfg_params_tbl = NULL;
    adk_cfg_eal_params_tbl_t *eal_params_tbl = NULL;
    adk_sif_cfg_eal_info_t *eal_info = NULL;

    cfg_params_tbl = (adk_cfg_params_t *)cfg_info_hdl->cfg_params_tbl;
    eal_params_tbl = (adk_cfg_eal_params_tbl_t *)cfg_params_tbl->eal_params_tbl;

    ret = adk_sif_cfg_eal_create(cfg_info_hdl, &eal_params_tbl->eal_params[0], &eal_info);
    if (ret != ADK_SUCCESS) 
    {
        return ret;
    }
    if (eal_info == NULL)
    {
        ADK_PREINIT_WRITE("EAL configuration failed\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
    }

    ADK_PREINIT_WRITE("EAL configured\n");
    return ret;
}

adk_status_t
adk_sif_cfg_eal_create(adk_sif_cfg_info_t *cfg_info_hdl,
                       adk_cfg_eal_params_t *params,
                       adk_sif_cfg_eal_info_t **eal_info)
{    
    adk_status_t ret = ADK_SUCCESS;
    int rte_ret = 0;
    adk_sif_cfg_eal_info_t *info = NULL;
    uint32_t i = 0, j = 0;
    int32_t eal_argc = ADK_SIF_CFG_EAL_NUM_ARGS;
    char **eal_argv = NULL;
    uint32_t master_lcoreid = sched_getcpu();

    /* Allocate memory to store configuration info */
    ret = adk_sif_cfg_eal_info_alloc(cfg_info_hdl->eal_info_tbl, &info);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }

    /* Initialize EAL */
    eal_argv = calloc(ADK_SIF_CFG_EAL_NUM_ARGS, sizeof(char *));
    if (eal_argv == NULL)
    {
        ADK_PREINIT_WRITE("Memory allocation failure\n");
        adk_sif_cfg_eal_info_free(cfg_info_hdl->eal_info_tbl, info);
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);
    }
    for (i = 0; i < ADK_SIF_CFG_EAL_NUM_ARGS; i++)
    {
        eal_argv[i] = calloc(ADK_CFG_VALUE_LEN, sizeof(char));
        if (eal_argv[i] == NULL)
        {
            ADK_PREINIT_WRITE("Memory allocation failure\n");
            for (j = 0; j < i; j++)
            {
                free(eal_argv[j]);
            }
            free(eal_argv);
            adk_sif_cfg_eal_info_free(cfg_info_hdl->eal_info_tbl, info);
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);
        }
    }

    strncpy(eal_argv[0], __func__, ADK_CFG_VALUE_LEN-1);
    eal_argv[0] [ADK_CFG_VALUE_LEN-1] = '\0';
    snprintf(eal_argv[1], ADK_CFG_VALUE_LEN, "-l");
    snprintf(eal_argv[2], ADK_CFG_VALUE_LEN, "%d", master_lcoreid);
    for (i = 0; i < params->num_worker_lcores; i++)
    {
        snprintf(eal_argv[2]+ADK_STRLEN(eal_argv[2]), 64-ADK_STRLEN(eal_argv[2]), ",%d", params->worker_lcores[i]);
    }
    snprintf(eal_argv[3], ADK_CFG_VALUE_LEN, "--master-lcore");
    snprintf(eal_argv[4], ADK_CFG_VALUE_LEN, "%d", master_lcoreid);
    snprintf(eal_argv[5], ADK_CFG_VALUE_LEN, "-n");
    snprintf(eal_argv[6], ADK_CFG_VALUE_LEN, "%d", params->mem_channels);
    snprintf(eal_argv[7], ADK_CFG_VALUE_LEN, "--proc-type");
    switch(params->process_type)
    {
        case ADK_SIF_CFG_EAL_PROCESS_TYPE_PRIMARY:
            snprintf(eal_argv[8], ADK_CFG_VALUE_LEN, "primary");
            break;
        
        case ADK_SIF_CFG_EAL_PROCESS_TYPE_SECONDARY:
            snprintf(eal_argv[8], ADK_CFG_VALUE_LEN, "secondary");
            break;
        
        default:
            ADK_PREINIT_WRITE("Invalid process type\n");
            ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
            adk_sif_cfg_eal_info_free(cfg_info_hdl->eal_info_tbl, info);
            goto adk_sif_cfg_eal_create_return;
    }

    params->master_lcore = master_lcoreid;
    
    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if (rte_ret < 0)
    {
        ADK_PREINIT_WRITE ("EAL creation failed\n");
        ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_HW_PROGRAMMING_ERROR, ADK_SEV_LEVEL_ERROR);
        adk_sif_cfg_eal_info_free(cfg_info_hdl->eal_info_tbl, info);
        goto adk_sif_cfg_eal_create_return;
    }

    /* Update eal configuration info */
    ret = adk_sif_cfg_eal_info_set(cfg_info_hdl->eal_info_tbl, params, &info);
    if (ret != ADK_SUCCESS)
    {
        adk_sif_cfg_eal_info_free(cfg_info_hdl->eal_info_tbl, info);
        goto adk_sif_cfg_eal_create_return;
    }
    *eal_info = info;

adk_sif_cfg_eal_create_return:
#if 1
    for (i = 0; i < ADK_SIF_CFG_EAL_NUM_ARGS; i++)
    {
        free(eal_argv[i]);
    }
    free(eal_argv);
#endif    
    return ret;
}

adk_status_t
adk_sif_cfg_eal_info_alloc(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                           adk_sif_cfg_eal_info_t **info)
{
    uint32_t i = 0;
    adk_sif_cfg_eal_info_t *p = eal_info_hdl->eal_info;

    for (i = 0; i < ADK_SIF_CFG_MAX_EAL; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *info = &p[i];
            eal_info_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    ADK_PREINIT_WRITE("EAL is already configured\n");
    return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
    
}

adk_status_t
adk_sif_cfg_eal_info_free(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                          adk_sif_cfg_eal_info_t *info)
{
    adk_status_t ret = ADK_SUCCESS;

    if (info == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    info->valid = _INIT;
    eal_info_hdl->size--;
    
    return ret;
}

adk_status_t
adk_sif_cfg_eal_info_rewind(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    eal_info_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_sif_cfg_eal_info_set(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl __attribute__ ((unused)),
                         adk_cfg_eal_params_t *params,
                         adk_sif_cfg_eal_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
 
    for (i = 0; i < params->num_worker_lcores; i++)
    {
        (*info)->worker_lcores[i] = params->worker_lcores[i];
        (*info)->worker_lcore_scoket[i] = params->worker_lcore_scoket[i];
    }
    (*info)->num_worker_lcores = params->num_worker_lcores;
    (*info)->master_lcore = params->master_lcore;
    (*info)->mem_channels = params->mem_channels;
    (*info)->process_type = params->process_type;
    
    return ret;
}

adk_status_t
adk_sif_cfg_eal_info_get(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                         adk_sif_cfg_eal_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_eal_info_t *eal_info = eal_info_hdl->eal_info;
   
    /* Fetch config info */
    *info = &eal_info[0];

    return ret;
}

adk_status_t
adk_sif_cfg_eal_lcore_check(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                            uint32_t lcore_id)
{
    adk_sif_cfg_eal_info_t *eal_info = eal_info_hdl->eal_info;
    uint32_t i = 0;

    for (i = 0; i < eal_info->num_worker_lcores; i++)
    {
        if ((eal_info->valid == _CONFIG) && (eal_info->worker_lcores[i] == lcore_id))
        {
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

adk_status_t
adk_sif_cfg_eal_worker_socket_check(adk_sif_cfg_eal_info_tbl_t *eal_info_hdl,
                                    int32_t socket_id)
{
    adk_sif_cfg_eal_info_t *eal_info = eal_info_hdl->eal_info;
    uint32_t i = 0;

    for (i = 0; i < eal_info->num_worker_lcores; i++)
    {
        if ((eal_info->valid == _CONFIG) && (eal_info->worker_lcore_scoket[i] == socket_id))
        {
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}





