/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* Table to store mempool configuration information */
adk_sif_cfg_mempool_info_tbl_t adkMempoolInfoTbl_g;


adk_status_t
adk_sif_cfg_mempool_init(adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_info->mempool_info_tbl = (void *)&adkMempoolInfoTbl_g;

    return ret;
}

adk_status_t
adk_sif_cfg_mempool_configure(adk_sif_cfg_info_t *cfg_info_hdl)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_params_t *cfg_params_tbl = NULL;
    adk_cfg_mempool_params_tbl_t *mempool_params_tbl = NULL;
    adk_sif_cfg_mempool_info_t *mempool_info = NULL;
    uint32_t i = 0;

    cfg_params_tbl = (adk_cfg_params_t *)cfg_info_hdl->cfg_params_tbl;
    mempool_params_tbl = (adk_cfg_mempool_params_tbl_t *)cfg_params_tbl->mempool_params_tbl;

    for (i = 0; i < mempool_params_tbl->size; i++)
    {
        /* To sanity check */
        mempool_info = NULL;

        ret = adk_sif_cfg_mempool_create(cfg_info_hdl, &mempool_params_tbl->mempool_params[i], &mempool_info);
        if (ret != ADK_SUCCESS) 
        {
            return ret;
        }
        if (mempool_info == NULL)
        {
            ADK_PREINIT_WRITE("Mempool configuration failed\n");
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
        }
    }

    ADK_PREINIT_WRITE("Mempools configured\n");
    return ret;
}

adk_status_t
adk_sif_cfg_mempool_create(adk_sif_cfg_info_t *cfg_info_hdl,
                           adk_cfg_mempool_params_t *params,
                           adk_sif_cfg_mempool_info_t **mempool_info)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_mempool_info_t *info = NULL;
    void *mem_buf_pool = NULL;
   
    /* Allocate memory to store configuration info */
    ret = adk_sif_cfg_mempool_info_alloc(cfg_info_hdl->mempool_info_tbl, &info);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }

    /* Create pkt Mbuf pool */
    mem_buf_pool = (void *)rte_pktmbuf_pool_create(params->name, params->pool_size,
                                              params->cache_size, 128, params->buffer_size,
                                              params->socket);
    if (mem_buf_pool == NULL)
    {
        ADK_PREINIT_WRITE ("Memory pool creation failed\n");
        ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);
        goto adk_sif_cfg_mempool_create_return;
    }

    /* Update mempool configuration info */
    ret = adk_sif_cfg_mempool_info_set(cfg_info_hdl->mempool_info_tbl, params,
                                       mem_buf_pool, &info);
    if (ret != ADK_SUCCESS)
    {
        goto adk_sif_cfg_mempool_create_return;
    }
    *mempool_info = info;
     
    return ret;

adk_sif_cfg_mempool_create_return:
    adk_sif_cfg_mempool_info_free(cfg_info_hdl->mempool_info_tbl, info);
    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_alloc(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl,
                               adk_sif_cfg_mempool_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
    adk_sif_cfg_mempool_info_t *p = mempool_info_hdl->mempool_info;

    for (i = 0; i < ADK_SIF_CFG_MAX_MEMPOOLS; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *info = &p[i];
            mempool_info_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    if (i == ADK_SIF_CFG_MAX_MEMPOOLS)
    {
        ADK_PREINIT_WRITE("Mempool is already configured\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
    }
    
    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_free(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl,
                              adk_sif_cfg_mempool_info_t *info)
{
    adk_status_t ret = ADK_SUCCESS;

    if (info == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    info->valid = _INIT;
    mempool_info_hdl->size--;
    
    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_rewind(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    mempool_info_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_set(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl __attribute__ ((unused)),
                             adk_cfg_mempool_params_t *params,
                             void *mem_buf_pool,
                             adk_sif_cfg_mempool_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
   
    (*info)->pool_id = params->pool_id; 
    (*info)->type = params->type;
    (*info)->buffer_size = params->buffer_size;
    (*info)->pool_size = params->pool_size;
    (*info)->cache_size = params->cache_size;
    (*info)->socket = params->socket;
    (*info)->mem_buf_pool = mem_buf_pool;

    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_get_next(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl,
                                  adk_sif_cfg_mempool_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_mempool_info_t *mempool_info = mempool_info_hdl->mempool_info;
    uint32_t i = 0;
 
    /* Boundary check */
    if (mempool_info_hdl->cur == mempool_info_hdl->size)
    {
        mempool_info_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }

    /* Skip deleted entries */ 
    for (i = mempool_info_hdl->cur; i < mempool_info_hdl->size; i++)
    {
        if (mempool_info[mempool_info_hdl->cur].valid == _CONFIG)
        {
            break;
        }
    }
    mempool_info_hdl->cur = i;

    /* Fetch config info */
    *info = &mempool_info[mempool_info_hdl->cur];

    /* Point to next entry */
    mempool_info_hdl->cur++;

    return ret;
}

adk_status_t
adk_sif_cfg_mempool_info_get(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl,
                             uint16_t mempool_id,
                             adk_sif_cfg_mempool_info_t **info)
{
    adk_sif_cfg_mempool_info_t *mempool_info = mempool_info_hdl->mempool_info;
    uint32_t i = 0;
    
    for (i = 0; i < mempool_info_hdl->size; i++)
    {
        if ((mempool_info[i].valid == _CONFIG) && (mempool_info[i].pool_id == mempool_id))
        {
            *info = &mempool_info[i]; 
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

adk_status_t
adk_sif_cfg_mempool_check(adk_sif_cfg_mempool_info_tbl_t *mempool_info_hdl,
                          uint16_t mempool_id)
{
    adk_sif_cfg_mempool_info_t *mempool_info = mempool_info_hdl->mempool_info;
    uint32_t i = 0;

    for (i = 0; i < mempool_info_hdl->size; i++)
    {
        if ((mempool_info[i].valid == _CONFIG) && (mempool_info[i].pool_id == mempool_id))
        {
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

