/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* Table to store Pkt Buff configuration information */
adk_sif_cfg_pkt_buffs_info_tbl_t adkPktBuffsInfoTbl_g;

adk_status_t
adk_sif_cfg_pkt_buffs_init(
        adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_info->pkt_buffs_info_tbl = (void *)&adkPktBuffsInfoTbl_g;

    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_configure(
        adk_sif_cfg_info_t *cfg_info_hdl)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_params_t *cfg_params_tbl = NULL;
    adk_cfg_pkt_buffs_params_tbl_t *pkt_buffs_params_tbl = NULL;
    adk_sif_cfg_pkt_buffs_info_t *pkt_buffs_info = NULL;
    uint32_t i = 0;

    cfg_params_tbl = (adk_cfg_params_t *)cfg_info_hdl->cfg_params_tbl;
    pkt_buffs_params_tbl = (adk_cfg_pkt_buffs_params_tbl_t *)
                                cfg_params_tbl->pkt_buffs_params_tbl;

    for (i = 0; i < pkt_buffs_params_tbl->size; i++)
    {
        /* To sanity check */
        pkt_buffs_info = NULL;

        ret = adk_sif_cfg_pkt_buffs_create(
                cfg_info_hdl, &pkt_buffs_params_tbl->pkt_buffs_params[i], &pkt_buffs_info);
        if (ret != ADK_SUCCESS) 
        {
            return ret;
        }
        if (pkt_buffs_info == NULL)
        {
            ADK_PREINIT_WRITE("Packet Buffers %d configuration failed\n", i);
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
        }
    }

    ADK_PREINIT_WRITE("Packet Buffers configured\n");
    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_create(
        adk_sif_cfg_info_t *cfg_info_hdl,
        adk_cfg_pkt_buffs_params_t *params,
        adk_sif_cfg_pkt_buffs_info_t **pkt_buffs_info)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_pkt_buffs_info_t *info = NULL;
   
    /* Allocate memory to store configuration info */
    ret = adk_sif_cfg_pkt_buffs_info_alloc(cfg_info_hdl->pkt_buffs_info_tbl, &info);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }

    /* Update pkt_buffs configuration info */
    ret = adk_sif_cfg_pkt_buffs_info_set(
            cfg_info_hdl->pkt_buffs_info_tbl, params, &info);
    if (ret != ADK_SUCCESS)
    {
        goto adk_sif_cfg_pkt_buffs_create_return;
    }

    *pkt_buffs_info = info;
     
    return ret;

adk_sif_cfg_pkt_buffs_create_return:
    adk_sif_cfg_pkt_buffs_info_free(cfg_info_hdl->pkt_buffs_info_tbl, info);
    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_alloc(
        adk_sif_cfg_pkt_buffs_info_tbl_t *pkt_buffs_info_hdl,
        adk_sif_cfg_pkt_buffs_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
    adk_sif_cfg_pkt_buffs_info_t *p = pkt_buffs_info_hdl->pkt_buffs_info;

    for (i = 0; i < ADK_SIF_CFG_MAX_PKT_BUFFS; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *info = &p[i];
            pkt_buffs_info_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    if (i == ADK_SIF_CFG_MAX_PKT_BUFFS)
    {
        ADK_PREINIT_WRITE("Buff is already configured\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
    }
    
    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_free(
        adk_sif_cfg_pkt_buffs_info_tbl_t *pkt_buffs_info_hdl,
        adk_sif_cfg_pkt_buffs_info_t *info)
{
    adk_status_t ret = ADK_SUCCESS;

    if (info == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    info->valid = _INIT;
    pkt_buffs_info_hdl->size--;
    
    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_rewind(
        adk_sif_cfg_pkt_buffs_info_tbl_t *pkt_buffs_info_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    pkt_buffs_info_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_set(
        adk_sif_cfg_pkt_buffs_info_tbl_t  __attribute__ ((unused))*pkt_buffs_info_hdl,
        adk_cfg_pkt_buffs_params_t *params,
        adk_sif_cfg_pkt_buffs_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
   
    (*info)->pkt_buffs_id = params->pkt_buffsid; 
    (*info)->pkt_buffs_mempool = params->rx_mempool; 
    (*info)->pkt_buffs_transmit = params->transmit; 
    (*info)->pkt_buffs_weight = params->weight; 

    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_get_next(
        adk_sif_cfg_pkt_buffs_info_tbl_t *pkt_buffs_info_hdl,
        adk_sif_cfg_pkt_buffs_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_pkt_buffs_info_t *pkt_buffs_info = pkt_buffs_info_hdl->pkt_buffs_info;
    uint32_t i = 0;
    
    /* Boundary check */
    if (pkt_buffs_info_hdl->cur == pkt_buffs_info_hdl->size)
    {
        pkt_buffs_info_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }
  
    /* Skip deleted entries */ 
    for (i = pkt_buffs_info_hdl->cur; i < pkt_buffs_info_hdl->size; i++)
    {
        if (pkt_buffs_info[pkt_buffs_info_hdl->cur].valid == _CONFIG)
        {
            break;
        }
    }
    pkt_buffs_info_hdl->cur = i;

    /* Fetch config info */
    *info = &pkt_buffs_info[pkt_buffs_info_hdl->cur];

    /* Point to next entry */
    pkt_buffs_info_hdl->cur++;

    return ret;
}

adk_status_t
adk_sif_cfg_pkt_buffs_info_get(
        adk_sif_cfg_pkt_buffs_info_tbl_t *pkt_buffs_info_hdl,
        uint8_t pkt_buffs_id,
        adk_sif_cfg_pkt_buffs_info_t **buff_info)
{
    adk_sif_cfg_pkt_buffs_info_t *pkt_buffs_info = pkt_buffs_info_hdl->pkt_buffs_info;
    uint32_t i = 0;

    for (i = 0; i < pkt_buffs_info_hdl->size; i++)
    {
        if ((pkt_buffs_info[i].valid == _CONFIG) && 
                (pkt_buffs_info[i].pkt_buffs_id == pkt_buffs_id))
        {
            *buff_info = &pkt_buffs_info[i];
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

