/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* Non Symmetric Key*/
/* X710 NICs default RSS Hash Key  */
static uint32_t rss_default_hashkey[] = {0x6b793944,
    0x23504cb5, 0x5bea75b6, 0x309f4f12, 0x3dc0a2b8,
    0x024ddcdf, 0x339b8ca0, 0x4c4af64a, 0x34fac605,
    0x55d85839, 0x3a58997d, 0x2ec938e1, 0x66031581
};
/* Table to store port configuration information */
adk_sif_cfg_port_info_tbl_t adkPortInfoTbl_g;

struct rte_eth_conf adkPortConfDefaults_g = {
	.rxmode = {
        .mq_mode = ETH_MQ_RX_NONE,
		.split_hdr_size = 0,
	},
	.txmode = {
		.mq_mode = ETH_MQ_TX_NONE,
	},
};

adk_status_t
adk_sif_cfg_port_init(adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_info->port_info_tbl = (void *)&adkPortInfoTbl_g;

    return ret;
}

adk_status_t
adk_sif_cfg_port_configure(adk_sif_cfg_info_t *cfg_info_hdl)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_params_t *cfg_params_tbl = NULL;
    adk_cfg_port_params_tbl_t *port_params_tbl = NULL;
    adk_sif_cfg_port_info_t *port_info = NULL;
    uint32_t i = 0;

    cfg_params_tbl = (adk_cfg_params_t *)cfg_info_hdl->cfg_params_tbl;
    port_params_tbl = (adk_cfg_port_params_tbl_t *)cfg_params_tbl->port_params_tbl;

    for (i = 0; i < port_params_tbl->size; i++)
    {
        /* To sanity check */
        port_info = NULL;

        ret = adk_sif_cfg_port_create(cfg_info_hdl, &port_params_tbl->port_params[i], &port_info);
        if (ret != ADK_SUCCESS) 
        {
            return ret;
        }
        if (port_info == NULL)
        {
            ADK_PREINIT_WRITE("Port configuration failed\n");
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
        }
    }

    ADK_PREINIT_WRITE("Ports configured\n");
    return ret;
}

adk_status_t
adk_sif_cfg_port_create(adk_sif_cfg_info_t *cfg_info_hdl,
                        adk_cfg_port_params_t *params,
                        adk_sif_cfg_port_info_t **port_info)
{    
    adk_status_t ret = ADK_SUCCESS;
    int rte_ret = 0;
    adk_sif_cfg_port_info_t *info = NULL;
    adk_sif_cfg_mempool_info_t *mempool_info = NULL;
    struct rte_eth_conf port_conf;
    uint32_t i = 0;
    struct rte_eth_txconf *txconf=NULL;
   
    /* Allocate memory to store configuration info */
    ret = adk_sif_cfg_port_info_alloc(cfg_info_hdl->port_info_tbl, &info);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }

    /* Create device port */
    port_conf = adkPortConfDefaults_g;
    port_conf.link_speeds = params->speed;
   
    switch(params->rx_mq_mode)
    {
        case ADK_CFG_PORT_RX_MQ_MODE_RSS:
            port_conf.rxmode.mq_mode = ETH_MQ_RX_RSS;
            port_conf.rx_adv_conf.rss_conf.rss_key = (uint8_t *) rss_default_hashkey; 
            port_conf.rx_adv_conf.rss_conf.rss_hf = ETH_RSS_IP; 
            break;

        default:
            ;
    }
    
    rte_ret = rte_eth_dev_configure(params->portid, params->num_rx_queues,
                                    params->num_tx_queues,
                                    &port_conf);
    if (rte_ret != 0)
    {
        ADK_PREINIT_WRITE ("Port %d creation failed, rte_ret=%d\n", params->portid, rte_ret);
        ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
        goto adk_sif_cfg_port_create_return;
    }

    /* Create device Rx queues */
    for (i = 0; i < params->num_rx_queues; i++)
    {
        ret = adk_sif_cfg_mempool_info_get(cfg_info_hdl->mempool_info_tbl,
                                           params->rx_queue_mempool[i],
                                           &mempool_info);
        if (ret != ADK_SUCCESS)
        {
            ADK_PREINIT_WRITE("Port %d -> Rx Queue %d creation failed - Invalid mempool %d\n",
                              params->portid, params->rx_queues[i], params->rx_queue_mempool[i]);
            ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
            goto adk_sif_cfg_port_create_return;
        }
         
        rte_ret = rte_eth_rx_queue_setup(params->portid, params->rx_queues[i], params->rx_queue_size[i],
                                         mempool_info->socket, NULL, 
                                         (struct rte_mempool * )mempool_info->mem_buf_pool);
        if (rte_ret < 0)
        {
            ADK_PREINIT_WRITE("Port %d -> Rx Queue %d creation failed - Invalid Rx queue setup parameters\n",
                              params->portid, params->rx_queues[i]);
            ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_HW_PROGRAMMING_ERROR, ADK_SEV_LEVEL_ERROR);
            goto adk_sif_cfg_port_create_return;
        }

    }

#ifdef ADK_TX_CKSUM_OL
    struct rte_eth_dev_info dev_info;
    rte_eth_dev_info_get(params->portid, &dev_info);
    txconf = &dev_info.default_txconf;
    /*Reset the the txq flags to enable all offloads*/
#endif

    /* Create device Tx queues */
    for (i = 0; i < params->num_tx_queues; i++)
    {
        ret = adk_sif_cfg_eal_worker_socket_check(cfg_info_hdl->eal_info_tbl, params->tx_queue_socket[i]);
        if (ret != ADK_SUCCESS)
        {
            ADK_PREINIT_WRITE("Port %d -> Tx Queue %d creation failed - Invalid socket %d\n",
                              params->portid, params->tx_queues[i], params->tx_queue_socket[i]);
            ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
            goto adk_sif_cfg_port_create_return;
        }
         
        rte_ret = rte_eth_tx_queue_setup(params->portid, params->tx_queues[i], params->tx_queue_size[i],
                                         params->tx_queue_socket[i], txconf);
        if (rte_ret < 0)
        {
            ADK_PREINIT_WRITE("Port %d -> Tx Queue %d creation failed - Invalid Tx queue setup parameters\n",
                              params->portid, params->tx_queues[i]);
            ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_HW_PROGRAMMING_ERROR, ADK_SEV_LEVEL_ERROR);
            goto adk_sif_cfg_port_create_return;
        }

    }

    /* Start device */
    rte_ret  = rte_eth_dev_start(params->portid);
    if (rte_ret < 0)
    {
        ADK_PREINIT_WRITE("Port %d start failed\n", params->portid);
        ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_HW_PROGRAMMING_ERROR, ADK_SEV_LEVEL_ERROR);
        goto adk_sif_cfg_port_create_return;
    }

    /* Configure promiscuous mode */
    if (params->promisc)
    {
	rte_eth_promiscuous_enable(params->portid);   
    }


    /* Update port configuration info */
    ret = adk_sif_cfg_port_info_set(cfg_info_hdl->port_info_tbl, params, &info);
    if (ret != ADK_SUCCESS)
    {
        goto adk_sif_cfg_port_create_return;
    }
    *port_info = info;
     
    return ret;

adk_sif_cfg_port_create_return:
    adk_sif_cfg_port_info_free(cfg_info_hdl->port_info_tbl, info);
    return ret;
}

adk_status_t
adk_sif_cfg_port_info_alloc(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                            adk_sif_cfg_port_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
    adk_sif_cfg_port_info_t *p = port_info_hdl->port_info;

    for (i = 0; i < ADK_SIF_CFG_MAX_PORT; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *info = &p[i];
            port_info_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    if (i == ADK_SIF_CFG_MAX_PORT)
    {
        ADK_PREINIT_WRITE("port is already configured\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
    }
    
    return ret;
}

adk_status_t
adk_sif_cfg_port_info_free(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                           adk_sif_cfg_port_info_t *info)
{
    adk_status_t ret = ADK_SUCCESS;

    if (info == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    info->valid = _INIT;
    port_info_hdl->size--;
    
    return ret;
}

adk_status_t
adk_sif_cfg_port_info_rewind(adk_sif_cfg_port_info_tbl_t *port_info_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    port_info_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_sif_cfg_port_info_set(adk_sif_cfg_port_info_tbl_t *port_info_hdl __attribute__ ((unused)),
                          adk_cfg_port_params_t *params,
                          adk_sif_cfg_port_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;

    (*info)->portid = params->portid;
    (*info)->speed = params->speed;
    (*info)->duplex = params->duplex;
    (*info)->promisc = params->promisc;  

    for (i = 0; i < params->num_rx_queues; i++)
    {
        (*info)->rx_queues[i] = params->rx_queues[i];
        (*info)->rx_queue_mempool[i] = params->rx_queue_mempool[i];
        (*info)->rx_queue_size[i] = params->rx_queue_size[i];
        (*info)->rx_queue_weight[i] = params->rx_queue_weight[i];
        (*info)->rx_queue_burst[i] = params->rx_queue_burst[i];
    }
    (*info)->num_rx_queues = params->num_rx_queues;
     
    for (i = 0; i < params->num_tx_queues; i++)
    {
        (*info)->tx_queues[i] = params->tx_queues[i];
        (*info)->tx_queue_socket[i] = params->tx_queue_socket[i];
        (*info)->tx_queue_size[i] = params->tx_queue_size[i];
        (*info)->tx_queue_burst[i] = params->tx_queue_burst[i];
    }
    (*info)->num_tx_queues = params->num_tx_queues;

    return ret;
}

adk_status_t
adk_sif_cfg_port_info_get_next(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                               adk_sif_cfg_port_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_port_info_t *port_info = port_info_hdl->port_info;
    uint32_t i = 0;
    
    /* Boundary check */
    if (port_info_hdl->cur == port_info_hdl->size)
    {
        port_info_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }

    /* Boundary check */
    if (port_info_hdl->cur == port_info_hdl->size)
    {
        port_info_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }

    /* Skip deleted entries */ 
    for (i = port_info_hdl->cur; i < port_info_hdl->size; i++)
    {
        if (port_info[port_info_hdl->cur].valid == _CONFIG)
        {
            break;
        }
    }
    port_info_hdl->cur = i;

    /* Fetch config info */
    *info = &port_info[port_info_hdl->cur];

    /* Point to next entry */
    port_info_hdl->cur++;

    return ret;
}

adk_status_t
adk_sif_cfg_port_check(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                       uint8_t port_id)
{
    adk_sif_cfg_port_info_t *port_info = port_info_hdl->port_info;
    uint32_t p = 0;

    for (p = 0; p < port_info_hdl->size; p++)
    {
        if ((port_info[p].valid == _CONFIG) && (port_info[p].portid == port_id))
        {
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

adk_status_t
adk_sif_cfg_port_rx_queue_check(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                                uint8_t port_id, uint16_t queue_id)
{
    adk_sif_cfg_port_info_t *port_info = port_info_hdl->port_info;
    uint32_t p = 0, q = 0;

    for (p = 0; p < port_info_hdl->size; p++)
    {
        if ((port_info[p].valid == _CONFIG) && (port_info[p].portid == port_id))
        {
            for (q = 0; q < port_info[p].num_rx_queues; q++)
            {
                if (port_info[p].rx_queues[q] == queue_id)
                {
                    return ADK_SUCCESS;
                }
            }
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

adk_status_t
adk_sif_cfg_port_tx_queue_check(adk_sif_cfg_port_info_tbl_t *port_info_hdl,
                                uint8_t port_id, uint16_t queue_id)
{
    adk_sif_cfg_port_info_t *port_info = port_info_hdl->port_info;
    uint32_t p = 0, q = 0;

    for (p = 0; p < port_info_hdl->size; p++)
    {
        if ((port_info[p].valid == _CONFIG) && (port_info[p].portid == port_id))
        {
            for (q = 0; q < port_info[p].num_tx_queues; q++)
            {
                if (port_info[p].tx_queues[q] == queue_id)
                {
                    return ADK_SUCCESS;
                }
            }
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}
