/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* Table to store ring configuration information */
adk_sif_cfg_ring_info_tbl_t adkRingInfoTbl_g;

adk_status_t
adk_sif_cfg_ring_init(adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_info->ring_info_tbl = (void *)&adkRingInfoTbl_g;

    return ret;
}

adk_status_t
adk_sif_cfg_ring_configure(adk_sif_cfg_info_t *cfg_info_hdl)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_params_t *cfg_params_tbl = NULL;
    adk_cfg_ring_params_tbl_t *ring_params_tbl = NULL;
    adk_sif_cfg_ring_info_t *ring_info = NULL;
    uint32_t i = 0;

    cfg_params_tbl = (adk_cfg_params_t *)cfg_info_hdl->cfg_params_tbl;
    ring_params_tbl = (adk_cfg_ring_params_tbl_t *)cfg_params_tbl->ring_params_tbl;

    for (i = 0; i < ring_params_tbl->size; i++)
    {
        /* To sanity check */
        ring_info = NULL;

        ret = adk_sif_cfg_ring_create(cfg_info_hdl, &ring_params_tbl->ring_params[i], &ring_info);
        if (ret != ADK_SUCCESS) 
        {
            return ret;
        }
        if (ring_info == NULL)
        {
            ADK_PREINIT_WRITE("Ring configuration failed\n");
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
        }
    }

    ADK_PREINIT_WRITE("Rings configured\n");
    return ret;
}

adk_status_t
adk_sif_cfg_ring_create(adk_sif_cfg_info_t *cfg_info_hdl,
                        adk_cfg_ring_params_t *params,
                        adk_sif_cfg_ring_info_t **ring_info)
{    
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_ring_info_t *info = NULL;
    struct rte_ring *ring_hdl;
   
    /* Allocate memory to store configuration info */
    ret = adk_sif_cfg_ring_info_alloc(cfg_info_hdl->ring_info_tbl, &info);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }

    /* Create ring */
    ret = adk_sif_cfg_eal_worker_socket_check(cfg_info_hdl->eal_info_tbl, params->socket);
    if (ret != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("Ring %d creation failed - Invalid socket %d\n",
                params->ring_id, params->socket);
        ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
        goto adk_sif_cfg_ring_create_return;
    }

    ring_hdl = rte_ring_create(params->name, params->ring_size, params->socket, 0);
    if (ring_hdl == NULL)
    {
        ADK_PREINIT_WRITE ("Ring %d creation failed\n", params->ring_id);
        ret = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INTERNAL_ERROR, ADK_SEV_LEVEL_ERROR);
        goto adk_sif_cfg_ring_create_return;
    }

    /* Update ring configuration info */
    ret = adk_sif_cfg_ring_info_set(cfg_info_hdl->ring_info_tbl, params, ring_hdl, &info);
    if (ret != ADK_SUCCESS)
    {
        goto adk_sif_cfg_ring_create_return;
    }
    *ring_info = info;
     
    return ret;

adk_sif_cfg_ring_create_return:
    adk_sif_cfg_ring_info_free(cfg_info_hdl->ring_info_tbl, info);
    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_alloc(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl,
                            adk_sif_cfg_ring_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
    adk_sif_cfg_ring_info_t *p = ring_info_hdl->ring_info;

    for (i = 0; i < ADK_SIF_CFG_MAX_RING_QUEUES; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *info = &p[i];
            ring_info_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    if (i == ADK_SIF_CFG_MAX_RING_QUEUES)
    {
        ADK_PREINIT_WRITE("Ring is already configured\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
    }
    
    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_free(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl,
                           adk_sif_cfg_ring_info_t *info)
{
    adk_status_t ret = ADK_SUCCESS;

    if (info == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    info->valid = _INIT;
    ring_info_hdl->size--;
    
    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_rewind(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    ring_info_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_set(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl __attribute__ ((unused)),
                          adk_cfg_ring_params_t *params,
                          void *ring,
                          adk_sif_cfg_ring_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
   
    (*info)->ring_id = params->ring_id; 
    (*info)->ring = ring;
    (*info)->ring_size = params->ring_size;
    (*info)->ring_enqueue_burst = params->ring_enqueue_burst;
    (*info)->ring_dequeue_burst = params->ring_dequeue_burst;
    (*info)->socket = params->socket;

    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_get_next(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl,
                               adk_sif_cfg_ring_info_t **info)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_sif_cfg_ring_info_t *ring_info = ring_info_hdl->ring_info;
    uint32_t i = 0;
    
    /* Boundary check */
    if (ring_info_hdl->cur == ring_info_hdl->size)
    {
        ring_info_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }
  
    /* Skip deleted entries */ 
    for (i = ring_info_hdl->cur; i < ring_info_hdl->size; i++)
    {
        if (ring_info[ring_info_hdl->cur].valid == _CONFIG)
        {
            break;
        }
    }
    ring_info_hdl->cur = i;

    /* Fetch config info */
    *info = &ring_info[ring_info_hdl->cur];

    /* Point to next entry */
    ring_info_hdl->cur++;

    return ret;
}

adk_status_t
adk_sif_cfg_ring_info_get(adk_sif_cfg_ring_info_tbl_t *ring_info_hdl,
                       uint32_t ring_id,
                       void **ring)
{
    adk_sif_cfg_ring_info_t *ring_info = ring_info_hdl->ring_info;
    uint32_t i = 0;

    for (i = 0; i < ring_info_hdl->size; i++)
    {
        if ((ring_info[i].valid == _CONFIG) && (ring_info[i].ring_id == ring_id))
        {
            *ring = ring_info[i].ring;
            return ADK_SUCCESS;
        }
    }

    return ADK_E_LIST_NOT_FOUND;
}

