/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS (Hardware Offload Services) system level
 * APIs for use by ADK modules e.g. initialization, shutdown, switch level
 * attributes, etc. It also has all debug dump APIs for troubleshooting.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

adk_hos_pvt_data_t *adk_hos_pvt_g_p;

const uint64_t adk_hos_mac_mask_g[] =
{
    0,
    0x800000000000, 0xC00000000000, 0xE00000000000, 0xF00000000000,
    0xF80000000000, 0xFC0000000000, 0xFE0000000000, 0xFF0000000000,
    0xFF8000000000, 0xFFC000000000, 0xFFE000000000, 0xFFF000000000,
    0xFFF800000000, 0xFFFC00000000, 0xFFFE00000000, 0xFFFF00000000,
    0xFFFF80000000, 0xFFFFC0000000, 0xFFFFE0000000, 0xFFFFF0000000,
    0xFFFFF8000000, 0xFFFFFC000000, 0xFFFFFE000000, 0xFFFFFF000000,
    0xFFFFFF800000, 0xFFFFFFC00000, 0xFFFFFFE00000, 0xFFFFFFF00000,
    0xFFFFFFF80000, 0xFFFFFFFC0000, 0xFFFFFFFE0000, 0xFFFFFFFF0000,
    0xFFFFFFFF8000, 0xFFFFFFFFC000, 0xFFFFFFFFE000, 0xFFFFFFFFF000,
    0xFFFFFFFFF800, 0xFFFFFFFFFC00, 0xFFFFFFFFFE00, 0xFFFFFFFFFF00,
    0xFFFFFFFFFF80, 0xFFFFFFFFFFC0, 0xFFFFFFFFFFE0, 0xFFFFFFFFFFF0,
    0xFFFFFFFFFFF8, 0xFFFFFFFFFFFC, 0xFFFFFFFFFFFE, 0xFFFFFFFFFFFF,
};

/* Should be called only once for the initial HLP configuration. This can be
 * done from any one process preferably, the one performing the ADK system
 * initialization at the beginning. */

adk_status_t
adk_hos_init (
    adk_sys_handle_t sys_handle)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    enum ies_pol_config_mode pol_cfg_mode;
    struct ies_hash_profile hash_profile;
    struct ies_hash_l2_cfg l2_hash_config;
    bool en_natt = true;
    bool mac_learning;
    int ies_stats_ids[] =
    {
        IES_STAT_TX_BYTE,
        IES_STAT_TX_BAD_BYTE,
        IES_STAT_TX_FRAME,
        IES_STAT_TX_BAD_FRAME,
        IES_STAT_TX_L2_MC_FRAME,
        IES_STAT_TX_L2_BC_FRAME,
        IES_STAT_TX_FCS_ERROR,
        IES_STAT_TX_CTRL,
        IES_STAT_TX_64,
        IES_STAT_TX_65,
        IES_STAT_TX_128,
        IES_STAT_TX_256,
        IES_STAT_TX_512,
        IES_STAT_TX_1024,
        IES_STAT_TX_1519,
        IES_STAT_TX_JABBER,
        IES_STAT_TX_UNDERRUN,

        IES_STAT_RX_BYTE,
        IES_STAT_RX_BAD_BYTE,
        IES_STAT_RX_FRAME,
        IES_STAT_RX_BAD_FRAME,
        IES_STAT_RX_CTRL,
        IES_STAT_RX_PAUSE,
        IES_STAT_RX_FCS_ERROR,
        IES_STAT_RX_CODE_ERROR,
        IES_STAT_RX_UNDERSIZE,
        IES_STAT_RX_64,
        IES_STAT_RX_65,
        IES_STAT_RX_128,
        IES_STAT_RX_256,
        IES_STAT_RX_512,
        IES_STAT_RX_1024,
        IES_STAT_RX_1519,
        IES_STAT_RX_OVERSIZE,
        IES_STAT_RX_JABBER,

        IES_STAT_IPV4_L2_MCAST_FRAME,
        IES_STAT_IPV6_L2_MCAST_FRAME,
        IES_STAT_NONIP_L2_MCAST_FRAME,

        IES_STAT_IPV4_L2_BCAST_FRAME,
        IES_STAT_IPV6_L2_BCAST_FRAME,
        IES_STAT_NONIP_L2_BCAST_FRAME
    };

    /* Create memzone to store all the HOS private data */
    retValue = adk_shm_create(ADK_HOS_SHM_MEMZONE_NAME,
                              sizeof(adk_hos_pvt_data_t),
                              ADK_SHM_FIRST_WORKER_IN_USE_LCORE,
                              (adk_shm_handle_t **) &adk_hos_pvt_g_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(sys_handle, ADK_SYS_MODULE_HANDLE_GET(sys_handle),
                      "Failed to create HOS memzone %s\n",
                      ADK_HOS_SHM_MEMZONE_NAME);
        goto _adk_hos_init_exit;
    }

    adk_hos_pvt_g_p->sys_handle = sys_handle;

    /* Disable MAC learning on host port */
    mac_learning = false;
    ADK_HOS_LOG_EXT_API("ies_port_set_attr", "Disable IES_PA_LEARNING"
                        " on CPK port\n");
    ies_ret = ies_port_set_attr(ADK_HLP_SWITCH_ID, ADK_HOS_CPK_PORT,
                                IES_PA_LEARNING, &mac_learning);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to disable MAC learning on CPK"
                      " port\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    /* Enable the support for NAT-T */
    ADK_HOS_LOG_EXT_API("ies_switch_set_attr", "NAT-T Enable\n");
    ies_ret = ies_switch_set_attr(ADK_HLP_SWITCH_ID, IES_SA_NAT_T_ENABLE,
                                  &en_natt);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to enable NAT-T support"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    /* Configure the policing mode for the switch */
    pol_cfg_mode = IES_POL_DOMAIN_L2_RED;
    ADK_HOS_LOG_EXT_API("ies_switch_set_attr", "Policer Mode DOMAIN_L2_RED\n");
    ies_ret = ies_switch_set_attr(ADK_HLP_SWITCH_ID, IES_SA_POLICER_MODE_CFG,
                                  &pol_cfg_mode);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure policing mode"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    /* Configure the hash profiles */

    ADK_HOS_LOG_EXT_API("ies_res_alloc", "L2 Hash Profile ID\n");
    ies_ret = ies_res_alloc(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                            &adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to allocate L2 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_LOG_INFO(adk_hos_pvt_g_p->sys_handle,
                 ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                 "Allocated L2 hash profile ID %d!\n",
                 adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX]);

    ADK_MEMSET(&hash_profile, 0, sizeof(hash_profile));
    hash_profile.nexthop.otr.smac = true;
    hash_profile.nexthop.otr.dmac = true;
    hash_profile.nexthop.otr.ether_type = true;
    hash_profile.nexthop.otr.vlan1_id = true;
    ADK_HOS_LOG_EXT_API("ies_hash_set_profile", "ID-%d, NextHop Hash,"
                        " keys=smac|dmac|etype|vlan1\n",
                        adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX]);
    ies_ret = ies_hash_set_profile(ADK_HLP_SWITCH_ID,
                                   adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX],
                                   &hash_profile,
                                   IES_HASH_PROF_NEXTHOP_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure L2 hash profile"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_alloc", "IPv4 Hash Profile ID\n");
    ies_ret = ies_res_alloc(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                            &adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to allocate IPv4 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_LOG_INFO(adk_hos_pvt_g_p->sys_handle,
                 ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                 "Allocated IPv4 hash profile ID %d!\n",
                 adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX]);

    ADK_MEMSET(&hash_profile, 0, sizeof(hash_profile));
    hash_profile.nexthop.otr.addr_type = IES_IP_ADDR_IPV4;
    hash_profile.nexthop.otr.ip_prot = true;
    hash_profile.nexthop.otr.sip = true;
    hash_profile.nexthop.otr.sip_mask = 0xFF;
    hash_profile.nexthop.otr.dip = true;
    hash_profile.nexthop.otr.dip_mask = 0xFF;
    ADK_HOS_LOG_EXT_API("ies_hash_set_profile", "ID-%d, NextHop Hash,"
                        " keys=ipv4|ip-proto|sip|dip\n",
                        adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX]);
    ies_ret = ies_hash_set_profile(ADK_HLP_SWITCH_ID,
                                   adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX],
                                   &hash_profile,
                                   IES_HASH_PROF_NEXTHOP_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure IPv4 hash profile"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_alloc", "IPv6 Hash Profile ID\n");
    ies_ret = ies_res_alloc(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                            &adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to allocate IPv6 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_LOG_INFO(adk_hos_pvt_g_p->sys_handle,
                 ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                 "Allocated IPv6 hash profile ID %d!\n",
                 adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX]);

    ADK_MEMSET(&hash_profile, 0, sizeof(hash_profile));
    hash_profile.nexthop.otr.addr_type = IES_IP_ADDR_IPV6;
    hash_profile.nexthop.otr.ip_prot = true;
    hash_profile.nexthop.otr.sip = true;
    hash_profile.nexthop.otr.sip_mask = 0xF;
    hash_profile.nexthop.otr.dip = true;
    hash_profile.nexthop.otr.dip_mask = 0xF;
    ADK_HOS_LOG_EXT_API("ies_hash_set_profile", "ID-%d, NextHop Hash,"
                        " keys=ipv6|ip-proto|sip|dip\n",
                        adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX]);
    ies_ret = ies_hash_set_profile(ADK_HLP_SWITCH_ID,
                                   adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX],
                                   &hash_profile,
                                   IES_HASH_PROF_NEXTHOP_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure IPv6 hash profile"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_alloc", "IPv4 ESP Hash Profile ID\n");
    ies_ret = ies_res_alloc(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                            &adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to allocate IPv4 esp hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_LOG_INFO(adk_hos_pvt_g_p->sys_handle,
                 ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                 "Allocated IPv4 esp hash profile ID %d!\n",
                 adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX]);

    ADK_MEMSET(&hash_profile, 0, sizeof(hash_profile));
    hash_profile.nexthop.otr.addr_type = IES_IP_ADDR_IPV4;
    hash_profile.nexthop.otr.ip_prot = true;
    hash_profile.nexthop.inr.addr_type = IES_IP_ADDR_IPV4;
    hash_profile.nexthop.inr.sip = true;
    hash_profile.nexthop.inr.sip_mask = 0xFF;
    ADK_HOS_LOG_EXT_API("ies_hash_set_profile", "ID-%d, NextHop Hash,"
                        " keys=ipv4|ip-proto|spi\n",
                        adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX]);
    ies_ret = ies_hash_set_profile(ADK_HLP_SWITCH_ID,
                                   adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX],
                                   &hash_profile,
                                   IES_HASH_PROF_NEXTHOP_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure IPv4 esp hash profile"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_alloc", "IPv6 ESP Hash Profile ID\n");
    ies_ret = ies_res_alloc(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                            &adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to allocate IPv6 esp hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_LOG_INFO(adk_hos_pvt_g_p->sys_handle,
                 ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                 "Allocated IPv6 esp hash profile ID %d!\n",
                 adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX]);

    ADK_MEMSET(&hash_profile, 0, sizeof(hash_profile));
    hash_profile.nexthop.otr.addr_type = IES_IP_ADDR_IPV6;
    hash_profile.nexthop.otr.ip_prot = true;
    hash_profile.nexthop.inr.addr_type = IES_IP_ADDR_IPV6;
    hash_profile.nexthop.inr.sip = true;
    hash_profile.nexthop.inr.sip_mask = 0xF;
    ADK_HOS_LOG_EXT_API("ies_hash_set_profile", "ID-%d, NextHop Hash,"
                        " keys=ipv6|ip-proto|spi\n",
                        adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX]);
    ies_ret = ies_hash_set_profile(ADK_HLP_SWITCH_ID,
                                   adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX],
                                   &hash_profile,
                                   IES_HASH_PROF_NEXTHOP_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure IPv6 esp hash profile"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    ADK_MEMSET(&l2_hash_config, 0, sizeof(l2_hash_config));
    l2_hash_config.keys.smac = true;
    l2_hash_config.keys.dmac = true;
    l2_hash_config.keys.ether_type = true;
    l2_hash_config.keys.use_l2_if_ip = true;
    l2_hash_config.keys.use_metadata = true;
    ADK_HOS_LOG_EXT_API("ies_hash_set_l2_config",
                        "keys=smac|dmac|eth-type|use_l2_if_ip|use_metadata\n");
    ies_ret = ies_hash_set_l2_config(ADK_HLP_SWITCH_ID, &l2_hash_config,
                                     IES_HASH_L2_CFG_HASH_KEYS);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure L2 hash for LAG"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    /* Configure the port stats profile */
    ADK_HOS_LOG_EXT_API("ies_stats_create_profile", "for port stats\n");
    ies_stats_create_profile(ADK_HLP_SWITCH_ID,
                             &adk_hos_pvt_g_p->port_stats_profile_id,
                             ies_stats_ids,
                             (sizeof(ies_stats_ids) / sizeof(int)));
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create port stats profile ID"
                      " in HLP\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_init_exit;
    }

    retValue = adk_hos_qos_init();
    if (retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Failed to initialize HOS QoS configurations\n");
        goto _adk_hos_init_exit;
    }

    retValue = adk_hos_policer_init();
    if (retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Failed to initialize HOS policers\n");
        goto _adk_hos_init_exit;
    }

    /* Alloc memory needed for clean slate mchanism */
    adk_hos_pvt_g_p->rule_ids = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(adk_hos_pvt_g_p->sys_handle),
                        1, ADK_HOS_MAX_TABLE_SIZE, sizeof(hos_rule_data_t)); 
    if (NULL == adk_hos_pvt_g_p->rule_ids)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Failed to allocate memory required for HOS clean slate mechanism\n");
        goto _adk_hos_init_exit;
    }

    retValue = adk_hos_init_tables();
    if (retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Failed to setup all HOS tables\n");
        goto _adk_hos_init_exit;
    }

    /* Initialize to zero the mac mapper */
    memset(adk_hos_pvt_g_p->mac_mapper, 0, sizeof(adk_hos_pvt_g_p->mac_mapper));

_adk_hos_init_exit:

    return retValue;
}

/* This API should be called from one process to remove all the default
 * configurations done at the HOS initialization time. Preferably, from
 * the process performing ADK sys-shutdown. */

adk_status_t
adk_hos_shutdown (
    void)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    ADK_NVM_FREE(adk_hos_pvt_g_p->rule_ids);
    adk_hos_pvt_g_p->rule_ids = NULL;

    retValue = adk_hos_shutdown_tables();
    if (retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Failed to remove all ADK HOS tables\n");
        goto _adk_hos_shutdown_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_free", "L2 Hash Profile ID\n");
    ies_ret = ies_res_free(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                           adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to free L2 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_free", "IPv4 Hash Profile ID\n");
    ies_ret = ies_res_free(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                           adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to free IPv4 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_free", "IPv6 Hash Profile ID\n");
    ies_ret = ies_res_free(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                           adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to free IPv6 hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_free", "IPv4 Hash ESP Profile ID\n");
    ies_ret = ies_res_free(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                           adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to free IPv4 esp hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_res_free", "IPv6 Hash ESP Profile ID\n");
    ies_ret = ies_res_free(ADK_HLP_SWITCH_ID, IES_RES_HASH_PROFILE_ID,
                           adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX]);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to free IPv6 esp hash profile ID\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

    /* Destroy the private memzone created during init time */
    adk_shm_destroy(ADK_HOS_SHM_MEMZONE_NAME);

    /* Terminate IES API services */
    ies_ret = ies_terminate(true);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to terminate HLP IES API"
                      " Services!\n", HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_shutdown_exit;
    }

_adk_hos_shutdown_exit:

    return retValue;
}

adk_status_t
adk_hos_mac_mapper_add (
    uint64_t mac_addr,
    uint8_t  mask_bits,
    uint8_t cmac,
    int *entry_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    union ies_mapper_value mapper_value;
    uint32_t available_index;

    for(available_index = 0; available_index < HOS_MAX_MAC_MAPPER; available_index++)
    {
        if(!adk_hos_pvt_g_p->mac_mapper[available_index].mac_addr)
            break;
    }
    
    /* no space, return error */
    if(available_index == HOS_MAX_MAC_MAPPER)
        return ADK_E_HW_PROGRAMMING_ERROR; 

    mapper_value.mac.mac_addr = mac_addr;
    mapper_value.mac.mac_addr_mask = adk_hos_mac_mask_g[mask_bits];
    mapper_value.mac.valid = IES_MAPPER_MAC_VALID_OTR_DMAC;
    mapper_value.mac.map_mac = cmac;
    mapper_value.mac.routable = true;
    ADK_HOS_LOG_EXT_API("ies_mapper_add_entry", "MAC address 0x%lx\n",
                        mac_addr);
    ies_ret = ies_mapper_add_entry(ADK_HLP_SWITCH_ID, IES_MAPPER_MAC,
                                   &mapper_value, entry_id);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add MAC mapper entry for"
                      " 0x%lx\n", HOS_IES_ERROR_NAME(ies_ret), mac_addr);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_mapper_add_exit;
    }

    adk_hos_pvt_g_p->mac_mapper[available_index].mac_addr = mac_addr;
    adk_hos_pvt_g_p->mac_mapper[available_index].cmac = cmac;
    adk_hos_pvt_g_p->mac_mapper[available_index].entry_id = *entry_id;

_adk_hos_mac_mapper_add_exit:

    return retValue;
}

adk_status_t
adk_hos_mac_mapper_delete (
    int entry_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    union ies_mapper_value mapper_value;
    uint32_t index;

    ADK_HOS_LOG_EXT_API("ies_mapper_get_entry", "entry ID %d\n", entry_id);
    ies_ret = ies_mapper_get_entry(ADK_HLP_SWITCH_ID, IES_MAPPER_MAC,
                                      entry_id, &mapper_value);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get MAC mapper entry %d\n",
                      HOS_IES_ERROR_NAME(ies_ret), entry_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_mapper_delete_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_mapper_delete_entry", "entry ID %d\n", entry_id);
    ies_ret = ies_mapper_delete_entry(ADK_HLP_SWITCH_ID, IES_MAPPER_MAC,
                                      entry_id);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete MAC mapper entry %d\n",
                      HOS_IES_ERROR_NAME(ies_ret), entry_id);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_mapper_delete_exit;
    }

    for(index = 0; index < HOS_MAX_MAC_MAPPER; index++)
    {
        if(adk_hos_pvt_g_p->mac_mapper[index].mac_addr == 
                    mapper_value.mac.mac_addr)
        {
            memset(&adk_hos_pvt_g_p->mac_mapper[index], 0,
                        sizeof(hos_mac_mapper_t));
            break;
        }
    }


_adk_hos_mac_mapper_delete_exit:

    return retValue;
}

adk_status_t
adk_hos_mac_mapper_get (
    uint64_t mac_addr,
    uint8_t *cmac,
    int *entry_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t index;

    for(index = 0; index < HOS_MAX_MAC_MAPPER; index++)
    {
        if(adk_hos_pvt_g_p->mac_mapper[index].mac_addr == mac_addr)
        {
            if(cmac)
                *cmac = adk_hos_pvt_g_p->mac_mapper[index].cmac;

            if(entry_id)
                *entry_id = adk_hos_pvt_g_p->mac_mapper[index].entry_id;

            retValue = ADK_SUCCESS;
            goto _adk_hos_mac_mapper_get_exit;
        }
    }

    /* Not found, return error */
    retValue = ADK_E_INVALID_PARAMETER; 

_adk_hos_mac_mapper_get_exit:

    return retValue;
}

adk_status_t
adk_hos_mac_mapper_get_by_cmac (
    uint8_t cmac,
    uint64_t *mac_addr,
    int *entry_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t index;

    for(index = 0; index < HOS_MAX_MAC_MAPPER; index++)
    {
        if(adk_hos_pvt_g_p->mac_mapper[index].cmac == cmac)
        {
            if(mac_addr)
                *mac_addr = adk_hos_pvt_g_p->mac_mapper[index].mac_addr;

            if(entry_id)
                *entry_id = adk_hos_pvt_g_p->mac_mapper[index].entry_id;

            retValue = ADK_SUCCESS;
            goto _adk_hos_mac_mapper_get_by_cmac_exit;
        }
    }

    /* Not found, return error */
    retValue = ADK_E_INVALID_PARAMETER; 

_adk_hos_mac_mapper_get_by_cmac_exit:

    return retValue;
}

/* Should be called from each secondary application process which requires
 * HOS API services. This should not be called from the process which performs
 * the ADK sys-init. */

adk_status_t
adk_hos_attach (
    void)
{
    adk_status_t retValue = ADK_SUCCESS;
    const struct rte_memzone *mz = NULL;

    /* Get access to the HOS private data */
    ADK_FORCE_LOG_EXT_API("rte_memzone_lookup", "Lookup for a memzone.\n");
    mz = rte_memzone_lookup(ADK_HOS_SHM_MEMZONE_NAME);
    if (mz == NULL)
    {
        ADK_FORCE_LOG_ERROR("Failed to retrieve HOS private data\n");
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _adk_hos_attach_exit;
    }

    adk_hos_pvt_g_p = (adk_hos_pvt_data_t *) mz->addr;

_adk_hos_attach_exit:

    return retValue;
}

/* This API must be called from all secondary processes, which attached
 * to the HOS API serivces, before terminating that process. This API
 * detaches process from the ADK HOS API services. */
adk_status_t
adk_hos_detach (
    void)
{
    adk_status_t retValue = ADK_SUCCESS;
    return retValue;
}

void
adk_hos_dump_hlp_version (
    void)
{
    char buf[64];

    if (ies_api_get_version(buf, 64) == IES_OK)
    {
        ADK_INFO_WRITE("IES API Version: %s\n", buf);
    }
    else
    {
        ADK_INFO_WRITE("IES API Version: Unknown\n");
    }

    return;
}

static bool hos_is_completely_in_hlp(
    adk_hos_object_t obj)
{
    switch(obj)
    {
        case ADK_HOS_L2_TERM_INTF_TBL:
	case ADK_HOS_L2_INGRESS_TBL:
        case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        case ADK_HOS_IP_TERM_TBL:
            return true;
        default:
            return false;
    }

    return false;
}

void adk_hos_disable(void)
{
    adk_hos_table_size_set(ADK_HOS_IP4_CLASSIFIER_TBL, 0);
    adk_hos_table_size_set(ADK_HOS_IP6_CLASSIFIER_TBL, 0);
    adk_hos_table_size_set(ADK_HOS_SPD_IN_TBL, 0);
    adk_hos_table_size_set(ADK_HOS_SPD_OUT_TBL, 0);
}

void adk_hos_enable(void)
{
    adk_hos_table_size_set(ADK_HOS_IP4_CLASSIFIER_TBL, ADK_HOS_DEFAULT_TABLE_SIZE);
    adk_hos_table_size_set(ADK_HOS_IP6_CLASSIFIER_TBL, ADK_HOS_DEFAULT_TABLE_SIZE);
    adk_hos_table_size_set(ADK_HOS_SPD_IN_TBL, ADK_HOS_DEFAULT_TABLE_SIZE);
    adk_hos_table_size_set(ADK_HOS_SPD_OUT_TBL, ADK_HOS_DEFAULT_TABLE_SIZE);
}

static int hos_rule_cmp_func (const void * rule_a, const void * rule_b)
{
    return (((hos_rule_data_t*)rule_a)->prec - ((hos_rule_data_t*)rule_b)->prec);
}

static adk_status_t hos_table_clean_internal(uint32_t tbl_idx,
                                                uint32_t num_elems)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    /* Lowest priority rule needs to be deleted first */
    qsort(adk_hos_pvt_g_p->rule_ids, num_elems, sizeof(hos_rule_data_t), hos_rule_cmp_func);

    for(uint32_t index = 0; index < num_elems; ++index)
    {
        ies_ret = ies_acl_delete_rule(ADK_HLP_SWITCH_ID, tbl_idx, adk_hos_pvt_g_p->rule_ids[index].acl_rule_id);
        if(ies_ret != IES_OK)
        {
            retValue = ADK_E_HW_PROGRAMMING_ERROR;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "IES Error %s, Failed to delete rule from HOS"
                          " table\n", HOS_IES_ERROR_NAME(ies_ret));
            return retValue;
        }
    }

    return retValue;
}

/* This removes all rules except default rule */
adk_status_t
adk_hos_table_clean(adk_hos_object_t obj,
                    bool followPrec)
{
    adk_status_t retValue = ADK_SUCCESS;

    struct ies_acl_rule_info acl_rule;
    uint32_t ies_ret, ies_ret1, tbl_idx;
    adk_hos_rule_id_t rule_id, next_rule_id;
    uint32_t index = 0;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_table_clean_exit;
        break;
    }

    rule_id = -1;
    while(true)
    {
        ies_ret = ies_acl_next_rule(ADK_HLP_SWITCH_ID, tbl_idx, (int) rule_id,
                                (int *) &next_rule_id);

        if (ies_ret != IES_OK && ies_ret != IES_NO_MORE)
        {
            retValue = ADK_E_HW_PROGRAMMING_ERROR;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "IES Error %s, Failed to get next rule ID from HOS"
                          " table\n", HOS_IES_ERROR_NAME(ies_ret));
            goto _adk_hos_table_clean_exit;
        }

        if(rule_id != -1)
        {
            ies_ret1 = ies_acl_get_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id, &acl_rule); 
            if(ies_ret1 != IES_OK)
            {
                retValue = ADK_E_HW_PROGRAMMING_ERROR;
                ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                              ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                              "IES Error %s, Failed to get rule from HOS"
                              " table\n", HOS_IES_ERROR_NAME(ies_ret1));
                goto _adk_hos_table_clean_exit;
            }

            /* Don't delete default rule */
            if(!acl_rule.precedence)
            {
                rule_id = next_rule_id;
                if(ies_ret == IES_NO_MORE)
                    goto _adk_hos_table_clean_exit;

                continue;
            }

            /* Can't delete the rule immediately if precedence has to be followed */ 
            if(followPrec)
            {
                adk_hos_pvt_g_p->rule_ids[index].prec = acl_rule.precedence;
                adk_hos_pvt_g_p->rule_ids[index].acl_rule_id = rule_id;
                ++index;
            }
            else
            {
                ies_ret1 = ies_acl_delete_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id);
                if(ies_ret1 != IES_OK)
                {
                    retValue = ADK_E_HW_PROGRAMMING_ERROR;
                    ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                                  ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                                  "IES Error %s, Failed to delete rule from HOS"
                                  " table\n", HOS_IES_ERROR_NAME(ies_ret1));
                    goto _adk_hos_table_clean_exit;
                }
            }
        }

        if (ies_ret == IES_NO_MORE)
        {
            retValue = ADK_SUCCESS;
            goto _adk_hos_table_clean_exit;
        }

        rule_id = next_rule_id;
    }

_adk_hos_table_clean_exit:
    if(retValue != ADK_SUCCESS)
        return retValue;

    if(followPrec && index)
        retValue = hos_table_clean_internal(tbl_idx, index);

    return retValue;
}

adk_status_t
adk_hos_table_size_set (
    adk_hos_object_t obj,
    uint32_t size)
{
    if(obj <= ADK_HOS_INVALID_OBJ || obj >= ADK_HOS_MAX_OBJ)
        return ADK_E_INVALID_PARAMETER; 

    if(hos_is_completely_in_hlp(obj))
        return ADK_E_INVALID_PARAMETER;
                
    adk_hos_pvt_g_p->tbl_usage[obj].size = size;
    
    return ADK_SUCCESS;
}

adk_status_t
adk_hos_table_size_get (
    adk_hos_object_t obj,
    uint32_t* size)
{
    if(obj <= ADK_HOS_INVALID_OBJ || obj >= ADK_HOS_MAX_OBJ ||
        NULL == size)
        return ADK_E_INVALID_PARAMETER; 

    if(hos_is_completely_in_hlp(obj))
        return ADK_E_INVALID_PARAMETER;

    *size = adk_hos_pvt_g_p->tbl_usage[obj].size;
    
    return ADK_SUCCESS;
}

adk_status_t
adk_hos_table_size_reset (
    adk_hos_object_t obj)
{
    if(obj <= ADK_HOS_INVALID_OBJ || obj >= ADK_HOS_MAX_OBJ)
        return ADK_E_INVALID_PARAMETER; 

    if(hos_is_completely_in_hlp(obj))
        return ADK_E_INVALID_PARAMETER;
                
    adk_hos_pvt_g_p->tbl_usage[obj].size = ADK_HOS_DEFAULT_TABLE_SIZE;
    
    return ADK_SUCCESS;
}

adk_status_t
adk_hos_table_occupancy_incr (
    adk_hos_object_t obj)
{
    if(obj <= ADK_HOS_INVALID_OBJ || obj >= ADK_HOS_MAX_OBJ)
        return ADK_E_INVALID_PARAMETER; 

    /* Looks like no debugging is set for partial offload */
    if(adk_hos_pvt_g_p->tbl_usage[obj].size ==
            UINT32_MAX)
        return ADK_SUCCESS;

    if(hos_is_completely_in_hlp(obj))
        return ADK_E_INVALID_PARAMETER;

    /* Overflow error */
    if(adk_hos_pvt_g_p->tbl_usage[obj].occupancy 
        == adk_hos_pvt_g_p->tbl_usage[obj].size)
        return ADK_E_INTERNAL_ERROR;

    adk_hos_pvt_g_p->tbl_usage[obj].occupancy++;
    return ADK_SUCCESS;    
}

adk_status_t
adk_hos_table_occupancy_decr (
    adk_hos_object_t obj)
{
    if(obj <= ADK_HOS_INVALID_OBJ || obj >= ADK_HOS_MAX_OBJ)
        return ADK_E_INVALID_PARAMETER; 

    /* Looks like no debugging is set for partial offload */
    if(adk_hos_pvt_g_p->tbl_usage[obj].size ==
            UINT32_MAX)
        return ADK_SUCCESS;

    if(hos_is_completely_in_hlp(obj))
        return ADK_E_INVALID_PARAMETER;

    /* Underflow error */
    if(!(adk_hos_pvt_g_p->tbl_usage[obj].occupancy))
        return ADK_E_INTERNAL_ERROR;

    adk_hos_pvt_g_p->tbl_usage[obj].occupancy--;
    return ADK_SUCCESS; 
}

bool
adk_hos_table_full (
    adk_hos_object_t obj)
{
    /* Looks like no debugging is set for partial offload */
    if(adk_hos_pvt_g_p->tbl_usage[obj].size ==
            UINT32_MAX)
        return false;
 
    return adk_hos_pvt_g_p->tbl_usage[obj].occupancy >=
             adk_hos_pvt_g_p->tbl_usage[obj].size ? true : false;
}

/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
