/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp_police.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS (Hardware Offload Services) APIs for use
 * by ADK modules for ingress policing.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

adk_status_t
adk_hos_policer_init (
    void)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_pol_profile profile;
    enum ies_pol_profile_mask mask;

    ADK_MEMSET(&profile, 0, sizeof(profile));

    mask = (IES_POL_PROF_BANK_SEL |
            IES_POL_PROF_COLOR_BLIND |
            IES_POL_PROF_PRECEDENCE |
            IES_POL_PROF_DEBIT_MODE |
            IES_POL_PROF_L3_LEN_MODE);
    profile.color_blind = true;
    profile.high_precedence = true;
    profile.debit_mode = IES_POL_DEBIT_MODE_SEGMENT;
    profile.l3_len_mode = IES_POL_L3_LENGTH_OUTER_PACKET;
    ADK_HOS_LOG_EXT_API("ies_pol_set_profile", "L2D profile ID %d\n",
                        adk_hos_pvt_g_p->police_profile_id[HOS_L2_COLOR_BIND]);
    ies_ret =
        ies_pol_set_profile(ADK_HLP_SWITCH_ID, IES_POL_SEL_BANK_L2D,
                            adk_hos_pvt_g_p->police_profile_id[HOS_L2_COLOR_BIND],
                            &profile, mask);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure L2 policer profile!\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_policer_init_exit;
    }

    profile.high_precedence = false;
    profile.debit_mode = IES_POL_DEBIT_MODE_PACKET;
    ADK_HOS_LOG_EXT_API("ies_pol_set_profile", "L3D profile ID %d\n",
                        adk_hos_pvt_g_p->police_profile_id[HOS_L3_COLOR_BIND]);
    ies_ret =
        ies_pol_set_profile(ADK_HLP_SWITCH_ID, IES_POL_SEL_BANK_L3D,
                            adk_hos_pvt_g_p->police_profile_id[HOS_L3_COLOR_BIND],
                            &profile, mask);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to configure L3 policer profile!\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_policer_init_exit;
    }

_adk_hos_policer_init_exit:
    
    return retValue;
}


/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
