/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp_qos.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS (Hardware Offload Services) APIs for use
 * by ADK modules for QoS configurations in HLP.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

adk_status_t
adk_hos_qos_init (
    void)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint8_t ingress_pri[] =
    {
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
        ADK_HOS_DEFAULT_INGRESS_PRI,
    };

    /* Configure the default priority map for PCP values */
    {
        retValue = adk_hos_ingress_pri_map_set(ingress_pri);
        if (retValue != ADK_SUCCESS)
        {
            goto _adk_hos_qos_init_exit;
        }
    }

_adk_hos_qos_init_exit:

    return retValue;
}

adk_status_t
adk_hos_ingress_pri_map_set (
    uint8_t pri[8])
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    union ies_qos_pri_map_value pcp_map_value;
    uint8_t pcp, pcp_cfi;

    ADK_MEMSET((void *) &pcp_map_value, 0x0, sizeof(pcp_map_value));
    pcp_map_value.rx_vpri.profile = ADK_HOS_DOM_PRI_MAP_PROFILE_ID;
    for (pcp = 0; pcp <= HOS_VLAN_PRI_MAX; pcp++)
    {
        /* Add mappings for both cases - CFI bit set and reset */
        pcp_cfi = pcp << 1;
        pcp_map_value.rx_vpri.tc[pcp_cfi] = pri[pcp];
        pcp_map_value.rx_vpri.int_vpri[pcp_cfi] = pri[pcp];
        pcp_cfi |= 1;
        pcp_map_value.rx_vpri.tc[pcp_cfi] = pri[pcp];
        pcp_map_value.rx_vpri.int_vpri[pcp_cfi] = pri[pcp];
    }
    ADK_HOS_LOG_EXT_API("ies_qos_set_pri_map", "\n");
    ies_ret = ies_qos_set_pri_map(ADK_HLP_SWITCH_ID, IES_QOS_PRI_MAP_RX_VPRI,
                                    &pcp_map_value);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set PRI map for PCP %d\n",
                      HOS_IES_ERROR_NAME(ies_ret), pcp);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_ingress_pri_map_set_exit;
    }

_adk_hos_ingress_pri_map_set_exit:

    return retValue;
}

/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
