/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp_rules.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS APIs for managing rules in the HLP FFU
 * for the tables defined and used by ADK modules.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

adk_status_t
adk_hos_rule_add (
    adk_hos_object_t obj,
    adk_hos_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        retValue = adk_hos_l2_term_intf_tbl_rule_add(
            (adk_hos_l2_term_intf_tbl_rule_attr_t *) rule_attr,
            rule_id);
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        ((adk_hos_ip_classifier_tbl_rule_attr_t*) rule_attr)->is_ipv6 = false;
        retValue = adk_hos_ip_classifier_tbl_rule_add(
            (adk_hos_ip_classifier_tbl_rule_attr_t *) rule_attr,
            rule_id);
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        ((adk_hos_ip_classifier_tbl_rule_attr_t*) rule_attr)->is_ipv6 = true;
        retValue = adk_hos_ip_classifier_tbl_rule_add(
            (adk_hos_ip_classifier_tbl_rule_attr_t *) rule_attr,
            rule_id);
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        retValue = adk_hos_supported_ethertype_tbl_rule_add(
            (adk_hos_ethertype_tbl_rule_attr_t*) rule_attr,
            rule_id);
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        retValue = adk_hos_supported_ipproto_tbl_rule_add(
            (adk_hos_ipproto_tbl_rule_attr_t*) rule_attr,
            rule_id);
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        retValue = adk_hos_l2_ingress_tbl_rule_add(
            (adk_hos_l2_ingress_tbl_rule_attr_t *) rule_attr,
            rule_id);
        break;

    case ADK_HOS_IP_TERM_TBL:
        retValue = adk_hos_ip_term_tbl_rule_add(
            (adk_hos_ip_term_tbl_rule_attr_t *) rule_attr,
            rule_id);
        break;

    case ADK_HOS_SPD_IN_TBL:
        retValue = adk_hos_spd_tbl_rule_add(
            (adk_hos_spd_tbl_rule_attr_t *) rule_attr,
            false, rule_id);
        break;

    case ADK_HOS_SPD_OUT_TBL:
        retValue = adk_hos_spd_tbl_rule_add(
            (adk_hos_spd_tbl_rule_attr_t *) rule_attr,
            true, rule_id);
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_add_exit;
        break;
    }

_adk_hos_rule_add_exit:

    return retValue;
}

static adk_status_t
hos_disallow_default_rule_delete (
    adk_hos_rule_id_t rule_id,
    uint32_t default_rule_idx
)
{
    adk_status_t retValue = ADK_SUCCESS;
    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[default_rule_idx])
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Default rule (rule ID - %d) cannot be deleted\n",
                      (int) rule_id);
        return retValue;
    }
    return retValue;
}

adk_status_t
adk_hos_rule_delete (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_L2_TERM_INTF_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_IP4_CLASSIFIER_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_IP6_CLASSIFIER_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_L2_INGRESS_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
         tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_IP4_TERM_DEF_RULE_IDX);
        retValue += hos_disallow_default_rule_delete(rule_id, HOS_IP6_TERM_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_SPD_IN_IP4_DEF_RULE_IDX);
        retValue += hos_disallow_default_rule_delete(rule_id, HOS_SPD_IN_IP6_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        retValue = hos_disallow_default_rule_delete(rule_id, HOS_SPD_OUT_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_delete_exit;
        }
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_delete_exit;
        break;
    }

    ADK_HOS_LOG_EXT_API("ies_domain_delete_entry", "Tbl ID = %u,"
                        " rule ID = %d\n", tbl_idx, rule_id);
    ies_ret = ies_acl_delete_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete rule from HOS table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_rule_delete_exit;
    }

    adk_hos_table_occupancy_decr(obj);

_adk_hos_rule_delete_exit:

    return retValue;
}

adk_status_t
adk_hos_rule_update (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id,
    adk_hos_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        if ((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_TERM_INTF_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        if ((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_CLASSIFIER_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;
    case ADK_HOS_IP6_CLASSIFIER_TBL:
        if ((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP6_CLASSIFIER_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;
    case ADK_HOS_L2_INGRESS_TBL:
        if ((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_INGRESS_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;
    case ADK_HOS_SPD_IN_TBL:
        if (((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP4_DEF_RULE_IDX]) ||
            (int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP6_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;
    case ADK_HOS_SPD_OUT_TBL:
        if ((int) rule_id ==
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_OUT_DEF_RULE_IDX])
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "Default rule (rule ID - %d) state cannot be changed\n",
                          (int) rule_id);
            goto _adk_hos_rule_update_exit;
        }
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_update_exit;
        break;
    }

    /* Get the rule attributes */
    ADK_HOS_LOG_EXT_API("ies_acl_get_rule", "tbl ID = %u, rule ID = %d\n",
                        tbl_idx, rule_id);
    ies_ret = ies_acl_get_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id, &acl_rule);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get rule from HOS table for"
                      " modification\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_rule_update_exit;
    }

    if (obj == ADK_HOS_L2_TERM_INTF_TBL)
    {
        retValue = adk_hos_l2_term_intf_tbl_rule_update(
            (adk_hos_l2_term_intf_tbl_rule_attr_t *) rule_attr,
            &acl_rule);
        if (retValue != ADK_SUCCESS)
        {
            goto _adk_hos_rule_update_exit;
        }
    }
    else if ((obj == ADK_HOS_IP4_CLASSIFIER_TBL) ||
                (obj == ADK_HOS_IP6_CLASSIFIER_TBL))
    {
        retValue = adk_hos_ip_classifier_tbl_rule_update(
            (adk_hos_ip_classifier_tbl_rule_attr_t *) rule_attr,
            &acl_rule);
        if (retValue != ADK_SUCCESS)
        {
            goto _adk_hos_rule_update_exit;
        }
    }
    else if (obj == ADK_HOS_L2_INGRESS_TBL)
    {
        retValue = adk_hos_l2_ingress_tbl_rule_update(
            (adk_hos_l2_ingress_tbl_rule_attr_t*) rule_attr,
            &acl_rule);
        if (retValue != ADK_SUCCESS)
        {
            goto _adk_hos_rule_update_exit;
        }
    }
    else if ((obj == ADK_HOS_SPD_IN_TBL) ||
                (obj == ADK_HOS_SPD_OUT_TBL))
    {
        retValue = adk_hos_spd_tbl_rule_update(
            (adk_hos_spd_tbl_rule_attr_t*) rule_attr,
            &acl_rule);
        if (retValue != ADK_SUCCESS)
        {
            goto _adk_hos_rule_update_exit;
        }
    }

    /* Write back the modified rule */
    ADK_HOS_LOG_EXT_API("ies_acl_modify_rule", "tbl ID = %u, rule ID = %d\n",
                        tbl_idx, rule_id);
    ies_ret = ies_acl_modify_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id, &acl_rule);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to update rule in the HOS table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_rule_update_exit;
    }

_adk_hos_rule_update_exit:

    return retValue;
}

static adk_status_t
hos_disallow_default_rule_state_set (
    adk_hos_rule_id_t rule_id,
    uint32_t default_rule_idx
)
{
    adk_status_t retValue = ADK_SUCCESS;
    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[default_rule_idx])
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Default rule (rule ID - %d) state cannot be changed\n",
                      (int) rule_id);
        return retValue;
    }
    return retValue;
}

adk_status_t
adk_hos_rule_state_set (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id,
    bool enable)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    enum ies_acl_entry_state state;

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_L2_TERM_INTF_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_IP4_CLASSIFIER_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_IP6_CLASSIFIER_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_IP4_TERM_DEF_RULE_IDX);
        retValue += hos_disallow_default_rule_state_set(rule_id, HOS_IP6_TERM_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_L2_INGRESS_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_SPD_IN_IP4_DEF_RULE_IDX);
        retValue += hos_disallow_default_rule_state_set(rule_id, HOS_SPD_IN_IP6_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        retValue = hos_disallow_default_rule_state_set(rule_id, HOS_SPD_OUT_DEF_RULE_IDX);
        if(ADK_SUCCESS != retValue)
        {
            goto _adk_hos_rule_state_set_exit;
        }
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_state_set_exit;
        break;
    }

    if (enable)
    {
        state = IES_ACL_RULE_ENTRY_STATE_VALID;
    }
    else
    {
        state = IES_ACL_RULE_ENTRY_STATE_INVALID;
    }

    /* Set the rule state */
    ADK_HOS_LOG_EXT_API("ies_acl_set_rule_state", "tbl ID = %u, rule ID = %d\n",
                        tbl_idx, rule_id);
    ies_ret = ies_acl_set_rule_state(ADK_HLP_SWITCH_ID, tbl_idx, rule_id, state);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set rule state for rule ID %d\n",
                      HOS_IES_ERROR_NAME(ies_ret), (int) rule_id);
        goto _adk_hos_rule_state_set_exit;
    }

_adk_hos_rule_state_set_exit:

    return retValue;
}

adk_status_t
adk_hos_rule_precedence_state_set (
    adk_hos_object_t obj,
    uint32_t num_precedences,
    uint32_t *precedence,
    enum ies_acl_entry_state* entryStates)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    if (num_precedences > ADK_HOS_MAX_PREC_STATE_UPDATE)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Number of precedence levels specified (%d) exceeds max"
                      " supported for atomic state update!\n",
                      (int) num_precedences);
        goto _adk_hos_rule_precedence_state_set_exit;
    }

    switch (obj)
    {
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_precedence_state_set_exit;
        break;
    }

    /* Set the rule state */
    ADK_HOS_LOG_EXT_API("ies_acl_set_precedences_state", "tbl ID = %u,"
                        " num levels = %d\n", tbl_idx, num_precedences);
    ies_ret = ies_acl_set_precedences_state(ADK_HLP_SWITCH_ID, tbl_idx,
                                            (int) num_precedences,
                                            (int *) precedence, entryStates);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set state for %d precedences\n",
                      HOS_IES_ERROR_NAME(ies_ret), (int) num_precedences);
        goto _adk_hos_rule_precedence_state_set_exit;
    }

_adk_hos_rule_precedence_state_set_exit:

    return retValue;
}

adk_status_t
adk_hos_next_rule_id_get (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id,  /* Set to -1 for first rule */
    adk_hos_rule_id_t *next_rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_next_rule_id_get_exit;
        break;
    }

    /* Get the rule attributes */
    /* ADK_HOS_LOG_EXT_API("ies_acl_next_rule", "tbl ID = %u, rule ID = %lu\n",
                        tbl_idx, rule_id); */
    ies_ret = ies_acl_next_rule(ADK_HLP_SWITCH_ID, tbl_idx, (int) rule_id,
                                (int *) next_rule_id);

    if (ies_ret == IES_NO_MORE)
    {
        *next_rule_id = (adk_hos_rule_id_t) -1;
        goto _adk_hos_next_rule_id_get_exit;
    }
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get next rule ID from HOS"
                      " table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_next_rule_id_get_exit;
    }

_adk_hos_next_rule_id_get_exit:

    return retValue;
}

adk_status_t
adk_hos_rule_get (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id,
    adk_hos_rule_attr_t *rule_attr) /* Set is_ipv6 to true or false */
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_get_exit;
        break;
    }

    /* Get the rule attributes */
    /* ADK_HOS_LOG_EXT_API("ies_acl_get_rule", "tbl ID = %u, rule ID = %lu\n",
                        tbl_idx, rule_id); */
    ies_ret = ies_acl_get_rule(ADK_HLP_SWITCH_ID, tbl_idx, rule_id, &acl_rule);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get rule-%d from HOS table-%d\n",
                      HOS_IES_ERROR_NAME(ies_ret), (int) rule_id,
                      (int) tbl_idx);
        goto _adk_hos_rule_get_exit;
    }

    if (obj == ADK_HOS_L2_TERM_INTF_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_l2_term_intf_tbl_rule_attr_t));
        retValue = adk_hos_l2_term_intf_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_l2_term_intf_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_IP4_CLASSIFIER_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_ip_classifier_tbl_rule_attr_t));
        ((adk_hos_ip_classifier_tbl_rule_attr_t*) rule_attr)->is_ipv6 = false;
        retValue = adk_hos_ip_classifier_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_ip_classifier_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_IP6_CLASSIFIER_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_ip_classifier_tbl_rule_attr_t));
        ((adk_hos_ip_classifier_tbl_rule_attr_t*) rule_attr)->is_ipv6 = true;
        retValue = adk_hos_ip_classifier_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_ip_classifier_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_SUPPORTED_ETHERTYPE_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_ethertype_tbl_rule_attr_t));
        retValue = adk_hos_supported_ethertype_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_ethertype_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_SUPPORTED_IPPROTO_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_ipproto_tbl_rule_attr_t));
        retValue = adk_hos_supported_ipproto_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_ipproto_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_L2_INGRESS_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_l2_ingress_tbl_rule_attr_t));
        retValue = adk_hos_l2_ingress_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_l2_ingress_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_IP_TERM_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_ip_term_tbl_rule_attr_t));
        retValue = adk_hos_ip_term_tbl_rule_get(
            rule_id,
            &acl_rule,
            (adk_hos_ip_term_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_SPD_IN_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_spd_tbl_rule_attr_t));
        retValue = adk_hos_spd_tbl_rule_get(
            rule_id,
            false,
            &acl_rule,
            (adk_hos_spd_tbl_rule_attr_t *) rule_attr);
    }
    else if (obj == ADK_HOS_SPD_OUT_TBL)
    {
        ADK_MEMSET(rule_attr, 0x0, sizeof(adk_hos_spd_tbl_rule_attr_t));
        retValue = adk_hos_spd_tbl_rule_get(
            rule_id,
            true,
            &acl_rule,
            (adk_hos_spd_tbl_rule_attr_t *) rule_attr);
    }
    else
    {
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_get_exit;
    }

_adk_hos_rule_get_exit:

    return retValue;
}

adk_status_t
adk_hos_rule_stats_get (
    adk_hos_object_t obj,
    adk_hos_rule_id_t rule_id,
    adk_hos_rule_stats_t *stats)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    uint64_t action_mask = 0;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_counters cntrs;

    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        action_mask = HOS_L2_TERM_INTF_TBL_ACTION;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        action_mask = HOS_IP4_CLASSIFIER_TBL_ACTION;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        action_mask = HOS_IP6_CLASSIFIER_TBL_ACTION;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        action_mask = HOS_SUPPORTED_ETHERTYPE_TBL_ACTION;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        action_mask = HOS_SUPPORTED_IPPROTO_TBL_ACTION;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        action_mask = HOS_L2_INGRESS_TBL_ACTION;
        break;

    case ADK_HOS_IP_TERM_TBL:
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        action_mask = HOS_IP_TERM_TBL_ACTION;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        action_mask = HOS_SPD_IN_TBL_ACTION;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        action_mask = HOS_SPD_OUT_TBL_ACTION;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_stats_get_exit;
        break;
    }

    /* Skip getting count for tables which don't have stats enabled */
    if ((action_mask & IES_ACL_ACTION_COUNT) == 0)
    {
        stats->num_pkts = 0;
        stats->num_octets = 0;
        goto _adk_hos_rule_stats_get_exit;
    }

    /* ADK_HOS_LOG_EXT_API("ies_acl_get_rule_count", "tbl ID = %u, rule ID = %lu\n",
                        tbl_idx, rule_id); */
    ies_ret = ies_acl_get_rule_count(ADK_HLP_SWITCH_ID, tbl_idx,
                                     rule_id, &cntrs);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get stats for the HOS rule\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_rule_stats_get_exit;
    }

    stats->num_pkts = cntrs.cnt_pkts;
    stats->num_octets = cntrs.cnt_octets;

_adk_hos_rule_stats_get_exit:

    return retValue;
}

adk_status_t
adk_hos_rule_num_get (
    adk_hos_object_t obj,
    uint32_t* num_rules)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_table_usage table_usage;

    if(NULL == num_rules)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        goto _adk_hos_rule_num_get_exit;
    }
        
    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        tbl_idx = HOS_L2_TERM_INTF_TBL_IDX;
        break;
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_IP6_CLASSIFIER_TBL:
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        tbl_idx = HOS_SUPPORTED_ETHERTYPE_TBL_IDX;
        break;

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        tbl_idx = HOS_SUPPORTED_IPPROTO_TBL_IDX;
        break;

    case ADK_HOS_L2_INGRESS_TBL:
        tbl_idx = HOS_L2_INGRESS_TBL_IDX;
        break;

    case ADK_HOS_IP_TERM_TBL:
        tbl_idx = HOS_IP_TERM_TBL_IDX;
        break;

    case ADK_HOS_SPD_IN_TBL:
        tbl_idx = HOS_SPD_IN_TBL_IDX;
        break;

    case ADK_HOS_SPD_OUT_TBL:
        tbl_idx = HOS_SPD_OUT_TBL_IDX;
        break;

    default:
        retValue = ADK_E_UNSUPPORTED;
        goto _adk_hos_rule_num_get_exit;
        break;
    }

    ies_ret = ies_acl_get_attr(ADK_HLP_SWITCH_ID, tbl_idx, IES_ACL_TABLE_USAGE, &table_usage);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get table usage attribute from HOS table-%d\n",
                      HOS_IES_ERROR_NAME(ies_ret), tbl_idx);
        goto _adk_hos_rule_num_get_exit;
    }

    *num_rules = table_usage.nb_rules;

_adk_hos_rule_num_get_exit:

    return retValue;
}

/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
