/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp_stats.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS APIs to retrieve statistics.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

#define TEST_TRIGGER_GROUP 11000

adk_status_t
adk_hos_mac_stats_get (
    int port,
    adk_hos_port_statistics_t *stats)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    int hos_stats[IES_STATS_NUM_PORT_STATS], num_stats, i;
    struct ies_stats_data data;

    ADK_HOS_LOG_EXT_API("ies_stats_get_profile", "profile ID %d\n",
                        adk_hos_pvt_g_p->port_stats_profile_id);
    ies_ret = ies_stats_get_profile(ADK_HLP_SWITCH_ID,
                                    adk_hos_pvt_g_p->port_stats_profile_id,
                                    hos_stats, IES_STATS_NUM_PORT_STATS,
                                    &num_stats);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get stats profile\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_stats_get_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_stats_read", "profile ID %d, HLP port %d\n",
                        adk_hos_pvt_g_p->port_stats_profile_id, port);
    ies_ret = ies_stats_read(ADK_HLP_SWITCH_ID,
                             adk_hos_pvt_g_p->port_stats_profile_id, &port, 1,
                             &data);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to read port profile stats\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_stats_get_exit;
    }

    /* Zero out the unavailable counters */
    for (i = 0; i < num_stats; i++)
    {
        if (data.counters[hos_stats[i]] == IES_STATS_INVALID)
        {
            data.counters[i] = 0;
        }
    }

    /* Make sure that all counters used below are enabled in the profile during
     * HOS initialization */

    stats->tx_octets = data.counters[IES_STAT_TX_BYTE];
    stats->tx_bad_octets = data.counters[IES_STAT_TX_BAD_BYTE];
    stats->tx_frm = data.counters[IES_STAT_TX_FRAME];
    stats->tx_bad_frm = data.counters[IES_STAT_TX_BAD_FRAME];
    stats->tx_mcast = data.counters[IES_STAT_TX_L2_MC_FRAME];
    stats->tx_bcast = data.counters[IES_STAT_TX_L2_BC_FRAME];
    stats->tx_pause = data.counters[IES_STAT_TX_CTRL];
    stats->tx_fcs = data.counters[IES_STAT_TX_FCS_ERROR];
    stats->tx_defer = 0; /* data.counters[]; */
    stats->tx_xsdfr = 0; /* data.counters[]; */
    stats->tx_1col = 0; /* data.counters[]; */
    stats->tx_mcol = 0; /* data.counters[]; */
    stats->tx_lcol = 0; /* data.counters[]; */
    stats->tx_xscol = 0; /* data.counters[]; */
    stats->tx_frag = 0; /* data.counters[]; */
    stats->tx_usize = 0; /* data.counters[]; */
    stats->tx_64 = data.counters[IES_STAT_TX_64];
    stats->tx_65 = data.counters[IES_STAT_TX_65];
    stats->tx_128 = data.counters[IES_STAT_TX_128];
    stats->tx_256 = data.counters[IES_STAT_TX_256];
    stats->tx_512 = data.counters[IES_STAT_TX_512];
    stats->tx_1024 = data.counters[IES_STAT_TX_1024];
    stats->tx_1519 = data.counters[IES_STAT_TX_1519];
    stats->tx_osize = 0; /* data.counters[]; */
    stats->tx_jab = data.counters[IES_STAT_TX_JABBER];
    stats->tx_urun = data.counters[IES_STAT_TX_UNDERRUN];
    stats->tx_stat_ovfl = 0; /* data.counters[]; */

    stats->rx_octets = data.counters[IES_STAT_RX_BYTE];
    stats->rx_bad_octets = data.counters[IES_STAT_RX_BAD_BYTE];
    stats->rx_frm = data.counters[IES_STAT_RX_FRAME];
    stats->rx_bad_frm = data.counters[IES_STAT_RX_BAD_FRAME];
    stats->rx_mcast = (data.counters[IES_STAT_IPV4_L2_MCAST_FRAME] +
                       data.counters[IES_STAT_IPV6_L2_MCAST_FRAME] +
                       data.counters[IES_STAT_NONIP_L2_MCAST_FRAME]);
    stats->rx_bcast = (data.counters[IES_STAT_IPV4_L2_BCAST_FRAME] +
                       data.counters[IES_STAT_IPV6_L2_BCAST_FRAME] +
                       data.counters[IES_STAT_NONIP_L2_BCAST_FRAME]);
    stats->rx_ctl = data.counters[IES_STAT_RX_CTRL];
    stats->rx_pause = data.counters[IES_STAT_RX_PAUSE];
    stats->rx_unk_op = 0; /* data.counters[]; */
    stats->rx_fcs = data.counters[IES_STAT_RX_FCS_ERROR];
    stats->rx_align = 0; /* data.counters[]; */
    stats->rx_length = 0; /* data.counters[]; */
    stats->rx_code = data.counters[IES_STAT_RX_CODE_ERROR];
    stats->rx_frag = 0; /* data.counters[]; */
    stats->rx_usize = data.counters[IES_STAT_RX_UNDERSIZE];
    stats->rx_64 = data.counters[IES_STAT_RX_64];
    stats->rx_65 = data.counters[IES_STAT_RX_65];
    stats->rx_128 = data.counters[IES_STAT_RX_128];
    stats->rx_256 = data.counters[IES_STAT_RX_256];
    stats->rx_512 = data.counters[IES_STAT_RX_512];
    stats->rx_1024 = data.counters[IES_STAT_RX_1024];
    stats->rx_1519 = data.counters[IES_STAT_RX_1519];
    stats->rx_osize = data.counters[IES_STAT_RX_OVERSIZE];
    stats->rx_jab = data.counters[IES_STAT_RX_JABBER];
    stats->rx_drop = 0; /* data.counters[]; */
    stats->rx_carrier = 0; /* data.counters[]; */
    stats->rx_urun = 0; /* data.counters[]; */
    stats->rx_stat_ovfl = 0; /* data.counters[]; */

_adk_hos_mac_stats_get_exit:

    return retValue;
}

adk_status_t
adk_hos_mac_stats_reset (
    int port)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    ADK_HOS_LOG_EXT_API("ies_stats_reset", "HLP port %d\n", port);
    ies_ret = ies_stats_reset(ADK_HLP_SWITCH_ID, &port, 1);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to reset port %d stats\n",
                      HOS_IES_ERROR_NAME(ies_ret), port);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_mac_stats_reset_exit;
    }

_adk_hos_mac_stats_reset_exit:

    return retValue;
}

adk_status_t
adk_hos_dump_stats (
    int port)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    int hos_stats[IES_STATS_NUM_PORT_STATS], num_stats, i;
    char cntr_name[IES_STATS_NAME_WIDTH];
    enum ies_stats ies_stats_id;
    struct ies_stats_data data;
    enum ies_port_state state;

    ies_ret = ies_stats_get_profile(ADK_HLP_SWITCH_ID, IES_STAT_PROFILE_ALL,
                                    hos_stats, IES_STATS_NUM_PORT_STATS,
                                    &num_stats);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get stats profile\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_dump_stats_exit;
    }

    ies_ret = ies_port_get_state(ADK_HLP_SWITCH_ID, port, &state);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get status for port-%d\n",
                      HOS_IES_ERROR_NAME(ies_ret), port);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_dump_stats_exit;
    }

    ies_ret = ies_stats_read(ADK_HLP_SWITCH_ID, IES_STAT_PROFILE_ALL, &port, 1,
                             &data);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to read stats\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_dump_stats_exit;
    }

    ADK_INFO_WRITE("========================================\n");
    ADK_INFO_WRITE("HOS non-zero stats for port %d", port);
    switch(state)
    {
    case IES_PORT_STATE_UP:
        ADK_INFO_WRITE(" (%s)\n", "Enabled");
        break;
    case IES_PORT_STATE_DOWN:
        ADK_INFO_WRITE(" (%s)\n", "Disabled");
        break;
    default:
        ADK_INFO_WRITE(" (%s)\n", "Unknown");
        break;
    }
    ADK_INFO_WRITE("----------------------------------------\n");

    for (i = 0; i < num_stats; i++)
    {
        ies_stats_id = hos_stats[i];

        if (data.counters[ies_stats_id] == IES_STATS_INVALID)
            continue;

        ies_ret = ies_stats_get_name(ies_stats_id, cntr_name,
                                     IES_STATS_NAME_WIDTH);
        if (ies_ret != IES_OK)
        {
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                          ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                          "IES Error %s, Failed to get stats name (IES stats ID"
                          " %d)\n", HOS_IES_ERROR_NAME(ies_ret), ies_stats_id);
            retValue = ADK_E_HW_PROGRAMMING_ERROR;
            goto _adk_hos_dump_stats_exit;
        }
        if (data.counters[ies_stats_id] != 0)    /* Only non-zero stats */
        {
            ADK_INFO_WRITE("%s = %ld\n", cntr_name, data.counters[hos_stats[i]]);
        }
    }

    ADK_INFO_WRITE("==============================\n");

_adk_hos_dump_stats_exit:

    return retValue;
}

/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
