/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_cp_tables.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _HOS SIF HOS Component
 * @{
 *     @addtogroup _hosFunctions  HOS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK HOS (Hardware Offload Services) APIs for use
 * by ADK modules for managing lookup tables in hardware.
 *
 *  **********************************************************************/

#include "adk_hos_pvt.h"

/* This function creates all the tables and add default rules for TCAM based
 * tables */

adk_status_t
adk_hos_init_tables(
    void)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_table_info acl_tbl;
    struct ies_acl_rule_info acl_rule;
    struct ies_acl_remap_config remap_cfg;

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));
    acl_tbl.location = HOS_L2_TERM_INTF_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_L2_TERM_INTF_TBL_INST;
    acl_tbl.scenario = HOS_L2_TERM_INTF_TBL_SCENARIO;
    acl_tbl.condition = HOS_L2_TERM_INTF_TBL_CONDITION;
    acl_tbl.mask.meta.sel = IES_ACL_MATCH_L2_DOMAIN;
    acl_tbl.mask.meta.l2_domain = HOS_L2_DOMAIN_MASK;
    acl_tbl.mask.map_dmac = HOS_MAC_MAP_MASK;
    acl_tbl.mask.vlan_id = HOS_VLAN_ID_MASK;
    acl_tbl.min_entries = HOS_L2_TERM_INTF_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_L2_TERM_INTF_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "L2 term-intf table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_L2_TERM_INTF_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create L2 Term Intf table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_INNER_ID;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "L2 term-intf table, Action Set Inner ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_INNER_ID);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_L2_TERM_INTF_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_INNER_ID);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set L2 Term-intf table remap slots"
                      " for Inner ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_PKT_TYPE;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "L2 term-intf table, Action Set Pkt-Type"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_PKT_TYPE);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_L2_TERM_INTF_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_PKT_TYPE);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set L2 term-intf table remap slots"
                      " for Pkt-Type\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create IPv4 Ingress Classifier table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_IP4_CLASSIFIER_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_IP4_CLASSIFIER_TBL_INST;
    acl_tbl.scenario = HOS_IP4_CLASSIFIER_TBL_SCENARIO;
    acl_tbl.condition = HOS_IP4_CLASSIFIER_TBL_CONDITION;
    acl_tbl.mask.protocol = HOS_IP_PROTO_MASK;
    acl_tbl.mask.sip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.l4_src = HOS_L4_PORT_MASK;
    acl_tbl.mask.l4_dst = HOS_L4_PORT_MASK;
    acl_tbl.mask.vlan_id = HOS_VLAN_ID_MASK;
    acl_tbl.mask.vlan_tag = HOS_VLAN_TAG_MASK;
    acl_tbl.min_entries = HOS_IP4_CLASSIFIER_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_IP4_CLASSIFIER_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "IPv4 ingress classifier table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_IP4_CLASSIFIER_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create IPv4 ingress classifier table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_ACL_ID;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "IPv4 ingress classifer table, Action Set ACL ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_ACL_ID);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_IP4_CLASSIFIER_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_ACL_ID);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set IPv4 ingress classifier table remap slots"
                      " for ACL ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create IPv6 Ingress Classifier table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_IP6_CLASSIFIER_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_IP6_CLASSIFIER_TBL_INST;
    acl_tbl.scenario = HOS_IP6_CLASSIFIER_TBL_SCENARIO;
    acl_tbl.condition = HOS_IP6_CLASSIFIER_TBL_CONDITION;
    acl_tbl.mask.protocol = HOS_IP_PROTO_MASK;
    acl_tbl.mask.sip.is_ipv6 = true;
    acl_tbl.mask.sip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.is_ipv6 = true;
    acl_tbl.mask.dip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.l4_src = HOS_L4_PORT_MASK;
    acl_tbl.mask.l4_dst = HOS_L4_PORT_MASK;
    acl_tbl.mask.vlan_id = HOS_VLAN_ID_MASK;
    acl_tbl.mask.vlan_tag = HOS_VLAN_TAG_MASK;
    acl_tbl.min_entries = HOS_IP6_CLASSIFIER_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_IP6_CLASSIFIER_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "IPv6 ingress classifier table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_IP6_CLASSIFIER_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create IPv4 ingress classifier table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_ACL_ID;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "IPv6 ingress classifer table, Action Set ACL ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_ACL_ID);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_IP6_CLASSIFIER_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_ACL_ID);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set IPv6 ingress classifier table remap slots"
                      " for ACL ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create supported ethertype table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_SUPPORTED_ETHERTYPE_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_SUPPORTED_ETHERTYPE_TBL_INST;
    acl_tbl.scenario = HOS_SUPPORTED_ETHERTYPE_TBL_SCENARIO;
    acl_tbl.condition = HOS_SUPPORTED_ETHERTYPE_TBL_CONDITION;
    acl_tbl.mask.etype = HOS_ETYPE_MASK;
    acl_tbl.em_action_entries = IES_ACL_EM_TBL_ACTION_ENTRIES_DEFAULT;
    acl_tbl.em_mode = IES_ACL_EM_TBL_MODE_SPLIT;
    acl_tbl.min_entries = HOS_SUPPORTED_ETHERTYPE_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_SUPPORTED_ETHERTYPE_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "Supported Ethertype table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_SUPPORTED_ETHERTYPE_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create supported ethertype table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create supported ip protocol table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_SUPPORTED_IPPROTO_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_SUPPORTED_IPPROTO_TBL_INST;
    acl_tbl.scenario = HOS_SUPPORTED_IPPROTO_TBL_SCENARIO;
    acl_tbl.condition = HOS_SUPPORTED_IPPROTO_TBL_CONDITION;
    acl_tbl.mask.protocol = HOS_IP_PROTO_MASK;
    acl_tbl.em_action_entries = IES_ACL_EM_TBL_ACTION_ENTRIES_DEFAULT;
    acl_tbl.em_mode = IES_ACL_EM_TBL_MODE_SPLIT;
    acl_tbl.min_entries = HOS_SUPPORTED_IPPROTO_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_SUPPORTED_IPPROTO_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "Supported ip protocol table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_SUPPORTED_IPPROTO_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create supported ip protocol table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create L2 ingress flow table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_L2_INGRESS_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_L2_INGRESS_TBL_INST;
    acl_tbl.scenario = HOS_L2_INGRESS_TBL_SCENARIO;
    acl_tbl.condition = HOS_L2_INGRESS_TBL_CONDITION;
    acl_tbl.mask.meta.sel = IES_ACL_MATCH_L2_DOMAIN;
    acl_tbl.mask.meta.l2_domain = HOS_L2_DOMAIN_MASK; 
    acl_tbl.mask.etype = HOS_ETYPE_MASK;
    acl_tbl.mask.dmac = HOS_MAC_ADDR_MASK;
    acl_tbl.min_entries = HOS_L2_INGRESS_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_L2_INGRESS_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "L2 ingress flow table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_L2_INGRESS_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create L2 ingress flow table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_METADATA;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "L2 ingress flow table, Action Set Meta data"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_L2_INGRESS);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_L2_INGRESS_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_L2_INGRESS);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set L2 ingress flow table remap slots"
                      " for L2 ingress\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create IPv4/IPv6 inner/outer Term table */

    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_IP_TERM_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_IP_TERM_TBL_INST;
    acl_tbl.scenario = HOS_IP_TERM_TBL_SCENARIO;
    acl_tbl.condition = HOS_IP_TERM_TBL_CONDITION;
    acl_tbl.mask.meta.sel = IES_ACL_MATCH_PKT_TYPE;
    acl_tbl.mask.meta.pkt_type = HOS_PKT_TYPE_MASK;
    acl_tbl.mask.dip.is_ipv6 = true;
    acl_tbl.mask.dip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.min_entries = HOS_IP_TERM_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_IP_TERM_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "IP term table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_IP_TERM_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create IP term table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_PKT_TYPE;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "IP term table, Action Set Pkt-Type"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_PKT_TYPE);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_IP_TERM_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_PKT_TYPE);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set IP term table remap slots"
                      " for Pkt-Type\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_METADATA;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "IP term table, Action Set Stats ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_IP_STATS);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_IP_TERM_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_IP_STATS);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set IP term table remap slots"
                      " for Stats ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create IPsec SPD IN table */
    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_SPD_IN_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_SPD_IN_TBL_INST;
    acl_tbl.scenario = HOS_SPD_IN_TBL_SCENARIO;
    acl_tbl.condition = HOS_SPD_IN_TBL_CONDITION;
    acl_tbl.mask.protocol = HOS_IP_PROTO_MASK;
    acl_tbl.mask.sip.is_ipv6 = true;
    acl_tbl.mask.sip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.is_ipv6 = true;
    acl_tbl.mask.dip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.l4_src = HOS_L4_PORT_MASK;
    acl_tbl.mask.l4_dst = HOS_L4_PORT_MASK;
    acl_tbl.min_entries = HOS_SPD_IN_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_SPD_IN_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "SPD IN table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_SPD_IN_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create SPD IN table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_SPD_ID;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "SPD IN table, Action Set SPD ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_SPD_ID);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_SPD_IN_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_SPD_ID);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set SPD IN table remap slots"
                      " for SPD ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create IPsec SPD OUT table */
    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_SPD_OUT_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_SPD_OUT_TBL_INST;
    acl_tbl.scenario = HOS_SPD_OUT_TBL_SCENARIO;
    acl_tbl.condition = HOS_SPD_OUT_TBL_CONDITION;
    acl_tbl.mask.protocol = HOS_IP_PROTO_MASK;
    acl_tbl.mask.sip.is_ipv6 = true;
    acl_tbl.mask.sip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.sip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.is_ipv6 = true;
    acl_tbl.mask.dip.addr[0] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[1] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[2] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.dip.addr[3] = HOS_IP4_ADDR_MASK;
    acl_tbl.mask.l4_src = HOS_L4_PORT_MASK;
    acl_tbl.mask.l4_dst = HOS_L4_PORT_MASK;
    acl_tbl.min_entries = HOS_SPD_OUT_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_SPD_OUT_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "SPD OUT table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_SPD_OUT_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create SPD OUT table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    remap_cfg.mode = IES_ACL_REMAP_MODE_ACTION;
    remap_cfg.action = IES_ACL_ACTION_SET_SPD_ID;
    ADK_HOS_LOG_EXT_API("ies_acl_set_remap_slots", "SPD OUT table, Action Set SPD ID"
                                                   " Remap slots 0x%x\n", HOS_REMAP_SLOT_SPD_ID);
    ies_ret = ies_acl_set_remap_slots(ADK_HLP_SWITCH_ID, HOS_SPD_OUT_TBL_IDX,
                                      &remap_cfg, HOS_REMAP_SLOT_SPD_ID);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to set SPD OUT table remap slots"
                      " for SPD ID\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create Plain Text Nexthop table */
    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_PLAINTEXT_HASH_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_PLAINTEXT_HASH_TBL_INST;
    acl_tbl.scenario = HOS_PLAINTEXT_HASH_TBL_SCENARIO;
    acl_tbl.condition = HOS_PLAINTEXT_HASH_TBL_CONDITION;
    acl_tbl.mask.etype = HOS_ETYPE_MASK;
    acl_tbl.min_entries = HOS_PLAINTEXT_HASH_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_PLAINTEXT_HASH_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "SPD OUT table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_PLAINTEXT_HASH_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create PLAIN TEXT HASH table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Create ESP Nexthop table */
    ADK_MEMSET(&acl_tbl, 0, sizeof(acl_tbl));

    acl_tbl.location = HOS_ESP_HASH_TBL_LOCATION;
    acl_tbl.acl_instance = HOS_ESP_HASH_TBL_INST;
    acl_tbl.scenario = HOS_ESP_HASH_TBL_SCENARIO;
    acl_tbl.condition = HOS_ESP_HASH_TBL_CONDITION;
    acl_tbl.mask.etype = HOS_ETYPE_MASK;
    acl_tbl.mask.frag_type = (IES_ACL_NOT_FRAG |
                              IES_ACL_FIRST_FRAG |
                              IES_ACL_NON_FIRST_FRAG);
    acl_tbl.min_entries = HOS_ESP_HASH_TBL_MIN_RULES;
    acl_tbl.max_entries = HOS_ESP_HASH_TBL_MAX_RULES;

    ADK_HOS_LOG_EXT_API("ies_acl_create_table", "ESP HASH table\n");
    ies_ret = ies_acl_create_table(ADK_HLP_SWITCH_ID,
                                   HOS_ESP_HASH_TBL_IDX, &acl_tbl);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to create ESP TEXT HASH table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add the default rule for all switched packets in L2 termination-interface table */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = IES_ACL_ACTION_SET_PKT_TYPE | IES_ACL_ACTION_SET_HASH_NEXTHOP | IES_ACL_ACTION_SET_INNER_ID; 
    acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L2_SWITCH;
    acl_rule.act_val.pkt_type_mask = HOS_PKT_TYPE_L2_MASK;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;
    acl_rule.act_val.inner_id = ADK_HOS_DEFAULT_L2_INTERFACE;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default rule in L2 term-intf table\n");
    ies_ret = ies_acl_add_rule(
            ADK_HLP_SWITCH_ID, HOS_L2_TERM_INTF_TBL_IDX, &acl_rule,
            &adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_TERM_INTF_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                "IES Error %s, Failed to add default entry in L2 Term-Intf"
                " table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule in Ingress IP4 classifier table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_IP4_CLASSIFIER_TBL_ACTION;
    acl_rule.act_val.acl_id = ADK_HOS_DEFAULT_IP_CLASSIFIER;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule",
                        "Default rule in IPv4 classifier table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_IP4_CLASSIFIER_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_CLASSIFIER_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in IP4"
                      " classifier table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule in Ingress IP6 classifier table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_IP6_CLASSIFIER_TBL_ACTION;
    acl_rule.act_val.acl_id = ADK_HOS_DEFAULT_IP_CLASSIFIER;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule",
                        "Default rule in IPv6 classifier table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_IP6_CLASSIFIER_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP6_CLASSIFIER_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in IP6"
                      " classifier table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule in supported ethertype table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_SUPPORTED_ETHERTYPE_TBL_ACTION;
    acl_rule.act_val.flag_mask = HOS_FLAG_ETHERTYPE_MASK;
    acl_rule.act_val.flag = HOS_FLAG_ETHERTYPE_DENY;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule",
                        "Default rule in supported ethertype table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_SUPPORTED_ETHERTYPE_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in supported"
                      " ethertype table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule in supported ip proto table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_SUPPORTED_IPPROTO_TBL_ACTION;
    acl_rule.act_val.flag_mask = HOS_FLAG_IPPROTO_MASK;
    acl_rule.act_val.flag = HOS_FLAG_IPPROTO_DENY;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule",
                        "Default rule in supported ip protocol table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_SUPPORTED_IPPROTO_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in supported"
                      " ip protocol table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add the default rule in L2 ingress flow table */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_L2_INGRESS_TBL_ACTION; 
    acl_rule.act_val.meta.byte_select = HOS_L2_INGRESS_BYTE_SELECT;
    acl_rule.act_val.meta.mask[HOS_L2_INGRESS_BYTE] =
        HOS_INGRESS_FLOW_MASK & 0xFF;
    acl_rule.act_val.meta.val[HOS_L2_INGRESS_BYTE] =
        ADK_HOS_DEFAULT_L2_INGRESS_FLOW & 0xFF;
    acl_rule.act_val.meta.mask[HOS_L2_INGRESS_BYTE+1] =
        (HOS_INGRESS_FLOW_MASK >> 8) & 0x3;
    acl_rule.act_val.meta.val[HOS_L2_INGRESS_BYTE+1] =
        (ADK_HOS_DEFAULT_L2_INGRESS_FLOW >> 8) & 0x3;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default rule in L2 ingress flow table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_L2_INGRESS_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_INGRESS_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in L2 ingress flow"
                      " table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule in IP term table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.condition = IES_ACL_MATCH_DIP;
    acl_rule.cond.val.dip.is_ipv6 = false;
    acl_rule.cond.mask.dip.addr[0] = 0x0;
    acl_rule.cond.val.dip.addr[0] = 0x0;
    acl_rule.action = HOS_IP_TERM_TBL_ACTION;
    acl_rule.act_val.pkt_type_mask = HOS_PKT_TYPE_L3_MASK; 
    acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L3_ROUTE;
    acl_rule.act_val.meta.byte_select = HOS_IP_STATS_BYTE_SELECT;
    acl_rule.act_val.meta.mask[HOS_IP_STATS_BYTE] =
        HOS_IP_STATS_BYTE_MASK << 2;
    acl_rule.act_val.meta.val[HOS_IP_STATS_BYTE] =
        (ADK_HOS_DEFAULT_IP_STATS_ID & HOS_IP_STATS_BYTE_MASK) << 2;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule",
                        "Default IPv4 rule in IP term table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_IP_TERM_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_TERM_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default IPv4 rule in IP"
                      " term table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default IPv4 rule for SPD IN table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.condition = IES_ACL_MATCH_DIP;
    acl_rule.cond.val.dip.is_ipv6 = false;
    acl_rule.cond.mask.dip.addr[0] = 0x0;
    acl_rule.cond.val.dip.addr[0] = 0x0;
    acl_rule.action = HOS_SPD_IN_TBL_ACTION; 
    acl_rule.act_val.spd_id = ADK_HOS_DEFAULT_SPD_RULE_ID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default rule in SPD IN table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_IN_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP4_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in SPD"
                      " IN table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default IPv6 rule for SPD IN table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.condition = IES_ACL_MATCH_DIP;
    acl_rule.cond.val.dip.is_ipv6 = true;
    acl_rule.cond.mask.dip.addr[0] = 0x0;
    acl_rule.cond.mask.dip.addr[1] = 0x0;
    acl_rule.cond.mask.dip.addr[1] = 0x0;
    acl_rule.cond.mask.dip.addr[1] = 0x0;
    acl_rule.cond.val.dip.addr[0] = 0x0;
    acl_rule.cond.val.dip.addr[1] = 0x0;
    acl_rule.cond.val.dip.addr[2] = 0x0;
    acl_rule.cond.val.dip.addr[3] = 0x0;
    acl_rule.action = HOS_SPD_IN_TBL_ACTION; 
    acl_rule.act_val.spd_id = ADK_HOS_DEFAULT_SPD_RULE_ID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default rule in SPD IN table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_IN_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP6_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in SPD"
                      " IN table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default rule for SPD OUT table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.action = HOS_SPD_OUT_TBL_ACTION;
    acl_rule.act_val.spd_id = ADK_HOS_DEFAULT_SPD_RULE_ID;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default rule in SPD OUT table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_OUT_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_OUT_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default entry in SPD"
                      " OUT table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default IPv4 rule for PLAINTEXT NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.condition = HOS_PLAINTEXT_HASH_TBL_CONDITION; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP4;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_PLAINTEXT_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default IPv4 rule in PLAINTEXT HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_PLAINTEXT_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_PLAINTEXT_HASH_DEF_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default IPv4 entry in PLAINTEXT"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default IPv6 rule for PLAINTEXT NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 1;
    acl_rule.condition = HOS_PLAINTEXT_HASH_TBL_CONDITION; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP6;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_PLAINTEXT_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default IPv6 rule in PLAINTEXT HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_PLAINTEXT_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_PLAINTEXT_HASH_DEF_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add default IPv6 entry in PLAINTEXT"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default frag IPv4 rule for ESP NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 0;
    acl_rule.condition = IES_ACL_MATCH_ETYPE; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP4;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_ESP_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default frag IPv4 rule in ESP HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_FRAG_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add frag IPv4 default entry in ESP"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default frag IPv6 rule for ESP NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 1;
    acl_rule.condition = IES_ACL_MATCH_ETYPE; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP6;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_ESP_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default frag IPv6 rule in ESP HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_FRAG_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add frag IPv6 default entry in ESP"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default unfrag IPv4 rule for ESP NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 2;
    acl_rule.condition = HOS_ESP_HASH_TBL_CONDITION;
    acl_rule.cond.val.frag_type = IES_ACL_NOT_FRAG;
    acl_rule.cond.mask.frag_type = IES_ACL_NOT_FRAG; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP4;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_ESP_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default unfrag IPv4 rule in ESP HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_UNFRAG_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add unfrag IPv4 default entry in ESP"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Add default unfrag IPv6 rule for ESP NEXTHOP table. */
    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    acl_rule.precedence = 3;
    acl_rule.condition = HOS_ESP_HASH_TBL_CONDITION;
    acl_rule.cond.val.frag_type = IES_ACL_NOT_FRAG;
    acl_rule.cond.mask.frag_type = IES_ACL_NOT_FRAG; 
    acl_rule.cond.val.etype = HOS_ETYPE_IP6;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    acl_rule.action = HOS_ESP_HASH_TBL_ACTION;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "Default unfrag IPv6 rule in ESP HASH table\n");
    ies_ret = ies_acl_add_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX, &acl_rule,
        &adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_UNFRAG_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add unfrag IPv6 default entry in ESP"
                      " HASH table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_init_tables_exit;
    }

    /* Initialize the logical size of each table to ADK_HOS_DEFAULT_TABLE_SIZE*/
    for(uint32_t iTable = 0; iTable < ADK_HOS_MAX_OBJ; iTable++)
    {
        adk_hos_pvt_g_p->tbl_usage[iTable].size = ADK_HOS_DEFAULT_TABLE_SIZE;
        adk_hos_pvt_g_p->tbl_usage[iTable].occupancy = 0;
    } 

_adk_hos_init_tables_exit:

    return retValue;
}

/* This function deletes default rules from all TCAM based tables and then
 * destroys all tables */

adk_status_t
adk_hos_shutdown_tables(
    void)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in L2 term-intf table\n");
    ies_ret = ies_acl_delete_rule(
            ADK_HLP_SWITCH_ID, HOS_L2_TERM_INTF_TBL_IDX,
            adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_TERM_INTF_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                "IES Error %s, Failed to delete default entry from L2 TERM"
                " INTF table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "L2 term-intf table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
            HOS_L2_TERM_INTF_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                "IES Error %s, Failed to destroy L2 Term-Intf table\n",
                HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in IPv4 classifier table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_IP4_CLASSIFIER_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_CLASSIFIER_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from IPv4"
                      " classifier table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "IPv4 classifier table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_IP4_CLASSIFIER_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy IPv4 classifier table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in IPv6 classifier table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_IP6_CLASSIFIER_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP6_CLASSIFIER_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from IPv6"
                      " classifier table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "IPv6 classifier table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_IP6_CLASSIFIER_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy IPv6 classifier table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in supported ethertype table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_SUPPORTED_ETHERTYPE_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from supported"
                      " ethertype table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "Supported ethertype table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_SUPPORTED_ETHERTYPE_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy supported ethertype table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in supported ip protocol table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_SUPPORTED_IPPROTO_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from supported"
                      " ip protocol table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "Supported ip protocol table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_SUPPORTED_IPPROTO_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy supported ip protocol table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in L2 ingress table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_L2_INGRESS_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_INGRESS_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from L2"
                      " INGRESS table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "L2 ingress table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_L2_INGRESS_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy L2 Ingress table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule",
                        "Default rule in IP term table for IPv4\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_IP_TERM_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_TERM_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv4 entry from"
                      " IP term table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule",
                        "Default rule in IP term table for IPv6\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_IP_TERM_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP6_TERM_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv6 entry from"
                      " IP term table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "IP term table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_IP_TERM_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy IPv4 Term table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in SPD IN table for IPv4\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_IN_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP4_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv4 entry from SPD"
                      " IN table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in SPD IN table for IPv6\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_IN_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP6_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv6 entry from SPD"
                      " IN table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "SPD IN table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_SPD_IN_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy SPD IN table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default rule in SPD OUT table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_SPD_OUT_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_OUT_DEF_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default entry from SPD"
                      " OUT table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "SPD OUT table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_SPD_OUT_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy SPD OUT table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default IPv4 rule in Plain text nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_PLAINTEXT_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_PLAINTEXT_HASH_DEF_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv4 entry from Plain"
                      " text nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default IPv6 rule in Plain text nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_PLAINTEXT_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_PLAINTEXT_HASH_DEF_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default IPv6 entry from Plain"
                      " text nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "Plain text nexthop table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_PLAINTEXT_HASH_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy Plain text nexthop table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default frag IPv4 rule in ESP nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_FRAG_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default frag IPv4 entry from ESP"
                      " nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default frag IPv6 rule in ESP nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_FRAG_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default frag IPv6 entry from ESP"
                      " nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default unfrag IPv4 rule in ESP nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_UNFRAG_IP4_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default unfrag IPv4 entry from ESP"
                      " nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_delete_rule", "Default unfrag IPv6 rule in ESP nexthop table\n");
    ies_ret = ies_acl_delete_rule(
        ADK_HLP_SWITCH_ID, HOS_ESP_HASH_TBL_IDX,
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_ESP_HASH_DEF_UNFRAG_IP6_RULE_IDX]);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to delete default unfrag IPv6 entry from ESP"
                      " nexthop table\n", HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_destroy_table", "ESP nexthop table\n");
    ies_ret = ies_acl_destroy_table(ADK_HLP_SWITCH_ID,
                                    HOS_ESP_HASH_TBL_IDX);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to destroy ESP nexthop table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_shutdown_tables_exit;
    }

_adk_hos_shutdown_tables_exit:

    return retValue;
}

static void 
hos_rule_add_l2_domain
(
    uint16_t l2_domain,
    struct ies_acl_rule_info* acl_rule
)
{
    if (l2_domain == ADK_HOS_DEFAULT_L2_DOMAIN)
        return;

    acl_rule->precedence = HOS_L2_DOMAIN_WIDTH;
    acl_rule->condition = IES_ACL_MATCH_METADATA;
    acl_rule->cond.mask.meta.sel = IES_ACL_MATCH_L2_DOMAIN;
    acl_rule->cond.mask.meta.l2_domain = HOS_L2_DOMAIN_MASK;
    acl_rule->cond.val.meta.l2_domain = l2_domain;
    return;
}

static void
hos_rule_add_dmac
(
    uint64_t dmac_mask,
    uint64_t dmac,
    struct ies_acl_rule_info* acl_rule
)
{
    
    if (!dmac_mask)
        return;

    /* Precedence is made highest if MAC addr is specified */
    uint64_t dmac_local = dmac_mask;
    acl_rule->condition |= IES_ACL_MATCH_DMAC;
    acl_rule->precedence += HOS_MAC_ADDR_WIDTH;
    while (dmac_local) 
    { 
        acl_rule->precedence += dmac_local & 1; 
        dmac_local >>= 1; 
    }
    acl_rule->cond.val.dmac =
        (dmac & dmac_mask);
    acl_rule->cond.mask.dmac = dmac_mask;
    return;
}

static adk_status_t
hos_rule_add_dmac_map
(
    uint64_t dmac,
    struct ies_acl_rule_info* acl_rule
)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint8_t cmac = 0;    
    if (!dmac)
        return retValue;

    retValue = adk_hos_mac_mapper_get(dmac, &cmac, NULL);
    if(retValue != ADK_SUCCESS)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                        ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                        "Mac mapper Failed to get mapped value for mac addr %ld\n",
                        dmac);
        return ADK_E_HW_PROGRAMMING_ERROR;
    }

    /* Precedence is made highest if MAC addr is specified */
    acl_rule->condition |= IES_ACL_MATCH_DMAC_MAP;
    acl_rule->precedence += HOS_MAC_MAP_WIDTH;
    acl_rule->cond.val.map_dmac = cmac;
    acl_rule->cond.mask.map_dmac = HOS_MAC_MAP_MASK;

    return retValue;
}

adk_status_t
adk_hos_l2_term_intf_tbl_rule_add(
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, dglort;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));

    ADK_HOS_LOG_EXT_API("ies_vsi_map_vsi_qid_to_port", "VSI %d, VSI-Q %d\n",
                        rule_attr->vsi_id, rule_attr->vsi_q);
    ies_ret = ies_vsi_map_vsi_qid_to_port(ADK_HLP_SWITCH_ID, rule_attr->vsi_id,
                                          rule_attr->vsi_q, (int *) &dglort);
    if (ies_ret != IES_OK)
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to get mapping for VSI/Q (%d/%d)\n",
                      HOS_IES_ERROR_NAME(ies_ret), rule_attr->vsi_id,
                      rule_attr->vsi_q);
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        goto _adk_hos_l2_term_intf_tbl_rule_add_exit;
    }

    hos_rule_add_l2_domain(rule_attr->l2_domain, &acl_rule);
    if(ADK_SUCCESS != (retValue = hos_rule_add_dmac_map(rule_attr->dmac, &acl_rule)))
        return retValue;

    if (rule_attr->vlan_id != 0) {
        if(rule_attr->vlan_id != ADK_HOS_ANY_VLAN)
        {
            /* Precedence is higher if VLAN is specified */
            acl_rule.precedence += HOS_VLAN_ID_WIDTH;
            acl_rule.condition |= IES_ACL_MATCH_VLAN;
            acl_rule.cond.val.vlan_id = rule_attr->vlan_id;
            acl_rule.cond.mask.vlan_id = HOS_VLAN_ID_MASK;
        }
    }

    if(!rule_attr->is_l2_m_b)
    {
        acl_rule.action = HOS_L2_TERM_INTF_TBL_ACTION;
        if (rule_attr->is_l2_term)
        {
            acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L2_TERM;
        }
        else
        {
            acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L2_SWITCH;
        }
        acl_rule.act_val.pkt_type_mask = HOS_PKT_TYPE_L2_MASK;
        acl_rule.act_val.logical_port = dglort;
    }
    else
    {
        acl_rule.action = HOS_L2_TERM_INTF_M_B_ACTION;
    }
    acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_L2_HASH_IDX];
    acl_rule.act_val.hash.metadata = true;
    acl_rule.act_val.inner_id = rule_attr->l2_interface;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in l2-term-intf table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, HOS_L2_TERM_INTF_TBL_IDX,
                               &acl_rule, (int *) rule_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in L2 Term-intf table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_l2_term_intf_tbl_rule_add_exit;
    }

_adk_hos_l2_term_intf_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
adk_hos_l2_ingress_tbl_rule_add(
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));

    hos_rule_add_l2_domain(rule_attr->l2_domain, &acl_rule);
    if (rule_attr->ether_type != 0) {
        /* Precedence is higher if ether-type is specified */
        acl_rule.precedence += HOS_ETYPE_WIDTH;
        acl_rule.condition |= IES_ACL_MATCH_ETYPE;
        acl_rule.cond.val.etype = rule_attr->ether_type;
        acl_rule.cond.mask.etype = HOS_ETYPE_MASK;
    }
    hos_rule_add_dmac(rule_attr->dmac_mask, rule_attr->dmac,
                        &acl_rule);

    acl_rule.action = HOS_L2_INGRESS_TBL_ACTION;
    acl_rule.act_val.meta.byte_select = HOS_L2_INGRESS_BYTE_SELECT;
    acl_rule.act_val.meta.mask[HOS_L2_INGRESS_BYTE] =
        HOS_INGRESS_FLOW_MASK & 0xFF;
    acl_rule.act_val.meta.val[HOS_L2_INGRESS_BYTE] =
        rule_attr->l2_ingress & 0xFF;
    acl_rule.act_val.meta.mask[HOS_L2_INGRESS_BYTE+1] =
        (ADK_HOS_DEFAULT_L2_INGRESS_FLOW >> 8) & 0x03;
    acl_rule.act_val.meta.val[HOS_L2_INGRESS_BYTE+1] =
        (rule_attr->l2_ingress >> 8) & 0x03;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    
    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in l2 ingress flow table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, HOS_L2_INGRESS_TBL_IDX,
                               &acl_rule, (int *) rule_id);
    if(ies_ret == IES_ERR_NO_RESOURCE)
    {
        retValue = ADK_E_HOS_TABLE_FULL;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "L2 Ingress HOS table is full\n");
        goto _adk_hos_l2_ingress_tbl_rule_add_exit;
    }
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in L2 ingress flow table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_l2_ingress_tbl_rule_add_exit;
    }

_adk_hos_l2_ingress_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
adk_hos_supported_ethertype_tbl_rule_add(
    adk_hos_ethertype_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));

    if (rule_attr->ether_type == 0)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Invalid ether type, Failed to add rule in supported ethertype table\n");
        goto _adk_hos_supported_ethertype_tbl_rule_add_exit;
    }
    
    /* Precedence is higher if ether-type is specified */
    acl_rule.precedence += HOS_ETYPE_WIDTH;
    acl_rule.condition |= IES_ACL_MATCH_ETYPE;
    acl_rule.cond.val.etype = rule_attr->ether_type;
    acl_rule.cond.mask.etype = HOS_ETYPE_MASK;

    acl_rule.action = HOS_SUPPORTED_ETHERTYPE_TBL_ACTION;
    acl_rule.act_val.flag_mask = HOS_FLAG_ETHERTYPE_MASK;
    acl_rule.act_val.flag = HOS_FLAG_ETHERTYPE_ALLOW;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    
    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in supported ethertype table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, HOS_SUPPORTED_ETHERTYPE_TBL_IDX,
                               &acl_rule, (int *) rule_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in supported ethertype table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_supported_ethertype_tbl_rule_add_exit;
    }

_adk_hos_supported_ethertype_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
adk_hos_supported_ipproto_tbl_rule_add(
    adk_hos_ipproto_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));

    if (rule_attr->proto == 0)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Invalid proto type, Failed to add rule in supported ip protocol table\n");
        goto _adk_hos_supported_ipproto_tbl_rule_add_exit;
    }
    
    /* Precedence is higher if ip protocol is specified */
    acl_rule.precedence += HOS_IP_PROTO_WIDTH;
    acl_rule.condition |= IES_ACL_MATCH_PROT;
    acl_rule.cond.val.protocol = rule_attr->proto;
    acl_rule.cond.mask.protocol = HOS_IP_PROTO_MASK;

    acl_rule.action = HOS_SUPPORTED_IPPROTO_TBL_ACTION;
    acl_rule.act_val.flag_mask = HOS_FLAG_IPPROTO_MASK;
    acl_rule.act_val.flag = HOS_FLAG_IPPROTO_ALLOW;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    
    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in supported ip protocol table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, HOS_SUPPORTED_IPPROTO_TBL_IDX,
                               &acl_rule, (int *) rule_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in supported ip protocol table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_supported_ipproto_tbl_rule_add_exit;
    }
_adk_hos_supported_ipproto_tbl_rule_add_exit:

    return retValue;
}

static uint32_t count_set_bits(uint32_t n)
{
    uint32_t count = 0; 
    while (n) { 
        count += n & 1; 
        n >>= 1; 
    }
    return count;
}

adk_status_t
adk_hos_ip_term_tbl_rule_add(
    adk_hos_ip_term_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;
    uint32_t tbl_idx;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));

    /* Do not allow default rule to be replaced */
    if (rule_attr->precedence == 0)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Rule precedence must be > 0\n");
        goto _adk_hos_ip_term_tbl_rule_add_exit;
    }

    switch (rule_attr->pkt_type)
    {
    case ADK_HOS_L2_SWITCHING:
        acl_rule.condition |= IES_ACL_MATCH_METADATA;
        acl_rule.cond.mask.meta.sel = IES_ACL_MATCH_PKT_TYPE;
        acl_rule.cond.val.meta.pkt_type = HOS_PKT_TYPE_L2_SWITCH;
        acl_rule.cond.mask.meta.pkt_type = HOS_PKT_TYPE_L2_MASK;
        break;

    case ADK_HOS_L2_TERMINATING:
        acl_rule.condition |= IES_ACL_MATCH_METADATA;
        acl_rule.cond.mask.meta.sel = IES_ACL_MATCH_PKT_TYPE;
        acl_rule.cond.val.meta.pkt_type = HOS_PKT_TYPE_L2_TERM;
        acl_rule.cond.mask.meta.pkt_type = HOS_PKT_TYPE_L2_MASK;
        break;
    default:
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Invalid Packet Type %x specified\n",
                      rule_attr->pkt_type);
        retValue = ADK_E_INVALID_PARAMETER;
        goto _adk_hos_ip_term_tbl_rule_add_exit;
        break;
    }

    acl_rule.cond.mask.dip.is_ipv6 = rule_attr->is_ipv6;
    /* IP address is mandatory parametere */
    if (rule_attr->is_ipv6)
    {
        acl_rule.precedence = rule_attr->precedence;        
        if(rule_attr->precedence == -1)
        {
            acl_rule.precedence += count_set_bits(rule_attr->ip_addr_mask[0]);
            acl_rule.precedence += count_set_bits(rule_attr->ip_addr_mask[0]);
            acl_rule.precedence += count_set_bits(rule_attr->ip_addr_mask[0]);
            acl_rule.precedence += count_set_bits(rule_attr->ip_addr_mask[0]);
        }
            
        acl_rule.condition |= IES_ACL_MATCH_DIP; 
        acl_rule.cond.val.dip.is_ipv6 = true;
        acl_rule.cond.mask.dip.addr[0] = rule_attr->ip_addr_mask[0];
        acl_rule.cond.mask.dip.addr[1] = rule_attr->ip_addr_mask[1];
        acl_rule.cond.mask.dip.addr[2] = rule_attr->ip_addr_mask[2];
        acl_rule.cond.mask.dip.addr[3] = rule_attr->ip_addr_mask[3];
        acl_rule.cond.val.dip.addr[0] = rule_attr->ip_addr[0];
        acl_rule.cond.val.dip.addr[1] = rule_attr->ip_addr[1];
        acl_rule.cond.val.dip.addr[2] = rule_attr->ip_addr[2];
        acl_rule.cond.val.dip.addr[3] = rule_attr->ip_addr[3];
    }
    else
    {
        acl_rule.precedence = rule_attr->precedence;        
        if(rule_attr->precedence == -1)
            acl_rule.precedence += count_set_bits(rule_attr->ip_addr_mask[0]);

        acl_rule.condition |= IES_ACL_MATCH_DIP; 
        acl_rule.cond.val.dip.is_ipv6 = false;
        acl_rule.cond.mask.dip.addr[0] = rule_attr->ip_addr_mask[0];
        acl_rule.cond.val.dip.addr[0] = rule_attr->ip_addr[0];
    }

    tbl_idx = HOS_IP_TERM_TBL_IDX;
    acl_rule.action = HOS_IP_TERM_TBL_ACTION;

    acl_rule.act_val.pkt_type_mask = HOS_PKT_TYPE_L3_MASK | HOS_PKT_TYPE_L3_MCAST_MASK | HOS_PKT_TYPE_L3_BCAST_MASK;

    acl_rule.action |= IES_ACL_ACTION_SET_L3T;
    if (rule_attr->is_ip_term)
    {
        acl_rule.act_val.l3t = true;
        acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L3_TERM;
    }
    else
    {
        acl_rule.act_val.l3t = false;
        acl_rule.act_val.pkt_type = HOS_PKT_TYPE_L3_ROUTE;
    }

    if(rule_attr->is_mcast)
        acl_rule.act_val.pkt_type |= HOS_PKT_TYPE_L3_MCAST; 

    if(rule_attr->is_bcast)
        acl_rule.act_val.pkt_type |= HOS_PKT_TYPE_L3_BCAST; 

    if(rule_attr->is_ipv6)
        acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_HASH_IDX];
    else
        acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_HASH_IDX];

    acl_rule.act_val.meta.byte_select = HOS_IP_STATS_BYTE_SELECT;
    acl_rule.act_val.meta.mask[HOS_IP_STATS_BYTE] =
        HOS_IP_STATS_BYTE_MASK << 2;
    acl_rule.act_val.meta.val[HOS_IP_STATS_BYTE] =
        (rule_attr->ip_addr_stats_id & HOS_IP_STATS_BYTE_MASK) << 2;

    acl_rule.act_val.hash.metadata = true;
    acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    
    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in IP term table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, tbl_idx,
                               &acl_rule, (int *) rule_id);
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in IP term table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_ip_term_tbl_rule_add_exit;
    }

_adk_hos_ip_term_tbl_rule_add_exit:

    return retValue;
}

static void
hos_rule_add_L4_info(
    uint8_t proto,
    uint16_t l4_info2,
    uint16_t l4_info2_mask,
    uint16_t l4_info1,
    uint16_t l4_info1_mask,
    struct ies_acl_rule_info* acl_rule
)
{
    if ((proto == HOS_IP_PROTO_UDP) ||
        (proto == HOS_IP_PROTO_TCP) ||
        (proto == HOS_IP_PROTO_SCTP))
    {
        if(l4_info2_mask)
        {
            acl_rule->condition |= IES_ACL_MATCH_L4_DST_PORT;
            acl_rule->cond.val.l4_dst = l4_info2;
            acl_rule->cond.mask.l4_dst = l4_info2_mask;
        }
        if(l4_info1_mask)
        {
            acl_rule->condition |= IES_ACL_MATCH_L4_SRC_PORT;
            acl_rule->cond.val.l4_src = l4_info1;
            acl_rule->cond.mask.l4_src = l4_info1_mask;
        }
    }
    else if ((proto == HOS_IP_PROTO_ICMP) ||
              (proto == HOS_IP_PROTO_ICMPV6))
    {
        acl_rule->condition |= IES_ACL_MATCH_L4_DST_PORT;
        acl_rule->condition |= IES_ACL_MATCH_L4_SRC_PORT;
        acl_rule->cond.val.l4_src =
            ((l4_info1 << HOS_BYTE_WIDTH) |
             (l4_info2 & HOS_BYTE_MASK));
        acl_rule->cond.mask.l4_src =
            ((l4_info1_mask << HOS_BYTE_WIDTH) |
             (l4_info2_mask & HOS_BYTE_MASK));
    }
    else
    {
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Classifier HOS rule add attempting unsupported IP protocol\n");
    }

    return;
}

static void
hos_rule_add_proto(
    uint8_t proto,
    uint8_t proto_mask,
    struct ies_acl_rule_info* acl_rule

)
{
    if(!proto_mask)
        return;
    acl_rule->condition |= IES_ACL_MATCH_PROT;
    acl_rule->cond.val.protocol = proto;
    acl_rule->cond.mask.protocol = proto_mask;

    return;
}

adk_status_t
adk_hos_ip_classifier_tbl_rule_add(
    adk_hos_ip_classifier_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;
    uint32_t ip6_mask[4];
    adk_hos_object_t tbl_obj;

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    ADK_MEMSET(ip6_mask, 0, sizeof(ip6_mask));

    tbl_obj = rule_attr->is_ipv6 ? ADK_HOS_IP6_CLASSIFIER_TBL:
                            ADK_HOS_IP4_CLASSIFIER_TBL;

    /* Do not allow default rule to be replaced */
    if (rule_attr->precedence == 0)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Rule precedence must be > 0\n");
        goto _adk_hos_ip_classifier_tbl_rule_add_exit;
    }

    acl_rule.precedence = rule_attr->precedence;
    if(rule_attr->useVlan)
    {
        if (rule_attr->vlan_id != ADK_HOS_ANY_VLAN)
        {
            acl_rule.condition |= IES_ACL_MATCH_VLAN;
            acl_rule.cond.val.vlan_id = rule_attr->vlan_id;
            acl_rule.cond.mask.vlan_id = HOS_VLAN_ID_MASK;
        }
        else
        {
            acl_rule.condition |= IES_ACL_MATCH_VLAN_TAG_TYPE;
            acl_rule.cond.val.vlan_tag = IES_ACL_UNTAG;
            acl_rule.cond.mask.vlan_tag = HOS_VLAN_TAG_MASK;
        }
    }

    hos_rule_add_proto(rule_attr->proto, 
                        rule_attr->proto_mask, &acl_rule);
    if (!rule_attr->is_ipv6)
    {
        tbl_idx = HOS_IP4_CLASSIFIER_TBL_IDX; 
        if(0 != rule_attr->dip_addr_mask[0])
        {
            acl_rule.condition |= IES_ACL_MATCH_DIP;
            acl_rule.cond.mask.dip.is_ipv6 = false;
            acl_rule.cond.val.dip.is_ipv6 = false;
            acl_rule.cond.mask.dip.addr[0] = rule_attr->dip_addr_mask[0];
            acl_rule.cond.val.dip.addr[0] = rule_attr->dip_addr[0];
        }
        if(0 != rule_attr->sip_addr_mask[0])
        {
            acl_rule.condition |= IES_ACL_MATCH_SIP;
            acl_rule.cond.mask.sip.is_ipv6 = false;
            acl_rule.cond.val.sip.is_ipv6 = false;
            acl_rule.cond.mask.sip.addr[0] = rule_attr->sip_addr_mask[0];
            acl_rule.cond.val.sip.addr[0] = rule_attr->sip_addr[0];
        }
    }
    else
    {
        tbl_idx = HOS_IP6_CLASSIFIER_TBL_IDX; 
        if(0 != memcmp(ip6_mask, rule_attr->dip_addr_mask, sizeof(ip6_mask)))
        {
            acl_rule.condition |= IES_ACL_MATCH_DIP;
            acl_rule.cond.mask.dip.is_ipv6 = true;
            acl_rule.cond.val.dip.is_ipv6 = true;
            acl_rule.cond.mask.dip.addr[0] = rule_attr->dip_addr_mask[0];
            acl_rule.cond.mask.dip.addr[1] = rule_attr->dip_addr_mask[1];
            acl_rule.cond.mask.dip.addr[2] = rule_attr->dip_addr_mask[2];
            acl_rule.cond.mask.dip.addr[3] = rule_attr->dip_addr_mask[3];
            acl_rule.cond.val.dip.addr[0] = rule_attr->dip_addr[0];
            acl_rule.cond.val.dip.addr[1] = rule_attr->dip_addr[1];
            acl_rule.cond.val.dip.addr[2] = rule_attr->dip_addr[2];
            acl_rule.cond.val.dip.addr[3] = rule_attr->dip_addr[3];
        }

        if(0 != memcmp(ip6_mask, rule_attr->sip_addr_mask, sizeof(ip6_mask)))
        {
            acl_rule.condition |= IES_ACL_MATCH_SIP;
            acl_rule.cond.mask.sip.is_ipv6 = true;
            acl_rule.cond.val.sip.is_ipv6 = true;
            acl_rule.cond.mask.sip.addr[0] = rule_attr->sip_addr_mask[0];
            acl_rule.cond.mask.sip.addr[1] = rule_attr->sip_addr_mask[1];
            acl_rule.cond.mask.sip.addr[2] = rule_attr->sip_addr_mask[2];
            acl_rule.cond.mask.sip.addr[3] = rule_attr->sip_addr_mask[3];
            acl_rule.cond.val.sip.addr[0] = rule_attr->sip_addr[0];
            acl_rule.cond.val.sip.addr[1] = rule_attr->sip_addr[1];
            acl_rule.cond.val.sip.addr[2] = rule_attr->sip_addr[2];
            acl_rule.cond.val.sip.addr[3] = rule_attr->sip_addr[3];
        }
    }
    if(rule_attr->proto_mask)
    {
        hos_rule_add_L4_info(rule_attr->proto, rule_attr->l4_info2,
            rule_attr->l4_info2_mask, rule_attr->l4_info1, rule_attr->l4_info1_mask,
            &acl_rule);
    }
    acl_rule.action = rule_attr->is_ipv6 ? HOS_IP6_CLASSIFIER_TBL_ACTION
                        : HOS_IP4_CLASSIFIER_TBL_ACTION;
    acl_rule.act_val.acl_id = rule_attr->rule_id;
    if (rule_attr->is_disabled)
    {
        acl_rule.state = IES_ACL_RULE_ENTRY_STATE_INVALID;
    }
    else
    {
        acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    }

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in ingress classifier table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, tbl_idx,
                               &acl_rule, (int *) rule_id);
    if(ies_ret == IES_ERR_NO_RESOURCE)
    {
        retValue = ADK_E_HOS_TABLE_FULL;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IP classifier HOS table is full\n");
        goto _adk_hos_ip_classifier_tbl_rule_add_exit;
    }
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in ingress classifier table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_ip_classifier_tbl_rule_add_exit;
    }

    adk_hos_table_occupancy_incr(tbl_obj);

_adk_hos_ip_classifier_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
adk_hos_spd_tbl_rule_add(
    adk_hos_spd_tbl_rule_attr_t *rule_attr,
    bool is_spd_out,
    adk_hos_rule_id_t *rule_id)
{
    adk_status_t retValue = ADK_SUCCESS;

    uint32_t ies_ret, tbl_idx;
    char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
    struct ies_acl_rule_info acl_rule;
    uint32_t ip6_mask[4];
    adk_hos_object_t tbl_obj;

    tbl_idx = is_spd_out ? HOS_SPD_OUT_TBL_IDX :
                            HOS_SPD_IN_TBL_IDX;

    tbl_obj = is_spd_out ? ADK_HOS_SPD_OUT_TBL :
                            ADK_HOS_SPD_IN_TBL; 

    ADK_MEMSET(&acl_rule, 0, sizeof(acl_rule));
    ADK_MEMSET(&ip6_mask, 0, sizeof(ip6_mask));

    /* Do not allow default rule to be replaced */
    if (rule_attr->precedence == 0)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Rule precedence must be > 0\n");
        goto _adk_hos_spd_tbl_rule_add_exit;
    }

    acl_rule.precedence = rule_attr->precedence;
    hos_rule_add_proto(rule_attr->proto, 
                        rule_attr->proto_mask, &acl_rule);
    acl_rule.cond.mask.sip.is_ipv6 = true;
    acl_rule.cond.mask.dip.is_ipv6 = true;
    if (!rule_attr->is_ipv6)
    {
        if(0 != rule_attr->dip_addr_mask[0])
        {
            acl_rule.condition |= IES_ACL_MATCH_DIP;
            acl_rule.cond.val.dip.is_ipv6 = false;
            acl_rule.cond.mask.dip.addr[0] = rule_attr->dip_addr_mask[0];
            acl_rule.cond.val.dip.addr[0] = rule_attr->dip_addr[0];
        }
        if(0 != rule_attr->sip_addr_mask[0])
        {
            acl_rule.condition |= IES_ACL_MATCH_SIP;
            acl_rule.cond.val.sip.is_ipv6 = false;
            acl_rule.cond.mask.sip.addr[0] = rule_attr->sip_addr_mask[0];
            acl_rule.cond.val.sip.addr[0] = rule_attr->sip_addr[0];
        }
    }
    else
    {
        if(0 != memcmp(ip6_mask, rule_attr->dip_addr_mask, sizeof(ip6_mask)))
        {
            acl_rule.condition |= IES_ACL_MATCH_DIP;
            acl_rule.cond.val.dip.is_ipv6 = true;
            acl_rule.cond.mask.dip.addr[0] = rule_attr->dip_addr_mask[0];
            acl_rule.cond.mask.dip.addr[1] = rule_attr->dip_addr_mask[1];
            acl_rule.cond.mask.dip.addr[2] = rule_attr->dip_addr_mask[2];
            acl_rule.cond.mask.dip.addr[3] = rule_attr->dip_addr_mask[3];
            acl_rule.cond.val.dip.addr[0] = rule_attr->dip_addr[0];
            acl_rule.cond.val.dip.addr[1] = rule_attr->dip_addr[1];
            acl_rule.cond.val.dip.addr[2] = rule_attr->dip_addr[2];
            acl_rule.cond.val.dip.addr[3] = rule_attr->dip_addr[3];
        }
        if(0 != memcmp(ip6_mask, rule_attr->sip_addr_mask, sizeof(ip6_mask)))
        {
            acl_rule.condition |= IES_ACL_MATCH_SIP;
            acl_rule.cond.val.sip.is_ipv6 = true;
            acl_rule.cond.mask.sip.addr[0] = rule_attr->sip_addr_mask[0];
            acl_rule.cond.mask.sip.addr[1] = rule_attr->sip_addr_mask[1];
            acl_rule.cond.mask.sip.addr[2] = rule_attr->sip_addr_mask[2];
            acl_rule.cond.mask.sip.addr[3] = rule_attr->sip_addr_mask[3];
            acl_rule.cond.val.sip.addr[0] = rule_attr->sip_addr[0];
            acl_rule.cond.val.sip.addr[1] = rule_attr->sip_addr[1];
            acl_rule.cond.val.sip.addr[2] = rule_attr->sip_addr[2];
            acl_rule.cond.val.sip.addr[3] = rule_attr->sip_addr[3];
        }
    }
    if(rule_attr->proto_mask)
    { 
        hos_rule_add_L4_info(rule_attr->proto, rule_attr->l4_info2,
            rule_attr->l4_info2_mask, rule_attr->l4_info1, rule_attr->l4_info1_mask,
            &acl_rule);
    }

    acl_rule.action = is_spd_out ? HOS_SPD_OUT_TBL_ACTION :
                                    HOS_SPD_IN_TBL_ACTION;
    acl_rule.act_val.spd_id = rule_attr->spd_rule_id;
    if (rule_attr->is_disabled)
    {
        acl_rule.state = IES_ACL_RULE_ENTRY_STATE_INVALID;
    }
    else
    {
        acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;
    }

    if(!is_spd_out)
    {
        if(rule_attr->is_ipv6)
            acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP6_ESP_HASH_IDX];
        else
            acl_rule.act_val.hash.profile = adk_hos_pvt_g_p->hash_id[HOS_IP4_ESP_HASH_IDX];
    }

    ADK_HOS_LOG_EXT_API("ies_acl_add_rule", "in SPD table\n");
    ies_ret = ies_acl_add_rule(ADK_HLP_SWITCH_ID, tbl_idx,
                               &acl_rule, (int *) rule_id);
    if(ies_ret == IES_ERR_NO_RESOURCE)
    {
        retValue = ADK_E_HOS_TABLE_FULL;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "SPD HOS table is full\n");
        goto _adk_hos_spd_tbl_rule_add_exit;
    }
    if (ies_ret != IES_OK)
    {
        retValue = ADK_E_HW_PROGRAMMING_ERROR;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "IES Error %s, Failed to add rule in SPD table\n",
                      HOS_IES_ERROR_NAME(ies_ret));
        goto _adk_hos_spd_tbl_rule_add_exit;
    }

    adk_hos_table_occupancy_incr(tbl_obj);

_adk_hos_spd_tbl_rule_add_exit:

    return retValue;
}

adk_status_t
adk_hos_l2_term_intf_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint8_t cmac;

    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_TERM_INTF_DEF_RULE_IDX])
    {
        rule_attr->l2_domain = ADK_HOS_DEFAULT_L2_DOMAIN; 
        goto _adk_hos_l2_term_intf_tbl_default_rule_found;
    }

    if ((acl_rule->condition & IES_ACL_MATCH_METADATA) &&
        (acl_rule->cond.mask.meta.sel == IES_ACL_MATCH_L2_DOMAIN) && 
        (acl_rule->cond.mask.meta.l2_domain == HOS_L2_DOMAIN_MASK))
    {
        rule_attr->l2_domain = acl_rule->cond.val.meta.l2_domain;
    }
    else
    {
        rule_attr->l2_domain = ADK_HOS_DEFAULT_L2_DOMAIN;
    }
    if (acl_rule->condition & IES_ACL_MATCH_DMAC_MAP)
    {
        cmac = acl_rule->cond.val.map_dmac;

        retValue = adk_hos_mac_mapper_get_by_cmac(cmac,
                    &rule_attr->dmac, NULL);
        if(ADK_SUCCESS != retValue)
        {
            ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected MAC MAP %d\n",
                      cmac);
            goto _adk_hos_l2_term_intf_tbl_rule_get_exit;
        }
    }
    if ((acl_rule->condition & IES_ACL_MATCH_VLAN) &&
            (acl_rule->cond.mask.vlan_id == HOS_VLAN_ID_MASK))
    {
        rule_attr->vlan_id = acl_rule->cond.val.vlan_id;
    }
    if (acl_rule->act_val.pkt_type_mask != HOS_PKT_TYPE_L2_MASK)
    {
        retValue = ADK_E_INCONSISTENT_STATE;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected pkt type mask 0x%x\n",
                      acl_rule->act_val.pkt_type_mask);
        goto _adk_hos_l2_term_intf_tbl_rule_get_exit;
    }

_adk_hos_l2_term_intf_tbl_default_rule_found:

    if ((acl_rule->act_val.pkt_type & HOS_PKT_TYPE_L2_MASK)
        == HOS_PKT_TYPE_L2_TERM)
    {
        rule_attr->is_l2_term = true;
    }
    else
    {
        rule_attr->is_l2_term = false;
    }

_adk_hos_l2_term_intf_tbl_rule_get_exit:

    return retValue;
}

adk_status_t
adk_hos_supported_ethertype_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ethertype_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX])
    {
        rule_attr->ether_type = 0; 
        goto _adk_hos_supported_ethertype_tbl_rule_get_exit;
    }

    if (acl_rule->condition & IES_ACL_MATCH_ETYPE)
    {
        rule_attr->ether_type = acl_rule->cond.val.etype;
    }

_adk_hos_supported_ethertype_tbl_rule_get_exit:

    return retValue;
}

adk_status_t
adk_hos_supported_ipproto_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ipproto_tbl_rule_attr_t* rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX])
    {
        rule_attr->proto = 0; 
        goto _adk_hos_supported_ipproto_tbl_rule_get_exit;
    }

    if (acl_rule->condition & IES_ACL_MATCH_PROT)
    {
        rule_attr->proto = acl_rule->cond.val.protocol;
    }

_adk_hos_supported_ipproto_tbl_rule_get_exit:

    return retValue;
}

adk_status_t
adk_hos_l2_ingress_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_L2_INGRESS_DEF_RULE_IDX])
    {
        rule_attr->l2_domain = ADK_HOS_DEFAULT_L2_DOMAIN; 
        goto _adk_hos_l2_ingress_tbl_default_rule_found;
    }

    if ((acl_rule->condition & IES_ACL_MATCH_METADATA) &&
        (acl_rule->cond.mask.meta.sel == IES_ACL_MATCH_L2_DOMAIN) && 
        (acl_rule->cond.mask.meta.l2_domain == HOS_L2_DOMAIN_MASK))
    {
        rule_attr->l2_domain = acl_rule->cond.val.meta.l2_domain;
    }
    else
    {
        rule_attr->l2_domain = ADK_HOS_DEFAULT_L2_DOMAIN;
    }
    if ((acl_rule->condition & IES_ACL_MATCH_ETYPE) &&
        (acl_rule->cond.mask.etype == HOS_ETYPE_MASK))
    {
        rule_attr->ether_type = acl_rule->cond.val.etype;
    }
    if (acl_rule->condition & IES_ACL_MATCH_DMAC)
    {
        rule_attr->dmac = (acl_rule->cond.val.dmac & acl_rule->cond.mask.dmac);
        rule_attr->dmac_mask = acl_rule->cond.mask.dmac;
    }

_adk_hos_l2_ingress_tbl_default_rule_found:

    rule_attr->l2_ingress = acl_rule->act_val.outer_id & HOS_INGRESS_FLOW_MASK;

    return retValue;
}

adk_status_t
adk_hos_ip_term_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ip_term_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if (((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_TERM_DEF_RULE_IDX]))
    {
        rule_attr->pkt_type = ADK_HOS_L2_SWITCHING;
        goto _adk_hos_ip_term_tbl_rule_get_exit;
    }

    if ((acl_rule->cond.val.meta.pkt_type & HOS_PKT_TYPE_L2_MASK) ==
        HOS_PKT_TYPE_L2_SWITCH)
    {
        rule_attr->pkt_type = ADK_HOS_L2_SWITCHING;
    }
    else
    {
        rule_attr->pkt_type = ADK_HOS_L2_TERMINATING;
    }

    rule_attr->is_ipv6 = acl_rule->cond.val.dip.is_ipv6;
    if (rule_attr->is_ipv6)
    {
        rule_attr->ip_addr_mask[0] = acl_rule->cond.mask.dip.addr[0];
        rule_attr->ip_addr_mask[1] = acl_rule->cond.mask.dip.addr[1];
        rule_attr->ip_addr_mask[2] = acl_rule->cond.mask.dip.addr[2];
        rule_attr->ip_addr_mask[3] = acl_rule->cond.mask.dip.addr[3];
        rule_attr->ip_addr[0] = acl_rule->cond.val.dip.addr[0];
        rule_attr->ip_addr[1] = acl_rule->cond.val.dip.addr[1];
        rule_attr->ip_addr[2] = acl_rule->cond.val.dip.addr[2];
        rule_attr->ip_addr[3] = acl_rule->cond.val.dip.addr[3];
    }
    else
    {
        rule_attr->ip_addr_mask[0] = acl_rule->cond.mask.dip.addr[0];
        rule_attr->ip_addr[0] = acl_rule->cond.val.dip.addr[0];
    }
    if ((acl_rule->action != HOS_IP_TERM_TBL_ACTION) &&
        (acl_rule->action != (HOS_IP_TERM_TBL_ACTION | IES_ACL_ACTION_SET_L3T)))
    {
        retValue = ADK_E_INCONSISTENT_STATE;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected action type 0x%lx\n", acl_rule->action);
        goto _adk_hos_ip_term_tbl_rule_get_exit;
    }

    if(acl_rule->act_val.meta.byte_select != HOS_IP_STATS_BYTE_SELECT)
    {
        retValue = ADK_E_INCONSISTENT_STATE;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected meta byte select 0x%x\n", acl_rule->act_val.meta.byte_select);
        goto _adk_hos_ip_term_tbl_rule_get_exit;
    }

    if ((acl_rule->act_val.pkt_type & HOS_PKT_TYPE_L3_MASK) ==
        HOS_PKT_TYPE_L3_TERM)
    {
        rule_attr->is_ip_term = true;

    }
    else
    {
        rule_attr->is_ip_term = false;
    }

    if ((acl_rule->cond.val.meta.pkt_type & HOS_PKT_TYPE_L3_MCAST_MASK) ==
        HOS_PKT_TYPE_L3_MCAST)
    {
        rule_attr->is_mcast = true;
    }

    if ((acl_rule->cond.val.meta.pkt_type & HOS_PKT_TYPE_L3_BCAST_MASK) ==
        HOS_PKT_TYPE_L3_BCAST)
    {
        rule_attr->is_bcast = true;
    }

    rule_attr->ip_addr_stats_id = acl_rule->act_val.meta.val[HOS_IP_STATS_BYTE] >> 2;

_adk_hos_ip_term_tbl_rule_get_exit:

    return retValue;
}

#define HOS_RULE_GET_PROTO(acl_rule, rule_attr) \
    if (acl_rule->condition & IES_ACL_MATCH_PROT) \
    { \
        rule_attr->proto = acl_rule->cond.val.protocol; \
        rule_attr->proto_mask = acl_rule->cond.mask.protocol; \
    }

#define HOS_RULE_GET_DIP(acl_rule, rule_attr) \
    if(acl_rule->condition & IES_ACL_MATCH_DIP) \
    { \
        if (rule_attr->is_ipv6) \
        { \
            rule_attr->dip_addr_mask[0] = acl_rule->cond.mask.dip.addr[0]; \
            rule_attr->dip_addr_mask[1] = acl_rule->cond.mask.dip.addr[1]; \
            rule_attr->dip_addr_mask[2] = acl_rule->cond.mask.dip.addr[2]; \
            rule_attr->dip_addr_mask[3] = acl_rule->cond.mask.dip.addr[3]; \
            rule_attr->dip_addr[0] = acl_rule->cond.val.dip.addr[0]; \
            rule_attr->dip_addr[1] = acl_rule->cond.val.dip.addr[1]; \
            rule_attr->dip_addr[2] = acl_rule->cond.val.dip.addr[2]; \
            rule_attr->dip_addr[3] = acl_rule->cond.val.dip.addr[3]; \
        } \
        else \
        { \
            rule_attr->dip_addr_mask[0] = acl_rule->cond.mask.dip.addr[0]; \
            rule_attr->dip_addr[0] = acl_rule->cond.val.dip.addr[0]; \
        } \
    }

#define HOS_RULE_GET_SIP(acl_rule, rule_attr) \
    if(acl_rule->condition & IES_ACL_MATCH_SIP) \
    { \
        if (rule_attr->is_ipv6) \
        { \
            rule_attr->sip_addr_mask[0] = acl_rule->cond.mask.sip.addr[0]; \
            rule_attr->sip_addr_mask[1] = acl_rule->cond.mask.sip.addr[1]; \
            rule_attr->sip_addr_mask[2] = acl_rule->cond.mask.sip.addr[2]; \
            rule_attr->sip_addr_mask[3] = acl_rule->cond.mask.sip.addr[3]; \
            rule_attr->sip_addr[0] = acl_rule->cond.val.sip.addr[0]; \
            rule_attr->sip_addr[1] = acl_rule->cond.val.sip.addr[1]; \
            rule_attr->sip_addr[2] = acl_rule->cond.val.sip.addr[2]; \
            rule_attr->sip_addr[3] = acl_rule->cond.val.sip.addr[3]; \
        } \
        else \
        { \
            rule_attr->sip_addr_mask[0] = acl_rule->cond.mask.sip.addr[0]; \
            rule_attr->sip_addr[0] = acl_rule->cond.val.sip.addr[0]; \
        } \
    }

#define HOS_RULE_GET_DST_PORT(acl_rule, rule_attr) \
    if(acl_rule->condition & IES_ACL_MATCH_L4_DST_PORT) \
    { \
        rule_attr->l4_info2 = acl_rule->cond.val.l4_dst; \
        rule_attr->l4_info2_mask = acl_rule->cond.mask.l4_dst; \
    }
 
#define HOS_RULE_GET_SRC_PORT(acl_rule, rule_attr) \
    if(acl_rule->condition & IES_ACL_MATCH_L4_SRC_PORT) \
    { \
        rule_attr->l4_info1 = acl_rule->cond.val.l4_src; \
        rule_attr->l4_info1_mask = acl_rule->cond.mask.l4_src; \
    }


adk_status_t
adk_hos_ip_classifier_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ip_classifier_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if (((int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP4_CLASSIFIER_DEF_RULE_IDX]) ||
        ((int) rule_id == 
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_IP6_CLASSIFIER_DEF_RULE_IDX]))
    
    {
        goto _adk_hos_ip_classifier_tbl_rule_get_exit;
    }

    if (rule_attr->is_ipv6 != acl_rule->cond.val.dip.is_ipv6)
    {
        retValue = ADK_E_INCONSISTENT_STATE;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected IP version %d\n",
                      acl_rule->cond.val.dip.is_ipv6);
        goto _adk_hos_ip_classifier_tbl_rule_get_exit;
    }

    rule_attr->precedence = acl_rule->precedence;

    HOS_RULE_GET_PROTO(acl_rule, rule_attr)
    HOS_RULE_GET_DIP(acl_rule, rule_attr)
    HOS_RULE_GET_SIP(acl_rule, rule_attr)
    HOS_RULE_GET_DST_PORT(acl_rule, rule_attr)
    HOS_RULE_GET_SRC_PORT(acl_rule, rule_attr)

_adk_hos_ip_classifier_tbl_rule_get_exit:

    rule_attr->rule_id = acl_rule->act_val.acl_id;

    return retValue;
}

adk_status_t
adk_hos_spd_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    bool is_spd_out,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_spd_tbl_rule_attr_t *rule_attr)
{
    adk_status_t retValue = ADK_SUCCESS;

    if (!is_spd_out && (int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP4_DEF_RULE_IDX])
    {
        goto _adk_hos_spd_tbl_default_rule_found;
    }

    if (!is_spd_out && (int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_IN_IP6_DEF_RULE_IDX])
    {
        goto _adk_hos_spd_tbl_default_rule_found;
    }

    if (is_spd_out && (int) rule_id ==
        adk_hos_pvt_g_p->default_acl_rule_id[HOS_SPD_OUT_DEF_RULE_IDX])
    {
        goto _adk_hos_spd_tbl_default_rule_found;
    }

    rule_attr->is_ipv6 = acl_rule->cond.val.dip.is_ipv6;
    rule_attr->precedence = acl_rule->precedence;

    HOS_RULE_GET_PROTO(acl_rule, rule_attr)
    HOS_RULE_GET_DIP(acl_rule, rule_attr)
    HOS_RULE_GET_SIP(acl_rule, rule_attr)
    HOS_RULE_GET_DST_PORT(acl_rule, rule_attr)
    HOS_RULE_GET_SRC_PORT(acl_rule, rule_attr)

    if ((!is_spd_out && acl_rule->action != HOS_SPD_IN_TBL_ACTION) ||
        (is_spd_out && acl_rule->action != HOS_SPD_OUT_TBL_ACTION))
    {
        retValue = ADK_E_INCONSISTENT_STATE;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Unexpected action type 0x%lx\n", acl_rule->action);
        goto _adk_hos_spd_tbl_rule_get_exit;
    }

_adk_hos_spd_tbl_default_rule_found:

    rule_attr->spd_rule_id = acl_rule->act_val.spd_id;

_adk_hos_spd_tbl_rule_get_exit:

    return retValue;
}

adk_status_t
adk_hos_l2_term_intf_tbl_rule_update(
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule)
{
    adk_status_t retValue = ADK_SUCCESS;

    /* For l2 interface, only interface ID can be modified */
    acl_rule->act_val.inner_id = rule_attr->l2_interface;

    return retValue;
}

adk_status_t
adk_hos_l2_ingress_tbl_rule_update (
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule)
{
    adk_status_t retValue = ADK_SUCCESS;

    /* For l2 ingress, only ingress flow ID can be modified */
    acl_rule->act_val.outer_id = rule_attr->l2_ingress;

    return retValue;
}

adk_status_t
adk_hos_ip_classifier_tbl_rule_update (
    adk_hos_ip_classifier_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule)
{
    adk_status_t retValue = ADK_SUCCESS;

    /* For classifier, only precedence and classifier ID can be modified */
    if (rule_attr->precedence == 0)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Rule precedence must be > 0\n");
        goto _adk_hos_ip_classifier_tbl_rule_update_exit;
    }

    acl_rule->precedence = rule_attr->precedence;
    acl_rule->act_val.acl_id = rule_attr->rule_id;

_adk_hos_ip_classifier_tbl_rule_update_exit:

    return retValue;
}

adk_status_t
adk_hos_spd_tbl_rule_update (
    adk_hos_spd_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule)
{
    adk_status_t retValue = ADK_SUCCESS;

    /* For SPD table, only precedence can be changed */

    if (rule_attr->precedence == 0)
    {
        retValue = ADK_E_INVALID_PARAMETER;
        ADK_LOG_ERROR(adk_hos_pvt_g_p->sys_handle,
                      ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle),
                      "Rule precedence must be > 0\n");
        goto _adk_hos_spd_tbl_rule_update_exit;
    }

    acl_rule->precedence = rule_attr->precedence;

_adk_hos_spd_tbl_rule_update_exit:

    return retValue;
}

const char *
adk_hos_tbl_name(adk_hos_object_t obj)
{
    switch (obj)
    {
    case ADK_HOS_L2_TERM_INTF_TBL:
        return ("ADK_HOS_L2_TERM_INTF_TBL");
    case ADK_HOS_IP4_CLASSIFIER_TBL:
        return ("ADK_HOS_IP4_CLASSIFIER_TBL");
    case ADK_HOS_IP6_CLASSIFIER_TBL:
        return ("ADK_HOS_IP6_CLASSIFIER_TBL");

    case ADK_HOS_SUPPORTED_ETHERTYPE_TBL:
        return ("ADK_HOS_SUPPORTED_ETHERTYPE_TBL");

    case ADK_HOS_SUPPORTED_IPPROTO_TBL:
        return ("ADK_HOS_SUPPORTED_IPPROTO_TBL");

    case ADK_HOS_L2_INGRESS_TBL:
        return ("ADK_HOS_L2_INGRESS_TBL");

    case ADK_HOS_IP_TERM_TBL:
        return ("ADK_HOS_IP_TERM_TBL");

    case ADK_HOS_SPD_IN_TBL:
        return ("ADK_HOS_SPD_IN_TBL");

    case ADK_HOS_SPD_OUT_TBL:
        return ("ADK_HOS_SPD_OUT_TBL");

    default:
        return ("ADK_HOS_TBL_UNKNOWN");
    }
}

const char *
adk_hos_pkt_type_name(adk_hos_packet_type_t pkt_type)
{
    switch (pkt_type)
    {
    case ADK_HOS_L2_SWITCHING:
        return ("L2 Switching");

    case ADK_HOS_L2_TERMINATING:
        return ("L2 Terminating");

    case ADK_HOS_IP_ROUTING:
        return ("IP Routing");

    case ADK_HOS_IP_TERMINATING:
        return ("IP Terminating");

    case ADK_HOS_PKT_TYPE_ANY:
        return ("*");

    default:
        return ("Unknown");
    }
}

/*!    end addtogroup _hosFunctions
 *     @}
 * end addtogroup _hos
 * @}
 */
