/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_conf.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _hos  SIF HOS Component
 * @{
 *
 * This file contains ADK HOS internal macros defining the HOS
 * configurations.
 *
 *  **********************************************************************/

#ifndef __ADK_HOS_CONF_INCLUDE_H__
#define __ADK_HOS_CONF_INCLUDE_H__

#define ADK_HOS_RESOURCE_CRUNCH 1

/* Common constants */

#define HOS_PORTS_MAX       32
#define HOS_IP_PROTO_ICMP   1
#define HOS_IP_PROTO_ICMPV6 58
#define HOS_IP_PROTO_IGMP   2
#define HOS_IP_PROTO_UDP    17
#define HOS_IP_PROTO_TCP    6
#define HOS_IP_PROTO_SCTP   132
#define HOS_DSCP_BIT_IN_TOS 2
#define HOS_VLAN_PRI_MAX    7
#define HOS_DEFAULT_MTU     9728

/* Field widths (in bits) and masks */
#define HOS_BYTE_WIDTH      8
#define HOS_BYTE_MASK       0xff
#define HOS_PORT_WIDTH      5
#define HOS_PORT_MASK       0x1f
#define HOS_L2_DOMAIN_WIDTH 9
#define HOS_L2_DOMAIN_MASK  0x1ff
#define HOS_L2_INTF_WIDTH   8
#define HOS_L2_INTF_MASK    0x1ff
#define HOS_L3_DOMAIN_WIDTH 6
#define HOS_L3_DOMAIN_MASK  0x3f
#define HOS_PKT_TYPE_WIDTH  7
#define HOS_PKT_TYPE_MASK   0x7f
#define HOS_INGRESS_FLOW_WIDTH  10
#define HOS_INGRESS_FLOW_MASK   0x3ff
#define HOS_IP_CLASSIFIER_WIDTH  10
#define HOS_IP_CLASSIFIER_MASK   0x3ff
#define HOS_MAC_ADDR_WIDTH  48
#define HOS_MAC_ADDR_MASK   0xffffffffffff
#define HOS_VLAN_ID_WIDTH   12
#define HOS_VLAN_ID_MASK    0xfff
#define HOS_VLAN_TAG_MASK   0x3
#define HOS_ETYPE_WIDTH     16
#define HOS_ETYPE_MASK      0xffff
#define HOS_PCP_WIDTH       4
#define HOS_PCP_MASK        0xe /* LSB for DEI is always wildcarded */
#define HOS_IP_TOS_WIDTH    8
#define HOS_IP_TOS_MASK     0xff
#define HOS_IP_PROTO_WIDTH  8
#define HOS_IP_PROTO_MASK   0xff
#define HOS_IP4_ADDR_WIDTH  32
#define HOS_IP4_ADDR_MASK   0xffffffff
#define HOS_IP6_ADDR_WIDTH  128
#define HOS_L4_PORT_WIDTH   16
#define HOS_L4_PORT_MASK    0xffff
#define HOS_ACL_SETID_WIDTH 8
#define HOS_ACL_SETID_MASK  0xff
#define HOS_ACL_ID_WIDTH    12
#define HOS_ACL_ID_MASK     0xfff
#define HOS_SPD_SETID_WIDTH 8
#define HOS_SPD_SETID_MASK  0xff
#define HOS_SPD_ID_WIDTH    10
#define HOS_SPD_ID_MASK     0x3ff
#define HOS_CPM_STATUS_MASK 0xF
#define HOS_MAC_MAP_WIDTH   8
#define HOS_MAC_MAP_MASK    0xff
#define HOS_IP_STATS_BYTE_WIDTH 6
#define HOS_IP_STATS_BYTE_MASK 0x3f
#define HOS_ETYPE_IP4       0x800
#define HOS_ETYPE_IP6       0x86DD

/* HOS ONPI metadata */
#define HOS_ONPI_TYPE_OFFSET                0
#define HOS_ONPI_TYPE_MASK                  0xFF

/* L2 ingress ID bits in meta data */
#define HOS_L2_INGRESS_BYTE                 24
#define HOS_L2_INGRESS_BYTE_SELECT          0X03000000
/* L2 ingress ID bits in meta data */
#define HOS_IP_STATS_BYTE                   25
#define HOS_IP_STATS_BYTE_SELECT            0X02000000

/* Data in ACL ID metadata field */
#define HOS_PCAP_DATA_BIT                   4
#define HOS_PCAP_DATA_MASK                  0xFFF0
#define HOS_PCAP_DATA_OFFSET                22
#define HOS_PCAP_TRAP_CODE                  0xF

/* HOS policing profile IDs */
#define HOS_L2_COLOR_BIND                   0
#define HOS_L3_COLOR_BIND                   1
#define HOS_NUM_POL_PROFILE_IDS             2
/* HOS_NUM_POL_PROFILE_IDS must not exceed IES_POL_PROFILE_INDEX_MAX = 16 */

/* HOS packet types */
#define HOS_PKT_TYPE_L2_SWITCH          0
#define HOS_PKT_TYPE_L2_TERM            (1 << 0)
#define HOS_PKT_TYPE_L2_MASK            HOS_PKT_TYPE_L2_TERM
#define HOS_PKT_TYPE_L3_ROUTE           0
#define HOS_PKT_TYPE_L3_TERM            (1 << 1)
#define HOS_PKT_TYPE_L3_MASK            HOS_PKT_TYPE_L3_TERM
#define HOS_PKT_TYPE_L3_MCAST           (1 << 2)
#define HOS_PKT_TYPE_L3_MCAST_MASK      HOS_PKT_TYPE_L3_MCAST
#define HOS_PKT_TYPE_L3_BCAST           (1 << 3)
#define HOS_PKT_TYPE_L3_BCAST_MASK      HOS_PKT_TYPE_L3_BCAST



/* Ingress flags */
#define HOS_FLAG_MACSEC_STATUS          (1 << 0)
#define HOS_FLAG_L4_CHKSUM_ERR          (1 << 1)
#define HOS_FLAG_VALID_TRAP             (1 << 2)

#define HOS_FLAG_IN_GENERIC_BIT         3
#define HOS_FLAG_IN_GENERIC_MASK        0xF
#define HOS_FLAG_ETHERTYPE_ALLOW        (1 << (HOS_FLAG_IN_GENERIC_BIT + 0))
#define HOS_FLAG_ETHERTYPE_DENY         (0 << (HOS_FLAG_IN_GENERIC_BIT + 0))
#define HOS_FLAG_ETHERTYPE_MASK         (1 << (HOS_FLAG_IN_GENERIC_BIT + 0))
#define HOS_FLAG_IPPROTO_ALLOW          (1 << (HOS_FLAG_IN_GENERIC_BIT + 1))
#define HOS_FLAG_IPPROTO_DENY           (0 << (HOS_FLAG_IN_GENERIC_BIT + 1))
#define HOS_FLAG_IPPROTO_MASK           (1 << (HOS_FLAG_IN_GENERIC_BIT + 1))

#define HOS_FLAG_TRAP_CODE_BIT          7
#define HOS_FLAG_TRAP_CODE_MASK         0xF
#define HOS_FLAG_TRAP_MASK              ((HOS_FLAG_TRAP_CODE_MASK << \
                                          HOS_FLAG_TRAP_CODE_BIT) | \
                                         HOS_FLAG_VALID_TRAP)

/* Policing color coding in metadata */
#define HOS_POLICER_NO_ACTION           0
#define HOS_POLICER_GREEN               1
#define HOS_POLICER_YELLOW              2
#define HOS_POLICER_RED                 3

/* Trap codes */
#define HOS_FFU_TRAP                    0
#define HOS_MAC_TRAP                    1
#define HOS_IGMP_TRAP                   2
#define HOS_ICMP_TRAP                   3
#define HOS_IP_OPT_TRAP                 4
#define HOS_CPU_MAC_TRAP                5
#define HOS_MTU_VOILATE_TRAP            6
#define HOS_TTL_VOILATE_TRAP            7
#define HOS_BAD_IP_HDR_TRAP             8

/* IP Header error has Trigger Trap code = 0 */
#define HOS_IP_HDR_ERROR                (HOS_FLAG_VALID_TRAP | \
                                         (HOS_BAD_IP_HDR_TRAP << \
                                          HOS_FLAG_TRAP_CODE_BIT))

/* Egress flags */
#define HOS_FLAG_MACSEC_CONTRL          (1 << 0)
#define HOS_FLAG_L4_CHKSUM_SET          (1 << 1)

#define HOS_TBL_IDX_BASE                1000

/* Default HOS table rules indices */
#define HOS_L2_TERM_INTF_DEF_RULE_IDX                0
#define HOS_IP4_CLASSIFIER_DEF_RULE_IDX         HOS_L2_TERM_INTF_DEF_RULE_IDX+1
#define HOS_IP6_CLASSIFIER_DEF_RULE_IDX         HOS_IP4_CLASSIFIER_DEF_RULE_IDX+1
#define HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX    HOS_IP6_CLASSIFIER_DEF_RULE_IDX+1
#define HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX      HOS_SUPPORTED_ETHERTYPE_DEF_RULE_IDX+1
#define HOS_L2_INGRESS_DEF_RULE_IDX             HOS_SUPPORTED_IPPROTO_DEF_RULE_IDX+1
#define HOS_IP4_TERM_DEF_RULE_IDX               HOS_L2_INGRESS_DEF_RULE_IDX+1
#define HOS_IP6_TERM_DEF_RULE_IDX               HOS_IP4_TERM_DEF_RULE_IDX+1
#define HOS_SPD_IN_IP4_DEF_RULE_IDX             HOS_IP6_TERM_DEF_RULE_IDX+1
#define HOS_SPD_IN_IP6_DEF_RULE_IDX             HOS_SPD_IN_IP4_DEF_RULE_IDX+1
#define HOS_SPD_OUT_DEF_RULE_IDX                HOS_SPD_IN_IP6_DEF_RULE_IDX+1
#define HOS_PLAINTEXT_HASH_DEF_IP4_RULE_IDX     HOS_SPD_OUT_DEF_RULE_IDX+1
#define HOS_PLAINTEXT_HASH_DEF_IP6_RULE_IDX     HOS_PLAINTEXT_HASH_DEF_IP4_RULE_IDX+1
#define HOS_ESP_HASH_DEF_FRAG_IP4_RULE_IDX      HOS_PLAINTEXT_HASH_DEF_IP6_RULE_IDX+1
#define HOS_ESP_HASH_DEF_FRAG_IP6_RULE_IDX      HOS_ESP_HASH_DEF_FRAG_IP4_RULE_IDX+1
#define HOS_ESP_HASH_DEF_UNFRAG_IP4_RULE_IDX    HOS_ESP_HASH_DEF_FRAG_IP6_RULE_IDX+1
#define HOS_ESP_HASH_DEF_UNFRAG_IP6_RULE_IDX    HOS_ESP_HASH_DEF_UNFRAG_IP4_RULE_IDX+1
#define HOS_MAX_DEF_RULE_IDX                    HOS_ESP_HASH_DEF_UNFRAG_IP6_RULE_IDX+1

#define HOS_CPK_HLP_PORT            20
#define HOS_CPM_ENCRYPT_PORT        21
#define HOS_CPM_DECRYPT_PORT        22

/* L2-Term table settings */
#define HOS_L2_TERM_INTF_TBL_IDX             0
#define HOS_L2_TERM_INTF_TBL_LOCATION        IES_ACL_FFU_GROUP_0
#define HOS_L2_TERM_INTF_TBL_INST            IES_ACL_NO_INSTANCE
#define HOS_L2_TERM_INTF_TBL_SCENARIO        (IES_ACL_SCENARIO_INGRS_NON_IP | \
                                         IES_ACL_SCENARIO_PARSER_ERR | \
                                         IES_ACL_SCENARIO_INGRS_IPV4 | \
                                         IES_ACL_SCENARIO_INGRS_IPV6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6)
#define HOS_L2_TERM_INTF_TBL_CONDITION       (IES_ACL_MATCH_METADATA | \
                                         IES_ACL_MATCH_DMAC_MAP |\
                                         IES_ACL_MATCH_VLAN)
#define HOS_L2_TERM_INTF_TBL_ACTION          (IES_ACL_ACTION_SET_PKT_TYPE | \
                                         IES_ACL_ACTION_REDIRECT | \
                                         IES_ACL_ACTION_SET_HASH_NEXTHOP | \
                                         IES_ACL_ACTION_SET_INNER_ID)

#define HOS_L2_TERM_INTF_M_B_ACTION          (IES_ACL_ACTION_SET_HASH_NEXTHOP | \
                                         IES_ACL_ACTION_SET_INNER_ID)

#define HOS_L2_TERM_INTF_TBL_MIN_RULES       256
#define HOS_L2_TERM_INTF_TBL_MAX_RULES       512

/* IPv4 clear text and inner Ingress classifier table settings */
#define HOS_IP4_CLASSIFIER_TBL_IDX          2
#define HOS_IP4_CLASSIFIER_TBL_LOCATION     IES_ACL_FFU_GROUP_0
#define HOS_IP4_CLASSIFIER_TBL_INST         IES_ACL_NO_INSTANCE
#define HOS_IP4_CLASSIFIER_TBL_SCENARIO     (IES_ACL_SCENARIO_INGRS_IPV4 | \
                                             IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                             IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                             IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V4)
#define HOS_IP4_CLASSIFIER_TBL_CONDITION    (IES_ACL_MATCH_PROT | \
                                             IES_ACL_MATCH_SIP | \
                                             IES_ACL_MATCH_DIP | \
                                             IES_ACL_MATCH_L4_SRC_PORT | \
                                             IES_ACL_MATCH_L4_DST_PORT | \
                                             IES_ACL_MATCH_VLAN | \
                                             IES_ACL_MATCH_VLAN_TAG_TYPE)
#define HOS_IP4_CLASSIFIER_TBL_ACTION       IES_ACL_ACTION_SET_ACL_ID
#define HOS_IP4_CLASSIFIER_TBL_MIN_RULES    512
#define HOS_IP4_CLASSIFIER_TBL_MAX_RULES    1024

/* IPv6 clear text and inner Ingress classifier table settings */
#define HOS_IP6_CLASSIFIER_TBL_IDX          3
#define HOS_IP6_CLASSIFIER_TBL_LOCATION     IES_ACL_FFU_GROUP_0
#define HOS_IP6_CLASSIFIER_TBL_INST         IES_ACL_NO_INSTANCE
#define HOS_IP6_CLASSIFIER_TBL_SCENARIO     (IES_ACL_SCENARIO_INGRS_IPV6 | \
                                             IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                             IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6 | \
                                             IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V6)
#define HOS_IP6_CLASSIFIER_TBL_CONDITION    (IES_ACL_MATCH_PROT | \
                                             IES_ACL_MATCH_SIP | \
                                             IES_ACL_MATCH_DIP | \
                                             IES_ACL_MATCH_L4_SRC_PORT | \
                                             IES_ACL_MATCH_L4_DST_PORT | \
                                             IES_ACL_MATCH_VLAN | \
                                             IES_ACL_MATCH_VLAN_TAG_TYPE)
#define HOS_IP6_CLASSIFIER_TBL_ACTION       IES_ACL_ACTION_SET_ACL_ID 
#define HOS_IP6_CLASSIFIER_TBL_MIN_RULES    512
#define HOS_IP6_CLASSIFIER_TBL_MAX_RULES    1024

/* Supported Ethertype */
#define HOS_SUPPORTED_ETHERTYPE_TBL_IDX         4
#define HOS_SUPPORTED_ETHERTYPE_TBL_LOCATION    IES_ACL_EXACT_MATCH_0 
#define HOS_SUPPORTED_ETHERTYPE_TBL_INST        IES_ACL_NO_INSTANCE 
#define HOS_SUPPORTED_ETHERTYPE_TBL_SCENARIO    (IES_ACL_SCENARIO_INGRS_NON_IP | \
                                                 IES_ACL_SCENARIO_PARSER_ERR | \
                                                 IES_ACL_SCENARIO_INGRS_IPV4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPV6 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6)
#define HOS_SUPPORTED_ETHERTYPE_TBL_CONDITION   IES_ACL_MATCH_ETYPE 
#define HOS_SUPPORTED_ETHERTYPE_TBL_ACTION      IES_ACL_ACTION_SET_FLAG 

#define HOS_SUPPORTED_ETHERTYPE_TBL_MIN_RULES   64
#define HOS_SUPPORTED_ETHERTYPE_TBL_MAX_RULES   64

/* Supported IP protocols*/
#define HOS_SUPPORTED_IPPROTO_TBL_IDX           5
#define HOS_SUPPORTED_IPPROTO_TBL_LOCATION      IES_ACL_EXACT_MATCH_0 
#define HOS_SUPPORTED_IPPROTO_TBL_INST          IES_ACL_NO_INSTANCE 
#define HOS_SUPPORTED_IPPROTO_TBL_SCENARIO      (IES_ACL_SCENARIO_INGRS_IPV4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPV6 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                                 IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6)
#define HOS_SUPPORTED_IPPROTO_TBL_CONDITION     IES_ACL_MATCH_PROT 
#define HOS_SUPPORTED_IPPROTO_TBL_ACTION        IES_ACL_ACTION_SET_FLAG 

#define HOS_SUPPORTED_IPPROTO_TBL_MIN_RULES     64
#define HOS_SUPPORTED_IPPROTO_TBL_MAX_RULES     64

/* L2-Ingress flow table settings */
#define HOS_L2_INGRESS_TBL_IDX          6
#define HOS_L2_INGRESS_TBL_LOCATION     IES_ACL_FFU_GROUP_1
#define HOS_L2_INGRESS_TBL_INST         IES_ACL_NO_INSTANCE
#define HOS_L2_INGRESS_TBL_SCENARIO     (IES_ACL_SCENARIO_INGRS_NON_IP | \
                                         IES_ACL_SCENARIO_PARSER_ERR | \
                                         IES_ACL_SCENARIO_INGRS_IPV4 | \
                                         IES_ACL_SCENARIO_INGRS_IPV6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6)
#define HOS_L2_INGRESS_TBL_CONDITION    (IES_ACL_MATCH_METADATA | \
                                         IES_ACL_MATCH_ETYPE | \
                                         IES_ACL_MATCH_DMAC)
#define HOS_L2_INGRESS_TBL_ACTION       IES_ACL_ACTION_SET_METADATA

#define HOS_L2_INGRESS_TBL_MIN_RULES    256
#define HOS_L2_INGRESS_TBL_MAX_RULES    1024

/* IPv4/IPv6 inner/outer Term table settings */
#define HOS_IP_TERM_TBL_IDX             7
#define HOS_IP_TERM_TBL_LOCATION        IES_ACL_FFU_GROUP_1
#define HOS_IP_TERM_TBL_INST            IES_ACL_NO_INSTANCE
#define HOS_IP_TERM_TBL_SCENARIO        (IES_ACL_SCENARIO_INGRS_IPV4 | \
                                         IES_ACL_SCENARIO_INGRS_IPV6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6 | \
                                         IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V4 | \
                                         IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V6)
#define HOS_IP_TERM_TBL_CONDITION       (IES_ACL_MATCH_METADATA | \
                                         IES_ACL_MATCH_DIP)
#define HOS_IP_TERM_TBL_ACTION          (IES_ACL_ACTION_SET_PKT_TYPE | \
                                         IES_ACL_ACTION_SET_METADATA)
#define HOS_IP_TERM_TBL_MIN_RULES       256
#define HOS_IP_TERM_TBL_MAX_RULES       1024

/* SPD In table settings */
#define HOS_SPD_IN_TBL_IDX              8
#define HOS_SPD_IN_TBL_LOCATION         IES_ACL_FFU_GROUP_1
#define HOS_SPD_IN_TBL_INST             IES_ACL_NO_INSTANCE
#define HOS_SPD_IN_TBL_SCENARIO         (IES_ACL_SCENARIO_INGRS_IPV4 | \
                                         IES_ACL_SCENARIO_INGRS_IPV6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_NAT_T_V6 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                         IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6)
#define HOS_SPD_IN_TBL_CONDITION        (IES_ACL_MATCH_PROT | \
                                         IES_ACL_MATCH_SIP | \
                                         IES_ACL_MATCH_DIP | \
                                         IES_ACL_MATCH_L4_SRC_PORT | \
                                         IES_ACL_MATCH_L4_DST_PORT)
#define HOS_SPD_IN_TBL_ACTION           IES_ACL_ACTION_SET_SPD_ID
#define HOS_SPD_IN_TBL_MIN_RULES        256
#define HOS_SPD_IN_TBL_MAX_RULES        1024

/* SPD OUT table settings */
#define HOS_SPD_OUT_TBL_IDX             9
#define HOS_SPD_OUT_TBL_LOCATION        IES_ACL_FFU_GROUP_2
#define HOS_SPD_OUT_TBL_INST            IES_ACL_NO_INSTANCE
#define HOS_SPD_OUT_TBL_SCENARIO        (IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V4 | \
                                         IES_ACL_SCENARIO_INGRS_POSTCRYPTO_DSI_IPSEC_V6) 
#define HOS_SPD_OUT_TBL_CONDITION       (IES_ACL_MATCH_PROT | \
                                         IES_ACL_MATCH_SIP | \
                                         IES_ACL_MATCH_DIP | \
                                         IES_ACL_MATCH_L4_SRC_PORT | \
                                         IES_ACL_MATCH_L4_DST_PORT)
#define HOS_SPD_OUT_TBL_ACTION          IES_ACL_ACTION_SET_SPD_ID
#define HOS_SPD_OUT_TBL_MIN_RULES       256
#define HOS_SPD_OUT_TBL_MAX_RULES       1024

/* PLAINTEXT NEXTHOP table settings */
#define HOS_PLAINTEXT_HASH_TBL_IDX          10
#define HOS_PLAINTEXT_HASH_TBL_LOCATION     IES_ACL_FFU_GROUP_1
#define HOS_PLAINTEXT_HASH_TBL_INST         1000
#define HOS_PLAINTEXT_HASH_TBL_SCENARIO     (IES_ACL_SCENARIO_INGRS_IPV4 | \
                                             IES_ACL_SCENARIO_INGRS_IPV6) 
#define HOS_PLAINTEXT_HASH_TBL_CONDITION    IES_ACL_MATCH_ETYPE 
#define HOS_PLAINTEXT_HASH_TBL_ACTION       IES_ACL_ACTION_SET_HASH_NEXTHOP
#define HOS_PLAINTEXT_HASH_TBL_MIN_RULES    64
#define HOS_PLAINTEXT_HASH_TBL_MAX_RULES    64

/* ESP NEXTHOP table settings */
#define HOS_ESP_HASH_TBL_IDX        11
#define HOS_ESP_HASH_TBL_LOCATION   IES_ACL_FFU_GROUP_1
#define HOS_ESP_HASH_TBL_INST       1000
#define HOS_ESP_HASH_TBL_SCENARIO   (IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V4 | \
                                     IES_ACL_SCENARIO_INGRS_IPSEC_ESP_V6) 
#define HOS_ESP_HASH_TBL_CONDITION  (IES_ACL_MATCH_ETYPE | \
                                     IES_ACL_MATCH_FRAG)
#define HOS_ESP_HASH_TBL_ACTION     IES_ACL_ACTION_SET_HASH_NEXTHOP 
#define HOS_ESP_HASH_TBL_MIN_RULES  64
#define HOS_ESP_HASH_TBL_MAX_RULES  64

/* Remap slots for FFU Group-0 */
#define HOS_REMAP_SLOT_INNER_ID 0x30
#define HOS_REMAP_SLOT_ACL_ID	0x0C
/* Remap slots for FFU Group-1 */
#define HOS_REMAP_SLOT_L2_INGRESS   0x30
#define HOS_REMAP_SLOT_SPD_ID	    0x0C 
#define HOS_REMAP_SLOT_IP_STATS     0x80

/* Remap slots for FFU0 and FFU1 */
#define HOS_REMAP_SLOT_PKT_TYPE     0x01

#endif /* __ADK_HOS_CONF_INCLUDE_H__ */

/*!
 * @} end addtogroup _hos
 */
