/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _hos  SIF HOS Component
 * @{
 *
 * This file contains ADK HOS funtions for use in DP processing
 *
 *  **********************************************************************/

#ifndef __ADK_HOS_DP_INCLUDE_H__
#define __ADK_HOS_DP_INCLUDE_H__

typedef struct ice_dsi_rx_metadata adk_dsi_rx_metadata_t;
typedef struct ice_dsi_tx_metadata adk_dsi_tx_metadata_t;

/* Return true if L2 terminating else false */
bool adk_hos_is_l2_term(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if L2 interface is invalid */
bool adk_hos_is_l2_interface_invalid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if L2 interface is invalid */
bool adk_hos_is_l2_interface_valid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return L2 interface ID */
uint8_t adk_hos_get_l2_interface(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if L2 interface is invalid */
bool adk_hos_is_ingress_flow_invalid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return L2 interface ID */
uint32_t adk_hos_get_ingress_flow(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if L3 terminating else false */
bool adk_hos_is_l3_term(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if IP multicast else false */
bool adk_hos_is_ip_mcast(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if IP broadcast else false */
bool adk_hos_is_ip_bcast(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if IP stats ID is invalid else false */
bool adk_hos_is_ip_stats_invalid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return IP stats ID */
uint32_t adk_hos_get_ip_stats_id(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if IP Ingress classifier is invalid else false */
bool adk_hos_is_ip_classifier_invalid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return if ethertype is allowed */
bool adk_hos_is_ethertype_invalid(adk_dsi_rx_metadata_t *dsi_md_p);

/* Return if IP protocol is allowed */
bool adk_hos_is_ipproto_invalid(adk_dsi_rx_metadata_t *dsi_md_p);

/* Return classifier rule ID */
uint32_t adk_hos_get_ip_classifier_rule_id(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if packet went through successful inline decryption */
bool adk_hos_is_inline_decrypted(adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if invalid SPD rule ID matched */
bool adk_hos_is_spd_id_invalid(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Return SPD rule ID */
uint16_t adk_hos_get_spd_id(const adk_dsi_rx_metadata_t *dsi_md_p);

/* Returns true if IP header has errors. The errors include header
 * checksum error & IP header length error. */
bool adk_hos_is_ip_hdr_bad(adk_dsi_rx_metadata_t *dsi_md_p);

/* Returns true if packet has been marked RED by the policer */
bool adk_hos_is_policer_red(adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if L4 checksum is incorrect */
bool adk_hos_is_l4_chksum_err(adk_dsi_rx_metadata_t *dsi_md_p);

/* Return true if trap code is valid. Output is trap code  */
bool adk_hos_trap_code_get(adk_dsi_rx_metadata_t *dsi_md_p,
                           uint8_t *trap_code);

/* Function to enable L4 checksum insertion in egress packet */
void adk_hos_enable_l4_chksum(adk_dsi_tx_metadata_t *dsi_md_p);

/* Function to disable L4 checksum insertion in egress packet */
void adk_hos_disable_l4_chksum(adk_dsi_tx_metadata_t *dsi_md_p);

#ifdef ADK_HOS_DP_TEST
adk_dsi_rx_metadata_t*
adk_hos_test_get_dsi_vsi_ptr(void* task);
#endif

#endif /* __ADK_HOS_DP_INCLUDE_H__ */
 
/*!
 * @} end addtogroup _hos
 */
