/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_hos_pvt.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _hos  SIF HOS Component
 * @{
 *
 * This file contains ADK HOS internal private data structures and
 * private function prototypes.
 *
 *  **********************************************************************/

#ifndef __ADK_HOS_PVT_INCLUDE_H__
#define __ADK_HOS_PVT_INCLUDE_H__

#include <rte_pmd_ice_dsi.h>
#include <ies_sdk.h>
#include <ies_event_types.h>
#include <ies_api_port.h>

#include <adk_os.h>
#include <adk_module_mgr.h>
#include <adk_shm.h>
#include <adk_hos.h>
#include <adk_hos_conf.h>

#define ADK_HOS_SHM_MEMZONE_NAME   "adk_hos_shm_zone"

#define HOS_L2_HASH_IDX         0
#define HOS_IP4_HASH_IDX        HOS_L2_HASH_IDX+1
#define HOS_IP6_HASH_IDX        HOS_IP4_HASH_IDX+1
#define HOS_IP4_ESP_HASH_IDX    HOS_IP6_HASH_IDX+1
#define HOS_IP6_ESP_HASH_IDX    HOS_IP4_ESP_HASH_IDX+1
#define HOS_HASH_MAX_IDX        HOS_IP6_ESP_HASH_IDX+1

#define HOS_MAX_MAC_MAPPER      96

typedef struct hos_table_usage_s
{
    uint32_t size;
    uint32_t occupancy;
} hos_table_usage_t;

typedef struct hos_mac_mapper_s
{
    uint64_t mac_addr;
    uint8_t cmac;
    int entry_id;
} hos_mac_mapper_t;

typedef struct hos_rule_data_s
{
    uint32_t prec;
    adk_hos_rule_id_t acl_rule_id;
} hos_rule_data_t;

typedef struct adk_hos_pvt_data_s
{
    adk_sys_handle_t sys_handle;
    int hash_id[HOS_HASH_MAX_IDX];
    hos_rule_data_t* rule_ids; 
    int port_stats_profile_id;
    int default_acl_rule_id[HOS_MAX_DEF_RULE_IDX];
    int police_profile_id[HOS_NUM_POL_PROFILE_IDS];
    hos_mac_mapper_t mac_mapper[HOS_MAX_MAC_MAPPER];
    hos_table_usage_t tbl_usage[ADK_HOS_MAX_OBJ];
} adk_hos_pvt_data_t;

#define HOS_IES_ERROR_NAME(ies_ret) \
    ((ies_error_get_name(ies_ret, ies_error_name, \
                         IES_ERROR_MAX_NAME_SIZE) == IES_OK) ? \
     ies_error_name : "HOS_UNKNOWN")

#define ADK_HOS_LOG_EXT_API(api, fmt, args ...) \
    ADK_LOG_EXT_API(adk_hos_pvt_g_p->sys_handle, \
                    ADK_SYS_MODULE_HANDLE_GET(adk_hos_pvt_g_p->sys_handle), \
                    api, fmt, ## args)

extern adk_hos_pvt_data_t *adk_hos_pvt_g_p;
extern const uint64_t adk_hos_mac_mask_g[];

adk_status_t
adk_hos_qos_init (
    void);

adk_status_t
adk_hos_policer_init (
    void);

adk_status_t
adk_hos_init_tables (
    void);

adk_status_t
adk_hos_shutdown_tables (
    void);

adk_status_t
adk_hos_l2_term_intf_tbl_rule_add (
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_l2_ingress_tbl_rule_add (
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_supported_ethertype_tbl_rule_add(
    adk_hos_ethertype_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_supported_ipproto_tbl_rule_add(
    adk_hos_ipproto_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_ip_term_tbl_rule_add (
    adk_hos_ip_term_tbl_rule_attr_t *rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_ip_classifier_tbl_rule_add (
    adk_hos_ip_classifier_tbl_rule_attr_t* rule_attr,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_spd_tbl_rule_add (
    adk_hos_spd_tbl_rule_attr_t *rule_attr,
    bool is_spd_out,
    adk_hos_rule_id_t *rule_id);

adk_status_t
adk_hos_l2_term_intf_tbl_rule_get (
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr);

adk_status_t
adk_hos_l2_ingress_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr);

adk_status_t
adk_hos_supported_ethertype_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ethertype_tbl_rule_attr_t *rule_attr);

adk_status_t
adk_hos_supported_ipproto_tbl_rule_get(
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ipproto_tbl_rule_attr_t* rule_attr);

adk_status_t
adk_hos_ip_term_tbl_rule_get (
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ip_term_tbl_rule_attr_t *rule_attr);

adk_status_t
adk_hos_ip_classifier_tbl_rule_get (
    adk_hos_rule_id_t rule_id,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_ip_classifier_tbl_rule_attr_t* rule_attr);

adk_status_t
adk_hos_spd_tbl_rule_get (
    adk_hos_rule_id_t rule_id,
    bool is_spd_out,
    struct ies_acl_rule_info *acl_rule,
    adk_hos_spd_tbl_rule_attr_t *rule_attr);

adk_status_t
adk_hos_l2_term_intf_tbl_rule_update(
    adk_hos_l2_term_intf_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule);

adk_status_t
adk_hos_l2_ingress_tbl_rule_update (
    adk_hos_l2_ingress_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule);

adk_status_t
adk_hos_ip_classifier_tbl_rule_update (
    adk_hos_ip_classifier_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule);

adk_status_t
adk_hos_spd_tbl_rule_update (
    adk_hos_spd_tbl_rule_attr_t *rule_attr,
    struct ies_acl_rule_info *acl_rule);

const char *
adk_hos_tbl_name (
    adk_hos_object_t obj);

const char *
adk_hos_pkt_type_name (
    adk_hos_packet_type_t pkt_type);

/* Increase the current occupancy of the table
    by one. Throws error in overflow case */
adk_status_t
adk_hos_table_occupancy_incr (
    adk_hos_object_t obj);

/* Decrease the current occupancy of the table
    by one. Throws error in underflow case */
adk_status_t
adk_hos_table_occupancy_decr (
    adk_hos_object_t obj);


#endif /* __ADK_HOS_PVT_INCLUDE_H__ */

/*!
 * @} end addtogroup _hos
 */
