/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * @file  adk_lttng_dp_trace_defn.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 *
 *
 * This file contains definition of functions that are
 * part of implementation for LTTng support in ADK.
 *
 ***********************************************************************/
#include "adk_modules_enabled.h"

#define TRACEPOINT_DEFINE
#define TRACEPOINT_PROBE_DYNAMIC_LINKAGE

#include "adk_dp_tp_sif.h"
#include "adk_dp_tp_sif_debug.h"
#include "adk_dp_tp_sif_qos.h"
#include "adk_dp_tp_sif_qos_debug.h"
#include "adk_dp_tp_sif_fps.h"
#include "adk_dp_tp_sif_fps_debug.h"
#include "adk_dp_tp_ph.h"
#include "adk_dp_tp_ph_debug.h"

#if defined(ADK_MODULE_ETHERNET_ENABLED)
#include "adk_dp_tp_ethernet.h"
#include "adk_dp_tp_ethernet_debug.h"
#endif

#if defined(ADK_MODULE_ETH_OAM_ENABLED)
#include "adk_dp_tp_eth_oam.h"
#include "adk_dp_tp_eth_oam_debug.h"
#endif

#if defined(ADK_MODULE_GTPU_ENABLED)
#include "adk_dp_tp_gtpu.h"
#include "adk_dp_tp_gtpu_debug.h"
#endif

#if defined(ADK_MODULE_IP_ENABLED)
#include "adk_dp_tp_ip.h"
#include "adk_dp_tp_ip_debug.h"
#endif

#if defined(ADK_MODULE_IPV6_ENABLED)
#include "adk_dp_tp_ipv6.h"
#include "adk_dp_tp_ipv6_debug.h"
#endif

#if defined(ADK_MODULE_BFD_ENABLED)
#include "adk_dp_tp_bfd.h"
#include "adk_dp_tp_bfd_debug.h"
#endif



#if defined(ADK_MODULE_IPSEC_ENABLED)
#include "adk_dp_tp_ipsec.h"
#include "adk_dp_tp_ipsec_debug.h"
#endif


#if defined(ADK_MODULE_L2TPV3_ENABLED)
#include "adk_dp_tp_l2tpv3.h"
#include "adk_dp_tp_l2tpv3_debug.h"
#endif

#if defined(ADK_MODULE_EHSFP_IWF_ENABLED)
#include "adk_dp_tp_ehsfp_iwf.h"
#include "adk_dp_tp_ehsfp_iwf_debug.h"
#endif

#if defined(ADK_MODULE_TWAMP_ENABLED)
#include "adk_dp_tp_twamp.h"
#include "adk_dp_tp_twamp_debug.h"
#endif

#include "adk_include.h"
#include "adk_module_id.h"

#include <stdarg.h>
#include "adk_lttng_dp_trace.h"

/*
 * =============================================================
 * 
 * Filename fragments (and filename) for determining the submodule
 *
 * =============================================================
 */
static const char * ADK_QOS_FILE_PREFIX="adk_qos_";
static const char * ADK_FPS_FILE_PREFIX="adk_fps_";
static const char * ADK_PH_FILE_PREFIX="adk_ph_";
static const char * ADK_PH_FILE_PREFIX_ALT="adk_ph.c";


/*!
 * =============================================================
 * @enum adk_lttng_dp_sys_submodule_t
 *
 * Internal enum to define the SIF type of logging to do
 *
 * =============================================================
 */
typedef enum adk_lttng_sys_submodule_e {
    ADK_LTTNG_SYS_SUBMODULE_QOS = 1,  /*!< Submodule is QoS */
    ADK_LTTNG_SYS_SUBMODULE_FPS,  /*!< Submodule is FPS */
    ADK_LTTNG_SYS_SUBMODULE_PH,  /*!< Submodule is PH */
    ADK_LTTNG_SYS_SUBMODULE_SYS,  /*!< Submodule is SYS by default */
} adk_lttng_dp_sys_submodule_t;   /* ------  end of enum adk_lttng_dp_sys_submodule_t  ------ */

/*!
 * =============================================================
 * @adk_lttng_dp_sys_submodule_get
 * @param[in] fileLine Name of the file
 * @param[out] subModule - the submodule to log against
 *
 *
 * We only have one module ID for PH, QoS and SIF. Since we want to have
 * different LTTng tags for PH and QoS, this function will return the
 * submodule ID based on the file name.
 *
 * Functiont get the submodule to log against based on file name. QoS and PH
 * are part of the system module, but we would like to log against their own
 * tracepoints.
 *
 * =============================================================
 */
static inline adk_status_t
adk_lttng_dp_sys_submodule_get(
                               const char *fileLine,
                               adk_lttng_dp_sys_submodule_t *subModule
                              )
{
    adk_status_t    retValue = ADK_SUCCESS;

    *subModule = ADK_LTTNG_SYS_SUBMODULE_SYS;

    if (fileLine[4] == ADK_QOS_FILE_PREFIX[4])
    {
        *subModule = ADK_LTTNG_SYS_SUBMODULE_QOS;
    } else
    {
        if (fileLine[4] == ADK_FPS_FILE_PREFIX[4])
        {
            *subModule = ADK_LTTNG_SYS_SUBMODULE_FPS;
        } else
        {
            if (fileLine[4] == ADK_PH_FILE_PREFIX[4])
            {
                *subModule = ADK_LTTNG_SYS_SUBMODULE_PH;
            } else
            {
                if (fileLine[4] == ADK_PH_FILE_PREFIX_ALT[4])
                {
                    *subModule = ADK_LTTNG_SYS_SUBMODULE_PH;
                }
            }
        }
    }

    ADK_RETURN( retValue );
}    /* -----  end of function adk_lttng_dp_sys_submodule_get  ----- */

/*
 * =============================================================
 * Generic macro definition for INFO/ERROR/WARNING events. 
 * This function calls the respective module's tracepoint 
 * provider. 
 * =============================================================
 */
#define DP_DEFINE_INFO_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, data1); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_2D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_3D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_4D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3, data4); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_5D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4, \
        const int32_t data5) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3, data4, data5); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_2X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_3X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_4X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3, data4); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_5X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3, data4, data5); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_6X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5, \
        const uint32_t data6) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2, data3, data4, data5, data6); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1P_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, data1_p); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_2P_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2_p); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_ERRNO_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const char* errnoStr, \
        const int32_t my_errno) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   errnoStr, my_errno); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1S_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const char* dataStr) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, dataStr); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1D_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const uint32_t data2) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1, data2); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1P_4D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4, \
        const int32_t data5) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2, data3, data4, data5); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1P_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const uint32_t data2) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_1P_2X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const uint32_t data2, \
        const uint32_t data3) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2, data3); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_2P_1D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p, \
        const int32_t data3) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2_p, data3); \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_INFO_2P_2D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p, \
        const int32_t data3, \
        const int32_t data4) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), fileLine, message, \
                   data1_p, data2_p, data3, data4); \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Generic macro definition for FORCE_ versions of the 
 * INFO/ERROR/WARNING events. 
 * This function calls the respective module's tracepoint 
 * provider. 
 * =============================================================
 */
#define DP_DEFINE_FORCE_INFO_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   "unknown:forced", fileLine, message); \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for INFO/ERROR/WARNING events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_INFO_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos_dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos_dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps_dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps_dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph_dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph_dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif_dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for INFO/ERROR/WARNING events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_INFO_1D_FN(module_name, event, _debug_) \
    adk_status_t adk_lttng_dp_trace_##module_name####_debug_####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##_debug_##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##_debug_##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##_debug_##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##_debug_##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##_debug_##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##_debug_##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##_debug_##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##_debug_##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_2D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_3D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_4D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_5D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4, \
        const int32_t data5) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_2X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_3X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_4X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_5X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, data5); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_6X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5, \
        const uint32_t data6) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, \
                       data5, data6); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, \
                       data5, data6); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, \
                       data5, data6); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2, data3, data4, \
                       data5, data6); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_1D_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const int32_t data1, \
        const uint32_t data2) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1, data2); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for INFO/ERROR/WARNING events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_INFO_1P_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_2P_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_1P_4D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4, \
        const int32_t data5) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3, data4, data5); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3, data4, data5); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3, data4, data5); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_1P_1X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const uint32_t data2) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_1P_2X_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        const uint32_t data2, \
        const uint32_t data3) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2, data3); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_2P_1D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p, \
        const int32_t data3) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3); \
        } \
    } \
    return ADK_SUCCESS; \
}

#define DP_DEFINE_SIF_INFO_2P_2D_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        void* data1_p, \
        void* data2_p, \
        const int32_t data3, \
        const int32_t data4) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3, data4); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3, data4); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, data1_p, data2_p, data3, data4); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for INFO/ERROR/WARNING events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_INFO_1S_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const char* dataStr) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, dataStr); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, dataStr); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, dataStr); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, dataStr); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for INFO/ERROR/WARNING events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_INFO_ERRNO_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message, \
        const char* errnoStr, \
        const int32_t my_errno) \
         \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, errnoStr, my_errno); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, errnoStr, my_errno); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, errnoStr, my_errno); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message, errnoStr, my_errno); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Generic macro definition for FORCE_INFO/FORCE_WARNING/
 * FORCE_ERROR events. This function calls the respective 
 * module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_FORCE_FNS(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       "unknown:forced", fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       "unknown:forced", fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       "unknown:forced", fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_SYS) \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       "unknown:forced", fileLine, message); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * ==============================================================
 * Macro definition for FUNCTION ENTRY/EXIT events. This function 
 * calls the SIF module's tracepoint provider. 
 * ==============================================================
 */
#define DP_DEFINE_SIF_ENTRY_EXIT_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* function) \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, function); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, function); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, function); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, function); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Generic macro definition for FUNC_ENTRY/EXIT events. This 
 * function calls the respective module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_ENTRY_EXIT_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* function) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), \
                   fileLine, function); \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Generic macro definition for FORCE_FUNC_ENTRY/EXIT events. This 
 * function calls the respective module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_FORCE_ENTRY_EXIT_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_force_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* function) \
{ \
    adk_lttng_dp_sys_submodule_t subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_SYS_QOS_MODULE_NAME, \
                       fileLine, function); \
        } \
    } \
    if( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_PH_MODULE_NAME, \
                       fileLine, function); \
        } \
    } \
    if( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_PH_MODULE_NAME, \
                       fileLine, function); \
        } \
    } \
    if( subModule == ADK_LTTNG_SYS_SUBMODULE_SYS) \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_SYS_MODULE_NAME, \
                       fileLine, function); \
        } \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Generic macro definition for HW events. This 
 * function calls the respective module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_HW_DEBUG_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
{ \
    if(__tracepoint_com_intel_adk_##module_name####debug##dp___intel_adk_dp_##event.state) \
    { \
        tracepoint(com_intel_adk_##module_name####debug##dp, intel_adk_dp_##event, \
                   ADK_DP_MODULE_NAME_GET(modId), \
                   fileLine, message); \
    } \
    return ADK_SUCCESS; \
}

/*
 * =============================================================
 * Macro definition for HW events. This function 
 * calls the SIF module's tracepoint provider. 
 * =============================================================
 */
#define DP_DEFINE_SIF_HW_DEBUG_FN(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileLine, \
        const char* message) \
{ \
    adk_lttng_dp_sys_submodule_t  subModule; \
    adk_lttng_dp_sys_submodule_get(fileLine, &subModule); \
    if ( subModule == ADK_LTTNG_SYS_SUBMODULE_QOS) \
    { \
        if(__tracepoint_com_intel_adk_sif_qos##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_qos##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_FPS) \
    { \
        if(__tracepoint_com_intel_adk_sif_fps##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif_fps##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else if ( subModule == ADK_LTTNG_SYS_SUBMODULE_PH) \
    { \
        if(__tracepoint_com_intel_adk_ph##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_ph##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    else \
    { \
        if(__tracepoint_com_intel_adk_sif##debug##dp___intel_adk_dp_##event.state) \
        { \
            tracepoint(com_intel_adk_sif##debug##dp, intel_adk_dp_##event, \
                       ADK_DP_MODULE_NAME_GET(modId), \
                       fileLine, message); \
        } \
    } \
    return ADK_SUCCESS; \
}

DP_DEFINE_SIF_INFO_FN(sif, info, _)
DP_DEFINE_SIF_FORCE_FNS(sif, force_info, _)
DP_DEFINE_SIF_INFO_FN(sif, error, _)
DP_DEFINE_SIF_FORCE_FNS(sif, force_error, _)
DP_DEFINE_SIF_INFO_FN(sif, fatal_error, _)
DP_DEFINE_SIF_FORCE_FNS(sif, force_fatal, _)
DP_DEFINE_SIF_INFO_FN(sif, warning, _)
DP_DEFINE_SIF_FORCE_FNS(sif, force_warning, _)
DP_DEFINE_SIF_ENTRY_EXIT_FN(sif, func_entry, _)
DP_DEFINE_SIF_ENTRY_EXIT_FN(sif, func_exit, _)
DP_DEFINE_SIF_FORCE_ENTRY_EXIT_FN(sif, func_entry, _)
DP_DEFINE_SIF_FORCE_ENTRY_EXIT_FN(sif, func_exit, _)
DP_DEFINE_SIF_INFO_1D_FN(sif, error_1d, _)
DP_DEFINE_SIF_INFO_2D_FN(sif, error_2d, _)
DP_DEFINE_SIF_INFO_1P_FN(sif, error_1p, _)
DP_DEFINE_SIF_INFO_ERRNO_FN(sif, error_errno, _)
DP_DEFINE_SIF_INFO_1S_FN(sif, error_1s, _)
DP_DEFINE_SIF_INFO_1D_FN(sif, info_1d, _)
DP_DEFINE_SIF_INFO_1D_FN(sif, info_1x, _)
DP_DEFINE_SIF_INFO_1P_FN(sif, info_1p, _)
DP_DEFINE_SIF_INFO_2D_FN(sif, info_2d, _)
DP_DEFINE_SIF_INFO_ERRNO_FN(sif, info_errno, _)
DP_DEFINE_SIF_INFO_1S_FN(sif, info_1s, _)
#ifdef ADK_DEBUG
DP_DEFINE_SIF_INFO_FN(sif, info, _debug_)
DP_DEFINE_SIF_INFO_1D_FN(sif, info_1d, _debug_)
DP_DEFINE_SIF_INFO_2D_FN(sif, info_2d, _debug_)
DP_DEFINE_SIF_INFO_3D_FN(sif, info_3d, _debug_)
DP_DEFINE_SIF_INFO_4D_FN(sif, info_4d, _debug_)
DP_DEFINE_SIF_INFO_5D_FN(sif, info_5d, _debug_)
DP_DEFINE_SIF_INFO_1X_FN(sif, info_1x, _debug_)
DP_DEFINE_SIF_INFO_2X_FN(sif, info_2x, _debug_)
DP_DEFINE_SIF_INFO_3X_FN(sif, info_3x, _debug_)
DP_DEFINE_SIF_INFO_4X_FN(sif, info_4x, _debug_)
DP_DEFINE_SIF_INFO_5X_FN(sif, info_5x, _debug_)
DP_DEFINE_SIF_INFO_6X_FN(sif, info_6x, _debug_)
DP_DEFINE_SIF_INFO_1P_FN(sif, info_1p, _debug_)
DP_DEFINE_SIF_INFO_2P_FN(sif, info_2p, _debug_)
DP_DEFINE_SIF_INFO_ERRNO_FN(sif, info_errno, _debug_)
DP_DEFINE_SIF_INFO_1S_FN(sif, info_1s, _debug_)
DP_DEFINE_SIF_INFO_1D_1X_FN(sif, info_1d_1x, _debug_)
DP_DEFINE_SIF_INFO_1P_4D_FN(sif, info_1p_4d, _debug_)
DP_DEFINE_SIF_INFO_1P_1X_FN(sif, info_1p_1x, _debug_)
DP_DEFINE_SIF_INFO_1P_2X_FN(sif, info_1p_2x, _debug_)
DP_DEFINE_SIF_INFO_2P_1D_FN(sif, info_2p_1d, _debug_)
DP_DEFINE_SIF_INFO_2P_2D_FN(sif, info_2p_2d, _debug_)
DP_DEFINE_SIF_ENTRY_EXIT_FN(sif, func_entry, _debug_)
DP_DEFINE_SIF_ENTRY_EXIT_FN(sif, func_exit, _debug_)
DP_DEFINE_SIF_HW_DEBUG_FN(sif, log_hw, _debug_)
#endif

#define ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(__module_adk__) \
    DP_DEFINE_INFO_FN(__module_adk__, info, _) \
    DP_DEFINE_FORCE_INFO_FN(__module_adk__, force_info, _) \
    DP_DEFINE_INFO_FN(__module_adk__, error, _) \
    DP_DEFINE_FORCE_INFO_FN(__module_adk__, force_error, _) \
    DP_DEFINE_INFO_FN(__module_adk__, fatal_error, _) \
    DP_DEFINE_FORCE_INFO_FN(__module_adk__, force_fatal, _) \
    DP_DEFINE_INFO_FN(__module_adk__, warning, _) \
    DP_DEFINE_FORCE_INFO_FN(__module_adk__, force_warning, _) \
    DP_DEFINE_ENTRY_EXIT_FN(__module_adk__, func_entry, _) \
    DP_DEFINE_ENTRY_EXIT_FN(__module_adk__, func_exit, _) \
    DP_DEFINE_INFO_1D_FN(__module_adk__, error_1d, _) \
    DP_DEFINE_INFO_1P_FN(__module_adk__, error_1p, _) \
    DP_DEFINE_INFO_2D_FN(__module_adk__, error_2d, _) \
    DP_DEFINE_INFO_ERRNO_FN(__module_adk__, error_errno, _) \
    DP_DEFINE_INFO_1S_FN(__module_adk__, error_1s, _) \
    DP_DEFINE_INFO_1D_FN(__module_adk__, info_1d, _) \
    DP_DEFINE_INFO_1P_FN(__module_adk__, info_1p, _) \
    DP_DEFINE_INFO_1D_FN(__module_adk__, info_1x, _) \
    DP_DEFINE_INFO_2D_FN(__module_adk__, info_2d, _) \
    DP_DEFINE_INFO_ERRNO_FN(__module_adk__, info_errno, _) \
    DP_DEFINE_INFO_1S_FN(__module_adk__, info_1s, _) \
    ADK_MODULE_DP_LTTNG_DEBUG_FUNCS_DEFINED(__module_adk__)

#ifdef ADK_DEBUG
#define ADK_MODULE_DP_LTTNG_DEBUG_FUNCS_DEFINED(__module_adk__) \
    DP_DEFINE_INFO_FN(__module_adk__, info, _debug_) \
    DP_DEFINE_INFO_1D_FN(__module_adk__, info_1d, _debug_) \
    DP_DEFINE_INFO_2D_FN(__module_adk__, info_2d, _debug_) \
    DP_DEFINE_INFO_3D_FN(__module_adk__, info_3d, _debug_) \
    DP_DEFINE_INFO_4D_FN(__module_adk__, info_4d, _debug_) \
    DP_DEFINE_INFO_5D_FN(__module_adk__, info_5d, _debug_) \
    DP_DEFINE_INFO_1X_FN(__module_adk__, info_1x, _debug_) \
    DP_DEFINE_INFO_2X_FN(__module_adk__, info_2x, _debug_) \
    DP_DEFINE_INFO_3X_FN(__module_adk__, info_3x, _debug_) \
    DP_DEFINE_INFO_4X_FN(__module_adk__, info_4x, _debug_) \
    DP_DEFINE_INFO_5X_FN(__module_adk__, info_5x, _debug_) \
    DP_DEFINE_INFO_6X_FN(__module_adk__, info_6x, _debug_) \
    DP_DEFINE_INFO_1P_FN(__module_adk__, info_1p, _debug_) \
    DP_DEFINE_INFO_2P_FN(__module_adk__, info_2p, _debug_) \
    DP_DEFINE_INFO_ERRNO_FN(__module_adk__, info_errno, _debug_) \
    DP_DEFINE_INFO_1S_FN(__module_adk__, info_1s, _debug_) \
    DP_DEFINE_INFO_1D_1X_FN(__module_adk__, info_1d_1x, _debug_) \
    DP_DEFINE_INFO_1P_4D_FN(__module_adk__, info_1p_4d, _debug_) \
    DP_DEFINE_INFO_1P_1X_FN(__module_adk__, info_1p_1x, _debug_) \
    DP_DEFINE_INFO_1P_2X_FN(__module_adk__, info_1p_2x, _debug_) \
    DP_DEFINE_INFO_2P_1D_FN(__module_adk__, info_2p_1d, _debug_) \
    DP_DEFINE_INFO_2P_2D_FN(__module_adk__, info_2p_2d, _debug_) \
    DP_DEFINE_ENTRY_EXIT_FN(__module_adk__, func_entry, _debug_) \
    DP_DEFINE_ENTRY_EXIT_FN(__module_adk__, func_exit, _debug_) \
    DP_DEFINE_HW_DEBUG_FN(__module_adk__, log_hw, _debug_)

#else
#define ADK_MODULE_DP_LTTNG_DEBUG_FUNCS_DEFINED(__module_adk__)
#endif

#if defined(ADK_MODULE_IP_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(ip)
#endif

#if defined(ADK_MODULE_IPV6_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(ipv6)
#endif

#if defined(ADK_MODULE_IPSEC_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(ipsec)
#endif

#if defined(ADK_MODULE_ETHERNET_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(ethernet)
#endif

#if defined(ADK_MODULE_GTPU_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(gtpu)
#endif

#if defined(ADK_MODULE_BFD_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(bfd)
#endif


#if defined(ADK_MODULE_L2TPV3_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(l2tpv3)
#endif

#if defined(ADK_MODULE_EHSFP_IWF_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(ehsfp_iwf)
#endif

#if defined(ADK_MODULE_ETH_OAM_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(eth_oam)
#endif

#if defined(ADK_MODULE_TWAMP_ENABLED)
ADK_MODULE_DP_LTTNG_FUNCS_DEFINED(twamp)
#endif

#define ADK_LTTNG_DP_MODULE_INIT(__modId, __modName) \
    [__modId][TRACE_DP_INFO_INDEX] = DP_FN_NAME(__modName, info), \
    [__modId][TRACE_DP_FORCE_INFO_INDEX] = DP_FN_NAME(__modName, force_info), \
    [__modId][TRACE_DP_ERROR_INDEX] = DP_FN_NAME(__modName, force_error), \
    [__modId][TRACE_DP_FORCE_ERROR_INDEX] = DP_FN_NAME(__modName, error), \
    [__modId][TRACE_DP_FATAL_ERROR_INDEX] = DP_FN_NAME(__modName, fatal_error), \
    [__modId][TRACE_DP_FORCE_FATAL_INDEX] = DP_FN_NAME(__modName, force_fatal), \
    [__modId][TRACE_DP_WARNING_INDEX] = DP_FN_NAME(__modName, warning), \
    [__modId][TRACE_DP_FORCE_WARNING_INDEX] = DP_FN_NAME(__modName, force_warning), \
    [__modId][TRACE_DP_FUNC_ENTRY_INDEX] = DP_FN_NAME(__modName, func_entry), \
    [__modId][TRACE_DP_FUNC_EXIT_INDEX] = DP_FN_NAME(__modName, func_exit), \
    [__modId][TRACE_DP_FUNC_ENTRY_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, func_entry), \
    [__modId][TRACE_DP_FUNC_EXIT_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, func_exit), \
    [__modId][TRACE_DP_FORCE_FUNC_ENTRY_INDEX] = NULL, \
    [__modId][TRACE_DP_FORCE_FUNC_EXIT_INDEX] = NULL, \
    [__modId][TRACE_DP_HW_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, log_hw), \
    [__modId][TRACE_DP_ERROR_1D_INDEX] = DP_FN_NAME(__modName, error_1d), \
    [__modId][TRACE_DP_ERROR_2D_INDEX] = DP_FN_NAME(__modName, error_2d), \
    [__modId][TRACE_DP_ERROR_1P_INDEX] = DP_FN_NAME(__modName, error_1p), \
    [__modId][TRACE_DP_ERROR_ERRNO_INDEX] = DP_FN_NAME(__modName, error_errno), \
    [__modId][TRACE_DP_ERROR_1S_INDEX] = DP_FN_NAME(__modName, error_1s), \
    [__modId][TRACE_DP_INFO_1D_INDEX] = DP_FN_NAME(__modName, info_1d), \
    [__modId][TRACE_DP_INFO_1P_INDEX] = DP_FN_NAME(__modName, info_1p), \
    [__modId][TRACE_DP_INFO_1X_INDEX] = DP_FN_NAME(__modName, info_1x), \
    [__modId][TRACE_DP_INFO_2D_INDEX] = DP_FN_NAME(__modName, info_2d), \
    [__modId][TRACE_DP_INFO_ERRNO_INDEX] = DP_FN_NAME(__modName, info_errno), \
    [__modId][TRACE_DP_INFO_1S_INDEX] = DP_FN_NAME(__modName, info_1s), \
    [__modId][TRACE_DP_INFO_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info), \
    [__modId][TRACE_DP_INFO_1D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1d), \
    [__modId][TRACE_DP_INFO_2D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_2d), \
    [__modId][TRACE_DP_INFO_3D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_3d), \
    [__modId][TRACE_DP_INFO_4D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_4d), \
    [__modId][TRACE_DP_INFO_5D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_5d), \
    [__modId][TRACE_DP_INFO_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1x), \
    [__modId][TRACE_DP_INFO_2X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_2x), \
    [__modId][TRACE_DP_INFO_3X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_3x), \
    [__modId][TRACE_DP_INFO_4X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_4x), \
    [__modId][TRACE_DP_INFO_5X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_5x), \
    [__modId][TRACE_DP_INFO_6X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_6x), \
    [__modId][TRACE_DP_INFO_1P_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1p), \
    [__modId][TRACE_DP_INFO_2P_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_2p), \
    [__modId][TRACE_DP_INFO_ERRNO_DEBUG_INDEX] = DP_FN_NAME(__modName, info_errno), \
    [__modId][TRACE_DP_INFO_1S_DEBUG_INDEX] = DP_FN_NAME(__modName, info_1s), \
    [__modId][TRACE_DP_INFO_1D_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1d_1x), \
    [__modId][TRACE_DP_INFO_1P_4D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1p_4d), \
    [__modId][TRACE_DP_INFO_1P_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1p_1x), \
    [__modId][TRACE_DP_INFO_1P_2X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_1p_2x), \
    [__modId][TRACE_DP_INFO_2P_2D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_2p_2d), \
    [__modId][TRACE_DP_INFO_2P_1D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(__modName, info_2p_1d), \

const adk_lttng_dp_info_t adk_lttng_dp_info_const_ptr = {
    .cookie = ADK_LTTNG_DP_COOKIE,
    .trace_fn_ptr = {
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_INDEX] = DP_FN_NAME(sif, info),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_INFO_INDEX] = DP_FN_NAME(sif, force_info),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_INDEX] = DP_FN_NAME(sif, error),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_ERROR_INDEX] = DP_FN_NAME(sif, force_error),
        [ADK_SYS_MODULE_ID][TRACE_DP_FATAL_ERROR_INDEX] = DP_FN_NAME(sif, fatal_error),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_FATAL_INDEX] = DP_FN_NAME(sif, force_fatal),
        [ADK_SYS_MODULE_ID][TRACE_DP_WARNING_INDEX] = DP_FN_NAME(sif, warning),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_WARNING_INDEX] = DP_FN_NAME(sif, force_warning),
        [ADK_SYS_MODULE_ID][TRACE_DP_FUNC_ENTRY_INDEX] = DP_FN_NAME(sif, func_entry),
        [ADK_SYS_MODULE_ID][TRACE_DP_FUNC_EXIT_INDEX] = DP_FN_NAME(sif, func_exit),
        [ADK_SYS_MODULE_ID][TRACE_DP_FUNC_ENTRY_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, func_entry),
        [ADK_SYS_MODULE_ID][TRACE_DP_FUNC_EXIT_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, func_exit),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_FUNC_ENTRY_INDEX] = ((adk_lttng_dp_trace_fn_ptr)adk_lttng_dp_trace_force_sif_func_entry),
        [ADK_SYS_MODULE_ID][TRACE_DP_FORCE_FUNC_EXIT_INDEX] = ((adk_lttng_dp_trace_fn_ptr)adk_lttng_dp_trace_force_sif_func_exit),
        [ADK_SYS_MODULE_ID][TRACE_DP_HW_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, log_hw),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_1D_INDEX] = DP_FN_NAME(sif, error_1d),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_2D_INDEX] = DP_FN_NAME(sif, error_2d),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_1P_INDEX] = DP_FN_NAME(sif, error_1p),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_ERRNO_INDEX] = DP_FN_NAME(sif, error_errno),
        [ADK_SYS_MODULE_ID][TRACE_DP_ERROR_1S_INDEX] = DP_FN_NAME(sif, error_1s),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1D_INDEX] = DP_FN_NAME(sif, info_1d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1P_INDEX] = DP_FN_NAME(sif, info_1p),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1X_INDEX] = DP_FN_NAME(sif, info_1x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2D_INDEX] = DP_FN_NAME(sif, info_2d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_ERRNO_INDEX] = DP_FN_NAME(sif, info_errno),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1S_INDEX] = DP_FN_NAME(sif, info_1s),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_2d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_3D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_3d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_4D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_4d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_5D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_5d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_2x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_3X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_3x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_4X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_4x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_5X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_5x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_6X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_6x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1P_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1p),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2P_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_2p),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_ERRNO_DEBUG_INDEX] = DP_FN_NAME(sif, info_errno),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1S_DEBUG_INDEX] = DP_FN_NAME(sif, info_1s),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1D_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1d_1x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1P_4D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1p_4d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1P_1X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1p_1x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_1P_2X_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_1p_2x),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2P_2D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_2p_2d),
        [ADK_SYS_MODULE_ID][TRACE_DP_INFO_2P_1D_DEBUG_INDEX] = DP_FN_NAME_DEBUG(sif, info_2p_1d),

#if defined(ADK_MODULE_IP_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_IP_MODULE_ID, ip)
#endif

#if defined(ADK_MODULE_IPV6_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_IPV6_MODULE_ID, ipv6)
#endif

#if defined(ADK_MODULE_IPSEC_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_IPSEC_MODULE_ID, ipsec)
#endif

#if defined(ADK_MODULE_ETHERNET_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_ETHERNET_MODULE_ID, ethernet)
#endif

#if defined(ADK_MODULE_GTPU_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_GTPU_MODULE_ID, gtpu)
#endif

#if defined(ADK_MODULE_BFD_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_BFD_MODULE_ID, bfd)
#endif


#if defined(ADK_MODULE_L2TPV3_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_L2TPV3_MODULE_ID, l2tpv3)
#endif

#if defined(ADK_MODULE_EHSFP_IWF_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_EHSFP_IWF_MODULE_ID, ehsfp_iwf)
#endif

#if defined(ADK_MODULE_ETH_OAM_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_ETH_OAM_MODULE_ID, eth_oam)
#endif

#if defined(ADK_MODULE_TWAMP_ENABLED)
        ADK_LTTNG_DP_MODULE_INIT(ADK_TWAMP_MODULE_ID, twamp)
#endif
    }
};


/*!    end addtogroup _sifFunctions
 *     @}
 * end addtogroup _sif
 * @}
 */


