/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * file  adk_dp_tp_class_macros.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * brief
 *
 * Base set of macros for building the standard DP LTTng files
 *
 ***********************************************************************/


#ifndef  _ADK_DP_TP_CLASS_MACROS_H_
#define  _ADK_DP_TP_CLASS_MACROS_H_

#include <stdint.h>
#include <inttypes.h>

#define ADK_DP_TP_CLASS_1S_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message)

#define ADK_DP_TP_CLASS_1S_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s, \
                           ADK_DP_TP_CLASS_1S_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1S_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const char*, errMessage)

#define ADK_DP_TP_CLASS_1S_1S_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1s, \
                           ADK_DP_TP_CLASS_1S_1S_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_string(ErrorMessage, errMessage) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1S_1X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const char*, errnoStr, \
            const uint32_t, errNo)

#define ADK_DP_TP_CLASS_1S_1S_1X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1s_1x, \
                           ADK_DP_TP_CLASS_1S_1S_1X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_string(ErrnoStr, errnoStr) \
                                      ctf_integer_hex(uint32_t, Errno, errNo) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int,   data1)

#define ADK_DP_TP_CLASS_1S_1D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1d, \
                           ADK_DP_TP_CLASS_1S_1D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int, Data1, data1) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_2D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int, data1, \
            const int, data2)

#define ADK_DP_TP_CLASS_1S_2D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_2d, \
                           ADK_DP_TP_CLASS_1S_2D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int, Data1, data1) \
                                      ctf_integer(int, Data2, data2) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_3D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int, data1, \
            const int, data2, \
            const int, data3)

#define ADK_DP_TP_CLASS_1S_3D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_3d, \
                           ADK_DP_TP_CLASS_1S_3D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int, Data1, data1) \
                                      ctf_integer(int, Data2, data2) \
                                      ctf_integer(int, Data3, data3) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_4D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int, data1, \
            const int, data2, \
            const int, data3, \
            const int, data4)

#define ADK_DP_TP_CLASS_1S_4D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_4d, \
                           ADK_DP_TP_CLASS_1S_4D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int, Data1, data1) \
                                      ctf_integer(int, Data2, data2) \
                                      ctf_integer(int, Data3, data3) \
                                      ctf_integer(int, Data4, data4) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_5D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int, data1, \
            const int, data2, \
            const int, data3, \
            const int, data4, \
            const int, data5)

#define ADK_DP_TP_CLASS_1S_5D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_5d, \
                           ADK_DP_TP_CLASS_1S_5D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int, Data1, data1) \
                                      ctf_integer(int, Data2, data2) \
                                      ctf_integer(int, Data3, data3) \
                                      ctf_integer(int, Data4, data4) \
                                      ctf_integer(int, Data5, data5) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1)

#define ADK_DP_TP_CLASS_1S_1X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1x, \
                           ADK_DP_TP_CLASS_1S_1X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_2X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1, \
            const uint32_t,   data2)

#define ADK_DP_TP_CLASS_1S_2X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_2x, \
                           ADK_DP_TP_CLASS_1S_2X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_3X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1, \
            const uint32_t,   data2, \
            const uint32_t,   data3)

#define ADK_DP_TP_CLASS_1S_3X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_3x, \
                           ADK_DP_TP_CLASS_1S_3X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_4X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1, \
            const uint32_t,   data2, \
            const uint32_t,   data3, \
            const uint32_t,   data4)

#define ADK_DP_TP_CLASS_1S_4X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_4x, \
                           ADK_DP_TP_CLASS_1S_4X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                      ctf_integer_hex(uint32_t, Data4, data4) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_5X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1, \
            const uint32_t,   data2, \
            const uint32_t,   data3, \
            const uint32_t,   data4, \
            const uint32_t,   data5)

#define ADK_DP_TP_CLASS_1S_5X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_5x, \
                           ADK_DP_TP_CLASS_1S_5X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                      ctf_integer_hex(uint32_t, Data4, data4) \
                                      ctf_integer_hex(uint32_t, Data5, data5) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_6X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const uint32_t,   data1, \
            const uint32_t,   data2, \
            const uint32_t,   data3, \
            const uint32_t,   data4, \
            const uint32_t,   data5, \
            const uint32_t,   data6)

#define ADK_DP_TP_CLASS_1S_6X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_6x, \
                           ADK_DP_TP_CLASS_1S_6X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(uint32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                      ctf_integer_hex(uint32_t, Data4, data4) \
                                      ctf_integer_hex(uint32_t, Data5, data5) \
                                      ctf_integer_hex(uint32_t, Data6, data6) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1P_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p)

#define ADK_DP_TP_CLASS_1S_1P_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1p, \
                           ADK_DP_TP_CLASS_1S_1P_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1D_1X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            const int32_t,   data1, \
            const uint32_t,   data2)

#define ADK_DP_TP_CLASS_1S_1D_1X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1d_1x, \
                           ADK_DP_TP_CLASS_1S_1D_1X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer(int32_t, Data1, data1) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1P_4D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            const int32_t, data2, \
            const int32_t, data3, \
            const int32_t, data4, \
            const int32_t, data5)

#define ADK_DP_TP_CLASS_1S_1P_4D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1p_4d, \
                           ADK_DP_TP_CLASS_1S_1P_4D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer(int32_t, Data2, data2) \
                                      ctf_integer(int32_t, Data3, data3) \
                                      ctf_integer(int32_t, Data4, data4) \
                                      ctf_integer(int32_t, Data5, data5) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1P_1X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            const uint32_t, data2)

#define ADK_DP_TP_CLASS_1S_1P_1X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1p_1x, \
                           ADK_DP_TP_CLASS_1S_1P_1X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_1P_2X_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            const uint32_t, data2, \
            const uint32_t, data3)

#define ADK_DP_TP_CLASS_1S_1P_2X_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_1p_2x, \
                           ADK_DP_TP_CLASS_1S_1P_2X_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer_hex(uint32_t, Data2, data2) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_2P_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            void*, ptr2_p)

#define ADK_DP_TP_CLASS_1S_2P_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_2p, \
                           ADK_DP_TP_CLASS_1S_2P_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer_hex(void*, Ptr2_p, ptr2_p) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_2P_1D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            void*, ptr2_p, \
            const uint32_t, data3)

#define ADK_DP_TP_CLASS_1S_2P_1D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_2p_1d, \
                           ADK_DP_TP_CLASS_1S_2P_1D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer_hex(void*, Ptr2_p, ptr2_p) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_1S_2P_2D_TP_ARGS \
    TP_ARGS(const char*, moduleName, \
            const char*, fileLine, \
            const char*, message, \
            void*, ptr1_p, \
            void*, ptr2_p, \
            const uint32_t, data3, \
            const uint32_t, data4)

#define ADK_DP_TP_CLASS_1S_2P_2D_MACRO(__module, __debug) \
    TRACEPOINT_EVENT_CLASS(com_intel_adk_##__module####__debug##dp, \
                           adk_lttng_##__module##_class_1s_2p_2d, \
                           ADK_DP_TP_CLASS_1S_2P_2D_TP_ARGS, \
                           TP_FIELDS( \
                                      ctf_string(Module, moduleName) \
                                      ctf_string(File, fileLine) \
                                      ctf_string(Message, message) \
                                      ctf_integer_hex(void*, Ptr1_p, ptr1_p) \
                                      ctf_integer_hex(void*, Ptr2_p, ptr2_p) \
                                      ctf_integer_hex(uint32_t, Data3, data3) \
                                      ctf_integer_hex(uint32_t, Data4, data4) \
                                    ) \
                          )

#define ADK_DP_TP_CLASS_ALL_MACROS(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1S_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1S_1X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_2D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_3D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_4D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_5D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_2X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_3X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_4X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_5X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_6X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1P_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_2P_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1D_1X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1P_4D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1P_1X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_1P_2X_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_2P_1D_MACRO(__module__, __debug__) \
    ADK_DP_TP_CLASS_1S_2P_2D_MACRO(__module__, __debug__)



#endif     /* -----  _ADK_DP_TP_CLASS_MACROS_H_  ----- */
