/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @internal
 * file  adk_lttng_dp_trace.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * brief
 *
 * This file contains declarations of functions that are
 * part of LTTng implementation in ADK. This file needs to be included to access
 * APIs defined by ADK Lttng.
 *
 ***********************************************************************/

#if defined(ADK_MODULE_SIF_LTTNG_DP_ENABLED)

#ifndef _ADK_LTTNG_DP_TRACE_H_
#define _ADK_LTTNG_DP_TRACE_H_
/*Function prototypes*/

#include "adk_platform.h"

#define TRACE_DP_INFO_INDEX                 0
#define TRACE_DP_FORCE_INFO_INDEX           1
#define TRACE_DP_ERROR_INDEX                2
#define TRACE_DP_FORCE_ERROR_INDEX          3
#define TRACE_DP_FATAL_ERROR_INDEX          4
#define TRACE_DP_FORCE_FATAL_INDEX          5
#define TRACE_DP_WARNING_INDEX              6
#define TRACE_DP_FORCE_WARNING_INDEX        7
#define TRACE_DP_FUNC_ENTRY_INDEX           8
#define TRACE_DP_FUNC_EXIT_INDEX            9
#define TRACE_DP_FORCE_FUNC_ENTRY_INDEX     10
#define TRACE_DP_FORCE_FUNC_EXIT_INDEX      11
#define TRACE_DP_ERROR_1D_INDEX             12
#define TRACE_DP_ERROR_1P_INDEX             13
#define TRACE_DP_ERROR_2D_INDEX             14
#define TRACE_DP_ERROR_ERRNO_INDEX          15
#define TRACE_DP_ERROR_1S_INDEX             16
#define TRACE_DP_INFO_1D_INDEX              17
#define TRACE_DP_INFO_2D_INDEX              18
#define TRACE_DP_INFO_1X_INDEX              19
#define TRACE_DP_INFO_1P_INDEX              20
#define TRACE_DP_INFO_ERRNO_INDEX           21
#define TRACE_DP_INFO_1S_INDEX              22
#define TRACE_DP_INFO_DEBUG_INDEX           23
#define TRACE_DP_INFO_1D_DEBUG_INDEX        24
#define TRACE_DP_INFO_2D_DEBUG_INDEX        25
#define TRACE_DP_INFO_3D_DEBUG_INDEX        26
#define TRACE_DP_INFO_4D_DEBUG_INDEX        27
#define TRACE_DP_INFO_5D_DEBUG_INDEX        28
#define TRACE_DP_INFO_1X_DEBUG_INDEX        29
#define TRACE_DP_INFO_2X_DEBUG_INDEX        30
#define TRACE_DP_INFO_3X_DEBUG_INDEX        31
#define TRACE_DP_INFO_4X_DEBUG_INDEX        32
#define TRACE_DP_INFO_5X_DEBUG_INDEX        33
#define TRACE_DP_INFO_6X_DEBUG_INDEX        34
#define TRACE_DP_INFO_1P_DEBUG_INDEX        35
#define TRACE_DP_INFO_2P_DEBUG_INDEX        36
#define TRACE_DP_INFO_1S_DEBUG_INDEX        37
#define TRACE_DP_INFO_ERRNO_DEBUG_INDEX     38
#define TRACE_DP_INFO_1D_1X_DEBUG_INDEX     39
#define TRACE_DP_INFO_1P_4D_DEBUG_INDEX     40
#define TRACE_DP_INFO_1P_1X_DEBUG_INDEX     41
#define TRACE_DP_INFO_1P_2X_DEBUG_INDEX     42
#define TRACE_DP_INFO_2P_1D_DEBUG_INDEX     43
#define TRACE_DP_INFO_2P_2D_DEBUG_INDEX     44
#define TRACE_DP_FUNC_ENTRY_DEBUG_INDEX     45
#define TRACE_DP_FUNC_EXIT_DEBUG_INDEX      46
#define TRACE_DP_HW_DEBUG_INDEX             47

#define MAX_DP_TRACE_EVENTS                 48

typedef adk_status_t
(* adk_lttng_dp_trace_fn_ptr)(const uint32_t moduleId,
                              const char* fileLine,
                              const char* message,
                              ...);

#define DP_FN_NAME(module_name, trace_event) \
    ((adk_lttng_dp_trace_fn_ptr)adk_lttng_dp_trace_##module_name##_##trace_event)

#ifdef ADK_DEBUG
#define DP_FN_NAME_DEBUG(module_name, trace_event) \
    ((adk_lttng_dp_trace_fn_ptr)adk_lttng_dp_trace_##module_name##_debug_##trace_event)
#else
#define DP_FN_NAME_DEBUG(module_name, trace_event) \
    ((adk_lttng_dp_trace_fn_ptr)NULL)
#endif

#define ADK_DECLARE_DP_FN_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message);

#define ADK_DECLARE_DP_FN_1D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1);

#define ADK_DECLARE_DP_FN_2D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2);

#define ADK_DECLARE_DP_FN_3D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3);

#define ADK_DECLARE_DP_FN_4D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4);

#define ADK_DECLARE_DP_FN_5D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4, \
        const int32_t data5);

#define ADK_DECLARE_DP_FN_1X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1);

#define ADK_DECLARE_DP_FN_2X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2);

#define ADK_DECLARE_DP_FN_3X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3);

#define ADK_DECLARE_DP_FN_4X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4);

#define ADK_DECLARE_DP_FN_5X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5);

#define ADK_DECLARE_DP_FN_6X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const uint32_t data1, \
        const uint32_t data2, \
        const uint32_t data3, \
        const uint32_t data4, \
        const uint32_t data5, \
        const uint32_t data6);

#define ADK_DECLARE_DP_FN_1P_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr_p);

#define ADK_DECLARE_DP_FN_2P_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        void* ptr2_p);

#define ADK_DECLARE_DP_FN_1S_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const char* dataStr);

#define ADK_DECLARE_DP_FN_1D_1X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const int32_t data1, \
        const uint32_t data2);

#define ADK_DECLARE_DP_FN_1P_4D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        const int32_t data1, \
        const int32_t data2, \
        const int32_t data3, \
        const int32_t data4);

#define ADK_DECLARE_DP_FN_1P_1X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        const uint32_t data1);

#define ADK_DECLARE_DP_FN_1P_2X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        const uint32_t data1, \
        const uint32_t data2);

#define ADK_DECLARE_DP_FN_2P_1X_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        void* ptr2_p, \
        const uint32_t data1);

#define ADK_DECLARE_DP_FN_2P_1D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        void* ptr2_p, \
        const int32_t data1);

#define ADK_DECLARE_DP_FN_2P_2D_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        void* ptr1_p, \
        void* ptr2_p, \
        const int32_t data1, \
        const int32_t data2);

#define ADK_DECLARE_DP_FN_ERRNO_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message, \
        const char* errnoStr, \
        const int32_t my_errno);

#define ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* message);

#define ADK_DECLARE_DP_FORCE_ENTRY_EXIT_FN_PROTO(module_name, event, debug) \
    adk_status_t adk_lttng_dp_trace_force_##module_name####debug####event( \
        const uint32_t modId, \
        const char* fileName, \
        const char* function);

#define ADK_DECLARE_DP_FN_PROTOTYPES(__module_name__) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, info, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, force_info, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, error, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, force_error, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, fatal_error, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, force_fatal, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, warning, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, force_warning, _) \
    ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(__module_name__, func_entry, _) \
    ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(__module_name__, func_exit, _) \
    ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(__module_name__, func_entry, _debug_) \
    ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(__module_name__, func_exit, _debug_) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, log_hw, _debug_) \
    ADK_DECLARE_DP_FN_1D_PROTO(__module_name__, error_1d, _) \
    ADK_DECLARE_DP_FN_1P_PROTO(__module_name__, error_1p, _) \
    ADK_DECLARE_DP_FN_2D_PROTO(__module_name__, error_2d, _) \
    ADK_DECLARE_DP_FN_ERRNO_PROTO(__module_name__, error_errno, _) \
    ADK_DECLARE_DP_FN_1S_PROTO(__module_name__, error_1s, _) \
    ADK_DECLARE_DP_FN_1D_PROTO(__module_name__, info_1d, _) \
    ADK_DECLARE_DP_FN_1P_PROTO(__module_name__, info_1p, _) \
    ADK_DECLARE_DP_FN_1D_PROTO(__module_name__, info_1x, _) \
    ADK_DECLARE_DP_FN_2D_PROTO(__module_name__, info_2d, _) \
    ADK_DECLARE_DP_FN_ERRNO_PROTO(__module_name__, info_errno, _) \
    ADK_DECLARE_DP_FN_1S_PROTO(__module_name__, info_1s, _) \
    ADK_DECLARE_DP_FN_PROTO(__module_name__, info, _debug_) \
    ADK_DECLARE_DP_FN_1D_PROTO(__module_name__, info_1d, _debug_) \
    ADK_DECLARE_DP_FN_2D_PROTO(__module_name__, info_2d, _debug_) \
    ADK_DECLARE_DP_FN_3D_PROTO(__module_name__, info_3d, _debug_) \
    ADK_DECLARE_DP_FN_4D_PROTO(__module_name__, info_4d, _debug_) \
    ADK_DECLARE_DP_FN_5D_PROTO(__module_name__, info_5d, _debug_) \
    ADK_DECLARE_DP_FN_1X_PROTO(__module_name__, info_1x, _debug_) \
    ADK_DECLARE_DP_FN_2X_PROTO(__module_name__, info_2x, _debug_) \
    ADK_DECLARE_DP_FN_3X_PROTO(__module_name__, info_3x, _debug_) \
    ADK_DECLARE_DP_FN_4X_PROTO(__module_name__, info_4x, _debug_) \
    ADK_DECLARE_DP_FN_5X_PROTO(__module_name__, info_5x, _debug_) \
    ADK_DECLARE_DP_FN_6X_PROTO(__module_name__, info_6x, _debug_) \
    ADK_DECLARE_DP_FN_1P_PROTO(__module_name__, info_1p, _debug_) \
    ADK_DECLARE_DP_FN_2P_PROTO(__module_name__, info_2p, _debug_) \
    ADK_DECLARE_DP_FN_1D_1X_PROTO(__module_name__, info_1d_1x, _debug_) \
    ADK_DECLARE_DP_FN_2D_PROTO(__module_name__, info_2d, _debug_) \
    ADK_DECLARE_DP_FN_ERRNO_PROTO(__module_name__, info_errno, _debug_) \
    ADK_DECLARE_DP_FN_1S_PROTO(__module_name__, info_1s, _debug_) \
    ADK_DECLARE_DP_FN_1P_4D_PROTO(__module_name__, info_1p_4d, _debug_) \
    ADK_DECLARE_DP_FN_1P_1X_PROTO(__module_name__, info_1p_1x, _debug_) \
    ADK_DECLARE_DP_FN_1P_2X_PROTO(__module_name__, info_1p_2x, _debug_) \
    ADK_DECLARE_DP_FN_2P_1D_PROTO(__module_name__, info_2p_1d, _debug_) \
    ADK_DECLARE_DP_FN_2P_2D_PROTO(__module_name__, info_2p_2d, _debug_) \

ADK_DECLARE_DP_FN_PROTO(sif, info, _)
ADK_DECLARE_DP_FN_PROTO(sif, force_info, _)
ADK_DECLARE_DP_FN_PROTO(sif, error, _)
ADK_DECLARE_DP_FN_PROTO(sif, force_error, _)
ADK_DECLARE_DP_FN_PROTO(sif, fatal_error, _)
ADK_DECLARE_DP_FN_PROTO(sif, force_fatal, _)
ADK_DECLARE_DP_FN_PROTO(sif, warning, _)
ADK_DECLARE_DP_FN_PROTO(sif, force_warning, _)
ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(sif, func_entry, _)
ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(sif, func_exit, _)
ADK_DECLARE_DP_FORCE_ENTRY_EXIT_FN_PROTO(sif, func_entry, _)
ADK_DECLARE_DP_FORCE_ENTRY_EXIT_FN_PROTO(sif, func_exit, _)
ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(sif, func_entry, _debug_)
ADK_DECLARE_DP_ENTRY_EXIT_FN_PROTO(sif, func_exit, _debug_)
ADK_DECLARE_DP_FN_PROTO(sif, log_hw, _debug_)
ADK_DECLARE_DP_FN_1D_PROTO(sif, error_1d, _)
ADK_DECLARE_DP_FN_1P_PROTO(sif, error_1p, _)
ADK_DECLARE_DP_FN_2D_PROTO(sif, error_2d, _)
ADK_DECLARE_DP_FN_ERRNO_PROTO(sif, error_errno, _)
ADK_DECLARE_DP_FN_1S_PROTO(sif, error_1s, _)
ADK_DECLARE_DP_FN_1D_PROTO(sif, info_1d, _)
ADK_DECLARE_DP_FN_1P_PROTO(sif, info_1p, _)
ADK_DECLARE_DP_FN_1D_PROTO(sif, info_1x, _)
ADK_DECLARE_DP_FN_2D_PROTO(sif, info_2d, _)
ADK_DECLARE_DP_FN_ERRNO_PROTO(sif, info_errno, _)
ADK_DECLARE_DP_FN_1S_PROTO(sif, info_1s, _)
ADK_DECLARE_DP_FN_PROTO(sif, info, _debug_)
ADK_DECLARE_DP_FN_1D_PROTO(sif, info_1d, _debug_)
ADK_DECLARE_DP_FN_2D_PROTO(sif, info_2d, _debug_)
ADK_DECLARE_DP_FN_3D_PROTO(sif, info_3d, _debug_)
ADK_DECLARE_DP_FN_4D_PROTO(sif, info_4d, _debug_)
ADK_DECLARE_DP_FN_5D_PROTO(sif, info_5d, _debug_)
ADK_DECLARE_DP_FN_1X_PROTO(sif, info_1x, _debug_)
ADK_DECLARE_DP_FN_2X_PROTO(sif, info_2x, _debug_)
ADK_DECLARE_DP_FN_3X_PROTO(sif, info_3x, _debug_)
ADK_DECLARE_DP_FN_4X_PROTO(sif, info_4x, _debug_)
ADK_DECLARE_DP_FN_5X_PROTO(sif, info_5x, _debug_)
ADK_DECLARE_DP_FN_6X_PROTO(sif, info_6x, _debug_)
ADK_DECLARE_DP_FN_1P_PROTO(sif, info_1p, _debug_)
ADK_DECLARE_DP_FN_2P_PROTO(sif, info_2p, _debug_)
ADK_DECLARE_DP_FN_ERRNO_PROTO(sif, info_errno, _debug_)
ADK_DECLARE_DP_FN_1S_PROTO(sif, info_1s, _debug_)
ADK_DECLARE_DP_FN_1D_1X_PROTO(sif, info_1d_1x, _debug_)
ADK_DECLARE_DP_FN_1P_4D_PROTO(sif, info_1p_4d, _debug_)
ADK_DECLARE_DP_FN_1P_1X_PROTO(sif, info_1p_1x, _debug_)
ADK_DECLARE_DP_FN_1P_2X_PROTO(sif, info_1p_2x, _debug_)
ADK_DECLARE_DP_FN_2P_1D_PROTO(sif, info_2p_1d, _debug_)
ADK_DECLARE_DP_FN_2P_2D_PROTO(sif, info_2p_2d, _debug_)

#if defined(ADK_MODULE_IP_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(ip)
#endif

#if defined(ADK_MODULE_IPV6_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(ipv6)
#endif

#if defined(ADK_MODULE_IPSEC_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(ipsec)
#endif

#if defined(ADK_MODULE_ETHERNET_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(ethernet)
#endif

#if defined(ADK_MODULE_GTPU_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(gtpu)
#endif

#if defined(ADK_MODULE_BFD_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(bfd)
#endif


#if defined(ADK_MODULE_L2TPV3_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(l2tpv3)
#endif

#if defined(ADK_MODULE_EHSFP_IWF_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(ehsfp_iwf)
#endif

#if defined(ADK_MODULE_ETH_OAM_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(eth_oam)
#endif

#if defined(ADK_MODULE_TWAMP_ENABLED)
ADK_DECLARE_DP_FN_PROTOTYPES(twamp)
#endif


#define ADK_LTTNG_DP_PTR_GET(__module_id, __index, __fn_ptr) \
        adk_lttng_dp_fn_ptr_get((__module_id), (__index), (__fn_ptr))


#define ADK_LTTNG_DP_COOKIE    0x931ed2b9

typedef struct adk_lttng_dp_info_s {
    uint32_t    cookie;
    adk_lttng_dp_trace_fn_ptr  trace_fn_ptr[ADK_MAX_SUPPORTED_MODULES][MAX_DP_TRACE_EVENTS];
} adk_lttng_dp_info_t;

extern const adk_lttng_dp_info_t adk_lttng_dp_info_const_ptr;
/*!
 * =============================================================
 * @adk_lttng_fn_ptr_get
 *
 * Retruns a pointer to the function for LTTng logging based on module ID and
 * event.
 *
 * =============================================================
 */
static inline void
adk_lttng_dp_fn_ptr_get(uint32_t moduleId,
                        uint32_t index,
                        adk_lttng_dp_trace_fn_ptr *lttngFnPtr_p)
{
    if ((adk_lttng_dp_info_const_ptr.trace_fn_ptr[moduleId][index]) == NULL)
    {
        moduleId = ADK_SYS_MODULE_ID;
    }

    *lttngFnPtr_p = adk_lttng_dp_info_const_ptr.trace_fn_ptr[moduleId][index];
}


#endif /* #ifndef _ADK_LTTNG_DP_TRACE_H_ */

#endif /* #if defined(ADK_MODULE_SIF_LTTNG_DP_ENABLED) */
