/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * file  adk_lttng_trace.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * brief
 *
 * This file contains declarations of functions that are
 * part of LTTng implementation in ADK. This file needs to be included to access
 * APIs defined by ADK Lttng.
 *
 ***********************************************************************/

#ifndef _ADK_LTTNG_TRACE_H_
#define _ADK_LTTNG_TRACE_H_

#include "adk_platform.h"


/*Function prototypes*/

#define MAX_LEN 1000

#define FN_NAME(module_name, trace_event) \
    adk_lttng_trace_##module_name##_##trace_event

#define FN_NAME_INTERNAL(module_name, trace_event) \
    adk_lttng_trace_internal_##module_name##_##trace_event

#define TRACE_INFO_INDEX                0
#define TRACE_FORCE_INFO_INDEX          1
#define TRACE_ERROR_INDEX               2
#define TRACE_FORCE_ERROR_INDEX         3
#define TRACE_FATAL_ERROR_INDEX         4
#define TRACE_FORCE_FATAL_INDEX         5
#define TRACE_WARNING_INDEX             6
#define TRACE_FORCE_WARNING_INDEX       7
#define TRACE_FUNC_ENTRY_INDEX          8
#define TRACE_FUNC_EXIT_INDEX           9
#define TRACE_INTERNAL_FUNC_ENTRY_INDEX 10
#define TRACE_INTERNAL_FUNC_EXIT_INDEX  11
#define TRACE_FORCE_FUNC_ENTRY_INDEX    12
#define TRACE_FORCE_FUNC_EXIT_INDEX     13
#define TRACE_HW_INDEX                  14
#define TRACE_LOCK_ACQUIRE_INDEX        15
#define TRACE_LOCK_RELEASE_INDEX        16

#define MAX_TRACE_EVENTS                17

#define ADK_DECLARE_SIF_LOCK_UNLOCK_FN_PROTO(module_name, event) \
adk_status_t adk_lttng_trace_##module_name##_##event \
(const adk_module_handle_t handle, const int32_t lineNo,const char* fileName, const char* fmt, ...);

#define ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(module_name, event) \
adk_status_t adk_lttng_trace_##module_name##_##event( \
    const adk_module_handle_t handle, \
    const int32_t lineNo, \
    const char* fileName, \
    const char* fmt, \
    ...);

#define ADK_DECLARE_FN_PROTO(module_name, event) \
adk_status_t adk_lttng_trace_##module_name##_##event( \
    const adk_module_handle_t handle, \
    const int32_t lineNo, \
    const char* fileName, \
    const char* fmt, \
    ...);

#define ADK_DECLARE_INTERNAL_FN_PROTO(module_name, event) \
adk_status_t adk_lttng_trace_internal_##module_name##_##event( \
    const adk_module_handle_t handle, \
    const int32_t lineNo, \
    const char* fileName, \
    const char* fmt, \
    ...);

#define ADK_DECLARE_FORCE_ENTRY_EXIT_FN_PROTO(module_name, event) \
adk_status_t adk_lttng_trace_force_##module_name##_##event( \
    const adk_module_handle_t handle, \
    const int32_t lineNo, \
    const char* fileName, \
    const char* fmt, \
    ...);

#define ADK_DECLARE_FN_PROTOTYPES(__module_name__) \
    ADK_DECLARE_FN_PROTO(__module_name__, info) \
    ADK_DECLARE_FN_PROTO(__module_name__, force_info) \
    ADK_DECLARE_FN_PROTO(__module_name__, error) \
    ADK_DECLARE_FN_PROTO(__module_name__, force_error) \
    ADK_DECLARE_FN_PROTO(__module_name__, fatal_error) \
    ADK_DECLARE_FN_PROTO(__module_name__, force_fatal) \
    ADK_DECLARE_FN_PROTO(__module_name__, warning) \
    ADK_DECLARE_FN_PROTO(__module_name__, force_warning) \
    ADK_DECLARE_FN_PROTO(__module_name__, func_entry) \
    ADK_DECLARE_FN_PROTO(__module_name__, func_exit) \
    ADK_DECLARE_INTERNAL_FN_PROTO(__module_name__, func_entry) \
    ADK_DECLARE_INTERNAL_FN_PROTO(__module_name__, func_exit) \
    ADK_DECLARE_FN_PROTO(__module_name__, log_hw) \
    ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(__module_name__, lock_acquire) \
    ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(__module_name__, lock_release)

ADK_DECLARE_FN_PROTO(sif, info)
ADK_DECLARE_FN_PROTO(sif, force_info)
ADK_DECLARE_FN_PROTO(sif, error)
ADK_DECLARE_FN_PROTO(sif, force_error)
ADK_DECLARE_FN_PROTO(sif, fatal_error)
ADK_DECLARE_FN_PROTO(sif, force_fatal)
ADK_DECLARE_FN_PROTO(sif, warning)
ADK_DECLARE_FN_PROTO(sif, force_warning)
ADK_DECLARE_FN_PROTO(sif, func_entry)
ADK_DECLARE_FN_PROTO(sif, func_exit)
ADK_DECLARE_FORCE_ENTRY_EXIT_FN_PROTO(sif, func_entry)
ADK_DECLARE_FORCE_ENTRY_EXIT_FN_PROTO(sif, func_exit)
ADK_DECLARE_INTERNAL_FN_PROTO(sif, func_entry)
ADK_DECLARE_INTERNAL_FN_PROTO(sif, func_exit)
ADK_DECLARE_FN_PROTO(sif, log_hw)
ADK_DECLARE_SIF_LOCK_UNLOCK_FN_PROTO(sif, lock_acquire)
ADK_DECLARE_SIF_LOCK_UNLOCK_FN_PROTO(sif, lock_release)

#if defined(ADK_MODULE_IP_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(ip)
#endif

#if defined(ADK_MODULE_IPV6_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(ipv6)
#endif

#if defined(ADK_MODULE_IPSEC_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(ipsec)
#endif

#if defined(ADK_MODULE_ETHERNET_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(ethernet)
#endif

#if defined(ADK_MODULE_GTPU_ENABLED)
ADK_DECLARE_FN_PROTO(gtpu, error)
ADK_DECLARE_FN_PROTO(gtpu, func_entry)
ADK_DECLARE_FN_PROTO(gtpu, func_exit)
ADK_DECLARE_FN_PROTO(gtpu, info)
ADK_DECLARE_INTERNAL_FN_PROTO(gtpu, func_entry)
ADK_DECLARE_INTERNAL_FN_PROTO(gtpu, func_exit)
ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(gtpu, lock_acquire)
ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(gtpu, lock_release)
#endif

#if defined(ADK_MODULE_BFD_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(bfd)
#endif


#if defined(ADK_MODULE_L2TPV3_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(l2tpv3)
#endif

#if defined(ADK_MODULE_EHSFP_IWF_ENABLED)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, error)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, fatal_error)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, func_entry)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, func_exit)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, info)
ADK_DECLARE_FN_PROTO(ehsfp_iwf, warning)
ADK_DECLARE_INTERNAL_FN_PROTO(ehsfp_iwf, func_entry)
ADK_DECLARE_INTERNAL_FN_PROTO(ehsfp_iwf, func_exit)
ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(ehsfp_iwf, lock_acquire)
ADK_DECLARE_LOCK_UNLOCK_FN_PROTO(ehsfp_iwf, lock_release)
#endif

#if defined(ADK_MODULE_ETH_OAM_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(eth_oam)
#endif

#if defined(ADK_MODULE_TWAMP_ENABLED)
ADK_DECLARE_FN_PROTOTYPES(twamp)
#endif


#define ADK_LTTNG_PTR_GET(__module_id, __index, __fn_ptr) \
        adk_lttng_fn_ptr_get((__module_id), (__index), (__fn_ptr))


typedef adk_status_t
(* adk_lttng_trace_fn_ptr)(const adk_module_handle_t,
                           const int32_t,
                           const char*,
                           const char*,
                           ...);

#define ADK_LTTNG_COOKIE    0x820dc1a8

typedef struct adk_lttng_info_s {
    uint32_t    cookie;
    adk_lttng_trace_fn_ptr  trace_fn_ptr[ADK_MAX_SUPPORTED_MODULES][MAX_TRACE_EVENTS];
} adk_lttng_info_t;

void adk_lttng_fn_ptr_get(uint32_t moduleId,
                     uint32_t index,
                     adk_lttng_trace_fn_ptr * const lttngFnPtr_p);


#endif /* #ifndef _ADK_LTTNG_TRACE_H_ */
