/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_qos_dp_export.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos QoS Module
 * @{
 *
 * @brief  This file contains ADK QoS handles
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_DP_EXPORT_H__
#define __ADK_QOS_DP_EXPORT_H__

#include <rte_meter.h>

/*!
 * PDU has been colored RED by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define ADK_QOS_DP_RED    e_RTE_METER_RED
/*!
 * PDU has been colored YELLOW by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define ADK_QOS_DP_YELLOW e_RTE_METER_YELLOW
/*!
 * PDU has been colored GREEN by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define ADK_QOS_DP_GREEN  e_RTE_METER_GREEN

typedef struct adk_qos_dp_process_data_s{
    adk_sys_handle_t sysHdl;
    uint32_t qosInstance;
    uint8_t loop;
} adk_qos_dp_process_data_t;

adk_status_t adk_qos_dp_init(void);

typedef uint16_t adk_qos_queue_id_t;
/*!
 * @addtogroup _qosFunctions QoS Functions
 * @{
 */

adk_dp_status_t
adk_qos_dp_policing_trtcm_color_blind(
                uint32_t policing_flow_id,
                uint32_t packetSize, uint8_t * policeColor);

adk_dp_status_t
adk_qos_dp_policing_srtcm_color_blind(
                uint32_t policing_flow_id,
                uint32_t packetSize, uint8_t * policeColor);

adk_dp_status_t
adk_qos_dp_policing_srtcm_color_blind_pps(
                uint32_t    policing_flow_id,
                uint8_t * policeColor);

adk_dp_status_t
adk_qos_dp_policing_trtcm_color_blind_pps(
                uint32_t    policing_flow_id,
                uint8_t * policeColor);

adk_dp_status_t
adk_qos_dp_policing_color_blind(
                uint32_t policing_flow_id,
                uint32_t packetSize, uint8_t * policeColor);

inline adk_qos_queue_id_t adk_qos_dp_get_default_tx_queue(void);
inline adk_qos_queue_id_t adk_qos_dp_get_ptp_queue(void);
inline void adk_qos_dp_increment_phb_counters(uint8_t ipVersion,
                                       uint16_t statsId,
                                       uint32_t ipLength,
                                       bool pktDropped);

inline void adk_qos_dp_increment_sl_queue_counters(bool isIPPacket,
                                            uint8_t ipVersion,
                                            uint16_t statsId,
                                            uint32_t ipLength,
                                            bool pktDropped);

inline uint32_t adk_qos_dp_get_queue_stats_id(uint16_t txQueueId,
                                              bool *isPHBQueue);
#endif /* __ADK_QOS_DP_EXPORT_H__ */

/*!
 *   @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 */
