/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_ds_profiler_include.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos  QoS Module
 * @{
 *
 *
 * This file contains ADK QoS DiffServ Profiler public data structures,
 * macros and API signatures
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_DS_PROFILER_INCLUDE_H__
#define __ADK_QOS_DS_PROFILER_INCLUDE_H__

#include "adk_sys.h"
#include "adk_qos_include.h"


/*
 * DiffServ Enums        {{{1
 */
/*!
 * =============================================================
 * @enum adk_qos_ds_phb_type_t    {{{2
 *
 *
 * The adk_qos_ds_phb_type_t enum is used to determine the QoS Pre Hop
 * Behavior type
 *
 * =============================================================
 */
typedef enum adk_qos_ds_phb_type_e {
    ADK_QOS_DS_PHB_MIN = 0,
    ADK_QOS_DS_PHB_EF,
    ADK_QOS_DS_PHB_AF,
    ADK_QOS_DS_PHB_BE,
    ADK_QOS_DS_PHB_MAX
} adk_qos_ds_phb_type_t;   /* ------  end of enum adk_qos_ds_phb_type_t  ------ */

/*!
 * =============================================================
 * @enum adk_qos_ds_profile_type_t    {{{2
 *
 *
 * The adk_qos_ds_profile_type_t enum is used to determine the DiffServ Profile
 * type
 *
 * =============================================================
 */
typedef enum adk_qos_ds_profile_type_e {
    ADK_QOS_DS_MIN = 0,
    ADK_QOS_DS_STANDARD,
    ADK_QOS_DS_ADAPTIVE,
    ADK_QOS_DS_MAX
} adk_qos_ds_profile_type_t;   /* ------  end of enum adk_qos_ds_profile_type_t  ------ */
/*!
 * =============================================================
 * @enum adk_qos_drop_latency_type_t    {{{2
 *
 *
 * The adk_qos_drop_latency_type_t enum is used to determine 
 * the drop latency to use
 *
 * =============================================================
 */
typedef enum adk_qos_drop_latency_type_e {
    EIOA_MTM_DEQUEUE_LATENCY = 1,
    MTM_ENQUEUE_DEQUEUE_LATENCY
} adk_qos_drop_latency_type_t;   /* ------  end of enum adk_qos_drop_latency_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_ds_class_drop_policy_t   {{{2
 *
 *
 * The adk_qos_ds_class_drop_policy_t enum defines the Buffer Management
 * drop algorithm to use.
 *
 * =============================================================
 */
typedef enum adk_qos_ds_class_drop_policy_e {
    ADK_BM_DS_TAIL_DROP = 1,
    ADK_BM_DS_TAIL_DROP_NORATEMATCH,
    ADK_BM_DS_RED,
    ADK_BM_DS_RED_NORATEMATCH,
    ADK_BM_DS_WRED,
    ADK_BM_DS_PKTS_TAIL_DROP,
    ADK_BM_DS_MINQ_TAIL_DROP,
    ADK_BM_DS_MINQ_RED
} adk_qos_ds_class_drop_policy_t;   /* ------  end of enum adk_qos_ds_class_drop_policy_t  ------ */


/*
 * DiffServ Structs        {{{1
 */
/*!
 * =============================================================
 * @struct adk_qos_ds_shapingRate_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * The adk_qos_ds_shapingRate_t data structure defines shaping rates for
 * the PHBs and the classes.
 *
 * When shaping mode is ADK_QOS_PR, the shapingRate should be peak.
 * When shaping mode is ADK_QOS_PR_MR (committed), the shapingRate should be 
 * peak for PR, and committed for MR. The mbs and maxPacketSize has to be
 * set for PR shaping mode.
 *
 * =============================================================
 */

typedef struct adk_qos_ds_shapingRate_s {
    uint64_t    peak;
    uint64_t    committed;
    uint32_t    mbs;
    uint32_t    maxPacketSize;
} adk_qos_ds_shapingRate_t;    /* ------  end of struct adk_qos_ds_shapingRate_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_ds_shaping_mode_t
 *
 *
 * The adk_qos_ds_shaping_mode_t data structure is used to determine 
 * the type of shaping required for a given queue or scheduler
 *
 * =============================================================
 */
typedef enum adk_qos_ds_shaping_mode_e {
    ADK_QOS_DS_HW_PR = 0,
    ADK_QOS_DS_HW_PR_SR,
    ADK_QOS_DS_HW_PR_MR,
    ADK_QOS_DS_SCRIPT_PPS,
    ADK_QOS_DS_SCRIPT,
    ADK_QOS_DS_NO_SHAPING
} adk_qos_ds_shaping_mode_t;   /* ------  end of enum adk_qos_ds_shaping_mode_t  ------ */


/*
 * DiffServ Structs        {{{1
 */
/*!
 * =============================================================
 * @struct adk_qos_ds_pps_shaping_parms_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * The adk_qos_ds_pps_shaping_parms_t data structure defines packets shaping rates for
 * the PHBs and the classes.
 *
 *
 * =============================================================
 */

typedef struct adk_qos_ds_pps_shaping_parms_s {
    adk_qos_ds_shaping_mode_t         trafficShapingScript;
    uint32_t                        burstSize;
    uint32_t                        peakRate;
} adk_qos_ds_pps_shaping_parms_t;    /* ------  end of struct adk_qos_ds_pps_shaping_parms_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_drop_latency_policy_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"  {{{2
 *
 *
 * The adk_qos_drop_latency_policy_t data structure defines the drop latency
 * information. 
 *
 * =============================================================
 */
typedef struct adk_qos_drop_latency_policy_s {
    bool                      enable;
    adk_qos_drop_latency_type_t     type;
    uint32_t                    time;
} adk_qos_drop_latency_policy_t;    /* ------  end of struct adk_qos_drop_latency_policy_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ds_bm_pkt_tail_config_params_t adk_qos_ds_profiler_include.h 
 * "adk_qos_ds_profiler_include.h"    {{{2
 *
 *
 * Data structure for the Buffer Management tail drop algorithm
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_pkt_tail_config_params_s {
    uint32_t       dropThreshold; /*!< Drop threshold in byte */
} adk_qos_ds_bm_pkt_tail_config_params_t;    
/* ------  end of struct adk_qos_ds_bm_pkt_tail_config_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ds_bm_tail_config_params_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"    {{{2
 *
 *
 * Data structure for the Buffer Management tail drop algorithm
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_tail_config_params_s {
    uint32_t       dropThreshold; /*!< Drop threshold in bytes */
} adk_qos_ds_bm_tail_config_params_t;    /* ------  end of struct adk_qos_ds_bm_tail_config_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ds_bm_tail_config_params_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"    {{{2
 *
 *
 * Data structure for the Buffer Management tail drop algorithm
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_minq_tail_config_params_s {
    uint32_t       dropThreshold; /*!< Drop threshold in bytes */
    uint32_t       minThreshold; /*!< minimum queue Length as designed by Application in bytes */
} adk_qos_ds_bm_minq_tail_config_params_t;    /* ------  end of struct adk_qos_ds_bm_tail_config_params_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_ds_bm_red_config_params_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"  {{{2
 *
 *
 * Data structure for the Buffer Management RED algorithm
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_red_config_params_s {
    uint32_t    minThreshold; /*!< Minimum thresold, blocks, where PDUs can begin to be dropped */
    uint32_t    maxThreshold; /*!< Maximum thresold, in blocks, where all PDUs will be dropped */
    uint32_t    max_queue_length; /*!< Maximum queue thresold, in bytes where all PDUs will be dropped */
    uint8_t     queueSizeAvgFactor;    
} adk_qos_ds_bm_red_config_params_t;    /* ------  end of struct adk_qos_ds_bm_red_config_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ds_bm_minq_red_config_params_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"  {{{2
 *
 *
 * Data structure for the Buffer Management RED algorithm
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_minq_red_config_params_s {
    uint32_t    minThreshold; /*!< Minimum thresold, blocks, where PDUs can begin to be dropped */
    uint32_t    maxThreshold; /*!< Maximum thresold, in blocks, where all PDUs will be dropped */
    uint32_t    min_queue_length; /*!< Minimum queue thresold, in percent for BM */
    uint32_t    max_queue_length; /*!< Maximum queue thresold, in bytes where all PDUs will be dropped */
    uint8_t     queueSizeAvgFactor;    
} adk_qos_ds_bm_minq_red_config_params_t;    /* ------  end of struct adk_qos_ds_bm_red_config_params_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_ds_bm_wred_config_params_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"    {{{2
 *
 *
 * Data structure for the Buffer Management WRED algorithm. The green
 * thresholds are used on PDUs marked green, and the amber thresholds on PDUs
 * marked amber. All thresholds are in bytes. The minimum threshold is the
 * point below which no PDUs will be dropped. The maximum threshold is the
 * point above wich all PDUs will be dropped.
 *
 * =============================================================
 */
typedef struct adk_qos_ds_bm_wred_config_params_s {
    uint32_t    greenMinThreshold;
    uint32_t    greenMaxThreshold;
    uint32_t    amberMinThreshold;
    uint32_t    amberMaxThreshold;
    uint8_t     queueSizeAvgFactor;
} adk_qos_ds_bm_wred_config_params_t;    /* ------  end of struct adk_qos_ds_bm_wred_config_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ds_class_buffer_mgmt_policy_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"    {{{2
 *
 *
 * Data structure to hold buffer management information like drop
 * thresholds and buffer management schemes
 *
 * =============================================================
 */
typedef struct adk_qos_ds_class_buffer_mgmt_policy_s {
    adk_qos_ds_class_drop_policy_t        dropPolicy;
    union {
        adk_qos_ds_bm_tail_config_params_t         tailConfigParams;
        adk_qos_ds_bm_minq_tail_config_params_t    minQtailConfigParams;
        adk_qos_ds_bm_red_config_params_t          redConfigParams;
        adk_qos_ds_bm_minq_red_config_params_t     minQredConfigParams;
        adk_qos_ds_bm_wred_config_params_t         wredConfigParams;
        adk_qos_ds_bm_pkt_tail_config_params_t     pktTailDropConfigParams;
    }params_u;
} adk_qos_ds_class_buffer_mgmt_policy_t;    /* ------  end of struct adk_qos_ds_class_buffer_mgmt_policy_t  ------ */



/*!
 * =============================================================
 * @struct adk_qos_ds_user_flow_data_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * Data structure to hold adaptive DiffServ class user flow information.
 *
 * Note: currently latency based drops are not supported.
 *       So the 'dropLatency' variable will not have any effect on the 
 *       user profile.
 * =============================================================
 */
typedef struct adk_qos_ds_user_flow_data_s {
    adk_qos_ds_phb_type_t                   classType;
    uint32_t                            classId;
    adk_qos_ds_shaping_mode_t               shapingMode;
    adk_qos_rates_t                         shapingRates;
    adk_qos_drop_latency_policy_t           dropLatency;
    adk_qos_ds_class_buffer_mgmt_policy_t   bufferMgmtPolicy;   
} adk_qos_ds_user_flow_data_t;    /* ------  end of struct adk_qos_ds_user_flow_data_t ------ */
#if 0

/*!
 * =============================================================
 * @struct adk_qos_ds_adpt_class_profile_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * Data structure to hold Adaptive DiffServ PHB class profiles. The classId is
 * the same as the array index in adptClass_p in adk_qos_ds_phb_t.
 *
 * =============================================================
 */
typedef struct adk_qos_ds_adpt_class_profile_s {
    adk_qos_scheduling_mode_t               schedulingMode;
    adk_qos_ds_shaping_mode_t               shapingMode;
    union{
        adk_qos_rates_t                     shapingRates;
        adk_qos_ds_pps_shaping_parms_t      ppsShapingParms;
    } u;
    adk_qos_service_quantum_t               serviceQuantum;
} adk_qos_ds_adpt_class_profile_t;    /* ------  end of struct adk_qos_ds_adpt_class_profile_t  ------ */
#endif
/*!
 * =============================================================
 * @struct adk_qos_ds_class_profile_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * Data structure to hold DiffServ PHB class profiles. The classId is
 * the same as the array index in stdClass_p in adk_qos_ds_phb_t .
 *
 * Note: currently latency based drops are not supported.
 *       So the 'dropLatency' variable will not have any effect on the class
 *       profile.*
 * =============================================================
 */
typedef struct adk_qos_ds_class_profile_s {
    adk_qos_ds_shaping_mode_t               shapingMode;
    union{
        adk_qos_rates_t                     shapingRates;
        adk_qos_ds_pps_shaping_parms_t      ppsShapingParms;
    } u;
    /*! @brief  The hierarchy level from which to get the second set of
     *          scheduler parameters for the traffic management script
     *          Used for queues only
     */
    uint8_t                       tmParamsRelativeHigherLevel;
    /*! @brief  The hierarchy level from which to get the second set of
     *          scheduler parameters for the traffic shaping script.
     *          Used for queues only
     */
    uint8_t                       tsParamsRelativeHigherLevel; 
    adk_qos_service_quantum_t               serviceQuantum;
    adk_qos_drop_latency_policy_t           dropLatency;
    adk_qos_ds_class_buffer_mgmt_policy_t   bufferMgmtPolicy;   
} adk_qos_ds_class_profile_t;    /* ------  end of struct adk_qos_ds_class_profile_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_ds_phb_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * Data structure to hold DiffServ Per Hop Behavior profile
 *
 * =============================================================
 */
typedef struct adk_qos_ds_phb_s {
    adk_qos_ds_phb_type_t               phbType; 
    uint16_t                        numClasses;
    adk_qos_service_quantum_t           serviceQuantum;
    adk_qos_ds_shaping_mode_t           shapingMode;
    union{
        adk_qos_rates_t                 shapingRates;
        adk_qos_ds_pps_shaping_parms_t  ppsShapingParms;
    } rates;
    union{
        adk_qos_ds_class_profile_t        *stdClass_p;
    } u;
} adk_qos_ds_phb_t;    /* ------  end of struct adk_qos_ds_phb_t  ------ */



/*!
 * =============================================================
 * @struct adk_qos_ds_profile_t adk_qos_ds_profiler_include.h "adk_qos_ds_profiler_include.h"   {{{2
 *
 *
 * Data structure for the DiffServ policy creation
 *
 * =============================================================
 */
typedef struct adk_qos_ds_profile_s {
    adk_qos_ds_profile_type_t   diffServType;
    uint32_t                numPhbGroups;
    adk_handle_t                parentHandle;
    adk_qos_ds_phb_t            *phbGroup_p;
} adk_qos_ds_profile_t;   /* ------  end of struct adk_qos_ds_profile_t  ------ */



/*
 * DiffServ Functions        {{{1
 */
/*!
 *
 *   @addtogroup _qosFunctions  QoS Functions
 *   @{
 */

adk_status_t
adk_qos_ds_policy_init(   /* {{{2 */
    adk_sys_handle_t  sysHandle);

adk_status_t
adk_qos_ds_policy_shutdown(   /* {{{2 */
    adk_sys_handle_t  sysHandle);

adk_status_t
adk_qos_ds_policy_create(     /* {{{2 */
    adk_sys_handle_t             sysHandle,
    const adk_qos_ds_profile_t   *dsProfile_p,
    adk_handle_t                 *dsPolicyHandle_p);

adk_status_t 
adk_qos_ds_policy_delete(     /* {{{2 */
    adk_sys_handle_t   sysHandle,
    adk_handle_t       dsPolicyHandle);

adk_status_t    
adk_qos_ds_policy_user_flow_add(      /* {{{2 */
    adk_sys_handle_t             sysHandle,
    adk_handle_t                 dsPolicyHandle,
    const adk_qos_ds_user_flow_data_t  *usrFlow_p,
    adk_handle_t                 *queueHandle);

adk_status_t    
adk_qos_ds_policy_user_flow_rem(      /* {{{2 */
    adk_sys_handle_t        sysHandle,
    adk_handle_t            queueHandle);

adk_status_t    
adk_qos_ds_policy_queue_get(      /* {{{2 */
    adk_sys_handle_t        sysHandle,
    adk_handle_t            dsPolicyHandle,
    adk_qos_ds_phb_type_t   classType,
    uint16_t            classId,
    adk_handle_t           *queueHandle);


adk_status_t    
adk_qos_ds_policy_queueId_get(    /* {{{2 */
    adk_sys_handle_t        sysHandle,
    adk_handle_t            dsPolicyHandle,
    adk_qos_ds_phb_type_t   classType,
    uint16_t            classId,
    uint32_t           *queueId);


#endif /* __ADK_QOS_DS_PROFILER_INCLUDE_H__ */
 
/*!
 * =============================================================
 * }}}2
 *
 *   @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 */
