/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_include.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos  QoS Module
 * @{
 *
 * This file contains ADK QoS data structures, macros and API signatures
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_INCLUDE_H__
#define __ADK_QOS_INCLUDE_H__

#include "adk_sys.h"
#include <rte_meter.h>
#include <rte_tm.h>
#include <acp_gen_interface.h>

#include <adk_ph_dev.h>
#include "adk_qos_policing_include.h"
#include "adk_qos_queue.h"

#define ADK_QOS_TOP_SCHED       ((adk_handle_t)0)
#define ADK_QOS_TEMP_SCHED      ((adk_handle_t)0)
#define ADK_QOS_GET_QUEUE_ID    0xffffffff
#define ADK_QOS_INVALID_QUEUE_ID    0xffff
#define ADK_QOS_DS_PHB_CLASS_NONE   0xFFFFFFFF
/*
 * DiffServ Enums        {{{1
 */
/*!
 * =============================================================
 * @enum adk_qos_ds_phb_type_t    {{{2
 *
 *
 * The adk_qos_ds_phb_type_t enum is used to determine the QoS Per Hop
 * Behavior type
 *
 * =============================================================
 */
typedef enum adk_qos_ds_phb_type_e {
    ADK_QOS_DS_PHB_MIN = 0,
    ADK_QOS_DS_PHB_EF,
    ADK_QOS_DS_PHB_AF,
    ADK_QOS_DS_PHB_BE,
    ADK_QOS_DS_PHB_MAX
} adk_qos_ds_phb_type_t;   /* ------  end of enum adk_qos_ds_phb_type_t  ------ */

/*!
 * =============================================================
 * @enum adk_qos_ds_phb_class_t    {{{2
 *
 *
 * The adk_qos_ds_phb_class_t enum is used to index the QoS Per Hop
 * Behavior and its classes.
 *
 * =============================================================
 */
typedef enum adk_qos_ds_phb_class_e {
    ADK_QOS_DS_PHB_CLASS_EF_1 = 0,
    ADK_QOS_DS_PHB_CLASS_EF_2,
    ADK_QOS_DS_PHB_CLASS_EF_3,
    ADK_QOS_DS_PHB_CLASS_AF_1,
    ADK_QOS_DS_PHB_CLASS_AF_2,
    ADK_QOS_DS_PHB_CLASS_AF_3,
    ADK_QOS_DS_PHB_CLASS_AF_4,
    ADK_QOS_DS_PHB_CLASS_BE,
} adk_qos_ds_phb_class_t;   /* ------  end of enum adk_qos_ds_phb_class_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_ingress_info_t
 *
 * @brief
 *
 * This data structure contains IP QoS related parameters used
 * to add/delete ingress QOS rules. This data structure is used
 * by API's 
 * - adk_ip_ingress_qos_policy_add/adk_ip_ingress_qos_policy_delete.
 * - adk_ipv6_ingress_qos_policy_add/adk_ipv6_ingress_qos_policy_delete.
 * Note: Default PHB class for any dscp is ADK_QOS_DS_PHB_CLASS_NONE.
 * =============================================================
 */
typedef struct adk_qos_ingress_info_s {
    /*!
     * dscp
     * DSCP bits from IP packet header.
     * DSCP value is expected in lower 6 bits.
     */
    uint8_t dscp;
    /*!
     * PHB class for the above dscp value.
     */
    adk_qos_ds_phb_class_t phbClass;
}adk_qos_ingress_info_t;

/*!
 * =============================================================
 * @struct adk_qos_ingress_stats_s
 *
 * @brief
 *
 * IP data structure for received packets and byte count.
 * This data structure is used to maintain statistics 
 * per IP interface per PHB.
 * =============================================================
 */
typedef struct adk_qos_ingress_stats_s {
    /* Received packet count */
    uint64_t rxPkts;
    /* Received byte count */
    uint64_t rxBytes;
} adk_qos_ingress_stats_t;


/*!
 * =============================================================
 * @struct adk_qos_rl_type_t adk_qos_include.h "adk_qos_include.h"
 *
 * The adk_qos_rl_type_t data structure is to define various ratelimiting types 
 *
 * =============================================================
 */
typedef enum adk_qos_rl_type_e {
    ADK_QOS_RL_TYPE_QUEUE,
    ADK_QOS_RL_TYPE_POLICER,
} adk_qos_rl_type_t;

/*!
 * =============================================================
 * @struct adk_qos_rl_info_t adk_qos_include.h "adk_qos_include.h"
 *
 * The adk_qos_rl_info_t data structure is to define ratelimiting object.
 * =============================================================
 */
typedef struct adk_qos_rl_info_s {
    adk_qos_rl_type_t    type;
    union{
        struct{
            uint16_t    eth_dev_id;
            uint16_t    queue_id;
        };
        adk_qos_policer_handle_t    policer;
    };
} adk_qos_rl_info_t;

/*
* Data Structure that defines various device types 
*/
typedef enum adk_qos_dev_type_e {
    ADK_QOS_DEV_TYPE_ETH,
    ADK_QOS_DEV_TYPE_PH_DEV, 
    ADK_QOS_DEV_TYPE_NET_DEV,
} adk_qos_dev_type_t;

#define ADK_QOS_MAX_PHB_CLASS_IDX (ADK_QOS_DS_PHB_CLASS_BE + 1)

/*!
* =============================================================
* @enum adk_qos_classifier_field_mask_t
*
* @brief
*
* This enum contains the bit mask for various QOS classifier
* parameters.
*
* =============================================================
*/
typedef enum adk_qos_classifier_field_mask_e {
	/*! QOS classifier DSCP  mask */
    ADK_QOS_CLASSIFIER_DSCP_MASK = 0x01,
	/*! QOS classifier protocol type mask */
	ADK_QOS_CLASSIFIER_PROTOCOL_TYPE_MASK = 0x02,
    /*! QOS classifier source IP address mask */
    ADK_QOS_CLASSIFIER_SRC_ADDR_MASK = 0x04,
	/*! QOS classifier destination IP address mask */
    ADK_QOS_CLASSIFIER_DEST_ADDR_MASK = 0x08,
} adk_qos_classifier_field_mask_t; /* ------  end of enum adk_qos_classifier_field_mask_t  ------ */

/*!
 *   @addtogroup _qosFunctions  QoS Functions
 *   @{
 */
/*
 * =============================================================
 * ADK QoS API signatures
 * =============================================================
 */
adk_status_t
adk_qos_init(adk_sys_handle_t  sysHandle);

adk_status_t
adk_qos_shutdown(
    adk_sys_handle_t  sysHandle);

#endif /* __ADK_QOS_INCLUDE_H__ */
 
/*!
 *   @} end addtogroup _qosFunctions
 *
 * @} end addtogroup _qos
 */
