/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_mef_profiler_include.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos  QoS Module
 * @{
 *
 *
 * This file contains ADK QoS MEF Profiler public data structures,
 * macros and API signatures
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_MEF_PROFILER_INCLUDE_H__
#define __ADK_QOS_MEF_PROFILER_INCLUDE_H__

#include "adk_sys.h"
#include "adk_qos_include.h"


/*
 * MEF Macros {{{1
 */
#define ADK_QOS_MEF_NUM_CHILD_SCHED  4
#define ADK_QOS_MEF_MAX_QUEUES  8
#define ADK_QOS_MEF_MAX_POLICE  8
#define ADK_QOS_MEF_MAX_BM_PARAM_LENGTH 16

/*
 * MEF Enums {{{1
 */
/*!
 * =============================================================
 * @enum adk_qos_mef_ingress_get_type_t {{{2
 *
 *
 * The different types of data that can be returned by the Get API
 *
 * =============================================================
 */
typedef enum adk_qos_mef_ingress_get_type_e {
    ADK_QOS_MEF_INGRESS_GET_POLICY_INFO = 1 /*!<  */
} adk_qos_mef_ingress_get_type_t;   /* ------  end of enum adk_qos_mef_ingress_get_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_egress_get_type_t {{{2
 *
 *
 * The different types of data that can be returned by the Get API
 *
 * =============================================================
 */
typedef enum adk_qos_mef_egress_get_type_e {
    ADK_QOS_MEF_EGRESS_GET_POLICY_INFO = 1 /*!<  */
} adk_qos_mef_egress_get_type_t;   /* ------  end of enum adk_qos_mef_egress_get_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_color_mode_t {{{2
 *
 *
 * The adk_qos_mef_color_mode_t data structure is used to determine
 * if the policing and buffer management algorithms are color blind or color
 * aware.
 *
 * =============================================================
 */
typedef enum adk_qos_mef_color_mode_e {
    ADK_QOS_MEF_COLOR_BLIND = 1,
    ADK_QOS_MEF_COLOR_AWARE
} adk_qos_mef_color_mode_t;   /* ------  end of enum adk_qos_mef_color_mode_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_bm_drop_policy_t {{{2
 *
 *
 * The adk_qos_mef_bm_drop_policy_t data structure is used to determine
 * the Buffer Manager algorithm to use
 *
 * =============================================================
 */
typedef enum adk_qos_mef_bm_drop_policy_e {
    ADK_QOS_MEF_BM_TAIL_DROP = 1,
    ADK_QOS_MEF_BM_RED,
    ADK_QOS_MEF_BM_WRED,
    ADK_QOS_MEF_BM_RAW 
} adk_qos_mef_bm_drop_policy_t;   /* ------  end of enum adk_qos_mef_bm_drop_policy_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_bucket_type_t {{{2
 *
 *
 * The adk_qos_mef_bucket_type_t data structure is used to determine
 * the Buffer Manager RED or WRED or the policing uses either single or dual
 * leaky bucket algorithem.
 *
 * =============================================================
 */
typedef enum adk_qos_mef_bucket_type_e {
    ADK_QOS_MEF_LEAKY_BUCKET_NONE = 1,
    ADK_QOS_MEF_LEAKY_BUCKET_DUAL,
    ADK_QOS_MEF_LEAKY_BUCKET_SINGLE
} adk_qos_mef_bucket_type_t;   /* ------  end of enum adk_qos_mef_bucket_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_profile_type_t {{{2
 *
 *
 * The adk_qos_mef_profile_type_t data structure is used to determine
 * the type of node for the MEF.
 *
 * =============================================================
 */
typedef enum adk_qos_mef_profile_type_e {
    ADK_QOS_MEF_NODE_UNI = 1,
    ADK_QOS_MEF_NODE_EVC,
    ADK_QOS_MEF_NODE_COS
} adk_qos_mef_profile_type_t;   /* ------  end of enum adk_qos_mef_profile_type_t  ------ */


/*!
 * =============================================================
 * @enum adk_qos_mef_scheduling_mode_t {{{2
 *
 *
 * This enum defines the scheduling of the Node. There
 * are 8 Strict Priority positions, a SDWRR, and a Best Effort. The
 * Strict Priority positions have a higher priority than the SDWRR/DWRR, and
 * the Best Effort is the lowest priority scheduler.
 *
 * =============================================================
 */
typedef enum adk_qos_mef_scheduling_mode_s {
    ADK_QOS_MEF_NODE_SP_0 = 1, /*!< Hightest priority of the Strict Priority schedulers */
    ADK_QOS_MEF_NODE_SP_1,
    ADK_QOS_MEF_NODE_SP_2,
    ADK_QOS_MEF_NODE_SP_3,
    ADK_QOS_MEF_NODE_SP_4,
    ADK_QOS_MEF_NODE_SP_5,
    ADK_QOS_MEF_NODE_SP_6,
    ADK_QOS_MEF_NODE_SP_7, /*!< Lowest Priority of the Strict Priority schedulers */
    ADK_QOS_MEF_NODE_SDWRR,/*!< SDWRR scheduler */
    ADK_QOS_MEF_NODE_BE /*!< Best Effort scheduler */
} adk_qos_mef_scheduling_mode_t;   /* ------  end of enum adk_qos_mef_scheduling_mode_t ------ */


/*
 * MEF Structures {{{1
 */
/*!
 * =============================================================
 * @struct adk_qos_mef_police_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold Tail configuration parameters
 *
 * =============================================================
 */
typedef struct adk_qos_mef_police_s {
    adk_qos_mef_bucket_type_t   type;   /*!< The type of policing algorthem to use: none, single leaky bucket, or dual leaky bucket  */
    uint64_t                cir;    /*!< Committed Information Rate  */
    uint32_t                cbs;    /*!< Committed Burst Size  */
    uint64_t                eir;    /*!< Excess Information Rate  */
    uint32_t                ebs;    /*!< Excess Burst Size  */
    uint8_t                 couplingFlag;   /*!< Coupling Flag determines the maximum long-term information rate. When set to 0, the maximum rate is bound by EIR. When set to 1, the maximum rate is bound by EIR+CIR  */
    adk_qos_mef_color_mode_t    colorMode;      /*!< colorMode determines if the algorithm used is color-blind or color-aware. Color-aware, implies multiple thresholds, verses a single threshold for color-blind mode  */
} adk_qos_mef_police_t;    /* ------  end of struct adk_qos_mef_police_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_shaper_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold Tail configuration parameters
 *
 * =============================================================
 */
typedef struct adk_qos_mef_shaper_s {
    adk_qos_mef_bucket_type_t   type;   /*!< The type of policing algorthem
                                             to use: single leaky bucket or
                                             dual leaky bucket  */
    uint64_t                cir;    /*!< Committed Information Rate  */
    uint32_t                cbs;    /*!< Committed Burst Size  */
    uint64_t                eir;    /*!< Excess Information Rate  */
    uint32_t                ebs;    /*!< Excess Burst Size  */
    uint32_t                maxPacketSize; /*!< Maximum packet seen on this interface */
} adk_qos_mef_shaper_t;    /* ------  end of struct adk_qos_mef_shaper_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_bm_tail_drop_color_blind_params_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 * Data structure to hold the tail drop thresholds.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_bm_tail_drop_color_blind_params_s {
    uint32_t    dropThreshold;
} adk_qos_mef_bm_tail_drop_color_blind_params_t;   /* ------  end of struct adk_qos_mef_bm_tail_drop_color_blind_params_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_bm_tail_drop_color_aware_params_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 * Data structure to hold the tail drop thresholds.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_bm_tail_drop_color_aware_params_s {
    uint32_t    greenDropThreshold;
    uint32_t    yellowDropThreshold;
    uint32_t    redDropThreshold;
} adk_qos_mef_bm_tail_drop_color_aware_params_t;   /* ------  end of struct adk_qos_mef_bm_tail_drop_color_aware_params_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_bm_red_params_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold RED configuration parameters
 *
 * =============================================================
 */
typedef struct adk_qos_mef_bm_red_config_params_s {
    uint32_t    minThreshold;
    uint32_t    maxThreshold;
    uint8_t     queueSizeAvgFactor;
} adk_qos_mef_bm_red_params_t;    /* ------  end of struct adk_qos_mef_bm_red_params_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_bm_wred_params_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold WRED configuration parameters
 *
 * =============================================================
 */
typedef struct adk_qos_mef_bm_wred_policy_s {
    uint32_t    greenMinThreshold;
    uint32_t    greenMaxThreshold;
    uint32_t    yellowMinThreshold;
    uint32_t    yellowMaxThreshold;
    uint8_t     queueSizeAvgFactor;
} adk_qos_mef_bm_wred_params_t;    /* ------  end of struct adk_qos_mef_bm_wred_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_mef_bm_wred_ramp_params_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold WRED RAMP configuration parameters
 *
 * =============================================================
 */
typedef struct adk_qos_mef_bm_wred_ramp_policy_s {
   uint16_t                   greenMinThreshold;
   uint16_t                   greenMaxThreshold;
   uint16_t                   yellowMinThreshold;
   uint16_t                   yellowMaxThreshold;
   double                         greenDP0;
   double                         yellowDP0;
   bool                     isIfEnabled;
} adk_qos_mef_bm_wred_ramp_params_t;    /* ------  end of struct adk_qos_mef_bm_wred_ramp_params_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_mef_buffer_management_policy_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold buffer management information like drop
 * thresholds and buffer management schemes
 *
 * =============================================================
 */
typedef struct adk_qos_mef_buffer_mgmt_policy_s {
    adk_qos_mef_bm_drop_policy_t        dropPolicy;
    adk_qos_mef_color_mode_t            colorMode;
    union {
        adk_qos_mef_bm_tail_drop_color_blind_params_t   tailDropColorBlind;
        adk_qos_mef_bm_tail_drop_color_aware_params_t   tailDropColorAware;
        adk_qos_mef_bm_red_params_t                     redConfigParams;
        adk_qos_mef_bm_wred_params_t                    wredConfigParams;
        adk_qos_mef_bm_wred_ramp_params_t               wredRampConfigParams;
        unsigned char                rawParamBuff[ADK_QOS_MEF_MAX_BM_PARAM_LENGTH];
    } bm_u;
} adk_qos_mef_buffer_management_policy_t;    /* ------  end of struct adk_qos_mef_buffer_management_policy_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_scheduler_attributes_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 * Data structure to hold the attribute parameters for configuring the
 * queues.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_scheduler_attributes_s {
    adk_qos_mef_scheduling_mode_t   mode;  /*!< Arbitration mode: Strict, SDWRR,
                                            *   DWRR
                                            */
    uint32_t                    weight;
    adk_qos_service_quantum_t       serviceQuantum;
} adk_qos_mef_scheduler_attributes_t;   /* ------  end of struct adk_qos_mef_scheduler_attributes_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_queue_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 * Main data structure for configuring the queues
 *
 * =============================================================
 */
typedef struct adk_qos_mef_queue_s {
    adk_qos_mef_buffer_management_policy_t  bufferMgmt;
    adk_qos_mef_scheduler_attributes_t      scheduler;
    adk_qos_mef_shaper_t                    shaper;
} adk_qos_mef_queue_t;   /* ------  end of struct adk_qos_mef_queue_t  ------ */


#if 0
** /*!
**  * =============================================================
**  * @struct adk_qos_mef_traffic_profile_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
**  *
**  *
**  * =============================================================
**  */
** typedef struct adk_qos_mef_traffic_profile_s {
**     adk_qos_shaping_mode_t                      shapingMode;
**     adk_qos_scheduling_mode_t                   schedulingMode;
**     adk_qos_rates_t                             shapingRates;
**     adk_qos_service_quantum_t                   serviceQuantum;
** /*    adk_qos_drop_latency_policy_t               dropLatency;*/
**     adk_qos_mef_buffer_management_policy_t      bufferMgmtPolicy;
** } adk_qos_mef_traffic_profile_t;    /* ------  end of struct adk_qos_mef_traffic_profile_t  ------ */
#endif


/*!
 * =============================================================
 * @struct adk_qos_mef_ingress_profile_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold ingress MEF profile. This is the policing
 * information.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_ingress_profile_s {
    uint32_t            numPoliceFlows;
    adk_qos_mef_police_t   *policeParams;
} adk_qos_mef_ingress_profile_t;    /* ------  end of struct adk_qos_mef_ingress_profile_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_ingress_get_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold ingress MEF Ingress Get function input get type as
 * well as the return data.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_ingress_get_s {
    adk_qos_mef_ingress_get_type_t  type;
    union {
        adk_qos_mef_ingress_profile_t   profile;
    } get_u; 
} adk_qos_mef_ingress_get_t;    /* ------  end of struct adk_qos_mef_ingress_get_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_mef_egress_profile_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold egress MEF profile. This is the buffer management,
 * shaping, and scheduling information.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_egress_profile_s {
    adk_handle_t                            parentHandle;
    adk_qos_mef_profile_type_t              type;
    adk_qos_mef_scheduler_attributes_t      scheduler;
    adk_qos_mef_shaper_t                    shaper;
    uint32_t                            numQueues;
    adk_qos_mef_queue_t                    *queues;
} adk_qos_mef_egress_profile_t;    /* ------  end of struct adk_qos_mef_egress_profile_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_mef_egress_policy_handle_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Handle to hold policy information
 *
 * =============================================================
 */
typedef struct adk_qos_mef_egress_policy_handle_s {
    adk_handle_t    policyHandle;
    uint32_t    numQueues;
    adk_handle_t   *queueHandles_p[];
} adk_qos_mef_egress_policy_handle_t;

/*!
 * =============================================================
 * @struct adk_qos_mef_egress_get_t adk_qos_mef_profiler_include.h "adk_qos_mef_profiler_include.h" {{{2
 *
 *
 * Data structure to hold egress MEF Ingress Get function input get type as
 * well as the return data.
 *
 * =============================================================
 */
typedef struct adk_qos_mef_egress_get_s {
    adk_qos_mef_egress_get_type_t  type;
    union {
        adk_qos_mef_egress_profile_t   profile;
    } get_u; 
} adk_qos_mef_egress_get_t;    /* ------  end of struct adk_qos_mef_egress_get_t  ------ */




/*
 * ADK QoS Functions {{{1
 */
/*!
 *   @addtogroup _qosFunctions  QoS Functions
 *   @{
 */
/*
 * MEF System Functions {{{2
 */
adk_status_t
adk_qos_mef_policy_init(
    adk_sys_handle_t  sysHandle);

adk_status_t
adk_qos_mef_policy_shutdown(
    adk_sys_handle_t  sysHandle);

/*
 * MEF Ingress Functions {{{2
 */
adk_status_t
adk_qos_mef_ingress_policy_create(
    adk_sys_handle_t                sysHandle,
    const adk_qos_mef_ingress_profile_t  *policy_p,
    adk_handle_t                   *ingressHandle_p);

adk_status_t
adk_qos_mef_ingress_policy_delete(
    adk_sys_handle_t    sysHandle,
    adk_handle_t        mefPolicyHandle);

adk_status_t
adk_qos_mef_ingress_policy_get(
    adk_sys_handle_t            sysHandle,
    adk_handle_t                mefPolicyHandle,
    adk_qos_mef_ingress_get_t  *getData_p);

adk_status_t
adk_qos_mef_ingress_policy_police_flow_get(
    adk_sys_handle_t     sysHandle,
    adk_handle_t         mefPoliceFlowHandle,
    uint32_t         policeIndex,
    adk_flow_handle_t   *policeFlowHandle);

/*
 * MEF Egress Functions {{{2
 */
adk_status_t
adk_qos_mef_egress_policy_create(
    adk_sys_handle_t                    sysHandle,
    const adk_qos_mef_egress_profile_t       *egressProfile_p,
    adk_handle_t                       *egressHandle_p);

adk_status_t
adk_qos_mef_egress_policy_delete(
    adk_sys_handle_t    sysHandle,
    adk_handle_t        dsPolicyHandle);

adk_status_t
adk_qos_mef_egress_policy_get(
    adk_sys_handle_t            sysHandle,
    adk_handle_t                mefPolicyHandle,
    adk_qos_mef_egress_get_t   *getData_p);

adk_status_t
adk_qos_mef_egress_policy_update(
    adk_sys_handle_t             sysHandle,
    const adk_qos_mef_egress_profile_t *egressProfile_p,
    adk_handle_t                 egressHandle);

adk_status_t
adk_qos_mef_egress_policy_queue_get(
    adk_sys_handle_t        sysHandle,
    adk_handle_t            nodeHandle,
    uint32_t            queueIndex,
    adk_handle_t           *queueHandle_p);

adk_status_t
adk_qos_mef_egress_policy_scheduler_get(
    adk_sys_handle_t        sysHandle,
    adk_handle_t            nodeHandle,
    adk_handle_t           *schedHandle_p);

adk_status_t
adk_qos_mef_egress_policy_queueId_get(
    adk_sys_handle_t        sysHandle,
    adk_handle_t            parentHandle,
    uint32_t            queueIndex,
    adk_handle_t           *queueId_p);

adk_status_t
adk_qos_mef_egress_policy_queue_add(
    adk_sys_handle_t        sysHandle,
    const adk_qos_mef_queue_t    *queue_p,
    adk_handle_t            parentHandle,
    adk_handle_t           *queueHandle_p);

adk_status_t
adk_qos_mef_egress_policy_queue_delete(
    adk_sys_handle_t        sysHandle,
    adk_handle_t            parentHandle,
    adk_handle_t            queueHandle_p);

#if 0
adk_status_t
adk_qos_mef_policy_create(
                             adk_sys_handle_t              sysHandle,
                             uint8_t                   numClassTypes,
                             uint32_t                  parentId,
                             adk_qos_mef_profile_t          *dsProfile_p,
                             adk_qos_mef_policy_handle_t    *dsPolicyHandle_p);

adk_status_t
adk_qos_mef_policy_delete(
                             adk_sys_handle_t            sysHandle,
                             adk_qos_mef_policy_handle_t  dsPolicyHandle);

adk_status_t
adk_qos_mef_policy_queueId_get(
                          adk_sys_handle_t             sysHandle,
                          adk_qos_mef_policy_handle_t   dsPolicyHandle,
                          adk_qos_mef_class_type_t      classType,
                          uint16_t                 classId,
                          uint32_t                 *queueId);
#endif


#endif /* __ADK_QOS_MEF_PROFILER_INCLUDE_H__ */

/*!
 * =============================================================
 * }}}2
 *
 *   @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 */
