/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_policing_profiler_include.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos  QoS Module
 * @{
 *
 * @brief  This file contains ADK QoS Policing profiler data structures,
 *         macros and API signatures
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_POLICING_PROFILER_INCLUDE_H__
#define __ADK_QOS_POLICING_PROFILER_INCLUDE_H__

#include "adk_sys.h"
#include "adk_qos_include.h"

typedef enum adk_qos_policing_type_s
{
    ADK_QOS_ATM_POLICING = 0,
    ADK_QOS_DS_POLICING,
    ADK_QOS_ETHERNET_POLICING
} adk_qos_policing_type_t;


typedef enum adk_qos_ds_policing_type_s
{
    ADK_QOS_DS_POLICING_SRTCM_COLOR_BLIND = 0,
    ADK_QOS_DS_POLICING_SRTCM_COLOR_AWARE,
    ADK_QOS_DS_POLICING_TRTCM_COLOR_BLIND,
    ADK_QOS_DS_POLICING_TRTCM_COLOR_AWARE
} adk_qos_ds_policing_type_t;


typedef struct adk_qos_ds_policing_params_s
{
    adk_qos_ds_policing_type_t      type;
    uint32_t                    flow_id;
    uint32_t                    cir;    /* Committed Information Rate in bps */
    uint32_t                    pir;    /* Peak Information Rate in bps */
    uint32_t                    cbs;    /* Committed Burst Size in bytes */
    uint32_t                    ebs;    /* Peak Burst Size in bytes */
} adk_qos_ds_policing_params_t;


typedef struct adk_qos_policing_s
{
    adk_qos_policing_type_t     type;
    union
    {
        uint32_t                   reserved;
        adk_qos_ds_policing_params_t    dsPolicing;
    } params_u;
} adk_qos_policing_t;

/*!
 *   @addtogroup _qosFunctions  QoS Functions
 *   @{
 */
adk_status_t
adk_qos_ds_policing_policy_init(
                      adk_sys_handle_t  sysHandle);


adk_status_t
adk_qos_policing_policy_create(
        adk_sys_handle_t              sysHandle,
        const adk_qos_policing_t      *policingParams_p,
        uint32_t                  *policingFlowId_p);

adk_status_t
adk_qos_policing_policy_delete(
        adk_sys_handle_t        sysHandle,
        uint32_t            policingFlowId);


/* TODO */
/* Add policing param get API */
/*!   end addtogroup _qosFunctions
 *   @}
 *
 * end addtogroup _qos
 * @}
 */
#endif /* __ADK_QOS_POLICING_PROFILER_INCLUDE_H__ */

