/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_queue.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos  QoS Module
 * @{
 *
 * This file contains ADK QoS data structures, macros and API signatures
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_QUEUE_H__
#define __ADK_QOS_QUEUE_H__

#include "adk_sys.h"

typedef uint16_t adk_qos_queue_id_t;

/*!
 * =============================================================
 * @struct adk_qos_sl_queue_statistics_t
 *
 * @brief
 *
 * This data structure is used to maintain statistics
 * per SL queue.
 * =============================================================
 */
typedef struct adk_qos_sl_queue_statistics_s{
    /* Non IP Transmitted packet count */
    uint64_t nonIPTxPkts;
    /* Non IP Transmitted byte count */
    uint64_t nonIPTxBytes;
    /* Non IP Dropped packet count */
    uint64_t nonIPTxPktsDropped;
    /* Non IP Dropped byte count */
    uint64_t nonIPTxBytesDropped;
    /* IPv4 Transmitted packet count */
    uint64_t ipv4TxPkts;
    /* IPv4 Transmitted byte count */
    uint64_t ipv4TxBytes;
    /* IPv4 Dropped packet count */
    uint64_t ipv4TxPktsDropped;
    /* IPv4 Dropped byte count */
    uint64_t ipv4TxBytesDropped;
    /* IPv6 Transmitted packet count */
    uint64_t ipv6TxPkts;
    /* IPv6 Transmitted byte count */
    uint64_t ipv6TxBytes;
    /* IPv6 Dropped packet count */
    uint64_t ipv6TxPktsDropped;
    /* IPv6 Dropped byte count */
    uint64_t ipv6TxBytesDropped;
} adk_qos_sl_queue_statistics_t;

/*!
 * =============================================================
 * @struct adk_qos_phb_queue_statistics_t
 *
 * @brief
 *
 * This data structure is used to maintain statistics
 * per FL PHB queue.
 * =============================================================
 */
typedef struct adk_qos_phb_queue_statistics_s{
    /* IPv4 Transmitted packet count */
    uint64_t ipv4TxPkts;
    /* IPv4 Transmitted byte count */
    uint64_t ipv4TxBytes;
    /* IPv4 Dropped packet count */
    uint64_t ipv4TxPktsDropped;
    /* IPv4 Dropped byte count */
    uint64_t ipv4TxBytesDropped;
    /* IPv6 Transmitted packet count */
    uint64_t ipv6TxPkts;
    /* IPv6 Transmitted byte count */
    uint64_t ipv6TxBytes;
    /* IPv6 Dropped packet count */
    uint64_t ipv6TxPktsDropped;
    /* IPv6 Dropped byte count */
    uint64_t ipv6TxBytesDropped;
} adk_qos_phb_queue_statistics_t;

adk_status_t
adk_qos_default_queue_id_set(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_default_queue_id_reset(
    adk_sys_handle_t        sysHandle);

adk_status_t
adk_qos_ptp_queue_id_set(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_ptp_queue_id_reset(
    adk_sys_handle_t        sysHandle);

adk_status_t
adk_qos_phb_queue_register(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_phb_queue_unregister(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_get_phb_queue_statistics(
    adk_sys_handle_t                sysHandle,
    adk_qos_queue_id_t              queueId,
    adk_qos_phb_queue_statistics_t  *phbQueueStats_p);

adk_status_t
adk_qos_sl_queue_register(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_sl_queue_unregister(
    adk_sys_handle_t        sysHandle,
    adk_qos_queue_id_t      queueId);

adk_status_t
adk_qos_get_sl_queue_statistics(
    adk_sys_handle_t                sysHandle,
    adk_qos_queue_id_t              queueId,
    adk_qos_sl_queue_statistics_t   *slQueueStats_p);

#endif /* __ADK_QOS_POLICING_INCLUDE_H__ */
 
/*!
 *   @} end addtogroup _qosFunctions
 *
 * @} end addtogroup _qos
 */
