/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2001 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 * @file  adk_qos_dscp_pcp.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos  QoS Module
 * @{
 *     @addtogroup _qosFunctions  ADK QoS Functions
 *     @{
 *
 * @brief
 *
 * This file contains the C functions for updating the DSCP/PCP mapping tree
 *
 * These are mainly pass throughs to SIF function calls in keeping with the
 * QoS model of not having it update an FPL trees. The function calls are part
 * of the QoS module, because DSCP and PCP are QoS constructs.
 *
 *
 *  **********************************************************************/

#include "adk_include.h"
#include "adk_qos_dscp_pcp.h"
#include "adk_sif_dscp_pcp.h"


/*!
 * =============================================================
 * adk_qos_cp_mapping_add
 * @param[in]  sysHandle    System handle
 * @param[in]  mappingData  The structure containing the profile ID, type,
 *                          input and output maps
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval
 *
 * @brief Fucntion to add a DSCP or PCP mapping entry
 *
 * This function is used to add a mapping entry. The two different CP types,
 * DSCP and PCP, can have a specific mapping associated with an input value
 * and a default for any unmapped values. The type field indicates which is to
 * be programmed. The DSCP and PCP maps can have different default values.
 *
 * =============================================================
 */
adk_status_t
adk_qos_cp_mapping_add(const adk_sys_handle_t       sysHandle,
                       const adk_qos_cp_mapping_t  *mappingData)
{
    return adk_sif_cp_mapping_add(sysHandle, mappingData);
}

/*!
 * =============================================================
 * adk_qos_cp_mapping_add
 * @param[in]  sysHandle    System handle
 * @param[in]  mappingData  The structure containing the profile ID, type,
 *                          input and output maps
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval
 *
 * @brief Fucntion to remove a DSCP or PCP mapping entry
 *
 * This function is used to remove a mapping entry. The specific as well as
 * the default values can be removed. Removing a specific value will cause
 * that to return the default value for the profile ID, if it is programmed,
 * or the value of 0 for both the DSCP and PCP mappings. If the default is
 * removed, the will return 0 for the DSCP and PCP input values that are not
 * mapped.
 *
 * =============================================================
 */
adk_status_t
adk_qos_cp_mapping_remove(const adk_sys_handle_t        sysHandle,
                          const adk_qos_cp_mapping_t   *mappingData)
{
    return adk_sif_cp_mapping_remove(sysHandle, mappingData);
}


/*!    end addtogroup _qosFunctions
 *     @}
 * end addtogroup _qos
 * @}
 */
