/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_policing.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos  QoS Module
 * @{
 *      @addtogroup _qosFunctions  QoS Functions
 *      @{
 *
 * @details  This file contains ADK QoS Policing flow configuration API
 *
 *
 *  **********************************************************************/

#include "adk_dpdk.h"
#include "adk_qos_include.h"
#include "adk_qos_include_internal.h"
#include "adk_qos_handles.h"
#include "adk_queue.h"
#include "adk_qos_private.h"
#include <rte_meter.h>

adk_status_t 
adk_qos_policing_flow_id_get(
        adk_sys_handle_t  sysHandle,
        adk_qos_policer_handle_t  policeHandle,
        uint32_t  *flow_id)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_qos_police_flow_handle_t    *police_info_p = NULL;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto adk_qos_policing_flow_id_get_return;
    }

    ADK_QOS_ENTRY_LOG_DEBUG(sysHandle);

    /* Validate policer pointer */
    if (!ADK_VALID_HANDLE(policeHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_id_get_return;
    }

    police_info_p = (adk_qos_police_flow_handle_t *)policeHandle;

    if (police_info_p->cookie != ADK_QOS_POLICE_COOKIE) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_POLICER_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_id_get_return;
    }

    /* allocate a policer */
    *flow_id =  police_info_p->policerId;

adk_qos_policing_flow_id_get_return:
    ADK_QOS_EXIT_LOG_DEBUG(sysHandle);
    ADK_RETURN(retValue);

}

adk_status_t 
adk_qos_policing_policy_get(
        adk_sys_handle_t  sysHandle,
        adk_qos_policer_handle_t policeHandle,
        adk_qos_policing_params_t *policer_info_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_qos_police_flow_handle_t    *police_info_p = NULL;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto adk_qos_policing_policy_get_return;
    }

    ADK_QOS_ENTRY_LOG_DEBUG(sysHandle);

    /* Validate policer pointer */
    if (!ADK_VALID_HANDLE(policeHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_policy_get_return;
    }

    if (ADK_INVALID_PTR(policer_info_p)) {
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid output pointer parameter\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        goto adk_qos_policing_policy_get_return;
    }

    police_info_p = (adk_qos_police_flow_handle_t *)policeHandle;

    if (police_info_p->cookie != ADK_QOS_POLICE_COOKIE) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_POLICER_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_policy_get_return;
    }

    ADK_MEMCPY(policer_info_p, &police_info_p->policeParams, 
                                    sizeof(adk_qos_policing_params_t));
adk_qos_policing_policy_get_return:
    ADK_QOS_EXIT_LOG_DEBUG(sysHandle);
    ADK_RETURN(retValue);
}

adk_status_t
adk_qos_policing_flow_create ( 
        adk_sys_handle_t            sysHandle,
        const adk_qos_policing_params_t   *policeParams_p,
        adk_qos_policer_handle_t          *policeFlow_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_qos_police_flow_handle_t    *police_info_p = NULL;
    adk_qos_handle_t                *qos_p = NULL;
    adk_sys_info_t                  *sysInfo_p = NULL;
    adk_qos_pol_instance_handle_t   *polInstance_p = NULL;
    struct rte_meter_srtcm_params   sr_params;
    int ret = 0;
    uint32_t    polFlowId;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto adk_qos_policing_flow_create_return;
    }

    ADK_QOS_ENTRY_LOG_DEBUG(sysHandle);
    sysInfo_p = (adk_sys_info_t *)sysHandle;
    qos_p = (adk_qos_handle_t *) sysInfo_p->qosHandle;

    /* Validate queue handle pointer */
    if (ADK_INVALID_PTR(((adk_sys_info_t *) sysHandle)->qosHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "QoS not Initialized.\n");
        goto adk_qos_policing_flow_create_return;
    }


    if (ADK_INVALID_PTR(policeParams_p) || ADK_INVALID_PTR(policeFlow_p)) {
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid input pointer parameter\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        goto adk_qos_policing_flow_create_return;
    }

    polInstance_p = &qos_p->polInstance[0];

    ADK_QOS_ACQUIRE_LOCK(qos_p->pol_mod_lock,retValue);

    /* allocate a policer */
    polFlowId = adk_rm_resource_reserve(polInstance_p->polRmHdl);
    if(polFlowId == ADK_RM_RSRC_INVALID) {
        retValue = ADK_ERROR_SEV_LEVEL_SET( ADK_E_RESOURCE_UNAVAILABLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "policer flow id allocation failed\n");
        goto adk_qos_policing_flow_create_lock_return;
    }

    police_info_p = &(polInstance_p->dpPolTable[polFlowId]);

    ADK_MEMSET(police_info_p, 0, sizeof( adk_qos_police_flow_handle_t));

    police_info_p->cookie   = ADK_QOS_POLICE_COOKIE;
    police_info_p->type     = policeParams_p->type; 
    police_info_p->policerId = polFlowId;
    ADK_MEMCPY(&police_info_p->policeParams, policeParams_p, 
                                    sizeof(adk_qos_policing_params_t));
    
    switch(policeParams_p->type){
        case ADK_QOS_POLICE_TYPE_SRTCM:

            sr_params.cir      = policeParams_p->cir;
            sr_params.cbs      = policeParams_p->cbs;
            sr_params.ebs      = policeParams_p->ebs;
#if ADK_DPDK_VERSION >= 18050016
            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_profile_config",
                            "srTCM profile configuration.\n");
            ret = rte_meter_srtcm_profile_config(&police_info_p->dpSrTcmProfile,
                                        &sr_params);

            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_profile_config",
                            "CBS %lu\t EBS %lu\n",
                            police_info_p->dpSrTcmProfile.cbs,
                            police_info_p->dpSrTcmProfile.ebs);
            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_profile_config",
                            "CIR period %lu\n",
                            police_info_p->dpSrTcmProfile.cir_period);
            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_profile_config",
                            "CIR bytes per period %lu\n",
                            police_info_p->dpSrTcmProfile.cir_bytes_per_period);

            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_config",
                            "srTCM configuration per metered traffic flow.\n");
            ret = rte_meter_srtcm_config(&police_info_p->dpSrtcmParams,
                                         &police_info_p->dpSrTcmProfile);
#else 
            ADK_LOG_EXT_API(sysHandle,
                            ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            "rte_meter_srtcm_config",
                            "srTCM configuration per metered traffic flow.\n");
            ret = rte_meter_srtcm_config(&police_info_p->dpSrtcmParams,
                                         &sr_params);
#endif
            break;
        default:
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_UNSUPPORTED);
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                "Unsuppoprted policing type\n");
            goto adk_qos_policing_flow_create_lock_return;
    }
    /*TODO: copy other data*/
    *policeFlow_p = (adk_qos_policer_handle_t )police_info_p;
    
    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
        " Policer Id %d, handle %"ADKPRIh"\n", polFlowId, *policeFlow_p);

 adk_qos_policing_flow_create_lock_return:
    if (retValue != ADK_SUCCESS) {
        *policeFlow_p = NULL;
        if (polFlowId != ADK_RM_RSRC_INVALID) {
            adk_rm_resource_free(polInstance_p->polRmHdl, polFlowId);
        }
    }
    ADK_QOS_RELEASE_LOCK(qos_p->pol_mod_lock,retValue);
_lock_acquire_failed:
_lock_release_failed:
adk_qos_policing_flow_create_return:
    ADK_QOS_EXIT_LOG_DEBUG(sysHandle);
    ADK_RETURN(retValue);
}

adk_status_t  
adk_qos_policing_flow_delete(
    adk_sys_handle_t sysHandle,
    adk_qos_policer_handle_t policeHandle)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_qos_police_flow_handle_t    *police_info_p = NULL;
    adk_qos_handle_t                *qos_p = NULL;
    adk_sys_info_t                  *sysInfo_p = NULL;
    adk_qos_pol_instance_handle_t   *polInstance_p = NULL;
    uint32_t    polFlowId;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto adk_qos_policing_flow_delete_return;
    }

    ADK_QOS_ENTRY_LOG_DEBUG(sysHandle);
    sysInfo_p = (adk_sys_info_t *)sysHandle;
    qos_p = (adk_qos_handle_t *) sysInfo_p->qosHandle;

    ADK_QOS_ACQUIRE_LOCK(qos_p->pol_mod_lock,retValue);

    /* Validate queue handle pointer */
    if (ADK_INVALID_PTR(((adk_sys_info_t *) sysHandle)->qosHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "QoS not Initialized.\n");
        goto adk_qos_policing_flow_delete_lock_return;
    }

    if (!ADK_VALID_HANDLE(policeHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_delete_lock_return;
    }

    polInstance_p = &qos_p->polInstance[0];

    police_info_p = (adk_qos_police_flow_handle_t *)policeHandle;

    if (police_info_p->cookie != ADK_QOS_POLICE_COOKIE) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_POLICER_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_delete_lock_return;
    }

    /* allocate a policer */
    polFlowId =  police_info_p->policerId;

    if (police_info_p != &(polInstance_p->dpPolTable[polFlowId])) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle in id%d, db id %d\n",
                           polFlowId, polInstance_p->dpPolTable[polFlowId].policerId );
        goto adk_qos_policing_flow_delete_lock_return;
    }

    adk_rm_resource_free(polInstance_p->polRmHdl, polFlowId);

    ADK_MEMSET(police_info_p, 0, sizeof( adk_qos_police_flow_handle_t));


 adk_qos_policing_flow_delete_lock_return:
    ADK_QOS_RELEASE_LOCK(qos_p->pol_mod_lock,retValue);
_lock_acquire_failed:
_lock_release_failed:
adk_qos_policing_flow_delete_return:
    ADK_QOS_EXIT_LOG_DEBUG(sysHandle);
    ADK_RETURN(retValue);

}

adk_status_t 
adk_qos_policing_flow_stats_get(
        adk_sys_handle_t  sysHandle,
        adk_qos_policer_handle_t policeHandle,
        adk_qos_policing_flow_stats_t *policing_stats_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_qos_police_flow_handle_t    *police_info_p = NULL;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto adk_qos_policing_flow_stats_get_return;
    }

    ADK_QOS_ENTRY_LOG_DEBUG(sysHandle);

    /* Validate policer pointer */
    if (!ADK_VALID_HANDLE(policeHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_stats_get_return;
    }

    if (ADK_INVALID_PTR(policing_stats_p)) {
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid output pointer parameter\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        goto adk_qos_policing_flow_stats_get_return;
    }
    police_info_p = (adk_qos_police_flow_handle_t *)policeHandle;

    if (police_info_p->cookie != ADK_QOS_POLICE_COOKIE) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_POLICER_HANDLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle), 
                           "Invalid policer handle \n");
        goto adk_qos_policing_flow_stats_get_return;
    }

    policing_stats_p->green_pkts = police_info_p->green_pkts;
    policing_stats_p->yellow_pkts = police_info_p->yellow_pkts;
    policing_stats_p->red_pkts = police_info_p->red_pkts;

adk_qos_policing_flow_stats_get_return:
    ADK_QOS_EXIT_LOG_DEBUG(sysHandle);
    ADK_RETURN(retValue);
}


/*!
 *  @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 *
 */
