/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_qos_ce_common.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _qos QoS Module
 * @{
 *
 *
 * Common macros and definition that are used in multiple scripts. 
 *
 * The file is split into a couple sections, depending on the compute engine
 * that the items will be used in. 
 *
 */

#ifndef __ADK_QOS_CE_COMMON_H__
#define __ADK_QOS_CE_COMMON_H__


/*
 * =======================================================================
 * ***********************************************************************
 * =======================================================================
 * 
 * These items are used in both the State Engine and the Buffer Manager
 * 
 * =======================================================================
 * ***********************************************************************
 * =======================================================================
 */
#if defined(_ACP_SE_H_) || defined(_ACP_TM_H_)

/*!
 * PDU has been colored RED by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define RED    0x03
/*!
 * PDU has been colored YELLOW by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define YELLOW 0x01
/*!
 * PDU has been colored GREEN by the policer. This value is used both in the
 * State Engine and the BUffer Manager
 */
#define GREEN  0x00
#endif

/*
 * =======================================================================
 * ***********************************************************************
 * =======================================================================
 * 
 * These items are used in the Buffer Manager
 * 
 * =======================================================================
 * ***********************************************************************
 * =======================================================================
 */
#if defined(_ACP_TM_H_)

/*! Mask for determining the PDU color used in the Buffer Manager */
#define COLOR   ((task_params[3] & 0x03))
/* #define ATM_COLOR   ((parameters_tm & 0x20)!=0) */


#define NUM_VLAN_TAGS  (task_params[4] )

/*! Mask to determine the output interface number of the current PDU */
#define OUT_INTF_NUM    (task_params[5])

/*! Mask to see if loop back stop check required or not */
#define IS_LB_CHECK_REQ  (((task_params[7]) & 0x1) != 0)

/*! Mask to determine the input interface number of the current PDU */
#define IN_INTF_NUM     (task_params[6])

#define OUT_INTF_TYPE   (task_params[7])

/*! INTF+PCP based statistics */
unsigned vlan_tagged_octets_tx  shared_params[0:7];
/*! INTF+PCP based statistics */
unsigned vlan_tagged_pkts_tx    shared_params[8:11];

#endif


#endif  /*  #ifndef __ADK_QOS_CE_COMMON_H__  */

/*!
 * @} end addtogroup _qos
 *
 * vim: set foldmethod=marker foldlevel=1 foldcolumn=3 columns=83:
 */
