/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_qos_ds_profiler_private.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos  QoS Module
 * @{
 *
 * @brief  This file contains ADK QoS Diffsrev Profiler private data
 *         structures, macros and API signatures
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_DS_PROFILER_PRIVATE_INCLUDE_H__
#define __ADK_QOS_DS_PROFILER_PRIVATE_INCLUDE_H__

#include "adk_qos_include.h"
#include "adk_qos_ds_profiler_include.h"
#include "adk_list.h"


#define ADK_QOS_DS_PROFILER_SCHEDID_NAMESPACE   "MTMSchedulersLevel1"
#define ADK_QOS_DS_PROFILER_QUEUEID_NAMESPACE   "adk_qos_queueId_namespace"

#define ADK_QOS_DS_MAX_PHB 3
#define ADK_QOS_DS_MAX_CLASSES 4

/*!
 * =============================================================
 * @struct adk_qos_diffserv_traffic_class_info_t adk_qos_ds_profiler_private.h "adk_qos_ds_profiler_private.h"
 *
 * @brief
 *
 * The adk_qos_diffserv_traffic_class_info_t data structure is to store diffserv
 * traffic class information like queueId allocated for the traffic profile and
 * the traffic class indentifier
 *
 * =============================================================
 */
typedef struct adk_qos_diffserv_traffic_class_info_s {
    uint32_t        classId;
    adk_handle_t        queueHandle;
} adk_qos_diffserv_traffic_class_info_t;   /* ------  end of struct adk_qos_diffserv_traffic_class_info_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_diffserv_phb_info_t adk_qos_ds_profiler_private.h "adk_qos_ds_profiler_private.h"
 *
 * @brief
 *
 * The adk_qos_diffserv_phb_info_t data structure is to store diffserv
 * profile information
 *
 * =============================================================
 */
typedef struct adk_qos_diffserv_phb_info_s {
    adk_qos_ds_phb_type_t        phbType;
    adk_handle_t                 schedHandle;
    uint16_t                 numTrafficClasses;
    adk_handle_t                 queueSchedHandle[ADK_QOS_DS_MAX_CLASSES];
} adk_qos_diffserv_phb_info_t;   /* ------  end of struct adk_qos_diffserv_phb_info_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_diffserv_policy_info_t adk_qos_ds_profiler_private.h "adk_qos_ds_profiler_private.h"
 *
 * @brief
 *
 * The adk_qos_diffserv_policy_info_t data structure is to store diffserv policy
 * information
 *
 * =============================================================
 */
#define ADK_QOS_STD_DS_COOKIE        0x5423acce
#define ADK_QOS_ADAPTIVE_DS_COOKIE   0xA423acce

typedef struct adk_qos_diffserv_policy_info_s {
    uint32_t                    cookie;
    adk_handle_t                    parentHandle;
    uint8_t                     numPhbGroups;
    adk_qos_diffserv_phb_info_t     phbInfo[4];
} adk_qos_diffserv_policy_info_t;   /* ------  end of struct adk_qos_diffserv_policy_info_t  ------ */


/*!
 * =============================================================
 * @brief  ADK QoS private API signatures
 *
 *   @addtogroup _qosFunctions QoS Functions
 *   @{
 * =============================================================
 */

adk_status_t
adk_qos_ds_policy_parent_validate(
    adk_sys_handle_t    sysHandle,
    adk_handle_t        parentHandle);
#if 0
adk_status_t
adk_qos_ds_policy_sched_info_fill(
    adk_sys_handle_t            sysHandle,
    const adk_qos_ds_profile_t       *dsProfile_p,
    uint32_t                phbIndex,
    adk_qos_sched_params_t     *sched_p);
#endif
adk_status_t
adk_qos_ds_policy_queue_info_fill(
    adk_sys_handle_t                sysHandle,
    adk_handle_t                    parentHandle,
    adk_qos_ds_class_profile_t     *class_p,
    adk_qos_queue_params_t         *queue_p);

adk_status_t
adk_qos_ds_policy_info_allocate(
    adk_sys_handle_t                    sysHandle,
    adk_qos_ds_profile_t               *dsProfile_p,
    adk_qos_diffserv_policy_info_t    **dsPolicyInfo_p);

int32_t
adk_qos_ds_policy_info_deallocate(
    adk_sys_handle_t    sysHandle,
    void               *data_p);

adk_status_t
adk_qos_ds_policy_info_fill(
    adk_sys_handle_t                    sysHandle,
    const adk_qos_ds_profile_t               *dsInfo_p,
    adk_qos_diffserv_policy_info_t     *dsPolicy_p);


adk_status_t
adk_qos_ds_policy_phb_validate(
    adk_sys_handle_t    sysHandle,
    adk_qos_ds_phb_t   *phb_p);

adk_status_t
adk_qos_ds_policy_phb_type_validate(
    adk_sys_handle_t          sysHandle,
    adk_qos_ds_phb_type_t     classType);

adk_status_t
adk_qos_ds_policy_class_id_validate(
    adk_sys_handle_t          sysHandle,
    uint32_t              classId);

adk_status_t
adk_qos_ds_class_queues_create(
    adk_sys_handle_t                __attribute__((unused)) sysHandle,
    const adk_qos_ds_profile_t           *dsProfile_p,
    adk_qos_diffserv_policy_info_t *dsPolicyInfo_p);

adk_status_t
adk_qos_ds_class_scheds_create(
    adk_sys_handle_t                __attribute__((unused)) sysHandle,
    const adk_qos_ds_profile_t           *dsProfile_p,
    adk_qos_diffserv_policy_info_t *dsPolicyInfo_p);

adk_status_t
adk_qos_ds_policy_user_flow_validate(
    adk_sys_handle_t                __attribute__((unused)) sysHandle,
    const adk_qos_ds_user_flow_data_t    *userFlow_p);

#endif /* __ADK_QOS_DS_PROFILER_PRIVATE_INCLUDE_H__ */

/*!
 *   @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 */
