/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_qos_handles.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos QoS Module
 * @{
 *
 * @brief  This file contains ADK QoS handles
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_HANDLES_INCLUDE_H__
#define __ADK_QOS_HANDLES_INCLUDE_H__

#include <adk_include.h>
#include "adk_list.h"
#include "adk_util_list.h"
#include "adk_qos_private.h"

/*!
 * =============================================================
 * @struct adk_qos_instance_handle_t adk_qos_handles.h "adk_qos_handles.h"
 *
 * @brief
 *
 * The adk_qos_instance_handle_t data structure is to maintain
 * per device instance ADK QoS handles
 *
 * =============================================================
 */
typedef struct adk_qos_instance_handle_s {
    uint32_t            cookie; 
    adk_rm_hdl_t        queueIdRmHdl;
    uint16_t            numQueueIds;
} adk_qos_instance_handle_t;   
/* ------  end of struct adk_qos_instance_handle_t  ------ */

/*!
 * =============================================================
 * @struct adk_qos_pol_instance_handle_t adk_qos_handles.h "adk_qos_handles.h"
 *
 * @brief
 *
 * The adk_qos_pol_instance_handle_t data structure is to maintain
 * per instance ADK QoS policer handles/tables
 *
 * =============================================================
 */
typedef struct adk_qos_pol_instance_handle_s {
    uint64_t            cookie; 
    adk_rm_hdl_t        polRmHdl;
    adk_qos_police_flow_handle_t *dpPolTable; 
    uint32_t            numPolIds;
} adk_qos_pol_instance_handle_t;   
/* ------  end of struct adk_qos_pol_instance_handle_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_handle_t adk_qos_handles.h "adk_qos_handles.h"
 *
 * @brief
 *
 * The adk_qos_handle_t data structure is to maintain ADK QoS handles
 *
 * =============================================================
 */
typedef struct adk_qos_handle_s {
    void                        *queue_mod_lock;
    void                        *pol_mod_lock;
    adk_qos_instance_handle_t    qosInstance[ADK_QOS_NUM_INSTANCES];
    adk_qos_pol_instance_handle_t polInstance[1];
    adk_qos_queue_id_t           defaultQueueId;
    adk_qos_queue_id_t           ptpQueueId;
    uint32_t                     queueStatsIdTbl[ADK_QOS_MAX_PHB_QUEUES];
    uint32_t                     slQueueStatsIdTbl[ADK_QOS_MAX_SL_QUEUES];
	uint32_t					 numDevices;
} adk_qos_handle_t;   /* ------  end of struct adk_qos_handle_t  ------ */

#endif /* __ADK_QOS_HANDLES_INCLUDE_H__ */

/*!
 * @} end addtogroup _qos
 */
