/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_qos_private.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _qos  QoS Module
 * @{
 *
 * @brief  This file contains ADK QoS private data structures,
 *         macros and API signatures
 *
 *
 *  **********************************************************************/

#ifndef __ADK_QOS_PRIVATE_INCLUDE_H__
#define __ADK_QOS_PRIVATE_INCLUDE_H__

#include "adk_include.h"
#include "adk_qos_include.h"
#include "adk_qos_policing_include.h"
#include "adk_pktio.h"
#include "adk_qos_dp.h"
#include "adk_list.h"
#include "adk_util_list.h"
#include <rte_meter.h>
#include "adk_queue.h"

/*!
 * Name of shared memory used in IPC between primary & secondary process
 */
#define ADK_QOS_SHM_MEMZONE_NAME "ADK_QOS_SHM_MEMZONE"
#define ADK_QOS_PHB_MEMZONE_NAME "ADK_QOS_PHB_SHM_MEMZONE"
#define ADK_QOS_SL_MEMZONE_NAME "ADK_QOS_SL_SHM_MEMZONE"
#define ADK_QOS_MAX_PHB_QUEUES (16*1024)
#define ADK_QOS_MAX_SL_QUEUES (256)   /*TODO: Fix the Max value*/

/* 1% Tolerance */
#define ADK_QOS_CONFORM_TOLERANCE(X, Y) \
    (((X) >= ((Y) - (Y)/100)) && ((X) <= ((Y) + (Y)/100)))


/*
 * Macros for Entry/Exit printing of the routines. These will be enabled via
 * qos_debug macros
 */
#define QOS_DEBUG

#ifdef QOS_DEBUG
#define ADK_QOS_ENTRY_LOG_DEBUG(__qos_sys_handle_) \
            ADK_LOG_INFO((__qos_sys_handle_), \
                         ADK_SYS_MODULE_HANDLE_GET((__qos_sys_handle_)), \
                         "\n\tEntering %s\n", __func__)

#define ADK_QOS_EXIT_LOG_DEBUG(__qos_sys_handle_) \
            ADK_LOG_INFO ((__qos_sys_handle_), \
                          ADK_SYS_MODULE_HANDLE_GET((__qos_sys_handle_)), \
                          "\n\tExiting %s\n", __func__)
#define ADK_QOS_INTERNAL_ENTRY_LOG_DEBUG(__qos_sys_handle_) \
            ADK_LOG_INTERNAL_FUNC_ENTRY((__qos_sys_handle_), \
                               ADK_SYS_MODULE_HANDLE_GET((__qos_sys_handle_)))

#define ADK_QOS_INTERNAL_EXIT_LOG_DEBUG(__qos_sys_handle_) \
            ADK_LOG_INTERNAL_FUNC_EXIT((__qos_sys_handle_), \
                               ADK_SYS_MODULE_HANDLE_GET((__qos_sys_handle_)))

#else
#define ADK_QOS_ENTRY_LOG_DEBUG(__qos_sys_handle_) 
#define ADK_QOS_EXIT_LOG_DEBUG(__qos_sys_handle_) 
#define ADK_QOS_INTERNAL_ENTRY_LOG_DEBUG(__qos_sys_handle_)
#define ADK_QOS_INTERNAL_EXIT_LOG_DEBUG(__qos_sys_handle_)
#endif

#define ADK_QOS_INFO_LOG_ENABLED(__sysHandle__) \
        IS_LOGGING_LEVEL_ENABLED_FOR_THIS_MODULE(__sysHandle__, \
                                        ADK_SYS_MODULE_HANDLE_GET(__sysHandle__), \
                                        ADK_SEV_LEVEL_INFO)
/* Locking macros */
#define ADK_QOS_ACQUIRE_LOCK(_lock,_retValue) \
    do { \
        if(_lock!=NULL) \
        { \
            ADK_ACQUIRE_LOCK(_lock,_retValue); \
        } \
    } while (0)

#define ADK_QOS_RELEASE_LOCK(_lock,_retValue) \
    do { \
        if(_lock!=NULL) \
        { \
            ADK_RELEASE_LOCK(_lock,_retValue); \
        } \
    } while (0)

#define ADK_QOS_SCHEDID_1_NAMESPACE "MTMSchedulersLevel1"
#define ADK_QOS_SCHEDID_2_NAMESPACE "MTMSchedulersLevel2"
#define ADK_QOS_SCHEDID_3_NAMESPACE "MTMSchedulersLevel3"
#define ADK_QOS_SCHEDID_4_NAMESPACE "MTMSchedulersLevel4"
#define ADK_QOS_SCHEDID_5_NAMESPACE "MTMSchedulersLevel5"

#define ADK_QOS_QUEUEID_NAMESPACE   "adk_qos_queueId_namespace"

#define ADK_QOS_POLICING_NAMESPACE  "adk_qos_policing_flow"


#define QOS_HANDLE_COOKIE  0xB7BDC82C

/* Is system handle invalid */
#define ADK_INVALID_QOS_HANDLE(qos_hndl) \
   (((qos_hndl) == 0) || (((adk_qos_instance_handle_t *)(qos_hndl))->cookie != QOS_HANDLE_COOKIE))



/*!
 * =============================================================
 * @internal
 * @enum adk_qos_sched_subscription_mode_t
 *
 * Are there just two states here or will there be more entries? Should we
 * have a different name?
 *
 * =============================================================
 */
typedef enum adk_qos_sched_subscription_mode_e {
    ADK_QOS_SCHED_SUBSCRIP_MODE_NONE    = 1, /*!<  */
    ADK_QOS_SCHED_SUBSCRIP_MODE_OVER /*!<  */
} adk_qos_sched_subscription_mode_t;   /* ------  end of enum adk_qos_sched_subscription_mode_t  ------ */


/*!
 * =============================================================
 * @struct adk_qos_police_flow_handle_t adk_qos_private.h "adk_qos_private.h"
 *
 * @brief 
 *
 * =============================================================
 */
#define ADK_QOS_POLICE_COOKIE    0x504f4c49 
typedef struct adk_qos_police_flow_handle_s {
    uint32_t    cookie;
    adk_qos_policing_types_t        type;
    uint32_t                        policerId;
    union{
        struct rte_meter_srtcm      dpSrtcmParams;
        struct rte_meter_trtcm      dpTrtcmParams;
    };
#if ADK_DPDK_VERSION >= 18050016
    union{
        struct rte_meter_srtcm_profile      dpSrTcmProfile;
        struct rte_meter_trtcm_profile      dpTrTcmProfile;
    };
#endif
    uint64_t green_pkts;
    uint64_t yellow_pkts;
    uint64_t red_pkts;
    adk_qos_policing_params_t      policeParams;
} adk_qos_police_flow_handle_t;   /* ------  end of struct adk_qos_police_flow_handle_t  ------ */

typedef struct adk_qos_sl_queue_dp_stats_s{
    uint64_t nonIPTxPkts;
    uint64_t nonIPTxBytes;
    uint64_t nonIPTxPktsDropped;
    uint64_t nonIPTxBytesDropped;
    uint64_t ipv4TxPkts;
    uint64_t ipv4TxBytes;
    uint64_t ipv4TxPktsDropped;
    uint64_t ipv4TxBytesDropped;
    uint64_t ipv6TxPkts;
    uint64_t ipv6TxBytes;
    uint64_t ipv6TxPktsDropped;
    uint64_t ipv6TxBytesDropped;
}adk_qos_sl_queue_dp_stats_t;

typedef struct adk_qos_phb_dp_stats_s{
    uint64_t ipv4TxPkts;
    uint64_t ipv4TxBytes;
    uint64_t ipv4TxPktsDropped;
    uint64_t ipv4TxBytesDropped;
    uint64_t ipv6TxPkts;
    uint64_t ipv6TxBytes;
    uint64_t ipv6TxPktsDropped;
    uint64_t ipv6TxBytesDropped;
}adk_qos_phb_dp_stats_t;

/*!
 * =============================================================
 * @struct adk_qos_shm_config_t adk_qos_private.h "adk_qos_private.h"
 *
 * @brief Structure to pass information from primary to secondary
 * process .
 *
 * =============================================================
 */
typedef struct adk_qos_shm_config_t{
    
    /*!
     * Data plane Policer table handler
     */
    adk_qos_police_flow_handle_t *dpPolTable_p;

    /*Default tranmsit queue */
    adk_qos_queue_id_t defaultQueueId;

    /*PTP traffic queue */
    adk_qos_queue_id_t ptpQueueId;

    /*PHB queue resources*/
    adk_rm_hdl_t phbQueuesRsrcHdl;
    
    /*SL queue resources*/
    adk_rm_hdl_t slQueuesRsrcHdl;

    /*PHB stats*/
    adk_qos_phb_dp_stats_t *phbStats_p;

    /*SL Queue Stats*/
    adk_qos_sl_queue_dp_stats_t *slQueueStats_p;

    /*PHB queue to stats id map*/
    uint32_t queueStatsIdTbl[ADK_QOS_MAX_PHB_QUEUES];

    /*SL queue to stats id map*/
    uint32_t slQueueStatsIdTbl[ADK_QOS_MAX_SL_QUEUES];
} adk_qos_shm_config_t __rte_cache_aligned;

/*!
 * =============================================================
 * @brief  ADK QoS private API signatures
 *
 *   @addtogroup _qosFunctions  QoS Functions
 *   @{
 *
 * =============================================================
 */
adk_status_t
adk_qos_get_dev_instance( uint16_t deviceId, uint16_t *instanceId_p);

adk_status_t  
adk_qos_policing_params_gen(
    adk_sys_handle_t              sysHandle,
    const adk_qos_policing_params_t     *policingParams_p,
    uint32_t                  *flowParams_p,
    uint8_t                   *script_id);

adk_status_t  
adk_qos_policing_params_validate(
    adk_sys_handle_t                 sysHandle,
    const adk_qos_policing_params_t     *policingParams_p);

#endif /* __ADK_QOS_PRIVATE_INCLUDE_H__ */

/*!
 *   @} end addtogroup _qosFunctions
 * @} end addtogroup _qos
 */
