/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_bind_interface.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 * @{
 *
 *
 * This file contains all the declarations that are required to support
 * ADK bind functions.
 *
 ***********************************************************************/

#ifndef __ADK_BIND_INTERFACE_H__
#define __ADK_BIND_INTERFACE_H__

#include <adk_sys.h>

#if 1
/*!
 * maximum number of destination modules per pipeline. This means that
 * the source module can use this pipeline to talk to this many number
 * of destination modules.
 */
#define ADK_BIND_MAX_MODULES_PER_PIPELINE 16

/*!
 * =============================================================
 * @struct adk_bind_cpu_table_entry_t adk_bind_interface.h "adk_bind_interface.h"
 *
 *
 * internal data structure that is used maintain the destination
 * information in cases where the source module is located on CPU.
 *
 * =============================================================
 */
typedef struct adk_bind_cpu_table_entry_s
{
    adk_bind_imc_service_fn_t  cpuImcFn;
    module_cpu_fn_t            destEntryFn[_DEST_FUNC_MAX];
    uint16_t               dmuxIndex;
    uint8_t                pipeline;
    adk_bind_module_location_t destLocation;
    adk_module_handle_t        destModule;
    adk_flow_handle_t          destFlow;
} adk_bind_cpu_table_entry_t;

/*!
 * =============================================================
 * @struct adk_bind_dmux_database_t adk_bind_interface.h "adk_bind_interface.h"
 */
typedef struct adk_bind_dmux_database_s
{
    bool inUse;
    uint16_t dmuxFnId;
} adk_bind_dmux_database_t;

/*!
 * =============================================================
 * @struct adk_bind_pipeline_dmux_table_t adk_bind_interface.h "adk_bind_interface.h"
 */
typedef struct adk_bind_pipelineInfo_s
{
    adk_bind_dmux_database_t pipelineInfo[ADK_BIND_MAX_MODULES_PER_PIPELINE];
} adk_bind_pipeline_dmux_table_t;
#endif

/* Function proto types */

extern adk_status_t
adk_bind_infrastructure_init (adk_sys_handle_t);
extern adk_status_t
adk_bind_infrastructure_deinit (adk_sys_handle_t);

#endif  /* #ifndef __ADK_BIND_INTERFACE_H__ */

/*!
 * @} end addtogroup _sif
 */
