/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_mq.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief  
 *
 * This file contains linux semaphore control definitions.
 *
 ***********************************************************************/

#ifndef __ADK_SEM_H__
#define __ADK_SEM_H__

#include <semaphore.h>
#include <mqueue.h>
#include <fcntl.h>
#include <sys/stat.h>

typedef sem_t    adk_sem_t;
typedef mqd_t    adk_mqd_t;
typedef struct mq_attr  adk_mq_attr;

typedef struct adk_mq_attr_s {
    long mq_flags;       /* Flags: 0 or O_NONBLOCK */
    long mq_maxmsg;      /* Max. # of messages on queue */
    long mq_msgsize;     /* Max. message size (bytes) */
    long mq_curmsgs;     /* # of messages currently in queue */
}adk_mq_attr_t;

adk_status_t
adk_mq_open_init(adk_mqd_t *mqd, 
                 const char *name); 

adk_status_t
adk_mq_open(adk_mqd_t *mqd,
            const char *name);

/*!
 * @fn adk_ (adk_mqd_t *mqd)
 */
adk_status_t
adk_mq_close(adk_mqd_t mqd);

/*!
 * @fn adk_ (adk_mqd_t *mqd)
 */
adk_status_t
adk_mq_unlink(const char *name);

adk_status_t
adk_mq_receive(adk_mqd_t    mqdes, 
               uint8_t  *msg_ptr,
               uint32_t msg_len, 
               uint32_t *msg_prio);

adk_status_t
adk_mq_send(adk_mqd_t    mqdes, 
            const uint8_t  *msg_ptr,
            uint32_t msg_len, 
            uint32_t msg_prio);

#endif /* __ADK_SEM_H__ */
 
/*! end addtogroup _sif
 * @}
 */
