/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_netd_ts_index.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of objects that are
 * used for TN port timestamping.
 *
 *
 ***********************************************************************/

#ifndef _ADK_NETD_TS_INDEX_H_
#define _ADK_NETD_TS_INDEX_H_

extern struct adk_netd_ts_index_t adk_netd_ts_index_table[ADK_NETD_MAX_PORT_GROUPS];
/*
 * Initializes internal data structures for index management
 */
void
adk_netd_ts_index_init(void);

/*
 * adk_netd_ts_index_alloc
 * Allocates a timestamp index
 * port - port for which index allocation is requested
 * ts_index - pointer for storing the allocated index
 * Returns 0 on succcess and negative value on error
 */
int
adk_netd_ts_index_alloc(int port, uint8_t *ts_index);

/*
 * adk_netd_ts_index_free
 * Frees a timestamp index
 * port - port number
 * ts_index - the timestap index to be freed-up
 * Returns 0 on succcess and negative value on error
 */
int
adk_netd_ts_index_free(int port, uint8_t ts_index);

/*
 * adk_netd_ts_index_is_allocated
 * Tests whether an index is allocated
 * port - port number
 * ts_index - the timestamp index
 * allocated - true if allocated and false otherwise
 * Returns 0 on succcess and negative value on error
 */
int
adk_netd_ts_index_is_allocated(int port, uint8_t ts_index,
		bool *allocated);

/*
 * adk_netd_ts_index_free_count
 * Returns number of free index spaces available
 * port - port number
 * Returns number of index spaces available
 * On error, it returns negative value
 */
int
adk_netd_ts_index_free_count(int port);

/*
 * adk_netd_ts_index_dump
 * Dumps the internal data structure. Used for debugging.
 * port - port number
 */
void
adk_netd_ts_index_dump(int port);

/*
 * adk_netd_ts_index_test
 * test function for verifying index management apis
 */
void
adk_netd_ts_index_test(void);

	static inline void
adjust_index(u8 *cur_index, u8 start_index, u8 end_index, u8 port_grp)
{
	if (*cur_index >= end_index) {
		*cur_index = (*cur_index %
				adk_netd_ts_index_table[port_grp].max_ts_index) +
			start_index;
	}
}

#endif /* _ADK_NETD_TS_INDEX_H_ */

/*! end addtogroup _sif
 * @}
 */
