/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_platform.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief  
 *
 * This header file contains definitions that are system specific. The user
 * may change these defintions to match with their system configuration. 
 * Some of the parameters here are chip resource limit definitions, this
 * may change between different variants of the chip. 
 *
 ***********************************************************************/
#ifndef __ADK_PLATFORM_H__
#define __ADK_PLATFORM_H__

#include <adk_sys.h>

/*!
 * module manager related definitions.
 */

/*! length of the module name. User can change this value just
 *  by updating this definition */
#define ADK_MAX_MODULE_NAME_LENGTH  32
/*!
 * maximum number of modules supported by module manager. User
 * can change this value to a maximum of 255 without changing
 * the C code. A value above 255 might requires minor 
 * changes in C-code. 
 */
#define ADK_MAX_SUPPORTED_MODULES   64
/*! The maximum number of modules supported is divided to cater
 *  for customer ADK modules. The customer module IDs should
 *  start from the following ID. Maximum allowed customer module
 *  ID is 31.
 */
#define ADK_CUSTOMER_MODULE_ID_START 40

#define ADK_ALL_REGISTERED_MODULES \
    (ADK_MAX_SUPPORTED_MODULES + 1)
#define ADK_VALID_MODULE_NAME(name) \
    ((ADK_STRLEN( (char *)name) < ADK_MAX_MODULE_NAME_LENGTH) && \
     (ADK_STRLEN( (char *)name) != 0))

/*! Changing this value might require minor changes in fastpath
 *  C-code. */
#define ADK_FLOWID_WIDTH    18

/*!
 * hardware limit. The user can not increase unless a higher 
 * value is supported by the hardware.
 */
#define ADK_MAX_PIPELINES 256

/*!
 * Maximum number of interfaces supported by ADK. The interface
 * ID is managed by ADK software and it is unique within ADK 
 * software.
 */
#define ADK_MAX_SUPPORTED_INTERFACES 16000

/*!
 * NVM user pointer index allocation. If the user application needs
 * NVM user pointer index, it should be defined here first.
 */
#define ADK_NVM_USER_INDEX_SYSHANDLE 0
#define ADK_NVM_USER_INDEX_MODHANDLE 1

/*!
 * Maximum number of ACPs supported by ADK.
 */
#define ADK_MAX_SUPPORTED_CHIPS 1

/* TODO: This should be probed */
#define ADK_SYS_CPK_DEV_ID  0
#define ADK_SYS_CPK_VSI_ID  22

#endif /* __ADK_PLATFORM_H__ */
 
/*! end addtogroup _sif
 * @}
 */
