/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_sif_constants.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 *
 ***********************************************************************/
#ifndef _ADK_SIF_CONSTANTS_H_
#define _ADK_SIF_CONSTANTS_H_

#include "adk_modules_enabled.h"

/*
 * Set the maximum number of ADK cores for each product
 */
#ifdef ADK_XEON_PROFILE_1
#  define ADK_MAX_NUM_CORES 12
#elif defined(ADK_XEON2_PROFILE_1)
#  define ADK_MAX_NUM_CORES 16
#elif defined(ADK_SNR_PROFILE_1)
#  define ADK_MAX_NUM_CORES 16
#else
#  error "Product profile not defined"
#endif

#if ADK_MAX_NUM_CORES > RTE_MAX_LCORE
#  undef ADK_MAX_NUM_CORES
#  define ADK_MAX_NUM_CORES RTE_MAX_LCORE
#endif

/*
 * Maximum number of Ethernet devices through which ADK can received or
 * send packets.
 */
#define ADK_MAX_NUM_ETH_DEVICES     16
/*
 * Maximum port number that can be assigned to any port across all Ethernet
 * devices.
 */
#define ADK_MAX_NUM_ETH_PORTS       32

#if 0
###/* Hardware Error counters */
####define  FLOW_OVERRUN_OFFSET            0x00
####define  FLOW_UNDERRUN_OFFSET           0x04
####define  TREE_OVERRUN_OFFSET            0x08
####define  TREE_UNDERRUN_OFFSET           0x0C
####define  UNDEF_TREE_INSTR_OFFSET        0x10
####define  UNDEF_FLOW_INSTR_OFFSET        0x14
####define  UNDEF_INTERNAL_FUNC_OFFSET     0x18
####define  STACK_OVERFLOW_OFFSET          0x1C
####define  STACK_UNDERFLOW_OFFSET         0x20
####define  FDC_HW_ERR_OFFSET              0x24
####define  TREE_MEM_READ_ERR_OFFSET       0x28
####define  FC_READ_ERR_OFFSET             0x2C
####define  PDU_DATA_READ_ERR_OFFSET       0x30
####define  FBI_BOUND_ERR_OFFSET           0x34
####define  PIM_MEM_ERR_OFFSET             0x38
####define  SEMAPHORE_ERR_OFFSET           0x3C
####define  PIC_DATA_ERR_OFFSET            0x40
####define  HE_DATA_READ_ERR_OFFSET        0x44
####define  SE_DATA_READ_ERR_OFFSET        0x48
####define  FBI_ORD_GEN_FUNC_ERR_OFFSET    0x4C
####define  FBI_UNDEF_TRAP_ERR_OFFSET      0x50
####define  ARG_OVERFLOW_ERR_OFFSET        0x54
####define  FRP_HW_ERR_OFFSET              0x58
####define  STACK_RDATA_ERR_OFFSET         0x5C
####define  HANDLE_RDATA_ERR_OFFSET        0x60
####define  GLOBAL_RDATA_ERR_OFFSET        0x64
####define  UNDEF_REG_ERR_OFFSET           0x68
####define  TASK_ERR_OFFSET                0x6C
####define  SOFTWARE_ERR_OFFSET            0x70
####define  UNKNOWN_HW_ERR_OFFSET          0x74
####define  SIF_RESERVE_1                  0x78
####define  SIF_RESERVE_2                  0x7C
###
###/* Other SIF Counters */
####define SIF_TOTAL_DROPS_OFFSET          0x80
####define BIND_LOOKUP_FAIL_OFFSET         0x84
####define DEST_DMUX_LOOKUP_FAIL_OFFSET    0x88
####define BIND_PH_CALL_FAIL_OFFSET        0x8C
####define BIND_FWD_CPU_CNT_OFFSET         0x90
####define BIND_INDIRECT_CNT_OFFSET        0x94
####define BIND_DIRECT_CNT_OFFSET          0x98
####define HEARTBEAT_GLOBAL_HASH_ENGINE_UNKNOWN_RESPONSE    0x9C   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_OVERFLOW_ERROR      0xa0   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_NAMESPACE_ERROR     0xa4   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_MMB_ALLOC_ERROR     0xa8   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_DYN_ALLOC_ERROR     0xac   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_TIMER_FAIL_ERROR    0xb0   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_MATCH_TIMER_ERROR   0xb4   /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_MATCH_NOT_TIMER_ERROR 0xb8 /* 4 bytes */
####define HEARTBEAT_GLOBAL_HASH_ENGINE_NO_MATCH_ERROR      0xbc   /* 4 bytes */
####define SIF_NETDRIVER_DEMUX_ERR_OFFSET 0xc0 /* 4 bytes */
###
###
###/* Input parameter specifics for bind tree */
####define BIND_FLOWID_WIDTH               24
####define BIND_MODULEID_WIDTH             8
####define BIND_MPP_FN_WIDTH               10   /* support 1024 MPP functions. */
####define BIND_VPID_WIDTH                 8    /* support 255 VPs */
####define BIND_DEST_WIDTH                 1
####define BIND_PATH_WIDTH                 1
####define BIND_TYPE_WIDTH                 2
###
####define BIND_TREE_INPUT_NUM_FIELDS      2
####define BIND_TREE_RETURN_NUM_FIELDS     6
####define BIND_TREE_VRETURN_NUM_FIELDS    2
###
####define BIND_PDU_FWDING_FAILED          0xffffff
###
####define BIND_TREE_WIDTH  \
###    BIND_MODULEID_WIDTH + BIND_FLOWID_WIDTH
###
####define ADK_BIND_FLOWID_WIDTH               ADK_FLOWID_WIDTH
####define ADK_BIND_MODULEID_WIDTH             8
####define ADK_BIND_MPP_FN_WIDTH               10   /* support 1024 MPP functions. */
####define ADK_BIND_PIPEID_WIDTH               8    /* support 255 pipelines */
####define ADK_BIND_DMUX_WIDTH                 4
####define ADK_BIND_IMCFN_WIDTH                10
####define ADK_BIND_DEST_WIDTH                 1
####define ADK_BIND_PATH_WIDTH                 2
####define ADK_BIND_TREE_INPUT_NUM_FIELDS      2
####define ADK_BIND_TREE_RETURN_NUM_FIELDS     6
####define ADK_BIND_TREE_VRETURN_NUM_FIELDS    1
####define ADK_BIND_TREE_WIDTH  (ADK_BIND_MODULEID_WIDTH + ADK_BIND_FLOWID_WIDTH)
###
####define ADK_BIND_DMUX_TREE_WIDTH                10
####define ADK_BIND_DMUX_TREE_INPUT_NUM_FIELDS     1
####define ADK_BIND_DMUX_TREE_RETURN_NUM_FIELDS    1
###
####define ADK_BIND_IMC_TREE_WIDTH                 ADK_BIND_IMCFN_WIDTH
####define ADK_BIND_IMC_TREE_INPUT_NUM_FIELDS      1
####define ADK_BIND_IMC_TREE_RETURN_NUM_FIELDS     1
###
####define ADK_BIND_M2M_BIND     01
####define ADK_BIND_F2M_BIND     10
####define ADK_BIND_F2F_BIND     11
###
###
###/* SIF mapping trees */
####define ADK_SIF_NETDRIVER_DEMUX_PORTID_WIDTH        12
####define ADK_SIF_NETDRIVER_DEMUX_PDU_TYPE_WIDTH      4
####define ADK_SIF_INTERFACE_TO_PORTID_MAPPING_WIDTH   24
#endif


/* DSCP / PCP mapping tree */
#define ADK_CP_PROFILE_ID_WIDTH             5
#define ADK_CP_INPUT_TYPE_WIDTH             1
#define ADK_CP_INPUT_VALUE_WIDTH            6

#define ADK_DSCP_INPUT_VALUE_WIDTH          6
#define ADK_CP_DSCP_VALUE_MAX               0x3f
#define ADK_DSCP_OUTPUT_VALUE_MASK          0x3f
#define ADK_DSCP_OUTPUT_VALID_MASK_SET      0x40 /* 0b01000000 */
#define ADK_DSCP_OUTPUT_INHERIT_MASK_SET    0x80 /* 0b10000000 */
#define ADK_CP_INPUT_TYPE_DSCP_SET          0

#define ADK_PCP_INPUT_VALUE_WIDTH           3
#define ADK_CP_PCP_VALUE_MAX                0x07
#define ADK_PCP_OUTPUT_VALUE_MASK           0x07
#define ADK_PCP_OUTPUT_VALID_MASK_SET       0x08 /* 0b01000 */
#define ADK_PCP_OUTPUT_INHERIT_MASK_SET     0x10 /* 0b10000 */
#define ADK_CP_INPUT_TYPE_PCP_SET           (1 << ADK_CP_PROFILE_ID_WIDTH)

#define ADK_SIF_PCAP_ID_MAX                 256

 /*!
 * List of processing funcion id of all modules.
 * Gaurded by module's #define macros
 *
 */
typedef enum {
	ADK_RESERVED = 0,  /* Error handler */
/*Don't move the first include*/
/*This is for EM packets*/
#include "adk_sif_em_proc_fn_ids.h"

#include "adk_sif_dp_proc_fn_ids.h"

#include "adk_ph_dev_dp_proc_fn_ids.h"

#ifdef ADK_MODULE_ETHERNET_ENABLED
#include <adk_eth_dp_proc_fn_ids.h>
#endif

#ifdef ADK_MODULE_ETH_OAM_ENABLED
#include <adk_eth_oam_dp_proc_fn_ids.h>
#endif

#ifdef ADK_MODULE_IP_ENABLED
#include <adk_ip_dp_proc_fn_ids.h>
#endif

#ifdef ADK_MODULE_IPV6_ENABLED
#include <adk_ipv6_dp_proc_fn_ids.h>
#endif

#ifdef ADK_MODULE_IPSEC_ENABLED
#include "adk_ipsec_dp_proc_fn_ids.h"
#endif

#ifdef ADK_MODULE_BFD_ENABLED
#include "adk_bfd_dp_proc_fn_ids.h"
#endif

#ifdef ADK_MODULE_TWAMP_ENABLED
#include "adk_twamp_dp_proc_fn_ids.h"
#endif


	ADK_PAL_ERR_HANDLER_FUNC_ID = 255,
	ADK_MAX_PROC_FUNC_ID = 256,
        /* Maximum func id, if it crosses the limit (255) then error out at
         * compile time itself */
} adk_pal_proc_fn_id_e;

#endif

/*! end addtogroup _sif
 * @}
 */
