/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_sys.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of objects that are
 * part of system infrastructure. This file needs to be included to access
 * APIs and data structures defined by ADK SIF.
 *
 ***********************************************************************/

#ifndef __ADK_SYS_H__
#define __ADK_SYS_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "ncp_types.h"
#include "ncp_eioa.h"

#include <adk_sys_types.h>
#include <adk_error.h>

#include <adk_dpdk.h>
#include "rte_timer.h"
#include <rte_bridge_eth.h>
#include <rte_bridge_cppi.h>
#include <rte_bridge_cppi_priv.h>

#include <adk_config_overrides.h>
#include "adk_sys_config.h"

/*ADK MetaData offsets and size */
#define ADK_META_DATA_SIZE      128

#ifndef ADK_DP_NO_BATCH_PROC
#define ADK_DP_BULK_RX_SIZE        8
#else
#define ADK_DP_BULK_RX_SIZE        1
#endif

#define ADK_DP_TASK_MULTIPLY_MAX   8 /*Currently fragmentation is the only possibility*/

/*#define ADK_DP_LOOP_RX_COUNT 1*/
#define INVALID_FRAG_COUNT           0xFF
#define ADK_MPP_NUM_GLOBAL_REGISTERS 16
#define ADK_MPP_NUM_IPARAM_REGISTERS  8
#define ADK_MPP_PDU_SIZE_IN_BYTES    64

#define ADK_DP_VPORT_NUM_TX_QUEUES  128   /*Depending upon the number of FL schedulers, this number may change*/

/* TODO: Most of the following are ADK internal declarations or belongs to the config file */

/*! Macros to user defined meta data/task parameters */

#define ADK_CONVERT_TPARAMS_TO_HDL(__PARAMS) ((adk_flow_handle_t)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT8(__PARAMS) ((uint8_t)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT8PTR(__PARAMS) ((uint8_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT16(__PARAMS) ((uint16_t)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT16PTR(__PARAMS) ((uint16_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT32(__PARAMS) ((uint32_t)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINT32_PTR(__PARAMS) ((uint32_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_PIPELINE_INFO(__PARAMS) ((adk_sif_cfg_pipeline_info_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_MODULE_FN(__PARAMS) ((module_cpu_fn_t)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_PKTIO_TASK(__PARAMS) ((adk_pktio_task_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_SPD_RET_INFO(__PARAMS) ((adk_ipsec_spd_ret_info_t *)(__PARAMS))
#define ADK_CONVERT_TPARAMS_TO_UINTPTR(__PARAMS) ((uintptr_t *)(__PARAMS))

#define ADK_CONVERT_TO_TPARAMS(__PARAMS) ((uintptr_t)(__PARAMS))

/* Define ADK_TASK_USER_PARAMS only if not defined already.
 * It could be defined due to other frameworks such as ODP.
 */
/* NOTE:
 * This requires that the other framework specific header file(s) are included before adk_sys.h!!!!!!
 */
#ifndef ADK_TASK_USER_PARAMS
#define ADK_TASK_USER_PARAMS(task, i) (((uintptr_t *)((uint8_t *)(task)+sizeof(struct rte_mbuf)+ADK_META_DATA_OFFSET))[(i)])
#endif

#ifndef ADK_TASK_PH_PARAMS
#define ADK_TASK_PH_PARAMS(task, i) (((uintptr_t *)((((adk_ph_pdu_buf_rx_t *)task))->pduParams))[(i)])
#endif
#ifndef ADK_PH_PARAMS
#define ADK_PH_PARAMS(phParams, i) (((uintptr_t *)(phParams))[(i)])
#endif
#define ADK_TASK_PDU_SIZE(task)       (task)->pkt_len
#define ADK_TASK_SEGMENT_SIZE(task)       (task)->data_len

#define ADK_PDU_PTR_GET(task, pduPtr, __pduPtrType__) \
    ((pduPtr) = rte_pktmbuf_mtod((struct rte_mbuf *)(task), __pduPtrType__))
#define ADK_PDU_OFFSET_PTR_GET(task, pduPtr, __pduPtrType__, offset) \
    ((pduPtr) = rte_pktmbuf_mtod_offset((struct rte_mbuf *)(task), __pduPtrType__, offset))
#define ADK_PDU_HEAD_PUSH(task, pduPtr, __pduPtrType__, len) ((pduPtr) = __pduPtrType__ rte_pktmbuf_prepend((struct rte_mbuf *)(task), (len)))
#define ADK_PDU_HEAD_PULL(task, pduPtr, __pduPtrType__, len) ((pduPtr) = __pduPtrType__ rte_pktmbuf_adj((struct rte_mbuf *)(task), (len)))
#define ADK_PDU_TAIL_PUSH(task, len) (rte_pktmbuf_append((struct rte_mbuf *)(task), (len)))
#define ADK_PDU_TAIL_PULL(task, len) (rte_pktmbuf_trim((struct rte_mbuf *)(task), (len)))
#define ADK_TASK_CHAIN(head_task, tail_task) \
    rte_pktmbuf_chain((struct rte_mbuf *)head_task,(struct rte_mbuf *)tail_task);

#define ADK_PREFETCH_FOR_READ3(ADDR) __builtin_prefetch((ADDR),0,3)

#define ADK_PREFETCH_FOR_WRITE3(ADDR) __builtin_prefetch((ADDR),1,3)

#define ADK_PREFETCH_FOR_READ0(ADDR) __builtin_prefetch((ADDR),0,0)

#define ADK_PREFETCH_FOR_WRITE0(ADDR) __builtin_prefetch((ADDR),1,0)


/* adk_sys_event_metadata structure
 ****This is used only in DL path. ADK -> EO**
 * 1. ip_offset
 ****This is used only in UL path. EO -> ADK**
 * 2. udp_cksum_gen (1->Generate UDP checksum and 0->Don't generate UDP checksum)
 */
struct adk_sys_event_metadata
{
    char reserved0[64];
    union {
        char meta[80];
        struct
        {
#ifdef ADK_PAL_CYCLE_MEASUREMENT
#ifndef ADK_PAL_CYCLE_MEASUREMENT_MBUF_UDATA
            char reserved1[49];
#endif
#else
            char reserved1[41];
#endif
            uint16_t ip_offset;
            uint8_t udp_cksum_gen;
        };
    };
};

/*
 * ADK task type definition
 */
typedef enum __attribute__ ((__packed__))  {
    ADK_TASK_TYPE_NEW = 0,
    ADK_TASK_TYPE_FWD,
} adk_task_type_e;

/*!
 * We need a host port so that we can map PDUs that originate
 * from the host/fastpath
 */
#define ADK_FP_ORIGIN_DOMAIN_PORT 0x3f
#define ADK_FP_OUTPUT_DOMAIN_PORT 0xff

#ifdef ADK_HANDLE_VOID_PTR

#define ADKPRIh "p"
#define ADKSCNh "p"
/*!
 * adk_handle_t definition. It is mapped to unsigned integer. This way
 * pass both pointers and integer values as handle.
 */
typedef struct adk_handle_s *adk_handle_t;

/*!
 * adk_sys_handle_t - this handle is returned to the caller of
 * adk_sys_init API. The application is expected pass this handle
 * as a input argument to almost all APIs.
 */
typedef struct adk_sys_handle_s *adk_sys_handle_t;

/*!
 * module handle type. This handle is returned by the
 *  adk_module_register API.
 */
typedef struct adk_module_handle_s *adk_module_handle_t;

/*!
 * adk_interface_handle_t identifies an interface within a module. The
 * interface handle internally gives a pointer to a generic interface
 * data structure adk_interface_info_t. The modules that create interface
 * must fill in appropriate fields in the adk_interface_info_t data structure
 * and returns a pointer to that as the flow handle.
 */
typedef adk_handle_t adk_interface_handle_t;

/*!
 * adk_flow_handle_t identifies a flow within a module. The flow
 * handle internally gives a pointer to a generic flow
 * data structure adk_flow_info_t. The modules that create flows
 * must fill in appropriate fields in the adk_flow_info_t data structure
 * and returns a pointer to that as the flow handle.
 */
typedef adk_handle_t adk_flow_handle_t;

/*!
 * adk_config_handle_t. This handle is returned to the caller of
 * adk_sys_config API.
 */
typedef adk_handle_t adk_config_handle_t;

#else /* ADK_HANDLE_VOID_PTR */

#define ADKPRIh "#08x"
#define ADKSCNh "08x"
/*!
 * adk_handle_t definition. It is mapped to unsigned integer. This way
 * pass both pointers and integer values as handle.
 */
typedef uint64_t adk_handle_t;

/*!
 * adk_sys_handle_t - this handle is returned to the caller of
 * adk_sys_init API. The application is expected pass this handle
 * as a input argument to almost all APIs.
 */
typedef adk_handle_t adk_sys_handle_t;

/*!
 * module handle type. This handle is returned by the
 *  adk_module_register API.
 */
typedef adk_handle_t adk_module_handle_t;

/*!
 * adk_interface_handle_t identifies an interface within a module. The
 * interface handle internally gives a pointer to a generic interface
 * data structure adk_interface_info_t. The modules that create interface
 * must fill in appropriate fields in the adk_interface_info_t data structure
 * and returns a pointer to that as the flow handle.
 */
typedef adk_handle_t adk_interface_handle_t;

/*!
 * adk_flow_handle_t identifies a flow within a module. The flow
 * handle internally gives a pointer to a generic flow
 * data structure adk_flow_info_t. The modules that create flows
 * must fill in appropriate fields in the adk_flow_info_t data structure
 * and returns a pointer to that as the flow handle.
 */
typedef adk_handle_t adk_flow_handle_t;


/*!
 * adk_config_handle_t. This handle is returned to the caller of
 * adk_sys_config API.
 * adk_sys_config API.
 */
typedef adk_handle_t adk_config_handle_t;
#endif /* ADK_HANDLE_VOID_PTR */

typedef uint16_t adk_eth_device_id_t;

/*!
 * =============================================================
 * @enum adk_port_number_t
 *
 *
 * EIOA Port numbers in ACP and AXM based systems.
 *
 * The application is expected to pass this as port number to the
 * ADK API's to configure TN/IC ports. For TN ports, Interface
 * creation is done through the Ethernet module. For IC ports,
 * the configuration is done through the BP_IWF module.
 * Netdriver and PH present this as port number to application
 * for the CPU bound packets.
 *
 *
 * =============================================================
 */
/* TODO: Update ADK_EIOAx_PORTy mapping */
typedef enum adk_port_number_e
{
    ADK_EIOA0_PORT0 = NCP_XGMAC0,
    ADK_EIOA0_PORT1 = NCP_XGMAC1,
    ADK_EIOA0_PORT2 = NCP_GMAC2,
    ADK_EIOA0_PORT3 = NCP_GMAC3,
    ADK_EIOA0_PORT4 = NCP_GMAC4,
    ADK_EIOA1_PORT0 = NCP_XGMAC16,
    ADK_EIOA1_PORT1 = NCP_XGMAC17,
    ADK_EIOA1_PORT2 = NCP_GMAC18,
    ADK_EIOA1_PORT3 = NCP_GMAC19,
    ADK_EIOA1_PORT4 = NCP_GMAC20,
    ADK_EIOA2_PORT0 = NCP_XGMAC32,
    ADK_EIOA2_PORT1 = NCP_XGMAC33,
    ADK_EIOA3_PORT0 = NCP_XGMAC48,
    ADK_EIOA3_PORT1 = NCP_XGMAC49,
    ADK_EIOA4_PORT0 = NCP_XGMAC64,
    ADK_EIOA4_PORT1 = NCP_XGMAC65,
    ADK_EIOA5_PORT0 = NCP_XGMAC80,
    ADK_EIOA5_PORT1 = NCP_XGMAC81,
    ADK_EIOA6_PORT0 = NCP_XGMAC96,
    ADK_EIOA6_PORT1 = NCP_XGMAC97,
    ADK_EIOA7_PORT0 = NCP_XGMAC112,
    ADK_EIOA7_PORT1 = NCP_XGMAC113,
    /*
     * We need a host port so that we can map PDUs that originate
     * from the host/fastpath
     */
    ADK_HOST_PORT   = ADK_FP_OUTPUT_DOMAIN_PORT
} adk_port_number_t;

/*!
 * =============================================================
 * @enum adk_sys_table_group_t
 *
 * @brief
 *
 * This enumeration defines different group of tables that can be
 * dumped in user readable csv format.
 * =============================================================
 */
typedef enum adk_sys_table_group_s {
    /*! Non inclusive lower bound on table group type */
    ADK_SYS_TABLES_NONE             = 0,
    /*! SYS flow binding related tables */
    ADK_SYS_TABLES_BIND             = (1 << 1),
    /*! SYS QoS related tables */
    ADK_SYS_TABLES_QOS              = (1 << 2),
    /*! SYS exceptions related tables */
    ADK_SYS_TABLES_EXCEPTIONS       = (1 << 3),
    /*! SYS Netdriver related tables */
    ADK_SYS_TABLES_NETDRIVER        = (1 << 4),
    /*! SYS Port ID Mapping related tables */
    ADK_SYS_TABLES_PORTID_MAPPING   = (1 << 5),
    /*! SYS Control Point (CP) Mapping related tables */
    ADK_SYS_TABLES_CP_MAPPING       = (1 << 6),
    /* Inclusive upper bound on table group type for all tables */
    ADK_SYS_TABLES_ALL              = ((1 << 7) - 1)
} adk_sys_table_group_t;



/*!
 * log level definitions.
 */

/*!
 * =============================================================
 * @enum adk_log_level_t
 *
 * @brief
 *
 * defines the logging levels supported by the ADK log library.
 * The values here are mapped to error severity levels defined
 * in adk_error.h. At the system initialization time, the user
 * can configure a default logging level for the entire system.
 * The user can overwrite this on a per module basis by calling
 * adk_module_attr_set API.
 *
 * =============================================================
 */
typedef enum adk_log_level_e
{
    ADK_LOG_LEVEL_DEBUG = ADK_SEV_LEVEL_DEBUG,
    ADK_LOG_LEVEL_INFO = ADK_SEV_LEVEL_INFO,
    ADK_LOG_LEVEL_WARNING = ADK_SEV_LEVEL_WARNING,
    ADK_LOG_LEVEL_ERROR = ADK_SEV_LEVEL_ERROR,
    ADK_LOG_LEVEL_FATAL = ADK_SEV_LEVEL_FATAL,
    ADK_LOG_LEVEL_DISABLE = ADK_SEV_LEVEL_DISABLE
} adk_log_level_t;

#define ADK_LOG_LEVEL_MASK      ADK_LOG_LEVEL_INFO

/*!
 * =============================================================
 *
 * @typedef adk_sys_log_type_t
 *
 * @brief   The various SYS logging types
 *
 *  This data type is used for enabling or disabling different types of logs
 *  that are available in SIF. Multiple log types can be enabled or disabled
 *  by specifying logical OR of these log-types. Note that, these need to be
 *  enabled or disabled explicitly as these do not fall under
 *  ADK_LOG_LEVEL_INFO category.
 *
 *  SYS_LOG_FUNCS       For logging entry to all internal/external functions
 *
 *  SYS_LOG_HW          For logging all access to EIOA hardware
 *
 *  SYS_LOG_PTN         For logging all learning/unlearning of Tree entries
 *
 *  SYS_LOG_SUBNET      For logging all the Trees subnet operations
 *
 *  SYS_LOG_EXT_API     For logging all external API calls at ADK boundary
 *
 * =============================================================
 */
typedef uint32_t    adk_sys_log_type_t;

#define SYS_LOG_FUNCS        (1 << 4)
#define SYS_LOG_HW           (1 << 5)
#define SYS_LOG_PTN          (1 << 6)
#define SYS_LOG_SUBNET       (1 << 7)
#define SYS_LOG_EXT_API      (1 << 8)

/*!
 * =============================================================
 * @enum adk_bind_type_t
 *
 *
 * different types of bind support by the ADK.
 *
 * =============================================================
 */
typedef enum adk_bind_type_e
{
    ADK_BIND_UNKNOWN = 0,
    ADK_BIND_MODULE_TO_MODULE = 1,
    ADK_BIND_FLOW_TO_MODULE,
    ADK_BIND_FLOW_TO_FLOW,
    /*! 32-bit ID to module */
    ADK_BIND_ID_TO_MODULE,
    /*! Flow to 32-bit ID */
    ADK_BIND_FLOW_TO_ID,
    /*! 32-bit ID to flow */
    ADK_BIND_ID_TO_FLOW
} adk_bind_type_t;

/*!
 * =============================================================
 * @enum adk_bind_module_location_t
 *
 *
 * indicates whether the concerned module function is located on
 * CPU or hardware engines.

 * =============================================================
 */
typedef enum adk_bind_module_location_e
{
    ADK_BIND_LOC_CPU = 1,
    ADK_BIND_LOC_ENGINES
} adk_bind_module_location_t;

/*!
 * =============================================================
 * @enum adk_bind_processing_direction_t
 *
 *
 * indicates whether the destination module is in the ingress
 * or egress processing path. Ingress processing indicates that
 * the PDU is traversing from Lower layer stack to an upper layer
 * stack (e.g. Ethernet->IP, IP->TCP). Egress processing indicates
 * that the PDU is traversing from upper layer module to lower
 * layer module (e.g. TCP->IP, IP->Ethernet, RLC->MAC). Egress
 * processing is also used to indicate the binding between ingress
 * processing path and egress processing path of same module
 * (e.g. IP->IP(L3 forwarding)).
 *
 * =============================================================
 */
typedef enum adk_bind_processing_direction_e
{
    ADK_BIND_INGRESS_PROCESSING = 1,
    ADK_BIND_EGRESS_PROCESSING
} adk_bind_processing_direction_t;

/*!
 * =============================================================
 * @enum adk_bind_processing_path_t
 *
 *
 * indicates whether the bind is implemented using direct or
 * in-direct path. In the direct path, MPP/CPU hands off the PDU
 * and the required parameters to the destination module by direct
 * function calls. In the indirect path, a pipeline is launched
 * to transport the PDU to the destination module. Note: pipeline
 * field in adk_bind_info_t is valid only if the bind path is
 * configured for indirect mode.
 *
 * =============================================================
 */
typedef enum adk_bind_processing_path_e {
    ADK_BIND_PATH_DIRECT = 1,
    ADK_BIND_PATH_INDIRECT
} adk_bind_processing_path_t;   /* ------  end of enum adk_bind_processing_path_t  ------ */


/*!
 * =============================================================
 * @enum adk_sys_heartbeat_mode_t
 *
 * Indicates the Heartbeat Mode.
 * ADK_SYS_HEARTBEAT_MODE_ENGINE - Heartbeat packet sequence
 *                                 involves MPP->SED->MTM->MPP.
 * ADK_SYS_HEARTBEAT_MODE_CPU    - Heartbeat packet sequence
 *                                 involves CPU->MPP->SED->MTM->MPP->CPU
 *
 * =============================================================
 */
typedef enum adk_sys_heartbeat_mode_e {
    /*! Engine Heartbeat Mode */
    ADK_SYS_HEARTBEAT_MODE_ENGINE = 1,
    /*! CPU Heartbeat Mode */
    ADK_SYS_HEARTBEAT_MODE_CPU = 2,
}adk_sys_heartbeat_mode_t;

/*!
 * =============================================================
 * @enum adk_sys_config_mode_t
 *
 * Indicates the ADK setup mode.
 * ADK_SYS_CONFIG_MODE_STANDALONE - ADK with full configuration. ADK does
 *                       DP thread creation, mempool configuration,
 *                       port/queue configuration, ADK module
 *                       staging, and packet polling/distrbution.
 * ADK_SYS_CONFIG_MODE_SLAVE      - ADK works as a library. DP thread creation,
 *                       mempool configuration, port/queue configuration,
 *                       ADK module staging, and packet polling/distrbution
 *                       are not in ADK scope.
 * ADK_SYS_CONFIG_MODE_DEDICATED_DATAPLANE - ADK works in dedicated dataplane mode.
 *                       
 * ADK_SYS_CONFIG_MODE_SHARED_DATAPLANE - ADK works in shared dataplane mode.
 *
 * =============================================================
 */
typedef enum adk_sys_config_mode_e {
    /*! In standalone mode, ADK manages configuration on its own. */
    ADK_SYS_CONFIG_MODE_STANDALONE = 0,
    /*! In slave mode, ADK works as a library. */
    ADK_SYS_CONFIG_MODE_SLAVE,
    ADK_SYS_CONFIG_MODE_DEDICATED_DATAPLANE,
    ADK_SYS_CONFIG_MODE_SHARED_DATAPLANE,
    ADK_SYS_CONFIG_MODE_MAX
}adk_sys_config_mode_t;

/*!
 * =============================================================
 * @enum adk_sys_config_process_types_t
 *
 * Indicates the ADK process type.
 * ADK_SYS_CONFIG_PROCESS_TYPE_PRIMARY   - Primary process.
 * ADK_SYS_CONFIG_PROCESS_TYPE_SECONDARY - Secondary process.
 *
 * =============================================================
 */
typedef enum adk_sys_config_process_types_t{
    /*! Primary process. System initialization is done in primary process.*/
    ADK_SYS_CONFIG_PROCESS_TYPE_PRIMARY = 0,
    /*! Secondary process. Secondary process attaches to the primary process.*/
    ADK_SYS_CONFIG_PROCESS_TYPE_SECONDARY,
    ADK_SYS_CONFIG_PROCESS_TYPE_MAX
}adk_sys_config_process_types_t;

/*!
 * =============================================================
 * @typedef adk_cfg_read_fn_t
 *
 * @brief
 *
 * Function prototype for reading and loading the configuration file on ACP.
 *
 * =============================================================
 */
typedef int (*adk_cfg_read_fn_t) (void *arg, void *buf, int len);

/*!
 * The function pointer to the module initialization function.
 * All the modules must follow the same prototype for the
 * module initialization function.
 */
typedef adk_status_t (*adk_module_init_func_t) (
        adk_module_handle_t);

/*!
 * The function pointer to the module's service function
 * update function.
 */
typedef adk_status_t (*adk_module_service_function_update_func_t) (
        adk_module_handle_t);

/*!
 * The function pointer to the module's resource notify function.
 */
typedef adk_status_t (*adk_module_resource_notify_func_t) (
        adk_module_handle_t, adk_sys_rsrc_oper_t, 
        adk_sys_resource_type_t, adk_sys_resource_info_t);

/*!
 * =============================================================
 * @typedef adk_bind_imc_service_fn_t
 */
typedef adk_status_t (*adk_bind_imc_service_fn_t) (adk_sys_handle_t,
        adk_module_handle_t, const void *);

/*!
 * =============================================================
 * @typedef adk_bind_cpu_imc_service_fn_t
 */
typedef adk_status_t (*adk_bind_cpu_imc_service_fn_t) (adk_sys_handle_t,
        adk_module_handle_t, const void *);

/*!
 * =============================================================
 * @struct adk_bind_info_t adk_sys.h "adk_sys.h"
 *
 *
 * adk_bind_info_t is the input parameter to adk_bind API. The
 * information provided here is used update hardware/software
 * resources that are required to support ADK bind.
 *
 * =============================================================
 */
typedef struct adk_bind_info_s
{
    /*! this field describes type of bind. supported types are
     * ADK_BIND_MODULE_TO_MODULE, ADK_BIND_FLOW_TO_MODULE,
     * ADK_BIND_FLOW_TO_FLOW, ADK_BIND_ID_TO_MODULE,
     * ADK_BIND_ID_TO_FLOW and ADK_BIND_FLOW_TO_ID  */
    adk_bind_type_t                 type;
    /*! specifies whether the destination module is in the ingress
     * processing path or egress processing path */
    adk_bind_processing_direction_t direction;

    /*! bind processing path, ADK_BIND_PATH_DIRECT or
     *  ADK_BIND_PATH_INDIRECT. See adk_bind_processing_path_t
     *  for more description.
     */
    adk_bind_processing_path_t path;
    /*! module handle for the source module.
     */
    adk_module_handle_t             srcModuleHandle;
    /*! flow handle for the source flow. For now assumption is
     *  fastpath flow ID and flow handles are same. We may have to add
     *  a new service function in the module manager to convert flow
     *  handle to flow ID if they are not the same. For CPU originated
     *  modules the handle would represent the flow or a meaningful
     *  value to identify the PDU within a module.
     */
    adk_flow_handle_t               srcFlowHandle;
    /*! This is valid for the bind type ADK_BIND_ID_TO_FLOW and
     *  ADK_BIND_ID_TO_MODULE. This is the 32-bit flow ID specific
     *  to the source module. Note that this is NOT the 24-bit ADK name
     *  space flow ID.
     */
    uint32_t                    srcFlowID;
    /*! specifies the location of the destination module. CPU or MPP */
    adk_bind_module_location_t      srcLocation;
    /*! module handle for the destination module */
    adk_module_handle_t             destModuleHandle;
    /*! flow handle for the source flow */
    adk_flow_handle_t               destFlowHandle;
    /*! This is valid for the bind type ADK_BIND_FLOW_TO_ID. This is
     *  the 32-bit flow ID specific to the destination module. Note that
     *  this is NOT the 24-bit ADK name space flow ID.
     */
    uint32_t                    dstFlowID;
    /*! specifies the location of the destination module. CPU or MPP */
    adk_bind_module_location_t      destLocation;

    union {
        /*! specifies the pointer to inter module communication service
         *  function if the source module resides on CPU. This field is
         *  ignored if source module resides on MPP */
        adk_bind_imc_service_fn_t       cpuImcFn;
        /*! specifies the name of the inter-module comm service function
         *  if the source module resides on MPP. This field is ignored if
         *  the source module resides on CPU. The name should be specified
         *  as a char * like "adk_atm.atm_ip_imc_function" */
        char                            *mppImcFn;
    } imc_u;

    /*! PDU handler bind handle (of type adk_ph_pipeline_bind_handle_t).
     * Application is expected to create all PDU handler objects
     * (receive group, receive queue, pipeline bind etc) and register
     * destination module's receiving function with PH using appropriate
     * call back index. This parameter is required only if the destination
     * module resides on CPU.
     */
    adk_handle_t                        phBindHandle;
    /*! PDU handler call back index. Call back index using which the
     * destination module's receive function is registered with the
     * PH. This parameter is required only if the destination module
     * resides on CPU.
     */
    uint32_t                        phCbIndex;
    /*! isCustomBind, determines whether the configuration is for
     *  custom bind. Set this field to TRUE if custom bind is required.
     *  Note: Not all ADK modules support custom bind. Please refer
     *  to the module specific guide for more information.
     */
    bool                      isCustomBind;

    /*! pointer to the custom bind/unbind parameter if the bind function is
     *  owned by the source module. In such cases, custom bind function
     *  is called with this field as one of the input parameters. */
    void                            *customBindParams_p;
} adk_bind_info_t;

/*!
 * =============================================================
 * @struct adk_bind_info_t adk_sys.h "adk_sys.h"
 *
 *
 * adk_bind_info_t is the input parameter to adk_bind API. The
 * information provided here is used update hardware/software
 * resources that are required to support ADK bind.
 *
 * =============================================================
 */
typedef struct adk_sys_conf_user_args_s
{
    int32_t argc;
    char **argv_p;
} adk_sys_conf_user_args_t;

/*!
 * =============================================================
 * @struct adk_sys_conf_t adk_sys.h "adk_sys.h"
 *
 * @brief
 *
 * ADK system configuration structure. This structure is an input
 * parameter to adk_sys_init API call. Depending upon the values in this
 * structure, one of the following ways is followed to load the configuration
 * file and get the NCP handle.
 *
 * 1. If configuration file name is provided in configFile_p then ADK uses
 * fopen/fread system calls to read and load the configuration file.
 *
 * 2. If pointer to configuration file is provided in config_p and
 *  a. If function to read the configuration file is provided in read_fn, then
 *     ADK uses this read_fn to read and load the configuration file.
 *  b. If function to read the configuration file is NOT provided in read_fn,
 *     then ADK uses fread system call to read and load the configuration file.
 *     It is assumed here that the application has already called fopen system
 *     call and has the FILE* which is being passed in config_p.
 *
 * 3. Providing both pointer to configuration file i.e. config_p and
 *    configuration file name i.e. configFile_p is treated as an error.
 *
 * 4. If both are not provided, then ADK assumes that the configuration file has
 *    already been loaded and tries to attached to the loaded configuration
 *    file.
 *
 * =============================================================
 */
typedef struct adk_sys_conf_s
{
    /*! The instance of the chip initialized. Currently only one instance is
     * supported and the value for that must be specified as 1.
     */
    uint32_t chipInstance;
    /*! Flag to enable support for multi-threaded application at the system
     * level. If set to TRUE, then all the module APIs use OS provided
     * locking mechanisms to ensure thread safety, otherwise, this features
     * is turned off on the assumption that application is single threaded. */
    bool multiThread;
    /*! The default logging level followed by the ADK software. This
     *  level is applied to all modules that are registered and can
     *  be modified on a per module basis by calling
     *  adk_module_attr_set API. */
    uint32_t logLevel;
    /*! Hardware configuration file name. This should be NULL is below
     * configuration pointer has been provided i.e. config_p is non-NULL */
    char *configFile_p;
    /*! Pointer to hardware configuration file. This should be NULL if above
     * configuration file name has been provided i.e. configFile_p is non-NULL */
    void *config_p;
    /*! Function provided by the application to read the configuration file
     * using the above config_p pointer */
    adk_cfg_read_fn_t read_fn;
    /*! User defined arguments */
    void *userArgs_p;
} adk_sys_conf_t;

#ifdef ADK_QOS_GEN3_INTERFACE
typedef struct adk_sys_sp_stack_attr_s {
            /*!
             * Struct that represents the dev_net_id when the packet is forwarded to
             * CPU when cpu processing path is SP STACK.
             */
            uint32_t dev_net_id;
            uint32_t queue_id;     /* Future proof. Un-used */
} adk_sys_sp_stack_attr_t;
#endif

/*!
 * =============================================================
 * @struct adk_restart_mode_t adk_sys.h "adk_sys.h"
 *
 * @brief
 *
 * Defines possible warm-restart options.
 */
typedef enum adk_restart_mode_e
{
    /*! Warm restart mode. All the previously stored states are retrieved
     * in this mode.
     */
    ADK_WARM_RESTART=1,

    /*! Cold reboot mode. The system is completely reinitialized in this mode.
     * Application has to call adk_sys_init API after this call.
     */
    ADK_COLD_RESTART=2

} adk_restart_mode_t;

/*!
 * =============================================================
 * @enum adk_module_attr_type_t
 *
 * @brief
 *
 * Data structure used for external APIs supported by module manager
 *
 * =============================================================
 */
typedef enum adk_module_attr_type_s
{
    ADK_MODULE_LOG_LEVEL=1
} adk_module_attr_type_t;

/*!
 * =============================================================
 * @struct adk_module_attr_info_t adk_sys.h "adk_sys.h"
 *
 * @brief
 *
 * defines the data structure that is passed in as input parameter
 * to adk_sys_attr_set API.
 *
 * =============================================================
 */
typedef struct adk_module_attr_info_s
{
    adk_module_attr_type_t attrType;
    /*!
     * union of attribute values. This field at any given instance
     * correspond to the attrType selected at that instance.
     */
    union module_attr
    {
        uint32_t logLevel;
    } attr_val_u;
} adk_module_attr_info_t;

/*!
 * =============================================================
 * @struct adk_sys_global_stats_t adk_sys.h "adk_sys.h"
 *
 * @brief
 *
 * ADK Global Statistics maintained by SIF. This mostly consists of counters
 * related to hardware errors in MPP or ADK bind related counters. See the F PL
 * Programming Guide, section F PL Trap and Error Codes for more information on
 * the error counters.
 *
 * =============================================================
 */
typedef struct adk_sys_global_stats_s
{
    uint32_t    reserved;
    uint32_t    error_flow_overrun;
    uint32_t    error_flow_underrun;
    uint32_t    error_tree_overrun;
    uint32_t    error_tree_underrun;
    uint32_t    error_undef_tree_instr;
    uint32_t    error_undef_flow_instr;
    uint32_t    error_undef_internal_func;
    uint32_t    error_stack_overflow;
    uint32_t    error_stack_underflow;
    uint32_t    error_fdc_hw_err;
    uint32_t    error_tree_mem_read_err;
    uint32_t    error_fc_read_err;
    uint32_t    error_pdu_data_read_err;
    uint32_t    error_fbi_bound_err;
    uint32_t    error_pim_mem_err;
    uint32_t    error_semaphore_err;
    uint32_t    error_pic_data_err;
    uint32_t    error_he_data_read_err;
    uint32_t    error_se_data_read_err;
    uint32_t    error_fbi_ord_gen_func_err;
    uint32_t    error_fbi_undef_trap_err;
    uint32_t    error_arg_overflow_err;
    uint32_t    error_frp_hw_err;
    uint32_t    error_stack_rdata_err;
    uint32_t    error_handle_rdata_err;
    uint32_t    error_global_rdata_err;
    uint32_t    error_undef_reg_err;
    uint32_t    error_task_err;
    uint32_t    error_software_err;

    uint32_t    unknown_hw_err;
    uint32_t    reserved1;

    /*!
     * Total number of PDUs dropped by the F PL error handler
     */
    uint32_t    sif_total_drops;
    /*!
     * Total number of failures when looking up a bind
     */
    uint32_t    bind_lookup_fail;
    /*!
     * Total number of times there was no entry in the destination demux tree
     */
    uint32_t    dest_dmux_lookup_fail;
    /*!
     * Total number of time when binding to the CPU, there was no handler for
     * the PH path
     */
    uint32_t    bind_ph_call_fail;
    /*!
     * Total number of PDUs forwarded to the CPU
     */
    uint32_t    bind_fwd_cpu_cnt;
    /*!
     * Total number of indirect binds that have happened
     */
    uint32_t    bind_indirect_cnt;
    /*!
     * Total number of direct binds that have happened
     */
    uint32_t    bind_direct_cnt;


    /*!
     *  Count of the number of misses in sif_netDriver_demux_tree lookup.
     *  This would indicate an invalid configuration
     */
    uint32_t    netDriver_demux_no_match_error;

    /*!
     *  back pressure counter
     */
    uint32_t    backpressure_counter;

    /*!
     *  back pressure status
     */
    uint32_t    backpressure_status;

    /*!
     *  MTM back pressure counter
     */
    uint32_t    mtm_backpressure_counter;

    /*!
     *  SED back pressure counter
     */
    uint32_t    sed_backpressure_counter;

    /*!
     *  VP PDU MEM back pressure counter
     */
    uint32_t    vp_pdu_mem_backpressure_counter;

    /*!
     *  back pressure mask
     */
    uint32_t    backpressure_mask;

    /*!
     *  SPP back pressure counter
     */
    uint32_t    spp_backpressure_counter;

    /*!
     *  PAB back pressure counter
     */
    uint32_t    pab_backpressure_counter;

    /*!
     *  PIC back pressure counter, currently this counter is not supported.
     *  place holder for future extention.
     */
    uint32_t    pic_backpressure_counter;

    /*!
     *  Counter that counts the number of packets arrived on MPP TQ0,
     *  that are processed in memory backpresure condition
     */
    uint32_t    backpressure_tq0_counter;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ1, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq1_drop_counter;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ2, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq2_drop_counter;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ3, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq3_drop_counter;

    /*!
     *  Counter that counts number of packets generated by ADK based on timer event
     *  and droped due to backpressure in system
     */
    uint32_t    backpressure_timer_event_based_adk_gen_pkts_drop_count; /* 0xf8 */


    /*!
     * This is the base address used for logging of the data when the error
     * handler is called. It is initially set to ADK_SIF_HW_ERROR_LOG and F PL
     * code will update to to be ADK_SIF_HW_ERROR_LOG_2
     */
    uint32_t    sif_hw_logging_base_addr; /* 0xfc */

} adk_sys_global_stats_t;

/*!
 * =============================================================
 * @struct adk_sys_overload_drop_stats_t adk_sys.h "adk_sys.h"
 *
 * @brief
 *
 * ADK Overload protection drop Statistics maintained by SIF.
 *
 * =============================================================
 */
typedef struct adk_sys_overload_drop_stats_s
{

    /*!
     * Resreved for future use
     */
    uint32_t    reserved1;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ1, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq1_drop_counter;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ2, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq2_drop_counter;

    /*!
     *  Counter that counts number of packets, arrived on MPP TQ3, droped
     *  due to backpressure in system
     */
    uint32_t    backpressure_tq3_drop_counter;

    /*!
     *  Counter that counts number of packets generated by ADK based on timer event
     *  and droped due to backpressure in system
     */
    uint32_t    backpressure_timer_event_based_adk_gen_pkts_drop_count; /* 0xf8 */

    /*!
     * Resreved for future use
     */
    uint32_t    reserved2;

    /*!
     * Resreved for future use
     */
    uint32_t    reserved3;

} adk_sys_overload_drop_stats_t;

#if 1
/*!
 * =============================================================
 * @struct adk_sys_global_pdu_data_t adk_sys.h "adk_sys.h"
 *
 * @brief This is the debug dump of the MPP state and PDU
 *
 * When an MPP error happens, the error is counted. This is described in
 * adk_sys_global_stats_t. The MPP state is
 *    - Error Code
 *    - PDU Length
 *    - PDU Offset
 *    - MPP Root Address
 *    - Task Header
 *    - iParam Registers
 *    - Global Registers
 *    - First 64 bytes of the PDU
 *
 * There are two instances of this data, the first for the first trap that
 * occurred and the second would be the last trap that occurred.
 *
 * =============================================================
 */
typedef struct adk_sys_global_pdu_data_s {
    /*!
     * MPP Error code. See F PL Programming Guide, section F PL Trap and Error
     * Codes. For First, this will be the first error. With Last, this will be
     * the last 4 errors, with the LSB being the more resent trap. For
     * example, if errorCode is 0x02010316, 0x16 is the more resent error.
     */
    uint32_t    errorCode;
    /*!
     * Length of the received PDU; $pduLength
     */
    uint16_t    pduLength;
    /*!
     * Current offset into the PDU when the error happened; $pduOffset
     */
    uint16_t    pduOffset;
    /*!
     * The MPP root address for this context. With the FPO symbol file
     * (*.fps), the entry point for the context can be determined.
     */
    uint32_t    rootAddress;
    /*!
     * Header information from the task that the F PL program is currently
     * processing. This information is composed of multiple smaller
     * variables/registers individually accessible by F PL. The Axxia
     * Communication Processor Technical Guide contains more information
     * about tasks and task parameters. See the F PL Programming Guide for more
     * information.
     *      - queueID:8
     *      - taskPriority:3
     *      - taskError:1
     *      - timerValid:1
     *      - launchID:18
     */
    uint32_t    taskHeader;
    /*!
     * The 8 iParam registers from the task
     */
    uint32_t    iParams[ADK_MPP_NUM_IPARAM_REGISTERS];
    /*!
     * Current value of the 16 global registers.
     */
    uint32_t    globalRegisters[ADK_MPP_NUM_GLOBAL_REGISTERS];
    /*!
     * This 32-bit value has 2 items in it; the context ID that was running
     * and the address that the error happened at. The contextID is in the
     * upper byte and the fault address in the lower 3 bytes.
     */
    uint32_t    contextId_FaultAddress;
    /*!
     * The first 64 bytes (or all if the PDU < 64 bytes) of the PDU being
     * processed.
     */
    uint8_t     pdu[ADK_MPP_PDU_SIZE_IN_BYTES];
} adk_sys_global_pdu_data_t;   /* ------  end of struct adk_sys_global_pdu_data_t  ------ */
#endif


/*!
 * =============================================================
 * @struct adk_sys_global_debug_t adk_sys.h "adk_sys.h"
 *
 * @brief Structure for the first and last set of debug data dumped in the
 * MPP
 *
 * =============================================================
 */
typedef struct adk_sys_global_debug_s {
#if 1
    adk_sys_global_pdu_data_t first;
    adk_sys_global_pdu_data_t last;
    uint32_t              errorStack[4];
    uint32_t              nsZero[4];
#endif
} adk_sys_global_debug_t;   /* ------  end of struct adk_sys_global_debug_t  ------ */


/*!
 * =============================================================
 * @struct adk_sys_version_info_t adk_sys.h "adk_sys.h"
 *
 * @brief This data structure defines the ADK system version info
 *
 * =============================================================
 */
typedef struct adk_sys_version_info_s {
    /*! ADK release version */
    const char    *adk_version;

    /*! ADK build date */
    const char    *adk_build_date;

    /*! RTE version used to build the ADK release */
    const char    *adk_rte_version;

    /*! ADK release date */
    const char    *adk_release_date;

    /*! ADK Source Code Management tag */
    const char    *adk_scm_tag;

    /*! Fastpath interface version to NetDriver */
    const char     *adk_netd_fp_intf;
} adk_sys_version_info_t;   /* ------  end of struct adk_sys_version_info_t  ------ */

/*!
 * =============================================================
 * @struct adk_sys_ipv4_addr_prefix_t
 *
 * @brief
 *
 * IPv4 subnet address which consists of 32-bit IP address
 * and prefix length.
 *
 * =============================================================
 */
typedef struct adk_sys_ipv4_addr_prefix_s {
    /*! 32-bit IPv4 address. */
    uint32_t addr;
    /*! 32-bit prefix value that indicates how many bits in the
     * addr field are used for lookup. Set this value to 32 for
     * IPv4 host address.
     */
    uint32_t prefixWidth;
} adk_sys_ipv4_addr_prefix_t;

/*!
 * =============================================================
 * @struct adk_sys_ipv6_addr_prefix_t
 *
 * @brief
 *
 * The IPv6 subnet address which consists of 128-bit IPv6 address
 * and prefix length information.
 *
 * =============================================================
 */
typedef struct adk_sys_ipv6_addr_prefix_s {
    /*! The array of 32 bit words containing the IPv6 address.
     *  The higher bytes of the IPv6 address are stored starting
     *  in addr[0].
     */
    uint32_t addr[4];
    /*! 32-bit prefix value that indicates how many bits in the
     * addr field are used for lookup. Set this value to 128 for
     * fully specified IPv6 host address.
     */
    uint32_t prefixWidth;
} adk_sys_ipv6_addr_prefix_t;

/*!
 * =============================================================
 * @enum adk_sys_ip_address_type_t
 *
 * @brief
 *
 * Enum to specify if given address is IPv4 or IPv6 address
 *
 * =============================================================
 */
typedef enum adk_sys_ip_address_type_e {
    ADK_IPV4_ADDRESS = 0,
    ADK_IPV6_ADDRESS
}adk_ip_address_type_t;

typedef struct adk_ip_address_s {
    adk_ip_address_type_t type;
    union {
        adk_sys_ipv4_addr_prefix_t v4Addr;
        adk_sys_ipv6_addr_prefix_t v6Addr;
    }addr_u; 
}adk_ip_address_t;

/*!
 * =============================================================
 * @enum adk_sys_overload_protection_state_t
 *
 * overload protection states. Different states that has to be
 * used while setting overload protection configuration.
 *
 *
 * =============================================================
 */
typedef enum adk_sys_overload_protection_state_e
{
    /*!
     * Don't change current configuration
     */
    ADK_OVRLDPROTECT_NOCHANGE  = 0,
    /*!
     * Enable overload protection
     */
    ADK_OVRLDPROTECT_ENABLE    = 1,
    /*!
     * Disable overload protection
     */
    ADK_OVRLDPROTECT_DISABLE   = 2,
} adk_sys_overload_protection_state_t;

/*!
 * =============================================================
 * @struct adk_sys_overload_protection_state_t
 *
 * Data structure that is used to represent overload protection
 * for different engines/objects in VP Cores. This is used in
 * overload protection configuration API. This is a system wide
 * setting.
 *
 * =============================================================
 */

typedef struct adk_sys_overload_protection_conf_data_e
{
    /*!
     * state to indicate whether MTM engine protection to be
     * enabled/disabled/no_change
     */
    adk_sys_overload_protection_state_t    mtm_state;
    /*!
     * state to indicate whether SED engine protection to be
     * enabled/disabled/no_change
     */
    adk_sys_overload_protection_state_t    sed_state;
    /*!
     * state to indicate whether VP cores pdu memory protection to be
     * enabled/disabled/no_change
     */
    adk_sys_overload_protection_state_t    vp_pdu_mem_state;
    /*!
     * state to indicate whether SPP engine protection to be
     * enabled/disabled/no_change.
     */
    adk_sys_overload_protection_state_t    spp_state;
    /*!
     * state to indicate whether EIOA engine protection to be
     * enabled/disabled/no_change. Currently this feature is not supported.
     * place holder for future extention.
     */
    adk_sys_overload_protection_state_t    eioa_state;
    /*!
     * state to indicate whether PAB engine protection to be
     * enabled/disabled/no_change.
     */
    adk_sys_overload_protection_state_t    pab_state;
    /*!
     * state to indicate whether PIC engine protection to be
     * enabled/disabled/no_change. Currently this feature is not supported.
     * place holder for future extention.
     */
    adk_sys_overload_protection_state_t    pic_state;
} adk_sys_overload_protection_conf_data_t;

/*!
 * =============================================================
 * @struct adk_sys_config_t adk_sys.h "adk_sys.h"
 *
 * @brief Structure for ADK in slave mode.
 *
 * =============================================================
 */
typedef struct adk_sys_config_s {
    /*!
     * To indicate ADK configuration mode standalone or slave.
     */
    adk_sys_config_mode_t    adk_mode;
    /*!
     * Pointer to pktio memorypool to be used in packet generation
     * in ADK like BFD.
     */
    void          *pktio_mempool;
    /*!
     * Pointer to memorypool to be used in IP fragmentation.
     */
    void          *ip_frag_mempool;
    /*!
     * Worker lcore list size.
     */
    uint32_t      num_worker_lcores;
    /*!
     * Pointer to list of DP logical core IDs on which ADK is executed.
     */
    uint32_t      *worker_lcores;
} adk_sys_config_t;

/*!
 * =============================================================
 * @enum adk_sys_timer_dev_type_e adk_sys.h "adk_sys.h"
 *
 * @brief Structure for passing timer device type information
 *
 * =============================================================
 */
typedef enum adk_sys_timer_dev_type_s {

    ADK_SYS_TIMER_DEV_TYPE_EVNT_TIMER_BD   = 1,
    ADK_SYS_DP_BRIDGE_DEV_TYPE_MAX
} adk_sys_timer_dev_type_e;

/*!
 * =============================================================
 * @struct adk_sys_dp_timer_bd_info_t adk_sys.h "adk_sys.h"
 *
 * @brief Structure for passing timer device information
 *
 * =============================================================
 */
typedef struct adk_sys_timer_dev_info_s {

    adk_sys_timer_dev_type_e    type;
    union {
        uint8_t     timer_bd_instance_id;
    };
} adk_sys_timer_dev_info_t;

typedef enum adk_sys_dp_service_timer_type_s
{
    ADK_SYS_DP_SERVICE_TIMER_TYPE_SINGE_SHOT,
    ADK_SYS_DP_SERVICE_TIMER_TYPE_PEARODIC,
} adk_sys_dp_service_timer_type_e;

adk_status_t
adk_sys_dp_service_timer_install(
    adk_sys_timer_dev_info_t           *timer_dev_info,
    adk_sys_dp_service_timer_type_e     timer_type,
    uint64_t                            time_value,
    uint8_t                             timer_handler_entry_fn_id,
    uint16_t                            load_balance_id,
    uintptr_t                           user_op_data,
    uint16_t                            event_queue_id,
    adk_handle_t                       *timer_handle
);

adk_dp_status_t
adk_sys_dp_service_timer_uninstall(adk_handle_t timer_handle);

adk_dp_status_t
adk_sys_dp_service_timer_remain_time(adk_handle_t  timer_handle,
                                     uint64_t *remain_time);

adk_dp_status_t
adk_sys_dp_service_timer_reset(
    adk_handle_t                    timer_handle,
    adk_sys_dp_service_timer_type_e timer_type,
    uint64_t                        new_time_value);

#ifdef ADK_PAL_CYCLE_MEASUREMENT
adk_dp_status_t
adk_sys_dp_proc_cycle_measure(struct rte_mbuf *dpdk_mbuf_p);
#endif

/*
 * Function prototypes from adk_sys.c
 */
adk_status_t adk_sys_init (
        const adk_sys_conf_t *sysConf_p,
        adk_sys_handle_t *sysHandle_p);

adk_status_t adk_sys_shutdown (
        adk_sys_handle_t sysHandle,
        bool destryConfig);

adk_status_t adk_sys_detach (
        adk_sys_handle_t sysHandle);

adk_status_t adk_sys_dp_config(const adk_sys_config_t *sysConfig_p);

adk_status_t adk_sys_timer_dev_register(
        const adk_sys_timer_dev_info_t *tdev_info_p);

adk_status_t adk_sys_config (
        const adk_sys_config_t *sysConfig_p,
        adk_config_handle_t *sysConfigHandle_p);

extern adk_status_t adk_sys_config_pktio_mempool_get (
        void **pktio_mempool);

extern adk_status_t adk_sys_config_ip_frag_mempool_get (
        void **ip_frag_mempool);

adk_status_t adk_sys_pcap_eth_dev_config (
        adk_sys_handle_t sysHandle,
        uint16_t ethDevId,
        bool enable);

adk_status_t adk_syshandle_get (
        bool configAttach,
        uint32_t chipInstance,
        adk_sys_handle_t *sysHandle_p);

adk_status_t adk_sys_hw_handle_get (
        adk_sys_handle_t sysHandle,
        ncp_hdl_t *hwHandle_p);

adk_status_t adk_sys_flow_id_get(
        adk_sys_handle_t sysHandle,
        adk_flow_handle_t flow_handle,
        uint32_t *flow_id);

adk_status_t adk_sys_interface_id_get(
        adk_sys_handle_t sysHandle,
        adk_interface_handle_t if_handle,
        uint32_t *interface_id);

adk_status_t adk_module_register (
        adk_sys_handle_t sysHandle,
        const uint8_t *moduleName_p,
        adk_module_init_func_t moduleInit,
        adk_module_handle_t *moduleHandle_p);

adk_status_t adk_module_register_with_id (
        adk_sys_handle_t sysHandle,
        const uint8_t *moduleName_p,
        uint8_t moduleId,
        adk_module_init_func_t moduleInit,
        adk_module_handle_t *moduleHandle_p);

adk_status_t adk_module_attr_set (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle,
        const void *attr_p);

adk_status_t adk_module_handle_get (
        adk_sys_handle_t sysHandle,
        uint8_t moduleId,
        adk_module_service_function_update_func_t serviceFnUpdateFn,
        adk_module_handle_t *moduleHandle_p);

adk_status_t adk_module_unregister (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle);

adk_status_t adk_module_start (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle);

adk_status_t adk_module_stop (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle);

adk_status_t adk_bind (
        adk_sys_handle_t,
        const adk_bind_info_t *);

adk_status_t adk_unbind (
        adk_sys_handle_t,
        const adk_bind_info_t *);

adk_status_t adk_sys_port_vp_set(
        adk_sys_handle_t sys_handle,
        uint32_t port_num,
        const char *vp_name);

#if 0
###adk_status_t adk_bind_forward_pdu (
###        adk_sys_handle_t,
###        adk_module_handle_t,
###        adk_flow_handle_t,
###        const void *);
#endif

adk_status_t adk_sys_global_stats_get (
        adk_sys_handle_t sysHandle,
        adk_sys_global_stats_t *stats);

adk_status_t adk_sys_overload_drop_stats_get (
        adk_sys_handle_t sysHandle,
        adk_sys_overload_drop_stats_t *stats);

adk_status_t adk_sys_log_enable(
        adk_sys_handle_t sysHandle,
        adk_sys_log_type_t log_type);

adk_status_t adk_sys_log_disable(
        adk_sys_handle_t sysHandle,
        adk_sys_log_type_t log_type);

adk_status_t adk_sys_udp_cksum_validate(
        adk_sys_handle_t sysHandle,
        bool flag);

adk_status_t adk_sys_udp_cksum_gen_config(
        adk_sys_handle_t sysHandle,
        bool compute);

adk_status_t adk_sys_ipsec_bypass_config(
        adk_sys_handle_t sysHandle,
        bool bypass);

adk_status_t adk_sys_global_debug_get (
        adk_sys_handle_t sysHandle,
        adk_sys_global_debug_t *debugDump_p);

adk_status_t adk_sys_fpl_trees_dump (
        adk_sys_handle_t sysHandle);

adk_status_t adk_sys_tables_dump (
        adk_sys_handle_t sysHandle,
        adk_sys_table_group_t table_group);

const char *adk_sys_status_get (
    adk_sys_handle_t sysHandle,
    adk_status_t errorCode);

adk_status_t
adk_sys_adk_version_info_get(
    adk_sys_handle_t        sysHandle,
    adk_sys_version_info_t *adk_version_p);

adk_status_t adk_sys_overload_protection_config(
        adk_sys_handle_t sysHandle,
        const adk_sys_overload_protection_conf_data_t  *op_conf_data_p);


adk_status_t adk_sys_ntp_time_resync(adk_sys_handle_t sys_handle);

typedef struct adk_sys_flow_cache_stats_s{
    uint64_t                hit;
    uint64_t                cleared;
    uint64_t                miss;
    uint64_t                alloc;
    uint64_t                freed;
    uint32_t                flows_active;
    uint32_t                flows_stale;
}adk_sys_flow_cache_stats_t;

adk_status_t
adk_sys_flow_cache_update_version(
        adk_sys_handle_t sysHandle);

adk_status_t
adk_sys_flow_cache_stats_get (
        adk_sys_handle_t sysHandle,
        adk_sys_flow_cache_stats_t **stats_p,
        uint8_t *num_worker_core);

adk_status_t
adk_sys_flow_cache_entries_print(
        adk_sys_handle_t sys_handle);

adk_status_t adk_sys_configure_flow_cache_counters(
        adk_sys_handle_t sysHandle,
        bool enable);

#ifdef ADK_SYS_OVERLOAD_PROTECTION
/*!
 * =============================================================
 * @struct adk_sys_pkt_priority_type_t adk_sys.h "adk_sys.h"
 *
 * @brief enum that defines types to represent DSCP & PCP. 
 * =============================================================
 */
typedef enum adk_sys_pkt_priority_type_e{
    /*!
     * Type to indicates that use IP DSCP value in deciding the 
     * packet priority and corresponding drop precedence.
     */
    ADK_SYS_PRIO_TYPE_IP_DSCP,
    /*!
     * Type to indicats that use VLAN PCP value in deciding the 
     * packet priority and corresponding drop precedence.
     */
    ADK_SYS_PRIO_TYPE_VLAN_PCP
} adk_sys_pkt_priority_type_t;

/*!
 * =============================================================
 * @struct adk_sys_drop_precedence_t adk_sys.h "adk_sys.h"
 *
 * @brief enum that defines drop precedences.
 * =============================================================
 */
typedef enum adk_sys_drop_precedence_e{

    /*!
     * type to indicate low drop precedence.  under system overload condition 
     * packets that are classified as low drop precedence will not be dropped.
     */
    ADK_SYS_DROP_PRECEDENCE_LOW,
    /*!
     * type to indicate high drop precedence.  under system overload condition 
     * packets that are classified as high drop precedence will be dropped.
     */
    ADK_SYS_DROP_PRECEDENCE_HIGH
} adk_sys_drop_precedence_t;

/*!
 * =============================================================
 * @struct adk_sys_pkt_prio_drop_precedence_map_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent packet priority to drop precedences
 * mapping.
 * =============================================================
 */
typedef struct adk_sys_pkt_prio_drop_precedence_map_s{
    /*!
     * packet priority. if the packet pkt_priority is of type DSCP, then the
     * pkt_priority &  dscp_mask@adk_sys_ethdev_drop_precedence_info_t together
     * represent a range of DSCP values.
     */
    uint8_t                     pkt_priority;
    /*!
     * drop precedence associated with the packet priority or range of
     * priorities if type is DSCP. 
     */
    adk_sys_drop_precedence_t   drop_precedence;

    /*!
     * Bridge driver action decision, TRUE drop FALSE enqueue
     */
    bool action_drop;
} adk_sys_pkt_prio_drop_precedence_map_t;

/*!
 * =============================================================
 * @struct adk_sys_ethdev_drop_precedence_info_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent drop precedences mapping deatils.
 * =============================================================
 */
typedef struct adk_sys_ethdev_drop_precedence_info_s{
    /*!
     * type to indicate whether to apply IP-DSCP drop policies or 
     * VLAN_PCP drop policies. 
     */
    adk_sys_pkt_priority_type_t type;
    /*!
     * The default drop precedence that is assigned for non-IP packets 
     * incase of type is IP_DSCP and un-tagged packets in
     * case of type is VLAN_PCP.
     */
    adk_sys_drop_precedence_t   default_drop_precedence;
    /*!
     * Default action decision, TRUE drop FALSE enqueue
     */
    bool action_drop;

    /*!
     * this filed is valid only if the type is ADK_SYS_PRIO_TYPE_IP_DSCP.
     * dscp_mask indicates whether the priority->drop mapping is provided
     * for all the DSCP values or sub-set of DSCP values.  
     * dscp_mask -> 0x00 : means all 64 dscp's have same drop precedence. The first
     * policy's drop-precedence in the "prio_to_drop_map_list" is applied for
     * all 64 DSCP values. 
     *
     * when the dscp_mask is set to non-zero value, each policy in the in
     * the "prio_to_drop_map_list" represents drop precedence for a range of DSCP values. 
     * The range is derived from dscp_mask value
     *
     * when the dscp_mask is set to 0x3F means each policy in the "prio_to_drop_map_list" 
     * represents drop precedence for a unique DSCP value. 
     *
     * Only ranges with power of '2' is supported. The allowed list of dscp_mask
     * values are {0x3F, 0x3E, 0x3C, 0x38, 0x30, 0x20, 0x00}
     */
    uint8_t                     dscp_mask;
    /*!
     *  priority to drop-precedence mapping list
     */
    adk_sys_pkt_prio_drop_precedence_map_t *prio_to_drop_map_list;
    /*!
     * number of priority to drop precedence mappings in "prio_to_drop_map_list"
     */
    uint8_t                     num_maps;
}adk_sys_ethdev_drop_precedence_info_t;

/*!
 * =============================================================
 * adk_sys_ethdev_drop_precedence_map
 *
 * @param[in] sysHandle    ADK System Handle
 * @param[in] eth_dev_id   The id of the eth-device to configure the drop-precedence.
 *                         To do the system wide configuration appliciaton is required 
 *                         pass "-1" as eth-dev-id. The system wide confiruation is only
 *                         suuported for XEON targets. 
 * @param[in] drop_info_p  pointer to a adk_sys_ethdev_drop_precedence_info_t structure
 *                         to pass the packet priority to drop-precedence mapping inforamtion. 
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * @brief API to configure the packet-priority to drop-precedence mapping 
 * either system-wide or per eth-device.
 *
 * ==============================================================
 */
adk_status_t adk_sys_ethdev_drop_precedence_map(
    adk_sys_handle_t sysHandle,
    int16_t eth_dev_id, 
    adk_sys_ethdev_drop_precedence_info_t *drop_info_p);
#endif

/* Set the required number of egress stats ids.
 * Should be <= ADK_SYS_EGRESS_STATS_ID_MASK
 */
#define ADK_SYS_EGRESS_STATS_ID_MAX (8*1024)

#define ADK_SYS_EGRESS_STATS_ID_LEN 16
#define ADK_SYS_EGRESS_STATS_ID_MASK 0xFFFF
#define ADK_SYS_EGRESS_STATS_VERSION_MASK 0xF
#define ADK_SYS_EGRESS_STATS_ID_INVALID 0xFFFFFFFF

/* ADK metadata defines for egress stats */

/* Max number of stats ids that can be set in metadata */
#define ADK_SYS_EGRESS_STATS_MDATA_ID_MAX 4
/* Location of the IP stats Id */
#define ADK_SYS_EGRESS_STATS_MDATA_IP_LOC 0
/* Location of Ethernet stats Id */
#define ADK_SYS_EGRESS_STATS_MDATA_ETHERNET_LOC 1
/* Location of the IPsec stats Id */
#define ADK_SYS_EGRESS_STATS_MDATA_IPSEC_LOC 2
/* Location of the QUEUE stats Id */
#define ADK_SYS_EGRESS_STATS_MDATA_QUEUE_LOC 3

/*ADK metadata for Address & Iface stats. For adding any new stats type
 * for post QoS increment, new macro should be added here.
 * */
#define ADK_STATS_MDATA_ID_MAX 4
/*INTERFACE STATS*/
#define ADK_STATS_MDATA_ID_INTERFACE_STATS 0
/*ADDRESS STATS FOR INNER IP*/
#define ADK_STATS_MDATA_ID_INNER_IP_STATS 1
/*ADDRESS STATS FOR OUTER IP*/
#define ADK_STATS_MDATA_ID_OUTER_IP_STATS 2
/*ETHERNET INTERFACE STATS*/
#define ADK_STATS_MDATA_ID_ETHERNET_INTERFACE_STATS 3
/* Egress stats metadata flag to count packet counter */
#define ADK_SYS_EGRESS_STATS_MDATA_FLAG_PKT_CNTR 1
/* Egress stats metadata flag to count bytes counter */
#define ADK_SYS_EGRESS_STATS_MDATA_FLAG_BYTES_CNTR 2
/* Egress stats metadata flag to set time stamp */
#define ADK_SYS_EGRESS_STATS_MDATA_FLAG_TS 4

typedef enum adk_egress_stats_mask_e{
    ADK_STATS_IP_INTERFACE_MASK = 0x1,
    ADK_STATS_INNER_IP_ADDRESS_MASK = 0x2,
    ADK_STATS_OUTER_IP_ADDRESS_MASK = 0x4,
    ADK_STATS_ETHERNET_INTERFACE_MASK = 0x8,
}adk_egress_stats_mask_t;

/* @struct adk_egress_user_param_s "adk_sys.h"
 * @brief data strucuture to represent user params passed 
 * to egress_stats_callback in ip uli
 * for incrementing counters post buffer mgmt
 * this can be scaled further on a need basis
 */
typedef struct adk_egress_user_param_s {
   bool xmit_cb;
   adk_egress_stats_mask_t stats_mask;
}adk_egress_user_param_t;


/* @struct adk_cppi_user_param_s "adk_sys.h"
 * @brief data strucuture to represent user params passed 
 * to cppi_stats_callback in ip LLI
 * for incrementing counters post policing towards host
 * this can be scaled further on a need basis
 */
typedef struct adk_slowpath_user_param_s {
   uint8_t  l4_protocol;
   uint16_t l4_len;
   uint32_t flow_id;
   uint32_t pkt_color;
   uint32_t policeFlowId;
}adk_slowpath_user_param_t;
/*!
 * =============================================================
 * @struct adk_sys_egress_stats_mdata_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent egress stats info in packet meta data 
 * =============================================================
 */
typedef struct adk_sys_egress_stats_mdata_s{
    union {
        uint32_t stats_info; /* to access 32 bits together*/
        struct {
            /* Length correction (-128 to +127) to be applied with byte counters */
            int32_t len_correction:8;
            /* Flags to denote which all counters to be counted
            * eg:- ADK_SYS_EGRESS_STATS_MDATA_FLAG_PKT_CNTR
            * Multiple flags can be set */
            uint32_t flags:4;
            /* Stats id allocated using adk_sys_egress_stats_id_alloc */
            uint32_t stats_id:20;
        };
    };
} adk_sys_egress_stats_mdata_t;

/*!
 * =============================================================
 * @struct adk_sys_commonn_egress_stats_mdata_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent egress stats info in packet meta data
 * ToDo: Can be enhanced to add additional mdata on a need basis
 * =============================================================
 */
typedef struct adk_sys_common_egress_stats_mdata_s{
            /* Stats id allocated using adk_sys_egress_stats_id_alloc */
    uint32_t stats_id;
} adk_sys_common_egress_stats_mdata_t;


/*!
 * =============================================================
 * @struct adk_sys_egress_stats_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent egress stats 
 * =============================================================
 */
typedef struct adk_sys_egress_stats_s{
    uint64_t pkt_drops;
    uint64_t byte_drops;
    uint64_t pkt_xmits;
    uint64_t byte_xmits;
    uint64_t timestamp;
} adk_sys_egress_stats_t;

/*!
 * =============================================================
 * @struct adk_sys_dp_egress_stats_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent egress stats in DP
 * =============================================================
 */
typedef struct adk_sys_dp_egress_stats_s{
#ifdef ADK_DEVICE_SNR
    /*Number of pkts enqueued successfully in the transmit queue*/
    uint64_t pkt_enqueued;
    /*Number of bytes enqueued successfully in the transmit queue*/
    uint64_t byte_enqueued;
    /*Number of pkts dropped while enqueue in the transmit queue*/
    uint64_t pkt_enqueue_drops;
    /*Number of bytes dropped while enqueue in the transmit queue*/
    uint64_t byte_enqueue_drops;
#endif
    /*Number of pkts dropped at HW while transmitting*/
    uint64_t pkt_drops;
    /*Number of bytes dropped at HW while transmitting*/
    uint64_t byte_drops;
    /*Number of pkts successfully transmited by HW */
    uint64_t pkt_xmits;
    /*Number of bytes successfully transmited by HW */
    uint64_t byte_xmits;
    /*Timestamp at which packets are successfully enqueued/transmited by HW */
    uint64_t timestamp;
    /*Version of packet*/
    uint8_t  version;
} adk_sys_dp_egress_stats_t __rte_cache_aligned;

/*!
 * =============================================================
 * @struct adk_slowpath_dev_dp_metadata_t adk_sys.h "adk_sys.h"
 *
 * @brief data structure to represent slowpath dev MD
 * =============================================================
 */
typedef struct adk_slowpath_dev_dp_metadata_s {
    uint8_t     moduleId;
    uint8_t     l4_protocol;
    uint16_t    l4_len;
    uint32_t    flowId;
    uint32_t    policerFlowId;
    void        *imc_param_start;
} adk_slowpath_dev_dp_metadata_t;

adk_status_t
adk_sys_egress_stats_id_alloc(
    adk_sys_handle_t sys_handle,
    uint32_t *stats_id_p);

adk_status_t
adk_sys_egress_stats_id_free(
    adk_sys_handle_t sys_handle,
    uint32_t stats_id);

adk_status_t
adk_sys_egress_stats_get(
    adk_sys_handle_t sys_handle,
    uint32_t stats_id,
    adk_sys_egress_stats_t* stats_p);

#ifdef ADK_DEVICE_SNR
void
adk_sys_dp_egress_stats_enqueue_cb(
    struct rte_mbuf *pkt,
    void *user_param);
#else
uint16_t
adk_sys_dp_egress_stats_cb(
#if ADK_DPDK_VERSION >= 18050016
    uint16_t port_id,
#else
    uint8_t port_id,
#endif
    uint16_t queue,
    struct rte_mbuf *pkts[],
    uint16_t nb_pkts,
    void *user_param);
#endif

#if defined(ADK_DEVICE_SNR) && defined(ADK_COMPLETION_CALLBACK_SUPPORT)
void
adk_sys_dp_egress_stats_xmit_cb(
    struct rte_mbuf *pkt,
    void *user_param);
#endif

#ifdef ADK_DP_MAINTENANCE
int
adk_dp_maintenance_fn(
	uint8_t event_dev_id,
	uint8_t event_port_id,
	void *arg);

adk_status_t
adk_dp_maintenance_init_v2(uint8_t lcore_id);

void
adk_dp_maintenance_fn_v2(struct rte_timer *timer, void *args);
#endif /* #ifdef ADK_DP_MAINTENANCE */

adk_status_t adk_sys_dp_init(void);

#ifdef ADK_SYS_OVERLOAD_PROTECTION
uint16_t
#if ((RDK_REL_MAJOR >= 12) || ((RDK_REL_MAJOR == 10) && (RDK_REL_MINOR == 2)))
adk_sys_dp_overlod_protection(uint16_t port_id,
#else
adk_sys_dp_overlod_protection(uint8_t port_id,
#endif
        uint16_t queue_id,
        uint32_t enqueue_buf_size,
        uint32_t enqueue_buf_count,
        struct rte_event *ev,
        uint16_t nb_events,
        void *cb_arg,
        uint16_t *nb_dropped);
#endif

#ifdef ADK_QOS_GEN3_INTERFACE
void 
adk_sys_dp_queue_stats_update(struct rte_mbuf *mbuf,
                            uint8_t policeColor);
#endif
enum rte_bridge_cppi_cb_ret_type
adk_sys_dp_slowpath_protection(uint16_t dev_id,
		uint16_t queue_id,
		uint32_t enqueue_buf_size,
		uint32_t enqueue_buf_count,
		struct rte_mbuf **mbuf,
		uint16_t nb_mbuf,
		void *cb_arg,
		enum rte_bridge_cppi_cb_action action[]);

adk_status_t
adk_sys_dp_pkt_send_to_port(void **task_p, uint8_t task_count,
                            uint32_t task_mask, int coreIndex);

adk_status_t
adk_sys_dp_pkt_send_to_linux(void *task_p);


#ifdef ADK_DEVICE_SNR
adk_status_t 
adk_sys_eth_dev_vsi_get (uint16_t eth_dev_id, uint32_t *vsi_id);
#endif

#ifdef QAT
void adk_ipsec_lli_task_read_param_set (
    void **task_p,
    uint8_t *taskCount_p,
    uint32_t *taskMask_p);

void adk_ipsec_uli_task_read_param_set (
    void **task_p,
    uint8_t *taskCount_p,
    uint32_t *taskMask_p);

void *adk_ipsec_get_next_qat_encrypt_response (void);

void adk_ipsec_uli_read_qat_response (void);

void *adk_ipsec_get_next_qat_decrypt_response (void);

void adk_ipsec_lli_read_qat_response (void);

void adk_sif_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#endif

#ifdef __cplusplus
}
#endif

#endif /* __ADK_SYS_H__ */

/*! end addtogroup _sif
 * @}
 */
