/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_sys_dpdk.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief  
 *
 * This file contains definitions and declarations of objects that are 
 * part of DPDK system infrastructure.
 *
 ***********************************************************************/

#ifndef __ADK_SYS_DPDK_H__
#define __ADK_SYS_DPDK_H__

#include "adk_dpdk.h"

extern struct rte_mempool *adkPktmbufPool_g;

/* DPDK specific macro definitions */

#define PH_RING_SIZE 1024

#define PKT_MEMPOOL_INDIRECT_1  0
#define PKT_MEMPOOL_1           1

/*#define MAX_PDU_PARAMS_PER_CORE (ADK_DP_BULK_RX_SIZE)*/

struct adk_rss_ipv4_tuple {
    uint32_t        src_addr;
    uint32_t        dst_addr;
    union {
        struct {
            uint16_t dport;
            uint16_t sport;
        };
        uint32_t        sctp_tag;
    };
};

struct adk_rss_ipv6_tuple {
    uint8_t         src_addr[16];
    uint8_t         dst_addr[16];
    union {
        struct {
            uint16_t dport;
            uint16_t sport;
        };
        uint32_t        sctp_tag;
    };
};

union adk_rss_tuple {
      struct adk_rss_ipv4_tuple   v4;
      struct adk_rss_ipv6_tuple   v6;
};

typedef struct _adk_rss_reta_info_t {
    union adk_rss_tuple tuple;
    uint16_t qid; 
    uint8_t portid;
} adk_soft_rss_reta_tbl_t;


/* 
 * Function prototypes 
 */
adk_status_t
adk_sys_dpdk_init(const adk_sys_conf_t *sysConf_p, adk_sif_cfg_info_t **cfg_hdl);

adk_status_t adk_sif_soft_rss_init(adk_sif_cfg_info_t *cfg_info);
adk_status_t adk_sif_soft_rss_update (adk_sys_handle_t sys_handle, adk_soft_rss_reta_tbl_t *reta_info);

#endif /* __ADK_SYS_DPDK_H__ */
 
/*! end addtogroup _sif
 * @}
 */

