/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_module_mgr.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief
 *
 * This file contains APIs that are used to manage module specific
 * information.
 *
 ***********************************************************************/

#include <adk_include.h>
#include <adk_hos.h>

module_cpu_fn_t   adkDestEntryFunction_g[ADK_MAX_DP_SERVICE_FN_ID] = {NULL};
#define  DYN_MEM_OBJ_SIZE	64

typedef struct dyn_mem_s {
	uint32_t memory;
	uint32_t count;
} dyn_mem_t;

dyn_mem_t mod_dyn_mem_cp[DYN_MEM_OBJ_SIZE][DYN_MEM_OBJ_SIZE];
dyn_mem_t mod_dyn_mem_dp[DYN_MEM_OBJ_SIZE][DYN_MEM_OBJ_SIZE];

void print_module_dynamic_mem(void)
{
    int i, j, k = 0;
    uint32_t cpsum, dpsum, cpcnt, dpcnt;

    for (i = 0; i < DYN_MEM_OBJ_SIZE; i++) {
	cpsum = 0;
	dpsum = 0;
	k = 0;

	for ( j = 0; j < DYN_MEM_OBJ_SIZE; j++) {

            cpcnt = 1;
            dpcnt = 1;
	    if (mod_dyn_mem_cp[i][j].memory || mod_dyn_mem_dp[i][j].memory) {
		if (!k) {
			printf("Module_Id      obj_id       CP(num OBJ)        CP(per OBJ)         DP(num OBJ)      DP(per OBJ) \n");
			printf("---------      ------       -----------        -----------         -----------      ---------- \n");
			k++;
		}
		cpsum += mod_dyn_mem_cp[i][j].memory;
		dpsum += mod_dyn_mem_dp[i][j].memory;

		if (mod_dyn_mem_cp[i][j].memory)
			cpcnt = mod_dyn_mem_cp[i][j].count;

		if (mod_dyn_mem_dp[i][j].memory)
			dpcnt = mod_dyn_mem_dp[i][j].count;

		printf("%5d  %12d    %8d(%5d)        %8d           %9d(%5d)      %5d\n", i, j, mod_dyn_mem_cp[i][j].memory,
                         mod_dyn_mem_cp[i][j].count, (mod_dyn_mem_cp[i][j].memory / cpcnt),
			 mod_dyn_mem_dp[i][j].memory, mod_dyn_mem_dp[i][j].count,
			 (mod_dyn_mem_dp[i][j].memory / dpcnt));
	    }
    	}
	if ((cpsum != 0) || (dpsum != 0))
		printf("\n                 total:%8d                                  %9d\n\n\n", cpsum, dpsum);
    }
}

void* nvm_mod_alloc(unsigned int __attribute__ ((unused)) mod_id, unsigned int __attribute__ ((unused)) obj_id,
			 size_t p, size_t s, size_t a, size_t __attribute__ ((unused)) b)
{
#ifdef ADK_MEASURE_DYN_MEM
    struct rte_malloc_socket_stats pre, post;
    rte_malloc_get_socket_stats(rte_socket_id(), &pre);
#endif
    void* ptr = rte_calloc(NULL, p, s, a);
    if (!ptr)
           printf("Could not allocate\n");

#ifdef ADK_MEASURE_DYN_MEM
    if (ptr)
    {

	if (mod_id > 63) {
			printf("Wrong module id passed dynamic memory cannot be computed!!!\n");
			return ptr;
	}

	rte_malloc_get_socket_stats(rte_socket_id(), &post);
	if (b == 0) {
		mod_dyn_mem_cp[mod_id][obj_id].memory += pre.heap_freesz_bytes - post.heap_freesz_bytes;
		mod_dyn_mem_cp[mod_id][obj_id].count++;
	} else if (b == 1) {
		mod_dyn_mem_dp[mod_id][obj_id].memory += pre.heap_freesz_bytes - post.heap_freesz_bytes;
		mod_dyn_mem_dp[mod_id][obj_id].count++;
	}
    }
#endif
    return ptr;
}

/*!
 * ADK module configuration
 */
static adk_module_config_t module_config_table[] =
{
    { (const char *)ADK_SYS_MODULE_NAME, ADK_SYS_MODULE_ID, NULL },
    { (const char *)ADK_IP_MODULE_NAME, ADK_IP_MODULE_ID, NULL},
    { (const char *)ADK_UDP_MODULE_NAME, ADK_UDP_MODULE_ID, NULL},
    { (const char *)ADK_ETHERNET_MODULE_NAME, ADK_ETHERNET_MODULE_ID, NULL},
    { (const char *)ADK_PTP_MODULE_NAME, ADK_PTP_MODULE_ID, NULL},
    { (const char *)ADK_LCPS_MODULE_NAME, ADK_LCPS_MODULE_ID, NULL},
    { (const char *)ADK_ATM_MODULE_NAME, ADK_ATM_MODULE_ID, NULL},
    { (const char *)ADK_AAL5_MODULE_NAME, ADK_AAL5_MODULE_ID, NULL},
    { (const char *)ADK_GTPU_MODULE_NAME, ADK_GTPU_MODULE_ID, NULL},
    { (const char *)ADK_IPV6_MODULE_NAME, ADK_IPV6_MODULE_ID, NULL},
    { (const char *)ADK_AAL2_MODULE_NAME, ADK_AAL2_MODULE_ID, NULL},
    { (const char *)ADK_IPSEC_MODULE_NAME, ADK_IPSEC_MODULE_ID, NULL},
    { (const char *)ADK_ETH_OAM_MODULE_NAME, ADK_ETH_OAM_MODULE_ID, NULL},
    { (const char *)ADK_LTE_PDCP_MODULE_NAME, ADK_LTE_PDCP_MODULE_ID, NULL},
    { (const char *)ADK_AR_MODULE_NAME, ADK_AR_MODULE_ID, NULL},
    { (const char *)ADK_BP_MODULE_NAME, ADK_BP_MODULE_ID, NULL},
    { (const char *)ADK_LAPD_MODULE_NAME, ADK_LAPD_MODULE_ID, NULL},
    { (const char *)ADK_CUSTRAWETH_MODULE_NAME, ADK_CUSTRAWETH_MODULE_ID, NULL},
    { (const char *)ADK_IPHC_MODULE_NAME, ADK_IPHC_MODULE_ID, NULL},
    { (const char *)ADK_RTP_MUX_MODULE_NAME, ADK_RTP_MUX_MODULE_ID, NULL},
    { (const char *)ADK_CUSTBP_MODULE_NAME, ADK_CUSTBP_MODULE_ID, NULL},
    { (const char *)ADK_NODEB_FP_MODULE_NAME, ADK_NODEB_FP_MODULE_ID, NULL},
    { (const char *)ADK_UDP_IWF_MODULE_NAME, ADK_UDP_IWF_MODULE_ID, NULL},
    { (const char *)ADK_BFD_MODULE_NAME, ADK_BFD_MODULE_ID, NULL},
    { (const char *)ADK_TWAMP_MODULE_NAME, ADK_TWAMP_MODULE_ID, NULL},
    { (const char *)ADK_HSFP_IWF_MODULE_NAME, ADK_HSFP_IWF_MODULE_ID, NULL},
    { (const char *)ADK_BP_IWF_MODULE_NAME, ADK_BP_IWF_MODULE_ID, NULL},
    { (const char *)ADK_MBA_MODULE_NAME, ADK_MBA_MODULE_ID, NULL},
    { (const char *)ADK_L2TPV3_MODULE_NAME, ADK_L2TPV3_MODULE_ID, NULL},
    { (const char *)ADK_EHSFP_IWF_MODULE_NAME, ADK_EHSFP_IWF_MODULE_ID, NULL},
    /* The following are for unit testing only, the valid
     * module IDs should appear above this comment.
     */
#ifdef ADK_UNIT_TEST
    { (const char *)"test-atm", 1, NULL },
    { (const char *)"test-ip", 21, NULL },
    { (const char *)"test-udp1", 22, NULL },
    { (const char *)"test-udp2", 25, NULL },
    { (const char *)"test-udp3", 31, NULL },
    { (const char *)"test-udp4", 32, NULL },
#endif
    /*
     * insert new modules here.
     */
    { (const char *)"last-module-invalid", 0xff, NULL }
};

adk_module_cpu_service_fn_t adkModuleMgrCpuServiceFn_g[ADK_MAX_SUPPORTED_MODULES];

static adk_status_t adk_sys_null_init(adk_module_handle_t);

adk_status_t adk_sys_null_init(adk_module_handle_t __attribute__ ((unused)) mod_handle)
{
    return ADK_SUCCESS;
}

/*!
 * =============================================================
 * @internal
 * @param[in]    sysHandle    system handle returned after
 * successfull completion of adk_sys_init API.
 *
 * @retval       ADK_SUCCESS    module manager initialization was
 * successful.
 * @retval       ADK_E_MEM_UNAVAILABLE    failed to allocate memory
 * for module manager internal database.
 *
 * @brief
 *
 * This function initializes module manager internal resources.
 *
 * =============================================================
 */
adk_status_t
adk_module_mgr_init (
       adk_sys_handle_t sysHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint16_t moduleIndex;
    adk_module_info_t *moduleInfo_p;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *) sysHandle;

    moduleInfo_p =
        (adk_module_info_t *) ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, ADK_MAX_SUPPORTED_MODULES,
                                             sizeof(adk_module_info_t));
    if (!ADK_VALID_PTR(moduleInfo_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        goto _adk_module_mgr_init_exit;
    }

    sysInfo_p->utils.moduleMgrMasterHandle = (adk_handle_t) moduleInfo_p;

    /* Set default values for all required state variables */
    for (moduleIndex = 0; moduleIndex < ADK_MAX_SUPPORTED_MODULES;
         moduleIndex++, moduleInfo_p++)
    {
        moduleInfo_p->inUse = FALSE;
    }

    retValue = adk_module_register(sysHandle,
                                   (uint8_t *) ADK_SYS_MODULE_NAME,
                                   (void *) adk_sys_null_init,
                                   &(ADK_SYS_MODULE_HANDLE_GET (sysHandle)));
    if (retValue != ADK_SUCCESS)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (retValue);
        goto _adk_module_mgr_init_exit;
    }

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully initialized module manager: handle %p\n",
                  moduleInfo_p);

    retValue = adk_module_start(sysHandle,
                                (ADK_SYS_MODULE_HANDLE_GET (sysHandle)));
    if (retValue != ADK_SUCCESS) 
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (retValue);
        goto _adk_module_mgr_init_exit;
    }

#ifdef ADK_MEASURE_DYN_MEM
    int i, j;
    for (i = 0; i < DYN_MEM_OBJ_SIZE; i++) {
        for (j = 0; j < DYN_MEM_OBJ_SIZE; j++) {
	    mod_dyn_mem_cp[i][j].memory = 0;
	    mod_dyn_mem_cp[i][j].count  = 0;

	    mod_dyn_mem_dp[i][j].memory = 0;
	    mod_dyn_mem_dp[i][j].count  = 0;
	}
    }
#endif

_adk_module_mgr_init_exit:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_mgr_init  ----- */


/*!
 * =============================================================
 * @internal
 * adk_module_mgr_deinit
 * @param[in]  sysHandle  ADK system handle (of type adk_sys_handle_t)
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval
 *
 * @brief
 *
 * This function de-allocate the NVM space allocated by the
 * adk_module_mgr_init function.
 *
 * =============================================================
 */
adk_status_t
adk_module_mgr_deinit (
        adk_sys_handle_t sysHandle
        )
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *) sysHandle;

    if ( sysInfo_p->utils.moduleMgrMasterHandle )
    {
        ADK_NVM_FREE (sysInfo_p->utils.moduleMgrMasterHandle);
    }

    ADK_RETURN( retValue );
}    /* -----  end of function adk_module_mgr_deinit  ----- */

/*!
 * =============================================================
 * adk_module_service_function_register
 * @param[in]    sysHandle    adk system handle
 *
 * @param[in]    moduleHandle    handle of the module that is
 * registering the service functions.
 *
 * @param[in]    serviceFn_p    pointer to a data structure that
 * carries pointers to all service functions that are supported by
 * module.
 *
 * @retval    ADK_SUCCESS
 * @retval    ADK_E_INVALID_HANDLE    invalid system or module handle.
 *
 * @brief
 *
 * This function registers all module specific service functions
 * with module manager databse.
 *
 * =============================================================
 */
adk_status_t
adk_module_service_function_register (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle,
        const adk_module_service_fn_t *serviceFn_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_module_engine_service_fn_t *moduleEngineServiceFn_p;
    adk_module_cpu_service_fn_t *moduleCpuServiceFn_p;
    adk_module_info_t *moduleInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) ||
        !ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system or module handle\n");
        goto _adk_module_service_function_register_exit;
    }

    if (!ADK_VALID_PTR(serviceFn_p)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid service function pointer\n");
        goto _adk_module_service_function_register_exit;
    }

    /* Copy the engine service functions */
    moduleInfo_p = (adk_module_info_t *) moduleHandle;
    moduleEngineServiceFn_p = (adk_module_engine_service_fn_t *)
        &moduleInfo_p->moduleEngineFunctions;
    moduleEngineServiceFn_p->mppEntryFn_LLI =
                                    serviceFn_p->mppEntryFn_LLI;
    moduleEngineServiceFn_p->mppEntryFn_ULI =
                                    serviceFn_p->mppEntryFn_ULI;
    moduleEngineServiceFn_p->mppEntryFn_LLI_direct =
                                    serviceFn_p->mppEntryFn_LLI_direct;
    moduleEngineServiceFn_p->mppEntryFn_ULI_direct =
                                    serviceFn_p->mppEntryFn_ULI_direct;

    /* Copy the CPU service functions */
    moduleCpuServiceFn_p = &(adkModuleMgrCpuServiceFn_g[moduleInfo_p->moduleId]);
    moduleCpuServiceFn_p->moduleTerminate = serviceFn_p->moduleTerminate;
    moduleCpuServiceFn_p->moduleCreate = serviceFn_p->moduleCreate;
    moduleCpuServiceFn_p->moduleBind = serviceFn_p->moduleBind;
    moduleCpuServiceFn_p->moduleUnbind = serviceFn_p->moduleUnbind;
    moduleCpuServiceFn_p->srcModuleBind = serviceFn_p->srcModuleBind;
    moduleCpuServiceFn_p->srcModuleUnbind = serviceFn_p->srcModuleUnbind;
    moduleCpuServiceFn_p->dstModuleBind = serviceFn_p->dstModuleBind;
    moduleCpuServiceFn_p->dstModuleUnbind = serviceFn_p->dstModuleUnbind;
    moduleCpuServiceFn_p->moduleDelete = serviceFn_p->moduleDelete;
    moduleCpuServiceFn_p->moduleModify = serviceFn_p->moduleModify;
    moduleCpuServiceFn_p->moduleGet = serviceFn_p->moduleGet;
    moduleCpuServiceFn_p->moduleExtension = serviceFn_p->moduleExtension;
    moduleCpuServiceFn_p->moduleGetImc = serviceFn_p->moduleGetImc;
    moduleCpuServiceFn_p->moduleGetEntryFunc = serviceFn_p->moduleGetEntryFunc;
    moduleCpuServiceFn_p->moduleLogSet = serviceFn_p->moduleLogSet;
    moduleCpuServiceFn_p->errorString = serviceFn_p->errorString;
    moduleCpuServiceFn_p->cpuEntryFn_LLI[_DEST_FUNC_DEFAULT] = serviceFn_p->cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    moduleCpuServiceFn_p->cpuEntryFn_LLI[_DEST_FUNC_RING] = serviceFn_p->cpuEntryFn_LLI[_DEST_FUNC_RING];
    moduleCpuServiceFn_p->cpuEntryFn_ULI[_DEST_FUNC_DEFAULT] = serviceFn_p->cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
    moduleCpuServiceFn_p->cpuEntryFn_ULI[_DEST_FUNC_RING] = serviceFn_p->cpuEntryFn_ULI[_DEST_FUNC_RING];

_adk_module_service_function_register_exit:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_service_function_register  ----- */

/*!
 * =============================================================
 * adk_module_service_function_update
 * @param[in]    sysHandle    adk system handle
 *
 * @param[in]    moduleHandle    handle of the module that is
 * updating the service functions.
 *
 * @param[in]    serviceFn_p    pointer to a data structure that
 * carries pointers to all C language based service functions
 * that are supported by module.
 *
 * @retval    ADK_SUCCESS
 * @retval    ADK_E_INVALID_HANDLE    invalid system or module handle.
 *
 * @brief
 *
 * This function updates all module specific C language based
 * service function pointers with module manager. This API is
 * required for 2 purposes. 1) To update the new function pointers
 * after the warm restart. 2) While using multiple instances of
 * SIF, to update the function pointers of a particular instance.
 *
 * =============================================================
 */
adk_status_t
adk_module_service_function_update (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle,
        const adk_module_service_fn_t *serviceFn_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_module_cpu_service_fn_t *moduleCpuServiceFn_p;
    adk_module_info_t *moduleInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) ||
        !ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system or module handle\n");
        goto _adk_module_service_function_update_exit;
    }

    if (!ADK_VALID_PTR(serviceFn_p)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid service function pointer\n");
        goto _adk_module_service_function_update_exit;
    }

    /* Copy the CPU service functions */
    moduleInfo_p = (adk_module_info_t *) moduleHandle;

    moduleCpuServiceFn_p = &(adkModuleMgrCpuServiceFn_g[moduleInfo_p->moduleId]);
    moduleCpuServiceFn_p->moduleTerminate = serviceFn_p->moduleTerminate;
    moduleCpuServiceFn_p->moduleCreate = serviceFn_p->moduleCreate;
    moduleCpuServiceFn_p->srcModuleBind = serviceFn_p->srcModuleBind;
    moduleCpuServiceFn_p->srcModuleUnbind = serviceFn_p->srcModuleUnbind;
    moduleCpuServiceFn_p->dstModuleBind = serviceFn_p->dstModuleBind;
    moduleCpuServiceFn_p->dstModuleUnbind = serviceFn_p->dstModuleUnbind;
    moduleCpuServiceFn_p->moduleBind = serviceFn_p->moduleBind;
    moduleCpuServiceFn_p->moduleUnbind = serviceFn_p->moduleUnbind;
    moduleCpuServiceFn_p->moduleDelete = serviceFn_p->moduleDelete;
    moduleCpuServiceFn_p->moduleModify = serviceFn_p->moduleModify;
    moduleCpuServiceFn_p->moduleGet = serviceFn_p->moduleGet;
    moduleCpuServiceFn_p->moduleExtension = serviceFn_p->moduleExtension;
    moduleCpuServiceFn_p->moduleGetImc = serviceFn_p->moduleGetImc;
    moduleCpuServiceFn_p->moduleGetEntryFunc = serviceFn_p->moduleGetEntryFunc;
    moduleCpuServiceFn_p->moduleLogSet = serviceFn_p->moduleLogSet;
    moduleCpuServiceFn_p->errorString = serviceFn_p->errorString;
    moduleCpuServiceFn_p->cpuEntryFn_LLI[_DEST_FUNC_DEFAULT] = serviceFn_p->cpuEntryFn_LLI[_DEST_FUNC_DEFAULT];
    moduleCpuServiceFn_p->cpuEntryFn_LLI[_DEST_FUNC_RING] = serviceFn_p->cpuEntryFn_LLI[_DEST_FUNC_RING];
    moduleCpuServiceFn_p->cpuEntryFn_ULI[_DEST_FUNC_DEFAULT] = serviceFn_p->cpuEntryFn_ULI[_DEST_FUNC_DEFAULT];
    moduleCpuServiceFn_p->cpuEntryFn_ULI[_DEST_FUNC_RING] = serviceFn_p->cpuEntryFn_ULI[_DEST_FUNC_RING];

_adk_module_service_function_update_exit:

    ADK_RETURN (retValue);

} /* -----  end of function adk_module_service_function_update  ----- */


/*!
 * =============================================================
 * adk_module_register
 * @param[in]    sysHandle    adk system handle.
 *
 * @param[in]    moduleName_p    module name string.
 *
 * @param[in]    moduleInit    pointer to module initialization
 * function. This can be set to NULL, if the user already configured
 * a valid init function pointer in module_config_table.
 *
 * @param[out]    moduleHandle    pointer to a location where the
 * module handle is returned.
 *
 * @retval    ADK_SUCCESS         successfully completed module registration.
 * @retval    ADK_E_INVALID_HANDLE    one of the input handles (sysHandle
 * or module name pointer) is invalid.
 * @retval    ADK_E_MEM_UNAVAILABLE    Failed to allocate memory for module
 * registration.
 * @retval    ADK_E_INCONSISTEN_STATE   Unsupported module name. Module name
 * must be defined in adk_module_config.h
 *
 * @brief
 *
 * This function registers module properties with the module
 * database.
 *
 * =============================================================
 */
adk_status_t
adk_module_register (
        adk_sys_handle_t sysHandle,
        const uint8_t *moduleName_p,
        adk_module_init_func_t moduleInit,
        adk_module_handle_t *moduleHandle)
{
    adk_status_t        retValue = ADK_SUCCESS;
    uint32_t            cfgIndex=0, moduleIndex;
    adk_module_info_t  *moduleInfo_p;
    adk_sys_info_t     *sysInfo_p;
    char               *tmpModName_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system handle\n");
        goto _adk_module_register_exit;
    }

    if (!ADK_VALID_PTR(moduleHandle)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid module handle pointer\n");
        goto _adk_module_register_exit;
    }

    if (!ADK_VALID_PTR(moduleInit)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid module initialization function\n");
        goto _adk_module_register_exit;
    }

    if (!ADK_VALID_MODULE_NAME(moduleName_p)) {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid module name\n");
        goto _adk_module_register_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    moduleInfo_p = (adk_module_info_t *) sysInfo_p->utils.moduleMgrMasterHandle;

    while (1)
    {
        if (module_config_table[cfgIndex].moduleId == 0xff)
        {
            /* We have hit the last entry in the module config table.
             * Exit now with error code */
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_MODULE);
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                           "No matching module name found\n");
            goto _adk_module_register_exit;
        }

        if (!ADK_STRNCMP(module_config_table[cfgIndex].moduleName_p,
                         moduleName_p, ADK_MAX_MODULE_NAME_LENGTH))
        {
            moduleIndex = module_config_table[cfgIndex].moduleId;

            if (moduleIndex >= ADK_CUSTOMER_MODULE_ID_START)
            {
                retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_MODULE);
                ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                               "Invalid module ID\n");
                goto _adk_module_register_exit;
            }

            tmpModName_p = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, ADK_MAX_MODULE_NAME_LENGTH);

            if (!ADK_VALID_PTR(tmpModName_p))
            {
                retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
                ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                              "Failed to allocate memory\n");
                goto _adk_module_register_exit;
            }

            moduleInfo_p[moduleIndex].moduleInit = moduleInit;
            ADK_MODULE_HDL_COOKIE_SET(&moduleInfo_p[moduleIndex]);
            moduleInfo_p[moduleIndex].inUse = TRUE;
            moduleInfo_p[moduleIndex].moduleId = moduleIndex;
            moduleInfo_p[moduleIndex].sysHandle = sysHandle;

            ADK_STRNCPY(tmpModName_p, moduleName_p, ADK_MAX_MODULE_NAME_LENGTH);
            ADK_MODULE_NAME_SET(&(moduleInfo_p[moduleIndex]), tmpModName_p);
            ADK_MODULE_LOG_LEVEL_SET(&(moduleInfo_p[moduleIndex]),
                                     ADK_SYS_LOG_LEVEL_GET);

            if (ADK_SYS_LOCK_ENABLE(sysHandle))
            {
                moduleInfo_p[moduleIndex].module_lock =
                    (void *) ADK_ALLOC_LOCK();
                if (moduleInfo_p[moduleIndex].module_lock) {
                    if (ADK_INIT_LOCK(moduleInfo_p[moduleIndex].module_lock,
                                      ADK_MODULE_LOCK_SHARED,
                                      ADK_MODULE_LOCK_VALUE)) {
                        retValue =
                            ADK_ERROR_SEV_LEVEL_SET(ADK_E_RESOURCE_UNAVAILABLE);
                        ADK_LOG_ERROR(sysHandle,
                                      ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                                      "Failed to initialize module lock\n");
                        goto _runtime_error_3;
                    }
                } else {
                    retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
                    ADK_LOG_ERROR(sysHandle,
                                  ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                                  "Failed to allocate memory for module lock\n");
                    goto _runtime_error_2;
                }
            }
            else
            {
                moduleInfo_p[moduleIndex].module_lock = NULL;
            }

            *moduleHandle = (adk_module_handle_t)&(moduleInfo_p[moduleIndex]);
            ADK_LOG_INFO (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                          "Successfully registered module: %s handle: %"ADKPRIh"\n",
                          moduleName_p, *moduleHandle);
            break;
        }

        cfgIndex++;
    }

    goto _adk_module_register_exit;

_runtime_error_3:
    ADK_DESTROY_LOCK(moduleInfo_p[moduleIndex].module_lock);
    ADK_FREE_LOCK(moduleInfo_p[moduleIndex].module_lock);

_runtime_error_2:
    /*! Free the memory allocated for the module name */
    ADK_NVM_FREE(ADK_MODULE_NAME_GET(&(moduleInfo_p[moduleIndex])));

_adk_module_register_exit:

    if ((retValue != ADK_SUCCESS) && (moduleHandle != NULL))
        *moduleHandle = (adk_module_handle_t) NULL;

    ADK_RETURN (retValue);
}    /* -----  end of function adk_module_register  ----- */

/*!
 * =============================================================
 * adk_module_register_with_id
 * @param[in]  sysHandle     Adk system handle.
 * @param[in]  moduleName_p  Module name string, maximum of 32
 * characters.
 * @param[in]  moduleId      Module ID. It should be from 40-64.
 * @param[in]  moduleInit    Pointer to module initialization
 *                           function. This can be set to NULL, if
 *                           the user already configured a valid
 *                           init function pointer in
 *                           module_config_table.
 * @param[out] moduleHandle_p Pointer to a location where the
 *                            module handle is returned.
 *
 * @retval ADK_SUCCESS Successfully completed module registration.
 * @retval ADK_E_MEM_UNAVAILABLE Failed to allocate memory for module
 *                               registration.
 *
 * @brief Registers a customer module.
 *
 * This function registers module properties with the module
 * database. This should be used by the customer to register
 * their own (other than LSI supplied ADK) modules with the
 * module manager.
 *
 * =============================================================
 */
adk_status_t
adk_module_register_with_id (
    adk_sys_handle_t sysHandle,
    const uint8_t *moduleName_p,
    uint8_t moduleId,
    adk_module_init_func_t moduleInit,
    adk_module_handle_t *moduleHandle_p)
{
    adk_status_t        retValue = ADK_SUCCESS;
    adk_module_info_t  *moduleInfo_p;
    adk_sys_info_t     *sysInfo_p;
    char               *tmpModName_p;
    adk_module_handle_t	handle;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) || (moduleName_p == NULL) ||
        (moduleInit == NULL) || (moduleHandle_p == NULL))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Failed to register. Invalid input parameter\n");
        goto adk_module_register_with_id_return;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;
    handle = ADK_SYS_MODULE_HANDLE_GET(sysHandle);

    /*! Validate the module name */
    if (ADK_STRLEN (moduleName_p) > ADK_MAX_MODULE_NAME_LENGTH)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_MODULE);
        ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                       "Failed to register. Invalid module name\n");

        goto adk_module_register_with_id_return;
    }

    /*! Validate the module ID */
    if ((moduleId < ADK_CUSTOMER_MODULE_ID_START) ||
        (moduleId >= ADK_MAX_SUPPORTED_MODULES))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_MODULE);
        ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                       "Failed to register. Invalid module ID\n");
        goto adk_module_register_with_id_return;
    }

    /*! Check if the entry is taken already */
    moduleInfo_p = (adk_module_info_t *)
        sysInfo_p->utils.moduleMgrMasterHandle;
    if (moduleInfo_p[moduleId].inUse == TRUE)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_MODULE);
        ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                       "Failed to register. Module already registered\n");
        goto adk_module_register_with_id_return;
    }

    /*! Module is not registered, register it. There is no check done
     *  to see if a module with the same name exists.
     */
    tmpModName_p = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, ADK_MAX_MODULE_NAME_LENGTH);

    if (!ADK_VALID_PTR(tmpModName_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
        ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                       "Failed to allocate memory\n");
        goto adk_module_register_with_id_return;
    }

    moduleInfo_p[moduleId].moduleInit = moduleInit;
    ADK_MODULE_HDL_COOKIE_SET(&moduleInfo_p[moduleId]);
    moduleInfo_p[moduleId].moduleId = moduleId;
    moduleInfo_p[moduleId].sysHandle = sysHandle;

    ADK_STRNCPY(tmpModName_p, moduleName_p, ADK_MAX_MODULE_NAME_LENGTH);
    ADK_MODULE_NAME_SET(&(moduleInfo_p[moduleId]), tmpModName_p);
    ADK_MODULE_LOG_LEVEL_SET(&(moduleInfo_p[moduleId]),
                             ADK_SYS_LOG_LEVEL_GET);

    if (ADK_SYS_LOCK_ENABLE(sysHandle))
    {
        moduleInfo_p[moduleId].module_lock = (void *) ADK_ALLOC_LOCK();
        if (moduleInfo_p[moduleId].module_lock == NULL)
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "Failed to allocate memory for module lock\n");
            goto lock_memory_alloc_failed;
        }

        if (ADK_INIT_LOCK(moduleInfo_p[moduleId].module_lock,
                          ADK_MODULE_LOCK_SHARED,
                          ADK_MODULE_LOCK_VALUE))
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_RESOURCE_UNAVAILABLE);
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                          "Failed to initialize module lock\n");
            goto lock_initialisation_failed;
        }

        ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                     "Alloc and init module lock for %s\n", moduleName_p);
    }
    else
    {
        moduleInfo_p[moduleId].module_lock = NULL;
    }

    *moduleHandle_p = (adk_module_handle_t)&(moduleInfo_p[moduleId]);
    moduleInfo_p[moduleId].inUse = TRUE;

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully registered module: %s Handle: %"ADKPRIh"\n",
                 moduleName_p, *moduleHandle_p);

    ADK_RETURN (retValue);

lock_initialisation_failed:

    ADK_DESTROY_LOCK(moduleInfo_p[moduleId].module_lock);
    ADK_FREE_LOCK(moduleInfo_p[moduleId].module_lock);

lock_memory_alloc_failed:

    /*! Free the memory allocated for the module name */
    ADK_NVM_FREE(ADK_MODULE_NAME_GET(&(moduleInfo_p[moduleId])));

adk_module_register_with_id_return:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_register_with_id  ----- */

/*!
 * =============================================================
 * adk_module_unregister
 * @param[in]    sysHandle    adk system handle.
 * @param[in]    moduleHandle   ADK module's handle. This handle
 * is returned at the time of module registration.
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER  Invalid module handle.
 *
 * @brief
 *
 * This module unregister a previously registered module from
 * ADK system infrastructure.
 *
 * =============================================================
 */
adk_status_t
adk_module_unregister (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle
        )
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_module_info_t *moduleInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system handle\n");
        goto adk_module_unregister_return;
    }

    /* Validate the module handle*/
    if (!ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR (NULL, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                       "Invalid module handle %"ADKPRIh"\n", moduleHandle);
        goto adk_module_unregister_return;
    }

    moduleInfo_p = (adk_module_info_t *) moduleHandle;
    ADK_NVM_FREE(ADK_MODULE_NAME_GET(moduleHandle));
    moduleInfo_p->inUse = FALSE;
    ADK_DESTROY_LOCK(moduleInfo_p->module_lock);
    ADK_FREE_LOCK(moduleInfo_p->module_lock);
    moduleInfo_p->module_lock = NULL;

    ADK_LOG_INFO(NULL, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                 "Successfully unregistered module %"ADKPRIh"\n", moduleHandle);

adk_module_unregister_return:

    ADK_RETURN( retValue );

}    /* -----  end of function adk_module_unregister  ----- */

/*!
 * =============================================================
 * adk_module_start
 * @param[in]    sysHandle    adk system handle
 *
 * @param[in]    moduleHandle    handle of the module that is being
 * started. The caller can pass ADK_ALL_REGISTERED_MODULES to start
 * all modules if they don't care about individual return codes from
 * the modules.
 *
 * @retval    ADK_SUCCESS
 * @retval    ADK_E_INVALID_HANDLE    one of the handles passed in is
 * invalid.
 * @retval    return value from specific module initialization function.
 *
 * @brief
 *
 * This function calls the initialization function of the requested
 * module or call initialization function of all registered modules if
 * the module handle passed in is ADK_ALL_MODULE_HANDLES value (This
 * model is not fully tested as it may have to include additional optional
 * parameters like argument size and argument parameter). The
 * return value may be ADK_SUCCESS, ADK_E_INVALID_HANDLE or module
 * specific initialization error code. The caller can decode the module
 * specific error code using ADK error decoding mechanism.
 *
 * =============================================================
 */
adk_status_t
adk_module_start (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t moduleIndex;
    adk_module_info_t *moduleInfo_p;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) ||
        !ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system or module handle\n");
        goto _adk_module_start_exit;
    }

    if (moduleHandle == (adk_module_handle_t) ADK_ALL_REGISTERED_MODULES)
    {
        moduleInfo_p = (adk_module_info_t *)
            sysInfo_p->utils.moduleMgrMasterHandle;

        for (moduleIndex = 0; moduleIndex < ADK_MAX_SUPPORTED_MODULES;
             moduleIndex++, moduleInfo_p++)
        {
            if (!ADK_VALID_PTR (moduleInfo_p))
            {
                retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
                ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                              "Invalid module handle found\n");
                break;
            }

            if (moduleInfo_p->inUse == FALSE)
                continue;

            if (moduleInfo_p->moduleInit)
            {
                retValue = (*(moduleInfo_p->moduleInit))
                    ((adk_module_handle_t) moduleInfo_p);
                if (retValue != ADK_SUCCESS)
                {
                    retValue = ADK_ERROR_SEV_LEVEL_SET (retValue);
                    ADK_LOG_ERROR (sysHandle,
                                    ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                                    "Module initialization function Failed\n");
                    break;
                }

                moduleInfo_p->isStarted = TRUE;
            } else
            {
                ADK_LOG_WARNING(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                                "Init function not registered for module: %s\n",
                                ADK_MODULE_NAME_GET(moduleHandle));
            }
        }
    } else
    {
        moduleInfo_p = (adk_module_info_t *) moduleHandle;
        if (moduleInfo_p->moduleInit)
        {
            retValue = (*(moduleInfo_p->moduleInit)) (moduleHandle);
            if (retValue != ADK_SUCCESS)
            {
                retValue = ADK_ERROR_SEV_LEVEL_SET (retValue);
                ADK_LOG_ERROR (sysHandle,
                               ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                               "Module initialization function Failed: "
                               "'%s' (0x%x)\n",
                               adk_sys_status_get(sysHandle, retValue),
                               retValue);
            }
        }

        moduleInfo_p->isStarted = TRUE;
    }

_adk_module_start_exit:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_start  ----- */


/*!
 * =============================================================
 * adk_module_stop
 *
 * @param[in]    sysHandle    adk system handle
 *
 * @param[in]    moduleHandle    handle of the module that is being
 * de-initialized.
 *
 * @retval    ADK_SUCCESS
 * @retval    ADK_E_INVALID_HANDLE    one of the handles passed in is
 * invalid.
 * @retval    return value from specific module initialization function.
 *
 * @brief
 *
 * This function calls the shut-down function of the module. The shutdown
 * function must be resitered using adk_module_service_function_register
 * API (moduleTerminate field in the data structure).
 *
 * =============================================================
 */
adk_status_t
adk_module_stop (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_module_info_t *moduleInfo_p;
    uint32_t modId;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) ||
        !ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system or module handle\n");
        goto _adk_module_stop_exit;
    }

    moduleInfo_p = (adk_module_info_t *) moduleHandle;
    modId = moduleInfo_p->moduleId;

    if (adkModuleMgrCpuServiceFn_g[moduleInfo_p->moduleId].moduleTerminate)
    {
        retValue = (*(adkModuleMgrCpuServiceFn_g[modId].moduleTerminate))(moduleHandle);
    }

_adk_module_stop_exit:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_stop  ----- */

/*!
 * =============================================================
 * adk_module_attr_set
 *
 * @param[in]    sysHandle    adk system handle
 *
 * @param[in]    moduleHandle    adk module handle/
 *
 * @param[in]    attr_p    pointer to the attribute specific
 * data structure.
 *
 * @retval    ADK_SUCCESS.
 *
 * @retval    ADK_E_INVALID_HANDLE    invalid system or module
 * handle
 *
 * @retval    ADK_E_INVALID_PARAM    invalid input parameter.
 *
 * @brief
 *
 * This function set the module specific atttributes. The attributes
 * supported here are -debug leve, -.
 *
 * =============================================================
 */
adk_status_t
adk_module_attr_set (
        adk_sys_handle_t sysHandle,
        adk_module_handle_t moduleHandle,
        const void *attr_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_module_attr_info_t *moduleAttr_p;
    adk_module_info_t *moduleInfo_p;

    if (ADK_INVALID_SYS_HANDLE (sysHandle) ||
        !ADK_MODULE_HANDLE_VALIDATE(moduleHandle) ||
        !ADK_VALID_PTR (attr_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid input handle or attribute pointer\n");
        goto _adk_module_attr_set_exit;
    }

    moduleAttr_p = (adk_module_attr_info_t *) attr_p;
    moduleInfo_p = (adk_module_info_t *) moduleHandle;
    switch (moduleAttr_p->attrType)
    {
        case ADK_MODULE_LOG_LEVEL:
            /* Check the log level type */
            if ((moduleAttr_p->attr_val_u.logLevel != ADK_SEV_LEVEL_FATAL) &&
                (moduleAttr_p->attr_val_u.logLevel != ADK_SEV_LEVEL_ERROR) &&
                (moduleAttr_p->attr_val_u.logLevel != ADK_SEV_LEVEL_WARNING) &&
                (moduleAttr_p->attr_val_u.logLevel != ADK_SEV_LEVEL_INFO)) {
                ADK_FORCE_LOG_ERROR("Failed to set the log-level."
                                    " Invalid log level specified");
                retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                                    ADK_SEV_LEVEL_ERROR);
                break;
            }

            /* If module has registered a function to set log level then use
             * it else, set the log level directly in the structure */
            if (adkModuleMgrCpuServiceFn_g[moduleInfo_p->moduleId].moduleLogSet)
            {
                retValue =
                    (*(adkModuleMgrCpuServiceFn_g[moduleInfo_p->moduleId].
                        moduleLogSet))
                    (moduleHandle, moduleAttr_p->attr_val_u.logLevel);
                if (retValue != ADK_SUCCESS) {
                    ADK_FORCE_LOG_ERROR("Failed to set the log-level\n");
                }
            }
            else
                ADK_MODULE_LOG_LEVEL_SET(moduleHandle,
                                         moduleAttr_p->attr_val_u.logLevel);
            break;

        default:
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                           "Invalid attribute type %x\n",
                           moduleAttr_p->attrType);
            break;
    }

_adk_module_attr_set_exit:

    ADK_RETURN (retValue);

}    /* -----  end of function adk_module_attr_set  ----- */

/*!
 * =============================================================
 * adk_module_handle_get
 *
 * @param[in]  sysHandle   Adk system handle
 *
 * @param[in]  moduleId    Module ID as defined in adk_module_id.h file.
 *
 * @param[in]  serviceFnUpdateFn Pointer to the function that
 *                               updates the service functions.
 *
 * @param[out]  moduleHandle_p Pointer to adk_module_handle_t. This
 * location will contain module handle upon successful completion of
 * the API call.
 *
 *
 * @retval ADK_SUCCESS Successfully completed the requested operation.
 *
 * @retval ADK_E_INVALID_PARAMETER  Invalid input parameter.
 *
 * @retval ADK_E_SYS_NOT_INITIALIZED  ADK system is not initialized.
 * Successful completion of adk_sys_init API call is pre-requirement
 * for this API.
 *
 * @retval ADK_E_MODULE_NOT_REGISTERED ADK module not registered.
 * Successful registeration of the module is pre-requirement for this
 * API call.
 *
 * @brief
 *
 * This API retrieves the module handle from the information stored in
 * NVM and return it to the caller. The function serviceFnUpdateFn
 * is called only if it is not NULL. The serviceFnUpdateFn is used
 * to update the C language based service functions with SIF
 * after warm restart or with multiple instances of SIF.
 *
 * =============================================================
 */
adk_status_t
adk_module_handle_get (
        adk_sys_handle_t sysHandle,
        uint8_t moduleId,
        adk_module_service_function_update_func_t serviceFnUpdateFn,
        adk_module_handle_t *moduleHandle_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_module_info_t *moduleInfo_p;
    adk_module_handle_t moduleHandle;

    if (ADK_INVALID_SYS_HANDLE (sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_HANDLE);
        ADK_FORCE_LOG_ERROR("Invalid system handle\n");
        goto adk_module_handle_get_return;
    }

    if (!moduleHandle_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid module handle pointer\n");
        goto adk_module_handle_get_return;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;
    moduleInfo_p = (adk_module_info_t *) sysInfo_p->utils.moduleMgrMasterHandle;
    moduleHandle = (adk_module_handle_t ) &moduleInfo_p[moduleId];
    if (!ADK_MODULE_HANDLE_VALIDATE(moduleHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MODULE_NOT_REGISTERED);
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET(sysInfo_p),
                       "Module not registered\n");
        goto adk_module_handle_get_return;
    }

    if (serviceFnUpdateFn != NULL)
    {
        retValue = (*serviceFnUpdateFn) (moduleHandle);
        if (retValue != ADK_SUCCESS)
        {
            ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET(sysInfo_p),
                           "Module failed to register the CPU service functions\n");
            goto adk_module_handle_get_return;
        }
    }

    *moduleHandle_p = moduleHandle;

    ADK_LOG_INFO (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET(sysInfo_p),
                  "Returning module handle %"ADKPRIh" for module Id %d\n",
                   moduleHandle, moduleId);

adk_module_handle_get_return:

    if ((retValue != ADK_SUCCESS) && (moduleHandle_p != NULL))
        *moduleHandle_p = 0;

    ADK_RETURN( retValue );
}    /* -----  end of function adk_module_handle_get  ----- */

/*!    end addtogroup _sifFunctions
 *     @}
 * end addtogroup _sif
 * @}
 */
