/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_sif.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief
 *
 * This file contains the adk software initialization functions and
 * the ACP hardware initialization functions.
 *
 *
 *  **********************************************************************/

#include <adk_include.h>

/*
 * Enable the flag that will define the release variables and have them placed
 * in the sif module.
 */
#define ADK_RELEASE_VARIABLES
#include "adk_release_id.h"

#ifdef ADK_DEVICE_SNR
adk_status_t 
adk_sys_eth_dev_vsi_get (uint16_t eth_dev_id, uint32_t *vsi_id);
#endif

const char *adk_sys_error_string(adk_sif_status_t errorCode)
{
    switch (ADK_STATUS_ERROR_CODE_GET(errorCode)) {

    case ADK_SUCCESS:
        return ("ADK_SUCCESS");
    case ADK_E_INVALID_HANDLE:
        return ("ADK_E_INVALID_HANDLE");
    case ADK_E_SYS_NOT_INITIALIZED:
        return ("ADK_E_SYS_NOT_INITIALIZED");
    case ADK_E_SYS_ALREADY_INITIALIZED:
        return ("ADK_E_SYS_ALREADY_INITIALIZED");
    case ADK_E_RESYNC_FAILED:
        return ("ADK_E_RESYNC_FAILED");
    case ADK_E_UNSUPPORTED_CHIP_INSTANCE:
        return ("ADK_E_UNSUPPORTED_CHIP_INSTANCE");
    case ADK_E_MODULE_NOT_REGISTERED:
        return ("ADK_E_MODULE_NOT_REGISTERED");
    case ADK_E_RESOURCE_UNAVAILABLE:
        return ("ADK_E_RESOURCE_UNAVAILABLE");
    case ADK_E_INVALID_PARAMETER:
        return ("ADK_E_INVALID_PARAMETER");
    case ADK_E_INVALID_MODULE:
        return ("ADK_E_INVALID_MODULE");
    case ADK_E_UNREGISTERED_FUNCTION:
        return ("ADK_E_UNREGISTERED_FUNCTION");
    case ADK_E_INCONSISTENT_STATE:
        return ("ADK_E_INCONSISTENT_STATE");
    case ADK_E_IMC_NOT_FOUND:
        return ("ADK_E_IMC_NOT_FOUND");
    case ADK_E_ENTRY_FUNC_NOT_FOUND:
        return ("ADK_E_ENTRY_FUNC_NOT_FOUND");
    case ADK_E_UNKNOWN:
        return ("ADK_E_UNKNOWN");
    case ADK_E_UNSUPPORTED:
        return ("ADK_E_UNSUPPORTED");
    case ADK_E_HW_PROGRAMMING_ERROR:
        return ("ADK_E_HW_PROGRAMMING_ERROR");
    case ADK_E_HW_CONFIG_LOAD_ERROR:
        return ("ADK_E_HW_CONFIG_LOAD_ERROR");
    case ADK_E_MEM_UNAVAILABLE:
        return ("ADK_E_MEM_UNAVAILABLE");
    case ADK_E_INTERNAL_ERROR:
        return ("ADK_E_INTERNAL_ERROR");
    case ADK_E_VP_HANDLE_NOT_FOUND:
        return ("ADK_E_VP_HANDLE_NOT_FOUND");
    case ADK_E_VP_FLOW_DATA_WRITE_FAILED:
        return ("ADK_E_VP_FLOW_DATA_WRITE_FAILED");
    case ADK_E_VP_FLOW_DATA_READ_FAILED:
        return ("ADK_E_VP_FLOW_DATA_READ_FAILED");
    case ADK_E_VP_MC_FLOW_ADD_FAILED:
        return ("ADK_E_VP_MC_FLOW_ADD_FAILED");
    case ADK_E_VP_MC_FLOW_REM_FAILED:
        return ("ADK_E_VP_MC_FLOW_REM_FAILED");
    case ADK_E_NS_RSRC_NOT_FOUND:
        return ("ADK_E_NS_RSRC_NOT_FOUND");
    case ADK_E_NS_RSRC_CREATE_FAILED:
        return ("ADK_E_NS_RSRC_CREATE_FAILED");
    case ADK_E_NS_RSRC_DESTROY_FAILED:
        return ("ADK_E_NS_RSRC_DESTROY_FAILED");
    case ADK_E_NS_RSRC_QUERY_FAILED:
        return ("ADK_E_NS_RSRC_QUERY_FAILED");
    case ADK_E_NS_RSRC_ALLOCATE_FAILED:
        return ("ADK_E_NS_RSRC_ALLOCATE_FAILED");
    case ADK_E_NS_RSRC_FREE_FAILED:
        return ("ADK_E_NS_RSRC_FREE_FAILED");
    case ADK_E_NS_RSRC_UNALIGNED:
        return ("ADK_E_NS_RSRC_UNALIGNED");
    case ADK_W_RX_HIGHWATER_MARK_HIT:
        return ("ADK_W_RX_HIGHWATER_MARK_HIT");
    case ADK_E_RX_BUFFER_FULL:
        return ("ADK_E_RX_BUFFER_FULL");
    case ADK_W_CBFN_EXISTS:
        return ("ADK_W_CBFN_EXISTS");
    case ADK_W_CBFN_DOESNOT_EXIST:
        return ("ADK_W_CBFN_DOESNOT_EXIST");
    case ADK_W_SHM_EXISTS:
        return ("ADK_W_SHM_EXISTS");

    case ADK_E_INVALID_SHAPING_MODE:
        return ("ADK_E_INVALID_SHAPING_MODE");
    case ADK_E_INVALID_SCHEDULING_MODE:
        return ("ADK_E_INVALID_SCHEDULING_MODE");
    case ADK_E_INVALID_ARBITRATION_MODE:
        return ("ADK_E_INVALID_ARBITRATION_MODE");
    case ADK_E_QOS_SCHEDULER_LIST_NOT_EMPTY:
        return ("ADK_E_QOS_SCHEDULER_LIST_NOT_EMPTY");
    case ADK_E_QOS_SCHEDULER_INVALID:
        return ("ADK_E_QOS_SCHEDULER_INVALID");
    case ADK_E_QOS_QUEUE_INVALID:
        return ("ADK_E_QOS_QUEUE_INVALID");
    case ADK_E_RESOURCE_NOT_FOUND:
        return ("ADK_E_RESOURCE_NOT_FOUND");
    case ADK_E_MTM_TABLE_ERROR:
        return ("ADK_E_MTM_TABLE_ERROR");
    case ADK_E_MTM_TABLE_NOT_FOUND:
        return ("ADK_E_MTM_TABLE_NOT_FOUND");
    case ADK_E_MTM_DRAIN_BUSY:
        return ("ADK_E_MTM_DRAIN_BUSY");
    case ADK_E_SE_ERROR:
        return ("ADK_E_SE_ERROR");
    case ADK_E_SE_STATE_GET_FAILED:
        return ("ADK_E_SE_STATE_GET_FAILED");
    case ADK_E_SE_STATE_SET_FAILED:
        return ("ADK_E_SE_STATE_SET_FAILED");
    case ADK_E_MTM_PARAMETER_LENGTH_INVALID:
        return ("ADK_E_MTM_PARAMETER_LENGTH_INVALID");
    case ADK_E_DS_POLICY_TABLE_ERROR:
        return ("ADK_E_DS_POLICY_TABLE_ERROR");
    case ADK_E_INVALID_DS_CLASS_TYPE:
        return ("ADK_E_INVALID_DS_CLASS_TYPE");
    case ADK_E_INVALID_DS_CLASS_ID:
        return ("ADK_E_INVALID_DS_CLASS_ID");
    case ADK_E_INVALID_BM_POLICY:
        return ("ADK_E_INVALID_BM_POLICY");
    case ADK_E_INVALID_ATM_SERVICE_CATEGORY:
        return ("ADK_E_INVALID_ATM_SERVICE_CATEGORY");
    case ADK_E_UNSUPPORTED_POLICING_TYPE:
        return ("ADK_E_UNSUPPORTED_POLICING_TYPE");
    case ADK_E_INVALID_DS_POLICING_MODE:
        return ("ADK_E_INVALID_DS_POLICING_MODE");
    case ADK_E_INVALID_POLICING_MODE:
        return ("ADK_E_INVALID_POLICING_MODE,");
    case ADK_E_INVALID_POLICING_ID:
        return ("ADK_E_INVALID_POLICING_ID");

    case ADK_E_INVALID_PATTERN:
        return ("ADK_E_INVALID_PATTERN");
    case ADK_E_INVALID_ACTION:
        return ("ADK_E_INVALID_ACTION");
    case ADK_E_PATTERN_CREATE_FAILED:
        return ("ADK_E_PATTERN_CREATE_FAILED");
    case ADK_E_SUBNET_CREATE_FAILED:
        return ("ADK_E_SUBNET_CREATE_FAILED");
    case ADK_E_SUBNET_DESTROY_FAILED:
        return ("ADK_E_SUBNET_DESTROY_FAILED");
    case ADK_E_SUBNET_ADD_FAILED:
        return ("ADK_E_SUBNET_ADD_FAILED");
    case ADK_E_SUBNET_DELETE_FAILED:
        return ("ADK_E_SUBNET_DELETE_FAILED");
    case ADK_E_FIND_HIGHER_SUBNET_FAILED:
        return ("ADK_E_FIND_HIGHER_SUBNET_FAILED");
    case ADK_E_FPL_SYMBOL_NOT_FOUND:
        return ("ADK_E_FPL_SYMBOL_NOT_FOUND");
    case ADK_E_LEARN_FAILED:
        return ("ADK_E_LEARN_FAILED");
    case ADK_E_UNLEARN_FAILED:
        return ("ADK_E_UNLEARN_FAILED");
    case ADK_E_UNLEARN_ANY_FAILED:
        return ("ADK_E_UNLEARN_ANY_FAILED");
    case ADK_E_UVTREE_LEARN_FAILED:
        return ("ADK_E_UVTREE_LEARN_FAILED");
    case ADK_E_UVTREE_UNLEARN_FAILED:
        return ("ADK_E_UVTREE_UNLEARN_FAILED");
    case ADK_E_UVTREE_UNLEARN_ANY_FAILED:
        return ("ADK_E_UVTREE_UNLEARN_ANY_FAILED");
    case ADK_E_OTREE_APPEND_FAILED:
        return ("ADK_E_OTREE_APPEND_FAILED");
    case ADK_E_OTREE_INSERT_FAILED:
        return ("ADK_E_OTREE_INSERT_FAILED");
    case ADK_E_OTREE_DELETE_FAILED:
        return ("ADK_E_OTREE_DELETE_FAILED");
    case ADK_E_OTREE_CLEAR_FAILED:
        return ("ADK_E_OTREE_CLEAR_FAILED");
    case ADK_E_OTREE_BUILD_FAILED:
        return ("ADK_E_OTREE_BUILD_FAILED");
    case ADK_E_OTREE_NOVRULES_GET_FAILED:
        return ("ADK_E_OTREE_NOVRULES_GET_FAILED");
    case ADK_E_OTREESET_APPEND_FAILED:
        return ("ADK_E_OTREESET_APPEND_FAILED");
    case ADK_E_OTREESET_INSERT_FAILED:
        return ("ADK_E_OTREESET_INSERT_FAILED");
    case ADK_E_OTREESET_DELETE_FAILED:
        return ("ADK_E_OTREESET_DELETE_FAILED");
    case ADK_E_OTREESET_CLEAR_FAILED:
        return ("ADK_E_OTREESET_CLEAR_FAILED");
    case ADK_E_OTREESET_BUILD_FAILED:
        return ("ADK_E_OTREESET_BUILD_FAILED");
    case ADK_E_OTREESET_NOVRULES_GET_FAILED:
        return ("ADK_E_OTREESET_NOVRULES_GET_FAILED");

    case ADK_E_OVTREESET_APPEND_FAILED:
        return ("ADK_E_OVTREESET_APPEND_FAILED");
    case ADK_E_OVTREESET_INSERT_FAILED:
        return ("ADK_E_OVTREESET_INSERT_FAILED");
    case ADK_E_OVTREESET_DELETE_FAILED:
        return ("ADK_E_OVTREESET_DELETE_FAILED");
    case ADK_E_OVTREESET_CLEAR_FAILED:
        return ("ADK_E_OVTREESET_CLEAR_FAILED");
    case ADK_E_OVTREESET_BUILD_FAILED:
        return ("ADK_E_OVTREESET_BUILD_FAILED");
    case ADK_E_OVTREESET_REBUILD_FAILED:
        return ("ADK_E_OVTREESET_REBUILD_FAILED");
    case ADK_E_OVTREESET_NOVRULES_GET_FAILED:
        return ("ADK_E_OVTREESET_NOVRULES_GET_FAILED");

    case ADK_E_EIOA_PORT_MAX_PKT_SIZE_SET_FAILED:
        return("ADK_E_EIOA_PORT_MAX_PKT_SIZE_SET_FAILED");
    case ADK_E_EIOA_PORT_ENABLE_GET_FAILED:
        return("ADK_E_EIOA_PORT_ENABLE_GET_FAILED");
    case ADK_E_EIOA_PORT_ENABLE_SET_FAILED:
        return("ADK_E_EIOA_PORT_ENABLE_SET_FAILED");
    case ADK_E_EIOA_VP_ID_GET_FAILED:
        return("ADK_E_EIOA_VP_ID_GET_FAILED");
    case ADK_E_EIOA_VP_ID_SET_FAILED:
        return("ADK_E_EIOA_VP_ID_SET_FAILED");
    case ADK_E_EIOA_HASH_VALUE_GET_FAILED:
        return("ADK_E_EIOA_HASH_VALUE_GET_FAILED");
    case ADK_E_EIOA_TAG_GET_FAILED:
        return("ADK_E_EIOA_TAG_GET_FAILED");
    case ADK_E_EIOA_TAG_SET_FAILED:
        return("ADK_E_EIOA_TAG_SET_FAILED");
    case ADK_E_EIOA_TAG_ENABLE_FAILED:
        return("ADK_E_EIOA_TAG_ENABLE_FAILED");
    case ADK_E_EIOA_VLAN_CONFIG_FAILED:
        return("ADK_E_EIOA_VLAN_CONFIG_FAILED");
    case ADK_E_EIOA_SMAC_SET_FAILED:
        return("ADK_E_EIOA_SMAC_SET_FAILED");
    case ADK_E_EIOA_QINQ_SET_FAILED:
        return("ADK_E_EIOA_QINQ_SET_FAILED");
    case ADK_E_EIOA_CRC_SET_FAILED:
        return("ADK_E_EIOA_CRC_SET_FAILED");
    case ADK_E_EIOA_UC_FILTER_ENABLE_FAILED:
        return("ADK_E_EIOA_UC_FILTER_ENABLE_FAILED");
    case ADK_E_EIOA_BC_FILTER_ENABLE_FAILED:
        return("ADK_E_EIOA_BC_FILTER_ENABLE_FAILED");
    case ADK_E_EIOA_UC_FILTER_SET_FAILED:
        return("ADK_E_EIOA_UC_FILTER_SET_FAILED");
    case ADK_E_EIOA_SPEED_DUPLEX_GET_FAILED:
        return("ADK_E_EIOA_SPEED_DUPLEX_GET_FAILED");
    case ADK_E_EIOA_SPEED_DUPLEX_SET_FAILED:
        return("ADK_E_EIOA_SPEED_DUPLEX_SET_FAILED");
    case ADK_E_EIOA_MAC_STATS_GET_FAILED:
        return("ADK_E_EIOA_MAC_STATS_GET_FAILED");
    case ADK_E_EIOA_AUTONEG_SET_FAILED:
        return("ADK_E_EIOA_AUTONEG_SET_FAILED");
    case ADK_E_EIOA_FLOW_CNTRL_SET_FAILED:
        return("ADK_E_EIOA_FLOW_CNTRL_SET_FAILED");
    case ADK_E_EIOA_LOOPBACK_SET_FAILED:
        return("ADK_E_EIOA_LOOPBACK_SET_FAILED");
    case ADK_E_EIOA_TX_TIMESTAMP_GET_FAILED:
        return("ADK_E_EIOA_TX_TIMESTAMP_GET_FAILED");
    case ADK_E_EIOA_MAC_LINK_STATUS_GET_FAILED:
        return("ADK_E_EIOA_MAC_LINK_STATUS_GET_FAILED");

    case ADK_E_FG_UNKNOWN:
        return ("ADK_E_FG_UNKNOWN");
    case ADK_E_FG_CFG_PARSE:
        return ("ADK_E_FG_CFG_PARSE");
    case ADK_E_FG_INVALID_CLASSIFIER_ID:
        return ("ADK_E_FG_INVALID_CLASSIFIER_ID");
    case ADK_E_FG_INVALID_FILTER_ID:
        return ("ADK_E_FG_INVALID_FILTER_ID");
    case ADK_E_FG_NOT_INITIALIZED:
        return ("ADK_E_FG_NOT_INITIALIZED");
    case ADK_E_FG_INVALID_FILTER_TYPE:
        return ("ADK_E_FG_INVALID_FILTER_TYPE");
    case ADK_E_FG_NO_MEMORY:
        return ("ADK_E_FG_NO_MEMORY");
    case ADK_E_FG_GET_TOPLEVEL_MAP_FAILED:
        return ("ADK_E_FG_GET_TOPLEVEL_MAP_FAILED");
    case ADK_E_FG_GET__MAP_FAILED:
        return ("ADK_E_FG_GET__MAP_FAILED");
    case ADK_E_FG_SYMBOL_NOT_FOUND:
        return ("ADK_E_FG_SYMBOL_NOT_FOUND");
    case ADK_E_RCU_SYNC_FAILED:
        return ("ADK_E_RCU_SYNC_FAILED");
    case ADK_W_FG_NUM_CLASSIFIERS_PASSED_GREATER_THAN_ACTUAL_NUM:
        return ("ADK_W_FG_NUM_CLASSIFIERS_PASSED_GREATER_THAN_ACTUAL_NUM");
    case ADK_W_FG_NUM_FILTERS_PASSED_GREATER_THAN_ACTUAL_NUM:
        return ("ADK_W_FG_NUM_FILTERS_PASSED_GREATER_THAN_ACTUAL_NUM");
    case ADK_DP_E_ERROR:
        return ("ADK_DP_E_ERROR");
    case ADK_DP_W_QOS_RED_PDU_DROPPED:
        return ("ADK_DP_W_QOS_RED_PDU_DROPPED");
    case ADK_DP_W_PH_RX_DISABLED:
        return ("ADK_DP_W_PH_RX_DISABLED");
    case ADK_E_LOCK_FAILURE:
        return ("ADK_E_LOCK_FAILURE");
    case ADK_I_LOCK_BUSY:
        return ("ADK_I_LOCK_BUSY");
    case ADK_E_EM_ATTACH_FAILURE:
        return ("ADK_E_EM_ATTACH_FAILURE");
    default:
        return ("Unknown ADK Error Code");
    }
}

/*!
 * =============================================================
 * @internal
 * adk_sys_interface_id_alloc
 * @param[in]  sysHandle   system handle.
 *
 * @param[in]  interfaceId_p interface Id pointer.
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER  invalid input parameters.
 * @retval ADK_E_RESOURCE_UNAVAILABLE Failed to allocate resource.
 *
 * @brief
 *
 * This is a convenient function that is used to allocate ADK
 * interface Id. User should not call this function directly.
 * ADK_SYS_INTERFACE_ID_ALLOC macro should be used to allocate
 * interface Ids.
 *
 * =============================================================
 */
adk_status_t
adk_sys_interface_id_alloc (
        adk_sys_handle_t sysHandle,
        uint32_t     *interfaceId_p
        )
{
    adk_status_t retValue = ADK_SUCCESS;

    if (ADK_INVALID_SYS_HANDLE(sysHandle) || !interfaceId_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh
                             " or NULL interface Id pointer\n", sysHandle);
        goto adk_sys_interface_id_get_return;
    }

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Allocated interface Id %d\n", *interfaceId_p);

adk_sys_interface_id_get_return:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_sys_interface_id_get  ----- */


/*!
 * =============================================================
 * @internal
 * adk_sys_interface_id_free
 * @param[in]  sysHandle   system handle.
 *
 * @param[in]  interfaceId_p interface Id pointer.
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER  invalid input parameters.
 * @retval ADK_E_INCONSISTENT_STATE Failed to free resource.
 *
 * @brief
 *
 * This is a convenient function that is used to free ADK
 * interface Id. User should not call this function directly.
 * ADK_SYS_INTERFACE_ID_FREE macro should be used to allocate
 * interface Ids.
 *
 * =============================================================
 */
adk_status_t
adk_sys_interface_id_free (
        adk_sys_handle_t sysHandle,
        uint32_t     interfaceId
        )
{
    adk_status_t retValue = ADK_SUCCESS;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);

        goto adk_sys_interface_id_free_return;
    }

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
            "Successfully freed interface Id %d\n", interfaceId);

adk_sys_interface_id_free_return:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_sys_interface_id_free  ----- */

/*!
 * =============================================================
 * @internal
 * adk_sif_pcap_vsi_id_get
 *
 * @param[in]  sysHandle    System handle
 *
 * @param[out] *vsi_id_p    Pointer to VSI ID for the pcap ethdev
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Invalid parameter
 * @retval ADK_E_RESOURCE_UNAVAILABLE pcap device unavailable
 *
 * @brief
 *
 * This is an internal convenience function that is used to get the VSI
 * ID corresponding to the eth device configured by the application for
 * packet capture.
 *
 * =============================================================
 */
adk_status_t adk_sif_pcap_vsi_id_get (
        adk_sys_handle_t sysHandle,
        uint32_t *vsi_id_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_pcap_vsi_id_get_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (sysInfo_p->pcap_ethDevId == (uint16_t) -1)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_RESOURCE_UNAVAILABLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "No ethdevice configured for packet capture\n");
        goto _adk_sif_pcap_vsi_id_get_exit;
    }

#ifdef ADK_DEVICE_SNR
    retValue = adk_sys_eth_dev_vsi_get(sysInfo_p->pcap_ethDevId, vsi_id_p);
    if (retValue != ADK_SUCCESS)
    {
        goto _adk_sif_pcap_vsi_id_get_exit;
    }
#else
    *vsi_id_p = 0;
    retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_UNSUPPORTED);
    goto _adk_sif_pcap_vsi_id_get_exit;
#endif

_adk_sif_pcap_vsi_id_get_exit:

    return retValue;
}

/*!
 * =============================================================
 * @internal
 * adk_sif_pcap_id_alloc
 *
 * @param[in]  sysHandle    System handle
 *
 * @param[out] pcap_id_p    Pointer to Packet Capture ID.
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Invalid parameter
 *
 * @brief
 *
 * This is an internal convenience function that is used to allocate
 * ADK packet capture ID.
 *
 * =============================================================
 */
adk_status_t adk_sif_pcap_id_alloc (
        adk_sys_handle_t sysHandle,
        uint32_t *pcap_id_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_pcap_id_alloc_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (pcap_id_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "NULL pcap ID pointer\n");
        goto _adk_sif_pcap_id_alloc_exit;
    }

    *pcap_id_p = adk_rm_resource_reserve(sysInfo_p->pcap_id_rm_hdl);
    if (*pcap_id_p == ADK_RM_RSRC_INVALID)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_RESOURCE_UNAVAILABLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Failed to allocate pcap ID\n");
        goto _adk_sif_pcap_id_alloc_exit;
    }

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully allocated pcap ID %d\n", *pcap_id_p);

_adk_sif_pcap_id_alloc_exit:

    return retValue;
}

/*!
 * =============================================================
 * @internal
 * adk_sif_pcap_id_free
 *
 * @param[in]  sysHandle    System handle
 *
 * @param[in]  pcap_id      Packet Capture ID
 *
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @brief
 *
 * This is an internal convenience function that is used to free ADK
 * packet capture ID.
 *
 * =============================================================
 */
adk_status_t adk_sif_pcap_id_free (
        adk_sys_handle_t sysHandle,
        uint32_t pcap_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_pcap_id_free_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (pcap_id >= ADK_SIF_PCAP_ID_MAX)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Invalid pcap ID %d\n", pcap_id);
        goto _adk_sif_pcap_id_free_exit;
    }

    adk_rm_resource_free(sysInfo_p->pcap_id_rm_hdl, pcap_id);

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully free pcap ID %d\n", pcap_id);

_adk_sif_pcap_id_free_exit:

    return retValue;
}

/*!
 * =============================================================
 * @internal
 * adk_sif_pcap_id_config
 *
 * @param[in]  sysHandle            System handle
 *
 * @param[in]  pcap_id              Packet Capture ID
 *
 * @param[in]  pcap_entry_fn_id     Entry Function ID
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Invalid parameter
 *
 * @brief
 *
 * This is an internal convenience function that is used to map
 * a PCAP ID to an entry function that processes captured packets.
 * Specify entry function ID as 0 to remove the existing mapping.
 *
 * =============================================================
 */
adk_status_t adk_sif_pcap_id_config (
        adk_sys_handle_t sysHandle,
        uint32_t pcap_id,
        uint32_t pcap_entry_fn_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_sif_dp_shm_info_t *dpShm_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_pcap_id_config_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (adk_rm_resource_isset(sysInfo_p->pcap_id_rm_hdl, pcap_id) == FALSE)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_RESOURCE_UNAVAILABLE);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Invalid or unavailable pcap ID %d\n", pcap_id);
        goto _adk_sif_pcap_id_config_exit;
    }

    dpShm_p = ADK_SIF_DP_INFO_PTR_GET;

    dpShm_p->pcapIdEntryFnIdMap[pcap_id] = pcap_entry_fn_id;

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully mapped pcap ID %d to Entry fn ID %d\n",
                 pcap_id, pcap_entry_fn_id);

_adk_sif_pcap_id_config_exit:

    return retValue;
}

adk_status_t adk_sif_module_slowpath_callback_register(adk_sys_handle_t sysHandle,
                                                     uint8_t moduleId,
                                                     adk_sif_module_slowpath_cb_fn_t moduleCbFn)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_sif_dp_shm_info_t *dpShm_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_module_slowpath_callback_register_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (NULL == moduleCbFn)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Invalid callback function %p\n", moduleCbFn);
        goto _adk_sif_module_slowpath_callback_register_exit;
    }

    dpShm_p = ADK_SIF_DP_INFO_PTR_GET;

    dpShm_p->moduleSlowPathCbFn[moduleId] = moduleCbFn;

    printf("\n dpShm_p->moduleSlowPathCbFn[%d] - %p\n", moduleId, moduleCbFn );

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully registered callback fn ID %p to module id %u\n",
                 moduleCbFn, moduleId);

_adk_sif_module_slowpath_callback_register_exit:

    return retValue;
}

adk_status_t adk_sif_module_slowpath_callback_unregister(adk_sys_handle_t sysHandle,
                                                     uint8_t moduleId)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_sif_dp_shm_info_t *dpShm_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_module_slowpath_callback_unregister_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    dpShm_p = ADK_SIF_DP_INFO_PTR_GET;

    dpShm_p->moduleSlowPathCbFn[moduleId] = NULL;

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully unregistered callback for module id %u\n",
                 moduleId);

_adk_sif_module_slowpath_callback_unregister_exit:

    return retValue;
}
/*!
 * =============================================================
 * @internal
 * adk_sif_module_egress_callback_register
 *
 * @param[in]  sysHandle            System handle
 *
 * @param[in]  moduleId             Module id
 *
 * @param[in]  moduleCbFn           Callback function
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Invalid parameter
 *
 * @brief
 *
 * This is an internal convenience function that is used to
 * register a module callback function. The purpose of this
 * callback function is to do postprocessing such as counters
 * accounting after packets are successfully enqueued/transmitted
 * out of the HW.
 *
 * =============================================================
 */
adk_status_t adk_sif_module_egress_callback_register(adk_sys_handle_t sysHandle,
                                                     uint8_t moduleId,
                                                     adk_sif_module_egress_cb_fn_t moduleCbFn)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_sif_dp_shm_info_t *dpShm_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_module_egress_callback_register_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (NULL == moduleCbFn)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Invalid callback function %p\n", moduleCbFn);
        goto _adk_sif_module_egress_callback_register_exit;
    }

    dpShm_p = ADK_SIF_DP_INFO_PTR_GET;

    dpShm_p->moduleEgressCbFn[moduleId] = moduleCbFn;

    printf("\n dpShm_p->moduleEgressCbFn[%d] - %p\n", moduleId, moduleCbFn );

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully registered callback fn ID %p to module id %u\n",
                 moduleCbFn, moduleId);

_adk_sif_module_egress_callback_register_exit:

    return retValue;
}

/*!
 * =============================================================
 * @internal
 * adk_sif_module_egress_callback_unregister 
 *
 * @param[in]  sysHandle            System handle
 *
 * @param[in]  moduleId             Module id 
 *
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Invalid parameter
 *
 * @brief
 *
 * This is an internal convenience function that is used to 
 * unregister a module callback function. The purpose of this 
 * callback function is to do postprocessing such as counters
 * accounting after packets are successfully enqueued/transmitted 
 * out of the HW.
 *
 * =============================================================
 */
adk_status_t adk_sif_module_egress_callback_unregister(adk_sys_handle_t sysHandle,
                                                     uint8_t moduleId)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p;
    adk_sif_dp_shm_info_t *dpShm_p;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR("Invalid system handle %"ADKPRIh"\n",
                            sysHandle);
        goto _adk_sif_module_egress_callback_unregister_exit;
    }

    sysInfo_p = (adk_sys_info_t *) sysHandle;

    dpShm_p = ADK_SIF_DP_INFO_PTR_GET;

    dpShm_p->moduleEgressCbFn[moduleId] = NULL;

    ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                 "Successfully unregistered callback for module id %u\n",
                 moduleId);

_adk_sif_module_egress_callback_unregister_exit:

    return retValue;
}

/*!    end addtogroup _sifFunctions
 *     @}
 * end addtogroup _sif
 * @}
 */
