/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_sys.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief
 *
 * This file contains the adk software initialization functions and
 * the ACP hardware initialization functions.
 *
 *
 *  **********************************************************************/

#include "ncp.h"

#include "adk_sys.h"
#include "adk_sys_private.h"

#include "adk_release_id.h"

#include <adk_rm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <dlfcn.h>

#include "adk_sif_cfg_common.h"
#include "adk_sif_cfg.h"
#include "adk_config_overrides.h"
#include "adk_sys_services.h"
#include "adk_module_id.h"
#include "adk_module_mgr.h"
#include "adk_dev_net_internal.h"
#include "adk_pal_internal.h"
#include "adk_bind_interface.h"
#include "adk_qos_include.h"
#include "adk_cp_acl_list.h"

#ifdef ADK_DEVICE_SNR
#include "adk_nd_common.h"
#endif

#include "adk_hos.h"
#ifdef ADK_TIMER
#include "adk_timer.h"
#include "adk_pmu.h"
#endif
#include <adk_sys_dpdk.h>
#include "adk_shm.h"
#include <sys/time.h>
#include <adk_ipsec.h>

extern adk_pktio_hdl_t adkTunTapHandle_g[RTE_MAX_ETHPORTS];
struct rte_ring * adkDpPhRcvQueue_g[ADK_MAX_NUM_CORES]={0};
struct rte_ring * adkCpNetdRcvQueue_g=0;
static int adk_sys_attached_pid;

uint16_t adk_ph_rx_worker_core_id;

adk_sys_shm_config_t *adkSysShmConf_p;

#ifdef ADK_SYS_OVERLOAD_PROTECTION
adk_sys_ovp_info_t  *adk_sys_cp_ovp_table_p = NULL;
#endif

#define ADK_SYS_MAX_DSCP_VALUE      0x3F
#define ADK_SYS_MAX_VLAN_PCP_VALUE  0x07



static adk_status_t
adk_sys_copy_evd_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_evdev_info_t *evd_p,
                      adk_sys_resource_info_t   *res_info_p);
static adk_status_t
adk_sys_copy_crypto_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_crypto_dev_info_t   *evd_p,
                      adk_sys_resource_info_t   *res_info_p);

static adk_status_t
adk_sys_copy_wcore_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_wcore_info_t   *wc_p,
                      adk_sys_resource_info_t   *res_info_p);

static adk_status_t
adk_sys_copy_timer_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_timer_dev_info_t   *timer_dev_p,
                      adk_sys_resource_info_t   *res_info_p);

static adk_status_t
adk_sys_copy_eth_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_eth_dev_info_t *eth_dev_p,
                      adk_sys_resource_info_t   *res_info_p);

adk_status_t
adk_sys_rem_eth_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_eth_dev_info_t   *eth_dev_p);

static adk_status_t
adk_sys_rsrc_propagate(adk_sys_rsrc_oper_t oper,
                       adk_sys_resource_type_t        type,
                        adk_sys_resource_info_t rsrc);

adk_status_t adk_sif_rsrc_notify_fn_cbfn(adk_module_handle_t mod_handle, 
                                         adk_sys_rsrc_oper_t oper,
                                         adk_sys_resource_type_t type,
                                         adk_sys_resource_info_t rsrc);


static adk_status_t
adk_sys_load_modules_notify_fns(adk_sys_handle_t sysHandle);


static adk_status_t
adk_sys_create_eth_dev_rsrc_nvm_copy(
     adk_sys_rsrc_eth_dev_info_t   *eth_dev_p,
     adk_sys_rsrc_eth_dev_info_t   **dst_eth_dev_p);

static adk_status_t
adk_sys_create_evdev_rsrc_nvm_copy(
     adk_sys_rsrc_evdev_info_t   *evd_p,
     adk_sys_rsrc_evdev_info_t   **dst_evd_p);

static adk_status_t
adk_sys_create_cdev_rsrc_nvm_copy(
     adk_sys_rsrc_crypto_dev_info_t   *cdev_p,
     adk_sys_rsrc_crypto_dev_info_t   **dst_cdev_p);


#define ADK_NUM_MODULES_MAX  (1 << ADK_MODULE_ID_WIDTH)



static adk_module_resource_notify_func_t array_notify_fn_ptrs[ADK_NUM_MODULES_MAX] =
{
    [ADK_SYS_MODULE_ID] = adk_sif_rsrc_notify_fn_cbfn,
};

static adk_status_t
adk_sys_egress_stats_init(adk_sys_handle_t sys_handle);

static adk_status_t
adk_sys_egress_stats_shutdown(adk_sys_handle_t sys_handle);

/*!
 * =============================================================
 *
 * @fn adk_sys_init
 *
 * @param[in]    sysConf_p    pointer to adk_sys_conf_t data
 * structure. The elements of this data structure are described
 * adk_sys.h
 *
 * @param[out]   sysHandle_p   pointer to adk_sys_handle_t. The
 * API will copy the ADK system handle to this location upon
 * successful completion. This pointer internally carries
 * master handles of all subcomponents of system infrastructure
 * and the relevant hardware handles.
 *
 * @retval       ADK_SUCCESS   system initialization was succesfull.
 * @retval       ADK_E_UNSUPPORTED_CHIP_INSTANCE Chip instance value requested
 * is not supported in current ADK implementation.
 * @retval       ADK_E_MEM_UNAVAILABLE   failed to allocate memory
 * @retval       ADK_E_INVALID_PARAMETER invalid input parameter. This
 * could be invalid input pointers or invalid config file name.
 * during system initialization.
 * @retval       ADK_E_HW_CONFIG_LOAD_ERROR Failed to load configuration
 * to NCP hardware.
 *
 * @brief
 *
 * This function initialize the SIF software resources and loads the
 * configuration info to the NCP hardware. Current ADK implementation supports
 * four chip instances simulataneously (values 1-4).
 *
 * =============================================================
 */
adk_status_t
adk_sys_init (
        const adk_sys_conf_t *sysConf_p,
        adk_sys_handle_t *sysHandle_p)
{
    adk_status_t                       retValue = ADK_SUCCESS;
    adk_sys_info_t                     *sysInfo_p = NULL;
    adk_sif_cfg_info_t                 *cfg_hdl_p = NULL;
    adk_sys_conf_t                      sys_conf;
    adk_sys_conf_t                     *sys_conf_p = &sys_conf;
    adk_sys_conf_user_args_t            cliArgs;
    adk_sys_config_mode_t               adkMode;
    adk_sif_cfg_info_t                 *cfgInfo_p;
    adk_sif_cfg_eal_info_t             *ealInfo_p = NULL;
    adk_sys_config_info_t              *sysConfInfo_p;
    adk_sys_config_worker_lcore_info_t *lcoreInfo_p;
    uint32_t                            i, lcoreId;
    struct timeval                      timeVal = {0};
    adk_sif_dp_shm_info_t               *shmInfo;
    adk_sif_base_ntp_info_t             *baseTime;


    /* Validate input parameters */
    if ( !sysConf_p || !sysHandle_p)
    {
        ADK_PREINIT_WRITE("adk_sys_init failed. Invalid input parameter\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_init_return;
    }
    if ((sysConf_p->chipInstance < 1) || (sysConf_p->chipInstance > ADK_MAX_SUPPORTED_CHIPS))
    {
        ADK_PREINIT_WRITE("adk_sys_init failed. Unsupported chip instance."
                           " Supported range 1 - %d\n", ADK_MAX_SUPPORTED_CHIPS);
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_UNSUPPORTED_CHIP_INSTANCE,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_init_return;

    }

    /* Validate the log level (ignore the optional add-ons log types) */
    if (((sysConf_p->logLevel & ADK_SEV_LEVEL_INFO) != ADK_SEV_LEVEL_FATAL) &&
        ((sysConf_p->logLevel & ADK_SEV_LEVEL_INFO) != ADK_SEV_LEVEL_ERROR) &&
        ((sysConf_p->logLevel & ADK_SEV_LEVEL_INFO) != ADK_SEV_LEVEL_WARNING) &&
        ((sysConf_p->logLevel & ADK_SEV_LEVEL_INFO) != ADK_SEV_LEVEL_INFO)) {
        ADK_PREINIT_WRITE("adk_sys_init failed. Invalid log level specified\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_init_return;
    }

    /*
     * Allocated shared memory for IPC between primary and secondary process
     *
     * Set the static global pointer to the memzone for adk_sys
     */
    retValue = adk_shm_create(ADK_SYS_SHM_MEMZONE_NAME,
                              sizeof(adk_sys_shm_config_t),
                              ADK_SHM_LOOKUP_LCORE,
                              (adk_shm_handle_t **)&adkSysShmConf_p);
    if ((retValue != ADK_SUCCESS) &&
        (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS))
    {
        ADK_PREINIT_WRITE("[%s]: shared memory allocation failed.\n", __func__);
        goto adk_sys_init_return;
    }

    retValue = adk_shm_create(ADK_SIF_DP_SHM_MEMZONE_NAME,
                    sizeof(adk_sif_dp_shm_info_t), ADK_SHM_LOOKUP_LCORE,
                    (adk_shm_handle_t **)&(adkSysShmConf_p->sifDpShmHandle));
    if ((retValue != ADK_SUCCESS) &&
        (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS))
    {
        ADK_PREINIT_WRITE("[%s]: DP shared memory allocation failed.\n",
                          __func__);
        goto adk_sys_init_return;
    }
    if (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS) {
        sysConfInfo_p = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, sizeof(adk_sys_config_info_t));
        if (!ADK_VALID_PTR (sysConfInfo_p))
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            ADK_PREINIT_WRITE("[%s()]failed. Memory allocation failed\n", __func__);
            goto adk_sys_init_shm_return;
        }

        /* Set the system config handle to shared memory segment. This will be read
         * by the secondary process.
         */
        adkSysShmConf_p->sysConfHandle = (adk_config_handle_t) sysConfInfo_p;
    } else {
        sysConfInfo_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;
    }

#ifdef ADK_SYS_OVERLOAD_PROTECTION
    /* Initialize the global pointer to system overload protection table*/
    adk_sys_cp_ovp_table_p =  &sysConfInfo_p->ovpInfo;

    /* Initilize the overload protection table */
    adk_sys_cp_ovp_table_p->system_wide_policy_flag  = 1;
    adk_sys_cp_ovp_table_p->prio_type                = ADK_SYS_PRIO_TYPE_IP_DSCP;
    adk_sys_cp_ovp_table_p->default_drop_precedence  = ADK_SYS_DROP_PRECEDENCE_LOW;
    adk_sys_cp_ovp_table_p->default_action_drop      = false;
    adk_sys_cp_ovp_table_p->dscp_mask                = 0x00;
    adk_sys_cp_ovp_table_p->mode                     = 0x01;
#endif

    retValue = adk_sys_config_mode_get(&adkMode);
    if(retValue != ADK_SUCCESS){
        return retValue;
    }

    ADK_MEMCPY(sys_conf_p, sysConf_p, sizeof(adk_sys_conf_t));

    if (adkMode == ADK_SYS_CONFIG_MODE_STANDALONE)
    {
        if (NULL == sys_conf_p->userArgs_p)
        {
            char  *argv[3];
            char  argv_0[] = "";
            char  argv_1[] = "-f";
            char  argv_2[] = ADK_CONFIG_DEFAULT;

            argv[0] = argv_0;
            argv[1] = argv_1;
            argv[2] = argv_2;
            cliArgs.argc   = 3;
            cliArgs.argv_p = argv;

            sys_conf_p->userArgs_p = (void *)&cliArgs;
            ADK_PREINIT_WRITE("adk_sys_init : Will use default userArgs_p=%s\n", ADK_CONFIG_DEFAULT);
        }

        /* Initialize DPDK */
        retValue = adk_sys_dpdk_init(sys_conf_p, &cfg_hdl_p);
        if (retValue != ADK_SUCCESS)
        {
            ADK_PREINIT_WRITE("adk_sys_init : DPDK init failed\n");
            goto adk_sys_init_return;
        }
    }

    /* Allocate memory for sys data structure */
    sysInfo_p = (adk_sys_info_t *) ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_info_t));
    if (!ADK_VALID_PTR (sysInfo_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                             ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("adk_sys_init failed. Memory allocation failed\n");
        goto adk_sys_init_return;
    }

    ADK_SYS_HDL_COOKIE_SET(sysInfo_p);

    sysInfo_p->multiThread = sys_conf_p->multiThread;

    ADK_SYS_LOG_LEVEL_SET(sys_conf_p->logLevel);

    sysInfo_p->cfgHandle = (adk_handle_t)cfg_hdl_p;

    if (sysInfo_p->multiThread) {
        sysInfo_p->sysLock = (void *) ADK_ALLOC_LOCK();
        if (sysInfo_p->sysLock)
        {
            if (ADK_INIT_LOCK(sysInfo_p->sysLock,
                              ADK_MODULE_LOCK_SHARED,
                              ADK_MODULE_LOCK_VALUE))
            {
                retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_RESOURCE_UNAVAILABLE);
                ADK_PREINIT_WRITE("Failed to initialize system lock\n");
            }
        } else {
            retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
            ADK_PREINIT_WRITE("Failed to allocate memory for system lock\n");
        }
    } else {
        sysInfo_p->sysLock = NULL;
    }

    /* Step-2 Initialize module manager. Function call will
     * internally register the system infrastructure with
     * module manager. This will allow the system infrastrcture
     * components to use features like message logging. */
    retValue = adk_module_mgr_init ((adk_sys_handle_t) sysInfo_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("adk_sys_init : module manager init failed\n");
        goto adk_sys_init_return;
    }

    /*! Initialize bind infrastructure */
    retValue = adk_bind_infrastructure_init ((adk_sys_handle_t) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                       "Failed to initialize bind framework\n");
        goto adk_sys_init_return;
    }

#ifdef ADK_QOS_ENABLE
    /* Initialize ADK QoS */
    retValue = adk_qos_init((adk_sys_handle_t ) sysInfo_p);

    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize QoS module\n");
        goto adk_sys_init_return;
    }
#endif

    retValue = adk_hos_init((adk_sys_handle_t ) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize HOS component!\n");
        goto adk_sys_init_return;
    }

    retValue = adk_acl_init((adk_sys_handle_t ) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize ACL component!\n");
        goto adk_sys_init_return;
    }

#ifdef ADK_RCU
    /*initialise the sysInfo variable */
    retValue = adk_rcu_init((adk_sys_handle_t)sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                       "Failed to initialize RCU\n");
        goto adk_sys_init_return;
    }
#endif

    /* Initialize the egress stats db */
    retValue = adk_sys_egress_stats_init((adk_sys_handle_t ) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize egress stats DB\n");
        goto adk_sys_init_return;
    }

    /* Initialize ADK PAL sub-module */
    retValue = adk_pal_init((adk_sys_handle_t ) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize PAL sub-module\n");
        goto adk_sys_init_return;
    }

    /* Initialize the egress stats db */
    retValue = adk_dev_net_init((adk_sys_handle_t ) sysInfo_p);
    if ( retValue != ADK_SUCCESS )
    {
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                "Failed to initialize ADK Slow Path Device ID DB\n");
        goto adk_sys_init_return;
    }

    /* Create packet capture ID resource */
    sysInfo_p->pcap_id_rm_hdl =
        adk_rm_resource_init(ADK_SYS_MODULE_ID, 0, ADK_SIF_PCAP_ID_MAX);
    if (sysInfo_p->pcap_id_rm_hdl == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
        ADK_LOG_ERROR (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                       "Failed to create pcap ID resource handle\n");
        goto adk_sys_init_return;
    }

    ADK_MEMSET(ADK_SIF_DP_INFO_PTR_GET->pcapIdEntryFnIdMap, 0,
               (sizeof(ADK_SIF_DP_INFO_PTR_GET->pcapIdEntryFnIdMap)));

    /* Set the eth dev for packet capture to invalid value */
    sysInfo_p->pcap_ethDevId = (uint16_t) -1;

    /* Set the system_handle to the allocated data struct pointer */
    *sysHandle_p = (adk_sys_handle_t ) sysInfo_p;
    /* Set the system handle to shared memory segment. This will be read
     * by the secondary process
     */
    adkSysShmConf_p->sysHandle = (adk_sys_handle_t ) sysInfo_p;
    if (adkMode == ADK_SYS_CONFIG_MODE_STANDALONE)
    {
        cfgInfo_p = (adk_sif_cfg_info_t *)(sysInfo_p->cfgHandle);
        if(!cfgInfo_p)
        {
            ADK_PREINIT_WRITE("adk_sys_init failed! Config handle is NULL\n");
            return 0;
        }

        retValue = adk_sif_cfg_eal_info_get(cfgInfo_p->eal_info_tbl,
                                            &ealInfo_p);
        if ((retValue != ADK_SUCCESS) || (ealInfo_p == NULL))
        {
            ADK_PREINIT_WRITE("EAL info get failed\n");
            return 0;
        }

        adk_ph_rx_worker_core_id = ealInfo_p->worker_lcores[0];

        lcoreInfo_p = &(sysConfInfo_p->lcoreInfo);
        lcoreInfo_p->adkMode = ADK_SYS_CONFIG_MODE_STANDALONE;

        lcoreInfo_p->numLcores = ealInfo_p->num_worker_lcores;
        for (i = 0; i < ealInfo_p->num_worker_lcores; i++)
        {
            lcoreId = ealInfo_p->worker_lcores[i];
            lcoreInfo_p->lcore[i].id = lcoreId;
            lcoreInfo_p->lcore[i].index = ADK_LCORE_INDEX_LOOKUP(lcoreId);
            lcoreInfo_p->lcore[i].socket = rte_lcore_to_socket_id(lcoreId);
        }

        /* Create master thread and worker threads for data path processing */
        adk_sys_create_dp_threads((adk_sys_handle_t)sysInfo_p, sys_conf_p->configFile_p);
    }

    ADK_LOG_INFO (sysInfo_p, ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                  "Successfully completed system initialization %"ADKPRIh"\n",
                  (adk_handle_t) sysInfo_p);

    retValue = adk_sys_load_modules_notify_fns((adk_sys_handle_t)(sysInfo_p));

    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);
    if(shmInfo != NULL) {
        shmInfo->cpuTicksPerSec = rte_get_tsc_hz();

        /* Required granularity is atleast 1 micro second */
        if(shmInfo->cpuTicksPerSec < 1000000) {
            ADK_LOG_ERROR (sysInfo_p,
                           ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                           "CPU frequency must be greater than 1MHz to support NTP time at 1 microsec granularity. \n");
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            goto adk_sys_init_return;
        }

        /*
         * Get the number of seconds and microseconds since the Epoch from linux
         */
        if(gettimeofday(&timeVal, NULL)) {
            ADK_LOG_ERROR (sysInfo_p,
                           ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                           "Retrieval of time since the Epoch failed \n");
        }

        baseTime = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, sizeof(adk_sif_base_ntp_info_t));
        if (!ADK_VALID_PTR (baseTime))
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            ADK_LOG_ERROR (sysInfo_p,
                           ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                           "Memory allocation failed \n");
            goto adk_sys_init_return;
        }

        /*
         * Seconds since 1st Jan 1900 in NTP format. This will overflow in the
         * year 2036.
         */
        baseTime->baseNtpTimeHi = timeVal.tv_sec + ADK_SIF_NTP_BASETIME;

        /*
         * Seconds fraction since 1st Jan 1900 in NTP format. The granularity is at
         * the order 200 pico seconds. 1 sec = 200 picoseconds * 2^32 - 1
         */
        baseTime->baseNtpTimeLo =
            ((uint32_t)((timeVal.tv_usec * ADK_SIF_NTP_SCALE_FRAC)/1000000));

        baseTime->epochCpuTicks = rte_get_tsc_cycles();

        /* Atomically update the base time */
        rte_atomic64_set(&shmInfo->baseTime, (int64_t) baseTime);

        /* Set default udp checksum configuration to true for generation and validation */
        shmInfo->udpComputeFlag = true;
        shmInfo->udpValidateFlag = true;
        shmInfo->flowCacheCountersFlag = false;

        /* Set the default value of ipIpsec bypass to false. IPv4/6 modules will send the packets to ipsec module by default */
        shmInfo->ipsecBypass = false;

    } else {
        ADK_PREINIT_WRITE("Failed to initialize SIF module due to NULL "
                          "shared memory handle.\n");
    }

adk_sys_init_return:
    if (retValue != ADK_SUCCESS)
    {
        if (NULL != sysInfo_p)
        {
            adk_rm_resource_destroy(sysInfo_p->pcap_id_rm_hdl);    
            ADK_NVM_FREE (sysInfo_p);
        }

        if (NULL != cfg_hdl_p)
        {
            adk_sif_cfg_shutdown(cfg_hdl_p);
        }
    }
    ADK_RETURN (retValue);

adk_sys_init_shm_return:
    if(adk_shm_destroy(ADK_SYS_SHM_MEMZONE_NAME)!=ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("[%s()]: ADK-SYS shared memory destory failed.\n",
                            __func__);
    }
    ADK_RETURN (retValue);
}    /* -----  end of function adk_sys_init  ----- */

/*!
 * =============================================================
 * adk_sys_shutdown
 * @param[in]  - sysHandle   ADK system handle.
 *
 * @retval       ADK_SUCCESS Successful completion
 *
 * @retval       ADK_E_INVALID_PARAMETER invalid input parameter. This
 * could be due to invalid system handle.
 *
 * @brief
 *
 * This API de-allocates all the resources allocated during
 * adk_sys_init API call.
 *
 * =============================================================
 */
adk_status_t
adk_sys_shutdown (
        adk_sys_handle_t sysHandle,
        bool __attribute__((unused)) destryConfig)
{
    adk_status_t retValue = ADK_SUCCESS;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto adk_sys_shutdown_return;
    }

    /* TODO. Qos de-init should be called here */

    /* de-init bind infrastructure component */
    if (adk_bind_infrastructure_deinit(sysHandle) != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }
    if (adk_module_mgr_deinit(sysHandle) != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }

    retValue = adk_acl_shutdown();
    if (retValue != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (retValue,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }

    retValue = adk_hos_shutdown();
    if (retValue != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (retValue,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }

    retValue = adk_sys_egress_stats_shutdown(sysHandle);
    if (retValue != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (retValue,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }

    retValue = adk_dev_net_shutdown(sysHandle);
    if (retValue != ADK_SUCCESS)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (retValue,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_shutdown_failure;
    }

    ADK_PREINIT_WRITE("ADK sys shutdown successfull\n");
    goto adk_sys_shutdown_return;

adk_sys_shutdown_failure:
    ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                   "Invalid internal pointer. possible memory corruption\n");

adk_sys_shutdown_return:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_sys_shutdown  ----- */

/*!
 * =============================================================
 * adk_sys_detach
 *
 * @param[in]  - sysHandle   ADK system handle.
 *
 * @retval       ADK_SUCCESS Successful completion
 *
 * @retval       ADK_E_INVALID_PARAMETER invalid input parameter. This
 * could be due to invalid system handle.
 *
 * @brief
 *
 * This API detaches the process from ADK SIF services. This must be
 * called from all secondary processes before terminating.
 *
 * =============================================================
 */
adk_status_t
adk_sys_detach (
        adk_sys_handle_t __attribute__((unused)) sysHandle)
{
    return(adk_hos_detach());
}

/*!
 * =============================================================
 * @param[in]    sysConfig_p         pointer to adk_sys_config_t data
 * structure. The elements of this data structure are described
 * adk_sys.h
 *
 * @param[out]   sysConfigHandle_p   pointer to adk_config_handle_t. The
 * API will copy the ADK system slave mode configuration handle to this
 * location upon successful completion.
 *
 * @retval       ADK_SUCCESS   system configuration was succesfull.
 * @retval       ADK_E_MEM_UNAVAILABLE   failed to allocate memory
 * @retval       ADK_E_INVALID_PARAMETER invalid input parameter. This
 * could be invalid input pointers.
 *
 * @brief
 *
 * This function configures the ADK slave mode parameters. This API
 * must be called after DPDK EAL/platform initialization.
 *
 * =============================================================
 */
adk_status_t
adk_sys_config(
        const adk_sys_config_t *sysConfig_p,
        adk_config_handle_t *sysConfigHandle_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t  *adk_slave_config_p = NULL;
    uint32_t                i, id;

    /* Validate input parameters */
    if (ADK_INVALID_PTR(sysConfig_p) || ADK_INVALID_PTR(sysConfigHandle_p))
    {
        ADK_PREINIT_WRITE("adk_sys_config failed. Invalid pointers\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_config_return;
    }

    if((sysConfig_p->adk_mode != ADK_SYS_CONFIG_MODE_SLAVE) &&
      (sysConfig_p->adk_mode != ADK_SYS_CONFIG_MODE_SHARED_DATAPLANE))
    {
        ADK_PREINIT_WRITE("adk_sys_config failed. Invalid adk mode "
                          "configuration. This API supports only ADK shared "
                          "dataplane mode or slave mode configuration\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        goto adk_sys_config_return;
    }

    /* check only when application truly registers workers at the time of config
     * otherwise don't check. application now can register the worker cores
     * via adk_sys_resource_register API */
    if (sysConfig_p->num_worker_lcores != 0)
    {
        if (ADK_INVALID_PTR(sysConfig_p->worker_lcores))
        {
            ADK_PREINIT_WRITE("adk_sys_config failed. Invalid worker lcore "
                            "list pointer\n");
            retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                                ADK_SEV_LEVEL_ERROR);
            goto adk_sys_config_return;
        }
    }

    /* Allocated shared memory for IPC between primary and secondary process */
    retValue = adk_shm_create(ADK_SYS_SHM_MEMZONE_NAME,
                              sizeof(adk_sys_shm_config_t),
                              ADK_SHM_LOOKUP_LCORE,
                              (adk_shm_handle_t **)&adkSysShmConf_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("[%s()]: shared memory allocation failed.\n", __func__);
        goto adk_sys_config_return;
    }
        adk_slave_config_p = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, sizeof(adk_sys_config_info_t));
        if (!ADK_VALID_PTR (adk_slave_config_p))
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            ADK_PREINIT_WRITE("[%s()]failed. Memory allocation failed\n", __func__);
            goto adk_sys_config_shm_return;
        }

        /* Set the system config handle to shared memory segment. This will be read
         * by the secondary process.
         */
        adkSysShmConf_p->sysConfHandle = (adk_config_handle_t) adk_slave_config_p;


    /* Set the system config handle to shared memory segment. This will be read
     * by the secondary process.
     */
    sysConfigHandle_p = (adk_config_handle_t *) adk_slave_config_p;

    /* Update ADK slave configuration */
    adk_slave_config_p->lcoreInfo.adkMode = sysConfig_p->adk_mode;
    adk_slave_config_p->pktio_mempool = sysConfig_p->pktio_mempool ;
    adk_slave_config_p->ip_frag_mempool = sysConfig_p->ip_frag_mempool ;
    adk_slave_config_p->lcoreInfo.numLcores = sysConfig_p->num_worker_lcores ;
    for (i = 0; i < sysConfig_p->num_worker_lcores; i++)
    {
        id = sysConfig_p->worker_lcores[i];
        adk_slave_config_p->lcoreInfo.lcore[i].id = id;
        adk_slave_config_p->lcoreInfo.lcore[i].index = ADK_LCORE_INDEX_LOOKUP(id);
        adk_slave_config_p->lcoreInfo.lcore[i].socket = rte_lcore_to_socket_id(id);
    }

    retValue = adk_shm_create(ADK_SIF_DP_SHM_MEMZONE_NAME,
                    sizeof(adk_sif_dp_shm_info_t), ADK_SHM_LOOKUP_LCORE,
                    (adk_shm_handle_t **)&(adkSysShmConf_p->sifDpShmHandle));
    if ((retValue != ADK_SUCCESS) &&
        (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS))
    {
        ADK_PREINIT_WRITE("[%s]: DP shared memory allocation failed.\n",
                          __func__);
        goto adk_sys_config_return;
    }

    /* Set the system config handle to shared memory segment. This will be read
     * by the secondary process.
     */
    adkSysShmConf_p->sysConfHandle = (adk_config_handle_t) adk_slave_config_p;

    sysConfigHandle_p = (adk_config_handle_t *) adk_slave_config_p;

#ifdef ADK_SYS_OVERLOAD_PROTECTION
    /* Initilize the overload protection table */
    adk_slave_config_p->ovpInfo.system_wide_policy_flag  = 1;
    adk_slave_config_p->ovpInfo.prio_type                = ADK_SYS_PRIO_TYPE_IP_DSCP;
    adk_slave_config_p->ovpInfo.default_drop_precedence  = ADK_SYS_DROP_PRECEDENCE_LOW;
    adk_slave_config_p->ovpInfo.default_action_drop      = false;
    adk_slave_config_p->ovpInfo.dscp_mask                = 0x00;
    adk_slave_config_p->ovpInfo.mode                     = 0x01;

    adk_sys_cp_ovp_table_p =  &adk_slave_config_p->ovpInfo;
#endif

adk_sys_config_return:
    if ((retValue != ADK_SUCCESS) && (adk_slave_config_p)) {
        ADK_NVM_FREE (adk_slave_config_p);
    }
    ADK_RETURN( retValue );

adk_sys_config_shm_return:
    if(adk_shm_destroy(ADK_SYS_SHM_MEMZONE_NAME)!=ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("[%s()]: ADK-SYS shared memory destory failed.\n",
                            __func__);
    }
    ADK_RETURN (retValue);
}    /* -----  end of function adk_sys_config ----- */


/*!
 * =============================================================
 *
 * @param[out]   workerInfo_p   Pointer to the configured worker info
 *
 * @retval       ADK_SUCCESS
 * @retval       ADK_E_SYS_NOT_INITIALIZED
 * @retval       ADK_E_INVALID_PARAMETER
 *
 * @brief
 *
 * Helper function to get ADK worker lcore info.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */
adk_status_t
adk_sys_config_worker_lcore_info_get(
    adk_sys_config_worker_lcore_info_t **workerInfo_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t  *sysConf_p;


    if (NULL == workerInfo_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        goto _parameter_validation_fail;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    sysConf_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;

    if (NULL == sysConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    *workerInfo_p = &(sysConf_p->lcoreInfo);

_runtime_failure_1:
_parameter_validation_fail:
    return retValue;
}

/*!
 * =============================================================
 *
 * @param[out]   evdev_rsrc_list_p Pointer to the configured event-dev 
 *               resources list
 *
 * @retval       ADK_SUCCESS
 * @retval       ADK_E_SYS_NOT_INITIALIZED
 * @retval       ADK_E_INVALID_PARAMETER
 *
 * @brief
 *
 * Helper function to get event-dev/s resource info.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */

adk_status_t 
adk_sys_config_evdev_info_get (
        adk_sys_resource_info_t **evdev_rsrc_list_p, 
        uint8_t *num_evdevs)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t   *sysConf_p;


    if (NULL == evdev_rsrc_list_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        goto _parameter_validation_fail;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    sysConf_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;

    if (NULL == sysConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    *evdev_rsrc_list_p= (adk_sys_resource_info_t *)
                        &(sysConf_p->res_list[ADK_SYS_RSRC_TYPE_EVNT_DEV]);
    *num_evdevs   = sysConf_p->num_res[ADK_SYS_RSRC_TYPE_EVNT_DEV];

_runtime_failure_1:
_parameter_validation_fail:
    return retValue;
}

/*!
 * =============================================================
 *
 * @param[out]   cdev_rsrc_list_p Pointer to the configured crypto-dev 
 *                                resources list
 *
 * @retval       ADK_SUCCESS
 * @retval       ADK_E_SYS_NOT_INITIALIZED
 * @retval       ADK_E_INVALID_PARAMETER
 *
 * @brief
 *
 * Helper function to get crypto-dev/s resource info.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */

adk_status_t 
adk_sys_config_eth_dev_info_get (
        adk_sys_resource_info_t  **ethdev_rsrc_list_p,
        uint8_t *num_ethdevs)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t   *sysConf_p;


    if (NULL == ethdev_rsrc_list_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        goto _parameter_validation_fail;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    sysConf_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;

    if (NULL == sysConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    *ethdev_rsrc_list_p = (adk_sys_resource_info_t *)
                        &(sysConf_p->res_list[ADK_SYS_RSRC_TYPE_ETH_DEV]);
    *num_ethdevs   = sysConf_p->num_res[ADK_SYS_RSRC_TYPE_ETH_DEV];

_runtime_failure_1:
_parameter_validation_fail:
    return retValue;
}


/*!
 * =============================================================
 *
 * @param[out]   cdev_rsrc_list_p Pointer to the configured crypto-dev 
 *                                resources list
 *
 * @retval       ADK_SUCCESS
 * @retval       ADK_E_SYS_NOT_INITIALIZED
 * @retval       ADK_E_INVALID_PARAMETER
 *
 * @brief
 *
 * Helper function to get crypto-dev/s resource info.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */

adk_status_t 
adk_sys_config_crypto_dev_info_get (
        adk_sys_resource_info_t  **cdev_rsrc_list_p, 
        uint8_t *num_cdevs)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t   *sysConf_p;


    if (NULL == cdev_rsrc_list_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        goto _parameter_validation_fail;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    sysConf_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;

    if (NULL == sysConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    *cdev_rsrc_list_p = (adk_sys_resource_info_t *)
                        &(sysConf_p->res_list[ADK_SYS_RSRC_TYPE_CRYPTO_DEV]);
    *num_cdevs   = sysConf_p->num_res[ADK_SYS_RSRC_TYPE_CRYPTO_DEV];

_runtime_failure_1:
_parameter_validation_fail:
    return retValue;
}

/*!
 * =============================================================
 *
 * @param[out]   tmrdev_rsrc_list_p Pointer to the configured 
 *              timer-devices resources list
 *
 * @retval       ADK_SUCCESS
 * @retval       ADK_E_SYS_NOT_INITIALIZED
 * @retval       ADK_E_INVALID_PARAMETER
 *
 * @brief
 *
 * Helper function to get timer-dev/s resource info.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */

adk_status_t 
adk_sys_config_timer_dev_info_get (
        adk_sys_resource_info_t  **tmrdev_rsrc_list_p, 
        uint8_t *num_tmrdevs)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_config_info_t   *sysConf_p;


    if (NULL == tmrdev_rsrc_list_p)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        goto _parameter_validation_fail;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    sysConf_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;

    if (NULL == sysConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        goto _runtime_failure_1;
    }

    *tmrdev_rsrc_list_p = (adk_sys_resource_info_t *)
                        &(sysConf_p->res_list[ADK_SYS_RSRC_TYPE_TIMER_DEV]);
    *num_tmrdevs   = sysConf_p->num_res[ADK_SYS_RSRC_TYPE_TIMER_DEV];

_runtime_failure_1:
_parameter_validation_fail:
    return retValue;
}


/*!
 * =============================================================
 *
 * @param[out]   adk_mode   ADK configuration mode
 *
 * @retval       None
 *
 * @brief
 *
 * Helper function to get ADK configuration mode.
 * Note: Shared memory is created by adk_sys_config() in slave mode and during
 * adk_sys_init() in master mode.
 *
 * =============================================================
 */
adk_status_t
adk_sys_config_mode_get(adk_sys_config_mode_t *adk_mode)
{
    adk_sys_config_info_t *sysConf_p;
    adk_status_t retValue = ADK_SUCCESS;

    *adk_mode = ADK_SYS_CONFIG_MODE_STANDALONE;

    if (NULL != adkSysShmConf_p)
    {
        sysConf_p = (adk_sys_config_info_t *) adkSysShmConf_p->sysConfHandle;
        if (NULL != sysConf_p)
        {
            *adk_mode = sysConf_p->lcoreInfo.adkMode;
        }
    }

    return retValue;
}


/*!
 * =============================================================
 *
 * @param[out]   pktio_mempool   Pointer to pktio memorypool to
 * be used in packet generation in ADK like BFD.
 *
 * @retval       None
 *
 * @brief
 *
 * Helper function to get pktio memorypool info.
 *
 * =============================================================
 */
adk_status_t
adk_sys_config_pktio_mempool_get(void **pktio_mempool)
{
    adk_sys_config_info_t *sysConf_p;
    adk_status_t retValue = ADK_SUCCESS;

    if (NULL != adkSysShmConf_p)
    {
        sysConf_p = (adk_sys_config_info_t *) adkSysShmConf_p->sysConfHandle;
        if (NULL != sysConf_p)
        {
            *pktio_mempool = sysConf_p->pktio_mempool;
        }
    } else {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED);
        ADK_FORCE_LOG_ERROR ("ADK RTE Mem zone lookup failed \n");
    }
    return retValue;
}


/*!
 * =============================================================
 *
 * @param[out]   pktio_mempool   Pointer to memorypool to be
 * used in IP fragmentation.
 *
 * @retval       None
 *
 * @brief
 *
 * Helper function to get IP fragmentation memorypool info.
 *
 * =============================================================
 */
adk_status_t
adk_sys_config_ip_frag_mempool_get(void **ip_frag_mempool)
{
    adk_sys_config_info_t *sysConf_p;
    adk_status_t retValue = ADK_SUCCESS;

    if (NULL != adkSysShmConf_p)
    {
        sysConf_p = (adk_sys_config_info_t *) adkSysShmConf_p->sysConfHandle;
        if (NULL != sysConf_p)
        {
            *ip_frag_mempool= sysConf_p->ip_frag_mempool;
        }
    } else {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED);
        ADK_FORCE_LOG_ERROR ("ADK RTE Mem zone lookup failed \n");
    }
    return retValue;
}

#ifdef ADK_DEVICE_SNR
/*!
 * =============================================================
 *
 * @param[in]    eth_dev_id      Ethernet device ID 
 * @param[out]   vsi_number      Pointer to vsi number of a given 
 *                               eth device  memorypool to be
 * @retval       None
 *
 * @brief
 *
 * Helper function to get VSI number of a ethernet device.
 *
 * =============================================================
 */

adk_status_t 
adk_sys_eth_dev_vsi_get (uint16_t eth_dev_id, uint32_t *vsi_number)
{
    adk_status_t retValue = ADK_SUCCESS;
    struct rte_pmd_ice_dsi_dev_info ice_dsi_info;
    *vsi_number = UINT_MAX;
    
    if(!rte_pmd_ice_dsi_get_dev_info(eth_dev_id, &ice_dsi_info)) {
        *vsi_number = ice_dsi_info.vsi_number;
    } else {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR (" rte_pmd_ice_dsi_get_dev_info returned error\n");
    }

    return retValue;
}
#endif

/*!
 * =============================================================
 * adk_syshandle_get
 * @param[in]    adk_attach Boolean flag, if set, indicates that the caller
 *                          belongs to a new process which needs to attach
 *                          to an ADK system handle configured by a different
 *                          process.
 * @param[in]    chipInstance The chip instance for the ACP chip to which the calling
 *                          process would attach to. This parameter is used only
 *                          if the above "attach" parameter is TRUE. Supported
 *                          values are 1-4.
 * @param[out]  *sysHandle_p  Pointer to system handle. System handle will be
 *                            stored at this location upon successful return.
 *
 * @retval       ADK_SUCCESS Successfully completed the request.
 *
 * @retval       ADK_E_INVALID_PARAMETER invalid input pointer.
 *
 * @retval       ADK_E_SYS_NOT_INITIALIZED  ADK system not initialized.
 *               adk_sys_init() API should be called before this.
 *
 * @brief
 *
 * This API returns ADK system handle allocated during adk_sys_init()
 * API call. This API call will fail if invoked before a successful
 * completion of adk_sys_init API call.
 *
 * =============================================================
 */
adk_status_t
adk_syshandle_get (
        bool                adk_attach,
        uint32_t            chipInstance __attribute__((unused)),
        adk_sys_handle_t    *sysHandle_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_config_info_t *sysConf_p;

    if (ADK_INVALID_PTR(sysHandle_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("adk_syshandle_get: invalid input parameter\n");
        goto adk_syshandle_get_return;
    }

    (void)adk_attach;
    (void)adk_sys_attached_pid;

    retValue = adk_shm_lookup(ADK_SYS_SHM_MEMZONE_NAME,
                              (adk_shm_handle_t **)&adkSysShmConf_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE ("adk_syshandle_get failed: adk_sys_init API call is"
                           " pre-requirement for this API call\n");
        goto adk_syshandle_get_return;
    }

    if (ADK_INVALID_SYS_HANDLE(adkSysShmConf_p->sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("adk_syshandle_get failed: Invalid system handle"
                             " read from shared memory\n");
        goto adk_syshandle_get_return;
    }

    *sysHandle_p = (adk_sys_handle_t)adkSysShmConf_p->sysHandle;


    retValue = adk_sys_load_modules_notify_fns((adk_sys_handle_t)(*sysHandle_p));

    /* Initialize the global pointer to ethernet device port info structure */
    sysConf_p = (adk_sys_config_info_t *) adkSysShmConf_p->sysConfHandle;

#ifdef ADK_SYS_OVERLOAD_PROTECTION
    adk_sys_cp_ovp_table_p =  &sysConf_p->ovpInfo;
#endif

    if (adk_attach)
    {
        retValue = adk_hos_attach();
        if (retValue != ADK_SUCCESS)
        {
            goto adk_syshandle_get_return;
        }
    }

    if (adk_attach)
    {
        retValue = adk_acl_attach();
        if (retValue != ADK_SUCCESS)
        {
            goto adk_syshandle_get_return;
        }
    }

#ifdef ADK_RCU
    if(adk_rcu_config_get((adk_sys_handle_t)adkSysShmConf_p->sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("adk_rcu_config_get failed \n");
        goto adk_syshandle_get_return;
    }
#endif

    /* get the ADK PAL sub-module handle  */
    retValue = adk_pal_handle_get((adk_sys_handle_t)adkSysShmConf_p->sysHandle);
    if ( retValue != ADK_SUCCESS )
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("adk_pal_handle_get failed \n");
        goto adk_syshandle_get_return;
    }

adk_syshandle_get_return:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_syshandle_get  ----- */





/*!
 * =============================================================
 * adk_sys_hw_handle_get
 * @param[in]   sysHandle  ADK system handle returned during
 * adk_sys_init call.
 *
 * @param[out]   hwHandle_p  pointer to the location where
 * NCP hardware handle is stored.
 *
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 *
 * @brief
 *
 * This function returns NCP hardware handle from the given
 * system handle. adk_sys_init API must be called before
 * calling this function.
 *
 * =============================================================
 */
adk_status_t
adk_sys_hw_handle_get (
        adk_sys_handle_t sysHandle,
        void * *hwHandle_p
        )
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((ADK_INVALID_SYS_HANDLE(sysHandle) || !hwHandle_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh" or NULL pointer\n",
                             sysHandle);
        goto adk_sys_hw_handle_get_return;
    }

    *hwHandle_p = (void*)0x1;

adk_sys_hw_handle_get_return:
    ADK_RETURN( retValue );
}    /* -----  end of function adk_sys_hw_handle_get  ----- */


/*!
 * =============================================================
 * adk_sys_flow_id_get
 * @param[in]   sysHandle  ADK system handle returned during
 *                         adk_sys_init call.
 *
 * @param[in]   flowHandle  The handle returned when the flow was created
 *
 * @param[out]  flowId_p  Returns The ID associated with the passed flow handle
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Indicates the system handle or flow handle
 *         are not valid handles for their data type or that flowId_p is a
 *         null pointer.
 *
 * @brief Returns the flow ID for a given flow handle
 *
 * =============================================================
 */
adk_status_t
adk_sys_flow_id_get(
        adk_sys_handle_t sysHandle,
        adk_flow_handle_t flowHandle,
        uint32_t *flowId_p)
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((ADK_INVALID_SYS_HANDLE(sysHandle) || !flowId_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh" or NULL pointer\n",
                             sysHandle);
        goto _adk_sys_flow_id_get_exit;
    }

    if (!ADK_SYS_VALID_FLOW_HANDLE(flowHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid flow handle %"ADKPRIh"\n", flowHandle);
        goto _adk_sys_flow_id_get_exit;
    }

    *flowId_p = ADK_SYS_FLOWID_GET(flowHandle);

_adk_sys_flow_id_get_exit:

    ADK_RETURN(retValue);
}

/*!
 * =============================================================
 * adk_sys_interface_id_get
 * @param[in]   sysHandle  ADK system handle returned during
 *                         adk_sys_init call.
 *
 * @param[in]   ifHandle  The interface handle when the interface was created
 *
 * @param[out]  interfaceId_p  Returns the ID associated with the handle
 *                             passed
 *
 * @retval ADK_SUCCESS Successful completion
 * @retval ADK_E_INVALID_PARAMETER Indicates the system handle or interface
 *         handle are not valid handles for their data type or that
 *         interfaceId_p is a null pointer.
 *
 * @brief Returns the interface ID for a given interface handle
 *
 * =============================================================
 */
adk_status_t
adk_sys_interface_id_get(
        adk_sys_handle_t sysHandle,
        adk_interface_handle_t ifHandle,
        uint32_t *interfaceId_p)
{
    adk_status_t retValue = ADK_SUCCESS;

    if ((ADK_INVALID_SYS_HANDLE(sysHandle) || !interfaceId_p))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh" or NULL pointer\n",
                             sysHandle);
        goto _adk_sys_interface_id_get_exit;
    }

    if (!ADK_SYS_VALID_INTERFACE_HANDLE(ifHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid interface handle %"ADKPRIh"\n", ifHandle);
        goto _adk_sys_interface_id_get_exit;
    }

    *interfaceId_p = ADK_SYS_INTERFACE_ID_GET(ifHandle);

_adk_sys_interface_id_get_exit:

    ADK_RETURN(retValue);
}

/*!
 * =============================================================
 * adk_sys_status_get
 *
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 *
 * @param[in]   errorCode  The error code returned by any ADK API
 *
 * @retval      The pointer to corresponding error string
 *
 * @brief
 *
 * This function returns a meaningful error string for a given ADK
 * error code. Note that, the ADK must have be initialized before
 * calling this function.
 *
 * =============================================================
 */
const char *
adk_sys_status_get (
    adk_sys_handle_t sysHandle,
    adk_status_t errorCode)
{
    adk_module_handle_t mod_handle;

    /* Mask out the severity level */
    errorCode = ADK_STATUS_SEV_LEVEL_MASK(errorCode);

    if (ADK_STATUS_MODULE_ID_GET(errorCode) == ADK_SYS_MODULE_ID) {
        return (adk_sys_error_string((enum adk_sif_status_e)(ADK_STATUS_ERROR_CODE_GET(errorCode))));
    }

    if (adk_module_handle_get(sysHandle,
                              ADK_STATUS_MODULE_ID_GET(errorCode),
                              NULL,
                              &mod_handle) == ADK_SUCCESS) {
        return (adk_module_error_string_get(mod_handle, errorCode));
    }

    return ("Unknown ADK Error Code");
}

/*!
 * =============================================================
 * adk_sys_global_stats_get
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 *
 * @param[out]  stats       The ADK global stats maintained by SIF.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 *
 * @brief
 *
 * This function returns the global ADK SIF stats which moslty consists
 * of errors which occur in MPP.
 *
 * =============================================================
 */
adk_status_t
adk_sys_global_stats_get (
        adk_sys_handle_t sysHandle,
        adk_sys_global_stats_t *stats_p)
{
    adk_status_t        retValue = ADK_SUCCESS;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_global_stats_get_exit;
    }

    ADK_MEMSET(stats_p, 0, sizeof(adk_sys_global_stats_t));

_adk_sys_global_stats_get_exit:
    return(retValue);
}

/*!
 * =============================================================
 * adk_sys_log_enable
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in]   log_type    The log type to be enabled
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 *
 * This function enables particular log type. The log type parameter
 * can be logical OR of multiple log types as well.
 *
 * =============================================================
 */
adk_status_t adk_sys_log_enable(
        adk_sys_handle_t sysHandle,
        adk_sys_log_type_t log_type)
{
    adk_status_t        retValue = ADK_SUCCESS;
    adk_sys_log_type_t  curr_log_level;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_log_enable_exit;
    }

    curr_log_level = ADK_SYS_LOG_LEVEL_GET;
    curr_log_level |= log_type;

    ADK_SYS_LOG_LEVEL_SET(curr_log_level);

_adk_sys_log_enable_exit:

    return(retValue);
}

/*!
 * =============================================================
 * adk_sys_log_disable
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in]   log_type    The log type to be disabled
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 *
 * This function disables particular log type. The log type parameter
 * can be logical OR of multiple log types as well.
 *
 * =============================================================
 */
adk_status_t adk_sys_log_disable(
        adk_sys_handle_t sysHandle,
        adk_sys_log_type_t log_type)
{
    adk_status_t        retValue = ADK_SUCCESS;
    adk_sys_log_type_t  curr_log_level;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_log_disable_exit;
    }

    curr_log_level = ADK_SYS_LOG_LEVEL_GET;
    curr_log_level &= ~log_type;

    ADK_SYS_LOG_LEVEL_SET(curr_log_level);

_adk_sys_log_disable_exit:

    return(retValue);
}

/*!
 * =============================================================
 * Enable/Disable checksum validation:
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in]    Flag : true -> Checksum validation will be performed when a UDP packet is received and packet will be dropped on validation failure.
                       false -> Checksum validation will not be performed.

 * @retval ADK_SUCCESS           : Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 *
 * This function is used to set global configuration for udp checksum validation.
 * By default checksum validation is enabled.
 *
 * =============================================================
 */
adk_status_t adk_sys_udp_cksum_validate(
        adk_sys_handle_t sysHandle,
        bool flag)
{
    adk_status_t        retValue = ADK_SUCCESS;
    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_udp_cksum_validate_exit;
    }

    ADK_SYS_UDP_VALIDATE_FLAG_SET(flag);

_adk_sys_udp_cksum_validate_exit:
        return(retValue);
}

/*!
 * =============================================================
 * Configure UDP checksum generation behaviour .
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in]    compute    true: Compute and write the checksum value
      False: Do not compute and overwrite it to 0
 *
 * @retval ADK_SUCCESS           : Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 *
 * This function is used to set global configuration for UDP checksum generation.
 * By default checksum generation is enabled.
 *
 * =============================================================
*/
adk_status_t adk_sys_udp_cksum_gen_config(
        adk_sys_handle_t sysHandle,
        bool compute)
{
    adk_status_t        retValue = ADK_SUCCESS;
    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_udp_cksum_gen_config_exit;
    }

    ADK_SYS_UDP_GEN_CONFIG_SET(compute);

_adk_sys_udp_cksum_gen_config_exit:
    return(retValue);
}


/*!
 * =============================================================
 * Configure IPSec Bypass behaviour
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in] bypass     :   0: Disable IPsec bypass (When IPv4/6 receives a packet, it will be first sent to IPSec.
 *                           1: Enable IPSec bypass (IPv4/6 will not send packet to IPSec module in the DL and UL paths.
 *
 * @retval ADK_SUCCESS           : Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 *       API to enable or disable bypass to IPSec module. This API is NOT used to enable or
 * disable IPSec module. This API sets whether IPv4/6 will send the packet to IPSec or not.
 * Its application's responsibility to make sure that IPSec module is enabled when bypass is
 * disabled through this API.  Default value of 'bypass' is 0.
 *
 * =============================================================
 */
adk_status_t adk_sys_ipsec_bypass_config(
        adk_sys_handle_t sysHandle,
        bool bypass)
{
    adk_status_t        retValue = ADK_SUCCESS;
    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_ipsec_bypass_config_exit;
    }

    ADK_SYS_IP_IPSEC_BYPASS_CONFIG_SET(bypass);

_adk_sys_ipsec_bypass_config_exit:
    return(retValue);
}

/*!
 * =============================================================
 * adk_sys_global_stats_print
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 *
 * @brief
 *
 * This function dumps the SIF global stats on the console. For all stats
 * except for bind_indirect_cnt and bind_direct_cnt, if the value is 0, it
 * will not be printed.
 *
 * =============================================================
 */
adk_status_t
adk_sys_global_stats_print (
        adk_sys_handle_t sysHandle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_global_stats_t stats;

    ADK_MEMSET(&stats, 0, sizeof(adk_sys_global_stats_t));

    retValue = adk_sys_global_stats_get(sysHandle, &stats);
    if (retValue != ADK_SUCCESS)
        goto adk_sys_global_stats_print_return;

    ADK_INFO_WRITE("\nADK SIF Global Stats"
                   " (Drop counters with zero count not shown):\n\n");


adk_sys_global_stats_print_return:
    ADK_RETURN( retValue );
}

#ifdef ADK_SYS_OVERLOAD_PROTECTION
/*!
 * =============================================================
 * adk_sys_overload_protection_stats_print
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 *
 * @brief
 *
 * This function dumps the SIF overload protection stats on the console.
 *
 * =============================================================
 */
adk_status_t
adk_sys_overload_protection_stats_print(
        adk_sys_handle_t sysHandle)
{
    adk_status_t retValue = ADK_SUCCESS;

    RTE_SET_USED(sysHandle);

    ADK_INFO_WRITE("\nADK SIF Overload Protection Stats:\n\n");

    ADK_INFO_WRITE("\t mode :          \t%10u\n",
                                adk_sys_cp_ovp_table_p->mode);
    ADK_INFO_WRITE("\t default-action-drop :          \t%10u\n",
                                adk_sys_cp_ovp_table_p->default_action_drop);
    ADK_INFO_WRITE("\t non-marked-pkts :          \t%10lu\n",
                                adk_sys_cp_ovp_table_p->non_marked_pkts);
    ADK_INFO_WRITE("\t marked-dscp-pkts :          \t%10lu\n",
                                adk_sys_cp_ovp_table_p->marked_dscp_pkts);
    ADK_INFO_WRITE("\t marked-non-dscp-pkts :          \t%10lu\n",
                                adk_sys_cp_ovp_table_p->marked_non_dscp_pkts);

    ADK_RETURN( retValue );
}
#endif


/* TODO: Currently just stub code returning OK */

adk_status_t
adk_sys_adk_version_info_get(
    adk_sys_handle_t        sysHandle,
    adk_sys_version_info_t *adk_version_p)
{
    RTE_SET_USED(sysHandle);

    ADK_MEMSET(adk_version_p, 0, sizeof(adk_sys_version_info_t));
    adk_version_p->adk_build_date   = adk_build_date;
    adk_version_p->adk_release_date = adk_release_date;
    adk_version_p->adk_scm_tag      = adk_scm_tag;
    adk_version_p->adk_version      = adk_release_id;
    adk_version_p->adk_rte_version  = adk_rte_version_id;
#ifdef ADK_DEVICE_SNR
    adk_version_p->adk_netd_fp_intf = ADK_NETD_FP_INTF_VERSION;
#else
    adk_version_p->adk_netd_fp_intf = "N/A";
#endif
    return ADK_SUCCESS;
}


/*!
 * ====================================================================
 * @param[in]    tdev_info_p pointer to adk_sys_timer_dev_info_t data
 * structure. The elements of this data structure are described adk_sys.h
 *
 * @retval       ADK_SUCCESS   system configuration was succesfull.
 * @retval       ADK_E_INVALID_PARAMETER invalid input parameter. This
 * could be invalid input pointers.
 *
 * @brief
 *
 * This function registres given timer-device to ADK System.
 * ADK dataplane uses this timer device to implement its timer depedent
 * functionality.
 *
 * This API must be called after DPDK EAL/platform initialization and before
 * ADK DP modules initializations.
 *
 * ====================================================================
 */

uint8_t     adkTimerBdInstance_g;

adk_status_t
adk_sys_timer_dev_register(const adk_sys_timer_dev_info_t *tdev_info_p)
{
    adk_status_t retValue = ADK_SUCCESS;

    /* Validate input parameters */
    if (!tdev_info_p)
    {
        ADK_FORCE_LOG_ERROR("[%s()] failed. Invalid pointers\n",__func__);
        retValue = ADK_E_INVALID_PARAMETER;
        goto _adk_sys_timer_dev_register_exit;
    }

    if (tdev_info_p->type != ADK_SYS_TIMER_DEV_TYPE_EVNT_TIMER_BD)
    {
        ADK_FORCE_LOG_ERROR("[%s()] Invalid Timer Dev-TYPE. Must be TIMER-BD-TYPE only.\n", __func__);
        retValue = ADK_E_INVALID_PARAMETER;
        goto _adk_sys_timer_dev_register_exit;
    }

    adkTimerBdInstance_g = tdev_info_p->timer_bd_instance_id;


_adk_sys_timer_dev_register_exit:

    ADK_RETURN( retValue );
}

#ifdef ADK_SYS_OVERLOAD_PROTECTION
/*!
 * =============================================================
 * adk_sys_ethdev_drop_precedence_map
 *
 * @param[in] sysHandle    ADK System Handle
 * @param[in] eth_dev_id   The id of the eth-device to configure the drop-precedence.
 *                         To do the system wide configuration appliciaton is required 
 *                         pass "-1" as eth-dev-id. The system wide confiruation is only
 *                         suuported for XEON targets. 
 * @param[in] drop_info_p  pointer to a adk_sys_ethdev_drop_precedence_info_t structure
 *                         to pass the packet priority to drop-precedence mapping inforamtion. 
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * @brief API to configure the packet-priority to drop-precedence mapping 
 * either system-wide or per eth-device.
 *
 * ==============================================================
 */
adk_status_t adk_sys_ethdev_drop_precedence_map(
    adk_sys_handle_t sysHandle,
    int16_t eth_dev_id, 
    adk_sys_ethdev_drop_precedence_info_t *drop_info_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    int index =0;
    adk_sys_pkt_prio_drop_precedence_map_t *map_list_p;
    uint8_t     pkt_prio, dscp_mask, inv_dscp_mask, i;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("[%s]: invalid system handle\n", __func__);
        goto adk_sys_ethdev_drop_precedence_map_return;
    }

    if(drop_info_p== NULL) {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "invalid parameter \n");
        goto adk_sys_ethdev_drop_precedence_map_return;
    }

    if(eth_dev_id == -1 ) {
        /*System Global drop policy*/
        map_list_p = drop_info_p->prio_to_drop_map_list;
        adk_sys_cp_ovp_table_p->system_wide_policy_flag  = 1;
        dscp_mask = drop_info_p->dscp_mask;
        inv_dscp_mask = ((~dscp_mask)&(0x3F));

        index = drop_info_p->num_maps;
        while(index!=0)
        {
            index--;
            pkt_prio = map_list_p[index].pkt_priority;

            if(((pkt_prio > ADK_SYS_MAX_DSCP_VALUE) && 
                (drop_info_p->type == ADK_SYS_PRIO_TYPE_IP_DSCP)) ||
               ((pkt_prio > ADK_SYS_MAX_VLAN_PCP_VALUE) &&
                (drop_info_p->type== ADK_SYS_PRIO_TYPE_VLAN_PCP))) {
                retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
                ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                                "invalid priority %d \n",pkt_prio);
                goto adk_sys_ethdev_drop_precedence_map_return;
            }
            
            for(i= (pkt_prio & dscp_mask) ;i<= (pkt_prio | inv_dscp_mask)  ; i++) {
                
                /* TODO: validate the array index before addressing array */
                adk_sys_cp_ovp_table_p->prio_to_drop_map_g[i].drop_precedence = 
                    map_list_p[index].drop_precedence;
                if(map_list_p[index].drop_precedence == ADK_SYS_DROP_PRECEDENCE_LOW ) {
                    adk_sys_cp_ovp_table_p->prio_to_drop_map_g[i].action_drop = false;
                }else{
                    adk_sys_cp_ovp_table_p->prio_to_drop_map_g[i].action_drop = true;
                }
            }
        }

    } else {
        /* per eth-dev drop policy*/
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          " per eth-dev drop-precedence policy is not supported \n");
        goto adk_sys_ethdev_drop_precedence_map_return;
    }
    adk_sys_cp_ovp_table_p->prio_type                = drop_info_p->type;
    adk_sys_cp_ovp_table_p->default_drop_precedence  = drop_info_p->default_drop_precedence;
    adk_sys_cp_ovp_table_p->dscp_mask                = drop_info_p->dscp_mask;

    if(drop_info_p->default_drop_precedence == ADK_SYS_DROP_PRECEDENCE_LOW) {
        adk_sys_cp_ovp_table_p->default_action_drop = false;
    } else {
        adk_sys_cp_ovp_table_p->default_action_drop = true;
    }

    if(adk_sys_cp_ovp_table_p->system_wide_policy_flag  == 1) {
        /* System wide global policy run for each packet*/
        if(adk_sys_cp_ovp_table_p->prio_type == ADK_SYS_PRIO_TYPE_IP_DSCP)
        {
            /* if IP_DSCP based policy*/
            if(adk_sys_cp_ovp_table_p->dscp_mask == 0x00)
            {
                adk_sys_cp_ovp_table_p->mode = 0x01;
            } else {
                adk_sys_cp_ovp_table_p->mode = 0x02;
            }
        } else {
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                            " VLAN PCP based policies are not supported\n");
            goto adk_sys_ethdev_drop_precedence_map_return;
        }
    } 


adk_sys_ethdev_drop_precedence_map_return:
    ADK_RETURN (retValue);
}
#endif


/*!
 * =============================================================
 *
 * adk_sys_resource_register
 *
 * @brief        API to register resources to ADK. 
 *
 * @param[in]    res_list   Pointer to hold resource info list
 * @param[in]    num_res    number of resources 
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * =============================================================
 */
adk_status_t adk_sys_resource_register(adk_sys_resource_t *res_list_p, 
                                       uint8_t num_res)
{
    adk_status_t                retValue   = ADK_SUCCESS;
    uint32_t                    res_index  = 0;
    adk_sys_resource_type_t     res_type;
    adk_sys_config_info_t       *sysConfInfo_p;
    adk_sys_resource_info_t     res_info;

    res_info.reserve_p      = NULL;
    /*
     * Allocated shared memory for IPC between primary and secondary process
     *
     * Set the static global pointer to the memzone for adk_sys
     */
    retValue = adk_shm_create(ADK_SYS_SHM_MEMZONE_NAME,
                              sizeof(adk_sys_shm_config_t),
                              ADK_SHM_LOOKUP_LCORE,
                              (adk_shm_handle_t **)&adkSysShmConf_p);
    if ((retValue != ADK_SUCCESS) &&
        (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS))
    {
        ADK_PREINIT_WRITE("[%s()]: shared memory allocation failed.\n",
                          __func__);
        return (retValue);
    }

    if (ADK_STATUS_ERROR_CODE_GET(retValue) != ADK_W_SHM_EXISTS) {
        sysConfInfo_p = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, sizeof(adk_sys_config_info_t));
        if (!ADK_VALID_PTR (sysConfInfo_p))
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            ADK_PREINIT_WRITE("[%s()]failed. Memory allocation failed\n", __func__);
            goto _adk_sys_resource_register_shm_exit;
        }

        /* Set the system config handle to shared memory segment. This will be read
         * by the secondary process.
         */
        adkSysShmConf_p->sysConfHandle = (adk_config_handle_t) sysConfInfo_p;
    } else {
        sysConfInfo_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;
    }

    if (!ADK_INVALID_SYS_HANDLE(adkSysShmConf_p->sysHandle))
    {
        adk_sys_handle_t tmp;
        retValue = adk_syshandle_get(TRUE, 0, &tmp );
        RTE_SET_USED(tmp);
    }

    if(num_res == 0)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid number of resources %d\n", num_res);
        goto _adk_sys_resource_register_exit;
    }

    if(res_list_p== NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("NULL resource list \n");
        goto _adk_sys_resource_register_exit;
    }

    for(res_index = 0; res_index < num_res ; res_index++)
    {
        res_type = res_list_p[res_index].type;
        switch(res_type){
        case ADK_SYS_RSRC_TYPE_EVNT_DEV:
            retValue = adk_sys_copy_evd_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.evdev_info_p, &res_info);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_register_exit;
            }
            break;
        case ADK_SYS_RSRC_TYPE_CRYPTO_DEV:
            retValue = adk_sys_copy_crypto_dev_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.crypto_dev_info_p, &res_info);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_register_exit;
            }
            break;

        case ADK_SYS_RSRC_TYPE_WCORE:
            retValue = adk_sys_copy_wcore_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.wcore_list_info_p, &res_info);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_register_exit;
            }
            break;

        case ADK_SYS_RSRC_TYPE_TIMER_DEV:
            retValue = adk_sys_copy_timer_dev_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.timer_dev_info_p, &res_info);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_register_exit;
            }
            break;

        case ADK_SYS_RSRC_TYPE_ETH_DEV:
            retValue = adk_sys_copy_eth_dev_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.eth_dev_info_p, &res_info);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_register_exit;
            }
            break;

        case ADK_SYS_RSRC_TYPE_PKT_MEMORY:
            /* Use same memory pool for IP fragments as well */
            sysConfInfo_p->pktio_mempool = 
                            res_list_p[res_index].res_info_p.pktio_mempool;
            sysConfInfo_p->ip_frag_mempool= 
                            res_list_p[res_index].res_info_p.pktio_mempool;
            res_info.pktio_mempool = res_list_p[res_index].res_info_p.pktio_mempool;
            break;


        default:
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_FORCE_LOG_ERROR ("Invalid/un-supported resources type %d\n",
                                 res_type);
            goto _adk_sys_resource_register_exit;
        }

        retValue = adk_sys_rsrc_propagate(
                                ADK_SYS_RSRC_ADD,
                                res_type, 
                                res_info);
    }

_adk_sys_resource_register_exit:
    ADK_RETURN( retValue );

_adk_sys_resource_register_shm_exit:
    if(adk_shm_destroy(ADK_SYS_SHM_MEMZONE_NAME)!=ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("[%s()]: ADK-SYS shared memory destory failed.\n",
                            __func__);
    }
    ADK_RETURN (retValue);

}

#if 1
/*!
 * =============================================================
 *
 * adk_sys_resource_unregister
 *
 * @brief        API to unregister resources to ADK. 
 *
 * @param[in]    res_list   Pointer to hold resource info list
 * @param[in]    num_res    number of resources 
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * =============================================================
 */
adk_status_t adk_sys_resource_unregister(adk_sys_resource_t *res_list_p, 
                                       uint8_t num_res)
{
    adk_status_t                retValue   = ADK_SUCCESS;
    uint32_t                    res_index  = 0;
    adk_sys_resource_type_t     res_type;
    adk_sys_config_info_t       *sysConfInfo_p;
    adk_sys_handle_t            sysHandle;

    /* Validate the shared memory pointer*/
    if (adkSysShmConf_p == NULL)
    {
        ADK_PREINIT_WRITE("[%s()]: ADK system shared memory not initialized.\n",
                          __func__);
        return (retValue);
    }
    /* Reaching this point means shared memory for ADK-SYSTEM is created */
    sysConfInfo_p = (adk_sys_config_info_t *)adkSysShmConf_p->sysConfHandle;
    if (sysConfInfo_p == NULL)
    {
        ADK_PREINIT_WRITE("[%s()]: ADK system configuration memory not initialized.\n",
                          __func__);
        return (retValue);
    }

    if (!ADK_INVALID_SYS_HANDLE(adkSysShmConf_p->sysHandle))
    {
        retValue = adk_syshandle_get(TRUE, 0, &sysHandle );
        if(retValue!=ADK_SUCCESS) {
            ADK_PREINIT_WRITE("[%s()]: ADK system is not initialized.\n",
                          __func__);
            return (retValue);
        }
    } else {
        sysHandle = adkSysShmConf_p->sysHandle;
    }

    RTE_SET_USED(sysHandle);

    if(num_res == 0)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                        "Invalid number of resources %d\n", num_res);
        goto _adk_sys_resource_unregister_exit;
    }

    if(res_list_p== NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                        "NULL resource list \n");
        goto _adk_sys_resource_unregister_exit;
    }

    for(res_index = 0; res_index < num_res ; res_index++)
    {
        res_type = res_list_p[res_index].type;

        /*As of now only ETh-DEV's are allowed to un-register */
        switch(res_type){

        case ADK_SYS_RSRC_TYPE_ETH_DEV:

            retValue = adk_sys_rem_eth_dev_rsrc(sysConfInfo_p, 
                            res_list_p[res_index].res_info_p.eth_dev_info_p);
            if(retValue!=ADK_SUCCESS)
            {
                goto _adk_sys_resource_unregister_exit;
            }
            break;

        default:
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                         "Invalid/un-supported resources type %d\n", res_type);
            goto _adk_sys_resource_unregister_exit;
        }

    }

_adk_sys_resource_unregister_exit:
    ADK_RETURN( retValue );

}

#endif

adk_status_t
adk_sys_copy_eth_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_eth_dev_info_t   *eth_dev_p,
                      adk_sys_resource_info_t       *res_info_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    uint8_t                 num_eth_devs, num_eth_dev_rsrc, index, i,j;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_ETH_DEV;
    uint16_t                     *eth_dev_list_local_p;
    adk_sys_resource_info_t     *sys_rsrc_table_p;
    adk_sys_rsrc_eth_dev_info_t *eth_dev_local_p;
    uint16_t tx_qs, tx_queue_id;
    struct rte_eth_dev_info dev_info = {0};

    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* validate input */
    if(eth_dev_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid parameter, null eth-dev info pointer\n");
        return (retValue);
    }
    num_eth_devs = eth_dev_p->num_eth_devs;
    if((num_eth_devs<1) || (num_eth_devs > RTE_MAX_ETHPORTS))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid number of eth-devs[%d]\n",
                       num_eth_devs);
        return (retValue);
    }

    /* Check space avalibility*/

    num_eth_dev_rsrc = sysConfInfo_p->num_res[res_type];
    if(num_eth_dev_rsrc == 255)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
        ADK_PREINIT_WRITE("Reached max capacity of for res-type %d\n",
                                res_type);
        return (retValue);
    }

    /* Check if this resource is previously registred*/
    index = num_eth_dev_rsrc;
    while(index!=0) {
        index--;
        eth_dev_local_p = sys_rsrc_table_p[index].eth_dev_info_p;
        eth_dev_list_local_p = eth_dev_local_p->eth_dev_list;
        for(i=0; i<eth_dev_p->num_eth_devs; i++) {
            for(j=0; j<eth_dev_local_p->num_eth_devs; j++) {
                if(eth_dev_list_local_p[j] == eth_dev_p->eth_dev_list[i] )
                {
                    retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
                    ADK_PREINIT_WRITE("eth-dev resource already registred %d\n",eth_dev_p->eth_dev_list[j]);
                    return (retValue);
                }
            }
        }
    }
#if 0
    /* Allocate memory for eth-dev data structure */
    eth_dev_local_p = (adk_sys_rsrc_eth_dev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_eth_dev_info_t));
    if (!ADK_VALID_PTR (eth_dev_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for Ethernet-Dev failed\n");
        return(retValue);
    }

    /* Copy event-dev information */
    ADK_MEMCPY(eth_dev_local_p, eth_dev_p, sizeof(adk_sys_rsrc_eth_dev_info_t));

#else
    if(adk_sys_create_eth_dev_rsrc_nvm_copy(eth_dev_p, &eth_dev_local_p)!=ADK_SUCCESS)
    {
        return(retValue);
    }
#endif
    sys_rsrc_table_p[sysConfInfo_p->num_res[res_type]].eth_dev_info_p = eth_dev_local_p;
    sysConfInfo_p->num_res[res_type]++;

    res_info_p->eth_dev_info_p  = eth_dev_local_p;

    /*Register eth dev tx cb*/
    for(i=0; i<eth_dev_p->num_eth_devs; ++i)
    {
#ifdef ADK_DEVICE_SNR
        if(rte_bridge_eth_tx_enqueue_notif_register(eth_dev_p->eth_dev_list[i],
            adk_sys_dp_egress_stats_enqueue_cb, NULL))
        {
            return ADK_E_INTERNAL_ERROR;
        }
#ifdef ADK_COMPLETION_CALLBACK_SUPPORT 
        if(rte_bridge_eth_tx_cb_register(eth_dev_p->eth_dev_list[i],
            adk_sys_dp_egress_stats_xmit_cb, NULL))
        {
            return ADK_E_INTERNAL_ERROR;
        }
#endif
#else
        if(!rte_eth_add_tx_callback(eth_dev_p->eth_dev_list[i],
                                    0, adk_sys_dp_egress_stats_cb, NULL))
        {
            return ADK_E_INTERNAL_ERROR;
        }
#endif
        rte_eth_dev_info_get(eth_dev_p->eth_dev_list[i], &dev_info);
        tx_qs = (dev_info.max_tx_queues > ADK_DP_VPORT_NUM_TX_QUEUES) ? ADK_DP_VPORT_NUM_TX_QUEUES : dev_info.max_tx_queues;
        for (tx_queue_id = 0; tx_queue_id < tx_qs; tx_queue_id++)
        {
            if(!rte_eth_add_tx_callback(eth_dev_p->eth_dev_list[i],
                                        tx_queue_id, adk_ipsec_dp_egress_tx_cb, NULL))
            {
                ADK_PREINIT_WRITE("Error occurred while adding TX callback in Eth Dev for IPsec\n");
                return ADK_E_INTERNAL_ERROR;
            }
        }
    }
    return(retValue);
}


adk_status_t
adk_sys_rem_eth_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_eth_dev_info_t   *eth_dev_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    uint8_t                 num_eth_devs, num_eth_dev_rsrc, index, i,j,k;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_ETH_DEV;
    uint16_t                     *eth_dev_list_local_p;
    adk_sys_resource_info_t     *sys_rsrc_table_p;
    adk_sys_rsrc_eth_dev_info_t *eth_dev_local_p;

    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* validate input */
    if(eth_dev_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid parameter, null eth-dev info pointer\n");
        return (retValue);
    }
    num_eth_devs = eth_dev_p->num_eth_devs;
    if((num_eth_devs<1) || (num_eth_devs > RTE_MAX_ETHPORTS))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid number of eth-devs[%d]\n",
                       num_eth_devs);
        return (retValue);
    }

    /* Check at least one eth-dev is registred*/

    num_eth_dev_rsrc = sysConfInfo_p->num_res[res_type];
    if(num_eth_dev_rsrc == 0)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
        ADK_PREINIT_WRITE("Reached min capacity of for res-type %d\n",
                                res_type);
        return (retValue);
    }

    /* Check if this resource is previously registred*/
    index = num_eth_dev_rsrc;
    while(index!=0) {
        index--;
        eth_dev_local_p = sys_rsrc_table_p[index].eth_dev_info_p;
        eth_dev_list_local_p = eth_dev_local_p->eth_dev_list;
        for(i=0; i<eth_dev_p->num_eth_devs; i++) {
            for(j=0; j<eth_dev_local_p->num_eth_devs; j++) {
                if(eth_dev_list_local_p[j] == eth_dev_p->eth_dev_list[i] )
                {
                    /* match found and remove the eth-dev from list */
                    for(k=j;k<RTE_MAX_ETHPORTS-1;k++)
                    {
                        eth_dev_list_local_p[k] =  eth_dev_list_local_p[k+1];
                    }
                    eth_dev_local_p->num_eth_devs -=1;
                    if(eth_dev_local_p->num_eth_devs==0)
                    {
                        /* free up that eth-dev-resource */
                        ADK_NVM_FREE(eth_dev_local_p);
                        sys_rsrc_table_p[index].eth_dev_info_p = NULL;
                        eth_dev_local_p = NULL;
                    }
                    /* Break the inner for loop */
                    break;
                }
            }
            if(eth_dev_local_p == NULL)
            {
                /* Break the outer for loop */
                break;
            }
        }
    }

    /* Scan through the all the eth-dev resources and clear holes  */
    for(index=0 ; index <=num_eth_dev_rsrc; index++) {
        eth_dev_local_p = sys_rsrc_table_p[index].eth_dev_info_p;
        if(eth_dev_local_p == 0)
        {
            for(i=index; i<num_eth_dev_rsrc-1; i++) {
                sys_rsrc_table_p[i].eth_dev_info_p = sys_rsrc_table_p[i+1].eth_dev_info_p;
            }
            sysConfInfo_p->num_res[res_type]--;
        }
    }

    if(adk_sys_create_eth_dev_rsrc_nvm_copy(eth_dev_p, &eth_dev_local_p)!=ADK_SUCCESS)
    {
        /* Propagate the resource */
        adk_sys_resource_info_t rsrc_info;

        rsrc_info.eth_dev_info_p = eth_dev_local_p;
        retValue = adk_sys_rsrc_propagate( 
                    ADK_SYS_RSRC_REM,
                    ADK_SYS_RSRC_TYPE_ETH_DEV,
                    rsrc_info);
        /* Free up the temp resource */
        ADK_NVM_FREE(eth_dev_local_p);
    }

    return(retValue);
}

adk_status_t
adk_sys_copy_evd_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_evdev_info_t *evd_p, 
                      adk_sys_resource_info_t *res_info_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sys_rsrc_evp_info_t *evp_p;
    adk_sys_rsrc_evq_info_t *evq_p;
    uint8_t                 num_evq;
    uint8_t                 num_evp;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_EVNT_DEV;
    adk_sys_rsrc_evdev_info_t   *evd_local_p;
    adk_sys_resource_info_t     *sys_rsrc_table_p;


    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* validate input */
    if(evd_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid parameter, null event-dev info\n");
        return (retValue);
    }
    num_evp = evd_p->num_ev_ports;
    num_evq = evd_p->num_ev_queues;
    evp_p   = evd_p->event_port_list_p;
    evq_p   = evd_p->event_q_list_p;
    if((num_evp<1) || (num_evq<1))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid number of event-queues[%d]/ports[%d]\n",
                       num_evq, num_evp);
        return (retValue);
    }
    if((evp_p == NULL) || (evq_p==NULL))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid parameter, null event-port/queue info list\n");
        return (retValue);
    }


    /* Check space avalibility*/
    if((sysConfInfo_p->num_res[res_type]) == 255)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
        ADK_PREINIT_WRITE("Reached max capacity of for res-type %d\n",
                                res_type);
        return (retValue);
    }

#if 0
    /*Allocate memory for the object */
    /* Allocate memory for event-dev data structure */
    evd_local_p = (adk_sys_rsrc_evdev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_evdev_info_t));
    if (!ADK_VALID_PTR (evd_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-dev failed\n");
        return(retValue);
    }
    /*Allocate memory for event port list */
    evp_local_p= (adk_sys_rsrc_evp_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_evp, sizeof (adk_sys_rsrc_evp_info_t));
    if (!ADK_VALID_PTR (evp_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-port list failed\n");
        return(retValue);
    }

    /*Allocate memory for event queue list */
    evq_local_p= (adk_sys_rsrc_evq_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_evq, sizeof (adk_sys_rsrc_evq_info_t));
    if (!ADK_VALID_PTR (evq_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-queue list failed\n");
        return(retValue);
    }

    /* Copy event-dev information */
    ADK_MEMCPY(evd_local_p, evd_p, sizeof(adk_sys_rsrc_evdev_info_t));

    /* Copy event-port information */
    ADK_MEMCPY(evp_local_p, evp_p, num_evp * sizeof(adk_sys_rsrc_evp_info_t));

    /* Copy event-queue information */
    ADK_MEMCPY(evq_local_p, evq_p, num_evq * sizeof(adk_sys_rsrc_evq_info_t));

    /*update ev_p and ev_q pointers in local ev-dev info  */
    evd_local_p->event_port_list_p = evp_local_p;
    evd_local_p->event_q_list_p = evq_local_p;
#else
    if(adk_sys_create_evdev_rsrc_nvm_copy(evd_p, &evd_local_p)!=ADK_SUCCESS)
    {
        return(retValue);
    }
#endif

    sys_rsrc_table_p[sysConfInfo_p->num_res[res_type]].evdev_info_p = evd_local_p;
    sysConfInfo_p->num_res[res_type]++;

    res_info_p->evdev_info_p = evd_local_p;

    return(retValue);
}

adk_status_t
adk_sys_copy_crypto_dev_rsrc(adk_sys_config_info_t     *sysConfInfo_p,
                      adk_sys_rsrc_crypto_dev_info_t   *cdev_p,
                      adk_sys_resource_info_t          *res_info_p)
{
    adk_status_t            retValue = ADK_SUCCESS;
    uint8_t                 num_qp;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_CRYPTO_DEV;
    adk_sys_rsrc_crypto_dev_info_t  *cdev_local_p;
    adk_sys_resource_info_t          *sys_rsrc_table_p;
    adk_sys_rsrc_crypto_dev_qpair_info_t   *qp_info_p;

    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* validate input */
    if(cdev_p == NULL)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("Invalid parameter, null event-dev info\n");
        return (retValue);
    }
    /* need Q-pair related processing only for lookaside
     * device
     */
    if (cdev_p->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
    {
        num_qp    = cdev_p->num_q_pairs;
        qp_info_p = cdev_p->qp_info_p;
        if(num_qp<1)
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_PREINIT_WRITE("Invalid number of queue pairs[%d]\n",
                    num_qp);
            return (retValue);
        }
        if(qp_info_p== NULL)
        {
            retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
            ADK_PREINIT_WRITE("Invalid parameter, null queue-pair list\n");
            return (retValue);
        }
    }

    /* Check space avalibility*/
    if((sysConfInfo_p->num_res[res_type]) == 255)
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE);
        ADK_PREINIT_WRITE("Reached max capacity of for res-type %d\n",
                                res_type);
        return (retValue);
    }
#if 0
    /*Allocate memory for the object */
    /* Allocate memory for event-dev data structure */
    cdev_local_p = (adk_sys_rsrc_crypto_dev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_crypto_dev_info_t ));
    if (!ADK_VALID_PTR (cdev_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for cryptodev resource failed\n");
        return(retValue);
    }
    /*Allocate memory for event port list */
    qp_local_info_p= (adk_sys_rsrc_crypto_dev_qpair_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_qp, sizeof (adk_sys_rsrc_crypto_dev_qpair_info_t));
    if (!ADK_VALID_PTR (qp_local_info_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for queue pair list failed\n");
        return(retValue);
    }

    /* Copy crypto-dev information */
    ADK_MEMCPY(cdev_local_p, cdev_p, sizeof(adk_sys_rsrc_crypto_dev_info_t));

    /* Copy queue-pair information */
    ADK_MEMCPY(qp_local_info_p, qp_info_p, num_qp*sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));


    /*update ev_p and ev_q pointers in local ev-dev info  */
    cdev_local_p->qp_info_p= qp_local_info_p;
#else
    if(adk_sys_create_cdev_rsrc_nvm_copy(cdev_p, &cdev_local_p) !=ADK_SUCCESS)
    {
        return(retValue);
    }
#endif

    sys_rsrc_table_p[sysConfInfo_p->num_res[res_type]].crypto_dev_info_p= cdev_local_p;
    sysConfInfo_p->num_res[res_type]++;

    res_info_p->crypto_dev_info_p   = cdev_local_p;

    return(retValue);
}

adk_status_t
adk_sys_copy_wcore_rsrc(
        adk_sys_config_info_t *sysConfInfo_p,
        adk_sys_rsrc_wcore_info_t *wc_p, 
        adk_sys_resource_info_t  *res_info_p)
{
    adk_status_t                retValue = ADK_SUCCESS;
    uint32_t                    i, id;
    adk_sys_resource_info_t     *sys_rsrc_table_p;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_WCORE;

    RTE_SET_USED(res_info_p);

    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* Validate input parameters */
    if (ADK_INVALID_PTR(wc_p))
    {
        ADK_PREINIT_WRITE("[%s()]failed. Invalid pointers\n", __func__);
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        return(retValue);
    }

    if (ADK_INVALID_PTR(wc_p->worker_lcores))
    {
        ADK_PREINIT_WRITE("[%s()]failed. Invalid worker lcore "
                           "list pointer\n", __func__);
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        return(retValue);
    }

    if (wc_p->num_worker_lcores == 0)
    {
        ADK_PREINIT_WRITE("[%s()]failed. ADK fast path requires "
                           "atleast one lcore\n",__func__);
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        return(retValue);
    }

    /* Update ADK slave configuration */
    sysConfInfo_p->lcoreInfo.numLcores = wc_p->num_worker_lcores ;
    for (i = 0; i < wc_p->num_worker_lcores; i++)
    {
        id = wc_p->worker_lcores[i];
        sysConfInfo_p->lcoreInfo.lcore[i].id = id;
        sysConfInfo_p->lcoreInfo.lcore[i].index = ADK_LCORE_INDEX_LOOKUP(id);
        sysConfInfo_p->lcoreInfo.lcore[i].socket = rte_lcore_to_socket_id(id);
    }

    ADK_RETURN( retValue );

}    /* -----  end of function adk_sys_copy_wcore_rsrc ----- */


adk_status_t
adk_sys_copy_timer_dev_rsrc(
        adk_sys_config_info_t *sysConfInfo_p,
        adk_sys_rsrc_timer_dev_info_t *timer_dev_info_p,
        adk_sys_resource_info_t     *res_info_p)
{
    adk_status_t                retValue = ADK_SUCCESS;
    adk_sys_resource_info_t     *sys_rsrc_table_p;
    adk_sys_rsrc_timer_dev_info_t    *timer_dev_info_local_p;
    adk_sys_resource_type_t     res_type = ADK_SYS_RSRC_TYPE_TIMER_DEV;

    /* Get the evdev-list */
    sys_rsrc_table_p = sysConfInfo_p->res_list[res_type];

    /* Validate input parameters */
    if (ADK_INVALID_PTR(timer_dev_info_p))
    {
        ADK_PREINIT_WRITE("[%s()]failed. Invalid pointers\n", __func__);
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        return(retValue);
    }

    if (timer_dev_info_p->type != ADK_SYS_RSRC_TIMER_DEV_TYPE_EVNT_TIMER_BD)
    {
        ADK_PREINIT_WRITE("[%s()] Invalid Timer Dev-TYPE."
                          "Must be TIMER-BD-TYPE only.\n", __func__);

        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER,
                                             ADK_SEV_LEVEL_ERROR);
        return(retValue);
    }

    /* Allocate memory to store timer-dev information */
    timer_dev_info_local_p = (adk_sys_rsrc_timer_dev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_timer_dev_info_t));
    if (!ADK_VALID_PTR (timer_dev_info_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for timer-dev resource failed\n");
        return(retValue);
    }

    /* Copy timer-dev information */
    ADK_MEMCPY(timer_dev_info_local_p, timer_dev_info_p, sizeof(adk_sys_rsrc_timer_dev_info_t));

    sys_rsrc_table_p[sysConfInfo_p->num_res[res_type]].timer_dev_info_p = timer_dev_info_local_p;
    sysConfInfo_p->num_res[res_type]++;

    res_info_p->timer_dev_info_p = timer_dev_info_local_p;

    ADK_RETURN( retValue );

}    /* -----  end of function adk_sys_copy_timer_dev_rsrc----- */

static adk_status_t
adk_sys_rsrc_propagate( 
                    adk_sys_rsrc_oper_t oper,
                    adk_sys_resource_type_t        type,
                    adk_sys_resource_info_t rsrc)
{

    adk_status_t                retValue = ADK_SUCCESS;
    adk_sys_handle_t            sysHandle = adkSysShmConf_p->sysHandle;
    adk_sys_info_t              *sysInfo_p = (adk_sys_info_t *)sysHandle;
    adk_module_info_t           *moduleInfo_p;
    uint32_t                    moduleIndex;
#if 0
    adk_sys_resource_info_t     *rsrc_list;
    uint8_t                     num_rsrc;
#endif
    RTE_SET_USED(oper);

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        ADK_FORCE_LOG_INFO("Invalid system handle %"ADKPRIh"\n", sysHandle);
        ADK_RETURN( retValue );
    }
    /* Check the list of modules initialized */

    moduleInfo_p = (adk_module_info_t *) sysInfo_p->utils.moduleMgrMasterHandle;

#if 0

    switch(type)
    {
        case ADK_SYS_RSRC_TYPE_EVNT_DEV:
            retValue = adk_sys_config_evdev_info_get ( &rsrc_list, &num_rsrc);
            if(retValue !=ADK_SUCCESS){
                ADK_FORCE_LOG_INFO("internal error; \
                                unable to retrive resource info from local database\n");
                ADK_RETURN( retValue );
            }
            break;
        case ADK_SYS_RSRC_TYPE_CRYPTO_DEV:
            retValue = adk_sys_config_crypto_dev_info_get ( &rsrc_list, &num_rsrc);
            if(retValue !=ADK_SUCCESS){
                ADK_FORCE_LOG_INFO("internal error; \
                                unable to retrive resource info from local database\n");
                ADK_RETURN( retValue );
            }
            break;
        case ADK_SYS_RSRC_TYPE_ETH_DEV:
            retValue = adk_sys_config_eth_dev_info_get ( &rsrc_list, &num_rsrc);
            if(retValue !=ADK_SUCCESS){
                ADK_FORCE_LOG_INFO("internal error; \
                                unable to retrive resource info from local database\n");
                ADK_RETURN( retValue );
            }
            break;
        case ADK_SYS_RSRC_TYPE_TIMER_DEV:
            retValue = adk_sys_config_timer_dev_info_get ( &rsrc_list, &num_rsrc);
            if(retValue !=ADK_SUCCESS){
                ADK_FORCE_LOG_INFO("internal error; \
                                unable to retrive resource info from local database\n");
                ADK_RETURN( retValue );
            }
            break;
        default:
            retValue = ADK_E_UNSUPPORTED;
            ADK_FORCE_LOG_INFO(" un-supported resource type for propagation \n");
            ADK_RETURN( retValue );
        break;
    }
#endif

    switch(type)
    {
        case ADK_SYS_RSRC_TYPE_EVNT_DEV:
        case ADK_SYS_RSRC_TYPE_CRYPTO_DEV:
        case ADK_SYS_RSRC_TYPE_ETH_DEV:
        case ADK_SYS_RSRC_TYPE_TIMER_DEV:
        /* scan through all the modules  */
        for (moduleIndex = 0; moduleIndex < ADK_MAX_SUPPORTED_MODULES;
            moduleIndex++, moduleInfo_p++)
        {
            /* if a modules is started then call the modules resource-update/notify
            * service function*/
            if((moduleInfo_p->inUse == TRUE) && (moduleInfo_p->isStarted == TRUE))
            {

                ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                            "module %d handle %"ADKPRIh" %d %"ADKPRIh"\n",
                            moduleInfo_p->moduleId,
                            (adk_module_handle_t)moduleInfo_p,
                            type, rsrc.pktio_mempool);
                if(array_notify_fn_ptrs[moduleIndex]!=NULL)
                retValue = array_notify_fn_ptrs[moduleIndex](
                                        (adk_module_handle_t)moduleInfo_p, 
                                        oper,
                                        type, 
                                        rsrc);
            }
        }
        break;

        default:
            retValue = ADK_E_UNSUPPORTED;
            ADK_FORCE_LOG_INFO(" un-supported resource type for propagation \n");
            ADK_RETURN( retValue );
        break;
    }

    ADK_RETURN(retValue);
}    /* -----  end of function adk_sys_rsrc_propagate ----- */

static adk_status_t
adk_sys_load_modules_notify_fns(adk_sys_handle_t sysHandle)
{

    adk_status_t                retValue = ADK_SUCCESS;
    static int                  state = 0;

    /* ISO C 89 and 99 does not allow casting of a void* to a function pointer.
       dlsym() is an older function that returns void*.
       The below line and it's usage further down forces GCC into allowing it anyway. */
    typedef void * __attribute__ ((__may_alias__)) pvoid_may_alias_t;
    void * lib;

    if(state==0){
#ifdef ADK_MODULE_ETHERNET_ENABLED
        lib = dlopen("libadk-ethernet.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "ethernet-modules library load failed %s \n", dlerror()); 
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_ETHERNET_MODULE_ID]) =
             dlsym(lib, "adk_ethernet_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_ETHERNET_MODULE_ID]) {
            ADK_LOG_WARNING(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_ethernet_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_ETH_OAM_ENABLED
        lib = dlopen("libadk-ethoam.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "ethernet-oam-module's library load failed %s \n", dlerror()); 
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_ETH_OAM_MODULE_ID]) =
             dlsym(lib, "adk_eth_oam_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_ETH_OAM_MODULE_ID]) {
            ADK_LOG_WARNING(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_eth_oam_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_IP_ENABLED
        lib = dlopen("libadk-ip.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " ip-modules library load failed %s \n", dlerror());
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_IP_MODULE_ID]) =
             dlsym(lib, "adk_ip_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_IP_MODULE_ID]) {
            ADK_LOG_WARNING(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_ip_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_IPV6_ENABLED
        lib = dlopen("libadk-ipv6.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " ipv6-modules library load failed %s \n", dlerror());
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_IPV6_MODULE_ID]) =
             dlsym(lib, "adk_ipv6_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_IPV6_MODULE_ID]) {
            ADK_LOG_WARNING (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_ipv6_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_IPSEC_ENABLED
        lib = dlopen("libadk-ipsec.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " ipsec-modules library load failed %s \n", dlerror());
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_IPSEC_MODULE_ID]) =
             dlsym(lib, "adk_ipsec_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_IPSEC_MODULE_ID]) {
            ADK_LOG_WARNING (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_ipsec_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_BFD_ENABLED
        lib = dlopen("libadk-bfd.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " bfd-modules library load failed %s \n", dlerror());
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_BFD_MODULE_ID]) =
             dlsym(lib, "adk_bfd_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_BFD_MODULE_ID]) {
            ADK_LOG_WARNING (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_bfd_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
#ifdef ADK_MODULE_TWAMP_ENABLED
        lib = dlopen("libadk-twamp.so", RTLD_NOW);
        if (!lib) {
            retValue = ADK_E_INTERNAL_ERROR;
            ADK_LOG_ERROR (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " twamp-modules library load failed %s \n", dlerror());
            ADK_RETURN(retValue);
        }

       *(pvoid_may_alias_t *) (&array_notify_fn_ptrs[ADK_TWAMP_MODULE_ID]) =
             dlsym(lib, "adk_twamp_rsrc_notify_fn_cbfn");
        if (!array_notify_fn_ptrs[ADK_TWAMP_MODULE_ID]) {
            ADK_LOG_WARNING (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                           " adk_twamp_rsrc_notify_fn_cbfn load failed %s \n", dlerror());
        }    
#endif
        state=0;
    } else {
        ADK_LOG_INFO (sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "resource notify functions are already loaded\n");
    }
    ADK_RETURN(retValue);
}    /* -----  end of function adk_sys_load_modules_notify_fns ----- */

adk_status_t adk_sif_rsrc_notify_fn_cbfn
            (adk_module_handle_t mod_handle, 
                                         adk_sys_rsrc_oper_t oper,
                                         adk_sys_resource_type_t type,
                                         adk_sys_resource_info_t rsrc)
{
    adk_status_t                retValue = ADK_SUCCESS;
    adk_sys_handle_t            sysHandle;

    RTE_SET_USED(mod_handle);
    RTE_SET_USED(type);
    RTE_SET_USED(rsrc);

    sysHandle = adkSysShmConf_p->sysHandle;
    if(oper== ADK_SYS_RSRC_ADD) 
    {
        ADK_LOG_INFO(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
                     "resource type %d %p\n", type, rsrc.pktio_mempool);
        retValue = adk_pal_rsrc_notify_fn_cbfn( mod_handle, oper, type, rsrc);
    } else {
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET (sysHandle),
        "Only ADD supported, other operation are not supported yet\n"); 
    }

    ADK_RETURN(retValue);
}

adk_status_t
adk_sys_create_eth_dev_rsrc_nvm_copy(
     adk_sys_rsrc_eth_dev_info_t   *eth_dev_p,
     adk_sys_rsrc_eth_dev_info_t   **dst_eth_dev_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_sys_rsrc_eth_dev_info_t     *eth_dev_local_p;
    uint8_t                         num_eth_devs;

    num_eth_devs = eth_dev_p->num_eth_devs;
    /* Allocate memory for eth-dev data structure */
    eth_dev_local_p = (adk_sys_rsrc_eth_dev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_eth_dev_info_t));
    if (!ADK_VALID_PTR (eth_dev_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for Ethernet-Dev failed\n");
        return(retValue);
    }

    /* Copy event-dev information */
    ADK_MEMCPY(eth_dev_local_p, eth_dev_p, sizeof(adk_sys_rsrc_eth_dev_info_t));

    /* Return back the newly created memory object */
    *dst_eth_dev_p = eth_dev_local_p;

    return(retValue);
}


adk_status_t
adk_sys_create_cdev_rsrc_nvm_copy(
     adk_sys_rsrc_crypto_dev_info_t   *cdev_p,
     adk_sys_rsrc_crypto_dev_info_t   **dst_cdev_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_sys_rsrc_crypto_dev_info_t  *cdev_local_p;
    adk_sys_rsrc_crypto_dev_qpair_info_t *qp_local_info_p = NULL;
    adk_sys_rsrc_crypto_dev_qpair_info_t   *qp_info_p = NULL;
    
    uint8_t                         num_qp;

    num_qp      = cdev_p->num_q_pairs;
    qp_info_p   = cdev_p->qp_info_p;

    /* Allocate memory for crypto-dev data structure */
    cdev_local_p = (adk_sys_rsrc_crypto_dev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_crypto_dev_info_t ));
    if (!ADK_VALID_PTR (cdev_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for cryptodev resource failed\n");
        return(retValue);
    }
    /* need Q-pair related processing only for lookaside
     * device
     */
    if (cdev_p->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
    {
        /*Allocate memory for Q-pair list */
        qp_local_info_p= (adk_sys_rsrc_crypto_dev_qpair_info_t *)
            ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_qp, sizeof (adk_sys_rsrc_crypto_dev_qpair_info_t));
        if (!ADK_VALID_PTR (qp_local_info_p))
        {
            retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                    ADK_SEV_LEVEL_FATAL);
            ADK_PREINIT_WRITE("Memory allocation for queue pair list failed\n");
            ADK_NVM_FREE(cdev_local_p);
            return(retValue);
        }
    }

    /* Copy crypto-dev information */
    ADK_MEMCPY(cdev_local_p, cdev_p, sizeof(adk_sys_rsrc_crypto_dev_info_t));

    if (cdev_p->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
    {
        /* Copy queue-pair information */
        ADK_MEMCPY(qp_local_info_p, qp_info_p, num_qp*sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));

        /*update ev_p and ev_q pointers in local ev-dev info  */
        cdev_local_p->qp_info_p= qp_local_info_p;
    }

    /* Return back the newly created memory object */
    *dst_cdev_p = cdev_local_p;

    return(retValue);
}


adk_status_t
adk_sys_create_evdev_rsrc_nvm_copy(
     adk_sys_rsrc_evdev_info_t   *evd_p,
     adk_sys_rsrc_evdev_info_t   **dst_evd_p)
{
    adk_status_t                    retValue = ADK_SUCCESS;
    adk_sys_rsrc_evdev_info_t       *evd_local_p;
    adk_sys_rsrc_evp_info_t         *evp_local_p;
    adk_sys_rsrc_evq_info_t         *evq_local_p;
    uint8_t                         num_evp, num_evq;
    uint8_t                         *wc_index_arr_p;
    uint8_t                         index = 0;
    uint8_t                         lcore_id = 0;
    adk_sys_rsrc_evp_info_t *evp_p;
    adk_sys_rsrc_evq_info_t *evq_p;

    num_evp = evd_p->num_ev_ports;
    num_evq = evd_p->num_ev_queues;
    evp_p   = evd_p->event_port_list_p;
    evq_p   = evd_p->event_q_list_p;

    /*Allocate memory for the object */
    /* Allocate memory for event-dev data structure */
    evd_local_p = (adk_sys_rsrc_evdev_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, 1, sizeof (adk_sys_rsrc_evdev_info_t));
    if (!ADK_VALID_PTR (evd_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-dev failed\n");
        return(retValue);
    }
    /*Allocate memory for event port list */
    evp_local_p= (adk_sys_rsrc_evp_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_evp, sizeof (adk_sys_rsrc_evp_info_t));
    if (!ADK_VALID_PTR (evp_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-port list failed\n");
        ADK_NVM_FREE(evd_local_p);
        return(retValue);
    }

    /*Allocate memory for storing worker core indexs 
     */
    wc_index_arr_p = (uint8_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, RTE_MAX_LCORE, sizeof (uint8_t));
    if (!ADK_VALID_PTR (wc_index_arr_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for worker-core index is failed\n");
        ADK_NVM_FREE(evd_local_p);
        ADK_NVM_FREE(evp_local_p);
        return(retValue);
    }


    /*Allocate memory for event queue list */
    evq_local_p= (adk_sys_rsrc_evq_info_t *) 
        ADK_NVM_CALLOC (ADK_SYS_MODULE_ID, 0, num_evq, sizeof (adk_sys_rsrc_evq_info_t));
    if (!ADK_VALID_PTR (evq_local_p))
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_MEM_UNAVAILABLE,
                                            ADK_SEV_LEVEL_FATAL);
        ADK_PREINIT_WRITE("Memory allocation for event-queue list failed\n");
        ADK_NVM_FREE(evd_local_p);
        ADK_NVM_FREE(evp_local_p);
        ADK_NVM_FREE(wc_index_arr_p);
        return(retValue);
    }

    /* Copy event-dev information */
    ADK_MEMCPY(evd_local_p, evd_p, sizeof(adk_sys_rsrc_evdev_info_t));

    /* Copy event-port information */
    ADK_MEMCPY(evp_local_p, evp_p, num_evp * sizeof(adk_sys_rsrc_evp_info_t));

    /* Get the worker core index and stor it in an array */
    for (index = 0; index < num_evp; index++)
    {
        lcore_id = evp_local_p[index].lcore_id;
        if (lcore_id >= RTE_MAX_LCORE)
        {
            retValue = ADK_E_INVALID_PARAMETER;
            ADK_NVM_FREE(evd_local_p);
            ADK_NVM_FREE(evp_local_p);
            ADK_NVM_FREE(wc_index_arr_p);
            ADK_NVM_FREE(evq_local_p);
            return ( retValue );
        }
        wc_index_arr_p[lcore_id] = ADK_LCORE_INDEX_LOOKUP(evp_local_p[index].lcore_id);
    }

    /* Copy event-queue information */
    ADK_MEMCPY(evq_local_p, evq_p, num_evq * sizeof(adk_sys_rsrc_evq_info_t));

    /*update ev_p and ev_q pointers in local ev-dev info  */
    evd_local_p->event_port_list_p = evp_local_p;
    evd_local_p->event_q_list_p = evq_local_p;
    evd_local_p->reserved  = (uint64_t) wc_index_arr_p;

    /* Return back the newly created memory object */
    *dst_evd_p = evd_local_p;

    return(retValue);
}

adk_status_t adk_sys_pcap_eth_dev_config(adk_sys_handle_t sysHandle,
                                         uint16_t ethDevId,
                                         bool enable)
{
    adk_status_t retValue = ADK_SUCCESS;

    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        goto _adk_sys_pcap_eth_dev_config_exit;
    }

    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *) sysHandle;

    if (enable)
    {
        sysInfo_p->pcap_ethDevId = ethDevId;
        retValue = adk_pal_eth_dev_subscribe(sysInfo_p->pcap_ethDevId,
                                             ADK_PAL_PKT_TYPE_ALL,
                                             ADK_SIF_PCAP_DEMUX_FN_ID);
        if (retValue != ADK_SUCCESS)
        {
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "Failed to register SYS pcap demux for"
                          " eth-dev ID %d with PAL\n",
                          sysInfo_p->pcap_ethDevId);
            goto _adk_sys_pcap_eth_dev_config_exit;
        }
    }
    else
    {
        retValue = adk_pal_eth_dev_unsubscribe(sysInfo_p->pcap_ethDevId,
                                               ADK_PAL_PKT_TYPE_ALL);
        if (retValue != ADK_SUCCESS)
        {
            ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                          "Failed to unregister SYS pcap demux for"
                          " eth-dev ID %d\n", sysInfo_p->pcap_ethDevId);
            goto _adk_sys_pcap_eth_dev_config_exit;
        }
        sysInfo_p->pcap_ethDevId = (uint16_t) -1;
    }

_adk_sys_pcap_eth_dev_config_exit:

    return retValue;
}

static adk_status_t
adk_sys_egress_stats_init(adk_sys_handle_t sys_handle)
{
    adk_status_t ret;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;

    /* Create User defined resources */
    sysInfo_p->egress_stats_rm_hdl =
        adk_rm_resource_init(ADK_SYS_MODULE_ID, 0, ADK_SYS_EGRESS_STATS_ID_MAX);
    if(sysInfo_p->egress_stats_rm_hdl == NULL)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
                "Failed to Allocate RM for Sys egress stats DB\n");
        return ADK_E_INTERNAL_ERROR;
    }

    /*The ID 0 is reserved for global errors */
    adk_rm_resource_reserve_fixed(sysInfo_p->egress_stats_rm_hdl,
                                  0);

    /* Create memzone for the sys egress stats */
    ret = adk_shm_create(ADK_SYS_EGRESS_STATS_SHM_MEMZONE_NAME,
             (sizeof(adk_sys_dp_egress_stats_t) * ADK_SYS_EGRESS_STATS_ID_MAX),
             ADK_SHM_FIRST_WORKER_IN_USE_LCORE,
             (adk_shm_handle_t **)&(sysInfo_p->egress_stats_p));
    if (ret != ADK_SUCCESS)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
                "Failed to create memzone for Sys egress stats DB\n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    return ADK_SUCCESS;
}

static adk_status_t
adk_sys_egress_stats_shutdown(adk_sys_handle_t sys_handle)
{
    adk_status_t ret;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;

    adk_rm_resource_destroy(sysInfo_p->egress_stats_rm_hdl);

    ret = adk_shm_destroy(ADK_SYS_EGRESS_STATS_SHM_MEMZONE_NAME);
    if (ret != ADK_SUCCESS)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
                "Failed to free memzone for Sys egress stats DB\n");
        return ADK_E_INTERNAL_ERROR;
    }

    return ADK_SUCCESS;
}

/*!
 * ============================================================================
 * adk_sys_egress_stats_id_alloc
 *
 * @brief  Allocates one stats id to be used for egress statistics by Tx
 * callback adk_sys_dp_egress_stats_cb
 *
 * ============================================================================
 */
adk_status_t
adk_sys_egress_stats_id_alloc(
    adk_sys_handle_t sys_handle,
    uint32_t *stats_id_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;
    size_t id;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    id = adk_rm_resource_reserve(sysInfo_p->egress_stats_rm_hdl);
    if(id == ADK_RM_RSRC_INVALID)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
                "Failed to Allocate Id for Sys egress stats DB\n");
        ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
        return ADK_E_RESOURCE_UNAVAILABLE;
    }

    sysInfo_p->egress_stats_p[id].version++;
    /*Reset the counters*/
    sysInfo_p->egress_stats_p[id].pkt_drops = 0;
    sysInfo_p->egress_stats_p[id].byte_drops = 0;
    sysInfo_p->egress_stats_p[id].pkt_xmits = 0;
    sysInfo_p->egress_stats_p[id].byte_xmits = 0;
    sysInfo_p->egress_stats_p[id].timestamp = 0;
    *stats_id_p = (sysInfo_p->egress_stats_p[id].version
                   << ADK_SYS_EGRESS_STATS_ID_LEN)
                  | (id & ADK_SYS_EGRESS_STATS_ID_MASK);

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
    return ADK_SUCCESS;

_lock_release_failed:
_lock_acquire_failed:

    ADK_RETURN(retValue);
}

/*!
 * ============================================================================
 * adk_sys_egress_stats_id_free
 *
 * @brief  Free one stats id that was allocated using
 * adk_sys_egress_stats_id_alloc
 *
 * ============================================================================
 */
adk_status_t
adk_sys_egress_stats_id_free(
    adk_sys_handle_t sys_handle,
    uint32_t stats_id)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;
    uint16_t id = stats_id & ADK_SYS_EGRESS_STATS_ID_MASK;
    uint8_t version = (stats_id >> ADK_SYS_EGRESS_STATS_ID_LEN)
                      & ADK_SYS_EGRESS_STATS_VERSION_MASK;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    if(!adk_rm_resource_isset(sysInfo_p->egress_stats_rm_hdl, id))
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
            "Invalid stats id, Failed to Free Id for Sys egress stats DB\n");
        ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
        return ADK_E_INVALID_PARAMETER;
    }

    if((sysInfo_p->egress_stats_p[id].version &
        ADK_SYS_EGRESS_STATS_VERSION_MASK) != version)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
            "Invalid stats id, version mismatch\n");
        ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
        return ADK_E_INVALID_PARAMETER;
    }

    sysInfo_p->egress_stats_p[id].version++;
    adk_rcu_synchronize();
    adk_rm_resource_free(sysInfo_p->egress_stats_rm_hdl, id);

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
    return ADK_SUCCESS;

_lock_release_failed:
_lock_acquire_failed:

    ADK_RETURN(retValue);
}

/*!
 * ============================================================================
 * adk_sys_egress_stats_get
 *
 * @brief  Get the egress statistics updated by Tx callback
 * adk_sys_dp_egress_stats_cb
 *
 * ============================================================================
 */
adk_status_t
adk_sys_egress_stats_get(
    adk_sys_handle_t sys_handle,
    uint32_t stats_id,
    adk_sys_egress_stats_t* stats_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;
    uint16_t id = stats_id & ADK_SYS_EGRESS_STATS_ID_MASK;
    uint8_t version = (stats_id >> ADK_SYS_EGRESS_STATS_ID_LEN)
                      & ADK_SYS_EGRESS_STATS_VERSION_MASK;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    if(!adk_rm_resource_isset(sysInfo_p->egress_stats_rm_hdl, id))
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
            "Invalid stats id, Failed to get sys egress stats\n");
        ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
        return ADK_E_INVALID_PARAMETER;
    }

    if((sysInfo_p->egress_stats_p[id].version &
        ADK_SYS_EGRESS_STATS_VERSION_MASK) != version)
    {
        ADK_LOG_ERROR (sys_handle, ADK_SYS_MODULE_HANDLE_GET (sys_handle),
            "Invalid stats id, version mismatch\n");
        ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
        return ADK_E_INVALID_PARAMETER;
    }

    /*Read the counters*/
    stats_p->pkt_drops = (sysInfo_p->egress_stats_p[id].pkt_drops + sysInfo_p->egress_stats_p[id].pkt_enqueue_drops);
    stats_p->byte_drops = (sysInfo_p->egress_stats_p[id].byte_drops + sysInfo_p->egress_stats_p[id].byte_enqueue_drops);
    stats_p->pkt_xmits = sysInfo_p->egress_stats_p[id].pkt_xmits;
    stats_p->byte_xmits = sysInfo_p->egress_stats_p[id].byte_xmits;
    stats_p->timestamp = sysInfo_p->egress_stats_p[id].timestamp;

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);
    return ADK_SUCCESS;

_lock_release_failed:
_lock_acquire_failed:

    ADK_RETURN(retValue);
}

/*!
 * ============================================================================
 * adk_sys_ntp_time_resync
 *
 * @brief  This API does the re-synchronization of NTP Time.
 *
 * ============================================================================
 */
adk_status_t adk_sys_ntp_time_resync(adk_sys_handle_t sys_handle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_info_t *sysInfo_p = (adk_sys_info_t *)sys_handle;
    struct timeval timeVal = {0};
    adk_sif_dp_shm_info_t   *shmInfo;
    adk_sif_base_ntp_info_t *newBaseTime;
    int64_t oldBaseTime;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    /*
     * Get the number of seconds and microseconds since the Epoch from linux
     */
    if(gettimeofday(&timeVal, NULL)) {
        ADK_LOG_ERROR (sysInfo_p,
                       ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                       "Retrieval of time since the Epoch failed \n");
    }

    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);

    newBaseTime = ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, sizeof(adk_sif_base_ntp_info_t));
    if (!ADK_VALID_PTR (newBaseTime))
    {
        retValue = ADK_E_MEM_UNAVAILABLE;
        ADK_LOG_ERROR (sysInfo_p,
                       ADK_SYS_MODULE_HANDLE_GET (sysInfo_p),
                       "Memory allocation failed \n");
        goto _adk_sys_ntp_time_resync;
    }

    /*
     * Seconds since 1st Jan 1900 in NTP format. This will overflow in the
     * year 2036.
     */
    newBaseTime->baseNtpTimeHi = timeVal.tv_sec + ADK_SIF_NTP_BASETIME;

    /*
     * Seconds fraction since 1st Jan 1900 in NTP format. The granularity is at
     * the order 200 pico seconds. 1 sec = 200 picoseconds * 2^32 - 1
     */
    newBaseTime->baseNtpTimeLo =
        ((uint32_t)((timeVal.tv_usec * ADK_SIF_NTP_SCALE_FRAC)/1000000));

    newBaseTime->epochCpuTicks = rte_get_tsc_cycles();

    /* Get the old base time */
    oldBaseTime = rte_atomic64_read(&shmInfo->baseTime);

    /* Atomically update the base time */
    rte_atomic64_set(&shmInfo->baseTime, (int64_t) newBaseTime);
    
    /* Free the old pointer after RCU sync */
    adk_rcu_synchronize();

    ADK_NVM_FREE((adk_sif_base_ntp_info_t *)(oldBaseTime));

_adk_sys_ntp_time_resync:
    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);

_lock_release_failed:
_lock_acquire_failed:

    ADK_RETURN(retValue);
}

/*!
 * =============================================================
 * adk_sys_flow_cache_update_version
 * @param[in]   sysHandle       ADK system handle returned during
 *                              adk_sys_init call.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 * @retval ADK_E_SYS_NOT_INITIALIZED system not initialized.
 *
 * @brief
 *
 * This function updates the flow cache version.
 *
 * =============================================================
 */
adk_status_t
adk_sys_flow_cache_update_version(
        adk_sys_handle_t sys_handle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sif_dp_shm_info_t   *shmInfo;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);

    /*Increment the flow cache version*/
    shmInfo->fc_ver++;

    /* RCU sync */
    adk_rcu_synchronize();

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    return(retValue);
}

/*!
 * =============================================================
 * adk_sys_flow_cache_stats_get
 * @param[in]   sysHandle       ADK system handle returned during
 *                              adk_sys_init call.
 *
 * @param[out]  stats_p         Pointer to an array of flow cache stats.
 * @param[out]  num_worker_core Number of worker cores.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 * @retval ADK_E_SYS_NOT_INITIALIZED system not initialized.
 *
 * @brief
 *
 * This function returns the ADK flow stats per core.
 *
 * =============================================================
 */
adk_status_t
adk_sys_flow_cache_stats_get (
        adk_sys_handle_t sys_handle,
        adk_sys_flow_cache_stats_t **stats_p,
        uint8_t *num_worker_core)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sif_dp_shm_info_t   *shmInfo;
    adk_sys_config_worker_lcore_info_t *worker_core_info;
    adk_sys_flow_cache_stats_t *cache_stats_p = *stats_p;
    uint8_t  i = 0;

    if ((NULL == stats_p) || (NULL == num_worker_core))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_PREINIT_WRITE("%s failed: invalid input parameter\n", __func__);
        return retValue;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        return retValue;
    }

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    retValue = adk_sys_config_worker_lcore_info_get(&worker_core_info);

    if (ADK_SUCCESS != retValue) {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);

    for (i = 0 ; i < worker_core_info->numLcores ; i++) {
        cache_stats_p->hit              = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.hit;
        cache_stats_p->cleared          = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.cleared;
        cache_stats_p->miss             = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.miss;
        cache_stats_p->alloc            = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.alloc;
        cache_stats_p->freed            = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.freed;
        cache_stats_p->flows_active     = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.flows_active;
        cache_stats_p->flows_stale      = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].stats.flows_stale;
        cache_stats_p++;
    }

    *num_worker_core = worker_core_info->numLcores;

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    return(retValue);
}

/*!
 * =============================================================
 * adk_sys_flow_cache_entries_print
 * @param[in]   sysHandle       ADK system handle returned during
 *                              adk_sys_init call.
 *
 * @retval ADK_SUCCESS Successful completion
 *
 * @retval ADK_E_INVALID_PARAM invalid system handle passed.
 * @retval ADK_E_SYS_NOT_INITIALIZED system not initialized.
 *
 * @brief
 *
 * This function prints the ADK flows per core.
 *
 * =============================================================
 */
adk_status_t
adk_sys_flow_cache_entries_print(
        adk_sys_handle_t sys_handle)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sif_dp_shm_info_t   *shmInfo;
    adk_sys_config_worker_lcore_info_t *worker_core_info;
    adk_sys_dp_uli_flow_cache_entry_t *flow_cache_entry_tbl;
    adk_sys_dp_uli_flow_cache_entry_stats_t *flow_cache_entry_stats;
    uint8_t  i = 0;
    uint32_t  j = 0;

    if (ADK_INVALID_SYS_HANDLE(sys_handle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sys_handle);
        return retValue;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        return retValue;
    }

    retValue = adk_sys_config_worker_lcore_info_get(&worker_core_info);

    if (ADK_SUCCESS != retValue) {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sys_handle, retValue);

    /*Sync data*/
    adk_rcu_synchronize();

    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);
 
    if (NULL != shmInfo) {

        for (i = 0 ; i < worker_core_info->numLcores ; i++) {
            printf("\n------------Core Index [%u]--------------", worker_core_info->lcore[i].index); 
            flow_cache_entry_tbl    = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].tbl;
            flow_cache_entry_stats  = shmInfo->sys_dp_uli_flow_cache_tbl[worker_core_info->lcore[i].index].fc_entry_stats;
            for ( j = 0 ; j < ADK_SYS_ULI_FLOW_CACHE_ENTRIES ; j++) {
                if (flow_cache_entry_tbl[j].isActive) {
                    if (false == flow_cache_entry_tbl[j].isInnerIPv6) {
                        /*IPv4 flow cache entry*/
                        printf("\n------------IPv4 entry--------------"); 
                        printf("\n[SIP:0x%x,", 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v4Addr.srcAddr)); 
                        printf(" DIP:0x%x,", 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v4Addr.dstAddr)); 
                    }else {
                        /*IPv6 flow cache entry*/
                        printf("\n------------IPv6 entry--------------");
                        printf("\n[SIP:0x%x:0x%x:0x%x:0x%x,", 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.srcAddr[3]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.srcAddr[2]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.srcAddr[1]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.srcAddr[0])); 
                        printf(" DIP:0x%x:0x%x:0x%x:0x%x,", 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.dstAddr[3]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.dstAddr[2]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.dstAddr[1]), 
                                adk_ntohl(flow_cache_entry_tbl[j].ipAddr.v6Addr.dstAddr[0])); 
                    }
                    printf(" Proto:%u, SP:%u, DP:%u,", 
                            flow_cache_entry_tbl[j].nextHdr,
                            adk_ntohs(flow_cache_entry_tbl[j].srcPort),
                            adk_ntohs(flow_cache_entry_tbl[j].dstPort)); 
                    printf(" dscp:%u]",  flow_cache_entry_tbl[j].dscp); 
                    if (ADK_SYS_FLOW_CACHE_COUNTERS_ENABLED) {
                        printf("\t[Alloc:%u, Hits:%lu]",
                                flow_cache_entry_stats[j].alloc,
                                flow_cache_entry_stats[j].hit); 
                    }
                    printf("\n------------------------------------");
                }
            }
        } 
    }

    ADK_RELEASE_SYS_LOCK(sys_handle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    return(retValue);
}

/*!
 * =============================================================
 * Configure SIF flow cache counters
 * @param[in]   sysHandle   ADK system handle returned during
 *                          adk_sys_init call.
 * @param[in]   enable      boolean flag to enable/disable counters 
 *                           
 * @retval ADK_SUCCESS           : Successful completion
 *
 * @retval ADK_E_INVALID_PARAM Invalid system handle or parameter
 *                          passed.
 *
 * @brief
 * API to enable or disable flow cache counters. 
 * Default behavior: Disabled.
 * =============================================================
 */
adk_status_t adk_sys_configure_flow_cache_counters(
        adk_sys_handle_t sysHandle,
        bool enable)
{
    adk_status_t            retValue = ADK_SUCCESS;
    adk_sif_dp_shm_info_t   *shmInfo;
    if (ADK_INVALID_SYS_HANDLE(sysHandle))
    {
        retValue = ADK_ERROR_SEV_LEVEL_SET (ADK_E_INVALID_PARAMETER);
        ADK_FORCE_LOG_ERROR ("Invalid system handle %"ADKPRIh"\n", sysHandle);
        return retValue;
    }

    if (NULL == adkSysShmConf_p)
    {
        ADK_FORCE_LOG_ERROR("Sys shared memory not configured\n");
        retValue = ADK_STATUS_SEV_LEVEL_SET (ADK_E_SYS_NOT_INITIALIZED,
                                             ADK_SEV_LEVEL_ERROR);
        return retValue;
    }

    ADK_ACQUIRE_SYS_LOCK(sysHandle, retValue);
    shmInfo = (adk_sif_dp_shm_info_t *)(adkSysShmConf_p->sifDpShmHandle);
    if (NULL != shmInfo) {
        shmInfo->flowCacheCountersFlag = enable;
    }
    ADK_RELEASE_SYS_LOCK(sysHandle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_RETURN(retValue);
}


/*!  @} end addtogroup _sifFunctions
 *
 * @} end addtogroup _sif
 */
