/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_sys_dpdk.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief  
 *
 * This file contains the Intel DPDK software initialization functions and 
 * hardware initialization functions.
 *
 *
 *  **********************************************************************/


#include <adk_dpdk.h>
#include "adk_include.h"
#include "adk_error.h"
#include "adk_sys_dpdk.h"
#include <adk_pktio.h>
#include <adk_rcu.h>

adk_pktio_hdl_t adkTunTapHandle_g[RTE_MAX_ETHPORTS]={0};

struct rte_mempool *adkPktmbufPool_g;
extern struct rte_eth_conf adkPortConfDefaults_g;
adk_soft_rss_reta_tbl_t *adk_soft_rss_reta_tbl[ADK_SIF_CFG_MAX_PORT];

#if defined(DPDK_IP_FRAG) || defined(DPDK_IPV6_FRAG)
struct rte_mempool *pktmbuf_pool_indirect_g;
#endif

adk_status_t
adk_sys_dpdk_init(const adk_sys_conf_t *sysConf_p, adk_sif_cfg_info_t **cfg_hdl)
{
    adk_status_t retValue = ADK_SUCCESS;
    adk_sys_conf_user_args_t *cliArgs = NULL;
    adk_sif_cfg_info_t *cfg_info_p = NULL;
    adk_sif_cfg_eal_info_t *eal_info = NULL;
    adk_sif_cfg_mempool_info_t *mempool_info = NULL;
    adk_sif_cfg_port_info_t *port_info = NULL;
    struct ether_addr addr;
    adk_pktio_cfg_t cfg;
    uint32_t i = 0;
    uint32_t len;
    
    if (sysConf_p == NULL)
    {
        retValue = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE, ADK_SEV_LEVEL_ERROR);
        ADK_PREINIT_WRITE("Invalid sys conf pointer. Error code: %d\n", retValue);
        goto _exit;
    }

    cliArgs = (adk_sys_conf_user_args_t *)sysConf_p->userArgs_p;

    retValue = adk_sif_cfg_init(&cfg_info_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("Sys DPDK init cfg failed\n");
        goto _exit;
    }

    retValue = adk_sif_cfg_load(cliArgs->argc, cliArgs->argv_p, cfg_info_p);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("Sys DPDK load cfg failed\n");
        goto _runtime_error_1;
    }
    *cfg_hdl = cfg_info_p;

    retValue = adk_sif_cfg_eal_info_get(cfg_info_p->eal_info_tbl, &eal_info);
    if ((retValue != ADK_SUCCESS) || (eal_info == NULL))
    {
        ADK_PREINIT_WRITE("EAL info get failed\n");
        ADK_PREINIT_WRITE("Sys DPDK init failed\n");
        goto _runtime_error_1;
    }

    /* TODO: Deprecated: Process_type shall not be specified in config file */
    if (eal_info->process_type != ADK_SIF_CFG_EAL_PROCESS_TYPE_PRIMARY)
    {
        /* No further configuration for secondary process */
        goto _exit;
    }

    /* TBD - Need to add mempool name entry and query API to fetch mempool handle based on mempool name */
    retValue = adk_sif_cfg_mempool_info_get(cfg_info_p->mempool_info_tbl, PKT_MEMPOOL_1, &mempool_info);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("Invalid mempool id %d\n", PKT_MEMPOOL_1);
        ADK_PREINIT_WRITE("Sys DPDK init failed\n");
        goto _runtime_error_1;
    }
    adkPktmbufPool_g = (struct rte_mempool *)mempool_info->mem_buf_pool;

#if  defined(DPDK_IP_FRAG)  || defined(DPDK_IPV6_FRAG)
    retValue = adk_sif_cfg_mempool_info_get(cfg_info_p->mempool_info_tbl, PKT_MEMPOOL_INDIRECT_1, &mempool_info);
    if (retValue != ADK_SUCCESS)
    {
        ADK_PREINIT_WRITE("Invalid mempool id %d\n", PKT_MEMPOOL_INDIRECT_1);
        ADK_PREINIT_WRITE("Sys DPDK init failed\n");
        goto _runtime_error_1;
    }
    pktmbuf_pool_indirect_g = (struct rte_mempool *)mempool_info->mem_buf_pool;
#endif

    /* Create tun/tap interface for each port */
    adk_sif_cfg_port_info_rewind(cfg_info_p->port_info_tbl);

    for (i = 0; i < ADK_SIF_CFG_MAX_PORT; i++)
    {
        retValue = adk_sif_cfg_port_info_get_next(cfg_info_p->port_info_tbl, &port_info);
        if (retValue != ADK_SUCCESS)
        {
            if (retValue == ADK_E_LIST_RANGE)
            {
                retValue = ADK_SUCCESS;
                break;
            }
            ADK_PREINIT_WRITE("Port get failed\n");
            ADK_PREINIT_WRITE("Sys DPDK init failed\n");
            goto _runtime_error_1;
        }

        rte_eth_macaddr_get(port_info->portid, &addr);

        cfg.type = ADK_PKTIO_TYPE_TUNTAP;
        cfg.u.tuntap_config.isTun = FALSE;
        len = sizeof(cfg.u.tuntap_config.dev_name) - 1;
        snprintf(cfg.u.tuntap_config.dev_name, len,
                 "adktap%d", port_info->portid);
        cfg.u.tuntap_config.dev_name[len] = '\0';
        ADK_MEMCPY(cfg.u.tuntap_config.dev_addr, addr.addr_bytes, sizeof(char) * 6);

        if(ADK_PKTIO_SUCCESS == adk_pktio_config(cfg, &(adkTunTapHandle_g[port_info->portid]))) {
            ADK_PREINIT_WRITE("Created tun/tap interface (%s) for netdriver support...\n",
                    cfg.u.tuntap_config.dev_name);
        } else {
            ADK_PREINIT_WRITE("Error: Failed to create tun/tap interface (%s)...\n", cfg.u.tuntap_config.dev_name);
            /* TODO: Set unique return code */
            retValue = ADK_STATUS_SEV_LEVEL_SET(ADK_E_FG_UNKNOWN, ADK_SEV_LEVEL_FATAL);
            goto _runtime_error_1;
        }
    }

    adk_sif_soft_rss_init(cfg_info_p);

_runtime_error_1:
    adk_sif_cfg_shutdown(cfg_info_p);

_exit:
    return retValue;
}    /* -----  end of function adk_sys_dpdk_init  ----- */

adk_status_t
adk_sif_soft_rss_init(adk_sif_cfg_info_t *cfg_info)
{
    adk_status_t retValue;
    int i;
    struct rte_eth_dev_info dev_info;
    adk_sif_cfg_port_info_t *port_info = NULL;
    adk_sif_cfg_port_info_rewind(cfg_info->port_info_tbl);
    for (i = 0; i < ADK_SIF_CFG_MAX_PORT; i++)
    {
        retValue = adk_sif_cfg_port_info_get_next(cfg_info->port_info_tbl, &port_info);
        if (retValue != ADK_SUCCESS)
        {
            if (retValue == ADK_E_LIST_RANGE)
            {
                retValue = ADK_SUCCESS;
                break;
            }
            ADK_PREINIT_WRITE("RSS RETA Init: Port get failed\n");
            ADK_PREINIT_WRITE("Sys DPDK Init failed\n");
            return retValue;
        }
        ADK_MEMSET(&dev_info, 0, sizeof(struct rte_eth_dev_info));
        rte_eth_dev_info_get(port_info->portid, &dev_info);
        adk_soft_rss_reta_tbl[port_info->portid] = (adk_soft_rss_reta_tbl_t *) ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0,
									 dev_info.reta_size ,
									 sizeof(adk_soft_rss_reta_tbl_t));
        ADK_PREINIT_WRITE("Port Id: %d RSS RETA Size:%hd\n", port_info->portid, dev_info.reta_size);
    }
    return retValue;
}

adk_status_t
adk_sif_soft_rss_update (adk_sys_handle_t sys_handle, adk_soft_rss_reta_tbl_t *reta_info)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint32_t rss_l3 = 0;
    uint16_t reta_idx = 0, hw_idx = 0, shift = 0;
    struct rte_eth_dev_info dev_info;
    uint8_t portid = reta_info->portid;
    uint16_t qid = reta_info->qid;
/* X710 NICs default RSS Hash Key  */
    static uint32_t rss_default_hashkey[] = {0x6b793944,
        0x23504cb5, 0x5bea75b6, 0x309f4f12, 0x3dc0a2b8,
        0x024ddcdf, 0x339b8ca0, 0x4c4af64a, 0x34fac605,
        0x55d85839, 0x3a58997d, 0x2ec938e1, 0x66031581};
    uint8_t rss_key_be[RTE_DIM(rss_default_hashkey)];

    adk_sys_info_t *sys_info = (adk_sys_info_t *) sys_handle;
    adk_sif_cfg_info_t *cfg_info = (adk_sif_cfg_info_t *) sys_info->cfgHandle;

    ADK_MEMSET(&dev_info, 0, sizeof(struct rte_eth_dev_info));
    rte_eth_dev_info_get(portid, &dev_info);
    struct rte_eth_rss_reta_entry64 reta_conf[dev_info.reta_size/RTE_RETA_GROUP_SIZE];
    ADK_MEMSET(&reta_conf, 0, ((dev_info.reta_size/RTE_RETA_GROUP_SIZE) * sizeof(struct rte_eth_rss_reta_entry64)));
    ADK_PREINIT_WRITE("RETA Conf Struct: %d\n", dev_info.reta_size/RTE_RETA_GROUP_SIZE);
    
    retValue = adk_sif_cfg_port_check(cfg_info->port_info_tbl, portid);
    if (retValue != ADK_SUCCESS)
    {
            ADK_PREINIT_WRITE("ADK RSS: Unconfigured Port [%d]\n", portid);
            return retValue;
    }

    retValue = adk_sif_cfg_port_rx_queue_check(cfg_info->port_info_tbl, portid, qid);
    if (retValue != ADK_SUCCESS)
    {
            ADK_PREINIT_WRITE("ADK RSS: Unconfigured Port [%d] Queue[%d] \n", portid, qid);
            return retValue;
    }

    rte_convert_rss_key((uint32_t *) &rss_default_hashkey, (uint32_t *) rss_key_be, RTE_DIM(rss_default_hashkey));
    rss_l3 = rte_softrss_be((uint32_t *)&reta_info->tuple, RTE_THASH_V4_L3_LEN, rss_key_be);
    reta_idx = (rss_l3 & (uint32_t) (dev_info.reta_size - 1));

    if ((adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.dst_addr != 0x0) &&
            (adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.src_addr != 0x0)) {
        if ((adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.dst_addr != reta_info->tuple.v4.dst_addr) &&
                (adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.src_addr != reta_info->tuple.v4.src_addr)) {
            ADK_PREINIT_WRITE("[SIP: %x DIP: %x] Tuple Hash [%x] matches an existing [SIP: %x DIP: %x] Tuple Hash \n",
                    reta_info->tuple.v4.src_addr, reta_info->tuple.v4.dst_addr, reta_idx,
                    adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.src_addr, adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.dst_addr);
            /* TODO: Set unique return code */
            return ADK_STATUS_SEV_LEVEL_SET(ADK_E_FG_UNKNOWN, ADK_SEV_LEVEL_FATAL);
        }
    }

    adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.src_addr = reta_info->tuple.v4.src_addr;
    adk_soft_rss_reta_tbl[portid][reta_idx].tuple.v4.dst_addr = reta_info->tuple.v4.dst_addr;
    adk_soft_rss_reta_tbl[portid][reta_idx].qid = qid;
    adk_soft_rss_reta_tbl[portid][reta_idx].portid = portid;

    int ret = 0;
    ret = rte_eth_dev_rss_reta_query(portid, reta_conf, dev_info.reta_size);
    if (ret == ENOTSUP || ret == EINVAL)
    {
        ADK_PREINIT_WRITE("Failed to update RSS RETA info: NO RSS SUPPORT or Invalid Parameter\n");
        /* TODO: Set unique return code */
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_FG_UNKNOWN, ADK_SEV_LEVEL_FATAL);
    }
    hw_idx = reta_idx / RTE_RETA_GROUP_SIZE;
    shift = reta_idx % RTE_RETA_GROUP_SIZE;
    reta_conf[hw_idx].mask |= (1ULL << shift);
    reta_conf[hw_idx].reta[shift] = qid;
    ADK_PREINIT_WRITE("RSS RETA Hash Index: %d port id: %hhd qid: %hd\n", reta_idx, portid, qid);
    
    ret = rte_eth_dev_rss_reta_update(portid, reta_conf, dev_info.reta_size);
    if (ret == ENOTSUP)
    {
        ADK_PREINIT_WRITE("Failed to update RSS RETA info: NO RSS SUPPORT\n");
        /* TODO: Set unique return code */
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_FG_UNKNOWN, ADK_SEV_LEVEL_FATAL);
    }
    else if (ret == EINVAL)
    {
        ADK_PREINIT_WRITE("Failed to update RSS RETA info: INVALID PARAMETER\n");
        /* TODO: Set unique return code */
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_FG_UNKNOWN, ADK_SEV_LEVEL_FATAL);
    }

    return retValue;
}


/*!    end addtogroup _sifFunctions
 *     @}
 * end addtogroup _sif
 * @}
 */
