/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_dp_threads_dpdk.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief
 *
 *
 ***********************************************************************/

#include <sched.h>
#include <pthread.h>

#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_pmu.h"
#include "adk_ph_dev.h"
#include "adk_ethernet.h"
#include "adk_sif_cfg_common.h"
#include "adk_sif_cfg.h"
#include "adk_cfg.h"
#include "adk_cfg_eal.h"
#include "adk_sif_cfg_eal.h"
#include "adk_sys_dpdk.h"
#include "adk_sys_private.h"
#include "adk_timer.h"
#include "adk_sif_dp_log_lib.h"
#ifdef ADK_RCU
#include "adk_rcu.h"
#endif
#ifdef INTEL_ITT
#include "ittnotify.h"
#endif
#include <adk_pal_dp.h>

#define OUTBOUND	0x0
#define INBOUND 	0x1

extern struct rte_ring * adkDpPhRcvQueue_g[ADK_MAX_NUM_CORES];
extern struct rte_ring * adkCpNetdRcvQueue_g;

adk_sys_thread_info_t adkT_Info_g;

pthread_t adkPktRcvThreadId_g[4], adkMasterThreadId_g;

#ifdef ENABLE_DPDK_PACKET_BUFFERS
volatile uint8_t adkWaitForCpFlag;
#endif

adk_pktio_hdl_t adkPktioPort_g[ADK_MAX_NUM_CORES];
adk_pktio_hdl_t adkPktioRing_g[ADK_MAX_NUM_CORES];
adk_pktio_hdl_t adkPktioBuff_g[ADK_MAX_NUM_CORES];
#ifdef PH_RING_QUEUE
    adk_pktio_hdl_t adkPktioPh_g[ADK_MAX_NUM_CORES];
#endif

int adk_dp_process_master(void *data);

adk_status_t
adk_dp_portq_ringq_in_set(adk_sif_cfg_pipeline_port_q_t *portq_in, uint32_t *num_portq_in,
                          adk_sif_cfg_pipeline_ring_q_t *ringq_in, uint32_t *num_ringq_in);

void
adk_sys_create_dp_threads(adk_sys_handle_t sysHandle,
        char __attribute__((unused))*configFile_p)
{
    adk_status_t ret = ADK_SUCCESS;
    unsigned int i = 0;
    adk_sif_cfg_info_t *cfg_hdl = (adk_sif_cfg_info_t *)((adk_sys_info_t *)sysHandle)->cfgHandle;
    adk_sif_cfg_eal_info_t *eal_info = NULL;

    ret = adk_sif_cfg_eal_info_get(cfg_hdl->eal_info_tbl, &eal_info);
    if ((ret != ADK_SUCCESS) || (eal_info == NULL))
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,
                    "EAL info get failed. Can't attach dataprocess function\n");
        exit(0);
    }

    adkT_Info_g.sysHandle = sysHandle;

    /* Check states of lcores */
    for (i = 0; i < eal_info->num_worker_lcores; i++)
    {
        if (lcore_config[eal_info->worker_lcores[i]].state != WAIT)
        {
            ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,
                        "Lcores are busy. Can't attach dataprocess function\n");
            exit(0);
        }
    }

    /* Attach dp process function to lcores */
    for (i = 0; i < eal_info->num_worker_lcores; i++)
    {
        rte_eal_remote_launch(adk_dp_process_master, (void *)&adkT_Info_g, eal_info->worker_lcores[i]);
    }
}

/* Master thread function */
int adk_dp_process_master(void *data)
{
    adk_pktio_task_t *task_p[ADK_DP_BULK_RX_SIZE] = {NULL};
    adk_pktio_cfg_t cfg;
    uint32_t st, task_mask;
    uint8_t task_count;
    adk_sys_handle_t sysHandle;
    uint32_t coreIndex;
    measure_cycle_t *mcd = ADK_MEASURE_CYCLE_INIT();
    measure_cycle_t *ittd = ADK_MEASURE_CYCLE_INIT();
#ifdef PH_RING_QUEUE
    uint8_t  ph_q_read_poll_count=0;
#endif
#ifdef ADK_TIMER
    uint8_t timer_proc_count = 0;
#endif

#ifdef ADK_RCU
    uint32_t rcu_read_poll_count = 0;
#endif
    uint16_t adk_pktioEntryFuncIdx = 0;

    adk_lcore_input_receive_t adk_lcore_input_receive = {
        .types = { FALSE, FALSE, FALSE, FALSE },
        .portq_in[ADK_SIF_CFG_MAX_PORT - 1] = {0},
        .ringq_in[ADK_SIF_CFG_MAX_RING_QUEUES - 1] = {0},
        .buffs_in[ADK_SIF_CFG_MAX_PKT_BUFFS - 1] = {0},
        0, 0, 0,
        FALSE, FALSE
    };
    adk_sif_cfg_portq_t portq;
    adk_sif_cfg_ringq_t ringq;
    uint32_t p_idx = 0, q_idx[ADK_SIF_CFG_MAX_PORT] = {0}, r_idx = 0;
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    uint32_t b_idx = 0;
    adk_sif_cfg_pkt_buffs_t buffQ;
#endif

#ifdef QAT
    uint16_t ipsec_entry_func = 0;
    uint8_t qat_read_freq = 0;
    adk_pktio_task_t *task_qat_p[32] = {NULL};
#endif

    sysHandle = ((adk_sys_thread_info_t *)data)->sysHandle;

    /* store the cpu id in thread-local varibale */
    coreIndex = ADK_LCORE_INDEX;

#ifdef ADK_TIMER
    adk_timer_init(coreIndex, adk_get_cycle_cntr());
#endif

    adk_lcore_input_receive.types.all = TRUE;
    st = adk_sif_cfg_pipeline_lcore_entry_in_get(coreIndex, &adk_lcore_input_receive);
    if (st != ADK_SUCCESS)
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,
                         "Failed to get input port/queue/buffer info from "
                         "pipeline table!\n");
        exit(0);
    }
    adk_sif_cfg_pipeline_lcore_entry_dump(
            coreIndex,
            &adk_lcore_input_receive);

#ifdef ADK_QOS_ENABLE
#if 0
    qData =  ADK_NVM_CALLOC(ADK_SYS_MODULE_ID, 0, 1, (sizeof(adk_qos_dp_process_data_t)));
    if (ADK_INVALID_PTR(qData))
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,
                         "Error: failed to allocate memory!\n");
        exit(0);
    }
    qData->sysHdl = sysHandle;
    qData->qosInstance = 0;
    qData->loop = 0; /* continuous loop */
#endif
#endif


#ifdef PH_RING_QUEUE
    /* configure PH pkt handler */
    cfg.type = ADK_PKTIO_TYPE_PH;
    cfg.u.dpdk_config.pktmbuf_pool = adkPktmbufPool_g;


    if(ADK_PKTIO_SUCCESS == adk_pktio_config(cfg, &adkPktioPh_g[coreIndex]))
    {
        ADK_DP_LOG_INFO_1P_DEBUG(ADK_SYS_MODULE_ID,
                                 "PH Config - Master thread handle %p\n",
                                 (void *)pthread_self());
    }
    else
    {
        ADK_DP_LOG_ERROR_1P(ADK_SYS_MODULE_ID,
                            "PH config failed - Master thread handle %p\n",
                            (void *)pthread_self());
        exit(0);
    }
#endif

    /* configure pkt handler for port rx/tx */
    ADK_MEMSET((void *) &cfg, 0 , sizeof(adk_pktio_cfg_t));
    cfg.type = ADK_PKTIO_TYPE_DPDK_PORT;
    cfg.u.dpdk_config.pktmbuf_pool = adkPktmbufPool_g;
    if(ADK_PKTIO_SUCCESS == adk_pktio_config(cfg, &adkPktioPort_g[coreIndex]))
    {
        ADK_DP_LOG_INFO_1P_DEBUG(ADK_SYS_MODULE_ID,
                                 "DPDK Port - Master thread handle %p\n",
                                 (void *)pthread_self());
    }
    else
    {
        ADK_DP_LOG_ERROR_1P(ADK_SYS_MODULE_ID,
                            "Failed: DPDK Port - Master thread handle %p\n",
                            (void *)pthread_self());
        exit(0);
    }

    ADK_MEMSET((void *) &cfg, 0 , sizeof(adk_pktio_cfg_t));
    /* configure pkt handler for ring enqueue/dequeue */
    cfg.type = ADK_PKTIO_TYPE_DPDK_RING;
    if(ADK_PKTIO_SUCCESS == adk_pktio_config(cfg, &adkPktioRing_g[coreIndex]))
    {
        ADK_DP_LOG_INFO_1P_DEBUG(ADK_SYS_MODULE_ID,
                                 "PktIO Ring - Master thread handle %p\n",
                                 (void *)pthread_self());
    }
    else
    {
        ADK_DP_LOG_ERROR_1P(ADK_SYS_MODULE_ID,
                           "Failed: PktIO Ring - Master thread handle %p\n",
                           (void *)pthread_self());
        exit(0);
    }

#ifdef ENABLE_DPDK_PACKET_BUFFERS
    /* configure pkt handler for buffer enqueue/dequeue */
    ADK_MEMSET((void *) &cfg, 0 , sizeof(adk_pktio_cfg_t));
    cfg.type = ADK_PKTIO_TYPE_DPDK_BUFF;
    cfg.u.dpdk_config.pktmbuf_pool = adkPktmbufPool_g;
    if (ADK_PKTIO_SUCCESS == adk_pktio_config(cfg, &adkPktioBuff_g[coreIndex]))
    {
        ADK_DP_LOG_INFO_1P_DEBUG(ADK_SYS_MODULE_ID,
                                 "PktIO Buff - Master thread handle %p\n",
                                 (void *)pthread_self());
    }
    else
    {
        ADK_DP_LOG_ERROR_1P(ADK_SYS_MODULE_ID,
                            "Failed: PktIO Buff - Master thread handle %p\n",
                            (void *)pthread_self());
        exit(0);
    }
#endif

    ADK_DP_LOG_INFO_1P_DEBUG(ADK_SYS_MODULE_ID,
                             "Master Thread attached to Lcore %p\n",
                             (void *)pthread_self());
#ifdef QAT
    if (qat_rx_decrypt)
    {
        adk_ipsec_lli_task_read_param_set((void **)task_qat_p,
                                          &task_count, &task_mask);
    }

    if (qat_rx_encrypt)
    {
        adk_ipsec_uli_task_read_param_set((void **)task_qat_p,
                                          &task_count, &task_mask);
    }
#endif

#ifdef ENABLE_DPDK_PACKET_BUFFERS
    ADK_DP_LOG_INFO_DEBUG(ADK_SYS_MODULE_ID,
                          "DP Thread waiting For Control Plane\n");

    while(adkWaitForCpFlag)
    {};

    ADK_DP_LOG_INFO_DEBUG(ADK_SYS_MODULE_ID, "DP Thread Ready!\n");
#endif
    /* Initialize to 0 before stating the loop */
    ADK_MEMSET(&portq, 0, sizeof(portq));

    /* listen to packets on recv queue */
    while (1)
    {
        if (adk_lcore_input_receive.num_portq_in)
        {
            portq.port_id = adk_lcore_input_receive.portq_in[p_idx].portid;
            portq.queue_id = adk_lcore_input_receive.portq_in[p_idx].queueid[q_idx[p_idx]];
            portq.entry_func = adk_lcore_input_receive.portq_in[p_idx].entry_func[q_idx[p_idx]];
            portq.entry_mod_hdl = adk_lcore_input_receive.portq_in[p_idx].entry_mod_hdl[q_idx[p_idx]];

            portq.weight++;
            if (portq.weight == adk_lcore_input_receive.portq_in[p_idx].weight[q_idx[p_idx]])
            {
                portq.weight = 0;
                q_idx[p_idx]++;
                if (q_idx[p_idx] == adk_lcore_input_receive.portq_in[p_idx].num_queues)
                {
                    q_idx[p_idx] = 0;
                    p_idx++;
                    p_idx = p_idx % adk_lcore_input_receive.num_portq_in;
                }
            }
            /* receive ethernet packet from network */
            task_count = 0;
            st = adk_pktio_recv(adkPktioPort_g[coreIndex], (void *)&portq, task_p,
                                    &task_count, &task_mask);
            if (likely (st == ADK_PKTIO_SUCCESS))
            {
                PIPELINE_STAGE_PORT_PACKET_COUNT_ADD((coreIndex - 18), task_count);
                INTEL_ITT_RESUME(&ittd);
                ADK_MEASURE_CYCLE_START(&mcd);

                adk_pktioEntryFuncIdx = ADK_METADATA_PARAM(task_p[0], func_idx);
                if(adkDestEntryFunction_g[adk_pktioEntryFuncIdx]) {
                    (adkDestEntryFunction_g[adk_pktioEntryFuncIdx])((void **)task_p, task_count, task_mask, NULL, NULL, coreIndex);
                }
                ADK_MEASURE_CYCLE_DIFF(&mcd, task_count);
                INTEL_ITT_INCR_ITER(&ittd, task_count);
            }
            else if (st == ADK_PKTIO_QUEUE_EMPTY) {}
            else
            {
                ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "Invalid task received!\n");
            }
        }

        if (adk_lcore_input_receive.num_ringq_in)
        {
            ringq.ring_hdl = adk_lcore_input_receive.ringq_in[r_idx].ringhdl;
            ringq.entry_func = adk_lcore_input_receive.ringq_in[r_idx].entry_func;
            r_idx++;
            r_idx = r_idx % adk_lcore_input_receive.num_ringq_in;

            /* receive ethernet packet from network */
            task_count = 0;
            st = adk_pktio_recv(adkPktioRing_g[coreIndex], (void *)&ringq, task_p,
                                    &task_count, &task_mask);
            PIPELINE_STAGE_RING_PACKET_COUNT_ADD((coreIndex - 18), task_count);
            if (likely (st == ADK_PKTIO_SUCCESS))
            {
                INTEL_ITT_RESUME(&ittd);
                ADK_MEASURE_CYCLE_START(&mcd);
#ifdef QAT
                /* Store the entry function */
#if 0 /* If enabled, packets doesnt reach the right pipeline stage */
                if (unlikely (adk_pktioEntryFuncIdx == 0))
#endif
                {
                    adk_pktioEntryFuncIdx = ADK_CONVERT_TPARAMS_TO_UINT16(
                                            ADK_TASK_USER_PARAMS(task_p[0], 9));
                }
#else
                adk_pktioEntryFuncIdx = ADK_CONVERT_TPARAMS_TO_UINT16(
                                            ADK_TASK_USER_PARAMS(task_p[0], 9));
#endif
                if(adkDestEntryFunction_g[adk_pktioEntryFuncIdx]) {
                    (adkDestEntryFunction_g[adk_pktioEntryFuncIdx])
                        ((void **)task_p, task_count, task_mask, NULL, NULL, coreIndex);
                }

                ADK_MEASURE_CYCLE_DIFF(&mcd, task_count);
                INTEL_ITT_INCR_ITER(&ittd, task_count);
            }
            else if (st == ADK_PKTIO_QUEUE_EMPTY) {}
            else
            {
                ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "Invalid task received!\n");
            }
#ifdef QAT
            qat_read_freq++;
#endif
        }

#ifdef ENABLE_DPDK_PACKET_BUFFERS
        if (adk_lcore_input_receive.num_buffs_in)
        {
            ADK_DP_LOG_INFO_1D_DEBUG(ADK_SYS_MODULE_ID,
                                     "Number of buffers %d\n",
                                     adk_lcore_input_receive.num_buffs_in);
            buffQ.buff_id = adk_lcore_input_receive.buffs_in[b_idx].buff_id;
            buffQ.entry_func = adk_lcore_input_receive.buffs_in[b_idx].entry_func;
            buffQ.transmit = adk_lcore_input_receive.buffs_in[b_idx].transmit;

            if (buffQ.weight == adk_lcore_input_receive.buffs_in[b_idx].weight)
            {
                buffQ.weight = 0;
                b_idx++;
                b_idx = b_idx % adk_lcore_input_receive.num_buffs_in;
            }

            buffQ.weight++;
            /* receive ethernet packet from pkt buffers */
            task_count = 0;
            st = adk_pktio_recv(adkPktioBuff_g[coreIndex], (void *)&buffQ, task_p,
                                    &task_count, &task_mask);
            /* PIPELINE_STAGE_RING_PACKET_COUNT_ADD((coreIndex - 18), task_count); */
            if (likely (st == ADK_PKTIO_SUCCESS))
            {
                /* INTEL_ITT_RESUME(&ittd); */
                /* ADK_MEASURE_CYCLE_START(&mcd); */
                adk_pktioEntryFuncIdx =  ADK_METADATA_PARAM(task_p[0], func_idx);
                if(adkDestEntryFunction_g[adk_pktioEntryFuncIdx]) {
                    /* Calling Destination Function */
                    (adkDestEntryFunction_g[adk_pktioEntryFuncIdx])
                        ((void **)task_p, task_count, task_mask, NULL, NULL, coreIndex);
                }

                /* ADK_MEASURE_CYCLE_DIFF(&mcd, task_count); */
                /* INTEL_ITT_INCR_ITER(&ittd, task_count); */
            }
            else if (st == ADK_PKTIO_QUEUE_EMPTY)
            {}
            else
            {
                ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "Invalid task received!\n");
            }
        }
#endif
#ifdef PH_RING_QUEUE
        if(ADK_PHQ_READ_POLL_CNT == ++ph_q_read_poll_count)
        {
            st = adk_pktio_recv(adkPktioPh_g[coreIndex],
                                (void *)adkDpPhRcvQueue_g[coreIndex],
                                task_p, &task_count, &task_mask);
            if(st == ADK_PKTIO_SUCCESS)
            {
                ADK_DP_LOG_INFO_DEBUG(ADK_SYS_MODULE_ID,
                                      "task rcvd in ph_queue\n");
                ADK_MEASURE_CYCLE_START(&mcd);
                adk_ph_dp_cpu_rcv((void **)task_p, task_count,
                                  task_mask, coreIndex);
                ADK_MEASURE_CYCLE_DIFF(&mcd, task_count);

            }
            else if (st == ADK_PKTIO_QUEUE_EMPTY) {}
            else
            {
                ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "Invalid task received!\n");
            }

            ph_q_read_poll_count = 0;
        }
#endif
#ifdef ADK_TIMER
        if( ADK_TIMER_PROC_PKT_PERIOD == ++timer_proc_count)
        {
#ifdef ENABLE_DPDK_PACKET_BUFFERS
            adk_pktio_dpdk_buff_flush(adkPktioBuff_g[coreIndex], coreIndex);
#else
            adk_pktio_dpdk_port_flush(coreIndex);
#endif
            adk_timer_proc(coreIndex, adk_get_cycle_cntr());
            timer_proc_count = 0;
        }
#endif
#ifdef ADK_RCU
        if(ADK_RCU_READ_POLL_CNT == ++rcu_read_poll_count)
        {
            adk_rcu_quiescent_state();
            rcu_read_poll_count = 0;
        }
#endif

#ifdef QAT
        if (adk_lcore_input_receive.qat_rx_decrypt)
        {
            qat_read_freq = 0;
            task_count = 0;
            task_mask = 0;

            ADK_MEASURE_CYCLE_START(&mcd);
            /* Poll the responses */
            adk_ipsec_lli_read_qat_response ();

            PIPELINE_STAGE_QAT_PACKET_COUNT_ADD((coreIndex - 18), task_count);
            /* Call the destination function */
            if (task_count != 0)
            {
                /* Store the entry function */
                ipsec_entry_func = ADK_CONVERT_TPARAMS_TO_UINT16(
                                        ADK_TASK_USER_PARAMS(task_qat_p[0], 9));

                if(adkDestEntryFunction_g[ipsec_entry_func]) {
                    (adkDestEntryFunction_g[ipsec_entry_func]) (
                    (void **)task_qat_p,
                    task_count,
                    task_mask,
                    NULL,
                    NULL,
                    coreIndex);
                }
                ADK_MEASURE_CYCLE_DIFF(&mcd, task_count);
            }
        }

        if ((adk_lcore_input_receive.qat_rx_encrypt))
        {
            qat_read_freq = 0;
            task_count = 0;
            task_mask = 0;

            ADK_MEASURE_CYCLE_START(&mcd);
            /* Poll the responses */
            adk_ipsec_uli_read_qat_response ();

            PIPELINE_STAGE_QAT_PACKET_COUNT_ADD((coreIndex - 18), task_count);
            /* Call the destination function */
            if (task_count != 0)
            {
                ipsec_entry_func = ADK_CONVERT_TPARAMS_TO_UINT16(
                                        ADK_TASK_USER_PARAMS(task_qat_p[0], 9));

                if(adkDestEntryFunction_g[ipsec_entry_func]) {
                    (adkDestEntryFunction_g[ipsec_entry_func]) (
                    (void **)task_qat_p,
                    task_count,
                    task_mask,
                    NULL,
                    NULL,
                    coreIndex);
                }
                ADK_MEASURE_CYCLE_DIFF(&mcd, task_count);
            }
        }
#endif
        ADK_MEASURE_CYCLE_CHECK_EXIT(mcd, coreIndex, 0);
        INTEL_ITT_PAUSE(&ittd);
    }
    pthread_exit(NULL);
}

adk_status_t
adk_sys_dp_pkt_send_to_port(void **task_p, __attribute__((unused)) uint8_t task_count,
                            uint32_t task_mask, __attribute__((unused)) int coreIndex)
{
    uint32_t ret, i, cur_bit;
    struct rte_mbuf *mbuf, *next;
    uint16_t    dev_tx_queue_id;
    adk_status_t retVal = ADK_SUCCESS;
    uint8_t pktColor;

    for(i=0,cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(task_mask & cur_bit)
        {
            mbuf = (struct rte_mbuf *)(task_p[i]);
            while(mbuf)
            {
                next = (struct rte_mbuf *)ADK_METADATA_PARAM((adk_pktio_task_t *)(mbuf), next_frag);
                dev_tx_queue_id = ADK_METADATA_PARAM((adk_pktio_task_t *)(mbuf), dev_tx_queue_id);
                pktColor = ADK_METADATA_PARAM((adk_pktio_task_t *)(mbuf), pktColor);
                ret = adk_pal_dp_nic_send(mbuf->port, pktColor, dev_tx_queue_id, mbuf, ADK_PAL_DP_BATCH_CB_FN_ETH, NULL);
                if(ret != 1)
                {
                    retVal = ADK_E_INTERNAL_ERROR;
                    rte_pktmbuf_free(mbuf);
                }
                mbuf = next;
            }
        }
    }

    return retVal;
}

adk_status_t
adk_sys_dp_pkt_send_to_linux(void *task_p)
{
    uint32_t ret=0;

    ret = adk_pal_dp_netd_send ((struct rte_mbuf *)(task_p), ADK_PAL_DP_BATCH_CB_FN_ETH, NULL);

    if(ret != ADK_E_TASK_CONSUMED) 
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,
                         "NetD PDU send to Linux via PAL failed\n");
        return ADK_E_INTERNAL_ERROR;
    }
    return ADK_SUCCESS;
}


/*!
 *     end addtogroup _sifFunctions  SIF Functions
 *     @}
 * end addtogroup _sif
 * @}
 */
