/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_em.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifFunctions  SIF Functions
 *     @{
 *
 * @brief
 *
 *
 ***********************************************************************/

#include <adk_include.h>
#include "adk_em.h"
#include "adk_sys_lcore.h"
#include <adk_sys_dpdk.h>
#include <adk_shm.h>
#include <adk_pmu.h>
#include <adk_hos.h>
#include <adk_timer.h>

#include <rte_bridge_timer.h>
#include <rte_bridge_eth.h>
#include "adk_pal_dp_export.h"
#include "adk_ipv6_dp_export.h"
#include "adk_ip_dp_export.h"
#include "adk_pal_dp.h"
#include "adk_sif_dp.h"
#include "adk_sif_dp_log_lib.h"
#include "adk_qos_dp_export.h"
#include "adk_nd_common.h"

#if ADK_DPDK_VERSION >= 18050016
#include <rte_service.h>
#include <rte_service_component.h>
#endif

adk_status_t adk_em_send_to_eo(struct rte_mbuf *mbuf, uint32_t dest)
{

	// called on ingress association action 'send_to_eo'
	// event = em_rte_mbuf_to_event();
	// em_send(event)
	adk_status_t ret = ADK_SUCCESS;
	em_event_t event = em_rte_mbuf_to_event(mbuf);
	ret = em_send(event, (em_queue_t)dest);
	return ret;
}

adk_status_t adk_sys_em_demux_fn_id(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex)

{
   //Register the function with PAL(done somewhere in adk_sys_init())
   //Find queue
   //Get data offset and iprarm offset
   //call entry point
   RTE_SET_USED(task_count);
   RTE_SET_USED(task_mask);
   RTE_SET_USED(ring);
   RTE_SET_USED(next_stage_entry);
   RTE_SET_USED(coreIndex);

   return adk_ip_dp_pkt_proc_em_uli(task_p);

}

adk_status_t adk_sys_em_ipv6_demux_fn_id(void **task_p,
                uint8_t task_count,
                uint32_t task_mask,
                void *ring,
                adk_sif_cfg_pipeline_info_t *next_stage_entry,
                int coreIndex)

{
   //Register the function with PAL(done somewhere in adk_sys_init())
   //Find queue
   //Get data offset and iprarm offset
   //call entry point
   RTE_SET_USED(task_count);
   RTE_SET_USED(task_mask);
   RTE_SET_USED(ring);
   RTE_SET_USED(next_stage_entry);
   RTE_SET_USED(coreIndex);

   return adk_ipv6_dp_pkt_proc_em_uli(task_p);

}

adk_status_t adk_em_attach(struct nonem_config config)
{
    //Called somewhere in adk_sys_init()
    //create nonem config
    //call em_attach()
    (void) config;
#if 0
	int i;
    struct nonem_config nonem_cfg = {0};
    nonem_cfg.nonem_eventdev_id = 0;
    /*currently supporting 1 entry point for IP*/
    nonem_cfg.num_dest = 1;

    for (i = 0; i < nonem_cfg.num_dest; i++) {
            /* i == 0 for IP module*/
            nonem_cfg.core_queue_map[i].virt_core_id = 25 + i; //TODO need: virtual code start
            nonem_cfg.core_queue_map[i].eventdev_qid = 7; //TODO need :queueid for ip entry point
    }
    nonem_cfg.start_port_id = 4; //TODO need :start number of port that em will use
    nonem_cfg.num_port = 4; //TODO need :number of em core

    nonem_cfg.nonem_coremask._u64[0] = 0xf0;    //TODO need : ADK's worker core mask

    /*Call em_attach()*/
    //TODO get EM Device ID
    if (0 != em_attach(1, &nonem_cfg))
    {
            printf("adk_sys_init failed. em_attach failed\n");
    }
#endif
    return 0;

}

void* adk_em_mbuf_to_iparam(struct rte_mbuf *mbuf)
{
	void *ret = NULL;
	em_event_t event = em_rte_mbuf_to_event(mbuf);
	ret = em_event_to_nonem_param(event);
	return ret;
}

adk_status_t adk_sys_em_pkt_proc_lli(void **task_p,
        uint8_t task_count,
        uint32_t task_mask,
        void *ring,
        adk_sif_cfg_pipeline_info_t *next_stage_entry,
        int coreIndex)
{
    RTE_SET_USED(task_count);
    RTE_SET_USED(task_mask);
    RTE_SET_USED(ring);
    RTE_SET_USED(next_stage_entry);
    RTE_SET_USED(coreIndex);

    adk_status_t ret = ADK_SUCCESS;
    adk_sif_em_dp_lli_meta_data_t *em_lli_md_p;
    adk_pktio_task_t *task = (adk_pktio_task_t *)(*task_p);

   ADK_DP_MEASURE_BLOCK_CYCLE_END_TO_END_LLI_TX_TS();
#ifdef ADK_HOS_DP_TEST
    /* Copy DSI VSI meta data from original location to post EM header */
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(task);
    uint8_t* post_em_header_ptr = (uint8_t*)((uintptr_t) task + sizeof(struct rte_mbuf) + EM_HDR_SIZE);
    memcpy(post_em_header_ptr, dsi_md_p, sizeof(adk_dsi_rx_metadata_t));
#endif

    em_lli_md_p = (adk_sif_em_dp_lli_meta_data_t *)&(ADK_METADATA_PARAM(task, imc_param_start));

    ret = adk_pal_dp_em_send(task, em_lli_md_p->eventMachineQueueId,
            ADK_PAL_DP_BATCH_CB_FN_EM, NULL);

    if (ret != ADK_E_TASK_CONSUMED) {
         ADK_DP_LOG_ERROR_2D(ADK_SYS_MODULE_ID,
                         "Error:adk_pal_dp_em_send() failed! status: %i, queue: %u \n", ret, em_lli_md_p->eventMachineQueueId);
        adk_pktio_task_buffer_free(task);
    }

    return ret;
}

void adk_em_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess)
{
    RTE_SET_USED(cbParam);

    if(cbEventType == ADK_PAL_DP_BATCH_CB_EVENT_EM)
    {
        if(sendSuccess)
        {
            return;
        }
        else
        {   
            //Add counter here for drop
            adk_pktio_task_buffer_free((adk_pktio_task_t *)(dpdkMbuf));
        }
    }
}
/*!
 *     end addtogroup _sifFunctions  SIF Functions
 *     @}
 * end addtogroup _sif
 * @}
 */
