/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_netd_ts_index.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifDpFunctions  SIF Data Plane Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK Dataplane System Configuration functions
 * for TN port timestamping index management.
 *
 *  **********************************************************************/
#ifdef ADK_DEVICE_SNR
#include <adk_netd_fp_ts_index.h>
#include "adk_qos_include_internal.h" 
#include "adk_ethernet_conf.h"

enum adk_fp_ts_state adk_fp_ts_index_state[ADK_NETD_MAX_PORT_GROUPS][ADK_NETD_MAX_TS_INDEX];

/* For storing the dev net id */
int adk_fp_ts_dev_net_id;

/* This function is for allocating a new mbuf from common pool,
 * which is used for sending control message to netd*/
static inline struct rte_mbuf *
adk_netd_fp_ts_cmsg_alloc(void)
{
    struct rte_mempool *mempool = NULL;
    struct rte_mbuf *cmsg = NULL;
    /* Get mempool info*/
    adk_sys_config_pktio_mempool_get((void **)&mempool);
    if(!mempool)
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "Failed to get mempool info from sys_config api\n");
        /*increment some fail counter*/
        return NULL;
    }
    cmsg  = rte_pktmbuf_alloc(mempool);
    if(!cmsg)
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID, "mbuf allocation failed for cmsg generation\n");
        /*increment some fail counter*/
    }
    return cmsg;
}

static int
adk_netd_fp_ts_send_to_linux(enum adk_netd_ts_message_type msg_type,
    enum adk_netd_ts_status_code status_code, uint8_t port,
    uint8_t ts_index, uint32_t corrl)
{
    int ret;
    struct rte_mbuf *cmsg = NULL;
    adk_status_t ret_val = ADK_E_TASK_CONSUMED;

    cmsg = adk_netd_fp_ts_cmsg_alloc();
    if(!cmsg)
    {
        /*Increment some fail counter*/
        return -1;
    }
    adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(cmsg);
    /* put status code in upper nibble */
    msg_type |= (status_code << 4);
    /* Update netdriver specific metadata*/
    ADK_MD_SET_TS_CORR(cmsg, corrl);
    ADK_MD_SET_TS_MSG_TYPE(cmsg, msg_type);
    ADK_MD_SET_TS_INDEX(cmsg, ts_index);
    ADK_MD_SET_LINUX_INTF_ID(cmsg, port);
    /* Update ADK dev-net device id here*/
    ADK_MD_SET_ADK_DEV_ID(cmsg, adk_fp_ts_dev_net_id);
    /* Update dev-net queue id here */
    ADK_MD_SET_DEV_TX_QUEUE_ID(cmsg, ADK_QOS_DEV_NET_CMSG_QUEUE_ID);
    /* make task type as new*/
    ADK_MD_SET_TASK_TYPE(cmsg, ADK_TASK_TYPE_NEW);

    cmsg->port = port;
    dsi_md_p->port = port;

    adk_slowpath_dev_dp_metadata_t *spDevMd_p = (adk_slowpath_dev_dp_metadata_t *)&(ADK_METADATA_PARAM(cmsg,
                imc_param_start));

    spDevMd_p->policerFlowId = ADK_QOS_INVALID_POLICER_FLOW_ID; 
    spDevMd_p->moduleId = ADK_ETHERNET_MODULE_ID; 
    spDevMd_p->flowId= ADK_ETH_NETD_TIMESTAMP_FLOWS_ID; 

    ret_val = adk_pal_dp_netd_send(cmsg, ADK_PAL_DP_BATCH_CB_FN_ETH, NULL);
    if(ret_val != ADK_E_TASK_CONSUMED)
    {
        /*Increment some fail counter*/
        return -1;
    }
    return ret;
}

int
adk_netd_fp_ts_index_alloc(uint8_t port,
    uint32_t corrl, uint8_t *ts_index)
{
    int ret = 0, ret_val = 1;
    uint8_t port_grp = port >> 2;

    ret = adk_netd_ts_index_alloc(port, ts_index);
    if(ret < 0)
    {
        ret_val = adk_netd_fp_ts_send_to_linux(ADK_NETD_TS_REPLY,
            ADK_NETD_TS_INDEX_ALLOC_FAIL, port, *ts_index, corrl);
    }
    else
    {
        /* Allocation Success*/
        ret_val = adk_netd_fp_ts_send_to_linux(ADK_NETD_TS_REPLY,
            ADK_NETD_TS_INDEX_ALLOC_SUCCESS, port, *ts_index, corrl);
        /*Update ADK FP index state to allocated*/
        adk_fp_ts_index_state[port_grp][*ts_index] = ADK_FP_TS_INDEX_ALLOCATED;
    }

    if(ret_val <= 0)
    {
        /* netd_xmt_fail counter is incremented in PAL
         * Timeout will happen and free netd table entry*/
    }
    return ret;
}

int
adk_netd_fp_ts_index_free(uint8_t port, uint8_t ts_index,
    uint32_t corrl)
{
    int ret = 0;
    uint8_t port_grp = port >> 2;
    enum adk_fp_ts_state current_state;

    /* Since no reply is sent to netd, corrl may not be needed*/
    RTE_SET_USED(corrl);

    if (unlikely(port_grp >= ADK_NETD_MAX_PORT_GROUPS))
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group in free call\n");
        return -EINVAL;
    }

    current_state = adk_fp_ts_index_state[port_grp][ts_index];

    if(current_state == ADK_FP_TS_INDEX_ALLOCATED ||
        current_state == ADK_FP_TS_TRANSMITTED)
    {
        ret = adk_netd_ts_index_free(port, ts_index);
        /*Update the current index state to Free*/
        adk_fp_ts_index_state[port_grp][ts_index] = ADK_FP_TS_INDEX_FREE;
    }
    else
    {
        ret = -EINVAL; /*Case of spurious free call*/
    }
    return ret;
}

int
adk_netd_fp_ts_pkt_transmitted(uint8_t port, uint8_t ts_index,
  uint32_t corrl, struct rte_mbuf *dpdk_mbuf_p)
{
    int ret = 0;
    uint8_t port_grp = port >> 2;
    enum adk_fp_ts_state current_state;
    enum adk_netd_ts_message_type msg_type;
    struct rte_event ev;
    uint32_t num_prepared;

    if (unlikely(port_grp >= ADK_NETD_MAX_PORT_GROUPS))
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
        return -EINVAL;
    }
    current_state = adk_fp_ts_index_state[port_grp][ts_index];

    if(current_state == ADK_FP_TS_INDEX_ALLOCATED)
    {
        adk_dsi_rx_metadata_t *dsi_md_p = ADK_DSI_RX_METADATA_PTR(dpdk_mbuf_p);
        /* put status code in upper nibble */
        /* Only drop notifications are sent to netd*/
        msg_type = ADK_NETD_TS_TX |(ADK_NETD_TS_TX_FAIL<< 4);
        /* Update netdriver specific metadata*/
        ADK_MD_SET_TS_CORR(dpdk_mbuf_p, corrl);
        ADK_MD_SET_TS_MSG_TYPE(dpdk_mbuf_p, msg_type);
        ADK_MD_SET_TS_INDEX(dpdk_mbuf_p, ts_index);
        ADK_MD_SET_LINUX_INTF_ID(dpdk_mbuf_p, port);

        /* Update ADK dev-net device id here*/
        ADK_MD_SET_ADK_DEV_ID(dpdk_mbuf_p, adk_fp_ts_dev_net_id);
        /* Update dev-net queue id here */
        ADK_MD_SET_DEV_TX_QUEUE_ID(dpdk_mbuf_p, ADK_QOS_DEV_NET_CMSG_QUEUE_ID);

        dpdk_mbuf_p->port = port;
        dsi_md_p->port = port;

        num_prepared = adk_pal_netd_send_prepare(dpdk_mbuf_p, &ev);
        if(!num_prepared)
        {
            ret = -EINVAL;
            goto adk_netd_fp_ts_pkt_transmitted_exit;
        }

        struct rte_event *bd_event;
        bd_event= RTE_BRIDGE_ETH_MBUF_EVENT_INFO(dpdk_mbuf_p);

        bd_event->flow_id   = ev.flow_id;
        bd_event->sub_event_type = ev.sub_event_type;
        bd_event->event_type= ev.event_type;
        bd_event->sched_type= ev.sched_type;
        bd_event->queue_id  = ev.queue_id;
        bd_event->priority  = ev.priority;
        bd_event->op        =
            (RTE_BRIDGE_ETH_TX_NOTIFY_EVENT_ON_TX_FLAG |
            RTE_BRIDGE_ETH_TX_NOTIFY_EVENT_ON_DROP_FLAG);
    }
    else
    {
        ret = -EINVAL; /* case of Spurious call*/
    }
adk_netd_fp_ts_pkt_transmitted_exit:
    return ret;
}

int
adk_netd_fp_ts_fail(int port, int ts_index,
     uint32_t corrl, enum adk_fp_ts_error_code error_code)
{
    int ret = 0;
    uint8_t port_grp;
    enum adk_netd_ts_status_code status_code =
        ADK_NETD_TS_INDEX_ALLOC_SUCCESS; /*To avoid compilation error*/
    enum adk_fp_ts_state current_state;
    /*Initialize To avoid compilation error*/
    enum adk_netd_ts_message_type msg_type = ADK_NETD_TS_REPLY;

    switch (error_code)
    {
        case ADK_FP_TS_ALLOC_FAIL:
            status_code = ADK_NETD_TS_INDEX_ALLOC_FAIL;
            msg_type = ADK_NETD_TS_REPLY;
            goto adk_netd_fp_ts_fail_reply;
            break;
        case ADK_FP_TS_TX_FAIL:
            status_code = ADK_NETD_TS_TX_FAIL;
            msg_type = ADK_NETD_TS_TX;
            break;
        case ADK_FP_TS_TX_QUEUE_FAIL:
            status_code = ADK_NETD_TS_TX_QUEUE_FAIL;
            msg_type = ADK_NETD_TS_TX;
            break;
        default:
            goto adk_netd_fp_ts_fail_reply;
            break;
    }

    port_grp = port >> 2;

    if (unlikely(port_grp >= ADK_NETD_MAX_PORT_GROUPS))
    {
        ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
        return -EINVAL;
    }
    current_state = adk_fp_ts_index_state[port_grp][ts_index];
    if(current_state != ADK_FP_TS_INDEX_ALLOCATED)
        goto adk_netd_fp_ts_fail_exit;

adk_netd_fp_ts_fail_reply:
    ret = adk_netd_fp_ts_send_to_linux(msg_type,
            status_code, port, ts_index, corrl);

adk_netd_fp_ts_fail_exit:
    return ret;
}

int
adk_netd_fp_ts_index_free_count(int port)
{
    return adk_netd_ts_index_free_count(port);
}

void
adk_netd_fp_ts_index_dump(int port)
{
    return adk_netd_ts_index_dump(port);
}
#endif

