/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_netd_ts_index.c
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 * @addtogroup _sif  SIF Module
 * @{
 *     @addtogroup _sifDpFunctions  SIF Data Plane Functions
 *     @{
 *
 * @brief
 *
 * This file contains the ADK Dataplane System Configuration functions
 * for TN port timestamping index management.
 *
 *  **********************************************************************/
#ifdef ADK_DEVICE_SNR
#include <adk_netd_fp_ts_index.h>

#if 0
#include "adk_netd_defines.h"

#include <linux/device.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include "compat.h"
#include "adk_netd_dev.h"
#include "adk_netd_main.h"
#include "adk_netd_txrx.h"
#include "adk_netd_log.h"
#include "adk_netd_tstamp.h"
#include "adk_netd_ts_index.h"
#endif

struct adk_netd_ts_index_t adk_netd_ts_index_table[ADK_NETD_MAX_PORT_GROUPS];

void
adk_netd_ts_index_init(void)
{
	u8 port_grp;

	for (port_grp = 0; port_grp < ADK_NETD_MAX_PORT_GROUPS; port_grp++) {
		rte_spinlock_init(&adk_netd_ts_index_table[port_grp].lock);
		adk_netd_ts_index_table[port_grp].max_ts_index =
			ADK_NETD_MAX_TS_INDEX;
		adk_netd_ts_index_table[port_grp].start_index = 0;

		adk_netd_ts_index_table[port_grp].next_free_index =
			adk_netd_ts_index_table[port_grp].start_index;
	}
}

int
adk_netd_ts_index_alloc(int port, u8 *ts_index)
{
	u8 allocated_index = 0;
	u8 port_grp = port >> 2;
	int ret = 0, i;
	u8 start_index, end_index, cur_index;
    bool index_overflow = false;

	if (port_grp >= ADK_NETD_MAX_PORT_GROUPS) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
		return -EINVAL;
	}

	if (adk_netd_ts_index_table[port_grp].alloc_count >=
			adk_netd_ts_index_table[port_grp].max_ts_index) {
            index_overflow = true;
	}
        

	start_index = adk_netd_ts_index_table[port_grp].start_index;
	end_index = adk_netd_ts_index_table[port_grp].start_index +
		adk_netd_ts_index_table[port_grp].max_ts_index;

	/* Atomic operation starts*/
	rte_spinlock_lock(&adk_netd_ts_index_table[port_grp].lock);
	cur_index = adk_netd_ts_index_table[port_grp].next_free_index;

	for (i = start_index; i < end_index; i++) {
		if (!(adk_netd_ts_index_table[port_grp].bit_mask &
					((u64)1 << cur_index))) {
			allocated_index = cur_index;
			break;
		}
        else if(index_overflow)
        {
            allocated_index = cur_index;
            break;
        }
		cur_index++;
		adjust_index(&cur_index, start_index, end_index, port_grp);
	}

	cur_index++;
	adjust_index(&cur_index, start_index, end_index, port_grp);

	/* Store the next free index*/
	adk_netd_ts_index_table[port_grp].next_free_index = cur_index;
	/* Increment alloc count*/
    if(!index_overflow)
    {
	    adk_netd_ts_index_table[port_grp].alloc_count++;
    }
	/* update bitmask*/
	adk_netd_ts_index_table[port_grp].bit_mask |=
			((u64)1 << allocated_index);
	/* Atomic operation ends */
	rte_spinlock_unlock(&adk_netd_ts_index_table[port_grp].lock);
	*ts_index = allocated_index;
	return ret;
}

int
adk_netd_ts_index_free(int port, u8 ts_index)
{
	int ret = 0;
	u8 start_index, end_index;
	u8 port_grp = port >> 2;

	if (port_grp >= ADK_NETD_MAX_PORT_GROUPS) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group in free call\n");
		return -EINVAL;
	}
	start_index = adk_netd_ts_index_table[port_grp].start_index;
	end_index = adk_netd_ts_index_table[port_grp].start_index +
	    adk_netd_ts_index_table[port_grp].max_ts_index;

	if ((ts_index < start_index) || (ts_index >= end_index)) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid ts index in free call\n");
		return -EINVAL;
	}

	/* Atomic operation starts*/
	rte_spinlock_lock(&adk_netd_ts_index_table[port_grp].lock);
	/* Clear the corresponding bit in bitmask */
	adk_netd_ts_index_table[port_grp].bit_mask &= ~(((u64)1 << ts_index));
	adk_netd_ts_index_table[port_grp].alloc_count--;
	/* Atomic operation ends */
	rte_spinlock_unlock(&adk_netd_ts_index_table[port_grp].lock);

	return ret;
}

int
adk_netd_ts_index_is_allocated(int port, u8 ts_index,
		bool *allocated)
{
	u8 port_grp = port >> 2;
	u8 start_index, end_index;

	if (port_grp >= ADK_NETD_MAX_PORT_GROUPS) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
		return -EINVAL;
	}
	start_index = adk_netd_ts_index_table[port_grp].start_index;
	end_index = adk_netd_ts_index_table[port_grp].start_index +
	    adk_netd_ts_index_table[port_grp].max_ts_index;

	if ((ts_index < start_index) || (ts_index >= end_index)) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid ts index\n");
		return -EINVAL;
	}

	if (adk_netd_ts_index_table[port_grp].bit_mask &
		(((u64)1 << ts_index)))
		*allocated = true;
	else
		*allocated = false;

	return 0;
}

int
adk_netd_ts_index_free_count(int port)
{
	u8 port_grp = port >> 2;

	if (port_grp >= ADK_NETD_MAX_PORT_GROUPS) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
		return -EINVAL;
	}
	return (adk_netd_ts_index_table[port_grp].max_ts_index -
		adk_netd_ts_index_table[port_grp].alloc_count);
}

void
adk_netd_ts_index_dump(int port)
{
	u8 port_grp = port >> 2;

	if (port_grp >= ADK_NETD_MAX_PORT_GROUPS) {
		ADK_DP_LOG_ERROR(ADK_SYS_MODULE_ID,"Invalid port group\n");
		return;
	}

	printf(" bit mask for port group %d: 0x%lx\n",
		port_grp, adk_netd_ts_index_table[port_grp].bit_mask);
}

#if 0
void
adk_netd_ts_index_test(void)
{
	int i;
	u8 ts_index;
	bool allocated = 0;
	int ret;

	for (i = 0; i < 65; i++) {
		ret = adk_netd_ts_index_alloc(0, &ts_index);
		pr_info("allocated index: %d, ret :%d\n", ts_index, ret);
	}
	pr_info("free count: %d\n", adk_netd_ts_index_free_count(0));
	adk_netd_ts_index_dump(0);
	adk_netd_ts_index_is_allocated(0, 44, &allocated);
	pr_info("index 44 allocate status on port 0: %d\n", allocated);
	adk_netd_ts_index_is_allocated(0, 66, &allocated);
	pr_info("index 66 allocate status on port 0: %d\n", allocated);
	adk_netd_ts_index_free(0, 44);
	adk_netd_ts_index_free(0, 66);
	pr_info("free count: %d\n", adk_netd_ts_index_free_count(0));
	adk_netd_ts_index_dump(0);
	adk_netd_ts_index_is_allocated(0, 44, &allocated);
	pr_info("index 44 allocate status on port 0: %d\n", allocated);
}
#endif
#endif

