/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

/*!
 * @file  adk_dev_net_internal.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of ADK Linux Device 
 * interface definition (APIs and data structure definitions).
 *
 ***********************************************************************/

#ifndef __ADK_DEV_NET_INTERNAL_H__
#define __ADK_DEV_NET_INTERNAL_H__

#include <rte_eventdev.h>
#include <rte_bridge_cppi.h>
#include "adk_dev_net.h"

#ifdef __cplusplus
extern "C" {
#endif

#define ADK_DEV_NET_COOKIE_SIZE   16
#define ADK_DEV_NET_COOKIE        "ADK_LNX_DEV"

#define ADK_DEV_NET_IDS_MAX    (8 + ADK_PH_DEV_MAX_NUM_DEVICES)
#define ADK_DEV_NET_ID_MAX     (RTE_MAX_ETHPORTS + ADK_DEV_NET_IDS_MAX)

#define ADK_DEV_NET_ID_RSVD_MIN  0
#define ADK_DEV_NET_ID_RSVD_MAX  ((RTE_MAX_ETHPORTS-1))
#define ADK_DEV_NET_ID_RSVD_MAX_PH  ((RTE_MAX_ETHPORTS-1) + (ADK_PH_DEV_MAX_NUM_DEVICES))
#define ADK_DEV_NET_ID_BASE      (ADK_DEV_NET_ID_RSVD_MAX_PH + 1)

#define ADK_DEV_NET_CPPI_DEV_BASE_ID ADK_PH_DEV_MAX_NUM_DEVICES

#define ADK_DEV_NET_SDM_RX_QUEUES_MAX 8
#define ADK_DEV_NET_SDM_TX_QUEUES_MAX 8

#define ADK_DEV_NET_N_QUEUES_MAX (8 + 1)
#define ADK_DEV_NET_QUEUE_SIZE_MAX  1024


#define ADK_DEV_NET_CPPI_BURST_SIZE         16
#define ADK_DEV_NET_CPPI_QUEUE_CACHE_SZ     64

#if RDK_REL_MAJOR >= 13
#define ADK_DEV_NET_CPPI_FQ_ID_BASE         0
#define ADK_DEV_NET_CPPI_AQ_ID_BASE         0
#define ADK_DEV_NET_CPPI_RXQ_ID_BASE        0
#define ADK_DEV_NET_CPPI_TXQ_ID_BASE        0
#else
#define ADK_DEV_NET_CPPI_FQ_ID_BASE         0
#define ADK_DEV_NET_CPPI_AQ_ID_BASE         1
#define ADK_DEV_NET_CPPI_RXQ_ID_BASE        2
#define ADK_DEV_NET_CPPI_TXQ_ID_BASE        3
#endif

/*!
 * adk_dev_net_queue_info
 *
 */
typedef struct adk_dev_net_queue_info_s {
    uint16_t                 queue_id;
    uint8_t                 setup_done;
    uint8_t                 priority;
    uint8_t                 weight;
    uint16_t                size;
    uint16_t                cppi_device_id;
    uint16_t                cppi_queue_id;
#if RDK_REL_MAJOR >= 13
    struct rte_cppi_queue_info cppi_info;
#else
    struct rte_bridge_cppi_queue_info cppi_info;
#endif
} adk_dev_net_queue_info_t;

/*!
 * adk_dev_net_pvt_info
 *
 */
typedef struct adk_dev_net_pvt_info_s {
    char                        cookie_name[ADK_DEV_NET_COOKIE_SIZE];
    uint16_t                     device_id;
    uint8_t                     device_id_internal;
    uint8_t                     device_configured;
    uint8_t                     device_started;
    uint16_t                     n_rx_queues;
    uint16_t                     n_tx_queues;
    int32_t                     adk_netd_fd;
    adk_dev_net_sched_mode_t  rx_sched_mode;
    adk_dev_net_sched_mode_t  tx_sched_mode;
    adk_dev_net_info_t        info;
    adk_dev_net_queue_info_t  rx_q_info[ADK_DEV_NET_N_QUEUES_MAX];
    adk_dev_net_queue_info_t  tx_q_info[ADK_DEV_NET_N_QUEUES_MAX];
    adk_dev_net_queue_info_t  aq_info;
    adk_dev_net_queue_info_t  fq_info;
} adk_dev_net_pvt_info_t;

/*
 * adk_dev_net_init()
 * @param[in] sysHandle    ADK System Handle
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * @brief API to the ADK Slow Path Device init functionality
 */
adk_status_t
adk_dev_net_init(adk_sys_handle_t sys_handle);

/*
 * adk_dev_net_shutdown()
 * @param[in] sysHandle    ADK System Handle
 *
 * @return Returns ADK_SUCCESS if success, error code otherwise.
 *
 * @brief API to the ADK Slow Path Device shutdown functionality
 */
adk_status_t
adk_dev_net_shutdown(adk_sys_handle_t sys_handle);




#ifdef __cplusplus
}
#endif

#endif /* __ADK_DEV_NET_INTERNALH__ */

/*! end addtogroup _sif
 * @}
 */
