/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @file  adk_em.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 *  This file contains EM interoperability codes. This file should not
 *  be directly included in the source C file.
 *
 *  **********************************************************************/
#ifndef __ADK_EM_H__
#define __ADK_EM_H__
#include "adk_sif_constants.h"
#include "adk_platform.h"
#include "adk_sys_lcore.h"
#include "adk_sif_cfg_pipeline.h"
#include "adk_dpdk.h"

#include "pdk_include.h"

#include "adk_pktio.h"

/*Need to set it to a proper value*/
#define EM_TO_ADK_IP_Q 7

#define ADK_EM_UDP_GEN_CONFIG_GET(mbuf) ((struct adk_sys_event_metadata *)em_event_to_nonem_param (em_rte_mbuf_to_event(mbuf)))->udp_cksum_gen

typedef struct adk_sif_em_dp_lli_meta_data_s
{
    uint32_t eventMachineQueueId;
}adk_sif_em_dp_lli_meta_data_t;
/**
 * API to send rte_mbuf to EM
 *
 * @brief This function converts rte_mbuf to event_t and send the
 * event to dest EM queue
 *
 * @param mbuf_p       rte_mbuf pointer that need to be send to EM
 *
 * @param dest         destination EM queue ID
 *
 * @return ADK_SUCCESS if successful.
 *
 */
adk_status_t adk_em_send_to_eo(struct rte_mbuf *mbuf, uint32_t dest );

/**
 * Api to receive events from EM
 *
 * @brief This function is registered to PAL infrastructure and
 * 	receive events from EM. On receiving the event, this will pass
 *  the event to ADK entry point
 *
 * @param task_p	task pointer
 *
 * @return ADK_SUCCESS  if successful.
 *
 */
adk_status_t adk_sys_em_demux_fn_id(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

/**
 * Api to receive IPv6 events from EM
 *
 * @brief This function is registered to PAL infrastructure and
 * 	receive events from EM. On receiving the event, this will pass
 *  the event to ADK entry point
 *
 * @param task_p	task pointer
 *
 * @return ADK_SUCCESS  if successful.
 *
 */
adk_status_t adk_sys_em_ipv6_demux_fn_id(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

/**
 * API to attach ADK app to Event Machine
 *
 * @brief This api attach the ADK instance to EM instance.
 *
 *
 * @param config    EM runtime config options
 *
 * @return ADK_SUCCESS if successful.
 *
 */
adk_status_t adk_em_attach(struct nonem_config config);

/**
 * API to get iparam from rte_mbuf
 *
 * @brief This api will fetch the iparam pointer from rte_mbuf.
 *
 * @param mbuf    rte_mbuf. From which iparam is fetched.
 *
 * @return iparam pointer
 *
 */
void* adk_em_mbuf_to_iparam( struct rte_mbuf *mbuf );

/**
 * Api to send events to EM
 *
 * @brief This function is registered to PAL infrastructure and
 *  receive events from ADK IP/IPv6 Modules. On receiving the event, this will pass
 *  the event to EM
 *
 * @param task_p    task pointer
 *
 * @return ADK_SUCCESS  if successful.
 *
 */

adk_status_t adk_sys_em_pkt_proc_lli(void **task_p,
        uint8_t task_count,
        uint32_t task_mask,
        void *ring,
        adk_sif_cfg_pipeline_info_t *next_stage_entry,
        int coreIndex);

void adk_em_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#endif /* #ifndef _ADK_EM_H_ */

	/*! end addtogroup _sif
 * @}
 */
