/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_sif_dp.h
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of objects that are
 * part of system infrastructure data plane.
 *
 *
 ***********************************************************************/

#ifndef __ADK_SIF_DP_H__
#define __ADK_SIF_DP_H__

#include "adk_sif_constants.h"
#include "adk_platform.h"
#include "adk_sys_lcore.h"
#include "adk_sif_cfg_pipeline.h"
#include "adk_sys_dp_flow_cache.h"
#include "adk_pktio.h"
/* #include <adk_include.h> */
#include <adk_rcu.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <adk_misc.h>

/* NTP related constant */
/*! Offset between NTP base and UTC base */
/* Number of seconds from 1st January 1900 to start of
 * Unix epoch which is 1st January 1970
 */
#define ADK_SIF_NTP_BASETIME                  2208988800ul
/*! NTP scaling fraction */
#define ADK_SIF_NTP_SCALE_FRAC                ((uint64_t)1 << 32)

/* Number of entries for the DSCP/PCP mapping tree */
#define DSCP_PCP_MAP_PROF_ID_NUM_ENTRIES    (1 << (ADK_CP_PROFILE_ID_WIDTH))
#define DSCP_PCP_MAP_TYPE_NUM_ENTRIES       (1 << (ADK_CP_INPUT_TYPE_WIDTH))
#define DSCP_PCP_MAP_INP_VALUE_NUM_ENTRIES  (1 << (ADK_CP_INPUT_VALUE_WIDTH))

/* Values for indexing into dscpPcpMapTree array */
#define DSCP_INDEX  0
#define PCP_INDEX   1


/* Packet sent by ethernet and it is outer packet */
#define IPLLI_IMC_ETH_OUTER 0x0
/* Packet sent by ipsec and it is outer packet i.e. not decypted (could be plaintext or has bypass policy */
#define IPLLI_IMC_IPSEC_OUTER 0x1
/* Packet sent by ipsec. This is an inner packet and decryption was done inline by HLP */
#define IPLLI_IMC_IPSEC_INNER_INLINE 0x2
/* Packet sent by ipsec. This is an inner packet and decryption was done by ADK IPSec module using look-aside mechanism */
#define IPLLI_IMC_IPSEC_INNER_LOOKASIDE 0x4
/** end of IP LLI IMC: ipsec_data values **/

/* Flags indicating the Path of the packet */
#define IPSEC_NOT_VISITED 0x0
#define IPSEC_DONE_PLAINTEXT_PACKET 0x1
#define IPSEC_DONE_PACKET_DECRYPTED 0x2 

#define INC_NON_NULL(pointer, member, count) do{\
if(pointer) \
    pointer->member += count;\
}while(0);

/*!
 * =============================================================
 * @internal
 * @struct adk_sif_dp_dscp_pcp_new_value_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @brief Return value from the DSPC PCP mapping tree
 *
 * =============================================================
 */
typedef struct adk_sif_dp_dscp_pcp_new_value_s {
    uint8_t dscp;
    uint8_t dscpDefault;
    uint8_t pcp;
    uint8_t pcpDefault;
} adk_sif_dp_dscp_pcp_new_value_t;   /* ------  end of struct adk_sif_dp_dscp_pcp_new_value_t  ------ */

/*!
 * =============================================================
 * @internal
 * @struct adk_sif_dp_module_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @brief Stores the log level per module
 *
 * =============================================================
 */
typedef struct adk_sif_dp_module_s {
    uint32_t    logLevel;
    char       *name_p;
} adk_sif_dp_module_t;


/*!
 * =============================================================i
 * @internal
 * @struct adk_sif_base_ntp_info_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @brief
 *
 * Structure to hold base NTP linux time and corresponding CPU time
 *
 * =============================================================
 */
typedef struct adk_sif_base_ntp_info_s
{
    uint32_t            baseNtpTimeHi;
    uint32_t            baseNtpTimeLo;
    /* Linux epoch time equivalent CPU ticks. It is a relative time */
    uint64_t            epochCpuTicks;
} adk_sif_base_ntp_info_t;

/**
 * @internal 
 * @struct adk_sif_module_egress_cb_fn_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @param m
 *   Pointer to struct rte_mbuf
 * @param data
 *  Callback arg
 * @return
 *   - 0: Success
 *   - <0: Error code on failure
 */
typedef void (*adk_sif_module_egress_cb_fn_t)(struct rte_mbuf *m,
                                              void *data);

/**
 * @internal 
 * @struct adk_sif_module_cppi_cb_fn_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @param m
 *   Pointer to struct rte_mbuf
 * @param data
 *  Callback arg
 * @return
 *   - 0: Success
 *   - <0: Error code on failure
 */
typedef void (*adk_sif_module_slowpath_cb_fn_t)(void *data);

typedef struct adk_sif_dp_scratchpad_space_s{
    adk_sys_dp_uli_flow_cache_entry_t *flow_cache_entry;
    uint8_t pktPath;
    uint8_t ip_version;
    uint8_t ip_offset;
    uint8_t pktOrigin;
    uint8_t l4_offset;
    uint16_t mtu;
    uint8_t data[50];
}adk_sif_dp_scratchpad_space_t;

/*!
 * =============================================================i
 * @internal
 * @struct adk_sif_dp_shm_info_t adk_sif_dp.h "adk_sif_dp.h"
 *
 * @brief
 *
 * Aggregate handles and global states that are shared by all
 * components of SIF in the DP.
 *
 * =============================================================
 */
typedef struct adk_sif_dp_shm_info_s
{
    uint32_t            cookie;
    adk_sif_dp_module_t module[ADK_MAX_SUPPORTED_MODULES];
    uint32_t            sysLogLevel;
    /* Following elements are required for NTP timestamp conversions */
    uint64_t            cpuTicksPerSec;
    /*
     * For Resync, a new pointer can be atomically updated. The old pointer can
     * then be freed after RCU sync.
     */
    rte_atomic64_t  baseTime;

    /* Tree for mapping DSCP or PCP to PCP or DSCP */
    adk_sif_dp_dscp_pcp_new_value_t
                        dscpPcpMapTree[DSCP_PCP_MAP_PROF_ID_NUM_ENTRIES]
                                      [DSCP_PCP_MAP_TYPE_NUM_ENTRIES]
                                      [DSCP_PCP_MAP_INP_VALUE_NUM_ENTRIES];

    /* Map from Packet Capture ID to Entry Function ID */
    uint32_t pcapIdEntryFnIdMap[ADK_SIF_PCAP_ID_MAX];
    adk_sif_module_egress_cb_fn_t moduleEgressCbFn[ADK_MAX_SUPPORTED_MODULES];
    adk_sif_module_slowpath_cb_fn_t moduleSlowPathCbFn[ADK_MAX_SUPPORTED_MODULES];
    adk_sys_dp_uli_flow_cache_table_t sys_dp_uli_flow_cache_tbl[ADK_MAX_NUM_CORES]; 
    adk_sif_dp_scratchpad_space_t  sys_dp_scratchpad[ADK_MAX_NUM_CORES];
    bool udpValidateFlag;
    bool udpComputeFlag;
    bool ipsecBypass;
    bool flowCacheCountersFlag;
    uint8_t fc_ver; /*Flow cache version*/
} adk_sif_dp_shm_info_t;

#define ADK_SYS_UDP_VALIDATE_FLAG_GET \
    (ADK_SIF_DP_INFO_PTR_GET->udpValidateFlag)

#define ADK_SYS_UDP_GEN_CONFIG_GET \
        (ADK_SIF_DP_INFO_PTR_GET->udpComputeFlag)

#define ADK_SYS_IPSEC_BYPASS_GET \
        (ADK_SIF_DP_INFO_PTR_GET->ipsecBypass)

#define ADK_SYS_FLOW_CACHE_COUNTERS_ENABLED \
        (likely(ADK_SIF_DP_INFO_PTR_GET->flowCacheCountersFlag))
/*!
 * =============================================================
 * @internal
 * @struct adk_sif_ntp_time_t adk_sif.h "adk_sif.h"
 *
 * @brief Structure to hold NTP Timestamp
 *
 *
 * =============================================================
 */
typedef struct adk_sif_ntp_time_s {
    uint32_t ntpSeconds;
    uint32_t ntpFractionSeconds;
}adk_sif_ntp_time_t;


typedef enum adk_sif_dp_code_point_type_e
{
    ADK_SIF_DP_CODE_POINT_TYPE_DSCP = 1,
    ADK_SIF_DP_CODE_POINT_TYPE_PCP
} adk_sif_dp_code_point_type_t;

typedef struct adk_sif_code_point_mapped_values_s
{
    int8_t  dscpOut;
    int8_t  pcpOut;
} adk_sif_code_point_mapped_values_t;


#define SYS_DP_HANDLE_COOKIE  0x6e696071

/*SIF DP table pointer*/
extern adk_sif_dp_shm_info_t   *adkSifDpShm_p;

/* Is system handle invalid */
#define ADK_INVALID_SYS_DP_HANDLE \
    (((adkSifDpShm_p) == 0) || ( ! ADK_SIF_DP_HDL_COOKIE_VALID))

/* useful util macros that are tied to the sys_info data structure */
#define ADK_SIF_DP_LOG_LEVEL_GET \
    (adkSifDpShm_p->sysLogLevel)

#define ADK_SIF_DP_LOG_LEVEL_SET(_newlogLevel) \
    (adkSifDpShm_p->sysLogLevel = (_newlogLevel))

#define ADK_SIF_DP_HDL_COOKIE_VALID \
    (adkSifDpShm_p->cookie == SYS_HANDLE_COOKIE)

#define ADK_SIF_DP_HDL_COOKIE_SET \
    (adkSifDpShm_p->cookie = SYS_HANDLE_COOKIE)

#define ADK_SIF_DP_HDL_COOKIE_CLEAR \
    (adkSifDpShm_p->cookie = 0)

#define ADK_SIF_DP_MODULE_LOG_LEVEL_SET(_modId, _logLevel) \
    (adkSifDpShm_p->module[(_modId)].logLevel = (_logLevel))

#define ADK_SIF_DP_MODULE_LOG_LEVEL_GET(_modId)  \
    (adkSifDpShm_p->module[(_modId)].logLevel)

#define ADK_SIF_DP_MODULE_NAME_SET(_modId, _name)  \
    (adkSifDpShm_p->module[(_modId)].name_p = (_name))

#define ADK_SIF_DP_MODULE_NAME_GET(_modId)  \
    (adkSifDpShm_p->module[(_modId)].name_p)

#define ADK_SIF_DP_GET_SCRATCHPAD_PTR(_coreId)  \
    (&adkSifDpShm_p->sys_dp_scratchpad[(_coreId)])

adk_status_t
adk_sys_dp_pcap_demux(
    void **task_p,
    uint8_t task_count,
    uint32_t task_mask,
    void *ring,
    adk_sif_cfg_pipeline_info_t *next_stage_entry,
    int coreIndex);

adk_status_t 
adk_sys_dp_tund_ip_demux(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

adk_status_t 
adk_sys_dp_bridge_demux(void **task_p,
          uint8_t task_count,
          uint32_t task_mask,
          void *ring,
          adk_sif_cfg_pipeline_info_t *next_stage_entry,
          int coreIndex);

adk_dp_status_t
adk_sif_get_ntp_time( uint64_t cpuTicks,
                      adk_sif_ntp_time_t *ntpTime);

adk_status_t
adk_sif_dp_code_point_mapping_get(
    uint8_t                              profileId,
    adk_sif_dp_code_point_type_t         type,
    uint8_t                              inputValue,
    adk_sif_code_point_mapped_values_t  *outputValues);

void
adk_sys_dp_scratchpad_dump(adk_sif_dp_scratchpad_space_t *scratch_pad_p);

void adk_sif_dp_pal_batch_cb_fn(
        adk_pktio_task_t *dpdkMbuf,
        uint8_t       cbEventType,
        void          *cbParam,
        bool          sendSuccess);

#ifdef __cplusplus
}
#endif

#endif /* __ADK_SIF_DP_H__ */

/*! end addtogroup _sif
 * @}
 */
