/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_sys_dp_flow_cache.h 
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of flow cache objects that are
 * part of system infrastructure data plane.
 *
 *
 ***********************************************************************/

#ifndef __ADK_SYS_DP_FLOW_CACHE_H__
#define __ADK_SYS_DP_FLOW_CACHE_H__

#include "adk_platform.h"
#include "adk_sys_lcore.h"
#include "rte_ip.h"

#ifdef __cplusplus
extern "C" {
#endif

/*Flowcache definitions*/
#define ADK_SYS_NUM_FLOW_CACHE_COL_ENTRIES          4
#define ADK_SYS_FLOW_CACHE_AGING_TIMER              10000 /* in msec */
#define ADK_SYS_FLOW_CACHE_AGING_ITER               4 /* 4 iterations to complete entire FLOW cache traversing */


/* Number of ULI Flow cache entries */
#define ADK_SYS_ULI_FLOW_CACHE_WIDTH                11
#define ADK_SYS_ULI_FLOW_CACHE_ENTRIES              (1<<ADK_SYS_ULI_FLOW_CACHE_WIDTH) /* shall be power of 2 */
#define ADK_SYS_ULI_FLOW_CACHE_MASK                 (ADK_SYS_ULI_FLOW_CACHE_ENTRIES-1)
#define ADK_SYS_ULI_FLOW_CACHE_AGING_THRESHOLD      (ADK_SYS_ULI_FLOW_CACHE_ENTRIES>>2) /* number of free entries left */
#define ADK_SYS_ULI_FLOW_CACHE_AGING_SEG_SIZE       (ADK_SYS_ULI_FLOW_CACHE_ENTRIES>>8) /* */

#define MAX_VALUE(_size) ((((unsigned long long)1 << (_size) * CHAR_BIT)) - 1)

#define ADK_SYS_DP_FLOW_CACHE_SET_PARAM(dst, src) \
        (dst = src)

#define ADK_SYS_DP_IS_FLOW_CACHE_ACTIVE(state) \
        (state == ADK_SYS_DP_FLOW_CACHE_ACTIVE)

#define ADK_SYS_DP_FREE_IDX(list, tbl, index) \
    do{ \
        if(!(list).count) \
        { \
            (list).headIdx = index; \
            (list).tailIdx = index; \
        } \
        else \
        { \
            tbl[(list).tailIdx].nextIdx = tbl[(index)].idx; \
            (list).tailIdx = index; \
        } \
        tbl[(index)].nextIdx = UINT16_MAX; \
        (list).count++; \
    }while(0);


#define ADK_SYS_DP_RSRV_IDX(free_list, tbl, index) \
        do{ \
            if((free_list).count) \
            { \
                (index) = (free_list).headIdx; \
                (free_list).headIdx = tbl[(index)].nextIdx; \
                (free_list).count--; \
            } \
            else \
            { \
                return ADK_DP_E_ERROR; \
            } \
        }while(0);

#define ADK_SYS_DP_TABLE_IDX_INIT(dp_tbl, num_entries) \
    do { \
        uint32_t ne; \
        for(ne=0; ne<(num_entries-1); ne++) \
        { \
            dp_tbl.tbl[ne].idx = ne; \
            dp_tbl.tbl[ne].nextIdx = ne+1; \
        } \
        dp_tbl.tbl[ne].idx = ne; \
        dp_tbl.tbl[ne].nextIdx = MAX_VALUE(sizeof(dp_tbl.tbl[ne].idx)); \
        dp_tbl.free_list.headIdx = 0; \
        dp_tbl.free_list.tailIdx = num_entries-1; \
        dp_tbl.free_list.count = num_entries; \
        dp_tbl.pending_list.headIdx = MAX_VALUE(sizeof(dp_tbl.tbl[ne].idx)); \
        dp_tbl.pending_list.tailIdx = MAX_VALUE(sizeof(dp_tbl.tbl[ne].idx)); \
        dp_tbl.pending_list.count = 0; \
    }while(0);

#define ADK_SYS_DP_FC_RSRV_IDX(free_list, tbl, index) \
        ADK_SYS_DP_RSRV_IDX(free_list, tbl, index)


#define ADK_SYS_DP_FC_RELEASE_IDX(free_list, tbl, index)\
        ADK_SYS_DP_FREE_IDX(free_list, tbl, index)\

#define ADK_SYS_ULI_FC_INCR(cnt) \
   (adkSifDpShm_p->sys_dp_uli_flow_cache_tbl[coreIndex].uli_fc_stats).cnt++; 

static inline void adk_sys_dp_fc_bitmap_set(uint64_t *array, uint16_t index)
{
    uint64_t bit_set = 1;
    array[(index>>6)] |= (bit_set<<((index)&0x3F)); 
    return;
}

static inline void adk_sys_dp_fc_bitmap_reset(uint64_t *array, uint16_t index)
{
    uint64_t bit_reset = 1;
    array[(index>>6)] &= (~(bit_reset<<((index)&0x3F)));
    return;
}

typedef enum adk_ip_hdr_type_s{
    ADK_IPV4 = 0,
    ADK_IPV6,
    ADK_INVALID_HDR_TYPE,
}adk_ip_hdr_type_e;

typedef struct adk_ip_hdr_s{
    union{
        struct ipv4_hdr v4Hdr;
        struct ipv6_hdr v6Hdr;
    };
}adk_ip_hdr_t;

typedef struct adk_sys_ip_addr_s{
    union {
        struct ipv4_addr_t{
            uint32_t srcAddr;
            uint32_t dstAddr;
        }v4Addr;
        struct ipv6_addr_t{
            uint32_t srcAddr[4];
            uint32_t dstAddr[4];
        }v6Addr;
    };
}adk_sys_ip_addr_t;

#define ADK_SYS_DP_FLOW_CACHE_INIT 0
#define ADK_SYS_DP_FLOW_CACHE_NEW 1
#define ADK_SYS_DP_FLOW_CACHE_ACTIVE 2
#define ADK_SYS_DP_FLOW_CACHE_STALE 3
#define ADK_SYS_DP_FLOW_CACHE_RELEASED 4

/********************************************************************************
 *                  Fast path flow cache table structures                             *
 ********************************************************************************/
typedef struct adk_sys_eth_dp_uli_flow_cache_entry_s{
 /*Ethernet Action candidates Start*/
    uint8_t                 destMac[6]; /*TODO use MACRO instead of magic number*/
    uint8_t                 srcMac[6]; /*TODO use MACRO instead of magic number*/
    uint16_t                l2Domain;
    uint16_t                vlanId;
    uint16_t                ethIntfStatsId;
    uint16_t                ethDevId;
    uint8_t                 neighStatus:2;
    uint8_t                 pcpVal:3;       /*For DSCP to PCP mapping*/
    uint8_t                 isV6:1;         /*For Ethtype encap*/
    uint8_t                 priorityTag:1;  /*Valid only for bridge interface*/
    uint8_t                 reserved4:1;
    bool                    *haveInflight;
    adk_handle_t            ifHandle;
    /*Ethernet Action candidates End*/
}adk_sys_eth_dp_uli_flow_cache_entry_t;

typedef struct adk_sys_dp_uli_flow_cache_entry_s{
    uint32_t                idx;
    uint32_t                nextIdx;
    uint8_t                 isActive:1;
    uint8_t                 state:3;
    uint8_t                 reserved1:4;
    /*IPsec Action candidates Start*/
    /*SPD Out Action fields*/
    uint16_t                polIndex;
    uint16_t                pathMtu;
    uint16_t                SAId;
    uint8_t                 polAction:2;
    uint8_t                 eSAstate:1;
    uint8_t                 preFragEnabled:1;
    uint8_t                 tunnelMode:1;
    uint8_t                 espProto:1;
    uint8_t                 useSingleSAInPrefrag:1;
    uint8_t                 nonInitialESAstate:1;
    uint8_t                 configDF:2;
    uint8_t                 nonInitialpolAction:2;
    uint8_t                 nonInitialConfigDF:2;
    uint8_t                 reserved2:2;
    uint16_t                nonIntialPolIndex;
    uint16_t                nonInitialSAId;
    /*IPsec Action candidates End*/

    /*IP Action candidates Start*/
    adk_sys_ip_addr_t       ipAddr;
    uint32_t                nextHop;
    uint16_t                srcPort;
    uint16_t                dstPort;
    uint16_t                routeTblId;
    uint16_t                interfaceStatsId;
    uint16_t                outerSipStatsId;
    uint16_t                innerSipStatsId;
    uint16_t                egressIntfId;
    uint16_t                routeMtu;
    uint8_t                 nextHdr;
    uint8_t                 dscp;
    uint16_t                dev_tx_queue_id;
    uint8_t                 pktColor;
    uint8_t                 isOriginated:1;
    uint8_t                 isForwardingEnabled:1;
    uint8_t                 isQoSEnabled:1;
    uint8_t                 isInnerIPv6:1;
    /*Tx queue - Either a PHB queue or SL queue
      true -  PHB queue
      false - SL queue*/
    uint8_t                 isPHBQueue:1;
    uint8_t                 reserved3:3;
    /*IP Action candidates End*/
    
    /*Ethernet Action candidates Start*/
    adk_sys_eth_dp_uli_flow_cache_entry_t ethActionParams;
    /*Ethernet Action candidates End*/

    /*Mbuf fields required to be set*/
    uint64_t                 ol_flags;
    /* required for flow cache aging */
    uint8_t                 version;
    uint16_t                hashIdx;
    uint8_t                 eth_version;
    uint16_t                qosStatsId;
}__attribute__ ((aligned(64))) adk_sys_dp_uli_flow_cache_entry_t;

typedef struct adk_sys_dp_uli_flow_cache_entry_stats_s{
    uint32_t                alloc;
    uint64_t                hit;
}adk_sys_dp_uli_flow_cache_entry_stats_t;

typedef struct adk_sys_dp_array_list_s
{
    uint32_t    headIdx;
    uint32_t    tailIdx;
    uint32_t    count;
}adk_sys_dp_array_list_t;

typedef struct adk_sys_dp_flow_cache_hash_entry_s
{
    uint32_t hashValue;
    uint32_t isActive:1;
    uint32_t reserved:15;
    uint32_t index:16;
}adk_sys_dp_flow_cache_hash_entry_t;

typedef struct adk_sys_dp_uli_flow_cache_aging_info_s{
    uint32_t time_in_msec;
    uint16_t prev_fc_idx;

    uint64_t active_flows[ADK_SYS_ULI_FLOW_CACHE_ENTRIES>>6];
    uint64_t aging_bitmap[ADK_SYS_ULI_FLOW_CACHE_ENTRIES>>6];
}adk_sys_dp_uli_flow_cache_aging_info_t;

typedef struct adk_sys_dp_flow_cache_stats_s{
    uint64_t                hit;
    uint64_t                cleared;
    uint64_t                miss;
    uint64_t                alloc;
    uint64_t                freed;
    uint32_t                flows_active;
    uint32_t                flows_stale;
}__attribute__ ((aligned(64))) adk_sys_dp_flow_cache_stats_t;

/*
 * IP fast-path flow-cache table
 */
typedef struct adk_sys_dp_uli_flow_cache_table_s
{
    adk_sys_dp_flow_cache_hash_entry_t          *hash_tbl;
    adk_sys_dp_uli_flow_cache_entry_t           *tbl;
    adk_sys_dp_uli_flow_cache_entry_stats_t     *fc_entry_stats;
    adk_sys_dp_array_list_t                     pending_list;
    adk_sys_dp_array_list_t                     free_list;
    adk_sys_dp_uli_flow_cache_aging_info_t      age_info;
    adk_sys_dp_flow_cache_stats_t               stats;
}adk_sys_dp_uli_flow_cache_table_t;


#ifdef __cplusplus
}
#endif

#endif /*__ADK_SYS_DP_FLOW_CACHE_H__*/ 

/*! end addtogroup _sif
 * @}
 */
