/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
 * @internal
 * @file  adk_sys_dp_flow_cache_uli_func.h 
 *
 *      $HeadURL$
 *      $Revision$
 *      $Date$
 *
 * @addtogroup _sif  SIF Module
 *
 * @{
 *
 * @brief
 *
 * This file contains definitions and declarations of flow cache objects that are
 * part of system infrastructure data plane.
 *
 *
 ***********************************************************************/

#ifndef __ADK_SYS_DP_FLOW_CACHE_ULI_FUNC_H__
#define __ADK_SYS_DP_FLOW_CACHE_ULI_FUNC_H__

#include "adk_platform.h"
#include "adk_sys_lcore.h"
#include "adk_sys_dp_flow_cache.h"

#ifdef __cplusplus
extern "C" {
#endif

adk_dp_status_t
adk_sys_dp_uli_flow_cache_entry_alloc(
        adk_sys_dp_flow_cache_hash_entry_t **uli_flow_cache_hash_entry,
        adk_sys_dp_uli_flow_cache_entry_t **flow_cache_entry,
        uint32_t hash_value);

adk_status_t
adk_sys_dp_uli_flow_cache_entry_release(adk_sys_dp_uli_flow_cache_entry_t *uli_fc_entry);

inline adk_dp_status_t
adk_sys_dp_uli_flow_cache_lookup(adk_ip_hdr_t *ipHdr,
                                 adk_ip_hdr_type_e hdrType,
                                 uint8_t nextHdr,
                                 uint16_t srcPort, 
                                 uint16_t dstPort, 
                                 adk_sys_dp_uli_flow_cache_entry_t **flow_cache_entry,
                                 uint32_t *hash_key);

inline adk_dp_status_t
adk_sys_dp_uli_flow_cache_get(adk_ip_hdr_t *ipHdr,
                                 adk_ip_hdr_type_e hdrType,
                                 uint8_t nextHdr,
                                 uint16_t srcPort, 
                                 uint16_t dstPort, 
                                 adk_sys_dp_uli_flow_cache_entry_t **flow_cache_entry,
                                 uint32_t *hash_key);
void
adk_sys_dp_flow_cache_entry_dump(adk_sys_dp_uli_flow_cache_entry_t *flow_cache_entry);

void
adk_sys_dp_flow_cache_monitor(void);
#ifdef __cplusplus
}
#endif

#endif /*__ADK_SYS_DP_FLOW_CACHE_H__*/ 

/*! end addtogroup _sif
 * @}
 */
