/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/

#ifndef __ADK_CKSUM_H
#define __ADK_CKSUM_H

static inline uint32_t
adk_raw_cksum_nonalign(const uint8_t *buf, uint32_t len)
{
    uint32_t sum = 0, i;

    for (i = 0; i < (len / sizeof(uint16_t)); i++)
    {
        /* copy value for non aligned data */
        uint16_t u16_val;
        memcpy(&u16_val, &buf[i*2], sizeof(uint16_t));
        sum += u16_val;
    }

    /* if length is in odd bytes */
    if (len & 1)
        sum += (uint32_t) buf[len - 1];

    return sum;
}

static inline uint32_t
adk_raw_cksum_align(const uint16_t *u16, uint32_t len)
{
    uint32_t sum=0;
    
    while (len >= 2)
    {
        sum += *u16;
        len -= 2;
        u16 += 1;
    }

    /* if length is in odd bytes */
    if (len == 1)
        sum += *((const uint8_t *)u16);
    
    return sum;
}

static inline uint16_t adk_raw_cksum_reduce(uint32_t sum)
{
    /* reduce to 16bit */
    sum = ((sum & 0xffff0000) >> 16) + (sum & 0xffff);
    sum = ((sum & 0xffff0000) >> 16) + (sum & 0xffff);

    return (uint16_t)sum;
}

static inline uint16_t adk_raw_cksum(void *buf, uint16_t len)
{
    uint32_t sum;

    if((uintptr_t)buf & 0x1)
        sum = adk_raw_cksum_nonalign(buf, len);
    else
        sum = adk_raw_cksum_align(buf, len);
    
    /* reduce to 16bit */
    sum = ((sum & 0xffff0000) >> 16) + (sum & 0xffff);
    sum = ((sum & 0xffff0000) >> 16) + (sum & 0xffff);
    
    return (uint16_t)sum;
}

#endif /* __ADK_CKSUM_H */
