/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_pmu.h>
#include "adk_sif_dp_log_lib.h"
#include <emmintrin.h>
#ifdef INTEL_ITT
#include "ittnotify.h"
#endif
#define __STDC_FORMAT_MACROS
#include <inttypes.h>

#ifdef PIPELINE_STAGE_PKT_COUNT_ENABLED
pipeline_stage_pkt_count_t
        pipelineStagePktCount[PIPELINE_STAGE_PKT_COUNT_MAX_CPU]
        __attribute__((__aligned__(RTE_CACHE_LINE_SIZE))) = {{0, },};
 
void adk_pmu_pipeline_stage_pkt_count_print (void)
{
    uint32_t j;

    printf ("\n\nMatch the array index with the CPU ID manually\n");
    printf ("CPU ID \t Port Recv \t Ring Recv \t QAT Recv \t PH Recv\n");

    for (j = 0; j < PIPELINE_STAGE_PKT_COUNT_MAX_CPU; j++)
    {
        printf ("%d \t %10d \t %10d \t %10d \t %10d\n",
                j,
                pipelineStagePktCount[j].portRecv,
                pipelineStagePktCount[j].ringRecv,
                pipelineStagePktCount[j].qatRecv,
                pipelineStagePktCount[j].phRecv);
    }

    return;
}
#else
void adk_pmu_pipeline_stage_pkt_count_print (void)
{
    return;
}
#endif

static int num_iterations = 0;
static uint32_t rxTime, txTime;
static uint64_t diffTime;
static uint32_t maxTime = 0;
static adk_pmu_blk_cycle_cnt_info_t blkCycleInfo[PMU_MODULE_MAX] = {{0}};
#ifdef ADK_MEASURE_END_TO_END_CYCLES
uint64_t g_adk_entry_ts[ADK_MAX_NUM_CORES];
uint64_t g_lli_rx_ts[ADK_MAX_NUM_CORES], g_lli_tx_ts[ADK_MAX_NUM_CORES];
uint64_t g_uli_rx_ts[ADK_MAX_NUM_CORES], g_uli_tx_ts[ADK_MAX_NUM_CORES];
#endif


void adk_enable_pmu(void)
{
}

void adk_reset_cycle_cntr(void)
{
}

void adk_start_cycle_cntr(void)
{
}

void adk_stop_cycle_cntr(void)
{
}


void adk_config_evt_cntr(uint32_t __attribute__((unused)) cntr_id, 
			 uint32_t __attribute__((unused))evt_id)
{
}

void adk_start_evt_cntr(uint32_t __attribute__((unused))cntr_id)
{
}

void adk_stop_evt_cntr(uint32_t __attribute__((unused))cntr_id)
{
}

char * adk_get_evt_str(uint32_t __attribute__((unused))eventid)
{
    return NULL;
}

void adk_dp_measure_cycle_config(void)
{
}

void adk_dp_measure_cycle_rx_ts(void)
{
    uint32_t tmp = 0;
    debug_print("\nInside adk_dp_measure_cycle_rx_ts\n");
    _mm_mfence();
    ADK_INSTRUCTION_BARRIER();
    __asm__ __volatile__("rdtsc":"=a" (rxTime) , "=d" (tmp));
}

void adk_dp_measure_cycle_tx_ts(void)
{
    uint32_t tmp = 0;
    debug_print("\nInside adk_dp_measure_cycle_tx_ts\n");
    __asm__ __volatile__("rdtscp":"=a" (txTime) , "=d" (tmp));
    ADK_INSTRUCTION_BARRIER();
    
    num_iterations++;
    
    if(txTime > rxTime)
    {
	/*printf("diff time for sample - %u\n", txTime - rxTime);*/
	tmp = txTime - rxTime;
    }
    else
    {
	tmp = (0xffffffff - rxTime + txTime);
    }

	if(tmp > maxTime) {
	    maxTime = tmp; 
	}
        diffTime += tmp;

    if(num_iterations == (NUM_MAX_PKTS/ADK_DP_BULK_RX_SIZE))
    {
	printf("\n\nProcessor cycle statistics\n");
	printf("=========================================================\n");
	printf("Average clocks per packet   = %"PRIu64"\n", diffTime/NUM_MAX_PKTS);
	printf("Total clocks                = %"PRIu64"\n", diffTime);
	printf("Clocks (max value)          = %d\n\n", maxTime);
 
	printf("=========================================================\n");
	printf("Total samples used          = %d\n", NUM_MAX_PKTS);
        printf("Done\n");

    }

}

void adk_dp_measure_block_cycle_rx_ts(uint64_t *rx_ts)
{
    uint32_t cycles_hi, cycles_lo;
    debug_print("\nInside adk_dp_measure_block_cycle_rx_ts\n");
    _mm_mfence();
    ADK_INSTRUCTION_BARRIER();
    __asm__ __volatile__("rdtscp" : "=a" (cycles_lo), "=d" (cycles_hi));

    *rx_ts = (((uint64_t)cycles_hi << 32) | cycles_lo);
}
void adk_dp_measure_block_cycle_tx_inter_ts(uint64_t  *rx_ts,
                      uint64_t  *tx_ts,
                                      uint32_t moduleId)
{
    uint32_t cycles_hi, cycles_lo;

    debug_print("\nInside adk_dp_measure_block_cycle_tx_ts \n");
    __asm__ __volatile__("rdtscp" : "=a" (cycles_lo), "=d" (cycles_hi));
    ADK_INSTRUCTION_BARRIER();
    *tx_ts = (((uint64_t)cycles_hi << 32) | cycles_lo);

    if((*tx_ts) > (*rx_ts))
    {
        blkCycleInfo[moduleId].diffTime = blkCycleInfo[moduleId].diffTime + (*tx_ts) - (*rx_ts);
    }

}

void adk_dp_measure_block_cycle_tx_ts(uint64_t  *rx_ts, 
				      uint64_t  *tx_ts, 
				      const char *module,
                                      uint32_t moduleId)
{
    uint32_t cycles_hi, cycles_lo;

    debug_print("\nInside adk_dp_measure_block_cycle_tx_ts \n");
    __asm__ __volatile__("rdtscp" : "=a" (cycles_lo), "=d" (cycles_hi));
    ADK_INSTRUCTION_BARRIER();
    *tx_ts = (((uint64_t)cycles_hi << 32) | cycles_lo);

    num_iterations++;

    if((*tx_ts) > (*rx_ts))
    {
        blkCycleInfo[moduleId].diffTime = blkCycleInfo[moduleId].diffTime + (*tx_ts) - (*rx_ts);
        blkCycleInfo[moduleId].totalCnt++;
    }

    if(blkCycleInfo[moduleId].totalCnt == (NUM_MAX_ITRNS))
    {
        printf("totalcnt = %d \n", blkCycleInfo[moduleId].totalCnt);
        printf("\n======%s=======\n", module);
        /*blkCycleInfo[moduleId].avgTime = (blkCycleInfo[moduleId].diffTime/blkCycleInfo[moduleId].totalCnt);*/
        blkCycleInfo[moduleId].avgTime = (blkCycleInfo[moduleId].diffTime/(NUM_MAX_PKTS));
        printf("Core[%d]-->Average clocks per packet = %d\n", ADK_LCORE_INDEX, blkCycleInfo[moduleId].avgTime);
        printf("Total samples used        = %d\n", blkCycleInfo[moduleId].totalCnt);
        printf("=================================================\n");
        num_iterations=0;
        blkCycleInfo[moduleId].totalCnt = 0;
        *rx_ts = 0;
        *tx_ts = 0;
        blkCycleInfo[moduleId].diffTime = 0;
        blkCycleInfo[moduleId].avgTime = 0;
    }
}

uint32_t
adk_dp_get_cycle_count(bool __attribute__((unused))reset)
{
    unsigned a,d;
    __asm__ __volatile__("rdtsc":"=a" (a) , "=d" (d));
    /* return only lower value */
    return a;
}

