/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#include "adk_include.h"
#include "adk_sif_cfg_include.h"

/* Table to store EAL section parameters */
static adk_cfg_eal_params_tbl_t eal_params_tbl_g;


adk_status_t
adk_cfg_eal_init(adk_cfg_params_t *cfg_params)
{
    adk_status_t ret = ADK_SUCCESS;

    cfg_params->eal_params_tbl = (void *)&eal_params_tbl_g;

    return ret;
}

adk_status_t
adk_cfg_eal_parse(adk_cfg_params_t *cfg_params_hdl,
                  const char *section_name)
{
    adk_status_t ret = ADK_SUCCESS;
    adk_cfg_eal_params_t *eal_params = NULL;
    int32_t section_entries_cnt = 0, i = 0, idx = 0;
    struct rte_cfgfile_entry *section_entries = NULL, *entry = NULL;
    char tmp_buf[CFG_VALUE_LEN * 2], *p = NULL;
    adk_cfg_eal_flags_t flags;

    ADK_MEMSET(&flags, 0, sizeof(adk_cfg_eal_flags_t));

    /* Allocate memory to store parsed parameters */
    ret = adk_cfg_eal_param_alloc(cfg_params_hdl->eal_params_tbl, &eal_params);
    if (ret != ADK_SUCCESS)
    {
        return ret;
    }
 
    section_entries_cnt = rte_cfgfile_section_num_entries(cfg_params_hdl->cfg_handle, section_name);
    ADK_PREINIT_WRITE("\nSection:%s entries = %d\n", section_name, section_entries_cnt);

    /* Get all entries of current sections */
    section_entries = calloc(section_entries_cnt, sizeof(struct rte_cfgfile_entry));
    if (section_entries == NULL)
    {
        ADK_PREINIT_WRITE("Memory allocation failure\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);
    }

    if (rte_cfgfile_section_entries(cfg_params_hdl->cfg_handle, section_name, 
                                    section_entries, section_entries_cnt) == 0)
    {
        ADK_PREINIT_WRITE("Get section entries failed\n");
        ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_CFG_PARSE_ERROR, ADK_SEV_LEVEL_ERROR);
        goto adk_cfg_eal_parse_return;
    }

#ifdef ADK_DEBUG
    ADK_PREINIT_WRITE("[%s]\n", section_name);
    for (i = 0; i < section_entries_cnt; i++)
    {
        ADK_PREINIT_WRITE("\t%s = %s\n", section_entries[i].name, section_entries[i].value);
    }
#endif

    ADK_MEMCPY(eal_params->name, section_name, CFG_NAME_LEN);
    if(eal_params->name[CFG_NAME_LEN - 1] != '\0')
        eal_params->name[CFG_NAME_LEN - 1] = '\0';

    /* Parse each entry of current section */
    for (idx = 0; idx < section_entries_cnt; idx++)
    {
        entry = &section_entries[idx];
       
        /* Skip commneted lines */ 
        if (ADK_STRNCMP(entry->name, "#", 1) == 0)
        {
            continue;
        }

        if (ADK_STRNCMP(entry->name, "enable", sizeof("enable")) == 0)
        {
            if (strcasecmp(entry->value, "false") == 0)
            {
                /* Skip disabled section */
                ADK_MEMSET(eal_params, 0, sizeof(adk_cfg_eal_params_t));
                ret = ADK_W_CFG_SECTION_NOT_ENABLED;
                goto adk_cfg_eal_parse_return;
            }
            else
            {
                continue;
            }
        }
        else if (ADK_STRNCMP(entry->name, "process_type", 
                             sizeof("process_type")) == 0)
        {
            /* TODO: Deprecated: Process_type shall not be specified in config file */
            ret = adk_cfg_eal_process_type_get(entry->value, &eal_params->process_type);
            if (ret != ADK_SUCCESS)
            {
                goto adk_cfg_eal_parse_return;
            }
            flags.process_type++;
            continue;
        }
        else if (ADK_STRNCMP(entry->name, "master_lcore", 
                             sizeof("master_lcore")) == 0)
        {
            eal_params->master_lcore = atoi(entry->value); 
            flags.master_lcore++;
            continue;
        }
        else if (ADK_STRNCMP(entry->name, "worker_lcores",
                             sizeof("worker_lcores")) == 0)
        {
            ADK_MEMSET(tmp_buf, 0, sizeof(tmp_buf));
            ADK_STRNCPY(tmp_buf, entry->value, CFG_VALUE_LEN);
            i = 0;
            p = strtok (tmp_buf, " ");
            while (p != NULL)
            {
                eal_params->worker_lcores[i++] = atoi(p);
                if ( i >= ADK_SIF_CFG_MAX_WORKER_LCORES)
                {
                    ADK_PREINIT_WRITE("Number of worker lcores exceeds max supported %d\n", ADK_SIF_CFG_MAX_WORKER_LCORES);
                    ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
                    goto adk_cfg_eal_parse_return;
                }
                p = strtok(NULL, " ");
            }
            eal_params->num_worker_lcores = i;
            flags.worker_lcores += i;
            continue;
        }
        else if (ADK_STRNCMP(entry->name, "worker_lcore_socket",
                             sizeof("worker_lcore_socket")) == 0)
        {
            ADK_MEMSET(tmp_buf, 0, sizeof(tmp_buf));
            ADK_STRNCPY(tmp_buf, entry->value, CFG_VALUE_LEN);
            i = 0;
            p = strtok (tmp_buf, " ");
            while (p != NULL)
            {
                eal_params->worker_lcore_scoket[i++] = atoi(p);
                if ( i >= ADK_SIF_CFG_MAX_WORKER_LCORES)
                {
                    ADK_PREINIT_WRITE("Number of worker lcore socket %d exceeds max supported lcores %d\n", i, ADK_SIF_CFG_MAX_WORKER_LCORES);
                    ret = ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
                    goto adk_cfg_eal_parse_return;
                }
                p = strtok(NULL, " ");
            }
            flags.worker_lcore_socket += i;
            continue;
        }       
        else if (ADK_STRNCMP(entry->name, "mem_channels",
                             sizeof("mem_channels")) == 0)
        {
            eal_params->mem_channels = atoi(entry->value); 
            flags.mem_channels++;
            continue;
        }
        else
        {
            flags.unknown++;
        }
    }
    
    /* Validation */
    ret = adk_cfg_eal_validate(eal_params, &flags);
    if (ret != ADK_SUCCESS)
    {
        goto adk_cfg_eal_parse_return;
    }

    /* Set defaults */
    ret = adk_cfg_eal_defaults_set(eal_params, &flags);
    if (ret != ADK_SUCCESS)
    {
        goto adk_cfg_eal_parse_return;
    }

    free(section_entries);
    return ret;

adk_cfg_eal_parse_return:
    free(section_entries);
    adk_cfg_eal_param_free(cfg_params_hdl->eal_params_tbl, eal_params);
    return ret;
}

adk_status_t
adk_cfg_eal_param_alloc(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                        adk_cfg_eal_params_t **params)
{
    uint32_t i = 0;
    adk_cfg_eal_params_t *p = eal_params_hdl->eal_params;

    for (i = 0; i < ADK_SIF_CFG_MAX_EAL; i++)
    {
        if (p[i].valid == _INIT)
        {
            p[i].valid = _CONFIG;
            *params = &p[i];
            eal_params_hdl->size++;
            return ADK_SUCCESS;
        }
    }

    ADK_PREINIT_WRITE("Number of eals exceeds max supported %d\n", ADK_SIF_CFG_MAX_EAL);
    return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);        
}

adk_status_t
adk_cfg_eal_param_free(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                       adk_cfg_eal_params_t *params)
{
    adk_status_t ret = ADK_SUCCESS;

    if (params == NULL)
    {
        ADK_PREINIT_WRITE("Invalid memory access\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE, ADK_SEV_LEVEL_ERROR);        
    }

    params->valid = _INIT;
    eal_params_hdl->size--;
    
    return ret;
}

adk_status_t
adk_cfg_eal_param_rewind(adk_cfg_eal_params_tbl_t *eal_params_hdl)
{
    adk_status_t ret = ADK_SUCCESS;

    eal_params_hdl->cur = 0;
    
    return ret;
}

adk_status_t
adk_cfg_eal_param_get_next(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                           adk_cfg_eal_params_t **params)
{
    adk_status_t ret = ADK_SUCCESS;
    uint32_t i = 0;
    
    /* Boundary check */
    if (eal_params_hdl->cur == eal_params_hdl->size)

    {
        eal_params_hdl->cur = 0;
        return ADK_E_LIST_RANGE;
    }
  
    /* Skip deleted entries */ 
    for (i = eal_params_hdl->cur; i < eal_params_hdl->size; i++)
    {
        if (eal_params_hdl->eal_params[eal_params_hdl->cur].valid == _CONFIG)
        {
            break;
        }
    }
    eal_params_hdl->cur = i;

    /* Fetch config params */
    *params = &eal_params_hdl->eal_params[eal_params_hdl->cur];

    /* Point to next entry */
    eal_params_hdl->cur++;

    return ret;
}

adk_status_t
adk_cfg_eal_process_type_get(const char *value, uint32_t *process_type)
{
    /* TODO: Deprecated: Process_type shall not be specified in config file */
    adk_status_t ret = ADK_SUCCESS;
    
    if (strncasecmp(value, "primary", sizeof("primary")) == 0)
    {
        *process_type = ADK_SIF_CFG_EAL_PROCESS_TYPE_PRIMARY;
    }
    else if (strncasecmp(value, "secondary", sizeof("secondary")) == 0)
    {
        *process_type = ADK_SIF_CFG_EAL_PROCESS_TYPE_SECONDARY;
    }
    else
    {
        ADK_PREINIT_WRITE("Invalid process type\n");
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
    }

    return ret;
}

adk_status_t
adk_cfg_eal_validate(adk_cfg_eal_params_t *params,
                     adk_cfg_eal_flags_t *flags)
{
    adk_status_t ret = ADK_SUCCESS;

    if (!flags->worker_lcores)
    {
        ADK_PREINIT_WRITE("Section - %s:  worker lcore entry not configured\n", params->name);
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
    }

    if (!flags->worker_lcore_socket)
    {
        ADK_PREINIT_WRITE("Section - %s:  worker lcore socket entry not configured\n", params->name);
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
    }

    if (flags->worker_lcores != flags->worker_lcore_socket)
    {
        ADK_PREINIT_WRITE("Section - %s:  worker lcore list and worker lcore socket list size mismatch\n", params->name);
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
    }

    if (!flags->mem_channels)
    {
        ADK_PREINIT_WRITE("Section - %s:  memory channels entry not configured\n", params->name);
        return ADK_STATUS_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER, ADK_SEV_LEVEL_ERROR);
    }

    return ret;
}

adk_status_t
adk_cfg_eal_defaults_set(adk_cfg_eal_params_t *params,
                         adk_cfg_eal_flags_t *flags)
{
    adk_status_t ret = ADK_SUCCESS;

    if (!flags->process_type)
    {
        params->process_type = ADK_SIF_CFG_EAL_PROCESS_TYPE_PRIMARY;
    }
    
    return ret;
}



