/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_CFG_EAL_H__
#define __ADK_CFG_EAL_H__

/* Data structures */
typedef struct adk_cfg_eal_params_s{
    uint8_t     valid;
    char        name[CFG_NAME_LEN];                             /* Name of the section */
    uint32_t    master_lcore;                                   /* Control logical core */
    uint32_t    worker_lcores[ADK_SIF_CFG_MAX_WORKER_LCORES];   /* List of DP logical cores */
    int32_t     worker_lcore_scoket[ADK_SIF_CFG_MAX_WORKER_LCORES]; /* Socket ID of DP logical cores */
    uint32_t    mem_channels;                                   /* Number of memory channels */
    uint32_t    num_worker_lcores;                              /* Number of DP logical cores */
    uint32_t    process_type;                                   /* Process type - primary/secondary/auto */
}adk_cfg_eal_params_t;

typedef struct adk_cfg_eal_params_tbl_s{
    adk_cfg_eal_params_t   eal_params[ADK_SIF_CFG_MAX_EAL];
    uint32_t               size;
    uint32_t               cur;
}adk_cfg_eal_params_tbl_t;

typedef struct adk_cfg_eal_flags_s{
    uint16_t    master_lcore;
    uint16_t    worker_lcores;
    uint16_t    worker_lcore_socket;
    uint16_t    mem_channels;
    uint16_t    process_type;
    uint16_t    unknown;
}adk_cfg_eal_flags_t;


/* Function prototypes */
adk_status_t
adk_cfg_eal_init(adk_cfg_params_t *cfg_params);

adk_status_t
adk_cfg_eal_parse(adk_cfg_params_t *cfg_params,
                  const char *section_name);

adk_status_t
adk_cfg_eal_param_alloc(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                        adk_cfg_eal_params_t **params);

adk_status_t
adk_cfg_eal_param_free(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                       adk_cfg_eal_params_t *params);
adk_status_t
adk_cfg_eal_param_rewind(adk_cfg_eal_params_tbl_t *eal_params_hdl);

adk_status_t
adk_cfg_eal_param_get_next(adk_cfg_eal_params_tbl_t *eal_params_hdl,
                           adk_cfg_eal_params_t **params);

adk_status_t
adk_cfg_eal_process_type_get(const char *value, uint32_t *process_type);

adk_status_t
adk_cfg_eal_validate(adk_cfg_eal_params_t *params,
                     adk_cfg_eal_flags_t *flags);

adk_status_t
adk_cfg_eal_defaults_set(adk_cfg_eal_params_t *params,
                         adk_cfg_eal_flags_t *flags);

#endif
