/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_CFG_PIPELINE_H__
#define __ADK_CFG_PIPELINE_H__

#include "adk_cfg.h"
#include "adk_cfg_port.h"

typedef struct adk_cfg_pipeline_flags_s{
    uint16_t    stage;
    uint16_t    lcore;
    uint16_t    modules;
    uint16_t    portqin;
    uint16_t    portqout;
    uint16_t    ringqin;
    uint16_t    ringqout;
    uint16_t    buffout;
    uint16_t    buffin;
    uint16_t    unknown;
}adk_cfg_pipeline_flags_t;

typedef struct adk_cfg_pipeline_params_s{
    uint8_t                             valid;
    /* Input parameters */
    char                                name[CFG_NAME_LEN];
    uint32_t                            lcore;
    uint16_t                            entry_func;
    adk_sif_cfg_pipeline_stage_t        stage;

    adk_sif_cfg_pipeline_modules_t      modules[ADK_SIF_CFG_PIPELINE_MODULE_MAX];
    adk_sif_cfg_pipeline_port_q_t       port_q_in[ADK_SIF_CFG_MAX_PORT];
    adk_sif_cfg_pipeline_port_q_t       port_q_out[ADK_SIF_CFG_MAX_PORT];
    adk_sif_cfg_pipeline_ring_q_t       ring_q_in[ADK_SIF_CFG_MAX_RING_QUEUES];
    adk_sif_cfg_pipeline_ring_q_t       ring_q_out[ADK_SIF_CFG_MAX_RING_QUEUES];
    adk_sif_cfg_pipeline_buff_t         buff_in[ADK_SIF_CFG_MAX_PKT_BUFFS];
    adk_sif_cfg_pipeline_buff_t         buff_out[ADK_SIF_CFG_MAX_PKT_BUFFS];

    /* Is this pipeline receiving/sending from/to QAT? */
    bool qat_rx;
    bool qat_tx;

    /* Occupancy of input parameters lists */
    uint32_t                            num_modules;
    uint32_t                            num_port_q_in;
    uint32_t                            num_port_q_out;
    uint32_t                            num_ring_q_in;
    uint32_t                            num_ring_q_out;
    uint32_t                            num_buff_in;
    uint32_t                            num_buff_out;
}adk_cfg_pipeline_params_t;

typedef struct adk_cfg_pipeline_params_tbl_s{
    adk_cfg_pipeline_params_t   pipeline_params[ADK_SIF_CFG_MAX_PIPELINES];
    uint32_t                        size;
    uint32_t                        cur;
}adk_cfg_pipeline_params_tbl_t;

/* Function prototypes */
adk_status_t
adk_cfg_pipeline_init(adk_cfg_params_t *cfg_params);

adk_status_t
adk_cfg_pipeline_parse(adk_cfg_params_t *cfg_params,
                       const char *section_name);
adk_status_t
adk_cfg_pipeline_param_alloc(adk_cfg_pipeline_params_tbl_t *pipeline_params_hdl,
                             adk_cfg_pipeline_params_t **params);
adk_status_t
adk_cfg_pipeline_param_free(adk_cfg_pipeline_params_tbl_t *pipeline_params_hdl,
                            adk_cfg_pipeline_params_t *params);
adk_status_t
adk_cfg_pipeline_param_get_next(adk_cfg_pipeline_params_tbl_t *pipeline_params_hdl,
                                adk_cfg_pipeline_params_t **params);
adk_status_t
adk_cfg_pipeline_param_rewind(adk_cfg_pipeline_params_tbl_t *pipeline_params_hdl);

adk_status_t
adk_cfg_pipeline_param_check(adk_cfg_pipeline_params_tbl_t *pipeline_params_hdl);

adk_status_t
adk_cfg_pipeline_stage_get(const char *value, adk_sif_cfg_pipeline_stage_t *stage);

adk_status_t
adk_cfg_pipeline_module_get(const char *value, adk_sif_cfg_pipeline_modules_t *modules);

adk_status_t
adk_cfg_pipeline_port_q_get(const char *value,
                            adk_cfg_port_params_tbl_t *port_params_hdl,
                            adk_sif_cfg_pipeline_port_q_t *port_q,
                            uint32_t *num);
adk_status_t
adk_cfg_pipeline_port_idx_get(const uint8_t port,
                              adk_sif_cfg_pipeline_port_q_t *p,
                              uint32_t *n,
                              uint32_t *idx);
adk_status_t
adk_cfg_pipeline_queue_idx_get(const uint16_t queue,
                               uint16_t *q,
                               uint32_t *n,
                               uint32_t *idx);
adk_status_t
adk_cfg_pipeline_ring_idx_get(const uint16_t ringid,
                              adk_sif_cfg_pipeline_ring_q_t *r,
                              uint32_t *n,
                              uint32_t *idx);
adk_status_t
adk_cfg_pipeline_buff_idx_get(const uint16_t buffid,
                              adk_sif_cfg_pipeline_buff_t *r,
                              uint32_t *n,
                              uint32_t *idx);
adk_status_t
adk_cfg_pipeline_buff_get(const char *value,
                            adk_sif_cfg_pipeline_buff_t *port_q,
                            uint32_t *num);
adk_status_t
adk_cfg_pipeline_entry_function_get(const adk_sif_cfg_pipeline_modules_t entry_mod,
                                    const adk_sif_cfg_pipeline_modules_t exit_mod,
                                    uint16_t *entry_func);
adk_status_t
adk_cfg_pipeline_stage_validate(adk_cfg_pipeline_params_t *pipeline_params,
                                adk_cfg_pipeline_flags_t *flags);
adk_status_t
adk_cfg_pipeline_module_validate(adk_sif_cfg_pipeline_modules_t mod);

adk_status_t
adk_cfg_pipeline_stage_defaults_set(adk_cfg_pipeline_params_t *pipeline_params);

#endif
