/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_CFG_PKT_BUFFS_H__
#define __ADK_CFG_PKT_BUFFS_H__

/* Data structures */
typedef enum adk_cfg_pkt_buffs_type_e{
    ADK_CFG_PKT_BUFFS_TYPE_MIN = 0,
    ADK_CFG_PKT_BUFFS_TYPE_LOOP,
    ADK_CFG_PKT_BUFFS_TYPE_FINISH,
    ADK_CFG_PKT_BUFFS_TYPE_MAX
} adk_cfg_pkt_buffs_type_t;

typedef struct adk_cfg_pkt_buffs_params_s {
    uint8_t     valid;
    char        name[CFG_NAME_LEN];
    uint8_t     pkt_buffsid;
    uint16_t    rx_mempool;
    uint16_t    transmit;
    uint16_t    weight;
} adk_cfg_pkt_buffs_params_t;

typedef struct adk_cfg_pkt_buffs_params_tbl_s {
    adk_cfg_pkt_buffs_params_t  pkt_buffs_params[ADK_SIF_CFG_MAX_PKT_BUFFS];
    uint32_t               size;
    uint32_t               cur;
}adk_cfg_pkt_buffs_params_tbl_t;

typedef struct adk_cfg_pkt_buffs_flags_s {
    uint16_t    pkt_buffs;
    uint16_t    rx_mempool;
    uint16_t    transmit;
    uint16_t    weight;
    uint16_t    unknown;
}adk_cfg_pkt_buffs_flags_t;


/* Function prototypes */
adk_status_t
adk_cfg_pkt_buffs_init(adk_cfg_params_t *cfg_params);

adk_status_t
adk_cfg_pkt_buffs_parse(adk_cfg_params_t *cfg_params,
                   const char *section_name);
adk_status_t
adk_cfg_pkt_buffs_param_alloc(adk_cfg_pkt_buffs_params_tbl_t *pkt_buffs_params_hdl,
                         adk_cfg_pkt_buffs_params_t **params);
adk_status_t
adk_cfg_pkt_buffs_param_free(adk_cfg_pkt_buffs_params_tbl_t *pkt_buffs_params_hdl,
                        adk_cfg_pkt_buffs_params_t *params);
adk_status_t
adk_cfg_pkt_buffs_param_rewind(adk_cfg_pkt_buffs_params_tbl_t *pkt_buffs_params_hdl);

adk_status_t
adk_cfg_pkt_buffs_param_get_next(adk_cfg_pkt_buffs_params_tbl_t *pkt_buffs_params_hdl,
                            adk_cfg_pkt_buffs_params_t **params);

adk_status_t
adk_cfg_pkt_buffs_validate(adk_cfg_pkt_buffs_params_t *params,
                      adk_cfg_pkt_buffs_flags_t *flags);
adk_status_t
adk_cfg_pkt_buffs_defaults_set(adk_cfg_pkt_buffs_params_t *params);

adk_status_t
adk_sif_cfg_buff_info_get(adk_cfg_pkt_buffs_params_tbl_t *buff_info_hdl,
                       uint32_t buff_id);
adk_status_t
adk_cfg_pkt_buffs_transmit_type_get(const char *value, uint16_t *transmit);

#endif
