/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_CFG_PORT_H__
#define __ADK_CFG_PORT_H__

/* Data structures */
typedef enum adk_cfg_port_duplex_e{
    ADK_CFG_PORT_DUPLEX_AUTO = 0,
    ADK_CFG_PORT_DUPLEX_HALF,
    ADK_CFG_PORT_DUPLEX_FULL,
    ADK_CFG_PORT_DUPLEX_MAX
}adk_cfg_port_duplex_t;

typedef enum adk_cfg_port_rx_mq_mode_e{
    ADK_CFG_PORT_RX_MQ_MODE_NONE = 0,
    ADK_CFG_PORT_RX_MQ_MODE_RSS
}adk_cfg_port_rx_mq_mode_t;

typedef struct adk_cfg_port_params_s{
    uint8_t     valid;
    char        name[CFG_NAME_LEN];
    uint8_t     portid;
    uint16_t    speed;
    uint32_t    duplex;
    uint8_t     promisc;
    uint32_t    rx_mq_mode;
    uint16_t    rx_queues[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    rx_queue_mempool[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    rx_queue_size[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    rx_queue_weight[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    rx_queue_burst[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    tx_queues[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    int32_t     tx_queue_socket[ADK_SIF_CFG_MAX_QUEUES_PER_PORT]; 
    uint16_t    tx_queue_size[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    tx_queue_burst[ADK_SIF_CFG_MAX_QUEUES_PER_PORT];
    uint16_t    num_rx_queues;
    uint16_t    num_tx_queues;
}adk_cfg_port_params_t;

typedef struct adk_cfg_port_params_tbl_s{
    adk_cfg_port_params_t  port_params[ADK_SIF_CFG_MAX_PORT];
    uint32_t               size;
    uint32_t               cur;
}adk_cfg_port_params_tbl_t;

typedef struct adk_cfg_port_flags_s{
    uint16_t    port;
    uint16_t    speed;
    uint16_t    duplex;
    uint16_t    promisc;
    uint16_t    rx_mq_mode;
    uint16_t    rx_queues;
    uint16_t    rx_queue_mempool;
    uint16_t    rx_queue_size;
    uint16_t    rx_queue_weight;
    uint16_t    rx_queue_burst;
    uint16_t    tx_queues;
    uint16_t    tx_queue_socket; 
    uint16_t    tx_queue_size;
    uint16_t    tx_queue_burst;
    uint16_t    unknown;
}adk_cfg_port_flags_t;


/* Function prototypes */
adk_status_t
adk_cfg_port_init(adk_cfg_params_t *cfg_params);

adk_status_t
adk_cfg_port_parse(adk_cfg_params_t *cfg_params,
                   const char *section_name);
adk_status_t
adk_cfg_port_param_alloc(adk_cfg_port_params_tbl_t *port_params_hdl,
                         adk_cfg_port_params_t **params);
adk_status_t
adk_cfg_port_param_free(adk_cfg_port_params_tbl_t *port_params_hdl,
                        adk_cfg_port_params_t *params);
adk_status_t
adk_cfg_port_param_rewind(adk_cfg_port_params_tbl_t *port_params_hdl);

adk_status_t
adk_cfg_port_param_get_next(adk_cfg_port_params_tbl_t *port_params_hdl,
                            adk_cfg_port_params_t **params);
adk_status_t
adk_cfg_port_param_queue_weight_get(adk_cfg_port_params_tbl_t *port_params_hdl,
                                    uint8_t port_id,
                                    uint16_t queue_id,
                                    uint16_t *weight);
adk_status_t
adk_cfg_port_duplex_get(const char *value, uint32_t *duplex);

adk_status_t
adk_cfg_port_promisc_get(const char *value, uint8_t *promisc);

adk_status_t
adk_cfg_port_rx_mq_mode_get(const char *value, uint32_t *rx_mq_mode);

adk_status_t
adk_cfg_port_util_list_get(const char *value,
                           uint16_t max_list,
                           uint16_t *list,
                           uint16_t *num_list);
adk_status_t
adk_cfg_port_util_socket_list_get(const char *value,
                                  uint16_t max_list,
                                  int32_t *list,
                                  uint16_t *num_list);

adk_status_t
adk_cfg_port_validate(adk_cfg_port_params_t *params,
                      adk_cfg_port_flags_t *flags);
adk_status_t
adk_cfg_port_defaults_set(adk_cfg_port_params_t *params);

#endif
