/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
#ifndef __ADK_CFG_RING_H__
#define __ADK_CFG_RING_H__

/* Data structures */
typedef struct adk_cfg_ring_params_s{
    uint8_t     valid;
    char        name[CFG_NAME_LEN];
    uint32_t    ring_id;
    uint32_t    ring_size;
    uint32_t    ring_enqueue_burst;
    uint32_t    ring_dequeue_burst;
    int32_t     socket;
}adk_cfg_ring_params_t;

typedef struct adk_cfg_ring_params_tbl_s{
    adk_cfg_ring_params_t  ring_params[ADK_SIF_CFG_MAX_RING_QUEUES];
    uint32_t               size;
    uint32_t               cur;
}adk_cfg_ring_params_tbl_t;

typedef struct adk_cfg_ring_flags_s{
    uint16_t    ring;
    uint16_t    ring_size;
    uint16_t    ring_enqueue_burst;
    uint16_t    ring_dequeue_burst;
    uint16_t    socket;
    uint16_t    unknown;
}adk_cfg_ring_flags_t;


/* Function prototypes */
adk_status_t
adk_cfg_ring_init(adk_cfg_params_t *cfg_params);

adk_status_t
adk_cfg_ring_parse(adk_cfg_params_t *cfg_params,
                   const char *section_name);
adk_status_t
adk_cfg_ring_param_alloc(adk_cfg_ring_params_tbl_t *ring_params_hdl,
                         adk_cfg_ring_params_t **params);
adk_status_t
adk_cfg_ring_param_free(adk_cfg_ring_params_tbl_t *ring_params_hdl,
                        adk_cfg_ring_params_t *params);
adk_status_t
adk_cfg_ring_param_rewind(adk_cfg_ring_params_tbl_t *ring_params_hdl);

adk_status_t
adk_cfg_ring_param_get_next(adk_cfg_ring_params_tbl_t *ring_params_hdl,
                            adk_cfg_ring_params_t **params);
adk_status_t
adk_cfg_ring_type_get(const char *value, uint32_t *type);

adk_status_t
adk_cfg_ring_validate(adk_cfg_ring_params_t *params,
                      adk_cfg_ring_flags_t *flags);
adk_status_t
adk_cfg_ring_defaults_set(adk_cfg_ring_params_t *params);

#endif
