/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_HASH_H__
#define __ADK_HASH_H__

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <adk_sif.h>

typedef enum {
    ADK_HASH_KEY_TYPE_CHAR = 0,
    ADK_HASH_KEY_TYPE_WORD
} adk_hash_key_type_t;

typedef struct {
    uint8_t *buf;
    uint32_t size;
} adk_hash_buf_t;

typedef union {
    adk_hash_buf_t char_buf;
    uint32_t   word;
} adk_hash_key_val_t;

typedef struct {
    adk_hash_key_type_t key_type;
    adk_hash_key_val_t  key_val;
} adk_hash_key_t;

#if 0

typedef void* adk_hash_t;

typedef uint32_t adk_hash_value_t;

typedef enum {
    ADK_HASH_SUCCESS = 0,
    ADK_HASH_ERROR = 1
} adk_hash_st_t;

adk_hash_t adk_hash_new(void);

adk_hash_st_t
adk_hash_insert(adk_hash_t* tbl,
		adk_hash_key_t *key_info,
		adk_hash_value_t value);

adk_hash_st_t
adk_hash_delete(adk_hash_t* tbl,
	      adk_hash_key_t *key_info);
#endif


#undef get16bits

#if (defined(__GNUC__) && defined(__i386__)) || defined(__WATCOMC__) \
  || defined(_MSC_VER) || defined (__BORLANDC__) || defined (__TURBOC__)
#define get16bits(d) (*((const uint16_t *) (d)))
#endif

#if !defined (get16bits)
#define get16bits(d) ((((uint32_t)(((const uint8_t *)(d))[1])) << 8)\
                       +(uint32_t)(((const uint8_t *)(d))[0]) )
#endif

static inline uint32_t
SISD_CRC_HASH ( uint32_t init, uint32_t data) {
    return(_mm_crc32_u32( init, data));
}

static inline uint64_t
SISD_CRC_HASH64 ( uint64_t init, uint64_t data) {
    return(_mm_crc32_u64( init, data));
}
#endif
