/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_util_list.h"

adk_list_hdl_t
adk_list_create(uint32_t size, adk_sys_handle_t  sysHandle)
{
    adk_newutil_list_t *newList = NULL;
    adk_newutil_list_node_t *newNodeList = NULL;
    newList = (adk_newutil_list_t *) ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(
		 ADK_SYS_MODULE_HANDLE_GET(sysHandle)), 0,
		 1, sizeof(adk_newutil_list_t));
    if(newList == NULL)
    {
	debug_print("Error: Failed to create List!\n");
	return 0;
    }

    newNodeList = (adk_newutil_list_node_t *) ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(
		ADK_SYS_MODULE_HANDLE_GET(sysHandle)), 0,
		 size, sizeof(adk_newutil_list_node_t));
    if(newNodeList == NULL)
    {
	debug_print("Error: Failed to create sub-list!\n");
        ADK_NVM_FREE(newList);
	return 0;
    }

    newList->size = size;
    newList->node_list = newNodeList;
    newList->curr_node = 0;
    newList->iter = 0;

    return ((adk_list_hdl_t)newList);
}

int
adk_list_node_add(adk_list_hdl_t listHdl,
		  void *data)
{
    adk_newutil_list_t *list = (adk_newutil_list_t *)listHdl;
    adk_newutil_list_node_t *newNode = NULL;

    if(list->curr_node == (list->size - 1))
	return 0;

    newNode = &(list->node_list[list->curr_node++]);
    newNode->data = data;

    /* increment number of elements */
    list->num_elements++;
    return 1;
}

int adk_list_next_node_get_rollover(adk_list_hdl_t listHdl,
				    void **data)
{
    adk_newutil_list_t *list = (adk_newutil_list_t *)listHdl;
    adk_newutil_list_node_t *newNode = NULL;

    newNode = &(list->node_list[list->iter++]);
    *data = newNode->data;

    if(list->iter == list->num_elements)
	list->iter = 0; /* restart */
    return 1;
}

