/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_LPM_H__
#define __ADK_LPM_H__


#define ADK_LPM_VALUE_INVALID 0xFFFFFFFF
#define ADK_LPM_DEFAULT_PENDING_SIZE 1000

/**
 * LPM Object structure.
 */

typedef adk_handle_t adk_lpm_handle_t; /*change to handle*/

/**
 * Create a new LPM object.
 *
 * @return    Pointer to a new LPM object, or NULL if it
 *            was not possible to allocate memory for the LPM.
 */
adk_lpm_handle_t adk_lpm_create(size_t mod_id, size_t obj_id, uint16_t width, uint16_t pending_size);

/** 
 * Destroy LPM object.
 */
void adk_lpm_free(adk_lpm_handle_t lpm);

/**
 * Insert a new rule to LPM object.
 * @return      0 on success, non-zero value otherwise
 */
int adk_lpm_add(size_t mod_id, size_t obj_id, adk_lpm_handle_t lpm, uint8_t *key, uint16_t depth,
                adk_handle_t value);

/**
 * Check if a rule is present in the LPM object, and provide its value if it is.
 * @return      1 if the rule exists, 0 if it does not,
 *              a negative value on failure.
 */
int adk_lpm_is_rule_present(adk_lpm_handle_t lpm_handle, uint8_t *key,
                            uint16_t depth, adk_handle_t *value);

/**
 * Look up a value from its key in LPM object.
 * @return      LPM value on finding a valid lpm match.
 *              ADK_LPM_VALUE_INVALID on failure.
 */
adk_handle_t adk_lpm_lookup(adk_lpm_handle_t lpm, uint8_t *key);

/**
 * Remove an entry from LPM object.
 * @return      0 on success, non-zero value otherwise
 */
int adk_lpm_delete(adk_lpm_handle_t lpm, uint8_t *key, uint16_t depth);
        

/**
 * The function signature for the callback function that gets invoked for
 * for each entry in the LPM object during the LPM dump.
 */
typedef void (*adk_lpm_dump_cb_fn_t) (adk_lpm_handle_t lpm, void *rock,
                              uint8_t *key, uint16_t depth, adk_handle_t value);

/**
 * Dump the entries from LPM object. The callback will be invoked for each entry
 * in the LPM object.
 */
void adk_lpm_dump(adk_lpm_handle_t lpm_handle, adk_lpm_dump_cb_fn_t cb, void *rock);

#endif

