/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_pktio.h"
#include "adk_pktio_pvt.h"

static adk_pktio_hdl_t pktIoHdlr;

extern struct rte_mempool *adkPktmbufPool_g;

adk_pktio_ret_t 
adk_pktio_config(adk_pktio_cfg_t cfg, 
		 adk_pktio_hdl_t *hdl)
{
    adk_pktio_ret_t ret = ADK_PKTIO_SUCCESS;
    adk_pktio_internal_t *newHdlr = NULL;

    newHdlr = ADK_NVM_CALLOC(ADK_MISC_MODULE_ID, 0, 1, sizeof(adk_pktio_internal_t));
    if(newHdlr == NULL)
    {
	debug_print("Error: Could not allocate memory for pkt io!\n");
	return (ADK_PKTIO_NO_MEMORY);
    }
    newHdlr->info.type = cfg.type;
    /* switch based on pktio type */
    switch(cfg.type)
    {
    case ADK_PKTIO_TYPE_DPDK_PORT:
	ret = adk_pktio_dpdk_port_config(cfg, &newHdlr->info);
	newHdlr->sendFn = adk_pktio_dpdk_port_send;
	newHdlr->recvFn = adk_pktio_dpdk_port_recv;
	break;

    case ADK_PKTIO_TYPE_DPDK_RING:
	ret = adk_pktio_dpdk_ring_config(cfg, &newHdlr->info);
	newHdlr->sendFn = adk_pktio_dpdk_ring_send;
	newHdlr->recvFn = adk_pktio_dpdk_ring_recv;
	break;

    case ADK_PKTIO_TYPE_DPDK_BUFF:
	ret = adk_pktio_dpdk_buff_config(cfg, &newHdlr->info);
	newHdlr->sendFn = adk_pktio_dpdk_buff_send;
	newHdlr->recvFn = adk_pktio_dpdk_buff_recv;
	break;

    case ADK_PKTIO_TYPE_PH:
	ret = adk_pktio_ph_config(cfg, &newHdlr->info);
	newHdlr->sendFn = adk_pktio_ph_send;
	newHdlr->recvFn = adk_pktio_ph_recv;
	break;

    case ADK_PKTIO_TYPE_TUNTAP:
	ret = adk_pktio_tuntap_config(cfg, &newHdlr->info);
	newHdlr->sendFn = adk_pktio_tuntap_send;
	newHdlr->recvFn = adk_pktio_tuntap_recv;
	break;
    default:
	debug_print("Error: Unsupported pktio type!\n");
	ADK_NVM_FREE(newHdlr);
	return (ADK_PKTIO_ERROR);
    }
    *hdl = (adk_pktio_hdl_t)newHdlr;
    pktIoHdlr = (adk_pktio_hdl_t)newHdlr;

    return (ret);
}

adk_pktio_ret_t 
adk_pktio_recv(adk_pktio_hdl_t hdl,
               void *userData,
	       adk_pktio_task_t **task_p,
               uint8_t *task_count,
               uint32_t *task_mask)
{
    adk_pktio_ret_t ret;
    adk_pktio_internal_t *pktio_hdlr = (adk_pktio_internal_t *)hdl;
    ret = (pktio_hdlr->recvFn)(hdl, userData, task_p, task_count, task_mask);
    return (ret);
}

adk_pktio_ret_t 
adk_pktio_send(adk_pktio_hdl_t hdl, 
	       adk_pktio_task_t **task_p,
               uint8_t task_count,
               uint32_t task_mask)
{
    adk_pktio_ret_t ret;
    adk_pktio_internal_t *pktio_hdlr = (adk_pktio_internal_t *)hdl;
    ret = (pktio_hdlr->sendFn)(hdl, task_p, task_count, task_mask);
    return (ret);
}

void adk_pktio_task_buffer_alloc(adk_pktio_task_t **task_p)
{
    /* This function will initialize pduSize to 0 and userparam[7] will 
     * point to the first byte of the packet data after headroom
     */
    *task_p = (adk_pktio_task_t *)rte_pktmbuf_alloc(adkPktmbufPool_g);
    if(*task_p == NULL)
    {
        debug_print("Failed to allocate mbuf \n");
        return;
    }
    ADK_TASK_PDU_SIZE((*task_p)) = 0;
    (*task_p)->userdata = (void *)((*task_p)->buf_addr);
    ADK_TASK_USER_PARAMS((*task_p), 7) = 
        ADK_CONVERT_TO_TPARAMS(rte_pktmbuf_mtod((struct rte_mbuf *) *task_p, void*));
    ADK_TASK_USER_PARAMS((*task_p), 6) = ADK_CONVERT_TO_TPARAMS(0);
    ADK_TASK_USER_PARAMS((*task_p), 4) = ADK_CONVERT_TO_TPARAMS(pktIoHdlr);
    debug_print("\nTask buffer allocated, task:%p, pdu:%px, pduSize:%d",
           *task_p,
           ADK_CONVERT_TPARAMS_TO_UINTPTR(ADK_TASK_USER_PARAMS((*task_p), 7)),
           ADK_TASK_PDU_SIZE((*task_p)));

    return;
}

void adk_pktio_task_buffer_free(adk_pktio_task_t *task_p)
{
    rte_pktmbuf_free((struct rte_mbuf *)task_p);

    return;
}

/*
 * Function to stage pktio rx on a separate lcore
 */
adk_status_t
adk_pktio_mod_rx(void **task_p, 
                 uint8_t task_count,
                 __attribute__((unused)) uint32_t task_mask,
                 __attribute__((unused)) int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;
    uint16_t nb_tx = 0;
    uint32_t i = 0;
    
    adk_sif_cfg_pipeline_info_t *pipeline_info = NULL;
    adk_sif_cfg_pipeline_exit_module_t *pipeline_src_info = NULL;
    adk_sif_cfg_pipeline_dest_module_t *pipeline_dest_info = NULL;

    /* Get the pipeline info */
    pipeline_info =
        ADK_CONVERT_TPARAMS_TO_PIPELINE_INFO(ADK_TASK_USER_PARAMS(
                    (adk_pktio_task_t *)(task_p[0]), 8));
    pipeline_src_info = &pipeline_info->exit_module[ADK_SIF_CFG_PIPELINE_MODULE_PKTIO_RX];
    pipeline_dest_info = &pipeline_src_info->dest_module[ADK_SIF_CFG_PIPELINE_MODULE_RING_TX];

    if (likely(pipeline_dest_info->valid == _CONFIG))
    {
        nb_tx = rte_ring_sp_enqueue_burst((struct rte_ring *)pipeline_dest_info->ring, (void *)*task_p, task_count, NULL);
        if (unlikely(nb_tx != task_count))
        {
            for(i = nb_tx; i < task_count; ++i)
                rte_pktmbuf_free((struct rte_mbuf *)task_p[i]);
        }
    }
    else
    {
        for(i = 0; i < task_count; ++i)
            rte_pktmbuf_free((struct rte_mbuf *)task_p[i]);
    }

    return ret;
}


/*
 * Function to stage pktio tx on a separate lcore
 */
adk_status_t
adk_pktio_mod_tx(void **task_p, 
                 uint8_t task_count,
                 uint32_t task_mask,
                 __attribute__((unused)) void *ring,
                 __attribute__((unused)) adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                 int coreIndex)
{
    adk_status_t ret = ADK_SUCCESS;

    /* Place holder to stage additional functionality before transmission */
    /* < Additional functionalities > */

    /* Transmit on hardware port */
    ret = adk_sys_dp_pkt_send_to_port(task_p, task_count, task_mask, coreIndex);

    return ret;
}

adk_status_t
adk_pktio_mod_ring_tx(void **task_p, 
                      __attribute__((unused)) uint8_t task_count,
                      uint32_t task_mask,
                      void *ring,
                      adk_sif_cfg_pipeline_info_t *next_stage_entry, 
                      int __attribute__((unused)) coreIndex)
{
    adk_dp_status_t ret = ADK_SUCCESS;
    adk_pktio_task_t **task = (adk_pktio_task_t **)task_p;
    uint16_t nb_tx;
    uint32_t i, num = 0;
    uint32_t cur_bit;
    struct rte_mbuf *tx_task[ADK_DP_BULK_RX_SIZE * ADK_DP_TASK_MULTIPLY_MAX] = {0};

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for(i = 0, cur_bit = 1; i < ADK_DP_BULK_RX_SIZE; ++i, (cur_bit = cur_bit << 1))
    {
        if(task_mask & cur_bit)
        {
            tx_task[num] = (struct rte_mbuf *)(task[i]);
            ADK_TASK_USER_PARAMS(tx_task[num], 8) =
                ADK_CONVERT_TO_TPARAMS(next_stage_entry);
            ADK_TASK_USER_PARAMS(tx_task[num], 9) =
                ADK_CONVERT_TO_TPARAMS(adk_pktio_mod_tx);
            num++;
        }
    }

    nb_tx = rte_ring_sp_enqueue_burst((struct rte_ring *)ring, (void *)tx_task, num, NULL);
    if (unlikely(nb_tx != num))
    {
        for(i = nb_tx; i < num; ++i)
            rte_pktmbuf_free(tx_task[i]);
    }
   
    return ret;
}

