/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <stdio.h>
#include <signal.h>
#include <pthread.h>
#include "adk_dpdk.h"
#include "adk_include.h"
#include "adk_sys_lcore.h"
#include "adk_sif_cfg_include.h"
#include "adk_pktio.h"
#include "adk_pktio_pvt.h"
#include "adk_sys_dpdk.h"

#define ADK_MBUFF_INDEX 0
#define ADK_HROOM_0_INDEX 128
#define ADK_HROOM_1_INDEX 192
#define ADK_PDU_0_INDEX 256
#define ADK_PDU_1_INDEX 320

struct rte_eth_burst_rxtx {
        struct rte_mbuf *bufs[32];
        int num;
        int index;
};

/* DPDK PKTIO Declarations */
static uint8_t num_tasks[ADK_MAX_NUM_CORES][ADK_SIF_CFG_MAX_PORT] = {{0}};
static struct rte_mbuf *adkRTE_PktMBuffs_g[ADK_MAX_NUM_CORES][ADK_SIF_CFG_MAX_PORT] \
                    [ADK_DP_BULK_RX_SIZE * ADK_DP_TASK_MULTIPLY_MAX]={{{0}}};
/* DPDK PKTIO Port Function Definitions */

adk_pktio_ret_t adk_pktio_dpdk_port_config(adk_pktio_cfg_t cfg,
			       adk_pktio_info_t *hdl)
{
    hdl->u.dpdk_info.pktmbuf_pool = cfg.u.dpdk_config.pktmbuf_pool;
 
    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_dpdk_port_recv(__attribute__((unused)) adk_pktio_hdl_t hdl,
                             void *userData,
                             adk_pktio_task_t **task_p,
                             uint8_t *task_count,
                             uint32_t *task_mask)
{
    struct rte_mbuf *bufs[ADK_DP_BULK_RX_SIZE];
    uint16_t nb_rx;
    void *pdu = NULL;
    uint32_t i = 0;
    adk_sif_cfg_portq_t *portq = (adk_sif_cfg_portq_t *)userData;

    nb_rx = rte_eth_rx_burst(portq->port_id, portq->queue_id, bufs, ADK_DP_BULK_RX_SIZE);
    *task_mask = 0;
    *task_count = nb_rx;
    if (unlikely(nb_rx == 0))
        return ADK_PKTIO_QUEUE_EMPTY;

    for(i=0; i<nb_rx; ++i)
    {
        if(bufs[i] == NULL)
        {
	    debug_print("Error: Mbuf Pointer is NULL!\n");
	    return ADK_PKTIO_ERROR;
        }

        /* Fetch PDU pointer from mbuf */
        task_p[i] = (adk_pktio_task_t *) bufs[i];
        pdu = rte_pktmbuf_mtod(bufs[i], char *);

#ifdef ADK_RSS
    /*    debug_print("Lcore-%d: Port-%d: Queue-%d: RSS Hash-%d\n", ADK_LCORE_INDEX, portq->port_id, portq->queue_id, (bufs[i]->hash.rss & 0x1FF)); */
#endif
#ifdef OPTIMIZED_FOR_ATOM_CLASS
        rte_prefetch0((void *)pdu);
#endif
        adk_pkt_metadata_init(task_p[i], ADK_TN_PORT_DOMAIN, portq->entry_func, ADK_TASK_TYPE_FWD);
#if 0 /*Remove the old metadata refernces*/

	/* Task Param [1-3]: 
	 * Existence	   : Across all modules.
         */

	/* Task Param [7]: Set the pdu pointer
	 * Existence	 : Modules specific
         */
        ADK_TASK_USER_PARAMS(bufs[i], 7) = (uintptr_t) pdu;
	/* Task Param [4]: Store the pkio hdl for later use
	 * Existence	 : Modules specific
         */
        ADK_TASK_USER_PARAMS(bufs[i], 4) = (uintptr_t) hdl;

	/* Task Param [0]: Store port id
	 * Existence	 : Across all modules
         */
        ADK_TASK_USER_PARAMS(bufs[i], 0) = task_p[i]->port;

	/* Task Param [6]: Initialize UD_6 chained task
	 * Existence	 : Modules specific
         */
        ADK_TASK_USER_PARAMS(bufs[i], 6) = 0;

	/* Task Param [8]: Set pipeline entry module handle
	 * Existence	 : Modules specific
         */
        ADK_TASK_USER_PARAMS(bufs[i], 8) = (uintptr_t) portq->entry_mod_hdl;

	/* Task Param [8]: Set pipeline entry function
	 * Existence	 : Modules specific
         */
        ADK_TASK_USER_PARAMS(bufs[i], 9) = (uintptr_t) portq->entry_func;

	/* Task Param [10]: [Reserved] Setup DestFlow Handle*/
	/* Task Param [11]: Setup exit queue
	 * Existence	  : Across all modules.
         * Assumption: all packets in a burst goes out the same queue*/
        ADK_TASK_USER_PARAMS(bufs[i], 11) = portq->queue_id;
#endif
    
        *task_mask |= (1<<i); 
    }

    return ADK_PKTIO_SUCCESS;
}

#if 1 

adk_pktio_ret_t adk_pktio_dpdk_port_send(__attribute__((unused)) adk_pktio_hdl_t hdl,
		             adk_pktio_task_t **task,
                             __attribute__((unused)) uint8_t task_count,
                             uint32_t task_mask)
{
    
    uint16_t nb_tx, qid = 0;
    uint8_t port=0, i,j;
    uint32_t cur_bit;
    struct rte_mbuf *dpdk_task={0};
    int coreIndex = ADK_LCORE_INDEX;
    uint8_t tasks_num=0;

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for(i=0,cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(task_mask & cur_bit)
        {
            dpdk_task = (struct rte_mbuf *)(task[i]);
            port = dpdk_task->port;
            tasks_num = num_tasks[coreIndex][port];
            adkRTE_PktMBuffs_g[coreIndex][port][tasks_num] = (struct rte_mbuf *)(task[i]);
            num_tasks[coreIndex][port]++;
            tasks_num++;
            /*check for chain task*/
            adkRTE_PktMBuffs_g[coreIndex][port][tasks_num] = ADK_METADATA_PARAM(task[i], next_frag);
            while(unlikely(0!=adkRTE_PktMBuffs_g[coreIndex][port][tasks_num]))
            {
                adkRTE_PktMBuffs_g[coreIndex][port][tasks_num+1] =
                    ADK_METADATA_PARAM(adkRTE_PktMBuffs_g[coreIndex][port][tasks_num],
                            next_frag);
                num_tasks[coreIndex][port]++;
                tasks_num++;
            }

            if(tasks_num >= ADK_DP_BULK_RX_SIZE) {
#if 0
                /* Assumption: all packets in a burst goes out the same queue */
                qid = ADK_TASK_USER_PARAMS(adkRTE_PktMBuffs_g[coreIndex][port][0], 11);
#endif
                nb_tx = rte_eth_tx_burst(port, qid, adkRTE_PktMBuffs_g[coreIndex][port], tasks_num);
                if (unlikely(nb_tx != tasks_num))
                {
                    for(j=nb_tx; j<tasks_num; ++j)
                        rte_pktmbuf_free(adkRTE_PktMBuffs_g[coreIndex][port][j]);
                }
                num_tasks[coreIndex][port] = 0;
            }
        }
    }

    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_dpdk_port_flush(int coreIndex)
{
    
    uint16_t nb_tx, qid = 0;
    uint8_t port,j;

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for(port=0; port < ADK_SIF_CFG_MAX_PORT; port++)    
    {
        uint8_t tasks_num = num_tasks[coreIndex][port];
        if(tasks_num)
        {
            /* Assumption: all packets in a burst goes out the same queue */
            qid = ADK_TASK_USER_PARAMS(adkRTE_PktMBuffs_g[coreIndex][port][0], 11);
            
            debug_print("flushing %d packet for port %d\n", tasks_num, port);
            nb_tx = rte_eth_tx_burst(port, qid, adkRTE_PktMBuffs_g[coreIndex][port], tasks_num);
            if (unlikely(nb_tx != tasks_num))
            {
                debug_print("Failed to flush packets for port %d\n", port);
                for(j=nb_tx; j<tasks_num; ++j)
                    rte_pktmbuf_free(adkRTE_PktMBuffs_g[coreIndex][port][j]);
            }
            num_tasks[coreIndex][port] = 0;
        }
    }

    return ADK_PKTIO_SUCCESS;
}

#else
adk_pktio_ret_t adk_pktio_dpdk_port_send(__attribute__((unused)) adk_pktio_hdl_t hdl,
		             adk_pktio_task_t **task,
                             __attribute__((unused)) uint8_t task_count,
                             uint32_t task_mask)
{
    
    uint16_t nb_tx, qid = 0;
    uint8_t port, i, num=0;
    uint32_t cur_bit;
    struct rte_mbuf *dpdk_task[ADK_DP_BULK_RX_SIZE*ADK_DP_TASK_MULTIPLY_MAX]={0};
    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for(i=0,cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(task_mask & cur_bit)
        {
        dpdk_task[num] = (struct rte_mbuf *)(task[i]);
        num++;
        /*check for chain task*/
        dpdk_task[num] = (struct rte_mbuf *)(ADK_TASK_USER_PARAMS(task[i], 6));
        while(unlikely(0!=dpdk_task[num]))
        {
            dpdk_task[num+1] = (struct rte_mbuf *)(ADK_TASK_USER_PARAMS(dpdk_task[num], 6));
            num++;
        }
        }
    }

    /*Get the port number*/
    port = dpdk_task[0]->port;

    /* Assumption: all packets in a burst goes out the same queue */
    qid = ADK_TASK_USER_PARAMS(dpdk_task[0], 11);
    
    nb_tx = rte_eth_tx_burst(port, qid, dpdk_task, num);
    if (unlikely(nb_tx != num))
    {
        for(i=nb_tx; i<num; ++i)
            rte_pktmbuf_free(dpdk_task[i]);
    }
    return ADK_PKTIO_SUCCESS;
}
#endif

/* DPDK PKTIO Ring Function Definitions */

adk_pktio_ret_t adk_pktio_dpdk_ring_config(__attribute__((unused)) adk_pktio_cfg_t cfg,
			       __attribute__((unused)) adk_pktio_info_t *hdl)
{
    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_dpdk_ring_recv(adk_pktio_hdl_t hdl,
                             void *userData,
                             adk_pktio_task_t **task_p,
                             uint8_t *task_count,
                             uint32_t *task_mask)
{
#ifndef MULTI_RECV_BUFFERS
    struct rte_mbuf *bufs[ADK_DP_BULK_RX_SIZE];
    uint16_t nb_rx = 0;
#else
    static __thread struct rte_mbuf *bufs[2][ADK_DP_BULK_RX_SIZE];
    static __thread uint32_t toggler = 0;
    static __thread uint32_t firstReceive = 1;
    static __thread uint16_t nb_rx[2] = {0};
#endif
    uint32_t i = 0;
    adk_sif_cfg_ringq_t *ringq = (adk_sif_cfg_ringq_t *)userData;
#ifndef MULTI_RECV_BUFFERS
    nb_rx = rte_ring_sc_dequeue_burst(ringq->ring_hdl, (void **)bufs, ADK_DP_BULK_RX_SIZE, NULL);
    *task_mask = 0;
    *task_count = nb_rx;

    if (unlikely(nb_rx == 0))
        return ADK_PKTIO_QUEUE_EMPTY;

    for(i=0; i<nb_rx; ++i)
    {
        if(bufs[i] == NULL)
        {
	    debug_print("Error: Mbuf Pointer is NULL!\n");
	    return ADK_PKTIO_ERROR;
        }
        /* Fetch PDU pointer from mbuf */
        task_p[i] = (adk_pktio_task_t *) bufs[i];
        
        ADK_TASK_USER_PARAMS(bufs[i], 4) = (uintptr_t) hdl;
        /* Set the entry function to be called */
        if (ADK_TASK_USER_PARAMS(bufs[i], 9) == 0)
        {
            ADK_TASK_USER_PARAMS(bufs[i], 9) = (uintptr_t) ringq->entry_func;
        }

        *task_mask |= (1<<i); 
    }
#else
    nb_rx[toggler] = rte_ring_sc_dequeue_burst(ringq->ring_hdl, (void **)&bufs[toggler], ADK_DP_BULK_RX_SIZE, NULL);

    for(i=0; i<nb_rx[toggler]; ++i)
    {
        if(unlikely(bufs[toggler][i] == NULL))
        {
            debug_print("Error: Mbuf Pointer is NULL!\n");
            return ADK_PKTIO_ERROR;
        }
        ADK_PREFETCH_FOR_WRITE3((uint8_t  *)(&bufs[toggler][i]) + ADK_MBUFF_INDEX);
        ADK_PREFETCH_FOR_WRITE3((uint8_t  *)(&bufs[toggler][i]) + ADK_HROOM_0_INDEX);
        ADK_PREFETCH_FOR_WRITE3((uint8_t  *)(&bufs[toggler][i]) + ADK_PDU_0_INDEX);
    }

    if(unlikely(firstReceive) && unlikely(nb_rx[toggler] != 0))
    {
        firstReceive = 0;
        return ADK_PKTIO_QUEUE_EMPTY;
    }

    toggler ^= 1;
    if(unlikely(nb_rx[toggler] == 0))
    {
        return ADK_PKTIO_QUEUE_EMPTY;
    }

    *task_count = nb_rx[toggler];
    *task_mask = 0;

    for(i=0; i<nb_rx[toggler]; ++i)
    {
        if(unlikely(bufs[toggler][i] == NULL))
        {
            debug_print("Error: Mbuf Pointer is NULL!\n");
            return ADK_PKTIO_ERROR;
        }
        /* Fetch PDU pointer from mbuf */
        task_p[i] = (adk_pktio_task_t *) bufs[toggler][i];
        /* Store the pkio hdl for later use*/
        ADK_TASK_USER_PARAMS(bufs[i], 4) = (uintptr_t) hdl;

        /*  Set the entry function to be called */
        if (likely(ADK_TASK_USER_PARAMS(bufs[i], 9) == 0))
        {
            ADK_TASK_USER_PARAMS(bufs[i], 9) = (uintptr_t) ringq->entry_func;
        }

        *task_mask |= (1<<i);
    }
#endif
    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_dpdk_ring_send(__attribute__((unused)) adk_pktio_hdl_t hdl,
		             __attribute__((unused)) adk_pktio_task_t **task,
                             __attribute__((unused)) uint8_t task_count,
                             __attribute__((unused)) uint32_t task_mask)
{
    return ADK_PKTIO_SUCCESS;
}

#ifdef ENABLE_DPDK_PACKET_BUFFERS
/* DPDK PKTIO Buffer Declarations */
adk_dpdk_pkt_buffers_config_t adkPacketBufferConfig_g;
uint32_t adkPktBuffersIndex_g[ADK_MAX_NUM_CORES][ADK_SIF_CFG_MAX_PKT_BUFFS];
adk_dpdk_pkt_buff_stats_t adkPktBuffsStats_g[ADK_SIF_CFG_MAX_PKT_BUFFS];
#endif

/* DPDK PKTIO Buffer Function Definitions */
adk_pktio_ret_t adk_pktio_dpdk_buff_config (adk_pktio_cfg_t cfg, adk_pktio_info_t *hdl)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    hdl->u.dpdk_info.pktmbuf_pool = cfg.u.dpdk_config.pktmbuf_pool;
    return ADK_PKTIO_SUCCESS;
#else
    RTE_SET_USED(cfg);
    RTE_SET_USED(hdl);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
    return ADK_PKTIO_ERROR;
#endif
}

unsigned int 
adk_pktio_buffer_rx_burst(
        adk_pktio_hdl_t  hdl,
        adk_sif_cfg_pkt_buffs_t *buffQ,
        struct rte_mbuf **bufs,
        uint16_t nbPkts)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    struct rte_mempool *mp = ((adk_pktio_info_t *)hdl)->u.dpdk_info.pktmbuf_pool;
    uint32_t idx = 0;
    int ret = 0;
    int coreIndex = ADK_LCORE_INDEX;
    uint32_t rxPkts = 0, rxBytes = 0;

    if (unlikely(RTE_MAX_MBUFS_IN_BUFFER < 
                adkPacketBufferConfig_g.adkPacketsInBuffer[buffQ->buff_id]))
    {
        debug_print("Error: Packets in Buffer Exceeding RTE_MAX_MBUFS_IN_BUFFER:\
                %d\n", RTE_MAX_MBUFS_IN_BUFFER);
        raise(SIGUSR1);
        pthread_exit(NULL);
        return 0;
    }

    if (likely(adkPktBuffersIndex_g[coreIndex][buffQ->buff_id] <
                adkPacketBufferConfig_g.adkPacketsInBuffer[buffQ->buff_id]))
    {
        if (likely(adkPktBuffersIndex_g[coreIndex][buffQ->buff_id] + nbPkts >
                    adkPacketBufferConfig_g.adkPacketsInBuffer[buffQ->buff_id]))
        {
            nbPkts = adkPacketBufferConfig_g.adkPacketsInBuffer[buffQ->buff_id] - 
                adkPktBuffersIndex_g[coreIndex][buffQ->buff_id];
        }
    }
    else
    {
        if (ADK_SIF_CFG_PKT_BUFFS_TYPE_LOOP == buffQ->transmit)
        {
            adkPktBuffersIndex_g[coreIndex][buffQ->buff_id] = 0;
        }
        else if (ADK_SIF_CFG_PKT_BUFFS_TYPE_FINISH == buffQ->transmit)
        {
            debug_print("Core %d done processing. Wait for other cores to complete. \
                    Press Enter or Crtl-C for statistics\n", coreIndex);
            raise(SIGUSR1);
            pthread_exit(NULL);
            return 0; 
        }
    }

    ret = rte_mempool_get_bulk(mp, (void **) bufs, nbPkts);
    if (unlikely(ret == -ENOENT))
    {
        debug_print("Error: Packet mempool is full - Packet count %d!\n", 
                adkPktBuffersIndex_g[coreIndex][buffQ->buff_id]);
        exit(0);
    }

    for (idx = 0; idx < nbPkts; idx++)
    {
        struct rte_mbuf *mb = bufs[idx];

        ADK_MEMCPY((char *)(mb->buf_addr) + RTE_PKTMBUF_HEADROOM,
                &(adkPacketBufferConfig_g.adkPktBuffs[buffQ->buff_id][(adkPktBuffersIndex_g[coreIndex]
                        [buffQ->buff_id] + idx)][0]), adkPacketBufferConfig_g.adkPacketsSizeInBuffer[buffQ->buff_id]);

        mb->pkt_len = mb->data_len = adkPacketBufferConfig_g.adkPacketsSizeInBuffer[buffQ->buff_id];
        mb->next = NULL;
        mb->data_off = RTE_PKTMBUF_HEADROOM;
        mb->nb_segs = 1;
        mb->port = buffQ->buff_id;

        /* Update Buffer Statistics */
        rxBytes += mb->pkt_len;
        rxPkts++;
    }

    adkPktBuffersIndex_g[coreIndex][buffQ->buff_id] += idx;
    adkPktBuffsStats_g[buffQ->buff_id].rxBytes += rxBytes;
    adkPktBuffsStats_g[buffQ->buff_id].rxPkts += rxPkts;
    return nbPkts;
#else
    RTE_SET_USED(hdl);
    RTE_SET_USED(buffQ);
    RTE_SET_USED(bufs);
    RTE_SET_USED(nbPkts);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
    return 0;
#endif
}

adk_pktio_ret_t adk_pktio_dpdk_buff_recv(adk_pktio_hdl_t hdl, 
                          void *userData,
                          adk_pktio_task_t **task_p,
                          uint8_t *task_count,
                          uint32_t *task_mask)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    struct rte_mbuf *rxBuffers[ADK_DP_BULK_RX_SIZE];
    uint16_t nb_rx;
    void *pdu = NULL;
    uint32_t i = 0;
    adk_sif_cfg_pkt_buffs_t *buffQ = (adk_sif_cfg_pkt_buffs_t *)userData;

    nb_rx = adk_pktio_buffer_rx_burst(hdl, buffQ, rxBuffers, ADK_DP_BULK_RX_SIZE); 
    *task_mask = 0;
    *task_count = nb_rx;
    if (unlikely(nb_rx == 0))
        return ADK_PKTIO_QUEUE_EMPTY;

    for(i=0; i<nb_rx; ++i)
    {
        if(rxBuffers[i] == NULL)
        {
            debug_print("Error: Mbuf Pointer is NULL!\n");
            return ADK_PKTIO_ERROR;
        }

        /* Fetch PDU pointer from mbuf */
        task_p[i] = (adk_pktio_task_t *) rxBuffers[i];
        pdu = rte_pktmbuf_mtod(rxBuffers[i], char *);

        if(pdu == NULL)
        {
            debug_print("Error: PDU pointer is NULL!\n");
            return ADK_PKTIO_ERROR;
        }

        adk_pkt_metadata_init(task_p[i], ADK_TN_PORT_DOMAIN, buffQ->entry_func,
                ADK_TASK_TYPE_FWD);
        *task_mask |= (1<<i); 
    }

    return ADK_PKTIO_SUCCESS;
#else
    RTE_SET_USED(hdl);
    RTE_SET_USED(userData);
    RTE_SET_USED(task_p);
    RTE_SET_USED(task_count);
    RTE_SET_USED(task_mask);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
    return ADK_PKTIO_ERROR;
#endif
}


void adk_pktio_buffer_tx_burst(adk_pktio_hdl_t hdl,
                               struct rte_mbuf **bufs,
                               const uint16_t  nb_pkts)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    uint32_t txPkts = 0, txBytes = 0;
    uint16_t buff_id;
    uint32_t idx;

    struct rte_mempool *mp = ((adk_pktio_info_t *)hdl)->u.dpdk_info.pktmbuf_pool;
    /* Assumption: all packets in a burst goes out the same pool */
    for (idx = 0; idx < nb_pkts; idx++)
    {
        struct rte_mbuf *mb = bufs[idx];
        txBytes += mb->data_len;
        txPkts++;
    }
    buff_id = ((struct rte_mbuf *)bufs[0])->port;
    adkPktBuffsStats_g[buff_id].txBytes += txBytes;
    adkPktBuffsStats_g[buff_id].txPkts += txPkts;

    rte_mempool_put_bulk( mp, (void * const *) bufs, nb_pkts);
    debug_print("Sending %d packets - coreIndex %d\n", nb_pkts,
                ADK_LCORE_INDEX);
#else
    RTE_SET_USED(hdl);
    RTE_SET_USED(bufs);
    RTE_SET_USED(nb_pkts);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
#endif
}

adk_pktio_ret_t adk_pktio_dpdk_buff_send(
                   adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t  __attribute__((unused)) task_count,
                   uint32_t task_mask)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    uint8_t buffId=0, i;
    uint32_t cur_bit;
    struct rte_mbuf *dpdk_task={0};
    int coreIndex = ADK_LCORE_INDEX;
    uint8_t tasks_num=0;

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for (i=0, cur_bit=1; i < ADK_DP_BULK_RX_SIZE; ++i, (cur_bit = cur_bit<<1))
    {
        if (task_mask & cur_bit)
        {
            dpdk_task = (struct rte_mbuf *)(task[i]);
            buffId = ADK_METADATA_PARAM(task[i], input_intf_id);
            tasks_num = num_tasks[coreIndex][buffId];
            adkRTE_PktMBuffs_g[coreIndex][buffId][tasks_num] = 
                (struct rte_mbuf *)(task[i]);
            num_tasks[coreIndex][buffId]++;
            tasks_num++;
            /*check for chain task*/
            adkRTE_PktMBuffs_g[coreIndex][buffId][tasks_num] =
                ADK_METADATA_PARAM(task[i],next_frag); 
            while (unlikely(0!=adkRTE_PktMBuffs_g[coreIndex][buffId][tasks_num]))
            {
                adkRTE_PktMBuffs_g[coreIndex][buffId][tasks_num+1] = 
                    ADK_METADATA_PARAM(task[i], next_frag); 
                num_tasks[coreIndex][buffId]++;
                tasks_num++;
            }

            if (tasks_num >= ADK_DP_BULK_RX_SIZE) {
                adk_pktio_buffer_tx_burst(hdl, 
                        adkRTE_PktMBuffs_g[coreIndex][buffId], tasks_num);
                num_tasks[coreIndex][buffId] = 0;
            }
        }
    }
#else
    RTE_SET_USED(hdl);
    RTE_SET_USED(task);
    RTE_SET_USED(task_count);
    RTE_SET_USED(task_mask);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
#endif
    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_dpdk_buff_flush(adk_pktio_hdl_t hdl, int coreIndex)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    uint8_t buff_id;

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for (buff_id=0; buff_id < ADK_SIF_CFG_MAX_PORT; buff_id++)    
    {
        uint16_t tasks_num = num_tasks[coreIndex][buff_id];
        if (tasks_num)
        {
            /* Assumption: all packets in a burst goes out the same queue */
            uint32_t *pduParams;
            pduParams = (uint32_t *)((adkRTE_PktMBuffs_g[coreIndex][buff_id][0])->buf_addr);
            
            debug_print("flushing %d packet for buff_id %d\n", tasks_num, buff_id);
            adk_pktio_buffer_tx_burst(hdl, adkRTE_PktMBuffs_g[coreIndex][buff_id], tasks_num);
            num_tasks[coreIndex][buff_id] = 0;
        }
    }
    return ADK_PKTIO_SUCCESS;
#else
    RTE_SET_USED(hdl);
    RTE_SET_USED(coreIndex);
    debug_print("Error: ADK not built with ENABLE_DPDK_PACKET_BUFFERS flag.\n"); 
    return ADK_PKTIO_ERROR;
#endif
}

void adk_pktio_buffs_stats_print(void)
{
#ifdef ENABLE_DPDK_PACKET_BUFFERS
    int i;
    for (i = 0; i < ADK_SIF_CFG_MAX_PKT_BUFFS; i++)
    {
        if (( adkPktBuffsStats_g[i].rxBytes != 0 ||
                adkPktBuffsStats_g[i].rxPkts != 0 ) &&
            ( adkPktBuffsStats_g[i].txBytes != 0 ||
                adkPktBuffsStats_g[i].txPkts != 0 ) )
        {
            debug_print("\tBuffer Id: %d\n", i);
            debug_print("\t\tRx Packets:%"PRIu64"\n", adkPktBuffsStats_g[i].rxPkts);
            debug_print("\t\tTx Packets:%"PRIu64"\n", adkPktBuffsStats_g[i].txPkts);
            debug_print("\t\tRx Bytes:  %"PRIu64"\n", adkPktBuffsStats_g[i].rxBytes);
            debug_print("\t\tTx Bytes:  %"PRIu64"\n", adkPktBuffsStats_g[i].txBytes);
        }
    }
#endif
}
