/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include <adk_include.h>
#include "adk_sys_lcore.h"
#include <adk_pktio.h>
#include "adk_pktio_pvt.h"
#include "adk_sys_dpdk.h" 

extern struct rte_ring * adkDpPhRcvQueue_g[ADK_MAX_NUM_CORES];

adk_pktio_ret_t adk_pktio_ph_config(__attribute__((unused)) adk_pktio_cfg_t cfg,
			adk_pktio_info_t *hdl)
{
    int coreIndex;
    struct rte_ring * queueHdl;
    char ring_name[128];

    coreIndex = ADK_LCORE_INDEX;
    /*TBD: Find out a better mechanism to assign name for ring than sprintf */    
    sprintf(ring_name, "ph_ring_cpuid_%d", coreIndex);
    queueHdl = rte_ring_create(ring_name, PH_RING_SIZE, rte_socket_id(), RING_F_SP_ENQ|RING_F_SC_DEQ);
    hdl->u.ph_info.queueHdl = queueHdl;

    adkDpPhRcvQueue_g[coreIndex] = queueHdl;

    return ADK_PKTIO_SUCCESS;
}

adk_pktio_ret_t adk_pktio_ph_recv(  __attribute__((unused)) adk_pktio_hdl_t hdl,
                        void *userData,
		        adk_pktio_task_t **task_p,
                        uint8_t *task_count,
                        uint32_t *task_mask)
{
    int qret=1;
    if(userData)
    {
        qret = rte_ring_sc_dequeue_bulk(((struct rte_ring *)(userData)),
                               (void **) task_p,
                               1, NULL);
    }
    if(qret == 0) {
        *task_count = 0;
        *task_mask = 0x0;
        return ADK_PKTIO_QUEUE_EMPTY;
    } else {
        *task_count = 1;
        *task_mask = 0x1;
        return ADK_PKTIO_SUCCESS;
    }
}

adk_pktio_ret_t adk_pktio_ph_send(__attribute__((unused)) adk_pktio_hdl_t hdl,
		        __attribute__((unused)) adk_pktio_task_t **task,
                        __attribute__((unused)) uint8_t task_count,
                        __attribute__((unused)) uint32_t task_mask)
{
    
#if 0
    uint16_t nb_tx;
    uint8_t port, i,num=0;
    uint32_t cur_bit;
    struct rte_mbuf *dpdk_task[ADK_DP_BULK_RX_SIZE*ADK_DP_TASK_MULTIPLY_MAX]={0};

    /*Tasks may not be packed since some tasks are dropped/consumed/fragmented. Pack the tasks*/
    for(i=0,cur_bit=1; i<ADK_DP_BULK_RX_SIZE; ++i,(cur_bit=cur_bit<<1))
    {
        if(task_mask & cur_bit)
        {
            dpdk_task[num] = (struct rte_mbuf *)(task[i]);
            num++;
            /*check for chain task*/
            dpdk_task[num] = (struct rte_mbuf *)(ADK_TASK_USER_PARAMS(task[i], 6));
            while(unlikely(0!=dpdk_task[num]))
            {
                dpdk_task[num+1] = (struct rte_mbuf *)(ADK_TASK_USER_PARAMS(dpdk_task[num], 6));
                num++;
            }
        }
    }
    /*Get the port number*/
    /* For rerformance measurement packet is transmitted on the received port. Need to be removed */
    port = dpdk_task[0]->port;

    nb_tx = rte_eth_tx_burst(port, 0, dpdk_task, num);
    if (unlikely(nb_tx != num))
    {
        for(i=nb_tx; i<num; ++i)
            rte_pktmbuf_free(dpdk_task[i]);
    }
#endif
    return ADK_PKTIO_SUCCESS;
}
