/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_PKTIO_PVT_H__
#define __ADK_PKTIO_PVT_H__

#include <adk_pktio.h>

#include <adk_queue.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <adk_sys_dpdk.h>

#define TASK_BUFF_SIZE	2048
#define TASK_BUFF_PDU_HEADROOM 64

typedef struct {
    int sock_fd;
    int udp_port;
    struct sockaddr_in remaddr;	        /* remote address */
    uint32_t inPort;
} adk_pktio_socket_info_t;

typedef struct {
    void* pcap_descr; 
} adk_pktio_pcap_info_t;

typedef struct {
    adk_queue_handle_t queue_hdl;
    int size;
} adk_pktio_queue_info_t;

typedef struct {
    int fd;
    int flags;
} adk_pktio_tuntap_info_t;

typedef struct {
    struct rte_mempool *pktmbuf_pool;
} adk_pktio_dpdk_info_t;

typedef struct {
    struct rte_ring *queueHdl;
} adk_pktio_ph_info_t;

typedef struct {
    adk_pktio_type_t type;
    union {
        adk_pktio_socket_info_t   sock_info;
        adk_pktio_pcap_info_t     pcap_info;
        adk_pktio_queue_info_t	  queue_info;
        adk_pktio_tuntap_info_t   tuntap_info;
        adk_pktio_dpdk_info_t     dpdk_info;
        adk_pktio_ph_info_t       ph_info;
    } u;
} adk_pktio_info_t;

typedef adk_pktio_ret_t (*pktio_recv_fn_t) (adk_pktio_hdl_t , void *, adk_pktio_task_t **, uint8_t *, uint32_t *);
typedef adk_pktio_ret_t (*pktio_send_fn_t) (adk_pktio_hdl_t , adk_pktio_task_t **, uint8_t, uint32_t);

typedef struct {
    adk_pktio_info_t info;
    pktio_send_fn_t sendFn;
    pktio_recv_fn_t recvFn;
} adk_pktio_internal_t;


/* Functions for different types of pkt io */
adk_pktio_ret_t adk_pktio_dpdk_buff_config(adk_pktio_cfg_t cfg,
                                           adk_pktio_info_t *hdl);

adk_pktio_ret_t  adk_pktio_dpdk_buff_recv(adk_pktio_hdl_t __attribute__((unused))hdl,
                   void __attribute__((unused))*userData,
                   adk_pktio_task_t __attribute__((unused))**task_p,
                   uint8_t __attribute__((unused))*task_count,
                   uint32_t __attribute__((unused))*task_mask);

adk_pktio_ret_t adk_pktio_dpdk_buff_send(adk_pktio_hdl_t __attribute__((unused))hdl,
                   adk_pktio_task_t __attribute__((unused))**task,
                   uint8_t __attribute__((unused))task_count,
                   uint32_t __attribute__((unused))task_mask);

adk_pktio_ret_t adk_pktio_buffer_flush(adk_pktio_hdl_t hdl, int coreIndex);
void printADK_PktBuffStatistics(void);

adk_pktio_ret_t adk_pktio_socket_config(adk_pktio_cfg_t cfg,
		     adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_socket_recv(adk_pktio_hdl_t hdl,
                   void *userData,
		   adk_pktio_task_t **task_p,
                   uint8_t *task_count,
                   uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_socket_send(adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t task_count,
                   uint32_t task_mask);

adk_pktio_ret_t adk_pktio_pcap_config(adk_pktio_cfg_t cfg,
		     adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_pcap_recv(adk_pktio_hdl_t hdl,
                   void *userData,
		   adk_pktio_task_t **task_p,
                   uint8_t *task_count,
                   uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_pcap_send(adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t task_count,
                   uint32_t task_mask);

adk_pktio_ret_t adk_pktio_queue_config(adk_pktio_cfg_t cfg,
		     adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_queue_recv(adk_pktio_hdl_t hdl,
                   void *userData,
		   adk_pktio_task_t **task_p,
                   uint8_t *task_count,
                   uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_queue_send(adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t task_count,
                   uint32_t task_mask);

adk_pktio_ret_t adk_pktio_tuntap_config(adk_pktio_cfg_t cfg,
		     adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_tuntap_recv(adk_pktio_hdl_t hdl,
                   void *userData,
		   adk_pktio_task_t **task_p,
                   uint8_t *task_count,
                   uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_tuntap_send(adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t task_count,
                   uint32_t task_mask);
adk_pktio_ret_t adk_pktio_dpdk_port_config(adk_pktio_cfg_t cfg,
                               adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_dpdk_port_recv(adk_pktio_hdl_t hdl,
                             void *userData,
                             adk_pktio_task_t **task_p,
                             uint8_t *task_count,
                             uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_dpdk_port_send(adk_pktio_hdl_t hdl,
                             adk_pktio_task_t **task,
                             uint8_t task_count,
                             uint32_t task_mask);

adk_pktio_ret_t adk_pktio_dpdk_ring_config(adk_pktio_cfg_t cfg,
                               adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_dpdk_ring_recv(adk_pktio_hdl_t hdl,
                             void *userData,
                             adk_pktio_task_t **task_p,
                             uint8_t *task_count,
                             uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_dpdk_ring_send(adk_pktio_hdl_t hdl,
                             adk_pktio_task_t **task,
                             uint8_t task_count,
                             uint32_t task_mask);

adk_pktio_ret_t adk_pktio_ph_config(adk_pktio_cfg_t cfg,
                   adk_pktio_info_t *hdl);

adk_pktio_ret_t adk_pktio_ph_recv(adk_pktio_hdl_t hdl,
                        void *userData,
		        adk_pktio_task_t **task_p,
                        uint8_t *task_count,
                        uint32_t *task_mask);

adk_pktio_ret_t adk_pktio_ph_send(adk_pktio_hdl_t hdl,
                   adk_pktio_task_t **task,
                   uint8_t task_count,
                   uint32_t task_mask);

#ifdef ENABLE_DPDK_PACKET_BUFFERS

typedef struct adk_dpdk_pkt_buff_stats_s {
    uint64_t rxPkts;
    uint64_t txPkts;
    uint64_t rxBytes;
    uint64_t txBytes;
} adk_dpdk_pkt_buff_stats_t;

void adk_pktio_buffer_tx_burst(adk_pktio_hdl_t hdl,
                               struct rte_mbuf **bufs,
                               const uint16_t  nb_pkts);

unsigned int adk_pktio_buffer_rx_burst(adk_pktio_hdl_t hdl,
                                       adk_sif_cfg_pkt_buffs_t *buffQ,
                                       struct rte_mbuf **bufs,
                                       uint16_t  nb_pkts);

#else
void adk_pktio_buffer_tx_burst(adk_pktio_hdl_t __attribute__((unused))hdl,
                               struct rte_mbuf __attribute__((unused))**bufs,
                               const uint16_t  __attribute__((unused))nb_pkts);

unsigned int adk_pktio_buffer_rx_burst(adk_pktio_hdl_t __attribute__((unused))hdl,
                                       adk_sif_cfg_pkt_buffs_t __attribute__((unused))*buffQ,
                                       struct rte_mbuf __attribute__((unused))**bufs,
                                       uint16_t __attribute__((unused))nb_pkts);

#endif

#endif
