/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_queue.h"

typedef struct _adk_queue_node_s
{
    void *data;
} adk_queue_node_t;

typedef struct _adk_queue_s
{
    adk_queue_node_t *root;
    uint32_t head;
    uint32_t tail;
    uint32_t size;
    uint32_t numElements;
} adk_queue_t;

adk_queue_handle_t 
adk_queue_create(uint32_t size)
{
    adk_queue_t *newQueue = NULL;
    adk_queue_node_t *newList = NULL;
    adk_queue_handle_t qHdl;

    newQueue = (adk_queue_t *) ADK_NVM_CALLOC(ADK_MISC_MODULE_ID, 0, 1, sizeof(adk_queue_t));
    if(newQueue == NULL)
    {
	debug_print("Error: Failed to create Queue!\n");
	return 0;
    }

    newList = (adk_queue_node_t *)
	ADK_NVM_CALLOC(ADK_MISC_MODULE_ID, 0, size, sizeof(adk_queue_node_t));
    if(newList == 0)
    {
	debug_print("Error: Failed to allocate memory for queue!\n");
        ADK_NVM_FREE(newQueue);
	return 0;
    }

    newQueue->root = newList;
    newQueue->head = 0;
    newQueue->tail = 0;
    newQueue->size = size;
    newQueue->numElements = 0;
    qHdl = (adk_queue_handle_t)newQueue;
    debug_print("Successfully created queue of size - %d\n", size);

    return qHdl;
}

int adk_queue_delete(adk_queue_handle_t qHdl)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;
    ADK_NVM_FREE(queue_p->root);
    ADK_NVM_FREE(queue_p);
    return 1;
}

int adk_queue_dequeue(adk_queue_handle_t qHdl,
		      void **data)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;
    adk_queue_node_t *node = NULL;

    /* check if queue is empty */
    if(queue_p->numElements == 0)
    {
	*data = NULL;
	debug_print("Error: Queue is empty!\n");
	return 0;
    } else {
	node = &(queue_p->root[queue_p->head]);
	*data= node->data;
	queue_p->head++;
	if(queue_p->head == queue_p->size)
	{
	    /* wrap around to beginning */
	    queue_p->head = 0;
	}
	queue_p->numElements--;
	debug_print("Successfully dequeued from queue!\n");
    }
    return 1;
}


int adk_queue_enqueue(adk_queue_handle_t qHdl,
		      void *data)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;
    adk_queue_node_t *node = NULL;

    /* check if queue is full */
    if(queue_p->numElements == queue_p->size)
    {
	debug_print("Error: Failed to enqueue packet. Queue is FULL!\n");
	return 0;
    } else {
	/* enqueue to the tail */
	node = &(queue_p->root[queue_p->tail]);
	node->data = data;

	/* update queue information */
	queue_p->tail++;
	if(queue_p->tail == queue_p->size)
	{
	    /* wrap around to beginning */
	    queue_p->tail = 0;
	}
	queue_p->numElements++;
	debug_print("Successfully inserted in queue at index %d\n", 
	       (queue_p->tail - 1));
    }
    /* enqueue successful */
    return 1;
}

uint32_t adk_queue_get_size(adk_queue_handle_t qHdl)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;

    return (queue_p->size);
}

uint32_t adk_queue_get_depth(adk_queue_handle_t qHdl)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;

    return (queue_p->numElements);
}

int adk_queue_get_data(adk_queue_handle_t qHdl,
		       void **data)
{
    adk_queue_t *queue_p = (adk_queue_t *)qHdl;
    adk_queue_node_t *node = NULL;

    /* check if queue is empty */
    if(queue_p->numElements == 0)
    {
	*data = NULL;
	debug_print("Error: Queue is empty!\n");
	return 0;
    } else {
	node = &(queue_p->root[queue_p->head]);
	*data= node->data;
	debug_print("Successfully passed data from queue!\n");
    }
    return 1;
}
