/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifdef ADK_RCU
#include <emmintrin.h>
#include "adk_include.h"
#include "adk_rcu.h"
#include "adk_rcu_distributed_util.h"

#if 0
/* Global grace period counter update by CP thread and access by DP thread(s) */
int64_t adk_rcu_qp_ctr_g = 0;

/* Per DP thread quiescent state grace period counter. Assume one DP thread per core */
int64_t adk_rcu_reader_qs_gp[ADK_MAX_NUM_CORES];
#endif

adk_rcu_state_t *adkRcuState_p = NULL;


adk_status_t
adk_rcu_init(adk_sys_handle_t sysHandle)
{
    adk_status_t            retValue = ADK_SUCCESS;

    /*! Validate system handle */
    if (!ADK_VALID_HANDLE(sysHandle))
    {
        ADK_FORCE_LOG_ERROR("Failed to retrieve ADK System Handle!\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_HANDLE);
        goto _return;
    }

    ADK_LOG_INTERNAL_FUNC_ENTRY(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle));

    adkRcuState_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ADK_SYS_MODULE_HANDLE_GET(sysHandle)), 0,
			 1, sizeof(adk_rcu_state_t));

    if(adkRcuState_p == NULL)
    {
        ADK_LOG_ERROR(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle),
                      "Memroy alloc failed\n");
        retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_MEM_UNAVAILABLE);
        goto _return;
    }

    ((adk_sys_info_t *)sysHandle)->rcuState_p = adkRcuState_p;

    ADK_LOG_INTERNAL_FUNC_EXIT(sysHandle, ADK_SYS_MODULE_HANDLE_GET(sysHandle));
_return:
    return retValue;
}

int adk_rcu_config_get(adk_sys_handle_t sysHandle)
{
    if(((adk_sys_info_t *)sysHandle)->rcuState_p != NULL)
    {
        adkRcuState_p = ((adk_sys_info_t *)sysHandle)->rcuState_p;
        return 0;
    }

    return 1;
}
/**************************************************************************
 *                 RCU update side functions                              *
 **************************************************************************/

static inline int16_t  adk_rcu_gp_ongoing(int32_t threadIdx)
{
    return (adkRcuState_p->adk_rcu_reader_qs_gp[threadIdx].ctr & 0x1);
}

void adk_rcu_distributed_ver_update(void)
{
    _mm_mfence();
    adkRcuState_p->adk_rcu_qp_ctr_g.ctr += 2;
    _mm_mfence();
}

adk_status_t adk_rcu_distributed_ver_verify(void)
{
    int32_t t;
    uint32_t i;
    adk_sys_config_worker_lcore_info_t *workerInfo_p;

    /*
     * Lookup the worker info for later use
     */
    adk_sys_config_worker_lcore_info_get(&workerInfo_p);
    _mm_mfence();
    /*
     * Need to run across all possible indices since there could be holes in
     * the list
     */
    for (i = 0; i < workerInfo_p->numLcores; ++i)
    {
        t = workerInfo_p->lcore[i].index;
        if(adk_rcu_gp_ongoing(t) &&
           ((adkRcuState_p->adk_rcu_reader_qs_gp[t].ctr - adkRcuState_p->adk_rcu_qp_ctr_g.ctr) < 0))
        {
            return ADK_E_RCU_SYNC_FAILED;
        }
    }
    _mm_mfence();
    return ADK_SUCCESS;
}

void adk_rcu_distributed_wait(void)
{
    int32_t t;
    uint32_t i;
    adk_sys_config_worker_lcore_info_t *workerInfo_p;

    /*
     * Lookup the worker info for later use
     */
    adk_sys_config_worker_lcore_info_get(&workerInfo_p);
    _mm_mfence();
    /*
     * Need to run across all possible indices since there could be holes in
     * the list
     */
    for (i = 0; i < workerInfo_p->numLcores; ++i)
    {
        t = workerInfo_p->lcore[i].index;
        while(adk_rcu_gp_ongoing(t) &&
           ((adkRcuState_p->adk_rcu_reader_qs_gp[t].ctr - adkRcuState_p->adk_rcu_qp_ctr_g.ctr) < 0))
        {
            usleep(ADK_RCU_UPDATE_POLL_TIMEOUT);
            barrier();
        }
    }
    _mm_mfence();
    return;
}

void adk_rcu_synchronize(void)
{
    int32_t t;
    uint32_t i;
    adk_sys_config_worker_lcore_info_t *workerInfo_p;
    /*
     * Lookup the worker info for later use
     */
    adk_sys_config_worker_lcore_info_get(&workerInfo_p);

    _mm_mfence();
    adkRcuState_p->adk_rcu_qp_ctr_g.ctr += 2;
    /*
     * Need to run across all possible indices since there could be holes in
     * the list
     */
    for (i = 0; i < workerInfo_p->numLcores; ++i)
    {
        t = workerInfo_p->lcore[i].index;
        while (adk_rcu_gp_ongoing(t) &&
               ((adkRcuState_p->adk_rcu_reader_qs_gp[t].ctr - adkRcuState_p->adk_rcu_qp_ctr_g.ctr) < 0))
        {
            usleep(ADK_RCU_UPDATE_POLL_TIMEOUT);
            barrier();
        }
    }
    _mm_mfence();
}

void adk_rcu_shm_free_cb_fn(void __attribute__((unused)) *rock, void  *handle)
{
    ADK_DP_SHM_FREE(handle);
}
#endif

