/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_RCU_H__
#define __ADK_RCU_H__

#ifdef ADK_RCU

/* #include "adk_sif.h" */
#include "adk_sys_lcore.h"

#define ACCESS_ONCE(x)   (*(volatile typeof(x) *)&(x))
#define barrier()        __asm__ __volatile__("": : :"memory")

#define THREAD_IDX                  ADK_LCORE_INDEX
#define ADK_RCU_READ_POLL_CNT       10  /* iteration */
#define ADK_RCU_UPDATE_POLL_CNT     100 /* number of delete calls */
#define ADK_RCU_UPDATE_POLL_TIMEOUT 25  /* micro seconds */

#if 0
extern int64_t adk_rcu_qp_ctr_g;
extern int64_t adk_rcu_reader_qs_gp[];
#endif

typedef struct adk_rcu_ctr_s
{
    int64_t ctr;
}__attribute__ ((aligned (64))) adk_rcu_ctr_t;

/* each variable shall be cache aligned */
typedef struct adk_rcu_state_s
{
    adk_rcu_ctr_t adk_rcu_qp_ctr_g;
    adk_rcu_ctr_t adk_rcu_reader_qs_gp[ADK_MAX_NUM_CORES];
}adk_rcu_state_t;

extern adk_rcu_state_t *adkRcuState_p;


/**************************************************************************
 *                 RCU read side functions                                *
 **************************************************************************/
#define adk_rcu_read_lock
#define adk_rcu_read_unlock

static inline void adk_rcu_quiescent_state(void) __attribute__((always_inline));
static inline void adk_rcu_quiescent_state(void)
{
    _mm_mfence();
    if(adkRcuState_p)
        adkRcuState_p->adk_rcu_reader_qs_gp[THREAD_IDX].ctr = ACCESS_ONCE(adkRcuState_p->adk_rcu_qp_ctr_g.ctr) + 1;
    _mm_mfence();
}

static inline void adk_rcu_quiescent_offline(void) __attribute__((always_inline));
static inline void adk_rcu_quiescent_offline(void)
{
    _mm_mfence();
    if(adkRcuState_p)
        adkRcuState_p->adk_rcu_reader_qs_gp[THREAD_IDX].ctr = ACCESS_ONCE(adkRcuState_p->adk_rcu_qp_ctr_g.ctr);
}

static inline void adk_rcu_quiescent_online(void) __attribute__((always_inline));
static inline void adk_rcu_quiescent_online(void)
{
    if(adkRcuState_p)
        adkRcuState_p->adk_rcu_reader_qs_gp[THREAD_IDX].ctr = ACCESS_ONCE(adkRcuState_p->adk_rcu_qp_ctr_g.ctr) + 1;
    _mm_mfence();
}


/* Prototypes */
adk_status_t adk_rcu_init(adk_sys_handle_t sysHandle);
int adk_rcu_config_get(adk_sys_handle_t sys_handle);
void adk_rcu_synchronize(void);
void adk_rcu_distributed_ver_update(void);
adk_status_t adk_rcu_distributed_ver_verify(void);
void adk_rcu_distributed_wait(void);

#endif
#endif /* __ADK_RCU_H__ */
