/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_RCU_DISTRIBUTED_H__
#define __ADK_RCU_DISTRIBUTED_H__


typedef struct adk_rcu_pending_list_s
{
    void        *entry;
    struct adk_rcu_pending_list_s *next;
}adk_rcu_pending_list_t;

void adk_rcu_shm_free_cb_fn(void* rock, void *handle);

#define ADK_RCU_DISTRIBUTED_PENDING_LIST_FREE(pending_list, list_count, callback, rock)\
    do{\
        adk_rcu_pending_list_t *list_next = NULL; \
        adk_rcu_pending_list_t *free_pending_list = NULL; \
        if(pending_list)\
        {\
            callback(rock,pending_list->entry); \
            list_next = pending_list->next; \
            while(list_next != NULL) \
            { \
                free_pending_list = list_next; \
                list_next = list_next->next; \
                callback(rock,free_pending_list->entry); \
                ADK_DP_SHM_FREE(free_pending_list); \
            } \
            ADK_DP_SHM_FREE(pending_list);\
            pending_list = NULL; \
            list_count = 0; \
        }\
    }while(0)

#define ADK_RCU_DISTRIBUTED_PENDING_LIST_ARRAY_FREE(pending_list, list_count, callback, rock)\
    do{\
          uint32_t temp_count = 0;\
          while(temp_count < list_count) \
          { \
              callback(rock,pending_list[temp_count]); \
              temp_count++;\
          } \
          list_count = 0; \
    }while(0)

#define ADK_RCU_DISTRIBUTED_PENDING_LIST_CHECK(pending_list, list_count, callback, rock)\
    do{\
      if ( (pending_list != NULL) && (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)) \
      { \
          ADK_RCU_DISTRIBUTED_PENDING_LIST_FREE(pending_list, list_count, callback, rock);\
      } \
    }while(0)

#define ADK_RCU_DISTRIBUTED_PENDING_LIST_ARRAY_CHECK(pending_list, list_count, callback, rock)\
    do{\
      if ( (list_count != 0) && (adk_rcu_distributed_ver_verify() == ADK_SUCCESS)) \
      { \
          ADK_RCU_DISTRIBUTED_PENDING_LIST_ARRAY_FREE(pending_list, list_count, callback, rock);\
      } \
    }while(0)

#define ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST( module_id, obj_id, pending_list, entry_val, count, max, callback, rock) \
    do{ \
        adk_rcu_pending_list_t *pending_list_next = NULL; \
        adk_rcu_distributed_ver_update(); \
        \
        if ( count >= max) \
        { \
            adk_rcu_distributed_wait();\
            callback(rock, (void*)entry_val); \
            ADK_RCU_DISTRIBUTED_PENDING_LIST_FREE(pending_list, count, callback, rock); \
        } \
        else \
        { \
            pending_list_next = pending_list; \
            pending_list = (adk_rcu_pending_list_t*) \
                               ADK_DP_SHM_CALLOC(module_id, obj_id, \
			       1, sizeof(adk_rcu_pending_list_t), 0); \
            if ( pending_list == NULL)\
            {\
               adk_rcu_distributed_wait();\
               callback(rock,(void*)entry_val); \
               ADK_RCU_DISTRIBUTED_PENDING_LIST_FREE(pending_list_next, count, callback, rock); \
            } \
            else \
            { \
                pending_list->entry = (void*) entry_val; \
                pending_list->next = pending_list_next; \
                count++; \
            }\
         } \
    }while(0)


#define ADK_RCU_RELEASE_DISTRIBUTED_PENDING_LIST_ARRAY( pending_list, entry_val, count, max, callback, rock) \
    do{ \
        adk_rcu_distributed_ver_update(); \
        \
        if ( count >= max) \
        { \
            adk_rcu_distributed_wait();\
            callback(rock, (void*)entry_val); \
            ADK_RCU_DISTRIBUTED_PENDING_LIST_ARRAY_FREE(pending_list, count, callback, rock); \
        } \
        else \
        { \
            pending_list[count] = (void*) entry_val; \
            count++; \
        } \
    }while(0)



#endif /* __ADK_RCU_DISTRIBUTED_H__ */
