/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#include "adk_include.h"
#include "adk_rm.h"

/*
 * TODO: Replace return bool type with proper error code.
 * TODO: Use one bit per resource instead of one byte.
 * TODO: Embed proper logging esp. upon error.
 */

adk_rm_hdl_t
adk_rm_resource_init(size_t mod_id, size_t obj_id, size_t size)
{
    adk_rm_rsrc_t *new_res = NULL;

    if (size > ADK_MAX_RM_RSRC_SIZE)
    {
        goto _adk_rm_resource_init_exit;
    }

    new_res = (adk_rm_rsrc_t *)ADK_NVM_CALLOC(mod_id, obj_id, 1, sizeof(adk_rm_rsrc_t));
    if (new_res == NULL)
    {
        goto _adk_rm_resource_init_exit;
    }

    new_res->resource = (char *)ADK_NVM_CALLOC(mod_id, obj_id, size, sizeof(char));
    if (new_res->resource == NULL)
    {
        ADK_NVM_FREE(new_res);
        new_res = NULL;
        goto _adk_rm_resource_init_exit;
    }
    new_res->size = size;

_adk_rm_resource_init_exit:

    return ((adk_rm_hdl_t)new_res);
}

size_t
adk_rm_resource_reserve(adk_rm_hdl_t hdl)
{
    uint32_t id = 0;
    adk_rm_rsrc_t *res = (adk_rm_rsrc_t *)hdl;

    /* Skip invalid ID = 0 */
    for(id = 0;id < res->size;id++)
    {
        if(res->resource[id] == 0)
        {
            res->resource[id] = 1;
            return id;
        }
    }
    return ADK_RM_RSRC_INVALID;
}

bool
adk_rm_resource_reserve_fixed(adk_rm_hdl_t hdl, 
		              size_t id)
{
    adk_rm_rsrc_t *res = (adk_rm_rsrc_t *)hdl;
    
    if (id >= res->size)
    {
        return FALSE;
    }
    
    if(res->resource[id] == 1)
    {
        return FALSE;
    }
    else
    {
        res->resource[id] = 1;
        return TRUE;
    }
}

bool
adk_rm_resource_free(adk_rm_hdl_t hdl, 
		     size_t id)
{
    adk_rm_rsrc_t *res = (adk_rm_rsrc_t *)hdl;

    if (id >= res->size)
    {
        return FALSE;
    }
    
    res->resource[id] = 0;
    return TRUE;
}

void adk_rm_resource_destroy(adk_rm_hdl_t hdl)
{
    adk_rm_rsrc_t *res = (adk_rm_rsrc_t *)hdl;

    if (res)
    {
        if (res->resource)
        {
            ADK_NVM_FREE(res->resource);
        }
        ADK_NVM_FREE(res);
    }
}

bool
adk_rm_resource_isset(adk_rm_hdl_t hdl,
                      size_t id)
{
    adk_rm_rsrc_t *res = (adk_rm_rsrc_t *)hdl;

    if (id >= res->size)
    {
        return FALSE;
    }

    if(res->resource[id] == 1)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


