/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
#ifndef __ADK_TIMER_H__
#define __ADK_TIMER_H__

#ifdef ADK_TIMER


#define ADK_TIMER_PROC_PKT_PERIOD 100
#define ADK_TIMER_WHEEL_SIZE      256
#define ADK_TIMER_MAX_NUMBER      8192 /*Max number of timers, should be less than 2^16*/

typedef enum adk_timer_event_type_e {
    ADK_TIMER_EVENT_TYPE_TIMEOUT=0,
    ADK_TIMER_EVENT_TYPE_DELETE
} __attribute__((__packed__)) adk_timer_event_type_t;

typedef void (*adk_timer_cb_fn_t) (void *rock,
                                   adk_timer_event_type_t event_type);

typedef struct _adk_timer_info_s
{
    uint32_t interval_milli_sec; /* interval milli seconds */
} adk_timer_info_t;

typedef struct _adk_timer_list_node_info_s adk_timer_list_node_info_t;
struct _adk_timer_list_node_info_s
{
    adk_timer_cb_fn_t cbFn;
    void *rock;
    uint32_t timer_hash_offset;
    uint16_t cur;
    uint16_t next;
    uint16_t prev;
    uint16_t next_free;
    uint8_t timer_wheel_index;
    uint8_t coreIndex;
    uint8_t version_id;
}__attribute__((aligned(32))) ;

typedef struct _adk_timer_wheel_entry_info_s
{
    uint16_t list_head;
} adk_timer_wheel_entry_info_t;

typedef uint32_t adk_timer_hdl_t;

typedef struct _adk_timer_wheel_instance_s
{
    uint32_t cpu_tick_prev;
    uint32_t cpu_ticks_per_timer_tick;
    uint16_t timer_node_free_list;
    uint8_t cur_timer_wheel_index;
    adk_timer_wheel_entry_info_t timer_wheel[ADK_TIMER_WHEEL_SIZE];
    adk_timer_list_node_info_t timer_node_pool[ADK_TIMER_MAX_NUMBER];
    struct rte_ring* early_expiry_list[ADK_MAX_NUM_CORES];
}__attribute__((aligned(64))) adk_timer_wheel_instance_t;

/* =============================================================
 * This API creates a timer on the current lcore_id. Upon the timeout, the
 * "cbFn" will be invoked with event "ADK_TIMER_EVENT_TYPE_TIMEOUT" on the same
 * lcore. This API is not multi-worker thread safe & hence should be invoked
 * only by one worker thread (Recommended to be invoked by worker thread running
 * on lcore "coreIndex")
 * =============================================================
 */
int adk_timer_create(uint16_t coreIndex,
                     adk_timer_info_t timer_info,
		     adk_timer_cb_fn_t cbFn,
		     void *rock,
		     adk_timer_hdl_t *timer_hdl);

/* =============================================================
 * This API is multi-worker thread safe API that deletes a timer from the
 * timer wheel runnig on any lcore. It is not always guaranteed to be symmetric.
 * Hence upon the actual deletion of the timer, "cbFn" will be invoked with
 * event "ADK_TIMER_EVENT_TYPE_DELETE" on the same lcore on which the timer was
 * installed. 
 * =============================================================
 */
int adk_timer_delete(uint16_t coreIndex, adk_timer_hdl_t timer_hdl);

/* =============================================================
 * This API updates the timeout value for a timer on the current lcore_id.
 * This API is not multi-worker thread safe & hence should be invoked
 * only by one worker thread (Recommended to be invoked by worker thread running
 * on lcore "coreIndex")
 * =============================================================
 */
int adk_timer_update(uint16_t coreIndex, 
                     adk_timer_hdl_t timer_hdl,
                     adk_timer_info_t timer_info);

int adk_timer_remaining_time(uint16_t coreIndex, 
                             adk_timer_hdl_t timer_hdl,
                             adk_timer_info_t *timer_info);

void adk_timer_proc(uint16_t coreIndex, uint32_t tick);

void adk_timer_init(uint16_t coreIndex, uint32_t cpu_tick);

#endif 

#endif
