/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **  This file contains the source code for adkBfdStats utility
 **  
 ***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_bfd.h>
#include <adkCommonUtilDefs.h>


static const char *
bfdmon_state2str(uint32_t state);

static const char *
bfdmon_diag2str(uint32_t diag);

int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t bfd_mod_handle;
    adk_bfd_ctx_stat_t      ctxStat;
    adk_bfd_global_stat_t   bfdGlobalStat;
    adk_flow_handle_t       bfdFlow;

    uint32_t                hours, minutes, seconds;

    if ((argc != 1) && (argc != 2)) {
        printf("Usage: %s \n", argv[0]);
        return 1;
    }

     /* Initialize EAL as secondary process */
    int     rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "secondary";
    char    eal_argv_3[] = "--log-level=1";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if(rte_ret < 0) {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_BFD_MODULE_ID, NULL,
                                &bfd_mod_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: BFD module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }

    if (argc == 2) {
        printf("---------------BFD Flow state ----------\n");

        bfdFlow  = (adk_flow_handle_t) strtoul(argv[1], NULL, 10);

        memset(&ctxStat, 0x0, sizeof(ctxStat));
        ret = adk_bfd_ctx_stat_get(bfd_mod_handle, bfdFlow, &ctxStat);
        if( ret != ADK_SUCCESS)
        {
            printf("Fail to retrieve BFD context statistics\n");
            ADK_UTILS_RETURN(1);
        }

        printf("BFD flow statistics: \n");
        printf("\t Flow handle: %"ADKPRIh", role:%s \n",
                bfdFlow,
                ctxStat.role ? "Passive":"Active");
        printf("\t Local Stats:\n");
        printf("\t\tDiscr: 0x%08x, state: %s, mode:%s, diag:%s\n",
                ctxStat.localDiscr,
                bfdmon_state2str(ctxStat.localState),
                ctxStat.localMode ? "Demand" : "Async",
                bfdmon_diag2str(ctxStat.localDiag));
        printf("\t\tDesired min Tx: %u (us), Required min Rx: %u (us), Required min Echo Rx: %u (us)\n\t\tSlow time: %u (us), Multiplier:%d\n",
                ctxStat.localMinTx,
                ctxStat.localMinRx,
                ctxStat.localMinEchoRx,
                ctxStat.slowTime,
                ctxStat.localMultiplier);

        printf("\t\tDetection Time: %u (us)\n", ctxStat.detectionTime);

        printf("\t Remote Stats:\n");
        printf("\t\tDiscr: 0x%08x, state: %s, mode:%s, diag:%s\n",
                ctxStat.remoteDiscr,
                bfdmon_state2str(ctxStat.remoteState),
                ctxStat.remoteMode ? "Demand" : "Async",
                bfdmon_diag2str(ctxStat.remoteDiag));
        printf("\t\tDesired min Tx: %u (us), Required min Rx: %u (us), Required min Echo Rx: %u (us)\n\t\tMultiplier:%d\n",
                ctxStat.remoteMinTx,
                ctxStat.remoteMinRx,
                ctxStat.remoteMinEchoRx,
                ctxStat.remoteMultiplier);

        printf("\t Packet counters:\n");
        printf("\t\tReceived Ctrl packets: %u\n", ctxStat.numCtrlPktRx);
        printf("\t\tDropped Ctrl packets: %u\n", ctxStat.numCtrlPktDrop);
        printf("\t\tTransmitted Ctrl packets: %u\n", ctxStat.numCtrlPktTx);

        hours = ctxStat.upTime / 3600;
        minutes = (ctxStat.upTime % 3600) / 60;
        seconds = (ctxStat.upTime % 3600) % 60;
        printf("\t Up time: %02u:%02u:%02u\n\n", hours, minutes, seconds);

        ADK_UTILS_RETURN(0);
           
    } else {

        ret = adk_bfd_global_stat_get(bfd_mod_handle, &bfdGlobalStat);
        if (ret != ADK_SUCCESS) {
            printf("Error: Failed to dump global stats!\n");
            ADK_UTILS_RETURN(1);
        }

        printf("---------------BFD Global Counters------------------\n");
        printf("\t\t Received Ctrl packets:     %u\n", bfdGlobalStat.numCtrlPktRx);
        printf("\t\t Transmitted Ctrl packets:  %u\n", bfdGlobalStat.numCtrlPktTx);
        printf("\t\t numTtlExc packets:         %u\n", bfdGlobalStat.numTtlExc);
        printf("\t\t numVerExc packets:         %u\n", bfdGlobalStat.numVerExc);
        printf("\t\t numDetMulExc packets:      %u\n", bfdGlobalStat.numDetMulExc);
        printf("\t\t numMbitExc packets:        %u\n", bfdGlobalStat.numMbitExc);
        printf("\t\t numPFSetExc packets:       %u\n", bfdGlobalStat.numPFSetExc);
        printf("\t\t numAuthExc packets:        %u\n", bfdGlobalStat.numAuthExc);
        printf("\t\t numPktlenExc packets:      %u\n", bfdGlobalStat.numPktlenExc);
        printf("\t\t numMyDiscExc packets:      %u\n", bfdGlobalStat.numMyDiscExc);
        printf("\t\t numYourDiscExc packets:    %u\n", bfdGlobalStat.numYourDiscExc);
        printf("\t\t numHashExc packets:        %u\n", bfdGlobalStat.numHashExc);
        printf("\t\t numDummyCtxExc packets:    %u\n", bfdGlobalStat.numDummyCtxExc);
        printf("\t\t numInvalidCtxExc packets:  %u\n", bfdGlobalStat.numInvalidCtxExc);
        printf("----------------------------------------------------\n");
    }
    ADK_UTILS_RETURN(0);
}

static const char *
bfdmon_state2str(uint32_t state)
{
    switch(state)
    {
        case BFD_STATE_ADMIN_DOWN:  return "AdminDown";
        case BFD_STATE_DOWN:        return "Down";
        case BFD_STATE_INIT:        return "Init";
        case BFD_STATE_UP:          return "Up";
        default:                    return "Unknown";
    }
}


static const char *
bfdmon_diag2str(uint32_t diag)
{
    switch(diag)
    {
        case BFD_DIAG_NONE: return "None";
        case BFD_DIAG_DETECT_TIME_EXPIRED:      return "Detect Time Expired";
        case BFD_DIAG_ECHO_FUNC_FAILED:         return "Echo Func Failed";
        case BFD_DIAG_NEIGH_SESSION_DOWN:       return "Neigh Session Down";
        case BFD_DIAG_FORWARD_PLANE_RESET:      return "Forward Plane Reset";
        case BFD_DIAG_PATH_DOWN:                return "Path Down";
        case BFD_DIAG_CONCAT_PATH_DOWN:         return "Concat Path Down";
        case BFD_DIAG_ADMIN_DOWN:               return "Admin Down";
        case BFD_DIAG_REVERSE_CONCAT_PATH_DOWN: return "Reverse Concat Path Down";
        default:                                return "Unknown";
                            
    }
}

