/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2001 - 2016 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/

/*
 * =====================================================================================
 *
 *       Filename:  adkETHOAMDumpCCMonStates.c
 *
 *    Description:  Utility to print OAM and RDI states 
 *
 *        Version:  1.0
 *        Created:  02/01/2016 12:00:31 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Scott Titus (sdt), scott.titus@lsi.com
 *        Company:  LSI
 *
 * =====================================================================================
 */

/*!
 **
 **     $HeadURL: http://txasoft2.an.intel.com/svn/software/adk/leo/trunk/ncp_adk/test/utils/adkETHOAMDumpCCMonStates.c $
 **     $Revision: 189389 $
 **     $Date: 2016-02-02 23:31:22 +0530 (Tues, 02 Feb 2016) $
 **
 **  This file contains the source code for adkETHOAMDumpCCMonStates utility
 **  
 ***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_eth_oam.h>
#include <adk_eth_oam_tables.h>
#include <adkCommonUtilDefs.h>

void print_usage(char *argv[]);

void print_usage(char *argv[])
{
	printf("Usage: %s <CC Mon hdl>\n", argv[0]);
}

int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t eth_oam_mod_hdl;
    adk_flow_handle_t mon_hdl;

    if (argc != 2) {
        printf("Error: Invalid number of arguments to %s\n", argv[0]);
	print_usage(argv);
        return 1;
    }

    /* Initialize EAL as secondary process */
    int     rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];
    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "secondary";
    char    eal_argv_3[] = "--log-level=1";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if (rte_ret < 0) {
        printf("Error: rte_eal_init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_ETH_OAM_MODULE_ID,
                                NULL,
                                &eth_oam_mod_hdl);
    if (ret != ADK_SUCCESS) {
        printf("Error: Ethernet OAM module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }
    mon_hdl = (adk_flow_handle_t) strtoul(argv[1], NULL, 0);
    if(mon_hdl == 0)
    {
        printf("Error: Invalid CC Mon Flow Handle \n");
    	print_usage(argv);
        ADK_UTILS_RETURN(1);
    }
    adk_eth_oam_ccm_mon_state_dump(eth_oam_mod_hdl, mon_hdl);
    ADK_UTILS_RETURN(0);
}
