/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **     $HeadURL: http://txasoft2.an.intel.com/svn/software/adk/leo/trunk/ncp_adk/test/utils/adkETHOAMStats.c $
 **     $Revision: 190125 $
 **     $Date: 2016-01-24 21:11:09 +0530 (Sun, 24 Jan 2016) $
 **
 **  This file contains the source code for adkETHOAMStats utility
 **  
 ***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_sif.h>
#include <adk_eth_oam.h>
#include <adkCommonUtilDefs.h>

void print_usage(char *argv[]);

void print_usage(char *argv[])
{
	printf("Usage: %s <stats_type> <hdl>\n", argv[0]);
	printf("Where,\n");
	printf("a. Options for <stats_type> are: [1-3] \n");
	printf("\t (1) Eth Oam CFM Flow Stats\n");
	printf("\t (2) Eth Oam Global Stats (Use <hdl> as 0)\n");
	printf("\t (3) Eth Oam FDM Stats\n");
	printf("b. Options for <hdl> are: \n");
	printf("\t - Use stats handle in hex for CFM Flow statistics.\n");
	printf("\t - Use monitoring flow id for FDM Flow statistics.\n");
	printf("\t - Use 0 for global statistics.\n");
}

int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t eth_oam_mod_handle;
    uint32_t stats_type = 0;
    uint32_t flow_id = 0;
    adk_eth_oam_stats_hdl_t stats_hdl = 0;

    if (argc != 3) {
        printf("Error: Invalid number of arguments to %s\n", argv[0]);
        print_usage(argv);
        return 1;
    }
    stats_type = (uint32_t) atoi(argv[1]);
    printf("Stats Type is %d\n", stats_type);
    if(stats_type == 1)
    {
        stats_hdl = (adk_flow_handle_t) strtoul(argv[2], NULL, 0);
        if(stats_hdl == 0)
        {
            printf("Error: Invalid Stats Handle or Flow Id\n");
            print_usage(argv);
            return 1;
        }
    }else if(stats_type == 3) {
       flow_id = (uint32_t )atoi(argv[2]);
    }

    printf("Stats Handle is %p\n", stats_hdl);
    /* Initialize EAL as secondary process */
    int     rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "secondary";
    char    eal_argv_3[] = "--log-level=1";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if(rte_ret < 0) {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_ETH_OAM_MODULE_ID, NULL,
                                &eth_oam_mod_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: ETH OAM module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }
    
    switch(stats_type)
    {
        case 1:
            adk_eth_oam_stats_print(eth_oam_mod_handle, stats_hdl);
            break;
        case 2:
            adk_eth_oam_global_stats_print(eth_oam_mod_handle);    
            break;
        case 3:
            adk_eth_oam_fdm_statistics_print(eth_oam_mod_handle, flow_id);    
            break;
        default:
            printf("Error: Unknown Stats Type Specified!\n");
            print_usage(argv);
            break;
    }

    ADK_UTILS_RETURN(0);
}
