/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
**
**  This file contains the source code for adkEthernetStats utility.
**  This utility can dump the Ethernet global stats if no command line
**  argument is specified. It can dump Ethernet physical interface stats
**  if port number is specified as command line argument.
**
***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_ethernet.h>
#include <adkCommonUtilDefs.h>

int
main (
    int argc,
    char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t eth_mod_handle;
#ifdef ADK_DEBUG
    /*provide the if_handle created by CP application*/
    adk_interface_handle_t if_handle = 0;
#endif
    if ((argc != 1) && (argc != 2))
    {
        printf("Usage: %s [port_number]\n", argv[0]);
        printf("    Note: Dumps global stats if no port number specified.\n");
        return 1;
    }

    /* Initialize EAL as secondary process */
    int rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];

    char eal_argv_0[] = "";
    char eal_argv_1[] = "--proc-type";
    char eal_argv_2[] = "secondary";
    char eal_argv_3[] = "--log-level=1";
    char eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if (rte_ret < 0)
    {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS)
    {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_ETHERNET_MODULE_ID, NULL,
                                &eth_mod_handle);
    if (ret != ADK_SUCCESS)
    {
        printf("Error: Ethernet module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }

    if (argc == 2)
    {
#ifdef ADK_DEBUG
        ret = adk_ethernet_bridge_interface_statistics_print(eth_mod_handle, if_handle);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump L2 interface stats!\n");
            ADK_UTILS_RETURN(1);
        }
#endif
    }
    else
    {
#ifdef ADK_DEBUG
        ret = adk_ethernet_global_stats_print(eth_mod_handle);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump global stats!\n");
            ADK_UTILS_RETURN(1);
        }
#endif
    }

    ADK_UTILS_RETURN(0);
}
