/* ***************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/*!
**
**  This file contains the source code for adkHosDumpTable utility.
**  This utility can dump the tables maintained in the hardware offload
**  engines.
**
***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_hos.h>
#include <adkCommonUtilDefs.h>

void
print_usage (
    char *prog_name);

int
main (
    int argc,
    char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    char optchar;
    adk_hos_object_t obj;
    bool dump_stats = false, dump_tables = false;
    uint8_t ffu_dump_level = 0, i;
    uint16_t intopt = 0;

    /* At least one option must be specified */
    if (argc < 2)
    {
        print_usage(argv[0]);
        return (1);
    }

    /* Initialize EAL as secondary process */
    int rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];

    char eal_argv_0[] = "";
    char eal_argv_1[] = "--proc-type";
    char eal_argv_2[] = "secondary";
    char eal_argv_3[] = "--log-level=1";
    char eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if (rte_ret < 0)
    {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    optchar = getopt(argc, argv, "hfstbd:");
    switch (optchar)
    {
    case 'h': print_usage(argv[0]);
        ADK_UTILS_RETURN(0);

    case 'f':
        if (argc == 3)
            ffu_dump_level = strtol(argv[2], NULL, 16);
        else 
            ffu_dump_level = 1;
        break;

    case 'd':
        if (!strcmp(optarg, "port"))
        {
            if (argc == 4)
            {
                intopt = atoi(argv[3]);
            }
            else
            {
                print_usage(argv[0]);
                ADK_UTILS_RETURN(1);
            }
        }
        else
        {
            print_usage(argv[0]);
            ADK_UTILS_RETURN(1);
        }
        break;

    default: /* Unknown option */
        print_usage(argv[0]);
        ADK_UTILS_RETURN(1);
        break;
    }

    if (dump_tables == true)
    {
        for (obj = ADK_HOS_INVALID_OBJ + 1; obj < ADK_HOS_MAX_OBJ; obj++)
        {
            /* Application should iterate through rules and write it's own dump utility */
            /* adk_hos_rules_dump(obj); */
        }
    }


    if (dump_stats == true)
    {
        if (argc == 3)
        {
            intopt = atoi(argv[2]);
            adk_hos_dump_stats(intopt);
        }
        else
        {
            for (i = 0; i < 23; i++)
            {
                adk_hos_dump_stats(i);
            }
        }
    }


    ret = adk_sys_detach(sys_handle);
    if (ret != ADK_SUCCESS)
    {
        printf("Error: Failed to detach from ADK System!\n");
        ADK_UTILS_RETURN(1);

    }

    ADK_UTILS_RETURN(0);
}

void
print_usage (
    char *prog_name)
{
    printf("Usage: %s <-h|-f|-d|-s|-t|-b> <option>\n", prog_name);
    printf("Note: Dumps HLP and HOS tables and HLP stats\n");
    printf("Options: \n");
    printf("\t-h\tPrint this help on usage\n");
    printf("\t-f <level, 1 or 2>\tDump HLP FFU tables\n");
    printf("\t-s <port>\tDump HLP statistics\n");
    printf("\t-t\tDump ADK defined tables\n");
    printf("\t-b [L2 domain]\tDump floodsets\n");
    printf("\t-d [dump-type], where dump-type can be\n");
    printf("\t\tdomains\tFor Domain table entries\n");
    printf("\t\tmacfdb\tFor MAC FDB table\n");
    printf("\t\ttriggers\tFor HLP triggers\n");
    printf("\t\tport [port no]\tFor HLP port\n");
    printf("\t\tvlans\tFor VLANs\n");

    adk_hos_dump_hlp_version();

    return;
}
