/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **     $HeadURL:$
 **     $Revision:$
 **     $Date:$
 **
 **  This file contains the source code for adkIPv6DumpTables utility
 **  
 ***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_ipsec.h>
#include <adkCommonUtilDefs.h>

void print_usage(char *prog_name);

int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t ipsecHandle;
    char optchar;
    bool dumpAllSAs = true, dumpISAs = false, dumpOSAs = false;
    bool dumpPol = true, dumpMap = true;


    if (argc > 3)
    {
        print_usage(argv[0]);
    }
            
    if ((argc == 2) || (argc == 3)) {
        optchar = getopt(argc, argv, "siopmh:");
        switch(optchar) {
        case 'h': print_usage(argv[0]);
                  return(0);
        case 's': dumpAllSAs = true;
                  dumpPol = false;
                  dumpMap = false;
                  break;
        case 'i': dumpISAs = true;
                  dumpAllSAs = false;
                  dumpPol = false;
                  dumpMap = false;
                  break;
        case 'o': dumpOSAs = true;
                  dumpAllSAs = false;
                  dumpPol = false;
                  dumpMap = false;
                  break;
        case 'p': dumpPol = true;
                  dumpAllSAs = false;
                  dumpMap = false;
                  break;
        case 'm': dumpPol = false;
                  dumpAllSAs = false;
                  break;
        default: /* Unknown option */
                  print_usage(argv[0]);
                  return(1);
                  break;
        }
    }

    /* Initialize EAL as secondary process */
    int     rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "auto";
    char    eal_argv_3[] = "--log-level=5";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if (rte_ret < 0) {
        printf("Error: rte_eal_init failed!\n");
        return 1;
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_IPSEC_MODULE_ID,
                                adk_ipsec_service_function_update,
                                &ipsecHandle);
    if (ret != ADK_SUCCESS) {
        printf("Error: IPsec module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }

#ifdef ADK_DEBUG
    if (dumpAllSAs == true)
    {
        ret = adk_ipsec_dump_sa(ipsecHandle, ADK_IPSEC_OUTBOUND);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump IPsec SA tables!\n");
            ADK_UTILS_RETURN(1);
        }
        ret = adk_ipsec_dump_sa(ipsecHandle, ADK_IPSEC_INBOUND);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump IPsec SA tables!\n");
            ADK_UTILS_RETURN(1);
        }
    }
    else if (dumpISAs == true)
    {
        ret = adk_ipsec_dump_sa(ipsecHandle, ADK_IPSEC_INBOUND);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump IPsec inbound SA table!\n");
            ADK_UTILS_RETURN(1);
        }
    }
    else if (dumpOSAs == true)
    {
        ret = adk_ipsec_dump_sa(ipsecHandle, ADK_IPSEC_OUTBOUND);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump IPsec outbound SA table!\n");
            ADK_UTILS_RETURN(1);
        }
    }

    if (dumpPol == true)
    {
        ret = adk_ipsec_dump_policies(ipsecHandle);
        if (ret != ADK_SUCCESS)
        {
            printf("Error: Failed to dump IPsec outbound SA table!\n");
            ADK_UTILS_RETURN(1);
        }
    }
#endif

    ADK_UTILS_RETURN(0);
}

void print_usage(char *prog_name)
{
    printf("Usage: %s [-h|-s|-i|-o|-p|-m]\n", prog_name);
    printf("Options: \n");
    printf("\t-h\tPrint this help on usage\n");
    printf("\t-s\tSA Tables\n");
    printf("\t-i\tInbound SA Table \n");
    printf("\t-o\tOutbound SA Table\n");
    printf("\t-p\tPolicies \n");
    printf("\t-m\tSA SP Map \n");

    return;
}

