/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **     $HeadURL:$
 **     $Revision:$
 **     $Date:$
 **
 **  This file contains the source code for adkIPSecstats utility
 **
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <adk_sys.h>
#include <adk_ipsec.h>
#include <adkCommonUtilDefs.h>

adk_module_handle_t ipsecModuleHandle_g;
adk_ipsec_sa_stat_t ipsecSAStat;
adk_handle_t saHandle;
void print_usage(char * argv[]);
void getIngressSA(void);
void getEgressSA(void);

#define ADK_IPSEC_INGRESS_SA 0x01
#define ADK_IPSEC_EGRESS_SA  0x02

void getIngressSA(void)
{
      if(adk_ipsec_get_sa_stat(ipsecModuleHandle_g, saHandle, ADK_IPSEC_INBOUND, &ipsecSAStat) != ADK_SUCCESS)
	{
              printf("Unable to get Ingress IPsec SA stats\n");
	        exit(0);
        }
}

void getEgressSA(void)
{
      
      if(adk_ipsec_get_sa_stat(ipsecModuleHandle_g, saHandle, ADK_IPSEC_OUTBOUND, &ipsecSAStat) != ADK_SUCCESS)
	{
              printf("Unable to get Egress IPsec SA stats\n");
	        exit(0);
        }
}
void print_usage(char * argv[])
{
        printf("\n IPSec SA statistics utility usage :");
        printf("\n %s ADK_IPSEC_INGRESS_SA <Ingress SA handle>", argv[0]);
        printf("\n %s ADK_IPSEC_EGRESS_SA <Egress SA handle>", argv[0]);
}


int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    int rte_ret = 0;
    int command;
    int32_t eal_argc = 5;
    char *eal_argv[5];

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "secondary";
    char    eal_argv_3[] = "--log-level=1";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    rte_ret = rte_eal_init(eal_argc, eal_argv);
    if(rte_ret < 0) {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }

    if ((argc != 2)) {
       print_usage(&argv[0]);
       ADK_UTILS_RETURN(1);
    }

    if(strcmp(argv[1],"ADK_IPSEC_INGRESS_SA") == 0)
    {
        command = ADK_IPSEC_INGRESS_SA;
    }
    else if(strcmp(argv[1],"ADK_IPSEC_EGRESS_SA") == 0)
    {
        command = ADK_IPSEC_EGRESS_SA;
    }
    else
    {
	printf("\n Invalid argument for SA Statistics. Usage is :");
        printf("\n\t%s ADK_IPSEC_INGRESS_SA", argv[1]);
	printf("\n\t%s ADK_IPSEC_EGRESS_SA\n", argv[1]);
        print_usage(&argv[0]);
       ADK_UTILS_RETURN(0);
    }

    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_IPSEC_MODULE_ID, NULL,
                                &ipsecModuleHandle_g);
    if (ret != ADK_SUCCESS) {
        printf("Error: IPSec module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }

    switch (command) {
    case ADK_IPSEC_INGRESS_SA:
        saHandle = (adk_handle_t)strtoul(argv[2], NULL, 16);
        getIngressSA();
        break;
    case ADK_IPSEC_EGRESS_SA:
        saHandle = (adk_handle_t)strtoul(argv[2], NULL, 16);
        getEgressSA();
        break;
    default:
        break;
    }
    ADK_UTILS_RETURN(0);
}
