/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **     $HeadURL:$
 **     $Revision:$
 **     $Date:$
 **
 **  This file contains the source code for adkIPv6DumpTables utility
 **  
 ***********************************************************************/

#include <stdio.h>
#include <adk_sys.h>
#include <adk_ipv6.h>
#include <adkCommonUtilDefs.h>

adk_ipv6_tree_conf_t ipv6TreeConf;

void print_usage(char *prog_name);

int main(int argc, char *argv[])
{
    adk_status_t ret;
    adk_sys_handle_t sys_handle;
    adk_module_handle_t ipv6_mod_handle;
    adk_ipv6_table_group_t table_group = ADK_IPV6_TABLES_ALL;
    char optchar;


    if (argc > 3) {
        print_usage(argv[0]);
    }
            
    if ((argc == 2) || (argc == 3)) {
        optchar = getopt(argc, argv, "ihrvemaRf:");
        switch(optchar) {
        case 'h': print_usage(argv[0]);
                  return(0);
        case 'r': table_group = ADK_IPV6_TABLES_ROUTING;
                  break;
        case 'R': table_group = ADK_IPV6_TABLES_REASSEMBLY;
                  break;
        case 'i': table_group = ADK_IPV6_TABLES_INTF_ATTR;
                  break;
        case 'v': table_group = ADK_IPV6_TABLES_VRD;
                  break;
        case 'e': table_group = ADK_IPV6_TABLES_EXCEPTIONS;
                  break;
        case 'm': table_group = ADK_IPV6_TABLES_DEST_MTU;
                  break;
        case 'a': table_group = ADK_IPV6_TABLES_FILTERS;
                  break;
        case 'f': if (!strcmp(optarg, "ingAssoc")) {
                      table_group = ADK_IPV6_TABLES_INGRESS_ASSOC;
                  } else if (!strcmp(optarg, "egAssoc")) {
                      table_group = ADK_IPV6_TABLES_EGRESS_ASSOC;
                  } else if (!strcmp(optarg, "egFlow")) {
                      table_group = ADK_IPV6_TABLES_NEXT_HOP;
                  } else if (!strcmp(optarg, "fcFlow")) {
                      table_group = ADK_IPV6_TABLES_FLOWCACHE;
                  } else {
                    print_usage(argv[0]);
                    return(1);
                  }
                  break;
        default: /* Unknown option */
                  print_usage(argv[0]);
                  return(1);
                  break;
        }
    }

    /* Initialize EAL as secondary process */
    int     rte_ret = 0;

    int32_t eal_argc = 5;
    char    *eal_argv[5];
    int i=0;

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "auto";
    char    eal_argv_3[] = "--log-level=5";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    do 
    {
        printf("rte_eal_init trial %d\n", i++);
        rte_ret = rte_eal_init(eal_argc, eal_argv);
    }while (rte_ret < 0);


    ret = adk_syshandle_get(TRUE, 1, &sys_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: No existing configuration found!\n");
        ADK_UTILS_RETURN(1);
    }

    ret = adk_module_handle_get(sys_handle, ADK_IPV6_MODULE_ID,
                                adk_ipv6_service_function_update,
                                &ipv6_mod_handle);
    if (ret != ADK_SUCCESS) {
        printf("Error: ipv6 module handle not found!\n");
        ADK_UTILS_RETURN(1);
    }

        /* Table group validation */
    if ((table_group <= ADK_IPV6_TABLES_NONE)
        || (table_group > ADK_IPV6_TABLES_ALL))
    {
        printf("Error: Invalid table group %d\n", table_group);
        ADK_UTILS_RETURN(1);
    }

    memset(&ipv6TreeConf, 0, sizeof(ipv6TreeConf));

    ipv6TreeConf.treeType = ADK_IPV6_LPM_TREE;
    ipv6TreeConf.group    = table_group;

    ret = adk_ipv6_tables_dump(ipv6_mod_handle, &(ipv6TreeConf));
    if (ret != ADK_SUCCESS) {
        printf("Error: Failed to dump IPv6 FPL tables!\n");
        ADK_UTILS_RETURN(1);
    }

    ADK_UTILS_RETURN(0);
}

void print_usage(char *prog_name)
{
    printf("Usage: %s [-h|-v|-e|-r|-i|-m|-f <flow-type>]\n", prog_name);
    printf("Options: \n");
    printf("\t-h\tPrint this help on usage\n");
    printf("\t-i\tIP Interface Related Tables\n");
    printf("\t-r\tRouting Related Tables\n");
    printf("\t-v\tVirtual Routing Domain Tables\n");
    printf("\t-e\tExceptions Related Tables\n");
    printf("\t-m\tIP Destination Address MTU Tables\n");
    printf("\t-a\tIPv6 Filter Tables\n");
    printf("\t-f <flow-type>, where flow-type can be\n");
    printf("\t\tingAssoc\tFor Ingress\tAssociation Flow Related Tables\n");
    printf("\t\t egAssoc\tFor Egress\tAssociation Flow Related Tables\n");
    printf("\t\t  egFlow\tFor Egress\tFlow or Next Hop Related Tables\n");
    printf("\t\t  fcFlow\tFor FC flows dump\n");

    return;
}

