/* ***************************************************************************
 **                                                                          *
 ** INTEL CONFIDENTIAL                                                       *
 ** Copyright 2011 - 2018 Intel Corporation All Rights Reserved.             *
 **                                                                          *
 ** The source code contained or described herein and all documents related  *
 ** to the source code ("Material") are owned by Intel Corporation or its    *
 ** suppliers or licensors. Title to the Material remains with Intel         *
 ** Corporation or its suppliers and licensors. The Material contains trade  *
 ** secrets and proprietary and confidential information of Intel or its     *
 ** suppliers and licensors. The Material is protected by worldwide          *
 ** copyright and trade secret laws and treaty provisions. No part of the    *
 ** Material may be used, copied, reproduced, modified, published, uploaded, *
 ** posted, transmitted, distributed, or disclosed in any way without        *
 ** Intel's prior express written permission.                                *
 **                                                                          *
 ** No license under any patent, copyright, trade secret or other            *
 ** intellectual property right is granted to or conferred upon you by       *
 ** disclosure or delivery of the Materials, either expressly, by            *
 ** implication, inducement, estoppel or otherwise. Any license under such   *
 ** intellectual property rights must be express and approved by Intel       *
 ** in writing.                                                              *
 **                                                                          *
 ** **************************************************************************
 **/
/*!
 **
 **     $HeadURL:$
 **     $Revision:$
 **     $Date:$
 **
 **  This file contains the source code for adkPhDevStats utility
 **
 ***********************************************************************/

#include <stdio.h>

#include "adk_sys.h"
#include "adk_ph_dev.h"
#include "adk_ph_dev_export.h"
#include <adkCommonUtilDefs.h>

#include "rte_eventdev.h"
#include "rte_event_framework.h"

int main(int argc, char *argv[])
{
    adk_status_t                retValue = ADK_SUCCESS;
    int32_t                     err;
    int                         i;
    adk_sys_handle_t            sysHandle;
    uint32_t                    devId;
    int                         rte_ret = 0;
    adk_ph_dev_device_info_t    phDevInfo;
    adk_ph_dev_rx_queue_info_t  rxQInfo;

    int32_t            *argc_p;
    char              **argv_p;

    int32_t             eal_argc = 5;
    char               *eal_argv[5];

    char    eal_argv_0[] = "";
    char    eal_argv_1[] = "--proc-type";
    char    eal_argv_2[] = "secondary";
    char    eal_argv_3[] = "--log-level=1";
    char    eal_argv_4[] = "--no-pci";

    eal_argv[0] = eal_argv_0;
    eal_argv[1] = eal_argv_1;
    eal_argv[2] = eal_argv_2;
    eal_argv[3] = eal_argv_3;
    eal_argv[4] = eal_argv_4;

    if (argc < 2) {
        printf("Error: Insufficient number of arguments to  %s\n", argv[0]);
        printf("Usage: %s <PH Dev ID>\n", argv[0]);
        return 1;
    }

    /* Initialize EAL as secondary process */

    if (argc == 2)
    {
        argc_p = &eal_argc;
        argv_p = eal_argv;
    } else {
        argc_p = &argc;
        argv_p = argv;
    }

    rte_ret = rte_eal_init(*argc_p, argv_p);
    if (rte_ret < 0)
    {
        printf("Error: RTE EAL init failed!\n");
        return 1;
    }
    printf("EAL Initialization successfully!\n");

#if 1
    err = rte_event_framework_subsystem_init();
    if (err)
    {
        rte_panic("rte_event_framework_subsystem_init as secondary failed!\n");
    }
#endif


    devId  = strtoul(argv[argc - 1], NULL, 16);

    retValue = adk_syshandle_get(TRUE, 1, &sysHandle);
    if (retValue != ADK_SUCCESS) {
        printf("Failed to attach to existing ADK configuration!\n");
        ADK_UTILS_RETURN(1);
    }
    printf("Retrieved ADK System handle: %"ADKPRIh"\n", sysHandle);

    retValue = adk_ph_dev_instance_get(sysHandle);
    if (retValue != ADK_SUCCESS)
    {
        printf("ADK PH Instance get Failed\n");
        ADK_UTILS_RETURN(1);
    }
    printf("\n\nADK PH Instance get Success\n");

    printf("devId: %d\n", devId);

    retValue = adk_ph_dev_device_info_get(devId, &phDevInfo);
    if (retValue != ADK_SUCCESS)
    {
        printf("Error: Failed to get PH Dev info for devID %d (%s) %d\n",
               devId, adk_sys_status_get(sysHandle, retValue), retValue);
    }

    if (phDevInfo.isActive == false)
    {
        printf("PH Dev device %d is not active\n", devId);
        ADK_UTILS_RETURN(retValue);
    }

    retValue = adk_ph_dev_device_stats_print(devId);
    if (retValue != ADK_SUCCESS)
    {
        printf("Error: Failed to dump PH Dev stats for devID %d, (%s) %d\n",
               devId, adk_sys_status_get(sysHandle, retValue), retValue);
    }

    for (i = 0; i < phDevInfo.numRxQueues; i++)
    {
        retValue = adk_ph_dev_rx_queue_info_get(devId, i, &rxQInfo);
        if (retValue != ADK_SUCCESS)
        {
            printf("Error: Failed to get PH Dev queue info for devID %d queueId %d"
                   " (%s) %d\n",
                   devId, i,
                   adk_sys_status_get(sysHandle, retValue), retValue);
        }

        retValue = adk_ph_dev_rx_queue_stats_print(devId, i);
        if (retValue != ADK_SUCCESS)
        {
            printf("Error: Failed to dump PH Dev Rx stats for devID %d, "
                    "queueId %d (%s) %d\n",
                    devId, i,
                    adk_sys_status_get(sysHandle, retValue), retValue);
        }
    }

    ADK_UTILS_RETURN(retValue);
}
