#set -x
check_sku()
{
    skuval=$(setpci -d 8086:18a0 0x354.l)
    if [ $skuval = "00000000" ]; then
        rv="high"
    elif [ $skuval = "f000f000" ]; then
        rv="medium"
    elif [ $skuval = "ffc0ffc0" ]; then
        rv="low"
    fi
    echo $rv
}

declare -A SKUMAP_QATCONF
    SKUMAP_QATCONF=([high]=./c4xxx_dev0.conf.sym.inline
                    [medium]=./c4xxx_dev0.conf.sym.inline.med
                    [low]=./c4xxx_dev0.conf.sym.inline.low)

if [[ "${BASH_SOURCE[0]}" == "${0}" ]]
then
    echo "script ${BASH_SOURCE[0]} needs to be sourced ..."
    exit 1
fi

#if [ ! -d /workspace ]
#then
#    ln -s /host/workspace /workspace
#fi

#export RDK_INSTALL=/workspace/sw/adk_snr/rte_installations/snr_ase_rdk_2002/RDK/rdk
#export _DPDK_INSTALL=${RDK_INSTALL}/dpdk
#export RTE_SDK=${_DPDK_INSTALL}
export RTE_SDK=${RDK_INSTALL}/dpdk
export RTE_TARGET=x86_64-native-linuxapp-gcc

pushd ${RDK_INSTALL}/drivers
rmmod adk_netd
rmmod ipsec_inline
rmmod usdm_drv
rmmod igb_uio
rmmod qat_c4xxx
rmmod intel_qat
rmmod hqm
rmmod ies
rmmod ice_sw_ae
rmmod ice_sw
sleep 2
modprobe crc8
modprobe uio
modprobe authenc
modprobe dh_generic


for DRIVER in ice_sw ice_sw_ae ies hqm
do
    if [ ${DRIVER} = 'ice_sw' ]
    then
        PARAMS="nd_vis=0"
    else
        PARAMS=
    fi

    echo -n "Installing ${DRIVER} ${PARAMS} ... "
    if [ -f ${DRIVER}.ko ]
    then
        insmod ${DRIVER}.ko ${PARAMS}
        RET=$?
        if [ ${RET} != 0 ]
        then
            echo "insmod returned '${RET}'"
        fi
    else
        modprobe ${DRIVER} ${PARAMS}
        RET=$?
        if [ ${RET} != 0 ]
        then
            echo "modprobe returned '${RET}'"
        fi
    fi

    echo "Done"
done

ND_VIS=`cat /sys/module/ice_sw/parameters/nd_vis`
if [ ${ND_VIS} -ne 0 ]
then
    echo "CPK base driver (ice_sw) not loaded in stack mode"
    return 1
fi

# Add the ADK NetD
PARAMS="port_type=0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1"
echo "Installing adk_netd ${PARAMS}"
insmod ${RDK_INSTALL}/drivers/adk_netd.ko ${PARAMS}

mkdir -p /lib/firmware
cp ./qat_c4xxx*.bin /lib/firmware
if [ $? -ne 0 ]
then
    echo "Copying of firmware file qat_c4xxx*.bin failed"
    return 1
fi

cd ${RDK_INSTALL}/user_modules/qat/conf_files

skuval=$(check_sku)
echo "skuval::$skuval"
echo ${SKUMAP_QATCONF[$skuval]}
cp -f ${SKUMAP_QATCONF[$skuval]} /etc/c4xxx_dev0.conf

if [ $? -ne 0 ]
then
    echo "Copying of config file c4xxx_dev0.conf failed"
    return 1
fi

cd ${RDK_INSTALL}/drivers
for DRIVER in intel_qat qat_c4xxx
do
    echo -n "Installing ${DRIVER} ... "
    if [ -f ${DRIVER}.ko ]
    then
        insmod ${DRIVER}.ko
    else
        modprobe ${DRIVER}
    fi
    echo "Done"
done


${RDK_INSTALL}/user_modules/qat/bin/adf_ctl down
if [ $? -ne 0 ]
then
    echo "ADF_CTL failed, IPsec won't work"
    return 1
fi

${RDK_INSTALL}/user_modules/qat/bin/adf_ctl up
if [ $? -ne 0 ]
then
    echo "ADF_CTL failed, IPsec won't work"
    return 1
fi

#################################
# Get the PCIe address for the QAT
DEVICE_ID=18a0
lspci -n | grep ${DEVICE_ID} || { echo "can't find qat device" && return b; }

BUS=$( lspci -n | grep ${DEVICE_ID} | cut -b 1-2 )
DEVICE=$( lspci -n | grep ${DEVICE_ID} | cut -b 4-5 )
let FUNCTION=$( lspci -n | grep ${DEVICE_ID} | cut -b 7 )

PCI_SYSFS=$( find /sys/devices/ -name "*${BUS}\:${DEVICE}\.${FUNCTION}*" )

cd ${PCI_SYSFS}
cat sriov_numvfs;
echo 2 > sriov_numvfs

if [ $? -ne 0 ]
then
    echo "Setting sriov_numvfs failed"
    return 1
fi


cd ${RTE_SDK}/${RTE_TARGET}/kmod
for DRIVER in igb_uio
do
    echo -n "Installing ${DRIVER} ... "
    if [ -f ${DRIVER}.ko ]
    then
        insmod ${DRIVER}.ko
    else
        modprobe ${DRIVER}
    fi
    echo "Done"
done

cd ${RTE_SDK}/usertools
let FUNCTION=${FUNCTION}+1
./dpdk-devbind.py -b igb_uio ${BUS}:${DEVICE}.${FUNCTION}

cd ${RDK_INSTALL}/drivers
for DRIVER in usdm_drv ipsec_inline
do
    if [ ${DRIVER} = 'ipsec_inline' ]
    then
        PARAMS="type=1"
    else
        PARAMS=
    fi

    echo -n "Installing ${DRIVER} ${PARAMS} ... "
    if [ -f ${DRIVER}.ko ]
    then
        insmod ${DRIVER}.ko ${PARAMS}
    else
        modprobe ${DRIVER} ${PARAMS}
    fi
    echo "Done"
done

mkdir -p /mnt/hugepages
mount -t hugetlbfs nodev /mnt/hugepages
echo 2048 > /proc/sys/vm/nr_hugepages

sysctl -w kernel.randomize_va_space=0

popd

export ADK_LTTNG_PATH=
#source ./adk_SNR_profile1_linux-x86_64_runtime.sh
export LD_LIBRARY_PATH=${RDK_INSTALL}/user_modules/cpk-ae-lib:${RDK_INSTALL}/user_modules/ies-api/lib:${RDK_INSTALL}/user_modules/qat/lib:/usr/lib/:${LD_LIBRARY_PATH}

#set +x
