#ifndef _PDK_INCLUDE_H
#define _PDK_INCLUDE_H

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

/**
 *
 * This file includes the APIs needed for ADK to send/receive events between 
 * Ethdev and ADK though Eventdev. 
 *
 */

#include <getopt.h>
#include <stdint.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <errno.h>
#include <semaphore.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/prctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/mman.h>
#include <assert.h>

#include <rte_mbuf.h>
#include <ies_sdk.h>
#include <ies_api_port.h>
#include <rte_pmd_ice_dsi.h>
#include <libae_client.h>
#include <rte_eventdev.h>


#define MAX_EVENTDEV_ENQUEUE_DEPTH  128

#define PDK_DEVICE_ID 50
#define EM_OK 0
#define EM_ERR 0xffffffff
#define MAX_ETH_PORTS 8
#define EM_MAX_CORES 24
#define MAX_ETH_QUEUES 20

#define ETH_TYPE_VLAN   0x8100
#define ETH_TYPE_PDK    0x8888

#define EM_EVENTDEV_ID (0x0)
#define NONEM_EVENTDEV_ID (0x1)
#define IES_MASTER 1
#define IES_SLAVE 0

#define APPL_EXIT_FAILURE(...)  do {			\
	fprintf(stderr,					\
		"Appl Error: %s:%i, %s() - ",		\
		__FILE__, __LINE__, __func__); \
	fprintf(stderr, __VA_ARGS__);			\
	fprintf(stderr, "\n\n");			\
	fflush(stderr);					\
	exit(EXIT_FAILURE);				\
} while (0)

#define ENV_MBUF_HEADROOM  RTE_PKTMBUF_HEADROOM
#define QUEUE_UNDEF 0xFFFFFFFF
#define IP_HDR_LEN 20
#define UDP_HDR_LEN 8
#define PDK_DATA_OFFSET (18)

#define ADK_CMD_ARGS " -c 0x20 -w 0x10 -r 0x20 -t 0x20 -C 0x20 -p -i "

typedef uint32_t em_status_t;
typedef uint64_t em_queue_t;
struct _pdk_event_t;
typedef struct _pdk_event_t *em_event_t;
struct nonem_config {
	uint8_t nonem_eventdev_id;
};

typedef struct {
  uint32_t num_event_queue_flows;
  uint8_t  num_event_queues;
  uint8_t  num_ordered_event_queues;
  uint8_t  num_event_ports;
  uint8_t  num_directed_event_port_queues;
  int32_t  num_events_limit;
  uint8_t  event_port_dequeue_depth;
  uint8_t  event_port_enqueue_depth;
  uint8_t  eventdev_defer_sched; /* vdev option defer_sched=on/off */
  uint32_t cq_equal_one_core_mask; /* Cores needing dequeue depth of 1 */
} em_eventdev_cfg_t;

typedef struct {
} em_conf_t;
typedef struct rte_mbuf env_mbuf_t;

typedef uint64_t pdk_bitmask_t;
typedef struct {
	uint8_t app_done;
	uint8_t app_failed;
	uint8_t app_stop;
	uint16_t cmd; 
} shared_app_common_t;
typedef struct adk_app_msg_t
{
	unsigned char ipHdr[IP_HDR_LEN];
	unsigned char udpHdr[UDP_HDR_LEN];
}__attribute__((packed))adk_app_msg_t;

em_status_t em_send(em_event_t event, em_queue_t queue);
em_event_t em_rte_mbuf_to_event(struct rte_mbuf *mbuf);
em_status_t em_attach(uint16_t em_device_id, struct nonem_config * nonem_cfg);
em_status_t em_init(void);
int em_send_multi(em_event_t *const events, int num, em_queue_t queue);


/**
 * This API returns a pointer to NONEM param
 * @param event
 *
 * @return Pointer to NonEM param
 */
void * em_event_to_nonem_param(em_event_t event);

void ies_configure(int);
int setup_port_to_queue_mapping(void);

#define MAX_EVENTDEV_DEQUEUE_DEPTH  128
#define NUM_DOMAIN_ENTRIES      (5)
#define NUM_HLP_PORTS          (20)
#define PORT_UP_TIMEOUT         5
#define IES_SLEEP               1
#define PDK_DEFAULT_MAC (0x804340020000)
#define PDK_GET_MAC(dev) PDK_DEFAULT_MAC | ((uint64_t)htons(dev) << 48)
#define REM_DEV_ID (0x96)

void ies_event_handler_func(const struct ies_event *ies_event);

/** Device ID */
typedef union {
	uint16_t u16;
} dev_id_t;

/** Ethernet MAC address */
typedef union {
	uint8_t u8[6];
	uint16_t u16[3];
} mac_addr_t;

/** Ethernet MAC address wrapped in 64-bit */
typedef union {
	uint64_t u64;
	struct {
		uint8_t reserved[2];
		mac_addr_t mac_addr;
	} s;
} mac_addr_u64_t;

/** Ethernet header */
typedef struct {
	mac_addr_t dst;
	mac_addr_t src;
	uint16_t type;
	uint8_t data[];
} eth_hdr_t;

/** VLAN tagged Ethernet header */
typedef struct {
	mac_addr_t dst;
	mac_addr_t src;
	uint32_t vlan;
	uint16_t type;
	uint8_t data[];
} eth_hdr_vlan_t;

typedef union em_event_hdr_ {
	/* 128 bytes reserved for event header */
	uint8_t u8[ENV_MBUF_HEADROOM];

	struct {
		uint32_t is_ordered; /* Set for events received on ordered queue */
		/** Eventdev Enqueue Operation */
		uint32_t op;
		em_queue_t queue;
	};
} event_hdr_t;


/**
 * Copy an eth mac address
 */
static inline void
ethaddr_copy(mac_addr_t * const dst, mac_addr_t * const src)
{
	*dst = *src;
}

static inline int pdk_bitmask_idx(int n, const pdk_bitmask_t *mask)
{
	uint64_t mask64;
	int i;
	int idx = -1;

	if (unlikely((unsigned int)(n - 1) >= 64))
		return -1;

	mask64 = *mask;

	for (i = 1; (i < n) && (mask64 > 0); i++)
		mask64 = (mask64 & (mask64 - 1));

	if (likely(mask64 > 0))
		idx = __builtin_ctzll(mask64);

	return idx;
}


static inline int pdk_bitmask_count(const pdk_bitmask_t *mask)
{
	int cnt = 0;
	uint64_t n;

	n = *mask;
	for (; n; cnt++)
		n = (n & (n - 1));
	return cnt;
}


static inline int pdk_bitmask_isset(int bit, const pdk_bitmask_t *mask)
{
	return (*mask & ((uint64_t)1 << bit)) != 0;
}


static inline void pdk_bitmask_clr(int bit, pdk_bitmask_t *mask)
{
	*mask &= ~((uint64_t)1 << bit);
}


static inline void pdk_bitmask_set(int bit, pdk_bitmask_t *mask)
{
	*mask |= ((uint64_t)1 << bit);
}


static inline int
pdk_bitmask_set_str(char *mask_str, pdk_bitmask_t * mask)
{
	char *endptr = NULL;
	uint64_t tmp;

	errno = 0;
	tmp= strtoull(mask_str, &endptr, 16);
	if (errno != 0 || mask_str[0] == '\0' || endptr == NULL || *endptr != '\0' || tmp == 0) {
		return -1;
	}
	*mask |= tmp;

	return 0;
}


/** Convert from EM event to event header */
static inline event_hdr_t *
event_to_event_hdr(em_event_t event)
{
        return (event_hdr_t *)event;
}


/** Convert from event header to env_mbuf */
static inline env_mbuf_t *
event_hdr_to_mbuf(event_hdr_t *const ev_hdr)
{
   /* mbuf located before event header */
   return (env_mbuf_t *)((uintptr_t)ev_hdr - sizeof(env_mbuf_t));
}


/** Convert from env_mbuf to event header */
static inline event_hdr_t *
mbuf_to_event_hdr(env_mbuf_t *const mbuf)
{
	/* event header starts after mbuf */
	return (event_hdr_t *)&mbuf[1];
}


/** Convert from env_mbuf to rte_mbuf */
static inline struct rte_mbuf *
env_mbuf_to_rte_mbuf(env_mbuf_t *env_mbuf)
{
	return env_mbuf;
}

#endif
