#include <pthread.h>
#include <sched.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <errno.h>
#include <semaphore.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/prctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/mman.h>

#include "pdk_include.h"
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"
#include <rte_eventdev.h>
#include <rte_bus_vdev.h>
#pragma GCC diagnostic pop
#include "adk_dp_app_config.h"
#include "adk_sys.h"
#include "adk_ph_dev.h"
#include "adk_pal.h"
#include "adk_modules_enabled.h"
#include "adk_ethernet.h"
#include "adk_ip.h"
#include "adk_ipsec.h"
#pragma GCC diagnostic ignored "-Wconversion"
#pragma GCC diagnostic ignored "-Wsign-conversion"


/**
 * Dispatch rounds for em_dispatch() during termination to properly sync the
 * cores and shutdown actions and allow for a graceful shutdown.
 */
#define TERM_DISPATCH_ROUNDS 2

#define EM_APP_DATA_SIZE (16*1024*1024)
#define NUM_VALID_INPUT 21
#define ETH_MAC_COMMON  0x02404380
#define BUF_LEN 128
#define NUM_ETH_MODULES 3

#define NB_MBUF ((RTE_MAX_LCORE * RTE_MEMPOOL_CACHE_MAX_SIZE * 2) - 1)
#define MBUF_CACHE_SIZE RTE_MEMPOOL_CACHE_MAX_SIZE
#define ETH_RX_DESC_DEFAULT (224)
#define ETH_TX_DESC_DEFAULT (512)
#define MAX_FRAME_SIZE (9728)

char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
#define EM_IES_ERROR_NAME(ies_ret) \
    ((ies_error_get_name(ies_ret, ies_error_name, \
                         IES_ERROR_MAX_NAME_SIZE) == IES_OK) ? \
     ies_error_name : "EM_UNKNOWN")

#define MAX_RX_PKT_BURST (32)
#define BATCH_SIZE		32

/* Shared memory for application */
static int shm_app_id;
void *shm_app_addr;
char *app = NULL;
char net_dev[MAX_ETH_PORTS][BUF_LEN] = { "net_ice_dsi" };
pdk_bitmask_t phys_mask;
uint16_t n_rx_queue, n_tx_queue;
uint8_t start_lport_idx[MAX_ETH_PORTS];
int acl_tbl_idx;
static uint32_t port_up_event_count = 0;
static pdk_bitmask_t port_mask_25g = 0; 
static pdk_bitmask_t port_mask_10g = 0;
extern uint16_t my_device_id;
__thread em_event_t batch_buf[BATCH_SIZE];
__thread uint32_t batch_count;

void
appl_init(void);

pid_t child_pids[EM_MAX_CORES] = { 0 };

static void
 parse_args(int argc, char *argv[]);

static void
 launch_application(int (*appl_start_fptr) (void*));

static int
 application_start(void*);

static void
 em_cmd_process(void);

int
eth_init(void);
static int
pdk_pool_create(void);
static void
eth_port_init(uint16_t portid);
void
appl_receive(em_event_t event, em_queue_t queue, em_queue_t next);
/**
 * Application process command line parameters (modified from argc, argv[])
 */
typedef struct {
	int argc; /**< process argument count */
	char **argv; /**< process arguments, allocated */
} proc_args_t;

static void
 install_sigchld_handler(void);

static void
 sigchld_handler(int sig);

static void
pdk_poll(void);

static void
usage(char *progname)
{
	printf("%s\n", progname);
}

/* Set up event dev */
void init_event_dev(void);

static dev_id_t base_dev_id;
struct rte_mempool * pdk_mempool;
extern uint8_t is_unified_em_adk_set;
extern uint8_t number_em_cores;
extern uint8_t eventdev_port_id[2][24];
extern em_queue_t g_nonem_queue_ip[20];

int em_core_count = 4;
static struct rte_eth_conf eth_port_conf = {
	.rxmode = {
		.max_rx_pkt_len = MAX_FRAME_SIZE,
		.offloads = DEV_RX_OFFLOAD_JUMBO_FRAME,
	},
	.intr_conf = {.rxq = 0}
};

static struct ice_dsi_txq_conf ice_dsi_txconf = {
	.adv_conf = {
		.qtq_quanta_size = 1024,
		.qt_cache_prof_idx = 0,
	},
};

void
em_cmd_process(void)
{
	char userInput[BUF_LEN] = "\0";

	install_sigchld_handler();
	if (fork() != 0) {
		/* Parent: return */
		return;
	}
	/* Request SIGTERM if parent dies */
	prctl(PR_SET_PDEATHSIG, SIGTERM);
	sleep(30);

	printf("Press 'q' to Quit.\n");
	while (strcmp(userInput, "q") != 0) {
		printf(">> ");
		if (EOF == scanf("%9s", userInput)) {
			printf("Scanf failed!\n");
			exit(0);
		}
	}
	exit(0);
}

/**
 * Main function
 *
 * Initialize the example application/env, create processes/threads for EM
 * and launch the EM-core specific init on each EM-core.
 */
int
main(int argc, char *argv[])
{
	int ret;
	char buf[BUF_LEN];

	/*
	 * Parse command line arguments & set config options
	 * for both application and EM
	 */
	parse_args(argc, argv);

	/* Create shared memory for app */
	sprintf(buf, "/EMShm_app%d", my_device_id);
	shm_app_id = shm_open(buf, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR);
	if (shm_app_id == -1) {
		perror("main: shm_open() for app failed\n");
		exit(1);
	}

	if (0 != ftruncate(shm_app_id, EM_APP_DATA_SIZE)) {
		perror("ERROR: main(): ftruncate() failed\n");
		exit(1);
	}

	shm_app_addr =
		mmap(NULL, EM_APP_DATA_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, shm_app_id, 0);
	if (shm_app_addr == NULL) {
		perror("main: mmap() for app failed\n");
		exit(1);
	}
	memset(shm_app_addr, 0, EM_APP_DATA_SIZE);

	printf("Mapped Shared memory for app is ready\n");

	/*
	 * Initialize the run-time environment.
	 */

	em_cmd_process();


	ies_configure(IES_MASTER);
	/*
	 * Initialize the DPDK - called once, creates threads etc.
	 */
	ret = rte_eal_init(argc, argv);
	if (ret < 0) {
		perror("Erorr:");
		APPL_EXIT_FAILURE("Cannot init the DPDK EAL!");
		return -1;
	}
	/* Initialize event dev */
	init_event_dev();
	/*
	 * Init the eth poll mode drivers
	 */
	int nb_ports = eth_init();
	if (nb_ports < 0) {
		APPL_EXIT_FAILURE("Cannot init Eth PM drivers (nb_ports=%i)!",
						  nb_ports);
		return -2;
	}
	
	appl_init();

	launch_application(application_start);
	return 0;
}

/**
 * Start application entry on each EM-core
 *
 */
static int
application_start(void *arg)
{
	(void)arg;
	printf("Entering the event dispatch loop() on  pid=%d lcore=%d\n",
	       getpid(), rte_lcore_id());

	pdk_poll();

	return 0;
}

/**
 * Parse and store relevant command line arguments. Set config options for both
 * application and EM.
 *
 * @param argc       Command line argument count
 * @param argv[]     Command line arguments
 */
static void
parse_args(int argc, char *argv[])
{
	phys_mask = 0;

	/*
	 * Parse the mandatory DPDK args (-c, -n) and save the core mask.
	 * Note: Use '+' at the beginning of optstring - don't permute the
	 *       contents of argv[].
	 * Note 2: Stops at "--"
	 */
	opterr = 0;		/* don't complain about unknown options here */
	while (1) {
		int opt;
		int long_index;
		static struct option longopts[] = {
			{"help", no_argument, NULL, 'h'},	/* return 'h' */
			{NULL, 0, NULL, 0}
		};

		opt = getopt_long(argc, argv, "+c:s:n:h", longopts, &long_index);

		if (opt == -1)
			break;	/* No more options */

		switch (opt) {
		case 'c':{
				char *mask_str = optarg;
				/*
				 * Store the core mask for EM - usage depends on the
				 * process-per-core or thread-per-core mode selected.
				 */
				if (pdk_bitmask_set_str(mask_str, &phys_mask) != 0)
					APPL_EXIT_FAILURE("Invalid coremask (%s) given\n",
							  mask_str);
				printf("EAL Coremask:   %s \n", mask_str);
				break;
			}
		case 'n':
			/* Let DPDK worry about the number of memory channels */
			break;

		case 'h':
			usage(argv[0]);
			exit(EXIT_SUCCESS);
			break;

		default:
			break;
		}
	}

	phys_mask &= ( (1 << em_core_count) - 1);
	/* Sanity check: */
	if (!phys_mask) {
		usage(argv[0]);
		APPL_EXIT_FAILURE("Give mandatory coremask!\n");
	}
}

/**
 * Create a signal handler for the SIGCHLD signal that is sent to the parent
 * process when a forked child process dies.
 */
static void
install_sigchld_handler(void)
{
	struct sigaction sa;

	sigemptyset(&sa.sa_mask);

	sa.sa_flags = 0;
	sa.sa_handler = sigchld_handler;

	if (sigaction(SIGCHLD, &sa, NULL) == -1)
		APPL_EXIT_FAILURE("sigaction() fails (errno(%i)=%s)", errno, strerror(errno));
}

/**
 * Signal handler for SIGCHLD (parent receives when child process dies)
 */
static void
sigchld_handler(int sig)
{
	int status;
	pid_t child;
	static int num = 0;

	(void)sig;		/* unused */

	if (num++)
		return;
	/* Nonblocking waits until no more dead children are found */
	do {
		child = waitpid(-1, &status, WNOHANG);
	} while (child > 0);

	if (child == -1 && errno != ECHILD)
		_exit(EXIT_FAILURE);

	/*
	 * Exit the parent process - triggers SIGTERM in the remaining children
	 * (set by prctl(PR_SET_PDEATHSIG, SIGTERM)).
	 */
	_exit(EXIT_SUCCESS);
}

/**
 * Launch the application by calling the start function 'appl_start_fptr' with
 * argument 'arg' as given by the user.
 */
static void
launch_application(int (*appl_start_fptr) (void*))
{
	int lcore_id;

	if (appl_start_fptr != NULL) {

		RTE_LCORE_FOREACH_SLAVE(lcore_id) {
			if (pdk_bitmask_isset(lcore_id, &phys_mask))
				rte_eal_remote_launch((int (*)(void*))
						      appl_start_fptr, NULL, lcore_id);
		}

		appl_start_fptr(NULL);
	}
}

void
init_event_dev(void)
{
	int rte_ret;
	uint8_t port_id, prio = 0;
	uint8_t qid;
	struct rte_event_dev_config config = { 0 };
	struct rte_event_dev_info dev_info = { 0 };
	struct rte_event_port_conf port_config = { 0 };
	struct rte_event_queue_conf queue_config = { 0 };
	struct rte_event_queue_conf default_queue_config = { 0 };
	em_eventdev_cfg_t evCfg;
	int eventdev_id = EM_EVENTDEV_ID;
	char eventdev_name[BUF_LEN] = "event_ihqm0";
	char eventdev_args[BUF_LEN] = "";

	if (rte_vdev_init(eventdev_name, eventdev_args) < 0) {
		APPL_EXIT_FAILURE("Error creating eventdev\n");
	}
	printf("Initialized eventdev: %s\n", eventdev_name);
	/* HQM config parameters */
	evCfg.num_ordered_event_queues = 0;
	evCfg.num_event_ports = 48;
	evCfg.num_directed_event_port_queues = 0;
	evCfg.cq_equal_one_core_mask = 0;

	evCfg.num_event_queues = 96;
	evCfg.num_events_limit = 1024 * 4;
	evCfg.num_event_queue_flows = 32768;
	evCfg.event_port_enqueue_depth = 16;
	evCfg.event_port_dequeue_depth = 16;

	/* Get eventDev info */
	rte_ret = rte_event_dev_info_get(eventdev_id, &dev_info);
	if (rte_ret != 0) {
		APPL_EXIT_FAILURE("rte_event_dev_info_get() fails!");
	}

	if (evCfg.num_event_queue_flows > dev_info.max_event_queue_flows ||
	    evCfg.num_event_queues > dev_info.max_event_queues ||
	    evCfg.num_event_ports > dev_info.max_event_ports ||
	    evCfg.num_directed_event_port_queues > dev_info.max_event_ports ||
	    evCfg.num_events_limit > dev_info.max_num_events ||
	    evCfg.event_port_dequeue_depth > dev_info.max_event_port_dequeue_depth ||
	    evCfg.event_port_enqueue_depth > dev_info.max_event_port_enqueue_depth) {
		APPL_EXIT_FAILURE("Maximum eventDev configuration exceeded!");
	}

	/* Set eventDev parameters */
	config.nb_event_queues = evCfg.num_event_queues;
	config.nb_event_ports = evCfg.num_event_ports;
#ifndef EM_XEON
	config.nb_single_link_event_port_queues = evCfg.num_directed_event_port_queues;
#endif
	config.nb_events_limit = evCfg.num_events_limit;
	config.nb_event_queue_flows = evCfg.num_event_queue_flows;
	config.nb_event_port_dequeue_depth = evCfg.event_port_dequeue_depth;
	config.nb_event_port_enqueue_depth = evCfg.event_port_enqueue_depth;
	config.dequeue_timeout_ns = 0;
	config.event_dev_cfg = RTE_EVENT_DEV_CFG_PER_DEQUEUE_TIMEOUT;

	/* Configure eventDev */
	rte_ret = rte_event_dev_configure(eventdev_id, &config);

	if (rte_ret) {
		APPL_EXIT_FAILURE("Failed to configure the event dev!");
	} else {
		printf("eventdev configured!\n");
	}

	/* Create event ports. One port for each core, portId=coreId */
	port_config.dequeue_depth = evCfg.event_port_dequeue_depth;
	port_config.enqueue_depth = evCfg.event_port_enqueue_depth;
	port_config.new_event_threshold = 4096;

#ifndef EM_XEON
	uint8_t start_dir_port = config.nb_event_ports - config.nb_single_link_event_port_queues;
#endif
	for (port_id = 0; port_id < config.nb_event_ports; ++port_id) {
#ifndef EM_XEON
		port_config.dequeue_depth = evCfg.event_port_dequeue_depth;
		if (port_id >= start_dir_port)
			port_config.event_port_cfg = RTE_EVENT_PORT_CFG_SINGLE_LINK;
		else {
			if (evCfg.cq_equal_one_core_mask & (1 << port_id))
				port_config.dequeue_depth = 1;
		}
#endif
		rte_ret = rte_event_port_setup(eventdev_id, port_id, &port_config);
		if (rte_ret < 0) {
			APPL_EXIT_FAILURE("Failed to create eventDev port: %d", port_id);
		}
	}
	printf("%d Eventdev ports created\n", config.nb_event_ports);

	queue_config.priority = 0;
	queue_config.nb_atomic_flows = 32;
	uint8_t num_em_q = evCfg.num_event_queues;
	uint8_t num_ord_em_q = evCfg.num_ordered_event_queues;
#ifndef EM_XEON
	uint8_t num_dir_em_q = config.nb_single_link_event_port_queues;
	uint8_t start_dir_q = num_em_q - num_dir_em_q;
#endif

	for (qid = 0; qid < num_em_q; ++qid) {
		rte_event_queue_default_conf_get(eventdev_id, qid, &default_queue_config);

		/* Set _queue_cfg to RTE_EVENT_QUEUE_CFG_ATOMIC_ONLY for atomic and parallel */
		if (qid < (num_em_q - num_ord_em_q - num_dir_em_q)) {
				queue_config.event_queue_cfg = 0;
				queue_config.nb_atomic_order_sequences = 0;
				queue_config.schedule_type = RTE_SCHED_TYPE_ATOMIC;
			} else if (qid < (num_em_q - num_dir_em_q)) {
				queue_config.event_queue_cfg = RTE_EVENT_QUEUE_CFG_ALL_TYPES;
				queue_config.nb_atomic_order_sequences = 32;
				queue_config.schedule_type = RTE_SCHED_TYPE_ORDERED;
			} else {
				queue_config.event_queue_cfg = RTE_EVENT_QUEUE_CFG_SINGLE_LINK;
				queue_config.nb_atomic_order_sequences = 0;
				//queue_config.schedule_type = RTE_SCHED_TYPE_ATOMIC;
			}

			rte_ret = rte_event_queue_setup(eventdev_id, qid, &queue_config);
			if (rte_ret < 0) {
				APPL_EXIT_FAILURE("Failed to set up event queue %d\n!", qid);
			}
			if (qid >= start_dir_q)
				port_id = start_dir_port + (qid - start_dir_q);
			else
				port_id = qid / 8;

			/* Link all queues to a port. This is needed for starting eventDev. The ports will 
			   be unlinked after the event dev is started. The ports will be linked when EM queues 
			   are created, based on the coremask of the queues */
			rte_ret = rte_event_port_link(eventdev_id, port_id, &qid, &prio, 1);
			if (rte_ret < 1) {
				APPL_EXIT_FAILURE("Failed to link queues to port!");
			}
		}

	/* Start eventDev */
	rte_ret = rte_event_dev_start(eventdev_id);
	if (rte_ret) {
		APPL_EXIT_FAILURE("Failed to start eventDev!");
	}

	/* Unlink queues to allow the ports to be linked when EM queues are created */
	for (qid = 0; qid < config.nb_event_queues; ++qid) {
		if (qid >= start_dir_q)
			port_id = start_dir_port + (qid - start_dir_q);
		else
			port_id = qid / 8;
		rte_ret = rte_event_port_unlink(eventdev_id, port_id, &qid, 1);
		if (rte_ret < 1) {
			APPL_EXIT_FAILURE("Failed to unlink queues to port!");
		}
	}
}


int
eth_init(void)
{
	int nb_ports = -1;	/* return value */
	int num_ports = 1;
	for (int port = 0; port < num_ports; ++port) {
		if (rte_eth_dev_get_port_by_name(net_dev[port], 0) < 0) {
			uint16_t port_id;
			char dev_args[BUF_LEN];
			int rxq = 20;
			int txq = 20;
			sprintf(dev_args, "rxq=%d,txq=%d", rxq, txq);
			if (rte_vdev_init(net_dev[port], dev_args) < 0) {
				APPL_EXIT_FAILURE("Error creating ethdev\n");
			}
			if (rte_eth_dev_get_port_by_name(net_dev[port], &port_id) <
				0) {
				APPL_EXIT_FAILURE("Error finding ethdev\n");
			}
			printf("Initilaized Eth Dev: %s port %d\n", net_dev[port], port_id);
		}
	}

	/*
	 * Configure the Eth ports & Rx+Tx queues for each port
	 */
	nb_ports = rte_eth_dev_count_avail();

	if (unlikely(nb_ports <= 0)) {
		APPL_EXIT_FAILURE("Inv eth-dev cnt:%i", nb_ports);
		return -3;
	}

	printf("\n%s(): Eth ports:%i\n", __func__, nb_ports);

	pdk_pool_create();
	base_dev_id.u16 = my_device_id;
	/*
	 * Initialise each Eth port and its rx&tx queues
	 */
	for (int portid = 0; portid < nb_ports; portid++)
		eth_port_init(portid);


	return nb_ports;
}

/**
 * Create the memory pool used by pkt-io
 */
static int
pdk_pool_create(void)
{
	int pool = 0;
	char pool_name[RTE_MEMPOOL_NAMESIZE] = { 0 };
	snprintf(pool_name, sizeof(pool_name), "em_pktio-pool%d", pool++);
	pool_name[RTE_MEMPOOL_NAMESIZE - 1] = '\0';

	printf("Eth Rx Mempool:%s size(%d) num(%d)\n", pool_name,
		   RTE_MBUF_DEFAULT_DATAROOM, NB_MBUF);

#ifdef LOCK_FREE_MBUF_POOL
	const char *ops_name = "lf_stack";
	pdk_mempool =
		rte_pktmbuf_pool_create_by_ops(pool_name, NB_MBUF,
									   MBUF_CACHE_SIZE, 512,
									   (uint16_t) (RTE_PKTMBUF_HEADROOM + RTE_MBUF_DEFAULT_DATAROOM),
									   (int)rte_socket_id(), ops_name);
#else
	pdk_mempool =
		rte_pktmbuf_pool_create(pool_name, NB_MBUF, MBUF_CACHE_SIZE,
								512, (uint16_t) (RTE_PKTMBUF_HEADROOM + RTE_MBUF_DEFAULT_DATAROOM),
								(int)rte_socket_id());
#endif

	if (pdk_mempool == NULL)
		rte_panic("Cannot init mbuf pool, socket0! rte_errno:%i(%s)\n", rte_errno,
				  rte_strerror(rte_errno));

	return pool;
}

static void
eth_port_init(uint16_t portid)
{
	struct rte_eth_dev_info dev_info;
	int ret;
	uint16_t queueid;

	assert(portid <= UINT8_MAX);
	assert(portid < MAX_ETH_PORTS);

	/* Get Eth port info */
	rte_eth_dev_info_get(portid, &dev_info);

	/* Print Eth port info */
	printf("%s(port:%i):\n" "Eth dev info - port:%i\n" 
					"  Eth driver_name    = %s\n"
					"  Eth min_rx_bufsize = %u\n" 
					"  Eth max_rx_pktlen  = %u\n" 
					"  Eth max_rx_queues  = %u\n"
					"  Eth max_tx_queues  = %u\n", 
					__func__, portid, portid, dev_info.driver_name,
	       dev_info.min_rx_bufsize,	
	       dev_info.max_rx_pktlen,
	       dev_info.max_rx_queues,
	       dev_info.max_tx_queues);

	n_rx_queue = dev_info.max_rx_queues;
	n_tx_queue = dev_info.max_tx_queues;
	/* Initialize Eth port */
	printf("Initializing Eth port %u: RxQs:%u TxQs:%u \n", portid, n_rx_queue,
	       n_tx_queue);

	assert(n_rx_queue <= MAX_ETH_QUEUES);
	assert(n_tx_queue <= MAX_ETH_QUEUES);

	ret = rte_eth_dev_configure(portid, n_rx_queue, n_tx_queue, &eth_port_conf);
	if (unlikely(ret < 0)) {
		APPL_EXIT_FAILURE("rte_eth_dev_configure(): err=%d,port=%u", ret, portid);
	}
	/* Init several RX queues per port */
	for (queueid = 0; queueid < n_rx_queue; queueid++) {
		if (pdk_mempool) {
			ret = rte_eth_rx_queue_setup(portid, queueid, ETH_RX_DESC_DEFAULT,
						     rte_socket_id(),
						     &dev_info.default_rxconf, pdk_mempool);
		}
		if (unlikely(ret < 0 || pdk_mempool == NULL)) {
			APPL_EXIT_FAILURE("rte_eth_rx_queue_setup:%d, port:%u", ret, portid);
		}
	}

	enum rte_tm_cman_mode cman = RTE_TM_CMAN_TAIL_DROP;

	uint16_t cpk_port = portid;
	uint32_t shared_node_id = (uint32_t) (0xaaaa + base_dev_id.u16 + portid);
	uint32_t root_node_id = (uint32_t) (0x1111 + base_dev_id.u16 + portid);
	uint32_t int_node_id = (uint32_t) (0x2222 + base_dev_id.u16 + portid);
	ae_handle hdl;

	ae_client_open(0, 0, 0, &hdl);

#define DEFAULT_L1_NODE_PRIORITY 0
#define DEFAULT_PRIORITY 5
#define DEFAULT_WEIGHT 5
#define	NUM_CHILD_PER_NODE 7

	char vport_dev_name[36];
	char vport_dev_args[128];
	uint16_t vport_dev_port_id;
	struct rte_tm_node_params node_params = { 0 };
	struct rte_tm_error tm_error = { 0 };
	/* set up some valid node parameters; everything else is zeroed out */
	node_params.nonleaf.n_sp_priorities = 8;
	node_params.nonleaf.wfq_weight_mode = NULL;
	node_params.shaper_profile_id = RTE_TM_SHAPER_PROFILE_ID_NONE;

	ae_client_shared_hier_node_add(hdl, cpk_port, shared_node_id, cpk_port,
				       AE_SHAPER_PROFILE_ID_NONE, AE_SHARED_SHAPER_ID_NONE, 1, 0, 0,
				       0);

	snprintf(vport_dev_name, 36, "net_ice_tm%d%d", portid, cpk_port);
	snprintf(vport_dev_args, 128, "parent-port-id=%d,cpk-port=%d,root-shared-node=%d", portid,
		 cpk_port, shared_node_id);

#ifdef DEBUG
	printf("Creating vport_dev_name %s %s\n", vport_dev_name, vport_dev_args);
#endif
	/* Create a vport for the CPK port */
	ret = rte_vdev_init(vport_dev_name, vport_dev_args);
	if (unlikely(ret != 0)) {
		APPL_EXIT_FAILURE("rte_vdev_init(): err=%d,port=%u", ret, portid);
	}
	ret = rte_eth_dev_get_port_by_name(vport_dev_name, &vport_dev_port_id);
	if (unlikely(ret != 0)) {
		APPL_EXIT_FAILURE("rte_eth_dev_get_port_by_name(): err=%d,port=%u", ret, portid);
	}
#ifdef DEBUG
	printf("Adding TM node: node_id %d \n", root_node_id);
#endif
	/* add vport root */
	ret = rte_tm_node_add(vport_dev_port_id, root_node_id, RTE_TM_NODE_ID_NULL,
			      DEFAULT_L1_NODE_PRIORITY, DEFAULT_WEIGHT, RTE_TM_NODE_LEVEL_ID_ANY,
			      &node_params, &tm_error);
	if (unlikely(ret != 0)) {
		APPL_EXIT_FAILURE("rte_tm_node_add(): err=%d,port=%u", ret, portid);
	}
	for (queueid = 0; queueid < n_tx_queue; queueid++) {
		if (queueid % NUM_CHILD_PER_NODE == 0) {
			int_node_id += queueid / NUM_CHILD_PER_NODE;
#ifdef DEBUG
			printf("Adding TM node: parent %d node_id %d \n", root_node_id,
			       int_node_id);
#endif
			ret = rte_tm_node_add(vport_dev_port_id, int_node_id, root_node_id,
					      DEFAULT_L1_NODE_PRIORITY, DEFAULT_WEIGHT,
					      RTE_TM_NODE_LEVEL_ID_ANY, &node_params, &tm_error);
			if (unlikely(ret != 0)) {
				APPL_EXIT_FAILURE("rte_tm_node_add(): err=%d,port=%u", ret, portid);
			}
		}
		/* With RDK5, call to rte_eth_tx_queue_setup() should be before
		   rte_tm_node_add() */
		struct rte_eth_txconf txconf = { 0 };
		txconf.extended_conf = &ice_dsi_txconf;
		txconf.tx_rs_thresh = 0;
		txconf.tx_free_thresh = 0;
		txconf.tx_deferred_start = 0;
		txconf.qlen_mode = RTE_ETH_TXQ_LEN_PACKETS;
		txconf.db_mode = RTE_ETH_TX_DB_DIRECT;
		txconf.max_queue_len = 512;

		txconf.cp_mode = RTE_ETH_TX_CP_TX_DESC;
		txconf.offloads = 0;

		ret = rte_eth_tx_queue_setup((uint8_t) portid, (uint16_t) queueid,
					     ETH_TX_DESC_DEFAULT,
					     (unsigned int)rte_eth_dev_socket_id(portid), &txconf);

		if (unlikely(ret < 0))
			APPL_EXIT_FAILURE("rte_eth_tx_queue_setup:%d,prt:%u,q:%u", ret, portid,
							  queueid);

		node_params.leaf.cman = cman;

#ifdef DEBUG
		printf("Adding TM node: parent %d node_id %d \n", int_node_id,
		       queueid);
#endif
		/* configure leaf-level node */
		ret = rte_tm_node_add(vport_dev_port_id, queueid, int_node_id, DEFAULT_PRIORITY,
				      DEFAULT_WEIGHT, RTE_TM_NODE_LEVEL_ID_ANY, &node_params,
				      &tm_error);
		if (unlikely(ret != 0)) {
			APPL_EXIT_FAILURE("rte_tm_node_add(): err=%d,port=%u", ret, portid);
		}

		node_params.leaf.cman = 0;
	}

	/* Start the device */
	ret = rte_eth_dev_start(portid);
	if (unlikely(ret < 0))
		APPL_EXIT_FAILURE("rte_pmd_port_start:%d, port:%u", ret, portid);
}


static uint8_t
add_rules(int sw, int *logical_port, int acl_tbl_index)
{
	const uint64_t mac_mask = 0xffffffffffff;
	struct ies_acl_rule_info acl_rule = { 0 };
	uint8_t ret;
	int num_rule = em_core_count;

	/*Create rules for ipv4 traffic */
	acl_rule.cond.val.etype = ETH_TYPE_PDK;
	acl_rule.cond.mask.etype = 0xffff;

	acl_rule.cond.mask.dmac = mac_mask;

	acl_rule.action =
		/*IES_ACL_ACTION_COUNT | */ IES_ACL_ACTION_REDIRECT | IES_ACL_ACTION_SET_PKT_TYPE |
		IES_ACL_ACTION_SET_HASH_NEXTHOP;

	acl_rule.state = IES_ACL_RULE_ENTRY_STATE_VALID;

	/*
	 * Create rules. Each rule has specific condition values.
	 * Set action to redirect to different cpk queues
	 */
	int rule_index;
	for (int i = 0; i < num_rule; i++) {
		acl_rule.cond.val.dmac = 0x24043800032 + i;//mac_port[i].mac_addr;
		acl_rule.act_val.logical_port = logical_port[i];//mac_port[i].lport;
		acl_rule.condition = IES_ACL_MATCH_DMAC | IES_ACL_MATCH_ETYPE;
		acl_rule.cond.mask.smac = 0;
		acl_rule.cond.val.smac = 0;
		printf("Ingress Rule id = %d\tlogical port = %d", i, acl_rule.act_val.logical_port);
		if (acl_rule.condition & IES_ACL_MATCH_DMAC)
			printf("\tDMAC = 0x%lx ", acl_rule.cond.val.dmac);
		if (acl_rule.condition & IES_ACL_MATCH_SMAC)
			printf("\tSMAC = 0x%lx ", acl_rule.cond.val.smac);
		if (acl_rule.condition & IES_ACL_MATCH_ETYPE)
			printf("\tEtype = %x\n", acl_rule.cond.val.etype);

		ret = ies_acl_add_rule(sw, acl_tbl_index, &acl_rule, &rule_index);
		if (unlikely(ret != 0)) {
			printf("Ingress ies_acl_add_rule(): err=%s\n", EM_IES_ERROR_NAME(ret));
			return ret;
		}
	}

	return 0;
}

static uint8_t
create_acl(int sw, uint32_t acl_tbl_index)
{
	struct ies_acl_table_info acl_tbl;
	uint8_t ret;
	memset(&acl_tbl, 0, sizeof(acl_tbl));

	acl_tbl.location = IES_ACL_FFU_GROUP_2;
	acl_tbl.acl_instance = IES_ACL_NO_INSTANCE;
	acl_tbl.scenario = IES_ACL_SCENARIO_INGRS_NON_IP;
	acl_tbl.condition = IES_ACL_MATCH_DMAC | IES_ACL_MATCH_ETYPE | IES_ACL_MATCH_SMAC;
	acl_tbl.mask.dmac = 0xffffffffffff;
	acl_tbl.mask.smac = 0xffffffffffff;
	acl_tbl.mask.etype = 0xffff;
	acl_tbl.min_entries = 1024;
	acl_tbl.max_entries = 1024;

	ret = ies_acl_create_table(sw, acl_tbl_index, &acl_tbl);
	if (unlikely(ret != 0)) {
		printf("ies_acl_create_table: %s\n", EM_IES_ERROR_NAME(ret));
		return ret;
	}
	printf("Created ACL table. Index = %d\n", acl_tbl_index);
	return 0;
}

static uint8_t
add_domain_entries(int sw)
{
	struct ies_domain_cond domain_cond = { 0 };
	struct ies_domain_param domain_param = { 0 };
	uint8_t ret;

	/*Set domain params */
	domain_param.enable = true;
	domain_param.learning = true;
	domain_param.independent_learning = true;
	domain_param.routable = false;
	domain_param.update_domain = true;
	domain_param.l2d = 50;
	domain_param.l3d = 10;
	domain_param.nad = 0;
	domain_param.l2_pol_id = 0;
	domain_param.l3_pol_id = 0;
	domain_param.priority_profile = 0;
	domain_param.default_tc = 0;
	domain_param.tc_sel[0] = IES_DOMAIN_TC_SRC_VPRI;
	domain_param.tc_sel[1] = IES_DOMAIN_TC_SRC_DSCP;
	domain_param.tc_sel[2] = IES_DOMAIN_TC_SRC_METADATA;
	domain_param.tc_sel[3] = IES_DOMAIN_TC_SRC_MPLS_EXP;

	/*Set domain conditions */
	domain_cond.tag_type = IES_DOMAIN_VLAN_TAG_TYPE_ONLY_VLAN1;
	domain_cond.port_value = 0;
	domain_cond.port_mask = 0xfc;
	domain_cond.vlan1_value = 1000;
	domain_cond.vlan1_mask = 0xfff;
	domain_cond.vlan2_value = 0;
	domain_cond.vlan2_mask = 0;

	int domain_entry_id[NUM_DOMAIN_ENTRIES];
	int domain_prec = 1;
	for (int i = 0; i < NUM_DOMAIN_ENTRIES; i++) {
		domain_param.l3d += 1;
		domain_param.l2d += 1;
		domain_cond.vlan1_value += 1;

		ret = ies_domain_add_entry(sw, &domain_cond, &domain_param, domain_prec,
					   &domain_entry_id[i]);
		if (unlikely(ret != 0)) {
			printf("ies_domain_add_entry(): err=%d", ret);
			return ret;
		}
		domain_cond.port_value += NUM_HLP_PORTS / NUM_DOMAIN_ENTRIES;
	}

	return 0;
}


int
setup_port_to_queue_mapping(void)
{
	const int sw = 0;
	int logical_port[MAX_ETH_QUEUES] = { 0 };
	int ret, port = 0, qid = 0;
	int vsi_number;
	struct rte_pmd_ice_dsi_dev_info dev_info;
	struct rte_eth_dev_info eth_dev_info;

	port = 0;
	ret = rte_pmd_ice_dsi_get_dev_info(port, &dev_info);
	if (unlikely(ret != 0)) {
		printf("rte_pmd_ice_dsi_get_dev_info(): err=%d", ret);
		return ret;
	}

	vsi_number = dev_info.vsi_number;
	rte_eth_dev_info_get(port, &eth_dev_info);

	for (qid = 0; qid < eth_dev_info.max_rx_queues; qid++) {
		/*Get logical port number of vsi-qid tuple */
		ret = ies_vsi_map_vsi_qid_to_port(sw, vsi_number, qid,
										  &logical_port[qid]);
		if (unlikely(ret != 0)) {
			printf("ies_vsi_map_vsi_qid_to_port(): err=%d", ret);
			return ret;
		}
	}
	printf("Eth port=%d vsi=%d Logical ports %d -- %d\n", port, vsi_number,
		   logical_port[0], logical_port[qid - 1]);

	/*Add domain entries for port and vlan tuple. Domain entries are later used in
	 * acl rules*/
	ret = add_domain_entries(sw);
	if (ret != 0) {
		printf("Failed to add domain entries\n");
		return ret;
	}

	acl_tbl_idx = my_device_id;

	/*Setup acl tables, rules. Redirect packets to logical ports by matching rules */
	ret = create_acl(sw, acl_tbl_idx);
	if (ret != 0) {
		printf("Failed to create acl table\n");
		return ret;
	}

	/*Add rules to ACL table */
	ret = add_rules(sw, logical_port, acl_tbl_idx);
	if (ret != 0) {
		printf("Failed to add acl rules\n");
		return ret;
	}

	return 0;
}

void
ies_event_handler_func(const struct ies_event *ies_event)
{
	if (ies_event != NULL) {
		/**
		 * Now handle the event itself...
		 * Place holder for now with just print statements
		 */
		switch (ies_event->type) {
		case IES_EVENT_ADDR_UPDATE:
			printf("IES_EVENT : MAC Address Update\n");
			break;
		case IES_EVENT_ERROR:
			printf("IES_EVENT : Error: %d %d\n", ies_event->info.error.type,
			       ies_event->info.error.info.mod.rc);
			break;
		case IES_EVENT_PORT_STATE:
			printf("IES_EVENT : Port %d State: %s\n", ies_event->info.port_state.port,
			       (ies_event->info.port_state.status ==
				IES_PORT_STATUS_LINK_UP) ? "Up" : "Down");
			if ((ies_event->info.port_state.status == IES_PORT_STATUS_LINK_UP)
			    && (ies_event->info.port_state.port < NUM_HLP_PORTS)) {
				port_up_event_count += 1;
				if (pdk_bitmask_isset
				    (ies_event->info.port_state.port, &port_mask_25g))
					pdk_bitmask_clr(ies_event->info.port_state.port,
							&port_mask_25g);
				else if (pdk_bitmask_isset
					 (ies_event->info.port_state.port, &port_mask_10g))
					pdk_bitmask_clr(ies_event->info.port_state.port,
							&port_mask_10g);
			}
			break;
		case IES_EVENT_SWITCH_STATE:
			printf("IES_EVENT : Switch State\n");
			break;
		case IES_EVENT_TIMEOUT:
			printf("IES_EVENT : Timeout\n");
			break;
		default:
			break;
		}
	}
}

static void
print_switch_state(enum ies_switch_state ies_sw_state)
{
	switch (ies_sw_state) {
	case IES_SWITCH_STATE_DOWN:
		printf("Switch state is IES_SWITCH_STATE_DOWN\n");
		break;
	case IES_SWITCH_STATE_UP:
		printf("Switch state is IES_SWITCH_STATE_UP\n");
		break;
	case IES_SWITCH_STATE_HOLD:
		printf("Switch state is IES_SWITCH_STATE_HOLD\n");
		break;
	default:
		printf("Invalid Switch state\n");
		break;
	}
}

static void
port_config(int port, enum ies_port_phy_type phy_type)
{
	int sw = 0;
	enum ies_port_mode port_mode;
	int ret = ies_port_get_mode(sw, port, &port_mode);
	if (ret != IES_OK)
		printf(" Not able to get mode\n");
	if (port_mode == IES_PORT_MODE_ADMIN_UP) {
		port_mode = IES_PORT_MODE_ADMIN_PWRDOWN;
		ret = ies_port_set_mode(sw, port, port_mode);
		if (ret != IES_OK)
			printf(" Not able to set mode\n");
	}

	ret = ies_port_set_attr(sw, port, IES_PA_PHY_TYPE, &phy_type);

	ret = ies_port_set_mode(sw, port, IES_PORT_MODE_ADMIN_UP);
	usleep(100);
}

/*
 * Function to initialize IES device in master/slave mode
 */
void
ies_configure(int master)
{
	int sw = 0;
	int port, ret;
	enum ies_switch_state ies_sw_state;
	uint32_t event_mask;
	bool val = 1;
	int key = 1234;
	int cnt = 0;

	event_mask =
		IES_EVENT_SWITCH_STATE | IES_EVENT_PORT_STATE | IES_EVENT_ADDR_UPDATE |
		IES_EVENT_ERROR | IES_EVENT_TIMEOUT;

	ies_init_set_attr(IES_INIT_ATTR_SHARED_MEMORY, &val);
	ies_init_set_attr(IES_INIT_ATTR_SHM_KEY, &key);
	/* Initialize IES */
	do {
		ies_init_set_attr(IES_INIT_ATTR_IS_MASTER, &master);
		ies_property_setv2(IES_PROP_SWITCH_ALLOW_RESET, &master);
		if ((ret = ies_initialize(ies_event_handler_func, event_mask)) == IES_OK)
			break;
		if (ret == IES_ERR_NO_RESOURCE && cnt++ < 10) {
			printf("ies_initialize Retry...\n");
			sleep(1);
		} else {
			if (master) {
				master = 0;
			} else {
				printf("Error in ies_initialize...errno = %d\n", ret);
				exit(0);
			}
		}
	} while (ret != IES_OK);

	/* Get switch state */
	do {
		usleep(1);
		if (ies_switch_get_state(sw, &ies_sw_state) != IES_OK) {
			printf("Error in ies_initialize\n");
			return;
		}
	} while (ies_sw_state != IES_SWITCH_STATE_UP);

	if (!master) {
		return;
	}
	print_switch_state(ies_sw_state);

	enum ies_port_phy_type phy_type;
	uint64_t attr_val = 0;
	enum ies_port_attr get_attr = IES_PA_PHY_TYPE_ABILITY;
	/* Get port information */
	for (port = 0; port < NUM_HLP_PORTS; port++) {
		do {
			ret = ies_port_get_attr(sw, port, get_attr, &attr_val);
		} while (ret != IES_OK);

		/*Set the phy type for each port */
		if (attr_val & IES_PORT_PHY_TYPE_25GBASE_CR) {
			pdk_bitmask_set(port, &port_mask_25g);
		} else if (attr_val & IES_PORT_PHY_TYPE_10G_SFI) {
			pdk_bitmask_set(port, &port_mask_10g);
		} else {
			printf("Warning: Port %d does not support 25/10G. skipping!\n", port);
		}
	}

	for (port = 0; port < NUM_HLP_PORTS; port++) {
		if (pdk_bitmask_isset(port, &port_mask_25g))
			phy_type = IES_PORT_PHY_TYPE_25GBASE_CR;
		else if (pdk_bitmask_isset(port, &port_mask_10g))
			phy_type = IES_PORT_PHY_TYPE_10G_SFI;
		else
			continue;
		port_config(port, phy_type);
	}
	printf("Waiting for port UP events\n");
	sleep(PORT_UP_TIMEOUT);

	if (port_up_event_count != NUM_HLP_PORTS) {
		pdk_bitmask_t mask_copy = port_mask_25g;
		int num25g_ports = pdk_bitmask_count(&port_mask_25g);
		for (int k = 0; k < num25g_ports; ++k) {
			port = pdk_bitmask_idx(k + 1, &mask_copy);
			port_config(port, IES_PORT_PHY_TYPE_10G_SFI);
		}
	}
	printf("Received %d port UP events\n", port_up_event_count);
}

static adk_app_msg_t ipPacket_flow[] = {{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x09, 0x01, 0x01, 0x01, 0x31, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0A, 0x01, 0x01, 0x01, 0x32, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0B, 0x01, 0x01, 0x01, 0x33, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0C, 0x01, 0x01, 0x01, 0x34, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0D, 0x01, 0x01, 0x01, 0x35, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0E, 0x01, 0x01, 0x01, 0x36, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0F, 0x01, 0x01, 0x01, 0x37, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x10, 0x01, 0x01, 0x01, 0x38, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x11, 0x01, 0x01, 0x01, 0x39, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x12, 0x01, 0x01, 0x01, 0x3A, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x13, 0x01, 0x01, 0x01, 0x3B, 0x01, 0x01, 0x64},
    .udpHdr = {0x00, 0x64, 0x08, 0x68, 0x01, 0xD6, 0xC1, 0x35}},
	/*Plain text*/{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0A, 0x01, 0x01, 0x01, 0x32, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0B, 0x01, 0x01, 0x01, 0x33, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0C, 0x01, 0x01, 0x01, 0x34, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0D, 0x01, 0x01, 0x01, 0x35, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0E, 0x01, 0x01, 0x01, 0x36, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x0F, 0x01, 0x01, 0x01, 0x37, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x10, 0x01, 0x01, 0x01, 0x38, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x11, 0x01, 0x01, 0x01, 0x39, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x12, 0x01, 0x01, 0x01, 0x3A, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}},{
    .ipHdr = {0x45, 0x00, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11,
              0x66, 0xBD, 0x13, 0x01, 0x01, 0x01, 0x3B, 0x01, 0x02, 0x64},
    .udpHdr = {0x00, 0x16, 0x00, 0x16, 0x01, 0xD6, 0xC1, 0x35}}
};


void
appl_receive(em_event_t event, em_queue_t queue, em_queue_t next)
{
	int16_t size = 0, udplen=0;
	adk_app_msg_t *msg = NULL;
	int pktLen = 0;
	struct rte_mbuf *mbuf;
	em_queue_t dest_queue=QUEUE_UNDEF;

	batch_buf[batch_count++] = event;
	//printf("Rx: lcore %d event %p on queue %lx\n", rte_lcore_id(), event, queue);
#if 0
		uint8_t *c = (uint8_t*)msg;
		for (int i=0; i<20;++i)
			printf("%x ", *c++);
		printf("\n");
#endif
	if (queue <= 0xFF) {
		uint8_t flowId = queue;
		struct adk_sys_event_metadata *ev = (struct adk_sys_event_metadata *)em_event_to_nonem_param (event);

		mbuf = event_hdr_to_mbuf((event_hdr_t *)event);
		pktLen = mbuf->pkt_len;
		size = htons(pktLen);
		/*UDP length*/
		udplen= htons(pktLen - 20);
		msg = rte_pktmbuf_mtod(mbuf, adk_app_msg_t *);
		ev->udp_cksum_gen = 1;

		dest_queue = g_nonem_queue_ip[flowId];
		memcpy(&ipPacket_flow[flowId].ipHdr[2], &size, 2);
		memcpy(&ipPacket_flow[flowId].udpHdr[4], &udplen, 2);
		memcpy((void *)msg, (void *)&ipPacket_flow[flowId], sizeof(ipPacket_flow[flowId]));
	}
	else {
		dest_queue = ( (REM_DEV_ID << 16) | (0xFFFF & queue));
	}
	if(batch_count < BATCH_SIZE && queue == next)
		return;

	//printf("Sending %d events to Q: %lx\n", batch_count, dest_queue);
	uint32_t ret = em_send_multi(batch_buf, batch_count, dest_queue);
	if (ret != batch_count) {
		for(uint32_t j = ret; j < batch_count; j ++) {
			mbuf = event_hdr_to_mbuf((event_hdr_t *)batch_buf[j]);
			rte_pktmbuf_free(mbuf);
		}
	}
	batch_count = 0;
}

static void
eth_poll(void)
{
	int nb_rx;
	uint8_t portid = 0;
	uint8_t rxq_id = rte_lcore_id();
	int i;
	struct rte_mbuf *mbufs[MAX_RX_PKT_BURST];

	nb_rx = rte_eth_rx_burst(portid, rxq_id, mbufs, MAX_RX_PKT_BURST);
	if (likely(nb_rx > 0)) {
		rte_pktmbuf_adj(mbufs[0], PDK_DATA_OFFSET);
		adk_app_msg_t *msg = rte_pktmbuf_mtod(mbufs[0], adk_app_msg_t *);
		for (i = 0; ; ++i) {
			em_event_t event = (em_event_t)mbuf_to_event_hdr(mbufs[i]);
			if (i == nb_rx - 1) {
				appl_receive(event, (msg->ipHdr[0] & 0xff), QUEUE_UNDEF);
				break;
			}
			rte_pktmbuf_adj(mbufs[i + 1], PDK_DATA_OFFSET);
			adk_app_msg_t *nxt = rte_pktmbuf_mtod(mbufs[i + 1], adk_app_msg_t *);
			appl_receive(event, (msg->ipHdr[0] & 0xff), (nxt->ipHdr[0] & 0xff));
			msg = nxt;
		}
	}
}

static void
eventdev_poll(void)
{
	struct rte_event rcv_events[MAX_EVENTDEV_DEQUEUE_DEPTH];
	uint16_t num_dequeued;
	uint8_t event_port_id, eventdev_id = EM_EVENTDEV_ID;
	uint64_t timeout_ticks;

	event_port_id = eventdev_port_id[EM_EVENTDEV_ID][rte_lcore_id()]; /*Need local lcore ID???*/

	timeout_ticks = 0;

	num_dequeued =
		rte_event_dequeue_burst(eventdev_id, event_port_id, rcv_events, RTE_DIM(rcv_events),
					timeout_ticks);

	if (likely(num_dequeued > 0)) {
		struct rte_mbuf *mbuf = rcv_events[0].mbuf;
		event_hdr_t *ev_hdr = mbuf_to_event_hdr(mbuf);
		for (int i = 0; ; i++) {
			ev_hdr->op = RTE_EVENT_OP_FORWARD;
			if (i == num_dequeued - 1) {
				appl_receive( (em_event_t)ev_hdr, ev_hdr->queue, QUEUE_UNDEF);
				break;
			}
			mbuf = rcv_events[i + 1].mbuf;
			event_hdr_t * next_ev_hdr = mbuf_to_event_hdr(mbuf);
			appl_receive( (em_event_t)ev_hdr, ev_hdr->queue, next_ev_hdr->queue);
			ev_hdr = next_ev_hdr;
		}
	}
}

void
pdk_poll(void)
{
	while (1) {
		/*
		 * Poll for Eth frames and enqueue.
		 */
		eth_poll();

		/*
		 * Poll Eventdev ports
		 */
		eventdev_poll();
	}
}

void
appl_init(void)
{
	adk_sys_handle_t sys_handle;
	int argc = 1;
	char *argv [20];
	/* Setting the Unified DP setup mode*
	 * EAL and HLP initialization wont be happening in DP setup under this mode*
	 */
	is_unified_em_adk_set = 1;
	/* Number of EM cores used, will be passed to ADK *
	 * And ADK will create eventDev port for EM cores *
	 */
	number_em_cores = em_core_count;

	char str[512] = ADK_CMD_ARGS;

	char *p2 = strtok(str, " ");
	while (p2)
	{
		argv[argc++] = p2;
		p2 = strtok(0, " ");
	}
	argv[argc] = 0;
        argv[0] = NULL;

	int arg_c = argc;

	printf("Following are the arguments passed for ADK-DP setup :\n");
	while(arg_c ){
		printf("argv[%d] %s ", argc-arg_c, argv[argc-arg_c]);
		arg_c--;
	}

	if (adk_dp_setup(argc, argv) != 0) {
		printf("adk_dp_setup() failed");
		return ;
	}

	sys_handle = init_adk_modules();
	if(!sys_handle){
		printf("Failed in System init \n");
		return ;
	}

	if (adk_dp_init(sys_handle)){
		printf("Failed to init DP \n");
		return ;
	}

	em_init();
	setup_port_to_queue_mapping();
}
