#include "pdk_include.h"
#include <rte_eventdev.h>
#include <arpa/inet.h>

/* Define local device mask. T be used to determine if the device ID belongs to local SoC */
#define LOCAL_DEVICE_MASK  (0xFFF0)

/* HQM service priority */
#define EM_SERV_PRIO(priority) ((priority >> 8) & 0xff)	


#define MAX_EVENTDEV_ENQUEUE_DEPTH  128 
#define MAX_EVENTDEV_DEQUEUE_DEPTH  128
#define EM_ENQUEUE_RETRIES  (10000)

/* Queue priority */
typedef enum em_queue_prio_e {
  EM_QUEUE_PRIO_HIGHEST = 0,     /**< Highest */
  EM_QUEUE_PRIO_HIGH    = 1,    /**< High */
  EM_QUEUE_PRIO_NORMAL  = 3,    /**< Normal */
  EM_QUEUE_PRIO_LOW     = 6,    /**< Low */
  EM_QUEUE_PRIO_LOWEST  = 7,    /**< Lowest */
  EM_QUEUE_PRIO_UNDEF   = 0xFF /**< Undefined */
} em_queue_prio_e;

char ies_error_name[IES_ERROR_MAX_NAME_SIZE];
#define EM_IES_ERROR_NAME(ies_ret) \
    ((ies_error_get_name(ies_ret, ies_error_name, \
                         IES_ERROR_MAX_NAME_SIZE) == IES_OK) ? \
     ies_error_name : "EM_UNKNOWN")


typedef struct
{
  uint8_t did;
	uint8_t eventdev_id;
	uint8_t priority;
	uint8_t sched_type;
	uint8_t flow_id;
}did_decode_t;


/* Will be populated in em_init based on the queue configration */
did_decode_t  did_decode_tbl[256];

/*Port assignment for EM and non EM eventdev */
uint8_t eventdev_port_id[2][24] = {
{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23},
{10, 11, 12, 13, 0xff, 0xff, 0xff, 0xff, 10,11,0xff, 0xff, 0xff, 0xff,0xff, 0xff, 0xff, 0xff,0xff, 0xff, 0xff, 0xff,0xff, 0xff}
};

typedef struct
{
  uint32_t queue;
	uint8_t edev_q;
	uint8_t eventdev_id;
	uint16_t priority;
	uint32_t mask;
}emq_to_edevq_map_t;

/* EM queue to eventdev mapping */
emq_to_edevq_map_t queue_map[] =
{{0x320001, 5, EM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0xc},
{0x320101, 5, EM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0xc},
{0x322a01, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a02, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a03, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a04, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a05, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a06, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a07, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a08, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a09, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0a, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0b, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0c, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0d, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0e, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a0f, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a10, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a11, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a12, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a13, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322a14, 6, EM_EVENTDEV_ID,EM_QUEUE_PRIO_HIGHEST, 0xf},
{0x322b01, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b02, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b03, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b04, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b05, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b06, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b07, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b08, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b09, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0a, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0b, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0c, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0d, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0e, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b0f, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b10, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b11, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b12, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b13, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000},
{0x322b14, 1, NONEM_EVENTDEV_ID,EM_QUEUE_PRIO_LOW, 0x2000000}};

uint16_t my_device_id = PDK_DEVICE_ID;
uint32_t meta_tx_offset, meta_rx_offset;
em_queue_t g_nonem_queue_ip[20];

int send_remote(em_event_t * event, int num, em_queue_t queue);

static inline int
queue_islocal(em_queue_t queue)
{
	return ( (queue >> 16) & LOCAL_DEVICE_MASK) == (PDK_DEVICE_ID & LOCAL_DEVICE_MASK);
}

static inline uint16_t 
send_event(event_hdr_t *const ev_hdrs[], const int num, em_queue_t queue)
{
	struct rte_event events[MAX_EVENTDEV_ENQUEUE_DEPTH];
	uint16_t i;
	did_decode_t did;
	uint8_t event_port_id, eventdev_id;
	uint8_t qid;
	uint32_t lcore = rte_lcore_id();
	uint16_t num_enqueued = 0 ;
	int retry_count = 0;

	/* Get qid and eventdev_id from did table */
	did = did_decode_tbl[(queue>> 8)&0xff];
	qid = did.did;
	eventdev_id = did.eventdev_id;

	for (i = 0; i < num; ++i)
	{
		struct rte_event *ev = &events[i];
		ev_hdrs[i]->queue = queue;

		ev->flow_id = did.flow_id;
		ev->queue_id = qid;
		ev->priority = did.priority;

		ev->mbuf = (struct rte_mbuf*)event_hdr_to_mbuf(ev_hdrs[i]);
		ev->sched_type = did.sched_type;
		ev->event_type = RTE_EVENT_TYPE_CPU;

		ev->op = (uint8_t)(0x3 & ev_hdrs[i]->op);
	}

	event_port_id = eventdev_port_id[eventdev_id][lcore];
	do {
		num_enqueued = (uint16_t) (num_enqueued + rte_event_enqueue_burst(eventdev_id, event_port_id, events + num_enqueued, (uint16_t) (num - num_enqueued)));
	} while (num_enqueued != (uint16_t)num && retry_count++ < EM_ENQUEUE_RETRIES);

	return num_enqueued;
}

em_status_t
em_send(em_event_t event, em_queue_t queue)
{
	event_hdr_t *const ev_hdr = event_to_event_hdr(event);

	if (!queue_islocal(queue)) {
		return send_remote(&event, 1, queue) == 1;
	} 
	else {
		return send_event(&ev_hdr, 1, queue) == 1;
	}

	return EM_OK;
}

em_event_t
em_rte_mbuf_to_event(struct rte_mbuf *mbuf)
{
	return (em_event_t)&mbuf[1];
}

em_status_t
em_init(void)
{
	uint32_t i, num_queue = sizeof(queue_map)/12;
	int nonem_q = 0;

	for (i=0; i<num_queue; ++i)
	{
		uint8_t index;
		index = (queue_map[i].queue >> 8) & 0xff;
		did_decode_tbl[index].did = queue_map[i].edev_q;
		did_decode_tbl[index].priority = queue_map[i].priority;
		did_decode_tbl[index].eventdev_id = queue_map[i].eventdev_id;
		did_decode_tbl[index].sched_type = RTE_SCHED_TYPE_PARALLEL;
		uint32_t mask = 0xffffff & queue_map[i].mask;

		uint8_t service_prio = ((EM_SERV_PRIO(queue_map[i].priority) << 5) & 0xe0);;

		if(mask == 0) {
			g_nonem_queue_ip[nonem_q++] = queue_map[i].queue;
		} else {
			/* Link qid to ports */
			while (mask) {
				uint8_t port_id, lcore;
				int ret_val;
				uint32_t p;

				p = (uint32_t) __builtin_ffsl(mask);
				p--;

				lcore = (uint8_t)p;

				port_id = eventdev_port_id[EM_EVENTDEV_ID][lcore];
				ret_val =
					rte_event_port_link(EM_EVENTDEV_ID, port_id,
										&queue_map[i].edev_q, &service_prio, 1);

				if (ret_val != 1) {
					printf("Failed to map port to qid!");
					return -1;
				}
				//printf("Linked Q:%x port=%d\n", queue_map[i].edev_q, port_id);

				/* Clear mask bit */
				mask &= ~((uint32_t) 1 << p);
			}
		}
	}

	rte_pmd_ice_dsi_get_metadata_offset(0, &meta_rx_offset, &meta_tx_offset);
	return 0;
}


int
send_remote(em_event_t *events, int num, em_queue_t queue)
{
	uint16_t portid = 0, txq_id = rte_lcore_id();
	mac_addr_u64_t mac;
	uint16_t vlan_id = 0;
	struct ice_dsi_tx_metadata *dsi_tx_meta;
	struct rte_mbuf *mbuf[32];
	eth_hdr_vlan_t *hdr;

	for (int i = 0; i < num; ++i) {
		mbuf[i] = (struct rte_mbuf *)event_hdr_to_mbuf((event_hdr_t*)events[i]);
		rte_pktmbuf_prepend(mbuf[i], PDK_DATA_OFFSET);

		hdr = rte_pktmbuf_mtod(mbuf[i], eth_hdr_vlan_t *);

		mac.u64 = PDK_GET_MAC((queue >> 16) & 0xFFFF);
		ethaddr_copy(&hdr->dst, &mac.s.mac_addr);

		mac.u64 = PDK_GET_MAC(my_device_id);
		ethaddr_copy(&hdr->src, &mac.s.mac_addr);

		hdr->vlan = htonl((uint32_t) (ETH_TYPE_VLAN << 16 | vlan_id));
		hdr->type = htons(ETH_TYPE_PDK);

		mbuf[i]->ol_flags |= PKT_TX_METADATA;
		dsi_tx_meta = rte_ice_dsi_mbuf_tx_meta(mbuf[i], meta_tx_offset);
		dsi_tx_meta->dst_port = 20;
		dsi_tx_meta->overwrite = 1;
	}

#if 0
	uint8_t *c = (uint8_t*)hdr;
	for (int  i =0; i < 30; ++i)
		printf("%x ", *c++);
	printf("\n%s: Sending event to port=%d queue=%d\n", __func__, portid, txq_id);
#endif
	return rte_eth_tx_burst(portid, txq_id, mbuf, num);
}

void *
em_event_to_nonem_param(em_event_t event)
{
	return (void *)((uint8_t *) event + sizeof(event_hdr_t));
}

int
em_send_multi(em_event_t * const events, int num, em_queue_t queue)
{
	event_hdr_t *const *ev_hdrs = (event_hdr_t *const *)events;

	if (!queue_islocal(queue)) {
		return send_remote(events, num, queue);
	} 
	else {
		return send_event(ev_hdrs, num, queue);
	}
}
