/* **************************************************************************
**                                                                          *
** INTEL CONFIDENTIAL                                                       *
** Copyright 2011 - 2019 Intel Corporation All Rights Reserved.             *
**                                                                          *
** The source code contained or described herein and all documents related  *
** to the source code ("Material") are owned by Intel Corporation or its    *
** suppliers or licensors. Title to the Material remains with Intel         *
** Corporation or its suppliers and licensors. The Material contains trade  *
** secrets and proprietary and confidential information of Intel or its     *
** suppliers and licensors. The Material is protected by worldwide          *
** copyright and trade secret laws and treaty provisions. No part of the    *
** Material may be used, copied, reproduced, modified, published, uploaded, *
** posted, transmitted, distributed, or disclosed in any way without        *
** Intel's prior express written permission.                                *
**                                                                          *
** No license under any patent, copyright, trade secret or other            *
** intellectual property right is granted to or conferred upon you by       *
** disclosure or delivery of the Materials, either expressly, by            *
** implication, inducement, estoppel or otherwise. Any license under such   *
** intellectual property rights must be express and approved by Intel       *
** in writing.                                                              *
**                                                                          *
** **************************************************************************
**/
/* ADK common headers */
#include <adk_include.h>
#include <adk_rm.h>
#include <adk_hash.h>
#include <rte_security.h>
#include <adk_hos_conf.h>
#include <adk_hos.h>
#include <adk_cp_acl_list.h>
#include "adk_sys.h"
#include "adk_shm.h"

/* IPsec module specific headers */
#include "adk_ipsec.h"
#include "adk_ipsec_common.h"
#include "adk_ipsec_cp.h"

#define POL_PREPEND 0x1
#define POL_APPEND  0x2
#define POL_INSERT  0x3

struct rte_acl_field_def
    adk_ipsec_spd_fields[NUM_FIELDS_IPSEC] =
    {
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint8_t),
            .field_index = VERSION_FIELD_IPSEC,
            .input_index = VERSION_INPUT_IDX_IPSEC,
            .offset = offsetof(adk_ipsec_spd_sel_t, version),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IPV4_IPV6_0,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_0,
            .offset = offsetof(adk_ipsec_spd_sel_t, sip_addr[0]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IPV4_IPV6_1,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_1,
            .offset = offsetof(adk_ipsec_spd_sel_t, sip_addr[1]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IPV4_IPV6_2,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_2,
            .offset = offsetof(adk_ipsec_spd_sel_t, sip_addr[2]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IPV4_IPV6_3,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IPV4_IPV6_3,
            .offset = offsetof(adk_ipsec_spd_sel_t, sip_addr[3]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IPV4_IPV6_0,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_0,
            .offset = offsetof(adk_ipsec_spd_sel_t, dip_addr[0]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IPV4_IPV6_1,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_1,
            .offset = offsetof(adk_ipsec_spd_sel_t, dip_addr[1]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IPV4_IPV6_2,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_2,
            .offset = offsetof(adk_ipsec_spd_sel_t, dip_addr[2]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IPV4_IPV6_3,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IPV4_IPV6_3,
            .offset = offsetof(adk_ipsec_spd_sel_t, dip_addr[3]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = DSCP_FIELD_IPSEC,
            .input_index = DSCP_PROTO_INPUT_IDX_IPSEC,
            .offset = offsetof(adk_ipsec_spd_sel_t, dscpVal),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = PROTO_FIELD_IPSEC,
            .input_index = DSCP_PROTO_INPUT_IDX_IPSEC,
            .offset = offsetof(adk_ipsec_spd_sel_t, protocol),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = SP_FIELD_IPSEC,
            .input_index = SP_DP_INPUT_IDX_IPSEC,
            .offset = offsetof(adk_ipsec_spd_sel_t, sport),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = DP_FIELD_IPSEC,
            .input_index = SP_DP_INPUT_IDX_IPSEC,
            .offset = offsetof(adk_ipsec_spd_sel_t, dport),
        }
    };

struct rte_acl_field_def
    adk_ipsec_spd_in_fields[NUM_FIELDS_IPSEC_IN] =
    {
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint8_t),
            .field_index = VERSION_FIELD_IPSEC_IN,
            .input_index = VERSION_INPUT_IDX_IPSEC_IN,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, version),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_0,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, sip_addr[0]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_1,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, sip_addr[1]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_2,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, sip_addr[2]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3,
            .input_index = SRC_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_3,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, sip_addr[3]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_0,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, dip_addr[0]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_1,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, dip_addr[1]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_2,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, dip_addr[2]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3,
            .input_index = DST_IP_INPUT_IDX_IPSEC_IN_IPV4_IPV6_3,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, dip_addr[3]),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint32_t),
            .field_index = PROTO_FIELD_IPSEC_IN,
            .input_index = PROTO_INPUT_IDX_IPSEC_IN,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, protocol),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = SP_FIELD_IPSEC_IN,
            .input_index = SP_DP_INPUT_IDX_IPSEC_IN,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, sport),
        },
        {
            .type = RTE_ACL_FIELD_TYPE_RANGE,
            .size = sizeof(uint16_t),
            .field_index = DP_FIELD_IPSEC_IN,
            .input_index = SP_DP_INPUT_IDX_IPSEC_IN,
            .offset = offsetof(adk_ipsec_spd_in_sel_t, dport),
        }
    };

typedef enum adk_ipsec_rule_oper_e
{
    ADK_IPSEC_RULE_ADD = 0,
    ADK_IPSEC_RULE_DEL,
}adk_ipsec_rule_oper;

typedef struct adk_ipsec_spd_info_s
{
    uint16_t  ruleLoc;
    union
    {
        struct add_info
        {
            struct adk_ipsec_acl_rule aclRule;
            uint8_t   action;
            uint16_t  polHdlIdx;
        }addInfo;
    }operInfo;
}adk_ipsec_spd_info_t;

typedef struct adk_ipsec_spd_in_info_s
{
    uint16_t  ruleLoc;
    union
    {
        struct spd_in_add_info
        {
            struct adk_ipsec_in_acl_rule aclRule;
            uint8_t   action;
            uint16_t  polHdlIdx;
        }addInfo;
    }operInfo;
}adk_ipsec_spd_in_info_t;

typedef struct adk_ipsec_spd_node_s adk_ipsec_spd_node_t;
struct adk_ipsec_spd_node_s
{
    adk_ipsec_rule_oper   ruleOper;
    adk_ipsec_spd_info_t  ruleInfo;
    STAILQ_ENTRY(adk_ipsec_spd_node_s) entries;
};

typedef struct adk_ipsec_spd_in_node_s adk_ipsec_spd_in_node_t;
struct adk_ipsec_spd_in_node_s
{
    adk_ipsec_rule_oper   ruleOper;
    adk_ipsec_spd_in_info_t  ruleInfo;
    STAILQ_ENTRY(adk_ipsec_spd_in_node_s) entries;
};

static adk_ipsec_return_code_t
ipsec_sec_pol_sel_validate (adk_module_handle_t     ipsecHandle,
                            adk_ipsec_sec_pol_sel_t *secPolSel_p);

static adk_ipsec_return_code_t
ipsec_sec_pol_attr_validate(adk_module_handle_t  ipsecHandle,
                            adk_ipsec_sec_pol_t  *secPol_p);


/**
 *=============================================================
 * @internal
 *
 * This function creates SPD IN/OUT ordered tree
 * This is called at initialization time where validation of params
 * is already done. Not input arg validation, No lock required.
 * =============================================================
 */
adk_status_t adk_ipsec_spd_cp_create (adk_module_handle_t ipsecHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_acl_info_t cpListInfo={0};
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    adk_sys_handle_t sysHandle;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    /* SPD OUT ACL List */
    cpListInfo.user = ADK_ACL_SPD_OUT;
    if(ADK_SUCCESS != (retValue = adk_acl_create(&(ipsecModuleInfo_p->cpSpdAclList),
                                                            &cpListInfo)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Could not allocate CP list for SPD OUT!\n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    /* SPD OUT ACL List */
    cpListInfo.user = ADK_ACL_SPD_IN;
    if(ADK_SUCCESS != (retValue = adk_acl_create(&(ipsecModuleInfo_p->cpSpdInAclList),
                                                            &cpListInfo)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Could not allocate CP list for SPD IN!\n");
        return ADK_E_MEM_UNAVAILABLE;
    }

    return ADK_IPSEC_SUCCESS;
}

/**
 *=============================================================
 * @internal
 *
 * This function deletes SPD IN/OUT ordered tree
 * =============================================================
 */
adk_status_t adk_ipsec_spd_cp_delete (adk_module_handle_t ipsecHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    adk_sys_handle_t sysHandle;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    /* SPD OUT ACL List */
    if(ADK_SUCCESS != (retValue = adk_acl_free(ipsecModuleInfo_p->cpSpdAclList)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Could not free CP list for SPD OUT!\n");
        return ADK_E_INTERNAL_ERROR;
    }

    /* SPD OUT ACL List */
    if(ADK_SUCCESS != (retValue = adk_acl_free(ipsecModuleInfo_p->cpSpdInAclList)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Could not free CP list for SPD IN!\n");
        return ADK_E_INTERNAL_ERROR;
    }

    return retValue;
}

adk_status_t adk_ipsec_dp_table_init (adk_module_handle_t ipsecHandle,
                                      adk_sys_handle_t sysHandle,
                                      ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    uint32_t i=0, j=0;
    adk_sys_config_worker_lcore_info_t  *workerInfo;
    uint8_t   newSpdInTblIdx, oldSpdInTblIdx;
    uint8_t   newSpdTblIdx, oldSpdTblIdx;

    retValue = adk_sys_config_worker_lcore_info_get(&workerInfo);
    if (retValue != ADK_IPSEC_SUCCESS)
    {
        return retValue;
    }

    retValue = adk_shm_create(ADK_IPSEC_SHM_MEMZONE_NAME,
                              sizeof(adk_ipsec_dp_table_info_t),
                              ADK_SHM_FIRST_WORKER_IN_USE_LCORE,
                              (adk_shm_handle_t **)&(ipsecModuleInfo_p->dpTbl));
    if (retValue != ADK_IPSEC_SUCCESS)
    {
        ADK_IPSEC_LOG_FORCE("Failed to create IPsec memzone %s\n",
                          ADK_IPSEC_SHM_MEMZONE_NAME);
        return ADK_E_MEM_UNAVAILABLE;
    }

    /* Initialize data path tables */
    /* TODO once we make all the tables dynamic in nature
     * we can add the allocation scheme here for it, currently
     * all required DS are static
     */

    /* 1. Initialize spi table */

    for(i=0; i<IPSEC_HASH_MAX; i++)
    {
        for(j=0; j<IPSEC_MAX_COLLISION; j++)
        {
            ipsecModuleInfo_p->dpTbl->spiHashTbl[i][j].isValid = FALSE;
        }
    }
    
    for(i=0; i<ADK_IPSEC_HWSA_MAP_HASH_MAX; i++)
    {
        for(j=0; j<ADK_IPSEC_HWSA_MAP_MAX_COLLISION; j++)
        {
            ipsecModuleInfo_p->dpTbl->hwSAIdMap[i][j].isValid = FALSE;
        }
    }

    ipsecModuleInfo_p->dpTbl->cookie = ADK_IPSEC_MODULE_COOKIE;

    /* Initialize the worker list*/
    ipsecModuleInfo_p->dpTbl->numCores = workerInfo->numLcores;
    for (i = 0; i < workerInfo->numLcores; i++)
    {
        ipsecModuleInfo_p->dpTbl->workerCoreList[i] = workerInfo->lcore[i].index;
    }

    /* Ensure that this is set to NULL */
    ipsecModuleInfo_p->secCtxCP = NULL;
    
    /* SPD IN ACL creation for active and backup copies */
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdInTblIdx, newSpdInTblIdx, oldSpdInTblIdx);
    ipsecModuleInfo_p->dpTbl->spdInTbl[newSpdInTblIdx] =
                        adk_ipsec_spd_in_create (sysHandle, ipsecHandle, newSpdInTblIdx);
    if (NULL == ipsecModuleInfo_p->dpTbl->spdInTbl[newSpdInTblIdx])
    {
        ADK_IPSEC_LOG_FORCE("IPSEC: SPD IN ACL CTX creation failed!\n");
        return ADK_E_MEM_UNAVAILABLE;
    }
    ipsecModuleInfo_p->dpTbl->numInPolicies[newSpdInTblIdx] = 0;
    ipsecModuleInfo_p->dpTbl->spdInTblIdx = newSpdInTblIdx;
    
    ipsecModuleInfo_p->dpTbl->spdInTbl[oldSpdInTblIdx] =
                        adk_ipsec_spd_in_create (sysHandle, ipsecHandle, oldSpdInTblIdx);
    if (NULL == ipsecModuleInfo_p->dpTbl->spdInTbl[oldSpdInTblIdx])
    {
        ADK_IPSEC_LOG_FORCE("IPSEC: SPD IN ACL CTX creation failed!\n");
        rte_acl_free(ipsecModuleInfo_p->dpTbl->spdInTbl[newSpdInTblIdx]);
        return ADK_E_MEM_UNAVAILABLE;
    }
    ipsecModuleInfo_p->dpTbl->numInPolicies[oldSpdInTblIdx] = 0;
    
    /* SPD OUT ACL creation for active and backup copies */
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdTblIdx, newSpdTblIdx, oldSpdTblIdx);
    ipsecModuleInfo_p->dpTbl->spdTbl[newSpdTblIdx] =
                        adk_ipsec_spd_create (sysHandle, ipsecHandle, newSpdTblIdx);
    if (NULL == ipsecModuleInfo_p->dpTbl->spdTbl[newSpdTblIdx])
    {
        ADK_IPSEC_LOG_FORCE("IPSEC: SPD OUT ACL CTX creation failed!\n");
        return ADK_E_MEM_UNAVAILABLE;
    }
    ipsecModuleInfo_p->dpTbl->numPolicies[newSpdTblIdx] = 0;
    ipsecModuleInfo_p->dpTbl->spdTblIdx = newSpdTblIdx;

    ipsecModuleInfo_p->dpTbl->spdTbl[oldSpdTblIdx] =
                        adk_ipsec_spd_create (sysHandle, ipsecHandle, oldSpdTblIdx);
    if (NULL == ipsecModuleInfo_p->dpTbl->spdTbl[oldSpdTblIdx])
    {
        ADK_IPSEC_LOG_FORCE("IPSEC: SPD OUT ACL CTX creation failed!\n");
        rte_acl_free(ipsecModuleInfo_p->dpTbl->spdTbl[newSpdTblIdx]);
        return ADK_E_MEM_UNAVAILABLE;
    }
    ipsecModuleInfo_p->dpTbl->numPolicies[oldSpdTblIdx] = 0;

    /* Initialize Initial Frag and Buffered pkt Info Hash table */
    for(i=0; i < ADK_IPSEC_INITIAL_FRAG_INFO_TBL_HASH_MAX; i++)
    {
        for(j=0; j < ADK_IPSEC_SINGLE_SA_MAX_COLLISION; j++)
        {
            ipsecModuleInfo_p->dpTbl->ipsecInitialFragInfoBufferedPktsHashTbl[i].bucket[j].initial_frag_visited = FALSE;
            ipsecModuleInfo_p->dpTbl->ipsecInitialFragInfoBufferedPktsHashTbl[i].bucket[j].buffered_pkt_task_chain = NULL;
            ipsecModuleInfo_p->dpTbl->ipsecInitialFragInfoBufferedPktsHashTbl[i].bucket[j].numBufferedFrags = 0;
        }
    }
    /* By default, preFragBufferTimeOutVal is 100 milliseconds */
    ipsecModuleInfo_p->dpTbl->preFragBufferTimeOutVal = ADK_IPSEC_PREFRAG_BUFFER_TIMEOUT_MILLISECONDS;

    /* Utility counters are enabled by default */
    ipsecModuleInfo_p->dpTbl->isUtilityCntEnabled = true;
    ipsecModuleInfo_p->dpTbl->cbParamIndex = 0;

    return retValue;
}

adk_status_t
adk_ipsec_dp_table_shutdown(adk_module_handle_t ipsecHandle,
                            adk_sys_handle_t sysHandle,
                            ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint8_t i;
    uint32_t saIndex;
    uint8_t   newSpdInTblIdx, oldSpdInTblIdx;
    uint8_t   newSpdTblIdx, oldSpdTblIdx;

    adk_ipsec_dp_table_info_t *dpTbl = ipsecModuleInfo_p->dpTbl;

    /*  Any lingering hardware footprints need to be deleted prior to shutdown */

    for (i = 0; i < ipsecModuleInfo_p->numStaleEntries; i++)
    {
        saIndex = ipsecModuleInfo_p->staleSaEntries[i];
        saIndex &= ~(1 << ADK_IPSEC_SA_DIR_SHIFT);
        if ((ipsecModuleInfo_p->staleSaEntries[i] >> ADK_IPSEC_SA_DIR_SHIFT) &
                    ADK_IPSEC_INBOUND)
        {
            adk_ipsec_dp_lkaside_info_t *lkasideInfo = ipsecModuleInfo_p->dpTbl->iSATbl[saIndex].lkasideInfo;
            if (lkasideInfo != NULL)
            {
                if(lkasideInfo->cryptoOp)
                {
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_crypto_op_free", "Free "
                            "cryptodev operation buffer\n");
                    rte_crypto_op_free(lkasideInfo->cryptoOp);
                }
                if(lkasideInfo->cryptoSession)
                {
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_cryptodev_sym_session_free", "Free "
                            "cryptodev lookaside session\n");
                    rte_cryptodev_sym_session_clear(lkasideInfo->cryptoDevId,
                            lkasideInfo->cryptoSession);
                    rte_cryptodev_sym_session_free(
                            lkasideInfo->cryptoSession);
                }
                ADK_DP_SHM_FREE(lkasideInfo);
                lkasideInfo = NULL;
            }

            if (adk_rm_resource_isset (ipsecModuleInfo_p->iSASetHdl, saIndex))
            {
                ADK_NVM_FREE(ipsecModuleInfo_p->dpTbl->iSATbl[saIndex].window_buffer);
                adk_rm_resource_free(ipsecModuleInfo_p->iSASetHdl, saIndex);
            }

            if (ipsecModuleInfo_p->secCtxCP)
            {
                if (ipsecModuleInfo_p->dpTbl->iSATbl[saIndex].secSession)
                {
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_security_session_destroy", "Destroy "
                                        "inbound CPM security session to erase hardware footprint \n");
                    rte_security_session_destroy(ipsecModuleInfo_p->secCtxCP,
                            ipsecModuleInfo_p->dpTbl->iSATbl[saIndex].secSession);
                }
            }
        }
        else
        {
            adk_ipsec_del_eSA (sysHandle, ipsecHandle, ipsecModuleInfo_p,
                               saIndex);
        }
    }

    /* free all the memory first
     */
    if (dpTbl->inlineSessPool)
    {
        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_free", "Free "
                            "inline crypto device session mempool\n");
        rte_mempool_free(dpTbl->inlineSessPool);
    }
    for (i = 0; i < ADK_IPSEC_MAX_CRYPTO_DEV; i++)
    {
        if (dpTbl->cryptoDevMap[i].isValid)
        {
            if (dpTbl->cryptoDevMap[i].sessPool)
            {
                ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_free", "Free "
                                    "crypto device session mempool\n");
                rte_mempool_free(dpTbl->cryptoDevMap[i].sessPool);
            }
        }
    }
    /* Reset rules and free SPD IN ACL array for active and backup copies */
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdInTblIdx, newSpdInTblIdx, oldSpdInTblIdx);
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset", "Reset "
	                            "SPD IN ACL trie\n");
    if(ipsecModuleInfo_p->dpTbl->numInPolicies[newSpdInTblIdx] != 0)
    {
       rte_acl_reset(ipsecModuleInfo_p->dpTbl->spdInTbl[newSpdInTblIdx]);
    }
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Free "
	                            "SPD IN ACL trie\n");
    ipsecModuleInfo_p->dpTbl->spdInTblIdx = newSpdInTblIdx;
	rte_acl_free(ipsecModuleInfo_p->dpTbl->spdInTbl[newSpdInTblIdx]);
    adk_rcu_synchronize();
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Reset "
                                "SPD IN ACL trie shadow copy\n");
    if(ipsecModuleInfo_p->dpTbl->numInPolicies[oldSpdInTblIdx] != 0)
    {
       rte_acl_reset(ipsecModuleInfo_p->dpTbl->spdInTbl[oldSpdInTblIdx]);
    }
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Free "
	                            "SPD IN ACL trie shadow copy\n");
    rte_acl_free(ipsecModuleInfo_p->dpTbl->spdInTbl[oldSpdInTblIdx]);

    /* Reset rules and free SPD OUT ACL array for active and backup copies */
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdTblIdx, newSpdTblIdx, oldSpdTblIdx);
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset", "Reset "
				                        "ACL trie for specified rule set ID\n");
    if(ipsecModuleInfo_p->dpTbl->numPolicies[newSpdTblIdx] != 0)
    {
       rte_acl_reset(ipsecModuleInfo_p->dpTbl->spdTbl[newSpdTblIdx]);
    }
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Free "
				                        "ACL trie for specified rule set ID\n");
    rte_acl_free(ipsecModuleInfo_p->dpTbl->spdTbl[newSpdTblIdx]);
    ipsecModuleInfo_p->dpTbl->spdTblIdx = newSpdTblIdx;

    adk_rcu_synchronize();
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Reset "
				                        "ACL trie shadow copy for specified rule set ID\n");
    if(ipsecModuleInfo_p->dpTbl->numPolicies[oldSpdTblIdx] != 0)
    {
       rte_acl_reset(ipsecModuleInfo_p->dpTbl->spdTbl[oldSpdTblIdx]);
    }
    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_free", "Free "
				                        "ACL trie shadow copy for specified rule set ID\n");
    rte_acl_free(ipsecModuleInfo_p->dpTbl->spdTbl[oldSpdTblIdx]);

    adk_rm_resource_destroy(ipsecModuleInfo_p->policyHdl);
    adk_rm_resource_destroy(ipsecModuleInfo_p->policyInHdl);
    adk_rm_resource_destroy(ipsecModuleInfo_p->eSASetHdl);
    adk_rm_resource_destroy(ipsecModuleInfo_p->iSASetHdl);

    dpTbl->cookie = 0;

    if (ADK_SUCCESS != adk_shm_destroy(ADK_IPSEC_SHM_MEMZONE_NAME))
    {
        ADK_IPSEC_LOG_ERROR (ipsecHandle, ADK_E_INTERNAL_ERROR,
                             "Failed to destroy shared memory\n");
        return ADK_E_INTERNAL_ERROR;
    }

    return retValue;
}

adk_status_t
adk_ipsec_crypto_dev_init(adk_sys_handle_t sysHandle,
                          adk_module_handle_t ipsecHandle, 
                          ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_sys_resource_info_t         *cdevInfoList;
    adk_sys_rsrc_crypto_dev_info_t  *cryptoDevInfo;
    char poolName[RTE_MEMPOOL_NAMESIZE];
    uint32_t  maxSessSize = 0, socketId = 0;
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    uint8_t   numDevs;
    uint16_t i;
    if (ipsecModuleInfo_p->numCryptoDevices != 0)
    {
        ADK_IPSEC_LOG_INFO(ipsecHandle, "IPSEC: Crypto devices were already initialized\n");
        return ADK_IPSEC_SUCCESS;
    }
    retValue = adk_sys_config_crypto_dev_info_get (&cdevInfoList,
                                                   &numDevs);
    if (retValue != ADK_IPSEC_SUCCESS)
    {
        ADK_FORCE_LOG_ERROR ("[%s()] Cryptodev resource get function failed\n",
                                __func__);
        return ADK_E_INTERNAL_ERROR;
    }

    if ((numDevs != 0) && (cdevInfoList != NULL))
    {
        for (i = 0; i < numDevs; i++)
        {
            cryptoDevInfo = (adk_sys_rsrc_crypto_dev_info_t *)cdevInfoList[i].crypto_dev_info_p;
            ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices] =
                (adk_sys_rsrc_crypto_dev_info_t *)ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle), ADK_IPSEC_CRYPTO_INFO_OBJ,
					 1, sizeof(adk_sys_rsrc_crypto_dev_info_t));
            ADK_MEMCPY(ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices],
                       cryptoDevInfo,
                       sizeof(adk_sys_rsrc_crypto_dev_info_t));
            /* Q-pair info is valid only for lookaside type of crypto devices,
             * in case of inline type, Q-pair is not exposed to application
             */
            if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
            {
                ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices]->qp_info_p =
                    (adk_sys_rsrc_crypto_dev_qpair_info_t *)ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle), ADK_IPSEC_QPAIR_INFO_OBJ,
                            cryptoDevInfo->num_q_pairs, sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));
                ADK_MEMCPY(ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices]->qp_info_p,
                        cryptoDevInfo->qp_info_p,
                        cryptoDevInfo->num_q_pairs * sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));
                if (cryptoDevInfo->num_sas != 0)
                {
                    /* now do mempool lookup with poolname created in primary process
                    */
                    snprintf(poolName, RTE_MEMPOOL_NAMESIZE, "ipsec_sess_mp_%u", cryptoDevInfo->dev_id);
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_lookup", "Lookup"
                            "mempool for sessions on crypto device\n");
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].sessPool = rte_mempool_lookup(poolName);
                    if (ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].sessPool == NULL)
                    {
                        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "IPSEC: Session memool lookup failed\n");
                        return ADK_E_INTERNAL_ERROR;
                    }
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = true;
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].devId = cryptoDevInfo->dev_id;
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].numSAsAllowed = cryptoDevInfo->num_sas;
                }
                else
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = false;
            }
            /* currently, assumption is that there would be only one inline device,
             * in case of inline device, we need to store
             * dev_id only for the time being. Whenever we allow application to
             * create multiple such devices and split it for number of SAs, need
             * to enhance data strucure and this logic
             */
            else if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_INLINE)
            {
                if (ipsecModuleInfo_p->secCtxCP)
                {
                    ADK_IPSEC_LOG_INFO(ipsecHandle, "\n Inline device already registered \n");
                    continue;
                }
                ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = true;
                ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].devId = cryptoDevInfo->dev_id;
                /* doing this because we support only one inline device,
                 * else, need to have some scheme to decide
                 * which inline dev to use, e.g. round robin. Not required right now
                 */
                ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_eth_dev_get_sec_ctx", "Get "
                                    "security context for inline crypto device\n");
                ipsecModuleInfo_p->secCtxCP = (struct rte_security_ctx *)rte_eth_dev_get_sec_ctx(cryptoDevInfo->dev_id);
                if (ipsecModuleInfo_p->secCtxCP == NULL)
                {
                    ADK_IPSEC_LOG_INFO(ipsecHandle, "No inline device found! \n");
                }
                else
                {
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_eth_dev_socket_id", "Get "
                                        "socket ID of device needed to create crypto session mempool \n");
                    /* TODO is this correct? passing 0 ?
                     */
                    socketId = rte_eth_dev_socket_id(0);
                    ADK_MEMSET(poolName, 0, sizeof(poolName));
                    snprintf(poolName, RTE_MEMPOOL_NAMESIZE, "ipsec_inlinesess_mp_%u", socketId);
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_security_session_get_size", "Get "
                                        "maximum size of session of inline crypto device security context\n");
                    maxSessSize = rte_security_session_get_size(ipsecModuleInfo_p->secCtxCP);
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_create", "Create "
                                        "mempool for sessions on crypto device\n");
                    ipsecModuleInfo_p->dpTbl->inlineSessPool = rte_mempool_create(poolName,
                                                                      ADK_IPSEC_NUM_OF_SESSIONS,
                                                                      maxSessSize,
                                                                      0, 0, NULL, NULL, NULL, NULL,
                                                                      socketId, 0);
                    if (ipsecModuleInfo_p->dpTbl->inlineSessPool == NULL)
                    {
                        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                          "Failed to allocate session pool for inline\n"
                                          );
                        return ADK_E_MEM_UNAVAILABLE;
                    }
                }
            }
            ipsecModuleInfo_p->numCryptoDevices++;
        }
    }
    else
    {
        ADK_IPSEC_LOG_INFO(ipsecHandle, "IPSEC: Crypto device resource unavailable at init time - needs to be registered using callback function - adk_ipsec_rsrc_notify_fn_cbfn");
        ipsecModuleInfo_p->numCryptoDevices = 0;
    }
    return ADK_IPSEC_SUCCESS;	
}

static void create_spd_dpdk_acl_pattern(adk_ipsec_spd_rule_t *spdRule_p,
                                        struct adk_ipsec_acl_rule *aclRule)
{
    /* version */
    aclRule->field[VERSION_FIELD_IPSEC].value.u8 = spdRule_p->version;
    aclRule->field[VERSION_FIELD_IPSEC].mask_range.u8 = spdRule_p->version;

    /* SIP */
    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32 =
        spdRule_p->srcAddr_Begin[0];
    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32 =
        spdRule_p->srcAddr_End[0];

    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32 =
        spdRule_p->srcAddr_Begin[1];
    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32 =
        spdRule_p->srcAddr_End[1];

    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32 =
        spdRule_p->srcAddr_Begin[2];
    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32 =
        spdRule_p->srcAddr_End[2];

    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32 =
        spdRule_p->srcAddr_Begin[3];
    aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32 =
        spdRule_p->srcAddr_End[3];

    /* DIP */
    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32 =
        spdRule_p->dstAddr_Begin[0];
    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32 =
        spdRule_p->dstAddr_End[0];

    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32 =
        spdRule_p->dstAddr_Begin[1];
    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32 =
        spdRule_p->dstAddr_End[1];

    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32 =
        spdRule_p->dstAddr_Begin[2];
    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32 =
        spdRule_p->dstAddr_End[2];

    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32 =
        spdRule_p->dstAddr_Begin[3];
    aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32 =
        spdRule_p->dstAddr_End[3];

    /* DSCP value */
    aclRule->field[DSCP_FIELD_IPSEC].value.u16 = (uint16_t)spdRule_p->dscpVal_Begin;
    aclRule->field[DSCP_FIELD_IPSEC].mask_range.u16 = (uint16_t)spdRule_p->dscpVal_End;

    /* protocol */
    aclRule->field[PROTO_FIELD_IPSEC].value.u16 = (uint16_t)spdRule_p->proto_Begin;
    aclRule->field[PROTO_FIELD_IPSEC].mask_range.u16 = (uint16_t)spdRule_p->proto_End;

    /* SP */
    aclRule->field[SP_FIELD_IPSEC].value.u16 = spdRule_p->srcPort_Begin;
    aclRule->field[SP_FIELD_IPSEC].mask_range.u16 = spdRule_p->srcPort_End;

    /* DP */
    aclRule->field[DP_FIELD_IPSEC].value.u16 = spdRule_p->dstPort_Begin;
    aclRule->field[DP_FIELD_IPSEC].mask_range.u16 = spdRule_p->dstPort_End;
}

static void create_spd_in_dpdk_acl_pattern(adk_ipsec_spd_in_rule_t *spdRule_p,
                                           struct adk_ipsec_in_acl_rule *aclRule)
{
    /* version */
    aclRule->field[VERSION_FIELD_IPSEC_IN].value.u8 = spdRule_p->version;
    aclRule->field[VERSION_FIELD_IPSEC_IN].mask_range.u8 = spdRule_p->version;

    /* SIP */
    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32 =
        spdRule_p->srcAddr_Begin[0];
    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32 =
        spdRule_p->srcAddr_End[0];

    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32 =
        spdRule_p->srcAddr_Begin[1];
    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32 =
        spdRule_p->srcAddr_End[1];

    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32 =
        spdRule_p->srcAddr_Begin[2];
    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32 =
        spdRule_p->srcAddr_End[2];

    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32 =
        spdRule_p->srcAddr_Begin[3];
    aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32 =
        spdRule_p->srcAddr_End[3];

    /* DIP */
    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32 =
        spdRule_p->dstAddr_Begin[0];
    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32 =
        spdRule_p->dstAddr_End[0];

    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32 =
        spdRule_p->dstAddr_Begin[1];
    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32 =
        spdRule_p->dstAddr_End[1];

    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32 =
        spdRule_p->dstAddr_Begin[2];
    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32 =
        spdRule_p->dstAddr_End[2];

    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32 =
        spdRule_p->dstAddr_Begin[3];
    aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32 =
        spdRule_p->dstAddr_End[3];

    /* protocol */
    aclRule->field[PROTO_FIELD_IPSEC_IN].value.u32 = (uint32_t)spdRule_p->proto_Begin;
    aclRule->field[PROTO_FIELD_IPSEC_IN].mask_range.u32 = (uint32_t)spdRule_p->proto_End;

    /* SP */
    aclRule->field[SP_FIELD_IPSEC_IN].value.u16 = spdRule_p->srcPort_Begin;
    aclRule->field[SP_FIELD_IPSEC_IN].mask_range.u16 = spdRule_p->srcPort_End;

    /* DP */
    aclRule->field[DP_FIELD_IPSEC_IN].value.u16 = spdRule_p->dstPort_Begin;
    aclRule->field[DP_FIELD_IPSEC_IN].mask_range.u16 = spdRule_p->dstPort_End;
}

void ipsec_policy_ptn_create(adk_ipsec_spd_rule_t *ptn,
                             adk_ipsec_sec_pol_sel_t *secPolSel)
{
    uint16_t prefixWords = 0, maskBits = 0, i;
    uint32_t mask;
    /* need to copy SIP and DIP based on the version of the selectors */

    if (secPolSel->ipVersion == ADK_IPSEC_IPV4)
    {
        ptn->version = 4;

        if (secPolSel->ipAddr.ipv4Addr.srcAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Source IPv4 address subnet */
            if (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth !=0)
            {

                mask = (1<<(32-secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth)) -1;
            }
            else
            {
                mask = 0xffffffff;
            }
            ptn->srcAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.addr) & (~mask);
            ptn->srcAddr_End[0] = ptn->srcAddr_Begin[0] | mask;

        }
        else
        {
            /* Source IPv4 address range */
            ptn->srcAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin);
            ptn->srcAddr_End[0] =  (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd);

        }
        ptn->srcAddr_Begin[1] = 0x0;
        ptn->srcAddr_End[1] =  0xffffffff;
        ptn->srcAddr_Begin[2] = 0x0;
        ptn->srcAddr_End[2] =  0xffffffff;
        ptn->srcAddr_Begin[3] = 0x0;
        ptn->srcAddr_End[3] =  0xffffffff;

        if (secPolSel->ipAddr.ipv4Addr.dstAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Destination IPv4 address subnet */
            if (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth !=0)
            {

                mask = (1<<(32-secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth)) -1;
            }
            else
            {
                mask = 0xffffffff;
            }
            ptn->dstAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.addr) & (~mask);
            ptn->dstAddr_End[0] = ptn->dstAddr_Begin[0] | mask;

        }
        else
        {
            /* Destination IPv4 address range */
            ptn->dstAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin);
            ptn->dstAddr_End[0] =  (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd);
        }
        ptn->dstAddr_Begin[1] = 0x0;
        ptn->dstAddr_End[1] =  0xffffffff;
        ptn->dstAddr_Begin[2] = 0x0;
        ptn->dstAddr_End[2] =  0xffffffff;
        ptn->dstAddr_Begin[3] = 0x0;
        ptn->dstAddr_End[3] =  0xffffffff;
    }
    else
    {
        ptn->version = 6;
        if (secPolSel->ipAddr.ipv6Addr.srcAddr.rangeSelFlag != ADK_IPSEC_TRUE) {
            /* Source IPv6 address subnet */
            prefixWords = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth / 32;
            maskBits = 32 - (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth % 32);

            for(i=0; i<4; i++)
            {
                if (i < prefixWords)
                {
                    ptn->srcAddr_Begin[i] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i]);
                    ptn->srcAddr_End[i] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i]);
                }
                else if (i == prefixWords)
                {
                    if (maskBits == 32) {
                        ptn->srcAddr_Begin[i] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i] & 0);
                    } else {
                        ptn->srcAddr_Begin[i] = ((secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i]) & (0xffffffff << maskBits));
                    }
                    ptn->srcAddr_End[i] = ptn->srcAddr_Begin[i] | (0xffffffff >> (32 - maskBits));
                } else
                {
                    ptn->srcAddr_Begin[i] = 0x0;
                    ptn->srcAddr_End[i] = 0xffffffff;
                }
            }
        }
        else{
            /* Source IPv6 address range */
            ptn->srcAddr_Begin[0] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[0]);
            ptn->srcAddr_End[0] =  (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[0]);
            ptn->srcAddr_Begin[1] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[1]);
            ptn->srcAddr_End[1] =  (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[1]);
            ptn->srcAddr_Begin[2] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[2]);
            ptn->srcAddr_End[2] =  (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[2]);
            ptn->srcAddr_Begin[3] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[3]);
            ptn->srcAddr_End[3] =  (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[3]);
        }
        if (secPolSel->ipAddr.ipv6Addr.dstAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Destiantion IPv6 address subnet */
            prefixWords = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth / 32;
            maskBits = 32 - (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth % 32);

            for(i=0; i<4; i++)
            {
                if (i < prefixWords)
                {
                    ptn->dstAddr_Begin[i] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i]);
                    ptn->dstAddr_End[i] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i]);
                }
                else if (i == prefixWords)
                {
                    if (maskBits == 32) {
                        ptn->dstAddr_Begin[i] = ((secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i]) & 0);
                    } else {
                        ptn->dstAddr_Begin[i] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i] & (0xffffffff << maskBits));
                    }
                    ptn->dstAddr_End[i] = ptn->dstAddr_Begin[i] | (0xffffffff >> (32 - maskBits));
                } else
                {
                    ptn->dstAddr_Begin[i] = 0x0;
                    ptn->dstAddr_End[i] = 0xffffffff;
                }
            }
        }
        else{
            /* Destination IPv6 address range */
            ptn->dstAddr_Begin[0] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[0]);
            ptn->dstAddr_End[0] =  (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[0]);
            ptn->dstAddr_Begin[1] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[1]);
            ptn->dstAddr_End[1] =  (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[1]);
            ptn->dstAddr_Begin[2] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[2]);
            ptn->dstAddr_End[2] =  (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[2]);
            ptn->dstAddr_Begin[3] = (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[3]);
            ptn->dstAddr_End[3] =  (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[3]);
        }
    }

    if (secPolSel->protocol == 0x0)
    {
        ptn->proto_Begin = 0;
        ptn->proto_End = 255;
    }
    else
    {
        ptn->proto_Begin = ptn->proto_End = secPolSel->protocol;
    }

    /* sec pol protocol is not TCP/UDP, make src port as ANY */
    if ((secPolSel->protocol == PROTO_TCP) || (secPolSel->protocol == PROTO_UDP) ||
        (secPolSel->protocol == PROTO_SCTP))
    {
       ptn->srcPort_Begin = (secPolSel->l4Info.portInfo.srcPort.portBegin) ;
       ptn->srcPort_End =  (secPolSel->l4Info.portInfo.srcPort.portEnd) ;
       ptn->dstPort_Begin = (secPolSel->l4Info.portInfo.dstPort.portBegin);
       ptn->dstPort_End =  (secPolSel->l4Info.portInfo.dstPort.portEnd);
    }
    else if((secPolSel->protocol == PROTO_ICMP) || (secPolSel->protocol == PROTO_ICMPV6))
    {
        ptn->srcPort_Begin = rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.type.typeBegin) ;
        ptn->srcPort_End =  rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.type.typeEnd) ;
        ptn->dstPort_Begin = rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.code.codeBegin);
        ptn->dstPort_End =  rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.code.codeEnd);
    }
    else
    {
        ptn->srcPort_Begin = (0) ;
        ptn->srcPort_End = (0xffff);
        ptn->dstPort_Begin = (0) ;
        ptn->dstPort_End = (0xffff);
    }
    if (secPolSel->dscpVal == 0XFF)
    {
        ptn->dscpVal_Begin = 0;
        ptn->dscpVal_End = 255;
    }
    else
    {
        ptn->dscpVal_Begin = ptn->dscpVal_End = secPolSel->dscpVal;
    }
}

void ipsec_spd_in_policy_ptn_create(adk_ipsec_spd_in_rule_t *ptn,
                                           adk_ipsec_sec_pol_in_sel_t *secPolSel)
{
    uint16_t prefixWords = 0, maskBits = 0, i;
    uint32_t mask;
    /* need to copy SIP and DIP based on the version of the selectors */

    if (secPolSel->ipVersion == ADK_IPSEC_IPV4)
    {
        ptn->version = 4;

        if (secPolSel->ipAddr.ipv4Addr.srcAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Source IPv4 address subnet */
            if (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth !=0)
            {

                mask = (1<<(32-secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth)) -1;
            }
            else
            {
                mask = 0xffffffff;
            }
            ptn->srcAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.addr) & (~mask);
            ptn->srcAddr_End[0] = ptn->srcAddr_Begin[0] | mask;

        }
        else
        {
            /* Source IPv4 address range */
            ptn->srcAddr_Begin[0] = secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin;
            ptn->srcAddr_End[0] =  secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd;

        }
        ptn->srcAddr_Begin[1] = 0x0;
        ptn->srcAddr_End[1] =  0xffffffff;
        ptn->srcAddr_Begin[2] = 0x0;
        ptn->srcAddr_End[2] =  0xffffffff;
        ptn->srcAddr_Begin[3] = 0x0;
        ptn->srcAddr_End[3] =  0xffffffff;

        if (secPolSel->ipAddr.ipv4Addr.dstAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Destination IPv4 address subnet */
            if (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth !=0)
            {

                mask = (1<<(32-secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth)) -1;
            }
            else
            {
                mask = 0xffffffff;
            }
            ptn->dstAddr_Begin[0] = (secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.addr) & (~mask);
            ptn->dstAddr_End[0] = ptn->dstAddr_Begin[0] | mask;

        }
        else
        {
            /* Destination IPv4 address range */
            ptn->dstAddr_Begin[0] = secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin;
            ptn->dstAddr_End[0] =  secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd;
        }
        ptn->dstAddr_Begin[1] = 0x0;
        ptn->dstAddr_End[1] =  0xffffffff;
        ptn->dstAddr_Begin[2] = 0x0;
        ptn->dstAddr_End[2] =  0xffffffff;
        ptn->dstAddr_Begin[3] = 0x0;
        ptn->dstAddr_End[3] =  0xffffffff;
    }
    else
    {
        ptn->version = 6;
        if (secPolSel->ipAddr.ipv6Addr.srcAddr.rangeSelFlag != ADK_IPSEC_TRUE) {
            /* Source IPv6 address subnet */
            prefixWords = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth / 32;
            maskBits = 32 - (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth % 32);

            for(i=0; i<4; i++)
            {
                if (i < prefixWords)
                {
                    ptn->srcAddr_Begin[i] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i];
                    ptn->srcAddr_End[i] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i];
                }
                else if (i == prefixWords)
                {
                    if (maskBits == 32) {
                        ptn->srcAddr_Begin[i] = (secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i] & 0);
                    } else {
                        ptn->srcAddr_Begin[i] = ((secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.addr[i]) & (0xffffffff << maskBits));
                    }
                    ptn->srcAddr_End[i] = ptn->srcAddr_Begin[i] | (0xffffffff >> (32 - maskBits));
                } else
                {
                    ptn->srcAddr_Begin[i] = 0x0;
                    ptn->srcAddr_End[i] = 0xffffffff;
                }
            }
        }
        else{
            /* Source IPv6 address range */
            ptn->srcAddr_Begin[0] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[0];
            ptn->srcAddr_End[0] =  secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[0];
            ptn->srcAddr_Begin[1] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[1];
            ptn->srcAddr_End[1] =  secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[1];
            ptn->srcAddr_Begin[2] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[2];
            ptn->srcAddr_End[2] =  secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[2];
            ptn->srcAddr_Begin[3] = secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[3];
            ptn->srcAddr_End[3] =  secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[3];
        }
        if (secPolSel->ipAddr.ipv6Addr.dstAddr.rangeSelFlag != ADK_IPSEC_TRUE) {

            /* Destiantion IPv6 address subnet */
            prefixWords = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth / 32;
            maskBits = 32 - (secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth % 32);

            for(i=0; i<4; i++)
            {
                if (i < prefixWords)
                {
                    ptn->dstAddr_Begin[i] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i];
                    ptn->dstAddr_End[i] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i];
                }
                else if (i == prefixWords)
                {
                    if (maskBits == 32) {
                        ptn->dstAddr_Begin[i] = ((secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i]) & 0);
                    } else {
                        ptn->dstAddr_Begin[i] = ((secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.addr[i]) & (0xffffffff << maskBits));
                    }
                    ptn->dstAddr_End[i] = ptn->dstAddr_Begin[i] | (0xffffffff >> (32 - maskBits));
                } else
                {
                    ptn->dstAddr_Begin[i] = 0x0;
                    ptn->dstAddr_End[i] = 0xffffffff;
                }
            }
        }
        else{
            /* Destination IPv6 address range */
            ptn->dstAddr_Begin[0] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[0];
            ptn->dstAddr_End[0] =  secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[0];
            ptn->dstAddr_Begin[1] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[1];
            ptn->dstAddr_End[1] =  secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[1];
            ptn->dstAddr_Begin[2] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[2];
            ptn->dstAddr_End[2] =  secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[2];
            ptn->dstAddr_Begin[3] = secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[3];
            ptn->dstAddr_End[3] =  secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[3];
        }
    }

    if (secPolSel->protocol == 0x0)
    {
        ptn->proto_Begin = 0;
        ptn->proto_End = 255;
    }
    else
    {
        ptn->proto_Begin = ptn->proto_End = secPolSel->protocol;
    }

    /* sec pol protocol is not TCP/UDP, make src port as ANY */
    if ((secPolSel->protocol == PROTO_TCP) || (secPolSel->protocol == PROTO_UDP) ||
        (secPolSel->protocol == PROTO_SCTP))
    {
       ptn->srcPort_Begin = secPolSel->l4Info.portInfo.srcPort.portBegin;
       ptn->srcPort_End =  secPolSel->l4Info.portInfo.srcPort.portEnd;
       ptn->dstPort_Begin = secPolSel->l4Info.portInfo.dstPort.portBegin;
       ptn->dstPort_End =  secPolSel->l4Info.portInfo.dstPort.portEnd;
    }
    else if((secPolSel->protocol == PROTO_ICMP) || (secPolSel->protocol == PROTO_ICMPV6))
    {
        ptn->srcPort_Begin = rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.type.typeBegin) ;
        ptn->srcPort_End =  rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.type.typeEnd) ;
        ptn->dstPort_Begin = rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.code.codeBegin);
        ptn->dstPort_End =  rte_cpu_to_be_16(secPolSel->l4Info.icmpInfo.code.codeEnd);
    }
    else
    {
        ptn->srcPort_Begin = (0) ;
        ptn->srcPort_End = (0xffff);
        ptn->dstPort_Begin = (0) ;
        ptn->dstPort_End = (0xffff);
    }
}

/* Function to do below things,
 * 1. Add/update policy attributes for the 
 *    corresponding policy index in DP.
 * 2. Fetch the attributes from DP policy structure.
 */
void adk_ipsec_dp_policy_attributes (adk_ipsec_sec_pol_sainfo_t *dpPolAttr_p,
                                     adk_ipsec_sec_pol_attr_t *secPolAttr_p,
                                     bool update)
{
    if (update)
    {
            dpPolAttr_p->spIDEx = secPolAttr_p->spIDEx;
            dpPolAttr_p->action = secPolAttr_p->action;
            dpPolAttr_p->configDF = secPolAttr_p->configDF;
            dpPolAttr_p->useSingleSAInPrefrag = secPolAttr_p->useSingleSAInPrefrag;
            dpPolAttr_p->preFragEnable = secPolAttr_p->preFragEnable;
            dpPolAttr_p->tunnelMode = secPolAttr_p->tunnelMode;
            dpPolAttr_p->espProto = secPolAttr_p->espProto;
            dpPolAttr_p->pathMTU = secPolAttr_p->pathMTU;
    }
    else
    {
            secPolAttr_p->spIDEx = dpPolAttr_p->spIDEx;
            secPolAttr_p->action = dpPolAttr_p->action;
            secPolAttr_p->configDF = dpPolAttr_p->configDF;
            secPolAttr_p->useSingleSAInPrefrag = dpPolAttr_p->useSingleSAInPrefrag;
            secPolAttr_p->preFragEnable = dpPolAttr_p->preFragEnable;
            secPolAttr_p->tunnelMode = dpPolAttr_p->tunnelMode;
            secPolAttr_p->espProto = dpPolAttr_p->espProto;
            secPolAttr_p->pathMTU = dpPolAttr_p->pathMTU;
            secPolAttr_p->saNotFoundErrDrop = dpPolAttr_p->saNotFoundErrDrop;
    }
}

/* Function to verify SP parameter values */
static adk_ipsec_return_code_t
ipsec_sec_pol_sel_validate (adk_module_handle_t     ipsecHandle,
                            adk_ipsec_sec_pol_sel_t *secPolSel_p)
{
    if (secPolSel_p->ipVersion == ADK_IPSEC_IPV4)
    {
        if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth > IPV4_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr prefixwidth is more than 32 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        else
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd <
                    secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr - range end cannot be less than begin address\n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth > IPV4_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dst addr prefixwidth is more than 32 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        else
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd <
                    secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dest addr - range end cannot be less than begin address\n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
    }
    else if (secPolSel_p->ipVersion == ADK_IPSEC_IPV6)
    {
        if ( secPolSel_p->ipAddr.ipv6Addr.srcAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth > IPV6_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr prefixwidth is more than 128 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        if ( secPolSel_p->ipAddr.ipv6Addr.dstAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth > IPV6_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dst addr prefixwidth is more than 128 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
    }
    else
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                "Got secPolSel version: %x as non IP(4/6), invalid \
                SP input argument\n", secPolSel_p->ipVersion);
        return ADK_E_IPSEC_INVALID_POLICY;
    }

    if ((PROTO_UDP == secPolSel_p->protocol) ||
        (PROTO_TCP == secPolSel_p->protocol) ||
        (PROTO_SCTP == secPolSel_p->protocol))
    {
        if (secPolSel_p->l4Info.portInfo.srcPort.portEnd < 
            secPolSel_p->l4Info.portInfo.srcPort.portBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "Source Port Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
        if (secPolSel_p->l4Info.portInfo.dstPort.portEnd < 
            secPolSel_p->l4Info.portInfo.dstPort.portBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "Dest Port Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
    }
    else if (PROTO_ICMP == secPolSel_p->protocol)
    {
        if (secPolSel_p->l4Info.icmpInfo.type.typeEnd < 
            secPolSel_p->l4Info.icmpInfo.type.typeBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "ICMP Type Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
        if (secPolSel_p->l4Info.icmpInfo.code.codeEnd < 
            secPolSel_p->l4Info.icmpInfo.code.codeBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "ICMP Code Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
    }

    return ADK_IPSEC_SUCCESS;
} /* ipsec_sec_pol_sel_validate */

/* Function to verify SP IN parameter values */
static adk_ipsec_return_code_t
ipsec_spd_in_sec_pol_sel_validate (adk_module_handle_t        ipsecHandle,
                                   adk_ipsec_sec_pol_in_sel_t *secPolSel_p)
{
    if (secPolSel_p->ipVersion == ADK_IPSEC_IPV4)
    {
        if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_subnet_t.prefixWidth > IPV4_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr prefixwidth is more than 32 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        else
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd <
                    secPolSel_p->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr - range end cannot be less than begin address\n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_subnet_t.prefixWidth > IPV4_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dst addr prefixwidth is more than 32 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        else
        {
            if ( secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd <
                    secPolSel_p->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dest addr - range end cannot be less than begin address\n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
    }
    else if (secPolSel_p->ipVersion == ADK_IPSEC_IPV6)
    {
        if ( secPolSel_p->ipAddr.ipv6Addr.srcAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv6Addr.srcAddr.addr.addr_subnet_t.prefixWidth > IPV6_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy src addr prefixwidth is more than 128 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
        if ( secPolSel_p->ipAddr.ipv6Addr.dstAddr.rangeSelFlag == ADK_IPSEC_FALSE)
        {
            if ( secPolSel_p->ipAddr.ipv6Addr.dstAddr.addr.addr_subnet_t.prefixWidth > IPV6_ADDR_WIDTH )
            {
                ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                        "Security policy dst addr prefixwidth is more than 128 \n");
                return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
            }
        }
    }
    else
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                "Got secPolSel version: %x as non IP(4/6), invalid \
                SP input argument\n", secPolSel_p->ipVersion);
        return ADK_E_IPSEC_INVALID_POLICY;
    }
    
    if ((PROTO_UDP == secPolSel_p->protocol) ||
        (PROTO_TCP == secPolSel_p->protocol) ||
        (PROTO_SCTP == secPolSel_p->protocol))
    {
        if (secPolSel_p->l4Info.portInfo.srcPort.portEnd < 
            secPolSel_p->l4Info.portInfo.srcPort.portBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "Source Port Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
        if (secPolSel_p->l4Info.portInfo.dstPort.portEnd < 
            secPolSel_p->l4Info.portInfo.dstPort.portBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "Dest Port Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
    }
    else if (PROTO_ICMP == secPolSel_p->protocol)
    {
        if (secPolSel_p->l4Info.icmpInfo.type.typeEnd < 
            secPolSel_p->l4Info.icmpInfo.type.typeBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "ICMP Type Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
        if (secPolSel_p->l4Info.icmpInfo.code.codeEnd < 
            secPolSel_p->l4Info.icmpInfo.code.codeBegin)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_SP_SEL_PARAM,
                    "ICMP Code Range: Range end value cannot be less than begin value\n");
            return ADK_E_IPSEC_INVALID_SP_SEL_PARAM;
        }
    }

    return ADK_IPSEC_SUCCESS;
} /* ipsec_spd_in_sec_pol_sel_validate */

adk_ipsec_return_code_t
ipsec_sec_pol_attr_validate(adk_module_handle_t  ipsecHandle,
                            adk_ipsec_sec_pol_t  *secPol_p)
{
    if (NULL == secPol_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                "Got secPol_p as NULL, invalid SP input argument\n");
        return ADK_E_IPSEC_INVALID_POLICY;
    }
    if ((secPol_p->secPolAttr.action != ADK_IPSEC_SP_ACTION_BYPASS) &&
        (secPol_p->secPolAttr.action != ADK_IPSEC_SP_ACTION_DROP) &&
        (secPol_p->secPolAttr.action != ADK_IPSEC_SP_ACTION_IPSEC))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                            "Invalid SPD action input argument\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
    if (secPol_p->secPolAttr.configDF > ADK_IPSEC_SP_DF_CLEAR)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "invalid SP configDF input argument\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
    if ((secPol_p->secPolAttr.configDF ==  ADK_IPSEC_SP_DF_COPY_INNER_IP)&& (secPol_p->secPolSel.ipVersion ==  ADK_IPSEC_IPV6))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "invalid SP configDF input argument for IPv6 SPD OUT Policy\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    return ADK_IPSEC_SUCCESS;
}/* ipsec_sec_pol_attr_validate */

static adk_ipsec_return_code_t
ipsec_spd_in_sec_pol_attr_validate(adk_module_handle_t     ipsecHandle,
                                   adk_ipsec_sec_pol_in_t  *secPol_p)
{
    if (NULL == secPol_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                            "Got secPol_p as NULL, invalid SP input argument\n");
        return ADK_E_IPSEC_INVALID_POLICY;
    }
    if ((secPol_p->secPolAttr.action != ADK_IPSEC_SP_ACTION_BYPASS) &&
        (secPol_p->secPolAttr.action != ADK_IPSEC_SP_ACTION_DROP))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                            "Invalid SPD IN action input argument\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
    return ADK_IPSEC_SUCCESS;
}/* ipsec_spd_in_sec_pol_attr_validate */

adk_ipsec_return_code_t
adk_ipsec_verify_add_sp_input_args(adk_module_handle_t ipsecHandle,
                                   ipsec_module_info_t *ipsecModuleInfo_p,
                                   adk_ipsec_sec_pol_t  *secPol_p)
{
    adk_ipsec_return_code_t       retVal = ADK_IPSEC_SUCCESS;

    /* verify input argument secPol_p */
    if (secPol_p == NULL)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                "Got secPol_p as NULL, invalid SP input argument\n");
        return ADK_E_IPSEC_INVALID_POLICY;
    }

    if (ipsecModuleInfo_p->numCryptoDevices == 0)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_RESOURCE_UNAVAILABLE,
                            "Crypto device unavailable - needed for IPsec functionality, exiting!");
        return ADK_E_RESOURCE_UNAVAILABLE;
    }

    if ((secPol_p->secPolSel.ipVersion != ADK_IPSEC_IPV4) &&
        (secPol_p->secPolSel.ipVersion != ADK_IPSEC_IPV6))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_IP_VERSION,
                "Invalid IP version input argument\n");
        return ADK_E_IPSEC_INVALID_IP_VERSION;
    }

    /* verify security policy selector arguments */
    retVal = ipsec_sec_pol_sel_validate(ipsecHandle,
                                        &secPol_p->secPolSel);
    if (retVal != ADK_IPSEC_SUCCESS)
    {
        return retVal;
    }

    /* verify security policy attribute arguments */
    retVal = ipsec_sec_pol_attr_validate(ipsecHandle, secPol_p);
    if (retVal != ADK_IPSEC_SUCCESS)
    {
        return retVal;
    }

    return ADK_IPSEC_SUCCESS;
}

adk_ipsec_return_code_t
adk_ipsec_verify_add_sp_in_input_args(adk_module_handle_t ipsecHandle,
                                      adk_ipsec_sec_pol_in_t *secPol_p)
{
    adk_ipsec_return_code_t       retVal = ADK_IPSEC_SUCCESS;

    /* verify input argument secPol_p */
    if (secPol_p == NULL)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_POLICY,
                            "Got secPol_p as NULL, invalid SP input argument\n");
        return ADK_E_IPSEC_INVALID_POLICY;
    }

    if ((secPol_p->secPolSel.ipVersion != ADK_IPSEC_IPV4) &&
        (secPol_p->secPolSel.ipVersion != ADK_IPSEC_IPV6))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_IP_VERSION,
                            "Invalid IP version input argument\n");
        return ADK_E_IPSEC_INVALID_IP_VERSION;
    }

    /* verify security policy selector arguments */
    retVal = ipsec_spd_in_sec_pol_sel_validate(ipsecHandle,
                                               &secPol_p->secPolSel);
    if (retVal != ADK_IPSEC_SUCCESS)
    {
        return retVal;
    }

    /* verify security policy attribute arguments */
    retVal = ipsec_spd_in_sec_pol_attr_validate(ipsecHandle, secPol_p);
    if (retVal != ADK_IPSEC_SUCCESS)
    {
        return retVal;
    }

    return ADK_IPSEC_SUCCESS;
}

adk_ipsec_return_code_t
adk_ipsec_verify_upd_sp_input_args(adk_module_handle_t ipsecHandle,
                                   adk_ipsec_sec_pol_attr_t *secPolAttr_p)
{
    /* verify security policy attribute arguments */
    if ((secPolAttr_p->action != ADK_IPSEC_SP_ACTION_IPSEC) &&
        (secPolAttr_p->action != ADK_IPSEC_SP_ACTION_BYPASS) &&
        (secPolAttr_p->action != ADK_IPSEC_SP_ACTION_DROP))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid SP action input argument\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (secPolAttr_p->configDF > ADK_IPSEC_SP_DF_CLEAR)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "invalid SP configDF input argument\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
 
    return ADK_IPSEC_SUCCESS;
}

static __always_inline int fls(int x)
{
        return x ? sizeof(x) * 8 - __builtin_clz(x) : 0;
}

#define IPv4_BITS 32

static inline int min(int a, int b)
{
        return (a < b ? a : b);
}

/* Function to convert IPv4 address range to bitmask
 */
static int convert_addr_range_to_bitmask(uint32_t *start,
                                             uint32_t *end,
                                             adk_ipsec_ipv4_address_info_t *ipv4AddrInfo)
{
    uint32_t zeroBits, bits;
    uint32_t numOfRules = 0;

    /* special case handling!
     */
    if ((*start == 0) && (*end == 0xffffffff))
    {
        if (ipv4AddrInfo)
        {
            ipv4AddrInfo->addr = *start;
            ipv4AddrInfo->mask = 0;
        }
        return 1;
    }
    while (*start <= *end)
    {
        zeroBits = *start == 0 ? IPv4_BITS : ffs(*start) - 1;
        bits = min(fls(*end - *start + 1) - 1, zeroBits);

        if (ipv4AddrInfo)
        {
            ipv4AddrInfo->addr = *start;
            ipv4AddrInfo->mask = bits == 0 ? 0xFFFFFFFF : (~(0xFFFFFFFF >> (IPv4_BITS - bits)));
        }

        numOfRules++;
        if(numOfRules > ADK_ACL_MAX_HOS_RULES_PER_RULE)
            return -1;

        *start += 1 << bits;

        if (ipv4AddrInfo)
            ipv4AddrInfo += 1;
    }

    return numOfRules;
}

static void ipsec_build_hos_rule(adk_ipsec_ip_version_t ipVersion,
                    uint8_t protocol,
                    adk_ipsec_ipv4_address_info_t   *ipv4SIP,
                    adk_acl_addr_range_bitmask_t *ipv6SIP,
                    int sipRules,
                    adk_ipsec_ipv4_address_info_t   *ipv4DIP,
                    adk_acl_addr_range_bitmask_t *ipv6DIP,
                    int dipRules,
                    adk_acl_range_bitmask_t *sport_bitmask_p,
                    int sport_rule_count,
                    adk_acl_range_bitmask_t *dport_bitmask_p,
                    int dport_rule_count,
                    uint32_t polHdlIdx,
                    adk_hos_spd_tbl_rule_attr_t **rule_attr)
{
    int siploop, diploop, sportloop, dportloop, rule_count;

    rule_count = 0;
    adk_hos_spd_tbl_rule_attr_t *hos_rule = rule_attr[rule_count];

    for (siploop = 0; siploop < sipRules; siploop++)
    {
        for (diploop = 0; diploop < dipRules; diploop++)
        {
            for(sportloop = 0; sportloop < sport_rule_count; sportloop++)
            {
                for(dportloop=0; dportloop < dport_rule_count; dportloop++)
                {
                    hos_rule->is_ipv6 = (ipVersion == ADK_IPSEC_IPV6);
#if 0
                    hos_rule->precedence = precedence;
#endif
                    hos_rule->is_disabled = false;
                    hos_rule->spd_rule_id = polHdlIdx;

                    hos_rule->l4_info1 = sport_bitmask_p[sportloop].val;
                    hos_rule->l4_info1_mask = sport_bitmask_p[sportloop].mask;
                    
                    hos_rule->l4_info2 = dport_bitmask_p[dportloop].val;
                    hos_rule->l4_info2_mask = dport_bitmask_p[dportloop].mask;
                    
                    if (!(hos_rule->is_ipv6))
                    {
                        hos_rule->sip_addr[0] = adk_htonl(ipv4SIP[siploop].addr);
                        hos_rule->sip_addr_mask[0] = adk_htonl(ipv4SIP[siploop].mask);
                        hos_rule->dip_addr[0] = adk_htonl(ipv4DIP[diploop].addr);
                        hos_rule->dip_addr_mask[0] = adk_htonl(ipv4DIP[diploop].mask);
                    }
                    else
                    {
                        hos_rule->sip_addr[0] = adk_htonl(ipv6SIP[siploop].val[1] >> 32);
                        hos_rule->sip_addr_mask[0] = adk_htonl(ipv6SIP[siploop].mask[1] >> 32);
                        hos_rule->sip_addr[1] = adk_htonl(ipv6SIP[siploop].val[1] & 0xFFFFFFFF);
                        hos_rule->sip_addr_mask[1] = adk_htonl(ipv6SIP[siploop].mask[1] & 0xFFFFFFFF);
                        hos_rule->sip_addr[2] = adk_htonl(ipv6SIP[siploop].val[0] >> 32);
                        hos_rule->sip_addr_mask[2] = adk_htonl(ipv6SIP[siploop].mask[0] >> 32);
                        hos_rule->sip_addr[3] = adk_htonl(ipv6SIP[siploop].val[0] & 0xFFFFFFFF);
                        hos_rule->sip_addr_mask[3] = adk_htonl(ipv6SIP[siploop].mask[0] & 0xFFFFFFFF);

                        hos_rule->dip_addr[0] = adk_htonl(ipv6DIP[diploop].val[1] >> 32);
                        hos_rule->dip_addr_mask[0] = adk_htonl(ipv6DIP[diploop].mask[1] >> 32);
                        hos_rule->dip_addr[1] = adk_htonl(ipv6DIP[diploop].val[1] & 0xFFFFFFFF);
                        hos_rule->dip_addr_mask[1] = adk_htonl(ipv6DIP[diploop].mask[1] & 0xFFFFFFFF);
                        hos_rule->dip_addr[2] = adk_htonl(ipv6DIP[diploop].val[0] >> 32);
                        hos_rule->dip_addr_mask[2] = adk_htonl(ipv6DIP[diploop].mask[0] >> 32);
                        hos_rule->dip_addr[3] = adk_htonl(ipv6DIP[diploop].val[0] & 0xFFFFFFFF);
                        hos_rule->dip_addr_mask[3] = adk_htonl(ipv6DIP[diploop].mask[0] & 0xFFFFFFFF);
                    }

                    if(protocol)
                    {
                        hos_rule->proto = protocol;
                        hos_rule->proto_mask = HOS_IP_PROTO_MASK;
                    }

                    rule_count++;
                    hos_rule = rule_attr[rule_count]; //Move to next hos rule
                } /* Dport loop ends*/
            } /* Sport loop ends*/
        } /* Dip loop */
    } /* Sip loop */

    return;
}

adk_status_t
ipsec_spd_convert_to_hos_acls(adk_module_handle_t ipsecHandle,
                        adk_ipsec_sec_pol_t* secPol_p,
                        adk_ipsec_spd_rule_t *spdRule_p,
                        uint32_t polHdlIdx,
                        uint16_t *num_hos_rules_p,
                        adk_hos_spd_tbl_rule_attr_t **rule_attr)
{
    uint32_t sport_begin = 0, sport_end = 0xFFFF;
    uint32_t dport_begin = 0, dport_end = 0xFFFF;
    adk_acl_range_bitmask_t *sport_bitmask_p = NULL, *dport_bitmask_p = NULL;
    int sport_rule_count=1, dport_rule_count=1, sipRules=1, dipRules=1;
    int total_rule_count, loop;
    adk_status_t retValue= ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipsec_sec_pol_sel_t *secPolSel_p = &(secPol_p->secPolSel);
    adk_ipsec_ipv4_address_info_t   *ipv4SIP = NULL, *ipv4DIP = NULL;
    adk_acl_addr_range_bitmask_t   *ipv6SIP = NULL, *ipv6DIP = NULL;
    uint32_t  ipv4SIPStart, ipv4SIPEnd, ipv4DIPStart, ipv4DIPEnd;
    uint64_t  ipv6SIPStart[2], ipv6SIPEnd[2], ipv6DIPStart[2], ipv6DIPEnd[2];
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    if (secPolSel_p->protocol)
    {
        /* All Known protocol specific information is processed here */
        switch (secPolSel_p->protocol)
        {
        case PROTO_ICMP:
        case PROTO_ICMPV6:
                sport_begin = secPolSel_p->l4Info.icmpInfo.type.typeBegin;
                sport_end = secPolSel_p->l4Info.icmpInfo.type.typeEnd;
                dport_begin = secPolSel_p->l4Info.icmpInfo.code.codeBegin;
                dport_end = secPolSel_p->l4Info.icmpInfo.code.codeEnd;
            break;

        case PROTO_TCP:
        case PROTO_SCTP:
        case PROTO_UDP:
                /* Swap sport and dport */
                dport_begin = secPolSel_p->l4Info.portInfo.srcPort.portBegin;
                dport_end = secPolSel_p->l4Info.portInfo.srcPort.portEnd;
                sport_begin = secPolSel_p->l4Info.portInfo.dstPort.portBegin;
                sport_end = secPolSel_p->l4Info.portInfo.dstPort.portEnd;
            break;

        default:
            break;
        }               /* -----  end switch  ----- */
    }

    /*Allocate memory for storing bitmask and Range info*/
    sport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle),
         ADK_IPSEC_HOS_RANGE_MASK_OBJ,
         1,sizeof(adk_acl_range_bitmask_t )*
                        ADK_ACL_MAX_HOS_RULES_PER_RULE);
    if(!sport_bitmask_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                "Memory allocation failed\n");
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }

    sport_rule_count = adk_acl_convert_range_bitmask(
                        sport_begin,sport_end, 16, sport_bitmask_p);

    if(!sport_rule_count)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM, 
                            "SPort Range: End value is less than Begin value\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }
    else if(sport_rule_count == -1)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                "More SP rules than can be accommodated in HW!"
                                " Consider splitting range into multiple rules\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }

    /*Allocate memory for storing bitmask and Range info*/
    dport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle),
         ADK_IPSEC_HOS_RANGE_MASK_OBJ,
         1,sizeof(adk_acl_range_bitmask_t )*
                        ADK_ACL_MAX_HOS_RULES_PER_RULE);
    if(!dport_bitmask_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                "Memory allocation failed\n");
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }

    dport_rule_count = adk_acl_convert_range_bitmask(dport_begin, dport_end, 16, dport_bitmask_p);
    if(!dport_rule_count)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                            "DPort Range: End value is less than Begin Value\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }
    else if(dport_rule_count == -1)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                "More DP rules than can be accommodated in HW!"
                                " Consider splitting range into multiple rules\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }

    /* need to convert policy SIP and DIP to address/bitmask format
     */
    if (secPolSel_p->ipVersion == ADK_IPSEC_IPV4)
    {
        ipv4SIP = ADK_RAM_CALLOC(1, sizeof(adk_ipsec_ipv4_address_info_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv4SIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add SIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        ipv4DIP = ADK_RAM_CALLOC(1, sizeof(adk_ipsec_ipv4_address_info_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv4DIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add DIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        /* swap the SIP and DIP addresses. This table is used for policy
         */
        ipv4SIPStart = spdRule_p->dstAddr_Begin[0];
        ipv4SIPEnd =  spdRule_p->dstAddr_End[0];
        sipRules = convert_addr_range_to_bitmask(&ipv4SIPStart,
                                                 &ipv4SIPEnd,
                                                 ipv4SIP);
        if (sipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More SIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        ipv4DIPStart = spdRule_p->srcAddr_Begin[0];
        ipv4DIPEnd = spdRule_p->srcAddr_End[0];
        dipRules = convert_addr_range_to_bitmask(&ipv4DIPStart,
                                                 &ipv4DIPEnd,
                                                 ipv4DIP);
        if (dipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More DIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
    }
    else
    {
        ipv6SIP = ADK_RAM_CALLOC(1, sizeof(adk_acl_addr_range_bitmask_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv6SIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add SIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        ipv6DIP = ADK_RAM_CALLOC(1, sizeof(adk_acl_addr_range_bitmask_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv6DIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add DIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }

        /* swap the SIP and DIP addresses. This table is used for policy
         */
        ipv6SIPStart[1] = ((uint64_t) spdRule_p->dstAddr_Begin[0] << 32) | spdRule_p->dstAddr_Begin[1];
        ipv6SIPEnd[1] = ((uint64_t) spdRule_p->dstAddr_End[0] << 32) | spdRule_p->dstAddr_End[1];
        ipv6SIPStart[0] = ((uint64_t) spdRule_p->dstAddr_Begin[2] << 32) | spdRule_p->dstAddr_Begin[3];
        ipv6SIPEnd[0] = ((uint64_t) spdRule_p->dstAddr_End[2] << 32) | spdRule_p->dstAddr_End[3];

        sipRules = adk_acl_convert_addr_range_bitmask(ipv6SIPStart, ipv6SIPEnd,
                                                              128, ipv6SIP);

        if (sipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More SIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        else if (sipRules == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                                "Start address greater than end address !");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }


        /* swap the SIP and DIP addresses. This table is used for policy
         */
        ipv6DIPStart[1] = ((uint64_t) spdRule_p->srcAddr_Begin[0] << 32) | spdRule_p->srcAddr_Begin[1];
        ipv6DIPEnd[1] = ((uint64_t) spdRule_p->srcAddr_End[0] << 32) | spdRule_p->srcAddr_End[1];
        ipv6DIPStart[0] = ((uint64_t) spdRule_p->srcAddr_Begin[2] << 32) | spdRule_p->srcAddr_Begin[3];
        ipv6DIPEnd[0] = ((uint64_t) spdRule_p->srcAddr_End[2] << 32) | spdRule_p->srcAddr_End[3];
        dipRules = adk_acl_convert_addr_range_bitmask(ipv6DIPStart, ipv6DIPEnd,
                                                              128, ipv6DIP);

        if (dipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More DIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
        else if (dipRules == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                                "Start address greater than end address !");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
    }

    total_rule_count = sipRules * dipRules * sport_rule_count * dport_rule_count;
 
    ADK_IPSEC_LOG_INFO(ipsecHandle, "sip_rule_count: %d, dip_rule_count: %d "
                        "sport_rule_count: %d, dport_rule_count: %d \n", sipRules, dipRules,
                        sport_rule_count, dport_rule_count);

    if(total_rule_count > ADK_ACL_MAX_HOS_RULES_PER_RULE)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM, 
                        "Rule splitting crossing limits for hos rule count and so, the rule can't be loaded into HW"
                                        " Consider splitting range into multiple rules\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_convert_to_hos_acls_exit;
    }

    for(loop =0; loop< total_rule_count; loop++)
    {
        rule_attr[loop] = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle), ADK_IPSEC_HOS_RULE_OBJ,
				 1, sizeof(adk_hos_spd_tbl_rule_attr_t));
        if(!rule_attr[loop])
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_convert_to_hos_acls_exit;
        }
    }

    *num_hos_rules_p = total_rule_count;  //Update hos Rule count

    ipsec_build_hos_rule(secPol_p->secPolSel.ipVersion, secPol_p->secPolSel.protocol,
                    ipv4SIP, ipv6SIP, sipRules,
                    ipv4DIP, ipv6DIP, dipRules, sport_bitmask_p,
                    sport_rule_count, dport_bitmask_p,
                    dport_rule_count, polHdlIdx, rule_attr);

_ipsec_spd_convert_to_hos_acls_exit:
    if (ipv4SIP)
        ADK_RAM_FREE(ipv4SIP);
    if (ipv4DIP)
        ADK_RAM_FREE(ipv4DIP);
    if (ipv6SIP)
        ADK_RAM_FREE(ipv6SIP);
    if (ipv6DIP)
        ADK_RAM_FREE(ipv6DIP);
    if(dport_bitmask_p)
        ADK_NVM_FREE(dport_bitmask_p);
    if(sport_bitmask_p)
        ADK_NVM_FREE(sport_bitmask_p);
    return retValue;
}

adk_status_t
ipsec_spd_in_convert_to_hos_acls(adk_module_handle_t ipsecHandle,
                        adk_ipsec_sec_pol_in_t *secPol_p,
                        adk_ipsec_spd_in_rule_t *spdRule_p,
                        uint32_t polHdlIdx,
                        uint16_t *num_hos_rules_p,
                        adk_hos_spd_tbl_rule_attr_t **rule_attr)
{
    uint32_t sport_begin = 0, sport_end = 0xFFFF;
    uint32_t dport_begin = 0, dport_end = 0xFFFF;
    adk_acl_range_bitmask_t *sport_bitmask_p = NULL, *dport_bitmask_p = NULL;
    int sport_rule_count=1, dport_rule_count=1, sipRules=1, dipRules=1;
    int total_rule_count, loop;
    adk_status_t retValue= ADK_SUCCESS;
    adk_sys_handle_t sysHandle;
    adk_ipsec_sec_pol_in_sel_t *secPolSel_p = &(secPol_p->secPolSel);
    adk_ipsec_ipv4_address_info_t   *ipv4SIP = NULL, *ipv4DIP = NULL;
    adk_acl_addr_range_bitmask_t   *ipv6SIP = NULL, *ipv6DIP = NULL;
    uint32_t  ipv4SIPStart, ipv4SIPEnd, ipv4DIPStart, ipv4DIPEnd;
    uint64_t  ipv6SIPStart[2], ipv6SIPEnd[2], ipv6DIPStart[2], ipv6DIPEnd[2];
    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    if (secPolSel_p->protocol)
    {
        /* All Known protocol specific information is processed here */
        switch (secPolSel_p->protocol)
        {
        case PROTO_ICMP:
        case PROTO_ICMPV6:
                sport_begin = secPolSel_p->l4Info.icmpInfo.type.typeBegin;
                sport_end = secPolSel_p->l4Info.icmpInfo.type.typeEnd;
                dport_begin = secPolSel_p->l4Info.icmpInfo.code.codeBegin;
                dport_end = secPolSel_p->l4Info.icmpInfo.code.codeEnd;
            break;

        case PROTO_TCP:
        case PROTO_SCTP:
        case PROTO_UDP:
                sport_begin = secPolSel_p->l4Info.portInfo.srcPort.portBegin;
                sport_end = secPolSel_p->l4Info.portInfo.srcPort.portEnd;
                dport_begin = secPolSel_p->l4Info.portInfo.dstPort.portBegin;
                dport_end = secPolSel_p->l4Info.portInfo.dstPort.portEnd;
            break;

        default:
            break;
        }               /* -----  end switch  ----- */
    }

    /*Allocate memory for storing bitmask and Range info*/
    sport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle),
         ADK_IPSEC_HOS_RANGE_MASK_OBJ,
         1,sizeof(adk_acl_range_bitmask_t )*
                        ADK_ACL_MAX_HOS_RULES_PER_RULE);
    if(!sport_bitmask_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                "Memory allocation failed\n");
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }

    sport_rule_count = adk_acl_convert_range_bitmask(
                        sport_begin,sport_end, 16, sport_bitmask_p);

    if(!sport_rule_count)
    {
       ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM, 
                           "SPort Range: End value is less than Begin value\n");
       retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
       goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }
    else if(sport_rule_count == -1)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More SP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }

    /*Allocate memory for storing bitmask and Range info*/
    dport_bitmask_p = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle),
         ADK_IPSEC_HOS_RANGE_MASK_OBJ,
         1,sizeof(adk_acl_range_bitmask_t )*
                        ADK_ACL_MAX_HOS_RULES_PER_RULE);
    if(!dport_bitmask_p)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                "Memory allocation failed\n");
        retValue = ADK_E_MEM_UNAVAILABLE;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }

    dport_rule_count = adk_acl_convert_range_bitmask(dport_begin, dport_end, 16, dport_bitmask_p);
    if(!dport_rule_count)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                            "DPort Range: End value is less than Begin Value\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }
    else if(dport_rule_count == -1)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                "More DP rules than can be accommodated in HW!"
                                " Consider splitting range into multiple rules\n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }

    /* need to convert policy SIP and DIP to address/bitmask format
     */
    if (secPolSel_p->ipVersion == ADK_IPSEC_IPV4)
    {
        ipv4SIP = ADK_RAM_CALLOC(1, sizeof(adk_ipsec_ipv4_address_info_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv4SIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add SIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        ipv4DIP = ADK_RAM_CALLOC(1, sizeof(adk_ipsec_ipv4_address_info_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv4DIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add DIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        ipv4SIPStart = spdRule_p->srcAddr_Begin[0];
        ipv4SIPEnd =  spdRule_p->srcAddr_End[0];
        sipRules = convert_addr_range_to_bitmask(&ipv4SIPStart,
                                                 &ipv4SIPEnd,
                                                 ipv4SIP);
        if (sipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More SIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        ipv4DIPStart = spdRule_p->dstAddr_Begin[0];
        ipv4DIPEnd = spdRule_p->dstAddr_End[0];
        dipRules = convert_addr_range_to_bitmask(&ipv4DIPStart,
                                                 &ipv4DIPEnd,
                                                 ipv4DIP);
        if (dipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More DIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
    }
    else
    {
        ipv6SIP = ADK_RAM_CALLOC(1, sizeof(adk_acl_addr_range_bitmask_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv6SIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add SIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        ipv6DIP = ADK_RAM_CALLOC(1, sizeof(adk_acl_addr_range_bitmask_t) * ADK_ACL_MAX_HOS_RULES_PER_RULE);
        if (NULL == ipv6DIP)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                "Could not allocate memory to add DIP rules!");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }

        ipv6SIPStart[1] = ((uint64_t) spdRule_p->srcAddr_Begin[0] << 32) | spdRule_p->srcAddr_Begin[1];
        ipv6SIPEnd[1] = ((uint64_t) spdRule_p->srcAddr_End[0] << 32) | spdRule_p->srcAddr_End[1];
        ipv6SIPStart[0] = ((uint64_t) spdRule_p->srcAddr_Begin[2] << 32) | spdRule_p->srcAddr_Begin[3];
        ipv6SIPEnd[0] = ((uint64_t) spdRule_p->srcAddr_End[2] << 32) | spdRule_p->srcAddr_End[3];

        sipRules = adk_acl_convert_addr_range_bitmask(ipv6SIPStart, ipv6SIPEnd,
                                                              128, ipv6SIP);

        if (sipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More SIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        else if (sipRules == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                                "Start address greater than end address !");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }


        ipv6DIPStart[1] = ((uint64_t) spdRule_p->dstAddr_Begin[0] << 32) | spdRule_p->dstAddr_Begin[1];
        ipv6DIPEnd[1] = ((uint64_t) spdRule_p->dstAddr_End[0] << 32) | spdRule_p->dstAddr_End[1];
        ipv6DIPStart[0] = ((uint64_t) spdRule_p->dstAddr_Begin[2] << 32) | spdRule_p->dstAddr_Begin[3];
        ipv6DIPEnd[0] = ((uint64_t) spdRule_p->dstAddr_End[2] << 32) | spdRule_p->dstAddr_End[3];
        dipRules = adk_acl_convert_addr_range_bitmask(ipv6DIPStart, ipv6DIPEnd,
                                                              128, ipv6DIP);

        if (dipRules == -1)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                                    "More DIP rules than can be accommodated in HW!"
                                    " Consider splitting range into multiple rules\n");
            retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
        else if (dipRules == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                                "Start address greater than end address !");
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
    }

    total_rule_count = sipRules * dipRules * sport_rule_count * dport_rule_count;
 
    ADK_IPSEC_LOG_INFO(ipsecHandle, "sip_rule_count: %d, dip_rule_count: %d "
                        "sport_rule_count: %d, dport_rule_count: %d \n", sipRules, dipRules,
                        sport_rule_count, dport_rule_count);

    if(total_rule_count > ADK_ACL_MAX_HOS_RULES_PER_RULE)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM, 
                      "Rule splitting crossing limits for hos rule count and so, the rule can't be loaded into HW"
                      " Consider splitting range into multiple rules \n");
        retValue = ADK_E_IPSEC_INVALID_INPUT_PARAM;
        goto _ipsec_spd_in_convert_to_hos_acls_exit;
    }

    for(loop =0; loop< total_rule_count; loop++)
    {
        rule_attr[loop] = ADK_NVM_CALLOC(ADK_MODULE_MODULEID_GET(ipsecHandle), ADK_IPSEC_HOS_RULE_OBJ,
				 1, sizeof(adk_hos_spd_tbl_rule_attr_t));
        if(!rule_attr[loop])
        {
            retValue = ADK_E_MEM_UNAVAILABLE;
            goto _ipsec_spd_in_convert_to_hos_acls_exit;
        }
    }

    *num_hos_rules_p = total_rule_count;  //Update hos Rule count

    ipsec_build_hos_rule(secPol_p->secPolSel.ipVersion, secPol_p->secPolSel.protocol,
                    ipv4SIP, ipv6SIP, sipRules,
                    ipv4DIP, ipv6DIP, dipRules, sport_bitmask_p,
                    sport_rule_count, dport_bitmask_p,
                    dport_rule_count, polHdlIdx, rule_attr);

_ipsec_spd_in_convert_to_hos_acls_exit:
    if (ipv4SIP)
        ADK_RAM_FREE(ipv4SIP);
    if (ipv4DIP)
        ADK_RAM_FREE(ipv4DIP);
    if (ipv6SIP)
        ADK_RAM_FREE(ipv6SIP);
    if (ipv6DIP)
        ADK_RAM_FREE(ipv6DIP);
    if(dport_bitmask_p)
        ADK_NVM_FREE(dport_bitmask_p);
    if(sport_bitmask_p)
        ADK_NVM_FREE(sport_bitmask_p);
    return retValue;
}

void
ipsec_spd_convert_to_dpdk_acl (adk_module_handle_t ipsecHandle,
                        adk_ipsec_spd_rule_t *spdRulePtn,
                        uint32_t policyHdl,
                        struct adk_ipsec_acl_rule *dpdkAclRule)
{
    adk_sys_handle_t sysHandle;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    ADK_MEMSET(dpdkAclRule, 0, sizeof(struct adk_ipsec_acl_rule));

    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, VERSION - 0x%x\n", spdRulePtn->version);
    if (ADK_IPSEC_IPV4 == spdRulePtn->version)
    {
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, SIP_BEGIN - 0x%x\n", spdRulePtn->srcAddr_Begin[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, SIP_END - 0x%x\n", spdRulePtn->srcAddr_End[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, DIP_BEGIN - 0x%x\n", spdRulePtn->dstAddr_Begin[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, DIP_END - 0x%x\n", spdRulePtn->dstAddr_End[0]);
    }
	else
    {
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, SIP_BEGIN - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn->srcAddr_Begin[0],
                      spdRulePtn->srcAddr_Begin[1], spdRulePtn->srcAddr_Begin[2], spdRulePtn->srcAddr_Begin[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, SIP_END - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn->srcAddr_End[0],
                       spdRulePtn->srcAddr_End[1], spdRulePtn->srcAddr_End[2], spdRulePtn->srcAddr_End[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, DIP_BEGIN - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn->dstAddr_Begin[0],
                       spdRulePtn->dstAddr_Begin[1], spdRulePtn->dstAddr_Begin[2], spdRulePtn->dstAddr_Begin[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD ADD, DIP_END - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn->dstAddr_End[0],
	                   spdRulePtn->dstAddr_End[1], spdRulePtn->dstAddr_End[2], spdRulePtn->dstAddr_End[3]);
	}
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, DSCPVAL - 0x%x\n", spdRulePtn->dscpVal_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, DSCPVAL - 0x%x\n", spdRulePtn->dscpVal_End);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, PROTO_BEGIN - 0x%x\n", spdRulePtn->proto_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, PROTO_END - 0x%x\n", spdRulePtn->proto_End);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, SP_BEGIN - 0x%x\n", spdRulePtn->srcPort_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, SP_END - 0x%x\n", spdRulePtn->srcPort_End);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, DP_BEGIN - 0x%x\n", spdRulePtn->dstPort_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD ADD, DP_END - 0x%x\n", spdRulePtn->dstPort_End);
    /* now convert this into DPDK ACL specific DS */
    create_spd_dpdk_acl_pattern(spdRulePtn, dpdkAclRule);

    dpdkAclRule->data.category_mask = 1;
    dpdkAclRule->data.userdata = policyHdl;

    return;
}

void
ipsec_spd_in_convert_to_dpdk_acl (adk_module_handle_t ipsecHandle,
                        adk_ipsec_spd_in_rule_t *spdRulePtn_p,
                        uint32_t policyHdl,
                        struct adk_ipsec_in_acl_rule *dpdkAclRule)
{
    struct adk_ipsec_in_acl_rule aclRule;
    adk_sys_handle_t sysHandle;

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);

    ADK_MEMSET(&aclRule, 0, sizeof(struct adk_ipsec_in_acl_rule));

    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, VERSION - 0x%x\n", spdRulePtn_p->version);
    if (ADK_IPSEC_IPV4 == spdRulePtn_p->version)
    {
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, SIP_BEGIN - 0x%x\n", spdRulePtn_p->srcAddr_Begin[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, SIP_END - 0x%x\n", spdRulePtn_p->srcAddr_End[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, DIP_BEGIN - 0x%x\n", spdRulePtn_p->dstAddr_Begin[0]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, DIP_END - 0x%x\n", spdRulePtn_p->dstAddr_End[0]);
    }
	else
    {
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, SIP_BEGIN - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn_p->srcAddr_Begin[0],
                      spdRulePtn_p->srcAddr_Begin[1], spdRulePtn_p->srcAddr_Begin[2], spdRulePtn_p->srcAddr_Begin[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, SIP_END - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn_p->srcAddr_End[0],
                       spdRulePtn_p->srcAddr_End[1], spdRulePtn_p->srcAddr_End[2], spdRulePtn_p->srcAddr_End[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, DIP_BEGIN - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn_p->dstAddr_Begin[0],
                       spdRulePtn_p->dstAddr_Begin[1], spdRulePtn_p->dstAddr_Begin[2], spdRulePtn_p->dstAddr_Begin[3]);
        ADK_IPSEC_LOG_INFO (ipsecHandle,
                      "SPD IN ADD, DIP_END - 0x%x::0x%x::0x%x::0x%x\n", spdRulePtn_p->dstAddr_End[0],
	                   spdRulePtn_p->dstAddr_End[1], spdRulePtn_p->dstAddr_End[2], spdRulePtn_p->dstAddr_End[3]);
	}
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, PROTO_BEGIN - 0x%x\n", spdRulePtn_p->proto_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, PROTO_END - 0x%x\n", spdRulePtn_p->proto_End);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, SP_BEGIN - 0x%x\n", spdRulePtn_p->srcPort_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, SP_END - 0x%x\n", spdRulePtn_p->srcPort_End);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, DP_BEGIN - 0x%x\n", spdRulePtn_p->dstPort_Begin);
    ADK_IPSEC_LOG_INFO (ipsecHandle,
                  "SPD IN ADD, DP_END - 0x%x\n", spdRulePtn_p->dstPort_End);
    /* now convert this into DPDK ACL specific DS */
    create_spd_in_dpdk_acl_pattern(spdRulePtn_p, dpdkAclRule);

    dpdkAclRule->data.category_mask = 1;
    dpdkAclRule->data.userdata = policyHdl;

    return;
}

adk_status_t
adk_ipsec_get_cp_spd_rule_idx(ipsec_module_info_t *ipsecModuleInfo_p,
                              adk_ipsec_sec_pol_sel_lkup_t *secPolSelLkup_p,
                              uint16_t numOfRules, uint16_t *polIdx,
                              ipsec_sec_pol_info_t** ipsecSecPol)
{
    struct adk_ipsec_acl_rule *aclRule;
    uint8_t i;
    ipsec_sec_pol_info_t *ipsecSecPolLocal = NULL;
    adk_acl_rule_t cpRule;
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    adk_acl_get_reset(ipsecModuleInfo_p->cpSpdAclList);
    for (i = 0; i < numOfRules; i++)
    {
         retValue = adk_acl_next_rule_get(ipsecModuleInfo_p->cpSpdAclList, &cpRule);
         if (ADK_SUCCESS != retValue)
         {
             return ADK_E_IPSEC_POLICY_NOT_FOUND;
         }
         ipsecSecPolLocal = (ipsec_sec_pol_info_t*)(cpRule.user_data);
         aclRule = &(ipsecSecPolLocal->dpdkAclRule);

         if (!((secPolSelLkup_p->protocol >= aclRule->field[PROTO_FIELD_IPSEC].value.u16) &&
             (secPolSelLkup_p->protocol <= aclRule->field[PROTO_FIELD_IPSEC].mask_range.u16)))
             continue;
         if (!((secPolSelLkup_p->dscpVal >= aclRule->field[DSCP_FIELD_IPSEC].value.u16) &&
             (secPolSelLkup_p->dscpVal <= aclRule->field[DSCP_FIELD_IPSEC].mask_range.u16)))
             continue;
         if ((secPolSelLkup_p->protocol == IPPROTO_ICMP) || (secPolSelLkup_p->protocol == IPPROTO_ICMPV6))
         {
             if (!((secPolSelLkup_p->l4Info.icmpInfo.type >= (uint8_t)ntohs(aclRule->field[SP_FIELD_IPSEC].value.u16)) &&
                 (secPolSelLkup_p->l4Info.icmpInfo.type <= (uint8_t)ntohs(aclRule->field[SP_FIELD_IPSEC].mask_range.u16))))
                 continue;
             if (!((secPolSelLkup_p->l4Info.icmpInfo.code >= (uint8_t)ntohs(aclRule->field[DP_FIELD_IPSEC].value.u16)) &&
                 (secPolSelLkup_p->l4Info.icmpInfo.code <= (uint8_t)ntohs(aclRule->field[DP_FIELD_IPSEC].mask_range.u16))))
                 continue;
         }
         else
         {
             if (!((secPolSelLkup_p->l4Info.portInfo.srcPort >= aclRule->field[SP_FIELD_IPSEC].value.u16) &&
                 (secPolSelLkup_p->l4Info.portInfo.srcPort <= aclRule->field[SP_FIELD_IPSEC].mask_range.u16)))
                 continue;
             if (!((secPolSelLkup_p->l4Info.portInfo.dstPort >= aclRule->field[DP_FIELD_IPSEC].value.u16) &&
                 (secPolSelLkup_p->l4Info.portInfo.dstPort <= aclRule->field[DP_FIELD_IPSEC].mask_range.u16)))
                 continue;
         }
         if ((ADK_IPSEC_IPV4 == secPolSelLkup_p->ipVersion) &&
             (ADK_IPSEC_IPV4 == aclRule->field[VERSION_FIELD_IPSEC].value.u8))
         {
             /* Checking SIP */
             if(!((secPolSelLkup_p->ipAddr.ipv4Addr.srcAddr >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv4Addr.srcAddr <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32)))
                continue;
             /* Checking DIP */
             if(!((secPolSelLkup_p->ipAddr.ipv4Addr.dstAddr >=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv4Addr.dstAddr <=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32)))
                continue;
             *polIdx = i;
             *ipsecSecPol = ipsecSecPolLocal;
             return ADK_IPSEC_SUCCESS;	
         }
         if ((ADK_IPSEC_IPV6 == secPolSelLkup_p->ipVersion) &&
             (ADK_IPSEC_IPV6 == aclRule->field[VERSION_FIELD_IPSEC].value.u8))
         {
             /* checking SIP */
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[0] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[0] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[1] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[1] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[2] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[2] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[3] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[3] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32)))
                continue;
              
             /* checking DIP */
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[0] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[0] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[1] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[1] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[2] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[2] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[3] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[3] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32)))
                continue;
             *polIdx = i;
             *ipsecSecPol = ipsecSecPolLocal;
             return ADK_IPSEC_SUCCESS;	
         }
    }
    return ADK_E_IPSEC_POLICY_NOT_FOUND;
}

adk_status_t
adk_ipsec_get_cp_spd_in_rule_idx(ipsec_module_info_t *ipsecModuleInfo_p,
                                 adk_ipsec_sec_pol_sel_lkup_t *secPolSelLkup_p,
                                 uint16_t numOfRules, uint16_t *polIdx,
                                 ipsec_sec_pol_in_info_t** ipsecSecPolIn)
{
    struct adk_ipsec_in_acl_rule *aclRule;
    uint8_t i;
    ipsec_sec_pol_in_info_t *ipsecSecPolInLocal = NULL;
    adk_acl_rule_t cpRule;
    adk_status_t retValue = ADK_IPSEC_SUCCESS;
    adk_acl_get_reset(ipsecModuleInfo_p->cpSpdInAclList);
    for (i = 0; i < numOfRules; i++)
    {
         retValue = adk_acl_next_rule_get(ipsecModuleInfo_p->cpSpdInAclList, &cpRule);
         if (ADK_SUCCESS != retValue)
         {
             return ADK_E_IPSEC_POLICY_NOT_FOUND;
         }
         ipsecSecPolInLocal = (ipsec_sec_pol_in_info_t*)(cpRule.user_data);
         aclRule = &(ipsecSecPolInLocal->dpdkAclInRule);

         if (!((secPolSelLkup_p->protocol >= aclRule->field[PROTO_FIELD_IPSEC_IN].value.u32) &&
             (secPolSelLkup_p->protocol <= aclRule->field[PROTO_FIELD_IPSEC_IN].mask_range.u32)))
             continue;
         if ((secPolSelLkup_p->protocol == IPPROTO_ICMP) || (secPolSelLkup_p->protocol == IPPROTO_ICMPV6))
         {
             if (!((secPolSelLkup_p->l4Info.icmpInfo.type >= (uint8_t)ntohs(aclRule->field[SP_FIELD_IPSEC_IN].value.u16)) &&
                 (secPolSelLkup_p->l4Info.icmpInfo.type <= (uint8_t)ntohs(aclRule->field[SP_FIELD_IPSEC_IN].mask_range.u16))))
                 continue;
             if (!((secPolSelLkup_p->l4Info.icmpInfo.code >= (uint8_t)ntohs(aclRule->field[DP_FIELD_IPSEC_IN].value.u16)) &&
                 (secPolSelLkup_p->l4Info.icmpInfo.code <= (uint8_t)ntohs(aclRule->field[DP_FIELD_IPSEC_IN].mask_range.u16))))
                 continue;
         }
         else
         {
             if (!((secPolSelLkup_p->l4Info.portInfo.srcPort >= aclRule->field[SP_FIELD_IPSEC_IN].value.u16) &&
                 (secPolSelLkup_p->l4Info.portInfo.srcPort <= aclRule->field[SP_FIELD_IPSEC_IN].mask_range.u16)))
                 continue;
             if (!((secPolSelLkup_p->l4Info.portInfo.dstPort >= aclRule->field[DP_FIELD_IPSEC_IN].value.u16) &&
                 (secPolSelLkup_p->l4Info.portInfo.dstPort <= aclRule->field[DP_FIELD_IPSEC_IN].mask_range.u16)))
                 continue;
         }
         if ((ADK_IPSEC_IPV4 == secPolSelLkup_p->ipVersion) &&
             (ADK_IPSEC_IPV4 == aclRule->field[VERSION_FIELD_IPSEC_IN].value.u8))
         {
             /* Checking SIP */
             if(!((secPolSelLkup_p->ipAddr.ipv4Addr.srcAddr >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv4Addr.srcAddr <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32)))
                continue;
             /* Checking DIP */
             if(!((secPolSelLkup_p->ipAddr.ipv4Addr.dstAddr >=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv4Addr.dstAddr <=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32)))
                continue;
             *polIdx = i;
             *ipsecSecPolIn = ipsecSecPolInLocal;
             return ADK_IPSEC_SUCCESS;	
         }
         if ((ADK_IPSEC_IPV6 == secPolSelLkup_p->ipVersion) &&
             (ADK_IPSEC_IPV6 == aclRule->field[VERSION_FIELD_IPSEC_IN].value.u8))
         {
             /* checking SIP */
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[0] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[0] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[1] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[1] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[2] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[2] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[3] >=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.srcAddr[3] <=
                aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32)))
                continue;
              
             /* checking DIP */
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[0] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[0] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[1] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[1] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[2] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[2] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32)))
                continue;
             if(!((secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[3] >=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32) &&
                (secPolSelLkup_p->ipAddr.ipv6Addr.dstAddr[3] <=
                aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32)))
                continue;
             *polIdx = i;
             *ipsecSecPolIn = ipsecSecPolInLocal;
             return ADK_IPSEC_SUCCESS;	
         }
    }
    return ADK_E_IPSEC_POLICY_NOT_FOUND;
}

void adk_ipsec_get_spd_rule(adk_ipsec_sec_pol_sel_t *secPolSel,
                            struct adk_ipsec_acl_rule *aclRule)
{
    /* version */
    secPolSel->ipVersion = aclRule->field[VERSION_FIELD_IPSEC].value.u8;

    if (secPolSel->ipVersion == ADK_IPSEC_IPV4)
    {
        /* SIP */
        secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin =
            aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd =
            aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32;
        /* DIP */
        secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin =
            aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd =
            aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32;
        secPolSel->ipAddr.ipv4Addr.srcAddr.rangeSelFlag = TRUE;
        secPolSel->ipAddr.ipv4Addr.dstAddr.rangeSelFlag = TRUE;
    }
    else
    {
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[0] =
            aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[0] =
            aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[1] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[1] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[2] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[2] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[3] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[3] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32;

        /* DIP */
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[0] =
            aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[0] =
            aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_0].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[1] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[1] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_1].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[2] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[2] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_2].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[3] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[3] =
           aclRule->field[DST_IP_FIELD_IPSEC_IPV4_IPV6_3].mask_range.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.rangeSelFlag = TRUE;
        secPolSel->ipAddr.ipv6Addr.dstAddr.rangeSelFlag = TRUE;
    }

    /* DSCP value */
    secPolSel->dscpVal = aclRule->field[DSCP_FIELD_IPSEC].value.u16;

    /* protocol */
    secPolSel->protocol = aclRule->field[PROTO_FIELD_IPSEC].value.u16;

    if((secPolSel->protocol == PROTO_ICMP) || (secPolSel->protocol == PROTO_ICMPV6))
    {
        /* SP */
        secPolSel->l4Info.icmpInfo.type.typeBegin = ntohs(aclRule->field[SP_FIELD_IPSEC].value.u16);
        secPolSel->l4Info.icmpInfo.type.typeEnd = ntohs(aclRule->field[SP_FIELD_IPSEC].mask_range.u16);

        /* DP */
        secPolSel->l4Info.icmpInfo.code.codeBegin = ntohs(aclRule->field[DP_FIELD_IPSEC].value.u16);
        secPolSel->l4Info.icmpInfo.code.codeEnd = ntohs(aclRule->field[DP_FIELD_IPSEC].mask_range.u16);
    }
    else
    {
        /* SP */
        secPolSel->l4Info.portInfo.srcPort.portBegin = aclRule->field[SP_FIELD_IPSEC].value.u16;
        secPolSel->l4Info.portInfo.srcPort.portEnd = aclRule->field[SP_FIELD_IPSEC].mask_range.u16;

        /* DP */
        secPolSel->l4Info.portInfo.dstPort.portBegin = aclRule->field[DP_FIELD_IPSEC].value.u16;
        secPolSel->l4Info.portInfo.dstPort.portEnd = aclRule->field[DP_FIELD_IPSEC].mask_range.u16;
    }
}

void adk_ipsec_get_spd_in_rule(adk_ipsec_sec_pol_in_sel_t *secPolSel,
                               struct adk_ipsec_in_acl_rule *aclRule)
{
    /* version */
    secPolSel->ipVersion = aclRule->field[VERSION_FIELD_IPSEC_IN].value.u8;

    if (secPolSel->ipVersion == ADK_IPSEC_IPV4)
    {
        /* SIP */
        secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrBegin =
            aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv4Addr.srcAddr.addr.addr_range_t.addrEnd =
            aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32;
        /* DIP */
        secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrBegin =
            aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv4Addr.dstAddr.addr.addr_range_t.addrEnd =
            aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32;
        secPolSel->ipAddr.ipv4Addr.srcAddr.rangeSelFlag = TRUE;
        secPolSel->ipAddr.ipv4Addr.dstAddr.rangeSelFlag = TRUE;
    }
    else
    {
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[0] =
            aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[0] =
            aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[1] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[1] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[2] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[2] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrBegin[3] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.addr.addr_range_t.addrEnd[3] =
          aclRule->field[SRC_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32;

        /* DIP */
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[0] =
            aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[0] =
            aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_0].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[1] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[1] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_1].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[2] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[2] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_2].mask_range.u32;

        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrBegin[3] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].value.u32;
        secPolSel->ipAddr.ipv6Addr.dstAddr.addr.addr_range_t.addrEnd[3] =
           aclRule->field[DST_IP_FIELD_IPSEC_IN_IPV4_IPV6_3].mask_range.u32;
        secPolSel->ipAddr.ipv6Addr.srcAddr.rangeSelFlag = TRUE;
        secPolSel->ipAddr.ipv6Addr.dstAddr.rangeSelFlag = TRUE;
    }

    /* protocol */
    secPolSel->protocol = aclRule->field[PROTO_FIELD_IPSEC_IN].value.u32;

    if((secPolSel->protocol == PROTO_ICMP) || (secPolSel->protocol == PROTO_ICMPV6))
    {
        /* SP */
        secPolSel->l4Info.icmpInfo.type.typeBegin = ntohs(aclRule->field[SP_FIELD_IPSEC_IN].value.u16);
        secPolSel->l4Info.icmpInfo.type.typeEnd = ntohs(aclRule->field[SP_FIELD_IPSEC_IN].mask_range.u16);

        /* DP */
        secPolSel->l4Info.icmpInfo.code.codeBegin = ntohs(aclRule->field[DP_FIELD_IPSEC_IN].value.u16);
        secPolSel->l4Info.icmpInfo.code.codeEnd = ntohs(aclRule->field[DP_FIELD_IPSEC_IN].mask_range.u16);
    }
    else
    {
        /* SP */
        secPolSel->l4Info.portInfo.srcPort.portBegin = aclRule->field[SP_FIELD_IPSEC_IN].value.u16;
        secPolSel->l4Info.portInfo.srcPort.portEnd = aclRule->field[SP_FIELD_IPSEC_IN].mask_range.u16;

        /* DP */
        secPolSel->l4Info.portInfo.dstPort.portBegin = aclRule->field[DP_FIELD_IPSEC_IN].value.u16;
        secPolSel->l4Info.portInfo.dstPort.portEnd = aclRule->field[DP_FIELD_IPSEC_IN].mask_range.u16;
    }
}

adk_status_t adk_ipsec_delete_stale_entries (adk_sys_handle_t sysHandle,
                                     adk_module_handle_t ipsecHandle,
                                     ipsec_module_info_t *ipsecModuleInfo_p)
{
    adk_status_t retValue = ADK_SUCCESS;
    uint16_t saIndex;
    uint32_t totInFlightCnt;
    int i;
    adk_sys_egress_stats_t egStats;
    uint64_t  currentCycles = 0, diffCycles = 0, timeElapsed = 0;

    ADK_MEMSET (&egStats, 0x00, sizeof(adk_sys_egress_stats_t));

    for (i = 0; i < ipsecModuleInfo_p->numStaleEntries; i++)
    {
        /*  Check for MSB of SA ID in stale entry DB.
         *  If MSB = 0, the SA is outbound and if MSB = 1,
         *  the SA is inbound.
         *
         *  To get the actual SAID value without the MSB set for direction,
         *  nullify the MSB. This will be needed for indexing
         *  into 'iSATbl'/'eSATbl'.
         */

        saIndex = ipsecModuleInfo_p->staleSaEntries[i];
        saIndex &= ~(1 << ADK_IPSEC_SA_DIR_SHIFT);
        if ((ipsecModuleInfo_p->staleSaEntries[i] >> ADK_IPSEC_SA_DIR_SHIFT) &
                    ADK_IPSEC_INBOUND)
        {
            if (ipsecModuleInfo_p->dpTbl->iSATbl[saIndex].inFlightCnt == 0)
            {
                retValue = adk_ipsec_del_iSA (sysHandle, ipsecHandle, ipsecModuleInfo_p,
                                              saIndex);
                if (ADK_SUCCESS != retValue)
                {
                    ADK_IPSEC_LOG_ERROR (ipsecHandle, retValue,
                                         "Internal Error while deleting ingress SA\n");
                    return retValue;
                }
                ipsecModuleInfo_p->numIngSAEntries--;
                /*  Use the second MSB to indicate if stale SA can be deleted */
                ipsecModuleInfo_p->staleSaEntries[i] |= ADK_IPSEC_DELETED_SA_MASK;
            }
        }
        else
        {
            /*  Delete stale entry if inflight count is 0. Else, do nothing
             *  and wait for next add/update for deletion opportunity
             */

            if (ipsecModuleInfo_p->dpTbl->eSATbl[saIndex].saMode != ADK_IPSEC_SA_LOOKASIDE)
            {
                retValue = adk_sys_egress_stats_get (sysHandle,
                                ipsecModuleInfo_p->dpTbl->eSATbl[saIndex].egressStatsId,
                                &egStats);
                if (retValue != ADK_SUCCESS)
                {
                    retValue = ADK_ERROR_SEV_LEVEL_SET(ADK_E_INTERNAL_ERROR);
                    ADK_IPSEC_LOG_ERROR (ipsecHandle, retValue,
                                         "Error in reading egress stats for SA\n");
                    return retValue;
                }

                /*  The inflight count stored in the DP table accounts for both
                 *  lookaside and inline inflight counts, which makes it
                 *  equivalent to the overall Tx count stats at the SA level.
                 *  Thus, totInFlightCnt will be a reflection of in flight
                 *  packets remaining across inline and lookaside paths. */
                totInFlightCnt = ipsecModuleInfo_p->dpTbl->eSATbl[saIndex].inFlightCnt -
                                    (egStats.pkt_xmits + egStats.pkt_drops);
            }
            else
            {
                totInFlightCnt = ipsecModuleInfo_p->dpTbl->eSATbl[saIndex].inFlightCnt;
            }
            currentCycles = rte_rdtsc();
            diffCycles = currentCycles - egStats.timestamp;
            ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_get_timer_hz", "Get "
                                "number of cycles converted to seconds\n");
            timeElapsed = diffCycles/rte_get_timer_hz();
            if (totInFlightCnt == 0)
            {
                /* we need to protect against a worst case where SA gets deleted
                 * even when CPM is processing packets for it! and also for cases
                 * where for some reason the timestamp in the egStats is 0!
                 */
                if ( (currentCycles == diffCycles) ||
                     (timeElapsed/1000 >= ADK_IPSEC_CPM_LATENCY ))
                {
                    adk_ipsec_del_eSA (sysHandle, ipsecHandle, ipsecModuleInfo_p,
                                       saIndex);
                    /*  Use the second MSB to indicate if stale SA can be deleted */
                    ipsecModuleInfo_p->staleSaEntries[i] |= ADK_IPSEC_DELETED_SA_MASK;
                }
            }
            else
            {
                /* now lets check for how long the inFlightCnt hasn't changed
                 * if that is longer than our tolerance value, delete it
                 */
                if (ADK_IPSEC_STALE_SA_TOLERANCE <= timeElapsed)
                {
                    ADK_IPSEC_LOG_INFO(ipsecHandle,
                            "\n inFlightCnt for SA = %d is still %d, beyond tolerance, so forcefully deleting it\n",
                            saIndex, totInFlightCnt);
                    adk_ipsec_del_eSA (sysHandle, ipsecHandle, ipsecModuleInfo_p,
                                       saIndex);
                    /*  Use the second MSB to indicate if stale SA can be deleted */
                    ipsecModuleInfo_p->staleSaEntries[i] |= ADK_IPSEC_DELETED_SA_MASK;
                }
            }
        }
    }

    /*  Now, delete the stale entries with zero inflight packet count
     *  and shift pending entries to the left */
    i = ipsecModuleInfo_p->numStaleEntries - 1;
    while (i >= 0)
    {
        /*  Boundary condition of index being the maximum possible index in
         *  stale array needs to be checked, so that 'index + 1' element isn't
         *  tried to be accessed.
         */
        if (i != (ADK_IPSEC_NUM_OF_SA - 1))
        {
            /*  Second MSB is an indicator of whether stale entry is to be deleted.
            */
            if ((ipsecModuleInfo_p->staleSaEntries[i] >> (ADK_IPSEC_SA_DIR_SHIFT - 1)) & 0x1)
            {
                ADK_MEMCPY(&ipsecModuleInfo_p->staleSaEntries[i],
                           &ipsecModuleInfo_p->staleSaEntries[i + 1],
                           (sizeof(uint16_t) * (ipsecModuleInfo_p->numStaleEntries - i - 1)));
                ipsecModuleInfo_p->numStaleEntries--;
            }
        }
        i--;
    }
    return retValue;
}

static void convert_8bit_to_32bit (uint32_t *auditIpv6,
                                   uint8_t *metaIpv6)
{
    uint8_t i, j;
    for (i = 0; i < 4; i++)
    {
        j = (i * 4);
        auditIpv6[i] = (((uint32_t)metaIpv6[j] << 24) | ((uint32_t)metaIpv6[j+1] << 16) |
                        ((uint32_t)metaIpv6[j+2] << 8) | (uint32_t)metaIpv6[j+3]);
    }
}

void adk_ipsec_extract_expiry_exception_data (adk_pktio_task_t *task,
                                              adk_ipsec_sa_expiry_excep_data_t *saExpiryData)
{
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    uint8_t *pdu;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(task);
    struct esp_header *espHdr = NULL;
    uint8_t nextHdr, innerHdrOffset;

    ADK_PDU_PTR_GET (task, pdu, uint8_t *);
    
    saExpiryData->ipVersion = (*(uint8_t *)(pdu + adkMetaData->ip_offset)) >> 4;

    if (saExpiryData->ipVersion == ADK_IPSEC_IPV4)
    {
        ipv4Hdr = (struct ipv4_hdr *) (pdu + adkMetaData->ip_offset);
        saExpiryData->ipAddrInfo.ipv4dstAddr.addr = adk_ntohl(ipv4Hdr->dst_addr);
        saExpiryData->ipAddrInfo.ipv4dstAddr.prefixWidth = 32;
        saExpiryData->protocol = ipv4Hdr->next_proto_id;
    }
    else
    {
        ipv6Hdr = (struct ipv6_hdr *) (pdu + adkMetaData->ip_offset);
        convert_8bit_to_32bit (saExpiryData->ipAddrInfo.ipv6dstAddr.addr,
                ipv6Hdr->dst_addr);
        saExpiryData->ipAddrInfo.ipv6dstAddr.prefixWidth = 128;
        nextHdr = ipv6Hdr->proto;
        innerHdrOffset = adkMetaData->ip_offset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ESP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {   
            nextHdr = (*(uint8_t *)(pdu + innerHdrOffset));
            innerHdrOffset += ((*(((uint8_t *)pdu) + innerHdrOffset + 1))<<3) + 8;
        }
        saExpiryData->protocol = nextHdr;
    }

    if(saExpiryData->protocol == PROTO_ESP && saExpiryData->spi == 0)
    {
        espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
        saExpiryData->spi = adk_ntohl(espHdr->spi);
    }
}
void adk_ipsec_extract_headers_sa_message (adk_pktio_task_t *task,
                                           adk_ipsec_sa_msg_t *saMessage_p)
{
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    uint8_t *pdu;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(task);
    struct udp_hdr *udpHdr = NULL;
    struct icmp_hdr *icmpHdr = NULL;
    uint8_t nextHdr, innerHdrOffset;

    ADK_PDU_PTR_GET (task, pdu, uint8_t *);

    saMessage_p->ipVersion = (*(uint8_t *)(pdu + adkMetaData->ip_offset)) >> 4;

    if (saMessage_p->ipVersion == ADK_IPSEC_IPV4)
    {
        ipv4Hdr = (struct ipv4_hdr *) (pdu + adkMetaData->ip_offset);
        saMessage_p->ipAddr.ipv4Addr.srcAddr.addr = adk_ntohl(ipv4Hdr->src_addr);
        saMessage_p->ipAddr.ipv4Addr.dstAddr.addr = adk_ntohl(ipv4Hdr->dst_addr);
        saMessage_p->protocol = ipv4Hdr->next_proto_id;
    }
    else
    {
        ipv6Hdr = (struct ipv6_hdr *) (pdu + adkMetaData->ip_offset);
        convert_8bit_to_32bit (saMessage_p->ipAddr.ipv6Addr.srcAddr.addr,
                            ipv6Hdr->src_addr);
        convert_8bit_to_32bit (saMessage_p->ipAddr.ipv6Addr.dstAddr.addr,
                            ipv6Hdr->dst_addr);
        nextHdr = ipv6Hdr->proto;
        innerHdrOffset = adkMetaData->ip_offset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {   
            nextHdr = (*(uint8_t *)(pdu + innerHdrOffset));
            innerHdrOffset += ((*(((uint8_t *)pdu) + innerHdrOffset + 1))<<3) + 8;
        }
        saMessage_p->protocol = nextHdr;
    }

    if ((PROTO_TCP == saMessage_p->protocol) || (PROTO_UDP == saMessage_p->protocol) || (PROTO_SCTP == saMessage_p->protocol))
    {
        udpHdr = (struct udp_hdr *) (pdu + adkMetaData->l4_offset);
        saMessage_p->l4Info.portInfo.srcPort.port = adk_ntohs(udpHdr->src_port);
        saMessage_p->l4Info.portInfo.dstPort.port = adk_ntohs(udpHdr->dst_port);
    }
    else if(IPPROTO_ICMP == saMessage_p->protocol)
    {
        icmpHdr = (struct icmp_hdr *)(ipv4Hdr + 1);
        saMessage_p->l4Info.icmpInfo.type = icmpHdr->icmp_type;
        saMessage_p->l4Info.icmpInfo.code = icmpHdr->icmp_code;
    }
    else if(IPPROTO_ICMPV6 == saMessage_p->protocol)
    {
        icmpHdr = (struct icmp_hdr *)(pdu + adkMetaData->l4_offset);
        saMessage_p->l4Info.icmpInfo.type = icmpHdr->icmp_type;
        saMessage_p->l4Info.icmpInfo.code = icmpHdr->icmp_code;
    }
}

void adk_ipsec_fill_encrypted_pkt_audit_info (adk_pktio_task_t *task,
                                              adk_ipsec_audit_event_encr_pkt_data_t *auditEventData_p)
{
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    uint8_t *pdu;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(task);
    struct esp_header *espHdr = NULL;
    uint8_t nextHdr, innerHdrOffset;

    ADK_PDU_PTR_GET (task, pdu, uint8_t *);

    auditEventData_p->ipVersion = (*(uint8_t *)(pdu + adkMetaData->ip_offset)) >> 4;

    if (auditEventData_p->ipVersion == ADK_IPSEC_IPV4)
    {
        ipv4Hdr = (struct ipv4_hdr *) (pdu + adkMetaData->ip_offset);
        auditEventData_p->ipSrcAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->src_addr);
        auditEventData_p->ipDstAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->dst_addr);
        auditEventData_p->secProtocol = ipv4Hdr->next_proto_id;
    }
    else
    {
        ipv6Hdr = (struct ipv6_hdr *) (pdu + adkMetaData->ip_offset);
        convert_8bit_to_32bit (auditEventData_p->ipSrcAddr_u.ipv6Addr,
                               ipv6Hdr->src_addr);
        convert_8bit_to_32bit (auditEventData_p->ipDstAddr_u.ipv6Addr,
                               ipv6Hdr->dst_addr);
        nextHdr = ipv6Hdr->proto;
        innerHdrOffset = adkMetaData->ip_offset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ESP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {
            nextHdr = (*(uint8_t *)(pdu + innerHdrOffset));
            innerHdrOffset += ((*(((uint8_t *)pdu) + innerHdrOffset + 1))<<3) + 8;
        }
        auditEventData_p->secProtocol = nextHdr;
        /* Take first 32 bits of which lower 20 bits are flow ID */
        auditEventData_p->flowLabel = *((uint32_t*)(pdu + adkMetaData->ip_offset));
        auditEventData_p->flowLabel = adk_ntohl(auditEventData_p->flowLabel);
        /* Flow label is only lower 20 bits */
        auditEventData_p->flowLabel = auditEventData_p->flowLabel & 0x000FFFFF;
    }
    if (auditEventData_p->secProtocol == PROTO_ESP)
    {
        espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
        auditEventData_p->spiVal = adk_ntohl(espHdr->spi);
        auditEventData_p->seqNum = adk_ntohl(espHdr->seqNum);
    }
    else
    {
       auditEventData_p->spiVal = 0;
       auditEventData_p->seqNum = 0;
    }
}

void adk_ipsec_fill_before_encr_pkt_audit_info(adk_pktio_task_t *task,
                                               adk_ipsec_audit_event_before_encr_pkt_data_t *auditEventData_p)
{
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    uint8_t *pdu = NULL;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(task);
    struct esp_header *espHdr = NULL;
    struct udp_hdr *udpHdr = NULL;
    struct icmp_hdr *icmpHdr = NULL;
    uint8_t nextHdr, innerHdrOffset;

    ADK_PDU_PTR_GET (task, pdu, uint8_t *);

    auditEventData_p->ipVersion = (*(uint8_t *)(pdu + adkMetaData->ip_offset)) >> 4;

    if (ADK_IPSEC_IPV4 == auditEventData_p->ipVersion)
    {
        ipv4Hdr = (struct ipv4_hdr *) (pdu + adkMetaData->ip_offset);
        auditEventData_p->ipSrcAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->src_addr);
        auditEventData_p->ipDstAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->dst_addr);
        auditEventData_p->protocol = ipv4Hdr->next_proto_id;
        auditEventData_p->dscp = (ipv4Hdr->type_of_service >> 2) & 0x3F;
    }
    else
    {
        ipv6Hdr = (struct ipv6_hdr *) (pdu + adkMetaData->ip_offset);
        convert_8bit_to_32bit (auditEventData_p->ipSrcAddr_u.ipv6Addr,
                               ipv6Hdr->src_addr);
        convert_8bit_to_32bit (auditEventData_p->ipDstAddr_u.ipv6Addr,
                               ipv6Hdr->dst_addr);
        nextHdr = ipv6Hdr->proto;
        innerHdrOffset = adkMetaData->ip_offset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ESP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {
            nextHdr = (*(uint8_t *)(pdu + innerHdrOffset));
            innerHdrOffset += ((*(((uint8_t *)pdu) + innerHdrOffset + 1))<<3) + 8;
        }
        auditEventData_p->protocol = nextHdr;
        auditEventData_p->dscp = ((ntohl(ipv6Hdr->vtc_flow) >> 22) & 0x3F); 
        /* Take first 32 bits of which lower 20 bits are flow ID */
        auditEventData_p->flowLabel = *((uint32_t*)(pdu + adkMetaData->ip_offset));
        auditEventData_p->flowLabel = adk_ntohl(auditEventData_p->flowLabel);
        /* Flow label is only lower 20 bits */
        auditEventData_p->flowLabel = auditEventData_p->flowLabel & 0x000FFFFF;
    }
    if (PROTO_ESP == auditEventData_p->protocol)
    {
        espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
        auditEventData_p->l4Ports.ipsecEspAhEvent.spiVal = adk_ntohl(espHdr->spi);
        auditEventData_p->l4Ports.ipsecEspAhEvent.seqNum = adk_ntohl(espHdr->seqNum);
    }
    else if ((PROTO_UDP == auditEventData_p->protocol) || (PROTO_TCP == auditEventData_p->protocol)
             || (PROTO_SCTP == auditEventData_p->protocol))
    {
        udpHdr = (struct udp_hdr *) (pdu + adkMetaData->l4_offset);
        auditEventData_p->l4Ports.ipsecTCPUDPEvent.srcPort = adk_ntohs(udpHdr->src_port);
        auditEventData_p->l4Ports.ipsecTCPUDPEvent.dstPort = adk_ntohs(udpHdr->dst_port);
    }
    else if((IPPROTO_ICMP == auditEventData_p->protocol) || (IPPROTO_ICMPV6 == auditEventData_p->protocol))
    {
        icmpHdr = (struct icmp_hdr *)(pdu + adkMetaData->l4_offset);
        auditEventData_p->l4Ports.ipsecICMPEvent.type = icmpHdr->icmp_type;
        auditEventData_p->l4Ports.ipsecICMPEvent.code = icmpHdr->icmp_code;
    }
}

void adk_ipsec_fill_ing_sa_validation_audit_info(adk_pktio_task_t *task,
                                                 adk_ipsec_audit_event_ing_sa_validation_t *auditEventSAData_p)
{
    struct ipv4_hdr *ipv4Hdr = NULL;
    struct ipv6_hdr *ipv6Hdr = NULL;
    uint8_t *pdu = NULL;
    adk_metadata_t *adkMetaData = ADK_METADATA_PTR(task);
    struct esp_header *espHdr = NULL;
    struct udp_hdr *udpHdr = NULL;
    struct icmp_hdr *icmpHdr = NULL;
    uint8_t nextHdr, innerIpv6HdrOffset, innerOffset;
    adk_ipsec_audit_event_before_encr_pkt_data_t *auditEventData_p;

    innerOffset = auditEventSAData_p->secProtocol;
    auditEventSAData_p->secProtocol = PROTO_ESP;
    
    ADK_PDU_PTR_GET (task, pdu, uint8_t *);
    espHdr = (struct esp_header *)(pdu + adkMetaData->l4_offset);
    auditEventSAData_p->spiVal = adk_ntohl(espHdr->spi);

    auditEventData_p = &auditEventSAData_p->data_from_rx_pkt;
    auditEventData_p->ipVersion = (*(uint8_t *)(pdu + innerOffset)) >> 4;

    if (ADK_IPSEC_IPV4 == auditEventData_p->ipVersion)
    {
        ipv4Hdr = (struct ipv4_hdr *) (pdu + innerOffset);
        auditEventData_p->ipSrcAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->src_addr);
        auditEventData_p->ipDstAddr_u.ipv4Addr = adk_ntohl(ipv4Hdr->dst_addr);
        auditEventData_p->protocol = ipv4Hdr->next_proto_id;
        auditEventData_p->dscp = (ipv4Hdr->type_of_service >> 2) & 0x3F;
        if ((PROTO_UDP == auditEventData_p->protocol) || (PROTO_TCP == auditEventData_p->protocol)
            || (PROTO_SCTP == auditEventData_p->protocol))
        {
            udpHdr = (struct udp_hdr *)(ipv4Hdr + 1);
        }
        else if (IPPROTO_ICMP == auditEventData_p->protocol)
        {
            icmpHdr = (struct icmp_hdr *)(ipv4Hdr + 1);
        }
    }
    else
    {
        ipv6Hdr = (struct ipv6_hdr *) (pdu + innerOffset);
        convert_8bit_to_32bit (auditEventData_p->ipSrcAddr_u.ipv6Addr,
                               ipv6Hdr->src_addr);
        convert_8bit_to_32bit (auditEventData_p->ipDstAddr_u.ipv6Addr,
                               ipv6Hdr->dst_addr);
        nextHdr = ipv6Hdr->proto;
        innerIpv6HdrOffset = innerOffset + sizeof(struct ipv6_hdr);
        while ( (nextHdr != IPPROTO_TCP) &&
                (nextHdr != IPPROTO_UDP) &&
                (nextHdr != IPPROTO_SCTP) &&
                (nextHdr != IPPROTO_ESP) &&
                (nextHdr != IPPROTO_ICMPV6) )
        {
            nextHdr = (*(uint8_t *)(pdu + innerIpv6HdrOffset));
            innerIpv6HdrOffset += ((*(((uint8_t *)pdu) + innerIpv6HdrOffset + 1))<<3) + 8;
        }
        auditEventData_p->protocol = nextHdr;
        auditEventData_p->dscp = ((ntohl(ipv6Hdr->vtc_flow) >> 22) & 0x3F); 
        /* Take first 32 bits of which lower 20 bits are flow ID */
        auditEventData_p->flowLabel = *((uint32_t*)(pdu + innerOffset));
        auditEventData_p->flowLabel = adk_ntohl(auditEventData_p->flowLabel);
        /* Flow label is only lower 20 bits */
        auditEventData_p->flowLabel = auditEventData_p->flowLabel & 0x000FFFFF;
        if ((PROTO_UDP == auditEventData_p->protocol) || (PROTO_TCP == auditEventData_p->protocol)
            || (PROTO_SCTP == auditEventData_p->protocol))
        {
            udpHdr = (struct udp_hdr *)(pdu + innerIpv6HdrOffset);
        }
        else if (IPPROTO_ICMPV6 == auditEventData_p->protocol)
        {
            icmpHdr = (struct icmp_hdr *)(pdu + innerIpv6HdrOffset);
        }
    }
    
    if ((PROTO_UDP == auditEventData_p->protocol) || (PROTO_TCP == auditEventData_p->protocol)
        || (PROTO_SCTP == auditEventData_p->protocol))
    {
        if(udpHdr)
        {
            auditEventData_p->l4Ports.ipsecTCPUDPEvent.srcPort = adk_ntohs(udpHdr->src_port);
            auditEventData_p->l4Ports.ipsecTCPUDPEvent.dstPort = adk_ntohs(udpHdr->dst_port);
        }
    }
    else if((IPPROTO_ICMP == auditEventData_p->protocol) || (IPPROTO_ICMPV6 == auditEventData_p->protocol))
    {
        if(icmpHdr)
        {
            auditEventData_p->l4Ports.ipsecICMPEvent.type = icmpHdr->icmp_type;
            auditEventData_p->l4Ports.ipsecICMPEvent.code = icmpHdr->icmp_code;
        }
    }
}

/* This function rebuilds the SPD tree in DPDK
 * and applies the delta of changes to HLP DS
 */
int adk_ipsec_spd_commit (adk_sys_handle_t sysHandle,
                          adk_module_handle_t ipsecHandle,
                          ipsec_module_info_t *ipsecModuleInfo_p)
{
    struct rte_acl_config aclConfig;
    int retValue = ADK_IPSEC_SUCCESS;
    adk_acl_rule_t cpRule;
    ipsec_sec_pol_info_t *ipsecSecPolLocal = NULL;
    uint32_t  ruleIndex;
	uint8_t tblIdx, newSpdTblIdx, oldSpdTblIdx;

    ADK_IPSEC_LOG_FUNC_ENTRY(ipsecHandle);
    ADK_MEMSET (&aclConfig, 0, sizeof(struct rte_acl_config));

    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset_rules", "Reset "
                        "rules in ACL trie prior to actually committing them\n");
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdTblIdx, newSpdTblIdx, oldSpdTblIdx);
    tblIdx = newSpdTblIdx;
    rte_acl_reset_rules((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdTbl[tblIdx]));

    /* Update Active copy. Get the rules from CP list & add them to S/W ACL*/
    adk_acl_get_reset(ipsecModuleInfo_p->cpSpdAclList);
    ruleIndex = 0;

    aclConfig.num_fields = NUM_FIELDS_IPSEC;
    ADK_MEMCPY (&aclConfig.defs, &adk_ipsec_spd_fields,
                sizeof(adk_ipsec_spd_fields));
    aclConfig.num_categories = 1;

    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_add_rules", "Add "
                            "policies/rules to ACL trie\n");
     while(ADK_SUCCESS == adk_acl_next_rule_get(
                ipsecModuleInfo_p->cpSpdAclList, &cpRule))
    {
        ipsecSecPolLocal = (ipsec_sec_pol_info_t *)(cpRule.user_data);
        ipsecSecPolLocal->dpdkAclRule.data.priority =
                            ADK_IPSEC_SPD_MAX_PRIORITY - ruleIndex;

        retValue = rte_acl_add_rules (
                        (struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdTbl[tblIdx]),
                        (struct rte_acl_rule *)&ipsecSecPolLocal->dpdkAclRule,
                        1);
        if (retValue != 0)
        {
            ADK_IPSEC_LOG_ERROR (ipsecHandle, ADK_E_INCONSISTENT_STATE,
                                 "\nFailed to add rules into ACL context, "
                                 "error code: 0x%x!\n", retValue);
            ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
            return ADK_E_IPSEC_SPD_ENTRY_ADD_FAIL;
        }

        ruleIndex++;
    }

    if(ruleIndex)
    {
        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_build", "Generate "
                            "ACL based on rules added\n");
        retValue = rte_acl_build ((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdTbl[tblIdx]), &aclConfig);
        if (retValue != 0)
        {
            ADK_IPSEC_LOG_ERROR (ipsecHandle, ADK_E_INCONSISTENT_STATE,
                                 "\nFailed to build ACL context. Error:%d\n",
                                 retValue);
            ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
            return ADK_E_IPSEC_SPD_TREE_BUILD_FAIL;
        }
        ipsecModuleInfo_p->numSpdRules = ruleIndex;
        ipsecModuleInfo_p->dpTbl->numPolicies[tblIdx] = ruleIndex;
    }
    else
    {
        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset", "No rules exist - Reset "
                            "ACL context\n");
        ipsecModuleInfo_p->numSpdRules = 0;
        rte_acl_reset ((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdTbl[tblIdx]));
        ipsecModuleInfo_p->dpTbl->numPolicies[tblIdx] = 0;
    }

    /* RCU sync */
    adk_rcu_synchronize();

    /* CP ACL List HOS Commit */
    retValue = adk_acl_hos_commit(ipsecModuleInfo_p->cpSpdAclList);
    if(retValue != ADK_SUCCESS)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Error: Could not commit in HOS...\n");
        retValue = ADK_E_INTERNAL_ERROR;
        return retValue;
    }
 
    /* toggle index */
    ipsecModuleInfo_p->dpTbl->spdTblIdx = newSpdTblIdx;
    if (ADK_IPSEC_FC_VER_TO_BE_UPDATED == ipsecModuleInfo_p->ipsec_fc_ver_status)
    {
        retValue = adk_sys_flow_cache_update_version(sysHandle);
        if(retValue != ADK_SUCCESS)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                    "Error: Could not update Flow Cache version\n");
            retValue = ADK_E_INTERNAL_ERROR;
            return retValue;
        }
        ipsecModuleInfo_p->ipsec_fc_ver_status = ADK_IPSEC_FC_VER_UPDATED;
    }
    ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
    return ADK_IPSEC_SUCCESS;
}

/* This function rebuilds the SPD tree in DPDK
 * and applies the delta of changes to HLP DS
 */
int adk_ipsec_spd_in_commit (adk_sys_handle_t sysHandle,
                             adk_module_handle_t ipsecHandle,
                             ipsec_module_info_t *ipsecModuleInfo_p)
{
    struct rte_acl_config aclConfig;
    int retValue = ADK_IPSEC_SUCCESS;
    adk_acl_rule_t cpRule;
    ipsec_sec_pol_in_info_t *ipsecSecPolInLocal = NULL;
    uint32_t  ruleIndex;
	uint8_t tblIdx, newSpdTblIdx, oldSpdTblIdx;

    ADK_IPSEC_LOG_FUNC_ENTRY(ipsecHandle);
    ADK_MEMSET (&aclConfig, 0, sizeof(struct rte_acl_config));

    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset_rules", "Reset "
                      "rules in SPD IN ACL trie prior to actually committing them\n");
    mFindIdx(ipsecModuleInfo_p->dpTbl->spdInTblIdx, newSpdTblIdx, oldSpdTblIdx);
    tblIdx = newSpdTblIdx;
    rte_acl_reset_rules((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdInTbl[tblIdx]));

    /* Update Active copy. Get the rules from CP list & add them to S/W ACL*/
    adk_acl_get_reset(ipsecModuleInfo_p->cpSpdInAclList);
    ruleIndex = 0;

   aclConfig.num_fields = NUM_FIELDS_IPSEC_IN;
   ADK_MEMCPY (&aclConfig.defs, &adk_ipsec_spd_in_fields,
               sizeof(adk_ipsec_spd_in_fields));
   aclConfig.num_categories = 1;

   ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_add_rules", "Add "
                    "policies/rules to SPD IN ACL trie\n");

   while(ADK_SUCCESS == adk_acl_next_rule_get(
                ipsecModuleInfo_p->cpSpdInAclList, &cpRule))
    {
        ipsecSecPolInLocal = (ipsec_sec_pol_in_info_t *)(cpRule.user_data);
        ipsecSecPolInLocal->dpdkAclInRule.data.priority =
                            ADK_IPSEC_SPD_MAX_PRIORITY - ruleIndex;

        retValue = rte_acl_add_rules (
                        (struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdInTbl[tblIdx]),
                        (struct rte_acl_rule *)&ipsecSecPolInLocal->dpdkAclInRule,
                        1);
        if (retValue != 0)
        {
            ADK_IPSEC_LOG_ERROR (ipsecHandle, ADK_E_INCONSISTENT_STATE,
                                 "\nFailed to add rules into ACL context, "
                                 "error code: 0x%x!\n", retValue);
            ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
            return ADK_E_IPSEC_SPD_ENTRY_ADD_FAIL;
        }

        ruleIndex++;
    }


    if(ruleIndex)
    {
        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_build", "Generate "
                        "SPD IN ACL based on rules added\n");
        retValue = rte_acl_build ((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdInTbl[tblIdx]), &aclConfig);
        if (retValue != 0)
        {
            ADK_IPSEC_LOG_ERROR (ipsecHandle, ADK_E_INCONSISTENT_STATE,
                                 "\nFailed to build ACL context. Error:%d\n",
                                 retValue);
            ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
            return ADK_E_IPSEC_SPD_TREE_BUILD_FAIL;
        }
        ipsecModuleInfo_p->numSpdInRules = ruleIndex;
        ipsecModuleInfo_p->dpTbl->numInPolicies[tblIdx] = ruleIndex;
    }
    else
    {
        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_acl_reset", "No rules exist - Reset "
                            "SPD IN ACL context\n");
        ipsecModuleInfo_p->numSpdInRules = 0;
        rte_acl_reset ((struct rte_acl_ctx *)(ipsecModuleInfo_p->dpTbl->spdInTbl[tblIdx]));
        ipsecModuleInfo_p->dpTbl->numInPolicies[tblIdx] = 0;
    }

    /* RCU sync */
    adk_rcu_synchronize();

    /* CP ACL List HOS Commit */
    retValue = adk_acl_hos_commit(ipsecModuleInfo_p->cpSpdInAclList);
    if(retValue != ADK_SUCCESS)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, retValue,
                           "Error: Could not commit in HOS...\n");
        retValue = ADK_E_INTERNAL_ERROR;
        return retValue;
    }

    /* toggle index */
    ipsecModuleInfo_p->dpTbl->spdInTblIdx = newSpdTblIdx;
    ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
    return ADK_IPSEC_SUCCESS;
}

adk_ipsec_return_code_t
adk_ipsec_verify_add_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_sa_t        *secAsc_p,
                               adk_handle_t         *saNsHandle,
                               uint8_t              numPol,
                               uint16_t             startPolIndex)
{
    /* verify input arguments */
    if((saNsHandle == NULL)  || (secAsc_p == NULL))
    {
        /* log error */
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_INPUT_PARAM,
                            "Invalid security Association input parameter \n");
        return ADK_E_IPSEC_INVALID_INPUT_PARAM;
    }

    if ((startPolIndex > (ADK_IPSEC_MAX_NUM_SPD_RULES - 1)) ||
        ((startPolIndex + numPol) > (ADK_IPSEC_MAX_NUM_SPD_RULES)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INDEX_OUT_OF_BOUND,
                            "Given Policy index is out of boundary\n");
        ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INDEX_OUT_OF_BOUND);
    }
	
    if ((startPolIndex > ipsecModuleInfo_p->numSpdRules) ||
        ((startPolIndex + numPol) > ipsecModuleInfo_p->numSpdRules))
    {
         ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                             "Given policy index is greater than number of rules configured\n");
         ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_INPUT_PARAM);
    }

    if (ipsecModuleInfo_p->numCryptoDevices == 0)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_RESOURCE_UNAVAILABLE,
                            "Crypto device unavailable - needed for IPsec functionality, exiting!");
        return ADK_E_RESOURCE_UNAVAILABLE;
    }

    if ((secAsc_p->ipVersion != ADK_IPSEC_IPV4) &&
        (secAsc_p->ipVersion != ADK_IPSEC_IPV6))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_IP_VERSION,
                            "Invalid ip version input argument\n");
        return ADK_E_IPSEC_INVALID_IP_VERSION;
    }

    if((secAsc_p->ipVersion == ADK_IPSEC_IPV6) && (secAsc_p->isNATTEnable))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid configuration, for IPv6 Security Association NAT-T cannot be enabled\n");
        return ADK_E_IPSEC_INVALID_CONFIG;

    }

    if (secAsc_p->SASPPContext.protocol != ADK_IPSEC_ESP)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Current ADK version supports only ESP protcol SA configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (secAsc_p->SASPPContext.mode != ADK_IPSEC_TUNNEL)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Current ADK version supports only Tunnel mode configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if ((secAsc_p->SASPPContext.direction != ADK_IPSEC_OUTBOUND) &&
        (secAsc_p->SASPPContext.direction != ADK_IPSEC_INBOUND))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Current ADK version supports only Outbound and Inbound direction configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (secAsc_p->SASPPContext.sn.antiReplayEnable)
    {
        if ( (secAsc_p->SASPPContext.antiReplayWindowSize < ADK_IPSEC_ARW_MIN) ||
             (secAsc_p->SASPPContext.antiReplayWindowSize > ADK_IPSEC_ARW_MAX) )
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid ARW configuration\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
    }

    if ( (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV8) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV12) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV16) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV8) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV12) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV16))
    {
        if ((secAsc_p->SASPPContext.encrKey.len != 16) && (secAsc_p->SASPPContext.encrKey.len != 24)
             && (secAsc_p->SASPPContext.encrKey.len != 32))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid ENCR key length, received %d, need 16/24/32 bit key length\n",
                    secAsc_p->SASPPContext.encrKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This encryption algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
    }

    /* cipher key lenght sanity is done by DPDK/RDK APIs,
     * for some reasons, it is not done for auth key length, so
     * added that logic here
     */
    switch(secAsc_p->SASPPContext.auth)
    {
    case ADK_IPSEC_AUTH_NULL:
        break;
    case ADK_IPSEC_AUTH_HMAC_MD5:
        if (secAsc_p->SASPPContext.authKey.len == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA1:
        /* as per RFC2404 */
        if (secAsc_p->SASPPContext.authKey.len != 20)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_AES_CMAC:
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_AES_XCBC_MAC:
        /* as per RFC3566 */
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_256:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 32)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_384:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 48)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_512:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 64)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_128_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 16 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_192_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 24)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 24 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_256_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 32)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 32 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    default:
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid AUTH algo\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    } 

    if ((secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_NULL) &&
        (secAsc_p->SASPPContext.auth == ADK_IPSEC_AUTH_NULL))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Both encryption and integrity algorithms cannot be NULL at the same time!\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    /* These are the supported encryption alogorithms as mentioned ice_dsi_ipsec_capabilities.h for inline crypto*/
    if((secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_NULL) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CBC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_DES_CBC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CTR) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_NULL_GMAC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV8) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV12) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV16) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV8) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV12) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV16) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_3DES_CBC))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "IPSEC: ADD_SA - Non endorsed encryption algorithm(s) provided for configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
    else if((secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_DES_CBC) && (ADK_IPSEC_SA_MODE_INLINE == secAsc_p->saMode))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "IPSEC: ADD_SA - AES DES is supported only in lookaside mode\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    return ADK_IPSEC_SUCCESS;
}


adk_ipsec_return_code_t
adk_ipsec_verify_upd_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_upd_sa_t  *secAsc_p,
                               adk_handle_t             saNsHandle,
                               uint8_t              numPol,
                               uint16_t             startPolIndex)
{
    uint16_t saIdx;
    /* verify input arguments */
    if((secAsc_p == NULL))
    {
        /* log error */
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_INPUT_PARAM,
                "Invalid security Association input parameter \n");
        return ADK_E_IPSEC_INVALID_INPUT_PARAM;
    }
    
    if ((startPolIndex > (ADK_IPSEC_MAX_NUM_SPD_RULES - 1)) ||
        ((startPolIndex + numPol) > (ADK_IPSEC_MAX_NUM_SPD_RULES)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INDEX_OUT_OF_BOUND,
                            "Given Policy index is out of boundary\n");
        ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INDEX_OUT_OF_BOUND);
    }
	
    if ((startPolIndex > ipsecModuleInfo_p->numSpdRules) ||
        ((startPolIndex + numPol) > ipsecModuleInfo_p->numSpdRules))
    {
         ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                             "Given policy index is greater than number of rules configured\n");
         ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_INPUT_PARAM);
    }

    saIdx = ((ipsec_sa_idx_t *)saNsHandle)->index;

    if ((secAsc_p->SASPPContext.direction != ADK_IPSEC_OUTBOUND) &&
            (secAsc_p->SASPPContext.direction != ADK_IPSEC_INBOUND))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid SA direction specified\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (secAsc_p->SASPPContext.direction == ADK_IPSEC_OUTBOUND)
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->eSASetHdl, saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_INVALID_PARAMETER,
                    "Invalid SA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }
    else
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->iSASetHdl, saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_INVALID_PARAMETER,
                    "Invalid SA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }

    if (secAsc_p->SASPPContext.protocol != ADK_IPSEC_ESP)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "Current ADK version supports only ESP protocol SA configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (secAsc_p->SASPPContext.mode != ADK_IPSEC_TUNNEL)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Current ADK version supports only Tunnel mode configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if ((secAsc_p->SASPPContext.direction != ADK_IPSEC_OUTBOUND) &&
            (secAsc_p->SASPPContext.direction != ADK_IPSEC_INBOUND))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid SA direction\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if ((secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_NULL) &&
        (secAsc_p->SASPPContext.auth == ADK_IPSEC_AUTH_NULL))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Both encryption and integrity algorithms cannot be NULL at the same time!\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if ( (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV8) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV12) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_GCM_ICV16) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV8) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV12) ||
         (secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_AES_CCM_ICV16))
    {
        if ((secAsc_p->SASPPContext.encrKey.len != 16) && (secAsc_p->SASPPContext.encrKey.len != 24)
             && (secAsc_p->SASPPContext.encrKey.len != 32))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid ENCR key length, received %d, need 16/24/32 bit key length\n",
                    secAsc_p->SASPPContext.encrKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This encryption algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
    }

    /* cipher key lenght sanity is done by DPDK/RDK APIs,
     * for some reasons, it is not done for auth key length, so
     * added that logic here
     */
    switch(secAsc_p->SASPPContext.auth)
    {
    case ADK_IPSEC_AUTH_NULL:
        break;
    case ADK_IPSEC_AUTH_HMAC_MD5:
        if (secAsc_p->SASPPContext.authKey.len == 0)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA1:
        /* as per RFC2404 */
        if (secAsc_p->SASPPContext.authKey.len != 20)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_AES_CMAC:
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_AES_XCBC_MAC:
        /* as per RFC3566 */
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_256:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 32)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_384:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 48)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    case ADK_IPSEC_AUTH_HMAC_SHA_512:
        /* as per RFC4868 */
        if (secAsc_p->SASPPContext.authKey.len != 64)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_128_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 16)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 16 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_192_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 24)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 24 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;

    case ADK_IPSEC_AUTH_AES_256_GMAC:
        /* as per RFC4543 */
        if (secAsc_p->SASPPContext.authKey.len != 32)
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                    "Invalid AUTH key length, received %d, need 32 bit key length\n",
                    secAsc_p->SASPPContext.authKey.len);
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        if ((secAsc_p->SASPPContext.salt.len != 4) ||
            (secAsc_p->SASPPContext.salt.buf == NULL))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                        "This algorithm needs four byte salt value\n");
            return ADK_E_IPSEC_INVALID_CONFIG;
        }
        break;
    default:
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid AUTH algo\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    /* These are the supported encryption alogorithms as mentioned ice_dsi_ipsec_capabilities.h for inline crypto*/
    if((secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_NULL) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CBC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_DES_CBC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CTR) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_NULL_GMAC) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV8) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV12) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_GCM_ICV16) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV8) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV12) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_AES_CCM_ICV16) &&
       (secAsc_p->SASPPContext.encr != ADK_IPSEC_ENCR_3DES_CBC))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "IPSEC: UPDATE_SA - Non endorsed encryption algorithm(s) provided for configuration\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }
    else if(secAsc_p->SASPPContext.encr == ADK_IPSEC_ENCR_DES_CBC)
    {
        if(((secAsc_p->SASPPContext.direction == ADK_IPSEC_OUTBOUND) && (ADK_IPSEC_SA_MODE_INLINE == ipsecModuleInfo_p->dpTbl->eSATbl[saIdx].saMode))
           || ((secAsc_p->SASPPContext.direction == ADK_IPSEC_INBOUND) && (ADK_IPSEC_SA_MODE_INLINE == ipsecModuleInfo_p->dpTbl->iSATbl[saIdx].saMode)))
        {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_CONFIG,
                "IPSEC: ADD_SA - AES DES is supported only in lookaside mode\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
        }
    }

    return ADK_IPSEC_SUCCESS;
}

adk_ipsec_return_code_t
adk_ipsec_verify_del_sa_input_args(adk_sys_handle_t     sysHandle,
                               adk_module_handle_t  ipsecHandle,
                               ipsec_module_info_t *ipsecModuleInfo_p,
                               adk_ipsec_sa_direction_t    saDir,
                               uint8_t              numPol,
                               uint16_t             startPolIndex,
                               adk_handle_t         saNsHandle)
{
    uint32_t saIdx;
    /* verify input arguments */
    if(saNsHandle == NULL)
    {
        /* log error */
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_INPUT_PARAM,
                "Invalid SA handle input parameter while deleting SA \n");
        return ADK_E_IPSEC_INVALID_INPUT_PARAM;
    }

    if ((startPolIndex > (ADK_IPSEC_MAX_NUM_SPD_RULES - 1)) ||
        ((startPolIndex + numPol) > (ADK_IPSEC_MAX_NUM_SPD_RULES)))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INDEX_OUT_OF_BOUND,
                            "Given Policy index is out of boundary\n");
        ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INDEX_OUT_OF_BOUND);
    }
	
    if ((startPolIndex > ipsecModuleInfo_p->numSpdRules) ||
        ((startPolIndex + numPol) > ipsecModuleInfo_p->numSpdRules))
    {
         ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                             "Given policy index is greater than number of rules configured\n");
         ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_INPUT_PARAM);
    }

    if ((saDir != ADK_IPSEC_OUTBOUND) &&
        (saDir != ADK_IPSEC_INBOUND))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid SA direction specified\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    saIdx = ((ipsec_sa_idx_t *)saNsHandle)->index;

    if (saDir == ADK_IPSEC_OUTBOUND)
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->eSASetHdl, saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_INVALID_PARAMETER,
                    "Invalid eSA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }
    else
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->iSASetHdl, saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_INVALID_PARAMETER,
                    "Invalid iSA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }

    return ADK_IPSEC_SUCCESS;
}

adk_ipsec_return_code_t
adk_ipsec_verify_get_sa_input_args(adk_module_handle_t  ipsecHandle,
                                   ipsec_module_info_t *ipsecModuleInfo_p,
                                   adk_handle_t         saNsHandle,
                                   adk_ipsec_sa_direction_t saDirection)
{
    uint32_t saIdx;

    if (saNsHandle == NULL)
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_INVALID_PARAMETER,
                "Invalid (NULL) SA namespace handle\n");
        return ADK_E_IPSEC_INVALID_INPUT_PARAM;
    }

    saIdx = ((ipsec_sa_idx_t *)saNsHandle)->index;

    if ((saDirection != ADK_IPSEC_OUTBOUND) &&
        (saDirection != ADK_IPSEC_INBOUND))
    {
        ADK_IPSEC_LOG_ERROR(ipsecHandle,ADK_E_IPSEC_INVALID_CONFIG,
                "Invalid SA direction specified\n");
        return ADK_E_IPSEC_INVALID_CONFIG;
    }

    if (saDirection == ADK_IPSEC_OUTBOUND)
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->eSASetHdl,
                                          saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                    "Invalid (unset)egress SA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }
    else if (saDirection == ADK_IPSEC_INBOUND)
    {
        if(FALSE == adk_rm_resource_isset(ipsecModuleInfo_p->iSASetHdl,
                                          saIdx))
        {
            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_IPSEC_INVALID_INPUT_PARAM,
                    "Invalid (unset)ingress SA handle \n");
            return ADK_E_IPSEC_INVALID_INPUT_PARAM;
        }
    }
    return ADK_IPSEC_SUCCESS;
}

adk_status_t adk_ipsec_rsrc_notify_fn_cbfn (adk_module_handle_t ipsecHandle,
                                            adk_sys_rsrc_oper_t oper,
                                            adk_sys_resource_type_t type,
                                            adk_sys_resource_info_t rsrc)
{
    adk_status_t                    retValue = ADK_IPSEC_SUCCESS;
    ipsec_module_info_t             *ipsecModuleInfo_p;
    adk_sys_handle_t                sysHandle;
    adk_sys_rsrc_crypto_dev_info_t  *cryptoDevInfo;
    char poolName[RTE_MEMPOOL_NAMESIZE];
    uint32_t  maxSessSize = 0, socketId = 0;

    if (!ADK_MODULE_HANDLE_VALIDATE(ipsecHandle))
    {
        ADK_IPSEC_LOG_FORCE ("Invalid IPSec module handle %"ADKPRIh"\n",
                              ipsecHandle);
        return ADK_ERROR_SEV_LEVEL_SET(ADK_E_INVALID_PARAMETER);
    }

    sysHandle = ADK_MODULE_SYS_HANDLE_GET(ipsecHandle);
    ADK_IPSEC_LOG_FUNC_ENTRY(ipsecHandle);
    ADK_ACQUIRE_MODULE_LOCK(ipsecHandle, retValue);

    ipsecModuleInfo_p = ADK_MODULE_INFO_PTR_GET(ipsecHandle);
    mVerifyArguments(ipsecModuleInfo_p, ipsecHandle, retValue);

    if(oper == ADK_SYS_RSRC_ADD)
    {
        switch(type){
        case ADK_SYS_RSRC_TYPE_CRYPTO_DEV:
            {
                if (ipsecModuleInfo_p->numCryptoDevices == ADK_IPSEC_MAX_CRYPTO_DEV-1)
                {
                    /* can't accpet this! */
                    ADK_IPSEC_LOG_FORCE ("Already configured max allowed crypto devices\n");
                    ADK_RELEASE_MODULE_LOCK(ipsecHandle, retValue);
                    ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_INPUT_PARAM);
                }
                cryptoDevInfo = (adk_sys_rsrc_crypto_dev_info_t *)
                                        rsrc.crypto_dev_info_p;
                ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices] =
                    (adk_sys_rsrc_crypto_dev_info_t *)ADK_NVM_CALLOC(ADK_IPSEC_MODULE_ID, ADK_IPSEC_CRYPTO_INFO_OBJ,
							 1, sizeof(adk_sys_rsrc_crypto_dev_info_t));
                ADK_MEMCPY(ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices],
                           cryptoDevInfo,
                           sizeof(adk_sys_rsrc_crypto_dev_info_t));
                if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_LA)
                {
                    ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices]->qp_info_p =
                        (adk_sys_rsrc_crypto_dev_qpair_info_t *)ADK_NVM_CALLOC(ADK_IPSEC_MODULE_ID, ADK_IPSEC_CRYPTO_INFO_OBJ,
                                cryptoDevInfo->num_q_pairs, sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));
                    ADK_MEMCPY(ipsecModuleInfo_p->cryptoDev[ipsecModuleInfo_p->numCryptoDevices]->qp_info_p,
                            cryptoDevInfo->qp_info_p,
                            cryptoDevInfo->num_q_pairs * sizeof(adk_sys_rsrc_crypto_dev_qpair_info_t));
                    if (cryptoDevInfo->num_sas != 0)
                    {
                        /* now allocate sess pool for this device!
                        */
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_cryptodev_socket_id", "Get "
                                "the socket ID on which the crypto device exists\n");
                        socketId = rte_cryptodev_socket_id(cryptoDevInfo->dev_id);
                        snprintf(poolName, RTE_MEMPOOL_NAMESIZE, "ipsec_sess_mp_%u", cryptoDevInfo->dev_id);
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_cryptodev_get_private_session_size", "Get "
                                "the private session size for crypto device\n");
#if ADK_DPDK_VERSION >= 18050016
                        maxSessSize = rte_cryptodev_sym_get_private_session_size(cryptoDevInfo->dev_id);
#else
                        maxSessSize = rte_cryptodev_get_private_session_size(cryptoDevInfo->dev_id);
#endif
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_create", "Create "
                                "mempool for sessions on crypto device\n");
                        ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].sessPool =
                                            rte_mempool_create(poolName,
                                                               ADK_IPSEC_NUM_OF_SESSIONS,
                                                               maxSessSize,
                                                               0, 0, NULL, NULL, NULL, NULL,
                                                               socketId, 0);
                        if (ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].sessPool == NULL)
                        {
                            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                    "IPSEC: Failed to allocate memory\n");
                            ADK_RELEASE_MODULE_LOCK(ipsecHandle, retValue);
                            ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
                            ADK_RETURN_ERROR_SET(ADK_E_IPSEC_INVALID_INPUT_PARAM);
                        }
                        ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = true;
                        ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].devId = cryptoDevInfo->dev_id;
                        ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].numSAsAllowed = cryptoDevInfo->num_sas;
                    }
                    else
                        ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = false;
                }
                /* currently, assumption is that there would be only one inline device,
                 * in case of inline device, we need to store
                 * dev_id only for the time being. Whenever we allow application to
                 * create multiple such devices and split it for number of SAs, need
                 * to enhance data strucure and this logic
                 */
                else if (cryptoDevInfo->dev_type == ADK_SYS_RSRC_CRYPTO_DEV_TYPE_INLINE)
                {
                    /* what if this was already registered? reject it then..
                     */
                    if (ipsecModuleInfo_p->secCtxCP != NULL)
                    {
                        /* it means we already have registered inline device
                         */
                        ADK_IPSEC_LOG_INFO(ipsecHandle, "inline device already registered! \n");
                        break;
                    }
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].isValid = true;
                    ipsecModuleInfo_p->dpTbl->cryptoDevMap[ipsecModuleInfo_p->numCryptoDevices].devId = cryptoDevInfo->dev_id;
                    /* doing this just to optimize, else, need to have some logic to decide
                     * which inline dev to use, e.g. round robin. Not required right now
                     */
                    ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_eth_dev_get_sec_ctx", "Get "
                                        "security context for inline crypto device\n");
                    ipsecModuleInfo_p->secCtxCP = (struct rte_security_ctx *)rte_eth_dev_get_sec_ctx(cryptoDevInfo->dev_id);
                    if (ipsecModuleInfo_p->secCtxCP == NULL)
                    {
                        ADK_IPSEC_LOG_INFO(ipsecHandle, "No inline device found! \n");
                    }
                    else
                    {
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_eth_dev_socket_id", "Get "
                                            "socket ID of device needed to create crypto session mempool \n");
                        /* TODO is this correct? passing 0 ?
                         */
                        socketId = rte_eth_dev_socket_id(0);
                        ADK_MEMSET(poolName, 0, sizeof(poolName));
                        snprintf(poolName, RTE_MEMPOOL_NAMESIZE, "ipsec_inlinesess_mp_%u", socketId);
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_security_session_get_size", "Get "
                                            "maximum size of session of inline crypto device security context\n");
                        maxSessSize = rte_security_session_get_size(ipsecModuleInfo_p->secCtxCP);
                        ADK_LOG_EXT_API (sysHandle, ipsecHandle, "rte_mempool_create", "Create "
                                            "mempool for sessions on crypto device\n");
                        ipsecModuleInfo_p->dpTbl->inlineSessPool =
                            rte_mempool_create(poolName,
                                    ADK_IPSEC_NUM_OF_SESSIONS,
                                    maxSessSize,
                                    0, 0, NULL, NULL, NULL, NULL,
                                    socketId, 0);
                        if (ipsecModuleInfo_p->dpTbl->inlineSessPool == NULL)
                        {
                            ADK_IPSEC_LOG_ERROR(ipsecHandle, ADK_E_MEM_UNAVAILABLE,
                                              "Failed to allocate session pool for inline\n"
                                              );
                            ADK_RELEASE_MODULE_LOCK(ipsecHandle, retValue);
                            return ADK_E_MEM_UNAVAILABLE;
                        }
                    }
                }
                ipsecModuleInfo_p->numCryptoDevices++;
            }
            break;
        default:
            break;
        }
    }
    else
    {
        ADK_FORCE_LOG_ERROR("Only resource ADD is supported, \
                            other operations are not supported yet\n");
    }

    ADK_RELEASE_MODULE_LOCK(ipsecHandle, retValue);

_lock_release_failed:
_lock_acquire_failed:
    ADK_IPSEC_LOG_FUNC_EXIT(ipsecHandle);
    ADK_RETURN(retValue);
}
